/*****************************************************************************
 *  Rohde&Schwarz Spectrum Analyzer instrument driver
 *  NB IoT option K106
 *
 *  Original Release: November 2018
 *  By: Petr Kominek
 *
 *  Should you have any technical questions please contact the hotline of
 *  Rohde & Schwarz Vertriebs-GmbH Rohde & Schwarz Support Center
 *
 *  e-mail: CustomerSupport@rohde-schwarz.com
 *
 *****************************************************************************/

#include "rsspecan.h"

/// HIFN  Selects a new measurement channel with the defined channel type.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsspecan_SetNBIoTDownlinkMode(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_NB_IOT_DOWNLINK_MODE, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Configures the MIMO.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR dut/Sets the number of antennas in the MIMO setup.
/// HIPAR antenna/Selects the antenna for measurements with MIMO setups.
ViStatus _VI_FUNC rsspecan_ConfigureNBIoTDownlinkMIMO(ViSession instrSession,
                                                      ViInt32   dut,
                                                      ViInt32   antenna)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_NB_IOT_DOWNLINK_MIMO_CONFIG, dut),
            2, "DUT");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_NB_IOT_DOWNLINK_MIMO_ANTENNA_SELECTION, antenna),
            3, "Antenna");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects the NB-IoT carrier type.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR type/Selects the NB-IoT carrier type.
ViStatus _VI_FUNC rsspecan_ConfigureNBIoTDownlinkCarrierType(ViSession instrSession,
                                                             ViInt32   type)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, NULL, RSSPECAN_ATTR_NB_IOT_DOWNLINK_CARRIER_TYPE, type),
            2, "Type");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Saves the signal description.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR filename/Saves the signal description.
ViStatus _VI_FUNC rsspecan_StoreNBIoTDemodulationSettings(ViSession instrSession,
                                                          ViString  filename)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViString(instrSession, NULL, RSSPECAN_ATTR_NB_IOT_DOWNLINK_STORE_DEMODULATION_SETTINGS, filename),
            3, "Filename");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Loads an EUTRA test model (N-TM).
/// HIFN  The EUTRA test models are in accordance with 3GPP 36.141.
/// HIFN  You can also select an O-RAN test case with the command.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR componentCarrier/Component Carrier.
/// HIPAR filename/Loads an EUTRA test model (N-TM).
ViStatus _VI_FUNC rsspecan_ConfigureNBIoTDownlinkTestModel(ViSession instrSession,
                                                           ViInt32   componentCarrier,
                                                           ViString  filename)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_NB_IOT_DOWNLINK_TEST_MODEL,
            "cc", componentCarrier - 1 , RS_REPCAP_BUF_SIZE, repCap));

    viCheckParm(rsspecan_SetAttributeViString(instrSession, repCap, RSSPECAN_ATTR_NB_IOT_DOWNLINK_TEST_MODEL, filename),
            3, "Filename");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Initiates a preset to the default state of the software, and, if
/// HIFN  connected to an analyzer, also presets the analyzer.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsspecan_NBIoTDownlinkPreset(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_NB_IOT_DOWNLINK_PRESET, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Restores previously saved demodulation settings.
/// HIFN     The file must be of type .allocation and depends on the link direction
/// HIFN  that was currently selected when the file was saved. You can load only
/// HIFN  files with correct link directions.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR fileName/File name parameter.
ViStatus _VI_FUNC rsspecan_NBIoTDownlinkLoadDemodulationSettings(
                            ViSession instrSession,
                            ViString  fileName)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_NB_IOT_DOWNLINK_LOAD_DEMODULATION_SETTINGS, fileName),
            2, "File Name");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Restores I/Q data from a file.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR fileName/File name parameter.
ViStatus _VI_FUNC rsspecan_NBIoTDownlinkLoadIQ(ViSession instrSession,
                                               ViString  fileName)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_NB_IOT_DOWNLINK_LOAD_IQ, fileName),
            2, "File Name");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Selects the measurement window.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/Selects the measurement window.
ViStatus _VI_FUNC rsspecan_ConfigureNBIoTDownlinkWindowSelect(
                            ViSession instrSession,
                            ViInt32   window)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_NB_IOT_DOWNLINK_WINDOW_SELECT,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap));

    checkErr(rsspecan_SetAttributeViString(instrSession, repCap, RSSPECAN_ATTR_NB_IOT_DOWNLINK_WINDOW_SELECT, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Selects the deployment of the NB-IoT carrier.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR deployment/Selects the deployment of the NB-IoT carrier.
ViStatus _VI_FUNC rsspecan_ConfigureNBIoTDownlinkDeployment(ViSession instrSession,
                                                            ViInt32   deployment)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_NB_IOT_DOWNLINK_DEPLOYMENT, deployment),
            2, "Deployment");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Configures the Signal Characteristics for In Band deployment.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR frequency/Defines the center frequency of an E-UTRA channel.
/// HIPAR bandwidth/Selects the channel bandwidth.
/// HIPAR crsSequence/Defines the CRS sequence info of the E-UTRA channel.
/// HIPAR prbIndex/Defines the PRB index of the E-UTRA channel.
ViStatus _VI_FUNC rsspecan_ConfigureNBIoTDownlinkInBand(ViSession instrSession,
                                                        ViReal64  frequency,
                                                        ViInt32   bandwidth,
                                                        ViInt32   crsSequence,
                                                        ViInt32   prbIndex)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_NB_IOT_DOWNLINK_EUTRA_FREQUENCY, frequency),
            2, "Frequency");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_NB_IOT_DOWNLINK_BANDWIDTH, bandwidth),
            3, "Bandwidth");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_NB_IOT_DOWNLINK_CRS_SEQUENCE, crsSequence),
            4, "CRS Sequence");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_NB_IOT_DOWNLINK_PRB_INDEX, prbIndex),
            5, "PRB Index");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Configures the Signal Characteristics for Stand Alone deployment.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR frequency/Sets the center frequency for RF measurements.
ViStatus _VI_FUNC rsspecan_ConfigureNBIoTDownlinkStandAlone(ViSession instrSession,
                                                            ViReal64  frequency)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_NB_IOT_DOWNLINK_RF_CENTER_FREQUENCY, frequency),
            2, "Frequency");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This atribute defines the location of the NB-IoT carrier in the E-UTRA
/// HIFN  guard band. Prerequisites for this atribute
/// HIFN     - Select guard band deployment of an NB-IoT carrier
/// HIFN  (RSSPECAN_ATTR_NB_IOT_DOWNLINK_DEPLOYMENT).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR carrierLocation/Sets the center frequency for RF measurements.
ViStatus _VI_FUNC rsspecan_ConfigureNBIoTDownlinkGuardband(ViSession instrSession,
                                                           ViInt32   carrierLocation)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, NULL, RSSPECAN_ATTR_NB_IOT_DOWNLINK_GUARDBAND_CARRIER_LOCATION, carrierLocation),
            2, "Carrier Location");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Custom location of the NB-IoT carrier.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR customLocation/Sets the center frequency for RF measurements.
ViStatus _VI_FUNC rsspecan_ConfigureNBIoTDownlinkGuardbandUser(
                            ViSession instrSession,
                            ViReal64  customLocation)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsspecan_SetAttributeViString(instrSession, NULL, RSSPECAN_ATTR_NB_IOT_DOWNLINK_GUARDBAND_CARRIER_CUSTOM_LOCATION, NULL));

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, NULL, RSSPECAN_ATTR_NB_IOT_DOWNLINK_GUARDBAND_CARRIER_CUSTOM_LOCATION_VALUE, customLocation),
            2, "Custom Location");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Initiates a measurement that determines the ideal reference level.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsspecan_ConfigureNBIoTDownlinkLevelOnce(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_NB_IOT_DOWNLINK_AUTO_REFERENCE_LEVEL_ONCE, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Configures the Level Reference.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR autoRef/Initiates a measurement that determines the ideal reference level.
/// HIPAR level/Defines the reference level when the input source is RF.
ViStatus _VI_FUNC rsspecan_ConfigureNBIoTDownlinkLevelReference(
                            ViSession instrSession,
                            ViBoolean autoRef,
                            ViReal64  level)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_NB_IOT_DOWNLINK_AUTO_REFERENCE_LEVEL, autoRef),
            2, "Auto");

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_NB_IOT_DOWNLINK_REFERENCE_LEVEL_IN_RF_MODE, level),
            3, "Level");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Defines the reference level when the input source is baseband.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR level/Defines the reference level when the input source is baseband.
ViStatus _VI_FUNC rsspecan_ConfigureNBIoTDownlinkLevelReferenceBaseband(
                            ViSession instrSession,
                            ViReal64  level)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_NB_IOT_DOWNLINK_REFERENCE_LEVEL_IN_BB_MODE, level),
            2, "Level");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Selects the external attenuation or gain applied to the RF signal.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/Sets the window parameter.
/// HIPAR level/Selects the external attenuation or gain applied to the RF signal.
/// HIPAR trace/Sets the trace parameter.
ViStatus _VI_FUNC rsspecan_ConfigureNBIoTDownlinkLevelExternalAttenuation(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   trace,
                            ViReal64  level)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, window, 0, 16),
            2, "Window");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, trace, 0, 8),
            3, "Trace");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "Win%ld,TR%ld", window, trace);

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_NB_IOT_DOWNLINK_TRACE_EXTERNAL_ATTENUATION, level),
            4, "Level");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Turns a swap of the I and Q branches on and off.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR swapIQ/Turns a swap of the I and Q branches on and off.
ViStatus _VI_FUNC rsspecan_ConfigureNBIoTDownlinkSwapIQ(ViSession instrSession,
                                                        ViBoolean swapIQ)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_NB_IOT_DOWNLINK_SWAP_IQ, swapIQ),
            2, "Swap IQ");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Defines the track time for the auto level process.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR autoLevelTrackTime/Defines the track time for the auto level process.
ViStatus _VI_FUNC rsspecan_ConfigureNBIoTDownlinkAutoLevelTrackTime(
                            ViSession instrSession,
                            ViReal64  autoLevelTrackTime)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_NB_IOT_DOWNLINK_AUTO_LEVEL_TRACK_TIME, autoLevelTrackTime),
            2, "Auto Level Track Time");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Configures the Frame.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR enabled/Turns manual selection of the number of frames you want to analyze on
/// HIPAR enabled/and off.
/// HIPAR autoCount/Sets the number of frames you want to analyze.
/// HIPAR count/Sets the number of frames you want to analyze.
ViStatus _VI_FUNC rsspecan_ConfigureNBIoTDownlinkFrame(ViSession instrSession,
                                                       ViBoolean enabled,
                                                       ViBoolean autoCount,
                                                       ViInt32   count)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_NB_IOT_DOWNLINK_FRAME_COUNT_ENABLED, enabled),
            2, "Enabled");

    if (enabled == VI_TRUE)
    {
        viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_NB_IOT_DOWNLINK_FRAME_COUNT_AUTO, autoCount),
                3, "Auto");

        if (autoCount == VI_FALSE)
        {
            viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_NB_IOT_DOWNLINK_FRAME_COUNT, count),
                    4, "Count");
        }
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Turns the wideband filter on and off.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR type/Turns the wideband filter on and off.
ViStatus _VI_FUNC rsspecan_ConfigureNBIoTDownlinkFilter(ViSession instrSession,
                                                        ViInt32   type)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_NB_IOT_DOWNLINK_FILTER_TYPE, type),
            2, "Type");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Configures the YIG Filter.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR enabled/Removes or adds the YIG filter from the signal path.
/// HIPAR autoFilter/Turns automatic control of the YIG filter on and off.
ViStatus _VI_FUNC rsspecan_ConfigureNBIoTDownlinkYIGFilter(ViSession instrSession,
                                                           ViBoolean enabled,
                                                           ViBoolean autoFilter)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_NB_IOT_DOWNLINK_YIG_FILTER_ENABLED, enabled),
            2, "Enabled");

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_NB_IOT_DOWNLINK_YIG_FILTER_AUTO, autoFilter),
            3, "Auto Filter");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures NPDSCH subframe.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR enabled/Turns automatic NPDSCH and NPDCCH demodulation on and off.
/// HIPAR subframes/Defines the subframes that contain the NPDSCH.
/// HIPAR ueID/Defines the N_RNTI (user equipment ID) that contain the NPDSCH.
ViStatus _VI_FUNC rsspecan_ConfigureNBIoTDownlinkNPDSCH(ViSession instrSession,
                                                        ViBoolean enabled,
                                                        ViString  subframes,
                                                        ViInt32   ueID)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, NULL, RSSPECAN_ATTR_NB_IOT_DOWNLINK_NPDSCH_ENABLED, enabled),
            2, "Enabled");

    viCheckParm(rsspecan_SetAttributeViString(instrSession, NULL, RSSPECAN_ATTR_NB_IOT_DOWNLINK_NPDSCH_SUBFRAMES, subframes),
            3, "Subframes");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, NULL, RSSPECAN_ATTR_NB_IOT_DOWNLINK_NPDSCH_UE_ID, ueID),
            4, "UE ID");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Configures the Demodulation Data Analysis.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channelEstimation/Selects the channel estimation type for downlink signals.
/// HIPAR evmCalculationMethod/Selects the EVM calculation method.
/// HIPAR npdschReferenceData/Selects the type of reference data to calculate the EVM for the
/// HIPAR npdschReferenceData/NPDSCH.
/// HIPAR multicarrierFilter/Turns suppression of interfering neighboring carriers on and off (e.g.
/// HIPAR multicarrierFilter/LTE, WCDMA, GSM etc).
ViStatus _VI_FUNC rsspecan_ConfigureNBIoTDownlinkDataAnalysis(
                            ViSession instrSession,
                            ViInt32   channelEstimation,
                            ViInt32   evmCalculationMethod,
                            ViInt32   npdschReferenceData,
                            ViBoolean multicarrierFilter)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_NB_IOT_DOWNLINK_CHANNEL_ESTIMATION_TYPE, channelEstimation),
            2, "Channel Estimation");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_NB_IOT_DOWNLINK_EVM_CALCULATION, evmCalculationMethod),
            3, "EVM Calculation Method");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_NB_IOT_DOWNLINK_REFERENCE_DATA, npdschReferenceData),
            4, "npdsch Reference Data");

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_NB_IOT_DOWNLINK_MULTICARRIER_FILTER, multicarrierFilter),
            5, "Multicarrier Filter");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Turns MIMO crosstalk compensation on and off.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR compensateCrosstalk/Turns MIMO crosstalk compensation on and off.
ViStatus _VI_FUNC rsspecan_ConfigureNBIoTDownlinkCompensateCrosstalk(
                            ViSession instrSession,
                            ViBoolean compensateCrosstalk)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, NULL, RSSPECAN_ATTR_NB_IOT_DOWNLINK_COMPENSATE_CROSSTALK, compensateCrosstalk),
            2, "Compensate Crosstalk");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Configures the Demodulation Tracking.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR phase/Selects the phase tracking type.
/// HIPAR timing/Turns timing tracking on and off.
ViStatus _VI_FUNC rsspecan_ConfigureNBIoTDownlinkTracking(ViSession instrSession,
                                                          ViInt32   phase,
                                                          ViBoolean timing)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_NB_IOT_DOWNLINK_PHASE_TRACKING_TYPE, phase),
            2, "Phase");

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_NB_IOT_DOWNLINK_TRACKING_TIME, timing),
            3, "Timing");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Configures the Demodulation Physical Layer Cell Identity.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR autoIdentity/Defines the physical layer cell identity for downlink signals.
/// HIPAR cellID/Defines the physical layer cell identity for downlink signals.
/// HIPAR cellIDGroup/Configures the cell identity group that has been detected.
/// HIPAR identity/Configures the cell identity that has been detected.
ViStatus _VI_FUNC rsspecan_ConfigureNBIoTDownlinkPhysicalLayerCellIdentity(
                            ViSession instrSession,
                            ViBoolean autoIdentity,
                            ViInt32   cellID,
                            ViInt32   cellIDGroup,
                            ViInt32   identity)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_NB_IOT_DOWNLINK_PHYSICAL_LAYER_CELL_IDENTITY_AUTO, NULL));

    if (autoIdentity == VI_FALSE)
    {
        viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_NB_IOT_DOWNLINK_PHYSICAL_LAYER_CELL_IDENTITY, cellID),
                3, "Cell ID");

        viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_NB_IOT_DOWNLINK_PHYSICAL_LAYER_CELL_GROUP, cellIDGroup),
                4, "Cell ID Group");

        viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_NB_IOT_DOWNLINK_PHYSICAL_LAYER_CELL_IDENTITY_DETECTED, identity),
                5, "Identity");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Activates or deactivates scrambling of coded bits for DL.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR scrambling/Activates or deactivates scrambling of coded bits for DL.
ViStatus _VI_FUNC rsspecan_ConfigureNBIoTDownlinkScramblingofCodedBits(
                            ViSession instrSession,
                            ViBoolean scrambling)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, NULL, RSSPECAN_ATTR_NB_IOT_DOWNLINK_SCRAMBLING_OF_CODED_BITS, scrambling),
            2, "Scrambling");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures SEM settings.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR category/Selects the baste station category to test. The base station category
/// HIPAR category/defines the shape of the SEM limit line.
/// HIPAR txPowerEnabled/Turns automatic detection of the TX channel power on and off.
/// HIPAR txPower/Defines the Tx power for medium range base stations. Prerequisites for
/// HIPAR txPower/this atributte.
/// HIPAR carrierPowerMode/Turns automatic detection of the TX channel power on and off.
/// HIPAR carrierPower/This atribute defines the power of the NB-IoT carrier on which the
/// HIPAR carrierPower/calculation of the
ViStatus _VI_FUNC rsspecan_ConfigureNBIoTDownlinkSEM(ViSession instrSession,
                                                     ViInt32   category,
                                                     ViBoolean txPowerEnabled,
                                                     ViReal64  txPower,
                                                     ViInt32   carrierPowerMode,
                                                     ViReal64  carrierPower)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, NULL, RSSPECAN_ATTR_NB_IOT_DOWNLINK_SEM_CATEGORY, category),
            2, "Category");

    if (category == RSSPECAN_VAL_NB_IOT_SEM_CATEGORY_MED)
    {
        viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, NULL, RSSPECAN_ATTR_NB_IOT_DOWNLINK_SEM_TX_POWER_ENABLED, txPowerEnabled),
                3, "Power Enabled");

        if (txPowerEnabled == VI_FALSE)
        {
            viCheckParm(rsspecan_SetAttributeViReal64(instrSession, NULL, RSSPECAN_ATTR_NB_IOT_DOWNLINK_SEM_TX_POWER, txPower),
                    4, "Tx Power");
        }
    }

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, NULL, RSSPECAN_ATTR_NB_IOT_DOWNLINK_SEM_NB_IOT_CARRIER_POWER_MODE, carrierPowerMode),
            5, "Carrier Power Mode");

    if (carrierPowerMode == RSSPECAN_VAL_NB_IOT_SEM_POWER_CARRIER_MODE_MAN)
    {
        viCheckParm(rsspecan_SetAttributeViReal64(instrSession, NULL, RSSPECAN_ATTR_NB_IOT_DOWNLINK_SEM_NB_IOT_CARRIER_POWER, carrierPower),
                6, "Carrier Power");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function adds a window to the display.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR stringSize/This control sets the allocated length of Returned Window Name string.
/// HIPAR referenceWindowName/String containing the number or name of the window the new window is
/// HIPAR referenceWindowName/inserted next to. In the default state, the name of the window is its
/// HIPAR referenceWindowName/index.
/// HIPAR direction/Direction the window is added relative to the window set with
/// HIPAR direction/"Reference Window Name".
/// HIPAR windowType/Type of result display you want to add.
/// HIPAR returnedWindowName/When adding a new window, the function returns its name (by
ViStatus _VI_FUNC rsspecan_NBIoTDownlinkAddWindow(ViSession instrSession,
                                                  ViInt32   stringSize,
                                                  ViString  referenceWindowName,
                                                  ViInt32   direction,
                                                  ViInt32   windowType,
                                                  ViChar    returnedWindowName[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViChar   response[RS_MAX_MESSAGE_BUF_SIZE];
    ViChar*  p2direction;
    ViChar*  p2type;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &rsspecan_rngLayoutDirection, direction, &p2direction),
                4, "Direction");
    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &rsspecan_rngNBIoTDownlinkLayoutType, windowType, &p2type),
                5, "Window Type");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "LAY:ADD? '%s',%s,%s", referenceWindowName, p2direction, p2type);
    checkErr(RsCore_QueryViString(instrSession, cmd, response));

    if (returnedWindowName != NULL)
    {
        RsCore_TrimString(response, RS_VAL_TRIM_WHITESPACES_AND_ALL_QUOTES);
        RsCore_CopyToUserBufferAsciiData(returnedWindowName, stringSize, response);
    }

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function replaces a window with another one.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR windowName/String containing the number or name of the window you want to
/// HIPAR windowType/Type of result display you want to add.
ViStatus _VI_FUNC rsspecan_NBIoTDownlinkReplaceWindow(ViSession instrSession,
                                                      ViString  windowName,
                                                      ViInt32   windowType)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViChar*  p2type;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &rsspecan_rngNBIoTDownlinkLayoutType, windowType, &p2type),
                3, "Window Type");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "LAY:REPL '%s',%s", windowName, p2type);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects a tab in diagrams with multiple subwindows (or views). Note
/// HIFN  that selecting a tab does not actually select a subwindow. To select a
/// HIFN  subwindow, for example to query the results of a subwindow, use
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR tab/Selects the tab.
ViStatus _VI_FUNC rsspecan_NBIoTDownlinkSelectTab(ViSession instrSession,
                                                  ViInt32   window,
                                                  ViInt32   tab)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_tab[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_NB_IOT_DOWNLINK_TAB_SELECT,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap));
    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_NB_IOT_DOWNLINK_TAB_SELECT,
            "Tab", tab - 1, RS_REPCAP_BUF_SIZE, repCap_tab));
    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_tab, NULL);

    checkErr(rsspecan_SetAttributeViString(instrSession, repCap, RSSPECAN_ATTR_NB_IOT_DOWNLINK_TAB_SELECT, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function selects the measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR measurement/Selects the measurement.
ViStatus _VI_FUNC rsspecan_ConfigureNBIoTDownlinkMeasurementMode(
                            ViSession instrSession,
                            ViInt32   measurement)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, NULL, RSSPECAN_ATTR_NB_IOT_DOWNLINK_MEASUREMENT, measurement),
            2, "Measurement");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects the EVM unit.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR evmUnit/Selects the EVM unit.
ViStatus _VI_FUNC rsspecan_ConfigureNBIoTDownlinkAnalysisUnit(
                            ViSession instrSession,
                            ViInt32   evmUnit)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_NB_IOT_DOWNLINK_ANALYSIS_EVM_UNIT, evmUnit),
            2, "EVM Unit");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Turns the result summary on and off.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR resultTable/Turns the result summary on and off.
ViStatus _VI_FUNC rsspecan_ConfigureNBIoTDownlinkAnalysisResultTableEnabled(
                            ViSession instrSession,
                            ViBoolean resultTable)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_NB_IOT_DOWNLINK_ANALYSIS_RESULT_TABLE_ENABLED, resultTable),
            2, "Result Table");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the current synchronization state.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR state/Queries the current synchronization state.
ViStatus _VI_FUNC rsspecan_QueryNBIoTDownlinkAnalysisSynchronizationState(
                            ViSession instrSession,
                            ViChar    state[])
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_GetAttributeViString(instrSession, "", RSSPECAN_ATTR_NB_IOT_DOWNLINK_ANALYSIS_SYNCHRONIZATION_STATE,
            512, state),
            2, "State");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Positions a marker on the minimum or maximum value of the trace.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR marker/Sets the marker parameter.
/// HIPAR searchType/Positions a marker on the minimum or maximum value of the trace.
ViStatus _VI_FUNC rsspecan_ConfigureNBIoTDownlinkAnalysisMarkerSearch(
                            ViSession instrSession,
                            ViInt32   marker,
                            ViInt32   searchType)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, searchType, RSSPECAN_VAL_NBIOT_SEARCH_MARKER_MIN, RSSPECAN_VAL_NBIOT_SEARCH_MARKER_MAX),
            2, "Search Type");

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_NB_IOT_DOWNLINK_ANALYSIS_MARKER_MINIMUM,
            "Marker", marker , RS_REPCAP_BUF_SIZE, repCap));

    if (searchType == RSSPECAN_VAL_NBIOT_SEARCH_MARKER_MIN)
    {
        checkErr(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_NB_IOT_DOWNLINK_ANALYSIS_MARKER_MINIMUM, NULL));
    }
    else
    {
        checkErr(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_NB_IOT_DOWNLINK_ANALYSIS_MARKER_MAXIMUM, NULL));
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Configures the Y Scale.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR YAuto/Automatically adjusts the scale of the y-axis to the current
/// HIPAR YAuto/measurement results.
/// HIPAR pointOfOrigin/Defines the point of origin of the y-axis and thus has an effect on
/// HIPAR pointOfOrigin/the scale of the y-axis.
/// HIPAR distanceBetweenGrid/Defines the distance between two grid lines on the y-axis and thus has
/// HIPAR distanceBetweenGrid/an effect on the scale of the y-axis.
ViStatus _VI_FUNC rsspecan_ConfigureNBIoTDownlinkAnalysisYScale(
                            ViSession instrSession,
                            ViBoolean YAuto,
                            ViReal64  pointOfOrigin,
                            ViReal64  distanceBetweenGrid)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_NB_IOT_DOWNLINK_ANALYSIS_Y_AUTO, YAuto),
            2, "Auto");

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_NB_IOT_DOWNLINK_ANALYSIS_Y_POINT_OF_ORIGIN, pointOfOrigin),
            3, "Point Of Origin");

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_NB_IOT_DOWNLINK_ANALYSIS_Y_DISTANCE_BETWEEN_GRID, distanceBetweenGrid),
            4, "Distance Between Grid");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the Y Scale.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR subwindow/This control selects the measurement subwindow.
/// HIPAR type/Selects the type of auto scale
/// HIPAR maxminValue/Defines the maximum value displayed on the vertical diagram axis. Only
/// HIPAR maxminValue/availible if type is set to maximum or minimum.
ViStatus _VI_FUNC rsspecan_ConfigureNBIoTDownlinkAnalysisYAxisScaling(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   subwindow,
                            ViInt32   type,
                            ViReal64  maxminValue)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_tab[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_NB_IOT_DOWNLINK_SUBWINDOW_Y_AXIS_SCALING_AUTO,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap));
    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_NB_IOT_DOWNLINK_SUBWINDOW_Y_AXIS_SCALING_AUTO,
            "Subwindow", subwindow - 1, RS_REPCAP_BUF_SIZE, repCap_tab));
    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_tab, NULL);

    switch (type)
    {
    case RSSPECAN_VAL_SCALING_AUTO:
        {
            checkErr(rsspecan_SetAttributeViString(instrSession, repCap, RSSPECAN_ATTR_NB_IOT_DOWNLINK_SUBWINDOW_Y_AXIS_SCALING_AUTO, NULL));
            break;
        }
    case RSSPECAN_VAL_SCALING_RESTORE:
        {
            checkErr(rsspecan_SetAttributeViString(instrSession, repCap, RSSPECAN_ATTR_NB_IOT_DOWNLINK_SUBWINDOW_Y_AXIS_SCALING_RESTORE, NULL));
            break;
        }
    case RSSPECAN_VAL_SCALING_ALL_WINDOWS:
        {
            checkErr(rsspecan_SetAttributeViString(instrSession, repCap, RSSPECAN_ATTR_NB_IOT_DOWNLINK_SUBWINDOW_Y_AXIS_SCALING_ALL_WINDOWS, NULL));
            break;
        }
    case RSSPECAN_VAL_SCALING_MAXIMUM:
        {
            viCheckParm(rsspecan_SetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_NB_IOT_DOWNLINK_SUBWINDOW_Y_AXIS_SCALING_MAXIMUM, maxminValue),
                    5, "Max/Min Value");
            break;
        }
    case RSSPECAN_VAL_SCALING_MINIMUM:
        {
            viCheckParm(rsspecan_SetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_NB_IOT_DOWNLINK_SUBWINDOW_Y_AXIS_SCALING_MINIMUM, maxminValue),
                    5, "Max/Min Value");
            break;
        }
    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, type), 4, "Type");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Selects the measurement and result display.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR measurement/Selects the measurement and result display.
ViStatus _VI_FUNC rsspecan_ConfigureNBIoTDownlinkMeasurement(ViSession instrSession,
                                                             ViInt32   window,
                                                             ViInt32   measurement)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_NB_IOT_DOWNLINK_RESULT,
            "Calculate", window , RS_REPCAP_BUF_SIZE, repCap));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_NB_IOT_DOWNLINK_RESULT, measurement),
            3, "Measurement");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Specifies if the bit stream gets displayed using bits or using
/// HIFN  symbols.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR bitStreamFormat/Specifies if the bit stream gets displayed using bits or using
/// HIPAR bitStreamFormat/symbols.
ViStatus _VI_FUNC rsspecan_ConfigureNBIoTDownlinkBitStreamFormat(
                            ViSession instrSession,
                            ViInt32   bitStreamFormat)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, NULL, RSSPECAN_ATTR_NB_IOT_DOWNLINK_BIT_STREAM_FORMAT, bitStreamFormat),
            2, "Bit Stream Format");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Filters the displayed results in the constellation diagram by a
/// HIFN  particular type of allocation.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR select/Selects the values to set.
/// HIPAR allocation/Filters the displayed results in the constellation diagram by a
/// HIPAR allocation/particular type of allocation.
ViStatus _VI_FUNC rsspecan_ConfigureNBIoTDownlinkAnalysisSelectAllocation(
                            ViSession instrSession,
                            ViInt32   select,
                            ViInt32   allocation)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, select, RSSPECAN_VAL_NBIOT_SELECT_ALL, RSSPECAN_VAL_NBIOT_SELECT_VALUE),
            2, "Select");

    if (select == RSSPECAN_VAL_NBIOT_SELECT_ALL)
    {
        checkErr(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_NB_IOT_DOWNLINK_ANALYSIS_ALLOCATION_SELECT_ALL, NULL));
    }
    else
    {
        viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_NB_IOT_DOWNLINK_ANALYSIS_ALLOCATION_SELECT, allocation),
                3, "Allocation");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Filters the displayed results in the constellation diagram by a
/// HIFN  particular subcarrier.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR select/Selects the values to set.
/// HIPAR carrier/Filters the displayed results in the constellation diagram by a
/// HIPAR carrier/particular subcarrier.
ViStatus _VI_FUNC rsspecan_ConfigureNBIoTDownlinkAnalysisSelectCarrier(
                            ViSession instrSession,
                            ViInt32   select,
                            ViInt32   carrier)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, select, RSSPECAN_VAL_NBIOT_SELECT_ALL, RSSPECAN_VAL_NBIOT_SELECT_VALUE),
            2, "Select");

    if (select == RSSPECAN_VAL_NBIOT_SELECT_ALL)
    {
        checkErr(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_NB_IOT_DOWNLINK_ANALYSIS_CARRIER_SELECT_ALL, NULL));
    }
    else
    {
        viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_NB_IOT_DOWNLINK_ANALYSIS_CARRIER_SELECT, carrier),
                3, "Carrier");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Filters the displayed results in the constellation diagram by a
/// HIFN  particular type of modulation.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR select/Selects the values to set.
/// HIPAR modulation/Filters the displayed results in the constellation diagram by a
/// HIPAR modulation/particular type of modulation.
ViStatus _VI_FUNC rsspecan_ConfigureNBIoTDownlinkAnalysisSelectModulation(
                            ViSession instrSession,
                            ViInt32   select,
                            ViInt32   modulation)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, select, RSSPECAN_VAL_NBIOT_SELECT_ALL, RSSPECAN_VAL_NBIOT_SELECT_VALUE),
            2, "Select");

    if (select == RSSPECAN_VAL_NBIOT_SELECT_ALL)
    {
        checkErr(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_NB_IOT_DOWNLINK_ANALYSIS_MODULATION_SELECT_ALL, NULL));
    }
    else
    {
        viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_NB_IOT_DOWNLINK_ANALYSIS_MODULATION_SELECT, modulation),
                3, "Modulation");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Filters the displayed results in the constellation diagram by a
/// HIFN  particular OFDM symbol.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR select/Selects the values to set.
/// HIPAR symbol/Filters the displayed results in the constellation diagram by a
/// HIPAR symbol/particular OFDM symbol.
ViStatus _VI_FUNC rsspecan_ConfigureNBIoTDownlinkAnalysisSelectSymbol(
                            ViSession instrSession,
                            ViInt32   select,
                            ViInt32   symbol)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, select, RSSPECAN_VAL_NBIOT_SELECT_ALL, RSSPECAN_VAL_NBIOT_SELECT_VALUE),
            2, "Select");

    if (select == RSSPECAN_VAL_NBIOT_SELECT_ALL)
    {
        checkErr(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_NB_IOT_DOWNLINK_ANALYSIS_SYMBOL_SELECT_ALL, NULL));
    }
    else
    {
        viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_NB_IOT_DOWNLINK_ANALYSIS_SYMBOL_SELECT, symbol),
                3, "Symbol");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects the data source of the constellation diagram.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR select/Selects the data source of the constellation diagram.
ViStatus _VI_FUNC rsspecan_ConfigureNBIoTDownlinkAnalysisLocationSelect(
                            ViSession instrSession,
                            ViInt32   select)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_NB_IOT_DOWNLINK_ANALYSIS_LOCATION_SELECT, select),
            2, "Select");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Selects all the subframes to be analyzed.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsspecan_ConfigureNBIoTDownlinkAnalysisSubframeSelectAll(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_NB_IOT_DOWNLINK_ANALYSIS_SUBFRAME_SELECT_ALL, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Selects the subframe to be analyzed.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR subframe/Selects the subframe to be analyzed.
ViStatus _VI_FUNC rsspecan_ConfigureNBIoTDownlinkAnalysisSubframeSelect(
                            ViSession instrSession,
                            ViInt32   subframe)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_NB_IOT_DOWNLINK_ANALYSIS_SUBFRAME_SELECT, subframe),
            2, "Subframe");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries NB IoT Downlink measurement EVM All result.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR measurementType/Selects measurement type.
/// HIPAR result/Returns selected result. Depending on the EVM unit selected in the
/// HIPAR result/"General Settings" menu the value will be either in dB or in percent.
ViStatus _VI_FUNC rsspecan_QueryNBIoTDownlinkMeasurementEVMAll(
                            ViSession instrSession,
                            ViInt32   measurementType,
                            ViReal64* result)
{
    ViStatus                                              error = VI_SUCCESS;
    ViAttr                                                attr[4] = {
                                                          RSSPECAN_ATTR_NB_IOT_DOWNLINK_EVM_ALL_RESULT_MINIMUM,
    RSSPECAN_ATTR_NB_IOT_DOWNLINK_EVM_ALL_RESULT_MAXIMUM, 0,
                                                          RSSPECAN_ATTR_NB_IOT_DOWNLINK_EVM_ALL_RESULT_AVERAGE
                                                          };

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, measurementType, RSSPECAN_VAL_MEASTYPE_MIN, RSSPECAN_VAL_MEASTYPE_AVER),
            2, "Measurement Type");

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, "",
                    attr[measurementType - 1],
                    result), 3, "Result");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries NB IoT Downlink measurement EVM Physical Channel result.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR measurementType/Selects measurement type.
/// HIPAR result/Returns selected result. Depending on the EVM unit selected in the
/// HIPAR result/"General Settings" menu the value will be either in dB or in percent.
ViStatus _VI_FUNC rsspecan_QueryNBIoTDownlinkMeasurementEVMPhysicalChannel(
                            ViSession instrSession,
                            ViInt32   measurementType,
                            ViReal64* result)
{
    ViStatus                                                           error = VI_SUCCESS;
    ViAttr                                                             attr[4] = {
                                                                       RSSPECAN_ATTR_NB_IOT_DOWNLINK_EVM_PHYSICAL_CHANNEL_RESULT_MIN,
    RSSPECAN_ATTR_NB_IOT_DOWNLINK_EVM_PHYSICAL_CHANNEL_RESULT_MAXIMUM, 0,
                                                                       RSSPECAN_ATTR_NB_IOT_DOWNLINK_EVM_PHYSICAL_CHANNEL_RESULT_AVERAGE
                                                                       };

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, measurementType, RSSPECAN_VAL_MEASTYPE_MIN, RSSPECAN_VAL_MEASTYPE_AVER),
            2, "Measurement Type");

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, "",
                    attr[measurementType - 1],
                    result), 3, "Result");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries NB IoT Downlink measurement EVM Physical Signal result.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR measurementType/Selects measurement type.
/// HIPAR result/Returns selected result. Depending on the EVM unit selected in the
/// HIPAR result/"General Settings" menu the value will be either in dB or in percent.
ViStatus _VI_FUNC rsspecan_QueryNBIoTDownlinkMeasurementEVMPhysicalSignal(
                            ViSession instrSession,
                            ViInt32   measurementType,
                            ViReal64* result)
{
    ViStatus                                                      error = VI_SUCCESS;
    ViAttr                                                        attr[4] = {
                                                                  RSSPECAN_ATTR_NB_IOT_DOWNLINK_EVM_PHYSICAL_SIGNAL_RESULT_MIN,
    RSSPECAN_ATTR_NB_IOT_DOWNLINK_EVM_PHYSICAL_SIGNAL_RESULT_MAX, 0,
                                                                  RSSPECAN_ATTR_NB_IOT_DOWNLINK_EVM_PHYSICAL_SIGNAL_RESULT_AVERAGE
                                                                  };

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, measurementType, RSSPECAN_VAL_MEASTYPE_MIN, RSSPECAN_VAL_MEASTYPE_AVER),
            2, "Measurement Type");

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, "",
                    attr[measurementType - 1],
                    result), 3, "Result");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries "Frequency Error" value from the result summary list in Hz.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR measurementType/    Selects measurement type.
/// HIPAR result/Returns selected result.
ViStatus _VI_FUNC rsspecan_QueryNBIoTDownlinkMeasurementFrequencyError(
                            ViSession instrSession,
                            ViInt32   measurementType,
                            ViReal64* result)
{
    ViStatus                                                  error = VI_SUCCESS;
    ViAttr                                                    attr[4] = {
                                                              RSSPECAN_ATTR_NB_IOT_DOWNLINK_FREQUENCY_ERROR_RESULT_MIN,
    RSSPECAN_ATTR_NB_IOT_DOWNLINK_FREQUENCY_ERROR_RESULT_MAX, 0,
                                                              RSSPECAN_ATTR_NB_IOT_DOWNLINK_FREQUENCY_ERROR_RESULT_AVERAGE
                                                              };

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, measurementType, RSSPECAN_VAL_MEASTYPE_MIN, RSSPECAN_VAL_MEASTYPE_AVER),
            2, "Measurement Type");

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, "",
                    attr[measurementType - 1],
                    result), 3, "Result");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Returns the reference signal transmit power (RSTP) from the result
/// HIFN  list.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR measurementType/Selects measurement type.
/// HIPAR result/Returns the RSTP value from the result list as defined in 3GPP TS
/// HIPAR result/36.141.
ViStatus _VI_FUNC rsspecan_QueryNBIoTDownlinkMeasurementRSTP(ViSession instrSession,
                                                             ViInt32   measurementType,
                                                             ViReal64* result)
{
    ViStatus                                       error = VI_SUCCESS;
    ViAttr                                         attr[4] = {
                                                   RSSPECAN_ATTR_NB_IOT_DOWNLINK_RSTP_RESULT_MIN,
    RSSPECAN_ATTR_NB_IOT_DOWNLINK_RSTP_RESULT_MAX, 0,
                                                   RSSPECAN_ATTR_NB_IOT_DOWNLINK_RSTP_RESULT_AVERAGE
                                                   };

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, measurementType, RSSPECAN_VAL_MEASTYPE_MIN, RSSPECAN_VAL_MEASTYPE_AVER),
            2, "Measurement Type");

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, "",
                    attr[measurementType - 1],
                    result), 3, "Result");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Returns the OSTP value from the result list in dBm.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR measurementType/Selects measurement type.
/// HIPAR result/Returns the OSTP value from the result list in dBm.
ViStatus _VI_FUNC rsspecan_QueryNBIoTDownlinkMeasurementOSTP(ViSession instrSession,
                                                             ViInt32   measurementType,
                                                             ViReal64* result)
{
    ViStatus                                       error = VI_SUCCESS;
    ViAttr                                         attr[4] = {
                                                   RSSPECAN_ATTR_NB_IOT_DOWNLINK_OSTP_RESULT_MIN,
    RSSPECAN_ATTR_NB_IOT_DOWNLINK_OSTP_RESULT_MAX, 0,
                                                   RSSPECAN_ATTR_NB_IOT_DOWNLINK_OSTP_RESULT_AVERAGE
                                                   };

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, measurementType, RSSPECAN_VAL_MEASTYPE_MIN, RSSPECAN_VAL_MEASTYPE_AVER),
            2, "Measurement Type");

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, "",
                    attr[measurementType - 1],
                    result), 3, "Result");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Returns the "Frame Power" from the result summary list in dBm.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR measurementType/Selects measurement type.
/// HIPAR result/Returns selected result.
ViStatus _VI_FUNC rsspecan_QueryNBIoTDownlinkMeasurementFramePower(
                            ViSession instrSession,
                            ViInt32   measurementType,
                            ViReal64* result)
{
    ViStatus                                              error = VI_SUCCESS;
    ViAttr                                                attr[4] = {
                                                          RSSPECAN_ATTR_NB_IOT_DOWNLINK_FRAME_POWER_RESULT_MIN,
    RSSPECAN_ATTR_NB_IOT_DOWNLINK_FRAME_POWER_RESULT_MAX, 0,
                                                          RSSPECAN_ATTR_NB_IOT_DOWNLINK_FRAME_POWER_RESULT_AVERAGE
                                                          };

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, measurementType, RSSPECAN_VAL_MEASTYPE_MIN, RSSPECAN_VAL_MEASTYPE_AVER),
            2, "Measurement Type");

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, "",
                    attr[measurementType - 1],
                    result), 3, "Result");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries "Sampling Error" value from the result summary list in ppm.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR measurementType/Selects measurement type.
/// HIPAR result/Returns selected result.
ViStatus _VI_FUNC rsspecan_QueryNBIoTDownlinkMeasurementSamplingError(
                            ViSession instrSession,
                            ViInt32   measurementType,
                            ViReal64* result)
{
    ViStatus                                                 error = VI_SUCCESS;
    ViAttr                                                   attr[4] = {
                                                             RSSPECAN_ATTR_NB_IOT_DOWNLINK_SAMPLING_ERROR_RESULT_MIN,
    RSSPECAN_ATTR_NB_IOT_DOWNLINK_SAMPLING_ERROR_RESULT_MAX, 0,
                                                             RSSPECAN_ATTR_NB_IOT_DOWNLINK_SAMPLING_ERROR_RESULT_AVERAGE
                                                             };

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, measurementType, RSSPECAN_VAL_MEASTYPE_MIN, RSSPECAN_VAL_MEASTYPE_AVER),
            2, "Measurement Type");

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, "",
                    attr[measurementType - 1],
                    result), 3, "Result");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the (sub)frame start offset as shown in the Capture Buffer
/// HIFN  result display.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR result/Queries the (sub)frame start offset as shown in the Capture Buffer
/// HIPAR result/result display.
ViStatus _VI_FUNC rsspecan_QueryNBIoTDownlinkMeasurementTriggerToFrame(
                            ViSession instrSession,
                            ViReal64* result)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_NB_IOT_DOWNLINK_TRIGGER_TO_FRAME_RESULT, result),
            2, "Result");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the average crest factor as shown in the result summary.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR result/Queries the (sub)frame start offset as shown in the Capture Buffer
/// HIPAR result/result display.
ViStatus _VI_FUNC rsspecan_QueryNBIoTDownlinkMeasurementCrestFactor(
                            ViSession instrSession,
                            ViReal64* result)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_NB_IOT_CREST_FACTOR_RESULT_AVERAGE, result),
            2, "Result");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  For the ACLR result display.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR arraySize/Sets the Array Size.
/// HIPAR result/For the ACLR result display.
/// HIPAR returnedValues/Returns the number of returned values.
ViStatus _VI_FUNC rsspecan_FetchNBIoTDownlinkACLR(ViSession instrSession,
                                                  ViInt32   window,
                                                  ViInt32   arraySize,
                                                  ViReal64  results[],
                                                  ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, window, 0, 16),
            2, "Window");
    checkErr(rsspecan_dataReadTrace(instrSession, window, "TRACE1", arraySize, results, returnedValues));
    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  For the allocation ID vs symbol x carrier, the method returns one
/// HIFN  value for each resource element.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR arraySize/Sets the Array Size.
/// HIPAR result/For the allocation ID vs symbol x carrier, the method returns one
/// HIPAR result/value for each resource element.
/// HIPAR returnedValues/Returns the number of returned values.
ViStatus _VI_FUNC rsspecan_FetchNBIoTDownlinkAllocationIDvsSymbolxCarrier(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   arraySize,
                            ViInt32   results[],
                            ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K106"));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, window, 0, 16),
            2, "Window");

    if (window == 0)
    {
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, ":TRAC? TRACE1");
    }
    else
    {
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, ":TRAC%ld? TRACE1", window);
    }

    checkErr(RsCore_QueryIntegerArrayToUserBuffer(instrSession, cmd, arraySize, results, returnedValues));
    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  For the allocation summary, the method returns NBIoTAllocationSummary
/// HIFN  object  for each line of the table.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR arraySize/Sets the Array Size.
/// HIPAR results/Order of results:
/// HIPAR returnedValues/Returns the number of returned values.
ViStatus _VI_FUNC rsspecan_FetchNBIoTDownlinkAllocationSummary(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   arraySize,
                            ViInt32   subframe[],
                            ViInt32   allocationID[],
                            ViInt32   numberOfRB[],
                            ViReal64  relativePower[],
                            ViInt32   modulation[],
                            ViReal64  absolutePower[],
                            ViReal64  evm[],
                            ViReal64  layerEVM[],
                            ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = ":TRAC? TRACE1";
    ViChar*  data = NULL;
    ViChar*  pbuf;
    ViChar*  pbufEnd;
    ViInt32  idx = 0;
    ViInt32  elements_count;

    checkErr(RsCore_LockSession(instrSession));

    if (window != 0)
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, ":TRAC%ld? TRACE1", window);
    checkErr(RsCore_QueryViStringUnknownLength(instrSession, cmd, &data));

    elements_count = RsCore_GetElementsCount(data, ',') / 8;

    if (arraySize < elements_count)
        viCheckParm(RsCore_InvalidViInt32RangeMin(instrSession, arraySize, elements_count), 3, "Array Size");

    pbuf = data;
    while (idx < (elements_count))
    {
        pbufEnd = strchr(pbuf, ',');
        if (pbufEnd == NULL)
        {
            checkErr(RS_ERROR_UNEXPECTED_RESPONSE);
        }
        *pbufEnd = 0;
        if (!isNullOrEmpty(pbuf))
            subframe[idx] = RsCore_Convert_String_To_ViInt32(pbuf);
        else
            subframe[idx] = RS_VAL_NAN_VI_INT32;
        pbuf = pbufEnd + 1;

        pbufEnd = strchr(pbuf, ',');
        if (pbufEnd == NULL)
        {
            checkErr(RS_ERROR_UNEXPECTED_RESPONSE);
        }
        *pbufEnd = 0;
        if (!isNullOrEmpty(pbuf))
            allocationID[idx] = RsCore_Convert_String_To_ViInt32(pbuf);
        else
            allocationID[idx] = RS_VAL_NAN_VI_INT32;
        pbuf = pbufEnd + 1;

        pbufEnd = strchr(pbuf, ',');
        if (pbufEnd == NULL)
        {
            checkErr(RS_ERROR_UNEXPECTED_RESPONSE);
        }
        *pbufEnd = 0;
        if (!isNullOrEmpty(pbuf))
            numberOfRB[idx] = RsCore_Convert_String_To_ViInt32(pbuf);
        else
            numberOfRB[idx] = RS_VAL_NAN_VI_INT32;
        pbuf = pbufEnd + 1;

        pbufEnd = strchr(pbuf, ',');
        if (pbufEnd == NULL)
        {
            checkErr(RS_ERROR_UNEXPECTED_RESPONSE);
        }
        *pbufEnd = 0;
        if (!isNullOrEmpty(pbuf))
            relativePower[idx] = RsCore_Convert_String_To_ViReal64(pbuf);
        else
            relativePower[idx] = RS_VAL_NAN_VI_REAL64;
        pbuf = pbufEnd + 1;

        pbufEnd = strchr(pbuf, ',');
        if (pbufEnd == NULL)
        {
            checkErr(RS_ERROR_UNEXPECTED_RESPONSE);
        }
        *pbufEnd = 0;
        if (!isNullOrEmpty(pbuf))
            modulation[idx] = RsCore_Convert_String_To_ViInt32(pbuf);
        else
            modulation[idx] = RS_VAL_NAN_VI_INT32;
        pbuf = pbufEnd + 1;

        pbufEnd = strchr(pbuf, ',');
        if (pbufEnd == NULL)
        {
            checkErr(RS_ERROR_UNEXPECTED_RESPONSE);
        }
        *pbufEnd = 0;
        if (!isNullOrEmpty(pbuf))
            absolutePower[idx] = RsCore_Convert_String_To_ViReal64(pbuf);
        else
            absolutePower[idx] = RS_VAL_NAN_VI_REAL64;
        pbuf = pbufEnd + 1;

        pbufEnd = strchr(pbuf, ',');
        if (pbufEnd == NULL)
        {
            checkErr(RS_ERROR_UNEXPECTED_RESPONSE);
        }
        *pbufEnd = 0;
        if (!isNullOrEmpty(pbuf))
            evm[idx] = RsCore_Convert_String_To_ViReal64(pbuf);
        else
            evm[idx] = RS_VAL_NAN_VI_REAL64;
        pbuf = pbufEnd + 1;

        pbufEnd = strchr(pbuf, ',');
        if (pbufEnd == NULL)
        {
            layerEVM[idx] = RsCore_Convert_String_To_ViReal64(pbuf);
        }
        else
        {
            *pbufEnd = 0;
            if (!isNullOrEmpty(pbuf))
                layerEVM[idx] = RsCore_Convert_String_To_ViReal64(pbuf);
            else
                layerEVM[idx] = RS_VAL_NAN_VI_REAL64;
            pbuf = pbufEnd + 1;
        }
        idx++;
    }

    *returnedValues = idx - 1;

Error:
    if (data) free(data);
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  For the capture buffer result display, the method returns one value
/// HIFN  for each I/Q sample in the capture buffer.
/// HIFN     The unit is always dBm.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR arraySize/Sets the Array Size.
/// HIPAR result/For the capture buffer result display, the method returns one value
/// HIPAR result/for each I/Q sample in the capture buffer.
/// HIPAR returnedValues/Returns the number of returned values.
ViStatus _VI_FUNC rsspecan_FetchNBIoTDownlinkCaptureBuffer(ViSession instrSession,
                                                           ViInt32   window,
                                                           ViInt32   arraySize,
                                                           ViReal64  results[],
                                                           ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, window, 0, 16),
            2, "Window");
    checkErr(rsspecan_dataReadTrace(instrSession, window, "TRACE1", arraySize, results, returnedValues));
    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  For the capture buffer result display, the method returns one value
/// HIFN  for each I/Q sample in the capture buffer.
/// HIFN     The unit is always dBm.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR axis/Sets the axis.
/// HIPAR arraySize/Sets the Array Size.
/// HIPAR result/For the capture buffer result display, the method returns one value
/// HIPAR result/for each I/Q sample in the capture buffer.
/// HIPAR returnedValues/Returns the number of returned values.
ViStatus _VI_FUNC rsspecan_FetchNBIoTDownlinkCCDF(ViSession instrSession,
                                                  ViInt32   window,
                                                  ViInt32   axis,
                                                  ViInt32   arraySize,
                                                  ViReal64  results[],
                                                  ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;
    ViString trace = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, window, 0, 16),
            2, "Window");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, axis, RSSPECAN_VAL_AXIS_X, RSSPECAN_VAL_AXIS_Y),
            3, "Axis");
    if (axis == RSSPECAN_VAL_AXIS_Y)
    {
        trace = "TRACE1";
    }
    else
    {
        trace = "TRACE2";
    }

    checkErr(rsspecan_dataReadTrace(instrSession, window, trace, arraySize, results, returnedValues));
    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  For the channel flatness result display, the method returns one value
/// HIFN  for each trace point.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR statistical/Selects the statistical type.
/// HIPAR arraySize/Sets the Array Size.
/// HIPAR result/For the channel flatness result display, the method returns one value
/// HIPAR result/for each trace point.
/// HIPAR returnedValues/Returns the number of returned values.
ViStatus _VI_FUNC rsspecan_FetchNBIoTDownlinkChannelAndSpectrumFlatness(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   statistical,
                            ViInt32   arraySize,
                            ViReal64  results[],
                            ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;
    ViPChar  trace;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, window, 0, 16),
            2, "Window");
    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &RsSpecAn_rngMinMaxAvg, statistical, &trace),
                3, "Statistical");

    checkErr(rsspecan_dataReadTrace(instrSession, window, trace, arraySize, results, returnedValues));
    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  For the channel flatness result display, the method returns one value
/// HIFN  for each trace point.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR statistical/Selects the statistical type.
/// HIPAR arraySize/Sets the Array Size.
/// HIPAR result/For the channel flatness result display, the method returns one value
/// HIPAR result/for each trace point.
/// HIPAR returnedValues/Returns the number of returned values.
ViStatus _VI_FUNC rsspecan_FetchNBIoTDownlinkChannelAndSpectrumFlatnessDifference(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   statistical,
                            ViInt32   arraySize,
                            ViReal64  results[],
                            ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;
    ViPChar  trace;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, window, 0, 16),
            2, "Window");
    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &RsSpecAn_rngMinMaxAvg, statistical, &trace),
                3, "Statistical");

    checkErr(rsspecan_dataReadTrace(instrSession, window, trace, arraySize, results, returnedValues));
    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  For the group delay result display, the method returns one value for
/// HIFN  each trace point.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR arraySize/Sets the Array Size.
/// HIPAR result/For the group delay result display, the method returns one value for
/// HIPAR result/each trace point.
/// HIPAR returnedValues/Returns the number of returned values.
ViStatus _VI_FUNC rsspecan_FetchNBIoTDownlinkGroupDelay(ViSession instrSession,
                                                        ViInt32   window,
                                                        ViInt32   arraySize,
                                                        ViReal64  results[],
                                                        ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, window, 0, 16),
            2, "Window");
    checkErr(rsspecan_dataReadTrace(instrSession, window, "TRACE1", arraySize, results, returnedValues));
    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  For the constellation diagram, the method returns one value pair
/// HIFN  for each constellation.
/// HIFN  The I and Q values have no unit.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR arraySize/Sets the Array Size.
/// HIPAR i/For the I of constellation.
/// HIPAR q/For the Q of constellation.
/// HIPAR returnedValues/Returns the number of returned values.
ViStatus _VI_FUNC rsspecan_FetchNBIoTDownlinkConstellation(ViSession instrSession,
                                                           ViInt32   window,
                                                           ViInt32   arraySize,
                                                           ViReal64  i[],
                                                           ViReal64  q[],
                                                           ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "TRAC? TRACE1";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, window, 0, 16),
            2, "Window");

    if (window != 0)
    {
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "TRAC%ld? TRACE1", window);
    }

    checkErr(rsspecan_dataReadInterleaved(instrSession, cmd, 10000, arraySize, i, q, returnedValues));
    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  For the EVM vs carrier result display, the method  returns one value
/// HIFN  for each subcarrier that has been analyzed.
/// HIFN
/// HIFN     The unit depends on UNIT:EVM
/// HIFN  (EVMUnit/RSSPECAN_ATTR_NB_IOT_DOWNLINK_ANALYSIS_EVM_UNIT).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR statistical/Selects the statistical type.
/// HIPAR arraySize/Sets the Array Size.
/// HIPAR result/For the EVM vs carrier result display, the method  returns one value
/// HIPAR result/for each subcarrier that has been analyzed.
/// HIPAR returnedValues/Returns the number of returned values.
ViStatus _VI_FUNC rsspecan_FetchNBIoTDownlinkEVMvsCarrier(ViSession instrSession,
                                                          ViInt32   window,
                                                          ViInt32   statistical,
                                                          ViInt32   arraySize,
                                                          ViReal64  results[],
                                                          ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;
    ViPChar  trace;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, window, 0, 16),
            2, "Window");
    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &RsSpecAn_rngMinMaxAvg, statistical, &trace),
                3, "Statistical");

    checkErr(rsspecan_dataReadTrace(instrSession, window, trace, arraySize, results, returnedValues));
    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  For the EVM vs subframe result display, the method  returns one value
/// HIFN  for each subframe that has been analyzed.
/// HIFN
/// HIFN     The unit depends on UNIT:EVM
/// HIFN  (EVMUnit/RSSPECAN_ATTR_NB_IOT_DOWNLINK_ANALYSIS_EVM_UNIT).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR arraySize/Sets the Array Size.
/// HIPAR result/For the EVM vs subframe result display, the method  returns one value
/// HIPAR result/for each subframe that has been analyzed.
/// HIPAR returnedValues/Returns the number of returned values.
ViStatus _VI_FUNC rsspecan_FetchNBIoTDownlinkEVMvsSubframe(ViSession instrSession,
                                                           ViInt32   window,
                                                           ViInt32   arraySize,
                                                           ViReal64  results[],
                                                           ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, window, 0, 16),
            2, "Window");
    checkErr(rsspecan_dataReadTrace(instrSession, window, "TRACE1", arraySize, results, returnedValues));
    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  For the EVM vs symbol result display, the method  returns one value
/// HIFN  for each OFDM symbol that has been analyzed.
/// HIFN
/// HIFN     The unit depends on UNIT:EVM
/// HIFN  (EVMUnit/RSSPECAN_ATTR_NB_IOT_DOWNLINK_ANALYSIS_EVM_UNIT).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR arraySize/Sets the Array Size.
/// HIPAR result/For the EVM vs symbol result display, the method  returns one value
/// HIPAR result/for each OFDM symbol that has been analyzed.
/// HIPAR returnedValues/Returns the number of returned values.
ViStatus _VI_FUNC rsspecan_FetchNBIoTDownlinkEVMvsSymbol(ViSession instrSession,
                                                         ViInt32   window,
                                                         ViInt32   arraySize,
                                                         ViReal64  results[],
                                                         ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, window, 0, 16),
            2, "Window");
    checkErr(rsspecan_dataReadTrace(instrSession, window, "TRACE1", arraySize, results, returnedValues));
    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  For the EVM vs symbol x carrier, the method returns one value for each
/// HIFN  resource element.
/// HIFN
/// HIFN     The unit depends on UNIT:EVM
/// HIFN  (EVMUnit/RSSPECAN_ATTR_NB_IOT_DOWNLINK_ANALYSIS_EVM_UNIT).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR arraySize/Sets the Array Size.
/// HIPAR result/For the EVM vs symbol x carrier, the method returns one value for each
/// HIPAR result/resource element.
/// HIPAR returnedValues/Returns the number of returned values.
ViStatus _VI_FUNC rsspecan_FetchNBIoTDownlinkEVMvsSymbolxCarrier(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   arraySize,
                            ViReal64  results[],
                            ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, window, 0, 16),
            2, "Window");
    checkErr(rsspecan_dataReadTrace(instrSession, window, "TRACE1", arraySize, results, returnedValues));
    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  For the frequency error vs symbol result display, the method returns
/// HIFN  one value for each OFDM symbol that has been analyzed.
/// HIFN
/// HIFN     The unit is always Hz.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR arraySize/Sets the Array Size.
/// HIPAR result/For the frequency error vs symbol result display, the method returns
/// HIPAR result/one value for each OFDM symbol that has been analyzed.
/// HIPAR returnedValues/Returns the number of returned values.
ViStatus _VI_FUNC rsspecan_FetchNBIoTDownlinkFrequencyErrorvsSymbol(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   arraySize,
                            ViReal64  results[],
                            ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, window, 0, 16),
            2, "Window");
    checkErr(rsspecan_dataReadTrace(instrSession, window, "TRACE1", arraySize, results, returnedValues));
    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  For the power spectrum result display, the method returns one value
/// HIFN  for each trace point.
/// HIFN
/// HIFN     The unit is always dBm/Hz.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR arraySize/Sets the Array Size.
/// HIPAR result/For the power spectrum result display, the method returns one value
/// HIPAR result/for each trace point.
/// HIPAR returnedValues/Returns the number of returned values.
ViStatus _VI_FUNC rsspecan_FetchNBIoTDownlinkPowerSpectrum(ViSession instrSession,
                                                           ViInt32   window,
                                                           ViInt32   arraySize,
                                                           ViReal64  results[],
                                                           ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, window, 0, 16),
            2, "Window");
    checkErr(rsspecan_dataReadTrace(instrSession, window, "TRACE1", arraySize, results, returnedValues));
    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  For the power vs symbol x carrier, the method returns one value for
/// HIFN  each resource element.
/// HIFN     <P[Symbol(0),Carrier(1)]>, ..., <P[Symbol(0),Carrier(n)]>,
/// HIFN     <P[Symbol(1),Carrier(1)]>, ..., <P[Symbol(1),Carrier(n)]>,
/// HIFN     ...
/// HIFN     <P[Symbol(n),Carrier(1)]>, ..., <P[Symbol(n),Carrier(n)]>
/// HIFN     with P = Power of a resource element.
/// HIFN     The unit is always dBm.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR arraySize/Sets the Array Size.
/// HIPAR result/For the power vs symbol x carrier, the method returns one value for
/// HIPAR result/each resource element.
/// HIPAR returnedValues/Returns the number of returned values.
ViStatus _VI_FUNC rsspecan_FetchNBIoTDownlinkPowervsSymbolxCarrier(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   arraySize,
                            ViReal64  results[],
                            ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, window, 0, 16),
            2, "Window");
    checkErr(rsspecan_dataReadTrace(instrSession, window, "TRACE1", arraySize, results, returnedValues));
    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  For the SEM  result display.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR arraySize/Sets the Array Size.
/// HIPAR result/For the SEM  result display.
/// HIPAR returnedValues/Returns the number of returned values.
ViStatus _VI_FUNC rsspecan_FetchNBIoTDownlinkSEM(ViSession instrSession,
                                                 ViInt32   window,
                                                 ViInt32   arraySize,
                                                 ViReal64  results[],
                                                 ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, window, 0, 16),
            2, "Window");
    checkErr(rsspecan_dataReadTrace(instrSession, window, "TRACE1", arraySize, results, returnedValues));
    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Returns the contents of the SEM table.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR arraySize/Sets the Array Size.
/// HIPAR result/Returns the contents of the SEM table.
/// HIPAR returnedValues/Returns the number of returned values.
ViStatus _VI_FUNC rsspecan_FetchNBIoTDownlinkSEMTable(ViSession instrSession,
                                                      ViInt32   window,
                                                      ViInt32   arraySize,
                                                      ViInt32   index[],
                                                      ViReal64  startFrequency[],
                                                      ViReal64  stopFrequency[],
                                                      ViReal64  rbw[],
                                                      ViReal64  limitFailFrequency[],
                                                      ViReal64  absolutePower[],
                                                      ViReal64  relativePower[],
                                                      ViReal64  limitDistance[],
                                                      ViInt32   limitCheckResult[],
                                                      ViReal64  reserved1[],
                                                      ViReal64  reserved2[],
                                                      ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = ":TRAC? LIST";
    ViChar*  data = NULL;
    ViChar*  pbuf;
    ViChar*  pbufEnd;
    ViInt32  idx = 0;
    ViInt32  elements_count;

    checkErr(RsCore_LockSession(instrSession));

    if (window != 0)
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, ":TRAC%ld? LIST", window);
    checkErr(RsCore_QueryViStringUnknownLength(instrSession, cmd, &data));

    elements_count = RsCore_GetElementsCount(data, ',') / 11;

    if (arraySize < elements_count)
        viCheckParm(RsCore_InvalidViInt32RangeMin(instrSession, arraySize, elements_count), 3, "Array Size");

    pbuf = data;
    while (idx < (elements_count))
    {
        pbufEnd = strchr(pbuf, ',');
        if (pbufEnd == NULL)
        {
            checkErr(RS_ERROR_UNEXPECTED_RESPONSE);
        }
        *pbufEnd = 0;
        if (!isNullOrEmpty(pbuf))
            index[idx] = (ViInt32)RsCore_Convert_String_To_ViReal64(pbuf);  // NOLINT(clang-diagnostic-bad-function-cast)
        else
            index[idx] = RS_VAL_NAN_VI_INT32;
        pbuf = pbufEnd + 1;

        pbufEnd = strchr(pbuf, ',');
        if (pbufEnd == NULL)
        {
            checkErr(RS_ERROR_UNEXPECTED_RESPONSE);
        }
        *pbufEnd = 0;
        if (!isNullOrEmpty(pbuf))
            startFrequency[idx] = RsCore_Convert_String_To_ViReal64(pbuf);
        else
            startFrequency[idx] = RS_VAL_NAN_VI_REAL64;
        pbuf = pbufEnd + 1;

        pbufEnd = strchr(pbuf, ',');
        if (pbufEnd == NULL)
        {
            checkErr(RS_ERROR_UNEXPECTED_RESPONSE);
        }
        *pbufEnd = 0;
        if (!isNullOrEmpty(pbuf))
            stopFrequency[idx] = RsCore_Convert_String_To_ViReal64(pbuf);
        else
            stopFrequency[idx] = RS_VAL_NAN_VI_REAL64;
        pbuf = pbufEnd + 1;

        pbufEnd = strchr(pbuf, ',');
        if (pbufEnd == NULL)
        {
            checkErr(RS_ERROR_UNEXPECTED_RESPONSE);
        }
        *pbufEnd = 0;
        if (!isNullOrEmpty(pbuf))
            rbw[idx] = RsCore_Convert_String_To_ViReal64(pbuf);
        else
            rbw[idx] = RS_VAL_NAN_VI_REAL64;
        pbuf = pbufEnd + 1;

        pbufEnd = strchr(pbuf, ',');
        if (pbufEnd == NULL)
        {
            checkErr(RS_ERROR_UNEXPECTED_RESPONSE);
        }
        *pbufEnd = 0;
        if (!isNullOrEmpty(pbuf))
            limitFailFrequency[idx] = RsCore_Convert_String_To_ViReal64(pbuf);
        else
            limitFailFrequency[idx] = RS_VAL_NAN_VI_REAL64;
        pbuf = pbufEnd + 1;

        pbufEnd = strchr(pbuf, ',');
        if (pbufEnd == NULL)
        {
            checkErr(RS_ERROR_UNEXPECTED_RESPONSE);
        }
        *pbufEnd = 0;
        if (!isNullOrEmpty(pbuf))
            absolutePower[idx] = RsCore_Convert_String_To_ViReal64(pbuf);
        else
            absolutePower[idx] = RS_VAL_NAN_VI_REAL64;
        pbuf = pbufEnd + 1;

        pbufEnd = strchr(pbuf, ',');
        if (pbufEnd == NULL)
        {
            checkErr(RS_ERROR_UNEXPECTED_RESPONSE);
        }
        *pbufEnd = 0;
        if (!isNullOrEmpty(pbuf))
            relativePower[idx] = RsCore_Convert_String_To_ViReal64(pbuf);
        else
            relativePower[idx] = RS_VAL_NAN_VI_REAL64;
        pbuf = pbufEnd + 1;

        pbufEnd = strchr(pbuf, ',');
        if (pbufEnd == NULL)
        {
            checkErr(RS_ERROR_UNEXPECTED_RESPONSE);
        }
        *pbufEnd = 0;
        if (!isNullOrEmpty(pbuf))
            limitDistance[idx] = RsCore_Convert_String_To_ViReal64(pbuf);
        else
            limitDistance[idx] = RS_VAL_NAN_VI_REAL64;
        pbuf = pbufEnd + 1;

        pbufEnd = strchr(pbuf, ',');
        if (pbufEnd == NULL)
        {
            checkErr(RS_ERROR_UNEXPECTED_RESPONSE);
        }
        *pbufEnd = 0;
        if (!isNullOrEmpty(pbuf))
            limitCheckResult[idx] = (ViInt32)RsCore_Convert_String_To_ViReal64(pbuf);  // NOLINT(clang-diagnostic-bad-function-cast)
        else
            limitCheckResult[idx] = RS_VAL_NAN_VI_INT32;
        pbuf = pbufEnd + 1;

        pbufEnd = strchr(pbuf, ',');
        if (pbufEnd == NULL)
        {
            checkErr(RS_ERROR_UNEXPECTED_RESPONSE);
        }
        *pbufEnd = 0;
        if (!isNullOrEmpty(pbuf))
            reserved1[idx] = RsCore_Convert_String_To_ViReal64(pbuf);
        else
            reserved1[idx] = RS_VAL_NAN_VI_REAL64;
        pbuf = pbufEnd + 1;

        pbufEnd = strchr(pbuf, ',');
        if (pbufEnd == NULL)
        {
            reserved2[idx] = RsCore_Convert_String_To_ViReal64(pbuf);
        }
        else
        {
            *pbufEnd = 0;
            if (!isNullOrEmpty(pbuf))
                reserved2[idx] = RsCore_Convert_String_To_ViReal64(pbuf);
            else
                reserved2[idx] = RS_VAL_NAN_VI_REAL64;
            pbuf = pbufEnd + 1;
        }
        idx++;
    }

    *returnedValues = idx - 1;

Error:
    if (data) free(data);
    (void)RsCore_UnlockSession(instrSession);
    return error;
}
