/*****************************************************************************
 *  Rohde&Schwarz Spectrum Analyzer instrument driver
 *  B10 - External Generator Control
 *
 *  Original Release: September 2005
 *  By: Martin Koutny (instrument driver core)
 *      Helena Sucha (instrument specific implementation)
 *      Jiri Kominek (DrBase repository & tools)
 *
 *  Should you have any technical questions please contact the hotline of
 *  Rohde & Schwarz Vertriebs-GmbH Rohde & Schwarz Support Center
 *
 *  e-mail: CustomerSupport@rohde-schwarz.com
 *

 *****************************************************************************/

#include "rsspecan.h"

/// HIFN This function configures the tracking generator normalization.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR state/Specifies whether the spectrum analyzer is enabled for auto
/// HIPAR state/attenuation.
ViStatus _VI_FUNC rsspecan_ExternalGeneratorNormalization(ViSession instrSession,
                                                          ViInt32   reserved,
                                                          ViBoolean state)
{
    ViStatus error = VI_SUCCESS;
             reserved;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_TGEN_NORMALIZATION_STATE, state),
            3, "State");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function activates the selected tracking generator
/// calibration.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR method/This control selects the kind of measurement with active
/// HIPAR method/tracking generator (transmission/reflexion) and kind of
/// HIPAR method/measurement for the reference values of the normalization
/// HIPAR method/(response calibration).
/// HIPAR timeout/This control sets the timeout value for data readout in
/// HIPAR timeout/milliseconds.
ViStatus _VI_FUNC rsspecan_ExternalGeneratorCalibration(ViSession instrSession,
                                                        ViInt32   reserved,
                                                        ViInt32   method,
                                                        ViUInt32  timeout)
{
    ViStatus error = VI_SUCCESS;
    ViUInt32 old_timeout = 0;
             reserved;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViUInt32Range(instrSession, timeout, 0, 4294967295UL), 4, "Timeout");

    checkErr(rsspecan_GetTimeout(instrSession, &old_timeout));

    checkErr(rsspecan_SetTimeout(instrSession, timeout));

    switch (method)
    {
    case RSSPECAN_VAL_TGEN_CAL_TRAN:
        checkErr(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_TGEN_METHOD, RSSPECAN_VAL_CORR_TRAN));
        break;
    case RSSPECAN_VAL_TGEN_CAL_REFLO:
        checkErr(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_TGEN_METHOD, RSSPECAN_VAL_CORR_REFL));
        checkErr(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_TGEN_ACQUIRE, RSSPECAN_VAL_CORR_ACQ_OPEN));
        break;
    case RSSPECAN_VAL_TGEN_CAL_REFLS:
        checkErr(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_TGEN_METHOD, RSSPECAN_VAL_CORR_REFL));
        checkErr(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_TGEN_ACQUIRE, RSSPECAN_VAL_CORR_ACQ_THR));
        break;
    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, method), 3, "Method");
    }

Error:
    if (old_timeout > 0)
        rsspecan_SetTimeout(instrSession, old_timeout);
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function restores the instrument setting that was valid for
/// HIFN the measurement of the reference data, provided that the
/// HIFN tracking generator is active.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
ViStatus _VI_FUNC rsspecan_ExternalGeneratorRecall(ViSession instrSession,
                                                   ViInt32   reserved)
{
    ViStatus error = VI_SUCCESS;
             reserved;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_TGEN_RECALL, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN This function uses the normalized measurement data to
/// HIFN generate a transducer factor with up to 625 points.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR name/This control sets the name of saved transducer factor.
ViStatus _VI_FUNC rsspecan_ExternalGeneratorSaveAsTrdFactor(ViSession instrSession,
                                                            ViInt32   reserved,
                                                            ViString  name)
{
    ViStatus error = VI_SUCCESS;
             reserved;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_TFAC_GEN_NAME, name));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function defines the reference position and value.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR referencePosition/Defines the position of the Result Frequency Stopin percent of
/// HIPAR referencePosition/the total y-axis range.
/// HIPAR referenceValue/Defines the reference value to be displayed at the specified
/// HIPAR referenceValue/Result Frequency Start.
ViStatus _VI_FUNC rsspecan_ExternalGeneratorReference(ViSession instrSession,
                                                      ViReal64  referencePosition,
                                                      ViReal64  referenceValue)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_DISP_REF_POSITION, referencePosition),
            2, "Reference Position");

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_DISP_REF_VALUE, referenceValue),
            3, "Reference Value");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function selects the external generator.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR generator/This control selects the external generator 1 or 2.
/// HIPAR generatorAddress/This control changes the IEC/IEEE-bus address of the device
/// HIPAR generatorAddress/selected as external generator.
/// HIPAR TTLHandshake/This control selects the link type of the external generator if
/// HIPAR TTLHandshake/the GPIB interface is used.
/// HIPAR generatorType/This control selects the type of external generator. The
/// HIPAR generatorType/available generator types including the associated interface are
/// HIPAR generatorType/listed in the documentation.
/// HIPAR reference/This control sets the reference signal for the external
/// HIPAR reference/generator.
ViStatus _VI_FUNC rsspecan_ConfigureExternalGeneratorGPIB(ViSession instrSession,
                                                          ViInt32   generator,
                                                          ViInt32   generatorAddress,
                                                          ViInt32   TTLHandshake,
                                                          ViString  generatorType,
                                                          ViInt32   reference)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_EXT_GEN_IEEE_ADDR,
            "Generator", generator - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Generator");

    viCheckParm(rsspecan_SetAttributeViString(instrSession, repCap, RSSPECAN_ATTR_EXT_GEN_TYPE, generatorType),
            5, "Generator Type");

    checkErr(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_EXT_GEN_INTERFACE_TYPE, RSSPECAN_VAL_INTERFACE_GPIB));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_EXT_GEN_IEEE_ADDR, generatorAddress),
            3, "Generator Address");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_EXT_GEN_INTF_TYPE, TTLHandshake),
            4, "TTL Handshake");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_EXT_GEN_ROSC_SOURCE, reference),
            7, "Reference");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function selects the external generator.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR generator/This control selects the external generator 1 or 2.
/// HIPAR generatorAddress/This control configures the TCP/IP address for the external
/// HIPAR generatorAddress/generator.
/// HIPAR generatorType/This control selects the type of external generator. The
/// HIPAR generatorType/available generator types including the associated interface are
/// HIPAR generatorType/listed in the documentation.
/// HIPAR reference/This control sets the reference signal for the external
/// HIPAR reference/generator.
ViStatus _VI_FUNC rsspecan_ConfigureExternalGeneratorTCPIP(ViSession instrSession,
                                                           ViInt32   generator,
                                                           ViString  generatorAddress,
                                                           ViString  generatorType,
                                                           ViInt32   reference)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_EXT_GEN_TYPE,
            "Generator", generator - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Generator");

    viCheckParm(rsspecan_SetAttributeViString(instrSession, repCap, RSSPECAN_ATTR_EXT_GEN_TYPE, generatorType),
            4, "Generator Type");

    checkErr(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_EXT_GEN_INTERFACE_TYPE, RSSPECAN_VAL_INTERFACE_TCPIP));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SYST:COMM:TCP:RDEV:GEN:ADDR '%s'", generatorAddress);
    checkErr(RsCore_Write(instrSession, cmd));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_EXT_GEN_ROSC_SOURCE, reference),
            5, "Reference");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function configures the selected external generator
/// HIFN Tmeasurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR generator/This control selects the external generator.
/// HIPAR state/This command switches on or off external source.
/// HIPAR power/Sets the output power of the generator.
/// HIPAR offset/Sets a level offset for the external generator level.
ViStatus _VI_FUNC rsspecan_ConfigureExternalGeneratorMeasurement(
                            ViSession instrSession,
                            ViInt32   generator,
                            ViBoolean state,
                            ViReal64  power,
                            ViReal64  offset)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_EXT_GEN_STATE,
            "Generator", generator - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Generator");

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, repCap, RSSPECAN_ATTR_EXT_GEN_STATE, state),
            3, "State");

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_EXT_GEN_OUTPUT_POWER, power),
            4, "Power");

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_EXT_GEN_LEVEL_OFFSET, offset),
            5, "Level Offset");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function configures the external generator frequency.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR generator/This control selects the external generator.
/// HIPAR frequency/Defines a fixed source frequency for the external generator
ViStatus _VI_FUNC rsspecan_ConfigureExternalGeneratorManualFrequency(
                            ViSession instrSession,
                            ViInt32   generator,
                            ViReal64  frequency)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_EXT_GEN_OUTPUT_FREQUENCY_COUPLING,
            "Generator", generator - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Generator");

    checkErr(rsspecan_SetAttributeViBoolean(instrSession, repCap, RSSPECAN_ATTR_EXT_GEN_OUTPUT_FREQUENCY_COUPLING, VI_FALSE));
    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_EXT_GEN_OUTPUT_FREQUENCY, frequency),
            3, "Frequency");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function is used for setting the generator multiplier and
/// HIFN the offset used to derive the generator frequency from the
/// HIFN analyzer frequency.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR generator/This control selects the external generator.
/// HIPAR frequencySweep/This control activates or deactivates the frequency sweep for generator.
/// HIPAR frequencyOffset/This control defines the frequency offset of the selected
/// HIPAR frequencyOffset/generator with reference to the receive frequency.
/// HIPAR denominator/This control defines the denominator of the factor with which
/// HIPAR denominator/the analyzer frequency is multiplied in order to obtain the
/// HIPAR denominator/transmit frequency of the  generator.
/// HIPAR numerator/This control defines the numerator of the factor with which the
/// HIPAR numerator/analyzer frequency is multiplied to obtain the transmit
/// HIPAR numerator/frequency of the generator.
ViStatus _VI_FUNC rsspecan_ConfigureExternalGeneratorAutomaticFrequency(
                            ViSession instrSession,
                            ViInt32   generator,
                            ViBoolean frequencySweep,
                            ViReal64  frequencyOffset,
                            ViInt32   denominator,
                            ViInt32   numerator)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_EXT_GEN_OUTPUT_FREQUENCY_COUPLING,
            "Generator", generator - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Generator");

    checkErr(rsspecan_SetAttributeViBoolean(instrSession, repCap, RSSPECAN_ATTR_EXT_GEN_OUTPUT_FREQUENCY_COUPLING, VI_TRUE));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, repCap, RSSPECAN_ATTR_EXT_GEN_FREQ_SWEEP_STATE, frequencySweep),
            3, "Frequency Sweep");

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_EXT_GEN_FREQ_OFFSET, frequencyOffset),
            4, "Frequency Offset");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_EXT_GEN_DENOMINATOR, denominator),
            5, "Denominator");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_EXT_GEN_NUMERATOR, numerator),
            6, "Numerator");


    checkErr(RsCore_LockSession(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN This function makes an external gain known to the analyzer,
/// HIFN which will take it into account during the display of measurement
/// HIFN results.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR gain/This control sets the antena gain
ViStatus _VI_FUNC rsspecan_ConfigureExternalGain(ViSession instrSession,
                                                 ViInt32   reserved,
                                                 ViReal64  gain)
{
    ViStatus error = VI_SUCCESS;
             reserved;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_CORR_EGA_INP, gain),
            3, "Gain");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Before this attribute can be used, the external preamplifier must be
/// HIFN  connected to the instrument. See the preamplifier's documentation for
/// HIFN  details.
/// HIFN     When activated, the instrument automatically compensates the magnitude
/// HIFN  and phase characteristics of the external preamplifier in the
/// HIFN  measurement results.
/// HIFN     Note that when an optional external preamplifier is activated, the
/// HIFN  internal preamplifier is automatically disabled, and vice versa.
/// HIFN     When deactivated, no compensation is performed even if an external
/// HIFN  preamplifier remains connected.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR paCorrectionEnabled/Before this control can be used, the external preamplifier must be
/// HIPAR paCorrectionEnabled/connected to the instrument. See the preamplifier's documentation for
/// HIPAR paCorrectionEnabled/details.
ViStatus _VI_FUNC rsspecan_ConfigureExternalGainPACorrectionEnabled(
                            ViSession instrSession,
                            ViBoolean paCorrectionEnabled)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, NULL, RSSPECAN_ATTR_EXTERNAL_GAIN_PA_CORRECTION_ENABLED, paCorrectionEnabled),
            2, "PA Correction Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function sends a command or query string to the device
/// HIFN using the GPIB interface.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR deviceAddress/This control sets the GPIB address
/// HIPAR commandString/This control sethe command string, send to the device using GPIB
/// HIPAR commandString/interface.
ViStatus _VI_FUNC rsspecan_SendCommandToExternalGenerator(ViSession instrSession,
                                                          ViInt32   deviceAddress,
                                                          ViString  commandString)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, deviceAddress, 0, 30),
            2, "Device Address");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, ":SYST:COMM:GPIB:RDEV:COMM %ld,%s", deviceAddress, commandString);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/*****************************************************************************
 *------------------- B9 - Tracking Generator   -----------------------------*
 *****************************************************************************/
/// HIFN This function sets the tracking generator state.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR state/Sets the tracking generator state.
ViStatus _VI_FUNC rsspecan_SetTrackingGeneratorState(ViSession instrSession,
                                                     ViInt32   reserved,
                                                     ViBoolean state)
{
    ViStatus error = VI_SUCCESS;
             reserved;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_TGEN_STATE, state),
            3, "State");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function configures the tracking generator.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR sourcePower/Specifies the output level of the tracking generator.
/// HIPAR powerOffset/Specifies a level offset for the tracking generator level.
/// HIPAR frequencyOffset/Sets a frequency offset of the tracking generator.
ViStatus _VI_FUNC rsspecan_ConfigureTrackingGenerator(ViSession instrSession,
                                                      ViInt32   reserved,
                                                      ViReal64  sourcePower,
                                                      ViReal64  powerOffset,
                                                      ViReal64  frequencyOffset)
{
    ViStatus error = VI_SUCCESS;
             reserved;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_TGEN_OUTPUT_LEVEL, sourcePower),
            3, "Source Power");

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_TGEN_LEVEL_OFFSET, powerOffset),
            4, "Power Offset");

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_TGEN_FREQUENCY_OFFSET, frequencyOffset),
            5, "Frequency Offset");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function switches on or off the external modulations of
/// HIFN the tracking generator.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR modulation/Sets a frequency offset of the tracking generator.
/// HIPAR modState/This command switches on or off selected modulation.
/// HIPAR FMDeviation/Sets the maximum frequency deviation at 1 V input voltage at the
/// HIPAR FMDeviation/FM input of the tracking generator.
ViStatus _VI_FUNC rsspecan_ConfigureTrackingGeneratorExternalModulation(
                            ViSession instrSession,
                            ViInt32   reserved,
                            ViInt32   modulation,
                            ViBoolean modState,
                            ViReal64  FMDeviation)
{
    ViStatus error = VI_SUCCESS;
             reserved;

    checkErr(RsCore_LockSession(instrSession));

    switch (modulation)
    {
    case RSSPECAN_VAL_TGEN_EXT_AM:
        viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_TGEN_AM_STATE, modState),
                4, "Mod State");
        break;
    case RSSPECAN_VAL_TGEN_EXT_FM:
        viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_TGEN_FM_STATE, modState),
                4, "Mod State");

        viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_TGEN_FM_DEVIATION, FMDeviation),
                5, "FM Deviation");
        break;
    case RSSPECAN_VAL_TGEN_EXT_IQ:
        viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_TGEN_DM_STATE, modState),
                4, "Mod State");
        break;
    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, modulation), 3, "Modulation");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function configures the power sweep of the tracking
/// HIFN generator.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR powerSweep/Activates or deactivates the power sweep.
/// HIPAR powerStart/Sets the start power of the power sweep.
/// HIPAR powerStop/Sets the stop power of the power sweep.
ViStatus _VI_FUNC rsspecan_ConfigureTrackingGeneratorPowerSweep(
                            ViSession instrSession,
                            ViInt32   reserved,
                            ViInt32   powerSweep,
                            ViReal64  powerStart,
                            ViReal64  powerStop)
{
    ViStatus error = VI_SUCCESS;
             reserved;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_TGEN_POW_MODE, powerSweep),
            3, "Power Sweep");

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_TGEN_POW_START, powerStart),
            4, "Power Sweep");

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_TGEN_POW_STOP, powerStop),
            5, "Power Sweep");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/*****************************************************************************
 *- User defined callbacks --------------------------------------------------*
 *****************************************************************************/
