/*****************************************************************************
 *  Rohde&Schwarz Spectrum Analyzer instrument driver
 *  B17 - Digital Baseband Input
 *
 *  Original Release: October 2007
 *  By: Jiri Kominek (instrument driver core)
 *      Helena Kominkova (instrument specific implementation)
 *      Jiri Kominek (DrBase repository & tools)
 *
 *  Should you have any technical questions please contact the hotline of
 *  Rohde & Schwarz Vertriebs-GmbH Rohde & Schwarz Support Center
 *
 *  e-mail: CustomerSupport@rohde-schwarz.com
 *

 *****************************************************************************/

#include "rsspecan.h"

/// HIFN This function configures digital baseband input.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR basebandInputs/Switches the baseband inputs on or off.
ViStatus _VI_FUNC rsspecan_ConfigureDigitalBasebandInput(ViSession instrSession,
                                                         ViInt32   basebandInputs)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_BB_INPUT_SELECTION, basebandInputs),
            2, "Baseband Inputs");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  If enabled, the reference level for digital input is adjusted to the full scale level automatically if the fullscale level changes.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR basebandInputCoupling/If enabled, the reference level for digital input is adjusted to the full scale level automatically if the fullscale level changes.

ViStatus _VI_FUNC rsspecan_ConfigureDigitalBasebandInputCoupling(
                            ViSession instrSession,
                            ViBoolean basebandInputCoupling)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_DIGITAL_INPUT_COUPLING, basebandInputCoupling),
            2, "Baseband Input Coupling");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function activates/deactivates auto set of digital baseband input parameters.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR autoRange/This control activates/deactivates the automatically adjust of digital in full scale.
/// HIPAR autoSampleRate/This control activates/deactivates the automatically adjust of digital in sample rate.

ViStatus _VI_FUNC rsspecan_ConfigureDigitalBasebandInputParametersAuto(
                            ViSession instrSession,
                            ViBoolean autoRange,
                            ViBoolean autoSampleRate)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_DIGITAL_INPUT_RANGE_AUTO, autoRange),
            2, "Digital Input Range Auto");

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_DIGITAL_INPUT_SAMPLE_RATE_AUTO, autoSampleRate),
            3, "Auto Sample Rate");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures digital baseband input parameters.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR range/This control sets the voltage corresponding to the maximum input
/// HIPAR range/value of the digital baseband input.
/// HIPAR sampleRate/This control sets the input date sample rate read by the digital
/// HIPAR sampleRate/baseband input.
ViStatus _VI_FUNC rsspecan_ConfigureDigitalBasebandInputParameters(
                            ViSession instrSession,
                            ViReal64  range,
                            ViReal64  sampleRate)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_DIGITAL_INPUT_RANGE, range),
            2, "Range");

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_DIGITAL_INPUT_SRATE, sampleRate),
            3, "Sample Rate");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function sets the decimation filter state.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR filter/Sets the decimation filter state. If enabled, no digital
/// HIPAR filter/decimation filter is used during data acquisition.
ViStatus _VI_FUNC rsspecan_ConfigureDigitalBasebandDecimationFilter(
                            ViSession instrSession,
                            ViBoolean filter)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_DIGITAL_DECIMATION_FILTER, filter),
            2, "Filter");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function configures the IQ data output of the Digital
/// HIFN Baseband interface.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR output/Configures the IQ data output of the Digital Baseband interface.
ViStatus _VI_FUNC rsspecan_ConfigureDigitalBasebandOutput(ViSession instrSession,
                                                          ViBoolean output)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_DIGITAL_BASEBAND_OUTPUT_STATE, output),
            2, "Output");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN This function returns the current configuration and status of the
/// HIFN Digital Baseband input or output of option R&S FSQ-B17.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR digitalBasebandSelect/This control selects the digital baseband.
/// HIPAR arraySize/Defines size of array passed to 'Configuration'.
/// HIPAR configuration/Returns the current configuration and status of the
/// HIPAR configuration/Digital Baseband input of option R&S FSQ-B17.
ViStatus _VI_FUNC rsspecan_QueryDigitalBasebandInputOutputConfiguration(
                            ViSession instrSession,
                            ViInt32   digitalBasebandSelect,
                            ViInt32   arraySize,
                            ViChar    configuration[])
{
    ViStatus error = VI_SUCCESS;
    ViAttr   attr = 0;

    checkErr(RsCore_LockSession(instrSession));

    switch (digitalBasebandSelect)
    {
    case RSSPECAN_VAL_DIQ_IN:
        attr = RSSPECAN_ATTR_QUERY_DIGITAL_INPUT_CONF;
        break;
    case RSSPECAN_VAL_DIQ_OUT:
        attr = RSSPECAN_ATTR_QUERY_DIGITAL_OUTPUT_CONF;
        break;
    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, digitalBasebandSelect), 2, "Digital Baseband Select");
    }

    checkErr(rsspecan_GetAttributeViString(instrSession, "", attr,
                                               arraySize, configuration));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}
