/*****************************************************************************
 *  Rohde&Schwarz Spectrum Analyzer instrument driver
 *  B71 - FFT Analyzer
 *
 *  Original Release: September 2005
 *  By: Martin Koutny (instrument driver core)
 *      Helena Sucha (instrument specific implementation)
 *      Jiri Kominek (DrBase repository & tools)
 *
 *  Should you have any technical questions please contact the hotline of
 *  Rohde & Schwarz Vertriebs-GmbH Rohde & Schwarz Support Center
 *
 *  e-mail: CustomerSupport@rohde-schwarz.com
 *

 *****************************************************************************/

#include "rsspecan.h"

/// HIFN This function configures analog baseband input trigger.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init
/// HIPAR instrSession/or rsspecan_InitWithOptions function.
/// HIPAR trigger/Specifies the source of the trigger signal.
ViStatus _VI_FUNC rsspecan_ConfigureAnalogBasebandTrigger(ViSession instrSession,
                                                          ViInt32   trigger)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_TRIGGER_SOURCE, trigger),
            2, "Trigger");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function sets the baseband power trigger level
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init
/// HIPAR instrSession/or rsspecan_InitWithOptions function.
/// HIPAR triggerLevel/Sets the level of the baseband power trigger source.
ViStatus _VI_FUNC rsspecan_ConfigureBasebandPowerTriggerLevel(
                            ViSession instrSession,
                            ViReal64  triggerLevel)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_DIGITAL_BASEBAND_INPUT_TRIGGER_LEVEL, triggerLevel),
            2, "Trigger Level");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function configures analog baseband input.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init
/// HIPAR instrSession/or rsspecan_InitWithOptions function.
/// HIPAR basebandInputs/Switches the baseband inputs on or off.
ViStatus _VI_FUNC rsspecan_ConfigureAnalogBasebandInput(ViSession instrSession,
                                                        ViInt32   basebandInputs)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_BB_INPUT_SELECTION, basebandInputs),
            2, "Baseband Inputs");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function configures the signal path.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init
/// HIPAR instrSession/or rsspecan_InitWithOptions function.
/// HIPAR iQMode/Defines the input signal path used.
/// HIPAR inputConfiguration/This attribute toggles the baseband inputs between balanced and
/// HIPAR inputConfiguration/unbalanced.
/// HIPAR swapIQ/Swaps the real (I branch) and the imaginary (Q branch) parts of
/// HIPAR swapIQ/the signal.
ViStatus _VI_FUNC rsspecan_ConfigureAnalogBasebandInputParameters(
                            ViSession instrSession,
                            ViInt32   iQMode,
                            ViBoolean inputConfiguration,
                            ViBoolean swapIQ)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_BB_INPUT_SIGNAL_PATH, iQMode),
            2, "I/Q Mode");

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_BB_INPUT_BALANCED, inputConfiguration),
            3, "Input Configuration");

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_BB_SWAP_IQ, swapIQ),
            4, "Swap I/Q");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function configures the fullscale level. The fullscale
/// HIFN  level defines the maximum power you can input at the Baseband
/// HIFN  Input connector without clipping the signal.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init
/// HIPAR instrSession/or rsspecan_InitWithOptions function.
/// HIPAR mode/This control defines whether the fullscale level (i.e. the
/// HIPAR mode/maximum input power on the Baseband Input connector) is defined
/// HIPAR mode/automatically according to the reference level, or manually.
/// HIPAR level/This control defines the peak voltage at the Baseband Input
/// HIPAR level/connector if the fullscale level is set to manual mode (Mode
/// HIPAR level/control is set to Off).
ViStatus _VI_FUNC rsspecan_ConfigureAnalogBasebandFullscaleLevel(
                            ViSession instrSession,
                            ViBoolean mode,
                            ViReal64  level)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_BB_FULLSCALE_MODE, mode),
            2, "Mode");

    if (mode == VI_FALSE)
    {
        viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_BB_FULLSCALE_LEVEL, level),
                3, "Level");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function configures the Analog Baseband interface.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR signalType/Defines the type of calibration signal to be used for Analog Baseband.
ViStatus _VI_FUNC rsspecan_ConfigureAnalogBasebandCalibrationSignalType(
                            ViSession instrSession,
                            ViInt32   signalType)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_SERVICE_INPUT_SOURCE, RSSPECAN_VAL_INPUT_AIQ));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_BB_SIGNAL_CALIBRATION_TYPE, signalType),
            2, "Signal Type");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN This function configures the signal path.
/// HIRET Document return value here.
/// HIRET You may use multiple lines for documentation.
/// HIPAR instrSession/Document parameter here.
/// HIPAR path/Defines the input signal path used.
ViStatus _VI_FUNC rsspecan_ConfigureAnalogBasebandSignalPath(ViSession instrSession,
                                                             ViInt32   path)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_BB_INPUT_SIGNAL_PATH, path),
            2, "Path");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Swaps the real (I branch) and the imaginary (Q branch) parts of the signal.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init
/// HIPAR instrSession/or rsspecan_InitWithOptions function.
/// HIPAR swapIQ/Swaps the real (I branch) and the imaginary (Q branch) parts of the signal.
ViStatus _VI_FUNC rsspecan_ConfigureAnalogBasebandSwapIQ(ViSession instrSession,
                                                         ViBoolean swapIQ)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_BB_SWAP_IQ, swapIQ),
            2, "Swap IQ");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Activates a mode with enhanced timing accuracy between analog
/// HIFN  baseband, RF and external trigger signals.
/// HIFN     Note:
/// HIFN
/// HIFN     (1) This function is valid only with B71 option.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR highAccuracyTiming/Activates a mode with enhanced timing accuracy between analog
/// HIPAR highAccuracyTiming/baseband, RF and external trigger signals.
ViStatus _VI_FUNC rsspecan_ConfigureAnalogBasebandHighAccuracyTiming(
                            ViSession instrSession,
                            ViBoolean highAccuracyTiming)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_BB_HIGH_ACCURACY_TIMING, highAccuracyTiming),
            2, "High Accuracy Timing");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the common mode offset.
/// HIFN
/// HIFN     Note(s):
/// HIFN
/// HIFN     (1) The setting is only available if a differential probe is connected
/// HIFN  to the R&S FSW. If the probe is disconnected, the common mode offset of
/// HIFN  the probe is reset to 0.0 V.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR probeConnector/Selects the connector.
/// HIPAR cmOffset/Sets the common mode offset. The setting is only available if a
/// HIPAR cmOffset/differential probe is connected to the R&S FSW. If the probe is
/// HIPAR cmOffset/disconnected, the common mode offset of the probe is reset to 0.0 V.
ViStatus _VI_FUNC rsspecan_ConfigureAnalogBasebandProbeCommonModeOffset(
                            ViSession instrSession,
                            ViInt32   probeConnector,
                            ViReal64  cmOffset)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, probeConnector, 1, 3),
            2, "Probe Connector");

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_BB_PROBE_COMMON_MODE_OFFSET,
            "Probe", probeConnector - 1, RS_REPCAP_BUF_SIZE, repCap));

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_BB_PROBE_COMMON_MODE_OFFSET, cmOffset),
            3, "CM Offset");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the DC offset of the I/Q input from the
/// HIFN  Analog Baseband interface.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR iqMode/Selects the input signal path used.
/// HIPAR dcOffset/This control defines a DC offset of the I/Q input from the Analog
/// HIPAR dcOffset/Baseband interface.
ViStatus _VI_FUNC rsspecan_ConfigureAnalogBasebandDCOffset(ViSession instrSession,
                                                           ViInt32   iqMode,
                                                           ViReal64  dcOffset)
{
    ViStatus error = VI_SUCCESS;
    ViAttr   attr[2] = {RSSPECAN_ATTR_BB_I_DC_OFFSET, RSSPECAN_ATTR_BB_Q_DC_OFFSET};

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, iqMode, RSSPECAN_VAL_BB_SIG_PATH_I, RSSPECAN_VAL_BB_SIG_PATH_Q),
            2, "I/Q Mode");

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", attr[iqMode], dcOffset),
            3, "DC Offset");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function configures analog baseband input impedance.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR impedance/Selects the nominal input impedance of the analog baseband input.
/// HIPAR padType/Defines the type of matching pad used for impedance conversion for analog baseband input.
ViStatus _VI_FUNC rsspecan_ConfigureAnalogBasebandInputImpedance(
                            ViSession instrSession,
                            ViReal64  impedance,
                            ViInt32   padType)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, NULL, RSSPECAN_ATTR_BASEBAND_INPUT_IMPEDANCE, impedance),
            2, "Impedance");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, NULL, RSSPECAN_ATTR_BASEBAND_INPUT_IMPEDANCE_PAD_TYPE, padType),
            3, "PAD Type");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines an additional attenuation to be used during an auto level
/// HIFN  measurement in order to optimize the signal level at the A/D converter.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR iqInterface/Defines an additional attenuation to be used during an auto level
/// HIPAR iqInterface/measurement in order to optimize the signal level at the A/D converter.
ViStatus _VI_FUNC rsspecan_ConfigureAnalogBasebandInputAttenuationIQInterface(
                            ViSession instrSession,
                            ViInt32   iqInterface)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_BASEBAND_INPUT_ATTENUATION_IQ_INTERFACE, iqInterface),
            2, "IQ Interface");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  If the option R&S FSVA-B11 is installed, the YIG preselector can be
/// HIFN  bypassed.
/// HIFN     This attribute is only available for R&S FSVA instruments.
/// HIFN     If the YIG preselector at the input of the R&S FSVA is removed from
/// HIFN  the signal path, you can use the maximum bandwidth for signal analysis.
/// HIFN  However, image-frequency rejection is no longer ensured.
/// HIFN     Note that the YIG preselector is active only on frequencies greater
/// HIFN  than 7 GHz. Therefore, switching the YIG preselector on and off has no
/// HIFN  effect if the frequency is below that value.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR enabled/If the option R&S FSVA-B11 is installed, the YIG preselector can be
/// HIPAR enabled/bypassed.
ViStatus _VI_FUNC rsspecan_ConfigureAnalogBasebandYIGFilterEnabled(
                            ViSession instrSession,
                            ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_BASEBAND_YIG_FILTER_ENABLED, enabled),
            2, "Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}
