/*****************************************************************************
 *  Rohde&Schwarz Spectrum Analyzer instrument driver
 *  Callbacks
 *
 *  Original Release: September 2008
 *  By: Jiri Kominek
 *
 *  Should you have any technical questions please contact the hotline of
 *  Rohde & Schwarz Vertriebs-GmbH Rohde & Schwarz Support Center
 *
 *  e-mail: CustomerSupport@rohde-schwarz.com
 *

 *****************************************************************************/

#include "rsspecan.h"
#include <float.h>
#include <math.h>

#pragma warning (disable:4100) // unreferenced formal parameter
#pragma warning (disable:4055)

/****/

ViStatus rsspecan_doubleQuotedString_WriteCallback(ViSession instrSession, ViConstString repCapName,
                                                   RsCoreAttributePtr attr, void* value);

/***/

/*****************************************************************************
 *- Read Callbacks ----------------------------------------------------------*
 *****************************************************************************/

/****************************************************************************************************
 *  rsspecan_FormatUniversalFrequencyBandRepCap_ReadCallback
Replaces <FrequencyBand> with value of ViAttr selectedFrame
******************************************************************************************************/
ViStatus rsspecan_FormatUniversalFrequencyBandRepCap_ReadCallback(
                            ViSession          instrSession,
                            ViConstString      repCapName,
                            RsCoreAttributePtr attr,
                            ViAttr             selectedFrequencyBand)
{
    ViStatus error = VI_SUCCESS;
    ViChar*  cmd = NULL;
    ViChar   responseFixed[RS_MAX_MESSAGE_BUF_SIZE];
    ViChar   aux[RS_MAX_MESSAGE_BUF_SIZE];
    ViChar*  responseUnknownLen = NULL; // For string attribute, use dynamic buffer
    ViInt32  responseSize = 0;
    ViInt32  rangeTableValue = 0;
    ViChar*  p2response = NULL;
    ViInt32  frequencyBand;

    // If no command available, exit the function
    if (isNullOrEmpty(attr->command))
        goto Error;

    // Command string building from repeated capabilities
    checkErr(RsCore_BuildAttrCommandString(instrSession, attr, repCapName, &cmd));

    if (selectedFrequencyBand > 0)
    {
        // Replaces <Frame> with number
        checkErr(RsCore_GetAttributeViInt32(instrSession, NULL, selectedFrequencyBand, 0, &frequencyBand));
        snprintf(aux, RS_MAX_MESSAGE_BUF_SIZE, "%ld", frequencyBand);
        RsCore_ReplaceString(cmd, RS_MAX_MESSAGE_BUF_SIZE, "<FrequencyBand>", aux, VI_TRUE);
    }

    if (attr->dataType == RS_VAL_STRING || attr->dataType == RS_VAL_RAWSTRING)
    {
        // Query the instrument, get the unlimited size response
        checkErr(
            RsCore_QueryAttributeCommandUnknownLength(instrSession, attr, cmd, &responseUnknownLen, &responseSize));
        // For string-type attributes remove trailing Whitespaces and pair quotations
        RsCore_TrimString(responseUnknownLen, RS_VAL_TRIM_WHITESPACES_AND_ALL_QUOTES);
        p2response = responseUnknownLen;
    }
    else
    {
        // Query the instrument, get the limited size response - RS_MAX_MESSAGE_BUF_SIZE
        checkErr(RsCore_QueryAttributeCommand(instrSession, attr, cmd, responseFixed));
        p2response = responseFixed;
    }

    // In simulation, end here. Do not touch the cache value.
    // It is read by the GetAttribute and returned to the caller.
    if (RsCore_Simulating(instrSession))
        goto Error;

    if (attr->rangeTableDiscrete)
    {
        // Value has to converted to the range table value
        checkErr(
            RsCore_FindRangeTableDiscreteNumericValue(instrSession, attr->rangeTableDiscrete, p2response, &rangeTableValue));
        *(ViInt32*)attr->p2value = rangeTableValue;
    }
    else
    {
        // No range table, convert the response directly to the appropriate type
        switch (attr->dataType)
        {
        case RS_VAL_INT32:
            *((ViInt32*)attr->p2value) = (ViInt32)RsCore_Convert_String_To_ViReal64(p2response);  // NOLINT(clang-diagnostic-bad-function-cast)
            break;
        case RS_VAL_INT64:
            *((ViInt64*)attr->p2value) = (ViInt64)RsCore_Convert_String_To_ViReal64(p2response); // NOLINT(clang-diagnostic-bad-function-cast)
            break;
        case RS_VAL_REAL64:
            *((ViReal64*)attr->p2value) = RsCore_Convert_String_To_ViReal64(p2response);
            break;
        case RS_VAL_STRING:
        case RS_VAL_RAWSTRING:
            checkErr(RsCore_SetAttributeP2Value(instrSession, attr, p2response));
            break;
        case RS_VAL_BOOLEAN:
            *((ViBoolean*)attr->p2value) = RsCore_Convert_String_To_Boolean(p2response);
            break;
        default:
            checkErr(RsCore_GenerateInvalidDataTypeError(instrSession, attr->dataType, NULL));
        }
    }

Error:
    if (cmd)
        free(cmd);

    if (responseUnknownLen)
        free(responseUnknownLen);

    return error;
}

/*****************************************************************************
 * Function: rsspecan_empty_ReadCallback
 * Purpose:  This function overrides standard calback funtion. Returns value
 *           from cache
 *****************************************************************************/
ViStatus rsspecan_empty_ReadCallback(ViSession          instrSession,
                                     ViConstString      repCapName,
                                     RsCoreAttributePtr attr)
{
    return VI_SUCCESS;
}

/*****************************************************************************
 * Function: rsspecan_ReadOnlyCollection_ReadCallback
 * Purpose:  This function overrides standard calback funtion. Returns value
 *           from cache
 *****************************************************************************/
ViStatus rsspecan_ReadOnlyCollection_ReadCallback(ViSession          instrSession,
                                                  ViConstString      repCapName,
                                                  RsCoreAttributePtr attr)
{
    return RsCore_ReadCallback(instrSession, repCapName, attr);
}

/****************************************************************************************************
 *  rsspecan_FormatFrequencyBandRepCapReal64_ReadCallback
Replaces <FrequencyBand> with value of RSSPECAN_ATTR_EXTERNAL_FRONTEND_SELECTED_FREQUENCY_BAND
******************************************************************************************************/
ViStatus rsspecan_FormatFrequencyBandRepCapReal64_ReadCallback(ViSession io, ViConstString repCapName, RsCoreAttributePtr attr)
{
    return rsspecan_FormatUniversalFrequencyBandRepCap_ReadCallback(io, repCapName, attr, RSSPECAN_ATTR_EXTERNAL_FRONTEND_SELECTED_FREQUENCY_BAND);
}

/*****************************************************************************
 * Function: rsspecan_stringToBin_ReadCallback
 * Purpose:  This function overrides standard calback funtion.
 *****************************************************************************/
ViStatus rsspecan_stringToBin_ReadCallback(ViSession          instrSession,
                                           ViConstString      repCapName,
                                           RsCoreAttributePtr attr)
{
    return RsCore_SpecialCallback_BinOrHexPatternRead(instrSession, repCapName, attr);
}

/*****************************************************************************
 * Function: rsspecan_stringToHex_ReadCallback
 * Purpose:  This function overrides standard calback funtion.
 *****************************************************************************/
ViStatus rsspecan_stringToHex_ReadCallback(ViSession          instrSession,
                                           ViConstString      repCapName,
                                           RsCoreAttributePtr attr)
{
    return RsCore_SpecialCallback_BinOrHexPatternRead(instrSession, repCapName, attr);
}

/*****************************************************************************
* Function: rsspecan_quotedInt_ReadCallback
* Purpose:  This function overrides standard calback funtion. Reads int
*           returned with apostrophes
*****************************************************************************/
ViStatus rsspecan_quotedInt_ReadCallback(ViSession          instrSession,
                                         ViConstString      repCapName,
                                         RsCoreAttributePtr attr)
{
    return RsCore_SpecialCallback_AnyQuotedParamRead(instrSession, repCapName, attr);
}

/*****************************************************************************
 * Function: rsspecan_DeltaMarkerReferenceMarker_ReadCallback
 * Purpose:  If the response is 'FIX', the function returns 0
 *****************************************************************************/
ViStatus rsspecan_DeltaMarkerReferenceMarker_ReadCallback(ViSession          instrSession,
                                                          ViConstString      repCapName,
                                                          RsCoreAttributePtr attr)
{
    ViStatus error = VI_SUCCESS;
    ViChar*  cmd = NULL;
    ViChar   response[RS_MAX_MESSAGE_BUF_SIZE];
    ViInt32  value;

    if (attr->command == NULL)
        return VI_SUCCESS;

    if (attr->dataType != RS_VAL_INT32)
    {
        checkErr(
            RsCore_GenerateInvalidDataTypeError(instrSession, attr->dataType,
                "rsspecan_DeltaMarkerReferenceMarker_ReadCallback only accepts type RS_VAL_INT32"));
    }

    checkErr(RsCore_BuildAttrCommandString(instrSession, attr, repCapName, &cmd));
    checkErr(RsCore_QueryAttributeCommand(instrSession, attr, cmd, response));
    if (strstr(response, "FIX") != NULL)
        value = RsCore_Convert_String_To_ViInt32(response);
    else
        (void)sscanf(response, "%ld", &value);

    *((ViInt32*)attr->p2value) = value;

Error:
    if (cmd)
        free(cmd);

    return error;
}

/*****************************************************************************
 * Function: rsspecan_GapLimit_ReadCallback
 * Purpose:  From returned tuple parses only the first float
 *****************************************************************************/
ViStatus rsspecan_GapLimit_ReadCallback(ViSession          instrSession,
                                        ViConstString      repCapName,
                                        RsCoreAttributePtr attr)
{
    ViStatus error = VI_SUCCESS;
    ViChar*  cmd = NULL;
    ViChar   response[RS_MAX_MESSAGE_BUF_SIZE];
    ViReal64 parsed1, parsed2;

    // If no command available, exit the function
    if (attr->command == NULL)
        return VI_SUCCESS;

    if (attr->dataType != RS_VAL_REAL64)
    {
        checkErr(
            RsCore_GenerateInvalidDataTypeError(instrSession, attr->dataType,
                "rsspecan_GapLimit_ReadCallback only accepts type RS_VAL_REAL64"));
    }

    checkErr(RsCore_BuildAttrCommandString(instrSession, attr, repCapName, &cmd));
    checkErr(RsCore_QueryAttributeCommand(instrSession, attr, cmd, response));
    (void)sscanf(response, "%le,%le", &parsed1, &parsed2);
    *((ViReal64*)attr->p2value) = parsed1;

Error:
    if (cmd)
        free(cmd);

    return error;
}

/*****************************************************************************
 * Function: rsspecan_LTEUplinkSubframe_ReadCallback
 * Purpose:  Integer value -1 if the response was SUBF
 *****************************************************************************/
ViStatus rsspecan_LTEUplinkSubframe_ReadCallback(ViSession          instrSession,
                                                 ViConstString      repCapName,
                                                 RsCoreAttributePtr attr)
{
    ViStatus error = VI_SUCCESS;
    ViChar*  cmd = NULL;
    ViChar   response[RS_MAX_MESSAGE_BUF_SIZE] = "";

    // If no command available, exit the function
    if (isNullOrEmpty(attr->command))
        goto Error;

    // Command string building from repeated capabilities
    checkErr(RsCore_BuildAttrCommandString(instrSession, attr, repCapName, &cmd));

    checkErr(RsCore_QueryAttributeCommand(instrSession, attr, cmd, response));

    // In simulation, end here. Do not touch the cache value.
    // It is read by the GetAttribute and returned to the caller.
    if (RsCore_Simulating(instrSession))
        goto Error;

    // No range table, covert the response directly to the appropriate type
    switch (attr->dataType)
    {
    case RS_VAL_INT32:
        if (strstr(response, "SUBF") != NULL)
            *((ViInt32*)attr->p2value) = -1;
        else
            (void)sscanf(response, "%ld", ((ViInt32*)attr->p2value));
        break;
    default:
        checkErr(RsCore_GenerateInvalidDataTypeError(instrSession, attr->dataType,
            "rsspecan_LTEUplinkSubframe_ReadCallback: Allowed data types: RS_VAL_INT32"));
    }

Error:
    if (cmd)
        free(cmd);

    return error;
}

/*****************************************************************************
 * Function: rsspecan_parseNAN_ReadCallback
 * Purpose:  Obsolete, use the default callback
 *****************************************************************************/
ViStatus rsspecan_parseNAN_ReadCallback(ViSession          instrSession,
                                        ViConstString      repCapName,
                                        RsCoreAttributePtr attr)
{
    return RsCore_ReadCallback(instrSession, repCapName, attr);
}


/*****************************************************************************
 * Function: rsspecan_logging_ReadCallback
 * Purpose:  Returns value from cache
 *****************************************************************************/
ViStatus rsspecan_logging_ReadCallback(ViSession          instrSession,
                                       ViConstString      repCapName,
                                       RsCoreAttributePtr attr)
{
    return VI_SUCCESS;
}

/*****************************************************************************
 *- Write Callbacks ----------------------------------------------------------*
 *****************************************************************************/

 /****************************************************************************************************
  *  rsspecan_FormatUniversalFrequencyBandRepCap_WriteCallback
 Replaces <FrequencyBand> with value of ViAttr selectedFrame
     ******************************************************************************************************/
ViStatus rsspecan_FormatUniversalFrequencyBandRepCap_WriteCallback(
                            ViSession          instrSession,
                            ViConstString      repCapName,
                            RsCoreAttributePtr attr,
                            void*              value,
                            ViAttr             selectedFreqBand)
{
    ViStatus error = VI_SUCCESS;
    ViChar*  cmd = NULL;
    ViChar   aux[RS_MAX_SHORT_MESSAGE_BUF_SIZE];
    ViInt32  frequencyBand;

    // If no command available, exit the function
    if (isNullOrEmpty(attr->command))
        goto Error;

    // Command string building from repeated capabilities
    checkErr(RsCore_BuildAttrCommandString(instrSession, attr, repCapName, &cmd)) // TODO: Check semicolon
        if (selectedFreqBand > 0)
        {
            // Replaces <FrequencyBand> with number
            checkErr(RsCore_GetAttributeViInt32(instrSession, NULL, selectedFreqBand, 0, &frequencyBand));
            snprintf(aux, RS_MAX_SHORT_MESSAGE_BUF_SIZE, "%ld", frequencyBand);
            RsCore_ReplaceString(cmd, RS_MAX_MESSAGE_BUF_SIZE, "<FrequencyBand>", aux, VI_TRUE);
        }

    if (attr->rangeTableDiscrete)
    {
        ViChar* p2cmdString = NULL;
        checkErr(RsCore_FindRangeTableDiscreteCmdString(instrSession, attr->rangeTableDiscrete, *(ViInt32*)value, &p2cmdString));
        RsCore_StrCatDynamicString(instrSession, &cmd, " ", p2cmdString, NULL);
    }
    else
    {
        // No range table, cmdString is composed based on the data type
        switch (attr->dataType)
        {
        case RS_VAL_INT32:
            RsCore_StrCatDynamicViInt32(instrSession, &cmd, " ", *(ViInt32*)value, NULL);
            break;
        case RS_VAL_INT64:
            RsCore_StrCatDynamicViInt64(instrSession, &cmd, " ", *(ViInt64*)value, NULL);
            break;
        case RS_VAL_REAL64:
            RsCore_StrCatDynamicViReal64(instrSession, &cmd, " ", *(ViReal64*)value, NULL);
            break;
        case RS_VAL_STRING:
        case RS_VAL_RAWSTRING:
            RsCore_StrCatDynamicString(instrSession, &cmd, " \'", (ViString)value, "\'");
            break;
        case RS_VAL_BOOLEAN:
            RsCore_StrCatDynamicViBoolean(instrSession, &cmd, " ", *(ViBoolean*)value, NULL);
            break;
        case RS_VAL_EVENT:
            break;
        case RS_VAL_UNKNOWN_TYPE:
        default:
            checkErr(RsCore_GenerateInvalidDataTypeError(instrSession, attr->dataType, NULL));
        }
    }

    checkErr(RsCore_WriteAttributeCommand(instrSession, attr, cmd));

Error:
    if (cmd)
        free(cmd);

    return error;
}


/*****************************************************************************
 * Function: rsspecan_logging_WriteCallback
 * Purpose:  Does not write anything to instrument, just updates the cache
 *****************************************************************************/
ViStatus rsspecan_logging_WriteCallback(ViSession          instrSession,
                                        ViConstString      repCapName,
                                        RsCoreAttributePtr attr,
                                        void*              value)
{
    return VI_SUCCESS;
}

/*****************************************************************************
 * Function: rsspecan_empty_WriteCallback
 * Purpose:  Does not write anything to instrument, just updates the cache
 *****************************************************************************/
ViStatus rsspecan_empty_WriteCallback(ViSession          instrSession,
                                      ViConstString      repCapName,
                                      RsCoreAttributePtr attr,
                                      void*              value)
{
    return VI_SUCCESS;
}

/*************************************************************************************************************
 * Function: rsspecan_FormatFrequencyBandRepCapReal64_WriteCallback
 * Purpose:  Replaces <FrequencyBand> with value of RSSPECAN_ATTR_EXTERNAL_FRONTEND_SELECTED_FREQUENCY_BAND
 ************************************************************************************************************/
ViStatus rsspecan_FormatFrequencyBandRepCapReal64_WriteCallback(
                            ViSession          instrSession,
                            ViConstString      repCapName,
                            RsCoreAttributePtr attr,
                            void*              value)
{

    return rsspecan_FormatUniversalFrequencyBandRepCap_WriteCallback(instrSession, repCapName, attr, value, RSSPECAN_ATTR_EXTERNAL_FRONTEND_SELECTED_FREQUENCY_BAND);
}

/*****************************************************************************
 * Function: rsspecan_stringToBin_WriteCallback
 * Purpose:  This function overrides standard calback funtion.
 *****************************************************************************/
ViStatus rsspecan_stringToBin_WriteCallback(ViSession          instrSession,
                                            ViConstString      repCapName,
                                            RsCoreAttributePtr attr,
                                            void*              value)
{
    return RsCore_SpecialCallback_BinPatternWrite(instrSession, repCapName, attr, value);
}

/*****************************************************************************
 * Function: rsspecan_stringToHex_WriteCallback
 * Purpose:  This function overrides standard calback funtion.
 *****************************************************************************/
ViStatus rsspecan_stringToHex_WriteCallback(ViSession          instrSession,
                                            ViConstString      repCapName,
                                            RsCoreAttributePtr attr,
                                            void*              value)
{
    return RsCore_SpecialCallback_HexPatternWriteNoLength(instrSession, repCapName, attr, value);
}

/*****************************************************************************
 * Function: rsspecan_hex_WriteCallback
 * Purpose:  This function overrides standard calback funtion.
 *****************************************************************************/
ViStatus rsspecan_hex_WriteCallback(ViSession          instrSession,
                                    ViConstString      repCapName,
                                    RsCoreAttributePtr attr,
                                    void*              value)
{
    return RsCore_SpecialCallback_FormatNumberHexWrite(instrSession, repCapName, attr, value);
}

/*****************************************************************************
 * Function: rsspecan_InstrumentCreate_WriteCallback
 * Purpose:  This function overrides standard calback funtion.
 *****************************************************************************/
ViStatus rsspecan_InstrumentCreate_WriteCallback(ViSession          instrSession,
                                                 ViConstString      repCapName,
                                                 RsCoreAttributePtr attr,
                                                 void*              value)
{
    return RsCore_SpecialCallback_SurroundParamWrite(instrSession, repCapName, attr, " 'Spectrum ", value, "'");
}

/*****************************************************************************
* Function: rsspecan_quotedString_WriteCallback
* Purpose: Writes string with single quotes
*****************************************************************************/
ViStatus rsspecan_quotedString_WriteCallback(ViSession          instrSession,
                                             ViConstString      repCapName,
                                             RsCoreAttributePtr attr,
                                             void*              value)
{
    return RsCore_WriteCallback(instrSession, repCapName, attr, value);
}

/*******************************************************************************
 * Function: rsspecan_outputSelectionCallback_WriteCallback
 * Purpose:  Writes string with apostrophes
             When string is FOC, writes it without apostrophes
 *******************************************************************************/
ViStatus rsspecan_outputSelectionCallback_WriteCallback(ViSession          instrSession,
                                                        ViConstString      repCapName,
                                                        RsCoreAttributePtr attr,
                                                        void*              value)
{
    ViStatus error = VI_SUCCESS;

    if (attr->dataType != RS_VAL_STRING)
        checkErr(RsCore_GenerateInvalidDataTypeError(instrSession, attr->dataType,
        "rsspecan_outputSelectionCallback_WriteCallback: Data type must be RS_VAL_STRING"));

    if (strcmp((char*)value, "FOC") == 0)
    {
        checkErr(RsCore_SpecialCallback_SurroundParamWrite(instrSession, repCapName, attr, " ", value, ""));
    }
    else
    {
        checkErr(RsCore_SpecialCallback_SurroundParamWrite(instrSession, repCapName, attr, " '", value, "'"));
    }

Error:
    return error;
}

/*****************************************************************************
* Function: rsspecan_doubleQuotedString_WriteCallback
* Purpose:  Writes string with double quotes
*****************************************************************************/
ViStatus rsspecan_doubleQuotedString_WriteCallback(ViSession          instrSession,
                                                   ViConstString      repCapName,
                                                   RsCoreAttributePtr attr,
                                                   void*              value)
{
    return RsCore_SpecialCallback_SurroundParamWrite(instrSession, repCapName, attr, " \"", value, "\"");
}

/*****************************************************************************
* Function: rsspecan_doubleQuotedString_WriteCallback
* Purpose:  Writes integer with double quotes
*****************************************************************************/
ViStatus rsspecan_doubleQuotedInt_WriteCallback(ViSession          instrSession,
                                                ViConstString      repCapName,
                                                RsCoreAttributePtr attr,
                                                void*              value)
{
    return RsCore_SpecialCallback_SurroundParamWrite(instrSession, repCapName, attr, " \"", value, "\"");
}

/*****************************************************************************
* Function: rsspecan_quotedInt_WriteCallback
* Purpose:  Writes int single quotes
*****************************************************************************/
ViStatus rsspecan_quotedInt_WriteCallback(ViSession          instrSession,
                                          ViConstString      repCapName,
                                          RsCoreAttributePtr attr,
                                          void*              value)
{
    return RsCore_SpecialCallback_SurroundParamWrite(instrSession, repCapName, attr, " '", value, "'");
}

/*****************************************************************************
 * Function: rsspecan_FileStateSpecialFormat_WriteCallback
 * Purpose:  This function formats 1, before the argument string
 *****************************************************************************/
ViStatus rsspecan_FileStateSpecialFormat_WriteCallback(ViSession          instrSession,
                                                       ViConstString      repCapName,
                                                       RsCoreAttributePtr attr,
                                                       void*              value)
{
    return RsCore_SpecialCallback_SurroundParamWrite(instrSession, repCapName, attr, " 1,'", value, "'");
}

/*****************************************************************************
* Function: rsspecan_HcopyColorDef_WriteCallback
* Purpose:  Writes int without a space: "<cmd>%ld"
*****************************************************************************/
ViStatus rsspecan_HcopyColorDef_WriteCallback(ViSession          instrSession,
                                              ViConstString      repCapName,
                                              RsCoreAttributePtr attr,
                                              void*              value)
{
    return RsCore_SpecialCallback_SurroundParamWrite(instrSession, repCapName, attr, "", value, "");
}

/*****************************************************************************
* Function: rsspecan_VsaDemodRecLengthSym_WriteCallback
* Purpose:  Writes values with SYM afterwards
*****************************************************************************/
ViStatus rsspecan_VsaDemodRecLengthSym_WriteCallback(ViSession          instrSession,
                                                     ViConstString      repCapName,
                                                     RsCoreAttributePtr attr,
                                                     void*              value)
{
    return RsCore_SpecialCallback_SurroundParamWrite(instrSession, repCapName, attr, " ", value, "SYM");
}

/*****************************************************************************
* Function: rsspecan_VsaDemodRecLengthSec_WriteCallback
* Purpose:  This function overrides standard calback funtion.
*****************************************************************************/
ViStatus rsspecan_VsaDemodRecLengthSec_WriteCallback(ViSession          instrSession,
                                                     ViConstString      repCapName,
                                                     RsCoreAttributePtr attr,
                                                     void*              value)
{
    return RsCore_SpecialCallback_SurroundParamWrite(instrSession, repCapName, attr, " ", value, "S");
}

/******************************************************************************************************
    rsspecan_DeltaMarkerReferenceMarker_WriteCallback
    Supported data types: RS_VAL_INT32, RS_VAL_REAL64
    Writes as integer, but for 0 it writes "FIX" instead of the number
*****************************************************************************************************/
ViStatus rsspecan_DeltaMarkerReferenceMarker_WriteCallback(ViSession          instrSession,
                                                           ViConstString      repCapName,
                                                           RsCoreAttributePtr attr,
                                                           void*              value)
{
    ViStatus error = VI_SUCCESS;
    ViChar*  cmd = NULL;
    ViChar   aux[RS_MAX_MESSAGE_BUF_SIZE];

    // If no command available, exit the function
    if (attr->command == NULL)
        return VI_SUCCESS;

    checkErr(RsCore_BuildAttrCommandString(instrSession, attr, repCapName, &cmd));

    switch (attr->dataType)
    {
    case RS_VAL_INT32:
        if (*(ViInt32*)value == 0)
            snprintf(aux, RS_MAX_MESSAGE_BUF_SIZE, "FIX");
        else
            snprintf(aux, RS_MAX_MESSAGE_BUF_SIZE, _PERCD, *(ViInt32*)value);

        break;
    case RS_VAL_REAL64:
        if (fabs(*(ViReal64*)value) <= DBL_MIN)
            snprintf(aux, RS_MAX_MESSAGE_BUF_SIZE, "FIX");
        else
            RsCore_Convert_ViReal64_To_String(instrSession, RS_MAX_MESSAGE_BUF_SIZE, aux);

        break;
    default:
        checkErr(RsCore_GenerateInvalidDataTypeError(instrSession, attr->dataType,
            "rsspecan_DeltaMarkerReferenceMarker_WriteCallback only accepts type RS_VAL_INT32, RS_VAL_REAL64"));
    }

    checkErr(RsCore_StrCatDynamicString(instrSession, &cmd, " ", aux, NULL));
    checkErr(RsCore_WriteAttributeCommand(instrSession, attr, cmd));

Error:
    if (cmd)
        free(cmd);

    return error;
}

/*****************************************************************************
 * Function: rsspecan_GapLimit_WriteCallback
 * Purpose:  This function overrides standard calback funtion.
 *****************************************************************************/
ViStatus rsspecan_GapLimit_WriteCallback(ViSession          instrSession,
                                         ViConstString      repCapName,
                                         RsCoreAttributePtr attr,
                                         void*              value)
{
    return RsCore_SpecialCallback_SurroundParamWrite(instrSession, repCapName, attr, " ", value, ",0");
}

/*****************************************************************************
 * Function: rsspecan_DisplayColorPreset_WriteCallback
 * Purpose:  This function overrides standard calback funtion.
 *****************************************************************************/
ViStatus rsspecan_DisplayColorPreset_WriteCallback(ViSession          instrSession,
                                                   ViConstString      repCapName,
                                                   RsCoreAttributePtr attr,
                                                   void*              value)
{
    return RsCore_SpecialCallback_SurroundParamWrite(instrSession, repCapName, attr, "", value, "");
}

/*****************************************************************************
 * Function: rsspecan_LTEUplinkSubframe_WriteCallback
 * Purpose:  If value is integer -1, the parameter is SUBF.
             Otherwise it behaves as a standard callback
 *****************************************************************************/
ViStatus rsspecan_LTEUplinkSubframe_WriteCallback(ViSession          instrSession,
                                                  ViConstString      repCapName,
                                                  RsCoreAttributePtr attr,
                                                  void*              value)
{
    ViStatus error = VI_SUCCESS;
    ViChar*  cmd = NULL;

    // If no command is available, exit the function
    if (isNullOrEmpty(attr->command))
        goto Error;

    checkErr(RsCore_BuildAttrCommandString(instrSession, attr, repCapName, &cmd));

    // No range table, cmdString is composed based on the data type
    switch (attr->dataType)
    {
    case RS_VAL_INT32:
        if (*(ViInt32*)value != -1)
        {
            checkErr(RsCore_StrCatDynamicViInt32(instrSession, &cmd, " ", *(ViInt32 *)value, NULL));
        }
        else
        {
            checkErr(RsCore_StrCatDynamicString(instrSession, &cmd, " SUBF", NULL, NULL));
        }
        break;
    default:
        checkErr(RsCore_GenerateInvalidDataTypeError(instrSession, attr->dataType, NULL));
    }

    checkErr(RsCore_WriteAttributeCommand(instrSession, attr, cmd));

Error:
    if (cmd)
        free(cmd);

    return error;
}

/*****************************************************************************
 * Function: rsspecan_SetActiveWindowRepCapString_ReadCallback
 * Purpose:  Replaces <Window> with numbers in the command string
 *****************************************************************************/
ViStatus rsspecan_SetActiveWindowRepCapString_ReadCallback(ViSession          instrSession,
                                                           ViConstString      repCapName,
                                                           RsCoreAttributePtr attr)
{
    ViStatus error = VI_SUCCESS;
    ViInt32  window = 0;
    ViChar   windowValue[RS_MAX_SHORT_MESSAGE_BUF_SIZE];
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_GetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_ACTIVE_WINDOW, 0, &window));

    if (window == 0)
        snprintf(windowValue, RS_MAX_SHORT_MESSAGE_BUF_SIZE, "");
    else
        snprintf(windowValue, RS_MAX_SHORT_MESSAGE_BUF_SIZE, "%ld", window);

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "%s", attr->command);
    RsCore_ReplaceString(cmd, RS_MAX_MESSAGE_BUF_SIZE, "<Window>", windowValue, VI_TRUE);
    checkErr(RsCore_SpecialCallback_CustomCommandRead(instrSession, repCapName, attr, cmd));

Error:
    return error;
}

/*****************************************************************************
 * Function: rsspecan_SetActiveWindowRepCap_WriteCallback
 * Purpose:  Replaces <Window> with numbers in the command string
 *****************************************************************************/
ViStatus rsspecan_SetActiveWindowRepCap_WriteCallback(ViSession          instrSession,
                                                      ViConstString      repCapName,
                                                      RsCoreAttributePtr attr,
                                                      void*              value)
{
    ViStatus error = VI_SUCCESS;
    ViInt32  window = 0;
    ViChar   windowValue[RS_MAX_SHORT_MESSAGE_BUF_SIZE];
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_GetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_ACTIVE_WINDOW, 0, &window));

    if (window == 0)
        snprintf(windowValue, RS_MAX_SHORT_MESSAGE_BUF_SIZE, "");
    else
        snprintf(windowValue, RS_MAX_SHORT_MESSAGE_BUF_SIZE, "%ld", window);

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "%s", attr->command);
    RsCore_ReplaceString(cmd, RS_MAX_MESSAGE_BUF_SIZE, "<Window>", windowValue, VI_TRUE);

    checkErr(RsCore_SpecialCallback_CustomCommandWrite(instrSession, repCapName, attr, cmd, value));

Error:
    return error;
}

//Replaces <Window> with numbers in the command string
ViStatus rsspecan_SetActiveWindowRepCapInt32_ReadCallback(ViSession          instrSession,
                                                          ViConstString      repCapName,
                                                          RsCoreAttributePtr attr)
{
    return rsspecan_SetActiveWindowRepCapString_ReadCallback(instrSession, repCapName, attr);
}

//Replaces <Window> with numbers in the command string
ViStatus rsspecan_SetActiveWindowRepCapString_WriteCallback(ViSession          instrSession,
                                                            ViConstString      repCapName,
                                                            RsCoreAttributePtr attr,
                                                            void*              value)
{
    return rsspecan_SetActiveWindowRepCap_WriteCallback(instrSession, repCapName, attr, value);
}

//Replaces <Window> with numbers in the command string
ViStatus rsspecan_SetActiveWindowRepCapInt32_WriteCallback(ViSession          instrSession,
                                                           ViConstString      repCapName,
                                                           RsCoreAttributePtr attr,
                                                           void*              value)
{
    return rsspecan_SetActiveWindowRepCap_WriteCallback(instrSession, repCapName, attr, value);
}

//Replaces <Window> with numbers in the command string
ViStatus rsspecan_SetActiveWindowRepCapEvent_WriteCallback(ViSession          instrSession,
                                                           ViConstString      repCapName,
                                                           RsCoreAttributePtr attr,
                                                           void*              value)
{
    return rsspecan_SetActiveWindowRepCap_WriteCallback(instrSession, repCapName, attr, value);
}

//Replaces <Window> with numbers in the command string
ViStatus rsspecan_SetActiveWindowRepCapBoolean_ReadCallback(ViSession          instrSession,
                                                            ViConstString      repCapName,
                                                            RsCoreAttributePtr attr)
{
    return rsspecan_SetActiveWindowRepCapString_ReadCallback(instrSession, repCapName, attr);
}

//Replaces <Window> with numbers in the command string
ViStatus rsspecan_SetActiveWindowRepCapBoolean_WriteCallback(ViSession          instrSession,
                                                             ViConstString      repCapName,
                                                             RsCoreAttributePtr attr,
                                                             void*              value)
{
    return rsspecan_SetActiveWindowRepCap_WriteCallback(instrSession, repCapName, attr, value);
}

//Replaces <Window> with numbers in the command string
ViStatus rsspecan_SetActiveWindowRepCapReal64_ReadCallback(ViSession          instrSession,
                                                           ViConstString      repCapName,
                                                           RsCoreAttributePtr attr)
{
    return rsspecan_SetActiveWindowRepCapString_ReadCallback(instrSession, repCapName, attr);
}

//Replaces <Window> with numbers in the command string
ViStatus rsspecan_SetActiveWindowRepCapReal64_WriteCallback(ViSession          instrSession,
                                                            ViConstString      repCapName,
                                                            RsCoreAttributePtr attr,
                                                            void*              value)
{
    return rsspecan_SetActiveWindowRepCap_WriteCallback(instrSession, repCapName, attr, value);
}

/*****************************************************************************
 * Function: rsspecan_SyncCoupling_WriteCallback
 * Purpose:  This function overrides standard calback funtion.
 *****************************************************************************/
ViStatus rsspecan_SyncCoupling_WriteCallback(ViSession          instrSession,
                                             ViConstString      repCapName,
                                             RsCoreAttributePtr attr,
                                             void*              value)
{
    ViStatus error = VI_SUCCESS;
    ViChar*  cmd = NULL;

    // If no command is available, exit the function
    if (isNullOrEmpty(attr->command))
        goto Error;

    checkErr(RsCore_BuildAttrCommandString(instrSession, attr, repCapName, &cmd));

    switch (attr->dataType)
    {
    case RS_VAL_BOOLEAN:
        checkErr(RsCore_StrCatDynamicString(instrSession, &cmd, " ", *(ViInt32 *)value ? "ALL" : "NONE", NULL));
        break;
    default:
        checkErr(RsCore_GenerateInvalidDataTypeError(instrSession, attr->dataType, NULL));
    }

    checkErr(RsCore_WriteAttributeCommand(instrSession, attr, cmd));

Error:
    if (cmd)
        free(cmd);

    return error;
}

/*****************************************************************************
*- Check Callbacks ---------------------------------------------------------*
*****************************************************************************/

/*****************************************************************************
 *- Coerce Callbacks --------------------------------------------------------*
 *****************************************************************************/

/*****************************************************************************
 *- Range Table Callbacks ---------------------------------------------------*
 *****************************************************************************/
