/*****************************************************************************
 *  Rohde&Schwarz Spectrum Analyzer instrument driver - K14 Real Time Spectrum
 *  VXIpnp Style Instrument Driver
 *  Original Release: September 2005
 *  By: Martin Koutny (instrument driver core)
 *      Helena Sucha (instrument specific implementation)
 *      Jiri Kominek (DrBase repository & tools)
 *
 *  Should you have any technical questions please contact the hotline of
 *  Rohde & Schwarz Vertriebs-GmbH Rohde & Schwarz Support Center
 *
 *  e-mail: CustomerSupport@rohde-schwarz.com
 *
 *****************************************************************************/

#include "rsspecan.h"

/*****************************************************************************
 *- RSSpecAn K14 - Spectrogram Measurement ----------------------------------------*
 *****************************************************************************/

//static ViReal64 rateArr[] = {15.625E3, 31.25E3, 62.5E3, 125E3, 250E3, 500E3, 1E6, 2E6, 4E6, 8E6, 16E6, 32E6};
static ViString specPeakArr[] = {"", ":NEXT", ":ABOV", ":BEL"};

/// HIFN This function activates realtime mode.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.
ViStatus _VI_FUNC rsspecan_RealtimeSpectrumAnalysisMode(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_REALTIME_MODE, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function Selects a realtime measurement optimized either for high
/// HIFN  frequency resolution or providing additional evaluation in the time
/// HIFN  domain.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR realtimeMeasurement/This control selects a realtime measurement optimized either for high
/// HIPAR realtimeMeasurement/frequency resolution or providing additional evaluation in the time
/// HIPAR realtimeMeasurement/domain.
ViStatus _VI_FUNC rsspecan_SelectRealtimeSpectrumMeasurement(ViSession instrSession,
                                                             ViInt32   realtimeMeasurement)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_REALTIME_MEASUREMENT, realtimeMeasurement),
            2, "Realtime Measurement");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/*****************************************************************************
 * Function: Spectrogram Mode
 * Purpose:  This function activates or deactivates the Spectrogram
 *           Measurement option.
 *****************************************************************************/
ViStatus _VI_FUNC rsspecan_SpectrogramMode(ViSession instrSession,
                                           ViBoolean state)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_SPEM_STATE, state),
            2, "State");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/*****************************************************************************
 * Function: Configure Spectrogram FFT Window
 * Purpose:  This function selects the type of FFT window that you want to
 *           use in realtime mode.
 *****************************************************************************/
ViStatus _VI_FUNC rsspecan_ConfigureSpectrogramFFTWindow(ViSession instrSession,
                                                         ViInt32   FFTWindowType)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_SPEM_FFT_WINDOW, FFTWindowType),
            2, "FFT Window Type");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/*****************************************************************************
 * Function: Spectrogram Window
 * Purpose:  This function selects the current window for the realtime
 *           spectrum.
 *****************************************************************************/
ViStatus _VI_FUNC rsspecan_SpectrogramWindow(ViSession instrSession,
                                             ViInt32   window)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_SPEM_WINDOW, window));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function defines the dwell time.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR _auto/Determines whether the dwell time is defined automatically or
/// HIPAR _auto/manually.
/// HIPAR dwellTime/Determines the amount of time used to sample a continuous stream
/// HIPAR dwellTime/of I/Q data.
ViStatus _VI_FUNC rsspecan_DwellTime(ViSession instrSession,
                                     ViBoolean _auto,
                                     ViReal64  dwellTime)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_DWELL_TIME_AUTO, _auto),
            2, "Auto");
    if (_auto == VI_FALSE)
    {
        viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_DWELL_TIME, dwellTime),
                3, "Dwell Time");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function enables/disables the persistence spectrum.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR state/This control switches persistence in the persistence spectrum on
/// HIPAR state/and off.
ViStatus _VI_FUNC rsspecan_ConfigurePersistenceSpectrumState(ViSession instrSession,
                                                             ViBoolean state)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_PSPEM_STATE, state));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function sets the persistence spectrum parameters.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR style/This control sets the display style of the persistence spectrum.
/// HIPAR persistence/This control sets the duration of the persistence.
/// HIPAR granularity/This control sets the persistence granularity.
ViStatus _VI_FUNC rsspecan_ConfigurePersistenceSpectrum(ViSession instrSession,
                                                        ViInt32   style,
                                                        ViReal64  persistence,
                                                        ViReal64  granularity)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_PSPEM_TRACE_STYLE, style),
            2, "Style");

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_PSPEM_DURATION, persistence),
            3, "Persistence");

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_PSPEM_GRAN, granularity),
            4, "Granularity");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function sets the persistence spectrum maxhold trace
/// HIFN parameters.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR enableMaxhold/This control switches the maxhold trace in the persistence
/// HIPAR enableMaxhold/spectrum on and off.
/// HIPAR intensity/The control defines the color intensity of the maxhold
/// HIPAR intensity/persistence spectrum.
ViStatus _VI_FUNC rsspecan_ConfigurePersistenceSpectrumMaxholdIntensity(
                            ViSession instrSession,
                            ViInt32   reserved,
                            ViBoolean enableMaxhold,
                            ViInt32   intensity)
{
    ViStatus error = VI_SUCCESS;
             reserved;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_PSPEM_MAXH_TRACE, enableMaxhold),
            3, "Enable Maxhold");

    if (enableMaxhold == VI_TRUE)
    {
        viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_PSPEM_MAXH_INTENS, intensity),
                4, "Intensity");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function resets the maxhold trace in the persistence
/// HIFN spectrum result display.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
ViStatus _VI_FUNC rsspecan_ConfigurePersistenceSpectrumMaxholdReset(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_PSPEM_MAXH_RESET, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function defines if the delta marker is positioned on the
/// HIFN persistence trace or the maxhold trace. .
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the
/// HIPAR instrSession/Initialize function to select the desired instrument driver session.
/// HIPAR marker/This control selects delta marker.
/// HIPAR markerPostion/This control defines if the marker is positioned on the
/// HIPAR markerPostion/persistence trace or the maxhold trace.
ViStatus _VI_FUNC rsspecan_ConfigurePersistenceSpectrumMarkerPosition(
                            ViSession instrSession,
                            ViInt32   marker,
                            ViInt32   markerPostion)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_PSPEM_MARKER_POSITION,
            "Marker", marker - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Marker");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_PSPEM_MARKER_POSITION, markerPostion),
            3, "Marker Postion");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN This function queries the z-axis of the indicated marker in the
/// HIFN persistence spectrum result display.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the
/// HIPAR instrSession/Initialize function to select the desired instrument driver session.
/// HIPAR marker/This control selects marker.
/// HIPAR zAxis/queries the z-axis of the indicated marker in the persistence
/// HIPAR zAxis/spectrum result display.
ViStatus _VI_FUNC rsspecan_QueryPersistenceMarkerZAxis(ViSession instrSession,
                                                       ViInt32   marker,
                                                       ViReal64* zAxis)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_QUERY_PSPEM_MARKER_Z_AXIS,
            "Marker", marker - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Marker");

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_QUERY_PSPEM_MARKER_Z_AXIS, zAxis),
            3, "Z-Axis");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function defines if the delta marker is positioned on the
/// HIFN persistence trace or the maxhold trace. .
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the
/// HIPAR instrSession/Initialize function to select the desired instrument driver session.
/// HIPAR marker/This control selects delta marker.
/// HIPAR deltaMarkerPostion/This control defines if the delta marker is positioned on the
/// HIPAR deltaMarkerPostion/persistence trace or the maxhold trace.
ViStatus _VI_FUNC rsspecan_ConfigurePersistenceSpectrumDeltaMarkerPosition(
                            ViSession instrSession,
                            ViInt32   marker,
                            ViInt32   deltaMarkerPostion)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_PSPEM_DELTA_MARKER_POSITION,
            "DeltaMarker", marker - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Marker");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_PSPEM_DELTA_MARKER_POSITION, deltaMarkerPostion),
            3, "Delta Marker Postion");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/*****************************************************************************
 * Function: Query Persistence Delta Marker Z-Axis
 * Purpose:  This function queries the z-axis of the indicated delta marker
 *           in the persistence spectrum result display.
 *****************************************************************************/
ViStatus _VI_FUNC rsspecan_QueryPersistenceDeltaMarkerZAxis(ViSession instrSession,
                                                            ViInt32   deltaMarker,
                                                            ViReal64* zAxis)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_QUERY_PSPEM_DELTA_Z_AXIS,
            "DeltaMarker", deltaMarker - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Delta Marker");

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_QUERY_PSPEM_DELTA_Z_AXIS, zAxis),
            3, "Z-Axis");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/*****************************************************************************
 * Function: Configure Persistence Spectrum Display Parameters
 * Purpose:  This function sets the persistence spectrum display parameters.
 *****************************************************************************/
ViStatus _VI_FUNC rsspecan_ConfigurePersistenceSpectrumDisplayParameters(
                            ViSession instrSession,
                            ViReal64  shape,
                            ViInt32   style,
                            ViReal64  colorLower,
                            ViReal64  colorUpper)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_PSPEM_SHAPE, shape),
            3, "Shape");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_PSPEM_COLOR_STYLE, style),
            4, "Style");

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_PSPEM_COLOR_LOWER, colorLower),
            5, "Color Lower");

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_PSPEM_COLOR_UPPER, colorUpper),
            5, "Color Upper");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function sets the color settings for the persistence
/// HIFN spectrum result display to its default state.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.
ViStatus _VI_FUNC rsspecan_ConfigurePersistenceSpectrumDefaultColors(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_PSPEM_COLOR_DEFAULT, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/*****************************************************************************
 * Function: Configure Persistence Spectrum Truncate
 * Purpose:  This function reduces the range of the color map of the
             persistence spectrum if there are no hits at the start or end of
             the value range.
 *****************************************************************************/
ViStatus _VI_FUNC rsspecan_ConfigurePersistenceSpectrumTruncate(
                            ViSession instrSession,
                            ViBoolean truncate)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_PSPEM_TRUN, truncate),
            5, "Truncate");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/*****************************************************************************
 * Function: Configure Spectrogram Measurement Parameters
 * Purpose:  This function configures Spectogram measurement parameters.
 *****************************************************************************/
ViStatus _VI_FUNC rsspecan_ConfigureSpectrogramMeasurementParameters(
                            ViSession instrSession,
                            ViInt32   historyDepth,
                            ViBoolean timeStamp,
                            ViInt32   frameCount,
                            ViBoolean continueFrame)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_SPEM_HDEP, historyDepth),
            2, "History Depth");

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_SPEM_TIME_STAMP_STATE, timeStamp),
            3, "Time Stamp");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_SPEM_FRAME_COUNT, frameCount),
            4, "Frame Count");

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_SPEM_CONTINUOUS, continueFrame),
            5, "Continue Frame");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/*****************************************************************************
 * Function: Configure Spectrogram Selected Frame
 * Purpose:  This function selects the frame if no measurement is running.
             During a measurement, the value is fixed to 0 and this command
             is available only if no measurement is running or after a single
             sweep has ended.
 *****************************************************************************/
ViStatus _VI_FUNC rsspecan_ConfigureSpectrogramSelectedFrame(ViSession instrSession,
                                                             ViInt32   frame_number)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_SPEM_FRAME_SELECT, frame_number),
            2, "Frame number");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/*****************************************************************************
 * Function: Get Spectrogram Time Stamp
 * Purpose:  This function queries the time stamp of the frames.
 *****************************************************************************/
ViStatus _VI_FUNC rsspecan_GetSpectrogramTimeStamp(ViSession instrSession,
                                                   ViInt32   frame,
                                                   ViInt32   arraySize,
                                                   ViInt32   part1[],
                                                   ViInt32   part2[],
                                                   ViInt32   part3[],
                                                   ViInt32   part4[],
                                                   ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViChar*  pbuffer = NULL;
    ViInt32  i;
    ViChar*  pstring_value;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, frame, 0, 1),
            2, "Frame");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, ":CALC:SPEC:TST:DATA? %s", (frame == 0) ? "CURR" : "ALL");
    checkErr(RsCore_QueryViStringUnknownLength(instrSession, cmd, &pbuffer));

    pstring_value = strtok(pbuffer, ",");
    i = 0;
    while (pstring_value)
    {
        if (i < arraySize)
        {
            part1[i] = RsCore_Convert_String_To_ViInt32(pstring_value);
            pstring_value = strtok(NULL, ",");
            part2[i] = RsCore_Convert_String_To_ViInt32(pstring_value);
            pstring_value = strtok(NULL, ",");
            part3[i] = RsCore_Convert_String_To_ViInt32(pstring_value);
            pstring_value = strtok(NULL, ",");
            part4[i] = RsCore_Convert_String_To_ViInt32(pstring_value);
            pstring_value = strtok(NULL, ",");
        }
        else
        {
            (void)strtok(NULL, ",");
            (void)strtok(NULL, ",");
            (void)strtok(NULL, ",");
            pstring_value = strtok(NULL, ",");
        }
        i++;
    };
    if (returnedValues) *returnedValues = i;

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    if (pbuffer) free(pbuffer);
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/*****************************************************************************
 * Function: Spectrogram Clear
 * Purpose:  This function deletes the spectrogram result display and the history buffer.
 *****************************************************************************/
ViStatus _VI_FUNC rsspecan_SpectrogramClear(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_SPEM_CLEAR, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/*****************************************************************************
 * Function: Configure Spectrogram Display Parameters
 * Purpose:  This function configures Spectogram display parameters.
 *****************************************************************************/
ViStatus _VI_FUNC rsspecan_ConfigureSpectrogramDisplayParameters(
                            ViSession instrSession,
                            ViInt32   color,
                            ViInt32   size)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_SPEM_COLOR, color),
            2, "Color");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_SPEM_SIZE, size),
            3, "Size");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/*****************************************************************************
 * Function: Configure Spectrogram Colors
 * Purpose:  This function configures Spectogram color parameters.
 *****************************************************************************/
ViStatus _VI_FUNC rsspecan_ConfigureSpectrogramColors(ViSession instrSession,
                                                      ViReal64  shape,
                                                      ViInt32   colorStyle,
                                                      ViReal64  colorLower,
                                                      ViReal64  colorUpper)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_SPEM_SHAPE, shape),
            2, "Shape");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_SPEM_COLOR_STYLE, colorStyle),
            3, "Color Style");

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_SPEM_COLOR_LOW, colorLower),
            4, "Color Lower");

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_SPEM_COLOR_UPPER, colorUpper),
            5, "Color Upper");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/*****************************************************************************
 * Function: Configure Spectrogram Default Colors
 * Purpose:  This function sets the color settings for the spectrogram result
             display to its default state.
 *****************************************************************************/
ViStatus _VI_FUNC rsspecan_ConfigureSpectrogramDefaultColors(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_SPEM_COLOR_DEFAULT, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/*****************************************************************************
 * Function: Configure Spectrogram Marker Frame Number
 * Purpose:  This function defines the frame number of the specified marker.
 *****************************************************************************/
ViStatus _VI_FUNC rsspecan_ConfigureSpectrogramMarkerFrameNumber(
                            ViSession instrSession,
                            ViInt32   markerNumber,
                            ViInt32   frameNumber)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_SPEM_MARK_FRAME_NUMBER,
            "Marker", markerNumber - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Marker Number");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_SPEM_MARK_FRAME_NUMBER, frameNumber),
            3, "Frame Number");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/*****************************************************************************
 * Function: Get Spectrogram Marker Frame Number
 * Purpose:  This function returns the frame number of the specified marker.
 *****************************************************************************/
ViStatus _VI_FUNC rsspecan_GetSpectrogramMarkerFrameNumber(ViSession instrSession,
                                                           ViInt32   markerNumber,
                                                           ViInt32*  frameNumber)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_SPEM_MARK_FRAME_NUMBER,
            "Marker", markerNumber - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Marker Number");

    viCheckParm(rsspecan_GetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_SPEM_MARK_FRAME_NUMBER, frameNumber),
            3, "Frame Number");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/*****************************************************************************
 * Function: Configure Spectrogram Marker Search Area
 * Purpose:  This function defines the search area of the specified marker.
 *****************************************************************************/
ViStatus _VI_FUNC rsspecan_ConfigureSpectrogramMarkerSearchArea(
                            ViSession instrSession,
                            ViInt32   markerNumber,
                            ViInt32   searchArea)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_SPEM_MARKER_SEARCH_AREA,
            "Marker", markerNumber - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Marker Number");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_SPEM_MARKER_SEARCH_AREA, searchArea),
            3, "Search Area");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/*****************************************************************************
 * Function: Configure Spectrogram Marker Search
 * Purpose:  This function positions the specified marker on the specified place.
 *****************************************************************************/
ViStatus _VI_FUNC rsspecan_SpectrogramMarkerSearch(ViSession instrSession,
                                                   ViInt32   markerNumber,
                                                   ViInt32   searchMode)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_MARKER_SEARCH_PEAK,
            "Marker", markerNumber - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Marker Number");

    switch (searchMode)
    {
    case RSSPECAN_VAL_SPECM_MARK_MAX_PEAK:
        checkErr(rsspecan_SetAttributeViString(instrSession, repCap, RSSPECAN_ATTR_MARKER_SEARCH_PEAK, NULL));
        break;
    case RSSPECAN_VAL_SPECM_MARK_MAX_NEXT:
        checkErr(rsspecan_SetAttributeViString(instrSession, repCap, RSSPECAN_ATTR_MARKER_SEARCH_PEAK_NEXT, NULL));
        break;
    case RSSPECAN_VAL_SPECM_MARK_MAX_LEFT:
        checkErr(rsspecan_SetAttributeViString(instrSession, repCap, RSSPECAN_ATTR_MARKER_SEARCH_PEAK_LEFT, NULL));
        break;
    case RSSPECAN_VAL_SPECM_MARK_MAX_RIGHT:
        checkErr(rsspecan_SetAttributeViString(instrSession, repCap, RSSPECAN_ATTR_MARKER_SEARCH_PEAK_RIGHT, NULL));
        break;
    case RSSPECAN_VAL_SPECM_MARK_MIN_PEAK:
        checkErr(rsspecan_SetAttributeViString(instrSession, repCap, RSSPECAN_ATTR_MARKER_SEARCH_MIN, NULL));
        break;
    case RSSPECAN_VAL_SPECM_MARK_MIN_NEXT:
        checkErr(rsspecan_SetAttributeViString(instrSession, repCap, RSSPECAN_ATTR_MARKER_SEARCH_MIN_NEXT, NULL));
        break;
    case RSSPECAN_VAL_SPECM_MARK_MIN_LEFT:
        checkErr(rsspecan_SetAttributeViString(instrSession, repCap, RSSPECAN_ATTR_MARKER_SEARCH_MIN_LEFT, NULL));
        break;
    case RSSPECAN_VAL_SPECM_MARK_MIN_RIGHT:
        checkErr(rsspecan_SetAttributeViString(instrSession, repCap, RSSPECAN_ATTR_MARKER_SEARCH_MIN_RIGHT, NULL));
        break;
    case RSSPECAN_VAL_SPECM_MARK_Y_MAX_PEAK:
    case RSSPECAN_VAL_SPECM_MARK_Y_MAX_NEXT:
    case RSSPECAN_VAL_SPECM_MARK_Y_MAX_ABOVE:
    case RSSPECAN_VAL_SPECM_MARK_Y_MAX_BELOW:
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CALC:MARK%ld:SPEC:Y:MAX%s", markerNumber,
                 specPeakArr[searchMode - RSSPECAN_VAL_SPECM_MARK_Y_MAX_PEAK]);
        checkErr(RsCore_Write(instrSession, cmd));
        break;
    case RSSPECAN_VAL_SPECM_MARK_Y_MIN_PEAK:
    case RSSPECAN_VAL_SPECM_MARK_Y_MIN_NEXT:
    case RSSPECAN_VAL_SPECM_MARK_Y_MIN_ABOVE:
    case RSSPECAN_VAL_SPECM_MARK_Y_MIN_BELOW:
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CALC:MARK%ld:SPEC:Y:MIN%s", markerNumber,
                 specPeakArr[searchMode - RSSPECAN_VAL_SPECM_MARK_Y_MIN_PEAK]);
        checkErr(RsCore_Write(instrSession, cmd));
        break;
    case RSSPECAN_VAL_SPECM_MARK_XY_MAX:
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CALC:MARK%ld:SPEC:XY:MAX", markerNumber);
        checkErr(RsCore_Write(instrSession, cmd));
        break;
    case RSSPECAN_VAL_SPECM_MARK_XY_MIN:
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CALC:MARK%ld:SPEC:XY:MIN", markerNumber);
        checkErr(RsCore_Write(instrSession, cmd));
        break;
    case RSSPECAN_VAL_SPECM_MARK_Y_TRIGGER:
        viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_MARKER_SEARCH_Y_TRIGGER,
                "Delta Marker", markerNumber - 1, RS_REPCAP_BUF_SIZE, repCap),
                2, "Marker Number");
        checkErr(rsspecan_SetAttributeViString(instrSession, repCap, RSSPECAN_ATTR_MARKER_SEARCH_Y_TRIGGER, NULL));
        break;
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/*****************************************************************************
 * Function: Configure Spectrogram Delta Marker Frame Number
 * Purpose:  This function defines the frame number of the specified delta marker.
 *****************************************************************************/
ViStatus _VI_FUNC rsspecan_ConfigureSpectrogramDeltaMarkerFrameNumber(
                            ViSession instrSession,
                            ViInt32   deltaMarkerNumber,
                            ViInt32   frameNumber)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_SPEM_DELT_FRAME_NUMBER,
            "DeltaMarker", deltaMarkerNumber - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Delta Marker Number");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_SPEM_DELT_FRAME_NUMBER, frameNumber),
            3, "Frame Number");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/*****************************************************************************
 * Function: Get Spectrogram Delta Marker Frame Number
 * Purpose:  This function returns the frame number of the specified delta marker.
 *****************************************************************************/
ViStatus _VI_FUNC rsspecan_GetSpectrogramDeltaMarkerFrameNumber(
                            ViSession instrSession,
                            ViInt32   deltaMarker,
                            ViInt32*  frameNumber)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_SPEM_DELT_FRAME_NUMBER,
            "DeltaMarker", deltaMarker - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Delta Marker");

    viCheckParm(rsspecan_GetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_SPEM_DELT_FRAME_NUMBER, frameNumber),
            3, "Frame Number");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/*****************************************************************************
 * Function: Configure Spectrogram Delta Marker Search Area
 * Purpose:  This function defines the search area of the specified delta marker.
 *****************************************************************************/
ViStatus _VI_FUNC rsspecan_ConfigureSpectrogramDeltaMarkerSearchArea(
                            ViSession instrSession,
                            ViInt32   deltaMarker,
                            ViInt32   searchArea)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_SPEM_DELT_SEARCH_AREA,
            "DeltaMarker", deltaMarker - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Delta Marker");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_SPEM_DELT_SEARCH_AREA, searchArea),
            3, "Search Area");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/*****************************************************************************
 * Function: Configure Spectrogram Delta Marker Search
 * Purpose:  This function positions the specified delta marker on the specified place.
 *****************************************************************************/
ViStatus _VI_FUNC rsspecan_SpectrogramDeltaMarkerSearch(ViSession instrSession,
                                                        ViInt32   deltaMarker,
                                                        ViInt32   searchMode)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_REFERENCE_MARKER_PEAK,
            "Marker", deltaMarker - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Delta Marker");

    switch (searchMode)
    {
    case RSSPECAN_VAL_SPECM_MARK_MAX_PEAK:
        checkErr(rsspecan_SetAttributeViString(instrSession, repCap, RSSPECAN_ATTR_REFERENCE_MARKER_PEAK, NULL));
        break;
    case RSSPECAN_VAL_SPECM_MARK_MAX_NEXT:
        checkErr(rsspecan_SetAttributeViString(instrSession, repCap, RSSPECAN_ATTR_REFERENCE_MARKER_PEAK_NEXT, NULL));
        break;
    case RSSPECAN_VAL_SPECM_MARK_MAX_LEFT:
        checkErr(rsspecan_SetAttributeViString(instrSession, repCap, RSSPECAN_ATTR_REFERENCE_MARKER_PEAK_NEXT_LEFT, NULL));
        break;
    case RSSPECAN_VAL_SPECM_MARK_MAX_RIGHT:
        checkErr(rsspecan_SetAttributeViString(instrSession, repCap, RSSPECAN_ATTR_REFERENCE_MARKER_PEAK_NEXT_RIGHT, NULL));
        break;
    case RSSPECAN_VAL_SPECM_MARK_MIN_PEAK:
        checkErr(rsspecan_SetAttributeViString(instrSession, repCap, RSSPECAN_ATTR_REFERENCE_MARKER_MIN, NULL));
        break;
    case RSSPECAN_VAL_SPECM_MARK_MIN_NEXT:
        checkErr(rsspecan_SetAttributeViString(instrSession, repCap, RSSPECAN_ATTR_REFERENCE_MARKER_MIN_NEXT, NULL));
        break;
    case RSSPECAN_VAL_SPECM_MARK_MIN_LEFT:
        checkErr(rsspecan_SetAttributeViString(instrSession, repCap, RSSPECAN_ATTR_REFERENCE_MARKER_MIN_NEXT_LEFT, NULL));
        break;
    case RSSPECAN_VAL_SPECM_MARK_MIN_RIGHT:
        checkErr(rsspecan_SetAttributeViString(instrSession, repCap, RSSPECAN_ATTR_REFERENCE_MARKER_MIN_NEXT_RIGHT, NULL));
        break;
    case RSSPECAN_VAL_SPECM_MARK_Y_MAX_PEAK:
    case RSSPECAN_VAL_SPECM_MARK_Y_MAX_NEXT:
    case RSSPECAN_VAL_SPECM_MARK_Y_MAX_ABOVE:
    case RSSPECAN_VAL_SPECM_MARK_Y_MAX_BELOW:
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CALC:DELT%ld:SPEC:Y:MAX%s", deltaMarker,
                 specPeakArr[searchMode - RSSPECAN_VAL_SPECM_MARK_Y_MAX_PEAK]);
        checkErr(RsCore_Write(instrSession, cmd));
        break;
    case RSSPECAN_VAL_SPECM_MARK_Y_MIN_PEAK:
    case RSSPECAN_VAL_SPECM_MARK_Y_MIN_NEXT:
    case RSSPECAN_VAL_SPECM_MARK_Y_MIN_ABOVE:
    case RSSPECAN_VAL_SPECM_MARK_Y_MIN_BELOW:
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CALC:DELT%ld:SPEC:Y:MIN%s", deltaMarker,
                 specPeakArr[searchMode - RSSPECAN_VAL_SPECM_MARK_Y_MIN_PEAK]);
        checkErr(RsCore_Write(instrSession, cmd));
        break;
    case RSSPECAN_VAL_SPECM_MARK_XY_MAX:
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CALC:DELT%ld:SPEC:XY:MAX", deltaMarker);
        checkErr(RsCore_Write(instrSession, cmd));
        break;
    case RSSPECAN_VAL_SPECM_MARK_XY_MIN:
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CALC:DELT%ld:SPEC:XY:MIN", deltaMarker);
        checkErr(RsCore_Write(instrSession, cmd));
        break;
    case RSSPECAN_VAL_SPECM_MARK_Y_TRIGGER:

        checkErr(rsspecan_SetAttributeViString(instrSession, repCap, RSSPECAN_ATTR_REFERENCE_MARKER_Y_TRIGGER, NULL));
        break;
    }
    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/*****************************************************************************
 * Function: Configure Spectrogram Result Display Mode
 * Purpose:  This function selects the result display in realtime mode.
 *****************************************************************************/
ViStatus _VI_FUNC rsspecan_ConfigureSpectrogramResultDisplayMode(
                            ViSession instrSession,
                            ViInt32   resultDisplay)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_RESULT_DISPLAY, resultDisplay),
            2, "Result Display");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/*****************************************************************************
 * Function: Read Spectrogram Trace Data
 * Purpose:  This function ReadSpectrogramTraceData
 *****************************************************************************/
ViStatus _VI_FUNC rsspecan_ReadSpectrogramTraceData(ViSession instrSession,
                                                    ViInt32   arraySize,
                                                    ViReal64  results[],
                                                    ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;
    ViChar   trace[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    snprintf(trace, RS_MAX_MESSAGE_BUF_SIZE, "SPEC");
    checkErr(rsspecan_dataReadTrace(instrSession, 0, trace, arraySize, results, returnedValues));

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/*****************************************************************************
 * Function: Read Persistence Spectrum Trace Data
 * Purpose:  This function returns the results of the persistence spectrum
 *           result display. The function returns 801*600 percentages, one
 *           for each pixel in the histogram.
 *****************************************************************************/
ViStatus _VI_FUNC rsspecan_ReadPersistenceSpectrumTraceData(ViSession instrSession,
                                                            ViUInt32  timeout,
                                                            ViInt32   arraySize,
                                                            ViReal64  results[],
                                                            ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;
    ViChar   trace[RS_MAX_MESSAGE_BUF_SIZE];
    ViUInt32 old_timeout = 0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViUInt32Range(instrSession, timeout, 0, 4294967295UL), 2, "Timeout");

    checkErr(rsspecan_GetTimeout(instrSession, &old_timeout));
    checkErr(rsspecan_SetTimeout(instrSession, timeout));

    snprintf(trace, RS_MAX_MESSAGE_BUF_SIZE, "PSP");
    checkErr(rsspecan_dataReadTrace(instrSession, 0, trace, arraySize, results, returnedValues));

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    if (old_timeout > 0)
        rsspecan_SetTimeout(instrSession, timeout);
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/*****************************************************************************
 * Function: Read Maxhold Trace Data
 * Purpose:  This function returns the results of the maxhold trace in the
 *           persistence spectrum result display. The function returns
 *           801*600 percentages, one for each pixel in the maxhold trace.
 *****************************************************************************/
ViStatus _VI_FUNC rsspecan_ReadMaxholdTraceData(ViSession instrSession,
                                                ViUInt32  timeout,
                                                ViInt32   arraySize,
                                                ViReal64  results[],
                                                ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;
    ViChar   traceCmd[RS_MAX_MESSAGE_BUF_SIZE] = "";
    ViUInt32 old_timeout = 0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViUInt32Range(instrSession, timeout, 0, 4294967295UL), 2, "Timeout");

    checkErr(rsspecan_GetTimeout(instrSession, &old_timeout));
    checkErr(rsspecan_SetTimeout(instrSession, timeout));

    snprintf(traceCmd, RS_MAX_MESSAGE_BUF_SIZE, "HMAX");
    checkErr(rsspecan_dataReadTrace(instrSession, 0, traceCmd, arraySize, results, returnedValues));

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    if (old_timeout > 0)
        rsspecan_SetTimeout(instrSession, timeout);
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/*****************************************************************************
 * Function: Get Spectrogram X-Axis Scaling
 * Purpose:  This function queries the scaling of the x-axis in the
             spectrogram.
 *****************************************************************************/
ViStatus _VI_FUNC rsspecan_GetSpectrogramXAxisScaling(ViSession instrSession,
                                                      ViInt32   frame,
                                                      ViInt32   arraySize,
                                                      ViReal64  xAxis[],
                                                      ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    if (rsspecan_IsFSV(instrSession))
    {
        checkErr(RsCore_CheckInstrumentOptions(instrSession, "K14"));
    }

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, frame, RSSPECAN_VAL_SPECM_FRAME_CURRENT, RSSPECAN_VAL_SPECM_FRAME_ALL),
            2, "Frame");
    viCheckParm(RsCore_InvalidViInt32RangeMin(instrSession, arraySize, 1),
                3, "Array Size");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, xAxis),
                4, "X-Axis");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CALC:SPEC:X:DATA? %s",
             (frame == RSSPECAN_VAL_SPECM_FRAME_CURRENT) ? "CURR" : "ALL");
    checkErr(RsCore_QueryFloatArrayToUserBuffer(instrSession, cmd, arraySize, xAxis, returnedValues));

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/*****************************************************************************
 * Function: Store Spectrogram To File
 * Purpose:  This function stores the spectrogram data in ASCII format to a file.
 *****************************************************************************/
ViStatus _VI_FUNC rsspecan_StoreSpectrogramToFile(ViSession instrSession,
                                                  ViString  fileName)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_SPEM_STORE_TO_FILE, fileName),
            2, "File Name");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/*****************************************************************************
 * Function: Store Spectrogram To File
 * Purpose:  This function stores the spectrogram data in ASCII format to a file.
 *****************************************************************************/
ViStatus _VI_FUNC rsspecan_ConfigureSpectrogramContinuousTrigger(
                            ViSession instrSession,
                            ViInt32   trigger)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_SPEM_CONTINUOUS_TRIGGER, trigger),
            2, "Trigger");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/*****************************************************************************
 * Function: Configure Spectrogram Time Domain Trigger
 * Purpose:  This function configures time domain trigger as source and level.
 *****************************************************************************/
ViStatus _VI_FUNC rsspecan_ConfigureSpectrogramTimeDomainTrigger(
                            ViSession instrSession,
                            ViReal64  timeDomainTriggerLevel)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_SPEM_TIME_DOMAIN_TRIGGER, NULL));

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_SPEM_TIME_DOMAIN_TRIGGER_LEVEL, timeDomainTriggerLevel),
            2, "Time Domain Trigger Level");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/*****************************************************************************
 *- Callback Definition   ---------------------------------------------------*
 *****************************************************************************/

/*****************************************************************************
 *- End Of RSSpecAn K14 - Spectrogram Measurement ---------------------------*
 *****************************************************************************/
