/*****************************************************************************
 *  Rohde&Schwarz Spectrum Analyzer instrument driver - K15 Avionics
 *  VXIpnp Style Instrument Driver
 *  Original Release: June 2016
 *  By: Nabil Ouddane
 *
 *  Should you have any technical questions please contact the hotline of
 *  Rohde & Schwarz Vertriebs-GmbH Rohde & Schwarz Support Center
 *
 *  e-mail: CustomerSupport@rohde-schwarz.com
 *
 *****************************************************************************/

#include "rsspecan.h"

/*****************************************************************************
 *- RSSpecAn K15 - Avionics (VOR/ILS) measurements application ----------------------------------------*
 *****************************************************************************/

static ViString FrequencyIdentityArr[] = {"'30'", "'30FM'", "'90'", "'150'", "'90+150'", "'9960'", "'ID'"};

/// HIFN This function activates a new measurement channel for R&S FSW Avionics (VOR/ILS) measurements application.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.
ViStatus _VI_FUNC rsspecan_SetAvionicsMode(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_AVIONICS_MODE, ""));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN Defines the standard for which the signal parameters are measured.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.
/// HIPAR standard/Selects the standard for which the signal parameters are measured.
ViStatus _VI_FUNC rsspecan_ConfigureAvionicsMeasurementType(ViSession instrSession,
                                                            ViInt32   standard)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_AVIONICS_MEASUREMENT_TYPE, standard),
            2, "Standard");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN Configures the fundamental frequency of the harmonic distortion measurement and defines
/// HIFN the upper frequency limit for its measurements.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.
/// HIPAR fundamentalFrequencyIdentity/Selects the fundamental frequency of the harmonic distortion measurement.
/// HIPAR frequency/Defines the upper frequency limit for most total harmonic distortion measurements.
ViStatus _VI_FUNC rsspecan_ConfigureAvionicsTotalHarmonicDistortionFrequency(
                            ViSession instrSession,
                            ViInt32   fundamentalFrequencyIdentity,
                            ViReal64  frequency)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_AVIONICS_FUNDAMENTAL_FREQUENCY_IDENTITY, fundamentalFrequencyIdentity),
            2, "Fundamental Frequency Identity");

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_AVIONICS_TOTAL_HARMONIC_DISTORTION_FREQUENCY, frequency),
            3, "Frequency");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN Queries the result of the RF level measurement.
/// HIFN the upper frequency limit for its measurements.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.
/// HIPAR RFLevel/Returns the result of the RF level measurement.
ViStatus _VI_FUNC rsspecan_QueryAvionicsRFLevelResults(ViSession instrSession,
                                                       ViReal64* RFLevel)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_AVIONICS_RF_LEVEL_RESULTS, RFLevel),
            2, "RF Level");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN Queries the measured AF frequency for the specified fundamental frequency.
/// HIFN the upper frequency limit for its measurements.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.
/// HIPAR fundamentalFrequencyIdentity/Sets the signal component.
/// HIPAR AFFrequency/Returns the measured AF frequency for the specified fundamental frequency.
ViStatus _VI_FUNC rsspecan_QueryAvionicsAFFrequency(ViSession instrSession,
                                                    ViInt32   fundamentalFrequencyIdentity,
                                                    ViReal64* AFFrequency)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, fundamentalFrequencyIdentity, RSSPECAN_VAL_AVIONICS_FREQUENCY_30, RSSPECAN_VAL_AVIONICS_FREQUENCY_ID),
            2, "Fundamental Frequency Identity");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CALC:AVI:AM:FREQ? %s", FrequencyIdentityArr[fundamentalFrequencyIdentity]);
    checkErr(rsspecan_QueryViReal64(instrSession, cmd, AFFrequency));

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN Queries the result of the ILS DDM measurement.
/// HIFN the upper frequency limit for its measurements.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.
/// HIPAR DDM/Returns the difference in depth of modulation (DDM) between 90 Hz and 150 Hz AM signal (m90 Hz - m150 Hz).
ViStatus _VI_FUNC rsspecan_QueryAvionicsDifferenceInDepthofModulation(
                            ViSession instrSession,
                            ViReal64* DDM)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_AVIONICS_DIFFERENCE_IN_DEPTH_OF_MODULATION, DDM),
            2, "DDM");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN Queries the carrier offset.
/// HIFN the upper frequency limit for its measurements.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.
/// HIPAR carrierOffset/Returns the carrier offset.
ViStatus _VI_FUNC rsspecan_QueryAvionicsCarrierOffsetResults(ViSession instrSession,
                                                             ViReal64* carrierOffset)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_AVIONICS_CARRIER_OFFSET_RESULTS, carrierOffset),
            2, "Carrier Offset");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN Queries the result of the ILS or VOR phase measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.
/// HIPAR phase/Retruns the result of the ILS or VOR phase measurement.
ViStatus _VI_FUNC rsspecan_QueryAvionicsPhaseResults(ViSession instrSession,
                                                     ViReal64* phase)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_AVIONICS_PHASE_RESULTS, phase),
            2, "Phase");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the result of the RF frequency counter measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.
/// HIPAR RFFrequency/Returns the result of the RF frequency counter measurement.
ViStatus _VI_FUNC rsspecan_QueryAvionicsRFFrequencySignal(ViSession instrSession,
                                                          ViReal64* RFFrequency)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_AVIONICS_RF_FREQUENCY_SIGNAL, RFFrequency),
            2, "RF Frequency");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the ILS Sum in Depth of Modulation (SDM) result.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.
/// HIPAR SDM/Returns the ILS Sum in Depth of Modulation (SDM) result.
ViStatus _VI_FUNC rsspecan_QueryAvionicsILSSDM(ViSession instrSession,
                                               ViReal64* SDM)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_AVIONICS_ILS_SDM, SDM),
            2, "Sum in Depth of Modulation");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the harmonic distortion measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.
/// HIPAR state/Enables and disables the harmonic distortion.
/// HIPAR harmonicFrequency/Defines the frequency for which a harmonic distortion measurement is
/// HIPAR required. The fixed marker H1 is positioned at this frequency.
ViStatus _VI_FUNC rsspecan_ConfigureAvionicsHarmonicDistortion(
                            ViSession instrSession,
                            ViBoolean state,
                            ViReal64  harmonicFrequency)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_AVIONICS_HARMONIC_DISTORTION_STATE, state),
            2, "State");

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_AVIONICS_TOTAL_HARMONIC_DISTORTION_FREQUENCY, harmonicFrequency),
            3, "Harmonic Frequency");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the result of the harmonic distortion measurement performed at the position of the marker H1.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.
/// HIPAR result/Returns the result of the harmonic distortion measurement.
ViStatus _VI_FUNC rsspecan_QueryAvionicsHarmonicDistortionResults(
                            ViSession instrSession,
                            ViReal64* result)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_AVIONICS_HARMONIC_DISTORTION_RESULT, result),
            2, "Result");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Determines the unit for ILS DDM results (relevant for ILS measurements only).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.
/// HIPAR unit/Selects the unit for ILS DDM results.
ViStatus _VI_FUNC rsspecan_ConfigureILSDDMUnit(ViSession instrSession,
                                               ViInt32   unit)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_AVIONICS_ILS_DDM_UNIT, unit),
            2, "Unit");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Switches between a phase display in "from" or "to" notation.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.
/// HIPAR VORDirection/Switches between a phase display in from or to notation.
ViStatus _VI_FUNC rsspecan_ConfigureVORMeasurementUnit(ViSession instrSession,
                                                       ViInt32   VORDirection)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_AVIONICS_VOR_DIRECTION, VORDirection),
            2, "VOR Direction");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the amplitude modulation depth result for the specified signal component.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.
/// HIPAR fundamentalFrequencyIdentity/Sets the signal component, defined by its fundamental frequency.
/// HIPAR results/Returns the amplitude modulation depth of the selected signal component in percent.
ViStatus _VI_FUNC rsspecan_QueryAvionicsAMDepthResults(ViSession instrSession,
                                                       ViInt32   fundamentalFrequencyIdentity,
                                                       ViReal64* depth)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, fundamentalFrequencyIdentity, RSSPECAN_VAL_AVIONICS_FREQUENCY_30, RSSPECAN_VAL_AVIONICS_FREQUENCY_ID),
            2, "Fundamental Frequency Identity");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CALC:AVI:AM? %s", FrequencyIdentityArr[fundamentalFrequencyIdentity]);
    checkErr(RsCore_QueryViReal64(instrSession, cmd, depth));

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the total harmonic distortion (THD) for the specified fundamental frequency.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.
/// HIPAR fundamentalFrequencyIdentity/Sets the signal component.
/// HIPAR result/Returns the total harmonic distortion relative to the fundamental frequency.
ViStatus _VI_FUNC rsspecan_QueryAvionicsTotalHarmonicDistortion(
                            ViSession instrSession,
                            ViInt32   fundamentalFrequencyIdentity,
                            ViReal64* result)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, fundamentalFrequencyIdentity, RSSPECAN_VAL_AVIONICS_FREQUENCY_30, RSSPECAN_VAL_AVIONICS_FREQUENCY_ID),
            2, "Fundamental Frequency Identity");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CALC:AVI:THD? %s", FrequencyIdentityArr[fundamentalFrequencyIdentity]);
    checkErr(RsCore_QueryViReal64(instrSession, cmd, result));

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the frequency deviation (typically 480 Hz) used to modulate the carrier at typically 9960 Hz with the 30 Hz FM
/// HIFN  signal (typically 30 Hz frequency).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.
/// HIPAR modDepthDev/Returns the frequency deviation (typically 480 Hz) used to modulate the carrier at typically 9960 Hz with the 30 Hz FM
/// HIPAR signal (typically 30 Hz frequency).
ViStatus _VI_FUNC rsspecan_QueryAvionics30HzFMDeviation(ViSession instrSession,
                                                        ViReal64* modDepthDev)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_AVIONICS_VOR_FM_DEVIATION, modDepthDev),
            2, "Mod Depth/Dev");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the frequency (typically 30 Hz) of the signal that was used to modulate the carrier at typically 9960 Hz (VOR only).
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.
/// HIPAR fundamentalFrequencyIdentity/Sets the signal component, defined by its fundamental frequency.
/// HIPAR frequency/Returns the frequency (typically 30 Hz) of the signal that was used to modulate the carrier at typically 9960 Hz
ViStatus _VI_FUNC rsspecan_QueryAvionics30HzFMFrequency(ViSession instrSession,
                                                        ViReal64* frequency)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_AVIONICS_VOR_FM_FREQUENCY, frequency),
            2, "Frequency");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the relative amplitude of an AF signal's second (K2) or third (K3) harmonic for the specified signal component.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.
/// HIPAR fundamentalFrequencyIdentity/Sets the signal component, defined by its fundamental frequency.
/// HIPAR k2/Returns the relative amplitude of an AF signal's second harmonic.
/// HIPAR k3/Returns the relative amplitude of an AF signal's third harmonic.
ViStatus _VI_FUNC rsspecan_QueryAvionicsRelativeAmplitudeHarmonics(
                            ViSession instrSession,
                            ViInt32   harmonics,
                            ViInt32   fundamentalFrequencyIdentity,
                            ViReal64* relativeAmplitude)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, harmonics, RSSPECAN_VAL_AVIONICS_HARMONIC_K2, RSSPECAN_VAL_AVIONICS_HARMONIC_K3),
            2, "Harmonics");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, fundamentalFrequencyIdentity, RSSPECAN_VAL_AVIONICS_FREQUENCY_30, RSSPECAN_VAL_AVIONICS_FREQUENCY_ID),
            3, "Fundamental Frequency Identity");

    switch (harmonics)
    {
    case RSSPECAN_VAL_AVIONICS_HARMONIC_K2:
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CALC:AVI:THD:K2? %s", FrequencyIdentityArr[fundamentalFrequencyIdentity]);
        break;
    case RSSPECAN_VAL_AVIONICS_HARMONIC_K3:
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CALC:AVI:THD:K3? %s", FrequencyIdentityArr[fundamentalFrequencyIdentity]);
        break;
    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, harmonics),
                    2, "Harmonics");
    }

    checkErr(RsCore_QueryViReal64(instrSession, cmd, relativeAmplitude));

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function defines the demodulation bandwidth manually.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR demodulationBandwidth/Sets the demodulation bandwidth manually.
ViStatus _VI_FUNC rsspecan_ConfigureAvionicsDemodulationBandwidth(
                            ViSession instrSession,
                            ViBoolean state,
                            ViReal64  demodulationBandwidth)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_AVIONICS_BANDWIDTH_AUTO, state),
            2, "State");

    if (state == VI_FALSE)
    {
        viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_AVIONICS_DEMODULATION_BANDWIDTH, demodulationBandwidth),
                3, "Demodulation Bandwidth");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN Sets the resolution bandwidth for Modulation Spectrum results.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR state/Defines whether the resolution bandwidth for Modulation Spectrum results is determined automatically or not.
/// HIPAR resolutionBW/Sets the resolution bandwidth for Modulation Spectrum results.
ViStatus _VI_FUNC rsspecan_ConfigureAvionicsBandwidthResolution(
                            ViSession instrSession,
                            ViBoolean state,
                            ViReal64  resolutionBW)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_AVIONICS_RESOLUTION_BANDWIDTH_STATE, state),
            2, "State");

    if (state == VI_FALSE)
    {
        viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_AVIONICS_RESOLUTION_BANDWIDTH, resolutionBW),
                3, "Resolution BW");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the Morse code of the demodulated identifier.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.
/// HIPAR arraySize/Defines the size of array.
/// HIPAR morseCode/Returns the Morse code of the demodulated identifier.
ViStatus _VI_FUNC rsspecan_QueryAvionicsMorseCode(ViSession instrSession,
                                                  ViInt32   bufferSize,
                                                  ViChar    morseCode[])
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_GetAttributeViString(instrSession, "", RSSPECAN_ATTR_AVIONICS_MORSE_CODE,
            bufferSize, morseCode),
            3, "Morse Code");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function adds a measurement window to the display.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.
/// HIPAR window/Selects the measurement window.
/// HIPAR stringSize/Sets the allocated length of Returned Window Name string.
/// HIPAR direction/Selects the direction the window.
/// HIPAR windowType/Selects the type of result display to add.
/// HIPAR returnedWindowName/When adding a new window, the command returns its name (by default the same as its number) as a result.
ViStatus _VI_FUNC rsspecan_AddAvionicsWindow(ViSession instrSession,
                                             ViInt32   window,
                                             ViInt32   stringSize,
                                             ViInt32   direction,
                                             ViInt32   windowType,
                                             ViChar    returnedWindowName[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViChar   response[RS_MAX_MESSAGE_BUF_SIZE];
    ViChar*  p2direction;
    ViChar*  p2type;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &rsspecan_rngLayoutDirection, direction, &p2direction),
                4, "Direction");
    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &RsSpecAn_rngAvionicsWindowType, windowType, &p2type),
                5, "Window Type");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "LAY:WIND%ld:ADD? %s,%s", window, p2direction, p2type);
    checkErr(RsCore_QueryViString(instrSession, cmd, response));

    if (returnedWindowName != NULL)
    {
        RsCore_TrimString(response, RS_VAL_TRIM_WHITESPACES_AND_ALL_QUOTES);
        RsCore_CopyToUserBufferAsciiData(returnedWindowName, stringSize, response);
    }

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This funtion changes the window type of an existing window (specified by the suffix <n>) in the active measurement channel
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.
/// HIPAR window/Selects the measurement window.
/// HIPAR windowType/Selects the type of result display to add.
ViStatus _VI_FUNC rsspecan_ReplaceAvionicsWindow(ViSession instrSession,
                                                 ViString  windowName,
                                                 ViInt32   windowType)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViChar*  p2type;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &rsspecan_rngAvionicsLayoutType, windowType, &p2type),
                3, "Window Type");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "LAY:REPL '%s',%s", windowName, p2type);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the frequency range of the result display AF spectrum using start frequency and stop frequency.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.
/// HIPAR startFrequency/Defines the start frequency of the demodulated data to evaluate in the Modulation Spectrum display.
/// HIPAR stopFrequency/Defines the stop frequency of the demodulated data to evaluate in the Modulation Spectrum display.
ViStatus _VI_FUNC rsspecan_ConfigureAvionicsAFFrequencyStartStop(
                            ViSession instrSession,
                            ViReal64  startFrequency,
                            ViReal64  stopFrequency)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_AVIONICS_AF_START_FREQUENCY, startFrequency),
            2, "Start Frequency");

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_AVIONICS_AF_STOP_FREQUENCY, stopFrequency),
            3, "Stop Frequency");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the frequency range of the result display AF spectrum using the center frequency and the frequency span.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.
/// HIPAR centerFrequency/Defines the center frequency of the demodulated data to evaluate in the Modulation Spectrum display.
/// HIPAR span/Defines the span of the demodulated data to evaluate in the Modulation Spectrum display.
ViStatus _VI_FUNC rsspecan_ConfigureAvionicsAFFrequencyCenterSpan(
                            ViSession instrSession,
                            ViReal64  centerFrequency,
                            ViReal64  span)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_AVIONICS_AF_CENTER_FREQUENCY, centerFrequency),
            2, "center Frequency");

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_AVIONICS_AF_SPAN, span),
            3, "span");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function defines the span of the demodulated data to evaluate in the Modulation Spectrum display to be the maximum span possible (DBW/2).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.
ViStatus _VI_FUNC rsspecan_ConfigureAvionicsAFFullSpan(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_AVIONICS_AF_FULL_SPAN, ""));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/*****************************************************************************
 *- Callback Definition   ---------------------------------------------------*
 *****************************************************************************/

/*****************************************************************************
 *- End Of RSSpecAn K15 - Avionics (VOR/ILS) measurements application  ------*
 *****************************************************************************/
