/*****************************************************************************
 *  Rohde&Schwarz Spectrum Analyzer instrument driver
 *  K17 Multi-Carrier Group Delay
 *
 *  Original Release: July 2012
 *  By: Martin Krcmar
 *
 *  Should you have any technical questions please contact the hotline of
 *  Rohde & Schwarz Vertriebs-GmbH Rohde & Schwarz Support Center
 *
 *  e-mail: CustomerSupport@rohde-schwarz.com
 *
 *****************************************************************************/

#include "rsspecan.h"

/*****************************************************************************
 *- RSSpecAn K17 - Multi-Carrier Group Delay --------------------------------*
 *****************************************************************************/

/// HIFN  This function selects the Multi-Carrier Group Delay mode.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsspecan_MultiCarrierGroupDelayMode(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_MCGD_MODE, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures radio frequency input.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR inputCoupling/The RF input of the R&S FSW can be coupled by alternating current (AC)
/// HIPAR inputCoupling/or direct current (DC).
/// HIPAR impedance/This control selects the nominal input impedance.
/// HIPAR highpassFilter/Activates an additional high pass filter for RF input signals from
/// HIPAR highpassFilter/1...3 GHz.
/// HIPAR yigPreselector/This control turns the YIG-preselector on and off.
ViStatus _VI_FUNC rsspecan_ConfigureMultiCarrierGroupDelayInput(
                            ViSession instrSession,
                            ViInt32   inputCoupling,
                            ViInt32   impedance,
                            ViBoolean highpassFilter,
                            ViBoolean yigPreselector)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_AMPL_RF_COUPLING, inputCoupling),
            2, "Input Coupling");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_INPUT_IMPEDANCE, impedance),
            3, "Impedance");

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_AMPL_HIGHPASS_FILTER, highpassFilter),
            4, "Highpass Filter");

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_AMPL_YIG_FILTER, yigPreselector),
            5, "YIG Preselector");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function configures the reference level and offset.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR referenceLevel/The calibrated vertical position of the captured data used as a
/// HIPAR referenceLevel/reference for amplitude measurements. This is typically set to a value
/// HIPAR referenceLevel/slightly higher than the highest expected signal level.
/// HIPAR referenceLevelOffset/Specifies the offset value to the reference level. The driver uses
/// HIPAR referenceLevelOffset/this value to set the RSSPECAN_ATTR_REFERENCE_LEVEL_OFFSET attribute.
ViStatus _VI_FUNC rsspecan_ConfigureMultiCarrierGroupDelayReferenceLevel(
                            ViSession instrSession,
                            ViInt32   reserved,
                            ViReal64  referenceLevel,
                            ViReal64  referenceLevelOffset)
{
    ViStatus error = VI_SUCCESS;
             reserved;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_REFERENCE_LEVEL, referenceLevel),
            3, "Reference Level");

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_REFERENCE_LEVEL_OFFSET, referenceLevelOffset),
            4, "Reference Level Offset");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function defines the mechanical attenuation for RF input.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR autoMode/This control couples or decouples the attenuation to the reference
/// HIPAR autoMode/level.
/// HIPAR attenuationValue/This control defines the attenuation.
ViStatus _VI_FUNC rsspecan_ConfigureMultiCarrierGroupDelayMechanicalAttenuator(
                            ViSession instrSession,
                            ViBoolean autoMode,
                            ViReal64  value)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_ATTENUATION_AUTO, autoMode),
            2, "Auto Mode");

    if (!autoMode)
    {
        viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_ATTENUATION, value),
                3, "Value");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function configures the signal characteristics.
/// HIFN
/// HIFN     Note:
/// HIFN     (1) This function is available only with option B25.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR state/This contorl switches the electronic input attenuation into the signal
/// HIPAR state/path (state ON) or removes it from the signal path (state OFF).
/// HIPAR electronicAttenuatorAuto/This control specifies, that instrument automatically couples the
/// HIPAR electronicAttenuatorAuto/electronic input attenuation to the reference level and the attenuation
/// HIPAR electronicAttenuatorAuto/of the mechanical attenuator (state ON) or switches the input
/// HIPAR electronicAttenuatorAuto/attenuation to manual entry (state OFF).
/// HIPAR manualValue_dB/This control sets the manual input electronic attenuation level.
ViStatus _VI_FUNC rsspecan_ConfigureMultiCarrierGroupDelayElectronicAttenuator(
                            ViSession instrSession,
                            ViBoolean state,
                            ViBoolean electronicAttenuatorAuto,
                            ViInt32   manualValue_dB)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_AMPL_EATT_STATE, state),
            2, "State");

    if (state)
    {
        viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_AMPL_EATT_AUTO, electronicAttenuatorAuto),
                3, "Electronic Attenuator Auto");
        if (!electronicAttenuatorAuto)
        {
            viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_AMPL_EATT_VALUE, manualValue_dB),
                    4, "Manual Value dB");
        }
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function configures the preamplifier for the instrument.
/// HIFN
/// HIFN     Note(s):
/// HIFN
/// HIFN     (1) This function requires FSW-B24 option.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR state/Switches on or off the preamplifier for the instrument.
/// HIPAR preamplifierLevel/Selects the preamplification level.
ViStatus _VI_FUNC rsspecan_ConfigureMultiCarrierGroupDelayPreamplifier(
                            ViSession instrSession,
                            ViBoolean state,
                            ViInt32   preamplifierLevel)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_AMPL_PREAMPLIFIER, state),
            2, "State");
    if (state)
    {
        viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_AMPL_PREAMPLIFIER_LEVEL, preamplifierLevel),
                3, "Preamplifier Level");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function configures the Y-axis scaling.
/// HIFN
/// HIFN     Note(s):
/// HIFN
/// HIFN     (1) This function specifies settings for Magnitude display.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR range/Defines the display range of the Y axis (level axis) in the selected
/// HIPAR range/measurement window with logarithmic scaling (DISP:TRAC:Y:SPAC LOG).
/// HIPAR referencePosition/Defines the position of the reference value in the selected
/// HIPAR referencePosition/measurement window.
/// HIPAR verticalScale/Pass the vertical scale that you want the spectrum analyzer to use.
/// HIPAR yAxisGridMode/Defines the scale type of the Y axis (absolute or relative) in the
/// HIPAR yAxisGridMode/selected measurement window.
ViStatus _VI_FUNC rsspecan_ConfigureMultiCarrierGroupDelayYAxisScaling(
                            ViSession instrSession,
                            ViInt32   reserved,
                            ViReal64  range,
                            ViReal64  referencePosition,
                            ViInt32   verticalScale,
                            ViInt32   yAxisGridMode)
{
    ViStatus error = VI_SUCCESS;
             reserved;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_DISP_LOG_RANGE, range),
            3, "Range");

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_DISP_REF_POSITION, referencePosition),
            4, "Reference Position");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_VERTICAL_SCALE, verticalScale),
            5, "Vertical Scale");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_DISP_AMPLITUDE_GRID_MODE, yAxisGridMode),
            6, "Y-Axis Grid Mode");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function configures displayed range using minimum and maximum
/// HIFN  values.
/// HIFN
/// HIFN     Note(s):
/// HIFN
/// HIFN     (1) This function specifies settings for Phase and Group Delay
/// HIFN  displays.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR maximumValue/Defines the minimum value of the y-axis for the selected result
/// HIPAR maximumValue/display.
/// HIPAR miniumumValue/Defines the maximum value of the y-axis for the selected result
/// HIPAR miniumumValue/display.
ViStatus _VI_FUNC rsspecan_ConfigureMultiCarrierGroupDelayYAxisScalingMaxMin(
                            ViSession instrSession,
                            ViInt32   reserved,
                            ViReal64  maximumValue,
                            ViReal64  miniumumValue)
{
    ViStatus error = VI_SUCCESS;
             reserved;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_MCGD_VERTICAL_SCALE_MAX, maximumValue),
            3, "Maximum Value");

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_MCGD_VERTICAL_SCALE_MIN, miniumumValue),
            4, "Minimum Value");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function configures the Y-axis scaling.
/// HIFN
/// HIFN     Note(s):
/// HIFN
/// HIFN     (1) This function specifies settings for Phase and Group Delay
/// HIFN  displays.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR scaling/Defines the grid spacing on the Y-axis for all diagrams, where
/// HIPAR scaling/possible.
/// HIPAR referencePosition/Defines the position of the reference value in the selected
/// HIPAR referencePosition/measurement window.
/// HIPAR referenceValue/Defines the power value assigned to the reference position in the
/// HIPAR referenceValue/grid.
ViStatus _VI_FUNC rsspecan_ConfigureMultiCarrierGroupDelayYAxisScalingReferencePerDivision(
                            ViSession instrSession,
                            ViInt32   reserved,
                            ViReal64  scaling,
                            ViReal64  referencePosition,
                            ViReal64  referenceValue)
{
    ViStatus error = VI_SUCCESS;
             reserved;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_DISP_Y_AXIS_SCALING, scaling),
            2, "Scaling");

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_DISP_REF_POSITION, referencePosition),
            3, "Reference Position");

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_DISP_REF_VALUE, referenceValue),
            4, "Reference Value");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function selects the unit for angles (for PM display).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR unit/This control selects the unit for angles (for PM display).
ViStatus _VI_FUNC rsspecan_ConfigureMultiCarrierGroupDelayPhaseUnit(
                            ViSession instrSession,
                            ViInt32   unit)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_MCGD_RESULT_PHASE_UNIT, unit),
            2, "Unit");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function configures the carrier settings.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR numberOfCarriers/Sets the number of carriers.
/// HIPAR carrierSpacing/Sets the carrier spacing in Hz between the multiple carriers.
ViStatus _VI_FUNC rsspecan_ConfigureMultiCarrierGroupDelaySignalDescription(
                            ViSession instrSession,
                            ViInt32   numberOfCarriers,
                            ViReal64  carrierSpacing)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_MCGD_CARRIER_COUNT, numberOfCarriers),
            2, "Number Of Carriers");

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_MCGD_CARRIER_SPACING, carrierSpacing),
            3, "Carrier Spacing");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function configures the measurement time.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR mode/Enables or disables automatic measurement time selection.
/// HIPAR measurementTime/Sets the measurement time.
ViStatus _VI_FUNC rsspecan_ConfigureMultiCarrierGroupDelayMeasurementTime(
                            ViSession instrSession,
                            ViBoolean mode,
                            ViReal64  measurementTime)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_MCGD_MEASUREMENT_TIME_AUTO, mode),
            2, "Mode");

    if (mode == VI_FALSE)
    {
        viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_MCGD_MEASUREMENT_TIME, measurementTime),
                3, "Measurement Time");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function configures the measurement parameters for group delay
/// HIFN  calculation.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR sweepCount/The number of sweeps to take.
/// HIPAR groupDelay/Defines how the group delay is calculated.
/// HIPAR carrierOffset/Defines the offset of the carrier compared to the center frequency so
/// HIPAR carrierOffset/the offset can be compensated in the display.
ViStatus _VI_FUNC rsspecan_ConfigureMultiCarrierGroupDelaySettings(
                            ViSession instrSession,
                            ViInt32   sweepCount,
                            ViInt32   groupDelay,
                            ViInt32   carrierOffset)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_NUMBER_OF_SWEEPS, sweepCount),
            2, "Sweep Count");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_MCGD_MODE_ABS_REL, groupDelay),
            3, "Group Delay");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_MCGD_CARRIER_OFFSET_MODE, carrierOffset),
            4, "Carrier Offset");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Determines the reference used for relative group delay measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR referenceType/Selects the reference used for relative group delay measurement.
ViStatus _VI_FUNC rsspecan_ConfigureMultiCarrierGroupDelayReference(
                            ViSession instrSession,
                            ViInt32   referenceType)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_MCGD_CARRIER_REFERENCE_TYPE, referenceType),
            2, "Reference Type");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Determines the frequency of the reference carrier used for relative group delay calculation.
/// HIFN  The group delay measured at this frequency is used as a reference.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR referenceCarrierFrequency/Sets the frequency of the reference carrier used for relative group delay calculation.
ViStatus _VI_FUNC rsspecan_ConfigureMultiCarrierGroupDelayReferenceFrequency(
                            ViSession instrSession,
                            ViReal64  referenceCarrierFrequency)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_MCGD_CARRIER_REFERENCE_FREQUENCY, referenceCarrierFrequency),
            2, "Reference Carrier Frequency");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function configures the group delay smooting.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR trace/Selects the trace.
/// HIPAR smoothing/Turns trace smoothing for a particular trace on and off.
/// HIPAR apertureValue/This control defines the degree (aperture) of the trace
/// HIPAR apertureValue/smoothing
ViStatus _VI_FUNC rsspecan_ConfigureMultiCarrierGroupDelaySmoothing(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   trace,
                            ViBoolean smoothing,
                            ViInt32   value)
{
    ViStatus      error = VI_SUCCESS;
    ViChar        repCap[RS_REPCAP_BUF_SIZE] = "",
    rep_win[10]   = "",
    rep_trace[10] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession,
                    RSSPECAN_ATTR_MCGD_DISP_TRACE_SMO,
                    "Window",
                    window,
                    10,
                    rep_win), 2, "Window");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession,
                    RSSPECAN_ATTR_MCGD_DISP_TRACE_SMO,
                    "Trace",
                    trace - 1,
                    10,
                    rep_trace), 3, "Trace");
    snprintf(repCap, RS_REPCAP_BUF_SIZE, "%s,%s", rep_win, rep_trace);
    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, repCap, RSSPECAN_ATTR_MCGD_DISP_TRACE_SMO, smoothing),
            4, "Smoothing");

    if (smoothing == VI_TRUE)
        viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_MCGD_DISP_TRACE_SMO_APER, value),
                5, "Value");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function queries the offset in Hz of the carriers compared to the
/// HIFN  center frequency.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR carrierOffset/Queries the offset in Hz of the carriers compared to the center
/// HIPAR carrierOffset/frequency.
ViStatus _VI_FUNC rsspecan_QueryMultiCarrierGroupDelayCarrierOffset(
                            ViSession instrSession,
                            ViReal64* carrierOffset)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_MCGD_CARRIER_OFFSET, carrierOffset),
            2, "Carrier Offset");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets/queries the carrier estimation mode. This defines how the carrier
/// HIFN  frequencies of the captured multi-carrier signal are estimated and how
/// HIFN  differences compared to the multi-carrier signal description are being
/// HIFN  compensated.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR estimation/Sets/queries the carrier estimation mode. This defines how the carrier
/// HIPAR estimation/frequencies of the captured multi-carrier signal are estimated and how
/// HIPAR estimation/differences compared to the multi-carrier signal description are being
/// HIPAR estimation/compensated.
ViStatus _VI_FUNC rsspecan_ConfigureMultiCarrierGroupDelayModeEstimation(
                            ViSession instrSession,
                            ViInt32   estimation)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_MCGD_MODE_ESTIMATION, estimation),
            2, "Estimation");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Defines the maximum clock offset the frequency estimation algorithm of
/// HIFN  the R&S FSW MCGD application is able to handle. Larger values for this
/// HIFN  parameter result in slower measurement speed. For satellite in-orbit
/// HIFN  measurements, set this parameter according to the maximum relative
/// HIFN  velocity vrel between satellite and ground station.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR offset/Defines the maximum clock offset the frequency estimation algorithm of
/// HIPAR offset/the R&S FSW MCGD application is able to handle. Larger values for this
/// HIPAR offset/parameter result in slower measurement speed. For satellite in-orbit
/// HIPAR offset/measurements, set this parameter according to the maximum relative
/// HIPAR offset/velocity vrel between satellite and ground station.
ViStatus _VI_FUNC rsspecan_ConfigureMultiCarrierGroupDelayMaxClockOffset(
                            ViSession instrSession,
                            ViReal64  offset)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_MCGD_MAX_CLOCK_OFFSET, offset),
            2, "Offset");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  If enabled, the raw, unprocessed phase data is displayed in the
/// HIFN  'Phase' and 'Phase Difference' result displays.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR rawPhase/If enabled, the raw, unprocessed phase data is displayed in the
/// HIPAR rawPhase/'Phase' and 'Phase Difference' result displays.
ViStatus _VI_FUNC rsspecan_ConfigureMultiCarrierGroupRawPhaseEnabled(
                            ViSession instrSession,
                            ViBoolean rawPhase)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, NULL, RSSPECAN_ATTR_MCGD_RAW_PHASE_ENABLED, rawPhase),
            2, "Raw Phase");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Enables or disables the Large Offset Compensation.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR enabled/Enables or disables the Large Offset Compensation.
ViStatus _VI_FUNC rsspecan_ConfigureMultiCarrierGroupDelayLargeOffsetCompensationEnabled(
                            ViSession instrSession,
                            ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_MCGD_LARGE_OFFSET_COMPENSATION_ENABLED, enabled),
            2, "Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Improves the measurement speed by decimating the number of carriers
/// HIFN  used for the frequency estimation synchronization as a speed vs
/// HIFN  accuracy tradeoff in carrier estimation mode 'All Carriers'. For
/// HIFN  example, a value of '1000' configures the synchronization to only use
/// HIFN  each 1000th carrier for frequency estimation and significantly speeds
/// HIFN  up the measurement for scenarios with large number of carriers (100k
/// HIFN  carriers scenarios) while still providing good measurement accuracy. A
/// HIFN  value of '1' provides the best possible measurement accuracy. The value
/// HIFN  is clipped internally to 2 carriers if set larger then the available
/// HIFN  number of carriers configured in the multi-carrier signal description.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR decimationFactor/Improves the measurement speed by decimating the number of carriers
/// HIPAR decimationFactor/used for the frequency estimation synchronization as a speed vs
/// HIPAR decimationFactor/accuracy tradeoff in carrier estimation mode 'All Carriers'. For
/// HIPAR decimationFactor/example, a value of '1000' configures the synchronization to only use
/// HIPAR decimationFactor/each 1000th carrier for frequency estimation and significantly speeds
/// HIPAR decimationFactor/up the measurement for scenarios with large number of carriers (100k
/// HIPAR decimationFactor/carriers scenarios) while still providing good measurement accuracy. A
/// HIPAR decimationFactor/value of '1' provides the best possible measurement accuracy. The value
/// HIPAR decimationFactor/is clipped internally to 2 carriers if set larger then the available
/// HIPAR decimationFactor/number of carriers configured in the multi-carrier signal description.
ViStatus _VI_FUNC rsspecan_ConfigureMultiCarrierGroupDelayCarrierDecimationFactor(
                            ViSession instrSession,
                            ViInt32   decimationFactor)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_MCGD_CARRIER_DECIMATION_FACTOR, decimationFactor),
            2, "Decimation Factor");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Frequency spread of the captured multi-carrier signal compared to the
/// HIFN  setting given by multi-carrier signal description. This frequency
/// HIFN  spread is estimated and compensated when using carrier estimation mode
/// HIFN  'All Carriers'. The clock offset reading shows the influence of Doppler
/// HIFN  shift in satellite communication measurements.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR clockOffset/Frequency spread of the captured multi-carrier signal compared to the
/// HIPAR clockOffset/setting given by multi-carrier signal description. This frequency
/// HIPAR clockOffset/spread is estimated and compensated when using carrier estimation mode
/// HIPAR clockOffset/'All Carriers'. The clock offset reading shows the influence of Doppler
/// HIPAR clockOffset/shift in satellite communication measurements.
ViStatus _VI_FUNC rsspecan_QueryMultiCarrierGroupDelayClockOffset(
                            ViSession instrSession,
                            ViReal64* clockOffset)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_MCGD_CLOCK_OFFSET, clockOffset),
            2, "Clock Offset");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function adds a window to the display.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR stringSize/This control sets the allocated length of Returned Window Name string.
/// HIPAR referenceWindowName/String containing the number or name of the window the new window is
/// HIPAR referenceWindowName/inserted next to. In the default state, the name of the window is its
/// HIPAR referenceWindowName/index.
/// HIPAR direction/Direction the window is added relative to the window set with "Reference Window
/// HIPAR direction/Name".
/// HIPAR windowType/Type of result display you want to add.
/// HIPAR returnedWindowName/When adding a new window, the function returns its name (by
ViStatus _VI_FUNC rsspecan_AddMultiCarrierGroupDelayWindow(ViSession instrSession,
                                                           ViInt32   stringSize,
                                                           ViString  referenceWindowName,
                                                           ViInt32   direction,
                                                           ViInt32   windowType,
                                                           ViChar    returnedWindowName[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViChar   response[RS_MAX_MESSAGE_BUF_SIZE];
    ViChar*  p2direction;
    ViChar*  p2type;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &rsspecan_rngLayoutDirection, direction, &p2direction),
                4, "Direction");
    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &RsSpecAn_rngMCGDCarrierReplaceWindow, windowType, &p2type),
                5, "Window Type");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "LAY:ADD? '%s',%s,%s", referenceWindowName, p2direction, p2type);
    checkErr(RsCore_QueryViString(instrSession, cmd, response));

    if (returnedWindowName != NULL)
    {
        RsCore_TrimString(response, RS_VAL_TRIM_WHITESPACES_AND_ALL_QUOTES);
        RsCore_CopyToUserBufferAsciiData(returnedWindowName, stringSize, response);
    }

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function changes the window type of an existing window
/// HIFN  (specified by the suffixn<n>) in the active measurement channel.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR windowType/Selects the type of measurement window you want to replace
/// HIPAR another one with.Type of result display you want to add.
ViStatus _VI_FUNC rsspecan_ReplaceMultiCarrierGroupDelayWindow(
                            ViSession instrSession,
                            ViString  windowName,
                            ViInt32   windowType)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViChar*  p2type;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &RsSpecAn_rngMCGDCarrierReplaceWindow, windowType, &p2type),
                3, "Window Type");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "LAY:REPL '%s',%s", windowName, p2type);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Contains settings for connecting an external signal generator.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR connectToGenerator/Connects or disconnects the signal generator specified by
/// HIPAR connectToGenerator/RSSPECAN_ATTR_GENERATOR_IP_ADDRESS. The IP address must be specified
/// HIPAR connectToGenerator/before you use this command.
/// HIPAR generatorIPAddress/This control defines the IP address of the connected signal generator.
/// HIPAR rfOutput/This control enables or disables RF output on the connected generator.
/// HIPAR connectionStatus/Queries the state of the connected signal generator.
ViStatus _VI_FUNC rsspecan_ConfigureMCGDGeneratorConnectionSettings(
                            ViSession instrSession,
                            ViBoolean connectToGenerator,
                            ViString  generatorIPAddress,
                            ViBoolean rfOutput)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_CONNECT_TO_GENERATOR, connectToGenerator),
            2, "Connect To Generator");

    viCheckParm(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_GENERATOR_IP_ADDRESS, generatorIPAddress),
            3, "Connect IP Address");

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_GENERATOR_RF_OUTPUT, rfOutput),
            4, "RF Output");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the state of the connected signal generator.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR bufferSize/Size of the buffer passed to the Connection State parameter
/// HIPAR connectionStatus/Returns the state of the connected signal generator.
/// HIPAR returnedBytes/The number of bytes returned from the instrument. If Returned Bytes >
/// HIPAR returnedBytes/Buffer Size, some data was lost. Consider resizing the buffer and
/// HIPAR returnedBytes/calling again.RSSPECAN_ATTR_GENERATOR_CONNECTION_STATUS
ViStatus _VI_FUNC rsspecan_QueryMCGDGeneratorConnectionStatus(
                            ViSession instrSession,
                            ViInt32   bufferSize,
                            ViChar    connectionStatus[],
                            ViInt32*  returnedBytes)
{
    ViStatus error = VI_SUCCESS;
    ViChar*  cmd = NULL;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_BuildAttrCommandString(instrSession,
        &g_RSSPECAN_ATTR_GENERATOR_CONNECTION_STATUS,
        NULL, &cmd));

    checkErr(RsCore_QueryViStringUnknownLengthToUserBuffer(instrSession, cmd, bufferSize,
        connectionStatus, returnedBytes));

Error:
    if (cmd)
        free(cmd);
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the state of the connected signal generator and its
/// HIFN  availability for the R&S FSW MCGD application.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR mcgdConnectionStatus/Queries the state of the connected signal generator and its
/// HIPAR mcgdConnectionStatus/availability for the R&S FSW MCGD application.
ViStatus _VI_FUNC rsspecan_QueryMCGDConnectionStatus(ViSession instrSession,
                                                     ViInt32*  mcgdConnectionStatus)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_GetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_MCGD_CONNECTION_STATUS, mcgdConnectionStatus),
            2, "MCGD Connection Status");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Contains controls to enable or disable the application of settings
/// HIFN  from the R&S FSW to the generator.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR carrierCount/If activated, any changes to the number of carriers on the R&S FSW are
/// HIPAR carrierCount/automatically also applied to the connected signal generator.
/// HIPAR carrierCount/Initially, the value defined by [SENSe:]CARRier:COUNt is applied.
/// HIPAR centerFrequency/If activated, any changes to the center frequency on the R&S FSW are
/// HIPAR centerFrequency/automatically also applied to the connected signal generator.
/// HIPAR centerFrequency/Initially, the value defined by [SENSe:]FREQuency:CENTer is applied.
/// HIPAR carrierSpacing/If activated, any changes to the carrier spacting on the R&S FSW are
/// HIPAR carrierSpacing/automatically also applied to the connected signal generator.
/// HIPAR carrierSpacing/Initially, the value defined by [SENSe:]CARRier:SPACing is applied.
ViStatus _VI_FUNC rsspecan_ConfigureMCGDSettingsAppliedToGenerator(
                            ViSession instrSession,
                            ViBoolean carrierCount,
                            ViBoolean centerFrequency,
                            ViBoolean carrierSpacing)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_CARRIER_COUNT_TO_GENERATOR_ENABLED, carrierCount),
            2, "Carrier Count");

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_CENTER_FREQ_TO_GENERATOR_ENABLED, centerFrequency),
            3, "Center Frequency");

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_SPACING_TO_GENERATOR_ENABLED, carrierSpacing),
            4, "Carrier Spacing");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Applies all generator setup settings to the connected signal generator
/// HIFN  once.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsspecan_UpdateMCGDGeneratorSettings(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_GENERATOR_UPDATE_SETTINGS, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Contains controls to configure DUT settings.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR dutGain/A gain due to the DUT is taken into consideration when determining the
/// HIPAR dutGain/reference level on the R&S FSW and the signal level on the generator
/// HIPAR dutGain/during the reference calibration.
/// HIPAR dutLimitState/If activated, the generator does not exceed the maximum input power
/// HIPAR dutLimitState/(peak envelope power, 'PEP') that is currently allowed by the DUT. To
/// HIPAR dutLimitState/query or define the current PEP limit value, use
/// HIPAR dutLimitState/RSSPECAN_ATTR_GENERATOR_DUT_LIMIT.
/// HIPAR dutLimit/Sets or queries the maximum input power (peak envelope power, 'PEP')
/// HIPAR dutLimit/that is currently allowed by the DUT and that is specified on the
/// HIPAR dutLimit/generator. The generator output does not exceed this value.
ViStatus _VI_FUNC rsspecan_ConfigureMCGDGeneratorDUTSettings(ViSession instrSession,
                                                             ViReal64  dutGain,
                                                             ViBoolean dutLimitState,
                                                             ViReal64  dutLimit)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_GENERATOR_DUT_GAIN, dutGain),
            2, "DUT Gain");

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_GENERATOR_DUT_LIMIT_ENABLED, dutLimitState),
            3, "DUT Limit State");

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_GENERATOR_DUT_LIMIT, dutLimit),
            4, "DUT Limit");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries all currently defined generator setup settings on the
/// HIFN  connected signal generator and applies the settings to the R&S FSW.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsspecan_TransferMCGDAllSettingsFromGenerator(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_MCGD_SETTINGS, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Contains controls for configuration of the signal generator by the R&S
/// HIFN  FSW MCGD.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR controlState/Activates or deactivates control of the signal generator by the R&S
/// HIPAR controlState/FSW.
ViStatus _VI_FUNC rsspecan_ConfigureMCGDGeneratorControlSettings(
                            ViSession instrSession,
                            ViBoolean controlState)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_GENERATOR_CONTROL_ENABLED, controlState),
            2, "Control State");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the overall status of the generator control settings.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR overallSettingsStatus/Queries the overall status of the generator control settings.
ViStatus _VI_FUNC rsspecan_QueryMCGDControlSettingsOverallStatus(
                            ViSession instrSession,
                            ViInt32*  overallSettingsStatus)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_GetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_GENERATOR_SETTINGS_OVERALL_STATUS, overallSettingsStatus),
            3, "Overall Setting Status");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Contains settings for MCGD Level.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR referenceLevelState/Determines the behavior of the reference level.
/// HIPAR generatorAutoLevel/If enabled, the R&S FSW automatically controls the signal level
/// HIPAR generatorAutoLevel/provided by the signal generator as input to the R&S FSW. Initially,the
/// HIPAR generatorAutoLevel/value defined by RSSPECAN_ATTR_GENERATOR_LEVEL is applied. Note that
/// HIPAR generatorAutoLevel/the reference level on the R&S FSW is also affected by the signal
/// HIPAR generatorAutoLevel/level.
/// HIPAR generatorLevel/Sets the specified value on the connected signal generator or queries
/// HIPAR generatorLevel/which value is used.
ViStatus _VI_FUNC rsspecan_ConfigureMCGDGeneratorLevelSettings(
                            ViSession instrSession,
                            ViBoolean referenceLevelState,
                            ViBoolean generatorAutoLevel,
                            ViReal64  generatorLevel)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_GENERATOR_REFERENCE_LEVEL_ENABLED, referenceLevelState),
            2, "Reference Level State");

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_GENERATOR_LEVEL_CONTROL_ENABLED, generatorAutoLevel),
            3, "Generator Auto Level");

    if (generatorAutoLevel == VI_FALSE)
        viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_GENERATOR_LEVEL, generatorLevel),
                4, "Generator Level");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines a time period between the time a parameter changes on the generator and
/// HIFN  the start of the next measurement. The R&S FSW automatically waits for the defined
/// HIFN  time period whenever one of the relevant generator settings has been changed.
/// HIFN
/// HIFN  Defining a delay time is especially useful for measurements that automatically
/// HIFN  change generator settings (for example the parameter sweep). The delay time
/// HIFN  considers the settling time of the generator's hardware components between individual
/// HIFN  measurements.
/// HIFN  connector, if available.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR settlingDelay/Defines settling delay.
ViStatus _VI_FUNC rsspecan_ConfigureMCGDGeneratorSettlingDelay(
                            ViSession instrSession,
                            ViReal64  settlingDelay)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_GENERATOR_SETTLING_DELAY, settlingDelay),
            2, "Settling Delay");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Activates or deactivates automated measurement of a frequency
/// HIFN  translating DUT using generator control by the R&S FSW.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR enabled/Activates or deactivates automated measurement of a frequency
/// HIPAR enabled/translating DUT using generator control by the R&S FSW.
ViStatus _VI_FUNC rsspecan_ConfigureMultiCarrierGroupDelayAFTMEnabled(
                            ViSession instrSession,
                            ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_AFTM_ENABLED, enabled),
            2, "Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Contains frequency settings for AFTM.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR analyzerFrequency/Sets or queries the analyzer frequency for the frequency translating
/// HIPAR analyzerFrequency/measurement. During the group delay measurement, this value corresponds
/// HIPAR analyzerFrequency/to the center frequency of the R&S FSW MCGD application.
/// HIPAR generatorFrequency/Determines the signal generator frequency. Any changes here are
/// HIPAR generatorFrequency/automatically applied to the connected generator.
/// HIPAR frequencyOffset/Defines a fixed offset to be applied to the generator frequency as an
/// HIPAR frequencyOffset/effect of the DUT.
/// HIPAR numerator/Defines the numerator of the frequency translating factor of the DUT.
/// HIPAR denominator/Defines the denominator of the frequency translating factor of the
/// HIPAR denominator/DUT.
ViStatus _VI_FUNC rsspecan_ConfigureMultiCarrierGroupDelayAFTMGeneratorFrequency(
                            ViSession instrSession,
                            ViReal64  analyzerFrequency,
                            ViReal64  generatorFrequency,
                            ViReal64  frequencyOffset,
                            ViReal64  numerator,
                            ViReal64  denominator)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_AFTM_ANALYZER_FREQUENCY, analyzerFrequency),
            2, "Analyzer Frequency");

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_AFTM_GENERATOR_FREQUENCY, generatorFrequency),
            3, "Generator Frequency");

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_AFTM_FREQUENCY_OFFSET, frequencyOffset),
            4, "Frequency Offset");

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_AFTM_NUMERATOR, numerator),
            5, "Numerator");

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_AFTM_DENOMINATOR, denominator),
            6, "Denominator");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the overall status of the automated frequency translating
/// HIFN  measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR aftmOverallStatus/Queries the overall status of the automated frequency translating
/// HIPAR aftmOverallStatus/measurement.
ViStatus _VI_FUNC rsspecan_QueryMultiCarrierGroupDelayAFTMOverallStatus(
                            ViSession instrSession,
                            ViInt32*  aftmOverallStatus)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_GetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_AFTM_OVERALL_STATUS, aftmOverallStatus),
            2, "AFTM Overall Status");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function starts a new sweep to perform a calibration measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsspecan_InitiateMultiCarrierGroupDelayCalibration(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_MCGD_INITIATE_CALIBRATION, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function queries the calibration status of the Multi Carrier
/// HIFN  Group Delay application.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR calibrationStatus/Queries the calibration status of the Multi Carrier Group Delay
/// HIPAR calibrationStatus/application.
ViStatus _VI_FUNC rsspecan_QueryMultiCarrierGroupDelayCalibrationStatus(
                            ViSession instrSession,
                            ViInt32*  calibrationStatus)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_GetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_MCGD_CALIBRATION_STATUS, calibrationStatus),
            2, "Calibration Status");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function stores the current calibration data to the selected
/// HIFN  file.
/// HIFN
/// HIFN     Note(s):
/// HIFN
/// HIFN     (1) The change the decimal separator use attribute
/// HIFN  RSSPECAN_ATTR_FILE_DEC_SEPARATOR or function
/// HIFN  rsspecan_FileDecimalSeparator.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR fileName/Defines the path and file name of the .csv file that will contain the
/// HIPAR fileName/calibration data.
ViStatus _VI_FUNC rsspecan_StoreMultiCarrierGroupDelayCalibrationData(
                            ViSession instrSession,
                            ViString  fileName)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_MCGD_STORE_CALIBRATION_DATA, fileName),
            2, "FileName");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function loads the calibration data stored in the selected file
/// HIFN  and replaces the current data.
/// HIFN
/// HIFN     Note(s):
/// HIFN
/// HIFN     (1) The change the decimal separator use attribute
/// HIFN  RSSPECAN_ATTR_FILE_DEC_SEPARATOR or function
/// HIFN  rsspecan_FileDecimalSeparator.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR fileName/Defines the path and file name of the .csv file that will contain the
/// HIPAR fileName/calibration data.
ViStatus _VI_FUNC rsspecan_LoadMultiCarrierGroupDelayCalibrationData(
                            ViSession instrSession,
                            ViString  fileName)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_MCGD_LOAD_CALIBRATION_DATA, fileName),
            2, "FileName");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function stores the currently displayed measurement results (i.e.
/// HIFN  all traces in all windows) to the selected file (.csv format).
/// HIFN
/// HIFN     Note(s):
/// HIFN
/// HIFN     (1) The change the decimal separator use attribute
/// HIFN  RSSPECAN_ATTR_FILE_DEC_SEPARATOR or function
/// HIFN  rsspecan_FileDecimalSeparator.
/// HIFN
/// HIFN     (2) To store the results of an individual trace in ASCII format
/// HIFN  (.txt), use function rsspecan_StoreMultiCarrierGroupDelayTraceToFile.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR fileName/Defines the path and file name of the .csv file that will contain the
/// HIPAR fileName/calibration data.
ViStatus _VI_FUNC rsspecan_StoreMultiCarrierGroupDelayMeasurementResults(
                            ViSession instrSession,
                            ViString  fileName)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_MCGD_STORE_MEASUREMENT_RESULTS, fileName),
            2, "FileName");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function stores the selected trace in the measurement window
/// HIFN  indicated by window in a file with ASCII format.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR trace/Selects the trace.
/// HIPAR fileName/Specifies full path of the file, where trace data will be stored.
ViStatus _VI_FUNC rsspecan_StoreMultiCarrierGroupDelayTraceToFile(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   trace,
                            ViString  fileName)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "MMEM:STOR%ld:TRAC %ld,'%s'", window, trace, fileName);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function queries the trace length.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR trace/Selects the trace.
/// HIPAR numberOfTracePoints/Queries the trace length.
ViStatus _VI_FUNC rsspecan_QueryMultiCarrierGroupDelayNumberOfTracePoints(
                            ViSession instrSession,
                            ViInt32   reserved,
                            ViInt32   trace,
                            ViInt32*  numberOfTracePoints)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
             reserved;

    checkErr(RsCore_LockSession(instrSession));

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "TR%ld", trace);

    viCheckParm(rsspecan_GetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_MCGD_TRACE_POINTS, numberOfTracePoints),
            4, "Number Of Trace Points");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function queries the measurement bandwidth in Hz.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR bandwidth/Queries the measurement bandwidth in Hz.
ViStatus _VI_FUNC rsspecan_QueryMultiCarrierGroupDelayMeasurementBandwidth(
                            ViSession instrSession,
                            ViReal64* bandwidth)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_MCGD_BANDWIDTH, bandwidth),
            2, "Bbandwidth");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function returns the record length determined by the current
/// HIFN  measurement settings.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR recordLength/Returns the record length determined by the current measurement
/// HIPAR recordLength/settings.
ViStatus _VI_FUNC rsspecan_QueryMultiCarrierGroupDelayRecordLength(
                            ViSession instrSession,
                            ViInt32*  recordLength)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_GetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_MCGD_RECORD_LENGTH, recordLength),
            2, "Record Length");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function returns the sample rate set up for current measurement
/// HIFN  settings.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR sampleRate/Returns the sample rate set up for current measurement settings.
ViStatus _VI_FUNC rsspecan_QueryMultiCarrierGroupDelaySampleRate(
                            ViSession instrSession,
                            ViReal64* sampleRate)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_MCGD_SAMPLE_RATE, sampleRate),
            2, "Sample Rate");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/*****************************************************************************
*- Callback Definition   ---------------------------------------------------*
*****************************************************************************/

/*****************************************************************************
 *- End Of RSSpecAn K17 Multi-Carrier Group Delay  --------------------------*
 *****************************************************************************/
