/*****************************************************************************
 *  Rohde&Schwarz Spectrum Analyzer instrument driver
 *  K192 - DOCSIS
 *
 *  Original Release: April 2015
 *  By: Marketa Migdalova
 *
 *  Should you have any technical questions please contact the hotline of
 *  Rohde & Schwarz Vertriebs-GmbH Rohde & Schwarz Support Center
 *
 *  e-mail: CustomerSupport@rohde-schwarz.com
 *
 *****************************************************************************/

#include "rsspecan.h"

static ViString profileListArr[] = {
    "NONE", "A", "B", "C", "D", "E", "F", "G", "H", "I", "J", "K", "L", "M", "N", "O", "P", NULL
};
static ViString modulationArr[] = {
    "BPSK", "QPSK", "QAM16", "QAM64", "QAM128", "QAM256", "QAM512", "QAM1024", "QAM2048", "QAM4096", "QAM8192",
    "QAM16384", "NONE", "MIXED", NULL
};
static ViString objectInformationArr[] = {
    "PIL", "PLCP", "PLCD", "EXCL", "NCPA", "PROFA", "PROFB", "PROFC", "PROFD", "PROFE", "PROFF", "PROFG", "PROFH",
    "PROFI", "PROFJ", "PROFK", "PROFL", "PROFM", "PROFN", "PROFO", "PROFP",NULL
};
static ViString offOnArr[] = {"OFF", "ON", NULL};
static ViString DOCSISModulationArr[] = {
    "QPSK", "QAM16", "QAM64", "QAM128", "QAM256", "QAM512", "QAM1024", "QAM2048", "QAM4096", "QAM8192", "QAM16384",
    "ZEROBIT", NULL
};
static ViString gsObjectInformationTypeArr[] = {
    "PLCD", "NCPCWA", "NCPCWB", "NCPCWC", "NCPCWD", "NCPCWE", "NCPCWF", "NCPCWG",
    "NCPCWH", "NCPCWI", "NCPCWJ", "NCPCWK", "NCPCWL", "NCPCWM", "NCPCWN",
    "NCPCWO", "NCPCWP", "NCPC24", "NCPN", "CWA", "CWB", "CWC", "CWD",
    "CWE", "CWF", "CWG", "CWH", "CWI", "CWJ", "CWK", "CWL", "CWM", "CWN", "CWO", "CWP", NULL
};
static ViString gsSCDetailedObjectInformationTypeArr[] = {
    "PIL", "PLCP", "PLCD", "EXCL", "PLCD", "NCPCWA", "NCPCWB", "NCPCWC", "NCPCWD", "NCPCWE", "NCPCWF", "NCPCWG",
    "NCPCWH", "NCPCWI", "NCPCWJ", "NCPCWK", "NCPCWL", "NCPCWM", "NCPCWN",
    "NCPCWO", "NCPCWP", "NCPC24", "NCPN", "CWA", "CWB", "CWC", "CWD",
    "CWE", "CWF", "CWG", "CWH", "CWI", "CWJ", "CWK", "CWL", "CWM", "CWN", "CWO", "CWP",
    "CW", "CPIL", "SPIL", "CONP",NULL
};

/// HIFN  This function enables the DOCSIS 3.1  mode.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsspecan_SetDOCSISMode(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_DOCSIS_MODE, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function defines the DOCSIS 3.1 OFDM Channel Description.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR ofdmSpectrumLocation/Specifies the center frequency in Hz of the subcarrier 0 of the OFDM
/// HIPAR ofdmSpectrumLocation/channel (fsc0), which defines the beginning of the OFDM spectrum. The
/// HIPAR ofdmSpectrumLocation/default value is derived from the current center frequency.
/// HIPAR nFFT/Defines the length of the FFT duration, which corresponds to the
/// HIPAR nFFT/number of phsyical subcarriers.
/// HIPAR cyclicPrefixCP/Defines the cyclic prefix, which determines where the useful data
/// HIPAR cyclicPrefixCP/starts and allows the application to detect delay spreads during
/// HIPAR cyclicPrefixCP/transmission. The longer the delay spread, the longer the CP must be.
/// HIPAR rollOff/Defines the roll-off period for the Tukey raised-cosine window which
/// HIPAR rollOff/is applied at the beginning (and end) of an OFDM symbol.
/// HIPAR timeInterleavingDepth/Defines the maximum number of delay lines used for time interleaving.
/// HIPAR timeInterleavingDepth/The required depth depends on the symbol duration, that is the
/// HIPAR timeInterleavingDepth/subcarrier spacing.
/// HIPAR npcModulation/Defines the modulation used by the Next Codeword Pointer (NCP).
ViStatus _VI_FUNC rsspecan_ConfigureDOCSISOFDMChannelDescription(
                            ViSession instrSession,
                            ViReal64  ofdmSpectrumLocation,
                            ViInt32   nFFT,
                            ViInt32   cyclicPrefixCP,
                            ViInt32   rollOff,
                            ViInt32   timeInterleavingDepth,
                            ViInt32   npcModulation)
{
    ViStatus error = VI_SUCCESS;
    ViInt32* direction = 0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_DOCSIS_OFDM_SPECTRUM_LOCATION, ofdmSpectrumLocation),
            2, "OFDM Spectrum Location");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_DOCSIS_N_FFT, nFFT),
            3, "NFFT");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_DOCSIS_CYCLIC_PREFIX_CP, cyclicPrefixCP),
            4, "Cyclic Prefix CP");
    if (cyclicPrefixCP != RSSPECAN_VAL_DOCSIS_CYCLIC_PREFIX_CP_AUTO)
    {
        viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_DOCSIS_ROLL_OFF, rollOff),
                5, "Roll Off");
    }

    checkErr(rsspecan_GetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_DOCSIS_STREAM_DIRECTION, direction));

    if (direction == RSSPECAN_DOCSIS_STREAM_DIRECTION_DS)
    {
        viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_DOCSIS_TIME_INTERLEAVING_DEPTH, timeInterleavingDepth),
                6, "Time Interleaving Depth");

        viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_DOCSIS_NCP_MODULATION, npcModulation),
                7, "NCP Modulation");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN Sets the center frequency in Hz of the subcarrier 0 of the OFDM channel (fsc0), which defines the beginning of the OFDM spectrum.
/// HIFN band starts and stops.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR OFDMSpectrumLocation/Sets the center frequency in Hz of the subcarrier 0 of the OFDM channel (fsc0), which defines the beginning of the OFDM spectrum.
ViStatus _VI_FUNC rsspecan_ConfigureDOCSISOFDMSpectrumLocation(
                            ViSession instrSession,
                            ViReal64  OFDMSpectrumLocation)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_DOCSIS_OFDM_US_SPECTRUM_LOCATION, OFDMSpectrumLocation),
            2, "OFDM Spectrum Location");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function defines the direction of the signal stream to be
/// HIFN  analyzed. Various configuration parameters for the DOCSIS 3.1
/// HIFN  measurement depend on the stream direction.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSSPECAN_ATTR_DOCSIS_STREAM_DIRECTION
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     CONFigure:SDIRection
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR streamDirection/Defines the direction of the signal stream to be analyzed. Various
/// HIPAR streamDirection/configuration parameters for the DOCSIS 3.1 measurement depend on the
/// HIPAR streamDirection/stream direction.
ViStatus _VI_FUNC rsspecan_ConfigureDOCSISStreamDirection(ViSession instrSession,
                                                          ViInt32   streamDirection)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_DOCSIS_STREAM_DIRECTION, streamDirection),
            2, "Stream Direction");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function Queries the PLC settings.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR startIndexAuto/If enabled, the start index of the physical link channel (PLC) is
/// HIPAR startIndexAuto/detected automatically. If disabled, the numeric value defined by
/// HIPAR startIndexAuto/CONFigure:DS:CHANnel:PLC:INDex is used.
/// HIPAR startIndex/Defines the start index of the physical link channel (PLC) if
/// HIPAR startIndex/automatic detection is disabled.
ViStatus _VI_FUNC rsspecan_ConfigureDOCSISPLCStartIndexL(ViSession instrSession,
                                                         ViBoolean startIndexAuto,
                                                         ViInt32   startIndex)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_DOCSIS_PLC_START_INDEX_L_AUTO, startIndexAuto),
            2, "Start IndexAuto");

    if (startIndexAuto == VI_FALSE)
    {
        viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_DOCSIS_PLC_START_INDEX_L, startIndex),
                3, "Start Index");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the number of entries in the Continuous Pilots and Excluded
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR numberofEntries/Queries the number of entries in the Continuous Pilots and Excluded
/// HIPAR numberofEntries/Subcarrier Assignment table.
ViStatus _VI_FUNC rsspecan_QueryDOCSISCPAESANumberOfEntries(ViSession instrSession,
                                                            ViInt32*  numberofEntries)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_GetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_DOCSIS_CONTINUOUS_PILOTS_AND_EXCLUDED_NUMBER_OF_ENTRIES, numberofEntries),
            2, "Numberof Entries");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Deletes all rows or the specified row in the Continuous Pilots and
/// HIFN  Excluded Subcarrier Table.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR setIndex/Control sets the index
/// HIPAR delete/Selects if single or all rows will to be deleted from the table.
ViStatus _VI_FUNC rsspecan_DeleteDOCSISCPAESATableRows(ViSession instrSession,
                                                       ViInt32   setIndex,
                                                       ViInt32   deleteRow)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, setIndex, 1, 200),
            2, "Set Index");

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_DOCSIS_CONTINUOUS_PILOTS_AND_EXCLUDED_SUBCARRIER_TABLE_DELETE_ROW,
            "DOCSISSetIndex", setIndex - 1, RS_REPCAP_BUF_SIZE, repCap));

    if (deleteRow == RSSPECAN_VAL_DOCSIS_SELECTED)
    {
        checkErr(rsspecan_SetAttributeViString(instrSession, repCap, RSSPECAN_ATTR_DOCSIS_CONTINUOUS_PILOTS_AND_EXCLUDED_SUBCARRIER_TABLE_DELETE_ROW, NULL));
    }
    else
    {
        checkErr(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_DOCSIS_CONTINUOUS_PILOTS_AND_EXCLUDED_SUBCARRIER_TABLE_DELETE_ALL, NULL));
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the type of configuration for the specified entry in the  Continuous Pilots and
/// HIFN     Excluded Subcarrier Assignment table.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR setIndex/Control sets the index
/// HIPAR type/Defines the type of configuration for the specified entry in the
/// HIPAR type/Continuous Pilots and Excluded Subcarrier Assignment table.
ViStatus _VI_FUNC rsspecan_ConfigureDOCSISCPAESAType(ViSession instrSession,
                                                     ViInt32   setIndex,
                                                     ViInt32   type)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, setIndex, 1, 200),
            2, "Set Index");
    snprintf(repCap, RS_REPCAP_BUF_SIZE, "I%ld", setIndex);

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_DOCSIS_CONFIGURE_TYPE, type),
            3, "Type");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Defines start, stop and increment for a series of subcarriers to be
/// HIFN  configured identically in the Continuous Pilots and Excluded Subcarrier
/// HIFN  Assignment table.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR setIndex/Control sets the index
/// HIPAR start/Defines the first subcarrier in a series of subcarriers to be
/// HIPAR start/configured identically in the Continuous Pilots and Excluded Subcarrier
/// HIPAR start/Assignment table.Must be lower than the parameter used by CONFigure:DS:
/// HIPAR start/CHANnel:CPES:SUBCarrier:STOP
/// HIPAR stop/Defines the last subcarrier in a series of subcarriers to be
/// HIPAR stop/configured identically in the Continuous Pilots and Excluded Subcarrier
/// HIPAR stop/Assignment table.Must be higher than the parameter used by
/// HIPAR stop/CONFigure:DS:CHANnel:CPES:SUBCarrier:STARt
/// HIPAR increment/Defines the increment for a series of subcarriers to be configured
/// HIPAR increment/identically in the Continuous Pilots and Excluded Subcarrier Assignment
/// HIPAR increment/table. Value between 1 and maximum number of subcarriers.
ViStatus _VI_FUNC rsspecan_ConfigureDOCSISCPAESASubcarrierStartStop(
                            ViSession instrSession,
                            ViInt32   setIndex,
                            ViInt32   start,
                            ViInt32   stop,
                            ViInt32   increment)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, setIndex, 1, 200),
            2, "Set Index");
    snprintf(repCap, RS_REPCAP_BUF_SIZE, "I%ld", setIndex);

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_DOCSIS_CONTINUOUS_PILOTS_AND_EXCLUDED_SUBCARRIER_START, start),
            3, "Start");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_DOCSIS_CONTINUOUS_PILOTS_AND_EXCLUDED_SUBCARRIER_STOP, stop),
            4, "Stop");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_DOCSIS_CONTINUOUS_PILOTS_AND_EXCLUDED_SUBCARRIER_RANGE_INCREMENT, increment),
            5, "Increment");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Defines a comma-separated list of (discrete) subcarriers to be
/// HIFN  configured identically in the Continuous Pilots and Excluded Subcarrier
/// HIFN  Assignment table.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR setIndex/Control sets the index
/// HIPAR arraySize/Configures number of elements for array "Set"
/// HIPAR set/Array of subcarrier numbers
ViStatus _VI_FUNC rsspecan_ConfigureDOCSISCPAESASubcarrierSet(
                            ViSession instrSession,
                            ViInt32   setIndex,
                            ViInt32   arraySize,
                            ViInt32   set[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, setIndex, 1, 200),
            2, "Set Index");

    if (arraySize > 0)
    {
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CONF:DS:CHAN:CPES%ld:SUBC:SET ", setIndex);
        checkErr(RsCore_WriteAsciiViInt32Array(instrSession, cmd, set, arraySize));

        checkErr(rsspecan_CheckStatus(instrSession));
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Defines how continuous pilots are detected in the symbols
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR continuousPilots/Defines how continuous pilots are detected in the symbols
ViStatus _VI_FUNC rsspecan_ConfigureDOCSISAutoDetectionContinuousPilots(
                            ViSession instrSession,
                            ViInt32   continuousPilots)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_DOCSIS_AUTO_DETECTION_CONTINUOUS_PILOTS, continuousPilots),
            2, "Continuous Pilots");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function selects the specified profile for further operation
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR profileList/This control selects the specified profile for further operation
ViStatus _VI_FUNC rsspecan_SelectDOCSISProfileList(ViSession instrSession,
                                                   ViInt32   profileList)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_DOCSIS_SELECT_PROFILE_LIST, profileList),
            2, "Profile List");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the number of entries in the Modulation Subcarrier Assignment
/// HIFN  table for the selected profile.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR numberofEntries/Queries the number of entries in the Modulation Subcarrier Assignment
/// HIPAR numberofEntries/table for the selected profile.
ViStatus _VI_FUNC rsspecan_QueryDOCSISProfileConfigurationNumberOfEntries(
                            ViSession instrSession,
                            ViInt32*  numberofEntries)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_GetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_DOCSIS_PROFILE_CONFIGURATION_NUMBER_OF_ENTRIES, numberofEntries),
            2, "Numberof Entries");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function deletes the currently selected profile.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsspecan_DeleteDOCSISProfileConfiguration(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_DOCSIS_PROFILE_CONFIGURATION_DELETE, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function returns the state of the currently selected profile,
/// HIFN  that is: whether the profile contains configuration entries or not.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR profileState/This command returns the state of the currently selected profile, that
/// HIPAR profileState/is: whether the profile contains configuration entries or not.
ViStatus _VI_FUNC rsspecan_QueryDOCSISProfileState(ViSession  instrSession,
                                                   ViBoolean* profileState)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_GetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_DOCSIS_PROFILE_QUERY_STATE, profileState),
            2, "Profile State ");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Defines the modulation used by the specified subcarriers in the
/// HIFN  currently selected profile.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR setIndex/Control sets the index
/// HIPAR profileList/Defines the modulation used by the specified subcarriers in the
/// HIPAR profileList/currently selected profile.
ViStatus _VI_FUNC rsspecan_ConfigureDOCSISModulation(ViSession instrSession,
                                                     ViInt32   setIndex,
                                                     ViInt32   profileList)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, setIndex, 1, 200),
            2, "Set Index");
    snprintf(repCap, RS_REPCAP_BUF_SIZE, "I%ld", setIndex);

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_DOCSIS_MODULATION, profileList),
            3, "Profile List");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Defines start, stop and increment for a series of subcarriers to be
/// HIFN  configured identically in the Modulation Subcarrier Assignment table
/// HIFN  for the currently selected profile
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR setIndex/Control sets the index
/// HIPAR start/Defines the first subcarrier in a series of subcarriers to be
/// HIPAR start/configured identically in the Modulation Subcarrier Assignment table
/// HIPAR start/for the currently selected profile.Must be lower than the parameter
/// HIPAR start/used by CONFigure:DS:CHANnel:PCONfig:SUBCarrier:STOP
/// HIPAR stop/Defines the first subcarrier in a series of subcarriers to be
/// HIPAR stop/configured identically in the Modulation Subcarrier Assignment table
/// HIPAR stop/for the currently selected profile.Must be higher than the parameter
/// HIPAR stop/used by CONFigure:DS: CHANnel:PCONfig:SUBCarrier:STARt
/// HIPAR increment/Defines the increment for a series of subcarriers to be configured
/// HIPAR increment/identically in the Modulation Subcarrier Assignment table for the
/// HIPAR increment/currently selected profile
ViStatus _VI_FUNC rsspecan_ConfigureDOCSISModulationSubcarrierStartStop(
                            ViSession instrSession,
                            ViInt32   setIndex,
                            ViInt32   start,
                            ViInt32   stop,
                            ViInt32   increment)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, setIndex, 1, 200),
            2, "Set Index");
    snprintf(repCap, RS_REPCAP_BUF_SIZE, "I%ld", setIndex);

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_DOCSIS_MODULATION_SUBCARRIER_START, start),
            3, "Start");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_DOCSIS_MODULATION_SUBCARRIER_STOP, stop),
            4, "Stop");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_DOCSIS_MODULATION_SUBCARRIER_INCREMENT, increment),
            5, "Increment");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Defines a comma-separated list of (discrete) subcarriers to be
/// HIFN  configured identically in the Modulation Subcarrier Assignment table
/// HIFN  for the currently selected profile.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR setIndex/Control sets the index
/// HIPAR arraySize/Configures number of elements for array "Set"
/// HIPAR set/Array of subcarrier numbers
ViStatus _VI_FUNC rsspecan_ConfigureDOCSISModulationSubcarrierSet(
                            ViSession instrSession,
                            ViInt32   setIndex,
                            ViInt32   arraySize,
                            ViInt32   set[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, setIndex, 1, 200),
            2, "Set Index");

    if (arraySize > 0)
    {
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CONF:DS:CHAN:PCON%ld:SUBC:SET ", setIndex);
        checkErr(RsCore_WriteAsciiViInt32Array(instrSession, cmd, set, arraySize));
        checkErr(rsspecan_CheckStatus(instrSession));
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Deletes all sets or the specified subcarrier set in the downstream
/// HIFN  Modulation Subcarrier Assignment table.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR setIndex/Control sets the index
/// HIPAR delete/Selects if subcarrier or subcarrier set will to be deleted from the
/// HIPAR delete/table.
ViStatus _VI_FUNC rsspecan_DeleteDOCSISModulationSubcarrierTableSet(
                            ViSession instrSession,
                            ViInt32   setIndex,
                            ViInt32   deleteRow)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, setIndex, 1, 200),
            2, "Set Index");

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_DOCSIS_MODULATION_SUBCARRIER_ASSIGNMENT_TABLE_DELETE_SUBCARRIER,
            "DOCSISSetIndex", setIndex - 1, RS_REPCAP_BUF_SIZE, repCap));

    if (deleteRow == RSSPECAN_VAL_DOCSIS_SUBCARRIER)
    {
        checkErr(rsspecan_SetAttributeViString(instrSession, repCap, RSSPECAN_ATTR_DOCSIS_MODULATION_SUBCARRIER_ASSIGNMENT_TABLE_DELETE_SUBCARRIER, NULL));
    }
    else
    {
        checkErr(rsspecan_SetAttributeViString(instrSession, repCap, RSSPECAN_ATTR_DOCSIS_MODULATION_SUBCARRIER_ASSIGNMENT_TABLE_DELETE_SUBCARRIER_SET, NULL));
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the number of entries in the modulation table for the NCP profile.
/// HIFN  Configuration" table.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR setIndex/sets the index
/// HIPAR numberOfEntries/Returns the number of entries in the modulation table for the NCP profile.
ViStatus _VI_FUNC rsspecan_QueryDOCSISNextCodewordPointerNumberOfEntries(
                            ViSession instrSession,
                            ViInt32   setIndex,
                            ViInt32*  numberOfEntries)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, setIndex, 1, 200),
            2, "Set Index");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "I%ld", setIndex);

    viCheckParm(rsspecan_GetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_DOCSIS_NEXT_CODEWORD_POINTER_NUMBER_OF_ENTRIES, numberOfEntries),
            3, "Number Of Entries");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Deletes all rows or the specified row of the NCP profile.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR setIndex/Control sets the index
/// HIPAR delete/Selects if single or all rows will to be deleted from the NCP profile.
ViStatus _VI_FUNC rsspecan_DeleteDOCSISNextCodewordPointerProfileRow(
                            ViSession instrSession,
                            ViInt32   setIndex,
                            ViInt32   deleteRow)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, setIndex, 1, 200),
            2, "Set Index");

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_DOCSIS_NEXT_CODEWORD_POINTER_PROFILE_DELETE_ROW,
            "DOCSISSetIndex", setIndex - 1, RS_REPCAP_BUF_SIZE, repCap));

    if (deleteRow == RSSPECAN_VAL_DOCSIS_SELECTED)
    {
        checkErr(rsspecan_SetAttributeViString(instrSession, repCap, RSSPECAN_ATTR_DOCSIS_NEXT_CODEWORD_POINTER_PROFILE_DELETE_ROW, NULL));
    }
    else
    {
        checkErr(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_DOCSIS_NEXT_CODEWORD_POINTER_PROFILE_DELETE_ALL, NULL));
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the modulation used by the specified row in the NCP profile.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR setIndex/sets the index
/// HIPAR modulation/Selects the modulation used by the specified row in the NCP profile.
ViStatus _VI_FUNC rsspecan_ConfigureDOCSISNextCodewordPointerModulation(
                            ViSession instrSession,
                            ViInt32   setIndex,
                            ViInt32   modulation)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, setIndex, 1, 200),
            2, "Set Index");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "I%ld", setIndex);

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_DOCSIS_NEXT_CODEWORD_POINTER_MODULATION, modulation),
            3, "Modulation");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines start, stop and increment for a series of subcarriers to
/// HIFN  be configured identically in the Modulation Subcarrier
/// HIFN  Assignment table for the Next Codeword Pointer profile.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR setIndex/Defines the first subcarrier in a series of subcarriers in the selected row to be configured identically in the NCP profile.
/// HIPAR start/Defines the first subcarrier in a series of subcarriers in the selected row to be configured identically in the NCP profile.
/// HIPAR stop/Defines the last subcarrier in a series of subcarriers in the selected row to be configured identically in the NCP profile.
/// HIPAR increment/Defines the increment in a series of subcarriers in the selected row to be configured identically in the NCP profile
ViStatus _VI_FUNC rsspecan_ConfigureDOCSISNextCodewordPointerStartStop(
                            ViSession instrSession,
                            ViInt32   setIndex,
                            ViInt32   start,
                            ViInt32   stop,
                            ViInt32   increment)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, setIndex, 1, 200),
            2, "Set Index");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "I%ld", setIndex);

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_DOCSIS_NEXT_CODEWORD_POINTER_START, start),
            3, "Start");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_DOCSIS_NEXT_CODEWORD_POINTER_STOP, stop),
            4, "Stop");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_DOCSIS_NEXT_CODEWORD_POINTER_INCREMENT, increment),
            5, "Increment");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines a comma-separated list of (discrete) subcarriers to be
/// HIFN  configured identically in the selected row of the NCP profile.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR setIndex/Sets the index
/// HIPAR arraySize/Configures number of elements for array "Set"
/// HIPAR set/Array of subcarrier numbers for the NCP profile.
ViStatus _VI_FUNC rsspecan_ConfigureDOCSISNextCodewordPointerSet(
                            ViSession instrSession,
                            ViInt32   setIndex,
                            ViInt32   arraySize,
                            ViInt32   set[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, setIndex, 1, 200),
            2, "Set Index");

    if (arraySize > 0)
    {
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CONF:DS:CHAN:NCP:PCON%ld:SUBC:SET ", setIndex);
        checkErr(RsCore_WriteAsciiViInt32Array(instrSession, cmd, set, arraySize));
        checkErr(rsspecan_CheckStatus(instrSession));
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This command returns the number of codewords (rows) in the "Frame
/// HIFN  Configuration" table.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR numberofRows/This command returns the number of codewords (rows) in the "Frame
/// HIPAR numberofRows/Configuration" table.
ViStatus _VI_FUNC rsspecan_QueryDOCSISCodewordNumberOfRows(ViSession instrSession,
                                                           ViInt32*  numberofRows)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_GetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_DOCSIS_CODEWORD_NUMBER_OF_ROWS, numberofRows),
            2, "Number Of Rows");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Deletes all rows or the specified row in the downlink frame config
/// HIFN  table.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR codewordIndex/Control sets the index
/// HIPAR delete/Selects if single or all rows will to be deleted from the frame config
/// HIPAR delete/table.
ViStatus _VI_FUNC rsspecan_DeleteDOCSISCodewordFrameTableRow(ViSession instrSession,
                                                             ViInt32   codewordIndex,
                                                             ViInt32   deleteRow)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, codewordIndex, 1, 1536),
            2, "Codeword Index");

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_DOCSIS_CODEWORD_FRAME_TABLE_DELETE_ROW,
            "DOCSISCodewordIndex", codewordIndex - 1, RS_REPCAP_BUF_SIZE, repCap));

    if (deleteRow == RSSPECAN_VAL_DOCSIS_SUBCARRIER)
    {
        checkErr(rsspecan_SetAttributeViString(instrSession, repCap, RSSPECAN_ATTR_DOCSIS_CODEWORD_FRAME_TABLE_DELETE_ROW, NULL));
    }
    else
    {
        checkErr(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_DOCSIS_CODEWORD_FRAME_TABLE_DELETE_ALL, NULL));
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This command returns the number of codewords (rows) in the "Frame
/// HIFN  Configuration" table.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR codewordIndex/Control sets the index
/// HIPAR assignProfile/Assigns one of the active profiles defined in the "Profile List" to
/// HIPAR assignProfile/the selected codeword.
ViStatus _VI_FUNC rsspecan_SelectDOCSISCodewordAssignProfile(ViSession instrSession,
                                                             ViInt32   codewordIndex,
                                                             ViInt32   assignProfile)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, codewordIndex, 1, 1535),
            2, "Codeword Index");
    snprintf(repCap, RS_REPCAP_BUF_SIZE, "CI%ld", codewordIndex);

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_DOCSIS_CODEWORD_ASSIGN_PROFILE, assignProfile),
            3, "Assign Profile");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Defines the first subcarrier and number of subcariers to which the
/// HIFN  selected codeword is assigned.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR codewordIndex/Control sets the index
/// HIPAR firstSubcarrier/Defines the first subcarrier to which the selected codeword is
/// HIPAR firstSubcarrier/assigned.
/// HIPAR numberOfSubcarriers/Defines the number of subcarriers to which the selected codeword is
/// HIPAR numberOfSubcarriers/assigned.
ViStatus _VI_FUNC rsspecan_ConfigureDOCSISCodewordSubcarrierFrame(
                            ViSession instrSession,
                            ViInt32   codewordIndex,
                            ViInt32   firstSubcarrier,
                            ViInt32   numberOfSubcarriers)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, codewordIndex, 1, 1535),
            2, "Codeword Index");
    snprintf(repCap, RS_REPCAP_BUF_SIZE, "CI%ld", codewordIndex);

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_DOCSIS_CODEWORD_FRAME_FIRST_SUBCARRIER, firstSubcarrier),
            3, "First Subcarrier");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_DOCSIS_CODEWORD_FRAME_NUMBER_OF_SUBCARRIERS, numberOfSubcarriers),
            4, "Number Of Subcarriers");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Defines the first symbol and number of symbols to which the selected
/// HIFN  codeword is assigned.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR codewordIndex/Control sets the index
/// HIPAR firstSymbol/Defines the first symbol to which the selected codeword is assigned.
/// HIPAR numberOfSymbols/Defines the number of symbols to which the selected codeword is
/// HIPAR numberOfSymbols/assigned
ViStatus _VI_FUNC rsspecan_ConfigureDOCSISCodewordSymbolFrame(
                            ViSession instrSession,
                            ViInt32   codewordIndex,
                            ViInt32   firstSymbol,
                            ViInt32   numberOfSymbols)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, codewordIndex, 1, 1535),
            2, "Codeword Index");
    snprintf(repCap, RS_REPCAP_BUF_SIZE, "CI%ld", codewordIndex);

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_DOCSIS_CODEWORD_FRAME_FIRST_SYMBOL, firstSymbol),
            3, "First Symbol");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_DOCSIS_CODEWORD_FRAME_NUMBER_OF_SYMBOLS, numberOfSymbols),
            4, "Number Of Symbols");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This attribute determines how frames are configured.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR npcAuto/This control determines how frames are configured.
ViStatus _VI_FUNC rsspecan_ConfigureDOCSISCodewordDetectionNCPAuto(
                            ViSession instrSession,
                            ViInt32   npcAuto)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_DOCSIS_CODEWORD_DETECTION_NCP_AUTO, npcAuto),
            2, "NPC Auto");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the number of symbols per frame to be expected.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR startIndex/Defines the number of symbols per frame to be expected.
ViStatus _VI_FUNC rsspecan_ConfigureDOCSISUpstreamSymbolsPerFrame(
                            ViSession instrSession,
                            ViInt32   startIndex)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_DOCSIS_UPSTREAM_SYMBOLS_PER_FRAME, startIndex),
            2, "Start Index");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the number of entries in the Excluded Subcarrier Assignment
/// HIFN  table.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR numberofEntries/Queries the number of entries in the Excluded Subcarrier Assignment
/// HIPAR numberofEntries/table.
ViStatus _VI_FUNC rsspecan_QueryDOCSISUpstreamESANumberOfEntries(
                            ViSession instrSession,
                            ViInt32*  numberofEntries)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_GetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_DOCSIS_UPSTREAM_EXCLUDED_SUBCARRIER_NUMBER_OF_ENTRIES, numberofEntries),
            2, "Number of Entries");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Deletes all rows or the specified row in the upstream Excluded
/// HIFN  Subcarrier Table.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR setIndex/Control sets the index
/// HIPAR delete/Selects if single or all rows will to be deleted from the upstream
/// HIPAR delete/Excluded Subcarrier Table.
ViStatus _VI_FUNC rsspecan_DeleteDOCSISUpstreamESATableRow(ViSession instrSession,
                                                           ViInt32   setIndex,
                                                           ViInt32   deleteRow)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, setIndex, 1, 200),
            2, "Set Index");

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_DOCSIS_UPSTREAM_EXCLUDED_SUBCARRIER_TABLE_DELETE_ROW,
            "DOCSISSetIndex", setIndex - 1, RS_REPCAP_BUF_SIZE, repCap));

    if (deleteRow == RSSPECAN_VAL_DOCSIS_SELECTED)
    {
        checkErr(rsspecan_SetAttributeViString(instrSession, repCap, RSSPECAN_ATTR_DOCSIS_UPSTREAM_EXCLUDED_SUBCARRIER_TABLE_DELETE_ROW, NULL));
    }
    else
    {
        checkErr(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_DOCSIS_UPSTREAM_EXCLUDED_SUBCARRIER_TABLE_DELETE_ALL, NULL));
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the type of special subcarrier.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR setIndex/Control sets the index
/// HIPAR type/Defines the type of configuration for the specified entry in the
/// HIPAR type/Continuous Pilots and Excluded Subcarrier Assignment table.
ViStatus _VI_FUNC rsspecan_QueryDOCSISUpstreamESAType(ViSession instrSession,
                                                      ViInt32   setIndex,
                                                      ViInt32*  type)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, setIndex, 1, 200),
            2, "Set Index");
    snprintf(repCap, RS_REPCAP_BUF_SIZE, "I%ld", setIndex);

    viCheckParm(rsspecan_GetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_DOCSIS_UPSTREAM_EXCLUDED_SUBCARRIER_TYPE, type),
            3, "Type");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Defines start, stop and increment for a series of subcarriers to be
/// HIFN  configured identically in the Continuous Pilots and Excluded Subcarrier
/// HIFN  Assignment table.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR setIndex/Control sets the index
/// HIPAR start/Defines the last excluded subcarrier in a series of subcarriers in the
/// HIPAR start/Excluded Subcarrier Assignment table. Must be lower than  stop.
/// HIPAR stop/Defines the first excluded subcarrier in a series of subcarriers in
/// HIPAR stop/the Excluded Subcarrier Assignment table. Must be higher than the
/// HIPAR stop/parameter used by start.
/// HIPAR increment/Defines the increment for a series of subcarriers to be configured
/// HIPAR increment/identically in the Excluded Subcarrier Assignment table. Value between
/// HIPAR increment/1 and maximum number of subcarriers.
ViStatus _VI_FUNC rsspecan_ConfigureDOCSISUpstreamESASubcarrierStartStop(
                            ViSession instrSession,
                            ViInt32   setIndex,
                            ViInt32   start,
                            ViInt32   stop,
                            ViInt32   increment)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, setIndex, 1, 200),
            2, "Set Index");
    snprintf(repCap, RS_REPCAP_BUF_SIZE, "I%ld", setIndex);

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_DOCSIS_UPSTREAM_EXCLUDED_SUBCARRIER_START, start),
            3, "Start");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_DOCSIS_UPSTREAM_EXCLUDED_SUBCARRIER_STOP, stop),
            4, "Stop");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_DOCSIS_UPSTREAM_EXCLUDED_SUBCARRIER_RANGE_INCREMENT, increment),
            5, "Increment");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Defines a list of (discrete) subcarriers to be configured identically
/// HIFN  in the Excluded Subcarrier Assignment table.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR setIndex/Control sets the index
/// HIPAR arraySize/Configures number of elements for array "Set"
/// HIPAR set/Array of subcarrier numbers
ViStatus _VI_FUNC rsspecan_ConfigureDOCSISUpstreamESASubcarrierSet(
                            ViSession instrSession,
                            ViInt32   setIndex,
                            ViInt32   arraySize,
                            ViInt32   set[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, setIndex, 1, 200),
            2, "Set Index");

    if (arraySize > 0)
    {
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CONF:US:CHAN:ESUB%ld:SUBC:SET ", setIndex);
        checkErr(RsCore_WriteAsciiViInt32Array(instrSession, cmd, set, arraySize));
        checkErr(rsspecan_CheckStatus(instrSession));
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Defines the upstream profile..
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR setIndex/Control sets the index
/// HIPAR modulation/Defines the modulation used by the specified subcarriers.
/// HIPAR pilotPattern/Defines the pilot pattern used by the specified minislots. Which
/// HIPAR pilotPattern/patterns are available depends on the FFT mode.
/// HIPAR numberOfMinislots/Defines the number of minislots for which the modulation is used.
ViStatus _VI_FUNC rsspecan_ConfigureDOCSISUpstreamProfile(ViSession instrSession,
                                                          ViInt32   setIndex,
                                                          ViInt32   modulation,
                                                          ViInt32   pilotPattern,
                                                          ViInt32   numberOfMinislots)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, setIndex, 1, 200),
            2, "Set Index");
    snprintf(repCap, RS_REPCAP_BUF_SIZE, "I%ld", setIndex);

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_DOCSIS_UPSTREAM_PROFILE_CONFIGURATION_MODULATION, modulation),
            3, "Modulation");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_DOCSIS_UPSTREAM_PROFILE_CONFIGURATION_PILOT_PATTERN, pilotPattern),
            4, "Pilot Pattern");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_DOCSIS_UPSTREAM_PROFILE_CONFIGURATION_NUMBER_OF_MINISLOTS, numberOfMinislots),
            5, "Number Of Minislots");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Deletes all minislot rows or the specified minislot row in the US
/// HIFN  Profile Config Table.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR setIndex/Control sets the index
/// HIPAR delete/Selects if single or all rows will to be deleted from the US Profile
/// HIPAR delete/Config Table.
ViStatus _VI_FUNC rsspecan_DeleteDOCSISUpstreamProfileTableRow(
                            ViSession instrSession,
                            ViInt32   setIndex,
                            ViInt32   deleteRow)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, setIndex, 1, 200),
            2, "Set Index");

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_DOCSIS_UPSTREAM_PROFILE_CONFIGURATION_TABLE_DELETE_ROW,
            "DOCSISSetIndex", setIndex - 1, RS_REPCAP_BUF_SIZE, repCap));

    if (deleteRow == RSSPECAN_VAL_DOCSIS_SELECTED)
    {
        checkErr(rsspecan_SetAttributeViString(instrSession, repCap, RSSPECAN_ATTR_DOCSIS_UPSTREAM_PROFILE_CONFIGURATION_TABLE_DELETE_ROW, NULL));
    }
    else
    {
        checkErr(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_DOCSIS_UPSTREAM_PROFILE_CONFIGURATION_TABLE_DELETE_ALL, NULL));
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Enables or disables the use of the direct path for frequencies close
/// HIFN  to 0 Hz.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR path/Enables or disables the use of the direct path for frequencies close
/// HIPAR path/to 0 Hz.
ViStatus _VI_FUNC rsspecan_ConfigureDOCSISInputDirectPath(ViSession instrSession,
                                                          ViInt32   path)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_DOCSIS_INPUT_DIRECT_PATH, path),
            2, "Path");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the current record length, that is: the number of samples
/// HIFN  captured during the measurement. The maximum number of samples depends
/// HIFN  on the specified NFFT .0 to 4K mode: 1228800000 samples; 8K mode:
/// HIFN  2457600000 samples
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR numberofSamples/Queries the current record length, that is: the number of samples
/// HIPAR numberofSamples/captured during the measurement. The maximum number of samples depends
/// HIPAR numberofSamples/on the specified NFFT .0 to 4K mode: 1228800000 samples; 8K mode:
/// HIPAR numberofSamples/2457600000 samples
ViStatus _VI_FUNC rsspecan_QueryDOCSISNumberOfSamples(ViSession instrSession,
                                                      ViInt64*  numberofSamples)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_GetAttributeViInt64(instrSession, "",
                    RSSPECAN_ATTR_DOCSIS_SIGNAL_CAPTURE_NUMBER_OF_SAMPLES,
                    numberofSamples), 2, "Numberof Samples");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Enables or disables use of the adjacent channel filter. If activated,
/// HIFN  only the useful signal is analyzed, all signal data in adjacent
/// HIFN  channels is removed by the filter. This setting improves the signal to
/// HIFN  noise ratio and thus the MER results for signals with strong or a large
/// HIFN  number of adjacent channels. However, for some measurements,
/// HIFN  information on the effects of adjacent channels on the measured signal
/// HIFN  may be of interest.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR filterOutAdjacentChannels/Enables or disables use of the adjacent channel filter.
ViStatus _VI_FUNC rsspecan_ConfigureDOCSISFilterOutAdjacentChannels(
                            ViSession instrSession,
                            ViBoolean filterOutAdjacentChannels)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, NULL, RSSPECAN_ATTR_DOCSIS_SIGNAL_CAPTURE_FILTER_OUT_ADJACENT_CHANNELS, filterOutAdjacentChannels),
            2, "Filter Out Adjacent Channels");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Determines the area within the capture buffer in which the frame start
/// HIFN  is searched.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR enabled/Determines the area within the capture buffer in which the frame start
/// HIPAR enabled/is searched.
ViStatus _VI_FUNC rsspecan_ConfigureDOCSISPowerIntervalSearchEnabled(
                            ViSession instrSession,
                            ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_DOCSIS_POWER_INTERVAL_SEARCH, enabled),
            2, "Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function determines how channels are detected and compensated for in the input signal.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channelEstimation/Determines how channels are detected and compensated for in the
/// HIPAR channelEstimation/input signal.
ViStatus _VI_FUNC rsspecan_ConfigureDOCSISChannelEstimation(ViSession instrSession,
                                                            ViInt32   channelEstimation)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_DOCSIS_CHANNEL_ESTIMATION, channelEstimation),
            2, "Channel Estimation");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Activates or deactivates the compensation for phase and timing drift.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR phaseTracking/Activates or deactivates the compensation for phase drifts. If
/// HIPAR phaseTracking/activated, the measurementresults are compensated for phase drifts on a
/// HIPAR phaseTracking/per-symbol basis
/// HIPAR timeTracking/Activates or deactivates the compensation for timing drift. If
/// HIPAR timeTracking/activated, the measurement results are compensated for timing error on
/// HIPAR timeTracking/a per-symbol basis.
ViStatus _VI_FUNC rsspecan_ConfigureDOCSISTracking(ViSession instrSession,
                                                   ViBoolean phaseTracking,
                                                   ViBoolean timeTracking)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_DOCSIS_PHASE_TRACKING, phaseTracking),
            2, "Phase Tracking");

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_DOCSIS_TIME_TRACKING, timeTracking),
            3, "Time Tracking");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This attribute determines whether codewords are decoded or not.

/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR decoded/This control determines whether codewords are decoded or not.
ViStatus _VI_FUNC rsspecan_ConfigureDOCSISDecodedCodewords(ViSession instrSession,
                                                           ViBoolean decoded)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_DOCSIS_CODEWORDS_DECODED, decoded),
            2, "Decoded");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function determines which bits of the data stream are decoded and then
/// HIFN  displayed in a Bitstream result display.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR bitstream/Determines which bits of the data stream are decoded and then
/// HIPAR bitstream/displayed in a Bitstream result display.
ViStatus _VI_FUNC rsspecan_ConfigureDOCSISDecodedBitstream(ViSession instrSession,
                                                           ViInt32   bitstream)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_DOCSIS_BITSTREAM_DECODED, bitstream),
            2, "Bitstream");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function determines the auto detection mode for Upstream
/// HIFN  measurements. Settings are either auto-detected from the Signal, or
/// HIFN  User defined.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR autoDetection/This control determines the auto detection mode for Upstream
/// HIPAR autoDetection/measurements. Settings are either auto-detected from the Signal, or
/// HIPAR autoDetection/User defined.
ViStatus _VI_FUNC rsspecan_ConfigureDOCSISUpstreamAutoDetection(
                            ViSession instrSession,
                            ViInt32   autoDetection)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_DOCSIS_UPSTREAM_AUTO_DETECTION, autoDetection),
            2, "Auto Detection");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIRET This function configures single frame analyze.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR specifiedFrame/Enabled/Disabled specified frame
/// HIPAR frame/Specified frame number is evaluated in all graphical and numeric result displays.
ViStatus _VI_FUNC rsspecan_ConfigureDOCSISAnalyzeSingleFrame(ViSession instrSession,
                                                             ViBoolean specifiedFrame,
                                                             ViInt32   frame)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_DOCSIS_EVALUATION_SPECIFIED_FRAME_STATE, specifiedFrame),
            2, "Specified Frame");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_DOCSIS_EVALUATION_SELECTED_FRAME, frame),
            3, "Frame");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIRET This function configures single frame analyze.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR frameStatisticCount/Enabled/Disabled frame statistic count
/// HIPAR numberOfFrame/Specifies number of frames is taken into consideration for the
/// HIPAR numberOfFrame/statistical evaluation
ViStatus _VI_FUNC rsspecan_ConfigureDOCSISFrameStatistic(ViSession instrSession,
                                                         ViBoolean frameStatisticCount,
                                                         ViInt32   numberOfFrame)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_DOCSIS_EVALUATION_FRAME_STATISTIC_COUNT_STATE, frameStatisticCount),
            2, "Frame Statistic Count");

    if (frameStatisticCount)
    {
        viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_DOCSIS_EVALUATION_NUMBER_OF_FRAME, numberOfFrame),
                3, "Number Of Frame");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIRET Determines the behavior of the application after a limit check fails.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR state/Enables or disables the behavior of the application after a limit check fails.
ViStatus _VI_FUNC rsspecan_ConfigureDOCSISLimitCheckState(ViSession instrSession,
                                                          ViBoolean state)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_DOCSIS_LIMIT_CHECK_STATE, state),
            2, "State");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function excludes specific subcarriers from modulation error
/// HIFN  ratio (MER) calculation.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR autoState/Enables/disable excluding of the subcarriers.
/// HIPAR numberOfSubcarriers/Defines the number of worst subcarriers to be excluded from MER
/// HIPAR numberOfSubcarriers/calculation. Auto has to be ON.
ViStatus _VI_FUNC rsspecan_ConfigureDOCSISExcludingSubcarriers(
                            ViSession instrSession,
                            ViBoolean autoState,
                            ViInt32   numberOfSubcarriers)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_DOCSIS_EVALUATION_RANGE_MER_EXCLUDING_SUBCARRIERS_MODE, autoState),
            2, "Auto State");

    if (autoState)
    {
        viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_DOCSIS_EVALUATION_RANGE_MER_EXCLUDING_SUBCARRIERS_COUNT, numberOfSubcarriers),
                3, "Number Of Subcarriers");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function defines a list of subcarriers to be excluded from MER
/// HIFN  calculation.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR arraySize/Configures number of elements for array "Subcarriers"
/// HIPAR subcarriers/Array of subcarrier numbers.
ViStatus _VI_FUNC rsspecan_ConfigureDOCSISExcludingUserSubcarriers(
                            ViSession instrSession,
                            ViInt32   arraySize,
                            ViInt32   subcarriers[])
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_DOCSIS_EVALUATION_RANGE_MER_EXCLUDING_SUBCARRIERS_MODE, RSSPECAN_VAL_DOCSIS_EXCLUDING_SUBCARRIERS_MODE_USER));

    if (arraySize > 0)
    {
        checkErr(RsCore_WriteAsciiViInt32Array(instrSession, "CONF:MEXC:SUBC:SET ", subcarriers, arraySize));
        checkErr(rsspecan_CheckStatus(instrSession));
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIRET This function configures display settings for constellation diagram.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR object/Defines the objects for which the Constellation diagram is displayed.
/// HIPAR modulation/Defines the modulation for which the Constellation diagram is displayed.
ViStatus _VI_FUNC rsspecan_ConfigureDOCSISDisplaySettings(ViSession instrSession,
                                                          ViInt32   object,
                                                          ViInt32   modulation)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_DOCSIS_EVALUATION_CONSTELLATION_OBJECT, object),
            2, "Object");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_DOCSIS_EVALUATION_CONSTELLATION_MODULATION, modulation),
            3, "Modulation");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  The Constellation diagram is displayed for all configured or detected
/// HIFN  subcarriers.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR diagramSubcarrier/Defines the evaluation range for the Constellation diagram.The
/// HIPAR diagramSubcarrier/constellation diagram is restricted to the specified subcarrier.
/// HIPAR type/This control selects the type.
ViStatus _VI_FUNC rsspecan_ConfigureDOCSISEvaluationRangeSubcarrier(
                            ViSession instrSession,
                            ViInt32   type,
                            ViInt32   diagramSubcarrier)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    if (type == 0)
    {
        checkErr(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_DOCSIS_EVALUATION_CONSTELLATION_DIAGRAM_SUBCARRIER_ALL, NULL));
    }
    else
    {
        viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_DOCSIS_EVALUATION_RANGE_CONSTELLATION_DIAGRAM_SUBCARRIER, diagramSubcarrier),
                2, "Diagram Subcarrier");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Defines the evaluation range for the Constellation diagram.The
/// HIFN  constellation diagram is restricted to the specified subcarrier.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR diagramSymbol/Defines the evaluation range for the Constellation diagram. The
/// HIPAR diagramSymbol/constellation diagram is restricted to the specified symbol.
/// HIPAR type/This control selects the type.
ViStatus _VI_FUNC rsspecan_ConfigureDOCSISEvaluationRangeSymbol(
                            ViSession instrSession,
                            ViInt32   type,
                            ViInt32   diagramSymbol)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    if (type == 0)
    {
        checkErr(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_DOCSIS_EVALUATION_RANGE_CONSTELLATION_DIAGRAM_SYMBOL_ALL, NULL));
    }
    else
    {
        viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_DOCSIS_EVALUATION_RANGE_CONSTELLATION_DIAGRAM_SYMBOL, diagramSymbol),
                2, "Diagram Symbol");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  If enabled, the constellation uses a single color for all modulations,
/// HIFN  which improves performance. However, individual modulation types cannot
/// HIFN  be selected. If disabled, different colors are used for different
/// HIFN  modulation types. Which modulation types are displayed (depending on
/// HIFN  the selected object) is configurable.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR fastMode/Enables or disables fast mode.
ViStatus _VI_FUNC rsspecan_ConfigureDOCSISEvaluationRangeFastMode(
                            ViSession instrSession,
                            ViBoolean fastMode)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, NULL, RSSPECAN_ATTR_DOCSIS_EVALUATION_RANGE_CONSTELLATION_FAST_MODE, fastMode),
            2, "Fast Mode");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the size of the individual symbol points in the Constellation diagram.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR symbolPointsSize/Selects the size of the individual symbol points in the Constellation diagram.
ViStatus _VI_FUNC rsspecan_ConfigureDOCSISEvaluationRangeSymbolSize(
                            ViSession instrSession,
                            ViInt32   symbolPointsSize)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_DOCSIS_EVALUATION_RANGE_SYMBOL_SIZE, symbolPointsSize),
            2, "Symbol Points Size");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  For result displays that evaluate a parameter per carrier (e.g. MER vs
/// HIFN  Carrier), this control defines whether the carrier number or the
/// HIFN  carrier frequency (in Hz) is displayed on the x-axis. Note, however,
/// HIFN  that this setting applies to ALL result displays based on carriers.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR axesUnit/For result displays that evaluate a parameter per carrier (e.g. MER vs
/// HIPAR axesUnit/Carrier), this control defines whether the carrier number or the
/// HIPAR axesUnit/carrier frequency (in Hz) is displayed on the x-axis. Note, however,
/// HIPAR axesUnit/that this setting applies to ALL result displays based on carriers.
ViStatus _VI_FUNC rsspecan_ConfigureDOCSISCarrierAxesUnit(ViSession instrSession,
                                                          ViInt32   axesUnit)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_DOCSIS_CARRIER_AXES_UNIT, axesUnit),
            2, "Axes Unit");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Performs an initial measurement in order to determine the
/// HIFN  required signal description settings automatically from the
/// HIFN  detected signal characteristics before starting (or continuing)
/// HIFN  the actual Modulation Accuracy measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsspecan_ConfigureDOCSISAutoDetect(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_DOCSIS_AUTO_DETECT, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function sets bitstream result displays.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR bitstreamFormat/Determines whether the data is displayed as bits or bytes
/// HIPAR bitstreamFormat/(default) in a Bitstream result display.
/// HIPAR bitstreamLayout/Determines whether a compact or expanded view of the bits are
/// HIPAR bitstreamLayout/displayed in the Bitstream result display.
ViStatus _VI_FUNC rsspecan_ConfigureDOCSISBitstreamDisplaySettings(
                            ViSession instrSession,
                            ViInt32   bitstreamFormat,
                            ViInt32   bitstreamLayout)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_DOCSIS_BITSREAM_FORMAT, bitstreamFormat),
            2, "Bitstream Format");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_DOCSIS_BITSTREAM_LAYOUT, bitstreamLayout),
            3, "Bitstream Layout");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function defines which items are displayed in the Result Summary. Note that
/// HIFN  the results are always calculated, regardless of their visibility in the
/// HIFN  Result Summary.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR item/
/// HIPAR displayItem/
ViStatus _VI_FUNC rsspecan_ConfigureDOCSISResultSummaryDisplay(
                            ViSession instrSession,
                            ViInt32   item,
                            ViBoolean displayItem)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";
    ViChar*  p2string;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentModel(instrSession, "FSW"));

    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &rsspecan_rngDOCSISTableItem, item, &p2string),
                3, "Item");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "DISP:WIND:TABL:ITEM %s,%s", p2string, offOnArr[displayItem]);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Determines whether the power results in the Result Summary are
/// HIFN  calculated as absolute values or relative to the power of the data subcarriers.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR powerUnit/Selects whether the power results in the Result Summary are
/// HIPAR powerUnit/calculated as absolute values or relative to the power of the data subcarriers.
ViStatus _VI_FUNC rsspecan_ConfigureDOCSISResultSummaryUnit(ViSession instrSession,
                                                            ViInt32   powerUnit)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_DOCSIS_RESULT_SUMMARY_UNIT, powerUnit),
            2, "Power Unit");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the result summary display settings.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR normalizeCMSampleClockError/If enabled, the sample clock error (CERR) of the cable modem is
/// HIPAR normalizeCMSampleClockError/normalized with reference to the reference sample clock of the CMTS. If
/// HIPAR normalizeCMSampleClockError/disabled, the calculated sample clock error is determined with
/// HIPAR normalizeCMSampleClockError/reference to the 102.4 MHz specified by the DOCSIS 3.1 standard.
/// HIPAR cmtsReferenceSampleClock/Defines the reference value used to normalize the CM sample clock
/// HIPAR cmtsReferenceSampleClock/error, if normalization is enabled (see
/// HIPAR cmtsReferenceSampleClock/RSSPECAN_ATTR_DOCSIS_NORMALIZE_CM_SAMPLE_CLOCK_ERROR_RESULT). Specify
/// HIPAR cmtsReferenceSampleClock/the unit (and thus type) of the reference value using
/// HIPAR cmtsReferenceSampleClock/RSSPECAN_ATTR_DOCSIS_CMTS_REFERENCE_SAMPLE_CLOCK_UNIT.
/// HIPAR cmtsRefSampleClockUnit/Configures the unit of the reference value for the CMTS sample clock
/// HIPAR cmtsRefSampleClockUnit/(see RSSPECAN_ATTR_DOCSIS_CMTS_REFERENCE_SAMPLE_CLOCK).
ViStatus _VI_FUNC rsspecan_ConfigureDOCSISResultSummaryDisplaySettings(
                            ViSession instrSession,
                            ViBoolean normalizeCMSampleClockError,
                            ViReal64  cmtsReferenceSampleClock,
                            ViInt32   cmtsRefSampleClockUnit)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, NULL, RSSPECAN_ATTR_DOCSIS_NORMALIZE_CM_SAMPLE_CLOCK_ERROR_RESULT, normalizeCMSampleClockError),
            2, "Normalize CM Sample Clock Error");

    if (normalizeCMSampleClockError == VI_TRUE)
    {
        viCheckParm(rsspecan_SetAttributeViReal64(instrSession, NULL, RSSPECAN_ATTR_DOCSIS_CMTS_REFERENCE_SAMPLE_CLOCK, cmtsReferenceSampleClock),
                3, "CMTS Reference Sample Clock");

        viCheckParm(rsspecan_SetAttributeViInt32(instrSession, NULL, RSSPECAN_ATTR_DOCSIS_CMTS_REFERENCE_SAMPLE_CLOCK_UNIT, cmtsRefSampleClockUnit),
                4, "CMTS Ref Sample Clock Unit");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines whether power results are provided as absolute values (dBm) or
/// HIFN  relative to the carrier (dBc).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR powerUnit/Defines whether power results are provided as absolute values (dBm) or
/// HIPAR powerUnit/relative to the carrier (dBc).
ViStatus _VI_FUNC rsspecan_ConfigureDOCSISPowerUnit(ViSession instrSession,
                                                    ViInt32   powerUnit)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_DOCSIS_POWER_UNIT, powerUnit),
            2, "Power Unit");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Determines how the bands to be measured are configured - automatically
/// HIFN  by the R&S FSW DOCSIS 3.1 application, or manually by the user.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR autoConfiguration/Determines how the bands to be measured are configured - automatically
/// HIPAR autoConfiguration/by the R&S FSW DOCSIS 3.1 application, or manually by the user.
ViStatus _VI_FUNC rsspecan_ConfigureDOCSISAutoConfigurationPerContinuousMinislotBlock(
                            ViSession instrSession,
                            ViBoolean autoConfiguration)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_DOCSIS_AUTO_CONFIGURATION_PER_CONTINUOUS_MINISLOT_BLOCK, autoConfiguration),
            2, "Auto Configuration");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines whether the bands for which the power is calculated are
/// HIFN  detected strictly according to the DOCSIS 3.1 standard, or by
/// HIFN  user-defined settings.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR bandAutoConfig/Defines whether the bands for which the power is calculated are
/// HIPAR bandAutoConfig/detected strictly according to the DOCSIS 3.1 standard, or by
/// HIPAR bandAutoConfig/user-defined settings.
ViStatus _VI_FUNC rsspecan_ConfigureDOCSISBandAutoConfig(ViSession instrSession,
                                                         ViInt32   bandAutoConfig)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_DOCSIS_AUTOMATIC_BAND_CONFIGURATION, bandAutoConfig),
            2, "Band Auto Config");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the offsets and bandwidths for the bands (in relation to the
/// HIFN  detected miniblock sets) as a number of subcarriers.
/// HIFN
/// HIFN     Note(s):
/// HIFN
/// HIFN     (1) This function is available when
/// HIFN  rsspecan_ConfigureDOCSISBandAutoConfig is set to 'User Defined'.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR adjBandsOffsetSC/This control specifies the offset to the adjacent channel as the
/// HIPAR adjBandsOffsetSC/number of subcarriers it contains.
/// HIPAR alt1BandsOffsetSC/This control specifies the offset to the alternate channel as the
/// HIPAR alt1BandsOffsetSC/number of subcarriers it contains.
/// HIPAR adjBandsBandwidthSC/This control specifies the bandwidth to the adjacent channel as the
/// HIPAR adjBandsBandwidthSC/number of subcarriers it contains.
/// HIPAR alt1BandsBandwidthSC/This control specifies the bandwidth to the alternate channel as the
/// HIPAR alt1BandsBandwidthSC/number of subcarriers it contains.
ViStatus _VI_FUNC rsspecan_ConfigureDOCSISBandConfigurationTableSubcarriers(
                            ViSession instrSession,
                            ViInt32   adjBandsOffsetSC,
                            ViInt32   alt1BandsOffsetSC,
                            ViInt32   adjBandsBandwidthSC,
                            ViInt32   alt1BandsBandwidthSC)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_DOCSIS_ADJACENT_CHANNEL_BANDS_OFFSET_SUBCARRIERS, adjBandsOffsetSC),
            2, "Adj Bands Offset SC");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_DOCSIS_ALTERNATE_CHANNEL_BANDS_OFFSET_SUBCARRIERS, alt1BandsOffsetSC),
            3, "Alt1 Bands Offset SC");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_DOCSIS_ADJACENT_CHANNEL_BANDS_BANDWIDTH_SUBCARRIERS, adjBandsBandwidthSC),
            4, "Adj Bands Bandwidth SC");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_DOCSIS_ALTERNATE_CHANNEL_BANDS_BANDWIDTH_SUBCARRIERS, alt1BandsBandwidthSC),
            5, "Alt1 Bands Bandwidth SC");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the offsets and bandwidths for the bands (in relation to the
/// HIFN  detected miniblock sets) as a frequency range.
/// HIFN
/// HIFN     Note(s):
/// HIFN
/// HIFN     (1) This function is available when
/// HIFN  rsspecan_ConfigureDOCSISBandAutoConfig is set to 'User Defined'.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR adjBandsOffsetFrequency/This control specifies the frequency offset to the adjacent channel.
/// HIPAR alt1BandsOffsetFrequency/This control specifies the frequency offset to the alternate channel.
/// HIPAR adjBandsBandwidthFrequency/This control specifies the bandwidth to the adjacent channel as a
/// HIPAR adjBandsBandwidthFrequency/frequency span.
/// HIPAR alt1BandsBandwidthFrequency/This control specifies the bandwidth to the alternate channel as a
/// HIPAR alt1BandsBandwidthFrequency/frequency span.
ViStatus _VI_FUNC rsspecan_ConfigureDOCSISBandConfigurationTableFrequency(
                            ViSession instrSession,
                            ViReal64  adjBandsOffsetFrequency,
                            ViReal64  alt1BandsOffsetFrequency,
                            ViReal64  adjBandsBandwidthFrequency,
                            ViReal64  alt1BandsBandwidthFrequency)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_DOCSIS_ADJACENT_CHANNEL_BANDS_OFFSET_FREQUENCY, adjBandsOffsetFrequency),
            2, "Adj Bands Offset Frequency");

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_DOCSIS_ALTERNATE_CHANNEL_BANDS_OFFSET_FREQUENCY, alt1BandsOffsetFrequency),
            3, "Alt1 Bands Offset Frequency");

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_DOCSIS_ADJACENT_CHANNEL_BANDS_BANDWIDTH_FREQUENCY, adjBandsBandwidthFrequency),
            4, "Adj Bands Bandwidth Frequency");

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_DOCSIS_ALTERNATE_CHANNEL_BANDS_BANDWIDTH_FREQUENCY, alt1BandsBandwidthFrequency),
            5, "Alt1 Bands Bandwidth Frequency");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function specifies whether auto defined bands should be displayed
/// HIFN  at the start and end of each minislot block, or only at the first and
/// HIFN  last edges.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR autoBandsAppliedTo/This control specifies whether auto defined bands should be displayed
/// HIPAR autoBandsAppliedTo/at the start and end of each minislot block, or only at the first and
/// HIPAR autoBandsAppliedTo/last edges.
ViStatus _VI_FUNC rsspecan_ConfigureDOCSISAutoBandsAppliedTo(ViSession instrSession,
                                                             ViInt32   autoBandsAppliedTo)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_DOCSIS_AUTO_BANDS_APPLIED_TO, autoBandsAppliedTo),
            2, "Auto Bands Applied To");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function defines up to 20 independent (fixed) frequency bands.
/// HIFN
/// HIFN     Note(s):
/// HIFN
/// HIFN     (1) This function is available when
/// HIFN  rsspecan_ConfigureDOCSISAutoConfigurationPerContinuousMinislotBlock is
/// HIFN  set to Off.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR band/This control sets the user-configured power band.
/// HIPAR state/This control specifies whether the user configured band will be
/// HIPAR state/calculated as part of the Spectrum Band Power measurement results.
/// HIPAR locationReference/Defines whether the indicated 'Location' is located on the left or on
/// HIPAR locationReference/the right edge of the band to be measured.
/// HIPAR locationSC/Defines the subcarrier at the specified edge of the power band to be
/// HIPAR locationSC/measured. The edge is set with control 'Location Reference'.
/// HIPAR bandwidthSC/Defines the number of subcarriers in the power band to be measured.
ViStatus _VI_FUNC rsspecan_ConfigureDOCSISUserConfig(ViSession instrSession,
                                                     ViInt32   band,
                                                     ViBoolean state,
                                                     ViInt32   locationReference,
                                                     ViInt32   locationSC,
                                                     ViInt32   bandwidthSC)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, band, 1, 20),
            2, "Band");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_DOCSIS_USER_CONFIG_BAND_STATE,
            "DOCSISBand", band - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Band");

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, repCap, RSSPECAN_ATTR_DOCSIS_USER_CONFIG_BAND_STATE, state),
            3, "State");

    if (state == VI_TRUE)
    {
        viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_DOCSIS_USER_CONFIG_LOCATION_REFERENCE, locationReference),
                4, "Location Reference");

        viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_DOCSIS_USER_CONFIG_LOCATION_SUBCARRIERS, locationSC),
                5, "Location SC");

        viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_DOCSIS_USER_CONFIG_BANDWIDTH_SUBCARRIERS, bandwidthSC),
                6, "Bandwidth SC");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function defines automatic Y-scalling.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR automaticGridScaling/This control activates or deactivates automatic scaling of the y-axis
/// HIPAR automaticGridScaling/for the specified trace display. If enabled, the R&S FSW DOCSIS 3.1
/// HIPAR automaticGridScaling/application automatically scales the y-axis to best fit the measurement
/// HIPAR automaticGridScaling/results.
/// HIPAR autoMode/This control determines which algorithm is used to determine whether
/// HIPAR autoMode/the y-axis requires automatic rescaling
/// HIPAR autoFixRange/This control defines the use of fixed value limits.
ViStatus _VI_FUNC rsspecan_ConfigureDOCSISYScaling(ViSession instrSession,
                                                   ViInt32   window,
                                                   ViBoolean automaticGridScaling,
                                                   ViInt32   autoMode,
                                                   ViInt32   autoFixRange)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, window, 0, 16),
            2, "Window");
    snprintf(repCap, RS_REPCAP_BUF_SIZE, "Win%ld", window);

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, repCap, RSSPECAN_ATTR_DOCSIS_AUTOMATIC_GRID_SCALING, automaticGridScaling),
            3, "Automatic Grid Scaling");
    if (automaticGridScaling == VI_TRUE)
    {
        viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_DOCSIS_AUTO_MODE, autoMode),
                4, "Auto Mode");

        viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_DOCSIS_AUTO_FIX_RANGE, autoFixRange),
                5, "Auto Fix Range");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function defines automatic Y-scalling.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR minimum/Defines the minimum value to be displayed on the y-axis of the
/// HIPAR minimum/specified evaluation diagram.
/// HIPAR maximum/Defines the maximum value to be displayed on the y-axis of the
/// HIPAR maximum/specified evaluation diagram.
ViStatus _VI_FUNC rsspecan_ConfigureDOCSISYScalingMinMax(ViSession instrSession,
                                                         ViInt32   window,
                                                         ViReal64  minimum,
                                                         ViReal64  maximum)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, window, 0, 16),
            2, "Window");
    snprintf(repCap, RS_REPCAP_BUF_SIZE, "Win%ld", window);

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_DOCSIS_Y_SCALE_MINIMUM, minimum),
            3, "Minimum");

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_DOCSIS_Y_SCALE_MAXIMUM, maximum),
            4, "Maximum");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function defines automatic Y-scalling.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR hysteresisIntervalLowerUpper/For automatic scaling based on hysteresis, this control defines the
/// HIPAR hysteresisIntervalLowerUpper/upper limit of the lower hysteresis interval.
/// HIPAR hysteresisIntervalLowerLower/For automatic scaling based on hysteresis, this control defines the
/// HIPAR hysteresisIntervalLowerLower/lower limit of the lower hysteresis interval.
/// HIPAR hysteresisIntervalUpperLower/For automatic scaling based on hysteresis, this control defines the
/// HIPAR hysteresisIntervalUpperLower/upper limit of the lower hysteresis interval. If the minimum value in
/// HIPAR hysteresisIntervalUpperLower/the current measurement exceeds this limit, the y-axis is rescaled
/// HIPAR hysteresisIntervalUpperLower/automatically.
/// HIPAR hysteresisIntervalUpperUpper/For automatic scaling based on hysteresis, this control defines the
/// HIPAR hysteresisIntervalUpperUpper/lower limit of the upper hysteresis interval. If the maximum value in
/// HIPAR hysteresisIntervalUpperUpper/the current measurement drops below this limit, the y-axis is rescaled
/// HIPAR hysteresisIntervalUpperUpper/automatically.
ViStatus _VI_FUNC rsspecan_ConfigureDOCSISYScalingHysteresis(ViSession instrSession,
                                                             ViInt32   window,
                                                             ViReal64  hysteresisIntervalLowerUpper,
                                                             ViReal64  hysteresisIntervalLowerLower,
                                                             ViReal64  hysteresisIntervalUpperLower,
                                                             ViReal64  hysteresisIntervalUpperUpper)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, window, 0, 16),
            2, "Window");
    snprintf(repCap, RS_REPCAP_BUF_SIZE, "Win%ld", window);

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_DOCSIS_HYSTERESIS_INTERVAL_LOWER_UPPER, hysteresisIntervalLowerUpper),
            3, "Hysteresis Interval Lower Upper");

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_DOCSIS_HYSTERESIS_INTERVAL_LOWER_LOWER, hysteresisIntervalLowerLower),
            4, "Hysteresis Interval Lower Lower");

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_DOCSIS_HYSTERESIS_INTERVAL_UPPER_LOWER, hysteresisIntervalUpperLower),
            5, "Hysteresis Interval Upper Lower");

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_DOCSIS_HYSTERESIS_INTERVAL_UPPER_UPPER, hysteresisIntervalUpperUpper),
            6, "Hysteresis Interval Upper Upper");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function defines automatic Y-scalling.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR memoryDepth/For automatic scaling based on memory, this value defines the number
/// HIPAR memoryDepth/<x> of previous results to be considered when determining if rescaling
/// HIPAR memoryDepth/is required. The minimum and maximum value of each measurement are
/// HIPAR memoryDepth/added to the memory.
ViStatus _VI_FUNC rsspecan_ConfigureDOCSISYScalingMemoryDepth(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   memoryDepth)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, window, 0, 16),
            2, "Window");
    snprintf(repCap, RS_REPCAP_BUF_SIZE, "Win%ld", window);

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_DOCSIS_MEMORY_DEPTH, memoryDepth),
            3, "Memory Depth");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the number of divisions to be used for the y-axis in the
/// HIFN  specified window. Separate division settings can be configured for
/// HIFN  individual result displays.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR numberofDivisions/Defines the number of divisions to be used for the y-axis in the
/// HIPAR numberofDivisions/specified window. Separate division settings can be configured for
/// HIPAR numberofDivisions/individual result displays.
/// HIPAR arraySize/Sets the number of elements of the Scaling Per Division array to be used.
/// HIPAR scalingPerDivision/Defines the number of divisions to be used for the y-axis in the
/// HIPAR scalingPerDivision/specified window. Separate division settings can be configured
/// HIPAR scalingPerDivision/for individual result displays.
ViStatus _VI_FUNC rsspecan_ConfigureDOCSISYScalingNumberofDivisions(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   numberofDivisions,
                            ViInt32   arraySize,
                            ViInt32   scalingPerDivision[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViString scalingArray[] = {"1.0", "2.0", "2.5", "5.0"};

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, window, 0, 16),
            2, "Window");
    snprintf(repCap, RS_REPCAP_BUF_SIZE, "Win%ld", window);

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_DOCSIS_NUMBER_OF_DIVISIONS, numberofDivisions),
            3, "Number Of Divisions");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "DISP:WIND%ld:TRAC:Y:SCAL:PDIV ", window);
    checkErr(RsCore_WriteViStringArray(instrSession, cmd, scalingArray, scalingPerDivision, arraySize));

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function adds a window to the display.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR stringSize/This control sets the allocated length of Returned Window Name string.
/// HIPAR windowName/String containing the number or name of the window the new window is
/// HIPAR windowName/inserted next to. In the default state, the name of the window is its
/// HIPAR windowName/index.
/// HIPAR direction/Direction the window is added relative to the window set with "Reference Window
/// HIPAR direction/Name".
/// HIPAR windowType/Type of result display you want to add.
/// HIPAR returnedWindowName/When adding a new window, the function returns its name (by
ViStatus _VI_FUNC rsspecan_AddDOCSISWindow(ViSession instrSession,
                                           ViInt32   stringSize,
                                           ViString  referenceWindowName,
                                           ViInt32   direction,
                                           ViInt32   windowType,
                                           ViChar    returnedWindowName[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViChar   response[RS_MAX_MESSAGE_BUF_SIZE];
    ViChar*  p2direction;
    ViChar*  p2type;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &rsspecan_rngLayoutDirection, direction, &p2direction),
                4, "Direction");
    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &rsspecan_rngDOCSISLayoutType, windowType, &p2type),
                5, "Window Type");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "LAY:ADD? '%s',%s,%s", referenceWindowName, p2direction, p2type);
    checkErr(RsCore_QueryViString(instrSession, cmd, response));

    if (returnedWindowName != NULL)
    {
        RsCore_TrimString(response, RS_VAL_TRIM_WHITESPACES_AND_ALL_QUOTES);
        RsCore_CopyToUserBufferAsciiData(returnedWindowName, stringSize, response);
    }

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function replaces a window with another one.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR windowName/String containing the number or name of the window you want to
/// HIPAR windowType/Type of result display you want to add.
ViStatus _VI_FUNC rsspecan_ReplaceDOCSISWindow(ViSession instrSession,
                                               ViString  windowName,
                                               ViInt32   windowType)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViChar*  p2type;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &rsspecan_rngDOCSISLayoutType, windowType, &p2type),
                3, "Window Type");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "LAY:REPL '%s',%s", windowName, p2type);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the window type of the window specified by the index <n>.
/// HIFN
/// HIFN     Note(s):
/// HIFN     (1) This function is only available for R&S FSW.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR windowType/Returns the window type of the window specified by the index <n>.
ViStatus _VI_FUNC rsspecan_QueryDOCSISWindowType(ViSession instrSession,
                                                 ViInt32   window,
                                                 ViInt32*  windowType)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_DOCSIS_WINDOW_TYPE,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap));

    viCheckParm(rsspecan_GetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_DOCSIS_WINDOW_TYPE, windowType),
            3, "Window Type");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Performs an initial measurement in order to determine the required
/// HIFN  signal description settings automatically from the detected PLC before
/// HIFN  starting (or continuing) the actual Modulation Accuracy measurement.
/// HIFN  This function is only available for downstream DOCSIS 3.1 signals.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsspecan_DOCSISAutoSetFromPLCAndRun(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsspecan_SetAttributeViString(instrSession, NULL, RSSPECAN_ATTR_DOCSIS_AUTO_SET_FROM_PLC_AND_RUN, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Performs an initial measurement in order to determine the required
/// HIFN  signal description settings automatically from the detected signal
/// HIFN  characteristics before starting (or continuing) the actual Modulation
/// HIFN  Accuracy measurement. This function is only available for upstream
/// HIFN  DOCSIS 3.1 signals.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsspecan_DOCSISAutoDetectionAndRun(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsspecan_SetAttributeViString(instrSession, NULL, RSSPECAN_ATTR_DOCSIS_AUTO_DETECTION_AND_RUN, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  These function return the average, maximum or minimum result of the
/// HIFN  specified parameter.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR specifiedParameter/Select Measurement Type
/// HIPAR average/These control return the average result of the specified parameter.
/// HIPAR maximum/These control return the average, maximum or minimum result of the
/// HIPAR maximum/specified parameter.
/// HIPAR minimum/These control return the average, maximum or minimum result of the
/// HIPAR minimum/specified parameter.
ViStatus _VI_FUNC rsspecan_FetchDOCSISResults(ViSession instrSession,
                                              ViInt32   specifiedParameter,
                                              ViReal64* average,
                                              ViReal64* maximum,
                                              ViReal64* minimum)
{
    ViStatus error = VI_SUCCESS;
    ViAttr   attrAVG = 0, attrMIN = 0, attrMAX = 0;

    checkErr(RsCore_LockSession(instrSession));

    switch (specifiedParameter)
    {
    case RSSPECAN_VAL_DOCSIS_MEAS_TYPE_CENTER_ERROR:
        attrAVG = RSSPECAN_ATTR_DOCSIS_FETCH_CENTER_ERROR_AVERAGE;
        attrMAX = RSSPECAN_ATTR_DOCSIS_FETCH_CENTER_ERROR_MAXIMUM;
        attrMIN = RSSPECAN_ATTR_DOCSIS_FETCH_CENTER_ERROR_MINIMUM;
        break;
    case RSSPECAN_VAL_DOCSIS_ANALYSIS_RESULT_FETCH_FREQUENCY_ERROR:
        attrAVG = RSSPECAN_ATTR_DOCSIS_FETCH_FREQUENCY_ERROR_AVERAGE;
        attrMAX = RSSPECAN_ATTR_DOCSIS_FETCH_FREQUENCY_ERROR_MAXIMUM;
        attrMIN = RSSPECAN_ATTR_DOCSIS_FETCH_FREQUENCY_ERROR_MINIMUM;
        break;
    case RSSPECAN_VAL_DOCSIS_ANALYSIS_RESULT_FETCH_MER:
        attrAVG = RSSPECAN_ATTR_DOCSIS_FETCH_MODULATION_ERROR_DATA_PILOT_AVERAGE;
        attrMAX = RSSPECAN_ATTR_DOCSIS_FETCH_MODULATION_ERROR_DATA_PILOT_MAXIMUM;
        attrMIN = RSSPECAN_ATTR_DOCSIS_FETCH_MODULATION_ERROR_DATA_PILOT_MINIMUM;
        break;
    case RSSPECAN_VAL_DOCSIS_ANALYSIS_RESULT_FETCH_MER_DATA:
        attrAVG = RSSPECAN_ATTR_DOCSIS_FETCH_MODULATION_ERROR_DATA_AVERAGE;
        attrMAX = RSSPECAN_ATTR_DOCSIS_FETCH_MODULATION_ERROR_DATA_MAXIMUM;
        attrMIN = RSSPECAN_ATTR_DOCSIS_FETCH_MODULATION_ERROR_DATA_MINIMUM;
        break;
    case RSSPECAN_VAL_DOCSIS_ANALYSIS_RESULT_FETCH_MER_PILOT:
        attrAVG = RSSPECAN_ATTR_DOCSIS_FETCH_MODULATION_ERROR_PILOT_AVERAGE;
        attrMAX = RSSPECAN_ATTR_DOCSIS_FETCH_MODULATION_ERROR_PILOT_MAXIMUM;
        attrMIN = RSSPECAN_ATTR_DOCSIS_FETCH_MODULATION_ERROR_PILOT_MINIMUM;
        break;
    case RSSPECAN_VAL_DOCSIS_ANALYSIS_RESULT_PHASE_ERROR:
        attrAVG = RSSPECAN_ATTR_DOCSIS_FETCH_PHASE_ERROR_AVERAGE;
        attrMAX = RSSPECAN_ATTR_DOCSIS_FETCH_PHASE_ERROR_MAXIMUM;
        attrMIN = RSSPECAN_ATTR_DOCSIS_FETCH_PHASE_ERROR_MINIMUM;
        break;
    case RSSPECAN_VAL_DOCSIS_ANALYSIS_RESULT_FETCH_POWER:
        attrAVG = RSSPECAN_ATTR_DOCSIS_FETCH_POWER_AVERAGE;
        attrMAX = RSSPECAN_ATTR_DOCSIS_FETCH_POWER_MAXIMUM;
        attrMIN = RSSPECAN_ATTR_DOCSIS_FETCH_POWER_MINIMUM;
        break;
    case RSSPECAN_VAL_DOCSIS_ANALYSIS_RESULT_FETCH_TPLC:
        attrAVG = RSSPECAN_ATTR_DOCSIS_FETCH_TPLC_AVERAGE;
        attrMAX = RSSPECAN_ATTR_DOCSIS_FETCH_TPLC_MAXIMUM;
        attrMIN = RSSPECAN_ATTR_DOCSIS_FETCH_TPLC_MINIMUM;
        break;
    case RSSPECAN_VAL_DOCSIS_ANALYSIS_RESULT_FETCH_ZBIT:
        attrAVG = RSSPECAN_ATTR_DOCSIS_FETCH_ZBIT_AVERAGE;
        attrMAX = RSSPECAN_ATTR_DOCSIS_FETCH_ZBIT_MAXIMUM;
        attrMIN = RSSPECAN_ATTR_DOCSIS_FETCH_ZBIT_MINIMUM;
        break;
    case RSSPECAN_VAL_DOCSIS_ANALYSIS_RESULT_FETCH_AMINISLOTS:
        attrAVG = RSSPECAN_ATTR_DOCSIS_FETCH_POWER_ANALYZED_MINISLOTS_AVERAGE;
        attrMAX = RSSPECAN_ATTR_DOCSIS_FETCH_POWER_ANALYZED_MINISLOTS_MAXIMUM;
        attrMIN = RSSPECAN_ATTR_DOCSIS_FETCH_POWER_ANALYZED_MINISLOTS_MINIMUM;
        break;
    case RSSPECAN_VAL_DOCSIS_ANALYSIS_RESULT_FETCH_TRIGGER_TO_FRAME:
        attrAVG = RSSPECAN_ATTR_DOCSIS_FETCH_TRIGGER_TO_FRAME_AVERAGE;
        attrMAX = RSSPECAN_ATTR_DOCSIS_FETCH_TRIGGER_TO_FRAME_MAXIMUM;
        attrMIN = RSSPECAN_ATTR_DOCSIS_FETCH_TRIGGER_TO_FRAME_MINIMUM;
        break;
    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, specifiedParameter), 2, "Specified Parameter");
        break;
    }
    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, "", attrAVG, average),
            3, "Average");

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, "", attrMAX, maximum),
            4, "Maximum");

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, "", attrMIN, minimum),
            5, "Minimum");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Returns the average, maximum or minimum result of the Continuous Pilots.
/// HIFN  list in the order they are displayed in the result display.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR average/This control returns the average result of Continuous Pilots.
/// HIPAR maximum/This control return the maximum result of Continuous Pilots.
/// HIPAR minimum/This control return the minimum result of Continuous Pilots.
ViStatus _VI_FUNC rsspecan_FetchDOCSISContinuousPilotsResults(
                            ViSession instrSession,
                            ViReal64* average,
                            ViReal64* maximum,
                            ViReal64* minimum)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_DOCSIS_FETCH_CONTINUOUS_PILOTS_AVERAGE, average),
            2, "Average");

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_DOCSIS_FETCH_CONTINUOUS_PILOTS_MAXIMUM, maximum),
            3, "Maximum");

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_DOCSIS_FETCH_CONTINUOUS_PILOTS_MINIMUM, minimum),
            4, "Minimum");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Returns the average, maximum or minimum result of data subcarriers.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR average/This control returns the average result of Data Pilots.
/// HIPAR maximum/This control return the maximum result of Data Pilots.
/// HIPAR minimum/This control return the minimum result of Data Pilots.
ViStatus _VI_FUNC rsspecan_FetchDOCSISDataResults(ViSession instrSession,
                                                  ViReal64* average,
                                                  ViReal64* maximum,
                                                  ViReal64* minimum)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_DOCSIS_FETCH_DATA_AVERAGE, average),
            2, "Average");

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_DOCSIS_FETCH_DATA_MAXIMUM, maximum),
            3, "Maximum");

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_DOCSIS_FETCH_DATA_MINIMUM, minimum),
            4, "Minimum");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Returns the average, maximum or minimum result of power pilots.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR average/This control returns the average result of power Pilots.
/// HIPAR maximum/This control return the maximum result of power Pilots.
/// HIPAR minimum/This control return the minimum result of power Pilots.
ViStatus _VI_FUNC rsspecan_FetchDOCSISPowerPilotsResults(ViSession instrSession,
                                                         ViReal64* average,
                                                         ViReal64* maximum,
                                                         ViReal64* minimum)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_DOCSIS_FETCH_POWER_PILOTS_AVERAGE, average),
            2, "Average");

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_DOCSIS_FETCH_POWER_PILOTS_MAXIMUM, maximum),
            3, "Maximum");

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_DOCSIS_FETCH_POWER_PILOTS_MINIMUM, minimum),
            4, "Minimum");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Returns the average, maximum or minimum result of the Scattered Pilots.
/// HIFN  list in the order they are displayed in the result display.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR average/This control returns the average result of Scattered Pilots
/// HIPAR maximum/This control return the maximum result of Scattered Pilots.
/// HIPAR minimum/This control return the minimum result of Scattered Pilots.
ViStatus _VI_FUNC rsspecan_FetchDOCSISScatteredPilotsResults(ViSession instrSession,
                                                             ViReal64* average,
                                                             ViReal64* maximum,
                                                             ViReal64* minimum)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_DOCSIS_FETCH_SCATTERED_PILOTS_AVERAGE, average),
            2, "Average");

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_DOCSIS_FETCH_SCATTERED_PILOTS_MAXIMUM, maximum),
            3, "Maximum");

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_DOCSIS_FETCH_SCATTERED_PILOTS_MINIMUM, minimum),
            4, "Minimum");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Returns the average, maximum or minimum result of the Physical Link Channel.
/// HIFN  list in the order they are displayed in the result display.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR average/This control returns the average result of Physical Link Channel.
/// HIPAR maximum/This control return the maximum result of Physical Link Channel.
/// HIPAR minimum/This control return the minimum result of Physical Link Channel.
ViStatus _VI_FUNC rsspecan_FetchDOCSISPhysicalLinkChannelResults(
                            ViSession instrSession,
                            ViReal64* average,
                            ViReal64* maximum,
                            ViReal64* minimum)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_DOCSIS_FETCH_PHYSICAL_LINK_CHANNEL_AVERAGE, average),
            2, "Average");

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_DOCSIS_FETCH_PHYSICAL_LINK_CHANNEL_MAXIMUM, maximum),
            3, "Maximum");

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_DOCSIS_FETCH_PHYSICAL_LINK_CHANNEL_MINIMUM, minimum),
            4, "Minimum");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function returns all result summary values as a comma-separated
/// HIFN  list in the order they are displayed in the result display.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR arraySize/This control defines the size of the array Results.
/// HIPAR results/This control returns all result summary values as an array in the
/// HIPAR results/order they are displayed in the result display.
/// HIPAR returnedValues/Queries number of returned Results
ViStatus _VI_FUNC rsspecan_FetchDOCSISAllResults(ViSession instrSession,
                                                 ViInt32   arraySize,
                                                 ViReal64  results[],
                                                 ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32RangeMin(instrSession, arraySize, 1),
                3, "Array Size");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, results), 4, "Results");

    checkErr(RsCore_QueryFloatArrayToUserBuffer(instrSession, "FETC:SUMM:ALL?", arraySize, results, returnedValues));

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function returns all result summary values as a comma-separated
/// HIFN  list in the order they are displayed in the result display.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR arraySize/This control defines the size of the arrays.
/// HIPAR returnedValues/Queries number of returned Results
/// HIPAR cwIndex/Returns Array of Codeword indexes (0..1535) Not available for PLC,
/// HIPAR cwIndex/pilots and excluded subcarriers (Remote: -1)
/// HIPAR symbolStart/Returns Array of OFDM symbols (0..127) Not available for PLC, pilots
/// HIPAR symbolStart/and excluded subcarriers (Remote: -1)
/// HIPAR objectInformationType/Returns Array of Information type:
/// HIPAR profileList/Returns Array of Profiles assigned to the codeword.
/// HIPAR modulation/Returns Array of Modulations
/// HIPAR mer/Returns Array of Modulation error ratios in dB
/// HIPAR power/Returns Array of Powers in dBm
/// HIPAR numberOfSubcarrier/Returns Array of Numbers of subcarrier
/// HIPAR ldpciIterations/Returns Array of Low density parity check Number of iterations
/// HIPAR ldpciBitErrPre/Returns Array of Low density parity check Absolute number of bit
/// HIPAR ldpciBitErrPre/errors before decoding
/// HIPAR ldpciBERPre/Returns Array of Low density parity check
/// HIPAR ldpciBitErrPost/Returns Array of Low density parity check
/// HIPAR ldpciBERPost/Returns Array of Low density parity check
/// HIPAR ldpciCWErrPost/Returns Array of Low density parity check
/// HIPAR ldpciCBLErrPost/Returns Array of Low density parity check
/// HIPAR reserved1/This control is reserved for future use.
/// HIPAR reserved2/This control is reserved for future use.
/// HIPAR reserved3/This control is reserved for future use.
/// HIPAR reserved4/This control is reserved for future use.
ViStatus _VI_FUNC rsspecan_FetchDOCSISSignalContentDetailed(ViSession instrSession,
                                                            ViInt32   arraySize,
                                                            ViInt32*  returnedValues,
                                                            ViInt32   cwIndex[],
                                                            ViInt32   symbolStart[],
                                                            ViInt32   objectInformationType[],
                                                            ViInt32   profileList[],
                                                            ViInt32   modulation[],
                                                            ViReal64  mer[],
                                                            ViReal64  power[],
                                                            ViInt32   numberOfSubcarrier[],
                                                            ViInt32   ldpciIterations[],
                                                            ViInt32   ldpciBitErrPre[],
                                                            ViReal64  ldpciBERPre[],
                                                            ViInt32   ldpciBitErrPost[],
                                                            ViReal64  ldpciBERPost[],
                                                            ViInt32   ldpciCWErrPost[],
                                                            ViReal64  ldpciCBLErrPost[],
                                                            ViInt32   reserved1[],
                                                            ViReal64  reserved2[],
                                                            ViInt32   reserved3[],
                                                            ViReal64  reserved4[])
{
    ViStatus error = VI_SUCCESS;
    ViChar*  pbuffer = NULL;
    ViChar*  p2buf;
    ViInt32  i = 0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32RangeMin(instrSession, arraySize, 1),
                3, "Array Size");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, returnedValues), 4, "Returned Values");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, cwIndex), 5, "CW Index");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, symbolStart), 6, "Symbol Start");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, objectInformationType), 7, "Object Information Type");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, profileList), 8, "Profile List");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, modulation), 9, "Modulation");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, mer), 10, "MER");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, power), 11, "Power");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, numberOfSubcarrier), 12, "Number Of Subcarrier");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, ldpciIterations), 13, "LDPC Iterations");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, ldpciBitErrPre), 14, "LDPC Bit Err Pre");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, ldpciBERPre), 15, "LDPC BER Pre");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, ldpciBitErrPost), 16, "LDPC Bit Err Post");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, ldpciBERPost), 17, "LDPCI BER Post");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, ldpciCWErrPost), 18, "LDPC CW Err Post");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, ldpciCBLErrPost), 19, "LDPC CBL Err Post");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, reserved1), 20, "Reserved 1");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, reserved2), 21, "Reserved 2");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, reserved3), 21, "Reserved 3");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, reserved4), 21, "Reserved 4");

    checkErr(RsCore_QueryViStringUnknownLength(instrSession, "FETC:SCD:ALL?", &pbuffer));

    p2buf = strtok(pbuffer, ",");

    while (p2buf && (i < arraySize))
    {
        cwIndex[i] = RsCore_Convert_String_To_ViInt32(p2buf);
        p2buf = strtok(NULL, ",");

        symbolStart[i] = RsCore_Convert_String_To_ViInt32(p2buf);
        p2buf = strtok(NULL, ",");

        objectInformationType[i] = RsCore_Convert_String_To_ViInt32(p2buf);
        p2buf = strtok(NULL, ",");

        profileList[i] = RsCore_FindStringIndex(profileListArr, p2buf);
        p2buf = strtok(NULL, ",\n");

        modulation[i] = RsCore_FindStringIndex(modulationArr, p2buf);
        p2buf = strtok(NULL, ",\n");

        mer[i] = RsCore_Convert_String_To_ViReal64(p2buf);
        p2buf = strtok(NULL, ",");

        power[i] = RsCore_Convert_String_To_ViReal64(p2buf);
        p2buf = strtok(NULL, ",");

        numberOfSubcarrier[i] = RsCore_Convert_String_To_ViInt32(p2buf);
        p2buf = strtok(NULL, ",");

        ldpciIterations[i] = RsCore_Convert_String_To_ViInt32(p2buf);
        p2buf = strtok(NULL, ",");

        ldpciBitErrPre[i] = RsCore_Convert_String_To_ViInt32(p2buf);
        p2buf = strtok(NULL, ",");

        ldpciBERPre[i] = RsCore_Convert_String_To_ViReal64(p2buf);
        p2buf = strtok(NULL, ",");

        ldpciBitErrPost[i] = RsCore_Convert_String_To_ViInt32(p2buf);
        p2buf = strtok(NULL, ",");

        ldpciBERPost[i] = RsCore_Convert_String_To_ViReal64(p2buf);
        p2buf = strtok(NULL, ",");

        ldpciCWErrPost[i] = RsCore_Convert_String_To_ViInt32(p2buf);
        p2buf = strtok(NULL, ",");

        ldpciCBLErrPost[i] = RsCore_Convert_String_To_ViReal64(p2buf);
        p2buf = strtok(NULL, ",");

        reserved1[i] = RsCore_Convert_String_To_ViInt32(p2buf);
        p2buf = strtok(NULL, ",");

        reserved2[i] = RsCore_Convert_String_To_ViReal64(p2buf);
        p2buf = strtok(NULL, ",");

        reserved3[i] = RsCore_Convert_String_To_ViInt32(p2buf);
        p2buf = strtok(NULL, ",");

        reserved4[i] = RsCore_Convert_String_To_ViReal64(p2buf);
        p2buf = strtok(NULL, ",");

        i++;
    }

    if (returnedValues)
        *returnedValues = i;

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    if (pbuffer) free(pbuffer);
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function returns the following detailed signal content values.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR arraySize/This control defines the size of the arrays.
/// HIPAR returnedValues/Queries number of returned Results
/// HIPAR cwIndex/Returns Array of Codeword indexes (0..1535) Not available for PLC, pilots
/// HIPAR cwIndex/and excluded subcarriers (Remote: -1)
/// HIPAR symbolStart/
/// HIPAR objectInformationType/Returns Array of Information type
/// HIPAR modulation/Returns Array of Modulations
/// HIPAR mer/Returns Array of Modulation error ratios in dB
/// HIPAR power/Returns Array of Powers in dBm
/// HIPAR numberOfSubcarrier/Returns Array of Numbers of subcarrier (0..3800|0..7600)
/// HIPAR ldpcIterations/Returns Array of Low density parity check Number of iterations
/// HIPAR bitErrPre/Returns Array of Low density parity check - Absolute number of
/// HIPAR bitErrPre/bit errors before decoding
/// HIPAR bitErrPreN/Returns Array of Low density parity check - Bit error ratio
/// HIPAR bitErrPreN/before decoding (the ratio of errored bits to the total number
/// HIPAR bitErrPreN/of transmitted bits)
/// HIPAR bitErrPost/Returns Array of Low density parity check - Absolute number of
/// HIPAR bitErrPost/bit errors after decoding
/// HIPAR bitErrPostN/Returns Array of Low density parity check - Bit error rate after
/// HIPAR bitErrPostN/decoding (the ratio of falsely decoded bits to the total number
/// HIPAR bitErrPostN/of transmitted bits)
/// HIPAR cwErrPost/Returns Array of Low density parity check - Absolute number of
/// HIPAR cwErrPost/codeword errors after decoding
/// HIPAR cwErrPostN/Returns Array of Low density parity check - Block error rate
/// HIPAR cwErrPostN/after decoding (the ratio of falsely decoded codewords to the
/// HIPAR cwErrPostN/total number of transmitted codewords)
/// HIPAR reserved1/This control is reserved for future use
/// HIPAR reserved2/This control is reserved for future use
/// HIPAR reserved3/This control is reserved for future use
/// HIPAR reserved4/This control is reserved for future use
ViStatus _VI_FUNC rsspecan_FetchDOCSISSignalContentDetailedFormatted(
                            ViSession instrSession,
                            ViInt32   arraySize,
                            ViInt32*  returnedValues,
                            ViInt32   cwIndex[],
                            ViInt32   symbolStart[],
                            ViInt32   objectInformationType[],
                            ViInt32   modulation[],
                            ViReal64  mer[],
                            ViReal64  power[],
                            ViInt32   numberOfSubcarrier[],
                            ViInt32   ldpcIterations[],
                            ViInt32   bitErrPre[],
                            ViReal64  bitErrPreN[],
                            ViInt32   bitErrPost[],
                            ViReal64  bitErrPostN[],
                            ViInt32   cwErrPost[],
                            ViReal64  cwErrPostN[],
                            ViInt32   reserved1[],
                            ViReal64  reserved2[],
                            ViInt32   reserved3[],
                            ViReal64  reserved4[])
{
    ViStatus error = VI_SUCCESS;
    ViChar*  pbuffer = NULL;
    ViChar*  p2buf;
    ViInt32  i = 0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32RangeMin(instrSession, arraySize, 1),
                3, "Array Size");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, returnedValues), 4, "Returned Values");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, cwIndex), 5, "CW Index");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, symbolStart), 6, "Symbol Start");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, objectInformationType), 7, "Object Information Type");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, modulation), 8, "Modulation");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, mer), 9, "MER");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, power), 10, "Power");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, numberOfSubcarrier), 11, "Number Of Subcarrier");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, ldpcIterations), 12, "LDPC Iterations");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, bitErrPre), 13, "Bit Err Pre");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, bitErrPreN), 14, "Bit Err Pre N");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, bitErrPost), 15, "Bit Err Post");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, bitErrPostN), 17, "Bit Err Post N");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, cwErrPost), 18, "CW Err Post");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, cwErrPostN), 19, "CW Err Post N");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, reserved1), 20, "Reserved 1");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, reserved2), 21, "Reserved 2");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, reserved3), 21, "Reserved 3");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, reserved4), 21, "Reserved 4");

    checkErr(RsCore_QueryViStringUnknownLength(instrSession, "FETC:SCD:ALL:FORM?", &pbuffer));

    p2buf = strtok(pbuffer, ",");

    while (p2buf && (i < arraySize))
    {
        cwIndex[i] = RsCore_Convert_String_To_ViInt32(p2buf);
        p2buf = strtok(NULL, ",");

        symbolStart[i] = RsCore_Convert_String_To_ViInt32(p2buf);
        p2buf = strtok(NULL, ",");

        objectInformationType[i] = RsCore_FindStringIndex(gsSCDetailedObjectInformationTypeArr, p2buf);
        p2buf = strtok(NULL, ",");

        modulation[i] = RsCore_FindStringIndex(modulationArr, p2buf);
        p2buf = strtok(NULL, ",\n");

        mer[i] = RsCore_Convert_String_To_ViReal64(p2buf);
        p2buf = strtok(NULL, ",");

        power[i] = RsCore_Convert_String_To_ViReal64(p2buf);
        p2buf = strtok(NULL, ",");

        numberOfSubcarrier[i] = RsCore_Convert_String_To_ViInt32(p2buf);
        p2buf = strtok(NULL, ",");

        ldpcIterations[i] = RsCore_Convert_String_To_ViInt32(p2buf);
        p2buf = strtok(NULL, ",");

        bitErrPre[i] = RsCore_Convert_String_To_ViInt32(p2buf);
        p2buf = strtok(NULL, ",");

        bitErrPreN[i] = RsCore_Convert_String_To_ViReal64(p2buf);
        p2buf = strtok(NULL, ",");

        bitErrPost[i] = RsCore_Convert_String_To_ViInt32(p2buf);
        p2buf = strtok(NULL, ",");

        bitErrPostN[i] = RsCore_Convert_String_To_ViReal64(p2buf);
        p2buf = strtok(NULL, ",");

        cwErrPost[i] = RsCore_Convert_String_To_ViInt32(p2buf);
        p2buf = strtok(NULL, ",");

        cwErrPostN[i] = RsCore_Convert_String_To_ViReal64(p2buf);
        p2buf = strtok(NULL, ",");

        reserved1[i] = RsCore_Convert_String_To_ViInt32(p2buf);
        p2buf = strtok(NULL, ",");

        reserved2[i] = RsCore_Convert_String_To_ViReal64(p2buf);
        p2buf = strtok(NULL, ",");

        reserved3[i] = RsCore_Convert_String_To_ViInt32(p2buf);
        p2buf = strtok(NULL, ",");

        reserved4[i] = RsCore_Convert_String_To_ViReal64(p2buf);
        p2buf = strtok(NULL, ",");

        i++;
    }

    if (returnedValues)
        *returnedValues = i;

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    if (pbuffer) free(pbuffer);
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function returns the decoded payload data stream for each detected codeword
/// HIFN  in the currently captured I/Q data.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR arraySize/This control defines the size of the arrays.
/// HIPAR returnedValues/Queries number of returned Results
/// HIPAR cwIndex/Returns Array of Codeword indexes (0..1535) Not available for PLC, pilots
/// HIPAR cwIndex/and excluded subcarriers (Remote: -1)
/// HIPAR objectInformationType/Returns Array of Information type
/// HIPAR modulation/Returns Array of Modulations
/// HIPAR numberOfBits/Returns Array of total number of bits in object
/// HIPAR numberOfEntries/Returns Array of number of decoded payload bits
/// HIPAR bits/Returns Array of decoded payload bits in hexadecimal format
ViStatus _VI_FUNC rsspecan_FetchDOCSISBitstreamResults(ViSession instrSession,
                                                       ViInt32   arraySize,
                                                       ViInt32*  returnedValues,
                                                       ViInt32   cwIndex[],
                                                       ViInt32   objectInformationType[],
                                                       ViInt32   modulation[],
                                                       ViInt32   numberOfBits[],
                                                       ViInt32   numberOfEntries[],
                                                       ViInt32   bits[])
{
    ViStatus error = VI_SUCCESS;
    ViChar*  pbuffer = NULL;
    ViChar*  p2buf;
    ViInt32  i = 0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32RangeMin(instrSession, arraySize, 1),
                3, "Array Size");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, returnedValues), 4, "Returned Values");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, cwIndex), 5, "CW Index");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, objectInformationType), 6, "Object Information Type");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, modulation), 7, "Modulation");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, numberOfBits), 8, "Number Of Bits");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, numberOfEntries), 9, "Number Of Entries");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, bits), 10, "Bits");

    checkErr(RsCore_QueryViStringUnknownLength(instrSession, "FETC:BITS:ALL?", &pbuffer));

    p2buf = strtok(pbuffer, ",");

    while (p2buf && (i < arraySize))
    {
        cwIndex[i] = RsCore_Convert_String_To_ViInt32(p2buf);
        p2buf = strtok(NULL, ",");

        objectInformationType[i] = RsCore_FindStringIndex(gsObjectInformationTypeArr, p2buf);
        p2buf = strtok(NULL, ",");

        modulation[i] = RsCore_FindStringIndex(modulationArr, p2buf);
        p2buf = strtok(NULL, ",");

        numberOfBits[i] = RsCore_Convert_String_To_ViInt32(p2buf);
        p2buf = strtok(NULL, ",\n");

        numberOfEntries[i] = RsCore_Convert_String_To_ViInt32(p2buf);
        p2buf = strtok(NULL, ",\n");

        bits[i] = RsCore_Convert_String_To_ViInt32(p2buf);
        p2buf = strtok(NULL, ",");

        i++;
    }

    if (returnedValues)
        *returnedValues = i;

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    if (pbuffer) free(pbuffer);
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function returns the following summarized signal content values for
/// HIFN  all codewords.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR arraySize/This control defines the size of the arrays.
/// HIPAR returnedValues/Queries number of returned Results
/// HIPAR objectInformationType/Returns Array of Information type
/// HIPAR modulation/Returns Array of Modulations
/// HIPAR mer/Returns Array of Modulation error ratios in dB
/// HIPAR numberOfObject/Returns Array of Numbers of object
/// HIPAR ldpcIterations/Returns Array of Low density parity check Number of iterations
/// HIPAR bitErrPre/Returns Array of Low density parity check - Absolute number of
/// HIPAR bitErrPre/bit errors before decoding
/// HIPAR bitErrPreN/Returns Array of Low density parity check - Bit error ratio
/// HIPAR bitErrPreN/before decoding (the ratio of errored bits to the total number of
/// HIPAR bitErrPreN/transmitted bits)
/// HIPAR bitErrPost/Returns Array of Low density parity check - Absolute number of
/// HIPAR bitErrPost/bit errors after decoding
/// HIPAR bitErrPostN/Returns Array of Low density parity check - Bit error rate after
/// HIPAR bitErrPostN/decoding (the ratio of falsely decoded bits to the total number
/// HIPAR bitErrPostN/of transmitted bits)
/// HIPAR cwErrPost/Returns Array of Low density parity check - Absolute number of
/// HIPAR cwErrPost/codeword errors after decoding
/// HIPAR cwErrPostN/Returns Array of Low density parity check - Block error rate
/// HIPAR cwErrPostN/after decoding (the ratio of falsely decoded codewords to the
/// HIPAR cwErrPostN/total number of transmitted codewords)
ViStatus _VI_FUNC rsspecan_FetchDOCSISSignalContentSummary(ViSession instrSession,
                                                           ViInt32   arraySize,
                                                           ViInt32*  returnedValues,
                                                           ViInt32   objectInformationType[],
                                                           ViInt32   modulation[],
                                                           ViReal64  mer[],
                                                           ViInt32   numberOfObject[],
                                                           ViInt32   ldpcIterations[],
                                                           ViInt32   bitErrPre[],
                                                           ViReal64  bitErrPreN[],
                                                           ViInt32   bitErrPost[],
                                                           ViReal64  bitErrPostN[],
                                                           ViInt32   cwErrPost[],
                                                           ViReal64  cwErrPostN[])
{
    ViStatus error = VI_SUCCESS;
    ViChar*  pbuffer = NULL;
    ViChar*  p2buf;
    ViInt32  i = 0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32RangeMin(instrSession, arraySize, 1),
                3, "Array Size");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, returnedValues), 4, "Returned Values");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, objectInformationType), 5, "Object Information Type");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, modulation), 6, "Modulation");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, mer), 7, "MER");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, numberOfObject), 8, "Number Of Object");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, ldpcIterations), 9, "LDPC Iterations");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, bitErrPre), 10, "Bit Err Pre");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, bitErrPreN), 11, "Bit Err Pre N");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, bitErrPost), 12, "Bit Err Post");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, bitErrPostN), 13, "Bit Err Post N");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, cwErrPost), 14, "CW Err Post");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, cwErrPostN), 15, "CW Err Post N");

    checkErr(RsCore_QueryViStringUnknownLength(instrSession, "FETC:SCS:ALL?", &pbuffer));

    p2buf = strtok(pbuffer, ",");

    while (p2buf && (i < arraySize))
    {
        objectInformationType[i] = RsCore_FindStringIndex(objectInformationArr, p2buf);
        p2buf = strtok(NULL, ",");

        modulation[i] = RsCore_FindStringIndex(modulationArr, p2buf);
        p2buf = strtok(NULL, ",");

        mer[i] = RsCore_Convert_String_To_ViReal64(p2buf);
        p2buf = strtok(NULL, ",");

        numberOfObject[i] = RsCore_Convert_String_To_ViInt32(p2buf);
        p2buf = strtok(NULL, ",\n");

        bitErrPre[i] = RsCore_Convert_String_To_ViInt32(p2buf);
        p2buf = strtok(NULL, ",");

        bitErrPreN[i] = RsCore_Convert_String_To_ViReal64(p2buf);
        p2buf = strtok(NULL, ",");

        bitErrPost[i] = RsCore_Convert_String_To_ViInt32(p2buf);
        p2buf = strtok(NULL, ",");

        bitErrPostN[i] = RsCore_Convert_String_To_ViReal64(p2buf);
        p2buf = strtok(NULL, ",");

        cwErrPost[i] = RsCore_Convert_String_To_ViInt32(p2buf);
        p2buf = strtok(NULL, ",");

        cwErrPostN[i] = RsCore_Convert_String_To_ViReal64(p2buf);
        p2buf = strtok(NULL, ",");

        i++;
    }

    if (returnedValues)
    {
        *returnedValues = i;
    }

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    if (pbuffer) free(pbuffer);
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function returns the number of analyzed frames from the current capture buffer.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR frameCount/This control returns the number of analyzed frames from the
/// HIPAR frameCount/current capture buffer.
ViStatus _VI_FUNC rsspecan_FetchDOCSISFrameCount(ViSession instrSession,
                                                 ViInt32*  frameCount)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_GetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_DOCSIS_FETCH_FRAME_COUNT, frameCount),
            2, "Frame Count");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function returns the number of analyzed frames for the entire measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR frameCountAll/This control  returns the number of analyzed frames for the
/// HIPAR frameCountAll/entire measurement.
ViStatus _VI_FUNC rsspecan_FetchDOCSISFrameCountAll(ViSession instrSession,
                                                    ViInt32*  frameCountAll)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_GetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_DOCSIS_FETCH_FRAME_COUNT_ALL, frameCountAll),
            2, "Frame Count All");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the automatically determined or specified cyclic prefix for
/// HIFN  the signal description.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR cyclicPrefixCP/Queries the automatically determined or specified cyclic prefix for
/// HIPAR cyclicPrefixCP/the signal description.
ViStatus _VI_FUNC rsspecan_FetchDOCSISCyclicPrefixCP(ViSession instrSession,
                                                     ViInt32*  cyclicPrefixCP)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_GetAttributeViInt32(instrSession, NULL, RSSPECAN_ATTR_DOCSIS_FETCH_CYCLIC_PREFIX_CP, cyclicPrefixCP),
            2, "Cyclic Prefix CP");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function queries the band power result of the specified band.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR band/This control sets the user-configured power band.
/// HIPAR result/This control returns the band power result of the specified band.
ViStatus _VI_FUNC rsspecan_QueryDOCSISSynchronousPowerBandResult(
                            ViSession instrSession,
                            ViInt32   band,
                            ViReal64* result)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, band, 1, 20),
            2, "Band");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_DOCSIS_SYNCHRONOUS_POWER_BAND_RESULTS,
            "DOCSISBand", band - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Band");

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_DOCSIS_SYNCHRONOUS_POWER_BAND_RESULTS, result),
            3, "Result");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the value of a marker in 3-dimensional result displays ('MER
/// HIFN  vs Symbol X Carrier', 'Power vs Symbol X Carrier'). To get a valid
/// HIFN  result, you have to perform a complete measurement with synchronization
/// HIFN  to the end of the measurement before reading out the result. This is
/// HIFN  only possible for single measurement mode.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR marker/This control selects marker.
/// HIPAR zAxis/Returns the value of a marker in 3-dimensional result displays ('MER
/// HIPAR zAxis/vs Symbol X Carrier', 'Power vs Symbol X Carrier').
ViStatus _VI_FUNC rsspecan_QueryDOCSISMarkerZAxis(ViSession instrSession,
                                                  ViInt32   window,
                                                  ViInt32   marker,
                                                  ViReal64* zAxis)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_marker[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_DOCSIS_MARKER_Z_AXIS,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap));
    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_DOCSIS_MARKER_Z_AXIS,
            "Marker", marker - 1, RS_REPCAP_BUF_SIZE, repCap_marker));

    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_marker, NULL);

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_DOCSIS_MARKER_Z_AXIS, zAxis),
            4, "Z Axis");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function returns the time the PLC was created.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR timestamp/Returns the time the PLC was created.
ViStatus _VI_FUNC rsspecan_FetchDOCSISPLCTimestamp(ViSession instrSession,
                                                   ViInt32*  timestamp)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_GetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_DOCSIS_FETCH_PLC_TIMESTAMP, timestamp),
            2, "Timestamp");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function returns the DOCSIS 3.1 OFDM Channel Information
/// HIFN from PLC message.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR spectrumLocation/Returns the center frequency of the subcarrier 0 of the OFDM
/// HIPAR spectrumLocation/channel (fsc0) from the PLC messages
/// HIPAR FFTLength/Returns the length of the FFT defining the OFDM transmission
/// HIPAR FFTLength/from the PLC messages.
/// HIPAR cyclicPrefixCP/Returns the length of the configurable cyclic prefix from the
/// HIPAR cyclicPrefixCP/PLC messages.
/// HIPAR rollOff/Returns  the roll-off period for the Tukey raised-cosine window
/// HIPAR rollOff/from the PLC messages.
/// HIPAR timeInterleavingDepth/Returns the maximum number of delay lines used for time
/// HIPAR timeInterleavingDepth/interleaving from the PLC messages.
/// HIPAR startIndexL/Returns the starting subcarrier index of the PLC from the PLC
/// HIPAR startIndexL/messages.
/// HIPAR channelID/Returns  the OFDM downstream channel ID from the PLC messages.
/// HIPAR changeCount/Returns the OFDM downstream configuration change count from the
/// HIPAR changeCount/PLC messages.
ViStatus _VI_FUNC rsspecan_FetchDOCSISPLCOFDMChannelInformation(
                            ViSession instrSession,
                            ViReal64* spectrumLocation,
                            ViInt32*  FFTLength,
                            ViInt32*  cyclicPrefixCP,
                            ViInt32*  rollOff,
                            ViInt32*  timeInterleavingDepth,
                            ViInt32*  startIndexL,
                            ViInt32*  channelID,
                            ViInt32*  changeCount)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_DOCSIS_FETCH_PLC_OFDM_SPECTRUM_LOCATION, spectrumLocation),
            2, "Spectrum Location");

    viCheckParm(rsspecan_GetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_DOCSIS_FETCH_PLC_OFDM_FFT_LENGTH, FFTLength),
            3, "FFT Length");

    viCheckParm(rsspecan_GetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_DOCSIS_FETCH_PLC_OFDM_CYCLIC_PREFIX_LENGTH, cyclicPrefixCP),
            4, "Cyclic Prefix CP");

    viCheckParm(rsspecan_GetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_DOCSIS_FETCH_PLC_OFDM_ROLL_OFF, rollOff),
            5, "Roll Off");

    viCheckParm(rsspecan_GetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_DOCSIS_FETCH_PLC_OFDM_TIME_INTERLEAVING_DEPTH, timeInterleavingDepth),
            6, "Time Interleaving Depth");

    viCheckParm(rsspecan_GetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_DOCSIS_FETCH_PLC_OFDM_START_INDEX_L, startIndexL),
            6, "Start Index L");

    viCheckParm(rsspecan_GetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_DOCSIS_FETCH_PLC_OFDM_CHANNEL_ID, channelID),
            7, "Channel ID");

    viCheckParm(rsspecan_GetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_DOCSIS_FETCH_PLC_OFDM_CHANGE_COUNT, changeCount),
            8, "Change Count");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function returns the subcarriers used as continuous pilots
/// HIFN from the PLC messages.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR arraySize/This control defines the size of the array Subcarriers.
/// HIPAR subcarriers/Returns the array of subcarriers.
/// HIPAR returnedValues/Returns number of returned values.
ViStatus _VI_FUNC rsspecan_FetchDOCSISPLCOFDMPilotsSubcarriers(
                            ViSession instrSession,
                            ViInt32   arraySize,
                            ViInt32   subcarriers[],
                            ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;
    ViChar*  pbuffer = NULL;
    ViChar*  p2buf;
    ViInt32  i = 0;
    ViInt32  start;
    ViInt32  stop;
    ViInt32  inc;
    ViInt32  val;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_QueryViStringUnknownLength(instrSession, "FETC:PLCM:OCD:CPIL?", &pbuffer));

    checkErr(rsspecan_CheckStatus(instrSession));

    p2buf = strtok(pbuffer, ",");
    while (p2buf)
    {
        if (strstr(p2buf, ":"))
        {
            //set returned, generate subcarriers
            if (sscanf(p2buf, "%ld:%ld:%ld", &start, &inc, &stop) != 3)
            {
                checkErr(VI_ERROR_INV_RESPONSE);
            }
            else
            {
                for (val = start; val <= stop; val = val + inc)
                    if (i < arraySize)
                        subcarriers[i++] = val;
            }
        }
        else
        {
            //one subcarrier
            if (sscanf(p2buf, "%ld", &val) != 1)
            {
                checkErr(RS_ERROR_UNEXPECTED_RESPONSE);
            }
            if (i < arraySize)
                subcarriers[i++] = val;
        }
        //next subcarrier
        p2buf = strtok(NULL, ",\n");
    }

    if (returnedValues != NULL)
        *returnedValues = i;

Error:
    if (pbuffer) free(pbuffer);
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function returns excluded subcarriers from the PLC
/// HIFN messages.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR arraySize/This control defines the size of the array Subcarriers.
/// HIPAR subcarriers/Returns the array of subcarriers.
/// HIPAR returnedValues/Returns number of returned values.
ViStatus _VI_FUNC rsspecan_FetchDOCSISPLCOFDMExcludedSubcarriers(
                            ViSession instrSession,
                            ViInt32   arraySize,
                            ViInt32   subcarriers[],
                            ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;
    ViChar*  pbuffer = NULL;
    ViChar*  p2buf;
    ViInt32  i = 0;
    ViInt32  start;
    ViInt32  stop;
    ViInt32  inc;
    ViInt32  val;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_QueryViStringUnknownLength(instrSession, "FETC:PLCM:OCD:ESUB?", &pbuffer));

    checkErr(rsspecan_CheckStatus(instrSession));

    p2buf = strtok(pbuffer, ",");
    while (p2buf)
    {
        if (strstr(p2buf, ":"))
        {
            //set returned, generate subcarriers
            if (sscanf(p2buf, "%ld:%ld:%ld", &start, &inc, &stop) != 3)
            {
                checkErr(RS_ERROR_UNEXPECTED_RESPONSE);
            }
            else
            {
                for (val = start; val <= stop; val = val + inc)
                    if (i < arraySize)
                        subcarriers[i++] = val;
            }
        }
        else
        {
            //one subcarrier
            if (sscanf(p2buf, "%ld", &val) != 1)
            {
                checkErr(RS_ERROR_UNEXPECTED_RESPONSE);
            }
            if (i < arraySize)
                subcarriers[i++] = val;
        }
        //next subcarrier
        p2buf = strtok(NULL, ",\n");
    }

    if (returnedValues != NULL)
        *returnedValues = i;

Error:
    if (pbuffer) free(pbuffer);
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function returns the codeword (NCP) information.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR channelID/Returns the NCP downstream channel ID from the PLC messages.
/// HIPAR changeCount/Returns the NCP configuration change count from the PLC
/// HIPAR changeCount/messages.
/// HIPAR modulation/Returns the NCP downstream channel ID from the PLC messages.
ViStatus _VI_FUNC rsspecan_FetchDOCSISPLCNCPInformation(ViSession instrSession,
                                                        ViInt32*  channelID,
                                                        ViInt32*  changeCount,
                                                        ViInt32*  modulation)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_GetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_DOCSIS_FETCH_PLC_NCP_CHANNEL_ID, channelID),
            2, "Channel ID");

    viCheckParm(rsspecan_GetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_DOCSIS_FETCH_PLC_NCP_CHANGE_COUNT, changeCount),
            3, "Change Count");

    viCheckParm(rsspecan_GetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_DOCSIS_FETCH_PLC_NCP_MODULATION, modulation),
            4, "Modulation");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function returns the subcarriers to codeword assignment
/// HIFN defined by the Next Codeword Pointer
/// HIFN (NCP) from the PLC messages.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR instrSession/Document parameter here.
/// HIPAR arraySize/This control defines the size of the array Subcarriers.
/// HIPAR subcarriers/Returns the array of subcarriers.
/// HIPAR returnedValues/Returns number of returned values.
ViStatus _VI_FUNC rsspecan_FetchDOCSISPLCNCPSubcarriers(ViSession instrSession,
                                                        ViInt32   arraySize,
                                                        ViInt32   subcarriers[],
                                                        ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;
    ViChar*  pbuffer = NULL;
    ViChar*  p2buf;
    ViInt32  i = 0;
    ViInt32  start;
    ViInt32  stop;
    ViInt32  inc;
    ViInt32  val;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_QueryViStringUnknownLength(instrSession, "FETC:PLCM:NCP:ASS?", &pbuffer));

    checkErr(rsspecan_CheckStatus(instrSession));

    p2buf = strtok(pbuffer, ",");
    while (p2buf)
    {
        if (strstr(p2buf, ":"))
        {
            //set returned, generate subcarriers
            if (sscanf(p2buf, "%ld:%ld:%ld", &start, &inc, &stop) != 3)
            {
                checkErr(RS_ERROR_UNEXPECTED_RESPONSE);
            }
            else
            {
                for (val = start; val <= stop; val = val + inc)
                    if (i < arraySize)
                        subcarriers[i++] = val;
            }
        }
        else
        {
            //one subcarrier
            if (sscanf(p2buf, "%ld", &val) != 1)
            {
                checkErr(RS_ERROR_UNEXPECTED_RESPONSE);
            }
            if (i < arraySize)
                subcarriers[i++] = val;
        }
        //next subcarrier
        p2buf = strtok(NULL, ",\n");
    }

    if (returnedValues != NULL)
        *returnedValues = i;

Error:
    if (pbuffer) free(pbuffer);
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function returns the profile to subcarrier assignment from
/// HIFN the PLC messages.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR setIndex/Control sets the index.
/// HIPAR arraySize/This control defines the size of the array Modulation.
/// HIPAR modulation/Returns the array of available modulations.
/// HIPAR returnedValues/Returns number of returned values.
ViStatus _VI_FUNC rsspecan_FetchDOCSISPLCProfileSubcarrierModulation(
                            ViSession instrSession,
                            ViInt32   setIndex,
                            ViInt32   arraySize,
                            ViInt32   modulation[],
                            ViInt32*  returnedValues)
{
    ViStatus  error = VI_SUCCESS;
    ViChar    cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViChar*   pbuffer = NULL;
    ViChar*   p2buf;
    ViInt32   i = 0;
    ViInt32   temp_modulation;
    ViInt32   index;
    ViBoolean already_found = VI_FALSE;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, setIndex, 1, 200),
            2, "Set Index");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "FETC:PLCM:PROF%ld:ASS?", setIndex);
    checkErr(RsCore_QueryViStringUnknownLength(instrSession, cmd, &pbuffer));

    checkErr(rsspecan_CheckStatus(instrSession));

    p2buf = strtok(pbuffer, ",");
    while (p2buf && i < arraySize)
    {
        temp_modulation = RsCore_FindStringIndex(DOCSISModulationArr, p2buf);

        if (temp_modulation != -1)
        {
            already_found = VI_FALSE;

            for (index = 0; index < i; index++)
            {
                if (modulation[index] == temp_modulation)
                    already_found = VI_TRUE;
            }

            if (already_found == VI_FALSE)
            {
                modulation[i++] = temp_modulation;
            }
        }

        //next subcarrier
        p2buf = strtok(NULL, ",\n");
    }

    if (returnedValues != NULL)
        *returnedValues = i;

Error:
    if (pbuffer) free(pbuffer);
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function returns the profile to subcarrier assignment from
/// HIFN the PLC messages.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR setIndex/Control sets the index.
/// HIPAR arraySize/This control defines the size of the array Subcarriers.
/// HIPAR modulation/Select the modulation to query its subcarrier numbers.
/// HIPAR subcarriers/Returns the array of subcarriers.
/// HIPAR returnedValues/Returns number of returned values.
ViStatus _VI_FUNC rsspecan_FetchDOCSISPLCProfileSubcarriers(ViSession instrSession,
                                                            ViInt32   setIndex,
                                                            ViInt32   arraySize,
                                                            ViInt32   modulation,
                                                            ViInt32   subcarriers[],
                                                            ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViChar*  pbuffer = NULL;
    ViChar*  p2buf;
    ViInt32  i = 0;
    ViInt32  start;
    ViInt32  stop;
    ViInt32  inc;
    ViInt32  val;
    ViInt32  temp_modulation;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, setIndex, 1, 200),
            2, "Set Index");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "FETC:PLCM:PROF%ld:ASS?", setIndex);
    checkErr(RsCore_QueryViStringUnknownLength(instrSession, cmd, &pbuffer));

    checkErr(rsspecan_CheckStatus(instrSession));

    p2buf = strtok(pbuffer, ",");

    while (p2buf && i < arraySize)
    {
        temp_modulation = RsCore_FindStringIndex(DOCSISModulationArr, p2buf);

        //modulation found - not subcarrier number
        if (temp_modulation != -1)
        {
            //this modulation was requested
            if (temp_modulation == modulation)
            {
                p2buf = strtok(NULL, ",");

                while (p2buf != NULL && (p2buf[0] >= '0' && p2buf[0] <= '9' && i < arraySize)) //isdigit (atoi (p2buf))
                {
                    if (strstr(p2buf, ":"))
                    {
                        //set returned, generate subcarriers
                        if (sscanf(p2buf, "%ld:%ld:%ld", &start, &inc, &stop) != 3)
                        {
                            checkErr(RS_ERROR_UNEXPECTED_RESPONSE);
                        }
                        else
                        {
                            for (val = start; val <= stop; val = val + inc)
                                if (i < arraySize)
                                    subcarriers[i++] = val;
                        }
                    }
                    else
                    {
                        //one subcarrier
                        if (sscanf(p2buf, "%ld", &val) != 1)
                        {
                            checkErr(RS_ERROR_UNEXPECTED_RESPONSE);
                        }
                        if (i < arraySize)
                            subcarriers[i++] = val;
                    }
                    //next subcarrier
                    p2buf = strtok(NULL, ",\n");
                }
            }
        }

        if (temp_modulation != modulation)
            p2buf = strtok(NULL, ",\n");
    }

    if (returnedValues != NULL)
        *returnedValues = i;

Error:
    if (pbuffer) free(pbuffer);
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function returns the profile information from PLC messages.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR setIndex/Control sets the index.
/// HIPAR channelID/Returns the NCP downstream channel ID from the PLC messages.
/// HIPAR changeCount/Returns the NCP configuration change count from the PLC messages.
ViStatus _VI_FUNC rsspecan_FetchDOCSISPLCProfileInformation(ViSession instrSession,
                                                            ViInt32   setIndex,
                                                            ViInt32*  channelID,
                                                            ViInt32*  changeCount)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, setIndex, 1, 200),
            2, "Set Index");
    snprintf(repCap, RS_REPCAP_BUF_SIZE, "I%ld", setIndex);

    viCheckParm(rsspecan_GetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_DOCSIS_FETCH_PLC_PROFILE_CHANNEL_ID, channelID),
            3, "Channel ID");

    viCheckParm(rsspecan_GetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_DOCSIS_FETCH_PLC_PROFILE_CHANGE_COUNT, changeCount),
            4, "Change Count");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function queries the average and maximum sample/symbol clock
/// HIFN  error limit for all frames as determined by the default DOCSIS 3.1
/// HIFN  measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR average/This control queries the average sample/symbol clock error limit for
/// HIPAR average/all frames as determined by the default DOCSIS 3.1 measurement.
/// HIPAR maximum/This control queries the maximum sample/symbol clock error limit for
/// HIPAR maximum/all frames as determined by the default DOCSIS 3.1 measurement.
ViStatus _VI_FUNC rsspecan_QueryDOCSISClockErrorLimit(ViSession instrSession,
                                                      ViReal64* average,
                                                      ViReal64* maximum)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_DOCSIS_QUERY_CLOCK_ERROR_LIMIT_AVERAGE, average),
            2, "Average");

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_DOCSIS_QUERY_CLOCK_ERROR_LIMIT_MAXIMUM, maximum),
            3, "Maximum");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function returns the result of the limit check for the specified
/// HIFN  parameter.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR average/This control returns the result of the limit check for the specified
/// HIPAR average/parameter.
/// HIPAR maximum/This control returns the result of the limit check for the specified
/// HIPAR maximum/parameter.
ViStatus _VI_FUNC rsspecan_QueryDOCSISClockErrorLimitCheckResult(
                            ViSession instrSession,
                            ViInt32*  average,
                            ViInt32*  maximum)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_GetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_DOCSIS_QUERY_CLOCK_ERROR_LIMIT_AVERAGE_RESULT, average),
            2, "Average");

    viCheckParm(rsspecan_GetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_DOCSIS_QUERY_CLOCK_ERROR_LIMIT_CHECK_MAXIMUM_RESULT, maximum),
            3, "Maximum");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This attribute queries the average and maximum frequency error limit
/// HIFN  for all frames as determined by the default DOCSIS 3.1 measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR maximum/This control queries the maximum sample/symbol clock error limit for
/// HIPAR maximum/all frames as determined by the default DOCSIS 3.1 measurement.
/// HIPAR average/This control queries the average sample/symbol clock error limit for
/// HIPAR average/all frames as determined by the default DOCSIS 3.1 measurement.
ViStatus _VI_FUNC rsspecan_QueryDOCSISFrequencyErrorLimit(ViSession instrSession,
                                                          ViReal64* average,
                                                          ViReal64* maximum)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_DOCSIS_QUERY_FREQUENCY_ERROR_LIMIT_AVERAGE, average),
            2, "Average");

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_DOCSIS_QUERY_FREQUENCY_ERROR_LIMIT_MAXIMUM, maximum),
            3, "Maximum");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function returns the result of the limit check for the specified
/// HIFN  parameter.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR average/This control returns the result of the limit check for the specified
/// HIPAR average/parameter.
/// HIPAR maximum/This control returns the result of the limit check for the specified
/// HIPAR maximum/parameter.
ViStatus _VI_FUNC rsspecan_QueryDOCSISFrequencyErrorLimitCheckResult(
                            ViSession instrSession,
                            ViInt32*  average,
                            ViInt32*  maximum)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_GetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_DOCSIS_QUERY_FREQUENCY_ERROR_LIMIT_CHECK_AVERAGE_RESULT, average),
            2, "Average");

    viCheckParm(rsspecan_GetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_DOCSIS_QUERY_FREQUENCY_ERROR_LIMIT_CHECK_MAXIMUM_RESULT, maximum),
            3, "Maximum");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function queries the minimum modulation error ratio (MER) data
/// HIFN  pilot limit for all pilots as determined by the default DOCSIS 3.1
/// HIFN  measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR minimum/This control queries the minimum modulation error ratio (MER) limit
/// HIPAR minimum/for all data + pilot carriers as determined by the default DOCSIS 3.1
/// HIPAR minimum/measurement.
/// HIPAR resultType/Select the Result Type
ViStatus _VI_FUNC rsspecan_QueryDOCSISModulationErrorRatioLimitsMinimum(
                            ViSession instrSession,
                            ViInt32   resultType,
                            ViReal64* minimum)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    switch (resultType)
    {
    case 0:
        viCheckParm(rsspecan_GetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_DOCSIS_QUERY_MODULATION_ERROR_RATIO_LIMIT_MINIMUM, minimum),
                3, "Minimum");
        break;
    case 1:
        viCheckParm(rsspecan_GetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_DOCSIS_QUERY_MODULATION_ERROR_RATIO_DATA_LIMIT_MINIMUM, minimum),
                3, "Minimum");
        break;

    case 2:
        viCheckParm(rsspecan_GetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_DOCSIS_QUERY_MODULATION_ERROR_RATIO_PILOT_LIMIT_MINIMUM, minimum),
                3, "Minimum");
        break;
    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, resultType), 2, "Result Type");
        break;
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function returns the result of the limit check for the specified
/// HIFN  parameter.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR resultType/Select the Result Type
/// HIPAR minimum/This function returns the result of the limit check for the specified
/// HIPAR minimum/parameter.
ViStatus _VI_FUNC rsspecan_QueryDOCSISModulationErrorRatioLimitsMinimumResult(
                            ViSession instrSession,
                            ViInt32   resultType,
                            ViInt32*  minimum)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    switch (resultType)
    {
    case 0:
        viCheckParm(rsspecan_GetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_DOCSIS_QUERY_MODULATION_ERROR_RATIO_LIMIT_CHECK_MINIMUM_RESULT, minimum),
                3, "Minimum");
        break;
    case 1:
        viCheckParm(rsspecan_GetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_DOCSIS_QUERY_MODULATION_ERROR_RATIO_DATA_LIMIT_CHECK_MINIMUM_RESULT, minimum),
                3, "Minimum");
        break;

    case 2:
        viCheckParm(rsspecan_GetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_DOCSIS_QUERY_MODULATION_ERROR_RATIO_PILOT_LIMIT_CHECK_MINIMUM_RESULT, minimum),
                3, "Minimum");
        break;
    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, resultType), 2, "Result Type");
        break;
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/*****************************************************************************
 *- End Of RSSpecAn K77 -----------------------------------------------------*
 *****************************************************************************/
