/*****************************************************************************
 *  Rohde&Schwarz Spectrum Analyzer instrument driver - K7 Analog Demod Demodulator
 *  VXIpnp Style Instrument Driver
 *  Original Release: September 2005
 *  By: Martin Koutny (instrument driver core)
 *      Helena Sucha (instrument specific implementation)
 *      Jiri Kominek (DrBase repository & tools)
 *
 *  Should you have any technical questions please contact the hotline of
 *  Rohde & Schwarz Vertriebs-GmbH Rohde & Schwarz Support Center
 *
 *  e-mail: CustomerSupport@rohde-schwarz.com
 *
 *****************************************************************************/

#include <math.h>
#include "rsspecan.h"

/*****************************************************************************
 *- RSSpecAn K7 - Analog Demodulator ----------------------------------------*
 *****************************************************************************/
static ViString resultTypeArr[] = {"OFF", "WRIT", "AVER", "MAXH", "MINH", "VIEW", NULL};

/// HIFN This function selects the Analog Demod Demodulator mode.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.
ViStatus _VI_FUNC rsspecan_ADemodMode(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_FM_DEMOD_MODE, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function activates the Analog Demod demodulator of the instrument.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.
/// HIPAR state/This control sets the analog demodulation state.
ViStatus _VI_FUNC rsspecan_SetADemodState(ViSession instrSession,
                                          ViBoolean state)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_FM_DEMOD_ENABLED, state),
            2, "State");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function can configure the Analog Demodulation application to use
/// HIFN  predefined standard settings. This allows for quick and easy
/// HIFN  configuration for commonly performed measurements.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR operation/This control selects the operation to be performed.
/// HIPAR standard/Specifies the file name. If you have stored the file in a subdirectory
/// HIPAR standard/of the directory mentioned above, you have to include the relative path
/// HIPAR standard/to the file.
ViStatus _VI_FUNC rsspecan_ConfigureAnalogDemodPredefinedStandard(
                            ViSession instrSession,
                            ViInt32   operation,
                            ViString  standard)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    switch (operation)
    {
    case RSSPECAN_VAL_ADEM_PRESET_LOAD:
        viCheckParm(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_ANALOG_DEMOD_PRESET_LOAD, standard),
                3, "Standard");
        break;
    case RSSPECAN_VAL_ADEM_PRESET_STORE:
        viCheckParm(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_ANALOG_DEMOD_PRESET_STORE, standard),
                3, "Standard");
        break;
    case RSSPECAN_VAL_ADEM_PRESET_RESTORE:
        checkErr(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_ANALOG_DEMOD_PRESET_RESTORE, NULL));
        break;
    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, operation), 2, "Operation");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function configures the Analog Demodulation Trigger
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR triggerSource/Specifies the source of the trigger signal that causes the
/// HIPAR triggerSource/analyzer to leave the Wait-For-Trigger state.
/// HIPAR triggerOffset/Sets the length of the trigger delay.
/// HIPAR triggerPolarity/Specifies the polarity for triggering.
/// HIPAR triggerLevel/This command sets the level of the video trigger source.
ViStatus _VI_FUNC rsspecan_ConfigureADemodTrigger(ViSession instrSession,
                                                  ViInt32   triggerSource,
                                                  ViReal64  triggerOffset,
                                                  ViInt32   triggerPolarity,
                                                  ViReal64  triggerLevel)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_FM_DEMOD_TRIGGER_SOURCE, triggerSource),
            2, "Trigger Source");

    if (triggerSource != RSSPECAN_VAL_FM_TRIGGER_SOURCE_TIME)
        viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_TRIGGER_DELAY, triggerOffset),
                3, "Trigger Offset");
    switch (triggerSource)
    {
    case RSSPECAN_VAL_FM_TRIGGER_SOURCE_EXT:
        viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "Port1", RSSPECAN_ATTR_EXTERNAL_TRIGGER_LEVEL_PORT, triggerLevel),
                5, "Trigger Level");
        break;
    case RSSPECAN_VAL_FM_TRIGGER_SOURCE_EXT2:
        viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "Port2", RSSPECAN_ATTR_EXTERNAL_TRIGGER_LEVEL_PORT, triggerLevel),
                5, "Trigger Level");
        break;

    case RSSPECAN_VAL_FM_TRIGGER_SOURCE_EXT3:
        viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "Port3", RSSPECAN_ATTR_EXTERNAL_TRIGGER_LEVEL_PORT, triggerLevel),
                5, "Trigger Level");
        break;
    case RSSPECAN_VAL_FM_TRIGGER_SOURCE_RFP:
        viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_TRIGGER_RFP_LEVEL, triggerLevel),
                5, "Trigger Level");
        break;
    case RSSPECAN_VAL_FM_TRIGGER_SOURCE_IFP:
        viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_TRIGGER_IFP_LEVEL, triggerLevel),
                5, "Trigger Level");
        break;
    case RSSPECAN_VAL_FM_TRIGGER_SOURCE_IQP:
        viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_TRIGGER_IQP_LEVEL, triggerLevel),
                5, "Trigger Level");
        break;
    case RSSPECAN_VAL_FM_TRIGGER_SOURCE_BBP:
        viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_DIGITAL_BASEBAND_INPUT_TRIGGER_LEVEL, triggerLevel),
                5, "Trigger Level");
        break;

    case RSSPECAN_VAL_FM_TRIGGER_SOURCE_FM:
        viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_ANALOG_DEMOD_TRIGGER_FM_LEVEL, triggerLevel),
                5, "Trigger Level");
        break;
    case RSSPECAN_VAL_FM_TRIGGER_SOURCE_AM:
        viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_ANALOG_DEMOD_TRIGGER_AM_LEVEL_ABS, triggerLevel),
                5, "Trigger Level");
        break;
    case RSSPECAN_VAL_FM_TRIGGER_SOURCE_PM:
        viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_ANALOG_DEMOD_TRIGGER_PM_LEVEL, triggerLevel),
                5, "Trigger Level");
        break;
    case RSSPECAN_VAL_FM_TRIGGER_SOURCE_AMR:
        viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_ANALOG_DEMOD_TRIGGER_AM_LEVEL_RELATIVE, triggerLevel),
                5, "Trigger Level");
        break;
    case RSSPECAN_VAL_FM_TRIGGER_SOURCE_GP0:
    case RSSPECAN_VAL_FM_TRIGGER_SOURCE_GP1:
    case RSSPECAN_VAL_FM_TRIGGER_SOURCE_GP2:
    case RSSPECAN_VAL_FM_TRIGGER_SOURCE_GP3:
    case RSSPECAN_VAL_FM_TRIGGER_SOURCE_GP4:
    case RSSPECAN_VAL_FM_TRIGGER_SOURCE_GP5:
    case RSSPECAN_VAL_FM_TRIGGER_SOURCE_AF:
    case RSSPECAN_VAL_FM_TRIGGER_SOURCE_TIME:
    case RSSPECAN_VAL_FM_TRIGGER_SOURCE_PSEN:
        break;
    }

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_TRIGGER_SLOPE, triggerPolarity),
            4, "Trigger Polarity");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function configures the online output of analog demodulator.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR outputState/Enables or disables online demodulation output to the IF/VIDEO/DEMOD
/// HIPAR outputState/output connector on the rear panel of the R&S FSW.
/// HIPAR outputSelection/Selects the result display whose results are output.
/// HIPAR ACCutoffFrequency/Defines the cutoff frequency for the AC highpass filter.
/// HIPAR phonesOutput/In addition to sending the output to the IF/VIDEO/DEMOD OUTPUT connector
/// HIPAR phonesOutput/(on the rear panel of the R&S FSW), it can also be output to headphones
/// HIPAR phonesOutput/connected on the front panel (PHONES connector).
ViStatus _VI_FUNC rsspecan_ConfigureADemodOnlineOutput(ViSession instrSession,
                                                       ViBoolean outputState,
                                                       ViString  outputSelection,
                                                       ViReal64  ACCutoffFrequency,
                                                       ViBoolean phonesOutput)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_ANALOG_DEMOD_OUTPUT_STATE, outputState),
            2, "Output State");

    viCheckParm(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_ANALOG_DEMOD_OUTPUT_SELECTION, outputSelection),
            3, "Output Selection");

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_ANALOG_DEMOD_OUTPUT_CUTOFF_FREQUENCY, ACCutoffFrequency),
            4, "AC Cutoff Frequency");

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_ANALOG_DEMOD_PHONE_OUTPUT, phonesOutput),
            5, "Phones Output");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN This function configures the Analog Demod demodulator.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR sampleRate/Selects the sampling rate for the data acquisition.
/// HIPAR recordLength/Number of samples to be stored in I/Q memory.
/// HIPAR triggerSource/Specifies the source of the trigger signal that causes the
/// HIPAR triggerSource/analyzer to leave the Wait-For-Trigger state.
/// HIPAR triggerSlope/Selecs the trigger slope.
/// HIPAR pretriggerSamples/Number of samples to be used as an offset to the trigger signal.
/// HIPAR numberOfMeas/Number of measurements. The value indicated here is particularly
/// HIPAR numberOfMeas/necessary for the AVERage/MAXHold/MINHold function.
ViStatus _VI_FUNC rsspecan_ConfigureADemodSet(ViSession instrSession,
                                              ViReal64  sampleRate,
                                              ViInt32   recordLength,
                                              ViInt32   triggerSource,
                                              ViInt32   triggerSlope,
                                              ViInt32   pretriggerSamples,
                                              ViInt32   numberOfMeas)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViChar*  p2triggerSource;
    ViChar*  p2triggerSlope;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K7"));

    viCheckParm(RsCore_InvalidViReal64Range(instrSession, sampleRate, 122.0703125, 640.0e+6), 2, "Sample Rate");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, recordLength, 1, 480001),
            3, "Record Length");
    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &rsspecan_rngTriggerSource, triggerSource, &p2triggerSource),
                4, "Trigger Source");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, triggerSlope, 0, 1),
            5, "Trigger Slope");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, numberOfMeas, 0, 32767),
            7, "Number of Measurements");
    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &rsspecan_rngPolarity, triggerSlope, &p2triggerSlope),
                5, "Trigger Slope");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SENS:ADEM:SET %.12lGHZ,%ld,%s,%s,%ld,%ld",
            sampleRate, recordLength, p2triggerSource, p2triggerSlope, pretriggerSamples, numberOfMeas);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function defines the scaling.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR perDivison/This control defines the scaling of the Y axis in the current
/// HIPAR perDivison/unit.
ViStatus _VI_FUNC rsspecan_ConfigureAnalogDemodScaling(ViSession instrSession,
                                                       ViInt32   window,
                                                       ViReal64  perDivison)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_FMDEM_DISPLAY_PDIV,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap),
            2, "Window");

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_FMDEM_DISPLAY_PDIV, perDivison),
            3, "Per Division");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function configures analog demodulator bandwidth parameters.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR demodulationBandwidth/This control sets the bandwidth for analog demodulation.
/// HIPAR demodulationBandwidth/Depending on the selected demodulation bandwidth, the instrument
/// HIPAR demodulationBandwidth/selects the required sampling rate. The available values of the
/// HIPAR demodulationBandwidth/demodulation bandwidths are determined by the sampling rates.
ViStatus _VI_FUNC rsspecan_ConfigureADemodBandwidth(ViSession instrSession,
                                                    ViReal64  demodulationBandwidth)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_FM_DEMOD_BANDWIDTH, demodulationBandwidth),
            2, "Demodulation Bandwidth");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function configures analog demodulator measuring time.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR measuringTime/This control sets the measuring time for the analog  Demodulation.
ViStatus _VI_FUNC rsspecan_ConfigureADemodMeasuringTime(ViSession instrSession,
                                                        ViReal64  measuringTime)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_FM_DEMOD_MEAS_TIME, measuringTime),
            2, "Measuring Time");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function configures the analog demodulation.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR squelchState/Activates the squelch function, i.e. if the signal falls below a
/// HIPAR squelchState/defined threshold (Squelch Level), the demodulated data is
/// HIPAR squelchState/automatically set to 0.
/// HIPAR squelchLevel/Defines the level threshold below which the demodulated data is
/// HIPAR squelchLevel/set to 0 if squelching is enabled.
/// HIPAR AFCoupling/This control selects selects the coupling of the AF path.
/// HIPAR trace/Selects the trace.
ViStatus _VI_FUNC rsspecan_ConfigureAnalogDemod(ViSession instrSession,
                                                ViBoolean squelchState,
                                                ViReal64  squelchLevel,
                                                ViInt32   AFCoupling,
                                                ViInt32   trace)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_ADEM_SELECT_TRACE,
            "Trace", trace - 1, RS_REPCAP_BUF_SIZE, repCap),
            5, "Trace");

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_FM_DEMOD_SQUELCH, squelchState),
            2, "Squelch State");

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_FM_DEMOD_SQUELCH_LEVEL, squelchLevel),
            3, "Squelch Level");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_FM_DEMOD_AF_COUPLING, AFCoupling),
            4, "AF Coupling");

    checkErr(rsspecan_SetAttributeViString(instrSession, repCap, RSSPECAN_ATTR_ADEM_SELECT_TRACE, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines how the reference position in time for 0 rad is determined.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR mode/Defines how the reference position in time for 0 rad is determined.
ViStatus _VI_FUNC rsspecan_ConfigureAnalogDemodZeroPhaseRefPointMode(
                            ViSession instrSession,
                            ViInt32   mode)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, NULL, RSSPECAN_ATTR_ANALOG_DEMOD_ZERO_PHASE_REF_POINT_MODE, mode),
            2, "Mode");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN This function sets PM zero phase reference point
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR referencePoint/The reference point, where the phase is set to 0 rad.
ViStatus _VI_FUNC rsspecan_ConfigureAnalogDemodPMZeroPhaseReferencePoint(
                            ViSession instrSession,
                            ViReal64  referencePoint)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_ANALOG_DEMOD_ZERO_PHASE_REF_POINT, referencePoint),
            2, "Reference Point");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function sets PM phase wrap
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR phaseWrap/Activates / Deactivates the phase wrapping to -PI to +PI (-180
/// HIPAR phaseWrap/to +180).
ViStatus _VI_FUNC rsspecan_ConfigureAnalogDemodPMPhaseWrap(ViSession instrSession,
                                                           ViInt32   phaseWrap)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_ANALOG_DEMOD_PHASE_WRAP, phaseWrap),
            2, "Phase Wrap");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN This function sets analog demodulation parameters such as zoom
/// HIFN state, start time.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR zoomState/This function sets analog demodulation parameters such as zoom
/// HIPAR zoomState/state, start time.
/// HIPAR startTime/Selects the start time for the display of the single measured
/// HIPAR startTime/values of the FM demodulator.
ViStatus _VI_FUNC rsspecan_ConfigureADemodZoom(ViSession instrSession,
                                               ViBoolean zoomState,
                                               ViReal64  startTime)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_FM_DEMOD_ZOOM_ENABLED, zoomState),
            2, "Zoom State");

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_FM_DEMOD_ZOOM_START, startTime),
            3, "Start Time");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function configures the zoom length settings.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR zoomLengthMode/Defines whether the length of the zoom area for the
/// HIPAR zoomLengthMode/analog-demodulated measurement data is defined automatically or
/// HIPAR zoomLengthMode/manually.
/// HIPAR zoomLength/Defines the length of the zoom area for the analog-demodulated
/// HIPAR zoomLength/measurement data manually.
ViStatus _VI_FUNC rsspecan_ConfigureADemodZoomLength(ViSession instrSession,
                                                     ViInt32   zoomLengthMode,
                                                     ViReal64  zoomLength)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_FM_DEMOD_ZOOM_LENGTH_MODE, zoomLengthMode),
            2, "Zoom Length Mode");

    if (zoomLengthMode == RSSPECAN_VAL_MAN)
    {
        viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_FM_DEMOD_ZOOM_LENGTH, zoomLength),
                3, "Zoom Length");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function defines the analog demodulation units. It
/// HIFN specifies,  how the demodulated data is displayed.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR phaseUnit/Sets the default unit for angles.
/// HIPAR THDUnit/This control selects between % and dB for displaying the
/// HIPAR THDUnit/THD measurement result.
ViStatus _VI_FUNC rsspecan_ConfigureADemodUnit(ViSession instrSession,
                                               ViInt32   phaseUnit,
                                               ViInt32   THDUnit)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_ANALOG_DEMOD_UNIT_ANGLE, phaseUnit),
            2, "Phase Unit");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_ANALOG_DEMOD_THD_UNIT, THDUnit),
            3, "THD Unit");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function selects the unit for relative demodulation results.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR relativeUnit/This control selects the unit for relative demodulation results.
ViStatus _VI_FUNC rsspecan_ConfigureADemodRelativeUnit(ViSession instrSession,
                                                       ViInt32   relativeUnit)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_ANALOG_DEMOD_RELATIVE_UNIT, relativeUnit),
            2, "Relative Unit");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN Select Analog Demod Result
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR operatingMode/This control selects the type of evaluation for the measured
/// HIPAR operatingMode/data.
/// HIPAR trace/This control sets the reference trace for modes.
ViStatus _VI_FUNC rsspecan_SelectADemodResult(ViSession instrSession,
                                              ViInt32   operatingMode,
                                              ViInt32   trace)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViChar*  p2string;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K7"));
    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &rsspecan_rngAdemOperatingMode, operatingMode, &p2string),
                2, "Operating Mode");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, trace, 1, 6),
            3, "Trace");

    switch (operatingMode)
    {
    case RSSPECAN_VAL_FM_FEED_AM_REL:
    case RSSPECAN_VAL_FM_FEED_AM:
    case RSSPECAN_VAL_FM_FEED_SPEC:
    case RSSPECAN_VAL_FM_FEED_FM:
    case RSSPECAN_VAL_FM_FEED_PM:
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, ":CALC1:FEED '%s'", p2string);
        break;
    case RSSPECAN_VAL_FM_FEED_AMS_REL:
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, ":CALC1:FEED '%s%ld:REL'", p2string, trace);
        break;
    case RSSPECAN_VAL_FM_FEED_AMS:
    case RSSPECAN_VAL_FM_FEED_FMS:
    case RSSPECAN_VAL_FM_FEED_PMS:
    case RSSPECAN_VAL_FM_FEED_FM_AFSP:
    case RSSPECAN_VAL_FM_FEED_PM_AFSP:
    case RSSPECAN_VAL_FM_FEED_AM_REL_AFSP:
    case RSSPECAN_VAL_FSM_FEED_SUMM:
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, ":CALC1:FEED '%s%ld'", p2string, trace);
        break;
    }

    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function defines the mode with which the demodulation result is
/// HIFN  determined.
/// HIFN
/// HIFN     Notes:
/// HIFN
/// HIFN     (1) This function is available only in conjunction with option FS-K7.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR demodulationType/This parameter selects the demodulation.
/// HIPAR detectorType/This control selects the detector type.
/// HIPAR mode/This control defines the mode with which the demodulation result is
/// HIPAR mode/determined.
ViStatus _VI_FUNC rsspecan_ConfigureADemodDetectorMode(ViSession instrSession,
                                                       ViInt32   demodulationType,
                                                       ViInt32   detectorType,
                                                       ViInt32   mode)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViAttr   attr[3] = {
             RSSPECAN_ATTR_ANALOG_DEMOD_AM_DETECTOR_MODE,
             RSSPECAN_ATTR_ANALOG_DEMOD_FM_DETECTOR_MODE,
             RSSPECAN_ATTR_ANALOG_DEMOD_PM_DETECTOR_MODE
             };

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, demodulationType, RSSPECAN_VAL_FMDEM_AM, RSSPECAN_VAL_FMDEM_PM),
            2, "Demodulation Type");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, detectorType, RSSPECAN_VAL_FMDEM_DET_PPE, RSSPECAN_VAL_FMDEM_DET_RMS),
            3, "Deterctor Type");

    checkErr(RsCore_GetAttributeRepCapName(instrSession, attr[demodulationType],
            "Detector", detectorType , RS_REPCAP_BUF_SIZE, repCap));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, attr[demodulationType], mode),
            4, "Mode");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function defines the mode with which the demodulation result is
/// HIFN  determined.
/// HIFN
/// HIFN     Notes:
/// HIFN
/// HIFN     (1) This function is available only in conjunction with option FS-K7.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR demodulationType/This parameter selects the demodulation.
/// HIPAR detectorType/This control selects the detector type.
/// HIPAR detectorEnable/Activates relative demodulation for the selected detector.
ViStatus _VI_FUNC rsspecan_ConfigureADemodDetectorState(ViSession instrSession,
                                                        ViInt32   demodulationType,
                                                        ViInt32   detectorType,
                                                        ViBoolean detectorEnable)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViAttr   attr[3] = {
             RSSPECAN_ATTR_ANALOG_DEMOD_AM_DETECTOR_STATE,
             RSSPECAN_ATTR_ANALOG_DEMOD_FM_DETECTOR_STATE,
             RSSPECAN_ATTR_ANALOG_DEMOD_PM_DETECTOR_STATE
             };

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, demodulationType, RSSPECAN_VAL_FMDEM_AM, RSSPECAN_VAL_FMDEM_PM),
            2, "Demodulation Type");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, detectorType, RSSPECAN_VAL_FMDEM_DET_PPE, RSSPECAN_VAL_FMDEM_DET_RMS),
            3, "Deterctor Type");

    checkErr(RsCore_GetAttributeRepCapName(instrSession, attr[demodulationType],
            "Detector", detectorType , RS_REPCAP_BUF_SIZE, repCap));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, repCap, attr[demodulationType], detectorEnable),
            4, "Detector Enable");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function defines the reference value to be used for relative
/// HIFN  demodulation results and recalculates the results.
/// HIFN
/// HIFN     Notes:
/// HIFN
/// HIFN     (1) This function is available only in conjunction with option FS-K7.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR demodulationType/This parameter selects the demodulation.
/// HIPAR detectorType/This control selects the detector type.
/// HIPAR reference/Defines the reference value to be used for relative demodulation
/// HIPAR reference/results and recalculates the results. If necessary, the detector is
/// HIPAR reference/activated. A reference value 0 would provide infinite results and is
/// HIPAR reference/thus automatically corrected to 0.1. The unit is %.
ViStatus _VI_FUNC rsspecan_ConfigureADemodDetectorReference(ViSession instrSession,
                                                            ViInt32   demodulationType,
                                                            ViInt32   detectorType,
                                                            ViReal64  reference)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViAttr   attr[3] = {
             RSSPECAN_ATTR_ANALOG_DEMOD_AM_DETECTOR_REFERENCE_VALUE,
             RSSPECAN_ATTR_ANALOG_DEMOD_FM_DETECTOR_REFERENCE_VALUE,
             RSSPECAN_ATTR_ANALOG_DEMOD_PM_DETECTOR_REFERENCE_VALUE
             };

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, demodulationType, RSSPECAN_VAL_FMDEM_AM, RSSPECAN_VAL_FMDEM_PM),
            2, "Demodulation Type");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, detectorType, RSSPECAN_VAL_FMDEM_DET_PPE, RSSPECAN_VAL_FMDEM_DET_RMS),
            3, "Deterctor Type");

    checkErr(RsCore_GetAttributeRepCapName(instrSession, attr[demodulationType],
            "Detector", detectorType , RS_REPCAP_BUF_SIZE, repCap));

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, repCap, attr[demodulationType], reference),
            4, "Reference");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function sets the reference value to be used for relative
/// HIFN  demodulation results to the currently measured value for all relative
/// HIFN  detectors.
/// HIFN
/// HIFN     Notes:
/// HIFN
/// HIFN     (1) This function is available only in conjunction with option FS-K7.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR demodulationType/This parameter selects the demodulation.
/// HIPAR detectorType/This control selects the detector type.
ViStatus _VI_FUNC rsspecan_ConfigureADemodDetectorMeasToRef(ViSession instrSession,
                                                            ViInt32   demodulationType,
                                                            ViInt32   detectorType)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViAttr   attr[3] = {
             RSSPECAN_ATTR_ANALOG_DEMOD_AM_DETECTOR_MEAS_TO_REF,
             RSSPECAN_ATTR_ANALOG_DEMOD_FM_DETECTOR_MEAS_TO_REF,
             RSSPECAN_ATTR_ANALOG_DEMOD_PM_DETECTOR_MEAS_TO_REF
             };

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, demodulationType, RSSPECAN_VAL_FMDEM_AM, RSSPECAN_VAL_FMDEM_PM),
            2, "Demodulation Type");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, detectorType, RSSPECAN_VAL_FMDEM_DET_PPE, RSSPECAN_VAL_FMDEM_DET_RMS),
            3, "Deterctor Type");

    checkErr(RsCore_GetAttributeRepCapName(instrSession, attr[demodulationType],
            "Detector", detectorType , RS_REPCAP_BUF_SIZE, repCap));

    checkErr(rsspecan_SetAttributeViString(instrSession, repCap, attr[demodulationType], NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function configures the frequency range of the result
/// HIFN display AF spectrum using start frequency and stop frequency.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR startFrequency/Sets the start frequency for result display AF spectrum.
/// HIPAR stopFrequency/Sets the stop frequency for result display AF spectrum.
/// HIPAR stopFrequency/The stop frequency is limited to half the measurement bandwidth
/// HIPAR stopFrequency/of analog demodulation.
ViStatus _VI_FUNC rsspecan_ConfigureADemodAFFrequencyStartStop(
                            ViSession instrSession,
                            ViReal64  startFrequency,
                            ViReal64  stopFrequency)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_FM_DEMOD_AF_START_FREQUENCY, startFrequency),
            2, "Start Frequency");

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_FM_DEMOD_AF_STOP_FREQUENCY, stopFrequency),
            3, "Stop Frequency");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the frequency range of the result
/// HIFN  display AF spectrum using the center frequency and the frequency
/// HIFN  span.
/// HIRET Document return value here.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR centerFrequency/Sets the center frequency for result display AF spectrum.
/// HIPAR span/Sets the span for result display AF spectrum.
ViStatus _VI_FUNC rsspecan_ConfigureADemodAFFrequencyCenterSpan(
                            ViSession instrSession,
                            ViReal64  centerFrequency,
                            ViReal64  span)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_FM_DEMOD_AF_CENTER_FREQUENCY, centerFrequency),
            2, "Center Frequency");

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_FM_DEMOD_AF_SPAN, span),
            3, "Span");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function sets the maximum span for the display of the AF spectrum
/// HIRET Document return value here.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
ViStatus _VI_FUNC rsspecan_ConfigureADemodAFFullSpan(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_FM_DEMOD_AF_FULL_SPAN, ""));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function configures bandwidth resolution and span for
/// HIFN display of the AF spectrum.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR resolutionBandwidth/This control sets the resolution bandwidth for display of the RF
/// HIPAR resolutionBandwidth/spectrum from the data of analog demodulation.
ViStatus _VI_FUNC rsspecan_ConfigureADemodAFParam(ViSession instrSession,
                                                  ViReal64  resolutionBandwidth)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_FM_DEMOD_RF_SPECTRUM_RESOLUTION_BANDWIDTH, resolutionBandwidth),
            2, "Resolution Bandwidth");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/*****************************************************************************
 * Function: Configure Analog Demod RF Param
 * Purpose:  This function configures bandwidth resolution and span for
 *           display of the RF spectrum.
 *****************************************************************************/
/// HIFN  This function configures bandwidth resolution and span for
/// HIFN  display of the RF spectrum.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR resolutionBandwidth/This control sets the resolution bandwidth for display of the RF
/// HIPAR resolutionBandwidth/spectrum from the data of analog demodulation.
/// HIPAR span/This control sets the frequency range for display of the RF
/// HIPAR span/spectrum resulting from the data of FM demodulation.
ViStatus _VI_FUNC rsspecan_ConfigureADemodRFParam(ViSession instrSession,
                                                  ViReal64  resolutionBandwidth,
                                                  ViReal64  span)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_FM_DEMOD_RF_SPECTRUM_RESOLUTION_BANDWIDTH, resolutionBandwidth),
            2, "Resolution Bandwidth");

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_FM_DEMOD_RF_SPECTRUM_SPAN, span),
            3, "Span");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN This function configures the frequency range for result display
/// HIFN of RF spectrum determined from Analog Demod demodulation data.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR zoom/This control sets the frequency range for display of the RF
/// HIPAR zoom/spectrum resulting from the data of FM demodulation.
ViStatus _VI_FUNC rsspecan_ConfigureADemodRFZoom(ViSession instrSession,
                                                 ViReal64  zoom)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_FM_DEMOD_RF_SPECTRUM_ZOOM, zoom),
            2, "Zoom");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function selects demodulation bandwidth AF filters of Analog Demod
/// HIFN demodulation.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR filterType/This control selects the filter type.
/// HIPAR filterState/This control activates the selected filter.
/// HIPAR filterFrequency/This control sets the filter frequency.
/// HIPAR amfmDeemphasisValue/This control sets the time constant of the de-emphasis for
/// HIPAR amfmDeemphasisValue/analog demodulation.
ViStatus _VI_FUNC rsspecan_ConfigureADemodFilter(ViSession instrSession,
                                                 ViInt32   filterType,
                                                 ViBoolean filterState,
                                                 ViReal64  filterFrequency,
                                                 ViReal64  amfmDeemphasisValue)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    switch (filterType)
    {
    case RSSPECAN_VAL_FMDEM_FILTER_HP:
        viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_FM_DEMOD_HIGH_PASS_AF_FILTER_ENABLED, filterState),
                3, "Filter State");
        if (filterState)
            viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_FM_DEMOD_HIGH_PASS_AF_FILTER_FREQUENCY, filterFrequency),
                    4, "Filter Frequency");
        break;

    case RSSPECAN_VAL_FMDEM_FILTER_LP:
        viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_FM_DEMOD_LOW_PASS_AF_FILTER_ENABLED, filterState),
                3, "Filter State");
        if (filterState)
            viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_FM_DEMOD_LOW_PASS_AF_FILTER_FREQUENCY, filterFrequency),
                    4, "Filter Frequency");
        break;

    case RSSPECAN_VAL_FMDEM_FILTER_DEEMP:
        viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_FM_DEMOD_DEEMPHASIS_ENABLED, filterState),
                3, "Filter State");
        if (filterState)
            viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_FM_DEMOD_DEEMPHASIS_TIME_CONSTANT, amfmDeemphasisValue),
                    5, "Deemphasis Value");
        break;
    case RSSPECAN_VAL_FMDEM_FILTER_LP_REL:

        viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_FM_DEMOD_LOW_PASS_AF_FILTER_ENABLED, filterState),
                3, "Filter State");
        if (filterState)
        {
            viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_ANALOG_DEMOD_LOW_PASS_FILTER_FREQUENCY_RELATIVE, (ViInt32)filterFrequency),
                        4, "Filter Frequency");
        }
        break;

    default: viCheckParm(RsCore_InvalidViInt32Value(instrSession, filterType), 2, "Filter Type");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function selects the cutoff frequency of the low/high pass filter for the specified evaluation.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR filterType/This control selects the filter type.
/// HIPAR filterFrequency/This control selects the cutoff frequency of the low pass filter
/// HIPAR filterFrequency/for the specified evaluation.
ViStatus _VI_FUNC rsspecan_ConfigureADemodFilterManual(ViSession instrSession,
                                                       ViInt32   filterType,
                                                       ViReal64  filterFrequency)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    switch (filterType)
    {
    case RSSPECAN_VAL_FMDEM_FILTER_HP:
        viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_ANALOG_DEMOD_FILTER_FREQUENCY_MANUAL_HIGHPASS, filterFrequency),
                3, "Filter Frequency");
        break;
    case RSSPECAN_VAL_FMDEM_FILTER_LP:
        viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_ANALOG_DEMOD_FILTER_FREQUENCY_MANUAL_LOWPASS, filterFrequency),
                3, "Filter Frequency");
        break;
    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, filterType), 2, "Filter Type");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function selects weighting filter.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR CCITT/This control activates/deactivate CCITT filter weightening.
/// HIPAR CCIR/This control activates/deactivate CCIR filter weightening.
ViStatus _VI_FUNC rsspecan_ConfigureADemodFilterWeighting(ViSession instrSession,
                                                          ViBoolean CCITT,
                                                          ViBoolean CCIR)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_ANALOG_DEMOD_FILTER_CCITT_WEIGHTING, CCITT),
            2, "CCITT");
    if (CCITT == VI_FALSE)
    {
        viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_ANALOG_DEMOD_FILTER_CCIR_WEIGHTING, CCIR),
                3, "CCIR");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN This function activates/deactivates the "A" weighting filter.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR aWeightingFilter/This control activates/deactivate A weighting filter.
ViStatus _VI_FUNC rsspecan_ConfigureADemodFilterAWeighting(ViSession instrSession,
                                                           ViBoolean aWeightingFilter)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_ANALOG_DEMOD_FILTER_A_WEIGHTED_ENABLED, aWeightingFilter),
            2, "A Weighting Filter");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Enables or disables the calculation and display of the settling time.
/// HIFN  Available for all time domain displays.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR settlingTime/Enables or disables the calculation and display of the settling time.
/// HIPAR settlingTime/Available for all time domain displays.
ViStatus _VI_FUNC rsspecan_ConfigureAnalogDemodSettlingTimeEnabled(
                            ViSession instrSession,
                            ViBoolean settlingTime)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, NULL, RSSPECAN_ATTR_ANALOG_DEMOD_SETTLING_TIME_ENABLED, settlingTime),
            2, "Settling Time");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the lower and upper phase limit of the settling time
/// HIFN  measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR lowerLimit/Defines the lower phase limit of the settling time measurement.
/// HIPAR upperLimit/Defines the upper phase limit of the settling time measurement.
ViStatus _VI_FUNC rsspecan_ConfigureTransientAnalysisSettlingTimePhaseLimits(
                            ViSession instrSession,
                            ViInt32   lowerLimit,
                            ViInt32   upperLimit)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, NULL, RSSPECAN_ATTR_TRANSIENT_ANALYSIS_SETTLING_TIME_LOWER_PHASE_LIMIT, lowerLimit),
            2, "Lower Limit");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, NULL, RSSPECAN_ATTR_TRANSIENT_ANALYSIS_SETTLING_TIME_UPPER_PHASE_LIMIT, upperLimit),
            3, "Upper Limit");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the upper and lower limit of the settling time corridor. The
/// HIFN  value is defined with reference to the reference value, see also
/// HIFN  RSSPECAN_ATTR_SUBWINDOW_DISPLAY_REFERENCE_LEVEL and
/// HIFN  RSSPECAN_ATTR_ANALOG_DEMOD_ZERO_PHASE_REF_POINT.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR lowerLimit/Defines the upper limit of the settling time corridor. The value is
/// HIPAR lowerLimit/defined with reference to the reference value, see also
/// HIPAR lowerLimit/RSSPECAN_ATTR_SUBWINDOW_DISPLAY_REFERENCE_LEVEL and
/// HIPAR lowerLimit/RSSPECAN_ATTR_ANALOG_DEMOD_ZERO_PHASE_REF_POINT.
/// HIPAR upperLimit/Defines the upper limit of the settling time corridor. The value is
/// HIPAR upperLimit/defined with reference to the reference value, see also
/// HIPAR upperLimit/RSSPECAN_ATTR_SUBWINDOW_DISPLAY_REFERENCE_LEVEL and
/// HIPAR upperLimit/RSSPECAN_ATTR_ANALOG_DEMOD_ZERO_PHASE_REF_POINT.
ViStatus _VI_FUNC rsspecan_ConfigureAnalogDemodSettlingLimits(
                            ViSession instrSession,
                            ViReal64  lowerLimit,
                            ViReal64  upperLimit)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, NULL, RSSPECAN_ATTR_ANALOG_DEMOD_SETTLING_LOWER_LIMIT, lowerLimit),
            2, "Lower Limit");

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, NULL, RSSPECAN_ATTR_ANALOG_DEMOD_SETTLING_UPPER_LIMIT, upperLimit),
            3, "Upper Limit");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function sets the result types to be measured
/// HIRET Document return value here.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR demodulationType/This parameter selects the demodulation.
/// HIPAR resultTypes/Selects the trace mode result types, to be measured in parallel
/// HIPAR resultTypes/by selected demodulation.
ViStatus _VI_FUNC rsspecan_ConfigureADemodResultType(ViSession instrSession,
                                                     ViInt32   demodulationType,
                                                     ViInt32   resultTypes[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViChar*  p2string;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K7"));
    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &rsspecan_rngFMDemodType, demodulationType, &p2string),
                2, "Demodulation Type");

    if (rsspecan_IsFSV(instrSession))
    {
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SENS:ADEM:%s:TYPE %s,%s,%s",
                 p2string, resultTypeArr[resultTypes[0]],
                 resultTypeArr[resultTypes[1]], resultTypeArr[resultTypes[2]]);
    }
    else
    {
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SENS:ADEM:%s:TYPE %s,%s,%s,%s,%s,%s",
                 p2string, resultTypeArr[resultTypes[0]],
                 resultTypeArr[resultTypes[1]], resultTypeArr[resultTypes[2]], resultTypeArr[resultTypes[3]],
                 resultTypeArr[resultTypes[4]], resultTypeArr[resultTypes[5]]);
    }

    checkErr(RsCore_Write(instrSession, cmd));
    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function reads selected value from instrument.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR value/Selects the parameter to be read from instrument.
/// HIPAR result/This control returns the value of selected parameter.
ViStatus _VI_FUNC rsspecan_GetADemodCurrentlySetValue(ViSession instrSession,
                                                      ViInt32   value,
                                                      ViReal64* result)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidNullPointer(instrSession, result), 3, "Result");
    switch (value)
    {
    case RSSPECAN_VAL_FMDEM_RLEN:
        viCheckParm(rsspecan_GetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_FM_DEMOD_RECORD_LENGTH, result),
                3, "Result");
        break;

    case RSSPECAN_VAL_FMDEM_SRATE:
        viCheckParm(rsspecan_GetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_FM_DEMOD_SAMPLING_RATE, result),
                3, "Result");
        break;
    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, value), 2, "Value");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function returns analog demodulation result values.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR demodulationType/This parameter selects the demodulation.
/// HIPAR resultType/Selects the result type to be measured in parallel by selected
/// HIPAR resultType/demodulation.
/// HIPAR timeout/This control sets the timeout value.
/// HIPAR arraySize/This parameter determines the size of array result values. The
/// HIPAR arraySize/Result Values array must contain at least this number of items.
/// HIPAR resultValues/Returns the result values.
/// HIPAR returnedValues/Returns the number of returned values.
ViStatus _VI_FUNC rsspecan_GetADemodResultValues(ViSession instrSession,
                                                 ViInt32   demodulationType,
                                                 ViInt32   resultType,
                                                 ViUInt32  timeout,
                                                 ViInt32   arraySize,
                                                 ViReal64  resultValues[],
                                                 ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViChar*  p2string;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K7"));
    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &rsspecan_rngFMDemodType, demodulationType, &p2string),
                2, "Demodulation Type");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, resultType, 1, 4),
            3, "Result Type");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, ":FORM REAL,32;:SENS:ADEM:%s:RES? %s", p2string, resultTypeArr[resultType]);
    checkErr(
        RsCore_QueryFloatArrayToUserBufferWithOpc(instrSession, cmd, timeout, arraySize, resultValues, returnedValues));

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function calculates the Analog Demod offset of the current measured
/// HIFN data set.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR resultType/This control determines the result type.
/// HIPAR value/Returns measured value.
ViStatus _VI_FUNC rsspecan_GetADemodOffset(ViSession instrSession,
                                           ViInt32   resultType,
                                           ViReal64* value)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    switch (resultType)
    {
    case RSSPECAN_VAL_FMDEM_IMM:
        viCheckParm(rsspecan_GetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_ANALOG_DEMOD_FM_OFFSET_IMMEDIATE, value),
                3, "Value");
        break;
    case RSSPECAN_VAL_FMDEM_AVG:
        viCheckParm(rsspecan_GetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_ANALOG_DEMOD_FM_OFFSET_AVERAGE, value),
                3, "Value");
        break;
    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, resultType), 2, "Result Type");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);

    return error;
}

/// HIFN  This function returns analog modulation measurement summary.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR markerNumber/Selects the marker.
/// HIPAR demodulationType/This parameter selects the demodulation.
/// HIPAR detectorType/This control selects the detector type.
/// HIPAR trace/Selects the trace.
/// HIPAR value/Returns measured value.
ViStatus _VI_FUNC rsspecan_GetADemodMarkerModulationValue(ViSession instrSession,
                                                          ViInt32   markerNumber,
                                                          ViInt32   demodulationType,
                                                          ViInt32   detectorType,
                                                          ViInt32   trace,
                                                          ViReal64* value)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_m[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_t[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_ANALOG_DEMOD_SUMMARY_RESULT_VALUE_AM_PPEAK,
            "Trace", trace - 1, RS_REPCAP_BUF_SIZE, repCap_t),
            5, "Trace");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_ANALOG_DEMOD_SUMMARY_RESULT_VALUE_AM_PPEAK,
            "Marker", markerNumber - 1, RS_REPCAP_BUF_SIZE, repCap_m),
            2, "Marker Number");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "%s,%s", repCap_m, repCap_t);

    switch (demodulationType)
    {
    case RSSPECAN_VAL_FMDEM_AM:
        switch (detectorType)
        {
        case RSSPECAN_VAL_FMDEM_DET_PPE:
            viCheckParm(rsspecan_GetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_ANALOG_DEMOD_SUMMARY_RESULT_VALUE_AM_PPEAK, value),
                    6, "Value");
            break;
        case RSSPECAN_VAL_FMDEM_DET_MPE:
            viCheckParm(rsspecan_GetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_ANALOG_DEMOD_SUMMARY_RESULT_VALUE_AM_MPEAK, value),
                    6, "Value");
            break;
        case RSSPECAN_VAL_FMDEM_DET_MID:
            viCheckParm(rsspecan_GetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_ANALOG_DEMOD_SUMMARY_RESULT_VALUE_AM_MIDDLE, value),
                    6, "Value");
            break;
        case RSSPECAN_VAL_FMDEM_DET_RMS:
            viCheckParm(rsspecan_GetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_ANALOG_DEMOD_SUMMARY_RESULT_VALUE_AM_RMS, value),
                    6, "Value");
            break;
        default:
            viCheckParm(RsCore_InvalidViInt32Value(instrSession, demodulationType), 4, "Detector Type");
        }
        break;
    case RSSPECAN_VAL_FMDEM_FM:
        switch (detectorType)
        {
        case RSSPECAN_VAL_FMDEM_DET_PPE:
            viCheckParm(rsspecan_GetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_ANALOG_DEMOD_SUMMARY_RESULT_VALUE_FM_PPEAK, value),
                    6, "Value");
            break;
        case RSSPECAN_VAL_FMDEM_DET_MPE:
            viCheckParm(rsspecan_GetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_ANALOG_DEMOD_SUMMARY_RESULT_VALUE_FM_MPEAK, value),
                    6, "Value");
            break;
        case RSSPECAN_VAL_FMDEM_DET_MID:
            viCheckParm(rsspecan_GetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_ANALOG_DEMOD_SUMMARY_RESULT_VALUE_FM_MIDDLE, value),
                    6, "Value");
            break;
        case RSSPECAN_VAL_FMDEM_DET_RMS:
            viCheckParm(rsspecan_GetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_ANALOG_DEMOD_SUMMARY_RESULT_VALUE_FM_RMS, value),
                    6, "Value");
            break;
        default:
            viCheckParm(RsCore_InvalidViInt32Value(instrSession, demodulationType), 4, "Detector Type");
        }
        break;
    case RSSPECAN_VAL_FMDEM_PM:
        switch (detectorType)
        {
        case RSSPECAN_VAL_FMDEM_DET_PPE:
            viCheckParm(rsspecan_GetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_ANALOG_DEMOD_SUMMARY_RESULT_VALUE_PM_PPEAK, value),
                    6, "Value");
            break;
        case RSSPECAN_VAL_FMDEM_DET_MPE:
            viCheckParm(rsspecan_GetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_ANALOG_DEMOD_SUMMARY_RESULT_VALUE_PM_MPEAK, value),
                    6, "Value");
            break;
        case RSSPECAN_VAL_FMDEM_DET_MID:
            viCheckParm(rsspecan_GetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_ANALOG_DEMOD_SUMMARY_RESULT_VALUE_PM_MIDDLE, value),
                    6, "Value");
            break;
        case RSSPECAN_VAL_FMDEM_DET_RMS:
            viCheckParm(rsspecan_GetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_ANALOG_DEMOD_SUMMARY_RESULT_VALUE_PM_RMS, value),
                    6, "Value");
            break;
        default:
            viCheckParm(RsCore_InvalidViInt32Value(instrSession, demodulationType), 4, "Detector Type");
        }
        break;
    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, demodulationType), 3, "Demodulation Type");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);

    return error;
}


/// HIFN This function returns current relative value of the demodulated
/// HIFN signal for the specified trace.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR markerNumber/Selects the marker.
/// HIPAR demodulationType/This parameter selects the demodulation.
/// HIPAR detectorType/This control selects the detector type.
/// HIPAR trace/Selects the trace.
/// HIPAR value/Returns measured value.
ViStatus _VI_FUNC rsspecan_GetADemodMarkerModulationRelativeValue(
                            ViSession instrSession,
                            ViInt32   markerNumber,
                            ViInt32   demodulationType,
                            ViInt32   detectorType,
                            ViInt32   trace,
                            ViReal64* value)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap_m[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_t[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_ANALOG_DEMOD_SUMMARY_RESULT_VALUE_AM_RELATIVE_PPEAK,
            "Trace", trace - 1, RS_REPCAP_BUF_SIZE, repCap_t),
            5, "Trace");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_ANALOG_DEMOD_SUMMARY_RESULT_VALUE_AM_RELATIVE_PPEAK,
            "Marker", markerNumber - 1, RS_REPCAP_BUF_SIZE, repCap_m),
            2, "Marker Number");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "%s,%s", repCap_m, repCap_t);

    switch (demodulationType)
    {
    case RSSPECAN_VAL_FMDEM_AM:
        switch (detectorType)
        {
        case RSSPECAN_VAL_FMDEM_DET_PPE:
            viCheckParm(rsspecan_GetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_ANALOG_DEMOD_SUMMARY_RESULT_VALUE_AM_RELATIVE_PPEAK, value),
                    6, "Value");
            break;
        case RSSPECAN_VAL_FMDEM_DET_MPE:
            viCheckParm(rsspecan_GetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_ANALOG_DEMOD_SUMMARY_RESULT_VALUE_AM_RELATIVE_MPEAK, value),
                    6, "Value");
            break;
        case RSSPECAN_VAL_FMDEM_DET_MID:
            viCheckParm(rsspecan_GetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_ANALOG_DEMOD_SUMMARY_RESULT_VALUE_AM_RELATIVE_MIDDLE, value),
                    6, "Value");
            break;
        case RSSPECAN_VAL_FMDEM_DET_RMS:
            viCheckParm(rsspecan_GetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_ANALOG_DEMOD_SUMMARY_RESULT_VALUE_AM_RELATIVE_RMS, value),
                    6, "Value");
            break;
        default:
            viCheckParm(RsCore_InvalidViInt32Value(instrSession, demodulationType), 4, "Detector Type");
        }
        break;
    case RSSPECAN_VAL_FMDEM_FM:
        switch (detectorType)
        {
        case RSSPECAN_VAL_FMDEM_DET_PPE:
            viCheckParm(rsspecan_GetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_ANALOG_DEMOD_SUMMARY_RESULT_VALUE_FM_RELATIVE_PPEAK, value),
                    6, "Value");
            break;
        case RSSPECAN_VAL_FMDEM_DET_MPE:
            viCheckParm(rsspecan_GetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_ANALOG_DEMOD_SUMMARY_RESULT_VALUE_FM_RELATIVE_MPEAK, value),
                    6, "Value");
            break;
        case RSSPECAN_VAL_FMDEM_DET_MID:
            viCheckParm(rsspecan_GetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_ANALOG_DEMOD_SUMMARY_RESULT_VALUE_FM_RELATIVE_MIDDLE, value),
                    6, "Value");
            break;
        case RSSPECAN_VAL_FMDEM_DET_RMS:
            viCheckParm(rsspecan_GetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_ANALOG_DEMOD_SUMMARY_RESULT_VALUE_FM_RELATIVE_RMS, value),
                    6, "Value");
            break;
        default:
            viCheckParm(RsCore_InvalidViInt32Value(instrSession, demodulationType), 4, "Detector Type");
        }
        break;
    case RSSPECAN_VAL_FMDEM_PM:
        switch (detectorType)
        {
        case RSSPECAN_VAL_FMDEM_DET_PPE:
            viCheckParm(rsspecan_GetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_ANALOG_DEMOD_SUMMARY_RESULT_VALUE_PM_RELATIVE_PPEAK, value),
                    6, "Value");
            break;
        case RSSPECAN_VAL_FMDEM_DET_MPE:
            viCheckParm(rsspecan_GetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_ANALOG_DEMOD_SUMMARY_RESULT_VALUE_PM_RELATIVE_MPEAK, value),
                    6, "Value");
            break;
        case RSSPECAN_VAL_FMDEM_DET_MID:
            viCheckParm(rsspecan_GetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_ANALOG_DEMOD_SUMMARY_RESULT_VALUE_PM_RELATIVE_MIDDLE, value),
                    6, "Value");
            break;
        case RSSPECAN_VAL_FMDEM_DET_RMS:
            viCheckParm(rsspecan_GetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_ANALOG_DEMOD_SUMMARY_RESULT_VALUE_PM_RELATIVE_RMS, value),
                    6, "Value");
            break;
        default:
            viCheckParm(RsCore_InvalidViInt32Value(instrSession, demodulationType), 4, "Detector Type");
        }
        break;
    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, demodulationType), 3, "Demodulation Type");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);

    return error;
}

/*****************************************************************************
 * Function: Get Analog Demod Marker Measurement Value
 * Purpose:  This function returns analog modulation measurement summary.
 *****************************************************************************/
/// HIFN This function returns analog modulation measurement summary.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR markerNumber/Selects the marker.
/// HIPAR measurement/This parameter selects the measurement.
/// HIPAR trace/Selects the trace.
/// HIPAR value/Returns measured value.
ViStatus _VI_FUNC rsspecan_GetADemodMarkerMeasValue(ViSession instrSession,
                                                    ViInt32   markerNumber,
                                                    ViInt32   measurement,
                                                    ViInt32   trace,
                                                    ViReal64* value)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap_m[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_t[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_FM_DEMOD_AUDIO_FREQUENCY_RESULT,
            "Trace", trace - 1, RS_REPCAP_BUF_SIZE, repCap_t),
            5, "Trace");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_FM_DEMOD_AUDIO_FREQUENCY_RESULT,
            "Marker", markerNumber - 1, RS_REPCAP_BUF_SIZE, repCap_m),
            2, "Marker Number");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "%s,%s", repCap_m, repCap_t);
    switch (measurement)
    {
    case RSSPECAN_VAL_FMDEM_MEAS_AFR:
        viCheckParm(rsspecan_GetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_FM_DEMOD_AUDIO_FREQUENCY_RESULT, value),
                5, "Value");
        break;
    case RSSPECAN_VAL_FMDEM_MEAS_FER:
        viCheckParm(rsspecan_GetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_FM_DEMOD_FREQUENCY_ERROR_RESULT, value),
                5, "Value");
        break;
    case RSSPECAN_VAL_FMDEM_MEAS_SIN:
        viCheckParm(rsspecan_GetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_FM_DEMOD_SINAD_MEASUREMENT_RESULT, value),
                5, "Value");
        break;
    case RSSPECAN_VAL_FMDEM_MEAS_THD:
        viCheckParm(rsspecan_GetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_FM_DEMOD_THD_MEASUREMENT_RESULT, value),
                5, "Value");
        break;
    case RSSPECAN_VAL_FMDEM_MEAS_CAR:
        viCheckParm(rsspecan_GetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_FM_DEMOD_CARRIER_POWER_RESULT, value),
                5, "Value");
        break;
    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, measurement), 3, "Measurement");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the result of the modulation distortion measurement in the
/// HIFN  specified window for the specified trace.
/// HIFN
/// HIFN     Note that this value is only calculated if an AF Spectrum window is
/// HIFN  displayed.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR marker/Selects the marker.
/// HIPAR trace/Selects the trace.
/// HIPAR distortion/Queries the result of the modulation distortion measurement in the
/// HIPAR distortion/specified window for the specified trace.
ViStatus _VI_FUNC rsspecan_GetAnalogDemodDistortionResult(ViSession instrSession,
                                                          ViInt32   marker,
                                                          ViInt32   trace,
                                                          ViReal64* distortion)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCapMarker[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  repCapMarkerCount = 0;
    ViChar   repCapTrace[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  repCapTraceCount = 0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetRepCapIdItemsCount(instrSession, "Marker", &repCapMarkerCount),
        2, "Marker");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, marker, 1, repCapMarkerCount - 1),
        2, "Marker");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_ANALOG_DEMOD_DISTORTION_RESULT, "Marker", marker - 1, RS_REPCAP_BUF_SIZE, repCapMarker),
        2, "Marker");
    viCheckParm(RsCore_GetRepCapIdItemsCount(instrSession, "Trace", &repCapTraceCount),
        3, "Trace");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, trace, 1, repCapTraceCount - 1),
        3, "Trace");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_ANALOG_DEMOD_DISTORTION_RESULT, "Trace", trace - 1, RS_REPCAP_BUF_SIZE, repCapTrace),
        3, "Trace");

    (void)snprintf(repCap, RS_REPCAP_BUF_SIZE, "%s,%s", repCapMarker, repCapTrace);

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_ANALOG_DEMOD_DISTORTION_RESULT, distortion),
            4, "Distortion");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Returns the settling time after which the signal remains within a
/// HIFN  specified target corridor. The settling time is evaluated for the
/// HIFN  selected trace in each time domain window. The value is only determined
/// HIFN  for RSSPECAN_ATTR_ANALOG_DEMOD_SETTLING_TIME_ENABLED ON.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR trace/Selects the trace.
/// HIPAR settlingTime/Returns the settling time after which the signal remains within a
/// HIPAR settlingTime/specified target corridor. The settling time is evaluated for the
/// HIPAR settlingTime/selected trace in each time domain window. The value is only determined
/// HIPAR settlingTime/for RSSPECAN_ATTR_ANALOG_DEMOD_SETTLING_TIME_ENABLED ON.
ViStatus _VI_FUNC rsspecan_GetAnalogDemodSettlingTime(ViSession instrSession,
                                                      ViInt32   trace,
                                                      ViReal64* settlingTime)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  repCapCount = 0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetRepCapIdItemsCount(instrSession, "Trace", &repCapCount),
        2, "Trace");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, trace, 1, repCapCount - 1),
        2, "Trace");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_ANALOG_DEMOD_SETTLING_TIME,
            "Trace", trace - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Trace");

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, NULL, RSSPECAN_ATTR_ANALOG_DEMOD_SETTLING_TIME, settlingTime),
            3, "Settling Time");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/*****************************************************************************
 *- Callback Definition   ---------------------------------------------------*
 *****************************************************************************/

/*****************************************************************************
 *- End Of RSSpecAn K7 - Analog Demodulator -------------------------------------*
 *****************************************************************************/
