/*===========================================================================*
 *  Rohde&Schwarz Spectrum Analyzer instrument driver
 *  RsSpecAn K8 - Bluetooth Analyzer Group Functions
 *
 *  Original Release: September 2005
 *  By: Martin Koutny (instrument driver core)
 *      Helena Sucha (instrument specific implementation)
 *      Jiri Kominek (DrBase repository & tools)
 *
 *  Should you have any technical questions please contact the hotline of
 *  Rohde & Schwarz Vertriebs-GmbH Rohde & Schwarz Support Center
 *
 *  e-mail: CustomerSupport@rohde-schwarz.com
 *

 *===========================================================================*/

#include "rsspecan.h"

/*****************************************************************************
 * Function:    Bluetooth Mode
 * Purpose:     This function selects the Bluetooth analyzer mode
 *****************************************************************************/
ViStatus _VI_FUNC rsspecan_BTOMode(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_BTO_MODE, ""));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the Bluetooth standard the signal complies with.
/// HIFN
/// HIFN     For details on the different standards, see 'Measurement basics'.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR standard/Defines the Bluetooth standard the signal complies with.
ViStatus _VI_FUNC rsspecan_ConfigureBTOStandard(ViSession instrSession,
                                                ViInt32   standard)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, NULL, RSSPECAN_ATTR_BTO_STANDARD, standard),
            2, "Standard");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  For measurements based on the low-energy standard (selected with
/// HIFN  RSSPECAN_ATTR_BTO_STANDARD), defines the substandard depending on the
/// HIFN  used physical layer (PHY).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR standard/For measurements based on the low-energy standard (selected with
/// HIPAR standard/RSSPECAN_ATTR_BTO_STANDARD), defines the substandard depending on the
/// HIPAR standard/used physical layer (PHY).
ViStatus _VI_FUNC rsspecan_ConfigureBTOLowEnergyStandard(ViSession instrSession,
                                                         ViInt32   standard)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, NULL, RSSPECAN_ATTR_BTO_LOW_ENERGY_STANDARD, standard),
            2, "Standard");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Loads the measurement settings for the selected standard from a file
/// HIFN  with the defined name.
/// HIFN
/// HIFN     For an overview of predefined standards and settings, see 'Predefined
/// HIFN  standards and settings'.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR settingsFile/Loads the measurement settings for the selected standard from a file
/// HIPAR settingsFile/with the defined name.
ViStatus _VI_FUNC rsspecan_ConfigureBTOLoadSettings(ViSession instrSession,
                                                    ViString  settingsFile)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViString(instrSession, NULL, RSSPECAN_ATTR_BTO_LOAD_SETTINGS, settingsFile),
            2, "Settings File");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Saves the current measurement settings for a specific standard as a
/// HIFN  file with the defined name.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR settingsFile/Saves the current measurement settings for a specific standard as a
/// HIPAR settingsFile/file with the defined name.
ViStatus _VI_FUNC rsspecan_ConfigureBTOSaveSettings(ViSession instrSession,
                                                    ViString  settingsFile)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViString(instrSession, NULL, RSSPECAN_ATTR_BTO_SAVE_SETTINGS, settingsFile),
            2, "Settings File");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Exports trace data for all active traces in all active windows to a
/// HIFN  file in .csv format.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR measurementFile/Exports trace data for all active traces in all active windows to a
/// HIPAR measurementFile/file in .csv format.
ViStatus _VI_FUNC rsspecan_ConfigureBTOSaveMeasurement(ViSession instrSession,
                                                       ViString  measurementFile)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViString(instrSession, NULL, RSSPECAN_ATTR_BTO_SAVE_MEASUREMENT, measurementFile),
            2, "Measurement File");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}



/*****************************************************************************
 * Function:    Configure BTOMeasurement
 * Purpose:     This function configures the common parameters of measurement
 *              according Bluetooth standard.
 *****************************************************************************/
ViStatus _VI_FUNC rsspecan_ConfigureBTOMeasurement(ViSession instrSession,
                                                   ViInt32   geographicalRegion,
                                                   ViInt32   channel,
                                                   ViInt32   outputPowerClass,
                                                   ViInt32   samplesPerSymbols,
                                                   ViInt32   packetType,
                                                   ViReal64  anntennaGain)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_BTO_GEOGRAPHICAL_REGION, geographicalRegion),
            2, "Geographical Region");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_BTO_FREQUENCY_CHANNEL, channel),
            3, "Channel");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_BTO_OUTPUT_POWER_CLASS, outputPowerClass),
            4, "Output Power Class");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_BTO_SAMPLES_PER_SYMBOLS, samplesPerSymbols),
            5, "Samples Per Symbol");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_BTO_PACKET_TYPE, packetType),
            6, "Packet Type");

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_BTO_ANTENNA_GAIN, anntennaGain),
            7, "Antenna Gain");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Enables or disables the use of an optional measurement filter (see
/// HIFN  'Measurement filter').
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR filter/Enables or disables the use of an optional measurement filter (see
/// HIPAR filter/'Measurement filter').
ViStatus _VI_FUNC rsspecan_ConfigureBTOMeasurementFilterEnabled(
                            ViSession instrSession,
                            ViBoolean filter)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, NULL, RSSPECAN_ATTR_BTO_MEASUREMENT_FILTER_ENABLED, filter),
            2, "Filter");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the used demodulation bandwidth, which depends on whether or
/// HIFN  not a measurement filter is used.
/// HIFN
/// HIFN     If the measurement filter is used, the demodulation bandwidth is
/// HIFN  1.3MHz. Otherwise, it is 3MHz.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR demodulationBandwidth/Queries the used demodulation bandwidth, which depends on whether or
/// HIPAR demodulationBandwidth/not a measurement filter is used.
ViStatus _VI_FUNC rsspecan_QueryBTOMeasurementDemodulationBandwidth(
                            ViSession instrSession,
                            ViReal64* demodulationBandwidth)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, NULL, RSSPECAN_ATTR_BTO_MEASUREMENT_DEMODULATION_BANDWIDTH, demodulationBandwidth),
            2, "Demodulation Bandwidth");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/*****************************************************************************
 * Function:    Configure BTO AveragingMethod
 * Purpose:     This function configures the averaging area of the average
 *              power measurement.
 *****************************************************************************/
ViStatus _VI_FUNC rsspecan_ConfigureBTOAveragingMethod(ViSession instrSession,
                                                       ViReal64  startPosition,
                                                       ViReal64  stopPosition)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_BTO_POSITION_OF_AVERAGE_POWER_START, startPosition),
            2, "Start Position");

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_BTO_POSITION_OF_AVERAGE_POWER_STOP, stopPosition),
            2, "Stop Position");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/*****************************************************************************
 * Function:    ConfigureBTOSynchronization
 * Purpose:     This function configures the synchronization of the measurement.
 *****************************************************************************/
ViStatus _VI_FUNC rsspecan_ConfigureBTOSynchronization(ViSession instrSession,
                                                       ViBoolean findSync,
                                                       ViInt32   syncOffset,
                                                       ViBoolean findBurst,
                                                       ViReal64  burstOffset,
                                                       ViBoolean searchLengthAuto,
                                                       ViReal64  searchLength)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_BTO_FIND_SYNC_ENABLED, findSync),
            2, "Find Sync");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_BTO_SYNC_OFFSET, syncOffset),
            3, "Sync Offset");

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_BTO_FIND_BURST_ENABLED, findBurst),
            4, "Find Burst");

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_BTO_BURST_OFFSET, burstOffset),
            5, "Burst Offset");

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_BTO_SEARCH_TIME_AUTO, searchLengthAuto),
            6, "Search Length Auto");
    if (!searchLengthAuto)
    {
        viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_BTO_SEARCH_TIME, searchLength),
                7, "Search Length");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/*****************************************************************************
 * Function:    ConfigureBTOLAP
 * Purpose:     This function configures the LAP for synchronization of the measurement.
 *****************************************************************************/
ViStatus _VI_FUNC rsspecan_ConfigureBTOLAP(ViSession instrSession,
                                           ViInt32   lowest24Bits_ofDevAddress)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_BTO_LOWEST_24_BITS_OF_BLUETOOTH_DEVICE_ADDRESS, lowest24Bits_ofDevAddress),
            2, "Lowest 24 Bits of Dev. Address");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets or queries the lower address part of the Bluetooth device address
/// HIFN  (LAP, the 24 least significant bits). The LAP determines the sync word
/// HIFN  used for sync search.
/// HIFN
/// HIFN     Not available for LE mode.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR lap/Sets or queries the lower address part of the Bluetooth device address
/// HIPAR lap/(LAP, the 24 least significant bits). The LAP determines the sync word
/// HIPAR lap/used for sync search.
ViStatus _VI_FUNC rsspecan_ConfigureBTOSynchronizationLAP(ViSession instrSession,
                                                          ViString  lap)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViString(instrSession, NULL, RSSPECAN_ATTR_BTO_SYNCHRONIZATION_LAP, lap),
            2, "LAP");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Enables or disables the search for the sync word. The only results
/// HIFN  that can be determined without synchronization are the output power
/// HIFN  results. See 'Synchronization'.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR syncSearch/Enables or disables the search for the sync word. The only results
/// HIPAR syncSearch/that can be determined without synchronization are the output power
/// HIPAR syncSearch/results. See 'Synchronization'.
ViStatus _VI_FUNC rsspecan_ConfigureBTOSynchronizationSearchEnabled(
                            ViSession instrSession,
                            ViBoolean syncSearch)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, NULL, RSSPECAN_ATTR_BTO_SYNCHRONIZATION_SEARCH_ENABLED, syncSearch),
            2, "Sync Search");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Enables or disables the search for a signal burst based on the
/// HIFN  measured power.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR pulseSearch/Enables or disables the search for a signal burst based on the
/// HIPAR pulseSearch/measured power.
ViStatus _VI_FUNC rsspecan_ConfigureBTOPulseSearchEnabled(ViSession instrSession,
                                                          ViBoolean pulseSearch)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, NULL, RSSPECAN_ATTR_BTO_PULSE_SEARCH_ENABLED, pulseSearch),
            2, "Pulse Search");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/*****************************************************************************
 * Function:    Configure BTO Measurement Mode
 * Purpose:     This function selects the current measurement according to the
 *              Bluetooth standard.
 *****************************************************************************/
ViStatus _VI_FUNC rsspecan_ConfigureBTOMeasurementMode(ViSession instrSession,
                                                       ViInt32   measurementMode)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_BTO_MEASUREMENT_MODE, measurementMode),
            2, "Measurement Mode");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/*****************************************************************************
 * Function:    Configure BTO Number Of Adjacent Channels
 * Purpose:     This function selects the number of adjacent channel pairs
 *              during the measurement of the adjacent channel power.
 *****************************************************************************/
ViStatus _VI_FUNC rsspecan_ConfigureBTONumberOfAdjacentChannels(
                            ViSession instrSession,
                            ViInt32   number_ofAdjacentChannels)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_BTO_NUMBER_OF_ADJACENT_CHANNELS, number_ofAdjacentChannels),
            2, "Number of Adjacent Channels");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/*****************************************************************************
 * Function:    Configure BTO Receive Filter
 * Purpose:     This function selects the receive filter for the signal to
 *              measure.
 *****************************************************************************/
ViStatus _VI_FUNC rsspecan_ConfigureBTOReceiveFilter(ViSession instrSession,
                                                     ViInt32   filter)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_BTO_MEASUREMENT_FILTER, filter),
            2, "Filter");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/*****************************************************************************
 * Function:    Configure BTO Sweep Coupling Auto
 * Purpose:     This function activates auto coupling of selected coupling
 *              method.
 *****************************************************************************/
ViStatus _VI_FUNC rsspecan_ConfigureBTOSweepCouplingAuto(ViSession instrSession,
                                                         ViInt32   sweepCoupling)
{
    ViStatus error = VI_SUCCESS;
    ViAttr   attribute = 0;

    checkErr(RsCore_LockSession(instrSession));

    switch (sweepCoupling)
    {
    case RSSPECAN_VAL_COUPLING_RBW:
        attribute = RSSPECAN_ATTR_BTO_RESOLUTION_BANDWIDTH_AUTO;
        break;

    case RSSPECAN_VAL_COUPLING_VBW:
        attribute = RSSPECAN_ATTR_BTO_VIDEO_BANDWIDTH_AUTO;
        break;

    case RSSPECAN_VAL_COUPLING_SWEEP_TIME:
        attribute = RSSPECAN_ATTR_BTO_SWEEP_TIME_AUTO;
        break;
    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, sweepCoupling),
                    2, "Sweep Coupling");
    }

    checkErr(rsspecan_SetAttributeViBoolean(instrSession, "", attribute, VI_TRUE));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/*****************************************************************************
 * Function:    Configure BTO Sweep Coupling
 * Purpose:     This function configures the coupling value
 *              of the currently selected Bluetooth measurement.
 *****************************************************************************/
ViStatus _VI_FUNC rsspecan_ConfigureBTOSweepCoupling(ViSession instrSession,
                                                     ViInt32   sweepCoupling,
                                                     ViReal64  couplingValue)
{
    ViStatus error = VI_SUCCESS;
    ViAttr   attribute = 0;
    ViAttr   attribute2 = 0;

    checkErr(RsCore_LockSession(instrSession));

    switch (sweepCoupling)
    {
    case RSSPECAN_VAL_COUPLING_RBW:
        attribute = RSSPECAN_ATTR_BTO_RESOLUTION_BANDWIDTH;
        attribute2 = RSSPECAN_ATTR_BTO_RESOLUTION_BANDWIDTH_AUTO;
        break;

    case RSSPECAN_VAL_COUPLING_VBW:
        attribute = RSSPECAN_ATTR_BTO_VIDEO_BANDWIDTH;
        attribute2 = RSSPECAN_ATTR_BTO_VIDEO_BANDWIDTH_AUTO;
        break;

    case RSSPECAN_VAL_COUPLING_SWEEP_TIME:
        attribute = RSSPECAN_ATTR_BTO_SWEEP_TIME;
        attribute2 = RSSPECAN_ATTR_BTO_SWEEP_TIME_AUTO;
        break;
    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, sweepCoupling),
                    2, "Sweep Coupling");
    }
    checkErr(rsspecan_SetAttributeViBoolean(instrSession, "", attribute2, VI_FALSE));
    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", attribute, couplingValue),
            3, "Coupling Value");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  The specifications for in-band spurious emissions measurements differ
/// HIFN  for France and the rest of the world. Thus, in order to check the
/// HIFN  measurement results against limits for a particular standard, you must
/// HIFN  define the geographical location to consider. See also 'Measurement
/// HIFN  settings depending on standard'.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR geography/The specifications for in-band spurious emissions measurements differ
/// HIPAR geography/for France and the rest of the world. Thus, in order to check the
/// HIPAR geography/measurement results against limits for a particular standard, you must
/// HIPAR geography/define the geographical location to consider. See also 'Measurement
/// HIPAR geography/settings depending on standard'.
ViStatus _VI_FUNC rsspecan_ConfigureBTOSpuriousEmissionsGeography(
                            ViSession instrSession,
                            ViInt32   geography)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, NULL, RSSPECAN_ATTR_BTO_SPURIOUS_EMISSIONS_GEOGRAPHY, geography),
            2, "Geography");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Defines the number of channels that are captured by the in-band
/// HIFN  spurious emissions measurement. The number must be an odd number.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR capturedChannels/Defines the number of channels that are captured by the in-band
/// HIPAR capturedChannels/spurious emissions measurement. The number must be an odd number.
ViStatus _VI_FUNC rsspecan_ConfigureBTOSpuriousEmissionsNumberOfCapturedChannels(
                            ViSession instrSession,
                            ViInt32   capturedChannels)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, NULL, RSSPECAN_ATTR_BTO_SPURIOUS_EMISSIONS_NUMBER_OF_CAPTURED_CHANNELS, capturedChannels),
            2, "Captured Channels");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/*****************************************************************************
 * Function:    Configure BTO Acquisition
 * Purpose:     This function configures the acquisition attributes of the
 *              currently selected Bluetooth measurement.
 *****************************************************************************/
ViStatus _VI_FUNC rsspecan_ConfigureBTOAcquisition(ViSession instrSession,
                                                   ViBoolean sweepModeContinuous,
                                                   ViInt32   numberOfSweeps)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_SWEEP_MODE_CONTINUOUS, sweepModeContinuous),
            2, "Sweep Mode Continuous");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_BTO_SWEEP_COUNT, numberOfSweeps),
            3, "Number Of Sweeps");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/*****************************************************************************
 * Function:    Select BTO Trace
 * Purpose:     This function selects the measurement curve for evaluation
 *              of the modulation characteristics.
 *****************************************************************************/
ViStatus _VI_FUNC rsspecan_SelectBTOTrace(ViSession instrSession,
                                          ViInt32   trace)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_MARKER_DEMOD_STATE,
            "Trace", trace - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Trace");

    checkErr(rsspecan_SetAttributeViString(instrSession, repCap, RSSPECAN_ATTR_BTO_TRACE_SELECT, ""));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/*****************************************************************************
 * Function:    Configure BTO Trace
 * Purpose:     This function configures the averaging area of the average
 *              power measurement.
 *****************************************************************************/
ViStatus _VI_FUNC rsspecan_ConfigureBTOTrace(ViSession instrSession,
                                             ViInt32   trace,
                                             ViInt32   traceMode,
                                             ViInt32   traceDetector)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_MARKER_DEMOD_STATE,
            "Trace", trace - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Trace");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_BTO_TRACE_MODE, traceMode),
            3, "Trace Mode");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_BTO_TRACE_DETECTOR, traceDetector),
            4, "Trace Detector");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Enables or disables a limit check for the measured results.
/// HIFN
/// HIFN     Note that this command replaces the two commands from previous signal
/// HIFN  and spectrum analyzers (which are still supported):
/// HIFN  CALCulate<n>:LIMit<li>:TRACe<t> + CALCulate<n>:LIMit<li>:STATe.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR trace/Sets the trace. The driver uses this value to set the repeated
/// HIPAR trace/capability Trace of the attributes.
/// HIPAR limitCheck/Enables or disables a limit check for the measured results.
ViStatus _VI_FUNC rsspecan_ConfigureBTOTraceLimitCheckEnabled(
                            ViSession instrSession,
                            ViInt32   trace,
                            ViBoolean limitCheck)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  repCapCount = 0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetRepCapIdItemsCount(instrSession, "Trace", &repCapCount),
        2, "Trace");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, trace, 1, repCapCount - 1),
        2, "Trace");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_BTO_TRACE_LIMIT_CHECK_ENABLED,
            "Trace", trace - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Trace");

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, repCap, RSSPECAN_ATTR_BTO_TRACE_LIMIT_CHECK_ENABLED, limitCheck),
            3, "Limit Check");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/*****************************************************************************
 * Function:    Configure BTO Packet Bytes SCO
 * Purpose:     This function selects the number of payload bytes that are
 *              transmitted in a packet.
 *****************************************************************************/
ViStatus _VI_FUNC rsspecan_ConfigureBTOPacketBytesSCO(ViSession instrSession,
                                                      ViInt32   packetBytesSCO)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_BTO_PBSC, packetBytesSCO),
            2, "Packet Bytes SCO");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/*****************************************************************************
 * Function:    Configure BTO Display Y-Axis Scaling
 * Purpose:     This function defines the scaling of the y-axis in
 *              the currently selected unit.
 *****************************************************************************/
ViStatus _VI_FUNC rsspecan_ConfigureBTODisplayYAxisScaling(ViSession instrSession,
                                                           ViReal64  scaling)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_DISP_Y_AXIS_SCALING, scaling),
            2, "Scaling");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the lower and upper limit for the frequency drift in LE
/// HIFN  signals.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR lower/Defines the lower limit for the frequency drift in LE signals.
/// HIPAR upper/Defines the upper limit for the frequency drift in LE signals.
ViStatus _VI_FUNC rsspecan_ConfigureBTOLimitChecksFrequencyDrift(
                            ViSession instrSession,
                            ViReal64  lower,
                            ViReal64  upper)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, NULL, RSSPECAN_ATTR_BTO_LIMIT_CHECKS_FREQUENCY_DRIFT_LOWER, lower),
            2, "Lower");

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, NULL, RSSPECAN_ATTR_BTO_LIMIT_CHECKS_FREQUENCY_DRIFT_UPPER, upper),
            3, "Upper");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Defines the lower limit for the frequency offset in LE
/// HIFN  signals.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR lower/Defines the upper limit for the frequency offset in LE signals.
ViStatus _VI_FUNC rsspecan_ConfigureBTOLimitChecksFrequencyOffsetLower(
                            ViSession instrSession,
                            ViReal64  lower)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, NULL, RSSPECAN_ATTR_BTO_LIMIT_CHECKS_FREQUENCY_OFFSET_LOWER, lower),
            2, "Lower");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the upper limit for the frequency offset in LE
/// HIFN  signals.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR upper/Defines the lower limit for the frequency offset in LE signals.
ViStatus _VI_FUNC rsspecan_ConfigureBTOLimitChecksFrequencyOffsetUpper(
                            ViSession instrSession,
                            ViReal64  upper)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, NULL, RSSPECAN_ATTR_BTO_LIMIT_CHECKS_FREQUENCY_OFFSET_UPPER, upper),
            2, "Upper");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Defines the lower and upper limit for the initial carrier frequency
/// HIFN  tolerance (ICFT).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR lower/Defines the lower limit for the initial carrier frequency tolerance
/// HIPAR lower/(ICFT).
/// HIPAR upper/Defines the upper limit for the initial carrier frequency tolerance
/// HIPAR upper/(ICFT).
ViStatus _VI_FUNC rsspecan_ConfigureBTOLimitChecksICFT(ViSession instrSession,
                                                       ViReal64  lower,
                                                       ViReal64  upper)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, NULL, RSSPECAN_ATTR_BTO_LIMIT_CHECKS_ICFT_LOWER, lower),
            2, "Lower");

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, NULL, RSSPECAN_ATTR_BTO_LIMIT_CHECKS_ICFT_UPPER, upper),
            3, "Upper");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Defines the lower and upper limit for the initial frequency drift of
/// HIFN  LE packets.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR lower/Defines the lower limit for the initial frequency drift of LE packets.
/// HIPAR upper/Defines the upper limit for the initial frequency drift of LE packets.
ViStatus _VI_FUNC rsspecan_ConfigureBTOLimitChecksInitialFrequencyDrift(
                            ViSession instrSession,
                            ViReal64  lower,
                            ViReal64  upper)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, NULL, RSSPECAN_ATTR_BTO_LIMIT_CHECKS_INITIAL_FREQUENCY_DRIFT_LOWER, lower),
            2, "Lower");

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, NULL, RSSPECAN_ATTR_BTO_LIMIT_CHECKS_INITIAL_FREQUENCY_DRIFT_UPPER, upper),
            3, "Upper");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Defines the lower and upper limit for the maximum frequency drift rate
/// HIFN  in LE signals.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR lower/Defines the lower limit for the maximum frequency drift rate in LE
/// HIPAR lower/signals.
/// HIPAR upper/Defines the lower limit for the maximum frequency drift rate in LE
/// HIPAR upper/signals.
ViStatus _VI_FUNC rsspecan_ConfigureBTOLimitChecksMaxDriftRate(
                            ViSession instrSession,
                            ViReal64  lower,
                            ViReal64  upper)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, NULL, RSSPECAN_ATTR_BTO_LIMIT_CHECKS_MAX_DRIFT_RATE_LOWER, lower),
            2, "Lower");

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, NULL, RSSPECAN_ATTR_BTO_LIMIT_CHECKS_MAX_DRIFT_RATE_UPPER, upper),
            3, "Upper");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Defines the lower and upper limit for the average frequency offset for
/// HIFN  the first test sequence (00001111).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR lower/Defines the lower limit for the average frequency offset for the first
/// HIPAR lower/test sequence (00001111).
/// HIPAR upper/Defines the upper limit for the average frequency offset for the first
/// HIPAR upper/test sequence (00001111).
ViStatus _VI_FUNC rsspecan_ConfigureBTOLimitChecksF1Avg(ViSession instrSession,
                                                        ViReal64  lower,
                                                        ViReal64  upper)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, NULL, RSSPECAN_ATTR_BTO_LIMIT_CHECKS_F1_AVG_LOWER, lower),
            2, "Lower");

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, NULL, RSSPECAN_ATTR_BTO_LIMIT_CHECKS_F1_AVG_UPPER, upper),
            3, "Upper");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Defines the limit for the maximum frequency offset for the first test
/// HIFN  sequence to be in range (for LE Coded).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR limit/Defines the limit for the maximum frequency offset for the first test
/// HIPAR limit/sequence to be in range (for LE Coded).
ViStatus _VI_FUNC rsspecan_ConfigureBTOLimitChecksF1MaxInRange(
                            ViSession instrSession,
                            ViReal64  limit)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, NULL, RSSPECAN_ATTR_BTO_LIMIT_CHECKS_F1_MAX_IN_RANGE, limit),
            2, "Limit");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Defines the lower limit for the maximum frequency offset for the first
/// HIFN  test sequence (for LE coded).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR limit/Defines the lower limit for the maximum frequency offset for the first
/// HIPAR limit/test sequence (for LE coded).
ViStatus _VI_FUNC rsspecan_ConfigureBTOLimitChecksF1MaxLower(ViSession instrSession,
                                                             ViReal64  limit)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, NULL, RSSPECAN_ATTR_BTO_LIMIT_CHECKS_F1_MAX_LOWER, limit),
            2, "Limit");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Defines the lower limit for the ratio of average f2 offset to average
/// HIFN  f1 offset.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR limit/Defines the lower limit for the ratio of average f2 offset to average
/// HIPAR limit/f1 offset.
ViStatus _VI_FUNC rsspecan_ConfigureBTOLimitChecksF2AvgF1AvgRatioLower(
                            ViSession instrSession,
                            ViReal64  limit)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, NULL, RSSPECAN_ATTR_BTO_LIMIT_CHECKS_F2_AVG_F1_AVG_RATIO_LOWER, limit),
            2, "Limit");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Defines the limit for the maximum frequency offset for the second test
/// HIFN  sequence (01010101) to be in range.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR limit/Defines the limit for the maximum frequency offset for the second test
/// HIPAR limit/sequence (01010101) to be in range.
ViStatus _VI_FUNC rsspecan_ConfigureBTOLimitChecksF2MaxInRange(
                            ViSession instrSession,
                            ViReal64  limit)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, NULL, RSSPECAN_ATTR_BTO_LIMIT_CHECKS_F2_MAX_IN_RANGE, limit),
            2, "Limit");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Defines the lower limit for the maximum frequency offset for the
/// HIFN  second test sequence (01010101).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR limit/Defines the lower limit for the maximum frequency offset for the
/// HIPAR limit/second test sequence (01010101).
ViStatus _VI_FUNC rsspecan_ConfigureBTOLimitChecksF2MaxLower(ViSession instrSession,
                                                             ViReal64  limit)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, NULL, RSSPECAN_ATTR_BTO_LIMIT_CHECKS_F2_MAX_LOWER, limit),
            2, "Limit");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Defines the lower and upper limit for the average output power.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR lower/Defines the lower limit for the average output power.
/// HIPAR upper/Defines the upper limit for the average output power.
ViStatus _VI_FUNC rsspecan_ConfigureBTOLimitChecksPowerAverage(
                            ViSession instrSession,
                            ViReal64  lower,
                            ViReal64  upper)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, NULL, RSSPECAN_ATTR_BTO_LIMIT_CHECKS_POWER_AVERAGE_LOWER, lower),
            2, "Lower");

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, NULL, RSSPECAN_ATTR_BTO_LIMIT_CHECKS_POWER_AVERAGE_UPPER, upper),
            3, "Upper");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  The (calculated) limit for the peak output power relative to the
/// HIFN  average output power.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR limit/The (calculated) limit for the peak output power relative to the
/// HIPAR limit/average output power.
ViStatus _VI_FUNC rsspecan_ConfigureBTOLimitChecksPowerRelativePeakUpper(
                            ViSession instrSession,
                            ViReal64  limit)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, NULL, RSSPECAN_ATTR_BTO_LIMIT_CHECKS_POWER_RELATIVE_PEAK_UPPER, limit),
            2, "Limit");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Defines the upper limit for the peak output power.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR limit/Defines the upper limit for the peak output power.
ViStatus _VI_FUNC rsspecan_ConfigureBTOLimitChecksPowerPeakUpper(
                            ViSession instrSession,
                            ViReal64  limit)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, NULL, RSSPECAN_ATTR_BTO_LIMIT_CHECKS_POWER_PEAK_UPPER, limit),
            2, "Limit");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Defines the lower and upper limit of the relative TX power.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR lower/Defines the lower limit of the relative TX power.
/// HIPAR upper/Defines the upper limit of the relative TX power.
ViStatus _VI_FUNC rsspecan_ConfigureBTOLimitChecksRelativeTXPowerAvg(
                            ViSession instrSession,
                            ViReal64  lower,
                            ViReal64  upper)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, NULL, RSSPECAN_ATTR_BTO_LIMIT_CHECKS_RELATIVE_TX_POWER_AVG_LOWER, lower),
            2, "Lower");

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, NULL, RSSPECAN_ATTR_BTO_LIMIT_CHECKS_RELATIVE_TX_POWER_AVG_UPPER, upper),
            3, "Upper");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Defines the lower and upper limit for the maximum frequency drift
/// HIFN  rate.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR packetType/Selects repeated capability BTOPacketType.
/// HIPAR lower/Defines the lower limit for the maximum frequency drift rate.
/// HIPAR upper/Defines the upper limit for the maximum frequency drift rate.
ViStatus _VI_FUNC rsspecan_ConfigureBTOLimitChecksPacketMaximumDrift(
                            ViSession instrSession,
                            ViInt32   packetType,
                            ViReal64  lower,
                            ViReal64  upper)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, packetType, RSSPECAN_REPCAP_BTOPACKETTYPE_BTOPACKETTYPE1, RSSPECAN_REPCAP_BTOPACKETTYPE_BTOPACKETTYPE5),
            2, "Packet Type");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_BTO_LIMIT_CHECKS_PACKET_MAXIMUM_DRIFT_LOWER,
            "BTOPacketType", packetType - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Packet Type");

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_BTO_LIMIT_CHECKS_PACKET_MAXIMUM_DRIFT_LOWER, lower),
            3, "Lower");

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_BTO_LIMIT_CHECKS_PACKET_MAXIMUM_DRIFT_UPPER, upper),
            4, "Upper");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Defines the lower and upper limit for the frequency drift for the
/// HIFN  specified packet type.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR packetType/Selects repeated capability BTOPacketType.
/// HIPAR lower/Defines the lower limit for the frequency drift for the specified
/// HIPAR lower/packet type.
/// HIPAR upper/Defines the upper limit for the frequency drift for the specified
/// HIPAR upper/packet type.
ViStatus _VI_FUNC rsspecan_ConfigureBTOLimitChecksPacketFrequencyDrift(
                            ViSession instrSession,
                            ViInt32   packetType,
                            ViReal64  lower,
                            ViReal64  upper)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, packetType, RSSPECAN_REPCAP_BTOPACKETTYPE_BTOPACKETTYPE1, RSSPECAN_REPCAP_BTOPACKETTYPE_BTOPACKETTYPE5),
            2, "Packet Type");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_BTO_LIMIT_CHECKS_PACKET_FREQUENCY_DRIFT_LOWER,
            "BTOPacketType", packetType - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Packet Type");

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_BTO_LIMIT_CHECKS_PACKET_FREQUENCY_DRIFT_LOWER, lower),
            3, "Lower");

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_BTO_LIMIT_CHECKS_PACKET_FREQUENCY_DRIFT_UPPER, upper),
            4, "Upper");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Defines the lower and upper limit of the initial frequency.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR lower/Defines the lower limit of the block frequency.
/// HIPAR upper/Defines the upper limit of the initial frequency.
ViStatus _VI_FUNC rsspecan_ConfigureBTOLimitChecksBlockFrequency(
                            ViSession instrSession,
                            ViReal64  lower,
                            ViReal64  upper)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, NULL, RSSPECAN_ATTR_BTO_LIMIT_CHECKS_BLOCK_FREQUENCY_LOWER, lower),
            2, "Lower");

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, NULL, RSSPECAN_ATTR_BTO_LIMIT_CHECKS_BLOCK_FREQUENCY_UPPER, upper),
            3, "Upper");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Defines the lower and upper limit of the initial frequency.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR lower/Defines the lower limit of the initial frequency.
/// HIPAR upper/Defines the upper limit of the initial frequency.
ViStatus _VI_FUNC rsspecan_ConfigureBTOLimitChecksInitialFrequency(
                            ViSession instrSession,
                            ViReal64  lower,
                            ViReal64  upper)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, NULL, RSSPECAN_ATTR_BTO_LIMIT_CHECKS_INITIAL_FREQUENCY_LOWER, lower),
            2, "Lower");

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, NULL, RSSPECAN_ATTR_BTO_LIMIT_CHECKS_INITIAL_FREQUENCY_UPPER, upper),
            3, "Upper");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Defines the lower and upper limit of the total frequency offset.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR lower/Defines the lower limit of the total frequency offset.
/// HIPAR upper/Defines the upper limit of the total frequency offset.
ViStatus _VI_FUNC rsspecan_ConfigureBTOLimitChecksTotalFrequencyOffset(
                            ViSession instrSession,
                            ViReal64  lower,
                            ViReal64  upper)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, NULL, RSSPECAN_ATTR_BTO_LIMIT_CHECKS_TOTAL_FREQUENCY_OFFSET_LOWER, lower),
            2, "Lower");

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, NULL, RSSPECAN_ATTR_BTO_LIMIT_CHECKS_TOTAL_FREQUENCY_OFFSET_UPPER, upper),
            3, "Upper");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Defines the upper limit for the peak DEVM for the DQPSK-modulated part
/// HIFN  of the signal.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR limit/Defines the upper limit for the peak DEVM for the DQPSK-modulated part
/// HIPAR limit/of the signal.
ViStatus _VI_FUNC rsspecan_ConfigureBTOLimitChecksPeakDQPSKUpper(
                            ViSession instrSession,
                            ViReal64  limit)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, NULL, RSSPECAN_ATTR_BTO_LIMIT_CHECKS_PEAK_DQPSK_UPPER, limit),
            2, "Limit");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Defines the upper limit for the RMS DEVM for the DQPSK-modulated part
/// HIFN  of the signal.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR limit/Defines the upper limit for the RMS DEVM for the DQPSK-modulated part
/// HIPAR limit/of the signal.
ViStatus _VI_FUNC rsspecan_ConfigureBTOLimitChecksRMSDQPSKUpper(
                            ViSession instrSession,
                            ViReal64  limit)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, NULL, RSSPECAN_ATTR_BTO_LIMIT_CHECKS_RMS_DQPSK_UPPER, limit),
            2, "Limit");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Defines the upper limit for the 99-percentile DEVM for the
/// HIFN  DQPSK-modulated part of the signal.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR limit/Defines the upper limit for the 99-percentile DEVM for the
/// HIPAR limit/DQPSK-modulated part of the signal.
ViStatus _VI_FUNC rsspecan_ConfigureBTOLimitChecks99DQPSKUpper(
                            ViSession instrSession,
                            ViReal64  limit)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, NULL, RSSPECAN_ATTR_BTO_LIMIT_CHECKS_99_DQPSK_UPPER, limit),
            2, "Limit");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Defines the upper limit for the peak DEVM for the 8PSK-modulated part
/// HIFN  of the signal.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR limit/Defines the upper limit for the peak DEVM for the 8PSK-modulated part
/// HIPAR limit/of the signal.
ViStatus _VI_FUNC rsspecan_ConfigureBTOLimitChecksPeak8PSKUpper(
                            ViSession instrSession,
                            ViReal64  limit)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, NULL, RSSPECAN_ATTR_BTO_LIMIT_CHECKS_PEAK_8PSK_UPPER, limit),
            2, "Limit");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Defines the upper limit for the RMS DEVM for the 8PSK-modulated part
/// HIFN  of the signal.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR limit/Defines the upper limit for the RMS DEVM for the 8PSK-modulated part
/// HIPAR limit/of the signal.
ViStatus _VI_FUNC rsspecan_ConfigureBTOLimitChecksRMS8PSKUpper(
                            ViSession instrSession,
                            ViReal64  limit)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, NULL, RSSPECAN_ATTR_BTO_LIMIT_CHECKS_RMS_8PSK_UPPER, limit),
            2, "Limit");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Defines the upper limit for the 99-percentile DEVM for the
/// HIFN  8PSK-modulated part of the signal.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR limit/Defines the upper limit for the 99-percentile DEVM for the
/// HIPAR limit/8PSK-modulated part of the signal.
ViStatus _VI_FUNC rsspecan_ConfigureBTOLimitChecks998PSKUpper(
                            ViSession instrSession,
                            ViReal64  limit)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, NULL, RSSPECAN_ATTR_BTO_LIMIT_CHECKS_99_8PSK_UPPER, limit),
            2, "Limit");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Defines the minimum power difference between the TX channel and the
/// HIFN  second or further alternate channels.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR limit/Defines the minimum power difference between the TX channel and the
/// HIPAR limit/second or further alternate channels.
ViStatus _VI_FUNC rsspecan_ConfigureBTOLimitChecksFarLimit(ViSession instrSession,
                                                           ViReal64  limit)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, NULL, RSSPECAN_ATTR_BTO_LIMIT_CHECKS_FAR_LIMIT, limit),
            2, "Limit");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Defines the minimum power difference between the TX channel and the
/// HIFN  first alternate channel.
/// HIFN
/// HIFN     This setting is not available for BT LE mode.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR limit/Defines the minimum power difference between the TX channel and the
/// HIPAR limit/first alternate channel.
ViStatus _VI_FUNC rsspecan_ConfigureBTOLimitChecksNearLimit(ViSession instrSession,
                                                            ViReal64  limit)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, NULL, RSSPECAN_ATTR_BTO_LIMIT_CHECKS_NEAR_LIMIT, limit),
            2, "Limit");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Defines the minimum power difference at a specified frequency offset
/// HIFN  from the transmit frequency.
/// HIFN
/// HIFN     This setting is only available for BT LE 1M and LE coded mode.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR powerDifference/Defines the minimum power difference at a specified frequency offset
/// HIPAR powerDifference/from the transmit frequency.
ViStatus _VI_FUNC rsspecan_ConfigureBTOLimitChecksL2FrequencyPowerDifference(
                            ViSession instrSession,
                            ViReal64  powerDifference)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, NULL, RSSPECAN_ATTR_BTO_LIMIT_CHECKS_L2_FREQUENCY_POWER_DIFFERENCE, powerDifference),
            2, "Power Difference");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Defines the minimum power difference at a specified frequency offset
/// HIFN  from the transmit frequency.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR powerDifference/Defines the minimum power difference at a specified frequency offset
/// HIPAR powerDifference/from the transmit frequency.
ViStatus _VI_FUNC rsspecan_ConfigureBTOLimitChecksL4FrequencyPowerDifference(
                            ViSession instrSession,
                            ViReal64  powerDifference)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, NULL, RSSPECAN_ATTR_BTO_LIMIT_CHECKS_L4_FREQUENCY_POWER_DIFFERENCE, powerDifference),
            2, "Power Difference");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Defines the minimum power difference at a specified frequency offset
/// HIFN  from the transmit frequency.
/// HIFN
/// HIFN     This setting is only available for BT LE 2M mode.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR powerDifference/Defines the minimum power difference at a specified frequency offset
/// HIPAR powerDifference/from the transmit frequency.
ViStatus _VI_FUNC rsspecan_ConfigureBTOLimitChecksL5FrequencyPowerDifference(
                            ViSession instrSession,
                            ViReal64  powerDifference)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, NULL, RSSPECAN_ATTR_BTO_LIMIT_CHECKS_L5_FREQUENCY_POWER_DIFFERENCE, powerDifference),
            2, "Power Difference");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Defines the number of acceptable exceptions for the limit check to be
/// HIFN  considered passed.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR allowedExepctions/Defines the number of acceptable exceptions for the limit check to be
/// HIPAR allowedExepctions/considered passed.
ViStatus _VI_FUNC rsspecan_ConfigureBTOLimitChecksAllowedExceptions(
                            ViSession instrSession,
                            ViInt32   allowedExceptions)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, NULL, RSSPECAN_ATTR_BTO_LIMIT_CHECKS_ALLOWED_EXCEPTIONS, allowedExceptions),
            2, "Allowed Exceptions");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Defines the minimum power difference between the TX channel and the
/// HIFN  adjacent channel.
/// HIFN
/// HIFN     This setting is not available for BT LE mode.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR limit/Defines the minimum power difference between the TX channel and the
/// HIPAR limit/adjacent channel.
ViStatus _VI_FUNC rsspecan_ConfigureBTOLimitChecksRelative(ViSession instrSession,
                                                           ViReal64  limit)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, NULL, RSSPECAN_ATTR_BTO_LIMIT_CHECKS_RELATIVE, limit),
            2, "Limit");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/*****************************************************************************
 * Function:   Configure BTO Analog Demodulation Zoom
 * Purpose:    This function sets analog demodulation parameters such as
 *             zoom state, start time.
 *****************************************************************************/
ViStatus _VI_FUNC rsspecan_ConfigureBTOAnalogDemodulationZoom(
                            ViSession instrSession,
                            ViBoolean zoomState,
                            ViReal64  startTime)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_FM_DEMOD_ZOOM_ENABLED, zoomState),
            2, "Zoom State");
    if (zoomState == VI_TRUE)
    {
        viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_FM_DEMOD_ZOOM_START, startTime),
                3, "Start Time");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/*****************************************************************************
 *- Measurement -------------------------------------------------------------*
 *****************************************************************************/

/*****************************************************************************
 * Function:    Get BTO Mesurement Status
 * Purpose:     This function returns the status of a preceeding measurement.
 *****************************************************************************/
ViStatus _VI_FUNC rsspecan_GetBTOMesurementStatus(ViSession instrSession,
                                                  ViInt32*  measurementStatus)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_GetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_BTO_GET_STATUS, measurementStatus),
            2, "Status");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/*****************************************************************************
 * Function:    Get BTO Packet Info
 * Purpose:     This function returns the packet info of of the packet
 *              analyzed by the preceeding measurement.
 *****************************************************************************/
ViStatus _VI_FUNC rsspecan_GetBTOPacketInfo(ViSession instrSession,
                                            ViInt32*  type,
                                            ViInt32*  length)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_GetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_BTO_GET_PACKET_TYPE, type),
            2, "Type");

    viCheckParm(rsspecan_GetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_BTO_GET_PACKET_LENGTH, length),
            3, "Length");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/*****************************************************************************
 * Function:    Get BTO Output Power
 * Purpose:     This function returns the output power results.
 *****************************************************************************/
ViStatus _VI_FUNC rsspecan_GetBTOOutputPower(ViSession instrSession,
                                             ViInt32   type,
                                             ViReal64* power)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    switch (type)
    {
    case RSSPECAN_VAL_MEASTYPE_MIN:
        viCheckParm(rsspecan_GetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_BTO_GET_OUTPUT_POWER_AVERAGE_MIN, power),
                3, "Power");
        break;
    case RSSPECAN_VAL_MEASTYPE_MAX:
        viCheckParm(rsspecan_GetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_BTO_GET_OUTPUT_POWER_AVERAGE_MAX, power),
                3, "Power");
        break;
    case RSSPECAN_VAL_MEASTYPE_PEAK:
        viCheckParm(rsspecan_GetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_BTO_GET_OUTPUT_POWER_PEAK, power),
                3, "Power");
        break;
    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, type), 2, "Type");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/*****************************************************************************
 * Function:    Get BTO Number Of Exceptions
 * Purpose:     This function returns the number of exceptions which occured
 *              during the adjacent channel power measurement according to
 *              the Bluetooth specification.
 *****************************************************************************/
ViStatus _VI_FUNC rsspecan_GetBTONumberOfExceptions(ViSession instrSession,
                                                    ViInt32*  numberOfExceptions)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_GetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_BTO_GET_NUMBER_OF_EXCEPTIONS, numberOfExceptions),
            2, "Number Of Exceptions");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/*****************************************************************************
 * Function:    Get BTO Power Of Channels
 * Purpose:     This function the power of selected adjacent channels of
 *              the Bluetooth standard.
 *****************************************************************************/
ViStatus _VI_FUNC rsspecan_GetBTOPowerOfChannels(ViSession instrSession,
                                                 ViInt32   arraySize,
                                                 ViReal64  power[],
                                                 ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K8"));

    viCheckParm(RsCore_InvalidViInt32RangeMin(instrSession, arraySize, 0),
                2, "Array Size");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, power), 3, "Power List");

    checkErr(
        RsCore_QueryFloatArrayToUserBuffer(instrSession, ":CALC:BTO:ACLR:LIST?", arraySize, power, returnedValues));

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/*****************************************************************************
 * Function:    Get BTO Frequency Deviation
 * Purpose:     This function returns the value of frequency deviation for
 *               varying bit patterns of the payload.
 *****************************************************************************/
ViStatus _VI_FUNC rsspecan_GetBTOFrequencyDeviation(ViSession instrSession,
                                                    ViInt32   frequencyDeviation,
                                                    ViInt32   type,
                                                    ViReal64* value)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_BTO_GET_MAXIMUM_FREQUENCY_DEVIATION_VALUE_MIN,
            "Frequency", frequencyDeviation - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Frequency Deviation");

    switch (type)
    {
    case RSSPECAN_VAL_MEASTYPE_MAXMIN:
        viCheckParm(rsspecan_GetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_BTO_GET_MAXIMUM_FREQUENCY_DEVIATION_VALUE_MIN, value),
                4, "Value");
        break;
    case RSSPECAN_VAL_MEASTYPE_MAXMAX:
        viCheckParm(rsspecan_GetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_BTO_GET_MAXIMUM_FREQUENCY_DEVIATION_VALUE_MAX, value),
                4, "Value");
        break;
    case RSSPECAN_VAL_MEASTYPE_MAXAVER:
        viCheckParm(rsspecan_GetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_BTO_GET_MAXIMUM_FREQUENCY_DEVIATION_VALUE_AVERAGE, value),
                4, "Value");
        break;
    case RSSPECAN_VAL_MEASTYPE_AVERMIN:
        viCheckParm(rsspecan_GetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_BTO_GET_AVERAGE_FREQUENCY_DEVIATION_MIN, value),
                4, "Value");
        break;
    case RSSPECAN_VAL_MEASTYPE_AVERMAX:
        viCheckParm(rsspecan_GetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_BTO_GET_AVERAGE_FREQUENCY_DEVIATION_MAX, value),
                4, "Value");
        break;
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/*****************************************************************************
 * Function:    Get BTO Ratio Average Frequency Deviation
 * Purpose:     This function returns the value of ratio of the average
 *              frequency deviations for varying bit patterns of the payload.
 *****************************************************************************/
ViStatus _VI_FUNC rsspecan_GetBTORatioAverageFrequencyDeviation(
                            ViSession instrSession,
                            ViInt32   type,
                            ViReal64* value)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    switch (type)
    {
    case RSSPECAN_VAL_MEASTYPE_MIN:
        viCheckParm(rsspecan_GetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_BTO_GET_RATIO_OF_AVERAGE_FREQUENCY_DEVIATIONS_VALUE_MIN, value),
                4, "Value");
        break;
    case RSSPECAN_VAL_MEASTYPE_MAX:
        viCheckParm(rsspecan_GetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_BTO_GET_RATIO_OF_AVERAGE_FREQUENCY_DEVIATIONS_VALUE_MAX, value),
                4, "Value");
        break;
    case RSSPECAN_VAL_MEASTYPE_AVER:
        viCheckParm(rsspecan_GetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_BTO_GET_RATIO_OF_AVERAGE_FREQUENCY_DEVIATIONS_VALUE_AVERAGE, value),
                4, "Value");
        break;
    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, type), 2, "Type");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/*****************************************************************************
 * Function:    Get BTO Percentage Deviation
 * Purpose:     This function returns the percentage of measurements of
 *              the frequency deviation.
 *****************************************************************************/
ViStatus _VI_FUNC rsspecan_GetBTOPercentageDeviation(ViSession instrSession,
                                                     ViReal64* value)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_BTO_GET_PERCENTAGE_DEVIATION, value),
            2, "Value");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/*****************************************************************************
 * Function:    Get BTO Packet Data Bits
 * Purpose:     This function returns the packet data bits.
 *****************************************************************************/
ViStatus _VI_FUNC rsspecan_GetBTOPacketDataBits(ViSession instrSession,
                                                ViUInt32  maximumTime,
                                                ViInt32   arrayLength,
                                                ViInt32*  actualPoints,
                                                ViReal64  values[])
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViUInt32Range(instrSession, maximumTime, 0, 4294967295UL), 2, "Maximum Time");

    checkErr(rsspecan_dataReadTrace(instrSession, 0, "BTOB", arrayLength, values, actualPoints));

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/*****************************************************************************
 * Function:    Read BTO FM Trace
 * Purpose:     This function returns FM trace with selected oversampling
 *              factor (Pointer per Symbol).
 *****************************************************************************/
ViStatus _VI_FUNC rsspecan_ReadBTOFMTrace(ViSession instrSession,
                                          ViUInt32  maximumTime,
                                          ViInt32   arrayLength,
                                          ViInt32*  actualPoints,
                                          ViReal64  amplitude[])
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViUInt32Range(instrSession, maximumTime, 0, 4294967295UL), 2, "Maximum Time");

    checkErr(rsspecan_dataReadTrace(instrSession, 0, "BTOF", arrayLength, amplitude, actualPoints));

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/*****************************************************************************
 * Function:    Get BTO Init Carr Freq Tol
 * Purpose:     This function returns value of the Initial Carrier
 *              Frequency Tolerance.
 *****************************************************************************/
ViStatus _VI_FUNC rsspecan_GetBTOInitCarrFreqTol(ViSession instrSession,
                                                 ViInt32   type,
                                                 ViReal64* value)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    switch (type)
    {
    case RSSPECAN_VAL_MEASTYPE_MIN:
        viCheckParm(rsspecan_GetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_BTO_GET_INITIAL_CARRIER_FREQUENCY_TOLERANCE_VALUE_MIN, value),
                4, "Value");
        break;
    case RSSPECAN_VAL_MEASTYPE_MAX:
        viCheckParm(rsspecan_GetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_BTO_GET_INITIAL_CARRIER_FREQUENCY_TOLERANCE_VALUE_MAX, value),
                4, "Value");
        break;
    case RSSPECAN_VAL_MEASTYPE_AVER:
        viCheckParm(rsspecan_GetAttributeViReal64(instrSession, "Aver", RSSPECAN_ATTR_BTO_GET_INITIAL_CARRIER_FREQUENCY_TOLERANCE_VALUE_AVERAGE, value),
                4, "Value");
        break;
    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, type), 2, "Type");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/*****************************************************************************
 * Function:    Get BTO Carr Freq Drift
 * Purpose:     This function returns the Carrier Frequency Drift.
 *****************************************************************************/
ViStatus _VI_FUNC rsspecan_GetBTOCarrFreqDrift(ViSession instrSession,
                                               ViInt32   type,
                                               ViReal64* value)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    switch (type)
    {
    case RSSPECAN_VAL_MEASTYPE_NONE:
        viCheckParm(rsspecan_GetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_BTO_GET_CARRIER_FREQUENCY_DRIFT, value),
                4, "Value");
        break;
    case RSSPECAN_VAL_MEASTYPE_MAX:
        viCheckParm(rsspecan_GetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_BTO_GET_MAXIMUM_CARRIER_FREQUENCY_DRIFT, value),
                4, "Value");
        break;
    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, type), 2, "Type");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/*****************************************************************************
 * Function: Configure BTO SE Num Adjacent Channels
 * Purpose:  This function selects the number of adjacent channel pairs during
 *           the In-band Spurious Emissions measurement.
 *****************************************************************************/
ViStatus _VI_FUNC rsspecan_ConfigureBTOSENumAdjacentChannels(ViSession instrSession,
                                                             ViInt32   number)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_BTO_IBS_ACP, number),
            2, "Number");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/*****************************************************************************
 * Function: Adjust BTO SE Gate Setting
 * Purpose:  This function adjusts the gate settings for the In-band
 *           Spurious Emissions automatically.
 *****************************************************************************/
ViStatus _VI_FUNC rsspecan_AdjustBTOSEGateSetting(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_BTO_IBS_GATE_AUTO, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/*****************************************************************************
 * Function: Get BTO Channel Power of SE Measurement
 * Purpose:  This function returns the power of the TX channel in the In-band Spurious
 *           Emissions measurement.
 *****************************************************************************/
ViStatus _VI_FUNC rsspecan_GetBTOChannelPowerOfSEMeasurement(ViSession instrSession,
                                                             ViInt32   parameter,
                                                             ViReal64* value)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    switch (parameter)
    {
    case RSSPECAN_VAL_BTO_SE_TX:
        viCheckParm(rsspecan_GetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_BTO_IBS_TXCHANNEL, value),
                4, "Value");
        break;
    case RSSPECAN_VAL_BTO_SE_HADJ_LOW:
        viCheckParm(rsspecan_GetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_BTO_IBS_HADJ_LOW, value),
                4, "Value");
        break;
    case RSSPECAN_VAL_BTO_SE_HADJ_UPP:
        viCheckParm(rsspecan_GetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_BTO_IBS_HADJ_UPP, value),
                4, "Value");
        break;
    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, parameter), 2, "Parameter");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/*****************************************************************************
 * Function: Get BTO SE Power Of Adjacent Channels
 * Purpose:  This function returns the power of all adjacent channels in the
 *           In-band Spurious Emissions measurement.
 *****************************************************************************/
ViStatus _VI_FUNC rsspecan_GetBTOSEPowerOfAdjacentChannels(ViSession instrSession,
                                                           ViInt32   arraySize,
                                                           ViReal64  power[],
                                                           ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K8"));

    viCheckParm(RsCore_InvalidViInt32RangeMin(instrSession, arraySize, 0),
                2, "Array Size");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, power), 3, "Power List");

    checkErr(RsCore_QueryFloatArrayToUserBuffer(instrSession, ":CALC:BTO:IBS?", arraySize, power, returnedValues));

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/*****************************************************************************
 * Function:    Get BTO SE Number of Exceptions
 * Purpose:     This function returns the number of results that exceeded
 *              the specified limits in the In-band Spurious Emissions measurement.
 *****************************************************************************/
ViStatus _VI_FUNC rsspecan_GetBTOSENumberOfExceptions(ViSession instrSession,
                                                      ViInt32*  exceptions)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_GetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_BTO_IBS_EXC, exceptions),
            2, "Exceptions");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/*****************************************************************************
 * Function:    Configure BTO CFS Block Count
 * Purpose:     This function sets the number of blocks to be measured.
 *****************************************************************************/
ViStatus _VI_FUNC rsspecan_ConfigureBTOCFSBlockCount(ViSession instrSession,
                                                     ViInt32   blockCount)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_BTO_CFST_BLOCK_COUNT, blockCount),
            2, "Block Count");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/*****************************************************************************
 * Function:    Get BTO Carrier Frequency Stability Deviation
 * Purpose:     This function returns value of the Carrier Frequency
 *              Stability Frequency Error.
 *****************************************************************************/
ViStatus _VI_FUNC rsspecan_GetBTOCFSDeviation(ViSession instrSession,
                                              ViInt32   parameter,
                                              ViInt32   type,
                                              ViReal64* value)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    switch (parameter)
    {
    case RSSPECAN_VAL_BTO_CFS_FERR_INITIAL:
        switch (type)
        {
        case RSSPECAN_VAL_MEASTYPE_MIN:
            viCheckParm(rsspecan_GetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_BTO_CFST_FERR_INITIAL_MIN, value),
                    4, "Value");
            break;
        case RSSPECAN_VAL_MEASTYPE_MAX:
            viCheckParm(rsspecan_GetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_BTO_CFST_FERR_INITIAL_MAX, value),
                    4, "Value");
            break;
        case RSSPECAN_VAL_MEASTYPE_AVER:
            viCheckParm(rsspecan_GetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_BTO_CFST_FERR_INITIAL_AVER, value),
                    4, "Value");
            break;
        default:
            viCheckParm(RsCore_InvalidViInt32Value(instrSession, parameter), 3, "Type");
        }

        break;
    case RSSPECAN_VAL_BTO_CFS_FERR_BLOCK:
        switch (type)
        {
        case RSSPECAN_VAL_MEASTYPE_MIN:
            viCheckParm(rsspecan_GetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_BTO_CFST_FERR_BLOCK_MIN, value),
                    4, "Value");
            break;
        case RSSPECAN_VAL_MEASTYPE_MAX:
            viCheckParm(rsspecan_GetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_BTO_CFST_FERR_BLOCK_MAX, value),
                    4, "Value");
            break;
        case RSSPECAN_VAL_MEASTYPE_AVER:
            viCheckParm(rsspecan_GetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_BTO_CFST_FERR_BLOCK_AVER, value),
                    4, "Value");
            break;
        default:
            viCheckParm(RsCore_InvalidViInt32Value(instrSession, parameter), 3, "Type");
        }
        break;
    case RSSPECAN_VAL_BTO_CFS_FERR_TOTAL:
        switch (type)
        {
        case RSSPECAN_VAL_MEASTYPE_MIN:
            viCheckParm(rsspecan_GetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_BTO_CFST_FERR_TOTAL_MIN, value),
                    4, "Value");
            break;
        case RSSPECAN_VAL_MEASTYPE_MAX:
            viCheckParm(rsspecan_GetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_BTO_CFST_FERR_TOTAL_MAX, value),
                    4, "Value");
            break;
        case RSSPECAN_VAL_MEASTYPE_AVER:
            viCheckParm(rsspecan_GetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_BTO_CFST_FERR_TOTAL_AVER, value),
                    4, "Value");
            break;
        default:
            viCheckParm(RsCore_InvalidViInt32Value(instrSession, parameter), 3, "Type");
        }
        break;
    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, parameter), 2, "Parameter");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/*****************************************************************************
 * Function:    Get BTO Carrier Frequency Stability Modul. Accuracy
 * Purpose:     This function returns value of the Carrier Frequency
 *              Stability Modulation Accuracy.
 *****************************************************************************/
ViStatus _VI_FUNC rsspecan_GetBTOCFSModulAccuracy(ViSession instrSession,
                                                  ViInt32   parameter,
                                                  ViInt32   type,
                                                  ViReal64* value)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    switch (type)
    {
    case RSSPECAN_VAL_MEASTYPE_MIN:
        snprintf(repCap, RS_REPCAP_BUF_SIZE, "Min");
        break;
    case RSSPECAN_VAL_MEASTYPE_MAX:
        snprintf(repCap, RS_REPCAP_BUF_SIZE, "Max");
        break;
    case RSSPECAN_VAL_MEASTYPE_AVER:
        snprintf(repCap, RS_REPCAP_BUF_SIZE, "Aver");
        break;
    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, type), 3, "Type");
    }
    switch (parameter)
    {
    case RSSPECAN_VAL_BTO_CFS_DEVM_RMS:
        switch (type)
        {
        case RSSPECAN_VAL_MEASTYPE_MIN:
            viCheckParm(rsspecan_GetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_BTO_CFST_DEVM_RMS_MIN, value),
                    4, "Value");
            break;
        case RSSPECAN_VAL_MEASTYPE_MAX:
            viCheckParm(rsspecan_GetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_BTO_CFST_DEVM_RMS_MAX, value),
                    4, "Value");
            break;
        case RSSPECAN_VAL_MEASTYPE_AVER:
            viCheckParm(rsspecan_GetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_BTO_CFST_DEVM_RMS_AVER, value),
                    4, "Value");
            break;
        default:
            viCheckParm(RsCore_InvalidViInt32Value(instrSession, parameter), 3, "Type");
        }
        break;
    case RSSPECAN_VAL_BTO_CFS_DEVM_99PCT:
        viCheckParm(rsspecan_GetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_BTO_CFST_DEVM_99PCT, value),
                4, "Value");
        break;
    case RSSPECAN_VAL_BTO_CFS_DEVM_PEAK:
        viCheckParm(rsspecan_GetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_BTO_CFST_DEVM_PEAK, value),
                4, "Value");
        break;
    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, parameter), 2, "Parameter");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/*****************************************************************************
 * Function:    Get BTO Differential Phase Encoding Values
 * Purpose:     This function returns the measured valeus of the differential
 *              phase encoding.
 *****************************************************************************/
ViStatus _VI_FUNC rsspecan_GetBTODifferentialPhaseEncodingValues(
                            ViSession instrSession,
                            ViInt32   parameter,
                            ViInt32*  value)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    switch (parameter)
    {
    case RSSPECAN_VAL_BTO_DPEN_TESTED:
        viCheckParm(rsspecan_GetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_BTO_DPEN_PACKETS, value),
                4, "Value");
        break;
    case RSSPECAN_VAL_BTO_DPEN_BER:
        viCheckParm(rsspecan_GetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_BTO_DPEN_BER, value),
                4, "Value");
        break;
    case RSSPECAN_VAL_BTO_DPEN_PASSED:
        viCheckParm(rsspecan_GetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_BTO_DPEN_PACKETS_PASSED, value),
                4, "Value");
        break;
    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, parameter), 2, "Parameter");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/*****************************************************************************
 * Function:    Configure BTO RTP Power Measurement
 * Purpose:     This function configurest the start and stop time for power
 *              measurement on slected packet sections.
 *****************************************************************************/
ViStatus _VI_FUNC rsspecan_ConfigureBTORTPMeasurement(ViSession instrSession,
                                                      ViInt32   packetSection,
                                                      ViReal64  start,
                                                      ViReal64  stop)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    switch (packetSection)
    {
    case RSSPECAN_VAL_BTO_RTP_GFSK:
        viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_BTO_RTP_TIME_GFSK_START, start),
                3, "Start");

        viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_BTO_RTP_TIME_GFSK_STOP, stop),
                4, "Stop");
        break;
    case RSSPECAN_VAL_BTO_RTP_DPSK:
        viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_BTO_RTP_TIME_DPSK_START, start),
                3, "Start");

        viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_BTO_RTP_TIME_DPSK_STOP, stop),
                4, "Stop");
        break;
    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, packetSection), 2, "Packet Section");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/*****************************************************************************
 * Function:    Get BTO Relative Transmission Power
 * Purpose:     This function returns the transmission power of the Relative
 *              Transmission Power measurement.

 *****************************************************************************/
ViStatus _VI_FUNC rsspecan_GetBTORelativeTransmissionPower(ViSession instrSession,
                                                           ViInt32   parameter,
                                                           ViInt32   modificator,
                                                           ViReal64* value)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    switch (parameter)
    {
    case RSSPECAN_VAL_BTO_RTP_DPSK:
        switch (modificator)
        {
        case RSSPECAN_VAL_MEASTYPE_MIN:
            viCheckParm(rsspecan_GetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_BTO_RTP_DPSK_MIN, value),
                    4, "Value");
            break;
        case RSSPECAN_VAL_MEASTYPE_MAX:
            viCheckParm(rsspecan_GetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_BTO_RTP_DPSK_MAX, value),
                    4, "Value");
            break;
        case RSSPECAN_VAL_MEASTYPE_AVER:
            viCheckParm(rsspecan_GetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_BTO_RTP_DPSK_AVER, value),
                    4, "Value");
            break;
        default:
            viCheckParm(RsCore_InvalidViInt32Value(instrSession, parameter), 3, "Modificator");
        }

        break;
    case RSSPECAN_VAL_BTO_RTP_GFSK:
        switch (modificator)
        {
        case RSSPECAN_VAL_MEASTYPE_MIN:
            viCheckParm(rsspecan_GetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_BTO_RTP_GFSK_MIN, value),
                    4, "Value");
            break;
        case RSSPECAN_VAL_MEASTYPE_MAX:
            viCheckParm(rsspecan_GetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_BTO_RTP_GFSK_MAX, value),
                    4, "Value");
            break;
        case RSSPECAN_VAL_MEASTYPE_AVER:
            viCheckParm(rsspecan_GetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_BTO_RTP_GFSK_AVER, value),
                    4, "Value");
            break;
        default:
            viCheckParm(RsCore_InvalidViInt32Value(instrSession, parameter), 3, "Modificator");
        }
        break;
    case RSSPECAN_VAL_BTO_RTP_RATIO:
        switch (modificator)
        {
        case RSSPECAN_VAL_MEASTYPE_MIN:
            viCheckParm(rsspecan_GetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_BTO_RTP_RATIO_MIN, value),
                    4, "Value");
            break;
        case RSSPECAN_VAL_MEASTYPE_MAX:
            viCheckParm(rsspecan_GetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_BTO_RTP_RATIO_MAX, value),
                    4, "Value");
            break;
        case RSSPECAN_VAL_MEASTYPE_AVER:
            viCheckParm(rsspecan_GetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_BTO_RTP_RATIO_AVER, value),
                    4, "Value");
            break;
        default:
            viCheckParm(RsCore_InvalidViInt32Value(instrSession, parameter), 3, "Modificator");
        }
        break;
    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, parameter), 2, "Parameter");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/*****************************************************************************
 *- End Of RsSpecAn K8 - Bluetooth Analyzer Group Functions------------------*
 *****************************************************************************/
