/*****************************************************************************
 *  Rohde&Schwarz Spectrum Analyzer instrument driver
 *  K9 - Power Meter
 *
 *  Original Release: September 2005
 *  By: Martin Koutny (instrument driver core)
 *      Helena Sucha (instrument specific implementation)
 *      Jiri Kominek (DrBase repository & tools)
 *
 *  Should you have any technical questions please contact the hotline of
 *  Rohde & Schwarz Vertriebs-GmbH Rohde & Schwarz Support Center
 *
 *  e-mail: CustomerSupport@rohde-schwarz.com
 *

 *****************************************************************************/

#include "rsspecan.h"

/*****************************************************************************
 * Function: PWM Mode
 * Purpose:  This function switches measurements with a power sensor on or
 *           off.
*****************************************************************************/
ViStatus _VI_FUNC rsspecan_PWMMode(ViSession instrSession,
                                   ViInt32   sensor,
                                   ViBoolean state)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_PWM_ENABLED,
            "PowerMeter", sensor , RS_REPCAP_BUF_SIZE, repCap),
            2, "Sensor");
    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, repCap, RSSPECAN_ATTR_PWM_ENABLED, state),
            2, "State");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/*****************************************************************************
 * Function: Configure PWM Measurement
 * Purpose:  This function configures the power meter measurement
*****************************************************************************/
ViStatus _VI_FUNC rsspecan_ConfigurePWMMeasurement(ViSession instrSession,
                                                   ViInt32   sensor,
                                                   ViInt32   measurementTime,
                                                   ViInt32   coupling,
                                                   ViReal64  frequency,
                                                   ViInt32   averageCount)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_PWM_AVERAGE_AUTO,
            "PowerMeter", sensor , RS_REPCAP_BUF_SIZE, repCap),
            2, "Sensor");

    switch (measurementTime)
    {
    case RSSPECAN_VAL_PWMET_MEASTIME_MANUAL:
        checkErr(rsspecan_SetAttributeViBoolean(instrSession, repCap, RSSPECAN_ATTR_PWM_AVERAGE_AUTO, VI_TRUE));
        checkErr(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_PWM_AVERAGE_COUNT, averageCount));
        break;
    case RSSPECAN_VAL_PWMET_MEASTIME_NORM:
    case RSSPECAN_VAL_PWMET_MEASTIME_SHORT:
    case RSSPECAN_VAL_PWMET_MEASTIME_LONG:
        checkErr(rsspecan_SetAttributeViBoolean(instrSession, repCap, RSSPECAN_ATTR_PWM_AVERAGE_AUTO, VI_FALSE));
        checkErr(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_PWM_MEASUREMENT_TIME, measurementTime));
        break;
    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, measurementTime), 3, "Measurement Time");
    }
    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_PWM_FREQUENCY_COUPLING, coupling),
            4, "Coupling");
    if (coupling == RSSPECAN_VAL_PMET_COUP_OFF)
    {
        viCheckParm(rsspecan_SetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_PWM_FREQUENCY, frequency),
                5, "Frequency");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/*****************************************************************************
 * Function: Configure PWM Relative
 * Purpose:  This function configures the relative measurement.
*****************************************************************************/
ViStatus _VI_FUNC rsspecan_ConfigurePWMRelative(ViSession instrSession,
                                                ViInt32   sensor,
                                                ViReal64  referenceValue,
                                                ViInt32   unit)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_PWM_RELABS_DISPLAY,
            "PowerMeter", sensor , RS_REPCAP_BUF_SIZE, repCap),
            2, "Sensor");

    checkErr(rsspecan_SetAttributeViBoolean(instrSession, repCap, RSSPECAN_ATTR_PWM_RELABS_DISPLAY, VI_TRUE));

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_PWM_REFERENCE_VALUE, referenceValue),
            3, "Reference Value");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_PWM_RELATIVE_UNITS, unit),
            4, "Unit");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/*****************************************************************************
 * Function: Configure PWM Absolute
 * Purpose:  This function configures the relative measurement.
*****************************************************************************/
ViStatus _VI_FUNC rsspecan_ConfigurePWMAbsolute(ViSession instrSession,
                                                ViInt32   sensor,
                                                ViInt32   unit)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_PWM_RELABS_DISPLAY,
            "PowerMeter", sensor , RS_REPCAP_BUF_SIZE, repCap),
            2, "Sensor");

    checkErr(rsspecan_SetAttributeViBoolean(instrSession, repCap, RSSPECAN_ATTR_PWM_RELABS_DISPLAY, VI_FALSE));
    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_PWM_ABSOLUTE_UNIT, unit),
            3, "Unit");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/*****************************************************************************
 * Function: Configure PWM Reference Level Offset State
 * Purpose:  This function defines whether the reference level offset set for the analyzer
is taken into account for the measured power or not.
*****************************************************************************/
ViStatus _VI_FUNC rsspecan_ConfigurePWMReferenceLevelOffsetState(
                            ViSession instrSession,
                            ViInt32   sensor,
                            ViBoolean state)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_PWM_REFERENCE_LEVEL_OFFSET_ENABLED,
            "PowerMeter", sensor , RS_REPCAP_BUF_SIZE, repCap),
            2, "Sensor");

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, repCap, RSSPECAN_ATTR_PWM_REFERENCE_LEVEL_OFFSET_ENABLED, state),
            3, "State");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/*****************************************************************************
 * Function: Configure PWM External Power Trigger
 * Purpose:  This function accepts the current result as a reference value
 *           for relative measurements.
*****************************************************************************/
ViStatus _VI_FUNC rsspecan_ConfigurePWMExternalPowerTrigger(ViSession instrSession,
                                                            ViInt32   sensor,
                                                            ViBoolean state,
                                                            ViReal64  level)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_PWM_EXTERNAL_POWER_TRIGGER_ENABLED,
            "PowerMeter", sensor , RS_REPCAP_BUF_SIZE, repCap),
            2, "Sensor");

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, repCap, RSSPECAN_ATTR_PWM_EXTERNAL_POWER_TRIGGER_ENABLED, state),
            3, "State");

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_PWM_EXTERNAL_POWER_TRIGGER_LEVEL, level),
            4, "Level");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function configures the external power trigger advanced settings.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init.
/// HIPAR hysteresis/Sets the value for the trigger hysteresis of an external power trigger
/// HIPAR dropoutTime/Sets the time the input signal must stay below the IF power
/// HIPAR dropoutTime/trigger level in order to allow a trigger to start the measurement.
/// HIPAR holdoffTime/Sets the holdoff value in s, which is the time which must pass
/// HIPAR holdoffTime/before triggering, in case another trigger event happens.
/// HIPAR slope/Defines whether the sweep starts after a positive or negative
/// HIPAR slope/edge of the trigger signal.
ViStatus _VI_FUNC rsspecan_ConfigurePWMExternalPowerTriggerAdvanced(
                            ViSession instrSession,
                            ViInt32   sensor,
                            ViInt32   hysteresis,
                            ViReal64  dropoutTime,
                            ViReal64  holdoffTime,
                            ViInt32   slope)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_PWM_EXTERNAL_POWER_TRIGGER_HYSTERESIS,
            "PowerMeter", sensor , RS_REPCAP_BUF_SIZE, repCap),
            2, "Sensor");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_PWM_EXTERNAL_POWER_TRIGGER_HYSTERESIS, hysteresis),
            3, "Hysteresis");

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_PWM_EXTERNAL_POWER_TRIGGER_DROPOUT_TIME, dropoutTime),
            4, "Dropout Time");

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_PWM_EXTERNAL_POWER_TRIGGER_HOLDOFF_TIME, holdoffTime),
            4, "Holdoff Time");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_PWM_EXTERNAL_POWER_TRIGGER_SLOPE, slope),
            3, "Slope");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets and queries the toggle to apply the auto level correction.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR autoLevel/Sets and queries the toggle to apply the auto level correction.
ViStatus _VI_FUNC rsspecan_ConfigurePWMCorrectionLevelAuto(ViSession instrSession,
                                                           ViBoolean autoLevel)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, NULL, RSSPECAN_ATTR_PWM_CORRECTION_LEVEL_AUTO, autoLevel),
            2, "Auto Level");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Calculates the level correction for power sensors.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR level/Calculates the level correction for power sensors.
ViStatus _VI_FUNC rsspecan_QueryPWMCorrectionLevel(ViSession instrSession,
                                                   ViReal64* level)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, NULL, RSSPECAN_ATTR_PWM_CORRECTION_LEVEL, level),
            2, "Level");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}



/*****************************************************************************
 * Function: Configure PWM Duty Cycle
 * Purpose:  This function configures the duty cycle correction.
*****************************************************************************/
ViStatus _VI_FUNC rsspecan_ConfigurePWMDutyCycle(ViSession instrSession,
                                                 ViInt32   sensor,
                                                 ViBoolean dutyCycle,
                                                 ViReal64  dutyCycleValue)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_PWM_DUTY_CYCLE_ENABLED,
            "PowerMeter", sensor , RS_REPCAP_BUF_SIZE, repCap),
            2, "Sensor");

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, repCap, RSSPECAN_ATTR_PWM_DUTY_CYCLE_ENABLED, dutyCycle),
            2, "Duty Cycle");

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_PWM_DUTY_CYCLE, dutyCycleValue),
            3, "Duty Cycle Value");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function assign the connected power sensors to the configuration sets.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
/// HIPAR autoAssignment/Defines whether the selected power sensor index is automatically assigned to a subsequently connected power sensor.
/// HIPAR placeholder/This control is reserved for future use.
/// HIPAR type/The power sensor type.
/// HIPAR intfc/This control is reserved for future use.
/// HIPAR serialNumber/The power sensor serial number.
ViStatus _VI_FUNC rsspecan_ConfigurePowerSensorAssignment(ViSession instrSession,
                                                          ViInt32   sensor,
                                                          ViBoolean autoAssignment,
                                                          ViString  placeholder,
                                                          ViString  type,
                                                          ViString  intfc,
                                                          ViString  serialNumber)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    // For future use
    placeholder;
    intfc;

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_PWM_AUTOMATIC_ASSIGNMENT,
            "PowerMeter", sensor , RS_REPCAP_BUF_SIZE, repCap),
            2, "Sensor");

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, repCap, RSSPECAN_ATTR_PWM_AUTOMATIC_ASSIGNMENT, autoAssignment),
            2, "Auto Assignment");

    if (autoAssignment == VI_FALSE) // if not AutoAssign only
    {
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SYST:COMM:RDEV:PMET%ld:DEF '%s','%s','%s','%s'", sensor, placeholder,
                 type, intfc, serialNumber);
        checkErr(RsCore_Write(instrSession, cmd));
    }

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the number of power sensors currently connected to the instrument.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
/// HIPAR powerSensorCount/Queries the number of power sensors currently connected to the instrument.
ViStatus _VI_FUNC rsspecan_QueryPowerSensorCount(ViSession instrSession,
                                                 ViInt32*  powerSensorCount)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_GetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_PWM_NUMBER_OF_SENSORS, powerSensorCount),
            2, "Power Sensor Count");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN Turns continuous update of power sensor measurements on and off. If on, the results are update even if a single sweep is complete.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR sensor/Document parameter here.
/// HIPAR state/Document parameter here.
ViStatus _VI_FUNC rsspecan_ConfigurePWMContinuousUpdate(ViSession instrSession,
                                                        ViInt32   sensor,
                                                        ViBoolean state)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_PWM_CONTINUOUS_UPDATE_ENABLED,
            "PowerMeter", sensor , RS_REPCAP_BUF_SIZE, repCap),
            2, "Sensor");

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, repCap, RSSPECAN_ATTR_PWM_CONTINUOUS_UPDATE_ENABLED, state),
            3, "State");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Takes the specified offset into account for the measured power.
/// HIFN
/// HIFN     Note:
/// HIFN     (1) This function is available for R&S FSW or with option R&S FSV-K9
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR sensor/Selects the sensor.
/// HIPAR offset/Takes the specified offset into account for the measured power.
ViStatus _VI_FUNC rsspecan_ConfigurePWMSensorOffset(ViSession instrSession,
                                                    ViInt32   sensor,
                                                    ViReal64  offset)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_PWM_SENSOR_OFFSET,
            "PowerMeter", sensor , RS_REPCAP_BUF_SIZE, repCap),
            2, "Sensor");

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_PWM_SENSOR_OFFSET, offset),
            3, "Offset");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/*****************************************************************************
 * Function: Configure PWM Measure To Reference
 * Purpose:  This function accepts the current result as a reference value
 *           for relative measurements.
*****************************************************************************/
ViStatus _VI_FUNC rsspecan_ConfigurePWMMeasToRef(ViSession instrSession,
                                                 ViInt32   sensor)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_PWM_MEASURE_TO_REFERENCE,
            "PowerMeter", sensor , RS_REPCAP_BUF_SIZE, repCap),
            2, "Sensor");

    checkErr(rsspecan_SetAttributeViString(instrSession, repCap, RSSPECAN_ATTR_PWM_MEASURE_TO_REFERENCE, ""));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/*****************************************************************************
 * Function: Configure PWM Zero
 * Purpose:  This function zeroes the power sensor.
*****************************************************************************/
ViStatus _VI_FUNC rsspecan_ConfigurePWMZero(ViSession instrSession,
                                            ViInt32   sensor)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_PWM_ZERO,
            "PowerMeter", sensor , RS_REPCAP_BUF_SIZE, repCap),
            2, "Sensor");

    checkErr(rsspecan_SetAttributeViString(instrSession, repCap, RSSPECAN_ATTR_PWM_ZERO, ""));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/*****************************************************************************
 * Function: Read PWM Result
 * Purpose:  This function triggers a measurement with the power sensor and
 *           then outputs the result.
*****************************************************************************/
ViStatus _VI_FUNC rsspecan_ReadPWMResult(ViSession instrSession,
                                         ViInt32   sensor,
                                         ViUInt32  timeout,
                                         ViReal64* result)
{
    ViStatus error = VI_SUCCESS;
    ViUInt32 old_timeout = 0;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_PWM_READ_RESULT,
            "PowerMeter", sensor , RS_REPCAP_BUF_SIZE, repCap),
            2, "Sensor");
    viCheckParm(RsCore_InvalidViUInt32Range(instrSession, timeout, 0, 4294967295UL), 3, "Timeout");

    checkErr(rsspecan_GetTimeout(instrSession, &old_timeout));
    checkErr(rsspecan_SetTimeout(instrSession, timeout));
    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_PWM_READ_RESULT, result),
            3, "Result");

Error:
    if (old_timeout > 0)
        rsspecan_SetTimeout(instrSession, old_timeout);
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/*****************************************************************************
 * Function: Fetch PWM Result
 * Purpose:  This function outputs the result of the power sensor.
*****************************************************************************/
ViStatus _VI_FUNC rsspecan_FetchPWMResult(ViSession instrSession,
                                          ViInt32   sensor,
                                          ViReal64* result)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_PMET_FETCH,
            "PowerMeter", sensor , RS_REPCAP_BUF_SIZE, repCap),
            2, "Sensor");
    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_PMET_FETCH, result),
            3, "Result");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/*****************************************************************************
 *- End Of RSSpecAn K9 - Power Meter ----------------------------------------*
 *****************************************************************************/
