/*****************************************************************************
 *  Rohde&Schwarz Spectrum Analyzer instrument driver
 *  WIGIG option K95
 *
 *  Original Release: December 2015
 *  By: Nabil Ouddane
 *
 *  Should you have any technical questions please contact the hotline of
 *  Rohde & Schwarz Vertriebs-GmbH Rohde & Schwarz Support Center
 *
 *  e-mail: CustomerSupport@rohde-schwarz.com
 *
 *****************************************************************************/

#include "rsspecan.h"

static ViString offOnArr[] = {"OFF", "ON", NULL};

/// HIFN This function selects the WiGig (IEEE 802.11ad) mode.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
ViStatus _VI_FUNC rsspecan_SetWiGigMode(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_WIGIG_MODE, ""));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function performs the automatic power level detection once at the start of the next measurement sweep, and the reference level is adapted accordingly.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
ViStatus _VI_FUNC rsspecan_ConfigureWiGigWlanAutoLevelOnce(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_WIGIG_AUTO_LEVEL_ONCE, ""));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function configures the reference and signal level and offset.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR autoLevel/Switches on or off automatic power level detection.
/// HIPAR referenceLevel/ The calibrated vertical position of the captured data used as a
/// HIPAR referenceLevel/reference for amplitude measurements. This is typically set to a
/// HIPAR referenceLevel/value slightly higher than the highest expected signal level.
/// HIPAR referenceLevelOffset/Specifies the offset value to the reference level.
/// HIPAR units/Specifies the amplitude units for input, output and display
/// HIPAR units/Pass the units you want the spectrum analyzer to use.
ViStatus _VI_FUNC rsspecan_ConfWiGigReferenceLevel(ViSession instrSession,
                                                   ViReal64  referenceLevel,
                                                   ViReal64  referenceLevelOffset,
                                                   ViInt32   units)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_REFERENCE_LEVEL, referenceLevel),
            3, "Reference Level");

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_REFERENCE_LEVEL_OFFSET, referenceLevelOffset),
            4, "Reference Level Offset");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_AMPLITUDE_UNITS, units),
            5, "Units");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function configures the tracking parameters.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR phase/Activates or deactivates the compensation for phase drifts.
/// HIPAR timing/Activates or deactivates the compensation for timing drift.
/// HIPAR level/Activates or deactivates the compensation for level variations within a single PPDU.
ViStatus _VI_FUNC rsspecan_ConfigureWiGigTracking(ViSession instrSession,
                                                  ViBoolean phase,
                                                  ViBoolean timing,
                                                  ViBoolean level)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_WIGIG_TRACKING_PHASE_ENABLED, phase),
            2, "Phase");

    // not yet implemented
    timing;
    /*viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "",
                                                RSSPECAN_ATTR_WIGIG_TRACKING_TIMING_ENABLED,
                                                timing), 3, "Timing");*/

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_WIGIG_TRACKING_LEVEL_ENABLED, level),
            4, "Level");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function configures the compensation for I/Q mismatch
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR IQMismatchCompensation/Activates or deactivates the compensation for I/Q mismatch.
ViStatus _VI_FUNC rsspecan_ConfigureWiGigIQMismatchCompensation(
                            ViSession instrSession,
                            ViBoolean IQMismatchCompensation)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_WIGIG_IQ_COMPENSATION_ENABLED, IQMismatchCompensation),
            2, "IQ Mismatch Compensation");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Contains settings for data capture rates.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR contiguousNumber/Indicates the number of contiguous 2.16 GHz channels the measurement
/// HIPAR contiguousNumber/was made for. Note that the R&S FSW hardware currently only supports
/// HIPAR contiguousNumber/measurement bandwidths up to a maximum of 5 GHz (using bandwidth
/// HIPAR contiguousNumber/extension options). Thus, measurements with 3 or 4 channels can only be
/// HIPAR contiguousNumber/performed on data from input files or using downsampling (see
/// HIPAR contiguousNumber/CONFigure:EDMG:CRATe).
/// HIPAR usedChipRate/Determines whether the chip rate according to standard is used or a
/// HIPAR usedChipRate/non-standard measurement is performed with a user-defined chip rate.
/// HIPAR chipRate/Chip rate used for transmission; specified in the IEEE 802.11 ay
/// HIPAR chipRate/standard as: NCB * 1.7 GHz
ViStatus _VI_FUNC rsspecan_ConfigureWiGigDataAcquisitionRates(
                            ViSession instrSession,
                            ViInt32   contiguousNumber,
                            ViInt32   usedChipRate,
                            ViReal64  chipRate)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_EDMG_CONTIGUOUS_NUMBER, contiguousNumber),
            2, "Contiguous Number");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_EDMG_USER_CHIP_RATE, usedChipRate),
            3, "Used Chip Rate");

    if (usedChipRate == RSSPECAN_VAL_USER_DEFINED_CHIP_RATE_CUSTOM)
    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_EDMG_CHIP_RATE, chipRate),
            4, "Chip Rate");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Selects the I/Q data file to be used as input for further
/// HIFN  measurements. The I/Q data must have a specific format described in
/// HIFN  operating manual.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR fileName/String containing the path and name of the source file. The file
/// HIPAR fileName/extension is *.iq.tar.
/// HIPAR analysisBW/Optional. The analysis bandwidth to be used by the measurement. The
/// HIPAR analysisBW/bandwidth must be smaller than or equal to the bandwidth of the data
/// HIPAR analysisBW/that was stored in the file. If value is set to 0.0, the control is
/// HIPAR analysisBW/ignored.
ViStatus _VI_FUNC rsspecan_SelectWiGigIQFile(ViSession instrSession,
                                             ViString  fileName,
                                             ViReal64  analysisBW)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "INP:FILE:PATH '%s',%.12G", fileName, analysisBW);
    checkErr(RsCore_Write(instrSession, cmd));
    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Loads a spectrum emission mask setup from an xml file.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR fileName/This control is used to load a spectrum emission mask setup from an
/// HIPAR fileName/xml file
ViStatus _VI_FUNC rsspecan_LoadWiGigSEMFile(ViSession instrSession,
                                            ViString  fileName)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViString(instrSession, NULL, RSSPECAN_ATTR_WIGIG_SEM_FILE, fileName),
            2, "File Name");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function adds a window to the display.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR stringSize/Sets the allocated length of Returned Window Name string.
/// HIPAR referenceWindowName/String containing the number or name of the window
/// HIPAR direction/Direction the window is added relative to the window set with "Reference Window Name".
/// HIPAR windowType/Type of result display you want to add.
/// HIPAR returnedWindowName/When adding a new window, the function returns
/// HIPAR returnedWindowName/its name (by default the same as its number) at the same time.
ViStatus _VI_FUNC rsspecan_AddWiGigWindow(ViSession instrSession,
                                          ViInt32   stringSize,
                                          ViString  referenceWindowName,
                                          ViInt32   direction,
                                          ViInt32   windowType,
                                          ViChar    returnedWindowName[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViChar   response[RS_MAX_MESSAGE_BUF_SIZE];
    ViChar*  p2direction;
    ViChar*  p2type;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &rsspecan_rngLayoutDirection, direction, &p2direction),
                4, "Direction");
    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &rsspecan_rngWIGIGLayoutWindowType, windowType, &p2type),
                5, "Window Type");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "LAY:ADD? '%s',%s,%s", referenceWindowName, p2direction, p2type);
    checkErr(RsCore_QueryViString(instrSession, cmd, response));

    if (returnedWindowName != NULL)
    {
        RsCore_TrimString(response, RS_VAL_TRIM_WHITESPACES_AND_ALL_QUOTES);
        RsCore_CopyToUserBufferAsciiData(returnedWindowName, stringSize, response);
    }

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function replaces a window with another one.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR windowName/String containing the number or name of the window you want to replace.
/// HIPAR windowType/Type of result display you want to add.
ViStatus _VI_FUNC rsspecan_ReplaceWiGigWindow(ViSession instrSession,
                                              ViString  windowName,
                                              ViInt32   windowType)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViChar*  p2type;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &rsspecan_rngWIGIGLayoutWindowType, windowType, &p2type),
                3, "Window Type");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "LAY:REPL '%s',%s", windowName, p2type);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Forces the IQ measurement results to be recalculated according to the
/// HIFN  current settings.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsspecan_WiGigRecalculateIQMeasurementResults(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsspecan_SetAttributeViString(instrSession, NULL, RSSPECAN_ATTR_WIGIG_RECALCULATE_IQ_MEASUREMENT_RESULTS, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function configures the statistic count.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR state/Specifies whether the specified number of PPDUs is taken into
/// HIPAR state/consideration for the statistical evaluation (maximally the
/// HIPAR state/number of PPDUs detected in the current capture buffer).
/// HIPAR noOfPPDU/Sets the specified number of PPDUs is taken into consideration
/// HIPAR noOfPPDU/for the statistical evaluation
ViStatus _VI_FUNC rsspecan_ConfigureWiGigStatisticCount(ViSession instrSession,
                                                        ViBoolean state,
                                                        ViInt32   noOfPPDU)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_WIGIG_STATISTIC_COUNT_ENABLED, state),
            2, "State");

    if (state)
    {
        viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_WIGIG_STATISTIC_COUNT, noOfPPDU),
                3, "No of PPDU");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function configures the statistic count.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR state/Specifies whether the specified number of PPDUs is taken into
/// HIPAR state/consideration for the statistical evaluation (maximally the
/// HIPAR state/number of PPDUs detected in the current capture buffer).
/// HIPAR PPDUIndex/Specifies the PPDU for PPDU analysis.
ViStatus _VI_FUNC rsspecan_ConfigureWiGigPPDUToAnalyze(ViSession instrSession,
                                                       ViBoolean state,
                                                       ViInt32   PPDUIndex)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_WIGIG_PPDU_TO_ANALYZE, state),
            2, "State");

    if (!state)
    {
        viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_WIGIG_PPDU_INDEX, PPDUIndex),
                3, "PPDU Index");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function configures data symbol evaluation range.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR equalBurstLength/Enables/disables equal burst length to analyze.
/// HIPAR minNumberOfSymbols/Specifies This attribute specifies the minimum number of payload
/// HIPAR minNumberOfSymbols/symbols required for a PPDU to take part in measurement analysis.
/// HIPAR maxNumberOfSymbols/Specifies the maximum number of payload symbols allowed for a
/// HIPAR maxNumberOfSymbols/PPDU to take part in measurement analysis.
ViStatus _VI_FUNC rsspecan_ConfigureWiGigSymbolNumberToAnalzye(
                            ViSession instrSession,
                            ViBoolean equalBurstLength,
                            ViInt32   minNumberOfSymbols,
                            ViInt32   maxNumberOfSymbols)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_WIGIG_EQUAL_BURST_LENGTH_ENABLED, equalBurstLength),
            2, "Equal Burst Length");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_WIGIG_NUMBER_OF_SYMBOLS_MINIMUM, minNumberOfSymbols),
            3, "Min Number Of Symbols");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_WIGIG_NUMBER_OF_SYMBOLS_MAXIMUM, maxNumberOfSymbols),
            4, "Max Number Of Symbols");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function returns all the results.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR results/This control returns all the results.
/// HIPAR returnedValues/Returns the number of values in Results array.
ViStatus _VI_FUNC rsspecan_FetchWiGigBurstAll(ViSession instrSession,
                                              ViReal64  results[],
                                              ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K95"));

    checkErr(RsCore_QueryFloatArrayToUserBuffer(instrSession, "FETC:BURS:ALL?", 32001, results, returnedValues));

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function returns the number of analyzed PPDUs from the current capture buffer.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR coiunt/This control returns the count value.
ViStatus _VI_FUNC rsspecan_FetchWiGigBurstCount(ViSession instrSession,
                                                ViInt32*  count)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_GetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_WIGIG_FETC_BURS_COUNT, count),
            2, "Count");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function returns the number of analyzed PPDUs from the current capture buffer.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR arraySize/This control sets the number of values returned in arrays.
/// HIPAR lengths/This control returns the array of symbol counts, one for each
/// HIPAR lengths/PPDU. Number of values returned in arrays is limited with the
/// HIPAR lengths/parameter Array Size.
ViStatus _VI_FUNC rsspecan_FetchWiGigBurstLengths(ViSession instrSession,
                                                  ViInt32   arraySize,
                                                  ViInt32   lengths[])
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32RangeMin(instrSession, arraySize, 1),
                2, "Array Size");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, lengths), 3, "Lengths");

    checkErr(RsCore_QueryIntegerArrayToUserBuffer(instrSession, "FETC:BURS:LENG?", arraySize, lengths, NULL));

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function returns the start position of each analyzed PPDU in the current capture buffer.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR arraySize/This control sets the number of values returned in arrays.
/// HIPAR starts/This control returns the array of start position of each
/// HIPAR starts/analyzed PPDU in the current capture buffer. Number of values
/// HIPAR starts/returned in arrays is limited with the parameter Array Size.
ViStatus _VI_FUNC rsspecan_FetchWiGigBurstStart(ViSession instrSession,
                                                ViInt32   arraySize,
                                                ViInt32   starts[])
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32RangeMin(instrSession, arraySize, 1),
                2, "Array Size");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, starts), 3, "Starts");

    checkErr(RsCore_QueryIntegerArrayToUserBuffer(instrSession, "FETC:BURS:STAR?", arraySize, starts, NULL));

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function returns the power measurement value, measured during the measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR powerMeasurement/Sets the power measurement. The driver uses this value to select the attribute.
/// HIPAR mode/Sets the measurement mode.
/// HIPAR value/This control returns the selected power value.
ViStatus _VI_FUNC rsspecan_FetchWiGigPower(ViSession instrSession,
                                           ViInt32   powerMeasurement,
                                           ViInt32   mode,
                                           ViReal64* value)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    switch (powerMeasurement)
    {
    case RSSPECAN_VAL_WIGIG_MEAS_CREST:
        {
            switch (mode)
            {
            case RSSPECAN_VAL_MEASTYPE_MIN:

                viCheckParm(rsspecan_GetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_WIGIG_FETCH_CREST_FACTOR_MINIMUM, value),
                        4, "Value");

                break;

            case RSSPECAN_VAL_MEASTYPE_MAX:

                viCheckParm(rsspecan_GetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_WIGIG_FETCH_CREST_FACTOR_MAXIMUM, value),
                        4, "Value");

                break;

            case RSSPECAN_VAL_MEASTYPE_AVER:

                viCheckParm(rsspecan_GetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_WIGIG_FETCH_CREST_FACTOR_AVERAGE, value),
                        4, "Value");

                break;

            default:
                viCheckParm(RsCore_InvalidViInt32Value(instrSession, powerMeasurement), 3, "Mode");
            }
        }
        break;

    case RSSPECAN_VAL_WIGIG_MEAS_TDOM:
        {
            switch (mode)
            {
            case RSSPECAN_VAL_MEASTYPE_MIN:

                viCheckParm(rsspecan_GetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_WIGIG_FETCH_TIME_DOMAIN_POWER_MINIMUM, value),
                        4, "Value");

                break;

            case RSSPECAN_VAL_MEASTYPE_MAX:

                viCheckParm(rsspecan_GetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_WIGIG_FETCH_TIME_DOMAIN_POWER_MAXIMUM, value),
                        4, "Value");

                break;

            case RSSPECAN_VAL_MEASTYPE_AVER:

                viCheckParm(rsspecan_GetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_WIGIG_FETCH_TIME_DOMAIN_POWER_AVERAGE, value),
                        4, "Value");

                break;

            default:
                viCheckParm(RsCore_InvalidViInt32Value(instrSession, powerMeasurement), 3, "Mode");
            }
        }
        break;

    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, powerMeasurement), 2, "Power Measurement");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function returns the Modulation Accuracy Parameters values.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR errorType/Sets the result summary parameter. The driver uses this value to select the attribute.
/// HIPAR mode/Sets the measurement mode.
/// HIPAR value/This control returns the selected result summary parameter value.
ViStatus _VI_FUNC rsspecan_FetchWiGigError(ViSession instrSession,
                                           ViInt32   errorType,
                                           ViInt32   mode,
                                           ViReal64* value)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    switch (errorType)
    {
    case RSSPECAN_VAL_WIGIG_ERR_CFREQ:

        switch (mode)

        {
        case RSSPECAN_VAL_MEASTYPE_MIN:

            viCheckParm(rsspecan_GetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_WIGIG_FETCH_CENTER_FREQUENCY_ERROR_MINIMUM, value),
                    4, "Value");

            break;

        case RSSPECAN_VAL_MEASTYPE_MAX:

            viCheckParm(rsspecan_GetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_WIGIG_FETCH_CENTER_FREQUENCY_ERROR_MAXIMUM, value),
                    4, "Value");

            break;

        case RSSPECAN_VAL_MEASTYPE_AVER:

            viCheckParm(rsspecan_GetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_WIGIG_FETCH_CENTER_FREQUENCY_ERROR_AVERAGE, value),
                    4, "Value");

            break;
        }

        break;

    case RSSPECAN_VAL_WIGIG_ERR_SYMB:

        switch (mode)

        {
        case RSSPECAN_VAL_MEASTYPE_MIN:

            viCheckParm(rsspecan_GetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_WIGIG_FETCH_SYMBOL_CLOCK_ERROR_MINIMUM, value),
                    4, "Value");

            break;

        case RSSPECAN_VAL_MEASTYPE_MAX:

            viCheckParm(rsspecan_GetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_WIGIG_FETCH_SYMBOL_CLOCK_ERROR_MAXIMUM, value),
                    4, "Value");

            break;

        case RSSPECAN_VAL_MEASTYPE_AVER:

            viCheckParm(rsspecan_GetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_WIGIG_FETCH_SYMBOL_CLOCK_ERROR_AVERAGE, value),
                    4, "Value");

            break;
        }

        break;

    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, errorType), 2, "Error Type");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function returns the Modulation Accuracy Parameters values.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR IQImpairment/Sets the modulation accurancy parameter. The driver uses this value to select the attribute.
/// HIPAR mode/Sets the measurement mode.
/// HIPAR value/This control returns the selected power value.
ViStatus _VI_FUNC rsspecan_FetchWiGigIQImp(ViSession instrSession,
                                           ViInt32   IQImpairment,
                                           ViInt32   mode,
                                           ViReal64* value)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    switch (IQImpairment)
    {
    case RSSPECAN_VAL_WIGIG_IQ_OFFS:

        switch (mode)

        {
        case RSSPECAN_VAL_MEASTYPE_MIN:

            viCheckParm(rsspecan_GetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_WIGIG_FETCH_IQ_OFFSET_MINIMUM, value),
                    4, "Value");

            break;

        case RSSPECAN_VAL_MEASTYPE_MAX:

            viCheckParm(rsspecan_GetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_WIGIG_FETCH_IQ_OFFSET_MAXIMUM, value),
                    4, "Value");

            break;

        case RSSPECAN_VAL_MEASTYPE_AVER:

            viCheckParm(rsspecan_GetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_WIGIG_FETCH_IQ_OFFSET_AVERAGE, value),
                    4, "Value");

            break;
        }

        break;

    case RSSPECAN_VAL_WIGIG_IQ_IMBA:

        switch (mode)

        {
        case RSSPECAN_VAL_MEASTYPE_MIN:

            viCheckParm(rsspecan_GetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_WIGIG_FETCH_GAIN_IMBALANCE_MINIMUM, value),
                    4, "Value");

            break;

        case RSSPECAN_VAL_MEASTYPE_MAX:

            viCheckParm(rsspecan_GetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_WIGIG_FETCH_GAIN_IMBALANCE_MAXIMUM, value),
                    4, "Value");

            break;

        case RSSPECAN_VAL_MEASTYPE_AVER:

            viCheckParm(rsspecan_GetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_WIGIG_FETCH_GAIN_IMBALANCE_AVERAGE, value),
                    4, "Value");

            break;
        }

        break;

    case RSSPECAN_VAL_WIGIG_IQ_QUAD:

        switch (mode)

        {
        case RSSPECAN_VAL_MEASTYPE_MIN:

            viCheckParm(rsspecan_GetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_WIGIG_FETCH_QUADRATURE_ERROR_MINIMUM, value),
                    4, "Value");

            break;

        case RSSPECAN_VAL_MEASTYPE_MAX:

            viCheckParm(rsspecan_GetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_WIGIG_FETCH_QUADRATURE_ERROR_MAXIMUM, value),
                    4, "Value");

            break;

        case RSSPECAN_VAL_MEASTYPE_AVER:

            viCheckParm(rsspecan_GetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_WIGIG_FETCH_QUADRATURE_ERROR_AVERAGE, value),
                    4, "Value");

            break;
        }

        break;

    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, IQImpairment), 2, "IQ Impairment");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function returns the EVM measurement value.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR EVM/Sets the EVM measurement. The driver uses this value to select teh attribute.
/// HIPAR mode/Sets the measurement mode.
/// HIPAR value/This control returns the selected power value.
ViStatus _VI_FUNC rsspecan_FetchWiGigEVM(ViSession instrSession,
                                         ViInt32   EVM,
                                         ViInt32   mode,
                                         ViReal64* value)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    switch (EVM)
    {
    case RSSPECAN_VAL_WIGIG_EVM_ALL:

        switch (mode)

        {
        case RSSPECAN_VAL_MEASTYPE_MIN:

            viCheckParm(rsspecan_GetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_WIGIG_FETCH_EVM_ALL_SYMBOLS_MINIMUM, value),
                    4, "Value");

            break;

        case RSSPECAN_VAL_MEASTYPE_MAX:

            viCheckParm(rsspecan_GetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_WIGIG_FETCH_EVM_ALL_SYMBOLS_MAXIMUM, value),
                    4, "Value");

            break;

        case RSSPECAN_VAL_MEASTYPE_AVER:

            viCheckParm(rsspecan_GetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_WIGIG_FETCH_EVM_ALL_SYMBOLS_AVERAGE, value),
                    4, "Value");

            break;
        }

        break;

    case RSSPECAN_VAL_WIGIG_EVM_DATA:

        switch (mode)

        {
        case RSSPECAN_VAL_MEASTYPE_MIN:

            viCheckParm(rsspecan_GetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_WIGIG_FETCH_EVM_DATA_SYMBOLS_MINIMUM, value),
                    4, "Value");

            break;

        case RSSPECAN_VAL_MEASTYPE_MAX:

            viCheckParm(rsspecan_GetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_WIGIG_FETCH_EVM_DATA_SYMBOLS_MAXIMUM, value),
                    4, "Value");

            break;

        case RSSPECAN_VAL_MEASTYPE_AVER:

            viCheckParm(rsspecan_GetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_WIGIG_FETCH_EVM_DATA_SYMBOLS_AVERAGE, value),
                    4, "Value");

            break;
        }

        break;

    case RSSPECAN_VAL_WIGIG_EVM_PILOT:

        switch (mode)

        {
        case RSSPECAN_VAL_MEASTYPE_MIN:

            viCheckParm(rsspecan_GetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_WIGIG_FETCH_EVM_PILOT_SYMBOLS_MINIMUM, value),
                    4, "Value");

            break;

        case RSSPECAN_VAL_MEASTYPE_MAX:

            viCheckParm(rsspecan_GetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_WIGIG_FETCH_EVM_PILOT_SYMBOLS_MAXIMUM, value),
                    4, "Value");

            break;

        case RSSPECAN_VAL_MEASTYPE_AVER:

            viCheckParm(rsspecan_GetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_WIGIG_FETCH_EVM_PILOT_SYMBOLS_AVERAGE, value),
                    4, "Value");

            break;
        }

        break;

    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, EVM), 2, "EVM");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function returns the Time measurement value.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR time/Sets the Time measurement. The driver uses this value to select teh attribute.
/// HIPAR mode/Sets the measurement mode.
/// HIPAR value/This control returns the selected power value.
ViStatus _VI_FUNC rsspecan_FetchWiGigTime(ViSession instrSession,
                                          ViInt32   time,
                                          ViInt32   mode,
                                          ViReal64* value)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    switch (time)
    {
    case RSSPECAN_VAL_WIGIG_TIME_FALL:

        switch (mode)

        {
        case RSSPECAN_VAL_MEASTYPE_MIN:

            viCheckParm(rsspecan_GetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_WIGIG_FETCH_FALL_TIME_MINIMUM, value),
                    4, "Value");

            break;

        case RSSPECAN_VAL_MEASTYPE_MAX:

            viCheckParm(rsspecan_GetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_WIGIG_FETCH_FALL_TIME_MAXIMUM, value),
                    4, "Value");

            break;

        case RSSPECAN_VAL_MEASTYPE_AVER:

            viCheckParm(rsspecan_GetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_WIGIG_FETCH_FALL_TIME_AVERAGE, value),
                    4, "Value");

            break;
        }

        break;

    case RSSPECAN_VAL_WIGIG_TIME_RISE:

        switch (mode)

        {
        case RSSPECAN_VAL_MEASTYPE_MIN:

            viCheckParm(rsspecan_GetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_WIGIG_FETCH_RISE_TIME_MINIMUM, value),
                    4, "Value");

            break;

        case RSSPECAN_VAL_MEASTYPE_MAX:

            viCheckParm(rsspecan_GetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_WIGIG_FETCH_RISE_TIME_MAXIMUM, value),
                    4, "Value");

            break;

        case RSSPECAN_VAL_MEASTYPE_AVER:

            viCheckParm(rsspecan_GetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_WIGIG_FETCH_RISE_TIME_AVERAGE, value),
                    4, "Value");

            break;
        }

        break;

    case RSSPECAN_VAL_WIGIG_TIME_SKEW:

        switch (mode)

        {
        case RSSPECAN_VAL_MEASTYPE_MIN:

            viCheckParm(rsspecan_GetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_WIGIG_FETCH_TIME_SKEW_MINIMUM, value),
                    4, "Value");

            break;

        case RSSPECAN_VAL_MEASTYPE_MAX:

            viCheckParm(rsspecan_GetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_WIGIG_FETCH_TIME_SKEW_MAXIMUM, value),
                    4, "Value");

            break;

        case RSSPECAN_VAL_MEASTYPE_AVER:

            viCheckParm(rsspecan_GetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_WIGIG_FETCH_TIME_SKEW_AVERAGE, value),
                    4, "Value");

            break;
        }

        break;

    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, time), 2, "time");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function returns the average, maximum or minimum Bit Error Rate
/// HIFN  of the PPDU header.
/// HIFN
/// HIFN     Note:
/// HIFN     (1) If no measurement has been performed yet, a query error will
/// HIFN  occur.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSSPECAN_ATTR_WLAN_FETCH_BER_PPDU_HEADER_MIN
/// HIFN     RSSPECAN_ATTR_WLAN_FETCH_BER_PPDU_HEADER_MAX
/// HIFN     RSSPECAN_ATTR_WLAN_FETCH_BER_PPDU_HEADER_AVERAGE
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     FETCh:HBERate:MINimum?
/// HIFN     FETCh:HBERate:MAXimum?
/// HIFN     FETCh:HBERate:AVERage?
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR mode/Sets the measurement mode.
/// HIPAR value/This control returns the selected power value.
ViStatus _VI_FUNC rsspecan_FetchWiGigBERPPDUHeader(ViSession instrSession,
                                                   ViInt32   mode,
                                                   ViReal64* value)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    switch (mode)
    {
    case RSSPECAN_VAL_MEASTYPE_MIN:
        viCheckParm(rsspecan_GetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_WIGIG_FETCH_BER_PPDU_HEADER_MIN, value),
                4, "Value");
        break;
    case RSSPECAN_VAL_MEASTYPE_MAX:
        viCheckParm(rsspecan_GetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_WIGIG_FETCH_BER_PPDU_HEADER_MAX, value),
                4, "Value");
        break;
    case RSSPECAN_VAL_MEASTYPE_AVER:
        viCheckParm(rsspecan_GetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_WIGIG_FETCH_BER_PPDU_HEADER_AVERAGE, value),
                4, "Value");
        break;
    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, mode), 3, "Mode");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function returns the average, maximum or minimum Bit Error Rate
/// HIFN  of the PPDU payload.
/// HIFN
/// HIFN     Note:
/// HIFN     (1) If no measurement has been performed yet, a query error will
/// HIFN  occur.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSSPECAN_ATTR_WLAN_FETCH_BER_PPDU_PAYLOAD_MIN
/// HIFN     RSSPECAN_ATTR_WLAN_FETCH_BER_PPDU_PAYLOAD_MAX
/// HIFN     RSSPECAN_ATTR_WLAN_FETCH_BER_PPDU_PAYLOAD_AVERAGE
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     FETCh:PBERate:MINimum?
/// HIFN     FETCh:PBERate:MAXimum?
/// HIFN     FETCh:PBERate:AVERage?
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR mode/Sets the measurement mode.
/// HIPAR value/This control returns the selected power value.
ViStatus _VI_FUNC rsspecan_FetchWiGigBERPPDUPayload(ViSession instrSession,
                                                    ViInt32   mode,
                                                    ViReal64* value)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    switch (mode)
    {
    case RSSPECAN_VAL_MEASTYPE_MIN:
        viCheckParm(rsspecan_GetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_WIGIG_FETCH_BER_PPDU_PAYLOAD_MIN, value),
                4, "Value");
        break;
    case RSSPECAN_VAL_MEASTYPE_MAX:
        viCheckParm(rsspecan_GetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_WIGIG_FETCH_BER_PPDU_PAYLOAD_MAX, value),
                4, "Value");
        break;
    case RSSPECAN_VAL_MEASTYPE_AVER:
        viCheckParm(rsspecan_GetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_WIGIG_FETCH_BER_PPDU_PAYLOAD_AVERAGE, value),
                4, "Value");
        break;
    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, mode), 3, "Mode");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Returns the average, maximum and minimum signal-to-noise ratio for the
/// HIFN  PPDU in dB.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR minimumSNR/Returns the minimum signal-to-noise ratio for the PPDU in dB.
/// HIPAR averageSNR/This command returns the average signal-to-noise ratio for the PPDU in
/// HIPAR averageSNR/dB.
/// HIPAR maximumSNR/This command returns the maximum signal-to-noise ratio for the PPDU in
/// HIPAR maximumSNR/dB.
ViStatus _VI_FUNC rsspecan_FetchWiGigSignalToRatioStatistics(ViSession instrSession,
                                                             ViReal64* minimumSNR,
                                                             ViReal64* averageSNR,
                                                             ViReal64* maximumSNR)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_WIGIG_FETCH_SIGNAL_TO_RATIO_MINIMUM, minimumSNR),
            2, "Minimum SNR");

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_WIGIG_FETCH_SIGNAL_TO_RATIO_AVERAGE, averageSNR),
            3, "Average SNR");

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_WIGIG_FETCH_SIGNAL_TO_RATIO_MAXIMUM, maximumSNR),
            4, "Maximum SNR");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN This function sets the format of the bitstream
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR format/This parameter specifies the format of the bitstream.
ViStatus _VI_FUNC rsspecan_ConfigureWiGigBitSTreamFormat(ViSession instrSession,
                                                         ViInt32   format)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_WIGIG_BITSTREAM_FORMAT, format),
            2, "Format");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  If enabled, all traces for the currently selected graphical result
/// HIFN  display are included in the export file.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR resultsToASCIIFileEnabled/If enabled, all traces for the currently selected graphical result
/// HIPAR resultsToASCIIFileEnabled/display are included in the export file.
ViStatus _VI_FUNC rsspecan_ConfigureWiGigExportingTraceResultsToASCIIFileEnabled(
                            ViSession instrSession,
                            ViBoolean resultsToASCIIFileEnabled)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, NULL, RSSPECAN_ATTR_WIGIG_EXPORTING_TRACE_RESULTS_TO_ASCII_FILE_ENABLED, resultsToASCIIFileEnabled),
            2, "Results To ASCII File Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function This function defines which items are displayed in the Result
/// HIFN Summary. Note that the results are always calculated, regardless
/// HIFN of their visibility in the Result Summary.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR item/Selects the item to be included in Result Summary.
/// HIPAR displayItem/This parameter selects whether the results summary item is displayed.
ViStatus _VI_FUNC rsspecan_ConfigureWiGigResultSummaryDisplay(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   item,
                            ViBoolean displayItem)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";
    ViChar*  p2string;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentModel(instrSession, "FSW"));

    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &RsSpecAn_rngConfigureWIGIGResultTable, item, &p2string),
                3, "Item");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "DISP:WIND%ld:TABL:ITEM %s,%s", window, p2string, offOnArr[displayItem]);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}
