/*****************************************************************************
 *  Rohde&Schwarz Spectrum Analyzer instrument driver
 *  OFDM VSA option K96
 *
 *  Original Release: February 2023
 *  By: Tereza Zavodna
 *
 *  Should you have any technical questions please contact the hotline of
 *  Rohde & Schwarz Vertriebs-GmbH Rohde & Schwarz Support Center
 *
 *  e-mail: CustomerSupport@rohde-schwarz.com
 *
 *****************************************************************************/

#include "rsspecan.h"
#include "rsspecan_internal.h"

 /// HIFN  Selects the R&S FS-K96 OFDM VSA option to be active.
 /// HIRET Returns the status code of this operation.
 /// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
 /// HIPAR instrSession/rsspecan_InitWithOptions function. The handle identifies a particular
 /// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsspecan_SetOFDMVSAMode(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsspecan_SetAttributeViString(instrSession, NULL, RSSPECAN_ATTR_OFDM_VSA_MODE, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Determines whether the configuration from the currently loaded file is
/// HIFN  used for the measurement. Alternatively, you can configure the OFDM
/// HIFN  signal manually.
/// HIFN
/// HIFN     Note: when you load a configuration file using the
/// HIFN  RSSPECAN_ATTR_OFDM_VSA_LOAD_CONFIGURATION_FILE, the use of the file is
/// HIFN  automatically set to ON.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR configurationFile/Determines whether the configuration from the currently loaded file is
/// HIPAR configurationFile/used for the measurement. Alternatively, you can configure the OFDM
/// HIPAR configurationFile/signal manually.
ViStatus _VI_FUNC rsspecan_UseOFDMVSAConfigurationFile(ViSession instrSession,
                                                       ViBoolean configurationFile)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, NULL, RSSPECAN_ATTR_OFDM_VSA_USE_CONFIGURATION_FILE, configurationFile),
            2, "Configuration File");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Loads an OFDM configuration file and activates its use.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR configurationFile/Loads an OFDM configuration file and activates its use.
ViStatus _VI_FUNC rsspecan_LoadOFDMVSAConfigurationFile(ViSession instrSession,
                                                        ViString  configurationFile)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViString(instrSession, NULL, RSSPECAN_ATTR_OFDM_VSA_LOAD_CONFIGURATION_FILE, configurationFile),
            2, "Configuration File");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Configures the phase configuration.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR enabled/Enables or disables phase compensation during the upconversion of the
/// HIPAR enabled/baseband signal to the radio frequency.
/// HIPAR mode/If phase compensation is enabled (see
/// HIPAR mode/RSSPECAN_ATTR_OFDM_VSA_PHASE_COMPENSATION_ENABLED), this command
/// HIPAR mode/defines the frequency for phase shift.
/// HIPAR frequency/If phase compensation is enabled (see
/// HIPAR frequency/RSSPECAN_ATTR_OFDM_VSA_PHASE_COMPENSATION_ENABLED) and the phase shift
/// HIPAR frequency/frequency is defined manually (see
/// HIPAR frequency/RSSPECAN_ATTR_OFDM_VSA_PHASE_COMPENSATION_MODE), defines the frequency
/// HIPAR frequency/for phase shift.
ViStatus _VI_FUNC rsspecan_ConfigureOFDMVSAPhaseCompensationSettings(
                            ViSession instrSession,
                            ViBoolean enabled,
                            ViInt32   mode,
                            ViReal64  frequency)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, NULL, RSSPECAN_ATTR_OFDM_VSA_PHASE_COMPENSATION_ENABLED, enabled),
            2, "Enabled");

    if (enabled)
    {
        viCheckParm(rsspecan_SetAttributeViInt32(instrSession, NULL, RSSPECAN_ATTR_OFDM_VSA_PHASE_COMPENSATION_MODE, mode),
                3, "Mode");

        if (mode == RSSPECAN_VAL_OFDM_VSA_PHASE_COMPENSATION_MODE_MANUAL)
        {
            viCheckParm(rsspecan_SetAttributeViReal64(instrSession, NULL, RSSPECAN_ATTR_OFDM_VSA_PHASE_COMPENSATION_FREQUENCY, frequency),
                    4, "Frequency");
        }
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Defines the length of a block of repeating samples within a preamble
/// HIFN  symbol.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR preambleBlockLength/Defines the length of a block of repeating samples within a preamble
/// HIPAR preambleBlockLength/symbol.
ViStatus _VI_FUNC rsspecan_ConfigureOFDMVSAPreambleBlockLength(
                            ViSession instrSession,
                            ViInt32   preambleBlockLength)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, NULL, RSSPECAN_ATTR_OFDM_VSA_PREAMBLE_BLOCK_LENGTH, preambleBlockLength),
            2, "Preamble Block Length");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Defines the frame offset, that is the start of the actual OFDM frame
/// HIFN  relative to the start of the first detected preamble block.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR frameStartOffset/Defines the frame offset, that is the start of the actual OFDM frame
/// HIPAR frameStartOffset/relative to the start of the first detected preamble block.
ViStatus _VI_FUNC rsspecan_ConfigureOFDMVSAFrameStartOffset(ViSession instrSession,
                                                            ViInt32   frameStartOffset)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, NULL, RSSPECAN_ATTR_OFDM_VSA_FRAME_START_OFFSET, frameStartOffset),
            2, "Frame Start Offset");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Configures the transform precoding.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR transformPrecoding/Enables or disables transform precoding. See 'DFT-s-OFDM /
/// HIPAR transformPrecoding/SC-FDMA:Transform Precoding'.
/// HIPAR ignore/For RSSPECAN_ATTR_OFDM_VSA_TRANSFORM_PRECODING_ENABLED ON, defines how
/// HIPAR ignore/to process OFDM symbols that contain 'Don't Care' and pilot cells.
ViStatus _VI_FUNC rsspecan_ConfigureOFDMVSATransformPrecodingSettings(
                            ViSession instrSession,
                            ViBoolean transformPrecoding,
                            ViBoolean ignore)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, NULL, RSSPECAN_ATTR_OFDM_VSA_TRANSFORM_PRECODING_ENABLED, transformPrecoding),
            2, "Transform Precoding");

    if (transformPrecoding)
        viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, NULL, RSSPECAN_ATTR_OFDM_VSA_TRANSFORM_PRECODING_IGNORE, ignore),
                3, "Ignore");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Defines the FFT length of an OFDM symbol. Only available if no
/// HIFN  configuration file has been loaded.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR fftSize/Defines the FFT length of an OFDM symbol. Only available if no
/// HIPAR fftSize/configuration file has been loaded.
ViStatus _VI_FUNC rsspecan_ConfigureOFDMVSAFFTSize(ViSession instrSession,
                                                   ViInt32   fftSize)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, NULL, RSSPECAN_ATTR_OFDM_VSA_FFT_SIZE, fftSize),
            2, "FFT Size");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Configures the cyclic prefix mode.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR mode/Selects the type of cyclic prefix.
ViStatus _VI_FUNC rsspecan_ConfigureOFDMVSACyclicPrefixMode(ViSession instrSession,
                                                            ViInt32   mode)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, NULL, RSSPECAN_ATTR_OFDM_VSA_CYCLIC_PREFIX_MODE, mode),
            2, "Mode");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Turns periodic cyclic prefix ranges onand off. Available for
/// HIFN  non-conventional cyclic prefixes.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR periodicRange/Turns periodic cyclic prefix ranges on and off. Available for
/// HIPAR periodicRange/non-conventional cyclic prefixes.
ViStatus _VI_FUNC rsspecan_ConfigureOFDMVSACyclicPrefixPeriodicRange(
                            ViSession instrSession,
                            ViBoolean periodicRange)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, NULL, RSSPECAN_ATTR_OFDM_VSA_CYCLIC_PREFIX_PERIODIC_RANGE_ENABLED, periodicRange),
            2, "Periodic Range");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the number of symbols for which the firstand second
/// HIFN  non-conventional cyclic prefix is used.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR cyclicPrefix/Selects repeated capability CyclicPrefix.
/// HIPAR symbolCount/Defines the number of symbols for which the first and second
/// HIPAR symbolCount/non-conventional cyclic prefix is used.
ViStatus _VI_FUNC rsspecan_ConfigureOFDMVSACyclicPrefixSymbolCount(
                            ViSession instrSession,
                            ViInt32   cyclicPrefix,
                            ViInt32   symbolCount)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  repCapCount = 0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetRepCapIdItemsCount(instrSession, "CyclicPrefix", &repCapCount),
        2, "Cyclic Prefix");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, cyclicPrefix, 1, repCapCount),
            2, "Cyclic Prefix");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_OFDM_VSA_CYCLIC_PREFIX_SYMBOL_COUNT,
            "CyclicPrefix", cyclicPrefix - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Cyclic Prefix");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_OFDM_VSA_CYCLIC_PREFIX_SYMBOL_COUNT, symbolCount),
            3, "Symbol Count");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Defines the cyclic prefix length.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR cyclicPrefix/Selects repeated capability CyclicPrefix.
/// HIPAR length/Defines the cyclic prefix length.
ViStatus _VI_FUNC rsspecan_ConfigureOFDMVSACyclicPrefixLength(
                            ViSession instrSession,
                            ViInt32   cyclicPrefix,
                            ViInt32   length)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  repCapCount = 0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetRepCapIdItemsCount(instrSession, "CyclicPrefix", &repCapCount),
        2, "Cyclic Prefix");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, cyclicPrefix, 1, repCapCount),
            2, "Cyclic Prefix");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_OFDM_VSA_CYCLIC_PREFIX_LENGTH,
            "CyclicPrefix", cyclicPrefix - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Cyclic Prefix");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_OFDM_VSA_CYCLIC_PREFIX_LENGTH, length),
            3, "Length");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Configures the lowpass filter settings.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR filter/Turns an adjustable (lowpass) channel filter in the signal path on and
/// HIPAR filter/off.
ViStatus _VI_FUNC rsspecan_ConfigureOFDMVSALowpassFilterEnabled(
                            ViSession instrSession,
                            ViBoolean filter)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, NULL, RSSPECAN_ATTR_OFDM_VSA_LOWPASS_FILTER_ENABLED, filter),
            2, "Filter");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the lowpass filter settings.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR bandwidth6dB/Configures the 6-dB bandwidth of the channel filter. The filter
/// HIPAR bandwidth6dB/bandwidth cannot be higher than the current 50-dB frequency (see
/// HIPAR bandwidth6dB/RSSPECAN_ATTR_OFDM_VSA_LOWPASS_FILTER_50_DB_FREQUENCY).
/// HIPAR bandwidth50dB/Configures the 50-dB frequency of the channel filter. The 50-dB
/// HIPAR bandwidth50dB/frequency is the distance from the center of the filter to the point at
/// HIPAR bandwidth50dB/which the filter reaches an attenuation of 50dB. This frequency must
/// HIPAR bandwidth50dB/always be larger than the 6-dB passband (see
/// HIPAR bandwidth50dB/RSSPECAN_ATTR_OFDM_VSA_LOWPASS_FILTER_6_DB_FREQUENCY).
ViStatus _VI_FUNC rsspecan_ConfigureOFDMVSALowpassFilterSettings(
                            ViSession instrSession,
                            ViReal64  bandwidth6dB,
                            ViReal64  bandwidth50dB)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, NULL, RSSPECAN_ATTR_OFDM_VSA_LOWPASS_FILTER_6_DB_FREQUENCY, bandwidth6dB),
            2, "Bandwidth 6 dB");

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, NULL, RSSPECAN_ATTR_OFDM_VSA_LOWPASS_FILTER_50_DB_FREQUENCY, bandwidth50dB),
            3, "Bandwidth 50 dB");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Configures the internal highpass filter settings.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR highpassFilter/Activates an additional internal highpass filter.
ViStatus _VI_FUNC rsspecan_ConfigureOFDMVSAHighpassFilterEnabled(
                            ViSession instrSession,
                            ViBoolean highpassFilter)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, NULL, RSSPECAN_ATTR_OFDM_VSA_HIGHPASS_FILTER_ENABLED, highpassFilter),
            2, "Highpass Filter");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Configures the internal highpass filter settings.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR bandwidth6dB/Configures the bandwidth of the high pass filter at which an
/// HIPAR bandwidth6dB/attenuation of 6dB is reached. The filter bandwidth cannot be higher
/// HIPAR bandwidth6dB/than the current sample rate. If necessary, the filter bandwidth is
/// HIPAR bandwidth6dB/adapted to the current sample rate.
ViStatus _VI_FUNC rsspecan_ConfigureOFDMVSAHighpassFilterSettings(
                            ViSession instrSession,
                            ViReal64  bandwidth6dB)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, NULL, RSSPECAN_ATTR_OFDM_VSA_HIGHPASS_FILTER_6_DB_FREQUENCY, bandwidth6dB),
            2, "Bandwidth 6 dB");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the 50-dB frequency of the channel filter.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR bandwidth50dB/Queries the 50-dB frequency of the channel filter.
ViStatus _VI_FUNC rsspecan_QueryOFDMVSAHighpassFilter50dBFrequency(
                            ViSession instrSession,
                            ViReal64* bandwidth50dB)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, NULL, RSSPECAN_ATTR_OFDM_VSA_HIGHPASS_FILTER_50_DB_FREQUENCY, bandwidth50dB),
            2, "Bandwidth 50 dB");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Turns a search for bursted OFDM signals on and off.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR burstSearch/Turns a search for bursted OFDM signals on and off.
ViStatus _VI_FUNC rsspecan_ConfigureOFDMVSABurstSearchEnabled(
                            ViSession instrSession,
                            ViBoolean burstSearch)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, NULL, RSSPECAN_ATTR_OFDM_VSA_BURST_SEARCH_ENABLED, burstSearch),
            2, "Burst Search");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Configures the result range.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR maximumFrames/Defines the maximum number of frames to be demodulated.
/// HIPAR resultLength/Defines the number of symbols in a frame. Note that frames with fewer
/// HIPAR resultLength/symbols are not analyzed.
ViStatus _VI_FUNC rsspecan_ConfigureOFDMVSAMaximumFrames(ViSession instrSession,
                                                         ViInt32   maximumFrames,
                                                         ViInt32   resultLength)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, NULL, RSSPECAN_ATTR_OFDM_VSA_MAXIMUM_FRAMES, maximumFrames),
            2, "Maximum Frames");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, NULL, RSSPECAN_ATTR_OFDM_VSA_RESULT_LENGTH_SYMBOLS, resultLength),
            3, "Result Length");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Selects the time synchronization mode.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR timeSynchronization/Selects the time synchronization mode.
ViStatus _VI_FUNC rsspecan_ConfigureOFDMVSASynchronizationTimeMode(
                            ViSession instrSession,
                            ViInt32   timeSynchronization)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, NULL, RSSPECAN_ATTR_OFDM_VSA_SYNCHRONIZATION_TIME_MODE, timeSynchronization),
            2, "Time Synchronization");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Selects the parameter estimation mode.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR parameterEstimation/Selects the parameter estimation mode.
ViStatus _VI_FUNC rsspecan_ConfigureOFDMVSASynchronizationParameterEstimation(
                            ViSession instrSession,
                            ViInt32   parameterEstimation)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, NULL, RSSPECAN_ATTR_OFDM_VSA_SYNCHRONIZATION_PARAMETER_ESTIMATION, parameterEstimation),
            2, "Parameter Estimation");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Selects the auto demodulation mode.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR modulationDetection/Selects the auto demodulation mode.
ViStatus _VI_FUNC rsspecan_ConfigureOFDMVSASynchronizationModulationDetection(
                            ViSession instrSession,
                            ViInt32   modulationDetection)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, NULL, RSSPECAN_ATTR_OFDM_VSA_SYNCHRONIZATION_MODULATION_DETECTION, modulationDetection),
            2, "Modulation Detection");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Sets and queries the reliability threshold for frame and time
/// HIFN  synchronisation.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR minimumFrameSync/Sets and queries the reliability threshold for frame synchronisation.
/// HIPAR minimumTimeSync/Sets and queries the reliability threshold for time synchronisation.
ViStatus _VI_FUNC rsspecan_ConfigureOFDMVSAThresholds(ViSession instrSession,
                                                      ViReal64  minimumFrameSync,
                                                      ViReal64  minimumTimeSync)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, NULL, RSSPECAN_ATTR_OFDM_VSA_THRESHOLDS_MINIMUM_FRAME, minimumFrameSync),
            2, "Minimum Frame Sync");

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, NULL, RSSPECAN_ATTR_OFDM_VSA_THRESHOLDS_MINIMUM_TIME, minimumTimeSync),
            3, "Minimum Time Sync");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Defines an offset for the FFT start sample in the guard interval.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR fftShift/Defines an offset for the FFT start sample in the guard interval.
ViStatus _VI_FUNC rsspecan_ConfigureOFDMVSADemodulationFFTShift(
                            ViSession instrSession,
                            ViReal64  fftShift)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, NULL, RSSPECAN_ATTR_OFDM_VSA_DEMODULATION_FFT_SHIFT, fftShift),
            2, "FFT Shift");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Defines the maximum allowed carrier offset for frame synchronization.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR maximumCarrierOffset/Defines the maximum allowed carrier offset for frame synchronization.
ViStatus _VI_FUNC rsspecan_ConfigureOFDMVSADemodulationMaximumCarrierOffset(
                            ViSession instrSession,
                            ViInt32   maximumCarrierOffset)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, NULL, RSSPECAN_ATTR_OFDM_VSA_DEMODULATION_MAXIMUM_CARRIER_OFFSET, maximumCarrierOffset),
            2, "Maximum Carrier Offset");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Defines the cyclic delay.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR cyclicDelay/Defines the cyclic delay.
ViStatus _VI_FUNC rsspecan_ConfigureOFDMVSADemodulationCyclicDelay(
                            ViSession instrSession,
                            ViInt32   cyclicDelay)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, NULL, RSSPECAN_ATTR_OFDM_VSA_DEMODULATION_CYCLIC_DELAY, cyclicDelay),
            2, "Cyclic Delay");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Turns phase tracking on and off.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR phaseTracking/Turns phase tracking on and off.
ViStatus _VI_FUNC rsspecan_ConfigureOFDMVSATrackingPhaseEnabled(
                            ViSession instrSession,
                            ViBoolean phaseTracking)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, NULL, RSSPECAN_ATTR_OFDM_VSA_TRACKING_PHASE_ENABLED, phaseTracking),
            2, "Phase Tracking");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Turns tracking of the sample clock deviation on and off.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR timingTracking/Turns tracking of the sample clock deviation on and off.
ViStatus _VI_FUNC rsspecan_ConfigureOFDMVSATrackingTimingEnabled(
                            ViSession instrSession,
                            ViBoolean timingTracking)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, NULL, RSSPECAN_ATTR_OFDM_VSA_TRACKING_TIMING_ENABLED, timingTracking),
            2, "Timing Tracking");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Turns tracking of the power level on and off.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR levelTracking/Turns tracking of the power level on and off.
ViStatus _VI_FUNC rsspecan_ConfigureOFDMVSATrackingLevelEnabled(
                            ViSession instrSession,
                            ViBoolean levelTracking)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, NULL, RSSPECAN_ATTR_OFDM_VSA_TRACKING_LEVEL_ENABLED, levelTracking),
            2, "Level Tracking");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Turns compensation for the estimated channel transfer function on and
/// HIFN  off.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channelCompensation/Turns compensation for the estimated channel transfer function on and
/// HIPAR channelCompensation/off.
ViStatus _VI_FUNC rsspecan_ConfigureOFDMVSATrackingChannelCompensationEnabled(
                            ViSession instrSession,
                            ViBoolean channelCompensation)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, NULL, RSSPECAN_ATTR_OFDM_VSA_TRACKING_CHANNEL_COMPENSATION_ENABLED, channelCompensation),
            2, "Channel Compensation");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  The constellation diagram includes symbols for all or only for the
/// HIFN  specified carrier number.
/// HIFN
/// HIFN     The range of valid carrier numbers is:
/// HIFN
/// HIFN     [- 'FFT Size'/2, +'FFT Size'/2]
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR carrier/The constellation diagram includes symbols for all or only for the
/// HIPAR carrier/specified carrier number.
ViStatus _VI_FUNC rsspecan_IncludeOFDMVSAConstellationDiagramCarrier(
                            ViSession instrSession,
                            ViInt32   carrier)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, NULL, RSSPECAN_ATTR_OFDM_VSA_CONSTELLATION_DIAGRAM_SPECIFIED_CARRIER, carrier),
            2, "Carrier");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  The constellation diagram includes symbols for all carriers.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsspecan_IncludeOFDMVSAConstellationDiagramCarriersAll(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsspecan_SetAttributeViString(instrSession, NULL, RSSPECAN_ATTR_OFDM_VSA_CONSTELLATION_DIAGRAM_CARRIERS_ALL, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  The constellation diagram includes only symbols for the selected
/// HIFN  modulation.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR selectedModulation/The constellation diagram includes only symbols for the selected
/// HIPAR selectedModulation/modulation.
ViStatus _VI_FUNC rsspecan_IncludeOFDMVSAConstellationDiagramSelectedModulation(
                            ViSession instrSession,
                            ViString  selectedModulation)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViString(instrSession, NULL, RSSPECAN_ATTR_OFDM_VSA_CONSTELLATION_DIAGRAM_SELECTED_MODULATION, selectedModulation),
            2, "Selected Modulation");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  The constellation diagram includes only symbols for all modulations.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsspecan_IncludeOFDMVSAConstellationDiagramModulationAll(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsspecan_SetAttributeViString(instrSession, NULL, RSSPECAN_ATTR_OFDM_VSA_CONSTELLATION_DIAGRAM_MODULATION_ALL, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  The constellation diagram includes only cells for the selected
/// HIFN  modulation type.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR modulationType/The constellation diagram includes only cells for the selected
/// HIPAR modulationType/modulation type.
ViStatus _VI_FUNC rsspecan_ConfigureOFDMVSAConstellationDiagramModulationType(
                            ViSession instrSession,
                            ViInt32   modulationType)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, NULL, RSSPECAN_ATTR_OFDM_VSA_CONSTELLATION_DIAGRAM_MODULATION_TYPE, modulationType),
            2, "Modulation Type");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  The constellation diagram includes only the specified symbol number.
/// HIFN  The first symbol is 0.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR symbolNumber/The constellation diagram includes only the specified symbol number.
/// HIPAR symbolNumber/The first symbol is 0.
ViStatus _VI_FUNC rsspecan_IncludeOFDMVSAConstellationDiagramSpecifiedSymbolNumber(
                            ViSession instrSession,
                            ViInt32   symbolNumber)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, NULL, RSSPECAN_ATTR_OFDM_VSA_CONSTELLATION_DIAGRAM_SPECIFIED_SYMBOL_NUMBER, symbolNumber),
            2, "Symbol Number");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  The constellation diagram includes all symbol numbers.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsspecan_IncludeOFDMVSAConstellationDiagramSymbolNumberAll(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsspecan_SetAttributeViString(instrSession, NULL, RSSPECAN_ATTR_OFDM_VSA_CONSTELLATION_DIAGRAM_SYMBOL_NUMBER_ALL, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Selects the averaging method for the mean EVM over multiple frames.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR averagingMethod/Selects the averaging method for the mean EVM over multiple frames.
ViStatus _VI_FUNC rsspecan_ConfigureOFDMVSAEVMAveragingMethod(
                            ViSession instrSession,
                            ViInt32   averagingMethod)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, NULL, RSSPECAN_ATTR_OFDM_VSA_EVM_AVERAGING_METHOD, averagingMethod),
            2, "Averaging Method");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Selects the normalization method for EVM results.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR normalizationMethod/Selects the normalization method for EVM results.
ViStatus _VI_FUNC rsspecan_ConfigureOFDMVSAEVMNormalizationMethod(
                            ViSession instrSession,
                            ViInt32   normalizationMethod)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, NULL, RSSPECAN_ATTR_OFDM_VSA_EVM_NORMALIZATION_METHOD, normalizationMethod),
            2, "Normalization Method");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Selects the units for OFDM VSA mode.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR evm/Selects the unit for EVM results.
/// HIPAR symbolAxes/Selects the unit for result displays that show results on symbol
/// HIPAR symbolAxes/level, for example the EVM vs Symbol.
/// HIPAR carrierAxes/Selects the unit for result displays that show results on carrier
/// HIPAR carrierAxes/level, for example the EVM vs Carrier.
/// HIPAR magnitudeCapture/Selects  the unit for result displays that show results over time, for
/// HIPAR magnitudeCapture/example the Magnitude Capture display.
/// HIPAR powerSpectrum/Selects the unit for result displays that show results over the
/// HIPAR powerSpectrum/frequency, for example the Power Spectrum.
/// HIPAR impulseResponse/Selects the unit for impulse response results.
ViStatus _VI_FUNC rsspecan_ConfigureOFDMVSAUnits(ViSession instrSession,
                                                 ViInt32   evm,
                                                 ViInt32   symbolAxes,
                                                 ViInt32   carrierAxes,
                                                 ViInt32   magnitudeCapture,
                                                 ViInt32   powerSpectrum,
                                                 ViInt32   impulseResponse)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, NULL, RSSPECAN_ATTR_OFDM_VSA_UNITS_EVM, evm),
            2, "EVM");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, NULL, RSSPECAN_ATTR_OFDM_VSA_UNITS_SYMBOL, symbolAxes),
            3, "Symbol Axes");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, NULL, RSSPECAN_ATTR_OFDM_VSA_UNITS_CARRIER, carrierAxes),
            4, "Carrier Axes");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, NULL, RSSPECAN_ATTR_OFDM_VSA_UNITS_MAGNITUDE_CAPTURE, magnitudeCapture),
            5, "Magnitude Capture");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, NULL, RSSPECAN_ATTR_OFDM_VSA_UNITS_POWER_SPECTRUM, powerSpectrum),
            6, "Power Spectrum");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, NULL, RSSPECAN_ATTR_OFDM_VSA_UNITS_IMPULSE_RESPONSE, impulseResponse),
            7, "Impulse Response");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Adds a window to the display.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR windowName/String containing the name of the existing window the new window is
/// HIPAR windowName/inserted next to. By default, the name of a window is the same as its
/// HIPAR windowName/index.
/// HIPAR direction/Direction the new window is added relative to the existing window.
/// HIPAR windowType/Type of result display (evaluation method) you want to add.
/// HIPAR returnedWindowName/When adding a new window, the command returns its name (by default the
/// HIPAR returnedWindowName/same as its number) as a result.
/// HIPAR bufferSize/This control sets the allocated length of Returned Window Name string.
ViStatus _VI_FUNC rsspecan_OFDMVSAAddWindow(ViSession instrSession,
                                            ViInt32   bufferSize,
                                            ViString  windowName,
                                            ViInt32   direction,
                                            ViInt32   windowType,
                                            ViChar    returnedWindowName[],
                                            ViInt32*  bytesRead)
{
    ViStatus error = VI_SUCCESS;
    ViChar   param[RS_PARAM_BUF_SIZE] = "";
    ViChar*  directionString;
    ViChar*  windowTypeString;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &rsspecan_rngLayoutDirection, direction, &directionString),
        2, "Direction");
    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &rsspecan_rngOFDMVSALayoutType, windowType, &windowTypeString),
        3, "Window Type");

    (void)snprintf(param, RS_PARAM_BUF_SIZE, "\"%s\",%s,%s", windowName, directionString, windowTypeString);

    checkErr(RsCore_GetAttributeViStringWithQparam(instrSession, NULL, RSSPECAN_ATTR_HIDDEN_ADD_WINDOW, param, bufferSize, returnedWindowName, bytesRead));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Replaces the window type (for example from 'Diagram' to 'Result
/// HIFN  Summary') of an already existing window while keeping its position,
/// HIFN  index and window name.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR windowName/String containing the name of the existing window.
/// HIPAR windowType/Type of result display you want to use in the existing window.
ViStatus _VI_FUNC rsspecan_OFDMVSAReplaceWindow(ViSession instrSession,
                                                ViString  windowName,
                                                ViInt32   windowType)
{
    ViStatus error = VI_SUCCESS;
    ViChar   param[RS_PARAM_BUF_SIZE];
    ViChar*  windowTypeString;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &rsspecan_rngOFDMVSALayoutType, windowType, &windowTypeString),
        3, "Window Type");

    (void)snprintf(param, RS_PARAM_BUF_SIZE, "\"%s\",%s", windowName, windowTypeString);

    checkErr(RsCore_SetAttributeViRawString(instrSession, NULL, RSSPECAN_ATTR_HIDDEN_REPLACE_WINDOW, RS_VAL_DIRECT_USER_CALL, param));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Defines the format of the symbols for the Bitstream display.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR format/Defines the format of the symbols for the Bitstream display.
ViStatus _VI_FUNC rsspecan_ConfigureOFDMVSABitstreamDisplayFormat(
                            ViSession instrSession,
                            ViInt32   format)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, NULL, RSSPECAN_ATTR_CONFIGURE_OFDM_VSA_BITSTREAM_DISPLAY_FORMAT, format),
            2, "Format");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Returns the state of the individual stages of the signal flow. The
/// HIFN  result is a comma-separated list of states, one for each stage. The
/// HIFN  stages are in the following order:
/// HIFN
/// HIFN     Burst Detection,
/// HIFN     Time Sync,
/// HIFN     Frame Sync,
/// HIFN     Data-Aided Parameter estimation,
/// HIFN     Modulation detection,
/// HIFN     Pilot-aided parameter estimation,
/// HIFN     CompensateEVM meas
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR bufferSize/Size of output array.
/// HIPAR allStates/Returns the state of the individual stages of the signal flow. The
/// HIPAR allStates/result is a comma-separated list of states, one for each stage. The
/// HIPAR allStates/stages are in the following order:
/// HIPAR bytesRead/Returns the number of bytes read from instrument.
ViStatus _VI_FUNC rsspecan_FetchOFDMVSAAllStates(ViSession instrSession,
                                                 ViInt32   bufferSize,
                                                 ViChar    allStates[],
                                                 ViInt32*  bytesRead)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeViStringWithRealLen(instrSession, NULL, RSSPECAN_ATTR_FETCH_OFDM_VSA_ALL_STATES, RS_VAL_DIRECT_USER_CALL, bufferSize, allStates, bytesRead));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Returns the state of the burst detection stage of the signal flow.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR burstDetection/Returns the state of the burst detection stage of the signal flow.
ViStatus _VI_FUNC rsspecan_FetchOFDMVSABurstDetectionState(ViSession instrSession,
                                                           ViInt32*  burstDetection)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_GetAttributeViInt32(instrSession, NULL, RSSPECAN_ATTR_FETCH_OFDM_VSA_BURST_DETECTION_STATE, burstDetection),
            2, "Burst Detection");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Returns the state of the compensation stage of the signal flow.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR compensation/Returns the state of the compensation stage of the signal flow.
ViStatus _VI_FUNC rsspecan_FetchOFDMVSACompensationState(ViSession instrSession,
                                                         ViInt32*  compensation)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_GetAttributeViInt32(instrSession, NULL, RSSPECAN_ATTR_FETCH_OFDM_VSA_COMPENSATION_STATE, compensation),
            2, "Compensation");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Returns the state of the data-aided parameter estimation stage of the
/// HIFN  signal flow.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR dataAidedEstimation/Returns the state of the data-aided parameter estimation stage of the
/// HIPAR dataAidedEstimation/signal flow.
ViStatus _VI_FUNC rsspecan_FetchOFDMVSADataAidedEstimationState(
                            ViSession instrSession,
                            ViInt32*  dataAidedEstimation)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_GetAttributeViInt32(instrSession, NULL, RSSPECAN_ATTR_FETCH_OFDM_VSA_DATA_AIDED_ESTIMATION_STATE, dataAidedEstimation),
            2, "Data Aided Estimation");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Returns the state of the pilot-aided parameter estimation stage of the
/// HIFN  signal flow.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR pilotAidedEstimation/Returns the state of the pilot-aided parameter estimation stage of the
/// HIPAR pilotAidedEstimation/signal flow.
ViStatus _VI_FUNC rsspecan_FetchOFDMVSAPilotAidedEstimationState(
                            ViSession instrSession,
                            ViInt32*  pilotAidedEstimation)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_GetAttributeViInt32(instrSession, NULL, RSSPECAN_ATTR_FETCH_OFDM_VSA_PILOT_AIDED_ESTIMATION_STATE, pilotAidedEstimation),
            2, "Pilot Aided Estimation");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Returns the state of the EVM measurement stage of the signal flow.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR evmMeasurement/Returns the state of the EVM measurement stage of the signal flow.
ViStatus _VI_FUNC rsspecan_FetchOFDMVSAEVMMeasurementState(ViSession instrSession,
                                                           ViInt32*  evmMeasurement)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_GetAttributeViInt32(instrSession, NULL, RSSPECAN_ATTR_FETCH_OFDM_VSA_EVM_MEASUREMENT_STATE, evmMeasurement),
            2, "EVM Measurement");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Returns the state of the modulation detection stage of the signal
/// HIFN  flow.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR modulationDetection/Returns the state of the modulation detection stage of the signal
/// HIPAR modulationDetection/flow.
ViStatus _VI_FUNC rsspecan_FetchOFDMVSAModulationDetectionState(
                            ViSession instrSession,
                            ViInt32*  modulationDetection)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_GetAttributeViInt32(instrSession, NULL, RSSPECAN_ATTR_FETCH_OFDM_VSA_MODULATION_DETECTION_STATE, modulationDetection),
            2, "Modulation Detection");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Returns the frame synchronization stage of the signal flow and its
/// HIFN  result.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR state/Returns the state of the frame synchronization stage of the signal
/// HIPAR state/flow.
/// HIPAR frameSynchronization/Returns the Frame Synchronisation value.
ViStatus _VI_FUNC rsspecan_FetchOFDMVSAFrameSynchronizationResults(
                            ViSession instrSession,
                            ViInt32*  state,
                            ViReal64* frameSynchronization)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_GetAttributeViInt32(instrSession, NULL, RSSPECAN_ATTR_FETCH_OFDM_VSA_FRAME_SYNCHRONIZATION_STATE, state),
            2, "State");

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, NULL, RSSPECAN_ATTR_FETCH_OFDM_VSA_FRAME_SYNCHRONIZATION, frameSynchronization),
            3, "Frame Synchronization");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Returns the time synchronization stage of the signal flow and its
/// HIFN  result.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR state/Returns the state of the time synchronization stage of the signal
/// HIPAR state/flow.
/// HIPAR timeSynchronization/Returns the Time Synchronisation value.
ViStatus _VI_FUNC rsspecan_FetchOFDMVSATimeSynchronizationResults(
                            ViSession instrSession,
                            ViInt32*  state,
                            ViReal64* timeSynchronization)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_GetAttributeViInt32(instrSession, NULL, RSSPECAN_ATTR_FETCH_OFDM_VSA_TIME_SYNCHRONIZATION_STATE, state),
            2, "State");

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, NULL, RSSPECAN_ATTR_FETCH_OFDM_VSA_TIME_SYNCHRONIZATION, timeSynchronization),
            3, "Time Synchronization");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Retrieves the time offset between the trigger event and the start of
/// HIFN  the first OFDM frame.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR triggerToSync/Retrieves the time offset between the trigger event and the start of
/// HIPAR triggerToSync/the first OFDM frame.
ViStatus _VI_FUNC rsspecan_FetchOFDMVSATriggerToSync(ViSession instrSession,
                                                     ViReal64* triggerToSync)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, NULL, RSSPECAN_ATTR_FETCH_OFDM_VSA_TRIGGER_TO_SYNC, triggerToSync),
            2, "Trigger To Sync");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Returns the number of analyzed PPDUs from the current capture buffer.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR burstCount/Returns the number of analyzed PPDUs from the current capture buffer.
ViStatus _VI_FUNC rsspecan_FetchOFDMVSABurstCount(ViSession instrSession,
                                                  ViInt32*  burstCount)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_GetAttributeViInt32(instrSession, NULL, RSSPECAN_ATTR_FETCH_OFDM_VSA_BURST_COUNT, burstCount),
            2, "Burst Count");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Returns the length of the analyzed bursts from the current
/// HIFN  measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR burstNumber/Size of output array.
/// HIPAR burstLengths/Returns the length of the analyzed bursts from the current
/// HIPAR burstLengths/measurement.
/// HIPAR valuesRead/Returns the number of results read from instrument.
ViStatus _VI_FUNC rsspecan_FetchOFDMVSABurstLengths(ViSession instrSession,
                                                    ViInt32   burstNumber,
                                                    ViInt32   burstLengths[],
                                                    ViInt32*  valuesRead)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeViStringAsViInt32ArrayToUserBuffer(instrSession, NULL, RSSPECAN_ATTR_FETCH_OFDM_VSA_BURST_LENGTH, NULL, burstNumber, burstLengths, valuesRead));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Returns the start position of each analyzed burst in the current
/// HIFN  capture buffer.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR burstOffsets/Returns the start position of each analyzed burst in the current
/// HIPAR burstOffsets/capture buffer.
/// HIPAR burstNumber/Size of output array.
/// HIPAR valuesRead/Returns the number of results read from instrument.
ViStatus _VI_FUNC rsspecan_FetchOFDMVSABurstOffset(ViSession instrSession,
                                                   ViInt32   burstNumber,
                                                   ViReal64  burstOffsets[],
                                                   ViInt32*  valuesRead)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeViStringAsViReal64ArrayToUserBuffer(instrSession, NULL, RSSPECAN_ATTR_FETCH_OFDM_VSA_BURST_OFFSET, NULL, burstNumber, burstOffsets, valuesRead));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Returns all values in the result summary, in the same order as in the
/// HIFN  display
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR bufferSize/Size of output array.
/// HIPAR results/Returns all values in the result summary, in the same order as in the
/// HIPAR results/display
/// HIPAR bytesRead/Returns the number of bytes read from instrument.
ViStatus _VI_FUNC rsspecan_FetchOFDMVSAAllResults(ViSession instrSession,
                                                  ViInt32   bufferSize,
                                                  ViChar    results[],
                                                  ViInt32*  bytesRead)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeViStringWithRealLen(instrSession, NULL, RSSPECAN_ATTR_FETCH_OFDM_VSA_ALL_RESULTS, RS_VAL_DIRECT_USER_CALL, bufferSize, results, bytesRead));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Returns the crest factor results.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR maximum/Returns the maximum crest factor.
/// HIPAR minimum/Returns the minimum crest factor.
/// HIPAR average/Returns the average Crest Factor.
ViStatus _VI_FUNC rsspecan_FetchOFDMVSACrestFactorResults(ViSession instrSession,
                                                          ViReal64* maximum,
                                                          ViReal64* minimum,
                                                          ViReal64* average)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, NULL, RSSPECAN_ATTR_FETCH_OFDM_VSA_CREST_FACTOR_MAXIMUM, maximum),
            2, "Maximum");

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, NULL, RSSPECAN_ATTR_FETCH_OFDM_VSA_CREST_FACTOR_MINIMUM, minimum),
            3, "Minimum");

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, NULL, RSSPECAN_ATTR_FETCH_OFDM_VSA_CREST_FACTOR_AVERAGE, average),
            4, "Average");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Returns the EVM Data results.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR maximum/Returns the maximum EVM Data.
/// HIPAR minimum/Returns the minimum EVM Data.
/// HIPAR average/Returns the average EVM Data.
ViStatus _VI_FUNC rsspecan_FetchOFDMVSAEVMDataResult(ViSession instrSession,
                                                     ViReal64* maximum,
                                                     ViReal64* minimum,
                                                     ViReal64* average)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, NULL, RSSPECAN_ATTR_FETCH_OFDM_VSA_EVM_DATA_MAXIMUM, maximum),
            2, "Maximum");

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, NULL, RSSPECAN_ATTR_FETCH_OFDM_VSA_EVM_DATA_MINIMUM, minimum),
            3, "Minimum");

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, NULL, RSSPECAN_ATTR_FETCH_OFDM_VSA_EVM_DATA_AVERAGE, average),
            4, "Average");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Returns the EVM Pilot results.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR maximum/Returns the maximum EVM Pilot.
/// HIPAR minimum/Returns the minimum EVM Pilot.
/// HIPAR average/Returns the average EVM Pilot.
ViStatus _VI_FUNC rsspecan_FetchOFDMVSAEVMPilotResults(ViSession instrSession,
                                                       ViReal64* maximum,
                                                       ViReal64* minimum,
                                                       ViReal64* average)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, NULL, RSSPECAN_ATTR_FETCH_OFDM_VSA_EVM_PILOT_MAXIMUM, maximum),
            2, "Maximum");

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, NULL, RSSPECAN_ATTR_FETCH_OFDM_VSA_EVM_PILOT_MINIMUM, minimum),
            3, "Minimum");

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, NULL, RSSPECAN_ATTR_FETCH_OFDM_VSA_EVM_PILOT_AVERAGE, average),
            4, "Average");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Returns the average EVM Pilot in percents.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR percentAverage/Returns the average EVM Pilot in percents.
ViStatus _VI_FUNC rsspecan_FetchOFDMVSAEVMPilotPercentAverage(
                            ViSession instrSession,
                            ViReal64* percentAverage)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, NULL, RSSPECAN_ATTR_FETCH_OFDM_VSA_EVM_PILOT_PERCENT_AVERAGE, percentAverage),
            2, "Percent Average");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Returns the EVM All results in percents.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR maximum/Returns the maximum EVM All in percents.
/// HIPAR minimum/Returns the minimum EVM All in percents.
/// HIPAR average/Returns the average EVM All in percent.
ViStatus _VI_FUNC rsspecan_FetchOFDMVSAEVMAllPercentResults(ViSession instrSession,
                                                            ViReal64* maximum,
                                                            ViReal64* minimum,
                                                            ViReal64* average)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, NULL, RSSPECAN_ATTR_FETCH_OFDM_VSA_EVM_ALL_PERCENT_MAXIMUM, maximum),
            2, "Maximum");

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, NULL, RSSPECAN_ATTR_FETCH_OFDM_VSA_EVM_ALL_PERCENT_MINIMUM, minimum),
            3, "Minimum");

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, NULL, RSSPECAN_ATTR_FETCH_OFDM_VSA_EVM_ALL_PERCENT_AVERAGE, average),
            4, "Average");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Returns the maximum EVM Peak Data results.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR maximum/Returns the maximum EVM Peak Data.
/// HIPAR percentMaximum/Returns the maximum EVM Peak Data in percents.
ViStatus _VI_FUNC rsspecan_FetchOFDMVSAEVMPeakDataMaximum(ViSession instrSession,
                                                          ViReal64* maximum,
                                                          ViReal64* percentMaximum)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, NULL, RSSPECAN_ATTR_FETCH_OFDM_VSA_EVM_PEAK_DATA_MAXIMUM, maximum),
            2, "Maximum");

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, NULL, RSSPECAN_ATTR_FETCH_OFDM_VSA_EVM_PEAK_DATA_PERCENT_MAXIMUM, percentMaximum),
            3, "Percent Maximum");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Returns the maximum EVM Peak Pilot results.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR maximum/Returns the maximum EVM Peak Pilot.
/// HIPAR percentMaximum/Returns the maximum EVM Peak Pilot in percents.
ViStatus _VI_FUNC rsspecan_FetchOFDMVSAEVMPeakPilotMaximum(ViSession instrSession,
                                                           ViReal64* maximum,
                                                           ViReal64* percentMaximum)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, NULL, RSSPECAN_ATTR_FETCH_OFDM_VSA_EVM_PEAK_PILOT_MAXIMUM, maximum),
            2, "Maximum");

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, NULL, RSSPECAN_ATTR_FETCH_OFDM_VSA_EVM_PEAK_PILOT_PERCENT_MAXIMUM, percentMaximum),
            3, "Percent Maximum");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Returns the maximum EVM Peak All results.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR maximum/Returns the maximum EVM Peak All.
/// HIPAR percentMaximum/Returns the maximum EVM Peak All in percents.
ViStatus _VI_FUNC rsspecan_FetchOFDMVSAEVMPeakAllMaximum(ViSession instrSession,
                                                         ViReal64* maximum,
                                                         ViReal64* percentMaximum)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, NULL, RSSPECAN_ATTR_FETCH_OFDM_VSA_EVM_PEAK_ALL_MAXIMUM, maximum),
            2, "Maximum");

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, NULL, RSSPECAN_ATTR_FETCH_OFDM_VSA_EVM_PEAK_ALL_PERCENT_MAXIMUM, percentMaximum),
            3, "Percent Maximum");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Returns the frequency error reuslts.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR maximum/Returns the maximum result of the specified parameter.
/// HIPAR minimum/Returns the minimum result of the specified parameter.
/// HIPAR average/Returns the average result of the specified parameter.
ViStatus _VI_FUNC rsspecan_FetchOFDMVSAFrequencyErrorResults(ViSession instrSession,
                                                             ViReal64* maximum,
                                                             ViReal64* minimum,
                                                             ViReal64* average)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, NULL, RSSPECAN_ATTR_FETCH_OFDM_VSA_FREQUENCY_ERROR_MAXIMUM, maximum),
            2, "Maximum");

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, NULL, RSSPECAN_ATTR_FETCH_OFDM_VSA_FREQUENCY_ERROR_MINIMUM, minimum),
            3, "Minimum");

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, NULL, RSSPECAN_ATTR_FETCH_OFDM_VSA_FREQUENCY_ERROR_AVERAGE, average),
            4, "Average");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Returns the sampling error results.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR maximum/Returns the maximum result of the specified parameter.
/// HIPAR minimum/Returns the minimum result of the specified parameter.
/// HIPAR average/Returns the average result of the specified parameter.
ViStatus _VI_FUNC rsspecan_FetchOFDMVSASamplingErrorResults(ViSession instrSession,
                                                            ViReal64* maximum,
                                                            ViReal64* minimum,
                                                            ViReal64* average)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, NULL, RSSPECAN_ATTR_FETCH_OFDM_VSA_SAMPLING_ERROR_MAXIMUM, maximum),
            2, "Maximum");

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, NULL, RSSPECAN_ATTR_FETCH_OFDM_VSA_SAMPLING_ERROR_MINIMUM, minimum),
            3, "Minimum");

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, NULL, RSSPECAN_ATTR_FETCH_OFDM_VSA_SAMPLING_ERROR_AVERAGE, average),
            4, "Average");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Returns the quadrature error results.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR maximum/Returns the maximum result of the specified parameter.
/// HIPAR minimum/Returns the minimum result of the specified parameter.
/// HIPAR average/Returns the average result of the specified parameter.
ViStatus _VI_FUNC rsspecan_FetchOFDMVSAQuadratureErrorResults(
                            ViSession instrSession,
                            ViReal64* maximum,
                            ViReal64* minimum,
                            ViReal64* average)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, NULL, RSSPECAN_ATTR_FETCH_OFDM_VSA_QUADRATURE_ERROR_MAXIMUM, maximum),
            2, "Maximum");

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, NULL, RSSPECAN_ATTR_FETCH_OFDM_VSA_QUADRATURE_ERROR_MINIMUM, minimum),
            3, "Minimum");

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, NULL, RSSPECAN_ATTR_FETCH_OFDM_VSA_QUADRATURE_ERROR_AVERAGE, average),
            4, "Average");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Returns the modulation error results.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR maximum/Returns the maximum result of the specified parameter.
/// HIPAR minimum/Returns the minimum result of the specified parameter.
/// HIPAR average/Returns the average result of the specified parameter.
ViStatus _VI_FUNC rsspecan_FetchOFDMVSAModulationErrorResults(
                            ViSession instrSession,
                            ViReal64* maximum,
                            ViReal64* minimum,
                            ViReal64* average)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, NULL, RSSPECAN_ATTR_FETCH_OFDM_VSA_MODULATION_ERROR_MAXIMUM, maximum),
            2, "Maximum");

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, NULL, RSSPECAN_ATTR_FETCH_OFDM_VSA_MODULATION_ERROR_MINIMUM, minimum),
            3, "Minimum");

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, NULL, RSSPECAN_ATTR_FETCH_OFDM_VSA_MODULATION_ERROR_AVERAGE, average),
            4, "Average");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Returns the gain imbalance results.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR maximum/Returns the maximum result of the specified parameter.
/// HIPAR minimum/Returns the minimum result of the specified parameter.
/// HIPAR average/Returns the average result of the specified parameter.
ViStatus _VI_FUNC rsspecan_FetchOFDMVSAGainImbalanceResults(ViSession instrSession,
                                                            ViReal64* maximum,
                                                            ViReal64* minimum,
                                                            ViReal64* average)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, NULL, RSSPECAN_ATTR_FETCH_OFDM_VSA_GAIN_IMBALANCE_MAXIMUM, maximum),
            2, "Maximum");

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, NULL, RSSPECAN_ATTR_FETCH_OFDM_VSA_GAIN_IMBALANCE_MINIMUM, minimum),
            3, "Minimum");

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, NULL, RSSPECAN_ATTR_FETCH_OFDM_VSA_GAIN_IMBALANCE_AVERAGE, average),
            4, "Average");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Returns the IQ offset results.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR maximum/Returns the maximum result of the specified parameter.
/// HIPAR minimum/Returns the minimum result of the specified parameter.
/// HIPAR average/Returns the average result of the specified parameter.
ViStatus _VI_FUNC rsspecan_FetchOFDMVSAIQOffsetResults(ViSession instrSession,
                                                       ViReal64* maximum,
                                                       ViReal64* minimum,
                                                       ViReal64* average)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, NULL, RSSPECAN_ATTR_FETCH_OFDM_VSA_IQ_OFFSET_MAXIMUM, maximum),
            2, "Maximum");

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, NULL, RSSPECAN_ATTR_FETCH_OFDM_VSA_IQ_OFFSET_MINIMUM, minimum),
            3, "Minimum");

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, NULL, RSSPECAN_ATTR_FETCH_OFDM_VSA_IQ_OFFSET_AVERAGE, average),
            4, "Average");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Returns the power results.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR maximum/Returns the maximum result of the specified parameter.
/// HIPAR minimum/Returns the minimum result of the specified parameter.
/// HIPAR average/Returns the average result of the specified parameter.
ViStatus _VI_FUNC rsspecan_FetchOFDMVSAPowerResults(ViSession instrSession,
                                                    ViReal64* maximum,
                                                    ViReal64* minimum,
                                                    ViReal64* average)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, NULL, RSSPECAN_ATTR_FETCH_OFDM_VSA_POWER_MAXIMUM, maximum),
            2, "Maximum");

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, NULL, RSSPECAN_ATTR_FETCH_OFDM_VSA_POWER_MINIMUM, minimum),
            3, "Minimum");

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, NULL, RSSPECAN_ATTR_FETCH_OFDM_VSA_POWER_AVERAGE, average),
            4, "Average");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

