/*****************************************************************************
 *  Rohde&Schwarz Spectrum Analyzer instrument driver
 *  LTE option K100/K102
 *
 *  Original Release: May 2014
 *  By: Martin Krcmar
 *
 *  Should you have any technical questions please contact the hotline of
 *  Rohde & Schwarz Vertriebs-GmbH Rohde & Schwarz Support Center
 *
 *  e-mail: CustomerSupport@rohde-schwarz.com
 *
 *****************************************************************************/

#include "rsspecan.h"

/// HIFN  This function selects LTE Downlink analyzer mode.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsspecan_SetLTEDownlinkMode(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_LTE_DOWNLINK_MODE, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the general settings concerning the physical
/// HIFN  attributes of the signals to be measured.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR componentCarrier/This control selects the component carrier to be used.
/// HIPAR duplexing/Configures the duplexing mode.
/// HIPAR frequency/The control defines the center frequency of the analyzer.
/// HIPAR bandwidthSettingType/Setting type of bandwidth
/// HIPAR channelBandwidth/Setting cannel bandwidth.
/// HIPAR number_ofResourceBlocks/Configures the number of resource blocks used for DL.
/// HIPAR cyclicPrefix/Configures the cyclic prefix type for DL.
ViStatus _VI_FUNC rsspecan_ConfigureLTEDownlinkSignalCharacteristics(
                            ViSession instrSession,
                            ViInt32   componentCarrier,
                            ViInt32   duplexing,
                            ViReal64  frequency,
                            ViInt32   bandwidthSettingType,
                            ViInt32   channelBandwidth,
                            ViInt32   number_ofResourceBlocks,
                            ViInt32   cyclicPrefix)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, componentCarrier, 1, 5),
            2, "Component Carrier");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_LTE_DOWNLINK_DUPLEXING, duplexing),
            3, "Duplexing");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "cc%ld", componentCarrier);

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_LTE_DOWNLINK_CARRIER_CENTER_FREQUENCY, frequency),
            4, "Frequency");

    switch (bandwidthSettingType)
    {
    case RSSPECAN_VAL_LTE_DOWNLINK_CHBW:
        viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_LTE_DOWNLINK_CHANNEL_BANDWIDTH, channelBandwidth),
                6, "Channel bandwidth");
        break;
    case RSSPECAN_VAL_LTE_DOWNLINK_NRB:
        viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_LTE_DOWNLINK_NUMBER_OF_RESOURCE_BLOCKS, number_ofResourceBlocks),
                7, "Number of Resource Blocks");
        break;
    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, bandwidthSettingType), 5, "Bandwidth Setting Type");
        break;
    }

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_LTE_DOWNLINK_CYCLIC_PREFIX, cyclicPrefix),
            8, "Cyclic Prefix");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the TDD UL/DL Allocations for DL and TDD special subframe.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR componentCarrier/This control selects the component carrier to be used.
/// HIPAR tddULDLAllocations/Configures the TDD UL/DL Allocations for DL.
/// HIPAR specialSubframe/Selects the special subframe configuration for LTE downlink signals.
ViStatus _VI_FUNC rsspecan_ConfigureLTEDownlinkTDDFrames(ViSession instrSession,
                                                         ViInt32   componentCarrier,
                                                         ViInt32   tddULDLAllocations,
                                                         ViInt32   specialSubframe)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, componentCarrier, 1, 5),
            2, "Component Carrier");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "cc%ld", componentCarrier);

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_LTE_DOWNLINK_TDD_UL_DL_ALLOCATIONS, tddULDLAllocations),
            3, "TDD UL/DL Allocations");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_LTE_DOWNLINK_TDD_SPECIAL_SUBFRAME, specialSubframe),
            4, "Special Subframe");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures physical layer cell identity. There are 504 unique
/// HIFN  physical-layer cell identities. The physical-layer cell identities are
/// HIFN  grouped into 168 unique physical-layer cell-identity groups, each group
/// HIFN  containing  three unique identities.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR componentCarrier/This control selects the component carrier to be used.
/// HIPAR autoDetection/Enables/disables automatic cell identity group for DL.
/// HIPAR cellIdentityGroup/Configures the current cell identity group for DL.
/// HIPAR cellIdentity/Configures the current cell identity for DL.
ViStatus _VI_FUNC rsspecan_ConfigureLTEDownlinkPhysicalLayerCellIdentity(
                            ViSession instrSession,
                            ViInt32   componentCarrier,
                            ViBoolean autoDetection,
                            ViInt32   cellIdentityGroup,
                            ViInt32   cellIdentity)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, componentCarrier, 1, 5),
            2, "Component Carrier");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "cc%ld", componentCarrier);

    if (autoDetection == VI_TRUE)
    {
        viCheckParm(rsspecan_SetAttributeViString(instrSession, repCap, RSSPECAN_ATTR_LTE_DOWNLINK_CID_GROUP_AUTO, NULL),
                0, "");

        viCheckParm(rsspecan_SetAttributeViString(instrSession, repCap, RSSPECAN_ATTR_LTE_DOWNLINK_CELL_IDENTITY_AUTO, NULL),
                0, "");
    }
    else
    {
        viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_LTE_DOWNLINK_CELL_IDENTITY_GROUP, cellIdentityGroup),
                4, "Cell Identity Group");

        viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_LTE_DOWNLINK_CELL_IDENTITY, cellIdentity),
                5, "Auto Cell Identity");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the physical layer cell identity.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR componentCarrier/This control selects the component carrier to be used.
/// HIPAR cellIdentityGroup/Returns the current cell identity group for DL.
/// HIPAR cellIdentity/Returns the current cell identity for DL.
ViStatus _VI_FUNC rsspecan_QueryLTEDownlinkPhysicalLayerCellIdentity(
                            ViSession instrSession,
                            ViInt32   componentCarrier,
                            ViInt32*  cellIdentityGroup,
                            ViInt32*  cellIdentity)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, componentCarrier, 1, RSSPECAN_VAL_LTE_DL_CC_REPCAP_MAX),
            2, "Component Carrier");
    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_LTE_DOWNLINK_DETECTED_CELL_IDENTITY_GROUP,
            "cc", componentCarrier - 1, RS_REPCAP_BUF_SIZE, repCap));

    viCheckParm(rsspecan_GetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_LTE_DOWNLINK_DETECTED_CELL_IDENTITY_GROUP, cellIdentityGroup),
            3, "Cell Identity Group");

    viCheckParm(rsspecan_GetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_LTE_DOWNLINK_DETECTED_CELL_IDENTITY, cellIdentity),
            4, "Cell Identity");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Excludes the resource blocks used by the NB-IoT signal from the
/// HIFN  measurement results. When this feature is turned on, the location of
/// HIFN  the NB-IoT signal within the LTE carrier as an resource block offset
/// HIFN  can be defined.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR componentCarrier/This control selects the component carrier to be used.
/// HIPAR enabled/Turns the exclusion of resource blocks used by an NB-IoT inband
/// HIPAR enabled/deployment from the LTE measurement results on and off.
/// HIPAR nbiotRBOffset/Defines an the location of the NB-IoT signal within the LTE carrier as
/// HIPAR nbiotRBOffset/a resource block offset. Prerequisites for this control - turn on
/// HIPAR nbiotRBOffset/exclusion of inband NB-IoT with control Enabled.
ViStatus _VI_FUNC rsspecan_ConfigureLTEDownlinkExcludeInbandNBIoT(
                            ViSession instrSession,
                            ViInt32   componentCarrier,
                            ViBoolean enabled,
                            ViInt32   nbiotRBOffset)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, componentCarrier, 1, RSSPECAN_VAL_LTE_DL_CC_REPCAP_MAX),
            2, "Component Carrier");
    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_LTE_DOWNLINK_EXCLUDE_INBAND_NB_IOT_ENABLED,
            "cc", componentCarrier - 1, RS_REPCAP_BUF_SIZE, repCap));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, repCap, RSSPECAN_ATTR_LTE_DOWNLINK_EXCLUDE_INBAND_NB_IOT_ENABLED, enabled),
            3, "Enabled");

    if (enabled == VI_TRUE)
    {
        viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_LTE_DOWNLINK_EXCLUDED_INBAND_NB_IOT_RB_OFFSET, nbiotRBOffset),
                4, "NBIoT RB Offset");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Saves the signal description.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR componentCarrier/This control selects the component carrier to be used.
/// HIPAR filename/Saves the signal description.
ViStatus _VI_FUNC rsspecan_StoreLTEDownlinkDemodulationSettings(
                            ViSession instrSession,
                            ViInt32   componentCarrier,
                            ViString  filename)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, componentCarrier, 1, RSSPECAN_VAL_LTE_DL_CC_REPCAP_MAX),
            2, "Component Carrier");
    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_LTE_DOWNLINK_STORE_DEMODULATION_SETTINGS,
            "cc", componentCarrier - 1, RS_REPCAP_BUF_SIZE, repCap));

    viCheckParm(rsspecan_SetAttributeViString(instrSession, repCap, RSSPECAN_ATTR_LTE_DOWNLINK_STORE_DEMODULATION_SETTINGS, filename),
            3, "File Name");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures essential antenna configuration parameters.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR componentCarrier/This control selects the component carrier to be used.
/// HIPAR configuration/Configures the current MIMO antenna configuration.
/// HIPAR antennaSelection/Configures the antenna selection.
ViStatus _VI_FUNC rsspecan_ConfigureLTEDownlinkMIMO(ViSession instrSession,
                                                    ViInt32   componentCarrier,
                                                    ViInt32   configuration,
                                                    ViInt32   antennaSelection)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, componentCarrier, 1, 5),
            2, "Component Carrier");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "cc%ld", componentCarrier);

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_LTE_DOWNLINK_MIMO_CONFIGURATION, configuration),
            3, "Configuration");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_LTE_DOWNLINK_MIMO_ANTENNA, antennaSelection),
            4, "Antenna Selection");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures multiple analyzers connection. MIMO
/// HIFN  measurements require several R&S FSWs (input sources), depending on the
/// HIFN  number of antennas you have to measure. One of these analyzers (master)
/// HIFN  controls the other analyzers. The master analyzer has to be equipped
/// HIFN  with the LTE measurement application.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR rxAnalyzer/Defines the analyzer number.
/// HIPAR state/Includes or excludes an analyzer from a MIMO setup.
/// HIPAR analyzerIPAddress/Defines the network address of an analyzer in the test setup.
ViStatus _VI_FUNC rsspecan_ConfigureLTEDownlinkMIMOMultipleAnalyzers(
                            ViSession instrSession,
                            ViInt32   rxAnalyzer,
                            ViBoolean state,
                            ViString  analyzerIPAddress)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, rxAnalyzer, 2, 4),
            2, "RX Analyzer");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "AntMatrix%ld", rxAnalyzer);

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, repCap, RSSPECAN_ATTR_LTE_DOWNLINK_MIMO_ANALYZER_STATE, state),
            3, "State");

    viCheckParm(rsspecan_SetAttributeViString(instrSession, repCap, RSSPECAN_ATTR_LTE_DOWNLINK_MIMO_ANALYZER_ADDRESS, analyzerIPAddress),
            4, "Analyzer IP Address");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures PDSCH settings.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR autoDemodulation/Activates or deactivates automatic demodulation of the PDSCH.
/// HIPAR pdschSubframeDetect/Selects the method of identifying the PDSCH resource allocation.
ViStatus _VI_FUNC rsspecan_ConfigureLTEDownlinkPDSCHSettings(ViSession instrSession,
                                                             ViBoolean autoDemodulation,
                                                             ViInt32   pdschSubframeDetect)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_LTE_DOWNLINK_AUTO_DEMODULATION, autoDemodulation),
            2, "Auto Demodulation");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_LTE_DOWNLINK_PDSCH_SUBFRAME_DETECT, pdschSubframeDetect),
            3, "PDSCH Subframe Detect");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the number of configurable subframes for DL.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR componentCarrier/This control selects the component carrier to be used.
/// HIPAR configurableSubframes/Sets the number of configurable subframes for DL.
ViStatus _VI_FUNC rsspecan_ConfigureLTEDownlinkPDSCHConfigurableSubframes(
                            ViSession instrSession,
                            ViInt32   componentCarrier,
                            ViInt32   configurableSubframes)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, componentCarrier, 1, 5),
            2, "Component Carrier");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "cc%ld", componentCarrier);

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_LTE_DOWNLINK_PDSCH_CONFIGURABLE_SUBFRAMES, configurableSubframes),
            3, "Configurable Subframes");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the number of used allocations for a specific DL subframe.
/// HIFN  The suffix stands for the corresponding subframe number.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR componentCarrier/This control selects the component carrier to be used.
/// HIPAR subframe/Selects configurable subframe.
/// HIPAR usedAllocations/Configures the number of used allocations for a specific DL subframe.
ViStatus _VI_FUNC rsspecan_ConfigureLTEDownlinkPDSCHUsedAllocations(
                            ViSession instrSession,
                            ViInt32   componentCarrier,
                            ViInt32   subframe,
                            ViInt32   usedAllocations)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, componentCarrier, 1, 5),
            2, "Component Carrier");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, subframe, 0, 9),
            3, "Subframe");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "cc%ld,SFR%ld", componentCarrier, subframe);

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_LTE_DOWNLINK_PDSCH_USED_ALLOCATIONS, usedAllocations),
            4, "Used Allocations");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures specified allocation in specified subframe.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR componentCarrier/This control selects the component carrier to be used.
/// HIPAR subframe/Selects configurable subframe.
/// HIPAR usedAllocation/Selects allocation.
/// HIPAR id_N_RNTI/Selects the allocation's ID. The ID corresponds to the N_RNTI. By
/// HIPAR id_N_RNTI/default, the application assigns consecutive numbers starting with 0.
/// HIPAR codeword/Selects codeword.
/// HIPAR modulation/Configures the modulation type for an allocation in a specific DL
/// HIPAR modulation/subframe. In case of MIMO there might be more than one codewords which
/// HIPAR modulation/can be configured separately. The first suffix stands for the
/// HIPAR modulation/corresponding subframe number and the second one stands for the
/// HIPAR modulation/corresponding allocation number. The third suffix stands for the
/// HIPAR modulation/codeword index.
/// HIPAR vrbGap/This control turns the use of virtual resource blocks (VRB) on and
/// HIPAR vrbGap/off.
/// HIPAR resourceBlocks/Configures the number of resource blocks for an allocation in a
/// HIPAR resourceBlocks/specific DL subframe.
/// HIPAR resourceBlocksOffset/Configures the resource block offset for an allocation in a specific
/// HIPAR resourceBlocksOffset/DL subframe.
/// HIPAR power/Configures the relative power of an allocation in a specific DL
/// HIPAR power/subframe.
ViStatus _VI_FUNC rsspecan_ConfigureLTEDownlinkPDSCHUsedAllocation(
                            ViSession instrSession,
                            ViInt32   componentCarrier,
                            ViInt32   subframe,
                            ViInt32   usedAllocation,
                            ViInt32   id_N_RNTI,
                            ViInt32   codeword,
                            ViInt32   modulation,
                            ViInt32   vrbGap,
                            ViInt32   resourceBlocks,
                            ViInt32   resourceBlocksOffset,
                            ViReal64  power)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap2[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, componentCarrier, 1, 5),
            2, "Component Carrier");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, subframe, 0, 9),
            3, "Subframe");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, usedAllocation, 0, 99),
            4, "Used Allocation");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, codeword, 1, 2),
            6, "Codeword");

    snprintf(repCap2, RS_REPCAP_BUF_SIZE, "cc%ld,SFR%ld,AL%ld", componentCarrier, subframe, usedAllocation);
    snprintf(repCap, RS_REPCAP_BUF_SIZE, "cc%ld,SFR%ld,AL%ld,CW%ld", componentCarrier, subframe, usedAllocation,
             codeword);

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap2, RSSPECAN_ATTR_LTE_DOWNLINK_PDSCH_ALLOCATION_ID, id_N_RNTI),
            5, "ID / N_RNTI");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_LTE_DOWNLINK_PDSCH_ALLOCATION_MODULATION, modulation),
            7, "Modulation");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap2, RSSPECAN_ATTR_LTE_DOWNLINK_PDSCH_ALLOCATION_VRB_GAP, vrbGap),
            8, "VRB Gap");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap2, RSSPECAN_ATTR_LTE_DOWNLINK_PDSCH_ALLOCATION_RESOURCE_BLOCKS, resourceBlocks),
            9, "Resource Blocks");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap2, RSSPECAN_ATTR_LTE_DOWNLINK_PDSCH_ALLOCATION_RESOURCE_BLOCKS_OFFSET, resourceBlocksOffset),
            10, "Resource Blocks Offset");

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, repCap2, RSSPECAN_ATTR_LTE_DOWNLINK_PDSCH_ALLOCATION_POWER, power),
            11, "Power");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the precoding scheme of a physical channel.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR subframe/Selects configurable subframe.
/// HIPAR usedAllocation/Selects allocation.
/// HIPAR precoding/This control selects the precoding scheme of an allocation.
/// HIPAR codewordToLayerMapping/This control selects the codeword to layer mapping.
/// HIPAR scramblingIdentity/This control selects the scrambling identity (nSCID). The control is
/// HIPAR scramblingIdentity/available for antenna ports 7 and 8.
/// HIPAR singleLayerAntennaPort/This control selects the antenna port for the beamforming scheme. The
/// HIPAR singleLayerAntennaPort/control is available for measurements on a single antenna.
/// HIPAR codebookIndex/This control selects the codebook index for an allocation with spatial
/// HIPAR codebookIndex/multiplexing precoding scheme.
/// HIPAR cyclicDelayDiversity/This control turns the cyclic delay diversity of an allocation with
/// HIPAR cyclicDelayDiversity/spatial multiplexing precoding scheme on and off.
/// HIPAR startOffset/This control defines the PDSCH start offset for a particular PDSCH
/// HIPAR startOffset/allocation.
ViStatus _VI_FUNC rsspecan_ConfigureLTEDownlinkPDSCHUsedAllocationEnhancedSettings(
                            ViSession instrSession,
                            ViInt32   componentCarrier,
                            ViInt32   subframe,
                            ViInt32   usedAllocation,
                            ViInt32   precoding,
                            ViInt32   codewordToLayerMapping,
                            ViInt32   scramblingIdentity,
                            ViInt32   singleLayerAntennaPort,
                            ViInt32   codebookIndex,
                            ViBoolean cyclicDelayDiversity,
                            ViInt32   startOffset)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, componentCarrier, 1, 5),
            2, "Component Carrier");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, subframe, 0, 9),
            3, "Subframe");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, usedAllocation, 0, 99),
            4, "Used Allocation");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "cc%ld,SFR%ld,AL%ld", componentCarrier, subframe, usedAllocation);

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_LTE_DOWNLINK_PDSCH_ALLOCATION_PRECODING, precoding),
            5, "Precoding");

    if (precoding == RSSPECAN_VAL_LTE_DOWNLINK_PRECODING_BF)
    {
        viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_LTE_DOWNLINK_PDSCH_ALLOCATION_CODEWORD_TO_LAYER_MAPPING, codewordToLayerMapping),
                6, "Codeword To Layer Mapping");

        if (codewordToLayerMapping == RSSPECAN_VAL_LTE_DOWNLINK_LAYER_MAPPING_11)
        {
            viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_LTE_DOWNLINK_PDSCH_ALLOCATION_SINGLE_LAYER_ANTENNA_PORT, singleLayerAntennaPort),
                    8, "Single Layer Antenna Port");
        }
        else
        {
            viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_LTE_DOWNLINK_PDSCH_ALLOCATION_SCRAMBLING_IDENTITY, scramblingIdentity),
                    7, "Scrambling Identity");
        }
    }
    if (precoding == RSSPECAN_VAL_LTE_DOWNLINK_PRECODING_SPM)
    {
        viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_LTE_DOWNLINK_PDSCH_ALLOCATION_CODEWORD_TO_LAYER_MAPPING, codewordToLayerMapping),
                6, "Codeword To Layer Mapping");

        viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_LTE_DOWNLINK_PDSCH_ALLOCATION_CODEBOOK_INDEX, codebookIndex),
                9, "Codebook Index");

        viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, repCap, RSSPECAN_ATTR_LTE_DOWNLINK_PDSCH_ALLOCATION_CYCLIC_DELAY_DIVERSITY, cyclicDelayDiversity),
                10, "Cyclic Delay Diversity");
    }

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_LTE_DOWNLINK_PDSCH_ALLOCATION_START_OFFSET, startOffset),
            11, "Start Offset");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures synchronization signal.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR componentCarrier/This control selects the component carrier to be used.
/// HIPAR p_SSyncTxAntenna/This control selects the antenna that transmits the P-SYNC and the
/// HIPAR p_SSyncTxAntenna/S-SYNC.
/// HIPAR pSYNCRelativePower/Configures the P-SYNC relative power for DL.
/// HIPAR sSYNCRelativePower/Configures the S-SYNC relative power for DL.
ViStatus _VI_FUNC rsspecan_ConfigureLTEDownlinkSynchronizationSignal(
                            ViSession instrSession,
                            ViInt32   componentCarrier,
                            ViInt32   p_SSyncTxAntenna,
                            ViReal64  pSYNCRelativePower,
                            ViReal64  sSYNCRelativePower)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, componentCarrier, 1, 5),
            2, "Component Carrier");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "cc%ld", componentCarrier);

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_LTE_DOWNLINK_SYNC_ANTENNA, p_SSyncTxAntenna),
            3, "P-/S-Sync Tx Antenna");

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_LTE_DOWNLINK_PSYNC_RELATIVE_POWER, pSYNCRelativePower),
            4, "P-SYNC Relative Power");

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_LTE_DOWNLINK_SSYNC_RELATIVE_POWER, sSYNCRelativePower),
            5, "S-SYNC Relative Power");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN Configures the relative power of the reference signal for DL.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR componentCarrier/This control selects the component carrier to be used.
/// HIPAR referencePower/Configures the relative power of the reference signal for DL.
ViStatus _VI_FUNC rsspecan_ConfigureLTEDownlinkReferenceSignalCarr(
                            ViSession instrSession,
                            ViInt32   componentCarrier,
                            ViReal64  referencePower)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, componentCarrier, 1, RSSPECAN_VAL_LTE_DL_CC_REPCAP_MAX),
            2, "Component Carrier");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_LTE_DOWNLINK_REFERENCE_POWER_CARR,
            "cc", componentCarrier - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Component Carrier");

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_LTE_DOWNLINK_REFERENCE_POWER_CARR, referencePower),
            3, "Reference Power");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the relative power of the reference signal for DL.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR referencePower/Configures the relative power of the reference signal for DL.
ViStatus _VI_FUNC rsspecan_ConfigureLTEDownlinkReferenceSignal(
                            ViSession instrSession,
                            ViReal64  referencePower)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_LTE_DOWNLINK_REFERENCE_POWER, referencePower),
            2, "Reference Power");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the physical attributes and structure of the positioning
/// HIFN  reference signal.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR componentCarrier/This control selects the component carrier to be used.
/// HIPAR present/Turns the positioning reference signal on and off.
/// HIPAR bandwidth/Defines the bandwidth and thus the number of resource blocks the
/// HIPAR bandwidth/positioning reference signal occupies.
/// HIPAR configurationIndex/Defines the PRS Configuration Index I PRS  as defined in 3GPP TS
/// HIPAR configurationIndex/36.211, table 6.10.4.3-1.
/// HIPAR relativePower/Defines the power of a PRS resource element in relation to the power
/// HIPAR relativePower/of a common reference signal resource element.
/// HIPAR numberofSubframes/Defines the number of consecutive DL subframes in that PRS are
/// HIPAR numberofSubframes/transmitted.
/// HIPAR frameNumberOffset/Defines the system frame number of the current frame that you want to
/// HIPAR frameNumberOffset/analyze. Because the positioning reference signal and the CSI reference
/// HIPAR frameNumberOffset/signal usually have a periodicity of several frames, for some reference
/// HIPAR frameNumberOffset/signal configurations is it necessary to change the expected system
/// HIPAR frameNumberOffset/frame number of the frame to be analyzed.
ViStatus _VI_FUNC rsspecan_ConfigureLTEDownlinkPositioningReferenceSignal(
                            ViSession instrSession,
                            ViInt32   componentCarrier,
                            ViBoolean present,
                            ViInt32   bandwidth,
                            ViInt32   configurationIndex,
                            ViReal64  relativePower,
                            ViInt32   numberofSubframes,
                            ViInt32   frameNumberOffset)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, componentCarrier, 1, 5),
            2, "Component Carrier");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "cc%ld", componentCarrier);

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, repCap, RSSPECAN_ATTR_LTE_DOWNLINK_POSITIONING_REFERENCE_SIGNAL_ENABLED, present),
            3, "Present");

    if (present == VI_TRUE)
    {
        viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_LTE_DOWNLINK_POSITIONING_REFERENCE_SIGNAL_BANDWIDTH, bandwidth),
                4, "Bandwidth");

        viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_LTE_DOWNLINK_POSITIONING_REFERENCE_SIGNAL_CONFIGURATION_INDEX, configurationIndex),
                5, "Configuration Index");

        viCheckParm(rsspecan_SetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_LTE_DOWNLINK_POSITIONING_REFERENCE_SIGNAL_RELATIVE_POWER, relativePower),
                6, "Relative Power");

        viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_LTE_DOWNLINK_POSITIONING_REFERENCE_SIGNAL_SUBFRAMES, numberofSubframes),
                7, "Number of Subframes");

        viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_LTE_DOWNLINK_POSITIONING_REFERENCE_SIGNAL_FRAME_NUMBER_OFFSET, frameNumberOffset),
                8, "Frame Number Offset");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the channel state information reference signal (CSI-RS)
/// HIFN  settings.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR componentCarrier/This control selects the component carrier to be used.
/// HIPAR present/This control switches the CSI reference signal on and off.
/// HIPAR antenaPorts/This control selects the number of antenna ports that transmit the CSI
/// HIPAR antenaPorts/reference signal.
/// HIPAR configurationIndex/This control selects the configuration index for the CSI reference
/// HIPAR configurationIndex/signal.
/// HIPAR overwritePDSCH/This control turns overwriting of PDSCH resource elements for UEs that
/// HIPAR overwritePDSCH/do not consider the CSI reference signal on and off.
/// HIPAR relativePower/This control defines the relative power of the CSI reference signal.
/// HIPAR subframeConfiguration/This control defines the subframe configuration for the CSI reference
/// HIPAR subframeConfiguration/signal.
/// HIPAR frameNumberOffset/This control defines the frame number offset for the positioning
/// HIPAR frameNumberOffset/reference signal.
ViStatus _VI_FUNC rsspecan_ConfigureLTEDownlinkCSIReferenceSignal(
                            ViSession instrSession,
                            ViInt32   componentCarrier,
                            ViBoolean present,
                            ViInt32   antenaPorts,
                            ViInt32   configurationIndex,
                            ViBoolean overwritePDSCH,
                            ViReal64  relativePower,
                            ViInt32   subframeConfiguration,
                            ViInt32   frameNumberOffset)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, componentCarrier, 1, 5),
            2, "Component Carrier");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "cc%ld", componentCarrier);

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, repCap, RSSPECAN_ATTR_LTE_DOWNLINK_CSI_RS_STATE, present),
            3, "Present");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_LTE_DOWNLINK_CSI_RS_ANTENNA_PORTS, antenaPorts),
            4, "antenaPorts");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_LTE_DOWNLINK_CSI_RS_CONFIGURATION_INDEX, configurationIndex),
            5, "configurationIndex");

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, repCap, RSSPECAN_ATTR_LTE_DOWNLINK_CSI_RS_OVERWRITE_PDSCH, overwritePDSCH),
            6, "overwritePDSCH");

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_LTE_DOWNLINK_CSI_RS_RELATIVE_POWER, relativePower),
            7, "relativePower");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_LTE_DOWNLINK_CSI_RS_SUBFRAME_CONFIGURATION, subframeConfiguration),
            8, "subframeConfiguration");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_LTE_DOWNLINK_CSI_RS_FRAME_NUMBER_OFFSET, frameNumberOffset),
            9, "frameNumberOffset");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN Configures the PRB Symbol Offset (Control Region for PDCCH).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR componentCarrier/This control selects the component carrier to be used.
/// HIPAR PRBSymbolOffset/Specifies the PRB Symbol Offset (Control Region for PDCCH).
ViStatus _VI_FUNC rsspecan_ConfigureLTEDownlinkPRBSymbolOffset(
                            ViSession instrSession,
                            ViInt32   componentCarrier,
                            ViInt32   PRBSymbolOffset)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, componentCarrier, 1, 5),
            2, "Component Carrier");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "cc%ld", componentCarrier);

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_LTE_DOWNLINK_PRB_SYMBOL_OFFSET, PRBSymbolOffset),
            3, "PRB Symbol Offset");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures physical broadcast channel.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR componentCarrier/This control selects the component carrier to be used.
/// HIPAR PBCHPresent/Activates or deactivates the physical broadcast channel.
/// HIPAR PBCHRelativePower/Configures the PBCH relative power for DL.
ViStatus _VI_FUNC rsspecan_ConfigureLTEDownlinkPBCH(ViSession instrSession,
                                                    ViInt32   componentCarrier,
                                                    ViBoolean PBCHPresent,
                                                    ViReal64  PBCHRelativePower)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, componentCarrier, 1, 5),
            2, "Component Carrier");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "cc%ld", componentCarrier);

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, repCap, RSSPECAN_ATTR_LTE_DOWNLINK_PBCH_STATE, PBCHPresent),
            3, "PBCH Present");

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_LTE_DOWNLINK_PBCH_RELATIVE_POWER, PBCHRelativePower),
            4, "PBCH Relative Power");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures PCFICH.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR componentCarrier/This control selects the component carrier to be used.
/// HIPAR PCFICHPresent/Activates or deactivates the PCFICH for DL.
/// HIPAR PCFICHRelativePower/Configures the PCFICH relative power for DL.
ViStatus _VI_FUNC rsspecan_ConfigureLTEDownlinkPCFICH(ViSession instrSession,
                                                      ViInt32   componentCarrier,
                                                      ViBoolean PCFICHPresent,
                                                      ViReal64  PCFICHRelativePower)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, componentCarrier, 1, 5),
            2, "Component Carrier");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "cc%ld", componentCarrier);

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, repCap, RSSPECAN_ATTR_LTE_DOWNLINK_PCFICH_STATE, PCFICHPresent),
            3, "PCFICH Present");

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_LTE_DOWNLINK_PCFICH_RELATIVE_POWER, PCFICHRelativePower),
            4, "PCFICH Relative Power");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures PHICH.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR componentCarrier/This control selects the component carrier to be used.
/// HIPAR PHICHNumber_ofGroups/Configures the number of PHICH groups for DL.
/// HIPAR PHICHDuration/Configure the PHICH duration for DL.
/// HIPAR PHICHRelativePower/Configures the PHICH relative power for DL.
ViStatus _VI_FUNC rsspecan_ConfigureLTEDownlinkPHICH(ViSession instrSession,
                                                     ViInt32   componentCarrier,
                                                     ViInt32   PHICHNumber_ofGroups,
                                                     ViInt32   PHICHDuration,
                                                     ViReal64  PHICHRelativePower)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, componentCarrier, 1, 5),
            2, "Component Carrier");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "cc%ld", componentCarrier);

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_LTE_DOWNLINK_PHICH_NUMBER_OF_GROUPS, PHICHNumber_ofGroups),
            3, "PHICH Number of Groups");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_LTE_DOWNLINK_PHICH_DURATION, PHICHDuration),
            4, "PHICH Duration");

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_LTE_DOWNLINK_PHICH_RELATIVE_POWER, PHICHRelativePower),
            5, "PHICH Relative Power");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures PHICH Ng parameter. Ng in combination with the number
/// HIFN  of resource blocks defines the number of PHICH groups in a
/// HIFN  downlink subframe.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init
/// HIPAR instrSession/or rsspecan_InitWithOptions function.
/// HIPAR componentCarrier/This control selects the component carrier to be used.
/// HIPAR PHICHNgParameter/Selects the method that determines the number of PHICH groups in a subframe.
/// HIPAR PHICHNumberOfGroups/Configures the number of PHICH groups for DL.
ViStatus _VI_FUNC rsspecan_ConfigureLTEDownlinkPHICHNgParameter(
                            ViSession instrSession,
                            ViInt32   componentCarrier,
                            ViInt32   PHICHNgParameter,
                            ViInt32   PHICHNumberOfGroups)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, componentCarrier, 1, 5),
            2, "Component Carrier");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "cc%ld", componentCarrier);

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_LTE_DOWNLINK_PHICH_NG_PARAMETER, PHICHNgParameter),
            3, "PHICH Ng Parameter");

    if (PHICHNgParameter == RSSPECAN_VAL_LTE_DOWNLINK_PHICH_NG_CUSTOM)
    {
        viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_LTE_DOWNLINK_PHICH_NUMBER_OF_GROUPS, PHICHNumberOfGroups),
                4, "PHICH Number Of Groups");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function includes or excludes the use of the PHICH special
/// HIFN  setting for enhanced test models. The special setting is defined
/// HIFN  in 36.141 V9.0.0, 6.1.2.6: For frame structure type 2 the
/// HIFN  factor m_i shall not be set as per TS36.211, Table 6.9-1, but
/// HIFN  instead shall be set to m_i=1 for all transmitted subframes.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init
/// HIPAR instrSession/or rsspecan_InitWithOptions function.
/// HIPAR componentCarrier/This control selects the component carrier to be used.
/// HIPAR PHICHTDDM_i1/Includes or excludes the use of the PHICH special setting for enhanced test models.
ViStatus _VI_FUNC rsspecan_ConfigureLTEDownlinkPHICHEnhancedTestModels(
                            ViSession instrSession,
                            ViInt32   componentCarrier,
                            ViBoolean PHICHTDDM_i1)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, componentCarrier, 1, 5),
            2, "Component Carrier");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "cc%ld", componentCarrier);

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, repCap, RSSPECAN_ATTR_LTE_DOWNLINK_PHICH_ENHANCED_TEST_MODELS, PHICHTDDM_i1),
            3, "PHICH TDD m_i=1");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures PDCCH.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR componentCarrier/This control selects the component carrier to be used.
/// HIPAR number_ofPDCCHs/Configures the number of physical downlink control channels.
/// HIPAR PDCCHFormat/Configures the physical downlink control channel format.
/// HIPAR PDCCHRelativePower/Configures the PDCCH relative power for DL.
ViStatus _VI_FUNC rsspecan_ConfigureLTEDownlinkPDCCH(ViSession instrSession,
                                                     ViInt32   componentCarrier,
                                                     ViInt32   number_ofPDCCHs,
                                                     ViInt32   PDCCHFormat,
                                                     ViReal64  PDCCHRelativePower)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, componentCarrier, 1, 5),
            2, "Component Carrier");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "cc%ld", componentCarrier);

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_LTE_DOWNLINK_PDCCH_FORMAT, PDCCHFormat),
            4, "PDCCH Format");

    if (PDCCHFormat != -1)
    {
        viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_LTE_DOWNLINK_PDCCH_NUMBER_OF_PDCCHS, number_ofPDCCHs),
                3, "Number of PDCCHs");
    }

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_LTE_DOWNLINK_PDCCH_RELATIVE_POWER, PDCCHRelativePower),
            5, "PDCCH Relative Power");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the PDSCH P_B parameter that defines the
/// HIFN  cell-specific ratio of rho_B to rho_A according to 3GPP TS
/// HIFN  36.213.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init.
/// HIPAR componentCarrier/This control selects the component carrier to be used.
/// HIPAR PDSCHPowerRatio/Configures the PDSCH P_B parameter that defines the
/// HIPAR PDSCHPowerRatio/cell-specific ratio of rho_B to rho_A according to 3GPP TS
/// HIPAR PDSCHPowerRatio/36.213
ViStatus _VI_FUNC rsspecan_ConfigureLTEDownlinkPDSCHPowerRatio(
                            ViSession instrSession,
                            ViInt32   componentCarrier,
                            ViInt32   PDSCHPowerRatio)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, componentCarrier, 1, 5),
            2, "Component Carrier");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "cc%ld", componentCarrier);

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_LTE_DOWNLINK_PDSCH_POWER_RATIO, PDSCHPowerRatio),
            3, "PDSCH Power Ratio");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN Configures EPDCCH.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init.
/// HIPAR componentCarrier/This control selects the component carrier to be used.
/// HIPAR PRBPairs/Selects the number of resource blocks that the EPDCCH-PRB set uses.
/// HIPAR relativePower/Defines the relative power of the EPDCCH.
/// HIPAR localized/Turns localized transmission of the EPDCCH on and off..
/// HIPAR setID/Defines the EPDCCH set ID used to generate EPDCCH reference symbols.
/// HIPAR RBAssign/Defines the resource blocks that the EPDCCH uses..
ViStatus _VI_FUNC rsspecan_ConfigureLTEDownlinkEPDCCH(ViSession instrSession,
                                                      ViInt32   componentCarrier,
                                                      ViInt32   PRBPairs,
                                                      ViReal64  relativePower,
                                                      ViBoolean localized,
                                                      ViInt32   setID,
                                                      ViInt32   RBAssign)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, componentCarrier, 1, 5),
            2, "Component Carrier");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "cc%ld", componentCarrier);

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_LTE_DOWNLINK_EPDCCH_PRB_PAIRS, PRBPairs),
            3, "PRB Pairs");

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_LTE_DOWNLINK_EPDCCH_RELATIVE_POWER, relativePower),
            4, "Relative Power");

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, repCap, RSSPECAN_ATTR_LTE_DOWNLINK_EPDCCH_LOCALIZED, localized),
            5, "Localized");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_LTE_DOWNLINK_EPDCCH_SET_ID, setID),
            6, "Set ID");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_LTE_DOWNLINK_EPDCCH_RB_ASSIGNMENT, RBAssign),
            7, "RB Assign");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN Configures MBSFN.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init.
/// HIPAR componentCarrier/This control selects the component carrier to be used.
/// HIPAR present/Includes or excludes an MBSFN from the test setup.
/// HIPAR relativePower/This attribute defines the relative power of the MBSFN transmission.
/// HIPAR areaID/Defines the ID of an MBFSN area.
/// HIPAR nonMBSFNRegionLength/Selects the length of the control data region in an MBSFN subframe.
ViStatus _VI_FUNC rsspecan_ConfigureLTEDownlinkMBSFN(ViSession instrSession,
                                                     ViInt32   componentCarrier,
                                                     ViBoolean present,
                                                     ViReal64  relativePower,
                                                     ViInt32   areaID,
                                                     ViInt32   nonMBSFNRegionLength)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, componentCarrier, 1, 5),
            2, "Component Carrier");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "cc%ld", componentCarrier);

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, repCap, RSSPECAN_ATTR_LTE_DOWNLINK_MBSFN_ENABLED, present),
            3, "Present");

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_LTE_DOWNLINK_MBSFN_RELATIVE_POWER, relativePower),
            4, "Relative Power");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_LTE_DOWNLINK_MBSFN_AREA_ID, areaID),
            5, "Area ID");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_LTE_DOWNLINK_MBSFN_NON_MBSF_REGION_LENGTH, nonMBSFNRegionLength),
            6, "Non-MBSF Region Length");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN Configures MBSFN subframe.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init.
/// HIPAR componentCarrier/This control selects the component carrier to be used.
/// HIPAR subframe/Selects the subframe number
/// HIPAR active/Turns a subframe into a MBSFN subframe.
/// HIPAR PMCHPresent/Turns the Physical Multicast Channel (PMCH) on and off.
/// HIPAR modulation/Selects the modulation type for an MBSFN subframe.
ViStatus _VI_FUNC rsspecan_ConfigureLTEDownlinkMBSFNSubframe(ViSession instrSession,
                                                             ViInt32   componentCarrier,
                                                             ViInt32   subframe,
                                                             ViBoolean active,
                                                             ViBoolean PMCHPresent,
                                                             ViInt32   modulation)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, componentCarrier, 1, 5),
            2, "Component Carrier");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, subframe, 0, 9),
            3, "Subframe");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "cc%ld,SFR%ld", componentCarrier, subframe);

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, repCap, RSSPECAN_ATTR_LTE_DOWNLINK_MBSFN_SUBFRAME_ENABLED, active),
            4, "Active");
    if (active)
    {
        viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, repCap, RSSPECAN_ATTR_LTE_DOWNLINK_MBSFN_SUBFRAME_PMCH_ENABLED, PMCHPresent),
                5, "PMCH Present");

        viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_LTE_DOWNLINK_MBSFN_SUBFRAME_PMCH_MODULATION, modulation),
                6, "Modulation");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Turns custom synchronization signal weighting on and off.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR componentCarrier/This control selects the component carrier to be used.
/// HIPAR enabled/Turns custom synchronization signal weighting on and off.
ViStatus _VI_FUNC rsspecan_ConfigureLTEDownlinkCustomSyncWeightEnabled(
                            ViSession instrSession,
                            ViInt32   componentCarrier,
                            ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, componentCarrier, 1, RSSPECAN_VAL_LTE_DL_CC_REPCAP_MAX),
            2, "Component Carrier");
    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_LTE_DOWNLINK_CUSTOM_SYNC_WEIGHT_ENABLED,
            "cc", componentCarrier - 1, RS_REPCAP_BUF_SIZE, repCap));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, repCap, RSSPECAN_ATTR_LTE_DOWNLINK_CUSTOM_SYNC_WEIGHT_ENABLED, enabled),
            3, "Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Configures custom synchronization signal weighting first half of the
/// HIFN  frame.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR componentCarrier/This control selects the component carrier to be used.
/// HIPAR imaginary/Defines the signal weight for the imaginary part of the signal in the
/// HIPAR imaginary/first half frame.
/// HIPAR real/Defines the signal weight for the real part of the signal in the first
/// HIPAR real/half frame.
ViStatus _VI_FUNC rsspecan_ConfigureLTEDownlinkCustomSyncWeightFirstHalf(
                            ViSession instrSession,
                            ViInt32   componentCarrier,
                            ViReal64  imaginary,
                            ViReal64  real)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, componentCarrier, 1, RSSPECAN_VAL_LTE_DL_CC_REPCAP_MAX),
            2, "Component Carrier");
    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_LTE_DOWNLINK_CUSTOM_SYNC_WEIGHT_FIRST_HALF_FRAME_IMAGINARY,
            "cc", componentCarrier - 1, RS_REPCAP_BUF_SIZE, repCap));

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_LTE_DOWNLINK_CUSTOM_SYNC_WEIGHT_FIRST_HALF_FRAME_IMAGINARY, imaginary),
            3, "Imaginary");

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_LTE_DOWNLINK_CUSTOM_SYNC_WEIGHT_FIRST_HALF_FRAME_REAL, real),
            4, "Real");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Configures custom synchronization signal weighting second half of the
/// HIFN  frame.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR componentCarrier/This control selects the component carrier to be used.
/// HIPAR imaginary/Defines the signal weight for the imaginary part of the signal in the
/// HIPAR imaginary/second half frame.
/// HIPAR real/Defines the signal weight for the real part of the signal in the
/// HIPAR real/second half frame.
ViStatus _VI_FUNC rsspecan_ConfigureLTEDownlinkCustomSyncWeightSecondHalf(
                            ViSession instrSession,
                            ViInt32   componentCarrier,
                            ViReal64  imaginary,
                            ViReal64  real)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, componentCarrier, 1, RSSPECAN_VAL_LTE_DL_CC_REPCAP_MAX),
            2, "Component Carrier");
    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_LTE_DOWNLINK_CUSTOM_SYNC_WEIGHT_SECOND_HALF_FRAME_IMAGINARY,
            "cc", componentCarrier - 1, RS_REPCAP_BUF_SIZE, repCap));

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_LTE_DOWNLINK_CUSTOM_SYNC_WEIGHT_SECOND_HALF_FRAME_IMAGINARY, imaginary),
            3, "Imaginary");

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_LTE_DOWNLINK_CUSTOM_SYNC_WEIGHT_SECOND_HALF_FRAME_REAL, real),
            4, "Real");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the settings that control the data capture.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR captureTime/Defines the capture time.
/// HIPAR overallFrameCount/Turns manual selection of the number of frames you want to analyze on
/// HIPAR overallFrameCount/and off.
/// HIPAR autoAccordingtoStandard/Turns automatic selection of the number of frames to analyze on and
/// HIPAR autoAccordingtoStandard/off.
/// HIPAR numberOfFramesToAnalyzer/Sets the number of frames you want to analyze.
/// HIPAR swapI_Q/Activates or deactivates the inverted I/Q modulation. If the I and Q
/// HIPAR swapI_Q/parts of the signal from the DUT are interchanged, the instrument can
/// HIPAR swapI_Q/do the same to compensate for it.
ViStatus _VI_FUNC rsspecan_ConfigureLTEDownlinkSignalCapture(ViSession instrSession,
                                                             ViReal64  captureTime,
                                                             ViBoolean overallFrameCount,
                                                             ViBoolean autoAccordingtoStandard,
                                                             ViInt32   numberOfFramesToAnalyzer,
                                                             ViBoolean swapI_Q)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_SWEEP_TIME, captureTime),
            2, "Capture Time");

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_LTE_DOWNLINK_OVERALL_FRAME_COUNT, overallFrameCount),
            3, "Overall Frame Count");

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_LTE_DOWNLINK_AUTO_COUNT_ACCORDING_TO_STANDARD, autoAccordingtoStandard),
            4, "Auto According to Standard");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_LTE_DOWNLINK_NUMBER_OF_FRAMES_TO_ANALYZE, numberOfFramesToAnalyzer),
            5, "Number Of Frames To Analyzer");

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_BB_SWAP_IQ, swapI_Q),
            6, "Swap I/Q");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function configures the maximum number of subframes that
/// HIFN the application analyzes and therefore
/// HIFN improves measurement speed.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR subframesPerFrame/Sets the maximum number of subframes that the application
/// HIPAR subframesPerFrame/analyzes and therefore
/// HIPAR subframesPerFrame/improves measurement speed.
ViStatus _VI_FUNC rsspecan_ConfigureLTEDownlinkNumberOfSubframesToAnalyze(
                            ViSession instrSession,
                            ViInt32   subframesPerFrame)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    if (subframesPerFrame == 0)
    {
        checkErr(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_LTE_DOWNLINK_ALL_SUBFRAMES_TO_ANALYZE, NULL));
    }
    else
    {
        viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_LTE_DOWNLINK_NUMBER_OF_SUBFRAMES_TO_ANALYZE, subframesPerFrame),
                2, "Subframes per Frame");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the settings that estimate various parameters
/// HIFN  during the measurement. They increase the quality of measurement
/// HIFN  results.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR boostingEstimation/Turns boosting estimation on and off.
/// HIPAR channelEstimation/Configures the channel estimation type for DL.
ViStatus _VI_FUNC rsspecan_ConfigureLTEDownlinkParameterEstimation(
                            ViSession instrSession,
                            ViBoolean boostingEstimation,
                            ViInt32   channelEstimation)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_LTE_DOWNLINK_BOOSTING_ESTIMATION, boostingEstimation),
            2, "Boosting Estimation");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_LTE_DOWNLINK_CHANNEL_ESTIMATION, channelEstimation),
            3, "Channel Estimation");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures phase and timing tracking.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR phaseTracking/Configures the phase tracking type for DL.
/// HIPAR timing/Activates or deactivates timing tracking for DL.
ViStatus _VI_FUNC rsspecan_ConfigureLTEDownlinkTracking(ViSession instrSession,
                                                        ViInt32   phaseTracking,
                                                        ViBoolean timing)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_LTE_DOWNLINK_PHASE_TRACKING, phaseTracking),
            2, "Phase Tracking");

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_LTE_DOWNLINK_TIMING, timing),
            3, "Timing");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures signal processing and the way the signal is measured.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR componentCarrier/This control selects the component carrier to be used.
/// HIPAR multicarrierFilter/Turns the suppression of interference of neighboring carriers on and
/// HIPAR multicarrierFilter/off (e.g. LTE, WCDMA, GSM etc).
/// HIPAR compensateCrosstalk/This control turns MIMO crosstalk compensation on and off.
/// HIPAR scramblingofCodedBits/Activates or deactivates scrambling of coded bits for DL.
/// HIPAR decodeAllChannels/Turns the decoding of all control channels on and off.
/// HIPAR evmCalculationMethod/Sets the EVM calculation method for DL.
/// HIPAR pdschReferenceData/Selects the type of reference data to calculate the EVM for the PDSCH.
ViStatus _VI_FUNC rsspecan_ConfigureLTEDownlinkDemodulation(ViSession instrSession,
                                                            ViInt32   componentCarrier,
                                                            ViBoolean multicarrierFilter,
                                                            ViBoolean compensateCrosstalk,
                                                            ViBoolean scramblingofCodedBits,
                                                            ViBoolean decodeAllChannels,
                                                            ViInt32   evmCalculationMethod,
                                                            ViInt32   pdschReferenceData)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, componentCarrier, 1, 5),
            2, "Component Carrier");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "cc%ld", componentCarrier);

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_LTE_DOWNLINK_MULTICARRIER_FILTER, multicarrierFilter),
            3, "Multicarrier Filter");

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, repCap, RSSPECAN_ATTR_LTE_DOWNLINK_COMPENSATE_CROSSTALK, compensateCrosstalk),
            4, "Compensate Crosstalk");

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_LTE_DOWNLINK_SCRAMBLING_OF_CODED_BITS, scramblingofCodedBits),
            5, "Scrambling of Coded Bits");

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_LTE_DOWNLINK_DECODE_ALL_CHANNELS, decodeAllChannels),
            6, "Decode All Channels");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_LTE_DOWNLINK_EVM_CALCULATION_METHOD, evmCalculationMethod),
            7, "EVM Calculation Method");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_LTE_DOWNLINK_PDSCH_REFERENCE_DATA, pdschReferenceData),
            8, "PDSCH Reference Data");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Turns suppression of 5G resources with a 30 kHz subcarrier spacing
/// HIFN  (dynamic spectrum sharing) on and off.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR suppressInterferer/Turns suppression of 5G resources with a 30 kHz subcarrier spacing
/// HIPAR suppressInterferer/(dynamic spectrum sharing) on and off.
ViStatus _VI_FUNC rsspecan_ConfigureLTEDownlinkSuppressInterfererForSynchronization(
                            ViSession instrSession,
                            ViBoolean suppressInterferer)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, NULL, RSSPECAN_ATTR_LTE_DOWNLINK_SUPPRESS_INTERFERER_FOR_SYNCHRONIZATION, suppressInterferer),
            2, "Suppress Interferer");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures Time Alignment Error measurements with carrier aggregation.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR carrierAggregationState/This control turns carrier aggregation for Time Alignment measurements
/// HIPAR carrierAggregationState/on and off. You can select the number of component carriers with
/// HIPAR carrierAggregationState/RSSPECAN_ATTR_LTE_DOWNLINK_COMPONENT_CARRIERS.
/// HIPAR numberofCarriers/This control selects the number of component carriers evaluated in the
/// HIPAR numberofCarriers/Time Alignment measurement.
ViStatus _VI_FUNC rsspecan_ConfigureLTEDownlinkCarrierAggregation(
                            ViSession instrSession,
                            ViBoolean carrierAggregationState,
                            ViInt32   numberofCarriers)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_LTE_DOWNLINK_CARRIER_AGGREGATION, carrierAggregationState),
            2, "Carrier Aggregation State");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_LTE_DOWNLINK_COMPONENT_CARRIERS, numberofCarriers),
            3, "Number of Carriers");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function selects the number of R&S FSW used in a Time Alignment
/// HIFN  Error measurement with carrier aggregation.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR numberOfDevices/This control selects the number of R&S FSW used in a Time Alignment
/// HIPAR numberOfDevices/Error measurement with carrier aggregation.
ViStatus _VI_FUNC rsspecan_ConfigureLTEDownlinkNumberOfDevices(
                            ViSession instrSession,
                            ViInt32   numberOfDevices)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_LTE_DOWNLINK_NUMBER_OF_DEVICES, numberOfDevices),
            2, "Number Of Devices");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures Power On / Off measurements.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR numberOfFrames/Defines the number of frames that are analyzed for On/Off Power
/// HIPAR numberOfFrames/measurements.
/// HIPAR noiseCorrection/Turns noise correction for On/Off Power measurements on and off.
ViStatus _VI_FUNC rsspecan_ConfigureLTEDownlinkPowerOnOffMeasurement(
                            ViSession instrSession,
                            ViInt32   numberOfFrames,
                            ViBoolean noiseCorrection)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_LTE_DOWNLINK_ON_OFF_NUMBER_OF_FRAMES, numberOfFrames),
            2, "Number Of Frames");

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_LTE_DOWNLINK_ON_OFF_NOISE_CORRECTION, noiseCorrection),
            3, "Noise Correction");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures frequency sweep measurements.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR assumedAdjChannelCarrier/Selects the ACP Assumed Adj. Channel Carrier in the general settings
/// HIPAR assumedAdjChannelCarrier/menu.
/// HIPAR homeAreaBasestationPower/Defines the aggregated maximum power for home base stations.
ViStatus _VI_FUNC rsspecan_ConfigureLTEDownlinkFrequencySweepMeasurements(
                            ViSession instrSession,
                            ViInt32   assumedAdjChannelCarrier,
                            ViInt32   category,
                            ViReal64  homeAreaBasestationPower)
{
    ViStatus  error = VI_SUCCESS;
    ViBoolean auto_state = VI_FALSE;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_LTE_DOWNLINK_ACP_ASSUMED_ADJ_CHANNEL_CARRIER, assumedAdjChannelCarrier),
            2, "Assumed Adj Channel Carrier");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_LTE_DOWNLINK_SEM_CHANNEL, category),
            3, "Category");

    switch (category)
    {
    case RSSPECAN_VAL_LTE_DOWNLINK_CATEGORY_HOME:
        viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_LTE_DOWNLINK_HOME_AREA_BASESTATION_POWER, homeAreaBasestationPower),
                4, "Home Area Basestation Power");
        break;
    case RSSPECAN_VAL_LTE_DOWNLINK_CATEGORY_MED:
        checkErr(rsspecan_GetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_LTE_DOWNLINK_HOME_AREA_BASESTATION_POWER_AUTO, &auto_state));
        if (!auto_state)
        {
            viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_LTE_DOWNLINK_HOME_AREA_BASESTATION_POWER, homeAreaBasestationPower),
                    4, "Home Area Basestation Power");
        }
        break;
    default:
        break;
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function adds a window to the display.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR stringSize/This control sets the allocated length of Returned Window Name string.
/// HIPAR referenceWindowName/String containing the number or name of the window the new window is
/// HIPAR referenceWindowName/inserted next to. In the default state, the name of the window is its
/// HIPAR referenceWindowName/index.
/// HIPAR direction/Direction the window is added relative to the window set with "Reference Window
/// HIPAR direction/Name".
/// HIPAR windowType/Type of result display you want to add.
/// HIPAR returnedWindowName/When adding a new window, the function returns its name (by
ViStatus _VI_FUNC rsspecan_LTEDownlinkAddWindow(ViSession instrSession,
                                                ViInt32   stringSize,
                                                ViString  referenceWindowName,
                                                ViInt32   direction,
                                                ViInt32   windowType,
                                                ViChar    returnedWindowName[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViChar   response[RS_MAX_MESSAGE_BUF_SIZE];
    ViChar*  p2direction;
    ViChar*  p2type;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &rsspecan_rngLayoutDirection, direction, &p2direction),
                4, "Direction");
    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &rsspecan_rngLTEDownlinkLayoutType, windowType, &p2type),
                5, "Window Type");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "LAY:ADD? '%s',%s,%s", referenceWindowName, p2direction, p2type);
    checkErr(RsCore_QueryViString(instrSession, cmd, response));

    if (returnedWindowName != NULL)
    {
        RsCore_TrimString(response, RS_VAL_TRIM_WHITESPACES_AND_ALL_QUOTES);
        RsCore_CopyToUserBufferAsciiData(returnedWindowName, stringSize, response);
    }

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function replaces a window with another one.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR windowName/String containing the number or name of the window you want to
/// HIPAR windowType/Type of result display you want to add.
ViStatus _VI_FUNC rsspecan_LTEDownlinkReplaceWindow(ViSession instrSession,
                                                    ViString  windowName,
                                                    ViInt32   windowType)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViChar*  p2type;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &rsspecan_rngLTEDownlinkLayoutType, windowType, &p2type),
                3, "Window Type");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "LAY:REPL '%s',%s", windowName, p2type);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects a tab in diagrams with multiple subwindows (or views). Note
/// HIFN  that selecting a tab does not actually select a subwindow. To select a
/// HIFN  subwindow, for example to query the results of a subwindow, use
/// HIFN  RSSPECAN_ATTR_DISPLAY_FOCUS_AREA.
/// HIFN
/// HIFN     Note(s):
/// HIFN     (1) This function is only available for R&S FSW.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR tab/Selects the tab.
ViStatus _VI_FUNC rsspecan_LTEDownlinkSelectTab(ViSession instrSession,
                                                ViInt32   window,
                                                ViInt32   tab)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_tab[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_LTE_DOWNLINK_TAB_SELECT,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap));
    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_LTE_DOWNLINK_TAB_SELECT,
            "Tab", tab - 1, RS_REPCAP_BUF_SIZE, repCap_tab));
    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_tab, NULL);

    checkErr(rsspecan_SetAttributeViString(instrSession, repCap, RSSPECAN_ATTR_LTE_DOWNLINK_TAB_SELECT, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function selects the measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR measurement/This control selects the measurement.
ViStatus _VI_FUNC rsspecan_ConfigureLTEDownlinkMeasurement(ViSession instrSession,
                                                           ViInt32   measurement)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_LTE_DOWNLINK_MEASUREMENT, measurement),
            2, "measurement");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function automatically detects several signal characteristics and selects the appropriate parameters in the application.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsspecan_ConfigureLTEDownlinkMeasurementSignalDetection(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_LTE_DOWNLINK_MEASUREMENT_SIGNAL_DETECTION, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function queries the current synchronization state.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR componentCarrier/This control selects the component carrier to be used.
/// HIPAR ofdmSymbolTiming/Returns the coarse symbol timing.
/// HIPAR pSYNCSynchronization/Returns the P-SYNC synchronization state.
/// HIPAR sSYNCSynchronization/Returns the S-SYNC synchronization state.
ViStatus _VI_FUNC rsspecan_QueryLTEDownlinkMeasurementSynchronizationState(
                            ViSession  instrSession,
                            ViInt32    componentCarrier,
                            ViBoolean* ofdmSymbolTiming,
                            ViBoolean* pSYNCSynchronization,
                            ViBoolean* sSYNCSynchronization)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, componentCarrier, 1, 5),
            2, "Component Carrier");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SENS:SYNC:CC%ld:STAT?", componentCarrier);
    checkErr(
        RsCore_QueryTupleViBoolean(instrSession, cmd, ofdmSymbolTiming, pSYNCSynchronization, sSYNCSynchronization, NULL
        ));

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries LTE Downlink measurement result summary.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR componentCarrier/This control selects the component carrier to be used.
/// HIPAR frameResult/Selects result type.
/// HIPAR measurementType/Selects measurement type.
/// HIPAR result/Returns selected result. Depending on the EVM unit selected in the
/// HIPAR result/"General Settings" menu the value will be either in dB or in percent.
ViStatus _VI_FUNC rsspecan_QueryLTEDownlinkMeasurementResultSummary(
                            ViSession instrSession,
                            ViInt32   componentCarrier,
                            ViInt32   frameResult,
                            ViInt32   measurementType,
                            ViReal64* result)
{
    ViStatus                                                 error = VI_SUCCESS;
    ViChar                                                   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViAttr                                                   attr[5][4] = {
                                                             {
                                                             RSSPECAN_ATTR_LTE_DOWNLINK_EVM_PDSCH_QPSK_RESULT_MIN,
    RSSPECAN_ATTR_LTE_DOWNLINK_EVM_PDSCH_QPSK_RESULT_MAX,    0,
                                                             RSSPECAN_ATTR_LTE_DOWNLINK_EVM_PDSCH_QPSK_RESULT
                                                             },
                                                             {
                                                             RSSPECAN_ATTR_LTE_DOWNLINK_EVM_PDSCH_16QAM_RESULT_MIN,
    RSSPECAN_ATTR_LTE_DOWNLINK_EVM_PDSCH_16QAM_RESULT_MAX,   0,
                                                             RSSPECAN_ATTR_LTE_DOWNLINK_EVM_PDSCH_16QAM_RESULT
                                                             },
                                                             {
                                                             RSSPECAN_ATTR_LTE_DOWNLINK_EVM_PDSCH_64QAM_RESULT_MIN,
    RSSPECAN_ATTR_LTE_DOWNLINK_EVM_PDSCH_64QAM_RESULT_MAX,   0,
                                                             RSSPECAN_ATTR_LTE_DOWNLINK_EVM_PDSCH_64QAM_RESULT
                                                             },
                                                             {
                                                             RSSPECAN_ATTR_LTE_DOWNLINK_EVM_PDSCH_256QAM_RESULT_MIN,
    RSSPECAN_ATTR_LTE_DOWNLINK_EVM_PDSCH_256QAM_RESULT_MAX,  0,
                                                             RSSPECAN_ATTR_LTE_DOWNLINK_EVM_PDSCH_256QAM_RESULT
                                                             },
                                                             {
                                                             RSSPECAN_ATTR_LTE_DOWNLINK_EVM_PDSCH_1024QAM_RESULT_MIN,
    RSSPECAN_ATTR_LTE_DOWNLINK_EVM_PDSCH_1024QAM_RESULT_MAX, 0,
                                                             RSSPECAN_ATTR_LTE_DOWNLINK_EVM_PDSCH_1024QAM_RESULT
                                                             }
                                                             };

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, componentCarrier, 1, 5),
            2, "Component Carrier");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, frameResult, RSSPECAN_VAL_LTE_DOWNLINK_QPSK, RSSPECAN_VAL_LTE_DOWNLINK_1024QAM),
            3, "Frame result");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, measurementType, RSSPECAN_VAL_MEASTYPE_MIN, RSSPECAN_VAL_MEASTYPE_AVER),
            4, "Measurement Type");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "cc%ld", componentCarrier);

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, repCap,
                    attr[frameResult][measurementType - 1],
                    result), 5, "Result");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the results of the EVM limit check of all PDSCH resource elements with a the selected modulation.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR componentCarrier/This control selects the component carrier to be used.
/// HIPAR frameResult/Selects result type.
/// HIPAR average/Returns the average value result of the EVM limit check of all PDSCH resource elements with the selected modulation.
/// HIPAR maximum/Returns the maximum value result of the EVM limit check of all PDSCH resource elements with the selected modulation.
ViStatus _VI_FUNC rsspecan_QueryLTEDownlinkMeasurementLimitCheckResult(
                            ViSession instrSession,
                            ViInt32   componentCarrier,
                            ViInt32   frameResult,
                            ViInt32*  average,
                            ViInt32*  maximum)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, frameResult, RSSPECAN_VAL_LTE_DOWNLINK_QPSK, RSSPECAN_VAL_LTE_DOWNLINK_1024QAM),
            3, "Frame Result");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "cc%ld", componentCarrier);

    switch (frameResult)
    {
    case RSSPECAN_VAL_LTE_DOWNLINK_QPSK:

        viCheckParm(rsspecan_GetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_LTE_DOWNLINK_EVM_PDSCH_QPSK_LIMIT_CHECK_RESULT_AVERAGE, average),
                4, "Average");

        viCheckParm(rsspecan_GetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_LTE_DOWNLINK_EVM_PDSCH_QPSK_LIMIT_CHECK_RESULT_MAX, maximum),
                5, "Maximum");
        break;

    case RSSPECAN_VAL_LTE_DOWNLINK_16QAM:

        viCheckParm(rsspecan_GetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_LTE_DOWNLINK_EVM_PDSCH_16QAM_LIMIT_CHECK_RESULT_AVERAGE, average),
                4, "Average");

        viCheckParm(rsspecan_GetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_LTE_DOWNLINK_EVM_PDSCH_16QAM_LIMIT_CHECK_RESULT_MAX, maximum),
                5, "Maximum");
        break;

    case RSSPECAN_VAL_LTE_DOWNLINK_64QAM:

        viCheckParm(rsspecan_GetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_LTE_DOWNLINK_EVM_PDSCH_64QAM_LIMIT_CHECK_RESULT_AVERAGE, average),
                4, "Average");

        viCheckParm(rsspecan_GetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_LTE_DOWNLINK_EVM_PDSCH_64QAM_LIMIT_CHECK_RESULT_MAX, maximum),
                5, "Maximum");
        break;

    case RSSPECAN_VAL_LTE_DOWNLINK_1024QAM:

        viCheckParm(rsspecan_GetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_LTE_DOWNLINK_EVM_PDSCH_1024QAM_LIMIT_CHECK_RESULT_AVERAGE, average),
                4, "Average");

        viCheckParm(rsspecan_GetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_LTE_DOWNLINK_EVM_PDSCH_1024QAM_LIMIT_CHECK_RESULT_MAX, maximum),
                5, "Maximum");
        break;

    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, frameResult), 3, "Frame Result");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries LTE Downlink measurement EVM All result.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR componentCarrier/This control selects the component carrier to be used.
/// HIPAR measurementType/Selects measurement type.
/// HIPAR result/Returns selected result. Depending on the EVM unit selected in the
/// HIPAR result/"General Settings" menu the value will be either in dB or in percent.
ViStatus _VI_FUNC rsspecan_QueryLTEDownlinkMeasurementEVMAll(ViSession instrSession,
                                                             ViInt32   componentCarrier,
                                                             ViInt32   measurementType,
                                                             ViReal64* result)
{
    ViStatus                                       error = VI_SUCCESS;
    ViChar                                         repCap[RS_REPCAP_BUF_SIZE] = "";
    ViAttr                                         attr[4] = {
                                                   RSSPECAN_ATTR_LTE_DOWNLINK_EVM_ALL_RESULT_MIN,
    RSSPECAN_ATTR_LTE_DOWNLINK_EVM_ALL_RESULT_MAX, 0,
                                                   RSSPECAN_ATTR_LTE_DOWNLINK_EVM_ALL_RESULT_AVERAGE
                                                   };

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, componentCarrier, 1, 5),
            2, "Component Carrier");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, measurementType, RSSPECAN_VAL_MEASTYPE_MIN, RSSPECAN_VAL_MEASTYPE_AVER),
            3, "Measurement Type");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "cc%ld", componentCarrier);

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, repCap,
                    attr[measurementType - 1],
                    result), 4, "Result");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the "EVM Physical Channel" value from the result summary list.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR componentCarrier/This control selects the component carrier to be used.
/// HIPAR measurementType/Selects measurement type.
/// HIPAR result/Returns selected result. Depending on the EVM unit selected in the
/// HIPAR result/"General Settings" menu the value will be either in dB or in percent.
ViStatus _VI_FUNC rsspecan_QueryLTEDownlinkMeasurementEVMPhysicalChannel(
                            ViSession instrSession,
                            ViInt32   componentCarrier,
                            ViInt32   measurementType,
                            ViReal64* result)
{
    ViStatus                                                    error = VI_SUCCESS;
    ViChar                                                      repCap[RS_REPCAP_BUF_SIZE] = "";
    ViAttr                                                      attr[4] = {
                                                                RSSPECAN_ATTR_LTE_DOWNLINK_EVM_PHYSICAL_CHANNEL_RESULT_MIN,
    RSSPECAN_ATTR_LTE_DOWNLINK_EVM_PHYSICAL_CHANNEL_RESULT_MAX, 0,
                                                                RSSPECAN_ATTR_LTE_DOWNLINK_EVM_PHYSICAL_CHANNEL_RESULT_AVERAGE
                                                                };

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, componentCarrier, 1, 5),
            2, "Component Carrier");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, measurementType, RSSPECAN_VAL_MEASTYPE_MIN, RSSPECAN_VAL_MEASTYPE_AVER),
            3, "Measurement Type");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "cc%ld", componentCarrier);

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, repCap,
                    attr[measurementType - 1],
                    result), 4, "Result");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the results of the EVM limit check of all physical channel resource elements.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR componentCarrier/This control selects the component carrier to be used.
/// HIPAR average/Returns the average value result of the EVM limit check of all physical channel resource elements.
/// HIPAR maximum/Returns the maximum value result of the EVM limit check of all physical channel resource elements.
ViStatus _VI_FUNC rsspecan_QueryLTEDownlinkMeasurementEVMPhysicalChannelLimitCheck(
                            ViSession instrSession,
                            ViInt32   componentCarrier,
                            ViInt32*  average,
                            ViInt32*  maximum)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, componentCarrier, 1, RSSPECAN_VAL_LTE_DL_CC_REPCAP_MAX),
            2, "Component Carrier");
    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_LTE_DOWNLINK_EVM_PHYSICAL_CHANNEL_LIMIT_CHECK_RESULT_AVERAGE,
            "cc", componentCarrier - 1, RS_REPCAP_BUF_SIZE, repCap));

    viCheckParm(rsspecan_GetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_LTE_DOWNLINK_EVM_PHYSICAL_CHANNEL_LIMIT_CHECK_RESULT_AVERAGE, average),
            3, "Average");

    viCheckParm(rsspecan_GetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_LTE_DOWNLINK_EVM_PHYSICAL_CHANNEL_LIMIT_CHECK_RESULT_MAX, maximum),
            4, "Maximum");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the "EVM Physical Signal" value from the result summary list.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR componentCarrier/This control selects the component carrier to be used.
/// HIPAR measurementType/Selects measurement type.
/// HIPAR result/Returns selected result. Depending on the EVM unit selected in the
/// HIPAR result/"General Settings" menu the value will be either in dB or in percent.
ViStatus _VI_FUNC rsspecan_QueryLTEDownlinkMeasurementEVMPhysicalSignal(
                            ViSession instrSession,
                            ViInt32   componentCarrier,
                            ViInt32   measurementType,
                            ViReal64* result)
{
    ViStatus                                                   error = VI_SUCCESS;
    ViChar                                                     repCap[RS_REPCAP_BUF_SIZE] = "";
    ViAttr                                                     attr[4] = {
                                                               RSSPECAN_ATTR_LTE_DOWNLINK_EVM_PHYSICAL_SIGNAL_RESULT_MIN,
    RSSPECAN_ATTR_LTE_DOWNLINK_EVM_PHYSICAL_SIGNAL_RESULT_MAX, 0,
                                                               RSSPECAN_ATTR_LTE_DOWNLINK_EVM_PHYSICAL_SIGNAL_RESULT_AVERAGE
                                                               };

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, componentCarrier, 1, 5),
            2, "Component Carrier");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, measurementType, RSSPECAN_VAL_MEASTYPE_MIN, RSSPECAN_VAL_MEASTYPE_AVER),
            3, "Measurement Type");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "cc%ld", componentCarrier);

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, repCap,
                    attr[measurementType - 1],
                    result), 4, "Result");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the results of the EVM limit check of all physical signal resource elements.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR componentCarrier/This control selects the component carrier to be used.
/// HIPAR average/Returns the average result of the EVM limit check of all physical signal resource elements.
/// HIPAR maximum/Returns the maximum result of the EVM limit check of all physical signal resource elements.
ViStatus _VI_FUNC rsspecan_QueryLTEDownlinkMeasurementEVMPhysicalSignalLimitCheck(
                            ViSession instrSession,
                            ViInt32   componentCarrier,
                            ViInt32*  average,
                            ViInt32*  maximum)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, componentCarrier, 1, 5),
            2, "Component Carrier");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "cc%ld", componentCarrier);

    viCheckParm(rsspecan_GetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_LTE_DOWNLINK_EVM_PHYSICAL_SIGNAL_LIMIT_CHECK_RESULT_AVERAGE, average),
            3, "Average");

    viCheckParm(rsspecan_GetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_LTE_DOWNLINK_EVM_PHYSICAL_SIGNAL_LIMIT_CHECK_RESULT_MAX, maximum),
            4, "Maximum");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries "Frequency Error" value from the result summary list in Hz.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR componentCarrier/This control selects the component carrier to be used.
/// HIPAR measurementType/    Selects measurement type.
/// HIPAR result/Returns selected result.
ViStatus _VI_FUNC rsspecan_QueryLTEDownlinkMeasurementFrequencyError(
                            ViSession instrSession,
                            ViInt32   componentCarrier,
                            ViInt32   measurementType,
                            ViReal64* result)
{
    ViStatus                                               error = VI_SUCCESS;
    ViChar                                                 repCap[RS_REPCAP_BUF_SIZE] = "";
    ViAttr                                                 attr[4] = {
                                                           RSSPECAN_ATTR_LTE_DOWNLINK_FREQUENCY_ERROR_RESULT_MIN,
    RSSPECAN_ATTR_LTE_DOWNLINK_FREQUENCY_ERROR_RESULT_MAX, 0,
                                                           RSSPECAN_ATTR_LTE_DOWNLINK_FREQUENCY_ERROR_RESULT_AVERAGE
                                                           };

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, componentCarrier, 1, 5),
            2, "Component Carrier");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, measurementType, RSSPECAN_VAL_MEASTYPE_MIN, RSSPECAN_VAL_MEASTYPE_AVER),
            3, "Measurement Type");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "cc%ld", componentCarrier);

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, repCap,
                    attr[measurementType - 1],
                    result), 4, "Result");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the result of the frequency error limit check.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR componentCarrier/This control selects the component carrier to be used.
/// HIPAR average/Returns the average result of the frequency error limit check.
/// HIPAR maximum/Returns the maximum result of the frequency error limit check.
ViStatus _VI_FUNC rsspecan_QueryLTEDownlinkMeasurementFrequencyErrorLimitCheck(
                            ViSession instrSession,
                            ViInt32   componentCarrier,
                            ViInt32*  average,
                            ViInt32*  maximum)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, componentCarrier, 1, 5),
            2, "Component Carrier");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "cc%ld", componentCarrier);

    viCheckParm(rsspecan_GetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_LTE_DOWNLINK_FREQUENCY_ERROR_LIMIT_CHECK_RESULT_AVERAGE, average),
            3, "Average");

    viCheckParm(rsspecan_GetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_LTE_DOWNLINK_FREQUENCY_ERROR_LIMIT_CHECK_RESULT_MAX, maximum),
            4, "Maximum");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries "Sampling Error" value from the result summary list in ppm.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR componentCarrier/This control selects the component carrier to be used.
/// HIPAR measurementType/Selects measurement type.
/// HIPAR result/Returns selected result.
ViStatus _VI_FUNC rsspecan_QueryLTEDownlinkMeasurementSamplingError(
                            ViSession instrSession,
                            ViInt32   componentCarrier,
                            ViInt32   measurementType,
                            ViReal64* result)
{
    ViStatus                                              error = VI_SUCCESS;
    ViChar                                                repCap[RS_REPCAP_BUF_SIZE] = "";
    ViAttr                                                attr[4] = {
                                                          RSSPECAN_ATTR_LTE_DOWNLINK_SAMPLING_ERROR_RESULT_MIN,
    RSSPECAN_ATTR_LTE_DOWNLINK_SAMPLING_ERROR_RESULT_MAX, 0,
                                                          RSSPECAN_ATTR_LTE_DOWNLINK_SAMPLING_ERROR_RESULT_AVERAGE
                                                          };

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, componentCarrier, 1, 5),
            2, "Component Carrier");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, measurementType, RSSPECAN_VAL_MEASTYPE_MIN, RSSPECAN_VAL_MEASTYPE_AVER),
            3, "Measurement Type");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "cc%ld", componentCarrier);

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, repCap,
                    attr[measurementType - 1],
                    result), 4, "Result");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the results of the sampling error limit check.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR componentCarrier/This control selects the component carrier to be used.
/// HIPAR average/Returns the average result of the sampling error limit check.
/// HIPAR maximum/Returns the maximum result of the sampling error limit check.
ViStatus _VI_FUNC rsspecan_QueryLTEDownlinkMeasurementSamplingErrorLimitCheck(
                            ViSession instrSession,
                            ViInt32   componentCarrier,
                            ViInt32*  average,
                            ViInt32*  maximum)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, componentCarrier, 1, 5),
            2, "Component Carrier");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "cc%ld", componentCarrier);

    viCheckParm(rsspecan_GetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_LTE_DOWNLINK_SAMPLING_ERROR_LIMIT_CHECK_RESULT_AVERAGE, average),
            3, "Average");

    viCheckParm(rsspecan_GetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_LTE_DOWNLINK_SAMPLING_ERROR_LIMIT_CHECK_RESULT_MAX, maximum),
            4, "Maximum");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Returns the cyclic prefix type detected by the DSP kernel. If no valid
/// HIFN  value has been detected yet the attribute will return -1.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR componentCarrier/This control sets the component carrier.
/// HIPAR cyclicPrefix/Returns the cyclic prefix type detected by the DSP kernel. If no valid
/// HIPAR cyclicPrefix/value has been detected yet the attribute will return -1.
ViStatus _VI_FUNC rsspecan_QueryLTEDownlinkMeasurementCyclicPrefix(
                            ViSession instrSession,
                            ViInt32   componentCarrier,
                            ViInt32*  cyclicPrefix)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, componentCarrier, 1, RSSPECAN_VAL_LTE_DL_CC_REPCAP_MAX),
            2, "Component Carrier");
    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_LTE_DOWNLINK_DETECTED_CYCLIC_PREFIX,
            "cc", componentCarrier - 1, RS_REPCAP_BUF_SIZE, repCap));

    viCheckParm(rsspecan_GetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_LTE_DOWNLINK_DETECTED_CYCLIC_PREFIX, cyclicPrefix),
            3, "Cyclic Prefix");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Returns the IQ offset from the result summary list in dB.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR componentCarrier/This control selects the component carrier to be used.
/// HIPAR measurementType/Selects measurement type.
/// HIPAR result/Returns selected result.
ViStatus _VI_FUNC rsspecan_QueryLTEDownlinkMeasurementIQOffset(
                            ViSession instrSession,
                            ViInt32   componentCarrier,
                            ViInt32   measurementType,
                            ViReal64* result)
{
    ViStatus                                         error = VI_SUCCESS;
    ViChar                                           repCap[RS_REPCAP_BUF_SIZE] = "";
    ViAttr                                           attr[4] = {
                                                     RSSPECAN_ATTR_LTE_DOWNLINK_IQ_OFFSET_RESULT_MIN,
    RSSPECAN_ATTR_LTE_DOWNLINK_IQ_OFFSET_RESULT_MAX, 0,
                                                     RSSPECAN_ATTR_LTE_DOWNLINK_IQ_OFFSET_RESULT_AVERAGE
                                                     };

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, componentCarrier, 1, 5),
            2, "Component Carrier");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, measurementType, RSSPECAN_VAL_MEASTYPE_MIN, RSSPECAN_VAL_MEASTYPE_AVER),
            3, "Measurement Type");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "cc%ld", componentCarrier);

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, repCap,
                    attr[measurementType - 1],
                    result), 4, "Result");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the result of the I/Q offset limit check.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR componentCarrier/This control selects the component carrier to be used.
/// HIPAR average/Returns the average result of the I/Q offset limit check.
/// HIPAR maximum/Returns the maximum result of the I/Q offset limit check.
ViStatus _VI_FUNC rsspecan_QueryLTEDownlinkMeasurementIQOffsetLimitCheck(
                            ViSession instrSession,
                            ViInt32   componentCarrier,
                            ViInt32*  average,
                            ViInt32*  maximum)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, componentCarrier, 1, 5),
            2, "Component Carrier");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "cc%ld", componentCarrier);

    viCheckParm(rsspecan_GetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_LTE_DOWNLINK_IQ_OFFSET_LIMIT_CHECK_RESULT_AVERAGE, average),
            3, "Average");

    viCheckParm(rsspecan_GetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_LTE_DOWNLINK_IQ_OFFSET_LIMIT_CHECK_RESULT_MAX, maximum),
            4, "Maximum");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Returns the IQ gain imbalance from the result summary list in dB.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR componentCarrier/This control selects the component carrier to be used.
/// HIPAR measurementType/Selects measurement type.
/// HIPAR result/Returns selected result.
ViStatus _VI_FUNC rsspecan_QueryLTEDownlinkMeasurementIQGainImbalance(
                            ViSession instrSession,
                            ViInt32   componentCarrier,
                            ViInt32   measurementType,
                            ViReal64* result)
{
    ViStatus                                                 error = VI_SUCCESS;
    ViChar                                                   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViAttr                                                   attr[4] = {
                                                             RSSPECAN_ATTR_LTE_DOWNLINK_IQ_GAIN_IMBALANCE_RESULT_MIN,
    RSSPECAN_ATTR_LTE_DOWNLINK_IQ_GAIN_IMBALANCE_RESULT_MAX, 0,
                                                             RSSPECAN_ATTR_LTE_DOWNLINK_IQ_GAIN_IMBALANCE_RESULT_AVERAGE
                                                             };

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, componentCarrier, 1, 5),
            2, "Component Carrier");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, measurementType, RSSPECAN_VAL_MEASTYPE_MIN, RSSPECAN_VAL_MEASTYPE_AVER),
            3, "Measurement Type");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "cc%ld", componentCarrier);

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, repCap,
                    attr[measurementType - 1],
                    result), 4, "Result");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the result of the gain imbalance limit check.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR componentCarrier/This control selects the component carrier to be used.
/// HIPAR average/Returns the average result of the gain imbalance limit check.
/// HIPAR maximum/Returns the maximum result of the gain imbalance limit check.
ViStatus _VI_FUNC rsspecan_QueryLTEDownlinkMeasurementIQGainImbalanceLimitCheck(
                            ViSession instrSession,
                            ViInt32   componentCarrier,
                            ViInt32*  average,
                            ViInt32*  maximum)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, componentCarrier, 1, 5),
            2, "Component Carrier");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "cc%ld", componentCarrier);

    viCheckParm(rsspecan_GetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_LTE_DOWNLINK_IQ_GAIN_IMBALANCE_LIMIT_CHECK_RESULT_AVERAGE, average),
            3, "Average");

    viCheckParm(rsspecan_GetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_LTE_DOWNLINK_IQ_GAIN_IMBALANCE_LIMIT_CHECK_RESULT_MAX, maximum),
            4, "Maximum");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Returns the IQ quadrature error from the result summary list in
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR componentCarrier/This control selects the component carrier to be used.
/// HIPAR measurementType/Selects measurement type.
/// HIPAR result/Returns selected result.
ViStatus _VI_FUNC rsspecan_QueryLTEDownlinkMeasurementIQQuadratureError(
                            ViSession instrSession,
                            ViInt32   componentCarrier,
                            ViInt32   measurementType,
                            ViReal64* result)
{
    ViStatus                                                   error = VI_SUCCESS;
    ViChar                                                     repCap[RS_REPCAP_BUF_SIZE] = "";
    ViAttr                                                     attr[4] = {
                                                               RSSPECAN_ATTR_LTE_DOWNLINK_IQ_QUADRATURE_ERROR_RESULT_MIN,
    RSSPECAN_ATTR_LTE_DOWNLINK_IQ_QUADRATURE_ERROR_RESULT_MAX, 0,
                                                               RSSPECAN_ATTR_LTE_DOWNLINK_IQ_QUADRATURE_ERROR_RESULT_AVERAGE
                                                               };

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, componentCarrier, 1, 5),
            2, "Component Carrier");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, measurementType, RSSPECAN_VAL_MEASTYPE_MIN, RSSPECAN_VAL_MEASTYPE_AVER),
            3, "Measurement Type");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "cc%ld", componentCarrier);

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, repCap,
                    attr[measurementType - 1],
                    result), 4, "Result");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the result of the quadrature error limit check.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR componentCarrier/This control selects the component carrier to be used.
/// HIPAR average/Returns the average result of the quadrature error limit check.
/// HIPAR maximum/Returns the maximum value result of the quadrature error limit check.
ViStatus _VI_FUNC rsspecan_QueryLTEDownlinkMeasurementIQQuadratureErrorLimitCheck(
                            ViSession instrSession,
                            ViInt32   componentCarrier,
                            ViInt32*  average,
                            ViInt32*  maximum)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, componentCarrier, 1, 5),
            2, "Component Carrier");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "cc%ld", componentCarrier);

    viCheckParm(rsspecan_GetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_LTE_DOWNLINK_IQ_QUADRATURE_ERROR_LIMIT_CHECK_RESULT_AVERAGE, average),
            3, "Average");

    viCheckParm(rsspecan_GetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_LTE_DOWNLINK_IQ_QUADRATURE_ERROR_LIMIT_CHECK_RESULT_MAX, maximum),
            4, "Maximum");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Returns the reference signal transmit power (RSTP) from the result
/// HIFN  list.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR componentCarrier/This control selects the component carrier to be used.
/// HIPAR measurementType/Selects measurement type.
/// HIPAR result/Returns the RSTP value from the result list as defined in 3GPP TS
/// HIPAR result/36.141.
ViStatus _VI_FUNC rsspecan_QueryLTEDownlinkMeasurementRSTP(ViSession instrSession,
                                                           ViInt32   componentCarrier,
                                                           ViInt32   measurementType,
                                                           ViReal64* result)
{
    ViStatus                                    error = VI_SUCCESS;
    ViChar                                      repCap[RS_REPCAP_BUF_SIZE] = "";
    ViAttr                                      attr[4] = {
                                                RSSPECAN_ATTR_LTE_DOWNLINK_RSTP_RESULT_MIN,
    RSSPECAN_ATTR_LTE_DOWNLINK_RSTP_RESULT_MAX, 0,
                                                RSSPECAN_ATTR_LTE_DOWNLINK_RSTP_RESULT_AVERAGE
                                                };

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, componentCarrier, 1, 5),
            2, "Component Carrier");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, measurementType, RSSPECAN_VAL_MEASTYPE_MIN, RSSPECAN_VAL_MEASTYPE_AVER),
            3, "Measurement Type");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "cc%ld", componentCarrier);

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, repCap,
                    attr[measurementType - 1],
                    result), 4, "Result");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Returns the OSTP value from the result list in dBm.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR componentCarrier/This control selects the component carrier to be used.
/// HIPAR measurementType/Selects measurement type.
/// HIPAR result/Returns the OSTP value from the result list in dBm.
ViStatus _VI_FUNC rsspecan_QueryLTEDownlinkMeasurementOSTP(ViSession instrSession,
                                                           ViInt32   componentCarrier,
                                                           ViInt32   measurementType,
                                                           ViReal64* result)
{
    ViStatus                                    error = VI_SUCCESS;
    ViChar                                      repCap[RS_REPCAP_BUF_SIZE] = "";
    ViAttr                                      attr[4] = {
                                                RSSPECAN_ATTR_LTE_DOWNLINK_OSTP_RESULT_MIN,
    RSSPECAN_ATTR_LTE_DOWNLINK_OSTP_RESULT_MAX, 0,
                                                RSSPECAN_ATTR_LTE_DOWNLINK_OSTP_RESULT_AVERAGE
                                                };

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, componentCarrier, 1, 5),
            2, "Component Carrier");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, measurementType, RSSPECAN_VAL_MEASTYPE_MIN, RSSPECAN_VAL_MEASTYPE_AVER),
            3, "Measurement Type");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "cc%ld", componentCarrier);

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, repCap,
                    attr[measurementType - 1],
                    result), 4, "Result");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Returns the "Frame Power" from the result summary list in dBm.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR componentCarrier/This control selects the component carrier to be used.
/// HIPAR measurementType/Selects measurement type.
/// HIPAR result/Returns selected result.
ViStatus _VI_FUNC rsspecan_QueryLTEDownlinkMeasurementFramePower(
                            ViSession instrSession,
                            ViInt32   componentCarrier,
                            ViInt32   measurementType,
                            ViReal64* result)
{
    ViStatus                                           error = VI_SUCCESS;
    ViChar                                             repCap[RS_REPCAP_BUF_SIZE] = "";
    ViAttr                                             attr[4] = {
                                                       RSSPECAN_ATTR_LTE_DOWNLINK_FRAME_POWER_RESULT_MIN,
    RSSPECAN_ATTR_LTE_DOWNLINK_FRAME_POWER_RESULT_MAX, 0,
                                                       RSSPECAN_ATTR_LTE_DOWNLINK_FRAME_POWER_RESULT_AVERAGE
                                                       };

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, componentCarrier, 1, 5),
            2, "Component Carrier");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, measurementType, RSSPECAN_VAL_MEASTYPE_MIN, RSSPECAN_VAL_MEASTYPE_AVER),
            3, "Measurement Type");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "cc%ld", componentCarrier);

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, repCap,
                    attr[measurementType - 1],
                    result), 4, "Result");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Returns the crest factor from the result summary list in dB.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR componentCarrier/This control selects the component carrier to be used.
/// HIPAR measurementType/Sets the type of returned value.
/// HIPAR result/Returns the crest factor from the result summary list in dB.
ViStatus _VI_FUNC rsspecan_QueryLTEDownlinkMeasurementCrestFactor(
                            ViSession instrSession,
                            ViInt32   componentCarrier,
                            ViInt32   measurementType,
                            ViReal64* result)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViAttr   attr[4] = {
    0,       0, 0,
             RSSPECAN_ATTR_LTE_DOWNLINK_CREST_FACTOR_RESULT_AVERAGE
             };

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, componentCarrier, 1, 5),
            2, "Component Carrier");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, measurementType, RSSPECAN_VAL_MEASTYPE_AVER, RSSPECAN_VAL_MEASTYPE_AVER),
            3, "Measurement Type");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "cc%ld", componentCarrier);

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, repCap,
                    attr[measurementType - 1],
                    result), 4, "Result");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the RSSI as shown in the result summary.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR componentCarrier/This control selects the component carrier to be used.
/// HIPAR measurementType/Selects measurement type.
/// HIPAR result/Queries the RSSI as shown in the result summary.
ViStatus _VI_FUNC rsspecan_QueryLTEDownlinkMeasurementRSSI(ViSession instrSession,
                                                           ViInt32   componentCarrier,
                                                           ViInt32   measurementType,
                                                           ViReal64* result)
{
    ViStatus                                    error = VI_SUCCESS;
    ViChar                                      repCap[RS_REPCAP_BUF_SIZE] = "";
    ViAttr                                      attr[4] = {
                                                RSSPECAN_ATTR_LTE_DOWNLINK_RSSI_RESULT_MIN,
    RSSPECAN_ATTR_LTE_DOWNLINK_RSSI_RESULT_MAX, 0,
                                                RSSPECAN_ATTR_LTE_DOWNLINK_RSSI_RESULT_AVERAGE
                                                };

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, componentCarrier, 1, 5),
            2, "Component Carrier");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, measurementType, RSSPECAN_VAL_MEASTYPE_MIN, RSSPECAN_VAL_MEASTYPE_AVER),
            3, "Measurement Type");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "cc%ld", componentCarrier);

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, repCap,
                    attr[measurementType - 1],
                    result), 4, "Result");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Returns the (sub)frame start offset as shown in the Capture Buffer
/// HIFN  result display.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR componentCarrier/This control selects the component carrier to be used.
/// HIPAR result/Returns the (sub)frame start offset as shown in the Capture Buffer
/// HIPAR result/result display.
ViStatus _VI_FUNC rsspecan_QueryLTEDownlinkMeasurementFrameStartOffset(
                            ViSession instrSession,
                            ViInt32   componentCarrier,
                            ViReal64* result)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, componentCarrier, 1, 5),
            2, "Component Carrier");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "cc%ld", componentCarrier);

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_LTE_DOWNLINK_TRIGGER_TO_FRAME_RESULT, result),
            3, "Result");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Returns the results for the PBCH if PBCH decoding (or CRC check) was
/// HIFN  successful.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR subframeNumber/Subframe number.
/// HIPAR numberofAntennas/The MIMO configuration of the DUT (1, 2 or 4 TX antennas)
/// HIPAR systemBandwidth/Transmission bandwidth
/// HIPAR frame/System frame number.
/// HIPAR phichDuration/Represents the PHICH duration. The range is 1, 2.
/// HIPAR phichResource/Represents the parameter Ng. The range is 1 to 4.
ViStatus _VI_FUNC rsspecan_ReadLTEDownlinkChannelDecoderResultsPBCH(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32*  subframeNumber,
                            ViInt32*  numberofAntennas,
                            ViReal64* systemBandwidth,
                            ViInt32*  frame,
                            ViInt32*  phichDuration,
                            ViInt32*  phichResource)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViReal64 data[6];

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K100|K102"));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "TRACE%ld? PBCH", window);
    checkErr(RsCore_QueryFloatArrayToUserBuffer(instrSession, cmd, 6, data, NULL));

    *subframeNumber = (ViInt32)data[0];
    *numberofAntennas = (ViInt32)data[1];
    *systemBandwidth = data[2];
    *frame = (ViInt32)data[3];
    *phichDuration = (ViInt32)data[4];
    *phichResource = (ViInt32)data[5];

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Returns the results for the PCFICH.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR arraySize/Number of array points requested.
/// HIPAR subframeNumber/Subframe number.
/// HIPAR numberofSymbols/Number of OFDM symbols that are used for PDCCH at the beginning of a
/// HIPAR numberofSymbols/subframe.
/// HIPAR noofValues/Returns the number of trace data values.
ViStatus _VI_FUNC rsspecan_ReadLTEDownlinkChannelDecoderResultsPCFIC(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   arraySize,
                            ViInt32   subframeNumber[],
                            ViInt32   numberofSymbols[],
                            ViInt32*  noofValues)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViAInt32 data = NULL;
    ViInt32  sourceArraySize;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K100|K102"));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "TRACE%ld? PCFICH", window);
    checkErr(RsCore_QueryIntegerArray(instrSession, cmd, &data, &sourceArraySize));
    checkErr(RsCore_DecimateViInt32ArrayToUserBuffers(data, sourceArraySize, arraySize, noofValues, subframeNumber, numberofSymbols, NULL, NULL, NULL, NULL));

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    if (data) free(data);
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Returns the results for the PHICH.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR arraySize/Number of array points requested.
/// HIPAR subframeNumber/Subframe number.
/// HIPAR ack_NACK/Ack/Nack. The range is -1 to 1.
/// HIPAR relativePower/Relative power.
/// HIPAR noofValues/Returns the number of trace data values.
ViStatus _VI_FUNC rsspecan_ReadLTEDownlinkChannelDecoderResultsPHICH(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   arraySize,
                            ViInt32   subframeNumber[],
                            ViInt32   ack_NACK[],
                            ViReal64  relativePower[],
                            ViInt32*  noofValues)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViChar*  pbuffer = NULL;
    ViChar*  pstring_value;
    ViInt32  i = 0;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K100|K102"));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "TRACE%ld? PHICH", window);
    checkErr(RsCore_QueryViStringUnknownLength(instrSession, cmd, &pbuffer));

    pstring_value = strtok(pbuffer, ",");

    while (pstring_value != NULL && i < arraySize)
    {
        subframeNumber[i] = RsCore_Convert_String_To_ViInt32(pstring_value);

        pstring_value = strtok(NULL, ",\n");

        ack_NACK[i] = RsCore_Convert_String_To_ViInt32(pstring_value);

        pstring_value = strtok(NULL, ",\n");

        relativePower[i++] = RsCore_Convert_String_To_ViReal64(pstring_value);

        pstring_value = strtok(NULL, ",\n");
    }

    if (*noofValues) *noofValues = i;

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    if (pbuffer) free(pbuffer);
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Returns the results for the PDCCH.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR arraySize/Number of array points requested.
/// HIPAR bitstreamSize/Size of bitstream array.
/// HIPAR subframeNumber/Subframe number.
/// HIPAR rnti/RNTI.
/// HIPAR dciFormat/Represents the DCi format. The value is a number in the range 0 to 103.
/// HIPAR pdcchFormat/PDCCH format, valid range -1 to 3.
/// HIPAR cceOffset/CCE Offset
/// HIPAR numberofTransmittedBits/Number of Transmitted Bits
/// HIPAR bitStreams/Bit stream of binary numbers. Overall number of bytes required for
/// HIPAR bitStreams/this array is defined by multiplication of number of datablocks and its
/// HIPAR bitStreams/corresponding sizes.
/// HIPAR noofValues/Returns the number of trace data values.
ViStatus _VI_FUNC rsspecan_ReadLTEDownlinkChannelDecoderResultsPDCCH(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   arraySize,
                            ViInt32   bitstreamSize,
                            ViInt32   subframeNumber[],
                            ViInt32   rnti[],
                            ViInt32   dciFormat[],
                            ViInt32   pdcchFormat[],
                            ViInt32   cceOffset[],
                            ViInt32   numberofTransmittedBits[],
                            ViChar    bitStreams[],
                            ViInt32*  noofValues)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViChar*  pBuffer = NULL;
    ViChar*  p2Val;
    ViInt32  nType = 0;
    ViInt32  nIndexSymbol = 0, nCurrentSymbol = 0;
    ViInt32  nSymbols = 0;
    ViInt32  nSymbolValue;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K100|K102"));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "TRACE%ld? PDCCH", window);
    checkErr(RsCore_QueryViStringUnknownLength(instrSession, cmd, &pBuffer));

    *noofValues = 0;
    p2Val = strtok(pBuffer, ",");

    nType = 0;

    while (p2Val)
    {
        if (*noofValues >= arraySize)
        {
            viCheckParm(RsCore_InvalidViInt32Value(instrSession, arraySize),
                        2, "Array Size");
        }

        switch (nType)
        {
        case 0:
            subframeNumber[*noofValues] = RsCore_Convert_String_To_ViInt32(p2Val);
            nType++;
            break;
        case 1:
            rnti[*noofValues] = RsCore_Convert_String_To_ViInt32(p2Val);
            nType++;
            break;
        case 2:
            dciFormat[*noofValues] = RsCore_Convert_String_To_ViInt32(p2Val);
            nType++;
            break;
        case 3:
            pdcchFormat[*noofValues] = RsCore_Convert_String_To_ViInt32(p2Val);
            nType++;
            break;
        case 4:
            cceOffset[*noofValues] = RsCore_Convert_String_To_ViInt32(p2Val);
            nType++;
            break;
        case 5:
            nSymbols = RsCore_Convert_String_To_ViInt32(p2Val);
            nCurrentSymbol = 0;
            numberofTransmittedBits[*noofValues] = nSymbols;
            nType++;
            break;
        case 6: // stream
            (void)sscanf(p2Val, "%lx", &nSymbolValue);
            bitStreams[nIndexSymbol + nCurrentSymbol] = (ViChar)nSymbolValue;

            nCurrentSymbol++;
            if (nCurrentSymbol >= nSymbols)
            {
                nIndexSymbol += nSymbols;
                nType = 0;
                (*noofValues)++;
            }

            if (nIndexSymbol + nCurrentSymbol >= bitstreamSize)
            {
                viCheckParm(RsCore_InvalidViInt32Value(instrSession, bitstreamSize),
                            3, "Array Size of BitStream");
            }

            break;
        }

        p2Val = strtok(NULL, ",\n");
    }

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    if (pBuffer)
        free(pBuffer);

    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Returns the results for the PDSCH.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR arraySize/Number of array points requested.
/// HIPAR bitstreamSize/Size of bitstream array.
/// HIPAR subframeNumber/Subframe number.
/// HIPAR allocationID/Allocation ID where:
/// HIPAR codeword/Number of RB.
/// HIPAR numberofTransmittedBits/Number of Transmitted Bits
/// HIPAR bitStreams/Bit stream of binary numbers. Overall number of bytes required for
/// HIPAR bitStreams/this array is defined by multiplication of number of datablocks and its
/// HIPAR bitStreams/corresponding sizes.
/// HIPAR noofValues/Returns the number of trace data values.
ViStatus _VI_FUNC rsspecan_ReadLTEDownlinkChannelDecoderResultsPDSCH(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   arraySize,
                            ViInt32   bitstreamSize,
                            ViInt32   subframeNumber[],
                            ViInt32   allocationID[],
                            ViInt32   codeword[],
                            ViInt32   numberofTransmittedBits[],
                            ViChar    bitStreams[],
                            ViInt32*  noofValues)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViChar*  pBuffer = NULL;
    ViChar*  pValue;
    ViInt32  nType = 0;
    ViInt32  nIndexSymbol = 0, nCurrentSymbol = 0;
    ViInt32  nSymbols = 0;
    ViInt32  nSymbolValue;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K100|K102"));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "TRACE%ld? PDSCH", window);
    checkErr(RsCore_QueryViStringUnknownLength(instrSession, cmd, &pBuffer));

    *noofValues = 0;
    pValue = strtok(pBuffer, ",");

    nType = 0;

    while (pValue)
    {
        if (*noofValues >= arraySize)
        {
            viCheckParm(RsCore_InvalidViInt32Value(instrSession, arraySize),
                        2, "Array Size");
        }

        switch (nType)
        {
        case 0:
            subframeNumber[*noofValues] = RsCore_Convert_String_To_ViInt32(pValue);
            nType++;
            break;
        case 1:
            allocationID[*noofValues] = RsCore_Convert_String_To_ViInt32(pValue);
            nType++;
            break;
        case 2:
            codeword[*noofValues] = RsCore_Convert_String_To_ViInt32(pValue);
            nType++;
            break;
        case 3:
            nSymbols = RsCore_Convert_String_To_ViInt32(pValue);
            nCurrentSymbol = 0;
            numberofTransmittedBits[*noofValues] = nSymbols;
            nType++;
            break;
        case 4: // stream
            if (nSymbols == (ViInt32)RS_VAL_NAN_VI_INT32)
            {
                (*noofValues)++;
                nType = 0;
                nSymbols = 0;
            }
            else
            {
                (void)sscanf(pValue, "%lx", &nSymbolValue);
                bitStreams[nIndexSymbol + nCurrentSymbol] = (ViChar)nSymbolValue;

                nCurrentSymbol++;
                if (nCurrentSymbol >= nSymbols)
                {
                    nIndexSymbol += nSymbols;
                    nType = 0;
                    (*noofValues)++;
                }

                if (nIndexSymbol + nCurrentSymbol >= bitstreamSize)
                {
                    viCheckParm(RsCore_InvalidViInt32Value(instrSession, bitstreamSize),
                                3, "Array Size of BitStream");
                }
            }
            break;
        }

        if (nSymbols != (ViInt32)RS_VAL_NAN_VI_INT32)
            pValue = strtok(NULL, ",\n");
    }
    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    if (pBuffer)
        free(pBuffer);

    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects the subframe to be analyzed.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR componentCarrier/This control selects the component carrier to be used.
/// HIPAR subframe/Selects the subframe to be analyzed.
ViStatus _VI_FUNC rsspecan_ConfigureLTEDownlinkMeasurementSubframe(
                            ViSession instrSession,
                            ViInt32   componentCarrier,
                            ViInt32   subframe)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, componentCarrier, 1, 5),
            2, "Component Carrier");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "cc%ld", componentCarrier);

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_LTE_DOWNLINK_SUBFRAME_SELECTION, subframe),
            3, "Subframe");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects all the subframe to be analyzed.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR componentCarrier/This control selects the component carrier to be used.
ViStatus _VI_FUNC rsspecan_ConfigureLTEDownlinkMeasurementSubframeAll(
                            ViSession instrSession,
                            ViInt32   componentCarrier)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, componentCarrier, 1, RSSPECAN_VAL_LTE_DL_CC_REPCAP_MAX),
            2, "Component Carrier");
    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_LTE_DOWNLINK_SUBFRAME_SELECTION_ALL,
            "cc", componentCarrier - 1, RS_REPCAP_BUF_SIZE, repCap));

    checkErr(rsspecan_SetAttributeViString(instrSession, repCap, RSSPECAN_ATTR_LTE_DOWNLINK_SUBFRAME_SELECTION_ALL, ""));

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function filters the displayed results in the constellation diagram by a particular type of modulation.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR componentCarrier/This control selects the component carrier to be used.
/// HIPAR modulation/Shows the results for a particular modulation type.
ViStatus _VI_FUNC rsspecan_ConfigureLTEDownlinkMeasurementConstelllationModulation(
                            ViSession instrSession,
                            ViInt32   componentCarrier,
                            ViInt32   modulation)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, componentCarrier, 1, 5),
            2, "Component Carrier");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "cc%ld", componentCarrier);

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_LTE_DOWNLINK_CONSTELLATION_MODULATION, modulation),
            3, "Modulation");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Shows the results for all modulation types.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR componentCarrier/This control selects the component carrier to be used.
ViStatus _VI_FUNC rsspecan_ConfigureLTEDownlinkMeasurementConstelllationModulationAll(
                            ViSession instrSession,
                            ViInt32   componentCarrier)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, componentCarrier, 1, 5),
            2, "Component Carrier");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "cc%ld", componentCarrier);

    checkErr(rsspecan_SetAttributeViString(instrSession, repCap, RSSPECAN_ATTR_LTE_DOWNLINK_CONSTELLATION_MODULATION_ALL, ""));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function filters the displayed results in the constellation diagram by a particular type of allocation.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR componentCarrier/This control selects the component carrier to be used.
/// HIPAR allocation/Sets the displayed results in the constellation diagram by a particular type of allocation.
ViStatus _VI_FUNC rsspecan_ConfigureLTEDownlinkMeasurementConstelllationAllocation(
                            ViSession instrSession,
                            ViInt32   componentCarrier,
                            ViInt32   allocation)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, componentCarrier, 1, 5),
            2, "Component Carrier");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "cc%ld", componentCarrier);

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_LTE_DOWNLINK_CONSTELLATION_ALLOCATION, allocation),
            3, "Allocation");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function shows all the results for all allocation.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR componentCarrier/This control selects the component carrier to be used.
ViStatus _VI_FUNC rsspecan_ConfigureLTEDownlinkMeasurementConstelllationAllocationAll(
                            ViSession instrSession,
                            ViInt32   componentCarrier)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, componentCarrier, 1, 5),
            2, "Component Carrier");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "cc%ld", componentCarrier);

    checkErr(rsspecan_SetAttributeViString(instrSession, repCap, RSSPECAN_ATTR_LTE_DOWNLINK_CONSTELLATION_ALLOCATION_ALL, ""));

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries LTE Downlink measurement time alignment error.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR componentCarrier/This control selects the component carrier to be used.
/// HIPAR symbol_OFDM/Sets the results for a particular OFDM symbol.
ViStatus _VI_FUNC rsspecan_ConfigureLTEDownlinkMeasurementConstelllationOFDMSymbol(
                            ViSession instrSession,
                            ViInt32   componentCarrier,
                            ViInt32   symbol_OFDM)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, componentCarrier, 1, 5),
            2, "Component Carrier");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "cc%ld", componentCarrier);

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_LTE_DOWNLINK_CONSTELLATION_SYMBOL, symbol_OFDM),
            3, "Symbol OFDM");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Filters all the displayed results in the constellation diagram by a particular OFDM symbol.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR componentCarrier/This control selects the component carrier to be used.
ViStatus _VI_FUNC rsspecan_ConfigureLTEDownlinkMeasurementConstelllationOFDMSymbolAll(
                            ViSession instrSession,
                            ViInt32   componentCarrier)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, componentCarrier, 1, 5),
            2, "Component Carrier");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "cc%ld", componentCarrier);

    checkErr(rsspecan_SetAttributeViString(instrSession, repCap, RSSPECAN_ATTR_LTE_DOWNLINK_CONSTELLATION_SYMBOL_ALL, ""));

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Filters the displayed results in the constellation diagram by a particular subcarrier.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR componentCarrier/This control selects the component carrier to be used.
/// HIPAR carrier/Sets the displayed results in the constellation diagram by a particular subcarrier.
ViStatus _VI_FUNC rsspecan_ConfigureLTEDownlinkMeasurementConstelllationCarrier(
                            ViSession instrSession,
                            ViInt32   componentCarrier,
                            ViInt32   carrier)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, componentCarrier, 1, 5),
            2, "Component Carrier");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "cc%ld", componentCarrier);

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_LTE_DOWNLINK_CONSTELLATION_CARRIER, carrier),
            3, "Carrier");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Shows the results for all subcarriers.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR componentCarrier/This control selects the component carrier to be used.
ViStatus _VI_FUNC rsspecan_ConfigureLTEDownlinkMeasurementConstelllationCarrierAll(
                            ViSession instrSession,
                            ViInt32   componentCarrier)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, componentCarrier, 1, 5),
            2, "Component Carrier");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "cc%ld", componentCarrier);

    checkErr(rsspecan_SetAttributeViString(instrSession, repCap, RSSPECAN_ATTR_LTE_DOWNLINK_CONSTELLATION_CARRIER_ALL, ""));

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects the data source of the constellation diagram for measurements on downlink signals.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR componentCarrier/This control selects the component carrier to be used.
/// HIPAR location/Selects the data source of the constellation diagram for measurements on downlink signals.
ViStatus _VI_FUNC rsspecan_ConfigureLTEDownlinkMeasurementConstelllationLocation(
                            ViSession instrSession,
                            ViInt32   componentCarrier,
                            ViInt32   location)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, componentCarrier, 1, 5),
            2, "Component Carrier");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "cc%ld", componentCarrier);

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_LTE_DOWNLINK_CONSTELLATION_LOCATION, location),
            3, "Location");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects the antenna port for beamforming measurements.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR componentCarrier/This control selects the component carrier to be used.
/// HIPAR port/Selects the antenna port for beamforming measurements.
ViStatus _VI_FUNC rsspecan_ConfigureLTEDownlinkMeasurementAntennaPort(
                            ViSession instrSession,
                            ViInt32   componentCarrier,
                            ViInt32   port)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, componentCarrier, 1, 5),
            2, "Component Carrier");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "cc%ld", componentCarrier);

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_LTE_DOWNLINK_BEAMFORMING_ANTENNA_PORT, port),
            3, "Port");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function selects the antenna port for which beamforming
/// HIFN  measurement results are displayed. The availabilty of ports depends on
/// HIFN  the number of transmit antennas and number of beamforming layers.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR componentCarrier/This control selects the component carrier to be used.
/// HIPAR port/This control selects the antenna port for which beamforming
/// HIPAR port/measurement results are displayed. The availabilty of ports depends on
/// HIPAR port/the number of transmit antennas and number of beamforming layers.
ViStatus _VI_FUNC rsspecan_ConfigureLTEDownlinkMeasurementAntennaPortCellReferenceSignal(
                            ViSession instrSession,
                            ViInt32   componentCarrier,
                            ViInt32   port)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, componentCarrier, 1, 5),
            2, "Component Carrier");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "cc%ld", componentCarrier);

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_LTE_DOWNLINK_BEAMFORMING_ANTENNA_PORT_CELL, port),
            3, "Port");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function selects the antenna port for which beamforming
/// HIFN  measurement results are displayed. The availabilty of ports depends on
/// HIFN  the number of transmit antennas and number of beamforming layers.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR componentCarrier/This control selects the component carrier to be used.
/// HIPAR port/This control selects the antenna port for which beamforming
/// HIPAR port/measurement results are displayed. The availabilty of ports depends on
/// HIPAR port/the number of transmit antennas and number of beamforming layers.
ViStatus _VI_FUNC rsspecan_ConfigureLTEDownlinkMeasurementAntennaPortCSIReferenceSignal(
                            ViSession instrSession,
                            ViInt32   componentCarrier,
                            ViInt32   port)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, componentCarrier, 1, 5),
            2, "Component Carrier");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "cc%ld", componentCarrier);

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_LTE_DOWNLINK_BEAMFORMING_ANTENNA_PORT_CSI, port),
            3, "Port");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries LTE Downlink measurement time alignment error.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR componentCarrier/This control selects the component carrier to be used.
/// HIPAR antenna/This control selects the Time Alignment Antenna.
/// HIPAR frameResult/Selects result type.
/// HIPAR measurementType/Selects measurement type.
/// HIPAR result/Returns selected result.
ViStatus _VI_FUNC rsspecan_QueryLTEDownlinkMeasurementTimeAlignmentError(
                            ViSession instrSession,
                            ViInt32   componentCarrier,
                            ViInt32   antenna,
                            ViInt32   frameResult,
                            ViInt32   measurementType,
                            ViReal64* result)
{
    ViStatus                                             error = VI_SUCCESS;
    ViChar                                               repCap[RS_REPCAP_BUF_SIZE] = "";
    ViAttr                                               attr[4] = {
                                                         RSSPECAN_ATTR_LTE_DOWNLINK_TIME_ALIGNMENT_ERROR_MIN,
    RSSPECAN_ATTR_LTE_DOWNLINK_TIME_ALIGNMENT_ERROR_MAX, 0,
                                                         RSSPECAN_ATTR_LTE_DOWNLINK_TIME_ALIGNMENT_ERROR_AVERAGE
                                                         };

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, componentCarrier, 1, 5),
            2, "Component Carrier");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, antenna, 2, 4),
            3, "Antenna");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, frameResult, RSSPECAN_VAL_LTE_TAER, RSSPECAN_VAL_LTE_FRER),
            4, "Frame Result");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, measurementType, RSSPECAN_VAL_MEASTYPE_MIN, RSSPECAN_VAL_MEASTYPE_AVER),
            5, "Measurement Type");

    if (frameResult == RSSPECAN_VAL_LTE_TAER)
    {
        snprintf(repCap, RS_REPCAP_BUF_SIZE, "cc%ld,TAERAnt%ld", componentCarrier, antenna);

        viCheckParm(rsspecan_GetAttributeViReal64(instrSession, repCap,
                        attr[measurementType - 1],
                        result), 6, "Result");
    }
    else
    {
        snprintf(repCap, RS_REPCAP_BUF_SIZE, "cc%ld", componentCarrier);

        viCheckParm(rsspecan_GetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_LTE_DOWNLINK_TIME_ALIGNMENT_ERROR_FREQUENCY_ERROR_AVERAGE, result),
                6, "Result");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the results of the limit check in the "Off" periods of On/Off
/// HIFN  Power measurements.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR arraySize/Defines the memory size allocated for Results.
/// HIPAR results/Returns one value for every "Off" period.
/// HIPAR returnedValues/Returns number of returned values in Results array.
ViStatus _VI_FUNC rsspecan_QueryLTEDownlinkOffPeriodsLimitCheckResult(
                            ViSession instrSession,
                            ViInt32   arraySize,
                            ViBoolean results[],
                            ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;
    ViChar*  pbuffer = NULL;
    ViChar*  pstring_value;
    ViInt32  i = 0;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K100|K102"));

    checkErr(RsCore_QueryViStringUnknownLength(instrSession, "CALC:LIM:OOP:OFFP?", &pbuffer));

    pstring_value = strtok(pbuffer, ",");

    while (pstring_value != NULL && i < arraySize)
    {
        if (strncmp(pstring_value, "PASSED", 6) == 0)
            results[i] = VI_TRUE;
        else
            results[i] = VI_FALSE;

        pstring_value = strtok(NULL, ",");
        i++;
    }

    if (*returnedValues) *returnedValues = i;

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    if (pbuffer) free(pbuffer);
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the results of the limit check during the transient periods of
/// HIFN  the On/Off power measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR type/This control selects the limit check type.
/// HIPAR arraySize/Defines the memory size allocated for Results.
/// HIPAR results/Returns one value for every transient period.
/// HIPAR returnedValues/Returns number of returned values in Results array.
ViStatus _VI_FUNC rsspecan_QueryLTEDownlinkTransientLimitCheckResult(
                            ViSession instrSession,
                            ViInt32   type,
                            ViInt32   arraySize,
                            ViBoolean results[],
                            ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViChar*  pbuffer = NULL;
    ViChar*  pstring_value;
    ViInt32  i = 0;
    ViString typeArr[] = {"ALL", "FALL", "RIS"};

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, type, RSSPECAN_VAL_LIMIT_ALL, RSSPECAN_VAL_LIMIT_RISING),
            2, "Type");

    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K100|K102"));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CALC:LIM:OOP:TRAN? %s", typeArr[type]);
    checkErr(RsCore_QueryViStringUnknownLength(instrSession, cmd, &pbuffer));

    pstring_value = strtok(pbuffer, ",");

    while (pstring_value != NULL && i < arraySize)
    {
        if (strncmp(pstring_value, "PASSED", 6) == 0)
            results[i] = VI_TRUE;
        else
            results[i] = VI_FALSE;

        pstring_value = strtok(NULL, ",");
        i++;
    }

    if (*returnedValues)
        *returnedValues = i;

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    if (pbuffer) free(pbuffer);
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function queries the state of the limit check for the adjacent or
/// HIFN  alternate channels in an ACLR measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channelType/Selects the ACP channel type.
/// HIPAR limitCheckType/Selects the limit check type.
/// HIPAR results/Returns two results. The first is the result for the lower, the second
/// HIPAR results/for the upper adjacent or alternate channel.
ViStatus _VI_FUNC rsspecan_QueryLTEDownlinkACPChannelLimitCheckResults(
                            ViSession instrSession,
                            ViInt32   channelType,
                            ViInt32   limitCheckType,
                            ViInt32   results[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViChar   response[RS_MAX_MESSAGE_BUF_SIZE];
    ViChar*  p2buf;
    ViString typeArr[] = {"", "REL", "ABS"};

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, limitCheckType, RSSPECAN_VAL_LIMIT_CHECK_TYPE_ALL, RSSPECAN_VAL_LIMIT_CHECK_TYPE_ABS),
            3, "Limit Check Type");

    switch (channelType)
    {
    case RSSPECAN_VAL_ACP_ADJ:
        switch (limitCheckType)
        {
        case RSSPECAN_VAL_LIMIT_CHECK_TYPE_ALL:
            checkErr(RsCore_QueryViString(instrSession, "CALC:LIM:ACP:ACH:RES?", response));
            break;
        default:
            snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CALC:LIM:ACP:ACH:RES? %s", typeArr[limitCheckType]);
            checkErr(RsCore_QueryViString(instrSession, cmd, response));
            break;
        }
        break;
    case RSSPECAN_VAL_ACP_ALT:
        switch (limitCheckType)
        {
        case RSSPECAN_VAL_LIMIT_CHECK_TYPE_ALL:
            checkErr(RsCore_QueryViString(instrSession, "CALC:LIM:ACP:ALT:RES?", response));
            break;
        default:
            snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CALC:LIM:ACP:ALT:RES? %s", typeArr[limitCheckType]);
            checkErr(RsCore_QueryViString(instrSession, cmd, response));
            break;
        }
        break;
    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, channelType), 2, "Channel Type");
    }

    p2buf = strtok(response, ",\n\r");

    if (strcmp(p2buf, "PASSED") == 0)
        results[0] = 1;
    else
        results[0] = 0;

    p2buf = strtok(NULL, ",\r\n");

    if (strcmp(p2buf, "PASSED") == 0)
        results[1] = 1;
    else
        results[1] = 0;

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function queries the current results of the ACLR measurement or
/// HIFN  the total signal power level of the SEM measurement. To get a valid
/// HIFN  result, you have to perform a complete measurement with synchronization
/// HIFN  to the end of the measurement before reading out the result. This is
/// HIFN  only possible for single sweeps.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR powerMeasurement/Selects the ACP (adjacent channel power) standard.
/// HIPAR arraySize/Defines the memory size allocated for Results.
/// HIPAR results/Returns the measured values:
/// HIPAR returnedValues/Returns number of returned values in Results array.
ViStatus _VI_FUNC rsspecan_QueryLTEDownlinkMeasurementPowerResult(
                            ViSession instrSession,
                            ViInt32   powerMeasurement,
                            ViInt32   arraySize,
                            ViReal64  results[],
                            ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViString arr[] = {"ACP", "CPOW", "MCAC", "", "", "", "", "PPOW", "GACL"};

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K100|K102"));

    if ((RsCore_InvalidViInt32Range(instrSession, powerMeasurement, RSSPECAN_VAL_MEAS_POW_ACP,
                                    RSSPECAN_VAL_MEAS_POW_MCAC) == RS_ERROR_INVALID_PARAMETER) &&
        powerMeasurement != RSSPECAN_VAL_MEAS_POW_PPOW)
    {
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, powerMeasurement),
                    2, "Power Measurement");
    }

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CALC1:MARK:FUNC:POW:RES? %s", arr[powerMeasurement]);
    checkErr(RsCore_QueryFloatArrayToUserBuffer(instrSession, cmd, arraySize, results, returnedValues));

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Activates or desactivates the automatic detection of the TX channel power.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR state/Turns automatic detection of the TX channel power on and off.
ViStatus _VI_FUNC rsspecan_ConfigureLTEDownlinkFrequencySweepMeasurementsAuto(
                            ViSession instrSession,
                            ViBoolean state)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_LTE_DOWNLINK_HOME_AREA_BASESTATION_POWER_AUTO, state),
            2, "State");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures SEM limits defined for specific operating bands.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR operatingBandLimitEnabled/Turns SEM limits defined for specific operating bands on and off.
/// HIPAR operatingBand/Selects the SEM for a specific operating band.
ViStatus _VI_FUNC rsspecan_ConfigureLTEDownlinkSEMOperatingBand(
                            ViSession instrSession,
                            ViBoolean operatingBandLimitEnabled,
                            ViInt32   operatingBand)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, NULL, RSSPECAN_ATTR_LTE_DOWNLINK_SEM_OPERATING_BAND_LIMIT_ENABLED, operatingBandLimitEnabled),
            2, "Operating Band Limit Enabled");

    if (operatingBandLimitEnabled == VI_TRUE)
    {
        viCheckParm(rsspecan_SetAttributeViInt32(instrSession, NULL, RSSPECAN_ATTR_LTE_DOWNLINK_SEM_OPERATING_BAND, operatingBand),
                3, "Operating Band");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the results of the CCDF.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR trace/Selects the trace.
/// HIPAR probability/Selects the probability.
/// HIPAR result/Returns the results of the CCDF.
ViStatus _VI_FUNC rsspecan_QueryLTEDownlinkCCDFResults(ViSession instrSession,
                                                       ViInt32   trace,
                                                       ViInt32   probability,
                                                       ViReal64* result)
{
    ViStatus error = VI_SUCCESS;
    ViChar*  p2string;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, trace, 1, 6),
            2, "Trace");
    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &rsspecan_rngMeasStatProbability, probability, &p2string),
                3, "Probability");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, result),
                4, "Result");
    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CALC:STAT:CCDF:X%ld? %s", trace, p2string);
    checkErr(rsspecan_QueryViReal64(instrSession, cmd, result));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the results of a CCDF or ADP measurement for a specific trace.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR trace/Selects the trace.
/// HIPAR resultType/Selects the signal statistic measurement result type.
/// HIPAR result/Returns the selected measurement.
ViStatus _VI_FUNC rsspecan_QueryLTEDownlinkCCDFStatisticalResults(
                            ViSession instrSession,
                            ViInt32   trace,
                            ViInt32   resultType,
                            ViReal64* result)
{
    ViStatus error = VI_SUCCESS;
    ViChar*  p2string;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, trace, 1, 6),
            2, "Trace");
    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &rsspecan_rngStatisticsResultType, resultType, &p2string),
                3, "Result Type");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, result),
                4, "Result");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CALC:STAT:RES%ld? %s", trace, p2string);
    checkErr(rsspecan_QueryViReal64(instrSession, cmd, result));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries all results of all three measurements.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR trace/Selects the trace.
/// HIPAR arraySize/Size of results array.
/// HIPAR results/Returns all of measurement results.
ViStatus _VI_FUNC rsspecan_QueryLTEDownlinkCCDFStatisticalAllResults(
                            ViSession instrSession,
                            ViInt32   trace,
                            ViInt32   arraySize,
                            ViReal64  results[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, trace, 1, 6),
            2, "Trace");
    viCheckParm(RsCore_InvalidViInt32RangeMin(instrSession, arraySize, 1),
                3, "Array Size");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, results),
                4, "Results");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CALC:STAT:RES%ld? ALL", trace);
    checkErr(RsCore_QueryFloatArrayToUserBuffer(instrSession, cmd, arraySize, results, NULL));

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the marker position on the z-axis of three-dimensional result display.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR marker/This control selects delta marker.
/// HIPAR zAxis/Returns the marker position on the z-axis of three-dimensional result displays.
ViStatus _VI_FUNC rsspecan_QueryLTEDownlinkMarkerZAxis(ViSession instrSession,
                                                       ViReal64* zAxis)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_LTE_DOWNLINK_MARKER_Z_AXIS, zAxis),
            3, "Z-Axis");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the marker position on the z-axis of three-dimensional result displays.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR marker/This control selects delta marker.
/// HIPAR zAxis/Returns the marker position on the z-axis of three-dimensional result displays.
ViStatus _VI_FUNC rsspecan_QueryLTEDownlinkMarkerZAxisAll(ViSession instrSession,
                                                          ViInt32   arraySize,
                                                          ViReal64  zAxis[])
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32RangeMin(instrSession, arraySize, 1),
                2, "Array Size");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, zAxis),
                3, "Z-Axis");

    checkErr(RsCore_QueryFloatArrayToUserBuffer(instrSession, "CALC:MARK:Z:ALL?", arraySize, zAxis, NULL));

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}
