/*****************************************************************************
 *  Rohde&Schwarz Spectrum Analyzer instrument driver
 *  LTE option K101/K105
 *
 *  Original Release: May 2014
 *  By: Martin Krcmar
 *
 *  Should you have any technical questions please contact the hotline of
 *  Rohde & Schwarz Vertriebs-GmbH Rohde & Schwarz Support Center
 *
 *  e-mail: CustomerSupport@rohde-schwarz.com
 *
 *****************************************************************************/

#include "rsspecan.h"

/// HIFN  This function selects LTE uplink measurement mode.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsspecan_SetLTEUplinkMode(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_LTE_UPLINK_MODE, NULL));
    checkErr(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_LTE_LINK_DIRECTION, RSSPECAN_VAL_LTE_UPLINK));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN Configures number of carriers.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR numberofCarriers/This control selects the number of component carriers
ViStatus _VI_FUNC rsspecan_ConfigureLTEUplinkNumberOfCarriers(
                            ViSession instrSession,
                            ViInt32   numberOfCarriers)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_LTE_UPLINK_COMPONENT_CARRIERS, numberOfCarriers),
            2, "Number of Carriers");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function configures the general settings concerning the physical
/// HIFN  attributes of the signals to be measured.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR componentCarrier/This control sets the component carrier.
/// HIPAR duplexing/Configures the duplexing mode.
/// HIPAR frequency/The control defines the center frequency of the analyzer.
/// HIPAR bandwidthSettingType/Setting type of bandwidth
/// HIPAR channelBandwidth/Setting channel bandwidth.
/// HIPAR numberofResourceBlocks/Configures the number of resource blocks used for UL.
/// HIPAR cyclicPrefix/Configures the cyclic prefix type for UP.
ViStatus _VI_FUNC rsspecan_ConfigureLTEUplinkSignalCharacteristics(
                            ViSession instrSession,
                            ViInt32   componentCarrier,
                            ViInt32   duplexing,
                            ViReal64  frequency,
                            ViInt32   bandwidthSettingType,
                            ViInt32   channelBandwidth,
                            ViInt32   numberofResourceBlocks,
                            ViInt32   cyclicPrefix)
{
    ViStatus  error = VI_SUCCESS;
    ViChar    repCap[RS_REPCAP_BUF_SIZE] = "";
    ViBoolean isFSV = VI_FALSE;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_LTE_UPLINK_DUPLEXING, duplexing),
            3, "Duplexing");

     isFSV = rsspecan_IsFSV(instrSession);

    if (isFSV == VI_FALSE)
    {
        viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_LTE_UPLINK_CARRIER_FREQUENCY_CENTER,
                "cc", componentCarrier - 1, RS_REPCAP_BUF_SIZE, repCap),
                2, "Component Carrier");

        viCheckParm(rsspecan_SetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_LTE_UPLINK_CARRIER_FREQUENCY_CENTER, frequency),
                4, "Frequency");
    }
    else
    {
        viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_FREQUENCY_CENTER, frequency),
                4, "Frequency");
    }

    switch (bandwidthSettingType)
    {
    case RSSPECAN_VAL_LTE_DOWNLINK_CHBW:
        if (isFSV == VI_FALSE)
        {
            viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_LTE_UPLINK_CARRIER_CHANNEL_BANDWIDTH, channelBandwidth),
                    6, "Channel bandwidth");
        }
        else
        {
            viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_LTE_UPLINK_CHANNEL_BANDWIDTH, channelBandwidth),
                    6, "Channel bandwidth");
        }
        break;
    case RSSPECAN_VAL_LTE_DOWNLINK_NRB:
        viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_LTE_UPLINK_NUMBER_OF_RESOURCE_BLOCKS, numberofResourceBlocks),
                7, "Number of Resource Blocks");
        break;
    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, bandwidthSettingType), 5, "Bandwidth Setting Type");
        break;
    }

    if (isFSV == VI_FALSE)
    {
        viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_LTE_UPLINK_CYCLIC_PREFIX, cyclicPrefix),
                8, "Cyclic Prefix");
    }
    else
    {
        viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_LTE_UPLINK_CYCLIC_PREFIX_FSV, cyclicPrefix),
                8, "Cyclic Prefix");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the TDD UL/DL Allocations and TDD special subframe.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR componentCarrier/This control sets the component carrier.
/// HIPAR tddULDLAllocations/Configures the TDD UL/DL Allocations.
/// HIPAR specialSubframe/Selects the special subframe configuration for LTE uplink signals.
ViStatus _VI_FUNC rsspecan_ConfigureLTEUplinkTDDFrames(ViSession instrSession,
                                                       ViInt32   componentCarrier,
                                                       ViInt32   tddULDLAllocations,
                                                       ViInt32   specialSubframe)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    if (rsspecan_IsFSV(instrSession) == VI_FALSE)
    {
        viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_LTE_UPLINK_TDD_UL_DL_ALLOCATIONS,
                "cc", componentCarrier - 1, RS_REPCAP_BUF_SIZE, repCap),
                2, "Component Carrier");

        viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_LTE_UPLINK_TDD_UL_DL_ALLOCATIONS, tddULDLAllocations),
                3, "TDD UL/DL Allocations");

        viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_LTE_UPLINK_TDD_SPECIAL_SUBFRAME, specialSubframe),
                4, "Special Subframe");
    }
    else
    {
        viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_LTE_UPLINK_TDD_UL_DL_ALLOCATIONS_FSV, tddULDLAllocations),
                3, "TDD UL/DL Allocations");

        viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_LTE_UPLINK_TDD_SPECIAL_SUBFRAME_FSV, specialSubframe),
                4, "Special Subframe");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures physical layer cell identity. There are 504 unique
/// HIFN  physical-layer cell identities. The physical-layer cell identities are
/// HIFN  grouped into 168 unique physical-layer cell-identity groups, each group
/// HIFN  containing  three unique identities.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR componentCarrier/This control sets the component carrier.
/// HIPAR cellIdentityGroup/Cell Identity Group specifies the physical-layer cell identity group.
/// HIPAR identity/Identity specifies the physical-layer identity.
ViStatus _VI_FUNC rsspecan_ConfigureLTEUplinkPhysicalLayerCellIdentity(
                            ViSession instrSession,
                            ViInt32   componentCarrier,
                            ViInt32   cellIdentityGroup,
                            ViInt32   identity)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    if (rsspecan_IsFSV(instrSession) == VI_FALSE)
    {
        viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_LTE_UPLINK_CELL_IDENTITY_GROUP,
                "cc", componentCarrier - 1, RS_REPCAP_BUF_SIZE, repCap),
                2, "Component Carrier");

        viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_LTE_UPLINK_CELL_IDENTITY_GROUP, cellIdentityGroup),
                3, "Cell Identity Group");

        viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_LTE_UPLINK_CELL_IDENTITY, identity),
                4, "Identity");
    }
    else
    {
        viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_LTE_UPLINK_CELL_IDENTITY_GROUP_FSV, cellIdentityGroup),
                3, "Cell Identity Group");

        viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_LTE_UPLINK_CELL_IDENTITY_FSV, identity),
                4, "Identity");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the physical layer cell identity.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR componentCarrier/This control selects the component carrier to be used.
/// HIPAR cellIdentityGroup/Returns the current cell identity group for Uplink.
/// HIPAR cellIdentity/Returns the current cell identity for Uplink.
ViStatus _VI_FUNC rsspecan_QueryLTEUplinkPhysicalLayerCellIdentity(
                            ViSession instrSession,
                            ViInt32   componentCarrier,
                            ViInt32*  cellIdentityGroup,
                            ViInt32*  cellIdentity)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, componentCarrier, 1, 5),
            2, "Component Carrier");

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_LTE_UPLINK_DETECTED_CELL_IDENTITY_GROUP,
            "cc", componentCarrier - 1, RS_REPCAP_BUF_SIZE, repCap));

    viCheckParm(rsspecan_GetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_LTE_UPLINK_DETECTED_CELL_IDENTITY_GROUP, cellIdentityGroup),
            3, "Cell Identity Group");

    viCheckParm(rsspecan_GetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_LTE_UPLINK_DETECTED_CELL_IDENTITY, cellIdentity),
            4, "Cell Identity");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Configures spectrum flatness measurements.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR componentCarrier/This control sets the component carrier.
/// HIPAR extremeConditions/Turns extreme conditions for spectrum flatness measurements on an off.
/// HIPAR operatingBandIndex/Selects one of the 40 operating bands for spectrum flatness
/// HIPAR operatingBandIndex/measurements as defined in TS 36.101.
ViStatus _VI_FUNC rsspecan_ConfigureLTEUplinkSpectrumFlatness(
                            ViSession instrSession,
                            ViInt32   componentCarrier,
                            ViBoolean extremeConditions,
                            ViInt32   operatingBandIndex)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_LTE_UPLINK_EXTREME_CONDITIONS,
            "cc", componentCarrier - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Component Carrier");

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, repCap, RSSPECAN_ATTR_LTE_UPLINK_EXTREME_CONDITIONS, extremeConditions),
            3, "Extreme Conditions");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_LTE_UPLINK_OPERATING_BAND_INDEX, operatingBandIndex),
            4, "Operating Band Index");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN Selects the channel bandwidth(s) of the carriers in MC ACLR
/// HIFN measurements.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channelBandwidth/Sets the channel bandwidth..
ViStatus _VI_FUNC rsspecan_ConfigureLTEUplinkCarrierAgregationBandwidth(
                            ViSession instrSession,
                            ViInt32   channelBandwidth)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_LTE_UPLINK_CARRIERS_BANDWIDTHS, channelBandwidth),
            2, "Channel Bandwidth");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN Selects the channel bandwidth(s) of the carriers in MC ACLR
/// HIFN measurements.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR componentCarrier/This control sets the component carrier.
/// HIPAR frequency/The control sets the center frequency for RF measurements.
/// HIPAR frequencyOffset/Defines the frequency offset for a component carrier.
/// HIPAR channelBandwidth/Sets channel bandwidth..
ViStatus _VI_FUNC rsspecan_ConfigureLTEUplinkCarrierConfiguration(
                            ViSession instrSession,
                            ViInt32   componentCarrier,
                            ViReal64  frequency,
                            ViReal64  frequencyOffset,
                            ViInt32   channelBandwidth)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_LTE_UPLINK_CARRIER_FREQUENCY_CENTER,
            "cc", componentCarrier - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Component Carrier");

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_LTE_UPLINK_CARRIER_FREQUENCY_CENTER, frequency),
            3, "Frequency");

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_LTE_UPLINK_CARRIER_FREQUENCY_CENTER_OFFSET, frequencyOffset),
            4, "Frequency Offset");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_LTE_UPLINK_CARRIER_CHANNEL_BANDWIDTH, channelBandwidth),
            5, "Channel Bandwidth");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Saves the signal description.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR componentCarrier/This control selects the component carrier to be used.
/// HIPAR filename/Saves the signal description.
ViStatus _VI_FUNC rsspecan_StoreLTEUplinkDemodulationSettings(
                            ViSession instrSession,
                            ViInt32   componentCarrier,
                            ViString  filename)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_LTE_UPLINK_STORE_DEMODULATION_SETTINGS,
            "cc", componentCarrier - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Component Carrier");

    viCheckParm(rsspecan_SetAttributeViString(instrSession, repCap, RSSPECAN_ATTR_LTE_UPLINK_STORE_DEMODULATION_SETTINGS, filename),
            3, "Filename");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function configures essential antenna configuration parameters.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR componentCarrier/This control selects the component carrier to be used.
/// HIPAR configuration/This control is reserved for future use.
/// HIPAR antennaSelection/Configures the antenna selection.
ViStatus _VI_FUNC rsspecan_ConfigureLTEUplinkMIMO(ViSession instrSession,
                                                  ViInt32   componentCarrier,
                                                  ViInt32   configuration,
                                                  ViInt32   antennaSelection)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
             configuration;

    checkErr(RsCore_LockSession(instrSession));

    if (rsspecan_IsFSV(instrSession) == VI_FALSE)
    {
        viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_LTE_UPLINK_MIMO_ANTENNA,
                "cc", componentCarrier - 1, RS_REPCAP_BUF_SIZE, repCap),
                2, "Component Carrier");

        viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_LTE_UPLINK_MIMO_ANTENNA, antennaSelection),
                4, "Antenna Selection");
    }
    else
    {
        /*    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "",
                                                       RSSPECAN_ATTR_LTE_DOWNLINK_MIMO_CONFIGURATION,
                                                       configuration), 3, "Configuration");
        */
        viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_LTE_UPLINK_MIMO_ANTENNA_FSV, antennaSelection),
                4, "Antenna Selection");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the subframes settings.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR autoDemodulation/This control selects the method of automatic demodulation for uplink
/// HIPAR autoDemodulation/signals.
/// HIPAR subframeConfDetection/This control turns detection of the subframe configuration on and off.
/// HIPAR subframeConfDetection/
ViStatus _VI_FUNC rsspecan_ConfigureLTEUplinkSubframes(ViSession instrSession,
                                                       ViInt32   autoDemodulation,
                                                       ViBoolean subframeConfDetection)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_LTE_UPLINK_AUTOMATIC_DEMODULATION, autoDemodulation),
            2, "Auto Demodulation");

    if (autoDemodulation == RSSPECAN_VAL_LTE_AUTO_DEMOD_OFF)
    {
        viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_LTE_UPLINK_SUBFRAME_CONFIGURATION_DETECTION, subframeConfDetection),
                3, "Subframe Conf Detection");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the number of subframes that can be configured.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR componentCarrier/This control sets the component carrier.
/// HIPAR configurableSubframes/Sets the number of subframes that can be configured.
ViStatus _VI_FUNC rsspecan_ConfigureLTEUplinkConfigurableSubframes(
                            ViSession instrSession,
                            ViInt32   componentCarrier,
                            ViInt32   configurableSubframes)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    if (rsspecan_IsFSV(instrSession) == VI_FALSE)
    {
        viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_LTE_UPLINK_CONFIGURABLE_SUBFRAMES,
                "cc", componentCarrier - 1, RS_REPCAP_BUF_SIZE, repCap),
                2, "Component Carrier");

        viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_LTE_UPLINK_CONFIGURABLE_SUBFRAMES, configurableSubframes),
                3, "Configurable Subframes");
    }
    else
    {
        viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_LTE_UPLINK_CONFIGURABLE_SUBFRAMES_FSV, configurableSubframes),
                3, "Configurable Subframes");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures specified allocation in specified subframe.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR componentCarrier/This control sets the component carrier.
/// HIPAR subframe/Selects configurable subframe.
/// HIPAR enableAllocation/Activates and deactivates the PUCCH in the corresponding subframe.
/// HIPAR modulation/Configures the modulation type for an allocation in a specific UL
/// HIPAR modulation/subframe.
/// HIPAR cluster/This control selects the claster for Resource Blocks and
/// HIPAR cluster/Resource Blocks Offset controls.
/// HIPAR resourceBlocks/Configures the number of resource blocks for an allocation in a
/// HIPAR resourceBlocks/specific UL subframe.
/// HIPAR resourceBlocksOffset/Configures the resource block offset for an allocation in a specific
/// HIPAR resourceBlocksOffset/UL subframe.
ViStatus _VI_FUNC rsspecan_ConfigureLTEUplinkSubframeTable(ViSession instrSession,
                                                           ViInt32   componentCarrier,
                                                           ViInt32   subframe,
                                                           ViInt32   enableAllocation,
                                                           ViInt32   modulation,
                                                           ViInt32   cluster,
                                                           ViInt32   resourceBlocks,
                                                           ViInt32   resourceBlocksOffset)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap_s[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_cl[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, subframe, 0, 9),
            2, "Subframe");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_LTE_UPLINK_SUBFRAME_ALLOCATION_ENABLE_PUCCH,
            "LTESubframe", subframe , RS_REPCAP_BUF_SIZE, repCap_s),
            3, "Subframe");

    if (rsspecan_IsFSV(instrSession) == VI_FALSE)
    {
        viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_LTE_UPLINK_SUBFRAME_ALLOCATION_ENABLE_PUCCH,
                "cc", componentCarrier - 1, RS_REPCAP_BUF_SIZE, repCap),
                2, "Component Carrier");

        strcat(repCap, ",");
        strcat(repCap, repCap_s);

        viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_LTE_UPLINK_SUBFRAME_ALLOCATION_ENABLE_PUCCH, enableAllocation),
                4, "Enable Allocation");

        if (enableAllocation == RSSPECAN_VAL_LTE_UPLINK_SUBFRAME_PSCC || (enableAllocation ==
            RSSPECAN_VAL_LTE_UPLINK_SUBFRAME_PUSCH))
        {
            viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_LTE_UPLINK_SUBFRAME_ALLOCATION_MODULATION, modulation),
                    5, "Modulation");

            viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_LTE_UPLINK_SUBFRAME_ALLOCATION_RESOURCE_BLOCKS,
                    "LTECluster", cluster -1, RS_REPCAP_BUF_SIZE, repCap_cl),
                    2, "Cluster");

            strcat(repCap, ",");
            strcat(repCap, repCap_cl);

            viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_LTE_UPLINK_SUBFRAME_ALLOCATION_RESOURCE_BLOCKS, resourceBlocks),
                    7, "Resource Blocks");

            viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_LTE_UPLINK_SUBFRAME_ALLOCATION_RESOURCE_BLOCKS_OFFSET, resourceBlocksOffset),
                    8, "Resource Blocks Offset");
        }
    }
    else
    {
        viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap_s, RSSPECAN_ATTR_LTE_UPLINK_SUBFRAME_ALLOCATION_ENABLE_PUCCH_FSV, enableAllocation),
                4, "Enable Allocation");

        if (enableAllocation == RSSPECAN_VAL_LTE_UPLINK_SUBFRAME_PSCC || (enableAllocation ==
            RSSPECAN_VAL_LTE_UPLINK_SUBFRAME_PUSCH))
        {
            viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap_s, RSSPECAN_ATTR_LTE_UPLINK_SUBFRAME_ALLOCATION_MODULATION_FSV, modulation),
                    5, "Modulation");

            viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap_s, RSSPECAN_ATTR_LTE_UPLINK_SUBFRAME_ALLOCATION_RESOURCE_BLOCKS_FSV, resourceBlocks),
                    7, "Resource Blocks");

            viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap_s, RSSPECAN_ATTR_LTE_UPLINK_SUBFRAME_ALLOCATION_RESOURCE_BLOCKS_OFFSET_FSV, resourceBlocksOffset),
                    8, "Resource Blocks Offset");
        }
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Configures global settings that apply to the complete signal.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR componentCarrier/This control sets the component carrier.
/// HIPAR frameNumberOffset/Defines the system frame number offset. The application uses the
/// HIPAR frameNumberOffset/offset to demodulate the frame.
/// HIPAR ueID_n_RNTI/Sets the radio network temporary identifier (RNTI) of the UE.
ViStatus _VI_FUNC rsspecan_ConfigureLTEUplinkGlobalSettings(ViSession instrSession,
                                                            ViInt32   componentCarrier,
                                                            ViInt32   frameNumberOffset,
                                                            ViInt32   ueID_n_RNTI)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_LTE_UPLINK_FRAME_NUMBER_OFFSET,
            "cc", componentCarrier - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Component Carrier");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_LTE_UPLINK_FRAME_NUMBER_OFFSET, frameNumberOffset),
            3, "Frame Number Offset");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_LTE_UPLINK_UE_RADIO_NETWORK_TEMPORARY_IDENTIFIER, ueID_n_RNTI),
            4, "UE ID/n_RNTI");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the reference signal for PUSCH and PUCCH.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR componentCarrier/This control sets the component carrier.
/// HIPAR relativePowerPUSCH/Configures the relative power of the physical uplink shared channel.
/// HIPAR relativePowerPUCCH/Configures the relative power of the physical uplink control channel.
/// HIPAR groupHopping/Enables or disables group hopping in the 3GPP configuration for LTE
/// HIPAR groupHopping/uplink signals.
/// HIPAR sequenceHopping/Enables or disables sequence hopping in the 3GPP configuration for LTE
/// HIPAR sequenceHopping/uplink signals.
/// HIPAR deltaSequenceShift/Configures the delta sequence shift in the 3GPP configuration of the
/// HIPAR deltaSequenceShift/LTE uplink signal.
/// HIPAR n_dmrs/The n_DMRS parameter can be found in 3GPP TS36.211 V8.5.0, 5.5.2.1.1
/// HIPAR n_dmrs/Reference signal sequence. Currently, n_DMRS is defined as n_DMRS =
/// HIPAR n_dmrs/nDMRS(1) + nDMRS(2).
/// HIPAR activateDMRSWithOCC/Turns the configuration of the demodulation reference signal on a
/// HIPAR activateDMRSWithOCC/subframe basis via the 'Cyclic Field Shift' on and off.
ViStatus _VI_FUNC rsspecan_ConfigureLTEUplinkReferenceSignal(ViSession instrSession,
                                                             ViInt32   componentCarrier,
                                                             ViReal64  relativePowerPUSCH,
                                                             ViReal64  relativePowerPUCCH,
                                                             ViBoolean groupHopping,
                                                             ViBoolean sequenceHopping,
                                                             ViInt32   deltaSequenceShift,
                                                             ViInt32   n_dmrs,
                                                             ViBoolean activateDMRSWithOCC)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    if (rsspecan_IsFSV(instrSession) == VI_FALSE)
    {
        viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_LTE_UPLINK_POWER_PUSCH,
                "cc", componentCarrier - 1, RS_REPCAP_BUF_SIZE, repCap),
                2, "Component Carrier");

        viCheckParm(rsspecan_SetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_LTE_UPLINK_POWER_PUSCH, relativePowerPUSCH),
                3, "Relative Power PUSCH");

        viCheckParm(rsspecan_SetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_LTE_UPLINK_POWER_PUCCH, relativePowerPUCCH),
                4, "Relative Power PUCCH");

        viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, repCap, RSSPECAN_ATTR_LTE_UPLINK_GROUP_HOPPING, groupHopping),
                5, "Group Hopping");

        viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, repCap, RSSPECAN_ATTR_LTE_UPLINK_SEQUENCE_HOPPING, sequenceHopping),
                6, "Sequence Hopping");

        viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_LTE_UPLINK_DELTA_SEQUENCE_SHIFT, deltaSequenceShift),
                7, "Delta Sequence Shift");

        viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_LTE_UPLINK_N_DMRS, n_dmrs),
                8, "n_DMRS");

        viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, repCap, RSSPECAN_ATTR_LTE_UPLINK_ACTIVATE_DMRS_WITH_OCC, activateDMRSWithOCC),
                9, "Activate-DMRS-With OCC");
    }
    else
    {
        viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_LTE_UPLINK_POWER_PUSCH_FSV, relativePowerPUSCH),
                3, "Relative Power PUSCH");

        viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_LTE_UPLINK_POWER_PUCCH_FSV, relativePowerPUCCH),
                4, "Relative Power PUCCH");

        viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_LTE_UPLINK_GROUP_HOPPING_FSV, groupHopping),
                5, "Group Hopping");

        viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_LTE_UPLINK_SEQUENCE_HOPPING_FSV, sequenceHopping),
                6, "Sequence Hopping");

        viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_LTE_UPLINK_DELTA_SEQUENCE_SHIFT_FSV, deltaSequenceShift),
                7, "Delta Sequence Shift");

        viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_LTE_UPLINK_N_DMRS_FSV, n_dmrs),
                8, "n_DMRS");

        viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_LTE_UPLINK_ACTIVATE_DMRS_WITH_OCC_FSV, activateDMRSWithOCC),
                9, "Activate-DMRS-With OCC");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the (cell) Identity of the Demodulation Reference Signal (DRS).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR componentCarrier/This control sets the component carrier.
/// HIPAR cellID/Selects the use of the common Identity.
ViStatus _VI_FUNC rsspecan_ConfigureLTEUplinkDRSCellID(ViSession instrSession,
                                                       ViInt32   componentCarrier,
                                                       ViInt32   cellID)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, componentCarrier, 1, 5),
            2, "Component Carrier");

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_LTE_UPLINK_DRS_CELL_ID,
            "cc", componentCarrier - 1, RS_REPCAP_BUF_SIZE, repCap));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_LTE_UPLINK_DRS_CELL_ID, cellID),
            3, "Cell ID");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the (cell) Identity value of the Demodulation Reference Signal (DRS).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR componentCarrier/This control sets the component carrier.
/// HIPAR cellIDValue/Sets the identity for the DRS.
ViStatus _VI_FUNC rsspecan_ConfigureLTEUplinkDRSCellIDValue(ViSession instrSession,
                                                            ViInt32   componentCarrier,
                                                            ViInt32   cellIDValue)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, componentCarrier, 1, 5),
            2, "Component Carrier");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "cc%ld", componentCarrier);

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_LTE_UPLINK_DRS_CELL_ID_VALUE, cellIDValue),
            3, "Cell ID Value");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the sounding reference signal.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR componentCarrier/This control sets the component carrier.
/// HIPAR present/Configures whether the sounding reference signal is present or not.
/// HIPAR subframeConfiguration/Defines the subframe configuration of the SRS.
/// HIPAR bandwidthB_SRS/Defines the parameter B_SRS. B_SRS is a UE specific parameter that
/// HIPAR bandwidthB_SRS/defines the bandwidth of the SRS. The SRS either
/// HIPAR hoppingBWb_hop/Defines the parameter b hop.
/// HIPAR cyclixShiftN_CS/Defines the cyclic shift (n CS ) used for the generation of the SRS
/// HIPAR cyclixShiftN_CS/CAZAC sequence.
/// HIPAR relativePower/Relative Power of the sounding reference signal.
/// HIPAR bwConfC_SRS/Defines the bandwidth configuration of the SRS. The bandwidth
/// HIPAR bwConfC_SRS/configuration is a cell specific parameter that, in combination with
/// HIPAR bwConfC_SRS/the SRS bandwidth and the channel bandwidth, defines the length of the
/// HIPAR bwConfC_SRS/souunding reference signal sequence.
/// HIPAR confIndexI_SRS/Defines the configuration index of the SRS.
/// HIPAR transmissionCombk_TC/Defines the transmission comb k TC .
/// HIPAR frequencyDomainPosition/This control sets the UE specific parameter Frequency Domain Position
/// HIPAR frequencyDomainPosition/nRRC.
/// HIPAR anSimultaneousTX/This control turns simultaneous transmission of the Sounding Reference
/// HIPAR anSimultaneousTX/Signal (SRS) and ACK/NACK messages (via PUCCH) on and off.
ViStatus _VI_FUNC rsspecan_ConfigureLTEUplinkSoundingReferenceSignal(
                            ViSession instrSession,
                            ViInt32   componentCarrier,
                            ViBoolean present,
                            ViInt32   subframeConfiguration,
                            ViInt32   bandwidthB_SRS,
                            ViInt32   hoppingBWb_hop,
                            ViInt32   cyclixShiftN_CS,
                            ViReal64  relativePower,
                            ViInt32   bwConfC_SRS,
                            ViInt32   confIndexI_SRS,
                            ViInt32   transmissionCombk_TC,
                            ViInt32   frequencyDomainPosition,
                            ViBoolean anSimultaneousTX)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_LTE_UPLINK_SOUNDING_REFERENCE_SIGNAL_PRESENT,
            "cc", componentCarrier - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Component Carrier");

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, repCap, RSSPECAN_ATTR_LTE_UPLINK_SOUNDING_REFERENCE_SIGNAL_PRESENT, present),
            3, "Present");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_LTE_UPLINK_SOUNDING_REFERENCE_SIGNAL_SUBFRAME_CONFIGURATION, subframeConfiguration),
            4, "Subframe Configuration");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_LTE_UPLINK_SOUNDING_REFERENCE_SIGNAL_BANDWIDTH_B_SRS, bandwidthB_SRS),
            5, "Bandwidth B_SRS");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_LTE_UPLINK_SOUNDING_REFERENCE_SIGNAL_HOPPING_BW, hoppingBWb_hop),
            6, "Hopping BW b_hop");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_LTE_UPLINK_SOUNDING_REFERENCE_SIGNAL_CYCLIC_SHIFT_N_CS, cyclixShiftN_CS),
            7, "Cyclix Shift N_CS");

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_LTE_UPLINK_SOUNDING_REFERENCE_SIGNAL_POWER, relativePower),
            8, "Relative Power");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_LTE_UPLINK_SOUNDING_REFERENCE_SIGNAL_BANDWIDTH_CONFIGURATION_C_SRS, bwConfC_SRS),
            9, "BW Conf C_SRS");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_LTE_UPLINK_SOUNDING_REFERENCE_SIGNAL_CONFIGURATION_INDEX, confIndexI_SRS),
            10, "Conf Index I_SRS");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_LTE_UPLINK_SOUNDING_REFERENCE_SIGNAL_TRANSMISSION_COMB, transmissionCombk_TC),
            11, "Transmission Comb k_TC");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_LTE_UPLINK_SOUNDING_REFERENCE_SIGNAL_N_RRC, frequencyDomainPosition),
            12, "Frequency Domain Position");

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, repCap, RSSPECAN_ATTR_LTE_UPLINK_SOUNDING_REFERENCE_SIGNAL_AN_TX, anSimultaneousTX),
            13, "AN Simultaneous TX");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Enables or disables the SRS MaxUpPts.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR componentCarrier/This control sets the component carrier.
/// HIPAR maxUpPts/Enables or disables the SRS MaxUpPts.
ViStatus _VI_FUNC rsspecan_ConfigureLTEUplinkSoundingReferenceSignalState(
                            ViSession instrSession,
                            ViInt32   componentCarrier,
                            ViBoolean maxUpPts)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, componentCarrier, 1, 5),
            2, "Component Carrier");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "cc%ld", componentCarrier);

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, repCap, RSSPECAN_ATTR_LTE_UPLINK_SOUNDING_REFERENCE_SIGNAL_MAXUPPTS, maxUpPts),
            3, "Max Up Pts");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the PUSCH structure.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR componentCarrier/This control sets the component carrier.
/// HIPAR frequencyHoppingMode/Configures the frequency hopping mode in the PUSCH structure settings
/// HIPAR frequencyHoppingMode/for UL.
/// HIPAR numberofSubbands/Configures the number of subbands/M in the PUSCH structure settings
/// HIPAR numberofSubbands/for LTE uplink signals.
/// HIPAR hoppingOffset/Defines the frequency hopping offset for the PUSCH.
/// HIPAR infoinHoppingBits/Defines the information in hopping bits of the PUSCH.
ViStatus _VI_FUNC rsspecan_ConfigureLTEUplinkPUSCHStructure(ViSession instrSession,
                                                            ViInt32   componentCarrier,
                                                            ViInt32   frequencyHoppingMode,
                                                            ViInt32   numberofSubbands,
                                                            ViInt32   hoppingOffset,
                                                            ViInt32   infoinHoppingBits)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    if (rsspecan_IsFSV(instrSession) == VI_FALSE)
    {
        viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_LTE_UPLINK_PUSCH_FREQUENCY_HOPPING_MODE,
                "cc", componentCarrier - 1, RS_REPCAP_BUF_SIZE, repCap),
                2, "Component Carrier");

        viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_LTE_UPLINK_PUSCH_FREQUENCY_HOPPING_MODE, frequencyHoppingMode),
                3, "Frequency Hopping Mode");

        viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_LTE_UPLINK_PUSCH_SUBBANDS, numberofSubbands),
                4, "Number of Subbands");

        viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_LTE_UPLINK_PUSCH_HOPPING_OFFSET, hoppingOffset),
                5, "Hopping Offset");

        viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_LTE_UPLINK_PUSCH_INFO_IN_HOPPING_BITS, infoinHoppingBits),
                6, "Info in Hopping Bits");
    }
    else
    {
        viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_LTE_UPLINK_PUSCH_FREQUENCY_HOPPING_MODE_FSV, frequencyHoppingMode),
                3, "Frequency Hopping Mode");

        viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_LTE_UPLINK_PUSCH_SUBBANDS_FSV, numberofSubbands),
                4, "Number of Subbands");

        viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_LTE_UPLINK_PUSCH_HOPPING_OFFSET_FSV, hoppingOffset),
                5, "Hopping Offset");

        viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_LTE_UPLINK_PUSCH_INFO_IN_HOPPING_BITS_FSV, infoinHoppingBits),
                6, "Info in Hopping Bits");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the (cell) Identity of the PUSCH from Cell ID: Uses the common
/// HIFN  Identity defined with RSSPECAN_ATTR_LTE_UPLINK_CELL_IDENTITY.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR componentCarrier/This control sets the component carrier.
/// HIPAR cellID/Defines the (cell) Identity of the PUSCH from Cell ID: Uses the common
/// HIPAR cellID/Identity defined with RSSPECAN_ATTR_LTE_UPLINK_CELL_IDENTITY.
ViStatus _VI_FUNC rsspecan_ConfigureLTEUplinkPUSCHCellID(ViSession instrSession,
                                                         ViInt32   componentCarrier,
                                                         ViInt32   cellID)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_LTE_UPLINK_PUSCH_CELL_ID,
            "cc", componentCarrier - 1, RS_REPCAP_BUF_SIZE, repCap));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_LTE_UPLINK_PUSCH_CELL_ID, cellID),
            3, "Cell ID");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the (cell) Identity of the PUSCH.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR componentCarrier/This control sets the component carrier.
/// HIPAR cellIDValue/Defines the (cell) Identity of the PUSCH.
ViStatus _VI_FUNC rsspecan_ConfigureLTEUplinkPUSCHCellIDValue(
                            ViSession instrSession,
                            ViInt32   componentCarrier,
                            ViInt32   cellIDValue)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_LTE_UPLINK_PUSCH_CELL_ID_VALUE,
            "cc", componentCarrier - 1, RS_REPCAP_BUF_SIZE, repCap));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_LTE_UPLINK_PUSCH_CELL_ID_VALUE, cellIDValue),
            3, "Cell ID Value");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the PUCCH structure.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR componentCarrier/This control sets the component carrier.
/// HIPAR numberofRBsforPUCCH/Number of RBs for PUCCH configures the number of resource blocks for
/// HIPAR numberofRBsforPUCCH/PUCCH.
/// HIPAR deltaShift/Sets the delta shift parameter, i.e. the cyclic shift difference
/// HIPAR deltaShift/between two adjacent PUCCH resource indices with the same orthogonal
/// HIPAR deltaShift/cover sequence (OC).
/// HIPAR n_1_cs/Sets the number of cyclic shifts used for PUCCH format 1/1a/1b in a
/// HIPAR n_1_cs/resource block used for a combination of the formats 1/1a/1b and
/// HIPAR n_1_cs/2/2a/2b.
/// HIPAR n_2_RB/Sets bandwidth in terms of resource blocks that are reserved for PUCCH
/// HIPAR n_2_RB/formats 2/2a/2b transmission in each subframe.
/// HIPAR format/Configures the PUCCH format.
/// HIPAR n_pucch/Sets the resource index for PUCCH format 1/1a/1b respectively 2/2a/2b.
ViStatus _VI_FUNC rsspecan_ConfigureLTEUplinkPUCCHStructure(ViSession instrSession,
                                                            ViInt32   componentCarrier,
                                                            ViInt32   numberofRBsforPUCCH,
                                                            ViInt32   deltaShift,
                                                            ViInt32   n_1_cs,
                                                            ViInt32   n_2_RB,
                                                            ViInt32   format,
                                                            ViInt32   n_pucch)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_LTE_UPLINK_PUCCH_RESOURCE_BLOCKS,
            "cc", componentCarrier - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Component Carrier");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_LTE_UPLINK_PUCCH_RESOURCE_BLOCKS, numberofRBsforPUCCH),
            3, "Number of RBs for PUCCH");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_LTE_UPLINK_PUCCH_DELTA_SHIFT, deltaShift),
            4, "Delta Shift");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_LTE_UPLINK_PUCCH_CYCLIC_SHIFTS, n_1_cs),
            5, "N1_cs");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_LTE_UPLINK_PUCCH_BANDWIDTH, n_2_RB),
            6, "N2_RB");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_LTE_UPLINK_PUCCH_FORMAT, format),
            7, "Format");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_LTE_UPLINK_PUCCH_RESOURCE_INDEX, n_pucch),
            8, "N_PUCCH");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the (cell) Identity of the PUCCH.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR componentCarrier/This control sets the component carrier.
/// HIPAR cellID/Selects the use of the common Identity.
ViStatus _VI_FUNC rsspecan_ConfigureLTEUplinkPUCCHCellID(ViSession instrSession,
                                                         ViInt32   componentCarrier,
                                                         ViInt32   cellID)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, componentCarrier, 1, 5),
            2, "Component Carrier");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "cc%ld", componentCarrier);

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_LTE_UPLINK_PUCCH_CELL_ID, cellID),
            3, "Cell ID");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the (cell) Identity value of the PUCCH.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR componentCarrier/This control sets the component carrier.
/// HIPAR cellIDValue/Sets the identity value for the PUCCH.
ViStatus _VI_FUNC rsspecan_ConfigureLTEUplinkPUCCHCellIDValue(
                            ViSession instrSession,
                            ViInt32   componentCarrier,
                            ViInt32   cellIDValue)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, componentCarrier, 1, 5),
            2, "Component Carrier");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "cc%ld", componentCarrier);

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_LTE_UPLINK_PUCCH_CELL_ID_VALUE, cellIDValue),
            3, "Cell ID Value");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the PUCCH structure.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR componentCarrier/This control sets the component carrier.
/// HIPAR PRACHConfiguration/Selects the PRACH preamble format.
/// HIPAR restrictedSet/Turns the restricted preamble set for PRACH on and off.
/// HIPAR frequencyOffset/Defines the PRACH frequency offset. The command is available for
/// HIPAR frequencyOffset/preamble formats 0 to 3.
/// HIPAR ncsConfiguration/Defines the Ncs configuration for the PRACH.
/// HIPAR logicalRootSequenceIndex/Defines the PRACH logical root sequence index.
/// HIPAR sequenceIndex/Selects whether the PRACH sequence index will be determined
/// HIPAR sequenceIndex/automatically or manually. In such case use Sequence Index Value
/// HIPAR sequenceIndex/control to set the index.
/// HIPAR sequenceIndexValue/Selects the PRACH sequence index.
ViStatus _VI_FUNC rsspecan_ConfigureLTEUplinkPRACHStructure(ViSession instrSession,
                                                            ViInt32   componentCarrier,
                                                            ViInt32   PRACHConfiguration,
                                                            ViBoolean restrictedSet,
                                                            ViInt32   frequencyOffset,
                                                            ViInt32   ncsConfiguration,
                                                            ViInt32   logicalRootSequenceIndex,
                                                            ViInt32   sequenceIndex,
                                                            ViInt32   sequenceIndexValue)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_LTE_UPLINK_PRACH_CONFIGURATION,
            "cc", componentCarrier - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Component Carrier");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_LTE_UPLINK_PRACH_CONFIGURATION, PRACHConfiguration),
            3, "PRACH Configuration");

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, repCap, RSSPECAN_ATTR_LTE_UPLINK_PRACH_RESTRICTED_SET, restrictedSet),
            4, "Restricted Set");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_LTE_UPLINK_PRACH_FREQUENCY_OFFSET, frequencyOffset),
            5, "Frequency Offset");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_LTE_UPLINK_PRACH_NCS_CONFIGURATION, ncsConfiguration),
            6, "Ncs Configuration");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_LTE_UPLINK_PRACH_LOGICAL_ROOT_SEQ_INDEX, logicalRootSequenceIndex),
            7, "Logical Root SequenceIndex");

    if (sequenceIndex == RSSPECAN_VAL_LTE_UPLINK_PRACH_SEQ_INDEX_AUTO)
    {
        viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_LTE_UPLINK_PRACH_SEQUENCE_INDEX, sequenceIndex),
                8, "Sequence Index");
    }
    else
    {
        viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_LTE_UPLINK_PRACH_SEQUENCE_INDEX_VALUE, sequenceIndexValue),
                9, "Sequence Index Value");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the PRACH preamble mapping.
/// HIFN  RSSPECAN_ATTR_MARKER_TRACE or function rsspecan_ConfigureMarker.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR componentCarrier/This control sets the component carrier.
/// HIPAR automaticPreambleMapping/Turns automatic preamble mapping for the PRACH on and off.
/// HIPAR frequencyIndex/Selects the PRACH frequency index.
/// HIPAR halfFrameIndicator/Defines the PRACH half frame indicator.
ViStatus _VI_FUNC rsspecan_ConfigureLTEUplinkPRACHPreambleMapping(
                            ViSession instrSession,
                            ViInt32   componentCarrier,
                            ViBoolean automaticPreambleMapping,
                            ViInt32   frequencyIndex,
                            ViInt32   halfFrameIndicator)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_LTE_UPLINK_PRACH_AUTOMATIC_PREAMBLE_MAPPING,
            "cc", componentCarrier - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Component Carrier");

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, repCap, RSSPECAN_ATTR_LTE_UPLINK_PRACH_AUTOMATIC_PREAMBLE_MAPPING, automaticPreambleMapping),
            3, "Automatic Preamble Mapping");

    if (automaticPreambleMapping == VI_FALSE)
    {
        viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_LTE_UPLINK_PRACH_FREQUENCY_INDEX, frequencyIndex),
                4, "Frequency Index");

        viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_LTE_UPLINK_PRACH_HALF_FRAME_INDICATOR, halfFrameIndicator),
                5, "Half Frame Indicator");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN Configures the PUCCH number of resouce blocks automatically.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsspecan_ConfigureLTEUplinkPUCCHResourceBlocksAuto(
                            ViSession instrSession,
                            ViInt32   componentCarrier)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_LTE_UPLINK_PUCCH_RESOURCE_BLOCKS_AUTO,
            "cc", componentCarrier - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Component Carrier");

    checkErr(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_LTE_UPLINK_PUCCH_RESOURCE_BLOCKS_AUTO, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the settings that control the data capture.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR captureTime/Defines the capture time.
/// HIPAR overallFrameCount/Turns manual selection of the number of frames you want to analyze on
/// HIPAR overallFrameCount/and off.
/// HIPAR numberOfFramesToAnalyzer/Sets the number of frames you want to analyze.
/// HIPAR autoAccordingtoStandard/Turns automatic selection of the number of frames to analyze on and
/// HIPAR autoAccordingtoStandard/off.
/// HIPAR swapI_Q/Activates or deactivates the inverted I/Q modulation. If the I and Q
/// HIPAR swapI_Q/parts of the signal from the DUT are interchanged, the instrument can
/// HIPAR swapI_Q/do the same to compensate for it.
ViStatus _VI_FUNC rsspecan_ConfigureLTEUplinkSignalCapture(ViSession instrSession,
                                                           ViReal64  captureTime,
                                                           ViBoolean overallFrameCount,
                                                           ViBoolean autoAccordingtoStandard,
                                                           ViInt32   numberOfFramesToAnalyzer,
                                                           ViBoolean swapI_Q)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_SWEEP_TIME, captureTime),
            2, "Capture Time");

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_LTE_DOWNLINK_OVERALL_FRAME_COUNT, overallFrameCount),
            3, "Overall Frame Count");

    if (overallFrameCount == VI_TRUE)
    {
        viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_LTE_DOWNLINK_AUTO_COUNT_ACCORDING_TO_STANDARD, autoAccordingtoStandard),
                4, "Auto According to Standard");

        if (autoAccordingtoStandard == VI_FALSE)
        {
            viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_LTE_DOWNLINK_NUMBER_OF_FRAMES_TO_ANALYZE, numberOfFramesToAnalyzer),
                    5, "Number Of Frames To Analyzer");
        }
    }
    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_BB_SWAP_IQ, swapI_Q),
            6, "Swap I/Q");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures phase and timing tracking.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR phaseTracking/Configures the phase tracking type.
/// HIPAR timing/Specifies whether or not the measurement results should be compensated
/// HIPAR timing/for timing error. When timing compensation is used, the measurement
/// HIPAR timing/results will be compensated for timing error on a per-symbol basis.
ViStatus _VI_FUNC rsspecan_ConfigureLTEUplinkTracking(ViSession instrSession,
                                                      ViInt32   phaseTracking,
                                                      ViBoolean timing)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_LTE_UPLINK_PHASE_TRACKING, phaseTracking),
            2, "Phase Tracking");

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_LTE_UPLINK_TIMING, timing),
            3, "Timing");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures signal demodulation.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR analysisMode/This control selects the uplink analysis mode.
/// HIPAR channelEstimation/Configures the channel estimation type for uplink.
/// HIPAR evmWithExclusionPeriod/Turns exclusion periods for EVM measurements as defined in 3GPP TS
/// HIPAR evmWithExclusionPeriod/36.521 on and off.
/// HIPAR compensateDCOffset/Activates or deactivates DC offset compensation when calculating
/// HIPAR compensateDCOffset/measurement results.
/// HIPAR scramblingofCodedBits/Activates or deactivates scrambling of coded bits for UL.
/// HIPAR suppressedInterferenceSync/Specifies whether or not the suppressed interference synchronization
/// HIPAR suppressedInterferenceSync/feature shall be used. If this synchronization mode is enabled, the
/// HIPAR suppressedInterferenceSync/synchronization on signals containing more than one user equipment (UE)
/// HIPAR suppressedInterferenceSync/is more robust. Additionally, the EVM is lower in case the UEs have
/// HIPAR suppressedInterferenceSync/different frequency offsets. Note that Auto Demodulation is not
/// HIPAR suppressedInterferenceSync/supported in this synchronization mode and the EVM may be higher in
/// HIPAR suppressedInterferenceSync/case only one UE is present in the signal.
/// HIPAR multicarrierFilter/Turns the suppression of interference of neighboring carriers on and
/// HIPAR multicarrierFilter/off (e.g. LTE, WCDMA, GSM etc).
ViStatus _VI_FUNC rsspecan_ConfigureLTEUplinkDemodulation(ViSession instrSession,
                                                          ViInt32   analysisMode,
                                                          ViInt32   channelEstimation,
                                                          ViBoolean evmWithExclusionPeriod,
                                                          ViBoolean reserved,
                                                          ViBoolean scramblingofCodedBits,
                                                          ViBoolean suppressedInterferenceSync,
                                                          ViBoolean multicarrierFilter)
{
    ViStatus error = VI_SUCCESS;
             reserved;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_LTE_UPLINK_ANALYSIS_MODE, analysisMode),
            2, "Analysis Mode");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_LTE_UPLINK_CHANNEL_ESTIMATION, channelEstimation),
            3, "Channel Estimation");

    if (rsspecan_IsFSV(instrSession) == VI_FALSE)
    {
        viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_LTE_UPLINK_EVM_WITH_EXCLUSION_PERIOD, evmWithExclusionPeriod),
                4, "EVM With Exclusion Period");
    }

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_LTE_UPLINK_SCRAMBLING_OF_CODED_BITS, scramblingofCodedBits),
            6, "Scrambling of Coded Bits");

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_LTE_UPLINK_SUPPRESSED_INTERFERENCE_SYNC, suppressedInterferenceSync),
            7, "Suppressed Interference Sync");

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_LTE_UPLINK_MULTICARRIER_FILTER, multicarrierFilter),
            8, "Multicarrier Filter");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Includes or excludes the transient slots present after a switch from transientSlots/downlink to uplink in the analysis.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR transientSlots/Includes or excludes the transient slots present after a switch from transientSlots/downlink to uplink in the analysis.
ViStatus _VI_FUNC rsspecan_ConfigureLTEUplinkTransientSlots(ViSession instrSession,
                                                            ViBoolean transientSlots)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_LTE_UPLINK_TRANSIENT_SLOTS, transientSlots),
            2, "Transient Slots");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures Time Alignment Error measurements with carrier aggregation.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR carrierAggregationState/This control turns carrier aggregation for Time Alignment measurements
/// HIPAR carrierAggregationState/on and off. You can select the number of component carriers with
/// HIPAR carrierAggregationState/RSSPECAN_ATTR_LTE_UPLINK_COMPONENT_CARRIERS.
/// HIPAR numberofCarriers/This control selects the number of component carriers evaluated in the
/// HIPAR numberofCarriers/Time Alignment measurement.
ViStatus _VI_FUNC rsspecan_ConfigureLTEUplinkCarrierAggregation(
                            ViSession instrSession,
                            ViBoolean carrierAggregationState,
                            ViInt32   numberofCarriers)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_LTE_UPLINK_COMPONENT_CARRIERS, numberofCarriers),
            3, "Number of Carriers");

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_LTE_UPLINK_CARRIER_AGGREGATION, carrierAggregationState),
            2, "Carrier Aggregation State");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures frequency sweep measurements.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR assumedAdjChannelCarrier/Selects the ACLR Assumed Adj. Channel Carrier in the general settings
/// HIPAR assumedAdjChannelCarrier/menu.
/// HIPAR semRequirement/Selects the requirements for a spectrum emission mask.
ViStatus _VI_FUNC rsspecan_ConfigureLTEUplinkFrequencySweepMeasurements(
                            ViSession instrSession,
                            ViInt32   assumedAdjChannelCarrier,
                            ViInt32   semRequirement)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_LTE_UPLINK_ACLR_ASSUMED_ADJ_CHANNEL_CARRIER, assumedAdjChannelCarrier),
            2, "Assumed Adj Channel Carrier");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_LTE_UPLINK_SEM_REQUIREMENT, semRequirement),
            3, "SEM Requirement");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the results of the CCDF.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR trace/Selects the trace.
/// HIPAR probability/Selects the probability.
/// HIPAR result/Returns the results of the CCDF.
ViStatus _VI_FUNC rsspecan_QueryLTEUplinkCCDFResults(ViSession instrSession,
                                                     ViInt32   trace,
                                                     ViInt32   probability,
                                                     ViReal64* result)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViChar*  p2string;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, trace, 1, 6),
            2, "Trace");
    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &rsspecan_rngMeasStatProbability, probability, &p2string),
                3, "Probability");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, result),
                4, "Result");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CALC:STAT:CCDF:X%ld? %s", trace, p2string);
    checkErr(RsCore_QueryViReal64(instrSession, cmd, result));

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function adds a window to the display.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR stringSize/This control sets the allocated length of Returned Window Name string.
/// HIPAR referenceWindowName/String containing the number or name of the window the new window is
/// HIPAR referenceWindowName/inserted next to. In the default state, the name of the window is its
/// HIPAR referenceWindowName/index.
/// HIPAR direction/Direction the window is added relative to the window set with "Reference Window
/// HIPAR direction/Name".
/// HIPAR windowType/Type of result display you want to add.
/// HIPAR returnedWindowName/When adding a new window, the function returns its name (by
ViStatus _VI_FUNC rsspecan_LTEUplinkAddWindow(ViSession instrSession,
                                              ViInt32   stringSize,
                                              ViString  referenceWindowName,
                                              ViInt32   direction,
                                              ViInt32   windowType,
                                              ViChar    returnedWindowName[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViChar   response[RS_MAX_MESSAGE_BUF_SIZE];
    ViChar*  p2direction;
    ViChar*  p2type;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &rsspecan_rngLayoutDirection, direction, &p2direction),
                4, "Direction");
    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &rsspecan_rngLTEUplinkLayoutType, windowType, &p2type),
                5, "Window Type");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "LAY:ADD? '%s',%s,%s", referenceWindowName, p2direction, p2type);
    checkErr(RsCore_QueryViString(instrSession, cmd, response));

    if (returnedWindowName != NULL)
    {
        RsCore_TrimString(response, RS_VAL_TRIM_WHITESPACES_AND_ALL_QUOTES);
        RsCore_CopyToUserBufferAsciiData(returnedWindowName, stringSize, response);
    }

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function replaces a window with another one.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR windowName/String containing the number or name of the window you want to
/// HIPAR windowType/Type of result display you want to add.
ViStatus _VI_FUNC rsspecan_LTEUplinkReplaceWindow(ViSession instrSession,
                                                  ViString  windowName,
                                                  ViInt32   windowType)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViChar*  p2type;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &rsspecan_rngLTEUplinkLayoutType, windowType, &p2type),
                3, "Window Type");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "LAY:REPL '%s',%s", windowName, p2type);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Selects a tab in diagrams with multiple subwindows (or views). Note
/// HIFN  that selecting a tab does not actually select a subwindow. To select a
/// HIFN  subwindow, for example to query the results of a subwindow, use
/// HIFN  RSSPECAN_ATTR_DISPLAY_FOCUS_AREA.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR tab/Selects the tab.
ViStatus _VI_FUNC rsspecan_LTEUplinkSelectTab(ViSession instrSession,
                                              ViInt32   window,
                                              ViInt32   tab)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_tab[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_LTE_UPLINK_TAB_SELECT,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_LTE_UPLINK_TAB_SELECT,
            "Tab", tab - 1, RS_REPCAP_BUF_SIZE, repCap_tab));

    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_tab, NULL);

    checkErr(rsspecan_SetAttributeViString(instrSession, repCap, RSSPECAN_ATTR_LTE_UPLINK_TAB_SELECT, NULL));

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function selects the measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR measurement/This control selects the measurement.
ViStatus _VI_FUNC rsspecan_ConfigureLTEUplinkMeasurement(ViSession instrSession,
                                                         ViInt32   measurement)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_LTE_UPLINK_MEASUREMENT, measurement),
            2, "Measurement");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function automatically detects several signal characteristics and selects the appropriate parameters in the application.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsspecan_ConfigureLTEUplinkMeasurementSignalDetection(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_LTE_UPLINK_MEASUREMENT_SIGNAL_DETECTION, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function queries the current synchronization state.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR componentCarrier/This control selects the component carrier to be used.
/// HIPAR synchronization/Returns the coarse symbol timing.
ViStatus _VI_FUNC rsspecan_QueryLTEUplinkMeasurementSynchronizationState(
                            ViSession  instrSession,
                            ViInt32    componentCarrier,
                            ViBoolean* synchronization)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_LTE_UPLINK_CC_SYNCHRONIZATION,
            "cc", componentCarrier - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Component Carrier");

    viCheckParm(rsspecan_GetAttributeViBoolean(instrSession, repCap, RSSPECAN_ATTR_LTE_UPLINK_CC_SYNCHRONIZATION, synchronization),
            2, "Synchronization");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries LTE Uplink measurement result summary.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR componentCarrier/This control selects the component carrier to be used.
/// HIPAR frameResult/Selects result type.
/// HIPAR measurementType/Selects measurement type.
/// HIPAR result/Returns selected result. Depending on the EVM unit selected in the
/// HIPAR result/"General Settings" menu the value will be either in dB or in percent.
ViStatus _VI_FUNC rsspecan_QueryLTEUplinkMeasurementResultSummary(
                            ViSession instrSession,
                            ViInt32   componentCarrier,
                            ViInt32   frameResult,
                            ViInt32   measurementType,
                            ViReal64* result)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViAttr   attr[11] = {
             RSSPECAN_ATTR_LTE_UPLINK_EVM_PUSCH_QPSK_RESULT,
             RSSPECAN_ATTR_LTE_UPLINK_EVM_PUSCH_16QAM_RESULT,
             RSSPECAN_ATTR_LTE_UPLINK_EVM_PUSCH_64QAM_RESULT,
             RSSPECAN_ATTR_LTE_UPLINK_EVM_DMRS_PUSCH_QPSK_RESULT,
             RSSPECAN_ATTR_LTE_UPLINK_EVM_DMRS_PUSCH_16QAM_RESULT,
             RSSPECAN_ATTR_LTE_UPLINK_EVM_DMRS_PUSCH_64QAM_RESULT,
             RSSPECAN_ATTR_LTE_UPLINK_EVM_DMRS_PUCCH_RESULT,
             RSSPECAN_ATTR_LTE_UPLINK_EVM_PUCCH_RESULT,
             RSSPECAN_ATTR_LTE_UPLINK_EVM_PRACH_RESULT,
             RSSPECAN_ATTR_LTE_UPLINK_EVM_PUSCH_256QAM_RESULT,
             RSSPECAN_ATTR_LTE_UPLINK_EVM_DMRS_PUSCH_256QAM_RESULT
             };
             componentCarrier;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, componentCarrier, 1, 5),
            2, "Component Carrier");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, frameResult, RSSPECAN_VAL_LTE_UPLINK_PUSCH_QPSK, RSSPECAN_VAL_LTE_UPLINK_DMRS_PUSCH_256QAM),
            3, "Frame result");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, measurementType, RSSPECAN_VAL_MEASTYPE_AVER, RSSPECAN_VAL_MEASTYPE_AVER),
            4, "Measurement Type");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "cc%ld", componentCarrier);

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, repCap, attr[frameResult], result),
            5, "Result");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries LTE Uplink measurement limit check results.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR componentCarrier/This control selects the component carrier to be used.
/// HIPAR frameResult/Selects result type.
/// HIPAR average/Returns the limit check result.
ViStatus _VI_FUNC rsspecan_QueryLTEUplinkMeasurementLimitCheckResult(
                            ViSession instrSession,
                            ViInt32   componentCarrier,
                            ViInt32   frameResult,
                            ViInt32*  average)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, componentCarrier, 1, 5),
            2, "Component Carrier");

    switch (frameResult)
    {
    case RSSPECAN_VAL_LTE_UPLINK_PUSCH_QPSK:

        viCheckParm(rsspecan_GetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_LTE_UPLINK_EVM_PUSCH_QPSK_LIMIT_CHECK_RESULT, average),
                4, "Average");

        break;

    case RSSPECAN_VAL_LTE_UPLINK_PUSCH_16QAM:

        viCheckParm(rsspecan_GetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_LTE_UPLINK_EVM_PUSCH_16QAM_LIMIT_CHECK_RESULT, average),
                4, "Average");

        break;

    case RSSPECAN_VAL_LTE_UPLINK_PUSCH_64QAM:

        viCheckParm(rsspecan_GetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_LTE_UPLINK_EVM_PUSCH_64QAM_LIMIT_CHECK_RESULT, average),
                4, "Average");

        break;

    case RSSPECAN_VAL_LTE_UPLINK_DMRS_PUSCH_QPSK:

        viCheckParm(rsspecan_GetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_LTE_UPLINK_EVM_DMRS_PUSCH_QPSK_LIMIT_CHECK_RESULT, average),
                4, "Average");

        break;

    case RSSPECAN_VAL_LTE_UPLINK_DMRS_PUSCH_16QAM:

        viCheckParm(rsspecan_GetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_LTE_UPLINK_EVM_DMRS_PUSCH_16QAM_LIMIT_CHECK_RESULT, average),
                4, "Average");

        break;

    case RSSPECAN_VAL_LTE_UPLINK_DMRS_PUSCH_64QAM:

        viCheckParm(rsspecan_GetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_LTE_UPLINK_EVM_DMRS_PUSCH_64QAM_LIMIT_CHECK_RESULT, average),
                4, "Average");

        break;

    case RSSPECAN_VAL_LTE_UPLINK_DMRS_PUCCH:

        viCheckParm(rsspecan_GetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_LTE_UPLINK_EVM_DMRS_PUCCH_LIMIT_CHECK_RESULT, average),
                4, "Average");

        break;

    case RSSPECAN_VAL_LTE_UPLINK_PUCCH:

        viCheckParm(rsspecan_GetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_LTE_UPLINK_EVM_PUCCH_LIMIT_CHECK_RESULT, average),
                4, "Average");

        break;

    case RSSPECAN_VAL_LTE_UPLINK_PRACH:

        viCheckParm(rsspecan_GetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_LTE_UPLINK_EVM_PRACH_LIMIT_CHECK_RESULT, average),
                4, "Average");

        break;

    case RSSPECAN_VAL_LTE_UPLINK_PUSCH_256QAM:

        viCheckParm(rsspecan_GetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_LTE_UPLINK_EVM_PUSCH_256QAM_LIMIT_CHECK_RESULT, average),
                4, "Average");

        break;

    case RSSPECAN_VAL_LTE_UPLINK_DMRS_PUSCH_256QAM:

        viCheckParm(rsspecan_GetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_LTE_UPLINK_EVM_DMRS_PUSCH_256QAM_LIMIT_CHECK_RESULT, average),
                4, "Average");

        break;

    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, frameResult), 4, "Frame Result");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries LTE Uplink measurement EVM All result.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR componentCarrier/This control sets the component carrier.
/// HIPAR measurementType/Selects measurement type.
/// HIPAR result/Returns selected result. Depending on the EVM unit selected in the
/// HIPAR result/"General Settings" menu the value will be either in dB or in percent.
ViStatus _VI_FUNC rsspecan_QueryLTEUplinkMeasurementEVMAll(ViSession instrSession,
                                                           ViInt32   componentCarrier,
                                                           ViInt32   measurementType,
                                                           ViReal64* result)
{
    ViStatus                                     error = VI_SUCCESS;
    ViAttr                                       attr[4] = {
                                                 RSSPECAN_ATTR_LTE_UPLINK_EVM_ALL_RESULT_MIN,
    RSSPECAN_ATTR_LTE_UPLINK_EVM_ALL_RESULT_MAX, 0,
                                                 RSSPECAN_ATTR_LTE_UPLINK_EVM_ALL_RESULT_AVERAGE
                                                 };
    ViChar                                       repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, measurementType, RSSPECAN_VAL_MEASTYPE_MIN, RSSPECAN_VAL_MEASTYPE_AVER),
            3, "Measurement Type");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_LTE_UPLINK_EVM_ALL_RESULT_MIN,
            "cc", componentCarrier - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Component Carrier");

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, repCap,
                    attr[measurementType - 1],
                    result), 4, "Result");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the results of the EVM limit check of all resource elements.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR componentCarrier/This control sets the component carrier.
/// HIPAR average/Returns the average limit check result.
/// HIPAR maximum/Returns the maximum limit check result.
ViStatus _VI_FUNC rsspecan_QueryLTEUplinkMeasurementLimitCheckEVMAll(
                            ViSession instrSession,
                            ViInt32   componentCarrier,
                            ViInt32*  average,
                            ViInt32*  maximum)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, componentCarrier, 1, 5),
            2, "Component Carrier");

    viCheckParm(rsspecan_GetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_LTE_UPLINK_EVM_AL_LIMIT_CHECK_RESULT_AVERAGE, average),
            3, "Average");

    viCheckParm(rsspecan_GetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_LTE_UPLINK_EVM_AL_LIMIT_CHECK_RESULT_MAX, maximum),
            4, "Maximum");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the "EVM Physical Channel" value from the result summary list.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR componentCarrier/This control sets the component carrier.
/// HIPAR measurementType/Selects measurement type.
/// HIPAR result/Returns selected result. Depending on the EVM unit selected in the
/// HIPAR result/"General Settings" menu the value will be either in dB or in percent.
ViStatus _VI_FUNC rsspecan_QueryLTEUplinkMeasurementEVMPhysicalChannel(
                            ViSession instrSession,
                            ViInt32   componentCarrier,
                            ViInt32   measurementType,
                            ViReal64* result)
{
    ViStatus                                                  error = VI_SUCCESS;
    ViAttr                                                    attr[4] = {
                                                              RSSPECAN_ATTR_LTE_UPLINK_EVM_PHYSICAL_CHANNEL_RESULT_MIN,
    RSSPECAN_ATTR_LTE_UPLINK_EVM_PHYSICAL_CHANNEL_RESULT_MAX, 0,
                                                              RSSPECAN_ATTR_LTE_UPLINK_EVM_PHYSICAL_CHANNEL_RESULT_AVERAGE
                                                              };
    ViChar                                                    repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, measurementType, RSSPECAN_VAL_MEASTYPE_MIN, RSSPECAN_VAL_MEASTYPE_AVER),
            3, "Measurement Type");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_LTE_UPLINK_EVM_PHYSICAL_CHANNEL_RESULT_MIN,
            "cc", componentCarrier - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Component Carrier");

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, repCap,
                    attr[measurementType - 1],
                    result), 4, "Result");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the results of the EVM limit check of all physical channel resource elements.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR componentCarrier/This control sets the component carrier.
/// HIPAR average/Returns the average limit check result.
/// HIPAR maximum/Returns the maximum limit check result.
ViStatus _VI_FUNC rsspecan_QueryLTEUplinkMeasurementEVMPhysicalChannelLimitCheck(
                            ViSession instrSession,
                            ViInt32   componentCarrier,
                            ViInt32*  average,
                            ViInt32*  maximum)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, componentCarrier, 1, 5),
            2, "Component Carrier");

    viCheckParm(rsspecan_GetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_LTE_UPLINK_EVM_PHYSICAL_CHANNEL_LIMIT_CHECK_RESULT_AVERAGE, average),
            3, "Average");

    viCheckParm(rsspecan_GetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_LTE_UPLINK_EVM_PHYSICAL_CHANNEL_LIMIT_CHECK_RESULT_MAX, maximum),
            4, "Maximum");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the "EVM Physical Signal" value from the result summary list.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR componentCarrier/This control sets the component carrier.
/// HIPAR measurementType/Selects measurement type.
/// HIPAR result/Returns selected result. Depending on the EVM unit selected in the
/// HIPAR result/"General Settings" menu the value will be either in dB or in percent.
ViStatus _VI_FUNC rsspecan_QueryLTEUplinkMeasurementEVMPhysicalSignal(
                            ViSession instrSession,
                            ViInt32   componentCarrier,
                            ViInt32   measurementType,
                            ViReal64* result)
{
    ViStatus                                                 error = VI_SUCCESS;
    ViAttr                                                   attr[4] = {
                                                             RSSPECAN_ATTR_LTE_UPLINK_EVM_PHYSICAL_SIGNAL_RESULT_MIN,
    RSSPECAN_ATTR_LTE_UPLINK_EVM_PHYSICAL_SIGNAL_RESULT_MAX, 0,
                                                             RSSPECAN_ATTR_LTE_UPLINK_EVM_PHYSICAL_SIGNAL_RESULT_AVERAGE
                                                             };
    ViChar                                                   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, measurementType, RSSPECAN_VAL_MEASTYPE_MIN, RSSPECAN_VAL_MEASTYPE_AVER),
            3, "Measurement Type");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_LTE_UPLINK_EVM_PHYSICAL_SIGNAL_RESULT_MIN,
            "cc", componentCarrier - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Component Carrier");

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, repCap,
                    attr[measurementType - 1],
                    result), 4, "Result");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the results of the EVM limit check of all physical signal resource elements.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR componentCarrier/This control sets the component carrier.
/// HIPAR average/Returns the average limit check result.
/// HIPAR maximum/Returns the maximum limit check result.
ViStatus _VI_FUNC rsspecan_QueryLTEUplinkMeasurementEVMPhysicalSignalLimitCheck(
                            ViSession instrSession,
                            ViInt32   componentCarrier,
                            ViInt32*  average,
                            ViInt32*  maximum)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, componentCarrier, 1, 5),
            2, "Component Carrier");

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_LTE_UPLINK_EVM_PHYSICAL_SIGNAL_LIMIT_CHECK_RESULT_AVERAGE,
            "cc", componentCarrier - 1, RS_REPCAP_BUF_SIZE, repCap));

    viCheckParm(rsspecan_GetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_LTE_UPLINK_EVM_PHYSICAL_SIGNAL_LIMIT_CHECK_RESULT_AVERAGE, average),
            3, "Average");

    viCheckParm(rsspecan_GetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_LTE_UPLINK_EVM_PHYSICAL_SIGNAL_LIMIT_CHECK_RESULT_MAX, maximum),
            4, "Maximum");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries "Frequency Error" value from the result summary list in Hz.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR componentCarrier/This control sets the component carrier.
/// HIPAR measurementType/Selects measurement type.
/// HIPAR result/Returns selected result.
ViStatus _VI_FUNC rsspecan_QueryLTEUplinkMeasurementFrequencyError(
                            ViSession instrSession,
                            ViInt32   componentCarrier,
                            ViInt32   measurementType,
                            ViReal64* result)
{
    ViStatus                                             error = VI_SUCCESS;
    ViAttr                                               attr[4] = {
                                                         RSSPECAN_ATTR_LTE_UPLINK_FREQUENCY_ERROR_RESULT_MIN,
    RSSPECAN_ATTR_LTE_UPLINK_FREQUENCY_ERROR_RESULT_MAX, 0,
                                                         RSSPECAN_ATTR_LTE_UPLINK_FREQUENCY_ERROR_RESULT_AVERAGE
                                                         };
    ViChar                                               repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, measurementType, RSSPECAN_VAL_MEASTYPE_MIN, RSSPECAN_VAL_MEASTYPE_AVER),
            3, "Measurement Type");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_LTE_UPLINK_FREQUENCY_ERROR_RESULT_MIN,
            "cc", componentCarrier - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Component Carrier");

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, repCap,
                    attr[measurementType - 1],
                    result), 4, "Result");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the result of the frequency error limit check.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR componentCarrier/This control sets the component carrier.
/// HIPAR average/Returns the average limit check result.
/// HIPAR maximum/Returns the maximum limit check result.
ViStatus _VI_FUNC rsspecan_QueryLTEUplinkMeasurementFrequencyErrorLimitCheck(
                            ViSession instrSession,
                            ViInt32   componentCarrier,
                            ViInt32*  average,
                            ViInt32*  maximum)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, componentCarrier, 1, 5),
            2, "Component Carrier");

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_LTE_UPLINK_FREQUENCY_ERROR_LIMIT_CHECK_RESULT_AVERAGE,
            "cc", componentCarrier - 1, RS_REPCAP_BUF_SIZE, repCap));

    viCheckParm(rsspecan_GetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_LTE_UPLINK_FREQUENCY_ERROR_LIMIT_CHECK_RESULT_AVERAGE, average),
            3, "Average");

    viCheckParm(rsspecan_GetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_LTE_UPLINK_FREQUENCY_ERROR_LIMIT_CHECK_RESULT_MAX, maximum),
            4, "Maximum");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries "Sampling Error" value from the result summary list in ppm.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR componentCarrier/This control sets the component carrier.
/// HIPAR measurementType/Selects measurement type.
/// HIPAR result/Returns selected result.
ViStatus _VI_FUNC rsspecan_QueryLTEUplinkMeasurementSamplingError(
                            ViSession instrSession,
                            ViInt32   componentCarrier,
                            ViInt32   measurementType,
                            ViReal64* result)
{
    ViStatus                                            error = VI_SUCCESS;
    ViAttr                                              attr[4] = {
                                                        RSSPECAN_ATTR_LTE_UPLINK_SAMPLING_ERROR_RESULT_MIN,
    RSSPECAN_ATTR_LTE_UPLINK_SAMPLING_ERROR_RESULT_MAX, 0,
                                                        RSSPECAN_ATTR_LTE_UPLINK_SAMPLING_ERROR_RESULT_AVERAGE
                                                        };
    ViChar                                              repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, measurementType, RSSPECAN_VAL_MEASTYPE_MIN, RSSPECAN_VAL_MEASTYPE_AVER),
            3, "Measurement Type");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_LTE_UPLINK_SAMPLING_ERROR_RESULT_MIN,
            "cc", componentCarrier - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Component Carrier");

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, repCap,
                    attr[measurementType - 1],
                    result), 4, "Result");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Returns the cyclic prefix type detected by the DSP kernel. If no valid
/// HIFN  value has been detected yet the attribute will return -1.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR componentCarrier/This control sets the component carrier.
/// HIPAR cyclicPrefix/Returns the cyclic prefix type detected by the DSP kernel. If no valid
/// HIPAR cyclicPrefix/value has been detected yet the attribute will return -1.
ViStatus _VI_FUNC rsspecan_QueryLTEUplinkMeasurementCyclicPrefix(
                            ViSession instrSession,
                            ViInt32   componentCarrier,
                            ViInt32*  cyclicPrefix)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, componentCarrier, 1, 5),
            2, "Component Carrier");

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_LTE_UPLINK_DETECTED_CYCLIC_PREFIX,
            "cc", componentCarrier - 1, RS_REPCAP_BUF_SIZE, repCap));

    viCheckParm(rsspecan_GetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_LTE_UPLINK_DETECTED_CYCLIC_PREFIX, cyclicPrefix),
            3, "Cyclic Prefix");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the results of the sampling error limit check.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR componentCarrier/This control sets the component carrier.
/// HIPAR average/Returns the average limit check result.
/// HIPAR maximum/Returns the maximum limit check result.
ViStatus _VI_FUNC rsspecan_QueryLTEUplinkMeasurementSamplingErrorLimitCheck(
                            ViSession instrSession,
                            ViInt32   componentCarrier,
                            ViInt32*  average,
                            ViInt32*  maximum)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, componentCarrier, 1, 5),
            2, "Component Carrier");

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_LTE_UPLINK_SAMPLING_ERROR_LIMIT_CHECK_RESULT_AVERAGE,
            "cc", componentCarrier - 1, RS_REPCAP_BUF_SIZE, repCap));

    viCheckParm(rsspecan_GetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_LTE_UPLINK_SAMPLING_ERROR_LIMIT_CHECK_RESULT_AVERAGE, average),
            3, "Average");

    viCheckParm(rsspecan_GetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_LTE_UPLINK_SAMPLING_ERROR_LIMIT_CHECK_RESULT_MAX, maximum),
            4, "Maximum");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Returns the IQ offset from the result summary list in dB.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR componentCarrier/This control sets the component carrier.
/// HIPAR measurementType/Selects measurement type.
/// HIPAR result/Returns selected result.
ViStatus _VI_FUNC rsspecan_QueryLTEUplinkMeasurementIQOffset(ViSession instrSession,
                                                             ViInt32   componentCarrier,
                                                             ViInt32   measurementType,
                                                             ViReal64* result)
{
    ViStatus                                       error = VI_SUCCESS;
    ViAttr                                         attr[4] = {
                                                   RSSPECAN_ATTR_LTE_UPLINK_IQ_OFFSET_RESULT_MIN,
    RSSPECAN_ATTR_LTE_UPLINK_IQ_OFFSET_RESULT_MAX, 0,
                                                   RSSPECAN_ATTR_LTE_UPLINK_IQ_OFFSET_RESULT_AVERAGE
                                                   };
    ViChar                                         repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, measurementType, RSSPECAN_VAL_MEASTYPE_MIN, RSSPECAN_VAL_MEASTYPE_AVER),
            3, "Measurement Type");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_LTE_UPLINK_IQ_OFFSET_RESULT_MIN,
            "cc", componentCarrier - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Component Carrier");

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, repCap,
                    attr[measurementType - 1],
                    result), 4, "Result");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the result of the I/Q offset limit check.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR componentCarrier/This control sets the component carrier.
/// HIPAR average/Returns the average limit check result.
/// HIPAR maximum/Returns the maximum limit check result.
ViStatus _VI_FUNC rsspecan_QueryLTEUplinkMeasurementIQOffsetLimitCheck(
                            ViSession instrSession,
                            ViInt32   componentCarrier,
                            ViInt32*  average,
                            ViInt32*  maximum)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, componentCarrier, 1, 5),
            2, "Component Carrier");

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_LTE_UPLINK_IQ_OFFSET_LIMIT_CHECK_RESULT_AVERAGE,
            "cc", componentCarrier - 1, RS_REPCAP_BUF_SIZE, repCap));

    viCheckParm(rsspecan_GetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_LTE_UPLINK_IQ_OFFSET_LIMIT_CHECK_RESULT_AVERAGE, average),
            3, "Average");

    viCheckParm(rsspecan_GetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_LTE_UPLINK_IQ_OFFSET_LIMIT_CHECK_RESULT_MAX, maximum),
            4, "Maximum");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Returns the IQ gain imbalance from the result summary list in dB.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR componentCarrier/This control sets the component carrier.
/// HIPAR measurementType/Selects measurement type.
/// HIPAR result/Returns selected result.
ViStatus _VI_FUNC rsspecan_QueryLTEUplinkMeasurementIQGainImbalance(
                            ViSession instrSession,
                            ViInt32   componentCarrier,
                            ViInt32   measurementType,
                            ViReal64* result)
{
    ViStatus                                               error = VI_SUCCESS;
    ViAttr                                                 attr[4] = {
                                                           RSSPECAN_ATTR_LTE_UPLINK_IQ_GAIN_IMBALANCE_RESULT_MIN,
    RSSPECAN_ATTR_LTE_UPLINK_IQ_GAIN_IMBALANCE_RESULT_MAX, 0,
                                                           RSSPECAN_ATTR_LTE_UPLINK_IQ_GAIN_IMBALANCE_RESULT_AVERAGE
                                                           };
    ViChar                                                 repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, measurementType, RSSPECAN_VAL_MEASTYPE_MIN, RSSPECAN_VAL_MEASTYPE_AVER),
            3, "Measurement Type");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_LTE_UPLINK_IQ_GAIN_IMBALANCE_RESULT_MIN,
            "cc", componentCarrier - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Component Carrier");

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, repCap,
                    attr[measurementType - 1],
                    result), 4, "Result");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the result of the gain imbalance limit check.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR componentCarrier/This control sets the component carrier.
/// HIPAR average/Returns the average limit check result.
/// HIPAR maximum/Returns the maximum limit check result.
ViStatus _VI_FUNC rsspecan_QueryLTEUplinkMeasurementIQGainImbalanceLimitCheck(
                            ViSession instrSession,
                            ViInt32   componentCarrier,
                            ViInt32*  average,
                            ViInt32*  maximum)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, componentCarrier, 1, 5),
            2, "Component Carrier");

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_LTE_UPLINK_IQ_GAIN_IMBALANCE_LIMIT_CHECK_RESULT_AVERAGE,
            "cc", componentCarrier - 1, RS_REPCAP_BUF_SIZE, repCap));

    viCheckParm(rsspecan_GetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_LTE_UPLINK_IQ_GAIN_IMBALANCE_LIMIT_CHECK_RESULT_AVERAGE, average),
            3, "Average");

    viCheckParm(rsspecan_GetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_LTE_UPLINK_IQ_GAIN_IMBALANCE_LIMIT_CHECK_RESULT_MAX, maximum),
            4, "Maximum");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Returns the IQ quadrature error from the result summary list in
/// HIFN  degrees.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR componentCarrier/This control sets the component carrier.
/// HIPAR measurementType/Selects measurement type.
/// HIPAR result/Returns selected result.
ViStatus _VI_FUNC rsspecan_QueryLTEUplinkMeasurementIQQuadratureError(
                            ViSession instrSession,
                            ViInt32   componentCarrier,
                            ViInt32   measurementType,
                            ViReal64* result)
{
    ViStatus                                                 error = VI_SUCCESS;
    ViAttr                                                   attr[4] = {
                                                             RSSPECAN_ATTR_LTE_UPLINK_IQ_QUADRATURE_ERROR_RESULT_MIN,
    RSSPECAN_ATTR_LTE_UPLINK_IQ_QUADRATURE_ERROR_RESULT_MAX, 0,
                                                             RSSPECAN_ATTR_LTE_UPLINK_IQ_QUADRATURE_ERROR_RESULT_AVERAGE
                                                             };
    ViChar                                                   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, measurementType, RSSPECAN_VAL_MEASTYPE_MIN, RSSPECAN_VAL_MEASTYPE_AVER),
            3, "Measurement Type");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_LTE_UPLINK_IQ_QUADRATURE_ERROR_RESULT_MIN,
            "cc", componentCarrier - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Component Carrier");

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, repCap,
                    attr[measurementType - 1],
                    result), 4, "Result");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the result of the quadrature error limit check.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR componentCarrier/This control sets the component carrier.
/// HIPAR average/Returns the average limit check result.
/// HIPAR maximum/Returns the maximum limit check result.
ViStatus _VI_FUNC rsspecan_QueryLTEUplinkMeasurementIQQuadratureErrorLimitCheck(
                            ViSession instrSession,
                            ViInt32   componentCarrier,
                            ViInt32*  average,
                            ViInt32*  maximum)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, componentCarrier, 1, 5),
            2, "Component Carrier");

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_LTE_UPLINK_IQ_QUADRATURE_ERROR_LIMIT_CHECK_RESULT_AVERAGE,
            "cc", componentCarrier - 1, RS_REPCAP_BUF_SIZE, repCap));

    viCheckParm(rsspecan_GetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_LTE_UPLINK_IQ_QUADRATURE_ERROR_LIMIT_CHECK_RESULT_AVERAGE, average),
            3, "Average");

    viCheckParm(rsspecan_GetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_LTE_UPLINK_IQ_QUADRATURE_ERROR_LIMIT_CHECK_RESULT_MAX, maximum),
            4, "Maximum");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Returns the "Frame Power" from the result summary list in dBm.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR componentCarrier/This control sets the component carrier.
/// HIPAR measurementType/Selects measurement type.
/// HIPAR result/Returns selected result.
ViStatus _VI_FUNC rsspecan_QueryLTEUplinkMeasurementFramePower(
                            ViSession instrSession,
                            ViInt32   componentCarrier,
                            ViInt32   measurementType,
                            ViReal64* result)
{
    ViStatus                                         error = VI_SUCCESS;
    ViAttr                                           attr[4] = {
                                                     RSSPECAN_ATTR_LTE_UPLINK_FRAME_POWER_RESULT_MIN,
    RSSPECAN_ATTR_LTE_UPLINK_FRAME_POWER_RESULT_MAX, 0,
                                                     RSSPECAN_ATTR_LTE_UPLINK_FRAME_POWER_RESULT_AVERAGE
                                                     };
    ViChar                                           repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, measurementType, RSSPECAN_VAL_MEASTYPE_MIN, RSSPECAN_VAL_MEASTYPE_AVER),
            3, "Measurement Type");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_LTE_UPLINK_FRAME_POWER_RESULT_MIN,
            "cc", componentCarrier - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Component Carrier");

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, repCap,
                    attr[measurementType - 1],
                    result), 4, "Result");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Returns the crest factor from the result summary list in dB.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR componentCarrier/This control sets the component carrier.
/// HIPAR measurementType/Selects measurement type.
/// HIPAR result/Returns the crest factor from the result summary list in dB.
ViStatus _VI_FUNC rsspecan_QueryLTEUplinkMeasurementCrestFactor(
                            ViSession instrSession,
                            ViInt32   componentCarrier,
                            ViInt32   measurementType,
                            ViReal64* result)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
             measurementType;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_LTE_UPLINK_CREST_FACTOR_RESULT,
            "cc", componentCarrier - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Component Carrier");

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_LTE_UPLINK_CREST_FACTOR_RESULT, result),
            4, "Result");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Returns the trigger to frame value.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR componentCarrier/This control sets the component carrier.
/// HIPAR result/Returns the trigger to frame value.
ViStatus _VI_FUNC rsspecan_QueryLTEUplinkMeasurementFrameStartOffset(
                            ViSession instrSession,
                            ViInt32   componentCarrier,
                            ViReal64* result)
{
    ViStatus  error = VI_SUCCESS;
    ViBoolean isFSV = VI_FALSE;
    ViChar    repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    isFSV = rsspecan_IsFSV(instrSession);
    if (isFSV == VI_FALSE)
    {
        viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_LTE_UPLINK_TRIGGER_TO_FRAME_RESULT_CARR,
                "cc", componentCarrier - 1, RS_REPCAP_BUF_SIZE, repCap),
                2, "Component Carrier");

        viCheckParm(rsspecan_GetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_LTE_UPLINK_TRIGGER_TO_FRAME_RESULT_CARR, result),
                3, "Result");
    }
    else
    {
        viCheckParm(rsspecan_GetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_LTE_UPLINK_TRIGGER_TO_FRAME_RESULT, result),
                3, "Result");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects the subframe to be analyzed.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR componentCarrier/This control sets the component carrier.
/// HIPAR subframe/Selects the subframe to be analyzed.
ViStatus _VI_FUNC rsspecan_ConfigureLTEUplinkMeasurementSubframe(
                            ViSession instrSession,
                            ViInt32   componentCarrier,
                            ViInt32   subframe)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_LTE_UPLINK_SUBFRAME_SELECTION,
            "cc", componentCarrier - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Component Carrier");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_LTE_UPLINK_SUBFRAME_SELECTION, subframe),
            3, "Subframe");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects all the subframe to be analyzed.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR componentCarrier/This control sets the component carrier.
ViStatus _VI_FUNC rsspecan_ConfigureLTEUplinkMeasurementSubframeAll(
                            ViSession instrSession,
                            ViInt32   componentCarrier)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, componentCarrier, 1, 5),
            2, "Component Carrier");

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_LTE_UPLINK_SUBFRAME_SELECTION_ALL,
            "cc", componentCarrier - 1, RS_REPCAP_BUF_SIZE, repCap));

    checkErr(rsspecan_SetAttributeViString(instrSession, repCap, RSSPECAN_ATTR_LTE_UPLINK_SUBFRAME_SELECTION_ALL, ""));

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects the slot to analyze.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR componentCarrier/This control sets the component carrier.
/// HIPAR slot/Selects the slot to analyze.
ViStatus _VI_FUNC rsspecan_ConfigureLTEUplinkMeasurementSlot(ViSession instrSession,
                                                             ViInt32   componentCarrier,
                                                             ViInt32   slot)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_LTE_UPLINK_SLOT_SELECTION,
            "cc", componentCarrier - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Component Carrier");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_LTE_UPLINK_SLOT_SELECTION, slot),
            3, "Slot");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function filters the displayed results in the constellation diagram by a particular type of modulation.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR componentCarrier/This control sets the component carrier.
/// HIPAR modulation/Shows the results for a particular modulation type.
ViStatus _VI_FUNC rsspecan_ConfigureLTEUplinkMeasurementConstelllationModulation(
                            ViSession instrSession,
                            ViInt32   componentCarrier,
                            ViInt32   modulation)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, componentCarrier, 1, 5),
            2, "Component Carrier");

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_LTE_UPLINK_CONSTELLATION_MODULATION,
            "cc", componentCarrier - 1, RS_REPCAP_BUF_SIZE, repCap));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_LTE_UPLINK_CONSTELLATION_MODULATION, modulation),
            3, "Modulation");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Shows the results for all modulation types.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR componentCarrier/This control sets the component carrier.
ViStatus _VI_FUNC rsspecan_ConfigureLTEUplinkMeasurementConstelllationModulationAll(
                            ViSession instrSession,
                            ViInt32   componentCarrier)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, componentCarrier, 1, 5),
            2, "Component Carrier");

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_LTE_UPLINK_CONSTELLATION_MODULATION_ALL,
            "cc", componentCarrier - 1, RS_REPCAP_BUF_SIZE, repCap));

    checkErr(rsspecan_SetAttributeViString(instrSession, repCap, RSSPECAN_ATTR_LTE_UPLINK_CONSTELLATION_MODULATION_ALL, ""));

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function filters the displayed results in the constellation diagram by a particular type of allocation.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR componentCarrier/This control sets the component carrier.
ViStatus _VI_FUNC rsspecan_ConfigureLTEUplinkMeasurementConstelllationAllocation(
                            ViSession instrSession,
                            ViInt32   componentCarrier,
                            ViInt32   allocation)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, componentCarrier, 1, 5),
            2, "Component Carrier");

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_LTE_UPLINK_CONSTELLATION_ALLOCATION,
            "cc", componentCarrier - 1, RS_REPCAP_BUF_SIZE, repCap));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_LTE_UPLINK_CONSTELLATION_ALLOCATION, allocation),
            3, "Allocation");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function shows all the results for all allocation.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR componentCarrier/This control sets the component carrier.
ViStatus _VI_FUNC rsspecan_ConfigureLTEUplinkMeasurementConstelllationAllocationAll(
                            ViSession instrSession,
                            ViInt32   componentCarrier)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, componentCarrier, 1, 5),
            2, "Component Carrier");

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_LTE_UPLINK_CONSTELLATION_ALLOCATION_ALL,
            "cc", componentCarrier - 1, RS_REPCAP_BUF_SIZE, repCap));

    checkErr(rsspecan_SetAttributeViString(instrSession, repCap, RSSPECAN_ATTR_LTE_UPLINK_CONSTELLATION_ALLOCATION_ALL, ""));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function filters the displayed results in the constellation diagram by a particular OFDM symbol.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR componentCarrier/This control sets the component carrier.
/// HIPAR symbol_OFDM/Sets the results for a particular OFDM symbol.
ViStatus _VI_FUNC rsspecan_ConfigureLTEUplinkMeasurementConstelllationOFDMSymbol(
                            ViSession instrSession,
                            ViInt32   componentCarrier,
                            ViInt32   symbol_OFDM)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, componentCarrier, 1, 5),
            2, "Component Carrier");

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_LTE_UPLINK_CONSTELLATION_SYMBOL,
            "cc", componentCarrier - 1, RS_REPCAP_BUF_SIZE, repCap));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_LTE_UPLINK_CONSTELLATION_SYMBOL, symbol_OFDM),
            3, "Symbol OFDM");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN Filters all the displayed results in the constellation diagram by a particular OFDM symbol.
/// HIFN band starts and stops.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR componentCarrier/This control selects the component carrier to be used.
ViStatus _VI_FUNC rsspecan_ConfigureLTEUplinkMeasurementConstelllationOFDMSymbolAll(
                            ViSession instrSession,
                            ViInt32   componentCarrier)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, componentCarrier, 1, 5),
            2, "Component Carrier");

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_LTE_UPLINK_CONSTELLATION_SYMBOL_ALL,
            "cc", componentCarrier - 1, RS_REPCAP_BUF_SIZE, repCap));

    checkErr(rsspecan_SetAttributeViString(instrSession, repCap, RSSPECAN_ATTR_LTE_UPLINK_CONSTELLATION_SYMBOL_ALL, ""));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN Filters the displayed results in the constellation diagram by a particular subcarrier.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR componentCarrier/This control selects the component carrier to be used.
/// HIPAR carrier/Sets the displayed results in the constellation diagram by a particular subcarrier.
ViStatus _VI_FUNC rsspecan_ConfigureLTEUplinkMeasurementConstelllationCarrier(
                            ViSession instrSession,
                            ViInt32   componentCarrier,
                            ViInt32   carrier)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, componentCarrier, 1, 5),
            2, "Component Carrier");

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_LTE_UPLINK_CONSTELLATION_CARRIER,
            "cc", componentCarrier - 1, RS_REPCAP_BUF_SIZE, repCap));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_LTE_UPLINK_CONSTELLATION_CARRIER, carrier),
            3, "Carrier");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN Shows the results for all subcarriers.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR componentCarrier/This control selects the component carrier to be used.
ViStatus _VI_FUNC rsspecan_ConfigureLTEUplinkMeasurementConstelllationCarrierAll(
                            ViSession instrSession,
                            ViInt32   componentCarrier)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, componentCarrier, 1, 5),
            2, "Component Carrier");

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_LTE_UPLINK_CONSTELLATION_CARRIER_ALL,
            "cc", componentCarrier - 1, RS_REPCAP_BUF_SIZE, repCap));

    checkErr(rsspecan_SetAttributeViString(instrSession, repCap, RSSPECAN_ATTR_LTE_UPLINK_CONSTELLATION_CARRIER_ALL, ""));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries LTE Downlink measurement time alignment error.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR componentCarrier/This control selects the component carrier to be used.
/// HIPAR antenna/This control selects the Time Alignment Antenna.
/// HIPAR frameResult/Selects result type.
/// HIPAR measurementType/Selects measurement type.
/// HIPAR result/Returns selected result.
ViStatus _VI_FUNC rsspecan_QueryLTEUplinkMeasurementTimeAlignmentError(
                            ViSession instrSession,
                            ViInt32   componentCarrier,
                            ViInt32   antenna,
                            ViInt32   frameResult,
                            ViInt32   measurementType,
                            ViReal64* result)
{
    ViStatus                                                  error = VI_SUCCESS;
    ViChar                                                    repCap[RS_REPCAP_BUF_SIZE] = "";
    ViAttr                                                    attr[4] = {
                                                              RSSPECAN_ATTR_LTE_UPLINK_TIME_ALIGNMENT_ERROR_MIN,
    RSSPECAN_ATTR_LTE_UPLINK_TIME_ALIGNMENT_ERROR_MAX,        0,
                                                              RSSPECAN_ATTR_LTE_UPLINK_TIME_ALIGNMENT_ERROR_AVERAGE
                                                              };
    ViAttr                                                    attr_ppm[4] = {
                                                              RSSPECAN_ATTR_LTE_UPLINK_CARRIER_FREQUENCY_ERROR_PPM_MIN,
    RSSPECAN_ATTR_LTE_UPLINK_CARRIER_FREQUENCY_ERROR_PPM_MAX, 0,
                                                              RSSPECAN_ATTR_LTE_UPLINK_CARRIER_FREQUENCY_ERROR_PPM_AVERAGE
                                                              };
    ViAttr                                                    attr_hz[4] = {
                                                              RSSPECAN_ATTR_LTE_UPLINK_CARRIER_FREQUENCY_ERROR_HZ_MIN,
    RSSPECAN_ATTR_LTE_UPLINK_CARRIER_FREQUENCY_ERROR_HZ_MAX,  0,
                                                              RSSPECAN_ATTR_LTE_UPLINK_CARRIER_FREQUENCY_ERROR_HZ_AVERAGE
                                                              };

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, componentCarrier, 1, 5),
            2, "Component Carrier");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, antenna, 2, 4),
            3, "Antenna");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, measurementType, RSSPECAN_VAL_MEASTYPE_MIN, RSSPECAN_VAL_MEASTYPE_AVER),
            5, "Measurement Type");

    switch (frameResult)
    {
    case RSSPECAN_VAL_LTE_TAER:
        snprintf(repCap, RS_REPCAP_BUF_SIZE, "cc%ld,Ant%ld", componentCarrier, antenna);

        viCheckParm(rsspecan_GetAttributeViReal64(instrSession, repCap,
                        attr[measurementType - 1],
                        result), 6, "Result");
        break;
    case RSSPECAN_VAL_LTE_FRER_PPM:
        snprintf(repCap, RS_REPCAP_BUF_SIZE, "cc%ld", componentCarrier);

        viCheckParm(rsspecan_GetAttributeViReal64(instrSession, repCap,
                        attr_ppm[measurementType - 1],
                        result), 6, "Result");
        break;
    case RSSPECAN_VAL_LTE_FRER_HZ:
        snprintf(repCap, RS_REPCAP_BUF_SIZE, "cc%ld", componentCarrier);

        viCheckParm(rsspecan_GetAttributeViReal64(instrSession, repCap,
                        attr_hz[measurementType - 1],
                        result), 6, "Result");
        break;
    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, frameResult), 4, "Frame Result");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function queries the state of the limit check for the adjacent or
/// HIFN  alternate channels in an ACLR measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channelType/Selects the ACP channel type.
/// HIPAR limitCheckType/Selects the limit check type.
/// HIPAR results/Returns two results. The first is the result for the lower, the second
/// HIPAR results/for the upper adjacent or alternate channel.
ViStatus _VI_FUNC rsspecan_QueryLTEUplinkACPChannelLimitCheckResults(
                            ViSession instrSession,
                            ViInt32   channelType,
                            ViInt32   limitCheckType,
                            ViInt32   results[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViChar   response[RS_MAX_MESSAGE_BUF_SIZE] = "";
    ViChar*  p2buf;
    ViString typeArr[] = {"ALL", "REL", "ABS"};

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, limitCheckType, RSSPECAN_VAL_LIMIT_CHECK_TYPE_ALL, RSSPECAN_VAL_LIMIT_CHECK_TYPE_ABS),
            3, "Limit Check Type");

    switch (channelType)
    {
    case RSSPECAN_VAL_ACP_ADJ:
        switch (limitCheckType)
        {
        case RSSPECAN_VAL_LIMIT_CHECK_TYPE_ALL:
            checkErr(RsCore_QueryViString(instrSession, "CALC:LIM:ACP:ACH:RES?", response));
            break;
        default:
            snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CALC:LIM:ACP:ACH:RES? %s", typeArr[limitCheckType]);
            checkErr(RsCore_QueryViString(instrSession, cmd, response));
            break;
        }
        break;
    case RSSPECAN_VAL_ACP_ALT:
        switch (limitCheckType)
        {
        case RSSPECAN_VAL_LIMIT_CHECK_TYPE_ALL:
            checkErr(RsCore_QueryViString(instrSession, "CALC:LIM:ACP:ALT:RES?", response));
            break;
        default:
            snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CALC:LIM:ACP:ALT:RES? %s", typeArr[limitCheckType]);
            checkErr(RsCore_QueryViString(instrSession, cmd, response));
            break;
        }
        break;
    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, channelType), 2, "Channel Type");
    }

    p2buf = strtok(response, ",\n\r");

    if (strcmp(p2buf, "PASSED") == 0)
        results[0] = 1;
    else
        results[0] = 0;

    p2buf = strtok(NULL, ",\r\n");

    if (strcmp(p2buf, "PASSED") == 0)
        results[1] = 1;
    else
        results[1] = 0;

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function queries the current results of the ACLR measurement or
/// HIFN  the total signal power level of the SEM measurement. To get a valid
/// HIFN  result, you have to perform a complete measurement with synchronization
/// HIFN  to the end of the measurement before reading out the result. This is
/// HIFN  only possible
/// HIFN     for single sweeps.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     -
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     CALCulate<1|2>:MARKer<1>:FUNCtion:POWer:RESult[:CURRent]?
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR powerMeasurement/Selects the ACP (adjacent channel power) standard.
/// HIPAR arraySize/Defines the memory size allocated for Results.
/// HIPAR results/Returns the measured values:
/// HIPAR returnedValues/Returns number of returned values in Results array.
ViStatus _VI_FUNC rsspecan_QueryLTEUplinkMeasurementPowerResult(
                            ViSession instrSession,
                            ViInt32   powerMeasurement,
                            ViInt32   arraySize,
                            ViReal64  results[],
                            ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViString arr[] = {"ACP", "CPOW", "MCAC", "", "", "", "", "PPOW", "GACL"};

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K100|K102"));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CALC1:MARK:FUNC:POW:RES? %s", arr[powerMeasurement]);
    checkErr(RsCore_QueryFloatArrayToUserBuffer(instrSession, cmd, arraySize, results, returnedValues));

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN Selects the location of the local oscillator in a system with contiguous aggregated carriers
/// HIFN band starts and stops.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR location/Selects the location of the local oscillator in a system with contiguous aggregated carriers.
ViStatus _VI_FUNC rsspecan_ConfigureLTEUplinkLocalOscillatorLocation(
                            ViSession instrSession,
                            ViInt32   location)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_LTE_UPLINK_LOCAL_OSCILLATOR_LOCATION, location),
            2, "Location");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the LO frequency when its location is not at the center of the
/// HIFN  channel bandwidth. The attribute is available for a custom LO location
/// HIFN  (RSSPECAN_ATTR_LTE_UPLINK_LOCAL_OSCILLATOR_LOCATION).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR frequency/Defines the LO frequency when its location is not at the center of the
/// HIPAR frequency/channel bandwidth. The control is available for a custom LO location
/// HIPAR frequency/(RSSPECAN_ATTR_LTE_UPLINK_LOCAL_OSCILLATOR_LOCATION).
ViStatus _VI_FUNC rsspecan_ConfigureLTEUplinkLocalOscillatorFrequency(
                            ViSession instrSession,
                            ViReal64  frequency)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, NULL, RSSPECAN_ATTR_LTE_UPLINK_LOCAL_OSCILLATOR_FREQUENCY, frequency),
            2, "Frequency");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN Queries the results of a CCDF or ADP measurement for a specific trace.
/// HIFN band starts and stops.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR trace/Selects the trace.
/// HIPAR resultType/Selects the signal statistic measurement result type.
/// HIPAR result/Returns the selected measurement.
ViStatus _VI_FUNC rsspecan_QueryLTEUplinkCCDFStatisticalResults(
                            ViSession instrSession,
                            ViInt32   trace,
                            ViInt32   resultType,
                            ViReal64* result)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViChar*  p2string;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, trace, 1, 6),
            2, "Trace");
    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &rsspecan_rngStatisticsResultType, resultType, &p2string),
                3, "Result Type");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, result),
                4, "Result");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CALC:STAT:RES%ld? %s", trace, p2string);
    checkErr(RsCore_QueryViReal64(instrSession, cmd, result));

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN Queries the results of a CCDF or ADP measurement for a specific trace.
/// HIFN band starts and stops.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR trace/Selects the trace.
/// HIPAR arraySize/Size of results array.
/// HIPAR results/Returns all of measurement results.
ViStatus _VI_FUNC rsspecan_QueryLTEUplinkCCDFStatisticalAllResults(
                            ViSession instrSession,
                            ViInt32   trace,
                            ViInt32   arraySize,
                            ViReal64  results[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViChar*  pbuffer = NULL;
    ViChar*  p2buff;
    ViInt32  i = 0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, trace, 1, 6),
            2, "Trace");
    viCheckParm(RsCore_InvalidViInt32RangeMin(instrSession, arraySize, 1),
                3, "Array Size");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, results),
                4, "Results");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CALC:STAT:RES%ld? ALL", trace);
    checkErr(RsCore_QueryViStringUnknownLength(instrSession, cmd, &pbuffer));
    p2buff = strtok(pbuffer, ",;");

    while (p2buff && (i < arraySize))
    {
        results[i++] = RsCore_Convert_String_To_ViReal64(p2buff);
        p2buff = strtok(NULL, ",;");
    }

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    if (pbuffer) free(pbuffer);
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN Queries the marker position on the z-axis of three-dimensional
/// HIFN result display.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR zAxis/Returns the marker position on the z-axis of three-dimensional result displays.
ViStatus _VI_FUNC rsspecan_QueryLTEUplinkMarkerZAxis(ViSession instrSession,
                                                     ViReal64* zAxis)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_LTE_UPLINK_MARKER_Z_AXIS, zAxis),
            3, "Z-Axis");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN Queries the marker position on the z-axis of three-dimensional
/// HIFN result displays.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR arraySize/This control sets the number of values returned in arrays.
/// HIPAR zAxis/Returns the marker position on the z-axis of three-dimensional result displays.
ViStatus _VI_FUNC rsspecan_QueryLTEUplinkMarkerZAxisAll(ViSession instrSession,
                                                        ViInt32   arraySize,
                                                        ViReal64  zAxis[])
{
    ViStatus error = VI_SUCCESS;
    ViChar*  pbuffer = NULL;
    ViChar*  p2buff;
    ViInt32  i = 0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32RangeMin(instrSession, arraySize, 1),
                2, "Array Size");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, zAxis),
                3, "Z-Axis");

    checkErr(RsCore_QueryViStringUnknownLength(instrSession, "CALC:MARK:Z:ALL?", &pbuffer));
    p2buff = strtok(pbuffer, ",;");

    while (p2buff && (i < arraySize))
    {
        zAxis[i++] = RsCore_Convert_String_To_ViReal64(p2buff);
        p2buff = strtok(NULL, ",;");
    }

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    if (pbuffer) free(pbuffer);
    (void)RsCore_UnlockSession(instrSession);
    return error;
}
