/*****************************************************************************
 *  Rohde&Schwarz Spectrum Analyzer instrument driver
 *  OneWeb Reverse Link option K201
 *
 *  Original Release: November 2019
 *  By: Jiri Kominek
 *
 *  Should you have any technical questions please contact the hotline of
 *  Rohde & Schwarz Vertriebs-GmbH Rohde & Schwarz Support Center
 *
 *  e-mail: CustomerSupport@rohde-schwarz.com
 *
 *****************************************************************************/
#include "rsspecan.h"
#include <limits.h>

/// HIFN  Selects a new measurement channel with OneWeb Uplink channel type.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsspecan_SetOneWebULMode(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsspecan_SetAttributeViString(instrSession, NULL, RSSPECAN_ATTR_ONEWEB_UL_MODE, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects the measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR selectedMeasurement/Selects the measurement.
ViStatus _VI_FUNC rsspecan_ConfigureOneWebULMeasurement(ViSession instrSession,
                                                        ViInt32   selectedMeasurement)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, NULL, RSSPECAN_ATTR_ONEWEB_UL_SELECTED_MEASUREMENT, selectedMeasurement),
            2, "Selected Measurement");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  he data capture settings contain settings that control the data
/// HIFN  capture.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR overallFrameCountEnabled/Turns manual selection of the number of frames you want to analyze on
/// HIPAR overallFrameCountEnabled/and off.
/// HIPAR numberOfFramesToAnalyze/Defines the number of frames you want to analyze.
/// HIPAR autoAccordingToStandard/Turns automatic selection of the number of frames to analyze on and
/// HIPAR autoAccordingToStandard/off.
/// HIPAR singleSubframeModeEnabled/Turns the analysis of a single subframe only on and off.
ViStatus _VI_FUNC rsspecan_ConfigureOneWebULDataCapture(ViSession instrSession,
                                                        ViBoolean overallFrameCountEnabled,
                                                        ViInt32   numberOfFramesToAnalyze,
                                                        ViBoolean autoAccordingToStandard,
                                                        ViBoolean singleSubframeModeEnabled)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, NULL, RSSPECAN_ATTR_ONEWEB_UL_SINGLE_SUBFRAME_MODE_ENABLED, singleSubframeModeEnabled),
            5, "Single Subframe Mode Enabled");
    if (!singleSubframeModeEnabled)
    {
        viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, NULL, RSSPECAN_ATTR_ONEWEB_UL_OVERALL_FRAME_COUNT_ENABLED, overallFrameCountEnabled),
                2, "Overall Frame Count Enabled");
        if (overallFrameCountEnabled)
        {
            viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, NULL, RSSPECAN_ATTR_ONEWEB_UL_AUTO_ACCORDING_TO_STANDARD, autoAccordingToStandard),
                    4, "Auto According To Standard");
            if (!autoAccordingToStandard)
            {
                viCheckParm(rsspecan_SetAttributeViInt32(instrSession, NULL, RSSPECAN_ATTR_ONEWEB_UL_NUMBER_OF_FRAMES_TO_ANALYZE, numberOfFramesToAnalyze),
                        3, "Number Of Frames To Analyze");
            }
        }
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  The tracking settings contain settings that compensate for various
/// HIFN  common measurement errors that may occur.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR phaseTracking/Selects the phase tracking method.
/// HIPAR timeTrackingEnabled/Turns timing tracking on and off.
ViStatus _VI_FUNC rsspecan_ConfigureOneWebULTracking(ViSession instrSession,
                                                     ViInt32   phaseTracking,
                                                     ViBoolean timeTrackingEnabled)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, NULL, RSSPECAN_ATTR_ONEWEB_UL_TRACKING_PHASE, phaseTracking),
            2, "Phase Tracking");

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, NULL, RSSPECAN_ATTR_ONEWEB_UL_TRACKING_TIMING_ENABLED, timeTrackingEnabled),
            3, "Time Tracking Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the signal demodulation.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR analysisMode/Selects the uplink analysis mode.
/// HIPAR channelEstimationRange/Selects the channel estimation type.
/// HIPAR exclusionPeriodEnabled/Includes or excludes the exclusion period from EVM results.
/// HIPAR compensateDCOffsetEnabled/Turns DC offset compensation on and off.
/// HIPAR descrambleCodedBits/Turns scrambling of coded bits on and off.
/// HIPAR multicarrierFilterEnabled/Turns suppression of interfering neighboring carriers on and off (for
/// HIPAR multicarrierFilterEnabled/example LTE, WCDMA, GSM etc.).
ViStatus _VI_FUNC rsspecan_ConfigureOneWebULSignalDemodulation(
                            ViSession instrSession,
                            ViInt32   analysisMode,
                            ViInt32   channelEstimationRange,
                            ViBoolean exclusionPeriodEnabled,
                            ViBoolean compensateDCOffsetEnabled,
                            ViBoolean descrambleCodedBits,
                            ViBoolean multicarrierFilterEnabled)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, NULL, RSSPECAN_ATTR_ONEWEB_UL_DEMODULATION_MODE, analysisMode),
            2, "Analysis Mode");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, NULL, RSSPECAN_ATTR_ONEWEB_UL_DEMODULATION_CHANNEL_ESTIMATION_RANGE, channelEstimationRange),
            3, "Channel Estimation Range");

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, NULL, RSSPECAN_ATTR_ONEWEB_UL_DEMODULATION_EVM_WITH_EXCLUSION_PERIOD_ENABLED, exclusionPeriodEnabled),
            4, "Exclusion Period Enabled");

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, NULL, RSSPECAN_ATTR_ONEWEB_UL_DEMODULATION_COMPENSATE_DC_OFFSET_ENABLED, compensateDCOffsetEnabled),
            5, "Compensate DC Offset Enabled");

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, NULL, RSSPECAN_ATTR_ONEWEB_UL_DEMODULATION_SCRAMBLING_ENABLED, descrambleCodedBits),
            6, "Descramble Coded Bits");

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, NULL, RSSPECAN_ATTR_ONEWEB_UL_DEMODULATION_MULTICARRIER_FILTER_ENABLED, multicarrierFilterEnabled),
            7, "Multicarrier Filter Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects the number of component carriers analyzed in the measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR numberOfComponentCarriers/Selects the number of component carriers analyzed in the measurement.
ViStatus _VI_FUNC rsspecan_ConfigureOneWebULNumberOfComponentCarriers(
                            ViSession instrSession,
                            ViInt32   numberOfComponentCarriers)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, NULL, RSSPECAN_ATTR_ONEWEB_UL_NUMBER_OF_COMPONENT_CARRIERS, numberOfComponentCarriers),
            2, "Number Of Component Carriers");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects the location of the local oscillator in a system with
/// HIFN  contiguous aggregated carriers.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR location/Selects the location of the local oscillator in a system with
/// HIPAR location/contiguous aggregated carriers.
/// HIPAR manualFrequency/Defines the LO frequency when its location is not at the center of the
/// HIPAR manualFrequency/channel bandwidth.
ViStatus _VI_FUNC rsspecan_ConfigureOneWebULLO(ViSession instrSession,
                                               ViInt32   location,
                                               ViReal64  manualFrequency)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, NULL, RSSPECAN_ATTR_ONEWEB_UL_LO_LOCATION, location),
            2, "Location");
    if (location == RSSPECAN_VAL_OW_LO_USER)
    {
        viCheckParm(rsspecan_SetAttributeViReal64(instrSession, NULL, RSSPECAN_ATTR_ONEWEB_UL_LO_FREQUENCY, manualFrequency),
                3, "Manual Frequency");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  The global settings contain settings that apply to the complete
/// HIFN  signal.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR componentCarrier/Component carrier
/// HIPAR frameNumberOffset/Defines the system frame number offset.
/// HIPAR ueID/Defines the radio network temporary identifier (RNTI) of the UE.
ViStatus _VI_FUNC rsspecan_ConfigureOneWebULGlobalSignalCharacteristics(
                            ViSession instrSession,
                            ViInt32   componentCarrier,
                            ViInt32   frameNumberOffset,
                            ViInt32   ueID)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, componentCarrier, 1, 5),
            2, "Component Carrier");
    snprintf(repCap, RS_REPCAP_BUF_SIZE, "cc%ld", componentCarrier);

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_ONEWEB_UL_SYSTEM_FRAME_NUMBER_OFFSET, frameNumberOffset),
            3, "Frame Number Offset");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_ONEWEB_UL_UE_ID, ueID),
            4, "UE ID");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the secondary cell index.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR componentCarrier/Component carrier
/// HIPAR SCellIndex/Defines the secondary cell index.
ViStatus _VI_FUNC rsspecan_ConfigureOneWebULSCellIndex(ViSession instrSession,
                                                       ViInt32   componentCarrier,
                                                       ViInt32   SCellIndex)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, componentCarrier, 1, 5),
            2, "Component Carrier");
    snprintf(repCap, RS_REPCAP_BUF_SIZE, "cc%ld", componentCarrier);

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_ONEWEB_UL_SECONDARY_CELL_INDEX, SCellIndex),
            3, "SCell Index");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the center frequency for RF measurements.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR componentCarrier/Component carrier
/// HIPAR carrierFrequency/Sets the center frequency for RF measurements.
ViStatus _VI_FUNC rsspecan_ConfigureOneWebULCarrierFrequency(ViSession instrSession,
                                                             ViInt32   componentCarrier,
                                                             ViReal64  carrierFrequency)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, componentCarrier, 1, 5),
            2, "Component Carrier");
    snprintf(repCap, RS_REPCAP_BUF_SIZE, "cc%ld", componentCarrier);

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_ONEWEB_UL_CARRIER_FREQUENCY, carrierFrequency),
            3, "Carrier Frequency");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the general frequency offset.
/// HIFN     For measurements on multiple component carriers, the attribute defines
/// HIFN  the frequency offset for a component carrier.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR frequencyOffset/Defines the general frequency offset.
/// HIPAR componentCarrier/Component carrier
ViStatus _VI_FUNC rsspecan_ConfigureOneWebULFrequencyOffset(ViSession instrSession,
                                                            ViInt32   componentCarrier,
                                                            ViReal64  frequencyOffset)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, componentCarrier, 2, 5),
            2, "Component Carrier");
    snprintf(repCap, RS_REPCAP_BUF_SIZE, "cc%ld", componentCarrier);

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_ONEWEB_UL_FREQUENCY_OFFSET, frequencyOffset),
            3, "Frequency Offset");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  The general signal characteristics contain settings to describe the
/// HIFN  general physical attributes of the signal.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR componentCarrier/Component carrier
/// HIPAR bandwidth/Selects the channel bandwidth.
ViStatus _VI_FUNC rsspecan_ConfigureOneWebULBandwidth(ViSession instrSession,
                                                      ViInt32   componentCarrier,
                                                      ViInt32   bandwidth)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, componentCarrier, 1, 5),
            2, "Component Carrier");
    snprintf(repCap, RS_REPCAP_BUF_SIZE, "cc%ld", componentCarrier);

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_ONEWEB_UL_BANDWIDTH, bandwidth),
            3, "Bandwidth");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects the cell identity group.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR componentCarrier/Component carrier
/// HIPAR cellIdentityGroup/Selects the cell identity group.
ViStatus _VI_FUNC rsspecan_ConfigureOneWebULCellIdentityGroup(
                            ViSession instrSession,
                            ViInt32   componentCarrier,
                            ViInt32   cellIdentityGroup)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, componentCarrier, 1, 5),
            2, "Component Carrier");
    snprintf(repCap, RS_REPCAP_BUF_SIZE, "cc%ld", componentCarrier);

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_ONEWEB_UL_CELL_IDENTITY_GROUP, cellIdentityGroup),
            3, "Cell Identity Group");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the cell ID.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR componentCarrier/Component carrier
/// HIPAR cellID/Defines the cell ID.
ViStatus _VI_FUNC rsspecan_ConfigureOneWebULCellID(ViSession instrSession,
                                                   ViInt32   componentCarrier,
                                                   ViInt32   cellID)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, componentCarrier, 1, 5),
            2, "Component Carrier");
    snprintf(repCap, RS_REPCAP_BUF_SIZE, "cc%ld", componentCarrier);

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_ONEWEB_UL_CELL_ID, cellID),
            3, "Cell ID");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects the physical layer identity.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR componentCarrier/Component carrier
/// HIPAR onewebULIdentity/Selects the physical layer identity.
ViStatus _VI_FUNC rsspecan_ConfigureOneWebULIdentity(ViSession instrSession,
                                                     ViInt32   componentCarrier,
                                                     ViInt32   identity)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, componentCarrier, 1, 5),
            2, "Component Carrier");
    snprintf(repCap, RS_REPCAP_BUF_SIZE, "cc%ld", componentCarrier);

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_ONEWEB_UL_IDENTITY, identity),
            3, "Identity");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the cyclic prefix type that has been detected.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR ulDetectedCyclicPrefix/Queries the cyclic prefix type that has been detected.
/// HIPAR componentCarrier/Component carrier
ViStatus _VI_FUNC rsspecan_QueryOneWebULDetectedCyclicPrefix(ViSession instrSession,
                                                             ViInt32   componentCarrier,
                                                             ViInt32*  detectedCyclicPrefix)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, componentCarrier, 1, 5),
            2, "Component Carrier");
    snprintf(repCap, RS_REPCAP_BUF_SIZE, "cc%ld", componentCarrier);

    viCheckParm(rsspecan_GetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_ONEWEB_UL_DETECTED_CYCLIC_PREFIX, detectedCyclicPrefix),
            3, "Detected Cyclic Prefix");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the cell identity group that has been detected. Returns -1 if
/// HIFN  no valid result has been detected yet.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR detectedCellIdentityGroup/Queries the cell identity group that has been detected. Returns -1 if
/// HIPAR detectedCellIdentityGroup/no valid result has been detected yet.
/// HIPAR componentCarrier/Component carrier
ViStatus _VI_FUNC rsspecan_QueryOneWebULDetectedCellIdentityGroup(
                            ViSession instrSession,
                            ViInt32   componentCarrier,
                            ViInt32*  detectedCellIdentityGroup)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, componentCarrier, 1, 5),
            2, "Component Carrier");
    snprintf(repCap, RS_REPCAP_BUF_SIZE, "cc%ld", componentCarrier);

    checkErr(rsspecan_GetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_ONEWEB_UL_DETECTED_CELL_IDENTITY_GROUP, detectedCellIdentityGroup));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the cell identity that has been detected. Returns -1 if no
/// HIFN  valid result has been detected yet.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR onewebULDetectedIdentity/Queries the cell identity that has been detected. Returns -1 if no
/// HIPAR onewebULDetectedIdentity/valid result has been detected yet.
/// HIPAR componentCarrier/Component carrier
ViStatus _VI_FUNC rsspecan_QueryOneWebULDetectedIdentity(ViSession instrSession,
                                                         ViInt32   componentCarrier,
                                                         ViInt32*  detectedIdentity)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, componentCarrier, 1, 5),
            2, "Component Carrier");
    snprintf(repCap, RS_REPCAP_BUF_SIZE, "cc%ld", componentCarrier);

    checkErr(rsspecan_GetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_ONEWEB_UL_DETECTED_IDENTITY, detectedIdentity));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Restores previously saved demodulation settings.
/// HIFN     The file must be of type .allocation and depends on the link direction
/// HIFN  that was currently selected when the file was saved. You can load only
/// HIFN  files with correct link directions.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR componentCarrier/Component carrier
/// HIPAR fileName/Restores previously saved demodulation settings.
ViStatus _VI_FUNC rsspecan_ConfigureOneWebULRestoreSavedDemodSettings(
                            ViSession instrSession,
                            ViInt32   componentCarrier,
                            ViString  fileName)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, componentCarrier, 1, 5),
            2, "Component Carrier");
    snprintf(repCap, RS_REPCAP_BUF_SIZE, "cc%ld", componentCarrier);

    viCheckParm(rsspecan_SetAttributeViString(instrSession, repCap, RSSPECAN_ATTR_ONEWEB_UL_RESTORE_SAVED_DEMOD_SETTINGS, fileName),
            3, "File Name");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects the method of automatic demodulation.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR autoDemodulationEnabled/Selects the method of automatic demodulation.
ViStatus _VI_FUNC rsspecan_ConfigureOneWebULAutoDemodulationEnabled(
                            ViSession instrSession,
                            ViInt32   autoDemodulationEnabled)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, NULL, RSSPECAN_ATTR_ONEWEB_UL_AUTO_DEMODULATION_ENABLED, autoDemodulationEnabled),
            2, "Auto Demodulation Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Turns detection of the subframe configuration on and off.
/// HIFN     Turn off auto demodulation.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR detectionEnabled/Turns detection of the subframe configuration on and off.
ViStatus _VI_FUNC rsspecan_ConfigureOneWebULSubframeConfigurationDetectionEnabled(
                            ViSession instrSession,
                            ViBoolean detectionEnabled)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, NULL, RSSPECAN_ATTR_ONEWEB_UL_SUBFRAME_CONFIGURATION_DETECTION_ENABLED, detectionEnabled),
            2, "Detection Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects the number of configurable subframes in the uplink signal.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR componentCarrier/Component carrier
/// HIPAR configurableSubframes/Selects the number of configurable subframes in the uplink signal.
ViStatus _VI_FUNC rsspecan_ConfigureOneWebULConfigurableSubframes(
                            ViSession instrSession,
                            ViInt32   componentCarrier,
                            ViInt32   configurableSubframes)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, componentCarrier, 1, 5),
            2, "Component Carrier");
    snprintf(repCap, RS_REPCAP_BUF_SIZE, "cc%ld", componentCarrier);

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_ONEWEB_UL_CONFIGURABLE_SUBFRAMES, configurableSubframes),
            3, "Configurable Subframes");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Contains the characteristics for selected subframe.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR componentCarrier/Component carrier
/// HIPAR subframe/Subframe number
/// HIPAR allocationContent/Allocates a PUCCH or PUSCH to an uplink allocation.
/// HIPAR modulation/Selects the modulation of an uplink allocation.
ViStatus _VI_FUNC rsspecan_ConfigureOneWebULSubframe(ViSession instrSession,
                                                     ViInt32   componentCarrier,
                                                     ViInt32   subframe,
                                                     ViInt32   allocationContent,
                                                     ViInt32   modulation)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, componentCarrier, 1, 5),
            2, "Component Carrier");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, subframe, 0, 9),
            3, "Subframe");
    snprintf(repCap, RS_REPCAP_BUF_SIZE, "cc%ld,owulsubf%ld", componentCarrier, subframe);

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_ONEWEB_UL_SUBFRAME_ALLOCATION_CONTENT, allocationContent),
            4, "Allocation Content");
    if (allocationContent != RSSPECAN_VAL_OW_UL_ALLOCATION_CONTENT_NONE)
    {
        viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_ONEWEB_UL_SUBFRAME_MODULATION, modulation),
                5, "Modulation");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Turns the resource allocation type 1 on and off.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR allocationType1Enabled/Turns the resource allocation type 1 on and off.
/// HIPAR componentCarrier/Component carrier
/// HIPAR subframe/Subframe number
ViStatus _VI_FUNC rsspecan_ConfigureOneWebULSubframeResourceAllocationType1Enabled(
                            ViSession instrSession,
                            ViInt32   componentCarrier,
                            ViInt32   subframe,
                            ViBoolean allocationType1Enabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, componentCarrier, 1, 5),
            2, "Component Carrier");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, subframe, 0, 9),
            3, "Subframe");
    snprintf(repCap, RS_REPCAP_BUF_SIZE, "cc%ld,owulsubf%ld", componentCarrier, subframe);

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, repCap, RSSPECAN_ATTR_ONEWEB_UL_SUBFRAME_RESOURCE_ALLOCATION_TYPE_1_ENABLED, allocationType1Enabled),
            4, "Allocation Type 1 Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the PUCCH format in individual subframes.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR componentCarrier/Component carrier
/// HIPAR subframe/Subframe number
/// HIPAR format/Selects the PUCCH format for a specific subframe.
ViStatus _VI_FUNC rsspecan_ConfigureOneWebULSubframeEnhancedPUCCHFormat(
                            ViSession instrSession,
                            ViInt32   componentCarrier,
                            ViInt32   subframe,
                            ViInt32   format)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, componentCarrier, 1, 5),
            2, "Component Carrier");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, subframe, 0, 9),
            3, "Subframe");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, format, 0, 6),
            4, "Format");
    snprintf(repCap, RS_REPCAP_BUF_SIZE, "cc%ld,owulsubf%ld", componentCarrier, subframe);

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_ONEWEB_UL_SUBFRAME_PUCCH_FORMAT, format),
            4, "Format");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the N_PUCCH in individual subframes.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR componentCarrier/Component carrier
/// HIPAR subframe/Subframe number
/// HIPAR n_pucch/Defines N_PUCCH on a subframe basis.
ViStatus _VI_FUNC rsspecan_ConfigureOneWebULSubframeEnhancedPUCCHnPUCCH(
                            ViSession instrSession,
                            ViInt32   componentCarrier,
                            ViInt32   subframe,
                            ViInt32   n_pucch)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, componentCarrier, 1, 5),
            2, "Component Carrier");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, subframe, 0, 9),
            3, "Subframe");
    snprintf(repCap, RS_REPCAP_BUF_SIZE, "cc%ld,owulsubf%ld", componentCarrier, subframe);

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_ONEWEB_UL_SUBFRAME_N_PUCCH, n_pucch),
            4, "n_PUCCH");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the Demodulation Reference Signal in individual subframes.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR componentCarrier/Component carrier
/// HIPAR subframe/Subframe number
/// HIPAR n_2_DMRS/Defines the part of the DMRS index that is used for the uplink
/// HIPAR n_2_DMRS/scheduling assignment.
/// HIPAR delta_puach/Defines the parameter PUACH.
ViStatus _VI_FUNC rsspecan_ConfigureOneWebULSubframeEnhancedDMRS(
                            ViSession instrSession,
                            ViInt32   componentCarrier,
                            ViInt32   subframe,
                            ViInt32   n_2_DMRS,
                            ViInt32   delta_puach)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, componentCarrier, 1, 5),
            2, "Component Carrier");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, subframe, 0, 9),
            3, "Subframe");
    snprintf(repCap, RS_REPCAP_BUF_SIZE, "cc%ld,owulsubf%ld", componentCarrier, subframe);

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_ONEWEB_UL_SUBFRAME_DMRS_INDEX, n_2_DMRS),
            4, "n(2)_DMRS");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_ONEWEB_UL_SUBFRAME_DELTA_PUACH, delta_puach),
            5, "Delta_PUACH");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures selected cluster.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR numberOfRB/Selects the number of resource blocks in an uplink subframe.
/// HIPAR componentCarrier/Component carrier
/// HIPAR subframe/Subframe number
/// HIPAR cluster/Selects a cluster
/// HIPAR offsetRB/Defines the resource block offset in an uplink subframe.
ViStatus _VI_FUNC rsspecan_ConfigureOneWebULSubframeCluster(ViSession instrSession,
                                                            ViInt32   componentCarrier,
                                                            ViInt32   subframe,
                                                            ViInt32   cluster,
                                                            ViInt32   numberOfRB,
                                                            ViInt32   offsetRB)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, componentCarrier, 1, 5),
            2, "Component Carrier");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, subframe, 0, 9),
            3, "Subframe");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, cluster, 1, 2),
            4, "Cluster");
    snprintf(repCap, RS_REPCAP_BUF_SIZE, "cc%ld,owulsubf%ld,owulcl%ld", componentCarrier, subframe, cluster);

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_ONEWEB_UL_SUBFRAME_CLUSTER_RESOURCE_BLOCKS_COUNT, numberOfRB),
            5, "Number Of RB");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_ONEWEB_UL_SUBFRAME_CLUSTER_RESOURCE_BLOCK_OFFSET, offsetRB),
            6, "Offset RB");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets relative powers.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR componentCarrier/Component carrier
/// HIPAR relativePowerPUSCH/Sets the relative power of the PUSCH.
/// HIPAR relativePowerPUCCH/Sets the relative power of the PUCCH.
ViStatus _VI_FUNC rsspecan_ConfigureOneWebULDRSRelativePowers(
                            ViSession instrSession,
                            ViInt32   componentCarrier,
                            ViReal64  relativePowerPUSCH,
                            ViReal64  relativePowerPUCCH)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, componentCarrier, 1, 5),
            2, "Component Carrier");
    snprintf(repCap, RS_REPCAP_BUF_SIZE, "cc%ld", componentCarrier);

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_ONEWEB_UL_DRS_RELATIVE_POWER_PUSCH, relativePowerPUSCH),
            4, "Relative Power PUSCH");

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_ONEWEB_UL_DRS_RELATIVE_POWER_PUCCH, relativePowerPUCCH),
            5, "Relative Power PUCCH");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  The demodulation reference signal (DRS) settings contain settings that
/// HIFN  define the physical attributes and structure of the demodulation
/// HIFN  reference signal. This reference signal helps to demodulate the PUSCH.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR componentCarrier/Component carrier
/// HIPAR groupHoppingEnabled/Turns group hopping for uplink signals on and off.
/// HIPAR n_1_DMRS/Defines the nDMRS.
/// HIPAR sequenceHoppingEnabled/Turns sequence hopping for uplink signals on and off.
/// HIPAR deltaSequenceShift/Selects the delta sequence shift of the uplink signal.
ViStatus _VI_FUNC rsspecan_ConfigureOneWebULDRS(ViSession instrSession,
                                                ViInt32   componentCarrier,
                                                ViBoolean groupHoppingEnabled,
                                                ViInt32   n_1_DMRS,
                                                ViInt32   deltaSequenceShift)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, componentCarrier, 1, 5),
            2, "Component Carrier");
    snprintf(repCap, RS_REPCAP_BUF_SIZE, "cc%ld", componentCarrier);

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, repCap, RSSPECAN_ATTR_ONEWEB_UL_DRS_GROUP_HOPPING_ENABLED, groupHoppingEnabled),
            4, "Group Hopping Enabled");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_ONEWEB_UL_DRS_N1_DMRS, n_1_DMRS),
            5, "N(1)_DMRS");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_ONEWEB_UL_DRS_DELTA_SEQUENCE_SHIFT, deltaSequenceShift),
            6, "Delta Sequence Shift");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Turns sequence hopping for uplink signals on and off
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR componentCarrier/Component carrier
/// HIPAR sequenceHoppingEnabled/Turns sequence hopping for uplink signals on and off.
ViStatus _VI_FUNC rsspecan_ConfigureOneWebULDRSSequenceHopping(
                            ViSession instrSession,
                            ViInt32   componentCarrier,
                            ViBoolean sequenceHoppingEnabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, componentCarrier, 1, 5),
            2, "Component Carrier");
    snprintf(repCap, RS_REPCAP_BUF_SIZE, "cc%ld", componentCarrier);

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, repCap, RSSPECAN_ATTR_ONEWEB_UL_DRS_SEQUENCE_HOPPING_ENABLED, sequenceHoppingEnabled),
            3, "Sequence Hopping Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Turns the sounding reference signal on and off.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR componentCarrier/Component carrier
/// HIPAR srsEnabled/Turns the sounding reference signal on and off.
ViStatus _VI_FUNC rsspecan_ConfigureOneWebULSRSEnabled(ViSession instrSession,
                                                       ViInt32   componentCarrier,
                                                       ViBoolean srsEnabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, componentCarrier, 1, 5),
            2, "Component Carrier");
    snprintf(repCap, RS_REPCAP_BUF_SIZE, "cc%ld", componentCarrier);

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, repCap, RSSPECAN_ATTR_ONEWEB_UL_SRS_ENABLED, srsEnabled),
            3, "SRS Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the SRS subframe configuration.
/// HIFN     Turn on the sounding reference signal.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR componentCarrier/Component carrier
/// HIPAR srsSubframeConfiguration/Defines the SRS subframe configuration.
ViStatus _VI_FUNC rsspecan_ConfigureOneWebULSRSSubframeConfiguration(
                            ViSession instrSession,
                            ViInt32   componentCarrier,
                            ViInt32   srsSubframeConfiguration)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, componentCarrier, 1, 5),
            2, "Component Carrier");
    snprintf(repCap, RS_REPCAP_BUF_SIZE, "cc%ld", componentCarrier);

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_ONEWEB_UL_SRS_SUBFRAME_CONFIGURATION, srsSubframeConfiguration),
            3, "SRS Subframe Configuration");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the bandwidth of the SRS (BSRS).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR componentCarrier/Component carrier
/// HIPAR srsBandwidthB_SRS/Defines the bandwidth of the SRS (BSRS).
ViStatus _VI_FUNC rsspecan_ConfigureOneWebULSRSBandwidthB_SRS(
                            ViSession instrSession,
                            ViInt32   componentCarrier,
                            ViInt32   srsBandwidthB_SRS)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, componentCarrier, 1, 5),
            2, "Component Carrier");
    snprintf(repCap, RS_REPCAP_BUF_SIZE, "cc%ld", componentCarrier);

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_ONEWEB_UL_SRS_BANDWIDTH_B_SRS, srsBandwidthB_SRS),
            3, "SRS Bandwidth B_SRS");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the frequency hopping bandwidth bhop.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR componentCarrier/Component carrier
/// HIPAR srsHoppingBW/Defines the frequency hopping bandwidth bhop.
ViStatus _VI_FUNC rsspecan_ConfigureOneWebULSRSHoppingBW(ViSession instrSession,
                                                         ViInt32   componentCarrier,
                                                         ViInt32   srsHoppingBW)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, componentCarrier, 1, 5),
            2, "Component Carrier");
    snprintf(repCap, RS_REPCAP_BUF_SIZE, "cc%ld", componentCarrier);

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_ONEWEB_UL_SRS_HOPPING_BW, srsHoppingBW),
            3, "SRS Hopping BW");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the cyclic shift n_CS used for the generation of the sounding
/// HIFN  reference signal CAZAC sequence.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR componentCarrier/Component carrier
/// HIPAR srsCyclicShiftN_CS/Sets the cyclic shift n_CS used for the generation of the sounding
/// HIPAR srsCyclicShiftN_CS/reference signal CAZAC sequence.
ViStatus _VI_FUNC rsspecan_ConfigureOneWebULSRSCyclicShiftN_CS(
                            ViSession instrSession,
                            ViInt32   componentCarrier,
                            ViInt32   srsCyclicShiftN_CS)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, componentCarrier, 1, 5),
            2, "Component Carrier");
    snprintf(repCap, RS_REPCAP_BUF_SIZE, "cc%ld", componentCarrier);

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_ONEWEB_UL_SRS_CYCLIC_SHIFT_N_CS, srsCyclicShiftN_CS),
            3, "SRS Cyclic Shift N_CS");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the relative power of the sounding reference signal.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR componentCarrier/Component carrier
/// HIPAR srsRelPower/Defines the relative power of the sounding reference signal.
ViStatus _VI_FUNC rsspecan_ConfigureOneWebULSRSRelPower(ViSession instrSession,
                                                        ViInt32   componentCarrier,
                                                        ViReal64  srsRelPower)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, componentCarrier, 1, 5),
            2, "Component Carrier");
    snprintf(repCap, RS_REPCAP_BUF_SIZE, "cc%ld", componentCarrier);

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_ONEWEB_UL_SRS_REL_POWER, srsRelPower),
            3, "SRS Rel Power");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the SRS bandwidth configuration (CSRS).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR componentCarrier/Component carrier
/// HIPAR srsBWConfC_SRS/Defines the SRS bandwidth configuration (CSRS).
ViStatus _VI_FUNC rsspecan_ConfigureOneWebULSRSBWConfC_SRS(ViSession instrSession,
                                                           ViInt32   componentCarrier,
                                                           ViInt32   srsBWConfC_SRS)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, componentCarrier, 1, 5),
            2, "Component Carrier");
    snprintf(repCap, RS_REPCAP_BUF_SIZE, "cc%ld", componentCarrier);

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_ONEWEB_UL_SRS_BW_CONF_C_SRS, srsBWConfC_SRS),
            3, "SRS BW Conf C_SRS");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the SRS configuration index (ISRS).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR componentCarrier/Component carrier
/// HIPAR confIndexI_SRS/Defines the SRS configuration index (ISRS).
ViStatus _VI_FUNC rsspecan_ConfigureOneWebULSRSConfIndexI_SRS(
                            ViSession instrSession,
                            ViInt32   componentCarrier,
                            ViInt32   confIndexI_SRS)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, componentCarrier, 1, 5),
            2, "Component Carrier");
    snprintf(repCap, RS_REPCAP_BUF_SIZE, "cc%ld", componentCarrier);

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_ONEWEB_UL_SRS_CONF_INDEX_I_SRS, confIndexI_SRS),
            3, "Conf Index I_SRS");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the transmission comb (kTC).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR componentCarrier/Component carrier
/// HIPAR transmCombk_TC/Defines the transmission comb (kTC).
ViStatus _VI_FUNC rsspecan_ConfigureOneWebULSRSTransmCombk_TC(
                            ViSession instrSession,
                            ViInt32   componentCarrier,
                            ViInt32   transmCombk_TC)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, componentCarrier, 1, 5),
            2, "Component Carrier");
    snprintf(repCap, RS_REPCAP_BUF_SIZE, "cc%ld", componentCarrier);

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_ONEWEB_UL_SRS_TRANSM_COMB_K_TC, transmCombk_TC),
            3, "Transm Comb k_TC");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the UE-specific parameter frequency domain position nRRC.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR componentCarrier/Component carrier
/// HIPAR freqDomainPosn_RRC/Defines the UE-specific parameter frequency domain position nRRC.
ViStatus _VI_FUNC rsspecan_ConfigureOneWebULSRSFreqDomainPosn_RRC(
                            ViSession instrSession,
                            ViInt32   componentCarrier,
                            ViInt32   freqDomainPosn_RRC)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, componentCarrier, 1, 5),
            2, "Component Carrier");
    snprintf(repCap, RS_REPCAP_BUF_SIZE, "cc%ld", componentCarrier);

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_ONEWEB_UL_SRS_FREQ_DOMAIN_POS_N_RRC, freqDomainPosn_RRC),
            3, "Freq Domain Pos n_RRC");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Turns simultaneous transmission of the sounding reference signal (SRS)
/// HIFN  and ACK/NACK messages (via PUCCH) on and off.
/// HIFN     Simultaneous transmission works only if the PUCCH format ist either 1,
/// HIFN  1a, 1b or 3.
/// HIFN     ON - Allows simultaneous transmission of SRS and PUCCH.
/// HIFN     OFF - SRS not transmitted in the subframe for which you have
/// HIFN  configured simultaneous transmission of PUCCH and SRS.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR componentCarrier/Component carrier
/// HIPAR simultaneousTXEnabled/Turns simultaneous transmission of the sounding reference signal (SRS)
/// HIPAR simultaneousTXEnabled/and ACK/NACK messages (via PUCCH) on and off.
ViStatus _VI_FUNC rsspecan_ConfigureOneWebULSRSSimultaneousTXEnabled(
                            ViSession instrSession,
                            ViInt32   componentCarrier,
                            ViBoolean simultaneousTXEnabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, componentCarrier, 1, 5),
            2, "Component Carrier");
    snprintf(repCap, RS_REPCAP_BUF_SIZE, "cc%ld", componentCarrier);

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, repCap, RSSPECAN_ATTR_ONEWEB_UL_SRS_SIMULTANEOUS_TX_ENABLED, simultaneousTXEnabled),
            3, "Simultaneous TX Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the number of subbands/M of the PUSCH.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR componentCarrier/Component carrier
/// HIPAR numberOfSubbands/Defines the number of subbands/M of the PUSCH.
ViStatus _VI_FUNC rsspecan_ConfigureOneWebULPUSCHNumberOfSubbands(
                            ViSession instrSession,
                            ViInt32   componentCarrier,
                            ViInt32   numberOfSubbands)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, componentCarrier, 1, 5),
            2, "Component Carrier");
    snprintf(repCap, RS_REPCAP_BUF_SIZE, "cc%ld", componentCarrier);

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_ONEWEB_UL_PUSCH_NUMBER_OF_SUBBANDS, numberOfSubbands),
            3, "Number Of Subbands");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects the number of resource blocks for the PUCCH.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR onewebULPUCCHNoOfRBs/Selects the number of resource blocks for the PUCCH.
/// HIPAR componentCarrier/Component carrier
ViStatus _VI_FUNC rsspecan_ConfigureOneWebULPUCCHNoOfRBs(ViSession instrSession,
                                                         ViInt32   componentCarrier,
                                                         ViInt32   noOfRBs)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, componentCarrier, 1, 5),
            2, "Component Carrier");
    snprintf(repCap, RS_REPCAP_BUF_SIZE, "cc%ld", componentCarrier);

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_ONEWEB_UL_PUCCH_NO_OF_RBS, noOfRBs),
            3, "No Of RBs");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Detects the number of RBs automatically.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR componentCarrier/Component carrier
ViStatus _VI_FUNC rsspecan_OneWebULPUCCHNoOfRBsAutoDetect(ViSession instrSession,
                                                          ViInt32   componentCarrier)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, componentCarrier, 1, 5),
            2, "Component Carrier");
    snprintf(repCap, RS_REPCAP_BUF_SIZE, "cc%ld", componentCarrier);

    checkErr(rsspecan_SetAttributeViString(instrSession, repCap, RSSPECAN_ATTR_ONEWEB_UL_PUCCH_NO_OF_RBS_AUTO, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the N(1)_cs of the PUCCH.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR componentCarrier/Component carrier
/// HIPAR n_1_cs/Defines the N(1)_cs of the PUCCH.
ViStatus _VI_FUNC rsspecan_ConfigureOneWebULPUCCHN1_cs(ViSession instrSession,
                                                       ViInt32   componentCarrier,
                                                       ViInt32   n_1_cs)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, componentCarrier, 1, 5),
            2, "Component Carrier");
    snprintf(repCap, RS_REPCAP_BUF_SIZE, "cc%ld", componentCarrier);

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_ONEWEB_UL_PUCCH_N1_CS, n_1_cs),
            3, "N(1)_cs");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the delta shift of the PUCCH.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR componentCarrier/Component carrier
/// HIPAR deltaShift/Defines the delta shift of the PUCCH.
ViStatus _VI_FUNC rsspecan_ConfigureOneWebULPUCCHDeltaShift(ViSession instrSession,
                                                            ViInt32   componentCarrier,
                                                            ViInt32   deltaShift)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, componentCarrier, 1, 5),
            2, "Component Carrier");
    snprintf(repCap, RS_REPCAP_BUF_SIZE, "cc%ld", componentCarrier);

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_ONEWEB_UL_PUCCH_DELTA_SHIFT, deltaShift),
            3, "Delta Shift");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects the PUCCH format.
/// HIFN     Note that formats 2a and 2b are available for normal cyclic prefix
/// HIFN  length only.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR componentCarrier/Component carrier
/// HIPAR format/Selects the PUCCH format.
ViStatus _VI_FUNC rsspecan_ConfigureOneWebULPUCCHFormat(ViSession instrSession,
                                                        ViInt32   componentCarrier,
                                                        ViInt32   format)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, componentCarrier, 1, 5),
            2, "Component Carrier");
    snprintf(repCap, RS_REPCAP_BUF_SIZE, "cc%ld", componentCarrier);

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_ONEWEB_UL_PUCCH_FORMAT, format),
            3, "Format");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the N(2)_RB of the PUCCH.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR componentCarrier/Component carrier
/// HIPAR n_2_RB/Defines the N(2)_RB of the PUCCH.
ViStatus _VI_FUNC rsspecan_ConfigureOneWebULPUCCHN2_RB(ViSession instrSession,
                                                       ViInt32   componentCarrier,
                                                       ViInt32   n_2_RB)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, componentCarrier, 1, 5),
            2, "Component Carrier");
    snprintf(repCap, RS_REPCAP_BUF_SIZE, "cc%ld", componentCarrier);

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_ONEWEB_UL_PUCCH_N2_RB, n_2_RB),
            3, "N(2)_RB");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the N_PUCCH parameter in the PUCCH structure settings.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR componentCarrier/Component carrier
/// HIPAR n_pucch/Defines the N_PUCCH parameter in the PUCCH structure settings.
ViStatus _VI_FUNC rsspecan_ConfigureOneWebULPUCCHN_PUCCH(ViSession instrSession,
                                                         ViInt32   componentCarrier,
                                                         ViInt32   n_pucch)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, componentCarrier, 1, 5),
            2, "Component Carrier");
    snprintf(repCap, RS_REPCAP_BUF_SIZE, "cc%ld", componentCarrier);

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_ONEWEB_UL_PUCCH_N_PUCCH, n_pucch),
            3, "n_PUCCH");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the N_PUCCH parameter in the PUCCH structure settings.
/// HIFN  Determines the N_PUCCH based on the measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR componentCarrier/Component carrier
ViStatus _VI_FUNC rsspecan_OneWebULPUCCHN_PUCCHAutoDetect(ViSession instrSession,
                                                          ViInt32   componentCarrier)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, componentCarrier, 1, 5),
            2, "Component Carrier");
    snprintf(repCap, RS_REPCAP_BUF_SIZE, "cc%ld", componentCarrier);

    checkErr(rsspecan_SetAttributeViString(instrSession, repCap, RSSPECAN_ATTR_ONEWEB_UL_PUCCH_N_PUCCH_AUTO, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the N_PUCCH parameter in the PUCCH structure settings. Selects
/// HIFN  the definition of N_PUCCH on subframe level.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR componentCarrier/Component carrier
ViStatus _VI_FUNC rsspecan_SetOneWebULPUCCHN_PUCCHSubframe(ViSession instrSession,
                                                           ViInt32   componentCarrier)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, componentCarrier, 1, 5),
            2, "Component Carrier");
    snprintf(repCap, RS_REPCAP_BUF_SIZE, "cc%ld", componentCarrier);

    checkErr(rsspecan_SetAttributeViString(instrSession, repCap, RSSPECAN_ATTR_ONEWEB_UL_PUCCH_N_PUCCH_SUBFRAME, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects the PRACH preamble format.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR componentCarrier/Component carrier
/// HIPAR prachConfiguration/Selects the PRACH preamble format.
ViStatus _VI_FUNC rsspecan_ConfigureOneWebULPRACHConfiguration(
                            ViSession instrSession,
                            ViInt32   componentCarrier,
                            ViInt32   prachConfiguration)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, componentCarrier, 1, 5),
            2, "Component Carrier");
    snprintf(repCap, RS_REPCAP_BUF_SIZE, "cc%ld", componentCarrier);

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_ONEWEB_UL_PRACH_CONFIGURATION, prachConfiguration),
            3, "PRACH Configuration");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Turns the restricted preamble set for PRACH on and off.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR componentCarrier/Component carrier
/// HIPAR restrictedSet/Turns the restricted preamble set for PRACH on and off.
ViStatus _VI_FUNC rsspecan_ConfigureOneWebULPRACHRestrictedSet(
                            ViSession instrSession,
                            ViInt32   componentCarrier,
                            ViBoolean restrictedSet)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, componentCarrier, 1, 5),
            2, "Component Carrier");
    snprintf(repCap, RS_REPCAP_BUF_SIZE, "cc%ld", componentCarrier);

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, repCap, RSSPECAN_ATTR_ONEWEB_UL_PRACH_RESTRICTED_SET, restrictedSet),
            3, "Restricted Set");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the PRACH frequency offset.
/// HIFN     The attribute is available for preamble formats 0 to 3.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR componentCarrier/Component carrier
/// HIPAR frequencyOffset/Defines the PRACH frequency offset.
ViStatus _VI_FUNC rsspecan_ConfigureOneWebULPRACHFrequencyOffset(
                            ViSession instrSession,
                            ViInt32   componentCarrier,
                            ViReal64  frequencyOffset)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, componentCarrier, 1, 5),
            2, "Component Carrier");
    snprintf(repCap, RS_REPCAP_BUF_SIZE, "cc%ld", componentCarrier);

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_ONEWEB_UL_PRACH_FREQUENCY_OFFSET, frequencyOffset),
            3, "Frequency Offset");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the Ncs configuration for the PRACH.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR componentCarrier/Component carrier
/// HIPAR ncsConfiguration/Defines the Ncs configuration for the PRACH.
ViStatus _VI_FUNC rsspecan_ConfigureOneWebULPRACHNcsConf(ViSession instrSession,
                                                         ViInt32   componentCarrier,
                                                         ViInt32   ncsConfiguration)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, componentCarrier, 1, 5),
            2, "Component Carrier");
    snprintf(repCap, RS_REPCAP_BUF_SIZE, "cc%ld", componentCarrier);

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_ONEWEB_UL_PRACH_NCS_CONF, ncsConfiguration),
            3, "Ncs Conf");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the PRACH logical root sequence index.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR componentCarrier/Component carrier
/// HIPAR logicalRootSeqIdx/Defines the PRACH logical root sequence index.
ViStatus _VI_FUNC rsspecan_ConfigureOneWebULPRACHLogicalRootSeqIdx(
                            ViSession instrSession,
                            ViInt32   componentCarrier,
                            ViInt32   logicalRootSeqIdx)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, componentCarrier, 1, 5),
            2, "Component Carrier");
    snprintf(repCap, RS_REPCAP_BUF_SIZE, "cc%ld", componentCarrier);

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_ONEWEB_UL_PRACH_LOGICAL_ROOT_SEQ_IDX, logicalRootSeqIdx),
            3, "Logical Root Seq Idx");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects the PRACH sequence index.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR componentCarrier/Component carrier
/// HIPAR sequenceIndex/Selects the PRACH sequence index.
ViStatus _VI_FUNC rsspecan_ConfigureOneWebULPRACHSequenceIndex(
                            ViSession instrSession,
                            ViInt32   componentCarrier,
                            ViInt32   sequenceIndex)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, componentCarrier, 1, 5),
            2, "Component Carrier");
    snprintf(repCap, RS_REPCAP_BUF_SIZE, "cc%ld", componentCarrier);

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_ONEWEB_UL_PRACH_SEQUENCE_INDEX, sequenceIndex),
            3, "Sequence Index");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects the PRACH sequence index. Automatically determines the index.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR componentCarrier/Component carrier
ViStatus _VI_FUNC rsspecan_OneWebULPRACHSequenceIndexAuto(ViSession instrSession,
                                                          ViInt32   componentCarrier)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, componentCarrier, 1, 5),
            2, "Component Carrier");
    snprintf(repCap, RS_REPCAP_BUF_SIZE, "cc%ld", componentCarrier);

    checkErr(rsspecan_SetAttributeViString(instrSession, repCap, RSSPECAN_ATTR_ONEWEB_UL_PRACH_SEQUENCE_INDEX_AUTO, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function adds a window to the display.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR stringSize/This control sets the allocated length of Returned Window Name string.
/// HIPAR referenceWindowName/String containing the number or name of the window the new window is
/// HIPAR referenceWindowName/inserted next to. In the default state, the name of the window is its
/// HIPAR referenceWindowName/index.
/// HIPAR direction/Direction the window is added relative to the window set with
/// HIPAR direction/"Reference Window Name".
/// HIPAR windowType/Type of result display (evaluation method) you want to add.
/// HIPAR returnedWindowName/When adding a new window, the function returns its name (by
ViStatus _VI_FUNC rsspecan_AddOneWebULWindow(ViSession instrSession,
                                             ViInt32   stringSize,
                                             ViString  referenceWindowName,
                                             ViInt32   direction,
                                             ViInt32   windowType,
                                             ViChar    returnedWindowName[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViChar   response[RS_MAX_MESSAGE_BUF_SIZE];
    ViChar*  p2direction;
    ViChar*  p2type;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &rsspecan_rngLayoutDirection, direction, &p2direction),
                4, "Direction");
    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &RsSpecAn_rngOWULLayoutType, windowType, &p2type),
                5, "Window Type");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "LAY:ADD? '%s',%s,%s", referenceWindowName, p2direction, p2type);
    checkErr(RsCore_QueryViString(instrSession, cmd, response));

    if (returnedWindowName != NULL)
    {
        RsCore_TrimString(response, RS_VAL_TRIM_WHITESPACES_AND_ALL_QUOTES);
        RsCore_CopyToUserBufferAsciiData(returnedWindowName, stringSize, response);
    }

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function replaces a window with another one.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR windowName/String containing the number or name of the window you want to
/// HIPAR windowType/Type of result display (evaluation method) you want to add.
ViStatus _VI_FUNC rsspecan_ReplaceOneWebULWindow(ViSession instrSession,
                                                 ViString  windowName,
                                                 ViInt32   windowType)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViChar*  p2type;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &RsSpecAn_rngOWULLayoutType, windowType, &p2type),
                3, "Window Type");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "LAY:REPL '%s',%s", windowName, p2type);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the current synchronization state.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR synchronizationStatus/Queries the current synchronization state.
ViStatus _VI_FUNC rsspecan_QueryOneWebULSynchronizationStatus(
                            ViSession  instrSession,
                            ViBoolean* synchronizationStatus)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_GetAttributeViBoolean(instrSession, NULL, RSSPECAN_ATTR_ONEWEB_UL_SYNCHRONIZATION_STATUS, synchronizationStatus),
            2, "Synchronization Status");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects the scale of the x-axis for result displays that show
/// HIFN  subcarrier results.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR xAxisUnit/Selects the scale of the x-axis for result displays that show
/// HIPAR xAxisUnit/subcarrier results.
ViStatus _VI_FUNC rsspecan_ConfigureOneWebULXAxisUnit(ViSession instrSession,
                                                      ViInt32   xAxisUnit)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, NULL, RSSPECAN_ATTR_ONEWEB_UL_X_AXIS_UNIT, xAxisUnit),
            2, "X Axis Unit");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This attribute specifies the units for EVM trace results.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR evmUnit/This control specifies the units for EVM trace results.
ViStatus _VI_FUNC rsspecan_ConfigureOneWebULEVMUnit(ViSession instrSession,
                                                    ViInt32   evmUnit)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, NULL, RSSPECAN_ATTR_ONEWEB_UL_EVM_UNIT, evmUnit),
            2, "EVM Unit");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the marker position on the z-axis of three-dimensional result
/// HIFN  displays.
/// HIFN     Instead of returning a certain type of value (EVM or Power), which is
/// HIFN  possible with CALCulate<n>:MARKer<m>:Z?, this method returns all types
/// HIFN  of values (EVM and Power), regardless of the result display type.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/Select the window. 0=current window
/// HIPAR evm/EVM at the marker position.
/// HIPAR power/Power at the marker position.
ViStatus _VI_FUNC rsspecan_OneWebULMarkerZ(ViSession instrSession,
                                           ViInt32   window,
                                           ViReal64* evm,
                                           ViReal64* power)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd_buffer[RS_MAX_MESSAGE_BUF_SIZE] = "CALC:MARK:Z:ALL?";
    ViInt32  cnt = 0;
    ViPInt32 res = NULL;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, window, 0, 16),
            2, "Window");
    if (window != 0)
        snprintf(cmd_buffer, RS_MAX_MESSAGE_BUF_SIZE, "CALC%ld:MARK:Z:ALL?", window);
    checkErr(RsCore_QueryAsciiViInt32Array(instrSession, cmd_buffer, &res, &cnt));
    if (cnt >= 2)
    {
        *evm = res[0];
        *power = res[1];
    }
    else
        checkErr(RS_ERROR_UNEXPECTED_RESPONSE);

Error:
    if (res) free(res);
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the results of the selected limit check.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR componentCarrier/Component carrier
/// HIPAR limitType/Selects limit type.
/// HIPAR limitCheckResult/Queries the results of the selected limit check.
/// HIPAR statisticalType/Selects statistical type.
ViStatus _VI_FUNC rsspecan_QueryOneWebULLimitCheckResult(ViSession instrSession,
                                                         ViInt32   componentCarrier,
                                                         ViInt32   limitType,
                                                         ViInt32   statisticalType,
                                                         ViInt32*  limitCheckResult)
{
    ViStatus                                                              error = VI_SUCCESS;
    ViChar                                                                repCap[RS_REPCAP_BUF_SIZE] = "";
    ViUInt32                                                              attributes[8][2] = {
    {RSSPECAN_ATTR_ONEWEB_UL_EVM_ALL_MAX_LIMIT_CHECK_RESULT,              RSSPECAN_ATTR_ONEWEB_UL_EVM_ALL_AVG_LIMIT_CHECK_RESULT},
    {RSSPECAN_ATTR_ONEWEB_UL_EVM_PHYSICAL_CHANNEL_MAX_LIMIT_CHECK_RESULT, RSSPECAN_ATTR_ONEWEB_UL_EVM_PHYSICAL_CHANNEL_AVG_LIMIT_CHECK_RESULT},
    {RSSPECAN_ATTR_ONEWEB_UL_EVM_PHYSICAL_SIGNAL_MAX_LIMIT_CHECK_RESULT,  RSSPECAN_ATTR_ONEWEB_UL_EVM_PHYSICAL_SIGNAL_AVG_LIMIT_CHECK_RESULT},
    {RSSPECAN_ATTR_ONEWEB_UL_EVM_FERR_MAX_LIMIT_CHECK_RESULT,             RSSPECAN_ATTR_ONEWEB_UL_EVM_FERR_AVG_LIMIT_CHECK_RESULT},
    {RSSPECAN_ATTR_ONEWEB_UL_GAIN_IMBALANCE_MAX_LIMIT_CHECK_RESULT,       RSSPECAN_ATTR_ONEWEB_UL_GAIN_IMBALANCE_AVG_LIMIT_CHECK_RESULT},
    {RSSPECAN_ATTR_ONEWEB_UL_IQ_OFFSET_MAX_LIMIT_CHECK_RESULT,            RSSPECAN_ATTR_ONEWEB_UL_IQ_OFFSET_AVG_LIMIT_CHECK_RESULT},
    {RSSPECAN_ATTR_ONEWEB_UL_QUADRATURE_ERROR_MAX_LIMIT_CHECK_RESULT,     RSSPECAN_ATTR_ONEWEB_UL_QUADRATURE_ERROR_AVG_LIMIT_CHECK_RESULT},
    {RSSPECAN_ATTR_ONEWEB_UL_SAMPLING_ERROR_MAX_LIMIT_CHECK_RESULT,       RSSPECAN_ATTR_ONEWEB_UL_SAMPLING_ERROR_AVG_LIMIT_CHECK_RESULT}
                                                                          };

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, limitType, 0, 7),
            3, "Limit Type");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, statisticalType, 1, 2),
            4, "Statistical Type");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, componentCarrier, 1, 5),
            2, "Component Carrier");
    snprintf(repCap, RS_REPCAP_BUF_SIZE, "cc%ld", componentCarrier);

    viCheckParm(rsspecan_GetAttributeViInt32(instrSession, repCap,
                    attributes[limitType][statisticalType - 1], limitCheckResult), 5, "Limit Check Result");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Couples or decouples markers in different result displays to each
/// HIFN  other.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR markerCouplingEnabled/Couples or decouples markers in different result displays to each
/// HIPAR markerCouplingEnabled/other.
ViStatus _VI_FUNC rsspecan_ConfigureOneWebULMarkerCouplingEnabled(
                            ViSession instrSession,
                            ViBoolean markerCouplingEnabled)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, NULL, RSSPECAN_ATTR_ONEWEB_UL_MARKER_COUPLING_ENABLED, markerCouplingEnabled),
            2, "Marker Coupling Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects the subframe to be analyzed.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR componentCarrier/Component carrier
/// HIPAR selectedSubframe/Selects the subframe to be analyzed. -1 = All
/// HIPAR selectedSlot/Filters the results in the constellation diagram by a particular slot.
ViStatus _VI_FUNC rsspecan_ConfigureOneWebULEvaluationRangeGlobal(
                            ViSession instrSession,
                            ViInt32   componentCarrier,
                            ViInt32   selectedSubframe,
                            ViInt32   selectedSlot)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, componentCarrier, 1, 5),
            2, "Component Carrier");
    snprintf(repCap, RS_REPCAP_BUF_SIZE, "cc%ld", componentCarrier);

    if (selectedSubframe >= 0)
    {
        viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_ONEWEB_UL_SELECTED_SUBFRAME, selectedSubframe),
                3, "Selected Subframe");
    }
    else
    {
        checkErr(rsspecan_SetAttributeViString(instrSession, repCap, RSSPECAN_ATTR_ONEWEB_UL_SELECT_ALL_SUBFRAMES, NULL));
    }
    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_ONEWEB_UL_SELECTED_SLOT, selectedSlot),
            4, "Selected Slot");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Filters the results in the constellation diagram.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR componentCarrier/Component carrier
/// HIPAR modulation/Filters the results in the constellation diagram by a certain type of
/// HIPAR modulation/modulation.
/// HIPAR allocation/Filters the displayed results in the constellation diagram by a
/// HIPAR allocation/certain type of allocation. -1 = All Allocations
/// HIPAR symbol/Filters the results in the constellation diagram by a certain OFDM
/// HIPAR symbol/symbol. -1 = All Symbols.
/// HIPAR carrier/Filters the results in the constellation diagram by a certain
/// HIPAR carrier/subcarrier. -1 = All Carriers
ViStatus _VI_FUNC rsspecan_ConfigureOneWebULEvaluationRangeConstellationDiag(
                            ViSession instrSession,
                            ViInt32   componentCarrier,
                            ViInt32   modulation,
                            ViInt32   allocation,
                            ViInt32   symbol,
                            ViInt32   carrier)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, componentCarrier, 1, 5),
            2, "Component Carrier");
    snprintf(repCap, RS_REPCAP_BUF_SIZE, "cc%ld", componentCarrier);

    if (modulation >= 0)
    {
        viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_ONEWEB_UL_SELECTED_MODULATION, modulation),
                3, "Modulation");
    }
    else
    {
        checkErr(rsspecan_SetAttributeViString(instrSession, repCap, RSSPECAN_ATTR_ONEWEB_UL_SELECT_ALL_MODULATIONS, NULL));
    }
    if (allocation != 2)
    {
        viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_ONEWEB_UL_SELECTED_ALLOCATION, allocation),
                4, "Allocation");
    }
    else
    {
        checkErr(rsspecan_SetAttributeViString(instrSession, repCap, RSSPECAN_ATTR_ONEWEB_UL_SELECT_ALL_ALLOCATIONS, NULL));
    }
    if (symbol >= 0)
    {
        viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_ONEWEB_UL_SELECTED_SYMBOL, symbol),
                5, "Symbol");
    }
    else
    {
        checkErr(rsspecan_SetAttributeViString(instrSession, repCap, RSSPECAN_ATTR_ONEWEB_UL_SELECT_ALL_SYMBOLS, NULL));
    }
    if (carrier >= 0)
    {
        viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_ONEWEB_UL_SELECTED_CARRIER, carrier),
                6, "Carrier");
    }
    else
    {
        checkErr(rsspecan_SetAttributeViString(instrSession, repCap, RSSPECAN_ATTR_ONEWEB_UL_SELECT_ALL_CARRIERS, NULL));
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects a certain preamble for measurements that analyze individual
/// HIFN  preambles.
/// HIFN     Select PRACH analysis mode.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR componentCarrier/Component carrier
/// HIPAR preamble/Selects a certain preamble for measurements that analyze individual
/// HIPAR preamble/preambles. -1 = All Preambles
ViStatus _VI_FUNC rsspecan_ConfigureOneWebULEvaluationRangePreamble(
                            ViSession instrSession,
                            ViInt32   componentCarrier,
                            ViInt32   preamble)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, componentCarrier, 1, 5),
            2, "Component Carrier");
    snprintf(repCap, RS_REPCAP_BUF_SIZE, "cc%ld", componentCarrier);

    if (preamble >= 0)
    {
        viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_ONEWEB_UL_SELECTED_PREAMBLE, preamble),
                3, "Preamble");
    }
    else
    {
        checkErr(rsspecan_SetAttributeViString(instrSession, repCap, RSSPECAN_ATTR_ONEWEB_UL_SELECT_ALL_PREAMBLES, NULL));
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  For the ACLR result display.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR arraySize/Number of array points requested.
/// HIPAR result/Returns trace data.
/// HIPAR returnedValues/Returns the number of trace data values.
ViStatus _VI_FUNC rsspecan_OneWebULFetchACLR(ViSession instrSession,
                                             ViInt32   window,
                                             ViInt32   arraySize,
                                             ViReal64  result[],
                                             ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsspecan_FetchYTrace(instrSession, window, 1, arraySize, returnedValues, result));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  For the allocation summary, the method returns OWULAllocationSummary
/// HIFN  object  for each line of the table.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR arraySize/Number of array points requested.
/// HIPAR subframe/Returns subframe numbers.
/// HIPAR allocationID/Represents the allocation ID. The value is a number in the range
/// HIPAR allocationID/{1...-70}.
/// HIPAR numberOfRB/Returns numbers of resource blocks.
/// HIPAR offsetRB/Returns offset RB.
/// HIPAR modulation/Represents the modulation scheme.
/// HIPAR absolutePower/Absolute power in dBm.
/// HIPAR evm/EVM value
/// HIPAR returnedValues/Returns the number of trace data values.
ViStatus _VI_FUNC rsspecan_OneWebULFetchAllocationSummary(ViSession instrSession,
                                                          ViInt32   window,
                                                          ViInt32   arraySize,
                                                          ViInt32   subframe[],
                                                          ViInt32   allocationID[],
                                                          ViInt32   numberOfRB[],
                                                          ViInt32   offsetRB[],
                                                          ViInt32   modulation[],
                                                          ViReal64  absolutePower[],
                                                          ViReal64  evm[],
                                                          ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = ":TRAC? TRACE1";
    ViChar*  response = NULL;
    ViChar*  pbuf;
    ViChar*  pbufEnd;
    ViInt32  idx = 0;
    ViInt32  elements_count;

    checkErr(RsCore_LockSession(instrSession));

    if (window != 0)
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, ":TRAC%ld? TRACE1", window);
    checkErr(RsCore_QueryViStringUnknownLength(instrSession, cmd, &response));

    elements_count = RsCore_GetElementsCount(response, ',') / 7;

    if (arraySize < elements_count)
    {
        viCheckParm(RsCore_InvalidViInt32RangeMin(instrSession, arraySize, elements_count), 3, "Array Size");
    }

    pbuf = response;
    while (idx < elements_count)
    {
        pbufEnd = strchr(pbuf, ',');
        if (pbufEnd == NULL)
        {
            checkErr(RS_ERROR_UNEXPECTED_RESPONSE);
        }
        *pbufEnd = 0;
        if (!isNullOrEmpty(pbuf))
            subframe[idx] = RsCore_Convert_String_To_ViInt32(pbuf);
        else
            subframe[idx] = LONG_MIN;
        pbuf = pbufEnd + 1;

        pbufEnd = strchr(pbuf, ',');
        if (pbufEnd == NULL)
        {
            checkErr(RS_ERROR_UNEXPECTED_RESPONSE);
        }
        *pbufEnd = 0;
        if (!isNullOrEmpty(pbuf))
            allocationID[idx] = RsCore_Convert_String_To_ViInt32(pbuf);
        else
            allocationID[idx] = LONG_MIN;
        pbuf = pbufEnd + 1;

        pbufEnd = strchr(pbuf, ',');
        if (pbufEnd == NULL)
        {
            checkErr(RS_ERROR_UNEXPECTED_RESPONSE);
        }
        *pbufEnd = 0;
        if (!isNullOrEmpty(pbuf))
            numberOfRB[idx] = RsCore_Convert_String_To_ViInt32(pbuf);
        else
            numberOfRB[idx] = LONG_MIN;
        pbuf = pbufEnd + 1;

        pbufEnd = strchr(pbuf, ',');
        if (pbufEnd == NULL)
        {
            checkErr(RS_ERROR_UNEXPECTED_RESPONSE);
        }
        *pbufEnd = 0;
        if (!isNullOrEmpty(pbuf))
            offsetRB[idx] = RsCore_Convert_String_To_ViInt32(pbuf);
        else
            offsetRB[idx] = 0;
        pbuf = pbufEnd + 1;

        pbufEnd = strchr(pbuf, ',');
        if (pbufEnd == NULL)
        {
            checkErr(RS_ERROR_UNEXPECTED_RESPONSE);
        }
        *pbufEnd = 0;
        if (!isNullOrEmpty(pbuf))
            modulation[idx] = RsCore_Convert_String_To_ViInt32(pbuf);
        else
            modulation[idx] = LONG_MIN;
        pbuf = pbufEnd + 1;

        pbufEnd = strchr(pbuf, ',');
        if (pbufEnd == NULL)
        {
            checkErr(RS_ERROR_UNEXPECTED_RESPONSE);
        }
        *pbufEnd = 0;
        if (!isNullOrEmpty(pbuf))
            absolutePower[idx] = RsCore_Convert_String_To_ViReal64(pbuf);
        else
            absolutePower[idx] = RS_VAL_NAN_VI_REAL64;
        pbuf = pbufEnd + 1;

        pbufEnd = strchr(pbuf, ',');
        if (pbufEnd == NULL)
        {
            evm[idx] = RsCore_Convert_String_To_ViReal64(pbuf);
        }
        else
        {
            *pbufEnd = 0;
            if (!isNullOrEmpty(pbuf))
                evm[idx] = RsCore_Convert_String_To_ViReal64(pbuf);
            else
                evm[idx] = RS_VAL_NAN_VI_REAL64;
            pbuf = pbufEnd + 1;
        }
        idx++;
    }

    *returnedValues = idx - 1;

Error:
    if (response) free(response);
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  For the bit stream result display, the method returns five values and
/// HIFN  the bitstream for each line of the table.
/// HIFN     For symbols or bits that are not transmitted, the method returns '9'
/// HIFN     For symbols or bits that could not be decoded because the number of
/// HIFN  layer exceeds the number of receive antennas, the method returns '8'.
/// HIFN     Format:
/// HIFN
/// HIFN  <Subframe>,<AllocationID>,<Codeword>,<Modulation>,<SymbolIndex>,<Bistream
/// HIFN  - values 0,1,8 or 9>
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR arraySize/Number of array points requested.
/// HIPAR result/Returns trace data.
/// HIPAR returnedValues/Returns the number of trace data values.
ViStatus _VI_FUNC rsspecan_OneWebULFetchBitstream(ViSession instrSession,
                                                  ViInt32   window,
                                                  ViInt32   arraySize,
                                                  ViInt32   result[],
                                                  ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, window, 0, 16),
            2, "Window");

    if (window != 0)
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "UNIT:BSTR BIT;:TRAC%ld? TRACE1", window);
    else
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "UNIT:BSTR BIT;:TRAC? TRACE1");

    checkErr(RsCore_QueryIntegerArrayToUserBuffer(instrSession, cmd, arraySize, result, returnedValues));
    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  For the capture buffer result display, the method returns one value
/// HIFN  for each I/Q sample in the capture buffer.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR arraySize/Number of array points requested.
/// HIPAR result/Returns trace data.
/// HIPAR returnedValues/Returns the number of trace data values.
ViStatus _VI_FUNC rsspecan_OneWebULFetchCaptureBuffer(ViSession instrSession,
                                                      ViInt32   window,
                                                      ViInt32   arraySize,
                                                      ViReal64  result[],
                                                      ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsspecan_FetchYTrace(instrSession, window, 1, arraySize, returnedValues, result));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  For the CCDF result display, the type of return values depends on the
/// HIFN  parameter 'axis'.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR axis/Selects the axis to be retrieved.
/// HIPAR arraySize/Number of array points requested.
/// HIPAR result/Returns trace data.
/// HIPAR returnedValues/Returns the number of trace data values.
ViStatus _VI_FUNC rsspecan_OneWebULFetchCCDF(ViSession instrSession,
                                             ViInt32   window,
                                             ViInt32   axis,
                                             ViInt32   arraySize,
                                             ViReal64  result[],
                                             ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsspecan_FetchYTrace(instrSession, window, (axis == RSSPECAN_VAL_AXIS_X) ? 2 : 1, arraySize, returnedValues, result));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  For the channel flatness result display, the method returns one value
/// HIFN  for each trace point.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR statisticalType/Selects the statistical type.
/// HIPAR arraySize/Number of array points requested.
/// HIPAR result/Returns trace data.
/// HIPAR returnedValues/Returns the number of trace data values.
ViStatus _VI_FUNC rsspecan_OneWebULFetchChannelAndSpectrumFlatness(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   statisticalType,
                            ViInt32   arraySize,
                            ViReal64  result[],
                            ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;
    ViInt32  trace = 0;
    ViChar*  traceCmd;

    checkErr(RsCore_LockSession(instrSession));

    //Reading corresponding Trace number from RangeTable
    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &RsSpecAn_rngMinMaxAvg, statisticalType, &traceCmd), 3, "Statistical Type");
    // Convert 'x' from "TRACEx" to corresponding integer value (TRACE1 -> 1, etc)
    trace = (int)traceCmd[5] - 48;

    checkErr(rsspecan_FetchYTrace(instrSession, window, trace, arraySize, returnedValues, result));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  For the channel flatness difference result display, the method returns
/// HIFN  one value for each trace point.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR statisticalType/Selects the statistical type.
/// HIPAR arraySize/Number of array points requested.
/// HIPAR result/Returns trace data.
/// HIPAR returnedValues/Returns the number of trace data values.
ViStatus _VI_FUNC rsspecan_OneWebULFetchChannelAndSpectrumFlatnessDifference(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   statisticalType,
                            ViInt32   arraySize,
                            ViReal64  result[],
                            ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;
    ViInt32  trace = 0;
    ViChar*  traceCmd;

    checkErr(RsCore_LockSession(instrSession));

    //Reading corresponding Trace number from RangeTable
    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &RsSpecAn_rngMinMaxAvg, statisticalType, &traceCmd), 3, "Statistical Type");
    // Convert 'x' from "TRACEx" to corresponding integer value (TRACE1 -> 1, etc)
    trace = (int)traceCmd[5] - 48;

    checkErr(rsspecan_FetchYTrace(instrSession, window, trace, arraySize, returnedValues, result));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  For the channel flatness SRS result display, the method returns one
/// HIFN  value for each trace point.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR statisticalType/Selects the statistical type.
/// HIPAR arraySize/Number of array points requested.
/// HIPAR result/Returns trace data.
/// HIPAR returnedValues/Returns the number of trace data values.
ViStatus _VI_FUNC rsspecan_OneWebULFetchChannelFlatnessSRS(ViSession instrSession,
                                                           ViInt32   window,
                                                           ViInt32   statisticalType,
                                                           ViInt32   arraySize,
                                                           ViReal64  result[],
                                                           ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;
    ViInt32  trace = 0;
    ViChar*  traceCmd;

    checkErr(RsCore_LockSession(instrSession));

    //Reading corresponding Trace number from RangeTable
    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &RsSpecAn_rngMinMaxAvg, statisticalType, &traceCmd), 3, "Statistical Type");
    // Convert 'x' from "TRACEx" to corresponding integer value (TRACE1 -> 1, etc)
    trace = (int)traceCmd[5] - 48;

    checkErr(rsspecan_FetchYTrace(instrSession, window, trace, arraySize, returnedValues, result));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  For the group delay result display, the method returns one value for
/// HIFN  each trace point.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR statisticalType/Selects the statistical type.
/// HIPAR arraySize/Number of array points requested.
/// HIPAR result/Returns trace data.
/// HIPAR returnedValues/Returns the number of trace data values.
ViStatus _VI_FUNC rsspecan_OneWebULFetchGroupDelay(ViSession instrSession,
                                                   ViInt32   window,
                                                   ViInt32   statisticalType,
                                                   ViInt32   arraySize,
                                                   ViReal64  result[],
                                                   ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;
    ViInt32  trace = 0;
    ViChar*  traceCmd;

    checkErr(RsCore_LockSession(instrSession));

    //Reading corresponding Trace number from RangeTable
    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &RsSpecAn_rngMinMaxAvg, statisticalType, &traceCmd), 3, "Statistical Type");
    // Convert 'x' from "TRACEx" to corresponding integer value (TRACE1 -> 1, etc)
    trace = (int)traceCmd[5] - 48;

    checkErr(rsspecan_FetchYTrace(instrSession, window, trace, arraySize, returnedValues, result));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  For the constellation diagram, the method returns one value pair for
/// HIFN  each constellation.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR constellationType/Selects constellation diagram data.
/// HIPAR arraySize/Number of array points requested for each result arrays (both I and
/// HIPAR arraySize/Q).
/// HIPAR i/Returns trace data.
/// HIPAR q/Returns trace data.
/// HIPAR returnedValues/Returns the number of trace data values.
ViStatus _VI_FUNC rsspecan_OneWebULFetchConstellation(ViSession instrSession,
                                                      ViInt32   window,
                                                      ViInt32   constellationType,
                                                      ViInt32   arraySize,
                                                      ViReal64  i[],
                                                      ViReal64  q[],
                                                      ViInt32*  returnedValues)
{
    ViStatus  error = VI_SUCCESS;
    ViPReal64 res = NULL;
    ViInt32   ret_cnt;
    ViInt32   size;
    ViChar    cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, window, 0, 16),
            2, "Window");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, constellationType, 0, 2),
            3, "Constellation Type");

    if (window != 0)
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, ":FORM REAL,32;:TRAC%ld? TRACE%ld", window, constellationType + 1);
    else
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, ":FORM REAL,32;:TRAC? TRACE%ld", constellationType + 1);
    checkErr(RsCore_QueryFloatArray(instrSession, cmd, &res, &ret_cnt));
    checkErr(rsspecan_CheckStatus(instrSession));

    size = (arraySize > (ret_cnt / 2)) ? ret_cnt / 2 : arraySize;

    checkErr(RsCore_DecimateViReal64ArrayToUserBuffers(res, ret_cnt, size, returnedValues, i, q, NULL, NULL, NULL, NULL));

Error:
    if (res) free(res);
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  For the EVM vs carrier result display, the method  returns one value
/// HIFN  for each subcarrier that has been analyzed.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR statisticalType/Selects the statistical type.
/// HIPAR arraySize/Number of array points requested.
/// HIPAR result/Returns trace data.
/// HIPAR returnedValues/Returns the number of trace data values.
ViStatus _VI_FUNC rsspecan_OneWebULFetchEVMvsCarrier(ViSession instrSession,
                                                     ViInt32   window,
                                                     ViInt32   statisticalType,
                                                     ViInt32   arraySize,
                                                     ViReal64  result[],
                                                     ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;
    ViInt32  trace = 0;
    ViChar*  traceCmd;

    checkErr(RsCore_LockSession(instrSession));

    //Reading corresponding Trace number from RangeTable
    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &RsSpecAn_rngMinMaxAvg, statisticalType, &traceCmd), 3, "Statistical Type");
    // Convert 'x' from "TRACEx" to corresponding integer value (TRACE1 -> 1, etc)
    trace = (int)traceCmd[5] - 48;

    checkErr(rsspecan_FetchYTrace(instrSession, window, trace, arraySize, returnedValues, result));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  For the EVM vs subframe result display, the method  returns one value
/// HIFN  for each subframe that has been analyzed.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR arraySize/Number of array points requested.
/// HIPAR result/Returns trace data.
/// HIPAR returnedValues/Returns the number of trace data values.
ViStatus _VI_FUNC rsspecan_OneWebULFetchEVMvsSubframe(ViSession instrSession,
                                                      ViInt32   window,
                                                      ViInt32   arraySize,
                                                      ViReal64  result[],
                                                      ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsspecan_FetchYTrace(instrSession, window, 1, arraySize, returnedValues, result));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  For the EVM vs symbol result display, the method  returns one value
/// HIFN  for each OFDM symbol that has been analyzed.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR arraySize/Number of array points requested.
/// HIPAR result/Returns trace data.
/// HIPAR returnedValues/Returns the number of trace data values.
ViStatus _VI_FUNC rsspecan_OneWebULFetchEVMvsSymbol(ViSession instrSession,
                                                    ViInt32   window,
                                                    ViInt32   arraySize,
                                                    ViReal64  result[],
                                                    ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsspecan_FetchYTrace(instrSession, window, 1, arraySize, returnedValues, result));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  For the EVM vs symbol x carrier result display, the method  returns
/// HIFN  one value for each resource element.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR arraySize/Number of array points requested.
/// HIPAR result/Returns trace data.
/// HIPAR returnedValues/Returns the number of trace data values.
ViStatus _VI_FUNC rsspecan_OneWebULFetchEVMvsSymbolxCarrier(ViSession instrSession,
                                                            ViInt32   window,
                                                            ViInt32   arraySize,
                                                            ViReal64  result[],
                                                            ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsspecan_FetchYTrace(instrSession, window, 1, arraySize, returnedValues, result));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  For the inband emission result display, the number and type of returns
/// HIFN  values depend on the parameter.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR resultType/Selects the result type.
/// HIPAR arraySize/Number of array points requested.
/// HIPAR result/Returns trace data.
/// HIPAR returnedValues/Returns the number of trace data values.
ViStatus _VI_FUNC rsspecan_OneWebULFetchInbandEmission(ViSession instrSession,
                                                       ViInt32   window,
                                                       ViInt32   resultType,
                                                       ViInt32   arraySize,
                                                       ViReal64  result[],
                                                       ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;
    ViInt32  trace = 0;

    checkErr(RsCore_LockSession(instrSession));

    switch (resultType)
    {
    case RSSPECAN_VAL_OW_UL_INBAND_X:
        trace = 1;
        break;
    case RSSPECAN_VAL_OW_UL_INBAND_EMISSION:
        trace = 2;
        break;
    case RSSPECAN_VAL_OW_UL_INBAND_UPPER_LIMIT:
        trace = 3;
        break;
    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, resultType), 3, "Result Type");
        break;
    }

    checkErr(rsspecan_FetchYTrace(instrSession, window, trace, arraySize, returnedValues, result));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  For the power spectrum result display, the method returns one value
/// HIFN  for each trace point.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR arraySize/Number of array points requested.
/// HIPAR result/Returns trace data.
/// HIPAR returnedValues/Returns the number of trace data values.
ViStatus _VI_FUNC rsspecan_OneWebULFetchPowerSpectrum(ViSession instrSession,
                                                      ViInt32   window,
                                                      ViInt32   arraySize,
                                                      ViReal64  result[],
                                                      ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsspecan_FetchYTrace(instrSession, window, 1, arraySize, returnedValues, result));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  For the power vs symbol x carrier, the method returns one value for
/// HIFN  each resource element.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR arraySize/Number of array points requested.
/// HIPAR result/Returns trace data.
/// HIPAR returnedValues/Returns the number of trace data values.
ViStatus _VI_FUNC rsspecan_OneWebULFetchPowervsSymbolxCarrier(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   arraySize,
                            ViReal64  result[],
                            ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsspecan_FetchYTrace(instrSession, window, 1, arraySize, returnedValues, result));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the results of the ACLR measurement or the total signal power
/// HIFN  level of the SEM measurement.
/// HIFN     To get a valid result, you have to perform a complete measurement with
/// HIFN  synchronization to the end of the measurement before reading out the
/// HIFN  result. This is only possible for single sweeps.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR marker/This control selects marker.
/// HIPAR arraySize/Number of array points requested.
/// HIPAR result/Returns trace data.
/// HIPAR returnedValues/Returns the number of trace data values.
ViStatus _VI_FUNC rsspecan_QueryOneWebULACLRMeasurement(ViSession instrSession,
                                                        ViInt32   window,
                                                        ViInt32   marker,
                                                        ViInt32   arraySize,
                                                        ViReal64  result[],
                                                        ViInt32*  returnedValues)
{
    ViStatus  error = VI_SUCCESS;
    ViChar    cmd_buffer[RS_MAX_MESSAGE_BUF_SIZE] = "";
    ViInt32   cnt = 0;
    ViPReal64 res = NULL;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, window, 0, 16),
            2, "Window");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, marker, 0, 16),
            3, "Marker");
    snprintf(cmd_buffer, RS_MAX_MESSAGE_BUF_SIZE, "CALC%ld:MARK%ld:FUNC:POW:RES?", window, marker);

    checkErr(RsCore_QueryAsciiViReal64Array(instrSession, cmd_buffer, &res, &cnt));
    checkErr(rsspecan_CheckStatus(instrSession));

    checkErr(RsCore_CopyToUserBufferViReal64Array(result, arraySize, res, cnt));
    *returnedValues = (cnt > arraySize) ? arraySize : cnt;

Error:
    if (res) free(res);
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the EVM in % or dB, depending on the unit you have set, of
/// HIFN  selected resource elements with selected modulation.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR componentCarrier/Component carrier
/// HIPAR resourceElement/Selects the resource element.
/// HIPAR modulation/Selects the results by a certain type of modulation. This selector
/// HIPAR modulation/does not apply to DMRS PUCCH, PUCCH and PRACH resource elements.
/// HIPAR result/Queries the EVM in % or dB, depending on the unit you have set, of
/// HIPAR result/selected resource elements with selected modulation.
ViStatus _VI_FUNC rsspecan_QueryOneWebULResultFrameEVM(ViSession instrSession,
                                                       ViInt32   componentCarrier,
                                                       ViInt32   resourceElement,
                                                       ViInt32   modulation,
                                                       ViReal64* result)
{
    ViStatus                                                   error = VI_SUCCESS;
    ViChar                                                     repCap[RS_REPCAP_BUF_SIZE] = "";
    ViUInt32                                                   attr = 0;
    ViUInt32                                                   attributes_8psk[7] = {
    RSSPECAN_ATTR_ONEWEB_UL_RESULT_FRAME_EVM_DMRS_PUSCH_8PSK,  RSSPECAN_ATTR_ONEWEB_UL_RESULT_FRAME_EVM_DMRS_PUACH_8PSK,
    RSSPECAN_ATTR_ONEWEB_UL_RESULT_FRAME_EVM_PUSCH_8PSK,       RSSPECAN_ATTR_ONEWEB_UL_RESULT_FRAME_EVM_PUACH_8PSK,
    RSSPECAN_ATTR_ONEWEB_UL_RESULT_FRAME_EVM_DMRS_PUCCH,       RSSPECAN_ATTR_ONEWEB_UL_RESULT_FRAME_EVM_PUCCH, RSSPECAN_ATTR_ONEWEB_UL_RESULT_FRAME_EVM_PRACH
                                                               };
    ViUInt32                                                   attributes_qpsk[7] = {
    RSSPECAN_ATTR_ONEWEB_UL_RESULT_FRAME_EVM_DMRS_PUSCH_QPSK,  RSSPECAN_ATTR_ONEWEB_UL_RESULT_FRAME_EVM_DMRS_PUACH_QPSK,
    RSSPECAN_ATTR_ONEWEB_UL_RESULT_FRAME_EVM_PUSCH_QPSK,       RSSPECAN_ATTR_ONEWEB_UL_RESULT_FRAME_EVM_PUACH_QPSK,
    RSSPECAN_ATTR_ONEWEB_UL_RESULT_FRAME_EVM_DMRS_PUCCH,       RSSPECAN_ATTR_ONEWEB_UL_RESULT_FRAME_EVM_PUCCH, RSSPECAN_ATTR_ONEWEB_UL_RESULT_FRAME_EVM_PRACH
                                                               };
    ViUInt32                                                   attributes_16qam[7] = {
    RSSPECAN_ATTR_ONEWEB_UL_RESULT_FRAME_EVM_DMRS_PUSCH_16QAM, RSSPECAN_ATTR_ONEWEB_UL_RESULT_FRAME_EVM_DMRS_PUACH_16QAM,
    RSSPECAN_ATTR_ONEWEB_UL_RESULT_FRAME_EVM_PUSCH_16QAM,      RSSPECAN_ATTR_ONEWEB_UL_RESULT_FRAME_EVM_PUACH_16QAM,
    RSSPECAN_ATTR_ONEWEB_UL_RESULT_FRAME_EVM_DMRS_PUCCH,       RSSPECAN_ATTR_ONEWEB_UL_RESULT_FRAME_EVM_PUCCH, RSSPECAN_ATTR_ONEWEB_UL_RESULT_FRAME_EVM_PRACH
                                                               };

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, resourceElement, 0, 7),
            3, "Resource Element");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, componentCarrier, 1, 5),
            2, "Component Carrier");
    snprintf(repCap, RS_REPCAP_BUF_SIZE, "cc%ld", componentCarrier);

    switch (modulation)
    {
    case RSSPECAN_VAL_OW_UL_FILTER_MODULATION_QPSK:
        attr = attributes_qpsk[resourceElement];
        break;
    case RSSPECAN_VAL_OW_UL_FILTER_MODULATION_16QAM:
        attr = attributes_16qam[resourceElement];
        break;
    case RSSPECAN_VAL_OW_UL_FILTER_MODULATION_8PSK:
        attr = attributes_8psk[resourceElement];
        break;
    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, modulation), 4, "Modulation");
        break;
    }
    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, repCap, attr, result),
            5, "Result");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the selected result.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR componentCarrier/Component carrier
/// HIPAR measurement/Selects the resource element.
/// HIPAR statisticalType/Selects statistical type.
/// HIPAR result/Returns selected result.
ViStatus _VI_FUNC rsspecan_QueryOneWebULResultSelection(ViSession instrSession,
                                                        ViInt32   componentCarrier,
                                                        ViInt32   measurement,
                                                        ViInt32   statisticalType,
                                                        ViReal64* result)
{
    ViStatus                                                            error = VI_SUCCESS;
    ViChar                                                              repCap[RS_REPCAP_BUF_SIZE] = "";
    ViUInt32                                                            attributes[11][3] = {
    {RSSPECAN_ATTR_ONEWEB_UL_RESULT_SELECTION_CREST_FACTOR_AVG,         RSSPECAN_ATTR_ONEWEB_UL_RESULT_SELECTION_CREST_FACTOR_AVG, RSSPECAN_ATTR_ONEWEB_UL_RESULT_SELECTION_CREST_FACTOR_AVG},
    {RSSPECAN_ATTR_ONEWEB_UL_RESULT_SELECTION_EVM_MIN,                  RSSPECAN_ATTR_ONEWEB_UL_RESULT_SELECTION_EVM_MAX, RSSPECAN_ATTR_ONEWEB_UL_RESULT_SELECTION_EVM_AVG},
    {RSSPECAN_ATTR_ONEWEB_UL_RESULT_SELECTION_EVM_PHYSICAL_CHANNEL_MIN, RSSPECAN_ATTR_ONEWEB_UL_RESULT_SELECTION_EVM_PHYSICAL_CHANNEL_MAX, RSSPECAN_ATTR_ONEWEB_UL_RESULT_SELECTION_EVM_PHYSICAL_CHANNEL_AVG},
    {RSSPECAN_ATTR_ONEWEB_UL_RESULT_SELECTION_EVM_PHYSICAL_SIGNAL_MIN,  RSSPECAN_ATTR_ONEWEB_UL_RESULT_SELECTION_EVM_PHYSICAL_SIGNAL_MAX, RSSPECAN_ATTR_ONEWEB_UL_RESULT_SELECTION_EVM_PHYSICAL_SIGNAL_AVG},
    {RSSPECAN_ATTR_ONEWEB_UL_RESULT_SELECTION_FREQUENCY_ERROR_MIN,      RSSPECAN_ATTR_ONEWEB_UL_RESULT_SELECTION_FREQUENCY_ERROR_MAX, RSSPECAN_ATTR_ONEWEB_UL_RESULT_SELECTION_FREQUENCY_ERROR_AVG},
    {RSSPECAN_ATTR_ONEWEB_UL_RESULT_SELECTION_GAIN_IMBALANCE_MIN,       RSSPECAN_ATTR_ONEWEB_UL_RESULT_SELECTION_GAIN_IMBALANCE_MAX, RSSPECAN_ATTR_ONEWEB_UL_RESULT_SELECTION_GAIN_IMBALANCE_AVG},
    {RSSPECAN_ATTR_ONEWEB_UL_RESULT_SELECTION_IQ_OFFSET_MIN,            RSSPECAN_ATTR_ONEWEB_UL_RESULT_SELECTION_IQ_OFFSET_MAX, RSSPECAN_ATTR_ONEWEB_UL_RESULT_SELECTION_IQ_OFFSET_AVG},
    {RSSPECAN_ATTR_ONEWEB_UL_RESULT_SELECTION_POWER_MIN,                RSSPECAN_ATTR_ONEWEB_UL_RESULT_SELECTION_POWER_MAX, RSSPECAN_ATTR_ONEWEB_UL_RESULT_SELECTION_POWER_AVG},
    {RSSPECAN_ATTR_ONEWEB_UL_RESULT_SELECTION_QUADRATURE_ERROR_MIN,     RSSPECAN_ATTR_ONEWEB_UL_RESULT_SELECTION_QUADRATURE_ERROR_MAX, RSSPECAN_ATTR_ONEWEB_UL_RESULT_SELECTION_QUADRATURE_ERROR_AVG},
    {RSSPECAN_ATTR_ONEWEB_UL_RESULT_SELECTION_SAMPLING_ERROR_MIN,       RSSPECAN_ATTR_ONEWEB_UL_RESULT_SELECTION_SAMPLING_ERROR_MAX, RSSPECAN_ATTR_ONEWEB_UL_RESULT_SELECTION_SAMPLING_ERROR_AVG},
    {RSSPECAN_ATTR_ONEWEB_UL_RESULT_SELECTION_START_OFFSET,             RSSPECAN_ATTR_ONEWEB_UL_RESULT_SELECTION_START_OFFSET, RSSPECAN_ATTR_ONEWEB_UL_RESULT_SELECTION_START_OFFSET},
                                                                        };

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, measurement, 0, 10),
            3, "Measurement");
    if ((measurement != RSSPECAN_VAL_OW_UL_MEAS_CREST_FACTOR) && (measurement != RSSPECAN_VAL_OW_UL_MEAS_STOF))
    {
        viCheckParm(RsCore_InvalidViInt32Range(instrSession, statisticalType, 0, 2),
                4, "Statistical Type");
    }
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, componentCarrier, 1, 5),
            2, "Component Carrier");
    snprintf(repCap, RS_REPCAP_BUF_SIZE, "cc%ld", componentCarrier);

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, repCap, attributes[measurement][statisticalType], result),
            5, "Result");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}
