/*****************************************************************************
 *------------------------ Utility  -----------------------------------------*
 *****************************************************************************/
#include <float.h>
#include "rsspecan.h"
#include "rscore.h"

/*****************************************************************************
 *- Callback Declarations ---------------------------------------------------*
 *****************************************************************************/
#pragma warning( disable : 4152 )

/*****************************************************************************
 *- Common Callback Declarations --------------------------------------------*
 *****************************************************************************/

#define FSV_INSTRUMENTS "FSV|FSVR|FSVA|FPL"

/*****************************************************************************
 * Function: rsspecan_InitAttributes
 * Purpose:  This function inits attributes to the desired values if needed.
 *****************************************************************************/
ViStatus rsspecan_InitAttributes(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    // - Driver Identification -
    checkErr(RsCore_SetAttributeViString(instrSession, "", RS_ATTR_SPECIFIC_DRIVER_DESCRIPTION, 0, RSSPECAN_SPECIFIC_DRIVER_DESCRIPTION));
    checkErr(RsCore_SetAttributeViString(instrSession, "", RS_ATTR_SPECIFIC_DRIVER_PREFIX, 0, RSSPECAN_SPECIFIC_DRIVER_PREFIX));
    checkErr(RsCore_SetAttributeViString(instrSession, "", RS_ATTR_SPECIFIC_DRIVER_LOCATOR, 0, RSSPECAN_SPECIFIC_DRIVER_LOCATOR));
    checkErr(RsCore_SetAttributeViString(instrSession, "", RS_ATTR_SPECIFIC_DRIVER_VENDOR, 0, RSSPECAN_SPECIFIC_DRIVER_VENDOR));
    checkErr(RsCore_SetAttributeViInt32(instrSession, "", RS_ATTR_SPECIFIC_DRIVER_CLASS_SPEC_MAJOR_VERSION, 0, RSSPECAN_CLASS_SPEC_MAJOR_VERSION));
    checkErr(RsCore_SetAttributeViInt32(instrSession, "", RS_ATTR_SPECIFIC_DRIVER_CLASS_SPEC_MINOR_VERSION, 0, RSSPECAN_CLASS_SPEC_MINOR_VERSION));

    // - Driver Capabilities -
    checkErr(RsCore_SetAttributeViString(instrSession, "", RS_ATTR_SUPPORTED_INSTRUMENT_MODELS, 0, RSSPECAN_SUPPORTED_INSTRUMENT_MODELS));
    checkErr(RsCore_SetAttributeViString(instrSession, "", RS_ATTR_GROUP_CAPABILITIES, 0, RSSPECAN_GROUP_CAPABILITIES));
    checkErr(RsCore_SetAttributeViString(instrSession, "", RS_ATTR_FUNCTION_CAPABILITIES, 0, RSSPECAN_FUNCTION_CAPABILITIES));

    // - Version Info -
    checkErr(RsCore_SetAttributeViInt32(instrSession, "", RS_ATTR_SPECIFIC_DRIVER_MAJOR_VERSION, 0, RSSPECAN_MAJOR_VERSION));
    checkErr(RsCore_SetAttributeViInt32(instrSession, "", RS_ATTR_SPECIFIC_DRIVER_MINOR_VERSION, 0, RSSPECAN_MINOR_VERSION));
    checkErr(RsCore_SetAttributeViInt32(instrSession, "", RS_ATTR_SPECIFIC_DRIVER_MINOR_MINOR_VERSION, 0, RSSPECAN_MINOR_MINOR_VERSION));

    checkErr(RsCore_SetAttributeViInt32(instrSession, "", RS_ATTR_CLASS_DRIVER_MAJOR_VERSION, 0, RSSPECAN_CLASS_SPEC_MAJOR_VERSION));
    checkErr(RsCore_SetAttributeViInt32(instrSession, "", RS_ATTR_CLASS_DRIVER_MINOR_VERSION, 0, RSSPECAN_CLASS_SPEC_MINOR_VERSION));

    checkErr(RsCore_SetSpecificDriverRevision(instrSession, RS_ATTR_SPECIFIC_DRIVER_REVISION));

Error:
    return error;
}

/*****************************************************************************
 * Function: rsspecan_DefaultInstrSetup
 * Purpose:  This function sends a default setup to the instrument. The
 *           rsspecan_reset function calls this function. The
 *           rsspecan_RsInit function calls this function when the
 *           user passes VI_FALSE for the reset parameter. This function is
 *           useful for configuring settings that other instrument driver
 *           functions require.
 *
 *           Note:  Call this function only when the session is locked.
 *****************************************************************************/
ViStatus rsspecan_DefaultInstrSetup(ViSession instrSession)
{
    ViStatus         error = VI_SUCCESS;
    RsCoreSessionPtr rsSession = NULL;

    checkErr(RsCore_GetRsSession(instrSession, &rsSession));

    // Set all the attributes to the default state. Do not update inherent attributes!
    checkErr(RsCore_ApplyAttributeDefaults(instrSession, VI_FALSE));

    // Init attributes
    checkErr(rsspecan_InitAttributes(instrSession));

    rsSession->fastSweepInstrument = 0;
    checkErr(RsCore_ResetRegistersEseSre(instrSession));
    checkErr(RsCore_Write(instrSession, "INST SAN;:INIT:CONT 0"));
    checkErr(RsCore_QueryViInt32(instrSession, "*OPC?", NULL));

Error:
    return error;
}

/*****************************************************************************
 * Function: rsspecan_CheckStatus
 * Purpose:  This function checks the status of the instrument to detect
 *           whether the instrument has encountered an error. This function
 *           is called at the end of most exported driver functions. If the
 *           instrument reports an error, this function returns
 *           RS_ERROR_INSTRUMENT_SPECIFIC. The user can set the
 *           rsspecan_ATTR_QUERY_INSTRUMENT_STATUS attribute to VI_FALSE to disable this
 *           check and increase execution speed.
 *
 *           Note:  Call this function only when the session is locked.
 *****************************************************************************/
ViStatus rsspecan_CheckStatus(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    if (RsCore_QueryInstrStatus(instrSession) && !RsCore_Simulating(instrSession))
    {
        checkErr(RsCore_CheckStatus(instrSession, VI_SUCCESS));
    }

Error:
    return error;
}

/*****************************************************************************
 * Function: Parse Limit Line Catalog
 * Purpose:  Parse Limit Line Catalog returned from instrument and removes
 *           from it all unneeded parameters from this list.
*****************************************************************************/
ViStatus rsspecan_ParseLimitLineCatalog(ViChar *buffer, ViInt32 size, ViChar *catalogList, ViInt32 *nbValues)
{
    ViStatus error = VI_SUCCESS;
    ViChar *plist_of_files = NULL;
    ViChar *ptag = NULL;
    ViChar *pend = NULL;
    ViInt32 i = 0;
    ViInt32 format_result = -1;

    if (*buffer != '\0')
    {
        plist_of_files = (ViChar*)malloc(strlen(buffer) + 1);

        format_result = sscanf(buffer, "%*ld,%*ld,%[^\r\n]", plist_of_files);
        if ((format_result > 0) && (strlen(plist_of_files) > 0))
        {
            pend = plist_of_files;
            *pend = '\0';
            ptag = strtok(buffer, ",");
            while (ptag != NULL && error == VI_SUCCESS)
            {
                i++;
                pend += sprintf(pend, "%s,", ptag + 1);
                ptag = strtok(NULL, ",");
                if (ptag == NULL)
                    error = RS_ERROR_INVALID_VALUE;
                else
                    ptag = strtok(NULL, ",");
            }
            *--pend = '\0';
            strncpy(catalogList, plist_of_files, size);
        }
    }

    if (nbValues)
        *nbValues = i;

    if (plist_of_files)
        free(plist_of_files);

    return error;
}

/****************************************************************************************************
 *  rsspecan_dataReadTrace
    Queries Float array in BIN or ASCII format (sets BIN format, but can handle both)
    Sends command TRAC%ld? <trace> or TRAC? <trace>
    You have to provide an allocated traceData[] buffer
******************************************************************************************************/
ViStatus rsspecan_dataReadTrace(ViSession instrSession,
                                ViInt32   window,
                                ViString  trace,
                                ViInt32   arrayLength,
                                ViReal64  traceData[],
                                ViPInt32  noofPoints)
{  
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    if (window == 0)
    {
        if (trace != NULL && strlen(trace) > 0)
            snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, ":FORM REAL,32;:TRAC? %s", trace);
        else
            snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, ":FORM REAL,32;:TRAC?");
    }
    else
    {
        if (trace != NULL && strlen(trace) > 0)
            snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, ":FORM REAL,32;:TRAC%ld? %s", window, trace);
        else
            snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, ":FORM REAL,32;:TRAC%ld?", window);
    }

    checkErr(RsCore_QueryFloatArrayToUserBuffer(instrSession, cmd, arrayLength, traceData, noofPoints));
    checkErr(RsCore_Write(instrSession, ":FORM ASC"));

Error:
    return error;
}

/****************************************************************************************************
 *  rsspecan_dataReadTraceLinear
    Queries Float array in BIN or ASCII format (sets BIN format, but can handle both)
    Sends command TRAC%ld:LIN? <trace> or TRAC:LIN? <trace>
    You have to provide an allocated traceData[] buffer
******************************************************************************************************/
ViStatus rsspecan_dataReadTraceLinear(ViSession instrSession,
                                      ViInt32   window,
                                      ViString  trace,
                                      ViInt32   arrayLength,
                                      ViReal64  traceData[],
                                      ViPInt32  noofPoints)
{  
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    if (window == 0)
    {
        if (trace != NULL && strlen(trace) > 0)
            snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, ":FORM REAL,32;:TRAC:LIN? %s", trace);
        else
            snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, ":FORM REAL,32;:TRAC:LIN?");
    }
    else
    {
        if (trace != NULL && strlen(trace) > 0)
            snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, ":FORM REAL,32;:TRAC%ld:LIN? %s", window, trace);
        else
            snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, ":FORM REAL,32;:TRAC%ld:LIN?", window);
    }

    checkErr(RsCore_QueryFloatArrayToUserBuffer(instrSession, cmd, arrayLength, traceData, noofPoints));

Error:
    return error;
}

/****************************************************************************************************
 *  rsspecan_dataQueryInterleavedData
    Queries two Float arrays that are interleaved (ItemA_0, ItemB_0, ItemA_1, ItemB_1, ... ItemA_n, Item B_n)
    Use it e.g. for IQ data read
    arraySize = arrayA size = arrayB size
******************************************************************************************************/
ViStatus _VI_FUNC rsspecan_dataReadInterleaved(ViSession instrSession,
                                               ViString  cmd,
                                               ViInt32   timeout,
                                               ViInt32   arraySize,
                                               ViReal64  arrayA[],
                                               ViReal64  arrayB[],
                                               ViPInt32  actualPoints)
{  
    ViStatus error = VI_SUCCESS;
    ViInt32  i = 0;
    ViInt32  dataSize = 0;
    ViInt32  noofPoints = 0;
    ViReal64 *data = NULL;

    checkErr(RsCore_QueryFloatArrayWithOpc(instrSession, cmd, timeout, &data, &dataSize));

    noofPoints = dataSize / 2;
    if (actualPoints)
        *actualPoints = noofPoints;

    if (noofPoints > arraySize)
        noofPoints = arraySize;

    for (i = 0; i < noofPoints; i++)
    {
        arrayA[i] = data[i * 2];  // NOLINT(bugprone-implicit-widening-of-multiplication-result)
        arrayB[i] = data[i * 2 + 1];
    }

Error:
    if (data)
        free(data);

    return error;
}

/****************************************************************************************************
 *  rsspecan_IsFSV
    Returns true if the current instrument model is for FSV, FPL, FSVR, FSVA
******************************************************************************************************/
ViBoolean rsspecan_IsFSV(ViSession instrSession)
{
    return RsCore_IsInstrumentModel(instrSession, FSV_INSTRUMENTS);
}

/****************************************************************************************************
 *  rsspecan_AssertFSV
    Generates error, if the current model is not FSV, FPL, FSVR, FSVA
******************************************************************************************************/
ViStatus rsspecan_AssertFSV(ViSession instrSession)
{
    return RsCore_CheckInstrumentModel(instrSession, FSV_INSTRUMENTS);
}

/****************************************************************************************************
 *  rsspecan_AssertNotFSV
    Generates error, if the current model is FSV, FPL, FSVR, FSVA
******************************************************************************************************/
ViStatus rsspecan_AssertNotFSV(ViSession instrSession)
{
    return RsCore_CheckInstrumentModel(instrSession, "!"FSV_INSTRUMENTS);
}


ViStatus rsspecan_utility_error_message(ViSession instrSession,
                                        ViStatus  errorCode,
                                        ViChar    errorMessage[])
{
    ViStatus error = VI_SUCCESS;

    (void)RsCore_LockSession(instrSession);

    static RsCoreStringValueTable errorTable =
    {
        RSSPECAN_ERROR_CODES_AND_MSGS,
        {0, NULL}
    };

    // all VISA and RS error codes are handled as well as codes in the table
    viCheckParm(RsCore_InvalidNullPointer(instrSession, errorMessage), 3, "Null address for Error Message");

    checkErr(RsCore_GetSpecificDriverStatusDesc(instrSession, errorCode, errorMessage, 256, errorTable));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}