/*****************************************************************************
 *  Rohde&Schwarz Spectrum Analyzer instrument driver
 *  Verizon5G options K118/K119
 *
 *  Original Release: November 2019
 *  By: Jiri Kominek
 *
 *  Should you have any technical questions please contact the hotline of
 *  Rohde & Schwarz Vertriebs-GmbH Rohde & Schwarz Support Center
 *
 *  e-mail: CustomerSupport@rohde-schwarz.com
 *
 *****************************************************************************/
#include "rsspecan.h"

/// HIFN  Selects a new measurement channel with V5GTF channel type.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsspecan_SetV5GTFMode(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsspecan_SetAttributeViString(instrSession, NULL, RSSPECAN_ATTR_V5GTF_MODE, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects the link direction of the signal (V5GTF measurement mode).
/// HIFN     Measuring downlink signals required option R&SFSW-K118.
/// HIFN     Measuring uplink signals required option R&SFSW-K119.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR linkDirection/Selects the link direction of the signal (V5GTF measurement mode).
ViStatus _VI_FUNC rsspecan_ConfigureV5GTFLinkDirection(ViSession instrSession,
                                                       ViInt32   linkDirection)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, NULL, RSSPECAN_ATTR_V5GTF_LINK_DIRECTION, linkDirection),
            2, "Link Direction");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Adjusts the amplitude settings, including attenuator and preamplifier,
/// HIFN  to achieve the optimal EVM using the maximum dynamic range.
/// HIFN     Note that this process can up to several minutes, depending on the
/// HIFN  number of component carriers you are measuring.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsspecan_V5GTFAutoEVM(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsspecan_SetAttributeViString(instrSession, NULL, RSSPECAN_ATTR_V5GTF_AUTO_EVM, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects the number of component carriers analyzed in the measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR numberOfComponentCarriers/Selects the number of component carriers analyzed in the measurement.
ViStatus _VI_FUNC rsspecan_ConfigureV5GTFNumberOfComponentCarriers(
                            ViSession instrSession,
                            ViInt32   numberOfComponentCarriers)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, NULL, RSSPECAN_ATTR_V5GTF_NUMBER_OF_COMPONENT_CARRIERS, numberOfComponentCarriers),
            2, "Number Of Component Carriers");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects the capture mode for measurements on multiple component
/// HIFN  carriers.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR ccSignalCapture/Selects the capture mode for measurements on multiple component
/// HIPAR ccSignalCapture/carriers.
ViStatus _VI_FUNC rsspecan_ConfigureV5GTFCCSignalCapture(ViSession instrSession,
                                                         ViInt32   ccSignalCapture)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, NULL, RSSPECAN_ATTR_V5GTF_CC_SIGNAL_CAPTURE, ccSignalCapture),
            2, "CC Signal Capture");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Saves the signal description.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR componentCarrier/Component carrier
/// HIPAR fileName/String containing the path and name of the file.
ViStatus _VI_FUNC rsspecan_V5GTFSaveDemodSettings(ViSession instrSession,
                                                  ViInt32   componentCarrier,
                                                  ViString  fileName)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, componentCarrier, 1, 16),
            2, "Component Carrier");
    snprintf(repCap, RS_REPCAP_BUF_SIZE, "cc%ld", componentCarrier);

    viCheckParm(rsspecan_SetAttributeViString(instrSession, repCap, RSSPECAN_ATTR_V5GTF_SAVE_DEMOD_SETTINGS, fileName),
            2, "File Name");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Saves the signal description of multiple carriers in a single file.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR fileName/String containing the path and name of the file.
ViStatus _VI_FUNC rsspecan_V5GTFSaveDemodAllSettings(ViSession instrSession,
                                                     ViString  fileName)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViString(instrSession, NULL, RSSPECAN_ATTR_V5GTF_SAVE_DEMOD_ALL_SETTINGS, fileName),
            2, "File Name");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Restores saved signal description.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR componentCarrier/Component carrier
/// HIPAR fileName/String containing the path and name of the file.
ViStatus _VI_FUNC rsspecan_V5GTFRestoreSavedDemodSettings(ViSession instrSession,
                                                          ViInt32   componentCarrier,
                                                          ViString  fileName)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, componentCarrier, 1, 16),
            2, "Component Carrier");
    snprintf(repCap, RS_REPCAP_BUF_SIZE, "cc%ld", componentCarrier);

    viCheckParm(rsspecan_SetAttributeViString(instrSession, repCap, RSSPECAN_ATTR_V5GTF_RESTORE_SAVED_DEMOD_SETTINGS, fileName),
            3, "File Name");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Restores the signal description of multiple carriers from a single
/// HIFN  file.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR fileName/String containing the path and name of the file.
ViStatus _VI_FUNC rsspecan_V5GTFRestoreSavedDemodAllSettings(ViSession instrSession,
                                                             ViString  fileName)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViString(instrSession, NULL, RSSPECAN_ATTR_V5GTF_RESTORE_SAVED_DEMOD_ALL_SETTINGS, fileName),
            2, "File Name");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the center frequency for RF measurements.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR componentCarrier/Component carrier
/// HIPAR carrierFrequency/Sets the center frequency for RF measurements.
ViStatus _VI_FUNC rsspecan_ConfigureV5GTFCarrierFrequency(ViSession instrSession,
                                                          ViInt32   componentCarrier,
                                                          ViReal64  carrierFrequency)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, componentCarrier, 1, 16),
            2, "Component Carrier");
    snprintf(repCap, RS_REPCAP_BUF_SIZE, "cc%ld", componentCarrier);
    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_V5GTF_CARRIER_FREQUENCY, carrierFrequency),
            3, "Carrier Frequency");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the general frequency offset.
/// HIFN     For measurements on multiple component carriers, the attribute defines
/// HIFN  the frequency offset for a component carrier.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR componentCarrier/Component carrier
/// HIPAR frequencyOffset/Defines the general frequency offset.
ViStatus _VI_FUNC rsspecan_ConfigureV5GTFFrequencyOffset(ViSession instrSession,
                                                         ViInt32   componentCarrier,
                                                         ViReal64  frequencyOffset)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, componentCarrier, 2, 16),
            2, "Component Carrier");
    snprintf(repCap, RS_REPCAP_BUF_SIZE, "cc%ld", componentCarrier);

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_V5GTF_FREQUENCY_OFFSET, frequencyOffset),
            3, "Frequency Offset");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the downlink cell ID.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR linkDirection/Selects the link direction of the signal (V5GTF measurement mode).
/// HIPAR componentCarrier/Component carrier
/// HIPAR cellID/Defines the downlink cell ID.
ViStatus _VI_FUNC rsspecan_ConfigureV5GTFCellID(ViSession instrSession,
                                                ViInt32   linkDirection,
                                                ViInt32   componentCarrier,
                                                ViInt32   cellID)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, linkDirection, 0, 1),
            2, "Link Direction");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, componentCarrier, 1, 16),
            3, "Component Carrier");
    snprintf(repCap, RS_REPCAP_BUF_SIZE, "cc%ld", componentCarrier);

    switch (linkDirection)
    {
    case RSSPECAN_VAL_V5GTF_DIRECTION_DL:
        viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_V5GTF_DL_CELL_ID, cellID),
                4, "Cell ID");
        break;
    case RSSPECAN_VAL_V5GTF_DIRECTION_UL:
        viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_V5GTF_UL_CELL_ID, cellID),
                4, "Cell ID");
        break;
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the number of configurable subframes.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR linkDirection/Selects the link direction of the signal (V5GTF measurement mode).
/// HIPAR componentCarrier/Component carrier
/// HIPAR configurableSubframes/Defines the number of configurable subframes.
ViStatus _VI_FUNC rsspecan_ConfigureV5GTFConfigurableSubframes(
                            ViSession instrSession,
                            ViInt32   linkDirection,
                            ViInt32   componentCarrier,
                            ViInt32   configurableSubframes)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, linkDirection, 0, 1),
            2, "Link Direction");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, componentCarrier, 1, 16),
            3, "Component Carrier");
    snprintf(repCap, RS_REPCAP_BUF_SIZE, "cc%ld", componentCarrier);

    switch (linkDirection)
    {
    case RSSPECAN_VAL_V5GTF_DIRECTION_DL:
        viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_V5GTF_DL_CONFIGURABLE_SUBFRAMES, configurableSubframes),
                4, "Configurable Subframes");
        break;
    case RSSPECAN_VAL_V5GTF_DIRECTION_UL:
        viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_V5GTF_UL_CONFIGURABLE_SUBFRAMES, configurableSubframes),
                4, "Configurable Subframes");
        break;
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects the subframe allocation.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR allocation/Selects the subframe allocation.
/// HIPAR componentCarrier/Component carrier
/// HIPAR linkDirection/Selects the link direction of the signal (V5GTF measurement mode).
/// HIPAR subframe/Selects a specific subframe.
ViStatus _VI_FUNC rsspecan_ConfigureV5GTFRadioFrameSubframeAllocation(
                            ViSession instrSession,
                            ViInt32   linkDirection,
                            ViInt32   componentCarrier,
                            ViInt32   subframe,
                            ViInt32   allocation)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, linkDirection, 0, 1),
            2, "Link Direction");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, componentCarrier, 1, 16),
            3, "Component Carrier");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, subframe, 0, 49),
            4, "Subframe");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "cc%ld,v5gsubf%ld", componentCarrier, subframe);

    switch (linkDirection)
    {
    case RSSPECAN_VAL_V5GTF_DIRECTION_DL:
        viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_V5GTF_RADIO_FRAME_DL_SUBFRAME_ALLOCATION, allocation),
                5, "Allocation");
        break;
    case RSSPECAN_VAL_V5GTF_DIRECTION_UL:
        viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_V5GTF_RADIO_FRAME_UL_SUBFRAME_ALLOCATION, allocation),
                5, "Allocation");
        break;
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects an optional reference signal that is transmitted in a
/// HIFN  subframe.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR optionalRefSignals/Selects an optional reference signal that is transmitted in a
/// HIPAR optionalRefSignals/subframe.
/// HIPAR subframe/Selects a specific subframe.
/// HIPAR componentCarrier/Component carrier
ViStatus _VI_FUNC rsspecan_ConfigureV5GTFRadioFrameDLSubframeOptionalRefSignals(
                            ViSession instrSession,
                            ViInt32   componentCarrier,
                            ViInt32   subframe,
                            ViInt32   optionalRefSignals)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, componentCarrier, 1, 16),
            2, "Component Carrier");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, subframe, 0, 49),
            3, "Subframe");
    snprintf(repCap, RS_REPCAP_BUF_SIZE, "cc%ld,v5gsubf%ld", componentCarrier, subframe);
    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_V5GTF_RADIO_FRAME_DL_SUBFRAME_OPTIONAL_REF_SIGNALS, optionalRefSignals),
            4, "Optional Ref Signals");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects a specific subframe.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR linkDirection/Selects the link direction of the signal (V5GTF measurement mode).
/// HIPAR componentCarrier/Component carrier
/// HIPAR subframe/Selects a specific subframe.
/// HIPAR selectedSubframe/Selects a specific subframe.
ViStatus _VI_FUNC rsspecan_ConfigureV5GTFRadioFrameSubframeSelected(
                            ViSession instrSession,
                            ViInt32   linkDirection,
                            ViInt32   componentCarrier,
                            ViInt32   subframe,
                            ViInt32   selectedSubframe)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, linkDirection, 0, 1),
            2, "Link Direction");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, componentCarrier, 1, 16),
            3, "Component Carrier");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, subframe, 0, 49),
            4, "Subframe");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "cc%ld,v5gsubf%ld", componentCarrier, subframe);

    switch (linkDirection)
    {
    case RSSPECAN_VAL_V5GTF_DIRECTION_DL:
        viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_V5GTF_RADIO_FRAME_DL_SUBFRAME_SELECTED, selectedSubframe),
                5, "Selected Subframe");
        break;
    case RSSPECAN_VAL_V5GTF_DIRECTION_UL:
        viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_V5GTF_RADIO_FRAME_UL_SUBFRAME_SELECTED, selectedSubframe),
                5, "Selected Subframe");
        break;
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects the subframe type.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR linkDirection/Selects the link direction of the signal (V5GTF measurement mode).
/// HIPAR componentCarrier/Component carrier
/// HIPAR subframe/Selects a specific subframe.
/// HIPAR subframeType/Selects the subframe type.
ViStatus _VI_FUNC rsspecan_ConfigureV5GTFRadioFrameSubframeType(
                            ViSession instrSession,
                            ViInt32   linkDirection,
                            ViInt32   componentCarrier,
                            ViInt32   subframe,
                            ViInt32   subframeType)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, linkDirection, 0, 1),
            2, "Link Direction");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, componentCarrier, 1, 16),
            3, "Component Carrier");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, subframe, 0, 49),
            4, "Subframe");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "cc%ld,v5gsubf%ld", componentCarrier, subframe);

    switch (linkDirection)
    {
    case RSSPECAN_VAL_V5GTF_DIRECTION_DL:
        viCheckParm(RsCore_InvalidViInt32Range(instrSession, subframeType, RSSPECAN_VAL_V5GTF_SUBFRAME_TYPE_A, RSSPECAN_VAL_V5GTF_SUBFRAME_TYPE_B),
                5, "Subframe Type");

        viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_V5GTF_RADIO_FRAME_DL_SUBFRAME_TYPE, subframeType),
                5, "Subframe Type");
        break;
    case RSSPECAN_VAL_V5GTF_DIRECTION_UL:
        viCheckParm(RsCore_InvalidViInt32Range(instrSession, subframeType, RSSPECAN_VAL_V5GTF_SUBFRAME_TYPE_C, RSSPECAN_VAL_V5GTF_SUBFRAME_TYPE_D),
                5, "Subframe Type");

        viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_V5GTF_RADIO_FRAME_UL_SUBFRAME_TYPE, subframeType),
                5, "Subframe Type");
        break;
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Copies a specific subframe configuration to the clipboard.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR linkDirection/Selects the link direction of the signal (V5GTF measurement mode).
/// HIPAR componentCarrier/Component carrier
/// HIPAR subframe/Selects a specific subframe.
ViStatus _VI_FUNC rsspecan_V5GTFRadioFrameSubframeCopy(ViSession instrSession,
                                                       ViInt32   linkDirection,
                                                       ViInt32   componentCarrier,
                                                       ViInt32   subframe)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, linkDirection, 0, 1),
            2, "Link Direction");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, componentCarrier, 1, 16),
            3, "Component Carrier");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, subframe, 0, 49),
            4, "Subframe");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "cc%ld,v5gsubf%ld", componentCarrier, subframe);

    switch (linkDirection)
    {
    case RSSPECAN_VAL_V5GTF_DIRECTION_DL:
        checkErr(rsspecan_SetAttributeViString(instrSession, repCap, RSSPECAN_ATTR_V5GTF_RADIO_FRAME_DL_SUBFRAME_COPY, NULL));
        break;
    case RSSPECAN_VAL_V5GTF_DIRECTION_UL:
        checkErr(rsspecan_SetAttributeViString(instrSession, repCap, RSSPECAN_ATTR_V5GTF_RADIO_FRAME_UL_SUBFRAME_COPY, NULL));
        break;
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Applies a previously copied subframe configuration to another
/// HIFN  subframe.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR linkDirection/Selects the link direction of the signal (V5GTF measurement mode).
/// HIPAR componentCarrier/Component carrier
/// HIPAR subframe/Selects a specific subframe.
ViStatus _VI_FUNC rsspecan_V5GTFRadioFrameSubframePaste(ViSession instrSession,
                                                        ViInt32   linkDirection,
                                                        ViInt32   componentCarrier,
                                                        ViInt32   subframe)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, linkDirection, 0, 1),
            2, "Link Direction");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, componentCarrier, 1, 16),
            3, "Component Carrier");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, subframe, 0, 49),
            4, "Subframe");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "cc%ld,v5gsubf%ld", componentCarrier, subframe);

    switch (linkDirection)
    {
    case RSSPECAN_VAL_V5GTF_DIRECTION_DL:
        checkErr(rsspecan_SetAttributeViString(instrSession, repCap, RSSPECAN_ATTR_V5GTF_RADIO_FRAME_DL_SUBFRAME_PASTE, NULL));
        break;
    case RSSPECAN_VAL_V5GTF_DIRECTION_UL:
        checkErr(rsspecan_SetAttributeViString(instrSession, repCap, RSSPECAN_ATTR_V5GTF_RADIO_FRAME_UL_SUBFRAME_PASTE, NULL));
        break;
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Applies a previously copied subframe configuration to all other
/// HIFN  subframes.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR linkDirection/Selects the link direction of the signal (V5GTF measurement mode).
/// HIPAR componentCarrier/Component carrier
/// HIPAR subframe/Selects a specific subframe.
ViStatus _VI_FUNC rsspecan_V5GTFRadioFrameSubframePasteAll(ViSession instrSession,
                                                           ViInt32   linkDirection,
                                                           ViInt32   componentCarrier,
                                                           ViInt32   subframe)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, linkDirection, 0, 1),
            2, "Link Direction");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, componentCarrier, 1, 16),
            3, "Component Carrier");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, subframe, 0, 49),
            4, "Subframe");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "cc%ld,v5gsubf%ld", componentCarrier, subframe);

    switch (linkDirection)
    {
    case RSSPECAN_VAL_V5GTF_DIRECTION_DL:
        checkErr(rsspecan_SetAttributeViString(instrSession, repCap, RSSPECAN_ATTR_V5GTF_RADIO_FRAME_DL_SUBFRAME_PASTE_ALL, NULL));
        break;
    case RSSPECAN_VAL_V5GTF_DIRECTION_UL:
        checkErr(rsspecan_SetAttributeViString(instrSession, repCap, RSSPECAN_ATTR_V5GTF_RADIO_FRAME_UL_SUBFRAME_PASTE_ALL, NULL));
        break;
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Resets the radio frame configuration to its default state.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR linkDirection/Selects the link direction of the signal (V5GTF measurement mode).
/// HIPAR componentCarrier/Component carrier
/// HIPAR subframe/Selects a specific subframe.
ViStatus _VI_FUNC rsspecan_V5GTFRadioFrameSubframeReset(ViSession instrSession,
                                                        ViInt32   linkDirection,
                                                        ViInt32   componentCarrier,
                                                        ViInt32   subframe)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, linkDirection, 0, 1),
            2, "Link Direction");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, componentCarrier, 1, 16),
            3, "Component Carrier");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, subframe, 0, 49),
            4, "Subframe");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "cc%ld,v5gsubf%ld", componentCarrier, subframe);

    switch (linkDirection)
    {
    case RSSPECAN_VAL_V5GTF_DIRECTION_DL:
        checkErr(rsspecan_SetAttributeViString(instrSession, repCap, RSSPECAN_ATTR_V5GTF_RADIO_FRAME_DL_SUBFRAME_RESET, NULL));
        break;
    case RSSPECAN_VAL_V5GTF_DIRECTION_UL:
        checkErr(rsspecan_SetAttributeViString(instrSession, repCap, RSSPECAN_ATTR_V5GTF_RADIO_FRAME_UL_SUBFRAME_RESET, NULL));
        break;
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the number of allocations used in a subframe.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR componentCarrier/Component carrier
/// HIPAR subframe/Selects a specific subframe.
/// HIPAR allocations/Defines the number of allocations used in a subframe.
ViStatus _VI_FUNC rsspecan_ConfigureV5GTFxPDSCHSubframeAllocations(
                            ViSession instrSession,
                            ViInt32   componentCarrier,
                            ViInt32   subframe,
                            ViInt32   allocations)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, componentCarrier, 1, 16),
            2, "Component Carrier");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, subframe, 0, 49),
            3, "Subframe");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "cc%ld,v5gsubf%ld", componentCarrier, subframe);
    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_V5GTF_XPDSCH_SUBFRAME_ALLOCATIONS, allocations),
            4, "Allocations");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  The xPDSCH configuration table contains functionality to configure the
/// HIFN  allocations used in the selected subframe.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR componentCarrier/Component carrier
/// HIPAR subframe/Selects a specific subframe.
/// HIPAR allocation/Selects a specific allocation.
/// HIPAR idOrN_RNTI/Defines the ID or N_RNTI of a xPDSCH allocation.
/// HIPAR modulation/Selects the modulation of a xPDSCH allocation.
/// HIPAR numberOfRB/Defines the number of resource blocks occupied by a xPDSCH allocation.
/// HIPAR offsetRB/Defines a resource block offset of a xPDSCH allocation.
/// HIPAR power/Defines the relative power of a xPDSCH allocation.
ViStatus _VI_FUNC rsspecan_ConfigureV5GTFxPDSCHSubframeTable(ViSession instrSession,
                                                             ViInt32   componentCarrier,
                                                             ViInt32   subframe,
                                                             ViInt32   allocation,
                                                             ViInt32   idOrN_RNTI,
                                                             ViInt32   modulation,
                                                             ViInt32   numberOfRB,
                                                             ViInt32   offsetRB,
                                                             ViReal64  power)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, componentCarrier, 1, 16),
            2, "Component Carrier");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, subframe, 0, 49),
            3, "Subframe");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, allocation, 0, 16),
            4, "Allocation");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "cc%ld,v5gsubf%ld,v5gDLAlloc%ld", componentCarrier, subframe, allocation);
    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_V5GTF_XPDSCH_SUBFRAME_ID_OR_N_RNTI, idOrN_RNTI),
            5, "ID Or N_RNTI");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_V5GTF_XPDSCH_SUBFRAME_MODULATION, modulation),
            6, "Modulation");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_V5GTF_XPDSCH_SUBFRAME_NUMBER_OF_RB, numberOfRB),
            7, "Number Of RB");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_V5GTF_XPDSCH_SUBFRAME_OFFSET_RB, offsetRB),
            8, "Offset RB");

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_V5GTF_XPDSCH_SUBFRAME_POWER, power),
            9, "Power");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects the precoding scheme of a xPDSCH allocation.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR componentCarrier/Component carrier
/// HIPAR subframe/Selects a specific subframe.
/// HIPAR allocation/Selects a specific allocation.
/// HIPAR precoding/Selects the precoding scheme of a xPDSCH allocation.
ViStatus _VI_FUNC rsspecan_ConfigureV5GTFxPDSCHPrecoding(ViSession instrSession,
                                                         ViInt32   componentCarrier,
                                                         ViInt32   subframe,
                                                         ViInt32   allocation,
                                                         ViInt32   precoding)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, componentCarrier, 1, 16),
            2, "Component Carrier");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, subframe, 0, 49),
            3, "Subframe");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, allocation, 0, 16),
            4, "Allocation");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "cc%ld,v5gsubf%ld,v5gDLAlloc%ld", componentCarrier, subframe, allocation);
    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_V5GTF_XPDSCH_SUBFRAME_PRECODING, precoding),
            5, "Precoding");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects the antenna ports on which a xPDSCH allocation is transmitted.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR componentCarrier/Component carrier
/// HIPAR subframe/Selects a specific subframe.
/// HIPAR allocation/Selects a specific allocation.
/// HIPAR antennaPort/Selects the antenna ports on which a xPDSCH allocation is transmitted.
/// HIPAR noOfLayers/Selects the number of layers used by a xPDSCH allocation.
ViStatus _VI_FUNC rsspecan_ConfigureV5GTFxPDSCHGeneralSettings(
                            ViSession instrSession,
                            ViInt32   componentCarrier,
                            ViInt32   subframe,
                            ViInt32   allocation,
                            ViInt32   antennaPort,
                            ViInt32   noOfLayers)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, componentCarrier, 1, 16),
            2, "Component Carrier");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, subframe, 0, 49),
            3, "Subframe");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, allocation, 0, 16),
            4, "Allocation");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "cc%ld,v5gsubf%ld,v5gDLAlloc%ld", componentCarrier, subframe, allocation);
    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_V5GTF_XPDSCH_SUBFRAME_NO_OF_LAYERS, noOfLayers),
            6, "No Of Layers");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_V5GTF_XPDSCH_SUBFRAME_ANTENNA_PORT, antennaPort),
            5, "Antenna Port");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects the scrambling ID for the reference signals transmitted in a
/// HIFN  xPDSCH allocation.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR componentCarrier/Component carrier
/// HIPAR subframe/Selects a specific subframe.
/// HIPAR allocation/Selects a specific allocation.
/// HIPAR scramblingIdentityn_SCID/Selects the scrambling ID for the reference signals transmitted in a
/// HIPAR scramblingIdentityn_SCID/xPDSCH allocation.
ViStatus _VI_FUNC rsspecan_ConfigureV5GTFxPDSCHScramblingIdentityn_SCID(
                            ViSession instrSession,
                            ViInt32   componentCarrier,
                            ViInt32   subframe,
                            ViInt32   allocation,
                            ViInt32   scramblingIdentityn_SCID)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, componentCarrier, 1, 16),
            2, "Component Carrier");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, subframe, 0, 49),
            3, "Subframe");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, allocation, 0, 16),
            4, "Allocation");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "cc%ld,v5gsubf%ld,v5gDLAlloc%ld", componentCarrier, subframe, allocation);
    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_V5GTF_XPDSCH_SUBFRAME_SCRAMBLING_IDENTITY_N_SCID, scramblingIdentityn_SCID),
            5, "Scrambling Identity n_SCID");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  The V5G standard (V5G.211) defines two methods by which the DMRS
/// HIFN  sequence can be calculated. You can select the method with the
/// HIFN  'Sequence Generation' parameter. The sequence is either calculated with
/// HIFN  the 'n_ID_DMRS' variable (a pseudo-random seed value). Or, if the
/// HIFN  higher layers provide no value for n_ID^DMRS, the sequence is generated
/// HIFN  based on the cell ID with the 'n_ID_Cell'. n_ID_Cell has the same value
/// HIFN  as the cell ID.
/// HIFN
/// HIFN     You can define the power with which the DMRS is transmitted as a value
/// HIFN  relative to xPDSCH. By default, its power is 6 dB higher than the
/// HIFN  xPDSCH.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR componentCarrier/Component carrier
/// HIPAR subframe/Selects a specific subframe.
/// HIPAR allocation/Selects a specific allocation.
/// HIPAR n_id_dmrs/Defines the value that the DMRS sequence is based on.
/// HIPAR power/Defines the power of the DMRS relative to the xPDSCH.
/// HIPAR sequenceGeneration/Selects the method with which the DMRS sequence is calculated.
ViStatus _VI_FUNC rsspecan_ConfigureV5GTFxPDSCHDMRS(ViSession instrSession,
                                                    ViInt32   componentCarrier,
                                                    ViInt32   subframe,
                                                    ViInt32   allocation,
                                                    ViReal64  power,
                                                    ViInt32   sequenceGeneration,
                                                    ViInt32   n_id_dmrs)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, componentCarrier, 1, 16),
            2, "Component Carrier");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, subframe, 0, 49),
            3, "Subframe");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, allocation, 0, 16),
            4, "Allocation");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "cc%ld,v5gsubf%ld,v5gDLAlloc%ld", componentCarrier, subframe, allocation);

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_V5GTF_XPDSCH_SUBFRAME_DMRS_POWER, power),
            5, "Power");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_V5GTF_XPDSCH_SUBFRAME_DMRS_SEQUENCE_GENERATION, sequenceGeneration),
            6, "Sequence Generation");
    if (sequenceGeneration == RSSPECAN_VAL_V5GTF_SEQ_NIDMRS)
    {
        viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_V5GTF_XPDSCH_SUBFRAME_DMRS_ID, n_id_dmrs),
                7, "n_ID_DMRS");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  The PCRS is a reference signal for phase noise compensation that you
/// HIFN  can transmit in a xPDSCH allocation.
/// HIFN
/// HIFN     If the PCRS is present (turned on), you can define various properties
/// HIFN  of that reference signal.
/// HIFN     The 'Power' defines the power with which the PCRS is transmitted. The
/// HIFN  value is a power level in dB relative to the power of the xPDSCH
/// HIFN  allocation it is transmitted on.
/// HIFN     The 'Antenna Ports' selects the antenna port it is transmitted on (60
/// HIFN  or 61, or 60 and 61).
/// HIFN     The 'Sequence Generation' selects the method by which the PCRS
/// HIFN  sequence is calculated. The standard (V5G.211) defines two methods:
/// HIFN  n_ID^DMRS and n_ID^Cell. In the latter case, the value is the cell ID.
/// HIFN
/// HIFN     Note: The sequence generation method is always the same for both the
/// HIFN  DMRS and the PCRS. If you select "n_ID^Cell" for either, the R&S FSW
/// HIFN  automatically selects the method for the other, and vice versa.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR componentCarrier/Component carrier
/// HIPAR subframe/Selects a specific subframe.
/// HIPAR allocation/Selects a specific allocation.
/// HIPAR pcrsEnabled/Turns the PCRS in a xPDSCH allocation on and off.
/// HIPAR n_id_dmrs/Defines the value that the PCRS sequence is based on.
/// HIPAR power/Defines the power of the PCRS relative to the xPDSCH allocation it is
/// HIPAR power/transmitted on.
/// HIPAR antennaPort/Selects the antenna port on which the PCRS is transmitted.
/// HIPAR sequenceGeneration/Selects the method with which the PCRS sequence is calculated.
ViStatus _VI_FUNC rsspecan_ConfigureV5GTFxPDSCHPCRS(ViSession instrSession,
                                                    ViInt32   componentCarrier,
                                                    ViInt32   subframe,
                                                    ViInt32   allocation,
                                                    ViBoolean pcrsEnabled,
                                                    ViReal64  power,
                                                    ViInt32   antennaPort,
                                                    ViInt32   sequenceGeneration,
                                                    ViInt32   n_id_dmrs)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, componentCarrier, 1, 16),
            2, "Component Carrier");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, subframe, 0, 49),
            3, "Subframe");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, allocation, 0, 16),
            4, "Allocation");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "cc%ld,v5gsubf%ld,v5gDLAlloc%ld", componentCarrier, subframe, allocation);

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, repCap, RSSPECAN_ATTR_V5GTF_XPDSCH_SUBFRAME_PCRS_ENABLED, pcrsEnabled),
            5, "PCRS Enabled");

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_V5GTF_XPDSCH_SUBFRAME_PCRS_POWER, power),
            6, "Power");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_V5GTF_XPDSCH_SUBFRAME_PCRS_ANTENNA_PORT, antennaPort),
            7, "Antenna Port");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_V5GTF_XPDSCH_SUBFRAME_PCRS_SEQUENCE_GENERATION, sequenceGeneration),
            8, "Sequence Generation");
    if (sequenceGeneration == RSSPECAN_VAL_V5GTF_SEQ_NIPC)
    {
        viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_V5GTF_XPDSCH_SUBFRAME_PCRS_ID, n_id_dmrs),
                9, "n_ID_DMRS");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the number of allocations used in a subframe.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR componentCarrier/Component carrier
/// HIPAR subframe/Selects a specific subframe.
/// HIPAR allocations/Defines the number of allocations used in a subframe.
ViStatus _VI_FUNC rsspecan_ConfigureV5GTFxPUSCHSubframeAllocations(
                            ViSession instrSession,
                            ViInt32   componentCarrier,
                            ViInt32   subframe,
                            ViInt32   allocations)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, componentCarrier, 1, 16),
            2, "Component Carrier");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, subframe, 0, 49),
            3, "Subframe");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "cc%ld,v5gsubf%ld", componentCarrier, subframe);
    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_V5GTF_XPUSCH_SUBFRAME_ALLOCATIONS, allocations),
            4, "Allocations");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  The xPUSCH configuration table contains functionality to configure the
/// HIFN  allocations used in the selected subframe.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR componentCarrier/Component carrier
/// HIPAR subframe/Selects a specific subframe.
/// HIPAR allocation/Selects a specific allocation.
/// HIPAR modulation/Selects the modulation of a xPUSCH allocation.
/// HIPAR numberOfRB/Defines the number of resource blocks occupied by a xPUSCH allocation.
/// HIPAR offsetRB/Defines a resource block offset of a xPDSCH allocation.
/// HIPAR power/Defines the relative power of a xPUSCH allocation.
ViStatus _VI_FUNC rsspecan_ConfigureV5GTFxPUSCHSubframeTable(ViSession instrSession,
                                                             ViInt32   componentCarrier,
                                                             ViInt32   subframe,
                                                             ViInt32   allocation,
                                                             ViInt32   modulation,
                                                             ViInt32   numberOfRB,
                                                             ViInt32   offsetRB,
                                                             ViReal64  power)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, componentCarrier, 1, 16),
            2, "Component Carrier");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, subframe, 0, 49),
            3, "Subframe");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, allocation, 0, 1),
            4, "Allocation");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "cc%ld,v5gsubf%ld,v5gULAlloc%ld", componentCarrier, subframe, allocation);

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_V5GTF_XPUSCH_SUBFRAME_MODULATION, modulation),
            5, "Modulation");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_V5GTF_XPUSCH_SUBFRAME_NUMBER_OF_RB, numberOfRB),
            6, "Number Of RB");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_V5GTF_XPUSCH_SUBFRAME_OFFSET_RB, offsetRB),
            7, "Offset RB");

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_V5GTF_XPUSCH_SUBFRAME_POWER, power),
            8, "Power");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects the RE mapping index for the PCRS.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR componentCarrier/Component carrier
/// HIPAR subframe/Selects a specific subframe.
/// HIPAR allocation/Selects a specific allocation.
/// HIPAR reMappingIndex/Selects the RE mapping index for the PCRS.
ViStatus _VI_FUNC rsspecan_ConfigureV5GTFxPUSCHREMappingIndex(
                            ViSession instrSession,
                            ViInt32   componentCarrier,
                            ViInt32   subframe,
                            ViInt32   allocation,
                            ViInt32   reMappingIndex)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, componentCarrier, 1, 16),
            2, "Component Carrier");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, subframe, 0, 49),
            3, "Subframe");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, allocation, 0, 1),
            4, "Allocation");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "cc%ld,v5gsubf%ld,v5gULAlloc%ld", componentCarrier, subframe, allocation);
    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_V5GTF_XPUSCH_SUBFRAME_RE_MAPPING_INDEX, reMappingIndex),
            5, "RE Mapping Index");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects the scrambling ID for the reference signals transmitted in a
/// HIFN  xPUSCH allocation.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR scramblingIdentityn_SCID/Selects the scrambling ID for the reference signals transmitted in a
/// HIPAR scramblingIdentityn_SCID/xPUSCH allocation.
/// HIPAR componentCarrier/Component carrier
/// HIPAR subframe/Selects a specific subframe.
/// HIPAR allocation/Selects a specific allocation.
ViStatus _VI_FUNC rsspecan_ConfigureV5GTFxPUSCHScramblingIdentityn_SCID(
                            ViSession instrSession,
                            ViInt32   componentCarrier,
                            ViInt32   subframe,
                            ViInt32   allocation,
                            ViInt32   scramblingIdentityn_SCID)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, componentCarrier, 1, 16),
            2, "Component Carrier");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, subframe, 0, 49),
            3, "Subframe");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, allocation, 0, 1),
            4, "Allocation");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "cc%ld,v5gsubf%ld,v5gULAlloc%ld", componentCarrier, subframe, allocation);
    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_V5GTF_XPUSCH_SUBFRAME_UE_SCRAMBLING_IDENTITY_N_SCID, scramblingIdentityn_SCID),
            5, "Scrambling Identity n_SCID");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  The V5G standard (V5G.211) defines two methods by which the DMRS
/// HIFN  sequence can be calculated. You can select the method with the
/// HIFN  'Sequence Generation' parameter. The sequence is either calculated with
/// HIFN  the 'n_ID_DMRS' variable (a pseudo-random seed value). Or, if the
/// HIFN  higher layers provide no value for n_ID^DMRS, the sequence is generated
/// HIFN  based on the cell ID with the 'n_ID_Cell'. n_ID_Cell has the same value
/// HIFN  as the cell ID.
/// HIFN
/// HIFN     You can define the power with which the DMRS is transmitted as a value
/// HIFN  relative to xPUSCH. By default, its power is 6 dB higher than the
/// HIFN  xPUSCH.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR componentCarrier/Component carrier
/// HIPAR subframe/Selects a specific subframe.
/// HIPAR allocation/Selects a specific allocation.
/// HIPAR n_id_dmrs/Defines the value that the DMRS sequence is based on.
/// HIPAR power/Defines the power of the DMRS relative to the xPUSCH.
/// HIPAR sequenceGeneration/Selects the method with which the DMRS sequence is calculated.
ViStatus _VI_FUNC rsspecan_ConfigureV5GTFxPUSCHDMRS(ViSession instrSession,
                                                    ViInt32   componentCarrier,
                                                    ViInt32   subframe,
                                                    ViInt32   allocation,
                                                    ViReal64  power,
                                                    ViInt32   sequenceGeneration,
                                                    ViInt32   n_id_dmrs)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, componentCarrier, 1, 16),
            2, "Component Carrier");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, subframe, 0, 49),
            3, "Subframe");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, allocation, 0, 1),
            4, "Allocation");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "cc%ld,v5gsubf%ld,v5gULAlloc%ld", componentCarrier, subframe, allocation);

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_V5GTF_XPUSCH_SUBFRAME_DMRS_POWER, power),
            5, "Power");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_V5GTF_XPUSCH_SUBFRAME_DMRS_SEQUENCE_GENERATION, sequenceGeneration),
            6, "Sequence Generation");
    if (sequenceGeneration == RSSPECAN_VAL_V5GTF_SEQ_NIDMRS)
    {
        viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_V5GTF_XPUSCH_SUBFRAME_DMRS_ID, n_id_dmrs),
                7, "n_ID_DMRS");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  The PCRS is a reference signal for phase noise compensation that you
/// HIFN  can transmit in a xPUSCH allocation.
/// HIFN
/// HIFN     If the PCRS is present (turned on), you can define various properties
/// HIFN  of that reference signal.
/// HIFN     The 'Power' defines the power with which the PCRS is transmitted. The
/// HIFN  value is a power level in dB relative to the power of the xPDSCH
/// HIFN  allocation it is transmitted on.
/// HIFN     The 'Sequence Generation' selects the method by which the PCRS
/// HIFN  sequence is calculated. The standard (V5G.211) defines two methods:
/// HIFN  n_ID^DMRS and n_ID^Cell. In the latter case, the value is the cell ID.
/// HIFN
/// HIFN     Note: The sequence generation method is always the same for both the
/// HIFN  DMRS and the PCRS. If you select "n_ID^Cell" for either, the R&S FSW
/// HIFN  automatically selects the method for the other, and vice versa.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR componentCarrier/Component carrier
/// HIPAR subframe/Selects a specific subframe.
/// HIPAR allocation/Selects a specific allocation.
/// HIPAR pcrsEnabled/Turns the PCRS in a xPUSCH allocation on and off.
/// HIPAR n_id_dmrs/Defines the value that the PCRS sequence is based on.
/// HIPAR power/Defines the power of the PCRS relative to the xPUSCH allocation it is
/// HIPAR power/transmitted on.
/// HIPAR transmission/Selects the UL PCRS transmission mode.
/// HIPAR sequenceGeneration/Selects the method with which the PCRS sequence is calculated.
ViStatus _VI_FUNC rsspecan_ConfigureV5GTFxPUSCHPCRS(ViSession instrSession,
                                                    ViInt32   componentCarrier,
                                                    ViInt32   subframe,
                                                    ViInt32   allocation,
                                                    ViBoolean pcrsEnabled,
                                                    ViReal64  power,
                                                    ViInt32   transmission,
                                                    ViInt32   sequenceGeneration,
                                                    ViInt32   n_id_dmrs)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, componentCarrier, 1, 16),
            2, "Component Carrier");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, subframe, 0, 49),
            3, "Subframe");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, allocation, 0, 1),
            4, "Allocation");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "cc%ld,v5gsubf%ld,v5gULAlloc%ld", componentCarrier, subframe, allocation);

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, repCap, RSSPECAN_ATTR_V5GTF_XPUSCH_SUBFRAME_PCRS_ENABLED, pcrsEnabled),
            5, "PCRS Enabled");

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_V5GTF_XPUSCH_SUBFRAME_PCRS_POWER, power),
            6, "Power");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_V5GTF_XPUSCH_SUBFRAME_PCRS_TRANSMISSION, transmission),
            7, "Transmission");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_V5GTF_XPUSCH_SUBFRAME_PCRS_SEQUENCE_GENERATION, sequenceGeneration),
            8, "Sequence Generation");
    if (sequenceGeneration == RSSPECAN_VAL_V5GTF_SEQ_NIPC)
    {
        viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_V5GTF_XPUSCH_SUBFRAME_PCRS_ID, n_id_dmrs),
                9, "n_ID_DMRS");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  The synchronization signals in a V5GTF radio frame are always
/// HIFN  transmitted on subframes 0 and 25. The V5GTF standard specifies three
/// HIFN  synchronization signals, which are always present in the radio frame.
/// HIFN  The location of the synchronization signals within the subframe and the
/// HIFN  allocated resource elements are fix.
/// HIFN
/// HIFN     - Primary synchronization signal (P-Sync): The P-Sync is used for
/// HIFN  radio frame synchronization.
/// HIFN     - Secondary synchronization signal (S-Sync): The S-Sync is used for
/// HIFN  radio frame synchronization.
/// HIFN     - Extended synchronization signal (E-Sync): The E-Sync to identify the
/// HIFN  OFDM symbol index.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR componentCarrier/Component carrier
/// HIPAR eSyncRelPower/Defines the relative power of the extended synchronization signal
/// HIPAR eSyncRelPower/(E-SYNC).
/// HIPAR pSyncRelPower/Defines the relative power of the primary synchronization signal
/// HIPAR pSyncRelPower/(PSS).
/// HIPAR sSyncRelPower/Defines the relative power of the secondary synchronization signal
/// HIPAR sSyncRelPower/(SSS).
ViStatus _VI_FUNC rsspecan_ConfigureV5GTFSynchronizationSignal(
                            ViSession instrSession,
                            ViInt32   componentCarrier,
                            ViReal64  pSyncRelPower,
                            ViReal64  sSyncRelPower,
                            ViReal64  eSyncRelPower)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, componentCarrier, 1, 16),
            2, "Component Carrier");
    snprintf(repCap, RS_REPCAP_BUF_SIZE, "cc%ld", componentCarrier);

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_V5GTF_SYNCHRONIZATION_SIGNAL_PSS_POWER, pSyncRelPower),
            3, "P-Sync Rel Power");

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_V5GTF_SYNCHRONIZATION_SIGNAL_SSS_POWER, sSyncRelPower),
            4, "S-Sync Rel Power");

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_V5GTF_SYNCHRONIZATION_SIGNAL_ESYNC_POWER, eSyncRelPower),
            5, "E-Sync Rel Power");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the ePBCH parameters.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR componentCarrier/Component carrier
/// HIPAR precoding/Selects the precoding scheme of a ePBCH allocation.
/// HIPAR relPower/Defines the relative power of the ePBCH.
ViStatus _VI_FUNC rsspecan_ConfigureV5GTFReferenceSignalePBCH(
                            ViSession instrSession,
                            ViInt32   componentCarrier,
                            ViInt32   precoding,
                            ViReal64  relPower)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, componentCarrier, 1, 16),
            2, "Component Carrier");
    snprintf(repCap, RS_REPCAP_BUF_SIZE, "cc%ld", componentCarrier);

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_V5GTF_REFERENCE_SIGNAL_EPBCH_PRECODING, precoding),
            3, "Precoding");

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_V5GTF_REFERENCE_SIGNAL_EPBCH_POWER, relPower),
            4, "Rel Power");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects the precoding scheme of a xPDSCH allocation.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR precoding/Selects the precoding scheme of a xPDSCH allocation.
/// HIPAR componentCarrier/Component carrier
ViStatus _VI_FUNC rsspecan_ConfigureV5GTFReferenceSignalxPDSCHPrecoding(
                            ViSession instrSession,
                            ViInt32   componentCarrier,
                            ViInt32   precoding)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, componentCarrier, 1, 16),
            2, "Component Carrier");
    snprintf(repCap, RS_REPCAP_BUF_SIZE, "cc%ld", componentCarrier);

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_V5GTF_REFERENCE_SIGNAL_XPDSCH_PRECODING, precoding),
            3, "Precoding");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  The channel state information (CSI) reference signal is used to
/// HIFN  estimate the properties of the signal propagation channel from the base
/// HIFN  station to the user equipment. This information is quantized and fed
/// HIFN  back to the base station.
/// HIFN
/// HIFN     It can be transmitted in OFDM symbol 12 (antenna ports 16 to 23) and
/// HIFN  in the last OFDM symbol 13 (antenna ports 16 to 31).
/// HIFN
/// HIFN     You can define the 'Power' of a CSI reference signal resource element
/// HIFN  relative to the power of the BRS.
/// HIFN
/// HIFN     The 'N_ID_CSI' parameter defines the initial (seed) value by which the
/// HIFN  reference signal sequence is generated.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR componentCarrier/Component carrier
/// HIPAR subframe/Selects a specific subframe.
/// HIPAR n_id_csi/Defines the initial (seed) value by which the reference signal
/// HIPAR n_id_csi/sequence is generated (N_ID^CSI).
/// HIPAR power/Defines the relative power of the CSI reference signal.
ViStatus _VI_FUNC rsspecan_ConfigureV5GTFReferenceSignalCSI(ViSession instrSession,
                                                            ViInt32   componentCarrier,
                                                            ViInt32   subframe,
                                                            ViInt32   n_id_csi,
                                                            ViReal64  power)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, componentCarrier, 1, 16),
            2, "Component Carrier");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, subframe, 0, 49),
            3, "Subframe");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "cc%ld,v5gsubf%ld", componentCarrier, subframe);

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_V5GTF_REFERENCE_SIGNAL_SUBFRAME_N_ID_CSI, n_id_csi),
            4, "N_ID_CSI");

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_V5GTF_REFERENCE_SIGNAL_SUBFRAME_POWER, power),
            5, "Power");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Turns the transmission of the xPBCH and BRS on specific OFDM symbols
/// HIFN  in subframes 0 and 25 on and off.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR componentCarrier/Component carrier
/// HIPAR subframe/Selects a specific subframe.
/// HIPAR ofdmSymbol/Selects a specific OFDM symbol.
/// HIPAR symbolTransmissionEnabled/Turns the transmission of the xPBCH and BRS on specific OFDM symbols
/// HIPAR symbolTransmissionEnabled/in subframes 0 and 25 on and off.
ViStatus _VI_FUNC rsspecan_ConfigureV5GTFReferenceSignalSubframeSymbolTransmissionEnabled(
                            ViSession instrSession,
                            ViInt32   componentCarrier,
                            ViInt32   subframe,
                            ViInt32   ofdmSymbol,
                            ViInt32   symbolTransmissionEnabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, componentCarrier, 1, 16),
            2, "Component Carrier");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, subframe, 0, 49),
            3, "Subframe");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, ofdmSymbol, 0, 13),
            4, "OFDM Symbol");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "cc%ld,v5gsubf%ld,v5gOFDMs%ld", componentCarrier, subframe, ofdmSymbol);

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_V5GTF_REFERENCE_SIGNAL_SUBFRAME_SYMBOL_TRANSMISSION_ENABLED, symbolTransmissionEnabled),
            5, "Symbol Transmission Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  The V5GTF standard (V5G.211) defines two methods by which the DMRS
/// HIFN  sequence can be calculated. You can select the method with the
/// HIFN  'Sequence Generation' parameter. The sequence is either calculated with
/// HIFN  the "n_ID^DMRS" variable (a pseudo-random seed value). Or, if the
/// HIFN  higher layers provide no value for n_ID^DMRS, the sequence is generated
/// HIFN  based on the cell ID with the "n_ID^Cell". n_ID^Cell has the same value
/// HIFN  as the cell ID.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR sequenceGeneration/Selects the method with which the xPDCCH sequence is calculated.
/// HIPAR componentCarrier/Component carrier
/// HIPAR subframe/Selects a specific subframe.
/// HIPAR sequenceID/Defines the value that the xPDCCH sequence is based on.
ViStatus _VI_FUNC rsspecan_ConfigureV5GTFControlChannelxPDCCHSequenceGeneration(
                            ViSession instrSession,
                            ViInt32   componentCarrier,
                            ViInt32   subframe,
                            ViInt32   sequenceGeneration,
                            ViInt32   sequenceID)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, componentCarrier, 1, 16),
            2, "Component Carrier");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, subframe, 0, 49),
            3, "Subframe");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "cc%ld,v5gsubf%ld", componentCarrier, subframe);

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_V5GTF_CONTROL_CHANNEL_DL_XPDCCH_SEQUENCE_GENERATION, sequenceGeneration),
            4, "Sequence Generation");
    if (sequenceGeneration == RSSPECAN_VAL_V5GTF_SEQUENCE_NXPID)
    {
        snprintf(repCap, RS_REPCAP_BUF_SIZE, "cc%ld", componentCarrier);

        viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_V5GTF_CONTROL_CHANNEL_DL_XPDCCH_SEQUENCE_ID, sequenceID),
                5, "Sequence ID");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects the precoding scheme of a xPDSCH allocation.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR componentCarrier/Component carrier
/// HIPAR subframe/Selects a specific subframe.
/// HIPAR precoding/Selects the precoding scheme of a xPDSCH allocation.
ViStatus _VI_FUNC rsspecan_ConfigureV5GTFControlChannelxPDCCHPrecoding(
                            ViSession instrSession,
                            ViInt32   componentCarrier,
                            ViInt32   subframe,
                            ViInt32   precoding)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, componentCarrier, 1, 16),
            2, "Component Carrier");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, subframe, 0, 49),
            3, "Subframe");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "cc%ld,v5gsubf%ld", componentCarrier, subframe);

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_V5GTF_CONTROL_CHANNEL_DL_XPDCCH_PRECODING, precoding),
            4, "Precoding");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the relative power of the xPDCCH.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR componentCarrier/Component carrier
/// HIPAR subframe/Selects a specific subframe.
/// HIPAR power/Defines the relative power of the xPDCCH.
ViStatus _VI_FUNC rsspecan_ConfigureV5GTFControlChannelxPDCCHPower(
                            ViSession instrSession,
                            ViInt32   componentCarrier,
                            ViInt32   subframe,
                            ViReal64  power)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, componentCarrier, 1, 16),
            2, "Component Carrier");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, subframe, 0, 49),
            3, "Subframe");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "cc%ld,v5gsubf%ld", componentCarrier, subframe);

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_V5GTF_CONTROL_CHANNEL_DL_XPDCCH_POWER, power),
            4, "Power");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the number of xREG symbols on which the xPDCCH is transmitted.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR componentCarrier/Component carrier
/// HIPAR subframe/Selects a specific subframe.
/// HIPAR symbol/Selects the XREG symbol.
/// HIPAR numOfxREGs/Defines the number of xREG symbols on which the xPDCCH is transmitted.
ViStatus _VI_FUNC rsspecan_ConfigureV5GTFControlChannelxPDCCHNumOfxREGs(
                            ViSession instrSession,
                            ViInt32   componentCarrier,
                            ViInt32   numOfxREGs)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, componentCarrier, 1, 16),
            2, "Component Carrier");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "cc%ld", componentCarrier);
    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_V5GTF_CONTROL_CHANNEL_DL_XPDCCH_NUM_OF_XREGS, numOfxREGs),
            3, "Num Of xREGs");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Turns the transmission of the xPDCCH on specific xREG resource
/// HIFN  elements on and off.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR componentCarrier/Component carrier
/// HIPAR subframe/Selects a specific subframe.
/// HIPAR symbol/Selects the XREG symbol.
/// HIPAR xreg/Selects the XREG.
/// HIPAR xregEnabled/Turns the transmission of the xPDCCH on specific xREG resource
/// HIPAR xregEnabled/elements on and off.
ViStatus _VI_FUNC rsspecan_ConfigureV5GTFControlChannelxPDCCHxREGEnabled(
                            ViSession instrSession,
                            ViInt32   componentCarrier,
                            ViInt32   subframe,
                            ViInt32   symbol,
                            ViInt32   xreg,
                            ViInt32   xregEnabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, componentCarrier, 1, 16),
            2, "Component Carrier");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, subframe, 0, 49),
            3, "Subframe");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, symbol, 0, 1),
            4, "Symbol");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, xreg, 0, 16),
            5, "xReg");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "cc%ld,v5gsubf%ld,v5gxREGSymbol%ld,v5gxREG%ld", componentCarrier, subframe, symbol, xreg);

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_V5GTF_CONTROL_CHANNEL_DL_XPDCCH_XREG_ENABLED, xregEnabled),
            6, "xREG Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Turns analysis of the xPUCCH on and off.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR componentCarrier/Component carrier
/// HIPAR xpucchEnabled/Turns analysis of the xPUCCH on and off.
ViStatus _VI_FUNC rsspecan_ConfigureV5GTFControlChannelULxPUCCHState(
                            ViSession instrSession,
                            ViInt32   componentCarrier,
                            ViBoolean xpucchEnabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, componentCarrier, 1, 16),
            2, "Component Carrier");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "cc%ld", componentCarrier);

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, repCap, RSSPECAN_ATTR_V5GTF_CONTROL_CHANNEL_UL_XPUCCH_ENABLED, xpucchEnabled),
            3, "xPUCCH Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects the precoding scheme of a xPUCCH allocation.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR componentCarrier/Component carrier
/// HIPAR precoding/Selects the precoding scheme of a xPUCCH allocation.
ViStatus _VI_FUNC rsspecan_ConfigureV5GTFControlChannelULxPUCCHPrecoding(
                            ViSession instrSession,
                            ViInt32   componentCarrier,
                            ViInt32   precoding)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, componentCarrier, 1, 16),
            2, "Component Carrier");
    snprintf(repCap, RS_REPCAP_BUF_SIZE, "cc%ld", componentCarrier);

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_V5GTF_CONTROL_CHANNEL_UL_XPUCCH_PRECODING, precoding),
            3, "Precoding");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  The location of the xPUCCH within the OFDM symbol is defined by
/// HIFN  "n_xPUCCH^2". This is a value between 0 and 15 and defines the
/// HIFN  resources on which the xPUCCH is transmitted. "n_RNTI" is a parameter
/// HIFN  used for the random sequence generation of the DMRS.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR componentCarrier/Component carrier
/// HIPAR location/Defines the location of the xPUCCH in an OFDM symbol.
/// HIPAR n_rnti/Defines the n_RNTI parameter.
ViStatus _VI_FUNC rsspecan_ConfigureV5GTFControlChannelULxPUCCHUsed(
                            ViSession instrSession,
                            ViInt32   componentCarrier,
                            ViInt32   location,
                            ViInt32   n_rnti)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, componentCarrier, 1, 16),
            2, "Component Carrier");
    snprintf(repCap, RS_REPCAP_BUF_SIZE, "cc%ld", componentCarrier);

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_V5GTF_CONTROL_CHANNEL_UL_XPUCCH_LOCATION, location),
            3, "Location");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_V5GTF_CONTROL_CHANNEL_UL_XPUCCH_N_RNTI, n_rnti),
            4, "n_RNTI");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the relative power of the xPUCCH.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR componentCarrier/Component carrier
/// HIPAR xpucchPower/Defines the relative power of the xPUCCH.
/// HIPAR powerDMRS/Defines the relative power of the xPUCCH DMRS.
ViStatus _VI_FUNC rsspecan_ConfigureV5GTFControlChannelULxPUCCHRelativePower(
                            ViSession instrSession,
                            ViInt32   componentCarrier,
                            ViReal64  xpucchPower,
                            ViReal64  powerDMRS)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, componentCarrier, 1, 16),
            2, "Component Carrier");
    snprintf(repCap, RS_REPCAP_BUF_SIZE, "cc%ld", componentCarrier);

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_V5GTF_CONTROL_CHANNEL_UL_XPUCCH_POWER, xpucchPower),
            3, "xPUCCH Power");

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_V5GTF_CONTROL_CHANNEL_UL_XPUCCH_POWER_DMRS, powerDMRS),
            4, "Power DMRS");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects the method with which the xPUCCH DMRS sequence is calculated.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR sequenceGeneration/Selects the method with which the xPUCCH DMRS sequence is calculated.
/// HIPAR componentCarrier/Component carrier
/// HIPAR id/Defines the value that the DMRS sequence is based on.
ViStatus _VI_FUNC rsspecan_ConfigureV5GTFControlChannelULxPUCCHSequenceGeneration(
                            ViSession instrSession,
                            ViInt32   componentCarrier,
                            ViInt32   sequenceGeneration,
                            ViInt32   id)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, componentCarrier, 1, 16),
            2, "Component Carrier");
    snprintf(repCap, RS_REPCAP_BUF_SIZE, "cc%ld", componentCarrier);

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_V5GTF_CONTROL_CHANNEL_UL_XPUCCH_SEQUENCE_GENERATION, sequenceGeneration),
            3, "Sequence Generation");
    if (sequenceGeneration == RSSPECAN_VAL_V5GTF_SEQUENCE_NXPID)
    {
        viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_V5GTF_CONTROL_CHANNEL_UL_XPUCCH_SEQUENCE_ID, id),
                4, "ID");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Turns the antenna port configuration of a physical antenna on and off.
/// HIFN     The configuration you turn on is the one considered in the
/// HIFN  measurement. Note that only one configuration can be measured at the
/// HIFN  same time.
/// HIFN     When you turn on the configuration of one physical antenna, the R&S
/// HIFN  FSW automatically turn off the other one.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR componentCarrier/Component carrier
/// HIPAR physicalAntenna/Selects a physical antenna (for antenna port mapping).
/// HIPAR portConfigurationEnabled/Turns the antenna port configuration of a physical antenna on and off.
ViStatus _VI_FUNC rsspecan_ConfigureV5GTFAntennaPortConfigurationEnabled(
                            ViSession instrSession,
                            ViInt32   componentCarrier,
                            ViInt32   physicalAntenna,
                            ViBoolean portConfigurationEnabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, componentCarrier, 1, 16),
            2, "Component Carrier");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, physicalAntenna, 1, 2),
            3, "Physical Antenna");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "cc%ld,v5gPhysicalAntenna%ld", componentCarrier, physicalAntenna);

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, repCap, RSSPECAN_ATTR_V5GTF_ANTENNA_PORT_CONFIGURATION_ENABLED, portConfigurationEnabled),
            4, "Port Configuration Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Maps antenna ports on which the selected channel or signal is
/// HIFN  transmitted to a physical antenna.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     -
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     CONFigure[:V5G]:DL[:CC<cc>]:PAMapping<cf>:SSIGnal:AP<ap>
/// HIFN     CONFigure[:V5G]:DL[:CC<cc>]:PAMapping<cf>:XPDSch:AP<ap>
/// HIFN     CONFigure[:V5G]:DL[:CC<cc>]:PAMapping<cf>:XPDCch:AP<ap>
/// HIFN     CONFigure[:V5G]:DL[:CC<cc>]:PAMapping<cf>:BRS:AP<ap>
/// HIFN     CONFigure[:V5G]:DL[:CC<cc>]:PAMapping<cf>:EPBCh:AP<ap>
/// HIFN     CONFigure[:V5G]:DL[:CC<cc>]:PAMapping<cf>:CSIRs:AP<ap>
/// HIFN     CONFigure[:V5G]:DL[:CC<cc>]:PAMapping<cf>:PCRS:AP<ap>
/// HIFN
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR componentCarrier/Component carrier
/// HIPAR physicalAntenna/Selects a physical antenna (for antenna port mapping).
/// HIPAR channelOrSignal/Selects the channel or signal.
/// HIPAR antennaPort/Antenna port.
/// HIPAR state/Defines the mapping state.
ViStatus _VI_FUNC rsspecan_ConfigureV5GTFAntennaPortMapping(ViSession instrSession,
                                                            ViInt32   componentCarrier,
                                                            ViInt32   physicalAntenna,
                                                            ViInt32   channelOrSignal,
                                                            ViInt32   antennaPort,
                                                            ViInt32   state)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd_buffer[RS_MAX_MESSAGE_BUF_SIZE] = "";
    ViChar*  p2cmd;
    ViChar*  p2state;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, componentCarrier, 1, 16),
            2, "Component Carrier");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, physicalAntenna, 1, 2),
            4, "Physical Antenna");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, channelOrSignal, 0, 6),
            5, "Channel Or Signal");
    checkErr(RsCore_FindRangeTableDiscreteCmdString(instrSession, &RsSpecAn_rngV5GTFChannelOrSignal, channelOrSignal, &p2cmd));
    checkErr(RsCore_FindRangeTableDiscreteCmdString(instrSession, &RsSpecAn_rngV5GTFExtendedEnabled, state, &p2state));

    switch (channelOrSignal)
    {
    case RSSPECAN_VAL_V5GTF_SYNC:
        viCheckParm(RsCore_InvalidViInt32Range(instrSession, antennaPort, 300, 313),
                6, "Antenna Port");
        viCheckParm(RsCore_InvalidViInt32Range(instrSession, state, 0, 3),
                6, "State");
        break;
    case RSSPECAN_VAL_V5GTF_XPDSCH:
        viCheckParm(RsCore_InvalidViInt32Range(instrSession, antennaPort, 8, 15),
                6, "Antenna Port");
        viCheckParm(RsCore_InvalidViInt32Range(instrSession, state, 0, 3),
                6, "State");
        break;
    case RSSPECAN_VAL_V5GTF_XPDCCH: //107,109
        if ((antennaPort != 107) && (antennaPort != 109))
        {
            viCheckParm(RsCore_InvalidViInt32Value(instrSession, antennaPort), 6, "Antenna Port");
        }
        viCheckParm(RsCore_InvalidViInt32Range(instrSession, state, 0, 1),
                6, "State");
        break;
    case RSSPECAN_VAL_V5GTF_XPBCH:
        viCheckParm(RsCore_InvalidViInt32Range(instrSession, antennaPort, 0, 7),
                6, "Antenna Port");
        viCheckParm(RsCore_InvalidViInt32Range(instrSession, state, 0, 1),
                6, "State");
        break;
    case RSSPECAN_VAL_V5GTF_EPBCH:
        viCheckParm(RsCore_InvalidViInt32Range(instrSession, antennaPort, 500, 501),
                6, "Antenna Port");
        viCheckParm(RsCore_InvalidViInt32Range(instrSession, state, 0, 1),
                6, "State");
        break;
    case RSSPECAN_VAL_V5GTF_CSI_RS:
        viCheckParm(RsCore_InvalidViInt32Range(instrSession, antennaPort, 16, 31),
                6, "Antenna Port");
        viCheckParm(RsCore_InvalidViInt32Range(instrSession, state, 0, 3),
                6, "State");
        break;
    case RSSPECAN_VAL_V5GTF_PCRS:
        if ((antennaPort != 60) && (antennaPort != 61) && (antennaPort != 6061))
        {
            viCheckParm(RsCore_InvalidViInt32Value(instrSession, antennaPort), 6, "Antenna Port");
        }
        viCheckParm(RsCore_InvalidViInt32Range(instrSession, state, 0, 1),
                6, "State");
        break;
    }

    snprintf(cmd_buffer, RS_MAX_MESSAGE_BUF_SIZE, "CONF:DL:CC%ld:PAM%ld:%s:AP%ld %s", componentCarrier, physicalAntenna, p2cmd, antennaPort, p2state);
    checkErr(RsCore_Write(instrSession, cmd_buffer));
    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects the maximum number of subframes to analyze.
/// HIFN     Selecting a number of subframes different from the default one may
/// HIFN  become necessary if the capture time is less than 20.1ms.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR maxNumberOfSubframes/Selects the maximum number of subframes to analyze.
ViStatus _VI_FUNC rsspecan_ConfigureV5GTFMaxNumberOfSubframes(
                            ViSession instrSession,
                            ViInt32   maxNumberOfSubframes)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, NULL, RSSPECAN_ATTR_V5GTF_MAX_NUMBER_OF_SUBFRAMES, maxNumberOfSubframes),
            2, "Max Number Of Subframes");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Turns suppression of interfering neighboring carriers on and off (e.g.
/// HIFN  LTE, WCDMA, GSM etc).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR multicarrierFilterEnabled/Turns suppression of interfering neighboring carriers on and off (e.g.
/// HIPAR multicarrierFilterEnabled/LTE, WCDMA, GSM etc).
ViStatus _VI_FUNC rsspecan_ConfigureV5GTFDemodulationMulticarrierFilterEnabled(
                            ViSession instrSession,
                            ViBoolean multicarrierFilterEnabled)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, NULL, RSSPECAN_ATTR_V5GTF_DEMODULATION_MULTICARRIER_FILTER_ENABLED, multicarrierFilterEnabled),
            2, "Multicarrier Filter Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Turns level tracking on and off.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR levelTrackingEnabled/Turns level tracking on and off.
ViStatus _VI_FUNC rsspecan_ConfigureV5GTFLevelTrackingEnabled(
                            ViSession instrSession,
                            ViBoolean levelTrackingEnabled)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, NULL, RSSPECAN_ATTR_V5GTF_LEVEL_TRACKING_ENABLED, levelTrackingEnabled),
            2, "Level Tracking Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function adds a window to the display.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR stringSize/This control sets the allocated length of Returned Window Name string.
/// HIPAR referenceWindowName/String containing the number or name of the window the new window is
/// HIPAR referenceWindowName/inserted next to. In the default state, the name of the window is its
/// HIPAR referenceWindowName/index.
/// HIPAR direction/Direction the window is added relative to the window set with
/// HIPAR direction/"Reference Window Name".
/// HIPAR returnedWindowName/When adding a new window, the function returns its name (by
/// HIPAR windowType/Type of result display (evaluation method) you want to add.
ViStatus _VI_FUNC rsspecan_AddV5GTFWindow(ViSession instrSession,
                                          ViInt32   stringSize,
                                          ViString  referenceWindowName,
                                          ViInt32   direction,
                                          ViInt32   windowType,
                                          ViChar    returnedWindowName[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViChar   response[RS_MAX_MESSAGE_BUF_SIZE];
    ViChar*  p2direction;
    ViChar*  p2type;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &rsspecan_rngLayoutDirection, direction, &p2direction),
                4, "Direction");
    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &RsSpecAn_rngV5GTFLayoutType, windowType, &p2type),
                5, "Window Type");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "LAY:ADD? '%s',%s,%s", referenceWindowName, p2direction, p2type);
    checkErr(RsCore_QueryViString(instrSession, cmd, response));

    if (returnedWindowName != NULL)
    {
        RsCore_TrimString(response, RS_VAL_TRIM_WHITESPACES_AND_ALL_QUOTES);
        RsCore_CopyToUserBufferAsciiData(returnedWindowName, stringSize, response);
    }

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function replaces a window with another one.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR windowName/String containing the number or name of the window you want to
/// HIPAR windowType/Type of result display (evaluation method) you want to add.
ViStatus _VI_FUNC rsspecan_ReplaceV5GTFWindow(ViSession instrSession,
                                              ViString  windowName,
                                              ViInt32   windowType)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViChar*  p2type;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &RsSpecAn_rngV5GTFLayoutType, windowType, &p2type),
                3, "Window Type");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "LAY:REPL '%s',%s", windowName, p2type);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures scales of the y-axis of a diagram based on the displayed
/// HIFN  results.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR subwindow/This control selects the measurement subwindow.
/// HIPAR max/Defines the maximum value displayed on the y-axis of a diagram.
/// HIPAR min/Defines the minimum value displayed on the vertical diagram axis.
ViStatus _VI_FUNC rsspecan_ConfigureV5GTFSubwindowYAxisScaling(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   subwindow,
                            ViReal64  max,
                            ViReal64  min)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, window, 0, 16),
            2, "Window");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, subwindow, 1, 64),
            3, "Subwindow");
    snprintf(repCap, RS_REPCAP_BUF_SIZE, "Win%ld,SW%ld", window, subwindow);

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_V5GTF_Y_AXIS_MAX, max),
            5, "Max");

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_V5GTF_Y_AXIS_MIN, min),
            6, "Min");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures scales of the y-axis of a diagram based on the displayed
/// HIFN  results.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR subwindow/This control selects the measurement subwindow.
/// HIPAR autoScalingEnabled/Automatically scales the y-axis of a diagram based on the displayed
/// HIPAR autoScalingEnabled/results.
ViStatus _VI_FUNC rsspecan_ConfigureV5GTFSubwindowYAxisAutoScaling(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   subwindow,
                            ViInt32   autoScalingEnabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, window, 0, 16),
            2, "Window");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, subwindow, 1, 64),
            3, "Subwindow");
    snprintf(repCap, RS_REPCAP_BUF_SIZE, "Win%ld,SW%ld", window, subwindow);

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_V5GTF_Y_AXIS_AUTO_SCALING_ENABLED, autoScalingEnabled),
            5, "Auto Scaling Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Couples or decouples result display tabs (subwindows).
/// HIFN     Subwindow coupling is available for measurements with multiple data
/// HIFN  streams (like carrier aggregation).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR subwindow/This control selects the measurement subwindow.
/// HIPAR couplingEnabled/Couples or decouples result display tabs (subwindows).
ViStatus _VI_FUNC rsspecan_ConfigureV5GTFSubwindowCouplingEnabled(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   subwindow,
                            ViBoolean couplingEnabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, window, 0, 16),
            2, "Window");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, subwindow, 1, 64),
            3, "Subwindow");
    snprintf(repCap, RS_REPCAP_BUF_SIZE, "Win%ld,SW%ld", window, subwindow);

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, repCap, RSSPECAN_ATTR_V5GTF_SUBWINDOW_COUPLING_ENABLED, couplingEnabled),
            4, "Coupling Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the focus on the selected result display window.
/// HIFN     This window is then the active window.
/// HIFN     For measurements with multiple results in subwindows, the command also
/// HIFN  selects the subwindow. Use this command to select the (sub)window
/// HIFN  before querying trace data.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR subwindow/This control selects the measurement subwindow.
ViStatus _VI_FUNC rsspecan_SelectV5GTFSubwindow(ViSession instrSession,
                                                ViInt32   window,
                                                ViInt32   subwindow)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, window, 0, 16),
            2, "Window");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, subwindow, 1, 64),
            3, "Subwindow");
    snprintf(repCap, RS_REPCAP_BUF_SIZE, "Win%ld,SW%ld", window, subwindow);

    checkErr(rsspecan_SetAttributeViString(instrSession, repCap, RSSPECAN_ATTR_V5GTF_SUBWINDOW_SELECT, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the current synchronization state.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR synchronizationState/Queries the current synchronization state.
/// HIPAR bufferSize/Defines size of array 'synchronizationState'
ViStatus _VI_FUNC rsspecan_QueryV5GTFSynchronizationState(ViSession instrSession,
                                                          ViInt32   bufferSize,
                                                          ViChar    synchronizationState[])
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_GetAttributeViString(instrSession, NULL, RSSPECAN_ATTR_V5GTF_SYNCHRONIZATION_STATE,
            bufferSize, synchronizationState),
            3, "Synchronization State");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Couples or decouples markers in different result displays to each
/// HIFN  other.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR markerCouplingEnabled/Couples or decouples markers in different result displays to each
/// HIPAR markerCouplingEnabled/other.
ViStatus _VI_FUNC rsspecan_ConfigureV5GTFMarkerCouplingState(ViSession instrSession,
                                                             ViBoolean markerCouplingEnabled)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, NULL, RSSPECAN_ATTR_V5GTF_MARKER_COUPLING_ENABLED, markerCouplingEnabled),
            2, "Marker Coupling Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects the scale of the x-axis for result displays that show
/// HIFN  subcarrier results.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR xAxisUnit/Selects the scale of the x-axis for result displays that show
/// HIPAR xAxisUnit/subcarrier results.
ViStatus _VI_FUNC rsspecan_ConfigureV5GTFXAxisUnit(ViSession instrSession,
                                                   ViInt32   xAxisUnit)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, NULL, RSSPECAN_ATTR_V5GTF_X_AXIS_UNIT, xAxisUnit),
            2, "X Axis Unit");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This attribute specifies the units for EVM trace results.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR evmUnit/This control specifies the units for EVM trace results.
ViStatus _VI_FUNC rsspecan_ConfigureV5GTFEVMUnit(ViSession instrSession,
                                                 ViInt32   evmUnit)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, NULL, RSSPECAN_ATTR_V5GTF_EVM_UNIT, evmUnit),
            2, "EVM Unit");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the marker position on the z-axis of three-dimensional result
/// HIFN  displays.
/// HIFN     Instead of returning a certain type of value (EVM or Power), which is
/// HIFN  possible with CALCulate<n>:MARKer<m>:Z?, this method returns all types
/// HIFN  of values (EVM and Power), regardless of the result display type.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/Select the window. 0=current window
/// HIPAR evm/EVM at the marker position.
/// HIPAR power/Power at the marker position.
ViStatus _VI_FUNC rsspecan_QueryV5GTFMarkerZ(ViSession instrSession,
                                             ViInt32   window,
                                             ViReal64* evm,
                                             ViReal64* power)
{
    ViStatus  error = VI_SUCCESS;
    ViChar    cmd_buffer[RS_MAX_MESSAGE_BUF_SIZE] = "CALC:MARK:Z:ALL?";
    ViInt32   cnt = 0;
    ViPReal64 res = NULL;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, window, 0, 16),
            2, "Window");
    if (window != 0)
        snprintf(cmd_buffer, RS_MAX_MESSAGE_BUF_SIZE, "CALC%ld:MARK:Z:ALL?", window);
    checkErr(RsCore_QueryAsciiViReal64Array(instrSession, cmd_buffer, &res, &cnt));
    checkErr(rsspecan_CheckStatus(instrSession));
    if (cnt >= 2)
    {
        *evm = res[0];
        *power = res[1];
    }
    else
        checkErr(RS_ERROR_UNEXPECTED_RESPONSE);

Error:
    if (res) free(res);
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the average crest factor as shown in the result summary.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR componentCarrier/Component carrier
/// HIPAR crestFactor/Queries the average crest factor as shown in the result summary.
ViStatus _VI_FUNC rsspecan_QueryV5GTFResultCrestFactor(ViSession instrSession,
                                                       ViInt32   componentCarrier,
                                                       ViReal64* crestFactor)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, componentCarrier, 1, 16),
            2, "Component Carrier");
    snprintf(repCap, RS_REPCAP_BUF_SIZE, "cc%ld", componentCarrier);
    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_V5GTF_RESULT_CREST_FACTOR, crestFactor),
            3, "Crest Factor");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the EVM of all resource elements.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR componentCarrier/Component carrier
/// HIPAR evm/Queries the EVM of all resource elements.
ViStatus _VI_FUNC rsspecan_QueryV5GTFResultEVM(ViSession instrSession,
                                               ViInt32   componentCarrier,
                                               ViReal64* evm)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, componentCarrier, 1, 16),
            2, "Component Carrier");
    snprintf(repCap, RS_REPCAP_BUF_SIZE, "cc%ld", componentCarrier);
    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_V5GTF_RESULT_EVM, evm),
            3, "EVM");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the EVM of all xPDSCH resource elements with a QPSK
/// HIFN  modulation.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR componentCarrier/Component carrier
/// HIPAR evmxPDSCHQPSK/Queries the EVM of all xPDSCH resource elements with a QPSK
/// HIPAR evmxPDSCHQPSK/modulation.
ViStatus _VI_FUNC rsspecan_QueryV5GTFResultEVMxPDSCHQPSK(ViSession instrSession,
                                                         ViInt32   componentCarrier,
                                                         ViReal64* evmxPDSCHQPSK)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, componentCarrier, 1, 16),
            2, "Component Carrier");
    snprintf(repCap, RS_REPCAP_BUF_SIZE, "cc%ld", componentCarrier);

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_V5GTF_RESULT_EVM_XPDSCH_QPSK, evmxPDSCHQPSK),
            3, "EVM xPDSCH QPSK");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the EVM of all xPDSCH resource elements with a 16QAM
/// HIFN  modulation.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR evmxPDSCH16QAM/Queries the EVM of all xPDSCH resource elements with a 16QAM
/// HIPAR evmxPDSCH16QAM/modulation.
/// HIPAR componentCarrier/Component carrier
ViStatus _VI_FUNC rsspecan_QueryV5GTFResultEVMxPDSCH16QAM(ViSession instrSession,
                                                          ViInt32   componentCarrier,
                                                          ViReal64* evmxPDSCH16QAM)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, componentCarrier, 1, 16),
            2, "Component Carrier");
    snprintf(repCap, RS_REPCAP_BUF_SIZE, "cc%ld", componentCarrier);
    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_V5GTF_RESULT_EVM_XPDSCH_16QAM, evmxPDSCH16QAM),
            3, "V5GTF Result EVM xPDSCH 16QAM");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the EVM of all xPDSCH resource elements with a 64QAM
/// HIFN  modulation.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR componentCarrier/Component carrier
/// HIPAR evmxPDSCH64QAM/Queries the EVM of all xPDSCH resource elements with a 64QAM
/// HIPAR evmxPDSCH64QAM/modulation.
ViStatus _VI_FUNC rsspecan_QueryV5GTFResultEVMxPDSCH64QAM(ViSession instrSession,
                                                          ViInt32   componentCarrier,
                                                          ViReal64* evmxPDSCH64QAM)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, componentCarrier, 1, 16),
            2, "Component Carrier");
    snprintf(repCap, RS_REPCAP_BUF_SIZE, "cc%ld", componentCarrier);

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_V5GTF_RESULT_EVM_XPDSCH_64QAM, evmxPDSCH64QAM),
            3, "EVM xPDSCH 64QAM");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the EVM of all xPDSCH resource elements with a 256QAM
/// HIFN  modulation.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR componentCarrier/Component carrier
/// HIPAR evmxPDSCH256QAM/Queries the EVM of all xPDSCH resource elements with a 256QAM
/// HIPAR evmxPDSCH256QAM/modulation.
ViStatus _VI_FUNC rsspecan_QueryV5GTFResultEVMxPDSCH256QAM(ViSession instrSession,
                                                           ViInt32   componentCarrier,
                                                           ViReal64* evmxPDSCH256QAM)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, componentCarrier, 1, 16),
            2, "Component Carrier");
    snprintf(repCap, RS_REPCAP_BUF_SIZE, "cc%ld", componentCarrier);
    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_V5GTF_RESULT_EVM_XPDSCH_256QAM, evmxPDSCH256QAM),
            3, "EVM xPDSCH 256QAM");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the EVM of all physical channel resource elements.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR componentCarrier/Component carrier
/// HIPAR evmPhysicalChannel/Queries the EVM of all physical channel resource elements.
ViStatus _VI_FUNC rsspecan_QueryV5GTFResultEVMPhysicalChannel(
                            ViSession instrSession,
                            ViInt32   componentCarrier,
                            ViReal64* evmPhysicalChannel)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, componentCarrier, 1, 16),
            2, "Component Carrier");
    snprintf(repCap, RS_REPCAP_BUF_SIZE, "cc%ld", componentCarrier);
    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_V5GTF_RESULT_EVM_PHYSICAL_CHANNEL, evmPhysicalChannel),
            3, "EVM Physical Channel");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the EVM of all physical signal resource elements.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR componentCarrier/Component carrier
/// HIPAR evmPhysicalSignal/Queries the EVM of all physical signal resource elements.
ViStatus _VI_FUNC rsspecan_QueryV5GTFResultEVMPhysicalSignal(ViSession instrSession,
                                                             ViInt32   componentCarrier,
                                                             ViReal64* evmPhysicalSignal)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, componentCarrier, 1, 16),
            2, "Component Carrier");
    snprintf(repCap, RS_REPCAP_BUF_SIZE, "cc%ld", componentCarrier);
    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_V5GTF_RESULT_EVM_PHYSICAL_SIGNAL, evmPhysicalSignal),
            3, "EVM Physical Signal");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the EVM of all xPUSCH resource elements with QPSK modulation.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR componentCarrier/Component carrier
/// HIPAR evmxPUSCHQPSK/Queries the EVM of all xPUSCH resource elements with QPSK modulation.
ViStatus _VI_FUNC rsspecan_QueryV5GTFResultEVMxPUSCHQPSK(ViSession instrSession,
                                                         ViInt32   componentCarrier,
                                                         ViReal64* evmxPUSCHQPSK)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, componentCarrier, 1, 16),
            2, "Component Carrier");
    snprintf(repCap, RS_REPCAP_BUF_SIZE, "cc%ld", componentCarrier);
    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_V5GTF_RESULT_EVM_XPUSCH_QPSK, evmxPUSCHQPSK),
            3, "EVM xPUSCH QPSK");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the EVM of all xPUSCH resource elements with 16QAM modulation.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR componentCarrier/Component carrier
/// HIPAR evmxPUSCH16QAM/Queries the EVM of all xPUSCH resource elements with 16QAM modulation.
ViStatus _VI_FUNC rsspecan_QueryV5GTFResultEVMxPUSCH16QAM(ViSession instrSession,
                                                          ViInt32   componentCarrier,
                                                          ViReal64* evmxPUSCH16QAM)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, componentCarrier, 1, 16),
            2, "Component Carrier");
    snprintf(repCap, RS_REPCAP_BUF_SIZE, "cc%ld", componentCarrier);
    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_V5GTF_RESULT_EVM_XPUSCH_16QAM, evmxPUSCH16QAM),
            3, "EVM xPUSCH 16QAM");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the EVM of all xPUSCH resource elements with 64QAM modulation.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR componentCarrier/Component carrier
/// HIPAR evmxPUSCH64QAM/Queries the EVM of all xPUSCH resource elements with 64QAM modulation.
ViStatus _VI_FUNC rsspecan_QueryV5GTFResultEVMxPUSCH64QAM(ViSession instrSession,
                                                          ViInt32   componentCarrier,
                                                          ViReal64* evmxPUSCH64QAM)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, componentCarrier, 1, 16),
            2, "Component Carrier");
    snprintf(repCap, RS_REPCAP_BUF_SIZE, "cc%ld", componentCarrier);
    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_V5GTF_RESULT_EVM_XPUSCH_64QAM, evmxPUSCH64QAM),
            3, "EVM xPUSCH 64QAM");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the EVM of all xPUSCH resource elements with 256QAM
/// HIFN  modulation.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR componentCarrier/Component carrier
/// HIPAR evmxPUSCH256QAM/Queries the EVM of all xPUSCH resource elements with 256QAM
/// HIPAR evmxPUSCH256QAM/modulation.
ViStatus _VI_FUNC rsspecan_QueryV5GTFResultEVMxPUSCH256QAM(ViSession instrSession,
                                                           ViInt32   componentCarrier,
                                                           ViReal64* evmxPUSCH256QAM)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, componentCarrier, 1, 16),
            2, "Component Carrier");
    snprintf(repCap, RS_REPCAP_BUF_SIZE, "cc%ld", componentCarrier);
    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_V5GTF_RESULT_EVM_XPUSCH_256QAM, evmxPUSCH256QAM),
            3, "EVM xPUSCH 256QAM");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the frequency error.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR componentCarrier/Component carrier
/// HIPAR frequencyError/Queries the frequency error.
ViStatus _VI_FUNC rsspecan_QueryV5GTFResultFrequencyError(ViSession instrSession,
                                                          ViInt32   componentCarrier,
                                                          ViReal64* frequencyError)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, componentCarrier, 1, 16),
            2, "Component Carrier");
    snprintf(repCap, RS_REPCAP_BUF_SIZE, "cc%ld", componentCarrier);
    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_V5GTF_RESULT_FREQUENCY_ERROR, frequencyError),
            3, "Frequency Error");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the I/Q gain imbalance.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR componentCarrier/Component carrier
/// HIPAR iqGainImbalance/Queries the I/Q gain imbalance.
ViStatus _VI_FUNC rsspecan_QueryV5GTFResultIQGainImbalance(ViSession instrSession,
                                                           ViInt32   componentCarrier,
                                                           ViReal64* iqGainImbalance)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, componentCarrier, 1, 16),
            2, "Component Carrier");
    snprintf(repCap, RS_REPCAP_BUF_SIZE, "cc%ld", componentCarrier);
    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_V5GTF_RESULT_IQ_GAIN_IMBALANCE, iqGainImbalance),
            3, "IQ Gain Imbalance");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the I/Q offset.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR componentCarrier/Component carrier
/// HIPAR iqOffset/Queries the I/Q offset.
ViStatus _VI_FUNC rsspecan_QueryV5GTFResultIQOffset(ViSession instrSession,
                                                    ViInt32   componentCarrier,
                                                    ViReal64* iqOffset)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, componentCarrier, 1, 16),
            2, "Component Carrier");
    snprintf(repCap, RS_REPCAP_BUF_SIZE, "cc%ld", componentCarrier);
    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_V5GTF_RESULT_IQ_OFFSET, iqOffset),
            3, "IQ Offset");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the quadrature error.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR componentCarrier/Component carrier
/// HIPAR quadratureError/Queries the quadrature error.
ViStatus _VI_FUNC rsspecan_QueryV5GTFResultQuadratureError(ViSession instrSession,
                                                           ViInt32   componentCarrier,
                                                           ViReal64* quadratureError)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, componentCarrier, 1, 16),
            2, "Component Carrier");
    snprintf(repCap, RS_REPCAP_BUF_SIZE, "cc%ld", componentCarrier);
    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_V5GTF_RESULT_QUADRATURE_ERROR, quadratureError),
            3, "Quadrature Error");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the total power.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR componentCarrier/Component carrier
/// HIPAR totalPower/Queries the total power.
ViStatus _VI_FUNC rsspecan_QueryV5GTFResultTotalPower(ViSession instrSession,
                                                      ViInt32   componentCarrier,
                                                      ViReal64* totalPower)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, componentCarrier, 1, 16),
            2, "Component Carrier");
    snprintf(repCap, RS_REPCAP_BUF_SIZE, "cc%ld", componentCarrier);
    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_V5GTF_RESULT_TOTAL_POWER, totalPower),
            3, "Total Power");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the sampling error.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR componentCarrier/Component carrier
/// HIPAR samplingError/Queries the sampling error.
ViStatus _VI_FUNC rsspecan_QueryV5GTFResultSamplingError(ViSession instrSession,
                                                         ViInt32   componentCarrier,
                                                         ViReal64* samplingError)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, componentCarrier, 1, 16),
            2, "Component Carrier");
    snprintf(repCap, RS_REPCAP_BUF_SIZE, "cc%ld", componentCarrier);
    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_V5GTF_RESULT_SAMPLING_ERROR, samplingError),
            3, "Sampling Error");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the (sub)frame start offset as shown in the capture buffer.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR componentCarrier/Component carrier
/// HIPAR subframeStartOffset/Queries the (sub)frame start offset as shown in the capture buffer.
ViStatus _VI_FUNC rsspecan_QueryV5GTFResultSubframeStartOffset(
                            ViSession instrSession,
                            ViInt32   componentCarrier,
                            ViReal64* subframeStartOffset)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, componentCarrier, 1, 16),
            2, "Component Carrier");
    snprintf(repCap, RS_REPCAP_BUF_SIZE, "cc%ld", componentCarrier);
    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_V5GTF_RESULT_SUBFRAME_START_OFFSET, subframeStartOffset),
            3, "Subframe Start Offset");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  For the capture buffer result display, the method returns one value
/// HIFN  for each I/Q sample in the capture buffer. The unit is always dBm.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR arraySize/Number of array points requested.
/// HIPAR result/Returns trace data.
/// HIPAR returnedValues/Returns the number of trace data values.
ViStatus _VI_FUNC rsspecan_V5GTFFetchCaptureBuffer(ViSession instrSession,
                                                   ViInt32   window,
                                                   ViInt32   arraySize,
                                                   ViReal64  result[],
                                                   ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, window, 0, 16),
            2, "Window");
    checkErr(rsspecan_dataReadTrace(instrSession, window, "TRACE1", arraySize, result, returnedValues));
    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  For the EVM vs carrier result display, the method returns one value
/// HIFN  for each subcarrier that has been analyzed. The unit depends on
/// HIFN  UNIT:EVM (EVMUnit/RSSPECAN_ATTR_V5GTF_EVM_UNIT).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR arraySize/Number of array points requested.
/// HIPAR returnedValues/Returns the number of trace data values.
/// HIPAR result/Returns trace data.
ViStatus _VI_FUNC rsspecan_V5GTFFetchEVMvsCarrier(ViSession instrSession,
                                                  ViInt32   window,
                                                  ViInt32   arraySize,
                                                  ViReal64  result[],
                                                  ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, window, 0, 16),
            2, "Window");
    checkErr(rsspecan_dataReadTrace(instrSession, window, "TRACE1", arraySize, result, returnedValues));
    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  For the EVM vs symbol result display, the method returns one value for
/// HIFN  each OFDM symbol that has been analyzed. The unit depends on UNIT:EVM
/// HIFN  (EVMUnit/RSSPECAN_ATTR_V5GTF_EVM_UNIT).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR arraySize/Number of array points requested.
/// HIPAR returnedValues/Returns the number of trace data values.
/// HIPAR result/Returns trace data.
ViStatus _VI_FUNC rsspecan_V5GTFFetchEVMvsSymbol(ViSession instrSession,
                                                 ViInt32   window,
                                                 ViInt32   arraySize,
                                                 ViReal64  result[],
                                                 ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, window, 0, 16),
            2, "Window");
    checkErr(rsspecan_dataReadTrace(instrSession, window, "TRACE1", arraySize, result, returnedValues));
    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  For the power spectrum result display, the method returns one value
/// HIFN  for each trace point. The unit is always dBm/Hz.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR arraySize/Number of array points requested.
/// HIPAR returnedValues/Returns the number of trace data values.
/// HIPAR result/Returns trace data.
ViStatus _VI_FUNC rsspecan_V5GTFFetchPowerSpectrum(ViSession instrSession,
                                                   ViInt32   window,
                                                   ViInt32   arraySize,
                                                   ViReal64  result[],
                                                   ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, window, 0, 16),
            2, "Window");
    checkErr(rsspecan_dataReadTrace(instrSession, window, "TRACE1", arraySize, result, returnedValues));
    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  For the channel flatness result display, the method returns one value
/// HIFN  for each trace point. The unit is always dB.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR arraySize/Number of array points requested.
/// HIPAR returnedValues/Returns the number of trace data values.
/// HIPAR result/Returns trace data.
ViStatus _VI_FUNC rsspecan_V5GTFFetchChannelAndSpectrumFlatness(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   arraySize,
                            ViReal64  result[],
                            ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, window, 0, 16),
            2, "Window");
    checkErr(rsspecan_dataReadTrace(instrSession, window, "TRACE1", arraySize, result, returnedValues));
    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  For the group delay result display, the command returns one value for
/// HIFN  each trace point. The unit is always ns. The number of values depends
/// HIFN  on the selected V5GTF bandwidth.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR arraySize/Number of array points requested.
/// HIPAR returnedValues/Returns the number of trace data values.
/// HIPAR result/Returns trace data.
ViStatus _VI_FUNC rsspecan_V5GTFFetchGroupDelay(ViSession instrSession,
                                                ViInt32   window,
                                                ViInt32   arraySize,
                                                ViReal64  result[],
                                                ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, window, 0, 16),
            2, "Window");
    checkErr(rsspecan_dataReadTrace(instrSession, window, "TRACE1", arraySize, result, returnedValues));
    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  For the constellation diagram, the method returns two values for each
/// HIFN  constellation point.
/// HIFN     <I[SF0][Sym0][Carrier1]>, <Q[SF0][Sym0][Carrier1]>, ...,
/// HIFN  <I[SF0][Sym0][Carrier(n)]>, <Q[SF0][Sym0][Carrier(n)]>,
/// HIFN     <I[SF0][Sym1][Carrier1]>, <Q[SF0][Sym1][Carrier1]>, ...,
/// HIFN  <I[SF0][Sym1][Carrier(n)]>, <Q[SF0][Sym1][Carrier(n)]>,
/// HIFN     <I[SF0][Sym(n)][Carrier1]>, <Q[SF0][Sym(n)][Carrier1]>, ...,
/// HIFN  <I[SF0][Sym(n)][Carrier(n)]>, <Q[SF0][Sym(n)][Carrier(n)]>,
/// HIFN     <I[SF1][Sym0][Carrier1]>, <Q[SF1][Sym0][Carrier1]>, ...,
/// HIFN  <I[SF1][Sym0][Carrier(n)]>, <Q[SF1][Sym0][Carrier(n)]>,
/// HIFN     <I[SF1][Sym1][Carrier1]>, <Q[SF1][Sym1][Carrier1]>, ...,
/// HIFN  <I[SF1][Sym1][Carrier(n)]>, <Q[SF1][Sym1][Carrier(n)]>,
/// HIFN     <I[SF(n)][Sym(n)][Carrier1]>, <Q[SF(n)][Sym(n)][Carrier1]>, ...,
/// HIFN  <I[SF(n)][Sym(n)][Carrier(n)]>, <Q[SF(n)][Sym(n)][Carrier(n)]>
/// HIFN
/// HIFN     With SF = subframe and Sym = symbol of that subframe.
/// HIFN     The I and Q values have no unit.
/// HIFN     The number of return values depends on the constellation selection. By
/// HIFN  default, it returns all resource elements including the DC carrier.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR arraySize/Number of array points requested.
/// HIPAR returnedValues/Returns the number of trace data values.
/// HIPAR resultQ/Returns trace data, Q part.
/// HIPAR resultI/Returns trace data, I part.
ViStatus _VI_FUNC rsspecan_V5GTFFetchConstellation(ViSession instrSession,
                                                   ViInt32   window,
                                                   ViInt32   arraySize,
                                                   ViReal64  resultI[],
                                                   ViReal64  resultQ[],
                                                   ViInt32*  returnedValues)
{
    ViStatus  error = VI_SUCCESS;
    ViPReal64 res = NULL;
    ViInt32   ret_cnt;
    ViInt32   size;
    ViChar    cmd[RS_MAX_MESSAGE_BUF_SIZE] = ":FORM REAL,32;:TRAC? TRACE1";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, window, 0, 16),
            2, "Window");

    if (window != 0)
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, ":FORM REAL,32;:TRAC%ld? TRACE1", window);
    checkErr(RsCore_QueryFloatArray(instrSession, cmd, &res, &ret_cnt));
    checkErr(rsspecan_CheckStatus(instrSession));

    size = (arraySize > (ret_cnt / 2)) ? ret_cnt / 2 : arraySize;

    checkErr(RsCore_DecimateViReal64ArrayToUserBuffers(res, ret_cnt, size, returnedValues, resultI, resultQ, NULL, NULL, NULL, NULL));

Error:
    if (res) free(res);
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  For the allocation summary, the method returns V5GTFAllocationSummary
/// HIFN  object  for each line of the table.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR arraySize/Number of array points requested. Applies to all arrays.
/// HIPAR subframe/Subframe numbers.
/// HIPAR numberofRB/Number of resource blocks
/// HIPAR allocationID/Represents the allocation ID. The range is as follows.
/// HIPAR modulation/Represents the modulation scheme.
/// HIPAR relativePower/Relative power in dB
/// HIPAR absolutePower/Absolute power in dBm.
/// HIPAR evm/The unit depends on UNIT:EVM (EVMUnit/RSSPECAN_ATTR_V5GTF_EVM_UNIT)
/// HIPAR returnedValues/Returns the number of trace data values in each array.
ViStatus _VI_FUNC rsspecan_V5GTFFetchAllocationSummary(ViSession instrSession,
                                                       ViInt32   window,
                                                       ViInt32   arraySize,
                                                       ViInt32   subframe[],
                                                       ViInt32   numberofRB[],
                                                       ViInt32   allocationID[],
                                                       ViInt32   modulation[],
                                                       ViReal64  relativePower[],
                                                       ViReal64  absolutePower[],
                                                       ViReal64  evm[],
                                                       ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = ":TRAC? TRACE1";
    ViChar*  response = NULL;
    ViChar*  pbuf;
    ViInt32  i = 0;

    checkErr(RsCore_LockSession(instrSession));

    if (window != 0)
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, ":TRAC%ld? TRACE1", window);
    checkErr(RsCore_QueryViStringUnknownLength(instrSession, cmd, &response));
    pbuf = strtok(response, ",\n");
    if (isNullOrEmpty(pbuf))
    {
        checkErr(RS_ERROR_UNEXPECTED_RESPONSE);
    }

    while ((i < arraySize) && (!isNullOrEmpty(pbuf)))
    {
        subframe[i] = RsCore_Convert_String_To_ViInt32(pbuf);
        pbuf = strtok(NULL, ",\n");
        if (isNullOrEmpty(pbuf))
        {
            checkErr(RS_ERROR_UNEXPECTED_RESPONSE);
        }
        numberofRB[i] = RsCore_Convert_String_To_ViInt32(pbuf);
        pbuf = strtok(NULL, ",\n");
        if (isNullOrEmpty(pbuf))
        {
            checkErr(RS_ERROR_UNEXPECTED_RESPONSE);
        }
        allocationID[i] = RsCore_Convert_String_To_ViInt32(pbuf);
        pbuf = strtok(NULL, ",\n");
        if (isNullOrEmpty(pbuf))
        {
            checkErr(RS_ERROR_UNEXPECTED_RESPONSE);
        }
        modulation[i] = RsCore_Convert_String_To_ViInt32(pbuf);
        pbuf = strtok(NULL, ",\n");
        if (isNullOrEmpty(pbuf))
        {
            checkErr(RS_ERROR_UNEXPECTED_RESPONSE);
        }
        relativePower[i] = RsCore_Convert_String_To_ViReal64(pbuf);
        pbuf = strtok(NULL, ",\n");
        if (isNullOrEmpty(pbuf))
        {
            checkErr(RS_ERROR_UNEXPECTED_RESPONSE);
        }
        absolutePower[i] = RsCore_Convert_String_To_ViReal64(pbuf);
        pbuf = strtok(NULL, ",\n");
        if (isNullOrEmpty(pbuf))
        {
            checkErr(RS_ERROR_UNEXPECTED_RESPONSE);
        }
        evm[i] = RsCore_Convert_String_To_ViReal64(pbuf);
        pbuf = strtok(NULL, ",\n");
        i++;
    }
    *returnedValues = i;

Error:
    if (response) free(response);
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  For the EVM vs symbol x carrier result display, the method returns one
/// HIFN  value for each resource element. The unit depends on UNIT:EVM
/// HIFN  (EVMUnit/RSSPECAN_ATTR_V5GTF_EVM_UNIT). Resource elements that are
/// HIFN  unused return NAN.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR arraySize/Number of array points requested.
/// HIPAR returnedValues/Returns the number of trace data values.
/// HIPAR result/Returns trace data.
ViStatus _VI_FUNC rsspecan_V5GTFFetchEVMvsSymbolxCarrier(ViSession instrSession,
                                                         ViInt32   window,
                                                         ViInt32   arraySize,
                                                         ViReal64  result[],
                                                         ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, window, 0, 16),
            2, "Window");
    checkErr(rsspecan_dataReadTrace(instrSession, window, "TRACE1", arraySize, result, returnedValues));
    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  For the power vs symbol x carrier result display, the method returns
/// HIFN  one value for each resource element. The unit is always dBm. Resource
/// HIFN  elements that are unused return NAN.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR arraySize/Number of array points requested.
/// HIPAR returnedValues/Returns the number of trace data values.
/// HIPAR result/Returns trace data.
ViStatus _VI_FUNC rsspecan_V5GTFFetchPowervsSymbolxCarrier(ViSession instrSession,
                                                           ViInt32   window,
                                                           ViInt32   arraySize,
                                                           ViReal64  result[],
                                                           ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, window, 0, 16),
            2, "Window");
    checkErr(rsspecan_dataReadTrace(instrSession, window, "TRACE1", arraySize, result, returnedValues));
    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  For the allocation ID vs symbol x carrier, the method returns one
/// HIFN  value for each resource element. The unit is always dBm.
/// HIFN     Represents the allocation ID. The range is as follows.
/// HIFN     -3 = P-Sync
/// HIFN     -4 = S-Sync
/// HIFN     -5 = ESS
/// HIFN     -10 = BRS
/// HIFN     -11 = BRRS
/// HIFN     -12 = CSI RS
/// HIFN     -20 = xPDCCH
/// HIFN     -21 = xPDCCH RS
/// HIFN     -22 = xPBCH
/// HIFN     -23 = ePBCH
/// HIFN     -24 = ePBCH RS
/// HIFN     -1xxxxx = xPDSCH (Port 8 to 11)
/// HIFN     -2xxxxx = xPDSCH (Port 12 to 15)
/// HIFN     -3xxxxx = xPDSCH PCRS
/// HIFN     -4xxxxx = xPUSCH RS
/// HIFN     -5xxxxx = xPUSCH PCRS
/// HIFN     Note. xxxxx is a placeholder for the ID of the channel.
/// HIFN     If the channel has, for example, the ID 22, the return value would be
/// HIFN  -100022, -200022 or -300022 (depending on the configuration)
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR arraySize/Number of array points requested.
/// HIPAR returnedValues/Returns the number of trace data values.
/// HIPAR result/Returns trace data.
ViStatus _VI_FUNC rsspecan_V5GTFFetchAllocationIDvsSymbolxCarrier(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   arraySize,
                            ViInt32   result[],
                            ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;
    ViPInt32 res = NULL;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = ":TRAC? TRACE1";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, window, 0, 16),
            2, "Window");

    if (window != 0)
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, ":TRAC%ld? TRACE1", window);
    checkErr(RsCore_QueryAsciiViInt32Array(instrSession, cmd, &res, returnedValues));
    checkErr(RsCore_CopyToUserBufferViInt32Array(result, arraySize, res, (ViInt64) *returnedValues));
    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    if (res) free(res);
    (void)RsCore_UnlockSession(instrSession);
    return error;
}
