#include "rsspecan.h"

/*****************************************************************************
 *  Rohde&Schwarz Spectrum Analyzer instrument driver
 *  K72,73,74 - WCDMA FDD
 *
 *  Original Release: Januar 2014
 *  By: Helena Kominkova
 *
 *  Should you have any technical questions please contact the hotline of
 *  Rohde & Schwarz Vertriebs-GmbH Rohde & Schwarz Support Center
 *
 *  e-mail: CustomerSupport@rohde-schwarz.com
 *
 *****************************************************************************/


/// HIFN This function switches the instrument into 3GPP FDD BS mode.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
ViStatus _VI_FUNC rsspecan_Set3GPPBSMode(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_BWCD_MODE, ""));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function switches the instrument into 3GPP FDD UE mode.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
ViStatus _VI_FUNC rsspecan_Set3GPPUEMode(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_3GUE_MODE, ""));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN Switches between Normal mode and Home BS (Home Base Station)
/// HIFN mode for ACP and SEM measurements. Switching this parameter
/// HIFN changes the limits according to the specifications.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR baseStationMode/Selects the base station mode.
ViStatus _VI_FUNC rsspecan_Configure3GPPBSStandard(ViSession instrSession,
                                                   ViInt32   baseStationMode)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_3GPP_BS_STANDARD, baseStationMode),
            2, "Base Station Mode");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN Configures the 3GPP FDD BS additional measurement.
/// HIRET Document return value here.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR measurement/Selects the 3GPP FDD BS test.
ViStatus _VI_FUNC rsspecan_Set3GPPBSMeasurement(ViSession instrSession,
                                                ViInt32   measurement)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_3GBS_MEAS, measurement),
            2, "Measurement");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function configures the 3GPP FDD UE additional measurement.
/// HIRET Document return value here.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR measurement/Selects the 3GPP FDD UE test.
ViStatus _VI_FUNC rsspecan_Set3GPPUEMeasurement(ViSession instrSession,
                                                ViInt32   measurement)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_3GUE_MEAS, measurement),
            2, "Measurement");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function determines the grid spacing on the Y-axis for all diagrams, where
/// HIFN possible.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR yScaleDivision/This control sets the scaling of the y-axis in dB.
ViStatus _VI_FUNC rsspecan_Configure3GPPBSYScaleDivision(ViSession instrSession,
                                                         ViReal64  yScaleDivision)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_3GPP_PDIV, yScaleDivision),
            2, " Y Scale Division");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects the I/Q data file to be used as input for further
/// HIFN  measurements. The I/Q data must have a specific format described in
/// HIFN  operating manual.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     -
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     INPut<1|2>:FILE:PATH
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR fileName/String containing the path and name of the source file. The file
/// HIPAR fileName/extension is *.iq.tar.
/// HIPAR analysisBW/Optional. The analysis bandwidth to be used by the measurement. The
/// HIPAR analysisBW/bandwidth must be smaller than or equal to the bandwidth of the data
/// HIPAR analysisBW/that was stored in the file. If value is set to 0.0, the control is
/// HIPAR analysisBW/ignored.
ViStatus _VI_FUNC rsspecan_SelectWCDMAIQFile(ViSession instrSession,
                                             ViString  fileName,
                                             ViReal64  analysisBW)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "INP:FILE:PATH '%s',%.12G", fileName, analysisBW);
    checkErr(RsCore_Write(instrSession, cmd));
    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN Configures the input signal in BTS measurements
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR HSDPAUPA/Switches the HSUPA/HSDPA channel detection On or Off.
/// HIPAR compressedMode/Sets the power control measurement position.
/// HIPAR MIMO/Switches the single antenna MIMO mode on or off.
ViStatus _VI_FUNC rsspecan_Configure3GPPBSSignalDescription(ViSession instrSession,
                                                            ViBoolean HSDPAUPA,
                                                            ViInt32   compressedMode,
                                                            ViBoolean MIMO)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_3GBS_WCDP_HSDPAUPA_STATE, HSDPAUPA),
            2, "HSDPA/UPA");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_3GBS_WCDP_COMPARE_MODE, compressedMode),
            3, "Compressed Mode");

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_3GBS_MIMO_MODE, MIMO),
            4, "MIMO");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function configures the input signal in UE measurements
/// HIFN You may use multiple lines for documentation.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR scramblingCode/Sets the scrambling code in hex format.
/// HIPAR scramblingType/Switches between long and short scrambling code.
/// HIPAR HSDPAUPA/Switches the HSUPA/HSDPA channel detection On or Off.
ViStatus _VI_FUNC rsspecan_Configure3GPPUESignalDescription(ViSession instrSession,
                                                            ViInt32   scramblingCode,
                                                            ViInt32   scramblingType,
                                                            ViBoolean HSDPAUPA)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_3GBS_SCODE, scramblingCode),
            2, "Scrambling Code");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_3GUE_CDP_SCR_TYPE, scramblingType),
            3, "Scrambling Type");

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_3GBS_WCDP_HSDPAUPA_STATE, HSDPAUPA),
            4, "HSDPA/UPA");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function sets the scrambling code in hexadecimal format.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR scramblingCode/Sets the scrambling code in hex format.
ViStatus _VI_FUNC rsspecan_Configure3GPPScramblingCode(ViSession instrSession,
                                                       ViInt32   scramblingCode)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_3GBS_SCODE, scramblingCode),
            2, "Scrambling Code");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function searches automatically the scrambling code that
/// HIFN leads to the highest signal power and stores it as new
/// HIFN scrambling code for further measurements.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
ViStatus _VI_FUNC rsspecan_Search3GPPScramblingCode(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_3GPP_SCODE_SEARCH, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function configures the signal capture.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR invertQ/Inverts the Q component of the signal.
/// HIPAR RRCFilter/Switches the RRC filter ON and OFF.
ViStatus _VI_FUNC rsspecan_Configure3GPPSignalCapture(ViSession instrSession,
                                                      ViBoolean invertQ,
                                                      ViBoolean RRCFilter,
                                                      ViInt32   sideBand)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_3GBS_WCDP_INVERT_Q, invertQ),
            2, "Invert Q");

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_3GUE_RRC_FILTER, RRCFilter),
            3, "RRC Filter");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_3GBS_WCDP_SIDE_BAND, sideBand),
            4, "Side Band");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function configures the frame capture.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR captureLength/Sets the number of frames that are captured by one sweep.
/// HIPAR frameToAnalyze/Sets  the frame number to be analyzed.
ViStatus _VI_FUNC rsspecan_Configure3GPPBSMultiframe(ViSession instrSession,
                                                     ViInt32   captureLength,
                                                     ViInt32   frameToAnalyze)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_3GBS_WCDP_MULTIFRAME_CAPTURE_LENGTH, captureLength),
            2, "Capture Length");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_3GBS_WCDP_MULTIFRAME_FRAME_ANALYZE, frameToAnalyze),
            3, "Frame To Analyze");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function configures the frame analysis mode for UE  measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR analysisMode/Sets the base of the CDP analysis.
ViStatus _VI_FUNC rsspecan_Configure3GPPUEFrameMode(ViSession instrSession,
                                                    ViInt32   analysisMode)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_3GUE_CDP_BASE, analysisMode),
            2, "Analysis Mode");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function configures the signal capture.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR syncType/Sets the type of synchronization. CPICH Synchronization to CPICH
/// HIPAR syncType/(default setting). For this type of synchronization, the CPICH
/// HIPAR syncType/must be present in the transmit signal.
/// HIPAR antenna/Activates or deactivates the antenna diversity mode and selects
/// HIPAR antenna/the antenna to be used.
ViStatus _VI_FUNC rsspecan_Configure3GPPSynchronization(ViSession instrSession,
                                                        ViInt32   syncType,
                                                        ViInt32   antenna)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_3GBS_WCDP_SYNC_TYPE, syncType),
            2, "Sync Type");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_3GBS_ANTENNA, antenna),
            3, "Antenna");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN This function configures the antenna synchronization.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR antenna/Selects the antenna to be configured.
/// HIPAR CPICHMode/Defines whether the common pilot channel (CPICH) is defined by a
/// HIPAR CPICHMode/user-defined position instead of its default position.
/// HIPAR sCPICHCode/Sets the code of the CPICH based on the spreading factor 256.
/// HIPAR sCPICHAntennaPattern/Defines the pattern used for evaluation.
ViStatus _VI_FUNC rsspecan_Configure3GPPAntennaSync(ViSession instrSession,
                                                    ViInt32   antenna,
                                                    ViBoolean CPICHMode,
                                                    ViInt32   sCPICHCode,
                                                    ViInt32   sCPICHAntennaPattern)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_3GPP_CPICH_MODE,
            "WCDMAAntenna", antenna - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Input");
    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, repCap, RSSPECAN_ATTR_3GPP_CPICH_MODE, CPICHMode),
            2, "Sync Type");
    if (CPICHMode == VI_TRUE)
    {
        viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_3GPP_CPICH_CODE, sCPICHCode),
                3, "S-CPICH Code");

        viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_3GPP_CPICH_PATTERN, sCPICHAntennaPattern),
                4, "S-CPICH Antenna Pattern");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN This function configures the 3GPP FDD BS Multicarrier
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR multicarrierState/Switches multicarrier state on or off.
/// HIPAR autoscale/Switches the autoscale state  on or off.
ViStatus _VI_FUNC rsspecan_Configure3GPPBSMulticarrier(ViSession instrSession,
                                                       ViBoolean multicarrierState,
                                                       ViBoolean autoscale)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_3GBS_WCDP_MCACLR, multicarrierState),
            2, "Multicarrier State");
    if (multicarrierState)
    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_3GBS_WCDP_AUTO_SCALE, autoscale),
            3, "Autoscale");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/*****************************************************************************
 * Function: 3GPP FDD BS A Sequence
 * Purpose:  This function starts the following sequence:
 *           - Adjusts the reference level.
 *           - Searches automatically the scrambling code that leads to the
 *             highest signal power and stores it as new scrambling code.
 *           - Changes into the Result Summary result display.
*****************************************************************************/
ViStatus _VI_FUNC rsspecan_Set3GPPBSASequence(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;
    ViUInt32 old_timeout = 0;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsspecan_AssertFSV(instrSession));

    checkErr(rsspecan_GetTimeout(instrSession, &old_timeout));
    checkErr(rsspecan_SetTimeout(instrSession, 30000));

    checkErr(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_3GPP_ASEQ, NULL));

Error:
    if (old_timeout > 0)
        rsspecan_SetTimeout(instrSession, old_timeout);

    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function sets the using of QPSK modulation.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR state/Switches the QPSK modulation state.
ViStatus _VI_FUNC rsspecan_Configure3GPPQPSKModulationOnly(ViSession instrSession,
                                                           ViBoolean state)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_3GPP_QPSK_MODULATION_ONLY_ENABLED, state),
            2, "State");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

//Channel Table

/// HIFN Defines the minimum power that a single channel must have
/// HIFN compared to the total signal in order to be recognized as an
/// HIFN active channel.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR inactiveChannelTreshold/Sets the threshold value from which a channel is treated as
/// HIPAR inactiveChannelTreshold/active. The level entered refers to the total signal power.

ViStatus _VI_FUNC rsspecan_Configure3GPPBSInactiveChannelThreshold(
                            ViSession instrSession,
                            ViReal64  inactiveChannelTreshold)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_3GBS_WCDP_INACT_CHAN_THR, inactiveChannelTreshold),
            2, "Inactive Channel Threshold");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function configures the channel search.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR usingPredefinedChannelTable/Defines the channel search mode.
/// HIPAR measSignalCompare/Sets, if 3GPP FDD application compares the measured signal to
/// HIPAR measSignalCompare/the predefined channel tables or not.
/// HIPAR timingOffset/Sets whether the timing offset and pilot length are measured or
/// HIPAR timingOffset/if the values are taken from the predefined table.
ViStatus _VI_FUNC rsspecan_Configure3GPPBSPredefinedChannelTable(
                            ViSession instrSession,
                            ViBoolean usingPredefinedChannelTable,
                            ViBoolean measSignalCompare,
                            ViInt32   timingOffset)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_3GBS_WCDP_CTAB, usingPredefinedChannelTable),
            2, "Using Predefined Channel Table");
    if (usingPredefinedChannelTable)
    {
        viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_3GBS_MEASUREMENT_SIGNAL_COMPARING, measSignalCompare),
                3, "Meas Signal Compare");

        viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_3GBS_TIMING_OFFSET, timingOffset),
                4, "Timing Offset");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Creates a completely new channel table according to the current
/// HIFN  measurement data.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsspecan_CreateWCDMABSChannelTableAccordingToMeasurement(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsspecan_SetAttributeViString(instrSession, NULL, RSSPECAN_ATTR_BS_CHANNEL_TABLE_NEW_ACCORDING_TO_MEASUREMENT, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function performs the selected operation with channel table.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR operation/This control selects the type of memory operation to be performed.
/// HIPAR channelTable/This control selects a predefined channel table file.
ViStatus _VI_FUNC rsspecan_Configure3GPPFDDBSChannelTableOperations(
                            ViSession instrSession,
                            ViInt32   operation,
                            ViString  channelTable)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidNullPointer(instrSession, channelTable),
                3, "Channel Table");

    switch (operation)
    {
    case RSSPECAN_VAL_CDP_TABLE_OFF:
        checkErr(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_3GBS_WCDP_CTAB, VI_FALSE));
        break;
    case RSSPECAN_VAL_CDP_TABLE_SELECT:
        checkErr(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_3GBS_WCDP_CTAB, VI_TRUE));
        viCheckParm(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_3GBS_WCDP_CTAB_SEL, channelTable),
                3, "Channel Table");
        break;
    case RSSPECAN_VAL_CDP_TABLE_NEW:
        viCheckParm(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_3GBS_WCDP_CTAB_NAME, channelTable),
                3, "Channel Table");
        break;
    case RSSPECAN_VAL_CDP_TABLE_DELETE:
        checkErr(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_3GBS_WCDP_CTAB_DEL, NULL));
        break;
    case RSSPECAN_VAL_CDP_TABLE_COPY:
        viCheckParm(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_3GBS_WCDP_CTAB_COPY, channelTable),
                3, "Channel Table");
        break;
    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, operation), 2, "Operation");
        break;
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function defines a comment for the selected channel table
/// HIFN in the code domain power measurement mode.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR channelTableComment/This control defines a comment for the selected channel table.
ViStatus _VI_FUNC rsspecan_Configure3GPPFDDBSChannelTableComment(
                            ViSession instrSession,
                            ViString  channelTableComment)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_3GBS_WCDP_CTAB_HEAD_COMM, channelTableComment),
            2, "Channel Table Comment");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN This function defines the values of the selected channel table
/// HIFN in the code domain power measurement mode:
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR arraySize/Sets the size of arryas.
/// HIPAR codeClass/Sets array of code class of channel.
/// HIPAR codeNumber/Sets array of code number.
/// HIPAR TFCI/Sets array of the TFCI using.
/// HIPAR timingOffset/Sets array of timing offset.
/// HIPAR pilotLength/Sets array pilot length of channel DPCCH.
/// HIPAR channelType/Sets array of channel type.
/// HIPAR status/Sets the status.
/// HIPAR CDPRelative/Sets array of CDP relative.
ViStatus _VI_FUNC rsspecan_Configure3GPPFDDBSChannelTableData(
                            ViSession instrSession,
                            ViInt32   arraySize,
                            ViInt32   codeClass[],
                            ViInt32   codeNumber[],
                            ViBoolean TFCI[],
                            ViInt32   timingOffset[],
                            ViInt32   pilotLength[],
                            ViInt32   channelType[],
                            ViBoolean status[],
                            ViReal64  CDPRelative[])
{
    ViStatus error = VI_SUCCESS;
    ViInt32  i = 0;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";
    ViChar*  pbuffer = cmd;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32RangeMin(instrSession, arraySize, 1),
                2, "Array Size");

    pbuffer += sprintf(pbuffer, "CONF:WCDP:CTAB:DATA ");
    for (i = 0; i < arraySize; i++)
        pbuffer += sprintf(pbuffer, "%ld,%ld,%d,%ld,%ld,%ld,%d,%.12lG,",
                           codeClass[i], codeNumber[i], TFCI[i], timingOffset[i], pilotLength[i], channelType[i],
                           status[i], CDPRelative[i]);

    *--pbuffer = '\0'; // Remove remaining comma

    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function reads out the names of all channel tables stored
/// HIFN on the harddisk in the code domain power measurement mode.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR numberofChannelTables/This control returns number of channel tables, stored on
/// HIPAR numberofChannelTables/harddisk.
/// HIPAR bufferSize/Pass the number of bytes in the string you specify for the
/// HIPAR bufferSize/catalog list parameter.
/// HIPAR channelTablesList/This control returns array of channel tables, stored on
/// HIPAR channelTablesList/harddisk.
ViStatus _VI_FUNC rsspecan_Query3GPPFDDBSChannelTableCatalog(ViSession instrSession,
                                                             ViInt32*  numberofChannelTables,
                                                             ViInt32   bufferSize,
                                                             ViChar    channelTablesList[])
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32RangeMin(instrSession, bufferSize, 0),
                2, "Buffer Size");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, channelTablesList),
                3, "Channel Table List");

    checkErr(
        RsCore_QueryCatalog(instrSession, ":CONF:WCDP:CTAB:CAT?", bufferSize, channelTablesList, numberofChannelTables
        ));

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN This function configures the channel search.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR usingPredefinedChannelTable/Defines the channel search mode.
ViStatus _VI_FUNC rsspecan_Configure3GPPUEPredefinedChannelTable(
                            ViSession instrSession,
                            ViBoolean usingPredefinedChannelTable)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_3GUE_WCDP_CTAB, usingPredefinedChannelTable),
            2, "Using Predefined Channel Table");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Creates a completely new channel table according to the current
/// HIFN  measurement data.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsspecan_CreateWCDMAUEChannelTableAccordingToMeasurement(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsspecan_SetAttributeViString(instrSession, NULL, RSSPECAN_ATTR_UE_CHANNEL_TABLE_NEW_ACCORDING_TO_MEASUREMENT, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function performs the selected operation with channel table.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR operation/This control selects the type of memory operation to be performed.
/// HIPAR channelTable/This control selects a predefined channel table file.
ViStatus _VI_FUNC rsspecan_Configure3GPPFDDUEChannelTableOperations(
                            ViSession instrSession,
                            ViInt32   operation,
                            ViString  channelTable)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidNullPointer(instrSession, channelTable),
                3, "Channel Table");

    switch (operation)
    {
    case RSSPECAN_VAL_CDP_TABLE_OFF:
        checkErr(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_3GUE_WCDP_CTAB, VI_FALSE));
        break;
    case RSSPECAN_VAL_CDP_TABLE_SELECT:
        checkErr(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_3GUE_WCDP_CTAB, VI_TRUE));
        viCheckParm(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_3GUE_WCDP_CTAB_SEL, channelTable),
                3, "Channel Table");
        break;
    case RSSPECAN_VAL_CDP_TABLE_NEW:
        viCheckParm(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_3GUE_WCDP_CTAB_NAME, channelTable),
                3, "Channel Table");
        break;
    case RSSPECAN_VAL_CDP_TABLE_DELETE:
        viCheckParm(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_3GUE_WCDP_CTAB_DEL, channelTable),
                3, "Channel Table");
        break;
    case RSSPECAN_VAL_CDP_TABLE_COPY:
        viCheckParm(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_3GUE_WCDP_CTAB_COPY, channelTable),
                3, "Channel Table");
        break;
    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, operation), 2, "Operation");
        break;
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function defines a comment for the selected channel table
/// HIFN in the code domain power measurement mode.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR channelTableComment/This control defines a comment for the selected channel table.
ViStatus _VI_FUNC rsspecan_Configure3GPPFDDUEChannelTableComment(
                            ViSession instrSession,
                            ViString  channelTableComment)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_3GUE_WCDP_CTAB_COMMENT, channelTableComment),
            2, "Channel Table Comment");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function defines the values of the selected channel table
/// HIFN in the code domain power measurement mode:
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR arraySize/Sets the size of arryas.
/// HIPAR codeClass/Sets array of code class of channel.
/// HIPAR noActiveChannels/Number of active channels
/// HIPAR pilotLength/Sets pilot length of channel DPCCH.
ViStatus _VI_FUNC rsspecan_Configure3GPPUEChannelTableData(ViSession instrSession,
                                                           ViInt32   arraySize,
                                                           ViInt32   codeClass[],
                                                           ViInt32   noActiveChannels[],
                                                           ViInt32   pilotLength[])
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32RangeMin(instrSession, arraySize, 1),
                2, "Array Size");

    checkErr(RsCore_WriteAsciiViInt32ArraysInterleaved(instrSession, "CONF:WCDP:MS:CTAB:DATA ", arraySize,
        codeClass, noActiveChannels, pilotLength, NULL, NULL, NULL));

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function defines the values of the selected channel table
/// HIFN in the code domain power measurement mode:
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR arraySize/Sets the size of arryas.
/// HIPAR codeClass/Sets array of code class of channel.
/// HIPAR noActiveChannels/Number of active channels
ViStatus _VI_FUNC rsspecan_Configure3GPPUEChannelTableDataEDPC(
                            ViSession instrSession,
                            ViInt32   arraySize,
                            ViInt32   codeClass[],
                            ViInt32   noActiveChannels[])
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32RangeMin(instrSession, arraySize, 1),
                2, "Array Size");

    checkErr(RsCore_WriteAsciiViInt32ArraysInterleaved(instrSession, "CONF:WCDP:MS:CTAB:EDAT ", arraySize,
        codeClass, noActiveChannels, NULL, NULL, NULL, NULL));

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function activates or deactivates the E-DPCCH entry in a
/// HIFN predefined channel table
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR state/Activates/deactivates the E-DPCCH entry.
ViStatus _VI_FUNC rsspecan_Configure3GPPUEChannelTableEDPCEntry(
                            ViSession instrSession,
                            ViBoolean state)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_3GUE_WCDP_CTAB_EDPC, state),
            2, "State");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function activates/deactivates the high speed channel in
/// HIFN active channel table.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR state/Activates/deactivates the high speed channel.
ViStatus _VI_FUNC rsspecan_Configure3GPPFDDUEChannelTableHighSpeedChannel(
                            ViSession instrSession,
                            ViBoolean state)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_3GUE_WCDP_CTAB_HSDP, state),
            2, "State");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function reads out the names of all channel tables stored
/// HIFN on the harddisk in the code domain power measurement mode.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR numberOfChannelTables/This control returns number of channel tables, stored on
/// HIPAR numberOfChannelTables/harddisk.
/// HIPAR bufferSize/Pass the number of bytes in the string you specify for the
/// HIPAR bufferSize/catalog list parameter.
/// HIPAR channelTablesList/This control returns array of channel tables, stored on
/// HIPAR channelTablesList/harddisk.
ViStatus _VI_FUNC rsspecan_Query3GPPFDDUEChannelTableCatalog(ViSession instrSession,
                                                             ViInt32*  numberOfChannelTables,
                                                             ViInt32   bufferSize,
                                                             ViChar    channelTablesList[])
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32RangeMin(instrSession, bufferSize, 0),
                2, "Buffer Size");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, channelTablesList),
                3, "Channel Table List");

    checkErr(
        RsCore_QueryCatalog(instrSession, ":CONF:WCDP:MS:CTAB:CAT?", bufferSize, channelTablesList,
            numberOfChannelTables));

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function loads the specified carrier table as the default table
/// HIFN  ("RECENT") for Time Alignment Error measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR carrierTable/This control selects a filename of the stored carrier table.
ViStatus _VI_FUNC rsspecan_WCDMATAELoadDefaultCarrierTable(ViSession instrSession,
                                                           ViString  carrierTable)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_3GBS_LOAD_DEFAULT_CARRIER_TABLE, carrierTable),
            2, "Carrier Table");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function creates new, saves or deletes a specified carrier table
/// HIFN  for Time Alignment Error measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR operation/This control selects the type of operation to be performed.
/// HIPAR carrierTable/This control selects a filename of the new carrier table to be created
/// HIPAR carrierTable/in the default directory. The default directory for carrier tables is
/// HIPAR carrierTable/C:\R_SInstr\user\chan_tab\carrier_table\.
ViStatus _VI_FUNC rsspecan_WCDMATAECarrierTableOperations(ViSession instrSession,
                                                          ViInt32   operation,
                                                          ViString  carrierTable)
{
    ViStatus error = VI_SUCCESS;
    ViAttr   attr[] = {
             RSSPECAN_ATTR_3GBS_NEW_CARRIER_TABLE,
             RSSPECAN_ATTR_3GBS_SAVE_CARRIER_TABLE,
             RSSPECAN_ATTR_3GBS_DELETE_CARRIER_TABLE
             };

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, operation, RSSPECAN_VAL_TAER_CARRIER_TABLE_NEW, RSSPECAN_VAL_TAER_CARRIER_TABLE_DELETE),
            2, "Operation");

    viCheckParm(rsspecan_SetAttributeViString(instrSession, "", attr[operation], carrierTable),
            3, "Carrier Table");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function lists the carrier table names of all carrier table files
/// HIFN  found in the default directory. The default directory for carrier
/// HIFN  tables is C:\R_SInstr\user\chan_tab\carrier_table\.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR bufferSize/Pass the number of bytes in the string you specify for the catalog
/// HIPAR bufferSize/list parameter.
/// HIPAR carrierTablesList/This control returns a list of carrier tables.
ViStatus _VI_FUNC rsspecan_GetWCDMATAECarrierTableCatalog(ViSession instrSession,
                                                          ViInt32   bufferSize,
                                                          ViChar    carrierTablesList[])
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_GetAttributeViString(instrSession, "", RSSPECAN_ATTR_3GBS_CARRIER_TABLE_CATALOG,
            bufferSize, carrierTablesList),
            3, "Carrier Tables List");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function queries the number of carriers defined in the currently
/// HIFN  selected carrier table for Time Alignment Error measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR numberOfCarriers/Queries the number of carriers defined in the currently selected
/// HIPAR numberOfCarriers/carrier table for Time Alignment Error measurement.
ViStatus _VI_FUNC rsspecan_GetWCDMATAENumberOfCarriers(ViSession instrSession,
                                                       ViInt32*  numberOfCarriers)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_GetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_3GBS_NUMBER_OF_CARRIERS, numberOfCarriers),
            2, "Number Of Carriers");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function inserts or deletes a carrier in the carrier table for
/// HIFN  Time Alignment Error measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR carrier/Selects the carrier in carrier table. The suffix must refer to a
/// HIPAR carrier/carrier already defined in the current table, or to the first row after
/// HIPAR carrier/the last defined carrier.
/// HIPAR operation/This control selects the type of operation to be performed.
ViStatus _VI_FUNC rsspecan_WCDMATAECarrierOperations(ViSession instrSession,
                                                     ViInt32   carrier,
                                                     ViInt32   operation)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViAttr   attr[] = {
             RSSPECAN_ATTR_3GBS_INSERT_NEW_CARRIER,
             RSSPECAN_ATTR_3GBS_DELETE_CARRIER,
             RSSPECAN_ATTR_3GBS_DELETE_ALL_CARRIERS
             };

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, carrier, 1, 4),
            2, "Carrier");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, operation, RSSPECAN_VAL_TAER_CARRIER_INSERT, RSSPECAN_VAL_TAER_CARRIER_DELETE_ALL),
            3, "Operation");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_3GBS_INSERT_NEW_CARRIER,
            "WCDMACarrier", carrier - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Carrier");

    checkErr(rsspecan_SetAttributeViString(instrSession, repCap, attr[operation], NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the carrier table for Time Alignment Error
/// HIFN  measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR carrier/Selects the carrier in carrier table. The suffix must refer to a
/// HIPAR carrier/carrier already defined in the current table, or to the first row after
/// HIPAR carrier/the last defined carrier.
/// HIPAR frequencyOffset/Defines or queries the frequency offset of the carrier specified by
/// HIPAR frequencyOffset/the CARRier suffix in the currently selected carrier table for Time
/// HIPAR frequencyOffset/Alignment Error measurement. The frequency offset is defined with
/// HIPAR frequencyOffset/respect to the reference carrier. (The reference carrier is set to the
/// HIPAR frequencyOffset/current center frequency, thus the offset is always 0.)
/// HIPAR scramblingCode/Defines or queries the scrambling code of the carrier specified by the
/// HIPAR scramblingCode/CARRier suffix in the currently selected carrier table for Time
/// HIPAR scramblingCode/Alignment Error measurement. The scrambling code for the reference
/// HIPAR scramblingCode/carrier is defined/queried using RSSPECAN_ATTR_3GBS_SCODE.
/// HIPAR antenna/This control selects antenna to be configured.
/// HIPAR cpichNumber/Defines or queries the CPICH of the specified antenna for the carrier
/// HIPAR cpichNumber/specified by the CARRier suffix in the currently selected carrier
/// HIPAR cpichNumber/table for Time Alignment Error measurement. For antenna 1, the value
/// HIPAR cpichNumber/can be queried only, not defined.
/// HIPAR cpichPattern/Defines or queries the pattern of the specified antenna for the
/// HIPAR cpichPattern/carrier specified by the CARRier suffix in the currently selected
/// HIPAR cpichPattern/carrier table for Time Alignment Error measurement. For antenna 1, the
/// HIPAR cpichPattern/value can be queried only, not defined.
ViStatus _VI_FUNC rsspecan_ConfigureWCDMATAECarrierTable(ViSession instrSession,
                                                         ViInt32   carrier,
                                                         ViReal64  frequencyOffset,
                                                         ViInt32   scramblingCode,
                                                         ViInt32   antenna,
                                                         ViInt32   cpichNumber,
                                                         ViInt32   cpichPattern)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap_c[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_a[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, carrier, 1, 4),
            2, "Carrier");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, antenna, 1, 2),
            5, "Antenna");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_3GBS_CARRIER_FREQUENCY_OFFSET,
            "WCDMACarrier", carrier - 1, RS_REPCAP_BUF_SIZE, repCap_c),
            2, "Carrier");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_3GBS_CARRIER_CPICH,
            "WCDMAAntenna", antenna - 1, RS_REPCAP_BUF_SIZE, repCap_a),
            5, "Antenna");

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, repCap_c, RSSPECAN_ATTR_3GBS_CARRIER_FREQUENCY_OFFSET, frequencyOffset),
            3, "Frequency Offset");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap_c, RSSPECAN_ATTR_3GBS_CARRIER_SCRAMBLING_CODE, scramblingCode),
            4, "Scrambling Code");

    strcat(repCap, repCap_c);
    strcat(repCap, ",");
    strcat(repCap, repCap_a);

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_3GBS_CARRIER_CPICH, cpichNumber),
            6, "CPICH Number");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_3GBS_CARRIER_PATTERN, cpichPattern),
            7, "CPICH Pattern");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/* Analysis */

/// HIFN This function configures the 3GPP FDD BS results
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR channel/Sets the code number. The code number refers to code class 9 (spreading factor 512).
/// HIPAR cpichSlot/Sets the slot number of the common pilot channels (CPICH).
/// HIPAR spreadingFactor/Defines the spreading factor.
ViStatus _VI_FUNC rsspecan_Configure3GPPBSResults(ViSession instrSession,
                                                  ViInt32   channel,
                                                  ViInt32   cpichSlot,
                                                  ViInt32   spreadingFactor)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_3GBS_RESULTS_SELECT_CHAN, channel),
            2, "Channel");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_3GBS_RESULTS_SELECT_CPICH_SLOT, cpichSlot),
            3, "CPICH Slot");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_3GBS_RESULTS_SFAC, spreadingFactor),
            4, "Spreading Factor");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function configures evaluation analysis branch for UE.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR selectIQ/Switches between I and Q component of the signal.
ViStatus _VI_FUNC rsspecan_Configure3GPPUEBranch(ViSession instrSession,
                                                 ViInt32   selectIQ)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_3GUE_CDP_SEL_COMP, selectIQ),
            2, "Select I/Q");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function configures the 3GPP FDD BS Code Domain Analysis.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR compensateIQOffset/Switches elimination of IQ offset on or off.
/// HIPAR codePowerDisplay/Switches between showing the absolute or relative power.
/// HIPAR codePowerDisplay/This parameter only affects the Code Domain Power evaluation.
/// HIPAR powerReference/Switches between the use of total power or
/// HIPAR powerReference/CPICH power as the reference for the relative CDP measurement values.
/// HIPAR powerDifference/Activates or deactivates the slot power difference calculation
/// HIPAR powerDifference/of the POWER VERSUS SLOT display.
/// HIPAR constellationParameterB/Sets selects the constellation parameter B.
ViStatus _VI_FUNC rsspecan_Configure3GPPBSCodeDomain(ViSession instrSession,
                                                     ViBoolean compensateIQOffset,
                                                     ViInt32   codePowerDisplay,
                                                     ViInt32   powerReference,
                                                     ViBoolean powerDifference,
                                                     ViInt32   constellationParameterB)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_3GBS_WCDP_NORMALIZE, compensateIQOffset),
            2, "Compensate I/Q Offset");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_3GBS_POWER_DISPLAY, codePowerDisplay),
            3, "Code Power Display");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_3GBS_WCDP_POW_REF, powerReference),
            4, "Power Reference");

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_3GBS_WCDP_POW_DIFFERENCE, powerDifference),
            5, "Power Difference");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_3GBS_WCDP_CONST_B, constellationParameterB),
            6, "Constellation Parameter B");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN This function configures the 3GPP FDD UE Code Domain Analysis.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR measurementInterval/Switches between the analysis of half slots and full slots.
/// HIPAR eliminateTailChips/Selects length of the measurement interval for calculation of
/// HIPAR eliminateTailChips/error vector magnitude (EVM).
ViStatus _VI_FUNC rsspecan_Configure3GPPUECodeDomain(ViSession instrSession,
                                                     ViBoolean measurementInterval,
                                                     ViBoolean compensateIQOffset,
                                                     ViBoolean eliminateTailChips,
                                                     ViInt32   codePowerDisplay)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_3GUE_HSLOT, measurementInterval),
            2, "Measurement Interval");

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_3GBS_WCDP_NORMALIZE, compensateIQOffset),
            3, "Compensate I/Q Offset");

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_3GUE_CDP_ELIMINATE_TAIL_CHIPS, eliminateTailChips),
            4, "Eliminate Tail Chip");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_3GBS_POWER_DISPLAY, codePowerDisplay),
            5, "Code Power Display");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function configures the 3GPP FDD BS result type
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR resultType/Selects the measured data that are to be displayed.
ViStatus _VI_FUNC rsspecan_Configure3GPPBSResultType(ViSession instrSession,
                                                     ViInt32   resultType)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    switch (resultType)
    {
    case RSSPECAN_VAL_3GBS_RES_CDP:
    case RSSPECAN_VAL_3GBS_RES_CDPRAT:
    case RSSPECAN_VAL_3GBS_RES_CDEP:
    case RSSPECAN_VAL_3GBS_RES_CTAB:
    case RSSPECAN_VAL_3GBS_RES_SUM:
        viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "C1", RSSPECAN_ATTR_3GBS_RESULT, resultType),
                2, "Result Type");
        break;
    case RSSPECAN_VAL_3GBS_RES_PCD:
    case RSSPECAN_VAL_3GBS_RES_MACC:
    case RSSPECAN_VAL_3GBS_RES_PVSLA:
    case RSSPECAN_VAL_3GBS_RES_PVSLR:
    case RSSPECAN_VAL_3GBS_RES_BSTR:
    case RSSPECAN_VAL_3GBS_RES_SYMBCONST:
    case RSSPECAN_VAL_3GBS_RES_SYMBEVM:
    case RSSPECAN_VAL_3GBS_RES_FVSL:
    case RSSPECAN_VAL_3GBS_RES_PVSY:
    case RSSPECAN_VAL_3GBS_RES_EVM_MAGN:
    case RSSPECAN_VAL_3GBS_RES_EVM_PHAS:
    case RSSPECAN_VAL_3GBS_RES_COMP_CONS:
    case RSSPECAN_VAL_3GBS_RES_CHIP_EVM:
    case RSSPECAN_VAL_3GBS_RES_CHIP_MAGN:
    case RSSPECAN_VAL_3GBS_RES_CHIP_PHAS:
    case RSSPECAN_VAL_3GBS_RES_PSVSL:
    case RSSPECAN_VAL_3GBS_RES_CDPOVER:
        viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "C2", RSSPECAN_ATTR_3GBS_RESULT, resultType),
                2, "Result Type");
        break;
    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, resultType), 2, "Result Type");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function sets the view of the Code Domain Result Summary.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR view/This control sets the view of the Code Domain Result Summary.
ViStatus _VI_FUNC rsspecan_Configure3GPPBSResultSummaryView(ViSession instrSession,
                                                            ViInt32   view)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_3GBS_RESULT_SUM_VIEW, view),
            2, "View");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function sets the marker to selected channel
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR channel/Specifies the type of marker search type.
ViStatus _VI_FUNC rsspecan_Set3GPPBSMarkerToChannel(ViSession instrSession,
                                                    ViInt32   window,
                                                    ViInt32   marker,
                                                    ViInt32   channel)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, window, 0, 16),
            2, "Window");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "C%ld,M%ld", window, marker);

    switch (channel)
    {
    case RSSPECAN_VAL_3GPP_MARKER_CPICH:
        checkErr(rsspecan_SetAttributeViString(instrSession, repCap, RSSPECAN_ATTR_3GPP_MARKER_TO_CPICH, NULL));
        break;
    case RSSPECAN_VAL_3GPP_MARKER_PCCPCH:
        checkErr(rsspecan_SetAttributeViString(instrSession, repCap, RSSPECAN_ATTR_3GPP_MARKER_TO_PCCP, NULL));
        break;
    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, channel), 3, "Channel");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function sets the marker to selected channel
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR channel/Specifies the type of marker search type.
ViStatus _VI_FUNC rsspecan_Set3GPPBSDeltaMarkerToChannel(ViSession instrSession,
                                                         ViInt32   window,
                                                         ViInt32   marker,
                                                         ViInt32   channel)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, window, 0, 16),
            2, "Window");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "C%ld,M%ld", window, marker);

    switch (channel)
    {
    case RSSPECAN_VAL_3GPP_MARKER_CPICH:
        checkErr(rsspecan_SetAttributeViString(instrSession, repCap, RSSPECAN_ATTR_3GPP_REFERENCE_MARKER_TO_CPIC, NULL));
        break;
    case RSSPECAN_VAL_3GPP_MARKER_PCCPCH:
        checkErr(rsspecan_SetAttributeViString(instrSession, repCap, RSSPECAN_ATTR_3GPP_REFERENCE_MARKER_TO_PCCP, NULL));
        break;
    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, channel), 3, "Channel");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN This function returns result table of the automatic search
/// HIFN sequence containing the highest power values calculated and the
/// HIFN corresponding scrambling codes.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR arraySize/Defines the number of items in arrays Scrambilng Code and Power.
/// HIPAR scrambingCode/Returns scrambling code values.
/// HIPAR power/Returns power of selected channels.
/// HIPAR returnedValues/Returned number of values readed from instrument.
ViStatus _VI_FUNC rsspecan_Get3GPPBSCDPScramblingCodeResult(ViSession instrSession,
                                                            ViInt32   arraySize,
                                                            ViInt32   scrambingCode[],
                                                            ViReal64  power[],
                                                            ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;
    ViChar*  pbuffer = NULL;
    ViChar*  pstring_value;
    ViInt32  idx = 0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidNullPointer(instrSession, scrambingCode),
                3, "Scrambling Code");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, power),
                4, "WiPowerdow");

    checkErr(RsCore_QueryViStringUnknownLength(instrSession, "CDP:LCODe:SEAR:LIST?", &pbuffer));

    //returned values: List of: <Code1/decimal>,<Code2/hex>,<CPICHPower>,...
    if (strlen(pbuffer) > 1)
    {
        pstring_value = strtok(pbuffer, ",");

        while (pstring_value)
        {
            if (idx < arraySize)
            {
                scrambingCode[idx] = RsCore_Convert_String_To_ViInt32(pstring_value);
                (void)strtok(NULL, ",");
                pstring_value = strtok(NULL, ",");
                power[idx] = RsCore_Convert_String_To_ViReal64(pstring_value);
                pstring_value = strtok(NULL, ",");
            }
            idx++;
        }
    }
    if (returnedValues)
        *returnedValues = idx;

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    if (pbuffer) free(pbuffer);
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function returns data that represent the values in array.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR type/Selects the trace type.
/// HIPAR trace/Selects the trace number.
/// HIPAR arrayLength/Number of array points requested.
/// HIPAR actualPoints/Number of points actually returned from the instrument.
/// HIPAR values/A user allocated buffer into which the trace values are stored.
ViStatus _VI_FUNC rsspecan_Get3GPPTrace(ViSession instrSession,
                                        ViInt32   window,
                                        ViInt32   type,
                                        ViInt32   trace,
                                        ViInt32   arrayLength,
                                        ViInt32*  actualPoints,
                                        ViReal64  values[])
{
    ViStatus error = VI_SUCCESS;
    ViChar*  p2string;
    ViChar   traceString[RS_MAX_SHORT_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, window, 0, 16),
            2, "Window");
    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &rsspecan_rng3GPPDataTrace, type, &p2string),
                3, "Type");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, trace, 1, 4),
            4, "Trace");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, actualPoints),
                6, "Actual Points");

    snprintf(traceString, RS_MAX_SHORT_MESSAGE_BUF_SIZE, "%s%ld", p2string, trace);
    checkErr(rsspecan_dataReadTrace(instrSession, window, traceString, arrayLength, values, actualPoints));

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);

    return error;
}

/// HIFN This function returns data that represent the values in array.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR type/Selects the trace type.
/// HIPAR arrayLength/Number of array points requested.
/// HIPAR actualPoints/Number of points actually returned from the instrument.
/// HIPAR values/A user allocated buffer into which the trace values are stored.
ViStatus _VI_FUNC rsspecan_Get3GPPTraceAdditional(ViSession instrSession,
                                                  ViInt32   window,
                                                  ViInt32   type,
                                                  ViInt32   arrayLength,
                                                  ViInt32*  actualPoints,
                                                  ViReal64  values[])
{
    ViStatus error = VI_SUCCESS;
    ViChar*  p2trace;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, window, 0, 16),
            2, "Window");
    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &rsspecan_rng3GPPDataTraceAdditional, type, &p2trace),
                3, "Type");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, actualPoints),
                5, "Actual Points");

    checkErr(rsspecan_dataReadTrace(instrSession, window, p2trace, arrayLength, values, actualPoints));

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);

    return error;
}

/// HIFN This function returns the WCDP measurement Results
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR type/This control sets the result type.
/// HIPAR result/Returns the measured result.
ViStatus _VI_FUNC rsspecan_Get3GPPBSResult(ViSession instrSession,
                                           ViInt32   type,
                                           ViReal64* result)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViChar*  p2string;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &rsspecan_rng3GBSResults, type, &p2string),
                3, "Type");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, result),
                3, "Result");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CALC:MARK:FUNC:WCDP:RES? %s", p2string);
    checkErr(RsCore_QueryViReal64(instrSession, cmd, result));
    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function returns the power result.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR type/This control sets the measurement.
/// HIPAR marker/Sets the marker number.
/// HIPAR arraySize/Defines the memory size allocated for Results.
/// HIPAR results/Returns the measured results.
/// HIPAR returnedValues/Document parameter here.
ViStatus _VI_FUNC rsspecan_Get3GPPBSPowerResult(ViSession instrSession,
                                                ViInt32   marker,
                                                ViInt32   type,
                                                ViInt32   arraySize,
                                                ViReal64  results[],
                                                ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViChar*  p2string;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, marker, 1, 4),
            2, "Marker");
    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &rsspecan_rngMeasPowerSelect, type, &p2string),
                3, "Type");
    viCheckParm(RsCore_InvalidViInt32RangeMin(instrSession, arraySize, 0),
                3, "Array Size");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, results),
                4, "Results");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CALC:MARK%ld:FUNC:POW:RES? %s", marker, p2string);
    checkErr(RsCore_QueryFloatArrayToUserBuffer(instrSession, cmd, arraySize, results, returnedValues));

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function queries the list evaluation results. All results
/// HIFN are float values.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR noOfValues/This control sets the number of values returned in arrays.
/// HIPAR noOfValues/The array size has to be at least equal to this number.
/// HIPAR rangeNumber/Returns an array of range numbers
/// HIPAR startFrequency/Returns an array of start frequencies
/// HIPAR stopFrequency/Returns an array of stop frequencies
/// HIPAR resolutionBandwidth/Returns an array of resolution bandwidths
/// HIPAR frequency/Returns an array of frequencies
/// HIPAR powerAbs/Returns an array of absolute power of the peak within the range
/// HIPAR powerRel/Returns an array of relative power of the peak within the range
/// HIPAR delta/Returns an array of power differencies to margin power
/// HIPAR limitCheck/Returns an array of limit check state
/// HIPAR reserved1/Reserved (0.0)
/// HIPAR reserved2/Reserved (0.0)
/// HIPAR returnedValues/Returns number of value sets (frequency, level and delta to
/// HIPAR returnedValues/limit line) read (available to return) from the instrument.
/// HIPAR returnedValues/Number of values returned in arrays is limited with the
/// HIPAR returnedValues/parameter No of Values.
ViStatus _VI_FUNC rsspecan_Get3GPPBSCDPListEvaluationResult(ViSession instrSession,
                                                            ViInt32   noOfValues,
                                                            ViReal64  rangeNumber[],
                                                            ViReal64  startFrequency[],
                                                            ViReal64  stopFrequency[],
                                                            ViReal64  resolutionBandwidth[],
                                                            ViReal64  frequency[],
                                                            ViReal64  powerAbs[],
                                                            ViReal64  powerRel[],
                                                            ViReal64  delta[],
                                                            ViInt32   limitCheck[],
                                                            ViReal64  reserved1[],
                                                            ViReal64  reserved2[],
                                                            ViInt32*  returnedValues)
{
    ViStatus  error = VI_SUCCESS;
    ViInt32   count = 0;
    ViUInt32  retCnt;
    ViUInt32  i;
    ViReal64* data = NULL;
    ViInt32   arrayLength;
    ViInt32   actualPoints;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidNullPointer(instrSession, rangeNumber),
                2, "Range number");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, startFrequency),
                3, "Start Frequency");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, stopFrequency),
                4, "Stop Frequency");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, resolutionBandwidth),
                5, "Resolution Bandwidth");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, frequency),
                6, "Frequency");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, powerAbs),
                7, "Power Abs");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, powerRel),
                8, "Power Rel");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, delta),
                9, "Delta");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, limitCheck),
                10, "Limit Check");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, reserved1),
                11, "Reserved 1");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, reserved2),
                12, "Reserved 2");

    arrayLength = noOfValues * 11;
    viCheckAlloc(data = (ViReal64*)malloc(arrayLength * sizeof(ViReal64)));
    checkErr(rsspecan_dataReadTrace(instrSession, 0, "LIST", arrayLength, data, &actualPoints));
    count = 0;
    i = 0;
    retCnt = (arrayLength < actualPoints) ? arrayLength : actualPoints;

    do
    {
        rangeNumber[count] = data[i++];
        startFrequency[count] = data[i++];
        stopFrequency[count] = data[i++];
        resolutionBandwidth[count] = data[i++];
        frequency[count] = data[i++];
        powerAbs[count] = data[i++];
        powerRel[count] = data[i++];
        delta[count] = data[i++];
        limitCheck[count] = (ViInt32)data[i++];
        reserved1[count] = data[i++];
        reserved2[count] = data[i++];
        count++;
    }
    while (i < retCnt);

    if (returnedValues)
        *returnedValues = actualPoints / 11;

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    if (data) free(data);
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function queries the result of the Time Alignment Error measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
/// HIPAR type/This control sets the result.
/// HIPAR result/Returns the measured result.
ViStatus _VI_FUNC rsspecan_Get3GPPBSTimeAlignmentErrorResult(ViSession instrSession,
                                                             ViInt32   type,
                                                             ViReal64* result)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViChar*  p2string;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &rsspecan_rng3GPPTaerResult, type, &p2string),
                3, "Type");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, result),
                3, "Result");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CALC:MARK:FUNC:TAER:RES? %s", p2string);
    checkErr(RsCore_QueryViReal64(instrSession, cmd, result));

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN Returns the measured and calculated results of the 3GPP FDD code
/// HIFN domain power measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR type/This control sets the result type.
/// HIPAR result/Returns the measured result.
ViStatus _VI_FUNC rsspecan_Get3GPPUEResult(ViSession instrSession,
                                           ViInt32   type,
                                           ViReal64* result)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViChar*  p2string;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &rsspecan_rng3GBSResults, type, &p2string),
                3, "Type");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, result),
                3, "Result");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CALC:MARK:FUNC:WCDP:MS:RES? %s", p2string);
    checkErr(RsCore_QueryViReal64(instrSession, cmd, result));
    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Exports the marker peak list to a file. The file format is *.dat.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR fileName/Exports the marker peak list to a file. The file format is *.dat.
ViStatus _VI_FUNC rsspecan_ExportWCDMAMarkerPeakListToFile(ViSession instrSession,
                                                           ViString  fileName)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViString(instrSession, NULL, RSSPECAN_ATTR_3GPP_EXPORT_MARKER_PEAK_LIST_TO_FILE, fileName),
            2, "File Name");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/*****************************************************************************
 *- User defined callbacks --------------------------------------------------*
 *****************************************************************************/
