/*****************************************************************************
 *  Rohde&Schwarz Spectrum Analyzer instrument driver
 *  VXIpnp Style Instrument Driver
 *  Original Release: July 2008
 *  By: Martin Koutny (instrument driver core)
 *      Helena Sucha (instrument specific implementation)
 *      Jiri Kominek (DrBase repository & tools)
 *
 *  Should you have any technical questions please contact the hotline of
 *  Rohde & Schwarz Vertriebs-GmbH Rohde & Schwarz Support Center
 *
 *  e-mail: CustomerSupport@rohde-schwarz.com
 *
 *  Modification History:
 *     see ChangeLog
 *
 *****************************************************************************/
#include "rsspecan.h"
#include "rsspecan_internal.h"
#include "time.h"
#include "visatype.h"
#if (defined(WIN32) || defined(_WIN32) || defined(__WIN32__) || defined(__NT__))
#define ANSI
#endif
#ifndef ANSI                /* UNIX compatible */
    #include <sys/timeb.h>
    #include <sys/types.h>
    #include <unistd.h>
    #define Sleep sleep
#else                       /* ANSI compatible */
#include <Windows.h>
#endif
#if defined (_CVI_)
    #include <utility.h>
#endif

/*****************************************************************************
 *- Value Definition and Range Tables ---------------------------------------*
 *****************************************************************************/
//static ViString listFilterTypeArr[] = {"NORM","CFIL","RRC","FFT","PULS"};
static ViString messageTypeArr[] = {"ALL", "INFO", "WARN", "ERR", "FAT", "MESS", NULL};
static ViString userCorrectionDataType[] = {"FREQ", "MAGN", "PHAS", NULL};
static ViString gapLowUppArr[] = {"LOW","UPP", NULL};
static ViString gapsArr[] = {"AB","BC","CD","DE","EF","FG","GH", NULL};

/*****************************************************************************
 *- Callback Declarations ---------------------------------------------------*
 *****************************************************************************/


/*****************************************************************************
 *- User-Callable Functions (Exportable Functions) --------------------------*
 *****************************************************************************/
/*****************************************************************************
 * Function: rsspecan_init
 * Purpose:  VXIplug&play required function. Calls the
 *           rsspecan_InitWithOptions function.
 *****************************************************************************/
ViStatus _VI_FUNC rsspecan_init(ViRsrc resourceName, ViBoolean IDQuery,
                                ViBoolean resetDevice, ViSession *newInstrSession)
{
    ViStatus error = VI_SUCCESS;

    if (newInstrSession == NULL)
    {
        (void)RsCore_SetErrorInfo(0, VI_FALSE, RS_ERROR_INVALID_PARAMETER,
                                  VI_ERROR_PARAMETER4, "Null address for Instrument Handle");

        checkErr(RS_ERROR_INVALID_PARAMETER);
    }

    checkErr(rsspecan_InitWithOptions (resourceName, IDQuery, resetDevice, "", newInstrSession));

Error:
    return error;
}

/// HIFN This function creates a new RS session and calls the
/// HIFN RsInit function.
/// HIRET Returns the status code of this operation.
/// HIPAR resourceName/Pass the resource name of the device to initialize.
/// HIPAR IDQuery/Specify whether you want the instrument driver to perform an ID
/// HIPAR IDQuery/Query
/// HIPAR resetDevice/Specify whether you want the to reset the instrument during the
/// HIPAR resetDevice/initialization procedure.
/// HIPAR optionString/You can use this control to set the initial value of certain
/// HIPAR optionString/attributes for the session.  The following table lists the
/// HIPAR optionString/attributes and the name you use in this parameter to identify
/// HIPAR optionString/the attribute.
/// HIPAR newInstrSession/Returns a ViSession handle that you use to identify the
/// HIPAR newInstrSession/instrument in all subsequent instrument driver function calls.
/// HIPAR newInstrSession/ViStatus _VI_FUNC rsspecan_InitWithOptions(
ViStatus _VI_FUNC rsspecan_InitWithOptions(ViRsrc resourceName, ViBoolean IDQuery,
                                           ViBoolean resetDevice,
                                           ViString optionString,
                                           ViPSession newInstrSession)
{
    ViStatus error = VI_SUCCESS;
    ViSession instrSession = *newInstrSession;
    RsCoreSessionPtr sessionProperties = NULL;

    if (newInstrSession == NULL)
    {
        (void)RsCore_SetErrorInfo(0, VI_FALSE, RS_ERROR_INVALID_PARAMETER, VI_ERROR_PARAMETER5,
                                  "Null address for Instrument Handle");
        checkErr(RS_ERROR_INVALID_PARAMETER);
    }

    *newInstrSession = 0;

    checkErr(RsCore_NewSpecificDriver(
        resourceName, // VISA Resource Name
        "rsspecan", // driver prefix
        optionString, // Init Options string, is applied to the session settings
        rsspecan_RsCoreAttributeList, // List of all attributes from the rsxxx_AttrPropertiesList
        rsspecan_RsCoreAttributeListLength, // Length of the attributes list
        VI_FALSE, //idnModelFullName Model has the full *IDN? query name e.g. true: "CMW100A", VI_FALSE: "CMW" Used in CheckInstrument()
        0, // WriteDelay
        0, // ReadDelay
        1000000, // IO Segment Size
        RS_VAL_OPCWAIT_STBPOLLING, // OPC Wait Mode
        RSSPECAN_OPC_TIMEOUT, // OPC timeout
        15000, // VISA Timeout
        600000, // Self-test timeout
        RS_VAL_BIN_FLOAT_FORMAT_SINGLE_4BYTES, // BinaryFloatNumbersFormat
        RS_VAL_BIN_INTEGER_FORMAT_INT32_4BYTES, // binaryIntegerNumbersFormat
        &instrSession));

    // Minimal allowed rscore version is 4.4.0
    checkErr(RsCore_CheckMinimalEngineVersion(instrSession, 4, 4, 0));

    checkErr(RsCore_GetRsSession(instrSession, &sessionProperties));

    // No SCPI command has been sent yet.
    // Call viClear before sending any SCPI command
    checkErr(RsCore_ViClear(instrSession));

    /* --- Here perform settings that are default for this driver,
    but can be overwritten by the optionsString settings */

    sessionProperties->autoSystErrQuery = VI_FALSE;

    // Parse option string and optionally sets the initial state of the following session attributes
    checkErr(RsCore_ApplyOptionString(instrSession, optionString));
    checkErr(RsCore_BuildRepCapTable(instrSession, rsspecan_RsCoreRepCapTable, rsspecan_RsCoreRepCapTableCount));

    // Query *IDN? string, parse it and set the following attributes:
    // -RS_ATTR_INSTRUMENT_MANUFACTURER
    // - RS_ATTR_INSTRUMENT_MODEL - based on the idnModelFullName true: "FSW26", false : "FSW"
    // - RS_ATTR_INSTRUMENT_FIRMWARE_REVISION
    checkErr(RsCore_QueryAndParseIDNstringWithAliases(instrSession, RSSPECAN_SIMULATION_ID_QUERY, NULL,
        "FSV3%3[0-9] -> FSW, FSVA3%3[0-9] -> FSW, FSMR3%3[0-9] -> FSW, FSPN -> FSWP", NULL));

    // Query *OPT? string, parse the options, remove the duplicates, sort them and store the result string to RS_ATTR_OPTIONS_LIST
    checkErr(RsCore_QueryAndParseOPTstring(instrSession, RSSPECAN_SIMULATION_OPT_QUERY, RS_VAL_OPTIONS_PARSING_AUTO));

    // Default Instrument Setup + optional *RST
    if (resetDevice == VI_TRUE)
    {
        checkErr(rsspecan_reset(instrSession));
    }
    else
    {
        checkErr(rsspecan_DefaultInstrSetup(instrSession));
    }

    if (IDQuery == VI_TRUE)
    {
        checkErr(RsCore_FitsIDNpattern(instrSession, RSSPECAN_VALID_ID_RESPONSE_STRING, NULL));
    }

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_AfterErrorInInit(instrSession, sessionProperties, error, newInstrSession);

    return error;
}

/*===========================================================================*
 *= RsSpecAn Base Capabilities Group Functions =============================*
 *===========================================================================*/
/// HIFN This function selects the spectrum analyzer mode.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
ViStatus _VI_FUNC rsspecan_SetSANMode(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_SAN_MODE, ""));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN This function configures the acquisition attributes of the
/// HIFN spectrum analyzer. Single sweep, continuous sweep and sweep
/// HIFN count can be set.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR sweepModeContinuous/Turns on/off continuous sweeping.
/// HIPAR numberOfSweeps/The number of sweeps to take.
ViStatus _VI_FUNC rsspecan_ConfigureAcquisition(ViSession instrSession,
                                                ViInt32   reserved,
                                                ViBoolean sweepModeContinuous,
                                                ViInt32   numberOfSweeps)
{
    ViStatus error = VI_SUCCESS;
             reserved;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_SWEEP_MODE_CONTINUOUS, sweepModeContinuous),
            3, "Sweep Mode Continuous");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_NUMBER_OF_SWEEPS, numberOfSweeps),
            4, "Number Of Sweeps");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN Defines the sweep time
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.
/// HIPAR sweepTime/Defines the sweep time
ViStatus _VI_FUNC rsspecan_ConfigureSweepTime(ViSession instrSession,
                                              ViReal64  sweepTime)
{
    return rsspecan_ConfigureSweepCoupling(instrSession, 0, RSSPECAN_VAL_COUPLING_SWEEP_TIME, sweepTime);
}

/// HIFN This function turns on/off continuous sweeping.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.
/// HIPAR sweepModeContinuous/Document parameter here.
ViStatus _VI_FUNC rsspecan_ConfigureSweepMode(ViSession instrSession,
                                              ViBoolean sweepModeContinuous)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_SWEEP_MODE_CONTINUOUS, sweepModeContinuous),
            2, "Sweep Mode Continuous");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/********************************************************************
 *          Channel                                                 *
 ********************************************************************/
/// HIFN This function adds an additional measurement channel.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.
/// HIPAR channelType/This control selects the channel type of the new channel.
/// HIPAR channelName/String containing the name of the channel.
ViStatus _VI_FUNC rsspecan_AddMeasurementChannel(ViSession instrSession,
                                                 ViInt32   channelType,
                                                 ViString  channelName)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViChar   *p2string;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &rsspecan_rngChannelType, channelType, &p2string),
            2, "Channel Type");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "INST:CRE %s,'%s'", p2string, channelName);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function selects an existing measurement channel with the specified name.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.
/// HIPAR channelName/String containing the name of the channel.
ViStatus _VI_FUNC rsspecan_SelectMeasurementChannel(ViSession instrSession,
                                                    ViString  channelName)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_SELECT_MEASUREMENT_CHANNEL, channelName),
            3, "Channel Name");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function duplicates the currently selected measurement channel.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsspecan_DuplicateMeasurementChannel(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsspecan_AssertNotFSV(instrSession));

    checkErr(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_DUPLICATE_MEASUREMENT_CHANNEL, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function replaces a measurement channel with another one.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR oldChannelName/String containing the name of the channel you want to replace.
/// HIPAR newChannelType/This control selects the channel type of the new channel.
/// HIPAR newChannelName/String containing the name of the new channel.
ViStatus _VI_FUNC rsspecan_ReplaceMeasurementChannel(ViSession instrSession,
                                                     ViString  oldChannelName,
                                                     ViInt32   newChannelType,
                                                     ViString  newChannelName)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViChar   *p2string;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsspecan_AssertNotFSV(instrSession));

    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &rsspecan_rngChannelType, newChannelType, &p2string),
            3, "New Channel Type");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "INST:CRE:REPL '%s',%s,'%s'", oldChannelName, p2string, newChannelName);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function renames a measurement channel.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR oldChannelName/String containing the name of the channel you want to rename.
/// HIPAR newChannelName/String containing the new channel name.
ViStatus _VI_FUNC rsspecan_RenameMeasurementChannel(ViSession instrSession,
                                                    ViString  oldChannelName,
                                                    ViString  newChannelName)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsspecan_AssertNotFSV(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "INST:REN '%s','%s'", oldChannelName, newChannelName);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function deletes a measurement channel.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channelName/String containing the name of the channel you want to delete.
ViStatus _VI_FUNC rsspecan_DeleteMeasurementChannel(ViSession instrSession,
                                                    ViString  channelName)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_DELETE_MEASUREMENT_CHANNEL, channelName),
            3, "Channel Name");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function restores the default instrument settings in the current channel.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsspecan_PresetMeasurementChannel(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsspecan_AssertNotFSV(instrSession));

    checkErr(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_PRESET_MEASUREMENT_CHANNEL, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function queries all active measurement channels.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.
/// HIPAR arraySize/This parameter determines the size of array Channel Type and Channel Name
/// HIPAR channelType/This control returns the channel types of all channels.
/// HIPAR channelName/This control returns strings containing names of channels.
/// HIPAR returnedValues/Returns the number of returned values.
ViStatus _VI_FUNC rsspecan_QueryAllMeasurementChannels(ViSession instrSession,
                                                       ViInt32   arraySize,
                                                       ViInt32   channelType[],
                                                       ViChar    channelName[],
                                                       ViInt32   *returnedValues)
{
    ViStatus error = VI_SUCCESS;
    ViChar   *buffer = NULL;
    ViChar   *pstring_value;
    ViInt32  cnt = 0;
    ViChar   *p2buf;
    ViInt32  discrete;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsspecan_AssertNotFSV(instrSession));

    checkErr(RsCore_QueryViStringUnknownLength(instrSession, "INST:LIST?", &buffer));

    p2buf = channelName;
    pstring_value = strtok(buffer, ",");
    while (pstring_value && cnt < arraySize)
    {
        (void)sscanf(pstring_value, "'%[^']'", pstring_value);

        checkErr(RsCore_FindRangeTableDiscreteNumericValue(instrSession, &rsspecan_rngChannelType, pstring_value, &discrete));
        channelType[cnt] = discrete;
        pstring_value = strtok(NULL, ",");

        (void)sscanf(pstring_value, "'%[^']'", pstring_value);
        p2buf += sprintf(p2buf, "%s,", pstring_value);

        pstring_value = strtok(NULL, ",");
        cnt++;
    }
    *--p2buf = '\0';

    if (returnedValues != NULL)
        *returnedValues = cnt;

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    if (buffer) free(buffer);
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/*****************************************************************************
 *------------------- Common Measurement Settings ---------------------------*
 *****************************************************************************/

/*****************************************************************************
 *------------------- Input -------------------------------------------------*
 *****************************************************************************/
/// HIFN This functionresets the attenuator and reconnects the RF input with the input mixer
/// HIFN after an overload condition occured and the protection mechanism intervened.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
ViStatus _VI_FUNC rsspecan_InputProtectionReset(ViSession instrSession,
                                                ViInt32   reserved)
{
    ViStatus error = VI_SUCCESS;
             reserved;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_AMPL_PROTECTION_RESET, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function activates input from the RF INPUT connector.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR source/Selects the signal source for measurements
ViStatus _VI_FUNC rsspecan_ConfigureRFInputState(ViSession instrSession,
                                                 ViInt32   source)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_RF_INPUT, source),
            2, "Source");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function configures radio frequency input.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR inputCoupling/Selects the coupling type of the RF input.
/// HIPAR impedance/Selects the nominal input impedance of the RF input.
ViStatus _VI_FUNC rsspecan_ConfigureRFInput(ViSession instrSession,
                                            ViInt32   inputCoupling,
                                            ViReal64  impedance)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_AMPL_RF_COUPLING, inputCoupling),
            2, "Input Coupling");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "",
            RSSPECAN_ATTR_INPUT_IMPEDANCE, (ViInt32) impedance), 3, "Impedance");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function configures radio frequency input with user defined
/// HIFN impedance.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR inputCoupling/Selects the coupling type of the RF input.
/// HIPAR impedance/Selects the nominal input impedance of the RF input.
/// HIPAR padType/Defines the type of matching pad used for impedance instrSession.
ViStatus _VI_FUNC rsspecan_ConfigureRFInputWithUserImpedance(ViSession instrSession,
                                                             ViInt32   inputCoupling,
                                                             ViReal64  impedance,
                                                             ViInt32   padType)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_AMPL_RF_COUPLING, inputCoupling),
            2, "Input Coupling");

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_INPUT_IMPEDANCE_USER, impedance),
            3, "Impedance");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_INPUT_IMPEDANCE_PAD_TYPE, padType),
            4, "Pad Type");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects the input path.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR inputType/Selects the input path.
ViStatus _VI_FUNC rsspecan_ConfigureRFInputType(ViSession instrSession,
                                                ViInt32   inputType)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_INPUT_TYPE, inputType),
            2, "Input Type");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Determines which connector the input for the measurement is taken
/// HIFN  from.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR inputConnector/Determines which connector the input for the measurement is taken
/// HIPAR inputConnector/from.
ViStatus _VI_FUNC rsspecan_ConfigureInputConnector(ViSession instrSession,
                                                   ViInt32   inputConnector)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, NULL, RSSPECAN_ATTR_INPUT_CONNECTOR, inputConnector),
            3, "Input Connector");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function enables or disables the use of the direct path for
/// HIFN  frequencies close to 0 Hz.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR directPath/Enables or disables the use of the direct path for frequencies close
/// HIPAR directPath/to 0 Hz.
ViStatus _VI_FUNC rsspecan_ConfigureDirectPath(ViSession instrSession,
                                               ViInt32   directPath)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_DIRECT_PATH, directPath),
            2, "Direct Path");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function turns the highpass filter at the RF input on and off.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init.
/// HIPAR highpassFilter/Turns the highpass filter at the RF input on and off.
ViStatus _VI_FUNC rsspecan_ConfigureHighpassFilter(ViSession instrSession,
                                                   ViBoolean highpassFilter)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_AMPL_HIGHPASS_FILTER, highpassFilter),
            2, "Highpass Filter");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function switches the YIG filter for image rejection into
/// HIFN the signal path (ON) or removes it from the signal path (OFF).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init.
/// HIPAR YIGFilter/Switches the YIG filter for image rejection into the signal path
/// HIPAR YIGFilter/(ON) or removes it from the signal path (OFF).
ViStatus _VI_FUNC rsspecan_ConfigureYIGFilter(ViSession instrSession,
                                              ViInt32   reserved,
                                              ViBoolean YIGFilter)
{
    ViStatus error = VI_SUCCESS;
             reserved;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_AMPL_YIG_FILTER, YIGFilter),
            3, "YIG Filter");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Activates or deactivates the preselector adjustment. This control is
/// HIFN  only available for instrument models R&S FSW43/50/67, for frequency
/// HIFN  sweeps in the Spectrum application. If activated, the R&S FSW
/// HIFN  automatically performs a short internal adjustment. If you change the
/// HIFN  frequency or span settings, the adjustment is repeated.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR preselectorAdjustment/Activates or deactivates the preselector adjustment.
ViStatus _VI_FUNC rsspecan_ConfigurePreselectorAdjustment(ViSession instrSession,
                                                          ViBoolean preselectorAdjustment)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_PRESELECTOR_ADJUSTMENT, preselectorAdjustment),
            2, "Preselector Adjustment");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/*****************************************************************************
 *------------------- Input -> Probe  ---------------------------------------*
 *****************************************************************************/
/// HIFN This function defines which action is taken when the probe's
/// HIFN micro button is pressed.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR input/Selects the connector
/// HIPAR microButtonAction/This control selects which action is taken when the probe's
/// HIPAR microButtonAction/micro button is pressed.
ViStatus _VI_FUNC rsspecan_ProbesMicroButtonAction(ViSession instrSession,
                                                   ViInt32   input,
                                                   ViInt32   microButtonAction)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_PROBE_MICRO_BUTTON_ACTION,
            "Probe", input - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Input");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_PROBE_MICRO_BUTTON_ACTION, microButtonAction),
            3, "microButtonAction");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function returns the probe informations.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR probeConnector/Selects the connector:
/// HIPAR bufferSize/Pass the number of bytes in the string you specify for the catalog list parameter.
/// HIPAR state/This control shows whether a probe is connected to the instrument's RF INPUT
/// HIPAR state/and USB connectors and was recognized by the R&S FSV
/// HIPAR name/This control returns the name of the connected probe.
/// HIPAR serialNumber/This control returns the serial number of the connected probe.
/// HIPAR partNumber/This control returns the material part number of the connected probe.
/// HIPAR probeType/This control returns the probe type.
ViStatus _VI_FUNC rsspecan_GetProbesInformation(ViSession instrSession,
                                                ViInt32   probeConnector,
                                                ViInt32   bufferSize,
                                                ViInt32   *state,
                                                ViChar    name[],
                                                ViChar    serialNumber[],
                                                ViChar    partNumber[],
                                                ViChar    probeType[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, probeConnector, 1, 3),
            2, "Probe Connector");

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_PROBE_MICRO_BUTTON_ACTION,
            "Probe", probeConnector - 1, RS_REPCAP_BUF_SIZE, repCap));

    viCheckParm(rsspecan_GetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_PROBE_CONNECTION_STATE, state),
            3, "State");

    if (*state == 1)
    {
        viCheckParm(rsspecan_GetAttributeViString(instrSession, repCap, RSSPECAN_ATTR_PROBE_NAME,
                bufferSize, name),
                4, "Name");

        viCheckParm(rsspecan_GetAttributeViString(instrSession, repCap, RSSPECAN_ATTR_PROBE_SERIAL_NUMBER,
                bufferSize, serialNumber),
                5, "Serial Number");

        viCheckParm(rsspecan_GetAttributeViString(instrSession, repCap, RSSPECAN_ATTR_PROBE_PART_NUMBER,
                bufferSize, partNumber),
                6, "Part Number");

        viCheckParm(rsspecan_GetAttributeViString(instrSession, repCap, RSSPECAN_ATTR_PROBE_TYPE,
                bufferSize, probeType),
                7, "Probe Type");
    }
    else
    {
        snprintf(name, bufferSize, "Not detected");
        snprintf(serialNumber, bufferSize, "Not detected");
        snprintf(partNumber, bufferSize, "Not detected");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configure the mode of a multi-mode modular probe and offset.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR probeConnector/Selects the connector.
/// HIPAR probeMode/Determines the mode of a multi-mode modular probe.
/// HIPAR offset/Sets the offset value according to selected Probe Mode.
ViStatus _VI_FUNC rsspecan_ConfigureProbeMode(ViSession instrSession,
                                              ViInt32   probeConnector,
                                              ViInt32   probeMode,
                                              ViReal64  offset)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViAttr   offset_arr[4] = {
             RSSPECAN_ATTR_PROBE_DM_MODE_OFFSET,
             RSSPECAN_ATTR_BB_PROBE_COMMON_MODE_OFFSET,
             RSSPECAN_ATTR_PROBE_P_MODE_OFFSET,
             RSSPECAN_ATTR_PROBE_N_MODE_OFFSET
             };

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_PROBE_MODE,
            "Probe", probeConnector - 1, RS_REPCAP_BUF_SIZE, repCap));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_PROBE_MODE, probeMode),
            3, "Probe Mode");

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, repCap, offset_arr[probeMode], offset),
            4, "Offset");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/*****************************************************************************
 *------------------- Output ------------------------ -----------------------*
 *****************************************************************************/
/// HIFN This function defines the type of signal sent to the
/// HIFN IF/VIDEO/DEMOD connector on the rear panel of the instrument.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR outputSource/This control switches the source of the IF output.
/// HIPAR outputFrequency/This control sets the frequency for the IF output.
ViStatus _VI_FUNC rsspecan_ConfigureIFOutput(ViSession instrSession,
                                             ViInt32   outputSource,
                                             ViReal64  outputFrequency)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_IF_OUTPUT_SOURCE, outputSource),
            2, "Output Source");
    if (outputSource == RSSPECAN_VAL_IF_OUTPUT_SOURCE_IF)
    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_IF_OUTPUT_FREQUENCY, outputFrequency),
            3, "Output Frequency");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function queries the sideband provided at the IF OUT
/// HIFN connector compared to the sideband of the RF signal. The
/// HIFN sideband depends on the current center frequency.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR sideband/Returns the sideband type.
ViStatus _VI_FUNC rsspecan_QueryIFOutputSideband(ViSession instrSession,
                                                 ViInt32   *sideband)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_GetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_IF_OUTPUT_SIDEBAND, sideband),
            2, "Sideband");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function turns the 28 V supply of the noise source on the front panel on and off.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR noiseSource/Turns the 28 V supply of the noise source on the front panel on and off.
ViStatus _VI_FUNC rsspecan_ConfigureNoiseSource(ViSession instrSession,
                                                ViInt32   reserved,
                                                ViBoolean noiseSource)
{
    ViStatus error = VI_SUCCESS;
             reserved;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_NOISE_SOURCE_STATE, noiseSource),
            3, "Noise Source");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function configures the trigger direction as input.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR port/Selects the trigger port to which the output is sent.
ViStatus _VI_FUNC rsspecan_ConfigureTriggerInput(ViSession instrSession,
                                                 ViInt32   port)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, port, 2, 3),
            2, "Port");

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_TRIGGER_DIRECTION,
            "PortNumber", port - 1, RS_REPCAP_BUF_SIZE, repCap));

    checkErr(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_TRIGGER_DIRECTION, RSSPECAN_VAL_TRIG_DIRECTION_INP));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function selects the trigger output type, length and if
/// HIFN user defined output type is selected, also a level.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR port/Selects the trigger port to which the output is sent.
/// HIPAR outputType/Selects the type of signal generated at the trigger output.
/// HIPAR level/Selects signal level.
/// HIPAR length/Pulse length in seconds.
ViStatus _VI_FUNC rsspecan_ConfigureTriggerOutput(ViSession instrSession,
                                                  ViInt32   port,
                                                  ViInt32   outputType,
                                                  ViInt32   level,
                                                  ViReal64  length)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, port, 2, 3),
            2, "Port");

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_TRIGGER_DIRECTION,
            "PortNumber", port - 1, RS_REPCAP_BUF_SIZE, repCap));
    checkErr(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_TRIGGER_DIRECTION, RSSPECAN_VAL_TRIG_DIRECTION_OUTP));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_TRIGGER_OUTPUT_TYPE, outputType),
            3, "Output Type");

    if (outputType == RSSPECAN_VAL_TRIG_OTYPE_UDEFINED)
    {
        viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_TRIGGER_LEVEL, level),
                4, "Level");
    }

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_TRIGGER_PULSE_LENGTH, length),
            5, "Length");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function generates a pulse at the trigger output.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.
/// HIPAR port/Selects the trigger port to which the output is sent.
ViStatus _VI_FUNC rsspecan_GenerateTriggerPulse(ViSession instrSession,
                                                ViInt32   port)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, port, 2, 3),
            2, "Port");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "Port%ld", port);
    checkErr(rsspecan_SetAttributeViString(instrSession, repCap, RSSPECAN_ATTR_TRIGGER_PULSE_GENERATE, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/*****************************************************************************
 *------------------- User Port ---------------------*
 *****************************************************************************/
/// HIFN This function toggles the control lines of the user ports between
/// HIFN INPut and OUTPut.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.
/// HIPAR state/Toggles the control lines of the user ports between INPut and
/// HIPAR state/OUTPut.
ViStatus _VI_FUNC rsspecan_ConfigureControlLinesState(ViSession instrSession,
                                                      ViInt32   reserved,
                                                      ViBoolean state)
{
    ViStatus error = VI_SUCCESS;
             reserved;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_INP_UPORT_STATE, state),
            2, "State");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function sets the control lines of the user ports.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.
/// HIPAR userPorts/This control sets the control lines of the user ports.
ViStatus _VI_FUNC rsspecan_ConfigureOutputControlLines(ViSession instrSession,
                                                       ViInt32   reserved,
                                                       ViString  userPorts)
{
    ViStatus error = VI_SUCCESS;
             reserved;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_OUT_UPORT_VALUE, userPorts),
            2, "User Ports");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function toggles the control lines of the user ports for the AUX
/// HIFN  PORT connector.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSSPECAN_ATTR_USER_PORTS_OUTPUT_STATE
/// HIFN
/// HIFN     Note:
/// HIFN
/// HIFN     (1) This 9-pole SUB-D male connector is located on the rear panel of
/// HIFN  the R&S FSWP.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR state/This control toggles the control lines of the user ports for the AUX
/// HIPAR state/PORT connector.
ViStatus _VI_FUNC rsspecan_ConfigureAUXPortControlLinesState(ViSession instrSession,
                                                             ViBoolean state)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_USER_PORTS_OUTPUT_STATE, state),
            2, "State");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function sets the control lines of the user ports.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.
/// HIPAR lines/This control selects the control lines to be returned.
/// HIPAR userPorts/Returns the control lines of the user ports.
ViStatus _VI_FUNC rsspecan_GetControlLines(ViSession instrSession,
                                           ViInt32   reserved,
                                           ViInt32   lines,
                                           ViChar    userPorts[])
{
    ViStatus error = VI_SUCCESS;
             reserved;

    checkErr(RsCore_LockSession(instrSession));

    switch (lines)
    {
    case RSSPECAN_VAL_USER_PORT_INP:
        viCheckParm(rsspecan_GetAttributeViString(instrSession, "", RSSPECAN_ATTR_INP_UPORT_VALUE,
                3, userPorts),
                3, "User Ports");
        break;
    case RSSPECAN_VAL_USER_PORT_OUT:
        viCheckParm(rsspecan_GetAttributeViString(instrSession, "", RSSPECAN_ATTR_OUT_UPORT_VALUE,
                3, userPorts),
                3, "User Ports");
        break;
    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, lines), 2, "Lines");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/*****************************************************************************
 *------------------- Frequency adn Span ------------------------------------*
 *****************************************************************************/

/// HIFN This function configures the start/stop frequency of the spectrum
/// HIFN analyzer
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR startFrequency/The start frequency of the frequency sweep.
/// HIPAR stopFrequency/The stop frequency of the frequency sweep.
ViStatus _VI_FUNC rsspecan_ConfigureFrequencyStartStop(ViSession instrSession,
                                                       ViInt32   reserved,
                                                       ViReal64  startFrequency,
                                                       ViReal64  stopFrequency)
{
    ViStatus error = VI_SUCCESS;
             reserved;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_FREQUENCY_START, startFrequency),
            3, "Start Frequency");

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_FREQUENCY_STOP, stopFrequency),
            4, "Stop Frequency");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Switches the labelling of the y-axis for frequency-based result
/// HIFN  diagrams. The frequency range itself is not changed.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR mode/Switches the labelling of the y-axis for frequency-based result
/// HIPAR mode/diagrams. The frequency range itself is not changed.
ViStatus _VI_FUNC rsspecan_ConfigureFrequencyAnnotationMode(ViSession instrSession,
                                                            ViInt32   mode)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, NULL, RSSPECAN_ATTR_FREQUENCY_ANNOTATION_MODE, mode),
            2, "Mode");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function configures the center frequency of the spectrum
/// HIFN analyzer
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR centerFrequency/The center frequency of the frequency sweep.
ViStatus _VI_FUNC rsspecan_ConfigureFrequencyCenterSpan(ViSession instrSession,
                                                        ViInt32   reserved,
                                                        ViReal64  centerFrequency,
                                                        ViReal64  span)
{
    ViStatus error = VI_SUCCESS;
             reserved;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_FREQUENCY_CENTER, centerFrequency),
            3, "Center Frequency");

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_FREQUENCY_SPAN, span),
            4, "Span");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function configures the center frequency the spectrum analyzer
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR centerFrequency/Center Frequency
ViStatus _VI_FUNC rsspecan_ConfigureFrequencyCenter(ViSession instrSession,
                                                    ViInt32   reserved,
                                                    ViReal64  centerFrequency)
{
    ViStatus error = VI_SUCCESS;
             reserved;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_FREQUENCY_CENTER, centerFrequency),
            3, "Center Frequency");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function defines the center frequency step size.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR stepSize/Sets the step width of the center frequency.
ViStatus _VI_FUNC rsspecan_ConfigureFrequencyStepSize(ViSession instrSession,
                                                      ViInt32   reserved,
                                                      ViReal64  stepSize)
{
    ViStatus error = VI_SUCCESS;
             reserved;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_FREQUENCY_CENTER_LINK, RSSPECAN_VAL_CENT_FREQ_LINK_OFF));

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_FREQUENCY_CENTER_STEP, stepSize),
            3, "Step Size");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN This function couples or decouples the center frequency step
/// HIFN size to the span (On).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR stepSizeAuto/Couples the step size of the center frequency to the span (ON)
/// HIPAR stepSizeAuto/or sets the value of the center frequency entered via
/// HIPAR stepSizeAuto/SENSe:]FREQuency:CENTer:STEP (OFF).
ViStatus _VI_FUNC rsspecan_ConfigureFrequencyStepSizeAuto(ViSession instrSession,
                                                          ViInt32   reserved,
                                                          ViBoolean stepSizeAuto)
{
    ViStatus error = VI_SUCCESS;
             reserved;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_FREQUENCY_STEP_AUTO, stepSizeAuto),
            3, "Step Size Auto");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN Document your function here.
/// HIFN You may use multiple lines for documentation.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR horizontalScale/Toggles between linear and logarithmic display of the x-axis.
ViStatus _VI_FUNC rsspecan_ConfigureHorizontalScale(ViSession instrSession,
                                                    ViInt32   reserved,
                                                    ViInt32   horizontalScale)
{
    ViStatus error = VI_SUCCESS;
             reserved;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_HORIZONTAL_SCALE, horizontalScale),
            3, "Horizontal Scale");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function sets the frequency span in the analyzer mode to
/// HIFN its maximum.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
ViStatus _VI_FUNC rsspecan_ConfigureFrequencySpanFull(ViSession instrSession,
                                                      ViInt32   reserved)
{
    ViStatus error = VI_SUCCESS;
             reserved;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_FREQUENCY_SPAN_FULL, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function sets the span to 0 Hz. The x-axis becomes the time
/// HIFN axis with the grid lines corresponding to 1/10 of the current
/// HIFN sweep time.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
ViStatus _VI_FUNC rsspecan_ConfigureFrequencyZeroSpan(ViSession instrSession,
                                                      ViInt32   reserved)
{
    ViStatus error = VI_SUCCESS;
             reserved;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_FREQUENCY_SPAN,
                                               0.0));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function configures the frequency offset attribute of the
/// HIFN spectrum analyzer.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR frequencyOffset/Frequency offset.
ViStatus _VI_FUNC rsspecan_ConfigureFrequencyOffset(ViSession instrSession,
                                                    ViInt32   reserved,
                                                    ViReal64  frequencyOffset)
{
    ViStatus error = VI_SUCCESS;
             reserved;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_FREQUENCY_OFFSET, frequencyOffset),
            3, "Frequency Offset");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function couples the step width of the center frequency to
/// HIFN span (span >0) or to the resolution bandwidth (span = 0) or
/// HIFN cancels the couplings and sets the coupling factor.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR coupling/Couples and decouples the center frequency step size to the span
/// HIPAR coupling/or the resolution bandwidth.
/// HIPAR couplingFactor/Couples the step width of the center frequency with a factor to the span (span >0)
/// HIPAR couplingFactor/or to the resolution bandwidth (span = 0).
ViStatus _VI_FUNC rsspecan_ConfigureFrequencyCouplingFactor(ViSession instrSession,
                                                            ViInt32   reserved,
                                                            ViInt32   coupling,
                                                            ViInt32   couplingFactor)
{
    ViStatus error = VI_SUCCESS;
             reserved;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_FREQUENCY_CENTER_LINK, coupling),
            3, "Coupling");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_FREQUENCY_CENTER_LINK_FACTOR, couplingFactor),
            4, "Coupling Factor");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function configures signal-tracking.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR marker/This control selects the marker.
/// HIPAR signalTrackEnabled/Switches the signal-track function on or off for the selected
/// HIPAR signalTrackEnabled/measurement window.
/// HIPAR signalTrackBandwidth/Defines the bandwidth around the center frequency within which
/// HIPAR signalTrackBandwidth/the largest signal is searched.
/// HIPAR signalTrackThreshold/Defines the threshold above which the largest signal is searched
/// HIPAR signalTrackThreshold/for in the selected measurement window.
/// HIPAR trace/Defines the trace on which the largest signal is searched for in
/// HIPAR trace/the selected measurement window.
ViStatus _VI_FUNC rsspecan_ConfSignalTrack(ViSession instrSession,
                                           ViInt32   marker,
                                           ViBoolean signalTrackEnabled,
                                           ViReal64  signalTrackBandwidth,
                                           ViReal64  signalTrackThreshold,
                                           ViInt32   trace)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_SIGNAL_TRACK_ENABLED,
            "Marker", marker - 1, RS_REPCAP_BUF_SIZE, repCap));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, repCap, RSSPECAN_ATTR_SIGNAL_TRACK_ENABLED, signalTrackEnabled),
            3, "Signal Track Enabled");
    if (signalTrackEnabled == VI_TRUE)
    {
        viCheckParm(rsspecan_SetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_SIGNAL_TRACK_BWID, signalTrackBandwidth),
                4, "Signal Track Bandwidth");

        viCheckParm(rsspecan_SetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_SIGNAL_TRACK_THRESHOLD, signalTrackThreshold),
                5, "Signal Track Threshold");

        viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_SIGNAL_TRACK_TRACE, trace),
                6, "Trace");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function switches between frequency domain (SWEep) and time
/// HIFN domain (CW | FIXed) in the analyzer mode. For CW and FIXed, the
/// HIFN frequency setting is via command FREQuency:CENTer. In the SWEep
/// HIFN mode, the setting is via commands FREQuency:STARt, STOP, CENTer
/// HIFN and SPAN..
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR frequencyMode/Frequency Mode
ViStatus _VI_FUNC rsspecan_ConfigureFrequencyMode(ViSession instrSession,
                                                  ViInt32   reserved,
                                                  ViInt32   frequencyMode)
{
    ViStatus error = VI_SUCCESS;
             reserved;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsspecan_AssertFSV(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_FREQUENCY_MODE, frequencyMode),
            3, "Frequency Mode");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the current signal level setting of the measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR levelSetting/Queries the current signal level setting of the measurement.
ViStatus _VI_FUNC rsspecan_QueryFrequencyLevelSettingUsed(ViSession instrSession,
                                                          ViInt32*  levelSetting)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_GetAttributeViInt32(instrSession, NULL, RSSPECAN_ATTR_FREQUENCY_LEVEL_SETTING_USED, levelSetting),
            2, "Level Setting");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function selects the level measurement mode.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR frequencyMode/This control selects the level measurement mode.
ViStatus _VI_FUNC rsspecan_ConfigureFrequencyLevel(ViSession instrSession,
                                                   ViInt32   frequencyMode)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_FREQUENCY_LEVEL_SETTING, frequencyMode),
            2, "Frequency Mode");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects the scaling of the x-axis.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR subwindow/This control selects the measurement subwindow.
/// HIPAR scale/Selects the scaling of the x-axis.
ViStatus _VI_FUNC rsspecan_ConfigureSubwindowHorizontalScale(ViSession instrSession,
                                                             ViInt32   window,
                                                             ViInt32   subwindow,
                                                             ViInt32   scale)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, window, 0, 16),
            2, "Window");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, subwindow, 1, 64),
            3, "Subwindow");

    snprintf (repCap, RS_REPCAP_BUF_SIZE, "Win%ld,SW%ld", window, subwindow);

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_SUBWINDOW_HORIZONTAL_SCALE, scale),
            4, "Scale");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/*****************************************************************************
 *------------------- Amplitude ---------------------------------------------*
 *****************************************************************************/
/// HIFN This function configures the vertical scale of analyzer.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR verticalScale/Pass the vertical scale that you want the spectrum analyzer to use.
ViStatus _VI_FUNC rsspecan_ConfigureVerticalScaling(ViSession instrSession,
                                                    ViInt32   verticalScale,
                                                    ViInt32   yAxisGridMode)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_VERTICAL_SCALE, verticalScale),
            3, "Vertical Scale");
    switch (verticalScale)
    {
    case RSSPECAN_VAL_TRACE_SPACE_LOG:
    case RSSPECAN_VAL_TRACE_SPACE_LDB:
        viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_DISP_AMPLITUDE_GRID_MODE, yAxisGridMode),
                4, "Y Axis Grid Mode");
        break;
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Switches on or off the display of the corresponding trace in the
/// HIFN  selected measurement window.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR subwindow/This control selects the measurement subwindow.
/// HIPAR verticalScale/Selects the scaling of the y-axis.
ViStatus _VI_FUNC rsspecan_ConfigureSubwindowVerticalScale(ViSession instrSession,
                                                           ViInt32   window,
                                                           ViInt32   subwindow,
                                                           ViInt32   verticalScale)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, window, 0, 16),
            2, "Window");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, subwindow, 1, 64),
            3, "Subwindow");

    snprintf (repCap, RS_REPCAP_BUF_SIZE, "Win%ld,SW%ld", window, subwindow);

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_SUBWINDOW_VERTICAL_SCALE, verticalScale),
            4, "Vertical Scale");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function defines the display range of the Y axis (level
/// HIFN axis) in the selected measurement window with logarithmic
/// HIFN scaling (DISP:TRAC:Y:SPAC LOG) and reference level position.
/// HIFN For linear scaling, (DISP:TRAC:Y:SPAC LIN | PERC) the display
/// HIFN range is fixed and cannot be modified.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR range/Defines the display range of the Y axis
/// HIPAR referencePosition/Defines the position of the reference value in the selected
/// HIPAR referencePosition/measurement window.
ViStatus _VI_FUNC rsspecan_ConfigureVerticalRange(ViSession instrSession,
                                                  ViReal64  range,
                                                  ViReal64  referencePosition)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_DISP_LOG_RANGE, range),
            3, "Range");

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_DISP_REF_POSITION, referencePosition),
            4, "Reference Value Position");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function adapts the diagram limits to the current measurement
/// HIFN results. The value range depends on the selected display.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
ViStatus _VI_FUNC rsspecan_ConfigureDisplayAutoscaleOnce(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_DISP_TRACE_Y_AUTO, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Turns automatic scaling of the y-axis on and off.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control is reserved.
/// HIPAR autoScalingEnabled/Turns automatic scaling of the y-axis on and off.
ViStatus _VI_FUNC rsspecan_ConfigureAutoScalingEnabled(ViSession instrSession,
                                                       ViInt32   window,
                                                       ViBoolean autoScalingEnabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_DISPLAY_AUTO_SCALING_ENABLED,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, repCap, RSSPECAN_ATTR_DISPLAY_AUTO_SCALING_ENABLED, autoScalingEnabled),
            3, "Display Auto Scaling Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN Defines the reference level.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR referenceLevel/Defines the reference level.
ViStatus _VI_FUNC rsspecan_ConfigureReferenceLevel(ViSession instrSession,
                                                   ViInt32   reserved,
                                                   ViReal64  referenceLevel)
{
    ViStatus error = VI_SUCCESS;
             reserved;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, NULL, RSSPECAN_ATTR_REFERENCE_LEVEL, referenceLevel),
            2, "Reference Level");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN Document your function here.
/// HIFN You may use multiple lines for documentation.
/// HIRET Document return value here.
/// HIRET You may use multiple lines for documentation.
/// HIPAR instrSession/Document parameter here.
/// HIPAR limit/Document parameter here.
ViStatus _VI_FUNC rsspecan_SetReferenceLevelToLimit(ViSession instrSession,
                                                    ViInt32   limit)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_REFERENCE_LEVEL_SET_TO_MIN_MAX, limit),
            2, "Limit");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN Document your function here.
/// HIFN You may use multiple lines for documentation.
/// HIRET Document return value here.
/// HIRET You may use multiple lines for documentation.
/// HIPAR instrSession/Document parameter here.
/// HIPAR minimum/Document parameter here.
/// HIPAR maximum/Document parameter here.
ViStatus _VI_FUNC rsspecan_GetReferenceLevelRange(ViSession instrSession,
                                                  ViReal64  *minimum,
                                                  ViReal64  *maximum)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_REFERENCE_LEVEL_MIN, minimum),
            2, "Minimum");

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_REFERENCE_LEVEL_MAX, maximum),
            3, "Maximum");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN Specifies the offset value to the reference level.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR referenceLevelOffset/Specifies the offset value to the reference level.
ViStatus _VI_FUNC rsspecan_ConfigureReferenceLevelOffset(ViSession instrSession,
                                                         ViInt32   reserved,
                                                         ViReal64  referenceLevelOffset)
{
    ViStatus error = VI_SUCCESS;
             reserved;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_REFERENCE_LEVEL_OFFSET, referenceLevelOffset),
            3, "Reference Level Offset");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN Specifies the amplitude units for input, output and display. Pass the units you want the spectrum analyzer to use.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR amplitudeUnits/Specifies the amplitude units for input, output and display. Pass the units you want the spectrum analyzer to use.
ViStatus _VI_FUNC rsspecan_ConfigureReferenceLevelUnits(ViSession instrSession,
                                                        ViInt32   amplitudeUnits)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_AMPLITUDE_UNITS, amplitudeUnits),
            2, "Amplitude Units");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function configures the input attenuation.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR attenuationAuto/Specifies whether the spectrum analyzer is enabled for auto
/// HIPAR attenuationAuto/attenuation.
/// HIPAR attenuation/Specifies the input attenuation (in positive dB).
ViStatus _VI_FUNC rsspecan_ConfigureAttenuation(ViSession instrSession,
                                                ViInt32   reserved,
                                                ViBoolean attenuationAuto,
                                                ViReal64  attenuation)
{
    ViStatus error = VI_SUCCESS;
             reserved;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_ATTENUATION_AUTO, attenuationAuto),
            3, "Attenuation Auto");
    if (attenuationAuto == VI_FALSE)
    {
        viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_ATTENUATION, attenuation),
                4, "Attenuation");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures attenuation at the analyzer input for an active external
/// HIFN  frontend.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR automatic/Enables or disables automatic configuration of attenuation at the
/// HIPAR automatic/analyzer input for an active external frontend.
/// HIPAR attenuation/Configures attenuation at the analyzer input for an active external
/// HIPAR attenuation/frontend manually.
ViStatus _VI_FUNC rsspecan_ConfigureAnalyzerAttenuation(ViSession instrSession,
                                                        ViBoolean automatic,
                                                        ViReal64  attenuation)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, NULL, RSSPECAN_ATTR_AMPLITUDE_ANALYZER_ATTENUATION_AUTO, automatic),
            2, "Automatic");

    if(automatic == VI_FALSE)
        viCheckParm(rsspecan_SetAttributeViReal64(instrSession, NULL, RSSPECAN_ATTR_AMPLITUDE_ANALYZER_ATTENUATION, attenuation),
                3, "Attenuation");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN This function configures electronic attenuator parameters.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR mode/This control selects the mode of electronic attenuator.
/// HIPAR manualValue/Programs the attenuation of the electronic input attenuator.
ViStatus _VI_FUNC rsspecan_ConfigureElectronicAttenuator(ViSession instrSession,
                                                         ViInt32   reserved,
                                                         ViInt32   mode,
                                                         ViInt32   manualValue)
{
    ViStatus error = VI_SUCCESS;
             reserved;

    checkErr(RsCore_LockSession(instrSession));

    switch (mode)
    {
    case RSSPECAN_VAL_EATT_OFF:
        viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_AMPL_EATT_STATE, VI_FALSE),
                3, "Mode");
        break;
    case RSSPECAN_VAL_EATT_AUTO:
        viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_AMPL_EATT_STATE, VI_TRUE),
                3, "Mode");

        viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_AMPL_EATT_AUTO, VI_TRUE),
                3, "Mode");
        break;
    case RSSPECAN_VAL_EATT_MAN:
        viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_AMPL_EATT_STATE, VI_TRUE),
                3, "Mode");

        viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_AMPL_EATT_AUTO, VI_FALSE),
                3, "Mode");

        viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_AMPL_EATT_VALUE, manualValue),
                4, "Manual Value");
        break;
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects the signal path for signal processing. If enabled, the
/// HIFN  narrowband signal path is used. The wideband signal path is used when
/// HIFN  disabled.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR signalPath/Selects the signal path for signal processing. If enabled, the
/// HIPAR signalPath/narrowband signal path is used. The wideband signal path is used when
/// HIPAR signalPath/disabled.
ViStatus _VI_FUNC rsspecan_ConfigureSignalPathEnabled(ViSession instrSession,
                                                      ViBoolean signalPath)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, NULL, RSSPECAN_ATTR_AMPLITUDE_SIGNAL_PATH_ENABLED, signalPath),
            2, "Signal Path");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function configures the preamplifier for the RF input
/// HIFN signal. The switchable gain is fixed to 20 dB.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR state/Switches on the preamplifier for the instrument.
/// HIPAR preamplifierLevel/Selects the preamplification level.
ViStatus _VI_FUNC rsspecan_ConfigurePreamplifier(ViSession instrSession,
                                                 ViBoolean state,
                                                 ViInt32   preamplifierLevel)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_AMPL_PREAMPLIFIER, state),
            2, "State");

    if (RsCore_IsInstrumentModel(instrSession, "FSW"))
    {
        if (state == VI_TRUE && !rsspecan_IsFSV(instrSession))
        {
            viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_AMPL_PREAMPLIFIER_LEVEL, preamplifierLevel),
                    3, "Preamplifier Level");
        }
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/*****************************************************************************
 *------------------- Sweep -------------------------------------------------*
 *****************************************************************************/

/// HIFN This function configures the coupling values of the spectrum
/// HIFN analyzer.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR sweepCoupling/Document parameter here.
/// HIPAR couplingValue/Document parameter here.
ViStatus _VI_FUNC rsspecan_ConfigureSweepCoupling(ViSession instrSession,
                                                  ViInt32   reserved,
                                                  ViInt32   sweepCoupling,
                                                  ViReal64  couplingValue)
{
    ViStatus error = VI_SUCCESS;
    ViAttr   attribute = 0;
             reserved;

    checkErr(RsCore_LockSession(instrSession));

    switch (sweepCoupling)
    {
    case RSSPECAN_VAL_COUPLING_RBW:
        attribute = RSSPECAN_ATTR_RESOLUTION_BANDWIDTH;
        break;

    case RSSPECAN_VAL_COUPLING_VBW:
        attribute = RSSPECAN_ATTR_VIDEO_BANDWIDTH;
        break;

    case RSSPECAN_VAL_COUPLING_SWEEP_TIME:
        attribute = RSSPECAN_ATTR_SWEEP_TIME;
        break;
    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, sweepCoupling),
                3, "Sweep Coupling");
    }

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", attribute, couplingValue),
            4, "Coupling Value");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function activates auto coupling of selected coupling
/// HIFN method.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR sweepCoupling/This control selects coupling type.
ViStatus _VI_FUNC rsspecan_ConfigureSweepCouplingAuto(ViSession instrSession,
                                                      ViInt32   reserved,
                                                      ViInt32   sweepCoupling)
{
    ViStatus error = VI_SUCCESS;
    ViAttr   attribute = 0;
             reserved;

    checkErr(RsCore_LockSession(instrSession));

    switch (sweepCoupling)
    {
    case RSSPECAN_VAL_COUPLING_RBW:
        attribute = RSSPECAN_ATTR_RESOLUTION_BANDWIDTH_AUTO;
        break;

    case RSSPECAN_VAL_COUPLING_VBW:
        attribute = RSSPECAN_ATTR_VIDEO_BANDWIDTH_AUTO;
        break;

    case RSSPECAN_VAL_COUPLING_SWEEP_TIME:
        attribute = RSSPECAN_ATTR_SWEEP_TIME_AUTO;
        break;
    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, sweepCoupling),
                3, "Sweep Coupling");
    }

    checkErr(rsspecan_SetAttributeViBoolean(instrSession, "", attribute, VI_TRUE));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function configures coupling ratio.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR ratioSelection/Document parameter here.
/// HIPAR value/Document parameter here.
ViStatus _VI_FUNC rsspecan_ConfigureSweepCouplingAdvanced(ViSession instrSession,
                                                          ViInt32   reserved,
                                                          ViInt32   ratioSelection,
                                                          ViReal64  value)
{
    ViStatus error = VI_SUCCESS;
    ViAttr   attribute = 0;
             reserved;

    checkErr(RsCore_LockSession(instrSession));

    switch (ratioSelection)
    {
    case RSSPECAN_VAL_COUPLING_RBW:
        attribute = RSSPECAN_ATTR_RESOLUTION_BANDWIDTH_RATIO;
        break;
    case RSSPECAN_VAL_COUPLING_VBW:
        attribute = RSSPECAN_ATTR_VIDEO_BANDWIDTH_RATIO;
        break;
    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, ratioSelection),
                3, "Ratio Selection");
    }

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", attribute, value),
            4, "Value");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function sets the filter type for the resolution bandwidth.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR filterType/Sets the filter type for the resolution bandwidth.
ViStatus _VI_FUNC rsspecan_ConfigureFilterType(ViSession instrSession,
                                               ViInt32   reserved,
                                               ViInt32   filterType)
{
    ViStatus error = VI_SUCCESS;
             reserved;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_RESOLUTION_BANDWIDTH_FILTER_TYPE, filterType),
            3, "Filter Type");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function selects the position of the video filter in the
/// HIFN signal path
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR videoFilterType/Set the position of the video filter in the signal path
ViStatus _VI_FUNC rsspecan_ConfigureVBWFilterType(ViSession instrSession,
                                                  ViInt32   reserved,
                                                  ViInt32   videoFilterType)
{
    ViStatus error = VI_SUCCESS;
             reserved;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_VIDEO_BANDWIDTH_TYPE, videoFilterType),
            3, "Video Filter Type");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function configures the sweep parameters of the spectrum analyzer.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR sweepTimeAuto/This control sets automatic sweep time.
/// HIPAR time/The length of time to complete one sweep.
/// HIPAR points/Sets defines the number of measurement points for one sweep run.
/// HIPAR type/Sets the sweep type.
/// HIPAR optimization/Selects the sweep type optimization mode.
ViStatus _VI_FUNC rsspecan_ConfigureSweep(ViSession instrSession,
                                          ViBoolean sweepTimeAuto,
                                          ViReal64  time,
                                          ViInt32   points,
                                          ViInt32   type,
                                          ViInt32   optimization)
{
    ViStatus error = VI_SUCCESS;
    ViInt32  sweepOptimization = optimization;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_SWEEP_TIME_AUTO, sweepTimeAuto),
            2, "Sweep Time Auto");

    if (sweepTimeAuto == VI_FALSE)
    {
        viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_SWEEP_TIME, time),
                3, "Time");
    }
    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_SWEEP_POINTS, points),
            4, "Points");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_SWEEP_TYPE, type),
            5, "Type");

    if (type == RSSPECAN_VAL_SWEEP_TYPE_FFT)
    {
        if (!rsspecan_IsFSV(instrSession))
        {
            viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "",
                                                    RSSPECAN_ATTR_SWEEP_OPTIMIZATION,
                                                    optimization), 6, "Optimization")
        }
        else
        {
            switch (optimization)
            {
            case RSSPECAN_VAL_SWEEP_OPTIMIZATION_AUTO:
                sweepOptimization = RSSPECAN_VAL_FFT_MODE_AUTO;
                break;
            case RSSPECAN_VAL_SWEEP_OPTIMIZATION_DYNAMIC:
                sweepOptimization = RSSPECAN_VAL_FFT_MODE_NARROW;
                break;
            case RSSPECAN_VAL_SWEEP_OPTIMIZATION_SPEED:
                sweepOptimization = RSSPECAN_VAL_FFT_MODE_WIDE;
                break;
            default:
                viCheckParm(RsCore_InvalidViInt32Value(instrSession, optimization), 6, "Optimization");
            }
            checkErr(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_FFT_FILTER_MODE, sweepOptimization));
        }
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This attribute queries the sweep type if you have turned on automatic
/// HIFN  selection of the sweep type.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR autoType/This control queries the sweep type if you have turned on automatic
/// HIPAR autoType/selection of the sweep type.
ViStatus _VI_FUNC rsspecan_QuerySweepAutoType(ViSession instrSession,
                                              ViInt32*  autoType)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_GetAttributeViInt32(instrSession, NULL, RSSPECAN_ATTR_SWEEP_AUTO_TYPE, autoType),
            2, "Auto Type");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function configures the sweep parameters of the spectrum analyzer.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR time/The length of time to complete one sweep.
/// HIPAR points/Sets defines the number of measurement points for one sweep run.
/// HIPAR optimization/Selects the sweep type optimization mode.
ViStatus _VI_FUNC rsspecan_ConfigureSweepZeroSpan(ViSession instrSession,
                                                  ViReal64  time,
                                                  ViInt32   points,
                                                  ViInt32   optimization)
{
    ViStatus error = VI_SUCCESS;
    ViInt32  sweepOptimization = optimization;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_SWEEP_TIME, time),
            3, "Time");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_SWEEP_POINTS, points),
            4, "Points");

    if (!rsspecan_IsFSV(instrSession))
    {
        viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "",
                                                RSSPECAN_ATTR_SWEEP_OPTIMIZATION,
                                                optimization), 6, "Optimization")
    }
    else
    {
        switch (optimization)
        {
        case RSSPECAN_VAL_SWEEP_OPTIMIZATION_AUTO:
            sweepOptimization = RSSPECAN_VAL_FFT_MODE_AUTO;
            break;
        case RSSPECAN_VAL_SWEEP_OPTIMIZATION_DYNAMIC:
            sweepOptimization = RSSPECAN_VAL_FFT_MODE_NARROW;
            break;
        case RSSPECAN_VAL_SWEEP_OPTIMIZATION_SPEED:
            sweepOptimization = RSSPECAN_VAL_FFT_MODE_WIDE;
            break;
        default:
            viCheckParm(RsCore_InvalidViInt32Value(instrSession, optimization), 6, "Optimization");
        }
        checkErr(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_FFT_FILTER_MODE, sweepOptimization));
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/*****************************************************************************
 * Function:    Configure Sweep Points
 * Purpose:     This function defines the number of measurement points for
 *              one sweep run.
 *****************************************************************************/
ViStatus _VI_FUNC rsspecan_ConfigureSweepPoints(ViSession instrSession,
                                                ViInt32   window,
                                                ViInt32   sweepPoints)
{
    ViStatus error = VI_SUCCESS;
             window;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_SWEEP_POINTS, sweepPoints),
            3, "Sweep Points");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the number of sweep points to analyze after a sweep.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control is reserved.
/// HIPAR sweepPoints/Defines the number of sweep points to analyze after a sweep.
ViStatus _VI_FUNC rsspecan_ConfigureWindowSweepPoints(ViSession instrSession,
                                                      ViInt32   window,
                                                      ViInt32   sweepPoints)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_WINDOW_SWEEP_POINTS,
            "Window", window - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Window");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_WINDOW_SWEEP_POINTS, sweepPoints),
            3, "Sweep Points");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/*****************************************************************************
 * Function:    Query Sweep Points
 * Purpose:     This function returns the number of measurement points for
 *              one sweep run.
 *****************************************************************************/
ViStatus _VI_FUNC rsspecan_QuerySweepPoints(ViSession instrSession,
                                            ViInt32   window,
                                            ViInt32   *sweepPoints)
{
    ViStatus error = VI_SUCCESS;
             window;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_GetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_SWEEP_POINTS, sweepPoints),
            3, "Sweep Points");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function provides an estimation of the total time required to
/// HIFN  capture the data and process it.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR sweepDuration/This control provides an estimation of the total time required to
/// HIPAR sweepDuration/capture the data and process it. This time span may be considerably
/// HIPAR sweepDuration/longer than the actual sweep time.
ViStatus _VI_FUNC rsspecan_QuerySweepDuration(ViSession instrSession,
                                              ViReal64  *sweepDuration)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_SWEEP_DURATION, sweepDuration),
            2, "Sweep Duration");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Returns the number of FFT partial spans required to cover the entire
/// HIFN  measurement range.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR subspans/Returns the number of FFT partial spans required to cover the entire
/// HIPAR subspans/measurement range.
ViStatus _VI_FUNC rsspecan_QuerySweepSubspans(ViSession instrSession,
                                              ViInt32*  subspans)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_GetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_FFT_SUBSPANS, subspans),
            2, "Subspans");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/*****************************************************************************
 *------------------- Trigger/Gate ------------------------------------------*
 *****************************************************************************/
/// HIFN This function specifies the trigger source that causes the
/// HIFN spectrum analyzer to leave the Wait-for-Trigger state.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR triggerSource/Document parameter here.
ViStatus _VI_FUNC rsspecan_ConfigureTriggerSource(ViSession instrSession,
                                                  ViInt32   reserved,
                                                  ViInt32   triggerSource)
{
    ViStatus error = VI_SUCCESS;
             reserved;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_TRIGGER_SOURCE, triggerSource),
            3, "Trigger Source");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function specifies the trigger parameters.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR offset/Sets the length of the trigger delay.
/// HIPAR slope/Specifies whether triggering occurs when the signal rises to the
/// HIPAR slope/trigger level or falls down to it.
ViStatus _VI_FUNC rsspecan_ConfigureTrigger(ViSession instrSession,
                                            ViInt32   reserved,
                                            ViReal64  offset,
                                            ViInt32   slope)
{
    ViStatus error = VI_SUCCESS;
             reserved;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_TRIGGER_DELAY, offset),
            3, "Offset");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_TRIGGER_SLOPE, slope),
            4, "Slope");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function specifies the dropout time for triggering.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR dropoutTime/Specifies the dropout time for triggering.
ViStatus _VI_FUNC rsspecan_ConfigureTriggerDropoutTime(ViSession instrSession,
                                                       ViReal64  dropoutTime)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_TRIGGER_DROPOUT_TIME, dropoutTime),
            2, "Dropout Time");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function defines the external trigger level and slope. This
/// HIFN is applicable when the trigger source is set to external.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR level/Sets the external level for triggering.
ViStatus _VI_FUNC rsspecan_ConfigureExternalTrigger(ViSession instrSession,
                                                    ViInt32   reserved,
                                                    ViReal64  level)
{
    ViStatus error = VI_SUCCESS;
             reserved;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_EXTERNAL_TRIGGER_LEVEL, level),
            3, "Level");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function defines the external trigger level.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR port/Selects the trigger port to which the output is sent.
/// HIPAR externalTriggerLevel/Defines the external trigger level.
ViStatus _VI_FUNC rsspecan_ConfigureExternalTriggerPort(ViSession instrSession,
                                                        ViInt32   port,
                                                        ViReal64  externalTriggerLevel)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, port, 1, 3),
            2, "Port");

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_EXTERNAL_TRIGGER_LEVEL_PORT,
            "PortNumber", port - 1, RS_REPCAP_BUF_SIZE, repCap));

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_EXTERNAL_TRIGGER_LEVEL_PORT, externalTriggerLevel),
            2, "External Trigger Level");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function specifies the video level for triggering.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR triggerLevel/Sets the video level for triggering.
ViStatus _VI_FUNC rsspecan_ConfigureVideoTrigger(ViSession instrSession,
                                                 ViInt32   reserved,
                                                 ViReal64  triggerLevel)
{
    ViStatus error = VI_SUCCESS;
             reserved;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_VIDEO_TRIGGER_LEVEL, triggerLevel),
            3, "Trigger Level");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function sets the level of the IF power trigger source.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR triggerLevel/Sets the level for triggering.
ViStatus _VI_FUNC rsspecan_ConfIFPowerTrigger(ViSession instrSession,
                                              ViReal64  triggerLevel,
                                              ViReal64  hysteresis,
                                              ViReal64  holdoff)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_TRIGGER_IFP_LEVEL, triggerLevel),
            2, "Trigger Level");

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_TRIGGER_IFP_HYSTERESIS, hysteresis),
            3, "Hysteresis");

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_TRIGGER_IFP_OFFSET, holdoff),
            4, "Holdoff");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the coupling of the external trigger to the
/// HIFN  oscilloscope, if the selected trigger source is IF Power or External
/// HIFN  CH2.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR coupling/Configures the coupling of the external trigger to the oscilloscope,
/// HIPAR coupling/if the selected trigger source is IF Power or External CH2.
ViStatus _VI_FUNC rsspecan_ConfigureIFPowerTriggerCoupling(ViSession instrSession,
                                                           ViInt32   coupling)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_TRIGGER_IF_POWER_COUPLING, coupling),
            2, "Coupling");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function sets the level of the IQ power trigger source.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR triggerLevel/Sets the level of the IQ power trigger.
ViStatus _VI_FUNC rsspecan_ConfigureIQPowerTrigger(ViSession instrSession,
                                                   ViInt32   reserved,
                                                   ViReal64  triggerLevel)
{
    ViStatus error = VI_SUCCESS;
             reserved;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_TRIGGER_IQP_LEVEL, triggerLevel),
            3, "Trigger Level");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function specifies sets the level of the RF power trigger
/// HIFN source.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR triggerLevel/Sets the level for triggering.
ViStatus _VI_FUNC rsspecan_ConfigureRFPowerTrigger(ViSession instrSession,
                                                   ViInt32   reserved,
                                                   ViReal64  triggerLevel)
{
    ViStatus error = VI_SUCCESS;
             reserved;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_TRIGGER_RFP_LEVEL, triggerLevel),
            3, "Trigger Level");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN This function configures the time trigger.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR timeInterval/This control sets the time interval for the time trigger source.
ViStatus _VI_FUNC rsspecan_ConfigureTimeTrigger(ViSession instrSession,
                                                ViInt32   reserved,
                                                ViReal64  timeInterval)
{
    ViStatus error = VI_SUCCESS;
             reserved;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_TRIGGER_TIME_INTERVAL, timeInterval),
            3, "Time Interval");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function configures the external gate parameters.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR gatedTrigger/This parameter switches on/off the gating.
/// HIPAR gateSource/This parameter selects the signal source for
/// HIPAR gateSource/gated measurements.
/// HIPAR mode/Selects the way gated measurements are triggered
/// HIPAR polarity/Sets the polarity of the external gate signal.
/// HIPAR delay/Defines the delay time between the external gate signal and the
/// HIPAR delay/continuation of the sweep.
/// HIPAR length/Defines the gate length.
ViStatus _VI_FUNC rsspecan_ConfigureExternalGate(ViSession instrSession,
                                                 ViInt32   reserved,
                                                 ViBoolean gatedTrigger,
                                                 ViInt32   gateSource,
                                                 ViInt32   mode,
                                                 ViInt32   polarity,
                                                 ViReal64  delay,
                                                 ViReal64  length)
{
    ViStatus error = VI_SUCCESS;
             reserved;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_IsFSV(instrSession) && (gateSource>RSSPECAN_VAL_EGAT_SOUR_PSE), 4, "Gate Source");

    if (gatedTrigger == VI_TRUE)
    {
        viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_EXTERNAL_GATE_SIGNAL_SOURCE, gateSource),
                4, "Gate Source");

        viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_EXTERNAL_GATE_TRIGGER_TYPE, mode),
                5, "Mode");
        if (gateSource == RSSPECAN_VAL_EGAT_SOUR_EXT)
        {
            viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_EXTERNAL_GATE_POLARITY, polarity),
                    6, "Polarity");
        }
        viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_EXTERNAL_GATE_HOLD, delay),
                7, "Delay");

        if (mode == RSSPECAN_VAL_EGAT_TRIG_EDGE)
        viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_EXTERNAL_GATE_LENGTH, length),
                8, "Length");
    }
    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_EXTERNAL_GATE, gatedTrigger),
            3, "Gated Trigger");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures continuous gating.
/// HIFN     This setting is only available if [SENSe:]SWEep:EGATe[:STATe] is 'On'.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR continuousEnabled/Activates or deactivates continuous gating.
/// HIPAR periodLength/Defines the length in seconds of a single gate period in continuous
/// HIPAR periodLength/gating. The length is determined from the beginning of one gate
/// HIPAR periodLength/measurement to the beginning of the next one.
/// HIPAR periodCount/Defines the number of gate periods to be measured after a single
/// HIPAR periodCount/trigger event.
ViStatus _VI_FUNC rsspecan_ConfigureContinuousGate(ViSession instrSession,
                                                   ViBoolean continuousEnabled,
                                                   ViReal64  periodLength,
                                                   ViInt32   periodCount)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, NULL, RSSPECAN_ATTR_EXTERNAL_GATE_CONTINUOUS_ENABLED, continuousEnabled),
            2, "Continuous Enabled");
    if (continuousEnabled)
    {
        viCheckParm(rsspecan_SetAttributeViReal64(instrSession, NULL, RSSPECAN_ATTR_EXTERNAL_GATE_PERIOD_LENGTH, periodLength),
                3, "Period Length");

        viCheckParm(rsspecan_SetAttributeViInt32(instrSession, NULL, RSSPECAN_ATTR_EXTERNAL_GATE_PERIODS, periodCount),
                3, "Period Count");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Determines whether the same or different triggers are used for general
/// HIFN  measurement and gating.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR sameTriggerEnabled/Determines whether the same or different triggers are used for general
/// HIPAR sameTriggerEnabled/measurement and gating.
ViStatus _VI_FUNC rsspecan_ConfigureGateSourceMode(ViSession instrSession,
                                                   ViBoolean sameTriggerEnabled)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, NULL, RSSPECAN_ATTR_EXTERNAL_GATE_SOURCE_MODE, sameTriggerEnabled),
            2, "Same Trigger Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the gate level for which the gate is open. Note that any RF
/// HIFN  attenuation or preamplification is considered when the trigger level is
/// HIFN  analyzed. If defined, a reference level offset is also considered.
/// HIFN     The input signal must be between 500 MHz and 8 GHz.
/// HIFN     Is only available for triggered gated measurements.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR rfLevel/Defines the gate level for which the gate is open. Note that any RF
/// HIPAR rfLevel/attenuation or preamplification is considered when the trigger level is
/// HIPAR rfLevel/analyzed. If defined, a reference level offset is also considered.
ViStatus _VI_FUNC rsspecan_ConfigureExternalGateRFLevel(ViSession instrSession,
                                                        ViReal64  rfLevel)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, NULL, RSSPECAN_ATTR_EXTERNAL_GATE_LEVEL, rfLevel),
            2, "RF Level");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the gate level for which the gate is open.
/// HIFN     Is only available for triggered gated measurements.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR port/Selects the trigger port to which the output is sent.
/// HIPAR externalGateLevel/Defines the gate level for which the gate is open.
ViStatus _VI_FUNC rsspecan_ConfigureExternalGatePort(ViSession instrSession,
                                                     ViInt32   port,
                                                     ViReal64  externalGateLevel)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_EXTERNAL_GATE_EXTERNAL_LEVEL,
            "PortNumber", port -1, RS_REPCAP_BUF_SIZE, repCap));

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_EXTERNAL_GATE_EXTERNAL_LEVEL, externalGateLevel),
            3, "External Gate Level");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the frequency and hysteresis for  trigger
/// HIFN  settings.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR frequency/This command defines the trigger frequency.
/// HIPAR hysteresis/This command defines the trigger hysteresis.
ViStatus _VI_FUNC rsspecan_ConfigureFrequencyTrigger(ViSession instrSession,
                                                     ViReal64  frequency,
                                                     ViReal64  hysteresis)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_TRIGGER_FREQUENCY, frequency),
            2, "Frequency");

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_TRIGGER_HYSTERESIS, hysteresis),
            3, "Hysteresis");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/*****************************************************************************
 *------------------- Frequency Mask Triggering -----------------------------*
 *****************************************************************************/
/// HIFN This function selects the frequency mask trigger.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsspecan_ConfigureFrequencyMaskTriggerSource(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_TRIGGER_SOURCE, RSSPECAN_VAL_TRG_MASK));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function sets the frequency mask triggering name with
/// HIFN  comments.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR name/This control creates a frequency trigger mask with the name that
/// HIPAR name/you specify.
/// HIPAR comment/This control defines a comment for the selected frequency mask
/// HIPAR comment/trigger.
ViStatus _VI_FUNC rsspecan_ConfigureFrequencyMask(ViSession instrSession,
                                                  ViString  name,
                                                  ViString  comment)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_FREQUENCY_MASK_TRIGGER_NAME, name),
            2, "Name");

    viCheckParm(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_FREQUENCY_MASK_TRIGGER_COMMENT, comment),
            3, "Comment");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN This function sets the directory the instrument stores frequency masks in.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the
/// HIPAR instrSession/Initialize function to select the desired instrument driver session.
/// HIPAR directory/This control sets the directory to store the frequency mask file in.
ViStatus _VI_FUNC rsspecan_ConfigureFrequencyMaskDirectory(ViSession instrSession,
                                                           ViString  directory)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_FREQUENCY_MASK_TRIGGER_DIRECTORY, directory),
            2, "Directory");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function deletes a frequency mask trigger mask.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the
/// HIPAR instrSession/Initialize function to select the desired instrument driver session.
/// HIPAR name/This control deletes a frequency mask trigger mask.
ViStatus _VI_FUNC rsspecan_DeleteFrequencyMask(ViSession instrSession,
                                               ViString  name)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_FREQUENCY_MASK_TRIGGER_DELETE, name),
            2, "Name");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function sets the condition that activates the frequency
/// HIFN mask.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the
/// HIPAR instrSession/Initialize function to select the desired instrument driver session.
/// HIPAR condition/This control sets the condition that activates the frequency
/// HIPAR condition/mask.
ViStatus _VI_FUNC rsspecan_ConfigureFrequencyMaskCondition(ViSession instrSession,
                                                           ViInt32   condition)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_FREQUENCY_MASK_TRIGG_CONDITION, condition),
            2, "Condition");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function configures the frequency mask shape.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the
/// HIPAR instrSession/Initialize function to select the desired instrument driver session.
/// HIPAR arraySize/This control sets the number of values returned in arrays.
/// HIPAR arraySize/The array size has to be at least equal to this number.
/// HIPAR mode/This control defines the scalling of the level axis for the
/// HIPAR mode/frequency mask trigger masks.
/// HIPAR lower/This control activates and defines the shape of the selected
/// HIPAR lower/lower frequency mask trigger mask.
/// HIPAR upper/This control activates and defines the shape of the selected
/// HIPAR upper/upper frequency mask trigger mask.
ViStatus _VI_FUNC rsspecan_ConfigureFrequencyMaskShape(ViSession instrSession,
                                                       ViInt32   arraySize,
                                                       ViInt32   mode,
                                                       ViReal64  lower[],
                                                       ViReal64  upper[])
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_FREQUENCY_MASK_TRIGGER_MODE, mode),
            2, "Mode");

    checkErr(RsCore_WriteAsciiViReal64Array(instrSession, "CALC:MASK:LOW:DATA ", lower, arraySize));
    checkErr(rsspecan_CheckStatus(instrSession));

    checkErr(RsCore_WriteAsciiViReal64Array(instrSession, "CALC:MASK:UPP:DATA ", upper, arraySize));
    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function automatically defines the shape of the frequency
/// HIFN mask trigger mask according to the spectrum that is currently
/// HIFN measured.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the
/// HIPAR instrSession/Initialize function to select the desired instrument driver session.
ViStatus _VI_FUNC rsspecan_ConfigureFrequencyMaskAuto(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_FREQUENCY_MASK_UPP_AUTO, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function sets the frequency mask shift.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the
/// HIPAR instrSession/Initialize function to select the desired instrument driver session.
/// HIPAR type/This control selects the upper and the lower mask.
/// HIPAR frequency/This control shifts the selected frequency mask trigger by a
/// HIPAR frequency/specific distance.
/// HIPAR amplitude/This control shifts the selected amplitude mask trigger by a
/// HIPAR amplitude/specific distance.
ViStatus _VI_FUNC rsspecan_ConfigureFrequencyMaskShift(ViSession instrSession,
                                                       ViInt32   type,
                                                       ViReal64  frequency,
                                                       ViReal64  amplitude)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    switch (type)
    {
    case RSSPECAN_VAL_LOWER:
        viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_FREQUENCY_MASK_SHIFT_LOW_X, frequency),
                3, "Frequency");

        viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_FREQUENCY_MASK_SHIFT_LOW_Y, amplitude),
                4, "Amplitude");
        break;
    case RSSPECAN_VAL_UPPER:
        viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_FREQUENCY_MASK_SHIFT_UPP_X, frequency),
                3, "Frequency");

        viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_FREQUENCY_MASK_SHIFT_UPP_Y, amplitude),
                4, "Amplitude");
        break;
    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, type), 2, "Type");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function turns the lower or upper frequency mask on and off.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR type/This control selects the upper and the lower mask.
/// HIPAR frequencyMask/This control turns the lower or upper frequency mask on and off.
ViStatus _VI_FUNC rsspecan_ConfigureFrequencyMaskState(ViSession instrSession,
                                                       ViInt32   type,
                                                       ViBoolean frequencyMask)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    switch (type)
    {
    case RSSPECAN_VAL_LOWER:
        viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_FREQUENCY_MASK_LOWER_STATE, frequencyMask),
                3, "Frequency Mask");
        break;
    case RSSPECAN_VAL_UPPER:
        viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_FREQUENCY_MASK_UPPER_STATE, frequencyMask),
                3, "Frequency Mask");
        break;
    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, type), 2, "Type");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the frequency mask span.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR span/This control sets the frequency mask span.
ViStatus _VI_FUNC rsspecan_ConfigureFrequencyMaskSpan(ViSession instrSession,
                                                      ViReal64  span)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_FREQUENCY_MASK_SPAN, span),
            2, "Span");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/*****************************************************************************
 *------------------- Auto Set ----------------------------------------------*
 *****************************************************************************/
/// HIFN This function configures the auto adjust settings.
/// HIRET Document return value here.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR adjustSettings/This control selects the auto adjust setting.
/// HIPAR measurementTimeMode/This control defines whether the duration of the level
/// HIPAR measurementTimeMode/measurement used to determine the optimal reference level is
/// HIPAR measurementTimeMode/determined automatically or manually.
/// HIPAR measurementTime/This control defines the duration of the level measurement used
/// HIPAR measurementTime/to determine the optimal reference level automatically.
ViStatus _VI_FUNC rsspecan_ConfigureAutoAdjust(ViSession instrSession,
                                               ViInt32   adjustSettings,
                                               ViInt32   measurementTimeMode,
                                               ViReal64  measurementTime)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    switch (adjustSettings)
    {
    case RSSPECAN_VAL_ADJ_ALL:
        checkErr(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_AUTO_ADJUST_ALL, NULL));
        break;
    case RSSPECAN_VAL_ADJ_FREQ:
        checkErr(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_AUTO_ADJUST_FREQ, NULL));
        break;
    case RSSPECAN_VAL_ADJ_LEV:
        checkErr(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_AUTO_ADJUST_LEVEL, NULL));
        break;
    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, adjustSettings), 2, "Adjust Settings");
    }

    if (rsspecan_IsFSV(instrSession) == VI_TRUE)
    {
        viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_AUTO_ADJUST_LEVEL_MEAS_TIME_MODE_FSV, measurementTimeMode),
                3, "Measurement Time Mode");

        if (measurementTimeMode == RSSPECAN_VAL_MAN)
        {
            viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_AUTO_ADJUST_LEVEL_MEAS_TIME_FSV, measurementTime),
                    4, "Measurement Time");
        }
    }
    else
    {
        viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_AUTO_ADJUST_LEVEL_MEAS_TIME_MODE, measurementTimeMode),
                3, "Measurement Time Mode");

        if (measurementTimeMode == RSSPECAN_VAL_MAN)
        {
            viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_AUTO_ADJUST_LEVEL_MEAS_TIME, measurementTime),
                    4, "Measurement Time");
        }
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function configures the auto adjust hystersis settings.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init
/// HIPAR instrSession/or rsspecan_InitWithOptions function.
/// HIPAR lowerHystersis/Defines a lower threshold the signal must drop below before the reference
/// HIPAR lowerHystersis/level is automatically adjusted when the "Auto Level" function is performed
/// HIPAR upperHystersis/Defines an upper threshold the signal must exceed before the reference
/// HIPAR upperHystersis/level is automatically adjusted when the "Auto Level" function is performed.
ViStatus _VI_FUNC rsspecan_ConfigureAutoAdjustHystersis(ViSession instrSession,
                                                        ViReal64  lowerHystersis,
                                                        ViReal64  upperHystersis)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_AUTO_ADJUST_LOWER_HYSTERESIS, lowerHystersis),
            2, "Lower Hystersis");

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_AUTO_ADJUST_UPPER_HYSTERESIS, upperHystersis),
            3, "Upper Hystersis");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function defines the behaviour of the measurement when adjusting a setting automatically.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init
/// HIPAR instrSession/or rsspecan_InitWithOptions function.
/// HIPAR state/Defines the behaviour of the measurement when adjusting a setting automatically.
ViStatus _VI_FUNC rsspecan_ConfigureAutoAdjustTriggerState(ViSession instrSession,
                                                           ViBoolean state)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_AUTO_ADJUST_TRIGGER_STATE, state),
            2, "state");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Determines the search mode for the automatic measurement performed to
/// HIFN  determine the optimal measurement configuration.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR searchMode/Determines the search mode for the automatic measurement performed to
/// HIPAR searchMode/determine the optimal measurement configuration.
ViStatus _VI_FUNC rsspecan_ConfigureAutoAdjustSearchMode(ViSession instrSession,
                                                         ViInt32   searchMode)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_AUTO_ADJUST_SEARCH_MODE, searchMode),
            2, "Search Mode");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/*****************************************************************************
 *------------------- Common Analysis and Display ---------------------------*
 *****************************************************************************/
/*****************************************************************************
 *------------------- Display -----------------------------------------------*
 *****************************************************************************/
/// HIFN  Defines the y-axis range.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control is reserved.
/// HIPAR min/Defines the bottom value of the y-axis.
/// HIPAR max/Defines the top value of the y-axis.
ViStatus _VI_FUNC rsspecan_ConfigureDisplayYAxisScaleRange(ViSession instrSession,
                                                           ViInt32   window,
                                                           ViReal64  min,
                                                           ViReal64  max)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_DISPLAY_Y_AXIS_SCALE_MIN,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap));

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_DISPLAY_Y_AXIS_SCALE_MIN, min),
            3, "Min");

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_DISPLAY_Y_AXIS_SCALE_MAX, max),
            4, "Max");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the Y axis range.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR min/Defines the minimum value of the y-axis for all traces in the selected
/// HIPAR min/result display.
/// HIPAR max/Defines the maximum value of the y-axis for all traces in the selected
/// HIPAR max/result display.
ViStatus _VI_FUNC rsspecan_ConfigureDisplayYAxisRange(ViSession instrSession,
                                                      ViInt32   window,
                                                      ViReal64  min,
                                                      ViReal64  max)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_DISPLAY_WINDOW_Y_AXIS_MINIMUM,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap));
    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_DISPLAY_WINDOW_Y_AXIS_MINIMUM, min),
            3, "Min");

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_DISPLAY_WINDOW_Y_AXIS_MAXIMUM, max),
            4, "Max");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Determines the grid spacing on the Y-axis for all diagrams, where
/// HIFN  possible. In spectrum displays, for example, this attribute is not
/// HIFN  available.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR spacing/Determines the grid spacing on the Y-axis for all diagrams, where
/// HIPAR spacing/possible. In spectrum displays, for example, this control is not
/// HIPAR spacing/available.
ViStatus _VI_FUNC rsspecan_ConfigureDisplayWindowYAxisGridSpacing(
                            ViSession instrSession,
                            ViInt32   window,
                            ViReal64  spacing)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_DISPLAY_WINDOW_Y_AXIS_GRID_SPACING,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap));

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_DISPLAY_WINDOW_Y_AXIS_GRID_SPACING, spacing),
            3, "Spacing");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the vertical position of the reference level on the display
/// HIFN  grid (for all traces).
/// HIFN     The instrument adjusts the scaling of the y-axis accordingly.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR referenceLevelPosition/Defines the vertical position of the reference level on the display
/// HIPAR referenceLevelPosition/grid (for all traces).
ViStatus _VI_FUNC rsspecan_ConfigureReferenceLevelPosition(ViSession instrSession,
                                                           ViInt32   window,
                                                           ViReal64  referenceLevelPosition)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_DISPLAY_WINDOW_REFERENCE_LEVEL_POSITION,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap));

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_DISPLAY_WINDOW_REFERENCE_LEVEL_POSITION, referenceLevelPosition),
            3, "Reference Level Position");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the reference value assigned to the reference position in the
/// HIFN  specified window. Separate reference values are maintained for the
/// HIFN  various displays.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR displayWindowReferenceLevel/Defines the reference value assigned to the reference position in the
/// HIPAR displayWindowReferenceLevel/specified window. Separate reference values are maintained for the
/// HIPAR displayWindowReferenceLevel/various displays.
ViStatus _VI_FUNC rsspecan_ConfigureDisplayWindowReferenceLevel(
                            ViSession instrSession,
                            ViInt32   window,
                            ViReal64  displayWindowReferenceLevel)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_DISPLAY_WINDOW_REFERENCE_LEVEL,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap));

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_DISPLAY_WINDOW_REFERENCE_LEVEL, displayWindowReferenceLevel),
            3, "Display Window Reference Level");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the range of the y-axis for all traces in the specified
/// HIFN  window.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR min/Defines the minimum value on the y-axis for all traces in the
/// HIPAR min/specified window.
/// HIPAR max/Defines the maximum value on the y-axis for all traces in the
/// HIPAR max/specified window.
ViStatus _VI_FUNC rsspecan_ConfigureDisplayWindowReferenceLevelRange(
                            ViSession instrSession,
                            ViInt32   window,
                            ViReal64  min,
                            ViReal64  max)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_DISPLAY_WINDOW_REFERENCE_LEVEL_MIN,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap));
    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_DISPLAY_WINDOW_REFERENCE_LEVEL_MIN, min),
            3, "Min");

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_DISPLAY_WINDOW_REFERENCE_LEVEL_MAX, max),
            4, "Max");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects the global unit for all phase results.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR unitPhase/Selects the global unit for all phase results.
ViStatus _VI_FUNC rsspecan_ConfigureDisplayWindowUnitPhase(ViSession instrSession,
                                                           ViInt32   window,
                                                           ViInt32   unitPhase)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_DISPLAY_WINDOW_UNIT_PHASE,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap));
    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_DISPLAY_WINDOW_UNIT_PHASE, unitPhase),
            3, "Unit Phase");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects the global unit for all frequency results.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR unitFrequency/Selects the global unit for all frequency results.
ViStatus _VI_FUNC rsspecan_ConfigureDisplayWindowUnitFrequency(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   unitFrequency)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_DISPLAY_WINDOW_UNIT_FREQUENCY,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_DISPLAY_WINDOW_UNIT_FREQUENCY, unitFrequency),
            3, "Unit Frequency");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  If the external generator control option (FSP-B10) is mounted and the
/// HIFN  normalization in the NETWORK mode is activated, this value defines the
/// HIFN  power value assigned to the reference position in the selected
/// HIFN  measurement window. This value corresponds to the parameter REFERENCE
/// HIFN  VALUE in manual operation.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR referenceValue/If the external generator control option (FSP-B10) is mounted and the
/// HIPAR referenceValue/normalization in the NETWORK mode is activated, this value defines the
/// HIPAR referenceValue/power value assigned to the reference position in the selected
/// HIPAR referenceValue/measurement window. This value corresponds to the parameter REFERENCE
/// HIPAR referenceValue/VALUE in manual operation.
ViStatus _VI_FUNC rsspecan_ConfigureDisplayWindowReferenceValue(
                            ViSession instrSession,
                            ViInt32   window,
                            ViReal64  referenceValue)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_DISPLAY_WINDOW_REFERENCE_VALUE,
            "Window", window - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Window");

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_DISPLAY_WINDOW_REFERENCE_VALUE, referenceValue),
            3, "Reference Value");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Turns the marker information in all diagrams on and off.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR markerInfo/Turns the marker information in all diagrams on and off.
ViStatus _VI_FUNC rsspecan_ConfigureDisplayWindowMarkerInfo(ViSession instrSession,
                                                            ViInt32   window,
                                                            ViBoolean markerInfo)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_DISPLAY_WINDOW_MARKER_INFO,
            "Window", window - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Window");

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, repCap, RSSPECAN_ATTR_DISPLAY_WINDOW_MARKER_INFO, markerInfo),
            3, "Marker Info");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function configures the display zooming parameters.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR zoomState/This control activates and deactivates the zoom mode.
/// HIPAR x1/Defines the x1 value of the area
/// HIPAR y1/Defines the y1 value of the area
/// HIPAR x2/Defines the x2 value of the area
/// HIPAR y2/Defines the y1 value of the area
ViStatus _VI_FUNC rsspecan_ConfigureDisplayZoom(ViSession instrSession,
                                                ViInt32   reserved,
                                                ViBoolean zoomState,
                                                ViReal64  x1,
                                                ViReal64  y1,
                                                ViReal64  x2,
                                                ViReal64  y2)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
             reserved;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViReal64Range(instrSession, x1, 0.0, 100.0), 4, "x1");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, y1, 0.0, 100.0), 5, "y1");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, x2, 0.0, 100.0), 6, "x2");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, y2, 0.0, 100.0), 7, "y2");

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_DISPLAY_ZOOM_STATE, zoomState),
            3, "Zoom State");

    if (zoomState == VI_TRUE)
    {
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "DISP:WIND:ZOOM:AREA %.12lG,%.12lG,%.12lG,%.12lG", x1, y1, x2, y2);
        checkErr(RsCore_Write(instrSession, cmd));

        checkErr(rsspecan_CheckStatus(instrSession));
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN This function configures the multiple display zooming parameters.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init.
/// HIPAR zoomWindow/Selects the zoom area.
/// HIPAR enableZoom/Turns the multiple zoom on and off
/// HIPAR x1/X coordinate of lower left corner.
/// HIPAR y1/Y coordinate of lower left corner.
/// HIPAR x2/X coordinate of upper right corner.
/// HIPAR y2/Y coordinate of upper right corner.
ViStatus _VI_FUNC rsspecan_ConfigureDisplayMultipleZoom(ViSession instrSession,
                                                        ViInt32   reserved,
                                                        ViInt32   zoomWindow,
                                                        ViBoolean enableZoom,
                                                        ViReal64  x1,
                                                        ViReal64  y1,
                                                        ViReal64  x2,
                                                        ViReal64  y2)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
             reserved;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViReal64Range(instrSession, x1, 0.0, 100.0), 5, "x1");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, y1, 0.0, 100.0), 6, "y1");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, x2, 0.0, 100.0), 7, "x2");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, y2, 0.0, 100.0), 8, "y2");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_DISPLAY_MULTIPLE_ZOOM_STATE,
            "ZoomWindow", zoomWindow - 1, RS_REPCAP_BUF_SIZE, repCap),
            3, "Zoom Window");

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, repCap, RSSPECAN_ATTR_DISPLAY_MULTIPLE_ZOOM_STATE, enableZoom),
            4, "Enable Zoom");

    if (enableZoom == VI_TRUE)
    {
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "DISP:WIND:ZOOM:MULT%ld:AREA %.12lG,%.12lG,%.12lG,%.12lG", zoomWindow, x1, y1, x2, y2);
        checkErr(RsCore_Write(instrSession, cmd));

        checkErr(rsspecan_CheckStatus(instrSession));
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the multiple display zooming parameters.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR subwindow/This control selects the measurement subwindow.
/// HIPAR zoomWindow/This control selects the zoom area.
/// HIPAR enableZoom/This parameter turns the multiple zoom on and off.
/// HIPAR x1/Diagram coordinates in % of the complete diagram that define the
/// HIPAR y1/Diagram coordinates in % of the complete diagram that define the
/// HIPAR x2/Diagram coordinates in % of the complete diagram that define the
/// HIPAR y2/Diagram coordinates in % of the complete diagram that define the
ViStatus _VI_FUNC rsspecan_ConfigureDisplaySubwindowMultipleZoom(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   subwindow,
                            ViInt32   zoomWindow,
                            ViBoolean enableZoom,
                            ViReal64  x1,
                            ViReal64  y1,
                            ViReal64  x2,
                            ViReal64  y2)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViReal64Range(instrSession, x1, 0.0, 100.0), 6, "x1");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, y1, 0.0, 100.0), 7, "y1");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, x2, 0.0, 100.0), 8, "x2");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, y2, 0.0, 100.0), 9, "y2");

    snprintf (repCap, RS_REPCAP_BUF_SIZE, "Win%ld,SW%ld,ZWin%ld", window, subwindow, zoomWindow);

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, repCap, RSSPECAN_ATTR_SUBWINDOW_DISPLAY_MULTIPLE_ZOOM_STATE, enableZoom),
            5, "Enable Zoom");

    if (enableZoom == VI_TRUE)
    {
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "DISP:WIND%ld:SUBW%ld:ZOOM:MULT%ld:AREA %.12lG,%.12lG,%.12lG,%.12lG", window, subwindow, zoomWindow, x1, y1, x2, y2);
        checkErr(RsCore_Write(instrSession, cmd));

        checkErr(rsspecan_CheckStatus(instrSession));
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/*****************************************************************************
 *------------------- Average -----------------------------------------------*
 *****************************************************************************/
/// HIFN This function switches on or off the average calculation for
/// HIFN the selected trace
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR trace/Selects the trace for Detector definition.
/// HIPAR state/Switches on or off the average calculation for
/// HIPAR state/the selected trace.
ViStatus _VI_FUNC rsspecan_ConfigureAveraging(ViSession instrSession,
                                              ViInt32   reserved,
                                              ViInt32   trace,
                                              ViBoolean state)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_MAX_MESSAGE_BUF_SIZE] = "";
             reserved;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_AVG_STATE,
            "Trace", trace - 1, RS_REPCAP_BUF_SIZE, repCap),
            3, "Trace");

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, repCap, RSSPECAN_ATTR_AVG_STATE, state),
            4, "State");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function selects the type of average function.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR averagingType/Selects the type of average function.
ViStatus _VI_FUNC rsspecan_ConfigureAveragingType(ViSession instrSession,
                                                  ViInt32   reserved,
                                                  ViInt32   averagingType)
{
    ViStatus error = VI_SUCCESS;
             reserved;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_AVG_TYPE, averagingType),
            3, "Averaging Type");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function configures the averaging count.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR count/This control defines the number of measurements which contribute
/// HIPAR count/to the average value.
ViStatus _VI_FUNC rsspecan_ConfigureAveragingCount(ViSession instrSession,
                                                   ViInt32   reserved,
                                                   ViInt32   count)
{
    ViStatus error = VI_SUCCESS;
             reserved;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_AVG_COUNT, count),
            3, "Count");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/*****************************************************************************
 *------------------- Trace -------------------------------------------------*
 *****************************************************************************/
/// HIFN This function configures the trace to acquire.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR trace/Selects the trace.
/// HIPAR traceType/Document parameter here.
ViStatus _VI_FUNC rsspecan_ConfigureTrace(ViSession instrSession,
                                          ViInt32   window,
                                          ViInt32   trace,
                                          ViInt32   traceType)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_w[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, window, 0, 16),
            2, "Window");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_TRACE_TYPE,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap_w),
            2, "Window");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_TRACE_TYPE,
            "Trace", trace - 1, RS_REPCAP_BUF_SIZE, repCap),
            3, "Trace");

    strcat(repCap, ",");
    strcat(repCap, repCap_w);

    switch (traceType)
    {
    case RSSPECAN_TRAC_MOD_BLANK:
        checkErr(rsspecan_SetAttributeViBoolean(instrSession, repCap, RSSPECAN_ATTR_TRACE_STATE, VI_FALSE));
        break;
    default:
        checkErr(rsspecan_SetAttributeViBoolean(instrSession, repCap, RSSPECAN_ATTR_TRACE_STATE, VI_TRUE));
        viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_TRACE_TYPE, traceType),
                4, "Trace Type");
        break;
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function configures the trace detector.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR trace/Selects the trace.
/// HIPAR detectorTypeAuto/Turns auto detector on/off.
/// HIPAR detectorType/Specifies the method of capturing and processing
/// HIPAR detectorType/signal data.
ViStatus _VI_FUNC rsspecan_ConfigureTraceDetector(ViSession instrSession,
                                                  ViInt32   reserved,
                                                  ViInt32   trace,
                                                  ViBoolean detectorTypeAuto,
                                                  ViInt32   detectorType)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
             reserved;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_DETECTOR_TYPE_AUTO,
            "Trace", trace - 1, RS_REPCAP_BUF_SIZE, repCap),
            3, "Trace");

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, repCap, RSSPECAN_ATTR_DETECTOR_TYPE_AUTO, detectorTypeAuto),
            4, "Detector Type Auto");
    if (detectorTypeAuto == VI_FALSE)
    {
        viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_DETECTOR_TYPE, detectorType),
                5, "Detector Type");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures trace smoothing for a particular trace.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR trace/Selects the trace.
/// HIPAR smoothingEnabled/Turns trace smoothing for a particular trace on and off.
/// HIPAR smoothingAperture/Defines the degree (aperture) of the trace smoothing, if enabled.
ViStatus _VI_FUNC rsspecan_ConfigureTraceSmoothing(ViSession instrSession,
                                                   ViInt32   window,
                                                   ViInt32   trace,
                                                   ViBoolean smoothingEnabled,
                                                   ViReal64  smoothingAperture)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCapTrace[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_TRACE_SMOOTHING_ENABLED,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap));
    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_TRACE_SMOOTHING_ENABLED,
            "Trace", trace - 1, RS_REPCAP_BUF_SIZE, repCapTrace));

    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCapTrace, NULL);

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, repCap, RSSPECAN_ATTR_TRACE_SMOOTHING_ENABLED, smoothingEnabled),
            4, "Smoothing Enabled");
    if (smoothingEnabled)
    {
        viCheckParm(rsspecan_SetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_TRACE_SMOOTHING_APERTURE, smoothingAperture),
                5, "Smoothing Aperture");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Turns symbols that represent the measurement points on a trace on and
/// HIFN  off.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control is reserved.
/// HIPAR traceSymbolEnabled/Turns symbols that represent the measurement points on a trace on and
/// HIPAR traceSymbolEnabled/off.
ViStatus _VI_FUNC rsspecan_ConfigureTraceSymbolEnabled(ViSession instrSession,
                                                       ViInt32   window,
                                                       ViBoolean traceSymbolEnabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_DISPLAY_TRACE_SYMBOL_ENABLED,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, repCap, RSSPECAN_ATTR_DISPLAY_TRACE_SYMBOL_ENABLED, traceSymbolEnabled),
            3, "Trace Symbol Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function specifies whether or not the traces with peak or
/// HIFN minimum value detection are reset after specific parameter
/// HIFN changes.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR trace/Selects the trace.
/// HIPAR resetAtChange/Specifies whether or not the traces with peak or minimum value
/// HIPAR resetAtChange/detection are reset after specific parameter changes.
ViStatus _VI_FUNC rsspecan_ConfigureTraceResetBehavior(ViSession instrSession,
                                                       ViInt32   window,
                                                       ViInt32   trace,
                                                       ViBoolean resetAtChange)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap_trace[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_TRACE_RESET_BEHAVIOUR_SPECIFIC_WINDOW,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap_trace),
            2, "Trace");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_TRACE_RESET_BEHAVIOUR_SPECIFIC_WINDOW,
            "Trace", trace - 1, RS_REPCAP_BUF_SIZE, repCap_trace),
            3, "Trace");

    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_trace, NULL);

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, repCap, RSSPECAN_ATTR_TRACE_RESET_BEHAVIOUR_SPECIFIC_WINDOW, resetAtChange),
            4, "Reset At Change");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Switches on or off the display of the corresponding trace in the
/// HIFN  selected measurement window.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR subwindow/This control selects the measurement subwindow.
/// HIPAR trace/Selects the trace for Detector definition.
/// HIPAR resetatChange/Switches on or off the display of the corresponding trace in the
/// HIPAR resetatChange/selected measurement window.
ViStatus _VI_FUNC rsspecan_ConfigureTraceSubwindowState(ViSession instrSession,
                                                        ViInt32   window,
                                                        ViInt32   subwindow,
                                                        ViInt32   trace,
                                                        ViBoolean resetatChange)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, window, 0, 16),
            2, "Window");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, subwindow, 1, 64),
            3, "Subwindow");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, trace, 1, 6),
            4, "Trace");

    snprintf (repCap, RS_REPCAP_BUF_SIZE, "Win%ld,SW%ld,TR%ld", window, subwindow, trace);

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, repCap, RSSPECAN_ATTR_SUBWINDOW_TRACE_STATE, resetatChange),
            5, "Reset at Change");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Configures the trace subwindow smoothing.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR subwindow/This control selects the measurement subwindow.
/// HIPAR trace/Selects the trace for Detector definition.
/// HIPAR enabled/Turns trace smoothing for a particular trace on and off.
/// HIPAR aperture/Defines the degree (aperture) of the trace smoothing, if enabled.
ViStatus _VI_FUNC rsspecan_ConfigureTraceSubwindowSmoothing(ViSession instrSession,
                                                            ViInt32   window,
                                                            ViInt32   subwindow,
                                                            ViInt32   trace,
                                                            ViBoolean enabled,
                                                            ViReal64  aperture)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, window, 0, 16),
            2, "Window");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, subwindow, 1, 64),
            3, "Subwindow");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, trace, 1, 6),
            4, "Trace");

    snprintf (repCap, RS_REPCAP_BUF_SIZE, "Win%ld,SW%ld,TR%ld", window, subwindow, trace);

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, repCap, RSSPECAN_ATTR_SUBWINDOW_TRACE_SMOOTHING_ENABLED, enabled),
            5, "Enabled");

    if (enabled == VI_TRUE)
    {
        viCheckParm(rsspecan_SetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_SUBWINDOW_TRACE_SMOOTHING_APERTURE, aperture),
                6, "Aperture");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function specifies whether or not the traces with peak or minimum
/// HIFN  value detection are reset after specific parameter changes.
/// HIFN     Usually the measurement must be restarted after a parameter change,
/// HIFN  before an evaluation of the measurement results is performed (e.g. with
/// HIFN  a marker). In cases in which a change causes a compulsory new
/// HIFN  measurement, the trace is automatically reset in order to prevent
/// HIFN  erroneous measurements of previous measurement results (e.g. when the
/// HIFN  span changes). For applications in which this behavior is not desired,
/// HIFN  this mechanism can be switched off.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR subwindow/This control selects the measurement subwindow.
/// HIPAR trace/Selects the trace for Detector definition.
/// HIPAR resetatChange/Specifies whether or not the traces with peak or minimum value
/// HIPAR resetatChange/detection are reset after specific parameter changes.
ViStatus _VI_FUNC rsspecan_ConfigureTraceSubwindowResetBehavior(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   subwindow,
                            ViInt32   trace,
                            ViBoolean resetatChange)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, window, 0, 16),
            2, "Window");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, subwindow, 1, 64),
            3, "Subwindow");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, trace, 1, 6),
            4, "Trace");

    snprintf (repCap, RS_REPCAP_BUF_SIZE, "Win%ld,SW%ld,TR%ld", window, subwindow, trace);

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, repCap, RSSPECAN_ATTR_SUBWINDOW_TRACE_RESET_BEHAVIOUR_SPECIFIC_WINDOW, resetatChange),
            5, "Reset at Change");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Applies predefined, commonly required trace settings to the selected
/// HIFN  window.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR subwindow/This control selects the measurement subwindow.
/// HIPAR trace/Selects the trace for Detector definition.
ViStatus _VI_FUNC rsspecan_ConfigureTraceSubwindowPreset(ViSession instrSession,
                                                         ViInt32   window,
                                                         ViInt32   subwindow,
                                                         ViInt32   trace,
                                                         ViInt32   preset)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, window, 0, 16),
            2, "Window");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, subwindow, 1, 64),
            3, "Subwindow");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, trace, 1, 6),
            4, "Trace");

    snprintf (repCap, RS_REPCAP_BUF_SIZE, "Win%ld,SW%ld,TR%ld", window, subwindow, trace);

    checkErr(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_SUBWINDOW_TRACE_PRESET, preset));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function copies one trace array to another trace array.
/// HIFN Any data in the destination trace is over written.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR destinationTrace/Selects the destination trace.
/// HIPAR sourceTrace/Selects the source trace.
ViStatus _VI_FUNC rsspecan_CopyTrace(ViSession instrSession,
                                     ViInt32   reserved,
                                     ViInt32   destinationTrace,
                                     ViInt32   sourceTrace)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
             reserved;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, destinationTrace, 1, 6),
            3, "Destination Trace");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, sourceTrace, 1, 6),
            4, "Source Trace");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "TRAC:COPY TRACE%ld,TRACE%ld", destinationTrace, sourceTrace);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function configures subtraction of traces.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR position/Defines the position of the result of the trace mathematics.
/// HIPAR mode/Selects linear or logarithmic (= video) calculation of the
/// HIPAR mode/mathematical functions related to the traces.
/// HIPAR traceMathState/Switches the mathematical relation of traces on or off.
ViStatus _VI_FUNC rsspecan_ConfigureSubtractTraces(ViSession instrSession,
                                                   ViInt32   reserved,
                                                   ViReal64  position,
                                                   ViInt32   mode,
                                                   ViBoolean traceMathState)
{
    ViStatus error = VI_SUCCESS;
             reserved;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_TRACE_MATH_POSITION, position),
            3, "Position");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_TRACE_MATH_MODE, mode),
            4, "Mode");

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_TRACE_MATH_STATE, traceMathState),
            5, "Trace Math State");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN Defines which trace is subtracted from trace 1.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR trace2/Defines which trace is subtracted from trace 1.
ViStatus _VI_FUNC rsspecan_SubtractTraces(ViSession instrSession,
                                          ViInt32   reserved,
                                          ViInt32   trace2)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
             reserved;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, trace2, 2, 6),
            3, "Trace 2");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CALC:MATH (TRACE1-TRACE%ld)", trace2);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function determines the format of the ASCII file to be
/// HIFN imported or exported.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR fileType/Selects the data to be included in a data export file.
ViStatus _VI_FUNC rsspecan_ConfigureTraceFileType(ViSession instrSession,
                                                  ViInt32   fileType)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_TRACE_FILE_TYPE, fileType),
            2, "File Type");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function selects the data to be included in a data export file.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR exportMode/Selects the data to be included in a data export file.
/// HIPAR exportHeader/If enabled, additional instrument and measurement settings are
/// HIPAR exportHeader/included in the header of the export file for result data. If disabled,
/// HIPAR exportHeader/only the pure result data from the selected traces and tables is
/// HIPAR exportHeader/exported.
/// HIPAR decimalSeparator/This control sets the decimal separator for the ASCII data
/// HIPAR decimalSeparator/stored to the file.
ViStatus _VI_FUNC rsspecan_ConfigureTraceExport(ViSession instrSession,
                                                ViInt32   exportMode,
                                                ViBoolean exportHeader,
                                                ViInt32   decimalSeparator)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_TRACE_EXPORT_ALL, exportMode),
            2, "Export Mode");

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_FILE_EXP_HEADER, exportHeader),
            3, "Export Header");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_FILE_DEC_SEPARATOR, decimalSeparator),
            4, "Decimal Separator");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This attribute selects the column separator for exported trace data.
/// HIFN
/// HIFN     The selected value is not affected by a preset. The command therefore
/// HIFN  has no reset value.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR seperator/This control selects the column separator for exported trace data.
ViStatus _VI_FUNC rsspecan_ConfigureTraceExportColumnSeperator(
                            ViSession instrSession,
                            ViInt32   seperator)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, NULL, RSSPECAN_ATTR_FILE_EXPORT_COLUMN_SEPERATOR, seperator),
            2, "");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Defines how the x-values of the trace are determined in the frequency
/// HIFN  domain.
/// HIFN
/// HIFN     See 'X-Value of the Sweep Point'.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR distribution/Defines how the x-values of the trace are determined in the frequency
/// HIPAR distribution/domain.
ViStatus _VI_FUNC rsspecan_ConfigureTraceExportXValueDistribution(
                            ViSession instrSession,
                            ViInt32   distribution)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, NULL, RSSPECAN_ATTR_FILE_EXPORT_X_VALUE_DISTRIBUTION, distribution),
            2, "Distribution");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function stores the selected trace (1 to 6) in the
/// HIFN measurement window indicated by window in a file with ASCII
/// HIFN format.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR trace/Selects the trace.
/// HIPAR fileName/Specifies full path of the file, where trace data will be
/// HIPAR fileName/stored.
ViStatus _VI_FUNC rsspecan_StoreTraceToFile(ViSession instrSession,
                                            ViInt32   window,
                                            ViInt32   trace,
                                            ViString  fileName)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, window, 0, 16),
            2, "Window");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, trace, 1, 6),
            3, "Trace");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, fileName),
        4, "File Name");

    if (window != 0)
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "MMEM:STOR%ld:TRAC %ld,'%s'", window, trace, fileName);
    else
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "MMEM:STOR:TRAC %ld,'%s'", trace, fileName);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function selects the data to be included in a data export
/// HIFN file.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR importMode/Selects the data to be imported  from a data import file.
ViStatus _VI_FUNC rsspecan_ConfigureTraceImport(ViSession instrSession,
                                                ViInt32   importMode)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_TRACE_IMPORT_ALL, importMode),
            2, "Import Mode");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN Document your function here.
/// HIFN You may use multiple lines for documentation.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR trace/Selects the trace.
/// HIPAR fileName/Document parameter here.
ViStatus _VI_FUNC rsspecan_ImportSingleTraceFromFile(ViSession instrSession,
                                                     ViInt32   window,
                                                     ViInt32   trace,
                                                     ViString  fileName)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, window, 0, 16),
            2, "Window");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, trace, 1, 6),
            3, "Trace");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, fileName),
        4, "File Name");

    if (window != 0)
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "MMEM:LOAD%ld:TRAC %ld,'%s'", window, trace, fileName);
    else
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "MMEM:LOAD:TRAC %ld,'%s'", trace, fileName);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/*****************************************************************************
 *------------------- Marker ------------------------------------------------*
 *****************************************************************************/
/// HIFN This function configures the marker.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR marker/This control selects marker.
/// HIPAR markerState/Specifies whether the specified marker is enabled.
/// HIPAR trace/Selects the trace.
/// HIPAR window/This control selects the measurement window.
ViStatus _VI_FUNC rsspecan_ConfigureMarker(ViSession instrSession,
                                           ViInt32   window,
                                           ViInt32   marker,
                                           ViBoolean markerState,
                                           ViInt32   trace)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  old_win = 0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, window, 0, 16),
            2, "Window");

    checkErr(rsspecan_GetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_ACTIVE_WINDOW, &old_win));
    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_ACTIVE_WINDOW, window),
            2, "Window");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_MARKER_ENABLED,
            "Marker", marker - 1, RS_REPCAP_BUF_SIZE, repCap),
            3, "Marker");

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, repCap, RSSPECAN_ATTR_MARKER_ENABLED, markerState),
            4, "Marker State");

    if (markerState == VI_TRUE)
    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_MARKER_TRACE, trace),
            5, "Trace");

Error:
    rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_ACTIVE_WINDOW, old_win);
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function turns off all the markers
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR window/This control selects the measurement window.
ViStatus _VI_FUNC rsspecan_DisableAllMarkers(ViSession instrSession,
                                             ViInt32   window)
{
    ViStatus error = VI_SUCCESS;
    ViInt32  old_win = 0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, window, 0, 16),
            2, "Window");

    checkErr(rsspecan_GetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_ACTIVE_WINDOW, &old_win));
    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_ACTIVE_WINDOW, window),
            2, "Window");

    checkErr(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_MARKER_AOFF, NULL));

Error:
    rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_ACTIVE_WINDOW, old_win);
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function specifies the type of marker search and performs
/// HIFN the search.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR marker/This control selects marker.
/// HIPAR markerSearch/Specifies the type of marker search type.
/// HIPAR window/This control selects the measurement window.
ViStatus _VI_FUNC rsspecan_MarkerSearch(ViSession instrSession,
                                        ViInt32   window,
                                        ViInt32   marker,
                                        ViInt32   markerSearch)
{
    ViStatus error = VI_SUCCESS;
    ViInt32  old_win = 0;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, window, 0, 16),
            2, "Window");

    checkErr(rsspecan_GetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_ACTIVE_WINDOW, &old_win));
    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_ACTIVE_WINDOW, window),
            2, "Window");

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_MARKER_SEARCH_PEAK,
            "Marker", marker - 1, RS_REPCAP_BUF_SIZE, repCap),
            3, "Marker");
    switch (markerSearch)
    {
    case RSSPECAN_VAL_MARKER_SEARCH_HIGHEST:
        checkErr(rsspecan_SetAttributeViString(instrSession, repCap, RSSPECAN_ATTR_MARKER_SEARCH_PEAK, NULL));
        break;
    case RSSPECAN_VAL_MARKER_SEARCH_NEXT_PEAK:
        checkErr(rsspecan_SetAttributeViString(instrSession, repCap, RSSPECAN_ATTR_MARKER_SEARCH_PEAK_NEXT, NULL));
        break;
    case RSSPECAN_VAL_MARKER_SEARCH_NEXT_PEAK_LEFT:
        checkErr(rsspecan_SetAttributeViString(instrSession, repCap, RSSPECAN_ATTR_MARKER_SEARCH_PEAK_LEFT, NULL));
        break;
    case RSSPECAN_VAL_MARKER_SEARCH_NEXT_PEAK_RIGHT:
        checkErr(rsspecan_SetAttributeViString(instrSession, repCap, RSSPECAN_ATTR_MARKER_SEARCH_PEAK_RIGHT, NULL));
        break;
    case RSSPECAN_VAL_MARKER_SEARCH_MINIMUM:
        checkErr(rsspecan_SetAttributeViString(instrSession, repCap, RSSPECAN_ATTR_MARKER_SEARCH_MIN, NULL));
        break;
    case RSSPECAN_VAL_MARKER_SEARCH_NEXT_MINIMUM:
        checkErr(rsspecan_SetAttributeViString(instrSession, repCap, RSSPECAN_ATTR_MARKER_SEARCH_MIN_NEXT, NULL));
        break;
    case RSSPECAN_VAL_MARKER_SEARCH_NEXT_MINIMUM_LEFT:
        checkErr(rsspecan_SetAttributeViString(instrSession, repCap, RSSPECAN_ATTR_MARKER_SEARCH_MIN_LEFT, NULL));
        break;
    case RSSPECAN_VAL_MARKER_SEARCH_NEXT_MINIMUM_RIGHT:
        checkErr(rsspecan_SetAttributeViString(instrSession, repCap, RSSPECAN_ATTR_MARKER_SEARCH_MIN_RIGHT, NULL));
        break;

    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, markerSearch), 4, "Marker Search");
        break;
    }

Error:
    rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_ACTIVE_WINDOW, old_win);
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function moves the active marker to the specified
/// HIFN horizontal position.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR marker/This control selects marker.
/// HIPAR markerPosition/Horizontal position.
ViStatus _VI_FUNC rsspecan_MoveMarker(ViSession instrSession,
                                      ViInt32   window,
                                      ViInt32   marker,
                                      ViReal64  markerPosition)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  old_win = 0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, window, 0, 16),
            2, "Window");

    checkErr(rsspecan_GetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_ACTIVE_WINDOW, &old_win));
    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_ACTIVE_WINDOW, window),
            2, "Window");

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_MARKER_POSITION,
            "Marker", marker - 1, RS_REPCAP_BUF_SIZE, repCap),
            3, "Marker");

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_MARKER_POSITION, markerPosition),
            4, "Marker Position");

Error:
    rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_ACTIVE_WINDOW, old_win);
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function makes the selected marker frequency to be the
/// HIFN center frequency or step width of center frequency. It can also
/// HIFN make the active marker amplitude to be the reference level.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR marker/This control selects marker.
/// HIPAR instrumentSetting/Specifies the instrument setting to be set from the marker
/// HIPAR instrumentSetting/position.
/// HIPAR window/This control selects the measurement window.
ViStatus _VI_FUNC rsspecan_SetInstrumentFromMarker(ViSession instrSession,
                                                   ViInt32   window,
                                                   ViInt32   marker,
                                                   ViInt32   instrumentSetting)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  old_win = 0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, window, 0, 16),
            2, "Window");

    checkErr(rsspecan_GetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_ACTIVE_WINDOW, &old_win));
    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_ACTIVE_WINDOW, window),
            2, "Window");
    switch (instrumentSetting)
    {
    case RSSPECAN_VAL_INSTRUMENT_SETTING_FREQUENCY_CENTER:
        viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_MARKER_TO_CENTER,
                "Marker", marker - 1, RS_REPCAP_BUF_SIZE, repCap),
                3, "Marker");
        checkErr(rsspecan_SetAttributeViString(instrSession, repCap, RSSPECAN_ATTR_MARKER_TO_CENTER, NULL));
        break;
    case RSSPECAN_VAL_INSTRUMENT_SETTING_FREQUENCY_STEP:
        viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_MARKER_TO_STEP,
                "Marker", marker - 1, RS_REPCAP_BUF_SIZE, repCap),
                3, "Marker");
        checkErr(rsspecan_SetAttributeViString(instrSession, repCap, RSSPECAN_ATTR_MARKER_TO_STEP, NULL));
        break;
    case RSSPECAN_VAL_INSTRUMENT_SETTING_REFERENCE_LEVEL:
        viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_MARKER_TO_REFERENCE,
                "Marker", marker - 1, RS_REPCAP_BUF_SIZE, repCap),
                3, "Marker");
        checkErr(rsspecan_SetAttributeViString(instrSession, repCap, RSSPECAN_ATTR_MARKER_TO_REFERENCE, NULL));
        break;

    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, instrumentSetting), 4, "Instrument Setting");
        break;
    }

Error:
    rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_ACTIVE_WINDOW, old_win);
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function links normal marker <FirstMarker> to any active
/// HIFN normal marker <SecondMarker>.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR firstMarker/This control selects first marker.
/// HIPAR secondMarker/This control selects second marker.
/// HIPAR state/Specifies the marker link state.
/// HIPAR window/This control selects the measurement window.
ViStatus _VI_FUNC rsspecan_LinkMarkerToAnotherMarker(ViSession instrSession,
                                                     ViInt32   window,
                                                     ViInt32   firstMarker,
                                                     ViInt32   secondMarker,
                                                     ViBoolean state)
{
    ViStatus error = VI_SUCCESS;
    ViChar   marker[10] = "";
    ViChar   marker2[10] = "";
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  old_win = 0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, window, 0, 16),
            2, "Window");

    checkErr(rsspecan_GetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_ACTIVE_WINDOW, &old_win));
    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_ACTIVE_WINDOW, window),
            2, "Window");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession,
                                                 RSSPECAN_ATTR_MARKER_LINK_TO_MARKER,
                                                 "Marker",
                                                 firstMarker - 1,
                                                 10,
                                                 marker), 3, "First Marker");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession,
                                                 RSSPECAN_ATTR_MARKER_LINK_TO_MARKER,
                                                 "SecondMarker",
                                                 secondMarker - 1,
                                                 10,
                                                 marker2), 4, "Second Marker");
    snprintf(repCap, RS_REPCAP_BUF_SIZE, "%s,%s", marker, marker2);
    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, repCap, RSSPECAN_ATTR_MARKER_LINK_TO_MARKER, state),
            5, "State");

Error:
    rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_ACTIVE_WINDOW, old_win);
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Links the normal source marker <ms> to any active delta destination
/// HIFN  marker <md>. If you change the horizontal position of marker <md>,
/// HIFN  marker <ms> changes its horizontal position to the same value.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR marker/Selects marker.
/// HIPAR deltaMarker/Selects the delta marker to link to.
/// HIPAR link/Links the normal source marker <ms> to any active delta destination
/// HIPAR link/marker <md>. If you change the horizontal position of marker <md>,
/// HIPAR link/marker <ms> changes its horizontal position to the same value.
ViStatus _VI_FUNC rsspecan_LinkingToDeltaMarker(ViSession instrSession,
                                                ViInt32   window,
                                                ViInt32   marker,
                                                ViInt32   deltaMarker,
                                                ViBoolean link)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCapMarker[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCapDeltaMarker[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  markerCount = 0;
    ViInt32  oldWindow = 0;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsspecan_GetAttributeViInt32(instrSession, NULL, RSSPECAN_ATTR_ACTIVE_WINDOW, &oldWindow));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_ACTIVE_WINDOW, window),
            2, "Window");

    checkErr(RsCore_GetRepCapIdItemsCount(instrSession, "Marker", &markerCount));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, marker, 1, markerCount),
            3, "Marker");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_MARKER_LINK_TO_DELTA_MARKER, "Marker", marker - 1, RS_REPCAP_BUF_SIZE, repCapMarker),
        3, "Marker");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, deltaMarker, 1, markerCount),
            4, "Delta Marker");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_MARKER_LINK_TO_DELTA_MARKER, "DeltaMarker", deltaMarker - 1, RS_REPCAP_BUF_SIZE, repCapDeltaMarker),
        4, "Delta Marker");

    (void)snprintf(repCap, RS_REPCAP_BUF_SIZE, "%s,%s", repCapMarker, repCapDeltaMarker);

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, repCap, RSSPECAN_ATTR_MARKER_LINK_TO_DELTA_MARKER, link),
            4, "Link");

Error:
    rsspecan_SetAttributeViInt32(instrSession, NULL, RSSPECAN_ATTR_ACTIVE_WINDOW, oldWindow);
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN This function returns the horizontal position and the marker
/// HIFN amplitude level of the selected marker.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR marker/This control selects marker.
/// HIPAR markerPosition/The horizontal position of the selected marker.
/// HIPAR markerAmplitude/The amplitude value of the selected marker (Amplitude Units or
/// HIPAR markerAmplitude/relative amplitude units for delta markers).
ViStatus _VI_FUNC rsspecan_QueryMarker(ViSession instrSession,
                                       ViInt32   window,
                                       ViInt32   marker,
                                       ViReal64  *markerPosition,
                                       ViReal64  *markerAmplitude)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  old_win = 0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, window, 0, 16),
            2, "Window");

    checkErr(rsspecan_GetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_ACTIVE_WINDOW, &old_win));
    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_ACTIVE_WINDOW, window),
            2, "Window");

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_MARKER_POSITION,
            "Marker", marker - 1, RS_REPCAP_BUF_SIZE, repCap),
            3, "Marker");

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_MARKER_POSITION, markerPosition),
            4, "Marker Position");

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_MARKER_AMPLITUDE, markerAmplitude),
            5, "Marker Amplitude");

Error:
    rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_ACTIVE_WINDOW, old_win);
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN This function defines the range to be zoomed around marker 1
/// HIFN in the selected measurement window. Marker 1 is activated first,
/// HIFN if necessary.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR markerZoom/Defines the range to be zoomed around marker 1.
/// HIPAR timeout/This control sets the timeout value..
/// HIPAR window/This control selects the measurement window.
ViStatus _VI_FUNC rsspecan_ConfigureMarkerZoom(ViSession instrSession,
                                               ViInt32   window,
                                               ViReal64  markerZoom,
                                               ViUInt32  timeout)
{
    ViStatus error = VI_SUCCESS;
    ViUInt32 old_timeout = 0;
    ViInt32  old_win = 0;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsspecan_AssertFSV(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, window, 0, 16),
            2, "Window");

    checkErr(rsspecan_GetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_ACTIVE_WINDOW, &old_win));
    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_ACTIVE_WINDOW, window),
            2, "Window");
    checkErr(rsspecan_GetTimeout(instrSession, &old_timeout));
    checkErr(rsspecan_SetTimeout(instrSession, timeout));

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_MARKER_ZOOM, markerZoom),
            3, "Marker Zoom");

Error:
    rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_ACTIVE_WINDOW, old_win);
    if (old_timeout > 0)
        rsspecan_SetTimeout(instrSession, old_timeout);
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/*****************************************************************************
 *------------------- Marker -> Settings ------------------------------------*
 *****************************************************************************/
/// HIFN This function toggles the display of the marker table on and off.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR markerTable/This control toggles the display of the marker table.
ViStatus _VI_FUNC rsspecan_ConfigureDisplayMarkerTable(ViSession instrSession,
                                                       ViInt32   markerTable)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_DISP_MARKER_TABLE, markerTable),
            2, "Marker Table");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function defines the step size of the rotary knob for
/// HIFN marker or delta marker value changes.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR stepSize/Defines the step size.
ViStatus _VI_FUNC rsspecan_ConfigureMarkerStepSize(ViSession instrSession,
                                                   ViInt32   reserved,
                                                   ViInt32   stepSize)
{
    ViStatus error = VI_SUCCESS;
             reserved;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_MARKER_STEP_SIZE, stepSize),
            3, "Step Size");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function turns the marker information in all diagrams on and off.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR markerInfo/Turns the marker information in all diagrams on and off.
ViStatus _VI_FUNC rsspecan_ConfigureMarkerInfo(ViSession instrSession,
                                               ViBoolean markerInfo)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_DISPLAY_MARKER_INFO, markerInfo),
            2, "Marker Info");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/*****************************************************************************
 *------------------- Marker -> Search --------------------------------------*
 *****************************************************************************/
/// HIFN This function specifies the marker search parameters
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR localOscillatorSupression/ Switches the local oscillator suppression for peak search on or off.
/// HIPAR peakExcursion/Sets the minimum amplitude variation of the signal in dB that
/// HIPAR peakExcursion/the marker can identify as a peak.
ViStatus _VI_FUNC rsspecan_ConfigureMarkerSearch(ViSession instrSession,
                                                 ViBoolean localOscillatorSupression,
                                                 ViReal64  peakExcursion)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_MARKER_LOEX, localOscillatorSupression),
            2, "Local Oscillator Supression");

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_PEAK_EXCURSION, peakExcursion),
            3, "Peak Excursion");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function configures an automatic peak search action
/// HIFN for marker at the end of each particular sweep.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR marker/This control selects marker.
/// HIPAR markerSearch/Specifies the type of marker search type.
/// HIPAR autoPeak/Switches the peak auto search state.
ViStatus _VI_FUNC rsspecan_MarkerPeakSearchAuto(ViSession instrSession,
                                                ViInt32   reserved,
                                                ViInt32   marker,
                                                ViInt32   markerSearch,
                                                ViBoolean autoPeak)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
             reserved;

    checkErr(RsCore_LockSession(instrSession));

    switch (markerSearch)
    {
    case RSSPECAN_VAL_MARKER_SEARCH_HIGHEST:
        viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_MARKER_SEARCH_PEAK_AUTO,
                "Marker", marker - 1, RS_REPCAP_BUF_SIZE, repCap),
                3, "Marker");
        viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, repCap, RSSPECAN_ATTR_MARKER_SEARCH_PEAK_AUTO, autoPeak),
                4, "Auto Peak");
        break;
    case RSSPECAN_VAL_MARKER_SEARCH_MINIMUM:
        viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_MARKER_SEARCH_MIN_AUTO,
                "Marker", marker - 1, RS_REPCAP_BUF_SIZE, repCap),
                3, "Marker");
        viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, repCap, RSSPECAN_ATTR_MARKER_SEARCH_MIN_AUTO, autoPeak),
                4, "Auto Peak");
        break;
    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, markerSearch), 4, "Marker Search");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function configures the search area and search conditions.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR searchLimits/Switches between a limited (ON) and unlimited (OFF) search range
/// HIPAR searchLimitLeft/Sets the left limit of the search range for markers and delta
/// HIPAR searchLimitLeft/markers.
/// HIPAR searchLimitRight/Sets the right limit of the search range for markers and delta
/// HIPAR searchLimitRight/markers.
/// HIPAR thresholdState/Switches on or off the threshold line.
/// HIPAR markerThreshold/Sets the threshold value for the maximum/minimum search of
/// HIPAR markerThreshold/markers with marker search functions MAX PEAK, NEXT PEAK, etc.
/// HIPAR searchLimitsZoom/Sets the limits of the search range for markers and delta
/// HIPAR searchLimitsZoom/markers to the zoom area.
ViStatus _VI_FUNC rsspecan_ConfigMarkerSearchLimits(ViSession instrSession,
                                                    ViBoolean searchLimits,
                                                    ViReal64  searchLimitLeft,
                                                    ViReal64  searchLimitRight,
                                                    ViBoolean thresholdState,
                                                    ViReal64  markerThreshold,
                                                    ViBoolean searchLimitsZoom)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_MARKER_SEARCH_LIMITS_STATE, searchLimits),
            2, "Search Limit");
    if (searchLimits == VI_TRUE)
    {
        viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_MARKER_SEARCH_LIMITS_LEFT, searchLimitLeft),
                3, "Search Limit Left");

        viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_MARKER_SEARCH_LIMITS_RIGHT, searchLimitRight),
                4, "Search Limit Right");
    }

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_MARKER_THRESHOLD_STATE, thresholdState),
            5, "Threshold State");

    if (thresholdState == VI_TRUE)
    {
        viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_MARKER_THRESHOLD, markerThreshold),
                6, "Marker Threshold");
    }

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_MARKER_SEARCH_LIMITS_ZOOM, searchLimitsZoom),
            7, "Search Limits Zoom");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function deactivates the search range limits.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
ViStatus _VI_FUNC rsspecan_DeactivateAllSearchLimits(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_MARKER_SEARCH_LIMITS_STATE, VI_FALSE));
    checkErr(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_MARKER_THRESHOLD_STATE, VI_FALSE));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures frequency range in which spur tracking takes
/// HIFN  place.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR trackingState/This control turns spurious tracking on and off.
/// HIPAR frequencyRange/This control defines the frequency range in which spur tracking takes
/// HIPAR frequencyRange/place.
ViStatus _VI_FUNC rsspecan_ConfigureSpuriousTracking(ViSession instrSession,
                                                     ViBoolean trackingState,
                                                     ViReal64  frequencyRange)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_MARKER_SEARCH_SPURIOUS_TRACKING_STATE, trackingState),
            2, "Tracking State");

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_MARKER_SEARCH_SPURIOUS_TRACKING_FREQUENCY_RANGE, frequencyRange),
            3, "Frequency Range");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/*****************************************************************************
 *------------------- Marker -> Marker Functions ----------------------------*
 *****************************************************************************/

/// HIFN This function sets the marker frequency counter resolution and
/// HIFN turns the marker frequency counter on/off.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR marker/Selects the marker.
/// HIPAR markerFrequencyCounter/Specifies whether the frequency counter is enabled.
/// HIPAR frequencyCounterResolution/Specifies the frequency counter resolution
ViStatus _VI_FUNC rsspecan_ConfigureMarkerFrequencyCounter(ViSession instrSession,
                                                           ViInt32   reserved,
                                                           ViInt32   marker,
                                                           ViBoolean markerFrequencyCounter,
                                                           ViReal64  frequencyCounterResolution)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
             reserved;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_MARKER_FREQUENCY_COUNTER_ENABLED,
            "Marker", marker - 1, RS_REPCAP_BUF_SIZE, repCap),
            3, "Marker");

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, repCap, RSSPECAN_ATTR_MARKER_FREQUENCY_COUNTER_ENABLED, markerFrequencyCounter),
            4, "Marker Frequency Counter");

    if (markerFrequencyCounter == VI_TRUE)
    {
        viCheckParm(rsspecan_SetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_MARKER_FREQUENCY_COUNTER_RESOLUTION, frequencyCounterResolution),
                5, "Freqeuncy Counter Resolution");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN This function queries the result of the frequency counter for
/// HIFN the indicated marker.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR marker/Selects the marker.
/// HIPAR counterFrequency/Returns the result of the frequency counter for the indicated
/// HIPAR counterFrequency/marker.
ViStatus _VI_FUNC rsspecan_QueryMarkerFrequencyCounterResult(ViSession instrSession,
                                                             ViInt32   reserved,
                                                             ViInt32   marker,
                                                             ViReal64  *counterFrequency)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
             reserved;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_MARKER_COUNT,
            "Marker", marker - 1, RS_REPCAP_BUF_SIZE, repCap),
            3, "Marker");

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_MARKER_COUNT, counterFrequency),
            4, "Counter Freqeuncy");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function turns the noise measurement at the marker position on and off.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR marker/Selects the marker.
/// HIPAR state/This control turns the noise measurement at the marker position on and off.
ViStatus _VI_FUNC rsspecan_ConfigureMarkerNoiseMeasurement(ViSession instrSession,
                                                           ViInt32   reserved,
                                                           ViInt32   marker,
                                                           ViBoolean state)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
             reserved;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_MARKER_NOISE_STATE,
            "Marker", marker - 1, RS_REPCAP_BUF_SIZE, repCap),
            3, "Marker");

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, repCap, RSSPECAN_ATTR_MARKER_NOISE_STATE, state),
            4, "State");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function queries the result of the noise measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR marker/Selects the marker.
/// HIPAR result/This control returns the result of the noise measurement.
ViStatus _VI_FUNC rsspecan_QueryMarkerNoiseMeasurementResult(ViSession instrSession,
                                                             ViInt32   reserved,
                                                             ViInt32   marker,
                                                             ViReal64  *result)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
             reserved;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_MARKER_NOISE_RESULT,
            "Marker", marker - 1, RS_REPCAP_BUF_SIZE, repCap),
            3, "Marker");
    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_MARKER_NOISE_RESULT, result),
            4, "Result");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Removes all noise markers in the current window.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsspecan_MarkerNoiseMeasurementAllOff(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsspecan_SetAttributeViString(instrSession, NULL, RSSPECAN_ATTR_MARKER_NOISE_MEASUREMENT_ALL_OFF, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function configures phase noise marker measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR state/This control switches on or off the phase-noise measurement with
/// HIPAR state/all act�ve delta markers.
ViStatus _VI_FUNC rsspecan_ConfigureMarkerPhaseNoiseMeasurement(
                            ViSession instrSession,
                            ViInt32   reserved,
                            ViBoolean state)
{
    ViStatus error = VI_SUCCESS;
             reserved;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_REFERENCE_MARKER_PNO_STATE, state),
            3, "State");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function configures a fixed reference marker.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR level/Defines a new reference point level for delta marker 2.
/// HIPAR frequency/Defines a new reference frequency or time for delta marker 2.
ViStatus _VI_FUNC rsspecan_PhaseNoiseReferencePoint(ViSession instrSession,
                                                    ViReal64  level,
                                                    ViReal64  frequency)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_MARKER_REF_LEVEL, level),
            2, "Level");

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_MARKER_REF_FREQ, frequency),
            5, "Frequency");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function defines a new reference point level for delta
/// HIFN marker 2.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
ViStatus _VI_FUNC rsspecan_PhaseNoiseRefPointPeakSearch(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_MARKER_REF_PEAK, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN This function sests an automatic peak search action for the
/// HIFN reference fixed marker 1 at the end of each particular sweep.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR state/This control switches an automatic peak search action for the
/// HIPAR state/reference fixed marker 1 at the end of each particular sweep.
ViStatus _VI_FUNC rsspecan_PhaseNoisePeakSearchAuto(ViSession instrSession,
                                                    ViInt32   reserved,
                                                    ViBoolean state)
{
    ViStatus error = VI_SUCCESS;
             reserved;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_REFERENCE_MARKER_PNO_AUTO, state),
            3, "State");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function queries the result of the phase-noise measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR phaseNoiseResult/Returns the result of the phase-noise measurement.
ViStatus _VI_FUNC rsspecan_QueryPhaseNoiseResult(ViSession instrSession,
                                                 ViReal64  *phaseNoiseResult)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_REFERENCE_MARKER_PNO_RESULT, phaseNoiseResult),
            2, "Phase Noise Result");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Removes all phase noise markers in the current window.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsspecan_PhaseNoiseAllOff(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsspecan_SetAttributeViString(instrSession, NULL, RSSPECAN_ATTR_DELTA_MARKER_PHASE_NOISE_ALL_OFF, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function configures the N dB function.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR nDBState/Switches the "N dB Down" function on or off.
/// HIPAR nDBLevel/Defines the level spacing of the two delta markers to the right
/// HIPAR nDBLevel/and left of marker 1.
ViStatus _VI_FUNC rsspecan_ConfigureNdBPoints(ViSession instrSession,
                                              ViInt32   reserved,
                                              ViBoolean nDBState,
                                              ViReal64  nDBLevel)
{
    ViStatus error = VI_SUCCESS;
             reserved;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_MARKER_NDB_STATE, nDBState),
            3, "N dB State");

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_MARKER_NDB_VAL, nDBLevel),
            4, "N dB Level");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function queries the frequency spacing (bandwidth) of the N-dB-down
/// HIFN markers.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR nDBResult/Returns the frequency spacing (bandwidth) of the N-dB-down
/// HIPAR nDBResult/markers
ViStatus _VI_FUNC rsspecan_QueryNdBResult(ViSession instrSession,
                                          ViInt32   reserved,
                                          ViReal64  *nDBResult)
{
    ViStatus error = VI_SUCCESS;
             reserved;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_MARKER_NDB_RESULT, nDBResult),
            3, "N dB Result");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function queries the two frequencies of the N-dB-down
/// HIFN marker.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR nDBFrequencyLower/Returns the lower frequency of N dB down markers.
/// HIPAR nDBFrequencyHigher/Returns the higher frequency of N dB down markers.
ViStatus _VI_FUNC rsspecan_QueryNdBFrequencies(ViSession instrSession,
                                               ViInt32   reserved,
                                               ViReal64  *nDBFrequencyLower,
                                               ViReal64  *nDBFrequencyHigher)
{
    ViStatus error = VI_SUCCESS;
             reserved;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidNullPointer(instrSession, nDBFrequencyLower),
        3, "N dB Frequency Lower");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, nDBFrequencyHigher),
        4, "N dB Frequency Higher");

    checkErr(RsCore_QueryTupleViReal64(instrSession, "CALC:MARK:FUNC:NDBD:FREQ?", nDBFrequencyLower, nDBFrequencyHigher, NULL, NULL));

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN This function queries the two times of the N-dB-down marker.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR nDBTimeLower/Returns the lower frequency of N dB down markers.
/// HIPAR nDBTimeHigher/Returns the higher frequency of N dB down markers.
ViStatus _VI_FUNC rsspecan_QueryNdBTimes(ViSession instrSession,
                                         ViInt32   reserved,
                                         ViReal64  *nDBTimeLower,
                                         ViReal64  *nDBTimeHigher)
{
    ViStatus error = VI_SUCCESS;
             reserved;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidNullPointer(instrSession, nDBTimeLower),
        3, "N dB Frequency Lower");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, nDBTimeHigher),
        4, "N dB Frequency Higher");

    checkErr(RsCore_QueryTupleViReal64(instrSession, "CALC:MARK:FUNC:NDBD:TIME?", nDBTimeLower, nDBTimeHigher, NULL, NULL));

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function queries the Q factor (quality) of the measured
/// HIFN bandwidth for span>0.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR nDBQFactor/Returns the Q factor (quality) of the measured bandwidth for
/// HIPAR nDBQFactor/span>0.
ViStatus _VI_FUNC rsspecan_QueryNdBQFactor(ViSession instrSession,
                                           ViInt32   reserved,
                                           ViReal64  *nDBQFactor)
{
    ViStatus error = VI_SUCCESS;
             reserved;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_MARKER_NDB_Q_FACTOR, nDBQFactor),
            3, "N dB Q Factor");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN Configure a band power marker.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR displayMode/Activates the band power marker in the specified window.
/// HIPAR bandPowerEnabled/Defines whether the result is displayed as a power or density.
/// HIPAR span/Defines the span (band) around the marker for which the power is measured.

ViStatus _VI_FUNC rsspecan_ConfigureMarkerBandPower(ViSession instrSession,
                                                    ViBoolean bandPowerEnabled,
                                                    ViInt32   displayMode,
                                                    ViReal64  span)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_MARKER_BAND_POWER_STATE, bandPowerEnabled),
            2, "Band Power Enabled");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_MARKER_BAND_POWER_MODE, displayMode),
            3, "Display Mode");
    if (bandPowerEnabled)
    {
        viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_MARKER_BAND_POWER_SPAN, span),
                4, "Span");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function returns the value of the band power marker.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR result/Queries the value of the band power marker.

ViStatus _VI_FUNC rsspecan_QueryMarkerBandPower(ViSession instrSession,
                                                ViReal64  *result)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_QUERY_MARKER_BAND_POWER_RESULT, result),
            2, "Result");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Removes all band power markers in the current window.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsspecan_MarkerBandPowerAllOff(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsspecan_SetAttributeViString(instrSession, NULL, RSSPECAN_ATTR_MARKER_BAND_POWER_ALL_OFF, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function configures marker demodulation parameters.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR marker/This control selects marker.
/// HIPAR state/This control switches on or off the audio demodulator when the
/// HIPAR state/indicated marker is reached.
/// HIPAR demodulationType/Selects the demodulation type for the audio demodulator.
/// HIPAR markerStopTime/Defines the hold time at the marker position for the
/// HIPAR markerStopTime/demodulation in the frequency domain (span > 0).
/// HIPAR continuousDemodulation/This control switches on or off the continuous demodulation in
/// HIPAR continuousDemodulation/the frequency domain (span >0)
ViStatus _VI_FUNC rsspecan_ConfMarkerDemodulation(ViSession instrSession,
                                                  ViInt32   marker,
                                                  ViBoolean state,
                                                  ViInt32   demodulationType,
                                                  ViReal64  markerStopTime,
                                                  ViBoolean continuousDemodulation)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_MARKER_DEMOD_STATE,
            "Marker", marker - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Marker");

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, repCap, RSSPECAN_ATTR_MARKER_DEMOD_STATE, state),
            3, "State");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_MARKER_DEMOD_TYPE, demodulationType),
            4, "Demodulation Type");

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_MARKER_DEMOD_HOLDOFF, markerStopTime),
            5, "Marker Stop Time");

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, repCap, RSSPECAN_ATTR_MARKER_DEMOD_CONT, continuousDemodulation),
            6, "Continuous Demodulation");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures selective demodulation at the marker position.
/// HIFN     For selective demodulation, the instrument turns on a video trigger
/// HIFN  whose level correponds to the squelch level. Therefore it turns other
/// HIFN  triggers or gates off.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR selectiveDemodulationEnabled/Turns selective demodulation at the marker position on and off.
/// HIPAR demodulationThreshold/Defines the threshold for selective demodulation.
ViStatus _VI_FUNC rsspecan_ConfigureMarkerSelectiveDemodulation(
                            ViSession instrSession,
                            ViBoolean selectiveDemodulationEnabled,
                            ViReal64  demodulationThreshold)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, NULL, RSSPECAN_ATTR_MARKER_DEMODULATION_SELECTIVE_DEMODULATION_ENABLED, selectiveDemodulationEnabled),
            2, "Selective Demodulation Enabled");
    if (selectiveDemodulationEnabled)
    {
        viCheckParm(rsspecan_SetAttributeViReal64(instrSession, NULL, RSSPECAN_ATTR_MARKER_DEMODULATION_SELECTIVE_DEMODULATION_THRESHOLD, demodulationThreshold),
                3, "Demodulation Threshold");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function activates or deactivates the peak list.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR state/Activates or deactivates the peak list.
ViStatus _VI_FUNC rsspecan_ConfigureMarkerPeakListState(ViSession instrSession,
                                                        ViInt32   reserved,
                                                        ViInt32   marker,
                                                        ViBoolean state)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
             reserved;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_MARKER_PEAK_LIST_STATE,
            "Marker", marker - 1, RS_REPCAP_BUF_SIZE, repCap),
            3, "Marker");

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, repCap, RSSPECAN_ATTR_MARKER_PEAK_LIST_STATE, state),
            4, "State");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN This function configures marker peak list parameters.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR peakListSort/Sets the peak list sort mode.
/// HIPAR peakListSize/Sets the maximum length of the peak list.
/// HIPAR peakExcursion/Defines the minimum level value by which a signal must rise or fall so that it will be
/// HIPAR peakExcursion/identified as a maximum or a minimum by the search functions.
/// HIPAR displayMarkerNumbers/Activates or deactivates the peak list marker label..
ViStatus _VI_FUNC rsspecan_ConfMarkerPeakList(ViSession instrSession,
                                              ViInt32   marker,
                                              ViInt32   peakListSort,
                                              ViInt32   peakListSize,
                                              ViReal64  peakExcursion,
                                              ViBoolean displayMarkerNumbers)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_MARKER_PEAK_LIST_SORT,
            "Marker", marker - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Marker");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_MARKER_PEAK_LIST_SORT, peakListSort),
            3, "Peak List Sort");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_MARKER_PEAK_LIST_SIZE, peakListSize),
            4, "Peak List Size");

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_PEAK_EXCURSION, peakExcursion),
            5, "Peak Excursion");

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, repCap, RSSPECAN_ATTR_MARKER_PEAK_LIST_ANNOTATION_LABEL, displayMarkerNumbers),
            6, "Display Marker Numbers");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function initiates a peak search.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR peakListCount/This parameter defines the number of peaks to find during the
/// HIPAR peakListCount/search.
ViStatus _VI_FUNC rsspecan_MarkerPeakListSearch(ViSession instrSession,
                                                ViInt32   marker,
                                                ViInt32   peakListCount)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_MARKER_PEAK_LIST_COUNT,
            "Marker", marker - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Marker");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_MARKER_PEAK_LIST_COUNT, peakListCount),
            3, "Peak List Count");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function stores the current marker peak list in a *.dat
/// HIFN file. The file consists of a data section containing the peak
/// HIFN list.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR path/This control specifies the file path and name
ViStatus _VI_FUNC rsspecan_StoreMarkerPeakList(ViSession instrSession,
                                               ViString  path)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_MARKER_STORE_PEAK_LIST, path),
            2, "Path");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function reads out the number of maxima found during the
/// HIFN search. If no search for maxima has been performed, 0 is
/// HIFN returned.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR marker/This control selects marker.
/// HIPAR peakListFound/This control returns the number of maxima found during the
/// HIPAR peakListFound/search.
ViStatus _VI_FUNC rsspecan_QueryMarkerPeakListFound(ViSession instrSession,
                                                    ViInt32   reserved,
                                                    ViInt32   marker,
                                                    ViInt32   *peakListFound)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
             reserved;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_MARKER_PEAK_LIST_FOUND,
            "Marker", marker - 1, RS_REPCAP_BUF_SIZE, repCap),
            3, "Marker");

    viCheckParm(rsspecan_GetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_MARKER_PEAK_LIST_FOUND, peakListFound),
            4, "Peak List Found");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function reads out the list of X or Y values of the maxima
/// HIFN found.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR marker/This control selects marker.
/// HIPAR arraySize/This control defines the size of peak list array. It should
/// HIPAR arraySize/correspond with requested peak list count.
/// HIPAR peakListSelection/Selects the peak list values.
/// HIPAR peakList/This control returns the list of X values of the maxima found.
ViStatus _VI_FUNC rsspecan_QueryMarkerPeakList(ViSession instrSession,
                                               ViInt32   reserved,
                                               ViInt32   marker,
                                               ViInt32   arraySize,
                                               ViInt32   peakListSelection,
                                               ViReal64  peakList[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
             reserved;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, marker, 1, 16),
            3, "Marker");

    switch (peakListSelection)
    {
    case RSSPECAN_VAL_MARKER_SORT_X:
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CALC:MARK%ld:FUNC:FPE:X?", marker);
        break;
    case RSSPECAN_VAL_MARKER_SORT_Y:
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CALC:MARK%ld:FUNC:FPE:Y?", marker);
        break;
    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, peakListSelection), 5, "Peak List Selection");
        break;
    }

    checkErr(RsCore_QueryFloatArrayToUserBuffer(instrSession, cmd, arraySize, peakList, NULL));

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/*****************************************************************************
 *------------------- Delta Marker  -----------------------------------------*
 *****************************************************************************/
/// HIFN This function enables the active deltamarker on the specified
/// HIFN trace.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR deltaMarker/This control selects delta marker.
/// HIPAR state/Specifies whether the specified marker is enabled.
/// HIPAR trace/Selects the trace for Detector definition.
ViStatus _VI_FUNC rsspecan_ConfigureDeltaMarker(ViSession instrSession,
                                                ViInt32   window,
                                                ViInt32   deltaMarker,
                                                ViBoolean state,
                                                ViInt32   trace)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  window_old = 0;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsspecan_GetAttributeViInt32(instrSession, NULL, RSSPECAN_ATTR_ACTIVE_WINDOW, &window_old));
    checkErr(rsspecan_SetAttributeViInt32(instrSession, NULL, RSSPECAN_ATTR_ACTIVE_WINDOW, window));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_REFERENCE_MARKER_STATE,
            "Marker", deltaMarker - 1, RS_REPCAP_BUF_SIZE, repCap),
            3, "Delta Marker");

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, repCap, RSSPECAN_ATTR_REFERENCE_MARKER_STATE, state),
            4, "State");

    if (state == VI_TRUE)
    {
        viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_REFERENCE_MARKER_TRACE, trace),
                5, "Trace");
    }

Error:
    checkErr(rsspecan_SetAttributeViInt32(instrSession, NULL, RSSPECAN_ATTR_ACTIVE_WINDOW, window_old));

    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN Defines a marker as the reference marker which is used to
/// HIFN determine relative analysis results (delta marker values).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR marker/Selects the marker.
/// HIPAR referenceMarker/Defines the reference marker for the selected marker.
ViStatus _VI_FUNC rsspecan_ConfigureReferenceMarker(ViSession instrSession,
                                                    ViInt32   reserved,
                                                    ViInt32   marker,
                                                    ViInt32   referenceMarker)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
             reserved;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_DELTA_MARKER_REFERENCE_MARKER,
            "Marker", marker - 1, RS_REPCAP_BUF_SIZE, repCap),
            3, "Marker");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_DELTA_MARKER_REFERENCE_MARKER, referenceMarker),
            4, "Reference Marker");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN This function turns off all the delta markers.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
ViStatus _VI_FUNC rsspecan_DisableAllDeltaMarkers(ViSession instrSession,
                                                  ViInt32   reserved)
{
    ViStatus error = VI_SUCCESS;
             reserved;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_REFERENCE_MARKER_AOFF, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function specifies the type of delta marker search and
/// HIFN performs the search.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR deltaMarker/This control selects delta marker.
/// HIPAR markerSearch/Specifies the type of marker search type.
ViStatus _VI_FUNC rsspecan_DeltaMarkerSearch(ViSession instrSession,
                                             ViInt32   reserved,
                                             ViInt32   deltaMarker,
                                             ViInt32   markerSearch)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
             reserved;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_REFERENCE_MARKER_PEAK,
            "Marker", deltaMarker - 1, RS_REPCAP_BUF_SIZE, repCap),
            3, "Delta Marker");
    switch (markerSearch)
    {
    case RSSPECAN_VAL_MARKER_SEARCH_HIGHEST:
        checkErr(rsspecan_SetAttributeViString(instrSession, repCap, RSSPECAN_ATTR_REFERENCE_MARKER_PEAK, NULL));
        break;
    case RSSPECAN_VAL_MARKER_SEARCH_NEXT_PEAK:
        checkErr(rsspecan_SetAttributeViString(instrSession, repCap, RSSPECAN_ATTR_REFERENCE_MARKER_PEAK_NEXT, NULL));
        break;

    case RSSPECAN_VAL_MARKER_SEARCH_NEXT_PEAK_LEFT:
        checkErr(rsspecan_SetAttributeViString(instrSession, repCap, RSSPECAN_ATTR_REFERENCE_MARKER_PEAK_NEXT_LEFT, NULL));
        break;
    case RSSPECAN_VAL_MARKER_SEARCH_NEXT_PEAK_RIGHT:
        checkErr(rsspecan_SetAttributeViString(instrSession, repCap, RSSPECAN_ATTR_REFERENCE_MARKER_PEAK_NEXT_RIGHT, NULL));
        break;

    case RSSPECAN_VAL_MARKER_SEARCH_MINIMUM:
        checkErr(rsspecan_SetAttributeViString(instrSession, repCap, RSSPECAN_ATTR_REFERENCE_MARKER_MIN, NULL));
        break;
    case RSSPECAN_VAL_MARKER_SEARCH_NEXT_MINIMUM:
        checkErr(rsspecan_SetAttributeViString(instrSession, repCap, RSSPECAN_ATTR_REFERENCE_MARKER_MIN_NEXT, NULL));
        break;
    case RSSPECAN_VAL_MARKER_SEARCH_NEXT_MINIMUM_LEFT:
        checkErr(rsspecan_SetAttributeViString(instrSession, repCap, RSSPECAN_ATTR_REFERENCE_MARKER_MIN_NEXT_LEFT, NULL));
        break;
    case RSSPECAN_VAL_MARKER_SEARCH_NEXT_MINIMUM_RIGHT:
        checkErr(rsspecan_SetAttributeViString(instrSession, repCap, RSSPECAN_ATTR_REFERENCE_MARKER_MIN_NEXT_RIGHT, NULL));
        break;

    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, markerSearch), 4, "Marker Search");
        break;
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function configures the delta marker position.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR deltaMarker/This control selects delta marker.
/// HIPAR mode/Switches between relative and absolute frequency input of the
/// HIPAR mode/delta marker (or time with span = 0).
/// HIPAR position/Sets the position of selected delta marker.
ViStatus _VI_FUNC rsspecan_ConfigureDeltaMarkerPosition(ViSession instrSession,
                                                        ViInt32   window,
                                                        ViInt32   deltaMarker,
                                                        ViInt32   mode,
                                                        ViReal64  position)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  window_old = 0;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsspecan_GetAttributeViInt32(instrSession, NULL, RSSPECAN_ATTR_ACTIVE_WINDOW, &window_old));
    checkErr(rsspecan_SetAttributeViInt32(instrSession, NULL, RSSPECAN_ATTR_ACTIVE_WINDOW, window));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_REFERENCE_MARKER_MODE,
            "Marker", deltaMarker - 1, RS_REPCAP_BUF_SIZE, repCap),
            3, "Delta Marker");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_REFERENCE_MARKER_MODE, mode),
            4, "Mode");

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_REFERENCE_MARKER_POSITION, position),
            5, "Position");

Error:
    checkErr(rsspecan_SetAttributeViInt32(instrSession, NULL, RSSPECAN_ATTR_ACTIVE_WINDOW, window_old));

    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function links delta marker to any active normal marker
/// HIFN You may use multiple lines for documentation.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR deltaMarker/his control selects delta marker.
/// HIPAR secondMarker/This control selects second marker.
/// HIPAR state/Specifies the marker link state.
ViStatus _VI_FUNC rsspecan_LinkingDeltaMarkerToMarker(ViSession instrSession,
                                                      ViInt32   deltaMarker,
                                                      ViInt32   secondMarker,
                                                      ViBoolean state)
{
    ViStatus error = VI_SUCCESS;
    ViChar   marker1[10] = "";
    ViChar   marker2[10] = "";
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsspecan_AssertNotFSV(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession,
                                                 RSSPECAN_ATTR_DELTA_MARKER_LINK_TO_MARKER,
                                                 "DeltaMarker",
                                                 deltaMarker - 1,
                                                 10,
                                                 marker1));
    checkErr(RsCore_GetAttributeRepCapName(instrSession,
                                                 RSSPECAN_ATTR_DELTA_MARKER_LINK_TO_MARKER,
                                                 "Marker",
                                                 secondMarker - 1,
                                                 10,
                                                 marker2));
    snprintf(repCap, RS_REPCAP_BUF_SIZE, "%s,%s", marker1, marker2);
    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, repCap, RSSPECAN_ATTR_DELTA_MARKER_LINK_TO_MARKER, state),
            3, "State");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Links the delta source marker to any active destination delta marker.
/// HIFN
/// HIFN     If you change the horizontal position of the destination marker, the
/// HIFN  source marker changes its horizontal position to the same value.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR deltaMarker/This control selects delta marker.
/// HIPAR secondMarker/This control selects second marker.
/// HIPAR link/Links the delta source marker to any active destination delta marker.
ViStatus _VI_FUNC rsspecan_LinkingDeltaMarkerToAnotherDeltaMarker(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   deltaMarker,
                            ViInt32   secondMarker,
                            ViBoolean link)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCapMarker[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCapDeltaMarker[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  markerCount = 0;
    ViInt32  oldWindow = 0;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsspecan_GetAttributeViInt32(instrSession, NULL, RSSPECAN_ATTR_ACTIVE_WINDOW, &oldWindow));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_ACTIVE_WINDOW, window),
            2, "Window");

    checkErr(RsCore_GetRepCapIdItemsCount(instrSession, "Marker", &markerCount));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, deltaMarker, 1, markerCount),
            3, "Delta Marker");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_DELTA_MARKER_LINK_TO_ANOTHER_DELTA_MARKER, "Marker", deltaMarker - 1, RS_REPCAP_BUF_SIZE, repCapMarker),
        3, "Delta Marker");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, secondMarker, 1, markerCount),
            4, "Second Marker");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_DELTA_MARKER_LINK_TO_ANOTHER_DELTA_MARKER, "DeltaMarker", secondMarker - 1, RS_REPCAP_BUF_SIZE, repCapDeltaMarker),
        4, "Second Marker");

    (void)snprintf(repCap, RS_REPCAP_BUF_SIZE, "%s,%s", repCapMarker, repCapDeltaMarker);

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, repCap, RSSPECAN_ATTR_DELTA_MARKER_LINK_TO_ANOTHER_DELTA_MARKER, link),
            4, "Link");

Error:
    rsspecan_SetAttributeViInt32(instrSession, NULL, RSSPECAN_ATTR_ACTIVE_WINDOW, oldWindow);
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function returns the horizontal position and the marker
/// HIFN amplitude level of the selected delta marker.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR deltaMarker/This control selects delta marker.
/// HIPAR mode/Switches between absolute and relative position value.
/// HIPAR position/Returns the absolute or relative position of selected delta
/// HIPAR position/marker.
/// HIPAR amplitude/Returns the measured value of the selected delta marker.
ViStatus _VI_FUNC rsspecan_QueryDeltaMarker(ViSession instrSession,
                                            ViInt32   window,
                                            ViInt32   deltaMarker,
                                            ViInt32   mode,
                                            ViReal64  *position,
                                            ViReal64  *amplitude)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  window_old = 0;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsspecan_GetAttributeViInt32(instrSession, NULL, RSSPECAN_ATTR_ACTIVE_WINDOW, &window_old));
    checkErr(rsspecan_SetAttributeViInt32(instrSession, NULL, RSSPECAN_ATTR_ACTIVE_WINDOW, window));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_REFERENCE_MARKER_MODE,
            "Marker", deltaMarker - 1, RS_REPCAP_BUF_SIZE, repCap),
            3, "Delta Marker");
    switch (mode)
    {
    case RSSPECAN_VAL_ABS:
        viCheckParm(rsspecan_GetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_REFERENCE_MARKER_POSITION, position),
                5, "Position");
        break;
    case RSSPECAN_VAL_REL:
        viCheckParm(rsspecan_GetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_REFERENCE_MARKER_REL_POSITION, position),
                5, "Position");
        break;
    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, mode), 4, "Mode");
    }

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_REFERENCE_MARKER_AMPLITUDE, amplitude),
            6, "Amplitude");

Error:
    checkErr(rsspecan_SetAttributeViInt32(instrSession, NULL, RSSPECAN_ATTR_ACTIVE_WINDOW, window_old));

    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function configures fixed reference marker.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR fixedReference/This control activates or deactivates a marker that defines a
/// HIPAR fixedReference/fixed reference point for relative marker analysis.
ViStatus _VI_FUNC rsspecan_ConfigureReferenceFixed(ViSession instrSession,
                                                   ViInt32   window,
                                                   ViBoolean fixedReference)
{
    ViStatus error = VI_SUCCESS;
             window;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "M1", RSSPECAN_ATTR_MARKER_REF_STATE, fixedReference),
            3, "Fixed Reference");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function configures fixed reference marker.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR level/Defines the vertical position of the fixed delta marker
/// HIPAR level/reference point.
/// HIPAR levelOffset/Defines defines a level offset for the fixed delta marker
/// HIPAR levelOffset/reference point.
/// HIPAR frequency/Defines the horizontal position of the fixed delta marker
/// HIPAR frequency/reference point.
ViStatus _VI_FUNC rsspecan_ConfigureReferenceFixedPoint(ViSession instrSession,
                                                        ViInt32   window,
                                                        ViReal64  refPointLevel,
                                                        ViReal64  refPointLevelOffset,
                                                        ViReal64  refPointFrequencyTime)
{
    ViStatus error = VI_SUCCESS;
             window;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "M1", RSSPECAN_ATTR_MARKER_REF_LEVEL_OFFSET, refPointLevelOffset),
            4, "Level Offset");

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "M1", RSSPECAN_ATTR_MARKER_REF_LEVEL, refPointLevel),
            3, "Level");

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "M1", RSSPECAN_ATTR_MARKER_REF_FREQ, refPointFrequencyTime),
            5, "Reference Point Frequency");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function moves the fixed reference marker to the peak power.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
ViStatus _VI_FUNC rsspecan_ReferenceFixedPeakSearch(ViSession instrSession,
                                                    ViInt32   reserved)
{
    ViStatus error = VI_SUCCESS;
             reserved;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_MARKER_REF_PEAK, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/*****************************************************************************
 *------------------- Delta Marker -> Marker Functions ----------------------*
 *****************************************************************************/

/// HIFN  Configure a band power marker.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR displayMode/Activates the band power marker
/// HIPAR bandPowerEnabled/efines whether the result is displayed as a power or density.
/// HIPAR span/Defines the span (band) around the marker for which the power is measured.
ViStatus _VI_FUNC rsspecan_ConfigureDeltaMarkerBandPower(ViSession instrSession,
                                                         ViBoolean bandPowerEnabled,
                                                         ViInt32   displayMode,
                                                         ViReal64  span)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_DELTA_MARKER_BAND_POWER_STATE, bandPowerEnabled),
            2, "Band Power Enabled");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_DELTA_MARKER_BAND_POWER_MODE, displayMode),
            3, "Display Mode");
    if (bandPowerEnabled)
    {
        viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_DELTA_MARKER_BAND_POWER_SPAN, span),
                4, "Span");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function queries the value of the band power marker.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR result/Queries the value of the band power marker.

ViStatus _VI_FUNC rsspecan_QueryDeltaMarkerBandPower(ViSession instrSession,
                                                     ViReal64  *result)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_QUERY_DELTA_MARKER_BAND_POWER_RESULT, result),
            2, "Result");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/**************************************************************************
 * ------------------- Limit Lines ---------------------------------------*
 **************************************************************************/
/// HIFN This function configures the specified display line.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR line/This control selects line.
/// HIPAR type/Selects the line to be shown.
/// HIPAR state/Specifies whether selected line is enabled.
/// HIPAR position/Defines position of selected line.
ViStatus _VI_FUNC rsspecan_ConfigureDisplayLine(ViSession instrSession,
                                                ViInt32   line,
                                                ViInt32   type,
                                                ViBoolean state,
                                                ViReal64  position)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    switch (type)
    {
    case RSSPECAN_VAL_LINE_THR:
        viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_THRLINE_STATE, state),
                4, "State");
        if (state == VI_TRUE)
        viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_THRLINE_POSITION, position),
                5, "Position");
        break;
    case RSSPECAN_VAL_LINE_DISPLAY:
        checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_DLINE_STATE,
                "Line", line - 1, RS_REPCAP_BUF_SIZE, repCap));
        viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, repCap, RSSPECAN_ATTR_DLINE_STATE, state),
                4, "State");
        if (state == VI_TRUE)
        viCheckParm(rsspecan_SetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_DLINE_POSITION, position),
                5, "Position");
        break;
    case RSSPECAN_VAL_LINE_FREQ:
        checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_FLINE_STATE,
                "Line", line - 1, RS_REPCAP_BUF_SIZE, repCap));
        viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, repCap, RSSPECAN_ATTR_FLINE_STATE, state),
                4, "State");
        if (state == VI_TRUE)
        viCheckParm(rsspecan_SetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_FLINE_POSITION, position),
                5, "Position");
        break;
    case RSSPECAN_VAL_LINE_TIME:
        checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_TLINE_STATE,
                "Line", line - 1, RS_REPCAP_BUF_SIZE, repCap));
        viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, repCap, RSSPECAN_ATTR_TLINE_STATE, state),
                4, "State");
        if (state == VI_TRUE)
        viCheckParm(rsspecan_SetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_TLINE_POSITION, position),
                5, "Position");
        break;
    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, type), 4, "Type");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function creates new limit or selects existing one and
/// HIFN configures basic parameters.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR limit/This control selects limit line.
/// HIPAR name/This control sets the limit line name.
/// HIPAR domain/This control defines the frequency or time domain for the x axis
/// HIPAR domain/values.
/// HIPAR comment/This control sets the limit line comment.
/// HIPAR assignToTrace/Assigns a limit line to a trace.
/// HIPAR deleteExistingLimitLine/Removes specified limit line before new is created.
ViStatus _VI_FUNC rsspecan_CreateLimitLine(ViSession instrSession,
                                           ViInt32   reserved,
                                           ViInt32   limit,
                                           ViString  name,
                                           ViInt32   domain,
                                           ViString  comment,
                                           ViInt32   assignToTrace,
                                           ViBoolean deleteExistingLimitLine)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
             reserved;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_LIMIT_NAME,
            "Limit", limit - 1, RS_REPCAP_BUF_SIZE, repCap),
            3, "Limit");
    viCheckParm(RsCore_InvalidStringLength(instrSession, name, 0, 8),
        4, "Name (string length)");
    viCheckParm(RsCore_InvalidStringLength(instrSession, comment, 0, 40),
        6, "Comment (string length)");

    if (deleteExistingLimitLine)
        checkErr(rsspecan_SetAttributeViString(instrSession, repCap, RSSPECAN_LIMIT_DELETE, ""));

    viCheckParm(rsspecan_SetAttributeViString(instrSession, repCap, RSSPECAN_LIMIT_NAME, name),
            4, "Name");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_LIMIT_CONTROL_DOMAIN, domain),
            5, "Domain");

    viCheckParm(rsspecan_SetAttributeViString(instrSession, repCap, RSSPECAN_LIMIT_COMMENT, comment),
            6, "Comment");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_LIMIT_ASSIGN_TRACE, assignToTrace),
            7, "Assign to Trace");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function configures specified limit line.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR limit/This control selects limit..
/// HIPAR type/Specifies a limit line to be either an upper or lower type limit
/// HIPAR type/line.
/// HIPAR units/This control defines the unit of the selected limit line.
/// HIPAR xAxisInterpolation/Selects linear or logarithmic interpolation for the calculation
/// HIPAR xAxisInterpolation/of limit lines from frequency points.
/// HIPAR yAxisInterpolation/Selects linear or logarithmic interpolation for the upper or
/// HIPAR yAxisInterpolation/lower limit line.
/// HIPAR xAxisScaling/Selects the relative or absolute scaling for the X axis of the
/// HIPAR xAxisScaling/selected limit line.
/// HIPAR margin/Defines a margin for the limit line.
/// HIPAR threshold/Defines an absolute threshold value for limit lines with
/// HIPAR threshold/relative Y axis scaling.
ViStatus _VI_FUNC rsspecan_ConfigureLimitLine(ViSession instrSession,
                                              ViInt32   limit,
                                              ViInt32   type,
                                              ViInt32   units,
                                              ViInt32   xAxisInterpolation,
                                              ViInt32   yAxisInterpolation,
                                              ViInt32   xAxisScaling,
                                              ViReal64  margin,
                                              ViReal64  threshold)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  scaling;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_LIMIT_UNITS,
            "Limit", limit - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Limit");

    scaling = (units == RSSPECAN_VAL_UNIT_DB) ? RSSPECAN_VAL_REL : RSSPECAN_VAL_ABS;

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_LIMIT_CONTROL_SPACING, xAxisInterpolation),
            5, "X Axis Interpolation");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_LIMIT_CONTROL_SCALING, xAxisScaling),
            7, "X Axis Scaling");

    switch (type)
    {
    case RSSPECAN_VAL_LIMIT_LOWER:
        viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_LIMIT_LOWER_SPACING, yAxisInterpolation),
                6, "Y Axis Interpolation");

        viCheckParm(rsspecan_SetAttributeViReal64(instrSession, repCap, RSSPECAN_LIMIT_LOWER_MARGIN, margin),
                8, "Marging");
        if (scaling == RSSPECAN_VAL_REL)
        {
            viCheckParm(rsspecan_SetAttributeViReal64(instrSession, repCap, RSSPECAN_LIMIT_LOWER_THRESHOLD, threshold),
                    9, "Threshold");
        }
        //checkErr(rsspecan_SetAttributeViInt32(instrSession, buffer, RSSPECAN_LIMIT_LOWER_SCALING, scaling));
        break;

    case RSSPECAN_VAL_LIMIT_UPPER:
        viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_LIMIT_UPPER_SPACING, yAxisInterpolation),
                6, "Y Axis Interpolation");

        viCheckParm(rsspecan_SetAttributeViReal64(instrSession, repCap, RSSPECAN_LIMIT_UPPER_MARGIN, margin),
                8, "Marging");
        if (scaling == RSSPECAN_VAL_REL)
        {
            viCheckParm(rsspecan_SetAttributeViReal64(instrSession, repCap, RSSPECAN_LIMIT_UPPER_THRESHOLD, threshold),
                    9, "Threshold");
        }
        //checkErr(rsspecan_SetAttributeViInt32(instrSession, buffer, RSSPECAN_LIMIT_UPPER_SCALING, scaling));
        break;
    }

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_LIMIT_UNITS, units),
            4, "Units");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function defines specified limit line.
/// HIFN You may use multiple lines for documentation.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR limit/This control selects limit.
/// HIPAR type/Specifies a limit line to be either an upper or lower type limit
/// HIPAR type/line.
/// HIPAR count/Specifies number of values in X Axis and Amplitude arrays.
/// HIPAR xAxis/Specifies the frequency or time value for a limit point.
/// HIPAR amplitude/Specifies the amplitude value for the current limit point.
ViStatus _VI_FUNC rsspecan_DefineLimitLine(ViSession instrSession,
                                           ViInt32   limit,
                                           ViInt32   type,
                                           ViInt32   count,
                                           ViReal64  xAxis[],
                                           ViReal64  amplitude[])
{
    ViStatus error = VI_SUCCESS;
    ViString limTypeArr[] = {"LOW", "UPP", NULL};
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, limit, 1, 8),
            2, "Limit");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, type, 0, 1),
            3, "Type");
    viCheckParm(RsCore_InvalidViInt32RangeMin(instrSession, count, 1),
        4, "Count");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, xAxis),
        5, "X Axis");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, amplitude),
        6, "Amplitude");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CALC:LIM%ld:CONT ", limit);
    checkErr(RsCore_WriteAsciiViReal64Array(instrSession, cmd, xAxis, count));
    checkErr(rsspecan_CheckStatus(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CALC:LIM%ld:%s ", limit, limTypeArr[type]);
    checkErr(RsCore_WriteAsciiViReal64Array(instrSession, cmd, amplitude, count));
    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Turns the limit check for a specific trace on and off.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR limit/This control selects limit line.
/// HIPAR trace/Selects a trace.
/// HIPAR checkTrace/Turns the limit check for a specific trace on and off.
ViStatus _VI_FUNC rsspecan_ConfigureLimitLineCheckTrace(ViSession instrSession,
                                                        ViInt32   limit,
                                                        ViInt32   trace,
                                                        ViBoolean checkTrace)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_trace[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_LIMIT_LINE_TRACES_TO_BE_CHECKED,
            "Limit", limit - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Limit");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_LIMIT_LINE_TRACES_TO_BE_CHECKED,
            "Trace", trace - 1, RS_REPCAP_BUF_SIZE, repCap_trace),
            3, "Trace");

    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_trace, NULL);

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, repCap, RSSPECAN_ATTR_LIMIT_LINE_TRACES_TO_BE_CHECKED, checkTrace),
            4, "Check Trace");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function selects the limit line.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR limit/This control selects limit.
/// HIPAR name/This control sets the limit line name.
/// HIPAR limitEnabled/Turns the limit line display on and off.
ViStatus _VI_FUNC rsspecan_SelectLimitLine(ViSession instrSession,
                                           ViInt32   reserved,
                                           ViInt32   limit,
                                           ViString  name,
                                           ViBoolean limitEnabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
             reserved;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_LIMIT_NAME,
            "Limit", limit - 1, RS_REPCAP_BUF_SIZE, repCap),
            3, "Limit");
    if (name != NULL)
        viCheckParm(RsCore_InvalidStringLength(instrSession, name, 0, 8),
            4, "Name (string length)");

    if (name != NULL)
        viCheckParm(rsspecan_SetAttributeViString(instrSession, repCap, RSSPECAN_LIMIT_NAME, name),
                4, "Name");

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, repCap, RSSPECAN_LIMIT_STATE, limitEnabled),
            5, "Limit Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function switches on or off the limit check and/or limit
/// HIFN line for the selected limit line.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR limit/This control selects limit.
/// HIPAR type/Specifies a limit line to be either an upper or lower type limit
/// HIPAR type/line.
/// HIPAR limitEnabled/Turns the limit line display on and off.
/// HIPAR checkEnabled/Turns limit line testing on/off.
ViStatus _VI_FUNC rsspecan_EnableLimitCheck(ViSession instrSession,
                                            ViInt32   reserved,
                                            ViInt32   limit,
                                            ViInt32   type,
                                            ViBoolean limitEnabled,
                                            ViBoolean checkEnabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
             reserved;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_LIMIT_STATE,
            "Limit", limit - 1, RS_REPCAP_BUF_SIZE, repCap),
            3, "Limit");

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, repCap, RSSPECAN_LIMIT_STATE, limitEnabled),
            5, "Limit Enabled");

    switch (type)
    {
    case RSSPECAN_VAL_LIMIT_LOWER:
        viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, repCap, RSSPECAN_LIMIT_LOWER_STATE, checkEnabled),
                6, "Check Enabled");
        break;
    case RSSPECAN_VAL_LIMIT_UPPER:
        viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, repCap, RSSPECAN_LIMIT_UPPER_STATE, checkEnabled),
                6, "Check Enabled");
        break;
    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, type), 4, "Type");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function moves specified limit line.
/// HIRET Document return value here.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR limit/This control selects limit.
/// HIPAR type/Specifies a limit line to be either an upper or lower type limit
/// HIPAR type/line.
/// HIPAR method/This control selects the method of moving.
/// HIPAR offsetValue/Defines the move value.
ViStatus _VI_FUNC rsspecan_MoveLimitLine(ViSession instrSession,
                                         ViInt32   limit,
                                         ViInt32   type,
                                         ViInt32   method,
                                         ViReal64  value)
{
    ViStatus error = VI_SUCCESS;
    ViInt32  attributes[3][2] = {
             {RSSPECAN_LIMIT_LOWER_OFFSET,RSSPECAN_LIMIT_LOWER_SHIFT},
             {RSSPECAN_LIMIT_UPPER_OFFSET,RSSPECAN_LIMIT_UPPER_SHIFT},
             {RSSPECAN_LIMIT_CONTROL_OFFSET,RSSPECAN_LIMIT_CONTROL_SHIFT}
             };
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_LIMIT_LOWER_OFFSET,
            "Limit", limit - 1, RS_REPCAP_BUF_SIZE, repCap));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, type, 0, 2),
            3, "Type");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, method, 0, 1),
            4, "Method");

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, repCap, (ViUInt32)attributes[type][method], value), 5, "Value");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function copies specified limit line into another one.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR limit/This control selects limit.
/// HIPAR copyTo/This control selects target limit.
ViStatus _VI_FUNC rsspecan_CopyLimitLine(ViSession instrSession,
                                         ViInt32   limit,
                                         ViInt32   copyTo)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_LIMIT_COPY,
            "Limit", limit - 1, RS_REPCAP_BUF_SIZE, repCap));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_LIMIT_COPY, copyTo),
            3, "Copy To");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN This function removes specified limit line.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR limit/This control selects limit.
ViStatus _VI_FUNC rsspecan_DeleteLimitLine(ViSession instrSession,
                                           ViInt32   limit)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_LIMIT_DELETE,
            "Limit", limit - 1, RS_REPCAP_BUF_SIZE, repCap));

    checkErr(rsspecan_SetAttributeViString(instrSession, repCap, RSSPECAN_LIMIT_DELETE, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function deletes the result of the current limit check.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR limit/This control selects limit.
ViStatus _VI_FUNC rsspecan_ClearLimitLineResults(ViSession instrSession,
                                                 ViInt32   reserved,
                                                 ViInt32   limit)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
             reserved;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_LIMIT_CLEAR,
            "Limit", limit - 1, RS_REPCAP_BUF_SIZE, repCap),
            3, "Limit");

    checkErr(rsspecan_SetAttributeViString(instrSession, repCap, RSSPECAN_LIMIT_CLEAR, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function queries the names of all active limit lines.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR stringSize/This control sets the allocated length of Active Limit Lines
/// HIPAR stringSize/string.
/// HIPAR activeLimitLines/This control returns the control lines of the user ports.
/// HIPAR returnedStringLength/Returns the length of string returned from the instrument.
ViStatus _VI_FUNC rsspecan_GetActiveLimitLines(ViSession instrSession,
                                               ViInt32   stringSize,
                                               ViChar    activeLimitLines[],
                                               ViInt32   *returnedStringLength)
{
    ViStatus error = VI_SUCCESS;
    ViChar   *pbuffer = NULL;
    ViInt32  retCnt;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidNullPointer(instrSession, activeLimitLines),
        3, "Active Limit Lines");

    checkErr(RsCore_QueryViStringUnknownLength(instrSession, ":CALC:LIM:ACT?", &pbuffer));

    if (returnedStringLength)
    {
        *returnedStringLength = (ViInt32)strlen(pbuffer) - 2;
    }

    retCnt = (((ViInt32)strlen(pbuffer) - 2) <= stringSize) ? (ViInt32)strlen(pbuffer) - 2 : stringSize;
    strncpy(activeLimitLines, pbuffer + 1, retCnt);

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    if (pbuffer) free(pbuffer);
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function loads the limit line from the selected file in
/// HIFN .CSV format.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR fileName/Path and name of the CSV import file.
ViStatus _VI_FUNC rsspecan_ImportLimit(ViSession instrSession,
                                       ViString  fileName)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_LIMIT_IMPORT, fileName),
            2, "File Name");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function exports limit line data to an ASCII (CSV) file.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR fileName/Path and name of the target file.
/// HIPAR limitName/Name of the limit line to be exported.
ViStatus _VI_FUNC rsspecan_ExportLimit(ViSession instrSession,
                                       ViString  fileName,
                                       ViString  limitName)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidNullPointer(instrSession, fileName), 2, "File Name");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, limitName), 3, "Limit Name");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "MMEM:STOR:LIM '%s','%s'", fileName, limitName);
    checkErr(RsCore_Write(instrSession, cmd));
    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/**************************************************************************
 * -------------------  General Instrument Setup -------------------------*
 **************************************************************************/
/// HIFN This function sets the volume of the built-in loudspeaker
/// HIFN for demodulated signals.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR speakerVolume/Sets the volume of the built-in loudspeaker for demodulated
/// HIPAR speakerVolume/signals.
ViStatus _VI_FUNC rsspecan_SpeakerVolume(ViSession instrSession,
                                         ViReal64  speakerVolume)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsspecan_AssertFSV(instrSession));

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "",
                                                RSSPECAN_ATTR_SYST_SPEAKER, speakerVolume / 100.0),
                                                2, "Speaker Volume");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function controls the instrument behaviour when changing
/// HIFN the active application,
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR applicationSetupRecovery/This control sets the instrument behaviour when changing the
/// HIPAR applicationSetupRecovery/active application.
ViStatus _VI_FUNC rsspecan_ApplicationSetupRecovery(ViSession instrSession,
                                                    ViBoolean applicationSetupRecovery)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsspecan_AssertFSV(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_APPL_SETUP_REC, applicationSetupRecovery),
            2, "Application Setup Recovery");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/*****************************************************************************
 *------------------- Reference Frequency -----------------------------------*
 *****************************************************************************/
/// HIFN This function configures the reference oscillator.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR source/Sets the reference oscillator.
/// HIPAR externalFrequency/Defines the frequency of the external reference oscillator.
ViStatus _VI_FUNC rsspecan_ConfigureReferenceOscillator(ViSession instrSession,
                                                        ViInt32   source,
                                                        ViReal64  externalFrequency)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_ROSC_SOURCE, source),
            2, "Source");
    if (source != RSSPECAN_VAL_SOUR_INT)
    {
        viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_ROSC_FREQUENCY, externalFrequency),
                3, "External Frequency");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN Queries the current reference type in case you have activated an automatic
/// HIFN switch to the internal reference if the external reference is missing.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR source/Returns the source of the reference oscillator.
ViStatus _VI_FUNC rsspecan_QuerySelectedReferenceOscillatorSource(
                            ViSession instrSession,
                            ViInt32   *source)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_GetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_ROSC_SOURCE_EAUTO, source),
            2, "Source");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines how the instrument reacts if an external reference is selected
/// HIFN  but none is available.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR missingExtReferenceBehavior/Defines how the instrument reacts if an external reference is selected
/// HIPAR missingExtReferenceBehavior/but none is available.
ViStatus _VI_FUNC rsspecan_ConfigureMissingExternalReferenceBehavior(
                            ViSession instrSession,
                            ViInt32   missingExtReferenceBehavior)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, NULL, RSSPECAN_ATTR_MISSING_EXTERNAL_REFERENCE_BEHAVIOR, missingExtReferenceBehavior),
            2, "Missing Ext Reference Behavior");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures external reference frequency.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR tuningRange/Defines the tuning range. The tuning range is only available for the
/// HIPAR tuningRange/variable external reference frequency. It determines how far the
/// HIPAR tuningRange/frequency may deviate from the defined level in parts per million
/// HIPAR tuningRange/(10-6).
/// HIPAR frequency/This control informs the instrument on the frequency of the external
/// HIPAR frequency/reference oscillator. This frequency is used as starting point for the
/// HIPAR frequency/synchronization of the internal reference frequencies.
/// HIPAR loopBandwidth/Defines the loop bandwidth, that is, the speed of internal
/// HIPAR loopBandwidth/synchronization with the reference frequency. The setting requires a
/// HIPAR loopBandwidth/compromise between performance and increasing phase noise.
ViStatus _VI_FUNC rsspecan_ConfigureExternalReferenceFrequency(
                            ViSession instrSession,
                            ViInt32   tuningRange,
                            ViReal64  frequency,
                            ViReal64  loopBandwidth)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, NULL, RSSPECAN_ATTR_REFERENCE_OSCILLATOR_TUNING_RANGE, tuningRange),
            2, "Tuning Range");

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, NULL, RSSPECAN_ATTR_ROSC_FREQUENCY, frequency),
            3, "Frequency");

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, NULL, RSSPECAN_ATTR_REFERENCE_OSCILLATOR_LOOP_BANDWIDTH, loopBandwidth),
            4, "Loop Bandwidth");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function defines the frequency of the external reference
/// HIFN  oscillator.
/// HIFN
/// HIFN     If the external reference oscillator is selected, the reference signal
/// HIFN  must be connected to the rear panel of the instrument.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR frequency/This control defines the frequency of the external reference
/// HIPAR frequency/oscillator.
ViStatus _VI_FUNC rsspecan_ConfigureAdvancedReferenceFrequencyInput(
                            ViSession instrSession,
                            ViReal64  frequency)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_EXTERNAL_REFERENCE_OSCILLATOR_FREQUENCY, frequency),
            2, "Frequency");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function configures the advanced reference frequency.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR coupling/This command turns the coupling of the internal reference frequency on
/// HIPAR coupling/and off.
/// HIPAR bandwidth/This control selects coupling bandwidth mode for the internal
/// HIPAR bandwidth/reference frequency.
/// HIPAR bandwidthValue/This control defines coupling bandwidth for the internal reference
/// HIPAR bandwidthValue/frequency.
ViStatus _VI_FUNC rsspecan_ConfigureAdvancedExternalReferenceCoupling(
                            ViSession instrSession,
                            ViInt32   coupling,
                            ViInt32   bandwidth,
                            ViReal64  bandwidthValue)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_ADVANCED_REFERENCE_OSCILLATOR_FREQUENCY_COUPLING, coupling),
            2, "Coupling");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_ADVANCED_REFERENCE_OSCILLATOR_BANDWIDTH_MODE, bandwidth),
            3, "Bandwidth");

    if (bandwidth == RSSPECAN_VAL_MANUAL)
    {
        viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_ADVANCED_REFERENCE_OSCILLATOR_COUPLING_BANDWIDTH, bandwidthValue),
                4, "Bandwidth Value");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Enables or disables REF OUTPUT signals.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR output100MHzEnabled/If enabled, a 100 MHz reference signal is provided to the REF OUTPUT
/// HIPAR output100MHzEnabled/100 MHz connector.
/// HIPAR syncTriggerOutputEnabled/If enabled, a 100 MHz reference signal is provided to the SYNC TRIGGER
/// HIPAR syncTriggerOutputEnabled/OUTPUT connector.
/// HIPAR output640MHzEnabled/If enabled, a 640 MHz reference signal is provided to the REF OUTPUT
/// HIPAR output640MHzEnabled/640 MHz connector.
ViStatus _VI_FUNC rsspecan_ConfigureReferenceOscillatorOutputs(
                            ViSession instrSession,
                            ViBoolean output100MHzEnabled,
                            ViBoolean syncTriggerOutputEnabled,
                            ViBoolean output640MHzEnabled)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, NULL, RSSPECAN_ATTR_REFERENCE_OSCILLATOR_OUTPUT_100_MHZ_ENABLED, output100MHzEnabled),
            2, "Output 100 MHz Enabled");

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, NULL, RSSPECAN_ATTR_REFERENCE_OSCILLATOR_SYNC_TRIGGER_OUTPUT_ENABLED, syncTriggerOutputEnabled),
            3, "Sync Trigger Output Enabled");

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, NULL, RSSPECAN_ATTR_REFERENCE_OSCILLATOR_OUTPUT_640_MHZ_ENABLED, output640MHzEnabled),
            4, "Output 640 MHz Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/*****************************************************************************
 *------------------- Transducer --------------------------------------------*
 *****************************************************************************/
/// HIFN This function generates a transducer factor <name> using
/// HIFN normalized trace data.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR transducerState/This command switches the selected transducer factor on or off.
/// HIPAR transducerName/This command selects the transducer factor name.
ViStatus _VI_FUNC rsspecan_SelectTransducerFactor(ViSession instrSession,
                                                  ViInt32   reserved,
                                                  ViBoolean transducerState,
                                                  ViString  transducerName)
{
    ViStatus error = VI_SUCCESS;
             reserved;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_TFAC_SEL_NAME, transducerName),
            4, "Transducer Name");

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_TFAC_STATE, transducerState),
            3, "Transducer State");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function configures the transducer factor.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR name/This command selects the transducer factor designated by <name>.
/// HIPAR name/If <name> does not exist yet, a new transducer factor is
/// HIPAR name/created.
/// HIPAR unit/This control defines the unit of the transducer factor selected.
/// HIPAR interpolation/This control defines whether the frequency scaling of the
/// HIPAR interpolation/transducer factor is linear or logarithmic.
/// HIPAR comment/Sets the comment for the selected transducer factor.
/// HIPAR displayState/This control switches on or off the power-save mode of the
/// HIPAR displayState/display.
/// HIPAR refLevAdj/Allows automatic reference level offset adaption to restore the
/// HIPAR refLevAdj/original dynamic range by also shifting the reference level by
/// HIPAR refLevAdj/the maximum value of the transducer factor.
ViStatus _VI_FUNC rsspecan_ConfTransducerFactor(ViSession instrSession,
                                                ViString  name,
                                                ViInt32   unit,
                                                ViInt32   interpolation,
                                                ViString  comment,
                                                ViBoolean displayState,
                                                ViBoolean refLevAdj)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_TFAC_SEL_NAME, name),
            2, "Name");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_TFAC_UNIT, unit),
            3, "Unit");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_TFAC_SCALING, interpolation),
            4, "Interpolation");

    viCheckParm(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_TFAC_COMMENT, comment),
            5, "Comment");

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_TFAC_DISPLAY, displayState),
            6, "Display State");

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_TFAC_ADJ_STATE, refLevAdj),
            7, "Ref Lev Adj");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function defines the transducer factor data.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR name/This command selects the transducer factor designated by <name>.
/// HIPAR name/If <name> does not exist yet, a new transducer factor is
/// HIPAR name/created.
/// HIPAR noOfTestPoints/Specifies the number of frequency,level pairs. This value
/// HIPAR noOfTestPoints/corresponds with number of arrays members (Frequency and Level
/// HIPAR noOfTestPoints/controls).
/// HIPAR frequencyData/This array defines the frequency part (in Hz) of the test points
/// HIPAR frequencyData/for the selected transducer factor.  The frequencies must be in
/// HIPAR frequencyData/ascending order.
/// HIPAR levelData/This array defines the level part of the test points for the
/// HIPAR levelData/selected transducer factor.
ViStatus _VI_FUNC rsspecan_DefineTransducerFactor(ViSession instrSession,
                                                  ViString  name,
                                                  ViInt32   noOfTestPoints,
                                                  ViReal64  frequencyData[],
                                                  ViReal64  levelData[])
{
    ViStatus  error = VI_SUCCESS;
    ViBoolean transducerState = 0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_TFAC_SEL_NAME, name),
            2, "Name");

    viCheckParm(RsCore_InvalidViInt32RangeMin(instrSession, noOfTestPoints, 0),
        3, "No Of Test Points");

    // Switch the transducer state to off if it is active
    // Otherwise the instrument takes a long time to apply the new transducer data
    rsspecan_GetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_TFAC_STATE, &transducerState);
    if (transducerState)
    {
        checkErr(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_TFAC_STATE, VI_FALSE));
    }

    if (noOfTestPoints > 0)
    {
        checkErr(RsCore_WriteAsciiViReal64ArraysInterleaved(instrSession, "SENS:CORR:TRAN:DATA ",
            noOfTestPoints, frequencyData, levelData, NULL, NULL, NULL, NULL));
    }

    // If the transducer was active at the beginning, switch it back on
    if (transducerState)
    {
        checkErr(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_TFAC_STATE, VI_TRUE));
    }

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function reads out the names of all transducer factors
/// HIFN stored on the harddisk.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR numberofTransducerFactors/This control returns number of names of all transducer factors.
/// HIPAR bufferSize/Pass the number of bytes in the string you specify for the
/// HIPAR bufferSize/catalog list parameter.
/// HIPAR transducerFactorsList/This control returns the names of all transducer factors stored
/// HIPAR transducerFactorsList/on the hard disk.
ViStatus _VI_FUNC rsspecan_GetTransducerFactorCatalog(ViSession instrSession,
                                                      ViInt32   *numberofTransducerFactors,
                                                      ViInt32   bufferSize,
                                                      ViChar    transducerFactorsList[])
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32RangeMin(instrSession, bufferSize, 0),
        3, "Buffer Size");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, transducerFactorsList),
        4, "Transducer Factor List");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, transducerFactorsList), 4, "Transducer Factor List");

    checkErr(RsCore_QueryCatalog(instrSession, ":CORR:TRAN:CAT?", bufferSize, transducerFactorsList, numberofTransducerFactors));

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function deletes the active transducer factor.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR transducerName/This control selects the transducer factor name.
ViStatus _VI_FUNC rsspecan_DeleteTransducerFactor(ViSession instrSession,
                                                  ViInt32   reserved,
                                                  ViString  transducerName)
{
    ViStatus error = VI_SUCCESS;
             reserved;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_TFAC_SEL_NAME, transducerName),
            3, "Transducer Name");
    checkErr(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_TFAC_DELETE, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function selects a transducer set and turns that transducer set on or off.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR transducerSetState/This control sets the name of the transducer set. If the name
/// HIPAR transducerSetState/does not exist yet, the instrument creates a transducer set by
/// HIPAR transducerSetState/that name.
/// HIPAR transducerSetName/This control switches the selected transducer set on or off.
ViStatus _VI_FUNC rsspecan_SelectTransducerSet(ViSession instrSession,
                                               ViString  transducerSetName,
                                               ViBoolean transducerSetState)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_TSET_SELECT, transducerSetName),
            2, "Transducer Set Name");

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_TSET_STATE, transducerSetState),
            3, "Transducer Set State");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function selects a transducer set and turns that transducer set on or off.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR transducerSetName/This control sets the name of the transducer set. If the name
/// HIPAR transducerSetName/does not exist yet, the R&S  ESR creates a transducer set by
/// HIPAR transducerSetName/that name.
/// HIPAR comment/Defines a comment for a transducer set.
/// HIPAR unit/This control selects the unit of a transducer set.
/// HIPAR transducerBreak/This control turns a transducer break on and off.
/// HIPAR range/Selects the range.
/// HIPAR startFrequency/Defines the start frequency of the frequency range.
/// HIPAR stopFrequency/Defines the stop frequency of the frequency range.
/// HIPAR factors/String containing the name of the transducer factor.
ViStatus _VI_FUNC rsspecan_ConfigureTransducerSet(ViSession instrSession,
                                                  ViString  transducerSetName,
                                                  ViString  comment,
                                                  ViInt32   unit,
                                                  ViBoolean transducerBreak,
                                                  ViInt32   range,
                                                  ViReal64  startFrequency,
                                                  ViReal64  stopFrequency,
                                                  ViString  factors)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, range, 1, 10),
            6, "Range");

    viCheckParm(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_TSET_SELECT, transducerSetName),
            2, "Transducer Set Name");

    viCheckParm(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_TSET_COMMENT, comment),
            3, "Comment");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_TSET_UNIT, unit),
            4, "Unit");

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_TSET_BREAK, transducerBreak),
            5, "Transducer Break");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SENS:CORR:TSET:RANG%ld %.12lG,%.12lG,%s", range,
             startFrequency, stopFrequency, factors);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function reads out the names of all transducer sets stored
/// HIFN  on the harddisk.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR numberOfTransducerSets/This control returns number of names of all transducer sets.
/// HIPAR bufferSize/Pass the number of bytes in the string you specify for the catalog
/// HIPAR bufferSize/list parameter.
/// HIPAR transducerSetsList/This control returns the names of all transducer sets stored on the
/// HIPAR transducerSetsList/hard disk.
ViStatus _VI_FUNC rsspecan_QueryTransducerSetCatalog(ViSession instrSession,
                                                     ViInt32   *numberOfTransducerSets,
                                                     ViInt32   bufferSize,
                                                     ViChar    transducerSetsList[])
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32RangeMin(instrSession, bufferSize, 0),
        3, "Buffer Size");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, transducerSetsList),
        4, "Transducer Set List");

    checkErr(RsCore_QueryCatalog(instrSession, ":CORR:TSET:CAT?", bufferSize, transducerSetsList, numberOfTransducerSets));

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function deletes a transducer set.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR transducerSetName/This control selects the transducer set name.
ViStatus _VI_FUNC rsspecan_DeleteTransducerSet(ViSession instrSession,
                                               ViString  transducerSetName)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_TSET_SELECT, transducerSetName),
            2, "Transducer Set Name");

    checkErr(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_TSET_DELETE, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function queries the active transducer set. Any empty
/// HIFN  string is returned, if no transducer set is active.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR activeTransducerSet/This control the active transducer set. Any empty string is
/// HIPAR activeTransducerSet/returned, if no transducer set is active.
ViStatus _VI_FUNC rsspecan_QueryActiveTransducerSet(ViSession instrSession,
                                                    ViChar    activeTransducerSet[])
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_GetAttributeViString(instrSession, "", RSSPECAN_ATTR_TSET_ACTIVE,
            512, activeTransducerSet),
            2, "Active Transducer Set");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function loads the transducer factor from the selected file
/// in .CSV format.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR fileName/Path and name of the CSV import file.
ViStatus _VI_FUNC rsspecan_ImportTransducerFactor(ViSession instrSession,
                                                  ViString  fileName)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_TRANSDUCER_FACTOR_IMPORT, fileName),
            2, "File Name");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function exports transducer factor data to an ASCII (CSV) file.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR fileName/Path and name of the target file.
/// HIPAR factorName/Name of the transducer factor to be exported.
ViStatus _VI_FUNC rsspecan_ExportTransducerFactor(ViSession instrSession,
                                                  ViString  fileName,
                                                  ViString  factorName)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidNullPointer(instrSession, fileName), 2, "File Name");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, factorName), 3, "Factor Name");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "MMEM:STOR:TFAC '%s','%s'", fileName, factorName);
    checkErr(RsCore_Write(instrSession, cmd));
    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/*===========================================================================*
 *------------------- Alignment/Calibration----------------------------------*
 *===========================================================================*/
/// HIFN This function initiates the acquisition of system error correction
/// HIFN data. A "0" is returned if the acquisition was successful.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR timeout/Sets the timeout for the calibration routine to be finished and
/// HIPAR timeout/have obtained calibration result.
/// HIPAR result/This control returns the result of acquisition of system error
/// HIPAR result/correction data. A "0" is returned if the acquisition was
/// HIPAR result/successful.
ViStatus _VI_FUNC rsspecan_CalibrationSync(ViSession instrSession,
                                           ViUInt32  timeout,
                                           ViInt32   *result)
{
    ViStatus error = VI_SUCCESS;
    ViUInt32 old_timeout = 0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, timeout, 0, 14400000),
            2, "Timeout");

    checkErr(rsspecan_GetTimeout(instrSession, &old_timeout));
    checkErr(rsspecan_SetTimeout(instrSession, timeout));
    viCheckParm(rsspecan_GetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_CALIBRATION, result),
            3, "Result");

Error:
    if (old_timeout > 0)
        rsspecan_SetTimeout(instrSession, old_timeout);

    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function aborts the acquisition of correction data and
/// HIFN restores the last complete correction data set.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
ViStatus _VI_FUNC rsspecan_CalibrationAbort(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_CALIBRATION_ABORT, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN This function reads the results of the correction data acquisition.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR arraySize/This control defines the size of array passed to Result
/// HIPAR arraySize/argument.
/// HIPAR result/This control returns the results of the correction data
/// HIPAR result/acquisition.
ViStatus _VI_FUNC rsspecan_CalibrationResult(ViSession instrSession,
                                             ViInt32   arraySize,
                                             ViChar    result[])
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_QueryViStringUnknownLengthToUserBufferWithOpc(instrSession, "CAL:RES?", 0, arraySize, result, NULL));

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function determines whether the current calibration data
/// HIFN are taken into account by the instrument (ON) or not (OFF).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR state/Determines whether the current calibration data are taken into
/// HIPAR state/account by the instrument (ON) or not (OFF)..
ViStatus _VI_FUNC rsspecan_CalibrationState(ViSession instrSession,
                                            ViBoolean state)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_CALIBRATION_STATE, state),
            2, "State");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the calibration schedule settings.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR calibrationSchedule/If enabled, a self-alignment is performed regularly at specific days
/// HIPAR calibrationSchedule/and time.
/// HIPAR warmup/If enabled, self-alignment is started automatically after the warmup
/// HIPAR warmup/operation has completed.
/// HIPAR shutdown/If activated, the R&SFSW is automatically shut down after
/// HIPAR shutdown/self-alignment is completed. Note that the instrument cannot be
/// HIPAR shutdown/restarted via remote control.
ViStatus _VI_FUNC rsspecan_ConfigureCalibrationScheduleSettings(
                            ViSession instrSession,
                            ViBoolean calibrationSchedule,
                            ViBoolean warmup,
                            ViBoolean shutdown)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, NULL, RSSPECAN_ATTR_CALIBRATION_SCHEDULE_ENABLED, calibrationSchedule),
            2, "Calibration Schedule");

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, NULL, RSSPECAN_ATTR_CALIBRATION_SCHEDULE_WARMUP_ENABLED, warmup),
            3, "Warmup");

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, NULL, RSSPECAN_ATTR_CALIBRATION_SCHEDULE_SHUTDOWN_ENABLED, shutdown),
            4, "Shutdown");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Defines the days on which a self-alignment is scheduled for
/// HIFN  RSSPECAN_ATTR_CALIBRATION_SCHEDULE_ENABLED. Up to 7 different days can
/// HIFN  be scheduled.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR day1/Defines the days on which a self-alignment is scheduled for.
/// HIPAR day2/Defines the days on which a self-alignment is scheduled for
/// HIPAR day3/Defines the days on which a self-alignment is scheduled for
/// HIPAR day4/Defines the days on which a self-alignment is scheduled for
/// HIPAR day5/Defines the days on which a self-alignment is scheduled for
/// HIPAR day6/Defines the days on which a self-alignment is scheduled for
/// HIPAR day7/Defines the days on which a self-alignment is scheduled for
ViStatus _VI_FUNC rsspecan_ConfigureCalibrationSchedule(ViSession instrSession,
                                                        ViInt32   day1,
                                                        ViInt32   day2,
                                                        ViInt32   day3,
                                                        ViInt32   day4,
                                                        ViInt32   day5,
                                                        ViInt32   day6,
                                                        ViInt32   day7)
{
    ViStatus error = VI_SUCCESS;
    ViChar   param[RS_PARAM_BUF_SIZE];
    ViChar*  day1String;
    ViChar*  optionalDayStr;
    ViInt32  optionalDays[6] = { day2 , day3, day4, day5, day6, day7 };

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &RsSpecAn_rngDay, day1, &day1String),
        2, "Day 1");

    (void)snprintf(param, RS_PARAM_BUF_SIZE, "%s", day1String);

    for (int i = 0; i < 6; i++)
    {
        if (optionalDays[i] != RSSPECAN_VAL_DAY_NAN)
        {
            checkErr(RsCore_FindRangeTableDiscreteCmdString(instrSession, &RsSpecAn_rngDayOptional, optionalDays[i], &optionalDayStr));
            strcat(param, ",");
            strcat(param, optionalDayStr);
        }
    }

    checkErr(RsCore_SetAttributeViRawString(instrSession, NULL, RSSPECAN_ATTR_HIDDEN_CALIBRATION_SCHEDULE_DAYS, RS_VAL_DIRECT_USER_CALL, param));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Defines the time at which a self-alignment is scheduled for the days
/// HIFN  specified by RSSPECAN_FUNC_CALIBRATION_SCHEDULE_DAYS, if
/// HIFN  RSSPECAN_ATTR_CALIBRATION_SCHEDULE_ENABLED is ON.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR time/Defines the time at which a self-alignment is scheduled for the days
/// HIPAR time/specified by RSSPECAN_FUNC_CALIBRATION_SCHEDULE_DAYS, if
/// HIPAR time/RSSPECAN_ATTR_CALIBRATION_SCHEDULE_ENABLED is ON.
ViStatus _VI_FUNC rsspecan_ConfigureCalibrationScheduleTime(ViSession instrSession,
                                                            ViString  time)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViString(instrSession, NULL, RSSPECAN_ATTR_CALIBRATION_SCHEDULE_TIME, time),
            2, "Time");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Due to changes in temperature, the YIG-preselector frequency may
/// HIFN  become slightly offset. This command re-aligns the preselector quickly,
/// HIFN  without requiring a full self alignment of the R&S FSW.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsspecan_CalibrationPreselection(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_CALIBRATION_PRESELECTION, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures general usage of user-defined frequency response correction
/// HIFN  settings.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR userCorrectionEnabled/Activates or deactivates the general usage of user-defined frequency
/// HIPAR userCorrectionEnabled/response correction settings.
/// HIPAR frequencyResponseActive/Activates or deactivates the use of additional frequency response
/// HIPAR frequencyResponseActive/(.fres) files. The correction data is these files is applied after any
/// HIPAR frequencyResponseActive/correction settings in active touchstone files.
/// HIPAR previewEnabled/Activates or deactivates the preview of the user correction files for
/// HIPAR previewEnabled/all input types.
/// HIPAR applyTo/Determines whether the frequency response correction settings are
/// HIPAR applyTo/applied to all active measurement channels, or only the currently
/// HIPAR applyTo/selected channel.
ViStatus _VI_FUNC rsspecan_ConfigureUserCorrection(ViSession instrSession,
                                                   ViBoolean userCorrectionEnabled,
                                                   ViBoolean frequencyResponseActive,
                                                   ViBoolean previewEnabled,
                                                   ViInt32   applyTo)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, NULL, RSSPECAN_ATTR_USER_CORRECTION_ENABLED, userCorrectionEnabled),
            2, "User Correction Enabled");
    if (userCorrectionEnabled)
    {
        viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, NULL, RSSPECAN_ATTR_USER_CORRECTION_FREQUENCY_RESPONSE_ACTIVE, frequencyResponseActive),
                3, "Frequency Response Active");

        viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, NULL, RSSPECAN_ATTR_USER_CORRECTION_PREVIEW_ENABLED, previewEnabled),
                4, "Preview Enabled");

        viCheckParm(rsspecan_SetAttributeViInt32(instrSession, NULL, RSSPECAN_ATTR_USER_CORRECTION_APPLY_TO, applyTo),
                5, "Apply To");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the validity of the user-defined correction settings.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR isValid/Queries the validity of the user-defined correction settings.
ViStatus _VI_FUNC rsspecan_QueryUserCorrectionIsValid(ViSession instrSession,
                                                      ViBoolean *isValid)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_GetAttributeViBoolean(instrSession, NULL, RSSPECAN_ATTR_USER_CORRECTION_IS_VALID, isValid),
            2, "Is Valid");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the trace values for the combined user correction files
/// HIFN  (.snp+.fres) in IQ mode.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR selectedValues/Selects, which trace data shall be queried.
/// HIPAR data/Returns the trace values for the specified .snp file and ports.
/// HIPAR arraySize/This control defines the size of 'Data' array.
/// HIPAR returnedValues/Returns the number of values returned from the instrument.
ViStatus _VI_FUNC rsspecan_QueryUserCorrectionIQModeTraceValues(
                            ViSession instrSession,
                            ViInt32   selectedValues,
                            ViUInt32  arraySize,
                            ViReal64  data[],
                            ViUInt32  *returnedValues)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, selectedValues, 0, 2),
            2, "Selected Values");

    snprintf(cmd, RS_REPCAP_BUF_SIZE, "CORR:FRES:USER:IQ:DATA:%s?", userCorrectionDataType[selectedValues]);
    checkErr(RsCore_QueryFloatArrayToUserBuffer(instrSession, cmd, arraySize, data, (ViPInt32)returnedValues));

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the trace values for the combined user correction files
/// HIFN  (.snp+.fres) in Spectrum mode.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR selectedValues/Selects, which trace data shall be queried.
/// HIPAR data/Returns the trace values for the specified .snp file and ports.
/// HIPAR arraySize/This control defines the size of 'Data' array.
/// HIPAR returnedValues/Returns the number of values returned from the instrument.
ViStatus _VI_FUNC rsspecan_QueryUserCorrectionSpectrumModeTraceValues(
                            ViSession instrSession,
                            ViInt32   selectedValues,
                            ViUInt32  arraySize,
                            ViReal64  data[],
                            ViUInt32  *returnedValues)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, selectedValues, 0, 2),
            2, "Selected Values");

    snprintf(cmd, RS_REPCAP_BUF_SIZE, "CORR:FRES:USER:SPEC:DATA:%s?", userCorrectionDataType[selectedValues]);
    checkErr(RsCore_QueryFloatArrayToUserBuffer(instrSession, cmd, arraySize, data, (ViPInt32)returnedValues));

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Activates or deactivates the automatic adjustment of the reference
/// HIFN  level to the active filter calculation configuration. The offset is the
/// HIFN  mean of the filter calculation.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR adjustReferencelevel/Activates or deactivates the automatic adjustment of the reference
/// HIPAR adjustReferencelevel/level to the active filter calculation configuration. The offset is the
/// HIPAR adjustReferencelevel/mean of the filter calculation.
ViStatus _VI_FUNC rsspecan_ConfigureUserCorrectionFileAdjustReferenceLevel(
                            ViSession instrSession,
                            ViBoolean adjustReferencelevel)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_USER_CORRECTION_FILE_FREQUENCY_RESPONSE_ADJUST_REFERENCE_LEVEL, adjustReferencelevel),
            2, "Adjust Reference Level");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Activates or deactivates the automatic adjustment of the reference
/// HIFN  level to the active filter calculation configuration. The offset is the
/// HIFN  mean of the filter calculation.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR adjustReferencelevel/Activates or deactivates the automatic adjustment of the reference
/// HIPAR adjustReferencelevel/level to the active filter calculation configuration. The offset is the
/// HIPAR adjustReferencelevel/mean of the filter calculation.
ViStatus _VI_FUNC rsspecan_ConfigureUserCorrectionAdjustReferenceLevel(
                            ViSession instrSession,
                            ViBoolean adjustReferencelevel)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_USER_CORRECTION_FREQUENCY_RESPONSE_ADJUST_REFERENCE_LEVEL, adjustReferencelevel),
            2, "Adjust Reference Level");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Indicates whether the frequency correction file covers the full
/// HIFN  measurement span.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR spanCover/Indicates whether the frequency correction file covers the full
/// HIPAR spanCover/measurement span.
ViStatus _VI_FUNC rsspecan_QueryUserCorrectionFrequencyResponseSpanCover(
                            ViSession  instrSession,
                            ViBoolean* spanCover)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_GetAttributeViBoolean(instrSession, NULL, RSSPECAN_ATTR_USER_CORRECTION_FREQUENCY_RESPONSE_SPAN_COVER, spanCover),
            2, "Span Cover");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Activates or deactivates the general usage of user-defined frequency
/// HIFN  response correction settings.
/// HIFN     Only if activated, the filter is calculated and applied to the
/// HIFN  results.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR correctionBasebandEnabled/Activates or deactivates the general usage of user-defined frequency
/// HIPAR correctionBasebandEnabled/response correction settings.
ViStatus _VI_FUNC rsspecan_ConfigureUserCorrectionBBEnabled(ViSession instrSession,
                                                            ViBoolean basebandEnabled)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, NULL, RSSPECAN_ATTR_USER_CORRECTION_BASEBAND_ENABLED, basebandEnabled),
            2, "Baseband Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Restores the default frequency response correction settings
/// HIFN  (containing only files specific to the instrument itself). Frequency
/// HIFN  response correction using .fres files is deactivated.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsspecan_PresetUserCorrectionBB(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsspecan_SetAttributeViString(instrSession, NULL, RSSPECAN_ATTR_USER_CORRECTION_BASEBAND_PRESET, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Loads a stored user-defined frequency response correction scenario.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR fileName/Loads a stored user-defined frequency response correction scenario.
ViStatus _VI_FUNC rsspecan_LoadUserCorrectionBB(ViSession instrSession,
                                                ViString  fileName)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViString(instrSession, NULL, RSSPECAN_ATTR_USER_CORRECTION_BASEBAND_LOAD, fileName),
            2, "File Name");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Stores a saveset for a user-defined frequency response correction
/// HIFN  scenario. By default, the saveset is stored in the
/// HIFN  C:\R_S\INSTR\USER\FResponse directory.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR fileName/Stores a saveset for a user-defined frequency response correction
/// HIPAR fileName/scenario. By default, the saveset is stored in the
/// HIPAR fileName/C:\R_S\INSTR\USER\FResponse directory.
ViStatus _VI_FUNC rsspecan_StoreUserCorrectionBB(ViSession instrSession,
                                                 ViString  fileName)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViString(instrSession, NULL, RSSPECAN_ATTR_USER_CORRECTION_BASEBAND_STORE, fileName),
            2, "File Name");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Removes all touchstone files from the current configuration for the
/// HIFN  selected or all input types.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsspecan_ClearUserCorrectionBBTestSetup(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsspecan_SetAttributeViString(instrSession, NULL, RSSPECAN_ATTR_USER_CORRECTION_BASEBAND_TEST_SETUP_CLEAR, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the number of entries in the list of touchstone files for the
/// HIFN  current configuration.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR count/Queries the number of entries in the list of touchstone files for the
/// HIPAR count/current configuration.
ViStatus _VI_FUNC rsspecan_QueryUserCorrectionBBTestSetupCount(
                            ViSession instrSession,
                            ViInt32   *count)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_GetAttributeViInt32(instrSession, NULL, RSSPECAN_ATTR_USER_CORRECTION_BASEBAND_TEST_SETUP_COUNT, count),
            2, "Count");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Returns a list of currently configured touchstone format files.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR bufferSize/This control defines the size of 'Catalog' buffer.
/// HIPAR catalog/Returns a list of currently configured touchstone format files.
/// HIPAR returnedValues/Returns the number of characters returned from the instrument.
ViStatus _VI_FUNC rsspecan_QueryUserCorrectionBBTestSetupCatalog(
                            ViSession instrSession,
                            ViUInt32  bufferSize,
                            ViChar    catalog[],
                            ViUInt32  *returnedValues)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_QueryViStringUnknownLengthToUserBuffer(instrSession, "CORR:FRES:BAS:USER:SLIS:CAT?", bufferSize, catalog, (ViInt32*)returnedValues));
    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures selected test setup.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR index/Selects the index of test setup.
/// HIPAR testSetupEnabled/Activates or deactivates the loaded file for the current
/// HIPAR testSetupEnabled/configuration. Only active files are included in filter calculation.
/// HIPAR selected/Selects a Touchstone format file from the specified directory to be
/// HIPAR selected/loaded to the current configuration for the baseband input.
/// HIPAR toPort/SnP files can be defined for a varying number of input and output
/// HIPAR toPort/ports.
/// HIPAR fromPort/SnP files can be defined for a varying number of input and output
/// HIPAR fromPort/ports.
ViStatus _VI_FUNC rsspecan_ConfigureUserCorrectionBBTestSetup(
                            ViSession instrSession,
                            ViUInt32  index,
                            ViBoolean testSetupEnabled,
                            ViString  selected,
                            ViInt32   toPort,
                            ViInt32   fromPort)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_USER_CORRECTION_BASEBAND_TEST_SETUP_ENABLED,
            "Slist", index - 1, RS_REPCAP_BUF_SIZE, repCap));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, repCap, RSSPECAN_ATTR_USER_CORRECTION_BASEBAND_TEST_SETUP_ENABLED, testSetupEnabled),
            3, "Test Setup Enabled");
    if (testSetupEnabled)
    {
        viCheckParm(rsspecan_SetAttributeViString(instrSession, repCap, RSSPECAN_ATTR_USER_CORRECTION_BASEBAND_TEST_SETUP_SELECTED, selected),
                4, "Selected");

        viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_USER_CORRECTION_BASEBAND_TEST_SETUP_TO_PORT, toPort),
                5, "To Port");

        viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_USER_CORRECTION_BASEBAND_TEST_SETUP_FROM_PORT, fromPort),
                6, "From Port");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Removes the specified touchstone file from the list.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR index/Selects the index of test setup.
ViStatus _VI_FUNC rsspecan_RemoveUserCorrectionBBTestSetup(ViSession instrSession,
                                                           ViUInt32  index)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_USER_CORRECTION_BASEBAND_TEST_SETUP_REMOVE,
            "Slist", index - 1, RS_REPCAP_BUF_SIZE, repCap));

    checkErr(rsspecan_SetAttributeViString(instrSession, repCap, RSSPECAN_ATTR_USER_CORRECTION_BASEBAND_TEST_SETUP_REMOVE, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Removes all frequency response (.fres) files in the current
/// HIFN  configuration for the selected or all input types.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsspecan_ClearUserCorrectionBBFRES(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsspecan_SetAttributeViString(instrSession, NULL, RSSPECAN_ATTR_USER_CORRECTION_BASEBAND_FREQUENCY_RESPONSE_CLEAR, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the number of entries in the list of .fres files for the
/// HIFN  current configuration.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR count/Queries the number of entries in the list of .fres files for the
/// HIPAR count/current configuration.
ViStatus _VI_FUNC rsspecan_QueryUserCorrectionBBFRESCount(ViSession instrSession,
                                                          ViInt32   *count)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_GetAttributeViInt32(instrSession, NULL, RSSPECAN_ATTR_USER_CORRECTION_BASEBAND_FREQUENCY_RESPONSE_COUNT, count),
            2, "Count");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Returns a list of currently available .fres files in the directory.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR bufferSize/This control defines the size of 'Catalog' buffer.
/// HIPAR catalog/Returns a list of currently available .fres files in the directory.
/// HIPAR returnedValues/Returns the number of characters returned from the instrument.
ViStatus _VI_FUNC rsspecan_QueryUserCorrectionBBFRESCatalog(ViSession instrSession,
                                                            ViUInt32  bufferSize,
                                                            ViChar    catalog[],
                                                            ViUInt32  *returnedValues)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_QueryViStringUnknownLengthToUserBuffer(instrSession, "CORR:FRES:BAS:USER:FLIS:CAT?", bufferSize, catalog, (ViInt32*)returnedValues));
    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures selected frequency response (.fres) file.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR index/Selects the index of frequency response file.
/// HIPAR selected/Loads an additional frequency response (.fres) file to the current
/// HIPAR selected/configuration.
/// HIPAR magnitudeEnabled/Activates or deactivates the use of the correction data in the
/// HIPAR magnitudeEnabled/selected file for magnitude results.
/// HIPAR phaseEnabled/Activates or deactivates the use of the correction data in the
/// HIPAR phaseEnabled/selected file for magnitude results.
ViStatus _VI_FUNC rsspecan_ConfigureUserCorrectionBBFRES(ViSession instrSession,
                                                         ViUInt32  index,
                                                         ViString  selected,
                                                         ViBoolean magnitudeEnabled,
                                                         ViBoolean phaseEnabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_USER_CORRECTION_BASEBAND_FREQUENCY_RESPONSE_SELECTED,
            "Flist", index - 1, RS_REPCAP_BUF_SIZE, repCap));

    viCheckParm(rsspecan_SetAttributeViString(instrSession, repCap, RSSPECAN_ATTR_USER_CORRECTION_BASEBAND_FREQUENCY_RESPONSE_SELECTED, selected),
            3, "Selected");

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, repCap, RSSPECAN_ATTR_USER_CORRECTION_BASEBAND_FREQUENCY_RESPONSE_MAGNITUDE_ENABLED, magnitudeEnabled),
            4, "Magnitude Enabled");

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, repCap, RSSPECAN_ATTR_USER_CORRECTION_BASEBAND_FREQUENCY_RESPONSE_PHASE_ENABLED, phaseEnabled),
            4, "Phase Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Removes the selected frequency response (.fres) file from the current
/// HIFN  configuration.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR index/Selects the index of frequency response file.
ViStatus _VI_FUNC rsspecan_RemoveUserCorrectionBBFRES(ViSession instrSession,
                                                      ViUInt32  index)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_USER_CORRECTION_BASEBAND_FREQUENCY_RESPONSE_REMOVE,
            "Flist", index - 1, RS_REPCAP_BUF_SIZE, repCap));

    checkErr(rsspecan_SetAttributeViString(instrSession, repCap, RSSPECAN_ATTR_USER_CORRECTION_BASEBAND_FREQUENCY_RESPONSE_REMOVE, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Activates or deactivates the automatic adjustment of the reference
/// HIFN  level to the active filter calculation configuration. The offset is the
/// HIFN  mean of the filter calculation.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR adjustReferencelevel/Activates or deactivates the automatic adjustment of the reference
/// HIPAR adjustReferencelevel/level to the active filter calculation configuration. The offset is the
/// HIPAR adjustReferencelevel/mean of the filter calculation.
ViStatus _VI_FUNC rsspecan_ConfigureUserCorrectionBBFRESAdjustReferenceLevel(
                            ViSession instrSession,
                            ViBoolean adjustReferencelevel)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_USER_CORRECTION_BASEBAND_FREQUENCY_RESPONSE_ADJUST_REFERENCE_LEVEL, adjustReferencelevel),
            2, "Adjust Reference Level");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Recalculates the input source correction filter.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsspecan_RefreshUserCorrectionBBFRES(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsspecan_SetAttributeViString(instrSession, NULL, RSSPECAN_ATTR_USER_CORRECTION_BASEBAND_FREQUENCE_RESPONSE_REFRESH, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Loads a frequency response (.fres) file to the current configuration.
/// HIFN  The maximum number of files per configuration is 15. The new file is
/// HIFN  added below the entry specified by the <fli> index. All other entries
/// HIFN  with a higher suffix are moved down by one position.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR index/Selects the index of frequency response file.
/// HIPAR selected/Loads a frequency response (.fres) file to the current configuration.
/// HIPAR selected/The maximum number of files per configuration is 15. The new file is
/// HIPAR selected/added below the entry specified by the <fli> index. All other entries
/// HIPAR selected/with a higher suffix are moved down by one position.
ViStatus _VI_FUNC rsspecan_InsertCorrectionBBFRES(ViSession instrSession,
                                                  ViUInt32  index,
                                                  ViString  selected)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_USER_CORRECTION_BASEBAND_FREQUENCY_RESPONSE_INSERT,
            "Flist", index - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Index");

    viCheckParm(rsspecan_SetAttributeViString(instrSession, repCap, RSSPECAN_ATTR_USER_CORRECTION_BASEBAND_FREQUENCY_RESPONSE_INSERT, selected),
            3, "Selected");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Loads a frequency response (.fres) file to the current configuration.
/// HIFN  The maximum number of files per configuration is 15. The new file is
/// HIFN  added below the entry specified by the <fli> index. All other entries
/// HIFN  with a higher suffix are moved down by one position.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR index/Selects the index of frequency response file.
/// HIPAR selected/Loads a frequency response (.fres) file to the current configuration.
/// HIPAR selected/The maximum number of files per configuration is 15. The new file is
/// HIPAR selected/added below the entry specified by the <fli> index. All other entries
/// HIPAR selected/with a higher suffix are moved down by one position.
ViStatus _VI_FUNC rsspecan_InsertCorrectionInputFRES(ViSession instrSession,
                                                     ViUInt32  index,
                                                     ViString  selected)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_USER_CORRECTION_INPUT_FREQUENCY_RESPONSE_INSERT,
            "Flist", index - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Index");

    viCheckParm(rsspecan_SetAttributeViString(instrSession, repCap, RSSPECAN_ATTR_USER_CORRECTION_INPUT_FREQUENCY_RESPONSE_INSERT, selected),
            3, "Selected");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Loads a frequency response (.fres) file to the current configuration.
/// HIFN  The maximum number of files per configuration is 15. The new file is
/// HIFN  added below the entry specified by the <fli> index. All other entries
/// HIFN  with a higher suffix are moved down by one position.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR index/Selects the index of frequency response file.
/// HIPAR selected/Loads a frequency response (.fres) file to the current configuration.
/// HIPAR selected/The maximum number of files per configuration is 15. The new file is
/// HIPAR selected/added below the entry specified by the <fli> index. All other entries
/// HIPAR selected/with a higher suffix are moved down by one position.
ViStatus _VI_FUNC rsspecan_InsertCorrectionUserFRES(ViSession instrSession,
                                                    ViUInt32  index,
                                                    ViString  selected)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_USER_CORRECTION_USER_FREQUENCY_RESPONSE_INSERT,
            "Flist", index - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Index");

    viCheckParm(rsspecan_SetAttributeViString(instrSession, repCap, RSSPECAN_ATTR_USER_CORRECTION_USER_FREQUENCY_RESPONSE_INSERT, selected),
            3, "Selected");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Loads a new Touchstone file for the current configuration. The maximum
/// HIFN  number of files per configuration is 15. The new file is added below
/// HIFN  the entry specified by the <sli> index. All other entries with a higher
/// HIFN  suffix are moved down by one position.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR index/Selects the index of frequency response file.
/// HIPAR selected/Loads a new Touchstone file for the current configuration. The maximum
/// HIPAR selected/number of files per configuration is 15. The new file is added below
/// HIPAR selected/the entry specified by the <sli> index. All other entries with a higher
/// HIPAR selected/suffix are moved down by one position.
ViStatus _VI_FUNC rsspecan_InsertTouchstoneBBFRES(ViSession instrSession,
                                                  ViUInt32  index,
                                                  ViString  selected)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_USER_CORRECTION_BASEBAND_FREQUENCY_RESPONSE_TOUCHSTONE_INSERT,
            "Flist", index - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Index");

    viCheckParm(rsspecan_SetAttributeViString(instrSession, repCap, RSSPECAN_ATTR_USER_CORRECTION_BASEBAND_FREQUENCY_RESPONSE_TOUCHSTONE_INSERT, selected),
            3, "Selected");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Loads a new Touchstone file for the current configuration. The maximum
/// HIFN  number of files per configuration is 15. The new file is added below
/// HIFN  the entry specified by the <sli> index. All other entries with a higher
/// HIFN  suffix are moved down by one position.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR index/Selects the index of frequency response file.
/// HIPAR selected/Loads a new Touchstone file for the current configuration. The maximum
/// HIPAR selected/number of files per configuration is 15. The new file is added below
/// HIPAR selected/the entry specified by the <sli> index. All other entries with a higher
/// HIPAR selected/suffix are moved down by one position.
ViStatus _VI_FUNC rsspecan_InsertTouchstoneInputFRES(ViSession instrSession,
                                                     ViUInt32  index,
                                                     ViString  selected)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_USER_CORRECTION_INPUT_FREQUENCY_RESPONSE_TOUCHSTONE_INSERT,
            "Flist", index - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Index");

    viCheckParm(rsspecan_SetAttributeViString(instrSession, repCap, RSSPECAN_ATTR_USER_CORRECTION_INPUT_FREQUENCY_RESPONSE_TOUCHSTONE_INSERT, selected),
            3, "Selected");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Loads a new Touchstone file for the current configuration. The maximum
/// HIFN  number of files per configuration is 15. The new file is added below
/// HIFN  the entry specified by the <sli> index. All other entries with a higher
/// HIFN  suffix are moved down by one position.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR index/Selects the index of frequency response file.
/// HIPAR selected/Loads a new Touchstone file for the current configuration. The maximum
/// HIPAR selected/number of files per configuration is 15. The new file is added below
/// HIPAR selected/the entry specified by the <sli> index. All other entries with a higher
/// HIPAR selected/suffix are moved down by one position.
ViStatus _VI_FUNC rsspecan_InsertTouchstoneUserFRES(ViSession instrSession,
                                                    ViUInt32  index,
                                                    ViString  selected)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_USER_CORRECTION_USER_FREQUENCY_RESPONSE_TOUCHSTONE_INSERT,
            "Flist", index - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Index");

    viCheckParm(rsspecan_SetAttributeViString(instrSession, repCap, RSSPECAN_ATTR_USER_CORRECTION_USER_FREQUENCY_RESPONSE_TOUCHSTONE_INSERT, selected),
            3, "Selected");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Moves the selected Touchstone file one position up or down in the list
/// HIFN  of files, changing the order in which the correction data is applied.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR index/Selects the index of frequency response file.
/// HIPAR direction/Moves the selected Touchstone file one position up or down in the list
/// HIPAR direction/of files, changing the order in which the correction data is applied.
ViStatus _VI_FUNC rsspecan_MoveTouchstoneBBFRES(ViSession instrSession,
                                                ViUInt32  index,
                                                ViInt32   direction)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_USER_CORRECTION_BASEBAND_FREQUENCY_RESPONSE_TOUCHSTONE_MOVE,
            "Flist", index - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Index");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_USER_CORRECTION_BASEBAND_FREQUENCY_RESPONSE_TOUCHSTONE_MOVE, direction),
            3, "Direction");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Moves the selected Touchstone file one position up or down in the list
/// HIFN  of files, changing the order in which the correction data is applied.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR index/Selects the index of frequency response file.
/// HIPAR direction/Moves the selected Touchstone file one position up or down in the list
/// HIPAR direction/of files, changing the order in which the correction data is applied.
ViStatus _VI_FUNC rsspecan_MoveTouchstoneInputFRES(ViSession instrSession,
                                                   ViUInt32  index,
                                                   ViInt32   direction)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_USER_CORRECTION_INPUT_FREQUENCY_RESPONSE_TOUCHSTONE_MOVE,
            "Flist", index - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Index");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_USER_CORRECTION_INPUT_FREQUENCY_RESPONSE_TOUCHSTONE_MOVE, direction),
            3, "Direction");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Moves the selected Touchstone file one position up or down in the list
/// HIFN  of files, changing the order in which the correction data is applied.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR index/Selects the index of frequency response file.
/// HIPAR direction/Moves the selected Touchstone file one position up or down in the list
/// HIPAR direction/of files, changing the order in which the correction data is applied.
ViStatus _VI_FUNC rsspecan_MoveTouchstoneUserFRES(ViSession instrSession,
                                                  ViUInt32  index,
                                                  ViInt32   direction)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_USER_CORRECTION_USER_FREQUENCY_RESPONSE_TOUCHSTONE_MOVE,
            "Flist", index - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Index");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_USER_CORRECTION_USER_FREQUENCY_RESPONSE_TOUCHSTONE_MOVE, direction),
            3, "Direction");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Activates or deactivates the general usage of user-defined frequency
/// HIFN  response correction settings.
/// HIFN     Only if activated, the filter is calculated and applied to the
/// HIFN  results.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR correctionRFInputEnabled/Activates or deactivates the general usage of user-defined frequency
/// HIPAR correctionRFInputEnabled/response correction settings.
ViStatus _VI_FUNC rsspecan_ConfigureUserCorrectionRFInputEnabled(
                            ViSession instrSession,
                            ViBoolean RFInputEnabled)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, NULL, RSSPECAN_ATTR_USER_CORRECTION_RF_INPUT_ENABLED, RFInputEnabled),
            2, "RF Input Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Restores the default frequency response correction settings
/// HIFN  (containing only files specific to the instrument itself). Frequency
/// HIFN  response correction using .fres files is deactivated.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsspecan_PresetUserCorrectionRFInput(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsspecan_SetAttributeViString(instrSession, NULL, RSSPECAN_ATTR_USER_CORRECTION_RF_INPUT_PRESET, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Loads a stored user-defined frequency response correction scenario.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR fileName/Loads a stored user-defined frequency response correction scenario.
ViStatus _VI_FUNC rsspecan_LoadUserCorrectionRFInput(ViSession instrSession,
                                                     ViString  fileName)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViString(instrSession, NULL, RSSPECAN_ATTR_USER_CORRECTION_RF_INPUT_LOAD, fileName),
            2, "File Name");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Stores a saveset for a user-defined frequency response correction
/// HIFN  scenario. By default, the saveset is stored in the
/// HIFN  C:\R_S\INSTR\USER\FResponse directory.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR fileName/Stores a saveset for a user-defined frequency response correction
/// HIPAR fileName/scenario. By default, the saveset is stored in the
/// HIPAR fileName/C:\R_S\INSTR\USER\FResponse directory.
ViStatus _VI_FUNC rsspecan_StoreUserCorrectionRFInput(ViSession instrSession,
                                                      ViString  fileName)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViString(instrSession, NULL, RSSPECAN_ATTR_USER_CORRECTION_RF_INPUT_STORE, fileName),
            2, "File Name");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Activates or deactivates the automatic adjustment of the reference
/// HIFN  level to the active filter calculation configuration. The offset is the
/// HIFN  mean of the filter calculation.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR adjustReferencelevel/Activates or deactivates the automatic adjustment of the reference
/// HIPAR adjustReferencelevel/level to the active filter calculation configuration. The offset is the
/// HIPAR adjustReferencelevel/mean of the filter calculation.
ViStatus _VI_FUNC rsspecan_ConfigureUserCorrectionRFInputAdjustReferenceLevel(
                            ViSession instrSession,
                            ViBoolean adjustReferencelevel)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_USER_CORRECTION_RF_INPUT_FREQUENCY_RESPONSE_ADJUST_REFERENCE_LEVEL, adjustReferencelevel),
            2, "Adjust Reference Level");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Recalculates the input source correction filter.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsspecan_RefreshUserCorrectionRFInputFrequencyResponse(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsspecan_SetAttributeViString(instrSession, NULL, RSSPECAN_ATTR_USER_CORRECTION_RF_INPUT_FREQUENCY_RESPONSE_REFRESH, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Removes all touchstone files from the current configuration for the
/// HIFN  selected or all input types.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsspecan_ClearUserCorrectionRFInputTestSetup(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsspecan_SetAttributeViString(instrSession, NULL, RSSPECAN_ATTR_USER_CORRECTION_RF_INPUT_TEST_SETUP_CLEAR, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the number of entries in the list of touchstone files for the
/// HIFN  current configuration.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR count/Queries the number of entries in the list of touchstone files for the
/// HIPAR count/current configuration.
ViStatus _VI_FUNC rsspecan_QueryUserCorrectionRFInputTestSetupCount(
                            ViSession instrSession,
                            ViInt32   *count)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_GetAttributeViInt32(instrSession, NULL, RSSPECAN_ATTR_USER_CORRECTION_RF_INPUT_TEST_SETUP_COUNT, count),
            2, "Count");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Returns a list of currently configured touchstone format files.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR bufferSize/This control defines the size of 'Catalog' buffer.
/// HIPAR catalog/Returns a list of currently configured touchstone format files.
/// HIPAR returnedValues/Returns the number of characters returned from the instrument.
ViStatus _VI_FUNC rsspecan_QueryUserCorrectionRFInputTestSetupCatalog(
                            ViSession instrSession,
                            ViUInt32  bufferSize,
                            ViChar    catalog[],
                            ViUInt32  *returnedValues)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_QueryViStringUnknownLengthToUserBuffer(instrSession, "CORR:FRES:INP:USER:SLIS:CAT?", bufferSize, catalog, (ViInt32*)returnedValues));
    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures selected test setup.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR index/Selects the index of test setup.
/// HIPAR testSetupEnabled/Activates or deactivates the loaded file for the current
/// HIPAR testSetupEnabled/configuration. Only active files are included in filter calculation.
/// HIPAR selected/Selects a Touchstone format file from the specified directory to be
/// HIPAR selected/loaded to the current configuration for the specified input type.
/// HIPAR toPort/SnP files can be defined for a varying number of input and output
/// HIPAR toPort/ports.
/// HIPAR fromPort/SnP files can be defined for a varying number of input and output
/// HIPAR fromPort/ports.
ViStatus _VI_FUNC rsspecan_ConfigureUserCorrectionRFInputTestSetup(
                            ViSession instrSession,
                            ViUInt32  index,
                            ViBoolean testSetupEnabled,
                            ViString  selected,
                            ViInt32   toPort,
                            ViInt32   fromPort)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_USER_CORRECTION_RF_INPUT_TEST_SETUP_ENABLED,
            "Slist", index - 1, RS_REPCAP_BUF_SIZE, repCap));
    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, repCap, RSSPECAN_ATTR_USER_CORRECTION_RF_INPUT_TEST_SETUP_ENABLED, testSetupEnabled),
            3, "Test Setup Enabled");
    if (testSetupEnabled)
    {
        viCheckParm(rsspecan_SetAttributeViString(instrSession, repCap, RSSPECAN_ATTR_USER_CORRECTION_RF_INPUT_TEST_SETUP_SELECTED, selected),
                4, "Selected");

        viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_USER_CORRECTION_RF_INPUT_TEST_SETUP_TO_PORT, toPort),
                5, "To Port");

        viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_USER_CORRECTION_RF_INPUT_TEST_SETUP_FROM_PORT, fromPort),
                6, "From Port");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Removes the specified touchstone file from the list.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR index/Selects the index of test setup.
ViStatus _VI_FUNC rsspecan_RemoveUserCorrectionRFInputTestSetup(
                            ViSession instrSession,
                            ViUInt32  index)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_USER_CORRECTION_RF_INPUT_TEST_SETUP_ENABLED,
            "Slist", index - 1, RS_REPCAP_BUF_SIZE, repCap));
    checkErr(rsspecan_SetAttributeViString(instrSession, repCap, RSSPECAN_ATTR_USER_CORRECTION_RF_INPUT_TEST_SETUP_REMOVE, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Removes all frequency response (.fres) files in the current
/// HIFN  configuration for the selected or all input types.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsspecan_ClearUserCorrectionRFInputFRES(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsspecan_SetAttributeViString(instrSession, NULL, RSSPECAN_ATTR_USER_CORRECTION_RF_INPUT_FREQUENCY_RESPONSE_CLEAR, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the number of entries in the list of .fres files for the
/// HIFN  current configuration.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR count/Queries the number of entries in the list of .fres files for the
/// HIPAR count/current configuration.
ViStatus _VI_FUNC rsspecan_QueryUserCorrectionRFInputFRESCount(
                            ViSession instrSession,
                            ViInt32   *count)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_GetAttributeViInt32(instrSession, NULL, RSSPECAN_ATTR_USER_CORRECTION_RF_INPUT_FREQUENCY_RESPONSE_COUNT, count),
            2, "Count");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Returns a list of currently available .fres files in the directory.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR bufferSize/This control defines the size of 'Catalog' buffer.
/// HIPAR catalog/Returns a list of currently available .fres files in the directory.
/// HIPAR returnedValues/Returns the number of characters returned from the instrument.
ViStatus _VI_FUNC rsspecan_QueryUserCorrectionRFInputFRESCatalog(
                            ViSession instrSession,
                            ViUInt32  bufferSize,
                            ViChar    catalog[],
                            ViUInt32  *returnedValues)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_QueryViStringUnknownLengthToUserBuffer(instrSession, "CORR:FRES:INP:USER:FLIS:CAT?", bufferSize, catalog, (ViInt32*)returnedValues));
    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures selected frequency response (.fres) file.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR index/Selects the index of frequency response file.
/// HIPAR selected/Loads an additional frequency response (.fres) file to the current
/// HIPAR selected/configuration.
/// HIPAR magnitudeEnabled/Activates or deactivates the use of the correction data in the
/// HIPAR magnitudeEnabled/selected file for magnitude results.
/// HIPAR phaseEnabled/Activates or deactivates the use of the correction data in the
/// HIPAR phaseEnabled/selected file for magnitude results.
ViStatus _VI_FUNC rsspecan_ConfigureUserCorrectionRFInputFRES(
                            ViSession instrSession,
                            ViUInt32  index,
                            ViString  selected,
                            ViBoolean magnitudeEnabled,
                            ViBoolean phaseEnabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_USER_CORRECTION_RF_INPUT_FREQUENCY_RESPONSE_SELECTED,
            "Flist", index - 1, RS_REPCAP_BUF_SIZE, repCap));
    viCheckParm(rsspecan_SetAttributeViString(instrSession, repCap, RSSPECAN_ATTR_USER_CORRECTION_RF_INPUT_FREQUENCY_RESPONSE_SELECTED, selected),
            3, "Selected");

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, repCap, RSSPECAN_ATTR_USER_CORRECTION_RF_INPUT_FREQUENCY_RESPONSE_MAGNITUDE_ENABLED, magnitudeEnabled),
            4, "Magnitude Enabled");

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, repCap, RSSPECAN_ATTR_USER_CORRECTION_RF_INPUT_FREQUENCY_RESPONSE_PHASE_ENABLED, phaseEnabled),
            5, "Phase Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Removes the selected frequency response (.fres) file from the current
/// HIFN  configuration.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR index/Selects the index of frequency response file.
ViStatus _VI_FUNC rsspecan_RemoveUserCorrectionRFInputFRES(ViSession instrSession,
                                                           ViUInt32  index)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_USER_CORRECTION_RF_INPUT_FREQUENCY_RESPONSE_REMOVE,
            "Flist", index - 1, RS_REPCAP_BUF_SIZE, repCap));
    checkErr(rsspecan_SetAttributeViString(instrSession, repCap, RSSPECAN_ATTR_USER_CORRECTION_RF_INPUT_FREQUENCY_RESPONSE_REMOVE, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Restores the default frequency response correction settings
/// HIFN  (containing only files specific to the instrument itself). Frequency
/// HIFN  response correction using .fres files is deactivated.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsspecan_PresetUserCorrectionAllInputs(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsspecan_SetAttributeViString(instrSession, NULL, RSSPECAN_ATTR_USER_CORRECTION_ALL_INPUTS_PRESET, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Loads a stored user-defined frequency response correction scenario.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR fileName/Loads a stored user-defined frequency response correction scenario.
ViStatus _VI_FUNC rsspecan_LoadUserCorrectionAllInputs(ViSession instrSession,
                                                       ViString  fileName)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViString(instrSession, NULL, RSSPECAN_ATTR_USER_CORRECTION_ALL_INPUTS_LOAD, fileName),
            2, "File Name");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Stores a saveset for a user-defined frequency response correction
/// HIFN  scenario. By default, the saveset is stored in the
/// HIFN  C:\R_S\INSTR\USER\FResponse directory.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR fileName/Stores a saveset for a user-defined frequency response correction
/// HIPAR fileName/scenario. By default, the saveset is stored in the
/// HIPAR fileName/C:\R_S\INSTR\USER\FResponse directory.
ViStatus _VI_FUNC rsspecan_StoreUserCorrectionAllInputs(ViSession instrSession,
                                                        ViString  fileName)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViString(instrSession, NULL, RSSPECAN_ATTR_USER_CORRECTION_ALL_INPUTS_STORE, fileName),
            2, "File Name");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Removes all touchstone files from the current configuration for the
/// HIFN  selected or all input types.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsspecan_ClearUserCorrectionAllInputsTestSetup(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsspecan_SetAttributeViString(instrSession, NULL, RSSPECAN_ATTR_USER_CORRECTION_ALL_INPUTS_TEST_SETUP_CLEAR, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the number of entries in the list of touchstone files for the
/// HIFN  current configuration.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR count/Queries the number of entries in the list of touchstone files for the
/// HIPAR count/current configuration.
ViStatus _VI_FUNC rsspecan_QueryUserCorrectionAllInputsTestSetupCount(
                            ViSession instrSession,
                            ViInt32   *count)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_GetAttributeViInt32(instrSession, NULL, RSSPECAN_ATTR_USER_CORRECTION_ALL_INPUTS_TEST_SETUP_COUNT, count),
            2, "Count");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Returns a list of currently configured touchstone format files.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR bufferSize/This control defines the size of 'Catalog' buffer.
/// HIPAR catalog/Returns a list of currently configured touchstone format files.
/// HIPAR returnedValues/Returns the number of characters returned from the instrument.
ViStatus _VI_FUNC rsspecan_QueryUserCorrectionAllInputsTestSetupCatalog(
                            ViSession instrSession,
                            ViUInt32  bufferSize,
                            ViChar    catalog[],
                            ViUInt32  *returnedValues)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_QueryViStringUnknownLengthToUserBuffer(instrSession, "CORR:FRES:USER:SLIS:CAT?", bufferSize, catalog, (ViInt32*)returnedValues));
    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures selected test setup.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR index/Selects the index of test setup.
/// HIPAR testSetupEnabled/Activates or deactivates the loaded file for the current
/// HIPAR testSetupEnabled/configuration. Only active files are included in filter calculation.
/// HIPAR selected/Selects a Touchstone format file from the specified directory to be
/// HIPAR selected/loaded to the current configuration for the specified input type.
/// HIPAR toPort/SnP files can be defined for a varying number of input and output
/// HIPAR toPort/ports.
/// HIPAR fromPort/SnP files can be defined for a varying number of input and output
/// HIPAR fromPort/ports.
ViStatus _VI_FUNC rsspecan_ConfigureUserCorrectionAllInputsTestSetup(
                            ViSession instrSession,
                            ViUInt32  index,
                            ViBoolean testSetupEnabled,
                            ViString  selected,
                            ViInt32   toPort,
                            ViInt32   fromPort)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_USER_CORRECTION_ALL_INPUTS_TEST_SETUP_ENABLED,
            "Slist", index - 1, RS_REPCAP_BUF_SIZE, repCap));
    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, repCap, RSSPECAN_ATTR_USER_CORRECTION_ALL_INPUTS_TEST_SETUP_ENABLED, testSetupEnabled),
            3, "Test Setup Enabled");
    if (testSetupEnabled)
    {
        viCheckParm(rsspecan_SetAttributeViString(instrSession, repCap, RSSPECAN_ATTR_USER_CORRECTION_ALL_INPUTS_TEST_SETUP_SELECTED, selected),
                4, "Selected");

        viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_USER_CORRECTION_ALL_INPUTS_TEST_SETUP_TO_PORT, toPort),
                5, "To Port");

        viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_USER_CORRECTION_ALL_INPUTS_TEST_SETUP_FROM_PORT, fromPort),
                6, "From Port");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the frequency trace values for the specified .snp file and
/// HIFN  ports.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR index/Selects the index of test setup.
/// HIPAR sPortPairIndex/S-port pair index, where:
/// HIPAR selectedValues/Selects, which trace data shall be queried.
/// HIPAR data/Returns the trace values for the specified .snp file and ports.
/// HIPAR arraySize/This control defines the size of 'Data' array.
/// HIPAR returnedValues/Returns the number of values returned from the instrument.
ViStatus _VI_FUNC rsspecan_QueryUserCorrectionAllInputsTestSetupTraceValues(
                            ViSession instrSession,
                            ViUInt32  index,
                            ViInt32   sPortPairIndex,
                            ViInt32   selectedValues,
                            ViUInt32  arraySize,
                            ViReal64  data[],
                            ViUInt32  *returnedValues)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, selectedValues, 0, 2),
            4, "Selected Values");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, sPortPairIndex, 1, 4),
            3, "S-Port Pair Index");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, index, 1, 15),
            2, "Index");

    snprintf(cmd, RS_REPCAP_BUF_SIZE, "CORR:FRES:USER:SLIS%ld:DATA:%s%ld?", index, userCorrectionDataType[selectedValues], sPortPairIndex);
    checkErr(RsCore_QueryFloatArrayToUserBuffer(instrSession, cmd, arraySize, data, (ViPInt32)returnedValues));

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Removes the specified touchstone file from the list.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR index/Selects the index of test setup.
ViStatus _VI_FUNC rsspecan_RemoveUserCorrectionAllInputsTestSetup(
                            ViSession instrSession,
                            ViUInt32  index)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_USER_CORRECTION_ALL_INPUTS_TEST_SETUP_REMOVE,
            "Slist", index - 1, RS_REPCAP_BUF_SIZE, repCap));

    checkErr(rsspecan_SetAttributeViString(instrSession, repCap, RSSPECAN_ATTR_USER_CORRECTION_ALL_INPUTS_TEST_SETUP_REMOVE, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Removes all frequency response (.fres) files in the current
/// HIFN  configuration for the selected or all input types.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsspecan_ClearUserCorrectionAllInputsFRES(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsspecan_SetAttributeViString(instrSession, NULL, RSSPECAN_ATTR_USER_CORRECTION_ALL_INPUTS_FREQUENCY_RESPONSE_CLEAR, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the number of entries in the list of .fres files for the
/// HIFN  current configuration.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR count/Queries the number of entries in the list of .fres files for the
/// HIPAR count/current configuration.
ViStatus _VI_FUNC rsspecan_QueryUserCorrectionAllInputsFRESCount(
                            ViSession instrSession,
                            ViInt32   *count)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_GetAttributeViInt32(instrSession, NULL, RSSPECAN_ATTR_USER_CORRECTION_ALL_INPUTS_FREQUENCY_RESPONSE_COUNT, count),
            2, "Count");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Returns a list of currently available .fres files in the directory.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR bufferSize/This control defines the size of 'Catalog' buffer.
/// HIPAR catalog/Returns a list of currently available .fres files in the directory.
/// HIPAR returnedValues/Returns the number of characters returned from the instrument.
ViStatus _VI_FUNC rsspecan_QueryUserCorrectionAllInputsFRESCatalog(
                            ViSession instrSession,
                            ViUInt32  bufferSize,
                            ViChar    catalog[],
                            ViUInt32  *returnedValues)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_QueryViStringUnknownLengthToUserBuffer(instrSession, "CORR:FRES:USER:FLIS:CAT?", bufferSize, catalog, (ViInt32*)returnedValues));
    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures selected frequency response (.fres) file.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR index/Selects the index of frequency response file.
/// HIPAR selected/Loads an additional frequency response (.fres) file to the current
/// HIPAR selected/configuration.
/// HIPAR magnitudeEnabled/Activates or deactivates the use of the correction data in the
/// HIPAR magnitudeEnabled/selected file for magnitude results.
/// HIPAR phaseEnabled/Activates or deactivates the use of the correction data in the
/// HIPAR phaseEnabled/selected file for magnitude results.
ViStatus _VI_FUNC rsspecan_ConfigureUserCorrectionAllInputsFRES(
                            ViSession instrSession,
                            ViUInt32  index,
                            ViString  selected,
                            ViBoolean magnitudeEnabled,
                            ViBoolean phaseEnabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_USER_CORRECTION_ALL_INPUTS_FREQUENCY_RESPONSE_SELECTED,
            "Flist", index - 1, RS_REPCAP_BUF_SIZE, repCap));
    viCheckParm(rsspecan_SetAttributeViString(instrSession, repCap, RSSPECAN_ATTR_USER_CORRECTION_ALL_INPUTS_FREQUENCY_RESPONSE_SELECTED, selected),
            3, "Selected");

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, repCap, RSSPECAN_ATTR_USER_CORRECTION_ALL_INPUTS_FREQUENCY_RESPONSE_MAGNITUDE_ENABLED, magnitudeEnabled),
            4, "Magnitude Enabled");

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, repCap, RSSPECAN_ATTR_USER_CORRECTION_ALL_INPUTS_FREQUENCY_RESPONSE_PHASE_ENABLED, phaseEnabled),
            5, "Phase Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the frequency trace values for the specified .fres file.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR index/Selects the index of test setup.
/// HIPAR selectedValues/Selects, which trace data shall be queried.
/// HIPAR data/Returns the trace values for the specified .snp file and ports.
/// HIPAR arraySize/This control defines the size of 'Data' array.
/// HIPAR returnedValues/Returns the number of values returned from the instrument.
ViStatus _VI_FUNC rsspecan_QueryUserCorrectionAllInputsFRESTraceValues(
                            ViSession instrSession,
                            ViUInt32  index,
                            ViInt32   selectedValues,
                            ViUInt32  arraySize,
                            ViReal64  data[],
                            ViUInt32  *returnedValues)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, selectedValues, 0, 2),
            3, "Selected Values");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, index, 1, 15),
            2, "Index");

    snprintf(cmd, RS_REPCAP_BUF_SIZE, "CORR:FRES:USER:FLIS%ld:DATA:%s?", index, userCorrectionDataType[selectedValues]);
    checkErr(RsCore_QueryFloatArrayToUserBuffer(instrSession, cmd, arraySize, data, (ViPInt32)returnedValues));

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Removes the selected frequency response (.fres) file from the current
/// HIFN  configuration.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR index/Selects the index of frequency response file.
ViStatus _VI_FUNC rsspecan_RemoveUserCorrectionAllInputsFRES(ViSession instrSession,
                                                             ViUInt32  index)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_USER_CORRECTION_ALL_INPUTS_FREQUENCY_RESPONSE_REMOVE,
            "Flist", index , RS_REPCAP_BUF_SIZE, repCap));
    checkErr(rsspecan_SetAttributeViString(instrSession, repCap, RSSPECAN_ATTR_USER_CORRECTION_ALL_INPUTS_FREQUENCY_RESPONSE_REMOVE, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Turns synchronization of the frequency between measurement channels on
/// HIFN  and off.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR parameter/Selects the parameter for coupling.
/// HIPAR couplingEnabled/Turns synchronization of the frequency between measurement channels on
/// HIPAR couplingEnabled/and off.
ViStatus _VI_FUNC rsspecan_ConfigureSyncParameterCouplingEnabled(
                            ViSession instrSession,
                            ViInt32   parameter,
                            ViBoolean couplingEnabled)
{
    ViStatus error = VI_SUCCESS;
    ViAttr   parameters[16] = {
             RSSPECAN_ATTR_SYNC_CENTER_FREQUENCY_ENABLED,
             RSSPECAN_ATTR_SYNC_START_STOP_FREQUENCY_ENABLED,
             RSSPECAN_ATTR_SYNC_ATTENUATION_ENABLED,
             RSSPECAN_ATTR_SYNC_PREAMPLIFIER_ENABLED,
             RSSPECAN_ATTR_SYNC_VIDEO_BANDWIDTH_ENABLED,
             RSSPECAN_ATTR_SYNC_RBW_ENABLED,
             RSSPECAN_ATTR_SYNC_LIMIT_LINES_ENABLED,
             RSSPECAN_ATTR_SYNC_AC_DC_COUPLING_ENABLED,
             RSSPECAN_ATTR_SYNC_MARKER_FREQUENCY_ENABLED,
             RSSPECAN_ATTR_SYNC_PRESELECTOR_ENABLED,
             RSSPECAN_ATTR_SYNC_AUDIO_DEMODULATOR_ENABLED,
             RSSPECAN_ATTR_SYNC_AMPLITUDE_BASEBAND_IMPEDANCE_ENABLED,
             RSSPECAN_ATTR_SYNC_AMPLITUDE_UNIT_ENABLED,
             RSSPECAN_ATTR_SYNC_IMPEDANCE_ENABLED,
             RSSPECAN_ATTR_SYNC_LIMITED_LINES_ENABLED,
             RSSPECAN_ATTR_SYNC_REFERENCE_LEVEL_ENABLED
             };

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, parameter, RSSPECAN_VAL_PARM_COUPLING_CENTER_FREQUENCY, RSSPECAN_VAL_PARM_COUPLING_REFERENCE_LEVEL),
            2, "Parameter");
    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, NULL, parameters[parameter], couplingEnabled),
            3, "Coupling Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Creates a new user-defined parameter coupling.
/// HIFN     After the new coupling has been created, the method returns the index
/// HIFN  number of the new coupling. Therefore, the command is implemented as a
/// HIFN  query.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channelName1/String containing the name of a measurement channel or channel type or
/// HIPAR channelName1/use one of these:
/// HIPAR window1/String containing the name of a measurement window or use 'All
/// HIPAR window1/Windows' to synchronized all measurement windows.
/// HIPAR parameter1/String containing the name of a synchronizable parameter.
/// HIPAR channelName2/String containing the name of a measurement channel or channel type.
/// HIPAR window2/String containing the name of a measurement window.
/// HIPAR parameter2/String containing the name of a synchronizable parameter.
/// HIPAR direction/Selects the direction in which synchronization works.
/// HIPAR enabled/Enables or disables the coupling.
/// HIPAR index/Returns the index number of the new coupling.
ViStatus _VI_FUNC rsspecan_CreateSyncUserDefinedCoupling(ViSession instrSession,
                                                         ViString  channelName1,
                                                         ViString  window1,
                                                         ViString  parameter1,
                                                         ViString  channelName2,
                                                         ViString  window2,
                                                         ViString  parameter2,
                                                         ViInt32   direction,
                                                         ViBoolean enabled,
                                                         ViUInt32  *index)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViChar   segment[RS_MAX_MESSAGE_LEN];
    ViChar   *p2direction;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &RsSpecAn_rngParameterSynchronizationDirection, direction, &p2direction),
            4, "Direction");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "INST:COUP:USER:NEW? '%s','%s','%s'", channelName1, window1, parameter1);
    if (channelName2)
    {
        snprintf(segment, RS_MAX_MESSAGE_LEN, ",'%s'", channelName2);
        strcat(cmd, segment);
        if (window2)
        {
            snprintf(segment, RS_MAX_MESSAGE_LEN, ",'%s'", window2);
            strcat(cmd, segment);
            if (parameter2)
            {
                snprintf(segment, RS_MAX_MESSAGE_LEN, ",'%s'", parameter2);
                strcat(cmd, segment);
            }
        }
    }

    snprintf(segment, RS_MAX_MESSAGE_LEN, ",%s,%s", p2direction, enabled ? "ON" : "OFF");
    strcat(cmd, segment);

    checkErr(RsCore_QueryViInt32(instrSession, cmd, (ViInt32*) index));

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Edits an existing user-defined coupling definition.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channelName1/String containing the name of a measurement channel or channel type or
/// HIPAR channelName1/use one of these:
/// HIPAR index/Selects the index of user coupling.
/// HIPAR window1/String containing the name of a measurement window or use 'All
/// HIPAR window1/Windows' to synchronized all measurement windows.
/// HIPAR parameter1/String containing the name of a synchronizable parameter.
/// HIPAR channelName2/String containing the name of a measurement channel or channel type.
/// HIPAR window2/String containing the name of a measurement window.
/// HIPAR parameter2/String containing the name of a synchronizable parameter.
/// HIPAR direction/Selects the direction in which synchronization works.
/// HIPAR enabled/Enables or disables the coupling.
ViStatus _VI_FUNC rsspecan_EditSyncUserDefinedCoupling(ViSession instrSession,
                                                       ViUInt32  index,
                                                       ViString  channelName1,
                                                       ViString  window1,
                                                       ViString  parameter1,
                                                       ViString  channelName2,
                                                       ViString  window2,
                                                       ViString  parameter2,
                                                       ViInt32   direction,
                                                       ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViChar   segment[RS_MAX_MESSAGE_LEN];
    ViChar   *p2direction;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, index, 1, 20),
            2, "Index");
    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &RsSpecAn_rngParameterSynchronizationDirection, direction, &p2direction),
            4, "Direction");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "INST:COUP:USER%ld '%s','%s','%s'", index, channelName1, window1, parameter1);
    if (channelName2)
    {
        snprintf(segment, RS_MAX_MESSAGE_LEN, ",'%s'", channelName2);
        strcat(cmd, segment);
        if (window2)
        {
            snprintf(segment, RS_MAX_MESSAGE_LEN, ",'%s'", window2);
            strcat(cmd, segment);
            if (parameter2)
            {
                snprintf(segment, RS_MAX_MESSAGE_LEN, ",'%s'", parameter2);
                strcat(cmd, segment);
            }
        }
    }

    snprintf(segment, RS_MAX_MESSAGE_LEN, ",%s,%s", p2direction, enabled ? "ON" : "OFF");
    strcat(cmd, segment);

    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Enables or disables the specified user-defined parameter coupling.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR index/Selects the index of user coupling.
/// HIPAR parameterCouplingEnabled/Enables or disables the specified user-defined parameter coupling.
ViStatus _VI_FUNC rsspecan_ConfigureSyncUserDefinedParameterCouplingEnabled(
                            ViSession instrSession,
                            ViUInt32  index,
                            ViBoolean parameterCouplingEnabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_SYNC_USER_DEFINED_PARAMETER_COUPLING_ENABLED,
            "UserCoupling", index - 1, RS_REPCAP_BUF_SIZE, repCap));
    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, repCap, RSSPECAN_ATTR_SYNC_USER_DEFINED_PARAMETER_COUPLING_ENABLED, parameterCouplingEnabled),
            3, "Parameter Coupling Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects the direction in which synchronization works.
/// HIFN     Note that the command is not available if you synchronize over all
/// HIFN  channels or all channels of the same application.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR index/Selects the index of user coupling.
/// HIPAR direction/Selects the direction in which synchronization works.
ViStatus _VI_FUNC rsspecan_ConfigureSyncUserDefinedParameterCouplingDirection(
                            ViSession instrSession,
                            ViUInt32  index,
                            ViInt32   direction)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_SYNC_USER_DEFINED_PARAMETER_COUPLING_DIRECTION,
            "UserCoupling", index - 1, RS_REPCAP_BUF_SIZE, repCap));
    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_SYNC_USER_DEFINED_PARAMETER_COUPLING_DIRECTION, direction),
            3, "Direction");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the names of the measurement channels that can be
/// HIFN  synchronized.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR bufferSize/This control defines the size of 'List' buffer.
/// HIPAR list/Queries the names of the measurement channels that can be
/// HIPAR list/synchronized.
/// HIPAR returnedValues/Returns the number of characters returned from the instrument.
ViStatus _VI_FUNC rsspecan_QuerySyncUserDefinedParameterCouplingChannelList(
                            ViSession instrSession,
                            ViUInt32  bufferSize,
                            ViChar    list[],
                            ViUInt32  *returnedValues)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_QueryViStringUnknownLengthToUserBuffer(instrSession, "INST:COUP:USER:CHAN:LIST?", bufferSize, list, (ViInt32*)returnedValues));
    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries parameters that can be synchronized.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channelName/String containing the name of a measurement channel.
/// HIPAR parameter/String containing the name of a parameter that you can synchronize.
/// HIPAR list/Queries the names of the parameters that can be synchronized.
/// HIPAR bufferSize/This control defines the size of 'List' buffer.
/// HIPAR returnedValues/Returns the number of characters returned from the instrument.
ViStatus _VI_FUNC rsspecan_QuerySyncUserDefinedParameterCouplingParametersList(
                            ViSession instrSession,
                            ViString  channelName,
                            ViString  parameter,
                            ViUInt32  bufferSize,
                            ViChar    list[],
                            ViUInt32  *returnedValues)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "INST:COUP:USER:ELEM:LIST?");
    if (channelName)
    {
        RsCore_StrcatMaxLen(cmd, RS_MAX_MESSAGE_BUF_SIZE, " '", channelName, "'");
        if (parameter)
        {
            RsCore_StrcatMaxLen(cmd, RS_MAX_MESSAGE_BUF_SIZE, ",'", parameter, "'");
        }
    }

    checkErr(RsCore_QueryViStringUnknownLengthToUserBuffer(instrSession, cmd, bufferSize, list, (ViInt32*)returnedValues));
    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the index numbers of user-defined parameter couplings. The
/// HIFN  index numbers are used to refer to the specific coupling in remote
/// HIFN  commands with a USER<uc> suffix.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR bufferSize/This control defines the size of 'List' buffer.
/// HIPAR list/Queries the index numbers of user-defined parameter couplings. The
/// HIPAR list/index numbers are used to refer to the specific coupling in remote
/// HIPAR list/commands with a USER<uc> suffix.
/// HIPAR returnedValues/Returns the number of characters returned from the instrument.
ViStatus _VI_FUNC rsspecan_QuerySyncUserDefinedParameterCouplingNumbersList(
                            ViSession instrSession,
                            ViUInt32  bufferSize,
                            ViChar    list[],
                            ViUInt32  *returnedValues)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_QueryViStringUnknownLengthToUserBuffer(instrSession, "INST:COUP:USER:NUMB:LIST?", bufferSize, list, (ViInt32*)returnedValues));
    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the measurement windows that can be synchronized with another
/// HIFN  channel (or measurement window).
/// HIFN     Note that synchronizing with a specific measurement window is only
/// HIFN  possible in the Analog Demodulation application.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channelName/String containing the name of a measurement channel.
/// HIPAR parameter/String containing the name of a parameter that you can synchronize.
/// HIPAR list/Queries the measurement windows that can be synchronized with another
/// HIPAR list/channel (or measurement window)
/// HIPAR bufferSize/This control defines the size of 'List' buffer.
/// HIPAR returnedValues/Returns the number of characters returned from the instrument.
ViStatus _VI_FUNC rsspecan_SyncUserDefinedParameterCouplingWindowsList(
                            ViSession instrSession,
                            ViString  channelName,
                            ViString  parameter,
                            ViUInt32  bufferSize,
                            ViChar    list[],
                            ViUInt32  *returnedValues)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    if (channelName)
    {
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "INST:COUP:USER:WIND:LIST? '%s'", channelName);
        if (parameter)
        {
            snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "INST:COUP:USER:WIND:LIST? '%s','%s'", channelName, parameter);
        }
    }
    else
    {
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "INST:COUP:USER:WIND:LIST?");
    }

    checkErr(RsCore_QueryViStringUnknownLengthToUserBuffer(instrSession, cmd, bufferSize, list, (ViInt32*)returnedValues));
    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries additional information about the specified user-defined
/// HIFN  parameter coupling.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR index/Selects the index of user coupling.
/// HIPAR bufferSize/This control defines the size of 'Info' buffer.
/// HIPAR info/Queries additional information about the specified user-defined
/// HIPAR info/parameter coupling.
/// HIPAR returnedValues/Returns the number of characters returned from the instrument.
ViStatus _VI_FUNC rsspecan_QuerySyncUserDefinedParameterCouplingInfo(
                            ViSession instrSession,
                            ViUInt32  index,
                            ViUInt32  bufferSize,
                            ViChar    info[],
                            ViUInt32  *returnedValues)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, index, 1, 20),
            2, "Index");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "INST:COUP:USER%ld:INFO?", index);
    checkErr(RsCore_QueryViStringUnknownLengthToUserBuffer(instrSession, cmd, bufferSize, info, (ViInt32*)returnedValues));
    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Deletes a user-defined coupling mechanism.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR index/Selects the index of user coupling.
ViStatus _VI_FUNC rsspecan_RemoveSyncUserDefinedParameterCoupling(
                            ViSession instrSession,
                            ViUInt32  index)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_SYNC_USER_DEFINED_PARAMETER_COUPLING_REMOVE,
            "UserCoupling", index - 1, RS_REPCAP_BUF_SIZE, repCap));
    checkErr(rsspecan_SetAttributeViString(instrSession, repCap, RSSPECAN_ATTR_SYNC_USER_DEFINED_PARAMETER_COUPLING_REMOVE, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Deletes all user-defined coupling mechanisms.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsspecan_RemoveAllSyncUserDefinedParameterCoupling(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsspecan_SetAttributeViString(instrSession, NULL, RSSPECAN_ATTR_SYNC_USER_DEFINED_PARAMETER_COUPLING_REMOVE_ALL, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Enables or disables coupling between the R&SFSW and a connected signal
/// HIFN  generator.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR generatorCoupling/Enables or disables coupling between the R&SFSW and a connected signal
/// HIPAR generatorCoupling/generator.
ViStatus _VI_FUNC rsspecan_ConfigureGeneratorCouplingEnabled(ViSession instrSession,
                                                             ViBoolean generatorCoupling)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, NULL, RSSPECAN_ATTR_GENERATOR_COUPLING_ENABLED, generatorCoupling),
            2, "Generator Coupling");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Couples the center frequency of the connected signal generator to the
/// HIFN  R&SFSW.
/// HIFN
/// HIFN     Requires the RSSPECAN_ATTR_GENERATOR_COUPLING_ENABLED  to be ON.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR enabled/Couples the center frequency of the connected signal generator to the
/// HIPAR enabled/R&SFSW.
/// HIPAR offset/Defines a fixed offset to the center frequency of the R&SFSW for the
/// HIPAR offset/coupled signal generator.
ViStatus _VI_FUNC rsspecan_ConfigureGeneratorCouplingCenterFrequency(
                            ViSession instrSession,
                            ViBoolean enabled,
                            ViReal64  offset)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, NULL, RSSPECAN_ATTR_GENERATOR_COUPLING_CENTER_FREQUENCY_ENABLED, enabled),
            2, "Enabled");

    if (enabled == VI_TRUE)
    {
        viCheckParm(rsspecan_SetAttributeViReal64(instrSession, NULL, RSSPECAN_ATTR_GENERATOR_COUPLING_CENTER_FREQUENCY_OFFSET, offset),
                3, "Offset");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Couples the reference level of the connected signal generator to the
/// HIFN  R&SFSW.
/// HIFN
/// HIFN     Requires the RSSPECAN_ATTR_GENERATOR_COUPLING_ENABLED to be TRUE.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR enabled/Couples the reference level of the connected signal generator to the
/// HIPAR enabled/R&SFSW.
/// HIPAR offset/Defines a fixed offset to the reference level of the R&SFSW for the
/// HIPAR offset/coupled signal generator.
ViStatus _VI_FUNC rsspecan_ConfigureGeneratorCouplingReferenceLevelEnabled(
                            ViSession instrSession,
                            ViBoolean enabled,
                            ViReal64  offset)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, NULL, RSSPECAN_ATTR_GENERATOR_COUPLING_REFERENCE_LEVEL_ENABLED, enabled),
            2, "Enabled");

    if (enabled == VI_TRUE)
    {
        viCheckParm(rsspecan_SetAttributeViReal64(instrSession, NULL, RSSPECAN_ATTR_GENERATOR_COUPLING_REFERENCE_LEVEL_OFFSET, offset),
                3, "Offset");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/*===========================================================================*
 *------------------- System ------------------------------------------------*
 *===========================================================================*/
/// HIFN This function queries the contents of the module info table.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR arraySize/This control defines the size of array passed to HW Info.
/// HIPAR HWInfo/This control returns the contents of the module info table.
ViStatus _VI_FUNC rsspecan_ServiceHWInfo(ViSession instrSession,
                                         ViInt32   arraySize,
                                         ViChar    HWInfo[])
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_QueryViStringUnknownLengthToUserBuffer(instrSession, "DIAG:SERV:VERS?", arraySize, HWInfo, NULL));

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function queries the instrument firmware and option versions
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR arraySize/This control defines the size of array passed to Version Info
/// HIPAR arraySize/argument.
/// HIPAR versionInfo/This control returns the instrument firmware and option
/// HIPAR versionInfo/versions.
ViStatus _VI_FUNC rsspecan_ServiceVersionInfo(ViSession instrSession,
                                              ViInt32   arraySize,
                                              ViChar    versionInfo[])
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_QueryViStringUnknownLengthToUserBuffer(instrSession, "DIAG:SERV:VERS?", arraySize, versionInfo, NULL));

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This attributes queries the last date and time the instrument was
/// HIFN  serviced (ISO 8601 format).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR lastServiceDate/This controls sets the last date and time the instrument was serviced
/// HIPAR lastServiceDate/(ISO 8601 format).
ViStatus _VI_FUNC rsspecan_QueryServiceLastDate(ViSession instrSession,
                                                ViChar    lastServiceDate[])
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_GetAttributeViString(instrSession, NULL, RSSPECAN_ATTR_SERVICE_LAST_DATE,
            512, lastServiceDate),
            2, "Last Service Date");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function configures the input for servicing purposes.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR input/Selects the input calibration signal.
ViStatus _VI_FUNC rsspecan_ConfigureCalibrationSignal(ViSession instrSession,
                                                      ViInt32   input)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_SERVICE_INPUT_SOURCE, input),
            2, "Input");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function configures RF input signal.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR frequency/Selects the calibration frequency for the RF calibration signal
/// HIPAR RFSpectrum/Selects the bandwidth of the calibration signal.
ViStatus _VI_FUNC rsspecan_ConfigureCalibrationFreqRF(ViSession instrSession,
                                                      ViReal64  frequency,
                                                      ViInt32   RFSpectrum)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_SERVICE_PULSE_CAL_FREQ_FSV, frequency),
            2, "Frequency");

    if (rsspecan_IsFSV(instrSession) == VI_FALSE)
    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_SERVICE_CAL_FREQ_RF_SPECTRUM, RFSpectrum),
            3, "RF Spectrum");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the frequency of the internal broadband
/// HIFN  calibration signal to be used for IF filter calibration.
/// HIFN
/// HIFN     Note:
/// HIFN
/// HIFN     (1) This attribute is only available if the bandwidth extension option
/// HIFN  R&S FPS-B160 is installed. Before you can use the command, you have to
/// HIFN  feed in a calibration signal with RSSPECAN_ATTR_SERVICE_INPUT_SOURCE.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR frequency/Defines the frequency of the internal broadband calibration signal to
/// HIPAR frequency/be used for IF filter calibration.
ViStatus _VI_FUNC rsspecan_ConfigureWidebandCalibrationFrequency(
                            ViSession instrSession,
                            ViReal64  frequency)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_SERVICE_WIDEBAND_CALIBRATION_FREQUENCY, frequency),
            2, "Frequency");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN Defines the frequency of the internal broadband calibration signal to be used for IF filter calibration.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR frequencyWB2/Selects the frequency of the internal broadband calibration signal to be used for IF filter calibration.
ViStatus _VI_FUNC rsspecan_ConfigureInternalWidebandCalibrationFrequency(
                            ViSession instrSession,
                            ViInt32   frequencyWB2)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_SERVICE_INTERNAL_WIDEBAND_CALIBRATION_FREQUENCY, frequencyWB2),
            2, "Frequency WB2");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function selects the  calibration frequency for frequencies
/// HIFN greater than 7 GHz and distance of the peaks of the microwave
/// HIFN calibration signal for calibration of the YIG filter.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR frequency/Selects the calibration frequency for frequencies greater than 7 GHz.
/// HIPAR distance/Selects the distance of the peaks of the microwave calibration
/// HIPAR distance/signal for calibration of the YIG filter.
ViStatus _VI_FUNC rsspecan_ConfigureCalibrationFreqMW(ViSession instrSession,
                                                      ViReal64  frequency,
                                                      ViInt32   distance)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_SERVICE_CAL_FREQ_MW, frequency),
            2, "Frequency");

    if (!RsCore_IsInstrumentModel(instrSession, "FSV"))
    {
        viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_SERVICE_CAL_FREQ_MW_DIST, distance),
                3, "Distance");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function ines the frequency of the calibration signal for R&S FSW models
/// HIFN 43 MHz and higher and distance of the peaks of the microwave
/// HIFN calibration signal for calibration of the YIG filter.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR frequency/Selects the calibration frequency for frequencies greater than 7 GHz.
/// HIPAR distance/Selects the distance of the peaks of the microwave calibration
/// HIPAR distance/signal for calibration of the YIG filter.
ViStatus _VI_FUNC rsspecan_ConfigureCalibrationFreqMicrowave(ViSession instrSession,
                                                             ViReal64  frequency,
                                                             ViInt32   distance)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_SERVICE_CAL_FREQ_MICROWAVE, frequency),
            2, "Frequency");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_SERVICE_CAL_FREQ_MW_DIST, distance),
            3, "Distance");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This attribute sets last date and time the instrument was calibrated
/// HIFN  in ISO 8601 format.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR lastCalibrationDate/This control sets last date and time the instrument was calibrated in
/// HIPAR lastCalibrationDate/ISO 8601 format.
ViStatus _VI_FUNC rsspecan_ConfigureCalibrationLastDate(ViSession instrSession,
                                                        ViString  lastCalibrationDate)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViString(instrSession, NULL, RSSPECAN_ATTR_SERVICE_CALIBRATION_LAST_DATE, lastCalibrationDate),
            2, "Last Calibration Date");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This attribute sets next date and time the instrument needs
/// HIFN  calibration to be done in ISO 8601 format. The response may be empty in
/// HIFN  case of no fixed next calibration due.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR nextCalibrationDate/This control sets next date and time the instrument needs calibration
/// HIPAR nextCalibrationDate/to be done in ISO 8601 format. The response may be empty in case of no
/// HIPAR nextCalibrationDate/fixed next calibration due.
ViStatus _VI_FUNC rsspecan_ConfigureCalibrationNextDate(ViSession instrSession,
                                                        ViString  nextCalibrationDate)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViString(instrSession, NULL, RSSPECAN_ATTR_SERVICE_CALIBRATION_NEXT_DATE, nextCalibrationDate),
            2, "Next Calibration Date");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This attribute queries the recommended calibration interval (ISO 8601
/// HIFN  duration).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR calibrationInterval/This control queries the recommended calibration interval (ISO 8601
/// HIPAR calibrationInterval/duration).
ViStatus _VI_FUNC rsspecan_QueryCalibrationInterval(ViSession instrSession,
                                                    ViChar    calibrationInterval[])
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_GetAttributeViString(instrSession, NULL, RSSPECAN_ATTR_SERVICE_CALIBRATION_INTERVAL,
            512, calibrationInterval),
            2, "Calibration Interval");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function shuts down the instrument.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
ViStatus _VI_FUNC rsspecan_SystemShutdown(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_SYSTEM_SHUTDOWN, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Reboots the instrument, including the operating system.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsspecan_SystemReboot(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsspecan_SetAttributeViString(instrSession, NULL, RSSPECAN_ATTR_SYSTEM_REBOOT, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function defines the operating mode that is activated when you
/// HIFN  switch on the R&S FSW or press the PRESET key.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR presetOperatingMode/This control sets the type of limit check for all ranges.
ViStatus _VI_FUNC rsspecan_ConfigurePresetOperatingMode(ViSession instrSession,
                                                        ViInt32   presetOperatingMode)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_PRESET_OPERATING_MODE, presetOperatingMode),
            2, "Preset Operating Mode");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN selects the resolution filter type that is selected after a
/// HIFN preset in the Spectrum application.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR filter/This control selects the resolution filter type.
ViStatus _VI_FUNC rsspecan_ConfigurePresetFilter(ViSession instrSession,
                                                 ViInt32   filter)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_PRESET_FILTER, filter),
            2, "Filter");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN Activates secure user mode.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR secureUserModeEnabled/Activates secure user mode.
ViStatus _VI_FUNC rsspecan_ConfigureSecureUserMode(ViSession instrSession,
                                                   ViBoolean secureUserModeEnabled)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_SECURITY_ENABLED, secureUserModeEnabled),
            2, "Security Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN Defines how remote control programs are evaluated, in particular multiple commands in one command line.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR mode/Selects the mode of how remote control programs are evaluated, in particular multiple commands in one command line.
ViStatus _VI_FUNC rsspecan_SystemTreeWalking(ViSession instrSession,
                                             ViInt32   mode)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_SYSTEM_TREE_WALKING, mode),
            2, "Mode");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Creates an *.xml file with information on installed hardware,
/// HIFN  software, image and FPGA versions. The *.xml file is stored under
/// HIFN  C:\Program Files (x86)\Rohde-Schwarz\FSW\<version>\devicedata\xml\ on
/// HIFN  the instrument.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR bufferSize/This control defines the size of 'Device Footprint' buffer.
/// HIPAR deviceFootprint/Creates an *.xml file with information on installed hardware,
/// HIPAR deviceFootprint/software, image and FPGA versions. The *.xml file is stored under
/// HIPAR deviceFootprint/C:\Program Files (x86)\Rohde-Schwarz\FSW\<version>\devicedata\xml\ on
/// HIPAR deviceFootprint/the instrument.
/// HIPAR returnedValues/Returns the number of characters returned from the instrument.
ViStatus _VI_FUNC rsspecan_QuerySystemInfoDeviceFootprint(ViSession instrSession,
                                                          ViUInt32  bufferSize,
                                                          ViChar    deviceFootprint[],
                                                          ViUInt32  *returnedValues)
{
    ViStatus error = VI_SUCCESS;
    ViString cmd = NULL;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_BuildAttrCommandString(instrSession, &g_RSSPECAN_ATTR_SYSTEM_INFO_DEVICE_FOOTPRINT, NULL, &cmd));
    checkErr(RsCore_QueryViStringUnknownLengthToUserBuffer(instrSession, cmd, bufferSize, deviceFootprint, (ViInt32*)returnedValues));
    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    if (cmd) free(cmd);
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries how many switching cycles the individual relays have performed
/// HIFN  since they were installed.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR relay/Selects the relay.
/// HIPAR cyclesCount/Queries how many switching cycles the individual relays have performed
/// HIPAR cyclesCount/since they were installed.
ViStatus _VI_FUNC rsspecan_QuerySystemInfoSwitchingCyclesCount(
                            ViSession instrSession,
                            ViInt32   relay,
                            ViInt32   *cyclesCount)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViChar   *p2string;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, relay, RSSPECAN_VAL_RELAY_ACDC, RSSPECAN_VAL_RELAY_EXT_RELAIS),
            2, "Relay");
    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &RsSpecAn_rngRelay, relay, &p2string),
            2, "Relay");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "DIAG:INFO:CCO? %s", p2string);
    checkErr(rsspecan_QueryViInt32(instrSession, cmd, cyclesCount));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the BIOS version of the CPU board.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR bufferSize/This control defines the size of 'BIOS Version' buffer.
/// HIPAR biosVersion/Queries the BIOS version of the CPU board.
/// HIPAR returnedValues/Returns the number of characters returned from the instrument.
ViStatus _VI_FUNC rsspecan_QuerySystemInfoBIOSVersion(ViSession instrSession,
                                                      ViUInt32  bufferSize,
                                                      ViChar    biosVersion[],
                                                      ViUInt32  *returnedValues)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_QueryViStringUnknownLengthToUserBuffer(instrSession, "DIAG:SERV:BIOS?", bufferSize, biosVersion, (ViInt32*)returnedValues));
    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the frequency bands used for measurement by the R&S&nbsp;FSW
/// HIFN  hardware. The start frequency of each band is provided.
/// HIFN     The bands are instrument-specific and depend on the currently defined
/// HIFN  RBW, VBW and YIG preselector state. The precise frequency bands are
/// HIFN  required to define correction data for the correct bands, in particular
/// HIFN  for frequency-drifting DUTs.
/// HIFN     This query is only available in zero span mode.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR bufferSize/This control defines the size of 'Frequency Bands' buffer.
/// HIPAR frequencyBands/Queries the frequency bands used for measurement by the R&S&nbsp;FSW
/// HIPAR frequencyBands/hardware. The start frequency of each band is provided.
/// HIPAR returnedValues/Returns the number of characters returned from the instrument.
ViStatus _VI_FUNC rsspecan_QuerySystemInfoFrequencyBands(ViSession instrSession,
                                                         ViUInt32  bufferSize,
                                                         ViChar    frequencyBands[],
                                                         ViUInt32  *returnedValues)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_QueryViStringUnknownLengthToUserBuffer(instrSession, "DEV:INFO:HWB?", bufferSize, frequencyBands, (ViInt32*)returnedValues));
    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the availability of floating licenses for the R&SFSW.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR option/Name of an option for which license is set.
/// HIPAR state/State of the license
/// HIPAR days/The time period that an occupied license is stored locally after it is
/// HIPAR days/retrieved from the license server.
ViStatus _VI_FUNC rsspecan_ConfigureSystemFloatingOptionEnabled(
                            ViSession instrSession,
                            ViString  option,
                            ViInt32   state,
                            ViInt32   days)
{
    ViStatus error = VI_SUCCESS;
    ViChar   param[RS_PARAM_BUF_SIZE];
    ViChar*  stateString;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &RsSpecAn_rngFloatingLicenseState, state, &stateString),
        3, "State");

    switch (state)
    {
        case RSSPECAN_VAL_FLOATING_LICENSE_ON:
        case RSSPECAN_VAL_FLOATING_LICENSE_OFF:
            (void)snprintf(param, RS_PARAM_BUF_SIZE, "\"%s\",%s", option, stateString);
            break;

        case RSSPECAN_VAL_FLOATING_LICENSE_OCCUPY:
            (void)snprintf(param, RS_PARAM_BUF_SIZE, "\"%s\",%s,%ld", option, stateString, days);
            break;
    }

    checkErr(RsCore_SetAttributeViRawString(instrSession, NULL, RSSPECAN_ATTR_HIDDEN_SYSTEM_FLOATING_OPTION_ENABLED, RS_VAL_DIRECT_USER_CALL, param));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the availability of floating licenses on all license servers
/// HIFN  configured on the R&S FSW.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR bufferSize/Size of output array.
/// HIPAR option/Name of an option whose license is queried.
/// HIPAR respose/Returns the availability of floating licenses.
/// HIPAR bytesRead/Returns the number of bytes read from instrument.
ViStatus _VI_FUNC rsspecan_QuerySystemFloatingLicense(ViSession instrSession,
                                                      ViInt32   bufferSize,
                                                      ViString  option,
                                                      ViChar    response[],
                                                      ViInt32*  bytesRead)
{
    ViStatus error = VI_SUCCESS;
    ViChar   param[RS_PARAM_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    if (option)
    {
        (void)snprintf(param, RS_PARAM_BUF_SIZE, "\"%s\"", option);

        checkErr(RsCore_GetAttributeViRawStringWithQparam(instrSession, NULL, RSSPECAN_ATTR_HIDDEN_SYSTEM_FLOATING_LICENSE_QUERY, param, bufferSize, response, bytesRead));
    }
    else
    {
        checkErr(RsCore_GetAttributeViRawStringWithRealLen(instrSession, NULL, RSSPECAN_ATTR_HIDDEN_SYSTEM_FLOATING_LICENSE_QUERY, RS_VAL_DIRECT_USER_CALL, bufferSize, response, bytesRead));
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines whether the 'Display Update' function remains available in
/// HIFN  remote operation or not.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR lockEnabled/Defines whether the 'Display Update' function remains available in
/// HIPAR lockEnabled/remote operation or not.
ViStatus _VI_FUNC rsspecan_ConfigureSystemDisplayLock(ViSession instrSession,
                                                      ViBoolean lockEnabled)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, NULL, RSSPECAN_ATTR_SYSTEM_DISPLAY_LOCK_ENABLED, lockEnabled),
            3, "Lock Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Determines when the remote commands that change hardware settings on
/// HIFN  the R&S FSW are executed.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR hwEnabled/Determines when the remote commands that change hardware settings on
/// HIPAR hwEnabled/the R&S FSW are executed.
ViStatus _VI_FUNC rsspecan_ConfigureSystemChangingHardwareEnabled(
                            ViSession instrSession,
                            ViBoolean hwEnabled)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_CHANGING_HARDWARE_ENABLED, hwEnabled),
            2, "HW Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Executes any received remote commands that cause changes to the
/// HIFN  hardware and have not been executed yet due to a SYST:SHIM:STAT OFF
/// HIFN  command.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsspecan_ConfigureSystemChangingHardware(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_CHANGING_HARDWARE, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Determines availability of trial option T0. It is pre-installed at the
/// HIFN  factory on request.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR enabled/Determines availability of trial option T0. It is pre-installed at the
/// HIPAR enabled/factory on request.
ViStatus _VI_FUNC rsspecan_ConfigureSystemTrialOptionsEnabled(
                            ViSession instrSession,
                            ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_TRIAL_OPTIONS_ENABLED, enabled),
            2, "Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries availability of applications using trial option T0.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR bufferSize/Sets the buffer size.
/// HIPAR trialOptions/Queries availability of applications using trial option T0.
ViStatus _VI_FUNC rsspecan_ConfigureSystemTrialOptions(ViSession instrSession,
                                                       ViInt32   bufferSize,
                                                       ViChar    trialOptions[])
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_GetAttributeViString(instrSession, NULL, RSSPECAN_ATTR_TRIAL_OPTIONS,
            bufferSize, trialOptions),
            3, "Trial Options");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  If enabled, the SCPI Recorder automatically records the required SCPI
/// HIFN  commands and parameter values for the settings and functions you use
/// HIFN  while operating the R&SFSW.
/// HIFN
/// HIFN     See 'Automating tasks with remote command scripts'.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR autoRecording/If enabled, the SCPI Recorder automatically records the required SCPI
/// HIPAR autoRecording/commands and parameter values for the settings and functions you use
/// HIPAR autoRecording/while operating the R&SFSW.
ViStatus _VI_FUNC rsspecan_ConfigureSystemSCPIRecorderAutoRecordingEnabled(
                            ViSession instrSession,
                            ViBoolean autoRecording)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, NULL, RSSPECAN_ATTR_SYSTEM_SCPI_RECORDER_AUTO_RECORDING_ENABLED, autoRecording),
            2, "Auto Recording");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  If enabled, additional commands are included in the script to
/// HIFN  synchronize the recorded commands when necessary. For instance, when a
/// HIFN  measurement is started, a *WAI command is inserted to ensure that the
/// HIFN  next command is only executed after the measurement has finished.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR addSyncCommands/If enabled, additional commands are included in the script to
/// HIPAR addSyncCommands/synchronize the recorded commands when necessary. For instance, when a
/// HIPAR addSyncCommands/measurement is started, a *WAI command is inserted to ensure that the
/// HIPAR addSyncCommands/next command is only executed after the measurement has finished.
ViStatus _VI_FUNC rsspecan_ConfigureSystemSCPIRecorderAddSyncCommands(
                            ViSession instrSession,
                            ViBoolean addSyncCommands)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, NULL, RSSPECAN_ATTR_SYSTEM_SCPI_RECORDER_ADD_SYNC_COMMANDS, addSyncCommands),
            2, "Add Sync Commands");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Returns the currently recorded commands.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR bufferSize/Size of output array.
/// HIPAR record/Returns the currently recorded commands.
/// HIPAR bytesRead/Returns the number of bytes read from instrument.
ViStatus _VI_FUNC rsspecan_QuerySystemSCPIRecorderListRecording(
                            ViSession instrSession,
                            ViInt32   bufferSize,
                            ViChar    record[],
                            ViInt32*  bytesRead)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeViStringWithRealLen(instrSession, NULL, RSSPECAN_ATTR_SYSTEM_SCPI_RECORDER_LIST_RECORDING, RS_VAL_DIRECT_USER_CALL, bufferSize, record, bytesRead));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Exports the current SCPI command list to the specified file and
/// HIFN  directory in the selected format. By default, the file is stored in the
/// HIFN  C:\R_S\INSTR\ScpiRecordings directory. Besides the recorded commands
/// HIFN  themselves, the exported script includes all format-specific header
/// HIFN  data required to execute the script using an external program on the
/// HIFN  controller.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR format/Specifies the format for export.
/// HIPAR filename/String containing the path and name of the file.
ViStatus _VI_FUNC rsspecan_ExportSystemSCPIRecorder(ViSession instrSession,
                                                    ViInt32   format,
                                                    ViString  filename)
{
    ViStatus error = VI_SUCCESS;
    ViChar   param[RS_PARAM_BUF_SIZE];
    ViChar*  formatString;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &RsSpecAn_rngSCPIRecorderFormat, format, &formatString),
        2, "Format");

    (void)snprintf(param, RS_PARAM_BUF_SIZE, "%s,\"%s\"", formatString, filename);

    checkErr(RsCore_SetAttributeViRawString(instrSession, NULL, RSSPECAN_ATTR_HIDDEN_SYSTEM_SCPI_RECORDER_EXPORT, RS_VAL_DIRECT_USER_CALL, param));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Removes all recorded commands from the current SCPI command list.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsspecan_SystemSCPIRecorderClearAll(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsspecan_SetAttributeViString(instrSession, NULL, RSSPECAN_ATTR_SYSTEM_SCPI_RECORDER_CLEAR_ALL, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/*===========================================================================*
 *------------------- Display -----------------------------------------------*
 *===========================================================================*/
/// HIFN This function configures the display settings.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR statusBarState/This control switches the statusbar under the diagram on or off.
/// HIPAR softkeyBarState/This control switches the softkey bar on the right side of the
/// HIPAR softkeyBarState/diagram on or off.
/// HIPAR toolBarState/This control switches the toolbar above the diagram on or off.
/// HIPAR touchscreenState/This control switches the touch screen functionality on, off, or
/// HIPAR touchscreenState/partially on.
/// HIPAR channelInformation/Hides or displays the channel bar information.
/// HIPAR frequency/Switches the x-axis annotation on or off.
/// HIPAR time/Switches on or off the screen display of date and time.
/// HIPAR softFrontpanel/Enables or disables instrument display when in remote control mode.
ViStatus _VI_FUNC rsspecan_ConfigureDisplay(ViSession instrSession,
                                            ViBoolean statusBarState,
                                            ViBoolean softkeyBarState,
                                            ViBoolean toolBarState,
                                            ViInt32   touchscreenState,
                                            ViBoolean channelInformation,
                                            ViBoolean frequency,
                                            ViBoolean time,
                                            ViBoolean softFrontpanel)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_DISP_STATUS_BAR, statusBarState),
            2, "Status Bar State");

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_DISP_SOFTKEY_BAR, softkeyBarState),
            3, "Softkey Bar State");

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_DISP_TOOL_BAR, toolBarState),
            4, "Tool Bar State");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_DISP_TOUCHSCREEN_STATE, touchscreenState),
            5, "Touchscreen State");
    if (rsspecan_IsFSV(instrSession) == VI_FALSE)
    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_DISP_CHANNEL_BAR_STATE, channelInformation),
            6, "Channel Information");

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_DISP_FREQ_STATE, frequency),
            7, "Frequency");

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_DISP_TIME_STATE, time),
            8, "Time");

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_DISP_FP_KEYS, softFrontpanel),
            9, "Soft Front Panel");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function configures power-save mode of the display.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR state/This control switches on or off the power-save mode of the
/// HIPAR state/display.
/// HIPAR holdoff/Sets the holdoff time for the power-save mode of the display.
ViStatus _VI_FUNC rsspecan_ConfigureDisplayPowerSave(ViSession instrSession,
                                                     ViBoolean state,
                                                     ViInt32   holdoff)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_DISP_PWR_SAVE_STATE, state),
            2, "State");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_DISP_PWR_SAVE_HOLDOFF, holdoff),
            3, "Hold Off");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function selects a theme for the screen display.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR theme/This control selects a theme for the screen display.
ViStatus _VI_FUNC rsspecan_ConfigureDisplayThemeSelect(ViSession instrSession,
                                                       ViString  theme)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_DISP_SELECT_THEME, theme),
            2, "Theme");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function displays the selection list of available themes for
/// HIFN the screen display.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR number_ofThemes/This control returns number of names of all themes.
/// HIPAR bufferSize/Pass the number of bytes in the string you specify for the
/// HIPAR bufferSize/catalog list parameter.
/// HIPAR themeList/This control returns the names of all themes stored on the hard
/// HIPAR themeList/disk.
ViStatus _VI_FUNC rsspecan_GetDisplayThemeCatalog(ViSession instrSession,
                                                  ViInt32   *numberOfThemes,
                                                  ViInt32   bufferSize,
                                                  ViChar    themeList[])
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32RangeMin(instrSession, bufferSize, 0),
        3, "Buffer Size");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, themeList),
        4, "Theme List");

    checkErr(RsCore_QueryCatalog(instrSession, "DISP:THEM:CAT?", bufferSize, themeList, numberOfThemes));

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function moves the focus area to the selected window and
/// HIFN subwindow.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR subwindow/Document parameter here.
ViStatus _VI_FUNC rsspecan_ConfigureDisplayFocusedArea(ViSession instrSession,
                                                       ViInt32   window,
                                                       ViInt32   subwindow)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap_subwindow[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_DISPLAY_FOCUS_AREA,
            "Subwindow", subwindow - 1, RS_REPCAP_BUF_SIZE, repCap_subwindow),
            3, "Subwindow");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_DISPLAY_FOCUS_AREA,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap),
            2, "Window");
    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_subwindow, NULL);

    checkErr(rsspecan_SetAttributeViString(instrSession, repCap, RSSPECAN_ATTR_DISPLAY_FOCUS_AREA, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function controls the apperance of display title.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR state/This control switches the x-axis annotation on or off. The
/// HIPAR state/driver uses this value to set the RSSPECAN_ATTR_DISP_COMMENT_STATE attribute.
/// HIPAR comment/Defines a comment (max. 20 characters) which can be displayed on
/// HIPAR comment/the screen in the selected measurement window.
ViStatus _VI_FUNC rsspecan_ConfigureDisplayComment(ViSession instrSession,
                                                   ViInt32   reserved,
                                                   ViBoolean state,
                                                   ViString  comment)
{
    ViStatus error = VI_SUCCESS;
             reserved;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_DISP_COMMENT_STATE, state),
            3, "State");

    viCheckParm(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_DISP_COMMENT, comment),
            4, "Comment");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function switches the measurement result display between
/// HIFN FULL SCREEN and SPLIT SCREEN.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR format/Switches the measurement result display between FULL SCREEN and
/// HIPAR format/SPLIT SCREEN.
ViStatus _VI_FUNC rsspecan_ConfigureDisplayFormat(ViSession instrSession,
                                                  ViInt32   format)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_DISP_FORMAT, format),
            2, "Format");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function switches the measurement window for channel and
/// HIFN adjacent-channel power measurements to full screen or half screen.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR windowSize/Switches the measurement window for channel and adjacent-channel
/// HIPAR windowSize/power measurements to full screen or half screen.
ViStatus _VI_FUNC rsspecan_ConfigureDisplayWindowSize(ViSession instrSession,
                                                      ViInt32   windowSize)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_DISP_WINDOW_SIZE, windowSize),
            2, "Window Size");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function configures the behaviour of the display during a
/// HIFN single sweep.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR state/This control configures the behaviour of the display during a
/// HIPAR state/single sweep.
ViStatus _VI_FUNC rsspecan_ConfigureDisplaySingleSweep(ViSession instrSession,
                                                       ViBoolean state)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_DISP_SINGLE_SWEEP, state),
            2, "State");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN This function switches on or off the update of all display
/// HIFN elements during remote control.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR displayInRemote/This control enables or disables instrument display when in
/// HIPAR displayInRemote/remote control mode.
ViStatus _VI_FUNC rsspecan_ConfigureDisplayUpdate(ViSession instrSession,
                                                  ViBoolean displayInRemote)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_DISP_UPDATE, displayInRemote),
            2, "Display In Remote");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function turns a measurement window on or off.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR state/Turns a measurement window on or off.
ViStatus _VI_FUNC rsspecan_ConfigureDisplayMeasurementWindowState(
                            ViSession instrSession,
                            ViInt32   window,
                            ViBoolean state)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_DISPLAY_STATE,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap),
            2, "Window");

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, repCap, RSSPECAN_ATTR_DISPLAY_STATE, state),
            3, "State");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function selects the  active measurement window.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control returns the index number of the currently active measurement window.

ViStatus _VI_FUNC rsspecan_DisplaySetActiveWindow(ViSession instrSession,
                                                  ViInt32   window)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    //Should use RSSPECAN_ATTR_SELECT_WINDOW, but attribute is strange

    if (rsspecan_IsFSV(instrSession) == VI_TRUE)
    {
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "DISP:WIND%ld:SEL", window);
        checkErr(RsCore_Write(instrSession, cmd));
        checkErr(rsspecan_CheckStatus(instrSession));
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Switches between the MultiView tab and the most recently displayed
/// HIFN  channel. If only one channel is active, this attribute has no effect.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR multiviewTabEnabled/Switches between the MultiView tab and the most recently displayed
/// HIPAR multiviewTabEnabled/channel. If only one channel is active, this control has no effect.
ViStatus _VI_FUNC rsspecan_ConfigureDisplayMultiViewTabEnabled(
                            ViSession instrSession,
                            ViBoolean multiviewTabEnabled)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, NULL, RSSPECAN_ATTR_DISPLAY_MULTIVIEW_TAB_ENABLED, multiviewTabEnabled),
            3, "MultiView Tab Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects the time and date format.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR displayTimeFormat/Selects the time and date format.
ViStatus _VI_FUNC rsspecan_ConfigureDisplayTimeFormat(ViSession instrSession,
                                                      ViInt32   displayTimeFormat)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, NULL, RSSPECAN_ATTR_DISPLAY_TIME_FORMAT, displayTimeFormat),
            3, "Display Time Format");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Turns the zoom on and off.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR subwindow/This control selects the measurement subwindow.
/// HIPAR state/Turns the zoom on and off.
ViStatus _VI_FUNC rsspecan_ConfigureDisplaySubwindowZoomState(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   subwindow,
                            ViBoolean state)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, window, 0, 16),
            2, "Window");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, subwindow, 1, 64),
            3, "Subwindow");

    snprintf (repCap, RS_REPCAP_BUF_SIZE, "Win%ld,SW%ld", window, subwindow);

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, repCap, RSSPECAN_ATTR_SUBWINDOW_DISPLAY_ZOOM_STATE, state),
            4, "State");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function defines the zoom area. To define a zoom area, you first
/// HIFN  have to turn the zoom on.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR subwindow/This control selects the measurement subwindow.
/// HIPAR enableZoom/This parameter turns the multiple zoom on and off.
/// HIPAR x1/Diagram coordinates in % of the complete diagram that define the
/// HIPAR y1/Diagram coordinates in % of the complete diagram that define the
/// HIPAR x2/Diagram coordinates in % of the complete diagram that define the
/// HIPAR y2/Diagram coordinates in % of the complete diagram that define the
ViStatus _VI_FUNC rsspecan_ConfigureDisplaySubwindowZoom(ViSession instrSession,
                                                         ViInt32   window,
                                                         ViInt32   subwindow,
                                                         ViBoolean enableZoom,
                                                         ViReal64  x1,
                                                         ViReal64  y1,
                                                         ViReal64  x2,
                                                         ViReal64  y2)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsspecan_ConfigureDisplaySubwindowZoomState(instrSession, window, subwindow, enableZoom));

    if (enableZoom)
    {

        viCheckParm(RsCore_InvalidViReal64Range(instrSession, x1, 0.0, 100.0), 6, "x1");
        viCheckParm(RsCore_InvalidViReal64Range(instrSession, y1, 0.0, 100.0), 7, "y1");
        viCheckParm(RsCore_InvalidViReal64Range(instrSession, x2, 0.0, 100.0), 8, "x2");
        viCheckParm(RsCore_InvalidViReal64Range(instrSession, y2, 0.0, 100.0), 9, "y2");

        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "DISP:WIND%ld:SUBW%ld:ZOOM:AREA %.12lG,%.12lG,%.12lG,%.12lG",
                window, subwindow, x1, y1, x2, y2);
        checkErr(RsCore_Write(instrSession, cmd));

        checkErr(rsspecan_CheckStatus(instrSession));
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This attribute defines the scale type of the Y axis (absolute or
/// HIFN  relative) in the selected measurement window.
/// HIFN     When SYSTem:DISPlay is set to OFF, this command has no immediate
/// HIFN  effect on the screen.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR subwindow/This control selects the measurement subwindow.
/// HIPAR gridMode/This attribute defines the scale type of the Y axis (absolute or
/// HIPAR gridMode/relative) in the selected measurement window.
ViStatus _VI_FUNC rsspecan_ConfigureDisplaySubwindowAmplitudeGridMode(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   subwindow,
                            ViInt32   gridMode)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, window, 0, 16),
            2, "Window");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, subwindow, 1, 64),
            3, "Subwindow");

    snprintf (repCap, RS_REPCAP_BUF_SIZE, "Win%ld,SW%ld", window, subwindow);

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_SUBWINDOW_DISPLAY_AMPLITUDE_GRID_MODE, gridMode),
            4, "Grid Mode");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the reference value assigned to the reference position in the
/// HIFN  specified subwindow. Separate reference values are maintained for the
/// HIFN  various displays.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR subwindow/This control selects the measurement subwindow.
/// HIPAR referenceLevel/Defines the reference value assigned to the reference position in the
/// HIPAR referenceLevel/specified subwindow. Separate reference values are maintained for the
/// HIPAR referenceLevel/various displays.
ViStatus _VI_FUNC rsspecan_ConfigureDisplaySubwindowReferenceLevel(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   subwindow,
                            ViReal64  referenceLevel)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, window, 0, 16),
            2, "Window");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, subwindow, 1, 64),
            3, "Subwindow");

    snprintf (repCap, RS_REPCAP_BUF_SIZE, "Win%ld,SW%ld", window, subwindow);

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_SUBWINDOW_DISPLAY_REFERENCE_LEVEL, referenceLevel),
            4, "Reference Level");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/*===========================================================================*
 *------------------- Layout ------------------------------------------------*
 *===========================================================================*/
/// HIFN This function adds a window to the display.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init.
/// HIPAR stringSize/This control sets the allocated length of Returned Window Name string.
/// HIPAR referenceWindowName/String containing the number or name of the window the new window is inserted next to.
/// HIPAR direction/Direction the window is added relative to the window set with "Reference Window Name".
/// HIPAR windowType/Type of result display you want to add.
/// HIPAR returnedWindowName/Returns the name of the new window.
ViStatus _VI_FUNC rsspecan_AddWindow(ViSession instrSession,
                                     ViInt32   stringSize,
                                     ViString  referenceWindowName,
                                     ViInt32   direction,
                                     ViInt32   windowType,
                                     ViChar    returnedWindowName[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViChar   response[RS_MAX_MESSAGE_BUF_SIZE];
    ViChar   *p2direction;
    ViChar   *p2type;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &rsspecan_rngLayoutDirection, direction, &p2direction),
            4, "Direction");
    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &rsspecan_rngLayoutQueryWindowType, windowType, &p2type),
            5, "Window Type");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "LAY:ADD? '%s',%s,%s", referenceWindowName, p2direction, p2type);
    checkErr(RsCore_QueryViString(instrSession, cmd, response));

    if (returnedWindowName != NULL)
    {
        RsCore_TrimString(response, RS_VAL_TRIM_WHITESPACES_AND_ALL_QUOTES);
        RsCore_CopyToUserBufferAsciiData(returnedWindowName, stringSize, response);
    }

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function removes a window from the display.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init.
/// HIPAR windowName/String containing the number or name of the window the new window is inserted next to.
ViStatus _VI_FUNC rsspecan_RemoveWindow(ViSession instrSession,
                                        ViString  windowName)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_LAYOUT_WINDOW_REMOVE, windowName),
            2, "Window Name");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function replaces a window with another one.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init.
/// HIPAR windowName/String containing the number or name of the window the new window is inserted next to.
/// HIPAR windowType/Type of result display you want to add.
ViStatus _VI_FUNC rsspecan_ReplaceWindow(ViSession instrSession,
                                         ViString  windowName,
                                         ViInt32   windowType)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViChar   *p2type;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &rsspecan_rngLayoutQueryWindowType, windowType, &p2type),
            3, "Window Type");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "LAY:REPL '%s',%s", windowName, p2type);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Moves the window on the screen.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR windowName1/String containing the name of an existing window that is to be moved.
/// HIPAR windowName2/String containing the name of an existing window the selected window
/// HIPAR windowName2/is placed next to or replaces.
/// HIPAR direction/Destination the selected window is moved to, relative to the reference
/// HIPAR direction/window.
ViStatus _VI_FUNC rsspecan_MoveWindow(ViSession instrSession,
                                      ViString  windowName1,
                                      ViString  windowName2,
                                      ViInt32   direction)
{
    ViStatus error = VI_SUCCESS;
    ViChar   *p2direction;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &rsspecan_rngLayoutDirection, direction, &p2direction),
            4, "Direction");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "LAY:MOVE '%s','%s',%s", windowName1, windowName2, p2direction);
    checkErr(RsCore_Write(instrSession, cmd));
    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function queries the name and index of all active windows.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init.
/// HIPAR bufferSize/This control specifies the size of the buffer.
/// HIPAR results/Returns the name and index of all active windows.
ViStatus _VI_FUNC rsspecan_QueryWindowsList(ViSession instrSession,
                                            ViInt32   bufferSize,
                                            ViChar    results[])
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_GetAttributeViString(instrSession, "", RSSPECAN_ATTR_LAYOUT_WINDOW_LIST,
            bufferSize, results),
            3, "Results");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN This function queries the index of a particular display window.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init.
/// HIPAR windowName/String containing the number or name of the window the new window is inserted next to.
/// HIPAR index/Index number of the window.
ViStatus _VI_FUNC rsspecan_QueryWindowIndex(ViSession instrSession,
                                            ViString  windowName,
                                            ViInt32   *index)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "LAY:IDEN? '%s'", windowName);
    checkErr(RsCore_QueryViInt32(instrSession, cmd, index));

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function queries the name of a particular display window.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init.
/// HIPAR stringSize/This control sets the allocated length of Returned Window Name string.
/// HIPAR index/Index number of the window.
/// HIPAR windowName/String containing the number or name of the window the new window is inserted next to.
ViStatus _VI_FUNC rsspecan_QueryWindowName(ViSession instrSession,
                                           ViInt32   stringSize,
                                           ViInt32   index,
                                           ViChar    windowName[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_LAYOUT_QUERY_WINDOW_NAME,
            "Window", index , RS_REPCAP_BUF_SIZE, repCap));
    viCheckParm(rsspecan_GetAttributeViString(instrSession, repCap, RSSPECAN_ATTR_LAYOUT_QUERY_WINDOW_NAME,
            stringSize, windowName),
            3, "Window Name");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN This function changes the position of a splitter
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init.
/// HIPAR firstWindowName/String containing the number or name of the first window the splitter controls.
/// HIPAR secondWindowName/String containing the number or name of the second window the splitter controls.
/// HIPAR position/New vertical or horizontal position of the splitter in percent of the screen area.
ViStatus _VI_FUNC rsspecan_ConfigureSplitterPosition(ViSession instrSession,
                                                     ViString  firstWindowName,
                                                     ViString  secondWindowName,
                                                     ViInt32   position)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "LAY:SPL '%s','%s',%ld", firstWindowName, secondWindowName, position);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function resets the screen colors of all display items to
/// HIFN their default settings.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR defaultSetting/This control selects the predefined setting to be applied.
ViStatus _VI_FUNC rsspecan_DisplayColorDefault(ViSession instrSession,
                                               ViInt32   defaultSetting)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, defaultSetting, 1, 2),
            2, "Default Settings");

    checkErr(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_DISP_COL_PRESET, defaultSetting));

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/*===========================================================================*
 *------------------- Display Color Settings --------------------------------*
 *===========================================================================*/
/// HIFN This function defines the color table of the instrument.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR colorMap/This control defines the color table of the instrument.
/// HIPAR tint/This control sets the hue = TINT.
/// HIPAR saturation/This control sets the sat = SATURATION.
/// HIPAR brightness/This control sets the lum = BRIGHTNESS.
ViStatus _VI_FUNC rsspecan_ConfigureDisplayColor(ViSession instrSession,
                                                 ViInt32   colorMap,
                                                 ViReal64  tint,
                                                 ViReal64  saturation,
                                                 ViReal64  brightness)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, colorMap, 1, 41),
            2, "Color Map");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, tint, 0.0, 100.0), 3, "TINT");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, saturation, 0.0, 100.0), 4, "SATURATION");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, brightness, 0.0, 100.0), 5, "BRIGHTNESS");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "DISP:CMAP%ld:HSL %.12lG,%.12lG,%.12lG", colorMap, tint / 100, saturation / 100, brightness / 100);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function defines the color table of the instrument using
/// HIFN predefined colors.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR colorMap/This control defines the color table of the instrument.
/// HIPAR predefinedColors/This control defines the color table of the instrument using
/// HIPAR predefinedColors/predefined color values.
ViStatus _VI_FUNC rsspecan_ConfigureDisplayColorByPredefined(ViSession instrSession,
                                                             ViInt32   colorMap,
                                                             ViInt32   predefinedColors)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_DISP_COL_PREDEFINED,
            "CMAP", colorMap - 1, RS_REPCAP_BUF_SIZE, repCap));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_DISP_COL_PREDEFINED, predefinedColors),
            3, "Predefined Colors");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/*===========================================================================*
 *------------------- Hardcopy Color Settings -------------------------------*
 *===========================================================================*/
/// HIFN This function resets the colors for a hardcopy to the selected
/// HIFN default settings.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR color/This control sets color or monochrome hardcopy of the screen.
/// HIPAR defaultSetting/Document parameter here.
ViStatus _VI_FUNC rsspecan_HardcopyColor(ViSession instrSession,
                                         ViBoolean color,
                                         ViInt32   defaultSetting)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_HCOPY_COLOR, color),
            2, "Color");

    if (color == VI_TRUE)
    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_HCOPY_COLOR_DEF, defaultSetting),
            3, "Default Setting");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function defines the color table in USER DEFINED COLORS
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR colorMap/This control defines the color table of the instrument.
/// HIPAR tint/This control sets the hue = TINT.
/// HIPAR saturation/This control sets the sat = SATURATION.
/// HIPAR brightness/This control sets the lum = BRIGHTNESS.
ViStatus _VI_FUNC rsspecan_ConfigureHardcopyColor(ViSession instrSession,
                                                  ViInt32   colorMap,
                                                  ViReal64  tint,
                                                  ViReal64  saturation,
                                                  ViReal64  brightness)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, colorMap, 1, 41),
            2, "Color Map");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, tint, 0.0, 100.0), 3, "TINT");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, saturation, 0.0, 100.0), 4, "SATURATION");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, brightness, 0.0, 100.0), 5, "BRIGHTNESS");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "HCOP:CMAP%ld:HSL %.12lG,%.12lG,%.12lG", colorMap, tint / 100.0, saturation / 100.0, brightness / 100.0);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function defines the color table in USER DEFINED COLORS
/// HIFN using predefined color values.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR colorMap/This control defines the color table of the instrument.
/// HIPAR predefinedColors/This control defines the color table of the instrument using
/// HIPAR predefinedColors/predefined color values.
ViStatus _VI_FUNC rsspecan_ConfigureHardcopyColorByPredefined(
                            ViSession instrSession,
                            ViInt32   colorMap,
                            ViInt32   predefinedColors)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_HCOPY_COLOR_PREDEFINED,
            "CMAP", colorMap - 1, RS_REPCAP_BUF_SIZE, repCap));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_HCOPY_COLOR_PREDEFINED, predefinedColors),
            3, "Predefined Colors");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Adds an application to the specified group in the Application Starter
/// HIFN  dialog box.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR applicationGroup/The group (tab) of applications in the Application Starter that the
/// HIPAR applicationGroup/application belongs to.
/// HIPAR name/Name of the application in the Application Starter
/// HIPAR filePath/Path and file name of the application
ViStatus _VI_FUNC rsspecan_AddApplication(ViSession instrSession,
                                          ViInt32   applicationGroup,
                                          ViString  name,
                                          ViString  filePath)
{
    ViStatus error = VI_SUCCESS;
    ViChar   *p2string;
    ViChar   param[RS_PARAM_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &RsSpecAn_rngApplicationGroup, applicationGroup, &p2string),
                2, "Application Group");

    snprintf (param, RS_PARAM_BUF_SIZE, "%s,'%s','%s'", p2string, name, filePath);

    checkErr(rsspecan_SetAttributeViString(instrSession, NULL, RSSPECAN_ATTR_HIDDEN_APPLICATION_STARTER_ADD_APPLICATION, param));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Removes the specified application from the Application Starter dialog
/// HIFN  box.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR applicationGroup/The group (tab) of applications in the Application Starter that the
/// HIPAR applicationGroup/application belongs to.
/// HIPAR name/Name of the application in the Application Starter
ViStatus _VI_FUNC rsspecan_DeleteApplication(ViSession instrSession,
                                             ViInt32   applicationGroup,
                                             ViString  name)
{
    ViStatus error = VI_SUCCESS;
    ViChar*  p2appGroup;
    ViChar   param[RS_PARAM_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &RsSpecAn_rngApplicationGroup, applicationGroup, &p2appGroup),
                2, "Application Group");

    snprintf (param, RS_PARAM_BUF_SIZE, "%s,'%s'", p2appGroup, name);

    checkErr(rsspecan_SetAttributeViString(instrSession, NULL, RSSPECAN_ATTR_HIDDEN_APPLICATION_STARTER_DELETE_APPLICATION, param));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Selects the application to be executed in subsequent commands.
/// HIFN
/// HIFN     The query returns the currently selected application.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR applicationGroup/The group (tab) of applications in the Application Starter that the
/// HIPAR applicationGroup/application belongs to.
/// HIPAR name/Name of the application in the Application Starter
ViStatus _VI_FUNC rsspecan_SelectApplication(ViSession instrSession,
                                             ViInt32   applicationGroup,
                                             ViString  name)
{
    ViStatus error = VI_SUCCESS;
    ViChar   *p2appGroup;
    ViChar   param[RS_PARAM_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &RsSpecAn_rngApplicationGroup, applicationGroup, &p2appGroup),
                3, "Window Type");

    snprintf (param, RS_PARAM_BUF_SIZE, "%s,'%s'", p2appGroup, name);

    checkErr(rsspecan_SetAttributeViString(instrSession, NULL, RSSPECAN_ATTR_HIDDEN_APPLICATION_STARTER_SELECT, param));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Starts the specified application directly on the R&SFSW.
/// HIFN
/// HIFN     Note that for data security reasons, executable applications cannot be
/// HIFN  added to the Application Starter remotely. Only applications added to
/// HIFN  the firmware manually in advance can be executed remotely.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR applicationGroup/The group (tab) of applications in the Application Starter that the
/// HIPAR applicationGroup/application belongs to.
/// HIPAR name/Name of the application in the Application Starter
ViStatus _VI_FUNC rsspecan_ExecuteApplication(ViSession instrSession,
                                              ViInt32   applicationGroup,
                                              ViString  name)
{
    ViStatus error = VI_SUCCESS;
    ViChar   param[RS_PARAM_BUF_SIZE];
    ViChar*  applicationGroupString;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &RsSpecAn_rngApplicationGroup, applicationGroup, &applicationGroupString),
        2, "Application Group");

    (void)snprintf(param, RS_PARAM_BUF_SIZE, "%s,\"%s\"", applicationGroupString, name);

    checkErr(RsCore_SetAttributeViRawString(instrSession, NULL, RSSPECAN_ATTR_HIDDEN_APPLICATION_STARTER_EXECUTE, RS_VAL_DIRECT_USER_CALL, param));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the working directory used by the selected application.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR directoryPath/Defines the working directory used by the selected application.
ViStatus _VI_FUNC rsspecan_ConfigureApplicationWorkingDirectory(
                            ViSession instrSession,
                            ViString  directoryPath)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViString(instrSession, NULL, RSSPECAN_ATTR_APPLICATION_STARTER_WORKING_DIRECTORY, directoryPath),
            2, "Directory Path");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Defines an icon for the selected application to be displayed in the
/// HIFN  'Application Starter' dialog box.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR iconPath/Path and file name for icon
/// HIPAR index/Index
ViStatus _VI_FUNC rsspecan_ConfigureApplicationIcon(ViSession instrSession,
                                                    ViString  iconPath,
                                                    ViString  index)
{
    ViStatus error = VI_SUCCESS;
    ViChar   param[RS_PARAM_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf (param, RS_PARAM_BUF_SIZE, "'%s','%s'", iconPath, index);

    checkErr(rsspecan_SetAttributeViString(instrSession, NULL, RSSPECAN_ATTR_HIDDEN_APPLICATION_STARTER_CHOOSE_ICON, param));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Defines a new path to the selected application.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR path/Defines a new path to the selected application.
ViStatus _VI_FUNC rsspecan_ConfigureApplicationNewPath(ViSession instrSession,
                                                       ViString  path)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViString(instrSession, NULL, RSSPECAN_ATTR_APPLICATION_STARTER_PATH, path),
            2, "Path");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Defines the name of the application as displayed in the 'Application
/// HIFN  Starter' dialog box.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR name/Defines the name of the application as displayed in the 'Application
/// HIPAR name/Starter' dialog box.
ViStatus _VI_FUNC rsspecan_ConfigureApplicationName(ViSession instrSession,
                                                    ViString  name)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViString(instrSession, NULL, RSSPECAN_ATTR_APPLICATION_STARTER_DISPLAY_NAME, name),
            2, "Name");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Defines parameters that are provided with the application for
/// HIFN  execution, for example arguments for a custom script.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR applicationStarterParameters/Defines parameters that are provided with the application for
/// HIPAR applicationStarterParameters/execution, for example arguments for a custom script.
ViStatus _VI_FUNC rsspecan_ConfigureApplicationExecutionParameters(
                            ViSession instrSession,
                            ViString  applicationStarterParameters)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViString(instrSession, NULL, RSSPECAN_ATTR_APPLICATION_STARTER_PARAMETERS, applicationStarterParameters),
            2, "Application Starter Parameters");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/*===========================================================================*
 *------------------- Network -----------------------------------------------*
 *===========================================================================*/
/// HIFN  The instrument switches between manual and remote operation. If the
/// HIFN  local lockout function (RSSPECAN_ATTR_LOCK_LOCAL_KEY) has been
/// HIFN  activated in the remote control mode, manual operation is no longer
/// HIFN  available until the lock was released.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR remoteMode/The instrument switches between manual and remote operation. If the
/// HIPAR remoteMode/local lockout function (RSSPECAN_ATTR_LOCK_LOCAL_KEY) has been
/// HIPAR remoteMode/activated in the remote control mode, manual operation is no longer
/// HIPAR remoteMode/available until the lock was released.
ViStatus _VI_FUNC rsspecan_ConfigureRemoteModeEnabled(ViSession instrSession,
                                                      ViBoolean remoteMode)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, NULL, RSSPECAN_ATTR_NETWORK_REMOTE_MODE_ENABLED, remoteMode),
            2, "Remote Mode");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function Turns logging of remote commands on and off.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init
/// HIPAR instrSession/or rsspecan_InitWithOptions function.
/// HIPAR state/Turns logging of remote commands on and off.
ViStatus _VI_FUNC rsspecan_ConfigureRemoteLoggingState(ViSession instrSession,
                                                       ViBoolean state)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_REMOTE_LOGGING_STATE, state),
            2, "State");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures GPIB settings.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR address/Sets the GPIB address of the R&SFSW.
/// HIPAR terminator/This command selects the GPIB receive terminator.
ViStatus _VI_FUNC rsspecan_ConfigureGPIBSettings(ViSession instrSession,
                                                 ViInt32   address,
                                                 ViInt32   terminator)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, NULL, RSSPECAN_ATTR_GPIB_ADDRESS, address),
            2, "Address");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, NULL, RSSPECAN_ATTR_GPIB_TERMINATOR, terminator),
            3, "Terminator");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Configures LAN settings.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instrumentDescription/Defines the 'LAN' instrument description.
/// HIPAR password/Defines the LAN password.
ViStatus _VI_FUNC rsspecan_ConfigureLANSettings(ViSession instrSession,
                                                ViString  instrumentDescription,
                                                ViString  password)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViString(instrSession, NULL, RSSPECAN_ATTR_LAN_INSTRUMENT_DESCRIPTION, instrumentDescription),
            2, "Instrument Desription");

    viCheckParm(rsspecan_SetAttributeViString(instrSession, NULL, RSSPECAN_ATTR_LAN_PASSWORD, password),
            3, "Password");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Resets the LAN configuration, as well as the 'LAN' password and
/// HIFN  instrument description.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsspecan_ResetLANSettings(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsspecan_SetAttributeViString(instrSession, NULL, RSSPECAN_ATTR_LAN_RESET, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Configures HUMS settings.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR enabled/Turns the HUMS service and data collection on and off.
/// HIPAR dataFormat/Defines the format for the queried HUMS data. You can query the HUMS
/// HIPAR dataFormat/data either in JSON format or XML format.
ViStatus _VI_FUNC rsspecan_ConfigureHUMSGeneralSettings(ViSession instrSession,
                                                        ViBoolean enabled,
                                                        ViInt32   dataFormat)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, NULL, RSSPECAN_ATTR_HUMS_ENABLED, enabled),
            2, "Enabled");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, NULL, RSSPECAN_ATTR_HUMS_DATA_FORMAT, dataFormat),
            3, "Data Format");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Deletes the complete HUMS data. This includes device history, device
/// HIFN  tags, SCPI connections, utilization history and utilizations.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsspecan_DeleteHUMSAllData(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsspecan_SetAttributeViString(instrSession, NULL, RSSPECAN_ATTR_HUMS_DELETE_ALL_DATA, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Returns all key-value tags that you have assigend to the instrument.
/// HIFN  Depending on the set data format, the queried data is either displayed
/// HIFN  in XML or JSON format. For more information about setting the data
/// HIFN  format, see RSSPECAN_ATTR_NETWORK_HUMS_DATA_FORMAT
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR bufferSize/Buffer size allocated for query.
/// HIPAR tagCatalog/Returns the Tag Catalog. Each entry in the catalog has ID, Key and
/// HIPAR tagCatalog/Value specified.
/// HIPAR bytesRead/Returns the number of bytes read from instrument.
ViStatus _VI_FUNC rsspecan_QueryHUMSTagCatalog(ViSession instrSession,
                                               ViInt32   bufferSize,
                                               ViChar    tagCatalog[],
                                               ViInt32*  bytesRead)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeViRawStringWithRealLen(instrSession, NULL,
                    RSSPECAN_ATTR_HIDDEN_HUMS_TAG_CATALOG,
                    RS_VAL_DIRECT_USER_CALL,
                    bufferSize, tagCatalog, bytesRead),
                3, "Tag Catalog");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This command adds or modifies a key-value pair (device tag).
/// HIFN
/// HIFN     The query returns the key-value pair for a given ID or an empty string
/// HIFN  if the ID is unknown.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR id/ID number of the tag you want to modify. Values 0-31
/// HIPAR key/String containing key name of the tag.
/// HIPAR value/String containing value of the tag.
ViStatus _VI_FUNC rsspecan_AddHUMSTag(ViSession instrSession,
                                      ViInt32   id,
                                      ViString  key,
                                      ViString  value)
{
    ViStatus error = VI_SUCCESS;
    ViChar   param[RS_PARAM_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf (param, RS_PARAM_BUF_SIZE, "%ld,'%s','%s'", id, key, value);

    checkErr(rsspecan_SetAttributeViString(instrSession, NULL, RSSPECAN_ATTR_HIDDEN_HUMS_TAG_VALUE, param));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function deletes a certain tag you assigned to your instrument,
/// HIFN  including its key and value.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR tagID/ID of the tag for deletion.
ViStatus _VI_FUNC rsspecan_DeleteHUMSTag(ViSession instrSession,
                                         ViInt32   tagID)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, NULL, RSSPECAN_ATTR_HUMS_TAG_DELETE, tagID),
            2, "Tag ID");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Deletes all key-value tags you have assigned to the instrument.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsspecan_DeleteAllHUMSTags(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsspecan_SetAttributeViString(instrSession, NULL, RSSPECAN_ATTR_HUMS_TAG_DELETE_ALL, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function adds or modifies a key-value pair (device tag).
/// HIFN
/// HIFN     The query returns the key-value pair for a given ID or an empty string
/// HIFN  if the ID is unknown.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR id/ID number of the tag you want to query.
/// HIPAR key/String containing key name of the queried tag.
/// HIPAR value/String containing value of the queried tag.
ViStatus _VI_FUNC rsspecan_QueryHUMSTagByID(ViSession instrSession,
                                            ViInt32   id,
                                            ViInt32   bufferSize,
                                            ViChar    keyValuePair[],
                                            ViInt32*  returnedBytes)
{
    ViStatus error = VI_SUCCESS;
    ViChar   param[RS_PARAM_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf(param, RS_PARAM_BUF_SIZE, "%ld", id);

    viCheckParm(RsCore_GetAttributeViRawStringWithQparam(instrSession, NULL, RSSPECAN_ATTR_HIDDEN_HUMS_TAG_VALUE, param, bufferSize, keyValuePair, returnedBytes),
                4, "Key Value Pair");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This attribute selects the SNMP version.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR version/This control selects the SNMP version.
ViStatus _VI_FUNC rsspecan_ConfigureHUMSSNMPVersion(ViSession instrSession,
                                                    ViInt32   version)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, NULL, RSSPECAN_ATTR_HUMS_SNMP_VERSION, version),
            2, "Version");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This attribute returns information about the HUMS REST status and
/// HIFN  enables or disables the public HUMS REST endpoints.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR state/This control returns information about the HUMS REST status and
/// HIPAR state/enables or disables the public HUMS REST endpoints.
ViStatus _VI_FUNC rsspecan_QueryHUMSRESTState(ViSession  instrSession,
                                              ViBoolean* state)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_GetAttributeViBoolean(instrSession, NULL, RSSPECAN_ATTR_HUMS_REST_STATE, state),
            2, "State");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Sets the SNMP contact and location information for the administrator.
/// HIFN  Not setting the information via SCPI persistently allows to change the
/// HIFN  string via SNMP.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR contact/Administrator's contact information.
/// HIPAR location/Administrator's location information.
ViStatus _VI_FUNC rsspecan_ConfigureHUMSSNMPAdminInformation(ViSession instrSession,
                                                             ViString  contact,
                                                             ViString  location)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViString(instrSession, NULL, RSSPECAN_ATTR_HUMS_SNMP_CONTACT, contact),
            2, "Contact");

    viCheckParm(rsspecan_SetAttributeViString(instrSession, NULL, RSSPECAN_ATTR_HUMS_SNMP_LOCATION, location),
            3, "Location");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Sets the SNMP community string for read-only or read/write access.
/// HIFN  Select an SNMP version that supports communities
/// HIFN  (RSSPECAN_ATTR_HUMS_SNMP_VERSION).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR access/Selects the access for community string is definition.
/// HIPAR communityString/Sets the SNMP community string access. Select an SNMP version that
/// HIPAR communityString/supports communities (RSSPECAN_ATTR_HUMS_SNMP_VERSION).
ViStatus _VI_FUNC rsspecan_ConfigureHUMSSNMPCommunity(ViSession instrSession,
                                                      ViInt32   access,
                                                      ViString  communityString)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    switch (access)
    {
        case RSSPECAN_VAL_ACCESS_READ_ONLY:
            viCheckParm(rsspecan_SetAttributeViString(instrSession, NULL, RSSPECAN_ATTR_HUMS_SNMP_COMMUNITY_READ_ONLY, communityString),
                    3, "Community String");
            break;
        case RSSPECAN_VAL_ACCESS_READ_WRITE:
            viCheckParm(rsspecan_SetAttributeViString(instrSession, NULL, RSSPECAN_ATTR_HUMS_SNMP_COMUNITY_READ_WRITE, communityString),
                    3, "Community String");
            break;
        default:
            viCheckParm(RsCore_InvalidViInt32Range(instrSession, access, RSSPECAN_VAL_ACCESS_READ_ONLY, RSSPECAN_VAL_ACCESS_READ_WRITE),2, "Access")
            break;
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Defines a SNMP user profile.
/// HIFN
/// HIFN     Prerequisites for this command:
/// HIFN
/// HIFN     Select SNMPv3 (RSSPECAN_ATTR_HUMS_SNMP_VERSION).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR user/String containing name of the user.
/// HIPAR authenticationPassword/String containing the authentication password.
/// HIPAR privacyPassword/String containing the privacy password.
ViStatus _VI_FUNC rsspecan_AddHUMSSNMPv3User(ViSession instrSession,
                                             ViString  user,
                                             ViString  authenticationPassword,
                                             ViString  privacyPassword)
{
    ViStatus error = VI_SUCCESS;
    ViChar   param[RS_PARAM_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf (param, RS_PARAM_BUF_SIZE, "'%s','%s','%s'", user, authenticationPassword, privacyPassword);

    checkErr(rsspecan_SetAttributeViString(instrSession, NULL, RSSPECAN_ATTR_HIDDEN_HUMS_SNMP_V3_USER_ADD, param));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Deletes a specific SNMP user profile.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR user/Deletes a specific SNMP user profile.
ViStatus _VI_FUNC rsspecan_DeleteHUMSSNMPUser(ViSession instrSession,
                                              ViString  user)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViString(instrSession, NULL, RSSPECAN_ATTR_HUMS_SNMP_USER_DELETE, user),
            2, "User");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Deletes all SNMP user profiles.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsspecan_DeleteAllHUMSSNMPUsers(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsspecan_SetAttributeViString(instrSession, NULL, RSSPECAN_ATTR_HUMS_SNMP_V3_USER_DELETE_ALL, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This attribute returns the number of users and a list of all SNMP
/// HIFN  users for SNMPv3. To use this attribute select SNMP v3
/// HIFN  (RSSPECAN_ATTR_HUMS_SNMP_VERSION).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR bufferSize/Size of output array.
/// HIPAR userList/This control returns the number of users and a list of all SNMP users
/// HIPAR userList/for SNMPv3. To use this control select SNMP v3
/// HIPAR userList/(RSSPECAN_ATTR_HUMS_SNMP_VERSION).
/// HIPAR bytesRead/Returns the number of bytes read from instrument.
ViStatus _VI_FUNC rsspecan_QueryHUMSSNMPv3UserList(ViSession instrSession,
                                                   ViInt32   bufferSize,
                                                   ViChar    userList[])
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_GetAttributeViString(instrSession, NULL, RSSPECAN_ATTR_HUMS_SNMP_V3_USER_LIST,
            bufferSize, userList),
            3, "User List");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/*===========================================================================*
 *------------------- Measurement -------------------------------------------*
 *===========================================================================*/
/// HIFN This function initiates a signal acquisition based on the
/// HIFN present instrument configuration. It then waits for the acquisition
/// HIFN to complete, and returns the trace as an array of amplitude values.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR trace/Selects the trace for Detector definition.
/// HIPAR maximumTime/Pass the maximum length of time in which to allow the read
/// HIPAR maximumTime/waveform operation to complete.
/// HIPAR arrayLength/Number of array points requested.
/// HIPAR actualPoints/Number of points actually returned in the Amplitude array and X
/// HIPAR actualPoints/Array array.
/// HIPAR amplitude/A user allocated buffer into which the trace amplitudes is
/// HIPAR amplitude/stored.
ViStatus _VI_FUNC rsspecan_ReadYTrace(ViSession instrSession,
                                      ViInt32   window,
                                      ViInt32   trace,
                                      ViUInt32  maximumTime,
                                      ViInt32   arrayLength,
                                      ViInt32   *actualPoints,
                                      ViReal64  amplitude[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, window, 0, 16),
            2, "Window");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, trace, 1, 6),
            3, "Trace");
    viCheckParm(RsCore_InvalidViUInt32Range(instrSession, maximumTime, 0, 4294967295UL), 4, "Maximum Time");

    checkErr(rsspecan_Initiate(instrSession, 0, maximumTime));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "TRACE%ld", trace);
    checkErr(rsspecan_dataReadTrace(instrSession, window, cmd, arrayLength, amplitude, actualPoints));

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function returns the trace the spectrum analyzer acquires.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR trace/Selects the trace for Detector definition.
/// HIPAR arrayLength/Number of array points requested.
/// HIPAR actualPoints/Number of points actually returned in the Amplitude array and X
/// HIPAR actualPoints/Array array.
/// HIPAR amplitude/A user allocated buffer into which the trace amplitudes is
/// HIPAR amplitude/stored.
ViStatus _VI_FUNC rsspecan_FetchYTrace(ViSession instrSession,
                                       ViInt32   window,
                                       ViInt32   trace,
                                       ViInt32   arrayLength,
                                       ViInt32   *actualPoints,
                                       ViReal64  amplitude[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   traceCmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, window, 0, 16),
            2, "Window");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, trace, 1, 6),
            3, "Trace");

    snprintf(traceCmd, RS_MAX_MESSAGE_BUF_SIZE, "TRACE%ld", trace);
    checkErr(rsspecan_dataReadTrace(instrSession, window, traceCmd, arrayLength, amplitude, actualPoints));

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function queries the previously acquired trace data for the
/// HIFN specified trace from the memory.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR trace/Selects the trace.
/// HIPAR offsetSweepPoint/The offset in sweep points related to the start of the
/// HIPAR offsetSweepPoint/measurement at which data retrieval is to start.
/// HIPAR numberOfSweeps/Number of sweep points to be retrieved from the trace.
/// HIPAR arrayLength/Number of array points requested.
/// HIPAR actualPoints/Number of points actually returned in the array.
/// HIPAR amplitude/A user allocated buffer into which the trace amplitudes is
/// HIPAR amplitude/stored.
ViStatus _VI_FUNC rsspecan_ReadYTracePrevious(ViSession instrSession,
                                              ViInt32   window,
                                              ViInt32   trace,
                                              ViInt32   offsetSweepPoint,
                                              ViInt32   numberOfSweeps,
                                              ViInt32   arrayLength,
                                              ViInt32   *actualPoints,
                                              ViReal64  amplitude[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, window, 0, 16),
            2, "Window");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, trace, 1, 6),
            3, "Trace");

    if (window != 0)
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, ":FORM REAL,32;:TRAC%ld:MEM? TRACE%ld,%ld,%ld", window, trace, offsetSweepPoint, numberOfSweeps);
    else
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, ":FORM REAL,32;:TRAC:MEM? TRACE%ld,%ld,%ld", trace, offsetSweepPoint, numberOfSweeps);
    checkErr(RsCore_QueryFloatArrayToUserBuffer(instrSession, cmd, arrayLength, amplitude, actualPoints));

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN Reads the x values (time in seconds) of the measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or rsspecan_InitWithOptions function.
/// HIPAR window/This control selects the measurement window.
/// HIPAR trace/Selects the trace for Detector definition.
/// HIPAR arrayLength/Number of array points requested.
/// HIPAR actualPoints/Number of points actually returned in the array.
/// HIPAR x/A user allocated buffer into which the trace X values is stored.
ViStatus _VI_FUNC rsspecan_FetchXTrace(ViSession instrSession,
                                       ViInt32   window,
                                       ViInt32   trace,
                                       ViInt32   arrayLength,
                                       ViInt32   *actualPoints,
                                       ViReal64  x[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, window, 0, 16),
            2, "Window");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, trace, 1, 6),
            3, "Trace");

    if (window == 0)
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "FORM REAL,32;:TRAC:DATA:X? TRACE%ld", trace);
    else
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "FORM REAL,32;:TRAC%ld:DATA:X? TRACE%ld", window, trace);
    checkErr(RsCore_QueryFloatArrayToUserBuffer(instrSession, cmd, arrayLength, x, actualPoints));

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function queries the result of the limit check of the limit
/// HIFN line.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR limit/This control selects limit.
/// HIPAR state/The result of the limit check responds
ViStatus _VI_FUNC rsspecan_GetLimitCheckResult(ViSession instrSession,
                                               ViInt32   reserved,
                                               ViInt32   limit,
                                               ViInt32   *state)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_MAX_MESSAGE_BUF_SIZE] = "";
             reserved;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_LIMIT_CHECK_RESULT,
            "Limit", limit - 1, RS_REPCAP_BUF_SIZE, repCap),
            3, "Limit");

    viCheckParm(rsspecan_GetAttributeViInt32(instrSession, repCap, RSSPECAN_LIMIT_CHECK_RESULT, state),
            4, "State");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/*===========================================================================*
 *------------------- Low Level Measurement ---------------------------------*
 *===========================================================================*/
/// HIFN This function initiates an acquisition
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR timeout/This control sets the timeout value.
ViStatus _VI_FUNC rsspecan_Initiate(ViSession instrSession,
                                    ViInt32   reserved,
                                    ViUInt32  timeout)
{
    ViStatus error = VI_SUCCESS;
    ViUInt32 old_timeout = 0;
             reserved;

    checkErr(RsCore_LockSession(instrSession));

    if (timeout != 0)
    {
        checkErr(rsspecan_GetTimeout(instrSession, &old_timeout));
        checkErr(rsspecan_SetTimeout(instrSession, timeout));

        checkErr(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_INIT, NULL));
    }
    else
    {
        checkErr(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_INIT_WITHOUT_OPC, NULL));
    }

Error:
    if (old_timeout > 0)
        rsspecan_SetTimeout(instrSession, old_timeout);
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function starts a new spurious measurement and waits till
/// HIFN sweeping is finished.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR timeout/This control sets the timeout value.
ViStatus _VI_FUNC rsspecan_InitiateSpurious(ViSession instrSession,
                                            ViInt32   reserved,
                                            ViUInt32  timeout)
{
    ViStatus error = VI_SUCCESS;
    ViUInt32 old_timeout = 0;
             reserved;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViUInt32Range(instrSession, timeout, 0, 4294967295UL), 3, "Timeout");

    checkErr(rsspecan_GetTimeout(instrSession, &old_timeout));
    checkErr(rsspecan_SetTimeout(instrSession, timeout));
    checkErr(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_INIT_SPURIOUS, NULL));

Error:
    if (old_timeout > 0)
        rsspecan_SetTimeout(instrSession, old_timeout);
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function starts a Spectrum Emission Mask measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR timeout/This control sets the timeout value.
ViStatus _VI_FUNC rsspecan_InitiateSEMMeasurement(ViSession instrSession,
                                                  ViInt32   reserved,
                                                  ViUInt32  timeout)
{
    ViStatus error = VI_SUCCESS;
    ViUInt32 old_timeout = 0;
             reserved;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViUInt32Range(instrSession, timeout, 0, 4294967295UL), 3, "Timeout");

    checkErr(rsspecan_GetTimeout(instrSession, &old_timeout));
    checkErr(rsspecan_SetTimeout(instrSession, timeout));
    checkErr(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_INIT_SEM_MEAS, NULL));

Error:
    if (old_timeout > 0)
        rsspecan_SetTimeout(instrSession, old_timeout);
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function aborts a previously initiated measurement and
/// HIFN returns the spectrum analyzer to the idle state.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
ViStatus _VI_FUNC rsspecan_Abort(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_MEAS_ABORT, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function continues a stopped measurement at the current
/// HIFN position in single sweep mode.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR timeout/This control sets the timeout value.
ViStatus _VI_FUNC rsspecan_Continue(ViSession instrSession,
                                    ViInt32   reserved,
                                    ViUInt32  timeout)
{
    ViStatus error = VI_SUCCESS;
    ViUInt32 old_timeout = 0;
             reserved;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViUInt32Range(instrSession, timeout, 0, 4294967295UL), 3, "Timeout");

    checkErr(rsspecan_GetTimeout(instrSession, &old_timeout));
    checkErr(rsspecan_SetTimeout(instrSession, timeout));

    checkErr(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_INIT_CONMEAS, NULL));

Error:
    if (old_timeout > 0)
        rsspecan_SetTimeout(instrSession, old_timeout);
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function sends a software trigger to the instrument.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR timeout/This control sets the timeout value.
ViStatus _VI_FUNC rsspecan_SendSoftwareTrigger(ViSession instrSession,
                                               ViUInt32  timeout)
{
    ViStatus error = VI_SUCCESS;
    ViUInt32 old_timeout = 0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViUInt32Range(instrSession, timeout, 0, 4294967295UL), 2, "Timeout");

    checkErr(rsspecan_GetTimeout(instrSession, &old_timeout));
    checkErr(rsspecan_SetTimeout(instrSession, timeout));
    checkErr(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_INIT_SW_TRIGGER, NULL));

Error:
    if (old_timeout > 0)
        rsspecan_SetTimeout(instrSession, old_timeout);
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN his function returns the current number of started sweeps.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR numberOfSweeps/Returns the current sweep number.
ViStatus _VI_FUNC rsspecan_GetSweepNumber(ViSession instrSession,
                                          ViInt32   reserved,
                                          ViInt32   *numberOfSweeps)
{
    ViStatus error = VI_SUCCESS;
             reserved;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_GetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_SWEEP_COUNT_CURRENT, numberOfSweeps),
            3, "Number Of Sweeps");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/*===========================================================================*
 *------------------- Sequencer ---------------------------------------------*
 *===========================================================================*/
/// HIFN  This function starts a new sequence of measurements by the Sequencer.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR timeout/This control sets the timeout value.
ViStatus _VI_FUNC rsspecan_InitiateSequencer(ViSession instrSession,
                                             ViUInt32  timeout)
{
    ViStatus error = VI_SUCCESS;
    ViUInt32 old_timeout = 0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViUInt32Range(instrSession, timeout, 0, 4294967295UL), 3, "Timeout");

    checkErr(rsspecan_GetTimeout(instrSession, &old_timeout));
    checkErr(rsspecan_SetTimeout(instrSession, timeout));

    checkErr(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_INIT_SEQUENCER, NULL));

Error:
    if (old_timeout > 0)
        rsspecan_SetTimeout(instrSession, old_timeout);
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function stops the currently active sequence of measurements. The
/// HIFN  Sequencer itself is not deactivated, so you can start a new sequence
/// HIFN  immediately.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsspecan_AbortSequencer(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_ABORT_SEQUENCER, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function is only available if the Sequencer is deactivated
/// HIFN  (SYSTem:SEQuencer OFF) and only in MSRA mode. The data in the capture
/// HIFN  buffer is re-evaluated by all active MSRA applications.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsspecan_RefreshSequencer(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_REFRESH_SEQUENCER, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function turns the Sequencer on and off. The Sequencer must be
/// HIFN  active before any other Sequencer commands (INIT:SEQ...) are executed,
/// HIFN  otherwise an error will occur.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR state/This control switches the state of the sequencer.
ViStatus _VI_FUNC rsspecan_ConfigureSequencerState(ViSession instrSession,
                                                   ViBoolean state)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_SEQUENCER_STATE, state),
            2, "State");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function selects the way the R&S FSW application performs
/// HIFN  measurements sequentially. Before this function can be executed, the
/// HIFN  Sequencer must be activated.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR sequencerMode/This control selects the way the R&S FSW application performs
/// HIPAR sequencerMode/measurements sequentially.
ViStatus _VI_FUNC rsspecan_ConfigureSequencerMode(ViSession instrSession,
                                                  ViInt32   sequencerMode)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_SEQUENCER_MODE, sequencerMode),
            2, "Sequencer Mode");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Calculates the time trigger.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsspecan_CalculateSequencerTimeTrigger(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsspecan_SetAttributeViString(instrSession, NULL, RSSPECAN_ATTR_SEQUENCER_CALCULATE_TIME_TRIGGER, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/*===========================================================================*
 *------------------- Spectrum Analyze Measurement --------------------------*
 *===========================================================================*/
/// HIFN This function configures the power measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init
/// HIPAR instrSession/or rsspecan_InitWithOptions function.
/// HIPAR state/This control sets the state of selected power measurement.
/// HIPAR powerMeasurement/Selects the ACP (adjacent channel power) standard.
ViStatus _VI_FUNC rsspecan_SelectPowerMeasurement(ViSession instrSession,
                                                  ViInt32   reserved,
                                                  ViBoolean state,
                                                  ViInt32   powerMeasurement)
{
    ViStatus error = VI_SUCCESS;
             reserved;

    checkErr(RsCore_LockSession(instrSession));

    if (state)
    {
        viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_MEAS_POW_SELECT, powerMeasurement),
                4, "Power Measurement");
    }
    else
    {
        checkErr(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_MEAS_POW_OFF, NULL));
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function adjusts the frequency span, the measurement
/// HIFN bandwidths and the detector as required for the number of
/// HIFN channels, the channel bandwidths and the channel spacings
/// HIFN selected in the active power measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.
/// HIPAR channelPowerType/Specifies the channel power standard.
/// HIPAR timeout/This control sets the timeout value.
ViStatus _VI_FUNC rsspecan_PowerPresetMeasurement(ViSession instrSession,
                                                  ViInt32   reserved,
                                                  ViInt32   channelPowerType,
                                                  ViUInt32  timeout)
{
    ViStatus error = VI_SUCCESS;
    ViUInt32 old_timeout = 0;
             reserved;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViUInt32Range(instrSession, timeout, 0, 4294967295UL), 4, "Timeout");

    checkErr(rsspecan_GetTimeout(instrSession, &old_timeout));
    checkErr(rsspecan_SetTimeout(instrSession, timeout));
    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_MEAS_POW_ADJ_PRESET, channelPowerType),
            3, "Channel Power Type");

Error:
    if (old_timeout > 0)
        rsspecan_SetTimeout(instrSession, old_timeout);
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function returns the power measurement results.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.
/// HIPAR powerMeasurement/Selects the ACP (adjacent channel power) standard.
/// HIPAR arraySize/Defines the memory size allocated for Results.
/// HIPAR results/Returns the measured values:
/// HIPAR returnedValues/Returns number of returned values.
ViStatus _VI_FUNC rsspecan_QueryPowerResults(ViSession instrSession,
                                             ViInt32   reserved,
                                             ViInt32   powerMeasurement,
                                             ViInt32   arraySize,
                                             ViReal64  results[],
                                             ViInt32   *returnedValues)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViChar   *p2string;

    checkErr(RsCore_LockSession(instrSession));

    reserved;
    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &rsspecan_rngMeasPowerSelect, powerMeasurement, &p2string),
            3, "Power Measurement");
    viCheckParm(RsCore_InvalidViInt32RangeMin(instrSession, arraySize, 0),
        4, "Array Size");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, results),
        5, "Results");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CALC:MARK:FUNC:POW:RES? %s", p2string);
    checkErr(RsCore_QueryFloatArrayToUserBuffer(instrSession, cmd, arraySize, results, returnedValues));

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/*===========================================================================*
 *------------------- List Evaluation  --------------------------------------*
 *===========================================================================*/
/// HIFN This function configures the list of settings (max. 100 entries
/// HIFN for an individual frequency point) for the multiple power
/// HIFN measurement and starts a measurement sequence.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR noofListItems/Set number of list items.
/// HIPAR analyzerFrequency/Array of receive frequency for the signal to be measured (=
/// HIPAR analyzerFrequency/center frequency in manual operation).
/// HIPAR referenceLevel/Array of reference level values for each list item.
/// HIPAR RFInputAttenuation/Array of RF input attenuation values for each list item.
/// HIPAR RFInputElectronicAttn/Array of RF input attenuation values of electronic attenuator
/// HIPAR RFInputElectronicAttn/for each list item.
/// HIPAR filterType/Array of filter types for each list item.
/// HIPAR resolutionBandwidth/Array of resolution bandwidth values for each list item.
/// HIPAR videoBandwidth/Array of video bandwidth values for each list item.
/// HIPAR measTime/Array of measurement time values for each list item.
/// HIPAR triggerLevel/Array of trigger level values for each list item.
ViStatus _VI_FUNC rsspecan_ConfigureListPowerSequence(ViSession instrSession,
                                                      ViInt32   reserved,
                                                      ViInt32   noofListItems,
                                                      ViReal64  analyzerFrequency[],
                                                      ViReal64  referenceLevel[],
                                                      ViReal64  RFInputAttenuation[],
                                                      ViReal64  RFInputElectronicAttn[],
                                                      ViInt32   filterType[],
                                                      ViReal64  resolutionBandwidth[],
                                                      ViReal64  videoBandwidth[],
                                                      ViReal64  measTime[],
                                                      ViReal64  triggerLevel[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   *cmd = NULL;
    ViChar   *p2buf;
    ViChar   *p2string;
    ViInt32  i = 0;
             reserved;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, noofListItems, 1, 100),
            3, "No Of List Items");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, analyzerFrequency),
        4, "Analyzer Frequency");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, referenceLevel),
        5, "Reference Level");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, RFInputAttenuation),
        6, "RF Input Attenuation");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, RFInputElectronicAttn),
        7, "RF Input Electronic Attn");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, filterType),
        8, "Filter Type");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, resolutionBandwidth),
        9, "Resolutin Bandwidth");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, videoBandwidth),
        10, "Video Bandwidth");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, measTime),
        11, "Meas Time");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, triggerLevel),
        12, "Trigger Level");

    /*
     * Alloc buffer. Size is calclulated as:
     * size of header + size required for one entry * number of entries
     */

    viCheckAlloc(cmd = (ViChar*)malloc((size_t)(20 + 240 * noofListItems)));
    p2buf = cmd;
    p2buf += sprintf(p2buf, ":SENS:LIST:POW:SEQ ");

    for (i = 0; i < noofListItems; i++)
    {
        viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &rsspecan_rngBandFiltType, filterType[i], &p2string),
                7, "Filter Type");
        // Electronic attenuator is not in the signal path.
        if (RFInputElectronicAttn[i] < 0)
        {
            p2buf += sprintf(p2buf, "%.12lG,%.12lG,%.12lG,OFF,%s,%.12lG,%.12lG,%.12lG,%.12lG,",
                             analyzerFrequency[i],
                             referenceLevel[i],
                             RFInputAttenuation[i],
                             p2string, // RFInputEAttn[i],
                             resolutionBandwidth[i],
                             videoBandwidth[i],
                             measTime[i],
                             triggerLevel[i]);
        }
        else
        {
            p2buf += sprintf(p2buf, "%.12lG,%.12lG,%.12lG,%.12lG,%s,%.12lG,%.12lG,%.12lG,%.12lG,",
                             analyzerFrequency[i],
                             referenceLevel[i],
                             RFInputAttenuation[i],
                             RFInputElectronicAttn[i],
                             p2string,
                             resolutionBandwidth[i],
                             videoBandwidth[i],
                             measTime[i],
                             triggerLevel[i]);
        }
    }

    *--p2buf = '\0'; // Remove remaining comma

    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    if (cmd) free(cmd);
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function defines the constant settings for the list during
/// HIFN multiple power measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.
/// HIPAR peakMeas/Activates (deactivates) the measurement of the peak power (peak
/// HIPAR peakMeas/detector).
/// HIPAR RMSMeas/Activates (deactivates) the measurement of the RMS power (RMS
/// HIPAR RMSMeas/detector).
/// HIPAR AVGMeas/Activates (deactivates) the measurement of the average power
/// HIPAR AVGMeas/(average detector).
/// HIPAR triggerMode/Selection of the trigger source used for the list measurement.
/// HIPAR triggerSlope/Selection of the trigger source used for the list measurement.
/// HIPAR triggerOffset/This control sets the offset between the detection of the
/// HIPAR triggerOffset/trigger signal and the start of the measurement at the next
/// HIPAR triggerOffset/frequency point.
/// HIPAR gateLength/Gate length with Gated Sweep.
ViStatus _VI_FUNC rsspecan_ConfigureListPowerSet(ViSession instrSession,
                                                 ViInt32   reserved,
                                                 ViBoolean peakMeas,
                                                 ViBoolean RMSMeas,
                                                 ViBoolean AVGMeas,
                                                 ViInt32   triggerMode,
                                                 ViInt32   triggerSlope,
                                                 ViReal64  triggerOffset,
                                                 ViReal64  gateLength)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViChar   *p2triggMode;
    ViChar   *p2triggSlope;
             reserved;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &rsspecan_rngTriggerSource, triggerMode, &p2triggMode),
            6, "Trigger Mode");
    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &rsspecan_rngPolarity, triggerSlope, &p2triggSlope),
            7, "Trigger Slope");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, triggerOffset, 0.0, 100.0), 8, "Trigger Offset");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, gateLength, 0.0, 100.0), 9, "Gate Length");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SENS:LIST:POWer:SET %s,%s,%s,%s,%s,%.12f,%.12f", peakMeas ? "ON" : "OFF", RMSMeas ? "ON" : "OFF", AVGMeas ? "ON" : "OFF",
             p2triggMode, p2triggSlope,
             triggerOffset, gateLength);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function deactivates the list measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
ViStatus _VI_FUNC rsspecan_ListPowerMeasurementOff(ViSession instrSession,
                                                   ViInt32   reserved)
{
    ViStatus error = VI_SUCCESS;
             reserved;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_LIST_POW_STATE_OFF, ""));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function returns the list of settings (max. 100 entries for
/// HIFN an individual frequency point) for the multiple power
/// HIFN measurement, starts a measurement sequence and returns the
/// HIFN measured data.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR noofListItems/Set number of list items.
/// HIPAR analyzerFrequency/Array of receive frequency for the signal to be measured (=
/// HIPAR analyzerFrequency/center frequency in manual operation).
/// HIPAR referenceLevel/Array of reference level values for each list item.
/// HIPAR RFInputAttenuation/Array of RF input attenuation values for each list item.
/// HIPAR RFInputElectronicAttn/Array of RF input attenuation values of electronic attenuator
/// HIPAR RFInputElectronicAttn/for each list item.
/// HIPAR filterType/Array of filter types for each list item.
/// HIPAR resolutionBandwidth/Array of resolution bandwidth values for each list item.
/// HIPAR videoBandwidth/Array of video bandwidth values for each list item.
/// HIPAR measTime/Array of measurement time values for each list item.
/// HIPAR triggerLevel/Array of trigger level values for each list item.
/// HIPAR listPowerResults/This control returns array of elements as the result of a
/// HIPAR listPowerResults/previously configured and initiated list measurement.
ViStatus _VI_FUNC rsspecan_QueryListPowerSequence(ViSession instrSession,
                                                  ViInt32   reserved,
                                                  ViInt32   noofListItems,
                                                  ViReal64  analyzerFrequency[],
                                                  ViReal64  referenceLevel[],
                                                  ViReal64  RFInputAttenuation[],
                                                  ViReal64  RFInputElectronicAttn[],
                                                  ViInt32   filterType[],
                                                  ViReal64  resolutionBandwidth[],
                                                  ViReal64  videoBandwidth[],
                                                  ViReal64  measTime[],
                                                  ViReal64  triggerLevel[],
                                                  ViReal64  listPowerResults[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   *pwrite_buffer = NULL;
    ViChar   *pbuffer;
    ViChar   *p2string;
    ViInt32  i = 0;
             reserved;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, noofListItems, 1, 100),
            3, "No Of List Items");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, analyzerFrequency),
        4, "Analyzer Frequency");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, referenceLevel),
        5, "Reference Level");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, RFInputAttenuation),
        6, "RF Input Attenuation");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, RFInputElectronicAttn),
        7, "RF Input Electronic Attn");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, filterType),
        8, "Filter Type");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, resolutionBandwidth),
        9, "Resolutin Bandwidth");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, videoBandwidth),
        10, "Video Bandwidth");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, measTime),
        11, "Meas Time");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, triggerLevel),
        12, "Trigger Level");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, listPowerResults),
        13, "List Power Results");

    viCheckAlloc(pwrite_buffer = (ViChar*)malloc((size_t)(20 + 240 * (size_t)noofListItems)));
    pbuffer = pwrite_buffer;
    pbuffer += sprintf(pbuffer, ":SENS:LIST:POW:SEQ? ");

    for (i = 0; i < noofListItems; i++)
    {
        viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &rsspecan_rngBandFiltType, filterType[i], &p2string),
                7, "Filter Type");
        // Electronic attenuator is not in the signal path.
        if (RFInputElectronicAttn[i] < 0)
            pbuffer += sprintf(pbuffer, "%.12lG,%.12lG,%.12lG,OFF,%s,%.12lG,%.12lG,%.12lG,%.12lG,",
                               analyzerFrequency[i], referenceLevel[i], RFInputAttenuation[i],
                               /* RFInputEAttn[i], */ p2string, resolutionBandwidth[i],
                               videoBandwidth[i], measTime[i], triggerLevel[i]);
        else
            pbuffer += sprintf(pbuffer, "%.12lG,%.12lG,%.12lG,%.12lG,%s,%.12lG,%.12lG,%.12lG,%.12lG,",
                               analyzerFrequency[i], referenceLevel[i], RFInputAttenuation[i],
                               RFInputElectronicAttn[i], p2string, resolutionBandwidth[i],
                               videoBandwidth[i], measTime[i], triggerLevel[i]);
    }

    *--pbuffer = '\0'; // Remove remaining comma

    checkErr(RsCore_QueryFloatArrayToUserBuffer(instrSession, pwrite_buffer, noofListItems, listPowerResults, NULL));

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    if (pwrite_buffer) free(pwrite_buffer);
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN This function queries the result of a previous list measurement
/// HIFN as configured and initiated with List Power Sequence.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.
/// HIPAR noofResults/Expected number of values to be returned from the instrument.
/// HIPAR listPowerResults/This control returns array of elements as the result of a
/// HIPAR listPowerResults/previous list measurement as configured and initiated with List
/// HIPAR listPowerResults/Power Sequence (confListPwrSeq).
/// HIPAR returnedValues/Returns the number of Power Results.
ViStatus _VI_FUNC rsspecan_QueryListPowerResult(ViSession instrSession,
                                                ViInt32   reserved,
                                                ViInt32   noofResults,
                                                ViReal64  listPowerResults[],
                                                ViInt32   *returnedValues)
{
    ViStatus error = VI_SUCCESS;
             reserved;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32RangeMin(instrSession, noofResults, 1),
        3, "No Of Results");

    checkErr(RsCore_QueryFloatArrayToUserBuffer(instrSession, "SENS:LIST:POW:RES?", noofResults, listPowerResults, returnedValues));
    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/*===========================================================================*
 *---------- Channel Power Measurement/Adjacent Channel Power Measurement ---*
 *===========================================================================*/
/// HIFN This function configures the standard for channel power
/// HIFN measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init
/// HIPAR instrSession/or rsspecan_InitWithOptions function.
/// HIPAR ACPStandard/Selects the ACP (adjacent channel power) standard.
ViStatus _VI_FUNC rsspecan_ConfigureChannelPowerStandard(ViSession instrSession,
                                                         ViInt32   reserved,
                                                         ViInt32   ACPStandard)
{
    ViStatus error = VI_SUCCESS;
             reserved;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_MEAS_POW_STANDARD, ACPStandard),
            3, "ACP Standard");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function configures the user standard for channel power
/// HIFN measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init
/// HIPAR instrSession/or rsspecan_InitWithOptions function.
/// HIPAR operation/This control selects the operation to be performed with user
/// HIPAR operation/standard.
/// HIPAR userStandardName/This control selects name of ACP user standard.
ViStatus _VI_FUNC rsspecan_ConfigureChannelPowerUserStandard(ViSession instrSession,
                                                             ViInt32   reserved,
                                                             ViInt32   operation,
                                                             ViString  userStandardName)
{
    ViStatus error = VI_SUCCESS;
             reserved;

    checkErr(RsCore_LockSession(instrSession));

    switch (operation)
    {
    case RSSPECAN_VAL_USTD_SET:
        viCheckParm(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_ACP_USER_STD_SET, userStandardName),
                4, "User Standard Name");
        break;
    case RSSPECAN_VAL_USTD_SAVE:
        viCheckParm(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_ACP_USER_STD_SAVE, userStandardName),
                4, "User Standard Name");
        break;
    case RSSPECAN_VAL_USTD_DEL:
        viCheckParm(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_ACP_USER_STD_DELETE, userStandardName),
                4, "User Standard Name");
        break;
    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, operation), 3, "Operation");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function configures the adjacent power channels
/// HIFN measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
/// HIPAR noOfTXChannels/Sets the number of TX channels.
/// HIPAR noOfAdjChannels/The start frequency of the frequency sweep.
/// HIPAR channelMode/Sets absolute and relative adjacent channel measurement.
/// HIPAR powerMode/Sets the Clear Write or Maxhold for Channel Power values.
/// HIPAR powerUnit/Sets the query response of the power measurement results in the
/// HIPAR powerUnit/indicated measurement window between output of absolute values
/// HIPAR powerUnit/(OFF) and output referred to the measurement bandwidth (ON).
/// HIPAR noiseCorrection/Activates or deactivates a noise correction measurement.
/// HIPAR traceNumber/Selects trace number.
ViStatus _VI_FUNC rsspecan_ConfigureChannelPowerMeasurement(ViSession instrSession,
                                                            ViInt32   noOfTXChannels,
                                                            ViInt32   noOfAdjChannels,
                                                            ViInt32   channelMode,
                                                            ViInt32   powerMode,
                                                            ViBoolean powerUnit,
                                                            ViInt32   noiseCorrection,
                                                            ViInt32   traceNumber)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_MEAS_POW_CARR_SIG_NUM, noOfTXChannels),
            2, "No Of Tx Channels");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_MEAS_POW_ADJ_NUM, noOfAdjChannels),
            3, "No Of Adj Channels");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_MEAS_POW_ADJ_MODE, channelMode),
            4, "Channel Mode");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_MEAS_POW_MODE, powerMode),
            5, "Power Mode");

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_MEAS_POW_RESULT_MODE, powerUnit),
            6, "Power Unit");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_MEAS_POW_NOISE_CORR_AUTO, noiseCorrection),
            7, "Noise Correction");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_MEAS_POW_TRACE, traceNumber),
            8, "Trace Number");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This attribute selects the unit the R&S FSW returns results for power
/// HIFN  measurements.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR powerUnit/This control selects the unit the R&S FSW returns results for power
/// HIPAR powerUnit/measurements.
ViStatus _VI_FUNC rsspecan_ConfigurePowerUnit(ViSession instrSession,
                                              ViInt32   powerUnit)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, NULL, RSSPECAN_ATTR_POWER_UNIT, powerUnit),
            2, "Power Unit");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/*****************************************************************************
 * Function: Configure ACP Fast ACP State
 * Purpose:  This function switches on or off the high-speed channel/adjacent
 *           channel power measurement.
 *           The measurement itself is performed in the time domain on the
 *           center frequencies of the individual channels. The command
 *           automatically switches to the time domain and back. Depending on
 *           the selected mobile radio standard, weighting filters with sqrt(cos)
 *           characteristic or very steep-sided channel filters are used for band
 *           limitation.
 *****************************************************************************/
ViStatus _VI_FUNC rsspecan_ConfigureACPFastACPState(ViSession instrSession,
                                                    ViInt32   reserved,
                                                    ViBoolean fastACPState)
{
    ViStatus error = VI_SUCCESS;
             reserved;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_MEAS_ACP_HSP, fastACPState),
            3, "Fast ACP State");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN Activates the automatic selection of a transmission channel to
/// HIFN be used as a reference channel in relative adjacent-channel
/// HIFN power measurements.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.
/// HIPAR reference/This control sets the type of reference selection.
ViStatus _VI_FUNC rsspecan_ConfigureACPReferenceChannelAuto(ViSession instrSession,
                                                            ViInt32   reserved,
                                                            ViInt32   reference)
{
    ViStatus error = VI_SUCCESS;
             reserved;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_MEAS_POW_REF_CHAN_SEL_AUTO, reference),
            3, "Reference");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function selects a transmission channel to be used as a
/// HIFN reference channel in relative adjacent-channel power
/// HIFN measurements.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.
/// HIPAR channel/This control select the reference channel for relative adjacent
/// HIPAR channel/channel power measurement.
ViStatus _VI_FUNC rsspecan_ConfigureACPReferenceChannelMan(ViSession instrSession,
                                                           ViInt32   reserved,
                                                           ViInt32   channel)
{
    ViStatus error = VI_SUCCESS;
             reserved;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_MEAS_POW_ADJ_REF_TXCHANNEL, channel),
            3, "Channel");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This sets the reference value for the relative measurement to
/// HIFN the currently measured channel power.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init
/// HIPAR instrSession/or rsspecan_InitWithOptions function.
/// HIPAR timeout/This control sets the timeout value.
ViStatus _VI_FUNC rsspecan_PowerAdjustReferenceLevel(ViSession instrSession,
                                                     ViInt32   reserved,
                                                     ViUInt32  timeout)
{
    ViStatus error = VI_SUCCESS;
    ViUInt32 old_timeout = 0;
             reserved;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViUInt32Range(instrSession, timeout, 0, 4294967295UL), 3, "Timeout");

    checkErr(rsspecan_GetTimeout(instrSession, &old_timeout));
    checkErr(rsspecan_SetTimeout(instrSession, timeout));
    checkErr(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_MEAS_POW_REF_VALUE, NULL));

Error:
    if (old_timeout > 0)
        rsspecan_SetTimeout(instrSession, old_timeout);
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function adapts the reference level to the measured channel
/// HIFN power and - if required - switches on previously the adjacent
/// HIFN channel power measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init
/// HIPAR instrSession/or rsspecan_InitWithOptions function.
/// HIPAR timeout/This control sets the timeout value.
ViStatus _VI_FUNC rsspecan_ACPChannelPowerAutoAdjust(ViSession instrSession,
                                                     ViInt32   reserved,
                                                     ViUInt32  timeout)
{
    ViStatus error = VI_SUCCESS;
    ViUInt32 old_timeout = 0;
             reserved;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViUInt32Range(instrSession, timeout, 0, 4294967295UL), 3, "Timeout");

    checkErr(rsspecan_GetTimeout(instrSession, &old_timeout));
    checkErr(rsspecan_SetTimeout(instrSession, timeout));

    checkErr(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_MEAS_ACP_PRESET_REF_LEVEL, NULL));

Error:
    if (old_timeout > 0)
        rsspecan_SetTimeout(instrSession, old_timeout);
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function returns all predefined ACP standards and all user
/// HIFN standards.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.
/// HIPAR bufferSize/Pass the number of bytes in the string you specify for the
/// HIPAR bufferSize/catalog list parameter.
/// HIPAR powerStandardCatalog/This control returns all predefined ACP standards and all user
/// HIPAR powerStandardCatalog/standards.
ViStatus _VI_FUNC rsspecan_QueryChannelPowerStandardCatalog(ViSession instrSession,
                                                            ViInt32   bufferSize,
                                                            ViChar    powerStandardCatalog[])
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_QueryCatalog(instrSession, "CALC:MARK:FUNC:POW:STAN:CAT?", bufferSize, powerStandardCatalog, NULL));
    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/*===========================================================================*
 *------------------- CP - > Channel Setup ----------------------------------*
 *===========================================================================*/
/// HIFN This function configures the adjacent power channel bandwidth.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.
/// HIPAR channel/Selects the channel type.
/// HIPAR channelNumber/Sets the channel number.
/// HIPAR bandwidth/Sets the selected channel bandwidth.
ViStatus _VI_FUNC rsspecan_ConfigurePowerChannelBandwidth(ViSession instrSession,
                                                          ViInt32   reserved,
                                                          ViInt32   channel,
                                                          ViInt32   channelNumber,
                                                          ViReal64  bandwidth)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
             reserved;

    checkErr(RsCore_LockSession(instrSession));

    switch (channel)
    {
    case RSSPECAN_VAL_ACP_TX:
        viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_POW_CHANNEL_BANDWIDTH,
                "TXChannel", channelNumber - 1, RS_REPCAP_BUF_SIZE, repCap),
                4, "Channel Number");

        viCheckParm(rsspecan_SetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_POW_CHANNEL_BANDWIDTH, bandwidth),
                5, "Bandwidth");
        break;
    case RSSPECAN_VAL_ACP_ADJ:
        viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_MEAS_POW_ADJ_CHANNEL_BANDWIDTH, bandwidth),
                5, "Bandwidth");
        break;
    case RSSPECAN_VAL_ACP_ALT:
        viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_MEAS_POW_ALT_CHANNEL_BANDWIDTH,
                "Channel", channelNumber - 1, RS_REPCAP_BUF_SIZE, repCap),
                4, "Channel Number");
        viCheckParm(rsspecan_SetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_MEAS_POW_ALT_CHANNEL_BANDWIDTH, bandwidth),
                5, "Bandwidth");
        break;
    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, channel), 3, "Channel");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function configures the adjacent power channel spacing.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.
/// HIPAR channel/Selects the channel type.
/// HIPAR channelNumber/Sets the channel number.
/// HIPAR spacing/Document parameter here.
ViStatus _VI_FUNC rsspecan_ConfigurePowerChannelSpacing(ViSession instrSession,
                                                        ViInt32   reserved,
                                                        ViInt32   channel,
                                                        ViInt32   channelNumber,
                                                        ViReal64  spacing)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
             reserved;

    checkErr(RsCore_LockSession(instrSession));

    switch (channel)
    {
    case RSSPECAN_VAL_ACP_TX:
        viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_MEAS_POW_CHANNEL_SPACING,
                "TXChannel", channelNumber - 1, RS_REPCAP_BUF_SIZE, repCap),
                4, "Channel Number");

        viCheckParm(rsspecan_SetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_MEAS_POW_CHANNEL_SPACING, spacing),
                5, "Spacing");
        break;
    case RSSPECAN_VAL_ACP_ADJ:
        viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_MEAS_POW_ADJ_CHANNEL_SPACING, spacing),
                5, "Spacing");
        break;
    case RSSPECAN_VAL_ACP_ALT:
        viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_MEAS_POW_ALT_CHANNEL_SPACING,
                "Channel", channelNumber - 1, RS_REPCAP_BUF_SIZE, repCap),
                4, "Channel Number");
        viCheckParm(rsspecan_SetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_MEAS_POW_ALT_CHANNEL_SPACING, spacing),
                5, "Spacing");
        break;
    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, channel), 3, "Channel");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function configures the power channel weighting filters.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init
/// HIPAR instrSession/or rsspecan_InitWithOptions function.
/// HIPAR filterType/This control selects the type of the filter to be modified.
/// HIPAR channelNumber/Sets the channel number.
/// HIPAR filterState/This control sets the state of the selected filter.
/// HIPAR filterValue/This control sets the weighting filter alpha value.
ViStatus _VI_FUNC rsspecan_ConfPowerChannelWeightingFilters(ViSession instrSession,
                                                            ViInt32   filterType,
                                                            ViInt32   channelNumber,
                                                            ViBoolean filterState,
                                                            ViReal64  filterValue)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    switch (filterType)
    {
    case RSSPECAN_VAL_WFILTER_ALL:
        viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_SEM_WEIGHT_FILTER_ALL_STATE, filterState),
                3, "Filter State");
        if (filterState == VI_TRUE)
        viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_SEM_WEIGHT_FILTER_ALL_ALPHA, filterValue),
                4, "Filter Value");
        break;
    case RSSPECAN_VAL_WFILTER_ADJ:
        viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_SEM_WEIGHT_FILTER_ACH_STATE, filterState),
                3, "Filter State");
        if (filterState == VI_TRUE)
        viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_SEM_WEIGHT_FILTER_ACH_ALPHA, filterValue),
                4, "Filter Value");
        break;
    case RSSPECAN_VAL_WFILTER_ALT:
        checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_POWER_ALT_CHANNEL_WEIGHTING_FILTER_ALPHA_VALUE,
                "Channel", channelNumber - 1, RS_REPCAP_BUF_SIZE, repCap));
        viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, repCap, RSSPECAN_ATTR_POWER_ALT_CHANNEL_WEIGHTING_FILTER_STATE, filterState),
                3, "Filter State");
        if (filterState == VI_TRUE)
        viCheckParm(rsspecan_SetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_POWER_ALT_CHANNEL_WEIGHTING_FILTER_ALPHA_VALUE, filterValue),
                4, "Filter Value");
        break;
    case RSSPECAN_VAL_WFILTER_TX:
        checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_POWER_CHANNEL_WEIGHTING_FILTER_ALPHA,
                "TXChannel", channelNumber - 1, RS_REPCAP_BUF_SIZE, repCap));
        viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, repCap, RSSPECAN_ATTR_POWER_CHANNEL_WEIGHTING_FILTER, filterState),
                3, "Filter State");
        if (filterState == VI_TRUE)
        viCheckParm(rsspecan_SetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_POWER_CHANNEL_WEIGHTING_FILTER_ALPHA, filterValue),
                4, "Filter Value");
        break;
    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, filterType), 2, "Filter Type");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN This function configures the power channel name.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init
/// HIPAR instrSession/or rsspecan_InitWithOptions function.
/// HIPAR channel/Selects the channel type.
/// HIPAR channelNumber/Sets the channel number.
/// HIPAR name/Sets the power channel name.
ViStatus _VI_FUNC rsspecan_ConfigurePowerChannelName(ViSession instrSession,
                                                     ViInt32   reserved,
                                                     ViInt32   channel,
                                                     ViInt32   channelNumber,
                                                     ViString  name)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
             reserved;

    checkErr(RsCore_LockSession(instrSession));

    switch (channel)
    {
    case RSSPECAN_VAL_ACP_TX:
        viCheckParm(RsCore_InvalidViInt32Range(instrSession, channelNumber, 1, 18),
                4, "Channel Number");

        checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_MEAS_POW_CHANNEL_NAME,
                "TXChannel", channelNumber - 1, RS_REPCAP_BUF_SIZE, repCap));

        viCheckParm(rsspecan_SetAttributeViString(instrSession, repCap, RSSPECAN_ATTR_MEAS_POW_CHANNEL_NAME, name),
                5, "Name");
        break;
    case RSSPECAN_VAL_ACP_ADJ:
        viCheckParm(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_MEAS_POW_ADJ_CHANNEL_NAME, name),
                5, "Name");
        break;
    case RSSPECAN_VAL_ACP_ALT:
        viCheckParm(RsCore_InvalidViInt32Range(instrSession, channelNumber, 1, 12),
                4, "Channel Number");

        checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_MEAS_POW_ALT_CHANNEL_NAME,
                "Channel", channelNumber - 1, RS_REPCAP_BUF_SIZE, repCap));
        viCheckParm(rsspecan_SetAttributeViString(instrSession, repCap, RSSPECAN_ATTR_MEAS_POW_ALT_CHANNEL_NAME, name),
                5, "Name");
        break;
    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, channel), 3, "Channel");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/*===========================================================================*
 *------------------- CP -> Channel Setup -> Limit --------------------------*
 *===========================================================================*/
/// HIFN This function switches on and off the limit check for adjacent
/// HIFN channel power measurements.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.
/// HIPAR state/Switches on and off the limit check for adjacent channel power
/// HIPAR state/measurements.
ViStatus _VI_FUNC rsspecan_ConfigureACPLimitState(ViSession instrSession,
                                                  ViInt32   reserved,
                                                  ViBoolean state)
{
    ViStatus error = VI_SUCCESS;
             reserved;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_MEAS_ACP_LIMIT_STATE, state),
            3, "State");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function defines the limit value of the adjacent channel.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.
/// HIPAR relativeState/Switches the limit check for the relative limit value of the
/// HIPAR relativeState/adjacent channel when adjacent channel power measurement is
/// HIPAR relativeState/performed.
/// HIPAR relativeValue/Sets the relative limit value.
/// HIPAR absoluteState/Switches the limit check for the absolute limit value of the
/// HIPAR absoluteState/adjacent channel when adjacent channel power measurement is
/// HIPAR absoluteState/performed.
/// HIPAR absoluteValue/Sets the relative limit value.
ViStatus _VI_FUNC rsspecan_ConfigureACPAdjacentChannelLimit(ViSession instrSession,
                                                            ViInt32   reserved,
                                                            ViBoolean relativeState,
                                                            ViReal64  relativeValue,
                                                            ViBoolean absoluteState,
                                                            ViReal64  absoluteValue)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
             reserved;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViReal64Range(instrSession, relativeValue, 0.0, 100.0), 4, "Relative Value");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, absoluteValue, -200.0, 200.0), 6, "Absolute Value");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CALC:LIM:ACP:ACH %.12lG,0.0", relativeValue);
    checkErr(RsCore_Write(instrSession, cmd));
    checkErr(rsspecan_CheckStatus(instrSession));
    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_MEAS_ACP_LIMIT_REL_STATE, relativeState),
            3, "Relative State");
    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CALC:LIM:ACP:ACH:ABS %.12lG,0.0", absoluteValue);
    checkErr(RsCore_Write(instrSession, cmd));
    checkErr(rsspecan_CheckStatus(instrSession));
    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_MEAS_ACP_LIMIT_ABS_STATE, absoluteState),
            5, "Absolute State");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function defines the limit value of the alternate adjacent
/// HIFN channel.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.
/// HIPAR alternateChannelNumber/Sets the channel number.
/// HIPAR relativeState/Switches the limit check for the relative limit value of the
/// HIPAR relativeState/Salternate adjacent channel when adjacent channel power
/// HIPAR relativeState/Smeasurement is performed.
/// HIPAR relativeValue/Sets the relative limit value.
/// HIPAR absoluteState/Switches the limit check for the absolute limit value of the
/// HIPAR absoluteState/alternate adjacent channel when adjacent channel power
/// HIPAR absoluteState/measurement is performed.
/// HIPAR absoluteValue/Sets the absolute limit value.
ViStatus _VI_FUNC rsspecan_ConfigureACPAlternateChannelLimit(ViSession instrSession,
                                                             ViInt32   reserved,
                                                             ViInt32   alternateChannelNumber,
                                                             ViBoolean relativeState,
                                                             ViReal64  relativeValue,
                                                             ViBoolean absoluteState,
                                                             ViReal64  absoluteValue)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
             reserved;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_MEAS_ACP_ALT_LIMIT_REL_STATE,
            "Channel", alternateChannelNumber - 1, RS_REPCAP_BUF_SIZE, repCap));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, alternateChannelNumber, 1, 11),
            3, "Alternate Channel Number");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, relativeValue, 0.0, 100.0), 5, "Relative Value");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, absoluteValue, -200.0, 200.0), 7, "Absolute Value");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CALC:LIM:ACP:ALT%ld %.12lG,0.0", alternateChannelNumber, relativeValue);
    checkErr(RsCore_Write(instrSession, cmd));
    checkErr(rsspecan_CheckStatus(instrSession));
    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, repCap, RSSPECAN_ATTR_MEAS_ACP_ALT_LIMIT_REL_STATE, relativeState),
            4, "Relative State");
    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CALC:LIM:ACP:ALT%ld:ABS %.12lG,0.0", alternateChannelNumber, absoluteValue);
    checkErr(RsCore_Write(instrSession, cmd));
    checkErr(rsspecan_CheckStatus(instrSession));
    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, repCap, RSSPECAN_ATTR_MEAS_ACP_ALT_LIMIT_ABS_STATE, absoluteState),
            6, "Absolute State");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function returns the adjacent channel limit results.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.
/// HIPAR lowerResult/Returns the lower limit result.
/// HIPAR upperResult/Returns the upper limit result.
ViStatus _VI_FUNC rsspecan_QueryACPAdjacentChannelLimitCheckResults(
                            ViSession instrSession,
                            ViInt32   reserved,
                            ViInt32   *lowerResult,
                            ViInt32   *upperResult)
{
    ViStatus error = VI_SUCCESS;
    ViChar   response[RS_MAX_MESSAGE_BUF_SIZE] = "";
    ViString resultsTable = "FAILED,PASSED";
    ViChar   value[RS_MAX_MESSAGE_LEN];
             reserved;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidNullPointer(instrSession, lowerResult),
        3, "Lower Result");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, upperResult),
        4, "Upper Result");

    checkErr(RsCore_QueryViString(instrSession, "CALC:LIM:ACP:ACH:RES?", response));

    checkErr(RsCore_GetTokenAtIndex(response, ',', 0, value, RS_MAX_MESSAGE_LEN, NULL));
    checkErr(RsCore_GetIndexOfToken(resultsTable, ',', value, lowerResult, NULL));

    checkErr(RsCore_GetTokenAtIndex(response, ',', 1, value, RS_MAX_MESSAGE_LEN, NULL));
    checkErr(RsCore_GetIndexOfToken(resultsTable, ',', value, upperResult, NULL));

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN This function returns the alternate adjacent channel limit results.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.
/// HIPAR lowerResult/Returns the lower limit result.
/// HIPAR upperResult/Returns the upper limit result.
ViStatus _VI_FUNC rsspecan_QueryACPAlternateChannelLimitCheckResults(
                            ViSession instrSession,
                            ViInt32   reserved,
                            ViInt32   alternateChannelNumber,
                            ViInt32   *lowerResult,
                            ViInt32   *upperResult)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";
    ViChar   response[RS_MAX_MESSAGE_BUF_SIZE] = "";
    ViString resultsTable = "FAILED,PASSED,NONE";
    ViChar   value[RS_MAX_MESSAGE_LEN];
             reserved;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, alternateChannelNumber, 1, 11),
            3, "Alternate Channel Number");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, lowerResult),
        4, "Lower Result");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, upperResult),
        5, "Upper Result");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CALC:LIM:ACP:ALT%ld:RES?", alternateChannelNumber);
    checkErr(RsCore_QueryViString(instrSession, cmd, response));

    checkErr(RsCore_GetTokenAtIndex(response, ',', 0, value, RS_MAX_MESSAGE_LEN, NULL));
    checkErr(RsCore_GetIndexOfToken(resultsTable, ',', value, lowerResult, NULL));

    checkErr(RsCore_GetTokenAtIndex(response, ',', 1, value, RS_MAX_MESSAGE_LEN, NULL));
    checkErr(RsCore_GetIndexOfToken(resultsTable, ',', value, upperResult, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/*===========================================================================*
 *------------------- CP -> MSR ---------------------------------------------*
 *===========================================================================*/
/// HIFN This function configures the MSR power measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.
/// HIPAR ACLRMode/Sets absolute and relative adjacent channel measurement.
/// HIPAR powerMode/Sets the Clear Write or Maxhold for Channel Power values.
/// HIPAR powerUnit/Sets the query response of the power measurement results in the
/// HIPAR powerUnit/indicated measurement window between output of absolute values
/// HIPAR powerUnit/(OFF) and output referred to the measurement bandwidth (ON).
/// HIPAR noiseCorrection/Activates or deactivates a noise correction measurement.
/// HIPAR traceNumber/Selects trace number.
ViStatus _VI_FUNC rsspecan_ConfigureMSRPowerMeasurement(ViSession instrSession,
                                                        ViInt32   ACLRMode,
                                                        ViInt32   powerMode,
                                                        ViBoolean powerUnit,
                                                        ViInt32   noiseCorrection,
                                                        ViInt32   traceNumber)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_MEAS_POW_ADJ_MODE, ACLRMode),
            2, "ACLR Mode");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_MEAS_POW_MODE, powerMode),
            3, "Power Mode");

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_MEAS_POW_RESULT_MODE, powerUnit),
            4, "Power Unit");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_MEAS_POW_NOISE_CORR_AUTO, noiseCorrection),
            5, "Noise Correction");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_MEAS_POW_TRACE, traceNumber),
            6, "Trace Number");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the name of the GAP channel.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR gap/This control selects gap channel number.
/// HIPAR arraySize/Sets the Array Size parameter.
/// HIPAR gapName/Queries the name of the GAP channel.
ViStatus _VI_FUNC rsspecan_QueryMSRGapName(ViSession instrSession,
                                           ViInt32   gap,
                                           ViInt32   arraySize,
                                           ViChar    gapName[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_MSR_GAP_NAME,
            "Gap", gap - 1, RS_REPCAP_BUF_SIZE, repCap));

    viCheckParm(rsspecan_GetAttributeViString(instrSession, repCap, RSSPECAN_ATTR_MSR_GAP_NAME,
            arraySize, gapName),
            4, "Gap Name");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function defines the number of subblocks, i.e. groups of
/// HIFN  transmission channels in MSR signal.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR numberOfSubblocks/Defines the number of subblocks, i.e. groups of transmission channels
/// HIPAR numberOfSubblocks/in MSR signal.
ViStatus _VI_FUNC rsspecan_ConfigureMSRNumberOfSubblocks(ViSession instrSession,
                                                         ViInt32   numberOfSubblocks)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsspecan_AssertNotFSV(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_MSR_NUMBER_OF_SUBBLOCKS, numberOfSubblocks),
            2, "Number Of Subblocks");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function defines whether adjacent channels are defined
/// HIFN  symmetrically or not.
/// HIFN
/// HIFN     Note:
/// HIFN     This function is valid for R&S FSW only
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR symmetricalAdjacentSetup/This control defines whether adjacent channels are defined
/// HIPAR symmetricalAdjacentSetup/symmetrically or not.
ViStatus _VI_FUNC rsspecan_ConfigureMSRSymmetricalAdjacentSetup(
                            ViSession instrSession,
                            ViBoolean symmetricalAdjacentSetup)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_MSR_SYMMETRICAL_ADJACENT_SETUP_ENABLED, symmetricalAdjacentSetup),
            2, "Symmetrical Adjacent Setup");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Subblocks are groups of transmit channels in an MSR signal. Up to 5
/// HIFN  subblocks can be defined. They are defined as an RF bandwidth around a
/// HIFN  center frequency with a specific number of transmit channels (max. 18).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR centerFrequency/Defines the center of the specified MSR subblock. Note that the
/// HIPAR centerFrequency/position of the subblock also affects the position of the adjacent gap
/// HIPAR centerFrequency/(CACLR) channels.
/// HIPAR subblock/This control selects the subblock number.
/// HIPAR RFBandwidth/Defines the bandwidth of the individual MSR subblock. Note that
/// HIPAR RFBandwidth/subblock ranges also affect the position of the adjacent gap channels
/// HIPAR RFBandwidth/(CACLR).
/// HIPAR txChannelCount/Defines the bandwidth of the individual MSR subblock. Note that
/// HIPAR txChannelCount/subblock ranges also affect the position of the adjacent gap channels
/// HIPAR txChannelCount/(CACLR).
ViStatus _VI_FUNC rsspecan_ConfigureMSRSubblockDefinition(ViSession instrSession,
                                                          ViInt32   subblock,
                                                          ViReal64  centerFrequency,
                                                          ViReal64  RFBandwidth,
                                                          ViInt32   txChannelCount)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsspecan_AssertNotFSV(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_MSR_SUBBLOCK_CENTER_FREQUENCY,
            "Subblock", subblock - 1, RS_REPCAP_BUF_SIZE, repCap));

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_MSR_SUBBLOCK_CENTER_FREQUENCY, centerFrequency),
            3, "Center Frequency");

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_MSR_SUBBLOCK_RF_BANDWIDTH, RFBandwidth),
            4, "RF Bandwidth");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_MSR_SUBBLOCK_TX_CHANNEL_COUNT, txChannelCount),
            5, "Tx Channel Count");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  As opposed to common ACLR channel definitions, the TX channels are
/// HIFN  defined at absolute frequencies, rather than by a spacing relative to
/// HIFN  the (common) center frequency.
/// HIFN  Each transmit channel can be assigned a different technology, used to
/// HIFN  predefine the required bandwidth.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR subblock/This control selects the subblock number.
/// HIPAR txChannel/Tx channel number.
/// HIPAR txCenterFrequency/This control defines the (absolute) center frequency of the specified
/// HIPAR txCenterFrequency/MSR Tx channel.
/// HIPAR technology/This control defines the technology used for transmission by the
/// HIPAR technology/specified MSR Tx channel.
/// HIPAR txBandwidth/Defines the bandwidth of the specified MSR Tx channel.
/// HIPAR weightingFilterState/This control turns the weighting filter transmission channel on and
/// HIPAR weightingFilterState/off.
/// HIPAR rollOffFactor/This control defines the roll-off factor for the specified
/// HIPAR rollOffFactor/transmission channel's weighting filter.
ViStatus _VI_FUNC rsspecan_ConfigureMSRTxChannelDefinition(ViSession instrSession,
                                                           ViInt32   subblock,
                                                           ViInt32   txChannel,
                                                           ViReal64  txCenterFrequency,
                                                           ViInt32   technology,
                                                           ViReal64  txBandwidth,
                                                           ViBoolean weightingFilterState,
                                                           ViReal64  rollOffFactor)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   sub_rep_cap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   tx_rep_cap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsspecan_AssertNotFSV(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, subblock, 1, 8),
            2, "Subblock");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, txChannel, 1, 18),
            3, "Tx Channel");
    checkErr(RsCore_GetAttributeRepCapName(instrSession,
                                                 RSSPECAN_ATTR_MSR_TX_CENTER_FREQUENCY,
                                                 "Subblock",
                                                 subblock - 1,
                                                 RS_REPCAP_BUF_SIZE,
                                                 sub_rep_cap));
    checkErr(RsCore_GetAttributeRepCapName(instrSession,
                                                 RSSPECAN_ATTR_MSR_TX_CENTER_FREQUENCY,
                                                 "TXChannel",
                                                 txChannel - 1,
                                                 RS_REPCAP_BUF_SIZE,
                                                 tx_rep_cap));

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "%s,%s", sub_rep_cap, tx_rep_cap);

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_MSR_TX_CENTER_FREQUENCY, txCenterFrequency),
            4, "Tx Center Frequency");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_MSR_TX_TECHNOLOGY, technology),
            5, "Technology");

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_MSR_TX_BANDWIDTH, txBandwidth),
            6, "Tx Bandwidth");

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, repCap, RSSPECAN_ATTR_MSR_TX_WEIGHTING_FILTER_STATE, weightingFilterState),
            7, "Weighting Filter State");

    if (weightingFilterState == VI_TRUE)
    {
        viCheckParm(rsspecan_SetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_MSR_TX_WEIGHTING_FILTER_ROLL_OFF_FACTOR, rollOffFactor),
                8, "Roll Off Factor");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function queries the name of the specified MSR Tx channel.
/// HIFN  In MSR ACLR measurements, TX channel names correspond to the specified
/// HIFN  technology, followed by a consecutive number. The assigned subblock
/// HIFN  (A,B,C,D,E) is indicated as a prefix (e.g. A: WCDMA1). Channel names
/// HIFN  cannot be defined manually.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR subblock/This control selects the subblock number.
/// HIPAR txChannel/Tx channel number.
/// HIPAR name/String containing the name of the channel.
ViStatus _VI_FUNC rsspecan_QueryMSRTxName(ViSession instrSession,
                                          ViInt32   subblock,
                                          ViInt32   txChannel,
                                          ViInt32   bufferSize,
                                          ViChar    name[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   sub_rep_cap [10] = "";
    ViChar   tx_rep_cap[10] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsspecan_AssertNotFSV(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession,
                                                 RSSPECAN_ATTR_MSR_TX_CENTER_FREQUENCY,
                                                 "Subblock",
                                                 subblock - 1,
                                                 10,
                                                 sub_rep_cap));
    checkErr(RsCore_GetAttributeRepCapName(instrSession,
                                                 RSSPECAN_ATTR_MSR_TX_CENTER_FREQUENCY,
                                                 "TXChannel",
                                                 txChannel - 1,
                                                 10,
                                                 tx_rep_cap));

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "%s,%s", sub_rep_cap, tx_rep_cap);

    viCheckParm(rsspecan_GetAttributeViString(instrSession, repCap, RSSPECAN_ATTR_MSR_TX_CHANNEL_NAME,
            bufferSize, name),
            4, "Name");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the name for the specified upper alternate
/// HIFN  channel in asymmetrical MSR channel definitions. To define the name for
/// HIFN  the lower adjacent channels use the
/// HIFN  RSSPECAN_ATTR_MEAS_POW_ALT_CHANNEL_NAME command.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/Selects the channel type.
/// HIPAR channelNumber/Sets the channel number.
/// HIPAR name/Sets the power channel name.
ViStatus _VI_FUNC rsspecan_ConfigureMSRUpperChannelName(ViSession instrSession,
                                                        ViInt32   channel,
                                                        ViInt32   channelNumber,
                                                        ViString  name)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    switch (channel)
    {
    case RSSPECAN_VAL_ACP_ADJ:
        viCheckParm(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_MSR_UPPER_ADJACENT_CHANNEL_NAME, name),
                4, "Name");
        break;
    case RSSPECAN_VAL_ACP_ALT:
        viCheckParm(RsCore_InvalidViInt32Range(instrSession, channelNumber, 1, 11),
                3, "Channel Number");

        checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_MSR_UPPER_ALTERNATE_CHANNEL_NAME,
                "Channel", channelNumber - 1, RS_REPCAP_BUF_SIZE, repCap));

        viCheckParm(rsspecan_SetAttributeViString(instrSession, repCap, RSSPECAN_ATTR_MSR_UPPER_ALTERNATE_CHANNEL_NAME, name),
                4, "Name");
        break;
    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, channel), 2, "Channel");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function activates or deactivates gap channels in an MSR signal.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR activateGaps/This control activates or deactivates gap channels in an MSR signal.
ViStatus _VI_FUNC rsspecan_ConfigureMSRActivateGaps(ViSession instrSession,
                                                    ViBoolean activateGaps)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_MSR_ACTIVATE_GAPS, activateGaps),
            2, "Activate Gaps");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function queries the results of power measurements.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR subblock/This control selects the subblock number.
/// HIPAR powerMeasurement/Selects the ACP (adjacent channel power) standard.
/// HIPAR arraySize/Defines the memory size allocated for Results.
/// HIPAR results/Returns the measured values:
/// HIPAR returnedValues/Returns number of returned values in Results array.
ViStatus _VI_FUNC rsspecan_QueryMSRMeasurementPowerResult(ViSession instrSession,
                                                          ViInt32   subblock,
                                                          ViInt32   powerMeasurement,
                                                          ViInt32   arraySize,
                                                          ViReal64  results[],
                                                          ViInt32   *returnedValues)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViString arr[] = {"ACP", "CPOW", "MCAC", "OBAN", "CN", "CN0", "", "PPOW", "GACL", "MACM"};

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, powerMeasurement, RSSPECAN_VAL_MEAS_POW_ACP, RSSPECAN_VAL_MEAS_POW_MACM),
            3, "Power Measurement");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CALC:MARK:FUNC:POW%ld:RES? %s", subblock, arr[powerMeasurement]);
    checkErr(RsCore_QueryFloatArrayToUserBuffer(instrSession, cmd, arraySize, results, returnedValues));

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  If the gap between the sub blocks does not exceed the specified
/// HIFN  bandwidth, the gap channels are not displayed in the diagram, and the
/// HIFN  gap channel results are not calculated in the result summary.
/// HIFN
/// HIFN     Note(s):
/// HIFN
/// HIFN     (1) This function is available only for R&S FSW instrument.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR gap/This control selects gap channel number.
/// HIPAR gapSize/If the gap between the sub blocks does not exceed the specified
/// HIPAR gapSize/bandwidth, the gap channels are not displayed in the diagram, and the
/// HIPAR gapSize/gap channel results are not calculated in the result summary.
ViStatus _VI_FUNC rsspecan_ConfigureMSRGapSize(ViSession instrSession,
                                               ViInt32   gap,
                                               ViReal64  gapSize)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, gap, 1, 2),
            2, "Gap");

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_MSR_MINIMUM_GAP_SIZE,
            "Gap", gap - 1, RS_REPCAP_BUF_SIZE, repCap));

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_MSR_MINIMUM_GAP_SIZE, gapSize),
            3, "Gap Size");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function defines the distance from subblock to the specified gap
/// HIFN  channel, the bandwidth of the specified MSR gap (CACLR) channel, and
/// HIFN  the weighting filter.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR gap/This control selects the measurement window.
/// HIPAR channelSpacing/Defines the distance from subblock to the specified gap channel.
/// HIPAR channelBandwidths/Defines the bandwidth of the specified MSR gap (CACLR) channel.
/// HIPAR weightingFilterState/This control turns the weighting filter for the specified gap (CACLR)
/// HIPAR weightingFilterState/channel on and off.
/// HIPAR rollOffFactor/This control defines the roll-off factor for the specified gap (CACLR)
/// HIPAR rollOffFactor/channel's weighting filter.
ViStatus _VI_FUNC rsspecan_ConfigureMSRGapChannel(ViSession instrSession,
                                                  ViInt32   gap,
                                                  ViReal64  channelSpacing,
                                                  ViReal64  channelBandwidths,
                                                  ViBoolean weightingFilterState,
                                                  ViReal64  rollOffFactor)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsspecan_AssertNotFSV(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_MSR_GAP_CHANNEL_SPACING,
            "Gap", gap - 1, RS_REPCAP_BUF_SIZE, repCap));

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_MSR_GAP_CHANNEL_SPACING, channelSpacing),
            3, "channelSpacing");

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_MSR_GAP_CHANNEL_BANDWIDTHS, channelBandwidths),
            4, "channelBandwidths");

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, repCap, RSSPECAN_ATTR_MSR_GAP_WEIGHTING_FILTER_STATE, weightingFilterState),
            5, "Weighting Filter State");

    if (weightingFilterState == VI_TRUE)
    {
        viCheckParm(rsspecan_SetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_MSR_GAP_WEIGHTING_FILTER_ROLL_OFF_FACTOR, rollOffFactor),
                6, "Roll Off Factor");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function defines the limit value of the gap (CACLR) channel.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR gap/This control selects the measurement window.
/// HIPAR relativeState/Turns the relative limit check for the specified gap (CACLR) channel
/// HIPAR relativeState/on and off.
/// HIPAR relativeValue/Defines the relative limit of the specified gap (CACLR) channel. The
/// HIPAR relativeValue/reference value for the relative limit is the measured channel power.
/// HIPAR absoluteState/Turns the absolute limit check for the specified gap (CACLR) channel
/// HIPAR absoluteState/on and off.
/// HIPAR absoluteValue/Defines the absolute limit of the specified gap (CACLR) channel.
ViStatus _VI_FUNC rsspecan_ConfigureMSRGapLimitChecking(ViSession instrSession,
                                                        ViInt32   gap,
                                                        ViBoolean relativeState,
                                                        ViReal64  relativeValue,
                                                        ViBoolean absoluteState,
                                                        ViReal64  absoluteValue)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViChar   repCap[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsspecan_AssertNotFSV(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_MSR_GAP_LIMIT_RELATIVE_STATE,
            "Gap", gap - 1, RS_REPCAP_BUF_SIZE, repCap));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, repCap, RSSPECAN_ATTR_MSR_GAP_LIMIT_RELATIVE_STATE, relativeState),
            3, "Relative State");
    if (relativeState)
    {
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CALC:LIM:ACP:GAP%ld:REL %.12lG,0.0", gap, relativeValue);
        checkErr(RsCore_Write(instrSession, cmd));
        checkErr(rsspecan_CheckStatus(instrSession));
    }

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, repCap, RSSPECAN_ATTR_MSR_GAP_LIMIT_ABSOLUTE_STATE, absoluteState),
            5, "Absolute State");
    if (absoluteState)
    {
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CALC:LIM:ACP:GAP%ld:ABS %.12lG,0.0", gap, absoluteValue);
        checkErr(RsCore_Write(instrSession, cmd));
        checkErr(rsspecan_CheckStatus(instrSession));
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the distance from sub block to the specified lower or upper
/// HIFN  gap channel.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR gap/This control selects the measurement window.
/// HIPAR type/Sets the type.
/// HIPAR gaps/Name of the gap, defined by the letters of the surrounding sub blocks
/// HIPAR gaps/(e.g. "AB" for the gap between sub blocks A and B).
/// HIPAR spacing/Sets the bandwidth.
ViStatus _VI_FUNC rsspecan_ConfigureMSRGapChannelSpacing(ViSession instrSession,
                                                         ViInt32   gap,
                                                         ViInt32   type,
                                                         ViInt32   gaps,
                                                         ViReal64  spacing)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, gap, 1, 2),
            2, "Gap");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, type, 0, 1),
            3, "Type");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, gaps, RSSPECAN_VAL_MSR_SBGAPS_AB, RSSPECAN_VAL_MSR_SBGAPS_GH),
            4, "Gaps");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SENS:POW:ACH:SPAC:GAP%ld:MAN:%s %s,%.12lG", gap, gapLowUppArr[type], gapsArr[gaps], spacing);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the distance from sub block to the specified lower or upper
/// HIFN  gap channel.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR gap/This control selects the measurement window.
/// HIPAR type/Sets the type.
/// HIPAR gaps/Name of the gap, defined by the letters of the surrounding sub blocks
/// HIPAR gaps/(e.g. "AB" for the gap between sub blocks A and B).
/// HIPAR spacing/Queries the distance from sub block to the specified lower or upper
/// HIPAR spacing/gap channel.
ViStatus _VI_FUNC rsspecan_QueryMSRGapChannelSpacing(ViSession instrSession,
                                                     ViInt32   gap,
                                                     ViInt32   type,
                                                     ViInt32   gaps,
                                                     ViReal64* spacing)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, gap, 1, 2),
            2, "Gap");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, type, 0, 1),
            3, "Type");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, gaps, RSSPECAN_VAL_MSR_SBGAPS_AB, RSSPECAN_VAL_MSR_SBGAPS_GH),
            4, "Gaps");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SENS:POW:ACH:SPAC:GAP%ld:MAN:%s? %s", gap, gapLowUppArr[type], gapsArr[gaps]);
    checkErr(RsCore_QueryViReal64(instrSession, cmd, spacing));

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Defines the bandwidth of the lower or upper gap channel in the
/// HIFN  specified gap.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR gap/This control selects the measurement window.
/// HIPAR type/Sets the type.
/// HIPAR gaps/Name of the gap, defined by the letters of the surrounding sub blocks
/// HIPAR gaps/(e.g. "AB" for the gap between sub blocks A and B).
/// HIPAR bandwidth/Sets the bandwidth.
ViStatus _VI_FUNC rsspecan_ConfigureMSRGapChannelBandwidths(ViSession instrSession,
                                                            ViInt32   gap,
                                                            ViInt32   type,
                                                            ViInt32   gaps,
                                                            ViReal64  bandwidth)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, gap, 1, 2),
            2, "Gap");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, type, 0, 1),
            3, "Type");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, gaps, RSSPECAN_VAL_MSR_SBGAPS_AB, RSSPECAN_VAL_MSR_SBGAPS_GH),
            4, "Gaps");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SENS:POW:ACH:BAND:GAP%ld:MAN:%s %s,%.12lG", gap, gapLowUppArr[type], gapsArr[gaps], bandwidth);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the bandwidth of the lower gap channel in the specified gap.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR gap/This control selects the measurement window.
/// HIPAR type/Sets the type.
/// HIPAR gaps/Name of the gap, defined by the letters of the surrounding sub blocks
/// HIPAR gaps/(e.g. "AB" for the gap between sub blocks A and B).
/// HIPAR bandwidth/Queries the bandwidth of the lower gap channel in the specified gap.
ViStatus _VI_FUNC rsspecan_QueryMSRGapChannelBandwidths(ViSession instrSession,
                                                        ViInt32   gap,
                                                        ViInt32   type,
                                                        ViInt32   gaps,
                                                        ViReal64* bandwidth)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, gap, 1, 2),
            2, "Gap");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, type, 0, 1),
            3, "Type");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, gaps, RSSPECAN_VAL_MSR_SBGAPS_AB, RSSPECAN_VAL_MSR_SBGAPS_GH),
            4, "Gaps");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SENS:POW:ACH:BAND:GAP%ld:MAN:%s? %s", gap, gapLowUppArr[type], gapsArr[gaps]);
    checkErr(RsCore_QueryViReal64(instrSession, cmd, bandwidth));

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Turns the weighting filter for the specified lower gap channel on and
/// HIFN  off.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR gap/This control selects the measurement window.
/// HIPAR type/Sets the type.
/// HIPAR gaps/Name of the gap, defined by the letters of the surrounding sub blocks
/// HIPAR gaps/(e.g. "AB" for the gap between sub blocks A and B).
/// HIPAR state/Turns the weighting filter for the specified lower gap channel on and
/// HIPAR state/off.
ViStatus _VI_FUNC rsspecan_ConfigureMSRGapWeightingFilterState(
                            ViSession instrSession,
                            ViInt32   gap,
                            ViInt32   type,
                            ViInt32   gaps,
                            ViBoolean state)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, gap, 1, 2),
            2, "Gap");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, type, 0, 1),
            3, "Type");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, gaps, RSSPECAN_VAL_MSR_SBGAPS_AB, RSSPECAN_VAL_MSR_SBGAPS_GH),
            4, "Gaps");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SENS:POW:ACH:FILT:GAP%ld:MAN:%s %s,%s", gap, gapLowUppArr[type], gapsArr[gaps], state ? "ON" : "OFF");
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the state of the weighting filter.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR gap/This control selects the measurement window.
/// HIPAR type/Sets the type.
/// HIPAR gaps/Name of the gap, defined by the letters of the surrounding sub blocks
/// HIPAR gaps/(e.g. "AB" for the gap between sub blocks A and B).
/// HIPAR state/Queries the state of the weighting filter.
ViStatus _VI_FUNC rsspecan_QueryMSRGapWeightingFilterState(ViSession  instrSession,
                                                           ViInt32    gap,
                                                           ViInt32    type,
                                                           ViInt32    gaps,
                                                           ViBoolean* state)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, gap, 1, 2),
            2, "Gap");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, type, 0, 1),
            3, "Type");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, gaps, RSSPECAN_VAL_MSR_SBGAPS_AB, RSSPECAN_VAL_MSR_SBGAPS_GH),
            4, "Gaps");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SENS:POW:ACH:FILT:GAP%ld:MAN:%s? %s", gap, gapLowUppArr[type], gapsArr[gaps]);
    checkErr(RsCore_QueryViBoolean(instrSession, cmd, state));

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Defines the roll-off factor for the specified lower or upper gap
/// HIFN  channel's weighting filter.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR gap/This control selects the measurement window.
/// HIPAR type/Sets the type.
/// HIPAR gaps/Name of the gap, defined by the letters of the surrounding sub blocks
/// HIPAR gaps/(e.g. "AB" for the gap between sub blocks A and B).
/// HIPAR rollOffFactor/Defines the roll-off factor for the specified lower or upper gap
/// HIPAR rollOffFactor/channel's weighting filter.
ViStatus _VI_FUNC rsspecan_ConfigureMSRGapChannelRollOffFactor(
                            ViSession instrSession,
                            ViInt32   gap,
                            ViInt32   type,
                            ViInt32   gaps,
                            ViReal64  rollOffFactor)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, gap, 1, 2),
            2, "Gap");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, type, 0, 1),
            3, "Type");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, gaps, RSSPECAN_VAL_MSR_SBGAPS_AB, RSSPECAN_VAL_MSR_SBGAPS_GH),
            4, "Gaps");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, rollOffFactor, 0.0, 1.0), 5, "Roll Off Factor");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SENS:POW:ACH:FILT:ALPH:GAP%ld:MAN:%s %s,%.12lG", gap, gapLowUppArr[type], gapsArr[gaps], rollOffFactor);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the roll-off factor for the specified lower or upper gap
/// HIFN  channel's weighting filter.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR gap/This control selects the measurement window.
/// HIPAR type/Sets the type.
/// HIPAR gaps/Name of the gap, defined by the letters of the surrounding sub blocks
/// HIPAR gaps/(e.g. "AB" for the gap between sub blocks A and B).
/// HIPAR rollOffFactor/Queries the roll-off factor for the specified lower or upper gap
/// HIPAR rollOffFactor/channel's weighting filter.
ViStatus _VI_FUNC rsspecan_QueryMSRGapChannelRollOffFactor(ViSession instrSession,
                                                           ViInt32   gap,
                                                           ViInt32   type,
                                                           ViInt32   gaps,
                                                           ViReal64* rollOffFactor)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, gap, 1, 2),
            2, "Gap");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, type, 0, 1),
            3, "Type");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, gaps, RSSPECAN_VAL_MSR_SBGAPS_AB, RSSPECAN_VAL_MSR_SBGAPS_GH),
            4, "Gaps");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SENS:POW:ACH:FILT:ALPH:GAP%ld:MAN:%s? %s", gap, gapLowUppArr[type], gapsArr[gaps]);
    checkErr(RsCore_QueryViReal64(instrSession, cmd, rollOffFactor));

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Turns the relative limit check for the specified lower gap (CACLR)
/// HIFN  channel on and off.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR gap/This control selects the measurement window.
/// HIPAR type/Sets the type.
/// HIPAR gaps/Name of the gap, defined by the letters of the surrounding sub blocks
/// HIPAR gaps/(e.g. "AB" for the gap between sub blocks A and B).
/// HIPAR state/Turns the relative limit check for the specified lower gap (CACLR)
/// HIPAR state/channel on and off.
ViStatus _VI_FUNC rsspecan_ConfigureMSRGapLimitRelativeState(ViSession instrSession,
                                                             ViInt32   gap,
                                                             ViInt32   type,
                                                             ViInt32   gaps,
                                                             ViBoolean state)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, gap, 1, 2),
            2, "Gap");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, type, 0, 1),
            3, "Type");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, gaps, RSSPECAN_VAL_MSR_SBGAPS_AB, RSSPECAN_VAL_MSR_SBGAPS_GH),
            4, "Gaps");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CALC:LIM:ACP:GAP%ld:MAN:%s:STAT %s,%s", gap, gapLowUppArr[type], gapsArr[gaps], state ? "ON" : "OFF");
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the relative limit check for the specified lower gap (CACLR)
/// HIFN  channel on and off.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR gap/This control selects the measurement window.
/// HIPAR type/Sets the type.
/// HIPAR gaps/Name of the gap, defined by the letters of the surrounding sub blocks
/// HIPAR gaps/(e.g. "AB" for the gap between sub blocks A and B).
/// HIPAR state/Queries the relative limit check for the specified lower gap (CACLR)
/// HIPAR state/channel on and off.
ViStatus _VI_FUNC rsspecan_QueryMSRGapLimitRelativeState(ViSession  instrSession,
                                                         ViInt32    gap,
                                                         ViInt32    type,
                                                         ViInt32    gaps,
                                                         ViBoolean* state)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, gap, 1, 2),
            2, "Gap");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, type, 0, 1),
            3, "Type");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, gaps, RSSPECAN_VAL_MSR_SBGAPS_AB, RSSPECAN_VAL_MSR_SBGAPS_GH),
            4, "Gaps");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CALC:LIM:ACP:GAP%ld:MAN:%s:STAT? %s", gap, gapLowUppArr[type], gapsArr[gaps]);
    checkErr(RsCore_QueryViBoolean(instrSession, cmd, state));

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Defines the relative limit of the specified gap (CACLR) channel. The
/// HIFN  reference value for the relative limit is the measured channel power.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR gap/This control selects the measurement window.
/// HIPAR type/Sets the type.
/// HIPAR gaps/Name of the gap, defined by the letters of the surrounding sub blocks
/// HIPAR gaps/(e.g. "AB" for the gap between sub blocks A and B).
/// HIPAR limitRelative/Defines the relative limit of the specified gap (CACLR) channel. The
/// HIPAR limitRelative/reference value for the relative limit is the measured channel power.
ViStatus _VI_FUNC rsspecan_ConfigureMSRGapLimitRelative(ViSession instrSession,
                                                        ViInt32   gap,
                                                        ViInt32   type,
                                                        ViInt32   gaps,
                                                        ViReal64  limitRelative)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, gap, 1, 2),
            2, "Gap");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, type, 0, 1),
            3, "Type");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, gaps, RSSPECAN_VAL_MSR_SBGAPS_AB, RSSPECAN_VAL_MSR_SBGAPS_GH),
            4, "Gaps");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CALC:LIM:ACP:GAP%ld:MAN:%s %s,%.12lG", gap, gapLowUppArr[type], gapsArr[gaps], limitRelative);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the relative limit of the specified gap (CACLR) channel. The
/// HIFN  reference value for the relative limit is the measured channel power.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR gap/This control selects the measurement window.
/// HIPAR type/Sets the type.
/// HIPAR gaps/Name of the gap, defined by the letters of the surrounding sub blocks
/// HIPAR gaps/(e.g. "AB" for the gap between sub blocks A and B).
/// HIPAR limitRelative/Queries the relative limit of the specified gap (CACLR) channel. The
/// HIPAR limitRelative/reference value for the relative limit is the measured channel power.
ViStatus _VI_FUNC rsspecan_QueryMSRGapLimitRelative(ViSession instrSession,
                                                    ViInt32   gap,
                                                    ViInt32   type,
                                                    ViInt32   gaps,
                                                    ViReal64* limitRelative)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, gap, 1, 2),
            2, "Gap");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, type, 0, 1),
            3, "Type");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, gaps, RSSPECAN_VAL_MSR_SBGAPS_AB, RSSPECAN_VAL_MSR_SBGAPS_GH),
            4, "Gaps");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CALC:LIM:ACP:GAP%ld:MAN:%s? %s", gap, gapLowUppArr[type], gapsArr[gaps]);
    checkErr(RsCore_QueryViReal64(instrSession, cmd, limitRelative));

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Turns the absolute limit check for the specified gap (CACLR) channel
/// HIFN  on and off.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR gap/This control selects the measurement window.
/// HIPAR type/Sets the type.
/// HIPAR gaps/Name of the gap, defined by the letters of the surrounding sub blocks
/// HIPAR gaps/(e.g. "AB" for the gap between sub blocks A and B).
/// HIPAR state/Turns the absolute limit check for the specified gap (CACLR) channel
/// HIPAR state/on and off.
ViStatus _VI_FUNC rsspecan_ConfigureMSRGapLimitAbsoluteState(ViSession instrSession,
                                                             ViInt32   gap,
                                                             ViInt32   type,
                                                             ViInt32   gaps,
                                                             ViBoolean state)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, gap, 1, 2),
            2, "Window");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, type, 0, 1),
            3, "Type");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, gaps, RSSPECAN_VAL_MSR_SBGAPS_AB, RSSPECAN_VAL_MSR_SBGAPS_GH),
            4, "Gaps");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CALC:LIM:ACP:GAP%ld:MAN:%s:ABS:STAT %s,%s", gap, gapLowUppArr[type], gapsArr[gaps], state ? "ON" : "OFF");
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the absolute limit check for the specified gap (CACLR) channel
/// HIFN  on and off.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR gap/This control selects the measurement window.
/// HIPAR type/Sets the type.
/// HIPAR gaps/Name of the gap, defined by the letters of the surrounding sub blocks
/// HIPAR gaps/(e.g. "AB" for the gap between sub blocks A and B).
/// HIPAR state/Queries the absolute limit check for the specified gap (CACLR) channel
/// HIPAR state/on and off.
ViStatus _VI_FUNC rsspecan_QueryMSRGapLimitAbsoluteState(ViSession  instrSession,
                                                         ViInt32    gap,
                                                         ViInt32    type,
                                                         ViInt32    gaps,
                                                         ViBoolean* state)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, gap, 1, 2),
            2, "Gap");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, type, 0, 1),
            3, "Type");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, gaps, RSSPECAN_VAL_MSR_SBGAPS_AB, RSSPECAN_VAL_MSR_SBGAPS_GH),
            4, "Gaps");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CALC:LIM:ACP:GAP%ld:MAN:%s:ABS:STAT? %s", gap, gapLowUppArr[type], gapsArr[gaps]);
    checkErr(RsCore_QueryViBoolean(instrSession, cmd, state));

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Defines the absolute limit of the specified gap (CACLR) channel.
/// HIFN
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR gap/This control selects the measurement window.
/// HIPAR type/Sets the type.
/// HIPAR gaps/Name of the gap, defined by the letters of the surrounding sub blocks
/// HIPAR gaps/(e.g. "AB" for the gap between sub blocks A and B).
/// HIPAR limitAbsolute/Defines the absolute limit of the specified gap (CACLR) channel.
ViStatus _VI_FUNC rsspecan_ConfigureMSRGapLimitAbsolute(ViSession instrSession,
                                                        ViInt32   gap,
                                                        ViInt32   type,
                                                        ViInt32   gaps,
                                                        ViReal64  limitAbsolute)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, gap, 1, 2),
            2, "Gap");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, type, 0, 1),
            3, "Type");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, gaps, RSSPECAN_VAL_MSR_SBGAPS_AB, RSSPECAN_VAL_MSR_SBGAPS_GH),
            4, "Gaps");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CALC:LIM:ACP:GAP%ld:MAN:%s:ABS %s,%.12lG", gap, gapLowUppArr[type], gapsArr[gaps], limitAbsolute);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the absolute limit of the specified gap (CACLR) channel.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR gap/This control selects the measurement window.
/// HIPAR type/Sets the type.
/// HIPAR gaps/Name of the gap, defined by the letters of the surrounding sub blocks
/// HIPAR gaps/(e.g. "AB" for the gap between sub blocks A and B).
/// HIPAR limitAbsolute/Queries the absolute limit of the specified gap (CACLR) channel.
ViStatus _VI_FUNC rsspecan_QueryMSRGapLimitAbsolute(ViSession instrSession,
                                                    ViInt32   gap,
                                                    ViInt32   type,
                                                    ViInt32   gaps,
                                                    ViReal64* limitAbsolute)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, gap, 1, 2),
            2, "Gap");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, type, 0, 1),
            3, "Type");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, gaps, RSSPECAN_VAL_MSR_SBGAPS_AB, RSSPECAN_VAL_MSR_SBGAPS_GH),
            4, "Gaps");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CALC:LIM:ACP:GAP%ld:MAN:%s:ABS? %s", gap, gapLowUppArr[type], gapsArr[gaps]);
    checkErr(RsCore_QueryViReal64(instrSession, cmd, limitAbsolute));

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Defines which gap channels are active in the specified gap.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR gap/This control selects the measurement window.
/// HIPAR type/Sets the type.
/// HIPAR gaps/Name of the gap, defined by the letters of the surrounding sub blocks
/// HIPAR gaps/(e.g. "AB" for the gap between sub blocks A and B).
/// HIPAR enabled/Sets the state of the gap.
ViStatus _VI_FUNC rsspecan_ConfigureMSRGapLimitChannelsEnabled(
                            ViSession instrSession,
                            ViInt32   gap,
                            ViInt32   type,
                            ViInt32   gaps,
                            ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";
    ViChar*  p2LowUpp;
    ViChar*  p2rngGaps;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, gap, 1, 2),
            2, "Gap");
    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &RsSpecAn_rngMSRGapLowUpp, type, &p2LowUpp),
            3, "Type");
    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &RsSpecAn_rngMSRSBGaps, gaps, &p2rngGaps),
            4, "Gaps");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SENS:POW:ACH:GCH:GAP%ld:MAN:%s %s,%s", gap, p2LowUpp, p2rngGaps, enabled ? "ON" : "OFF");
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries if gap channels are active in the specified gap.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR gap/This control selects the measurement window.
/// HIPAR type/Sets the type.
/// HIPAR gaps/Name of the gap, defined by the letters of the surrounding sub blocks
/// HIPAR gaps/(e.g. "AB" for the gap between sub blocks A and B).
/// HIPAR enabled/Queries the state of the gap.
ViStatus _VI_FUNC rsspecan_QueryMSRGapLimitChannelsEnabled(ViSession  instrSession,
                                                           ViInt32    gap,
                                                           ViInt32    type,
                                                           ViInt32    gaps,
                                                           ViBoolean* enabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";
    ViChar   *p2LowUpp;
    ViChar   *p2rngGaps;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, gap, 1, 2),
            2, "Gap");
    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &RsSpecAn_rngMSRGapLowUpp, type, &p2LowUpp),
            3, "Type");
    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &RsSpecAn_rngMSRSBGaps, gaps, &p2rngGaps),
            4, "Gaps");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SENS:POW:ACH:GCH:GAP%ld:MAN:%s? %s", gap, p2LowUpp, p2rngGaps);
    checkErr(RsCore_QueryViBoolean(instrSession, cmd, enabled));

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines how gap channels are configured.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR gap/This control selects the measurement window.
/// HIPAR mode/Sets the type.
ViStatus _VI_FUNC rsspecan_ConfigureMSRGapMode(ViSession instrSession,
                                               ViInt32   gap,
                                               ViInt32   mode)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_MSR_GAP_MODE,
            "Gap", gap - 1, RS_REPCAP_BUF_SIZE, repCap));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_MSR_GAP_MODE, mode),
            3, "Mode");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Defines the relative limit for the ACLR power in the specified gap
/// HIFN  channel. The reference value for the relative limit is the measured
/// HIFN  channel power.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR gap/This control selects the measurement window.
/// HIPAR aclrRelative/Defines the relative limit for the ACLR power in the specified gap
/// HIPAR aclrRelative/channel. The reference value for the relative limit is the measured
/// HIPAR aclrRelative/channel power.
ViStatus _VI_FUNC rsspecan_ConfigureMSRGapLimitACLRRelative(ViSession instrSession,
                                                            ViInt32   gap,
                                                            ViReal64  aclrRelative)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_MSR_GAP_LIMIT_ACLR_RELATIVE,
            "Gap", gap - 1, RS_REPCAP_BUF_SIZE, repCap));

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_MSR_GAP_LIMIT_ACLR_RELATIVE, aclrRelative),
            3, "ACLR Relative");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Defines the relative limit for the ACLR power in the specified gap
/// HIFN  channel. The reference value for the relative limit is the measured
/// HIFN  channel power.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR gap/This control selects the measurement window.
/// HIPAR type/Sets the type.
/// HIPAR gaps/Name of the gap, defined by the letters of the surrounding sub blocks
/// HIPAR gaps/(e.g. "AB" for the gap between sub blocks A and B).
/// HIPAR aclrRelative/Defines the relative limit for the ACLR power in the specified gap
/// HIPAR aclrRelative/channel. The reference value for the relative limit is the measured
/// HIPAR aclrRelative/channel power.
ViStatus _VI_FUNC rsspecan_ConfigureMSRGapLimitACLRRelatives(ViSession instrSession,
                                                             ViInt32   gap,
                                                             ViInt32   type,
                                                             ViInt32   gaps,
                                                             ViReal64  aclrRelative)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, gap, 1, 2),
            2, "Gap");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, type, 0, 1),
            3, "Type");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, gaps, RSSPECAN_VAL_MSR_SBGAPS_AB, RSSPECAN_VAL_MSR_SBGAPS_GH),
            4, "Gaps");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CALC:LIM:ACP:GAP%ld:MAN:%s:ACLR %s,%.12lG", gap, gapLowUppArr[type], gapsArr[gaps], aclrRelative);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the relative limit for the ACLR power in the specified gap
/// HIFN  channel. The reference value for the relative limit is the measured
/// HIFN  channel power.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR gap/This control selects the measurement window.
/// HIPAR type/Sets the type.
/// HIPAR gaps/Name of the gap, defined by the letters of the surrounding sub blocks
/// HIPAR gaps/(e.g. "AB" for the gap between sub blocks A and B).
/// HIPAR aclrRelative/Queries the relative limit for the ACLR power in the specified gap
/// HIPAR aclrRelative/channel. The reference value for the relative limit is the measured
/// HIPAR aclrRelative/channel power.
ViStatus _VI_FUNC rsspecan_QueryMSRGapLimitACLRRelatives(ViSession instrSession,
                                                         ViInt32   gap,
                                                         ViInt32   type,
                                                         ViInt32   gaps,
                                                         ViReal64* aclrRelative)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, gap, 1, 2),
            2, "Gap");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, type, 0, 1),
            3, "Type");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, gaps, RSSPECAN_VAL_MSR_SBGAPS_AB, RSSPECAN_VAL_MSR_SBGAPS_GH),
            4, "Gaps");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CALC:LIM:ACP:GAP%ld:MAN:%s:ACLR? %s", gap, gapLowUppArr[type], gapsArr[gaps]);
    checkErr(RsCore_QueryViReal64(instrSession, cmd, aclrRelative));

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Turns the relative limit check for the specified gap (CACLR) channel
/// HIFN  on and off.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR gap/This control selects the measurement window.
/// HIPAR state/Turns the relative limit check for the specified gap (CACLR) channel
/// HIPAR state/on and off.
ViStatus _VI_FUNC rsspecan_ConfigureMSRGapLimitACLRRelativeState(
                            ViSession instrSession,
                            ViInt32   gap,
                            ViBoolean state)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_MSR_GAP_LIMIT_ACLR_RELATIVE_STATE,
            "Gap", gap - 1, RS_REPCAP_BUF_SIZE, repCap));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, repCap, RSSPECAN_ATTR_MSR_GAP_LIMIT_ACLR_RELATIVE_STATE, state),
            3, "State");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Turns the relative limit check for the specified lower or upper gap
/// HIFN  (CACLR) channel on and off.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR gap/This control selects the measurement window.
/// HIPAR type/Sets the type.
/// HIPAR gaps/Name of the gap, defined by the letters of the surrounding sub blocks
/// HIPAR gaps/(e.g. "AB" for the gap between sub blocks A and B).
/// HIPAR state/Turns the relative limit check for the specified lower or upper gap
/// HIPAR state/(CACLR) channel on and off.
ViStatus _VI_FUNC rsspecan_ConfigureMSRGapLimitACLRRelativeStates(
                            ViSession instrSession,
                            ViInt32   gap,
                            ViInt32   type,
                            ViInt32   gaps,
                            ViBoolean state)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, gap, 1, 2),
            2, "Window");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, type, 0, 1),
            3, "Type");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, gaps, RSSPECAN_VAL_MSR_SBGAPS_AB, RSSPECAN_VAL_MSR_SBGAPS_GH),
            4, "Gaps");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CALC:LIM:ACP:GAP%ld:MAN:%s:ACLR:STAT %s,%s", gap, gapLowUppArr[type], gapsArr[gaps], state ? "ON" : "OFF");
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the relative limit check for the specified lower or upper gap
/// HIFN  (CACLR) channel on and off.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR gap/This control selects the measurement window.
/// HIPAR type/Sets the type.
/// HIPAR gaps/Name of the gap, defined by the letters of the surrounding sub blocks
/// HIPAR gaps/(e.g. "AB" for the gap between sub blocks A and B).
/// HIPAR state/Queries the relative limit check for the specified lower or upper gap
/// HIPAR state/(CACLR) channel on and off.
ViStatus _VI_FUNC rsspecan_QueryMSRGapLimitACLRRelativeStates(
                            ViSession  instrSession,
                            ViInt32    gap,
                            ViInt32    type,
                            ViInt32    gaps,
                            ViBoolean* state)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, gap, 1, 2),
            2, "Gap");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, type, 0, 1),
            3, "Type");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, gaps, RSSPECAN_VAL_MSR_SBGAPS_AB, RSSPECAN_VAL_MSR_SBGAPS_GH),
            4, "Gaps");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CALC:LIM:ACP:GAP%ld:MAN:%s:ACLR:STAT? %s", gap, gapLowUppArr[type], gapsArr[gaps]);
    checkErr(RsCore_QueryViBoolean(instrSession, cmd, state));

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Defines the relative limit of the specified gap (CACLR) channel. The
/// HIFN  reference value for the relative limit is the measured channel power.
/// HIFN     If you define both an absolute limit and a relative limit, the R&S FSW
/// HIFN  uses the lower value for the limit check.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR gap/This control selects the measurement window.
/// HIPAR limit/Defines the relative limit of the specified gap channel in dB.
/// HIPAR upperLimit/Defines the relative upper limit of the specified gap channel.
ViStatus _VI_FUNC rsspecan_ConfigureMSRGapLimitCACLRRelative(ViSession instrSession,
                                                             ViInt32   gap,
                                                             ViReal64  limit,
                                                             ViReal64  upperLimit)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, gap, 1, 2),
            2, "Gap");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CALC:LIM:ACP:GAP%ld %.12lG,%.12lG", gap, limit, upperLimit);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Turns the relative limit check for the specified gap (CACLR) channel
/// HIFN  on and off.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR gap/This control selects the measurement window.
/// HIPAR state/Turns the relative limit check for the specified gap (CACLR) channel
/// HIPAR state/on and off.
ViStatus _VI_FUNC rsspecan_ConfigureMSRGapLimitCACLRRelativeState(
                            ViSession instrSession,
                            ViInt32   gap,
                            ViBoolean state)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_MSR_GAP_LIMIT_CACLR_RELATIVE_STATE,
            "Gap", gap - 1, RS_REPCAP_BUF_SIZE, repCap));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, repCap, RSSPECAN_ATTR_MSR_GAP_LIMIT_CACLR_RELATIVE_STATE, state),
            3, "State");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Returns the ACLR power limit check results for the selected gap
/// HIFN  channel in an MSR ACLR measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR gap/This control selects the measurement window.
/// HIPAR arraySize/Sets the array size.
/// HIPAR result/Returns the ACLR power limit check results for the selected gap
/// HIPAR result/channel in an MSR ACLR measurement.
ViStatus _VI_FUNC rsspecan_QueryMSRGapLimitACLRResult(ViSession instrSession,
                                                      ViInt32   gap,
                                                      ViInt32   arraySize,
                                                      ViReal64  result[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, gap, 1, 2),
            2, "Gap");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CALC:LIM:ACP:GAP%ld:ACLR:RES?", gap);
    checkErr(RsCore_QueryFloatArrayToUserBuffer(instrSession, cmd, arraySize, result, NULL));

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function defines the upper adjacent channel.
/// HIFN
/// HIFN     Note(s):
/// HIFN
/// HIFN     (1) This function is available only for R&S FSW instrument.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR channelSpacing/This control defines the distance from the transmission channel to the
/// HIPAR channelSpacing/upper adjacent channel. For MSR signals, this command defines the
/// HIPAR channelSpacing/distance from the CF of the last Tx channel in the last sub block to
/// HIPAR channelSpacing/the upper adjacent channel in asymmetrical configurations. To configure
/// HIPAR channelSpacing/the spacing for the lower adjacent channel use the
/// HIPAR channelSpacing/RSSPECAN_ATTR_MEAS_POW_ADJ_CHANNEL_SPACING command.
/// HIPAR channelBandwidth/This control defines the channel bandwidth of the upper adjacent
/// HIPAR channelBandwidth/channel in asymmetrical configurations. The adjacent channel is the
/// HIPAR channelBandwidth/first pair of channels next to the transmission channels. To configure
/// HIPAR channelBandwidth/the bandwidth for the lower adjacent channel, use the
/// HIPAR channelBandwidth/RSSPECAN_ATTR_MEAS_POW_ADJ_CHANNEL_BANDWIDTH command.
/// HIPAR weightingFilterState/This control turns the weighting filter for the upper adjacent channel
/// HIPAR weightingFilterState/on and off for asymmetrical MSR signals. To configure the factor for
/// HIPAR weightingFilterState/the lower adjacent channel, use the
/// HIPAR weightingFilterState/RSSPECAN_ATTR_SEM_WEIGHT_FILTER_ACH_STATE command.
/// HIPAR rollOffFactor/This control defines the roll-off factor for the upper adjacent
/// HIPAR rollOffFactor/channel weighting filter for asymmetrical MSR signals. To configure the
/// HIPAR rollOffFactor/factor for the upper adjacent channel, use the
/// HIPAR rollOffFactor/RSSPECAN_ATTR_SEM_WEIGHT_FILTER_ACH_ALPHA command.
ViStatus _VI_FUNC rsspecan_ConfigureMSRUpperAdjChannel(ViSession instrSession,
                                                       ViReal64  channelSpacing,
                                                       ViReal64  channelBandwidth,
                                                       ViBoolean weightingFilterState,
                                                       ViReal64  rollOffFactor)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_MSR_UPPER_ADJACENT_CHANNEL_SPACING, channelSpacing),
            2, "Channel Spacing");

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_MSR_UPPER_ADJACENT_CHANNEL_BANDWIDTH, channelBandwidth),
            3, "Channel Bandwidth");

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_MSR_UPPER_ADJACENT_CHANNEL_WEIGHTING_FILTER_STATE, weightingFilterState),
            4, "Weighting Filter State");

    if (weightingFilterState == VI_TRUE)
    {
        viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_MSR_UPPER_ADJACENT_CHANNEL_WEIGHTING_FILTER_ALPHA_VALUE, rollOffFactor),
                5, "Roll Off Factor");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function defines the upper alternate channel.
/// HIFN
/// HIFN     Note(s):
/// HIFN
/// HIFN     (1) This function is available only for R&S FSW instrument.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR channelNumber/Sets the channel number.
/// HIPAR channelSpacing/This control defines the distance from transmission channel to the
/// HIPAR channelSpacing/upper alternate channels. For MSR signals, this command defines the
/// HIPAR channelSpacing/distance from the CF of the last Tx channel in the last sub block to
/// HIPAR channelSpacing/the upper alternate channel in asymmetrical configurations. To
/// HIPAR channelSpacing/configure the spacing for the lower alternate channel, use the
/// HIPAR channelSpacing/RSSPECAN_ATTR_MEAS_POW_ALT_CHANNEL_SPACING command.
/// HIPAR channelBandwidth/This control defines the channel bandwidth of the upper alternate
/// HIPAR channelBandwidth/channels in asymmetrical configurations. To configure the bandwidth for
/// HIPAR channelBandwidth/the lower alternate channel, use the
/// HIPAR channelBandwidth/RSSPECAN_ATTR_MEAS_POW_ALT_CHANNEL_BANDWIDTH command.
/// HIPAR weightingFilterState/This control turns the weighting filter for the upper alternate
/// HIPAR weightingFilterState/channels on and off for asymmetrical MSR signals. To configure the
/// HIPAR weightingFilterState/factor for the lower alternate channels, use the
/// HIPAR weightingFilterState/RSSPECAN_ATTR_POWER_ALT_CHANNEL_WEIGHTING_FILTER_STATE command.
/// HIPAR rollOffFactor/This control defines the roll-off factor for the upper alternate
/// HIPAR rollOffFactor/channels' weighting filter for asymmetrical MSR signals. To configure
/// HIPAR rollOffFactor/the factor for the upper alternate channels, use the
/// HIPAR rollOffFactor/RSSPECAN_ATTR_POWER_ALT_CHANNEL_WEIGHTING_FILTER_ALPHA_VALUE command.
ViStatus _VI_FUNC rsspecan_ConfigureMSRUpperAlternateChannel(ViSession instrSession,
                                                             ViInt32   channelNumber,
                                                             ViReal64  channelSpacing,
                                                             ViReal64  channelBandwidth,
                                                             ViBoolean weightingFilterState,
                                                             ViReal64  rollOffFactor)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_MSR_UPPER_ALTERNATE_CHANNEL_SPACING,
            "Channel", channelNumber - 1, RS_REPCAP_BUF_SIZE, repCap));

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_MSR_UPPER_ALTERNATE_CHANNEL_SPACING, channelSpacing),
            3, "Channel Spacing");

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_MSR_UPPER_ALTERNATE_CHANNEL_BANDWIDTH, channelBandwidth),
            4, "Channel Bandwidth");

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, repCap, RSSPECAN_ATTR_MSR_UPPER_ALTERNATE_CHANNEL_WEIGHTING_FILTER_STATE, weightingFilterState),
            5, "Weighting Filter State");

    if (weightingFilterState == VI_TRUE)
    {
        viCheckParm(rsspecan_SetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_MSR_UPPER_ALTERNATE_CHANNEL_WEIGHTING_FILTER_ALPHA_VALUE, rollOffFactor),
                6, "Roll Off Factor");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function defines the limit value of the gap ACLR channel.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR gap/This control selects the measurement window.
/// HIPAR relativeState/Turns the relative limit check for the specified gap ACLR channel
/// HIPAR relativeState/on and off.
/// HIPAR relativeValue/Defines the relative limit of the specified gap ACLR channel. The
/// HIPAR relativeValue/reference value for the relative limit is the measured channel power.
/// HIPAR absoluteState/Turns the absolute limit check for the specified gap ACLR channel
/// HIPAR absoluteState/on and off.
/// HIPAR absoluteValue/Defines the absolute limit of the specified gap ACLR channel.
ViStatus _VI_FUNC rsspecan_ConfigureMSRGapACLRLimitChecking(ViSession instrSession,
                                                            ViInt32   gap,
                                                            ViBoolean relativeState,
                                                            ViReal64  relativeValue,
                                                            ViBoolean absoluteState,
                                                            ViReal64  absoluteValue)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViChar   repCap[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsspecan_AssertNotFSV(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_MSR_GAP_LIMIT_RELATIVE_STATE,
            "Gap", gap - 1, RS_REPCAP_BUF_SIZE, repCap));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, repCap, RSSPECAN_ATTR_MSR_GAP_LIMIT_RELATIVE_STATE, relativeState),
            3, "Relative State");
    if (relativeState)
    {
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CALC:LIM:ACP:GAP%ld:ACLR:REL %.12lG,0.0", gap, relativeValue);
        checkErr(RsCore_Write(instrSession, cmd));
        checkErr(rsspecan_CheckStatus(instrSession));
    }

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, repCap, RSSPECAN_ATTR_MSR_GAP_LIMIT_ABSOLUTE_STATE, absoluteState),
            5, "Absolute State");
    if (absoluteState)
    {
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CALC:LIM:ACP:GAP%ld:ABS %.12lG,0.0", gap, absoluteValue);
        checkErr(RsCore_Write(instrSession, cmd));
        checkErr(rsspecan_CheckStatus(instrSession));
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/*===========================================================================*
 *------------------- Carrier-to-noise --------------------------------------*
 *===========================================================================*/
/// HIFN This function configures the Carrier to Noise measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init
/// HIPAR instrSession/or rsspecan_InitWithOptions function.
/// HIPAR state/This control sets the state of selected measurement.
/// HIPAR measurement/Selects the measurement.
/// HIPAR bandwidth/Sets the channel bandwidth.
ViStatus _VI_FUNC rsspecan_ConfigureCarrierNoiseMeasurement(ViSession instrSession,
                                                            ViBoolean state,
                                                            ViInt32   measurement,
                                                            ViReal64  bandwidth)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    if (state)
    {
        viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_MEAS_POW_SELECT, measurement),
                3, "Measurement");
        if (measurement == RSSPECAN_VAL_MEAS_POW_CN)
        viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "TX1", RSSPECAN_ATTR_POW_CHANNEL_BANDWIDTH, bandwidth),
                4, "Bandwidth");
    }
    else
    {
        checkErr(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_MEAS_POW_OFF, NULL));
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/*===========================================================================*
 *------------------- Occupied Bandwidth ------------------------------------*
 *===========================================================================*/
/// HIFN This function defines the percentage of the power with respect
/// HIFN to the total power.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init
/// HIPAR instrSession/or rsspecan_InitWithOptions function.
/// HIPAR powerBandwidth/Sets the percentage of the power with respect to the total
/// HIPAR powerBandwidth/power.
/// HIPAR channelBandwidth/Sets the selected channel bandwidth.
ViStatus _VI_FUNC rsspecan_ConfigureOccupiedBandwidth(ViSession instrSession,
                                                      ViInt32   reserved,
                                                      ViReal64  powerBandwidth,
                                                      ViReal64  channelBandwidth)
{
    ViStatus error = VI_SUCCESS;
             reserved;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_MEAS_POW_BANDWIDTH, powerBandwidth),
            3, "Power Bandwidth");

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "TX1", RSSPECAN_ATTR_POW_CHANNEL_BANDWIDTH, channelBandwidth),
            4, "Channel Bandwidth");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function configures the limit lines.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init
/// HIPAR instrSession/or rsspecan_InitWithOptions function.
/// HIPAR searchLimits/Switches between a limited (ON) and unlimited (OFF) search
/// HIPAR searchLimits/range.
/// HIPAR searchLimitLeft/Sets the left limit of the search range for markers and delta
/// HIPAR searchLimitLeft/markers.
/// HIPAR searchLimitRight/Sets the right limit of the search range for markers and delta
/// HIPAR searchLimitRight/markers.
ViStatus _VI_FUNC rsspecan_ConfigureOccupiedBadwidthLimits(ViSession instrSession,
                                                           ViBoolean searchLimits,
                                                           ViReal64  searchLimitLeft,
                                                           ViReal64  searchLimitRight)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_MARKER_SEARCH_LIMITS_STATE, searchLimits),
            2, "Search Limit");
    if (searchLimits == VI_TRUE)
    {
        viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_MARKER_SEARCH_LIMITS_LEFT, searchLimitLeft),
                3, "Search Limit Left");

        viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_MARKER_SEARCH_LIMITS_RIGHT, searchLimitRight),
                4, "Search Limit Right");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/*===========================================================================*
 *------------------- Spectrum Emission Mask --------------------------------*
 *===========================================================================*/
/// HIFN This function configures the Spectrum EM Mask Measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR state/Switches on or off the spectrum emission mask measurement.
ViStatus _VI_FUNC rsspecan_ConfigureSEMState(ViSession instrSession,
                                             ViBoolean state)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    if (state)
    {
        checkErr(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_SWEEP_MODE, RSSPECAN_VAL_SWE_MODE_ESP));
    }
    else
        checkErr(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_SWEEP_MODE, RSSPECAN_VAL_SWE_MODE_AUTO));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function searches the peaks in spectrum emission data.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
ViStatus _VI_FUNC rsspecan_SEMSearchPeak(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_ESP_PEAK_SEARCH, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/*===========================================================================*
 *------------------- SEM -> Configuration ----------------------------------*
 *===========================================================================*/
/// HIFN This function configures the reference range.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR subBlock/This control selects the sub block in a Multi-SEM measurement.
/// HIPAR powerReferenceType/This control sets the power reference type.
/// HIPAR txBandwidth/This control defines the bandwidth used for measuring the
/// HIPAR txBandwidth/channel power (reference range).
/// HIPAR RRCFilterState/This control activates or deactivates the use of an RRC filter.
/// HIPAR RRCFilterAlpha/This control sets the alpha value of the RRC filter.
ViStatus _VI_FUNC rsspecan_ConfigureSEMReferenceRange(ViSession instrSession,
                                                      ViInt32   subBlock,
                                                      ViInt32   powerReferenceType,
                                                      ViReal64  txBandwidth,
                                                      ViBoolean RRCFilterState,
                                                      ViReal64  RRCFilterAlpha)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, subBlock, RSSPECAN_VAL_SUB_BLOCK_A, RSSPECAN_VAL_SUB_BLOCK_C),
            2, "Sub Block");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_SEM_POWER_REF_TYPE,
            "MultiSEMSubBlock", subBlock - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Sub Block");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_SEM_POWER_REF_TYPE, powerReferenceType),
            3, "Power Reference Type");

    if (powerReferenceType == RSSPECAN_VAL_SEM_REF_TYPE_CPOW)
    {
        viCheckParm(rsspecan_SetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_SEM_BANDWIDTH, txBandwidth),
                4, "TX Bandwidth");

        viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, repCap, RSSPECAN_ATTR_SEM_RRC_FILTER_STATE, RRCFilterState),
                5, "RRC Filter State");

        if (RRCFilterState == VI_TRUE)
        viCheckParm(rsspecan_SetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_SEM_RRC_FILTER_ALPHA_VALUE, RRCFilterAlpha),
                6, "RRC Filter Alpha");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function returns the current position (number) of the
/// HIFN reference range.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.
/// HIPAR subBlock/This control selects the sub block in a Multi-SEM measurement.
/// HIPAR referenceRangePosition/Returns the current position (number) of the reference range.
ViStatus _VI_FUNC rsspecan_QuerySEMReferenceRangePosition(ViSession instrSession,
                                                          ViInt32   subBlock,
                                                          ViInt32   *referenceRangePosition)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, subBlock, RSSPECAN_VAL_SUB_BLOCK_A, RSSPECAN_VAL_SUB_BLOCK_C),
            2, "Sub Block");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_SEM_REF_RANGE_POSITION,
            "MultiSEMSubBlock", subBlock - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Sub Block");

    viCheckParm(rsspecan_GetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_SEM_REF_RANGE_POSITION, referenceRangePosition),
            3, "Reference Range Position");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the number of sub blocks in the SEM
/// HIFN  measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR subBlockCount/This control defines the number of sub blocks in the SEM measurement.
ViStatus _VI_FUNC rsspecan_ConfigureSEMSubBlockCount(ViSession instrSession,
                                                     ViInt32   subBlockCount)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_SPECTRUM_EMISSION_MASK_SUB_BLOCK_COUNT, subBlockCount),
            2, "Sub Block Count");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the center frequency of the selected sub
/// HIFN  block in a Multi-SEM measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR subBlock/This control selects the sub block in a Multi-SEM measurement.
/// HIPAR centerFrequency/This control defines the center frequency of the selected sub block in
/// HIPAR centerFrequency/a Multi-SEM measurement.
ViStatus _VI_FUNC rsspecan_ConfigureSEMSubBlockCenterFrequency(
                            ViSession instrSession,
                            ViInt32   subBlock,
                            ViReal64  centerFrequency)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, subBlock, RSSPECAN_VAL_SUB_BLOCK_A, RSSPECAN_VAL_SUB_BLOCK_C),
            2, "Sub Block");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_SPECTRUM_EMISSION_MASK_SUB_BLOCK_CENTER_FREQUENCY,
            "MultiSEMSubBlock", subBlock - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Sub Block");

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_SPECTRUM_EMISSION_MASK_SUB_BLOCK_CENTER_FREQUENCY, centerFrequency),
            3, "Center Frequency");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Enables or disables symmetrical configuration of the range settings.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR subBlock/This control selects the sub block in a Multi-SEM measurement.
/// HIPAR symmetricalSetup/Enables or disables symmetrical configuration of the range settings.
ViStatus _VI_FUNC rsspecan_ConfigureSEMSymmetricalSetup(ViSession instrSession,
                                                        ViInt32   subBlock,
                                                        ViBoolean symmetricalSetup)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_SPECTRUM_EMMISION_MASK_SYMMETRICAL_SETUP,
            "MultiSEMSubBlock", subBlock - 1, RS_REPCAP_BUF_SIZE, repCap));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, repCap, RSSPECAN_ATTR_SPECTRUM_EMMISION_MASK_SYMMETRICAL_SETUP, symmetricalSetup),
            3, "Symmetrical Setup");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/*===========================================================================*
 *------------------- SEM -> Configuration-> Sweep List ---------------------*
 *===========================================================================*/
/// HIFN This function configures the start and stop frequency for the
/// HIFN specified range.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.
/// HIPAR subBlock/This control selects the sub block in a Multi-SEM measurement.
/// HIPAR range/Selects the range.
/// HIPAR rangeStart/This control sets the start frequency for the specified range.
/// HIPAR rangeStop/This control sets the stop frequency for the specified range.
ViStatus _VI_FUNC rsspecan_ConfigureSEMRangeStartStop(ViSession instrSession,
                                                      ViInt32   subBlock,
                                                      ViInt32   range,
                                                      ViReal64  rangeStart,
                                                      ViReal64  rangeStop)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_r[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, subBlock, RSSPECAN_VAL_SUB_BLOCK_A, RSSPECAN_VAL_SUB_BLOCK_C),
            2, "Sub Block");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, range, 1, 30),
            3, "Range");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_SEM_RANGE_FREQ_START,
            "MultiSEMSubBlock", subBlock - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Sub Block");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_SEM_RANGE_FREQ_START,
            "Range", range - 1, RS_REPCAP_BUF_SIZE, repCap_r),
            3, "Range");

    strcat(repCap, ",");
    strcat(repCap, repCap_r);

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_SEM_RANGE_FREQ_START, rangeStart),
            4, "Range Start");

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_SEM_RANGE_FREQ_STOP, rangeStop),
            5, "Range Stop");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN This function configures the filter type for the specified range.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.
/// HIPAR subBlock/This control selects the sub block in a Multi-SEM measurement.
/// HIPAR range/Selects the range.
/// HIPAR filterType/Document parameter here.
ViStatus _VI_FUNC rsspecan_ConfigureSEMRangeFilterType(ViSession instrSession,
                                                       ViInt32   subBlock,
                                                       ViInt32   range,
                                                       ViInt32   filterType)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_r[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, subBlock, RSSPECAN_VAL_SUB_BLOCK_A, RSSPECAN_VAL_SUB_BLOCK_C),
            2, "Sub Block");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, range, 1, 30),
            3, "Range");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_SEM_RANGE_FILTER_TYPE,
            "MultiSEMSubBlock", subBlock - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Sub Block");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_SEM_RANGE_FILTER_TYPE,
            "Range", range - 1, RS_REPCAP_BUF_SIZE, repCap_r),
            3, "Range");

    strcat(repCap, ",");
    strcat(repCap, repCap_r);

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_SEM_RANGE_FILTER_TYPE, filterType),
            4, "Filter Type");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN This function configures the resolution and video bandwidth values
/// HIFN for the specified range.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.
/// HIPAR subBlock/This control selects the sub block in a Multi-SEM measurement.
/// HIPAR range/Selects the range.
/// HIPAR resolutionBandwidth/This control sets the RBW value for the specified range.
/// HIPAR videoBandwidth/This control sets the VBW value for the specified range.
ViStatus _VI_FUNC rsspecan_ConfigureSEMRangeBandwidth(ViSession instrSession,
                                                      ViInt32   subBlock,
                                                      ViInt32   range,
                                                      ViReal64  resolutionBandwidth,
                                                      ViReal64  videoBandwidth)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_r[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, subBlock, RSSPECAN_VAL_SUB_BLOCK_A, RSSPECAN_VAL_SUB_BLOCK_C),
            2, "Sub Block");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, range, 1, 30),
            3, "Range");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_SEM_RANGE_BANDWIDTH,
            "MultiSEMSubBlock", subBlock - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Sub Block");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_SEM_RANGE_BANDWIDTH,
            "Range", range - 1, RS_REPCAP_BUF_SIZE, repCap_r),
            3, "Range");

    strcat(repCap, ",");
    strcat(repCap, repCap_r);

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_SEM_RANGE_BANDWIDTH, resolutionBandwidth),
            4, "Resolution Bandwidth");

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_SEM_RANGE_VBW, videoBandwidth),
            5, "Video Bandwidth");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function configures the sweep time values for the specified
/// HIFN range.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.
/// HIPAR subBlock/This control selects the sub block in a Multi-SEM measurement.
/// HIPAR range/Selects the range.
/// HIPAR sweepTimeMode/This control activates or deactivates the automatic sweep time
/// HIPAR sweepTimeMode/setting for the specified range.
/// HIPAR sweepTime/This control sets the sweep time for the specified range.
ViStatus _VI_FUNC rsspecan_ConfigureSEMRangeSweepTime(ViSession instrSession,
                                                      ViInt32   subBlock,
                                                      ViInt32   range,
                                                      ViBoolean sweepTimeMode,
                                                      ViReal64  sweepTime)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_r[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, subBlock, RSSPECAN_VAL_SUB_BLOCK_A, RSSPECAN_VAL_SUB_BLOCK_C),
            2, "Sub Block");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, range, 1, 30),
            3, "Range");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_SEM_RANGE_SWEEP_TIME_AUTO,
            "MultiSEMSubBlock", subBlock - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Sub Block");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_SEM_RANGE_SWEEP_TIME_AUTO,
            "Range", range - 1, RS_REPCAP_BUF_SIZE, repCap_r),
            3, "Range");

    strcat(repCap, ",");
    strcat(repCap, repCap_r);

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, repCap, RSSPECAN_ATTR_SEM_RANGE_SWEEP_TIME_AUTO, sweepTimeMode),
            4, "Sweep Time Mode");

    if (sweepTimeMode == VI_FALSE)
    {
        viCheckParm(rsspecan_SetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_SEM_RANGE_SWEEP_TIME, sweepTime),
                5, "Sweep Time");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function configures the reference level for the specified
/// HIFN range.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.
/// HIPAR subBlock/This control selects the sub block in a Multi-SEM measurement.
/// HIPAR range/Selects the range.
/// HIPAR referenceLevel/This control sets the reference level for the specified range.
ViStatus _VI_FUNC rsspecan_ConfigureSEMRangeReferenceLevel(ViSession instrSession,
                                                           ViInt32   subBlock,
                                                           ViInt32   range,
                                                           ViReal64  referenceLevel)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_r[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, subBlock, RSSPECAN_VAL_SUB_BLOCK_A, RSSPECAN_VAL_SUB_BLOCK_C),
            2, "Sub Block");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, range, 1, 30),
            3, "Range");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_SEM_RANGE_REF_LEVEL,
            "MultiSEMSubBlock", subBlock - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Sub Block");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_SEM_RANGE_REF_LEVEL,
            "Range", range - 1, RS_REPCAP_BUF_SIZE, repCap_r),
            3, "Range");

    strcat(repCap, ",");
    strcat(repCap, repCap_r);

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_SEM_RANGE_REF_LEVEL, referenceLevel),
            4, "Reference Level");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function configures the RF attenuation setting for the
/// HIFN specified range.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.
/// HIPAR subBlock/This control selects the sub block in a Multi-SEM measurement.
/// HIPAR range/Selects the range.
/// HIPAR RFAttenuationMode/This control activates or deactivates the automatic RF
/// HIPAR RFAttenuationMode/attenuation setting for the specified range.
/// HIPAR RFAttenuator/This control sets the attenuation for the specified range.
/// HIPAR preamplifier/This control switches the preamplifier on or off for the
/// HIPAR preamplifier/specified range.
ViStatus _VI_FUNC rsspecan_ConfigureSEMRangeRFAttenuation(ViSession instrSession,
                                                          ViInt32   subBlock,
                                                          ViInt32   range,
                                                          ViBoolean RFAttenuationMode,
                                                          ViReal64  RFAttenuator,
                                                          ViBoolean preamplifier)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_r[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, subBlock, RSSPECAN_VAL_SUB_BLOCK_A, RSSPECAN_VAL_SUB_BLOCK_C),
            2, "Sub Block");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, range, 1, 30),
            3, "Range");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_SEM_RANGE_INPUT_ATTENUATION_AUTO,
            "MultiSEMSubBlock", subBlock - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Sub Block");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_SEM_RANGE_INPUT_ATTENUATION_AUTO,
            "Range", range - 1, RS_REPCAP_BUF_SIZE, repCap_r),
            3, "Range");

    strcat(repCap, ",");
    strcat(repCap, repCap_r);

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, repCap, RSSPECAN_ATTR_SEM_RANGE_INPUT_ATTENUATION_AUTO, RFAttenuationMode),
            4, "RF Attenuation Mode");
    if (RFAttenuationMode == VI_FALSE)
    {
        viCheckParm(rsspecan_SetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_SEM_RANGE_INPUT_ATTENUATION, RFAttenuator),
                5, "RF Attenuator");
    }
    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, repCap, RSSPECAN_ATTR_SEM_RANGE_PREAMPLIFIER, preamplifier),
            6, "Preamplifier");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN This function configures level of pre-amplifier for selected sweep range.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.
/// HIPAR subBlock/This control selects the sub block in a Multi-SEM measurement.
/// HIPAR range/Sets the configured range.
/// HIPAR preamplificationLevel/Selects the preamplification level.
ViStatus _VI_FUNC rsspecan_ConfigureSEMRangeRFPreamplifierLevel(
                            ViSession instrSession,
                            ViInt32   subBlock,
                            ViInt32   range,
                            ViInt32   preamplificationLevel)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_r[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsspecan_AssertFSV(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, subBlock, RSSPECAN_VAL_SUB_BLOCK_A, RSSPECAN_VAL_SUB_BLOCK_C),
            2, "Sub Block");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, range, 1, 30),
            3, "Range");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_SEM_RANGE_PREAMPLIFIER_LEVEL,
            "MultiSEMSubBlock", subBlock - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Sub Block");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_SEM_RANGE_PREAMPLIFIER_LEVEL,
            "Range", range - 1, RS_REPCAP_BUF_SIZE, repCap_r),
            3, "Range");

    strcat(repCap, ",");
    strcat(repCap, repCap_r);

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_SEM_RANGE_PREAMPLIFIER_LEVEL, preamplificationLevel),
            4, "Preamplification Level");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN This function configures a transducer for the specified range.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.
/// HIPAR subBlock/This control selects the sub block in a Multi-SEM measurement.
/// HIPAR range/Sets the configured range.
/// HIPAR transducerFactor/This control sets a transducer for the specified range.
ViStatus _VI_FUNC rsspecan_ConfigureSEMRangeTransducerFactor(ViSession instrSession,
                                                             ViInt32   subBlock,
                                                             ViInt32   range,
                                                             ViString  transducerFactor)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_r[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, subBlock, RSSPECAN_VAL_SUB_BLOCK_A, RSSPECAN_VAL_SUB_BLOCK_C),
            2, "Sub Block");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, range, 1, 30),
            3, "Range");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_SEM_RANGE_TRANSDUCER,
            "MultiSEMSubBlock", subBlock - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Sub Block");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_SEM_RANGE_TRANSDUCER,
            "Range", range - 1, RS_REPCAP_BUF_SIZE, repCap_r),
            3, "Range");

    strcat(repCap, ",");
    strcat(repCap, repCap_r);

    viCheckParm(rsspecan_SetAttributeViString(instrSession, repCap, RSSPECAN_ATTR_SEM_RANGE_TRANSDUCER, transducerFactor),
            4, "Transducer Factor");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the minimum number of sweep points for the
/// HIFN  range.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR subBlock/This control selects the sub block in a Multi-SEM measurement.
/// HIPAR range/Selects the range.
/// HIPAR minimumSweepPoints/Defines the minimum number of sweep points for the range.
ViStatus _VI_FUNC rsspecan_ConfigureSEMRangeMinimumSweepPoints(
                            ViSession instrSession,
                            ViInt32   subBlock,
                            ViInt32   range,
                            ViInt32   minimumSweepPoints)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_r[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, subBlock, RSSPECAN_VAL_SUB_BLOCK_A, RSSPECAN_VAL_SUB_BLOCK_C),
            2, "Sub Block");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, range, 1, 30),
            3, "Range");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_SPECTRUM_EMISSION_MASK_RANGE_MINIMUM_SWEEP_POINTS,
            "MultiSEMSubBlock", subBlock - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Sub Block");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_SPECTRUM_EMISSION_MASK_RANGE_MINIMUM_SWEEP_POINTS,
            "Range", range - 1, RS_REPCAP_BUF_SIZE, repCap_r),
            3, "Range");

    strcat(repCap, ",");
    strcat(repCap, repCap_r);

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_SPECTRUM_EMISSION_MASK_RANGE_MINIMUM_SWEEP_POINTS, minimumSweepPoints),
            4, "Minimum Sweep Points");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function configures the function used to calculate the limit line
/// HIFN  for the n-th power class for overlapping ranges in Multi-SEM
/// HIFN  measurements.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR subBlock/This control selects the sub block in a Multi-SEM measurement.
/// HIPAR range/Selects the range.
/// HIPAR multiLimitCalc/Defines the function used to calculate the limit line for the n-th
/// HIPAR multiLimitCalc/power class for overlapping ranges in Multi-SEM measurements.
ViStatus _VI_FUNC rsspecan_ConfigureSEMRangeMultiLimitCalc(ViSession instrSession,
                                                           ViInt32   subBlock,
                                                           ViInt32   range,
                                                           ViInt32   multiLimitCalc)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_r[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, subBlock, RSSPECAN_VAL_SUB_BLOCK_A, RSSPECAN_VAL_SUB_BLOCK_C),
            2, "Sub Block");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, range, 1, 30),
            3, "Range");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_SPECTRUM_EMISSION_MASK_RANGE_MULTI_LIMIT_CALC,
            "MultiSEMSubBlock", subBlock - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Sub Block");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_SPECTRUM_EMISSION_MASK_RANGE_MULTI_LIMIT_CALC,
            "Range", range - 1, RS_REPCAP_BUF_SIZE, repCap_r),
            3, "Range");

    strcat(repCap, ",");
    strcat(repCap, repCap_r);

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_SPECTRUM_EMISSION_MASK_RANGE_MULTI_LIMIT_CALC, multiLimitCalc),
            4, "Multi-Limit Calc");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function configures the limit setting for the specified range.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.
/// HIPAR subBlock/This control selects the sub block in a Multi-SEM measurement.
/// HIPAR range/Sets the configured range.
/// HIPAR limitCheckType/This control sets the type of limit check for all ranges.
/// HIPAR absoluteLimitStart/This control sets an absolute limit value at the start frequency
/// HIPAR absoluteLimitStart/of the specified range.
/// HIPAR absoluteLimitStop/This control sets an absolute limit value at the stop frequency
/// HIPAR absoluteLimitStop/of the specified range.
/// HIPAR relativeLimitStart/This control sets a relative limit value at the start frequency
/// HIPAR relativeLimitStart/of the specified range.
/// HIPAR relativeLimitStop/This control sets a relative limit value at the stop frequency
/// HIPAR relativeLimitStop/of the specified range.
ViStatus _VI_FUNC rsspecan_ConfigureSEMRangeLimits(ViSession instrSession,
                                                   ViInt32   subBlock,
                                                   ViInt32   range,
                                                   ViInt32   powerClass,
                                                   ViInt32   limitCheckType,
                                                   ViReal64  absoluteLimitStart,
                                                   ViReal64  absoluteLimitStop,
                                                   ViReal64  relativeLimitStart,
                                                   ViReal64  relativeLimitStop)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_power_class[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_r[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, subBlock, RSSPECAN_VAL_SUB_BLOCK_A, RSSPECAN_VAL_SUB_BLOCK_C),
            2, "Sub Block");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, range, 1, 30),
            3, "Range");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, powerClass, 1, 4),
            4, "Power Class");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_SEM_RANGE_LIMIT_CHECK_TYPE,
            "MultiSEMSubBlock", subBlock - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Sub Block");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_SEM_RANGE_START_FREQ_ABS_LIMIT,
            "Range", range - 1, RS_REPCAP_BUF_SIZE, repCap_r),
            3, "Range");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_SEM_RANGE_LIMIT_CHECK_TYPE,
            "PClass", powerClass - 1, RS_REPCAP_BUF_SIZE, repCap_power_class),
            4, "Power Class");

    strcat(repCap, ",");
    strcat(repCap, repCap_r);
    strcat(repCap, ",");
    strcat(repCap, repCap_power_class);

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_SEM_RANGE_LIMIT_CHECK_TYPE, limitCheckType),
            5, "Limit Check Type");

    switch (limitCheckType)
    {
    case RSSPECAN_VAL_SEM_LIM_CHECK_TYPE_ABS:
        {
            viCheckParm(rsspecan_SetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_SEM_RANGE_START_FREQ_ABS_LIMIT, absoluteLimitStart),
                    6, "Absolute Limit Start");

            viCheckParm(rsspecan_SetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_SEM_RANGE_STOP_FREQ_ABS_LIMIT, absoluteLimitStop),
                    7, "Absolute Limit Stop");
            break;
        }
    case RSSPECAN_VAL_SEM_LIM_CHECK_TYPE_REL:
        {
            viCheckParm(rsspecan_SetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_SEM_RANGE_START_FREQ_REL_LIMIT, relativeLimitStart),
                    8, "Relative Limit Start");

            viCheckParm(rsspecan_SetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_SEM_RANGE_STOP_FREQ_REL_LIMIT, relativeLimitStop),
                    9, "Relative Limit Stop");
            break;
        }
    case RSSPECAN_VAL_SEM_LIM_CHECK_TYPE_AND:
    case RSSPECAN_VAL_SEM_LIM_CHECK_TYPE_OR:
        {
            viCheckParm(rsspecan_SetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_SEM_RANGE_START_FREQ_ABS_LIMIT, absoluteLimitStart),
                    6, "Absolute Limit Start");

            viCheckParm(rsspecan_SetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_SEM_RANGE_STOP_FREQ_ABS_LIMIT, absoluteLimitStop),
                    7, "Absolute Limit Stop");

            viCheckParm(rsspecan_SetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_SEM_RANGE_START_FREQ_REL_LIMIT, relativeLimitStart),
                    8, "Relative Limit Start");

            viCheckParm(rsspecan_SetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_SEM_RANGE_STOP_FREQ_REL_LIMIT, relativeLimitStop),
                    9, "Relative Limit Stop");
            break;
        }
    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, limitCheckType), 3, "Limit Check Type");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the limit setting for the specified range.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.
/// HIPAR subBlock/This control selects the sub block in a Multi-SEM measurement.
/// HIPAR range/Selects the range.
/// HIPAR powerClass/This control sets the power class for which the limit is defined.
/// HIPAR startFunction/Defines the function to be used to determine the relative limit line
/// HIPAR startFunction/start value.
/// HIPAR absoluteLimitStart/This control sets an absolute limit value at the start frequency of
/// HIPAR absoluteLimitStart/the specified range to be used in addition to the relative limit if the
/// HIPAR absoluteLimitStart/MAX function is enabled.
/// HIPAR stopFunction/Defines the function to be used to determine the relative limit line
/// HIPAR stopFunction/stop value.
/// HIPAR absoluteLimitStop/This control sets an absolute limit value at the stop frequency of the
/// HIPAR absoluteLimitStop/specified range to be used in addition to the relative limit if the MAX
/// HIPAR absoluteLimitStop/function is enabled.
ViStatus _VI_FUNC rsspecan_ConfigureSEMRangeRelativeLimitsFunctions(
                            ViSession instrSession,
                            ViInt32   subBlock,
                            ViInt32   range,
                            ViInt32   powerClass,
                            ViInt32   startFunction,
                            ViReal64  absoluteLimitStart,
                            ViInt32   stopFunction,
                            ViReal64  absoluteLimitStop)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_power_class[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_r[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, subBlock, RSSPECAN_VAL_SUB_BLOCK_A, RSSPECAN_VAL_SUB_BLOCK_C),
            2, "Sub Block");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, range, 1, 30),
            3, "Range");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, powerClass, 1, 4),
            4, "Power Class");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_SEM_RANGE_START_FREQ_REL_LIMIT_FUNCTION,
            "MultiSEMSubBlock", subBlock - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Sub Block");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_SEM_RANGE_START_FREQ_REL_LIMIT_FUNCTION,
            "Range", range - 1, RS_REPCAP_BUF_SIZE, repCap_r),
            3, "Range");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_SEM_RANGE_START_FREQ_REL_LIMIT_FUNCTION,
            "PClass", powerClass - 1, RS_REPCAP_BUF_SIZE, repCap_power_class),
            4, "Power Class");

    strcat(repCap, ",");
    strcat(repCap, repCap_r);
    strcat(repCap, ",");
    strcat(repCap, repCap_power_class);

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_SEM_RANGE_START_FREQ_REL_LIMIT_FUNCTION, startFunction),
            5, "Start Function");

    if (startFunction == RSSPECAN_VAL_SEM_REL_LIMIT_FUNC_MAX)
    {
        viCheckParm(rsspecan_SetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_SEM_RANGE_START_FREQ_REL_LIMIT_MAXIMUM, absoluteLimitStart),
                6, "Absolute Limit Start");
    }

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_SEM_RANGE_STOP_FREQ_REL_LIMIT_FUNCTION, stopFunction),
            7, "Stop Function");

    if (startFunction == RSSPECAN_VAL_SEM_REL_LIMIT_FUNC_MAX)
    {
        viCheckParm(rsspecan_SetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_SEM_RANGE_STOP_FREQ_REL_LIMIT_MAXIMUM, absoluteLimitStop),
                8, "Absolute Limit Stop");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function returns the number of defined ranges.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR subBlock/This control selects the sub block in a Multi-SEM measurement.
/// HIPAR rangeCount/Returns the number of defined ranges.
ViStatus _VI_FUNC rsspecan_GetSEMRangeCount(ViSession instrSession,
                                            ViInt32   subBlock,
                                            ViInt32   *rangeCount)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, subBlock, RSSPECAN_VAL_SUB_BLOCK_A, RSSPECAN_VAL_SUB_BLOCK_C),
            2, "Sub Block");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_SEM_RANGE_COUNT,
            "MultiSEMSubBlock", subBlock - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Sub Block");

    viCheckParm(rsspecan_GetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_SEM_RANGE_COUNT, rangeCount),
            2, "Range Count");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function inserts a new range before or after the specified
/// HIFN range.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR subBlock/This control selects the sub block in a Multi-SEM measurement.
/// HIPAR range/Selects the range.
/// HIPAR insertRange/This control inserts a new range before or after the specified
/// HIPAR insertRange/range.
ViStatus _VI_FUNC rsspecan_SEMInsertNewRange(ViSession instrSession,
                                             ViInt32   subBlock,
                                             ViInt32   range,
                                             ViInt32   insertRange)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_r[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, subBlock, RSSPECAN_VAL_SUB_BLOCK_A, RSSPECAN_VAL_SUB_BLOCK_C),
            2, "Sub Block");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, range, 1, 30),
            3, "Range");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_SEM_INSERT_NEW_RANGE,
            "MultiSEMSubBlock", subBlock - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Sub Block");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_SEM_INSERT_NEW_RANGE,
            "Range", range - 1, RS_REPCAP_BUF_SIZE, repCap_r),
            3, "Range");

    strcat(repCap, ",");
    strcat(repCap, repCap_r);

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_SEM_INSERT_NEW_RANGE, insertRange),
            3, "Insert Range");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN This function deletes the specified range.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR subBlock/This control selects the sub block in a Multi-SEM measurement.
/// HIPAR range/Selects the range.
ViStatus _VI_FUNC rsspecan_SEMDeleteRange(ViSession instrSession,
                                          ViInt32   subBlock,
                                          ViInt32   range)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_r[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, subBlock, RSSPECAN_VAL_SUB_BLOCK_A, RSSPECAN_VAL_SUB_BLOCK_C),
            2, "Sub Block");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, range, 1, 30),
            3, "Range");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_SEM_DELETE_RANGE,
            "MultiSEMSubBlock", subBlock - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Sub Block");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_SEM_DELETE_RANGE,
            "Range", range - 1, RS_REPCAP_BUF_SIZE, repCap_r),
            3, "Range");

    strcat(repCap, ",");
    strcat(repCap, repCap_r);

    checkErr(rsspecan_SetAttributeViString(instrSession, repCap, RSSPECAN_ATTR_SEM_DELETE_RANGE, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/*===========================================================================*
 *------------------- SEM -> Configuration-> Fast SEM -----------------------*
 *===========================================================================*/
/// HIFN This function activates Fast SEM mode to accelerate spectrum
/// HIFN emission mask measurements.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR subBlock/This control selects the sub block in a Multi-SEM measurement.
/// HIPAR fastSEM/This control activates Fast SEM mode to accelerate spectrum
/// HIPAR fastSEM/emission mask measurements.
ViStatus _VI_FUNC rsspecan_ConfigureFastSEM(ViSession instrSession,
                                            ViInt32   subBlock,
                                            ViBoolean fastSEM)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, subBlock, RSSPECAN_VAL_SUB_BLOCK_A, RSSPECAN_VAL_SUB_BLOCK_C),
            2, "Sub Block");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_SEM_FAST_SEM,
            "MultiSEMSubBlock", subBlock - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Sub Block");

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, repCap, RSSPECAN_ATTR_SEM_FAST_SEM, fastSEM),
            3, "Fast SEM");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/*===========================================================================*
 *------------------- SEM -> Configuration-> SEM Power Class -----------------*
 *===========================================================================*/
/// HIFN This function configures the Spectrum EM Mask Measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.
/// HIPAR subBlock/This control selects the sub block in a Multi-SEM measurement.
/// HIPAR limitLineType/This control defines wether the power class is determined
/// HIPAR limitLineType/automatically or manually.
/// HIPAR limitLineValue/This control sets the Limit Line Value.
ViStatus _VI_FUNC rsspecan_ConfigureSEM(ViSession instrSession,
                                        ViInt32   subBlock,
                                        ViInt32   limitLineType,
                                        ViReal64  limitLineValue,
                                        ViReal64  reserved)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
             reserved;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, subBlock, RSSPECAN_VAL_SUB_BLOCK_A, RSSPECAN_VAL_SUB_BLOCK_C),
            2, "Sub Block");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_ESP_MODE,
            "MultiSEMSubBlock", subBlock - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Sub Block");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_ESP_MODE, limitLineType),
            3, "Limit Line Type");

    if (limitLineType == RSSPECAN_VAL_ESP_MODE_MAN)
    {
        viCheckParm(rsspecan_SetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_ESP_MAN_VAL, limitLineValue),
                4, "Limit Line Value");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function sets the power classes used in the spectrum emission
/// HIFN mask measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR subBlock/This control selects the sub block in a Multi-SEM measurement.
/// HIPAR powerClassNumber/Document parameter here.
/// HIPAR usedPowerClass/Document parameter here.
ViStatus _VI_FUNC rsspecan_ConfigureSEMUsedPowerClasses(ViSession instrSession,
                                                        ViInt32   subBlock,
                                                        ViInt32   powerClassNumber,
                                                        ViInt32   usedPowerClass)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_all[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_p[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, subBlock, RSSPECAN_VAL_SUB_BLOCK_A, RSSPECAN_VAL_SUB_BLOCK_C),
            2, "Sub Block");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_SEM_FAST_SEM,
            "MultiSEMSubBlock", subBlock - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Sub Block");

    switch (usedPowerClass)
    {
    case RSSPECAN_VAL_ALL:
        snprintf(repCap_all, RS_REPCAP_BUF_SIZE, "%s,PC1", repCap);
        checkErr(rsspecan_SetAttributeViBoolean(instrSession, repCap_all, RSSPECAN_ATTR_SEM_POWER_CLASS_USED, VI_FALSE));
        snprintf(repCap_all, RS_REPCAP_BUF_SIZE, "%s,PC2", repCap);
        checkErr(rsspecan_SetAttributeViBoolean(instrSession, repCap_all, RSSPECAN_ATTR_SEM_POWER_CLASS_USED, VI_FALSE));
        snprintf(repCap_all, RS_REPCAP_BUF_SIZE, "%s,PC3", repCap);
        checkErr(rsspecan_SetAttributeViBoolean(instrSession, repCap_all, RSSPECAN_ATTR_SEM_POWER_CLASS_USED, VI_FALSE));
        snprintf(repCap_all, RS_REPCAP_BUF_SIZE, "%s,PC4", repCap);
        checkErr(rsspecan_SetAttributeViBoolean(instrSession, repCap_all, RSSPECAN_ATTR_SEM_POWER_CLASS_USED, VI_FALSE));
        break;
    case RSSPECAN_VAL_OFF:
    case RSSPECAN_VAL_ON:
        viCheckParm(RsCore_InvalidViInt32Range(instrSession, powerClassNumber, 1, 4),
                3, "Power Class Number");
        viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_SEM_POWER_CLASS_USED,
                "PClass", powerClassNumber - 1, RS_REPCAP_BUF_SIZE, repCap_p),
                3, "Power Class Number");

        strcat(repCap, ",");
        strcat(repCap, repCap_p);
        viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, repCap,
                                                    RSSPECAN_ATTR_SEM_POWER_CLASS_USED,
                                                    (ViBoolean) usedPowerClass), 4, "Used Power Class");
        break;
    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, usedPowerClass), 4, "Used Power Class");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function sets the number of power classes to be defined.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.
/// HIPAR subBlock/This control selects the sub block in a Multi-SEM measurement.
/// HIPAR count/This control sets the number of power classes to be defined.
ViStatus _VI_FUNC rsspecan_ConfigureSEMPowerClassCount(ViSession instrSession,
                                                       ViInt32   subBlock,
                                                       ViInt32   count)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, subBlock, RSSPECAN_VAL_SUB_BLOCK_A, RSSPECAN_VAL_SUB_BLOCK_C),
            2, "Sub Block");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_SEM_POWER_CLASS_COUNT,
            "MultiSEMSubBlock", subBlock - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Sub Block");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_SEM_POWER_CLASS_COUNT, count),
            3, "Count");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function configures the power class settings of the Spectrum
/// HIFN Emission Mask measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.
/// HIPAR subBlock/This control selects the sub block in a Multi-SEM measurement.
/// HIPAR powerClassNumber/This control selects the number of power class to be modified.
/// HIPAR limitState/This control defines which limits are evaluated in the
/// HIPAR limitState/measurement.
/// HIPAR lowerLevelLimit/This control sets the minimum lower level limit for one power
/// HIPAR lowerLevelLimit/class.
/// HIPAR upperLevelLimit/This control sets the upper limit level for one power class.
ViStatus _VI_FUNC rsspecan_ConfigureSEMPowerClassLimits(ViSession instrSession,
                                                        ViInt32   subBlock,
                                                        ViInt32   powerClassNumber,
                                                        ViInt32   limitState,
                                                        ViReal64  lowerLevelLimit,
                                                        ViReal64  upperLevelLimit)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_p[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, subBlock, RSSPECAN_VAL_SUB_BLOCK_A, RSSPECAN_VAL_SUB_BLOCK_C),
            2, "Sub Block");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, powerClassNumber, 1, 4),
            3, "Power Class Number");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_SEM_POWER_CLASS_LIMIT_STATE,
            "MultiSEMSubBlock", subBlock - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Sub Block");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_SEM_POWER_CLASS_LIMIT_STATE,
            "PClass", powerClassNumber - 1, RS_REPCAP_BUF_SIZE, repCap_p),
            3, "Power Class Number");

    strcat(repCap, ",");
    strcat(repCap, repCap_p);

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_SEM_POWER_CLASS_LIMIT_STATE, limitState),
            4, "Limit State");

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_SEM_POWER_CLASS_LOWER_LEVEL_LIMIT, lowerLevelLimit),
            5, "Lower Level Limit");

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_SEM_POWER_CLASS_UPPER_LEVEL_LIMIT, upperLevelLimit),
            6, "Upper Level Limit");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function sets up to 4 power classes in one step.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.
/// HIPAR subBlock/This control selects the sub block in a Multi-SEM measurement.
/// HIPAR numberOfLimits/This control sets the number of power class limits.
/// HIPAR limits/This control sets up to 4 power classes in one step.
ViStatus _VI_FUNC rsspecan_ConfigureSEMPowerClassAllLimits(ViSession instrSession,
                                                           ViInt32   subBlock,
                                                           ViInt32   numberOfLimits,
                                                           ViReal64  limits[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, subBlock, RSSPECAN_VAL_SUB_BLOCK_A, RSSPECAN_VAL_SUB_BLOCK_C),
            2, "Sub Block");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CALC:LIM:ESP%ld:LIM ", subBlock);
    checkErr(RsCore_WriteAsciiViReal64Array(instrSession, cmd, limits, numberOfLimits));

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function returns up to 4 power classes in one step.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.
/// HIPAR subBlock/This control selects the sub block in a Multi-SEM measurement.
/// HIPAR limits/This control returns up to 4 power classes in one step.
ViStatus _VI_FUNC rsspecan_QuerySEMPowerClassAllLimits(ViSession instrSession,
                                                       ViInt32   subBlock,
                                                       ViReal64  limits[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, subBlock, RSSPECAN_VAL_SUB_BLOCK_A, RSSPECAN_VAL_SUB_BLOCK_C),
            2, "Sub Block");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CALC:LIM:ESP%ld:LIM?", subBlock);
    checkErr(RsCore_QueryFloatArrayToUserBuffer(instrSession, cmd, 32001, limits, NULL));

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/*===========================================================================*
 *------------------- SEM->Configuration->MSR -------------------------------*
 *===========================================================================*/
/// HIFN  This function configures multi-standard radio (MSR) measurements,
/// HIFN  which allow you to perform SEM tests on multiple carriers using
/// HIFN  different digital standards.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR subBlock/This control selects the sub block in a Multi-SEM measurement.
/// HIPAR bandCategory/This control defines the band category for MSR measurements, i.e. the
/// HIPAR bandCategory/combination of available carriers to measure.
/// HIPAR baseStationRFBandwidth/This control defines the RF bandwidth of the base station for MSR
/// HIPAR baseStationRFBandwidth/measurements.
/// HIPAR gsm_edgePresent/This control defines whether a GSM/Edge carrier is located at the edge
/// HIPAR gsm_edgePresent/of the specified RF bandwidth. In this case, the specification demands
/// HIPAR gsm_edgePresent/specific limits for the SEM ranges.
/// HIPAR lteFDD1_4MHz_3MHzPresent/This control defines whether an LTE FDD 1.4 MHz or 3 MHz carrier is
/// HIPAR lteFDD1_4MHz_3MHzPresent/located at the edge  of the specified RF bandwidth. In this case, the
/// HIPAR lteFDD1_4MHz_3MHzPresent/specification demands specific limits for the SEM ranges.
ViStatus _VI_FUNC rsspecan_ConfigureSEMMSRSettings(ViSession instrSession,
                                                   ViInt32   subBlock,
                                                   ViInt32   bandCategory,
                                                   ViReal64  baseStationRFBandwidth,
                                                   ViBoolean GSMEDGEPresent,
                                                   ViBoolean LTEFDD14MHz3MHzPresent)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, subBlock, RSSPECAN_VAL_SUB_BLOCK_A, RSSPECAN_VAL_SUB_BLOCK_C),
            2, "Sub Block");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_SEM_MSR_BAND_CATEGORY,
            "MultiSEMSubBlock", subBlock - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Sub Block");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_SEM_MSR_BAND_CATEGORY, bandCategory),
            3, "Band Category");

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_SEM_MSR_RF_BANDWIDTH, baseStationRFBandwidth),
            4, "Base Station RF Bandwidth");

    if (bandCategory == RSSPECAN_VAL_SEM_MSR_BAND_CATEGORY_2)
    {
        viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, repCap, RSSPECAN_ATTR_SEM_MSR_GSM_PRESENT, GSMEDGEPresent),
                5, "Gsm Edge Present");

        viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, repCap, RSSPECAN_ATTR_SEM_MSR_LTE_PRESENT, LTEFDD14MHz3MHzPresent),
                6, "LTE FDD 1.4MHz 3MHz Present");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures additional settings of multi-standard radio
/// HIFN  (MSR) measurements, which allow you to perform SEM tests on multiple
/// HIFN  carriers using different digital standards.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR subBlock/This control selects the sub block in a Multi-SEM measurement.
/// HIPAR baseStationClass/Defines the class of the base station according to its sending range.
/// HIPAR bsMaxOutputPower/Defines the maximum output power of the base station. This setting is
/// HIPAR bsMaxOutputPower/only available for base stations with a medium range base station class
/// HIPAR bsMaxOutputPower/(RSSPECAN_ATTR_SPECTRUM_EMISSION_MASK_MSR_BASE_STATION_CLASS)
/// HIPAR bands/Defines the frequency range of the bands used by the base station.
/// HIPAR bands/This setting is only available for Band Category 1 or 3
/// HIPAR bands/(RSSPECAN_ATTR_SEM_MSR_BAND_CATEGORY).
ViStatus _VI_FUNC rsspecan_ConfigureSEMMSRAdditionalSettings(ViSession instrSession,
                                                             ViInt32   subBlock,
                                                             ViInt32   baseStationClass,
                                                             ViReal64  bsMaxOutputPower,
                                                             ViInt32   bands)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, subBlock, RSSPECAN_VAL_SUB_BLOCK_A, RSSPECAN_VAL_SUB_BLOCK_C),
            2, "Sub Block");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_SPECTRUM_EMISSION_MASK_MSR_BASE_STATION_CLASS,
            "MultiSEMSubBlock", subBlock - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Sub Block");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_SPECTRUM_EMISSION_MASK_MSR_BASE_STATION_CLASS, baseStationClass),
            3, "Base Station Class");

    if (baseStationClass == RSSPECAN_VAL_SEM_MSR_BS_CLASS_MEDIUM)
    {
        viCheckParm(rsspecan_SetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_SPECTRUM_EMISSION_MASK_MSR_BASE_STATION_MAXIMUM_OUTPUT_POWER, bsMaxOutputPower),
                4, "BS Max Output Power");
    }
    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_SPECTRUM_EMISSION_MASK_MSR_BANDS, bands),
            5, "Bands");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the SEM sweep list according to the MSR
/// HIFN  settings defined by function rsspecan_ConfigureSEMMSRSettings.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR subBlock/This control selects the sub block in a Multi-SEM measurement.
ViStatus _VI_FUNC rsspecan_ApplySEMMSRSettings(ViSession instrSession,
                                               ViInt32   subBlock)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, subBlock, RSSPECAN_VAL_SUB_BLOCK_A, RSSPECAN_VAL_SUB_BLOCK_C),
            2, "Sub Block");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_SEM_MSR_APPLY,
            "MultiSEMSubBlock", subBlock - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Sub Block");

    checkErr(rsspecan_SetAttributeViString(instrSession, repCap, RSSPECAN_ATTR_SEM_MSR_APPLY, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/*===========================================================================*
 *------------------- SEM->Configuration->Standard Files --------------------*
 *===========================================================================*/
/// HIFN Loads the selected measurement settings file
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.
/// HIPAR subBlock/This control selects the sub block in a Multi-SEM measurement.
/// HIPAR presetStandard/This control selects the specified XML file.
ViStatus _VI_FUNC rsspecan_ConfigureSEMPresetStandard(ViSession instrSession,
                                                      ViInt32   subBlock,
                                                      ViString  presetStandard)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, subBlock, RSSPECAN_VAL_SUB_BLOCK_A, RSSPECAN_VAL_SUB_BLOCK_C),
            2, "Sub Block");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_SEM_PRESET_STANDARD,
            "MultiSEMSubBlock", subBlock - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Sub Block");

    viCheckParm(rsspecan_SetAttributeViString(instrSession, repCap, RSSPECAN_ATTR_SEM_PRESET_STANDARD, presetStandard),
            3, "Preset Standard");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function stores the current settings as presettings in the
/// HIFN specified XML file under C:\r_s\instr\sem_backup.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.
/// HIPAR subBlock/This control selects the sub block in a Multi-SEM measurement.
/// HIPAR settingsFileName/This control specifies XML file that stores the current settings
/// HIPAR settingsFileName/as presettings.
ViStatus _VI_FUNC rsspecan_StoreSEMPresetSettings(ViSession instrSession,
                                                  ViInt32   subBlock,
                                                  ViString  settingsFileName)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, subBlock, RSSPECAN_VAL_SUB_BLOCK_A, RSSPECAN_VAL_SUB_BLOCK_C),
            2, "Sub Block");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_SEM_STORE_PRESET,
            "MultiSEMSubBlock", subBlock - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Sub Block");

    viCheckParm(rsspecan_SetAttributeViString(instrSession, repCap, RSSPECAN_ATTR_SEM_STORE_PRESET, settingsFileName),
            3, "Settings File Name");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN Restores the default configurations of predefined SEM standards
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.
/// HIPAR subBlock/This control selects the sub block in a Multi-SEM measurement.
ViStatus _VI_FUNC rsspecan_RestoreSEMStandardFiles(ViSession instrSession,
                                                   ViInt32   subBlock)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, subBlock, RSSPECAN_VAL_SUB_BLOCK_A, RSSPECAN_VAL_SUB_BLOCK_C),
            2, "Sub Block");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_SEM_RESTORE_STANDARD,
            "MultiSEMSubBlock", subBlock - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Sub Block");

    checkErr(rsspecan_SetAttributeViString(instrSession, repCap, RSSPECAN_ATTR_SEM_RESTORE_STANDARD, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN Restores the standards predefined by Rohde & Schwarz available
/// HIFN at the time of delivery.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.
ViStatus _VI_FUNC rsspecan_RestoreSEMStandardLines(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_ESP_RESTORE, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/*===========================================================================*
 *------------------- SEM->Configuration->List Evaluation -------------------*
 *===========================================================================*/
/// HIFN This function activates or deactivates spectrum emission mask
/// HIFN measurement LIST EVALUATION.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.
/// HIPAR state/Switches on or off the spectrum emission mask measurement.
ViStatus _VI_FUNC rsspecan_SEMListEvaluation(ViSession instrSession,
                                             ViBoolean state)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_SEM_LIST_EVAL_STATE, state),
            2, "State");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function configures the peak search for spectrum emission
/// HIFN mask measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.
/// HIPAR showPeaks/If activated, all peaks that have been detected during an active
/// HIPAR showPeaks/SEM measurement are marked with blue squares in the Spectrum
/// HIPAR showPeaks/diagram.
/// HIPAR margin/Defines an overall margin which is subtracted from the limit
/// HIPAR margin/line to make the peak search.
ViStatus _VI_FUNC rsspecan_ConfigureSEMListEvaluation(ViSession instrSession,
                                                      ViBoolean showPeaks,
                                                      ViReal64  margin,
                                                      ViInt32   reserved)
{
    ViStatus error = VI_SUCCESS;
             reserved;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_SEM_MARK_ALL_PEAKS, showPeaks),
            2, "Show Peaks");

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_SEM_PEAK_SEARCH_MARGIN, margin),
            3, "Margin");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN Exports the Result Summary of the SEM measurement to an ASCII file for evaluation
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init
/// HIPAR instrSession/or rsspecan_InitWithOptions function.
/// HIPAR fileName/This control specifies the file name.
ViStatus _VI_FUNC rsspecan_SEMSaveListToFile(ViSession instrSession,
                                             ViString  fileName)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_SEM_LIST_FILE_EXPORT, fileName),
            2, "File Name");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/*===========================================================================*
 *------------------- SEM->Results ------------------------------------------*
 *===========================================================================*/
/// HIFN This function returns the peak list evaluation. One peak per
/// HIFN range is returned for SEM.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.
/// HIPAR noOfValues/Document parameter here.
/// HIPAR rangeNumber/Document parameter here.
/// HIPAR startFrequency/Document parameter here.
/// HIPAR stopFrequency/Document parameter here.
/// HIPAR resolutionBandwidth/Document parameter here.
/// HIPAR frequency/Document parameter here.
/// HIPAR powerAbs/Document parameter here.
/// HIPAR powerRel/Document parameter here.
/// HIPAR delta/Document parameter here.
/// HIPAR limitCheck/Document parameter here.
/// HIPAR reserved1/Document parameter here.
/// HIPAR reserved2/Document parameter here.
/// HIPAR returnedValues/Document parameter here.
ViStatus _VI_FUNC rsspecan_ReadSEMListEvaluationResults(ViSession instrSession,
                                                        ViInt32   noOfValues,
                                                        ViReal64  rangeNumber[],
                                                        ViReal64  startFrequency[],
                                                        ViReal64  stopFrequency[],
                                                        ViReal64  resolutionBandwidth[],
                                                        ViReal64  frequency[],
                                                        ViReal64  powerAbs[],
                                                        ViReal64  powerRel[],
                                                        ViReal64  delta[],
                                                        ViReal64  limitCheck[],
                                                        ViReal64  reserved1[],
                                                        ViReal64  reserved2[],
                                                        ViInt32   *returnedValues)
{
    ViStatus error = VI_SUCCESS;
    ViReal64 *pArray = NULL;
    ViInt32  count = 0;
    ViInt32  retCnt;
    ViInt32  i;
    ViInt32  values = noOfValues * 11;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidNullPointer(instrSession, rangeNumber),
        2, "Range number");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, startFrequency),
        3, "Start Frequency");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, stopFrequency),
        4, "Stop Frequency");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, resolutionBandwidth),
        5, "Resolution Bandwidth");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, frequency),
        6, "Frequency");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, powerAbs),
        7, "Power Abs");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, powerRel),
        8, "Power Rel");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, delta),
        9, "Delta");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, limitCheck),
        10, "Limit Check");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, reserved1),
        11, "Reserved 1");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, reserved2),
        12, "Reserved 2");

    checkErr(RsCore_QueryFloatArray(instrSession, "TRAC:DATA? LIST", &pArray, &retCnt));
    values = (retCnt > values) ? values : retCnt;
    i = 0;
    while (count < values)
    {
        rangeNumber[i] = pArray[count++];
        startFrequency[i] = pArray[count++];
        stopFrequency[i] = pArray[count++];
        resolutionBandwidth[i] = pArray[count++];
        frequency[i] = pArray[count++];
        powerAbs[i] = pArray[count++];
        powerRel[i] = pArray[count++];
        delta[i] = pArray[count++];
        limitCheck[i] = pArray[count++];
        reserved1[i] = pArray[count++];
        reserved2[i] = pArray[count++];
        i++;
    }
    if (returnedValues)
        *returnedValues = retCnt / 11;
    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    if (pArray) free(pArray);
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/*===========================================================================*
 *------------------- Spurious Emissions ------------------------------------*
 *===========================================================================*/
/// HIFN This function switches on or off the spurious emissions
/// HIFN measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init
/// HIPAR instrSession/or rsspecan_InitWithOptions function.
/// HIPAR state/Switches on or off the spurious emissions measurement.
ViStatus _VI_FUNC rsspecan_ConfigureSEState(ViSession instrSession,
                                            ViBoolean state)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    if (state)
    {
        checkErr(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_SWEEP_MODE, RSSPECAN_VAL_SWE_MODE_LIST));
    }
    else
        checkErr(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_SWEEP_MODE, RSSPECAN_VAL_SWE_MODE_AUTO));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function searches the peaks in spurious emissions data.
/// HIRET Document return value here.
/// HIRET You may use multiple lines for documentation.
/// HIPAR instrSession/Document parameter here.
ViStatus _VI_FUNC rsspecan_SESearchPeak(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_SE_PEAK_SEARCH, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function stores the peak list of selected trace in ASCII
/// HIFN format to a file.
/// HIRET Document return value here.
/// HIRET You may use multiple lines for documentation.
/// HIPAR instrSession/Document parameter here.
/// HIPAR fileName/Specifies full path of the file, where spurios emissions data
/// HIPAR fileName/will be stored.
ViStatus _VI_FUNC rsspecan_StoreSEToFile(ViSession instrSession,
                                         ViString  fileName)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_SE_STORE_FILE, fileName),
            3, "File Name");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/*===========================================================================*
 *------------------- Spurious Emissions -> Configuration -> Sweep List -----*
 *===========================================================================*/
/// HIFN This function configures the start and stop frequency for the
/// HIFN specified range.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR range/Sets the configured range.
/// HIPAR startFrequency/Sets the start frequency of a range in the spurious measurement.
/// HIPAR stopFrequency/Sets the stop frequency of a range in the spurious measurement.
ViStatus _VI_FUNC rsspecan_ConfigureSESweepListStartStop(ViSession instrSession,
                                                         ViInt32   range,
                                                         ViReal64  startFrequency,
                                                         ViReal64  stopFrequency)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_SE_LIST_RANG_START,
            "Range", range - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Range");

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_SE_LIST_RANG_START, startFrequency),
            3, "Start Frequency");

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_SE_LIST_RANG_STOP, stopFrequency),
            4, "Stop Frequency");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function configures the filter type for the specified range.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR range/Sets the configured range.
/// HIPAR filter/This control sets the filter in the spurious measurement.
ViStatus _VI_FUNC rsspecan_ConfigureSESweepListFilterType(ViSession instrSession,
                                                          ViInt32   range,
                                                          ViInt32   filter)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_SE_LIST_RANG_FILT_TYPE,
            "Range", range - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Range");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_SE_LIST_RANG_FILT_TYPE, filter),
            3, "Filter");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function configures the resolution and video bandwidth
/// HIFN  values for the specified range.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR range/Sets the configured range.
/// HIPAR resolutionBandwidth/Sets the the resolution bandwidth.
/// HIPAR videoBandwidth/Sets the the video bandwidth.
ViStatus _VI_FUNC rsspecan_ConfigureSESweepListBandwidth(ViSession instrSession,
                                                         ViInt32   range,
                                                         ViReal64  resolutionBandwidth,
                                                         ViReal64  videoBandwidth)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_SE_LIST_RANG_BAND,
            "Range", range - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Range");

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_SE_LIST_RANG_BAND, resolutionBandwidth),
            3, "Resolution Bandwidth");

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_SE_LIST_RANG_BAND_VID, videoBandwidth),
            4, "Video Bandwidth");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function configures the sweep time values for the
/// HIFN specified range.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR range/Sets the configured range.
/// HIPAR sweepTimeAuto/Sets the automatic coupling of the sweep duration on the
/// HIPAR sweepTimeAuto/frequency span and the bandwidth settings in a spurious
/// HIPAR sweepTimeAuto/measurement range.
/// HIPAR sweepTime/Sets the sweep duration of a range in the spurious measurement.
ViStatus _VI_FUNC rsspecan_ConfigureSESweepSweepListTime(ViSession instrSession,
                                                         ViInt32   range,
                                                         ViBoolean sweepTimeAuto,
                                                         ViReal64  sweepTime)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_SE_LIST_RANG_SWE_TIME_AUTO,
            "Range", range - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Range");

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, repCap, RSSPECAN_ATTR_SE_LIST_RANG_SWE_TIME_AUTO, sweepTimeAuto),
            3, "Sweep Time Auto");
    if (sweepTimeAuto == VI_FALSE)
    {
        viCheckParm(rsspecan_SetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_SE_LIST_RANG_SWE_TIME, sweepTime),
                4, "Sweep Time");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function configures the detector value for the specified
/// HIFN range.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR range/Sets the configured range.
/// HIPAR detector/This control sets the detector in the spurious measurement.
ViStatus _VI_FUNC rsspecan_ConfigureSESweepListDetector(ViSession instrSession,
                                                        ViInt32   range,
                                                        ViInt32   detector)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_SE_LIST_RANG_DET,
            "Range", range - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Range");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_SE_LIST_RANG_DET, detector),
            3, "Detector");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function configures the reference level for the specified
/// HIFN  range.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR range/Sets the configured range.
/// HIPAR refLevelValue/Specifies the reference level of a range in the spurious
/// HIPAR refLevelValue/measurement.
ViStatus _VI_FUNC rsspecan_ConfigureSESweepListReferenceLevel(
                            ViSession instrSession,
                            ViInt32   range,
                            ViReal64  refLevelValue)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_SE_LIST_RANG_RLEV,
            "Range", range - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Range");

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_SE_LIST_RANG_RLEV, refLevelValue),
            3, "Ref Level Value");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function configures the RF attenuation settings for the
/// HIFN  specified range.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR range/Sets the configured range.
/// HIPAR attenuatorAuto/Sets the autoranging of a range in the spurious measurement on
/// HIPAR attenuatorAuto/or off.
/// HIPAR manualAttenuatorValuedB/This control sets the RF attenuation of a range.
ViStatus _VI_FUNC rsspecan_ConfigureSESweepListRFAttenuation(ViSession instrSession,
                                                             ViInt32   range,
                                                             ViBoolean attenuatorAuto,
                                                             ViReal64  manualAttenuatorValuedB)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_SE_LIST_RANG_INP_ATT_AUTO,
            "Range", range - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Range");

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, repCap, RSSPECAN_ATTR_SE_LIST_RANG_INP_ATT_AUTO, attenuatorAuto),
            3, "Attenuator Auto");
    if (attenuatorAuto == VI_FALSE)
    {
        viCheckParm(rsspecan_SetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_SE_LIST_RANG_INP_ATT, manualAttenuatorValuedB),
                4, "Manual Attenuator Value dB");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function configures state of pre-amplifier for selected
/// HIFN sweep range.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR range/Sets the configured range.
/// HIPAR preamplifier/Switches the preamplifier of a range on and off.
ViStatus _VI_FUNC rsspecan_ConfigureSESweepListPreamplifier(ViSession instrSession,
                                                            ViInt32   range,
                                                            ViBoolean preamplifier)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentOptions(instrSession, "B22|B24|B25"));
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_SE_LIST_RANG_INP_GAIN_STAT,
            "Range", range - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Range");

    checkErr(rsspecan_SetAttributeViBoolean(instrSession, repCap, RSSPECAN_ATTR_SE_LIST_RANG_INP_GAIN_STAT, preamplifier));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN This function configures level of pre-amplifier for selected sweep range.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init.
/// HIPAR range/Sets the configured range.
/// HIPAR preamplificationLevel/Selects the preamplification level.
ViStatus _VI_FUNC rsspecan_ConfigureSESweepListPreamplifierLevel(
                            ViSession instrSession,
                            ViInt32   range,
                            ViInt32   preamplificationLevel)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentOptions(instrSession, "B24"));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_SE_LIST_RANG_INP_GAIN_LEVEL,
            "Range", range - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Range");

    checkErr(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_SE_LIST_RANG_INP_GAIN_LEVEL, preamplificationLevel));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function configures the sweep settings for the specified
/// HIFN  range.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR range/Sets the configured range.
/// HIPAR sweepPoints/Sets the number of sweep points of a range in the spurious
/// HIPAR sweepPoints/measurement.
/// HIPAR stopAfterSweep/Determines whether a sweep is stopped in the spurious
/// HIPAR stopAfterSweep/measurement when a range is changed.
ViStatus _VI_FUNC rsspecan_ConfigureSESweepListSweep(ViSession instrSession,
                                                     ViInt32   range,
                                                     ViInt32   sweepPoints,
                                                     ViBoolean stopAfterSweep)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_SE_LIST_RANG_POINTS,
            "Range", range - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Range");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_SE_LIST_RANG_POINTS, sweepPoints),
            3, "Sweep Points");

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, repCap, RSSPECAN_ATTR_SE_LIST_RANG_BREAK, stopAfterSweep),
            4, "Stop After Sweep");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures a transducer for the specified range.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR range/Sets the configured range.
/// HIPAR transducerFactor/Sets the transducer factor for a range in the spurious
/// HIPAR transducerFactor/measurement.
ViStatus _VI_FUNC rsspecan_ConfigureSESweepListTransducerFactor(
                            ViSession instrSession,
                            ViInt32   range,
                            ViString  transducerFactor)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_SE_LIST_RANG_START,
            "Range", range - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Range");

    if (strlen(transducerFactor) > 0)
    viCheckParm(rsspecan_SetAttributeViString(instrSession, repCap, RSSPECAN_ATTR_SE_LIST_RANG_TRAN, transducerFactor),
            3, "Transducer Factor");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function configures the limits for the specified range.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR range/Sets the configured range.
/// HIPAR limitChecking/Activates/deactivates limit checking of spurious emission
/// HIPAR limitChecking/measurement.
/// HIPAR absoluteLimitStart/This control sets an absolute limit value at the start frequency
/// HIPAR absoluteLimitStart/of the specified range. Different from manual operation, this
/// HIPAR absoluteLimitStart/setting is independently of the defined limit check type.
/// HIPAR absoluteLimitStop/This control sets an absolute limit value at the stop frequency
/// HIPAR absoluteLimitStop/of the specified range. Different from manual operation, this
/// HIPAR absoluteLimitStop/setting is independently of the defined limit check type.
ViStatus _VI_FUNC rsspecan_ConfigureSESweepListLimit(ViSession instrSession,
                                                     ViInt32   range,
                                                     ViBoolean limitChecking,
                                                     ViReal64  absoluteLimitStart,
                                                     ViReal64  absoluteLimitStop)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_SE_LIST_RANG_LIMIT_STATE,
            "Range", range - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Range");

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, repCap, RSSPECAN_ATTR_SE_LIST_RANG_LIMIT_STATE, limitChecking),
            3, "Limit Checking");
    if (limitChecking == VI_TRUE)
    {
        viCheckParm(rsspecan_SetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_SE_LIST_RANG_LIMIT_START, absoluteLimitStart),
                4, "Absolute Limit Start");

        viCheckParm(rsspecan_SetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_SE_LIST_RANG_LIMIT_STOP, absoluteLimitStop),
                5, "Absolute Limit Stop");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the x-axis range for the spurious emission measurement from the
/// HIFN  start frequency of the first sweep range to the stop frequency of the
/// HIFN  last sweep range.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsspecan_AdjustSpuriousEmissionsXAxisToRangeDefinitions(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsspecan_SetAttributeViString(instrSession, NULL, RSSPECAN_ATTR_SE_ADJUST_X_AXIS_TO_RANGE_DEFINITIONS, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function deletes a range.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR range/Sets the configured range.
ViStatus _VI_FUNC rsspecan_SEDeleteRange(ViSession instrSession,
                                         ViInt32   range)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_SE_LIST_RANG_DEL,
            "Range", range - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Range");

    checkErr(rsspecan_SetAttributeViString(instrSession, repCap, RSSPECAN_ATTR_SE_LIST_RANG_DEL, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function returns the number of defined ranges of the spurious
/// HIFN measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR numberOfRanges/Returns indicates the number of defined ranges of the spurious
/// HIPAR numberOfRanges/measurement.
ViStatus _VI_FUNC rsspecan_GetSENumberOfListRanges(ViSession instrSession,
                                                   ViInt32   *numberOfRanges)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_GetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_SE_LIST_RANG_COUNT, numberOfRanges),
            2, "Number Of Ranges");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/*===========================================================================*
 *------------------- Spurious Emissions -> Configuration -> List Eval ------*
 *===========================================================================*/
/// HIFN This function activates or deactivates spurious emission
/// HIFN measurement LIST EVALUATION.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.
/// HIPAR state/Switches on or off the spurious emissions measurement.
ViStatus _VI_FUNC rsspecan_SEListEvaluation(ViSession instrSession,
                                            ViBoolean state)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_SE_LIST_EVAL_STATE, state),
            2, "State");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function configures the peak search for spurious emission
/// HIFN measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init
/// HIPAR instrSession/or rsspecan_InitWithOptions function.
/// HIPAR showPeaks/If activated, all peaks that have been detected during an active
/// HIPAR showPeaks/list evaluation are marked with blue squares in the diagram.
/// HIPAR margin/Defines an overall margin which is subtracted from the limit
/// HIPAR margin/line to make the peak search.
/// HIPAR peaksPerRange/Defines how many peaks are searched for within one range.
ViStatus _VI_FUNC rsspecan_ConfigureSEListEvaluation(ViSession instrSession,
                                                     ViBoolean showPeaks,
                                                     ViReal64  margin,
                                                     ViInt32   peaksPerRange)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_SE_MARK_ALL_PEAKS, showPeaks),
            2, "Show Peaks");

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_SE_PEAK_SEARCH_MARGIN, margin),
            3, "Margin");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_SE_PEAK_SEARCH_PEAKS, peaksPerRange),
            4, "Peaks Per Range");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This control configures how detailed the list in the Result Summary is.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR listDetail/This control configures how detailed the list in the Result Summary
/// HIPAR listDetail/is.
ViStatus _VI_FUNC rsspecan_ConfigureSEDetail(ViSession instrSession,
                                             ViBoolean listDetail)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_SE_DETAILS, listDetail),
            2, "List Detail");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN Exports the Result Summary of the SE measurement to an ASCII file for evaluation
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init
/// HIPAR instrSession/or rsspecan_InitWithOptions function.
/// HIPAR fileName/This control specifies the file name.
ViStatus _VI_FUNC rsspecan_SESaveListToFile(ViSession instrSession,
                                            ViString  fileName)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_SEM_LIST_FILE_EXPORT, fileName),
            2, "File Name");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/*===========================================================================*
 *------------------- Spurious Emissions -> Results -------------------------*
 *===========================================================================*/
/// HIFN This function returns the spurious emissions peak list.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.
/// HIPAR noOfValues/This control sets the number of values returned in arrays.
/// HIPAR frequency/Returns an array of frequency list in Hz.
/// HIPAR level/Returns an array of level list in dBm.
/// HIPAR deltaLimit/Returns an array of delta to limit line values in dB. A delta
/// HIPAR deltaLimit/limit of +200dB indicates no limit check is active.
/// HIPAR returnedValues/Returns number of value sets (frequency, level and delta to
/// HIPAR returnedValues/limit line) read (available to return) from the instrument.
/// HIPAR returnedValues/Number of values returned in arrays is limited with the
/// HIPAR returnedValues/parameter No of Values.
ViStatus _VI_FUNC rsspecan_ReadSEPeakList(ViSession instrSession,
                                          ViInt32   noOfValues,
                                          ViReal64  frequency[],
                                          ViReal64  level[],
                                          ViReal64  deltaLimit[],
                                          ViInt32   *returnedValues)
{
    ViStatus error = VI_SUCCESS;
    ViReal64 *data = NULL;
    ViInt32  dataSize = 0;
    ViInt32  arrayLength;

    checkErr(RsCore_LockSession(instrSession));

    arrayLength = 3 * noOfValues;
    viCheckAlloc(data = (ViReal64 *)malloc(arrayLength * sizeof(ViReal64)));
    checkErr(rsspecan_dataReadTrace(instrSession, 1, "SPUR", arrayLength, data, &dataSize));
    checkErr(RsCore_DecimateViReal64ArrayToUserBuffers(data, dataSize, noOfValues, returnedValues, frequency, level, deltaLimit, NULL, NULL, NULL));

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    if (data) free(data);
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN This function returns the resulting peak list of the spurious
/// HIFN emission measurement LIST EVALUATION.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.
/// HIPAR noOfValues/This control sets the number of values returned in arrays.
/// HIPAR rangeNumber/Returns an array of range numbers.
/// HIPAR startFrequency/Document parameter here.
/// HIPAR stopFrequency/Document parameter here.
/// HIPAR resolutionBandwidth/Document parameter here.
/// HIPAR frequency/Document parameter here.
/// HIPAR levelAbs/Document parameter here.
/// HIPAR levelRel/Document parameter here.
/// HIPAR delta/Document parameter here.
/// HIPAR limitCheck/Document parameter here.
/// HIPAR reserved1/Document parameter here.
/// HIPAR reserved2/Document parameter here.
/// HIPAR returnedValues/Document parameter here.
ViStatus _VI_FUNC rsspecan_ReadSEListEvaluation(ViSession instrSession,
                                                ViInt32   noOfValues,
                                                ViReal64  rangeNumber[],
                                                ViReal64  startFrequency[],
                                                ViReal64  stopFrequency[],
                                                ViReal64  resolutionBandwidth[],
                                                ViReal64  frequency[],
                                                ViReal64  levelAbs[],
                                                ViReal64  levelRel[],
                                                ViReal64  delta[],
                                                ViReal64  limitCheck[],
                                                ViReal64  reserved1[],
                                                ViReal64  reserved2[],
                                                ViInt32   *returnedValues)
{
    ViStatus error = VI_SUCCESS;
    ViChar   *buffer = NULL;
    ViChar   *pbuf;
    ViInt32  count = 0;
    ViInt32  i;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidNullPointer(instrSession, rangeNumber),
        2, "Range number");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, startFrequency),
        3, "Start Frequency");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, stopFrequency),
        4, "Stop Frequency");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, resolutionBandwidth),
        5, "Resolution Bandwidth");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, frequency),
        6, "Frequency");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, levelAbs),
        7, "Level Abs");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, levelRel),
        8, "Level Rel");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, delta),
        9, "Delta");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, limitCheck),
        10, "Limit Check");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, reserved1),
        11, "Reserved 1");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, reserved2),
        12, "Reserved 2");

    checkErr(RsCore_QueryViStringUnknownLength(instrSession, "TRAC:DATA? LIST", &buffer));

    pbuf = strtok(buffer, ",");
    if (pbuf)
        do
        {
            if (count < noOfValues)
            {
                (void)sscanf(pbuf, "%le", &rangeNumber[count]);
                pbuf = strtok(NULL, ",");
                (void)sscanf(pbuf, "%le", &startFrequency[count]);
                pbuf = strtok(NULL, ",");
                (void)sscanf(pbuf, "%le", &stopFrequency[count]);
                pbuf = strtok(NULL, ",");
                (void)sscanf(pbuf, "%le", &resolutionBandwidth[count]);
                pbuf = strtok(NULL, ",");
                (void)sscanf(pbuf, "%le", &frequency[count]);
                pbuf = strtok(NULL, ",");
                (void)sscanf(pbuf, "%le", &levelAbs[count]);
                pbuf = strtok(NULL, ",");
                (void)sscanf(pbuf, "%le", &levelRel[count]);
                pbuf = strtok(NULL, ",");
                (void)sscanf(pbuf, "%le", &delta[count]);
                pbuf = strtok(NULL, ",");
                (void)sscanf(pbuf, "%le", &limitCheck[count]);
                pbuf = strtok(NULL, ",");
                (void)sscanf(pbuf, "%le", &reserved1[count]);
                pbuf = strtok(NULL, ",");
                (void)sscanf(pbuf, "%le", &reserved2[count]);
                pbuf = strtok(NULL, ",");
            }
            else
            {
                for (i = 0; i < 11; i++)
                    pbuf = strtok(NULL, ",");
            }
            count++;
        }
        while (pbuf);

    if (returnedValues)
        *returnedValues = count;

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    if (buffer) free(buffer);
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function selects the Spurious Measurement Application mode.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
ViStatus _VI_FUNC rsspecan_SpuriousApplicationMode(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_SPURIOUS_APPLICATION_MODE, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function configures spruriou search.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.
/// HIPAR type/Defines the type of measurement to be configured and performed.
/// HIPAR removeResiduals/If enabled, residual spurs, which are generated by internal
/// HIPAR removeResiduals/components in the R&S FSW itself, are not included in the spur
/// HIPAR removeResiduals/results.
/// HIPAR markResiduals/Determines whether or not to mark residuals in the spur table.
ViStatus _VI_FUNC rsspecan_ConfigureSESearch(ViSession instrSession,
                                             ViInt32   type,
                                             ViBoolean removeResiduals,
                                             ViBoolean markResiduals)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_SE_TYPE_OF_SEARCH, type),
            2, "Type");

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_SE_REMOVE_RESIDUALS, removeResiduals),
            3, "Remove Residuals");

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_SE_MARK_RESIDUALS, markResiduals),
            4, "Mark Residuals");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN Defines which steps of the measurement process are performed.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.
/// HIPAR step/Defines the type of measurement to be configured and performed.
ViStatus _VI_FUNC rsspecan_ConfigureSEMeasurementSteps(ViSession instrSession,
                                                       ViInt32   step)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_SE_PERFORMED_STEPS, step),
            2, "Step");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function automatically detects the highest peak over the
/// HIFN complete frequency range of the analyzer.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.
ViStatus _VI_FUNC rsspecan_SpuriousAutoAdjustCarrier(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_SE_CARRIER_ADJUST_AUTO, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN Defines the condition for matching the measured to the predicted spurs.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.
/// HIPAR condition/Defines the condition for matching the measured to the predicted spurs.
ViStatus _VI_FUNC rsspecan_ConfigureSEMatchingCondition(ViSession instrSession,
                                                        ViInt32   condition)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, NULL, RSSPECAN_ATTR_SE_MATCHING_CONDITION, condition),
            2, "Condition");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function loads a stored range setup from a .csv file.
/// HIRET Document return value here.
/// HIRET You may use multiple lines for documentation.
/// HIPAR instrSession/Document parameter here.
/// HIPAR fileName/This control specifies the file name.
ViStatus _VI_FUNC rsspecan_SELoadRanges(ViSession instrSession,
                                        ViString  fileName)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_SE_WIDE_LOAD_RANGES, fileName),
            2, "File Name");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN Saves the current range setup to a user-defined comma-separated
/// HIFN (.csv) file for later use.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR fileName/This control specifies the file name.
ViStatus _VI_FUNC rsspecan_SESaveRanges(ViSession instrSession,
                                        ViString  fileName)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_SE_WIDE_SAVE_RANGES, fileName),
            2, "File Name");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN Removes all but the first range.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsspecan_SEClearRanges(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_SE_WIDE_CLEAR_RANGES, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function configures the carrie reference level.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR carrierLevel/Defines the maximum peak of the signal, which is considered to
/// HIPAR carrierLevel/be the reference carrier.
/// HIPAR powerReference/Sets the thresholds to either absolute or relative to the
/// HIPAR powerReference/carrier reference level.
ViStatus _VI_FUNC rsspecan_ConfigureSpuriousCarrierDetection(ViSession instrSession,
                                                             ViReal64  carrierLevel,
                                                             ViInt32   powerReference)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_SE_CARRIER_REFERENCE_CARRIER_LEVEL, carrierLevel),
            2, "Carrier Level");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_SE_CARRIER_REFERENCE_POWER_REFERENCE, powerReference),
            3, "Power Reference");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Determines whether frequency values in all results and settings for
/// HIFN  the Spurious measurement are defined as absolute values or relative to
/// HIFN  the 'Carrier Frequency' frequency.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR reference/Determines whether frequency values in all results and settings for
/// HIPAR reference/the Spurious measurement are defined as absolute values or relative to
/// HIPAR reference/the 'Carrier Frequency' frequency.
ViStatus _VI_FUNC rsspecan_ConfigureSpuriousCarrierFrequencyReference(
                            ViSession instrSession,
                            ViInt32   reference)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, NULL, RSSPECAN_ATTR_SE_CARRIER_SPUR_FREQUENCY_REFERENCE, reference),
            2, "Reference");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines or queries the frequency at which the maximum peak of the
/// HIFN  signal, that is: the reference carrier, was found.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR frequency/Defines or queries the frequency at which the maximum peak of the
/// HIPAR frequency/signal, that is: the reference carrier, was found.
ViStatus _VI_FUNC rsspecan_ConfigureSpuriousCarrierFrequency(ViSession instrSession,
                                                             ViReal64  frequency)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, NULL, RSSPECAN_ATTR_SE_CARRIER_FREQUENCY, frequency),
            2, "Frequency");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures harmonics of the carrier
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR identifyHarmonics/Enables or disables the identification of harmonics of the carrier
/// HIPAR toleranceforIdentification/Sets the frequency tolerance to match harmonics to measured spurs.
/// HIPAR maxHarmonicsNumber/Sets the maximum harmonics number to be measured.
ViStatus _VI_FUNC rsspecan_ConfigureSpuriousCarrierHarmonics(ViSession instrSession,
                                                             ViBoolean identifyHarmonics,
                                                             ViReal64  toleranceforIdentification,
                                                             ViInt32   maxHarmonicsNumber)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, NULL, RSSPECAN_ATTR_SE_CARRIER_HARMONICS_IDENTIFY_ENABLED, identifyHarmonics),
            2, "Identify Harmonics");

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, NULL, RSSPECAN_ATTR_SE_CARRIER_HARMONICS_TOLERANCE, toleranceforIdentification),
            3, "Tolerance for Identification");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, NULL, RSSPECAN_ATTR_SE_CARRIER_HARMONICS_MAX_NUMBER, maxHarmonicsNumber),
            4, "Max Harmonics Number");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the guard interval as a span around the reference carrier.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR guardIntervalIncluded/Determines whether the specified guard interval is included in the
/// HIPAR guardIntervalIncluded/spur search or not. If the guard interval is not included, the spectrum
/// HIPAR guardIntervalIncluded/displays contain gaps at the guard intervals.
/// HIPAR guardIntervalValue/Defines the guard interval as a span around the reference carrier.
/// HIPAR guardIntervalValue/This control is valid only if 'Guard Interval Included' == VI_FALSE.
ViStatus _VI_FUNC rsspecan_ConfigureSpuriousCarrierGuardInterval(
                            ViSession instrSession,
                            ViBoolean guardIntervalIncluded,
                            ViReal64  guardIntervalValue)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, NULL, RSSPECAN_ATTR_SE_CARRIER_GUARD_INTERVAL_INCLUDED, guardIntervalIncluded),
            2, "Guard Interval Included");
    if (!guardIntervalIncluded)
    {
        viCheckParm(rsspecan_SetAttributeViReal64(instrSession, NULL, RSSPECAN_ATTR_SE_CARRIER_GUARD_INTERVAL, guardIntervalValue),
                3, "Guard Interval Value");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function transfers all the segments in the Segment Table to
/// the Wide Search Settings.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsspecan_SpuriousTransferToWideSearch(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_SE_TRANSFER_SEARCH_SETTINGS_TO_WIDE, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function transfers the spurs (all or those specified) from
/// HIFN the Spur Table to the Directed Search Table.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR arraySize/This control defines the size of Spurs array.
/// HIPAR spurs/Array of spur numbers.
ViStatus _VI_FUNC rsspecan_SpuriousTransferSpursToDirected(ViSession instrSession,
                                                           ViInt32   arraySize,
                                                           ViInt32   spurs[])
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K50"));

    checkErr(RsCore_WriteAsciiViInt32Array(instrSession, "TRAN:SPUR ", spurs, arraySize));

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Select the numerical results to be displayed in the Spurious Detection
/// HIFN  Table.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR enabled/State of column.
/// HIPAR arraySize/This parameter determines the size of array 'Headers'.
/// HIPAR headers/Selects headers.
ViStatus _VI_FUNC rsspecan_ConfigureSpuriousDetectionTableContent(
                            ViSession instrSession,
                            ViBoolean enabled,
                            ViUInt32  arraySize,
                            ViInt32   headers[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";
    ViInt32  index = 0;
    ViChar   *p2tableValue;

    checkErr(RsCore_LockSession(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CALC:SSE:TABL:COL %s", enabled ? "ON" : "OFF");
    for (index = 0; index < (ViInt32)arraySize; index++)
    {
        viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &RsSpecAn_rngSESpuriousDetectionTableContentHeader, headers[index], &p2tableValue),
                3, "Headers");

        RsCore_StrcatMaxLen(cmd, RS_MAX_MESSAGE_BUF_SIZE, ",", p2tableValue, NULL);
    }

    checkErr(RsCore_Write(instrSession, cmd));
    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Contains the frequency plan settings.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR fplanState/Enables or disables the the use of the frequency plan for
/// HIPAR fplanState/identification of spurs.
/// HIPAR component/Defines the FPLAN Component number.
/// HIPAR centerFrequency/Defines the center of the search span that is evaluated for spur
/// HIPAR centerFrequency/identification within the frequency plan. By default, the defined
/// HIPAR centerFrequency/center frequency is used.
/// HIPAR span/Defines the span that is evaluated for spur identification within the
/// HIPAR span/frequency plan. By default, the full measurement span is used.
ViStatus _VI_FUNC rsspecan_ConfigureSpuriousFrequencyPlanSettings(
                            ViSession instrSession,
                            ViBoolean frequencyPlan,
                            ViInt32   component,
                            ViReal64  centerFrequency,
                            ViReal64  span)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K50"));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_FPLAN_ENABLED, frequencyPlan),
            2, "Frequency Plan");

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, component, 1, 6),
            3, "Component");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "Com%ld", component);

    if (frequencyPlan == VI_TRUE)
    {
        viCheckParm(rsspecan_SetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_FPLAN_CENTER_FREQUENCY, centerFrequency),
                4, "Center Frequency");

        viCheckParm(rsspecan_SetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_FPLAN_SPAN, span),
                5, "Span");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects the identifier for the second input frequency for mixers.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR component/Defines the FPLAN Component number.
/// HIPAR identifier/Selects the identifier for the second input frequency for mixers.
ViStatus _VI_FUNC rsspecan_ConfigureSpuriousFrequencyPlanIdentifier(
                            ViSession instrSession,
                            ViInt32   component,
                            ViInt32   identifier)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K50"));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, component, 1, 6),
            2, "Component");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "Com%ld", component);

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_FPLAN_IDENTIFIER, identifier),
            3, "Identifier");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Defines the type of component in the signal path. Depending on the
/// HIFN  type of component, different parameters are available.
/// HIFN     For details see Frequency Plan and Spur Identification.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR component/Defines the FPLAN Component number.
/// HIPAR type/Defines the type of component in the signal path. Depending on the
/// HIPAR type/type of component, different parameters are available.
ViStatus _VI_FUNC rsspecan_ConfigureSpuriousFrequencyPlanComponentType(
                            ViSession instrSession,
                            ViInt32   component,
                            ViInt32   type)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K50"));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, component, 1, 6),
            2, "Component");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "Com%ld", component);

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_FPLAN_COMPONENT_TYPE, type),
            3, "Type");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Deletes the selected row (component) from the frequency plan.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR component/Defines the FPLAN Component number.
ViStatus _VI_FUNC rsspecan_ConfigureSpuriousFrequencyPlanComponentDelete(
                            ViSession instrSession,
                            ViInt32   component)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K50"));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, component, 1, 6),
            2, "Component");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "Com%ld", component);

    checkErr(rsspecan_SetAttributeViString(instrSession, repCap, RSSPECAN_ATTR_FPLAN_COMPONENT_DELETE, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Adds a new component below the selected row <component> in the
/// HIFN  frequency plan. If the command is executed on a row that does not yet
/// HIFN  exist, this row and all that are missing up to this row are created.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR component/Selects repcap FPLANComponent.
ViStatus _VI_FUNC rsspecan_AddSpuriousFrequencyPlanComponent(ViSession instrSession,
                                                             ViInt32   component)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_FPLAN_COMPONENT_ADD,
            "FPLANComponent", component , RS_REPCAP_BUF_SIZE, repCap));

    checkErr(rsspecan_SetAttributeViString(instrSession, repCap, RSSPECAN_ATTR_FPLAN_COMPONENT_ADD, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Saves the current frequency plan configuration to a user-defined .csv
/// HIFN  file for later use. The result is a comma-separated list of values with
/// HIFN  the following syntax for each row of the frequency plan:
/// HIFN  <Num>,<Comp>,<InFreq1>,<MaxHarm1>,<InFreq2>,<Fact>,<MaxHarm2>,<Ident2>,<BandCtr>,<BandSpn>
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR filename/Saves the current frequency plan configuration to a user-defined .csv
/// HIPAR filename/file for later use. The result is a comma-separated list of values with
/// HIPAR filename/the following syntax for each row of the frequency plan:
/// HIPAR filename/<Num>,<Comp>,<InFreq1>,<MaxHarm1>,<InFreq2>,<Fact>,<MaxHarm2>,<Ident2>,<BandCtr>,<BandSpn>
ViStatus _VI_FUNC rsspecan_SaveSpuriousFrequencyPlan(ViSession instrSession,
                                                     ViString  filename)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViString(instrSession, NULL, RSSPECAN_ATTR_FPLAN_SAVE, filename),
            2, "Filename");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Loads a stored frequency plan configuration from a .csv file
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR filename/Loads a stored frequency plan configuration from a .csv file
ViStatus _VI_FUNC rsspecan_LoadSpuriousFrequencyPlan(ViSession instrSession,
                                                     ViString  filename)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViString(instrSession, NULL, RSSPECAN_ATTR_FPLAN_LOAD, filename),
            2, "Filename");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Saves the current predicted list to a .csv file
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR filename/Saves the current predicted list to a .csv file
ViStatus _VI_FUNC rsspecan_ExportSpuriousFrequencyPlan(ViSession instrSession,
                                                       ViString  filename)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViString(instrSession, NULL, RSSPECAN_ATTR_FPLAN_PREDICTED_EXPORT, filename),
            2, "Filename");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Provides functionality to set the frequency tolerance. This allows
/// HIFN  matching the predicted spurs to the measured spurs.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR toleranceforIdentification/Provides functionality to set the frequency tolerance. This allows
/// HIPAR toleranceforIdentification/matching the predicted spurs to the measured spurs.
ViStatus _VI_FUNC rsspecan_ConfigureSpuriousFrequencyPlanTolerance(
                            ViSession instrSession,
                            ViReal64  toleranceforIdentification)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, NULL, RSSPECAN_ATTR_FPLAN_TOLERANCE_FOR_IDENTIFICATION, toleranceforIdentification),
            2, "Tolerance for Identification");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Calculates the predicted spurs from the frequency plan and transfers
/// HIFN  them to the directed search table.
/// HIFN     For details see Frequency Plan and Spur Identification.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsspecan_TransferSpuriousFrequencySpursToTable(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_FPLAN_TRANSFER, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Calculates the predicted spurs from the frequency plan and transfers
/// HIFN  them to the directed search table.
/// HIFN     For details see Frequency Plan and Spur Identification.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsspecan_TransferSpuriousFrequencyPlanSpursToTable(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K50"));

    checkErr(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_FPLAN_TRANSFER, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Contains input frequency settings for the frequency plan.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR component/Defines the FPLAN Component number.
/// HIPAR inputPort/Defines the input port number.
/// HIPAR inputFrequency/Defines the frequency of the input signal. For all components after
/// HIPAR inputFrequency/the first one, the output frequency of the previous component is used
/// HIPAR inputFrequency/as the input frequency. For details see Frequency Plan and Spur
/// HIPAR inputFrequency/Identification.
/// HIPAR maximumHarmonics/Defines the maximum harmonic of each input frequency to be considered
/// HIPAR maximumHarmonics/in calculating mixer products for spur identification.
ViStatus _VI_FUNC rsspecan_ConfigureSpuriousFrequencyPlanInputFrequencySettings(
                            ViSession instrSession,
                            ViInt32   component,
                            ViInt32   inputPort,
                            ViReal64  inputFrequency,
                            ViInt32   maximumHarmonics)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K50"));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, component, 1, 6),
            2, "Component");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, inputPort, 1, 2),
            3, "Input Port");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "Com%ld,FPLANInp%ld", component, inputPort);

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_FPLAN_INPUT_FREQUENCY, inputFrequency),
            4, "Input Frequency");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_FPLAN_MAXIMUM_HARMONICS, maximumHarmonics),
            5, "Maximum Harmonics");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures LISN settings of spurious application.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR lisn/This control selects the V-network to be controlled via user port.
/// HIPAR highPassFilter/This control turns the 150 kHz highpass filter for the ENV216 network
/// HIPAR highPassFilter/on and off.
/// HIPAR phase/This control selects the phase of the LISN you want to measure.
ViStatus _VI_FUNC rsspecan_ConfigureSpuriousLISN(ViSession instrSession,
                                                 ViInt32   lisn,
                                                 ViBoolean highPassFilter,
                                                 ViInt32   phase)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, NULL, RSSPECAN_ATTR_SE_LISN_TYPE, lisn),
            2, "LISN");

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, NULL, RSSPECAN_ATTR_SE_LISN_HIGH_PASS_FILTER, highPassFilter),
            3, "High Pass Filter");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, NULL, RSSPECAN_ATTR_SE_LISN_PHASE, phase),
            4, "Phase");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Determines the search area for the automatic carrier measurement
/// HIFN  function.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR type/Determines the search area for the automatic carrier measurement
/// HIPAR type/function.
ViStatus _VI_FUNC rsspecan_ConfigureCarrierSearchRangeType(ViSession instrSession,
                                                           ViInt32   type)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, NULL, RSSPECAN_ATTR_SE_CARRIER_SEARCH_RANGE_TYPE, type),
            2, "Type");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the center and span of the range in which the maximum peak is
/// HIFN  searched.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR centerFrequency/Defines the center of the range in which the maximum peak is searched.
/// HIPAR span/Defines the width of the range in which the maximum peak is searched.
ViStatus _VI_FUNC rsspecan_ConfigureCarrierSearchRangeCenterSpan(
                            ViSession instrSession,
                            ViReal64  centerFrequency,
                            ViReal64  span)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, NULL, RSSPECAN_ATTR_SE_CARRIER_SEARCH_RANGE_CENTER, centerFrequency),
            2, "Center Frequency");

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, NULL, RSSPECAN_ATTR_SE_CARRIER_SEARCH_RANGE_SPAN, span),
            3, "Span");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the range in which the maximum peak is searched.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR startFrequency/Defines the beginning of the range in which the maximum peak is
/// HIPAR startFrequency/searched.
/// HIPAR stopFrequency/Defines the end of the range in which the maximum peak is searched.
ViStatus _VI_FUNC rsspecan_ConfigureCarrierSearchRangeStartStop(
                            ViSession instrSession,
                            ViReal64  startFrequency,
                            ViReal64  stopFrequency)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, NULL, RSSPECAN_ATTR_SE_CARRIER_SEARCH_RANGE_START, startFrequency),
            2, "Start Frequency");

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, NULL, RSSPECAN_ATTR_SE_CARRIER_SEARCH_RANGE_STOP, stopFrequency),
            3, "Stop Frequency");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function defines the detector to be used for all spurs in
/// HIFN the directed search measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR detector/Defines detector.
ViStatus _VI_FUNC rsspecan_ConfigureSpuriousDirectedDetector(ViSession instrSession,
                                                             ViInt32   detector)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_SE_DIRECTED_DETECTOR, detector),
            2, "Detector");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function configures common settings for all spans in the
/// HIFN directed search measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR limitOffset/Defines a limit line as an offset to the detection threshold for
/// HIPAR limitOffset/each range.
/// HIPAR peakExcursion/Defines the minimum level value by which the signal must rise or
/// HIPAR peakExcursion/fall after a detected spur so that a new spur is detected.
/// HIPAR maximumFinalRBW/Defines the maximum resolution bandwidth used in the final
/// HIPAR maximumFinalRBW/measurements for a spurious emission directed searches.
/// HIPAR referenceLevel/Defines the reference level for the directed search measurement.
/// HIPAR numberOfFFTAverages/Defines the number of FFTs to be performed for all spurs in the
/// HIPAR numberOfFFTAverages/directed search measurement.
ViStatus _VI_FUNC rsspecan_ConfigureSpuriousDirectedSearch(ViSession instrSession,
                                                           ViReal64  limitOffset,
                                                           ViReal64  peakExcursion,
                                                           ViReal64  maximumFinalRBW,
                                                           ViReal64  referenceLevel,
                                                           ViInt32   numberOfFFTAverages)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_SE_DIRECTED_SEARCH_LIMIT_OFFSET, limitOffset),
            2, "Limit Offset");

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_SE_DIRECTED_PEAK_EXCURSION, peakExcursion),
            3, "Peak Excursion");

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_SE_DIRECTED_MAXIMUM_FINAL_RBW, maximumFinalRBW),
            4, "Maximum Final RBW");

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_SE_DIRECTED_REFERENCE_LEVEL, referenceLevel),
            5, "Reference Level");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_SE_DIRECTED_NUMBER_FFT_AVERAGES, numberOfFFTAverages),
            6, "Number Of FFT Averages");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function defines the frequencies at which spurs are to be
/// HIFN expected and next parameters.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR arraySize/Sets the number of values in next arrays.
/// HIPAR frequency/Array of center frequencies for directed search measurement of the spur.
/// HIPAR searchSpan/Array of spans.
/// HIPAR detectionThreshold/Array of detection threshold.
/// HIPAR desiredSpurSNR/Array of Desired Spur SNR.
ViStatus _VI_FUNC rsspecan_ConfigureSpuriousDirectedSearchSettings(
                            ViSession instrSession,
                            ViInt32   arraySize,
                            ViReal64  frequency[],
                            ViReal64  searchSpan[],
                            ViReal64  detectionThreshold[],
                            ViReal64  desiredSpurSNR[])
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32RangeMin(instrSession, arraySize, 1),
            2, "Array Size");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, frequency), 3, "Frequency");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, searchSpan), 4, "Search Span");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, detectionThreshold), 5, "Detection Threshold");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, desiredSpurSNR), 6, "Desired Spur SNR");

    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K50"));

    checkErr(RsCore_WriteAsciiViReal64ArraysInterleaved(instrSession, "DIR:SETT ", arraySize, frequency, searchSpan, detectionThreshold, desiredSpurSNR, NULL, NULL));

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function configures RF attenuation for the directed search
/// HIFN measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR RFAttenuation/Defines the RF attenuation for the directed search measurement.
ViStatus _VI_FUNC rsspecan_ConfigureSpuriousDirectedRFAttenuation(
                            ViSession instrSession,
                            ViInt32   RFAttenuation)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_SE_DIRECTED_RF_ATTENUATION, RFAttenuation),
            2, "RF Attenuation");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function configures optional preamplifier.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR state/Switches the optional preamplifier on or off (if available) for
/// HIPAR state/the directed search measurement.
/// HIPAR gain/Defines the gain by the optional preamplifier.
ViStatus _VI_FUNC rsspecan_ConfigureSpuriousDirectedPreamplifier(
                            ViSession instrSession,
                            ViBoolean state,
                            ViInt32   gain)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_SE_DIRECTED_PREAMPLIFIER_STATE, state),
            2, "State");
    if (state == VI_TRUE)
    {
        viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_SE_DIRECTED_PREAMPLIFIER, gain),
                3, "Gain");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN Loads a stored search configuration from a .csv file.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR fileName/Specifies full path of the file.
ViStatus _VI_FUNC rsspecan_SpuriousDirectedLoadTable(ViSession instrSession,
                                                     ViString  fileName)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_SE_DIRECTED_LOAD_TABLE, fileName),
            2, "File Name");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN Saves the current directed search configuration to a user-defined .csv file for later use.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR fileName/Specifies full path of the file.
ViStatus _VI_FUNC rsspecan_SpuriousDirectedSaveTable(ViSession instrSession,
                                                     ViString  fileName)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_SE_DIRECTED_SAVE_TABLE, fileName),
            2, "File Name");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the number of ranges in the sweep list.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR numberOfRanges/Queries the number of ranges in the sweep list.
ViStatus _VI_FUNC rsspecan_QuerySpuriousRangeNumberOfRanges(ViSession instrSession,
                                                            ViInt32   *numberOfRanges)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_GetAttributeViInt32(instrSession, NULL, RSSPECAN_ATTR_SE_WIDE_SEARCH_NUMBER_OF_RANGES, numberOfRanges),
            2, "Number Of Ranges");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Adds a range right or left to the selected one. If the command is used
/// HIFN  on a range that does not yet exist, the range and all with lower
/// HIFN  indices up to this one are created.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR range/Selects repcap Range.
/// HIPAR insertDirection/Adds a range right or left to the selected one. If the command is used
/// HIPAR insertDirection/on a range that does not yet exist, the range and all with lower
/// HIPAR insertDirection/indices up to this one are created.
ViStatus _VI_FUNC rsspecan_InsertSpuriousRange(ViSession instrSession,
                                               ViInt32   range,
                                               ViInt32   insertDirection)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_SE_LIST_RANGE_INSERT_RANGE,
            "Range", range - 1, RS_REPCAP_BUF_SIZE, repCap));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_SE_LIST_RANGE_INSERT_RANGE, insertDirection),
            3, "Insert Direction");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Writes the value of the specified parameter to all of the currently
/// HIFN  defined ranges.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR range/Selects repcap Range.
/// HIPAR selectedSetting/Writes the value of the specified parameter to all of the currently
/// HIPAR selectedSetting/defined ranges.
ViStatus _VI_FUNC rsspecan_ConfigureSpuriousRangeUsSelectionForAllRanges(
                            ViSession instrSession,
                            ViInt32   range,
                            ViInt32   selectedSetting)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_SE_LIST_RANGE_USE_SELECTION_FOR_ALL_RANGES,
            "Range", range - 1, RS_REPCAP_BUF_SIZE, repCap));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_SE_LIST_RANGE_USE_SELECTION_FOR_ALL_RANGES, selectedSetting),
            3, "Selected Setting");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function configures the RF attenuation settings for the
/// HIFN specified range.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR range/Sets the configured range.
/// HIPAR attenuation/This control sets the RF attenuation of a range.
ViStatus _VI_FUNC rsspecan_ConfigureSERangeRFAttenuation(ViSession instrSession,
                                                         ViInt32   range,
                                                         ViReal64  attenuation)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_SE_LIST_RANG_INP_ATT,
            "Range", range - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Range");

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_SE_LIST_RANG_INP_ATT, attenuation),
            3, "Attenuation");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function configures the resolution bandwidth for the specified range.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR range/Sets the configured range.
/// HIPAR RBWAuto/Activates or deactivates automatic definition of the RBW for
/// HIPAR RBWAuto/individual ranges.
/// HIPAR resolutionBandwidth/Sets the the resolution bandwidth.
/// HIPAR maximumFinalRBW/Defines the maximum resolution bandwidth used in the final
/// HIPAR maximumFinalRBW/measurements for a spurious emission measurement range.
ViStatus _VI_FUNC rsspecan_ConfigureSERangeBandwidth(ViSession instrSession,
                                                     ViInt32   range,
                                                     ViBoolean RBWAuto,
                                                     ViReal64  resolutionBandwidth,
                                                     ViReal64  maximumFinalRBW)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_SE_LIST_RANG_BAND_AUTO,
            "Range", range - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Range");

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, repCap, RSSPECAN_ATTR_SE_LIST_RANG_BAND_AUTO, RBWAuto),
            3, "RBW Auto");
    if (RBWAuto == VI_FALSE)
    {
        viCheckParm(rsspecan_SetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_SE_LIST_RANG_BAND, resolutionBandwidth),
                4, "Resolution Bandwidth");
    }
    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_SE_LIST_RANGE_MAXIMUM_FINAL_RBW, maximumFinalRBW),
            5, "Maximum Final RBW");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function configures the spur detection for selected range.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR range/Sets the configured range.
/// HIPAR thresholdStart/Defines an absolute threshold that the power level must exceed
/// HIPAR thresholdStart/for a peak to be detected as a true spur.
/// HIPAR thresholdStop/Defines an absolute threshold that the power level must exceed
/// HIPAR thresholdStop/for a peak to be detected as a true spur.
/// HIPAR limitOffset/Defines a limit line as an offset to the detection threshold for
/// HIPAR limitOffset/each range.
/// HIPAR peakExcursion/Defines the minimum level value by which the signal must rise or
/// HIPAR peakExcursion/fall after a detected spur so that a new spur is detected.
/// HIPAR minimumSNR/Defines the minimum signal-to-noise ratio (in dB) that the power
/// HIPAR minimumSNR/level must exceed for
/// HIPAR minimumSNR/a spur to be recognized during the final spur frequency scan.
ViStatus _VI_FUNC rsspecan_ConfigureSERangeSpurDetection(ViSession instrSession,
                                                         ViInt32   range,
                                                         ViReal64  thresholdStart,
                                                         ViReal64  thresholdStop,
                                                         ViReal64  limitOffset,
                                                         ViReal64  peakExcursion,
                                                         ViReal64  minimumSNR)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_SE_LIST_RANGE_THRESHOLD_START,
            "Range", range - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Range");

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_SE_LIST_RANGE_THRESHOLD_START, thresholdStart),
            3, "Threshold Start");

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_SE_LIST_RANGE_THRESHOLD_STOP, thresholdStop),
            4, "Threshold Stop");

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_SE_LIST_RANGE_LIMIT_OFFSET, limitOffset),
            5, "Limit Offset");

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_SE_LIST_RANGE_PEAK_EXCURSION, peakExcursion),
            6, "Peak Excursion");

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_SE_LIST_RANGE_SIGNAL_NOISE_RATIO, minimumSNR),
            7, "Minimum SNR");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN This function configures the number of FFT averages to be
/// HIFN performed for each range or segment.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR range/Sets the configured range.
/// HIPAR numberOfFFTAverages/Defines the number of FFT averages to be performed for each
/// HIPAR numberOfFFTAverages/range or segment.
ViStatus _VI_FUNC rsspecan_ConfigureSERangeNumberFFTAverage(ViSession instrSession,
                                                            ViInt32   range,
                                                            ViInt32   numberOfFFTAverages)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_SE_LIST_RANGE_NUMBER_FFT_AVERAGES,
            "Range", range - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Range");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_SE_LIST_RANGE_NUMBER_FFT_AVERAGES, numberOfFFTAverages),
            2, "Number Of FFT Averages");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/*===========================================================================*
 *= RsSpecAn Signal Statistics Group Functions ==============================*
 *===========================================================================*/
/// HIFN This function configures parameters of signal statistic measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.
/// HIPAR measurement/Selects the signal statistic measurement.
/// HIPAR numberofSamples/Defines the number of measurement points to be acquired for the
/// HIPAR numberofSamples/statistical measurement functions.
/// HIPAR resolutionBandwidth/Specifies the measurement resolution bandwidth.
ViStatus _VI_FUNC rsspecan_ConfigureSignalStatisticMeasurement(
                            ViSession instrSession,
                            ViInt32   measurement,
                            ViInt32   numberofSamples,
                            ViReal64  resolutionBandwidth)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    switch (measurement)
    {
    case RSSPECAN_VAL_SSTAT_OFF:
        checkErr(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_MEAS_STAT_APD_STATE, VI_FALSE));
        checkErr(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_MEAS_STAT_CCDF_STATE, VI_FALSE));
        break;
    case RSSPECAN_VAL_SSTAT_APD:
        checkErr(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_MEAS_STAT_CCDF_STATE, VI_FALSE));
        checkErr(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_MEAS_STAT_APD_STATE, VI_TRUE));
        viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_MEAS_STAT_SAMPLES, numberofSamples),
                4, "Number Of Samples");
        break;
    case RSSPECAN_VAL_SSTAT_CCDF:
        checkErr(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_MEAS_STAT_APD_STATE, VI_FALSE));
        checkErr(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_MEAS_STAT_CCDF_STATE, VI_TRUE));
        viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_MEAS_STAT_SAMPLES, numberofSamples),
                3, "Number Of Samples");
        break;
    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, measurement), 2, "Measurement");
    }
    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_RESOLUTION_BANDWIDTH, resolutionBandwidth),
            4, "Resolution Bandwidth");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Enables or disables the red trace in the CCDF display indicating the
/// HIFN  normal distribution.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR showNormalDistribution/Enables or disables the red trace in the CCDF display indicating the
/// HIPAR showNormalDistribution/normal distribution.
ViStatus _VI_FUNC rsspecan_ConfigureSignalStatisticNormalDistributionEnabled(
                            ViSession instrSession,
                            ViBoolean showNormalDistribution)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, NULL, RSSPECAN_ATTR_SIGNAL_STATISTIC_NORMAL_DISTRIBUTION_ENABLED, showNormalDistribution),
            2, "Show Normal Distribution");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function positions the selected marker to the given
/// HIFN probability.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.
/// HIPAR marker/This control selects marker.
/// HIPAR probability/Defines the probability.
ViStatus _VI_FUNC rsspecan_ProbabilityMarkerPosition(ViSession instrSession,
                                                     ViInt32   reserved,
                                                     ViInt32   marker,
                                                     ViReal64  probability)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
             reserved;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_MARKER_PROBABILITY,
            "Marker", marker - 1, RS_REPCAP_BUF_SIZE, repCap),
            3, "Marker");

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_MARKER_PROBABILITY, probability),
            4, "Probability");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function configures the common gate range parameters
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR trace/This control selects trace.
/// HIPAR comment/Defines the comment for the gate range settings of each trace.
/// HIPAR period/Defines length of the period to be traced. The control Trace has
/// HIPAR period/no effect for this setting.
ViStatus _VI_FUNC rsspecan_ConfigureSignalStatisticGate(ViSession instrSession,
                                                        ViInt32   trace,
                                                        ViString  comment,
                                                        ViReal64  period)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_EXTERNAL_GATE_TRACE_COMMENT,
            "Trace", trace - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Trace");

    viCheckParm(rsspecan_SetAttributeViString(instrSession, repCap, RSSPECAN_ATTR_EXTERNAL_GATE_TRACE_COMMENT, comment),
            3, "Probability");

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_EXTERNAL_GATE_TRACE_PERIOD_LENGTH, period),
            4, "Period");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN This function configures the gate range.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init
/// HIPAR instrSession/or rsspecan_InitWithOptions function.
/// HIPAR trace/This control selects trace.
/// HIPAR range/This control selects range.
/// HIPAR state/This control includes or excludes a gate range for a particular trace.
/// HIPAR start/Sets the start time for a gate range
/// HIPAR stop/Sets the stop time for a gate range
ViStatus _VI_FUNC rsspecan_ConfigureSignalStatisticGateRange(ViSession instrSession,
                                                             ViInt32   trace,
                                                             ViInt32   range,
                                                             ViBoolean state,
                                                             ViReal64  start,
                                                             ViReal64  stop)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_trace[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_range[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_EXTERNAL_GATE_TRACE_STATE,
            "Trace", trace - 1, RS_REPCAP_BUF_SIZE, repCap_trace),
            2, "Trace");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_EXTERNAL_GATE_TRACE_STATE,
            "EGateRange", range - 1, RS_REPCAP_BUF_SIZE, repCap_range),
            3, "Range");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "%s,%s", repCap_trace, repCap_range);

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, repCap, RSSPECAN_ATTR_EXTERNAL_GATE_TRACE_STATE, state),
            4, "State");

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_EXTERNAL_GATE_TRACE_START, start),
            5, "Start");

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_EXTERNAL_GATE_TRACE_STOP, stop),
            6, "Stop");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function configures parameters of signal statistic
/// HIFN measurement X-Axis scaling.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR xAxisRefLevel/Toggles the scaling of Y axis between percentage and absolute.
/// HIPAR xAxisRange/Defines the level range for the X axis of the measurement
/// HIPAR xAxisRange/diagram.
/// HIPAR offset/ Defines an arithmetic level offset.
ViStatus _VI_FUNC rsspecan_ConfigureSignalStatisticScaleXAxis(
                            ViSession instrSession,
                            ViReal64  xAxisRefLevel,
                            ViReal64  xAxisRange,
                            ViReal64  offset)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_MEAS_STAT_X_REF, xAxisRefLevel),
            2, "X Axis Ref Level");

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_MEAS_STAT_X_RANGE, xAxisRange),
            3, "X Axis Range");

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_REFERENCE_LEVEL_OFFSET, offset),
            4, "Offset");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function configures parameters of signal statistic
/// HIFN measurement Y Axis scaling.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR yUnit/Toggles the scaling of Y axis between percentage and absolute.
/// HIPAR yAxisMaxValue/Defines the upper limit for the Y axis of the diagram in
/// HIPAR yAxisMaxValue/statitistical measurements.
/// HIPAR yAxisMinValue/Defines the lower limit for the Y axis of the diagram in
/// HIPAR yAxisMinValue/statistical measurements.
ViStatus _VI_FUNC rsspecan_ConfigureSignalStatisticMeasurementYAxis(
                            ViSession instrSession,
                            ViInt32   yUnit,
                            ViReal64  yAxisMaxValue,
                            ViReal64  yAxisMinValue)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_MEAS_STAT_Y_UNIT, yUnit),
            2, "Y Unit");

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_MEAS_STAT_Y_MAX, yAxisMaxValue),
            3, "Y Axis Max Value");

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_MEAS_STAT_Y_MIN, yAxisMinValue),
            4, "Y Axis Min Value");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function optimizes the level setting of the instrument
/// HIFN depending on the measured peak power, in order to obtain maximum
/// HIFN  instrument sensitivity.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
ViStatus _VI_FUNC rsspecan_SignalStatisticMeasurementPresetScaling(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_MEAS_STAT_PRESET, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function optimizes the level setting of the instrument
/// HIFN  depending on the measured peak power, in order to obtain maximum
/// HIFN  instrument sensitivity.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR timeout/This control sets the timeout value.
ViStatus _VI_FUNC rsspecan_SignalStatisticMeasurementAdjustSettings(
                            ViSession instrSession,
                            ViUInt32  timeout)
{
    ViStatus error = VI_SUCCESS;
    ViUInt32 old_timeout = 0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViUInt32Range(instrSession, timeout, 0, 4294967295UL), 2, "Timeout");

    checkErr(rsspecan_GetTimeout(instrSession, &old_timeout));
    checkErr(rsspecan_SetTimeout(instrSession, timeout));
    checkErr(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_MEAS_STAT_ADJ, NULL));

Error:
    if (old_timeout > 0)
        rsspecan_SetTimeout(instrSession, old_timeout);
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/*===========================================================================*
 *------------------- Signal Statistics -> Results --------------------------*
 *===========================================================================*/
/// HIFN This function reads out the level values for the probabilities
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR trace/Selects the trace.
/// HIPAR probability/Selects the probability.
/// HIPAR result/Returns the selected level value.
ViStatus _VI_FUNC rsspecan_QuerySignalStatisticCCDFLevel(ViSession instrSession,
                                                         ViInt32   trace,
                                                         ViInt32   probability,
                                                         ViReal64  *result)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViChar   *p2string;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, trace, 1, 6),
            2, "Trace");
    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &rsspecan_rngMeasStatProbability, probability, &p2string),
        3, "Probability");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, result),
        4, "Result");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CALC:STAT:CCDF:X%ld? %s", trace, p2string);
    checkErr(RsCore_QueryViReal64(instrSession, cmd, result));

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function reads out the results of statistical measurements
/// HIFN of a recorded trace.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.
/// HIPAR trace/Selects the trace.
/// HIPAR resultType/Selects the signal statistic measurement result type.
/// HIPAR result/Returns the selected measurement.
ViStatus _VI_FUNC rsspecan_QuerySignalStatisticResult(ViSession instrSession,
                                                      ViInt32   trace,
                                                      ViInt32   resultType,
                                                      ViReal64  *result)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViChar   *p2string;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, trace, 1, 6),
            2, "Trace");
    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &rsspecan_rngStatisticsResultType, resultType, &p2string),
            3, "Result Type");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, result),
        4, "Result");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CALC:STAT:RES%ld? %s", trace, p2string);
    checkErr(RsCore_QueryViReal64(instrSession, cmd, result));

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN This function reads out all results of statistical measurements
/// HIFN of a recorded trace.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.
/// HIPAR trace/Selects the trace.
/// HIPAR arraySize/Size of Results array.
/// HIPAR results/Returns all of measurement results. At least 3 values has to be
/// HIPAR results/allocated.
ViStatus _VI_FUNC rsspecan_QuerySignalStatisticAllResults(ViSession instrSession,
                                                          ViInt32   trace,
                                                          ViInt32   arraySize,
                                                          ViReal64  results[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, trace, 1, 6),
            2, "Trace");
    viCheckParm(RsCore_InvalidViInt32RangeMin(instrSession, arraySize, 1),
        3, "Array Size");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, results),
        4, "Results");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CALC:STAT:RES%ld? ALL", trace);
    checkErr(RsCore_QueryFloatArrayToUserBuffer(instrSession, cmd, arraySize, results, NULL));

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/*===========================================================================*
 *------------------- Time Domain Power -------------------------------------*
 *===========================================================================*/
/// HIFN This function switches on or off the previously selected time
/// HIFN domain power measurements. Thus one or several measurements can
/// HIFN be first selected and then switched on and off together with
/// HIFN this attribute.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR state/Switches on or off the previously selected time domain power
/// HIPAR state/measurements.
ViStatus _VI_FUNC rsspecan_ConfigureTimeDomainPowerMeasurementState(
                            ViSession instrSession,
                            ViInt32   reserved,
                            ViBoolean state)
{
    ViStatus error = VI_SUCCESS;
             reserved;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_MEAS_TDOM_STATE, state),
            3, "State");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function configures parameters of time domain power
/// HIFN measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR peak/Sets the time domain power peak masurement.
/// HIPAR RMS/Sets the time domain power RMS masurement.
/// HIPAR mean/Sets the time domain power mean masurement.
/// HIPAR standardDeviation/Sets the time domain power standard deviation masurement.
/// HIPAR average/Switches on or off averaging for the active time domain power
/// HIPAR average/measurement.
/// HIPAR maxHold/Switches on or off the peak-hold function for the active time
/// HIPAR maxHold/domain power measurement.
/// HIPAR power/Selects absolute or relative time domain power measurement.
ViStatus _VI_FUNC rsspecan_ConfigureTimeDomainPowerMeasurement(
                            ViSession instrSession,
                            ViInt32   reserved,
                            ViBoolean peak,
                            ViBoolean RMS,
                            ViBoolean mean,
                            ViBoolean standardDeviation,
                            ViBoolean average,
                            ViBoolean maxHold,
                            ViInt32   power)
{
    ViStatus error = VI_SUCCESS;
             reserved;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_MEAS_TDOM_PEAK, peak),
            3, "Peak");

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_MEAS_TDOM_RMS, RMS),
            4, "RMS");

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_MEAS_TDOM_MEAN, mean),
            5, "Mean");
    if (mean == VI_TRUE)
    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_MEAS_TDOM_SDEV, standardDeviation),
            6, "Standard Deviation");

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_MEAS_TDOM_AVG, average),
            7, "Average");

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_MEAS_TDOM_MAX_HOLD, maxHold),
            8, "Max Hold");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_MEAS_TDOM_MODE, power),
            9, "Power");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function configures the limit lines. Only results within
/// HIFN the limited search range are considered.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR searchLimits/Switches between a limited (ON) and unlimited (OFF) search
/// HIPAR searchLimits/range. The function is independent of the selection of a marker.
/// HIPAR searchLimitLeft/Sets the left limit of the search range for markers and delta
/// HIPAR searchLimitLeft/markers.
/// HIPAR searchLimitRight/Sets the right limit of the search range for markers and delta
/// HIPAR searchLimitRight/markers.
ViStatus _VI_FUNC rsspecan_ConfigureTimeDomainPowerLimits(ViSession instrSession,
                                                          ViBoolean searchLimits,
                                                          ViReal64  searchLimitLeft,
                                                          ViReal64  searchLimitRight)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_MARKER_SEARCH_LIMITS_STATE, searchLimits),
            2, "Search Limits");
    if (searchLimits)
    {
        viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_MARKER_SEARCH_LIMITS_LEFT, searchLimitLeft),
                3, "Search Limits Left");

        viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_MARKER_SEARCH_LIMITS_RIGHT, searchLimitRight),
                3, "Search Limits Right");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function switches off all time domain measurements.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
ViStatus _VI_FUNC rsspecan_TimeDomainPowerMeasurementAllOff(ViSession instrSession,
                                                            ViInt32   reserved)
{
    ViStatus error = VI_SUCCESS;
             reserved;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_MEAS_TDOM_AOFF, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN With this function the currently measured average value and RMS
/// HIFN value are declared as reference values for relative
/// HIFN measurements.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
ViStatus _VI_FUNC rsspecan_TimeDomainPowerMeasurementSetReference(
                            ViSession instrSession,
                            ViInt32   reserved)
{
    ViStatus error = VI_SUCCESS;
             reserved;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_MEAS_TDOM_SET_REFERENCE, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function queries selected time domain power measurement
/// HIFN result.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR measurement/Selects the time domain power measurement.
/// HIPAR resultType/Selects the time domain power measurement result type.
/// HIPAR result/Returns the selected result of time domain power measurement.
ViStatus _VI_FUNC rsspecan_QueryTDomPowerResult(ViSession instrSession,
                                                ViInt32   reserved,
                                                ViInt32   measurement,
                                                ViInt32   resultType,
                                                ViReal64  *result)
{
    ViStatus error = VI_SUCCESS;
             reserved;

    checkErr(RsCore_LockSession(instrSession));

    switch (measurement)
    {
    case RSSPECAN_VAL_TDOM_PEAK:
        switch (resultType)
        {
        case RSSPECAN_VAL_TDOM_RESULT:
            viCheckParm(rsspecan_GetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_MEAS_TDOM_PEAK_RESULT, result),
                    5, "Result");
            break;
        case RSSPECAN_VAL_TDOM_AVG_RESULT:
            viCheckParm(rsspecan_GetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_MEAS_TDOM_AVG_PEAK_RESULT, result),
                    5, "Result");
            break;
        case RSSPECAN_VAL_TDOM_MAX_HOLD_RESULT:
            viCheckParm(rsspecan_GetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_MEAS_TDOM_PEAK_HOLD_RESULT, result),
                    5, "Result");
            break;
        default:
            viCheckParm(RsCore_InvalidViInt32Value(instrSession, measurement), 3, "Result Type");
        }
        break;
    case RSSPECAN_VAL_TDOM_RMS:
        switch (resultType)
        {
        case RSSPECAN_VAL_TDOM_RESULT:
            viCheckParm(rsspecan_GetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_MEAS_TDOM_RMS_RESULT, result),
                    5, "Result");
            break;
        case RSSPECAN_VAL_TDOM_AVG_RESULT:
            viCheckParm(rsspecan_GetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_MEAS_TDOM_AVG_RMS_RESULT, result),
                    5, "Result");
            break;
        case RSSPECAN_VAL_TDOM_MAX_HOLD_RESULT:
            viCheckParm(rsspecan_GetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_MEAS_TDOM_PEAK_HOLD_RMS_RESULT, result),
                    5, "Result");
            break;
        default:
            viCheckParm(RsCore_InvalidViInt32Value(instrSession, measurement), 3, "Result Type");
        }
        break;
    case RSSPECAN_VAL_TDOM_MEAN:
        switch (resultType)
        {
        case RSSPECAN_VAL_TDOM_RESULT:
            viCheckParm(rsspecan_GetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_MEAS_TDOM_MEAN_RESULT, result),
                    5, "Result");
            break;
        case RSSPECAN_VAL_TDOM_AVG_RESULT:
            viCheckParm(rsspecan_GetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_MEAS_TDOM_AVG_MEAN_RESULT, result),
                    5, "Result");
            break;
        case RSSPECAN_VAL_TDOM_MAX_HOLD_RESULT:
            viCheckParm(rsspecan_GetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_MEAS_TDOM_PEAK_HOLD_MEAN_RESULT, result),
                    5, "Result");
            break;
        default:
            viCheckParm(RsCore_InvalidViInt32Value(instrSession, measurement), 4, "Result Type");
        }
        break;
    case RSSPECAN_VAL_TDOM_SDEV:
        switch (resultType)
        {
        case RSSPECAN_VAL_TDOM_RESULT:
            viCheckParm(rsspecan_GetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_MEAS_TDOM_SDEV_RESULT, result),
                    5, "Result");
            break;
        case RSSPECAN_VAL_TDOM_AVG_RESULT:
            viCheckParm(rsspecan_GetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_MEAS_TDOM_AVG_SDEV_RESULT, result),
                    5, "Result");
            break;
        case RSSPECAN_VAL_TDOM_MAX_HOLD_RESULT:
            viCheckParm(rsspecan_GetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_MEAS_TDOM_PEAK_HOLD_SDEV_RESULT, result),
                    5, "Result");
            break;
        default:
            viCheckParm(RsCore_InvalidViInt32Value(instrSession, measurement), 4, "Result Type");
        }
        break;
    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, measurement), 3, "Measurement");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function configures power measurements on pulses in the
/// HIFN time domain.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR timeOffsetOfFirstPulse/Defines the time offset of first pulse to define the position of
/// HIPAR timeOffsetOfFirstPulse/the first pulse in the trace.
/// HIPAR measurementTime/Defines the measurement time.
/// HIPAR period/Defines the period.
/// HIPAR numberOfPulses/Defines number of pulses to be measured.
/// HIPAR result/Returns the array of power of a sequence of signal pulses having
/// HIPAR result/the same interval.
ViStatus _VI_FUNC rsspecan_QueryTDomPowerPulseSequenceResult(ViSession instrSession,
                                                             ViInt32   reserved,
                                                             ViReal64  timeOffsetOfFirstPulse,
                                                             ViReal64  measurementTime,
                                                             ViReal64  period,
                                                             ViInt32   numberOfPulses,
                                                             ViReal64  result[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
             reserved;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32RangeMin(instrSession, numberOfPulses, 1),
        6, "Number Of Pulses");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, result),
        7, "Result");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CALC:MARK:FUNC:MSUM? %.12f,%.12f,%.12f,%ld", timeOffsetOfFirstPulse, measurementTime, period, numberOfPulses);
    checkErr(RsCore_QueryFloatArrayToUserBuffer(instrSession, cmd, numberOfPulses, result, NULL));

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/*===========================================================================*
 *------------------- Harmonic Distortion -----------------------------------*
 *===========================================================================*/
/// HIFN This function configures harmonic distortion measurement state.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR state/Switches on or off the measurement of the harmonics of a carrier
/// HIPAR state/signal.
ViStatus _VI_FUNC rsspecan_ConfigureHDistState(ViSession instrSession,
                                               ViBoolean state)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_MEAS_HDIST_STATE, state),
            2, "State");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function configures harmonic distortion measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR noOfHarmonics/Defines the number of harmonics of a carrier signal to be
/// HIPAR noOfHarmonics/measured.
/// HIPAR harmonicSweepTime/The length of time to complete one sweep.
/// HIPAR harmonicRBWAuto/Specifies whether the resolution bandwidth of the 2nd to the nth
/// HIPAR harmonicRBWAuto/harmonic should be identical to the bandwidth of the first
/// HIPAR harmonicRBWAuto/harmonic (OFF) or to the next largest bandwidth (corresponding
/// HIPAR harmonicRBWAuto/to the harmonic) of the multiple of the bandwidth of the first
/// HIPAR harmonicRBWAuto/harmonic (ON).
ViStatus _VI_FUNC rsspecan_ConfigureHDist(ViSession instrSession,
                                          ViInt32   noOfHarmonics,
                                          ViReal64  harmonicSweepTime,
                                          ViBoolean harmonicRBWAuto)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_MEAS_HDIST_NOOFHARM, noOfHarmonics),
            2, "No Of Harmonic");

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_SWEEP_TIME, harmonicSweepTime),
            4, "Harmonic Sweep Time");

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_MEAS_HDIST_RBWAUTO, harmonicRBWAuto),
            5, "Harmonic RBW Auto");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function configures harmonic distortion measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR timeout/This control sets the timeout value.
ViStatus _VI_FUNC rsspecan_AdjustHDistSettings(ViSession instrSession,
                                               ViUInt32  timeout)
{
    ViStatus error = VI_SUCCESS;
    ViUInt32 old_timeout = 0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViUInt32Range(instrSession, timeout, 0, 4294967295UL), 2, "Timeout");

    checkErr(rsspecan_GetTimeout(instrSession, &old_timeout));
    checkErr(rsspecan_SetTimeout(instrSession, timeout));
    checkErr(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_MEAS_HDIST_PRESET, NULL));

Error:
    if (old_timeout > 0)
        rsspecan_SetTimeout(instrSession, old_timeout);
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function queries the results of the total harmonic distortion
/// HIFN (THD).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR harmonicDistortion_in_percent/Returns the results of the total harmonic distortion (THD) in %.
/// HIPAR harmonicDistortion_in_dB/Returns the results of the total harmonic distortion (THD) in
/// HIPAR harmonicDistortion_in_dB/dB.
ViStatus _VI_FUNC rsspecan_QueryHDist(ViSession instrSession,
                                      ViReal64* harmonicDistortionInPercent,
                                      ViReal64* harmonicDistortionInDB)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidNullPointer(instrSession, harmonicDistortionInPercent),
        2, "Harmonic Distortion in percent");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, harmonicDistortionInDB),
        3, "Harmonic Distortion in dB");

    checkErr(RsCore_QueryTupleViReal64(instrSession, ":CALC:MARK:FUNC:HARM:DIST? TOT", harmonicDistortionInPercent, harmonicDistortionInDB, NULL, NULL));

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function reads out the list of harmonics.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init
/// HIPAR instrSession/or rsspecan_InitWithOptions function.
/// HIPAR arraySize/This control defines the size of harmony list array.
/// HIPAR harmonicsList/Returns the list of harmonics.
/// HIPAR returnedValues/Returns the number of values in harmonic list returned from the
/// HIPAR returnedValues/instrument.
ViStatus _VI_FUNC rsspecan_QueryHDistHarmList(ViSession instrSession,
                                              ViInt32   arraySize,
                                              ViReal64  harmonicsList[],
                                              ViInt32   *returnedValues)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidNullPointer(instrSession, harmonicsList),
        2, "Harmonic List");

    checkErr(RsCore_QueryFloatArrayToUserBuffer(instrSession, "CALC:MARK:FUNC:HARM:LIST?", arraySize, harmonicsList, returnedValues));
    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/*===========================================================================*
 *------------------- Third Order Intercept ---------------------------------*
 *===========================================================================*/
/// HIFN This command initiates a measurement to determine the third
/// HIFN intercept point
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init
/// HIPAR instrSession/or rsspecan_InitWithOptions function.
/// HIPAR instrSession/Document parameter here.
/// HIPAR state/Initiates the measurement of the third-order intercept point.
ViStatus _VI_FUNC rsspecan_ConfigureTOIMeasurement(ViSession instrSession,
                                                   ViInt32   reserved,
                                                   ViBoolean state)
{
    ViStatus error = VI_SUCCESS;
             reserved;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_MEAS_TOI_STATE, state),
            3, "State");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function initiates a search for signals in the current
/// HIFN trace to determine the third intercept point.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init
/// HIPAR instrSession/or rsspecan_InitWithOptions function.
ViStatus _VI_FUNC rsspecan_TOISignalSearch(ViSession instrSession,
                                           ViInt32   reserved,
                                           ViInt32   reserved2)
{
    ViStatus error = VI_SUCCESS;
             reserved;
             reserved2;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_MEAS_TOI_SEARCH, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN This function queries the third-order intercept point
/// HIFN measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init
/// HIPAR instrSession/or rsspecan_InitWithOptions function.
/// HIPAR TOIMeasurement/Returns the third-order intercept point measurement.
ViStatus _VI_FUNC rsspecan_QueryTOI(ViSession instrSession,
                                    ViInt32   reserved,
                                    ViReal64  *TOIMeasurement)
{
    ViStatus error = VI_SUCCESS;
             reserved;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_MEAS_TOI_RESULT, TOIMeasurement),
            3, "TOI Measurement");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function queries the third-order intercept results.
/// HIFN
/// HIFN     To get a valid result, you have to perform a complete measurement with
/// HIFN  synchronization to the end of the measurement before reading out the
/// HIFN  result. This is only possible for single sweep mode.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR maximum/Queries the results for the maximum third order intercept point
/// HIPAR maximum/measurement.
/// HIPAR minimum/Queries the results for the minimum third order intercept point
/// HIPAR minimum/measurement.
ViStatus _VI_FUNC rsspecan_QueryTOIResults(ViSession instrSession,
                                           ViReal64* maximum,
                                           ViReal64* minimum)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_TOI_RESULT_MAXIMUM, maximum),
            2, "Maximum");

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_TOI_RESULT_MINIMUM, minimum),
            3, "Minimum");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/*===========================================================================*
 *------------------- Modulation Depth --------------------------------------*
 *===========================================================================*/
/// HIFN This functionturns the AM Modulation Depth measurement on and off
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init
/// HIPAR instrSession/or rsspecan_InitWithOptions function.
/// HIPAR state/Switches on the measurement of the AM modulation depth.
ViStatus _VI_FUNC rsspecan_ConfigureModDepthMeasurement(ViSession instrSession,
                                                        ViInt32   reserved,
                                                        ViBoolean state)
{
    ViStatus error = VI_SUCCESS;
             reserved;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_MEAS_MDEPTH_STATE, state),
            3, "State");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN This function starts the search of the signals required for the
/// HIFN modulation depth measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init
/// HIPAR instrSession/or rsspecan_InitWithOptions function.
ViStatus _VI_FUNC rsspecan_ModulationDepthSignalSearch(ViSession instrSession,
                                                       ViInt32   reserved)
{
    ViStatus error = VI_SUCCESS;
             reserved;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_MEAS_MDEPTH_SEARCH, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN This function queries the AM modulation depth.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init
/// HIPAR instrSession/or rsspecan_InitWithOptions function.
/// HIPAR modulationDepth/Returns the AM modulation depth.
ViStatus _VI_FUNC rsspecan_QueryModulationDepth(ViSession instrSession,
                                                ViInt32   reserved,
                                                ViReal64  *modulationDepth)
{
    ViStatus error = VI_SUCCESS;
             reserved;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_MEAS_MDEPTH_RESULT, modulationDepth),
            3, "Modulation Depth");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/*===========================================================================*
 *------------------- Pulse (Burst) Power -----------------------------------*
 *===========================================================================*/
/// HIFN This function configures and initiates the pulse power
/// HIFN measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.
/// HIPAR frequencyHz/Document parameter here.
/// HIPAR resolutionBandwidthHz/Document parameter here.
/// HIPAR measTimes/Document parameter here.
/// HIPAR triggerSource/Document parameter here.
/// HIPAR triggerLevel/Document parameter here.
/// HIPAR triggerOffsets/Document parameter here.
/// HIPAR detector/Document parameter here.
/// HIPAR noOfMeasurements/Document parameter here.
ViStatus _VI_FUNC rsspecan_BurstPowerSequence(ViSession instrSession,
                                              ViInt32   reserved,
                                              ViReal64  frequencyHz,
                                              ViReal64  resolutionBandwidthHz,
                                              ViReal64  measTimes,
                                              ViInt32   triggerSource,
                                              ViReal64  triggerLevel,
                                              ViReal64  triggerOffsets,
                                              ViInt32   detector,
                                              ViInt32   noOfMeasurements)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViChar   *p2string;
    ViChar   *p2detector;
             reserved;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViReal64RangeMin(instrSession, frequencyHz, 0.0),
        3, "Frequency [Hz]");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, measTimes, 1.0e-6, 30.0),
        5, "Meas Time [s]");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, triggerLevel, 0.0, 100.0),
        7, "Trigger Level");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, triggerOffsets, 0.0, 30.0),
        8, "Trigger Offset [s]");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, detector, 0, 1),
            9, "Detector");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, noOfMeasurements, 1, 32001),
            10, "No Of Measurements");
    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &rsspecan_rngTriggerSource, triggerSource, &p2string),
            6, "Trigger Source");
    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &rsspecan_rngDetector, detector, &p2detector),
            9, "Detector");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SENS:MPOW:SEQ %lG,%lG,%lG,%s,%lG,%lG,%s,%ld", frequencyHz, resolutionBandwidthHz, measTimes, p2string,
             triggerLevel, triggerOffsets, p2detector, noOfMeasurements);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function the filter type for the measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.
/// HIPAR filterType/This control defines the filter type for the measurement.
ViStatus _VI_FUNC rsspecan_BurstPowerFilterType(ViSession instrSession,
                                                ViInt32   reserved,
                                                ViInt32   filterType)
{
    ViStatus error = VI_SUCCESS;
             reserved;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_MPOW_FTYPE, filterType),
            3, "Filter Type");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function configures the instrument setup for multiple burst
/// HIFN power measurement, starts a measurement sequence and reads the
/// HIFN results.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.
/// HIPAR instrSession/Document parameter here.
/// HIPAR frequencyHz/Document parameter here.
/// HIPAR resolutionBandwidthHz/Document parameter here.
/// HIPAR measTimes/Document parameter here.
/// HIPAR triggerSource/Document parameter here.
/// HIPAR triggerLevel/Document parameter here.
/// HIPAR triggerOffsets/Document parameter here.
/// HIPAR detector/Document parameter here.
/// HIPAR noOfMeasurements/Document parameter here.
/// HIPAR timeout/Document parameter here.
/// HIPAR noOfMeasurements/Document parameter here.
ViStatus _VI_FUNC rsspecan_GetBurstPowerSequence(ViSession instrSession,
                                                 ViInt32   reserved,
                                                 ViReal64  frequencyHz,
                                                 ViReal64  resolutionBandwidthHz,
                                                 ViReal64  measTimes,
                                                 ViInt32   triggerSource,
                                                 ViReal64  triggerLevel,
                                                 ViReal64  triggerOffsets,
                                                 ViInt32   detector,
                                                 ViInt32   noOfMeasurements,
                                                 ViUInt32  timeout,
                                                 ViReal64  pulsePowerResults[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViChar   *p2triggerSource;
    ViChar   *p2detector;
             reserved;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViReal64RangeMin(instrSession, frequencyHz, 0.0),
        3, "Frequency [Hz]");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, measTimes, 1.0e-6, 30.0), 5, "Meas Time [s]");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, triggerLevel, 0.0, 100.0), 7, "Trigger Level");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, triggerOffsets, 0.0, 30.0), 8, "Trigger Offset [s]");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, detector, 0, 1),
            9, "Detector");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, noOfMeasurements, 1, 32001),
            10, "No Of Measurements");
    viCheckParm(RsCore_InvalidViUInt32Range(instrSession, timeout, 0, 4294967295UL), 11, "Timeout");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, pulsePowerResults),
        12, "Pulse Power Results");
    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &rsspecan_rngTriggerSource, triggerSource, &p2triggerSource),
            6, "Trigger Source");
    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &rsspecan_rngDetector, detector, &p2detector),
            3, "Power Measurement");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, ":SENS:MPOW:SEQ? %.12lf,%.12lf,%.12lf,%s,%.12lf,%.12lf,%s,%ld;*OPC", frequencyHz, resolutionBandwidthHz, measTimes, p2triggerSource,
             triggerLevel, triggerOffsets, p2detector, noOfMeasurements);
    checkErr(RsCore_QueryFloatArrayToUserBufferWithOpc(instrSession, cmd, timeout, noOfMeasurements, pulsePowerResults, NULL));

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function queries the results of the pulse power
/// HIFN measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init
/// HIPAR instrSession/or rsspecan_InitWithOptions function.
/// HIPAR noOfResults/Expected number of values to be returned from the instrument
/// HIPAR pulsePowerResults/This control returns array of multiple pulse power measurement..
ViStatus _VI_FUNC rsspecan_GetBurstPowerResults(ViSession instrSession,
                                                ViInt32   reserved,
                                                ViInt32   noOfResults,
                                                ViReal64  pulsePowerResults[])
{
    ViStatus error = VI_SUCCESS;
             reserved;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32RangeMin(instrSession, noOfResults, 1),
        3, "No Of Results");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, pulsePowerResults),
        4, "burstPowerResults");

    checkErr(RsCore_QueryFloatArrayToUserBuffer(instrSession, "*WAI;SENS:MPOW:RES?", noOfResults, pulsePowerResults, NULL));

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function queries the lowest pulse power that has been
/// HIFN measured during a pulse power measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init
/// HIPAR instrSession/or rsspecan_InitWithOptions function.
/// HIPAR powerResultMin/This control queries the minimum power value in a multiple burst
/// HIPAR powerResultMin/power measurement as configured and initiated with Burst Power
/// HIPAR powerResultMin/Sequence.
ViStatus _VI_FUNC rsspecan_GetBurstPwrResultMin(ViSession instrSession,
                                                ViInt32   reserved,
                                                ViReal64  *powerResultMin)
{
    ViStatus error = VI_SUCCESS;
             reserved;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_MPOW_MIN, powerResultMin),
            3, "Power Result Min");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures base parameters of the noise power ratio (NPR) measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR nprEnabled/Activates or deactivates the noise power ratio (NPR) measurement.
/// HIPAR channelBandwidth/Defines the channel bandwidth as an absolute value. Bandwidths covered
/// HIPAR channelBandwidth/by defined notches are deducted from this value.
/// HIPAR notchCount/Defines the number of notches for which results are determined. Note
/// HIPAR notchCount/that even if bandwidths for further notches are defined, only the
/// HIPAR notchCount/number specified here are actually calculated and displayed.
ViStatus _VI_FUNC rsspecan_ConfigureNPR(ViSession instrSession,
                                        ViBoolean nprEnabled,
                                        ViReal64  channelBandwidth,
                                        ViInt32   notchCount)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, NULL, RSSPECAN_ATTR_NPR_ENABLED, nprEnabled),
            2, "NPR Enabled");
    if (nprEnabled)
    {
        viCheckParm(rsspecan_SetAttributeViReal64(instrSession, NULL, RSSPECAN_ATTR_NPR_CHANNEL_BANDWIDTH, channelBandwidth),
                3, "Channel Bandwidth");

        viCheckParm(rsspecan_SetAttributeViInt32(instrSession, NULL, RSSPECAN_ATTR_NPR_NOTCH_COUNT, notchCount),
                4, "Notch Count");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Determines whether the power measured in a channel or notch is
/// HIFN  indicated as a power or a density value.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR mode/Determines whether the power measured in a channel or notch is
/// HIPAR mode/indicated as a power or a density value.
ViStatus _VI_FUNC rsspecan_ConfigureNPRMode(ViSession instrSession,
                                            ViInt32   mode)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, NULL, RSSPECAN_ATTR_NPR_MODE, mode),
            2, "Mode");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures NPR integration bandwidth.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR bandwidthAutoEnabled/Determines which bandwidth is used for total power density
/// HIPAR bandwidthAutoEnabled/calculation.
/// HIPAR integrationBandwidth/Defines the bandwidth to be used for total power calculation as an
/// HIPAR integrationBandwidth/absolute value. This value is only considered for NPR Integration
/// HIPAR integrationBandwidth/Bandwidth Auto = false.
/// HIPAR frequencyOffset/Shifts the bandwidth to be used for total power density calculation
/// HIPAR frequencyOffset/away from the currently defined center frequency.
ViStatus _VI_FUNC rsspecan_ConfigureNPRIntegrationBandwidth(ViSession instrSession,
                                                            ViBoolean bandwidthAutoEnabled,
                                                            ViReal64  integrationBandwidth,
                                                            ViReal64  frequencyOffset)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, NULL, RSSPECAN_ATTR_NPR_INTEGRATION_BANDWIDTH_AUTO_ENABLED, bandwidthAutoEnabled),
            2, "Bandwidth Auto Enabled");
    if (!bandwidthAutoEnabled)
    {
        viCheckParm(rsspecan_SetAttributeViReal64(instrSession, NULL, RSSPECAN_ATTR_NPR_INTEGRATION_BANDWIDTH, integrationBandwidth),
                3, "Integration Bandwidth");

        viCheckParm(rsspecan_SetAttributeViReal64(instrSession, NULL, RSSPECAN_ATTR_NPR_FREQUENCY_OFFSET, frequencyOffset),
                4, "Frequency Offset");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures selected notch parameters.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR notch/Selects the notch to be configured.
/// HIPAR frequencyOffset/Defines the center position of the notch in relation to the currently
/// HIPAR frequencyOffset/defined center frequency.
/// HIPAR bandwidth/Defines the bandwidth of the individual notch as an absolute value.
ViStatus _VI_FUNC rsspecan_ConfigureNPRNotch(ViSession instrSession,
                                             ViUInt32  notch,
                                             ViReal64  frequencyOffset,
                                             ViReal64  bandwidth)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_NPR_NOTCH_FREQUENCY_OFFSET,
            "NPRNotch", notch - 1, RS_REPCAP_BUF_SIZE, repCap));
    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_NPR_NOTCH_FREQUENCY_OFFSET, frequencyOffset),
            3, "Frequency Offset");

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_NPR_NOTCH_BANDWIDTH, bandwidth),
            4, "Bandwidth");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the bandwidth of the individual notch as a percentage of the
/// HIFN  defined channel frequency.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR notch/Selects the notch to be configured.
/// HIPAR bandwidthRelative/Defines the bandwidth of the individual notch as a percentage of the
/// HIPAR bandwidthRelative/defined channel frequency.
ViStatus _VI_FUNC rsspecan_ConfigureNPRNotchBandwidthRelative(
                            ViSession instrSession,
                            ViUInt32  notch,
                            ViReal64  bandwidthRelative)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_NPR_NOTCH_BANDWIDTH_RELATIVE,
            "NPRNotch", notch - 1, RS_REPCAP_BUF_SIZE, repCap));
    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_NPR_NOTCH_BANDWIDTH_RELATIVE, bandwidthRelative),
            3, "Bandwidth Relative");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the power results of the noise power ratio measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR resultType/Selects the result type.
/// HIPAR result/Queries the power results of the noise power ratio measurement.
ViStatus _VI_FUNC rsspecan_QueryNPRResult(ViSession instrSession,
                                          ViInt32   resultType,
                                          ViReal64  *result)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViChar   *p2result_type;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &RsSpecAn_rngNPRResultType, resultType, &p2result_type),
            4, "Direction");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CALC:NPR:RES? %s", p2result_type);
    checkErr(rsspecan_QueryViReal64(instrSession, cmd, result));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the power results of the noise power ratio measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR arraySize/This parameter determines the size of array Result.
/// HIPAR result/Queries the power results of the noise power ratio measurement.
/// HIPAR returnedValues/Returns the number of returned values.
ViStatus _VI_FUNC rsspecan_QueryNPRResultAll(ViSession instrSession,
                                             ViUInt32  arraySize,
                                             ViReal64  result[],
                                             ViInt32   *returnedValues)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_QueryFloatArrayToUserBuffer(instrSession, "CALC:NPR:RES? ALL", (ViInt32)arraySize, result, returnedValues));
    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Activates or deactivates control of the signal generator by the R&S
/// HIFN  FSW.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR enabled/Activates or deactivates control of the signal generator by the R&S
/// HIPAR enabled/FSW.
ViStatus _VI_FUNC rsspecan_ConfigureNPRGeneratorControl(ViSession instrSession,
                                                        ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_NPR_GENERATOR_ENABLED, enabled),
            2, "Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Configures the NPR Generator.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR pathRF/Selects the RF signal path of the generator to be used for signal
/// HIPAR pathRF/generation.
/// HIPAR rfOutput/Enables or disables RF output on the connected generator.
/// HIPAR frequency/Defines the frequency of the signal provided by the signal generator.
/// HIPAR levelRMS/Defines the output power level used by the connected signal generator.
/// HIPAR levelOffset/Defines a fixed offset in the power level used by the generator, for
/// HIPAR levelOffset/example due to a gain from the DUT.
/// HIPAR referenceFrequency/Selects the source of the generator reference frequency.
ViStatus _VI_FUNC rsspecan_ConfigureNPRGenerator(ViSession instrSession,
                                                 ViInt32   pathRF,
                                                 ViBoolean rfOutput,
                                                 ViReal64  frequency,
                                                 ViReal64  levelRMS,
                                                 ViReal64  levelOffset,
                                                 ViInt32   referenceFrequency)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_NPR_GENERATOR_TARGET_PATH_RF, pathRF),
            2, "Path RF");

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_NPR_GENERATOR_RF_OUTPUT_ENABLED, rfOutput),
            3, "RF Output");

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_NPR_GENERATOR_FREQUENCY_CENTER, frequency),
            4, "Frequency");

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_NPR_GENERATOR_POWER_LEVEL, levelRMS),
            5, "Level RMS");

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_NPR_GENERATOR_POWER_LEVEL_OFFSET, levelOffset),
            6, "Level Offset");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_NPR_GENERATOR_FREQUENCY_SOURCE, referenceFrequency),
            7, "Reference Frequency");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Configures the NPR Generator Frequency Coupling.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR coupling/Enables or disables frequency coupling between the R&S FSW and the
/// HIPAR coupling/connected generator.
/// HIPAR numerator/Defines the numerator of the frequency-defining factor for the
/// HIPAR numerator/generator frequency.
/// HIPAR denominator/Defines the denominator of the frequency-defining factor for the
/// HIPAR denominator/generator frequency.
/// HIPAR frequencyOffset/Defines a fixed offset to be applied to the generator frequency.
ViStatus _VI_FUNC rsspecan_ConfigureNPRGeneratorFrequencyCoupling(
                            ViSession instrSession,
                            ViBoolean coupling,
                            ViReal64  numerator,
                            ViReal64  denominator,
                            ViReal64  frequencyOffset)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_NPR_GENERATOR_COUPLING_ENABLED, coupling),
            2, "Coupling");

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_NPR_GENERATOR_FREQUENCY_NUMERATOR, numerator),
            3, "Numerator");

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_NPR_GENERATOR_FREQUENCY_DENOMINATOR, denominator),
            4, "Denominator");

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_NPR_GENERATOR_FREQUENCY_OFFSET, frequencyOffset),
            5, "Frequency Offset");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the NPR Generator Settings.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR arraySize/Sets the array size parameter.
/// HIPAR pathBB/Queries the BB signal path of the generator used for signal
/// HIPAR pathBB/generation.
/// HIPAR standard/Queries the standard currently used by the signal generator.
/// HIPAR arbWaveformFile/Queries the ARB waveform file currently used by the signal generator.
ViStatus _VI_FUNC rsspecan_QueryNPRGeneratorSettings(ViSession instrSession,
                                                     ViInt32   arraySize,
                                                     ViInt32*  pathBB,
                                                     ViChar    standard[],
                                                     ViChar    arbWaveformFile[])
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_GetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_NPR_GENERATOR_TARGET_PATH_BB, pathBB),
            3, "Path BB");

    viCheckParm(rsspecan_GetAttributeViString(instrSession, "", RSSPECAN_ATTR_NPR_GENERATOR_STANDARD,
            arraySize, standard),
            4, "Standard");

    viCheckParm(rsspecan_GetAttributeViString(instrSession, "", RSSPECAN_ATTR_NPR_GENERATOR_SELECTED_WAVEFORM,
            arraySize, arbWaveformFile),
            5, "ARB Waveform File");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the NPR Generator States.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR generatorState/Queries the state of the connected signal generator and its
/// HIPAR generatorState/availability for the Spectrum application.
/// HIPAR rfOutput/Queries the state of the generator RF output.
/// HIPAR frequency/Queries the state of the generator center frequency.
/// HIPAR levelRMS/Queries the state of the generator power level.
/// HIPAR levelOffset/Queries the state of the generator level offset.
/// HIPAR referenceFrequency/Queries the state of the generator reference frequency.
/// HIPAR notchFilter/Queries the state of the generator notch filter.
ViStatus _VI_FUNC rsspecan_QueryNPRGeneratorStates(ViSession instrSession,
                                                   ViInt32*  generatorState,
                                                   ViInt32*  rfOutput,
                                                   ViInt32*  frequency,
                                                   ViInt32*  levelRMS,
                                                   ViInt32*  levelOffset,
                                                   ViInt32*  referenceFrequency,
                                                   ViInt32*  notchFilter)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_GetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_NPR_GENERATOR_STATE, generatorState),
            2, "Generator State");

    viCheckParm(rsspecan_GetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_NPR_GENERATOR_RF_OUTPUT_STATE, rfOutput),
            3, "RF Output");

    viCheckParm(rsspecan_GetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_NPR_GENERATOR_CENTER_FREQUENCY_STATE, frequency),
            4, "Frequency");

    viCheckParm(rsspecan_GetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_NPR_GENERATOR_POWER_LEVEL_STATE, levelRMS),
            5, "Level RMS");

    viCheckParm(rsspecan_GetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_NPR_GENERATOR_POWER_LEVEL_OFFSET_STATE, levelOffset),
            6, "Level Offset");

    viCheckParm(rsspecan_GetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_NPR_GENERATOR_EXTERNAL_STATE, referenceFrequency),
            7, "Reference Frequency");

    viCheckParm(rsspecan_GetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_NPR_GENERATOR_NOTCH_FILTER_STATE, notchFilter),
            8, "Notch Filter");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Configures the NPR Generator IP Address.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR ipAddress/The TCPIP address or computer name of the signal generator connected
/// HIPAR ipAddress/to the R&S FSW via LAN.
/// HIPAR connect/Connects or disconnects the signal generator.
ViStatus _VI_FUNC rsspecan_ConfigureNPRGeneratorIP(ViSession instrSession,
                                                   ViString  ipAddress,
                                                   ViBoolean connect)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_NPR_GENERATOR_CONTROL_IP_ADDRESS, ipAddress),
            2, "IP Address");

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_NPR_GENERATOR_CONTROL_CONNECTION_ENABLED, connect),
            3, "Connect");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the state of the connected signal generator.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR bufferSize/Size of the buffer passed to the Connection State parameter
/// HIPAR connectionState/Returns the state of the connected signal generator.
/// HIPAR returnedBytes/The number of bytes returned from the instrument. If Returned Bytes >
/// HIPAR returnedBytes/Buffer Size, some data was lost. Consider resizing the buffer and
/// HIPAR returnedBytes/calling again.
ViStatus _VI_FUNC rsspecan_QueryNPRGeneratorIPConnectionState(
                            ViSession instrSession,
                            ViInt32   bufferSize,
                            ViChar    connectionState[],
                            ViInt32*  returnedBytes)
{
    ViStatus error = VI_SUCCESS;
    ViChar*  cmd = NULL;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_BuildAttrCommandString(instrSession,
                                               &g_RSSPECAN_ATTR_NPR_GENERATOR_CONNECTION_STATE,
                                               NULL, &cmd));

    checkErr(RsCore_QueryViStringUnknownLengthToUserBuffer(instrSession, cmd, bufferSize,
             connectionState, returnedBytes));

Error:
    if (cmd)
        free (cmd);
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Applies all generator setup settings to the connected signal generator
/// HIFN  once.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsspecan_ApplyNPRGeneratorAllSettings(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_NPR_GENERATOR_APPLY_ALL_SETTINGS, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Applies generator settings.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsspecan_ApplyNPRGeneratorSettings(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_NPR_GENERATOR_SETTINGS, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Applies all generator notch settings.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsspecan_ApplyNPRGeneratorNotchSettings(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_NPR_GENERATOR_NOTCH_SETTINGS, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the state of the number of notches.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR notches/Queries the state of the number of notches.
ViStatus _VI_FUNC rsspecan_QueryNPRGeneratorNotchesNumber(ViSession instrSession,
                                                          ViInt32*  notches)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_GetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_NPR_GENERATOR_NUMBER_OF_NOTCHES, notches),
            2, "Notches");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Configures the NPR Generator Notch.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR notch/Sets the notch.
/// HIPAR enabled/Enables or disables the specified notch on the signal generator.
ViStatus _VI_FUNC rsspecan_ConfigureNPRGeneratorNotch(ViSession instrSession,
                                                      ViInt32   notch,
                                                      ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_NPR_GENERATOR_NOTCH_ENABLED,
            "NPRNotch", notch - 1, RS_REPCAP_BUF_SIZE, repCap));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, repCap, RSSPECAN_ATTR_NPR_GENERATOR_NOTCH_ENABLED, enabled),
            3, "Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the NPR Generator Notch States.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR notch/Sets the notch.
/// HIPAR bandwidth/Queries the state of the absolute notch bandwidth for the specified
/// HIPAR bandwidth/notch.
/// HIPAR frequencyOffset/Queries the state of the frequency offset for the specified notch.
/// HIPAR state/Queries the state of the generator notch setting for the specified
/// HIPAR state/notch.
ViStatus _VI_FUNC rsspecan_QueryNPRGeneratorNotchStates(ViSession instrSession,
                                                        ViInt32   notch,
                                                        ViInt32*  bandwidth,
                                                        ViInt32*  frequencyOffset,
                                                        ViInt32*  state)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_NPR_GENERATOR_ABSOLUTE_NOTCH_BANDWIDTH,
            "NPRNotch", notch - 1, RS_REPCAP_BUF_SIZE, repCap));

    viCheckParm(rsspecan_GetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_NPR_GENERATOR_ABSOLUTE_NOTCH_BANDWIDTH, bandwidth),
            3, "Bandwidth");

    viCheckParm(rsspecan_GetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_NPR_GENERATOR_NOTCH_FREQUENCY_OFFSET, frequencyOffset),
            4, "Frequency Offset");

    viCheckParm(rsspecan_GetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_NPR_GENERATOR_NOTCH_STATE, state),
            5, "State");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the generator clock frequency.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR clockFrequency/Queries the generator clock frequency.
ViStatus _VI_FUNC rsspecan_QueryNPRGeneratorNotchClockFrequency(
                            ViSession instrSession,
                            ViReal64* clockFrequency)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_NPR_GENERATOR_CLOCK_FREQUENCY, clockFrequency),
            2, "Clock Frequency");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Applies all notch settings to the connected signal generator once.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsspecan_ApplyNPRGeneratorAllNotchSettings(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_NPR_GENERATOR_UPLOAD_ALL_NOTCH_SETTINGS, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/*===========================================================================*
 *= RsSpecAn I/Q Analyzer ===================================================*
 *===========================================================================*/
/// HIFN This function sets the I/Q analyzer mode
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
ViStatus _VI_FUNC rsspecan_SetIQMode(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_IQ_MODE, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function loads the IQ data from the specified file.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR fileName/Specifies full path of the file, the IQ data are load from.
ViStatus _VI_FUNC rsspecan_LoadIQData(ViSession instrSession,
                                      ViString  fileName)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_LOAD_IQ_DATA, fileName),
            2, "File Name");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function stores the IQ data from/to the specified file.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR fileName/Specifies full path of the file, where IQ data are stored.
ViStatus _VI_FUNC rsspecan_StoreIQData(ViSession instrSession,
                                       ViString  fileName)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_SAVE_IQ_DATA, fileName),
            2, "File Name");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function defines a description of the export file which is
/// HIFN stored with the data and also displayed in the file selection
/// HIFN dialog box for I/Q data import and export
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR description/Specifies a description of the export file.
ViStatus _VI_FUNC rsspecan_ConfigureIQDataExportFileDescription(
                            ViSession instrSession,
                            ViString  description)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_IQ_DATA_EXPORT_FILE_DESCRIPTION, description),
            2, "Description");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function selects the I/Q data file to be used as input for
/// HIFN further measurements.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR fileName/String containing the path and name of the source file. The file
/// HIPAR fileName/extension is *.iq.tar.
ViStatus _VI_FUNC rsspecan_SelectIQDataFile(ViSession instrSession,
                                            ViString  fileName)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_IQ_DATA_FILE, fileName),
            2, "File Name");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN This function activates the bandwidth extension option R&S FSW-B160 / U160, if installed.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR state/Activates the bandwidth extension option R&S FSW-B160 / U160, if installed.
ViStatus _VI_FUNC rsspecan_ConfigureTraceIQBWExtension(ViSession instrSession,
                                                       ViInt32   reserved,
                                                       ViBoolean state)
{
    ViStatus error = VI_SUCCESS;
             reserved;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_IQ_MAX_BANDWIDTH_EXTENSION, state),
            2, "State");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the maximum analysis bandwidth. Any value can be specified;
/// HIFN  the next higher fixed bandwidth is used.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR type/Selects the type.
/// HIPAR maximumBandwidth/Defines the maximum analysis bandwidth. Any value can be specified;
/// HIPAR maximumBandwidth/the next higher fixed bandwidth is used.
ViStatus _VI_FUNC rsspecan_ConfigureTraceIQBW(ViSession instrSession,
                                              ViInt32   type,
                                              ViReal64  maximumBandwidth)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    if (type == RSSPECAN_VAL_MAX)
    {
        checkErr(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_IQ_MAXIMUM_BANDWIDTH_MAX, NULL));
    }
    else
    {
        viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_IQ_MAXIMUM_BANDWIDTH, maximumBandwidth),
                3, "Maximum Bandwidth");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function The bandwidth extension option R&S FSV-B160 is always used if no other restrictions for its use apply
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsspecan_ForceTraceIQBWExtension(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_FORCE_IQ_BANDWIDTH_EXTENSION, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function defines the settings of the analyzer hardware for
/// HIFN the measurement of I/Q data. This allows setting the bandwidth
/// HIFN of the analog filters in front of the A/D converter as well as
/// HIFN setting the sample rate, trigger conditions and the record
/// HIFN length.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR samplingRate/This control sets the sampling rate for the I/Q data
/// HIPAR samplingRate/acquisition.
/// HIPAR triggerMode/This parameter selects the trigger source for starting data
/// HIPAR triggerMode/acquisition.
/// HIPAR triggerSlope/This parameter selects the slope of the trigger signal for
/// HIPAR triggerSlope/external trigger.
/// HIPAR pretriggerSamples/This control sets the number of measurement values to be
/// HIPAR pretriggerSamples/recorded before the trigger point.
/// HIPAR noOfSamples/This control sets the number of measurement values to record.
ViStatus _VI_FUNC rsspecan_ConfigureIQSet(ViSession instrSession,
                                          ViReal64  samplingRate,
                                          ViInt32   triggerMode,
                                          ViInt32   triggerSlope,
                                          ViInt32   pretriggerSamples,
                                          ViInt32   noOfSamples)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViChar*  p2triggerMode;
    ViChar*  p2triggerSlope;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViReal64Range(instrSession, samplingRate, 100.0, 10.0e+9), 2, "Sampling Rate");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, triggerSlope, RSSPECAN_VAL_NEG, RSSPECAN_VAL_POS),
            4, "Trigger Slope");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, pretriggerSamples, -461373339, 461373339), 5, "Pretrigger Samples");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, noOfSamples, 1, 1409285632),
            8, "No Of Samples");
    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &rsspecan_rngTriggerSource, triggerMode, &p2triggerMode),
            3, "Trigger Mode");
    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &rsspecan_rngPolarity, triggerSlope, &p2triggerSlope),
            4, "Trigger Slope");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "TRAC:IQ:SET NORM,0,%.12lG,%s,%s,%ld,%ld", samplingRate, p2triggerMode,
             p2triggerSlope, pretriggerSamples, noOfSamples);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function configures the  IQ data acquisition.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.
/// HIPAR samplingRate/This control sets the sampling rate for the I/Q data
/// HIPAR samplingRate/acquisition.
/// HIPAR analysisBandwidth/This control sets the filter bandwidth of the resampling filter.
/// HIPAR maximumBandwidth/This control sets the the maximum bandwidth to be used by the
/// HIPAR maximumBandwidth/R&S FSW for I/Q data acquisition.
/// HIPAR recordLength/This control sets the number of measurement values to record.
/// HIPAR swapIQ/Swaps the real (I branch) and the imaginary (Q branch) parts of
/// HIPAR swapIQ/the signal.
ViStatus _VI_FUNC rsspecan_ConfigureIQDataAcquisition(ViSession instrSession,
                                                      ViReal64  samplingRate,
                                                      ViReal64  analysisBandwidth,
                                                      ViBoolean maximumBandwidth,
                                                      ViInt32   recordLength,
                                                      ViBoolean swapIQ)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_IQ_SAMPLE_RATE, samplingRate),
            2, "Sampling Rate");

    //RSSPECAN_ATTR_IQ_DATA_FILTER_BANDWIDTH read only for FSV
    //SWAP IQ not available for FSV
    if (RsCore_IsInstrumentModel(instrSession, "FSW"))
    {
        viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_IQ_DATA_FILTER_BANDWIDTH, analysisBandwidth),
                3, "Analysis Bandwidth");

        viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_BB_SWAP_IQ, swapIQ),
                7, "Swap I/Q");
    }

    if (RsCore_HasInstrumentOptions(instrSession, "B160|B320|B500|B2000|B5000"))
    {
        viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_IQ_MAX_BANDWIDTH_EXTENSION, maximumBandwidth),
                4, "Maximum Bandwidth");
    }
    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_IQ_RECORD_LENGTH, recordLength),
            6, "Record Length");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  changes the application of the current measurement channel to I/Q
/// HIFN  Analyzer, activating the simple I/Q data acquisition mode
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR state/This control switches IQ data acquisition on or off.
ViStatus _VI_FUNC rsspecan_ConfigureTraceIQDataAcquisition(ViSession instrSession,
                                                           ViInt32   reserved,
                                                           ViBoolean state)
{
    ViStatus error = VI_SUCCESS;
             reserved;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_IQ_DATA_STATE, state),
            3, "State");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function turns I/Q data analysis on and off.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize function to select the desired instrument driver session.
/// HIPAR state/Enables or disables the IQ data evaluation mode and is a prerequisite for the IQ Analyzer functions.
ViStatus _VI_FUNC rsspecan_ConfigureTraceIQDataAnalyzerEnabled(
                            ViSession instrSession,
                            ViInt32   reserved,
                            ViBoolean state)
{
    ViStatus error = VI_SUCCESS;
             reserved;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_IQ_DATA_ANALYZER_STATE, state),
            3, "State");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN This function configures the IQ bandwidth
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR mode/Configures the bandwidth mode.
/// HIPAR resolution/Sets the bandwidth resolution in manual mode.
ViStatus _VI_FUNC rsspecan_ConfigureIQBandwidth(ViSession instrSession,
                                                ViInt32   mode,
                                                ViReal64  resolution)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_IQ_BANDWIDTH_MODE, mode),
            2, "Mode");
    if (mode == RSSPECAN_VAL_IQ_BWID_MODE_MAN)
    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_IQ_BANDWIDTH_RESOLUTION, resolution),
            3, "Resolution");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN This function configures "Advanced Fourier Transformation"
/// HIFN parameters for I/Q analyzer
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR algorithm/Defines the FFT calculation method.
/// HIPAR length/Defines the number of frequency points determined by each FFT
/// HIPAR length/calculation.
/// HIPAR windowType/Defines the window type for I/Q analyzer.
/// HIPAR windowOverlap/Defines the part of a single FFT window that is re-calculated by
/// HIPAR windowOverlap/the next FFT calculation.
/// HIPAR windowLength/Defines the number of samples to be included in a single FFT
/// HIPAR windowLength/window when multiple FFT windows are used.
ViStatus _VI_FUNC rsspecan_ConfigureIQFFTMode(ViSession instrSession,
                                              ViInt32   algorithm,
                                              ViInt32   length,
                                              ViInt32   windowType,
                                              ViReal64  windowOverlap,
                                              ViInt32   windowLength)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_IQ_FFT_ALGORITHM, algorithm),
            2, "Algorithm");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_IQ_FFT_LENGTH, length),
            3, "Length");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_IQ_FFT_WINDOW_TYPE, windowType),
            4, "Window Type");

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_IQ_FFT_WIN_OVERLAP, windowOverlap),
            5, "Window Overlap");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_IQ_FFT_WIN_LENGTH, windowLength),
            6, "Window Length");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function configures the gate function for IQ data
/// HIFN measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR gate/This control activates or deactivates the gate function for IQ
/// HIPAR gate/data measurements.
/// HIPAR type/This control specifies the gate trigger type used for data
/// HIPAR type/collection.
/// HIPAR gateLength/This control defines the gate length in samples in edge mode.
/// HIPAR gateGap/This control defines the interval between several gate periods
/// HIPAR gateGap/in samples.
/// HIPAR numberOfGatePeriods/This control defines the number of gate periods after a trigger
/// HIPAR numberOfGatePeriods/signal.
ViStatus _VI_FUNC rsspecan_ConfigureTraceIQGate(ViSession instrSession,
                                                ViBoolean gate,
                                                ViInt32   type,
                                                ViInt32   gateLength,
                                                ViInt32   gateGap,
                                                ViInt32   numberOfGatePeriods)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_IQ_DATA_GATE_STATE, gate),
            2, "Gate");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_IQ_DATA_GATE_TYPE, type),
            3, "Type");

    if (type == RSSPECAN_VAL_GATE_TYPE_EDGE)
    {
        viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_IQ_DATA_GATE_LENGTH, gateLength),
                4, "Gate Length");
    }

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_IQ_DATA_GATE_GAP, gateGap),
            5, "Gate Gap");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_IQ_DATA_GATE_NUMBER_OF_PERIODS, numberOfGatePeriods),
            6, "Number Of Gate Periods");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function configures the display type of the IQ data.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR displayType/This control defines the display type of the IQ data.
ViStatus _VI_FUNC rsspecan_ConfigureTraceIQDataDisplayType(ViSession instrSession,
                                                           ViInt32   displayType)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_IQ_DATA_DISPLAY_TYPE, displayType),
            2, "Display Type");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function configures the average power consumption
/// HIFN calculated at the end of I/Q data measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR state/Enables/disables the average power consumption calculation at
/// HIPAR state/the end of the I/Q data measurement.
/// HIPAR factorA/Defines the conversion factor A.
/// HIPAR factorB/Defines the conversion factor B.
ViStatus _VI_FUNC rsspecan_ConfigureIQAveragePowerConsumption(
                            ViSession instrSession,
                            ViBoolean state,
                            ViReal64  factorA,
                            ViReal64  factorB)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_IQ_APC_STATE, state),
            2, "State");
    if (state == VI_TRUE)
    {
        viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_IQ_APC_FACTOR_A, factorA),
                3, "Factor A");

        viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_IQ_APC_FACTOR_B, factorB),
                4, "Factor B");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Determines the coupling of the oscilloscope to the R&S FSW.
/// HIFN
/// HIFN     Note:
/// HIFN     This function is valid only for R&S FSW
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR coupling/Determines the coupling of the oscilloscope to the R&S FSW.
ViStatus _VI_FUNC rsspecan_ConfigureIQOscilloscopeCoupling(ViSession instrSession,
                                                           ViInt32   coupling)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_IQ_OSCILLOSCOPE_COUPLING, coupling),
            2, "Coupling");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function queries the average power consumption for an
/// HIFN analog baseband input.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR result/Queries the average power consumption for an analog baseband input.
ViStatus _VI_FUNC rsspecan_QueryIQAveragePowerConsumptionResult(
                            ViSession instrSession,
                            ViReal64  *result)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_IQ_APC_RESULT, result),
            2, "Result");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function queries the filter bandwidth of the resampling
/// HIFN filter, which is dependent on the sampling rate.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR filterBandwidth/Queries the filter bandwidth of the resampling filter.
ViStatus _VI_FUNC rsspecan_QueryTraceIQFilterBandwidth(ViSession instrSession,
                                                       ViReal64  *filterBandwidth)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_IQ_DATA_FILTER_BANDWIDTH, filterBandwidth),
            2, "Filter Bandwidth");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the format of the I/Q data to be stored.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR format/Selects number format.
/// HIPAR dataFormat/Selects data format.
ViStatus _VI_FUNC rsspecan_ConfigureIQDataFormat(ViSession instrSession,
                                                 ViInt32   format,
                                                 ViInt32   dataFormat)
{
    ViStatus error = VI_SUCCESS;
    ViChar   *p2format;
    ViChar   *p2data_format;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &RsSpecan_rngIQFormat, format, &p2format),
            2, "Format");
    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &RsSpecan_rngIQDataFormat, dataFormat, &p2data_format),
            3, "Data Format");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "MMEM:STOR:IQ:FORM %s,%s", p2format, p2data_format);
    checkErr(RsCore_WriteWithOpc(instrSession, cmd, 0));

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function queries the format of the I/Q data to be stored.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR format/This control returns format.
/// HIPAR dataFormat/This control returns data format.
ViStatus _VI_FUNC rsspecan_QueryIQDataFormat(ViSession instrSession,
                                             ViInt32   *format,
                                             ViInt32   *dataFormat)
{
    ViStatus error = VI_SUCCESS;
    ViChar   response[RS_MAX_MESSAGE_BUF_SIZE];
    ViInt32  value = 0;
    ViChar   *p2buf;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidNullPointer(instrSession, format), 2, "Format");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, dataFormat), 3, "Data Format");

    checkErr(RsCore_QueryViString(instrSession, "MMEM:STOR:IQ:FORM?", response));
    checkErr(rsspecan_CheckStatus(instrSession));

    p2buf = strtok(response, ",");
    checkErr(RsCore_FindRangeTableDiscreteNumericValue(instrSession, &rsspecan_rngFormat, p2buf, &value));
    *format = value;
    p2buf = strtok(NULL, ",\n");
    checkErr(RsCore_FindRangeTableDiscreteNumericValue(instrSession, &rsspecan_rngDataFormat, p2buf, &value));
    *dataFormat = value;

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function determines the time offset of the trigger in the
/// HIFN sample (trigger position in sample = TPIS). This value can only
/// HIFN be determined in triggered I/Q measurements using external or
/// HIFN IFPower triggers, otherwise the value is 0. The value is not
/// HIFN user-definable.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR triggerPositionInSample/This control returns the time offset of the trigger in the
/// HIPAR triggerPositionInSample/sample (trigger position in sample = TPIS).
ViStatus _VI_FUNC rsspecan_QueryTriggerPositionInSample(ViSession instrSession,
                                                        ViInt32   *triggerPositionInSample)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_GetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_QUERY_TPIS_TIME_OFFSET, triggerPositionInSample),
            2, "Trigger Position In Sample");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function returns the current X and Y results of the sweep.
/// HIFN During a sweep the last measured values are read out.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR timeout/This control sets the timeout value.
/// HIPAR bufferSize/Pass the number of doubles in the array you specify for the real
/// HIPAR bufferSize/parts and imaginary parameters.
/// HIPAR noOfPoints/Returns the number of I/Q trace data points.
/// HIPAR realParts_I/Returns an array of trace data (I-value x, real part) point. The
/// HIPAR realParts_I/buffer must contain at least as many elements as the value you
/// HIPAR realParts_I/specify with the Buffer Size parameter.
/// HIPAR imaginaryParts_Q/Returns an array of trace data (Q-value y, imaginary part)
/// HIPAR imaginaryParts_Q/point. The buffer must contain at least as many elements as the
/// HIPAR imaginaryParts_Q/value you specify with the Buffer Size parameter.
ViStatus _VI_FUNC rsspecan_ReadTraceIQData(ViSession instrSession,
                                           ViInt32   reserved,
                                           ViUInt32  timeout,
                                           ViInt32   bufferSize,
                                           ViInt32   *noOfPoints,
                                           ViReal64  realParts_I[],
                                           ViReal64  imaginaryParts_Q[])
{
    ViStatus error = VI_SUCCESS;
             reserved;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsspecan_dataReadInterleaved(instrSession, ":FORM REAL,32;:TRAC:IQ:DATA:FORM IQP;:TRAC:IQ:DATA?", timeout, bufferSize, realParts_I, imaginaryParts_Q, noOfPoints));

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function permits the readout of previously acquired
/// HIFN  (and frequency response corrected) I/Q data from the memory,
/// HIFN  with indication of the offset related to the start of measurement
/// HIFN  and with indication of the number of measurement values.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR offsetSamples/This control sets offset of the values to be read related to the
/// HIPAR offsetSamples/start of the acquired data.
/// HIPAR bufferSize/Pass the number of doubles in the array you specify for the real
/// HIPAR bufferSize/parts and imaginary parameters.
/// HIPAR noofPoints/Returns the number of I/Q trace data points.
/// HIPAR realParts_I/Returns an array of trace data (I-value x, real part) point.
/// HIPAR realParts_I/The buffer must contain at least as many elements as the value
/// HIPAR realParts_I/you specify with the Buffer Size parameter.
/// HIPAR imaginaryParts_Q/Returns an array of trace data (Q-value y, imaginary part) point.
/// HIPAR imaginaryParts_Q/The buffer must contain at least as many elements as the value
/// HIPAR imaginaryParts_Q/you specify with the Buffer Size parameter.
ViStatus _VI_FUNC rsspecan_FetchTraceIQData(ViSession instrSession,
                                            ViInt32   offsetSamples,
                                            ViInt32   noofSamples,
                                            ViInt32   bufferSize,
                                            ViInt32   *noofPoints,
                                            ViReal64  realParts_I[],
                                            ViReal64  imaginaryParts_Q[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    if (offsetSamples < 0 || noofSamples < 0)
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, ":TRAC:IQ:DATA:FORM IQP;:FORM REAL,32;:TRAC:IQ:DATA:MEM?");
    else
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, ":TRAC:IQ:DATA:FORM IQP;:FORM REAL,32;:TRAC:IQ:DATA:MEM? %ld,%ld", offsetSamples, noofSamples);

    checkErr(rsspecan_dataReadInterleaved(instrSession, cmd, 0, bufferSize, realParts_I, imaginaryParts_Q, noofPoints));

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function activates the optional 2 GHz bandwidth extension (R&S
/// HIFN  FSW-B2000).
/// HIFN
/// HIFN     Note(s):
/// HIFN
/// HIFN     (1) Manual operation on the connected oscilloscope, or remote
/// HIFN  operation other than by the R&S FSW, is not possible while the B2000
/// HIFN  option is active.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR state/Activates the optional 2 GHz bandwidth extension (R&S FSW-B2000).
/// HIPAR state/Note: Manual operation on the connected oscilloscope, or remote
/// HIPAR state/operation other than by the R&S FSW, is not possible while the B2000
/// HIPAR state/option is active.
ViStatus _VI_FUNC rsspecan_ConfigureB2000State(ViSession instrSession,
                                               ViBoolean state)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_B2000_STATE, state),
            2, "State");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function enables or disables the display update on the
/// HIFN  oscilloscope during data acquisition with the optional 2 GHz bandwidth
/// HIFN  extension (R&S FSW-B2000).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR fastExport/Enables or disables the display update on the oscilloscope during data
/// HIPAR fastExport/acquisition with the optional 2 GHz bandwidth extension (R&S
/// HIPAR fastExport/FSW-B2000). As soon as the R&S FSW-B2000 is activated, the display on
/// HIPAR fastExport/the oscilloscope is turned off to improve performance during data
/// HIPAR fastExport/export. As soon as the R&S FSW closes the connection to the
/// HIPAR fastExport/oscilloscope, the display is reactivated and the oscilloscope can be
/// HIPAR fastExport/operated as usual. However, if the LAN connection is lost for any
/// HIPAR fastExport/reason, the display of the oscilloscope remains deactivated. Use this
/// HIPAR fastExport/control to re-activate it.
ViStatus _VI_FUNC rsspecan_ConfigureOscilloscopeDisplayUpdate(
                            ViSession instrSession,
                            ViBoolean fastExport)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_B2000_OSCILLOSCOPE_DISPLAY_UPDATE, fastExport),
            2, "Fast Export");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function defines the TCPIP address or computer name of the
/// HIFN  oscilloscope connected to the R&S FSW via LAN.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR tcpipAddress/Defines the TCPIP address or computer name of the oscilloscope
/// HIPAR tcpipAddress/connected to the R&S FSW via LAN.
ViStatus _VI_FUNC rsspecan_ConfigureOscilloscopeTCPIPAddress(ViSession instrSession,
                                                             ViString  tcpipAddress)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_OSCILLOSCOPE_TCPIP_ADDRESS, tcpipAddress),
            2, "TCPIP Address");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function returns the state of the LAN connection to the
/// HIFN  oscilloscope for the optional 2 GHz bandwidth extension (R&S
/// HIFN  FSW-B2000).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR lanConnectionState/Returns the state of the LAN connection to the oscilloscope for the
/// HIPAR lanConnectionState/optional 2 GHz bandwidth extension (R&S FSW-B2000).
ViStatus _VI_FUNC rsspecan_QueryOscilloscopeLANConnectionState(
                            ViSession instrSession,
                            ViInt32   *lanConnectionState)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_GetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_OSCILLOSCOPE_LAN_CONNECTION_STATE, lanConnectionState),
            2, "LAN Connection State");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function returns the information of the oscilloscope.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR bufferSize/Pass the number of bytes in the string you specify for the ID
/// HIPAR bufferSize/parameter.
/// HIPAR id/Returns the identification string of the oscilloscope connected to the
/// HIPAR id/R&S FSW.
/// HIPAR supportedInstrument/Queries whether the connected instrument is supported by the 2 GHz
/// HIPAR supportedInstrument/bandwidth extension option(R&S FSW-B2000).
/// HIPAR supportedFirmware/Queries whether the firmware on the connected oscilloscope is
/// HIPAR supportedFirmware/supported by the 2 GHz bandwidth extension (R&S FSW-B2000) option.
ViStatus _VI_FUNC rsspecan_QueryOscilloscopeInfo(ViSession instrSession,
                                                 ViInt32   bufferSize,
                                                 ViChar    id[],
                                                 ViBoolean *supportedInstrument,
                                                 ViBoolean *supportedFirmware)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_GetAttributeViString(instrSession, "", RSSPECAN_ATTR_OSCILLOSCOPE_ID,
            bufferSize, id),
            3, "ID");

    viCheckParm(rsspecan_GetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_OSCILLOSCOPE_SUPPORTED_INSTRUMENT, supportedInstrument),
            4, "Supported Instrument");

    viCheckParm(rsspecan_GetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_OSCILLOSCOPE_SUPPORTED_FIRMWARE, supportedFirmware),
            5, "Supported Firmware");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function performs the alignment of the oscilloscope itself and
/// HIFN  the oscilloscope ADC for the optional 2 GHz bandwidth extension (R&S
/// HIFN  FSW-B2000). The correction data for the oscilloscope (including the
/// HIFN  connection cable between the R&S FSW and the oscilloscope) is recorded.
/// HIFN  As a result, the state of the alignment is returned. Alignment is
/// HIFN  required only once after setup. If alignment was performed
/// HIFN  successfully, the alignment data is stored on the oscilloscope.
/// HIFN     Thus, alignment need only be repeated if one of the following applies:
/// HIFN     - A new oscilloscope is connected to the IF OUT 2 GHZ connector of the
/// HIFN  R&S FSW
/// HIFN     - A new cable is used between the IF OUT 2 GHZ connector of the R&S
/// HIFN  FSW and the oscilloscope
/// HIFN     - A new firmware is installed on the oscilloscope
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR timeout/This control sets the timeout value.
/// HIPAR alignmentState/Returns the state of the alignment.
ViStatus _VI_FUNC rsspecan_OscilloscopeAlignment(ViSession instrSession,
                                                 ViUInt32  timeout,
                                                 ViBoolean *alignmentState)
{
    ViStatus error = VI_SUCCESS;
    ViUInt32 old_timeout = 0;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsspecan_GetTimeout(instrSession, &old_timeout));
    checkErr(rsspecan_SetTimeout(instrSession, timeout));

    viCheckParm(rsspecan_GetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_OSCILLOSCOPE_ALIGNMENT, alignmentState),
            3, "Alignment State");

Error:
    if (old_timeout > 0)
        rsspecan_SetTimeout(instrSession, old_timeout);
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Performs a self-alignment on the oscilloscope before the B2000/B5000
/// HIFN  alignment on the instrument.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR selfAlignmentEnabled/Performs a self-alignment on the oscilloscope before the B2000/B5000
/// HIPAR selfAlignmentEnabled/alignment on the instrument.
ViStatus _VI_FUNC rsspecan_ConfigureOscilloscopeSelfAlignmentEnabled(
                            ViSession instrSession,
                            ViBoolean selfAlignmentEnabled)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, NULL, RSSPECAN_ATTR_OSCILLOSCOPE_SELF_ALIGNMENT_ENABLED, selfAlignmentEnabled),
            2, "Self-Alignment Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function returns the date of alignment of the IF OUT 2 GHZ to the
/// HIFN  oscilloscope for the optional 2 GHz bandwidth extension (R&S
/// HIFN  FSW-B2000).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR bufferSize/Pass the number of bytes in the string you specify for the Alignment
/// HIPAR bufferSize/Date parameter.
/// HIPAR alignmentDate/Returns the date of alignment of the IF OUT 2 GHZ to the oscilloscope
/// HIPAR alignmentDate/for the optional 2 GHz bandwidth extension (R&S FSW-B2000).
ViStatus _VI_FUNC rsspecan_QueryOscilloscopeAlignmentDate(ViSession instrSession,
                                                          ViInt32   bufferSize,
                                                          ViChar    alignmentDate[])
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_GetAttributeViString(instrSession, "", RSSPECAN_ATTR_OSCILLOSCOPE_ALIGNMENT_DATE,
            bufferSize, alignmentDate),
            3, "Alignment Date");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Determines whether the 10 GHz mode (default) or 20 GHz mode of the
/// HIFN  connected oscilloscope is used. The 20 GHZ mode achieves a higher
/// HIFN  decimation gain, but reduces the record length by half.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR sampleRate/Determines whether the 10 GHz mode (default) or 20 GHz mode of the
/// HIPAR sampleRate/connected oscilloscope is used. The 20 GHZ mode achieves a higher
/// HIPAR sampleRate/decimation gain, but reduces the record length by half.
ViStatus _VI_FUNC rsspecan_ConfigureOscilloscopeSampleRate(ViSession instrSession,
                                                           ViInt32   sampleRate)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_OSCILLOSCOPE_SAMPLE_RATE, sampleRate),
            2, "Sample Rate");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Activates the use of the power splitter inserted between the IF 2 GHZ
/// HIFN  OUT connector of the R&S FSW and the CH1 and CH3 input connectors of
/// HIFN  the oscilloscope. Note that this mode requires an additional alignment
/// HIFN  with the power splitter.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR powerSplitterMode/Activates the use of the power splitter inserted between the IF 2 GHZ
/// HIPAR powerSplitterMode/OUT connector of the R&S FSW and the CH1 and CH3 input connectors of
/// HIPAR powerSplitterMode/the oscilloscope. Note that this mode requires an additional alignment
/// HIPAR powerSplitterMode/with the power splitter.
ViStatus _VI_FUNC rsspecan_ConfigureOscilloscopePowerSplitterMode(
                            ViSession instrSession,
                            ViBoolean powerSplitterMode)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_OSCILLOSCOPE_POWER_SPLITTER_MODE, powerSplitterMode),
            2, "Power Splitter Mode");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/*===========================================================================*
 *------------------- MSRA --------------------------------------------------*
 *===========================================================================*/

/// HIFN  This function selects the MSRA mode.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsspecan_MSRAMode(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_MSRA_MODE, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function selects the RTMS mode.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsspecan_RTMSMode(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_RTMS_MODE, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function queries the analysis interval for the current window.
/// HIFN  This function is only available in application measurement channels,
/// HIFN  not the MSRA View or MSRA Master.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR intervalStart/Returns start value of the analysis interval.
/// HIPAR intervalStop/Returns stop value of the analysis interval.
ViStatus _VI_FUNC rsspecan_QueryMSRAAnalysisInterval(ViSession instrSession,
                                                     ViInt32   window,
                                                     ViReal64  *intervalStart,
                                                     ViReal64  *intervalStop)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CALC:MSRA:WIND%ld:IVAL?", window);
    checkErr(RsCore_QueryTupleViReal64(instrSession, cmd, intervalStart, intervalStop, NULL, NULL));

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function queries all analysis intervals for the current
/// HIFN  window. This function is only available in application
/// HIFN  measurement channels, not the MSRA View or MSRA Master.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR arraySize/Defines the number of values to be queried. The number should be
/// HIPAR arraySize/equal or higher than the sizes of array parameters.
/// HIPAR intervalStart/Returns an array of start values of all analysis intervals.
/// HIPAR intervalStop/Returns an array of stop values of all analysis intervals.
ViStatus _VI_FUNC rsspecan_QueryMSRAAllAnalysisInterval(ViSession instrSession,
                                                        ViInt32   window,
                                                        ViInt32   arraySize,
                                                        ViReal64  intervalStart[],
                                                        ViReal64  intervalStop[])
{
    ViStatus  error = VI_SUCCESS;
    ViChar    cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";
    ViAReal64 data = NULL;
    ViInt32   sourceArraySize = 0;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K10"));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CALC:MSRA:WIND%ld:MIV?", window);
    checkErr(RsCore_QueryFloatArray(instrSession, cmd, &data, &sourceArraySize));
    checkErr(RsCore_DecimateViReal64ArrayToUserBuffers(data, sourceArraySize, arraySize, NULL, intervalStart, intervalStop, NULL, NULL, NULL, NULL));

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    if (data) free(data);
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function is only available if the Sequencer is deactivated
/// HIFN  (SYSTem:SEQuencer OFF) and only for applications in MSRA mode, not the
/// HIFN  MSRA Master. The data in the capture buffer is re-evaluated by the
/// HIFN  currently active application only. The
/// HIFN     results for any other applications remain unchanged. The application
/// HIFN  channel must be selected before this function can be executed (with
/// HIFN  attribute RSSPECAN_ATTR_SELECT_MEASUREMENT_CHANNEL or function
/// HIFN  rsspecan_SelectMeasurementChannel).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
ViStatus _VI_FUNC rsspecan_MSRARefresh(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_MSRA_REFRESH, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function is only available for applications in MSRA mode, not for
/// HIFN  the MSRA Master. It has a similar effect as the trigger offset in other
/// HIFN  measurements.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR offset/This control defines the time offset between the capture buffer start
/// HIPAR offset/and the start of the extracted application data. The offset must be a
/// HIPAR offset/positive value, as the application can only analyze data that is
/// HIPAR offset/contained in the capture buffer.
ViStatus _VI_FUNC rsspecan_ConfigureMSRACaptureOffset(ViSession instrSession,
                                                      ViInt32   offset)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_MSRA_CAPTURE_OFFSET, offset),
            2, "Offset");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN This function shows/hides analysis line position and sets line position.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR showLine/This control hides or displays the analysis line in the time-based MSRA windows.
/// HIPAR position/This control defines the position of the analysis line for all
/// HIPAR position/time-based windows in all MSRA applications and the MSRA Master.
ViStatus _VI_FUNC rsspecan_ConfigureMSRAAnalysisLine(ViSession instrSession,
                                                     ViBoolean showLine,
                                                     ViReal64  position)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_MSRA_SHOW_ANALYSIS_LINE, showLine),
            2, "Show Line");

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_MSRA_ANALYSIS_LINE_POSITION, position),
            2, "Position");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function defines whether the captured I/Q data is evaluated
/// HIFN directly, or if it is converted (via FFT) to spectral or time
/// HIFN data first.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR dataEvaluation/Defines whether the captured I/Q data is evaluated directly, or
/// HIPAR dataEvaluation/if it is converted (via FFT) to spectral or time data first.
ViStatus _VI_FUNC rsspecan_ConfigureMSRADataEvaluation(ViSession instrSession,
                                                       ViInt32   dataEvaluation)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_MSRA_DATA_EVALUATION, dataEvaluation),
            2, "Data Evaluation");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function is only available for applications in RTMS mode, not for
/// HIFN  the RTMS Master. It has a similar effect as the trigger offset in other
/// HIFN  measurements.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR offset/This control defines the time offset between the capture buffer start
/// HIPAR offset/and the start of the extracted application data. The offset must be a
/// HIPAR offset/positive value, as the application can only analyze data that is
/// HIPAR offset/contained in the capture buffer.

ViStatus _VI_FUNC rsspecan_ConfigureRTMSCaptureOffset(ViSession instrSession,
                                                      ViInt32   offset)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_RTMS_CAPTURE_OFFSET, offset),
            2, "Offset");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function shows/hides analysis line position and sets line position.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR showLine/This control hides or displays the analysis line in the time-based RTMS windows.
/// HIPAR position/This control defines the position of the analysis line for all
/// HIPAR position/time-based windows in all MSRA applications and the RTMS Master.
ViStatus _VI_FUNC rsspecan_ConfigureRTMSAnalysisLine(ViSession instrSession,
                                                     ViBoolean showLine,
                                                     ViReal64  position)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_RTMS_SHOW_ANALYSIS_LINE, showLine),
            2, "Show Line");

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_RTMS_ANALYSIS_LINE_POSITION, position),
            2, "Position");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function queries the analysis interval for the current window.
/// HIFN  This function is only available in application measurement channels,
/// HIFN  not the RTMS View or RTMS Master.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR intervalStart/Returns start value of the analysis interval.
/// HIPAR intervalStop/Returns stop value of the analysis interval.
ViStatus _VI_FUNC rsspecan_QueryRTMSAnalysisInterval(ViSession instrSession,
                                                     ViInt32   window,
                                                     ViReal64  *intervalStart,
                                                     ViReal64  *intervalStop)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CALC:RTMS:WIND%ld:IVAL?", window);
    checkErr(RsCore_QueryTupleViReal64(instrSession, cmd, intervalStart, intervalStop, NULL, NULL));

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/*===========================================================================*
 *------------------- Data Management ---------------------------------------*
 *===========================================================================*/
/// HIFN This function initiates an instrument reset.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init
/// HIPAR instrSession/or rsspecan_InitWithOptions function.
ViStatus _VI_FUNC rsspecan_Preset(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_SYST_PRESET, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/*===========================================================================*
 *------------------- Hardcopy ----------------------------------------------*
 *===========================================================================*/
/// HIFN This function selects the printer output medium and the data
/// HIFN format of the printout.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init
/// HIPAR instrSession/or rsspecan_InitWithOptions function.
/// HIPAR device/Sets the device number.
/// HIPAR destination/Defines the output associated with device configuration.
/// HIPAR pageOrientation/The command selects the format of the output (portrait and
/// HIPAR pageOrientation/landscape) (hardcopy unit 1 or 2).
ViStatus _VI_FUNC rsspecan_ConfigureHardcopyDevice(ViSession instrSession,
                                                   ViInt32   device,
                                                   ViInt32   destination,
                                                   ViInt32   pageOrientation)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_HCOPY_DEVICE_DESTINATION,
            "Destination", device - 1, RS_REPCAP_BUF_SIZE, repCap));
    switch (destination)
    {
    case RSSPECAN_VAL_HCOPY_DEST_BMP:
        checkErr(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_HCOPY_DEVICE_DESTINATION, RSSPECAN_VAL_HCOPY_DEVICE_MEM));
        checkErr(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_HCOPY_DEVICE_LANG_OUT_FORM, RSSPECAN_VAL_HCOPY_DEVICE_LANG_BMP));
        break;
    case RSSPECAN_VAL_HCOPY_DEST_WMF:
        checkErr(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_HCOPY_DEVICE_DESTINATION, RSSPECAN_VAL_HCOPY_DEVICE_MEM));
        checkErr(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_HCOPY_DEVICE_LANG_OUT_FORM, RSSPECAN_VAL_HCOPY_DEVICE_LANG_WMF));
        break;
    case RSSPECAN_VAL_HCOPY_DEST_EMF:
        checkErr(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_HCOPY_DEVICE_DESTINATION, RSSPECAN_VAL_HCOPY_DEVICE_MEM));
        checkErr(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_HCOPY_DEVICE_LANG_OUT_FORM, RSSPECAN_VAL_HCOPY_DEVICE_LANG_EWMF));
        break;
    case RSSPECAN_VAL_HCOPY_DEST_CLP:
        checkErr(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_HCOPY_DEVICE_DESTINATION, RSSPECAN_VAL_HCOPY_DEVICE_CLP));
        break;
    case RSSPECAN_VAL_HCOPY_DEST_PRN:

        checkErr(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_HCOPY_DEVICE_LANG_OUT_FORM, RSSPECAN_VAL_HCOPY_DEVICE_LANG_GDI));
        checkErr(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_HCOPY_DEVICE_DESTINATION, RSSPECAN_VAL_HCOPY_DEVICE_PRN));
        viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_HCOPY_DEVICE_ORIENTATION, pageOrientation),
                4, "Page Orientation");
        break;
    case RSSPECAN_VAL_HCOPY_DEST_JPG:
        checkErr(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_HCOPY_DEVICE_DESTINATION, RSSPECAN_VAL_HCOPY_DEVICE_MEM));
        checkErr(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_HCOPY_DEVICE_LANG_OUT_FORM, RSSPECAN_VAL_HCOPY_DEVICE_LANG_JPG));
        break;
    case RSSPECAN_VAL_HCOPY_DEST_PNG:
        checkErr(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_HCOPY_DEVICE_DESTINATION, RSSPECAN_VAL_HCOPY_DEVICE_MEM));
        checkErr(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_HCOPY_DEVICE_LANG_OUT_FORM, RSSPECAN_VAL_HCOPY_DEVICE_LANG_PNG));
        break;

    case RSSPECAN_VAL_HCOPY_DEST_PDF:
        checkErr(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_HCOPY_DEVICE_DESTINATION, RSSPECAN_VAL_HCOPY_DEVICE_MEM));
        checkErr(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_HCOPY_DEVICE_LANG_OUT_FORM, RSSPECAN_VAL_HCOPY_DEVICE_LANG_PDF));
        break;

    case RSSPECAN_VAL_HCOPY_DEST_SVG:
        checkErr(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_HCOPY_DEVICE_DESTINATION, RSSPECAN_VAL_HCOPY_DEVICE_MEM));
        checkErr(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_HCOPY_DEVICE_LANG_OUT_FORM, RSSPECAN_VAL_HCOPY_DEVICE_LANG_SVG));
        break;

    case RSSPECAN_VAL_HCOPY_DEST_DOC:
        checkErr(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_HCOPY_DEVICE_DESTINATION, RSSPECAN_VAL_HCOPY_DEVICE_MEM));
        checkErr(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_HCOPY_DEVICE_LANG_OUT_FORM, RSSPECAN_VAL_HCOPY_DEVICE_LANG_DOC));
        break;

    case RSSPECAN_VAL_HCOPY_DEST_RTF:
        checkErr(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_HCOPY_DEVICE_DESTINATION, RSSPECAN_VAL_HCOPY_DEVICE_MEM));
        checkErr(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_HCOPY_DEVICE_LANG_OUT_FORM, RSSPECAN_VAL_HCOPY_DEVICE_LANG_RTF));
        break;

    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, destination),
                3, "Destination");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function configures the printout page margins.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init
/// HIPAR instrSession/or rsspecan_InitWithOptions function.
/// HIPAR device/Sets the device number.
/// HIPAR state/Enables or disables if the printing operation of the page.
/// HIPAR top/Defines the margin at the top of the printout page on which no elements are printed.
/// HIPAR bottom/Defines the margin at the bottom of the printout page on which no elements are printed.
/// HIPAR left/Defines the margin at the left side of the printout page on which no elements are printed.
/// HIPAR right/Defines the margin at the right side of the printout page on which no elements are printed.
/// HIPAR marginUnit/Selects defines the unit in which the margins for the printout page are configured.
ViStatus _VI_FUNC rsspecan_ConfigureHardcopyPrintMarginsSettings(
                            ViSession instrSession,
                            ViInt32   device,
                            ViBoolean state,
                            ViReal64  top,
                            ViReal64  bottom,
                            ViReal64  left,
                            ViReal64  right,
                            ViInt32   marginUnit)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_HCOPY_PAGE_MARGIN_TOP,
            "Destination", device - 1, RS_REPCAP_BUF_SIZE, repCap));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_HCOPY_PAGE_PRINTING_STATE, state),
            3, "State");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_HCOPY_PAGE_MARGIN_UNIT, marginUnit),
            8, "Margin Unit");

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_HCOPY_PAGE_MARGIN_TOP, top),
            4, "Top");

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_HCOPY_PAGE_MARGIN_BOTTOM, bottom),
            5, "Bottom");

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_HCOPY_PAGE_MARGIN_LEFT, left),
            6, "Left");

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_HCOPY_PAGE_MARGIN_RIGHT, right),
            7, "Right");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the windows settings.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init
/// HIPAR instrSession/or rsspecan_InitWithOptions function.
/// HIPAR device/Sets the device number.
/// HIPAR channelName/Sets string containing the name of the channel.
/// HIPAR state/Enables or disables the selection of all windows of the specified channel to be included in the printout for HCOP:CONT WIND.
/// HIPAR windowsPerPage/Sets how many windows are displayed on a single page of the printout for HCOP:CONT WIND.
/// HIPAR scaling/Enables or disables the scaling of the windows in the printout for HCOP:CONTWIND.
ViStatus _VI_FUNC rsspecan_ConfigureHardcopyWindowsSettings(ViSession instrSession,
                                                            ViInt32   device,
                                                            ViString  channelName,
                                                            ViBoolean state,
                                                            ViInt32   windowsPerPage,
                                                            ViBoolean scaling)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "HCOP:PAGE:WIND%ld:CHAN:STAT '%s', %s", device, channelName, state ? "ON" : "OFF");
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsspecan_CheckStatus(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_HCOPY_WINDOWS_COUNT,
            "Destination", device - 1, RS_REPCAP_BUF_SIZE, repCap));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_HCOPY_WINDOWS_COUNT, windowsPerPage),
            5, "Windows per Page");

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, repCap, RSSPECAN_ATTR_HCOPY_WINDOWS_SCALE, scaling),
            6, "Scaling");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function aborts a running hardcopy output.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init
/// HIPAR instrSession/or rsspecan_InitWithOptions function.
ViStatus _VI_FUNC rsspecan_HardcopyAbort(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_HCOPY_ABORT, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN Determines the type of content included in the printout. This setting is independent of the printing device.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init
/// HIPAR instrSession/or rsspecan_InitWithOptions function.
/// HIPAR content/Selects the type of content included in the printout.
ViStatus _VI_FUNC rsspecan_HardcopyContent(ViSession instrSession,
                                           ViInt32   content)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_HCOPY_CONTENT, content),
            2, "Content");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function starts a hardcopy output of the selected items.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init
/// HIPAR instrSession/or rsspecan_InitWithOptions function.
/// HIPAR device/Sets the device number.
/// HIPAR items/Selects the items to be output.
ViStatus _VI_FUNC rsspecan_HardcopyPrint(ViSession instrSession,
                                         ViInt32   device,
                                         ViInt32   items)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_HCOPY_PRINT,
            "Destination", device - 1, RS_REPCAP_BUF_SIZE, repCap));
    switch (items)
    {
    case RSSPECAN_VAL_HCOP_ALL:
        checkErr(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_HCOPY_PRINT_SCREEN, NULL));
        break;
    case RSSPECAN_VAL_HCOP_TABLE:
        checkErr(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_HCOPY_PRINT_TAB, VI_TRUE));
        break;
    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, items),
                3, "Items");
    }
    checkErr(rsspecan_SetAttributeViString(instrSession, repCap, RSSPECAN_ATTR_HCOPY_PRINT, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function starts a hardcopy output.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init
/// HIPAR instrSession/or rsspecan_InitWithOptions function.
/// HIPAR device/Sets the device number.
ViStatus _VI_FUNC rsspecan_HardcopyPrintNext(ViSession instrSession,
                                             ViInt32   device)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_HCOPY_PRINT_NEXT,
            "Destination", device - 1, RS_REPCAP_BUF_SIZE, repCap));

    checkErr(rsspecan_SetAttributeViString(instrSession, repCap, RSSPECAN_ATTR_HCOPY_PRINT_NEXT, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function defines a destination file for the hadcopy output.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init
/// HIPAR instrSession/or rsspecan_InitWithOptions function.
/// HIPAR name/Sets the hardcopy file name.
ViStatus _VI_FUNC rsspecan_HardcopySetFileName(ViSession instrSession,
                                               ViString  name)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_HCOPY_FILE_NAME, name));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function queries the list of printers.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR bufferSize/This control indicates the allocated memory in Printer List.
/// HIPAR printerList/This control returns list of printers.
ViStatus _VI_FUNC rsspecan_HardcopyGetPrinterList(ViSession instrSession,
                                                  ViInt32   bufferSize,
                                                  ViChar    printerList[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   response[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32RangeMin(instrSession, bufferSize, 1),
        2, "Buffer Size");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, printerList),
        3, "Printer List");

    checkErr(rsspecan_GetAttributeViString(instrSession, "", RSSPECAN_ATTR_HCOPY_PRINTER_FIRST,
                RS_MAX_MESSAGE_BUF_SIZE, response));

    snprintf(printerList, bufferSize, "%s", response);
    while (strlen(response) > 1)
    {
        checkErr(rsspecan_GetAttributeViString(instrSession, "", RSSPECAN_ATTR_HCOPY_PRINTER_NEXT,
            RS_MAX_MESSAGE_BUF_SIZE, response));

        if (strlen(response) > 1)
        {
            RsCore_StrcatMaxLen(printerList, bufferSize, ",", response, NULL);
        }
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN This function selects one of the installed printers
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR device/Sets the device number.
/// HIPAR printerName/This control sets printer name.
ViStatus _VI_FUNC rsspecan_HardcopySetPrinter(ViSession instrSession,
                                              ViInt32   device,
                                              ViString  printerName)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_HCOPY_PRINTER,
            "Destination", device - 1, RS_REPCAP_BUF_SIZE, repCap));

    viCheckParm(rsspecan_SetAttributeViString(instrSession, repCap, RSSPECAN_ATTR_HCOPY_PRINTER, printerName),
            3, "Printer Name");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function defines the comment text for measurement window.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR instrSession/Document parameter here.
/// HIPAR comment/Sets the hardcopy comment.
ViStatus _VI_FUNC rsspecan_HardcopyComment(ViSession instrSession,
                                           ViString  comment)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_HCOPY_COMM_SCR, comment));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function includes or removes the time and date in the printout.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR device/Sets the device number.
/// HIPAR time_andDateStamp/Includes or removes the time and date in the printout.
ViStatus _VI_FUNC rsspecan_HardcopyTimeDate(ViSession instrSession,
                                            ViInt32   device,
                                            ViBoolean timeAndDateStamp)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "DE%ld", device);
    checkErr(rsspecan_SetAttributeViBoolean(instrSession, repCap, RSSPECAN_ATTR_HCOPY_TIME_DATE, timeAndDateStamp));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function selects all windows of the specified channel to be
/// HIFN  included in the printout for HCOP:CONT WIND.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR device/Sets the device number.
/// HIPAR channelName/String containing the name of the channel.
/// HIPAR includeWindows/The channel windows printout state.
ViStatus _VI_FUNC rsspecan_HardcopyPageIncludeAllWindows(ViSession instrSession,
                                                         ViInt32   device,
                                                         ViString  channelName,
                                                         ViBoolean includeWindows)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, device, 1, 2),
            2, "Device");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "HCOP:PAGE:WIND%ld:CHAN:STAT '%s',%s", device, channelName, includeWindows ? "ON" : "OFF");
    checkErr(RsCore_Write(instrSession, cmd));
    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function selects the windows to be included in the printout for
/// HIFN  HCOP:CONT WIND.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR device/Sets the device number.
/// HIPAR channelName/String containing the name of the channel.
/// HIPAR windowName/String containing the name of the existing window.
/// HIPAR includeWindows/The channel windows printout state.
ViStatus _VI_FUNC rsspecan_HardcopyPageIncludeSelectedWindow(ViSession instrSession,
                                                             ViInt32   device,
                                                             ViString  channelName,
                                                             ViString  windowName,
                                                             ViBoolean includeWindows)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, device, 1, 2),
            2, "Device");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "HCOP:PAGE:WIND%ld:STAT '%s','%s',%s", device, channelName, windowName, includeWindows ? "ON" : "OFF");
    checkErr(RsCore_Write(instrSession, cmd));
    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects the print mode, screen or report.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR printMode/Selects the print mode, screen or report.
ViStatus _VI_FUNC rsspecan_ConfigurePrintMode(ViSession instrSession,
                                              ViInt32   printMode)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, NULL, RSSPECAN_ATTR_PRINT_MODE, printMode),
            2, "Print Mode");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines general properties of the report document.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR pageFormat/Selects the size of the test report document.
/// HIPAR useScreenColors/Turns the use of printer friendly colors on and off.
/// HIPAR date/Includes or excludes date and time from the test report.
/// HIPAR pageCount/Includes or excludes page number from the test report.
ViStatus _VI_FUNC rsspecan_ConfigureTestReportsGeneralSettings(
                            ViSession instrSession,
                            ViInt32   pageFormat,
                            ViBoolean useScreenColors,
                            ViBoolean date,
                            ViBoolean pageCount)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, NULL, RSSPECAN_ATTR_TEST_REPORTS_PAGE_FORMAT, pageFormat),
            2, "Page Format");

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, NULL, RSSPECAN_ATTR_TEST_REPORTS_USE_SCREEN_COLORS, useScreenColors),
            3, "Use Screen Colors");

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, NULL, RSSPECAN_ATTR_TEST_REPORTS_DATE, date),
            4, "Date");

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, NULL, RSSPECAN_ATTR_TEST_REPORTS_PAGE_COUNT, pageCount),
            5, "Page Count");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Define an optional title page for the test report with a title and a
/// HIFN  short description of the report contents. It is only included in the
/// HIFN  report if you enable it.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR titlePage/Includes or excludes the title page from the test report.
/// HIPAR title/Defines the title for the test report as shown on its title page.
/// HIPAR abstract/Defines the description of the test report as shown on its title page.
ViStatus _VI_FUNC rsspecan_ConfigureTestReportsTitlePage(ViSession instrSession,
                                                         ViBoolean titlePage,
                                                         ViString  title,
                                                         ViString  abstract)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, NULL, RSSPECAN_ATTR_TEST_REPORTS_TITLE_PAGE, titlePage),
            2, "Title Page");

    viCheckParm(rsspecan_SetAttributeViString(instrSession, NULL, RSSPECAN_ATTR_TEST_REPORTS_TITLE, title),
            3, "Title");

    viCheckParm(rsspecan_SetAttributeViString(instrSession, NULL, RSSPECAN_ATTR_TEST_REPORTS_ABSTRACT, abstract),
            4, "Abstract");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Includes or excludes the complete set of measurement information from
/// HIFN  the test report.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR measurementInfoState/String containing the information you want to include in the test report.
ViStatus _VI_FUNC rsspecan_ConfigureTestReportsMeasurementInfoState(
                            ViSession instrSession,
                            ViBoolean measurementInfoState)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, NULL, RSSPECAN_ATTR_TEST_REPORTS_MEASUREMENT_INFO_STATE, measurementInfoState),
            2, "Measurement Info State");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Allows you to add user-defined information on the measurement to the
/// HIFN  report.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR line/Selects repeated capability TestReportLine.
/// HIPAR title/Defines a custom name for one of the items part of the report header.
/// HIPAR value/Defines a descriptive text for one of the items part of the report
/// HIPAR value/header.
/// HIPAR visibility/Selects how often the items in the report header are displayed in the
/// HIPAR visibility/document.
ViStatus _VI_FUNC rsspecan_ConfigureTestReportsMeasurementInfo(
                            ViSession instrSession,
                            ViInt32   line,
                            ViString  title,
                            ViString  value,
                            ViInt32   visibility)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_TEST_REPORTS_MEASUREMENT_INFO_TITLE,
            "TestReportLine", line - 1, RS_REPCAP_BUF_SIZE, repCap));

    viCheckParm(rsspecan_SetAttributeViString(instrSession, repCap, RSSPECAN_ATTR_TEST_REPORTS_MEASUREMENT_INFO_TITLE, title),
            3, "Title");

    viCheckParm(rsspecan_SetAttributeViString(instrSession, repCap, RSSPECAN_ATTR_TEST_REPORTS_MEASUREMENT_INFO_VALUE, value),
            4, "Value");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_TEST_REPORTS_MEASUREMENT_INFO_VISIBILITY, visibility),
            5, "Visibility");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects a graphic (for example a company logo) that is shown at the
/// HIFN  top of each page in the test report.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR logo/String containing the location and name of the picture
/// HIPAR logoVisibility/Selects how often the logo is displayed in the document.
ViStatus _VI_FUNC rsspecan_ConfigureTestReportsMeasurementInfoLogo(
                            ViSession instrSession,
                            ViString  logo,
                            ViInt32   logoVisibility)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViString(instrSession, NULL, RSSPECAN_ATTR_TEST_REPORTS_MEASUREMENT_INFO_LOGO, logo),
            2, "Logo");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, NULL, RSSPECAN_ATTR_TEST_REPORTS_MEASUREMENT_INFO_LOGO_VISIBILITY, logoVisibility),
            3, "Logo Visibility");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the type of information that a test report is made up out of.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR item/Defines the type of information that a test report is made up out of.
ViStatus _VI_FUNC rsspecan_ConfigureTestReportsItemSelect(ViSession instrSession,
                                                          ViString  item)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViString(instrSession, NULL, RSSPECAN_ATTR_TEST_REPORTS_ITEM_SELECT, item),
            2, "Item");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the selected information to be included in the test report for
/// HIFN  a specific channel type.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channelType/Selects the channel type that you want to query the test report
/// HIPAR channelType/configuration for. When you pass an empty string, the function returns
/// HIPAR channelType/the configuration of the currently selected channel.
/// HIPAR bufferSize/Size of output array.
/// HIPAR itemList/Queries the selected information to be included in the test report for
/// HIPAR itemList/a specific channel type.
/// HIPAR returnedValues/Returns the number of characters returned from the instrument.
ViStatus _VI_FUNC rsspecan_QueryTestReportsItemList(ViSession instrSession,
                                                    ViString  channelType,
                                                    ViInt32   bufferSize,
                                                    ViChar    itemList[],
                                                    ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeViStringWithQparam(instrSession, NULL,
                 RSSPECAN_ATTR_HIDDEN_TEST_REPORTS_ITEM_LIST,
                 channelType, bufferSize, itemList, returnedValues), 4, "Item List");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the test report templates available in the default report
/// HIFN  directory (and its subdirectories).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR bufferSize/Size of output array.
/// HIPAR templatesCatalog/Queries the test report templates available in the default report
/// HIPAR templatesCatalog/directory (and its subdirectories).
/// HIPAR returnedValues/Returns the number of characters returned from the instrument.
ViStatus _VI_FUNC rsspecan_QueryTestReportsTemplatesCatalog(ViSession instrSession,
                                                            ViInt32   bufferSize,
                                                            ViChar    templatesCatalog[],
                                                            ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeViStringWithRealLen(instrSession, NULL,
                 RSSPECAN_ATTR_TEST_REPORTS_TEMPLATES_CATALOG, RS_VAL_DIRECT_USER_CALL,
                 bufferSize, templatesCatalog, returnedValues), 3, "Templates Catalog");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Saves a test report template in XML format.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR templateName/String containing the name of the template. The .xml file extension is
/// HIPAR templateName/added automatically.
ViStatus _VI_FUNC rsspecan_SaveTestReportsTemplate(ViSession instrSession,
                                                   ViString  templateName)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViString(instrSession, NULL, RSSPECAN_ATTR_TEST_REPORTS_SAVE_TEMPLATE, templateName),
            2, "Template Name");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Loads a test report template.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR templateName/String containing the name of the template. The .xml file extension is
/// HIPAR templateName/added automatically.
ViStatus _VI_FUNC rsspecan_LoadTestReportsTemplate(ViSession instrSession,
                                                   ViString  templateName)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViString(instrSession, NULL, RSSPECAN_ATTR_TEST_REPORTS_LOAD_TEMPLATE, templateName),
            2, "Template Name");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Restores the default test report configuration.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsspecan_RestoreTestReportsDefaultConfiguration(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsspecan_SetAttributeViString(instrSession, NULL, RSSPECAN_ATTR_TEST_REPORTS_RESTORE_DEFAULT_CONFIGURATION, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Creates a new dataset for a new test report. Creating a new test
/// HIFN  report deletes all previously saved datasets. The current measurement
/// HIFN  results are added as the first dataset to the new report. To save the
/// HIFN  report, use RSSPECAN_ATTR_HCOPY_PRINT.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsspecan_NewTestReport(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsspecan_SetAttributeViString(instrSession, NULL, RSSPECAN_ATTR_TEST_REPORTS_NEW, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Adds the current measurement results to the test report.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsspecan_AppendTestReport(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsspecan_SetAttributeViString(instrSession, NULL, RSSPECAN_ATTR_TEST_REPORTS_APPEND, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Deletes one of the datasets that are currently part of a test report.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR dataset/Index number of the dataset as shown in the 'Test Report Content
/// HIPAR dataset/Selection' dialog box. If the index number is greater than the number
/// HIPAR dataset/of available datasets, the command returns an error.
ViStatus _VI_FUNC rsspecan_RemoveTestReportsDataset(ViSession instrSession,
                                                    ViInt32   dataset)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, NULL, RSSPECAN_ATTR_TEST_REPORTS_REMOVE_DATASET, dataset),
            2, "Dataset");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Removes all existing datasets from the test report.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsspecan_RemoveAllTestReportsDatasets(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsspecan_SetAttributeViString(instrSession, NULL, RSSPECAN_ATTR_TEST_REPORTS_REMOVE_ALL_DATASETS, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/*===========================================================================*
 *------------------- File --------------------------------------------------*
 *===========================================================================*/
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR decimalSeparator/This control sets the decimal separator for the ASCII data
/// HIPAR decimalSeparator/stored to the file.
ViStatus _VI_FUNC rsspecan_FileDecimalSeparator(ViSession instrSession,
                                                ViInt32   decimalSeparator)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_FILE_DEC_SEPARATOR, decimalSeparator),
            2, "Decimal Separator");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function provides commands which allow access to the
/// HIFN storage media of the instrument and for storing and loading
/// HIFN various instrument settings.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR operation/This control selects the type of memory operation to be
/// HIPAR operation/performed.
/// HIPAR source/This control specifies the source of the operation to be
/// HIPAR source/performed. If the operation requires a single parameter, this
/// HIPAR source/control specifies that parameter.
/// HIPAR destination/This control specifies the destination of the operation to be
/// HIPAR destination/performed.
ViStatus _VI_FUNC rsspecan_FileManagerOperations(ViSession instrSession,
                                                 ViInt32   operation,
                                                 ViString  source,
                                                 ViString  destination)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidNullPointer(instrSession, source),
        3, "Source");

    switch (operation)
    {
    case RSSPECAN_VAL_FILE_NEW:
        checkErr(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_FILE_MANAGER_MAKE_DIR, source));
        break;
    case RSSPECAN_VAL_FILE_COPY:
        viCheckParm(RsCore_InvalidNullPointer(instrSession, destination),
                4, "Destination");

        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, ":MMEM:COPY '%s','%s'", source, destination);
        checkErr(RsCore_Write(instrSession, cmd));
        break;
    case RSSPECAN_VAL_FILE_RENAME:
        viCheckParm(RsCore_InvalidNullPointer(instrSession, destination),
                4, "Destination");

        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, ":MMEM:MOVE '%s','%s'", source, destination);
        checkErr(RsCore_Write(instrSession, cmd));
        break;
    case RSSPECAN_VAL_FILE_DELETE:
        checkErr(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_FILE_MANAGER_DELETE, source));
        break;
    case RSSPECAN_VAL_FILE_RDIR:
        checkErr(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_FILE_MANAGER_DELETE_DIR, source));
        break;
    case RSSPECAN_VAL_FILE_CDIR:
        checkErr(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_FILE_MANAGER_EDIT_PATH, source));
        break;
    case RSSPECAN_VAL_FILE_CDISC:
        checkErr(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_FILE_MANAGER_EDIT_PATH_DEVICE, source));
        break;
    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, operation), 2, "Operation");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function reads the indicated directory.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR directory/This control specifies the directory in which the command will
/// HIPAR directory/be performed.
/// HIPAR bufferSize/Pass the number of bytes in the ViChar array you specify for the
/// HIPAR bufferSize/Output parameter.
/// HIPAR output/This indicator returns the files in the indicated directory as
/// HIPAR output/comma separated strings.
ViStatus _VI_FUNC rsspecan_FileDirectory(ViSession instrSession,
                                         ViString  directory,
                                         ViInt32   bufferSize,
                                         ViChar    output[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidNullPointer(instrSession, directory),
        2, "Directory");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "MMEM:CAT? '%s'", directory);
    checkErr(RsCore_QueryViStringUnknownLengthToUserBuffer(instrSession, cmd, bufferSize, output, NULL));

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function queries the directories and files in the given path.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR directory/This control specifies the directory in which the command will
/// HIPAR directory/be performed.
/// HIPAR bufferSize/Pass the number of bytes in the ViChar array you specify for the
/// HIPAR bufferSize/Output parameter.
/// HIPAR output/This indicator returns the file/directory information in the
/// HIPAR output/indicated directory as comma separated strings
ViStatus _VI_FUNC rsspecan_FileDirectoryPath(ViSession instrSession,
                                             ViString  directory,
                                             ViInt32   bufferSize,
                                             ViChar    output[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidNullPointer(instrSession, directory),
        2, "Directory");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "MMEM:CAT:LONG? '%s'", directory);
    checkErr(RsCore_QueryViStringUnknownLengthToUserBuffer(instrSession, cmd, bufferSize, output, NULL));

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Exports the signal configuration to the microservice.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR file/Exports the signal configuration to the microservice.
ViStatus _VI_FUNC rsspecan_FileExportMicroservice(ViSession instrSession,
                                                  ViString  file)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_FILE_EXPORT_TO_MICROSERVICE, file),
            2, "File");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/*===========================================================================*
 *------------------- Save/Recall -------------------------------------------*
 *===========================================================================*/
/// HIFN This function allows to provide data set operation for Save/Recall
/// HIFN items of memory subsystem.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR operation/This control selects the type of memory operation to be
/// HIPAR operation/performed.
/// HIPAR path/This control specifies the source of the operation to be
/// HIPAR path/performed.
ViStatus _VI_FUNC rsspecan_DataSetFileOperations(ViSession instrSession,
                                                 ViInt32   operation,
                                                 ViString  path)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    switch (operation)
    {
    case RSSPECAN_VAL_MEM_SAVE:
        viCheckParm(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_FILE_SAVE, path),
                2, "Path");
        break;
    case RSSPECAN_VAL_MEM_RECALL:
        viCheckParm(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_FILE_RECALL, path),
                2, "Path");
        break;
    case RSSPECAN_VAL_MEM_DELETE:
        viCheckParm(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_FILE_DATA_CLEAR, path),
                2, "Path");
        break;
    case RSSPECAN_VAL_MEM_STARTUP:
        viCheckParm(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_FILE_STARTUP_RECALL, path),
                2, "Path");
        break;
    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, operation), 2, "Operation");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function stores the current device settings in a *.FSL.dfl
/// HIFN file.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
ViStatus _VI_FUNC rsspecan_DataSetFileSaveNext(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_FILE_SAVE_NEXT, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function defines a comment ( max. 60 characters) for a device
/// HIFN setting to be stored.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR comment/This control specifies the file comment.
ViStatus _VI_FUNC rsspecan_DataSetFileComment(ViSession instrSession,
                                              ViString  comment)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_FILE_EDIT_COMM, comment),
            2, "Comment");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function defines a comment ( max. 60 characters) for a device
/// HIFN setting to be stored.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR directory/This control specifies the directory.
ViStatus _VI_FUNC rsspecan_DataSetFileClearAll(ViSession instrSession,
                                               ViString  directory)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_FILE_MANAGER_EDIT_PATH, directory),
            2, "Directory");
    checkErr(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_FILE_DATA_CLEAR_ALL, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN This function add/remove settings to/from the list of partial
/// HIFN datasets of a device setting to be stored/loaded.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR itemSelector/This control selects the items to save or recall.
/// HIPAR itemState/This control sets if selected item is active or not for to
/// HIPAR itemState/save/recall.
ViStatus _VI_FUNC rsspecan_DataSetFileSelectItems(ViSession instrSession,
                                                  ViInt32   itemSelector,
                                                  ViBoolean itemState)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    switch (itemSelector)
    {
    case RSSPECAN_VAL_FILE_ITEMS_NONE:
        checkErr(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_FILE_ITEMS_SAVE_RECAL_NONE, NULL));
        break;
    case RSSPECAN_VAL_FILE_ITEMS_DEFAULT:
        checkErr(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_FILE_ITEMS_SAVE_RECAL_DEFAULT, NULL));
        break;
    case RSSPECAN_VAL_FILE_ITEMS_ALL:
        checkErr(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_FILE_ITEMS_SAVE_RECAL_ALL, NULL));
        break;
    case RSSPECAN_VAL_FILE_ITEMS_HWSETTINGS:
        viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_FILE_ITEMS_SAVE_RECAL_SEL_HWSETTINGS, itemState),
                3, "Item State");
        break;
    case RSSPECAN_VAL_FILE_ITEMS_ALLTRACES:
        viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_FILE_ITEMS_SAVE_RECAL_SEL_TRAC, itemState),
                3, "Item State");
        break;
    case RSSPECAN_VAL_FILE_ITEMS_ALLLINES:
        viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_FILE_ITEMS_SAVE_RECAL_SEL_LINE, itemState),
                3, "Item State");
        break;
    case RSSPECAN_VAL_FILE_ITEMS_ALLTRAN:
        viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_FILE_ITEMS_SAVE_RECAL_SEL_TRAN, itemState),
                3, "Item State");
        break;
    case RSSPECAN_VAL_FILE_ITEMS_IQ_DATA:
        viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_FILE_ITEMS_SAVE_RECAL_INCLUDE_IQ_DATA, itemState),
                3, "Item State");
        break;
    case RSSPECAN_VAL_FILE_ITEMS_WEIGHTING:
        viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_FILE_ITEMS_SAVE_RECAL_WEIGHTING, itemState),
                3, "Item State");
        break;
    case RSSPECAN_VAL_FILE_ITEMS_CALIBRATION_SOURCE:
        viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_FILE_ITEMS_TO_SAVE_RECALL_SOURCE_CALIBRATION, itemState),
                3, "Item State");
        break;

    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, itemSelector), 2, "Item Selector");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function add/remove settings to/from the list of partial datasets
/// HIFN of a currently selected channel setting to be stored/loaded.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR itemSelector/This control selects the items to save or recall.
/// HIPAR itemState/This control sets if selected item is active or not for to save/recall
ViStatus _VI_FUNC rsspecan_DataSetFileSelectItemsFromChannel(ViSession instrSession,
                                                             ViInt32   itemSelector,
                                                             ViBoolean itemState)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    switch (itemSelector)
    {
    case RSSPECAN_VAL_FILE_ITEMS_NONE:
        checkErr(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_FILE_ITEMS_SAVE_RECAL_CHANNEL_NONE, NULL));
        break;
    case RSSPECAN_VAL_FILE_ITEMS_DEFAULT:
        checkErr(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_FILE_ITEMS_SAVE_RECAL_CHANNEL_DEFAULT, NULL));
        break;
    case RSSPECAN_VAL_FILE_ITEMS_ALL:
        checkErr(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_FILE_ITEMS_SAVE_RECAL_CHANNEL_ALL, NULL));
        break;
    case RSSPECAN_VAL_FILE_ITEMS_HWSETTINGS:
        viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_FILE_ITEMS_SAVE_RECAL_CHANNEL_SEL_HWSETTINGS, itemState),
                3, "Item State");
        break;
    case RSSPECAN_VAL_FILE_ITEMS_ALLTRACES:
        viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_FILE_ITEMS_SAVE_RECAL_CHANNEL_SEL_TRAC, itemState),
                3, "Item State");
        break;
    case RSSPECAN_VAL_FILE_ITEMS_ALLLINES:
        viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_FILE_ITEMS_SAVE_RECAL_CHANNEL_SEL_LINE, itemState),
                3, "Item State");
        break;
    case RSSPECAN_VAL_FILE_ITEMS_ALLTRAN:
        viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_FILE_ITEMS_SAVE_RECAL_CHANNEL_SEL_TRAN, itemState),
                3, "Item State");
        break;
    case RSSPECAN_VAL_FILE_ITEMS_SPECTROGRAM:
        viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_FILE_ITEMS_SAVE_RECAL_CHANNEL_SPEM, itemState),
                3, "Item State");
        break;
    case RSSPECAN_VAL_FILE_ITEMS_WEIGHTING:
        viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_FILE_ITEMS_SAVE_RECAL_CHANNEL_WEIGHTING, itemState),
                3, "Item State");
        break;
    case RSSPECAN_VAL_FILE_ITEMS_CALIBRATION_SOURCE:
        viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_FILE_ITEMS_TO_SAVE_RECALL_CHANNEL_SOURCE_CALIBRATION, itemState),
                3, "Item State");
        break;
    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, itemSelector), 2, "Item Selector");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function defines whether the channel-specific settings that will be loaded with the
/// HIFN rsspecan_DataSetFileOperations function will replace the current channel or activate a new channel.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR mode/This control selects the load mode.
ViStatus _VI_FUNC rsspecan_DataSetLoadMode(ViSession instrSession,
                                           ViInt32   mode)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_DATA_SET_LOAD_MODE, mode),
            2, "Mode");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function defines whether the data from the entire instrument
/// HIFN or only from the current channel is stored.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR mode/This control selects the store mode.
ViStatus _VI_FUNC rsspecan_DataSetStoreMode(ViSession instrSession,
                                            ViInt32   mode)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_DATA_SET_STORE_MODE, mode),
            2, "Mode");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/*===========================================================================*
 *------------------- Network -----------------------------------------------*
 *===========================================================================*/
/// HIFN This function maps a drive to a server or server folder of the
/// HIFN network.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR driveLetter/This control sets the drive letter.
/// HIPAR hostNameIPAddress/This control sets the host name or IP address and shared folder.
/// HIPAR useOptionalParameters/This control sets whether the user name, password and reconnect
/// HIPAR useOptionalParameters/at logon controls are used.
/// HIPAR userName/This control sets the user name.
/// HIPAR password/This control sets the password.
/// HIPAR reconnectAtLogon/This control sets the whether tha mapped drive should be
/// HIPAR reconnectAtLogon/reconnected at the logon time.
ViStatus _VI_FUNC rsspecan_NetworkMapDrive(ViSession instrSession,
                                           ViString  driveLetter,
                                           ViString  hostNameIPAddress,
                                           ViBoolean useOptionalParameters,
                                           ViString  userName,
                                           ViString  password,
                                           ViBoolean reconnectAtLogon)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    if (useOptionalParameters)
    {
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "MMEM:NETW:MAP '%s','%s','%s','%s',%s",
                 driveLetter, hostNameIPAddress, userName, password, reconnectAtLogon ? "ON" : "OFF");
    }
    else
    {
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "MMEM:NETW:MAP '%s','%s'", driveLetter, hostNameIPAddress);
    }

    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function disconnects the selected drive.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR driveLetter/This control sets the drive letter to be disconnected.
ViStatus _VI_FUNC rsspecan_NetworkDisconnectDrive(ViSession instrSession,
                                                  ViString  driveLetter)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_NETWORK_DISCONNECT_DRIVE, driveLetter),
            2, "Drive Letter");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function lists all unused network drive names.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR arraySize/This control defines the size of Unused Drives array.
/// HIPAR unusedDrives/This control lists all unused network drive names.
ViStatus _VI_FUNC rsspecan_NetworkUnusedDrivesList(ViSession instrSession,
                                                   ViInt32   arraySize,
                                                   ViChar    unusedDrives[])
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_GetAttributeViString(instrSession, "", RSSPECAN_ATTR_UNUSED_NETWORK_DRIVES,
            arraySize, unusedDrives),
            3, "Unused Drives");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function lists all mapped network drives.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR arraySize/This control defines the size of Used Drives.
/// HIPAR usedDrives/This control lists all mapped network drives.
ViStatus _VI_FUNC rsspecan_NetworkUsedDrivesList(ViSession instrSession,
                                                 ViInt32   arraySize,
                                                 ViChar    usedDrives[])
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_GetAttributeViString(instrSession, "", RSSPECAN_ATTR_USED_NETWORK_DRIVES,
            arraySize, usedDrives),
            3, "Used Drives");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/*===========================================================================*
 *------------------- Utility -----------------------------------------------*
 *===========================================================================*/
/// HIFN This function resets the instrument.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsspecan_reset(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    if (RsCore_Simulating(instrSession) == VI_FALSE)
    {
        checkErr(rsspecan_ClearStatus(instrSession));
        checkErr(RsCore_Write(instrSession, "*RST")); // IGNORE: Check double write
        checkErr(RsCore_QueryViStringShort(instrSession, "*OPC?", NULL));
    }

    checkErr(rsspecan_DefaultInstrSetup(instrSession));
    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);

    return error;
}

/// HIFN This function switches ON/OFF various error checkings performed inside of the driver.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR optionChecking/This control switches option checking On or Off.
/// HIPAR rangeChecking/This control enables or disables range checking.
/// HIPAR statusChecking/This control enables or disables instrument state checking
ViStatus _VI_FUNC rsspecan_ConfigureErrorChecking(ViSession instrSession,
                                                  ViBoolean optionChecking,
                                                  ViBoolean rangeChecking,
                                                  ViBoolean statusChecking)
{
    ViStatus         error = VI_SUCCESS;
    RsCoreSessionPtr rsSession = NULL;

    checkErr(RsCore_GetRsSession(instrSession, &rsSession));
    rsSession->optionChecking = optionChecking;
    checkErr(RsCore_SetAttributeViBoolean(instrSession, "", RS_ATTR_QUERY_INSTRUMENT_STATUS, 0, statusChecking));
    checkErr(RsCore_SetAttributeViBoolean(instrSession, "", RS_ATTR_RANGE_CHECK, 0, rangeChecking));

Error:
    return error;
}

/// HIFN If TRUE (default after init is FALSE), the driver calls SYST:ERR? automatically in CheckStatus() when Instrument Status Error is detected.
/// HIFN Use the function rsspecan_GetError afterwards to get the detailed information about the instrument error(s).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR autoSystErrQuery/If TRUE (default after init is FALSE), the driver calls SYST:ERR? automatically in CheckStatus() when Instrument Status Error is detected.
ViStatus _VI_FUNC rsspecan_ConfigureAutoSystemErrQuery(ViSession instrSession,
                                                       ViBoolean autoSystErrQuery)
{
    ViStatus         error = VI_SUCCESS;
    RsCoreSessionPtr rsSession = NULL;

    checkErr(RsCore_GetRsSession(instrSession, &rsSession));
    rsSession->autoSystErrQuery = autoSystErrQuery;

Error:
    return error;
}

/// HIFN Session-related mutex switch.
/// HIFN If TRUE (default after init is FALSE), the driver sets a mutual-exclusion mechanism for every driver's hi-level function and attribute access.
/// HIFN Set this value to TRUE if you plan to use one session in more than one thread.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR multiThreadLocking/Session-related mutex switch
ViStatus _VI_FUNC rsspecan_ConfigureMultiThreadLocking(ViSession instrSession,
                                                       ViBoolean multiThreadLocking)
{
    ViStatus         error = VI_SUCCESS;
    RsCoreSessionPtr rsSession = NULL;

    checkErr(RsCore_GetRsSession(instrSession, &rsSession));
    rsSession->multiThreadLocking = multiThreadLocking;

Error:
    return error;
}


/// HIFN This function executes the instrument self-test and returns the
/// HIFN result.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR testResult/This control contains the value returned from the instrument
/// HIPAR testResult/self test.
/// HIPAR testMessage/This control contains the string returned from the self test.
ViStatus _VI_FUNC rsspecan_self_test(ViSession instrSession,
                                     ViInt16   *testResult,
                                     ViChar    testMessage[])
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidNullPointer(instrSession, testResult), 2, "Null address for Test Result");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, testMessage), 3, "Null address for Test Message");

    checkErr(RsCore_SelfTest(instrSession, testResult, 48, testMessage));

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function executes the instrument self-test and returns the
/// HIFN result.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR bufferSize/This control specifies the size of the buffer.
/// HIPAR results/Returns contains the results of the selftest out of the
/// HIPAR results/instrument.
ViStatus _VI_FUNC rsspecan_SelfTestResults(ViSession instrSession,
                                           ViInt32   bufferSize,
                                           ViChar    results[])
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_QueryViStringUnknownLengthToUserBuffer(instrSession, "DIAG:SERV:STES:RES?", bufferSize, results, NULL));

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function returns the driver and instrument revisions.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR driverRev/Returns the instrument driver software revision numbers in the
/// HIPAR driverRev/form of a string.
/// HIPAR instrRev/Returns the instrument firmware revision numbers in the form of
/// HIPAR instrRev/a string.
ViStatus _VI_FUNC rsspecan_revision_query(ViSession instrSession,
                                          ViChar    instrumentDriverRevision[],
                                          ViChar    firmwareRevision[])
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidNullPointer(instrSession, instrumentDriverRevision), 2, "Null address for Instrument Driver Revision");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, firmwareRevision), 3, "Null address for Instrument Revision");

    checkErr(RsCore_GetAttributeViString(instrSession, NULL, RS_ATTR_SPECIFIC_DRIVER_REVISION, 0, 256, instrumentDriverRevision));
    checkErr(RsCore_GetAttributeViString(instrSession, NULL, RS_ATTR_INSTRUMENT_FIRMWARE_REVISION, 0, 256, firmwareRevision));
    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function queries the number of the SCPI version, which
/// HIFN is relevant for the instrument.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.
/// HIPAR length/This control specifies the size of the buffer.
/// HIPAR systemVersion/Returns the number of the SCPI version, which is relevant for
/// HIPAR systemVersion/the instrument.
ViStatus _VI_FUNC rsspecan_SystemVersion(ViSession instrSession,
                                         ViInt32   length,
                                         ViChar    systemVersion[])
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_GetAttributeViString(instrSession, "", RSSPECAN_ATTR_SYST_VERSION,
            length, systemVersion),
            2, "System Version");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function reads the version information of all available measurement applications.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.
/// HIPAR bufferSize/This control specifies the size of the buffer.
/// HIPAR results/Returns the version information of all available measurement
/// HIPAR results/applications. The lines of the result table are output as string
/// HIPAR results/data separated by commas.
ViStatus _VI_FUNC rsspecan_ApplicationsVersions(ViSession instrSession,
                                                ViInt32   bufferSize,
                                                ViChar    results[])
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_GetAttributeViString(instrSession, "", RSSPECAN_ATTR_APPLICATIONS_VERSIONS,
            bufferSize, results),
            3, "Results");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function queries the instrument error queue and returns
/// HIFN the result.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.
/// HIPAR errCode/Returns the error code read from the instrument's error queue.
/// HIPAR errMessage/Returns the error message string read from the instrument's
/// HIPAR errMessage/error message queue.
ViStatus _VI_FUNC rsspecan_error_query(ViSession instrSession,
                                       ViInt32*  errorCode,
                                       ViChar    errorMessage[])
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidNullPointer(instrSession, errorCode), 2, "Null address for Error Code");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, errorMessage), 3, "Null address for Error Message");

    checkErr(RsCore_ErrorQueryAll(instrSession, errorCode, 1024, errorMessage));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function reads all system messages and returns a list of
/// HIFN comma separated strings.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.
/// HIPAR bufferSize/Sets the size of Errors string.
/// HIPAR errors/Returns the error message string read from the instrument's
/// HIPAR errors/error message queue.
ViStatus _VI_FUNC rsspecan_ErrorList(ViSession instrSession,
                                     ViInt32   bufferSize,
                                     ViChar    errors[])
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32RangeMin(instrSession, bufferSize, 0),
            2, "Buffer Size");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, errors), 3, "Null address for Errors");

    checkErr(rsspecan_GetAttributeViString( instrSession, "", RSSPECAN_ATTR_SYST_ERR_LIST,
        bufferSize, errors));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN This function queries all system messages, or all messages of a defined type,
/// HIFN displayed in the status bar for a specific measurement channel (application).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR messageType/This control selects the message type.
/// HIPAR channelName/String containing the name of the channel.
/// HIPAR bufferSize/Sets the size of Messages string.
/// HIPAR messages/String containing all messages of the selected type for the specified measurement
/// HIPAR messages/channel. Each message is separated by a comma and inserted in parentheses.
/// HIPAR messages/If no messages are available, empty parentheses are returned.
ViStatus _VI_FUNC rsspecan_ErrorListSpecificType(ViSession instrSession,
                                                 ViInt32   messageType,
                                                 ViString  channelName,
                                                 ViInt32   bufferSize,
                                                 ViChar    messages[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsspecan_AssertNotFSV(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, messageType, RSSPECAN_VAL_MESSAGE_TYPE_ALL, RSSPECAN_VAL_MESSAGE_TYPE_MESSAGE),
            2, "Message Type");

    if (channelName == NULL)
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SYST:ERR:EXT? %s", messageTypeArr[messageType]);
    else
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SYST:ERR:EXT? %s,\'%s\'", messageTypeArr[messageType], channelName);
    checkErr(RsCore_QueryViStringUnknownLengthToUserBuffer(instrSession, cmd, bufferSize, messages, NULL));

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function translates the error codes returned by this
/// HIFN instrument driver into user-readable strings.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR errorCode/Pass the Status parameter that is returned from any of the
/// HIPAR errorCode/instrument driver functions..
/// HIPAR errorMessage/Returns the user-readable message string that corresponds to the
/// HIPAR errorMessage/status code you specify.
ViStatus _VI_FUNC rsspecan_error_message(ViSession instrSession,
                                         ViStatus  errorCode,
                                         ViChar    errorMessage[])
{
    return rsspecan_utility_error_message(instrSession, errorCode, errorMessage);
}

/// HIFN This function deletes all entries in the table SYSTEM MESSAGES.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.
ViStatus _VI_FUNC rsspecan_ClearInstrumentErrors(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_SYST_ERR_CLEAR, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function deletes all contents of the 'Remote Errors' table.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsspecan_ClearRemoteErrors(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_SYST_REMOTE_ERR_TABLE_CLEAR, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function displays the error display during remote operation on
/// HIFN  and off. If activated, the R&S FSW displays a message box at the bottom
/// HIFN  of the screen that contains the most recent type of error and the
/// HIFN  command that caused the error.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR displayRemoteErrors/This control displays the error display during remote operation on and
/// HIPAR displayRemoteErrors/off. If activated, the R&S FSW displays a message box at the bottom of
/// HIPAR displayRemoteErrors/the screen that contains the most recent type of error and the command
/// HIPAR displayRemoteErrors/that caused the error.
ViStatus _VI_FUNC rsspecan_DisplayRemoteErrors(ViSession instrSession,
                                               ViBoolean displayRemoteErrors)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_SYST_DISPLAY_REMOTE_ERRORS, displayRemoteErrors),
            2, "Display Remote Errors");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the error messages that occurred during remote operation.
/// HIFN  Note: The remote error list is automatically cleared when the R&S FSW
/// HIFN  is shut down.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR bufferSize/Pass the number of bytes in the ViChar array you specify for the
/// HIPAR bufferSize/Error Table parameter.
/// HIPAR errorTable/Comma-separated list of errors from the "Remote Errors" table
ViStatus _VI_FUNC rsspecan_QueryRemoteErrors(ViSession instrSession,
                                             ViInt32   bufferSize,
                                             ViChar    errorTable[])
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsspecan_QueryViString(instrSession, "SYST:ERR:LIST? REM", bufferSize, errorTable));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function sets the *IDN string to the factory default
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.
ViStatus _VI_FUNC rsspecan_IDStringFactory(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_SYST_ID_FACTORY, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function returns "1" for the factory default state and "0"
/// HIFN if the ID string is changed..
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.
/// HIPAR factoryDefault/Returns 1" for the factory default state and "0" if the ID
/// HIPAR factoryDefault/string is changed.
ViStatus _VI_FUNC rsspecan_GetIDStringFactory(ViSession instrSession,
                                              ViInt32   *factoryDefault)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsspecan_GetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_SYST_ID_FACTORY_QUERY, factoryDefault));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function sets the OPC timeout.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.
/// HIPAR OPCtimeout/This control sets the timeout value.
ViStatus _VI_FUNC rsspecan_SetTimeout(ViSession instrSession,
                                      ViUInt32  timeout)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RS_ATTR_OPC_TIMEOUT, (ViInt32)timeout),
            3, "Timeout");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN This function returns the OPC timeout.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.
/// HIPAR timeout/This control returns the timeout value.
ViStatus _VI_FUNC rsspecan_GetTimeout(ViSession instrSession,
                                      ViUInt32  *timeout)
{
    ViStatus error = VI_SUCCESS;
    ViInt32  timeoutInt;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_GetAttributeViInt32(instrSession, "", RS_ATTR_OPC_TIMEOUT, &timeoutInt),
            3, "Timeout");

    *timeout = (ViUInt32)timeoutInt;

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function specifies the minimum timeout value to use (in
/// HIFN milliseconds) when accessing the device associated with the
/// HIFN given session.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.
/// HIPAR VISATimeout/This control sets the timeout value.
ViStatus _VI_FUNC rsspecan_SetVISATimeout(ViSession instrSession,
                                          ViUInt32  VISATimeout)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_SetVisaTimeout(instrSession, VISATimeout));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function returns the minimum timeout value to use (in
/// HIFN milliseconds) when accessing the device associated with the
/// HIFN given session.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.
/// HIPAR VISATimeout/This control gets the timeout value.
ViStatus _VI_FUNC rsspecan_GetVISATimeout(ViSession instrSession,
                                          ViUInt32  *VISATimeout)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetVisaTimeout(instrSession, VISATimeout));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function is used to activate the LLO (local lockout) or to
/// HIFN return to the local mode (GTL go to local). When set to ON, the
/// HIFN local button on the instrument is deactivated.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.
/// HIPAR lockLocalKey/This control is used to activate the LLO (local lockout) or to
/// HIPAR lockLocalKey/return to the local mode GTL (go to local). Parameter ON is LLO,
/// HIPAR lockLocalKey/OFF is GTL.
ViStatus _VI_FUNC rsspecan_LockLocalKey(ViSession instrSession,
                                        ViBoolean lockLocalKey)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_LOCK_LOCAL_KEY, lockLocalKey),
            2, "Lock Local Key");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN This function switches ON/OFF instrument status checking of the
/// HIFN Tinstrument
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR statusChecking/This control selects ON or OFF instrument state checking.
ViStatus _VI_FUNC rsspecan_setCheckStatus(ViSession instrSession,
                                          ViBoolean state)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_SetAttributeViBoolean(instrSession, "", RS_ATTR_QUERY_INSTRUMENT_STATUS, 0, state));

Error:
    return error;
}

/// HIFN This function switches ON/OFF option checking of the instrument
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR state/Document parameter here.
ViStatus _VI_FUNC rsspecan_setCheckOption(ViSession instrSession,
                                          ViBoolean optionChecking)
{
    ViStatus         error = VI_SUCCESS;
    RsCoreSessionPtr rsSession = NULL;

    checkErr(RsCore_GetRsSession(instrSession, &rsSession));
    rsSession->optionChecking = optionChecking;

Error:
    return error;
}

/// HIFN  Returns an array of all the installed instrument options.
/// HIFN
/// HIFN     SCPI command:
/// HIFN     *OPT?
/// HIFN
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR bufferSize/Buffer size allocated for query.
/// HIPAR options/Returns a comma seperated list of options.
/// HIPAR bytesRead/Returns the number of bytes read from instrument.
ViStatus _VI_FUNC rsspecan_InstrumentOptions(ViSession instrSession,
                                             ViInt32   bufferSize,
                                             ViChar    options[],
                                             ViInt32*  bytesRead)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_QueryViStringUnknownLengthToUserBuffer(instrSession, "*OPT?", bufferSize, options, bytesRead));

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function clears status.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsspecan_ClearStatus(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_ClearStatus(instrSession));
    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function returns the ID Query response string.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR bufferSize/Size of IDQueryResponse buffer
/// HIPAR idQueryResponse/Returns the ID Query response string. The array should consist of at
/// HIPAR idQueryResponse/least 256 elements.
ViStatus _VI_FUNC rsspecan_IDQueryResponse(ViSession instrSession,
                                           ViUInt32  bufferSize,
                                           ViChar    IDQueryResponse[])
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsspecan_QueryViString(instrSession, "*IDN?", bufferSize, IDQueryResponse));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects the response format to the *IDN? query.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR idQueryFormat/Selects the response format to the *IDN? query.
ViStatus _VI_FUNC rsspecan_ConfigureIDQueryFormat(ViSession instrSession,
                                                  ViInt32   idQueryFormat)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, NULL, RSSPECAN_ATTR_ID_QUERY_FORMAT, idQueryFormat),
            2, "ID Query Format");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Stops further command processing until all commands sent before *WAI
/// HIFN  have been executed.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsspecan_ProcessAllPreviousCommands(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsspecan_WriteInstrData(instrSession, "*WAI"));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function queries the OPC.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR opc/Queries the OPC.
ViStatus _VI_FUNC rsspecan_QueryOPC(ViSession instrSession,
                                    ViInt32   *opc)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsspecan_QueryViInt32(instrSession, "*OPC?", opc));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function sets fast sweep mode.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR fastSweepMode/Fast sweep mode
ViStatus _VI_FUNC rsspecan_SetFastSweepMode(ViSession instrSession,
                                            ViInt32   fastSweepMode)
{  
    ViStatus         error = VI_SUCCESS;
    RsCoreSessionPtr rsSession = NULL;

    checkErr(RsCore_GetRsSession(instrSession, &rsSession));
    rsSession->fastSweepInstrument = fastSweepMode;

Error:
    return error;
}

/// HIFN  Configures the date and time on the internal real-time clock on the
/// HIFN  instrument.
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     SYSTem:DATE
/// HIFN     SYSTem:TIME
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR year/The year
/// HIPAR month/The month
/// HIPAR day/The day
/// HIPAR hours/The hours (0 to 23)
/// HIPAR minutes/The minutes (0 to 59)
/// HIPAR seconds/The seconds (0 to 60)
ViStatus _VI_FUNC rsspecan_SetInstrumentDateTime(ViSession instrSession,
                                                 ViInt32   year,
                                                 ViInt32   month,
                                                 ViInt32   day,
                                                 ViInt32   hours,
                                                 ViInt32   minutes,
                                                 ViInt32   seconds)
{
    ViStatus error = VI_SUCCESS;
    ViChar   param_date[RS_PARAM_BUF_SIZE] = "";
    ViChar   param_time[RS_PARAM_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf (param_date, RS_PARAM_BUF_SIZE, "%ld,%ld,%ld", year, month, day);
    snprintf (param_time, RS_PARAM_BUF_SIZE, "%ld,%ld,%ld", hours, minutes, seconds);

    checkErr(rsspecan_SetAttributeViString(instrSession, NULL, RSSPECAN_ATTR_HIDDEN_DATE, param_date));
    checkErr(rsspecan_SetAttributeViString(instrSession, NULL, RSSPECAN_ATTR_HIDDEN_TIME, param_time));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Returns the date and time from the internal real-time clock on the
/// HIFN  instrument.
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     SYSTem:DATE
/// HIFN     SYSTem:TIME
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR year/The year
/// HIPAR month/The month
/// HIPAR day/The day
/// HIPAR hours/The hours (0 to 23)
/// HIPAR minutes/The minutes (0 to 59)
/// HIPAR seconds/The seconds (0 to 60)
ViStatus _VI_FUNC rsspecan_GetInstrumentDateTime(ViSession instrSession,
                                                 ViInt32*  year,
                                                 ViInt32*  month,
                                                 ViInt32*  day,
                                                 ViInt32*  hours,
                                                 ViInt32*  minutes,
                                                 ViInt32*  seconds)
{
    ViStatus error = VI_SUCCESS;
    ViChar   param_date[RS_PARAM_BUF_SIZE];
    ViChar   param_time[RS_PARAM_BUF_SIZE];
    ViPInt32 date;
    ViPInt32 time;
    ViInt32  size;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsspecan_GetAttributeViString(instrSession, NULL,
                                              RSSPECAN_ATTR_HIDDEN_DATE, 128, param_date));
    checkErr(rsspecan_GetAttributeViString(instrSession, NULL,
                                              RSSPECAN_ATTR_HIDDEN_TIME, 128, param_time));

    checkErr(RsCore_Convert_String_To_ViInt32Array(instrSession, param_date, &date, &size));
    checkErr(RsCore_Convert_String_To_ViInt32Array(instrSession, param_time, &time, &size));

    *year = date[0];
    *month = date[1];
    *day = date[2];

    *hours = time[0];
    *minutes = time[1];
    *seconds = time[2];

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}



/*===========================================================================*
 *------------------- Error Info --------------------------------------------*
 *===========================================================================*/
/// HIFN These functions enable the instrument driver user to
/// HIFN get or clear the error information the driver associates with the
/// HIFN RS session.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR errorCode/Returns the error code for the session or execution thread.
/// HIPAR bufferSize/Pass the number of bytes in the ViChar array you specify for the
/// HIPAR bufferSize/Description parameter.
/// HIPAR description/Returns the error description for the session or execution
/// HIPAR description/thread. If there is no description, the function returns an
/// HIPAR description/empty string.
ViStatus _VI_FUNC rsspecan_GetError(ViSession instrSession,
                                    ViStatus  *errorCode,
                                    ViInt32   bufferSize,
                                    ViChar    description[])
{
    ViStatus error = VI_SUCCESS;

    // Lock Session - do not jump to the end, even with invalid instrSession the function continues further
    (void)(RsCore_LockSession(instrSession));

    // Test for nulls and acquire error data
    viCheckParm(RsCore_InvalidNullPointer(instrSession, errorCode), 2, "Null address for Error");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, description), 4, "Null address for Description");

    checkErr(RsCore_GetErrorCompleteDescription(instrSession, &rsspecan_utility_error_message, errorCode, bufferSize, description)
    );

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function clears the error code and error description for
/// HIFN the session.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
ViStatus _VI_FUNC rsspecan_ClearError(ViSession instrSession)
{
    return RsCore_ClearErrorInfo(instrSession);
           }

/*===========================================================================*
 *------------------- Instrument I/O ----------------------------------------*
 *===========================================================================*/
/// HIFN This function is used to read data from the instrument and write
/// HIFN it to a user specified file on the host computer.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR source/This control selects file for the data transfer from instrument
/// HIPAR source/to control computer.
/// HIPAR destination/This control defines destination file to which the data transfer
/// HIPAR destination/from instrument to control computer applies.
ViStatus _VI_FUNC rsspecan_ReadToFileFromInstrument(ViSession instrSession,
                                                    ViString  source,
                                                    ViString  destination)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "MMEM:DATA? '%s'", source);
    checkErr(RsCore_QueryBinaryDataBlockToFile(instrSession, cmd, destination, RS_VAL_TRUNCATE));

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function is used to read data from the host computer and
/// HIFN write it to a user specified file in the instrument.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR source/This control selects file for the data transfer from control
/// HIPAR source/computer to instrument.
/// HIPAR destination/This control defines destination file to which the data transfer
/// HIPAR destination/from control computer to instrument applies.
ViStatus _VI_FUNC rsspecan_WriteFromFileToInstrument(ViSession instrSession,
                                                     ViString  source,
                                                     ViString  destination)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "MMEM:DATA '%s',", destination);
    checkErr(RsCore_WriteBinaryDataFromFile(instrSession, cmd, source));
    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN These functions enable the instrument driver user to
/// HIFN write commands directly to the instrument.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR writeBuffer/Pass the string to be written to the instrument.
///  Note:  These functions bypass the RS attribute state caching.
///  Note:        WriteInstrData invalidates the cached values for all
///  Note:        attributes.
ViStatus _VI_FUNC rsspecan_WriteInstrData(ViSession instrSession,
                                          ViString  writeBuffer)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    error = RsCore_Write(instrSession, writeBuffer);

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN These functions enable the instrument driver user to
/// HIFN read commands directly from the instrument.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR numBytes/Pass the maximum number of bytes to read from the instruments.
/// HIPAR rdBuf/After this function executes, this parameter contains the data
/// HIPAR rdBuf/that was read from the instrument.
/// HIPAR bytesRead/Returns the number of bytes actually read from the instrument
/// HIPAR bytesRead/and stored in the Read Buffer.
ViStatus _VI_FUNC rsspecan_ReadInstrData(ViSession instrSession,
                                         ViInt32   numberOfBytesToRead,
                                         ViChar    readBuffer[],
                                         ViUInt32  * numBytesRead)
{
    ViStatus error = VI_SUCCESS;
    ViChar   *responseString = NULL;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_ReadViStringUnknownLength(instrSession, &responseString));
    checkErr(RsCore_CopyToUserBufferAsciiData(readBuffer, numberOfBytesToRead, responseString));
    *numBytesRead = (ViUInt32)strlen(responseString);

Error:
    if (responseString)
        free(responseString);

    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN This function queries the ViBoolean value.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR command/The query to be sent to the instrument.
/// HIPAR value/Returns the ViBoolean value
ViStatus _VI_FUNC rsspecan_QueryViBoolean(ViSession instrSession,
                                          ViString  command,
                                          ViBoolean *value)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, (ViInt32)strlen(command), 1, 2147483647), 2,
        "Command (null string length)");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, value), 3, "Value");

    checkErr(RsCore_QueryViBoolean(instrSession, command, value));

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN This function queries the ViInt32 value.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR command/The query to be sent to the instrument.
/// HIPAR value/Returns the ViInt32 value
ViStatus _VI_FUNC rsspecan_QueryViInt32(ViSession instrSession,
                                        ViString  command,
                                        ViInt32   *value)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, (ViInt32)strlen(command), 1, 2147483647), 2,
        "Command (null string length)");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, value), 3, "Value");

    checkErr(RsCore_QueryViInt32(instrSession, command, value));

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function queries the ViReal64 value.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR cmd/The query to be sent to the instrument.
/// HIPAR value/Returns the ViReal64 value
ViStatus _VI_FUNC rsspecan_QueryViReal64(ViSession instrSession,
                                         ViString  command,
                                         ViReal64  *value)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, (ViInt32)strlen(command), 1, 2147483647), 2,
        "Command (null string length)");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, value), 3, "Value");

    checkErr(RsCore_QueryViReal64(instrSession, command, value));

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function queries the ViString value.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR cmd/The query to be sent to the instrument.
/// HIPAR bufferSize/Pass the number of bytes in the ViChar array you specify for the
/// HIPAR bufferSize/Value parameter.
/// HIPAR value/Returns the ViString value
ViStatus _VI_FUNC rsspecan_QueryViString(ViSession instrSession,
                                         ViString  command,
                                         ViInt32   bufferSize,
                                         ViChar    value[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   *pbuffer = NULL;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, (ViInt32)strlen(command), 1, 2147483647), 2,
        "Command (null string length)");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, value), 3, "Value");

    checkErr(RsCore_QueryViStringUnknownLength(instrSession, command, &pbuffer));

    checkErr(rsspecan_CheckStatus(instrSession));

    checkErr(RsCore_CopyToUserBufferAsciiData(value, bufferSize, pbuffer));

Error:
    if (pbuffer)
        free(pbuffer);

    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN Sends a command to the instrument synchronised with OPC-polling.
/// HIFN Use this function to wait for an instrument operation that can
/// HIFN take a long time to complete, e.g. self-alignment.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR writeBuffer/Pass the string to be written to the instrument.
/// HIPAR timeout/This control sets the timeout value.
ViStatus _VI_FUNC rsspecan_WriteCommandWithOPCSync(ViSession instrSession,
                                                   ViString  writeBuffer,
                                                   ViUInt32  timeout)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidStringLength(instrSession, writeBuffer, 1, -1),
        2, "Write Buffer (string length)");

    checkErr(RsCore_WriteWithOpc(instrSession, writeBuffer, timeout));

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN Sends a query to the instrument synchronised with OPC-polling.
/// HIFN Use this function to wait for a response that can take a long
/// HIFN time to complete e.g. self-alignment, calibration or self-test.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR writeBuffer/Pass the string to be written to the instrument.
/// HIPAR timeout/This control sets the timeout value.
/// HIPAR bufferSize/Pass the maximum number of bytes to read from the instruments.
/// HIPAR readBuffer/After this function executes, this parameter contains the data
/// HIPAR readBuffer/that was read from the instrument.
/// HIPAR numBytesRead/Returns the number of bytes actually read from the instrument
/// HIPAR numBytesRead/and stored in the Read Buffer.
ViStatus _VI_FUNC rsspecan_QueryWithOPCSync(ViSession instrSession,
                                            ViString  writeBuffer,
                                            ViUInt32  timeout,
                                            ViInt32   bufferSize,
                                            ViChar    readBuffer[],
                                            ViUInt32* numBytesRead)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, (ViInt32)strlen(writeBuffer), 1, 2147483647), 2,
        "Command (null string length)");

    checkErr(RsCore_QueryViStringUnknownLengthToUserBufferWithOpc(instrSession, writeBuffer, timeout, bufferSize, readBuffer, (ViPInt32)numBytesRead));

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/*===========================================================================*
 *= RsSpecAn Set/Get/Check Attribute Group Functions ========================*
 *===========================================================================*/

/*****************************************************************************
 * Function: rsspecan_SetAttribute<type> Functions
 * Purpose:  These functions enable the instrument driver user to set
 *           attribute values directly.  There are typesafe versions for
 *           ViInt32, ViReal64, ViString, ViBoolean, and ViSession datatypes.
 *****************************************************************************/

/* ViInt32 */
ViStatus _VI_FUNC rsspecan_SetAttributeViInt32(ViSession instrSession,
                                               ViString  repeatedCapabilityName,
                                               ViAttr    attributeId,
                                               ViInt32   attributeValue)
{
    ViStatus error = VI_SUCCESS;

             (void)RsCore_LockSession(instrSession);

    error = RsCore_SetAttributeViInt32(instrSession, repeatedCapabilityName, attributeId, RS_VAL_DIRECT_USER_CALL, attributeValue);

    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/* ViInt64 */
ViStatus _VI_FUNC rsspecan_SetAttributeViInt64(ViSession instrSession,
                                               ViString  repeatedCapabilityName,
                                               ViAttr    attributeId,
                                               ViInt64   attributeValue)
{
    ViStatus error = VI_SUCCESS;

             (void)RsCore_LockSession(instrSession);

    error = RsCore_SetAttributeViInt64(instrSession, repeatedCapabilityName, attributeId, RS_VAL_DIRECT_USER_CALL, attributeValue);

    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/* ViReal64 */
ViStatus _VI_FUNC rsspecan_SetAttributeViReal64(ViSession instrSession,
                                                ViString  repeatedCapabilityName,
                                                ViAttr    attributeId,
                                                ViReal64  attributeValue)
{
    ViStatus error = VI_SUCCESS;

             (void)RsCore_LockSession(instrSession);

    error = RsCore_SetAttributeViReal64(instrSession, repeatedCapabilityName, attributeId, RS_VAL_DIRECT_USER_CALL, attributeValue);

    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/* ViString */
ViStatus _VI_FUNC rsspecan_SetAttributeViString(ViSession instrSession,
                                                ViString  repeatedCapabilityName,
                                                ViAttr    attributeId,
                                                ViString  attributeValue)
{
    ViStatus error = VI_SUCCESS;

             (void)RsCore_LockSession(instrSession);

    error = RsCore_SetAttributeViStringOrRawString(instrSession, repeatedCapabilityName, attributeId, RS_VAL_DIRECT_USER_CALL, attributeValue);

    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/* ViBoolean */
ViStatus _VI_FUNC rsspecan_SetAttributeViBoolean(ViSession instrSession,
                                                 ViString  repeatedCapabilityName,
                                                 ViAttr    attributeId,
                                                 ViBoolean attributeValue)
{
    ViStatus error = VI_SUCCESS;

             (void)RsCore_LockSession(instrSession);

    error = RsCore_SetAttributeViBoolean(instrSession, repeatedCapabilityName, attributeId, RS_VAL_DIRECT_USER_CALL, attributeValue);

    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/*****************************************************************************
 * Function: rsspecan_GetAttribute<type> Functions
 * Purpose:  These functions enable the instrument driver user to get
 *           attribute values directly.  There are typesafe versions for
 *           ViInt32, ViReal64, ViString, ViBoolean, and ViSession attributes.
 *****************************************************************************/

/* ViInt32 */
ViStatus _VI_FUNC rsspecan_GetAttributeViInt32(ViSession instrSession,
                                               ViString  repeatedCapabilityName,
                                               ViAttr    attributeId,
                                               ViInt32   *attributeValue)
{
    ViStatus error = VI_SUCCESS;

             (void)RsCore_LockSession(instrSession);
    error    = RsCore_GetAttributeViInt32(instrSession, repeatedCapabilityName, attributeId, RS_VAL_DIRECT_USER_CALL, attributeValue);

    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/* ViInt64 */
ViStatus _VI_FUNC rsspecan_GetAttributeViInt64(ViSession instrSession,
                                               ViString  repeatedCapabilityName,
                                               ViAttr    attributeId,
                                               ViInt64   *attributeValue)
{
    ViStatus error = VI_SUCCESS;

             (void)RsCore_LockSession(instrSession);

    error = RsCore_GetAttributeViInt64(instrSession, repeatedCapabilityName, attributeId, RS_VAL_DIRECT_USER_CALL, attributeValue);

    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/* ViReal64 */
ViStatus _VI_FUNC rsspecan_GetAttributeViReal64(ViSession instrSession,
                                                ViString  repeatedCapabilityName,
                                                ViAttr    attributeId,
                                                ViReal64  *attributeValue)
{
    ViStatus error = VI_SUCCESS;

             (void)RsCore_LockSession(instrSession);

    error = RsCore_GetAttributeViReal64(instrSession, repeatedCapabilityName, attributeId, RS_VAL_DIRECT_USER_CALL, attributeValue);

    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/* ViString */
ViStatus _VI_FUNC rsspecan_GetAttributeViString(ViSession instrSession,
                                                ViString  repeatedCapabilityName,
                                                ViAttr    attributeId,
                                                ViInt32   bufferSize,
                                                ViChar    attributeValue[])
{
    ViStatus error = VI_SUCCESS;

             (void)RsCore_LockSession(instrSession);

    error = RsCore_GetAttributeViStringOrRawString(instrSession, repeatedCapabilityName, attributeId, RS_VAL_DIRECT_USER_CALL, bufferSize,
        attributeValue);

    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/* ViBoolean */
ViStatus _VI_FUNC rsspecan_GetAttributeViBoolean(ViSession instrSession,
                                                 ViString  repeatedCapabilityName,
                                                 ViAttr    attributeId,
                                                 ViBoolean *value)
{
    ViStatus error = VI_SUCCESS;

             (void)RsCore_LockSession(instrSession);

    error = RsCore_GetAttributeViBoolean(instrSession, repeatedCapabilityName, attributeId, RS_VAL_DIRECT_USER_CALL, value);

    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/********************************************************************
 *          B21 - External Mixer                                    *
 ********************************************************************/

/// HIFN This function turns external mixing on/off.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR externalMixingEnabled/Specifies whether external mixing is enabled.
ViStatus _VI_FUNC rsspecan_ConfigureExternalMixerEnabled(ViSession instrSession,
                                                         ViBoolean externalMixingEnabled)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_EXTERNAL_MIXER_ENABLED, externalMixingEnabled),
            2, "External Mixer Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function specifies the mixer harmonic and average
/// HIFN conversion loss.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR band/Defines waveguide band in the Conversion Loss table.
/// HIPAR harmonic/Specifies harmonic number.
/// HIPAR harmonicType/Sets the type of harmonic.
ViStatus _VI_FUNC rsspecan_ConfigureExternalMixer(ViSession instrSession,
                                                  ViInt32   band,
                                                  ViInt32   harmonic,
                                                  ViInt32   harmonicType)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_EXTERNAL_MIXER_HARMONIC_BAND, band),
            2, "Band");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_EXTERNAL_MIXER_HARMONIC_TYPE, harmonicType),
            4, "Harmonic Type");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_EXTERNAL_MIXER_HARMONIC, harmonic),
            3, "Harmonic");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function turns cross-correlation with two external mixers on and
/// HIFN  off.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR mixingXCORR/This control turns cross-correlation with two external mixers on and
/// HIPAR mixingXCORR/off.
ViStatus _VI_FUNC rsspecan_ConfigureExternalMixerXCORR(ViSession instrSession,
                                                       ViBoolean mixingXCORR)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_EXTERNAL_MIXER_XCORR, mixingXCORR),
            2, "Mixing XCORR");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function specifies a 2-port or a 3-port external mixer.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR numberofPorts/Specifies a two-port mixer or a three-port mixer.
ViStatus _VI_FUNC rsspecan_ConfigureExternalMixerNumberOfPorts(
                            ViSession instrSession,
                            ViInt32   numberofPorts)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_EXTERNAL_MIXER_NUMBER_OF_PORTS, numberofPorts),
            2, "Number Of Ports");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN his function defines the frequency at which the mixer switches
/// HIFN from one range to the next (if two different ranges are
/// HIFN selected).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR handoverFrequency/Defines the frequency at which the mixer switches from one range
/// HIPAR handoverFrequency/to the next.
ViStatus _VI_FUNC rsspecan_ConfigureExternalMixerHandoverFrequency(
                            ViSession instrSession,
                            ViReal64  handoverFrequency)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_EXTERNAL_MIXER_HANDOVER_FREQUENCY, handoverFrequency),
            2, "Handover Frequency");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN This function specifies whether the frequencies beyond the
/// HIFN defined band limits are to be used.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR RFOverrange/Specifies whether the frequencies beyond the defined band limits
/// HIPAR RFOverrange/are to be used
ViStatus _VI_FUNC rsspecan_ConfigureExternalMixerRFOverrange(ViSession instrSession,
                                                             ViBoolean RFOverrange)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_EXTERNAL_MIXER_RF_OVERRANGE, RFOverrange),
            2, "RF Overrange");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function restores the preset frequency ranges for the
/// HIFN standard waveguide bands.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
ViStatus _VI_FUNC rsspecan_ConfigureExternalMixerPresetBand(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_EXTERNAL_MIXER_PRESET_BAND, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function specifies a second (high) harmonic.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR highHarmonic/Specifies whether a second (high) harmonic is to be used to
/// HIPAR highHarmonic/cover the band's frequency range.
/// HIPAR harmonicOrder/Specifies the harmonic order to be used for the high range.
ViStatus _VI_FUNC rsspecan_ConfigureExternalMixerHighHarmonic(
                            ViSession instrSession,
                            ViBoolean highHarmonic,
                            ViInt32   harmonicOrder)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_EXTERNAL_MIXER_HIGH_HARMONIC_STATE, highHarmonic),
            2, "High Harmonic");

    if (highHarmonic == VI_TRUE)
    {
        viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_EXTERNAL_MIXER_HIGH_HARMONIC, harmonicOrder),
                3, "Harmonic Order");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function sets the Signal ID or Auto ID mode.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR mode/Sets the type of harmonic.
/// HIPAR autoIDThreshold/Determines the maximum permissible level difference between
/// HIPAR autoIDThreshold/measurement and reference sweep in AUTO ID mode.
ViStatus _VI_FUNC rsspecan_ConfigureExternalMixerSignalIDMode(
                            ViSession instrSession,
                            ViInt32   mode,
                            ViReal64  autoIDThreshold)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_EXTERNAL_MIXER_SIGNAL_ID_MODE, mode),
            2, "Mode");
    if (mode == RSSPECAN_VAL_STATE_AUTO)
    {
        viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_EXTERNAL_MIXER_THRESHOLD, autoIDThreshold),
                3, "Auto ID Threshold");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function configures the bias current for the low and high
/// HIFN range.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR range/This control selects the conversion loss range.
/// HIPAR bias/Specifies the bias current.
ViStatus _VI_FUNC rsspecan_ConfigureExternalMixerBias(ViSession instrSession,
                                                      ViInt32   range,
                                                      ViReal64  bias)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    if (range == 1)
    {
        viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_EXTERNAL_MIXER_BIAS_LOW, bias),
                2, "Bias");
    }
    else
    {
        viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_EXTERNAL_MIXER_BIAS_HIGH, bias),
                2, "Bias");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function specifies the conversion loss for selected band.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR range/This control selects the conversion loss range.
/// HIPAR conversionLossMode/This control selects the converstion loss mode.
/// HIPAR conversionLoss/Specifies the average conversion loss.
/// HIPAR conversionLossTable/Sets the conversion loss table for the active band.
ViStatus _VI_FUNC rsspecan_ConfigureExternalMixerConversionLoss(
                            ViSession instrSession,
                            ViInt32   range,
                            ViInt32   conversionLossMode,
                            ViReal64  conversionLoss,
                            ViString  conversionLossTable)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, range, 1, 2),
            2, "Range");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, conversionLossMode, RSSPECAN_VAL_MIXER_LOSS_AVER, RSSPECAN_VAL_MIXER_LOSS_TABL),
            3, "Conversion Loss Mode");

    switch (range)
    {
    case 1:
        switch (conversionLossMode)
        {
        case RSSPECAN_VAL_MIXER_LOSS_AVER:
            viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_EXTERNAL_MIXER_CONVERSION_LOSS_LOW, conversionLoss),
                    4, "Conversion Loss");
            break;
        case RSSPECAN_VAL_MIXER_LOSS_TABL:
            viCheckParm(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_EXTERNAL_MIXER_CONVERSION_LOSS_TABLE_LOW, conversionLossTable),
                    5, "Conversion Loss Table");
            break;
        default:
            viCheckParm(RsCore_InvalidViInt32Value(instrSession, range), 3, "Conversion Loss Mode");
        }

        break;
    case 2:
        switch (conversionLossMode)
        {
        case RSSPECAN_VAL_MIXER_LOSS_AVER:
            viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_EXTERNAL_MIXER_CONVERSION_LOSS_HIGH, conversionLoss),
                    4, "Conversion Loss");
            break;
        case RSSPECAN_VAL_MIXER_LOSS_TABL:
            viCheckParm(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_EXTERNAL_MIXER_CONVERSION_LOSS_TABLE_HIGH, conversionLossTable),
                    5, "Conversion Loss Table");
            break;
        default:
            viCheckParm(RsCore_InvalidViInt32Value(instrSession, range), 3, "Conversion Loss Mode");
        }
        break;
    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, range), 2, "Range");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN This function selects a conversion loss table and sets it for
/// HIFN the active band.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR lowPower/Sets the LO level of the external mixer LO port.
ViStatus _VI_FUNC rsspecan_ConfigureExternalMixerLowPower(ViSession instrSession,
                                                          ViReal64  lowPower)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_EXTERNAL_MIXER_LO_POWER, lowPower),
            2, "Low POwer");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN This function creates or overwrites conversion loss table of
/// HIFN external mixer.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR tableName/Enter name of table (without extention) to edit or create. A new
/// HIPAR tableName/table will be created if the specified name does not exist.
/// HIPAR mixerType/Defines the type designation of the mixer in the Conversion Loss
/// HIPAR mixerType/Table.
/// HIPAR band/Defines waveguide band in the Conversion Loss table.
/// HIPAR serialNumber/Enter the serial number of the mixer.
/// HIPAR harmonic/Defines the harmonic of the mixer in the Conversion Loss Table.
/// HIPAR ports/Specifies the mixer type (two-port mixer or three-port mixer).
/// HIPAR bias/Enters the bias current.
/// HIPAR comment/Enter the conevrsion loss table comment.
/// HIPAR numberOfValues/Inputs number of reference value pairs to be send to the
/// HIPAR numberOfValues/instrument.
/// HIPAR frequencyValues/Specifies array of frequency values in Hz.
/// HIPAR conversionLossValues/Specifies array of level values in dB.
ViStatus _VI_FUNC rsspecan_CreateExternalMixerConversionLossTable(
                            ViSession instrSession,
                            ViInt32   reserved,
                            ViString  tableName,
                            ViString  mixerType,
                            ViInt32   band,
                            ViString  serialNumber,
                            ViInt32   harmonic,
                            ViInt32   ports,
                            ViReal64  bias,
                            ViString  comment,
                            ViInt32   numberOfValues,
                            ViReal64  frequencyValues[],
                            ViReal64  conversionLossValues[])
{
    ViStatus error = VI_SUCCESS;
             reserved;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_CVL_NAME, tableName),
            3, "Table Name");

    viCheckParm(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_CVL_MIXER, mixerType),
            4, "Mixer Type");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_CVL_BAND, band),
            5, "Band");

    viCheckParm(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_CVL_MIXER_SNUM, serialNumber),
            6, "Serial Number");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_CVL_HARM, harmonic),
            7, "Harmonic");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_CVL_PORT_MIXER, ports),
            8, "Ports");

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_CVL_BIAS, bias),
            9, "Bias");

    viCheckParm(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_CVL_COMMENT, comment),
            10, "Comment");

    if (numberOfValues > 0)
    {
        checkErr(RsCore_WriteAsciiViReal64ArraysInterleaved(instrSession, "CORR:CVL:DATA ",
            numberOfValues, frequencyValues, conversionLossValues, NULL, NULL, NULL, NULL));
    }

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function selects a conversion loss table of external mixer.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR tableName/Enter name of table (without extention) to select.
ViStatus _VI_FUNC rsspecan_ExternalMixerConversionLossTableSelect(
                            ViSession instrSession,
                            ViInt32   reserved,
                            ViString  tableName)
{
    ViStatus error = VI_SUCCESS;
             reserved;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_CVL_NAME, tableName),
            3, "Table Name");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function deletes a conversion loss table of external mixer.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR tableName/Enter name of table (without extention) to delete.
ViStatus _VI_FUNC rsspecan_ExternalMixerConversionLossTableDelete(
                            ViSession instrSession,
                            ViInt32   reserved,
                            ViString  tableName)
{
    ViStatus error = VI_SUCCESS;
             reserved;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_CVL_NAME, tableName),
            3, "Table Name");

    viCheckParm(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_CVL_CLEAR, tableName),
            3, "Table Name");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function queries the names of all conversion loss tables
/// HIFN saved on the hard disk..
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR numberofCVL/This control returns number of the names of all digital
/// HIPAR numberofCVL/standards available.
/// HIPAR bufferSize/Pass the number of bytes in the string you specify for the
/// HIPAR bufferSize/catalog list parameter.
/// HIPAR CVLNamesList/This control the names of all conversion loss tables saved on
/// HIPAR CVLNamesList/the hard disk.
ViStatus _VI_FUNC rsspecan_ExternalMixerConversionLossTableCatalog(
                            ViSession instrSession,
                            ViInt32   *numberofCVL,
                            ViInt32   bufferSize,
                            ViChar    CVLNamesList[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   *buffer = NULL;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K90|B21"));

    viCheckParm(RsCore_InvalidViInt32RangeMin(instrSession, bufferSize, 0),
        3, "Buffer Size");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, CVLNamesList),
        4, "CVL Names List");

    checkErr(RsCore_QueryViStringUnknownLength(instrSession, ":CORR:CVL:CAT?", &buffer));
    RsCore_TrimString(buffer, RS_VAL_TRIM_WHITESPACES_AND_SINGLE_QUOTES);
    checkErr(rsspecan_ParseLimitLineCatalog(buffer, bufferSize, CVLNamesList, numberofCVL));

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    if (buffer) free(buffer);
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function queries the frequency at which the external mixer
/// HIFN band starts and stops.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR RFStart/Queries the frequency at which the external mixer band starts.
/// HIPAR RFStop/Queries the frequency at which the external mixer band stops.
ViStatus _VI_FUNC rsspecan_QueryExternalMixerRFStartStop(ViSession instrSession,
                                                         ViReal64  *RFStart,
                                                         ViReal64  *RFStop)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_GET_EXTERNAL_MIXER_RF_FREQUENCY_START, RFStart),
            2, "RF Start");

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_GET_EXTERNAL_MIXER_RF_FREQUENCY_STOP, RFStop),
            3, "RF Stop");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Enables or disables the general use of an external frontend for the
/// HIFN  application.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR externalFrontend/Enables or disables the general use of an external frontend for the
/// HIPAR externalFrontend/application.
ViStatus _VI_FUNC rsspecan_ConfigureExternalFrontendEnabled(ViSession instrSession,
                                                            ViBoolean externalFrontend)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, NULL, RSSPECAN_ATTR_EXTERNAL_FRONTEND_ENABLED, externalFrontend),
            2, "External Frontend");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Configures the connection to the external frontend.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR type/String in double quotes containing the type of frontend to be
/// HIPAR type/connected.
/// HIPAR ipAddress/The IP address or computer name of the frontend connected to the R&S
/// HIPAR ipAddress/FSW via LAN. The IP address and computer name are indicated on the
/// HIPAR ipAddress/electronic ink display on the side panel of the frontend.
/// HIPAR deviceID/Unique device ID consisting of <type>-<serialnumber>
/// HIPAR symbolicName/Symbolic name of the external frontend.
ViStatus _VI_FUNC rsspecan_ConfigureExternalFrontendConnectionSettings(
                            ViSession instrSession,
                            ViString  type,
                            ViString  ipAddress,
                            ViString  deviceID,
                            ViString  symbolicName)
{
    ViStatus error = VI_SUCCESS;
    ViChar   param[RS_PARAM_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    (void)snprintf(param, RS_PARAM_BUF_SIZE, "\"%s\",\"%s\",\"%s\",\"%s\"", type, ipAddress, deviceID, symbolicName);

    checkErr(RsCore_SetAttributeViRawString(instrSession, NULL, RSSPECAN_ATTR_HIDDEN_EXTERNAL_FRONTEND_CONNECTION_CONFIGURATION, RS_VAL_DIRECT_USER_CALL, param));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Sets and queries the external frontend connection state in the
/// HIFN  firmware.
/// HIFN
/// HIFN     Note: to query the physical connection state of the external frontend,
/// HIFN  use RSSPECAN_ATTR_EXTERNAL_FRONTEND_CONNECTION_PHYSICAL_STATE.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR firmwareConnection/Sets and queries the external frontend connection state in the
/// HIPAR firmwareConnection/firmware.
ViStatus _VI_FUNC rsspecan_ConfigureExternalFrontendConnectionFirmwareState(
                            ViSession instrSession,
                            ViBoolean firmwareConnection)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, NULL, RSSPECAN_ATTR_EXTERNAL_FRONTEND_CONNECTION_FIRMWARE_STATE, firmwareConnection),
            2, "Firmware Connection");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the status of the physical connection to the external
/// HIFN  frontend.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR physicalConnection/Queries the status of the physical connection to the external
/// HIPAR physicalConnection/frontend.
ViStatus _VI_FUNC rsspecan_QueryExternalFrontendConnectionPhysicalState(
                            ViSession  instrSession,
                            ViBoolean* physicalConnection)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_GetAttributeViBoolean(instrSession, NULL, RSSPECAN_ATTR_EXTERNAL_FRONTEND_CONNECTION_PHYSICAL_STATE, physicalConnection),
            2, "Physical Connection");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Sets or queries the network information for the frontend.
/// HIFN
/// HIFN     This information is also indicated on the electronic paper display on
/// HIFN  the side panel of the device.
/// HIFN
/// HIFN     Beware that if you change the network setting to DHCP = ON, the
/// HIFN  connection is aborted and you must re-establish a connection to the
/// HIFN  frontend (see
/// HIFN  RSSPECAN_ATTR_EXTERNAL_FRONTEND_CONNECTION_FIRMWARE_STATE).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR ipAddress/IP address of the frontend.
/// HIPAR subnet/Subnet mask of the frontend.
/// HIPAR dhcp/Indicates whether a DHCP server is used.
ViStatus _VI_FUNC rsspecan_ConfigureExternalFrontendNetworkSettings(
                            ViSession instrSession,
                            ViString  ipAddress,
                            ViString  subnet,
                            ViBoolean dhcp)
{
    ViStatus error = VI_SUCCESS;
    ViChar   param[RS_PARAM_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    (void)snprintf(param, RS_PARAM_BUF_SIZE, "\"%s\",\"%s\",%s", ipAddress, subnet, dhcp ? "ON":"OFF");

    checkErr(RsCore_SetAttributeViRawString(instrSession, NULL, RSSPECAN_ATTR_HIDDEN_EXTERNAL_FRONTEND_NETWORK_CONFIGURATION, RS_VAL_DIRECT_USER_CALL, param));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the device identifcation information (*IDN?) of the frontend.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR bufferSize/Size of output array.
/// HIPAR identification/Queries the device identifcation information (*IDN?) of the frontend.
/// HIPAR bytesRead/Returns the number of bytes read from instrument.
ViStatus _VI_FUNC rsspecan_QueryExternalFrontendIdentification(
                            ViSession instrSession,
                            ViInt32   bufferSize,
                            ViChar    identification[],
                            ViInt32*  bytesRead)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeViStringWithRealLen(instrSession, NULL, RSSPECAN_ATTR_EXTERNAL_FRONTEND_IDENTIFICATION, RS_VAL_DIRECT_USER_CALL, bufferSize, identification, bytesRead));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Returns the intermediate frequency (output) range of the external
/// HIFN  frontend to be set with
/// HIFN  RSSPECAN_ATTR_EXTERNAL_FRONTEND_FREQUENCY_BAND_RANGE.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR bufferSize/Size of output array.
/// HIPAR rangeList/Returns the intermediate frequency (output) range of the external
/// HIPAR rangeList/frontend to be set with
/// HIPAR rangeList/RSSPECAN_ATTR_EXTERNAL_FRONTEND_FREQUENCY_BAND_RANGE.
/// HIPAR bytesRead/Returns the number of bytes read from instrument.
ViStatus _VI_FUNC rsspecan_QueryExternalFrontendFrequencyBandRangeList(
                            ViSession instrSession,
                            ViInt32   bufferSize,
                            ViChar    rangeList[],
                            ViInt32*  bytesRead)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeViStringWithRealLen(instrSession, NULL, RSSPECAN_ATTR_EXTERNAL_FRONTEND_FREQUENCY_BAND_RANGE_LIST, RS_VAL_DIRECT_USER_CALL, bufferSize, rangeList, bytesRead));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Defines the intermediate frequency (output) range of the external
/// HIFN  frontend.  Query
/// HIFN  RSSPECAN_ATTR_EXTERNAL_FRONTEND_FREQUENCY_BAND_RANGE_LIST to see
/// HIFN  acceptable values.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR range/Defines the intermediate frequency (output) range of the external
/// HIPAR range/frontend.  Query
/// HIPAR range/RSSPECAN_ATTR_EXTERNAL_FRONTEND_FREQUENCY_BAND_RANGE_LIST to see
/// HIPAR range/acceptable values.
ViStatus _VI_FUNC rsspecan_ConfigureExternalFrontendFrequencyBandRange(
                            ViSession instrSession,
                            ViString  range)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViString(instrSession, NULL, RSSPECAN_ATTR_EXTERNAL_FRONTEND_FREQUENCY_BAND_RANGE, range),
            2, "Range");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Determines whether the frequency band of the external frontend is
/// HIFN  configured automatically or manually.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR autoFrequencyBand/Determines whether the frequency band of the external frontend is
/// HIPAR autoFrequencyBand/configured automatically or manually.
ViStatus _VI_FUNC rsspecan_ConfigureExternalFrontendFrequencyBandAutoEnabled(
                            ViSession instrSession,
                            ViBoolean autoFrequencyBand)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, NULL, RSSPECAN_ATTR_EXTERNAL_FRONTEND_FREQUENCY_BAND_AUTO_ENABLED, autoFrequencyBand),
            2, "Auto Frequency Band");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the number of frequency bands provided by the selected
/// HIFN  frontend.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR frequencyBandCount/Queries the number of frequency bands provided by the selected
/// HIPAR frequencyBandCount/frontend.
ViStatus _VI_FUNC rsspecan_QueryExternalFrontendFrequencyBandCount(
                            ViSession instrSession,
                            ViInt32*  frequencyBandCount)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_GetAttributeViInt32(instrSession, NULL, RSSPECAN_ATTR_EXTERNAL_FRONTEND_FREQUENCY_BAND_COUNT, frequencyBandCount),
            2, "Frequency Band Count");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the start and the end of the frequency range supported by the
/// HIFN  selected frontend frequency band.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR frequencyBand/Sets the repeated capability value for Frequency Band.
/// HIPAR lower/Queries the start of the frequency range supported by the selected
/// HIPAR lower/frontend frequency band.
/// HIPAR upper/Queries the end of the frequency range supported by the selected
/// HIPAR upper/frontend frequency band.
ViStatus _VI_FUNC rsspecan_QueryExternalFrontendFrequencyBandRange(
                            ViSession instrSession,
                            ViInt32   frequencyBand,
                            ViReal64* lower,
                            ViReal64* upper)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, NULL, RSSPECAN_ATTR_EXTERNAL_FRONTEND_SELECTED_FREQUENCY_BAND, frequencyBand),
            2, "Frequency Band");

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, NULL, RSSPECAN_ATTR_EXTERNAL_FRONTEND_FREQUENCY_BAND_RANGE_LOWER, lower),
            3, "Lower");

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, NULL, RSSPECAN_ATTR_EXTERNAL_FRONTEND_FREQUENCY_BAND_RANGE_UPPER, upper),
            3, "Upper");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the currently used sideband for frequency conversion.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR bufferSize/Size of output array.
/// HIPAR frequencySideband/Queries the currently used sideband for frequency conversion.
/// HIPAR bytesRead/Returns the number of bytes read from instrument.
ViStatus _VI_FUNC rsspecan_QueryExternalFrontendIntermediateFrequencySideband(
                            ViSession instrSession,
                            ViInt32   bufferSize,
                            ViChar    frequencySideband[],
                            ViInt32*  bytesRead)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeViStringWithRealLen(instrSession, NULL, RSSPECAN_ATTR_EXTERNAL_FRONTEND_INTERMEDIATE_FREQUENCY_SIDEBAND, RS_VAL_DIRECT_USER_CALL, bufferSize, frequencySideband, bytesRead));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the currently used intermediate frequency (IF) for frequency
/// HIFN  conversion.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR intermediateFrequency/Queries the currently used intermediate frequency (IF) for frequency
/// HIPAR intermediateFrequency/conversion.
ViStatus _VI_FUNC rsspecan_QueryExternalFrontendIntermediateFrequency(
                            ViSession instrSession,
                            ViReal64* intermediateFrequency)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, NULL, RSSPECAN_ATTR_EXTERNAL_FRONTEND_INTERMEDIATE_FREQUENCY, intermediateFrequency),
            2, "Intermediate Frequency");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the available reference signals for the connected frontend
/// HIFN  type.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR bufferSize/Size of output array.
/// HIPAR referenceFrequencyList/Queries the available reference signals for the connected frontend
/// HIPAR referenceFrequencyList/type.
/// HIPAR bytesRead/Returns the number of bytes read from instrument.
ViStatus _VI_FUNC rsspecan_QueryExternalFrontendReferenceFrequencyList(
                            ViSession instrSession,
                            ViInt32   bufferSize,
                            ViChar    referenceFrequencyList[],
                            ViInt32*  bytesRead)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeViStringWithRealLen(instrSession, NULL, RSSPECAN_ATTR_EXTERNAL_FRONTEND_REFERENCE_FREQUENCY_LIST, RS_VAL_DIRECT_USER_CALL, bufferSize, referenceFrequencyList, bytesRead));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Sets the reference frequency that is used for frequency conversion on
/// HIFN  the frontend. Depending on the connected type of frontend, different
/// HIFN  values are available. To determine which reference levels are
/// HIFN  available, use
/// HIFN  RSSPECAN_ATTR_EXTERNAL_FRONTEND_REFERENCE_FREQUENCY_LIST.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR referenceFrequency/Sets the reference frequency that is used for frequency conversion on
/// HIPAR referenceFrequency/the frontend. Depending on the connected type of frontend, different
/// HIPAR referenceFrequency/values are available. To determine which reference levels are
/// HIPAR referenceFrequency/available, use
/// HIPAR referenceFrequency/RSSPECAN_ATTR_EXTERNAL_FRONTEND_REFERENCE_FREQUENCY_LIST.
ViStatus _VI_FUNC rsspecan_ConfigureExternalFrontendReferenceFrequency(
                            ViSession instrSession,
                            ViReal64  referenceFrequency)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, NULL, RSSPECAN_ATTR_EXTERNAL_FRONTEND_REFERENCE_FREQUENCY, referenceFrequency),
            2, "Reference Frequency");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Updates the firmware on the external frontend. Note that this process
/// HIFN  can take some time.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsspecan_ExternalFrontendUpdateFirmware(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsspecan_SetAttributeViString(instrSession, NULL, RSSPECAN_ATTR_EXTERNAL_FRONTEND_UPDATE_FIRMWARE, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Activates correction of the IF signal due to cable loss from the
/// HIFN  frontend to the analyzer and specifies the file with correction data.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR correction/Activates correction of the IF signal due to cable loss from the
/// HIPAR correction/frontend to the analyzer. Specify the file with correction data using
/// HIPAR correction/RSSPECAN_ATTR_EXTERNAL_FRONTEND_ALIGNMENT_CORRECTION_FILE.
/// HIPAR correctionFile/Selects or queries the touchstone file that contains correction data
/// HIPAR correctionFile/to compensate for signal losses in the cable occurring at different IF
/// HIPAR correctionFile/signal frequencies.
ViStatus _VI_FUNC rsspecan_ConfigureExternalFrontendAlignmentCorrection(
                            ViSession instrSession,
                            ViBoolean correction,
                            ViString  correctionFile)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, NULL, RSSPECAN_ATTR_EXTERNAL_FRONTEND_ALIGNMENT_CORRECTION_FILE_ENABLED, correction),
            2, "Correction");

    if(correction)
        viCheckParm(rsspecan_SetAttributeViString(instrSession, NULL, RSSPECAN_ATTR_EXTERNAL_FRONTEND_ALIGNMENT_CORRECTION_FILE, correctionFile),
                3, "Correction File");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Performs a selftest on the frontend to compare the current performance
/// HIFN  and characteristic values with the specified values for the frontend.
/// HIFN
/// HIFN     As a result, the success is returned.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR selftestResult/Performs a selftest on the frontend to compare the current performance
/// HIPAR selftestResult/and characteristic values with the specified values for the frontend.
ViStatus _VI_FUNC rsspecan_PerformExternalFrontendDiagnosticsSelftest(
                            ViSession instrSession,
                            ViInt32*  selftestResult)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_GetAttributeViInt32(instrSession, NULL, RSSPECAN_ATTR_EXTERNAL_FRONTEND_DIAGNOSTICS_START_SELFTEST, selftestResult),
            2, "Selftest Result");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the results of the selftest on the frontend.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR bufferSize/Size of output array.
/// HIPAR results/Queries the results of the selftest on the frontend.
/// HIPAR bytesRead/Returns the number of bytes read from instrument.
ViStatus _VI_FUNC rsspecan_QueryExternalFrontendDiagnosticsSelftestResults(
                            ViSession instrSession,
                            ViInt32   bufferSize,
                            ViChar    results[],
                            ViInt32*  bytesRead)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeViStringWithRealLen(instrSession, NULL, RSSPECAN_ATTR_EXTERNAL_FRONTEND_DIAGNOSTICS_SELFTEST_RESULTS, RS_VAL_DIRECT_USER_CALL, bufferSize, results, bytesRead));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the name of the external device for reference.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR bufferSize/Size of output array.
/// HIPAR deviceName/Queries the name of the external device for reference.
/// HIPAR bytesRead/Returns the number of bytes read from instrument.
ViStatus _VI_FUNC rsspecan_QueryExternalFrontendDeviceName(ViSession instrSession,
                                                           ViInt32   bufferSize,
                                                           ViChar    deviceName[],
                                                           ViInt32*  bytesRead)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeViStringWithRealLen(instrSession, NULL, RSSPECAN_ATTR_EXTERNAL_FRONTEND_DEVICE_NAME, RS_VAL_DIRECT_USER_CALL, bufferSize, deviceName, bytesRead));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the names of all detected connected devices at the external
/// HIFN  frontend.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR bufferSize/Size of output array.
/// HIPAR nameList/Queries the names of all detected connected devices at the external
/// HIPAR nameList/frontend.
/// HIPAR bytesRead/Returns the number of bytes read from instrument.
ViStatus _VI_FUNC rsspecan_QueryExternalFrontendDeviceNameList(
                            ViSession instrSession,
                            ViInt32   bufferSize,
                            ViChar    nameList[],
                            ViInt32*  bytesRead)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeViStringWithRealLen(instrSession, NULL, RSSPECAN_ATTR_EXTERNAL_FRONTEND_DEVICE_NAME_LIST, RS_VAL_DIRECT_USER_CALL, bufferSize, nameList, bytesRead));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the type of the detected device for reference.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR bufferSize/Size of output array.
/// HIPAR type/Queries the type of the detected device for reference.
/// HIPAR bytesRead/Returns the number of bytes read from instrument.
ViStatus _VI_FUNC rsspecan_QueryExternalFrontendDeviceType(ViSession instrSession,
                                                           ViInt32   bufferSize,
                                                           ViChar    type[],
                                                           ViInt32*  bytesRead)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeViStringWithRealLen(instrSession, NULL, RSSPECAN_ATTR_EXTERNAL_FRONTEND_DEVICE_TYPE, RS_VAL_DIRECT_USER_CALL, bufferSize, type, bytesRead));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the lower and upper limit of the supported frequency range of
/// HIFN  the connected device.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR lower/Queries the lower limit of the supported frequency range of the
/// HIPAR lower/connected device.
/// HIPAR upper/Queries the upper limit of the supported frequency range of the
/// HIPAR upper/connected device.
ViStatus _VI_FUNC rsspecan_QueryExternalFrontendDeviceFrequencyRange(
                            ViSession instrSession,
                            ViReal64* lower,
                            ViReal64* upper)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, NULL, RSSPECAN_ATTR_EXTERNAL_FRONTEND_DEVICE_FREQUENCY_RANGE_LOWER, lower),
            2, "Lower");

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, NULL, RSSPECAN_ATTR_EXTERNAL_FRONTEND_DEVICE_FREQUENCY_RANGE_UPPER, upper),
            3, "Upper");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the gain provided by a connected external amplifier.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR gain/Queries the gain provided by a connected external amplifier.
ViStatus _VI_FUNC rsspecan_QueryExternalFrontendDeviceGain(ViSession instrSession,
                                                           ViReal64* gain)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, NULL, RSSPECAN_ATTR_EXTERNAL_FRONTEND_DEVICE_GAIN, gain),
            2, "Gain");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Scans the ports of the external frontend and refreshes the display
/// HIFN  with detected information on connected devices. This is useful after
/// HIFN  connecting a new device to the frontend.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsspecan_ExternalFrontendDeviceRefresh(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsspecan_SetAttributeViString(instrSession, NULL, RSSPECAN_ATTR_EXTERNAL_FRONTEND_DEVICE_REFRESH, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Configures the use of a correction file for the connected external
/// HIFN  devices.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR correction/Enables or disables the use of a correction file for the connected
/// HIPAR correction/external devices. The file is defined by
/// HIPAR correction/RSSPECAN_ATTR_EXTERNAL_FRONTEND_DEVICE_CORRECTION_FILE.
/// HIPAR correctionFile/Defines the correction file to compensate for signal losses by the
/// HIPAR correctionFile/external devices occurring at different IF signal frequencies.
ViStatus _VI_FUNC rsspecan_ConfigureExternalFrontendDeviceCorrection(
                            ViSession instrSession,
                            ViBoolean correction,
                            ViString  correctionFile)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, NULL, RSSPECAN_ATTR_EXTERNAL_FRONTEND_DEVICE_CORRECTION_ENABLED, correction),
            2, "Correction");

    if (correction)
        viCheckParm(rsspecan_SetAttributeViString(instrSession, NULL, RSSPECAN_ATTR_EXTERNAL_FRONTEND_DEVICE_CORRECTION_FILE, correctionFile),
                3, "Correction File");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries whether the loaded correction file for an external device is
/// HIFN  valid or not.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR fileValidity/Queries whether the loaded correction file for an external device is
/// HIPAR fileValidity/valid or not.
ViStatus _VI_FUNC rsspecan_QueryExternalFrontendDeviceCorrectionFileValid(
                            ViSession  instrSession,
                            ViBoolean* fileValidity)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_GetAttributeViBoolean(instrSession, NULL, RSSPECAN_ATTR_EXTERNAL_FRONTEND_DEVICE_CORRECTION_FILE_VALID, fileValidity),
            2, "File Validity");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Enables or disables a digital output stream to the optional QSFP+
/// HIFN  connector, if available.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR enabled/Enables or disables a digital output stream to the optional QSFP+
/// HIPAR enabled/connector, if available.
ViStatus _VI_FUNC rsspecan_ConfigureTransientDigitalIQ40GEnabled(
                            ViSession instrSession,
                            ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, NULL, RSSPECAN_ATTR_TRANSIENT_DIGITAL_IQ_40G_ENABLED, enabled),
            2, "Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Returns the currently used sample rate to transfer data via the
/// HIFN  Digital I/Q 40G Streaming Output interface.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR sampleRate/Returns the currently used sample rate to transfer data via the
/// HIPAR sampleRate/Digital I/Q 40G Streaming Output interface.
ViStatus _VI_FUNC rsspecan_QueryTransientDigitalIQ40GSampleRate(
                            ViSession instrSession,
                            ViReal64* sampleRate)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, NULL, RSSPECAN_ATTR_TRANSIENT_DIGITAL_IQ_40G_SAMPLE_RATE, sampleRate),
            2, "Sample Rate");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Returns a comma-separated list of information on the instrument
/// HIFN  connected to the QSFP+ connector, if available.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR bufferSize/The size of the buffer passed to the Connected Device parameter.
/// HIPAR connectedDevice/Returns a comma-separated list of information on the instrument
/// HIPAR connectedDevice/connected to the QSFP+ connector, if available.
ViStatus _VI_FUNC rsspecan_QueryTransientDigitalIQ40GConnectedDevice(
                            ViSession instrSession,
                            ViInt32   bufferSize,
                            ViChar    connectedDevice[])
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_GetAttributeViString(instrSession, NULL, RSSPECAN_ATTR_TRANSIENT_DIGITAL_IQ_40G_CONNECTED_DEVICE,
            bufferSize, connectedDevice),
            2, "Connected Device");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function returns the repeated capability id(s) that
/// HIFN belongs to the attribute you specify. More then one id is listed
/// HIFN in comma separated string.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR attributeId/Pass the ID of an attribute.
/// HIPAR bufferSize/Pass the number of bytes in the ViChar array you specify for the
/// HIPAR bufferSize/Repeated Capability Id(s) parameter.
/// HIPAR repeatedCapabilityIds/The buffer in which the function returns the repeated capability
/// HIPAR repeatedCapabilityIds/id(s) belonging to the attribute. The buffer must be of type
/// HIPAR repeatedCapabilityIds/ViChar and have at least as many bytes as indicated in the
/// HIPAR repeatedCapabilityIds/Buffer Size parameter.
ViStatus _VI_FUNC rsspecan_GetAttributeRepeatedCapabilityIds(ViSession instrSession,
                                                             ViAttr    attributeId,
                                                             ViInt32   bufferSize,
                                                             ViChar    repeatedCapabilityIds[])
{
    ViStatus error = VI_SUCCESS;

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, bufferSize, 1, RS_MAX_SHORT_MESSAGE_BUF_SIZE),
            3, "Buffer Size");

    checkErr(RsCore_GetAttributeRepCapNameIds(instrSession, attributeId, bufferSize, repeatedCapabilityIds));

Error:
    return error;
}

/// HIFN This function returns the repeated capability name(s) that
/// HIFN belongs to the attribute and single repeated capability id you
/// HIFN may specify. More then one name is listed in comma separated
/// HIFN string.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR attributeId/Pass the ID of an attribute.
/// HIPAR repeatedCapabilityId/If the attribute applies to a repeated capability such as a
/// HIPAR repeatedCapabilityId/trace, this parameter defines the Id of the repeated capability
/// HIPAR repeatedCapabilityId/on which to query repeated capability id name(s).
/// HIPAR bufferSize/Pass the number of bytes in the ViChar array you specify for the
/// HIPAR bufferSize/Repeated Capability Id Name(s) parameter.
/// HIPAR repeatedCapabilityIdNames/The buffer in which the function returns the repeated capability
/// HIPAR repeatedCapabilityIdNames/id name(s) belonging to the attribute. The buffer must be of
/// HIPAR repeatedCapabilityIdNames/type ViChar and have at least as many bytes as indicated in the
/// HIPAR repeatedCapabilityIdNames/Buffer Size parameter.
ViStatus _VI_FUNC rsspecan_GetAttributeRepeatedCapabilityIdNames(
                            ViSession instrSession,
                            ViAttr    attributeId,
                            ViString  repeatedCapabilityId,
                            ViInt32   bufferSize,
                            ViChar    repeatedCapabilityIdNames[])
{
    ViStatus error = VI_SUCCESS;

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, bufferSize, 1, RS_MAX_SHORT_MESSAGE_BUF_SIZE),
            4, "Buffer Size");

    checkErr(RsCore_GetAttributeRepCapNamesAll(instrSession,
        attributeId,
        repeatedCapabilityId,
        bufferSize,
        repeatedCapabilityIdNames));

Error:
    return error;
}

/// HIFN Function used by Hi-level functions to obtain a repCap name from:
/// HIFN - attributeId
/// HIFN - repCapNameID
/// HIFN - 0-based index in the repCapNames
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR attributeId/Pass the ID of an attribute.
/// HIPAR repCapNameId/RepCap Name ID.
/// HIPAR index/Zero based index in the repCapNames.
/// HIPAR bufferSize/Pass the number of bytes in the string you allocate for the repCapName.
/// HIPAR repCapName/String returning RepCap name
ViStatus _VI_FUNC rsspecan_GetAttributeRepCapName(ViSession instrSession,
                                                  ViAttr    attributeId,
                                                  ViString  repCapNameId,
                                                  ViInt32   index,
                                                  ViInt32   bufferSize,
                                                  ViChar    repCapName[])
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, attributeId, repCapNameId, index, bufferSize, repCapName));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/*****************************************************************************
 *-------------------- Status Register Group Functions  ---------------------*
 *****************************************************************************/

/// HIFN This Function sets the Enable, NTransition, and PTransition bits
/// HIFN of the Main, Power, Limit, Left Margin, and Sync status
/// HIFN questionable registers.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR enable/Document parameter here.
/// HIPAR PTransition/Document parameter here.
/// HIPAR NTransition/Document parameter here.
ViStatus _VI_FUNC rsspecan_setStatusRegister(ViSession instrSession,
                                             ViInt32   registerOperation,
                                             ViInt32   questionableRegister,
                                             ViInt32   enable,
                                             ViInt32   PTransition,
                                             ViInt32   NTransition)
{
    ViStatus error = VI_SUCCESS;
    ViChar   *p2rtString;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    switch (registerOperation)
    {
    case RSSPECAN_VAL_STAT_REG_PRESET:
        //checkErr(RsCore_Write(instrSession, ":STAT:PRES"));
        checkErr(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_STATUS_PRESET, NULL));
        break;

    case RSSPECAN_VAL_STAT_REG_QUESTIONABLE:
        viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &rsspecan_rngStatusRegister, questionableRegister, &p2rtString),
                    3, "Questionable Register");
        switch (questionableRegister)
        {
        case RSSPECAN_VAL_STAT_REG_MAIN:
            snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, ":STAT:QUES:ENAB %ld;PTR %ld;NTR %ld", enable, PTransition, NTransition);
            checkErr(RsCore_Write(instrSession, cmd));
            break;

        default:
            snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, ":STAT:QUES:%s:ENAB %ld;PTR %ld;NTR %ld", p2rtString, enable, PTransition, NTransition);
            checkErr(RsCore_Write(instrSession, cmd));
        }
        break;

    case RSSPECAN_VAL_STAT_REG_OPERATION:
        viCheckParm(RsCore_InvalidViInt32Range(instrSession, enable, 0, 65535),
                4, "Enable");
        viCheckParm(RsCore_InvalidViInt32Range(instrSession, PTransition, 0, 65535),
                5, "PTransition");
        viCheckParm(RsCore_InvalidViInt32Range(instrSession, NTransition, 0, 65535),
                6, "NTransition");

        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, ":STAT:OPER:ENAB %ld;PTR %ld;NTR %ld", enable, PTransition, NTransition);
        checkErr(RsCore_Write(instrSession, cmd));
        break;

    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, registerOperation), 2, "Register Operation");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN This function contains the commands for the querying status     */
/// HIFN reporting system.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR registerOperation/Specifies the operation to perform.
/// HIPAR questionableRegister/This control selects the registers that will be configured.
/// HIPAR registerPart/Sets the register part.
/// HIPAR registerValue/Returns content of selected register.
ViStatus _VI_FUNC rsspecan_getStatusRegisterQuery(ViSession instrSession,
                                                  ViInt32   registerOperation,
                                                  ViInt32   questionableRegister,
                                                  ViInt32   registerPart,
                                                  ViInt32   *registerValue)
{
    ViStatus error = VI_SUCCESS;
    ViChar   *p2rtString;
    ViChar   *p2registerPart;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, registerPart, 0, 1),
            4, "Register Part");
    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &rsspecan_rngStatusRegisterQuery, registerPart, &p2registerPart),
            4, "Register Part");

    switch (registerOperation)
    {
    case RSSPECAN_VAL_STAT_REG_QUESTIONABLE:
        viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &rsspecan_rngStatusRegister, questionableRegister, &p2rtString),
                    3, "Questionable Register");
        switch (questionableRegister)
        {
        case RSSPECAN_VAL_STAT_REG_MAIN:
            snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, ":STAT:QUES:%s?", p2registerPart);
            checkErr(RsCore_QueryViInt32(instrSession, cmd, registerValue));
            break;
        default:
            snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, ":STAT:QUES:%s:%s?", p2rtString, p2registerPart);
            checkErr(RsCore_QueryViInt32(instrSession, cmd, registerValue));
        }

        break;
    case RSSPECAN_VAL_STAT_REG_OPERATION:
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, ":STAT:OPER:%s?", p2registerPart);
        checkErr(RsCore_QueryViInt32(instrSession, cmd, registerValue));
        break;
    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, registerOperation), 2, "Register Operation");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/*===========================================================================*
 *= RsSpecAn Close ==========================================================*
 *===========================================================================*/
/// HIFN This function closes the instrument.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.
ViStatus _VI_FUNC rsspecan_close(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_ViClose(instrSession));

Error:
    (void)RsCore_Dispose(instrSession);
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/*****************************************************************************
 *------------------------ Global Session Callbacks -------------------------*
 *****************************************************************************/

/*****************************************************************************
 *------------------- End Instrument Driver Source Code ---------------------*
 *****************************************************************************/
