/****************************************************************************
 *
 *  Rohde & Schwarz Spectrum Analyzer instrument driver include file
 *
 ****************************************************************************/

#ifndef __RSSPECAN_HEADER
#define __RSSPECAN_HEADER

#include "rscore.h"
#include "rsspecan_attributes.h"
#include "rsspecan_utility.h"

#include <float.h>
#include <math.h>

#include <string.h>
#include <stdlib.h>
#include <stdio.h>

#if defined(__cplusplus) || defined(__cplusplus__)
extern "C" {
#endif

    /*****************************************************************************
     *- Useful Macros & Constants -----------------------------------------------*
     *****************************************************************************/
#define RSSPECAN_SIMULATION_ID_QUERY      "Rohde&Schwarz,FSW-43,123456/789,2.00"  /* Default definition of ID Query Response for simulation */
#define RSSPECAN_VALID_ID_RESPONSE_STRING "Rohde&Schwarz"      /* Valid response for identification query */
#define RSSPECAN_SIMULATION_OPT_QUERY     ""      /* Simulated response for *OPT? command */

#define RSSPECAN_OPC_TIMEOUT              15000L   /* Maximum time to wait for OPC in milliseconds */
#define RSSPECAN_DEFAULT_DATA_BUFFER_SIZE 32001L   /* I/O buffer size for trace transfer if not defined*/

     /****************************************************************************
      *----------------- Instrument Driver Revision Information -----------------*
      ****************************************************************************/

      /* Class Driver Identification */

#define RSSPECAN_CLASS_DRIVER_DESCRIPTION           ""
#define RSSPECAN_CLASS_DRIVER_PREFIX                ""
#define RSSPECAN_CLASS_DRIVER_VENDOR                ""
#define RSSPECAN_CLASS_DRIVER_REVISION              ""
#define RSSPECAN_CLASS_SPEC_MAJOR_VERSION           1L    /* Class specification major version */
#define RSSPECAN_CLASS_SPEC_MINOR_VERSION           0L    /* Class specification minor version */

/* Driver Identification */

#define RSSPECAN_SPECIFIC_DRIVER_DESCRIPTION        ""
#define RSSPECAN_SPECIFIC_DRIVER_PREFIX             "RSSPECAN"
#define RSSPECAN_SPECIFIC_DRIVER_LOCATOR            ""
#define RSSPECAN_SPECIFIC_DRIVER_VENDOR             ""
#define RSSPECAN_MAJOR_VERSION                      5L    /* Instrument driver major version          */
#define RSSPECAN_MINOR_VERSION                      20L   /* Instrument driver minor version          */
#define RSSPECAN_MINOR_MINOR_VERSION                0L    /* Instrument driver minor minor version    */

/* Driver Capabilities */

#define RSSPECAN_SUPPORTED_INSTRUMENT_MODELS        "FSV,FSVR,FSW,FSP,FSWP,FSVA,FPL,FSMR,FSPN"    /* Instrument driver supported model(s)     */
#define RSSPECAN_GROUP_CAPABILITIES                 ""
#define RSSPECAN_FUNCTION_CAPABILITIES              ""

/****************************************************************************
 *---------------------------- Attribute Defines ---------------------------*
 ****************************************************************************/

 /*- Inherent Instrument Specific Attributes ----------------------------*/

     /* User Options */

#define RSSPECAN_ATTR_RANGE_CHECK                                           RS_ATTR_RANGE_CHECK                                     /* ViBoolean */
#define RSSPECAN_ATTR_QUERY_INSTRUMENT_STATUS                               RS_ATTR_QUERY_INSTRUMENT_STATUS                         /* ViBoolean */
#define RSSPECAN_ATTR_CACHE                                                 RS_ATTR_CACHE                                           /* ViBoolean */
#define RSSPECAN_ATTR_SIMULATE                                              RS_ATTR_SIMULATE                                        /* ViBoolean */
#define RSSPECAN_ATTR_RECORD_COERCIONS                                      RS_ATTR_RECORD_COERCIONS                                /* ViBoolean */
#define RSSPECAN_ATTR_INTERCHANGE_CHECK                                     RS_ATTR_INTERCHANGE_CHECK                               /* ViBoolean */
#define RSSPECAN_ATTR_SPY                                                   RS_ATTR_SPY                                             /* ViBoolean */
#define RSSPECAN_ATTR_USE_SPECIFIC_SIMULATION                               RS_ATTR_USE_SPECIFIC_SIMULATION                         /* ViBoolean */

        /* Class Driver Identification */

#define RSSPECAN_ATTR_CLASS_DRIVER_DESCRIPTION                              RS_ATTR_CLASS_DRIVER_DESCRIPTION                        /* ViString, read-only */
#define RSSPECAN_ATTR_CLASS_DRIVER_PREFIX                                   RS_ATTR_CLASS_DRIVER_PREFIX                             /* ViString, read-only */
#define RSSPECAN_ATTR_CLASS_DRIVER_VENDOR                                   RS_ATTR_CLASS_DRIVER_VENDOR                             /* ViString, read-only */
#define RSSPECAN_ATTR_CLASS_DRIVER_REVISION                                 RS_ATTR_CLASS_DRIVER_REVISION                           /* ViString, read-only */
#define RSSPECAN_ATTR_CLASS_DRIVER_CLASS_SPEC_MAJOR_VERSION                 RS_ATTR_CLASS_DRIVER_CLASS_SPEC_MAJOR_VERSION           /* ViInt32,  read-only */
#define RSSPECAN_ATTR_CLASS_DRIVER_CLASS_SPEC_MINOR_VERSION                 RS_ATTR_CLASS_DRIVER_CLASS_SPEC_MINOR_VERSION           /* ViInt32,  read-only */

        /* Driver Identification */

#define RSSPECAN_ATTR_SPECIFIC_DRIVER_DESCRIPTION                           RS_ATTR_SPECIFIC_DRIVER_DESCRIPTION                     /* ViString, read-only */
#define RSSPECAN_ATTR_SPECIFIC_DRIVER_PREFIX                                RS_ATTR_SPECIFIC_DRIVER_PREFIX                          /* ViString, read-only */
#define RSSPECAN_ATTR_SPECIFIC_DRIVER_LOCATOR                               RS_ATTR_SPECIFIC_DRIVER_LOCATOR                         /* ViString, read-only */
#define RSSPECAN_ATTR_SPECIFIC_DRIVER_VENDOR                                RS_ATTR_SPECIFIC_DRIVER_VENDOR                          /* ViString, read-only */
#define RSSPECAN_ATTR_SPECIFIC_DRIVER_REVISION                              RS_ATTR_SPECIFIC_DRIVER_REVISION                        /* ViString, read-only */
#define RSSPECAN_ATTR_SPECIFIC_DRIVER_CLASS_SPEC_MAJOR_VERSION              RS_ATTR_SPECIFIC_DRIVER_CLASS_SPEC_MAJOR_VERSION        /* ViInt32,  read-only */
#define RSSPECAN_ATTR_SPECIFIC_DRIVER_CLASS_SPEC_MINOR_VERSION              RS_ATTR_SPECIFIC_DRIVER_CLASS_SPEC_MINOR_VERSION        /* ViInt32,  read-only */

        /* Driver Capabilities */

#define RSSPECAN_ATTR_SUPPORTED_INSTRUMENT_MODELS                           RS_ATTR_SUPPORTED_INSTRUMENT_MODELS                     /* ViString, read-only */
#define RSSPECAN_ATTR_GROUP_CAPABILITIES                                    RS_ATTR_GROUP_CAPABILITIES                              /* ViString, read-only */
#define RSSPECAN_ATTR_FUNCTION_CAPABILITIES                                 RS_ATTR_FUNCTION_CAPABILITIES                           /* ViString, read-only */
#define RSSPECAN_ATTR_CHANNEL_COUNT                                         RS_ATTR_CHANNEL_COUNT                                   /* ViInt32,  read-only */

        /* Driver Setup */

#define RSSPECAN_ATTR_DRIVER_SETUP                                          RS_ATTR_DRIVER_SETUP                                    /* ViString            */

        /* Instrument Identification */

#define RSSPECAN_ATTR_INSTRUMENT_MANUFACTURER                               RS_ATTR_INSTRUMENT_MANUFACTURER                         /* ViString, read-only */
#define RSSPECAN_ATTR_INSTRUMENT_MODEL                                      RS_ATTR_INSTRUMENT_MODEL                                /* ViString, read-only */
#define RSSPECAN_ATTR_INSTRUMENT_FIRMWARE_REVISION                          RS_ATTR_INSTRUMENT_FIRMWARE_REVISION                    /* ViString, read-only */

        /* Advanced Session Information */

#define RSSPECAN_ATTR_IO_RESOURCE_DESCRIPTOR                                RS_ATTR_IO_RESOURCE_DESCRIPTOR                          /* ViString, read-only */
#define RSSPECAN_ATTR_LOGICAL_NAME                                          RS_ATTR_LOGICAL_NAME

/* ViString, read-only */

    /*- Instrument-Specific Attributes -------------------------------------*/

/* BEGIN GENERATE */
#define RSSPECAN_ATTR_NUMBER_OF_SWEEPS                                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 21
#define RSSPECAN_ATTR_SWEEP_MODE_CONTINUOUS                                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 28
#define RSSPECAN_ATTR_VERTICAL_SCALE                                                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 34
#define RSSPECAN_ATTR_SUBWINDOW_VERTICAL_SCALE                                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 7543
#define RSSPECAN_ATTR_SAN_MODE                                                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 44
#define RSSPECAN_ATTR_GET_INSTR_MODE                                                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 45
#define RSSPECAN_ATTR_ACTIVE_WINDOW                                                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 6225
#define RSSPECAN_ATTR_ACTIVE_CHANNELS                                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 6812
#define RSSPECAN_ATTR_AVG_STATE                                                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 58
#define RSSPECAN_ATTR_AVG_TYPE                                                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 59
#define RSSPECAN_ATTR_AVG_COUNT                                                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 60
#define RSSPECAN_ATTR_PRESET_MEASUREMENT_CHANNEL                                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 2860
#define RSSPECAN_ATTR_SELECT_MEASUREMENT_CHANNEL                                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 2861
#define RSSPECAN_ATTR_DUPLICATE_MEASUREMENT_CHANNEL                                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 2997
#define RSSPECAN_ATTR_DELETE_MEASUREMENT_CHANNEL                                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 2890
#define RSSPECAN_ATTR_TFAC_GEN_NAME                                                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 64
#define RSSPECAN_ATTR_TFAC_SEL_NAME                                                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 65
#define RSSPECAN_ATTR_TFAC_UNIT                                                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 66
#define RSSPECAN_ATTR_TFAC_SCALING                                                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 67
#define RSSPECAN_ATTR_TFAC_COMMENT                                                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 68
#define RSSPECAN_ATTR_TFAC_STATE                                                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 69
#define RSSPECAN_ATTR_TFAC_DELETE                                                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 70
#define RSSPECAN_ATTR_TFAC_DISPLAY                                                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 71
#define RSSPECAN_ATTR_TFAC_ADJ_STATE                                                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 72
#define RSSPECAN_ATTR_TRANSDUCER_FACTOR_CATALOG                                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 6813
#define RSSPECAN_ATTR_TSET_SELECT                                                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 3098
#define RSSPECAN_ATTR_TSET_STATE                                                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 3099
#define RSSPECAN_ATTR_TSET_COMMENT                                                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 3100
#define RSSPECAN_ATTR_TSET_UNIT                                                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 3101
#define RSSPECAN_ATTR_TSET_BREAK                                                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 3102
#define RSSPECAN_ATTR_TSET_DELETE                                                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 3103
#define RSSPECAN_ATTR_TSET_CATALOG                                                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 3198
#define RSSPECAN_ATTR_TSET_ACTIVE                                                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 3104
#define RSSPECAN_ATTR_TRANSDUCER_FACTOR_IMPORT                                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 6532
#define RSSPECAN_ATTR_USER_CORRECTION_ENABLED                                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 6814
#define RSSPECAN_ATTR_USER_CORRECTION_FREQUENCY_RESPONSE_ACTIVE                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 6815
#define RSSPECAN_ATTR_USER_CORRECTION_PREVIEW_ENABLED                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 6816
#define RSSPECAN_ATTR_USER_CORRECTION_APPLY_TO                                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 6817
#define RSSPECAN_ATTR_USER_CORRECTION_IS_VALID                                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 6818
#define RSSPECAN_ATTR_USER_CORRECTION_FILE_FREQUENCY_RESPONSE_ADJUST_REFERENCE_LEVEL                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 8609
#define RSSPECAN_ATTR_USER_CORRECTION_FREQUENCY_RESPONSE_ADJUST_REFERENCE_LEVEL                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 8610
#define RSSPECAN_ATTR_USER_CORRECTION_FREQUENCY_RESPONSE_SPAN_COVER                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 9838
#define RSSPECAN_ATTR_USER_CORRECTION_BASEBAND_ENABLED                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 6819
#define RSSPECAN_ATTR_USER_CORRECTION_BASEBAND_PRESET                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 6820
#define RSSPECAN_ATTR_USER_CORRECTION_BASEBAND_LOAD                                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 6821
#define RSSPECAN_ATTR_USER_CORRECTION_BASEBAND_STORE                                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 6823
#define RSSPECAN_ATTR_USER_CORRECTION_BASEBAND_TEST_SETUP_COUNT                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 6825
#define RSSPECAN_ATTR_USER_CORRECTION_BASEBAND_TEST_SETUP_CATALOG                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 6826
#define RSSPECAN_ATTR_USER_CORRECTION_BASEBAND_TEST_SETUP_CLEAR                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 6827
#define RSSPECAN_ATTR_USER_CORRECTION_BASEBAND_FREQUENCY_RESPONSE_COUNT                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 6828
#define RSSPECAN_ATTR_USER_CORRECTION_BASEBAND_FREQUENCY_RESPONSE_CATALOG                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 6829
#define RSSPECAN_ATTR_USER_CORRECTION_BASEBAND_FREQUENCY_RESPONSE_CLEAR                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 6830
#define RSSPECAN_ATTR_USER_CORRECTION_BASEBAND_FREQUENCY_RESPONSE_ADJUST_REFERENCE_LEVEL                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 8611
#define RSSPECAN_ATTR_USER_CORRECTION_BASEBAND_FREQUENCE_RESPONSE_REFRESH                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 9839
#define RSSPECAN_ATTR_USER_CORRECTION_BASEBAND_TEST_SETUP_ENABLED                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 6831
#define RSSPECAN_ATTR_USER_CORRECTION_BASEBAND_TEST_SETUP_SELECTED                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 6832
#define RSSPECAN_ATTR_USER_CORRECTION_BASEBAND_TEST_SETUP_TO_PORT                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 6833
#define RSSPECAN_ATTR_USER_CORRECTION_BASEBAND_TEST_SETUP_FROM_PORT                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 6834
#define RSSPECAN_ATTR_USER_CORRECTION_BASEBAND_TEST_SETUP_REMOVE                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 6835
#define RSSPECAN_ATTR_USER_CORRECTION_BASEBAND_FREQUENCY_RESPONSE_SELECTED                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 6836
#define RSSPECAN_ATTR_USER_CORRECTION_BASEBAND_FREQUENCY_RESPONSE_MAGNITUDE_ENABLED                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 6837
#define RSSPECAN_ATTR_USER_CORRECTION_BASEBAND_FREQUENCY_RESPONSE_PHASE_ENABLED                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 6838
#define RSSPECAN_ATTR_USER_CORRECTION_BASEBAND_FREQUENCY_RESPONSE_REMOVE                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 6839
#define RSSPECAN_ATTR_USER_CORRECTION_BASEBAND_FREQUENCY_RESPONSE_INSERT                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 7544
#define RSSPECAN_ATTR_USER_CORRECTION_INPUT_FREQUENCY_RESPONSE_INSERT                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 7545
#define RSSPECAN_ATTR_USER_CORRECTION_USER_FREQUENCY_RESPONSE_INSERT                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 7546
#define RSSPECAN_ATTR_USER_CORRECTION_BASEBAND_FREQUENCY_RESPONSE_TOUCHSTONE_INSERT                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 7547
#define RSSPECAN_ATTR_USER_CORRECTION_INPUT_FREQUENCY_RESPONSE_TOUCHSTONE_INSERT                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 7548
#define RSSPECAN_ATTR_USER_CORRECTION_USER_FREQUENCY_RESPONSE_TOUCHSTONE_INSERT                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 7549
#define RSSPECAN_ATTR_USER_CORRECTION_BASEBAND_FREQUENCY_RESPONSE_TOUCHSTONE_MOVE                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 7550
#define RSSPECAN_ATTR_USER_CORRECTION_INPUT_FREQUENCY_RESPONSE_TOUCHSTONE_MOVE                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 7551
#define RSSPECAN_ATTR_USER_CORRECTION_USER_FREQUENCY_RESPONSE_TOUCHSTONE_MOVE                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 7552
#define RSSPECAN_ATTR_USER_CORRECTION_RF_INPUT_ENABLED                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 6840
#define RSSPECAN_ATTR_USER_CORRECTION_RF_INPUT_PRESET                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 6841
#define RSSPECAN_ATTR_USER_CORRECTION_RF_INPUT_LOAD                                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 6842
#define RSSPECAN_ATTR_USER_CORRECTION_RF_INPUT_STORE                                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 6844
#define RSSPECAN_ATTR_USER_CORRECTION_RF_INPUT_TEST_SETUP_COUNT                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 6846
#define RSSPECAN_ATTR_USER_CORRECTION_RF_INPUT_TEST_SETUP_CATALOG                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 6847
#define RSSPECAN_ATTR_USER_CORRECTION_RF_INPUT_TEST_SETUP_CLEAR                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 6848
#define RSSPECAN_ATTR_USER_CORRECTION_RF_INPUT_FREQUENCY_RESPONSE_COUNT                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 6849
#define RSSPECAN_ATTR_USER_CORRECTION_RF_INPUT_FREQUENCY_RESPONSE_CATALOG                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 6850
#define RSSPECAN_ATTR_USER_CORRECTION_RF_INPUT_FREQUENCY_RESPONSE_CLEAR                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 6851
#define RSSPECAN_ATTR_USER_CORRECTION_RF_INPUT_FREQUENCY_RESPONSE_ADJUST_REFERENCE_LEVEL                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 8612
#define RSSPECAN_ATTR_USER_CORRECTION_RF_INPUT_FREQUENCY_RESPONSE_REFRESH                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 9840
#define RSSPECAN_ATTR_USER_CORRECTION_RF_INPUT_TEST_SETUP_ENABLED                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 6852
#define RSSPECAN_ATTR_USER_CORRECTION_RF_INPUT_TEST_SETUP_SELECTED                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 6853
#define RSSPECAN_ATTR_USER_CORRECTION_RF_INPUT_TEST_SETUP_TO_PORT                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 6854
#define RSSPECAN_ATTR_USER_CORRECTION_RF_INPUT_TEST_SETUP_FROM_PORT                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 6855
#define RSSPECAN_ATTR_USER_CORRECTION_RF_INPUT_TEST_SETUP_REMOVE                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 6856
#define RSSPECAN_ATTR_USER_CORRECTION_RF_INPUT_FREQUENCY_RESPONSE_SELECTED                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 6857
#define RSSPECAN_ATTR_USER_CORRECTION_RF_INPUT_FREQUENCY_RESPONSE_MAGNITUDE_ENABLED                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 6858
#define RSSPECAN_ATTR_USER_CORRECTION_RF_INPUT_FREQUENCY_RESPONSE_PHASE_ENABLED                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 6859
#define RSSPECAN_ATTR_USER_CORRECTION_RF_INPUT_FREQUENCY_RESPONSE_REMOVE                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 6860
#define RSSPECAN_ATTR_USER_CORRECTION_ALL_INPUTS_PRESET                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 6861
#define RSSPECAN_ATTR_USER_CORRECTION_ALL_INPUTS_LOAD                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 6862
#define RSSPECAN_ATTR_USER_CORRECTION_ALL_INPUTS_STORE                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 6864
#define RSSPECAN_ATTR_USER_CORRECTION_ALL_INPUTS_TEST_SETUP_COUNT                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 6866
#define RSSPECAN_ATTR_USER_CORRECTION_ALL_INPUTS_TEST_SETUP_CATALOG                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 6867
#define RSSPECAN_ATTR_USER_CORRECTION_ALL_INPUTS_TEST_SETUP_CLEAR                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 6868
#define RSSPECAN_ATTR_USER_CORRECTION_ALL_INPUTS_FREQUENCY_RESPONSE_CATALOG                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 6869
#define RSSPECAN_ATTR_USER_CORRECTION_ALL_INPUTS_FREQUENCY_RESPONSE_COUNT                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 6870
#define RSSPECAN_ATTR_USER_CORRECTION_ALL_INPUTS_FREQUENCY_RESPONSE_CLEAR                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 6871
#define RSSPECAN_ATTR_USER_CORRECTION_ALL_INPUTS_TEST_SETUP_ENABLED                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 6872
#define RSSPECAN_ATTR_USER_CORRECTION_ALL_INPUTS_TEST_SETUP_SELECTED                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 6873
#define RSSPECAN_ATTR_USER_CORRECTION_ALL_INPUTS_TEST_SETUP_TO_PORT                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 6874
#define RSSPECAN_ATTR_USER_CORRECTION_ALL_INPUTS_TEST_SETUP_FROM_PORT                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 6875
#define RSSPECAN_ATTR_USER_CORRECTION_ALL_INPUTS_TEST_SETUP_REMOVE                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 6879
#define RSSPECAN_ATTR_USER_CORRECTION_ALL_INPUTS_FREQUENCY_RESPONSE_SELECTED                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 6880
#define RSSPECAN_ATTR_USER_CORRECTION_ALL_INPUTS_FREQUENCY_RESPONSE_MAGNITUDE_ENABLED                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 6881
#define RSSPECAN_ATTR_USER_CORRECTION_ALL_INPUTS_FREQUENCY_RESPONSE_PHASE_ENABLED                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 6882
#define RSSPECAN_ATTR_USER_CORRECTION_ALL_INPUTS_FREQUENCY_RESPONSE_REMOVE                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 6886
#define RSSPECAN_ATTR_CALIBRATION                                                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 95
#define RSSPECAN_ATTR_CALIBRATION_ABORT                                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 96
#define RSSPECAN_ATTR_CALIBRATION_RESULT_QUERY                                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 97
#define RSSPECAN_ATTR_CALIBRATION_STATE                                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 98
#define RSSPECAN_ATTR_CALIBRATION_PRESELECTION                                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 7553
#define RSSPECAN_ATTR_CALIBRATION_SCHEDULE_ENABLED                                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 10756
#define RSSPECAN_ATTR_CALIBRATION_SCHEDULE_TIME                                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 10758
#define RSSPECAN_ATTR_CALIBRATION_SCHEDULE_WARMUP_ENABLED                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 10759
#define RSSPECAN_ATTR_CALIBRATION_SCHEDULE_SHUTDOWN_ENABLED                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 10760
#define RSSPECAN_ATTR_INP_UPORT_STATE                                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 110
#define RSSPECAN_ATTR_INP_UPORT_VALUE                                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 111
#define RSSPECAN_ATTR_USER_PORTS_OUTPUT_STATE                                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 7407
#define RSSPECAN_ATTR_OUT_UPORT_VALUE                                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 112
#define RSSPECAN_ATTR_AUTO_ADJUST_ALL                                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 113
#define RSSPECAN_ATTR_AUTO_ADJUST_FREQ                                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 114
#define RSSPECAN_ATTR_AUTO_ADJUST_LEVEL                                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 115
#define RSSPECAN_ATTR_AUTO_ADJUST_LEVEL_MEAS_TIME                                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 116
#define RSSPECAN_ATTR_AUTO_ADJUST_LEVEL_MEAS_TIME_MODE                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 117
#define RSSPECAN_ATTR_AUTO_ADJUST_LOWER_HYSTERESIS                                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 2769
#define RSSPECAN_ATTR_AUTO_ADJUST_UPPER_HYSTERESIS                                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 3162
#define RSSPECAN_ATTR_AUTO_ADJUST_TRIGGER_STATE                                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 2770
#define RSSPECAN_ATTR_AUTO_ADJUST_LEVEL_MEAS_TIME_FSV                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 4951
#define RSSPECAN_ATTR_AUTO_ADJUST_LEVEL_MEAS_TIME_MODE_FSV                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 4952
#define RSSPECAN_ATTR_AUTO_ADJUST_SEARCH_MODE                                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 8613
#define RSSPECAN_ATTR_SPECTRUM_DISPLAY_CREATE                                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 2358
#define RSSPECAN_ATTR_SPECTRUM_DISPLAY_SELECT                                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 2359
#define RSSPECAN_ATTR_SPECTRUM_DISPLAY_DELETE                                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 2360
#define RSSPECAN_ATTR_IF_OUTPUT_SOURCE                                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 43
#define RSSPECAN_ATTR_IF_OUTPUT_FREQUENCY                                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 2672
#define RSSPECAN_ATTR_NOISE_SOURCE_STATE                                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 2673
#define RSSPECAN_ATTR_IF_OUTPUT_SIDEBAND                                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 5146
#define RSSPECAN_ATTR_SYNC_CENTER_FREQUENCY_ENABLED                                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 6893
#define RSSPECAN_ATTR_SYNC_START_STOP_FREQUENCY_ENABLED                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 6894
#define RSSPECAN_ATTR_SYNC_ATTENUATION_ENABLED                                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 6895
#define RSSPECAN_ATTR_SYNC_PREAMPLIFIER_ENABLED                                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 6896
#define RSSPECAN_ATTR_SYNC_VIDEO_BANDWIDTH_ENABLED                                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 6897
#define RSSPECAN_ATTR_SYNC_RBW_ENABLED                                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 6898
#define RSSPECAN_ATTR_SYNC_LIMIT_LINES_ENABLED                                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 6899
#define RSSPECAN_ATTR_SYNC_AC_DC_COUPLING_ENABLED                                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 6900
#define RSSPECAN_ATTR_SYNC_MARKER_FREQUENCY_ENABLED                                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 6901
#define RSSPECAN_ATTR_SYNC_PRESELECTOR_ENABLED                                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 6902
#define RSSPECAN_ATTR_SYNC_AUDIO_DEMODULATOR_ENABLED                                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 6903
#define RSSPECAN_ATTR_SYNC_AMPLITUDE_BASEBAND_IMPEDANCE_ENABLED                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 7554
#define RSSPECAN_ATTR_SYNC_AMPLITUDE_UNIT_ENABLED                                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 7555
#define RSSPECAN_ATTR_SYNC_IMPEDANCE_ENABLED                                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 7556
#define RSSPECAN_ATTR_SYNC_LIMITED_LINES_ENABLED                                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 7557
#define RSSPECAN_ATTR_SYNC_REFERENCE_LEVEL_ENABLED                                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 7558
#define RSSPECAN_ATTR_SYNC_USER_DEFINED_PARAMETER_COUPLING_REMOVE_ALL                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 6905
#define RSSPECAN_ATTR_SYNC_USER_DEFINED_PARAMETER_COUPLING_CHANNEL_LIST                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 6906
#define RSSPECAN_ATTR_SYNC_USER_DEFINED_PARAMETER_COUPLING_NUMBERS_LIST                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 6908
#define RSSPECAN_ATTR_SYNC_USER_DEFINED_PARAMETER_COUPLING_ENABLED                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 6910
#define RSSPECAN_ATTR_SYNC_USER_DEFINED_PARAMETER_COUPLING_INFO                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 6912
#define RSSPECAN_ATTR_SYNC_USER_DEFINED_PARAMETER_COUPLING_DIRECTION                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 6913
#define RSSPECAN_ATTR_SYNC_USER_DEFINED_PARAMETER_COUPLING_REMOVE                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 6914
#define RSSPECAN_ATTR_GENERATOR_COUPLING_ENABLED                                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 9841
#define RSSPECAN_ATTR_GENERATOR_COUPLING_CENTER_FREQUENCY_ENABLED                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 9842
#define RSSPECAN_ATTR_GENERATOR_COUPLING_CENTER_FREQUENCY_OFFSET                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 9843
#define RSSPECAN_ATTR_GENERATOR_COUPLING_REFERENCE_LEVEL_ENABLED                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 9844
#define RSSPECAN_ATTR_GENERATOR_COUPLING_REFERENCE_LEVEL_OFFSET                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 9845
#define RSSPECAN_ATTR_FREQUENCY_MODE                                                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 17
#define RSSPECAN_ATTR_FREQUENCY_OFFSET                                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 8
#define RSSPECAN_ATTR_FREQUENCY_START                                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 6
#define RSSPECAN_ATTR_FREQUENCY_STOP                                                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 7
#define RSSPECAN_ATTR_FREQUENCY_ANNOTATION_MODE                                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 10761
#define RSSPECAN_ATTR_FREQUENCY_CENTER                                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 9
#define RSSPECAN_ATTR_FREQUENCY_CENTER_STEP                                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 10
#define RSSPECAN_ATTR_FREQUENCY_STEP_AUTO                                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 11
#define RSSPECAN_ATTR_FREQUENCY_CENTER_LINK                                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 12
#define RSSPECAN_ATTR_FREQUENCY_CENTER_LINK_FACTOR                                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 13
#define RSSPECAN_ATTR_FREQUENCY_SPAN                                                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 14
#define RSSPECAN_ATTR_FREQUENCY_SPAN_FULL                                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 15
#define RSSPECAN_ATTR_HORIZONTAL_SCALE                                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 42
#define RSSPECAN_ATTR_FREQUENCY_LEVEL_SETTING                                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 7408
#define RSSPECAN_ATTR_FREQUENCY_LEVEL_SETTING_USED                                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 11202
#define RSSPECAN_ATTR_WINDOW_HORIZONTAL_SCALE                                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 6915
#define RSSPECAN_ATTR_SUBWINDOW_HORIZONTAL_SCALE                                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 7559
#define RSSPECAN_ATTR_MARKER_AMPLITUDE                                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 120
#define RSSPECAN_ATTR_MARKER_ENABLED                                                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 121
#define RSSPECAN_ATTR_MARKER_POSITION                                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 122
#define RSSPECAN_ATTR_MARKER_TRACE                                                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 123
#define RSSPECAN_ATTR_ASSIGN_MARKER_TO_TRACE_RAISING_FALLING                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 2898
#define RSSPECAN_ATTR_MARKER_AOFF                                                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 124
#define RSSPECAN_ATTR_MARKER_LOEX                                                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 125
#define RSSPECAN_ATTR_MARKER_STEP_SIZE                                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 130
#define RSSPECAN_ATTR_MARKER_PROBABILITY                                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 131
#define RSSPECAN_ATTR_SIGNAL_TRACK_ENABLED                                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 126
#define RSSPECAN_ATTR_SIGNAL_TRACK_BWID                                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 127
#define RSSPECAN_ATTR_SIGNAL_TRACK_THRESHOLD                                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 128
#define RSSPECAN_ATTR_SIGNAL_TRACK_TRACE                                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 129
#define RSSPECAN_ATTR_MARKER_SEARCH_MIN_RIGHT                                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 134
#define RSSPECAN_ATTR_MARKER_SEARCH_MIN_LEFT                                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 135
#define RSSPECAN_ATTR_MARKER_SEARCH_PEAK                                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 136
#define RSSPECAN_ATTR_MARKER_SEARCH_PEAK_NEXT                                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 137
#define RSSPECAN_ATTR_MARKER_SEARCH_PEAK_RIGHT                                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 138
#define RSSPECAN_ATTR_MARKER_SEARCH_PEAK_LEFT                                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 139
#define RSSPECAN_ATTR_MARKER_SEARCH_MIN                                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 140
#define RSSPECAN_ATTR_MARKER_SEARCH_MIN_NEXT                                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 141
#define RSSPECAN_ATTR_MARKER_SEARCH_PEAK_AUTO                                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 142
#define RSSPECAN_ATTR_MARKER_SEARCH_MIN_AUTO                                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 143
#define RSSPECAN_ATTR_MARKER_MOVE_TO_START                                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 7560
#define RSSPECAN_ATTR_MARKER_THRESHOLD_STATE                                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 144
#define RSSPECAN_ATTR_MARKER_THRESHOLD                                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 145
#define RSSPECAN_ATTR_PEAK_EXCURSION                                                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 146
#define RSSPECAN_ATTR_MARKER_SEARCH_LIMITS_STATE                                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 147
#define RSSPECAN_ATTR_MARKER_SEARCH_LIMITS_LEFT                                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 148
#define RSSPECAN_ATTR_MARKER_SEARCH_LIMITS_RIGHT                                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 149
#define RSSPECAN_ATTR_MARKER_SEARCH_LIMITS_ZOOM                                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 150
#define RSSPECAN_ATTR_MARKER_SEARCH_SPURIOUS_TRACKING_STATE                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 7409
#define RSSPECAN_ATTR_MARKER_SEARCH_SPURIOUS_TRACKING_FREQUENCY_RANGE                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 7410
#define RSSPECAN_ATTR_MARKER_FREQUENCY_COUNTER_ENABLED                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 415
#define RSSPECAN_ATTR_MARKER_FREQUENCY_COUNTER_RESOLUTION                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 416
#define RSSPECAN_ATTR_MARKER_COUNT                                                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 417
#define RSSPECAN_ATTR_MARKER_NOISE_STATE                                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 410
#define RSSPECAN_ATTR_MARKER_NOISE_MEASUREMENT_ALL_OFF                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 6916
#define RSSPECAN_ATTR_MARKER_NOISE_RESULT                                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 411
#define RSSPECAN_ATTR_MARKER_DEMOD_STATE                                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 159
#define RSSPECAN_ATTR_MARKER_DEMOD_TYPE                                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 160
#define RSSPECAN_ATTR_MARKER_DEMOD_HOLDOFF                                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 161
#define RSSPECAN_ATTR_MARKER_DEMOD_CONT                                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 162
#define RSSPECAN_ATTR_MARKER_DEMODULATION_SELECTIVE_DEMODULATION_ENABLED                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 6917
#define RSSPECAN_ATTR_MARKER_DEMODULATION_SELECTIVE_DEMODULATION_THRESHOLD                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 6918
#define RSSPECAN_ATTR_MARKER_PEAK_LIST_STATE                                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 2579
#define RSSPECAN_ATTR_MARKER_STORE_PEAK_LIST                                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 175
#define RSSPECAN_ATTR_MARKER_PEAK_LIST_ANNOTATION_LABEL                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 2623
#define RSSPECAN_ATTR_MARKER_PEAK_LIST_COUNT                                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 151
#define RSSPECAN_ATTR_MARKER_PEAK_LIST_FOUND                                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 152
#define RSSPECAN_ATTR_MARKER_PEAK_LIST_SORT                                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 153
#define RSSPECAN_ATTR_MARKER_PEAK_LIST_SIZE                                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 2621
#define RSSPECAN_ATTR_MARKER_REF_STATE                                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 170
#define RSSPECAN_ATTR_MARKER_REF_LEVEL                                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 171
#define RSSPECAN_ATTR_MARKER_REF_LEVEL_OFFSET                                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 172
#define RSSPECAN_ATTR_MARKER_REF_FREQ                                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 173
#define RSSPECAN_ATTR_MARKER_REF_PEAK                                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 174
#define RSSPECAN_ATTR_MARKER_ZOOM                                                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 154
#define RSSPECAN_ATTR_MARKER_TO_CENTER                                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 165
#define RSSPECAN_ATTR_MARKER_TO_STEP                                                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 166
#define RSSPECAN_ATTR_MARKER_TO_REFERENCE                                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 167
#define RSSPECAN_ATTR_MARKER_LINK                                                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 169
#define RSSPECAN_ATTR_MARKER_LINK_TO_DELTA_MARKER                                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 10762
#define RSSPECAN_ATTR_MARKER_LINK_TO_MARKER                                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 2862
#define RSSPECAN_ATTR_MARKER_LABELS_FOR_PEAKS                                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 7411
#define RSSPECAN_ATTR_MARKER_BAND_POWER_STATE                                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 2361
#define RSSPECAN_ATTR_MARKER_BAND_POWER_ALL_OFF                                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 6919
#define RSSPECAN_ATTR_MARKER_BAND_POWER_MODE                                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 2362
#define RSSPECAN_ATTR_MARKER_BAND_POWER_SPAN                                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 2363
#define RSSPECAN_ATTR_QUERY_MARKER_BAND_POWER_RESULT                                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 2364
#define RSSPECAN_ATTR_DELTA_MARKER_BAND_POWER_STATE                                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 2365
#define RSSPECAN_ATTR_DELTA_MARKER_BAND_POWER_MODE                                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 2366
#define RSSPECAN_ATTR_DELTA_MARKER_BAND_POWER_SPAN                                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 2367
#define RSSPECAN_ATTR_QUERY_DELTA_MARKER_BAND_POWER_RESULT                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 2368
#define RSSPECAN_ATTR_REFERENCE_MARKER_PEAK                                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 185
#define RSSPECAN_ATTR_REFERENCE_MARKER_PEAK_NEXT                                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 186
#define RSSPECAN_ATTR_REFERENCE_MARKER_PEAK_NEXT_RIGHT                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 187
#define RSSPECAN_ATTR_REFERENCE_MARKER_PEAK_NEXT_LEFT                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 188
#define RSSPECAN_ATTR_REFERENCE_MARKER_MIN                                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 189
#define RSSPECAN_ATTR_REFERENCE_MARKER_MIN_NEXT                                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 190
#define RSSPECAN_ATTR_REFERENCE_MARKER_MIN_NEXT_RIGHT                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 191
#define RSSPECAN_ATTR_REFERENCE_MARKER_MIN_NEXT_LEFT                                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 192
#define RSSPECAN_ATTR_REFERENCE_MARKER_Y_TRIGGER                                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 2556
#define RSSPECAN_ATTR_REFERENCE_MARKER_STATE                                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 178
#define RSSPECAN_ATTR_REFERENCE_MARKER_MODE                                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 179
#define RSSPECAN_ATTR_REFERENCE_MARKER_POSITION                                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 180
#define RSSPECAN_ATTR_REFERENCE_MARKER_REL_POSITION                                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 181
#define RSSPECAN_ATTR_REFERENCE_MARKER_AMPLITUDE                                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 182
#define RSSPECAN_ATTR_REFERENCE_MARKER_TRACE                                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 183
#define RSSPECAN_ATTR_DELTA_MARKER_REFERENCE_MARKER                                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 2369
#define RSSPECAN_ATTR_DELTA_MARKER_LINK_TO_MARKER                                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 2863
#define RSSPECAN_ATTR_DELTA_MARKER_LINK_TO_ANOTHER_DELTA_MARKER                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 10764
#define RSSPECAN_ATTR_REFERENCE_MARKER_AOFF                                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 184
#define RSSPECAN_ATTR_RESOLUTION_BANDWIDTH                                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 24
#define RSSPECAN_ATTR_RESOLUTION_BANDWIDTH_AUTO                                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 25
#define RSSPECAN_ATTR_RESOLUTION_BANDWIDTH_RATIO                                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 26
#define RSSPECAN_ATTR_RESOLUTION_BANDWIDTH_FILTER_TYPE                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 61
#define RSSPECAN_ATTR_SWEEP_TIME                                                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 29
#define RSSPECAN_ATTR_SWEEP_TIME_AUTO                                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 30
#define RSSPECAN_ATTR_SWEEP_SAMPLES                                                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 9846
#define RSSPECAN_ATTR_SWEEP_DURATION                                                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 5288
#define RSSPECAN_ATTR_SWEEP_MODE                                                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 27
#define RSSPECAN_ATTR_SWEEP_POINTS                                                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 31
#define RSSPECAN_ATTR_SWEEP_OPTIMIZATION                                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 2859
#define RSSPECAN_ATTR_SWEEP_TYPE                                                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 32
#define RSSPECAN_ATTR_SWEEP_AUTO_TYPE                                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 9847
#define RSSPECAN_ATTR_SWEEP_COUNT_CURRENT                                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 33
#define RSSPECAN_ATTR_VIDEO_BANDWIDTH                                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 35
#define RSSPECAN_ATTR_VIDEO_BANDWIDTH_AUTO                                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 36
#define RSSPECAN_ATTR_VIDEO_BANDWIDTH_RATIO                                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 37
#define RSSPECAN_ATTR_VIDEO_BANDWIDTH_TYPE                                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 62
#define RSSPECAN_ATTR_FFT_FILTER_MODE                                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 38
#define RSSPECAN_ATTR_FFT_SUBSPANS                                                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 7561
#define RSSPECAN_ATTR_WINDOW_SWEEP_POINTS                                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 7562
#define RSSPECAN_ATTR_SYST_ERR                                                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 193
#define RSSPECAN_ATTR_SYST_ERR_LIST                                                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 194
#define RSSPECAN_ATTR_SYST_ERR_CLEAR                                                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 195
#define RSSPECAN_ATTR_SYST_DISPLAY_REMOTE_ERRORS                                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 5595
#define RSSPECAN_ATTR_SYST_REMOTE_ERR_TABLE_CLEAR                                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 5597
#define RSSPECAN_ATTR_SYST_PRESET                                                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 196
#define RSSPECAN_ATTR_PRESET_OPERATING_MODE                                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 2900
#define RSSPECAN_ATTR_PRESET_FILTER                                                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 6031
#define RSSPECAN_ATTR_SYST_SPEAKER                                                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 197
#define RSSPECAN_ATTR_SYST_VERSION                                                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 203
#define RSSPECAN_ATTR_APPLICATIONS_VERSIONS                                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 2706
#define RSSPECAN_ATTR_SYST_ID_FACTORY                                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 204
#define RSSPECAN_ATTR_SYST_ID_FACTORY_QUERY                                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 205
#define RSSPECAN_ATTR_APPL_SETUP_REC                                                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 206
#define RSSPECAN_ATTR_REMOTE_LOGGING_STATE                                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 2771
#define RSSPECAN_ATTR_SYSTEM_SHUTDOWN                                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 2370
#define RSSPECAN_ATTR_SYSTEM_REBOOT                                                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 6920
#define RSSPECAN_ATTR_LOCK_LOCAL_KEY                                                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 2557
#define RSSPECAN_ATTR_FRONTEND_TEMPERATURE                                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 2948
#define RSSPECAN_ATTR_SECURITY_ENABLED                                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 4948
#define RSSPECAN_ATTR_SYSTEM_TREE_WALKING                                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 6483
#define RSSPECAN_ATTR_INSTRUMENT_MODE                                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 6027
#define RSSPECAN_ATTR_SYSTEM_INFO_DEVICE_FOOTPRINT                                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 6921
#define RSSPECAN_ATTR_SYSTEM_INFO_BIOS_VERSION                                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 6923
#define RSSPECAN_ATTR_SYSTEM_INFO_FREQUENCY_BANDS                                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 6924
#define RSSPECAN_ATTR_SYSTEM_DISPLAY_LOCK_ENABLED                                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 6925
#define RSSPECAN_ATTR_SYSTEM_LANGUAGE                                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 7413
#define RSSPECAN_ATTR_CHANGING_HARDWARE_ENABLED                                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 7563
#define RSSPECAN_ATTR_CHANGING_HARDWARE                                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 7564
#define RSSPECAN_ATTR_TRIAL_OPTIONS                                                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 8614
#define RSSPECAN_ATTR_TRIAL_OPTIONS_ENABLED                                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 8615
#define RSSPECAN_ATTR_SYSTEM_SCPI_RECORDER_AUTO_RECORDING_ENABLED                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 10768
#define RSSPECAN_ATTR_SYSTEM_SCPI_RECORDER_ADD_SYNC_COMMANDS                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 10769
#define RSSPECAN_ATTR_SYSTEM_SCPI_RECORDER_LIST_RECORDING                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 10770
#define RSSPECAN_ATTR_SYSTEM_SCPI_RECORDER_CLEAR_ALL                                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 10772
#define RSSPECAN_ATTR_ROSC_SOURCE                                                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 211
#define RSSPECAN_ATTR_ROSC_SOURCE_EAUTO                                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 212
#define RSSPECAN_ATTR_EXTERNAL_REFERENCE_OSCILLATOR_FREQUENCY                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 7414
#define RSSPECAN_ATTR_ROSC_FREQUENCY                                                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 213
#define RSSPECAN_ATTR_MISSING_EXTERNAL_REFERENCE_BEHAVIOR                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 10704
#define RSSPECAN_ATTR_REFERENCE_OSCILLATOR_LOOP_BANDWIDTH                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 6926
#define RSSPECAN_ATTR_REFERENCE_OSCILLATOR_OUTPUT_100_MHZ_ENABLED                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 6927
#define RSSPECAN_ATTR_REFERENCE_OSCILLATOR_SYNC_TRIGGER_OUTPUT_ENABLED                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 6928
#define RSSPECAN_ATTR_REFERENCE_OSCILLATOR_OUTPUT_640_MHZ_ENABLED                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 6929
#define RSSPECAN_ATTR_REFERENCE_OSCILLATOR_TUNING_RANGE                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 6930
#define RSSPECAN_ATTR_ADVANCED_REFERENCE_OSCILLATOR_FREQUENCY_COUPLING                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 7415
#define RSSPECAN_ATTR_ADVANCED_REFERENCE_OSCILLATOR_BANDWIDTH_MODE                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 7416
#define RSSPECAN_ATTR_ADVANCED_REFERENCE_OSCILLATOR_COUPLING_BANDWIDTH                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 7417
#define RSSPECAN_ATTR_NETWORK_REMOTE_MODE_ENABLED                                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 11203
#define RSSPECAN_ATTR_LAN_PASSWORD                                                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 9848
#define RSSPECAN_ATTR_LAN_RESET                                                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 9849
#define RSSPECAN_ATTR_LAN_INSTRUMENT_DESCRIPTION                                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 9850
#define RSSPECAN_ATTR_GPIB_ADDRESS                                                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 9851
#define RSSPECAN_ATTR_GPIB_TERMINATOR                                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 9852
#define RSSPECAN_ATTR_HUMS_ENABLED                                                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 9853
#define RSSPECAN_ATTR_HUMS_DELETE_ALL_DATA                                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 9854
#define RSSPECAN_ATTR_HUMS_DATA_FORMAT                                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 9855
#define RSSPECAN_ATTR_HUMS_TAG_DELETE                                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 9858
#define RSSPECAN_ATTR_HUMS_TAG_DELETE_ALL                                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 9859
#define RSSPECAN_ATTR_HUMS_REST_STATE                                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 9860
#define RSSPECAN_ATTR_HUMS_SNMP_VERSION                                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 9861
#define RSSPECAN_ATTR_HUMS_SNMP_COMMUNITY_READ_ONLY                                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 9862
#define RSSPECAN_ATTR_HUMS_SNMP_COMUNITY_READ_WRITE                                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 9863
#define RSSPECAN_ATTR_HUMS_SNMP_CONTACT                                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 9864
#define RSSPECAN_ATTR_HUMS_SNMP_LOCATION                                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 9865
#define RSSPECAN_ATTR_HUMS_SNMP_USER_DELETE                                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 9866
#define RSSPECAN_ATTR_HUMS_SNMP_V3_USER_DELETE_ALL                                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 9867
#define RSSPECAN_ATTR_HUMS_SNMP_V3_USER_LIST                                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 9869
#define RSSPECAN_ATTR_TRACE_FILE_TYPE                                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 6535
#define RSSPECAN_ATTR_TRACE_RESET_BEHAVIOR                                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 85
#define RSSPECAN_ATTR_DETECTOR_TYPE                                                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 91
#define RSSPECAN_ATTR_DETECTOR_TYPE_AUTO                                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 92
#define RSSPECAN_ATTR_TRACE_STATE                                                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 84
#define RSSPECAN_ATTR_TRACE_TYPE                                                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 86
#define RSSPECAN_ATTR_TRACE_SMOOTHING_ENABLED                                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 6931
#define RSSPECAN_ATTR_TRACE_SMOOTHING_APERTURE                                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 6932
#define RSSPECAN_ATTR_TRACE_RESET_BEHAVIOUR_SPECIFIC_WINDOW                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 7119
#define RSSPECAN_ATTR_SUBWINDOW_TRACE_STATE                                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 7565
#define RSSPECAN_ATTR_SUBWINDOW_TRACE_SMOOTHING_ENABLED                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 7566
#define RSSPECAN_ATTR_SUBWINDOW_TRACE_SMOOTHING_APERTURE                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 7567
#define RSSPECAN_ATTR_SUBWINDOW_TRACE_RESET_BEHAVIOUR_SPECIFIC_WINDOW                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 7568
#define RSSPECAN_ATTR_SUBWINDOW_TRACE_PRESET                                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 7569
#define RSSPECAN_ATTR_TRACE_EXPORT_ALL                                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 3053
#define RSSPECAN_ATTR_TRACE_IMPORT_ALL                                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 6536
#define RSSPECAN_ATTR_TRACE_MATH_STATE                                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 88
#define RSSPECAN_ATTR_TRACE_MATH_POSITION                                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 89
#define RSSPECAN_ATTR_TRACE_MATH_MODE                                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 90
#define RSSPECAN_ATTR_TRACE_IQ_DPD_CURRENT                                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 10773
#define RSSPECAN_ATTR_TRIGGER_SOURCE                                                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 215
#define RSSPECAN_ATTR_TRIGGER_DELAY                                                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 216
#define RSSPECAN_ATTR_TRIGGER_SLOPE                                                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 217
#define RSSPECAN_ATTR_TRIGGER_DROPOUT_TIME                                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 2674
#define RSSPECAN_ATTR_TRIGGER_IQP_LEVEL                                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 2864
#define RSSPECAN_ATTR_TRIGGER_RFP_LEVEL                                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 956
#define RSSPECAN_ATTR_TRIGGER_FREQUENCY                                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 7418
#define RSSPECAN_ATTR_TRIGGER_HYSTERESIS                                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 7419
#define RSSPECAN_ATTR_TRIGGER_POINT_IN_SAMPLE                                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 10774
#define RSSPECAN_ATTR_TRIGGER_REPETITION_INTERVAL                                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 10775
#define RSSPECAN_ATTR_INSTRUMENT_TRIGGER_SOURCE                                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 7570
#define RSSPECAN_ATTR_INSTRUMENT_TRIGGER_OFFSET                                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 7571
#define RSSPECAN_ATTR_INSTRUMENT_TRIGGER_RF_POWER_LEVEL                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 7572
#define RSSPECAN_ATTR_INSTRUMENT_TRIGGER_IQ_POWER_LEVEL                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 7573
#define RSSPECAN_ATTR_INSTRUMENT_TRIGGER_IF_POWER_LEVEL                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 7574
#define RSSPECAN_ATTR_INSTRUMENT_DIGITAL_BASEBAND_INPUT_TRIGGER_LEVEL                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 7575
#define RSSPECAN_ATTR_INSTRUMENT_SELECTED_TRIGGER_PORT                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 7576
#define RSSPECAN_ATTR_INSTRUMENT_EXTERNAL_TRIGGER_LEVEL_PORT                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 7577
#define RSSPECAN_ATTR_TRIGGER_DIRECTION                                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 2675
#define RSSPECAN_ATTR_TRIGGER_OUTPUT_TYPE                                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 2676
#define RSSPECAN_ATTR_TRIGGER_LEVEL                                                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 2677
#define RSSPECAN_ATTR_TRIGGER_PULSE_LENGTH                                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 2678
#define RSSPECAN_ATTR_TRIGGER_PULSE_GENERATE                                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 2679
#define RSSPECAN_ATTR_EXTERNAL_TRIGGER_LEVEL_PORT                                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 2680
#define RSSPECAN_ATTR_EXTERNAL_GATE_TRACE_STATE                                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 226
#define RSSPECAN_ATTR_EXTERNAL_GATE_TRACE_START                                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 227
#define RSSPECAN_ATTR_EXTERNAL_GATE_TRACE_STOP                                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 228
#define RSSPECAN_ATTR_EXTERNAL_GATE_TRACE_PERIOD_LENGTH                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 229
#define RSSPECAN_ATTR_EXTERNAL_GATE_TRACE_COMMENT                                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 230
#define RSSPECAN_ATTR_EXTERNAL_GATE                                                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 220
#define RSSPECAN_ATTR_EXTERNAL_GATE_HOLD                                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 221
#define RSSPECAN_ATTR_EXTERNAL_GATE_TRIGGER_TYPE                                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 222
#define RSSPECAN_ATTR_EXTERNAL_GATE_POLARITY                                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 223
#define RSSPECAN_ATTR_EXTERNAL_GATE_LENGTH                                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 224
#define RSSPECAN_ATTR_EXTERNAL_GATE_SIGNAL_SOURCE                                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 225
#define RSSPECAN_ATTR_EXTERNAL_GATE_SOURCE_MODE                                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 6933
#define RSSPECAN_ATTR_EXTERNAL_GATE_PERIODS                                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 6934
#define RSSPECAN_ATTR_EXTERNAL_GATE_PERIOD_LENGTH                                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 6935
#define RSSPECAN_ATTR_EXTERNAL_GATE_CONTINUOUS_ENABLED                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 6936
#define RSSPECAN_ATTR_EXTERNAL_GATE_LEVEL                                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 6937
#define RSSPECAN_ATTR_EXTERNAL_GATE_LEVEL_INTERMEDIATE                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 10776
#define RSSPECAN_ATTR_EXTERNAL_GATE_EXTERNAL_LEVEL                                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 6938
#define RSSPECAN_ATTR_EXTERNAL_TRIGGER_LEVEL                                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 218
#define RSSPECAN_ATTR_VIDEO_TRIGGER_LEVEL                                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 231
#define RSSPECAN_ATTR_TRIGGER_IFP_LEVEL                                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 232
#define RSSPECAN_ATTR_TRIGGER_IFP_OFFSET                                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 233
#define RSSPECAN_ATTR_TRIGGER_IFP_HYSTERESIS                                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 234
#define RSSPECAN_ATTR_TRIGGER_IF_POWER_COUPLING                                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 5598
#define RSSPECAN_ATTR_TRIGGER_TIME_INTERVAL                                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 235
#define RSSPECAN_ATTR_FREQUENCY_MASK_TRIGGER_NAME                                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 2951
#define RSSPECAN_ATTR_FREQUENCY_MASK_TRIGGER_COMMENT                                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 2952
#define RSSPECAN_ATTR_FREQUENCY_MASK_TRIGGER_DIRECTORY                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 2953
#define RSSPECAN_ATTR_FREQUENCY_MASK_TRIGGER_DELETE                                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 2954
#define RSSPECAN_ATTR_FREQUENCY_MASK_TRIGG_CONDITION                                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 2955
#define RSSPECAN_ATTR_FREQUENCY_MASK_TRIGGER_MODE                                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 2956
#define RSSPECAN_ATTR_FREQUENCY_MASK_UPP_AUTO                                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 2957
#define RSSPECAN_ATTR_FREQUENCY_MASK_LOWER_STATE                                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 2958
#define RSSPECAN_ATTR_FREQUENCY_MASK_UPPER_STATE                                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 2959
#define RSSPECAN_ATTR_FREQUENCY_MASK_SHIFT_LOW_X                                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 2960
#define RSSPECAN_ATTR_FREQUENCY_MASK_SHIFT_LOW_Y                                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 2961
#define RSSPECAN_ATTR_FREQUENCY_MASK_SHIFT_UPP_X                                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 2962
#define RSSPECAN_ATTR_FREQUENCY_MASK_SHIFT_UPP_Y                                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 2963
#define RSSPECAN_ATTR_FREQUENCY_MASK_SPAN                                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 2964
#define RSSPECAN_ATTR_EXTERNAL_MIXER_CONVERSION_LOSS_LOW                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 237
#define RSSPECAN_ATTR_EXTERNAL_MIXER_CONVERSION_LOSS_TABLE_LOW                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 246
#define RSSPECAN_ATTR_EXTERNAL_MIXER_CONVERSION_LOSS_HIGH                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 244
#define RSSPECAN_ATTR_EXTERNAL_MIXER_CONVERSION_LOSS_TABLE_HIGH                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 245
#define RSSPECAN_ATTR_CVL_NAME                                                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 75
#define RSSPECAN_ATTR_CVL_MIXER                                                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 76
#define RSSPECAN_ATTR_CVL_MIXER_SNUM                                                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 77
#define RSSPECAN_ATTR_CVL_BAND                                                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 78
#define RSSPECAN_ATTR_CVL_HARM                                                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 79
#define RSSPECAN_ATTR_CVL_PORT_MIXER                                                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 80
#define RSSPECAN_ATTR_CVL_BIAS                                                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 81
#define RSSPECAN_ATTR_CVL_COMMENT                                                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 82
#define RSSPECAN_ATTR_CVL_CLEAR                                                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 83
#define RSSPECAN_ATTR_CVL_CATALOG                                                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 3200
#define RSSPECAN_ATTR_EXTERNAL_MIXER_HARMONIC_TYPE                                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 242
#define RSSPECAN_ATTR_EXTERNAL_MIXER_HIGH_HARMONIC_STATE                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 255
#define RSSPECAN_ATTR_EXTERNAL_MIXER_HARMONIC                                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 240
#define RSSPECAN_ATTR_EXTERNAL_MIXER_HIGH_HARMONIC                                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 256
#define RSSPECAN_ATTR_EXTERNAL_MIXER_BIAS_LOW                                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 247
#define RSSPECAN_ATTR_EXTERNAL_MIXER_BIAS_HIGH                                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 248
#define RSSPECAN_ATTR_EXTERNAL_MIXER_SIGNAL_ID_MODE                                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 236
#define RSSPECAN_ATTR_EXTERNAL_MIXER_LO_POWER                                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 238
#define RSSPECAN_ATTR_EXTERNAL_MIXER_ENABLED                                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 239
#define RSSPECAN_ATTR_EXTERNAL_MIXER_INTERMEDIATE_FREQUENCY                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 10777
#define RSSPECAN_ATTR_EXTERNAL_MIXER_NUMBER_OF_PORTS                                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 241
#define RSSPECAN_ATTR_EXTERNAL_MIXER_HARMONIC_BAND                                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 243
#define RSSPECAN_ATTR_EXTERNAL_MIXER_THRESHOLD                                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 249
#define RSSPECAN_ATTR_GET_EXTERNAL_MIXER_RF_FREQUENCY_START                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 250
#define RSSPECAN_ATTR_GET_EXTERNAL_MIXER_RF_FREQUENCY_STOP                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 251
#define RSSPECAN_ATTR_EXTERNAL_MIXER_HANDOVER_FREQUENCY                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 252
#define RSSPECAN_ATTR_EXTERNAL_MIXER_RF_OVERRANGE                                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 253
#define RSSPECAN_ATTR_EXTERNAL_MIXER_PRESET_BAND                                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 254
#define RSSPECAN_ATTR_EXTERNAL_MIXER_XCORR                                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 6646
#define RSSPECAN_ATTR_EXTERNAL_FRONTEND_ENABLED                                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 10778
#define RSSPECAN_ATTR_EXTERNAL_FRONTEND_CONNECTION_FIRMWARE_STATE                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 10780
#define RSSPECAN_ATTR_EXTERNAL_FRONTEND_CONNECTION_PHYSICAL_STATE                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 10781
#define RSSPECAN_ATTR_EXTERNAL_FRONTEND_IDENTIFICATION                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 10783
#define RSSPECAN_ATTR_EXTERNAL_FRONTEND_FREQUENCY_BAND_RANGE_LIST                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 10784
#define RSSPECAN_ATTR_EXTERNAL_FRONTEND_FREQUENCY_BAND_RANGE                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 10785
#define RSSPECAN_ATTR_EXTERNAL_FRONTEND_FREQUENCY_BAND_AUTO_ENABLED                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 10786
#define RSSPECAN_ATTR_EXTERNAL_FRONTEND_SELECTED_FREQUENCY_BAND                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 10787
#define RSSPECAN_ATTR_EXTERNAL_FRONTEND_FREQUENCY_BAND_COUNT                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 10788
#define RSSPECAN_ATTR_EXTERNAL_FRONTEND_FREQUENCY_BAND_RANGE_UPPER                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 10789
#define RSSPECAN_ATTR_EXTERNAL_FRONTEND_FREQUENCY_BAND_RANGE_LOWER                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 10790
#define RSSPECAN_ATTR_EXTERNAL_FRONTEND_INTERMEDIATE_FREQUENCY_SIDEBAND                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 10791
#define RSSPECAN_ATTR_EXTERNAL_FRONTEND_INTERMEDIATE_FREQUENCY                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 10792
#define RSSPECAN_ATTR_EXTERNAL_FRONTEND_REFERENCE_FREQUENCY_LIST                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 10793
#define RSSPECAN_ATTR_EXTERNAL_FRONTEND_REFERENCE_FREQUENCY                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 10794
#define RSSPECAN_ATTR_EXTERNAL_FRONTEND_UPDATE_FIRMWARE                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 10795
#define RSSPECAN_ATTR_EXTERNAL_FRONTEND_ALIGNMENT_CORRECTION_FILE_ENABLED                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 10796
#define RSSPECAN_ATTR_EXTERNAL_FRONTEND_ALIGNMENT_CORRECTION_FILE                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 10797
#define RSSPECAN_ATTR_EXTERNAL_FRONTEND_DIAGNOSTICS_START_SELFTEST                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 10798
#define RSSPECAN_ATTR_EXTERNAL_FRONTEND_DIAGNOSTICS_SELFTEST_RESULTS                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 10799
#define RSSPECAN_ATTR_EXTERNAL_FRONTEND_DEVICE_NAME                                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 10800
#define RSSPECAN_ATTR_EXTERNAL_FRONTEND_DEVICE_NAME_LIST                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 10801
#define RSSPECAN_ATTR_EXTERNAL_FRONTEND_DEVICE_TYPE                                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 10802
#define RSSPECAN_ATTR_EXTERNAL_FRONTEND_DEVICE_FREQUENCY_RANGE_LOWER                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 10803
#define RSSPECAN_ATTR_EXTERNAL_FRONTEND_DEVICE_FREQUENCY_RANGE_UPPER                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 10804
#define RSSPECAN_ATTR_EXTERNAL_FRONTEND_DEVICE_GAIN                                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 10805
#define RSSPECAN_ATTR_EXTERNAL_FRONTEND_DEVICE_REFRESH                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 10806
#define RSSPECAN_ATTR_EXTERNAL_FRONTEND_DEVICE_CORRECTION_ENABLED                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 10807
#define RSSPECAN_ATTR_EXTERNAL_FRONTEND_DEVICE_CORRECTION_FILE                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 10808
#define RSSPECAN_ATTR_EXTERNAL_FRONTEND_DEVICE_CORRECTION_FILE_VALID                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 10809
#define RSSPECAN_ATTR_AMPLITUDE_UNITS                                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 1
#define RSSPECAN_ATTR_ATTENUATION                                                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 2
#define RSSPECAN_ATTR_ATTENUATION_AUTO                                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 3
#define RSSPECAN_ATTR_ATTENUATION_MODE                                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 4
#define RSSPECAN_ATTR_REFERENCE_LEVEL                                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 22
#define RSSPECAN_ATTR_REFERENCE_LEVEL_SET_TO_MIN_MAX                                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 6028
#define RSSPECAN_ATTR_REFERENCE_LEVEL_MIN                                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 6029
#define RSSPECAN_ATTR_REFERENCE_LEVEL_MAX                                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 6030
#define RSSPECAN_ATTR_REFERENCE_LEVEL_OFFSET                                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 23
#define RSSPECAN_ATTR_AMPL_EATT_STATE                                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 258
#define RSSPECAN_ATTR_AMPL_EATT_AUTO                                                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 259
#define RSSPECAN_ATTR_AMPL_EATT_VALUE                                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 260
#define RSSPECAN_ATTR_AMPLITUDE_ANALYZER_ATTENUATION                                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 10810
#define RSSPECAN_ATTR_AMPLITUDE_ANALYZER_ATTENUATION_AUTO                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 10811
#define RSSPECAN_ATTR_AMPL_PREAMPLIFIER                                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 263
#define RSSPECAN_ATTR_AMPL_PREAMPLIFIER_LEVEL                                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 2681
#define RSSPECAN_ATTR_AMPL_PROTECTION_RESET                                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 264
#define RSSPECAN_ATTR_AMPL_RF_COUPLING                                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 257
#define RSSPECAN_ATTR_AMPL_YIG_FILTER                                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 265
#define RSSPECAN_ATTR_AMPL_HIGHPASS_FILTER                                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 2682
#define RSSPECAN_ATTR_RF_INPUT                                                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 4698
#define RSSPECAN_ATTR_AMPLITUDE_SIGNAL_PATH_ENABLED                                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 10812
#define RSSPECAN_ATTR_INPUT_CONNECTOR                                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 6939
#define RSSPECAN_ATTR_INPUT_IMPEDANCE                                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 20
#define RSSPECAN_ATTR_INPUT_IMPEDANCE_USER                                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 6537
#define RSSPECAN_ATTR_INPUT_IMPEDANCE_PAD_TYPE                                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 6538
#define RSSPECAN_ATTR_INPUT_TYPE                                                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 7578
#define RSSPECAN_ATTR_NOISE_INP_PRES_STAT                                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 1498
#define RSSPECAN_ATTR_DIRECT_PATH                                                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 5599
#define RSSPECAN_ATTR_PRESELECTOR_ADJUSTMENT                                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 6647
#define RSSPECAN_ATTR_INSTRUMENT_AUTO_LEVEL                                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 7579
#define RSSPECAN_ATTR_INSTRUMENT_ELECTRONIC_ATTENUATION_STATE                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 7580
#define RSSPECAN_ATTR_INSTRUMENT_ELECTRONIC_ATTENUATION_AUTO                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 7581
#define RSSPECAN_ATTR_INSTRUMENT_ELECTRONIC_ATTENUATION_MANUAL                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 7582
#define RSSPECAN_LIMIT_STATE                                                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 266
#define RSSPECAN_LIMIT_CLEAR                                                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 267
#define RSSPECAN_LIMIT_ASSIGN_TRACE                                                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 268
#define RSSPECAN_LIMIT_UNITS                                                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 269
#define RSSPECAN_LIMIT_COMMENT                                                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 270
#define RSSPECAN_LIMIT_COPY                                                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 271
#define RSSPECAN_LIMIT_NAME                                                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 272
#define RSSPECAN_LIMIT_DELETE                                                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 273
#define RSSPECAN_ATTR_LIMIT_LINE_ACTIVE                                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 3213
#define RSSPECAN_ATTR_LIMIT_LINE_COMPATIBLE                                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 7420
#define RSSPECAN_LIMIT_CHECK_RESULT                                                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 409
#define RSSPECAN_LIMIT_IMPORT                                                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 6539
#define RSSPECAN_LIMIT_CONTROL_DOMAIN                                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 277
#define RSSPECAN_LIMIT_CONTROL_SPACING                                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 278
#define RSSPECAN_LIMIT_CONTROL_SCALING                                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 279
#define RSSPECAN_LIMIT_CONTROL_OFFSET                                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 280
#define RSSPECAN_LIMIT_CONTROL_SHIFT                                                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 281
#define RSSPECAN_LIMIT_LOWER_STATE                                                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 282
#define RSSPECAN_LIMIT_LOWER_SCALING                                                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 283
#define RSSPECAN_LIMIT_LOWER_SPACING                                                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 284
#define RSSPECAN_LIMIT_LOWER_MARGIN                                                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 285
#define RSSPECAN_LIMIT_LOWER_THRESHOLD                                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 286
#define RSSPECAN_LIMIT_LOWER_OFFSET                                                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 287
#define RSSPECAN_LIMIT_LOWER_SHIFT                                                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 288
#define RSSPECAN_LIMIT_UPPER_STATE                                                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 289
#define RSSPECAN_LIMIT_UPPER_SCALING                                                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 290
#define RSSPECAN_LIMIT_UPPER_SPACING                                                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 291
#define RSSPECAN_LIMIT_UPPER_MARGIN                                                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 292
#define RSSPECAN_LIMIT_UPPER_THRESHOLD                                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 293
#define RSSPECAN_LIMIT_UPPER_OFFSET                                                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 294
#define RSSPECAN_LIMIT_UPPER_SHIFT                                                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 295
#define RSSPECAN_ATTR_LIMIT_LINE_TRACES_TO_BE_CHECKED                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 10705
#define RSSPECAN_ATTR_DLINE_STATE                                                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 298
#define RSSPECAN_ATTR_DLINE_POSITION                                                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 299
#define RSSPECAN_ATTR_FLINE_STATE                                                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 300
#define RSSPECAN_ATTR_FLINE_POSITION                                                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 301
#define RSSPECAN_ATTR_TLINE_STATE                                                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 302
#define RSSPECAN_ATTR_TLINE_POSITION                                                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 303
#define RSSPECAN_ATTR_THRLINE_STATE                                                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 296
#define RSSPECAN_ATTR_THRLINE_POSITION                                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 297
#define RSSPECAN_ATTR_DISP_REF_VALUE                                                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 304
#define RSSPECAN_ATTR_DISP_REF_POSITION                                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 305
#define RSSPECAN_ATTR_DISP_FP_KEYS                                                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 306
#define RSSPECAN_ATTR_SELECT_WINDOW                                                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 307
#define RSSPECAN_ATTR_DISP_CHANNEL_BAR_STATE                                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 4949
#define RSSPECAN_ATTR_DISP_FREQ_STATE                                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 308
#define RSSPECAN_ATTR_DISP_LOGO_STATE                                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 309
#define RSSPECAN_ATTR_DISP_TIME_STATE                                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 310
#define RSSPECAN_ATTR_DISP_COMMENT                                                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 311
#define RSSPECAN_ATTR_DISP_COMMENT_STATE                                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 312
#define RSSPECAN_ATTR_DISP_PWR_SAVE_STATE                                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 313
#define RSSPECAN_ATTR_DISP_PWR_SAVE_HOLDOFF                                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 314
#define RSSPECAN_ATTR_DISP_FORMAT                                                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 315
#define RSSPECAN_ATTR_DISP_WINDOW_SIZE                                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 316
#define RSSPECAN_ATTR_DISP_LOG_RANGE                                                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 321
#define RSSPECAN_ATTR_DISP_AMPLITUDE_GRID_MODE                                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 322
#define RSSPECAN_ATTR_DISP_SINGLE_SWEEP                                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 325
#define RSSPECAN_ATTR_DISP_UPDATE                                                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 326
#define RSSPECAN_ATTR_UNIT_POWER                                                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 329
#define RSSPECAN_ATTR_DISP_TRACE_Y_AUTO                                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 332
#define RSSPECAN_ATTR_DISP_Y_AXIS_SCALING                                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 334
#define RSSPECAN_ATTR_DISP_STATUS_BAR                                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 335
#define RSSPECAN_ATTR_DISP_SOFTKEY_BAR                                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 336
#define RSSPECAN_ATTR_DISP_TOOL_BAR                                                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 337
#define RSSPECAN_ATTR_DISP_TOUCHSCREEN_STATE                                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 338
#define RSSPECAN_ATTR_DISP_THEME_CATALOG                                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 3215
#define RSSPECAN_ATTR_DISP_SELECT_THEME                                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 339
#define RSSPECAN_ATTR_DISP_MARKER_TABLE                                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 340
#define RSSPECAN_ATTR_DISPLAY_MARKER_INFO                                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 5600
#define RSSPECAN_ATTR_DISPLAY_ZOOM_STATE                                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 344
#define RSSPECAN_ATTR_DISPLAY_MULTIVIEW_TAB_ENABLED                                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 6940
#define RSSPECAN_ATTR_DISPLAY_TIME_FORMAT                                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 6941
#define RSSPECAN_ATTR_DISPLAY_STATE                                                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 341
#define RSSPECAN_ATTR_DISP_WINDOW_WIN_SIZE                                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 6942
#define RSSPECAN_ATTR_DISPLAY_TRACE_SYMBOL_ENABLED                                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 6943
#define RSSPECAN_ATTR_DISPLAY_AUTO_SCALING_ENABLED                                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 6944
#define RSSPECAN_ATTR_DISPLAY_WINDOW_X_AXIS_AUTO_SCALING                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 9872
#define RSSPECAN_ATTR_DISPLAY_WINDOW_X_AXIS_MAXIMUM                                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 9873
#define RSSPECAN_ATTR_DISPLAY_WINDOW_X_AXIS_MINIMUM                                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 9874
#define RSSPECAN_ATTR_DISPLAY_WINDOW_X_AXIS_GRID_SPACING                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 9875
#define RSSPECAN_ATTR_DISPLAY_WINDOW_X_AXIS_UNIT                                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 9876
#define RSSPECAN_ATTR_DISPLAY_Y_AXIS_SCALE_MIN                                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 6945
#define RSSPECAN_ATTR_DISPLAY_Y_AXIS_SCALE_MAX                                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 6946
#define RSSPECAN_ATTR_DISPLAY_WINDOW_Y_AXIS_AUTO_SCALING                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 9877
#define RSSPECAN_ATTR_DISPLAY_WINDOW_Y_AXIS_MAXIMUM                                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 7099
#define RSSPECAN_ATTR_DISPLAY_WINDOW_Y_AXIS_MINIMUM                                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 7100
#define RSSPECAN_ATTR_DISPLAY_WINDOW_Y_AXIS_GRID_SPACING                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 7101
#define RSSPECAN_ATTR_DISPLAY_WINDOW_Y_AXIS_UNIT                                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 9878
#define RSSPECAN_ATTR_DISPLAY_WINDOW_REFERENCE_LEVEL_POSITION                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 7102
#define RSSPECAN_ATTR_DISPLAY_WINDOW_REFERENCE_LEVEL                                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 7103
#define RSSPECAN_ATTR_DISPLAY_WINDOW_REFERENCE_LEVEL_MAX                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 7104
#define RSSPECAN_ATTR_DISPLAY_WINDOW_REFERENCE_LEVEL_MIN                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 7105
#define RSSPECAN_ATTR_DISPLAY_WINDOW_UNIT_PHASE                                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 7106
#define RSSPECAN_ATTR_DISPLAY_WINDOW_UNIT_FREQUENCY                                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 7107
#define RSSPECAN_ATTR_DISPLAY_WINDOW_REFERENCE_VALUE                                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 7583
#define RSSPECAN_ATTR_DISPLAY_WINDOW_MARKER_INFO                                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 7584
#define RSSPECAN_ATTR_DISPLAY_SELECT_TAB                                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 7585
#define RSSPECAN_ATTR_DISPLAY_FOCUS_AREA                                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 2372
#define RSSPECAN_ATTR_SUBWINDOW_DISPLAY_ZOOM_STATE                                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 7587
#define RSSPECAN_ATTR_SUBWINDOW_DISPLAY_AMPLITUDE_GRID_MODE                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 7588
#define RSSPECAN_ATTR_SUBWINDOW_DISPLAY_REFERENCE_LEVEL                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 8576
#define RSSPECAN_ATTR_DISPLAY_MULTIPLE_ZOOM_STATE                                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 2683
#define RSSPECAN_ATTR_SUBWINDOW_DISPLAY_MULTIPLE_ZOOM_STATE                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 7590
#define RSSPECAN_ATTR_LAYOUT_WINDOW_REMOVE                                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 2684
#define RSSPECAN_ATTR_LAYOUT_WINDOW_LIST                                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 2685
#define RSSPECAN_ATTR_LAYOUT_DIRECTION                                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 9879
#define RSSPECAN_ATTR_LAYOUT_QUERY_WINDOW_NAME                                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 2686
#define RSSPECAN_ATTR_DISP_COL_PRESET                                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 317
#define RSSPECAN_ATTR_DISP_COL_PREDEFINED                                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 320
#define RSSPECAN_ATTR_HCOPY_ABORT                                                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 345
#define RSSPECAN_ATTR_HCOPY_CONTENT                                                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 6484
#define RSSPECAN_ATTR_HCOPY_PRINT_SCREEN                                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 353
#define RSSPECAN_ATTR_HCOPY_PRINT_TAB                                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 354
#define RSSPECAN_ATTR_HCOPY_COMM_SCR                                                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 355
#define RSSPECAN_ATTR_HCOPY_FILE_NAME                                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 358
#define RSSPECAN_ATTR_HCOPY_PRINTER_NEXT                                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 360
#define RSSPECAN_ATTR_HCOPY_PRINTER_FIRST                                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 361
#define RSSPECAN_ATTR_HCOPY_PAGE_PRINTING_STATE                                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 6485
#define RSSPECAN_ATTR_HCOPY_PRINT                                                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 348
#define RSSPECAN_ATTR_HCOPY_PRINT_NEXT                                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 349
#define RSSPECAN_ATTR_HCOPY_DEVICE_DESTINATION                                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 350
#define RSSPECAN_ATTR_HCOPY_DEVICE_ORIENTATION                                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 357
#define RSSPECAN_ATTR_HCOPY_PAGE_MARGIN_TOP                                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 6486
#define RSSPECAN_ATTR_HCOPY_PAGE_MARGIN_LEFT                                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 6487
#define RSSPECAN_ATTR_HCOPY_PAGE_MARGIN_BOTTOM                                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 6488
#define RSSPECAN_ATTR_HCOPY_PAGE_MARGIN_RIGHT                                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 6489
#define RSSPECAN_ATTR_HCOPY_PAGE_MARGIN_UNIT                                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 6490
#define RSSPECAN_ATTR_HCOPY_WINDOWS_COUNT                                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 6491
#define RSSPECAN_ATTR_HCOPY_WINDOWS_SCALE                                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 6492
#define RSSPECAN_ATTR_HCOPY_PRINTER                                                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 359
#define RSSPECAN_ATTR_HCOPY_TIME_DATE                                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 362
#define RSSPECAN_ATTR_HCOPY_DEVICE_LANG_OUT_FORM                                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 352
#define RSSPECAN_ATTR_HCOPY_COLOR_DEF                                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 346
#define RSSPECAN_ATTR_HCOPY_COLOR                                                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 347
#define RSSPECAN_ATTR_HCOPY_COLOR_PREDEFINED                                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 351
#define RSSPECAN_ATTR_PRINT_MODE                                                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 9880
#define RSSPECAN_ATTR_TEST_REPORTS_PAGE_FORMAT                                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 9881
#define RSSPECAN_ATTR_TEST_REPORTS_USE_SCREEN_COLORS                                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 9882
#define RSSPECAN_ATTR_TEST_REPORTS_DATE                                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 9883
#define RSSPECAN_ATTR_TEST_REPORTS_PAGE_COUNT                                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 9884
#define RSSPECAN_ATTR_TEST_REPORTS_TITLE_PAGE                                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 9885
#define RSSPECAN_ATTR_TEST_REPORTS_TITLE                                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 9886
#define RSSPECAN_ATTR_TEST_REPORTS_ABSTRACT                                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 9887
#define RSSPECAN_ATTR_TEST_REPORTS_MEASUREMENT_INFO_STATE                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 9888
#define RSSPECAN_ATTR_TEST_REPORTS_MEASUREMENT_INFO_TITLE                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 9889
#define RSSPECAN_ATTR_TEST_REPORTS_MEASUREMENT_INFO_VALUE                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 9890
#define RSSPECAN_ATTR_TEST_REPORTS_MEASUREMENT_INFO_VISIBILITY                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 9891
#define RSSPECAN_ATTR_TEST_REPORTS_MEASUREMENT_INFO_LOGO                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 9892
#define RSSPECAN_ATTR_TEST_REPORTS_MEASUREMENT_INFO_LOGO_VISIBILITY                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 9893
#define RSSPECAN_ATTR_TEST_REPORTS_ITEM_SELECT                                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 9894
#define RSSPECAN_ATTR_TEST_REPORTS_TEMPLATES_CATALOG                                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 9896
#define RSSPECAN_ATTR_TEST_REPORTS_SAVE_TEMPLATE                                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 9897
#define RSSPECAN_ATTR_TEST_REPORTS_LOAD_TEMPLATE                                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 9898
#define RSSPECAN_ATTR_TEST_REPORTS_RESTORE_DEFAULT_CONFIGURATION                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 9899
#define RSSPECAN_ATTR_TEST_REPORTS_NEW                                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 9900
#define RSSPECAN_ATTR_TEST_REPORTS_APPEND                                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 9901
#define RSSPECAN_ATTR_TEST_REPORTS_REMOVE_DATASET                                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 9902
#define RSSPECAN_ATTR_TEST_REPORTS_REMOVE_ALL_DATASETS                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 9903
#define RSSPECAN_ATTR_APPLICATION_STARTER_WORKING_DIRECTORY                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 9906
#define RSSPECAN_ATTR_APPLICATION_STARTER_DISPLAY_NAME                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 9908
#define RSSPECAN_ATTR_APPLICATION_STARTER_PARAMETERS                                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 9909
#define RSSPECAN_ATTR_APPLICATION_STARTER_PATH                                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 9910
#define RSSPECAN_ATTR_FILE_RECALL                                                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 363
#define RSSPECAN_ATTR_FILE_STARTUP_RECALL                                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 364
#define RSSPECAN_ATTR_FILE_SAVE                                                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 365
#define RSSPECAN_ATTR_FILE_SAVE_NEXT                                                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 366
#define RSSPECAN_ATTR_FILE_DATA_CLEAR                                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 367
#define RSSPECAN_ATTR_FILE_DATA_CLEAR_ALL                                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 368
#define RSSPECAN_ATTR_FILE_EDIT_COMM                                                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 369
#define RSSPECAN_ATTR_FILE_DEC_SEPARATOR                                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 370
#define RSSPECAN_ATTR_FILE_DECIMAL_TRACES                                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 7423
#define RSSPECAN_ATTR_FILE_EXP_HEADER                                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 4699
#define RSSPECAN_ATTR_FILE_EXPORT_TO_MICROSERVICE                                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 8616
#define RSSPECAN_ATTR_FILE_EXPORT_COLUMN_SEPERATOR                                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 9912
#define RSSPECAN_ATTR_FILE_EXPORT_X_VALUE_DISTRIBUTION                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 9913
#define RSSPECAN_ATTR_FILE_MANAGER_EDIT_PATH                                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 371
#define RSSPECAN_ATTR_FILE_MANAGER_DELETE                                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 372
#define RSSPECAN_ATTR_FILE_MANAGER_MAKE_DIR                                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 3317
#define RSSPECAN_ATTR_FILE_MANAGER_EDIT_PATH_DEVICE                                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 375
#define RSSPECAN_ATTR_FILE_MANAGER_DELETE_DIR                                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 3318
#define RSSPECAN_ATTR_FILE_ITEMS_SAVE_RECAL_SEL_TRAN                                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 378
#define RSSPECAN_ATTR_FILE_ITEMS_SAVE_RECAL_SEL_HWSETTINGS                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 379
#define RSSPECAN_ATTR_FILE_ITEMS_SAVE_RECAL_SEL_TRAC                                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 380
#define RSSPECAN_ATTR_FILE_ITEMS_SAVE_RECAL_SEL_LINE                                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 381
#define RSSPECAN_ATTR_FILE_ITEMS_SAVE_RECAL_ALL                                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 382
#define RSSPECAN_ATTR_FILE_ITEMS_SAVE_RECAL_NONE                                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 383
#define RSSPECAN_ATTR_FILE_ITEMS_SAVE_RECAL_DEFAULT                                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 384
#define RSSPECAN_ATTR_FILE_ITEMS_SAVE_RECAL_INCLUDE_IQ_DATA                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 2662
#define RSSPECAN_ATTR_FILE_ITEMS_TO_SAVE_RECALL_SOURCE_CALIBRATION                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 9914
#define RSSPECAN_ATTR_FILE_ITEMS_SAVE_RECAL_WEIGHTING                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 6033
#define RSSPECAN_ATTR_DATA_SET_LOAD_MODE                                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 2876
#define RSSPECAN_ATTR_DATA_SET_STORE_MODE                                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 2877
#define RSSPECAN_ATTR_FILE_ITEMS_SAVE_RECAL_CHANNEL_NONE                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 2868
#define RSSPECAN_ATTR_FILE_ITEMS_SAVE_RECAL_CHANNEL_DEFAULT                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 2869
#define RSSPECAN_ATTR_FILE_ITEMS_SAVE_RECAL_CHANNEL_ALL                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 2870
#define RSSPECAN_ATTR_FILE_ITEMS_SAVE_RECAL_CHANNEL_SEL_HWSETTINGS                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 2871
#define RSSPECAN_ATTR_FILE_ITEMS_SAVE_RECAL_CHANNEL_SEL_TRAC                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 2872
#define RSSPECAN_ATTR_FILE_ITEMS_SAVE_RECAL_CHANNEL_SEL_LINE                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 2873
#define RSSPECAN_ATTR_FILE_ITEMS_SAVE_RECAL_CHANNEL_SEL_TRAN                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 2874
#define RSSPECAN_ATTR_FILE_ITEMS_SAVE_RECAL_CHANNEL_SPEM                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 2875
#define RSSPECAN_ATTR_FILE_ITEMS_SAVE_RECAL_CHANNEL_WEIGHTING                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 6034
#define RSSPECAN_ATTR_FILE_ITEMS_TO_SAVE_RECALL_CHANNEL_SOURCE_CALIBRATION                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 9915
#define RSSPECAN_ATTR_NETWORK_DISCONNECT_DRIVE                                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 387
#define RSSPECAN_ATTR_UNUSED_NETWORK_DRIVES                                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 388
#define RSSPECAN_ATTR_USED_NETWORK_DRIVES                                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 389
#define RSSPECAN_ATTR_SERVICE_LAST_DATE                                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 9916
#define RSSPECAN_ATTR_SERVICE_INPUT_SOURCE                                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 391
#define RSSPECAN_ATTR_SERVICE_PULSE_CAL_FREQ_FSV                                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 403
#define RSSPECAN_ATTR_SERVICE_WIDEBAND_CALIBRATION_FREQUENCY                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 5289
#define RSSPECAN_ATTR_SERVICE_INTERNAL_WIDEBAND_CALIBRATION_FREQUENCY                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 6493
#define RSSPECAN_ATTR_SERVICE_CAL_FREQ_RF_SPECTRUM                                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 4700
#define RSSPECAN_ATTR_SERVICE_CAL_FREQ_MW_DIST                                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 4701
#define RSSPECAN_ATTR_SERVICE_CAL_FREQ_MW                                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 4702
#define RSSPECAN_ATTR_SERVICE_CAL_FREQ_MICROWAVE                                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 6006
#define RSSPECAN_ATTR_SERVICE_CALIBRATION_LAST_DATE                                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 9917
#define RSSPECAN_ATTR_SERVICE_CALIBRATION_NEXT_DATE                                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 9918
#define RSSPECAN_ATTR_SERVICE_CALIBRATION_INTERVAL                                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 9919
#define RSSPECAN_ATTR_SERVICE_HW_INFO                                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 404
#define RSSPECAN_ATTR_SERVICE_STEST_RESULTS                                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 405
#define RSSPECAN_ATTR_SERVICE_STEST                                                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 406
#define RSSPECAN_ATTR_SERVICE_FUNCTION_START                                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 9920
#define RSSPECAN_ATTR_SERVICE_FUNCTION_SAVE_RESULTS                                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 9921
#define RSSPECAN_ATTR_SERVICE_FUNCTION_CLEAR_RESULTS                                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 9922
#define RSSPECAN_ATTR_SERVICE_FUNCTION_LAST_RESULT                                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 9923
#define RSSPECAN_ATTR_SERVICE_FUNCTION_CREATE_SUPPORT_INFROMATION                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 9924
#define RSSPECAN_ATTR_SERVICE_FUNCTION_PASSWORD                                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 9925
#define RSSPECAN_ATTR_SERVICE_FUNCTIONS_CLEAR_PASSWORD                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 9926
#define RSSPECAN_ATTR_MEAS_ABORT                                                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 47
#define RSSPECAN_ATTR_INIT                                                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 48
#define RSSPECAN_ATTR_INIT_WITHOUT_OPC                                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 49
#define RSSPECAN_ATTR_INIT_CONMEAS                                                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 50
#define RSSPECAN_ATTR_INIT_SW_TRIGGER                                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 52
#define RSSPECAN_ATTR_INIT_SPURIOUS                                                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 53
#define RSSPECAN_ATTR_INIT_SEM_MEAS                                                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 56
#define RSSPECAN_ATTR_UPDATE_IQ_RESULTS                                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 57
#define RSSPECAN_ATTR_INIT_SEQUENCER                                                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 2893
#define RSSPECAN_ATTR_ABORT_SEQUENCER                                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 2894
#define RSSPECAN_ATTR_REFRESH_SEQUENCER                                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 2895
#define RSSPECAN_ATTR_SEQUENCER_STATE                                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 2896
#define RSSPECAN_ATTR_SEQUENCER_MODE                                                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 2897
#define RSSPECAN_ATTR_SEQUENCER_CALCULATE_TIME_TRIGGER                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 10816
#define RSSPECAN_ATTR_REFERENCE_MARKER_PNO_STATE                                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 412
#define RSSPECAN_ATTR_DELTA_MARKER_PHASE_NOISE_ALL_OFF                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 6949
#define RSSPECAN_ATTR_REFERENCE_MARKER_PNO_RESULT                                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 413
#define RSSPECAN_ATTR_REFERENCE_MARKER_PNO_AUTO                                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 414
#define RSSPECAN_ATTR_MARKER_NDB_STATE                                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 418
#define RSSPECAN_ATTR_MARKER_NDB_VAL                                                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 419
#define RSSPECAN_ATTR_MARKER_NDB_RESULT                                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 420
#define RSSPECAN_ATTR_MARKER_NDB_Q_FACTOR                                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 421
#define RSSPECAN_ATTR_LIST_POW_STATE_OFF                                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 423
#define RSSPECAN_ATTR_MEAS_TDOM_STATE                                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 424
#define RSSPECAN_ATTR_MEAS_TDOM_PEAK                                                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 425
#define RSSPECAN_ATTR_MEAS_TDOM_RMS                                                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 426
#define RSSPECAN_ATTR_MEAS_TDOM_MEAN                                                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 427
#define RSSPECAN_ATTR_MEAS_TDOM_SDEV                                                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 428
#define RSSPECAN_ATTR_MEAS_TDOM_AVG                                                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 429
#define RSSPECAN_ATTR_MEAS_TDOM_MAX_HOLD                                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 430
#define RSSPECAN_ATTR_MEAS_TDOM_MODE                                                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 431
#define RSSPECAN_ATTR_MEAS_TDOM_AOFF                                                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 432
#define RSSPECAN_ATTR_MEAS_TDOM_SET_REFERENCE                                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 433
#define RSSPECAN_ATTR_MEAS_TDOM_PEAK_RESULT                                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 434
#define RSSPECAN_ATTR_MEAS_TDOM_RMS_RESULT                                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 435
#define RSSPECAN_ATTR_MEAS_TDOM_MEAN_RESULT                                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 436
#define RSSPECAN_ATTR_MEAS_TDOM_SDEV_RESULT                                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 437
#define RSSPECAN_ATTR_MEAS_TDOM_AVG_PEAK_RESULT                                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 438
#define RSSPECAN_ATTR_MEAS_TDOM_AVG_RMS_RESULT                                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 439
#define RSSPECAN_ATTR_MEAS_TDOM_AVG_MEAN_RESULT                                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 440
#define RSSPECAN_ATTR_MEAS_TDOM_AVG_SDEV_RESULT                                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 441
#define RSSPECAN_ATTR_MEAS_TDOM_PEAK_HOLD_RESULT                                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 442
#define RSSPECAN_ATTR_MEAS_TDOM_PEAK_HOLD_RMS_RESULT                                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 443
#define RSSPECAN_ATTR_MEAS_TDOM_PEAK_HOLD_MEAN_RESULT                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 444
#define RSSPECAN_ATTR_MEAS_TDOM_PEAK_HOLD_SDEV_RESULT                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 445
#define RSSPECAN_ATTR_MEAS_POW_SELECT                                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 446
#define RSSPECAN_ATTR_MEAS_POW_OFF                                                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 447
#define RSSPECAN_ATTR_MEAS_POW_ADJ_NUM                                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 449
#define RSSPECAN_ATTR_MEAS_POW_ADJ_MODE                                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 450
#define RSSPECAN_ATTR_MEAS_POW_MODE                                                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 451
#define RSSPECAN_ATTR_POWER_UNIT                                                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 9927
#define RSSPECAN_ATTR_MEAS_POW_RESULT_MODE                                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 453
#define RSSPECAN_ATTR_MEAS_POW_ADJ_CHANNEL_SPACING                                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 455
#define RSSPECAN_ATTR_MEAS_POW_ADJ_CHANNEL_NAME                                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 2375
#define RSSPECAN_ATTR_MEAS_POW_ADJ_CHANNEL_BANDWIDTH                                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 459
#define RSSPECAN_ATTR_MEAS_POW_NOISE_CORR                                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 463
#define RSSPECAN_ATTR_MEAS_POW_NOISE_CORR_AUTO                                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 2772
#define RSSPECAN_ATTR_MEAS_POW_TRACE                                                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 464
#define RSSPECAN_ATTR_MEAS_POW_STANDARD                                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 465
#define RSSPECAN_ATTR_MEAS_POW_STANDARD_CATALOG                                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 3230
#define RSSPECAN_ATTR_MEAS_POW_REF_VALUE                                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 466
#define RSSPECAN_ATTR_MEAS_POW_ADJ_PRESET                                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 467
#define RSSPECAN_ATTR_MEAS_POW_CARR_SIG_NUM                                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 481
#define RSSPECAN_ATTR_MEAS_POW_REF_CHAN_SEL_AUTO                                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 482
#define RSSPECAN_ATTR_MEAS_POW_ADJ_REF_TXCHANNEL                                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 483
#define RSSPECAN_ATTR_MEAS_POW_BANDWIDTH                                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 484
#define RSSPECAN_ATTR_SEM_WEIGHT_FILTER_ALL_STATE                                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 601
#define RSSPECAN_ATTR_SEM_WEIGHT_FILTER_ALL_ALPHA                                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 602
#define RSSPECAN_ATTR_SEM_WEIGHT_FILTER_ACH_STATE                                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 603
#define RSSPECAN_ATTR_SEM_WEIGHT_FILTER_ACH_ALPHA                                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 604
#define RSSPECAN_ATTR_POWER_ADJACENT_CHANNEL_AUTOMATIC_MEASUREMENT_BANDWIDTH_STATE                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 9928
#define RSSPECAN_ATTR_MSR_NUMBER_OF_SUBBLOCKS                                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 3054
#define RSSPECAN_ATTR_MSR_SYMMETRICAL_ADJACENT_SETUP_ENABLED                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 5290
#define RSSPECAN_ATTR_MSR_SUBBLOCK_CENTER_FREQUENCY                                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 3055
#define RSSPECAN_ATTR_MSR_SUBBLOCK_RF_BANDWIDTH                                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 3056
#define RSSPECAN_ATTR_MSR_SUBBLOCK_TX_CHANNEL_COUNT                                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 3057
#define RSSPECAN_ATTR_MSR_TX_CENTER_FREQUENCY                                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 3058
#define RSSPECAN_ATTR_MSR_TX_TECHNOLOGY                                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 3059
#define RSSPECAN_ATTR_MSR_TX_BANDWIDTH                                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 3060
#define RSSPECAN_ATTR_MSR_TX_WEIGHTING_FILTER_STATE                                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 3061
#define RSSPECAN_ATTR_MSR_TX_WEIGHTING_FILTER_ROLL_OFF_FACTOR                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 3062
#define RSSPECAN_ATTR_MSR_TX_CHANNEL_NAME                                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 3063
#define RSSPECAN_ATTR_MSR_UPPER_ADJACENT_CHANNEL_SPACING                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 5292
#define RSSPECAN_ATTR_MSR_UPPER_ADJACENT_CHANNEL_BANDWIDTH                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 5293
#define RSSPECAN_ATTR_MSR_UPPER_ADJACENT_CHANNEL_NAME                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 5294
#define RSSPECAN_ATTR_MSR_UPPER_ADJACENT_CHANNEL_WEIGHTING_FILTER_STATE                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 5295
#define RSSPECAN_ATTR_MSR_UPPER_ADJACENT_CHANNEL_WEIGHTING_FILTER_ALPHA_VALUE                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 5296
#define RSSPECAN_ATTR_MSR_UPPER_ALTERNATE_CHANNEL_SPACING                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 5297
#define RSSPECAN_ATTR_MSR_UPPER_ALTERNATE_CHANNEL_BANDWIDTH                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 5298
#define RSSPECAN_ATTR_MSR_UPPER_ALTERNATE_CHANNEL_NAME                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 5299
#define RSSPECAN_ATTR_MSR_UPPER_ALTERNATE_CHANNEL_WEIGHTING_FILTER_STATE                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 5300
#define RSSPECAN_ATTR_MSR_UPPER_ALTERNATE_CHANNEL_WEIGHTING_FILTER_ALPHA_VALUE                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 5301
#define RSSPECAN_ATTR_MSR_GAP_NAME                                                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 7591
#define RSSPECAN_ATTR_MSR_ACTIVATE_GAPS                                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 5302
#define RSSPECAN_ATTR_MSR_MINIMUM_GAP_SIZE                                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 5303
#define RSSPECAN_ATTR_MSR_GAP_CHANNEL_SPACING                                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 3064
#define RSSPECAN_ATTR_MSR_GAP_CHANNEL_BANDWIDTHS                                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 3065
#define RSSPECAN_ATTR_MSR_GAP_WEIGHTING_FILTER_STATE                                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 3066
#define RSSPECAN_ATTR_MSR_GAP_WEIGHTING_FILTER_ROLL_OFF_FACTOR                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 3067
#define RSSPECAN_ATTR_MSR_GAP_LIMIT_RELATIVE_STATE                                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 3068
#define RSSPECAN_ATTR_MSR_GAP_LIMIT_RELATIVE                                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 3231
#define RSSPECAN_ATTR_MSR_GAP_LIMIT_ABSOLUTE_STATE                                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 3069
#define RSSPECAN_ATTR_MSR_GAP_LIMIT_ABSOLUTE                                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 3232
#define RSSPECAN_ATTR_MSR_GAP_MODE                                                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 7610
#define RSSPECAN_ATTR_MSR_GAP_LIMIT_ACLR_RELATIVE                                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 7611
#define RSSPECAN_ATTR_MSR_GAP_LIMIT_ACLR_RELATIVE_STATE                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 7614
#define RSSPECAN_ATTR_MSR_GAP_LIMIT_CACLR_RELATIVE_STATE                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 7618
#define RSSPECAN_ATTR_MEAS_POW_ALT_CHANNEL_SPACING                                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 456
#define RSSPECAN_ATTR_MEAS_POW_ALT_CHANNEL_NAME                                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 2376
#define RSSPECAN_ATTR_MEAS_POW_ALT_CHANNEL_BANDWIDTH                                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 460
#define RSSPECAN_ATTR_POWER_ALT_CHANNEL_WEIGHTING_FILTER_STATE                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 4703
#define RSSPECAN_ATTR_POWER_ALT_CHANNEL_WEIGHTING_FILTER_ALPHA_VALUE                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 4704
#define RSSPECAN_ATTR_POWER_TRANSMISSION_CHANNEL_BANDWIDTH                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 9929
#define RSSPECAN_ATTR_MEAS_POW_CHANNEL_NAME                                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 2374
#define RSSPECAN_ATTR_MEAS_POW_CHANNEL_SPACING                                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 454
#define RSSPECAN_ATTR_POW_CHANNEL_BANDWIDTH                                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 4705
#define RSSPECAN_ATTR_POWER_CHANNEL_WEIGHTING_FILTER                                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 4706
#define RSSPECAN_ATTR_POWER_CHANNEL_WEIGHTING_FILTER_ALPHA                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 4707
#define RSSPECAN_ATTR_MEAS_ACP_ALT_LIMIT_REL_STATE                                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 479
#define RSSPECAN_ATTR_MEAS_ACP_ALT_LIMIT_ABS_STATE                                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 480
#define RSSPECAN_ATTR_MEAS_ACP_LIMIT_STATE                                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 476
#define RSSPECAN_ATTR_MEAS_ACP_LIMIT_REL_STATE                                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 477
#define RSSPECAN_ATTR_MEAS_ACP_LIMIT_ABS_STATE                                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 478
#define RSSPECAN_ATTR_ACP_USER_STD_SET                                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 469
#define RSSPECAN_ATTR_ACP_USER_STD_SAVE                                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 470
#define RSSPECAN_ATTR_ACP_USER_STD_DELETE                                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 471
#define RSSPECAN_ATTR_MEAS_ACP_HSP                                                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 472
#define RSSPECAN_ATTR_MEAS_ACP_PRESET_REF_LEVEL                                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 473
#define RSSPECAN_ATTR_MEAS_STAT_APD_STATE                                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 486
#define RSSPECAN_ATTR_MEAS_STAT_CCDF_STATE                                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 487
#define RSSPECAN_ATTR_SIGNAL_STATISTIC_NORMAL_DISTRIBUTION_ENABLED                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 11204
#define RSSPECAN_ATTR_MEAS_STAT_SAMPLES                                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 488
#define RSSPECAN_ATTR_MEAS_STAT_X_REF                                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 489
#define RSSPECAN_ATTR_MEAS_STAT_X_RANGE                                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 490
#define RSSPECAN_ATTR_MEAS_STAT_Y_UNIT                                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 491
#define RSSPECAN_ATTR_MEAS_STAT_Y_MIN                                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 492
#define RSSPECAN_ATTR_MEAS_STAT_Y_MAX                                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 493
#define RSSPECAN_ATTR_MEAS_STAT_ADJ                                                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 495
#define RSSPECAN_ATTR_MEAS_STAT_PRESET                                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 497
#define RSSPECAN_ATTR_MEAS_MDEPTH_STATE                                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 499
#define RSSPECAN_ATTR_MEAS_MDEPTH_RESULT                                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 500
#define RSSPECAN_ATTR_MEAS_MDEPTH_SEARCH                                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 501
#define RSSPECAN_ATTR_MEAS_TOI_STATE                                                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 502
#define RSSPECAN_ATTR_MEAS_TOI_RESULT                                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 503
#define RSSPECAN_ATTR_TOI_RESULT_MAXIMUM                                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 8619
#define RSSPECAN_ATTR_TOI_RESULT_MINIMUM                                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 8620
#define RSSPECAN_ATTR_MEAS_TOI_SEARCH                                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 504
#define RSSPECAN_ATTR_MEAS_HDIST_STATE                                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 506
#define RSSPECAN_ATTR_MEAS_HDIST_NOOFHARM                                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 507
#define RSSPECAN_ATTR_MEAS_HDIST_RBWAUTO                                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 508
#define RSSPECAN_ATTR_MEAS_HDIST_PRESET                                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 509
#define RSSPECAN_ATTR_MPOW_MIN                                                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 510
#define RSSPECAN_ATTR_MPOW_FTYPE                                                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 511
#define RSSPECAN_ATTR_SE_PEAK_SEARCH_MARGIN                                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 512
#define RSSPECAN_ATTR_SE_PEAK_SEARCH_PEAKS                                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 513
#define RSSPECAN_ATTR_SE_DETAILS                                                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 5601
#define RSSPECAN_ATTR_SE_MARK_ALL_PEAKS                                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 4708
#define RSSPECAN_ATTR_SE_LIST_RANG_COUNT                                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 530
#define RSSPECAN_ATTR_SE_PEAK_SEARCH                                                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 531
#define RSSPECAN_ATTR_SE_STORE_FILE                                                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 532
#define RSSPECAN_ATTR_SE_LIST_EVAL_STATE                                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 537
#define RSSPECAN_ATTR_SE_ADJUST_X_AXIS_TO_RANGE_DEFINITIONS                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 6950
#define RSSPECAN_ATTR_SE_LIST_RANG_DEL                                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 529
#define RSSPECAN_ATTR_SE_LIST_RANG_START                                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 514
#define RSSPECAN_ATTR_SE_LIST_RANG_STOP                                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 515
#define RSSPECAN_ATTR_SE_LIST_RANG_FILT_TYPE                                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 516
#define RSSPECAN_ATTR_SE_LIST_RANG_BAND                                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 517
#define RSSPECAN_ATTR_SE_LIST_RANG_BAND_VID                                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 518
#define RSSPECAN_ATTR_SE_LIST_RANG_SWE_TIME_AUTO                                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 519
#define RSSPECAN_ATTR_SE_LIST_RANG_SWE_TIME                                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 520
#define RSSPECAN_ATTR_SE_LIST_RANG_DET                                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 521
#define RSSPECAN_ATTR_SE_LIST_RANG_RLEV                                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 522
#define RSSPECAN_ATTR_SE_LIST_RANG_INP_ATT_AUTO                                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 523
#define RSSPECAN_ATTR_SE_LIST_RANG_INP_ATT                                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 524
#define RSSPECAN_ATTR_SE_LIST_RANG_POINTS                                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 525
#define RSSPECAN_ATTR_SE_LIST_RANG_BREAK                                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 526
#define RSSPECAN_ATTR_SE_LIST_RANG_TRAN                                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 527
#define RSSPECAN_ATTR_SE_LIST_RANG_INP_GAIN_STAT                                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 528
#define RSSPECAN_ATTR_SE_LIST_RANG_INP_GAIN_LEVEL                                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 2687
#define RSSPECAN_ATTR_SE_LIST_RANG_LIMIT_STATE                                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 536
#define RSSPECAN_ATTR_SE_LIST_RANG_LIMIT_START                                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 534
#define RSSPECAN_ATTR_SE_LIST_RANG_LIMIT_STOP                                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 535
#define RSSPECAN_ATTR_SPURIOUS_APPLICATION_MODE                                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 6642
#define RSSPECAN_ATTR_SE_CARRIER_REFERENCE_POWER_REFERENCE                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 6541
#define RSSPECAN_ATTR_SE_CARRIER_REFERENCE_CARRIER_LEVEL                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 6542
#define RSSPECAN_ATTR_SE_CARRIER_ADJUST_AUTO                                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 6543
#define RSSPECAN_ATTR_SE_CARRIER_SPUR_FREQUENCY_REFERENCE                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 7149
#define RSSPECAN_ATTR_SE_CARRIER_FREQUENCY                                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 7150
#define RSSPECAN_ATTR_SE_CARRIER_GUARD_INTERVAL_INCLUDED                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 7151
#define RSSPECAN_ATTR_SE_CARRIER_GUARD_INTERVAL                                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 7152
#define RSSPECAN_ATTR_SE_CARRIER_HARMONICS_IDENTIFY_ENABLED                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 9930
#define RSSPECAN_ATTR_SE_CARRIER_HARMONICS_TOLERANCE                                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 9931
#define RSSPECAN_ATTR_SE_CARRIER_HARMONICS_MAX_NUMBER                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 9932
#define RSSPECAN_ATTR_SE_TYPE_OF_SEARCH                                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 6544
#define RSSPECAN_ATTR_SE_PERFORMED_STEPS                                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 6545
#define RSSPECAN_ATTR_SE_MARK_RESIDUALS                                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 6546
#define RSSPECAN_ATTR_SE_REMOVE_RESIDUALS                                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 6547
#define RSSPECAN_ATTR_SE_TRANSFER_SEARCH_SETTINGS_TO_WIDE                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 6548
#define RSSPECAN_ATTR_SE_MATCHING_CONDITION                                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 11124
#define RSSPECAN_ATTR_SE_CARRIER_SEARCH_RANGE_TYPE                                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 7154
#define RSSPECAN_ATTR_SE_CARRIER_SEARCH_RANGE_CENTER                                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 7155
#define RSSPECAN_ATTR_SE_CARRIER_SEARCH_RANGE_SPAN                                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 7156
#define RSSPECAN_ATTR_SE_CARRIER_SEARCH_RANGE_START                                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 7157
#define RSSPECAN_ATTR_SE_CARRIER_SEARCH_RANGE_STOP                                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 7158
#define RSSPECAN_ATTR_SE_LISN_TYPE                                                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 7159
#define RSSPECAN_ATTR_SE_LISN_HIGH_PASS_FILTER                                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 7160
#define RSSPECAN_ATTR_SE_LISN_PHASE                                                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 7161
#define RSSPECAN_ATTR_FPLAN_ENABLED                                                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 7620
#define RSSPECAN_ATTR_FPLAN_TOLERANCE_FOR_IDENTIFICATION                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 9933
#define RSSPECAN_ATTR_FPLAN_CENTER_FREQUENCY                                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 7621
#define RSSPECAN_ATTR_FPLAN_SPAN                                                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 7622
#define RSSPECAN_ATTR_FPLAN_COMPONENT_DELETE                                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 7623
#define RSSPECAN_ATTR_FPLAN_COMPONENT_ADD                                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 9934
#define RSSPECAN_ATTR_FPLAN_SAVE                                                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 9935
#define RSSPECAN_ATTR_FPLAN_LOAD                                                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 9936
#define RSSPECAN_ATTR_FPLAN_PREDICTED_EXPORT                                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 9937
#define RSSPECAN_ATTR_FPLAN_IDENTIFIER                                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 7624
#define RSSPECAN_ATTR_FPLAN_COMPONENT_TYPE                                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 7625
#define RSSPECAN_ATTR_FPLAN_TRANSFER                                                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 7626
#define RSSPECAN_ATTR_FPLAN_INPUT_FREQUENCY                                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 7627
#define RSSPECAN_ATTR_FPLAN_MAXIMUM_HARMONICS                                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 7628
#define RSSPECAN_ATTR_SE_DIRECTED_SEARCH_LIMIT_OFFSET                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 6550
#define RSSPECAN_ATTR_SE_DIRECTED_MAXIMUM_FINAL_RBW                                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 6551
#define RSSPECAN_ATTR_SE_DIRECTED_NUMBER_FFT_AVERAGES                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 6552
#define RSSPECAN_ATTR_SE_DIRECTED_PEAK_EXCURSION                                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 6553
#define RSSPECAN_ATTR_SE_DIRECTED_REFERENCE_LEVEL                                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 6554
#define RSSPECAN_ATTR_SE_DIRECTED_DETECTOR                                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 6555
#define RSSPECAN_ATTR_SE_DIRECTED_RF_ATTENUATION                                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 6556
#define RSSPECAN_ATTR_SE_DIRECTED_PREAMPLIFIER_STATE                                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 6557
#define RSSPECAN_ATTR_SE_DIRECTED_PREAMPLIFIER                                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 6558
#define RSSPECAN_ATTR_SE_DIRECTED_SAVE_TABLE                                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 6560
#define RSSPECAN_ATTR_SE_DIRECTED_LOAD_TABLE                                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 6562
#define RSSPECAN_ATTR_SE_WIDE_CLEAR_RANGES                                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 6564
#define RSSPECAN_ATTR_SE_WIDE_SAVE_RANGES                                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 6568
#define RSSPECAN_ATTR_SE_WIDE_LOAD_RANGES                                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 6566
#define RSSPECAN_ATTR_SE_WIDE_SEARCH_NUMBER_OF_RANGES                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 7162
#define RSSPECAN_ATTR_SE_LIST_RANG_BAND_AUTO                                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 6569
#define RSSPECAN_ATTR_SE_LIST_RANGE_LIMIT_OFFSET                                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 6570
#define RSSPECAN_ATTR_SE_LIST_RANGE_MAXIMUM_FINAL_RBW                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 6571
#define RSSPECAN_ATTR_SE_LIST_RANGE_NUMBER_FFT_AVERAGES                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 6572
#define RSSPECAN_ATTR_SE_LIST_RANGE_PEAK_EXCURSION                                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 6573
#define RSSPECAN_ATTR_SE_LIST_RANGE_SIGNAL_NOISE_RATIO                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 6574
#define RSSPECAN_ATTR_SE_LIST_RANGE_THRESHOLD_START                                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 6575
#define RSSPECAN_ATTR_SE_LIST_RANGE_THRESHOLD_STOP                                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 6576
#define RSSPECAN_ATTR_SE_LIST_RANGE_INSERT_RANGE                                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 9938
#define RSSPECAN_ATTR_SE_LIST_RANGE_USE_SELECTION_FOR_ALL_RANGES                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 9939
#define RSSPECAN_ATTR_IQ_MODE                                                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 4709
#define RSSPECAN_ATTR_LOAD_IQ_DATA_ALL                                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 9940
#define RSSPECAN_ATTR_STORE_IQ_DATA_ALL                                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 9941
#define RSSPECAN_ATTR_IQ_DATA_ANALYZER_STATE                                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 2377
#define RSSPECAN_ATTR_IQ_DATA_FORMAT                                                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 538
#define RSSPECAN_ATTR_IQ_SAMPLE_RATE                                                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 539
#define RSSPECAN_ATTR_IQ_AUTOMATIC_ADJUSTMENT_STATE                                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 9942
#define RSSPECAN_ATTR_IQ_SWEEP_LENGTH                                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 9943
#define RSSPECAN_ATTR_IQ_MAX_BANDWIDTH_EXTENSION                                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 2878
#define RSSPECAN_ATTR_IQ_MAXIMUM_BANDWIDTH                                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 6648
#define RSSPECAN_ATTR_IQ_MAXIMUM_BANDWIDTH_MAX                                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 6649
#define RSSPECAN_ATTR_FORCE_IQ_BANDWIDTH_EXTENSION                                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 6494
#define RSSPECAN_ATTR_IQ_BANDWIDTH_MODE                                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 4711
#define RSSPECAN_ATTR_IQ_BANDWIDTH_RESOLUTION                                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 4712
#define RSSPECAN_ATTR_IQ_RECORD_LENGTH                                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 2558
#define RSSPECAN_ATTR_IQ_DATA_FILE                                                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 4713
#define RSSPECAN_ATTR_IQ_DATA_STATE                                                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 541
#define RSSPECAN_ATTR_IQ_DATA_AVER_STATE                                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 544
#define RSSPECAN_ATTR_IQ_DATA_AVER_COUNT                                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 545
#define RSSPECAN_ATTR_IQ_DATA_GATE_STATE                                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 546
#define RSSPECAN_ATTR_IQ_DATA_GATE_TYPE                                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 547
#define RSSPECAN_ATTR_IQ_DATA_GATE_LENGTH                                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 548
#define RSSPECAN_ATTR_IQ_DATA_GATE_GAP                                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 549
#define RSSPECAN_ATTR_IQ_DATA_GATE_NUMBER_OF_PERIODS                                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 550
#define RSSPECAN_ATTR_IQ_DATA_DISPLAY_TYPE                                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 551
#define RSSPECAN_ATTR_IQ_DATA_FILTER_BANDWIDTH                                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 552
#define RSSPECAN_ATTR_BB_SWAP_IQ                                                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 3076
#define RSSPECAN_ATTR_LOAD_IQ_DATA                                                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 2559
#define RSSPECAN_ATTR_SAVE_IQ_DATA                                                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 2560
#define RSSPECAN_ATTR_IQ_FFT_ALGORITHM                                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 4714
#define RSSPECAN_ATTR_IQ_FFT_LENGTH                                                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 4715
#define RSSPECAN_ATTR_IQ_FFT_WIN_LENGTH                                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 4716
#define RSSPECAN_ATTR_IQ_FFT_WIN_OVERLAP                                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 4717
#define RSSPECAN_ATTR_IQ_FFT_WINDOW_TYPE                                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 4718
#define RSSPECAN_ATTR_IQ_APC_STATE                                                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 4719
#define RSSPECAN_ATTR_IQ_APC_FACTOR_A                                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 4720
#define RSSPECAN_ATTR_IQ_APC_FACTOR_B                                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 4721
#define RSSPECAN_ATTR_IQ_APC_RESULT                                                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 4722
#define RSSPECAN_ATTR_IQ_DATA_EXPORT_FILE_DESCRIPTION                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 2561
#define RSSPECAN_ATTR_QUERY_TPIS_TIME_OFFSET                                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 2707
#define RSSPECAN_ATTR_IQ_OSCILLOSCOPE_COUPLING                                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 7629
#define RSSPECAN_ATTR_B2000_STATE                                                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 5602
#define RSSPECAN_ATTR_B2000_OSCILLOSCOPE_DISPLAY_UPDATE                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 5603
#define RSSPECAN_ATTR_OSCILLOSCOPE_TCPIP_ADDRESS                                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 5604
#define RSSPECAN_ATTR_OSCILLOSCOPE_ID                                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 5605
#define RSSPECAN_ATTR_OSCILLOSCOPE_LAN_CONNECTION_STATE                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 5606
#define RSSPECAN_ATTR_OSCILLOSCOPE_SUPPORTED_INSTRUMENT                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 5607
#define RSSPECAN_ATTR_OSCILLOSCOPE_SUPPORTED_FIRMWARE                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 5608
#define RSSPECAN_ATTR_OSCILLOSCOPE_ALIGNMENT                                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 5609
#define RSSPECAN_ATTR_OSCILLOSCOPE_SELF_ALIGNMENT_ENABLED                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 7108
#define RSSPECAN_ATTR_OSCILLOSCOPE_ALIGNMENT_DATE                                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 5610
#define RSSPECAN_ATTR_OSCILLOSCOPE_SAMPLE_RATE                                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 6651
#define RSSPECAN_ATTR_OSCILLOSCOPE_POWER_SPLITTER_MODE                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 6652
#define RSSPECAN_ATTR_ESP_RESTORE                                                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 553
#define RSSPECAN_ATTR_ESP_MODE                                                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 554
#define RSSPECAN_ATTR_ESP_PEAK_SEARCH                                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 555
#define RSSPECAN_ATTR_ESP_MAN_VAL                                                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 556
#define RSSPECAN_ATTR_SEM_PEAK_SEARCH_MARGIN                                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 562
#define RSSPECAN_ATTR_SEM_PEAK_SEARCH_PEAKS                                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 563
#define RSSPECAN_ATTR_SEM_LIST_EVAL_STATE                                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 564
#define RSSPECAN_ATTR_SEM_MARK_ALL_PEAKS                                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 565
#define RSSPECAN_ATTR_SEM_POWER_REF_TYPE                                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 566
#define RSSPECAN_ATTR_SEM_BANDWIDTH                                                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 567
#define RSSPECAN_ATTR_SEM_RRC_FILTER_STATE                                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 568
#define RSSPECAN_ATTR_SEM_RRC_FILTER_ALPHA_VALUE                                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 569
#define RSSPECAN_ATTR_SEM_REF_RANGE_POSITION                                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 570
#define RSSPECAN_ATTR_SEM_FAST_SEM                                                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 571
#define RSSPECAN_ATTR_SEM_LIST_FILE_EXPORT                                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 575
#define RSSPECAN_ATTR_SEM_RANGE_COUNT                                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 593
#define RSSPECAN_ATTR_SEM_POWER_CLASS_COUNT                                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 600
#define RSSPECAN_ATTR_SPECTRUM_EMISSION_MASK_SUB_BLOCK_COUNT                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 5611
#define RSSPECAN_ATTR_SPECTRUM_EMISSION_MASK_SUB_BLOCK_CENTER_FREQUENCY                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 5612
#define RSSPECAN_ATTR_SPECTRUM_EMMISION_MASK_SYMMETRICAL_SETUP                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 7630
#define RSSPECAN_ATTR_SEM_INSERT_NEW_RANGE                                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 594
#define RSSPECAN_ATTR_SEM_DELETE_RANGE                                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 595
#define RSSPECAN_ATTR_SEM_RANGE_FREQ_START                                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 578
#define RSSPECAN_ATTR_SEM_RANGE_FREQ_STOP                                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 579
#define RSSPECAN_ATTR_SEM_RANGE_FILTER_TYPE                                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 585
#define RSSPECAN_ATTR_SEM_RANGE_BANDWIDTH                                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 576
#define RSSPECAN_ATTR_SEM_RANGE_VBW                                                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 577
#define RSSPECAN_ATTR_SEM_RANGE_SWEEP_TIME_AUTO                                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 591
#define RSSPECAN_ATTR_SEM_RANGE_SWEEP_TIME                                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 590
#define RSSPECAN_ATTR_SEM_RANGE_REF_LEVEL                                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 589
#define RSSPECAN_ATTR_SEM_RANGE_INPUT_ATTENUATION_AUTO                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 587
#define RSSPECAN_ATTR_SEM_RANGE_INPUT_ATTENUATION                                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 586
#define RSSPECAN_ATTR_SEM_RANGE_PREAMPLIFIER                                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 588
#define RSSPECAN_ATTR_SEM_RANGE_PREAMPLIFIER_LEVEL                                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 2688
#define RSSPECAN_ATTR_SEM_RANGE_TRANSDUCER                                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 592
#define RSSPECAN_ATTR_SPECTRUM_EMISSION_MASK_RANGE_MINIMUM_SWEEP_POINTS                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 5613
#define RSSPECAN_ATTR_SPECTRUM_EMISSION_MASK_RANGE_MULTI_LIMIT_CALC                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 5614
#define RSSPECAN_ATTR_SEM_RANGE_LIMIT_CHECK_TYPE                                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 584
#define RSSPECAN_ATTR_SEM_RANGE_START_FREQ_ABS_LIMIT                                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 580
#define RSSPECAN_ATTR_SEM_RANGE_STOP_FREQ_ABS_LIMIT                                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 581
#define RSSPECAN_ATTR_SEM_RANGE_START_FREQ_REL_LIMIT                                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 582
#define RSSPECAN_ATTR_SEM_RANGE_START_FREQ_REL_LIMIT_FUNCTION                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 2906
#define RSSPECAN_ATTR_SEM_RANGE_START_FREQ_REL_LIMIT_MAXIMUM                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 2907
#define RSSPECAN_ATTR_SEM_RANGE_STOP_FREQ_REL_LIMIT                                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 583
#define RSSPECAN_ATTR_SEM_RANGE_STOP_FREQ_REL_LIMIT_FUNCTION                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 2908
#define RSSPECAN_ATTR_SEM_RANGE_STOP_FREQ_REL_LIMIT_MAXIMUM                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 2909
#define RSSPECAN_ATTR_SEM_POWER_CLASS_USED                                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 596
#define RSSPECAN_ATTR_SEM_POWER_CLASS_LIMIT_STATE                                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 599
#define RSSPECAN_ATTR_SEM_POWER_CLASS_LOWER_LEVEL_LIMIT                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 597
#define RSSPECAN_ATTR_SEM_POWER_CLASS_UPPER_LEVEL_LIMIT                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 598
#define RSSPECAN_ATTR_SEM_MSR_BAND_CATEGORY                                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 2901
#define RSSPECAN_ATTR_SPECTRUM_EMISSION_MASK_MSR_BASE_STATION_CLASS                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 5615
#define RSSPECAN_ATTR_SPECTRUM_EMISSION_MASK_MSR_BASE_STATION_MAXIMUM_OUTPUT_POWER                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 5616
#define RSSPECAN_ATTR_SPECTRUM_EMISSION_MASK_MSR_BANDS                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 5617
#define RSSPECAN_ATTR_SEM_MSR_RF_BANDWIDTH                                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 2902
#define RSSPECAN_ATTR_SEM_MSR_GSM_PRESENT                                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 2903
#define RSSPECAN_ATTR_SPECTRUM_EMISSION_MASK_MSR_POWER_GSM_CARRIER                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 5618
#define RSSPECAN_ATTR_SEM_MSR_LTE_PRESENT                                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 2904
#define RSSPECAN_ATTR_SEM_MSR_APPLY                                                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 2905
#define RSSPECAN_ATTR_SEM_PRESET_STANDARD                                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 572
#define RSSPECAN_ATTR_SEM_RESTORE_STANDARD                                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 573
#define RSSPECAN_ATTR_SEM_STORE_PRESET                                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 574
#define RSSPECAN_ATTR_NPR_ENABLED                                                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 6951
#define RSSPECAN_ATTR_NPR_CHANNEL_BANDWIDTH                                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 6954
#define RSSPECAN_ATTR_NPR_NOTCH_COUNT                                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 6955
#define RSSPECAN_ATTR_NPR_MODE                                                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 9944
#define RSSPECAN_ATTR_NPR_INTEGRATION_BANDWIDTH_AUTO_ENABLED                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 6956
#define RSSPECAN_ATTR_NPR_INTEGRATION_BANDWIDTH                                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 6957
#define RSSPECAN_ATTR_NPR_FREQUENCY_OFFSET                                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 6958
#define RSSPECAN_ATTR_NPR_NOTCH_FREQUENCY_OFFSET                                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 6959
#define RSSPECAN_ATTR_NPR_NOTCH_BANDWIDTH                                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 6960
#define RSSPECAN_ATTR_NPR_NOTCH_BANDWIDTH_RELATIVE                                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 6961
#define RSSPECAN_ATTR_NPR_GENERATOR_ENABLED                                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 7631
#define RSSPECAN_ATTR_NPR_GENERATOR_COUPLING_ENABLED                                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 7632
#define RSSPECAN_ATTR_NPR_GENERATOR_FREQUENCY_SOURCE                                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 7633
#define RSSPECAN_ATTR_NPR_GENERATOR_FREQUENCY_CENTER                                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 7634
#define RSSPECAN_ATTR_NPR_GENERATOR_FREQUENCY_OFFSET                                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 7635
#define RSSPECAN_ATTR_NPR_GENERATOR_FREQUENCY_DENOMINATOR                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 7636
#define RSSPECAN_ATTR_NPR_GENERATOR_FREQUENCY_NUMERATOR                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 7637
#define RSSPECAN_ATTR_NPR_GENERATOR_SELECTED_WAVEFORM                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 7638
#define RSSPECAN_ATTR_NPR_GENERATOR_STANDARD                                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 7639
#define RSSPECAN_ATTR_NPR_GENERATOR_STATE                                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 7640
#define RSSPECAN_ATTR_NPR_GENERATOR_EXTERNAL_STATE                                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 7641
#define RSSPECAN_ATTR_NPR_GENERATOR_CENTER_FREQUENCY_STATE                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 7642
#define RSSPECAN_ATTR_NPR_GENERATOR_CLOCK_FREQUENCY                                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 7643
#define RSSPECAN_ATTR_NPR_GENERATOR_NUMBER_OF_NOTCHES                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 7644
#define RSSPECAN_ATTR_NPR_GENERATOR_POWER_LEVEL                                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 7645
#define RSSPECAN_ATTR_NPR_GENERATOR_POWER_LEVEL_STATE                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 7646
#define RSSPECAN_ATTR_NPR_GENERATOR_POWER_LEVEL_OFFSET                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 7647
#define RSSPECAN_ATTR_NPR_GENERATOR_POWER_LEVEL_OFFSET_STATE                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 7648
#define RSSPECAN_ATTR_NPR_GENERATOR_RF_OUTPUT_ENABLED                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 7649
#define RSSPECAN_ATTR_NPR_GENERATOR_RF_OUTPUT_STATE                                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 7650
#define RSSPECAN_ATTR_NPR_GENERATOR_NOTCH_FILTER_STATE                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 7651
#define RSSPECAN_ATTR_NPR_GENERATOR_UPLOAD_ALL_NOTCH_SETTINGS                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 7652
#define RSSPECAN_ATTR_NPR_GENERATOR_APPLY_ALL_SETTINGS                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 7653
#define RSSPECAN_ATTR_NPR_GENERATOR_NOTCH_FILTER_ENABLED                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 7654
#define RSSPECAN_ATTR_NPR_GENERATOR_TARGET_PATH_BB                                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 7655
#define RSSPECAN_ATTR_NPR_GENERATOR_TARGET_PATH_RF                                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 7656
#define RSSPECAN_ATTR_NPR_GENERATOR_SETTINGS                                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 7657
#define RSSPECAN_ATTR_NPR_GENERATOR_NOTCH_SETTINGS                                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 7658
#define RSSPECAN_ATTR_NPR_GENERATOR_CONNECTION_STATE                                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 7659
#define RSSPECAN_ATTR_NPR_GENERATOR_CONTROL_IP_ADDRESS                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 7660
#define RSSPECAN_ATTR_NPR_GENERATOR_CONTROL_CONNECTION_ENABLED                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 7661
#define RSSPECAN_ATTR_NPR_GENERATOR_NOTCH_ENABLED                                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 7662
#define RSSPECAN_ATTR_NPR_GENERATOR_ABSOLUTE_NOTCH_BANDWIDTH                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 7663
#define RSSPECAN_ATTR_NPR_GENERATOR_NOTCH_FREQUENCY_OFFSET                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 7664
#define RSSPECAN_ATTR_NPR_GENERATOR_NOTCH_STATE                                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 7665
#define RSSPECAN_ATTR_PROBE_PART_NUMBER                                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 3270
#define RSSPECAN_ATTR_PROBE_SERIAL_NUMBER                                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 3271
#define RSSPECAN_ATTR_PROBE_MICRO_BUTTON_ACTION                                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 2765
#define RSSPECAN_ATTR_PROBE_NAME                                                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 2767
#define RSSPECAN_ATTR_PROBE_CONNECTION_STATE                                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 2766
#define RSSPECAN_ATTR_PROBE_TYPE                                                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 3073
#define RSSPECAN_ATTR_PROBE_ATTENUATION_RATIO                                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 6962
#define RSSPECAN_ATTR_PROBE_MODE                                                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 6963
#define RSSPECAN_ATTR_PROBE_DM_MODE_OFFSET                                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 6964
#define RSSPECAN_ATTR_BB_PROBE_COMMON_MODE_OFFSET                                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 5307
#define RSSPECAN_ATTR_PROBE_P_MODE_OFFSET                                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 6965
#define RSSPECAN_ATTR_PROBE_N_MODE_OFFSET                                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 6966
#define RSSPECAN_ATTR_MSRA_MODE                                                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 2910
#define RSSPECAN_ATTR_MSRA_REFRESH                                                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 2911
#define RSSPECAN_ATTR_MSRA_CAPTURE_OFFSET                                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 2912
#define RSSPECAN_ATTR_MSRA_SHOW_ANALYSIS_LINE                                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 3000
#define RSSPECAN_ATTR_MSRA_ANALYSIS_LINE_POSITION                                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 3001
#define RSSPECAN_ATTR_MSRA_DATA_EVALUATION                                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 3095
#define RSSPECAN_ATTR_RTMS_MODE                                                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 4953
#define RSSPECAN_ATTR_RTMS_CAPTURE_OFFSET                                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 4954
#define RSSPECAN_ATTR_RTMS_SHOW_ANALYSIS_LINE                                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 4955
#define RSSPECAN_ATTR_RTMS_ANALYSIS_LINE_POSITION                                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 4956
#define RSSPECAN_ATTR_EXT_GEN_LEVEL_OFFSET                                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 3272
#define RSSPECAN_ATTR_CORR_EGA_INP                                                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 980
#define RSSPECAN_ATTR_EXTERNAL_GAIN_PA_CORRECTION_ENABLED                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 6967
#define RSSPECAN_ATTR_TGEN_STATE                                                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 987
#define RSSPECAN_ATTR_EXT_GEN_STATE                                                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 971
#define RSSPECAN_ATTR_EXT_GEN_OUTPUT_FREQUENCY                                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 2563
#define RSSPECAN_ATTR_EXT_GEN_OUTPUT_FREQUENCY_COUPLING                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 2564
#define RSSPECAN_ATTR_EXT_GEN_DENOMINATOR                                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 972
#define RSSPECAN_ATTR_EXT_GEN_NUMERATOR                                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 973
#define RSSPECAN_ATTR_EXT_GEN_FREQ_SWEEP_STATE                                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 975
#define RSSPECAN_ATTR_EXT_GEN_FREQ_OFFSET                                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 974
#define RSSPECAN_ATTR_EXT_GEN_OUTPUT_POWER                                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 976
#define RSSPECAN_ATTR_EXT_GEN_INTERFACE_TYPE                                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 3273
#define RSSPECAN_ATTR_EXT_GEN_IEEE_ADDR                                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 979
#define RSSPECAN_ATTR_EXT_GEN_INTF_TYPE                                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 968
#define RSSPECAN_ATTR_EXT_GEN_TYPE                                                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 978
#define RSSPECAN_ATTR_EXT_GEN_ROSC_SOURCE                                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 977
#define RSSPECAN_ATTR_TGEN_NORMALIZATION_STATE                                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 963
#define RSSPECAN_ATTR_TGEN_METHOD                                                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 964
#define RSSPECAN_ATTR_TGEN_ACQUIRE                                                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 965
#define RSSPECAN_ATTR_TGEN_RECALL                                                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 966
#define RSSPECAN_ATTR_TGEN_DM_STATE                                                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 981
#define RSSPECAN_ATTR_TGEN_FM_STATE                                                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 982
#define RSSPECAN_ATTR_TGEN_FM_DEVIATION                                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 983
#define RSSPECAN_ATTR_TGEN_FREQUENCY_OFFSET                                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 984
#define RSSPECAN_ATTR_TGEN_OUTPUT_LEVEL                                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 985
#define RSSPECAN_ATTR_TGEN_LEVEL_OFFSET                                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 986
#define RSSPECAN_ATTR_TGEN_AM_STATE                                                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 988
#define RSSPECAN_ATTR_TGEN_POW_MODE                                                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 967
#define RSSPECAN_ATTR_TGEN_POW_START                                                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 969
#define RSSPECAN_ATTR_TGEN_POW_STOP                                                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 970
#define RSSPECAN_ATTR_BB_INPUT_SELECTION                                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 996
#define RSSPECAN_ATTR_BB_SIGNAL_CALIBRATION_TYPE                                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 3002
#define RSSPECAN_ATTR_BB_INPUT_BALANCED                                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 997
#define RSSPECAN_ATTR_BB_INPUT_SIGNAL_PATH                                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 999
#define RSSPECAN_ATTR_BB_FULLSCALE_MODE                                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 3074
#define RSSPECAN_ATTR_BB_FULLSCALE_LEVEL                                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 3075
#define RSSPECAN_ATTR_FFT_FORM                                                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 1007
#define RSSPECAN_ATTR_BB_HIGH_ACCURACY_TIMING                                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 5304
#define RSSPECAN_ATTR_BB_I_DC_OFFSET                                                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 5305
#define RSSPECAN_ATTR_BB_Q_DC_OFFSET                                                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 5306
#define RSSPECAN_ATTR_BASEBAND_INPUT_IMPEDANCE                                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 6983
#define RSSPECAN_ATTR_BASEBAND_INPUT_IMPEDANCE_PAD_TYPE                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 6984
#define RSSPECAN_ATTR_BASEBAND_INPUT_ATTENUATION_IQ_INTERFACE                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 7424
#define RSSPECAN_ATTR_BASEBAND_YIG_FILTER_ENABLED                                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 7425
#define RSSPECAN_ATTR_DIGITAL_INPUT_RANGE                                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 1022
#define RSSPECAN_ATTR_DIGITAL_INPUT_RANGE_AUTO                                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 1023
#define RSSPECAN_ATTR_DIGITAL_INPUT_SRATE                                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 1024
#define RSSPECAN_ATTR_DIGITAL_INPUT_SAMPLE_RATE_AUTO                                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 1025
#define RSSPECAN_ATTR_QUERY_DIGITAL_INPUT_CONF                                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 1026
#define RSSPECAN_ATTR_QUERY_DIGITAL_OUTPUT_CONF                                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 1027
#define RSSPECAN_ATTR_DIGITAL_BASEBAND_OUTPUT_STATE                                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 1028
#define RSSPECAN_ATTR_DIGITAL_BASEBAND_INPUT_TRIGGER_LEVEL                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 1031
#define RSSPECAN_ATTR_DIGITAL_BASEBAND_INPUT_UNIT                                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 2378
#define RSSPECAN_ATTR_DIGITAL_INPUT_COUPLING                                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 2379
#define RSSPECAN_ATTR_DIGITAL_DECIMATION_FILTER                                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 4723
#define RSSPECAN_ATTR_REALTIME_MODE                                                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 2448
#define RSSPECAN_ATTR_SPEM_WINDOW                                                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 2393
#define RSSPECAN_ATTR_REALTIME_MEASUREMENT                                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 3159
#define RSSPECAN_ATTR_DWELL_TIME_AUTO                                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 6577
#define RSSPECAN_ATTR_DWELL_TIME                                                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 6578
#define RSSPECAN_ATTR_SPEM_STATE                                                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 1341
#define RSSPECAN_ATTR_SPEM_SIZE                                                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 1343
#define RSSPECAN_ATTR_SPEM_SHAPE                                                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 2394
#define RSSPECAN_ATTR_SPEM_HDEP                                                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 1344
#define RSSPECAN_ATTR_SPEM_TIME_STAMP_STATE                                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 1345
#define RSSPECAN_ATTR_SPEM_FRAME_COUNT                                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 1346
#define RSSPECAN_ATTR_SPEM_FRAME_SELECT                                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 1347
#define RSSPECAN_ATTR_SPEM_CONTINUOUS                                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 1348
#define RSSPECAN_ATTR_SPEM_CLEAR                                                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 1349
#define RSSPECAN_ATTR_SPEM_FFT_WINDOW                                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 2570
#define RSSPECAN_ATTR_FILE_ITEMS_SAVE_RECAL_SPEM                                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 1350
#define RSSPECAN_ATTR_SPEM_CONTINUOUS_TRIGGER                                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 2571
#define RSSPECAN_ATTR_SPEM_COLOR                                                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 1342
#define RSSPECAN_ATTR_SPEM_COLOR_STYLE                                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 2395
#define RSSPECAN_ATTR_SPEM_COLOR_LOW                                                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 2396
#define RSSPECAN_ATTR_SPEM_COLOR_UPPER                                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 2397
#define RSSPECAN_ATTR_SPEM_COLOR_DEFAULT                                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 2398
#define RSSPECAN_ATTR_SPEM_MARK_FRAME_NUMBER                                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 1351
#define RSSPECAN_ATTR_SPEM_MARKER_SEARCH_AREA                                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 1352
#define RSSPECAN_ATTR_SPEM_DELT_FRAME_NUMBER                                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 1355
#define RSSPECAN_ATTR_SPEM_DELT_SEARCH_AREA                                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 1356
#define RSSPECAN_ATTR_MARKER_SEARCH_Y_TRIGGER                                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 2572
#define RSSPECAN_ATTR_PSPEM_STATE                                                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 2399
#define RSSPECAN_ATTR_PSPEM_DURATION                                                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 2400
#define RSSPECAN_ATTR_PSPEM_GRAN                                                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 2401
#define RSSPECAN_ATTR_PSPEM_SHAPE                                                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 2402
#define RSSPECAN_ATTR_PSPEM_TRUN                                                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 2403
#define RSSPECAN_ATTR_PSPEM_COLOR_STYLE                                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 2404
#define RSSPECAN_ATTR_PSPEM_COLOR_LOWER                                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 2406
#define RSSPECAN_ATTR_PSPEM_COLOR_UPPER                                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 2407
#define RSSPECAN_ATTR_PSPEM_COLOR_DEFAULT                                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 2405
#define RSSPECAN_ATTR_PSPEM_TRACE_STYLE                                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 2408
#define RSSPECAN_ATTR_PSPEM_MAXH_TRACE                                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 2409
#define RSSPECAN_ATTR_PSPEM_MAXH_INTENS                                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 2410
#define RSSPECAN_ATTR_PSPEM_MAXH_RESET                                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 2411
#define RSSPECAN_ATTR_PSPEM_MARKER_POSITION                                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 2596
#define RSSPECAN_ATTR_QUERY_PSPEM_MARKER_Z_AXIS                                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 2597
#define RSSPECAN_ATTR_PSPEM_DELTA_MARKER_POSITION                                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 2598
#define RSSPECAN_ATTR_QUERY_PSPEM_DELTA_Z_AXIS                                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 2412
#define RSSPECAN_ATTR_RESULT_DISPLAY                                                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 2415
#define RSSPECAN_ATTR_SPEM_MASK_TRIGGER_SOURCE                                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 2573
#define RSSPECAN_ATTR_SPEM_MASK_TRIGGER_NAME                                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 2416
#define RSSPECAN_ATTR_SPEM_MASK_TRIGGER_COMMENT                                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 2417
#define RSSPECAN_ATTR_SPEM_MASK_TRIGGER_DIRECTORY                                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 2599
#define RSSPECAN_ATTR_SPEM_MASK_TRIGGER_DELETE                                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 2418
#define RSSPECAN_ATTR_SPEM_MASK_TRIGG_CONDITION                                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 2419
#define RSSPECAN_ATTR_SPEM_MASK_TRIGGER_MODE                                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 2420
#define RSSPECAN_ATTR_SPEM_MASK_UPP_AUTO                                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 2421
#define RSSPECAN_ATTR_SPEM_MASK_SHIFT_LOW_X                                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 2422
#define RSSPECAN_ATTR_SPEM_MASK_SHIFT_LOW_Y                                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 2423
#define RSSPECAN_ATTR_SPEM_MASK_SHIFT_UPP_X                                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 2424
#define RSSPECAN_ATTR_SPEM_MASK_SHIFT_UPP_Y                                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 2425
#define RSSPECAN_ATTR_SPECTROGRAM_MASK_SPAN                                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 4681
#define RSSPECAN_ATTR_SPEM_STORE_TO_FILE                                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 2574
#define RSSPECAN_ATTR_SPEM_TIME_DOMAIN_TRIGGER                                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 2575
#define RSSPECAN_ATTR_SPEM_TIME_DOMAIN_TRIGGER_LEVEL                                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 2576
#define RSSPECAN_ATTR_MCGD_MODE                                                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 3287
#define RSSPECAN_ATTR_MCGD_VERTICAL_SCALE_MIN                                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 3288
#define RSSPECAN_ATTR_MCGD_VERTICAL_SCALE_MAX                                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 3289
#define RSSPECAN_ATTR_MCGD_RESULT_PHASE_UNIT                                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 3291
#define RSSPECAN_ATTR_MCGD_CARRIER_COUNT                                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 3292
#define RSSPECAN_ATTR_MCGD_CARRIER_SPACING                                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 3293
#define RSSPECAN_ATTR_MCGD_MEASUREMENT_TIME_AUTO                                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 3294
#define RSSPECAN_ATTR_MCGD_MEASUREMENT_TIME                                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 3295
#define RSSPECAN_ATTR_MCGD_MODE_ABS_REL                                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 3296
#define RSSPECAN_ATTR_MCGD_CARRIER_OFFSET_MODE                                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 3297
#define RSSPECAN_ATTR_MCGD_CARRIER_REFERENCE_TYPE                                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 6439
#define RSSPECAN_ATTR_MCGD_CARRIER_REFERENCE_FREQUENCY                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 6440
#define RSSPECAN_ATTR_MCGD_CARRIER_OFFSET                                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 3298
#define RSSPECAN_ATTR_MCGD_MODE_ESTIMATION                                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 8621
#define RSSPECAN_ATTR_MCGD_MAX_CLOCK_OFFSET                                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 8622
#define RSSPECAN_ATTR_MCGD_LARGE_OFFSET_COMPENSATION_ENABLED                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 8623
#define RSSPECAN_ATTR_MCGD_CARRIER_DECIMATION_FACTOR                                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 8624
#define RSSPECAN_ATTR_MCGD_MAXIUMUM_RELATIVE_VELOCITY                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 9945
#define RSSPECAN_ATTR_MCGD_TRANSMISSION_TYPE                                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 9946
#define RSSPECAN_ATTR_MCGD_ADDITIONAL_TOLERANCE                                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 9947
#define RSSPECAN_ATTR_MCGD_CLOCK_OFFSET                                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 8625
#define RSSPECAN_ATTR_MCGD_RAW_PHASE_ENABLED                                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 10817
#define RSSPECAN_ATTR_MCGD_SUBSPAN_MEASUREMENT_STATE                                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 9948
#define RSSPECAN_ATTR_MCGD_SUBSPAN_MODE                                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 9949
#define RSSPECAN_ATTR_MCGD_SUBSPAN_FREQUENCY                                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 9950
#define RSSPECAN_ATTR_MCGD_SUBSPAN_NUMBER_OF_SUBSPANS                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 9951
#define RSSPECAN_ATTR_MCGD_SUBSPAN_NUMBER_OF_CARRIERS                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 9952
#define RSSPECAN_ATTR_MCGD_SUBSPAN_OVERLAP_FREQUENCY                                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 9953
#define RSSPECAN_ATTR_MCGD_SUBSPAN_OVERLAP_CARRIER_COUNT                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 9954
#define RSSPECAN_ATTR_GENERATOR_CONNECTION_STATUS                                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 7666
#define RSSPECAN_ATTR_GENERATOR_IP_ADDRESS                                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 7667
#define RSSPECAN_ATTR_CONNECT_TO_GENERATOR                                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 7668
#define RSSPECAN_ATTR_GENERATOR_RF_OUTPUT                                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 7669
#define RSSPECAN_ATTR_GENERATOR_SETTLING_DELAY                                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 9955
#define RSSPECAN_ATTR_MCGD_CONNECTION_STATUS                                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 7670
#define RSSPECAN_ATTR_CARRIER_COUNT_TO_GENERATOR_ENABLED                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 7671
#define RSSPECAN_ATTR_CENTER_FREQ_TO_GENERATOR_ENABLED                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 7672
#define RSSPECAN_ATTR_SPACING_TO_GENERATOR_ENABLED                                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 7673
#define RSSPECAN_ATTR_GENERATOR_CONTROL_ENABLED                                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 7674
#define RSSPECAN_ATTR_GENERATOR_REFERENCE_LEVEL_ENABLED                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 7675
#define RSSPECAN_ATTR_GENERATOR_LEVEL_CONTROL_ENABLED                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 7676
#define RSSPECAN_ATTR_GENERATOR_LEVEL                                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 7677
#define RSSPECAN_ATTR_GENERATOR_DUT_GAIN                                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 7678
#define RSSPECAN_ATTR_GENERATOR_DUT_LIMIT_ENABLED                                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 7679
#define RSSPECAN_ATTR_GENERATOR_DUT_LIMIT                                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 7680
#define RSSPECAN_ATTR_GENERATOR_SETTINGS_OVERALL_STATUS                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 7681
#define RSSPECAN_ATTR_GENERATOR_UPDATE_SETTINGS                                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 7682
#define RSSPECAN_ATTR_MCGD_SETTINGS                                                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 7683
#define RSSPECAN_ATTR_AFTM_ANALYZER_FREQUENCY                                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 7684
#define RSSPECAN_ATTR_AFTM_GENERATOR_FREQUENCY                                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 7685
#define RSSPECAN_ATTR_AFTM_FREQUENCY_OFFSET                                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 7686
#define RSSPECAN_ATTR_AFTM_DENOMINATOR                                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 7687
#define RSSPECAN_ATTR_AFTM_NUMERATOR                                                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 7688
#define RSSPECAN_ATTR_AFTM_OVERALL_STATUS                                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 7689
#define RSSPECAN_ATTR_AFTM_ENABLED                                                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 7690
#define RSSPECAN_ATTR_MCGD_CARRIER_TABLE_ENABLED                                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 9958
#define RSSPECAN_ATTR_MCGD_CARRIER_TABLE_EDIT_CARRIER_ENABLED                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 9961
#define RSSPECAN_ATTR_MCGD_CARRIER_TABLE_EDIT_APPLY_SETTINGS                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 9962
#define RSSPECAN_ATTR_MCGD_CARRIER_TABLE_EDIT_SELECT_START                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 9963
#define RSSPECAN_ATTR_MCGD_CARRIER_TABLE_EDIT_SELECT_STOP                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 9964
#define RSSPECAN_ATTR_MCGD_CARRIER_TABLE_EDIT_THRESHOLD                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 9965
#define RSSPECAN_ATTR_MCGD_INITIATE_CALIBRATION                                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 3299
#define RSSPECAN_ATTR_MCGD_CALIBRATION_STATUS                                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 3300
#define RSSPECAN_ATTR_MCGD_STORE_CALIBRATION_DATA                                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 3301
#define RSSPECAN_ATTR_MCGD_LOAD_CALIBRATION_DATA                                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 3303
#define RSSPECAN_ATTR_MCGD_STORE_MEASUREMENT_RESULTS                                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 3305
#define RSSPECAN_ATTR_MCGD_BANDWIDTH                                                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 3307
#define RSSPECAN_ATTR_MCGD_RECORD_LENGTH                                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 3308
#define RSSPECAN_ATTR_MCGD_SAMPLE_RATE                                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 3309
#define RSSPECAN_ATTR_MCGD_TRACE_POINTS                                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 3310
#define RSSPECAN_ATTR_MCGD_DISP_TRACE_SMO                                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 4958
#define RSSPECAN_ATTR_MCGD_DISP_TRACE_SMO_APER                                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 4959
#define RSSPECAN_ATTR_AMPLIFIER_MODE                                                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 5308
#define RSSPECAN_ATTR_AMPLIFIER_WAVEFORM_FILE                                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 6653
#define RSSPECAN_ATTR_AMPLIFIER_SAMPLE_RATE                                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 5309
#define RSSPECAN_ATTR_AMPLIFIER_SAMPLE_LENGTH                                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 5310
#define RSSPECAN_ATTR_AMPLIFIER_CREST_FACTOR                                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 9966
#define RSSPECAN_ATTR_AMPLIFIER_OBW                                                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 9967
#define RSSPECAN_ATTR_AMPLIFIER_FORCE_ARB_MODE                                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 8626
#define RSSPECAN_ATTR_AMPLIFIER_READ_REFERENCE_SIGNAL_FROM_SMW                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 5311
#define RSSPECAN_ATTR_AMPLIFIER_REFERENCE_SIGNAL_TRANSMISSION_STATE                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 5313
#define RSSPECAN_ATTR_AMPLIFIER_REFERENCE_SIGNAL_WAVEFORM_FILE_PATH                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 5314
#define RSSPECAN_ATTR_AMPLIFIER_REFERENCE_SIGNAL_WAVEFORM_FILE_EXPORT_TO_GENERATOR_ENABLED                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 5619
#define RSSPECAN_ATTR_AMPLIFIER_REFERENCE_SIGNAL_WAVEFORM_FILE_DUT_PEAK_INPUT_POWER                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 5620
#define RSSPECAN_ATTR_AMPLIFIER_REFERENCE_SIGNAL_WAVEFORM_FILE_SEGMENT                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 5621
#define RSSPECAN_ATTR_AMPLIFIER_WRITE_WAVEFORM_FILE_TO_SMW                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 5315
#define RSSPECAN_ATTR_AMPLIFIER_WAVEFORM_FILE_TRANSMISSION_STATE                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 5317
#define RSSPECAN_ATTR_AMPLIFIER_GENERATE_OWN_SIGNAL_CLOCK_RATE                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 5318
#define RSSPECAN_ATTR_AMPLIFIER_GENERATE_OWN_SIGNAL_BANDWIDTH                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 5319
#define RSSPECAN_ATTR_AMPLIFIER_GENERATE_OWN_SIGNAL_SAMPLE_LENGTH                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 5320
#define RSSPECAN_ATTR_AMPLIFIER_GENERATE_OWN_SIGNAL_CREST_FACTOR                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 5321
#define RSSPECAN_ATTR_AMPLIFIER_GENERATE_OWN_SIGNAL_NOTCH_WIDTH                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 5322
#define RSSPECAN_ATTR_AMPLIFIER_GENERATE_OWN_SIGNAL_NOTCH_POSITION                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 5323
#define RSSPECAN_ATTR_AMPLIFIER_GENERATE_OWN_SIGNAL_TRANSMISSION_STATE                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 5324
#define RSSPECAN_ATTR_AMPLIFIER_GENERATE_OWN_SIGNAL_PULSE_DUTY_CYCLE                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 5325
#define RSSPECAN_ATTR_AMPLIFIER_GENERATE_OWN_SIGNAL_RAMP_LENGTH                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 5326
#define RSSPECAN_ATTR_AMPLIFIER_GENERATE_OWN_SIGNAL_WAVEFORM_FILE_NAME                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 5327
#define RSSPECAN_ATTR_AMPLIFIER_GENERATE_OWN_SIGNAL_GENERATE_AND_PLAY                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 9968
#define RSSPECAN_ATTR_AMPLIFIER_REFERENCE_SIGNAL_DURATION                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 5328
#define RSSPECAN_ATTR_AMPLIFIER_CREST_FACTOR_REDUCTION_ENABLED                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 8627
#define RSSPECAN_ATTR_AMPLIFIER_CREST_FACTOR_REDUCTION_ENABLED_STATE                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 8628
#define RSSPECAN_ATTR_AMPLIFIER_CREST_FACTOR_REDUCTION_EVM_REFERENCE_SIGNAL                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 8629
#define RSSPECAN_ATTR_AMPLIFIER_CREST_FACTOR_REDUCTION_DELTA                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 8631
#define RSSPECAN_ATTR_AMPLIFIER_CREST_FACTOR_REDUCTION_DELTA_STATE                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 8632
#define RSSPECAN_ATTR_AMPLIFIER_CREST_FACTOR_REDUCTION_CURRENT_VALUE                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 8630
#define RSSPECAN_ATTR_AMPLIFIER_CREST_FACTOR_REDUCTION_MAX_ITERATIONS                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 8633
#define RSSPECAN_ATTR_AMPLIFIER_CREST_FACTOR_REDUCTION_ITERATIONS_STATE                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 8634
#define RSSPECAN_ATTR_AMPLIFIER_CREST_FACTOR_REDUCTION_FILTER_MODE                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 9969
#define RSSPECAN_ATTR_AMPLIFIER_CREST_FACTOR_REDUCTION_FILTER_MODE_STATE                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 9970
#define RSSPECAN_ATTR_AMPLIFIER_CREST_FACTOR_REDUCTION_SIGNAL_BANDWIDTH_AUTO                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 8635
#define RSSPECAN_ATTR_AMPLIFIER_CREST_FACTOR_REDUCTION_SIGNAL_BANDWIDTH                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 8636
#define RSSPECAN_ATTR_AMPLIFIER_CREST_FACTOR_REDUCTION_SIGNAL_BANDWIDTH_STATE                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 8637
#define RSSPECAN_ATTR_AMPLIFIER_CREST_FACTOR_REDUCTION_CHANNEL_SPACING_AUTO                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 8638
#define RSSPECAN_ATTR_AMPLIFIER_CREST_FACTOR_REDUCTION_CHANNEL_SPACING                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 8639
#define RSSPECAN_ATTR_AMPLIFIER_CREST_FACTOR_REDUCTION_CHANNEL_SPACING_STATE                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 8640
#define RSSPECAN_ATTR_AMPLIFIER_READ_CFR_FROM_GENERATOR_AND_LOAD                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 9971
#define RSSPECAN_ATTR_AMPLIFIER_READ_CFR_FROM_GENERATOR_AND_LOAD_STATE                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 9972
#define RSSPECAN_ATTR_AMPLIFIER_CREST_FACTOR_REDUCTION_PASSBAND_FREQUENCY                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 9973
#define RSSPECAN_ATTR_AMPLIFIER_CREST_FACTOR_REDUCTION_PASSBAND_FREQUENCY_STATE                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 9974
#define RSSPECAN_ATTR_AMPLIFIER_CREST_FACTOR_REDUCTION_STOPBAND_FREQUENCY                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 9975
#define RSSPECAN_ATTR_AMPLIFIER_CREST_FACTOR_REDUCTION_STOPBAND_FREQUENCY_STATE                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 9976
#define RSSPECAN_ATTR_AMPLIFIER_CREST_FACTOR_REDUCTION_MAXIMUM_FILTER_ORDER                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 9977
#define RSSPECAN_ATTR_AMPLIFIER_CREST_FACTOR_REDUCTION_MAXIMUM_FILTER_ORDER_STATE                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 9978
#define RSSPECAN_ATTR_AMPLIFIER_CREST_FACTOR_REDUCTION_APPLY                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 8641
#define RSSPECAN_ATTR_AMPLIFIER_CREST_FACTOR_REDUCTION_APPLY_STATE                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 8642
#define RSSPECAN_ATTR_SIGNAL_GENERATOR_CONTROL_ENABLED                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 7691
#define RSSPECAN_ATTR_AMPLIFIER_INPUT_SOURCE_CONNECTOR                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 5329
#define RSSPECAN_ATTR_AMPLIFIER_GENERATOR_SETUP_SETTINGS_UPDATE                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 5331
#define RSSPECAN_ATTR_AMPLIFIER_SETUP_FROM_GENERATOR                                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 6579
#define RSSPECAN_ATTR_AMPLIFIER_GENERATOR_RELAY_WRITE                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 10818
#define RSSPECAN_ATTR_AMPLIFIER_GENERATOR_RELAY_READ                                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 10819
#define RSSPECAN_ATTR_AMPLIFIER_GENERATOR_IP_ADDRESS                                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 5333
#define RSSPECAN_ATTR_AMPLIFIER_GENERATOR_IP_LED_STATE                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 5334
#define RSSPECAN_ATTR_AMPLIFIER_GENERATOR_RMS_LEVEL                                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 5335
#define RSSPECAN_ATTR_AMPLIFIER_GENERATOR_RMS_LEVEL_STATE                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 5336
#define RSSPECAN_ATTR_AMPLIFIER_ATTACH_TO_FSW_FREQUENCY                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 5337
#define RSSPECAN_ATTR_AMPLIFIER_GENERATOR_CENTER_FREQUENCY                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 5338
#define RSSPECAN_ATTR_AMPLIFIER_GENERATOR_CENTER_FREQUENCY_LED_STATE                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 5339
#define RSSPECAN_ATTR_AMPLIFIER_GENERATOR_TARGET_PATH_RF                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 5340
#define RSSPECAN_ATTR_AMPLIFIER_GENERATOR_TARGET_PATH_BASEBAND                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 5341
#define RSSPECAN_ATTR_AMPLIFIER_GENERATOR_EXTERNAL_OSCILATOR                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 5342
#define RSSPECAN_ATTR_AMPLIFIER_GENERATOR_EXTERNAL_OSCILATOR_LED_STATE                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 5343
#define RSSPECAN_ATTR_AMPLIFIER_GENERATOR_LEVEL_OFFSET                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 5622
#define RSSPECAN_ATTR_AMPLIFIER_GENERATOR_LEVEL_OFFSET_STATE                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 5623
#define RSSPECAN_ATTR_AMPLIFIER_GENERATOR_SEGMENT                                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 5624
#define RSSPECAN_ATTR_AMPLIFIER_GENERATOR_SEGMENT_STATE                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 5625
#define RSSPECAN_ATTR_AMPLIFIER_GENERATOR_DIGITAL_ATTENUATION                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 6581
#define RSSPECAN_ATTR_AMPLIFIER_GENERATOR_DIGITAL_ATTENUATION_STATE                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 6582
#define RSSPECAN_ATTR_AMPLIFIER_GENERATOR_RF_OUTPUT                                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 6583
#define RSSPECAN_ATTR_AMPLIFIER_GENERATOR_RF_OUTPUT_STATE                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 6584
#define RSSPECAN_ATTR_AMPLIFIER_GENERATOR_MAXIMUM_POWER                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 7692
#define RSSPECAN_ATTR_AMPLIFIER_GENERATOR_MAXIMUM_POWER_LED_STATE                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 7693
#define RSSPECAN_ATTR_AMPLIFIER_INPUT_SOURCE_ANALOG_BASEBAND_CAPTURE                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 5330
#define RSSPECAN_ATTR_AMPLIFIER_AVERAGING_I_Q_DATA                                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 6654
#define RSSPECAN_ATTR_AMPLIFIER_AVERAGING_I_Q_DATA_COUNT                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 6655
#define RSSPECAN_ATTR_AMPLIFIER_AVERAGING_I_Q_DATA_CURRENT_COUNT                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 6656
#define RSSPECAN_ATTR_AMPLIFIER_SWEEP_CONFIGURATION_STATISTICS_STATE                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 9979
#define RSSPECAN_ATTR_AMPLIFIER_SWEEP_CONFIGURATION_STATISTICS_MODE                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 9980
#define RSSPECAN_ATTR_AMPLIFIER_SWEEP_CONFIGURATION_CONTINUOUS_STATISTICS_STATE                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 9981
#define RSSPECAN_ATTR_AMPLIFIER_SWEEP_CONFIGURATION_STATISTICS_COUNT                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 9982
#define RSSPECAN_ATTR_AMPLIFIER_SWEEP_CONFIGURATION_SELECT_RESULT_RNG                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 9983
#define RSSPECAN_ATTR_AMPLIFIER_MESUREMENT_STOP                                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 5344
#define RSSPECAN_ATTR_AMPLIFIER_SYNCHRONIZATION_ENABLED                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 5626
#define RSSPECAN_ATTR_AMPLIFIER_SYNCHRONIZATION_DOMAIN                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 5345
#define RSSPECAN_ATTR_AMPLIFIER_SYNCHRONIZATION_CONFIDENCE_LEVEL                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 5627
#define RSSPECAN_ATTR_AMPLIFIER_SYNCHRONIZATION_STATE                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 5628
#define RSSPECAN_ATTR_AMPLIFIER_ESTIMATION_USE_FULL_REF_SIGNAL_ENABLED                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 5629
#define RSSPECAN_ATTR_AMPLIFIER_ESTIMATION_RANGE_START                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 5630
#define RSSPECAN_ATTR_AMPLIFIER_ESTIMATION_RANGE_STOP                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 5631
#define RSSPECAN_ATTR_AMPLIFIER_EVALUATION_USE_FULL_REF_SIGNAL_ENABLED                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 5632
#define RSSPECAN_ATTR_AMPLIFIER_EVALUATION_RANGE_START                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 5633
#define RSSPECAN_ATTR_AMPLIFIER_EVALUATION_RANGE_STOP                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 5634
#define RSSPECAN_ATTR_AMPLIFIER_ESTIMATION_AMPLITUDE_DROOP_STATE                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 5346
#define RSSPECAN_ATTR_AMPLIFIER_ESTIMATION_FREQUENCY_ERROR_STATE                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 9984
#define RSSPECAN_ATTR_AMPLIFIER_ESTIMATION_IQ_IMBALANCE_STATE                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 5347
#define RSSPECAN_ATTR_AMPLIFIER_ESTIMATION_IQ_OFFSET_STATE                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 9985
#define RSSPECAN_ATTR_AMPLIFIER_ESTIMATION_SAMPLE_ERROR_RATE_STATE                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 5348
#define RSSPECAN_ATTR_AMPLIFIER_COMPENSATION_AMPLITUDE_DROOP_STATE                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 5351
#define RSSPECAN_ATTR_AMPLIFIER_COMPENSATION_FREQUENCY_ERROR_STATE                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 9986
#define RSSPECAN_ATTR_AMPLIFIER_COMPENSATION_IQ_IMBALANCE_STATE                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 5352
#define RSSPECAN_ATTR_AMPLIFIER_COMPENSATION_IQ_OFFSET_STATE                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 9987
#define RSSPECAN_ATTR_AMPLIFIER_COMPENSATION_SAMPLE_ERROR_RATE_STATE                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 5353
#define RSSPECAN_ATTR_AMPLIFIER_EQUALIZER_FILTER_LENGTH_FOR_TRAINING                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 6657
#define RSSPECAN_ATTR_AMPLIFIER_EQUALIZER_FILTER_FILE_FORMAT                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 9988
#define RSSPECAN_ATTR_AMPLIFIER_EQUALIZER_TRAIN_FILTER                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 6658
#define RSSPECAN_ATTR_AMPLIFIER_EQUALIZER_STORE_FILTER                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 6660
#define RSSPECAN_ATTR_AMPLIFIER_EQUALIZER_RESTORE_FILTER                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 6662
#define RSSPECAN_ATTR_AMPLIFIER_EQUALIZER_STATE                                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 6664
#define RSSPECAN_ATTR_AMPLIFIER_MODELING_STATE                                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 5356
#define RSSPECAN_ATTR_AMPLIFIER_MODELING_ORDER_AMPM                                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 5357
#define RSSPECAN_ATTR_AMPLIFIER_MODELING_ORDER_AMAM                                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 5358
#define RSSPECAN_ATTR_AMPLIFIER_MODELING_RANGE                                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 5359
#define RSSPECAN_ATTR_AMPLIFIER_MODELING_POINTS                                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 5360
#define RSSPECAN_ATTR_AMPLIFIER_MODELING_SEQUENCE                                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 5635
#define RSSPECAN_ATTR_AMPLIFIER_MODELING_SCALE                                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 6210
#define RSSPECAN_ATTR_AMPLIFIER_DPD_SHAPING_METHOD                                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 5361
#define RSSPECAN_ATTR_AMPLIFIER_DPD_METHOD                                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 6585
#define RSSPECAN_ATTR_AMPLIFIER_DPD_GENERATE_WAVEFORM_FILE                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 6586
#define RSSPECAN_ATTR_AMPLIFIER_DPD_GENERATE_WAVEFORM_FILE_ALL                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 6665
#define RSSPECAN_ATTR_AMPLIFIER_DPD_EXPORT_WAVEFORM_FILE                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 6588
#define RSSPECAN_ATTR_AMPLIFIER_DPD_UPDATE_SHAPPING_TABLE                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 5362
#define RSSPECAN_ATTR_AMPLIFIER_DPD_UPDATE_SHAPPING_TABLE_STATUS                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 5364
#define RSSPECAN_ATTR_AMPLIFIER_DPD_UPDATE_ALL_SHAPING_TABLES                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 9990
#define RSSPECAN_ATTR_AMPLIFIER_DPD_CORECTION_TABLE_NAME                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 5365
#define RSSPECAN_ATTR_AMPLIFIER_DPD_SEQUENCE                                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 5636
#define RSSPECAN_ATTR_AMPLIFIER_DPD_AM_XM_STATE                                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 9991
#define RSSPECAN_ATTR_AMPLIFIER_DPD_AM_AM_STATE                                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 5367
#define RSSPECAN_ATTR_AMPLIFIER_DPD_AM_AM_LED_STATE                                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 6667
#define RSSPECAN_ATTR_AMPLIFIER_DPD_AM_PM_STATE                                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 5368
#define RSSPECAN_ATTR_AMPLIFIER_DPD_AM_PM_LED_STATE                                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 6668
#define RSSPECAN_ATTR_AMPLIFIER_DPD_TRADEOFF                                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 5638
#define RSSPECAN_ATTR_AMPLIFIER_DPD_WAVEFORM_PATH                                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 10820
#define RSSPECAN_ATTR_AMPLIFIER_DIRECT_DPD_STATE                                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 6669
#define RSSPECAN_ATTR_AMPLIFIER_DIRECT_DPD_START                                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 6670
#define RSSPECAN_ATTR_AMPLIFIER_DIRECT_DPD_ABORT                                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 6672
#define RSSPECAN_ATTR_AMPLIFIER_DIRECT_DPD_FINISH                                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 6674
#define RSSPECAN_ATTR_AMPLIFIER_DIRECT_DPD_CONTINUE                                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 10821
#define RSSPECAN_ATTR_AMPLIFIER_DIRECT_DPD_GAIN_EXPANSION                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 7694
#define RSSPECAN_ATTR_AMPLIFIER_DIRECT_DPD_ITERATIONS                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 6676
#define RSSPECAN_ATTR_AMPLIFIER_DIRECT_DPD_CURRENT_ITERATION                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 6677
#define RSSPECAN_ATTR_AMPLIFIER_DIRECT_DPD_POWER_LINEARITY_TRADEOFF                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 6678
#define RSSPECAN_ATTR_AMPLIFIER_DIRECT_DPD_NAME_ON_GENERATOR                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 6679
#define RSSPECAN_ATTR_AMPLIFIER_DIRECT_DPD_OPERATION_STATUS                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 7695
#define RSSPECAN_ATTR_AMPLIFIER_DIRECT_DPD_STORE_PREDISTORTION_WAVEFORM_FILE                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 6680
#define RSSPECAN_ATTR_AMPLIFIER_DIRECT_DPD_APPLY                                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 6682
#define RSSPECAN_ATTR_AMPLIFIER_DIRECT_DPD_WRAP_ENABLED                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 10822
#define RSSPECAN_ATTR_AMPLIFIER_DIRECT_DPD_RMS_CURRENT                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 10823
#define RSSPECAN_ATTR_AMPLIFIER_DIRECT_DPD_WAVEFORM_PATH                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 10824
#define RSSPECAN_ATTR_AMPLIFIER_MEMORY_POLYNOMIAL_DPD_STATE                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 9992
#define RSSPECAN_ATTR_AMPLIFIER_MEMORY_POLYNOMIAL_DPD_ITERATION_STEP                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 9993
#define RSSPECAN_ATTR_AMPLIFIER_MEMORY_POLYNOMIAL_DPD_POLYNOMIAL_ORDER                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 9994
#define RSSPECAN_ATTR_AMPLIFIER_MEMORY_POLYNOMIAL_DPD_MEMORY_ORDER                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 9021
#define RSSPECAN_ATTR_AMPLIFIER_MEMORY_POLYNOMIAL_DPD_MODEL                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 9995
#define RSSPECAN_ATTR_AMPLIFIER_MEMORY_POLYNOMIAL_DPD_MODEL_APPLY                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 10825
#define RSSPECAN_ATTR_AMPLIFIER_MEMORY_POLYNOMIAL_DPD_EXPORT_WAVEFORM_FILE                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 9996
#define RSSPECAN_ATTR_AMPLIFIER_MEMORY_POLYNOMIAL_DPD_EXPORT_COEFFICIENTS_FILE                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 9997
#define RSSPECAN_ATTR_AMPLIFIER_MEMORY_POLYNOMIAL_DPD_WAVEFORM_TYPE                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 9998
#define RSSPECAN_ATTR_AMPLIFIER_MEMORY_POLYNOMIAL_DPD_SEND_WAVEFORM_TO_GENERATOR                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 9999
#define RSSPECAN_ATTR_AMPLIFIER_MEMORY_POLYNOMIAL_DPD_RMS_CURRENT                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 10826
#define RSSPECAN_ATTR_AMPLIFIER_MEMORY_POLYNOMIAL_DPD_WAVEFORM_PATH                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 10827
#define RSSPECAN_ATTR_AMPLIFIER_HAMMERSTEIN_ENABLED                                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 10828
#define RSSPECAN_ATTR_AMPLIFIER_HAMMERSTEIN_ITERATION_STEP                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 10829
#define RSSPECAN_ATTR_AMPLIFIER_HAMMERSTEIN_MEMORY_ORDER                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 10830
#define RSSPECAN_ATTR_AMPLIFIER_HAMMERSTEIN_POLYNOMIAL_ORDER                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 10831
#define RSSPECAN_ATTR_AMPLIFIER_HAMMERSTEIN_GENERATOR_START                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 10832
#define RSSPECAN_ATTR_AMPLIFIER_HAMMERSTEIN_GENERATOR_WAVEFORM                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 10833
#define RSSPECAN_ATTR_AMPLIFIER_HAMMERSTEIN_NON_LINEARITY_ENABLED                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 10834
#define RSSPECAN_ATTR_AMPLIFIER_HAMMERSTEIN_FILTER_ENABLED                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 10835
#define RSSPECAN_ATTR_AMPLIFIER_CALCULATE_MSE                                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 10836
#define RSSPECAN_ATTR_AMPLIFIER_ENVELOPE_TRACKING_I_CHANNEL_MULTIPLIER                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 5369
#define RSSPECAN_ATTR_AMPLIFIER_ENVELOPE_TRACKING_I_CHANNEL_RESISTANCE                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 5370
#define RSSPECAN_ATTR_AMPLIFIER_ENVELOPE_TRACKING_I_CHANNEL_OFFSET                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 5371
#define RSSPECAN_ATTR_AMPLIFIER_ENVELOPE_TRACKING_Q_CHANNEL_MULTIPLIER                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 5372
#define RSSPECAN_ATTR_AMPLIFIER_ENVELOPE_TRACKING_Q_CHANNEL_OFFSET                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 5373
#define RSSPECAN_ATTR_AMPLIFIER_ENVELOPE_POWER_CONSUMPTION_PARAMETER_A                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 5374
#define RSSPECAN_ATTR_AMPLIFIER_ENVELOPE_POWER_CONSUMPTION_PARAMETER_B                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 5375
#define RSSPECAN_ATTR_AMPLIFIER_POWER_CALCULATE_OUTPUT_POWER_ONLY_ENABLED                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 5639
#define RSSPECAN_ATTR_AMPLIFIER_POWER_COMPRESSION_POINT_CALCULATION_ENABLED                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 5640
#define RSSPECAN_ATTR_AMPLIFIER_POWER_COMPRESSION_POINT_CALCULATION                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 5641
#define RSSPECAN_ATTR_AMPLIFIER_PARAMETER_SWEEP_ENABLED                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 5642
#define RSSPECAN_ATTR_AMPLIFIER_PARAMETER_SWEEP_X_AXIS_SETTING                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 5643
#define RSSPECAN_ATTR_AMPLIFIER_PARAMETER_SWEEP_X_AXIS_START                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 5644
#define RSSPECAN_ATTR_AMPLIFIER_PARAMETER_SWEEP_X_AXIS_STOP                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 5645
#define RSSPECAN_ATTR_AMPLIFIER_PARAMETER_SWEEP_X_AXIS_STEP                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 5646
#define RSSPECAN_ATTR_AMPLIFIER_PARAMETER_SWEEP_Y_AXIS_SETTING                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 5647
#define RSSPECAN_ATTR_AMPLIFIER_PARAMETER_SWEEP_Y_AXIS_START                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 5648
#define RSSPECAN_ATTR_AMPLIFIER_PARAMETER_SWEEP_Y_AXIS_STOP                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 5649
#define RSSPECAN_ATTR_AMPLIFIER_PARAMETER_SWEEP_Y_AXIS_STEP                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 5650
#define RSSPECAN_ATTR_AMPLIFIER_PARAMETER_SWEEP_Y_AXIS_ENABLED                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 5651
#define RSSPECAN_ATTR_AMPLIFIER_PARAMETER_SWEEP_SYNCHRONIZING_LEVELS_ENABLED                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 5652
#define RSSPECAN_ATTR_AMPLIFIER_PARAMETER_SWEEP_EXPECTED_GAIN                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 5653
#define RSSPECAN_ATTR_AMPLIFIER_POWER_SERVOING_MAXIMUM_INPUT                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 6443
#define RSSPECAN_ATTR_AMPLIFIER_POWER_SERVOING_EXPECTED_GAIN                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 6444
#define RSSPECAN_ATTR_AMPLIFIER_POWER_SERVOING_TARGET_OUTPUT_POWER                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 6445
#define RSSPECAN_ATTR_AMPLIFIER_POWER_SERVOING_MAXIMUM_OUTPUT_DEVIATION                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 6446
#define RSSPECAN_ATTR_AMPLIFIER_POWER_SERVOING_MEASUREMENT_AUTO                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 6447
#define RSSPECAN_ATTR_AMPLIFIER_POWER_SERVOING_MEASUREMENT_TIME                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 6448
#define RSSPECAN_ATTR_AMPLIFIER_POWER_SERVOING_CAPTURE_LENGTH                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 6449
#define RSSPECAN_ATTR_AMPLIFIER_POWER_SERVOING_START                                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 6450
#define RSSPECAN_ATTR_AMPLIFIER_POWER_SERVOING_STOP                                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 6451
#define RSSPECAN_ATTR_AMPLIFIER_SENSE_POWER_SERVOING_ENABLED                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 10837
#define RSSPECAN_ATTR_AMPLIFIER_SENSE_POWER_SERVOING_TARGET_PARAMETER                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 10838
#define RSSPECAN_ATTR_AMPLIFIER_SENSE_POWER_SERVOING_TARGET_VALUE                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 10839
#define RSSPECAN_ATTR_AMPLIFIER_SENSE_POWER_SERVOING_TARGET_TOLERANCE                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 10840
#define RSSPECAN_ATTR_AMPLIFIER_SENSE_POWER_SERVOING_MAXIMUM_ITERARION                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 10841
#define RSSPECAN_ATTR_AMPLIFIER_SENSE_POWER_SERVOING_GENERATOR_LEVEL                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 10842
#define RSSPECAN_ATTR_AMPLIFIER_SENSE_POWER_SERVOING_STEP_SIZE                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 10843
#define RSSPECAN_ATTR_AMPLIFIER_SENSE_POWER_SERVOING_START                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 10844
#define RSSPECAN_ATTR_AMPLIFIER_SENSE_POWER_SERVOING_STATUS                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 10845
#define RSSPECAN_ATTR_AMPLIFIER_TRACE_MODE                                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 5378
#define RSSPECAN_ATTR_AMPLIFIER_TRACE_DEFAULT_DETECTOR                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 10000
#define RSSPECAN_ATTR_AMPLIFIER_DETECTOR_MAX_TRACE_POINTS                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 10001
#define RSSPECAN_ATTR_AMPLIFIER_WINDOW_TRACE_MODE                                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 10002
#define RSSPECAN_ATTR_AMPLIFIER_WINDOW_TRACE_TYPE                                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 10003
#define RSSPECAN_ATTR_AMPLIFIER_WINDOW_TRACE_DETECTOR                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 10004
#define RSSPECAN_ATTR_AMPLIFIER_DISPLAY_WINDOW_STATISTICS_TABLE_ITEM_MACC_ALL                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 10006
#define RSSPECAN_ATTR_AMPLIFIER_DISPLAY_WINDOW_STATISTICS_TABLE_ITEM_POWER_ALL                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 10007
#define RSSPECAN_ATTR_AMPLIFIER_DISPLAY_WINDOW_STATISTICS_TABLE_ITEM_VOLT_CURR_ALL                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 10008
#define RSSPECAN_ATTR_AMPLIFIER_DISPLAY_WINDOW_TABLE_ITEM_MACC_ALL                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 6590
#define RSSPECAN_ATTR_AMPLIFIER_DISPLAY_WINDOW_TABLE_ITEM_POWER_ALL                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 6591
#define RSSPECAN_ATTR_AMPLIFIER_DISPLAY_WINDOW_TABLE_ITEM_VC_ALL                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 6592
#define RSSPECAN_ATTR_AMPLIFIER_GAIN_X_AXIS                                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 5380
#define RSSPECAN_ATTR_AMPLIFIER_AM_PM_DISPLAY_PHASE                                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 5381
#define RSSPECAN_ATTR_AMPLIFIER_AM_PM_DEFINITION                                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 6593
#define RSSPECAN_ATTR_AMPLIFIER_AM_PM_CURVE_WIDTH_REFERENCE_POINT_AUTO_ENABLED                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 10846
#define RSSPECAN_ATTR_AMPLIFIER_AM_PM_CURVE_WIDTH_REFERENCE_POINT                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 10847
#define RSSPECAN_ATTR_AMPLIFIER_TIME_DOMAIN_NORMALIZATION_STATE                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 5382
#define RSSPECAN_ATTR_AMPLIFIER_TIME_DOMAIN_POSITION                                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 5383
#define RSSPECAN_ATTR_AMPLIFIER_TIME_DOMAIN_OFFSET                                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 5384
#define RSSPECAN_ATTR_AMPLIFIER_TIME_DOMAIN_DURATION                                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 5385
#define RSSPECAN_ATTR_AMPLIFIER_PARAMETERS_SWEEP_RESULT_TYPE                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 5654
#define RSSPECAN_ATTR_AMPLIFIER_POWER_REFERENCE                                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 6594
#define RSSPECAN_ATTR_AMPLIFIER_PARAMETERS_SWEEP_TABLE_ITEM_ALL                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 6595
#define RSSPECAN_ATTR_AMPLIFIER_DISPLAY_WINDOW_RESULT_SUMMARY_TABLE_ITEM_MACC_ALL                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 7698
#define RSSPECAN_ATTR_AMPLIFIER_DISPLAY_WINDOW_RESULT_SUMMARY_TABLE_ITEM_POWER_ALL                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 7699
#define RSSPECAN_ATTR_AMPLIFIER_DISPLAY_WINDOW_RESULT_SUMMARY_TABLE_ITEM_VC_ALL                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 7700
#define RSSPECAN_ATTR_AMPLIFIER_DISPLAY_WINDOW_DDPD_RESULTS_TYPE                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 10009
#define RSSPECAN_ATTR_AMPLIFIER_FREQUENCY_ERROR_MAXIMUM                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 5387
#define RSSPECAN_ATTR_AMPLIFIER_FREQUENCY_ERROR_MINIMUM                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 5388
#define RSSPECAN_ATTR_AMPLIFIER_FREQUENCY_ERROR_CURRENT                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 5389
#define RSSPECAN_ATTR_AMPLIFIER_GAIN_IMBALANCE_MAXIMUM                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 5390
#define RSSPECAN_ATTR_AMPLIFIER_GAIN_IMBALANCE_MINIMUM                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 5391
#define RSSPECAN_ATTR_AMPLIFIER_GAIN_IMBALANCE_CURRENT                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 5392
#define RSSPECAN_ATTR_AMPLIFIER_IQ_IMBALANCE_MAXIMUM                                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 5393
#define RSSPECAN_ATTR_AMPLIFIER_IQ_IMBALANCE_MINIMUM                                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 5394
#define RSSPECAN_ATTR_AMPLIFIER_IQ_IMBALANCE_CURRENT                                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 5395
#define RSSPECAN_ATTR_AMPLIFIER_IQ_OFFSET_MAXIMUM                                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 5396
#define RSSPECAN_ATTR_AMPLIFIER_IQ_OFFSET_MINIMUM                                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 5397
#define RSSPECAN_ATTR_AMPLIFIER_IQ_OFFSET_CURRENT                                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 5398
#define RSSPECAN_ATTR_AMPLIFIER_MAGNITUDE_ERROR_MAXIMUM                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 5399
#define RSSPECAN_ATTR_AMPLIFIER_MAGNITUDE_ERROR_MINIMUM                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 5400
#define RSSPECAN_ATTR_AMPLIFIER_MAGNITUDE_ERROR_CURRENT                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 5401
#define RSSPECAN_ATTR_AMPLIFIER_PHASE_ERROR_MAXIMUM                                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 5402
#define RSSPECAN_ATTR_AMPLIFIER_PHASE_ERROR_MINIMUM                                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 5403
#define RSSPECAN_ATTR_AMPLIFIER_PHASE_ERROR_CURRENT                                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 5404
#define RSSPECAN_ATTR_AMPLIFIER_PHASE_ERROR_RESULT                                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 8643
#define RSSPECAN_ATTR_AMPLIFIER_QUADRATURE_ERROR_MAXIMUM                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 5405
#define RSSPECAN_ATTR_AMPLIFIER_QUADRATURE_ERROR_MINIMUM                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 5406
#define RSSPECAN_ATTR_AMPLIFIER_QUADRATURE_ERROR_CURRENT                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 5407
#define RSSPECAN_ATTR_AMPLIFIER_RAW_EVM_MAXIMUM                                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 5408
#define RSSPECAN_ATTR_AMPLIFIER_RAW_EVM_MINIMUM                                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 5409
#define RSSPECAN_ATTR_AMPLIFIER_RAW_EVM_CURRENT                                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 5410
#define RSSPECAN_ATTR_AMPLIFIER_RAW_MODEL_EVM_MAXIMUM                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 5411
#define RSSPECAN_ATTR_AMPLIFIER_RAW_MODEL_EVM_MINIMUM                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 5412
#define RSSPECAN_ATTR_AMPLIFIER_RAW_MODEL_EVM_CURRENT                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 5413
#define RSSPECAN_ATTR_AMPLIFIER_SAMPLE_RATE_ERROR_MAXIMUM                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 5414
#define RSSPECAN_ATTR_AMPLIFIER_SAMPLE_RATE_ERROR_MINIMUM                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 5415
#define RSSPECAN_ATTR_AMPLIFIER_SAMPLE_RATE_ERROR_CURRENT                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 5416
#define RSSPECAN_ATTR_AMPLIFIER_AMAM_CURVE_WIDTH_MAXIMUM                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 5417
#define RSSPECAN_ATTR_AMPLIFIER_AMAM_CURVE_WIDTH_MINIMUM                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 5418
#define RSSPECAN_ATTR_AMPLIFIER_AMAM_CURVE_WIDTH_CURRENT                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 5419
#define RSSPECAN_ATTR_AMPLIFIER_AMAM_CURVE_WIDTH_PEAK_CURRENT                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 10848
#define RSSPECAN_ATTR_AMPLIFIER_AMPM_CURVE_WIDTH_MAXIMUM                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 5420
#define RSSPECAN_ATTR_AMPLIFIER_AMPM_CURVE_WIDTH_MINIMUM                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 5421
#define RSSPECAN_ATTR_AMPLIFIER_AMPM_CURVE_WIDTH_CURRENT                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 5422
#define RSSPECAN_ATTR_AMPLIFIER_AMPM_CURVE_WIDTH_PEAK_CURRENT                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 10849
#define RSSPECAN_ATTR_AMPLIFIER_CREST_FACTOR_INPUT_MAXIMUM                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 5423
#define RSSPECAN_ATTR_AMPLIFIER_CREST_FACTOR_INPUT_MINIMUM                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 5424
#define RSSPECAN_ATTR_AMPLIFIER_CREST_FACTOR_INPUT_CURRENT                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 5425
#define RSSPECAN_ATTR_AMPLIFIER_CREST_FACTOR_OUTPUT_MAXIMUM                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 5426
#define RSSPECAN_ATTR_AMPLIFIER_CREST_FACTOR_OUTPUT_MINIMUM                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 5427
#define RSSPECAN_ATTR_AMPLIFIER_CREST_FACTOR_OUTPUT_CURRENT                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 5428
#define RSSPECAN_ATTR_AMPLIFIER_SIGNAL_GAIN_MAXIMUM                                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 5429
#define RSSPECAN_ATTR_AMPLIFIER_SIGNAL_GAIN_MINIMUM                                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 5430
#define RSSPECAN_ATTR_AMPLIFIER_SIGNAL_GAIN_CURRENT                                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 5431
#define RSSPECAN_ATTR_AMPLIFIER_POWER_INPUT_MAXIMUM                                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 5432
#define RSSPECAN_ATTR_AMPLIFIER_POWER_INPUT_MINIMUM                                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 5433
#define RSSPECAN_ATTR_AMPLIFIER_POWER_INPUT_CURRENT                                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 5434
#define RSSPECAN_ATTR_AMPLIFIER_POWER_OUTPUT_MAXIMUM                                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 5435
#define RSSPECAN_ATTR_AMPLIFIER_POWER_OUTPUT_MINIMUM                                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 5436
#define RSSPECAN_ATTR_AMPLIFIER_POWER_OUTPUT_CURRENT                                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 5437
#define RSSPECAN_ATTR_AMPLIFIER_AVERAGE_PAE_MAXIMUM                                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 5438
#define RSSPECAN_ATTR_AMPLIFIER_AVERAGE_PAE_MINIMUM                                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 5439
#define RSSPECAN_ATTR_AMPLIFIER_AVERAGE_PAE_CURRENT                                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 5440
#define RSSPECAN_ATTR_AMPLIFIER_CURRENT_MAXIMUM                                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 5441
#define RSSPECAN_ATTR_AMPLIFIER_CURRENT_MINIMUM                                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 5442
#define RSSPECAN_ATTR_AMPLIFIER_CURRENT_CURRENT                                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 5443
#define RSSPECAN_ATTR_AMPLIFIER_BASEBAND_POWER_MAXIMUM                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 7701
#define RSSPECAN_ATTR_AMPLIFIER_BASEBAND_POWER_MINIMUM                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 7702
#define RSSPECAN_ATTR_AMPLIFIER_BASEBAND_POWER_CURRENT                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 7703
#define RSSPECAN_ATTR_AMPLIFIER_BASEBAND_INPUT_VOLTAGE_I_MAXIMUM                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 5444
#define RSSPECAN_ATTR_AMPLIFIER_BASEBAND_INPUT_VOLTAGE_I_MINIMUM                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 5445
#define RSSPECAN_ATTR_AMPLIFIER_BASEBAND_INPUT_VOLTAGE_I_CURRENT                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 5446
#define RSSPECAN_ATTR_AMPLIFIER_POWER_CURRENT                                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 5447
#define RSSPECAN_ATTR_AMPLIFIER_BASEBAND_INPUT_VOLTAGE_Q_MAXIMUM                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 5448
#define RSSPECAN_ATTR_AMPLIFIER_BASEBAND_INPUT_VOLTAGE_Q_MINIMUM                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 5449
#define RSSPECAN_ATTR_AMPLIFIER_BASEBAND_INPUT_VOLTAGE_Q_CURRENT                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 5450
#define RSSPECAN_ATTR_AMPLIFIER_VOLTAGE_MAXIMUM                                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 5451
#define RSSPECAN_ATTR_AMPLIFIER_VOLTAGE_MINIMUM                                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 5452
#define RSSPECAN_ATTR_AMPLIFIER_VOLTAGE_CURRENT                                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 5453
#define RSSPECAN_ATTR_AMPLIFIER_COMPRESSION_POINT_1DB_CURRENT                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 5656
#define RSSPECAN_ATTR_AMPLIFIER_COMPRESSION_POINT_1_DB_MAXIMUM_POWER                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 10010
#define RSSPECAN_ATTR_AMPLIFIER_COMPRESSION_POINT_1_DB_MINIMUM_POWER                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 10011
#define RSSPECAN_ATTR_AMPLIFIER_COMPRESSION_POINT_1_DB_OUTPUT_CURRENT                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 10012
#define RSSPECAN_ATTR_AMPLIFIER_COMPRESSION_POINT_1_DB_OUTPUT_MAXIMUM_POWER                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 10013
#define RSSPECAN_ATTR_AMPLIFIER_COMPRESSION_POINT_1_DB_OUTPUT_MINIMUM_POWER                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 10014
#define RSSPECAN_ATTR_AMPLIFIER_COMPRESSION_POINT_2DB_CURRENT                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 5657
#define RSSPECAN_ATTR_AMPLIFIER_COMPRESSION_POINT_2_DB_MAXIMUM_POWER                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 10015
#define RSSPECAN_ATTR_AMPLIFIER_COMPRESSION_POINT_2_DB_MINIMUM_POWER                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 10016
#define RSSPECAN_ATTR_AMPLIFIER_COMPRESSION_POINT_2_DB_OUTPUT_CURRENT                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 10017
#define RSSPECAN_ATTR_AMPLIFIER_COMPRESSION_POINT_2_DB_OUTPUT_MAXIMUM_POWER                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 10018
#define RSSPECAN_ATTR_AMPLIFIER_COMPRESSION_POINT_2_DB_OUTPUT_MINIMUM_POWER                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 10019
#define RSSPECAN_ATTR_AMPLIFIER_COMPRESSION_POINT_3DB_CURRENT                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 5658
#define RSSPECAN_ATTR_AMPLIFIER_COMPRESSION_POINT_3_DB_MAXIMUM_POWER                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 10020
#define RSSPECAN_ATTR_AMPLIFIER_COMPRESSION_POINT_3_DB_MINIMUM_POWER                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 10021
#define RSSPECAN_ATTR_AMPLIFIER_COMPRESSION_POINT_3_DB_OUTPUT_CURRENT                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 10022
#define RSSPECAN_ATTR_AMPLIFIER_COMPRESSION_POINT_3_DB_OUTPUT_MAXIMUM_POWER                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 10023
#define RSSPECAN_ATTR_AMPLIFIER_COMPRESSION_POINT_3_DB_OUTPUT_MINIMUM_POWER                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 10024
#define RSSPECAN_ATTR_AMPLIFIER_INPUT_POWER_SENSOR_CURRENT                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 10850
#define RSSPECAN_ATTR_AMPLIFIER_INPUT_POWER_SENSOR_MINIMUM                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 10851
#define RSSPECAN_ATTR_AMPLIFIER_INPUT_POWER_SENSOR_MAXIMUM                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 10852
#define RSSPECAN_ATTR_AMPLIFIER_OUTPUT_POWER_SENSOR_CURRENT                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 10853
#define RSSPECAN_ATTR_AMPLIFIER_OUTPUT_POWER_SENSOR_MAXIMUM                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 10854
#define RSSPECAN_ATTR_AMPLIFIER_TRIGGERR_TO_FRAME                                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 5659
#define RSSPECAN_ATTR_AMPLIFIER_TRIGGER_TO_SYNC                                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 10025
#define RSSPECAN_ATTR_AMPLIFIER_MEMORY_DPD_COEFFICIENTS                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 10026
#define RSSPECAN_ATTR_AMPLIFIER_PTABLE_ACP_MAXIMUM                                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 5661
#define RSSPECAN_ATTR_AMPLIFIER_PTABLE_ACP_MINIMUM                                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 5662
#define RSSPECAN_ATTR_AMPLIFIER_PTABLE_ACP_X_MAXIMUM                                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 5663
#define RSSPECAN_ATTR_AMPLIFIER_PTABLE_ACP_X_MINIMUM                                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 5664
#define RSSPECAN_ATTR_AMPLIFIER_PTABLE_ACP_Y_MAXIMUM                                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 5665
#define RSSPECAN_ATTR_AMPLIFIER_PTABLE_ACP_Y_MINIMUM                                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 5666
#define RSSPECAN_ATTR_AMPLIFIER_PTABLE_ACP_LOWER_MAXIMUM                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 5667
#define RSSPECAN_ATTR_AMPLIFIER_PTABLE_ACP_LOWER_MINIMUM                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 5668
#define RSSPECAN_ATTR_AMPLIFIER_PTABLE_ACP_LOWER_X_MAXIMUM                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 5669
#define RSSPECAN_ATTR_AMPLIFIER_PTABLE_ACP_LOWER_X_MINIMUM                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 5670
#define RSSPECAN_ATTR_AMPLIFIER_PTABLE_ACP_LOWER_Y_MAXIMUM                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 5671
#define RSSPECAN_ATTR_AMPLIFIER_PTABLE_ACP_LOWER_Y_MINIMUM                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 5672
#define RSSPECAN_ATTR_AMPLIFIER_PTABLE_ACP_UPPER_MAXIMUM                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 5673
#define RSSPECAN_ATTR_AMPLIFIER_PTABLE_ACP_UPPER_MINIMUM                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 5674
#define RSSPECAN_ATTR_AMPLIFIER_PTABLE_ACP_UPPER_X_MAXIMUM                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 5675
#define RSSPECAN_ATTR_AMPLIFIER_PTABLE_ACP_UPPER_X_MINIMUM                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 5676
#define RSSPECAN_ATTR_AMPLIFIER_PTABLE_ACP_UPPER_Y_MAXIMUM                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 5677
#define RSSPECAN_ATTR_AMPLIFIER_PTABLE_ACP_UPPER_Y_MINIMUM                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 5678
#define RSSPECAN_ATTR_AMPLIFIER_PTABLE_ACP_BALANCED_MAXIMUM                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 6195
#define RSSPECAN_ATTR_AMPLIFIER_PTABLE_ACP_BALANCED_MINIMUM                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 6196
#define RSSPECAN_ATTR_AMPLIFIER_PTABLE_ACP_BALANCED_X_MAXIMUM                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 6197
#define RSSPECAN_ATTR_AMPLIFIER_PTABLE_ACP_BALANCED_X_MINIMUM                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 6198
#define RSSPECAN_ATTR_AMPLIFIER_PTABLE_ACP_BALANCED_Y_MAXIMUM                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 6199
#define RSSPECAN_ATTR_AMPLIFIER_PTABLE_ACP_BALANCED_Y_MINIMUM                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 6200
#define RSSPECAN_ATTR_AMPLIFIER_PTABLE_AMAM_CURVE_WIDTH_MAXIMUM                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 5679
#define RSSPECAN_ATTR_AMPLIFIER_PTABLE_AMAM_CURVE_WIDTH_MINIMUM                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 5680
#define RSSPECAN_ATTR_AMPLIFIER_PTABLE_AMAM_CURVE_WIDTH_X_MAXIMUM                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 5681
#define RSSPECAN_ATTR_AMPLIFIER_PTABLE_AMAM_CURVE_WIDTH_X_MINIMUM                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 5682
#define RSSPECAN_ATTR_AMPLIFIER_PTABLE_AMAM_CURVE_WIDTH_Y_MAXIMUM                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 5683
#define RSSPECAN_ATTR_AMPLIFIER_PTABLE_AMAM_CURVE_WIDTH_Y_MINIMUM                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 5684
#define RSSPECAN_ATTR_AMPLIFIER_PTABLE_AMPM_CURVE_WIDTH_MAXIMUM                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 5685
#define RSSPECAN_ATTR_AMPLIFIER_PTABLE_AMPM_CURVE_WIDTH_MINIMUM                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 5686
#define RSSPECAN_ATTR_AMPLIFIER_PTABLE_AMPM_CURVE_WIDTH_X_MAXIMUM                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 5687
#define RSSPECAN_ATTR_AMPLIFIER_PTABLE_AMPM_CURVE_WIDTH_X_MINIMUM                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 5688
#define RSSPECAN_ATTR_AMPLIFIER_PTABLE_AMPM_CURVE_WIDTH_Y_MAXIMUM                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 5689
#define RSSPECAN_ATTR_AMPLIFIER_PTABLE_AMPM_CURVE_WIDTH_Y_MINIMUM                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 5690
#define RSSPECAN_ATTR_AMPLIFIER_PTABLE_BB_POWER_MAXIMUM                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 5691
#define RSSPECAN_ATTR_AMPLIFIER_PTABLE_BB_POWER_MINIMUM                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 5692
#define RSSPECAN_ATTR_AMPLIFIER_PTABLE_BB_POWER_X_MAXIMUM                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 5693
#define RSSPECAN_ATTR_AMPLIFIER_PTABLE_BB_POWER_X_MINIMUM                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 5694
#define RSSPECAN_ATTR_AMPLIFIER_PTABLE_BB_POWER_Y_MAXIMUM                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 5695
#define RSSPECAN_ATTR_AMPLIFIER_PTABLE_BB_POWER_Y_MINIMUM                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 5696
#define RSSPECAN_ATTR_AMPLIFIER_PTABLE_CREST_FACTOR_MAXIMUM                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 5697
#define RSSPECAN_ATTR_AMPLIFIER_PTABLE_CREST_FACTOR_MINIMUM                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 5698
#define RSSPECAN_ATTR_AMPLIFIER_PTABLE_CREST_FACTOR_X_MAXIMUM                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 5699
#define RSSPECAN_ATTR_AMPLIFIER_PTABLE_CREST_FACTOR_X_MINIMUM                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 5700
#define RSSPECAN_ATTR_AMPLIFIER_PTABLE_CREST_FACTOR_Y_MAXIMUM                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 5701
#define RSSPECAN_ATTR_AMPLIFIER_PTABLE_CREST_FACTOR_Y_MINIMUM                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 5702
#define RSSPECAN_ATTR_AMPLIFIER_PTABLE_EVM_MAXIMUM                                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 5703
#define RSSPECAN_ATTR_AMPLIFIER_PTABLE_EVM_MINIMUM                                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 5704
#define RSSPECAN_ATTR_AMPLIFIER_PTABLE_EVM_X_MAXIMUM                                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 5705
#define RSSPECAN_ATTR_AMPLIFIER_PTABLE_EVM_X_MINIMUM                                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 5706
#define RSSPECAN_ATTR_AMPLIFIER_PTABLE_EVM_Y_MAXIMUM                                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 5707
#define RSSPECAN_ATTR_AMPLIFIER_PTABLE_EVM_Y_MINIMUM                                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 5708
#define RSSPECAN_ATTR_AMPLIFIER_PTABLE_GAIN_MAXIMUM                                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 5709
#define RSSPECAN_ATTR_AMPLIFIER_PTABLE_GAIN_MINIMUM                                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 5710
#define RSSPECAN_ATTR_AMPLIFIER_PTABLE_GAIN_X_MAXIMUM                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 5711
#define RSSPECAN_ATTR_AMPLIFIER_PTABLE_GAIN_X_MINIMUM                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 5712
#define RSSPECAN_ATTR_AMPLIFIER_PTABLE_GAIN_Y_MAXIMUM                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 5713
#define RSSPECAN_ATTR_AMPLIFIER_PTABLE_GAIN_Y_MINIMUM                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 5714
#define RSSPECAN_ATTR_AMPLIFIER_PTABLE_ICC_MAXIMUM                                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 5715
#define RSSPECAN_ATTR_AMPLIFIER_PTABLE_ICC_MINIMUM                                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 5716
#define RSSPECAN_ATTR_AMPLIFIER_PTABLE_ICC_X_MAXIMUM                                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 5717
#define RSSPECAN_ATTR_AMPLIFIER_PTABLE_ICC_X_MINIMUM                                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 5718
#define RSSPECAN_ATTR_AMPLIFIER_PTABLE_ICC_Y_MAXIMUM                                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 5719
#define RSSPECAN_ATTR_AMPLIFIER_PTABLE_ICC_Y_MINIMUM                                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 5720
#define RSSPECAN_ATTR_AMPLIFIER_PTABLE_VCC_MAXIMUM                                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 5721
#define RSSPECAN_ATTR_AMPLIFIER_PTABLE_VCC_MINIMUM                                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 5722
#define RSSPECAN_ATTR_AMPLIFIER_PTABLE_VCC_X_MAXIMUM                                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 5723
#define RSSPECAN_ATTR_AMPLIFIER_PTABLE_VCC_X_MINIMUM                                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 5724
#define RSSPECAN_ATTR_AMPLIFIER_PTABLE_VCC_Y_MAXIMUM                                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 5725
#define RSSPECAN_ATTR_AMPLIFIER_PTABLE_VCC_Y_MINIMUM                                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 5726
#define RSSPECAN_ATTR_AMPLIFIER_PTABLE_PAE_MAXIMUM                                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 5727
#define RSSPECAN_ATTR_AMPLIFIER_PTABLE_PAE_MINIMUM                                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 5728
#define RSSPECAN_ATTR_AMPLIFIER_PTABLE_PAE_X_MAXIMUM                                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 5729
#define RSSPECAN_ATTR_AMPLIFIER_PTABLE_PAE_X_MINIMUM                                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 5730
#define RSSPECAN_ATTR_AMPLIFIER_PTABLE_PAE_Y_MAXIMUM                                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 5731
#define RSSPECAN_ATTR_AMPLIFIER_PTABLE_PAE_Y_MINIMUM                                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 5732
#define RSSPECAN_ATTR_AMPLIFIER_PTABLE_RMS_POWER_MAXIMUM                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 5733
#define RSSPECAN_ATTR_AMPLIFIER_PTABLE_RMS_POWER_MINIMUM                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 5734
#define RSSPECAN_ATTR_AMPLIFIER_PTABLE_RMS_POWER_X_MAXIMUM                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 5735
#define RSSPECAN_ATTR_AMPLIFIER_PTABLE_RMS_POWER_X_MINIMUM                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 5736
#define RSSPECAN_ATTR_AMPLIFIER_PTABLE_RMS_POWER_Y_MAXIMUM                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 5737
#define RSSPECAN_ATTR_AMPLIFIER_PTABLE_RMS_POWER_Y_MINIMUM                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 5738
#define RSSPECAN_ATTR_AMPLIFIER_POWER_OUTPUT_MAXIMUM_X                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 7705
#define RSSPECAN_ATTR_AMPLIFIER_POWER_OUTPUT_MAXIMUM_Y                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 7706
#define RSSPECAN_ATTR_AMPLIFIER_PTABLE_POWER_OUTPUT_MAXIMUM                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 7704
#define RSSPECAN_ATTR_AMPLIFIER_PTABLE_POWER_OUTPUT_MINIMUM                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 7707
#define RSSPECAN_ATTR_AMPLIFIER_POWER_OUTPUT_MINIMUM_X                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 7708
#define RSSPECAN_ATTR_AMPLIFIER_POWER_OUTPUT_MINIMUM_Y                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 7709
#define RSSPECAN_ATTR_AMPLIFIER_COMPRESSION_POINT_1_DB_MAXIMUM                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 7710
#define RSSPECAN_ATTR_AMPLIFIER_COMPRESSION_POINT_1_DB_MAXIMUM_X                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 7711
#define RSSPECAN_ATTR_AMPLIFIER_COMPRESSION_POINT_1_DB_MAXIMUM_Y                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 7712
#define RSSPECAN_ATTR_AMPLIFIER_COMPRESSION_POINT_1_DB_MINIMUM                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 7713
#define RSSPECAN_ATTR_AMPLIFIER_COMPRESSION_POINT_1_DB_MINIMUM_X                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 7714
#define RSSPECAN_ATTR_AMPLIFIER_COMPRESSION_POINT_1_DB_MINIMUM_Y                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 7715
#define RSSPECAN_ATTR_AMPLIFIER_COMPRESSION_POINT_2_DB_MAXIMUM                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 7716
#define RSSPECAN_ATTR_AMPLIFIER_COMPRESSION_POINT_2_DB_MAXIMUM_X                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 7717
#define RSSPECAN_ATTR_AMPLIFIER_COMPRESSION_POINT_2_DB_MAXIMUM_Y                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 7718
#define RSSPECAN_ATTR_AMPLIFIER_COMPRESSION_POINT_2_DB_MINIMUM                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 7719
#define RSSPECAN_ATTR_AMPLIFIER_COMPRESSION_POINT_2_DB_MINIMUM_X                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 7720
#define RSSPECAN_ATTR_AMPLIFIER_COMPRESSION_POINT_2_DB_MINIMUM_Y                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 7721
#define RSSPECAN_ATTR_AMPLIFIER_COMPRESSION_POINT_3_DB_MAXIMUM                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 7722
#define RSSPECAN_ATTR_AMPLIFIER_COMPRESSION_POINT_3_DB_MAXIMUM_X                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 7723
#define RSSPECAN_ATTR_AMPLIFIER_COMPRESSION_POINT_3_DB_MAXIMUM_Y                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 7724
#define RSSPECAN_ATTR_AMPLIFIER_COMPRESSION_POINT_3_DB_MINIMUM                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 7725
#define RSSPECAN_ATTR_AMPLIFIER_COMPRESSION_POINT_3_DB_MINIMUM_X                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 7726
#define RSSPECAN_ATTR_AMPLIFIER_COMPRESSION_POINT_3_DB_MINIMUM_Y                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 7727
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_AMPLITUDE_DROOP_AVERAGE_ALL                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 10027
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_AMPLITUDE_DROOP_AVERAGE_CURRENT                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 10028
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_AMPLITUDE_DROOP_MAXIMUM_ALL                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 10029
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_AMPLITUDE_DROOP_MAXIMUM_CURRENT                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 10030
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_AMPLITUDE_DROOP_MINIMUM_ALL                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 10031
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_AMPLITUDE_DROOP_MIMIMUM_CURRENT                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 10032
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_AMPLITUDE_DROOP_STANDARD_DEVIATION_ALL                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 10033
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_AMPLITUDE_DROOP_STANDARD_DEVIATION_CURRENT                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 10034
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_AMPLITUDE_DROOP_SELECTED_RESULT                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 10035
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_AM_AM_CURVE_WIDTH_AVERAGE_ALL                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 10036
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_AM_AM_CURVE_WIDTH_AVERAGE_CURRENT                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 10037
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_AM_AM_CURVE_WIDTH_MAXIMUM_ALL                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 10038
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_AM_AM_CURVE_WIDTH_MAXIMUM_CURRENT                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 10039
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_AM_AM_CURVE_WIDTH_MINIMUM_ALL                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 10040
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_AM_AM_CURVE_WIDTH_MINIMUM_CURRENT                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 10041
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_AM_AM_CURVE_WIDTH_STANDARD_DEVIATION_ALL                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 10042
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_AM_AM_CURVE_WIDTH_STANDARD_DEVIATION_CURRENT                RS_SPECIFIC_PUBLIC_ATTR_BASE + 10043
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_AM_AM_CURVE_WIDTH_SELECTED_RESULT                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 10044
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_AM_PM_CURVE_WIDTH_AVERAGE_ALL                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 10045
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_AM_PM_CURVE_WIDTH_AVERAGE_CURRENT                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 10046
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_AM_PM_CURVE_WIDTH_MAXIMUM_ALL                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 10047
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_AM_PM_CURVE_WIDTH_MAXIMUM_CURRENT                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 10048
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_AM_PM_CURVE_WIDTH_MINIMUM_ALL                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 10049
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_AM_PM_CURVE_WIDTH_MINIMUM_CURRENT                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 10050
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_AM_PM_CURVE_WIDTH_STANDARD_DEVIATION_ALL                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 10051
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_AM_PM_CURVE_WIDTH_STANDARD_DEVIATION_CURRENT                RS_SPECIFIC_PUBLIC_ATTR_BASE + 10052
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_AM_PM_CURVE_WIDTH_SELECTED_RESULT                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 10053
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_AVERAGE_PAE_AVERAGE_AVERAGE_ALL                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 10054
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_AVERAGE_PAE_AVERAGE_AVERAGE_CURRENT                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 10055
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_AVERAGE_PAE_AVERAGE_MAXIMUM_ALL                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 10056
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_AVERAGE_PAE_AVERAGE_MAXIMUM_CURRENT                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 10057
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_AVERAGE_PAE_AVERAGE_MINIMUM_ALL                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 10058
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_AVERAGE_PAE_AVERAGE_MINIMUM_CURRENT                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 10059
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_AVERAGE_PAE_AVERAGE_STANDARD_DEVIATION_ALL                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 10060
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_AVERAGE_PAE_AVERAGE_STANDARD_DEVIATION_CURRENT              RS_SPECIFIC_PUBLIC_ATTR_BASE + 10061
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_AVERAGE_PAE_AVERAGE_SELECTED_RESULT                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 10062
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_AVERAGE_PAE_MAXIMUM_AVERAGE_ALL                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 10063
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_AVERAGE_PAE_MAXIMUM_AVERAGE_CURRENT                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 10064
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_AVERAGE_PAE_MAXIMUM_MAXIMUM_ALL                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 10065
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_AVERAGE_PAE_MAXIMUM_MAXIMUM_CURRENT                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 10066
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_AVERAGE_PAE_MAXIMUM_MINIMUM_ALL                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 10067
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_AVERAGE_PAE_MAXIMUM_MINIMUM_CURRENT                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 10068
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_AVERAGE_PAE_MAXIMUM_STANDARD_DEVIATION_ALL                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 10069
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_AVERAGE_PAE_MAXIMUM_STANDARD_DEVIATION_CURRENT              RS_SPECIFIC_PUBLIC_ATTR_BASE + 10070
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_AVERAGE_PAE_MAXIMUM_SELECTED_RESULT                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 10071
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_AVERAGE_PAE_MINIMUM_AVERAGE_ALL                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 10072
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_AVERAGE_PAE_MINIMUM_AVERAGE_CURRENT                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 10073
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_AVERAGE_PAE_MINIMUM_MAXIMUM_ALL                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 10074
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_AVERAGE_PAE_MINIMUM_MAXIMUM_CURRENT                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 10075
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_AVERAGE_PAE_MINIMUM_MINIMUM_ALL                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 10076
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_AVERAGE_PAE_MINIMUM_MINIMUM_CURRENT                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 10077
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_AVERAGE_PAE_MINIMUM_STANDARD_DEVIATION_ALL                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 10078
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_AVERAGE_PAE_MINIMUM_STANDARD_DEVIATION_CURRENT              RS_SPECIFIC_PUBLIC_ATTR_BASE + 10079
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_AVERAGE_PAE_MINIMUM_SELECTED_RESULT                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 10080
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_POWER_VCC_X_ICC_AVERAGE_AVERAGE_ALL                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 10081
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_POWER_VCC_X_ICC_AVERAGE_AVERAGE_CURRENT                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 10082
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_POWER_VCC_X_ICC_AVERAGE_MAXIMUM_ALL                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 10083
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_POWER_VCC_X_ICC_AVERAGE_MAXIMUM_CURRENT                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 10084
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_POWER_VCC_X_ICC_AVERAGE_MINIMUM_ALL                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 10085
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_POWER_VCC_X_ICC_AVERAGE_MINIMUM_CURRENT                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 10086
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_POWER_VCC_X_ICC_AVERAGE_STANDARD_DEVIATION_ALL              RS_SPECIFIC_PUBLIC_ATTR_BASE + 10087
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_POWER_VCC_X_ICC_AVERAGE_STANDARD_DEVIATION_CURRENT          RS_SPECIFIC_PUBLIC_ATTR_BASE + 10088
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_POWER_VCC_X_ICC_AVERAGE_SELECTED_RESULT                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 10089
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_POWER_VCC_X_ICC_MAXIMUM_AVERAGE_ALL                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 10090
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_POWER_VCC_X_ICC_MAXIMUM_AVERAGE_CURRENT                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 10091
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_POWER_VCC_X_ICC_MAXIMUM_MAXIMUM_ALL                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 10092
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_POWER_VCC_X_ICC_MAXIMUM_MAXIMUM_CURRENT                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 10093
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_POWER_VCC_X_ICC_MAXIMUM_MINIMUM_ALL                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 10094
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_POWER_VCC_X_ICC_MAXIMUM_MINIMUM_CURRENT                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 10095
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_POWER_VCC_X_ICC_MAXIMUM_STANDARD_DEVIATION_ALL              RS_SPECIFIC_PUBLIC_ATTR_BASE + 10096
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_POWER_VCC_X_ICC_MAXIMUM_STANDARD_DEVIATION_CURRENT          RS_SPECIFIC_PUBLIC_ATTR_BASE + 10097
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_POWER_VCC_X_ICC_MAXIMUM_SELECTED_RESULT                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 10098
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_POWER_VCC_X_ICC_MINIMUM_AVERAGE_ALL                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 10099
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_POWER_VCC_X_ICC_MINIMUM_AVERAGE_CURRENT                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 10100
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_POWER_VCC_X_ICC_MINIMUM_MAXIMUM_ALL                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 10101
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_POWER_VCC_X_ICC_MINIMUM_MAXIMUM_CURRENT                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 10102
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_POWER_VCC_X_ICC_MINIMUM_MINIMUM_ALL                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 10103
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_POWER_VCC_X_ICC_MINIMUM_MINIMUM_CURRENT                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 10104
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_POWER_VCC_X_ICC_MINIMUM_STANDARD_DEVIATION_ALL              RS_SPECIFIC_PUBLIC_ATTR_BASE + 10105
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_POWER_VCC_X_ICC_MINIMUM_STANDARD_DEVIATION_CURRENT          RS_SPECIFIC_PUBLIC_ATTR_BASE + 10106
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_POWER_VCC_X_ICC_MINIMUM_SELECTED_RESULT                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 10107
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_CREST_FACTOR_INPUT_AVERAGE_ALL                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 10108
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_CREST_FACTOR_INPUT_AVERAGE_CURRENT                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 10109
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_CREST_FACTOR_INPUT_MAXIMUM_ALL                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 10110
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_CREST_FACTOR_INPUT_MAXIMUM_CURRENT                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 10111
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_CREST_FACTOR_INPUT_MINIMUM_ALL                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 10112
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_CREST_FACTOR_INPUT_MINIMUM_CURRENT                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 10113
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_CREST_FACTOR_INPUT_STANDARD_DEVIATION_ALL                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 10114
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_CREST_FACTOR_INPUT_STANDARD_DEVIATION_CURRENT               RS_SPECIFIC_PUBLIC_ATTR_BASE + 10115
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_CREST_FACTOR_INPUT_SELECTED_RESULT                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 10116
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_CREST_FACTOR_OUTPUT_AVERAGE_ALL                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 10117
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_CREST_FACTOR_OUTPUT_AVERAGE_CURRENT                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 10118
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_CREST_FACTOR_OUTPUT_MAXIMUM_ALL                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 10119
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_CREST_FACTOR_OUTPUT_MAXIMUM_CURRENT                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 10120
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_CREST_FACTOR_OUTPUT_MINIMUM_ALL                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 10121
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_CREST_FACTOR_OUTPUT_MINIMUM_CURRENT                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 10122
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_CREST_FACTOR_OUTPUT_STANDARD_DEVIATION_ALL                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 10123
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_CREST_FACTOR_OUTPUT_STANDARD_DEVIATION_CURRENT              RS_SPECIFIC_PUBLIC_ATTR_BASE + 10124
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_CREST_FACTOR_OUTPUT_SELECTED_RESULT                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 10125
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_FREQUENCY_ERROR_AVERAGE_ALL                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 10126
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_FREQUENCY_ERROR_AVERAGE_CURRENT                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 10127
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_FREQUENCY_ERROR_MAXIMUM_ALL                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 10128
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_FREQUENCY_ERROR_MAXIMUM_CURRENT                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 10129
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_FREQUENCY_ERROR_MINIMUM_ALL                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 10130
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_FREQUENCY_ERROR_MINIMUM_CURRENT                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 10131
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_FREQUENCY_ERROR_STANDARD_DEVIATION_ALL                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 10132
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_FREQUENCY_ERROR_STANDARD_DEVIATION_CURRENT                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 10133
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_FREQUENCY_ERROR_SELECTED_RESULT                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 10134
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_GAIN_AVERAGE_ALL                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 10135
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_GAIN_AVERAGE_CURRENT                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 10136
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_GAIN_MAXIMUM_ALL                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 10137
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_GAIN_MAXIMUM_CURRENT                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 10138
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_GAIN_MINIMUM_ALL                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 10139
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_GAIN_MINIMUM_CURRENT                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 10140
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_GAIN_STANDARD_DEVIATION_ALL                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 10141
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_GAIN_STANDARD_DEVIATION_CURRENT                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 10142
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_GAIN_SELECTED_RESULT                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 10143
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_GAIN_IMBALANCE_AVERAGE_ALL                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 10144
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_GAIN_IMBALANCE_AVERAGE_CURRENT                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 10145
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_GAIN_IMBALANCE_MAXIMUM_ALL                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 10146
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_GAIN_IMBALANCE_MAXIMUM_CURRENT                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 10147
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_GAIN_IMBALANCE_MINIMUM_ALL                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 10148
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_GAIN_IMBALANCE_MINIMUM_CURRENT                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 10149
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_GAIN_IMBALANCE_STANDARD_DEVIATION_ALL                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 10150
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_GAIN_IMBALANCE_STANDARD_DEVIATION_CURRENT                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 10151
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_GAIN_IMBALANCE_SELECTED_RESULT                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 10152
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_CURRENT_ICC_AVERAGE_AVERAGE_ALL                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 10153
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_CURRENT_ICC_AVERAGE_AVERAGE_CURRENT                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 10154
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_CURRENT_ICC_AVERAGE_MAXIMUM_ALL                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 10155
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_CURRENT_ICC_AVERAGE_MAXIMUM_CURRENT                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 10156
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_CURRENT_ICC_AVERAGE_MINIMUM_ALL                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 10157
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_CURRENT_ICC_AVERAGE_MINIMUM_CURRENT                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 10158
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_CURRENT_ICC_AVERAGE_STANDARD_DEVIATION_ALL                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 10159
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_CURRENT_ICC_AVERAGE_STANDARD_DEVIATION_CURRENT              RS_SPECIFIC_PUBLIC_ATTR_BASE + 10160
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_CURRENT_ICC_AVERAGE_SELECTED_RESULT                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 10161
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_CURRENT_ICC_MAXIMUM_AVERAGE_ALL                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 10162
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_CURRENT_ICC_MAXIMUM_AVERAGE_CURRENT                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 10163
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_CURRENT_ICC_MAXIMUM_MAXIMUM_ALL                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 10164
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_CURRENT_ICC_MAXIMUM_MAXIMUM_CURRENT                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 10165
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_CURRENT_ICC_MAXIMUM_MINIMUM_ALL                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 10166
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_CURRENT_ICC_MAXIMUM_MINIMUM_CURRENT                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 10167
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_CURRENT_ICC_MAXIMUM_STANDARD_DEVIATION_ALL                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 10168
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_CURRENT_ICC_MAXIMUM_STANDARD_DEVIATION_CURRENT              RS_SPECIFIC_PUBLIC_ATTR_BASE + 10169
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_CURRENT_ICC_MAXIMUM_SELECTED_RESULT                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 10170
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_CURRENT_ICC_MINIMUM_AVERAGE_ALL                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 10171
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_CURRENT_ICC_MINIMUM_AVERAGE_CURRENT                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 10172
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_CURRENT_ICC_MINIMUM_MAXIMUM_ALL                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 10173
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_CURRENT_ICC_MINIMUM_MAXIMUM_CURRENT                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 10174
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_CURRENT_ICC_MINIMUM_MINIMUM_ALL                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 10175
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_CURRENT_ICC_MINIMUM_MINIMUM_CURRENT                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 10176
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_CURRENT_ICC_MINIMUM_STANDARD_DEVIATION_ALL                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 10177
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_CURRENT_ICC_MINIMUM_STANDARD_DEVIATION_CURRENT              RS_SPECIFIC_PUBLIC_ATTR_BASE + 10178
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_CURRENT_ICC_MINIMUM_SELECTED_RESULT                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 10179
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_I_Q_IMBALANCE_AVERAGE_ALL                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 10180
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_I_Q_IMBALANCE_AVERAGE_CURRENT                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 10181
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_I_Q_IMBALANCE_MAXIMUM_ALL                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 10182
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_I_Q_IMBALANCE_MAXIMUM_CURRENT                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 10183
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_I_Q_IMBALANCE_MINIMUM_ALL                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 10184
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_I_Q_IMBALANCE_MINIMUM_CURRENT                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 10185
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_I_Q_IMBALANCE_STANDARD_DEVIATION_ALL                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 10186
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_I_Q_IMBALANCE_STANDARD_DEVIATION_CURRENT                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 10187
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_I_Q_IMBALANCE_SELECTED_RESULT                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 10188
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_I_Q_OFFSET_AVERAGE_ALL                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 10189
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_I_Q_OFFSET_AVERAGE_CURRENT                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 10190
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_I_Q_OFFSET_MAXIMUM_ALL                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 10191
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_I_Q_OFFSET_MAXIMUM_CURRENT                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 10192
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_I_Q_OFFSET_MINIMUM_ALL                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 10193
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_I_Q_OFFSET_MINIMUM_CURRENT                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 10194
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_I_Q_OFFSET_STANDARD_DEVIATION_ALL                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 10195
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_I_Q_OFFSET_STANDARD_DEVIATION_CURRENT                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 10196
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_I_Q_OFFSET_SELECTED_RESULT                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 10197
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_BASEBAND_I_INPUT_VOLTAGE_AVERAGE_AVERAGE_ALL                RS_SPECIFIC_PUBLIC_ATTR_BASE + 10198
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_BASEBAND_I_INPUT_VOLTAGE_AVERAGE_AVERAGE_CURRENT            RS_SPECIFIC_PUBLIC_ATTR_BASE + 10199
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_BASEBAND_I_INPUT_VOLTAGE_AVERAGE_MAXIMUM_ALL                RS_SPECIFIC_PUBLIC_ATTR_BASE + 10200
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_BASEBAND_I_INPUT_VOLTAGE_AVERAGE_MAXIMUM_CURRENT            RS_SPECIFIC_PUBLIC_ATTR_BASE + 10201
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_BASEBAND_I_INPUT_VOLTAGE_AVERAGE_MINIMUM_ALL                RS_SPECIFIC_PUBLIC_ATTR_BASE + 10202
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_BASEBAND_I_INPUT_VOLTAGE_AVERAGE_MINIMUM_CURRENT            RS_SPECIFIC_PUBLIC_ATTR_BASE + 10203
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_BASEBAND_I_INPUT_VOLTAGE_AVERAGE_STANDARD_DEVIATION_ALL     RS_SPECIFIC_PUBLIC_ATTR_BASE + 10204
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_BASEBAND_I_INPUT_VOLTAGE_AVERAGE_STANDARD_DEVIATION_CURRENT RS_SPECIFIC_PUBLIC_ATTR_BASE + 10205
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_BASEBAND_I_INPUT_VOLTAGE_AVERAGE_SELECTED_RESULT            RS_SPECIFIC_PUBLIC_ATTR_BASE + 10206
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_BASEBAND_I_INPUT_VOLTAGE_MAXIMUM_AVERAGE_ALL                RS_SPECIFIC_PUBLIC_ATTR_BASE + 10207
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_BASEBAND_I_INPUT_VOLTAGE_MAXIMUM_AVERAGE_CURRENT            RS_SPECIFIC_PUBLIC_ATTR_BASE + 10208
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_BASEBAND_I_INPUT_VOLTAGE_MAXIMUM_MAXIMUM_ALL                RS_SPECIFIC_PUBLIC_ATTR_BASE + 10209
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_BASEBAND_I_INPUT_VOLTAGE_MAXIMUM_MAXIMUM_CURRENT            RS_SPECIFIC_PUBLIC_ATTR_BASE + 10210
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_BASEBAND_I_INPUT_VOLTAGE_MAXIMUM_MINIMUM_ALL                RS_SPECIFIC_PUBLIC_ATTR_BASE + 10211
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_BASEBAND_I_INPUT_VOLTAGE_MAXIMUM_MINIMUM_CURRENT            RS_SPECIFIC_PUBLIC_ATTR_BASE + 10212
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_BASEBAND_I_INPUT_VOLTAGE_MAXIMUM_STANDARD_DEVIATION_ALL     RS_SPECIFIC_PUBLIC_ATTR_BASE + 10213
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_BASEBAND_I_INPUT_VOLTAGE_MAXIMUM_STANDARD_DEVIATION_CURRENT RS_SPECIFIC_PUBLIC_ATTR_BASE + 10214
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_BASEBAND_I_INPUT_VOLTAGE_MAXIMUM_SELECTED_RESULT            RS_SPECIFIC_PUBLIC_ATTR_BASE + 10215
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_BASEBAND_I_INPUT_VOLTAGE_MINIMUM_AVERAGE_ALL                RS_SPECIFIC_PUBLIC_ATTR_BASE + 10216
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_BASEBAND_I_INPUT_VOLTAGE_MINIMUM_AVERAGE_CURRENT            RS_SPECIFIC_PUBLIC_ATTR_BASE + 10217
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_BASEBAND_I_INPUT_VOLTAGE_MINIMUM_MAXIMUM_ALL                RS_SPECIFIC_PUBLIC_ATTR_BASE + 10218
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_BASEBAND_I_INPUT_VOLTAGE_MINIMUM_MAXIMUM_CURRENT            RS_SPECIFIC_PUBLIC_ATTR_BASE + 10219
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_BASEBAND_I_INPUT_VOLTAGE_MINIMUM_MINIMUM_ALL                RS_SPECIFIC_PUBLIC_ATTR_BASE + 10220
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_BASEBAND_I_INPUT_VOLTAGE_MINIMUM_MINIMUM_CURRENT            RS_SPECIFIC_PUBLIC_ATTR_BASE + 10221
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_BASEBAND_I_INPUT_VOLTAGE_MINIMUM_STANDARD_DEVIATION_ALL     RS_SPECIFIC_PUBLIC_ATTR_BASE + 10222
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_BASEBAND_I_INPUT_VOLTAGE_MINIMUM_STANDARD_DEVIATION_CURRENT RS_SPECIFIC_PUBLIC_ATTR_BASE + 10223
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_BASEBAND_I_INPUT_VOLTAGE_MINIMUM_SELECTED_RESULT            RS_SPECIFIC_PUBLIC_ATTR_BASE + 10224
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_MAGNITUDE_ERROR_AVERAGE_ALL                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 10225
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_MAGNITUDE_ERROR_AVERAGE_CURRENT                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 10226
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_MAGNITUDE_ERROR_MAXIMUM_ALL                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 10227
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_MAGNITUDE_ERROR_MAXIMUM_CURRENT                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 10228
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_MAGNITUDE_ERROR_MINIMUM_ALL                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 10229
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_MAGNITUDE_ERROR_MINIMUM_CURRENT                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 10230
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_MAGNITUDE_ERROR_STANDARD_DEVIATION_ALL                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 10231
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_MAGNITUDE_ERROR_STANDARD_DEVIATION_CURRENT                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 10232
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_MAGNITUDE_ERROR_SELECTED_RESULT                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 10233
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_P1DB_INPUT_AVERAGE_ALL                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 10234
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_P1DB_INPUT_AVERAGE_CURRENT                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 10235
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_P1DB_INPUT_MAXIMUM_ALL                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 10236
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_P1DB_INPUT_MAXIMUM_CURRENT                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 10237
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_P1DB_INPUT_MINIMUM_ALL                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 10238
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_P1DB_INPUT_MINIMUM_CURRENT                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 10239
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_P1DB_INPUT_STANDARD_DEVIATION_ALL                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 10240
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_P1DB_INPUT_STANDARD_DEVIATION_CURRENT                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 10241
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_P1DB_INPUT_SELECTED_RESULT                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 10242
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_P1DB_OUTPUT_AVERAGE_ALL                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 10243
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_P1DB_OUTPUT_AVERAGE_CURRENT                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 10244
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_P1DB_OUTPUT_MAXIMUM_ALL                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 10245
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_P1DB_OUTPUT_MAXIMUM_CURRENT                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 10246
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_P1DB_OUTPUT_MINIMUM_ALL                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 10247
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_P1DB_OUTPUT_MINIMUM_CURRENT                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 10248
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_P1DB_OUTPUT_STANDARD_DEVIATION_ALL                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 10249
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_P1DB_OUTPUT_STANDARD_DEVIATION_CURRENT                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 10250
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_P1DB_OUTPUT_SELECTED_RESULT                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 10251
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_P2DB_INPUT_AVERAGE_ALL                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 10252
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_P2DB_INPUT_AVERAGE_CURRENT                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 10253
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_P2DB_INPUT_MAXIMUM_ALL                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 10254
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_P2DB_INPUT_MAXIMUM_CURRENT                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 10255
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_P2DB_INPUT_MINIMUM_ALL                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 10256
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_P2DB_INPUT_MINIMUM_CURRENT                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 10257
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_P2DB_INPUT_STANDARD_DEVIATION_ALL                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 10258
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_P2DB_INPUT_STANDARD_DEVIATION_CURRENT                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 10259
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_P2DB_INPUT_SELECTED_RESULT                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 10260
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_P2DB_OUTPUT_AVERAGE_ALL                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 10261
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_P2DB_OUTPUT_AVERAGE_CURRENT                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 10262
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_P2DB_OUTPUT_MAXIMUM_ALL                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 10263
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_P2DB_OUTPUT_MAXIMUM_CURRENT                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 10264
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_P2DB_OUTPUT_MINIMUM_ALL                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 10265
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_P2DB_OUTPUT_MINIMUM_CURRENT                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 10266
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_P2DB_OUTPUT_STANDARD_DEVIATION_ALL                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 10267
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_P2DB_OUTPUT_STANDARD_DEVIATION_CURRENT                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 10268
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_P2DB_OUTPUT_SELECTED_RESULT                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 10269
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_P3DB_INPUT_AVERAGE_ALL                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 10270
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_P3DB_INPUT_AVERAGE_CURRENT                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 10271
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_P3DB_INPUT_MAXIMUM_ALL                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 10272
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_P3DB_INPUT_MAXIMUM_CURRENT                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 10273
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_P3DB_INPUT_MINIMUM_ALL                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 10274
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_P3DB_INPUT_MINIMUM_CURRENT                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 10275
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_P3DB_INPUT_STANDARD_DEVIATION_ALL                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 10276
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_P3DB_INPUT_STANDARD_DEVIATION_CURRENT                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 10277
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_P3DB_INPUT_SELECTED_RESULT                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 10278
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_P3DB_OUTPUT_AVERAGE_ALL                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 10279
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_P3DB_OUTPUT_AVERAGE_CURRENT                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 10280
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_P3DB_OUTPUT_MAXIMUM_ALL                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 10281
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_P3DB_OUTPUT_MAXIMUM_CURRENT                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 10282
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_P3DB_OUTPUT_MINIMUM_ALL                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 10283
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_P3DB_OUTPUT_MINIMUM_CURRENT                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 10284
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_P3DB_OUTPUT_STANDARD_DEVIATION_ALL                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 10285
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_P3DB_OUTPUT_STANDARD_DEVIATION_CURRENT                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 10286
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_P3DB_OUTPUT_SELECTED_RESULT                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 10287
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_AVERAGE_POWER_CONSUMPTION_AVERAGE_ALL                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 10288
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_AVERAGE_POWER_CONSUMPTION_AVERAGE_CURRENT                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 10289
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_AVERAGE_POWER_CONSUMPTION_MAXIMUM_ALL                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 10290
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_AVERAGE_POWER_CONSUMPTION_MAXIMUM_CURRENT                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 10291
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_AVERAGE_POWER_CONSUMPTION_MINIMUM_ALL                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 10292
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_AVERAGE_POWER_CONSUMPTION_MINIMUM_CURRENT                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 10293
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_AVERAGE_POWER_CONSUMPTION_STANDARD_DEVIATION_ALL            RS_SPECIFIC_PUBLIC_ATTR_BASE + 10294
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_AVERAGE_POWER_CONSUMPTION_STANDARD_DEVIATION_CURRENT        RS_SPECIFIC_PUBLIC_ATTR_BASE + 10295
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_AVERAGE_POWER_CONSUMPTION_SELECTED_RESULT                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 10296
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_PC_BASED_AVERAGE_PAE_AVERAGE_ALL                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 10297
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_PC_BASED_AVERAGE_PAE_AVERAGE_CURRENT                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 10298
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_PC_BASED_AVERAGE_PAE_MAXIMUM_ALL                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 10299
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_PC_BASED_AVERAGE_PAE_MAXIMUM_CURRENT                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 10300
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_PC_BASED_AVERAGE_PAE_MINIMUM_ALL                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 10301
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_PC_BASED_AVERAGE_PAE_MINIMUM_CURRENT                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 10302
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_PC_BASED_AVERAGE_PAE_STANDARD_DEVIATION_ALL                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 10303
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_PC_BASED_AVERAGE_PAE_STANDARD_DEVIATION_CURRENT             RS_SPECIFIC_PUBLIC_ATTR_BASE + 10304
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_PC_BASED_AVERAGE_PAE_SELECTED_RESULT                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 10305
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_PHASE_ERROR_AVERAGE_ALL                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 10306
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_PHASE_ERROR_AVERAGE_CURRENT                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 10307
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_PHASE_ERROR_MAXIMUM_ALL                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 10308
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_PHASE_ERROR_MAXIMUM_CURRENT                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 10309
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_PHASE_ERROR_MINIMUM_ALL                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 10310
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_PHASE_ERROR_MINIMUM_CURRENT                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 10311
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_PHASE_ERROR_STANDARD_DEVIATION_ALL                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 10312
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_PHASE_ERROR_STANDARD_DEVIATION_CURRENT                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 10313
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_PHASE_ERROR_SELECTED_RESULT                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 10314
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_POWER_INPUT_AVERAGE_AVERAGE_ALL                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 10315
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_POWER_INPUT_AVERAGE_AVERAGE_CURRENT                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 10316
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_POWER_INPUT_AVERAGE_MAXIMUM_ALL                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 10317
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_POWER_INPUT_AVERAGE_MAXIMUM_CURRENT                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 10318
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_POWER_INPUT_AVERAGE_MINIMUM_ALL                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 10319
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_POWER_INPUT_AVERAGE_MINIMUM_CURRENT                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 10320
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_POWER_INPUT_AVERAGE_STANDARD_DEVIATION_ALL                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 10321
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_POWER_INPUT_AVERAGE_STANDARD_DEVIATION_CURRENT              RS_SPECIFIC_PUBLIC_ATTR_BASE + 10322
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_POWER_INPUT_AVERAGE_SELECTED_RESULT                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 10323
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_POWER_INPUT_MAXIMUM_AVERAGE_ALL                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 10324
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_POWER_INPUT_MAXIMUM_AVERAGE_CURRENT                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 10325
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_POWER_INPUT_MAXIMUM_MAXIMUM_ALL                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 10326
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_POWER_INPUT_MAXIMUM_MAXIMUM_CURRENT                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 10327
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_POWER_INPUT_MAXIMUM_MINIMUM_ALL                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 10328
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_POWER_INPUT_MAXIMUM_MINIMUM_CURRENT                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 10329
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_POWER_INPUT_MAXIMUM_STANDARD_DEVIATION_ALL                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 10330
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_POWER_INPUT_MAXIMUM_STANDARD_DEVIATION_CURRENT              RS_SPECIFIC_PUBLIC_ATTR_BASE + 10331
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_POWER_INPUT_MAXIMUM_SELECTED_RESULT                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 10332
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_POWER_INPUT_MINIMUM_AVERAGE_ALL                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 10333
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_POWER_INPUT_MINIMUM_AVERAGE_CURRENT                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 10334
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_POWER_INPUT_MINIMUM_MAXIMUM_ALL                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 10335
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_POWER_INPUT_MINIMUM_MAXIMUM_CURRENT                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 10336
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_POWER_INPUT_MINIMUM_MINIMUM_ALL                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 10337
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_POWER_INPUT_MINIMUM_MINIMUM_CURRENT                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 10338
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_POWER_INPUT_MINIMUM_STANDARD_DEVIATION_ALL                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 10339
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_POWER_INPUT_MINIMUM_STANDARD_DEVIATION_CURRENT              RS_SPECIFIC_PUBLIC_ATTR_BASE + 10340
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_POWER_INPUT_MINIMUM_SELECTED_RESULT                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 10341
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_POWER_OUTPUT_AVERAGE_AVERAGE_ALL                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 10342
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_POWER_OUTPUT_AVERAGE_AVERAGE_CURRENT                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 10343
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_POWER_OUTPUT_AVERAGE_MAXIMUM_ALL                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 10344
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_POWER_OUTPUT_AVERAGE_MAXIMUM_CURRENT                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 10345
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_POWER_OUTPUT_AVERAGE_MINIMUM_ALL                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 10346
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_POWER_OUTPUT_AVERAGE_MINIMUM_CURRENT                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 10347
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_POWER_OUTPUT_AVERAGE_STANDARD_DEVIATION_ALL                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 10348
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_POWER_OUTPUT_AVERAGE_STANDARD_DEVIATION_CURRENT             RS_SPECIFIC_PUBLIC_ATTR_BASE + 10349
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_POWER_OUTPUT_AVERAGE_SELECTED_RESULT                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 10350
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_POWER_OUTPUT_MAXIMUM_AVERAGE_ALL                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 10351
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_POWER_OUTPUT_MAXIMUM_AVERAGE_CURRENT                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 10352
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_POWER_OUTPUT_MAXIMUM_MAXIMUM_ALL                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 10353
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_POWER_OUTPUT_MAXIMUM_MAXIMUM_CURRENT                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 10354
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_POWER_OUTPUT_MAXIMUM_MINIMUM_ALL                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 10355
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_POWER_OUTPUT_MAXIMUM_MINIMUM_CURRENT                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 10356
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_POWER_OUTPUT_MAXIMUM_STANDARD_DEVIATION_ALL                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 10357
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_POWER_OUTPUT_MAXIMUM_STANDARD_DEVIATION_CURRENT             RS_SPECIFIC_PUBLIC_ATTR_BASE + 10358
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_POWER_OUTPUT_MAXIMUM_SELECTED_RESULT                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 10359
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_POWER_OUTPUT_MINIMUM_AVERAGE_ALL                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 10360
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_POWER_OUTPUT_MINIMUM_AVERAGE_CURRENT                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 10361
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_POWER_OUTPUT_MINIMUM_MAXIMUM_ALL                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 10362
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_POWER_OUTPUT_MINIMUM_MAXIMUM_CURRENT                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 10363
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_POWER_OUTPUT_MINIMUM_MINIMUM_ALL                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 10364
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_POWER_OUTPUT_MINIMUM_MINIMUM_CURRENT                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 10365
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_POWER_OUTPUT_MINIMUM_STANDARD_DEVIATION_ALL                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 10366
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_POWER_OUTPUT_MINIMUM_STANDARD_DEVIATION_CURRENT             RS_SPECIFIC_PUBLIC_ATTR_BASE + 10367
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_POWER_OUTPUT_MINIMUM_SELECTED_RESULT                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 10368
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_QUADRATURE_ERROR_AVERAGE_ALL                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 10369
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_QUADRATURE_ERROR_AVERAGE_CURRENT                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 10370
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_QUADRATURE_ERROR_MAXIMUM_ALL                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 10371
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_QUADRATURE_ERROR_MAXIMUM_CURRENT                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 10372
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_QUADRATURE_ERROR_MINIMUM_ALL                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 10373
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_QUADRATURE_ERROR_MINIMUM_CURRENT                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 10374
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_QUADRATURE_ERROR_STANDARD_DEVIATION_ALL                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 10375
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_QUADRATURE_ERROR_STANDARD_DEVIATION_CURRENT                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 10376
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_QUADRATURE_ERROR_SELECTED_RESULT                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 10377
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_BASEBAND_Q_INPUT_VOLTAGE_AVERAGE_AVERAGE_ALL                RS_SPECIFIC_PUBLIC_ATTR_BASE + 10378
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_BASEBAND_Q_INPUT_VOLTAGE_AVERAGE_AVERAGE_CURRENT            RS_SPECIFIC_PUBLIC_ATTR_BASE + 10379
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_BASEBAND_Q_INPUT_VOLTAGE_AVERAGE_MAXIMUM_ALL                RS_SPECIFIC_PUBLIC_ATTR_BASE + 10380
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_BASEBAND_Q_INPUT_VOLTAGE_AVERAGE_MAXIMUM_CURRENT            RS_SPECIFIC_PUBLIC_ATTR_BASE + 10381
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_BASEBAND_Q_INPUT_VOLTAGE_AVERAGE_MINIMUM_ALL                RS_SPECIFIC_PUBLIC_ATTR_BASE + 10382
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_BASEBAND_Q_INPUT_VOLTAGE_AVERAGE_MINIMUM_CURRENT            RS_SPECIFIC_PUBLIC_ATTR_BASE + 10383
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_BASEBAND_Q_INPUT_VOLTAGE_AVERAGE_STANDARD_DEVIATION_ALL     RS_SPECIFIC_PUBLIC_ATTR_BASE + 10384
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_BASEBAND_Q_INPUT_VOLTAGE_AVERAGE_STANDARD_DEVIATION_CURRENT RS_SPECIFIC_PUBLIC_ATTR_BASE + 10385
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_BASEBAND_Q_INPUT_VOLTAGE_AVERAGE_SELECTED_RESULT            RS_SPECIFIC_PUBLIC_ATTR_BASE + 10386
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_BASEBAND_Q_INPUT_VOLTAGE_MAXIMUM_AVERAGE_ALL                RS_SPECIFIC_PUBLIC_ATTR_BASE + 10387
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_BASEBAND_Q_INPUT_VOLTAGE_MAXIMUM_AVERAGE_CURRENT            RS_SPECIFIC_PUBLIC_ATTR_BASE + 10388
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_BASEBAND_Q_INPUT_VOLTAGE_MAXIMUM_MAXIMUM_ALL                RS_SPECIFIC_PUBLIC_ATTR_BASE + 10389
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_BASEBAND_Q_INPUT_VOLTAGE_MAXIMUM_MAXIMUM_CURRENT            RS_SPECIFIC_PUBLIC_ATTR_BASE + 10390
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_BASEBAND_Q_INPUT_VOLTAGE_MAXIMUM_MINIMUM_ALL                RS_SPECIFIC_PUBLIC_ATTR_BASE + 10391
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_BASEBAND_Q_INPUT_VOLTAGE_MAXIMUM_MINIMUM_CURRENT            RS_SPECIFIC_PUBLIC_ATTR_BASE + 10392
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_BASEBAND_Q_INPUT_VOLTAGE_MAXIMUM_STANDARD_DEVIATION_ALL     RS_SPECIFIC_PUBLIC_ATTR_BASE + 10393
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_BASEBAND_Q_INPUT_VOLTAGE_MAXIMUM_STANDARD_DEVIATION_CURRENT RS_SPECIFIC_PUBLIC_ATTR_BASE + 10394
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_BASEBAND_Q_INPUT_VOLTAGE_MAXIMUM_SELECTED_RESULT            RS_SPECIFIC_PUBLIC_ATTR_BASE + 10395
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_BASEBAND_Q_INPUT_VOLTAGE_MINIMUM_AVERAGE_ALL                RS_SPECIFIC_PUBLIC_ATTR_BASE + 10396
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_BASEBAND_Q_INPUT_VOLTAGE_MINIMUM_AVERAGE_CURRENT            RS_SPECIFIC_PUBLIC_ATTR_BASE + 10397
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_BASEBAND_Q_INPUT_VOLTAGE_MINIMUM_MAXIMUM_ALL                RS_SPECIFIC_PUBLIC_ATTR_BASE + 10398
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_BASEBAND_Q_INPUT_VOLTAGE_MINIMUM_MAXIMUM_CURRENT            RS_SPECIFIC_PUBLIC_ATTR_BASE + 10399
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_BASEBAND_Q_INPUT_VOLTAGE_MINIMUM_MINIMUM_ALL                RS_SPECIFIC_PUBLIC_ATTR_BASE + 10400
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_BASEBAND_Q_INPUT_VOLTAGE_MINIMUM_MINIMUM_CURRENT            RS_SPECIFIC_PUBLIC_ATTR_BASE + 10401
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_BASEBAND_Q_INPUT_VOLTAGE_MINIMUM_STANDARD_DEVIATION_ALL     RS_SPECIFIC_PUBLIC_ATTR_BASE + 10402
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_BASEBAND_Q_INPUT_VOLTAGE_MINIMUM_STANDARD_DEVIATION_CURRENT RS_SPECIFIC_PUBLIC_ATTR_BASE + 10403
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_BASEBAND_Q_INPUT_VOLTAGE_MINIMUM_SELECTED_RESULT            RS_SPECIFIC_PUBLIC_ATTR_BASE + 10404
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_RAW_EVM_AVERAGE_AVERAGE_ALL                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 10405
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_RAW_EVM_AVERAGE_AVERAGE_CURRENT                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 10406
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_RAW_EVM_AVERAGE_MAXIMUM_ALL                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 10407
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_RAW_EVM_AVERAGE_MAXIMUM_CURRENT                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 10408
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_RAW_EVM_AVERAGE_MINIMUM_ALL                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 10409
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_RAW_EVM_AVERAGE_MINIMUM_CURRENT                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 10410
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_RAW_EVM_AVERAGE_STANDARD_DEVIATION_ALL                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 10411
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_RAW_EVM_AVERAGE_STANDARD_DEVIATION_CURRENT                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 10412
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_RAW_EVM_AVERAGE_SELECTED_RESULT                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 10413
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_RAW_EVM_MAXIMUM_AVERAGE_ALL                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 10414
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_RAW_EVM_MAXIMUM_AVERAGE_CURRENT                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 10415
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_RAW_EVM_MAXIMUM_MAXIMUM_ALL                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 10416
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_RAW_EVM_MAXIMUM_MAXIMUM_CURRENT                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 10417
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_RAW_EVM_MAXIMUM_MINIMUM_ALL                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 10418
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_RAW_EVM_MAXIMUM_MINIMUM_CURRENT                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 10419
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_RAW_EVM_MAXIMUM_STANDARD_DEVIATION_ALL                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 10420
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_RAW_EVM_MAXIMUM_STANDARD_DEVIATION_CURRENT                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 10421
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_RAW_EVM_MAXIMUM_SELECTED_RESULT                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 10422
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_RAW_EVM_MINIMUM_AVERAGE_ALL                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 10423
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_RAW_EVM_MINIMUM_AVERAGE_CURRENT                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 10424
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_RAW_EVM_MINIMUM_MAXIMUM_ALL                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 10425
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_RAW_EVM_MINIMUM_MAXIMUM_CURRENT                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 10426
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_RAW_EVM_MINIMUM_MINIMUM_ALL                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 10427
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_RAW_EVM_MINIMUM_MINIMUM_CURRENT                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 10428
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_RAW_EVM_MINIMUM_STANDARD_DEVIATION_ALL                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 10429
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_RAW_EVM_MINIMUM_STANDARD_DEVIATION_CURRENT                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 10430
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_RAW_EVM_MINIMUM_SELECTED_RESULT                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 10431
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_RAW_MODEL_EVM_AVERAGE_AVERAGE_ALL                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 10432
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_RAW_MODEL_EVM_AVERAGE_AVERAGE_CURRENT                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 10433
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_RAW_MODEL_EVM_AVERAGE_MAXIMUM_ALL                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 10434
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_RAW_MODEL_EVM_AVERAGE_MAXIMUM_CURRENT                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 10435
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_RAW_MODEL_EVM_AVERAGE_MINIMUM_ALL                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 10436
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_RAW_MODEL_EVM_AVERAGE_MINIMUM_CURRENT                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 10437
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_RAW_MODEL_EVM_AVERAGE_STANDARD_DEVIATION_ALL                RS_SPECIFIC_PUBLIC_ATTR_BASE + 10438
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_RAW_MODEL_EVM_AVERAGE_STANDARD_DEVIATION_CURRENT            RS_SPECIFIC_PUBLIC_ATTR_BASE + 10439
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_RAW_MODEL_EVM_AVERAGE_SELECTED_RESULT                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 10440
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_RAW_MODEL_EVM_MAXIMUM_AVERAGE_ALL                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 10441
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_RAW_MODEL_EVM_MAXIMUM_AVERAGE_CURRENT                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 10442
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_RAW_MODEL_EVM_MAXIMUM_MAXIMUM_ALL                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 10443
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_RAW_MODEL_EVM_MAXIMUM_MAXIMUM_CURRENT                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 10444
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_RAW_MODEL_EVM_MAXIMUM_MINIMUM_ALL                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 10445
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_RAW_MODEL_EVM_MAXIMUM_MINIMUM_CURRENT                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 10446
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_RAW_MODEL_EVM_MAXIMUM_STANDARD_DEVIATION_ALL                RS_SPECIFIC_PUBLIC_ATTR_BASE + 10447
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_RAW_MODEL_EVM_MAXIMUM_STANDARD_DEVIATION_CURRENT            RS_SPECIFIC_PUBLIC_ATTR_BASE + 10448
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_RAW_MODEL_EVM_MAXIMUM_SELECTED_RESULT                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 10449
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_RAW_MODEL_EVM_MINIMUM_AVERAGE_ALL                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 10450
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_RAW_MODEL_EVM_MINIMUM_AVERAGE_CURRENT                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 10451
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_RAW_MODEL_EVM_MINIMUM_MAXIMUM_ALL                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 10452
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_RAW_MODEL_EVM_MINIMUM_MAXIMUM_CURRENT                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 10453
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_RAW_MODEL_EVM_MINIMUM_MINIMUM_ALL                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 10454
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_RAW_MODEL_EVM_MINIMUM_MINIMUM_CURRENT                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 10455
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_RAW_MODEL_EVM_MINIMUM_STANDARD_DEVIATION_ALL                RS_SPECIFIC_PUBLIC_ATTR_BASE + 10456
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_RAW_MODEL_EVM_MINIMUM_STANDARD_DEVIATION_CURRENT            RS_SPECIFIC_PUBLIC_ATTR_BASE + 10457
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_RAW_MODEL_EVM_MINIMUM_SELECTED_RESULT                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 10458
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_SAMPLE_RATE_ERROR_AVERAGE_ALL                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 10459
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_SAMPLE_RATE_ERROR_AVERAGE_CURRENT                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 10460
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_SAMPLE_RATE_ERROR_MAXIMUM_ALL                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 10461
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_SAMPLE_RATE_ERROR_MAXIMUM_CURRENT                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 10462
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_SAMPLE_RATE_ERROR_MINIMUM_ALL                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 10463
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_SAMPLE_RATE_ERROR_MINIMUM_CURRENT                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 10464
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_SAMPLE_RATE_ERROR_STANDARD_DEVIATION_ALL                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 10465
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_SAMPLE_RATE_ERROR_STANDARD_DEVIATION_CURRENT                RS_SPECIFIC_PUBLIC_ATTR_BASE + 10466
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_SAMPLE_RATE_ERROR_SELECTED_RESULT                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 10467
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_VOLTAGE_VCC_AVERAGE_AVERAGE_ALL                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 10468
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_VOLTAGE_VCC_AVERAGE_AVERAGE_CURRENT                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 10469
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_VOLTAGE_VCC_AVERAGE_MAXIMUM_ALL                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 10470
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_VOLTAGE_VCC_AVERAGE_MAXIMUM_CURRENT                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 10471
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_VOLTAGE_VCC_AVERAGE_MINIMUM_ALL                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 10472
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_VOLTAGE_VCC_AVERAGE_MINIMUM_CURRENT                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 10473
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_VOLTAGE_VCC_AVERAGE_STANDARD_DEVIATION_ALL                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 10474
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_VOLTAGE_VCC_AVERAGE_STANDARD_DEVIATION_CURRENT              RS_SPECIFIC_PUBLIC_ATTR_BASE + 10475
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_VOLTAGE_VCC_AVERAGE_SELECTED_RESULT                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 10476
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_VOLTAGE_VCC_MAXIMUM_AVERAGE_ALL                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 10477
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_VOLTAGE_VCC_MAXIMUM_AVERAGE_CURRENT                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 10478
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_VOLTAGE_VCC_MAXIMUM_MAXIMUM_ALL                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 10479
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_VOLTAGE_VCC_MAXIMUM_MAXIMUM_CURRENT                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 10480
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_VOLTAGE_VCC_MAXIMUM_MINIMUM_ALL                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 10481
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_VOLTAGE_VCC_MAXIMUM_MINIMUM_CURRENT                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 10482
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_VOLTAGE_VCC_MAXIMUM_STANDARD_DEVIATION_ALL                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 10483
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_VOLTAGE_VCC_MAXIMUM_STANDARD_DEVIATION_CURRENT              RS_SPECIFIC_PUBLIC_ATTR_BASE + 10484
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_VOLTAGE_VCC_MAXIMUM_SELECTED_RESULT                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 10485
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_VOLTAGE_VCC_MINIMUM_AVERAGE_ALL                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 10486
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_VOLTAGE_VCC_MINIMUM_AVERAGE_CURRENT                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 10487
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_VOLTAGE_VCC_MINIMUM_MAXIMUM_ALL                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 10488
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_VOLTAGE_VCC_MINIMUM_MAXIMUM_CURRENT                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 10489
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_VOLTAGE_VCC_MINIMUM_MINIMUM_ALL                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 10490
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_VOLTAGE_VCC_MINIMUM_MINIMUM_CURRENT                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 10491
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_VOLTAGE_VCC_MINIMUM_STANDARD_DEVIATION_ALL                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 10492
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_VOLTAGE_VCC_MINIMUM_STANDARD_DEVIATION_CURRENT              RS_SPECIFIC_PUBLIC_ATTR_BASE + 10493
#define RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_VOLTAGE_VCC_MINIMUM_SELECTED_RESULT                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 10494
#define RSSPECAN_ATTR_EMI_FMEAS_STATE                                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 2605
#define RSSPECAN_ATTR_EMI_FMEAS_DWELL_TIME                                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 2608
#define RSSPECAN_ATTR_EMI_FMEAS_MARKER_PEAK_SEARCH                                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 2607
#define RSSPECAN_ATTR_EMI_FMEAS_DELTA_MARKER_PEAK_SEARCH                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 2779
#define RSSPECAN_ATTR_EMI_RESOLUTION_BANDWIDTH_FILTER_TYPE                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 2619
#define RSSPECAN_ATTR_EMI_FMEAS_MARKER_DETECTOR                                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 2606
#define RSSPECAN_ATTR_EMI_FMEAS_MARKER_RESULT                                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 2611
#define RSSPECAN_ATTR_EMI_FINAL_MEASUREMENT_MARKER_DWELL_TIME                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 7728
#define RSSPECAN_ATTR_EMI_FMEAS_MARKER_LIMIT_CONDITION                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 2609
#define RSSPECAN_ATTR_EMI_FMEAS_MARKER_LIMIT_VERTICAL_DISTANCE                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 2610
#define RSSPECAN_ATTR_EMI_FMEAS_DELTA_MARKER_DETECTOR                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 2612
#define RSSPECAN_ATTR_EMI_FMEAS_DELTA_MARKER_RESULT                                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 2615
#define RSSPECAN_ATTR_EMI_FINAL_MEASUREMENT_DELTA_MARKER_DWELL_TIME                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 7729
#define RSSPECAN_ATTR_EMI_FMEAS_DELTA_MARKER_LIMIT_CONDITION                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 2613
#define RSSPECAN_ATTR_EMI_FMEAS_DELTA_MARKER_LIMIT_VERTICAL_DISTANCE                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 2614
#define RSSPECAN_ATTR_EMI_LISN_V_NETWORK_TYPE                                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 2616
#define RSSPECAN_ATTR_EMI_LISN_V_NETWORK_HIGH_PASS_FILTER                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 2617
#define RSSPECAN_ATTR_EMI_LISN_V_NETWORK_PHASE                                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 2618
#define RSSPECAN_ATTR_PULSE_MODE                                                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 2800
#define RSSPECAN_ATTR_PULSE_SIGNAL_PERIOD                                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 2802
#define RSSPECAN_ATTR_PULSE_SIGNAL_DROOP_ENABLED                                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 2803
#define RSSPECAN_ATTR_PULSE_SIGNAL_MODULATION                                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 2804
#define RSSPECAN_ATTR_PULSE_SIGNAL_TIMING_AUTO_MODE                                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 2805
#define RSSPECAN_ATTR_PULSE_SIGNAL_TIMING_WIDTH_MAXIMUM                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 4690
#define RSSPECAN_ATTR_PULSE_SIGNAL_TIMING_WIDTH_MINIMUM                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 3160
#define RSSPECAN_ATTR_PULSE_SIGNAL_TIMING_OFF_TIME                                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 2807
#define RSSPECAN_ATTR_PULSE_SIGNAL_OFFSET_AUTO_MODE                                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 2808
#define RSSPECAN_ATTR_PULSE_SIGNAL_FREQUENCY_OFFSET                                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 2809
#define RSSPECAN_ATTR_PULSE_SIGNAL_CHIRP_RATE_AUTO_MODE                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 2810
#define RSSPECAN_ATTR_PULSE_SIGNAL_CHIRP_RATE                                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 2811
#define RSSPECAN_ATTR_PULSE_REFERENCE_IQ_TYPE                                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 5739
#define RSSPECAN_ATTR_PULSE_REFERENCE_IQ_INPUT_FILE                                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 5740
#define RSSPECAN_ATTR_PULSE_REFERENCE_IQ_RANGE_AUTO                                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 5741
#define RSSPECAN_ATTR_PULSE_REFERENCE_IQ_RANGE_OFFSET                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 5742
#define RSSPECAN_ATTR_PULSE_REFERENCE_IQ_RANGE_LENGTH                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 5743
#define RSSPECAN_ATTR_PULSE_REFERENCE_IQ_WINDOW                                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 6118
#define RSSPECAN_ATTR_PULSE_REFERENCE_IQ_POLYNOMIAL_PHASE_WINDOW                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 6119
#define RSSPECAN_ATTR_PULSE_REFERENCE_IQ_POLYNOMIAL_PHASE_WIDTH                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 6120
#define RSSPECAN_ATTR_PULSE_REFERENCE_IQ_BARKER_WIDTH                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 6122
#define RSSPECAN_ATTR_PULSE_REFERENCE_IQ_BARKER_PRIMARY_CODE                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 6123
#define RSSPECAN_ATTR_PULSE_REFERENCE_IQ_EMBEDED_BARKER_PRIMARY_CODE                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 6124
#define RSSPECAN_ATTR_PULSE_REFERENCE_IQ_EMBEDED_BARKER_SECONDARY_CODE                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 6125
#define RSSPECAN_ATTR_PULSE_REFERENCE_IQ_EMBEDED_BARKER_WIDTH                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 6126
#define RSSPECAN_ATTR_PULSE_INPUT_SIGNAL_SOURCE                                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 2827
#define RSSPECAN_ATTR_PULSE_SELECT_IQ_DATA_FILE                                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 3070
#define RSSPECAN_ATTR_PULSE_SIGNAL_OUTPUT                                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 10713
#define RSSPECAN_ATTR_PULSE_BURST_MODE                                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 10714
#define RSSPECAN_ATTR_PULSE_BURST_COUNT                                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 10716
#define RSSPECAN_ATTR_PULSE_BURST_TABLE_DELETE                                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 10717
#define RSSPECAN_ATTR_PULSE_BURST_TABLE_SAVE                                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 10718
#define RSSPECAN_ATTR_PULSE_BURST_TABLE_LOAD                                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 10719
#define RSSPECAN_ATTR_PULSE_BURST_INTERNAL_TRIGGER_POSITION                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 10720
#define RSSPECAN_ATTR_PULSE_BURST_INTERNAL_TRIGGER_BURST_NUMBER                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 10721
#define RSSPECAN_ATTR_PULSE_SEGMENTED_DATA_CAPTURING_ENABLED                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 5149
#define RSSPECAN_ATTR_PULSE_SEGMENTED_DATA_CAPTURING_EVENTS                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 5150
#define RSSPECAN_ATTR_PULSE_SEGMENTED_DATA_CAPTURING_LENGTH                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 5151
#define RSSPECAN_ATTR_PULSE_SEGMENTED_DATA_CAPTURING_OFFSET                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 5152
#define RSSPECAN_ATTR_PULSE_ACQUISITION_FILTER_TYPE                                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 2828
#define RSSPECAN_ATTR_PULSE_ACQUISITION_BANDWIDTH                                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 2829
#define RSSPECAN_ATTR_PULSE_ACQUISITION_DIGITIZER_SELECTION                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 10722
#define RSSPECAN_ATTR_PULSE_ACQUISITION_DIGITIZER_LOW_NOISE_MODE                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 10723
#define RSSPECAN_ATTR_PULSE_DETECTION_ENABLED                                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 7730
#define RSSPECAN_ATTR_PULSE_DETECTION_START                                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 7731
#define RSSPECAN_ATTR_PULSE_DETECTION_LENGTH                                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 7732
#define RSSPECAN_ATTR_PULSE_DETECTION_REFERENCE_SOURCE                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 2830
#define RSSPECAN_ATTR_PULSE_DETECTION_THRESHOLD                                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 2831
#define RSSPECAN_ATTR_PULSE_DETECTION_HYSTERESIS                                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 2832
#define RSSPECAN_ATTR_PULSE_DETECTION_LIMIT_ENABLED                                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 2833
#define RSSPECAN_ATTR_PULSE_DETECTION_PULSE_COUNT                                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 2834
#define RSSPECAN_ATTR_PULSE_FM_VIDEO_BANDWIDTH                                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 5744
#define RSSPECAN_ATTR_PULSE_LINK_TREND_M1_TO_SELECTED_PULSE                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 5746
#define RSSPECAN_ATTR_PULSE_MARKER_LINK                                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 7397
#define RSSPECAN_ATTR_PULSE_DELTAMARKER_LINK_TO_MARKER                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 7398
#define RSSPECAN_ATTR_PULSE_TRACE_NORMALIZATION                                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 5747
#define RSSPECAN_ATTR_PULSE_TRACE_STATISTIC_TYPE                                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 7400
#define RSSPECAN_ATTR_PULSE_TRACE_RESULTS                                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 10724
#define RSSPECAN_ATTR_PULSE_TRACE_EVALUATION                                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 6684
#define RSSPECAN_ATTR_PULSE_TRACE_IQ_DETECTOR                                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 7401
#define RSSPECAN_ATTR_PULSE_TRACE_STABILITY_EVALUATION                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 10725
#define RSSPECAN_ATTR_PULSE_REFERENCE_POINT_TYPE                                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 2835
#define RSSPECAN_ATTR_PULSE_REFERENCE_POINT_OFFSET                                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 2836
#define RSSPECAN_ATTR_PULSE_REFERENCE_POINT_AVERAGING_WINDOW                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 2966
#define RSSPECAN_ATTR_PULSE_REFERENCE_PULSE_PULSE_MEASUREMENTS_MODE                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 5750
#define RSSPECAN_ATTR_PULSE_REFERENCE_PULSE_PULSE_MEASUREMENTS_VALUE                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 5751
#define RSSPECAN_ATTR_PULSE_REFERENCE_RANGE_TYPE                                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 2837
#define RSSPECAN_ATTR_PULSE_REFERENCE_RANGE_LENGTH                                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 2838
#define RSSPECAN_ATTR_PULSE_REFERENCE_RANGE_OFFSET_LEFT                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 3324
#define RSSPECAN_ATTR_PULSE_REFERENCE_RANGE_OFFSET_RIGHT                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 3325
#define RSSPECAN_ATTR_PULSE_TOP_LEVEL_POSITION                                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 2840
#define RSSPECAN_ATTR_PULSE_TOP_LEVEL_ALGORITHM                                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 2841
#define RSSPECAN_ATTR_PULSE_TOP_LEVEL_RIPPLE_PORTION                                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 2842
#define RSSPECAN_ATTR_PULSE_TOP_LEVEL_FIXED                                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 7733
#define RSSPECAN_ATTR_PULSE_REFERENCE_LEVEL_UNIT                                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 2843
#define RSSPECAN_ATTR_PULSE_REFERENCE_LEVEL_THRESHOLD_HREF                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 3326
#define RSSPECAN_ATTR_PULSE_REFERENCE_LEVEL_THRESHOLD_LREF                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 3327
#define RSSPECAN_ATTR_PULSE_REFERENCE_LEVEL_THRESHOLD_REF                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 3328
#define RSSPECAN_ATTR_PULSE_REFERENCE_LEVEL_BOUNDARY                                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 2845
#define RSSPECAN_ATTR_PULSE_TIME_SIDELOBE_RANGE                                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 5752
#define RSSPECAN_ATTR_PULSE_TIME_SIDELOBE_ALIGNMENT                                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 5753
#define RSSPECAN_ATTR_PULSE_TIME_SIDELOBE_LENGTH                                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 5754
#define RSSPECAN_ATTR_PULSE_TIME_SIDELOBE_KEEP_OUT_TIME                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 5755
#define RSSPECAN_ATTR_PULSE_TIME_SIDELOBE_KEEP_OUT_TIME_LENGTH                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 5756
#define RSSPECAN_ATTR_PULSE_STABILITY_BURST_MODE                                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 10726
#define RSSPECAN_ATTR_PULSE_STABILITY_BURST_LENGTH                                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 10727
#define RSSPECAN_ATTR_PULSE_STABILITY_SELECTED_PULSE                                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 10728
#define RSSPECAN_ATTR_PULSE_STABILITY_REFERENCE_START                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 10729
#define RSSPECAN_ATTR_PULSE_STABILITY_ANALYSIS_START                                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 10730
#define RSSPECAN_ATTR_PULSE_STABILITY_PULSE_PULSE_OFFSET                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 10731
#define RSSPECAN_ATTR_PULSE_STABILITY_FREQUENCY_OFFSET_COMPENSATION                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 10732
#define RSSPECAN_ATTR_CURRENT_BUFFER_PULSE_COUNT                                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 7734
#define RSSPECAN_ATTR_ALL_PULSE_COUNT                                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 7735
#define RSSPECAN_ATTR_PULSE_SELECTION                                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 2846
#define RSSPECAN_ATTR_PULSE_RESULT_AUTOMATIC_RANGE_SCALING_ONCE                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 3107
#define RSSPECAN_ATTR_PULSE_RESULT_AUTOMATIC_RANGE_SCALING                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 3108
#define RSSPECAN_ATTR_PULSE_RESULT_RANGE_REFERENCE                                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 2847
#define RSSPECAN_ATTR_PULSE_RESULT_RANGE_OFFSET                                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 2848
#define RSSPECAN_ATTR_PULSE_RESULT_RANGE_ALIGNMENT                                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 2849
#define RSSPECAN_ATTR_PULSE_RESULT_RANGE_LENGTH                                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 2850
#define RSSPECAN_ATTR_PULSE_RESULT_RANGE_IQ_STORED_IN_MEMORY                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 7402
#define RSSPECAN_ATTR_PULSE_RESULT_PHASE_UNIT                                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 2851
#define RSSPECAN_ATTR_PULSE_RESULT_FREQUENCY_UNIT                                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 5156
#define RSSPECAN_ATTR_PULSE_RESULT_RANGE_SPECTRUM_WINDOW_TYPE                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 5157
#define RSSPECAN_ATTR_PULSE_RESULT_RANGE_SPECTRUM_RBW_AUTO                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 5158
#define RSSPECAN_ATTR_PULSE_RESULT_RANGE_SPECTRUM_RBW                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 5159
#define RSSPECAN_ATTR_PULSE_RESULT_PARAMETER_TREND_POWER_Y_AXIS                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 3114
#define RSSPECAN_ATTR_PULSE_RESULT_PARAMETER_TREND_POWER_X_AXIS                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 3115
#define RSSPECAN_ATTR_PULSE_RESULT_PARAMETER_TREND_TIMING_Y_AXIS                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 3116
#define RSSPECAN_ATTR_PULSE_RESULT_PARAMETER_TREND_TIMING_X_AXIS                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 3117
#define RSSPECAN_ATTR_PULSE_RESULT_PARAMETER_TREND_FREQUENCY_Y_AXIS                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 3118
#define RSSPECAN_ATTR_PULSE_RESULT_PARAMETER_TREND_FREQUENCY_X_AXIS                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 3119
#define RSSPECAN_ATTR_PULSE_RESULT_PARAMETER_TREND_PHASE_Y_AXIS                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 3120
#define RSSPECAN_ATTR_PULSE_RESULT_PARAMETER_TREND_PHASE_X_AXIS                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 3121
#define RSSPECAN_ATTR_PULSE_RESULT_PARAMETER_TREND_TIME_SIDELOBE_Y_AXIS                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 5759
#define RSSPECAN_ATTR_PULSE_RESULT_PARAMETER_TREND_TIME_SIDELOBE_X_AXIS                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 5760
#define RSSPECAN_ATTR_PULSE_RESULT_PARAMETER_TREND_ENVELOPE_MODEL_Y_AXIS                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 6686
#define RSSPECAN_ATTR_PULSE_RESULT_PARAMETER_TREND_ENVELOPE_MODEL_X_AXIS                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 6687
#define RSSPECAN_ATTR_PULSE_RESULT_PARAMETER_TREND_STABILITY_Y_AXIS                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 10733
#define RSSPECAN_ATTR_PULSE_RESULT_PARAMETER_TREND_STABILITY_X_AXIS                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 10734
#define RSSPECAN_ATTR_PULSE_RESULT_PARAMETER_TREND_DISPLAY_STYLE                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 6688
#define RSSPECAN_ATTR_PULSE_RESULT_PARAMETER_TREND_DISPLAY_LIMIT_LINES_ENABLED                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 4960
#define RSSPECAN_ATTR_PULSE_RESULT_TABLE_POWER_ALL                                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 4658
#define RSSPECAN_ATTR_PULSE_RESULT_TABLE_POWER_TOP                                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 4659
#define RSSPECAN_ATTR_PULSE_RESULT_TABLE_POWER_BASE                                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 4660
#define RSSPECAN_ATTR_PULSE_RESULT_TABLE_POWER_AMPLITUDE                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 4661
#define RSSPECAN_ATTR_PULSE_RESULT_TABLE_POWER_ON                                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 4662
#define RSSPECAN_ATTR_PULSE_RESULT_TABLE_POWER_AVERAGE                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 4663
#define RSSPECAN_ATTR_PULSE_RESULT_TABLE_POWER_MIN                                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 4664
#define RSSPECAN_ATTR_PULSE_RESULT_TABLE_POWER_MAX                                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 4665
#define RSSPECAN_ATTR_PULSE_RESULT_TABLE_POWER_POWER_ON                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 4666
#define RSSPECAN_ATTR_PULSE_RESULT_TABLE_POWER_POWER_AVERAGE                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 4667
#define RSSPECAN_ATTR_PULSE_RESULT_TABLE_POWER_POWER_MIN                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 4668
#define RSSPECAN_ATTR_PULSE_RESULT_TABLE_POWER_ADD                                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 4669
#define RSSPECAN_ATTR_PULSE_RESULT_TABLE_POWER_ADDB                                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 4670
#define RSSPECAN_ATTR_PULSE_RESULT_TABLE_POWER_RPER                                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 4671
#define RSSPECAN_ATTR_PULSE_RESULT_TABLE_POWER_RDB                                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 4672
#define RSSPECAN_ATTR_PULSE_RESULT_TABLE_POWER_OVER                                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 4673
#define RSSPECAN_ATTR_PULSE_RESULT_TABLE_POWER_OVERDB                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 4674
#define RSSPECAN_ATTR_PULSE_RESULT_TABLE_POWER_POINT                                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 4675
#define RSSPECAN_ATTR_PULSE_RESULT_TABLE_POWER_PPR                                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 4676
#define RSSPECAN_ATTR_PULSE_RESULT_TABLE_POWER_AMPLITUDE_I                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 6596
#define RSSPECAN_ATTR_PULSE_RESULT_TABLE_POWER_AMPLITUDE_Q                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 6597
#define RSSPECAN_ATTR_PULSE_RESULT_TABLE_PHASE_ALL                                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 3335
#define RSSPECAN_ATTR_PULSE_RESULT_TABLE_PHASE_POINT                                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 3336
#define RSSPECAN_ATTR_PULSE_RESULT_TABLE_PHASE_PULSE_PULSE                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 3337
#define RSSPECAN_ATTR_PULSE_RESULT_TABLE_PHASE_RMS                                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 3338
#define RSSPECAN_ATTR_PULSE_RESULT_TABLE_PHASE_PEAK                                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 3339
#define RSSPECAN_ATTR_PULSE_RESULT_TABLE_PHASE_DEVIATION                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 3340
#define RSSPECAN_ATTR_PULSE_RESULT_TABLE_TIME_SIDELOBE_ALL                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 5761
#define RSSPECAN_ATTR_PULSE_RESULT_TABLE_TIME_SIDELOBE_PEAK_TO_SIDELOBE_LEVEL                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 5762
#define RSSPECAN_ATTR_PULSE_RESULT_TABLE_TIME_SIDELOBE_INTEGRATED_SIDELOBE_LEVEL                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 5763
#define RSSPECAN_ATTR_PULSE_RESULT_TABLE_TIME_SIDELOBE_COMPRESSION_RATIO                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 5766
#define RSSPECAN_ATTR_PULSE_RESULT_TABLE_TIME_SIDELOBE_INTEGRATED_MAINLOBE_POWER                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 5767
#define RSSPECAN_ATTR_PULSE_RESULT_TABLE_TIME_SIDELOBE_AVERAGE_MAINLOBE_POWER                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 5768
#define RSSPECAN_ATTR_PULSE_RESULT_TABLE_TIME_SIDELOBE_PEAK_CORRELATION                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 5769
#define RSSPECAN_ATTR_PULSE_RESULT_TABLE_TIME_SIDELOBE_MAINLOBE_PHASE                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 5770
#define RSSPECAN_ATTR_PULSE_RESULT_TABLE_ENVELOPE_MODEL_ALL                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 6689
#define RSSPECAN_ATTR_PULSE_RESULT_TABLE_ENVELOPE_MODEL_FALL_BASE_POINT_TIME                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 6690
#define RSSPECAN_ATTR_PULSE_RESULT_TABLE_ENVELOPE_MODEL_FALL_HIGH_POINT_LEVEL                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 6691
#define RSSPECAN_ATTR_PULSE_RESULT_TABLE_ENVELOPE_MODEL_FALL_HIGH_POINT_TIME                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 6692
#define RSSPECAN_ATTR_PULSE_RESULT_TABLE_ENVELOPE_MODEL_FALL_LOW_POINT_LEVEL                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 6693
#define RSSPECAN_ATTR_PULSE_RESULT_TABLE_ENVELOPE_MODEL_FALL_LOW_POINT_TIME                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 6694
#define RSSPECAN_ATTR_PULSE_RESULT_TABLE_ENVELOPE_MODEL_FALL_MID_POINT_LEVEL                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 6695
#define RSSPECAN_ATTR_PULSE_RESULT_TABLE_ENVELOPE_MODEL_FALL_MID_POINT_TIME                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 6696
#define RSSPECAN_ATTR_PULSE_RESULT_TABLE_ENVELOPE_MODEL_FALL_TOP_POINT_LEVEL                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 6697
#define RSSPECAN_ATTR_PULSE_RESULT_TABLE_ENVELOPE_MODEL_FALL_TOP_POINT_TIME                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 6698
#define RSSPECAN_ATTR_PULSE_RESULT_TABLE_ENVELOPE_MODEL_RISE_BASE_POINT_TIME                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 6699
#define RSSPECAN_ATTR_PULSE_RESULT_TABLE_ENVELOPE_MODEL_RISE_HIGH_POINT_LEVEL                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 6700
#define RSSPECAN_ATTR_PULSE_RESULT_TABLE_ENVELOPE_MODEL_RISE_HIGH_POINT_TIME                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 6701
#define RSSPECAN_ATTR_PULSE_RESULT_TABLE_ENVELOPE_MODEL_RISE_LOW_POINT_LEVEL                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 6702
#define RSSPECAN_ATTR_PULSE_RESULT_TABLE_ENVELOPE_MODEL_RISE_LOW_POINT_TIME                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 6703
#define RSSPECAN_ATTR_PULSE_RESULT_TABLE_ENVELOPE_MODEL_RISE_MID_POINT_LEVEL                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 6704
#define RSSPECAN_ATTR_PULSE_RESULT_TABLE_ENVELOPE_MODEL_RISE_MID_POINT_TIME                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 6705
#define RSSPECAN_ATTR_PULSE_RESULT_TABLE_ENVELOPE_MODEL_RISE_TOP_POINT_LEVEL                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 6706
#define RSSPECAN_ATTR_PULSE_RESULT_TABLE_ENVELOPE_MODEL_RISE_TOP_POINT_TIME                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 6707
#define RSSPECAN_ATTR_PULSE_RESULT_TABLE_STABILITY_ALL                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 10735
#define RSSPECAN_ATTR_PULSE_RESULT_TABLE_STABILITY_AMPLITUDE                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 10736
#define RSSPECAN_ATTR_PULSE_RESULT_TABLE_STABILITY_BURST_NUMBER                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 10737
#define RSSPECAN_ATTR_PULSE_RESULT_TABLE_STABILITY_PHASE                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 10738
#define RSSPECAN_ATTR_PULSE_RESULT_TABLE_STABILITY_POSITION_NUMBER_IN_BURST                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 10739
#define RSSPECAN_ATTR_PULSE_RESULT_TABLE_STABILITY_TOTAL                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 10740
#define RSSPECAN_ATTR_PULSE_RESULT_TABLE_LIMIT_ALL_ENABLED                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 4961
#define RSSPECAN_ATTR_PULSE_RESULT_TABLE_FREQUENCY_LIMIT_ALL_ENABLED                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 4963
#define RSSPECAN_ATTR_PULSE_RESULT_TABLE_FREQUENCY_LIMIT_POINT_ENABLED                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 4964
#define RSSPECAN_ATTR_PULSE_RESULT_TABLE_FREQUENCY_LIMIT_PPFR_ENABLED                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 4965
#define RSSPECAN_ATTR_PULSE_RESULT_TABLE_FREQUENCY_LIMIT_ERROR_RMS_ENABLED                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 4966
#define RSSPECAN_ATTR_PULSE_RESULT_TABLE_FREQUENCY_LIMIT_ERROR_PEAK_ENABLED                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 4967
#define RSSPECAN_ATTR_PULSE_RESULT_TABLE_FREQUENCY_LIMIT_DEVIATION_ENABLED                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 4968
#define RSSPECAN_ATTR_PULSE_RESULT_TABLE_FREQUENCY_LIMIT_CHIP_RATE_ENABLED                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 4969
#define RSSPECAN_ATTR_PULSE_RESULT_TABLE_PHASE_LIMIT_ALL_ENABLED                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 4971
#define RSSPECAN_ATTR_PULSE_RESULT_TABLE_PHASE_LIMIT_POINT_ENABLED                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 4972
#define RSSPECAN_ATTR_PULSE_RESULT_TABLE_PHASE_LIMIT_PPFR_ENABLED                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 4973
#define RSSPECAN_ATTR_PULSE_RESULT_TABLE_PHASE_LIMIT_ERROR_RMS_ENABLED                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 4974
#define RSSPECAN_ATTR_PULSE_RESULT_TABLE_PHASE_LIMIT_ERROR_PEAK_ENABLED                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 4975
#define RSSPECAN_ATTR_PULSE_RESULT_TABLE_PHASE_LIMIT_DEVIATION_ENABLED                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 4976
#define RSSPECAN_ATTR_PULSE_RESULT_TABLE_POWER_LIMIT_ALL_ENABLED                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 4978
#define RSSPECAN_ATTR_PULSE_RESULT_TABLE_POWER_LIMIT_TOP_ENABLED                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 4979
#define RSSPECAN_ATTR_PULSE_RESULT_TABLE_POWER_LIMIT_BASE_ENABLED                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 4980
#define RSSPECAN_ATTR_PULSE_RESULT_TABLE_POWER_LIMIT_AMPLITUDE_ENABLED                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 4981
#define RSSPECAN_ATTR_PULSE_RESULT_TABLE_POWER_LIMIT_ON_ENABLED                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 4982
#define RSSPECAN_ATTR_PULSE_RESULT_TABLE_POWER_LIMIT_AVERAGE_ENABLED                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 4983
#define RSSPECAN_ATTR_PULSE_RESULT_TABLE_POWER_LIMIT_MIN_ENABLED                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 4984
#define RSSPECAN_ATTR_PULSE_RESULT_TABLE_POWER_LIMIT_MAX_ENABLED                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 4985
#define RSSPECAN_ATTR_PULSE_RESULT_TABLE_POWER_LIMIT_POWER_ON_ENABLED                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 4986
#define RSSPECAN_ATTR_PULSE_RESULT_TABLE_POWER_LIMIT_POWER_AVERAGE_ENABLED                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 4987
#define RSSPECAN_ATTR_PULSE_RESULT_TABLE_POWER_LIMIT_POWER_MIN_ENABLED                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 4988
#define RSSPECAN_ATTR_PULSE_RESULT_TABLE_POWER_LIMIT_ADR_ENABLED                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 4989
#define RSSPECAN_ATTR_PULSE_RESULT_TABLE_POWER_LIMIT_ADDB_ENABLED                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 4990
#define RSSPECAN_ATTR_PULSE_RESULT_TABLE_POWER_LIMIT_RPER_ENABLED                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 4991
#define RSSPECAN_ATTR_PULSE_RESULT_TABLE_POWER_LIMIT_RPDB_ENABLED                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 4992
#define RSSPECAN_ATTR_PULSE_RESULT_TABLE_POWER_LIMIT_OVER_ENABLED                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 4993
#define RSSPECAN_ATTR_PULSE_RESULT_TABLE_POWER_LIMIT_OVERDB_ENABLED                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 4994
#define RSSPECAN_ATTR_PULSE_RESULT_TABLE_POWER_LIMIT_POINT_ENABLED                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 4995
#define RSSPECAN_ATTR_PULSE_RESULT_TABLE_POWER_LIMIT_PPR_ENABLED                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 4996
#define RSSPECAN_ATTR_PULSE_RESULT_TABLE_POWER_LIMIT_AMPLITUDE_I_ENABLED                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 6708
#define RSSPECAN_ATTR_PULSE_RESULT_TABLE_POWER_LIMIT_AMPLITUDE_Q_ENABLED                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 6709
#define RSSPECAN_ATTR_PULSE_RESULT_TABLE_TIMING_LIMIT_ALL_ENABLED                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 4998
#define RSSPECAN_ATTR_PULSE_RESULT_TABLE_TIMING_LIMIT_TIME_STAMP_ENABLED                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 4999
#define RSSPECAN_ATTR_PULSE_RESULT_TABLE_TIMING_LIMIT_SETTLING_TIME_ENABLED                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 5000
#define RSSPECAN_ATTR_PULSE_RESULT_TABLE_TIMING_LIMIT_RISE_ENABLED                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 5001
#define RSSPECAN_ATTR_PULSE_RESULT_TABLE_TIMING_LIMIT_FALL_ENABLED                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 5002
#define RSSPECAN_ATTR_PULSE_RESULT_TABLE_TIMING_LIMIT_PULSE_WIDTH_ENABLED                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 5003
#define RSSPECAN_ATTR_PULSE_RESULT_TABLE_TIMING_LIMIT_OFF_ENABLED                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 5004
#define RSSPECAN_ATTR_PULSE_RESULT_TABLE_TIMING_LIMIT_DUTY_RATIO_ENABLED                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 5005
#define RSSPECAN_ATTR_PULSE_RESULT_TABLE_TIMING_LIMIT_DUTY_CYCLE_ENABLED                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 5006
#define RSSPECAN_ATTR_PULSE_RESULT_TABLE_TIMING_LIMIT_PRI_ENABLED                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 5007
#define RSSPECAN_ATTR_PULSE_RESULT_TABLE_TIMING_LIMIT_PRF_ENABLED                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 5008
#define RSSPECAN_ATTR_PULSE_RESULT_TABLE_TIME_SIDELOBE_LIMIT_ALL_ENABLED                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 5773
#define RSSPECAN_ATTR_PULSE_RESULT_TABLE_TIME_SIDELOBE_LIMIT_PEAK_TO_SIDELOBE_LEVEL_ENABLED                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 5774
#define RSSPECAN_ATTR_PULSE_RESULT_TABLE_TIME_SIDELOBE_LIMIT_INTEGRATED_SIDELOBE_LEVEL_ENABLED               RS_SPECIFIC_PUBLIC_ATTR_BASE + 5775
#define RSSPECAN_ATTR_PULSE_RESULT_TABLE_TIME_SIDELOBE_LIMIT_MAINLOBE_3DB_WIDTH_ENABLED                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 5776
#define RSSPECAN_ATTR_PULSE_RESULT_TABLE_TIME_SIDELOBE_LIMIT_SIDELOBE_DELAY_ENABLED                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 5777
#define RSSPECAN_ATTR_PULSE_RESULT_TABLE_TIME_SIDELOBE_LIMIT_COMPRESSION_RATIO_ENABLED                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 5778
#define RSSPECAN_ATTR_PULSE_RESULT_TABLE_TIME_SIDELOBE_LIMIT_INTEGRATED_MAINLOBE_POWER_ENABLED               RS_SPECIFIC_PUBLIC_ATTR_BASE + 5779
#define RSSPECAN_ATTR_PULSE_RESULT_TABLE_TIME_SIDELOBE_LIMIT_AVERAGE_MAINLOBE_POWER_ENABLED                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 5780
#define RSSPECAN_ATTR_PULSE_RESULT_TABLE_TIME_SIDELOBE_LIMIT_PEAK_CORRELATION_ENABLED                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 5781
#define RSSPECAN_ATTR_PULSE_RESULT_TABLE_TIME_SIDELOBE_LIMIT_MAINLOBE_PHASE_ENABLED                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 5782
#define RSSPECAN_ATTR_PULSE_RESULT_TABLE_TIME_SIDELOBE_LIMIT_MAINLOBE_FREQUENCY_ENABLED                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 5783
#define RSSPECAN_ATTR_PULSE_RESULT_TABLE_ENVELOPE_MODEL_LIMIT_ALL_ENABLED                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 6711
#define RSSPECAN_ATTR_PULSE_RESULT_TABLE_ENVELOPE_MODEL_FALL_BASE_POINT_TIME_LIMIT_ENABLED                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 6712
#define RSSPECAN_ATTR_PULSE_RESULT_TABLE_ENVELOPE_MODEL_FALL_HIGH_POINT_LEVEL_LIMIT_ENABLED                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 6713
#define RSSPECAN_ATTR_PULSE_RESULT_TABLE_ENVELOPE_MODEL_FALL_HIGH_POINT_TIME_LIMIT_ENABLED                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 6714
#define RSSPECAN_ATTR_PULSE_RESULT_TABLE_ENVELOPE_MODEL_FALL_LOW_POINT_LEVEL_LIMIT_ENABLED                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 6715
#define RSSPECAN_ATTR_PULSE_RESULT_TABLE_ENVELOPE_MODEL_FALL_LOW_POINT_TIME_LIMIT_ENABLED                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 6716
#define RSSPECAN_ATTR_PULSE_RESULT_TABLE_ENVELOPE_MODEL_FALL_MID_POINT_LEVEL_LIMIT_ENABLED                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 6717
#define RSSPECAN_ATTR_PULSE_RESULT_TABLE_ENVELOPE_MODEL_FALL_MID_POINT_TIME_LIMIT_ENABLED                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 6718
#define RSSPECAN_ATTR_PULSE_RESULT_TABLE_ENVELOPE_MODEL_FALL_TOP_POINT_LEVEL_LIMIT_ENABLED                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 6719
#define RSSPECAN_ATTR_PULSE_RESULT_TABLE_ENVELOPE_MODEL_FALL_TOP_POINT_TIME_LIMIT_ENABLED                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 6720
#define RSSPECAN_ATTR_PULSE_RESULT_TABLE_ENVELOPE_MODEL_RISE_BASE_POINT_TIME_LIMIT_ENABLED                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 6721
#define RSSPECAN_ATTR_PULSE_RESULT_TABLE_ENVELOPE_MODEL_RISE_HIGH_POINT_LEVEL_LIMIT_ENABLED                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 6722
#define RSSPECAN_ATTR_PULSE_RESULT_TABLE_ENVELOPE_MODEL_RISE_HIGH_POINT_TIME_LIMIT_ENABLED                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 6723
#define RSSPECAN_ATTR_PULSE_RESULT_TABLE_ENVELOPE_MODEL_RISE_LOW_POINT_LEVEL_LIMIT_ENABLED                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 6724
#define RSSPECAN_ATTR_PULSE_RESULT_TABLE_ENVELOPE_MODEL_RISE_LOW_POINT_TIME_LIMIT_ENABLED                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 6725
#define RSSPECAN_ATTR_PULSE_RESULT_TABLE_ENVELOPE_MODEL_RISE_MID_POINT_LEVEL_LIMIT_ENABLED                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 6726
#define RSSPECAN_ATTR_PULSE_RESULT_TABLE_ENVELOPE_MODEL_RISE_MID_POINT_TIME_LIMIT_ENABLED                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 6727
#define RSSPECAN_ATTR_PULSE_RESULT_TABLE_ENVELOPE_MODEL_RISE_TOP_POINT_LEVEL_LIMIT_ENABLED                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 6728
#define RSSPECAN_ATTR_PULSE_RESULT_TABLE_ENVELOPE_MODEL_RISE_TOP_POINT_TIME_LIMIT_ENABLED                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 6729
#define RSSPECAN_ATTR_PULSE_RESULT_TABLE_STABILITY_ALL_LIMIT_ENABLED                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 10741
#define RSSPECAN_ATTR_PULSE_RESULT_TABLE_STABILITY_AMPLITUDE_LIMIT_ENABLED                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 10742
#define RSSPECAN_ATTR_PULSE_RESULT_TABLE_STABILITY_BURST_LIMIT_ENABLED                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 10743
#define RSSPECAN_ATTR_PULSE_RESULT_TABLE_STABILITY_PHASE_LIMIT_ENABLED                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 10744
#define RSSPECAN_ATTR_PULSE_RESULT_TABLE_STABILITY_POSITION_NUMBER_IN_BURST_LIMIT_ENABLED                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 10745
#define RSSPECAN_ATTR_PULSE_RESULT_TABLE_STABILITY_TOTAL_LIMIT_ENABLED                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 10746
#define RSSPECAN_ATTR_PULSE_ABSOLUTE_SCALING_MAX                                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 3345
#define RSSPECAN_ATTR_PULSE_ABSOLUTE_SCALING_MIN                                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 3346
#define RSSPECAN_ATTR_PULSE_AUTO_GRID_SCALING                                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 2854
#define RSSPECAN_ATTR_PULSE_REFERENCE_VALUE_MIN                                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 3347
#define RSSPECAN_ATTR_PULSE_REFERENCE_VALUE_MAX                                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 3348
#define RSSPECAN_ATTR_PULSE_Y_SCALE_UNIT                                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 7404
#define RSSPECAN_ATTR_PULSE_X_SCALE_UNIT                                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 7405
#define RSSPECAN_ATTR_PULSE_PARAMETER_SPECTRUM_FULL_AUTO                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 5009
#define RSSPECAN_ATTR_PULSE_PARAMETER_SPECTRUM_MAXIMUM_FREQUENCY                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 5010
#define RSSPECAN_ATTR_PULSE_PARAMETER_SPECTRUM_WINDOW_TYPE                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 5011
#define RSSPECAN_ATTR_PULSE_PARAMETER_SPECTRUM_BLOCK_SIZE                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 5012
#define RSSPECAN_ATTR_PULSE_PARAMETER_SPECTRUM_GAP_THRESHOLD                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 5013
#define RSSPECAN_ATTR_PULSE_PARAMETER_SPECTRUM_SEGMENT_THRESHOLD                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 5014
#define RSSPECAN_ATTR_PULSE_PARAMETER_SPECTRUM_FREQUENCY                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 3136
#define RSSPECAN_ATTR_PULSE_PARAMETER_SPECTRUM_PHASE                                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 3137
#define RSSPECAN_ATTR_PULSE_PARAMETER_SPECTRUM_POWER                                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 3138
#define RSSPECAN_ATTR_PULSE_PARAMETER_SPECTRUM_TIMING                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 3139
#define RSSPECAN_ATTR_PULSE_PARAMETER_SPECTRUM_TIME_SIDELOBE                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 5784
#define RSSPECAN_ATTR_PULSE_PARAMETER_SPECTRUM_ENVELOPE_MODEL                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 6730
#define RSSPECAN_ATTR_PULSE_PARAMETER_SPECTRUM_STABILITY                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 10747
#define RSSPECAN_ATTR_PULSE_QUERY_SPECTRUM_RESOLUTION_BANDWIDTH                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 5015
#define RSSPECAN_ATTR_PULSE_PULSE_AUTO_ENABLED                                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 7743
#define RSSPECAN_ATTR_PULSE_PULSE_GAP_THRESHOLD                                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 7744
#define RSSPECAN_ATTR_PULSE_PULSE_MAXIMUM_FREQUENCY                                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 7745
#define RSSPECAN_ATTR_PULSE_PULSE_RESOLUTION_BANDWITH                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 7746
#define RSSPECAN_ATTR_PULSE_PULSE_MINIMUM_SECTION_THRESHOLD                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 7747
#define RSSPECAN_ATTR_PULSE_PULSE_WINDOW_TYPE                                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 7748
#define RSSPECAN_ATTR_PULSE_PARAMETER_DISTRIBUTION_BINS                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 3109
#define RSSPECAN_ATTR_PULSE_RESULT_PARAMETER_DISTRIBUTION_DISPLAY_LIMIT_LINES_ENABLED                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 5016
#define RSSPECAN_ATTR_PULSE_STABILITY_WATERFALL_Z_AXIS                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 10748
#define RSSPECAN_ATTR_PULSE_STABILITY_WATERFALL_X_AXIS_GRID_LINES                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 10749
#define RSSPECAN_ATTR_PULSE_STABILITY_WATERFALL_Y_AXIS_GRID_LINES                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 10750
#define RSSPECAN_ATTR_PULSE_STABILITY_WATERFALL_MEASUREMENT_INFO                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 10751
#define RSSPECAN_ATTR_PULSE_STABILITY_WATERFALL_Z_AXIS_MEASUREMENT_PLANE                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 10752
#define RSSPECAN_ATTR_PULSE_STABILITY_WATERFALL_LINK_WINDOWS_CAMERA_ANGLE                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 10753
#define RSSPECAN_ATTR_PULSE_STABILITY_WATERFALL_LINK_WINDOWS_Z_AXIS_SCALE                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 10754
#define RSSPECAN_ATTR_PULSE_RESULT_PHASE_NORMALIZATION                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 5758
#define RSSPECAN_ATTR_FM_DEMOD_MODE                                                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 1127
#define RSSPECAN_ATTR_FM_DEMOD_TRIGGER_SOURCE                                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 1128
#define RSSPECAN_ATTR_ANALOG_DEMOD_TRIGGER_AM_LEVEL_ABS                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 1129
#define RSSPECAN_ATTR_ANALOG_DEMOD_TRIGGER_AM_LEVEL_RELATIVE                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 1130
#define RSSPECAN_ATTR_ANALOG_DEMOD_TRIGGER_FM_LEVEL                                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 1131
#define RSSPECAN_ATTR_ANALOG_DEMOD_TRIGGER_PM_LEVEL                                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 1132
#define RSSPECAN_ATTR_ADEM_SELECT_TRACE                                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 2967
#define RSSPECAN_ATTR_FMDEM_DISPLAY_PDIV                                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 1134
#define RSSPECAN_ATTR_FM_DEMOD_ENABLED                                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 1135
#define RSSPECAN_ATTR_FM_DEMOD_AF_COUPLING                                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 1136
#define RSSPECAN_ATTR_FM_DEMOD_MEAS_TIME                                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 1137
#define RSSPECAN_ATTR_FM_DEMOD_BANDWIDTH                                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 1138
#define RSSPECAN_ATTR_FM_DEMOD_RECORD_LENGTH                                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 1139
#define RSSPECAN_ATTR_FM_DEMOD_SAMPLING_RATE                                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 1140
#define RSSPECAN_ATTR_FM_DEMOD_ZOOM_ENABLED                                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 1141
#define RSSPECAN_ATTR_FM_DEMOD_ZOOM_START                                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 1142
#define RSSPECAN_ATTR_FM_DEMOD_ZOOM_LENGTH_MODE                                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 1143
#define RSSPECAN_ATTR_FM_DEMOD_ZOOM_LENGTH                                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 1144
#define RSSPECAN_ATTR_FM_DEMOD_SQUELCH                                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 2567
#define RSSPECAN_ATTR_FM_DEMOD_SQUELCH_LEVEL                                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 2568
#define RSSPECAN_ATTR_ANALOG_DEMOD_PHASE_WRAP                                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 1145
#define RSSPECAN_ATTR_ANALOG_DEMOD_UNIT_ANGLE                                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 1146
#define RSSPECAN_ATTR_ANALOG_DEMOD_ZERO_PHASE_REF_POINT_MODE                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 10855
#define RSSPECAN_ATTR_ANALOG_DEMOD_ZERO_PHASE_REF_POINT                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 1147
#define RSSPECAN_ATTR_ANALOG_DEMOD_OUTPUT_STATE                                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 3004
#define RSSPECAN_ATTR_ANALOG_DEMOD_OUTPUT_SELECTION                                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 3005
#define RSSPECAN_ATTR_ANALOG_DEMOD_OUTPUT_CUTOFF_FREQUENCY                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 3006
#define RSSPECAN_ATTR_ANALOG_DEMOD_PHONE_OUTPUT                                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 3007
#define RSSPECAN_ATTR_ANALOG_DEMOD_THD_UNIT                                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 1148
#define RSSPECAN_ATTR_ANALOG_DEMOD_PRESET_LOAD                                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 2968
#define RSSPECAN_ATTR_ANALOG_DEMOD_PRESET_STORE                                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 2969
#define RSSPECAN_ATTR_ANALOG_DEMOD_PRESET_RESTORE                                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 2970
#define RSSPECAN_ATTR_FM_DEMOD_AF_CENTER_FREQUENCY                                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 3171
#define RSSPECAN_ATTR_FM_DEMOD_AF_SPAN                                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 3172
#define RSSPECAN_ATTR_FM_DEMOD_AF_START_FREQUENCY                                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 3173
#define RSSPECAN_ATTR_FM_DEMOD_AF_STOP_FREQUENCY                                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 3174
#define RSSPECAN_ATTR_FM_DEMOD_AF_FULL_SPAN                                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 3175
#define RSSPECAN_ATTR_FM_DEMOD_RF_SPECTRUM_RESOLUTION_BANDWIDTH                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 3176
#define RSSPECAN_ATTR_FM_DEMOD_RF_SPECTRUM_SPAN                                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 3177
#define RSSPECAN_ATTR_FM_DEMOD_RF_SPECTRUM_ZOOM                                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 3178
#define RSSPECAN_ATTR_FM_DEMOD_HIGH_PASS_AF_FILTER_ENABLED                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 3179
#define RSSPECAN_ATTR_FM_DEMOD_HIGH_PASS_AF_FILTER_FREQUENCY                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 3180
#define RSSPECAN_ATTR_FM_DEMOD_LOW_PASS_AF_FILTER_ENABLED                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 3181
#define RSSPECAN_ATTR_FM_DEMOD_LOW_PASS_AF_FILTER_FREQUENCY                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 3182
#define RSSPECAN_ATTR_ANALOG_DEMOD_LOW_PASS_FILTER_FREQUENCY_RELATIVE                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 3183
#define RSSPECAN_ATTR_ANALOG_DEMOD_FILTER_FREQUENCY_MANUAL_LOWPASS                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 3351
#define RSSPECAN_ATTR_ANALOG_DEMOD_FILTER_FREQUENCY_MANUAL_HIGHPASS                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 3352
#define RSSPECAN_ATTR_FM_DEMOD_DEEMPHASIS_ENABLED                                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 3186
#define RSSPECAN_ATTR_FM_DEMOD_DEEMPHASIS_TIME_CONSTANT                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 3187
#define RSSPECAN_ATTR_ANALOG_DEMOD_FILTER_CCITT_WEIGHTING                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 3188
#define RSSPECAN_ATTR_ANALOG_DEMOD_FILTER_CCIR_WEIGHTING                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 3189
#define RSSPECAN_ATTR_ANALOG_DEMOD_FILTER_A_WEIGHTED_ENABLED                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 3190
#define RSSPECAN_ATTR_ANALOG_DEMOD_SETTLING_TIME_ENABLED                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 10856
#define RSSPECAN_ATTR_ANALOG_DEMOD_SETTLING_LOWER_LIMIT                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 10857
#define RSSPECAN_ATTR_ANALOG_DEMOD_SETTLING_UPPER_LIMIT                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 10858
#define RSSPECAN_ATTR_ANALOG_DEMOD_AM_DETECTOR_STATE                                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 5454
#define RSSPECAN_ATTR_ANALOG_DEMOD_AM_DETECTOR_MODE                                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 5455
#define RSSPECAN_ATTR_ANALOG_DEMOD_AM_DETECTOR_REFERENCE_VALUE                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 5456
#define RSSPECAN_ATTR_ANALOG_DEMOD_AM_DETECTOR_MEAS_TO_REF                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 5457
#define RSSPECAN_ATTR_ANALOG_DEMOD_FM_DETECTOR_STATE                                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 5459
#define RSSPECAN_ATTR_ANALOG_DEMOD_FM_DETECTOR_MODE                                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 5460
#define RSSPECAN_ATTR_ANALOG_DEMOD_FM_DETECTOR_REFERENCE_VALUE                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 5461
#define RSSPECAN_ATTR_ANALOG_DEMOD_FM_DETECTOR_MEAS_TO_REF                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 5462
#define RSSPECAN_ATTR_ANALOG_DEMOD_PM_DETECTOR_STATE                                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 5464
#define RSSPECAN_ATTR_ANALOG_DEMOD_PM_DETECTOR_MODE                                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 5465
#define RSSPECAN_ATTR_ANALOG_DEMOD_PM_DETECTOR_REFERENCE_VALUE                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 5466
#define RSSPECAN_ATTR_ANALOG_DEMOD_PM_DETECTOR_MEAS_TO_REF                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 5467
#define RSSPECAN_ATTR_ANALOG_DEMOD_RELATIVE_UNIT                                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 5469
#define RSSPECAN_ATTR_ANALOG_DEMOD_FM_OFFSET_IMMEDIATE                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 4724
#define RSSPECAN_ATTR_ANALOG_DEMOD_FM_OFFSET_AVERAGE                                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 3191
#define RSSPECAN_ATTR_FM_DEMOD_AUDIO_FREQUENCY_RESULT                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 1166
#define RSSPECAN_ATTR_FM_DEMOD_FREQUENCY_ERROR_RESULT                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 1167
#define RSSPECAN_ATTR_FM_DEMOD_SINAD_MEASUREMENT_RESULT                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 1168
#define RSSPECAN_ATTR_FM_DEMOD_THD_MEASUREMENT_RESULT                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 1169
#define RSSPECAN_ATTR_FM_DEMOD_CARRIER_POWER_RESULT                                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 1170
#define RSSPECAN_ATTR_ANALOG_DEMOD_DISTORTION_RESULT                                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 10859
#define RSSPECAN_ATTR_ANALOG_DEMOD_SETTLING_TIME                                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 10860
#define RSSPECAN_ATTR_ANALOG_DEMOD_SUMMARY_RESULT_VALUE_AM_PPEAK                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 3356
#define RSSPECAN_ATTR_ANALOG_DEMOD_SUMMARY_RESULT_VALUE_AM_MPEAK                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 3357
#define RSSPECAN_ATTR_ANALOG_DEMOD_SUMMARY_RESULT_VALUE_AM_MIDDLE                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 3358
#define RSSPECAN_ATTR_ANALOG_DEMOD_SUMMARY_RESULT_VALUE_AM_RMS                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 3359
#define RSSPECAN_ATTR_ANALOG_DEMOD_SUMMARY_RESULT_VALUE_FM_PPEAK                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 3360
#define RSSPECAN_ATTR_ANALOG_DEMOD_SUMMARY_RESULT_VALUE_FM_MPEAK                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 3361
#define RSSPECAN_ATTR_ANALOG_DEMOD_SUMMARY_RESULT_VALUE_FM_MIDDLE                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 3362
#define RSSPECAN_ATTR_ANALOG_DEMOD_SUMMARY_RESULT_VALUE_FM_RMS                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 3363
#define RSSPECAN_ATTR_ANALOG_DEMOD_SUMMARY_RESULT_VALUE_PM_PPEAK                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 3364
#define RSSPECAN_ATTR_ANALOG_DEMOD_SUMMARY_RESULT_VALUE_PM_MPEAK                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 3365
#define RSSPECAN_ATTR_ANALOG_DEMOD_SUMMARY_RESULT_VALUE_PM_MIDDLE                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 3366
#define RSSPECAN_ATTR_ANALOG_DEMOD_SUMMARY_RESULT_VALUE_PM_RMS                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 3367
#define RSSPECAN_ATTR_ANALOG_DEMOD_SUMMARY_RESULT_VALUE_AM_RELATIVE_PPEAK                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 5470
#define RSSPECAN_ATTR_ANALOG_DEMOD_SUMMARY_RESULT_VALUE_AM_RELATIVE_MPEAK                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 5471
#define RSSPECAN_ATTR_ANALOG_DEMOD_SUMMARY_RESULT_VALUE_AM_RELATIVE_MIDDLE                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 5472
#define RSSPECAN_ATTR_ANALOG_DEMOD_SUMMARY_RESULT_VALUE_AM_RELATIVE_RMS                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 5473
#define RSSPECAN_ATTR_ANALOG_DEMOD_SUMMARY_RESULT_VALUE_FM_RELATIVE_PPEAK                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 5474
#define RSSPECAN_ATTR_ANALOG_DEMOD_SUMMARY_RESULT_VALUE_FM_RELATIVE_MPEAK                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 5475
#define RSSPECAN_ATTR_ANALOG_DEMOD_SUMMARY_RESULT_VALUE_FM_RELATIVE_MIDDLE                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 5476
#define RSSPECAN_ATTR_ANALOG_DEMOD_SUMMARY_RESULT_VALUE_FM_RELATIVE_RMS                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 5477
#define RSSPECAN_ATTR_ANALOG_DEMOD_SUMMARY_RESULT_VALUE_PM_RELATIVE_PPEAK                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 5478
#define RSSPECAN_ATTR_ANALOG_DEMOD_SUMMARY_RESULT_VALUE_PM_RELATIVE_MPEAK                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 5479
#define RSSPECAN_ATTR_ANALOG_DEMOD_SUMMARY_RESULT_VALUE_PM_RELATIVE_MIDDLE                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 5480
#define RSSPECAN_ATTR_ANALOG_DEMOD_SUMMARY_RESULT_VALUE_PM_RELATIVE_RMS                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 5481
#define RSSPECAN_ATTR_FM_STEREO_MODE                                                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 1173
#define RSSPECAN_ATTR_FM_STEREO_ENABLED                                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 1174
#define RSSPECAN_ATTR_FM_STEREO_TRIGGER_SOURCE                                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 1175
#define RSSPECAN_ATTR_FM_STEREO_TRIGGER_LEVEL_LEFT                                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 3368
#define RSSPECAN_ATTR_FM_STEREO_TRIGGER_LEVEL_RIGHT                                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 3369
#define RSSPECAN_ATTR_FM_STEREO_TRIGGER_LEVEL_MPX                                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 3370
#define RSSPECAN_ATTR_FM_STEREO_TRIGGER_LEVEL_MONO                                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 3371
#define RSSPECAN_ATTR_FM_STEREO_TRIGGER_LEVEL_STEREO                                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 3372
#define RSSPECAN_ATTR_FM_STEREO_TRIGGER_LEVEL_RDS                                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 3373
#define RSSPECAN_ATTR_FM_STEREO_TRIGGER_LEVEL_PILOT                                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 3374
#define RSSPECAN_ATTR_FM_STEREO_SQUELCH_LEVEL                                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 2566
#define RSSPECAN_ATTR_FM_STEREO_SQUELCH                                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 2565
#define RSSPECAN_ATTR_FM_STEREO_DIFFERENCE_FREQUENCY_DISTORTION_UNIT                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 1179
#define RSSPECAN_ATTR_FM_STEREO_INTERMODULATION_DISTORTION_UNIT                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 1181
#define RSSPECAN_ATTR_FM_STEREO_DIFFERENCE_FREQUENCY_DISTORTION_ENABLED                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 1178
#define RSSPECAN_ATTR_FM_STEREO_INTERMODULATION_DISTORTION_STATE                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 1180
#define RSSPECAN_ATTR_FM_STEREO_PHASE_NOISE_MARKER_ENABLED                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 1177
#define RSSPECAN_ATTR_FM_STEREO_AF_AUTO_SCALE                                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 1182
#define RSSPECAN_ATTR_FM_STEREO_REFERENCE_DEVIATION                                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 1184
#define RSSPECAN_ATTR_FM_STEREO_ALL_AF_FILTERS_OFF                                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 1200
#define RSSPECAN_ATTR_FM_STEREO_AF_RANGE_DEV_UNIT_ABSOLUTE                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 1201
#define RSSPECAN_ATTR_FM_STEREO_AF_RANGE_DEV_UNIT_RELATIVE                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 1202
#define RSSPECAN_ATTR_FM_STEREO_LEFT_RESULT_SUMMARY_DETECTOR                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 4725
#define RSSPECAN_ATTR_FM_STEREO_RIGHT_RESULT_SUMMARY_DETECTOR                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 4526
#define RSSPECAN_ATTR_FM_STEREO_MPX_RESULT_SUMMARY_DETECTOR                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 4527
#define RSSPECAN_ATTR_FM_STEREO_MONO_RESULT_SUMMARY_DETECTOR                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 4528
#define RSSPECAN_ATTR_FM_STEREO_STEREO_RESULT_SUMMARY_DETECTOR                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 4529
#define RSSPECAN_ATTR_FM_STEREO_RDS_RESULT_SUMMARY_DETECTOR                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 4530
#define RSSPECAN_ATTR_FM_STEREO_PILOT_RESULT_SUMMARY_DETECTOR                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 4531
#define RSSPECAN_ATTR_FM_STEREO_LEFT_RESULT_SUMMARY_MODE                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 4726
#define RSSPECAN_ATTR_FM_STEREO_RIGHT_RESULT_SUMMARY_MODE                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 4532
#define RSSPECAN_ATTR_FM_STEREO_MPX_RESULT_SUMMARY_MODE                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 4533
#define RSSPECAN_ATTR_FM_STEREO_MONO_RESULT_SUMMARY_MODE                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 4534
#define RSSPECAN_ATTR_FM_STEREO_STEREO_RESULT_SUMMARY_MODE                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 4535
#define RSSPECAN_ATTR_FM_STEREO_RDS_RESULT_SUMMARY_MODE                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 4536
#define RSSPECAN_ATTR_FM_STEREO_PILOT_RESULT_SUMMARY_MODE                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 4537
#define RSSPECAN_ATTR_FM_STEREO_LEFT_RESULT_SUMMARY_COUPLED                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 4727
#define RSSPECAN_ATTR_FM_STEREO_RIGHT_RESULT_SUMMARY_COUPLED                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 4538
#define RSSPECAN_ATTR_FM_STEREO_MPX_RESULT_SUMMARY_COUPLED                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 4539
#define RSSPECAN_ATTR_FM_STEREO_MONO_RESULT_SUMMARY_COUPLED                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 4540
#define RSSPECAN_ATTR_FM_STEREO_STEREO_RESULT_SUMMARY_COUPLED                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 4541
#define RSSPECAN_ATTR_FM_STEREO_RDS_RESULT_SUMMARY_COUPLED                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 4542
#define RSSPECAN_ATTR_FM_STEREO_PILOT_RESULT_SUMMARY_COUPLED                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 4543
#define RSSPECAN_ATTR_FM_STEREO_LEFT_MEAS_TO_REF                                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 4728
#define RSSPECAN_ATTR_FM_STEREO_RIGHT_MEAS_TO_REF                                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 4544
#define RSSPECAN_ATTR_FM_STEREO_MPX_MEAS_TO_REF                                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 4545
#define RSSPECAN_ATTR_FM_STEREO_MONO_MEAS_TO_REF                                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 4546
#define RSSPECAN_ATTR_FM_STEREO_STEREO_MEAS_TO_REF                                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 4547
#define RSSPECAN_ATTR_FM_STEREO_RDS_MEAS_TO_REF                                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 4548
#define RSSPECAN_ATTR_FM_STEREO_PILOT_MEAS_TO_REF                                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 4549
#define RSSPECAN_ATTR_FM_STEREO_LEFT_AF_FILTER_HIGH_PASS_ENABLED                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 4729
#define RSSPECAN_ATTR_FM_STEREO_RIGHT_AF_FILTER_HIGH_PASS_ENABLED                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 4550
#define RSSPECAN_ATTR_FM_STEREO_MPX_AF_FILTER_HIGH_PASS_ENABLED                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 4551
#define RSSPECAN_ATTR_FM_STEREO_MONO_AF_FILTER_HIGH_PASS_ENABLED                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 4552
#define RSSPECAN_ATTR_FM_STEREO_STEREO_AF_FILTER_HIGH_PASS_ENABLED                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 4553
#define RSSPECAN_ATTR_FM_STEREO_RDS_AF_FILTER_HIGH_PASS_ENABLED                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 4554
#define RSSPECAN_ATTR_FM_STEREO_PILOT_AF_FILTER_HIGH_PASS_ENABLED                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 4555
#define RSSPECAN_ATTR_FM_STEREO_LEFT_AF_FILTER_HIGH_PASS_FREQUENCY                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 4730
#define RSSPECAN_ATTR_FM_STEREO_RIGHT_AF_FILTER_HIGH_PASS_FREQUENCY                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 4556
#define RSSPECAN_ATTR_FM_STEREO_MPX_AF_FILTER_HIGH_PASS_FREQUENCY                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 4557
#define RSSPECAN_ATTR_FM_STEREO_MONO_AF_FILTER_HIGH_PASS_FREQUENCY                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 4558
#define RSSPECAN_ATTR_FM_STEREO_STEREO_AF_FILTER_HIGH_PASS_FREQUENCY                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 4559
#define RSSPECAN_ATTR_FM_STEREO_RDS_AF_FILTER_HIGH_PASS_FREQUENCY                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 4560
#define RSSPECAN_ATTR_FM_STEREO_PILOT_AF_FILTER_HIGH_PASS_FREQUENCY                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 4561
#define RSSPECAN_ATTR_FM_STEREO_LEFT_AF_FILTER_LOW_PASS_ENABLED                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 4731
#define RSSPECAN_ATTR_FM_STEREO_RIGHT_AF_FILTER_LOW_PASS_ENABLED                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 4562
#define RSSPECAN_ATTR_FM_STEREO_MPX_AF_FILTER_LOW_PASS_ENABLED                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 4563
#define RSSPECAN_ATTR_FM_STEREO_MONO_AF_FILTER_LOW_PASS_ENABLED                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 4564
#define RSSPECAN_ATTR_FM_STEREO_STEREO_AF_FILTER_LOW_PASS_ENABLED                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 4565
#define RSSPECAN_ATTR_FM_STEREO_RDS_AF_FILTER_LOW_PASS_ENABLED                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 4566
#define RSSPECAN_ATTR_FM_STEREO_PILOT_AF_FILTER_LOW_PASS_ENABLED                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 4567
#define RSSPECAN_ATTR_FM_STEREO_LEFT_AF_FILTER_LOW_PASS_FREQUENCY_ABSOLUTE                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 4732
#define RSSPECAN_ATTR_FM_STEREO_RIGHT_AF_FILTER_LOW_PASS_FREQUENCY_ABSOLUTE                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 4568
#define RSSPECAN_ATTR_FM_STEREO_MPX_AF_FILTER_LOW_PASS_FREQUENCY_ABSOLUTE                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 4569
#define RSSPECAN_ATTR_FM_STEREO_MONO_AF_FILTER_LOW_PASS_FREQUENCY_ABSOLUTE                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 4570
#define RSSPECAN_ATTR_FM_STEREO_STEREO_AF_FILTER_LOW_PASS_FREQUENCY_ABSOLUTE                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 4571
#define RSSPECAN_ATTR_FM_STEREO_RDS_AF_FILTER_LOW_PASS_FREQUENCY_ABSOLUTE                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 4572
#define RSSPECAN_ATTR_FM_STEREO_PILOT_AF_FILTER_LOW_PASS_FREQUENCY_ABSOLUTE                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 4573
#define RSSPECAN_ATTR_FM_STEREO_LEFT_AF_FILTER_LOW_PASS_FREQUENCY_RELATIVE                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 4733
#define RSSPECAN_ATTR_FM_STEREO_RIGHT_AF_FILTER_LOW_PASS_FREQUENCY_RELATIVE                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 4574
#define RSSPECAN_ATTR_FM_STEREO_MPX_AF_FILTER_LOW_PASS_FREQUENCY_RELATIVE                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 4575
#define RSSPECAN_ATTR_FM_STEREO_MONO_AF_FILTER_LOW_PASS_FREQUENCY_RELATIVE                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 4576
#define RSSPECAN_ATTR_FM_STEREO_STEREO_AF_FILTER_LOW_PASS_FREQUENCY_RELATIVE                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 4577
#define RSSPECAN_ATTR_FM_STEREO_RDS_AF_FILTER_LOW_PASS_FREQUENCY_RELATIVE                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 4578
#define RSSPECAN_ATTR_FM_STEREO_PILOT_AF_FILTER_LOW_PASS_FREQUENCY_RELATIVE                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 4579
#define RSSPECAN_ATTR_FM_STEREO_LEFT_AF_FILTER_CCITT                                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 4734
#define RSSPECAN_ATTR_FM_STEREO_RIGHT_AF_FILTER_CCITT                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 4580
#define RSSPECAN_ATTR_FM_STEREO_MPX_AF_FILTER_CCITT                                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 4581
#define RSSPECAN_ATTR_FM_STEREO_MONO_AF_FILTER_CCITT                                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 4582
#define RSSPECAN_ATTR_FM_STEREO_STEREO_AF_FILTER_CCITT                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 4583
#define RSSPECAN_ATTR_FM_STEREO_RDS_AF_FILTER_CCITT                                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 4584
#define RSSPECAN_ATTR_FM_STEREO_PILOT_AF_FILTER_CCITT                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 4585
#define RSSPECAN_ATTR_FM_STEREO_LEFT_AF_FILTER_CCIR_UNWEIGHTED                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 4735
#define RSSPECAN_ATTR_FM_STEREO_RIGHT_AF_FILTER_CCIR_UNWEIGHTED                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 4586
#define RSSPECAN_ATTR_FM_STEREO_MPX_AF_FILTER_CCIR_UNWEIGHTED                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 4587
#define RSSPECAN_ATTR_FM_STEREO_MONO_AF_FILTER_CCIR_UNWEIGHTED                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 4588
#define RSSPECAN_ATTR_FM_STEREO_STEREO_AF_FILTER_CCIR_UNWEIGHTED                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 4589
#define RSSPECAN_ATTR_FM_STEREO_RDS_AF_FILTER_CCIR_UNWEIGHTED                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 4590
#define RSSPECAN_ATTR_FM_STEREO_PILOT_AF_FILTER_CCIR_UNWEIGHTED                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 4591
#define RSSPECAN_ATTR_FM_STEREO_LEFT_AF_FILTER_CCIR_WEIGHTED                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 4736
#define RSSPECAN_ATTR_FM_STEREO_RIGHT_AF_FILTER_CCIR_WEIGHTED                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 4592
#define RSSPECAN_ATTR_FM_STEREO_MPX_AF_FILTER_CCIR_WEIGHTED                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 4593
#define RSSPECAN_ATTR_FM_STEREO_MONO_AF_FILTER_CCIR_WEIGHTED                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 4594
#define RSSPECAN_ATTR_FM_STEREO_STEREO_AF_FILTER_CCIR_WEIGHTED                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 4595
#define RSSPECAN_ATTR_FM_STEREO_RDS_AF_FILTER_CCIR_WEIGHTED                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 4596
#define RSSPECAN_ATTR_FM_STEREO_PILOT_AF_FILTER_CCIR_WEIGHTED                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 4597
#define RSSPECAN_ATTR_FM_STEREO_LEFT_AF_FILTER_A_WEIGHTED                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 4737
#define RSSPECAN_ATTR_FM_STEREO_RIGHT_AF_FILTER_A_WEIGHTED                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 4598
#define RSSPECAN_ATTR_FM_STEREO_MPX_AF_FILTER_A_WEIGHTED                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 4599
#define RSSPECAN_ATTR_FM_STEREO_MONO_AF_FILTER_A_WEIGHTED                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 4600
#define RSSPECAN_ATTR_FM_STEREO_STEREO_AF_FILTER_A_WEIGHTED                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 4601
#define RSSPECAN_ATTR_FM_STEREO_RDS_AF_FILTER_A_WEIGHTED                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 4602
#define RSSPECAN_ATTR_FM_STEREO_PILOT_AF_FILTER_A_WEIGHTED                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 4603
#define RSSPECAN_ATTR_FM_STEREO_LEFT_AF_FILTER_DEEMPHASIS_ENABLED                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 4738
#define RSSPECAN_ATTR_FM_STEREO_RIGHT_AF_FILTER_DEEMPHASIS_ENABLED                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 4604
#define RSSPECAN_ATTR_FM_STEREO_MPX_AF_FILTER_DEEMPHASIS_ENABLED                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 4605
#define RSSPECAN_ATTR_FM_STEREO_MONO_AF_FILTER_DEEMPHASIS_ENABLED                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 4606
#define RSSPECAN_ATTR_FM_STEREO_STEREO_AF_FILTER_DEEMPHASIS_ENABLED                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 4607
#define RSSPECAN_ATTR_FM_STEREO_RDS_AF_FILTER_DEEMPHASIS_ENABLED                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 4608
#define RSSPECAN_ATTR_FM_STEREO_PILOT_AF_FILTER_DEEMPHASIS_ENABLED                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 4609
#define RSSPECAN_ATTR_FM_STEREO_LEFT_AF_FILTER_DEEMPHASIS_TIME_CONSTANT                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 4739
#define RSSPECAN_ATTR_FM_STEREO_RIGHT_AF_FILTER_DEEMPHASIS_TIME_CONSTANT                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 4610
#define RSSPECAN_ATTR_FM_STEREO_MPX_AF_FILTER_DEEMPHASIS_TIME_CONSTANT                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 4611
#define RSSPECAN_ATTR_FM_STEREO_MONO_AF_FILTER_DEEMPHASIS_TIME_CONSTANT                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 4612
#define RSSPECAN_ATTR_FM_STEREO_STEREO_AF_FILTER_DEEMPHASIS_TIME_CONSTANT                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 4613
#define RSSPECAN_ATTR_FM_STEREO_RDS_AF_FILTER_DEEMPHASIS_TIME_CONSTANT                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 4614
#define RSSPECAN_ATTR_FM_STEREO_PILOT_AF_FILTER_DEEMPHASIS_TIME_CONSTANT                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 4615
#define RSSPECAN_ATTR_FM_STEREO_LEFT_AF_FILTER_COUPLED                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 4740
#define RSSPECAN_ATTR_FM_STEREO_RIGHT_AF_FILTER_COUPLED                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 4616
#define RSSPECAN_ATTR_FM_STEREO_MPX_AF_FILTER_COUPLED                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 4617
#define RSSPECAN_ATTR_FM_STEREO_MONO_AF_FILTER_COUPLED                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 4618
#define RSSPECAN_ATTR_FM_STEREO_STEREO_AF_FILTER_COUPLED                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 4619
#define RSSPECAN_ATTR_FM_STEREO_RDS_AF_FILTER_COUPLED                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 4620
#define RSSPECAN_ATTR_FM_STEREO_PILOT_AF_FILTER_COUPLED                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 4621
#define RSSPECAN_ATTR_FMS_DISPLAY_CONFIG                                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 1183
#define RSSPECAN_ATTR_FM_STEREO_DIFFERENCE_FREQUENCY_DISTORTION_SEARCH_SIGNAL                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 1203
#define RSSPECAN_ATTR_FM_STEREO_INTERMODULATION_DISTORTION_SEARCH_SIGNAL                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 1204
#define RSSPECAN_ATTR_QUERY_FM_STEREO_CARRIER_FREQUENCY                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 1205
#define RSSPECAN_ATTR_QUERY_FM_STEREO_LEFT_RESULT_ADEV                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 3381
#define RSSPECAN_ATTR_QUERY_FM_STEREO_RIGHT_RESULT_ADEV                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 4622
#define RSSPECAN_ATTR_QUERY_FM_STEREO_MPX_RESULT_ADEV                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 4623
#define RSSPECAN_ATTR_QUERY_FM_STEREO_MONO_RESULT_ADEV                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 4624
#define RSSPECAN_ATTR_QUERY_FM_STEREO_STEREO_RESULT_ADEV                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 4625
#define RSSPECAN_ATTR_QUERY_FM_STEREO_RDS_RESULT_ADEV                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 4626
#define RSSPECAN_ATTR_QUERY_FM_STEREO_PILOT_RESULT_ADEV                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 4627
#define RSSPECAN_ATTR_QUERY_FM_STEREO_LEFT_RESULT_RDEV                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 3382
#define RSSPECAN_ATTR_QUERY_FM_STEREO_RIGHT_RESULT_RDEV                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 4628
#define RSSPECAN_ATTR_QUERY_FM_STEREO_MPX_RESULT_RDEV                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 4629
#define RSSPECAN_ATTR_QUERY_FM_STEREO_MONO_RESULT_RDEV                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 4630
#define RSSPECAN_ATTR_QUERY_FM_STEREO_STEREO_RESULT_RDEV                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 4631
#define RSSPECAN_ATTR_QUERY_FM_STEREO_RDS_RESULT_RDEV                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 4632
#define RSSPECAN_ATTR_QUERY_FM_STEREO_PILOT_RESULT_RDEV                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 4633
#define RSSPECAN_ATTR_QUERY_FM_STEREO_LEFT_RESULT_SINAD                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 3383
#define RSSPECAN_ATTR_QUERY_FM_STEREO_RIGHT_RESULT_SINAD                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 4634
#define RSSPECAN_ATTR_QUERY_FM_STEREO_MPX_RESULT_SINAD                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 4635
#define RSSPECAN_ATTR_QUERY_FM_STEREO_MONO_RESULT_SINAD                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 4636
#define RSSPECAN_ATTR_QUERY_FM_STEREO_STEREO_RESULT_SINAD                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 4637
#define RSSPECAN_ATTR_QUERY_FM_STEREO_RDS_RESULT_SINAD                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 4638
#define RSSPECAN_ATTR_QUERY_FM_STEREO_PILOT_RESULT_SINAD                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 4639
#define RSSPECAN_ATTR_QUERY_FM_STEREO_LEFT_RESULT_THD                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 3384
#define RSSPECAN_ATTR_QUERY_FM_STEREO_RIGHT_RESULT_THD                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 4640
#define RSSPECAN_ATTR_QUERY_FM_STEREO_MPX_RESULT_THD                                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 4641
#define RSSPECAN_ATTR_QUERY_FM_STEREO_MONO_RESULT_THD                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 4642
#define RSSPECAN_ATTR_QUERY_FM_STEREO_STEREO_RESULT_THD                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 4643
#define RSSPECAN_ATTR_QUERY_FM_STEREO_RDS_RESULT_THD                                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 4644
#define RSSPECAN_ATTR_QUERY_FM_STEREO_PILOT_RESULT_THD                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 4645
#define RSSPECAN_ATTR_QUERY_FM_STEREO_LEFT_RESULT_AFR                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 3385
#define RSSPECAN_ATTR_QUERY_FM_STEREO_RIGHT_RESULT_AFR                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 4646
#define RSSPECAN_ATTR_QUERY_FM_STEREO_MPX_RESULT_AFR                                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 4647
#define RSSPECAN_ATTR_QUERY_FM_STEREO_MONO_RESULT_AFR                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 4648
#define RSSPECAN_ATTR_QUERY_FM_STEREO_STEREO_RESULT_AFR                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 4649
#define RSSPECAN_ATTR_QUERY_FM_STEREO_RDS_RESULT_AFR                                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 4650
#define RSSPECAN_ATTR_QUERY_FM_STEREO_PILOT_RESULT_AFR                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 4651
#define RSSPECAN_ATTR_QUERY_FM_STEREO_LEFT_RESULT_XTALK                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 3386
#define RSSPECAN_ATTR_QUERY_FM_STEREO_RIGHT_RESULT_XTALK                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 4652
#define RSSPECAN_ATTR_QUERY_FM_STEREO_MPX_RESULT_XTALK                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 4653
#define RSSPECAN_ATTR_QUERY_FM_STEREO_MONO_RESULT_XTALK                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 4654
#define RSSPECAN_ATTR_QUERY_FM_STEREO_STEREO_RESULT_XTALK                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 4655
#define RSSPECAN_ATTR_QUERY_FM_STEREO_RDS_RESULT_XTALK                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 4656
#define RSSPECAN_ATTR_QUERY_FM_STEREO_PILOT_RESULT_XTALK                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 4657
#define RSSPECAN_ATTR_QUERY_FM_STEREO_PHASE_NOISE_RESULT                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 1206
#define RSSPECAN_ATTR_BTO_MODE                                                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 1223
#define RSSPECAN_ATTR_BTO_STANDARD                                                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 10861
#define RSSPECAN_ATTR_BTO_LOW_ENERGY_STANDARD                                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 10862
#define RSSPECAN_ATTR_BTO_SAVE_SETTINGS                                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 10863
#define RSSPECAN_ATTR_BTO_LOAD_SETTINGS                                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 10864
#define RSSPECAN_ATTR_BTO_SAVE_MEASUREMENT                                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 10865
#define RSSPECAN_ATTR_BTO_PACKET_TYPE                                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 1208
#define RSSPECAN_ATTR_BTO_SAMPLES_PER_SYMBOLS                                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 1209
#define RSSPECAN_ATTR_BTO_OUTPUT_POWER_CLASS                                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 1210
#define RSSPECAN_ATTR_BTO_POSITION_OF_AVERAGE_POWER_START                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 5592
#define RSSPECAN_ATTR_BTO_POSITION_OF_AVERAGE_POWER_STOP                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 3209
#define RSSPECAN_ATTR_BTO_RESOLUTION_BANDWIDTH_AUTO                                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 1212
#define RSSPECAN_ATTR_BTO_RESOLUTION_BANDWIDTH                                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 1213
#define RSSPECAN_ATTR_BTO_VIDEO_BANDWIDTH_AUTO                                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 1214
#define RSSPECAN_ATTR_BTO_VIDEO_BANDWIDTH                                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 1215
#define RSSPECAN_ATTR_BTO_MEASUREMENT_DEMODULATION_BANDWIDTH                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 10866
#define RSSPECAN_ATTR_BTO_MEASUREMENT_FILTER_ENABLED                                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 10867
#define RSSPECAN_ATTR_BTO_MEASUREMENT_FILTER                                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 1216
#define RSSPECAN_ATTR_BTO_SWEEP_COUNT                                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 1220
#define RSSPECAN_ATTR_BTO_SWEEP_TIME_AUTO                                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 1221
#define RSSPECAN_ATTR_BTO_SWEEP_TIME                                                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 1222
#define RSSPECAN_ATTR_BTO_MEASUREMENT_MODE                                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 1224
#define RSSPECAN_ATTR_BTO_SYNCHRONIZATION_SEARCH_ENABLED                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 10868
#define RSSPECAN_ATTR_BTO_SYNCHRONIZATION_LAP                                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 10869
#define RSSPECAN_ATTR_BTO_PULSE_SEARCH_ENABLED                                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 10870
#define RSSPECAN_ATTR_BTO_LOWEST_24_BITS_OF_BLUETOOTH_DEVICE_ADDRESS                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 1225
#define RSSPECAN_ATTR_BTO_FREQUENCY_CHANNEL                                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 1226
#define RSSPECAN_ATTR_BTO_NUMBER_OF_ADJACENT_CHANNELS                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 1227
#define RSSPECAN_ATTR_BTO_GEOGRAPHICAL_REGION                                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 1228
#define RSSPECAN_ATTR_BTO_ANTENNA_GAIN                                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 1229
#define RSSPECAN_ATTR_BTO_BURST_OFFSET                                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 1230
#define RSSPECAN_ATTR_BTO_FIND_BURST_ENABLED                                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 1231
#define RSSPECAN_ATTR_BTO_SYNC_OFFSET                                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 1232
#define RSSPECAN_ATTR_BTO_FIND_SYNC_ENABLED                                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 1233
#define RSSPECAN_ATTR_BTO_SEARCH_TIME                                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 1234
#define RSSPECAN_ATTR_BTO_SEARCH_TIME_AUTO                                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 1235
#define RSSPECAN_ATTR_BTO_CFST_BLOCK_COUNT                                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 1236
#define RSSPECAN_ATTR_BTO_IBS_ACP                                                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 1237
#define RSSPECAN_ATTR_BTO_IBS_GATE_AUTO                                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 1238
#define RSSPECAN_ATTR_BTO_SPURIOUS_EMISSIONS_GEOGRAPHY                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 10871
#define RSSPECAN_ATTR_BTO_SPURIOUS_EMISSIONS_NUMBER_OF_CAPTURED_CHANNELS                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 10872
#define RSSPECAN_ATTR_BTO_PBSC                                                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 1239
#define RSSPECAN_ATTR_BTO_RTP_TIME_DPSK_START                                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 4741
#define RSSPECAN_ATTR_BTO_RTP_TIME_DPSK_STOP                                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 4742
#define RSSPECAN_ATTR_BTO_RTP_TIME_GFSK_START                                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 4743
#define RSSPECAN_ATTR_BTO_RTP_TIME_GFSK_STOP                                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 4744
#define RSSPECAN_ATTR_BTO_LIMIT_CHECKS_FREQUENCY_DRIFT_LOWER                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 10873
#define RSSPECAN_ATTR_BTO_LIMIT_CHECKS_FREQUENCY_DRIFT_UPPER                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 10874
#define RSSPECAN_ATTR_BTO_LIMIT_CHECKS_FREQUENCY_OFFSET_LOWER                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 10875
#define RSSPECAN_ATTR_BTO_LIMIT_CHECKS_FREQUENCY_OFFSET_UPPER                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 10876
#define RSSPECAN_ATTR_BTO_LIMIT_CHECKS_ICFT_LOWER                                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 10877
#define RSSPECAN_ATTR_BTO_LIMIT_CHECKS_ICFT_UPPER                                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 10878
#define RSSPECAN_ATTR_BTO_LIMIT_CHECKS_INITIAL_FREQUENCY_DRIFT_LOWER                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 10879
#define RSSPECAN_ATTR_BTO_LIMIT_CHECKS_INITIAL_FREQUENCY_DRIFT_UPPER                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 10880
#define RSSPECAN_ATTR_BTO_LIMIT_CHECKS_MAX_DRIFT_RATE_LOWER                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 10881
#define RSSPECAN_ATTR_BTO_LIMIT_CHECKS_MAX_DRIFT_RATE_UPPER                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 10882
#define RSSPECAN_ATTR_BTO_LIMIT_CHECKS_F1_AVG_LOWER                                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 10883
#define RSSPECAN_ATTR_BTO_LIMIT_CHECKS_F1_AVG_UPPER                                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 10884
#define RSSPECAN_ATTR_BTO_LIMIT_CHECKS_F1_MAX_IN_RANGE                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 10885
#define RSSPECAN_ATTR_BTO_LIMIT_CHECKS_F1_MAX_LOWER                                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 10886
#define RSSPECAN_ATTR_BTO_LIMIT_CHECKS_F2_AVG_F1_AVG_RATIO_LOWER                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 10887
#define RSSPECAN_ATTR_BTO_LIMIT_CHECKS_F2_MAX_IN_RANGE                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 10888
#define RSSPECAN_ATTR_BTO_LIMIT_CHECKS_F2_MAX_LOWER                                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 10889
#define RSSPECAN_ATTR_BTO_LIMIT_CHECKS_POWER_AVERAGE_LOWER                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 10890
#define RSSPECAN_ATTR_BTO_LIMIT_CHECKS_POWER_AVERAGE_UPPER                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 10891
#define RSSPECAN_ATTR_BTO_LIMIT_CHECKS_POWER_RELATIVE_PEAK_UPPER                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 10892
#define RSSPECAN_ATTR_BTO_LIMIT_CHECKS_POWER_PEAK_UPPER                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 10893
#define RSSPECAN_ATTR_BTO_LIMIT_CHECKS_RELATIVE_TX_POWER_AVG_UPPER                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 10894
#define RSSPECAN_ATTR_BTO_LIMIT_CHECKS_RELATIVE_TX_POWER_AVG_LOWER                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 10895
#define RSSPECAN_ATTR_BTO_LIMIT_CHECKS_PACKET_MAXIMUM_DRIFT_UPPER                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 10896
#define RSSPECAN_ATTR_BTO_LIMIT_CHECKS_PACKET_MAXIMUM_DRIFT_LOWER                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 10897
#define RSSPECAN_ATTR_BTO_LIMIT_CHECKS_PACKET_FREQUENCY_DRIFT_UPPER                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 10898
#define RSSPECAN_ATTR_BTO_LIMIT_CHECKS_PACKET_FREQUENCY_DRIFT_LOWER                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 10899
#define RSSPECAN_ATTR_BTO_LIMIT_CHECKS_BLOCK_FREQUENCY_UPPER                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 10900
#define RSSPECAN_ATTR_BTO_LIMIT_CHECKS_BLOCK_FREQUENCY_LOWER                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 10901
#define RSSPECAN_ATTR_BTO_LIMIT_CHECKS_INITIAL_FREQUENCY_UPPER                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 10902
#define RSSPECAN_ATTR_BTO_LIMIT_CHECKS_INITIAL_FREQUENCY_LOWER                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 10903
#define RSSPECAN_ATTR_BTO_LIMIT_CHECKS_TOTAL_FREQUENCY_OFFSET_UPPER                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 10904
#define RSSPECAN_ATTR_BTO_LIMIT_CHECKS_TOTAL_FREQUENCY_OFFSET_LOWER                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 10905
#define RSSPECAN_ATTR_BTO_LIMIT_CHECKS_PEAK_DQPSK_UPPER                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 10906
#define RSSPECAN_ATTR_BTO_LIMIT_CHECKS_RMS_DQPSK_UPPER                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 10907
#define RSSPECAN_ATTR_BTO_LIMIT_CHECKS_99_DQPSK_UPPER                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 10908
#define RSSPECAN_ATTR_BTO_LIMIT_CHECKS_PEAK_8PSK_UPPER                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 10909
#define RSSPECAN_ATTR_BTO_LIMIT_CHECKS_RMS_8PSK_UPPER                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 10910
#define RSSPECAN_ATTR_BTO_LIMIT_CHECKS_99_8PSK_UPPER                                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 10911
#define RSSPECAN_ATTR_BTO_LIMIT_CHECKS_FAR_LIMIT                                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 10912
#define RSSPECAN_ATTR_BTO_LIMIT_CHECKS_NEAR_LIMIT                                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 10913
#define RSSPECAN_ATTR_BTO_LIMIT_CHECKS_L2_FREQUENCY_POWER_DIFFERENCE                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 10914
#define RSSPECAN_ATTR_BTO_LIMIT_CHECKS_L4_FREQUENCY_POWER_DIFFERENCE                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 10915
#define RSSPECAN_ATTR_BTO_LIMIT_CHECKS_L5_FREQUENCY_POWER_DIFFERENCE                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 10916
#define RSSPECAN_ATTR_BTO_LIMIT_CHECKS_ALLOWED_EXCEPTIONS                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 10917
#define RSSPECAN_ATTR_BTO_LIMIT_CHECKS_RELATIVE                                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 10918
#define RSSPECAN_ATTR_BTO_TRACE_MODE                                                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 1217
#define RSSPECAN_ATTR_BTO_TRACE_SELECT                                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 1218
#define RSSPECAN_ATTR_BTO_TRACE_DETECTOR                                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 1219
#define RSSPECAN_ATTR_BTO_TRACE_LIMIT_CHECK_ENABLED                                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 10919
#define RSSPECAN_ATTR_BTO_GET_OUTPUT_POWER_PEAK                                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 1244
#define RSSPECAN_ATTR_BTO_GET_OUTPUT_POWER_AVERAGE_MIN                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 3391
#define RSSPECAN_ATTR_BTO_GET_OUTPUT_POWER_AVERAGE_MAX                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 3392
#define RSSPECAN_ATTR_BTO_GET_STATUS                                                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 1246
#define RSSPECAN_ATTR_BTO_GET_NUMBER_OF_EXCEPTIONS                                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 1247
#define RSSPECAN_ATTR_BTO_GET_RATIO_OF_AVERAGE_FREQUENCY_DEVIATIONS_VALUE_MIN                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 3393
#define RSSPECAN_ATTR_BTO_GET_RATIO_OF_AVERAGE_FREQUENCY_DEVIATIONS_VALUE_MAX                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 3394
#define RSSPECAN_ATTR_BTO_GET_RATIO_OF_AVERAGE_FREQUENCY_DEVIATIONS_VALUE_AVERAGE                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 3395
#define RSSPECAN_ATTR_BTO_GET_PERCENTAGE_DEVIATION                                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 1251
#define RSSPECAN_ATTR_BTO_GET_INITIAL_CARRIER_FREQUENCY_TOLERANCE_VALUE_MIN                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 3396
#define RSSPECAN_ATTR_BTO_GET_INITIAL_CARRIER_FREQUENCY_TOLERANCE_VALUE_MAX                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 3397
#define RSSPECAN_ATTR_BTO_GET_INITIAL_CARRIER_FREQUENCY_TOLERANCE_VALUE_AVERAGE                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 3398
#define RSSPECAN_ATTR_BTO_GET_MAXIMUM_CARRIER_FREQUENCY_DRIFT                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 1253
#define RSSPECAN_ATTR_BTO_GET_CARRIER_FREQUENCY_DRIFT                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 1254
#define RSSPECAN_ATTR_BTO_GET_PACKET_LENGTH                                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 1255
#define RSSPECAN_ATTR_BTO_GET_PACKET_TYPE                                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 1256
#define RSSPECAN_ATTR_BTO_CFST_BLOCK_COUNT_VALUE                                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 4745
#define RSSPECAN_ATTR_BTO_CFST_DEVM_RMS_MIN                                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 4746
#define RSSPECAN_ATTR_BTO_CFST_DEVM_RMS_MAX                                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 4747
#define RSSPECAN_ATTR_BTO_CFST_DEVM_RMS_AVER                                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 4748
#define RSSPECAN_ATTR_BTO_CFST_DEVM_PEAK                                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 1259
#define RSSPECAN_ATTR_BTO_CFST_DEVM_99PCT                                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 1260
#define RSSPECAN_ATTR_BTO_CFST_FERR_TOTAL_MIN                                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 4749
#define RSSPECAN_ATTR_BTO_CFST_FERR_TOTAL_MAX                                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 4750
#define RSSPECAN_ATTR_BTO_CFST_FERR_TOTAL_AVER                                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 4751
#define RSSPECAN_ATTR_BTO_CFST_FERR_BLOCK_MIN                                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 4752
#define RSSPECAN_ATTR_BTO_CFST_FERR_BLOCK_MAX                                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 4753
#define RSSPECAN_ATTR_BTO_CFST_FERR_BLOCK_AVER                                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 4754
#define RSSPECAN_ATTR_BTO_CFST_FERR_INITIAL_MIN                                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 4755
#define RSSPECAN_ATTR_BTO_CFST_FERR_INITIAL_MAX                                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 4756
#define RSSPECAN_ATTR_BTO_CFST_FERR_INITIAL_AVER                                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 4757
#define RSSPECAN_ATTR_BTO_DPEN_PACKETS                                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 1264
#define RSSPECAN_ATTR_BTO_DPEN_PACKETS_PASSED                                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 1265
#define RSSPECAN_ATTR_BTO_DPEN_BER                                                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 1266
#define RSSPECAN_ATTR_BTO_IBS_EXC                                                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 1267
#define RSSPECAN_ATTR_BTO_IBS_HADJ_UPP                                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 5593
#define RSSPECAN_ATTR_BTO_IBS_HADJ_LOW                                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 4758
#define RSSPECAN_ATTR_BTO_IBS_TXCHANNEL                                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 1269
#define RSSPECAN_ATTR_BTO_RTP_DPSK_MIN                                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 4759
#define RSSPECAN_ATTR_BTO_RTP_DPSK_MAX                                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 4760
#define RSSPECAN_ATTR_BTO_RTP_DPSK_AVER                                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 4761
#define RSSPECAN_ATTR_BTO_RTP_GFSK_MIN                                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 4762
#define RSSPECAN_ATTR_BTO_RTP_GFSK_MAX                                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 4763
#define RSSPECAN_ATTR_BTO_RTP_GFSK_AVER                                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 4764
#define RSSPECAN_ATTR_BTO_RTP_RATIO_MIN                                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 4765
#define RSSPECAN_ATTR_BTO_RTP_RATIO_MAX                                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 4766
#define RSSPECAN_ATTR_BTO_RTP_RATIO_AVER                                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 4767
#define RSSPECAN_ATTR_BTO_GET_AVERAGE_FREQUENCY_DEVIATION_MIN                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 3399
#define RSSPECAN_ATTR_BTO_GET_AVERAGE_FREQUENCY_DEVIATION_MAX                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 3400
#define RSSPECAN_ATTR_BTO_GET_MAXIMUM_FREQUENCY_DEVIATION_VALUE_MIN                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 3401
#define RSSPECAN_ATTR_BTO_GET_MAXIMUM_FREQUENCY_DEVIATION_VALUE_MAX                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 3402
#define RSSPECAN_ATTR_BTO_GET_MAXIMUM_FREQUENCY_DEVIATION_VALUE_AVERAGE                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 3403
#define RSSPECAN_ATTR_PWM_SELECT                                                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 2381
#define RSSPECAN_ATTR_PWM_ZERO                                                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 1278
#define RSSPECAN_ATTR_PWM_DUTY_CYCLE_ENABLED                                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 1287
#define RSSPECAN_ATTR_PWM_DUTY_CYCLE                                                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 1288
#define RSSPECAN_ATTR_PWM_AUTOMATIC_ASSIGNMENT                                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 2382
#define RSSPECAN_ATTR_PWM_NUMBER_OF_SENSORS                                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 2383
#define RSSPECAN_ATTR_PWM_REFERENCE_VALUE                                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 1272
#define RSSPECAN_ATTR_PWM_MEASURE_TO_REFERENCE                                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 1273
#define RSSPECAN_ATTR_PWM_RELABS_DISPLAY                                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 3219
#define RSSPECAN_ATTR_PWM_ENABLED                                                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 1281
#define RSSPECAN_ATTR_PWM_CONTINUOUS_UPDATE_ENABLED                                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 6985
#define RSSPECAN_ATTR_PWM_FREQUENCY                                                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 1275
#define RSSPECAN_ATTR_PWM_MEASUREMENT_TIME                                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 1276
#define RSSPECAN_ATTR_PWM_FREQUENCY_COUPLING                                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 1277
#define RSSPECAN_ATTR_PWM_ABSOLUTE_UNIT                                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 1279
#define RSSPECAN_ATTR_PWM_RELATIVE_UNITS                                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 1280
#define RSSPECAN_ATTR_PWM_AVERAGE_AUTO                                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 1282
#define RSSPECAN_ATTR_PWM_AVERAGE_COUNT                                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 1283
#define RSSPECAN_ATTR_PWM_REFERENCE_LEVEL_OFFSET_ENABLED                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 1284
#define RSSPECAN_ATTR_PWM_EXTERNAL_POWER_TRIGGER_LEVEL                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 1286
#define RSSPECAN_ATTR_PWM_EXTERNAL_POWER_TRIGGER_ENABLED                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 1285
#define RSSPECAN_ATTR_PWM_EXTERNAL_POWER_TRIGGER_HYSTERESIS                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 2634
#define RSSPECAN_ATTR_PWM_EXTERNAL_POWER_TRIGGER_DROPOUT_TIME                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 2635
#define RSSPECAN_ATTR_PWM_EXTERNAL_POWER_TRIGGER_HOLDOFF_TIME                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 2636
#define RSSPECAN_ATTR_PWM_EXTERNAL_POWER_TRIGGER_SLOPE                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 2637
#define RSSPECAN_ATTR_PWM_CORRECTION_LEVEL_AUTO                                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 10920
#define RSSPECAN_ATTR_PWM_CORRECTION_LEVEL                                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 10921
#define RSSPECAN_ATTR_PWM_RESULT_DISPLAY                                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 3218
#define RSSPECAN_ATTR_PWM_SENSOR_OFFSET                                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 8644
#define RSSPECAN_ATTR_PWM_READ_RESULT                                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 1294
#define RSSPECAN_ATTR_PMET_FETCH                                                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 1295
#define RSSPECAN_ATTR_GSM_K10_MODE_FSV                                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 1297
#define RSSPECAN_ATTR_GSM_K10_DUT                                                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 1298
#define RSSPECAN_ATTR_GSM_K10_SIG_CHAR_MODE                                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 1299
#define RSSPECAN_ATTR_GSM_K10_SIG_CHAR_BAND                                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 1300
#define RSSPECAN_ATTR_GSM_K10_SIG_CHAR_BAND_ARFC                                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 1301
#define RSSPECAN_ATTR_GSM_K10_AUTO_PWR_LEV                                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 1302
#define RSSPECAN_ATTR_GSM_K10_AUTO_PWR_LEV_ONCE                                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 1303
#define RSSPECAN_ATTR_GSM_K10_LEVEL_SETTINGS_PWR_CLASS                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 1304
#define RSSPECAN_ATTR_GSM_K10_STATISTIC_COUNT                                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 3423
#define RSSPECAN_ATTR_GSM_K10_TRIGGER_TO_SYNC_ACQUISITIONS                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 3424
#define RSSPECAN_ATTR_GSM_K10_CORRELATION_THRESHOLD                                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 2384
#define RSSPECAN_ATTR_GSM_K10_SYNCHRONIZATION                                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 1306
#define RSSPECAN_ATTR_GSM_K10_MEASURE_ONLY_ON_SYNC                                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 1307
#define RSSPECAN_ATTR_GSM_K10_CAP_SET_TRACK_TIME                                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 1308
#define RSSPECAN_ATTR_GSM_K10_IQ_SWAP                                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 2385
#define RSSPECAN_ATTR_GSM_K10_MARKER_ZOOM_FACTOR                                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 2386
#define RSSPECAN_ATTR_GSM_K10_RESULT_DISPLAY                                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 2387
#define RSSPECAN_ATTR_GSM_MEASUREMENT                                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 5017
#define RSSPECAN_ATTR_GSM_K10_AUTO_SET_LEVEL                                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 1309
#define RSSPECAN_ATTR_GSM_K10_AUTO_SET_LEVEL_ONCE                                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 1310
#define RSSPECAN_ATTR_GSM_K10_AUTO_SET_FRAME                                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 1311
#define RSSPECAN_ATTR_GSM_K10_AUTO_SET_FRAME_ONCE                                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 1312
#define RSSPECAN_ATTR_GSM_K10_AUTO_SET_TRIGGER                                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 1313
#define RSSPECAN_ATTR_GSM_K10_AUTO_SET_TRIGGER_ONCE                                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 1314
#define RSSPECAN_ATTR_GSM_K10_DEMOD_SINGLE_SLOT_MEAS                                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 1315
#define RSSPECAN_ATTR_GSM_K10_DEM_MULTI_SLOT_MEAS_SLOT_NUM                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 1316
#define RSSPECAN_ATTR_GSM_K10_DEM_MULTI_SLOT_MEAS_FIRST_SLOT                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 1317
#define RSSPECAN_ATTR_GSM_K10_DEMODULATION_EQUAL_TIMESLOT_LENGTH                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 1318
#define RSSPECAN_ATTR_GSM_K10_BURST_SLOT_STATE                                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 1319
#define RSSPECAN_ATTR_GSM_K10_BURST_TYPE_BURST                                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 1320
#define RSSPECAN_ATTR_GSM_K10_BURST_MOD                                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 1322
#define RSSPECAN_ATTR_GSM_K10_BURST_FILTER                                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 1323
#define RSSPECAN_ATTR_GSM_K10_BURST_TSC                                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 1324
#define RSSPECAN_ATTR_GSM_K10_BURST_TSC_USER_VALUE                                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 2388
#define RSSPECAN_ATTR_GSM_K10_BURST_TIMING_ADVANCE                                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 2582
#define RSSPECAN_ATTR_GSM_K10_BURST_SYNC                                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 2583
#define RSSPECAN_ATTR_GSM_K10_BURST_SCPIR                                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 2584
#define RSSPECAN_ATTR_GSM_K10_BURST_SUBCH_TSC                                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 2585
#define RSSPECAN_ATTR_GSM_K10_BURST_SUBCH_TSC_USER_VALUE                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 2586
#define RSSPECAN_ATTR_GSM_K10_TRACE_TYPE                                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 2587
#define RSSPECAN_ATTR_GSM_MCWN_MAXIMUM_OUTPUT_POWER_PER_CARRIER_ENABLED                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 5018
#define RSSPECAN_ATTR_GSM_MCWN_MAXIMUM_OUTPUT_POWER_PER_CARRIER                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 5019
#define RSSPECAN_ATTR_GSM_MCWN_REFERENCE_POWER_MEASURE                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 5020
#define RSSPECAN_ATTR_GSM_MCWN_REFERENCE_POWER_LEVEL                                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 5021
#define RSSPECAN_ATTR_GSM_MCWN_WIDEBAND_NOISE_ENABLED                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 5023
#define RSSPECAN_ATTR_GSM_MCWN_NARROWBAND_NOISE_ENABLED                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 5024
#define RSSPECAN_ATTR_GSM_MCWN_INTERMODULATION_ORDER                                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 5025
#define RSSPECAN_ATTR_GSM_MCWN_LIMIT_EXCEPTION_ENABLED                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 5026
#define RSSPECAN_ATTR_GSM_MCWN_AVERAGE_COUNT                                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 5027
#define RSSPECAN_ATTR_GSM_MCWN_CARRIER_AUTO_ENABLED                                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 5028
#define RSSPECAN_ATTR_GSM_MCWN_CARRIER_NUMBER                                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 5029
#define RSSPECAN_ATTR_GSM_MCWN_CARRIER_ALLOCATION                                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 5030
#define RSSPECAN_ATTR_GSM_MCWN_CARRIER_GAP_START                                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 5031
#define RSSPECAN_ATTR_GSM_MCWN_CARRIER_ENABLED                                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 5032
#define RSSPECAN_ATTR_GSM_MCWN_CARRIER_FREQUENCY                                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 5033
#define RSSPECAN_ATTR_GSM_MCWN_CARRIER_MODULATION                                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 5034
#define RSSPECAN_ATTR_GSM_MEASUREMENT_BURST_TYPE_MACC                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 3427
#define RSSPECAN_ATTR_GSM_MEASUREMENT_BURST_TYPE_PFER                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 3428
#define RSSPECAN_ATTR_GSM_MEASUREMENT_BURST_TYPE_ETIM                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 3429
#define RSSPECAN_ATTR_GSM_MEASUREMENT_BURST_TYPE_MERR                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 3430
#define RSSPECAN_ATTR_GSM_MEASUREMENT_BURST_TYPE_CONS                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 3431
#define RSSPECAN_ATTR_GSM_MEASUREMENT_BURST_TYPE_PTEM                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 3432
#define RSSPECAN_ATTR_GSM_MEASUREMENT_SPECTRUM_TYPE_MODULATION                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 3433
#define RSSPECAN_ATTR_GSM_MEASUREMENT_SPECTRUM_TYPE_SWITCHING                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 3434
#define RSSPECAN_ATTR_GSM_K10_MEASUREMENT_WIDE_SPECTRUM                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 2389
#define RSSPECAN_ATTR_GSM_K10_WIDE_SPECTRUM_UNIT                                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 2638
#define RSSPECAN_ATTR_GSM_K10_MEASUREMENT_TRIGGER_TO_SYNC                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 2639
#define RSSPECAN_ATTR_GSM_K10_TRIGGER_TO_SYNC_BINS_COUNT                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 2640
#define RSSPECAN_ATTR_GSM_K10_TRIGGER_TO_SYNC_ADAPTIVE_DATA_SIZE                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 2641
#define RSSPECAN_ATTR_GSM_K10_PWR_VS_TIME_MEAS                                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 1327
#define RSSPECAN_ATTR_GSM_K10_POWER_VS_TIME_FILTER                                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 1328
#define RSSPECAN_ATTR_GSM_K10_POWER_VS_TIME_ALIGNMENT                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 2390
#define RSSPECAN_ATTR_GSM_K10_MODULATION_TRANSIENT_SPECTRUM_ENABLE_LEFT_LIMIT                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 1329
#define RSSPECAN_ATTR_GSM_K10_MODULATION_TRANSIENT_SPECTRUM_ENABLE_RIGHT_LIMIT                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 1330
#define RSSPECAN_ATTR_GSM_K10_MODULATION_TRANSIENT_SPECTRUM_FILTER_TYPE                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 3435
#define RSSPECAN_ATTR_GSM_K10_MODULATION_TRANSIENT_SPECTRUM_REF_POWER                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 3437
#define RSSPECAN_ATTR_GSM_K10_MODULATION_TRANSIENT_SPECTRUM_HIGH_DYNAMIC                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 3438
#define RSSPECAN_ATTR_GSM_K10_MODULATION_TRANSIENT_SPECTRUM_FREQUENCY_LIST                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 3439
#define RSSPECAN_ATTR_GSM_K10_DEMODULATION_SYMBOL_DECISION                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 2588
#define RSSPECAN_ATTR_GSM_K10_DEMODULATION_TAIL_TSC_BITS                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 2589
#define RSSPECAN_ATTR_GSM_K10_MULTI_MEAS_STATE                                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 2590
#define RSSPECAN_ATTR_GSM_K10_MULTI_MEAS_POWER_VS_TIME                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 2591
#define RSSPECAN_ATTR_GSM_K10_MULTI_MEAS_DEMODULATION                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 2592
#define RSSPECAN_ATTR_GSM_K10_MULTI_MEAS_CONSTELLATION                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 2593
#define RSSPECAN_ATTR_GSM_K10_MULTI_MEAS_MODULATION_SPECTRUM                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 2594
#define RSSPECAN_ATTR_GSM_K10_MULTI_MEAS_TRANSIENT_SPECTRUM                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 2595
#define RSSPECAN_ATTR_GSM_K10_MULTI_CARRIER_BTS                                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 1331
#define RSSPECAN_ATTR_GSM_K10_MULTI_CARRIER_NUMBER_OF_ACTIVE_CHANNELS                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 1332
#define RSSPECAN_ATTR_GSM_K10_MULTI_CARRIER_BTS_CLASS                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 1333
#define RSSPECAN_ATTR_GSM_K10_MULTI_CARRIER_FILTER                                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 1334
#define RSSPECAN_ATTR_GSM_K10_REFRESH_CAPTURED_DATA                                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 2660
#define RSSPECAN_ATTR_QUERY_GSM_MCWN_LIMIT_EXCEPTION_CURRENT                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 5035
#define RSSPECAN_ATTR_QUERY_GSM_MCWN_LIMIT_EXCEPTION_MAXIMUM                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 5036
#define RSSPECAN_ATTR_GSM_MCWN_LIMIT_CHECK_STATE                                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 5037
#define RSSPECAN_ATTR_READ_GSM_BURST_SLOT_RESULT_ALL_AVERAGE                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 3440
#define RSSPECAN_ATTR_READ_GSM_BURST_SLOT_RESULT_ALL_CREST                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 3441
#define RSSPECAN_ATTR_READ_GSM_BURST_SLOT_RESULT_ALL_MAXIMUM                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 3442
#define RSSPECAN_ATTR_READ_GSM_BURST_SLOT_RESULT_CURRENT_AVERAGE                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 3443
#define RSSPECAN_ATTR_READ_GSM_BURST_SLOT_RESULT_CURRENT_CREST                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 3444
#define RSSPECAN_ATTR_READ_GSM_BURST_SLOT_RESULT_CURRENT_MAXIMUM                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 3445
#define RSSPECAN_ATTR_FETCH_GSM_BURST_SLOT_RESULT_ALL_AVERAGE                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 3446
#define RSSPECAN_ATTR_FETCH_GSM_BURST_SLOT_RESULT_ALL_CREST                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 3447
#define RSSPECAN_ATTR_FETCH_GSM_BURST_SLOT_RESULT_ALL_MAXIMUM                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 3448
#define RSSPECAN_ATTR_FETCH_GSM_BURST_SLOT_RESULT_CURRENT_AVERAGE                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 3449
#define RSSPECAN_ATTR_FETCH_GSM_BURST_SLOT_RESULT_CURRENT_CREST                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 3450
#define RSSPECAN_ATTR_FETCH_GSM_BURST_SLOT_RESULT_CURRENT_MAXIMUM                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 3451
#define RSSPECAN_ATTR_READ_GSM_K10_BURST_SLOT_DELTA_TO_SYNC                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 2391
#define RSSPECAN_ATTR_FETCH_GSM_K10_BURST_SLOT_DELTA_TO_SYNC                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 2392
#define RSSPECAN_ATTR_READ_GSM_MEASUREMENT_RESULTS_RMS_AVERAGE                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 3461
#define RSSPECAN_ATTR_READ_GSM_MEASUREMENT_RESULTS_RMS_CURRENT                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 3462
#define RSSPECAN_ATTR_READ_GSM_MEASUREMENT_RESULTS_RMS_MAXIMUM                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 3463
#define RSSPECAN_ATTR_READ_GSM_MEASUREMENT_RESULTS_RMS_SDEV                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 3464
#define RSSPECAN_ATTR_READ_GSM_MEASUREMENT_RESULTS_MERROR_RMS_AVERAGE                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 3465
#define RSSPECAN_ATTR_READ_GSM_MEASUREMENT_RESULTS_MERROR_RMS_CURRENT                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 3466
#define RSSPECAN_ATTR_READ_GSM_MEASUREMENT_RESULTS_MERROR_RMS_MAXIMUM                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 3467
#define RSSPECAN_ATTR_READ_GSM_MEASUREMENT_RESULTS_MERROR_RMS_SDEV                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 3468
#define RSSPECAN_ATTR_READ_GSM_MEASUREMENT_RESULTS_PERROR_RMS_AVERAGE                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 3469
#define RSSPECAN_ATTR_READ_GSM_MEASUREMENT_RESULTS_PERROR_RMS_CURRENT                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 3470
#define RSSPECAN_ATTR_READ_GSM_MEASUREMENT_RESULTS_PERROR_RMS_MAXIMUM                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 3471
#define RSSPECAN_ATTR_READ_GSM_MEASUREMENT_RESULTS_PERROR_RMS_SDEV                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 3472
#define RSSPECAN_ATTR_READ_GSM_MEASUREMENT_RESULTS_PEAK_AVERAGE                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 3473
#define RSSPECAN_ATTR_READ_GSM_MEASUREMENT_RESULTS_PEAK_CURRENT                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 3474
#define RSSPECAN_ATTR_READ_GSM_MEASUREMENT_RESULTS_PEAK_MAXIMUM                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 3475
#define RSSPECAN_ATTR_READ_GSM_MEASUREMENT_RESULTS_PEAK_SDEV                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 3476
#define RSSPECAN_ATTR_READ_GSM_MEASUREMENT_RESULTS_MERROR_PEAK_AVERAGE                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 3477
#define RSSPECAN_ATTR_READ_GSM_MEASUREMENT_RESULTS_MERROR_PEAK_CURRENT                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 3478
#define RSSPECAN_ATTR_READ_GSM_MEASUREMENT_RESULTS_MERROR_PEAK_MAXIMUM                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 3479
#define RSSPECAN_ATTR_READ_GSM_MEASUREMENT_RESULTS_MERROR_PEAK_SDEV                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 3480
#define RSSPECAN_ATTR_READ_GSM_MEASUREMENT_RESULTS_PERROR_PEAK_AVERAGE                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 3481
#define RSSPECAN_ATTR_READ_GSM_MEASUREMENT_RESULTS_PERROR_PEAK_CURRENT                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 3482
#define RSSPECAN_ATTR_READ_GSM_MEASUREMENT_RESULTS_PERROR_PEAK_MAXIMUM                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 3483
#define RSSPECAN_ATTR_READ_GSM_MEASUREMENT_RESULTS_PERROR_PEAK_SDEV                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 3484
#define RSSPECAN_ATTR_READ_GSM_MEASUREMENT_RESULTS_OSUP_AVERAGE                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 3485
#define RSSPECAN_ATTR_READ_GSM_MEASUREMENT_RESULTS_OSUP_CURRENT                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 3486
#define RSSPECAN_ATTR_READ_GSM_MEASUREMENT_RESULTS_OSUP_MAXIMUM                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 3487
#define RSSPECAN_ATTR_READ_GSM_MEASUREMENT_RESULTS_OSUP_SDEV                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 3488
#define RSSPECAN_ATTR_READ_GSM_MEASUREMENT_RESULTS_IQ_OFFSET_AVERAGE                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 3489
#define RSSPECAN_ATTR_READ_GSM_MEASUREMENT_RESULTS_IQ_OFFSET_CURRENT                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 3490
#define RSSPECAN_ATTR_READ_GSM_MEASUREMENT_RESULTS_IQ_OFFSET_MAXIMUM                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 3491
#define RSSPECAN_ATTR_READ_GSM_MEASUREMENT_RESULTS_IQ_OFFSET_SDEV                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 3492
#define RSSPECAN_ATTR_READ_GSM_MEASUREMENT_RESULTS_IQ_IMBALANCE_AVERAGE                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 3493
#define RSSPECAN_ATTR_READ_GSM_MEASUREMENT_RESULTS_IQ_IMBALANCE_CURRENT                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 3494
#define RSSPECAN_ATTR_READ_GSM_MEASUREMENT_RESULTS_IQ_IMBALANCE_MAXIMUM                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 3495
#define RSSPECAN_ATTR_READ_GSM_MEASUREMENT_RESULTS_IQ_IMBALANCE_SDEV                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 3496
#define RSSPECAN_ATTR_READ_GSM_MEASUREMENT_RESULTS_FREQUENCY_AVERAGE                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 3497
#define RSSPECAN_ATTR_READ_GSM_MEASUREMENT_RESULTS_FREQUENCY_CURRENT                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 3498
#define RSSPECAN_ATTR_READ_GSM_MEASUREMENT_RESULTS_FREQUENCY_MAXIMUM                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 3499
#define RSSPECAN_ATTR_READ_GSM_MEASUREMENT_RESULTS_FREQUENCY_SDEV                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 3500
#define RSSPECAN_ATTR_READ_GSM_MEASUREMENT_RESULTS_BURST_POWER_AVERAGE                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 3501
#define RSSPECAN_ATTR_READ_GSM_MEASUREMENT_RESULTS_BURST_POWER_CURRENT                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 3502
#define RSSPECAN_ATTR_READ_GSM_MEASUREMENT_RESULTS_BURST_POWER_MAXIMUM                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 3503
#define RSSPECAN_ATTR_READ_GSM_MEASUREMENT_RESULTS_BURST_POWER_SDEV                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 3504
#define RSSPECAN_ATTR_READ_GSM_MEASUREMENT_RESULTS_AMPLITUDE_DROOP_AVERAGE                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 3505
#define RSSPECAN_ATTR_READ_GSM_MEASUREMENT_RESULTS_AMPLITUDE_DROOP_CURRENT                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 3506
#define RSSPECAN_ATTR_READ_GSM_MEASUREMENT_RESULTS_AMPLITUDE_DROOP_MAXIMUM                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 3507
#define RSSPECAN_ATTR_READ_GSM_MEASUREMENT_RESULTS_AMPLITUDE_DROOP_SDEV                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 3508
#define RSSPECAN_ATTR_READ_GSM_MEASUREMENT_RESULTS_95_PERCENTILE_EVM                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 3509
#define RSSPECAN_ATTR_READ_GSM_MEASUREMENT_RESULTS_95_PERCENTILE_MERROR                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 3510
#define RSSPECAN_ATTR_READ_GSM_MEASUREMENT_RESULTS_95_PERCENTILE_PERROR                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 3511
#define RSSPECAN_ATTR_FETCH_GSM_MEASUREMENT_RESULTS_RMS_AVERAGE                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 3512
#define RSSPECAN_ATTR_FETCH_GSM_MEASUREMENT_RESULTS_RMS_CURRENT                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 3513
#define RSSPECAN_ATTR_FETCH_GSM_MEASUREMENT_RESULTS_RMS_MAXIMUM                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 3514
#define RSSPECAN_ATTR_FETCH_GSM_MEASUREMENT_RESULTS_RMS_SDEV                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 3515
#define RSSPECAN_ATTR_FETCH_GSM_MEASUREMENT_RESULTS_MERROR_RMS_AVERAGE                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 3516
#define RSSPECAN_ATTR_FETCH_GSM_MEASUREMENT_RESULTS_MERROR_RMS_CURRENT                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 3517
#define RSSPECAN_ATTR_FETCH_GSM_MEASUREMENT_RESULTS_MERROR_RMS_MAXIMUM                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 3518
#define RSSPECAN_ATTR_FETCH_GSM_MEASUREMENT_RESULTS_MERROR_RMS_SDEV                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 3519
#define RSSPECAN_ATTR_FETCH_GSM_MEASUREMENT_RESULTS_PERROR_RMS_AVERAGE                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 3520
#define RSSPECAN_ATTR_FETCH_GSM_MEASUREMENT_RESULTS_PERROR_RMS_CURRENT                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 3521
#define RSSPECAN_ATTR_FETCH_GSM_MEASUREMENT_RESULTS_PERROR_RMS_MAXIMUM                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 3522
#define RSSPECAN_ATTR_FETCH_GSM_MEASUREMENT_RESULTS_PERROR_RMS_SDEV                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 3523
#define RSSPECAN_ATTR_FETCH_GSM_MEASUREMENT_RESULTS_PEAK_AVERAGE                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 3524
#define RSSPECAN_ATTR_FETCH_GSM_MEASUREMENT_RESULTS_PEAK_CURRENT                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 3525
#define RSSPECAN_ATTR_FETCH_GSM_MEASUREMENT_RESULTS_PEAK_MAXIMUM                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 3526
#define RSSPECAN_ATTR_FETCH_GSM_MEASUREMENT_RESULTS_PEAK_SDEV                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 3527
#define RSSPECAN_ATTR_FETCH_GSM_MEASUREMENT_RESULTS_MERROR_PEAK_AVERAGE                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 3528
#define RSSPECAN_ATTR_FETCH_GSM_MEASUREMENT_RESULTS_MERROR_PEAK_CURRENT                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 3529
#define RSSPECAN_ATTR_FETCH_GSM_MEASUREMENT_RESULTS_MERROR_PEAK_MAXIMUM                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 3530
#define RSSPECAN_ATTR_FETCH_GSM_MEASUREMENT_RESULTS_MERROR_PEAK_SDEV                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 3531
#define RSSPECAN_ATTR_FETCH_GSM_MEASUREMENT_RESULTS_PERROR_PEAK_AVERAGE                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 3532
#define RSSPECAN_ATTR_FETCH_GSM_MEASUREMENT_RESULTS_PERROR_PEAK_CURRENT                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 3533
#define RSSPECAN_ATTR_FETCH_GSM_MEASUREMENT_RESULTS_PERROR_PEAK_MAXIMUM                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 3534
#define RSSPECAN_ATTR_FETCH_GSM_MEASUREMENT_RESULTS_PERROR_PEAK_SDEV                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 3535
#define RSSPECAN_ATTR_FETCH_GSM_MEASUREMENT_RESULTS_OSUP_AVERAGE                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 3536
#define RSSPECAN_ATTR_FETCH_GSM_MEASUREMENT_RESULTS_OSUP_CURRENT                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 3537
#define RSSPECAN_ATTR_FETCH_GSM_MEASUREMENT_RESULTS_OSUP_MAXIMUM                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 3538
#define RSSPECAN_ATTR_FETCH_GSM_MEASUREMENT_RESULTS_OSUP_SDEV                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 3539
#define RSSPECAN_ATTR_FETCH_GSM_MEASUREMENT_RESULTS_IQ_OFFSET_AVERAGE                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 3540
#define RSSPECAN_ATTR_FETCH_GSM_MEASUREMENT_RESULTS_IQ_OFFSET_CURRENT                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 3541
#define RSSPECAN_ATTR_FETCH_GSM_MEASUREMENT_RESULTS_IQ_OFFSET_MAXIMUM                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 3542
#define RSSPECAN_ATTR_FETCH_GSM_MEASUREMENT_RESULTS_IQ_OFFSET_SDEV                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 3543
#define RSSPECAN_ATTR_FETCH_GSM_MEASUREMENT_RESULTS_IQ_IMBALANCE_AVERAGE                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 3544
#define RSSPECAN_ATTR_FETCH_GSM_MEASUREMENT_RESULTS_IQ_IMBALANCE_CURRENT                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 3545
#define RSSPECAN_ATTR_FETCH_GSM_MEASUREMENT_RESULTS_IQ_IMBALANCE_MAXIMUM                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 3546
#define RSSPECAN_ATTR_FETCH_GSM_MEASUREMENT_RESULTS_IQ_IMBALANCE_SDEV                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 3547
#define RSSPECAN_ATTR_FETCH_GSM_MEASUREMENT_RESULTS_FREQUENCY_AVERAGE                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 3548
#define RSSPECAN_ATTR_FETCH_GSM_MEASUREMENT_RESULTS_FREQUENCY_CURRENT                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 3549
#define RSSPECAN_ATTR_FETCH_GSM_MEASUREMENT_RESULTS_FREQUENCY_MAXIMUM                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 3550
#define RSSPECAN_ATTR_FETCH_GSM_MEASUREMENT_RESULTS_FREQUENCY_SDEV                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 3551
#define RSSPECAN_ATTR_FETCH_GSM_MEASUREMENT_RESULTS_BURST_POWER_AVERAGE                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 3552
#define RSSPECAN_ATTR_FETCH_GSM_MEASUREMENT_RESULTS_BURST_POWER_CURRENT                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 3553
#define RSSPECAN_ATTR_FETCH_GSM_MEASUREMENT_RESULTS_BURST_POWER_MAXIMUM                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 3554
#define RSSPECAN_ATTR_FETCH_GSM_MEASUREMENT_RESULTS_BURST_POWER_SDEV                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 3555
#define RSSPECAN_ATTR_FETCH_GSM_MEASUREMENT_RESULTS_AMPLITUDE_DROOP_AVERAGE                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 3556
#define RSSPECAN_ATTR_FETCH_GSM_MEASUREMENT_RESULTS_AMPLITUDE_DROOP_CURRENT                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 3557
#define RSSPECAN_ATTR_FETCH_GSM_MEASUREMENT_RESULTS_AMPLITUDE_DROOP_MAXIMUM                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 3558
#define RSSPECAN_ATTR_FETCH_GSM_MEASUREMENT_RESULTS_AMPLITUDE_DROOP_SDEV                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 3559
#define RSSPECAN_ATTR_FETCH_GSM_MEASUREMENT_RESULTS_95_PERCENTILE_EVM                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 3560
#define RSSPECAN_ATTR_FETCH_GSM_MEASUREMENT_RESULTS_95_PERCENTILE_MERROR                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 3561
#define RSSPECAN_ATTR_FETCH_GSM_MEASUREMENT_RESULTS_95_PERCENTILE_PERROR                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 3562
#define RSSPECAN_ATTR_READ_GSM_MEASUREMENT_RESULTS_TRIGGER_TO_SYNC_AVERAGE                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 4768
#define RSSPECAN_ATTR_READ_GSM_MEASUREMENT_RESULTS_TRIGGER_TO_SYNC_CURRENT                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 4769
#define RSSPECAN_ATTR_READ_GSM_MEASUREMENT_RESULTS_TRIGGER_TO_SYNC_MAXIMUM                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 4770
#define RSSPECAN_ATTR_READ_GSM_MEASUREMENT_RESULTS_TRIGGER_TO_SYNC_SDEV                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 4771
#define RSSPECAN_ATTR_READ_GSM_MEASUREMENT_RESULTS_TRIGGER_TO_SYNC_MINIMUM                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 4772
#define RSSPECAN_ATTR_FETCH_GSM_MEASUREMENT_RESULTS_TRIGGER_TO_SYNC_AVERAGE                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 4773
#define RSSPECAN_ATTR_FETCH_GSM_MEASUREMENT_RESULTS_TRIGGER_TO_SYNC_CURRENT                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 4774
#define RSSPECAN_ATTR_FETCH_GSM_MEASUREMENT_RESULTS_TRIGGER_TO_SYNC_MAXIMUM                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 4775
#define RSSPECAN_ATTR_FETCH_GSM_MEASUREMENT_RESULTS_TRIGGER_TO_SYNC_SDEV                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 4776
#define RSSPECAN_ATTR_FETCH_GSM_MEASUREMENT_RESULTS_TRIGGER_TO_SYNC_MINIMUM                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 4777
#define RSSPECAN_ATTR_C2K_MODE                                                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 1844
#define RSSPECAN_ATTR_MC2K_MODE                                                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 1845
#define RSSPECAN_ATTR_BDO_MODE                                                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 1846
#define RSSPECAN_ATTR_MDO_MODE                                                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 1847
#define RSSPECAN_ATTR_BDO_CDP_AVERAGE                                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 1905
#define RSSPECAN_ATTR_C2K_CDP_PNOFFSET                                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 1877
#define RSSPECAN_ATTR_BDO_REVISION                                                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 1908
#define RSSPECAN_ATTR_BDO_SUBTYPE                                                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 3563
#define RSSPECAN_ATTR_C2K_CDP_ANTENNA                                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 1892
#define RSSPECAN_ATTR_MC2K_CDP_LCODE_MASK                                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 1915
#define RSSPECAN_ATTR_MDO_CDP_LCODE_I                                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 1922
#define RSSPECAN_ATTR_MDO_CDP_LCODE_Q                                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 1923
#define RSSPECAN_ATTR_MC2K_CDP_LCODE_OFFSET                                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 1916
#define RSSPECAN_ATTR_MC2K_LCODE_MODE                                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 1921
#define RSSPECAN_ATTR_C2K_MCARRIER_STATE                                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 1879
#define RSSPECAN_ATTR_C2K_MCARRIER_ENH_ALG                                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 1880
#define RSSPECAN_ATTR_C2K_MCARRIER_FILTER_STATE                                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 1881
#define RSSPECAN_ATTR_C2K_MCARRIER_FILTER_TYPE                                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 1882
#define RSSPECAN_ATTR_C2K_MCARRIER_ROLL_OFF                                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 1883
#define RSSPECAN_ATTR_C2K_MCARRIER_CUT_OFF_FREQ                                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 1884
#define RSSPECAN_ATTR_C2K_CDP_QINVERT                                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 1856
#define RSSPECAN_ATTR_C2K_CDP_IQLENGTH                                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 1890
#define RSSPECAN_ATTR_C2K_CDP_SLOT                                                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 1857
#define RSSPECAN_ATTR_C2K_SET_COUNT                                                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 1872
#define RSSPECAN_ATTR_C2K_SET_TO_ANALYZE                                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 1867
#define RSSPECAN_ATTR_C2K_CDP_ICTHRES                                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 1859
#define RSSPECAN_ATTR_BDO_CDP_MAPPING                                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 1902
#define RSSPECAN_ATTR_C2K_CTABLE_STATE                                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 3564
#define RSSPECAN_ATTR_C2K_CTABLE_SELECT                                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 1849
#define RSSPECAN_ATTR_C2K_CTABLE_NAME                                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 1850
#define RSSPECAN_ATTR_C2K_CTABLE_COMMENT                                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 1851
#define RSSPECAN_ATTR_C2K_CTABLE_COPY                                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 1852
#define RSSPECAN_ATTR_C2K_CTABLE_DELETE                                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 1853
#define RSSPECAN_ATTR_C2K_CTABLE_RESTORE                                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 1854
#define RSSPECAN_ATTR_C2K_CDP_LEVEL_ADJUST                                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 1860
#define RSSPECAN_ATTR_C2K_ACH_PRES                                                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 1862
#define RSSPECAN_ATTR_C2K_ADJUST_REF_LEVEL                                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 1863
#define RSSPECAN_ATTR_C2K_ACH_REF_AUTO                                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 1865
#define RSSPECAN_ATTR_C2K_BCLASS                                                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 1871
#define RSSPECAN_ATTR_C2K_LIM_RESTORE                                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 1869
#define RSSPECAN_ATTR_C2K_LIM_VAL                                                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 1874
#define RSSPECAN_ATTR_C2K_LIM_MODE                                                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 1875
#define RSSPECAN_ATTR_BDO_RFSLOT                                                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 1900
#define RSSPECAN_ATTR_BDO_PVT_BURST_FIT                                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 1894
#define RSSPECAN_ATTR_BDO_LIM_PVT_REFERENCE                                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 1896
#define RSSPECAN_ATTR_BDO_LIM_PVT_VALUE                                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 1897
#define RSSPECAN_ATTR_BDO_PVT_FAIL_RESTART                                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 1895
#define RSSPECAN_ATTR_BDO_LIM_PVT_LIST                                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 1898
#define RSSPECAN_ATTR_BDO_LIM_PVT_RESTORE                                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 1899
#define RSSPECAN_ATTR_C2K_MEAS_SELECT                                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 1870
#define RSSPECAN_ATTR_MC2K_CDP_OVERVIEW                                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 1914
#define RSSPECAN_ATTR_C2K_RESULT_TYPE                                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 1873
#define RSSPECAN_ATTR_C2K_CDP_SFACTOR                                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 1887
#define RSSPECAN_ATTR_C2K_CDP_NORMALIZE                                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 1855
#define RSSPECAN_ATTR_C2K_CDP_TPMEAS                                                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 1861
#define RSSPECAN_ATTR_MDO_CDP_OPERATION                                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 1927
#define RSSPECAN_ATTR_C2K_CDP_DISPLAY                                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 2887
#define RSSPECAN_ATTR_C2K_PILOT_POWER_DISPLAY                                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 3567
#define RSSPECAN_ATTR_C2K_CDP_PREFERENCE                                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 1888
#define RSSPECAN_ATTR_C2K_CDP_ORDER                                                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 1889
#define RSSPECAN_ATTR_C2K_CDP_SBAND                                                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 1858
#define RSSPECAN_ATTR_C2K_CDP_CODE                                                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 1891
#define RSSPECAN_ATTR_MC2K_CDP_MAPPING                                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 1911
#define RSSPECAN_ATTR_BDO_CDP_MMODE                                                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 1901
#define RSSPECAN_ATTR_BDO_CDP_CTYPE                                                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 1903
#define RSSPECAN_ATTR_C2K_PDIV                                                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 1864
#define RSSPECAN_ATTR_C2K_MARKER_TDPICH                                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 1885
#define RSSPECAN_ATTR_C2K_MARKER_PICH                                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 1886
#define RSSPECAN_ATTR_BWCD_MODE                                                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 1703
#define RSSPECAN_ATTR_3GBS_MEAS                                                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 1704
#define RSSPECAN_ATTR_3GPP_BS_STANDARD                                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 2645
#define RSSPECAN_ATTR_3GUE_MODE                                                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 1749
#define RSSPECAN_ATTR_3GUE_MEAS                                                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 1752
#define RSSPECAN_ATTR_3GBS_WCDP_AUTO_SCALE                                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 1707
#define RSSPECAN_ATTR_3GBS_WCDP_MCACLR                                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 1708
#define RSSPECAN_ATTR_3GPP_ASEQ                                                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 1736
#define RSSPECAN_ATTR_3GBS_WCDP_HSDPAUPA_STATE                                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 1705
#define RSSPECAN_ATTR_3GBS_SCODE                                                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 1733
#define RSSPECAN_ATTR_3GPP_SCODE_SEARCH                                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 1737
#define RSSPECAN_ATTR_3GBS_RESULTS_SFAC                                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 1744
#define RSSPECAN_ATTR_3GPP_QPSK_MODULATION_ONLY_ENABLED                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 5160
#define RSSPECAN_ATTR_3GBS_WCDP_COMPARE_MODE                                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 1710
#define RSSPECAN_ATTR_3GBS_MIMO_MODE                                                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 2428
#define RSSPECAN_ATTR_3GBS_ANTENNA                                                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 1730
#define RSSPECAN_ATTR_3GBS_WCDP_SYNC_TYPE                                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 1706
#define RSSPECAN_ATTR_3GPP_CPICH_MODE                                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 5040
#define RSSPECAN_ATTR_3GPP_CPICH_CODE                                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 5041
#define RSSPECAN_ATTR_3GPP_CPICH_PATTERN                                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 5042
#define RSSPECAN_ATTR_3GUE_CDP_SCR_TYPE                                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 1759
#define RSSPECAN_ATTR_3GBS_WCDP_INVERT_Q                                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 1711
#define RSSPECAN_ATTR_3GUE_RRC_FILTER                                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 1751
#define RSSPECAN_ATTR_3GBS_WCDP_MULTIFRAME_CAPTURE_LENGTH                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 1717
#define RSSPECAN_ATTR_3GBS_WCDP_MULTIFRAME_FRAME_ANALYZE                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 1718
#define RSSPECAN_ATTR_3GBS_WCDP_SIDE_BAND                                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 1713
#define RSSPECAN_ATTR_3GUE_CDP_BASE                                                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 1753
#define RSSPECAN_ATTR_3GBS_WCDP_INACT_CHAN_THR                                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 1714
#define RSSPECAN_ATTR_3GBS_TIMING_OFFSET                                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 1734
#define RSSPECAN_ATTR_3GBS_MEASUREMENT_SIGNAL_COMPARING                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 4685
#define RSSPECAN_ATTR_3GBS_WCDP_CTAB                                                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 1742
#define RSSPECAN_ATTR_BS_CHANNEL_TABLE_NEW_ACCORDING_TO_MEASUREMENT                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 7114
#define RSSPECAN_ATTR_3GBS_WCDP_CTAB_SEL                                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 1743
#define RSSPECAN_ATTR_3GBS_WCDP_CTAB_HEAD_COMM                                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 1738
#define RSSPECAN_ATTR_3GBS_WCDP_CTAB_DEL                                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 1739
#define RSSPECAN_ATTR_3GBS_WCDP_CTAB_COPY                                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 1740
#define RSSPECAN_ATTR_3GBS_WCDP_CTAB_NAME                                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 1741
#define RSSPECAN_ATTR_3GUE_WCDP_CTAB                                                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 1775
#define RSSPECAN_ATTR_UE_CHANNEL_TABLE_NEW_ACCORDING_TO_MEASUREMENT                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 7115
#define RSSPECAN_ATTR_3GUE_WCDP_CTAB_SEL                                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 1774
#define RSSPECAN_ATTR_3GUE_WCDP_CTAB_COMMENT                                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 1770
#define RSSPECAN_ATTR_3GUE_WCDP_CTAB_DEL                                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 1772
#define RSSPECAN_ATTR_3GUE_WCDP_CTAB_COPY                                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 1771
#define RSSPECAN_ATTR_3GUE_WCDP_CTAB_NAME                                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 1773
#define RSSPECAN_ATTR_3GUE_WCDP_CTAB_HSDP                                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 1769
#define RSSPECAN_ATTR_3GUE_WCDP_CTAB_EDPC                                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 4686
#define RSSPECAN_ATTR_3GBS_LOAD_DEFAULT_CARRIER_TABLE                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 5790
#define RSSPECAN_ATTR_3GBS_NEW_CARRIER_TABLE                                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 5791
#define RSSPECAN_ATTR_3GBS_SAVE_CARRIER_TABLE                                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 5792
#define RSSPECAN_ATTR_3GBS_DELETE_CARRIER_TABLE                                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 5793
#define RSSPECAN_ATTR_3GBS_CARRIER_TABLE_CATALOG                                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 5794
#define RSSPECAN_ATTR_3GBS_NUMBER_OF_CARRIERS                                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 5795
#define RSSPECAN_ATTR_3GBS_INSERT_NEW_CARRIER                                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 5796
#define RSSPECAN_ATTR_3GBS_DELETE_CARRIER                                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 5798
#define RSSPECAN_ATTR_3GBS_DELETE_ALL_CARRIERS                                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 5800
#define RSSPECAN_ATTR_3GBS_CARRIER_FREQUENCY_OFFSET                                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 5802
#define RSSPECAN_ATTR_3GBS_CARRIER_SCRAMBLING_CODE                                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 5803
#define RSSPECAN_ATTR_3GBS_CARRIER_CPICH                                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 5804
#define RSSPECAN_ATTR_3GBS_CARRIER_PATTERN                                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 5805
#define RSSPECAN_ATTR_3GBS_WCDP_NORMALIZE                                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 1709
#define RSSPECAN_ATTR_3GBS_RESULT_SUM_VIEW                                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 1748
#define RSSPECAN_ATTR_3GBS_POWER_DISPLAY                                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 4688
#define RSSPECAN_ATTR_3GBS_WCDP_POW_REF                                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 1712
#define RSSPECAN_ATTR_3GBS_WCDP_POW_DIFFERENCE                                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 1715
#define RSSPECAN_ATTR_3GBS_WCDP_CONST_B                                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 1716
#define RSSPECAN_ATTR_3GUE_CDP_ELIMINATE_TAIL_CHIPS                                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 1764
#define RSSPECAN_ATTR_3GUE_HSLOT                                                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 4689
#define RSSPECAN_ATTR_3GPP_PDIV                                                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 1735
#define RSSPECAN_ATTR_3GBS_RESULTS_SELECT_CHAN                                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 1745
#define RSSPECAN_ATTR_3GBS_RESULTS_SELECT_CPICH_SLOT                                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 1746
#define RSSPECAN_ATTR_3GUE_CDP_SEL_COMP                                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 1755
#define RSSPECAN_ATTR_3GBS_MCACLR_ACLR_ACP_CHAN_POW                                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 1724
#define RSSPECAN_ATTR_3GPP_MARKER_TO_CPICH                                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 3575
#define RSSPECAN_ATTR_3GPP_MARKER_TO_PCCP                                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 1731
#define RSSPECAN_ATTR_3GPP_REFERENCE_MARKER_TO_CPIC                                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 3576
#define RSSPECAN_ATTR_3GPP_REFERENCE_MARKER_TO_PCCP                                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 1732
#define RSSPECAN_ATTR_3GBS_RESULT                                                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 1747
#define RSSPECAN_ATTR_3GPP_EXPORT_MARKER_PEAK_LIST_TO_FILE                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 7116
#define RSSPECAN_ATTR_3GUE_RESULT                                                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 1750
#define RSSPECAN_ATTR_TDBS_MODE                                                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 1777
#define RSSPECAN_ATTR_TDUE_MODE                                                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 1812
#define RSSPECAN_ATTR_TDUE_MEAS                                                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 1811
#define RSSPECAN_ATTR_TDBS_SIDE_BAND                                                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 1788
#define RSSPECAN_ATTR_TDBS_POWER_DISPLAY                                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 2998
#define RSSPECAN_ATTR_TDBS_NORMALIZE                                                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 1790
#define RSSPECAN_ATTR_TDBS_CDP_LEV_ADJUST                                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 1782
#define RSSPECAN_ATTR_TDS_PTS_ENABLED                                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 5043
#define RSSPECAN_ATTR_TDBS_CAPTURE_LENGTH                                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 1784
#define RSSPECAN_ATTR_TDBS_INVERT_Q                                                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 1789
#define RSSPECAN_ATTR_TDBS_RESULT_SET_COUNT                                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 1798
#define RSSPECAN_ATTR_TDBS_CELL_SHIFTS                                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 1786
#define RSSPECAN_ATTR_TDBS_SCODE                                                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 1785
#define RSSPECAN_ATTR_TDS_PTS_SYNC_CODE                                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 5044
#define RSSPECAN_ATTR_TDS_TIME_REFERENCE                                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 5045
#define RSSPECAN_ATTR_TDBS_SYNC_TO_SLOT                                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 1791
#define RSSPECAN_ATTR_TDBS_SYNC_TO_SLOT_ROTATE                                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 2602
#define RSSPECAN_ATTR_TDBS_STS_MODE                                                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 5594
#define RSSPECAN_ATTR_TDBS_CTAB_MMAX                                                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 3052
#define RSSPECAN_ATTR_TDBS_INACT_CHAN_THR                                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 1787
#define RSSPECAN_ATTR_TDBS_CTAB_COMM                                                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 3048
#define RSSPECAN_ATTR_TDBS_CTAB_NAME                                                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 3047
#define RSSPECAN_ATTR_TDBS_CTAB_ORDER                                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 3044
#define RSSPECAN_ATTR_TDBS_CTAB_MSHIFT                                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 3049
#define RSSPECAN_ATTR_TDBS_CTAB_COPY                                                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 3050
#define RSSPECAN_ATTR_TDBS_CTAB_DEL                                                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 3051
#define RSSPECAN_ATTR_TDBS_CTAB_SEL                                                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 3046
#define RSSPECAN_ATTR_TDBS_CTAB_STATE                                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 3045
#define RSSPECAN_ATTR_TDBS_RESULT_SEL_CHAN                                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 1796
#define RSSPECAN_ATTR_TDBS_RESULT_SET_VALUE                                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 1799
#define RSSPECAN_ATTR_TDBS_RESULT_SEL_SLOT                                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 1797
#define RSSPECAN_ATTR_TDBS_MEAS_PVT_SWITCH_POINT                                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 1808
#define RSSPECAN_ATTR_TDBS_MEAS_PVT_SFRAMES                                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 1809
#define RSSPECAN_ATTR_TDBS_ADAPT_SIGN_AUT_LEVEL_TIME                                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 1778
#define RSSPECAN_ATTR_TDBS_ADAPT_SIGN_SLOT_START                                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 1779
#define RSSPECAN_ATTR_TDBS_ADAPT_SIGN_SLOT_STOP                                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 1780
#define RSSPECAN_ATTR_TDBS_VERTICAL_SCALE_MIN                                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 3590
#define RSSPECAN_ATTR_TDBS_VERTICAL_SCALE_MAX                                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 3591
#define RSSPECAN_ATTR_TDBS_RESULT_TYPE                                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 1795
#define RSSPECAN_ATTR_LTE_DOWNLINK_MODE                                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 2240
#define RSSPECAN_ATTR_LTE_DOWNLINK_DUPLEXING                                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 2243
#define RSSPECAN_ATTR_LTE_DOWNLINK_TEST_MODEL_FSV                                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 3595
#define RSSPECAN_ATTR_LTE_DOWNLINK_CHANNEL_BANDWIDTH_FSV                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 2244
#define RSSPECAN_ATTR_LTE_DOWNLINK_NUMBER_OF_RESOURCE_BLOCKS_FSV                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 2245
#define RSSPECAN_ATTR_LTE_DOWNLINK_CYCLIC_PREFIX_FSV                                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 2246
#define RSSPECAN_ATTR_LTE_DOWNLINK_TDD_UL_DL_ALLOCATIONS_FSV                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 2260
#define RSSPECAN_ATTR_LTE_DOWNLINK_TDD_SPECIAL_SUBFRAME_FSV                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 2937
#define RSSPECAN_ATTR_LTE_DOWNLINK_CELL_IDENTITY_GROUP_FSV                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 2262
#define RSSPECAN_ATTR_LTE_DOWNLINK_CID_GROUP_AUTO_FSV                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 2261
#define RSSPECAN_ATTR_LTE_DOWNLINK_DETECTED_CELL_IDENTITY_GROUP_FSV                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 2308
#define RSSPECAN_ATTR_LTE_DOWNLINK_CELL_IDENTITY_FSV                                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 2264
#define RSSPECAN_ATTR_LTE_DOWNLINK_CELL_IDENTITY_AUTO_FSV                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 2263
#define RSSPECAN_ATTR_LTE_DOWNLINK_DETECTED_CELL_IDENTITY_FSV                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 2309
#define RSSPECAN_ATTR_LTE_DOWNLINK_STORE_DEMODULATION_SETTINGS_FSV                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 2241
#define RSSPECAN_ATTR_LTE_DOWNLINK_LOAD_DEMODULATION_SETTINGS_FSV                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 2242
#define RSSPECAN_ATTR_LTE_DOWNLINK_TEST_MODEL                                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 7360
#define RSSPECAN_ATTR_LTE_DOWNLINK_CHANNEL_BANDWIDTH                                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 4856
#define RSSPECAN_ATTR_LTE_DOWNLINK_NUMBER_OF_RESOURCE_BLOCKS                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 4857
#define RSSPECAN_ATTR_LTE_DOWNLINK_CYCLIC_PREFIX                                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 4943
#define RSSPECAN_ATTR_LTE_DOWNLINK_TDD_UL_DL_ALLOCATIONS                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 4859
#define RSSPECAN_ATTR_LTE_DOWNLINK_TDD_SPECIAL_SUBFRAME                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 4860
#define RSSPECAN_ATTR_LTE_DOWNLINK_CELL_IDENTITY_GROUP                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 4861
#define RSSPECAN_ATTR_LTE_DOWNLINK_CID_GROUP_AUTO                                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 4862
#define RSSPECAN_ATTR_LTE_DOWNLINK_DETECTED_CELL_IDENTITY_GROUP                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 4863
#define RSSPECAN_ATTR_LTE_DOWNLINK_CELL_IDENTITY                                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 4864
#define RSSPECAN_ATTR_LTE_DOWNLINK_CELL_IDENTITY_AUTO                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 4865
#define RSSPECAN_ATTR_LTE_DOWNLINK_DETECTED_CELL_IDENTITY                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 4944
#define RSSPECAN_ATTR_LTE_DOWNLINK_REFERENCE_POWER_CARR                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 6602
#define RSSPECAN_ATTR_LTE_DOWNLINK_EXCLUDE_INBAND_NB_IOT_ENABLED                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 7361
#define RSSPECAN_ATTR_LTE_DOWNLINK_EXCLUDED_INBAND_NB_IOT_RB_OFFSET                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 7362
#define RSSPECAN_ATTR_LTE_DOWNLINK_LOAD_DEMODULATION_SETTINGS                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 7363
#define RSSPECAN_ATTR_LTE_DOWNLINK_STORE_DEMODULATION_SETTINGS                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 8645
#define RSSPECAN_ATTR_LTE_DOWNLINK_MIMO_CONFIGURATION_FSV                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 2250
#define RSSPECAN_ATTR_LTE_DOWNLINK_MIMO_ANTENNA_FSV                                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 2251
#define RSSPECAN_ATTR_LTE_DOWNLINK_MIMO_CONFIGURATION                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 4945
#define RSSPECAN_ATTR_LTE_DOWNLINK_MIMO_ANTENNA                                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 4868
#define RSSPECAN_ATTR_LTE_DOWNLINK_MIMO_ANALYZER_STATE                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 4869
#define RSSPECAN_ATTR_LTE_DOWNLINK_MIMO_ANALYZER_ADDRESS                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 4870
#define RSSPECAN_ATTR_LTE_DOWNLINK_MIMO_INSTRUMENT_STATE                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 4858
#define RSSPECAN_ATTR_LTE_DOWNLINK_PDSCH_SUBFRAME_DETECT                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 2731
#define RSSPECAN_ATTR_LTE_DOWNLINK_AUTO_DEMODULATION                                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 2257
#define RSSPECAN_ATTR_LTE_DOWNLINK_PDSCH_CONFIGURABLE_SUBFRAMES_FSV                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 2265
#define RSSPECAN_ATTR_LTE_DOWNLINK_PDSCH_CONFIGURABLE_SUBFRAMES                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 6265
#define RSSPECAN_ATTR_LTE_DOWNLINK_PDSCH_USED_ALLOCATIONS_FSV                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 2266
#define RSSPECAN_ATTR_LTE_DOWNLINK_PDSCH_USED_ALLOCATIONS                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 6266
#define RSSPECAN_ATTR_LTE_DOWNLINK_PDSCH_ALLOCATION_ID_FSV                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 4872
#define RSSPECAN_ATTR_LTE_DOWNLINK_PDSCH_ALLOCATION_VRB_GAP_FSV                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 4873
#define RSSPECAN_ATTR_LTE_DOWNLINK_PDSCH_ALLOCATION_POWER_FSV                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 2270
#define RSSPECAN_ATTR_LTE_DOWNLINK_PDSCH_ALLOCATION_START_OFFSET_FSV                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 3134
#define RSSPECAN_ATTR_LTE_DOWNLINK_PDSCH_ALLOCATION_RESOURCE_BLOCKS_FSV                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 2268
#define RSSPECAN_ATTR_LTE_DOWNLINK_PDSCH_ALLOCATION_RESOURCE_BLOCKS_OFFSET_FSV                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 2269
#define RSSPECAN_ATTR_LTE_DOWNLINK_PDSCH_ALLOCATION_MODULATION_FSV                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 2267
#define RSSPECAN_ATTR_LTE_DOWNLINK_PDSCH_ALLOCATION_PRECODING_FSV                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 4874
#define RSSPECAN_ATTR_LTE_DOWNLINK_PDSCH_ALLOCATION_CODEWORD_TO_LAYER_MAPPING_FSV                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 4710
#define RSSPECAN_ATTR_LTE_DOWNLINK_PDSCH_ALLOCATION_SCRAMBLING_IDENTITY_FSV                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 4875
#define RSSPECAN_ATTR_LTE_DOWNLINK_PDSCH_ALLOCATION_SINGLE_LAYER_ANTENNA_PORT_FSV                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 4876
#define RSSPECAN_ATTR_LTE_DOWNLINK_PDSCH_ALLOCATION_CODEBOOK_INDEX_FSV                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 4877
#define RSSPECAN_ATTR_LTE_DOWNLINK_PDSCH_ALLOCATION_CYCLIC_DELAY_DIVERSITY_FSV                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 4878
#define RSSPECAN_ATTR_LTE_DOWNLINK_PDSCH_ALLOCATION_ID                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 6267
#define RSSPECAN_ATTR_LTE_DOWNLINK_PDSCH_ALLOCATION_VRB_GAP                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 6268
#define RSSPECAN_ATTR_LTE_DOWNLINK_PDSCH_ALLOCATION_POWER                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 6269
#define RSSPECAN_ATTR_LTE_DOWNLINK_PDSCH_ALLOCATION_START_OFFSET                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 6270
#define RSSPECAN_ATTR_LTE_DOWNLINK_PDSCH_ALLOCATION_RESOURCE_BLOCKS                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 6271
#define RSSPECAN_ATTR_LTE_DOWNLINK_PDSCH_ALLOCATION_RESOURCE_BLOCKS_OFFSET                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 6272
#define RSSPECAN_ATTR_LTE_DOWNLINK_PDSCH_ALLOCATION_MODULATION                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 6273
#define RSSPECAN_ATTR_LTE_DOWNLINK_PDSCH_ALLOCATION_PRECODING                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 6274
#define RSSPECAN_ATTR_LTE_DOWNLINK_PDSCH_ALLOCATION_CODEWORD_TO_LAYER_MAPPING                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 6275
#define RSSPECAN_ATTR_LTE_DOWNLINK_PDSCH_ALLOCATION_SCRAMBLING_IDENTITY                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 6276
#define RSSPECAN_ATTR_LTE_DOWNLINK_PDSCH_ALLOCATION_SINGLE_LAYER_ANTENNA_PORT                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 6277
#define RSSPECAN_ATTR_LTE_DOWNLINK_PDSCH_ALLOCATION_CODEBOOK_INDEX                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 6278
#define RSSPECAN_ATTR_LTE_DOWNLINK_PDSCH_ALLOCATION_CYCLIC_DELAY_DIVERSITY                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 6279
#define RSSPECAN_ATTR_LTE_DOWNLINK_SYNC_ANTENNA_FSV                                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 3599
#define RSSPECAN_ATTR_LTE_DOWNLINK_PSYNC_RELATIVE_POWER_FSV                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 2272
#define RSSPECAN_ATTR_LTE_DOWNLINK_SSYNC_RELATIVE_POWER_FSV                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 2273
#define RSSPECAN_ATTR_LTE_DOWNLINK_SYNC_ANTENNA                                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 4880
#define RSSPECAN_ATTR_LTE_DOWNLINK_CUSTOM_SYNC_WEIGHT_ENABLED                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 8647
#define RSSPECAN_ATTR_LTE_DOWNLINK_CUSTOM_SYNC_WEIGHT_FIRST_HALF_FRAME_IMAGINARY                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 8648
#define RSSPECAN_ATTR_LTE_DOWNLINK_CUSTOM_SYNC_WEIGHT_FIRST_HALF_FRAME_REAL                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 8649
#define RSSPECAN_ATTR_LTE_DOWNLINK_CUSTOM_SYNC_WEIGHT_SECOND_HALF_FRAME_IMAGINARY                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 8650
#define RSSPECAN_ATTR_LTE_DOWNLINK_CUSTOM_SYNC_WEIGHT_SECOND_HALF_FRAME_REAL                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 8651
#define RSSPECAN_ATTR_LTE_DOWNLINK_PSYNC_RELATIVE_POWER                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 4881
#define RSSPECAN_ATTR_LTE_DOWNLINK_SSYNC_RELATIVE_POWER                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 4882
#define RSSPECAN_ATTR_LTE_DOWNLINK_REFERENCE_POWER                                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 2271
#define RSSPECAN_ATTR_LTE_DOWNLINK_POSITIONING_REFERENCE_SIGNAL_ENABLED_FSV                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 3600
#define RSSPECAN_ATTR_LTE_DOWNLINK_POSITIONING_REFERENCE_SIGNAL_BANDWIDTH_FSV                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 3601
#define RSSPECAN_ATTR_LTE_DOWNLINK_POSITIONING_REFERENCE_SIGNAL_CONFIGURATION_INDEX_FSV                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 3602
#define RSSPECAN_ATTR_LTE_DOWNLINK_POSITIONING_REFERENCE_SIGNAL_SUBFRAMES_FSV                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 3603
#define RSSPECAN_ATTR_LTE_DOWNLINK_POSITIONING_REFERENCE_SIGNAL_RELATIVE_POWER_FSV                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 3604
#define RSSPECAN_ATTR_LTE_DOWNLINK_POSITIONING_REFERENCE_SIGNAL_FRAME_NUMBER_OFFSET_FSV                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 3605
#define RSSPECAN_ATTR_LTE_DOWNLINK_POSITIONING_REFERENCE_SIGNAL_ENABLED                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 6280
#define RSSPECAN_ATTR_LTE_DOWNLINK_POSITIONING_REFERENCE_SIGNAL_BANDWIDTH                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 6281
#define RSSPECAN_ATTR_LTE_DOWNLINK_POSITIONING_REFERENCE_SIGNAL_CONFIGURATION_INDEX                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 6282
#define RSSPECAN_ATTR_LTE_DOWNLINK_POSITIONING_REFERENCE_SIGNAL_SUBFRAMES                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 6283
#define RSSPECAN_ATTR_LTE_DOWNLINK_POSITIONING_REFERENCE_SIGNAL_RELATIVE_POWER                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 6284
#define RSSPECAN_ATTR_LTE_DOWNLINK_POSITIONING_REFERENCE_SIGNAL_FRAME_NUMBER_OFFSET                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 6285
#define RSSPECAN_ATTR_LTE_DOWNLINK_CSI_RS_STATE_FSV                                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 3127
#define RSSPECAN_ATTR_LTE_DOWNLINK_CSI_RS_ANTENNA_PORTS_FSV                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 3128
#define RSSPECAN_ATTR_LTE_DOWNLINK_CSI_RS_CONFIGURATION_INDEX_FSV                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 3129
#define RSSPECAN_ATTR_LTE_DOWNLINK_CSI_RS_OVERWRITE_PDSCH_FSV                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 3130
#define RSSPECAN_ATTR_LTE_DOWNLINK_CSI_RS_RELATIVE_POWER_FSV                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 3131
#define RSSPECAN_ATTR_LTE_DOWNLINK_CSI_RS_SUBFRAME_CONFIGURATION_FSV                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 3132
#define RSSPECAN_ATTR_LTE_DOWNLINK_CSI_RS_FRAME_NUMBER_OFFSET_FSV                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 3133
#define RSSPECAN_ATTR_LTE_DOWNLINK_CSI_RS_STATE                                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 6286
#define RSSPECAN_ATTR_LTE_DOWNLINK_CSI_RS_ANTENNA_PORTS                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 6287
#define RSSPECAN_ATTR_LTE_DOWNLINK_CSI_RS_CONFIGURATION_INDEX                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 6288
#define RSSPECAN_ATTR_LTE_DOWNLINK_CSI_RS_OVERWRITE_PDSCH                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 6289
#define RSSPECAN_ATTR_LTE_DOWNLINK_CSI_RS_RELATIVE_POWER                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 6290
#define RSSPECAN_ATTR_LTE_DOWNLINK_CSI_RS_SUBFRAME_CONFIGURATION                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 6291
#define RSSPECAN_ATTR_LTE_DOWNLINK_PRB_SYMBOL_OFFSET                                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 6292
#define RSSPECAN_ATTR_LTE_DOWNLINK_CSI_RS_FRAME_NUMBER_OFFSET                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 6293
#define RSSPECAN_ATTR_LTE_DOWNLINK_PRB_SYMBOL_OFFSET_FSV                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 2604
#define RSSPECAN_ATTR_LTE_DOWNLINK_PBCH_STATE_FSV                                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 2274
#define RSSPECAN_ATTR_LTE_DOWNLINK_PBCH_RELATIVE_POWER_FSV                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 2275
#define RSSPECAN_ATTR_LTE_DOWNLINK_PBCH_STATE                                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 6294
#define RSSPECAN_ATTR_LTE_DOWNLINK_PBCH_RELATIVE_POWER                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 6295
#define RSSPECAN_ATTR_LTE_DOWNLINK_PCFICH_STATE_FSV                                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 2276
#define RSSPECAN_ATTR_LTE_DOWNLINK_PCFICH_RELATIVE_POWER_FSV                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 2277
#define RSSPECAN_ATTR_LTE_DOWNLINK_PCFICH_STATE                                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 6296
#define RSSPECAN_ATTR_LTE_DOWNLINK_PCFICH_RELATIVE_POWER                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 6297
#define RSSPECAN_ATTR_LTE_DOWNLINK_PHICH_DURATION_FSV                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 2279
#define RSSPECAN_ATTR_LTE_DOWNLINK_PHICH_NUMBER_OF_GROUPS_FSV                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 2278
#define RSSPECAN_ATTR_LTE_DOWNLINK_PHICH_ENHANCED_TEST_MODELS_FSV                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 3082
#define RSSPECAN_ATTR_LTE_DOWNLINK_PHICH_NG_PARAMETER_FSV                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 3081
#define RSSPECAN_ATTR_LTE_DOWNLINK_PHICH_RELATIVE_POWER_FSV                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 2280
#define RSSPECAN_ATTR_LTE_DOWNLINK_PHICH_DURATION                                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 6298
#define RSSPECAN_ATTR_LTE_DOWNLINK_PHICH_NUMBER_OF_GROUPS                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 6299
#define RSSPECAN_ATTR_LTE_DOWNLINK_PHICH_ENHANCED_TEST_MODELS                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 6300
#define RSSPECAN_ATTR_LTE_DOWNLINK_PHICH_NG_PARAMETER                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 6301
#define RSSPECAN_ATTR_LTE_DOWNLINK_PHICH_RELATIVE_POWER                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 6302
#define RSSPECAN_ATTR_LTE_DOWNLINK_PDCCH_FORMAT_FSV                                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 2282
#define RSSPECAN_ATTR_LTE_DOWNLINK_PDCCH_NUMBER_OF_PDCCHS_FSV                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 2281
#define RSSPECAN_ATTR_LTE_DOWNLINK_PDCCH_RELATIVE_POWER_FSV                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 2283
#define RSSPECAN_ATTR_LTE_DOWNLINK_PDSCH_POWER_RATIO_FSV                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 3106
#define RSSPECAN_ATTR_LTE_DOWNLINK_PDCCH_FORMAT                                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 6303
#define RSSPECAN_ATTR_LTE_DOWNLINK_PDCCH_NUMBER_OF_PDCCHS                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 6304
#define RSSPECAN_ATTR_LTE_DOWNLINK_PDCCH_RELATIVE_POWER                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 6305
#define RSSPECAN_ATTR_LTE_DOWNLINK_PDSCH_POWER_RATIO                                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 6306
#define RSSPECAN_ATTR_LTE_DOWNLINK_EPDCCH_PRB_PAIRS_FSV                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 5161
#define RSSPECAN_ATTR_LTE_DOWNLINK_EPDCCH_LOCALIZED_FSV                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 5162
#define RSSPECAN_ATTR_LTE_DOWNLINK_EPDCCH_RELATIVE_POWER_FSV                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 5163
#define RSSPECAN_ATTR_LTE_DOWNLINK_EPDCCH_RB_ASSIGNMENT_FSV                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 5164
#define RSSPECAN_ATTR_LTE_DOWNLINK_EPDCCH_SET_ID_FSV                                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 5165
#define RSSPECAN_ATTR_LTE_DOWNLINK_EPDCCH_PRB_PAIRS                                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 6307
#define RSSPECAN_ATTR_LTE_DOWNLINK_EPDCCH_LOCALIZED                                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 6308
#define RSSPECAN_ATTR_LTE_DOWNLINK_EPDCCH_RELATIVE_POWER                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 6309
#define RSSPECAN_ATTR_LTE_DOWNLINK_EPDCCH_RB_ASSIGNMENT                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 6310
#define RSSPECAN_ATTR_LTE_DOWNLINK_EPDCCH_SET_ID                                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 6311
#define RSSPECAN_ATTR_LTE_DOWNLINK_MBSFN_ENABLED_FSV                                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 5166
#define RSSPECAN_ATTR_LTE_DOWNLINK_MBSFN_RELATIVE_POWER_FSV                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 5167
#define RSSPECAN_ATTR_LTE_DOWNLINK_MBSFN_AREA_ID_FSV                                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 5168
#define RSSPECAN_ATTR_LTE_DOWNLINK_MBSFN_NON_MBSF_REGION_LENGTH_FSV                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 5169
#define RSSPECAN_ATTR_LTE_DOWNLINK_MBSFN_ENABLED                                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 6312
#define RSSPECAN_ATTR_LTE_DOWNLINK_MBSFN_RELATIVE_POWER                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 6313
#define RSSPECAN_ATTR_LTE_DOWNLINK_MBSFN_AREA_ID                                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 6314
#define RSSPECAN_ATTR_LTE_DOWNLINK_MBSFN_NON_MBSF_REGION_LENGTH                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 6315
#define RSSPECAN_ATTR_LTE_DOWNLINK_MBSFN_SUBFRAME_ENABLED_FSV                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 5170
#define RSSPECAN_ATTR_LTE_DOWNLINK_MBSFN_SUBFRAME_PMCH_ENABLED_FSV                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 5171
#define RSSPECAN_ATTR_LTE_DOWNLINK_MBSFN_SUBFRAME_PMCH_MODULATION_FSV                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 5172
#define RSSPECAN_ATTR_LTE_DOWNLINK_MBSFN_SUBFRAME_ENABLED                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 6316
#define RSSPECAN_ATTR_LTE_DOWNLINK_MBSFN_SUBFRAME_PMCH_ENABLED                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 6317
#define RSSPECAN_ATTR_LTE_DOWNLINK_MBSFN_SUBFRAME_PMCH_MODULATION                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 6318
#define RSSPECAN_ATTR_LTE_DOWNLINK_AUTO_REFERENCE_LEVEL                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 2247
#define RSSPECAN_ATTR_LTE_DOWNLINK_REFERENCE_LEVEL                                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 2248
#define RSSPECAN_ATTR_LTE_DOWNLINK_REFERENCE_LEVEL_IN_BB_MODE                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 2249
#define RSSPECAN_ATTR_LTE_DOWNLINK_OVERALL_FRAME_COUNT                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 3606
#define RSSPECAN_ATTR_LTE_DOWNLINK_NUMBER_OF_FRAMES_TO_ANALYZE                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 3607
#define RSSPECAN_ATTR_LTE_DOWNLINK_AUTO_COUNT_ACCORDING_TO_STANDARD                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 3608
#define RSSPECAN_ATTR_LTE_DOWNLINK_NUMBER_OF_SUBFRAMES_TO_ANALYZE                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 5173
#define RSSPECAN_ATTR_LTE_DOWNLINK_ALL_SUBFRAMES_TO_ANALYZE                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 5174
#define RSSPECAN_ATTR_LTE_DOWNLINK_TRIGGER_PORT                                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 4883
#define RSSPECAN_ATTR_LTE_DOWNLINK_TRIGGER_OFFSET                                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 4884
#define RSSPECAN_ATTR_LTE_DOWNLINK_TRIGGER_LEVEL                                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 4885
#define RSSPECAN_ATTR_LTE_DOWNLINK_TRIGGER_USE_SAME_CONFIGURATION_AS_MASTER_ENABLED                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 7366
#define RSSPECAN_ATTR_LTE_DOWNLINK_BOOSTING_ESTIMATION                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 2732
#define RSSPECAN_ATTR_LTE_DOWNLINK_CHANNEL_ESTIMATION                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 2254
#define RSSPECAN_ATTR_LTE_DOWNLINK_PHASE_TRACKING                                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 2258
#define RSSPECAN_ATTR_LTE_DOWNLINK_TIMING                                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 2259
#define RSSPECAN_ATTR_LTE_DOWNLINK_MULTICARRIER_FILTER                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 2734
#define RSSPECAN_ATTR_LTE_DOWNLINK_COMPENSATE_CROSSTALK_FSV                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 3609
#define RSSPECAN_ATTR_LTE_DOWNLINK_SCRAMBLING_OF_CODED_BITS                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 2256
#define RSSPECAN_ATTR_LTE_DOWNLINK_DECODE_ALL_CHANNELS                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 3610
#define RSSPECAN_ATTR_LTE_DOWNLINK_EVM_CALCULATION_METHOD                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 2255
#define RSSPECAN_ATTR_LTE_DOWNLINK_PDSCH_REFERENCE_DATA                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 2733
#define RSSPECAN_ATTR_LTE_DOWNLINK_SUPPRESS_INTERFERER_FOR_SYNCHRONIZATION                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 10495
#define RSSPECAN_ATTR_LTE_DOWNLINK_COMPENSATE_CROSSTALK                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 4886
#define RSSPECAN_ATTR_LTE_DOWNLINK_CARRIER_AGGREGATION                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 4887
#define RSSPECAN_ATTR_LTE_DOWNLINK_COMPONENT_CARRIERS                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 4888
#define RSSPECAN_ATTR_LTE_DOWNLINK_NUMBER_OF_DEVICES                                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 5806
#define RSSPECAN_ATTR_LTE_DOWNLINK_CARRIER_CENTER_FREQUENCY                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 4889
#define RSSPECAN_ATTR_LTE_DOWNLINK_CARRIER_CENTER_FREQUENCY_OFFSET                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 7367
#define RSSPECAN_ATTR_LTE_DOWNLINK_ON_OFF_NUMBER_OF_FRAMES                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 3611
#define RSSPECAN_ATTR_LTE_DOWNLINK_ON_OFF_NOISE_CORRECTION                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 3612
#define RSSPECAN_ATTR_LTE_DOWNLINK_ON_OFF_ADJUST_TIMING                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 4893
#define RSSPECAN_ATTR_LTE_DOWNLINK_ON_OFF_UNIT                                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 5807
#define RSSPECAN_ATTR_LTE_DOWNLINK_ACP_ASSUMED_ADJ_CHANNEL_CARRIER                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 2290
#define RSSPECAN_ATTR_LTE_DOWNLINK_ACP_ASSUMED_UPPER_ADJ_CHANNEL_CARRIER                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 5808
#define RSSPECAN_ATTR_LTE_DOWNLINK_SEM_CHANNEL                                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 2289
#define RSSPECAN_ATTR_LTE_DOWNLINK_HOME_AREA_BASESTATION_POWER                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 3613
#define RSSPECAN_ATTR_LTE_DOWNLINK_HOME_AREA_BASESTATION_POWER_AUTO                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 5482
#define RSSPECAN_ATTR_LTE_DOWNLINK_SEM_OPERATING_BAND_LIMIT_ENABLED                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 7368
#define RSSPECAN_ATTR_LTE_DOWNLINK_SEM_OPERATING_BAND                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 7369
#define RSSPECAN_ATTR_LTE_DOWNLINK_TAB_SELECT                                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 7370
#define RSSPECAN_ATTR_LTE_DOWNLINK_MEASUREMENT                                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 3126
#define RSSPECAN_ATTR_LTE_DOWNLINK_MEASUREMENT_SIGNAL_DETECTION                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 6435
#define RSSPECAN_ATTR_LTE_DOWNLINK_CONSTELLATION_ALLOCATION_FSV                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 4896
#define RSSPECAN_ATTR_LTE_DOWNLINK_CONSTELLATION_ALLOCATION_ALL_FSV                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 4897
#define RSSPECAN_ATTR_LTE_DOWNLINK_CONSTELLATION_CARRIER_FSV                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 4900
#define RSSPECAN_ATTR_LTE_DOWNLINK_CONSTELLATION_CARRIER_ALL_FSV                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 4901
#define RSSPECAN_ATTR_LTE_DOWNLINK_CONSTELLATION_CODEWORD_FSV                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 4902
#define RSSPECAN_ATTR_LTE_DOWNLINK_CONSTELLATION_CODEWORD_ALL_FSV                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 4903
#define RSSPECAN_ATTR_LTE_DOWNLINK_BEAMFORMING_ANTENNA_PORT_FSV                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 4904
#define RSSPECAN_ATTR_LTE_DOWNLINK_SUBFRAME_SELECTION                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 6319
#define RSSPECAN_ATTR_LTE_DOWNLINK_SUBFRAME_SELECTION_ALL                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 6320
#define RSSPECAN_ATTR_LTE_DOWNLINK_CONSTELLATION_MODULATION                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 6321
#define RSSPECAN_ATTR_LTE_DOWNLINK_CONSTELLATION_MODULATION_ALL                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 6322
#define RSSPECAN_ATTR_LTE_DOWNLINK_CONSTELLATION_ALLOCATION                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 6323
#define RSSPECAN_ATTR_LTE_DOWNLINK_CONSTELLATION_ALLOCATION_ALL                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 6324
#define RSSPECAN_ATTR_LTE_DOWNLINK_CONSTELLATION_SYMBOL                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 6325
#define RSSPECAN_ATTR_LTE_DOWNLINK_CONSTELLATION_SYMBOL_ALL                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 6326
#define RSSPECAN_ATTR_LTE_DOWNLINK_CONSTELLATION_CARRIER                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 6327
#define RSSPECAN_ATTR_LTE_DOWNLINK_CONSTELLATION_CARRIER_ALL                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 6328
#define RSSPECAN_ATTR_LTE_DOWNLINK_CONSTELLATION_CODEWORD                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 6329
#define RSSPECAN_ATTR_LTE_DOWNLINK_CONSTELLATION_CODEWORD_ALL                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 6330
#define RSSPECAN_ATTR_LTE_DOWNLINK_CONSTELLATION_LOCATION                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 6331
#define RSSPECAN_ATTR_LTE_DOWNLINK_BEAMFORMING_ANTENNA_PORT                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 6332
#define RSSPECAN_ATTR_LTE_DOWNLINK_BEAMFORMING_ANTENNA_PORT_CELL                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 6733
#define RSSPECAN_ATTR_LTE_DOWNLINK_BEAMFORMING_ANTENNA_PORT_CSI                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 6734
#define RSSPECAN_ATTR_LTE_DOWNLINK_Y_AXIS_SCALING_AUTO                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 2938
#define RSSPECAN_ATTR_LTE_DOWNLINK_YAXIS_SCALING_AUTO_ONCE                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 3614
#define RSSPECAN_ATTR_LTE_DOWNLINK_Y_AXIS_SCALING_PER_DIVISION                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 2939
#define RSSPECAN_ATTR_LTE_DOWNLINK_Y_AXIS_SCALING_OFFSET                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 2940
#define RSSPECAN_ATTR_LTE_DOWNLINK_YAXIS_SCALING_MINIMUM                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 3615
#define RSSPECAN_ATTR_LTE_DOWNLINK_YAXIS_SCALING_MAXIMUM                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 3616
#define RSSPECAN_ATTR_LTE_DOWNLINK_SUBWINDOW_Y_AXIS_SCALING_AUTO                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 7371
#define RSSPECAN_ATTR_LTE_DOWNLINK_SUBWINDOW_Y_AXIS_SCALING_RESTORE                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 7372
#define RSSPECAN_ATTR_LTE_DOWNLINK_SUBWINDOW_Y_AXIS_SCALING_MAXIMUM                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 7373
#define RSSPECAN_ATTR_LTE_DOWNLINK_SUBWINDOW_Y_AXIS_SCALING_MINIMUM                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 7374
#define RSSPECAN_ATTR_LTE_DOWNLINK_MARKER_COUPLING                                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 5175
#define RSSPECAN_ATTR_LTE_DOWNLINK_EVM_UNITS                                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 2287
#define RSSPECAN_ATTR_LTE_DOWNLINK_BIT_STREAM_FORMAT                                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 2288
#define RSSPECAN_ATTR_LTE_DOWNLINK_CARRIER_AXES                                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 4905
#define RSSPECAN_ATTR_LTE_DOWNLINK_SUBWINDOW_COUPLING                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 7375
#define RSSPECAN_ATTR_LTE_DOWNLINK_MEASUREMENT_TYPE                                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 2284
#define RSSPECAN_ATTR_LTE_DOWNLINK_EVM_PDSCH_QPSK_RESULT_MIN_FSV                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 3617
#define RSSPECAN_ATTR_LTE_DOWNLINK_EVM_PDSCH_QPSK_RESULT_FSV                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 2293
#define RSSPECAN_ATTR_LTE_DOWNLINK_EVM_PDSCH_QPSK_RESULT_MAX_FSV                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 3618
#define RSSPECAN_ATTR_LTE_DOWNLINK_EVM_PDSCH_QPSK_LIMIT_CHECK_RESULT_AVERAGE_FSV                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 3619
#define RSSPECAN_ATTR_LTE_DOWNLINK_EVM_PDSCH_QPSK_LIMIT_CHECK_RESULT_MAX_FSV                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 3620
#define RSSPECAN_ATTR_LTE_DOWNLINK_EVM_PDSCH_QPSK_RESULT_MIN                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 4906
#define RSSPECAN_ATTR_LTE_DOWNLINK_EVM_PDSCH_QPSK_RESULT                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 4907
#define RSSPECAN_ATTR_LTE_DOWNLINK_EVM_PDSCH_QPSK_RESULT_MAX                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 4908
#define RSSPECAN_ATTR_LTE_DOWNLINK_EVM_PDSCH_QPSK_LIMIT_CHECK_RESULT_AVERAGE                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 6333
#define RSSPECAN_ATTR_LTE_DOWNLINK_EVM_PDSCH_QPSK_LIMIT_CHECK_RESULT_MAX                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 6334
#define RSSPECAN_ATTR_LTE_DOWNLINK_EVM_PDSCH_16QAM_RESULT_MIN_FSV                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 3621
#define RSSPECAN_ATTR_LTE_DOWNLINK_EVM_PDSCH_16QAM_RESULT_FSV                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 2294
#define RSSPECAN_ATTR_LTE_DOWNLINK_EVM_PDSCH_16QAM_RESULT_MAX_FSV                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 3622
#define RSSPECAN_ATTR_LTE_DOWNLINK_EVM_PDSCH_16QAM_LIMIT_CHECK_RESULT_AVERAGE_FSV                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 3623
#define RSSPECAN_ATTR_LTE_DOWNLINK_EVM_PDSCH_16QAM_LIMIT_CHECK_RESULT_MAX_FSV                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 3624
#define RSSPECAN_ATTR_LTE_DOWNLINK_EVM_PDSCH_16QAM_RESULT_MIN                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 4909
#define RSSPECAN_ATTR_LTE_DOWNLINK_EVM_PDSCH_16QAM_RESULT                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 4910
#define RSSPECAN_ATTR_LTE_DOWNLINK_EVM_PDSCH_16QAM_RESULT_MAX                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 4911
#define RSSPECAN_ATTR_LTE_DOWNLINK_EVM_PDSCH_16QAM_LIMIT_CHECK_RESULT_AVERAGE                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 6335
#define RSSPECAN_ATTR_LTE_DOWNLINK_EVM_PDSCH_16QAM_LIMIT_CHECK_RESULT_MAX                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 6336
#define RSSPECAN_ATTR_LTE_DOWNLINK_EVM_PDSCH_64QAM_RESULT_MIN_FSV                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 3625
#define RSSPECAN_ATTR_LTE_DOWNLINK_EVM_PDSCH_64QAM_RESULT_FSV                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 2295
#define RSSPECAN_ATTR_LTE_DOWNLINK_EVM_PDSCH_64QAM_RESULT_MAX_FSV                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 3626
#define RSSPECAN_ATTR_LTE_DOWNLINK_EVM_PDSCH_64QAM_LIMIT_CHECK_RESULT_AVERAGE_FSV                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 3627
#define RSSPECAN_ATTR_LTE_DOWNLINK_EVM_PDSCH_64QAM_LIMIT_CHECK_RESULT_MAX_FSV                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 3628
#define RSSPECAN_ATTR_LTE_DOWNLINK_EVM_PDSCH_64QAM_RESULT_MIN                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 4912
#define RSSPECAN_ATTR_LTE_DOWNLINK_EVM_PDSCH_64QAM_RESULT                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 4913
#define RSSPECAN_ATTR_LTE_DOWNLINK_EVM_PDSCH_64QAM_RESULT_MAX                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 4914
#define RSSPECAN_ATTR_LTE_DOWNLINK_EVM_PDSCH_64QAM_LIMIT_CHECK_RESULT_AVERAGE                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 6337
#define RSSPECAN_ATTR_LTE_DOWNLINK_EVM_PDSCH_64QAM_LIMIT_CHECK_RESULT_MAX                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 6338
#define RSSPECAN_ATTR_LTE_DOWNLINK_EVM_PDSCH_256QAM_RESULT_MIN_FSV                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 6339
#define RSSPECAN_ATTR_LTE_DOWNLINK_EVM_PDSCH_256QAM_RESULT_FSV                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 6340
#define RSSPECAN_ATTR_LTE_DOWNLINK_EVM_PDSCH_256QAM_RESULT_MAX_FSV                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 6341
#define RSSPECAN_ATTR_LTE_DOWNLINK_EVM_PDSCH_256QAM_RESULT_MIN                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 5811
#define RSSPECAN_ATTR_LTE_DOWNLINK_EVM_PDSCH_256QAM_RESULT                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 5812
#define RSSPECAN_ATTR_LTE_DOWNLINK_EVM_PDSCH_256QAM_RESULT_MAX                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 5813
#define RSSPECAN_ATTR_LTE_DOWNLINK_EVM_PDSCH_1024QAM_RESULT_MIN                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 6735
#define RSSPECAN_ATTR_LTE_DOWNLINK_EVM_PDSCH_1024QAM_RESULT                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 6736
#define RSSPECAN_ATTR_LTE_DOWNLINK_EVM_PDSCH_1024QAM_RESULT_MAX                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 6737
#define RSSPECAN_ATTR_LTE_DOWNLINK_EVM_PDSCH_1024QAM_LIMIT_CHECK_RESULT_AVERAGE                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 6738
#define RSSPECAN_ATTR_LTE_DOWNLINK_EVM_PDSCH_1024QAM_LIMIT_CHECK_RESULT_MAX                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 6739
#define RSSPECAN_ATTR_LTE_DOWNLINK_TIME_ALIGNMENT_ERROR_MIN                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 3629
#define RSSPECAN_ATTR_LTE_DOWNLINK_TIME_ALIGNMENT_ERROR_AVERAGE                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 3630
#define RSSPECAN_ATTR_LTE_DOWNLINK_TIME_ALIGNMENT_ERROR_MAX                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 3631
#define RSSPECAN_ATTR_LTE_DOWNLINK_TIME_ALIGNMENT_ERROR_FREQUENCY_ERROR_AVERAGE                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 4915
#define RSSPECAN_ATTR_LTE_DOWNLINK_CARRIER_FREQUENCY_ERROR_HZ_AVERAGE                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 7376
#define RSSPECAN_ATTR_LTE_DOWNLINK_CREST_FACTOR_RESULT_FSV                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 2305
#define RSSPECAN_ATTR_LTE_DOWNLINK_EVM_ALL_RESULT_MIN_FSV                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 3632
#define RSSPECAN_ATTR_LTE_DOWNLINK_EVM_ALL_RESULT_AVERAGE_FSV                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 3633
#define RSSPECAN_ATTR_LTE_DOWNLINK_EVM_ALL_RESULT_MAX_FSV                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 3634
#define RSSPECAN_ATTR_LTE_DOWNLINK_EVM_AL_LIMIT_CHECK_RESULT_AVERAGE                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 3635
#define RSSPECAN_ATTR_LTE_DOWNLINK_EVM_AL_LIMIT_CHECK_RESULT_MAX                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 3636
#define RSSPECAN_ATTR_LTE_DOWNLINK_EVM_ALL_RESULT_MIN                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 4916
#define RSSPECAN_ATTR_LTE_DOWNLINK_EVM_ALL_RESULT_AVERAGE                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 4917
#define RSSPECAN_ATTR_LTE_DOWNLINK_EVM_ALL_RESULT_MAX                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 4918
#define RSSPECAN_ATTR_LTE_DOWNLINK_EVM_PHYSICAL_CHANNEL_RESULT_MIN_FSV                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 3637
#define RSSPECAN_ATTR_LTE_DOWNLINK_EVM_PHYSICAL_CHANNEL_RESULT_AVERAGE_FSV                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 3638
#define RSSPECAN_ATTR_LTE_DOWNLINK_EVM_PHYSICAL_CHANNEL_RESULT_MAX_FSV                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 3639
#define RSSPECAN_ATTR_LTE_DOWNLINK_EVM_PHYSICAL_CHANNEL_LIMIT_CHECK_RESULT_AVERAGE_FSV                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 3640
#define RSSPECAN_ATTR_LTE_DOWNLINK_EVM_PHYSICAL_CHANNEL_LIMIT_CHECK_RESULT_MAX_FSV                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 3641
#define RSSPECAN_ATTR_LTE_DOWNLINK_EVM_PHYSICAL_CHANNEL_RESULT_MIN                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 4919
#define RSSPECAN_ATTR_LTE_DOWNLINK_EVM_PHYSICAL_CHANNEL_RESULT_AVERAGE                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 4920
#define RSSPECAN_ATTR_LTE_DOWNLINK_EVM_PHYSICAL_CHANNEL_RESULT_MAX                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 4921
#define RSSPECAN_ATTR_LTE_DOWNLINK_EVM_PHYSICAL_CHANNEL_LIMIT_CHECK_RESULT_AVERAGE                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 6342
#define RSSPECAN_ATTR_LTE_DOWNLINK_EVM_PHYSICAL_CHANNEL_LIMIT_CHECK_RESULT_MAX                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 6343
#define RSSPECAN_ATTR_LTE_DOWNLINK_EVM_PHYSICAL_SIGNAL_RESULT_MIN_FSV                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 3642
#define RSSPECAN_ATTR_LTE_DOWNLINK_EVM_PHYSICAL_SIGNAL_RESULT_AVERAGE_FSV                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 3643
#define RSSPECAN_ATTR_LTE_DOWNLINK_EVM_PHYSICAL_SIGNAL_RESULT_MAX_FSV                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 3644
#define RSSPECAN_ATTR_LTE_DOWNLINK_EVM_PHYSICAL_SIGNAL_LIMIT_CHECK_RESULT_AVERAGE_FSV                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 3645
#define RSSPECAN_ATTR_LTE_DOWNLINK_EVM_PHYSICAL_SIGNAL_LIMIT_CHECK_RESULT_MAX_FSV                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 3646
#define RSSPECAN_ATTR_LTE_DOWNLINK_EVM_PHYSICAL_SIGNAL_RESULT_MIN                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 4922
#define RSSPECAN_ATTR_LTE_DOWNLINK_EVM_PHYSICAL_SIGNAL_RESULT_AVERAGE                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 4923
#define RSSPECAN_ATTR_LTE_DOWNLINK_EVM_PHYSICAL_SIGNAL_RESULT_MAX                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 4924
#define RSSPECAN_ATTR_LTE_DOWNLINK_EVM_PHYSICAL_SIGNAL_LIMIT_CHECK_RESULT_AVERAGE                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 6344
#define RSSPECAN_ATTR_LTE_DOWNLINK_EVM_PHYSICAL_SIGNAL_LIMIT_CHECK_RESULT_MAX                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 6345
#define RSSPECAN_ATTR_LTE_DOWNLINK_FREQUENCY_ERROR_RESULT_MIN_FSV                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 3647
#define RSSPECAN_ATTR_LTE_DOWNLINK_FREQUENCY_ERROR_RESULT_AVERAGE_FSV                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 3648
#define RSSPECAN_ATTR_LTE_DOWNLINK_FREQUENCY_ERROR_RESULT_MAX_FSV                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 3649
#define RSSPECAN_ATTR_LTE_DOWNLINK_FREQUENCY_ERROR_LIMIT_CHECK_RESULT_AVERAGE_FSV                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 3650
#define RSSPECAN_ATTR_LTE_DOWNLINK_FREQUENCY_ERROR_LIMIT_CHECK_RESULT_MAX_FSV                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 3651
#define RSSPECAN_ATTR_LTE_DOWNLINK_FREQUENCY_ERROR_RESULT_MIN                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 4925
#define RSSPECAN_ATTR_LTE_DOWNLINK_FREQUENCY_ERROR_RESULT_AVERAGE                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 4926
#define RSSPECAN_ATTR_LTE_DOWNLINK_FREQUENCY_ERROR_RESULT_MAX                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 4927
#define RSSPECAN_ATTR_LTE_DOWNLINK_FREQUENCY_ERROR_LIMIT_CHECK_RESULT_AVERAGE                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 6346
#define RSSPECAN_ATTR_LTE_DOWNLINK_FREQUENCY_ERROR_LIMIT_CHECK_RESULT_MAX                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 6347
#define RSSPECAN_ATTR_LTE_DOWNLINK_IQ_GAIN_IMBALANCE_RESULT_MIN_FSV                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 3652
#define RSSPECAN_ATTR_LTE_DOWNLINK_IQ_GAIN_IMBALANCE_RESULT_AVERAGE_FSV                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 3653
#define RSSPECAN_ATTR_LTE_DOWNLINK_IQ_GAIN_IMBALANCE_RESULT_MAX_FSV                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 3654
#define RSSPECAN_ATTR_LTE_DOWNLINK_IQ_GAIN_IMBALANCE_LIMIT_CHECK_RESULT_AVERAGE_FSV                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 3655
#define RSSPECAN_ATTR_LTE_DOWNLINK_IQ_GAIN_IMBALANCE_LIMIT_CHECK_RESULT_MAX_FSV                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 3656
#define RSSPECAN_ATTR_LTE_DOWNLINK_IQ_GAIN_IMBALANCE_RESULT_MIN                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 4928
#define RSSPECAN_ATTR_LTE_DOWNLINK_IQ_GAIN_IMBALANCE_RESULT_AVERAGE                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 4929
#define RSSPECAN_ATTR_LTE_DOWNLINK_IQ_GAIN_IMBALANCE_RESULT_MAX                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 4930
#define RSSPECAN_ATTR_LTE_DOWNLINK_IQ_GAIN_IMBALANCE_LIMIT_CHECK_RESULT_AVERAGE                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 6348
#define RSSPECAN_ATTR_LTE_DOWNLINK_IQ_GAIN_IMBALANCE_LIMIT_CHECK_RESULT_MAX                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 6349
#define RSSPECAN_ATTR_LTE_DOWNLINK_IQ_OFFSET_RESULT_MIN_FSV                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 3657
#define RSSPECAN_ATTR_LTE_DOWNLINK_IQ_OFFSET_RESULT_AVERAGE_FSV                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 3658
#define RSSPECAN_ATTR_LTE_DOWNLINK_IQ_OFFSET_RESULT_MAX_FSV                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 3659
#define RSSPECAN_ATTR_LTE_DOWNLINK_IQ_OFFSET_LIMIT_CHECK_RESULT_AVERAGE_FSV                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 3660
#define RSSPECAN_ATTR_LTE_DOWNLINK_IQ_OFFSET_LIMIT_CHECK_RESULT_MAX_FSV                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 3661
#define RSSPECAN_ATTR_LTE_DOWNLINK_IQ_OFFSET_RESULT_MIN                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 4931
#define RSSPECAN_ATTR_LTE_DOWNLINK_IQ_OFFSET_RESULT_AVERAGE                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 4932
#define RSSPECAN_ATTR_LTE_DOWNLINK_IQ_OFFSET_RESULT_MAX                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 4933
#define RSSPECAN_ATTR_LTE_DOWNLINK_IQ_OFFSET_LIMIT_CHECK_RESULT_AVERAGE                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 6350
#define RSSPECAN_ATTR_LTE_DOWNLINK_IQ_OFFSET_LIMIT_CHECK_RESULT_MAX                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 6351
#define RSSPECAN_ATTR_LTE_DOWNLINK_OSTP_RESULT_MIN_FSV                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 3662
#define RSSPECAN_ATTR_LTE_DOWNLINK_OSTP_RESULT_FSV                                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 2307
#define RSSPECAN_ATTR_LTE_DOWNLINK_OSTP_RESULT_MAX_FSV                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 3663
#define RSSPECAN_ATTR_LTE_DOWNLINK_OSTP_RESULT_MIN                                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 4934
#define RSSPECAN_ATTR_LTE_DOWNLINK_OSTP_RESULT_AVERAGE                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 4935
#define RSSPECAN_ATTR_LTE_DOWNLINK_OSTP_RESULT_MAX                                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 4936
#define RSSPECAN_ATTR_LTE_DOWNLINK_FRAME_POWER_RESULT_MIN_FSV                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 3664
#define RSSPECAN_ATTR_LTE_DOWNLINK_FRAME_POWER_RESULT_AVERAGE_FSV                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 3665
#define RSSPECAN_ATTR_LTE_DOWNLINK_FRAME_POWER_RESULT_MAX_FSV                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 3666
#define RSSPECAN_ATTR_LTE_DOWNLINK_FRAME_POWER_RESULT_MIN                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 4937
#define RSSPECAN_ATTR_LTE_DOWNLINK_FRAME_POWER_RESULT_AVERAGE                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 4938
#define RSSPECAN_ATTR_LTE_DOWNLINK_FRAME_POWER_RESULT_MAX                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 4939
#define RSSPECAN_ATTR_LTE_DOWNLINK_IQ_QUADRATURE_ERROR_RESULT_MIN_FSV                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 3667
#define RSSPECAN_ATTR_LTE_DOWNLINK_IQ_QUADRATURE_ERROR_RESULT_AVERAGE_FSV                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 3668
#define RSSPECAN_ATTR_LTE_DOWNLINK_IQ_QUADRATURE_ERROR_RESULT_MAX_FSV                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 3669
#define RSSPECAN_ATTR_LTE_DOWNLINK_IQ_QUADRATURE_ERROR_LIMIT_CHECK_RESULT_AVERAGE_FSV                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 3670
#define RSSPECAN_ATTR_LTE_DOWNLINK_IQ_QUADRATURE_ERROR_LIMIT_CHECK_RESULT_MAX_FSV                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 3671
#define RSSPECAN_ATTR_LTE_DOWNLINK_IQ_QUADRATURE_ERROR_RESULT_MIN                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 4940
#define RSSPECAN_ATTR_LTE_DOWNLINK_IQ_QUADRATURE_ERROR_RESULT_AVERAGE                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 4941
#define RSSPECAN_ATTR_LTE_DOWNLINK_IQ_QUADRATURE_ERROR_RESULT_MAX                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 4778
#define RSSPECAN_ATTR_LTE_DOWNLINK_IQ_QUADRATURE_ERROR_LIMIT_CHECK_RESULT_AVERAGE                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 6352
#define RSSPECAN_ATTR_LTE_DOWNLINK_IQ_QUADRATURE_ERROR_LIMIT_CHECK_RESULT_MAX                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 6353
#define RSSPECAN_ATTR_LTE_DOWNLINK_RSTP_RESULT_MIN_FSV                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 3672
#define RSSPECAN_ATTR_LTE_DOWNLINK_RSTP_RESULT_AVERAGE_FSV                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 3673
#define RSSPECAN_ATTR_LTE_DOWNLINK_RSTP_RESULT_MAX_FSV                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 3674
#define RSSPECAN_ATTR_LTE_DOWNLINK_RSTP_RESULT_MIN                                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 4779
#define RSSPECAN_ATTR_LTE_DOWNLINK_RSTP_RESULT_AVERAGE                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 4780
#define RSSPECAN_ATTR_LTE_DOWNLINK_RSTP_RESULT_MAX                                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 4781
#define RSSPECAN_ATTR_LTE_DOWNLINK_RSSI_RESULT_MIN                                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 4782
#define RSSPECAN_ATTR_LTE_DOWNLINK_RSSI_RESULT_AVERAGE                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 4783
#define RSSPECAN_ATTR_LTE_DOWNLINK_RSSI_RESULT_MAX                                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 4784
#define RSSPECAN_ATTR_LTE_DOWNLINK_SAMPLING_ERROR_RESULT_MIN_FSV                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 3675
#define RSSPECAN_ATTR_LTE_DOWNLINK_SAMPLING_ERROR_RESULT_AVERAGE_FSV                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 3676
#define RSSPECAN_ATTR_LTE_DOWNLINK_SAMPLING_ERROR_RESULT_MAX_FSV                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 3677
#define RSSPECAN_ATTR_LTE_DOWNLINK_SAMPLING_ERROR_LIMIT_CHECK_RESULT_AVERAGE_FSV                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 3678
#define RSSPECAN_ATTR_LTE_DOWNLINK_SAMPLING_ERROR_LIMIT_CHECK_RESULT_MAX_FSV                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 3679
#define RSSPECAN_ATTR_LTE_DOWNLINK_SAMPLING_ERROR_RESULT_MIN                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 4785
#define RSSPECAN_ATTR_LTE_DOWNLINK_SAMPLING_ERROR_RESULT_AVERAGE                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 4786
#define RSSPECAN_ATTR_LTE_DOWNLINK_SAMPLING_ERROR_RESULT_MAX                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 4787
#define RSSPECAN_ATTR_LTE_DOWNLINK_SAMPLING_ERROR_LIMIT_CHECK_RESULT_AVERAGE                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 6354
#define RSSPECAN_ATTR_LTE_DOWNLINK_SAMPLING_ERROR_LIMIT_CHECK_RESULT_MAX                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 6355
#define RSSPECAN_ATTR_LTE_DOWNLINK_TRIGGER_TO_FRAME_RESULT_FSV                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 2306
#define RSSPECAN_ATTR_LTE_DOWNLINK_DETECTED_CYCLIC_PREFIX_FSV                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 2310
#define RSSPECAN_ATTR_LTE_DOWNLINK_CREST_FACTOR_RESULT_AVERAGE                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 4788
#define RSSPECAN_ATTR_LTE_DOWNLINK_TRIGGER_TO_FRAME_RESULT                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 4789
#define RSSPECAN_ATTR_LTE_DOWNLINK_DETECTED_CYCLIC_PREFIX                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 4790
#define RSSPECAN_ATTR_LTE_DOWNLINK_ACP_NOISE_CORRECTION                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 2291
#define RSSPECAN_ATTR_LTE_DOWNLINK_MARKER_Z_AXIS                                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 6356
#define RSSPECAN_ATTR_LTE_UPLINK_MODE                                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 2473
#define RSSPECAN_ATTR_LTE_UPLINK_COMPONENT_CARRIERS                                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 5844
#define RSSPECAN_ATTR_LTE_LINK_DIRECTION                                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 2474
#define RSSPECAN_ATTR_LTE_UPLINK_DUPLEXING                                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 2477
#define RSSPECAN_ATTR_LTE_UPLINK_CHANNEL_BANDWIDTH                                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 2478
#define RSSPECAN_ATTR_LTE_UPLINK_NUMBER_OF_RESOURCE_BLOCKS_FSV                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 2479
#define RSSPECAN_ATTR_LTE_UPLINK_CYCLIC_PREFIX_FSV                                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 2480
#define RSSPECAN_ATTR_LTE_UPLINK_TDD_UL_DL_ALLOCATIONS_FSV                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 2492
#define RSSPECAN_ATTR_LTE_UPLINK_TDD_SPECIAL_SUBFRAME_FSV                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 2941
#define RSSPECAN_ATTR_LTE_UPLINK_CELL_IDENTITY_GROUP_FSV                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 2493
#define RSSPECAN_ATTR_LTE_UPLINK_CELL_IDENTITY_FSV                                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 2494
#define RSSPECAN_ATTR_LTE_UPLINK_MIMO_ANTENNA_FSV                                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 3155
#define RSSPECAN_ATTR_LTE_UPLINK_CONFIGURABLE_SUBFRAMES_FSV                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 2495
#define RSSPECAN_ATTR_LTE_UPLINK_AUTOMATIC_DEMODULATION                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 4822
#define RSSPECAN_ATTR_LTE_UPLINK_SUBFRAME_CONFIGURATION_DETECTION                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 4823
#define RSSPECAN_ATTR_LTE_UPLINK_STORE_DEMODULATION_SETTINGS_FSV                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 2475
#define RSSPECAN_ATTR_LTE_UPLINK_LOAD_DEMODULATION_SETTINGS_FSV                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 2476
#define RSSPECAN_ATTR_LTE_UPLINK_OPERATING_BAND_INDEX_FSV                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 7378
#define RSSPECAN_ATTR_LTE_UPLINK_EXTREME_CONDITIONS_FSV                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 7379
#define RSSPECAN_ATTR_LTE_UPLINK_TEST_MODEL                                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 7380
#define RSSPECAN_ATTR_LTE_UPLINK_CARRIER_CHANNEL_BANDWIDTH                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 5176
#define RSSPECAN_ATTR_LTE_UPLINK_NUMBER_OF_RESOURCE_BLOCKS                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 6357
#define RSSPECAN_ATTR_LTE_UPLINK_CYCLIC_PREFIX                                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 5814
#define RSSPECAN_ATTR_LTE_UPLINK_TDD_UL_DL_ALLOCATIONS                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 5815
#define RSSPECAN_ATTR_LTE_UPLINK_TDD_SPECIAL_SUBFRAME                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 5816
#define RSSPECAN_ATTR_LTE_UPLINK_CELL_IDENTITY_GROUP                                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 5817
#define RSSPECAN_ATTR_LTE_UPLINK_DETECTED_CELL_IDENTITY_GROUP                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 8431
#define RSSPECAN_ATTR_LTE_UPLINK_CELL_IDENTITY                                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 5818
#define RSSPECAN_ATTR_LTE_UPLINK_DETECTED_CELL_IDENTITY                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 8432
#define RSSPECAN_ATTR_LTE_UPLINK_CONFIGURABLE_SUBFRAMES                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 5819
#define RSSPECAN_ATTR_LTE_UPLINK_MIMO_ANTENNA                                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 5820
#define RSSPECAN_ATTR_LTE_UPLINK_OPERATING_BAND_INDEX                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 6358
#define RSSPECAN_ATTR_LTE_UPLINK_EXTREME_CONDITIONS                                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 6359
#define RSSPECAN_ATTR_LTE_UPLINK_LOAD_DEMODULATION_SETTINGS                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 7381
#define RSSPECAN_ATTR_LTE_UPLINK_STORE_DEMODULATION_SETTINGS                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 8652
#define RSSPECAN_ATTR_LTE_UPLINK_SUBFRAME_ALLOCATION_ENABLE_PUCCH_FSV                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 2496
#define RSSPECAN_ATTR_LTE_UPLINK_SUBFRAME_ALLOCATION_MODULATION_FSV                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 2497
#define RSSPECAN_ATTR_LTE_UPLINK_SUBFRAME_ALLOCATION_RESOURCE_BLOCKS_FSV                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 2498
#define RSSPECAN_ATTR_LTE_UPLINK_SUBFRAME_ALLOCATION_RESOURCE_BLOCKS_OFFSET_FSV                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 2499
#define RSSPECAN_ATTR_LTE_UPLINK_SUBFRAME_ALLOCATION_POWER                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 2500
#define RSSPECAN_ATTR_LTE_UPLINK_SUBFRAME_ALLOCATION_RESOURCE_ALLOCATION_TYPE_1_FSV                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 4824
#define RSSPECAN_ATTR_LTE_UPLINK_SUBFRAME_ALLOCATION_N_DMRS_FSV                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 4825
#define RSSPECAN_ATTR_LTE_UPLINK_SUBFRAME_ALLOCATION_CYCLIC_SHIFT_FIELD_FSV                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 4826
#define RSSPECAN_ATTR_LTE_UPLINK_SUBFRAME_ALLOCATION_PUCCH_FORMAT_FSV                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 4827
#define RSSPECAN_ATTR_LTE_UPLINK_SUBFRAME_ALLOCATION_N_PUCCH_FSV                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 4828
#define RSSPECAN_ATTR_LTE_UPLINK_SUBFRAME_ALLOCATION_ENABLE_PUCCH                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 5821
#define RSSPECAN_ATTR_LTE_UPLINK_SUBFRAME_ALLOCATION_MODULATION                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 5822
#define RSSPECAN_ATTR_LTE_UPLINK_SUBFRAME_ALLOCATION_RESOURCE_ALLOCATION_TYPE_1                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 5823
#define RSSPECAN_ATTR_LTE_UPLINK_SUBFRAME_ALLOCATION_PRECODING_CODEWORD_TO_LAYER_MAPPING                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 5824
#define RSSPECAN_ATTR_LTE_UPLINK_SUBFRAME_ALLOCATION_PRECODING_CODEBOOK_INDEX                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 5825
#define RSSPECAN_ATTR_LTE_UPLINK_SUBFRAME_ALLOCATION_N_DMRS                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 5826
#define RSSPECAN_ATTR_LTE_UPLINK_SUBFRAME_ALLOCATION_CYCLIC_SHIFT_FIELD                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 5827
#define RSSPECAN_ATTR_LTE_UPLINK_SUBFRAME_ALLOCATION_PUCCH_FORMAT                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 5828
#define RSSPECAN_ATTR_LTE_UPLINK_SUBFRAME_ALLOCATION_N_PUCCH                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 5829
#define RSSPECAN_ATTR_LTE_UPLINK_SUBFRAME_ALLOCATION_RESOURCE_BLOCKS                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 5830
#define RSSPECAN_ATTR_LTE_UPLINK_SUBFRAME_ALLOCATION_RESOURCE_BLOCKS_OFFSET                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 5831
#define RSSPECAN_ATTR_LTE_UPLINK_FRAME_NUMBER_OFFSET_FSV                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 3691
#define RSSPECAN_ATTR_LTE_UPLINK_UE_RADIO_NETWORK_TEMPORARY_IDENTIFIER_FSV                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 3692
#define RSSPECAN_ATTR_LTE_UPLINK_POWER_PUSCH_FSV                                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 2501
#define RSSPECAN_ATTR_LTE_UPLINK_GROUP_HOPPING_FSV                                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 2503
#define RSSPECAN_ATTR_LTE_UPLINK_SEQUENCE_HOPPING_FSV                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 2504
#define RSSPECAN_ATTR_LTE_UPLINK_POWER_PUCCH_FSV                                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 2502
#define RSSPECAN_ATTR_LTE_UPLINK_N_DMRS_FSV                                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 2506
#define RSSPECAN_ATTR_LTE_UPLINK_DELTA_SEQUENCE_SHIFT_FSV                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 2505
#define RSSPECAN_ATTR_LTE_UPLINK_ACTIVATE_DMRS_WITH_OCC_FSV                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 3693
#define RSSPECAN_ATTR_LTE_UPLINK_FRAME_NUMBER_OFFSET                                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 6360
#define RSSPECAN_ATTR_LTE_UPLINK_UE_RADIO_NETWORK_TEMPORARY_IDENTIFIER                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 6361
#define RSSPECAN_ATTR_LTE_UPLINK_POWER_PUSCH                                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 5832
#define RSSPECAN_ATTR_LTE_UPLINK_GROUP_HOPPING                                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 5833
#define RSSPECAN_ATTR_LTE_UPLINK_SEQUENCE_HOPPING                                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 5834
#define RSSPECAN_ATTR_LTE_UPLINK_POWER_PUCCH                                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 5835
#define RSSPECAN_ATTR_LTE_UPLINK_N_DMRS                                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 5836
#define RSSPECAN_ATTR_LTE_UPLINK_DELTA_SEQUENCE_SHIFT                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 5837
#define RSSPECAN_ATTR_LTE_UPLINK_ACTIVATE_DMRS_WITH_OCC                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 5838
#define RSSPECAN_ATTR_LTE_UPLINK_DRS_CELL_ID                                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 6362
#define RSSPECAN_ATTR_LTE_UPLINK_DRS_CELL_ID_VALUE                                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 6363
#define RSSPECAN_ATTR_LTE_UPLINK_SOUNDING_REFERENCE_SIGNAL_PRESENT_FSV                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 2508
#define RSSPECAN_ATTR_LTE_UPLINK_SOUNDING_REFERENCE_SIGNAL_SUBFRAME_CONFIGURATION_FSV                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 3694
#define RSSPECAN_ATTR_LTE_UPLINK_SOUNDING_REFERENCE_SIGNAL_MAXUPPTS_FSV                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 3695
#define RSSPECAN_ATTR_LTE_UPLINK_SOUNDING_REFERENCE_SIGNAL_BANDWIDTH_B_SRS_FSV                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 4829
#define RSSPECAN_ATTR_LTE_UPLINK_SOUNDING_REFERENCE_SIGNAL_HOPPING_BW_FSV                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 3696
#define RSSPECAN_ATTR_LTE_UPLINK_SOUNDING_REFERENCE_SIGNAL_CYCLIC_SHIFT_N_CS_FSV                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 3697
#define RSSPECAN_ATTR_LTE_UPLINK_SOUNDING_REFERENCE_SIGNAL_POWER_FSV                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 2509
#define RSSPECAN_ATTR_LTE_UPLINK_SOUNDING_REFERENCE_SIGNAL_BANDWIDTH_CONFIGURATION_C_SRS_FSV                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 4830
#define RSSPECAN_ATTR_LTE_UPLINK_SOUNDING_REFERENCE_SIGNAL_CONFIGURATION_INDEX_FSV                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 3698
#define RSSPECAN_ATTR_LTE_UPLINK_SOUNDING_REFERENCE_SIGNAL_TRANSMISSION_COMB_FSV                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 3699
#define RSSPECAN_ATTR_LTE_UPLINK_SOUNDING_REFERENCE_SIGNAL_N_RRC_FSV                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 2792
#define RSSPECAN_ATTR_LTE_UPLINK_SOUNDING_REFERENCE_SIGNAL_AN_TX_FSV                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 3135
#define RSSPECAN_ATTR_LTE_UPLINK_SOUNDING_REFERENCE_SIGNAL_PRESENT                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 6364
#define RSSPECAN_ATTR_LTE_UPLINK_SOUNDING_REFERENCE_SIGNAL_SUBFRAME_CONFIGURATION                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 6365
#define RSSPECAN_ATTR_LTE_UPLINK_SOUNDING_REFERENCE_SIGNAL_MAXUPPTS                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 6366
#define RSSPECAN_ATTR_LTE_UPLINK_SOUNDING_REFERENCE_SIGNAL_BANDWIDTH_B_SRS                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 6367
#define RSSPECAN_ATTR_LTE_UPLINK_SOUNDING_REFERENCE_SIGNAL_HOPPING_BW                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 6368
#define RSSPECAN_ATTR_LTE_UPLINK_SOUNDING_REFERENCE_SIGNAL_CYCLIC_SHIFT_N_CS                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 6369
#define RSSPECAN_ATTR_LTE_UPLINK_SOUNDING_REFERENCE_SIGNAL_POWER                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 6370
#define RSSPECAN_ATTR_LTE_UPLINK_SOUNDING_REFERENCE_SIGNAL_BANDWIDTH_CONFIGURATION_C_SRS                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 6371
#define RSSPECAN_ATTR_LTE_UPLINK_SOUNDING_REFERENCE_SIGNAL_CONFIGURATION_INDEX                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 6372
#define RSSPECAN_ATTR_LTE_UPLINK_SOUNDING_REFERENCE_SIGNAL_TRANSMISSION_COMB                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 6373
#define RSSPECAN_ATTR_LTE_UPLINK_SOUNDING_REFERENCE_SIGNAL_AN_TX                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 6374
#define RSSPECAN_ATTR_LTE_UPLINK_SOUNDING_REFERENCE_SIGNAL_N_RRC                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 6375
#define RSSPECAN_ATTR_LTE_UPLINK_PUSCH_FREQUENCY_HOPPING_MODE_FSV                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 2517
#define RSSPECAN_ATTR_LTE_UPLINK_PUSCH_SUBBANDS_FSV                                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 2518
#define RSSPECAN_ATTR_LTE_UPLINK_PUSCH_HOPPING_OFFSET_FSV                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 3083
#define RSSPECAN_ATTR_LTE_UPLINK_PUSCH_INFO_IN_HOPPING_BITS_FSV                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 3084
#define RSSPECAN_ATTR_LTE_UPLINK_PUSCH_FREQUENCY_HOPPING_MODE                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 5839
#define RSSPECAN_ATTR_LTE_UPLINK_PUSCH_SUBBANDS                                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 5840
#define RSSPECAN_ATTR_LTE_UPLINK_PUSCH_HOPPING_OFFSET                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 5841
#define RSSPECAN_ATTR_LTE_UPLINK_PUSCH_INFO_IN_HOPPING_BITS                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 5842
#define RSSPECAN_ATTR_LTE_UPLINK_PUSCH_CELL_ID                                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 7383
#define RSSPECAN_ATTR_LTE_UPLINK_PUSCH_CELL_ID_VALUE                                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 7384
#define RSSPECAN_ATTR_LTE_UPLINK_PUCCH_RESOURCE_BLOCKS_FSV                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 2519
#define RSSPECAN_ATTR_LTE_UPLINK_PUCCH_RESOURCE_BLOCKS_AUTO_FSV                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 5177
#define RSSPECAN_ATTR_LTE_UPLINK_PUCCH_CYCLIC_SHIFTS_FSV                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 2522
#define RSSPECAN_ATTR_LTE_UPLINK_PUCCH_DELTA_SHIFT_FSV                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 2520
#define RSSPECAN_ATTR_LTE_UPLINK_PUCCH_FORMAT_FSV                                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 2524
#define RSSPECAN_ATTR_LTE_UPLINK_PUCCH_BANDWIDTH_FSV                                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 2523
#define RSSPECAN_ATTR_LTE_UPLINK_PUCCH_RESOURCE_INDEX_FSV                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 2525
#define RSSPECAN_ATTR_LTE_UPLINK_PUCCH_DELTA_OFFSET                                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 2521
#define RSSPECAN_ATTR_LTE_UPLINK_PUCCH_RESOURCE_BLOCKS                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 6376
#define RSSPECAN_ATTR_LTE_UPLINK_PUCCH_RESOURCE_BLOCKS_AUTO                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 6377
#define RSSPECAN_ATTR_LTE_UPLINK_PUCCH_CYCLIC_SHIFTS                                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 6378
#define RSSPECAN_ATTR_LTE_UPLINK_PUCCH_DELTA_SHIFT                                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 6379
#define RSSPECAN_ATTR_LTE_UPLINK_PUCCH_FORMAT                                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 6380
#define RSSPECAN_ATTR_LTE_UPLINK_PUCCH_BANDWIDTH                                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 6381
#define RSSPECAN_ATTR_LTE_UPLINK_PUCCH_RESOURCE_INDEX                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 6382
#define RSSPECAN_ATTR_LTE_UPLINK_PUCCH_N_PUCCH_AUTO                                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 7385
#define RSSPECAN_ATTR_LTE_UPLINK_PUCCH_CELL_ID                                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 6383
#define RSSPECAN_ATTR_LTE_UPLINK_PUCCH_CELL_ID_VALUE                                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 6384
#define RSSPECAN_ATTR_LTE_UPLINK_PRACH_CONFIGURATION_FSV                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 3085
#define RSSPECAN_ATTR_LTE_UPLINK_PRACH_RESTRICTED_SET_FSV                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 3086
#define RSSPECAN_ATTR_LTE_UPLINK_PRACH_FREQUENCY_OFFSET_FSV                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 3087
#define RSSPECAN_ATTR_LTE_UPLINK_PRACH_NCS_CONFIGURATION_FSV                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 3088
#define RSSPECAN_ATTR_LTE_UPLINK_PRACH_LOGICAL_ROOT_SEQ_INDEX_FSV                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 3089
#define RSSPECAN_ATTR_LTE_UPLINK_PRACH_SEQUENCE_INDEX_FSV                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 3090
#define RSSPECAN_ATTR_LTE_UPLINK_PRACH_SEQUENCE_INDEX_VALUE_FSV                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 3091
#define RSSPECAN_ATTR_LTE_UPLINK_PRACH_AUTOMATIC_PREAMBLE_MAPPING_FSV                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 3092
#define RSSPECAN_ATTR_LTE_UPLINK_PRACH_FREQUENCY_INDEX_FSV                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 3093
#define RSSPECAN_ATTR_LTE_UPLINK_PRACH_HALF_FRAME_INDICATOR_FSV                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 3094
#define RSSPECAN_ATTR_LTE_UPLINK_PRACH_CONFIGURATION                                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 6385
#define RSSPECAN_ATTR_LTE_UPLINK_PRACH_RESTRICTED_SET                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 6386
#define RSSPECAN_ATTR_LTE_UPLINK_PRACH_FREQUENCY_OFFSET                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 6387
#define RSSPECAN_ATTR_LTE_UPLINK_PRACH_NCS_CONFIGURATION                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 6388
#define RSSPECAN_ATTR_LTE_UPLINK_PRACH_LOGICAL_ROOT_SEQ_INDEX                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 6389
#define RSSPECAN_ATTR_LTE_UPLINK_PRACH_SEQUENCE_INDEX                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 6390
#define RSSPECAN_ATTR_LTE_UPLINK_PRACH_SEQUENCE_INDEX_VALUE                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 6391
#define RSSPECAN_ATTR_LTE_UPLINK_PRACH_AUTOMATIC_PREAMBLE_MAPPING                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 6392
#define RSSPECAN_ATTR_LTE_UPLINK_PRACH_FREQUENCY_INDEX                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 6393
#define RSSPECAN_ATTR_LTE_UPLINK_PRACH_HALF_FRAME_INDICATOR                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 6394
#define RSSPECAN_ATTR_LTE_UPLINK_REFERENCE_LEVEL                                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 2482
#define RSSPECAN_ATTR_LTE_UPLINK_AUTO_REFERENCE_LEVEL                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 2481
#define RSSPECAN_ATTR_LTE_UPLINK_REFERENCE_LEVEL_IN_BB_MODE                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 2483
#define RSSPECAN_ATTR_LTE_UPLINK_AUTO_LEVEL_TRACK_TIME                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 2486
#define RSSPECAN_ATTR_LTE_UPLINK_ANALYZE_SINGLE_SUBFRAME                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 6740
#define RSSPECAN_ATTR_LTE_UPLINK_TRIGGER_IF_POWER_LEVEL                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 2578
#define RSSPECAN_ATTR_LTE_UPLINK_TRIGGER_POWER_LEVEL_AUTO                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 2736
#define RSSPECAN_ATTR_LTE_UPLINK_TRIGGER_USE_SAME_CONFIGURATION_AS_MASTER_ENABLED                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 7386
#define RSSPECAN_ATTR_LTE_UPLINK_PHASE_TRACKING                                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 2490
#define RSSPECAN_ATTR_LTE_UPLINK_TIMING                                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 2491
#define RSSPECAN_ATTR_LTE_UPLINK_ANALYSIS_MODE                                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 3700
#define RSSPECAN_ATTR_LTE_UPLINK_CHANNEL_ESTIMATION                                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 2487
#define RSSPECAN_ATTR_LTE_UPLINK_EVM_WITH_EXCLUSION_PERIOD                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 3701
#define RSSPECAN_ATTR_LTE_UPLINK_TRANSIENT_SLOTS                                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 3154
#define RSSPECAN_ATTR_LTE_UPLINK_COMPENSATE_DC_OFFSET                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 7387
#define RSSPECAN_ATTR_LTE_UPLINK_SCRAMBLING_OF_CODED_BITS                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 2791
#define RSSPECAN_ATTR_LTE_UPLINK_SUPPRESSED_INTERFERENCE_SYNC                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 2737
#define RSSPECAN_ATTR_LTE_UPLINK_MULTICARRIER_FILTER                                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 3702
#define RSSPECAN_ATTR_LTE_UPLINK_AUTO_DEMODULATION                                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 2489
#define RSSPECAN_ATTR_LTE_UPLINK_LOCAL_OSCILLATOR_LOCATION                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 6395
#define RSSPECAN_ATTR_LTE_UPLINK_LOCAL_OSCILLATOR_FREQUENCY                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 7388
#define RSSPECAN_ATTR_LTE_UPLINK_CARRIER_AGGREGATION                                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 5843
#define RSSPECAN_ATTR_LTE_UPLINK_NUMBER_OF_DEVICES                                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 5845
#define RSSPECAN_ATTR_LTE_UPLINK_ACLR_ASSUMED_ADJ_CHANNEL_CARRIER                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 2532
#define RSSPECAN_ATTR_LTE_UPLINK_SEM_REQUIREMENT                                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 3703
#define RSSPECAN_ATTR_LTE_UPLINK_CARRIERS_BANDWIDTHS                                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 5178
#define RSSPECAN_ATTR_LTE_UPLINK_CARRIER_FREQUENCY_CENTER                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 5179
#define RSSPECAN_ATTR_LTE_UPLINK_CARRIER_FREQUENCY_CENTER_OFFSET                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 5180
#define RSSPECAN_ATTR_LTE_UPLINK_TAB_SELECT                                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 8433
#define RSSPECAN_ATTR_LTE_UPLINK_MEASUREMENT                                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 4839
#define RSSPECAN_ATTR_LTE_UPLINK_MEASUREMENT_SIGNAL_DETECTION                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 6436
#define RSSPECAN_ATTR_LTE_UPLINK_PREAMBLE_SELECTION_FSV                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 3704
#define RSSPECAN_ATTR_LTE_UPLINK_PREAMBLE_SELECTION_ALL_FSV                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 3705
#define RSSPECAN_ATTR_LTE_UPLINK_CONSTELLATION_ALLOCATION_FSV                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 4833
#define RSSPECAN_ATTR_LTE_UPLINK_CONSTELLATION_ALLOCATION_ALL_FSV                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 4834
#define RSSPECAN_ATTR_LTE_UPLINK_CONSTELLATION_CARRIER_FSV                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 4837
#define RSSPECAN_ATTR_LTE_UPLINK_CONSTELLATION_CARRIER_ALL_FSV                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 4838
#define RSSPECAN_ATTR_LTE_UPLINK_SLOT_SELECTION                                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 6396
#define RSSPECAN_ATTR_LTE_UPLINK_SUBFRAME_SELECTION                                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 6397
#define RSSPECAN_ATTR_LTE_UPLINK_SUBFRAME_SELECTION_ALL                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 6398
#define RSSPECAN_ATTR_LTE_UPLINK_PREAMBLE_SELECTION                                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 6399
#define RSSPECAN_ATTR_LTE_UPLINK_PREAMBLE_SELECTION_ALL                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 6400
#define RSSPECAN_ATTR_LTE_UPLINK_CONSTELLATION_MODULATION                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 6401
#define RSSPECAN_ATTR_LTE_UPLINK_CONSTELLATION_MODULATION_ALL                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 6402
#define RSSPECAN_ATTR_LTE_UPLINK_CONSTELLATION_ALLOCATION                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 6403
#define RSSPECAN_ATTR_LTE_UPLINK_CONSTELLATION_ALLOCATION_ALL                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 6404
#define RSSPECAN_ATTR_LTE_UPLINK_CONSTELLATION_SYMBOL                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 6405
#define RSSPECAN_ATTR_LTE_UPLINK_CONSTELLATION_SYMBOL_ALL                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 6406
#define RSSPECAN_ATTR_LTE_UPLINK_CONSTELLATION_CARRIER                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 6407
#define RSSPECAN_ATTR_LTE_UPLINK_CONSTELLATION_CARRIER_ALL                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 6408
#define RSSPECAN_ATTR_LTE_UPLINK_Y_AXIS_SCALING_AUTO                                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 2942
#define RSSPECAN_ATTR_LTE_UPLINK_YAXIS_SCALING_AUTO_ONCE                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 3706
#define RSSPECAN_ATTR_LTE_UPLINK_Y_AXIS_SCALING_PER_DIVISION                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 2943
#define RSSPECAN_ATTR_LTE_UPLINK_Y_AXIS_SCALING_OFFSET                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 2944
#define RSSPECAN_ATTR_LTE_UPLINK_YAXIS_SCALING_MINIMUM                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 3707
#define RSSPECAN_ATTR_LTE_UPLINK_YAXIS_SCALING_MAXIMUM                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 3708
#define RSSPECAN_ATTR_LTE_UPLINK_SUBWINDOW_Y_AXIS_SCALING_AUTO                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 7389
#define RSSPECAN_ATTR_LTE_UPLINK_SUBWINDOW_Y_AXIS_SCALING_RESTORE                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 7390
#define RSSPECAN_ATTR_LTE_UPLINK_SUBWINDOW_Y_AXIS_SCALING_MAXIMUM                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 7391
#define RSSPECAN_ATTR_LTE_UPLINK_SUBWINDOW_Y_AXIS_SCALING_MINIMUM                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 7392
#define RSSPECAN_ATTR_LTE_UPLINK_MARKER_COUPLING                                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 5181
#define RSSPECAN_ATTR_LTE_UPLINK_EVM_UNITS                                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 2529
#define RSSPECAN_ATTR_LTE_UPLINK_BIT_STREAM_FORMAT                                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 2530
#define RSSPECAN_ATTR_LTE_UPLINK_CARRIER_AXES                                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 4840
#define RSSPECAN_ATTR_LTE_UPLINK_SUBWINDOW_COUPLING                                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 7393
#define RSSPECAN_ATTR_LTE_UPLINK_MEASUREMENT_TYPE                                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 2526
#define RSSPECAN_ATTR_LTE_UPLINK_EVM_DMRS_PUSCH_QPSK_RESULT_FSV                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 2740
#define RSSPECAN_ATTR_LTE_UPLINK_EVM_DMRS_PUSCH_QPSK_LIMIT_CHECK_RESULT_FSV                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 3709
#define RSSPECAN_ATTR_LTE_UPLINK_EVM_DMRS_PUSCH_64QAM_RESULT_FSV                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 3710
#define RSSPECAN_ATTR_LTE_UPLINK_EVM_DMRS_PUSCH_64QAM_LIMIT_CHECK_RESULT_FSV                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 3711
#define RSSPECAN_ATTR_LTE_UPLINK_EVM_DMRS_PUSCH_16QAM_RESULT_FSV                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 2741
#define RSSPECAN_ATTR_LTE_UPLINK_EVM_DMRS_PUSCH_16QAM_LIMIT_CHECK_RESULT_FSV                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 3712
#define RSSPECAN_ATTR_LTE_UPLINK_EVM_DMRS_PUCCH_RESULT_FSV                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 3713
#define RSSPECAN_ATTR_LTE_UPLINK_EVM_DMRS_PUCCH_LIMIT_CHECK_RESULT_FSV                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 3714
#define RSSPECAN_ATTR_LTE_UPLINK_EVM_PUCCH_RESULT_FSV                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 2742
#define RSSPECAN_ATTR_LTE_UPLINK_EVM_PUCCH_LIMIT_CHECK_RESULT_FSV                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 3715
#define RSSPECAN_ATTR_LTE_UPLINK_EVM_PRACH_RESULT_FSV                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 2743
#define RSSPECAN_ATTR_LTE_UPLINK_EVM_PRACH_LIMIT_CHECK_RESULT_FSV                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 3716
#define RSSPECAN_ATTR_LTE_UPLINK_EVM_PUSCH_QPSK_RESULT_FSV                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 2537
#define RSSPECAN_ATTR_LTE_UPLINK_EVM_PUSCH_QPSK_LIMIT_CHECK_RESULT_FSV                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 3717
#define RSSPECAN_ATTR_LTE_UPLINK_EVM_PUSCH_64QAM_RESULT_FSV                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 2539
#define RSSPECAN_ATTR_LTE_UPLINK_EVM_PUSCH_64QAM_LIMIT_CHECK_RESULT_FSV                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 3718
#define RSSPECAN_ATTR_LTE_UPLINK_EVM_PUSCH_16QAM_RESULT_FSV                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 2538
#define RSSPECAN_ATTR_LTE_UPLINK_EVM_PUSCH_16QAM_LIMIT_CHECK_RESULT_FSV                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 3719
#define RSSPECAN_ATTR_LTE_UPLINK_EVM_PUSCH_256QAM_LIMIT_CHECK_RESULT_FSV                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 7394
#define RSSPECAN_ATTR_LTE_UPLINK_EVM_DMRS_PUSCH_QPSK_RESULT                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 4841
#define RSSPECAN_ATTR_LTE_UPLINK_EVM_DMRS_PUSCH_64QAM_RESULT                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 4842
#define RSSPECAN_ATTR_LTE_UPLINK_EVM_DMRS_PUSCH_16QAM_RESULT                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 4843
#define RSSPECAN_ATTR_LTE_UPLINK_EVM_DMRS_PUSCH_256QAM_RESULT                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 6741
#define RSSPECAN_ATTR_LTE_UPLINK_EVM_DMRS_PUCCH_RESULT                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 4844
#define RSSPECAN_ATTR_LTE_UPLINK_EVM_PUCCH_RESULT                                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 4845
#define RSSPECAN_ATTR_LTE_UPLINK_EVM_PRACH_RESULT                                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 4846
#define RSSPECAN_ATTR_LTE_UPLINK_EVM_PUSCH_QPSK_RESULT                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 4847
#define RSSPECAN_ATTR_LTE_UPLINK_EVM_PUSCH_64QAM_RESULT                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 4848
#define RSSPECAN_ATTR_LTE_UPLINK_EVM_PUSCH_16QAM_RESULT                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 4849
#define RSSPECAN_ATTR_LTE_UPLINK_EVM_PUSCH_256QAM_RESULT                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 6742
#define RSSPECAN_ATTR_LTE_UPLINK_EVM_DMRS_PUSCH_QPSK_LIMIT_CHECK_RESULT                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 6409
#define RSSPECAN_ATTR_LTE_UPLINK_EVM_DMRS_PUSCH_64QAM_LIMIT_CHECK_RESULT                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 6410
#define RSSPECAN_ATTR_LTE_UPLINK_EVM_DMRS_PUCCH_LIMIT_CHECK_RESULT                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 6411
#define RSSPECAN_ATTR_LTE_UPLINK_EVM_DMRS_PUSCH_16QAM_LIMIT_CHECK_RESULT                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 6412
#define RSSPECAN_ATTR_LTE_UPLINK_EVM_DMRS_PUSCH_256QAM_LIMIT_CHECK_RESULT                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 6743
#define RSSPECAN_ATTR_LTE_UPLINK_EVM_PUCCH_LIMIT_CHECK_RESULT                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 6413
#define RSSPECAN_ATTR_LTE_UPLINK_EVM_PRACH_LIMIT_CHECK_RESULT                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 6414
#define RSSPECAN_ATTR_LTE_UPLINK_EVM_PUSCH_QPSK_LIMIT_CHECK_RESULT                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 6415
#define RSSPECAN_ATTR_LTE_UPLINK_EVM_PUSCH_64QAM_LIMIT_CHECK_RESULT                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 6416
#define RSSPECAN_ATTR_LTE_UPLINK_EVM_PUSCH_16QAM_LIMIT_CHECK_RESULT                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 6417
#define RSSPECAN_ATTR_LTE_UPLINK_EVM_PUSCH_256QAM_LIMIT_CHECK_RESULT                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 6744
#define RSSPECAN_ATTR_LTE_UPLINK_CREST_FACTOR_RESULT_FSV                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 2549
#define RSSPECAN_ATTR_LTE_UPLINK_TRIGGER_TO_FRAME_RESULT                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 2550
#define RSSPECAN_ATTR_LTE_UPLINK_EVM_ALL_RESULT_MIN_FSV                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 3720
#define RSSPECAN_ATTR_LTE_UPLINK_EVM_ALL_RESULT_AVERAGE_FSV                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 3721
#define RSSPECAN_ATTR_LTE_UPLINK_EVM_ALL_RESULT_MAX_FSV                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 3722
#define RSSPECAN_ATTR_LTE_UPLINK_EVM_AL_LIMIT_CHECK_RESULT_AVERAGE_FSV                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 3723
#define RSSPECAN_ATTR_LTE_UPLINK_EVM_AL_LIMIT_CHECK_RESULT_MAX_FSV                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 3724
#define RSSPECAN_ATTR_LTE_UPLINK_EVM_ALL_RESULT_MIN                                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 5848
#define RSSPECAN_ATTR_LTE_UPLINK_EVM_ALL_RESULT_AVERAGE                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 5849
#define RSSPECAN_ATTR_LTE_UPLINK_EVM_ALL_RESULT_MAX                                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 5850
#define RSSPECAN_ATTR_LTE_UPLINK_EVM_AL_LIMIT_CHECK_RESULT_AVERAGE                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 6418
#define RSSPECAN_ATTR_LTE_UPLINK_EVM_AL_LIMIT_CHECK_RESULT_MAX                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 6419
#define RSSPECAN_ATTR_LTE_UPLINK_EVM_PHYSICAL_CHANNEL_RESULT_MIN_FSV                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 3725
#define RSSPECAN_ATTR_LTE_UPLINK_EVM_PHYSICAL_CHANNEL_RESULT_AVERAGE_FSV                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 3726
#define RSSPECAN_ATTR_LTE_UPLINK_EVM_PHYSICAL_CHANNEL_RESULT_MAX_FSV                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 3727
#define RSSPECAN_ATTR_LTE_UPLINK_EVM_PHYSICAL_CHANNEL_LIMIT_CHECK_RESULT_AVERAGE_FSV                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 3728
#define RSSPECAN_ATTR_LTE_UPLINK_EVM_PHYSICAL_CHANNEL_LIMIT_CHECK_RESULT_MAX_FSV                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 3729
#define RSSPECAN_ATTR_LTE_UPLINK_EVM_PHYSICAL_CHANNEL_RESULT_MIN                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 5851
#define RSSPECAN_ATTR_LTE_UPLINK_EVM_PHYSICAL_CHANNEL_RESULT_AVERAGE                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 5852
#define RSSPECAN_ATTR_LTE_UPLINK_EVM_PHYSICAL_CHANNEL_RESULT_MAX                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 5853
#define RSSPECAN_ATTR_LTE_UPLINK_EVM_PHYSICAL_CHANNEL_LIMIT_CHECK_RESULT_AVERAGE                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 6420
#define RSSPECAN_ATTR_LTE_UPLINK_EVM_PHYSICAL_CHANNEL_LIMIT_CHECK_RESULT_MAX                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 6421
#define RSSPECAN_ATTR_LTE_UPLINK_EVM_PHYSICAL_SIGNAL_RESULT_MIN_FSV                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 3730
#define RSSPECAN_ATTR_LTE_UPLINK_EVM_PHYSICAL_SIGNAL_RESULT_AVERAGE_FSV                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 3731
#define RSSPECAN_ATTR_LTE_UPLINK_EVM_PHYSICAL_SIGNAL_RESULT_MAX_FSV                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 3732
#define RSSPECAN_ATTR_LTE_UPLINK_EVM_PHYSICAL_SIGNAL_LIMIT_CHECK_RESULT_AVERAGE_FSV                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 3733
#define RSSPECAN_ATTR_LTE_UPLINK_EVM_PHYSICAL_SIGNAL_LIMIT_CHECK_RESULT_MAX_FSV                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 3734
#define RSSPECAN_ATTR_LTE_UPLINK_EVM_PHYSICAL_SIGNAL_RESULT_MIN                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 5854
#define RSSPECAN_ATTR_LTE_UPLINK_EVM_PHYSICAL_SIGNAL_RESULT_AVERAGE                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 5855
#define RSSPECAN_ATTR_LTE_UPLINK_EVM_PHYSICAL_SIGNAL_RESULT_MAX                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 5856
#define RSSPECAN_ATTR_LTE_UPLINK_EVM_PHYSICAL_SIGNAL_LIMIT_CHECK_RESULT_AVERAGE                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 6422
#define RSSPECAN_ATTR_LTE_UPLINK_EVM_PHYSICAL_SIGNAL_LIMIT_CHECK_RESULT_MAX                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 6423
#define RSSPECAN_ATTR_LTE_UPLINK_FREQUENCY_ERROR_RESULT_MIN_FSV                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 3735
#define RSSPECAN_ATTR_LTE_UPLINK_FREQUENCY_ERROR_RESULT_AVERAGE_FSV                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 3736
#define RSSPECAN_ATTR_LTE_UPLINK_FREQUENCY_ERROR_RESULT_MAX_FSV                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 3737
#define RSSPECAN_ATTR_LTE_UPLINK_FREQUENCY_ERROR_LIMIT_CHECK_RESULT_AVERAGE_FSV                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 3738
#define RSSPECAN_ATTR_LTE_UPLINK_FREQUENCY_ERROR_LIMIT_CHECK_RESULT_MAX_FSV                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 3739
#define RSSPECAN_ATTR_LTE_UPLINK_FREQUENCY_ERROR_RESULT_MIN                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 5857
#define RSSPECAN_ATTR_LTE_UPLINK_FREQUENCY_ERROR_RESULT_AVERAGE                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 5858
#define RSSPECAN_ATTR_LTE_UPLINK_FREQUENCY_ERROR_RESULT_MAX                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 5859
#define RSSPECAN_ATTR_LTE_UPLINK_FREQUENCY_ERROR_LIMIT_CHECK_RESULT_AVERAGE                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 6424
#define RSSPECAN_ATTR_LTE_UPLINK_FREQUENCY_ERROR_LIMIT_CHECK_RESULT_MAX                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 6425
#define RSSPECAN_ATTR_LTE_UPLINK_IQ_GAIN_IMBALANCE_RESULT_MIN_FSV                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 3740
#define RSSPECAN_ATTR_LTE_UPLINK_IQ_GAIN_IMBALANCE_RESULT_AVERAGE_FSV                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 3741
#define RSSPECAN_ATTR_LTE_UPLINK_IQ_GAIN_IMBALANCE_RESULT_MAX_FSV                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 3742
#define RSSPECAN_ATTR_LTE_UPLINK_IQ_GAIN_IMBALANCE_LIMIT_CHECK_RESULT_AVERAGE_FSV                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 3743
#define RSSPECAN_ATTR_LTE_UPLINK_IQ_GAIN_IMBALANCE_LIMIT_CHECK_RESULT_MAX_FSV                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 5860
#define RSSPECAN_ATTR_LTE_UPLINK_IQ_GAIN_IMBALANCE_RESULT_MIN                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 5861
#define RSSPECAN_ATTR_LTE_UPLINK_IQ_GAIN_IMBALANCE_RESULT_AVERAGE                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 5862
#define RSSPECAN_ATTR_LTE_UPLINK_IQ_GAIN_IMBALANCE_RESULT_MAX                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 5863
#define RSSPECAN_ATTR_LTE_UPLINK_IQ_GAIN_IMBALANCE_LIMIT_CHECK_RESULT_AVERAGE                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 6426
#define RSSPECAN_ATTR_LTE_UPLINK_IQ_GAIN_IMBALANCE_LIMIT_CHECK_RESULT_MAX                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 6427
#define RSSPECAN_ATTR_LTE_UPLINK_IQ_OFFSET_RESULT_MIN_FSV                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 3744
#define RSSPECAN_ATTR_LTE_UPLINK_IQ_OFFSET_RESULT_AVERAGE_FSV                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 3745
#define RSSPECAN_ATTR_LTE_UPLINK_IQ_OFFSET_RESULT_MAX_FSV                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 3746
#define RSSPECAN_ATTR_LTE_UPLINK_IQ_OFFSET_LIMIT_CHECK_RESULT_AVERAGE_FSV                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 3747
#define RSSPECAN_ATTR_LTE_UPLINK_IQ_OFFSET_LIMIT_CHECK_RESULT_MAX_FSV                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 5864
#define RSSPECAN_ATTR_LTE_UPLINK_IQ_OFFSET_RESULT_MIN                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 5865
#define RSSPECAN_ATTR_LTE_UPLINK_IQ_OFFSET_RESULT_AVERAGE                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 5866
#define RSSPECAN_ATTR_LTE_UPLINK_IQ_OFFSET_RESULT_MAX                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 5867
#define RSSPECAN_ATTR_LTE_UPLINK_IQ_OFFSET_LIMIT_CHECK_RESULT_AVERAGE                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 6428
#define RSSPECAN_ATTR_LTE_UPLINK_IQ_OFFSET_LIMIT_CHECK_RESULT_MAX                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 6429
#define RSSPECAN_ATTR_LTE_UPLINK_FRAME_POWER_RESULT_MIN_FSV                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 3748
#define RSSPECAN_ATTR_LTE_UPLINK_FRAME_POWER_RESULT_AVERAGE_FSV                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 3749
#define RSSPECAN_ATTR_LTE_UPLINK_FRAME_POWER_RESULT_MAX_FSV                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 3750
#define RSSPECAN_ATTR_LTE_UPLINK_FRAME_POWER_RESULT_MIN                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 5868
#define RSSPECAN_ATTR_LTE_UPLINK_FRAME_POWER_RESULT_AVERAGE                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 5869
#define RSSPECAN_ATTR_LTE_UPLINK_FRAME_POWER_RESULT_MAX                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 5870
#define RSSPECAN_ATTR_LTE_UPLINK_IQ_QUADRATURE_ERROR_RESULT_MIN_FSV                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 3751
#define RSSPECAN_ATTR_LTE_UPLINK_IQ_QUADRATURE_ERROR_RESULT_AVERAGE_FSV                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 3752
#define RSSPECAN_ATTR_LTE_UPLINK_IQ_QUADRATURE_ERROR_RESULT_MAX_FSV                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 3753
#define RSSPECAN_ATTR_LTE_UPLINK_IQ_QUADRATURE_ERROR_LIMIT_CHECK_RESULT_AVERAGE_FSV                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 3754
#define RSSPECAN_ATTR_LTE_UPLINK_IQ_QUADRATURE_ERROR_LIMIT_CHECK_RESULT_MAX_FSV                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 3755
#define RSSPECAN_ATTR_LTE_UPLINK_IQ_QUADRATURE_ERROR_RESULT_MIN                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 5871
#define RSSPECAN_ATTR_LTE_UPLINK_IQ_QUADRATURE_ERROR_RESULT_AVERAGE                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 5872
#define RSSPECAN_ATTR_LTE_UPLINK_IQ_QUADRATURE_ERROR_RESULT_MAX                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 5873
#define RSSPECAN_ATTR_LTE_UPLINK_IQ_QUADRATURE_ERROR_LIMIT_CHECK_RESULT_AVERAGE                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 6430
#define RSSPECAN_ATTR_LTE_UPLINK_IQ_QUADRATURE_ERROR_LIMIT_CHECK_RESULT_MAX                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 6431
#define RSSPECAN_ATTR_LTE_UPLINK_SAMPLING_ERROR_RESULT_MIN_FSV                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 3756
#define RSSPECAN_ATTR_LTE_UPLINK_SAMPLING_ERROR_RESULT_AVERAGE_FSV                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 3757
#define RSSPECAN_ATTR_LTE_UPLINK_SAMPLING_ERROR_RESULT_MAX_FSV                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 3758
#define RSSPECAN_ATTR_LTE_UPLINK_SAMPLING_ERROR_LIMIT_CHECK_RESULT_AVERAGE_FSV                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 3759
#define RSSPECAN_ATTR_LTE_UPLINK_SAMPLING_ERROR_LIMIT_CHECK_RESULT_MAX_FSV                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 3760
#define RSSPECAN_ATTR_LTE_UPLINK_SAMPLING_ERROR_RESULT_MIN                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 5874
#define RSSPECAN_ATTR_LTE_UPLINK_SAMPLING_ERROR_RESULT_AVERAGE                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 5875
#define RSSPECAN_ATTR_LTE_UPLINK_SAMPLING_ERROR_RESULT_MAX                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 5876
#define RSSPECAN_ATTR_LTE_UPLINK_SAMPLING_ERROR_LIMIT_CHECK_RESULT_AVERAGE                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 6432
#define RSSPECAN_ATTR_LTE_UPLINK_SAMPLING_ERROR_LIMIT_CHECK_RESULT_MAX                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 6433
#define RSSPECAN_ATTR_LTE_UPLINK_CREST_FACTOR_RESULT                                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 5877
#define RSSPECAN_ATTR_LTE_UPLINK_DETECTED_CYCLIC_PREFIX                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 8434
#define RSSPECAN_ATTR_LTE_UPLINK_TRIGGER_TO_FRAME_RESULT_CARR                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 6603
#define RSSPECAN_ATTR_LTE_UPLINK_TIME_ALIGNMENT_ERROR_MIN                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 5878
#define RSSPECAN_ATTR_LTE_UPLINK_TIME_ALIGNMENT_ERROR_AVERAGE                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 5879
#define RSSPECAN_ATTR_LTE_UPLINK_TIME_ALIGNMENT_ERROR_MAX                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 5880
#define RSSPECAN_ATTR_LTE_UPLINK_CARRIER_FREQUENCY_ERROR_HZ_MIN                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 5881
#define RSSPECAN_ATTR_LTE_UPLINK_CARRIER_FREQUENCY_ERROR_HZ_AVERAGE                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 5882
#define RSSPECAN_ATTR_LTE_UPLINK_CARRIER_FREQUENCY_ERROR_HZ_MAX                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 5883
#define RSSPECAN_ATTR_LTE_UPLINK_CARRIER_FREQUENCY_ERROR_PPM_MIN                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 5884
#define RSSPECAN_ATTR_LTE_UPLINK_CARRIER_FREQUENCY_ERROR_PPM_AVERAGE                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 5885
#define RSSPECAN_ATTR_LTE_UPLINK_CARRIER_FREQUENCY_ERROR_PPM_MAX                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 5886
#define RSSPECAN_ATTR_LTE_UPLINK_SEM_CHANNEL                                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 2531
#define RSSPECAN_ATTR_LTE_UPLINK_ACLR_NOISE_CORRECTION                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 2533
#define RSSPECAN_ATTR_LTE_UPLINK_MARKER_Z_AXIS                                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 6434
#define RSSPECAN_ATTR_LTE_UPLINK_SYNCHRONIZATION                                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 4947
#define RSSPECAN_ATTR_LTE_UPLINK_CC_SYNCHRONIZATION                                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 7395
#define RSSPECAN_ATTR_NB_IOT_DOWNLINK_MODE                                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 7426
#define RSSPECAN_ATTR_NB_IOT_DOWNLINK_PRESET                                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 7427
#define RSSPECAN_ATTR_NB_IOT_DOWNLINK_STORE_DEMODULATION_SETTINGS                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 11125
#define RSSPECAN_ATTR_NB_IOT_DOWNLINK_LOAD_DEMODULATION_SETTINGS                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 7428
#define RSSPECAN_ATTR_NB_IOT_DOWNLINK_LOAD_IQ                                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 7430
#define RSSPECAN_ATTR_NB_IOT_DOWNLINK_WINDOW_SELECT                                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 7432
#define RSSPECAN_ATTR_NB_IOT_DOWNLINK_TEST_MODEL                                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 11126
#define RSSPECAN_ATTR_NB_IOT_DOWNLINK_TRIGGER_LEVEL                                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 7433
#define RSSPECAN_ATTR_NB_IOT_DOWNLINK_TRIGGER_SOURCE                                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 7434
#define RSSPECAN_ATTR_NB_IOT_DOWNLINK_DEPLOYMENT                                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 7435
#define RSSPECAN_ATTR_NB_IOT_DOWNLINK_CARRIER_TYPE                                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 11127
#define RSSPECAN_ATTR_NB_IOT_DOWNLINK_EUTRA_FREQUENCY                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 7436
#define RSSPECAN_ATTR_NB_IOT_DOWNLINK_PRB_INDEX                                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 7437
#define RSSPECAN_ATTR_NB_IOT_DOWNLINK_CRS_SEQUENCE                                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 7438
#define RSSPECAN_ATTR_NB_IOT_DOWNLINK_BANDWIDTH                                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 7439
#define RSSPECAN_ATTR_NB_IOT_DOWNLINK_GUARDBAND_CARRIER_LOCATION                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 7749
#define RSSPECAN_ATTR_NB_IOT_DOWNLINK_GUARDBAND_CARRIER_CUSTOM_LOCATION                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 7750
#define RSSPECAN_ATTR_NB_IOT_DOWNLINK_GUARDBAND_CARRIER_CUSTOM_LOCATION_VALUE                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 7751
#define RSSPECAN_ATTR_NB_IOT_DOWNLINK_CELL_ID_AUTO                                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 7440
#define RSSPECAN_ATTR_NB_IOT_DOWNLINK_CELL_ID                                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 7441
#define RSSPECAN_ATTR_NB_IOT_DOWNLINK_CELL_ID_GROUP_AUTO                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 7442
#define RSSPECAN_ATTR_NB_IOT_DOWNLINK_CELL_ID_GROUP                                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 7443
#define RSSPECAN_ATTR_NB_IOT_DOWNLINK_PHYSICAL_LAYER_CELL_IDENTITY_AUTO                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 7444
#define RSSPECAN_ATTR_NB_IOT_DOWNLINK_PHYSICAL_LAYER_CELL_IDENTITY                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 7445
#define RSSPECAN_ATTR_NB_IOT_DOWNLINK_PHYSICAL_LAYER_CELL_GROUP                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 7446
#define RSSPECAN_ATTR_NB_IOT_DOWNLINK_PHYSICAL_LAYER_CELL_IDENTITY_DETECTED                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 7447
#define RSSPECAN_ATTR_NB_IOT_DOWNLINK_RF_CENTER_FREQUENCY                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 7448
#define RSSPECAN_ATTR_NB_IOT_DOWNLINK_MIMO_ANTENNA_SELECTION                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 7449
#define RSSPECAN_ATTR_NB_IOT_DOWNLINK_MIMO_CONFIG                                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 7450
#define RSSPECAN_ATTR_NB_IOT_DOWNLINK_CONTROL_CHANNEL_SYMBOL_OFFSET_AUTO                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 7451
#define RSSPECAN_ATTR_NB_IOT_DOWNLINK_CONTROL_CHANNEL_SYMBOL_OFFSET                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 7452
#define RSSPECAN_ATTR_NB_IOT_DOWNLINK_AUTO_REFERENCE_LEVEL                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 7453
#define RSSPECAN_ATTR_NB_IOT_DOWNLINK_AUTO_REFERENCE_LEVEL_ONCE                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 7454
#define RSSPECAN_ATTR_NB_IOT_DOWNLINK_REFERENCE_LEVEL_IN_BB_MODE                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 7455
#define RSSPECAN_ATTR_NB_IOT_DOWNLINK_REFERENCE_LEVEL_IN_RF_MODE                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 7456
#define RSSPECAN_ATTR_NB_IOT_DOWNLINK_TRACE_EXTERNAL_ATTENUATION                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 7457
#define RSSPECAN_ATTR_NB_IOT_DOWNLINK_YIG_FILTER_AUTO                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 7458
#define RSSPECAN_ATTR_NB_IOT_DOWNLINK_YIG_FILTER_ENABLED                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 7459
#define RSSPECAN_ATTR_NB_IOT_DOWNLINK_FILTER_TYPE                                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 7460
#define RSSPECAN_ATTR_NB_IOT_DOWNLINK_FRAME_COUNT                                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 7461
#define RSSPECAN_ATTR_NB_IOT_DOWNLINK_FRAME_COUNT_AUTO                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 7462
#define RSSPECAN_ATTR_NB_IOT_DOWNLINK_FRAME_COUNT_ENABLED                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 7463
#define RSSPECAN_ATTR_NB_IOT_DOWNLINK_SWAP_IQ                                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 7464
#define RSSPECAN_ATTR_NB_IOT_DOWNLINK_CAPTURE_LENGTH                                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 7465
#define RSSPECAN_ATTR_NB_IOT_DOWNLINK_AUTO_LEVEL_TRACK_TIME                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 7466
#define RSSPECAN_ATTR_NB_IOT_DOWNLINK_EVM_CALCULATION                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 7468
#define RSSPECAN_ATTR_NB_IOT_DOWNLINK_REFERENCE_DATA                                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 7469
#define RSSPECAN_ATTR_NB_IOT_DOWNLINK_MULTICARRIER_FILTER                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 7470
#define RSSPECAN_ATTR_NB_IOT_DOWNLINK_COMPENSATE_CROSSTALK                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 10496
#define RSSPECAN_ATTR_NB_IOT_DOWNLINK_SCRAMBLING_OF_CODED_BITS                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 11128
#define RSSPECAN_ATTR_NB_IOT_DOWNLINK_CHANNEL_ESTIMATION_TYPE                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 7471
#define RSSPECAN_ATTR_NB_IOT_DOWNLINK_PHASE_TRACKING_TYPE                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 7472
#define RSSPECAN_ATTR_NB_IOT_DOWNLINK_TRACKING_TIME                                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 7473
#define RSSPECAN_ATTR_NB_IOT_DOWNLINK_SEM_CATEGORY                                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 7752
#define RSSPECAN_ATTR_NB_IOT_DOWNLINK_SEM_TX_POWER_ENABLED                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 7753
#define RSSPECAN_ATTR_NB_IOT_DOWNLINK_SEM_TX_POWER                                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 7754
#define RSSPECAN_ATTR_NB_IOT_DOWNLINK_SEM_NB_IOT_CARRIER_POWER_MODE                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 7755
#define RSSPECAN_ATTR_NB_IOT_DOWNLINK_SEM_NB_IOT_CARRIER_POWER                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 7756
#define RSSPECAN_ATTR_NB_IOT_DOWNLINK_TAB_SELECT                                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 7757
#define RSSPECAN_ATTR_NB_IOT_DOWNLINK_NPDSCH_ENABLED                                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 11129
#define RSSPECAN_ATTR_NB_IOT_DOWNLINK_NPDSCH_SUBFRAMES                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 11130
#define RSSPECAN_ATTR_NB_IOT_DOWNLINK_NPDSCH_UE_ID                                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 11131
#define RSSPECAN_ATTR_NB_IOT_DOWNLINK_ANALYSIS_EVM_UNIT                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 7474
#define RSSPECAN_ATTR_NB_IOT_DOWNLINK_ANALYSIS_RESULT_TABLE_ENABLED                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 7475
#define RSSPECAN_ATTR_NB_IOT_DOWNLINK_ANALYSIS_SYNCHRONIZATION_STATE                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 7476
#define RSSPECAN_ATTR_NB_IOT_DOWNLINK_MEASUREMENT                                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 8575
#define RSSPECAN_ATTR_NB_IOT_DOWNLINK_BIT_STREAM_FORMAT                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 11132
#define RSSPECAN_ATTR_NB_IOT_DOWNLINK_ANALYSIS_ALLOCATION_SELECT_ALL                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 7477
#define RSSPECAN_ATTR_NB_IOT_DOWNLINK_ANALYSIS_ALLOCATION_SELECT                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 7478
#define RSSPECAN_ATTR_NB_IOT_DOWNLINK_ANALYSIS_CARRIER_SELECT_ALL                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 7479
#define RSSPECAN_ATTR_NB_IOT_DOWNLINK_ANALYSIS_CARRIER_SELECT                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 7480
#define RSSPECAN_ATTR_NB_IOT_DOWNLINK_ANALYSIS_MODULATION_SELECT_ALL                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 7481
#define RSSPECAN_ATTR_NB_IOT_DOWNLINK_ANALYSIS_MODULATION_SELECT                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 7482
#define RSSPECAN_ATTR_NB_IOT_DOWNLINK_ANALYSIS_SYMBOL_SELECT_ALL                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 7483
#define RSSPECAN_ATTR_NB_IOT_DOWNLINK_ANALYSIS_SYMBOL_SELECT                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 7484
#define RSSPECAN_ATTR_NB_IOT_DOWNLINK_ANALYSIS_LOCATION_SELECT                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 8654
#define RSSPECAN_ATTR_NB_IOT_DOWNLINK_ANALYSIS_SUBFRAME_SELECT_ALL                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 8655
#define RSSPECAN_ATTR_NB_IOT_DOWNLINK_ANALYSIS_SUBFRAME_SELECT                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 8656
#define RSSPECAN_ATTR_NB_IOT_DOWNLINK_ANALYSIS_MARKER_MINIMUM                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 7485
#define RSSPECAN_ATTR_NB_IOT_DOWNLINK_ANALYSIS_MARKER_MAXIMUM                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 7486
#define RSSPECAN_ATTR_NB_IOT_DOWNLINK_ANALYSIS_Y_AUTO                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 7487
#define RSSPECAN_ATTR_NB_IOT_DOWNLINK_ANALYSIS_Y_POINT_OF_ORIGIN                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 7488
#define RSSPECAN_ATTR_NB_IOT_DOWNLINK_ANALYSIS_Y_DISTANCE_BETWEEN_GRID                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 7489
#define RSSPECAN_ATTR_NB_IOT_DOWNLINK_SUBWINDOW_Y_AXIS_SCALING_AUTO                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 7758
#define RSSPECAN_ATTR_NB_IOT_DOWNLINK_SUBWINDOW_Y_AXIS_SCALING_RESTORE                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 7759
#define RSSPECAN_ATTR_NB_IOT_DOWNLINK_SUBWINDOW_Y_AXIS_SCALING_ALL_WINDOWS                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 7760
#define RSSPECAN_ATTR_NB_IOT_DOWNLINK_SUBWINDOW_Y_AXIS_SCALING_MAXIMUM                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 7761
#define RSSPECAN_ATTR_NB_IOT_DOWNLINK_SUBWINDOW_Y_AXIS_SCALING_MINIMUM                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 7762
#define RSSPECAN_ATTR_NB_IOT_CREST_FACTOR_RESULT_AVERAGE                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 7490
#define RSSPECAN_ATTR_NB_IOT_DOWNLINK_EVM_ALL_RESULT_MINIMUM                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 7491
#define RSSPECAN_ATTR_NB_IOT_DOWNLINK_EVM_ALL_RESULT_AVERAGE                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 7492
#define RSSPECAN_ATTR_NB_IOT_DOWNLINK_EVM_ALL_RESULT_MAXIMUM                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 7493
#define RSSPECAN_ATTR_NB_IOT_DOWNLINK_EVM_PHYSICAL_CHANNEL_RESULT_MIN                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 7494
#define RSSPECAN_ATTR_NB_IOT_DOWNLINK_EVM_PHYSICAL_CHANNEL_RESULT_AVERAGE                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 7495
#define RSSPECAN_ATTR_NB_IOT_DOWNLINK_EVM_PHYSICAL_CHANNEL_RESULT_MAXIMUM                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 7496
#define RSSPECAN_ATTR_NB_IOT_DOWNLINK_EVM_PHYSICAL_SIGNAL_RESULT_MIN                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 7497
#define RSSPECAN_ATTR_NB_IOT_DOWNLINK_EVM_PHYSICAL_SIGNAL_RESULT_AVERAGE                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 7498
#define RSSPECAN_ATTR_NB_IOT_DOWNLINK_EVM_PHYSICAL_SIGNAL_RESULT_MAX                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 7499
#define RSSPECAN_ATTR_NB_IOT_DOWNLINK_FREQUENCY_ERROR_RESULT_MIN                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 7500
#define RSSPECAN_ATTR_NB_IOT_DOWNLINK_FREQUENCY_ERROR_RESULT_AVERAGE                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 7501
#define RSSPECAN_ATTR_NB_IOT_DOWNLINK_FREQUENCY_ERROR_RESULT_MAX                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 7502
#define RSSPECAN_ATTR_NB_IOT_DOWNLINK_RSTP_RESULT_MIN                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 7503
#define RSSPECAN_ATTR_NB_IOT_DOWNLINK_RSTP_RESULT_AVERAGE                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 7504
#define RSSPECAN_ATTR_NB_IOT_DOWNLINK_RSTP_RESULT_MAX                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 7505
#define RSSPECAN_ATTR_NB_IOT_DOWNLINK_OSTP_RESULT_MIN                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 7506
#define RSSPECAN_ATTR_NB_IOT_DOWNLINK_OSTP_RESULT_AVERAGE                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 7507
#define RSSPECAN_ATTR_NB_IOT_DOWNLINK_OSTP_RESULT_MAX                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 7508
#define RSSPECAN_ATTR_NB_IOT_DOWNLINK_FRAME_POWER_RESULT_MIN                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 7509
#define RSSPECAN_ATTR_NB_IOT_DOWNLINK_FRAME_POWER_RESULT_AVERAGE                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 7510
#define RSSPECAN_ATTR_NB_IOT_DOWNLINK_FRAME_POWER_RESULT_MAX                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 7511
#define RSSPECAN_ATTR_NB_IOT_DOWNLINK_SAMPLING_ERROR_RESULT_MIN                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 7512
#define RSSPECAN_ATTR_NB_IOT_DOWNLINK_SAMPLING_ERROR_RESULT_AVERAGE                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 7513
#define RSSPECAN_ATTR_NB_IOT_DOWNLINK_SAMPLING_ERROR_RESULT_MAX                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 7514
#define RSSPECAN_ATTR_NB_IOT_DOWNLINK_TRIGGER_TO_FRAME_RESULT                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 7515
#define RSSPECAN_ATTR_NB_IOT_DOWNLINK_RESULT                                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 7516
#define RSSPECAN_ATTR_NOISE_MODE                                                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 1492
#define RSSPECAN_ATTR_NOISE_CONF_CORR                                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 1494
#define RSSPECAN_ATTR_NOISE_CONF_LIST_CONT                                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 1495
#define RSSPECAN_ATTR_NOISE_CONF_LIST_SING                                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 1496
#define RSSPECAN_ATTR_NOISE_CONF_SING                                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 1497
#define RSSPECAN_ATTR_NOISE_CONF_FREQ_CONT                                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 2426
#define RSSPECAN_ATTR_NOISE_CONF_FREQ_SING                                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 2427
#define RSSPECAN_ATTR_NOISE_SINGLE_FREQUENCY                                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 3079
#define RSSPECAN_ATTR_NOISE_SINGLE_FREQUENCY_COUPLING                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 3080
#define RSSPECAN_ATTR_NOISE_MEASUREMENT_POINTS                                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 10706
#define RSSPECAN_ATTR_NOISE_FREQ_STEP                                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 1507
#define RSSPECAN_ATTR_NOISE_FREQ_FIXED                                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 1508
#define RSSPECAN_ATTR_NOISE_LOCAL_OSCILLATOR_MODE                                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 6452
#define RSSPECAN_ATTR_NOISE_LOCAL_OSCILLATOR_FREQUENCY                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 6453
#define RSSPECAN_ATTR_NOISE_CONF_MODE_DUT                                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 1512
#define RSSPECAN_ATTR_NOISE_LO_FIXED                                                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 3765
#define RSSPECAN_ATTR_NOISE_CONF_SYST_LOSC                                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 1509
#define RSSPECAN_ATTR_NOISE_CONF_SYST_LOSC_FREQ                                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 1510
#define RSSPECAN_ATTR_NOISE_CONF_SYST_INT_FREQ                                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 1511
#define RSSPECAN_ATTR_NOISE_CORR_EREJ                                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 1505
#define RSSPECAN_ATTR_NOISE_CORR_ENR_TABLE_SELECT                                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 2879
#define RSSPECAN_ATTR_NOISE_CORR_ENR_TABLE_DELETE                                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 2880
#define RSSPECAN_ATTR_NOISE_ENR_TABLE_LIST                                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 6808
#define RSSPECAN_ATTR_NOISE_ENR_TEMPERATURE_TABLE_DELETE                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 5487
#define RSSPECAN_ATTR_NOISE_ENR_TEMPERATURE_TABLE_LIST                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 5488
#define RSSPECAN_ATTR_NOISE_ENR_CALIBRATION_NOISE_SOURCE                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 5489
#define RSSPECAN_ATTR_NOISE_CORR_ENR_CALIBRATION_MODE                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 2992
#define RSSPECAN_ATTR_NOISE_CORR_ENR_CALIBRATION_SPOT                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 2993
#define RSSPECAN_ATTR_NOISE_ENR_CALIBRATION_CONSTANT_RESISTOR_COLD_TEMPERATURE                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 5490
#define RSSPECAN_ATTR_NOISE_ENR_CALIBRATION_CONSTANT_RESISTOR_HOT_TEMPERATURE                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 5491
#define RSSPECAN_ATTR_NOISE_CORR_ENR_CALIBRATION_TABLE_SELECT                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 2994
#define RSSPECAN_ATTR_NOISE_ENR_CALIBRATION_SNS_SERIAL_NUMBER                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 10498
#define RSSPECAN_ATTR_NOISE_ENR_MEASUREMENT_NOISE_SOURCE                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 5483
#define RSSPECAN_ATTR_NOISE_CORR_ENR_MODE                                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 1500
#define RSSPECAN_ATTR_NOISE_CORR_ENR_SPOT                                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 1501
#define RSSPECAN_ATTR_NOISE_ENR_CONSTANT_RESISTOR_COLD_TEMPERATURE                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 5484
#define RSSPECAN_ATTR_NOISE_ENR_CONSTANT_RESISTOR_HOT_TEMPERATURE                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 5485
#define RSSPECAN_ATTR_NOISE_ENR_MEASUREMENT_SNS_SERIAL_NUMBER                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 10499
#define RSSPECAN_ATTR_NOISE_CORR_ENR_COMMON_MODE                                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 2991
#define RSSPECAN_ATTR_NOISE_CORR_TEMP                                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 1502
#define RSSPECAN_ATTR_NOISE_CORR_LOSS_MODE                                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 1503
#define RSSPECAN_ATTR_NOISE_CORR_LOSS_SPOT                                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 1504
#define RSSPECAN_ATTR_NOISE_LOSS_INPUT_TEMPERATURE                                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 6968
#define RSSPECAN_ATTR_NOISE_CORR_LOSS_INPUT_TABLE_SELECT                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 2881
#define RSSPECAN_ATTR_NOISE_CORR_LOSS_INPUT_TABLE_DELETE                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 2882
#define RSSPECAN_ATTR_NOISE_LOSS_INPUT_TABLE_LIST                                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 6969
#define RSSPECAN_ATTR_NOISE_CORR_LOSS_OUTPUT_MODE                                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 3167
#define RSSPECAN_ATTR_NOISE_CORR_LOSS_OUTPUT_SPOT                                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 3168
#define RSSPECAN_ATTR_NOISE_LOSS_OUTPUT_TEMPERATURE                                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 6970
#define RSSPECAN_ATTR_NOISE_CORR_LOSS_OUTPUT_TABLE_SELECT                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 2883
#define RSSPECAN_ATTR_NOISE_CORR_LOSS_OUTPUT_TABLE_DELETE                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 2884
#define RSSPECAN_ATTR_NOISE_LOSS_OUTPUT_TABLE_LIST                                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 6971
#define RSSPECAN_ATTR_NOISE_LOSS_CALIBRATION_MODE                                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 6972
#define RSSPECAN_ATTR_NOISE_LOSS_CALIBRATION_CONSTANT_INPUT_VALUE                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 6973
#define RSSPECAN_ATTR_NOISE_LOSS_CALIBRATION_TEMPERATURE                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 6974
#define RSSPECAN_ATTR_NOISE_LOSS_CALIBRATION_TABLE_SELECT                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 6975
#define RSSPECAN_ATTR_NOISE_LOSS_CALIBRATION_TABLE_DELETE                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 6976
#define RSSPECAN_ATTR_NOISE_LOSS_CALIBRATION_TABLE_LIST                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 6978
#define RSSPECAN_ATTR_NOISE_CORR_STAT                                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 1506
#define RSSPECAN_ATTR_NOISE_CONF_DUT_SETT_TIME                                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 1522
#define RSSPECAN_ATTR_QUERY_NOISE_AVAILABLE_CALIBRATION_RESULTS                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 10500
#define RSSPECAN_ATTR_NOISE_SAVE_CALIBRATION_RESULTS                                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 10501
#define RSSPECAN_ATTR_NOISE_RECALL_CALIBRATION_RESULTS                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 10502
#define RSSPECAN_ATTR_NOISE_DISP_TRAC_REF_LEV_STAT                                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 1530
#define RSSPECAN_ATTR_NOISE_CONF_DUT_GAIN                                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 1521
#define RSSPECAN_ATTR_NOISE_INP_GAIN_STAT                                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 1499
#define RSSPECAN_ATTR_NOISE_UNCERTAINTY_COMMON_SOURCE                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 3110
#define RSSPECAN_ATTR_NOISE_UNCERTAINTY_SOURCE_VSWR                                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 2971
#define RSSPECAN_ATTR_NOISE_UNCERTAINTY_SOURCE_RETURN_LOSS                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 2972
#define RSSPECAN_ATTR_NOISE_UNCERTAINTY_CALIBRATION_SOURCE_VSWR                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 3111
#define RSSPECAN_ATTR_NOISE_UNCERTAINTY_CALIBRATION_SOURCE_RETURN_LOSS                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 3112
#define RSSPECAN_ATTR_NOISE_UNCERTAINTY_ERN                                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 2973
#define RSSPECAN_ATTR_NOISE_UNCERTAINTY_CALIBRATION_ERN                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 3113
#define RSSPECAN_ATTR_NOISE_TEMPERATURE_UNCERTAINTY_COLD                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 5492
#define RSSPECAN_ATTR_NOISE_TEMPERATURE_UNCERTAINTY_HOT                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 5493
#define RSSPECAN_ATTR_NOISE_TEMPERATURE_UNCERTAINTY_CALIBRATION_COLD                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 5494
#define RSSPECAN_ATTR_NOISE_TEMPERATURE_UNCERTAINTY_CALIBRATION_HOT                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 5495
#define RSSPECAN_ATTR_NOISE_UNCERTAINTY_AUTO                                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 2978
#define RSSPECAN_ATTR_NOISE_UNCERTAINTY_NOISE                                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 2979
#define RSSPECAN_ATTR_NOISE_UNCERTAINTY_GAIN                                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 2980
#define RSSPECAN_ATTR_NOISE_UNCERTAINTY_DUT_INPUT_VSWR                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 2974
#define RSSPECAN_ATTR_NOISE_UNCERTAINTY_DUT_INPUT_RETURN_LOSS                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 2975
#define RSSPECAN_ATTR_NOISE_UNCERTAINTY_DUT_OUTPUT_VSWR                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 2976
#define RSSPECAN_ATTR_NOISE_UNCERTAINTY_DUT_OUTPUT_RETURN_LOSS                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 2977
#define RSSPECAN_ATTR_NOISE_UNCERTAINTY_PREAMPLIFIER_STATE                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 2982
#define RSSPECAN_ATTR_NOISE_UNCERTAINTY_PREAMPLIFIER_NOISE                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 2983
#define RSSPECAN_ATTR_NOISE_UNCERTAINTY_PREAMPLIFIER_GAIN                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 2984
#define RSSPECAN_ATTR_SPEC_ANALYZER_UNCERTAINTY_GAIN                                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 2995
#define RSSPECAN_ATTR_SPEC_ANALYZER_UNCERTAINTY_NOISE                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 2996
#define RSSPECAN_ATTR_NOISE_UNCERTAINTY_PREAMPLIFIER_VSWR                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 2985
#define RSSPECAN_ATTR_NOISE_UNCERTAINTY_PREAMPLIFIER_RETURN_LOSS                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 2986
#define RSSPECAN_ATTR_NOISE_UNCERTAINTY_FREQUENCY                                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 2981
#define RSSPECAN_ATTR_NOISE_UNCERTAINTY_RESULT                                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 2987
#define RSSPECAN_ATTR_NOISE_EXT_FREQ_OFFS                                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 1513
#define RSSPECAN_ATTR_NOISE_EXT_FREQ_NUM                                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 1514
#define RSSPECAN_ATTR_NOISE_EXT_FREQ_DEN                                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 1515
#define RSSPECAN_ATTR_NOISE_EXT_POW                                                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 1516
#define RSSPECAN_ATTR_NOISE_GPIB_GEN_ADDR                                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 1517
#define RSSPECAN_ATTR_NOISE_GEN_TYPE                                                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 1518
#define RSSPECAN_ATTR_NOISE_CONF_GEN_CONT_STAT                                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 1519
#define RSSPECAN_ATTR_NOISE_CONF_GEN_INIT_AUTO                                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 1520
#define RSSPECAN_ATTR_NOISE_INIT_GENERATOR                                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 2601
#define RSSPECAN_ATTR_NOISE_CONF_GEN_AUTO_SWITCH_OFF                                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 2600
#define RSSPECAN_ATTR_NOISE_MEASUREMENT_MODE                                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 5498
#define RSSPECAN_ATTR_NOISE_POWER_MEASUREMENT                                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 5499
#define RSSPECAN_ATTR_NOISE_DISP_DATA_TRAC                                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 1524
#define RSSPECAN_ATTR_NOISE_DISP_TRAC_STAT                                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 1525
#define RSSPECAN_ATTR_NOISE_DISP_SYMB                                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 1526
#define RSSPECAN_ATTR_NOISE_DISP_X_AXIS                                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 1534
#define RSSPECAN_ATTR_NOISE_DISP_AUTO_SCAL                                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 1527
#define RSSPECAN_ATTR_NOISE_DISP_TRAC_BOTTOM                                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 1528
#define RSSPECAN_ATTR_NOISE_DISP_TRAC_TOP                                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 1529
#define RSSPECAN_ATTR_NOISE_DISPLAY_UNCERTAINTY                                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 10707
#define RSSPECAN_ATTR_NOISE_DISP_GAIN_TRAC_STAT                                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 3170
#define RSSPECAN_ATTR_NOISE_DISP_GAIN_SYMB                                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 3769
#define RSSPECAN_ATTR_NOISE_DISP_GAIN_X_AXIS                                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 3770
#define RSSPECAN_ATTR_NOISE_DISP_AUTO_GAIN_SCAL                                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 3771
#define RSSPECAN_ATTR_NOISE_DISP_GAIN_TRAC_BOTTOM                                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 3772
#define RSSPECAN_ATTR_NOISE_DISP_GAIN_TRAC_TOP                                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 3773
#define RSSPECAN_ATTR_NOISE_DISP_TABLE                                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 1531
#define RSSPECAN_ATTR_NOISE_DISPL_CURR_RESULTS                                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 1533
#define RSSPECAN_ATTR_NOISE_ARRAY_MEMORY                                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 1523
#define RSSPECAN_ATTR_NOISE_DISP_TRACE_MEM                                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 1532
#define RSSPECAN_ATTR_NOISE_CALC_LIM_TRAC                                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 1493
#define RSSPECAN_ATTR_NOISE_CALC_LIMIT_RESULT_TYPE                                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 2885
#define RSSPECAN_ATTR_NOISE_ASSIGN_MARKER_TO_TRACE                                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 2644
#define RSSPECAN_ATTR_NOISE_ASSIGN_MARKER_TO_TRACE_FSV                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 4678
#define RSSPECAN_ATTR_NOISE_MARKER_GAIN                                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 3775
#define RSSPECAN_ATTR_NOISE_MARKER_NOISE                                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 3161
#define RSSPECAN_ATTR_NOISE_MARKER_NUNC                                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 3776
#define RSSPECAN_ATTR_NOISE_MARKER_COLD_POWER                                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 3777
#define RSSPECAN_ATTR_NOISE_MARKER_TEMPERATURE                                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 3778
#define RSSPECAN_ATTR_NOISE_MARKER_HOT_POWER                                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 3779
#define RSSPECAN_ATTR_NOISE_MARKER_Y_FACTOR                                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 3780
#define RSSPECAN_ATTR_NOISE_MARKER_CALIBRATION_Y_FACTOR                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 5501
#define RSSPECAN_ATTR_NOISE_MARKER_CALIBRATION_COLD_POWER                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 5502
#define RSSPECAN_ATTR_NOISE_MARKER_CALIBRATION_HOT_POWER                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 5503
#define RSSPECAN_ATTR_NOISE_DELTA_MARKER_GAIN                                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 5504
#define RSSPECAN_ATTR_NOISE_DELTA_MARKER_NOISE                                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 5505
#define RSSPECAN_ATTR_NOISE_DELTA_MARKER_TEMPERATURE                                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 5506
#define RSSPECAN_ATTR_NOISE_DELTA_MARKER_Y_FACTOR                                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 5507
#define RSSPECAN_ATTR_NOISE_DELTA_MARKER_COLD_POWER                                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 5508
#define RSSPECAN_ATTR_NOISE_DELTA_MARKER_HOT_POWER                                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 5509
#define RSSPECAN_ATTR_NOISE_DELTA_MARKER_CALIBRATION_Y_FACTOR                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 5510
#define RSSPECAN_ATTR_NOISE_DELTA_MARKER_CALIBRATION_COLD_POWER                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 5511
#define RSSPECAN_ATTR_NOISE_DELTA_MARKER_CALIBRATION_HOT_POWER                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 5512
#define RSSPECAN_ATTR_PHASE_MODE                                                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 1535
#define RSSPECAN_ATTR_PHASE_TRACKING_BANDWIDTH                                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 2988
#define RSSPECAN_ATTR_PHASE_ARITHMETIC_LEVEL_OFFSET                                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 6035
#define RSSPECAN_ATTR_CURRENTLY_MEASURED_SIGNAL_LEVEL                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 7525
#define RSSPECAN_ATTR_SIGNAL_SEARCH_AUTO_ENABLED                                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 6036
#define RSSPECAN_ATTR_SIGNAL_SEARCH_AUTO_COUNT                                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 6745
#define RSSPECAN_ATTR_SIGNAL_SEARCH_AUTO_LIMIT_MIN                                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 6037
#define RSSPECAN_ATTR_SIGNAL_SEARCH_AUTO_LIMIT_MAX                                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 6038
#define RSSPECAN_ATTR_SIGNAL_SEARCH_THRESHOLD                                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 6039
#define RSSPECAN_ATTR_PNOISE_BASEBAND_INPUT_CONNECTOR                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 6641
#define RSSPECAN_ATTR_PHASE_NOISE_LOCAL_OSCILLATOR_SOURCE                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 6746
#define RSSPECAN_ATTR_PHASE_NOISE_LOCAL_OSCILLATOR_EXTERNAL_LEVEL                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 6747
#define RSSPECAN_ATTR_OUTPUT_DC_POWER_ENABLED                                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 6040
#define RSSPECAN_ATTR_OUTPUT_DC_POWER_COUPLING_ENABLED                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 6041
#define RSSPECAN_ATTR_OUTPUT_DC_POWER_SUPPLY_PORT_ENABLED                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 6042
#define RSSPECAN_ATTR_OUTUPT_DC_POWER_SUPPLY_PORT_MODE                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 6043
#define RSSPECAN_ATTR_OUTPUT_DC_POWER_SUPPLY_PORT_LEVEL                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 6044
#define RSSPECAN_ATTR_OUTPUT_DC_POWER_SUPPLY_PORT_LIMIT_MAX                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 6045
#define RSSPECAN_ATTR_OUTPUT_DC_POWER_SUPPLY_PORT_LIMIT_MIN                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 6046
#define RSSPECAN_ATTR_OUTPUT_DC_POWER_SUPPLY_PORT_MAXIMUM                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 6047
#define RSSPECAN_ATTR_OUTPUT_DC_POWER_SUPPLY_PORT_CURRENT_MAXIMUM                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 6048
#define RSSPECAN_ATTR_OUTPUT_DC_POWER_TUNING_PORT_ENABLED                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 6049
#define RSSPECAN_ATTR_OUTPUT_DC_POWER_TUNING_PORT_LEVEL                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 6050
#define RSSPECAN_ATTR_OUTPUT_DC_POWER_TUNING_PORT_LIMIT_MAX                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 6051
#define RSSPECAN_ATTR_OUTPUT_DC_POWER_TUNING_PORT_LIMIT_MIN                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 6052
#define RSSPECAN_ATTR_OUTPUT_DC_POWER_TUNING_PORT_CURRENT_MAXIMUM                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 6053
#define RSSPECAN_ATTR_OUTPUT_DC_POWER_AUX_PORT_ENABLED                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 6054
#define RSSPECAN_ATTR_OUTPUT_DC_POWER_AUX_PORT_LEVEL                                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 6055
#define RSSPECAN_ATTR_OUTPUT_DC_POWER_AUX_PORT_LIMIT_MAX                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 6056
#define RSSPECAN_ATTR_OUTPUT_DC_POWER_AUX_PORT_LIMIT_MIN                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 6057
#define RSSPECAN_ATTR_OUTPUT_DC_POWER_AUX_CURRENT_MAXIMUM                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 6058
#define RSSPECAN_ATTR_OUTPUT_SIGNAL_ENABLED                                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 6062
#define RSSPECAN_ATTR_OUTPUT_SIGNAL_FREQUENCY                                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 6063
#define RSSPECAN_ATTR_SIGNAL_SOURCE_FREQUENCY_STEPSIZE                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 6748
#define RSSPECAN_ATTR_OUTPUT_SIGNAL_LEVEL                                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 6064
#define RSSPECAN_ATTR_OUTPUT_SIGNAL_SOURCE_COUPLING_ENABLED                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 6065
#define RSSPECAN_ATTR_SOURCE_DUT_BYPASS                                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 6495
#define RSSPECAN_ATTR_SOURCE_PULSE_MODULATION                                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 6496
#define RSSPECAN_ATTR_SOURCE_PULSE_PERIOD                                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 6497
#define RSSPECAN_ATTR_SOURCE_PULSE_WIDTH                                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 6498
#define RSSPECAN_ATTR_SOURCE_PULSE_TRIGGER_OUTPUT                                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 6499
#define RSSPECAN_ATTR_VCO_MEASUREMENT_STATE                                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 6604
#define RSSPECAN_ATTR_VCO_SWEEP_FIX_SOURCE                                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 6605
#define RSSPECAN_ATTR_VCO_SWEEP_INITIAL_SETTLING_TIME                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 6606
#define RSSPECAN_ATTR_VCO_SWEEP_SETTLING_TIME                                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 6607
#define RSSPECAN_ATTR_VCO_SWEEP_FREQUENCY_RESOLUTION                                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 6608
#define RSSPECAN_ATTR_VCO_SWEEP_POINTS                                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 6609
#define RSSPECAN_ATTR_VCO_SWEEP_SOURCE                                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 6610
#define RSSPECAN_ATTR_VCO_SWEEP_START                                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 6611
#define RSSPECAN_ATTR_VCO_SWEEP_STOP                                                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 6612
#define RSSPECAN_ATTR_SPOT_NOISE_VS_TUNE_MEASUREMENT_STATE                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 6613
#define RSSPECAN_ATTR_SPOT_NOISE_VS_TUNE_SWEEP_INITIAL_SETTLING_TIME                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 6614
#define RSSPECAN_ATTR_SPOT_NOISE_VS_TUNE_SWEEP_SETTLING_TIME                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 6615
#define RSSPECAN_ATTR_SPOT_NOISE_VS_TUNE_SWEEP_POINTS                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 6616
#define RSSPECAN_ATTR_SPOT_NOISE_VS_TUNE_SWEEP_SOURCE                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 6617
#define RSSPECAN_ATTR_SPOT_NOISE_VS_TUNE_SWEEP_START                                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 6618
#define RSSPECAN_ATTR_SPOT_NOISE_VS_TUNE_SWEEP_STOP                                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 6619
#define RSSPECAN_ATTR_TRANSIENT_ANALYSIS_MEASUREMENT_STATE                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 6749
#define RSSPECAN_ATTR_TRANSIENT_ANALYSIS_MODE                                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 6750
#define RSSPECAN_ATTR_TRANSIENT_ANALYSIS_REFERENCE_LEVEL                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 6751
#define RSSPECAN_ATTR_TRANSIENT_ANALYSIS_SETTLING_TIME_ENABLED                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 8594
#define RSSPECAN_ATTR_TRANSIENT_ANALYSIS_SETTLING_TIME_UPPER_PHASE_LIMIT                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 11205
#define RSSPECAN_ATTR_TRANSIENT_ANALYSIS_SETTLING_TIME_LOWER_PHASE_LIMIT                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 11206
#define RSSPECAN_ATTR_TRANSIENT_ANALYSIS_SETTLING_TIME_UPPER_FREQUENCY_LIMIT                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 8595
#define RSSPECAN_ATTR_TRANSIENT_ANALYSIS_SETTLING_TIME_LOWER_FREQUENCY_LIMIT                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 8596
#define RSSPECAN_ATTR_TRANSIENT_ANALYSIS_SETTLING_TIME_DIAGRAM                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 8597
#define RSSPECAN_ATTR_TRANSIENT_ANALYSIS_SETTLING_TIME_TRACE                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 8598
#define RSSPECAN_ATTR_TRANSIENT_ANALYSIS_SETTLING_TIME_RESULT                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 8599
#define RSSPECAN_ATTR_TRANSIENT_ANALYSIS_FREQUENCY_DEVIATION_ENABLED                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 8600
#define RSSPECAN_ATTR_TRANSIENT_ANALYSIS_FREQUENCY_DEVIATION_START                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 8601
#define RSSPECAN_ATTR_TRANSIENT_ANALYSIS_FREQUENCY_DEVIATION_STOP                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 8602
#define RSSPECAN_ATTR_TRANSIENT_ANALYSIS_FREQUENCY_DEVIATION_DIAGRAM                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 8603
#define RSSPECAN_ATTR_TRANSIENT_ANALYSIS_FREQUENCY_DEVIATION_TRACE                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 8604
#define RSSPECAN_ATTR_TRANSIENT_ANALYSIS_FREQUENCY_DEVIATION_REGRESSION_LINE_POINTS                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 8606
#define RSSPECAN_ATTR_TRANSIENT_ANALYSIS_FREQUENCY_DEVIATION_REGRESSION_LINE_SLOPE                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 8607
#define RSSPECAN_ATTR_TRANSIENT_ANALYSIS_Y_AXIS_SCALE_AF_COUPLING                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 6752
#define RSSPECAN_ATTR_TRANSIENT_ANALYSIS_Y_AXIS_SCALE_ZERO_PHASE_POSITIONING_MODE                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 6753
#define RSSPECAN_ATTR_TRANSIENT_ANALYSIS_Y_AXIS_SCALE_ZERO_PHASE_POSITION                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 6754
#define RSSPECAN_ATTR_TRANSIENT_ANALYSIS_Y_AXIS_UNIT                                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 6755
#define RSSPECAN_ATTR_TRANSIENT_ANALYSIS_Y_AXIS_GRID_MODE                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 7526
#define RSSPECAN_ATTR_TRANSIENT_ANALYSIS_PERSISTENCE_STATE                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 6756
#define RSSPECAN_ATTR_TRANSIENT_ANALYSIS_PERSISTENCE_DECAY                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 6757
#define RSSPECAN_ATTR_PHASE_CONF_POW_AUTO                                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 1537
#define RSSPECAN_ATTR_PHASE_CONF_POW_EXP_RF                                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 1540
#define RSSPECAN_ATTR_PHASE_CONF_REFM                                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 1560
#define RSSPECAN_ATTR_PHASE_FINISH_HALF_DECADE                                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 5046
#define RSSPECAN_ATTR_PHASE_MEASUREMENT                                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 6066
#define RSSPECAN_ATTR_PHASE_PULSE_DETECTION_MODE                                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 6067
#define RSSPECAN_ATTR_PHASE_PULSE_DETECTION_MODE_ONCE                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 6068
#define RSSPECAN_ATTR_PHASE_PULSE_EXTERNAL_GATE_TYPE                                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 6480
#define RSSPECAN_ATTR_PHASE_PULSE_REPETITION_INTERVAL                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 6069
#define RSSPECAN_ATTR_PHASE_PULSE_GATE_SOURCE                                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 7527
#define RSSPECAN_ATTR_PHASE_PULSE_GATE_LEVEL                                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 7528
#define RSSPECAN_ATTR_PHASE_FREQ_VER                                                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 1546
#define RSSPECAN_ATTR_PHASE_FREQ_VER_TOL                                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 1547
#define RSSPECAN_ATTR_PHASE_FREQ_VER_TOL_ABS                                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 1548
#define RSSPECAN_ATTR_PHASE_POW_RLEV_VER                                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 1550
#define RSSPECAN_ATTR_PHASE_POW_RLEV_VER_TOL                                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 1551
#define RSSPECAN_ATTR_PHASE_SIGNAL_VERIFICATION_FAILED                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 2812
#define RSSPECAN_ATTR_PHASE_SIGNAL_CAPTURE_RANGE                                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 6481
#define RSSPECAN_ATTR_PHASE_FREQ_TRACK                                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 1549
#define RSSPECAN_ATTR_PHASE_POW_TRACK                                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 1552
#define RSSPECAN_ATTR_PHASE_AM_REJECTION                                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 2934
#define RSSPECAN_ATTR_PHASE_MAX_FREQ_DRIFT                                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 3781
#define RSSPECAN_ATTR_PHASE_DIGITAL_PLL                                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 3122
#define RSSPECAN_ATTR_PHASE_DECIMATION                                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 5513
#define RSSPECAN_ATTR_PHASE_ONLINE_IQ                                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 5514
#define RSSPECAN_ATTR_PHASE_SWE_FORW                                                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 1542
#define RSSPECAN_ATTR_PHASE_SWEEP_MODE                                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 1541
#define RSSPECAN_ATTR_PHASE_BAND_RAT                                                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 1539
#define RSSPECAN_ATTR_PHASE_ULTRA_SMALL_RBWS                                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 10708
#define RSSPECAN_ATTR_PHASE_SWEEP_COUNT                                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 1536
#define RSSPECAN_ATTR_PHASE_MULTIPLIER_ENABLED                                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 3782
#define RSSPECAN_ATTR_PHASE_MULTIPLIER                                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 3783
#define RSSPECAN_ATTR_PHASE_BAND_TYPE                                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 1538
#define RSSPECAN_ATTR_PHASE_IQ_WINDOW_FUNCTION_ALL_DECADE                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 2818
#define RSSPECAN_ATTR_PHASE_CROSS_CORREL_OPTIMIZE_ENABLED                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 6070
#define RSSPECAN_ATTR_PHASE_CROSS_CORRELATION_OPTIMIZE_THRESHOLD                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 6758
#define RSSPECAN_ATTR_PHASE_CROSS_CORREL_FACTOR                                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 6071
#define RSSPECAN_ATTR_PHASE_FINISH_SEGMENT                                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 6759
#define RSSPECAN_ATTR_PHASE_LIST_RANG_BAND                                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 1543
#define RSSPECAN_ATTR_PHASE_LIST_RANG_FILT_TYPE                                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 1544
#define RSSPECAN_ATTR_PHASE_LIST_RANG_SWE_COUN                                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 1545
#define RSSPECAN_ATTR_PHASE_IQ_WINDOW_FUNCTION_PARTICULAR_DECADE                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 2817
#define RSSPECAN_ATTR_PHASE_RANGE_START_FREQUENCY_OFFSET                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 6072
#define RSSPECAN_ATTR_PHASE_RANGE_STOP_FREQUENCY_OFFSET                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 6073
#define RSSPECAN_ATTR_PHASE_RANGE_CROSS_CORRELATION_OPERATIONS                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 6074
#define RSSPECAN_ATTR_PHASE_COPY_TO_USER_LIMIT_LINE                                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 3071
#define RSSPECAN_ATTR_PHASE_NOISE_LIMIT_SHAPE_TYPE                                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 2797
#define RSSPECAN_ATTR_PHASE_NOISE_LIMIT_TRACE                                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 3785
#define RSSPECAN_ATTR_PHASE_NOISE_LIMIT_LEVEL                                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 2795
#define RSSPECAN_ATTR_PHASE_NOISE_LIMIT_CHECK_STATE                                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 2799
#define RSSPECAN_ATTR_PHASE_NOISE_LIMIT_CORNER_FREQUENCY                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 2794
#define RSSPECAN_ATTR_PHASE_NOISE_LIMIT_SLOPE                                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 2798
#define RSSPECAN_ATTR_PHASE_INTEGRATION_TRACE                                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 6075
#define RSSPECAN_ATTR_PHASE_INTEGRATION_RANGE_ENABLED                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 6076
#define RSSPECAN_ATTR_PHASE_INTEGRATION_RANGE_START                                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 6077
#define RSSPECAN_ATTR_PHASE_INTEGRATION_RANGE_STOP                                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 6078
#define RSSPECAN_ATTR_PHASE_INTEGRATION_WEIGHTING_FILTER                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 6079
#define RSSPECAN_ATTR_PHASE_INTEGRATION_WEIGHTING_FILTER_NONE                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 6080
#define RSSPECAN_ATTR_PHASE_INTEGRATION_WEIGHTING_FILTER_SELECT                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 6081
#define RSSPECAN_ATTR_PHASE_INTEGRATION_WEIGHTING_FILTER_NAME                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 6082
#define RSSPECAN_ATTR_PHASE_INTEGRATION_WEIGHTING_FILTER_COMMENT                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 6083
#define RSSPECAN_ATTR_PHASE_INTEGRATION_WEIGHTING_FILTER_DELETE                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 6084
#define RSSPECAN_ATTR_PHASE_SPURIOUS_SORT_ORDER                                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 8580
#define RSSPECAN_ATTR_PHASE_SPURIOUS_WEIGHTING_FILTER                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 8581
#define RSSPECAN_ATTR_PHASE_SPURIOUS_WEIGHTING_FILTER_NONE                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 8582
#define RSSPECAN_ATTR_PHASE_SPURIOUS_COLOR                                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 10709
#define RSSPECAN_ATTR_PHASE_SPURIOUS_FILTER_MODE                                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 8583
#define RSSPECAN_ATTR_PHASE_SPURIOUS_FILTER_INCLUDE_HARMONICS                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 8584
#define RSSPECAN_ATTR_PHASE_SPURIOUS_FILTER_RANGE                                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 8585
#define RSSPECAN_ATTR_PHASE_SPURIOUS_FILTER_NAME                                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 8586
#define RSSPECAN_ATTR_PHASE_SPURIOUS_FILTER_CLEAR                                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 8588
#define RSSPECAN_ATTR_PHASE_SPURIOUS_FILTER_SAVE                                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 8589
#define RSSPECAN_ATTR_PHASE_SPURIOUS_FILTER_LOAD                                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 8591
#define RSSPECAN_ATTR_PHASE_FREQUENCY_STABILITY_TAU_START                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 7529
#define RSSPECAN_ATTR_PHASE_FREQUENCY_STABILITY_TAU_STOP                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 7530
#define RSSPECAN_ATTR_PHASE_SPUR_SUPPRESSION                                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 2819
#define RSSPECAN_ATTR_PHASE_SPUR_THRESHOLD                                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 2820
#define RSSPECAN_ATTR_SMOOTHING_ALL_TRACES_STATE                                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 7531
#define RSSPECAN_ATTR_SMOOTHING_ALL_TRACES_APERTURE                                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 7532
#define RSSPECAN_ATTR_PHASE_X_AXIS_SCOPE                                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 2823
#define RSSPECAN_ATTR_PHASE_X_AXIS_START_FREQUENCY                                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 2824
#define RSSPECAN_ATTR_PHASE_X_AXIS_STOP_FREQUENCY                                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 2825
#define RSSPECAN_ATTR_PHASE_X_AXIS_HALF_DECADE                                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 2826
#define RSSPECAN_ATTR_PHASE_DISP_TRACE_Y_AUTO                                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 1562
#define RSSPECAN_ATTR_PHASE_DISP_TRACE_Y_AUTO_ONCE                                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 1563
#define RSSPECAN_ATTR_PHASE_Y_AXIS_MANUAL_SCALING                                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 2821
#define RSSPECAN_ATTR_PHASE_DISP_TRACE_Y_RLEV                                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 1564
#define RSSPECAN_ATTR_PHASE_DISP_TRACE_Y                                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 1561
#define RSSPECAN_ATTR_PHASE_DISP_TRACE_Y_BOTTOM                                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 2822
#define RSSPECAN_ATTR_PHASE_Y_AXIS_UNIT                                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 6482
#define RSSPECAN_ATTR_PHASE_DISPLAY_TRACE_Y_AUTO                                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 6088
#define RSSPECAN_ATTR_PHASE_DISPLAY_TRACE_Y_AUTO_ONCE                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 6089
#define RSSPECAN_ATTR_PHASE_DISPLAY_TRACE_Y_RLEV                                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 6090
#define RSSPECAN_ATTR_PHASE_DISPLAY_TRACE_Y                                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 6091
#define RSSPECAN_ATTR_PHASE_DISPLAY_TRACE_Y_BOTTOM                                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 6092
#define RSSPECAN_ATTR_PHASE_DISP_TRACE_Y_RLEV_OFFS                                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 1565
#define RSSPECAN_ATTR_PHASE_DISP_TRACE_SMO_TYPE                                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 1568
#define RSSPECAN_ATTR_PHASE_DISP_TRACE_SMO_APER                                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 1567
#define RSSPECAN_ATTR_PHASE_DISP_TRACE_XGAIN_ENABLED                                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 6093
#define RSSPECAN_ATTR_PHASE_DISP_TRACE_SMO                                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 1566
#define RSSPECAN_ATTR_PHASE_DISP_TRACE_SPUR_SUPP_ENABLED                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 5047
#define RSSPECAN_ATTR_PHASE_TRACE_SPUR_THRESHOLD                                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 6094
#define RSSPECAN_ATTR_PHASE_DISPLAY_TRACE_OFFSET_ENABLED                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 6095
#define RSSPECAN_ATTR_PHASE_DISPLAY_TRACE_OFFSET                                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 6096
#define RSSPECAN_ATTR_PHASE_TRACE_LABEL_ENABLED                                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 6097
#define RSSPECAN_ATTR_PHASE_TRACE_LABEL                                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 6098
#define RSSPECAN_ATTR_PHASE_TRACE_DISPLAYED_RESULT                                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 6099
#define RSSPECAN_ATTR_PHASE_DISPLAY_TRACE_SMO_APERTURE                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 6100
#define RSSPECAN_ATTR_PHASE_DISPLAY_TRACE_SMO_ENABLED                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 6101
#define RSSPECAN_ATTR_PHASE_DISPLAY_TRACE_SPUR_SUPP_ENABLED                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 6102
#define RSSPECAN_ATTR_PHASE_DISPLAY_TRACE_SPUR_THRESHOLD                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 6103
#define RSSPECAN_ATTR_PHASE_CALC_EVAL                                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 1553
#define RSSPECAN_ATTR_PHASE_CALC_EVAL_STAR                                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 1554
#define RSSPECAN_ATTR_PHASE_CALC_EVAL_STOP                                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 1555
#define RSSPECAN_ATTR_PHASE_USER_TRACE                                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 2813
#define RSSPECAN_ATTR_PHASE_USER_EVALUATE_FROM                                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 2814
#define RSSPECAN_ATTR_PHASE_USER_EVALUATE_TO                                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 2815
#define RSSPECAN_ATTR_PHASE_RESIDUAL_FM                                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 3787
#define RSSPECAN_ATTR_PHASE_RESIDUAL_RMS_JITTER                                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 3788
#define RSSPECAN_ATTR_PHASE_RESIDUAL_PM                                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 3789
#define RSSPECAN_ATTR_PHASE_RESIDUAL_DUT_GAIN                                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 7533
#define RSSPECAN_ATTR_PHASE_USER_RESIDUAL_FM                                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 3790
#define RSSPECAN_ATTR_PHASE_USER_RESIDUAL_RMS_JITTER                                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 3791
#define RSSPECAN_ATTR_PHASE_USER_RESIDUAL_PM                                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 3792
#define RSSPECAN_ATTR_PHASE_SPOT_NOISE_SELECT_TRACE                                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 5887
#define RSSPECAN_ATTR_PHASE_SPOT_NOISE_DISPLAY_INFO_ENABLED                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 6104
#define RSSPECAN_ATTR_PHASE_SPOT_NOISE_SELECT_TRACE_FSWP                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 6105
#define RSSPECAN_ATTR_PHASE_SPOT_NOISE_CALCULATION                                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 2816
#define RSSPECAN_ATTR_PHASE_CALC_SNO_AOFF                                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 1558
#define RSSPECAN_ATTR_PHASE_CUSTOM_SNO_ENABLED                                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 6106
#define RSSPECAN_ATTR_PHASE_CALC_SNO_STAT                                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 1559
#define RSSPECAN_ATTR_PHASE_CALC_SNO_X                                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 1556
#define RSSPECAN_ATTR_PHASE_CALC_SNO_Y                                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 1557
#define RSSPECAN_ATTR_PHASE_MEASURED_FREQUENCY                                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 2935
#define RSSPECAN_ATTR_PHASE_MEASURED_LEVEL                                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 2936
#define RSSPECAN_ATTR_PHASE_MEASURED_ALLAN_VARIANCE_AND_DEVIATION                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 7534
#define RSSPECAN_ATTR_PHASE_DISCRETE_JITTER_RESULT                                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 6107
#define RSSPECAN_ATTR_PHASE_RANDOM_JITTER_RESULT                                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 6108
#define RSSPECAN_ATTR_PHASE_SPURIOUS_LIST_COUNT                                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 8593
#define RSSPECAN_ATTR_PHASE_RANGE_RESIDUAL_IPN                                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 6110
#define RSSPECAN_ATTR_PHASE_RANGE_RESIDUAL_FM                                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 6111
#define RSSPECAN_ATTR_PHASE_RANGE_RESIDUAL_RMS_JITTER                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 6112
#define RSSPECAN_ATTR_PHASE_RANGE_RESIDUAL_PM                                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 6113
#define RSSPECAN_ATTR_PHASE_RANGE_RESIDUAL_AM                                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 7535
#define RSSPECAN_ATTR_VSA_MODE                                                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 1569
#define RSSPECAN_ATTR_VSA_FACTORY_DEFAULTS                                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 1570
#define RSSPECAN_ATTR_VSA_SCREEN_COUPLING                                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 1572
#define RSSPECAN_ATTR_VSA_DIGITAL_STANDARD                                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 1580
#define RSSPECAN_ATTR_VSA_FREQUENCY_STEP_AUTO                                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 1581
#define RSSPECAN_ATTR_VSA_DIGITAL_STANDARD_SAVE                                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 1582
#define RSSPECAN_ATTR_VSA_DIGITAL_STANDARD_COMMENT                                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 1584
#define RSSPECAN_ATTR_VSA_DIGITAL_STANDARD_DELETE                                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 1586
#define RSSPECAN_ATTR_VSA_DIGITAL_STANDARD_PRESET                                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 7164
#define RSSPECAN_ATTR_VSA_DIGITAL_IQ_40G_ENABLED                                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 7165
#define RSSPECAN_ATTR_VSA_DIGITAL_IQ_40G_SAMPLE_RATE                                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 7166
#define RSSPECAN_ATTR_VSA_DIGITAL_IQ_40G_CONNECTED_DEVICE                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 7167
#define RSSPECAN_ATTR_VSA_MODULATION_SRATE                                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 1627
#define RSSPECAN_ATTR_VSA_MODULATION_FORMAT                                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 1628
#define RSSPECAN_ATTR_VSA_MODULATION_QPSK_FORMAT                                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 1629
#define RSSPECAN_ATTR_VSA_MODULATION_PSK_FORMAT                                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 1630
#define RSSPECAN_ATTR_VSA_MODULATION_MSK_FORMAT                                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 1631
#define RSSPECAN_ATTR_VSA_MODULATION_QAM_FORMAT                                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 1632
#define RSSPECAN_ATTR_VSA_MODULATION_PSK_NSTAT                                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 1633
#define RSSPECAN_ATTR_VSA_MODULATION_QAM_NSTAT                                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 1634
#define RSSPECAN_ATTR_VSA_MODULATION_FSK_NSTAT                                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 1635
#define RSSPECAN_ATTR_VSA_MODULATION_UQAM_NSTAT                                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 1637
#define RSSPECAN_ATTR_VSA_MODULATION_ASK_MODE                                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 2945
#define RSSPECAN_ATTR_VSA_MODULATION_APSK_MODE                                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 2946
#define RSSPECAN_ATTR_VSA_MODULATION_MAPPING                                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 1638
#define RSSPECAN_ATTR_VSA_MODULATION_UQAM_MAPPING                                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 1639
#define RSSPECAN_ATTR_VSA_LOAD_USER_MODULATION                                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 2947
#define RSSPECAN_ATTR_VSA_MODULATION_FILTER_STATE                                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 1640
#define RSSPECAN_ATTR_VSA_MODULATION_ALPHABT                                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 1641
#define RSSPECAN_ATTR_VSA_MODULATION_PRATE                                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 1642
#define RSSPECAN_ATTR_VSA_DEMODULATION_SIDE_BAND                                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 1655
#define RSSPECAN_ATTR_VSA_MODULATION_FSK_DEV_COMP                                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 1643
#define RSSPECAN_ATTR_VSA_MODULATION_FSK_DEV                                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 1644
#define RSSPECAN_ATTR_VSA_MODULATION_FSK_DEV_REL                                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 2663
#define RSSPECAN_ATTR_VSA_TX_FILTER_STATE                                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 1648
#define RSSPECAN_ATTR_VSA_TX_FILTER_TYPE                                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 1647
#define RSSPECAN_ATTR_VSA_TX_FILTER_ALPHA                                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 1649
#define RSSPECAN_ATTR_VSA_TX_FILTER_USER_FILE                                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 2647
#define RSSPECAN_ATTR_VSA_SIGNAL_TYPE                                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 1624
#define RSSPECAN_ATTR_VSA_BURST_SEARCH_LENGTH_MIN                                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 1595
#define RSSPECAN_ATTR_VSA_BURST_SEARCH_LENGTH_MAX                                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 1596
#define RSSPECAN_ATTR_VSA_BURST_SEARCH_SKIP_RISE                                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 1597
#define RSSPECAN_ATTR_VSA_BURST_SEARCH_SKIP_FALL                                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 1598
#define RSSPECAN_ATTR_VSA_PATTERN_SIGNAL_STATE                                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 1625
#define RSSPECAN_ATTR_VSA_PATTERN_OFFSET_STATE                                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 1626
#define RSSPECAN_ATTR_VSA_FIT_SYNC_OFFSET                                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 1674
#define RSSPECAN_ATTR_VSA_FRAME_SAME_AS_DATA_SYMBOLS                                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 7168
#define RSSPECAN_ATTR_VSA_FRAME_MODULATION_TYPE                                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 7169
#define RSSPECAN_ATTR_VSA_FRAME_CONFIG                                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 7170
#define RSSPECAN_ATTR_VSA_FRAME_NEXT_BOOSTING                                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 7171
#define RSSPECAN_ATTR_VSA_FRAME_NEXT_MODULATION                                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 7172
#define RSSPECAN_ATTR_VSA_FRAME_PREVIOUS_BOOSTING                                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 7173
#define RSSPECAN_ATTR_VSA_FRAME_PREVIOUS_MODULATION                                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 7174
#define RSSPECAN_ATTR_VSA_FRAME_LOAD                                                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 7175
#define RSSPECAN_ATTR_VSA_FRAME_SAVE                                                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 7177
#define RSSPECAN_ATTR_VSA_FRAME_DESCRIPTION                                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 7180
#define RSSPECAN_ATTR_VSA_FRAME_MODE                                                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 7181
#define RSSPECAN_ATTR_VSA_FRAME_MAPPINGS_CATALOG                                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 7182
#define RSSPECAN_ATTR_VSA_FRAME_MAPPING_SELECT                                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 7183
#define RSSPECAN_ATTR_VSA_FRAME_PSK_FORMAT                                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 7184
#define RSSPECAN_ATTR_VSA_FRAME_PSK_N_STATE                                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 7185
#define RSSPECAN_ATTR_VSA_FRAME_QAM_FORMAT                                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 7186
#define RSSPECAN_ATTR_VSA_FRAME_QAM_N_STATE                                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 7187
#define RSSPECAN_ATTR_VSA_FRAME_QPSK_FORMAT                                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 7188
#define RSSPECAN_ATTR_VSA_FRAME_APSK_N_STATE                                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 7763
#define RSSPECAN_ATTR_VSA_FRAME_ASK_N_STATE                                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 7764
#define RSSPECAN_ATTR_VSA_FRAME_USER_FILE_NAME                                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 7189
#define RSSPECAN_ATTR_VSA_FRAME_START_SAMPLE                                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 7190
#define RSSPECAN_ATTR_VSA_KNOWN_DATA_STATE                                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 2784
#define RSSPECAN_ATTR_VSA_KNOWN_DATA_FILE_NAME                                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 2785
#define RSSPECAN_ATTR_VSA_KNOWN_DATA_NEGATE_FEEDBACK_PATH_AUTO                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 7191
#define RSSPECAN_ATTR_VSA_KNOWN_DATA_NEGATE_FEEDBACK_PATH_ENABLED                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 7192
#define RSSPECAN_ATTR_VSA_KNOWN_DATA_PRBS_PATTERN_ENABLED                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 7193
#define RSSPECAN_ATTR_VSA_KNOWN_DATA_PRBS_PATTERN_AUTO                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 7194
#define RSSPECAN_ATTR_VSA_KNOWN_DATA_POLYNOMIAL_AUTO                                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 7195
#define RSSPECAN_ATTR_VSA_KNOWN_DATA_PRBS_TYPE                                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 7197
#define RSSPECAN_ATTR_VSA_KNOWN_DATA_PRBS_SOURCE                                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 7198
#define RSSPECAN_ATTR_VSA_REF_LEVEL_ADJUST                                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 1571
#define RSSPECAN_ATTR_VSA_SCALE_Y_PDIV                                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 1681
#define RSSPECAN_ATTR_VSA_DISPLAY_YAXIS_MIN                                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 3799
#define RSSPECAN_ATTR_VSA_DISPLAY_YAXIS_MAX                                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 3800
#define RSSPECAN_ATTR_VSA_UNIT_ANGLE                                                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 1668
#define RSSPECAN_ATTR_VSA_Y_AXIS_UNIT_TIME                                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 5049
#define RSSPECAN_ATTR_VSA_SCALE_X_BAR_COUNT                                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 1677
#define RSSPECAN_ATTR_VSA_SCALE_X_RVAL                                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 1679
#define RSSPECAN_ATTR_VSA_SCALE_X_RPOS                                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 1680
#define RSSPECAN_ATTR_VSA_SCALE_X_PDIV                                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 1678
#define RSSPECAN_ATTR_VSA_UNIT_TIME                                                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 1667
#define RSSPECAN_ATTR_VSA_DEMODULATION_REC_LENGTH_AUTO                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 1650
#define RSSPECAN_ATTR_VSA_DEMODULATION_REC_LENGTH_SYM                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 1651
#define RSSPECAN_ATTR_VSA_DEMODULATION_REC_LENGTH_S                                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 1652
#define RSSPECAN_ATTR_VSA_DEMODULATION_RECORD_LENGTH                                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 7765
#define RSSPECAN_ATTR_VSA_RESULT_RANGE_NUMBER                                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 2670
#define RSSPECAN_ATTR_VSA_CURRENT_STATISTICS_COUNTER_CAPTURE                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 3801
#define RSSPECAN_ATTR_VSA_CURRENT_STATISTICS_COUNTER_EVALUATED                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 3802
#define RSSPECAN_ATTR_VSA_STATISTIC_COUNT                                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 2669
#define RSSPECAN_ATTR_VSA_BURST_SEARCH_STATE                                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 1589
#define RSSPECAN_ATTR_VSA_BURST_SEARCH_AUTO                                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 1588
#define RSSPECAN_ATTR_VSA_SEARCH_BURST_AUTO_CONFIGURATION                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 1600
#define RSSPECAN_ATTR_VSA_BURST_SEARCH_MODE                                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 1590
#define RSSPECAN_ATTR_VSA_BURST_SEARCH_GLEN                                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 1599
#define RSSPECAN_ATTR_VSA_SEARCH_BURST_TOLERANCE                                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 1601
#define RSSPECAN_ATTR_VSA_SYNC_SEARCH_STATE                                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 1603
#define RSSPECAN_ATTR_VSA_SYNC_SEARCH_AUTO                                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 1602
#define RSSPECAN_ATTR_VSA_SYNC_SEARCH_IQ_CORRELATION_THRESHOLD                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 1615
#define RSSPECAN_ATTR_VSA_SYNC_SEARCH_PATTERN_AUTO_CONFIGURATION                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 1614
#define RSSPECAN_ATTR_VSA_SYNC_SEARCH_MODE                                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 1604
#define RSSPECAN_ATTR_VSA_SYNC_SEARCH_SELECTED_PATTERN                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 3803
#define RSSPECAN_ATTR_VSA_SYNC_NAME                                                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 1605
#define RSSPECAN_ATTR_VSA_PATTERN_DATA                                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 3804
#define RSSPECAN_ATTR_VSA_SYNC_COMMENT                                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 1607
#define RSSPECAN_ATTR_VSA_SYNC_TEXT                                                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 1606
#define RSSPECAN_ATTR_VSA_SYNC_NSTAT                                                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 1608
#define RSSPECAN_ATTR_VSA_SYNC_PATT_ADD                                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 1609
#define RSSPECAN_ATTR_VSA_SYNC_PATT_REMOVE                                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 1611
#define RSSPECAN_ATTR_VSA_SYNC_PATT_REMOVE_ALL                                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 1612
#define RSSPECAN_ATTR_VSA_SYNC_COPY                                                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 1613
#define RSSPECAN_ATTR_VSA_SYNC_DELETE                                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 1610
#define RSSPECAN_ATTR_VSA_DEMODULATION_RESULT_LENGTH                                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 1653
#define RSSPECAN_ATTR_VSA_FIT_REF_POINT                                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 1669
#define RSSPECAN_ATTR_VSA_FIT_ALIGN                                                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 1670
#define RSSPECAN_ATTR_VSA_FIT_ALIGN_OFFSET                                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 1672
#define RSSPECAN_ATTR_VSA_FIT_SYMBOL_NUMBER                                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 1673
#define RSSPECAN_ATTR_VSA_RESULT_LENGTH_DEFINED                                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 7201
#define RSSPECAN_ATTR_VSA_DEMODULATION_NORMALIZE                                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 1656
#define RSSPECAN_ATTR_VSA_COMPENSATE_IQ_OFFSET                                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 1660
#define RSSPECAN_ATTR_VSA_COMPENSATE_IQ_IMBALANCE                                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 1661
#define RSSPECAN_ATTR_VSA_COMPENSATE_AMPLITUDE_DROOP                                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 1662
#define RSSPECAN_ATTR_VSA_COMPENSATE_SYMBOL_RATE_ERROR                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 2990
#define RSSPECAN_ATTR_VSA_COMPENSATE_CHANNEL                                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 2989
#define RSSPECAN_ATTR_VSA_COMPENSATE_CARRIER_FREQUENCY_DRIFT                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 2664
#define RSSPECAN_ATTR_VSA_COMPENSATE_FSK_DEVIATION                                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 2665
#define RSSPECAN_ATTR_VSA_COMPENSATE_IQ_SKEW                                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 7202
#define RSSPECAN_ATTR_VSA_EQUALIZER_STATE                                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 1616
#define RSSPECAN_ATTR_VSA_EQUALIZER_MODE                                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 3808
#define RSSPECAN_ATTR_VSA_EQUALIZER_LENGTH                                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 1619
#define RSSPECAN_ATTR_VSA_EQUALIZER_RESET                                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 1623
#define RSSPECAN_ATTR_VSA_EQUALIZER_FILE_FORMAT                                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 7766
#define RSSPECAN_ATTR_VSA_EQUALIZER_SAVE                                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 1621
#define RSSPECAN_ATTR_VSA_EQUALIZER_LOAD                                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 1620
#define RSSPECAN_ATTR_VSA_MODULATION_EVM_CALCULATION                                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 1645
#define RSSPECAN_ATTR_VSA_MODULATION_OPTIMIZATION                                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 3124
#define RSSPECAN_ATTR_VSA_AUTOMATIC_ESTIMATION_STATE                                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 2666
#define RSSPECAN_ATTR_VSA_ESTIMATION_VALUE                                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 2667
#define RSSPECAN_ATTR_VSA_USE_PATTERN_FOR_SYNC_MODE                                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 2630
#define RSSPECAN_ATTR_VSA_USE_PATTERN_FOR_SYNC_STATE                                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 2633
#define RSSPECAN_ATTR_VSA_FINE_SYNC_AUTO                                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 2780
#define RSSPECAN_ATTR_VSA_FINE_SYNC_MODE                                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 2781
#define RSSPECAN_ATTR_VSA_FINE_SYNC_SER                                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 2782
#define RSSPECAN_ATTR_VSA_FINE_SYNC_RESULT                                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 2783
#define RSSPECAN_ATTR_VSA_MODULATION_EVM_OFFSET                                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 1646
#define RSSPECAN_ATTR_VSA_MEASUREMENT_FILTER_STATE                                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 1665
#define RSSPECAN_ATTR_VSA_AUTO_MEASUREMENT_FILTER                                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 1663
#define RSSPECAN_ATTR_VSA_MEASUREMENT_FILTER_TYPE                                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 1664
#define RSSPECAN_ATTR_VSA_MEASUREMENT_FILTER_ALPHA                                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 1666
#define RSSPECAN_ATTR_VSA_MEASUREMENT_FILTER_USER_FILE                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 2648
#define RSSPECAN_ATTR_VSA_EVAL_LINE_STATE                                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 1657
#define RSSPECAN_ATTR_VSA_EVAL_LINE_START                                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 3811
#define RSSPECAN_ATTR_VSA_EVAL_LINE_STOP                                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 3812
#define RSSPECAN_ATTR_VSA_IQ_LOAD_STREAM_AUTO_ENABLED                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 7203
#define RSSPECAN_ATTR_VSA_IQ_LOAD_STREAM                                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 7204
#define RSSPECAN_ATTR_QUERY_VSA_IQ_STREAM_LISTS                                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 7205
#define RSSPECAN_ATTR_VSA_FILE_EXP_HEADER                                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 1693
#define RSSPECAN_ATTR_VSA_FILE_DATA_MODE                                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 1694
#define RSSPECAN_ATTR_VSA_INCLUDE_IQ_DATA                                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 2577
#define RSSPECAN_ATTR_VSA_TRACE_EVALUATION                                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 1685
#define RSSPECAN_ATTR_VSA_TRACE_AUTOMATIC_Y_SCALING                                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 7206
#define RSSPECAN_ATTR_VSA_TRACE_REPETITION_COUNT                                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 7207
#define RSSPECAN_ATTR_VSA_MARKER_SEARCH                                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 1573
#define RSSPECAN_ATTR_VSA_PRESET_SCREENS                                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 1577
#define RSSPECAN_ATTR_VSA_MARKER_PEAK_SEARCH                                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 1574
#define RSSPECAN_ATTR_VSA_DELTAMARKER_PEAK_SEARCH                                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 7208
#define RSSPECAN_ATTR_VSA_DELTAMARKER_ABSOLUTE_X                                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 7209
#define RSSPECAN_ATTR_VSA_BURST_LENGTH                                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 6007
#define RSSPECAN_ATTR_VSA_SPECTRUM_STATE                                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 1696
#define RSSPECAN_ATTR_VSA_RESULT                                                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 1578
#define RSSPECAN_ATTR_VSA_RESULT_FORMAT                                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 1579
#define RSSPECAN_ATTR_VSA_CAPTURE_OVERSAMPLING                                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 1684
#define RSSPECAN_ATTR_VSA_CONFIGURE_DISPLAY_POINTS_PER_SYMBOL_AUTO                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 1682
#define RSSPECAN_ATTR_VSA_CONFIGURE_DISPLAY_POINTS_PER_SYMBOL                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 1683
#define RSSPECAN_ATTR_VSA_CONFIGURE_DISPLAY_TRACE_SYMBOL_STATE                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 2668
#define RSSPECAN_ATTR_VSA_DISPLAY_TRACE_SYMBOL                                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 1675
#define RSSPECAN_ATTR_VSA_DISPLAY_INDIVIDUAL_RESULTS                                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 7406
#define RSSPECAN_ATTR_VSA_RESULT_EVM_RMS_CURRENT                                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 3815
#define RSSPECAN_ATTR_VSA_RESULT_EVM_RMS_AVERAGE                                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 3816
#define RSSPECAN_ATTR_VSA_RESULT_EVM_RMS_PEAK                                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 3817
#define RSSPECAN_ATTR_VSA_RESULT_EVM_RMS_SDEV                                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 3818
#define RSSPECAN_ATTR_VSA_RESULT_EVM_RMS_CURRENT_PCTL                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 3819
#define RSSPECAN_ATTR_VSA_RESULT_EVM_PEAK_CURRENT                                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 3820
#define RSSPECAN_ATTR_VSA_RESULT_EVM_PEAK_AVERAGE                                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 3821
#define RSSPECAN_ATTR_VSA_RESULT_EVM_PEAK_PEAK                                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 3822
#define RSSPECAN_ATTR_VSA_RESULT_EVM_PEAK_SDEV                                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 3823
#define RSSPECAN_ATTR_VSA_RESULT_EVM_PEAK_PCTL                                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 3824
#define RSSPECAN_ATTR_VSA_RESULT_MER_RMS_CURRENT                                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 3825
#define RSSPECAN_ATTR_VSA_RESULT_MER_RMS_AVERAGE                                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 3826
#define RSSPECAN_ATTR_VSA_RESULT_MER_RMS_PEAK                                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 3827
#define RSSPECAN_ATTR_VSA_RESULT_MER_RMS_SDEV                                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 3828
#define RSSPECAN_ATTR_VSA_RESULT_MER_RMS_PCTL                                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 3829
#define RSSPECAN_ATTR_VSA_RESULT_MER_PEAK_CURRENT                                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 3830
#define RSSPECAN_ATTR_VSA_RESULT_MER_PEAK_AVERAGE                                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 3831
#define RSSPECAN_ATTR_VSA_RESULT_MER_PEAK_PEAK                                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 3832
#define RSSPECAN_ATTR_VSA_RESULT_MER_PEAK_SDEV                                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 3833
#define RSSPECAN_ATTR_VSA_RESULT_MER_PEAK_PCTL                                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 3834
#define RSSPECAN_ATTR_VSA_RESULT_PHASE_ERROR_RMS_CURRENT                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 3835
#define RSSPECAN_ATTR_VSA_RESULT_PHASE_ERROR_RMS_AVERAGE                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 3836
#define RSSPECAN_ATTR_VSA_RESULT_PHASE_ERROR_RMS_PEAK                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 3837
#define RSSPECAN_ATTR_VSA_RESULT_PHASE_ERROR_RMS_SDEV                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 3838
#define RSSPECAN_ATTR_VSA_RESULT_PHASE_ERROR_RMS_PCTL                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 3839
#define RSSPECAN_ATTR_VSA_RESULT_PHASE_ERROR_PEAK_AVERAGE                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 3840
#define RSSPECAN_ATTR_VSA_RESULT_PHASE_ERROR_PEAK_PEAK                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 3841
#define RSSPECAN_ATTR_VSA_RESULT_PHASE_ERROR_PEAK_SDEV                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 3842
#define RSSPECAN_ATTR_VSA_RESULT_PHASE_ERROR_PEAK_CURRENT                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 3843
#define RSSPECAN_ATTR_VSA_RESULT_PHASE_ERROR_PEAK_PCTL                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 3844
#define RSSPECAN_ATTR_VSA_RESULT_MERR_RMS_CURRENT                                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 3845
#define RSSPECAN_ATTR_VSA_RESULT_MERR_RMS_AVERAGE                                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 3846
#define RSSPECAN_ATTR_VSA_RESULT_MERR_RMS_PEAK                                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 3847
#define RSSPECAN_ATTR_VSA_RESULT_MERR_RMS_SDEV                                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 3848
#define RSSPECAN_ATTR_VSA_RESULT_MERR_RMS_PCTL                                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 3849
#define RSSPECAN_ATTR_VSA_RESULT_MERR_PEAK_CURRENT                                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 3850
#define RSSPECAN_ATTR_VSA_RESULT_MERR_PEAK_AVERAGE                                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 3851
#define RSSPECAN_ATTR_VSA_RESULT_MERR_PEAK_PEAK                                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 3852
#define RSSPECAN_ATTR_VSA_RESULT_MERR_PEAK_SDEV                                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 3853
#define RSSPECAN_ATTR_VSA_RESULT_MERR_PEAK_PCTL                                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 3854
#define RSSPECAN_ATTR_VSA_RESULT_CFER_CURRENT                                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 3855
#define RSSPECAN_ATTR_VSA_RESULT_CFER_AVERAGE                                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 3856
#define RSSPECAN_ATTR_VSA_RESULT_CFER_PEAK                                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 3857
#define RSSPECAN_ATTR_VSA_RESULT_CFER_SDEV                                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 3858
#define RSSPECAN_ATTR_VSA_RESULT_CFER_PCTL                                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 3859
#define RSSPECAN_ATTR_VSA_RESULT_SRER_CURRENT                                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 3860
#define RSSPECAN_ATTR_VSA_RESULT_SRER_AVERAGE                                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 3861
#define RSSPECAN_ATTR_VSA_RESULT_SRER_PEAK                                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 3862
#define RSSPECAN_ATTR_VSA_RESULT_SRER_SDEV                                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 3863
#define RSSPECAN_ATTR_VSA_RESULT_SRER_PCTL                                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 3864
#define RSSPECAN_ATTR_VSA_RESULT_RHO_CURRENT                                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 3865
#define RSSPECAN_ATTR_VSA_RESULT_RHO_AVERAGE                                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 3866
#define RSSPECAN_ATTR_VSA_RESULT_RHO_PEAK                                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 3867
#define RSSPECAN_ATTR_VSA_RESULT_RHO_SDEV                                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 3868
#define RSSPECAN_ATTR_VSA_RESULT_RHO_PCTL                                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 3869
#define RSSPECAN_ATTR_VSA_RESULT_IQOF_CURRENT                                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 3870
#define RSSPECAN_ATTR_VSA_RESULT_IQOF_AVERAGE                                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 3871
#define RSSPECAN_ATTR_VSA_RESULT_IQOF_PEAK                                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 3872
#define RSSPECAN_ATTR_VSA_RESULT_IQOF_SDEV                                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 3873
#define RSSPECAN_ATTR_VSA_RESULT_IQOF_PCTL                                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 3874
#define RSSPECAN_ATTR_VSA_RESULT_IQIM_CURRENT                                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 3875
#define RSSPECAN_ATTR_VSA_RESULT_IQIM_AVERAGE                                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 3876
#define RSSPECAN_ATTR_VSA_RESULT_IQIM_PEAK                                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 3877
#define RSSPECAN_ATTR_VSA_RESULT_IQIM_SDEV                                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 3878
#define RSSPECAN_ATTR_VSA_RESULT_IQIM_PCTL                                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 3879
#define RSSPECAN_ATTR_VSA_RESULT_IQSK_CURRENT                                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 7211
#define RSSPECAN_ATTR_VSA_RESULT_IQSK_AVERAGE                                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 7212
#define RSSPECAN_ATTR_VSA_RESULT_IQSK_PEAK                                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 7213
#define RSSPECAN_ATTR_VSA_RESULT_IQSK_SDEV                                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 7214
#define RSSPECAN_ATTR_VSA_RESULT_IQSK_PCTL                                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 7215
#define RSSPECAN_ATTR_VSA_RESULT_GIMB_CURRENT                                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 3880
#define RSSPECAN_ATTR_VSA_RESULT_GIMB_AVERAGE                                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 3881
#define RSSPECAN_ATTR_VSA_RESULT_GIMB_PEAK                                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 3882
#define RSSPECAN_ATTR_VSA_RESULT_GIMB_CURRENT_SDEV                                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 3883
#define RSSPECAN_ATTR_VSA_RESULT_GIMB_PCTL                                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 3884
#define RSSPECAN_ATTR_VSA_RESULT_QERR_CURRENT                                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 3885
#define RSSPECAN_ATTR_VSA_RESULT_QERR_AVERAGE                                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 3886
#define RSSPECAN_ATTR_VSA_RESULT_QERR_PEAK                                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 3887
#define RSSPECAN_ATTR_VSA_RESULT_QERR_SDEV                                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 3888
#define RSSPECAN_ATTR_VSA_RESULT_QERR_PCTL                                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 3889
#define RSSPECAN_ATTR_VSA_RESULT_AMPLITUDE_DROOP_CURRENT                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 3890
#define RSSPECAN_ATTR_VSA_RESULT_AMPLITUDE_DROOP_AVERAGE                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 3891
#define RSSPECAN_ATTR_VSA_RESULT_AMPLITUDE_DROOP_PEAK                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 3892
#define RSSPECAN_ATTR_VSA_RESULT_AMPLITUDE_DROOP_SDEV                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 3893
#define RSSPECAN_ATTR_VSA_RESULT_AMPLITUDE_DROOP_PCTL                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 3894
#define RSSPECAN_ATTR_VSA_RESULT_POWER_CURRENT                                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 3895
#define RSSPECAN_ATTR_VSA_RESULT_POWER_AVERAGE                                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 3896
#define RSSPECAN_ATTR_VSA_RESULT_POWER_PEAK                                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 3897
#define RSSPECAN_ATTR_VSA_RESULT_POWER_SDEV                                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 3898
#define RSSPECAN_ATTR_VSA_RESULT_POWER_PCTL                                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 3899
#define RSSPECAN_ATTR_VSA_RESULT_FDER_CURRENT                                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 3900
#define RSSPECAN_ATTR_VSA_RESULT_FDER_AVERAGE                                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 3901
#define RSSPECAN_ATTR_VSA_RESULT_FDER_PEAK                                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 3902
#define RSSPECAN_ATTR_VSA_RESULT_FDER_SDEV                                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 3903
#define RSSPECAN_ATTR_VSA_RESULT_FDER_PCTL                                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 3904
#define RSSPECAN_ATTR_VSA_RESULT_FSK_CFDR_CURRENT                                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 3905
#define RSSPECAN_ATTR_VSA_RESULT_FSK_CFDR_AVERAGE                                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 3906
#define RSSPECAN_ATTR_VSA_RESULT_FSK_CFDR_PEAK                                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 3907
#define RSSPECAN_ATTR_VSA_RESULT_FSK_CFDR_SDEV                                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 3908
#define RSSPECAN_ATTR_VSA_RESULT_FSK_CFDR_CURRENT_PCTL                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 3909
#define RSSPECAN_ATTR_VSA_RESULT_FSK_DERR_RMS_CURRENT                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 3910
#define RSSPECAN_ATTR_VSA_RESULT_FSK_DERR_RMS_AVERAGE                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 3911
#define RSSPECAN_ATTR_VSA_RESULT_FSK_DERR_RMS_PEAK                                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 3912
#define RSSPECAN_ATTR_VSA_RESULT_FSK_DERR_RMS_SDEV                                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 3913
#define RSSPECAN_ATTR_VSA_RESULT_FSK_DERR_RMS_PCTL                                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 3914
#define RSSPECAN_ATTR_VSA_RESULT_FSK_DERR_PEAK_AVERAGE                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 3915
#define RSSPECAN_ATTR_VSA_RESULT_FSK_DERR_PEAK_PEAK                                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 3916
#define RSSPECAN_ATTR_VSA_RESULT_FSK_DERR_PEAK_SDEV                                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 3917
#define RSSPECAN_ATTR_VSA_RESULT_FSK_DERR_PEAK_PCTL                                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 3918
#define RSSPECAN_ATTR_VSA_RESULT_FSK_DERR_PEAK_CURRENT                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 3919
#define RSSPECAN_ATTR_VSA_RESULT_MDEV_CURRENT                                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 3920
#define RSSPECAN_ATTR_VSA_RESULT_MDEV_AVERAGE                                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 3921
#define RSSPECAN_ATTR_VSA_RESULT_MDEV_PEAK                                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 3922
#define RSSPECAN_ATTR_VSA_RESULT_MDEV_SDEV                                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 3923
#define RSSPECAN_ATTR_VSA_RESULT_MDEV_PCTL                                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 3924
#define RSSPECAN_ATTR_VSA_RESULT_RDEV_AVERAGE                                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 3925
#define RSSPECAN_ATTR_VSA_RESULT_RDEV_PEAK                                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 3926
#define RSSPECAN_ATTR_VSA_RESULT_RDEV_SDEV                                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 3927
#define RSSPECAN_ATTR_VSA_RESULT_RDEV_PCTL                                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 3928
#define RSSPECAN_ATTR_VSA_RESULT_RDEV_CURRENT                                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 3929
#define RSSPECAN_ATTR_VSA_LIMIT_MOD_ACC_LIMITS_ENABLED                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 3930
#define RSSPECAN_ATTR_VSA_LIMIT_MODULATION_ACCURACY_DEFAULTS                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 2649
#define RSSPECAN_ATTR_VSA_LIMIT_MOD_ACC_EVM_RMS_CURR                                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 3931
#define RSSPECAN_ATTR_VSA_LIMIT_MOD_ACC_EVM_RMS_CURR_VAL                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 3932
#define RSSPECAN_ATTR_VSA_QUERY_MOD_ACC_EVM_RMS_CURR_LIMIT_CHECK_RESULT                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 3933
#define RSSPECAN_ATTR_VSA_LIMIT_MOD_ACC_EVM_RMS_MEAN                                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 3934
#define RSSPECAN_ATTR_VSA_LIMIT_MOD_ACC_EVM_RMS_MEAN_VAL                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 3935
#define RSSPECAN_ATTR_VSA_QUERY_MOD_ACC_EVM_RMS_MEAN_LIMIT_CHECK_RESULT                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 3936
#define RSSPECAN_ATTR_VSA_LIMIT_MOD_ACC_EVM_RMS_PEAK                                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 3937
#define RSSPECAN_ATTR_VSA_LIMIT_MOD_ACC_EVM_RMS_PEAK_VAL                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 3938
#define RSSPECAN_ATTR_VSA_QUERY_MOD_ACC_EVM_RMS_PEAK_LIMIT_CHECK_RESULT                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 3939
#define RSSPECAN_ATTR_VSA_LIMIT_MOD_ACC_EVM_PEAK_CURR                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 3940
#define RSSPECAN_ATTR_VSA_LIMIT_MOD_ACC_EVM_PEAK_CURR_VAL                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 3941
#define RSSPECAN_ATTR_VSA_QUERY_MOD_ACC_EVM_PEAK_CURR_LIMIT_CHECK_RESULT                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 3942
#define RSSPECAN_ATTR_VSA_LIMIT_MOD_ACC_EVM_PEAK_MEAN                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 3943
#define RSSPECAN_ATTR_VSA_LIMIT_MOD_ACC_EVM_PEAK_MEAN_VAL                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 3944
#define RSSPECAN_ATTR_VSA_QUERY_MOD_ACC_EVM_PEAK_MEAN_LIMIT_CHECK_RESULT                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 3945
#define RSSPECAN_ATTR_VSA_LIMIT_MOD_ACC_EVM_PEAK_PEAK                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 3946
#define RSSPECAN_ATTR_VSA_LIMIT_MOD_ACC_EVM_PEAK_PEAK_VAL                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 3947
#define RSSPECAN_ATTR_VSA_QUERY_MOD_ACC_EVM_PEAK_PEAK_LIMIT_CHECK_RESULT                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 3948
#define RSSPECAN_ATTR_VSA_LIMIT_MOD_ACC_PERR_RMS_CURR                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 3949
#define RSSPECAN_ATTR_VSA_LIMIT_MOD_ACC_PHASE_ERROR_RMS_CURR_VAL                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 3950
#define RSSPECAN_ATTR_VSA_QUERY_MOD_ACC_PERR_RMS_CURR_LIMIT_CHECK_RESULT                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 3951
#define RSSPECAN_ATTR_VSA_LIMIT_MOD_ACC_PERR_RMS_MEAN                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 3952
#define RSSPECAN_ATTR_VSA_LIMIT_MOD_ACC_PHASE_ERROR_RMS_MEAN_VAL                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 3953
#define RSSPECAN_ATTR_VSA_QUERY_MOD_ACC_PERR_RMS_MEAN_LIMIT_CHECK_RESULT                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 3954
#define RSSPECAN_ATTR_VSA_LIMIT_MOD_ACC_PERR_RMS_PEAK                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 3955
#define RSSPECAN_ATTR_VSA_LIMIT_MOD_ACC_PHASE_ERROR_RMS_PEAK_VAL                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 3956
#define RSSPECAN_ATTR_VSA_QUERY_MOD_ACC_PERR_RMS_PEAK_LIMIT_CHECK_RESULT                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 3957
#define RSSPECAN_ATTR_VSA_LIMIT_MOD_ACC_PERR_PEAK_CURR                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 3958
#define RSSPECAN_ATTR_VSA_LIMIT_MOD_ACC_PHASE_ERROR_PEAK_CURR_VAL                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 3959
#define RSSPECAN_ATTR_VSA_QUERY_MOD_ACC_PERR_PEAK_CURR_LIMIT_CHECK_RESULT                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 3960
#define RSSPECAN_ATTR_VSA_LIMIT_MOD_ACC_PERR_PEAK_MEAN                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 3961
#define RSSPECAN_ATTR_VSA_LIMIT_MOD_ACC_PHASE_ERROR_PEAK_MEAN_VAL                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 3962
#define RSSPECAN_ATTR_VSA_QUERY_MOD_ACC_PERR_PEAK_MEAN_LIMIT_CHECK_RESULT                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 3963
#define RSSPECAN_ATTR_VSA_LIMIT_MOD_ACC_PERR_PEAK_PEAK                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 3964
#define RSSPECAN_ATTR_VSA_LIMIT_MOD_ACC_PHASE_ERROR_PEAK_PEAK_VAL                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 3965
#define RSSPECAN_ATTR_VSA_QUERY_MOD_ACC_PERR_PEAK_PEAK_LIMIT_CHECK_RESULT                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 3966
#define RSSPECAN_ATTR_VSA_LIMIT_MOD_ACC_FERR_RMS_CURR                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 3967
#define RSSPECAN_ATTR_VSA_LIMIT_MOD_ACC_FERR_RMS_CURR_VAL                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 3968
#define RSSPECAN_ATTR_VSA_QUERY_MOD_ACC_FERR_RMS_CURR_LIMIT_CHECK_RESULT                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 3969
#define RSSPECAN_ATTR_VSA_LIMIT_MOD_ACC_FERR_RMS_MEAN                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 3970
#define RSSPECAN_ATTR_VSA_LIMIT_MOD_ACC_FERR_RMS_MEAN_VAL                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 3971
#define RSSPECAN_ATTR_VSA_QUERY_MOD_ACC_FERR_RMS_MEAN_LIMIT_CHECK_RESULT                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 3972
#define RSSPECAN_ATTR_VSA_LIMIT_MOD_ACC_FERR_RMS_PEAK                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 3973
#define RSSPECAN_ATTR_VSA_LIMIT_MOD_ACC_FERR_RMS_PEAK_VAL                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 3974
#define RSSPECAN_ATTR_VSA_QUERY_MOD_ACC_FERR_RMS_PEAK_LIMIT_CHECK_RESULT                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 3975
#define RSSPECAN_ATTR_VSA_LIMIT_MOD_ACC_FERR_PEAK_CURR                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 3976
#define RSSPECAN_ATTR_VSA_LIMIT_MOD_ACC_FERR_PEAK_CURR_VAL                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 3977
#define RSSPECAN_ATTR_VSA_QUERY_MOD_ACC_FERR_PEAK_CURR_LIMIT_CHECK_RESULT                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 3978
#define RSSPECAN_ATTR_VSA_LIMIT_MOD_ACC_FERR_PEAK_MEAN                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 3979
#define RSSPECAN_ATTR_VSA_LIMIT_MOD_ACC_FERR_PEAK_MEAN_VAL                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 3980
#define RSSPECAN_ATTR_VSA_QUERY_MOD_ACC_FERR_PEAK_MEAN_LIMIT_CHECK_RESULT                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 3981
#define RSSPECAN_ATTR_VSA_LIMIT_MOD_ACC_FERR_PEAK_PEAK                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 3982
#define RSSPECAN_ATTR_VSA_LIMIT_MOD_ACC_FERR_PEAK_PEAK_VAL                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 3983
#define RSSPECAN_ATTR_VSA_QUERY_MOD_ACC_FERR_PEAK_PEAK_LIMIT_CHECK_RESULT                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 3984
#define RSSPECAN_ATTR_VSA_LIMIT_MOD_ACC_MERR_RMS_CURR                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 3985
#define RSSPECAN_ATTR_VSA_LIMIT_MOD_ACC_MAGNITUDE_ERROR_RMS_CURR_VAL                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 3986
#define RSSPECAN_ATTR_VSA_QUERY_MOD_ACC_MERR_RMS_CURR_LIMIT_CHECK_RESULT                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 3987
#define RSSPECAN_ATTR_VSA_LIMIT_MOD_ACC_MERR_RMS_MEAN                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 3988
#define RSSPECAN_ATTR_VSA_LIMIT_MOD_ACC_MAGNITUDE_ERROR_RMS_MEAN_VAL                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 3989
#define RSSPECAN_ATTR_VSA_QUERY_MOD_ACC_MERR_RMS_MEAN_LIMIT_CHECK_RESULT                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 3990
#define RSSPECAN_ATTR_VSA_LIMIT_MOD_ACC_MERR_RMS_PEAK                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 3991
#define RSSPECAN_ATTR_VSA_LIMIT_MOD_ACC_MAGNITUDE_ERROR_RMS_PEAK_VAL                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 3992
#define RSSPECAN_ATTR_VSA_QUERY_MOD_ACC_MERR_RMS_PEAK_LIMIT_CHECK_RESULT                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 3993
#define RSSPECAN_ATTR_VSA_LIMIT_MOD_ACC_MERR_PEAK_CURR                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 3994
#define RSSPECAN_ATTR_VSA_LIMIT_MOD_ACC_MAGNITUDE_ERROR_PEAK_CURR_VAL                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 3995
#define RSSPECAN_ATTR_VSA_QUERY_MOD_ACC_MERR_PEAK_CURR_LIMIT_CHECK_RESULT                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 3996
#define RSSPECAN_ATTR_VSA_LIMIT_MOD_ACC_MERR_PEAK_MEAN                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 3997
#define RSSPECAN_ATTR_VSA_LIMIT_MOD_ACC_MAGNITUDE_ERROR_PEAK_MEAN_VAL                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 3998
#define RSSPECAN_ATTR_VSA_QUERY_MOD_ACC_MERR_PEAK_MEAN_LIMIT_CHECK_RESULT                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 3999
#define RSSPECAN_ATTR_VSA_LIMIT_MOD_ACC_MERR_PEAK_PEAK                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 4000
#define RSSPECAN_ATTR_VSA_LIMIT_MOD_ACC_MAGNITUDE_ERROR_PEAK_PEAK_VAL                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 4001
#define RSSPECAN_ATTR_VSA_QUERY_MOD_ACC_MERR_PEAK_PEAK_LIMIT_CHECK_RESULT                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 4002
#define RSSPECAN_ATTR_VSA_LIMIT_MOD_ACC_CFER_CURR                                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 4003
#define RSSPECAN_ATTR_VSA_LIMIT_MOD_ACC_CARRIER_FREQUENCY_ERROR_CURR_VAL                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 4004
#define RSSPECAN_ATTR_VSA_QUERY_MOD_ACC_CFER_CURR_LIMIT_CHECK_RESULT                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 4005
#define RSSPECAN_ATTR_VSA_LIMIT_MOD_ACC_CFER_MEAN                                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 4006
#define RSSPECAN_ATTR_VSA_LIMIT_MOD_ACC_CARRIER_FREQUENCY_ERROR_MEAN_VAL                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 4007
#define RSSPECAN_ATTR_VSA_QUERY_MOD_ACC_CFER_MEAN_LIMIT_CHECK_RESULT                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 4008
#define RSSPECAN_ATTR_VSA_LIMIT_MOD_ACC_CFER_PEAK                                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 4009
#define RSSPECAN_ATTR_VSA_LIMIT_MOD_ACC_CARRIER_FREQUENCY_ERROR_PEAK_VAL                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 4010
#define RSSPECAN_ATTR_VSA_QUERY_MOD_ACC_CFER_PEAK_LIMIT_CHECK_RESULT                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 4011
#define RSSPECAN_ATTR_VSA_LIMIT_MOD_ACC_RHO_CURR                                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 4012
#define RSSPECAN_ATTR_VSA_LIMIT_MOD_ACC_RHO_CURR_VAL                                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 4013
#define RSSPECAN_ATTR_VSA_QUERY_MOD_ACC_RHO_CURR_LIMIT_CHECK_RESULT                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 4014
#define RSSPECAN_ATTR_VSA_LIMIT_MOD_ACC_RHO_MEAN                                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 4015
#define RSSPECAN_ATTR_VSA_LIMIT_MOD_ACC_RHO_MEAN_VAL                                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 4016
#define RSSPECAN_ATTR_VSA_QUERY_MOD_ACC_RHO_MEAN_LIMIT_CHECK_RESULT                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 4017
#define RSSPECAN_ATTR_VSA_LIMIT_MOD_ACC_RHO_PEAK                                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 4018
#define RSSPECAN_ATTR_VSA_LIMIT_MOD_ACC_RHO_PEAK_VAL                                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 4019
#define RSSPECAN_ATTR_VSA_QUERY_MOD_ACC_RHO_PEAK_LIMIT_CHECK_RESULT                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 4020
#define RSSPECAN_ATTR_VSA_LIMIT_MOD_ACC_IQOF_CURR                                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 4021
#define RSSPECAN_ATTR_VSA_LIMIT_MOD_ACC_IQ_OFFSET_CURR_VAL                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 4022
#define RSSPECAN_ATTR_VSA_QUERY_MOD_ACC_IQ_OFFSET_CURR_LIMIT_CHECK_RESULT                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 4023
#define RSSPECAN_ATTR_VSA_LIMIT_MOD_ACC_IQOF_MEAN                                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 4024
#define RSSPECAN_ATTR_VSA_LIMIT_MOD_ACC_IQ_OFFSET_MEAN_VAL                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 4025
#define RSSPECAN_ATTR_VSA_QUERY_MOD_ACC_IQ_OFFSET_MEAN_LIMIT_CHECK_RESULT                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 4026
#define RSSPECAN_ATTR_VSA_LIMIT_MOD_ACC_IQOF_PEAK                                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 4027
#define RSSPECAN_ATTR_VSA_LIMIT_MOD_ACC_IQ_OFFSET_PEAK_VAL                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 4028
#define RSSPECAN_ATTR_VSA_QUERY_MOD_ACC_IQ_OFFSET_PEAK_LIMIT_CHECK_RESULT                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 4029
#define RSSPECAN_ATTR_VSA_LIMIT_MOD_ACC_FDER_CURR                                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 4030
#define RSSPECAN_ATTR_VSA_LIMIT_MOD_ACC_FREQUENCY_DEVIATION_CURR_VAL                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 4031
#define RSSPECAN_ATTR_VSA_QUERY_MOD_ACC_FDER_CURR_LIMIT_CHECK_RESULT                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 4032
#define RSSPECAN_ATTR_VSA_LIMIT_MOD_ACC_FDER_MEAN                                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 4033
#define RSSPECAN_ATTR_VSA_LIMIT_MOD_ACC_FREQUENCY_DEVIATION_MEAN_VAL                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 4034
#define RSSPECAN_ATTR_VSA_QUERY_MOD_ACC_FDER_MEAN_LIMIT_CHECK_RESULT                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 4035
#define RSSPECAN_ATTR_VSA_LIMIT_MOD_ACC_FDER_PEAK                                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 4036
#define RSSPECAN_ATTR_VSA_LIMIT_MOD_ACC_FREQUENCY_DEVIATION_PEAK_VAL                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 4037
#define RSSPECAN_ATTR_VSA_QUERY_MOD_ACC_FDER_PEAK_LIMIT_CHECK_RESULT                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 4038
#define RSSPECAN_ATTR_VSA_EYE_DIAGRAM_VERTICAL_LIMIT_LINE_ABSOLUTE_ENABLED                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 6202
#define RSSPECAN_ATTR_VSA_EYE_DIAGRAM_VERTICAL_LIMIT_LINE_ABSOLUTE                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 6203
#define RSSPECAN_ATTR_VSA_EYE_DIAGRAM_VERTICAL_LIMIT_LINE_RELATIVE_ENABLED                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 6204
#define RSSPECAN_ATTR_VSA_EYE_DIAGRAM_VERTICAL_LIMIT_LINE_RELATIVE                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 6205
#define RSSPECAN_ATTR_VSA_EYE_DIAGRAM_HORIZONTAL_LIMIT_LINE_ABSOLUTE_ENABLED                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 6206
#define RSSPECAN_ATTR_VSA_EYE_DIAGRAM_HORIZONTAL_LIMIT_LINE_ABSOLUTE                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 6207
#define RSSPECAN_ATTR_VSA_EYE_DIAGRAM_HORIZONTAL_LIMIT_LINE_RELATIVE_ENABLED                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 6208
#define RSSPECAN_ATTR_VSA_EYE_DIAGRAM_HORIZONTAL_LIMIT_LINE_RELATIVE                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 6209
#define RSSPECAN_ATTR_VSA_BER_BIT_ERROR_RATE_CURRENT                                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 4039
#define RSSPECAN_ATTR_VSA_BER_BIT_ERROR_RATE_MIN                                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 4040
#define RSSPECAN_ATTR_VSA_BER_BIT_ERROR_RATE_MAX                                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 4041
#define RSSPECAN_ATTR_VSA_BER_BIT_ERROR_RATE_ACC                                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 4042
#define RSSPECAN_ATTR_VSA_BER_NUMBER_OF_ERRORS_CURRENT                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 4043
#define RSSPECAN_ATTR_VSA_BER_NUMBER_OF_ERRORS_MIN                                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 4044
#define RSSPECAN_ATTR_VSA_BER_NUMBER_OF_ERRORS_MAX                                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 4045
#define RSSPECAN_ATTR_VSA_BER_NUMBER_OF_ERRORS_ACC                                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 4046
#define RSSPECAN_ATTR_VSA_BER_NUMBER_OF_BITS_CURRENT                                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 4047
#define RSSPECAN_ATTR_VSA_BER_NUMBER_OF_BITS_MIN                                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 4048
#define RSSPECAN_ATTR_VSA_BER_NUMBER_OF_BITS_MAX                                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 4049
#define RSSPECAN_ATTR_VSA_BER_NUMBER_OF_BITS_ACC                                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 4050
#define RSSPECAN_ATTR_VSA_BER_NUMBER_OF_SYMBOL_ERRORS_CURRENT                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 7767
#define RSSPECAN_ATTR_VSA_BER_NUMBER_OF_SYMBOL_ERRORS_MINIMUM                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 7768
#define RSSPECAN_ATTR_VSA_BER_NUMBER_OF_SYMBOL_ERRORS_MAXIMUM                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 7769
#define RSSPECAN_ATTR_VSA_BER_NUMBER_OF_SYMBOL_ERRORS_ACCUMULATED                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 7770
#define RSSPECAN_ATTR_DSP_BURST_COUNT                                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 7771
#define RSSPECAN_ATTR_DSP_BURST_LENGTH                                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 7772
#define RSSPECAN_ATTR_DSP_BURST_PRESENT                                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 7773
#define RSSPECAN_ATTR_DSP_BURST_START                                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 7774
#define RSSPECAN_ATTR_DSP_PATTERN_COUNT                                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 7775
#define RSSPECAN_ATTR_DSP_PATTERN_CONFIDENCE                                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 7776
#define RSSPECAN_ATTR_DSP_PATTERN_CORRECTNESS                                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 7777
#define RSSPECAN_ATTR_DSP_PATTERN_START                                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 7778
#define RSSPECAN_ATTR_DSP_PATTERN_PRESENT                                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 7779
#define RSSPECAN_ATTR_VSA_X_FIRST_VALUE_QUERY                                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 1701
#define RSSPECAN_ATTR_VSA_X_LAST_VALUE_QUERY                                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 5182
#define RSSPECAN_ATTR_VSA_REFRESH_CAPTURED_DATA                                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 4051
#define RSSPECAN_ATTR_VSA_CURRENT_RESULT_RANGE_START_VALUE                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 5183
#define RSSPECAN_ATTR_OFDM_VSA_MODE                                                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 10922
#define RSSPECAN_ATTR_OFDM_VSA_USE_CONFIGURATION_FILE                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 10923
#define RSSPECAN_ATTR_OFDM_VSA_LOAD_CONFIGURATION_FILE                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 10924
#define RSSPECAN_ATTR_OFDM_VSA_PHASE_COMPENSATION_ENABLED                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 10925
#define RSSPECAN_ATTR_OFDM_VSA_PHASE_COMPENSATION_MODE                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 10926
#define RSSPECAN_ATTR_OFDM_VSA_PHASE_COMPENSATION_FREQUENCY                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 10927
#define RSSPECAN_ATTR_OFDM_VSA_PREAMBLE_BLOCK_LENGTH                                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 10928
#define RSSPECAN_ATTR_OFDM_VSA_FRAME_START_OFFSET                                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 10929
#define RSSPECAN_ATTR_OFDM_VSA_TRANSFORM_PRECODING_ENABLED                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 10930
#define RSSPECAN_ATTR_OFDM_VSA_TRANSFORM_PRECODING_IGNORE                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 10931
#define RSSPECAN_ATTR_OFDM_VSA_FFT_SIZE                                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 10932
#define RSSPECAN_ATTR_OFDM_VSA_CYCLIC_PREFIX_MODE                                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 10933
#define RSSPECAN_ATTR_OFDM_VSA_CYCLIC_PREFIX_PERIODIC_RANGE_ENABLED                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 10934
#define RSSPECAN_ATTR_OFDM_VSA_CYCLIC_PREFIX_SYMBOL_COUNT                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 10935
#define RSSPECAN_ATTR_OFDM_VSA_CYCLIC_PREFIX_LENGTH                                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 10936
#define RSSPECAN_ATTR_OFDM_VSA_HIGHPASS_FILTER_ENABLED                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 10937
#define RSSPECAN_ATTR_OFDM_VSA_HIGHPASS_FILTER_6_DB_FREQUENCY                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 10938
#define RSSPECAN_ATTR_OFDM_VSA_HIGHPASS_FILTER_50_DB_FREQUENCY                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 10939
#define RSSPECAN_ATTR_OFDM_VSA_LOWPASS_FILTER_ENABLED                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 10940
#define RSSPECAN_ATTR_OFDM_VSA_LOWPASS_FILTER_6_DB_FREQUENCY                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 10941
#define RSSPECAN_ATTR_OFDM_VSA_LOWPASS_FILTER_50_DB_FREQUENCY                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 10942
#define RSSPECAN_ATTR_OFDM_VSA_BURST_SEARCH_ENABLED                                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 10943
#define RSSPECAN_ATTR_OFDM_VSA_MAXIMUM_FRAMES                                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 10944
#define RSSPECAN_ATTR_OFDM_VSA_RESULT_LENGTH_SYMBOLS                                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 10945
#define RSSPECAN_ATTR_OFDM_VSA_SYNCHRONIZATION_TIME_MODE                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 10946
#define RSSPECAN_ATTR_OFDM_VSA_SYNCHRONIZATION_PARAMETER_ESTIMATION                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 10947
#define RSSPECAN_ATTR_OFDM_VSA_SYNCHRONIZATION_MODULATION_DETECTION                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 10948
#define RSSPECAN_ATTR_OFDM_VSA_THRESHOLDS_MINIMUM_FRAME                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 10949
#define RSSPECAN_ATTR_OFDM_VSA_THRESHOLDS_MINIMUM_TIME                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 10950
#define RSSPECAN_ATTR_OFDM_VSA_DEMODULATION_FFT_SHIFT                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 10951
#define RSSPECAN_ATTR_OFDM_VSA_DEMODULATION_MAXIMUM_CARRIER_OFFSET                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 10952
#define RSSPECAN_ATTR_OFDM_VSA_DEMODULATION_CYCLIC_DELAY                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 10953
#define RSSPECAN_ATTR_OFDM_VSA_TRACKING_PHASE_ENABLED                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 10954
#define RSSPECAN_ATTR_OFDM_VSA_TRACKING_TIMING_ENABLED                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 10955
#define RSSPECAN_ATTR_OFDM_VSA_TRACKING_LEVEL_ENABLED                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 10956
#define RSSPECAN_ATTR_OFDM_VSA_TRACKING_CHANNEL_COMPENSATION_ENABLED                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 10957
#define RSSPECAN_ATTR_OFDM_VSA_CONSTELLATION_DIAGRAM_SPECIFIED_CARRIER                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 10958
#define RSSPECAN_ATTR_OFDM_VSA_CONSTELLATION_DIAGRAM_CARRIERS_ALL                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 10959
#define RSSPECAN_ATTR_OFDM_VSA_CONSTELLATION_DIAGRAM_SELECTED_MODULATION                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 10960
#define RSSPECAN_ATTR_OFDM_VSA_CONSTELLATION_DIAGRAM_MODULATION_ALL                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 10961
#define RSSPECAN_ATTR_OFDM_VSA_CONSTELLATION_DIAGRAM_MODULATION_TYPE                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 10962
#define RSSPECAN_ATTR_OFDM_VSA_CONSTELLATION_DIAGRAM_SPECIFIED_SYMBOL_NUMBER                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 10963
#define RSSPECAN_ATTR_OFDM_VSA_CONSTELLATION_DIAGRAM_SYMBOL_NUMBER_ALL                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 10964
#define RSSPECAN_ATTR_OFDM_VSA_EVM_AVERAGING_METHOD                                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 10965
#define RSSPECAN_ATTR_OFDM_VSA_EVM_NORMALIZATION_METHOD                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 10966
#define RSSPECAN_ATTR_OFDM_VSA_UNITS_CARRIER                                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 10967
#define RSSPECAN_ATTR_OFDM_VSA_UNITS_SYMBOL                                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 10968
#define RSSPECAN_ATTR_OFDM_VSA_UNITS_POWER_SPECTRUM                                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 10969
#define RSSPECAN_ATTR_OFDM_VSA_UNITS_IMPULSE_RESPONSE                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 10970
#define RSSPECAN_ATTR_OFDM_VSA_UNITS_MAGNITUDE_CAPTURE                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 10971
#define RSSPECAN_ATTR_OFDM_VSA_UNITS_EVM                                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 10972
#define RSSPECAN_ATTR_CONFIGURE_OFDM_VSA_BITSTREAM_DISPLAY_FORMAT                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 10973
#define RSSPECAN_ATTR_FETCH_OFDM_VSA_ALL_STATES                                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 10974
#define RSSPECAN_ATTR_FETCH_OFDM_VSA_BURST_DETECTION_STATE                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 10975
#define RSSPECAN_ATTR_FETCH_OFDM_VSA_COMPENSATION_STATE                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 10976
#define RSSPECAN_ATTR_FETCH_OFDM_VSA_DATA_AIDED_ESTIMATION_STATE                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 10977
#define RSSPECAN_ATTR_FETCH_OFDM_VSA_PILOT_AIDED_ESTIMATION_STATE                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 10978
#define RSSPECAN_ATTR_FETCH_OFDM_VSA_EVM_MEASUREMENT_STATE                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 10979
#define RSSPECAN_ATTR_FETCH_OFDM_VSA_MODULATION_DETECTION_STATE                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 10980
#define RSSPECAN_ATTR_FETCH_OFDM_VSA_FRAME_SYNCHRONIZATION_STATE                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 10981
#define RSSPECAN_ATTR_FETCH_OFDM_VSA_FRAME_SYNCHRONIZATION                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 10982
#define RSSPECAN_ATTR_FETCH_OFDM_VSA_TIME_SYNCHRONIZATION_STATE                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 10983
#define RSSPECAN_ATTR_FETCH_OFDM_VSA_TIME_SYNCHRONIZATION                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 10984
#define RSSPECAN_ATTR_FETCH_OFDM_VSA_TRIGGER_TO_SYNC                                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 10985
#define RSSPECAN_ATTR_FETCH_OFDM_VSA_BURST_COUNT                                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 10986
#define RSSPECAN_ATTR_FETCH_OFDM_VSA_BURST_LENGTH                                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 10987
#define RSSPECAN_ATTR_FETCH_OFDM_VSA_BURST_OFFSET                                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 10988
#define RSSPECAN_ATTR_FETCH_OFDM_VSA_ALL_RESULTS                                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 10989
#define RSSPECAN_ATTR_FETCH_OFDM_VSA_CREST_FACTOR_MAXIMUM                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 10990
#define RSSPECAN_ATTR_FETCH_OFDM_VSA_CREST_FACTOR_MINIMUM                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 10991
#define RSSPECAN_ATTR_FETCH_OFDM_VSA_CREST_FACTOR_AVERAGE                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 10992
#define RSSPECAN_ATTR_FETCH_OFDM_VSA_EVM_DATA_MAXIMUM                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 10993
#define RSSPECAN_ATTR_FETCH_OFDM_VSA_EVM_DATA_MINIMUM                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 10994
#define RSSPECAN_ATTR_FETCH_OFDM_VSA_EVM_DATA_AVERAGE                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 10995
#define RSSPECAN_ATTR_FETCH_OFDM_VSA_EVM_PILOT_MAXIMUM                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 10996
#define RSSPECAN_ATTR_FETCH_OFDM_VSA_EVM_PILOT_MINIMUM                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 10997
#define RSSPECAN_ATTR_FETCH_OFDM_VSA_EVM_PILOT_AVERAGE                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 10998
#define RSSPECAN_ATTR_FETCH_OFDM_VSA_EVM_PILOT_PERCENT_AVERAGE                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 10999
#define RSSPECAN_ATTR_FETCH_OFDM_VSA_EVM_ALL_PERCENT_MAXIMUM                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 11000
#define RSSPECAN_ATTR_FETCH_OFDM_VSA_EVM_ALL_PERCENT_MINIMUM                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 11001
#define RSSPECAN_ATTR_FETCH_OFDM_VSA_EVM_ALL_PERCENT_AVERAGE                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 11002
#define RSSPECAN_ATTR_FETCH_OFDM_VSA_EVM_PEAK_DATA_MAXIMUM                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 11003
#define RSSPECAN_ATTR_FETCH_OFDM_VSA_EVM_PEAK_DATA_PERCENT_MAXIMUM                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 11004
#define RSSPECAN_ATTR_FETCH_OFDM_VSA_EVM_PEAK_PILOT_MAXIMUM                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 11005
#define RSSPECAN_ATTR_FETCH_OFDM_VSA_EVM_PEAK_PILOT_PERCENT_MAXIMUM                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 11006
#define RSSPECAN_ATTR_FETCH_OFDM_VSA_EVM_PEAK_ALL_MAXIMUM                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 11007
#define RSSPECAN_ATTR_FETCH_OFDM_VSA_EVM_PEAK_ALL_PERCENT_MAXIMUM                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 11008
#define RSSPECAN_ATTR_FETCH_OFDM_VSA_FREQUENCY_ERROR_MAXIMUM                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 11009
#define RSSPECAN_ATTR_FETCH_OFDM_VSA_FREQUENCY_ERROR_MINIMUM                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 11010
#define RSSPECAN_ATTR_FETCH_OFDM_VSA_FREQUENCY_ERROR_AVERAGE                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 11011
#define RSSPECAN_ATTR_FETCH_OFDM_VSA_SAMPLING_ERROR_MAXIMUM                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 11012
#define RSSPECAN_ATTR_FETCH_OFDM_VSA_SAMPLING_ERROR_MINIMUM                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 11013
#define RSSPECAN_ATTR_FETCH_OFDM_VSA_SAMPLING_ERROR_AVERAGE                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 11014
#define RSSPECAN_ATTR_FETCH_OFDM_VSA_QUADRATURE_ERROR_MAXIMUM                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 11015
#define RSSPECAN_ATTR_FETCH_OFDM_VSA_QUADRATURE_ERROR_MINIMUM                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 11016
#define RSSPECAN_ATTR_FETCH_OFDM_VSA_QUADRATURE_ERROR_AVERAGE                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 11017
#define RSSPECAN_ATTR_FETCH_OFDM_VSA_MODULATION_ERROR_MAXIMUM                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 11018
#define RSSPECAN_ATTR_FETCH_OFDM_VSA_MODULATION_ERROR_MINIMUM                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 11019
#define RSSPECAN_ATTR_FETCH_OFDM_VSA_MODULATION_ERROR_AVERAGE                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 11020
#define RSSPECAN_ATTR_FETCH_OFDM_VSA_GAIN_IMBALANCE_MAXIMUM                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 11021
#define RSSPECAN_ATTR_FETCH_OFDM_VSA_GAIN_IMBALANCE_MINIMUM                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 11022
#define RSSPECAN_ATTR_FETCH_OFDM_VSA_GAIN_IMBALANCE_AVERAGE                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 11023
#define RSSPECAN_ATTR_FETCH_OFDM_VSA_IQ_OFFSET_MAXIMUM                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 11024
#define RSSPECAN_ATTR_FETCH_OFDM_VSA_IQ_OFFSET_MINIMUM                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 11025
#define RSSPECAN_ATTR_FETCH_OFDM_VSA_IQ_OFFSET_AVERAGE                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 11026
#define RSSPECAN_ATTR_FETCH_OFDM_VSA_POWER_MAXIMUM                                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 11027
#define RSSPECAN_ATTR_FETCH_OFDM_VSA_POWER_MINIMUM                                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 11028
#define RSSPECAN_ATTR_FETCH_OFDM_VSA_POWER_AVERAGE                                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 11029
#define RSSPECAN_ATTR_WLAN_MODE                                                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 1957
#define RSSPECAN_ATTR_WLAN_ADJUST_REFERENCE_LEVEL_DURATION                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 11133
#define RSSPECAN_ATTR_WLAN_ADJUST_REFERENCE_LEVEL_DURATION_MODE                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 11134
#define RSSPECAN_ATTR_WLAN_STAN                                                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 3012
#define RSSPECAN_ATTR_WLAN_SWAP_IQ                                                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 3014
#define RSSPECAN_ATTR_WLAN_SRAT                                                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 3013
#define RSSPECAN_ATTR_WLAN_RBW_FILTER                                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 3015
#define RSSPECAN_ATTR_WLAN_SIGS                                                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 2033
#define RSSPECAN_ATTR_WLAN_BTYP                                                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 1934
#define RSSPECAN_ATTR_WLAN_CHAN                                                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 1940
#define RSSPECAN_ATTR_WLAN_TOLERANCE_LIMIT                                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 5050
#define RSSPECAN_ATTR_WLAN_CHANNEL_BANDWIDTH_AUTO                                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 7016
#define RSSPECAN_ATTR_WLAN_STANDARD_VERSION_ERROR_VECTOR_MAGNITUDE                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 7017
#define RSSPECAN_ATTR_WLAN_POWER_OPTIMISATION                                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 11135
#define RSSPECAN_ATTR_WLAN_POW_EXP_IQ                                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 1936
#define RSSPECAN_ATTR_WLAN_POW_EXP_RF                                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 1937
#define RSSPECAN_ATTR_WLAN_IQ_FILE_AUTO_INPUT_ENABLED                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 7019
#define RSSPECAN_ATTR_WLAN_IQ_FILE_SELECTED_DATA_STREAM                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 7020
#define RSSPECAN_ATTR_WLAN_IQ_FILE_CHANNEL_LIST                                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 7021
#define RSSPECAN_ATTR_WLAN_IQ_FILE_REPETITION_COUNT                                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 7022
#define RSSPECAN_ATTR_WLAN_TRIG_MODE                                                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 2029
#define RSSPECAN_ATTR_WLAN_TRIG_POW_LEV                                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 2030
#define RSSPECAN_ATTR_WLAN_TRIG_POW_LEV_AUTO                                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 2031
#define RSSPECAN_ATTR_WLAN_POWER_INTERVAL_SEARCH                                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 3009
#define RSSPECAN_ATTR_WLAN_FFT_START_OFFSET                                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 3016
#define RSSPECAN_ATTR_WLAN_CEST                                                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 3017
#define RSSPECAN_ATTR_WLAN_INTERPOLATION_TYPE                                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 7780
#define RSSPECAN_ATTR_WLAN_PEAMBLE_CHANNEL_ESTIMATION                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 6761
#define RSSPECAN_ATTR_WLAN_TRAC_PHAS                                                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 3019
#define RSSPECAN_ATTR_WLAN_TRAC_TIME                                                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 3020
#define RSSPECAN_ATTR_WLAN_TRAC_LEV                                                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 3018
#define RSSPECAN_ATTR_WLAN_IQ_COMPENSATION                                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 3150
#define RSSPECAN_ATTR_WLAN_COMPENSATE_CROSSTALK                                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 7023
#define RSSPECAN_ATTR_WLAN_TRAC_PILOTS                                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 3021
#define RSSPECAN_ATTR_WLAN_TRACKING_WIENER_RELATIVE_DELAY_SPREAD_ENABLED                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 11136
#define RSSPECAN_ATTR_WLAN_TRACKING_WIENER_RELATIVE_DELAY_SPREAD                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 11137
#define RSSPECAN_ATTR_WLAN_BAN                                                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 2035
#define RSSPECAN_ATTR_WLAN_BCON_AUTO                                                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 3022
#define RSSPECAN_ATTR_WLAN_MCS_INDEX_TO_USE_FBUR                                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 3023
#define RSSPECAN_ATTR_WLAN_MCS_INDEX                                                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 3024
#define RSSPECAN_ATTR_WLAN_STBC                                                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 3025
#define RSSPECAN_ATTR_WLAN_GUARD_INTERVAL_LENGTH                                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 3027
#define RSSPECAN_ATTR_WLAN_GUARD_INTERVAL_LENGTH_AUTO_ENABLED                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 7024
#define RSSPECAN_ATTR_WLAN_GUARD_INTERVAL_LENGTH_GUARD_TIME                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 7025
#define RSSPECAN_ATTR_WLAN_DEMODULATION_DATA_MODE                                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 11138
#define RSSPECAN_ATTR_WLAN_DEMODULATION_DATA_BIT_RATE                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 11139
#define RSSPECAN_ATTR_WLAN_PPDU_FORMAT_TO_MEASURE                                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 3010
#define RSSPECAN_ATTR_WLAN_CHANNEL_BANDWIDTH_TO_MEASURE                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 3031
#define RSSPECAN_ATTR_WLAN_PSDU_MODULATION                                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 3011
#define RSSPECAN_ATTR_WLAN_PPDU_FORMAT_TO_MEASURE_802_11AC                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 3033
#define RSSPECAN_ATTR_WLAN_CHANNEL_BANDWIDTH_TO_MEASURE_802_11AC                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 3034
#define RSSPECAN_ATTR_WLAN_NSTS_INDEX_MODE                                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 3035
#define RSSPECAN_ATTR_WLAN_NSTS_INDEX                                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 3036
#define RSSPECAN_ATTR_WLAN_GAIN_ENABLED                                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 7536
#define RSSPECAN_ATTR_WLAN_PPDU_FORMAT_TO_MEASURE_802_11B_G                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 3097
#define RSSPECAN_ATTR_WLAN_PSDU_MODULATION_802_11B_G                                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 3096
#define RSSPECAN_ATTR_WLAN_EQUALIZER_FILTER_LENGTH                                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 2646
#define RSSPECAN_ATTR_WLAN_PPDU_FORMAT_TO_MEASURE_802_11N                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 3029
#define RSSPECAN_ATTR_WLAN_CHANNEL_BANDWIDTH_TO_MEASURE_802_11N                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 3030
#define RSSPECAN_ATTR_WLAN_MIMO_PAYLOAD_LENGTH_SOURCE                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 2892
#define RSSPECAN_ATTR_WLAN_EXT_SPATIAL_STREAMS                                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 3032
#define RSSPECAN_ATTR_WLAN_PPDU_FORMAT_TO_MEASURE_802_11AX                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 6762
#define RSSPECAN_ATTR_WLAN_CHANNEL_BANDWIDTH_TO_MEASURE_802_11AX                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 6763
#define RSSPECAN_ATTR_WLAN_HE_PPDU_CONFIG_FORMAT                                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 6764
#define RSSPECAN_ATTR_WLAN_EHT_PPDU_FORMAT                                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 10503
#define RSSPECAN_ATTR_WLAN_HE_PPDU_CONFIG_N_HE_LTF                                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 6765
#define RSSPECAN_ATTR_WLAN_HE_PPDU_CONFG_HIGHEST_RU_INDEX                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 7026
#define RSSPECAN_ATTR_WLAN_CONFIG_RU_ACTIVE_COUNT                                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 11140
#define RSSPECAN_ATTR_WLAN_CONFIG_RU_REFRESH                                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 11141
#define RSSPECAN_ATTR_WLAN_HE_PPDU_CONFIG_RU_INDEX_COUNT                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 7027
#define RSSPECAN_ATTR_WLAN_HE_PPDU_CONFIG_RU_INDEX_HIGHEST                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 7028
#define RSSPECAN_ATTR_WLAN_HE_PPDU_CONFIG_RU_INDEX                                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 6768
#define RSSPECAN_ATTR_WLAN_HE_PPDU_CONFIG_RU_INDEX_MRU                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 10504
#define RSSPECAN_ATTR_WLAN_HE_PPDU_CONFIG_RU_INDEX_RU26                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 10505
#define RSSPECAN_ATTR_WLAN_HE_PPDU_CONFIG_RU_INDEX_RU6                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 11142
#define RSSPECAN_ATTR_WLAN_HE_PPDU_CONFIG_USER_INDEX                                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 6769
#define RSSPECAN_ATTR_WLAN_HE_PPDU_CONFIG_RU_SIZE                                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 6770
#define RSSPECAN_ATTR_WLAN_HE_PPDU_CONFIG_MCS_INDEX                                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 6771
#define RSSPECAN_ATTR_WLAN_HE_PPDU_CONFIG_NSTS_PER_USER                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 6772
#define RSSPECAN_ATTR_WLAN_HE_PPDU_CONFIG_TX_BEAMFORMING                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 6773
#define RSSPECAN_ATTR_WLAN_HE_PPDU_CONFIG_DCM                                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 6774
#define RSSPECAN_ATTR_WLAN_HE_PPDU_CONFIG_CODING                                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 6775
#define RSSPECAN_ATTR_WLAN_HE_PPDU_CONFIG_INSERT_USER                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 6776
#define RSSPECAN_ATTR_WLAN_HE_PPDU_CONFIG_DELETE_USER                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 6778
#define RSSPECAN_ATTR_WLAN_PPDU_FORMAT_TO_MEASURE_802_11BE                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 11143
#define RSSPECAN_ATTR_WLAN_CHANNEL_BANDWIDTH_TO_MEASURE_802_11BE                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 11144
#define RSSPECAN_ATTR_WLAN_MIMO_SPATIAL_MAPPING_MODE                                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 2726
#define RSSPECAN_ATTR_WLAN_MIMO_POWER_NORMALISE                                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 2727
#define RSSPECAN_ATTR_WLAN_NUMBER_OF_ANTENNAS                                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 2708
#define RSSPECAN_ATTR_WLAN_MIMO_DUT_CONFIGURATION                                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 2709
#define RSSPECAN_ATTR_WLAN_MIMO_ANT_SIGNAL_CAPTURE_TYPE                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 2710
#define RSSPECAN_ATTR_WLAN_MIMO_ANTENNA_SIGNAL_CAPTURE_PATH                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 7029
#define RSSPECAN_ATTR_WLAN_MIMO_ANTENNA_SIGNAL_CAPTURE_TIME_SYNC                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 7030
#define RSSPECAN_ATTR_WLAN_MIMO_ANTENNA_SIGNAL_CAPTURE_OSP_MODULE                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 7031
#define RSSPECAN_ATTR_WLAN_MIMO_ANTENNA_SIGNAL_CAPTURE_OSP_IP_ADDRESS                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 7032
#define RSSPECAN_ATTR_WLAN_ANTENNA_SIMULTANEOUS_CAPTURE_STATE                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 2789
#define RSSPECAN_ATTR_WLAN_CALC_BURS                                                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 1970
#define RSSPECAN_ATTR_WLAN_REFERENCE_FREQUENCY_COUPLING                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 5890
#define RSSPECAN_ATTR_WLAN_AMPLITUDE_SETTINGS_COUPLING                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 6780
#define RSSPECAN_ATTR_WLAN_REFERENCE_LEVEL_OFFSET_COUPLING                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 6781
#define RSSPECAN_ATTR_WLAN_ANALYZER_STATE                                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 2715
#define RSSPECAN_ATTR_WLAN_ANALYZER_ADDRESS                                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 2716
#define RSSPECAN_ATTR_WLAN_ANTENNA_CONFIGURATION                                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 2717
#define RSSPECAN_ATTR_WLAN_REFERENCE_LEVEL_OFFSET                                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 6782
#define RSSPECAN_ATTR_WLAN_MIMO_LAN_STATUS                                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 7033
#define RSSPECAN_ATTR_WLAN_MIMO_UDFSPATIAL_MAPPING_TIME_SHIFT                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 2729
#define RSSPECAN_ATTR_WLAN_MIMO_UDFSPATIAL_MAPPING_STREAM                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 2728
#define RSSPECAN_ATTR_WLAN_BURS_COUN_STAT                                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 3037
#define RSSPECAN_ATTR_WLAN_BURS_COUN                                                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 1958
#define RSSPECAN_ATTR_WLAN_SOURCE_OF_PAYLOAD_LENGTH                                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 3039
#define RSSPECAN_ATTR_WLAN_SYM_EQU                                                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 2036
#define RSSPECAN_ATTR_WLAN_DUR_EQU                                                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 2040
#define RSSPECAN_ATTR_WLAN_DBYT_EQU                                                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 2038
#define RSSPECAN_ATTR_WLAN_SYM_MIN                                                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 4056
#define RSSPECAN_ATTR_WLAN_SYM_MAX                                                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 4057
#define RSSPECAN_ATTR_WLAN_ANALYSIS_INTERVAL_OFFSET                                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 6783
#define RSSPECAN_ATTR_WLAN_ANALYSIS_INTERVAL_LENGTH_STATE                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 6784
#define RSSPECAN_ATTR_WLAN_ANALYSIS_INTERVAL_LENGTH                                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 6785
#define RSSPECAN_ATTR_WLAN_DBYT_MIN                                                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 4058
#define RSSPECAN_ATTR_WLAN_DBYT_MAX                                                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 4059
#define RSSPECAN_ATTR_WLAN_DUR_MIN                                                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 4060
#define RSSPECAN_ATTR_WLAN_DUR_MAX                                                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 4061
#define RSSPECAN_ATTR_WLAN_PVT_AVER                                                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 1935
#define RSSPECAN_ATTR_WLAN_PVT_RPOW                                                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 1930
#define RSSPECAN_ATTR_WLAN_PEAK_ERROR_VECTOR_MEAS_RANGE                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 2603
#define RSSPECAN_ATTR_WLAN_FERR_AVERAGE                                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 4063
#define RSSPECAN_ATTR_WLAN_FERR_MAX                                                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 4064
#define RSSPECAN_ATTR_WLAN_SYMB_AVERAGE                                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 4065
#define RSSPECAN_ATTR_WLAN_SYMB_MAX                                                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 4066
#define RSSPECAN_ATTR_WLAN_IQOF_AVERAGE                                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 4067
#define RSSPECAN_ATTR_WLAN_IQOF_MAX                                                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 4068
#define RSSPECAN_ATTR_WLAN_EVM_ALL_AVERAGE                                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 4069
#define RSSPECAN_ATTR_WLAN_EVM_ALL_MAX                                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 4070
#define RSSPECAN_ATTR_WLAN_EVM_PIL_AVERAGE                                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 4073
#define RSSPECAN_ATTR_WLAN_EVM_PIL_MAX                                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 4074
#define RSSPECAN_ATTR_WLAN_EVM_DATA_AVERAGE                                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 4071
#define RSSPECAN_ATTR_WLAN_EVM_DATA_MAX                                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 4072
#define RSSPECAN_ATTR_WLAN_TRIS                                                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 1976
#define RSSPECAN_ATTR_WLAN_TRIS_MAXIMUM                                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 4075
#define RSSPECAN_ATTR_WLAN_TFAL_AVERAGE                                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 1977
#define RSSPECAN_ATTR_WLAN_TFAL_MAXIMUM                                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 4076
#define RSSPECAN_ATTR_WLAN_EVM_AVERAGE                                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 1981
#define RSSPECAN_ATTR_WLAN_EVM_MAXIMUM                                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 4077
#define RSSPECAN_ATTR_WLAN_POW_AUTO                                                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 1938
#define RSSPECAN_ATTR_WLAN_POW_AUTO_ONCE                                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 1939
#define RSSPECAN_ATTR_WLAN_POW_AUTO_SWE_TIME                                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 1967
#define RSSPECAN_ATTR_WLAN_DISP_TABL                                                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 1973
#define RSSPECAN_ATTR_WLAN_DISP_AUTO_Y                                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 1974
#define RSSPECAN_ATTR_WLAN_Y_PDIV                                                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 1975
#define RSSPECAN_ATTR_WLAN_PVT_SEL                                                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 1931
#define RSSPECAN_ATTR_WLAN_POLYNOMIAL_DEGREE                                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 7034
#define RSSPECAN_ATTR_WLAN_AM_AM_X_AXIS_AUTOSCALE_ENABLED                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 7035
#define RSSPECAN_ATTR_WLAN_AM_AM_X_AXIS_AUTO_MODE                                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 7036
#define RSSPECAN_ATTR_WLAN_AM_AM_X_AXIS_AUTO_FIX_RANGE                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 7037
#define RSSPECAN_ATTR_WLAN_AM_AM_X_AXIS_AUTO_FIX_RANGE_MIN                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 7038
#define RSSPECAN_ATTR_WLAN_AM_AM_X_AXIS_AUTO_FIX_RANGE_MAX                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 7039
#define RSSPECAN_ATTR_WLAN_AM_AM_X_AXIS_AUTO_HYSTERESIS_LOWER_LOWER                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 7040
#define RSSPECAN_ATTR_WLAN_AM_AM_X_AXIS_AUTO_HYSTERESIS_LOWER_UPPER                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 7041
#define RSSPECAN_ATTR_WLAN_AM_AM_X_AXIS_AUTO_HYSTERESIS_UPPER_LOWER                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 7042
#define RSSPECAN_ATTR_WLAN_AM_AM_X_AXIS_AUTO_HYSTERESIS_UPPER_UPPER                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 7043
#define RSSPECAN_ATTR_WLAN_AM_AM_X_AXIS_AUTO_HYSTERESIS_MEMORY_DEPTH                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 7044
#define RSSPECAN_ATTR_WLAN_AM_AM_X_AXIS_NUMBER_OF_DIVISIONS                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 7045
#define RSSPECAN_ATTR_WLAN_AM_AM_Y_AXIS_AUTOSCALE_ENABLED                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 7047
#define RSSPECAN_ATTR_WLAN_AM_AM_Y_AXIS_AUTO_MODE                                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 7048
#define RSSPECAN_ATTR_WLAN_AM_AM_Y_AXIS_AUTO_FIX_RANGE                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 7049
#define RSSPECAN_ATTR_WLAN_AM_AM_Y_AXIS_AUTO_FIX_RANGE_MIN                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 7050
#define RSSPECAN_ATTR_WLAN_AM_AM_Y_AXIS_AUTO_FIX_RANGE_MAX                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 7051
#define RSSPECAN_ATTR_WLAN_AM_AM_Y_AXIS_AUTO_HYSTERESIS_LOWER_LOWER                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 7052
#define RSSPECAN_ATTR_WLAN_AM_AM_Y_AXIS_AUTO_HYSTERESIS_LOWER_UPPER                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 7053
#define RSSPECAN_ATTR_WLAN_AM_AM_Y_AXIS_AUTO_HYSTERESIS_UPPER_LOWER                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 7054
#define RSSPECAN_ATTR_WLAN_AM_AM_Y_AXIS_AUTO_HYSTERESIS_UPPER_UPPER                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 7055
#define RSSPECAN_ATTR_WLAN_AM_AM_Y_AXIS_AUTO_HYSTERESIS_MEMORY_DEPTH                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 7056
#define RSSPECAN_ATTR_WLAN_AM_AM_Y_AXIS_NUMBER_OF_DIVISIONS                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 7057
#define RSSPECAN_ATTR_WLAN_IQ_LOAD                                                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 1968
#define RSSPECAN_ATTR_WLAN_IQ_SAVE                                                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 1969
#define RSSPECAN_ATTR_WLAN_SEM_TYPE                                                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 2045
#define RSSPECAN_ATTR_WLAN_SEM_CLASS                                                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 2046
#define RSSPECAN_ATTR_WLAN_SEM_FILE                                                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 2047
#define RSSPECAN_ATTR_WLAN_SPECTRUM_EMISSION_MASK_CHANNEL_BANDWIDTH                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 7059
#define RSSPECAN_ATTR_WLAN_SPEC_CSEL                                                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 4161
#define RSSPECAN_ATTR_WLAN_EGAT_HOLD_SAMP                                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 2023
#define RSSPECAN_ATTR_WLAN_EGAT_HOLD_TIME                                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 2024
#define RSSPECAN_ATTR_WLAN_EGAT_LENG_SAMP                                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 2025
#define RSSPECAN_ATTR_WLAN_EGAT_LENG_TIME                                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 2026
#define RSSPECAN_ATTR_WLAN_EGAT_LINK                                                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 2027
#define RSSPECAN_ATTR_WLAN_EGAT                                                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 2028
#define RSSPECAN_ATTR_WLAN_CONS_CARR_SEL                                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 3038
#define RSSPECAN_ATTR_WLAN_CONS_CARR_SEL_NR                                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 1951
#define RSSPECAN_ATTR_WLAN_BURST_SELECT                                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 3148
#define RSSPECAN_ATTR_WLAN_BURST_STATE                                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 3149
#define RSSPECAN_ATTR_WLAN_CONS_CCAR                                                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 1949
#define RSSPECAN_ATTR_WLAN_CONS_CSYM                                                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 1947
#define RSSPECAN_ATTR_WLAN_EVM_ECAR                                                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 1941
#define RSSPECAN_ATTR_WLAN_EVM_ESYM                                                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 1942
#define RSSPECAN_ATTR_WLAN_PRE                                                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 1964
#define RSSPECAN_ATTR_WLAN_PRE_SEL                                                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 1971
#define RSSPECAN_ATTR_WLAN_EVM_ECHIP                                                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 4079
#define RSSPECAN_ATTR_WLAN_PVT                                                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 1932
#define RSSPECAN_ATTR_WLAN_SPEC_FFT                                                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 1945
#define RSSPECAN_ATTR_WLAN_SPEC_SEL                                                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 4162
#define RSSPECAN_ATTR_WLAN_SPEC_FLAT                                                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 1933
#define RSSPECAN_ATTR_WLAN_SPECTRUM_FLATNESS_UNIT                                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 6437
#define RSSPECAN_ATTR_WLAN_STAT_BSTR                                                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 1953
#define RSSPECAN_ATTR_WLAN_STAT_SFI                                                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 1954
#define RSSPECAN_ATTR_WLAN_AM_AM                                                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 5051
#define RSSPECAN_ATTR_WLAN_AM_EVM                                                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 5052
#define RSSPECAN_ATTR_WLAN_AM_PM                                                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 5053
#define RSSPECAN_ATTR_WLAN_GAIN_IMBALANCE_VS_CARRIER                                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 5891
#define RSSPECAN_ATTR_WLAN_PHASE_TRACKING_VS_SYMBOL                                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 5892
#define RSSPECAN_ATTR_WLAN_QUADRATURE_ERROR_VS_CARRIER                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 5893
#define RSSPECAN_ATTR_WLAN_HYSTERESIS_THRESHOLD_LOWER                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 11145
#define RSSPECAN_ATTR_WLAN_HYSTERESIS_THRESHOLD_UPPER                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 11146
#define RSSPECAN_ATTR_WLAN_SPEC_ACPR                                                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 1946
#define RSSPECAN_ATTR_WLAN_SPEC_MASK_SEL                                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 1944
#define RSSPECAN_ATTR_WLAN_SPEC_MASK                                                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 1943
#define RSSPECAN_ATTR_WLAN_SPEC_OCCUPIED_BANDWIDTH                                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 3041
#define RSSPECAN_ATTR_WLAN_STAT_CCDF                                                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 1952
#define RSSPECAN_ATTR_WLAN_ACH_MODE                                                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 1960
#define RSSPECAN_ATTR_WLAN_MARK_SYMB                                                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 2001
#define RSSPECAN_ATTR_WLAN_MARK_CARR                                                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 2002
#define RSSPECAN_ATTR_WLAN_MARK_Y                                                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 2003
#define RSSPECAN_ATTR_WLAN_PPDU_SELECT_STATE                                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 7060
#define RSSPECAN_ATTR_WLAN_FETC_BURS_COUNT                                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 2004
#define RSSPECAN_ATTR_WLAN_FETC_BURST_COUNT_ALL                                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 3042
#define RSSPECAN_ATTR_WLAN_FETC_SYMB_COUNT                                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 2022
#define RSSPECAN_ATTR_WLAN_FETCH_MCS_INDEX                                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 10506
#define RSSPECAN_ATTR_WLAN_FETC_BURST_LENGTH                                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 3125
#define RSSPECAN_ATTR_WLAN_FETCH_BURST_TYPE                                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 10507
#define RSSPECAN_ATTR_WLAN_FETCH_BURST_GUARD_INTERVAL                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 10508
#define RSSPECAN_ATTR_WLAN_FETCH_BURST_START                                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 3147
#define RSSPECAN_ATTR_WLAN_BURST_UNIT                                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 3151
#define RSSPECAN_ATTR_WLAN_UNIT_EVM                                                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 1962
#define RSSPECAN_ATTR_WLAN_UNIT_GIMB                                                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 3040
#define RSSPECAN_ATTR_WLAN_UNIT_PRE                                                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 1961
#define RSSPECAN_ATTR_WLAN_FETC_CRES_MIN                                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 4084
#define RSSPECAN_ATTR_WLAN_FETC_CRES_AVERAGE                                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 4085
#define RSSPECAN_ATTR_WLAN_FETC_CRES_MAX                                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 4086
#define RSSPECAN_ATTR_WLAN_FETC_EVM_ALL_MIN                                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 4087
#define RSSPECAN_ATTR_WLAN_FETC_EVM_ALL_AVERAGE                                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 4088
#define RSSPECAN_ATTR_WLAN_FETC_EVM_ALL_MAX                                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 4089
#define RSSPECAN_ATTR_WLAN_FETC_EVM_DATA_MIN                                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 4090
#define RSSPECAN_ATTR_WLAN_FETC_EVM_DATA_AVERAGE                                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 4091
#define RSSPECAN_ATTR_WLAN_FETC_EVM_DATA_MAX                                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 4092
#define RSSPECAN_ATTR_WLAN_FETC_EVM_PILOT_MIN                                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 4093
#define RSSPECAN_ATTR_WLAN_FETC_EVM_PILOT_AVERAGE                                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 4094
#define RSSPECAN_ATTR_WLAN_FETC_EVM_PILOT_MAX                                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 4095
#define RSSPECAN_ATTR_WLAN_FETC_FERR_MIN                                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 4096
#define RSSPECAN_ATTR_WLAN_FETC_FERR_AVERAGE                                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 4097
#define RSSPECAN_ATTR_WLAN_FETC_FERR_MAX                                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 4098
#define RSSPECAN_ATTR_WLAN_FETC_GIMB_MIN                                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 4099
#define RSSPECAN_ATTR_WLAN_FETC_GIMB_AVERAGE                                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 4100
#define RSSPECAN_ATTR_WLAN_FETC_GIMB_MAX                                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 4101
#define RSSPECAN_ATTR_WLAN_FETC_IQOF_MIN                                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 4102
#define RSSPECAN_ATTR_WLAN_FETC_IQOF_AVERAGE                                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 4103
#define RSSPECAN_ATTR_WLAN_FETC_IQOF_MAX                                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 4104
#define RSSPECAN_ATTR_WLAN_FETC_IQSK_MIN                                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 5184
#define RSSPECAN_ATTR_WLAN_FETC_IQSK_AVERAGE                                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 5185
#define RSSPECAN_ATTR_WLAN_FETC_IQSK_MAX                                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 5186
#define RSSPECAN_ATTR_WLAN_FETC_PAYL_AVERAGE                                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 4105
#define RSSPECAN_ATTR_WLAN_FETC_PAYL_MIN                                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 4106
#define RSSPECAN_ATTR_WLAN_FETC_PAYL_MAX                                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 4107
#define RSSPECAN_ATTR_WLAN_FETC_PEAK_MIN                                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 4108
#define RSSPECAN_ATTR_WLAN_FETC_PEAK_AVERAGE                                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 4109
#define RSSPECAN_ATTR_WLAN_FETC_PEAK_MAX                                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 4110
#define RSSPECAN_ATTR_WLAN_FETC_PRE_MIN                                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 4111
#define RSSPECAN_ATTR_WLAN_FETC_PRE_AVERAGE                                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 4112
#define RSSPECAN_ATTR_WLAN_FETC_PRE_MAX                                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 4113
#define RSSPECAN_ATTR_WLAN_FETC_QUAD_MIN                                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 4114
#define RSSPECAN_ATTR_WLAN_FETC_QUAD_AVERAGE                                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 4115
#define RSSPECAN_ATTR_WLAN_FETC_QUAD_MAX                                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 4116
#define RSSPECAN_ATTR_WLAN_FETC_RMS_MIN                                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 4117
#define RSSPECAN_ATTR_WLAN_FETC_RMS_AVERAGE                                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 4118
#define RSSPECAN_ATTR_WLAN_FETC_RMS_MAX                                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 4119
#define RSSPECAN_ATTR_WLAN_FETC_SYMB_MIN                                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 4120
#define RSSPECAN_ATTR_WLAN_FETC_SYMB_AVERAGE                                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 4121
#define RSSPECAN_ATTR_WLAN_FETC_SYMB_MAX                                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 4122
#define RSSPECAN_ATTR_WLAN_FETC_EVM_MIN                                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 4123
#define RSSPECAN_ATTR_WLAN_FETC_EVM_AVERAGE                                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 4124
#define RSSPECAN_ATTR_WLAN_FETC_EVM_MAX                                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 4125
#define RSSPECAN_ATTR_WLAN_FETC_EVM_DIR_MIN                                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 4126
#define RSSPECAN_ATTR_WLAN_FETC_EVM_DIR_AVERAGE                                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 4127
#define RSSPECAN_ATTR_WLAN_FETC_EVM_DIR_MAX                                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 4128
#define RSSPECAN_ATTR_WLAN_FETC_TRIS_MIN                                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 4129
#define RSSPECAN_ATTR_WLAN_FETC_TRIS_AVERAGE                                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 4130
#define RSSPECAN_ATTR_WLAN_FETC_TRIS_MAX                                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 4131
#define RSSPECAN_ATTR_WLAN_FETC_TFAL_MIN                                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 4132
#define RSSPECAN_ATTR_WLAN_FETC_TFAL_AVERAGE                                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 4133
#define RSSPECAN_ATTR_WLAN_FETC_TFAL_MAX                                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 4134
#define RSSPECAN_ATTR_WLAN_FETC_BERPILOT_MIN                                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 4135
#define RSSPECAN_ATTR_WLAN_FETC_BERPILOT_AVERAGE                                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 4136
#define RSSPECAN_ATTR_WLAN_FETC_BERPILOT_MAX                                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 4137
#define RSSPECAN_ATTR_WLAN_EVM_ALL_RES_AVERAGE                                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 4138
#define RSSPECAN_ATTR_WLAN_EVM_ALL_RES_MAX                                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 4139
#define RSSPECAN_ATTR_WLAN_FERR_RES_AVERAGE                                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 4140
#define RSSPECAN_ATTR_WLAN_FERR_RES_MAX                                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 4141
#define RSSPECAN_ATTR_WLAN_IQOF_RES_AVERAGE                                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 4142
#define RSSPECAN_ATTR_WLAN_IQOF_RES_MAX                                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 4143
#define RSSPECAN_ATTR_WLAN_SYMB_RES_AVERAGE                                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 4144
#define RSSPECAN_ATTR_WLAN_SYMB_RES_MAX                                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 4145
#define RSSPECAN_ATTR_WLAN_EVM_DATA_RES_AVERAGE                                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 4146
#define RSSPECAN_ATTR_WLAN_EVM_DATA_RES_MAX                                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 4147
#define RSSPECAN_ATTR_WLAN_EVM_PIL_RES_AVERAGE                                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 4148
#define RSSPECAN_ATTR_WLAN_EVM_PIL_RES_MAX                                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 4149
#define RSSPECAN_ATTR_WLAN_TFAL_RES_AVERAGE                                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 1985
#define RSSPECAN_ATTR_WLAN_TFAL_RES_MAXIMUM                                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 4150
#define RSSPECAN_ATTR_WLAN_TRIS_RES_AVERAGE                                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 4151
#define RSSPECAN_ATTR_WLAN_TRIS_RES_MAX                                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 4152
#define RSSPECAN_ATTR_WLAN_EVM_RES_AVERAGE                                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 4153
#define RSSPECAN_ATTR_WLAN_EVM_RES_MAX                                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 4154
#define RSSPECAN_ATTR_WLAN_LIM_FAIL_RES                                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 1991
#define RSSPECAN_ATTR_WLAN_ACP_ADJ                                                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 1995
#define RSSPECAN_ATTR_WLAN_ACP_ADJ_RES                                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 1996
#define RSSPECAN_ATTR_WLAN_ACP_ALT                                                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 1997
#define RSSPECAN_ATTR_WLAN_ACP_ALT_RES                                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 1998
#define RSSPECAN_ATTR_WLAN_SPEC_MASK_X                                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 1999
#define RSSPECAN_ATTR_WLAN_SPEC_MASK_Y                                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 2000
#define RSSPECAN_ATTR_WLAN_OUTPUT_BITSTREAM_DATA                                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 7098
#define RSSPECAN_ATTR_WIGIG_MODE                                                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 6127
#define RSSPECAN_ATTR_WIGIG_AUTO_LEVEL                                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 6128
#define RSSPECAN_ATTR_WIGIG_AUTO_LEVEL_ONCE                                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 6129
#define RSSPECAN_ATTR_WIGIG_SEM_FILE                                                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 7110
#define RSSPECAN_ATTR_EDMG_CHIP_RATE                                                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 7782
#define RSSPECAN_ATTR_EDMG_CONTIGUOUS_NUMBER                                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 7783
#define RSSPECAN_ATTR_EDMG_USER_CHIP_RATE                                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 7784
#define RSSPECAN_ATTR_WIGIG_TRACKING_PHASE_ENABLED                                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 6130
#define RSSPECAN_ATTR_WIGIG_TRACKING_TIMING_ENABLED                                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 6131
#define RSSPECAN_ATTR_WIGIG_TRACKING_LEVEL_ENABLED                                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 6132
#define RSSPECAN_ATTR_WIGIG_IQ_COMPENSATION_ENABLED                                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 6133
#define RSSPECAN_ATTR_WIGIG_BITSTREAM_FORMAT                                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 6136
#define RSSPECAN_ATTR_WIGIG_RECALCULATE_IQ_MEASUREMENT_RESULTS                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 7111
#define RSSPECAN_ATTR_WIGIG_STATISTIC_COUNT_ENABLED                                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 6137
#define RSSPECAN_ATTR_WIGIG_STATISTIC_COUNT                                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 6138
#define RSSPECAN_ATTR_WIGIG_PPDU_TO_ANALYZE                                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 6139
#define RSSPECAN_ATTR_WIGIG_PPDU_INDEX                                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 6140
#define RSSPECAN_ATTR_WIGIG_EQUAL_BURST_LENGTH_ENABLED                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 6141
#define RSSPECAN_ATTR_WIGIG_NUMBER_OF_SYMBOLS_MAXIMUM                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 6142
#define RSSPECAN_ATTR_WIGIG_NUMBER_OF_SYMBOLS_MINIMUM                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 6143
#define RSSPECAN_ATTR_WIGIG_FETC_BURS_COUNT                                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 6144
#define RSSPECAN_ATTR_WIGIG_EXPORTING_TRACE_RESULTS_TO_ASCII_FILE_ENABLED                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 7112
#define RSSPECAN_ATTR_WIGIG_FETCH_CREST_FACTOR_AVERAGE                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 6148
#define RSSPECAN_ATTR_WIGIG_FETCH_CREST_FACTOR_MAXIMUM                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 6149
#define RSSPECAN_ATTR_WIGIG_FETCH_CREST_FACTOR_MINIMUM                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 6150
#define RSSPECAN_ATTR_WIGIG_FETCH_CENTER_FREQUENCY_ERROR_AVERAGE                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 6151
#define RSSPECAN_ATTR_WIGIG_FETCH_CENTER_FREQUENCY_ERROR_MAXIMUM                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 6152
#define RSSPECAN_ATTR_WIGIG_FETCH_CENTER_FREQUENCY_ERROR_MINIMUM                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 6153
#define RSSPECAN_ATTR_WIGIG_FETCH_EVM_ALL_SYMBOLS_AVERAGE                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 6154
#define RSSPECAN_ATTR_WIGIG_FETCH_EVM_ALL_SYMBOLS_MAXIMUM                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 6155
#define RSSPECAN_ATTR_WIGIG_FETCH_EVM_ALL_SYMBOLS_MINIMUM                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 6156
#define RSSPECAN_ATTR_WIGIG_FETCH_EVM_DATA_SYMBOLS_AVERAGE                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 6157
#define RSSPECAN_ATTR_WIGIG_FETCH_EVM_DATA_SYMBOLS_MAXIMUM                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 6158
#define RSSPECAN_ATTR_WIGIG_FETCH_EVM_DATA_SYMBOLS_MINIMUM                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 6159
#define RSSPECAN_ATTR_WIGIG_FETCH_EVM_PILOT_SYMBOLS_AVERAGE                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 6160
#define RSSPECAN_ATTR_WIGIG_FETCH_EVM_PILOT_SYMBOLS_MAXIMUM                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 6161
#define RSSPECAN_ATTR_WIGIG_FETCH_EVM_PILOT_SYMBOLS_MINIMUM                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 6162
#define RSSPECAN_ATTR_WIGIG_FETCH_FALL_TIME_AVERAGE                                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 6163
#define RSSPECAN_ATTR_WIGIG_FETCH_FALL_TIME_MAXIMUM                                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 6164
#define RSSPECAN_ATTR_WIGIG_FETCH_FALL_TIME_MINIMUM                                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 6165
#define RSSPECAN_ATTR_WIGIG_FETCH_RISE_TIME_AVERAGE                                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 6166
#define RSSPECAN_ATTR_WIGIG_FETCH_RISE_TIME_MAXIMUM                                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 6167
#define RSSPECAN_ATTR_WIGIG_FETCH_RISE_TIME_MINIMUM                                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 6168
#define RSSPECAN_ATTR_WIGIG_FETCH_GAIN_IMBALANCE_AVERAGE                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 6169
#define RSSPECAN_ATTR_WIGIG_FETCH_GAIN_IMBALANCE_MAXIMUM                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 6170
#define RSSPECAN_ATTR_WIGIG_FETCH_GAIN_IMBALANCE_MINIMUM                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 6171
#define RSSPECAN_ATTR_WIGIG_FETCH_IQ_OFFSET_AVERAGE                                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 6172
#define RSSPECAN_ATTR_WIGIG_FETCH_IQ_OFFSET_MAXIMUM                                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 6173
#define RSSPECAN_ATTR_WIGIG_FETCH_IQ_OFFSET_MINIMUM                                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 6174
#define RSSPECAN_ATTR_WIGIG_FETCH_QUADRATURE_ERROR_AVERAGE                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 6175
#define RSSPECAN_ATTR_WIGIG_FETCH_QUADRATURE_ERROR_MAXIMUM                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 6176
#define RSSPECAN_ATTR_WIGIG_FETCH_QUADRATURE_ERROR_MINIMUM                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 6177
#define RSSPECAN_ATTR_WIGIG_FETCH_SIGNAL_TO_RATIO_AVERAGE                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 7785
#define RSSPECAN_ATTR_WIGIG_FETCH_SIGNAL_TO_RATIO_MAXIMUM                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 7786
#define RSSPECAN_ATTR_WIGIG_FETCH_SIGNAL_TO_RATIO_MINIMUM                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 7787
#define RSSPECAN_ATTR_WIGIG_FETCH_SYMBOL_CLOCK_ERROR_AVERAGE                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 6178
#define RSSPECAN_ATTR_WIGIG_FETCH_SYMBOL_CLOCK_ERROR_MAXIMUM                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 6179
#define RSSPECAN_ATTR_WIGIG_FETCH_SYMBOL_CLOCK_ERROR_MINIMUM                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 6180
#define RSSPECAN_ATTR_WIGIG_FETCH_TIME_DOMAIN_POWER_AVERAGE                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 6181
#define RSSPECAN_ATTR_WIGIG_FETCH_TIME_DOMAIN_POWER_MAXIMUM                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 6182
#define RSSPECAN_ATTR_WIGIG_FETCH_TIME_DOMAIN_POWER_MINIMUM                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 6183
#define RSSPECAN_ATTR_WIGIG_FETCH_TIME_SKEW_AVERAGE                                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 6184
#define RSSPECAN_ATTR_WIGIG_FETCH_TIME_SKEW_MAXIMUM                                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 6185
#define RSSPECAN_ATTR_WIGIG_FETCH_TIME_SKEW_MINIMUM                                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 6186
#define RSSPECAN_ATTR_WIGIG_FETCH_BER_PPDU_HEADER_MIN                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 6511
#define RSSPECAN_ATTR_WIGIG_FETCH_BER_PPDU_HEADER_AVERAGE                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 6512
#define RSSPECAN_ATTR_WIGIG_FETCH_BER_PPDU_HEADER_MAX                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 6513
#define RSSPECAN_ATTR_WIGIG_FETCH_BER_PPDU_PAYLOAD_MIN                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 6514
#define RSSPECAN_ATTR_WIGIG_FETCH_BER_PPDU_PAYLOAD_AVERAGE                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 6515
#define RSSPECAN_ATTR_WIGIG_FETCH_BER_PPDU_PAYLOAD_MAX                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 6516
#define RSSPECAN_ATTR_TRANSIENT_MODE                                                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 5187
#define RSSPECAN_ATTR_TRANSIENT_MODEL                                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 5188
#define RSSPECAN_ATTR_TRANSIENT_HOP_NUMBER_OF_STATES                                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 5515
#define RSSPECAN_ATTR_TRANSIENT_HOP_AUTO                                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 5189
#define RSSPECAN_ATTR_TRANSIENT_HOP_LOAD_SIGNAL_STATE_TABLE                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 5516
#define RSSPECAN_ATTR_TRANSIENT_HOP_SAVE_SIGNAL_STATE_TABLE                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 5518
#define RSSPECAN_ATTR_GET_TRANSIENT_HOP_STATE_TABLE_START_FREQUENCY                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 5520
#define RSSPECAN_ATTR_GET_TRANSIENT_HOP_STATE_TABLE_STEP_SIZE                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 5521
#define RSSPECAN_ATTR_GET_TRANSIENT_HOP_STATE_TABLE_NUMBER_OF_STATES                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 5522
#define RSSPECAN_ATTR_TRANSIENT_HOP_STATE_TABLE_TOLERANCE                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 5525
#define RSSPECAN_ATTR_TRANSIENT_HOP_STATE_TABLE_OFFSET                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 5526
#define RSSPECAN_ATTR_TRANSIENT_HOP_TIMING_AUTO_MODE                                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 5191
#define RSSPECAN_ATTR_TRANSIENT_HOP_TIMING_MINIMUM                                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 5192
#define RSSPECAN_ATTR_TRANSIENT_HOP_TIMING_MAXIMUM                                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 5193
#define RSSPECAN_ATTR_TRANSIENT_CHIRP_NUMBER_OF_STATES                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 5527
#define RSSPECAN_ATTR_TRANSIENT_CHIRP_AUTO                                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 5194
#define RSSPECAN_ATTR_TRANSIENT_CHIRP_LOAD_SIGNAL_STATE_TABLE                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 5528
#define RSSPECAN_ATTR_TRANSIENT_CHIRP_SAVE_SIGNAL_STATE_TABLE                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 5530
#define RSSPECAN_ATTR_TRANSIENT_CHIRP_TIMING_AUTO_MODE                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 5196
#define RSSPECAN_ATTR_TRANSIENT_CHIRP_TIMING_MINIMUM                                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 5197
#define RSSPECAN_ATTR_TRANSIENT_CHIRP_TIMING_MAXIMUM                                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 5198
#define RSSPECAN_ATTR_TRANSIENT_MEASUREMENT_BANDWIDTH                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 5199
#define RSSPECAN_ATTR_TRANSIENT_SAMPLE_RATE                                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 5200
#define RSSPECAN_ATTR_TRANSIENT_MEASUREMENT_TIME                                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 5201
#define RSSPECAN_ATTR_TRANSIENT_RECORD_LENGTH                                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 5202
#define RSSPECAN_ATTR_TRANSIENT_ANALYSIS_BANDWIDTH                                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 5203
#define RSSPECAN_ATTR_TRANSIENT_DELTA_FREQUENCY                                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 5204
#define RSSPECAN_ATTR_TRANSIENT_TIME_GATE_LENGTH                                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 5205
#define RSSPECAN_ATTR_TRANSIENT_TIME_GATE_START                                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 5206
#define RSSPECAN_ATTR_TRANSIENT_LINK_BANDWIDTH_ENABLED                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 5207
#define RSSPECAN_ATTR_TRANSIENT_LINK_BANDWIDTH                                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 5208
#define RSSPECAN_ATTR_TRANSIENT_LINK_TIME_ENABLED                                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 5209
#define RSSPECAN_ATTR_TRANSIENT_LINK_TIME                                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 5210
#define RSSPECAN_ATTR_TRANSIENT_FFT_WINDOW                                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 5211
#define RSSPECAN_ATTR_TRANSIENT_FM_VIDEO_BANDWIDTH                                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 5212
#define RSSPECAN_ATTR_TRANSIENT_TIME_RESOLUTION_AUTO                                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 5213
#define RSSPECAN_ATTR_TRANSIENT_TIME_RESOLUTION                                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 5214
#define RSSPECAN_ATTR_TRANSIENT_HOP_FREQUENCY_REFERENCE                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 5215
#define RSSPECAN_ATTR_TRANSIENT_HOP_FREQUENCY_LENGTH                                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 5216
#define RSSPECAN_ATTR_TRANSIENT_HOP_FREQUENCY_OFFSET_BEGIN                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 5217
#define RSSPECAN_ATTR_TRANSIENT_HOP_FREQUENCY_OFFSET_END                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 5218
#define RSSPECAN_ATTR_TRANSIENT_HOP_POWER_REFERENCE                                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 5219
#define RSSPECAN_ATTR_TRANSIENT_HOP_POWER_LENGTH                                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 5220
#define RSSPECAN_ATTR_TRANSIENT_HOP_POWER_OFFSET_BEGIN                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 5221
#define RSSPECAN_ATTR_TRANSIENT_HOP_POWER_OFFSET_END                                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 5222
#define RSSPECAN_ATTR_TRANSIENT_HOP_FREQUENCY_DEVIATION_REFERENCE                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 7120
#define RSSPECAN_ATTR_TRANSIENT_HOP_FREQUENCY_DEVIATION_LENGTH                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 7121
#define RSSPECAN_ATTR_TRANSIENT_HOP_FREQUENCY_DEVIATION_OFFSET_BEGIN                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 7122
#define RSSPECAN_ATTR_TRANSIENT_HOP_FREQUENCY_DEVIATION_OFFSET_END                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 7123
#define RSSPECAN_ATTR_TRANSIENT_HOP_PHASE_DEVIATION_REFERENCE                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 7124
#define RSSPECAN_ATTR_TRANSIENT_HOP_PHASE_DEVIATION_LENGTH                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 7125
#define RSSPECAN_ATTR_TRANSIENT_HOP_PHASE_DEVIATION_OFFSET_BEGIN                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 7126
#define RSSPECAN_ATTR_TRANSIENT_HOP_PHASE_DEVIATION_OFFSET_END                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 7127
#define RSSPECAN_ATTR_TRANSIENT_HOP_COMPENSATE_HOP_FREQUENCY_DEVIATION                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 6786
#define RSSPECAN_ATTR_TRANSIENT_HOP_COHERENT_PHASE_DEVIATION_MEASUREMENT                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 7128
#define RSSPECAN_ATTR_TRANSIENT_HOP_FM_SETTLING_TOLERANCE                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 7129
#define RSSPECAN_ATTR_TRANSIENT_HOP_PM_SETTLING_TOLERANCE                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 7130
#define RSSPECAN_ATTR_TRANSIENT_HOP_PHASE_NOISE_REFERENCE                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 11030
#define RSSPECAN_ATTR_TRANSIENT_HOP_PHASE_NOISE_FREQUENCY_START                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 11031
#define RSSPECAN_ATTR_TRANSIENT_HOP_PHASE_NOISE_FREQUENCY_STOP                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 11032
#define RSSPECAN_ATTR_TRANSIENT_HOP_PHASE_NOISE_LENGTH                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 11033
#define RSSPECAN_ATTR_TRANSIENT_HOP_PHASE_NOISE_OFFSET_BEGIN                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 11034
#define RSSPECAN_ATTR_TRANSIENT_HOP_PHASE_NOISE_OFFSET_END                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 11035
#define RSSPECAN_ATTR_TRANSIENT_CHIRP_DETECTION_ENABLED                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 11036
#define RSSPECAN_ATTR_TRANSIENT_CHIRP_FREQUENCY_REFERENCE                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 5223
#define RSSPECAN_ATTR_TRANSIENT_CHIRP_FREQUENCY_LENGTH                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 5224
#define RSSPECAN_ATTR_TRANSIENT_CHIRP_FREQUENCY_OFFSET_BEGIN                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 5225
#define RSSPECAN_ATTR_TRANSIENT_CHIRP_FREQUENCY_OFFSET_END                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 5226
#define RSSPECAN_ATTR_TRANSIENT_CHIRP_POWER_REFERENCE                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 5227
#define RSSPECAN_ATTR_TRANSIENT_CHIRP_POWER_LENGTH                                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 5228
#define RSSPECAN_ATTR_TRANSIENT_CHIRP_POWER_OFFSET_BEGIN                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 5229
#define RSSPECAN_ATTR_TRANSIENT_CHIRP_POWER_OFFSET_END                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 5230
#define RSSPECAN_ATTR_TRANSIENT_CHIRP_FREQUENCY_DEVIATION_REFERENCE                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 7131
#define RSSPECAN_ATTR_TRANSIENT_CHIRP_FREQUENCY_DEVIATION_LENGTH                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 7132
#define RSSPECAN_ATTR_TRANSIENT_CHIRP_FREQUENCY_DEVIATION_OFFSET_BEGIN                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 7133
#define RSSPECAN_ATTR_TRANSIENT_CHIRP_FREQUENCY_DEVIATION_OFFSET_END                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 7134
#define RSSPECAN_ATTR_TRANSIENT_CHIRP_PHASE_DEVIATION_REFERENCE                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 7135
#define RSSPECAN_ATTR_TRANSIENT_CHIRP_PHASE_DEVIATION_LENGTH                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 7136
#define RSSPECAN_ATTR_TRANSIENT_CHIRP_PHASE_DEVIATION_OFFSET_BEGIN                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 7137
#define RSSPECAN_ATTR_TRANSIENT_CHIRP_PHASE_DEVIATION_OFFSET_END                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 7138
#define RSSPECAN_ATTR_TRANSIENT_CHIRP_COMPENSATE_CHIRP_RATE_DEVIATION                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 6787
#define RSSPECAN_ATTR_TRANSIENT_CHIRP_FM_SETTLING_TOLERANCE                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 7139
#define RSSPECAN_ATTR_TRANSIENT_CHIRP_PM_SETTLING_TOLERANCE                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 7140
#define RSSPECAN_ATTR_TRANSIENT_CHIRP_PHASE_NOISE_REFERENCE                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 11037
#define RSSPECAN_ATTR_TRANSIENT_CHIRP_PHASE_NOISE_FREQUENCY_START                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 11038
#define RSSPECAN_ATTR_TRANSIENT_CHIRP_PHASE_NOISE_FREQUENCY_STOP                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 11039
#define RSSPECAN_ATTR_TRANSIENT_CHIRP_PHASE_NOISE_LENGTH                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 11040
#define RSSPECAN_ATTR_TRANSIENT_CHIRP_PHASE_NOISE_OFFSET_BEGIN                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 11041
#define RSSPECAN_ATTR_TRANSIENT_CHIRP_PHASE_NOISE_OFFSET_END                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 11042
#define RSSPECAN_ATTR_TRANSIENT_CONTINUE_SINGLE_SWEEP                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 5231
#define RSSPECAN_ATTR_TRANSIENT_WAIT_FOR_END_OF_AUTO_REFRESH                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 5232
#define RSSPECAN_ATTR_TRANSIENT_FFT_WINDOW_LENGTH                                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 5233
#define RSSPECAN_ATTR_TRANSIENT_DIGITAL_IQ_40G_ENABLED                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 8577
#define RSSPECAN_ATTR_TRANSIENT_DIGITAL_IQ_40G_SAMPLE_RATE                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 8578
#define RSSPECAN_ATTR_TRANSIENT_DIGITAL_IQ_40G_CONNECTED_DEVICE                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 8579
#define RSSPECAN_ATTR_TRANSIENT_RESULT_AUTOMATIC_RANGE_SCALING                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 5234
#define RSSPECAN_ATTR_TRANSIENT_RESULT_RANGE_REFERENCE                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 5235
#define RSSPECAN_ATTR_TRANSIENT_RESULT_RANGE_OFFSET                                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 5236
#define RSSPECAN_ATTR_TRANSIENT_RESULT_RANGE_ALIGNMENT                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 5237
#define RSSPECAN_ATTR_TRANSIENT_RESULT_RANGE_LENGTH                                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 5238
#define RSSPECAN_ATTR_TRANSIENT_HOP_RESULT_TABLE_POWER_ALL                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 5536
#define RSSPECAN_ATTR_TRANSIENT_HOP_RESULT_TABLE_POWER_AVERAGE                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 5537
#define RSSPECAN_ATTR_TRANSIENT_HOP_RESULT_TABLE_POWER_MAXIMUM                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 5538
#define RSSPECAN_ATTR_TRANSIENT_HOP_RESULT_TABLE_POWER_MINIMUM                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 5539
#define RSSPECAN_ATTR_TRANSIENT_HOP_RESULT_TABLE_POWER_RIPPLE                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 5540
#define RSSPECAN_ATTR_TRANSIENT_HOP_RESULT_TABLE_STATE_ALL                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 5541
#define RSSPECAN_ATTR_TRANSIENT_HOP_RESULT_TABLE_STATE_INDEX                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 5542
#define RSSPECAN_ATTR_TRANSIENT_CHIRP_RESULT_TABLE_POWER_ALL                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 5546
#define RSSPECAN_ATTR_TRANSIENT_CHIRP_RESULT_TABLE_POWER_AVERAGE                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 5547
#define RSSPECAN_ATTR_TRANSIENT_CHIRP_RESULT_TABLE_POWER_MAXIMUM                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 5548
#define RSSPECAN_ATTR_TRANSIENT_CHIRP_RESULT_TABLE_POWER_MINIMUM                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 5549
#define RSSPECAN_ATTR_TRANSIENT_CHIRP_RESULT_TABLE_POWER_RIPPLE                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 5550
#define RSSPECAN_ATTR_TRANSIENT_CHIRP_RESULT_TABLE_STATE_ALL                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 7141
#define RSSPECAN_ATTR_TRANSIENT_CHIRP_RESULT_TABLE_STATE_INDEX                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 5551
#define RSSPECAN_ATTR_TRANSIENT_CHIRP_RESULT_TABLE_FREQUENCY_INL_AVERAGE                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 7792
#define RSSPECAN_ATTR_TRANSIENT_CHIRP_RESULT_TABLE_FREQUENCY_INL_PEAK                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 7793
#define RSSPECAN_ATTR_TRANSIENT_CHIRP_RESULT_TABLE_FREQUENCY_INL_RMS                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 7794
#define RSSPECAN_ATTR_TRANSIENT_PARAMETER_DISTRIBUTION_HISTOGRAM_BINS                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 5552
#define RSSPECAN_ATTR_TRANSIENT_PARAMETER_DISTRIBUTION_X_AXIS                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 7142
#define RSSPECAN_ATTR_TRANSIENT_PARAMETER_DISTRIBUTION_Y_AXIS                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 7143
#define RSSPECAN_ATTR_TRANSIENT_PARAMETER_TREND_SWAP_XY                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 5563
#define RSSPECAN_ATTR_TRANSIENT_PARAMETER_TREND_X_AXIS                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 7144
#define RSSPECAN_ATTR_TRANSIENT_PARAMETER_TREND_Y_AXIS                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 7145
#define RSSPECAN_ATTR_TRANSIENT_AUTO_GRID_SCALING                                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 5242
#define RSSPECAN_ATTR_TRANSIENT_ABSOLUTE_SCALING_MAX                                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 5243
#define RSSPECAN_ATTR_TRANSIENT_ABSOLUTE_SCALING_MIN                                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 5244
#define RSSPECAN_ATTR_TRANSIENT_SCALING_DISPLAY_RANGE                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 5245
#define RSSPECAN_ATTR_TRANSIENT_UNIT_ANGLE                                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 5246
#define RSSPECAN_ATTR_TRANSIENT_EVALUATION_BASIS                                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 5247
#define RSSPECAN_ATTR_TRANSIENT_HOP_SELECTION                                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 5248
#define RSSPECAN_ATTR_TRANSIENT_CHIRP_SELECTION                                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 5249
#define RSSPECAN_ATTR_TRANSIENT_STATISTIC_TYPE                                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 5250
#define RSSPECAN_ATTR_TRANSIENT_TRACE_MAXIMUM_POINTS                                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 5251
#define RSSPECAN_ATTR_TRANSIENT_TRACE_LONG_CAPTURE_BUFFER                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 11043
#define RSSPECAN_ATTR_TRANSIENT_WINDOW_STATISTIC_TYPE                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 7146
#define RSSPECAN_ATTR_TRANSIENT_TRACE_LENGTH                                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 5252
#define RSSPECAN_ATTR_TRANSIENT_TRACE_EVALUATION                                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 6438
#define RSSPECAN_ATTR_TRANSIENT_SPECTROGRAM_DETECTOR                                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 5253
#define RSSPECAN_ATTR_TRANSIENT_LINKED_MARKER                                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 5254
#define RSSPECAN_ATTR_TRANSIENT_LINKED_DELTAMARKER                                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 5255
#define RSSPECAN_ATTR_TRANSIENT_STORE_CURRENT_MEASUREMENT                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 5256
#define RSSPECAN_ATTR_TRANSIENT_TOTAL_HOPS                                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 5257
#define RSSPECAN_ATTR_TRANSIENT_TOTAL_HOPS_IN_CAPTURE_BUFFER                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 7147
#define RSSPECAN_ATTR_TRANSIENT_TOTAL_CHIRPS                                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 5265
#define RSSPECAN_ATTR_TRANSIENT_TOTAL_CHIRPS_IN_CAPTURE_BUFFER                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 7148
#define RSSPECAN_ATTR_DOCSIS_MODE                                                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 5908
#define RSSPECAN_ATTR_DOCSIS_OFDM_SPECTRUM_LOCATION                                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 5909
#define RSSPECAN_ATTR_DOCSIS_STREAM_DIRECTION                                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 6211
#define RSSPECAN_ATTR_DOCSIS_N_FFT                                                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 5910
#define RSSPECAN_ATTR_DOCSIS_CYCLIC_PREFIX_CP                                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 5911
#define RSSPECAN_ATTR_DOCSIS_ROLL_OFF                                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 5912
#define RSSPECAN_ATTR_DOCSIS_TIME_INTERLEAVING_DEPTH                                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 5913
#define RSSPECAN_ATTR_DOCSIS_PLC_START_INDEX_L                                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 5914
#define RSSPECAN_ATTR_DOCSIS_PLC_START_INDEX_L_AUTO                                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 5915
#define RSSPECAN_ATTR_DOCSIS_PLC_MODULATION                                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 5916
#define RSSPECAN_ATTR_DOCSIS_PLC_NUMBER_OF_SUBCARRIERS                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 5917
#define RSSPECAN_ATTR_DOCSIS_NCP_MODULATION                                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 6621
#define RSSPECAN_ATTR_DOCSIS_OFDM_US_SPECTRUM_LOCATION                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 6455
#define RSSPECAN_ATTR_DOCSIS_CONTINUOUS_PILOTS_AND_EXCLUDED_NUMBER_OF_ENTRIES                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 5919
#define RSSPECAN_ATTR_DOCSIS_CONTINUOUS_PILOTS_AND_EXCLUDED_SUBCARRIER_TABLE_DELETE_ROW                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 6986
#define RSSPECAN_ATTR_DOCSIS_CONTINUOUS_PILOTS_AND_EXCLUDED_SUBCARRIER_TABLE_DELETE_ALL                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 6987
#define RSSPECAN_ATTR_DOCSIS_CONFIGURE_TYPE                                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 5920
#define RSSPECAN_ATTR_DOCSIS_CONTINUOUS_PILOTS_AND_EXCLUDED_SUBCARRIER_START                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 5921
#define RSSPECAN_ATTR_DOCSIS_CONTINUOUS_PILOTS_AND_EXCLUDED_SUBCARRIER_STOP                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 5922
#define RSSPECAN_ATTR_DOCSIS_CONTINUOUS_PILOTS_AND_EXCLUDED_SUBCARRIER_RANGE_INCREMENT                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 5923
#define RSSPECAN_ATTR_DOCSIS_AUTO_DETECTION_CONTINUOUS_PILOTS                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 5925
#define RSSPECAN_ATTR_DOCSIS_SELECT_PROFILE_LIST                                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 5926
#define RSSPECAN_ATTR_DOCSIS_PROFILE_CONFIGURATION_NUMBER_OF_ENTRIES                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 5927
#define RSSPECAN_ATTR_DOCSIS_PROFILE_CONFIGURATION_DELETE                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 5928
#define RSSPECAN_ATTR_DOCSIS_PROFILE_QUERY_STATE                                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 5929
#define RSSPECAN_ATTR_DOCSIS_MODULATION                                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 5930
#define RSSPECAN_ATTR_DOCSIS_MODULATION_SUBCARRIER_START                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 5931
#define RSSPECAN_ATTR_DOCSIS_MODULATION_SUBCARRIER_STOP                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 5932
#define RSSPECAN_ATTR_DOCSIS_MODULATION_SUBCARRIER_INCREMENT                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 5933
#define RSSPECAN_ATTR_DOCSIS_MODULATION_SUBCARRIER_ASSIGNMENT_TABLE_DELETE_SUBCARRIER                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 6988
#define RSSPECAN_ATTR_DOCSIS_MODULATION_SUBCARRIER_ASSIGNMENT_TABLE_DELETE_SUBCARRIER_SET                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 6989
#define RSSPECAN_ATTR_DOCSIS_NEXT_CODEWORD_POINTER_NUMBER_OF_ENTRIES                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 6456
#define RSSPECAN_ATTR_DOCSIS_NEXT_CODEWORD_POINTER_PROFILE_DELETE_ROW                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 6990
#define RSSPECAN_ATTR_DOCSIS_NEXT_CODEWORD_POINTER_PROFILE_DELETE_ALL                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 6991
#define RSSPECAN_ATTR_DOCSIS_NEXT_CODEWORD_POINTER_MODULATION                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 6457
#define RSSPECAN_ATTR_DOCSIS_NEXT_CODEWORD_POINTER_START                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 6458
#define RSSPECAN_ATTR_DOCSIS_NEXT_CODEWORD_POINTER_STOP                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 6459
#define RSSPECAN_ATTR_DOCSIS_NEXT_CODEWORD_POINTER_INCREMENT                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 6460
#define RSSPECAN_ATTR_DOCSIS_CODEWORD_NUMBER_OF_ROWS                                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 5935
#define RSSPECAN_ATTR_DOCSIS_CODEWORD_FRAME_TABLE_DELETE_ROW                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 6992
#define RSSPECAN_ATTR_DOCSIS_CODEWORD_FRAME_TABLE_DELETE_ALL                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 6993
#define RSSPECAN_ATTR_DOCSIS_CODEWORD_ASSIGN_PROFILE                                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 5936
#define RSSPECAN_ATTR_DOCSIS_CODEWORD_FRAME_FIRST_SUBCARRIER                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 5937
#define RSSPECAN_ATTR_DOCSIS_CODEWORD_FRAME_NUMBER_OF_SUBCARRIERS                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 5938
#define RSSPECAN_ATTR_DOCSIS_CODEWORD_FRAME_FIRST_SYMBOL                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 5939
#define RSSPECAN_ATTR_DOCSIS_CODEWORD_FRAME_NUMBER_OF_SYMBOLS                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 5940
#define RSSPECAN_ATTR_DOCSIS_CODEWORD_DETECTION_NCP_AUTO                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 5941
#define RSSPECAN_ATTR_DOCSIS_UPSTREAM_SYMBOLS_PER_FRAME                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 6212
#define RSSPECAN_ATTR_DOCSIS_UPSTREAM_EXCLUDED_SUBCARRIER_NUMBER_OF_ENTRIES                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 6213
#define RSSPECAN_ATTR_DOCSIS_UPSTREAM_EXCLUDED_SUBCARRIER_TABLE_DELETE_ROW                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 6994
#define RSSPECAN_ATTR_DOCSIS_UPSTREAM_EXCLUDED_SUBCARRIER_TABLE_DELETE_ALL                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 6995
#define RSSPECAN_ATTR_DOCSIS_UPSTREAM_EXCLUDED_SUBCARRIER_TYPE                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 6214
#define RSSPECAN_ATTR_DOCSIS_UPSTREAM_EXCLUDED_SUBCARRIER_START                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 6215
#define RSSPECAN_ATTR_DOCSIS_UPSTREAM_EXCLUDED_SUBCARRIER_STOP                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 6216
#define RSSPECAN_ATTR_DOCSIS_UPSTREAM_EXCLUDED_SUBCARRIER_RANGE_INCREMENT                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 6217
#define RSSPECAN_ATTR_DOCSIS_UPSTREAM_PROFILE_CONFIGURATION_NUMBER_OF_MINISLOTS                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 6219
#define RSSPECAN_ATTR_DOCSIS_UPSTREAM_PROFILE_CONFIGURATION_TABLE_DELETE_ROW                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 6996
#define RSSPECAN_ATTR_DOCSIS_UPSTREAM_PROFILE_CONFIGURATION_TABLE_DELETE_ALL                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 6997
#define RSSPECAN_ATTR_DOCSIS_UPSTREAM_PROFILE_CONFIGURATION_FIRST_MINISLOT                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 6788
#define RSSPECAN_ATTR_DOCSIS_UPSTREAM_PROFILE_CONFIGURATION_MODULATION                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 6220
#define RSSPECAN_ATTR_DOCSIS_UPSTREAM_PROFILE_CONFIGURATION_PILOT_PATTERN                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 6221
#define RSSPECAN_ATTR_DOCSIS_INPUT_DIRECT_PATH                                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 5942
#define RSSPECAN_ATTR_DOCSIS_SIGNAL_CAPTURE_NUMBER_OF_SAMPLES                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 5943
#define RSSPECAN_ATTR_DOCSIS_SIGNAL_CAPTURE_FILTER_OUT_ADJACENT_CHANNELS                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 6998
#define RSSPECAN_ATTR_DOCSIS_POWER_INTERVAL_SEARCH                                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 8675
#define RSSPECAN_ATTR_DOCSIS_CHANNEL_ESTIMATION                                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 6008
#define RSSPECAN_ATTR_DOCSIS_PHASE_TRACKING                                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 5944
#define RSSPECAN_ATTR_DOCSIS_TIME_TRACKING                                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 5945
#define RSSPECAN_ATTR_DOCSIS_CODEWORDS_DECODED                                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 5946
#define RSSPECAN_ATTR_DOCSIS_BITSTREAM_DECODED                                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 6009
#define RSSPECAN_ATTR_DOCSIS_UPSTREAM_AUTO_DETECTION                                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 6789
#define RSSPECAN_ATTR_DOCSIS_EVALUATION_SPECIFIED_FRAME_STATE                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 6010
#define RSSPECAN_ATTR_DOCSIS_EVALUATION_SELECTED_FRAME                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 6011
#define RSSPECAN_ATTR_DOCSIS_EVALUATION_FRAME_STATISTIC_COUNT_STATE                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 6012
#define RSSPECAN_ATTR_DOCSIS_EVALUATION_NUMBER_OF_FRAME                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 6013
#define RSSPECAN_ATTR_DOCSIS_LIMIT_CHECK_STATE                                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 6461
#define RSSPECAN_ATTR_DOCSIS_EVALUATION_RANGE_MER_EXCLUDING_SUBCARRIERS_MODE                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 6222
#define RSSPECAN_ATTR_DOCSIS_EVALUATION_RANGE_MER_EXCLUDING_SUBCARRIERS_COUNT                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 6223
#define RSSPECAN_ATTR_DOCSIS_EVALUATION_CONSTELLATION_MODULATION                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 6014
#define RSSPECAN_ATTR_DOCSIS_EVALUATION_CONSTELLATION_OBJECT                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 6015
#define RSSPECAN_ATTR_DOCSIS_EVALUATION_CONSTELLATION_DIAGRAM_SUBCARRIER_ALL                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 5947
#define RSSPECAN_ATTR_DOCSIS_EVALUATION_RANGE_CONSTELLATION_DIAGRAM_SUBCARRIER                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 5948
#define RSSPECAN_ATTR_DOCSIS_EVALUATION_RANGE_CONSTELLATION_DIAGRAM_SYMBOL_ALL                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 5949
#define RSSPECAN_ATTR_DOCSIS_EVALUATION_RANGE_CONSTELLATION_DIAGRAM_SYMBOL                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 5950
#define RSSPECAN_ATTR_DOCSIS_EVALUATION_RANGE_CONSTELLATION_FAST_MODE                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 6999
#define RSSPECAN_ATTR_DOCSIS_EVALUATION_RANGE_SYMBOL_SIZE                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 6462
#define RSSPECAN_ATTR_DOCSIS_CARRIER_AXES_UNIT                                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 5951
#define RSSPECAN_ATTR_DOCSIS_AUTO_DETECT                                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 6463
#define RSSPECAN_ATTR_DOCSIS_RESULT_SUMMARY_UNIT                                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 6464
#define RSSPECAN_ATTR_DOCSIS_NORMALIZE_CM_SAMPLE_CLOCK_ERROR_RESULT                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 7000
#define RSSPECAN_ATTR_DOCSIS_CMTS_REFERENCE_SAMPLE_CLOCK                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 7001
#define RSSPECAN_ATTR_DOCSIS_CMTS_REFERENCE_SAMPLE_CLOCK_UNIT                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 7002
#define RSSPECAN_ATTR_DOCSIS_BITSREAM_FORMAT                                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 6017
#define RSSPECAN_ATTR_DOCSIS_BITSTREAM_LAYOUT                                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 6018
#define RSSPECAN_ATTR_DOCSIS_AUTOMATIC_GRID_SCALING                                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 5952
#define RSSPECAN_ATTR_DOCSIS_AUTO_FIX_RANGE                                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 5953
#define RSSPECAN_ATTR_DOCSIS_AUTO_MODE                                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 5954
#define RSSPECAN_ATTR_DOCSIS_HYSTERESIS_INTERVAL_LOWER_UPPER                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 5955
#define RSSPECAN_ATTR_DOCSIS_HYSTERESIS_INTERVAL_LOWER_LOWER                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 5956
#define RSSPECAN_ATTR_DOCSIS_HYSTERESIS_INTERVAL_UPPER_LOWER                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 5957
#define RSSPECAN_ATTR_DOCSIS_HYSTERESIS_INTERVAL_UPPER_UPPER                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 5958
#define RSSPECAN_ATTR_DOCSIS_MEMORY_DEPTH                                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 5959
#define RSSPECAN_ATTR_DOCSIS_NUMBER_OF_DIVISIONS                                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 5960
#define RSSPECAN_ATTR_DOCSIS_Y_SCALE_MINIMUM                                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 6004
#define RSSPECAN_ATTR_DOCSIS_Y_SCALE_MAXIMUM                                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 6005
#define RSSPECAN_ATTR_DOCSIS_POWER_UNIT                                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 6790
#define RSSPECAN_ATTR_DOCSIS_AUTO_CONFIGURATION_PER_CONTINUOUS_MINISLOT_BLOCK                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 6791
#define RSSPECAN_ATTR_DOCSIS_AUTOMATIC_BAND_CONFIGURATION                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 6792
#define RSSPECAN_ATTR_DOCSIS_ADJACENT_CHANNEL_BANDS_OFFSET_SUBCARRIERS                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 6793
#define RSSPECAN_ATTR_DOCSIS_ALTERNATE_CHANNEL_BANDS_OFFSET_SUBCARRIERS                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 6794
#define RSSPECAN_ATTR_DOCSIS_ADJACENT_CHANNEL_BANDS_OFFSET_FREQUENCY                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 6795
#define RSSPECAN_ATTR_DOCSIS_ALTERNATE_CHANNEL_BANDS_OFFSET_FREQUENCY                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 6796
#define RSSPECAN_ATTR_DOCSIS_ADJACENT_CHANNEL_BANDS_BANDWIDTH_SUBCARRIERS                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 6797
#define RSSPECAN_ATTR_DOCSIS_ALTERNATE_CHANNEL_BANDS_BANDWIDTH_SUBCARRIERS                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 6798
#define RSSPECAN_ATTR_DOCSIS_ADJACENT_CHANNEL_BANDS_BANDWIDTH_FREQUENCY                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 6799
#define RSSPECAN_ATTR_DOCSIS_ALTERNATE_CHANNEL_BANDS_BANDWIDTH_FREQUENCY                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 6800
#define RSSPECAN_ATTR_DOCSIS_AUTO_BANDS_APPLIED_TO                                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 6801
#define RSSPECAN_ATTR_DOCSIS_USER_CONFIG_BAND_STATE                                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 6802
#define RSSPECAN_ATTR_DOCSIS_USER_CONFIG_LOCATION_REFERENCE                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 6803
#define RSSPECAN_ATTR_DOCSIS_USER_CONFIG_LOCATION_SUBCARRIERS                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 6804
#define RSSPECAN_ATTR_DOCSIS_USER_CONFIG_BANDWIDTH_SUBCARRIERS                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 6805
#define RSSPECAN_ATTR_DOCSIS_USER_CONFIG_BANDWIDTH_FREQUENCY                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 6806
#define RSSPECAN_ATTR_DOCSIS_WINDOW_TYPE                                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 7004
#define RSSPECAN_ATTR_DOCSIS_AUTO_SET_FROM_PLC_AND_RUN                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 7005
#define RSSPECAN_ATTR_DOCSIS_AUTO_DETECTION_AND_RUN                                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 7006
#define RSSPECAN_ATTR_DOCSIS_FETCH_CENTER_ERROR_AVERAGE                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 5963
#define RSSPECAN_ATTR_DOCSIS_FETCH_CENTER_ERROR_MAXIMUM                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 5964
#define RSSPECAN_ATTR_DOCSIS_FETCH_CENTER_ERROR_MINIMUM                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 5965
#define RSSPECAN_ATTR_DOCSIS_FETCH_FREQUENCY_ERROR_AVERAGE                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 5966
#define RSSPECAN_ATTR_DOCSIS_FETCH_FREQUENCY_ERROR_MAXIMUM                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 5967
#define RSSPECAN_ATTR_DOCSIS_FETCH_FREQUENCY_ERROR_MINIMUM                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 5968
#define RSSPECAN_ATTR_DOCSIS_FETCH_MODULATION_ERROR_DATA_PILOT_AVERAGE                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 5969
#define RSSPECAN_ATTR_DOCSIS_FETCH_MODULATION_ERROR_DATA_PILOT_MAXIMUM                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 5970
#define RSSPECAN_ATTR_DOCSIS_FETCH_MODULATION_ERROR_DATA_PILOT_MINIMUM                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 5971
#define RSSPECAN_ATTR_DOCSIS_FETCH_MODULATION_ERROR_DATA_AVERAGE                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 5972
#define RSSPECAN_ATTR_DOCSIS_FETCH_MODULATION_ERROR_DATA_MAXIMUM                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 5973
#define RSSPECAN_ATTR_DOCSIS_FETCH_MODULATION_ERROR_DATA_MINIMUM                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 5974
#define RSSPECAN_ATTR_DOCSIS_FETCH_MODULATION_ERROR_PILOT_AVERAGE                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 5975
#define RSSPECAN_ATTR_DOCSIS_FETCH_MODULATION_ERROR_PILOT_MAXIMUM                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 5976
#define RSSPECAN_ATTR_DOCSIS_FETCH_MODULATION_ERROR_PILOT_MINIMUM                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 5977
#define RSSPECAN_ATTR_DOCSIS_FETCH_PHASE_ERROR_AVERAGE                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 5978
#define RSSPECAN_ATTR_DOCSIS_FETCH_PHASE_ERROR_MAXIMUM                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 5979
#define RSSPECAN_ATTR_DOCSIS_FETCH_PHASE_ERROR_MINIMUM                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 5980
#define RSSPECAN_ATTR_DOCSIS_FETCH_POWER_AVERAGE                                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 5981
#define RSSPECAN_ATTR_DOCSIS_FETCH_POWER_MAXIMUM                                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 5982
#define RSSPECAN_ATTR_DOCSIS_FETCH_POWER_MINIMUM                                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 5983
#define RSSPECAN_ATTR_DOCSIS_FETCH_POWER_ANALYZED_MINISLOTS_AVERAGE                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 7007
#define RSSPECAN_ATTR_DOCSIS_FETCH_POWER_ANALYZED_MINISLOTS_MAXIMUM                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 7008
#define RSSPECAN_ATTR_DOCSIS_FETCH_POWER_ANALYZED_MINISLOTS_MINIMUM                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 7009
#define RSSPECAN_ATTR_DOCSIS_FETCH_TPLC_AVERAGE                                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 5984
#define RSSPECAN_ATTR_DOCSIS_FETCH_TPLC_MAXIMUM                                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 5985
#define RSSPECAN_ATTR_DOCSIS_FETCH_TPLC_MINIMUM                                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 5986
#define RSSPECAN_ATTR_DOCSIS_FETCH_ZBIT_AVERAGE                                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 6022
#define RSSPECAN_ATTR_DOCSIS_FETCH_ZBIT_MAXIMUM                                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 6023
#define RSSPECAN_ATTR_DOCSIS_FETCH_ZBIT_MINIMUM                                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 6024
#define RSSPECAN_ATTR_DOCSIS_FETCH_CONTINUOUS_PILOTS_AVERAGE                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 6465
#define RSSPECAN_ATTR_DOCSIS_FETCH_CONTINUOUS_PILOTS_MAXIMUM                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 6466
#define RSSPECAN_ATTR_DOCSIS_FETCH_CONTINUOUS_PILOTS_MINIMUM                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 6467
#define RSSPECAN_ATTR_DOCSIS_FETCH_DATA_AVERAGE                                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 6468
#define RSSPECAN_ATTR_DOCSIS_FETCH_DATA_MAXIMUM                                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 6469
#define RSSPECAN_ATTR_DOCSIS_FETCH_DATA_MINIMUM                                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 6470
#define RSSPECAN_ATTR_DOCSIS_FETCH_POWER_PILOTS_AVERAGE                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 6471
#define RSSPECAN_ATTR_DOCSIS_FETCH_POWER_PILOTS_MAXIMUM                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 6472
#define RSSPECAN_ATTR_DOCSIS_FETCH_POWER_PILOTS_MINIMUM                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 6473
#define RSSPECAN_ATTR_DOCSIS_FETCH_SCATTERED_PILOTS_AVERAGE                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 6474
#define RSSPECAN_ATTR_DOCSIS_FETCH_SCATTERED_PILOTS_MAXIMUM                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 6475
#define RSSPECAN_ATTR_DOCSIS_FETCH_SCATTERED_PILOTS_MINIMUM                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 6476
#define RSSPECAN_ATTR_DOCSIS_FETCH_PHYSICAL_LINK_CHANNEL_AVERAGE                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 6477
#define RSSPECAN_ATTR_DOCSIS_FETCH_PHYSICAL_LINK_CHANNEL_MAXIMUM                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 6478
#define RSSPECAN_ATTR_DOCSIS_FETCH_PHYSICAL_LINK_CHANNEL_MINIMUM                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 6479
#define RSSPECAN_ATTR_DOCSIS_FETCH_TRIGGER_TO_FRAME_AVERAGE                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 7010
#define RSSPECAN_ATTR_DOCSIS_FETCH_TRIGGER_TO_FRAME_MAXIMUM                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 7011
#define RSSPECAN_ATTR_DOCSIS_FETCH_TRIGGER_TO_FRAME_MINIMUM                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 7012
#define RSSPECAN_ATTR_DOCSIS_FETCH_FRAME_COUNT                                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 6025
#define RSSPECAN_ATTR_DOCSIS_FETCH_FRAME_COUNT_ALL                                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 6026
#define RSSPECAN_ATTR_DOCSIS_FETCH_CYCLIC_PREFIX_CP                                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 7013
#define RSSPECAN_ATTR_DOCSIS_SYNCHRONOUS_POWER_BAND_RESULTS                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 6807
#define RSSPECAN_ATTR_DOCSIS_FETCH_PLC_TIMESTAMP                                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 6622
#define RSSPECAN_ATTR_DOCSIS_FETCH_PLC_NCP_CHANGE_COUNT                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 6624
#define RSSPECAN_ATTR_DOCSIS_FETCH_PLC_NCP_CHANNEL_ID                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 6625
#define RSSPECAN_ATTR_DOCSIS_FETCH_PLC_NCP_MODULATION                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 6626
#define RSSPECAN_ATTR_DOCSIS_FETCH_PLC_OFDM_CHANGE_COUNT                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 6627
#define RSSPECAN_ATTR_DOCSIS_FETCH_PLC_OFDM_CYCLIC_PREFIX_LENGTH                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 6628
#define RSSPECAN_ATTR_DOCSIS_FETCH_PLC_OFDM_CHANNEL_ID                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 6630
#define RSSPECAN_ATTR_DOCSIS_FETCH_PLC_OFDM_FFT_LENGTH                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 6631
#define RSSPECAN_ATTR_DOCSIS_FETCH_PLC_OFDM_START_INDEX_L                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 6633
#define RSSPECAN_ATTR_DOCSIS_FETCH_PLC_OFDM_ROLL_OFF                                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 6634
#define RSSPECAN_ATTR_DOCSIS_FETCH_PLC_OFDM_SPECTRUM_LOCATION                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 6635
#define RSSPECAN_ATTR_DOCSIS_FETCH_PLC_OFDM_TIME_INTERLEAVING_DEPTH                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 6636
#define RSSPECAN_ATTR_DOCSIS_FETCH_PLC_PROFILE_CHANGE_COUNT                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 6638
#define RSSPECAN_ATTR_DOCSIS_FETCH_PLC_PROFILE_CHANNEL_ID                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 6639
#define RSSPECAN_ATTR_DOCSIS_QUERY_CLOCK_ERROR_LIMIT_AVERAGE                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 5987
#define RSSPECAN_ATTR_DOCSIS_QUERY_CLOCK_ERROR_LIMIT_AVERAGE_RESULT                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 5988
#define RSSPECAN_ATTR_DOCSIS_QUERY_CLOCK_ERROR_LIMIT_MAXIMUM                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 5989
#define RSSPECAN_ATTR_DOCSIS_QUERY_CLOCK_ERROR_LIMIT_CHECK_MAXIMUM_RESULT                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 5990
#define RSSPECAN_ATTR_DOCSIS_QUERY_FREQUENCY_ERROR_LIMIT_AVERAGE                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 5991
#define RSSPECAN_ATTR_DOCSIS_QUERY_FREQUENCY_ERROR_LIMIT_CHECK_AVERAGE_RESULT                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 5992
#define RSSPECAN_ATTR_DOCSIS_QUERY_FREQUENCY_ERROR_LIMIT_MAXIMUM                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 5993
#define RSSPECAN_ATTR_DOCSIS_QUERY_FREQUENCY_ERROR_LIMIT_CHECK_MAXIMUM_RESULT                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 5994
#define RSSPECAN_ATTR_DOCSIS_QUERY_MODULATION_ERROR_RATIO_LIMIT_MINIMUM                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 5995
#define RSSPECAN_ATTR_DOCSIS_QUERY_MODULATION_ERROR_RATIO_LIMIT_CHECK_MINIMUM_RESULT                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 5996
#define RSSPECAN_ATTR_DOCSIS_QUERY_MODULATION_ERROR_RATIO_DATA_LIMIT_MINIMUM                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 5997
#define RSSPECAN_ATTR_DOCSIS_QUERY_MODULATION_ERROR_RATIO_DATA_LIMIT_CHECK_MINIMUM_RESULT                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 5998
#define RSSPECAN_ATTR_DOCSIS_QUERY_MODULATION_ERROR_RATIO_PILOT_LIMIT_MINIMUM                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 5999
#define RSSPECAN_ATTR_DOCSIS_QUERY_MODULATION_ERROR_RATIO_PILOT_LIMIT_CHECK_MINIMUM_RESULT                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 6000
#define RSSPECAN_ATTR_DOCSIS_MARKER_Z_AXIS                                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 7014
#define RSSPECAN_ATTR_AVIONICS_MODE                                                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 6258
#define RSSPECAN_ATTR_AVIONICS_MEASUREMENT_TYPE                                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 6233
#define RSSPECAN_ATTR_AVIONICS_FUNDAMENTAL_FREQUENCY_IDENTITY                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 6234
#define RSSPECAN_ATTR_AVIONICS_TOTAL_HARMONIC_DISTORTION_FREQUENCY                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 6235
#define RSSPECAN_ATTR_AVIONICS_HARMONIC_DISTORTION_STATE                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 6245
#define RSSPECAN_ATTR_AVIONICS_HARMONIC_FREQUENCY                                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 6246
#define RSSPECAN_ATTR_AVIONICS_BANDWIDTH_AUTO                                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 6257
#define RSSPECAN_ATTR_AVIONICS_DEMODULATION_BANDWIDTH                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 6500
#define RSSPECAN_ATTR_AVIONICS_RESOLUTION_BANDWIDTH_STATE                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 6501
#define RSSPECAN_ATTR_AVIONICS_RESOLUTION_BANDWIDTH                                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 6259
#define RSSPECAN_ATTR_AVIONICS_AF_START_FREQUENCY                                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 6260
#define RSSPECAN_ATTR_AVIONICS_AF_STOP_FREQUENCY                                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 6261
#define RSSPECAN_ATTR_AVIONICS_AF_CENTER_FREQUENCY                                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 6262
#define RSSPECAN_ATTR_AVIONICS_AF_SPAN                                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 6263
#define RSSPECAN_ATTR_AVIONICS_AF_FULL_SPAN                                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 6264
#define RSSPECAN_ATTR_AVIONICS_ILS_DDM_UNIT                                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 6248
#define RSSPECAN_ATTR_AVIONICS_VOR_DIRECTION                                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 6249
#define RSSPECAN_ATTR_AVIONICS_MORSE_CODE                                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 6236
#define RSSPECAN_ATTR_AVIONICS_RF_LEVEL_RESULTS                                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 6237
#define RSSPECAN_ATTR_AVIONICS_DIFFERENCE_IN_DEPTH_OF_MODULATION                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 6238
#define RSSPECAN_ATTR_AVIONICS_CARRIER_OFFSET_RESULTS                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 6239
#define RSSPECAN_ATTR_AVIONICS_VOR_FM_DEVIATION                                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 6241
#define RSSPECAN_ATTR_AVIONICS_VOR_FM_FREQUENCY                                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 6240
#define RSSPECAN_ATTR_AVIONICS_PHASE_RESULTS                                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 6242
#define RSSPECAN_ATTR_AVIONICS_RF_FREQUENCY_SIGNAL                                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 6243
#define RSSPECAN_ATTR_AVIONICS_ILS_SDM                                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 6244
#define RSSPECAN_ATTR_AVIONICS_HARMONIC_DISTORTION_RESULT                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 6247
#define RSSPECAN_ATTR_5G_NR_MODE                                                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 7216
#define RSSPECAN_ATTR_5G_NR_MEASUREMENT                                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 7217
#define RSSPECAN_ATTR_5G_NR_DIRECTION                                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 7218
#define RSSPECAN_ATTR_5G_NR_EVM_UNIT                                                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 8369
#define RSSPECAN_ATTR_5G_NR_O_RAN_TESTCASE                                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 11147
#define RSSPECAN_ATTR_5G_NR_CHANNEL_SPACING_RESET                                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 11148
#define RSSPECAN_ATTR_5G_NR_SWEEP_MODE                                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 11149
#define RSSPECAN_ATTR_5G_NR_DL_CONFIGURATION_MULTICARRIER_FILTER                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 7219
#define RSSPECAN_ATTR_5G_NR_DL_COMPONENT_CARRIER_NUMBER                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 7220
#define RSSPECAN_ATTR_5G_NR_DL_COMPONENT_CARRIER_DATA_CAPTURE                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 7221
#define RSSPECAN_ATTR_5G_NR_DL_AUTO_EVM                                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 8437
#define RSSPECAN_ATTR_5G_NR_DL_AUTO_EVM_SLOTS                                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 10509
#define RSSPECAN_ATTR_5G_NR_DL_AUTO_LEVEL_ALL                                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 8438
#define RSSPECAN_ATTR_5G_NR_DL_PHYSICAL_SETTINGS_OPERATING_BAND                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 7222
#define RSSPECAN_ATTR_5G_NR_DL_PHYSICAL_SETTINGS_SAVE_SIGNAL_DESCRIPTION                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 7225
#define RSSPECAN_ATTR_5G_NR_DL_PHYSICAL_SETTINGS_LOAD_SIGNAL_DESCRIPTION                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 7223
#define RSSPECAN_ATTR_5G_NR_DL_GENERAL_FREQUENCY_OFFSET                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 7227
#define RSSPECAN_ATTR_5G_NR_DL_GENERAL_CHANNEL_RASTER                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 10510
#define RSSPECAN_ATTR_5G_NR_DL_GENERAL_GLOBAL_MC_FREQUENCY                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 10511
#define RSSPECAN_ATTR_5G_NR_DL_GENERAL_SYNC_GLOBAL_MC_FREQUENCY_TO_CENTER                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 10512
#define RSSPECAN_ATTR_5G_NR_DL_GENERAL_FREQUENCY_OFFSET_REFERENCE_POINT                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 10513
#define RSSPECAN_ATTR_5G_NR_DL_GENERAL_FREQUENCY_OFFSET_MODE                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 10514
#define RSSPECAN_ATTR_5G_NR_DL_GENERAL_FREQUENCY_OFFSET_CARRIER_SPACING                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 10515
#define RSSPECAN_ATTR_5G_NR_DL_GENERAL_FREQUENCY_OFFSET_FIXED                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 10516
#define RSSPECAN_ATTR_5G_NR_DL_PHYSICAL_SETTINGS_BANDWIDTH                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 7228
#define RSSPECAN_ATTR_5G_NR_DL_DEPLOYMENT_FREQUENCY_RANGE                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 7229
#define RSSPECAN_ATTR_5G_NR_DL_CELL_ID_AUTO                                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 7230
#define RSSPECAN_ATTR_5G_NR_DL_CELL_ID                                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 7231
#define RSSPECAN_ATTR_5G_NR_DL_CENTER_FREQUENCY                                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 7232
#define RSSPECAN_ATTR_5G_NR_DL_COMPONENT_CARRIER_GENERAL_FREQUENCY_OFFSET                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 7233
#define RSSPECAN_ATTR_5G_NR_DL_COMPONENT_CARRIER_GENERAL_MC_FREQUENCY_OFFSET                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 10517
#define RSSPECAN_ATTR_5G_NR_DL_SAVE_SIGNAL_DESCRIPTION                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 7236
#define RSSPECAN_ATTR_5G_NR_DL_LOAD_SIGNAL_DESCRIPTION                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 7234
#define RSSPECAN_ATTR_5G_NR_DL_LOAD_TEST_MODEL                                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 8439
#define RSSPECAN_ATTR_5G_NR_DL_DATA_CAPTURE_FRAME_COUNT_ENABLED                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 7238
#define RSSPECAN_ATTR_5G_NR_DL_DATA_CAPTURE_FRAME_COUNT_AUTO_ENABLED                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 8441
#define RSSPECAN_ATTR_5G_NR_DL_DATA_CAPTURE_FRAME_COUNT                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 7239
#define RSSPECAN_ATTR_5G_NR_DL_DATA_CAPTURE_MAXIMUM_SLOTS_PER_FRAME_TO_ANALYZE                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 8442
#define RSSPECAN_ATTR_5G_NR_DL_DATA_CAPTURE_ALL_SLOTS_PER_FRAME_TO_ANALYZE                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 8443
#define RSSPECAN_ATTR_5G_NR_DL_DATA_CAPTURE_SUBFRAMES_TO_ANALYZE                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 7240
#define RSSPECAN_ATTR_5G_NR_DL_DATA_CAPTURE_LONG_CAPTURE_ENABLED                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 8444
#define RSSPECAN_ATTR_5G_NR_DL_DATA_CAPTURE_SIGNAL_REPEATS_MAX_NO_OF_SLOTS_TO_ANALYZE                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 8676
#define RSSPECAN_ATTR_5G_NR_DL_BANDWIDTH_PART_ADD                                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 7241
#define RSSPECAN_ATTR_5G_NR_DL_BANDWIDTH_PART_CLEAR                                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 7242
#define RSSPECAN_ATTR_5G_NR_DL_BANDWIDTH_PART_ANALYZED                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 7243
#define RSSPECAN_ATTR_5G_NR_DL_BANDWIDTH_PART_DUPLICATE                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 7244
#define RSSPECAN_ATTR_5G_NR_DL_BANDWIDTH_PART_RB_COUNT                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 7245
#define RSSPECAN_ATTR_5G_NR_DL_BANDWIDTH_PART_RB_OFFSET                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 7246
#define RSSPECAN_ATTR_5G_NR_DL_BANDWIDTH_PART_DELETE                                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 7247
#define RSSPECAN_ATTR_5G_NR_DL_BANDWIDTH_PART_SUBCARRIER_SPACING                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 7248
#define RSSPECAN_ATTR_5G_NR_DL_GENERATOR_IP_ADDRESS                                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 9546
#define RSSPECAN_ATTR_5G_NR_DL_GENERATOR_LED_STATE                                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 9547
#define RSSPECAN_ATTR_5G_NR_DL_GENERATOR_CONTROL_ENABLED                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 9548
#define RSSPECAN_ATTR_5G_NR_DL_GENERATOR_RF_OUTPUT_ENABLED                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 9549
#define RSSPECAN_ATTR_5G_NR_DL_GENERATOR_FREQUENCY_SYNCHRONIZATION                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 9550
#define RSSPECAN_ATTR_5G_NR_DL_GENERATOR_LEVEL_CONTROL_ENABLED                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 9551
#define RSSPECAN_ATTR_5G_NR_DL_GENERATOR_POWER_LEVEL                                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 9552
#define RSSPECAN_ATTR_5G_NR_DL_GENERATOR_LEVEL_DUT_LIMIT_ENABLED                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 9553
#define RSSPECAN_ATTR_5G_NR_DL_GENERATOR_LEVEL_DUT_LIMIT                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 9554
#define RSSPECAN_ATTR_5G_NR_DL_GENERATOR_LEVEL_DUT_GAIN                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 9555
#define RSSPECAN_ATTR_5G_NR_DL_GENERATOR_UPLOAD_RF_SETTINGS                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 9556
#define RSSPECAN_ATTR_5G_NR_DL_GENERATOR_DOWNLOAD_RF_SETTINGS                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 9557
#define RSSPECAN_ATTR_5G_NR_DL_GENERATOR_DOWNLOAD_NR5G_SETTINGS                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 9558
#define RSSPECAN_ATTR_5G_NR_DL_GENERATOR_SETTINGS_PERIODIC_SYNCHRONIZATION_ENABLED                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 9559
#define RSSPECAN_ATTR_5G_NR_DL_GENERATOR_SETTINGS_PERIODIC_SYNCHRONIZATION_INTERVAL                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 9560
#define RSSPECAN_ATTR_5G_NR_DL_FRAME_PASTE_ALL                                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 7249
#define RSSPECAN_ATTR_5G_NR_DL_NUMBER_OF_CONFIGURABLE_FRAMES                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 7250
#define RSSPECAN_ATTR_5G_NR_DL_COMPLETE_SIGNAL_DEMODULATION_ENABLED                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 8445
#define RSSPECAN_ATTR_5G_NR_DL_BANDWIDTH_PARTS_DEMODULATION                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 8446
#define RSSPECAN_ATTR_5G_NR_DL_FRAME_COPY                                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 7251
#define RSSPECAN_ATTR_5G_NR_DL_FRAME_PASTE                                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 7252
#define RSSPECAN_ATTR_5G_NR_DL_NUMBER_OF_SLOTS_TO_CONFIGURE                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 7253
#define RSSPECAN_ATTR_5G_NR_DL_SLOTS_COUNTS                                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 7254
#define RSSPECAN_ATTR_5G_NR_DL_PRS_ENABLED                                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 9561
#define RSSPECAN_ATTR_5G_NR_DL_PRS_SLOT_CONFIG                                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 10518
#define RSSPECAN_ATTR_5G_NR_DL_PRS_NUMBER_OF_RBS                                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 9563
#define RSSPECAN_ATTR_5G_NR_DL_PRS_START_RB                                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 9564
#define RSSPECAN_ATTR_5G_NR_DL_PRS_L_PRS_START                                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 9565
#define RSSPECAN_ATTR_5G_NR_DL_PRS_L_PRS                                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 9566
#define RSSPECAN_ATTR_5G_NR_DL_PRS_N_PRS_ID_SEQ                                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 9567
#define RSSPECAN_ATTR_5G_NR_DL_PRS_K_PRS_COMB                                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 9568
#define RSSPECAN_ATTR_5G_NR_DL_PRS_K_PRS_OFFSET                                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 9569
#define RSSPECAN_ATTR_5G_NR_DL_PRS_RELATIVE_POWER                                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 9570
#define RSSPECAN_ATTR_5G_NR_DL_PDSCH_ALLOCATIONS_IN_SLOT                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 7255
#define RSSPECAN_ATTR_5G_NR_DL_SLOT_PASTE_ALL                                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 7256
#define RSSPECAN_ATTR_5G_NR_DL_SLOT_PASTE                                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 7257
#define RSSPECAN_ATTR_5G_NR_DL_SLOT_PASTE_TO                                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 10519
#define RSSPECAN_ATTR_5G_NR_DL_SLOT_PASTE_MODE                                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 10520
#define RSSPECAN_ATTR_5G_NR_DL_SLOT_PASTE_PERIOD                                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 10521
#define RSSPECAN_ATTR_5G_NR_DL_SLOT_PASTE_DURATION                                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 10522
#define RSSPECAN_ATTR_5G_NR_DL_SLOT_PASTE_SLOT                                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 10523
#define RSSPECAN_ATTR_5G_NR_DL_SLOT_PRESET                                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 7258
#define RSSPECAN_ATTR_5G_NR_DL_CORESET_NUMBERS                                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 7259
#define RSSPECAN_ATTR_5G_NR_DL_SLOT_COMBINE_PDSCH_ALLOCATIONS_SAME_ID                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 10524
#define RSSPECAN_ATTR_5G_NR_DL_SLOT_FORMAT                                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 7260
#define RSSPECAN_ATTR_5G_NR_DL_PDSCH_COPY_SLOT_CONFIGURATION                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 7261
#define RSSPECAN_ATTR_5G_NR_DL_PDSCH_SLOT_ALLOCATION                                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 7262
#define RSSPECAN_ATTR_5G_NR_DL_ENHANCED_PDSCH_MAPPING                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 7263
#define RSSPECAN_ATTR_5G_NR_DL_ENHANCED_PDSCH_ANTENNA_PORT                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 8447
#define RSSPECAN_ATTR_5G_NR_DL_ENHANCED_PDSCH_CDM_GROUPS_WITHOUT_DATA                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 8448
#define RSSPECAN_ATTR_5G_NR_DL_ENHANCED_PDSCH_REFERENCE_POINT                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 8449
#define RSSPECAN_ATTR_5G_NR_DL_ENHANCED_PDSCH_DMRS_DOWNLINK_R16                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 10525
#define RSSPECAN_ATTR_5G_NR_DL_ENHANCED_PDSCH_POWER                                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 7264
#define RSSPECAN_ATTR_5G_NR_DL_ENHANCED_PDSCH_USER_ID                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 7265
#define RSSPECAN_ATTR_5G_NR_DL_ENHANCED_PDSCH_VRB_TO_PRB_INTERLEAVER                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 8677
#define RSSPECAN_ATTR_5G_NR_DL_ENHANCED_PDSCH_DM_RS_TYPE                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 7266
#define RSSPECAN_ATTR_5G_NR_DL_ENHANCED_PDSCH_ADDITIONAL_DM_RS                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 7267
#define RSSPECAN_ATTR_5G_NR_DL_ENHANCED_PDSCH_DM_RS_LENGTH                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 7268
#define RSSPECAN_ATTR_5G_NR_DL_ENHANCED_PDSCH_DM_RS_MAPPING_TYPE                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 7269
#define RSSPECAN_ATTR_5G_NR_DL_ENHANCED_PDSCH_DM_RS_SCRAMBLING_ID                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 7270
#define RSSPECAN_ATTR_5G_NR_DL_ENHANCED_PDSCH_DM_RS_POWER                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 7271
#define RSSPECAN_ATTR_5G_NR_DL_ENHANCED_PDSCH_DM_RS_SEQUENCE_GENERATION_METHOD                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 7272
#define RSSPECAN_ATTR_5G_NR_DL_ENHANCED_PDSCH_DM_RS_SID                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 7273
#define RSSPECAN_ATTR_5G_NR_DL_ENHANCED_PDSCH_DM_RS_FIRST_SYMBOL                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 7274
#define RSSPECAN_ATTR_5G_NR_DL_ENHANCED_PDSCH_PTRS_ENABLED                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 7275
#define RSSPECAN_ATTR_5G_NR_DL_ENHANCED_PDSCH_PTRS_TIME_DOMAIN_OFFSET                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 7276
#define RSSPECAN_ATTR_5G_NR_DL_ENHANCED_PDSCH_PTRS_POWER                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 7277
#define RSSPECAN_ATTR_5G_NR_DL_ENHANCED_PDSCH_PTRS_FREQUENCY_DOMAIN_OFFSET                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 7278
#define RSSPECAN_ATTR_5G_NR_DL_ENHANCED_PDSCH_PTRS_RELATIVE_OFFSET                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 7279
#define RSSPECAN_ATTR_5G_NR_DL_ENHANCED_PDSCH_CHANNEL_CODING_MCS_TABLE                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 8450
#define RSSPECAN_ATTR_5G_NR_DL_ENHANCED_PDSCH_CHANNEL_CODING_MCS_INDEX                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 8451
#define RSSPECAN_ATTR_5G_NR_DL_ENHANCED_PDSCH_CHANNEL_CODING_REDUNDANCY_VERSION_INDEX                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 8452
#define RSSPECAN_ATTR_5G_NR_DL_ENHANCED_PDSCH_CHANNEL_CODING_SCALING_FACTOR                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 10526
#define RSSPECAN_ATTR_5G_NR_DL_ENHANCED_PDSCH_SCRAMBLING                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 8453
#define RSSPECAN_ATTR_5G_NR_DL_ENHANCED_PDSCH_SCRAMBLING_ID                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 8454
#define RSSPECAN_ATTR_5G_NR_DL_PDSCH_MODULATION                                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 7280
#define RSSPECAN_ATTR_5G_NR_DL_PDSCH_RB_OFFSET                                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 7281
#define RSSPECAN_ATTR_5G_NR_DL_PDSCH_RB_COUNT                                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 7282
#define RSSPECAN_ATTR_5G_NR_DL_PDSCH_SYMBOL_COUNT                                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 7283
#define RSSPECAN_ATTR_5G_NR_DL_PDSCH_SYMBOL_OFFSET                                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 7284
#define RSSPECAN_ATTR_5G_NR_DL_PDSCH_COPY                                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 10527
#define RSSPECAN_ATTR_5G_NR_DL_PDSCH_COPY_MODE                                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 10528
#define RSSPECAN_ATTR_5G_NR_DL_PDSCH_COPY_PERIOD                                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 10529
#define RSSPECAN_ATTR_5G_NR_DL_PDSCH_COPY_DURATION                                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 10530
#define RSSPECAN_ATTR_5G_NR_DL_PDSCH_COPY_SLOT                                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 10531
#define RSSPECAN_ATTR_5G_NR_DL_CORESET_PRECODER_GRANULARITY                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 8456
#define RSSPECAN_ATTR_5G_NR_DL_CORESET_DM_RS_POWER                                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 7285
#define RSSPECAN_ATTR_5G_NR_DL_CORESET_DM_RS_REFERENCE_POINT                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 8457
#define RSSPECAN_ATTR_5G_NR_DL_CORESET_DM_RS_SEQUENCE_GENERATION                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 7286
#define RSSPECAN_ATTR_5D_NR_DL_CORESET_DM_RS_USE_SCRAMBLING_ID                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 8678
#define RSSPECAN_ATTR_5G_NR_DL_CORESET_DM_RS_SID                                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 7287
#define RSSPECAN_ATTR_5G_NR_DL_CORESET_INTERLEAVING_ENABLED                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 8458
#define RSSPECAN_ATTR_5G_NR_DL_CORESET_INTERLEAVING_BUNDLE_SIZE                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 8459
#define RSSPECAN_ATTR_5G_NR_DL_CORESET_INTERLEAVING_SHIFT_INDEX                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 8460
#define RSSPECAN_ATTR_5G_NR_DL_CORESET_INTERLEAVING_SHIFT_INDEX_VALUE                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 8461
#define RSSPECAN_ATTR_5G_NR_DL_CORESET_INTERLEAVING_SIZE                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 8462
#define RSSPECAN_ATTR_5G_NR_DL_CORESET_POWER                                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 7288
#define RSSPECAN_ATTR_5G_NR_DL_CORESET_RESOURCE_BLOCKS                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 7289
#define RSSPECAN_ATTR_5G_NR_DL_CORESET_RESOURCE_BLOCKS_OFFSET                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 7290
#define RSSPECAN_ATTR_5G_NR_DL_CORESET_SYMBOL_COUNT                                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 7291
#define RSSPECAN_ATTR_5G_NR_DL_CORESET_SYMBOL_OFFSET                                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 7292
#define RSSPECAN_ATTR_5G_NR_DL_CORESET_COPY                                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 10532
#define RSSPECAN_ATTR_5G_NR_DL_CORESET_COPY_MODE                                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 10533
#define RSSPECAN_ATTR_5G_NR_DL_CORESET_COPY_PERIOD                                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 10534
#define RSSPECAN_ATTR_5G_NR_DL_CORESET_COPY_DURATION                                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 10535
#define RSSPECAN_ATTR_5G_NR_DL_CORESET_COPY_SLOT                                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 10536
#define RSSPECAN_ATTR_5G_NR_DL_CORESET_PDCCH_CONFIG_RNTI                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 8679
#define RSSPECAN_ATTR_5G_NR_DL_CORESET_PDCCH_CONFIG_USAGE                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 10537
#define RSSPECAN_ATTR_5G_NR_DL_CORESET_PDCCH_CONFIG_DCI_FORMAT                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 10538
#define RSSPECAN_ATTR_5G_NR_DL_CORESET_PDCCH_CONFIG_AGGREGATION_LEVEL                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 8463
#define RSSPECAN_ATTR_5G_NR_DL_CORESET_PDCCH_CONFIG_CCE_INDEX                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 8464
#define RSSPECAN_ATTR_5G_NR_DL_CORESET_PDCCH_CONFIG_PATTERN_LENGTH                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 8680
#define RSSPECAN_ATTR_5G_NR_DL_CORESET_PDCCH_CONFIG_FREQUENCY_RESOURCE_ASSIGNMENT                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 10539
#define RSSPECAN_ATTR_5G_NR_DL_CORESET_PDCCH_CONFIG_DCI_AVAILABLE_FIELDS                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 10541
#define RSSPECAN_ATTR_5G_NR_DL_CORESET_PDCCH_CONFIG_DCI_NUMBER_OF_BLOCKS                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 10542
#define RSSPECAN_ATTR_5G_NR_DL_CORESET_PDCCH_CONFIG_DCI_SCOPE                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 10543
#define RSSPECAN_ATTR_5G_NR_DL_CORESET_PDCCH_CONFIG_DCI_TCP_COMMAND                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 10544
#define RSSPECAN_ATTR_5G_NR_DL_CSI_RS_ENABLED                                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 8465
#define RSSPECAN_ATTR_5G_NR_DL_CSI_RS_RESOURCES                                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 10545
#define RSSPECAN_ATTR_5G_NR_DL_CSI_RS_SLOT_MODE                                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 8466
#define RSSPECAN_ATTR_5G_NR_DL_CSI_RS_SLOT_PERIODICITY                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 8467
#define RSSPECAN_ATTR_5G_NR_DL_CSI_RS_SLOT_OFFSET                                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 8468
#define RSSPECAN_ATTR_5G_NR_DL_CSI_RS_APERIODIC_SLOTS                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 8469
#define RSSPECAN_ATTR_5G_NR_DL_CSI_RS_ZERO_POWER_ENABLED                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 8470
#define RSSPECAN_ATTR_5G_NR_DL_CSI_RS_NUMBER_OF_RBS                                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 8681
#define RSSPECAN_ATTR_5G_NR_DL_CSI_RS_START_RB                                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 8682
#define RSSPECAN_ATTR_5G_NR_DL_CSI_RS_ROW                                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 8471
#define RSSPECAN_ATTR_5G_NR_DL_CSI_RS_PORTS                                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 8472
#define RSSPECAN_ATTR_5G_NR_DL_CSI_RS_CDM_TYPE                                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 8473
#define RSSPECAN_ATTR_5G_NR_DL_CSI_RS_DENSITY                                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 8474
#define RSSPECAN_ATTR_5G_NR_DL_CSI_RS_BITMAP                                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 8475
#define RSSPECAN_ATTR_5G_NR_DL_CSI_RS_L0                                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 8476
#define RSSPECAN_ATTR_5G_NR_DL_CSI_RS_L1                                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 8477
#define RSSPECAN_ATTR_5G_NR_DL_CSI_RS_SCRAMBLING_ID                                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 8478
#define RSSPECAN_ATTR_5G_NR_DL_CSI_RS_RELATIVE_POWER                                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 8479
#define RSSPECAN_ATTR_5G_NR_DL_SYNCHRONIZATION_SIGNAL_OFFSET                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 7293
#define RSSPECAN_ATTR_5G_NR_DL_SYNCHRONIZATION_SIGNAL_BURST_SET_PERIODICITY                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 7294
#define RSSPECAN_ATTR_5G_NR_DL_SYNCHRONIZATION_SIGNAL_CONFIGURATION_MODE                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 7295
#define RSSPECAN_ATTR_5G_NR_DL_SYNCHRONIZATION_SIGNAL_FREQUENCY_OFFSET                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 7296
#define RSSPECAN_ATTR_5G_NR_DL_SYNCHRONIZATION_SIGNAL_PATTERN                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 7297
#define RSSPECAN_ATTR_5G_NR_DL_SYNCHRONIZATION_SIGNAL_PBCH_POWER                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 7298
#define RSSPECAN_ATTR_5G_NR_DL_SYNCHRONIZATION_SIGNAL_PBCH_DMRS_POWER                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 7299
#define RSSPECAN_ATTR_5G_NR_DL_SYNCHRONIZATION_SIGNAL_PSS_POWER                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 7300
#define RSSPECAN_ATTR_5G_NR_DL_SYNCHRONIZATION_SIGNAL_SUBCARRIER_SPACING                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 7301
#define RSSPECAN_ATTR_5G_NR_DL_SYNCHRONIZATION_SIGNAL_SSS_POWER                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 7302
#define RSSPECAN_ATTR_5G_NR_DL_SYNCHRONIZATION_SIGNAL_OFFSET_RELATIVE_TO                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 8480
#define RSSPECAN_ATTR_5G_NR_DL_SYNCHRONIZATION_SIGNAL_L_SELECTION                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 8481
#define RSSPECAN_ATTR_5G_NR_DL_SYNCHRONIZATION_SIGNAL_HALF_FRAME_OFFSET                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 10546
#define RSSPECAN_ATTR_5G_NR_DL_SYNCHRONIZATION_SIGNAL_ENABLED                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 7303
#define RSSPECAN_ATTR_5G_NR_DL_SYNCHRONIZATION_SIGNAL_ALL_ENABLED                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 8482
#define RSSPECAN_ATTR_5G_NR_DL_SYNCHRONIZATION_SIGNAL_ALL_DISABLED                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 8483
#define RSSPECAN_ATTR_5G_NR_DL_ADVANCED_SETTINGS_DC_CARRIER_ENABLED                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 7304
#define RSSPECAN_ATTR_5G_NR_DL_ADVANCED_SETTINGS_HANDLING_OF_CARRIER_LEAKAGE                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 8683
#define RSSPECAN_ATTR_5G_NR_DL_ADVANCED_SETTINGS_FRAME_NUMBER                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 10547
#define RSSPECAN_ATTR_5G_NR_DL_ADVANCED_SETTINGS_RF_UPCONVERSION_ENABLED                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 7305
#define RSSPECAN_ATTR_5G_NR_DL_ADVANCED_SETTINGS_RF_UPCONVERSION_MODE                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 7306
#define RSSPECAN_ATTR_5G_NR_DL_ADVANCED_SETTINGS_RF_UPCONVERSION_FREQUENCY                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 7307
#define RSSPECAN_ATTR_5G_NR_DL_ADVANCED_SETTINGS_EXCLUDE_USER_IDS                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 8684
#define RSSPECAN_ATTR_5G_NR_DL_ADVANCED_SETTINGS_FREQUENCY_ERROR_LIMIT_CHECK                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 10548
#define RSSPECAN_ATTR_5G_NR_DL_ADVANCED_SETTINGS_SHARED_SPECTRUM_CHANNEL_ACCESS                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 10549
#define RSSPECAN_ATTR_5G_NR_DL_ADVANCED_SETTINGS_REF_POINT_A_FREQUENCY                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 7308
#define RSSPECAN_ATTR_5G_NR_DL_ADVANCED_SETTINGS_REF_POINT_A_15_KHZ_OFFSET                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 7309
#define RSSPECAN_ATTR_5G_NR_DL_ADVANCED_SETTINGS_REF_POINT_A_30_KHZ_OFFSET                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 7310
#define RSSPECAN_ATTR_5G_NR_DL_ADVANCED_SETTINGS_REF_POINT_A_60_KHZ_OFFSET                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 7311
#define RSSPECAN_ATTR_5G_NR_DL_ADVANCED_SETTINGS_REF_POINT_A_120_KHZ_OFFSET                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 7312
#define RSSPECAN_ATTR_5G_NR_DL_ADVANCED_SETTINGS_REF_POINT_A_CENTER_FREQUENCY_OFFSET                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 7313
#define RSSPECAN_ATTR_5G_NR_DL_ADVANCED_SETTINGS_LTE_CRS_COEXISTENCE_STATE                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 8685
#define RSSPECAN_ATTR_5G_NR_DL_ADVANCED_SETTINGS_LTE_CRS_COEXISTENCE_OFFSET_TO_POINT_A                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 8686
#define RSSPECAN_ATTR_5G_NR_DL_ADVANCED_SETTINGS_LTE_CRS_COEXISTENCE_LTE_BANDWIDTH                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 8687
#define RSSPECAN_ATTR_5G_NR_DL_ADVANCED_SETTINGS_LTE_CRS_COEXISTENCE_VSHIFT                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 8688
#define RSSPECAN_ATTR_5G_NR_DL_ADVANCED_SETTINGS_LTE_CRS_COEXISTENCE_LTE_ANTENNA_PORTS                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 8689
#define RSSPECAN_ATTR_5G_NR_DL_ADVANCED_SETTINGS_LTE_CRS_COEXISTENCE_MBSFN_SUBFRAMES                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 10550
#define RSSPECAN_ATTR_5G_NR_DL_ANTENNA_PORT_MAPPING_ENABLED                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 8484
#define RSSPECAN_ATTR_5G_NR_DL_TRACE_TYPE                                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 7314
#define RSSPECAN_ATTR_5G_NR_MARKER_POSITION_Z                                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 7316
#define RSSPECAN_ATTR_5G_NR_DL_Y_SCALING_AUTO                                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 7317
#define RSSPECAN_ATTR_5G_NR_DL_Y_SCALING_MAXIMUM                                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 7318
#define RSSPECAN_ATTR_5G_NR_DL_Y_SCALING_MINIMUM                                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 7319
#define RSSPECAN_ATTR_5G_NR_DL_TAB_SELECT                                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 7320
#define RSSPECAN_ATTR_5G_NR_DL_SEGMENTED_CAPTURE_ENABLED                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 8487
#define RSSPECAN_ATTR_5G_NR_DL_SEGMENTED_CAPTURE_EVENTS                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 8488
#define RSSPECAN_ATTR_5G_NR_DL_SEGMENTED_CAPTURE_OFFSET                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 8489
#define RSSPECAN_ATTR_5G_NR_DL_SEGMENTED_CAPTURE_LENGTH                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 8490
#define RSSPECAN_ATTR_5G_NR_DL_TRACKING_CHANNEL_ESTIMATION                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 8491
#define RSSPECAN_ATTR_5G_NR_DL_TRACKING_PHASE                                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 8492
#define RSSPECAN_ATTR_5G_NR_DL_TRACKING_TIME                                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 8690
#define RSSPECAN_ATTR_5G_NR_DL_TRACKING_LEVEL                                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 8691
#define RSSPECAN_ATTR_5G_NR_DL_TRACKING_GAIN_IMBALANCE_QUADRATURE_ERROR_ENABLED                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 8493
#define RSSPECAN_ATTR_5G_NR_DL_TRACKING_CHANNEL_ESTIMATION_TIME_AVERAGING                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 8494
#define RSSPECAN_ATTR_5G_NR_DL_DEMODULATION_DATA                                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 8495
#define RSSPECAN_ATTR_5G_NR_DL_CORESET_ANALYSIS_MODE                                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 8496
#define RSSPECAN_ATTR_5G_NR_DL_EVM_CALCULATION_METHOD                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 8692
#define RSSPECAN_ATTR_5G_NR_DL_SYMBOL_TIME_ADJUSTMENT                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 10551
#define RSSPECAN_ATTR_5G_NR_DL_PDSCH_REFERENCE_DATA                                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 10552
#define RSSPECAN_ATTR_5G_NR_DL_CORESET_REFERENCE_DATA                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 10553
#define RSSPECAN_ATTR_5G_NR_DL_EXTENDED_FREQUENCY_LOCK_RANGE                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 10554
#define RSSPECAN_ATTR_5G_NR_DL_ON_OFF_ADJUST_TIMING                                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 8497
#define RSSPECAN_ATTR_5G_NR_DL_ON_OFF_NOISE_CORRECTION_ENABLED                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 8498
#define RSSPECAN_ATTR_5G_NR_DL_ON_OFF_UNIT                                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 8499
#define RSSPECAN_ATTR_5G_NR_DL_ON_OFF_NUMBER_OF_FRAMES_TO_ANALYZE                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 10555
#define RSSPECAN_ATTR_5G_NR_DL_ON_OFF_TRANSITION_PERIOD                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 8500
#define RSSPECAN_ATTR_5G_NR_DL_ON_OFF_BASE_STATION_OUTPUT_POWER_EIRP                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 8693
#define RSSPECAN_ATTR_5G_NR_DL_ON_OFF_BASE_STATION_OUTPUT_POWER_TRP                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 8694
#define RSSPECAN_ATTR_5G_NR_DL_COMBINED_MEASUREMENT_ACLR                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 9607
#define RSSPECAN_ATTR_5G_NR_DL_COMBINED_MEASUREMENT_EVM                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 9608
#define RSSPECAN_ATTR_5G_NR_DL_COMBINED_MEASUREMENT_SEM                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 9609
#define RSSPECAN_ATTR_5G_NR_DL_COMBINED_MEASUREMENT_LOAD_ACLR_SETTINGS                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 9610
#define RSSPECAN_ATTR_5G_NR_DL_COMBINED_MEASUREMENT_CLEAR_ACLR_SETTINGS                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 10556
#define RSSPECAN_ATTR_5G_NR_DL_COMBINED_MEASUREMENT_LOAD_SEM_SETTINGS                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 10557
#define RSSPECAN_ATTR_5G_NR_DL_COMBINED_MEASUREMENT_CLEAR_SEM_SETTINGS                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 10558
#define RSSPECAN_ATTR_5G_NR_DL_COMBINED_MEASUREMENT_NR_OF_EVENTS                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 10559
#define RSSPECAN_ATTR_5G_NR_DL_COMBINED_MEASUREMENT_OPTIMIZATION                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 9613
#define RSSPECAN_ATTR_5G_NR_DL_COMBINED_MEASUREMENT_SEQUENCE_AUTO_LEVEL                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 9614
#define RSSPECAN_ATTR_5G_NR_DL_COMBINED_MEASUREMENT_FRAME_TRIGGER                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 9615
#define RSSPECAN_ATTR_5G_NR_DL_COMBINED_MEASUREMENT_TRIGGER_SEQUENCE                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 9616
#define RSSPECAN_ATTR_5G_NR_DL_COMBINED_MEASUREMENT_TRIGGER_EVENT_DELAY                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 9617
#define RSSPECAN_ATTR_5G_NR_DL_COMBINED_MEASUREMENT_TRIGGER_RECALIBRATION_INTERVAL                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 9618
#define RSSPECAN_ATTR_5G_NR_DL_COMBINED_MEASUREMENT_SEND_TRIGGER_3                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 9619
#define RSSPECAN_ATTR_5G_NR_DL_COMBINED_MEASUREMENT_TRIGGER_AUTO_GATING                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 9620
#define RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_SHOW_RESULT_SUMMARY                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 7321
#define RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_CC_RESULT                                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 8504
#define RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_X_SCALE                                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 7322
#define RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_CARRIER_AXES_REFERENCE                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 8505
#define RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_CARRIER_AXES                                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 8506
#define RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_EVM_MAX_HOLD                                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 8695
#define RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_3D_VIEW                                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 10560
#define RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_ACLR_LIMIT_PASS_MODE                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 10561
#define RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_SUMMARY_ALL_CARRIERS_EVM                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 9623
#define RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_SUMMARY_ALL_CARRIERS_EVM_PDSCH_WITH_QPSK                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 10562
#define RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_SUMMARY_ALL_CARRIERS_EVM_PDSCH_WITH_64QAM                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 10563
#define RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_SUMMARY_ALL_CARRIERS_EVM_PDSCH_WITH_16QAM                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 10564
#define RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_SUMMARY_ALL_CARRIERS_EVM_PDSCH_WITH_256QAM                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 10565
#define RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_SUMMARY_ALL_CARRIERS_EVM_PDSCH_WITH_1024QAM                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 11150
#define RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_SUMMARY_ALL_CARRIERS_EVM_PEAK                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 10566
#define RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_SUMMARY_ALL_CARRIERS_EVM_PHYSICAL_CHANNEL                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 10567
#define RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_SUMMARY_ALL_CARRIERS_EVM_PHYSICAL_SIGNAL                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 10568
#define RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_SUMMARY_ALL_CARRIERS_SIGNAL_POWER                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 10569
#define RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_EVALUATION_RANGE_ALLOCATION_ALL                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 7323
#define RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_EVALUATION_RANGE_ALLOCATION                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 7324
#define RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_EVALUATION_RANGE_BANDWIDTH_PART_ALL                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 7325
#define RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_EVALUATION_RANGE_BANDWIDTH_PART_SS_PBCH_BLOCK                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 8508
#define RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_EVALUATION_RANGE_BANDWIDTH_PART                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 7326
#define RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_EVALUATION_RANGE_CARRIER_ALL                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 7327
#define RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_EVALUATION_RANGE_CARRIER                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 7328
#define RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_EVALUATION_RANGE_FRAME                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 7330
#define RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_EVALUATION_RANGE_SELECTED_MEAS_ID                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 10570
#define RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_EVALUATION_RANGE_MODULATION_ALL                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 7331
#define RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_EVALUATION_RANGE_MODULATION                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 7332
#define RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_EVALUATION_RANGE_SLOT_ALL                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 7333
#define RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_EVALUATION_RANGE_SLOT                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 7334
#define RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_EVALUATION_RANGE_SUBFRAME_ALL                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 7335
#define RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_EVALUATION_RANGE_SUBFRAME                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 7336
#define RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_EVALUATION_RANGE_SYMBOL_ALL                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 7337
#define RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_EVALUATION_RANGE_SYMBOL                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 7338
#define RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_EVALUATION_RANGE_SEGMENT                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 8509
#define RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_EVALUATION_RANGE_BEAMFORMING_REFERENCE_AP                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 8510
#define RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_EVALUATION_RANGE_BEAMFORMING_ANTENNA_PORT                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 8511
#define RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_EVALUATION_RANGE_BEAMFORMING_ANTENNA_PORT_ALL                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 8512
#define RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_SYNCHRONIZATION_STATE                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 7339
#define RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_CELL_ID                                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 7340
#define RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_SUMMARY_EVM_ALL                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 7341
#define RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_SUMMARY_EVM_ALL_MAXIMUM                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 8514
#define RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_SUMMARY_EVM_ALL_MINIMUM                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 8515
#define RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_SUMMARY_EVM_ALL_PDSCH_WITH_QPSK                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 7342
#define RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_SUMMARY_EVM_ALL_PDSCH_WITH_QPSK_MAXIMUM                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 8516
#define RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_SUMMARY_EVM_ALL_PDSCH_WITH_QPSK_MINIMUM                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 8517
#define RSSPECAN_ATTR_5G_NR_DL_EVM_PDSCH_WITH_QPSK_LIMIT_CHECK_RESULT_AVERAGE                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 8518
#define RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_SUMMARY_EVM_ALL_PDSCH_WITH_64QAM                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 7343
#define RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_SUMMARY_EVM_ALL_PDSCH_WITH_64QAM_MAXIMUM                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 8519
#define RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_SUMMARY_EVM_ALL_PDSCH_WITH_64QAM_MINIMUM                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 8520
#define RSSPECAN_ATTR_5G_NR_DL_EVM_PDSCH_WITH_64QAM_LIMIT_CHECK_RESULT_AVERAGE                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 8521
#define RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_SUMMARY_EVM_ALL_PDSCH_WITH_256QAM                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 7344
#define RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_SUMMARY_EVM_ALL_PDSCH_WITH_256QAM_MAXIMUM                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 8522
#define RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_SUMMARY_EVM_ALL_PDSCH_WITH_256QAM_MINIMUM                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 8523
#define RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_SUMMARY_EVM_ALL_PDSCH_WITH_1024QAM                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 11151
#define RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_SUMMARY_EVM_ALL_PDSCH_WITH_1024QAM_MAXIMUM                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 11152
#define RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_SUMMARY_EVM_ALL_PDSCH_WITH_1024QAM_MINIMUM                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 11153
#define RSSPECAN_ATTR_5G_NR_DL_EVM_PDSCH_WITH_256QAM_LIMIT_CHECK_RESULT_AVERAGE                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 8524
#define RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_SUMMARY_EVM_ALL_PDSCH_WITH_16QAM                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 7345
#define RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_SUMMARY_EVM_ALL_PDSCH_WITH_16QAM_MAXIMUM                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 8525
#define RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_SUMMARY_EVM_ALL_PDSCH_WITH_16QAM_MINIMUM                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 8526
#define RSSPECAN_ATTR_5G_NR_DL_EVM_PDSCH_WITH_16QAM_LIMIT_CHECK_RESULT_AVERAGE                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 8527
#define RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_SUMMARY_EVM_PEAK                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 8696
#define RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_SUMMARY_EVM_PEAK_MAXIMUM                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 8697
#define RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_SUMMARY_EVM_PEAK_MINIMUM                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 8698
#define RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_SUMMARY_EVM_PHYSICAL_CHANNEL                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 7346
#define RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_SUMMARY_EVM_PHYSICAL_CHANNEL_MAXIMUM                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 8528
#define RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_SUMMARY_EVM_PHYSICAL_CHANNEL_MINIMUM                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 8529
#define RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_SUMMARY_EVM_PHYSICAL_SIGNAL                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 7347
#define RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_SUMMARY_EVM_PHYSICAL_SIGNAL_MAXIMUM                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 8530
#define RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_SUMMARY_EVM_PHYSICAL_SIGNAL_MINIMUM                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 8531
#define RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_SUMMARY_SIGNAL_POWER                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 7348
#define RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_SUMMARY_SIGNAL_POWER_MAXIMUM                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 8532
#define RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_SUMMARY_SIGNAL_POWER_MINIMUM                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 8533
#define RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_SUMMARY_FREQUENCY_ERROR                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 7349
#define RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_SUMMARY_FREQUENCY_ERROR_MAXIMUM                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 8534
#define RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_SUMMARY_FREQUENCY_ERROR_MINIMUM                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 8535
#define RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_SUMMARY_FREQUENCY_ERROR_LIMIT_CHECK_RESULT                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 10571
#define RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_SUMMARY_SAMPLING_ERROR                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 7350
#define RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_SUMMARY_SAMPLING_ERROR_MAXIMUM                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 8536
#define RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_SUMMARY_SAMPLING_ERROR_MINIMUM                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 8537
#define RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_SUMMARY_IQ_GAIN_IMBALANCE_AVERAGE                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 8538
#define RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_SUMMARY_IQ_GAIN_IMBALANCE_MAXIMUM                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 8539
#define RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_SUMMARY_IQ_GAIN_IMBALANCE_MINIMUM                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 8540
#define RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_SUMMARY_IQ_OFFSET_AVERAGE                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 8541
#define RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_SUMMARY_IQ_OFFSET_MAXIMUM                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 8542
#define RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_SUMMARY_IQ_OFFSET_MINIMUM                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 8543
#define RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_SUMMARY_IQ_QUADRATURE_ERROR_AVERAGE                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 8544
#define RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_SUMMARY_IQ_QUADRATURE_ERROR_MAXIMUM                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 8545
#define RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_SUMMARY_IQ_QUADRATURE_ERROR_MINIMUM                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 8546
#define RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_SUMMARY_OSTP_AVERAGE                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 8547
#define RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_SUMMARY_OSTP_MAXIMUM                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 8548
#define RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_SUMMARY_OSTP_MINIMUM                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 8549
#define RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_SUMMARY_CSI_RSRP_AVERAGE                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 8550
#define RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_SUMMARY_CSI_RSRP_MAXIMUM                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 8551
#define RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_SUMMARY_CSI_RSRP_MINIMUM                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 8552
#define RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_SUMMARY_SS_RSRP_AVERAGE                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 8553
#define RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_SUMMARY_SS_RSRP_MAXIMUM                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 8554
#define RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_SUMMARY_SS_RSRP_MINIMUM                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 8555
#define RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_SUMMARY_RSTP_AVERAGE                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 8556
#define RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_SUMMARY_RSTP_MAXIMUM                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 8557
#define RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_SUMMARY_RSTP_MINIMUM                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 8558
#define RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_SUMMARY_SUBFRAME_START_OFFSET                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 7351
#define RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_CREST_FACTOR_RESULT_AVERAGE                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 8513
#define RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_SUMMARY_ACLR_LIMIT_CHECK                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 10572
#define RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_SUMMARY_ACLR_LIMIT_CHECK_ABSOLUTE                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 10573
#define RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_SUMMARY_ACLR_LIMIT_CHECK_RELATIVE                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 10574
#define RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_SUMMARY_OVERLOAD                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 10575
#define RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_SUMMARY_SEM_PASS_FAIL                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 10576
#define RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_SUMMARY_SYNC_STATE                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 10577
#define RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_SUMMARY_TIME_STAMP_DELTA                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 10578
#define RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_SUMMARY_TIME_STAMP                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 10579
#define RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_SUMMARY_BLER_AVERAGE                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 11154
#define RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_SUMMARY_BLER_MAXIMUM                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 11155
#define RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_SUMMARY_BLER_MINIMUM                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 11156
#define RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_SUMMARY_TPUT_AVERAGE                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 11157
#define RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_SUMMARY_TPUT_MAXIMUM                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 11158
#define RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_SUMMARY_TPUT_MINIMUM                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 11159
#define RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_VIEW_CARRIER                                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 7352
#define RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_VIEW_FRAME                                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 7353
#define RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_VIEW_WINDOW_COUPLING                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 7354
#define RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_VIEW_SELECT_RESULT_WINDOW                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 7355
#define RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_FREQUENCY_SWEEP_BASE_STATION_TYPE                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 8559
#define RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_FREQUENCY_SWEEP_BW_FOR_ACLR                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 7356
#define RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_FREQUENCY_SWEEP_BS_CATEGORY                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 7357
#define RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_FREQUENCY_SWEEP_BS_CATEGORY_B_OPTION                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 8699
#define RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_FREQUENCY_SWEEP_BS_MEDIUM_RANGE_POWER                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 7358
#define RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_FREQUENCY_SWEEP_BS_MEDIUM_RANGE_AUTO                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 7359
#define RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_FREQUENCY_SWEEP_SEM_N_TABCONNECTORS                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 8560
#define RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_FREQUENCY_SWEEP_SEM_N_TXU                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 8561
#define RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_FREQUENCY_SWEEP_TOTAL_LIMIT_PASS_MODE                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 10580
#define RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_FREQUENCY_SWEEP_GET_ADJACENT_ACLR_LIMIT_CHECK_RESULT                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 8700
#define RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_FREQUENCY_SWEEP_GET_ALTERNATE_ACLR_LIMIT_CHECK_RESULT                RS_SPECIFIC_PUBLIC_ATTR_BASE + 8701
#define RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_COMBINED_MEASUREMENT_EVENT_FILTER                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 9653
#define RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_COMBINED_MEASUREMENT_EVENT_FILTER_RESET                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 9654
#define RSSPECAN_ATTR_5G_NR_UL_CONFIGURATION_FILTER_TYPE                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 11160
#define RSSPECAN_ATTR_5G_NR_UL_CONFIGURATION_MULTICARRIER_FILTER                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 7811
#define RSSPECAN_ATTR_5G_NR_UL_DEMODULATION_DECODED_DATA                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 7812
#define RSSPECAN_ATTR_5G_NR_UL_DEMODULATION_EVM_CALCULATION_METHOD                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 8702
#define RSSPECAN_ATTR_5G_NR_UL_EXTENDED_FREQUENCY_LOCK_RANGE                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 10591
#define RSSPECAN_ATTR_5G_NR_UL_COMPONENT_CARRIER_NUMBER                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 7813
#define RSSPECAN_ATTR_5G_NR_UL_COMPONENT_CARRIER_DATA_CAPTURE                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 7814
#define RSSPECAN_ATTR_5G_NR_UL_FRAME_COUNT_AUTO                                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 7815
#define RSSPECAN_ATTR_5G_NR_UL_FRAME_SLOT_ALL                                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 7816
#define RSSPECAN_ATTR_5G_NR_UL_FRAME_SLOT                                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 7817
#define RSSPECAN_ATTR_5G_NR_UL_REPEATED_SLOT_ANALYSIS                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 8703
#define RSSPECAN_ATTR_5G_NR_UL_SEGMENT_CAPTURE                                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 7818
#define RSSPECAN_ATTR_5G_NR_UL_SEGMENT_CAPTURE_EVENTS                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 8571
#define RSSPECAN_ATTR_5G_NR_UL_SEGMENT_CAPTURE_OFFSET                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 8572
#define RSSPECAN_ATTR_5G_NR_UL_SEGMENT_CAPTURE_LENGTH                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 8573
#define RSSPECAN_ATTR_5G_NR_UL_LONG_CAPTURE                                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 7819
#define RSSPECAN_ATTR_5G_NR_UL_AUTOMATICALLY_DEMODULATES_ONCE                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 11161
#define RSSPECAN_ATTR_5G_NR_UL_CONSTELLATION_COLORS                                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 11162
#define RSSPECAN_ATTR_5G_NR_UL_BOOSTING_FACTOR_ENABLED                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 11163
#define RSSPECAN_ATTR_5G_NR_UL_PUSCH_REFERENCE_DATA                                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 11164
#define RSSPECAN_ATTR_5G_NR_UL_DEMODULATION_FILTER                                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 11165
#define RSSPECAN_ATTR_5G_NR_UL_BIT_STREAM_FORMAT                                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 11166
#define RSSPECAN_ATTR_5G_NR_UL_PHYSICAL_SETTINGS_OPERATING_BAND                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 7821
#define RSSPECAN_ATTR_5G_NR_UL_PHYSICAL_SETTINGS_LOAD_SIGNAL_DESCRIPTION                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 7822
#define RSSPECAN_ATTR_5G_NR_UL_PHYSICAL_SETTINGS_SAVE_SIGNAL_DESCRIPTION                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 7824
#define RSSPECAN_ATTR_5G_NR_UL_GENERAL_CHANNEL_RASTER                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 10592
#define RSSPECAN_ATTR_5G_NR_UL_GENERAL_GLOBAL_MC_FREQUENCY                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 10593
#define RSSPECAN_ATTR_5G_NR_UL_GENERAL_SYNC_GLOBAL_MC_FREQUENCY_TO_CENTER                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 10594
#define RSSPECAN_ATTR_5G_NR_UL_GENERAL_FREQUENCY_OFFSET_REFERENCE_POINT                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 10595
#define RSSPECAN_ATTR_5G_NR_UL_GENERAL_FREQUENCY_OFFSET_MODE                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 10596
#define RSSPECAN_ATTR_5G_NR_UL_GENERAL_FREQUENCY_OFFSET_CARRIER_SPACING                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 10597
#define RSSPECAN_ATTR_5G_NR_UL_GENERAL_FREQUENCY_OFFSET_FIXED                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 10598
#define RSSPECAN_ATTR_5G_NR_UL_PHYSICAL_SETTINGS_BANDWIDTH                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 7826
#define RSSPECAN_ATTR_5G_NR_UL_PHYSICAL_SETTINGS_DETECTION                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 10599
#define RSSPECAN_ATTR_5G_NR_UL_DEPLOYMENT_FREQUENCY_RANGE                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 7827
#define RSSPECAN_ATTR_5G_NR_UL_CELL_ID                                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 7828
#define RSSPECAN_ATTR_5G_NR_UL_POWER_CLASS                                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 8704
#define RSSPECAN_ATTR_5G_NR_UL_CENTER_FREQUENCY                                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 7829
#define RSSPECAN_ATTR_5G_NR_UL_COMPONENT_CARRIER_GENERAL_FREQUENCY_OFFSET                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 7830
#define RSSPECAN_ATTR_5G_NR_UL_COMPONENT_CARRIER_GENERAL_MC_FREQUENCY_OFFSET                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 10600
#define RSSPECAN_ATTR_5G_NR_UL_LOAD_SIGNAL_DESCRIPTION                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 7831
#define RSSPECAN_ATTR_5G_NR_UL_SAVE_SIGNAL_DESCRIPTION                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 7833
#define RSSPECAN_ATTR_5G_NR_UL_PRACH_PREAMBLE_FORMAT                                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 8705
#define RSSPECAN_ATTR_5G_NR_UL_PRACH_RELATIVE_POWER                                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 8706
#define RSSPECAN_ATTR_5G_NR_UL_PRACH_PREAMBLE_INDEX                                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 8707
#define RSSPECAN_ATTR_5G_NR_UL_PRACH_RB_OFFSET                                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 8708
#define RSSPECAN_ATTR_5G_NR_UL_PRACH_LOGICAL_ROOT_SEQUENCE_INDEX                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 8709
#define RSSPECAN_ATTR_5G_NR_UL_PRACH_RESTRICTED_SET                                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 8710
#define RSSPECAN_ATTR_5G_NR_UL_PRACH_SUBCARRIER_SPACING                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 8711
#define RSSPECAN_ATTR_5G_NR_UL_PRACH_ZERO_CORRELATION_ZONE                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 8712
#define RSSPECAN_ATTR_5G_NR_UL_PRACH_LRA                                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 11167
#define RSSPECAN_ATTR_5G_NR_UL_DATA_CAPTURE_FRAME_COUNT_ENABLED                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 7835
#define RSSPECAN_ATTR_5G_NR_UL_DATA_CAPTURE_FRAME_COUNT                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 7836
#define RSSPECAN_ATTR_5G_NR_UL_BANDWIDTH_PART_ADD                                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 7837
#define RSSPECAN_ATTR_5G_NR_UL_BANDWIDTH_PART_CLEAR                                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 7838
#define RSSPECAN_ATTR_5G_NR_UL_BANDWIDTH_PART_ANALYZED                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 7839
#define RSSPECAN_ATTR_5G_NR_UL_BANDWIDTH_PART_AUTOMATIC_SIGNAL_DEMODULATION                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 10601
#define RSSPECAN_ATTR_5G_NR_UL_BANDWIDTH_PART_DUPLICATE                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 7840
#define RSSPECAN_ATTR_5G_NR_UL_BANDWIDTH_PART_RB_COUNT                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 7841
#define RSSPECAN_ATTR_5G_NR_UL_BANDWIDTH_PART_RB_OFFSET                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 7842
#define RSSPECAN_ATTR_5G_NR_UL_BANDWIDTH_PART_DELETE                                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 7843
#define RSSPECAN_ATTR_5G_NR_UL_BANDWIDTH_PART_SUBCARRIER_SPACING                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 7844
#define RSSPECAN_ATTR_5G_NR_UL_GENERATOR_IP_ADDRESS                                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 10602
#define RSSPECAN_ATTR_5G_NR_UL_GENERATOR_LED_STATE                                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 10603
#define RSSPECAN_ATTR_5G_NR_UL_GENERATOR_CONTROL_ENABLED                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 10604
#define RSSPECAN_ATTR_5G_NR_UL_GENERATOR_RF_OUTPUT_ENABLED                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 10605
#define RSSPECAN_ATTR_5G_NR_UL_GENERATOR_FREQUENCY_SYNCHRONIZATION                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 10606
#define RSSPECAN_ATTR_5G_NR_UL_GENERATOR_LEVEL_CONTROL_ENABLED                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 10607
#define RSSPECAN_ATTR_5G_NR_UL_GENERATOR_POWER_LEVEL                                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 10608
#define RSSPECAN_ATTR_5G_NR_UL_GENERATOR_LEVEL_DUT_LIMIT_ENABLED                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 10609
#define RSSPECAN_ATTR_5G_NR_UL_GENERATOR_LEVEL_DUT_LIMIT                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 10610
#define RSSPECAN_ATTR_5G_NR_UL_GENERATOR_LEVEL_DUT_GAIN                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 10611
#define RSSPECAN_ATTR_5G_NR_UL_GENERATOR_UPLOAD_RF_SETTINGS                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 10612
#define RSSPECAN_ATTR_5G_NR_UL_GENERATOR_DOWNLOAD_RF_SETTINGS                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 10613
#define RSSPECAN_ATTR_5G_NR_UL_GENERATOR_DOWNLOAD_NR5G_SETTINGS                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 10614
#define RSSPECAN_ATTR_5G_NR_UL_GENERATOR_SETTINGS_PERIODIC_SYNCHRONIZATION_ENABLED                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 10615
#define RSSPECAN_ATTR_5G_NR_UL_GENERATOR_SETTINGS_PERIODIC_SYNCHRONIZATION_INTERVAL                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 10616
#define RSSPECAN_ATTR_5G_NR_UL_FRAME_PASTE_ALL                                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 7845
#define RSSPECAN_ATTR_5G_NR_UL_NUMBER_OF_CONFIGURABLE_FRAMES                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 7846
#define RSSPECAN_ATTR_5G_NR_UL_LOAD_TEST_MODEL                                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 11168
#define RSSPECAN_ATTR_5G_NR_UL_FRAME_COPY                                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 7847
#define RSSPECAN_ATTR_5G_NR_UL_FRAME_PASTE                                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 7848
#define RSSPECAN_ATTR_5G_NR_UL_NUMBER_OF_SLOTS_TO_CONFIGURE                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 7849
#define RSSPECAN_ATTR_5G_NR_UL_SLOTS_COUNTS                                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 7850
#define RSSPECAN_ATTR_5G_NR_UL_PUSCH_ALLOCATIONS_IN_SLOT                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 7851
#define RSSPECAN_ATTR_5G_NR_UL_SLOT_PASTE_ALL                                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 7852
#define RSSPECAN_ATTR_5G_NR_UL_SLOT_PASTE                                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 7853
#define RSSPECAN_ATTR_5G_NR_UL_SLOT_PASTE_TO_MODE                                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 10617
#define RSSPECAN_ATTR_5G_NR_UL_SLOT_PASTE_TO_DURATION                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 10618
#define RSSPECAN_ATTR_5G_NR_UL_SLOT_PASTE_TO_PERIOD                                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 10619
#define RSSPECAN_ATTR_5G_NR_UL_SLOT_PASTE_TO_SLOT                                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 10620
#define RSSPECAN_ATTR_5G_NR_UL_SLOT_PASTE_TO                                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 10621
#define RSSPECAN_ATTR_5G_NR_UL_SLOT_PRESET                                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 7854
#define RSSPECAN_ATTR_5G_NR_UL_SLOT_FORMAT                                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 7855
#define RSSPECAN_ATTR_5G_NR_UL_PUSCH_COPY_SLOT_CONFIGURATION                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 7856
#define RSSPECAN_ATTR_5G_NR_UL_PUSCH_SLOT_ALLOCATION                                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 7857
#define RSSPECAN_ATTR_5G_NR_UL_NUMBER_OF_PUCCH                                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 7858
#define RSSPECAN_ATTR_5G_NR_UL_ENHANCED_PUSCH_MAPPING                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 7859
#define RSSPECAN_ATTR_5G_NR_UL_ENHANCED_PUSCH_POWER                                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 7860
#define RSSPECAN_ATTR_5G_NR_UL_ENHANCED_PUSCH_USER_ID                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 7861
#define RSSPECAN_ATTR_5G_NR_UL_ENHANCED_PUSCH_ANTENNA_PORTS                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 7902
#define RSSPECAN_ATTR_5G_NR_UL_ENHANCED_PUSCH_CDM_GROUPS_WITHOUT_DATA                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 7903
#define RSSPECAN_ATTR_5G_NR_UL_ENHANCED_PUSCH_DM_RS_TYPE                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 7862
#define RSSPECAN_ATTR_5G_NR_UL_ENHANCED_PUSCH_ADDITIONAL_DM_RS                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 7863
#define RSSPECAN_ATTR_5G_NR_UL_ENHANCED_PUSCH_DM_RS_LENGTH                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 7864
#define RSSPECAN_ATTR_5G_NR_UL_ENHANCED_PUSCH_DM_RS_MAPPING_TYPE                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 7865
#define RSSPECAN_ATTR_5G_NR_UL_ENHANCED_PUSCH_DM_RS_POWER                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 7866
#define RSSPECAN_ATTR_5G_NR_UL_ENHANCED_PUSCH_DM_RS_SEQUENCE_GENERATION_METHOD                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 7867
#define RSSPECAN_ATTR_5G_NR_UL_ENHANCED_PUSCH_DM_RS_SEED_VALUE                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 7904
#define RSSPECAN_ATTR_5G_NR_UL_ENHANCED_PUSCH_DM_RS_FIRST_SYMBOL                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 7868
#define RSSPECAN_ATTR_5G_NR_UL_ENHANCED_PUSCH_DM_RS_SCRAMBLING_ID                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 8713
#define RSSPECAN_ATTR_5G_NR_UL_ENHANCED_PUSCH_DM_RS_N_ID_RS                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 10622
#define RSSPECAN_ATTR_5G_NR_UL_ENHANCED_PUSCH_DM_RS_N_ID_RS_SEED_VALUE                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 10623
#define RSSPECAN_ATTR_5G_NR_UL_ENHANCED_PUSCH_DM_RS_UPLINK_R16                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 10624
#define RSSPECAN_ATTR_5G_NR_UL_ENHANCED_PUSCH_PTRS_ENABLED                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 7869
#define RSSPECAN_ATTR_5G_NR_UL_ENHANCED_PUSCH_PTRS_TIME_DOMAIN_OFFSET                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 7870
#define RSSPECAN_ATTR_5G_NR_UL_ENHANCED_PUSCH_PTRS_POWER                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 7871
#define RSSPECAN_ATTR_5G_NR_UL_ENHANCED_PUSCH_PTRS_FREQUENCY_DOMAIN_OFFSET                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 7872
#define RSSPECAN_ATTR_5G_NR_UL_ENHANCED_PUSCH_PTRS_RELATIVE_OFFSET                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 7873
#define RSSPECAN_ATTR_5G_NR_UL_ENHANCED_PUSCH_PTRS_GROUP                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 7874
#define RSSPECAN_ATTR_5G_NR_UL_ENHANCED_PUSCH_PTRS_SAMPLES                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 7875
#define RSSPECAN_ATTR_5G_NR_UL_ENHANCED_PUSCH_PTRS_N_ID                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 10625
#define RSSPECAN_ATTR_5G_NR_UL_ENHANCED_PUSCH_PTRS_N_ID_SEED_VALUE                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 10626
#define RSSPECAN_ATTR_5G_NR_UL_PUSCH_MODULATION                                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 7876
#define RSSPECAN_ATTR_5G_NR_UL_PUSCH_RB_OFFSET                                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 7877
#define RSSPECAN_ATTR_5G_NR_UL_PUSCH_RB_COUNT                                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 7878
#define RSSPECAN_ATTR_5G_NR_UL_PUSCH_SYMBOL_COUNT                                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 7879
#define RSSPECAN_ATTR_5G_NR_UL_PUSCH_SYMBOL_OFFSET                                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 7880
#define RSSPECAN_ATTR_5G_NR_UL_PUSCH_COPY_MODE                                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 10627
#define RSSPECAN_ATTR_5G_NR_UL_PUSCH_COPY_PERIOD                                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 10628
#define RSSPECAN_ATTR_5G_NR_UL_PUSCH_COPY_DURATION                                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 10629
#define RSSPECAN_ATTR_5G_NR_UL_PUSCH_COPY_SLOT                                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 10630
#define RSSPECAN_ATTR_5G_NR_UL_PUSCH_COPY                                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 10631
#define RSSPECAN_ATTR_5G_NR_UL_PUSCH_SCRAMBLING_MCS_INDEX                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 7881
#define RSSPECAN_ATTR_5G_NR_UL_PUSCH_SCRAMBLING_MCS_TABLE_ORDER                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 7882
#define RSSPECAN_ATTR_5G_NR_UL_PUSCH_SCRAMBLING_REDUNDANCY_VERSION_INDEX                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 7883
#define RSSPECAN_ATTR_5G_NR_UL_PUSCH_SCRAMBLING_METHOD                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 7884
#define RSSPECAN_ATTR_5G_NR_UL_PUSCH_SCRAMBLING_DEFINE_SEED                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 7885
#define RSSPECAN_ATTR_5G_NR_UL_PUCCH_POWER                                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 7886
#define RSSPECAN_ATTR_5G_NR_UL_PUCCH_RESOURCE_BLOCKS                                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 7887
#define RSSPECAN_ATTR_5G_NR_UL_PUCCH_RB_OFFSET                                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 7888
#define RSSPECAN_ATTR_5G_NR_UL_PUCCH_SYMBOLS                                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 7889
#define RSSPECAN_ATTR_5G_NR_UL_PUCCH_SYMBOLS_OFFSET                                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 7890
#define RSSPECAN_ATTR_5G_NR_UL_PUCCH_COPY_MODE                                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 10632
#define RSSPECAN_ATTR_5G_NR_UL_PUCCH_COPY_DURATION                                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 10633
#define RSSPECAN_ATTR_5G_NR_UL_PUCCH_COPY_PERIOD                                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 10634
#define RSSPECAN_ATTR_5G_NR_UL_PUCCH_COPY_SLOTS                                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 10635
#define RSSPECAN_ATTR_5G_NR_UL_PUCCH_COPY                                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 10636
#define RSSPECAN_ATTR_5G_NR_UL_PUCCH_INTRA_SLOT_ENABLED                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 7891
#define RSSPECAN_ATTR_5G_NR_UL_PUCCH_GROUP_HOPPING                                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 7892
#define RSSPECAN_ATTR_5G_NR_UL_PUCCH_HOPPING_ID                                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 7893
#define RSSPECAN_ATTR_5G_NR_UL_PUCCH_INITIAL_CYCLIC_SHIFT                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 7894
#define RSSPECAN_ATTR_5G_NR_UL_PUCCH_SEED                                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 7895
#define RSSPECAN_ATTR_5G_NR_UL_PUCCH_DMRS_POWER                                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 7896
#define RSSPECAN_ATTR_5G_NR_UL_PUCCH_DMRS_SEQUENCE_GENERATION_METHOD                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 7897
#define RSSPECAN_ATTR_5G_NR_UL_PUCCH_DMRS_SCRAMBLING_ID                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 7898
#define RSSPECAN_ATTR_5G_NR_UL_PUCCH_DMRS_TIME_DOMAIN_INDEX                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 7899
#define RSSPECAN_ATTR_5G_NR_UL_PUCCH_DMRS_ADDITIONAL_ENABLED                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 11169
#define RSSPECAN_ATTR_5G_NR_UL_PUCCH_DMRS_OCC_LENGTH                                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 11170
#define RSSPECAN_ATTR_5G_NR_UL_PUCCH_HOP_PRB                                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 7900
#define RSSPECAN_ATTR_5G_NR_UL_PUCCH_FORMAT                                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 7901
#define RSSPECAN_ATTR_5G_NR_UL_SRS_ENABLED                                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 7905
#define RSSPECAN_ATTR_5G_NR_UL_SRS_B_HOP                                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 7906
#define RSSPECAN_ATTR_5G_NR_UL_SRS_B_SRS                                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 7907
#define RSSPECAN_ATTR_5G_NR_UL_SRS_CSRS                                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 7908
#define RSSPECAN_ATTR_5G_NR_UL_SRS_FREQUENCY_POSITION                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 7909
#define RSSPECAN_ATTR_5G_NR_UL_SRS_FREQUENCY_SHIFT                                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 7910
#define RSSPECAN_ATTR_5G_NR_UL_SRS_ANTENNA_PORTS                                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 7911
#define RSSPECAN_ATTR_5G_NR_UL_SRS_SYMBOLS                                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 7912
#define RSSPECAN_ATTR_5G_NR_UL_SRS_RELATIVE_POWER                                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 7913
#define RSSPECAN_ATTR_5G_NR_UL_SRS_REPETITION_FACTOR                                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 7914
#define RSSPECAN_ATTR_5G_NR_UL_SRS_CYCLIC_SHIFT                                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 7915
#define RSSPECAN_ATTR_5G_NR_UL_SRS_HOPPING                                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 7916
#define RSSPECAN_ATTR_5G_NR_UL_SRS_SEQUENCE_ID                                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 7917
#define RSSPECAN_ATTR_5G_NR_UL_SRS_SLOT_PERIODICITY                                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 7918
#define RSSPECAN_ATTR_5G_NR_UL_SRS_SLOT_OFFSET                                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 7919
#define RSSPECAN_ATTR_5G_NR_UL_SRS_START_POSITION                                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 7920
#define RSSPECAN_ATTR_5G_NR_UL_SRS_TRANSMISSION_COMB_OFFSET                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 7921
#define RSSPECAN_ATTR_5G_NR_UL_SRS_TRANSMISSION_COMB                                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 7922
#define RSSPECAN_ATTR_5G_NR_UL_ADVANCED_SETTINGS_DC_CARRIER_ENABLED                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 7923
#define RSSPECAN_ATTR_5G_NR_UL_ADVANCED_SETTINGS_RF_UPCONVERSION_ENABLED                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 7924
#define RSSPECAN_ATTR_5G_NR_UL_ADVANCED_SETTINGS_RF_UPCONVERSION_MODE                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 7925
#define RSSPECAN_ATTR_5G_NR_UL_ADVANCED_SETTINGS_RF_UPCONVERSION_FREQUENCY                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 7926
#define RSSPECAN_ATTR_5G_NR_UL_ADVANCED_SETTINGS_REF_POINT_A_FREQUENCY                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 7927
#define RSSPECAN_ATTR_5G_NR_UL_ADVANCED_SETTINGS_REF_POINT_A_15_KHZ_OFFSET                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 7928
#define RSSPECAN_ATTR_5G_NR_UL_ADVANCED_SETTINGS_REF_POINT_A_30_KHZ_OFFSET                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 7929
#define RSSPECAN_ATTR_5G_NR_UL_ADVANCED_SETTINGS_REF_POINT_A_60_KHZ_OFFSET                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 7930
#define RSSPECAN_ATTR_5G_NR_UL_ADVANCED_SETTINGS_REF_POINT_A_120_KHZ_OFFSET                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 7931
#define RSSPECAN_ATTR_5G_NR_UL_ADVANCED_SETTINGS_REF_POINT_A_480_KHZ_OFFSET                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 11171
#define RSSPECAN_ATTR_5G_NR_UL_ADVANCED_SETTINGS_REF_POINT_A_960_KHZ_OFFSET                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 11172
#define RSSPECAN_ATTR_5G_NR_UL_ADVANCED_SETTINGS_REF_POINT_A_TX_BW_OFFSET                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 11173
#define RSSPECAN_ATTR_5G_NR_UL_ADVANCED_SETTINGS_REF_POINT_A_CENTER_FREQUENCY_OFFSET                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 7932
#define RSSPECAN_ATTR_5G_NR_UL_ADVANCED_SETTINGS_TRANSFORM_PRECODING                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 7933
#define RSSPECAN_ATTR_5G_NR_UL_ADVANCED_SETTINGS_PUSCH_HOPPING                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 7934
#define RSSPECAN_ATTR_5G_NR_UL_ADVANCED_SETTINGS_PUSCH_FREQUENCY_HOPPING                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 10637
#define RSSPECAN_ATTR_5G_NR_UL_ADVANCED_SETTINGS_FRAME_NUMBER                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 7820
#define RSSPECAN_ATTR_5G_NR_UL_ADVANCED_SETTINGS_EXTREME_CONDITIONS                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 8714
#define RSSPECAN_ATTR_5G_NR_UL_ANTENNA_PORT_ENABLED                                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 7935
#define RSSPECAN_ATTR_5G_NR_UL_ANTENNA_PORT_SELECT                                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 7936
#define RSSPECAN_ATTR_5G_NR_UL_TRACE_TYPE                                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 7937
#define RSSPECAN_ATTR_5G_NR_UL_Y_SCALING_AUTO                                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 7939
#define RSSPECAN_ATTR_5G_NR_UL_Y_SCALING_MAXIMUM                                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 7940
#define RSSPECAN_ATTR_5G_NR_UL_Y_SCALING_MINIMUM                                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 7941
#define RSSPECAN_ATTR_5G_NR_UL_TAB_SELECT                                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 7942
#define RSSPECAN_ATTR_5G_NR_UL_TRACKING_CHANNEL_ESTIMATION_METHOD                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 7943
#define RSSPECAN_ATTR_5G_NR_UL_TRACKING_GAIN_IMBALANCE_CALCULATION                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 7944
#define RSSPECAN_ATTR_5G_NR_UL_TRACKING_PHASE                                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 7945
#define RSSPECAN_ATTR_5G_NR_UL_TRACKING_TIME                                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 8715
#define RSSPECAN_ATTR_5G_NR_UL_TRACKING_LEVEL                                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 8716
#define RSSPECAN_ATTR_5G_NR_UL_TRIGGER_OFFSET                                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 7946
#define RSSPECAN_ATTR_5G_NR_UL_TRIGGER_LEVEL                                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 7947
#define RSSPECAN_ATTR_5G_NR_UL_TRIGGER_IQ_POWER                                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 7948
#define RSSPECAN_ATTR_5G_NR_UL_TRIGGER_RF_POWER                                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 7949
#define RSSPECAN_ATTR_5G_NR_UL_TRIGGER_PORT                                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 7950
#define RSSPECAN_ATTR_5G_NR_UL_TRIGGER_SOURCE                                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 7951
#define RSSPECAN_ATTR_5G_NR_UL_TRIGGER_EXTERNAL_POWER                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 7952
#define RSSPECAN_ATTR_5G_NR_UL_INPUT_COUPLING                                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 7953
#define RSSPECAN_ATTR_5G_NR_UL_INPUT_SAMPLE_RATE_ENABLED                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 7954
#define RSSPECAN_ATTR_5G_NR_UL_INPUT_SAMPLE_RATE                                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 7955
#define RSSPECAN_ATTR_5G_NR_UL_INPUT_FULL_SCALE_LEVEL_UNIT                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 7956
#define RSSPECAN_ATTR_5G_NR_UL_INPUT_FULL_SCALE_LEVEL_AUTO                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 7957
#define RSSPECAN_ATTR_5G_NR_UL_INPUT_FULL_SCALE_LEVEL                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 7958
#define RSSPECAN_ATTR_5G_NR_UL_INPUT_IQ_BALANCED_ENABLED                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 7959
#define RSSPECAN_ATTR_5G_NR_UL_INPUT_IQ_TYPE                                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 7960
#define RSSPECAN_ATTR_5G_NR_UL_INPUT_TYPE                                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 7961
#define RSSPECAN_ATTR_5G_NR_UL_COMBINED_MEASUREMENT_ACLR                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 10638
#define RSSPECAN_ATTR_5G_NR_UL_COMBINED_MEASUREMENT_EVM                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 10639
#define RSSPECAN_ATTR_5G_NR_UL_COMBINED_MEASUREMENT_SEM                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 10640
#define RSSPECAN_ATTR_5G_NR_UL_COMBINED_MEASUREMENT_LOAD_ACLR_SETTINGS                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 10641
#define RSSPECAN_ATTR_5G_NR_UL_COMBINED_MEASUREMENT_CLEAR_ACLR_SETTINGS                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 10642
#define RSSPECAN_ATTR_5G_NR_UL_COMBINED_MEASUREMENT_LOAD_SEM_SETTINGS                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 10643
#define RSSPECAN_ATTR_5G_NR_UL_COMBINED_MEASUREMENT_CLEAR_SEM_SETTINGS                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 10644
#define RSSPECAN_ATTR_5G_NR_UL_COMBINED_MEASUREMENT_NR_OF_EVENTS                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 10645
#define RSSPECAN_ATTR_5G_NR_UL_COMBINED_MEASUREMENT_OPTIMIZATION                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 10646
#define RSSPECAN_ATTR_5G_NR_UL_COMBINED_MEASUREMENT_SEQUENCE_AUTO_LEVEL                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 10647
#define RSSPECAN_ATTR_5G_NR_UL_COMBINED_MEASUREMENT_FRAME_TRIGGER                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 10648
#define RSSPECAN_ATTR_5G_NR_UL_COMBINED_MEASUREMENT_TRIGGER_SEQUENCE                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 10649
#define RSSPECAN_ATTR_5G_NR_UL_COMBINED_MEASUREMENT_TRIGGER_EVENT_DELAY                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 10650
#define RSSPECAN_ATTR_5G_NR_UL_COMBINED_MEASUREMENT_TRIGGER_RECALIBRATION_INTERVAL                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 10651
#define RSSPECAN_ATTR_5G_NR_UL_COMBINED_MEASUREMENT_SEND_TRIGGER_3                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 10652
#define RSSPECAN_ATTR_5G_NR_UL_COMBINED_MEASUREMENT_TRIGGER_AUTO_GATING                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 10653
#define RSSPECAN_ATTR_5G_NR_UL_COMBINED_MEASUREMENT_RESTORE_SETTINGS                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 11174
#define RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_SHOW_RESULT_SUMMARY                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 7962
#define RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_X_SCALE                                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 7963
#define RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_SUBCARRIER_X_AXIS                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 7964
#define RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_REFERENCE_X_AXIS                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 7965
#define RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_EVM_MAX_HOLD                                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 8717
#define RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_MULTIPLE_CARRIERS                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 7966
#define RSSPECAN_ATTR_5G_NR_UL_AUTOMATIC_CONFIGURATION_EVM                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 7967
#define RSSPECAN_ATTR_5G_NR_UL_AUTO_EVM_SLOTS                                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 10654
#define RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_3D_VIEW                                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 10655
#define RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_ACLR_LIMIT_PASS_MODE                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 10656
#define RSSPECAN_ATTR_5G_NR_UL_POWER_AVERAGING_MODE                                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 11175
#define RSSPECAN_ATTR_5G_NR_UL_THROUGHPUT_MEASUREMENT_ENABLED                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 11176
#define RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_SUMMARY_ALL_CARRIERS_EVM                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 10657
#define RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_SUMMARY_ALL_CARRIERS_EVM_DMRS_PI2_BPSK                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 10658
#define RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_SUMMARY_ALL_CARRIERS_EVM_DMRS_PUSCH_QPSK                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 10659
#define RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_SUMMARY_ALL_CARRIERS_EVM_DMRS_PUSCH_16QAM                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 10660
#define RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_SUMMARY_ALL_CARRIERS_EVM_DMRS_PUSCH_64QAM                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 10661
#define RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_SUMMARY_ALL_CARRIERS_EVM_DMRS_PUSCH_256QAM                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 10662
#define RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_SUMMARY_ALL_CARRIERS_EVM_DMRS_PUSCH_1024QAM                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 11177
#define RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_SUMMARY_ALL_CARRIERS_EVM_DMRS_PUSCH_RESOURCE_ELEMENTS_1024QAM        RS_SPECIFIC_PUBLIC_ATTR_BASE + 11178
#define RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_SUMMARY_ALL_CARRIERS_EVM_PUCCH_DMRS                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 10663
#define RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_SUMMARY_ALL_CARRIERS_EVM_PUCCH                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 10664
#define RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_SUMMARY_ALL_CARRIERS_EVM_PUSCH_PI2_BPSK                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 10665
#define RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_SUMMARY_ALL_CARRIERS_EVM_PUSCH_QPSK                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 10666
#define RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_SUMMARY_ALL_CARRIERS_EVM_PUSCH_16QAM                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 10667
#define RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_SUMMARY_ALL_CARRIERS_EVM_PUSCH_64QAM                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 10668
#define RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_SUMMARY_ALL_CARRIERS_EVM_PUSCH_256QAM                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 10669
#define RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_SUMMARY_ALL_CARRIERS_EVM_PEAK                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 10670
#define RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_SUMMARY_ALL_CARRIERS_EVM_PHYSICAL_CHANNEL                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 10671
#define RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_SUMMARY_ALL_CARRIERS_EVM_PHYSICAL_SIGNAL                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 10672
#define RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_SUMMARY_ALL_CARRIERS_SIGNAL_POWER                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 10673
#define RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_SUMMARY_ALL_CARRIERS_EVM_BLER_MAXIMUM                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 11179
#define RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_SUMMARY_ALL_CARRIERS_EVM_BLER_MINIMUM                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 11180
#define RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_SUMMARY_ALL_CARRIERS_EVM_TPUT_MAXIMUM                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 11181
#define RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_SUMMARY_ALL_CARRIERS_EVM_TPUT_MINIMUM                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 11182
#define RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_EVALUATION_RANGE_ALLOCATION_ALL                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 7968
#define RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_EVALUATION_RANGE_ALLOCATION                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 7969
#define RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_EVALUATION_RANGE_BANDWIDTH_PART_ALL                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 7970
#define RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_EVALUATION_RANGE_BANDWIDTH_PART                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 7971
#define RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_EVALUATION_RANGE_CARRIER_ALL                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 7972
#define RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_EVALUATION_RANGE_CARRIER                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 7973
#define RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_EVALUATION_RANGE_FRAME                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 7974
#define RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_EVALUATION_RANGE_SELECTED_MEAS_ID                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 10674
#define RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_EVALUATION_RANGE_MODULATION_ALL                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 7975
#define RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_EVALUATION_RANGE_MODULATION                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 7976
#define RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_EVALUATION_RANGE_SLOT_ALL                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 7977
#define RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_EVALUATION_RANGE_SLOT                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 7978
#define RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_EVALUATION_RANGE_PREAMBLE_ALL                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 8718
#define RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_EVALUATION_RANGE_PREAMBLE                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 8719
#define RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_EVALUATION_RANGE_SUBFRAME_ALL                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 7979
#define RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_EVALUATION_RANGE_SUBFRAME                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 7980
#define RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_EVALUATION_RANGE_SYMBOL_ALL                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 7981
#define RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_EVALUATION_RANGE_SYMBOL                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 7982
#define RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_EVALUATION_RANGE_SEGMENT                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 7983
#define RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_SYNCHRONIZATION_STATE                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 7984
#define RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_CELL_ID                                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 7985
#define RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_CREST_FACTOR                                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 8569
#define RSSPECAN_ATTR_5G_NR_UL_FRAME_START_OFFSET                                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 7987
#define RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_SUMMARY_EVM_ALL                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 11183
#define RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_SUMMARY_EVM_ALL_MAXIMUM                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 7988
#define RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_SUMMARY_EVM_ALL_MINIMUM                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 7989
#define RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_SUMMARY_EVM_ALL_AVERAGE                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 7990
#define RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_SUMMARY_EVM_BLER                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 11184
#define RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_SUMMARY_EVM_TPUT                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 11185
#define RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_SUMMARY_EVM_PHYSICAL_CHANNEL_MAXIMUM                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 7991
#define RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_SUMMARY_EVM_PHYSICAL_CHANNEL_MINIMUM                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 7992
#define RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_SUMMARY_EVM_PHYSICAL_CHANNEL_AVERAGE                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 7993
#define RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_SUMMARY_EVM_PEAK_MAXIMUM                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 8720
#define RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_SUMMARY_EVM_PEAK_MINIMUM                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 8721
#define RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_SUMMARY_EVM_PEAK_AVERAGE                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 8722
#define RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_SUMMARY_EVM_PRACH_MAXIMUM                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 8723
#define RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_SUMMARY_EVM_PRACH_MINIMUM                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 8724
#define RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_SUMMARY_EVM_PRACH_AVERAGE                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 8725
#define RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_SUMMARY_EVM_PHYSICAL_SIGNAL_MAXIMUM                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 7994
#define RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_SUMMARY_EVM_PHYSICAL_SIGNAL_MINIMUM                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 7995
#define RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_SUMMARY_EVM_PHYSICAL_SIGNAL_AVERAGE                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 7996
#define RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_SUMMARY_EVM_PI2_BPSK_MAXIMUM                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 7997
#define RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_SUMMARY_EVM_PI2_BPSK_MINIMUM                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 7998
#define RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_SUMMARY_EVM_PI2_BPSK_AVERAGE                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 7999
#define RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_SUMMARY_EVM_QPSK_MAXIMUM                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 8000
#define RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_SUMMARY_EVM_QPSK_MINIMUM                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 8001
#define RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_SUMMARY_EVM_QPSK_AVERAGE                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 8570
#define RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_SUMMARY_EVM_64QAM_MAXIMUM                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 8003
#define RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_SUMMARY_EVM_64QAM_MINIMUM                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 8004
#define RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_SUMMARY_EVM_64QAM_AVERAGE                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 8005
#define RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_SUMMARY_EVM_16QAM_MAXIMUM                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 8006
#define RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_SUMMARY_EVM_16QAM_MINIMUM                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 8007
#define RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_SUMMARY_EVM_16QAM_AVERAGE                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 8008
#define RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_SUMMARY_EVM_256QAM_MAXIMUM                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 8009
#define RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_SUMMARY_EVM_256QAM_MINIMUM                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 8010
#define RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_SUMMARY_EVM_256QAM_AVERAGE                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 8011
#define RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_SUMMARY_EVM_DMRS_RESOURCE_ELEMENTS_MAXIMUM                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 8012
#define RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_SUMMARY_EVM_DMRS_RESOURCE_ELEMENTS_MINIMUM                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 8013
#define RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_SUMMARY_EVM_DMRS_RESOURCE_ELEMENTS_AVERAGE                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 8014
#define RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_SUMMARY_EVM_RESOURCE_ELEMENTS_MAXIMUM                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 8015
#define RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_SUMMARY_EVM_RESOURCE_ELEMENTS_MINIMUM                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 8016
#define RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_SUMMARY_EVM_RESOURCE_ELEMENTS_AVERAGE                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 8017
#define RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_SUMMARY_EVM_PUSCH_PI2_BPSK_MAXIMUM                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 8018
#define RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_SUMMARY_EVM_PUSCH_PI2_BPSK_MINIMUM                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 8019
#define RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_SUMMARY_EVM_PUSCH_PI2_BPSK_AVERAGE                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 8020
#define RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_SUMMARY_EVM_PUSCH_QPSK_MAXIMUM                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 8021
#define RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_SUMMARY_EVM_PUSCH_QPSK_MINIMUM                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 8022
#define RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_SUMMARY_EVM_PUSCH_QPSK_AVERAGE                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 8023
#define RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_SUMMARY_EVM_PUSCH_64QAM_MAXIMUM                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 8024
#define RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_SUMMARY_EVM_PUSCH_64QAM_MINIMUM                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 8025
#define RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_SUMMARY_EVM_PUSCH_64QAM_AVERAGE                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 8026
#define RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_SUMMARY_EVM_PUSCH_16QAM_MAXIMUM                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 8027
#define RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_SUMMARY_EVM_PUSCH_16QAM_MINIMUM                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 8028
#define RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_SUMMARY_EVM_PUSCH_16QAM_AVERAGE                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 8029
#define RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_SUMMARY_EVM_PUSCH_256QAM_MAXIMUM                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 8030
#define RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_SUMMARY_EVM_PUSCH_256QAM_MINIMUM                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 8031
#define RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_SUMMARY_EVM_PUSCH_256QAM_AVERAGE                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 8032
#define RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_SUMMARY_EVM_PUSCH_1024QAM_MAXIMUM                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 11186
#define RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_SUMMARY_EVM_PUSCH_1024QAM_MINIMUM                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 11187
#define RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_SUMMARY_EVM_PUSCH_1024QAM_AVERAGE                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 11188
#define RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_SUMMARY_RESOURCE_ELEMENTS_1024QAM_MAXIMUM                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 11189
#define RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_SUMMARY_RESOURCE_ELEMENTS_1024QAM_MINIMUM                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 11190
#define RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_SUMMARY_RESOURCE_ELEMENTS_1024QAM_AVERAGE                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 11191
#define RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_SUMMARY_FREQUENCY_ERROR_MAXIMUM                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 8033
#define RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_SUMMARY_FREQUENCY_ERROR_MINIMUM                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 8034
#define RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_SUMMARY_FREQUENCY_ERROR_AVERAGE                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 8035
#define RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_SUMMARY_GAIN_IMBALANCE_MAXIMUM                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 8036
#define RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_SUMMARY_GAIN_IMBALANCE_MINIMUM                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 8037
#define RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_SUMMARY_GAIN_IMBALANCE_AVERAGE                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 8038
#define RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_SUMMARY_IQ_OFFSET_MAXIMUM                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 8039
#define RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_SUMMARY_IQ_OFFSET_MINIMUM                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 8040
#define RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_SUMMARY_IQ_OFFSET_AVERAGE                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 8041
#define RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_SUMMARY_OSTP_MAXIMUM                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 8042
#define RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_SUMMARY_OSTP_MINIMUM                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 8043
#define RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_SUMMARY_OSTP_AVERAGE                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 8044
#define RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_SUMMARY_POWER_MAXIMUM                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 8045
#define RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_SUMMARY_POWER_MINIMUM                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 8046
#define RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_SUMMARY_POWER_AVERAGE                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 8047
#define RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_SUMMARY_QUADRATURE_ERROR_MAXIMUM                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 8048
#define RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_SUMMARY_QUADRATURE_ERROR_MINIMUM                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 8049
#define RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_SUMMARY_QUADRATURE_ERROR_AVERAGE                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 8050
#define RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_SUMMARY_SAMPLING_ERROR_MAXIMUM                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 8051
#define RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_SUMMARY_SAMPLING_ERROR_MINIMUM                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 8052
#define RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_SUMMARY_SAMPLING_ERROR_AVERAGE                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 8053
#define RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_SUMMARY_ACLR_LIMIT_CHECK                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 10675
#define RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_SUMMARY_ACLR_LIMIT_CHECK_ABSOLUTE                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 10676
#define RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_SUMMARY_ACLR_LIMIT_CHECK_RELATIVE                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 10677
#define RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_SUMMARY_OVERLOAD                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 10678
#define RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_SUMMARY_SEM_PASS_FAIL                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 10679
#define RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_SUMMARY_SYNC_STATE                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 10680
#define RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_SUMMARY_TIME_STAMP_DELTA                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 10681
#define RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_SUMMARY_TIME_STAMP                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 10682
#define RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_VIEW_CARRIER                                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 8054
#define RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_VIEW_FRAME                                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 8055
#define RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_VIEW_WINDOW_COUPLING                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 8056
#define RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_VIEW_SELECT_RESULT_WINDOW                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 8057
#define RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_FREQUENCY_SWEEP_TEST_ID                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 11192
#define RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_FREQUENCY_SWEEP_BW_FOR_ACLR                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 8058
#define RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_FREQUENCY_SWEEP_BS_CATEGORY                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 8059
#define RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_FREQUENCY_SWEEP_POWER_CLASS                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 8060
#define RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_FREQUENCY_SWEEP_IFF                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 8726
#define RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_FREQUENCY_SWEEP_ACLR_IFF                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 10683
#define RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_FREQUENCY_SWEEP_TOTAL_LIMIT_PASS_MODE                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 10684
#define RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_FREQUENCY_SWEEP_GET_ADJACENT_ACLR_LIMIT_CHECK_RESULT                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 8728
#define RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_FREQUENCY_SWEEP_GET_ALTERNATE_ACLR_LIMIT_CHECK_RESULT                RS_SPECIFIC_PUBLIC_ATTR_BASE + 8730
#define RSSPECAN_ATTR_5G_NR_UL_EVM_LIMIT_FREQUENCY_ERROR                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 10693
#define RSSPECAN_ATTR_5G_NR_UL_EVM_LIMITS_PUSCH_DMRS_PI2_BPSK                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 8061
#define RSSPECAN_ATTR_5G_NR_UL_EVM_LIMITS_PUSCH_DMRS_QPSK                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 8062
#define RSSPECAN_ATTR_5G_NR_UL_EVM_LIMITS_PUSCH_DMRS_64QAM                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 8063
#define RSSPECAN_ATTR_5G_NR_UL_EVM_LIMITS_PUSCH_DMRS_16QAM                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 8064
#define RSSPECAN_ATTR_5G_NR_UL_EVM_LIMITS_PUSCH_DMRS_256QAM                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 8065
#define RSSPECAN_ATTR_5G_NR_UL_EVM_LIMITS_PUSCH_DMRS_1024QAM                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 11193
#define RSSPECAN_ATTR_5G_NR_UL_EVM_LIMITS_PUCCH_DMRS                                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 8066
#define RSSPECAN_ATTR_5G_NR_UL_EVM_LIMITS_PUCCH_RESOURCE_ELEMENTS                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 8067
#define RSSPECAN_ATTR_5G_NR_UL_EVM_LIMITS_PDSCH_PI2_BPSK                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 8068
#define RSSPECAN_ATTR_5G_NR_UL_EVM_LIMITS_PUSCH_RESOURCE_WITH_QPSK                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 8069
#define RSSPECAN_ATTR_5G_NR_UL_EVM_LIMITS_PUSCH_RESOURCE_WITH_64QAM                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 8070
#define RSSPECAN_ATTR_5G_NR_UL_EVM_LIMITS_PUSCH_RESOURCE_WITH_16QAM                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 8071
#define RSSPECAN_ATTR_5G_NR_UL_EVM_LIMITS_PUSCH_RESOURCE_WITH_256QAM                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 8072
#define RSSPECAN_ATTR_5G_NR_UL_EVM_LIMITS_PUSCH_RESOURCE_WITH_1024QAM                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 11194
#define RSSPECAN_ATTR_5G_NR_UL_EVM_LIMITS_3GPP                                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 11195
#define RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_COMBINED_MEASUREMENT_EVENT_FILTER                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 10694
#define RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_COMBINED_MEASUREMENT_FILTERED_EVENTS                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 11196
#define RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_COMBINED_MEASUREMENT_EVENT_FILTER_RESET                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 10695
#define RSSPECAN_ATTR_ONEWEB_UL_MODE                                                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 8073
#define RSSPECAN_ATTR_ONEWEB_UL_SELECTED_MEASUREMENT                                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 8074
#define RSSPECAN_ATTR_ONEWEB_UL_NUMBER_OF_COMPONENT_CARRIERS                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 8077
#define RSSPECAN_ATTR_ONEWEB_UL_LO_LOCATION                                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 8078
#define RSSPECAN_ATTR_ONEWEB_UL_LO_FREQUENCY                                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 8079
#define RSSPECAN_ATTR_ONEWEB_UL_AUTO_DEMODULATION_ENABLED                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 8080
#define RSSPECAN_ATTR_ONEWEB_UL_SUBFRAME_CONFIGURATION_DETECTION_ENABLED                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 8081
#define RSSPECAN_ATTR_ONEWEB_UL_BANDWIDTH                                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 8082
#define RSSPECAN_ATTR_ONEWEB_UL_CARRIER_FREQUENCY                                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 8083
#define RSSPECAN_ATTR_ONEWEB_UL_FREQUENCY_OFFSET                                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 8084
#define RSSPECAN_ATTR_ONEWEB_UL_CELL_ID                                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 8085
#define RSSPECAN_ATTR_ONEWEB_UL_CELL_IDENTITY_GROUP                                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 8087
#define RSSPECAN_ATTR_ONEWEB_UL_IDENTITY                                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 8088
#define RSSPECAN_ATTR_ONEWEB_UL_DETECTED_CYCLIC_PREFIX                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 8089
#define RSSPECAN_ATTR_ONEWEB_UL_DETECTED_CELL_IDENTITY_GROUP                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 8090
#define RSSPECAN_ATTR_ONEWEB_UL_DETECTED_IDENTITY                                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 8091
#define RSSPECAN_ATTR_ONEWEB_UL_RESTORE_SAVED_DEMOD_SETTINGS                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 8092
#define RSSPECAN_ATTR_ONEWEB_UL_CONFIGURABLE_SUBFRAMES                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 8094
#define RSSPECAN_ATTR_ONEWEB_UL_SUBFRAME_ALLOCATION_CONTENT                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 8095
#define RSSPECAN_ATTR_ONEWEB_UL_SUBFRAME_MODULATION                                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 8096
#define RSSPECAN_ATTR_ONEWEB_UL_SUBFRAME_DELTA_PUACH                                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 8097
#define RSSPECAN_ATTR_ONEWEB_UL_SUBFRAME_PUCCH_FORMAT                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 8098
#define RSSPECAN_ATTR_ONEWEB_UL_SUBFRAME_N_PUCCH                                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 8099
#define RSSPECAN_ATTR_ONEWEB_UL_SUBFRAME_DMRS_INDEX                                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 8100
#define RSSPECAN_ATTR_ONEWEB_UL_SUBFRAME_RESOURCE_ALLOCATION_TYPE_1_ENABLED                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 8101
#define RSSPECAN_ATTR_ONEWEB_UL_SUBFRAME_CLUSTER_RESOURCE_BLOCKS_COUNT                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 8102
#define RSSPECAN_ATTR_ONEWEB_UL_SUBFRAME_CLUSTER_RESOURCE_BLOCK_OFFSET                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 8103
#define RSSPECAN_ATTR_ONEWEB_UL_SECONDARY_CELL_INDEX                                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 8104
#define RSSPECAN_ATTR_ONEWEB_UL_SYSTEM_FRAME_NUMBER_OFFSET                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 8105
#define RSSPECAN_ATTR_ONEWEB_UL_UE_ID                                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 8106
#define RSSPECAN_ATTR_ONEWEB_UL_DRS_DELTA_SEQUENCE_SHIFT                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 8107
#define RSSPECAN_ATTR_ONEWEB_UL_DRS_GROUP_HOPPING_ENABLED                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 8108
#define RSSPECAN_ATTR_ONEWEB_UL_DRS_N1_DMRS                                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 8109
#define RSSPECAN_ATTR_ONEWEB_UL_DRS_RELATIVE_POWER_PUCCH                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 8110
#define RSSPECAN_ATTR_ONEWEB_UL_DRS_RELATIVE_POWER_PUSCH                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 8111
#define RSSPECAN_ATTR_ONEWEB_UL_DRS_SEQUENCE_HOPPING_ENABLED                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 8112
#define RSSPECAN_ATTR_ONEWEB_UL_SRS_ENABLED                                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 8113
#define RSSPECAN_ATTR_ONEWEB_UL_SRS_SUBFRAME_CONFIGURATION                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 8114
#define RSSPECAN_ATTR_ONEWEB_UL_SRS_BANDWIDTH_B_SRS                                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 8115
#define RSSPECAN_ATTR_ONEWEB_UL_SRS_HOPPING_BW                                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 8116
#define RSSPECAN_ATTR_ONEWEB_UL_SRS_CYCLIC_SHIFT_N_CS                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 8117
#define RSSPECAN_ATTR_ONEWEB_UL_SRS_REL_POWER                                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 8118
#define RSSPECAN_ATTR_ONEWEB_UL_SRS_BW_CONF_C_SRS                                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 8119
#define RSSPECAN_ATTR_ONEWEB_UL_SRS_CONF_INDEX_I_SRS                                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 8120
#define RSSPECAN_ATTR_ONEWEB_UL_SRS_TRANSM_COMB_K_TC                                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 8121
#define RSSPECAN_ATTR_ONEWEB_UL_SRS_FREQ_DOMAIN_POS_N_RRC                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 8122
#define RSSPECAN_ATTR_ONEWEB_UL_SRS_SIMULTANEOUS_TX_ENABLED                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 8123
#define RSSPECAN_ATTR_ONEWEB_UL_PUSCH_NUMBER_OF_SUBBANDS                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 8124
#define RSSPECAN_ATTR_ONEWEB_UL_PUCCH_NO_OF_RBS                                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 8125
#define RSSPECAN_ATTR_ONEWEB_UL_PUCCH_NO_OF_RBS_AUTO                                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 8126
#define RSSPECAN_ATTR_ONEWEB_UL_PUCCH_N1_CS                                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 8127
#define RSSPECAN_ATTR_ONEWEB_UL_PUCCH_DELTA_SHIFT                                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 8128
#define RSSPECAN_ATTR_ONEWEB_UL_PUCCH_FORMAT                                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 8129
#define RSSPECAN_ATTR_ONEWEB_UL_PUCCH_N2_RB                                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 8130
#define RSSPECAN_ATTR_ONEWEB_UL_PUCCH_N_PUCCH                                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 8131
#define RSSPECAN_ATTR_ONEWEB_UL_PUCCH_N_PUCCH_AUTO                                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 8132
#define RSSPECAN_ATTR_ONEWEB_UL_PUCCH_N_PUCCH_SUBFRAME                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 8133
#define RSSPECAN_ATTR_ONEWEB_UL_PRACH_CONFIGURATION                                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 8134
#define RSSPECAN_ATTR_ONEWEB_UL_PRACH_RESTRICTED_SET                                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 8135
#define RSSPECAN_ATTR_ONEWEB_UL_PRACH_FREQUENCY_OFFSET                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 8136
#define RSSPECAN_ATTR_ONEWEB_UL_PRACH_NCS_CONF                                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 8137
#define RSSPECAN_ATTR_ONEWEB_UL_PRACH_LOGICAL_ROOT_SEQ_IDX                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 8138
#define RSSPECAN_ATTR_ONEWEB_UL_PRACH_SEQUENCE_INDEX                                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 8139
#define RSSPECAN_ATTR_ONEWEB_UL_PRACH_SEQUENCE_INDEX_AUTO                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 8140
#define RSSPECAN_ATTR_ONEWEB_UL_OVERALL_FRAME_COUNT_ENABLED                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 8141
#define RSSPECAN_ATTR_ONEWEB_UL_NUMBER_OF_FRAMES_TO_ANALYZE                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 8142
#define RSSPECAN_ATTR_ONEWEB_UL_AUTO_ACCORDING_TO_STANDARD                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 8143
#define RSSPECAN_ATTR_ONEWEB_UL_SINGLE_SUBFRAME_MODE_ENABLED                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 8144
#define RSSPECAN_ATTR_ONEWEB_UL_DEMODULATION_MODE                                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 8145
#define RSSPECAN_ATTR_ONEWEB_UL_DEMODULATION_CHANNEL_ESTIMATION_RANGE                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 8146
#define RSSPECAN_ATTR_ONEWEB_UL_DEMODULATION_EVM_WITH_EXCLUSION_PERIOD_ENABLED                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 8147
#define RSSPECAN_ATTR_ONEWEB_UL_DEMODULATION_COMPENSATE_DC_OFFSET_ENABLED                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 8148
#define RSSPECAN_ATTR_ONEWEB_UL_DEMODULATION_SCRAMBLING_ENABLED                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 8149
#define RSSPECAN_ATTR_ONEWEB_UL_DEMODULATION_MULTICARRIER_FILTER_ENABLED                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 8150
#define RSSPECAN_ATTR_ONEWEB_UL_TRACKING_TIMING_ENABLED                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 8151
#define RSSPECAN_ATTR_ONEWEB_UL_TRACKING_PHASE                                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 8152
#define RSSPECAN_ATTR_ONEWEB_UL_SYNCHRONIZATION_STATUS                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 8153
#define RSSPECAN_ATTR_ONEWEB_UL_X_AXIS_UNIT                                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 8184
#define RSSPECAN_ATTR_ONEWEB_UL_EVM_UNIT                                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 8412
#define RSSPECAN_ATTR_ONEWEB_UL_MARKER_COUPLING_ENABLED                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 8154
#define RSSPECAN_ATTR_ONEWEB_UL_SELECTED_ALLOCATION                                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 8155
#define RSSPECAN_ATTR_ONEWEB_UL_SELECT_ALL_ALLOCATIONS                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 8156
#define RSSPECAN_ATTR_ONEWEB_UL_SELECTED_CARRIER                                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 8157
#define RSSPECAN_ATTR_ONEWEB_UL_SELECT_ALL_CARRIERS                                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 8158
#define RSSPECAN_ATTR_ONEWEB_UL_SELECTED_MODULATION                                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 8159
#define RSSPECAN_ATTR_ONEWEB_UL_SELECT_ALL_MODULATIONS                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 8160
#define RSSPECAN_ATTR_ONEWEB_UL_SELECTED_PREAMBLE                                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 8161
#define RSSPECAN_ATTR_ONEWEB_UL_SELECT_ALL_PREAMBLES                                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 8162
#define RSSPECAN_ATTR_ONEWEB_UL_SELECTED_SLOT                                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 8163
#define RSSPECAN_ATTR_ONEWEB_UL_SELECTED_SUBFRAME                                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 8164
#define RSSPECAN_ATTR_ONEWEB_UL_SELECT_ALL_SUBFRAMES                                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 8165
#define RSSPECAN_ATTR_ONEWEB_UL_SELECTED_SYMBOL                                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 8166
#define RSSPECAN_ATTR_ONEWEB_UL_SELECT_ALL_SYMBOLS                                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 8167
#define RSSPECAN_ATTR_ONEWEB_UL_EVM_ALL_MAX_LIMIT_CHECK_RESULT                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 8168
#define RSSPECAN_ATTR_ONEWEB_UL_EVM_ALL_AVG_LIMIT_CHECK_RESULT                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 8169
#define RSSPECAN_ATTR_ONEWEB_UL_EVM_PHYSICAL_CHANNEL_MAX_LIMIT_CHECK_RESULT                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 8170
#define RSSPECAN_ATTR_ONEWEB_UL_EVM_PHYSICAL_CHANNEL_AVG_LIMIT_CHECK_RESULT                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 8171
#define RSSPECAN_ATTR_ONEWEB_UL_EVM_PHYSICAL_SIGNAL_MAX_LIMIT_CHECK_RESULT                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 8172
#define RSSPECAN_ATTR_ONEWEB_UL_EVM_PHYSICAL_SIGNAL_AVG_LIMIT_CHECK_RESULT                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 8173
#define RSSPECAN_ATTR_ONEWEB_UL_EVM_FERR_MAX_LIMIT_CHECK_RESULT                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 8174
#define RSSPECAN_ATTR_ONEWEB_UL_EVM_FERR_AVG_LIMIT_CHECK_RESULT                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 8175
#define RSSPECAN_ATTR_ONEWEB_UL_GAIN_IMBALANCE_MAX_LIMIT_CHECK_RESULT                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 8176
#define RSSPECAN_ATTR_ONEWEB_UL_GAIN_IMBALANCE_AVG_LIMIT_CHECK_RESULT                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 8177
#define RSSPECAN_ATTR_ONEWEB_UL_IQ_OFFSET_MAX_LIMIT_CHECK_RESULT                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 8178
#define RSSPECAN_ATTR_ONEWEB_UL_IQ_OFFSET_AVG_LIMIT_CHECK_RESULT                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 8179
#define RSSPECAN_ATTR_ONEWEB_UL_QUADRATURE_ERROR_MAX_LIMIT_CHECK_RESULT                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 8180
#define RSSPECAN_ATTR_ONEWEB_UL_QUADRATURE_ERROR_AVG_LIMIT_CHECK_RESULT                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 8181
#define RSSPECAN_ATTR_ONEWEB_UL_SAMPLING_ERROR_MAX_LIMIT_CHECK_RESULT                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 8182
#define RSSPECAN_ATTR_ONEWEB_UL_SAMPLING_ERROR_AVG_LIMIT_CHECK_RESULT                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 8183
#define RSSPECAN_ATTR_ONEWEB_UL_RESULT_FRAME_EVM_DMRS_PUSCH_8PSK                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 8187
#define RSSPECAN_ATTR_ONEWEB_UL_RESULT_FRAME_EVM_DMRS_PUSCH_QPSK                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 8188
#define RSSPECAN_ATTR_ONEWEB_UL_RESULT_FRAME_EVM_DMRS_PUSCH_16QAM                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 8189
#define RSSPECAN_ATTR_ONEWEB_UL_RESULT_FRAME_EVM_PUSCH_8PSK                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 8190
#define RSSPECAN_ATTR_ONEWEB_UL_RESULT_FRAME_EVM_PUSCH_QPSK                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 8191
#define RSSPECAN_ATTR_ONEWEB_UL_RESULT_FRAME_EVM_PUSCH_16QAM                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 8192
#define RSSPECAN_ATTR_ONEWEB_UL_RESULT_FRAME_EVM_DMRS_PUACH_8PSK                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 8193
#define RSSPECAN_ATTR_ONEWEB_UL_RESULT_FRAME_EVM_DMRS_PUACH_QPSK                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 8194
#define RSSPECAN_ATTR_ONEWEB_UL_RESULT_FRAME_EVM_DMRS_PUACH_16QAM                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 8195
#define RSSPECAN_ATTR_ONEWEB_UL_RESULT_FRAME_EVM_PUACH_8PSK                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 8196
#define RSSPECAN_ATTR_ONEWEB_UL_RESULT_FRAME_EVM_PUACH_QPSK                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 8197
#define RSSPECAN_ATTR_ONEWEB_UL_RESULT_FRAME_EVM_PUACH_16QAM                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 8198
#define RSSPECAN_ATTR_ONEWEB_UL_RESULT_FRAME_EVM_DMRS_PUCCH                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 8199
#define RSSPECAN_ATTR_ONEWEB_UL_RESULT_FRAME_EVM_PUCCH                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 8200
#define RSSPECAN_ATTR_ONEWEB_UL_RESULT_FRAME_EVM_PRACH                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 8201
#define RSSPECAN_ATTR_ONEWEB_UL_RESULT_SELECTION_CREST_FACTOR_AVG                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 8202
#define RSSPECAN_ATTR_ONEWEB_UL_RESULT_SELECTION_EVM_MAX                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 8203
#define RSSPECAN_ATTR_ONEWEB_UL_RESULT_SELECTION_EVM_MIN                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 8204
#define RSSPECAN_ATTR_ONEWEB_UL_RESULT_SELECTION_EVM_AVG                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 8205
#define RSSPECAN_ATTR_ONEWEB_UL_RESULT_SELECTION_EVM_PHYSICAL_CHANNEL_MAX                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 8206
#define RSSPECAN_ATTR_ONEWEB_UL_RESULT_SELECTION_EVM_PHYSICAL_CHANNEL_MIN                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 8207
#define RSSPECAN_ATTR_ONEWEB_UL_RESULT_SELECTION_EVM_PHYSICAL_CHANNEL_AVG                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 8208
#define RSSPECAN_ATTR_ONEWEB_UL_RESULT_SELECTION_EVM_PHYSICAL_SIGNAL_MAX                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 8209
#define RSSPECAN_ATTR_ONEWEB_UL_RESULT_SELECTION_EVM_PHYSICAL_SIGNAL_MIN                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 8210
#define RSSPECAN_ATTR_ONEWEB_UL_RESULT_SELECTION_EVM_PHYSICAL_SIGNAL_AVG                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 8211
#define RSSPECAN_ATTR_ONEWEB_UL_RESULT_SELECTION_FREQUENCY_ERROR_MAX                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 8212
#define RSSPECAN_ATTR_ONEWEB_UL_RESULT_SELECTION_FREQUENCY_ERROR_MIN                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 8213
#define RSSPECAN_ATTR_ONEWEB_UL_RESULT_SELECTION_FREQUENCY_ERROR_AVG                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 8214
#define RSSPECAN_ATTR_ONEWEB_UL_RESULT_SELECTION_GAIN_IMBALANCE_MAX                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 8215
#define RSSPECAN_ATTR_ONEWEB_UL_RESULT_SELECTION_GAIN_IMBALANCE_MIN                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 8216
#define RSSPECAN_ATTR_ONEWEB_UL_RESULT_SELECTION_GAIN_IMBALANCE_AVG                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 8217
#define RSSPECAN_ATTR_ONEWEB_UL_RESULT_SELECTION_IQ_OFFSET_MAX                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 8218
#define RSSPECAN_ATTR_ONEWEB_UL_RESULT_SELECTION_IQ_OFFSET_MIN                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 8219
#define RSSPECAN_ATTR_ONEWEB_UL_RESULT_SELECTION_IQ_OFFSET_AVG                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 8220
#define RSSPECAN_ATTR_ONEWEB_UL_RESULT_SELECTION_POWER_MAX                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 8221
#define RSSPECAN_ATTR_ONEWEB_UL_RESULT_SELECTION_POWER_MIN                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 8222
#define RSSPECAN_ATTR_ONEWEB_UL_RESULT_SELECTION_POWER_AVG                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 8223
#define RSSPECAN_ATTR_ONEWEB_UL_RESULT_SELECTION_QUADRATURE_ERROR_MAX                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 8224
#define RSSPECAN_ATTR_ONEWEB_UL_RESULT_SELECTION_QUADRATURE_ERROR_MIN                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 8225
#define RSSPECAN_ATTR_ONEWEB_UL_RESULT_SELECTION_QUADRATURE_ERROR_AVG                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 8226
#define RSSPECAN_ATTR_ONEWEB_UL_RESULT_SELECTION_SAMPLING_ERROR_MAX                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 8227
#define RSSPECAN_ATTR_ONEWEB_UL_RESULT_SELECTION_SAMPLING_ERROR_MIN                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 8228
#define RSSPECAN_ATTR_ONEWEB_UL_RESULT_SELECTION_SAMPLING_ERROR_AVG                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 8229
#define RSSPECAN_ATTR_ONEWEB_UL_RESULT_SELECTION_START_OFFSET                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 8230
#define RSSPECAN_ATTR_V5GTF_MODE                                                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 8231
#define RSSPECAN_ATTR_V5GTF_LINK_DIRECTION                                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 8232
#define RSSPECAN_ATTR_V5GTF_AUTO_EVM                                                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 8233
#define RSSPECAN_ATTR_V5GTF_NUMBER_OF_COMPONENT_CARRIERS                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 8234
#define RSSPECAN_ATTR_V5GTF_CC_SIGNAL_CAPTURE                                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 8235
#define RSSPECAN_ATTR_V5GTF_SAVE_DEMOD_SETTINGS                                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 8236
#define RSSPECAN_ATTR_V5GTF_RESTORE_SAVED_DEMOD_SETTINGS                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 8238
#define RSSPECAN_ATTR_V5GTF_SAVE_DEMOD_ALL_SETTINGS                                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 8240
#define RSSPECAN_ATTR_V5GTF_RESTORE_SAVED_DEMOD_ALL_SETTINGS                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 8242
#define RSSPECAN_ATTR_V5GTF_CARRIER_FREQUENCY                                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 8244
#define RSSPECAN_ATTR_V5GTF_FREQUENCY_OFFSET                                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 8245
#define RSSPECAN_ATTR_V5GTF_DL_CELL_ID                                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 8246
#define RSSPECAN_ATTR_V5GTF_UL_CELL_ID                                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 8247
#define RSSPECAN_ATTR_V5GTF_DL_CONFIGURABLE_SUBFRAMES                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 8248
#define RSSPECAN_ATTR_V5GTF_UL_CONFIGURABLE_SUBFRAMES                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 8249
#define RSSPECAN_ATTR_V5GTF_RADIO_FRAME_DL_SUBFRAME_ALLOCATION                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 8250
#define RSSPECAN_ATTR_V5GTF_RADIO_FRAME_DL_SUBFRAME_OPTIONAL_REF_SIGNALS                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 8251
#define RSSPECAN_ATTR_V5GTF_RADIO_FRAME_DL_SUBFRAME_SELECTED                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 8252
#define RSSPECAN_ATTR_V5GTF_RADIO_FRAME_DL_SUBFRAME_TYPE                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 8253
#define RSSPECAN_ATTR_V5GTF_RADIO_FRAME_DL_SUBFRAME_COPY                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 8254
#define RSSPECAN_ATTR_V5GTF_RADIO_FRAME_DL_SUBFRAME_PASTE                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 8255
#define RSSPECAN_ATTR_V5GTF_RADIO_FRAME_DL_SUBFRAME_PASTE_ALL                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 8256
#define RSSPECAN_ATTR_V5GTF_RADIO_FRAME_DL_SUBFRAME_RESET                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 8257
#define RSSPECAN_ATTR_V5GTF_RADIO_FRAME_UL_SUBFRAME_ALLOCATION                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 8258
#define RSSPECAN_ATTR_V5GTF_RADIO_FRAME_UL_SUBFRAME_SELECTED                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 8259
#define RSSPECAN_ATTR_V5GTF_RADIO_FRAME_UL_SUBFRAME_TYPE                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 8260
#define RSSPECAN_ATTR_V5GTF_RADIO_FRAME_UL_SUBFRAME_COPY                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 8261
#define RSSPECAN_ATTR_V5GTF_RADIO_FRAME_UL_SUBFRAME_PASTE                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 8262
#define RSSPECAN_ATTR_V5GTF_RADIO_FRAME_UL_SUBFRAME_PASTE_ALL                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 8263
#define RSSPECAN_ATTR_V5GTF_RADIO_FRAME_UL_SUBFRAME_RESET                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 8264
#define RSSPECAN_ATTR_V5GTF_XPDSCH_SUBFRAME_ALLOCATIONS                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 8265
#define RSSPECAN_ATTR_V5GTF_XPDSCH_SUBFRAME_ID_OR_N_RNTI                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 8272
#define RSSPECAN_ATTR_V5GTF_XPDSCH_SUBFRAME_MODULATION                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 8266
#define RSSPECAN_ATTR_V5GTF_XPDSCH_SUBFRAME_NUMBER_OF_RB                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 8269
#define RSSPECAN_ATTR_V5GTF_XPDSCH_SUBFRAME_OFFSET_RB                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 8270
#define RSSPECAN_ATTR_V5GTF_XPDSCH_SUBFRAME_POWER                                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 8267
#define RSSPECAN_ATTR_V5GTF_XPDSCH_SUBFRAME_PRECODING                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 8268
#define RSSPECAN_ATTR_V5GTF_XPDSCH_SUBFRAME_SCRAMBLING_IDENTITY_N_SCID                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 8271
#define RSSPECAN_ATTR_V5GTF_XPDSCH_SUBFRAME_ANTENNA_PORT                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 8273
#define RSSPECAN_ATTR_V5GTF_XPDSCH_SUBFRAME_NO_OF_LAYERS                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 8274
#define RSSPECAN_ATTR_V5GTF_XPDSCH_SUBFRAME_DMRS_ID                                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 8275
#define RSSPECAN_ATTR_V5GTF_XPDSCH_SUBFRAME_DMRS_POWER                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 8276
#define RSSPECAN_ATTR_V5GTF_XPDSCH_SUBFRAME_DMRS_SEQUENCE_GENERATION                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 8277
#define RSSPECAN_ATTR_V5GTF_XPDSCH_SUBFRAME_PCRS_ENABLED                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 8278
#define RSSPECAN_ATTR_V5GTF_XPDSCH_SUBFRAME_PCRS_ID                                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 8279
#define RSSPECAN_ATTR_V5GTF_XPDSCH_SUBFRAME_PCRS_POWER                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 8280
#define RSSPECAN_ATTR_V5GTF_XPDSCH_SUBFRAME_PCRS_ANTENNA_PORT                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 8281
#define RSSPECAN_ATTR_V5GTF_XPDSCH_SUBFRAME_PCRS_SEQUENCE_GENERATION                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 8282
#define RSSPECAN_ATTR_V5GTF_XPUSCH_SUBFRAME_ALLOCATIONS                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 8283
#define RSSPECAN_ATTR_V5GTF_XPUSCH_SUBFRAME_MODULATION                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 8284
#define RSSPECAN_ATTR_V5GTF_XPUSCH_SUBFRAME_NUMBER_OF_RB                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 8286
#define RSSPECAN_ATTR_V5GTF_XPUSCH_SUBFRAME_OFFSET_RB                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 8287
#define RSSPECAN_ATTR_V5GTF_XPUSCH_SUBFRAME_POWER                                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 8285
#define RSSPECAN_ATTR_V5GTF_XPUSCH_SUBFRAME_RE_MAPPING_INDEX                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 8288
#define RSSPECAN_ATTR_V5GTF_XPUSCH_SUBFRAME_UE_SCRAMBLING_IDENTITY_N_SCID                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 8289
#define RSSPECAN_ATTR_V5GTF_XPUSCH_SUBFRAME_DMRS_ID                                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 8290
#define RSSPECAN_ATTR_V5GTF_XPUSCH_SUBFRAME_DMRS_POWER                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 8291
#define RSSPECAN_ATTR_V5GTF_XPUSCH_SUBFRAME_DMRS_SEQUENCE_GENERATION                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 8292
#define RSSPECAN_ATTR_V5GTF_XPUSCH_SUBFRAME_PCRS_ENABLED                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 8293
#define RSSPECAN_ATTR_V5GTF_XPUSCH_SUBFRAME_PCRS_ID                                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 8294
#define RSSPECAN_ATTR_V5GTF_XPUSCH_SUBFRAME_PCRS_POWER                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 8295
#define RSSPECAN_ATTR_V5GTF_XPUSCH_SUBFRAME_PCRS_TRANSMISSION                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 8297
#define RSSPECAN_ATTR_V5GTF_XPUSCH_SUBFRAME_PCRS_SEQUENCE_GENERATION                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 8296
#define RSSPECAN_ATTR_V5GTF_SYNCHRONIZATION_SIGNAL_PSS_POWER                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 8299
#define RSSPECAN_ATTR_V5GTF_SYNCHRONIZATION_SIGNAL_SSS_POWER                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 8300
#define RSSPECAN_ATTR_V5GTF_SYNCHRONIZATION_SIGNAL_ESYNC_POWER                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 8298
#define RSSPECAN_ATTR_V5GTF_REFERENCE_SIGNAL_EPBCH_PRECODING                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 8302
#define RSSPECAN_ATTR_V5GTF_REFERENCE_SIGNAL_EPBCH_POWER                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 8301
#define RSSPECAN_ATTR_V5GTF_REFERENCE_SIGNAL_XPDSCH_PRECODING                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 8303
#define RSSPECAN_ATTR_V5GTF_REFERENCE_SIGNAL_SUBFRAME_N_ID_CSI                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 8304
#define RSSPECAN_ATTR_V5GTF_REFERENCE_SIGNAL_SUBFRAME_POWER                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 8305
#define RSSPECAN_ATTR_V5GTF_REFERENCE_SIGNAL_SUBFRAME_SYMBOL_TRANSMISSION_ENABLED                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 8306
#define RSSPECAN_ATTR_V5GTF_CONTROL_CHANNEL_DL_XPDCCH_NUM_OF_XREGS                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 8307
#define RSSPECAN_ATTR_V5GTF_CONTROL_CHANNEL_DL_XPDCCH_SEQUENCE_ID                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 8308
#define RSSPECAN_ATTR_V5GTF_CONTROL_CHANNEL_DL_XPDCCH_PRECODING                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 8309
#define RSSPECAN_ATTR_V5GTF_CONTROL_CHANNEL_DL_XPDCCH_POWER                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 8310
#define RSSPECAN_ATTR_V5GTF_CONTROL_CHANNEL_DL_XPDCCH_SEQUENCE_GENERATION                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 8311
#define RSSPECAN_ATTR_V5GTF_CONTROL_CHANNEL_DL_XPDCCH_XREG_ENABLED                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 8312
#define RSSPECAN_ATTR_V5GTF_CONTROL_CHANNEL_UL_XPUCCH_ENABLED                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 8313
#define RSSPECAN_ATTR_V5GTF_CONTROL_CHANNEL_UL_XPUCCH_PRECODING                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 8314
#define RSSPECAN_ATTR_V5GTF_CONTROL_CHANNEL_UL_XPUCCH_LOCATION                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 8315
#define RSSPECAN_ATTR_V5GTF_CONTROL_CHANNEL_UL_XPUCCH_N_RNTI                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 8316
#define RSSPECAN_ATTR_V5GTF_CONTROL_CHANNEL_UL_XPUCCH_POWER                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 8317
#define RSSPECAN_ATTR_V5GTF_CONTROL_CHANNEL_UL_XPUCCH_POWER_DMRS                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 8318
#define RSSPECAN_ATTR_V5GTF_CONTROL_CHANNEL_UL_XPUCCH_SEQUENCE_GENERATION                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 8319
#define RSSPECAN_ATTR_V5GTF_CONTROL_CHANNEL_UL_XPUCCH_SEQUENCE_ID                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 8320
#define RSSPECAN_ATTR_V5GTF_ANTENNA_PORT_CONFIGURATION_ENABLED                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 8321
#define RSSPECAN_ATTR_V5GTF_MAX_NUMBER_OF_SUBFRAMES                                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 8323
#define RSSPECAN_ATTR_V5GTF_DEMODULATION_MULTICARRIER_FILTER_ENABLED                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 8325
#define RSSPECAN_ATTR_V5GTF_LEVEL_TRACKING_ENABLED                                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 8326
#define RSSPECAN_ATTR_V5GTF_SUBWINDOW_COUPLING_ENABLED                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 8329
#define RSSPECAN_ATTR_V5GTF_SUBWINDOW_SELECT                                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 8330
#define RSSPECAN_ATTR_V5GTF_Y_AXIS_AUTO_SCALING_ENABLED                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 8331
#define RSSPECAN_ATTR_V5GTF_Y_AXIS_MAX                                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 8332
#define RSSPECAN_ATTR_V5GTF_Y_AXIS_MIN                                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 8333
#define RSSPECAN_ATTR_V5GTF_SYNCHRONIZATION_STATE                                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 8334
#define RSSPECAN_ATTR_V5GTF_MARKER_COUPLING_ENABLED                                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 8335
#define RSSPECAN_ATTR_V5GTF_X_AXIS_UNIT                                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 8337
#define RSSPECAN_ATTR_V5GTF_EVM_UNIT                                                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 8338
#define RSSPECAN_ATTR_V5GTF_RESULT_CREST_FACTOR                                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 8339
#define RSSPECAN_ATTR_V5GTF_RESULT_EVM                                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 8340
#define RSSPECAN_ATTR_V5GTF_RESULT_EVM_XPDSCH_QPSK                                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 8341
#define RSSPECAN_ATTR_V5GTF_RESULT_EVM_XPDSCH_16QAM                                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 8342
#define RSSPECAN_ATTR_V5GTF_RESULT_EVM_XPDSCH_64QAM                                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 8343
#define RSSPECAN_ATTR_V5GTF_RESULT_EVM_XPDSCH_256QAM                                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 8344
#define RSSPECAN_ATTR_V5GTF_RESULT_EVM_PHYSICAL_CHANNEL                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 8345
#define RSSPECAN_ATTR_V5GTF_RESULT_EVM_PHYSICAL_SIGNAL                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 8346
#define RSSPECAN_ATTR_V5GTF_RESULT_EVM_XPUSCH_QPSK                                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 8347
#define RSSPECAN_ATTR_V5GTF_RESULT_EVM_XPUSCH_16QAM                                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 8348
#define RSSPECAN_ATTR_V5GTF_RESULT_EVM_XPUSCH_64QAM                                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 8349
#define RSSPECAN_ATTR_V5GTF_RESULT_EVM_XPUSCH_256QAM                                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 8350
#define RSSPECAN_ATTR_V5GTF_RESULT_FREQUENCY_ERROR                                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 8351
#define RSSPECAN_ATTR_V5GTF_RESULT_IQ_GAIN_IMBALANCE                                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 8352
#define RSSPECAN_ATTR_V5GTF_RESULT_IQ_OFFSET                                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 8353
#define RSSPECAN_ATTR_V5GTF_RESULT_QUADRATURE_ERROR                                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 8354
#define RSSPECAN_ATTR_V5GTF_RESULT_TOTAL_POWER                                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 8355
#define RSSPECAN_ATTR_V5GTF_RESULT_SAMPLING_ERROR                                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 8356
#define RSSPECAN_ATTR_V5GTF_RESULT_SUBFRAME_START_OFFSET                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 8357
#define RSSPECAN_ATTR_HRP_UWB_MODE                                                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 9680
#define RSSPECAN_ATTR_HRP_UWB_IQ_LOAD_STREAM_AUTO_ENABLED                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 9681
#define RSSPECAN_ATTR_HRP_UWB_IQ_LOAD_STREAM                                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 9682
#define RSSPECAN_ATTR_QUERY_HRP_UWB_IQ_STREAM_LISTS                                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 9683
#define RSSPECAN_ATTR_HRP_UWB_MEASUREMENT_DEFAULT_VALUES                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 9684
#define RSSPECAN_ATTR_HRP_UWB_MEASUREMENT_CHANNEL_NUMBER                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 9685
#define RSSPECAN_ATTR_HRP_UWB_SET_CENTER_FREQUENCY_TO_CHANNEL_FREQUENCY                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 9686
#define RSSPECAN_ATTR_HRP_UWB_OVERSAMPLING_FACTOR                                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 9687
#define RSSPECAN_ATTR_HRP_UWB_BURST_SEARCH_REFERENCE                                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 9688
#define RSSPECAN_ATTR_HRP_UWB_BURST_SEARCH_THRESHOLD                                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 9689
#define RSSPECAN_ATTR_HRP_UWB_BURST_LENGTH_MAXIMUM                                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 11044
#define RSSPECAN_ATTR_HRP_UWB_BURST_LENGTH_MINIMUM                                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 11045
#define RSSPECAN_ATTR_HRP_UWB_BURST_DETECTED_OFFSET                                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 11046
#define RSSPECAN_ATTR_HRP_UWB_BURST_MAX_OFF_TIME_WITHIN_BURST                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 11047
#define RSSPECAN_ATTR_HRP_UWB_DEMODULATION_MODE                                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 9690
#define RSSPECAN_ATTR_HRP_UWB_DEMODULATION_PHY_DATA_RATE_MODE                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 9691
#define RSSPECAN_ATTR_HRP_UWB_DEMODULATION_HPRF_PAYLOAD_SIZE                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 9692
#define RSSPECAN_ATTR_HRP_UWB_DEMODULATION_PACKET_FORMAT                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 9693
#define RSSPECAN_ATTR_HRP_UWB_DEMODULATION_ACTIVE_SEGMENT_LENGTH                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 9694
#define RSSPECAN_ATTR_HRP_UWB_DEMODULATION_ACTIVE_SEGMENTS                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 9695
#define RSSPECAN_ATTR_HRP_UWB_DEMODULATION_MAC_FCS                                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 11197
#define RSSPECAN_ATTR_HRP_UWB_DEMODULATION_STS_GAP                                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 11049
#define RSSPECAN_ATTR_HRP_UWB_DEMODULATION_SELECTED_PACKET                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 9696
#define RSSPECAN_ATTR_HRP_UWB_DEMODULATION_RANGE                                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 9697
#define RSSPECAN_ATTR_HRP_UWB_DEMODULATION_COUNT                                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 9698
#define RSSPECAN_ATTR_HRP_UWB_EVALUATION_RANGE_STATISTICS_GROUP                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 11050
#define RSSPECAN_ATTR_HRP_UWB_EVALUATION_RANGE_PACKET_GROUP                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 11051
#define RSSPECAN_ATTR_HRP_UWB_EVALUATION_PACKETS_ANALYZED                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 11052
#define RSSPECAN_ATTR_HRP_UWB_EVALUATION_FILTER_SET                                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 11053
#define RSSPECAN_ATTR_HRP_UWB_EVALUATION_FILTER_NAME                                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 11054
#define RSSPECAN_ATTR_HRP_UWB_EVALUATION_FILTER_DELETE                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 11055
#define RSSPECAN_ATTR_HRP_UWB_PACKET_ALL                                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 9699
#define RSSPECAN_ATTR_HRP_UWB_PACKET_SYNC_CODE_INDEX                                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 9700
#define RSSPECAN_ATTR_HRP_UWB_PACKET_SYNC_CODE_INDEX_ALL                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 9701
#define RSSPECAN_ATTR_HRP_UWB_PACKET_SYNC_CODE_LENGTH                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 9702
#define RSSPECAN_ATTR_HRP_UWB_PACKET_SYNC_CODE_LENGTH_ALL                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 9703
#define RSSPECAN_ATTR_HRP_UWB_PACKET_SYNC_DELTA_LENGTH                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 9704
#define RSSPECAN_ATTR_HRP_UWB_PACKET_SYNC_DELTA_LENGTH_ALL                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 9705
#define RSSPECAN_ATTR_HRP_UWB_PACKET_SYNC_SYNC_LENGTH                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 9706
#define RSSPECAN_ATTR_HRP_UWB_PACKET_SYNC_SYNC_LENGTH_ALL                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 9707
#define RSSPECAN_ATTR_HRP_UWB_PACKET_SYNC_SFD                                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 11198
#define RSSPECAN_ATTR_HRP_UWB_PACKET_SYNC_SFD_ALL                                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 11199
#define RSSPECAN_ATTR_HRP_UWB_PACKET_DATA_RATE                                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 9708
#define RSSPECAN_ATTR_HRP_UWB_PACKET_DATA_RATE_ALL                                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 9709
#define RSSPECAN_ATTR_HRP_UWB_PACKET_DATA_LENGTH                                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 10698
#define RSSPECAN_ATTR_HRP_UWB_PACKET_DATA_LENGTH_ALL                                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 10699
#define RSSPECAN_ATTR_HRP_UWB_PACKET_DATA_RANGING_BIT                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 10700
#define RSSPECAN_ATTR_HRP_UWB_PACKET_DATA_RANGING_BIT_ALL                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 9713
#define RSSPECAN_ATTR_HRP_UWB_PACKET_DATA_SECDEC                                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 9714
#define RSSPECAN_ATTR_HRP_UWB_PACKET_DATA_SECDEC_ALL                                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 9715
#define RSSPECAN_ATTR_HRP_UWB_PACKET_DATA_CONSTRAINT_LENGTH                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 9716
#define RSSPECAN_ATTR_HRP_UWB_PACKET_DATA_CONSTRAINT_LENGTH_ALL                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 9717
#define RSSPECAN_ATTR_HRP_UWB_PACKET_DATA_A0A1                                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 9718
#define RSSPECAN_ATTR_HRP_UWB_PACKET_DATA_A0A1_ALL                                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 9719
#define RSSPECAN_ATTR_HRP_UWB_PACKET_DATA_PAYLOAD                                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 9720
#define RSSPECAN_ATTR_HRP_UWB_PACKET_DATA_RATE_PHR                                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 11056
#define RSSPECAN_ATTR_HRP_UWB_PACKET_DATA_C_BURST                                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 11057
#define RSSPECAN_ATTR_HRP_UWB_PACKET_DATA_C_BURST_ALL                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 11058
#define RSSPECAN_ATTR_HRP_UWB_PACKET_DATA_H_BURST                                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 11059
#define RSSPECAN_ATTR_HRP_UWB_PACKET_DATA_H_BURST_ALL                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 11060
#define RSSPECAN_ATTR_HRP_UWB_PACKET_DATA_MAC_FCS                                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 11061
#define RSSPECAN_ATTR_HRP_UWB_PACKET_DATA_MAC_FCS_ALL                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 11062
#define RSSPECAN_ATTR_HRP_UWB_PACKET_DATA_RESERVED_BIT                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 11063
#define RSSPECAN_ATTR_HRP_UWB_PACKET_DATA_RESERVED_BIT_ALL                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 11064
#define RSSPECAN_ATTR_HRP_UWB_PACKET_DATA_STS_BST                                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 11200
#define RSSPECAN_ATTR_HRP_UWB_RESULT_ALL                                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 9721
#define RSSPECAN_ATTR_HRP_UWB_RESULT_CAPTURE_LENGTH                                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 11069
#define RSSPECAN_ATTR_HRP_UWB_RESULT_CENTER_FREQUENCY_OFFSET_SEL_PACKET                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 9722
#define RSSPECAN_ATTR_HRP_UWB_RESULT_CENTER_FREQUENCY_OFFSET_AVERAGE                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 9723
#define RSSPECAN_ATTR_HRP_UWB_RESULT_CENTER_FREQUENCY_OFFSET_MINIMUM                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 9724
#define RSSPECAN_ATTR_HRP_UWB_RESULT_CENTER_FREQUENCY_OFFSET_MAXIMUM                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 9725
#define RSSPECAN_ATTR_HRP_UWB_RESULT_CENTER_FREQUENCY_OFFSET_PPM_SEL_PACKET                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 9726
#define RSSPECAN_ATTR_HRP_UWB_RESULT_CENTER_FREQUENCY_OFFSET_PPM_AVERAGE                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 9727
#define RSSPECAN_ATTR_HRP_UWB_RESULT_CENTER_FREQUENCY_OFFSET_PPM_MINIMUM                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 9728
#define RSSPECAN_ATTR_HRP_UWB_RESULT_CENTER_FREQUENCY_OFFSET_PPM_MAXIMUM                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 9729
#define RSSPECAN_ATTR_HRP_UWB_RESULT_FREQUENCY_CHIP_CLOCK_ERROR_SEL_PACKET                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 9730
#define RSSPECAN_ATTR_HRP_UWB_RESULT_FREQUENCY_CHIP_CLOCK_ERROR_AVERAGE                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 9731
#define RSSPECAN_ATTR_HRP_UWB_RESULT_FREQUENCY_CHIP_CLOCK_ERROR_MINIMUM                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 9732
#define RSSPECAN_ATTR_HRP_UWB_RESULT_FREQUENCY_CHIP_CLOCK_ERROR_MAXIMUM                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 9733
#define RSSPECAN_ATTR_HRP_UWB_RESULT_SYMBOL_TIME_JITTER_RMS_SEL_PACKET                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 9734
#define RSSPECAN_ATTR_HRP_UWB_RESULT_SYMBOL_TIME_JITTER_RMS_AVERAGE                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 9735
#define RSSPECAN_ATTR_HRP_UWB_RESULT_SYMBOL_TIME_JITTER_RMS_MINIMUM                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 9736
#define RSSPECAN_ATTR_HRP_UWB_RESULT_SYMBOL_TIME_JITTER_RMS_MAXIMUM                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 9737
#define RSSPECAN_ATTR_HRP_UWB_RESULT_CHIP_TIME_JITTER_RMS_SEL_PACKET                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 9738
#define RSSPECAN_ATTR_HRP_UWB_RESULT_CHIP_TIME_JITTER_RMS_AVERAGE                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 9739
#define RSSPECAN_ATTR_HRP_UWB_RESULT_CHIP_TIME_JITTER_RMS_MINIMUM                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 9740
#define RSSPECAN_ATTR_HRP_UWB_RESULT_CHIP_TIME_JITTER_RMS_MAXIMUM                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 9741
#define RSSPECAN_ATTR_HRP_UWB_RESULT_CORRELATED_PULSE_NMSE_SEL_PACKET                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 9742
#define RSSPECAN_ATTR_HRP_UWB_RESULT_CORRELATED_PULSE_NMSE_AVERAGE                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 9743
#define RSSPECAN_ATTR_HRP_UWB_RESULT_CORRELATED_PULSE_NMSE_MINIMUM                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 9744
#define RSSPECAN_ATTR_HRP_UWB_RESULT_CORRELATED_PULSE_NMSE_MAXIMUM                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 9745
#define RSSPECAN_ATTR_HRP_UWB_RESULT_CORRELATED_PULSE_MAIN_LOBE_PEAK_SEL_PACKET                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 9746
#define RSSPECAN_ATTR_HRP_UWB_RESULT_CORRELATED_PULSE_MAIN_LOBE_PEAK_AVERAGE                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 9747
#define RSSPECAN_ATTR_HRP_UWB_RESULT_CORRELATED_PULSE_MAIN_LOBE_PEAK_MINIMUM                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 9748
#define RSSPECAN_ATTR_HRP_UWB_RESULT_CORRELATED_PULSE_MAIN_LOBE_PEAK_MAXIMUM                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 9749
#define RSSPECAN_ATTR_HRP_UWB_RESULT_CORRELATED_PULSE_MAIN_LOBE_WIDTH_SEL_PACKET                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 9750
#define RSSPECAN_ATTR_HRP_UWB_RESULT_CORRELATED_PULSE_MAIN_LOBE_WIDTH_AVERAGE                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 9751
#define RSSPECAN_ATTR_HRP_UWB_RESULT_CORRELATED_PULSE_MAIN_LOBE_WIDTH_MINIMUM                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 9752
#define RSSPECAN_ATTR_HRP_UWB_RESULT_CORRELATED_PULSE_MAIN_LOBE_WIDTH_MAXIMUM                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 9753
#define RSSPECAN_ATTR_HRP_UWB_RESULT_CORRELATED_PULSE_MAIN_LOBE_WIDTH_PASSED_SEL_PACKET                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 9754
#define RSSPECAN_ATTR_HRP_UWB_RESULT_CORRELATED_PULSE_MAIN_LOBE_WIDTH_PASSED_AVERAGE                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 9755
#define RSSPECAN_ATTR_HRP_UWB_RESULT_CORRELATED_PULSE_MAIN_LOBE_WIDTH_PASSED_MINIMUM                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 9756
#define RSSPECAN_ATTR_HRP_UWB_RESULT_CORRELATED_PULSE_MAIN_LOBE_WIDTH_PASSED_MAXIMUM                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 9757
#define RSSPECAN_ATTR_HRP_UWB_RESULT_CORRELATED_PULSE_MAIN_LOBE_MIN_WIDTH_SEL_PACKET                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 9758
#define RSSPECAN_ATTR_HRP_UWB_RESULT_CORRELATED_PULSE_MAIN_LOBE_MIN_WIDTH_AVERAGE                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 9759
#define RSSPECAN_ATTR_HRP_UWB_RESULT_CORRELATED_PULSE_MAIN_LOBE_MIN_WIDTH_MINIMUM                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 9760
#define RSSPECAN_ATTR_HRP_UWB_RESULT_CORRELATED_PULSE_MAIN_LOBE_MIN_WIDTH_MAXIMUM                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 9761
#define RSSPECAN_ATTR_HRP_UWB_RESULT_CORRELATED_PULSE_SIDE_LOBE_PEAK_LOCATION_SEL_PACKET                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 9762
#define RSSPECAN_ATTR_HRP_UWB_RESULT_CORRELATED_PULSE_SIDE_LOBE_PEAK_LOCATION_AVERAGE                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 9763
#define RSSPECAN_ATTR_HRP_UWB_RESULT_CORRELATED_PULSE_SIDE_LOBE_PEAK_LOCATION_MINIMUM                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 9764
#define RSSPECAN_ATTR_HRP_UWB_RESULT_CORRELATED_PULSE_SIDE_LOBE_PEAK_LOCATION_MAXIMUM                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 9765
#define RSSPECAN_ATTR_HRP_UWB_RESULT_CORRELATED_PULSE_SIDE_LOBE_PEAK_SEL_PACKET                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 9766
#define RSSPECAN_ATTR_HRP_UWB_RESULT_CORRELATED_PULSE_SIDE_LOBE_PEAK_AVERAGE                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 9767
#define RSSPECAN_ATTR_HRP_UWB_RESULT_CORRELATED_PULSE_SIDE_LOBE_PEAK_MINIMUM                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 9768
#define RSSPECAN_ATTR_HRP_UWB_RESULT_CORRELATED_PULSE_SIDE_LOBE_PEAK_MAXIMUM                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 9769
#define RSSPECAN_ATTR_HRP_UWB_RESULT_CORRELATED_PULSE_SIDE_LOBE_PEAK_PASSED_SEL_PACKET                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 9770
#define RSSPECAN_ATTR_HRP_UWB_RESULT_CORRELATED_PULSE_SIDE_LOBE_PEAK_PASSED_AVERAGE                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 9771
#define RSSPECAN_ATTR_HRP_UWB_RESULT_CORRELATED_PULSE_SIDE_LOBE_PEAK_PASSED_MINIMUM                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 9772
#define RSSPECAN_ATTR_HRP_UWB_RESULT_CORRELATED_PULSE_SIDE_LOBE_PEAK_PASSED_MAXIMUM                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 9773
#define RSSPECAN_ATTR_HRP_UWB_RESULT_PULSE_MASK_SEL_PACKET                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 9774
#define RSSPECAN_ATTR_HRP_UWB_RESULT_PULSE_MASK_AVERAGE                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 9775
#define RSSPECAN_ATTR_HRP_UWB_RESULT_PULSE_MASK_MINIMUM                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 9776
#define RSSPECAN_ATTR_HRP_UWB_RESULT_PULSE_MASK_MAXIMUM                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 9777
#define RSSPECAN_ATTR_HRP_UWB_RESULT_PULSE_RISE_MONOTONIC_SEL_PACKET                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 9778
#define RSSPECAN_ATTR_HRP_UWB_RESULT_PULSE_RISE_MONOTONIC_AVERAGE                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 9779
#define RSSPECAN_ATTR_HRP_UWB_RESULT_PULSE_RISE_MONOTONIC_MINIMUM                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 9780
#define RSSPECAN_ATTR_HRP_UWB_RESULT_PULSE_RISE_MONOTONIC_MAXIMUM                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 9781
#define RSSPECAN_ATTR_HRP_UWB_RESULT_PULSE_RISE_TIME_SEL_PACKET                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 11070
#define RSSPECAN_ATTR_HRP_UWB_RESULT_PULSE_RISE_TIME_AVERAGE                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 11071
#define RSSPECAN_ATTR_HRP_UWB_RESULT_PULSE_RISE_TIME_MINIMUM                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 11072
#define RSSPECAN_ATTR_HRP_UWB_RESULT_PULSE_RISE_TIME_MAXIMUM                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 11073
#define RSSPECAN_ATTR_HRP_UWB_RESULT_PULSE_RISE_TIME_PASSED_SEL_PACKET                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 11074
#define RSSPECAN_ATTR_HRP_UWB_RESULT_PULSE_RISE_TIME_PASSED_AVERAGE                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 11075
#define RSSPECAN_ATTR_HRP_UWB_RESULT_PULSE_RISE_TIME_PASSED_MINIMUM                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 11076
#define RSSPECAN_ATTR_HRP_UWB_RESULT_PULSE_RISE_TIME_PASSED_MAXIMUM                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 11077
#define RSSPECAN_ATTR_HRP_UWB_RESULT_PULSE_LOCATION_SFD_SEL_PACKET                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 11078
#define RSSPECAN_ATTR_HRP_UWB_RESULT_PULSE_LOCATION_SFD_AVERAGE                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 11079
#define RSSPECAN_ATTR_HRP_UWB_RESULT_PULSE_LOCATION_SFD_MINIMUM                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 11080
#define RSSPECAN_ATTR_HRP_UWB_RESULT_PULSE_LOCATION_SFD_MAXIMUM                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 11081
#define RSSPECAN_ATTR_HRP_UWB_RESULT_PULSE_LOCATION_STS_SEL_PACKET                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 11082
#define RSSPECAN_ATTR_HRP_UWB_RESULT_PULSE_LOCATION_STS_AVERAGE                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 11083
#define RSSPECAN_ATTR_HRP_UWB_RESULT_PULSE_LOCATION_STS_MINIMUM                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 11084
#define RSSPECAN_ATTR_HRP_UWB_RESULT_PULSE_LOCATION_STS_MAXIMUM                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 11085
#define RSSPECAN_ATTR_HRP_UWB_RESULT_PULSE_LOCATION_SYNC_SEL_PACKET                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 11086
#define RSSPECAN_ATTR_HRP_UWB_RESULT_PULSE_LOCATION_SYNC_AVERAGE                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 11087
#define RSSPECAN_ATTR_HRP_UWB_RESULT_PULSE_LOCATION_SYNC_MINIMUM                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 11088
#define RSSPECAN_ATTR_HRP_UWB_RESULT_PULSE_LOCATION_SYNC_MAXIMUM                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 11089
#define RSSPECAN_ATTR_HRP_UWB_RESULT_EVM_SHR_NRMSE_SEL_PACKET                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 9782
#define RSSPECAN_ATTR_HRP_UWB_RESULT_EVM_SHR_NRMSE_AVERAGE                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 9783
#define RSSPECAN_ATTR_HRP_UWB_RESULT_EVM_SHR_NRMSE_MINIMUM                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 9784
#define RSSPECAN_ATTR_HRP_UWB_RESULT_EVM_SHR_NRMSE_MAXIMUM                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 9785
#define RSSPECAN_ATTR_HRP_UWB_RESULT_EVM_PHR_NRMSE_SEL_PACKET                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 9786
#define RSSPECAN_ATTR_HRP_UWB_RESULT_EVM_PHR_NRMSE_AVERAGE                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 9787
#define RSSPECAN_ATTR_HRP_UWB_RESULT_EVM_PHR_NRMSE_MINIMUM                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 9788
#define RSSPECAN_ATTR_HRP_UWB_RESULT_EVM_PHR_NRMSE_MAXIMUM                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 9789
#define RSSPECAN_ATTR_HRP_UWB_RESULT_EVM_PSDU_NRMSE_SEL_PACKET                                               RS_SPECIFIC_PUBLIC_ATTR_BASE + 9790
#define RSSPECAN_ATTR_HRP_UWB_RESULT_EVM_PSDU_NRMSE_AVERAGE                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 9791
#define RSSPECAN_ATTR_HRP_UWB_RESULT_EVM_PSDU_NRMSE_MINIMUM                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 9792
#define RSSPECAN_ATTR_HRP_UWB_RESULT_EVM_PSDU_NRMSE_MAXIMUM                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 9793
#define RSSPECAN_ATTR_HRP_UWB_RESULT_EVM_STS_NRMSE_SEL_PACKET                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 9794
#define RSSPECAN_ATTR_HRP_UWB_RESULT_EVM_STS_NRMSE_AVERAGE                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 9795
#define RSSPECAN_ATTR_HRP_UWB_RESULT_EVM_STS_NRMSE_MINIMUM                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 9796
#define RSSPECAN_ATTR_HRP_UWB_RESULT_EVM_STS_NRMSE_MAXIMUM                                                   RS_SPECIFIC_PUBLIC_ATTR_BASE + 9797
#define RSSPECAN_ATTR_HRP_UWB_RESULT_EVM_PHR_PULSE_LEVEL_SEL_PACKET                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 9798
#define RSSPECAN_ATTR_HRP_UWB_RESULT_EVM_PHR_PULSE_LEVEL_AVERAGE                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 9799
#define RSSPECAN_ATTR_HRP_UWB_RESULT_EVM_PHR_PULSE_LEVEL_MINIMUM                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 9800
#define RSSPECAN_ATTR_HRP_UWB_RESULT_EVM_PHR_PULSE_LEVEL_MAXIMUM                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 9801
#define RSSPECAN_ATTR_HRP_UWB_RESULT_EVM_PSDU_PULSE_LEVEL_SEL_PACKET                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 9802
#define RSSPECAN_ATTR_HRP_UWB_RESULT_EVM_PSDU_PULSE_LEVEL_AVERAGE                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 9803
#define RSSPECAN_ATTR_HRP_UWB_RESULT_EVM_PSDU_PULSE_LEVEL_MINIMUM                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 9804
#define RSSPECAN_ATTR_HRP_UWB_RESULT_EVM_PSDU_PULSE_LEVEL_MAXIMUM                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 9805
#define RSSPECAN_ATTR_HRP_UWB_RESULT_EVM_STS_PULSE_LEVEL_SEL_PACKET                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 9806
#define RSSPECAN_ATTR_HRP_UWB_RESULT_EVM_STS_PULSE_LEVEL_AVERAGE                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 9807
#define RSSPECAN_ATTR_HRP_UWB_RESULT_EVM_STS_PULSE_LEVEL_MINIMUM                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 9808
#define RSSPECAN_ATTR_HRP_UWB_RESULT_EVM_STS_PULSE_LEVEL_MAXIMUM                                             RS_SPECIFIC_PUBLIC_ATTR_BASE + 9809
#define RSSPECAN_ATTR_HRP_UWB_RESULT_SHR_MEAN_SEL_PACKET                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 9810
#define RSSPECAN_ATTR_HRP_UWB_RESULT_SHR_MEAN_AVERAGE                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 9811
#define RSSPECAN_ATTR_HRP_UWB_RESULT_SHR_MEAN_MINIMUM                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 9812
#define RSSPECAN_ATTR_HRP_UWB_RESULT_SHR_MEAN_MAXIMUM                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 9813
#define RSSPECAN_ATTR_HRP_UWB_RESULT_SHR_PEAK_SEL_PACKET                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 9814
#define RSSPECAN_ATTR_HRP_UWB_RESULT_SHR_PEAK_AVERAGE                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 9815
#define RSSPECAN_ATTR_HRP_UWB_RESULT_SHR_PEAK_MINIMUM                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 9816
#define RSSPECAN_ATTR_HRP_UWB_RESULT_SHR_PEAK_MAXIMUM                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 9817
#define RSSPECAN_ATTR_HRP_UWB_RESULT_PSDU_MEAN_SEL_PACKET                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 9818
#define RSSPECAN_ATTR_HRP_UWB_RESULT_PSDU_MEAN_AVERAGE                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 9819
#define RSSPECAN_ATTR_HRP_UWB_RESULT_PSDU_MEAN_MINIMUM                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 9820
#define RSSPECAN_ATTR_HRP_UWB_RESULT_PSDU_MEAN_MAXIMUM                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 9821
#define RSSPECAN_ATTR_HRP_UWB_RESULT_PSDU_PEAK_SEL_PACKET                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 9822
#define RSSPECAN_ATTR_HRP_UWB_RESULT_PSDU_PEAK_AVERAGE                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 9823
#define RSSPECAN_ATTR_HRP_UWB_RESULT_PSDU_PEAK_MINIMUM                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 9824
#define RSSPECAN_ATTR_HRP_UWB_RESULT_PSDU_PEAK_MAXIMUM                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 9825
#define RSSPECAN_ATTR_HRP_UWB_RESULT_PACKET_MEAN_SEL_PACKET                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 11090
#define RSSPECAN_ATTR_HRP_UWB_RESULT_PACKET_MEAN_AVERAGE                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 11091
#define RSSPECAN_ATTR_HRP_UWB_RESULT_PACKET_MEAN_MINIMUM                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 11092
#define RSSPECAN_ATTR_HRP_UWB_RESULT_PACKET_MEAN_MAXIMUM                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 11093
#define RSSPECAN_ATTR_HRP_UWB_RESULT_PACKET_PEAK_SEL_PACKET                                                  RS_SPECIFIC_PUBLIC_ATTR_BASE + 11094
#define RSSPECAN_ATTR_HRP_UWB_RESULT_PACKET_PEAK_AVERAGE                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 11095
#define RSSPECAN_ATTR_HRP_UWB_RESULT_PACKET_PEAK_MINIMUM                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 11096
#define RSSPECAN_ATTR_HRP_UWB_RESULT_PACKET_PEAK_MAXIMUM                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 11097
#define RSSPECAN_ATTR_HRP_UWB_RESULT_IQ_OFFSET_SEL_PACKET                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 11098
#define RSSPECAN_ATTR_HRP_UWB_RESULT_IQ_OFFSET_AVERAGE                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 11099
#define RSSPECAN_ATTR_HRP_UWB_RESULT_IQ_OFFSET_MINIMUM                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 11100
#define RSSPECAN_ATTR_HRP_UWB_RESULT_IQ_OFFSET_MAXIMUM                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 11101
#define RSSPECAN_ATTR_HRP_UWB_RESULT_STS_MEAN_SEL_PACKET                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 11102
#define RSSPECAN_ATTR_HRP_UWB_RESULT_STS_MEAN_AVERAGE                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 11103
#define RSSPECAN_ATTR_HRP_UWB_RESULT_STS_MEAN_MINIMUM                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 11104
#define RSSPECAN_ATTR_HRP_UWB_RESULT_STS_MEAN_MAXIMUM                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 11105
#define RSSPECAN_ATTR_HRP_UWB_RESULT_STS_PEAK_SEL_PACKET                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 11106
#define RSSPECAN_ATTR_HRP_UWB_RESULT_STS_PEAK_AVERAGE                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 11107
#define RSSPECAN_ATTR_HRP_UWB_RESULT_STS_PEAK_MINIMUM                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 11108
#define RSSPECAN_ATTR_HRP_UWB_RESULT_STS_PEAK_MAXIMUM                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 11109
#define RSSPECAN_ATTR_HRP_UWB_RESULT_MAX_SPECTRAL_POWER_SEL_PACKET                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 9826
#define RSSPECAN_ATTR_HRP_UWB_RESULT_MAX_SPECTRAL_POWER_AVERAGE                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 9827
#define RSSPECAN_ATTR_HRP_UWB_RESULT_MAX_SPECTRAL_POWER_MINIMUM                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 9828
#define RSSPECAN_ATTR_HRP_UWB_RESULT_MAX_SPECTRAL_POWER_MAXIMUM                                              RS_SPECIFIC_PUBLIC_ATTR_BASE + 9829
#define RSSPECAN_ATTR_HRP_UWB_RESULT_SPECTRUM_MASK_PASSED_SEL_PACKET                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 9830
#define RSSPECAN_ATTR_HRP_UWB_RESULT_SPECTRUM_MASK_PASSED_AVERAGE                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 9831
#define RSSPECAN_ATTR_HRP_UWB_RESULT_SPECTRUM_MASK_PASSED_MINIMUM                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 9832
#define RSSPECAN_ATTR_HRP_UWB_RESULT_SPECTRUM_MASK_PASSED_MAXIMUM                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 9833
#define RSSPECAN_ATTR_HRP_UWB_RESULT_RANGING_MARKER                                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 9834
#define RSSPECAN_ATTR_HRP_UWB_RESULT_RANGING_SELECTED_MARKER                                                 RS_SPECIFIC_PUBLIC_ATTR_BASE + 11110
#define RSSPECAN_ATTR_HRP_UWB_MAGNITUDE_CAPTURE_START_ALL                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 11111
#define RSSPECAN_ATTR_HRP_UWB_MAGNITUDE_CAPTURE_STOP_ALL                                                     RS_SPECIFIC_PUBLIC_ATTR_BASE + 11112
#define RSSPECAN_ATTR_HRP_UWB_MAGNITUDE_CAPTURE_STATE_ALL                                                    RS_SPECIFIC_PUBLIC_ATTR_BASE + 11201
#define RSSPECAN_ATTR_HRP_UWB_MAGNITUDE_CAPTURE_START                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 11114
#define RSSPECAN_ATTR_HRP_UWB_MAGNITUDE_CAPTURE_STOP                                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 11115
#define RSSPECAN_ATTR_HRP_UWB_MAGNITUDE_CAPTURE_STATE                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 11116
#define RSSPECAN_ATTR_HRP_UWB_RESULT_DISPLAY_BINS                                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 11117
#define RSSPECAN_ATTR_HRP_UWB_RESULT_DISPLAY_X_SCALE                                                         RS_SPECIFIC_PUBLIC_ATTR_BASE + 11118
#define RSSPECAN_ATTR_HRP_UWB_RESULT_DISPLAY_CONFIG                                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 11119
#define RSSPECAN_ATTR_HRP_UWB_RESULT_DISPLAY_CONFIGURE                                                       RS_SPECIFIC_PUBLIC_ATTR_BASE + 11120
#define RSSPECAN_ATTR_HRP_UWB_RESULT_DISPLAY_GROUP                                                           RS_SPECIFIC_PUBLIC_ATTR_BASE + 11121
#define RSSPECAN_ATTR_HRP_UWB_RESULT_DISPLAY_LINK                                                            RS_SPECIFIC_PUBLIC_ATTR_BASE + 11122
#define RSSPECAN_ATTR_HRP_UWB_RESULT_DISPLAY_PACKET                                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 11123
#define RSSPECAN_ATTR_STATUS_PRESET                                                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 3267
#define RSSPECAN_ATTR_ID_QUERY_RESPONSE                                                                      RS_SPECIFIC_PUBLIC_ATTR_BASE + 2354
#define RSSPECAN_ATTR_ID_QUERY_FORMAT                                                                        RS_SPECIFIC_PUBLIC_ATTR_BASE + 10701
#define RSSPECAN_ATTR_SYSTEM_PRESET                                                                          RS_SPECIFIC_PUBLIC_ATTR_BASE + 5287
#define RSSPECAN_ATTR_LOGGING                                                                                RS_SPECIFIC_PUBLIC_ATTR_BASE + 6117

// rsspecan_rngUnits
#define RSSPECAN_VAL_UNIT_DBM                                               0
#define RSSPECAN_VAL_UNIT_DBPW                                              1
#define RSSPECAN_VAL_UNIT_WATT                                              2
#define RSSPECAN_VAL_UNIT_DBUV                                              3
#define RSSPECAN_VAL_UNIT_DBMV                                              4
#define RSSPECAN_VAL_UNIT_VOLT                                              5
#define RSSPECAN_VAL_UNIT_DBUA                                              6
#define RSSPECAN_VAL_UNIT_AMP                                               7
#define RSSPECAN_VAL_UNIT_DB                                                8
#define RSSPECAN_VAL_UNIT_DEG                                               9
#define RSSPECAN_VAL_UNIT_RAD                                               10
#define RSSPECAN_VAL_UNIT_S                                                 11
#define RSSPECAN_VAL_UNIT_HZ                                                12
#define RSSPECAN_VAL_UNIT_PCT                                               13
#define RSSPECAN_VAL_UNIT_UNITLESS                                          14
#define RSSPECAN_VAL_UNIT_DBMV_MHZ                                          16
#define RSSPECAN_VAL_UNIT_DBUV_MHZ                                          17
#define RSSPECAN_VAL_UNIT_DBUA_MHZ                                          18
#define RSSPECAN_VAL_UNIT_DBMV_M                                            19
#define RSSPECAN_VAL_UNIT_DBUA_M                                            20
#define RSSPECAN_VAL_UNIT_DBMV_MMHZ                                         21
#define RSSPECAN_VAL_UNIT_DBUA_MMHZ                                         22
#define RSSPECAN_VAL_UNIT_DBUV_M                                            23
#define RSSPECAN_VAL_UNIT_DBPW_MHZ                                          24
#define RSSPECAN_VAL_UNIT_DB_HZ                                             25
#define RSSPECAN_VAL_UNIT_DBC                                               26
#define RSSPECAN_VAL_UNIT_DBC_HZ                                            27
#define RSSPECAN_VAL_UNIT_DBM_HZ                                            28
#define RSSPECAN_VAL_UNIT_DBUV_HZ                                           29
#define RSSPECAN_VAL_UNIT_NV                                                30
#define RSSPECAN_VAL_UNIT_DBV                                               31
#define RSSPECAN_VAL_UNIT_DBV_HZ                                            32
#define RSSPECAN_VAL_UNIT_HZ_SQRT_HZ                                        33
#define RSSPECAN_VAL_UNIT_INV_SQRT_HZ                                       34
#define RSSPECAN_VAL_UNIT_MA                                                35
#define RSSPECAN_VAL_UNIT_V_SQRT_HZ                                         36

// rsspecan_rngTraceMode
#define RSSPECAN_TRAC_MOD_WRITE                                             0
#define RSSPECAN_TRAC_MOD_VIEW                                              1
#define RSSPECAN_TRAC_MOD_AVER                                              2
#define RSSPECAN_TRAC_MOD_MAXH                                              3
#define RSSPECAN_TRAC_MOD_MINH                                              4
#define RSSPECAN_TRAC_MOD_BLANK                                             5
#define RSSPECAN_TRAC_MOD_WRH                                               6
#define RSSPECAN_TRAC_MOD_DENS                                              7

// rsspecan_rngAmplitudeUnitsRangeTable
#define RSSPECAN_VAL_AMPLITUDE_UNITS_DBM                                    0
#define RSSPECAN_VAL_AMPLITUDE_UNITS_DBMV                                   1
#define RSSPECAN_VAL_AMPLITUDE_UNITS_DBUV                                   2
#define RSSPECAN_VAL_AMPLITUDE_UNITS_VOLT                                   3
#define RSSPECAN_VAL_AMPLITUDE_UNITS_WATT                                   4
#define RSSPECAN_VAL_AMPLITUDE_UNITS_DBPW                                   5
#define RSSPECAN_VAL_AMPLITUDE_UNITS_DBUA                                   6
#define RSSPECAN_VAL_AMPLITUDE_UNITS_A                                      7

// rsspecan_rngAbsRel
#define RSSPECAN_VAL_ABS                                                    0
#define RSSPECAN_VAL_REL                                                    1

// rsspecan_rngESPMode
#define RSSPECAN_VAL_ESP_MODE_AUTO                                          0
#define RSSPECAN_VAL_ESP_MODE_MAN                                           2

// rsspecan_rngDetectorTypeRangeTable
#define RSSPECAN_VAL_DETECTOR_TYPE_AUTO_PEAK                                0
#define RSSPECAN_VAL_DETECTOR_TYPE_AVERAGE                                  1
#define RSSPECAN_VAL_DETECTOR_TYPE_MAX_PEAK                                 2
#define RSSPECAN_VAL_DETECTOR_TYPE_MIN_PEAK                                 3
#define RSSPECAN_VAL_DETECTOR_TYPE_SAMPLE                                   4
#define RSSPECAN_VAL_DETECTOR_TYPE_RMS                                      5
#define RSSPECAN_VAL_DETECTOR_TYPE_QPK                                      6
#define RSSPECAN_VAL_DETECTOR_TYPE_CRMS                                     7
#define RSSPECAN_VAL_DETECTOR_TYPE_CAV                                      8

// rsspecan_rngCoupling
#define RSSPECAN_VAL_DC                                                     0
#define RSSPECAN_VAL_AC                                                     1

// rsspecan_rngListRangDet
#define RSSPECAN_VAL_LIST_RANG_DET_NEG                                      0
#define RSSPECAN_VAL_LIST_RANG_DET_POS                                      1
#define RSSPECAN_VAL_LIST_RANG_DET_SAMP                                     2
#define RSSPECAN_VAL_LIST_RANG_DET_RMS                                      3
#define RSSPECAN_VAL_LIST_RANG_DET_AVER                                     4
#define RSSPECAN_VAL_LIST_RANG_DET_APE                                      5

// rsspecan_rngDISPConfPreDefColour
#define RSSPECAN_VAL_DISP_COL_BLAC                                          0
#define RSSPECAN_VAL_DISP_COL_BLUE                                          1
#define RSSPECAN_VAL_DISP_COL_BROW                                          2
#define RSSPECAN_VAL_DISP_COL_GRE                                           3
#define RSSPECAN_VAL_DISP_COL_CYAN                                          4
#define RSSPECAN_VAL_DISP_COL_RED                                           5
#define RSSPECAN_VAL_DISP_COL_MAG                                           6
#define RSSPECAN_VAL_DISP_COL_YELL                                          7
#define RSSPECAN_VAL_DISP_COL_WHIT                                          8
#define RSSPECAN_VAL_DISP_COL_DGRA                                          9
#define RSSPECAN_VAL_DISP_COL_LGRA                                          10
#define RSSPECAN_VAL_DISP_COL_LBLU                                          11
#define RSSPECAN_VAL_DISP_COL_LGRE                                          12
#define RSSPECAN_VAL_DISP_COL_LCY                                           13
#define RSSPECAN_VAL_DISP_COL_LRED                                          14
#define RSSPECAN_VAL_DISP_COL_LMAG                                          15

// rsspecan_rngHcopyDeviceLang
#define RSSPECAN_VAL_HCOPY_DEVICE_LANG_GDI                                  0
#define RSSPECAN_VAL_HCOPY_DEVICE_LANG_WMF                                  1
#define RSSPECAN_VAL_HCOPY_DEVICE_LANG_EWMF                                 2
#define RSSPECAN_VAL_HCOPY_DEVICE_LANG_BMP                                  3
#define RSSPECAN_VAL_HCOPY_DEVICE_LANG_JPG                                  4
#define RSSPECAN_VAL_HCOPY_DEVICE_LANG_PNG                                  5
#define RSSPECAN_VAL_HCOPY_DEVICE_LANG_PDF                                  6
#define RSSPECAN_VAL_HCOPY_DEVICE_LANG_SVG                                  7
#define RSSPECAN_VAL_HCOPY_DEVICE_LANG_DOC                                  8
#define RSSPECAN_VAL_HCOPY_DEVICE_LANG_RTF                                  9

// rsspecan_rngHcopyDeviceOrient
#define RSSPECAN_VAL_HCOPY_DEVICE_ORIENT_LAND                               0
#define RSSPECAN_VAL_HCOPY_DEVICE_ORIENT_PORT                               1

// rsspecan_rngDISPFormat
#define RSSPECAN_VAL_DISP_FORMAT_SINGLE                                     0
#define RSSPECAN_VAL_DISP_FORMAT_SPLIT                                      1

// rsspecan_rngDomain
#define RSSPECAN_VAL_FREQ                                                   0
#define RSSPECAN_VAL_TIME                                                   1

// rsspecan_rngLinLog
#define RSSPECAN_VAL_LIN                                                    0
#define RSSPECAN_VAL_LOG                                                    1

// rsspecan_rngServiceInput
#define RSSPECAN_VAL_INPUT_RF                                               0
#define RSSPECAN_VAL_INPUT_CAL                                              1
#define RSSPECAN_VAL_INPUT_AIQ                                              2
#define RSSPECAN_VAL_INPUT_MCAL                                             3
#define RSSPECAN_VAL_INPUT_WBCAL                                            4
#define RSSPECAN_VAL_INPUT_WB2CAL                                           5
#define RSSPECAN_VAL_INPUT_SYNTH                                            6

// rsspecan_rngAvgType
#define RSSPECAN_VAL_AVG_LIN                                                0
#define RSSPECAN_VAL_AVG_VID                                                1
#define RSSPECAN_VAL_AVG_POW                                                2
#define RSSPECAN_VAL_AVG_LOG                                                3

// rsspecan_rngDISPSizeWindow
#define RSSPECAN_VAL_DISP_SIZE_LARGE                                        0
#define RSSPECAN_VAL_DISP_SIZE_SMALL                                        1

// rsspecan_rngBBInput
#define RSSPECAN_VAL_BB_INPUT_RF                                            0
#define RSSPECAN_VAL_BB_INPUT_AIQ                                           1
#define RSSPECAN_VAL_BB_INPUT_DIQ                                           2
#define RSSPECAN_VAL_BB_INPUT_FILE                                          3

// rsspecan_rngMarkerPeakListSort
#define RSSPECAN_VAL_MARKER_SORT_X                                          0
#define RSSPECAN_VAL_MARKER_SORT_Y                                          1

// rsspecan_rngMarkerDemodType
#define RSSPECAN_VAL_MARKER_DEMOD_AM                                        0
#define RSSPECAN_VAL_MARKER_DEMOD_FM                                        1
#define RSSPECAN_VAL_MARKER_DEMOD_PM                                        2
#define RSSPECAN_VAL_MARKER_DEMOD_AC                                        3
#define RSSPECAN_VAL_MARKER_DEMOD_AUD                                       4

// rsspecan_rngFFTFormat
#define RSSPECAN_VAL_FFT_MAGN                                               0
#define RSSPECAN_VAL_FFT_FREQ                                               1
#define RSSPECAN_VAL_FFT_VECTOR                                             2
#define RSSPECAN_VAL_FFT_RIM                                                3

// rsspecan_rngMeasPowerSelect
#define RSSPECAN_VAL_MEAS_POW_ACP                                           0
#define RSSPECAN_VAL_MEAS_POW_CPOW                                          1
#define RSSPECAN_VAL_MEAS_POW_MCAC                                          2
#define RSSPECAN_VAL_MEAS_POW_OBAN                                          3
#define RSSPECAN_VAL_MEAS_POW_CN                                            4
#define RSSPECAN_VAL_MEAS_POW_CN0                                           5
#define RSSPECAN_VAL_MEAS_POW_AOB                                           6
#define RSSPECAN_VAL_MEAS_POW_PPOW                                          7
#define RSSPECAN_VAL_MEAS_POW_GACL                                          8
#define RSSPECAN_VAL_MEAS_POW_MACM                                          9
#define RSSPECAN_VAL_MEAS_POW_COB                                           10

// rsspecan_rngMeasPowerStandard
#define RSSPECAN_VAL_MEAS_POW_STD_NONE                                      0
#define RSSPECAN_VAL_MEAS_POW_STD_NADC                                      1
#define RSSPECAN_VAL_MEAS_POW_STD_TETRA                                     2
#define RSSPECAN_VAL_MEAS_POW_STD_PDC                                       3
#define RSSPECAN_VAL_MEAS_POW_STD_PHS                                       4
#define RSSPECAN_VAL_MEAS_POW_STD_CDPD                                      5
#define RSSPECAN_VAL_MEAS_POW_STD_FIS95A                                    6
#define RSSPECAN_VAL_MEAS_POW_STD_RIS95A                                    7
#define RSSPECAN_VAL_MEAS_POW_STD_FIS95C0                                   8
#define RSSPECAN_VAL_MEAS_POW_STD_RIS95C0                                   9
#define RSSPECAN_VAL_MEAS_POW_STD_FJ008                                     10
#define RSSPECAN_VAL_MEAS_POW_STD_RJ008                                     11
#define RSSPECAN_VAL_MEAS_POW_STD_FIS95C1                                   12
#define RSSPECAN_VAL_MEAS_POW_STD_RIS95C1                                   13
#define RSSPECAN_VAL_MEAS_POW_STD_FWCD                                      14
#define RSSPECAN_VAL_MEAS_POW_STD_RWCD                                      15
#define RSSPECAN_VAL_MEAS_POW_STD_FW3G                                      16
#define RSSPECAN_VAL_MEAS_POW_STD_RW3G                                      17
#define RSSPECAN_VAL_MEAS_POW_STD_D2CD                                      18
#define RSSPECAN_VAL_MEAS_POW_STD_S2CD                                      19
#define RSSPECAN_VAL_MEAS_POW_STD_M2CD                                      20
#define RSSPECAN_VAL_MEAS_POW_STD_TCDM                                      21
#define RSSPECAN_VAL_MEAS_POW_STD_AWL                                       22
#define RSSPECAN_VAL_MEAS_POW_STD_BWL                                       23
#define RSSPECAN_VAL_MEAS_POW_STD_WIM                                       24
#define RSSPECAN_VAL_MEAS_POW_STD_WIBRO                                     25
#define RSSPECAN_VAL_MEAS_POW_STD_EUTR                                      29
#define RSSPECAN_VAL_MEAS_POW_STD_REUT                                      30
#define RSSPECAN_VAL_MEAS_POW_STD_PAPC                                      31
#define RSSPECAN_VAL_MEAS_POW_STD_MSR                                       32
#define RSSPECAN_VAL_MEAS_POW_STD_FTCD                                      33
#define RSSPECAN_VAL_MEAS_POW_STD_TRCD                                      34
#define RSSPECAN_VAL_MEAS_POW_STD_GSM                                       35
#define RSSPECAN_VAL_MEAS_POW_STD_RFID14443                                 36
#define RSSPECAN_VAL_MEAS_POW_STD_USER                                      37

// rsspecan_rngMeasPowerMode
#define RSSPECAN_VAL_MEAS_POW_MODE_WRITE                                    0
#define RSSPECAN_VAL_MEAS_POW_MODE_MAXH                                     1

// rsspecan_rngMeasStatUnit
#define RSSPECAN_VAL_PCT                                                    1

// rsspecan_rngFFTFilterMode
#define RSSPECAN_VAL_FFT_MODE_AUTO                                          0
#define RSSPECAN_VAL_FFT_MODE_WIDE                                          1
#define RSSPECAN_VAL_FFT_MODE_NARROW                                        2

// rsspecan_rngBandFiltType
#define RSSPECAN_VAL_FILT_TYPE_NORM                                         0
#define RSSPECAN_VAL_FILT_TYPE_FFT                                          1
#define RSSPECAN_VAL_FILT_TYPE_CFIL                                         2
#define RSSPECAN_VAL_FILT_TYPE_RRC                                          3
#define RSSPECAN_VAL_FILT_TYPE_P5                                           4
#define RSSPECAN_VAL_FILT_TYPE_PULSE                                        5

// rsspecan_rngVbwFiltType
#define RSSPECAN_VAL_VBW_FILT_LIN                                           0
#define RSSPECAN_VAL_VBW_FILT_LOG                                           1

// rsspecan_rngCorrMethod
#define RSSPECAN_VAL_CORR_TRAN                                              0
#define RSSPECAN_VAL_CORR_REFL                                              1

// rsspecan_rngCorrAcquire
#define RSSPECAN_VAL_CORR_ACQ_THR                                           0
#define RSSPECAN_VAL_CORR_ACQ_OPEN                                          1

// rsspecan_rngTFacUnit
#define RSSPECAN_VAL_TFAC_UNIT_DB                                           0
#define RSSPECAN_VAL_TFAC_UNIT_DBM                                          1
#define RSSPECAN_VAL_TFAC_UNIT_DBMV                                         2
#define RSSPECAN_VAL_TFAC_UNIT_DBUV                                         3
#define RSSPECAN_VAL_TFAC_UNIT_DBUVM                                        4
#define RSSPECAN_VAL_TFAC_UNIT_DBUA                                         5
#define RSSPECAN_VAL_TFAC_UNIT_DBUAM                                        6
#define RSSPECAN_VAL_TFAC_UNIT_DBPW                                         7
#define RSSPECAN_VAL_TFAC_UNIT_DBPT                                         8

// rsspecan_rngCvlBand
#define RSSPECAN_VAL_CVL_BAND_A                                             0
#define RSSPECAN_VAL_CVL_BAND_Q                                             1
#define RSSPECAN_VAL_CVL_BAND_U                                             2
#define RSSPECAN_VAL_CVL_BAND_V                                             3
#define RSSPECAN_VAL_CVL_BAND_E                                             4
#define RSSPECAN_VAL_CVL_BAND_W                                             5
#define RSSPECAN_VAL_CVL_BAND_F                                             6
#define RSSPECAN_VAL_CVL_BAND_D                                             7
#define RSSPECAN_VAL_CVL_BAND_G                                             8
#define RSSPECAN_VAL_CVL_BAND_Y                                             9
#define RSSPECAN_VAL_CVL_BAND_J                                             10
#define RSSPECAN_VAL_CVL_BAND_USER                                          11
#define RSSPECAN_VAL_CVL_BAND_K                                             12
#define RSSPECAN_VAL_CVL_BAND_KA                                            13

// rsspecan_rngFreqCentLink
#define RSSPECAN_VAL_CENT_FREQ_LINK_OFF                                     0
#define RSSPECAN_VAL_CENT_FREQ_LINK_SPAN                                    1
#define RSSPECAN_VAL_CENT_FREQ_LINK_RBW                                     2
#define RSSPECAN_VAL_CENT_FREQ_LINK_DIVT                                    3

// rsspecan_rngFreqMode
#define RSSPECAN_VAL_FREQ_MODE_FIX                                          1
#define RSSPECAN_VAL_FREQ_MODE_SWE                                          2

// rsspecan_rngListFiltType
#define RSSPECAN_VAL_LIST_FTYPE_NORM                                        0
#define RSSPECAN_VAL_LIST_FTYPE_CHAN                                        1
#define RSSPECAN_VAL_LIST_FTYPE_RRC                                         2
#define RSSPECAN_VAL_LIST_FTYPE_P5                                          3
#define RSSPECAN_VAL_LIST_FTYPE_PULS                                        4
#define RSSPECAN_VAL_LIST_FTYPE_MIL                                         5

// rsspecan_rngOffOnAuto
#define RSSPECAN_VAL_STATE_OFF                                              0
#define RSSPECAN_VAL_STATE_ON                                               1
#define RSSPECAN_VAL_STATE_AUTO                                             2
#define RSSPECAN_VAL_STATE_ALL                                              3

// rsspecan_rngExtMixerHarmonicType
#define RSSPECAN_VAL_EXT_MIX_HARM_ODD                                       0
#define RSSPECAN_VAL_EXT_MIX_HARM_EVEN                                      1
#define RSSPECAN_VAL_EXT_MIX_HARM_EODD                                      2

// rsspecan_rngExtMixerHarmBand
#define RSSPECAN_VAL_EXT_MIX_BAND_A                                         0
#define RSSPECAN_VAL_EXT_MIX_BAND_Q                                         1
#define RSSPECAN_VAL_EXT_MIX_BAND_U                                         2
#define RSSPECAN_VAL_EXT_MIX_BAND_V                                         3
#define RSSPECAN_VAL_EXT_MIX_BAND_E                                         4
#define RSSPECAN_VAL_EXT_MIX_BAND_W                                         5
#define RSSPECAN_VAL_EXT_MIX_BAND_F                                         6
#define RSSPECAN_VAL_EXT_MIX_BAND_D                                         7
#define RSSPECAN_VAL_EXT_MIX_BAND_G                                         8
#define RSSPECAN_VAL_EXT_MIX_BAND_Y                                         9
#define RSSPECAN_VAL_EXT_MIX_BAND_J                                         10
#define RSSPECAN_VAL_EXT_MIX_BAND_USER                                      11
#define RSSPECAN_VAL_EXT_MIX_BAND_K                                         12
#define RSSPECAN_VAL_EXT_MIX_BAND_KA                                        13

// rsspecan_rngAdjChAutoSel
#define RSSPECAN_VAL_ADJ_RCHAN_AUTO_MIN                                     0
#define RSSPECAN_VAL_ADJ_RCHAN_AUTO_MAX                                     1
#define RSSPECAN_VAL_ADJ_RCHAN_AUTO_LHIG                                    2

// rsspecan_rngAdjPreset
#define RSSPECAN_VAL_ADJ_PRE_ACP                                            0
#define RSSPECAN_VAL_ADJ_PRE_CPOW                                           1
#define RSSPECAN_VAL_ADJ_PRE_MCAC                                           2
#define RSSPECAN_VAL_ADJ_PRE_OBAN                                           3
#define RSSPECAN_VAL_ADJ_PRE_CN                                             4
#define RSSPECAN_VAL_ADJ_PRE_CN0                                            5

// rsspecan_rngSourceIntExt
#define RSSPECAN_VAL_SOUR_INT                                               0
#define RSSPECAN_VAL_SOUR_EXT                                               1
#define RSSPECAN_VAL_SOUR_EAUT                                              2
#define RSSPECAN_VAL_SOUR_E10                                               3
#define RSSPECAN_VAL_SOUR_E100                                              4
#define RSSPECAN_VAL_SOUR_SYNC                                              5
#define RSSPECAN_VAL_SOUR_E1000                                             6

// rsspecan_rngTgenPowMode
#define RSSPECAN_VAL_TGEN_POW_FIX                                           0
#define RSSPECAN_VAL_TGEN_POW_SWE                                           1

// rsspecan_rngExtGenIntfType
#define RSSPECAN_VAL_EXT_GEN_INTF_GPIB                                      0
#define RSSPECAN_VAL_EXT_GEN_INTF_TTL                                       1

// rsspecan_rngExtGateTrigType
#define RSSPECAN_VAL_EGAT_TRIG_LEV                                          0
#define RSSPECAN_VAL_EGAT_TRIG_EDGE                                         1
#define RSSPECAN_VAL_EGAT_TRIG_OFF                                          2

// rsspecan_rngHcopyDevice
#define RSSPECAN_VAL_HCOPY_DEVICE_MEM                                       0
#define RSSPECAN_VAL_HCOPY_DEVICE_PRN                                       1
#define RSSPECAN_VAL_HCOPY_DEVICE_CLP                                       2

// rsspecan_rngPolarity
#define RSSPECAN_VAL_NEG                                                    0
#define RSSPECAN_VAL_POS                                                    1

// rsspecan_rngExtGateSource
#define RSSPECAN_VAL_EGAT_SOUR_EXT                                          0
#define RSSPECAN_VAL_EGAT_SOUR_IFP                                          1
#define RSSPECAN_VAL_EGAT_SOUR_VID                                          2
#define RSSPECAN_VAL_EGAT_SOUR_RFP                                          3
#define RSSPECAN_VAL_EGAT_SOUR_PSE                                          4
#define RSSPECAN_VAL_EGAT_SOUR_EXT2                                         5
#define RSSPECAN_VAL_EGAT_SOUR_EXT3                                         6
#define RSSPECAN_VAL_EGAT_SOUR_IQP                                          7
#define RSSPECAN_VAL_EGAT_SOUR_EXT4                                         8

// rsspecan_rngSweepContMode
#define RSSPECAN_VAL_SWE_MODE_AUTO                                          0
#define RSSPECAN_VAL_SWE_MODE_LIST                                          1
#define RSSPECAN_VAL_SWE_MODE_ESP                                           2

// rsspecan_rngTraceFormat
#define RSSPECAN_VAL_TRACE_FORM_COMP                                        0
#define RSSPECAN_VAL_TRACE_FORM_IQBL                                        1
#define RSSPECAN_VAL_TRACE_FORM_IQP                                         2

// rsspecan_rngTriggerSource
#define RSSPECAN_VAL_TRG_IMM                                                0
#define RSSPECAN_VAL_TRG_EXT                                                1
#define RSSPECAN_VAL_TRG_IFP                                                2
#define RSSPECAN_VAL_TRG_VID                                                3
#define RSSPECAN_VAL_TRG_TIME                                               4
#define RSSPECAN_VAL_TRG_RFP                                                5
#define RSSPECAN_VAL_TRG_PSEN                                               6
#define RSSPECAN_VAL_TRG_TDTR                                               7
#define RSSPECAN_VAL_TRG_BB_PWR                                             8
#define RSSPECAN_VAL_TRG_EXT2                                               9
#define RSSPECAN_VAL_TRG_EXT3                                               10
#define RSSPECAN_VAL_TRG_IQP                                                11
#define RSSPECAN_VAL_TRG_GP0                                                12
#define RSSPECAN_VAL_TRG_GP1                                                13
#define RSSPECAN_VAL_TRG_GP2                                                14
#define RSSPECAN_VAL_TRG_GP3                                                15
#define RSSPECAN_VAL_TRG_GP4                                                16
#define RSSPECAN_VAL_TRG_GP5                                                17
#define RSSPECAN_VAL_TRG_TUN                                                18
#define RSSPECAN_VAL_TRG_MASK                                               19
#define RSSPECAN_VAL_TRG_EXT4                                               20
#define RSSPECAN_VAL_TRG_FREQ                                               21

// rsspecan_rngFileDecSeparator
#define RSSPECAN_VAL_DEC_SEP_POIN                                           0
#define RSSPECAN_VAL_DEC_SEP_COMMA                                          1

// rsspecan_rngVSAConfDisplayTraceSpacing
#define RSSPECAN_VAL_TRACE_SPACE_LIN                                        0
#define RSSPECAN_VAL_TRACE_SPACE_LOG                                        1
#define RSSPECAN_VAL_TRACE_SPACE_PCT                                        2
#define RSSPECAN_VAL_TRACE_SPACE_LDB                                        3

// rsspecan_rngMarkerStepSize
#define RSSPECAN_VAL_MARK_STEP_SIZE_STAN                                    0
#define RSSPECAN_VAL_MARK_STEP_SIZE_POIN                                    1

// rsspecan_rngMpowFiltType
#define RSSPECAN_VAL_MPOW_FTYPE_NORM                                        0
#define RSSPECAN_VAL_MPOW_FTYPE_CFIL                                        1
#define RSSPECAN_VAL_MPOW_FTYPE_RRC                                         2
#define RSSPECAN_VAL_MPOW_FTYPE_P5                                          3
#define RSSPECAN_VAL_MPOW_FTYPE_PULSE                                       4
#define RSSPECAN_VAL_MPOW_FTYPE_MIL                                         5

// rsspecan_rngSpemColor
#define RSSPECAN_VAL_SPEM_COL_COLOR                                         0
#define RSSPECAN_VAL_SPEM_COL_RADAR                                         1
#define RSSPECAN_VAL_SPEM_COL_GRAY                                          2

// rsspecan_rngSpemSearchArea
#define RSSPECAN_VAL_SPEM_SAR_MEMORY                                        1
#define RSSPECAN_VAL_SPEM_SAR_VISIBLE                                       0

// rsspecan_rngBBTriggerSource
#define RSSPECAN_VAL_TRG_BBP                                                5

// rsspecan_rngSEMFilterType
#define RSSPECAN_VAL_SEM_FILTER_TYPE_NORM                                   0
#define RSSPECAN_VAL_SEM_FILTER_TYPE_CFIL                                   1
#define RSSPECAN_VAL_SEM_FILTER_TYPE_RRC                                    2
#define RSSPECAN_VAL_SEM_FILTER_TYPE_PULS                                   3
#define RSSPECAN_VAL_SEM_FILTER_TYPE_P5                                     4
#define RSSPECAN_VAL_SEM_FILTER_TYPE_MIL                                    6

// rsspecan_rngSEMInsertRange
#define RSSPECAN_VAL_SEM_INSERT_RANGE_AFTER                                 0
#define RSSPECAN_VAL_SEM_INSERT_RANGE_BEFORE                                1

// rsspecan_rngSEMRangeLimitCheckType
#define RSSPECAN_VAL_SEM_LIM_CHECK_TYPE_ABS                                 0
#define RSSPECAN_VAL_SEM_LIM_CHECK_TYPE_REL                                 1
#define RSSPECAN_VAL_SEM_LIM_CHECK_TYPE_AND                                 2
#define RSSPECAN_VAL_SEM_LIM_CHECK_TYPE_OR                                  3

// rsspecan_rngSEMPowerRefType
#define RSSPECAN_VAL_SEM_REF_TYPE_PEAK                                      0
#define RSSPECAN_VAL_SEM_REF_TYPE_CPOW                                      1

// rsspecan_rngIFOutputSource
#define RSSPECAN_VAL_IF_OUTPUT_SOURCE_IF                                    0
#define RSSPECAN_VAL_IF_OUTPUT_SOURCE_VID                                   1
#define RSSPECAN_VAL_IF_OUTPUT_SOURCE_IF2                                   3

// rsspecan_rngTouchscreenDisplay
#define RSSPECAN_VAL_TOUCH_OFF                                              0
#define RSSPECAN_VAL_TOUCH_ON                                               1
#define RSSPECAN_VAL_TOUCH_FRAM                                             2

// rsspecan_rngAutoMan
#define RSSPECAN_VAL_AUTO                                                   0
#define RSSPECAN_VAL_MAN                                                    1

// rsspecan_rngSweepType
#define RSSPECAN_VAL_SWEEP_TYPE_AUTO                                        0
#define RSSPECAN_VAL_SWEEP_TYPE_SWEEP                                       1
#define RSSPECAN_VAL_SWEEP_TYPE_FFT                                         2

// rsspecan_rngAmplAttenMode
#define RSSPECAN_VAL_ATT_MODE_NORM                                          0
#define RSSPECAN_VAL_ATT_MODE_LDIS                                          1
#define RSSPECAN_VAL_ATT_MODE_LNO                                           2

// rsspecan_rngIQDataGateType
#define RSSPECAN_VAL_GATE_TYPE_LEVEL                                        0
#define RSSPECAN_VAL_GATE_TYPE_EDGE                                         1

// rsspecan_rngIQDataDisplayType
#define RSSPECAN_VAL_IQ_DATA_TYPE_RIMAG                                     0
#define RSSPECAN_VAL_IQ_DATA_TYPE_MAGN                                      1
#define RSSPECAN_VAL_IQ_DATA_TYPE_FREQ                                      2
#define RSSPECAN_VAL_IQ_DATA_TYPE_VECT                                      3

// rsspecan_rngBandPowerMode
#define RSSPECAN_VAL_BPOWER_POWER                                           0
#define RSSPECAN_VAL_BPOWER_DENSITY                                         1
#define RSSPECAN_VAL_BPOWER_REL_POWER                                       2

// rsspecan_rngInputUnit
#define RSSPECAN_VAL_DINPUT_UNIT_V                                          0
#define RSSPECAN_VAL_DINPUT_UNIT_DBM                                        1
#define RSSPECAN_VAL_DINPUT_UNIT_DBPV                                       2
#define RSSPECAN_VAL_DINPUT_UNIT_W                                          3
#define RSSPECAN_VAL_DINPUT_UNIT_DBMV                                       4
#define RSSPECAN_VAL_DINPUT_UNIT_DBUV                                       5
#define RSSPECAN_VAL_DINPUT_UNIT_A                                          6
#define RSSPECAN_VAL_DINPUT_UNIT_DBUA                                       7

// rsspecan_rngSpecColorStyle
#define RSSPECAN_VAL_SPEM_HOT                                               0
#define RSSPECAN_VAL_SPEM_COLD                                              1
#define RSSPECAN_VAL_SPEM_COL                                               2
#define RSSPECAN_VAL_SPEM_RAD                                               3
#define RSSPECAN_VAL_SPEM_GRAY                                              4

// rsspecan_rngPersisColorStyle
#define RSSPECAN_VAL_COL_HOT                                                0
#define RSSPECAN_VAL_COL_COLD                                               1
#define RSSPECAN_VAL_COL_RAD                                                2
#define RSSPECAN_VAL_COL_GRAY                                               3

// rsspecan_rngPersisTraceSymb
#define RSSPECAN_VAL_TRA_DOTS                                               0
#define RSSPECAN_VAL_TRA_VECT                                               1

// rsspecan_rngSpecMaskCondition
#define RSSPECAN_VAL_MASK_ENT                                               0
#define RSSPECAN_VAL_MASK_LEAV                                              1
#define RSSPECAN_VAL_MASK_INS                                               2
#define RSSPECAN_VAL_MASK_OUTS                                              3

// rsspecan_rngMaskMode
#define RSSPECAN_VAL_MASK_ABS                                               0
#define RSSPECAN_VAL_MASK_REL                                               1

// rsspecan_rngResultDisplay
#define RSSPECAN_VAL_PSPEM                                                  0
#define RSSPECAN_VAL_SPEM                                                   1
#define RSSPECAN_VAL_REAL_SPEM                                              2
#define RSSPECAN_VAL_POWER_TIME                                             3
#define RSSPECAN_VAL_PVT_WATER                                              4

// rsspecan_rngTriggerDirection
#define RSSPECAN_VAL_TRIG_DIRECTION_INP                                     0
#define RSSPECAN_VAL_TRIG_DIRECTION_OUTP                                    1

// rsspecan_rngTriggerOType
#define RSSPECAN_VAL_TRIG_OTYPE_DEVICE                                      0
#define RSSPECAN_VAL_TRIG_OTYPE_TARMED                                      1
#define RSSPECAN_VAL_TRIG_OTYPE_UDEFINED                                    2

// rsspecan_rngTriggerLevel
#define RSSPECAN_VAL_TRIG_LEVEL_HIGH                                        0
#define RSSPECAN_VAL_TRIG_LEVEL_LOW                                         1

// rsspecan_rngPreampLevel
#define RSSPECAN_VAL_PREAMPLIFICATION_LEVEL_15DB                            0
#define RSSPECAN_VAL_PREAMPLIFICATION_LEVEL_30DB                            1

// rsspecan_rngPowerNoiseCorrection
#define RSSPECAN_VAL_MEAS_POW_NOISE_CORR_OFF                                0
#define RSSPECAN_VAL_MEAS_POW_NOISE_CORR_ON                                 1
#define RSSPECAN_VAL_MEAS_POW_NOISE_CORR_AUTO                               2

// rsspecan_rngProbeConnectionState
#define RSSPECAN_VAL_PROBE_CONN_STATE_DET                                   1
#define RSSPECAN_VAL_PROBE_CONN_STATE_NDET                                  0

// rsspecan_rngSweepOptimization
#define RSSPECAN_VAL_SWEEP_OPTIMIZATION_AUTO                                0
#define RSSPECAN_VAL_SWEEP_OPTIMIZATION_DYNAMIC                             1
#define RSSPECAN_VAL_SWEEP_OPTIMIZATION_SPEED                               2

// rsspecan_rngDataSetLoadMode
#define RSSPECAN_VAL_MEM_LOAD_MODE_NEW                                      0
#define RSSPECAN_VAL_MEM_LOAD_MODE_REPLACE                                  1

// rsspecan_rngDataSetStoreMode
#define RSSPECAN_VAL_MEM_STORE_MODE_INSTRUMENT                              0
#define RSSPECAN_VAL_MEM_STORE_MODE_CHANNEL                                 1

// rsspecan_rngSequencerMode
#define RSSPECAN_VAL_SEQUENCER_MODE_SING                                    0
#define RSSPECAN_VAL_SEQUENCER_MODE_CONT                                    1

// rsspecan_rngContinuousTrigger
#define RSSPECAN_VAL_TRIGGER_CONTINUOUS                                     0
#define RSSPECAN_VAL_TRIGGER_STOP                                           1

// rsspecan_rngFFTWinType
#define RSSPECAN_VAL_SPEM_FFT_WINDOW_BLACKHARRIS                            0
#define RSSPECAN_VAL_SPEM_FFT_WINDOW_FLATTOP                                1
#define RSSPECAN_VAL_SPEM_FFT_WINDOW_GAUSSIAN                               2
#define RSSPECAN_VAL_SPEM_FFT_WINDOW_HANNING                                3
#define RSSPECAN_VAL_SPEM_FFT_WINDOW_KAISERBESSER                           4
#define RSSPECAN_VAL_SPEM_FFT_WINDOW_RECTANGULAR                            5
#define RSSPECAN_VAL_SPEM_FFT_WINDOW_HAMMING                                6

// rsspecan_rngMarkerPosition
#define RSSPECAN_VAL_MARKER_POSITION_MAXHOLD                                0
#define RSSPECAN_VAL_MARKER_POSITION_PERSISTENCE                            1
#define RSSPECAN_VAL_MARKER_POSITION_TRACE_1                                2
#define RSSPECAN_VAL_MARKER_POSITION_TRACE_2                                3
#define RSSPECAN_VAL_MARKER_POSITION_TRACE_3                                4
#define RSSPECAN_VAL_MARKER_POSITION_TRACE_4                                5

// rsspecan_rngEMIFmeasDetectorType
#define RSSPECAN_VAL_FMEAS_DETECTOR_TYPE_OFF                                0
#define RSSPECAN_VAL_FMEAS_DETECTOR_TYPE_AVER                               1
#define RSSPECAN_VAL_FMEAS_DETECTOR_TYPE_CAV                                2
#define RSSPECAN_VAL_FMEAS_DETECTOR_TYPE_CRMS                               3
#define RSSPECAN_VAL_FMEAS_DETECTOR_TYPE_POS                                4
#define RSSPECAN_VAL_FMEAS_DETECTOR_TYPE_QPE                                5

// rsspecan_rngEMILISNVnetworkType
#define RSSPECAN_VAL_EMI_VNET_TYPE_OFF                                      0
#define RSSPECAN_VAL_EMI_VNET_TYPE_ESH3Z5                                   1
#define RSSPECAN_VAL_EMI_VNET_TYPE_ESH2Z5                                   2
#define RSSPECAN_VAL_EMI_VNET_TYPE_4200                                     3
#define RSSPECAN_VAL_EMI_VNET_TYPE_216                                      4
#define RSSPECAN_VAL_EMI_VNET_TYPE_TWO                                      5
#define RSSPECAN_VAL_EMI_VNET_TYPE_FOUR                                     6
#define RSSPECAN_VAL_EMI_VNET_TYPE_432                                      7

// rsspecan_rngEMILISNVNetworkPhase
#define RSSPECAN_VAL_EMI_VNET_PHAS_N                                        0
#define RSSPECAN_VAL_EMI_VNET_PHAS_L1                                       1
#define RSSPECAN_VAL_EMI_VNET_PHAS_L2                                       2
#define RSSPECAN_VAL_EMI_VNET_PHAS_L3                                       3

// rsspecan_rngEMIBandwidthFilterType
#define RSSPECAN_VAL_EMI_FILT_TYPE_NORM                                     0
#define RSSPECAN_VAL_EMI_FILT_TYPE_FFT                                      1
#define RSSPECAN_VAL_EMI_FILT_TYPE_CFIL                                     2
#define RSSPECAN_VAL_EMI_FILT_TYPE_RRC                                      3
#define RSSPECAN_VAL_EMI_FILT_TYPE_P5                                       4
#define RSSPECAN_VAL_EMI_FILT_TYPE_CISPR                                    5
#define RSSPECAN_VAL_EMI_FILT_TYPE_MIL                                      6

// rsspecan_rngProbeMicroButton
#define RSSPECAN_VAL_PROBE_MICRO_BUTTON_RUN_SINGLE                          0
#define RSSPECAN_VAL_PROBE_MICRO_BUTTON_NO_ACTION                           1

// rsspecan_rngSpectrumEmissionMaskMSRBandCategory
#define RSSPECAN_VAL_SEM_MSR_BAND_CATEGORY_1                                1
#define RSSPECAN_VAL_SEM_MSR_BAND_CATEGORY_2                                2
#define RSSPECAN_VAL_SEM_MSR_BAND_CATEGORY_3                                3

// rsspecan_rngSpectrumEmissionMaskRangeStartFrequencyRelativeLimitFunction
#define RSSPECAN_VAL_SEM_REL_LIMIT_FUNC_OFF                                 0
#define RSSPECAN_VAL_SEM_REL_LIMIT_FUNC_MAX                                 1

// rsspecan_rngAssignMarkertoTraceRaisingFalling
#define RSSPECAN_VAL_MARKER_TRACE_OFF1                                      0
#define RSSPECAN_VAL_MARKER_TRACE_OFF2                                      1
#define RSSPECAN_VAL_MARKER_TRACE_FALLING1                                  2
#define RSSPECAN_VAL_MARKER_TRACE_FALLING2                                  3
#define RSSPECAN_VAL_MARKER_TRACE_RAISING1                                  4
#define RSSPECAN_VAL_MARKER_TRACE_RAISING2                                  5

// rsspecan_rngPresetOperatingMode
#define RSSPECAN_VAL_PRESET_MSRA                                            0
#define RSSPECAN_VAL_PRESET_SAN                                             1
#define RSSPECAN_VAL_PRESET_RTMS                                            2
#define RSSPECAN_VAL_PRESET_PNO                                             3

// rsspecan_rngLayoutQueryWindowType
#define RSSPECAN_VAL_LAYOUT_TYPE_DIAGRAM                                    0
#define RSSPECAN_VAL_LAYOUT_TYPE_MTABLE                                     1
#define RSSPECAN_VAL_LAYOUT_TYPE_PEAKLIST                                   2
#define RSSPECAN_VAL_LAYOUT_TYPE_RSUMMARY                                   3
#define RSSPECAN_VAL_LAYOUT_TYPE_SGRAM                                      4
#define RSSPECAN_VAL_LAYOUT_TYPE_MAGN                                       5
#define RSSPECAN_VAL_LAYOUT_TYPE_FREQ                                       6
#define RSSPECAN_VAL_LAYOUT_TYPE_VECT                                       7
#define RSSPECAN_VAL_LAYOUT_TYPE_RIMAG                                      8
#define RSSPECAN_VAL_LAYOUT_TYPE_MCAP                                       9
#define RSSPECAN_VAL_LAYOUT_TYPE_PRES                                       10
#define RSSPECAN_VAL_LAYOUT_TYPE_PST                                        11
#define RSSPECAN_VAL_LAYOUT_TYPE_PMAG                                       12
#define RSSPECAN_VAL_LAYOUT_TYPE_PPH                                        13
#define RSSPECAN_VAL_LAYOUT_TYPE_PFR                                        14
#define RSSPECAN_VAL_LAYOUT_TYPE_PTR                                        15
#define RSSPECAN_VAL_LAYOUT_TYPE_NOISE                                      16
#define RSSPECAN_VAL_LAYOUT_TYPE_GAIN                                       17
#define RSSPECAN_VAL_LAYOUT_TYPE_TEMP                                       18
#define RSSPECAN_VAL_LAYOUT_TYPE_YFAC                                       19
#define RSSPECAN_VAL_LAYOUT_TYPE_PHOT                                       20
#define RSSPECAN_VAL_LAYOUT_TYPE_PCOLD                                      21
#define RSSPECAN_VAL_LAYOUT_TYPE_RTAB                                       22
#define RSSPECAN_VAL_LAYOUT_TYPE_PNOISE                                     23
#define RSSPECAN_VAL_LAYOUT_TYPE_RNOISE                                     24
#define RSSPECAN_VAL_LAYOUT_TYPE_SNOISE                                     25
#define RSSPECAN_VAL_LAYOUT_TYPE_SPURS                                      26
#define RSSPECAN_VAL_LAYOUT_TYPE_GDEL                                       27
#define RSSPECAN_VAL_LAYOUT_TYPE_PHASE                                      28
#define RSSPECAN_VAL_LAYOUT_TYPE_REF_MAGN                                   29
#define RSSPECAN_VAL_LAYOUT_TYPE_REF_PHAS                                   30
#define RSSPECAN_VAL_LAYOUT_TYPE_SPECTRUM                                   31
#define RSSPECAN_VAL_LAYOUT_TYPE_BISTREAM                                   32
#define RSSPECAN_VAL_LAYOUT_TYPE_CMP_CONS                                   33
#define RSSPECAN_VAL_LAYOUT_TYPE_CDE_POW                                    34
#define RSSPECAN_VAL_LAYOUT_TYPE_CD_POWER                                   35
#define RSSPECAN_VAL_LAYOUT_TYPE_CMP_EVM                                    36
#define RSSPECAN_VAL_LAYOUT_TYPE_CH_TABLE                                   37
#define RSSPECAN_VAL_LAYOUT_TYPE_LST_EVAL                                   38
#define RSSPECAN_VAL_LAYOUT_TYPE_PCD_ERR                                    39
#define RSSPECAN_VAL_LAYOUT_TYPE_PWR_PCG                                    40
#define RSSPECAN_VAL_LAYOUT_TYPE_PWR_SYMB                                   41
#define RSSPECAN_VAL_LAYOUT_TYPE_SYM_CONS                                   42
#define RSSPECAN_VAL_LAYOUT_TYPE_SYM_EVM                                    43
#define RSSPECAN_VAL_LAYOUT_TYPE_CURRENT                                    44
#define RSSPECAN_VAL_LAYOUT_TYPE_PPWRAP                                     45
#define RSSPECAN_VAL_LAYOUT_TYPE_CBUFFER                                    46
#define RSSPECAN_VAL_LAYOUT_TYPE_MEAS                                       47
#define RSSPECAN_VAL_LAYOUT_TYPE_REFERENCE                                  48
#define RSSPECAN_VAL_LAYOUT_TYPE_EQUALIZER                                  49
#define RSSPECAN_VAL_LAYOUT_TYPE_EVECTOR                                    50
#define RSSPECAN_VAL_LAYOUT_TYPE_MACC                                       51
#define RSSPECAN_VAL_LAYOUT_TYPE_MERR                                       52
#define RSSPECAN_VAL_LAYOUT_TYPE_SYMBOLS                                    53
#define RSSPECAN_VAL_LAYOUT_TYPE_CMEMORY                                    54
#define RSSPECAN_VAL_LAYOUT_TYPE_CONSTEL                                    55
#define RSSPECAN_VAL_LAYOUT_TYPE_CVCARRIER                                  56
#define RSSPECAN_VAL_LAYOUT_TYPE_EVCARRIER                                  57
#define RSSPECAN_VAL_LAYOUT_TYPE_EVSYMBOL                                   58
#define RSSPECAN_VAL_LAYOUT_TYPE_FSPECTRUM                                  59
#define RSSPECAN_VAL_LAYOUT_TYPE_RSDETAIL                                   60
#define RSSPECAN_VAL_LAYOUT_TYPE_RSGLOBAL                                   61
#define RSSPECAN_VAL_LAYOUT_TYPE_SFIELD                                     62
#define RSSPECAN_VAL_LAYOUT_TYPE_SFLATNESS                                  63
#define RSSPECAN_VAL_LAYOUT_TYPE_PFPPDU                                     64
#define RSSPECAN_VAL_LAYOUT_TYPE_ASUM                                       65
#define RSSPECAN_VAL_LAYOUT_TYPE_BSTR                                       66
#define RSSPECAN_VAL_LAYOUT_TYPE_CCDF                                       67
#define RSSPECAN_VAL_LAYOUT_TYPE_FLAT                                       68
#define RSSPECAN_VAL_LAYOUT_TYPE_FSRS                                       69
#define RSSPECAN_VAL_LAYOUT_TYPE_EVSU                                       70
#define RSSPECAN_VAL_LAYOUT_TYPE_FEVS                                       71
#define RSSPECAN_VAL_LAYOUT_TYPE_IE                                         72
#define RSSPECAN_VAL_LAYOUT_TYPE_IEA                                        73
#define RSSPECAN_VAL_LAYOUT_TYPE_PSPE                                       74
#define RSSPECAN_VAL_LAYOUT_TYPE_EVRP                                       75
#define RSSPECAN_VAL_LAYOUT_TYPE_PVRP                                       76
#define RSSPECAN_VAL_LAYOUT_TYPE_PVRR                                       77
#define RSSPECAN_VAL_LAYOUT_TYPE_EVM_CHIP                                   78
#define RSSPECAN_VAL_LAYOUT_TYPE_FDRIFT                                     79
#define RSSPECAN_VAL_LAYOUT_TYPE_SRESULT                                    80
#define RSSPECAN_VAL_LAYOUT_TYPE_PDIS                                       81
#define RSSPECAN_VAL_LAYOUT_TYPE_PSP                                        82
#define RSSPECAN_VAL_LAYOUT_TYPE_AISC                                       83
#define RSSPECAN_VAL_LAYOUT_TYPE_EVSC                                       84
#define RSSPECAN_VAL_LAYOUT_TYPE_PVSC                                       85
#define RSSPECAN_VAL_LAYOUT_TYPE_URWM                                       86
#define RSSPECAN_VAL_LAYOUT_TYPE_CDEC                                       87
#define RSSPECAN_VAL_LAYOUT_TYPE_TDOM                                       88
#define RSSPECAN_VAL_LAYOUT_TYPE_FDIF                                       89
#define RSSPECAN_VAL_LAYOUT_TYPE_EVCARRIER_WLAN                             90
#define RSSPECAN_VAL_LAYOUT_TYPE_AM_AM                                      92
#define RSSPECAN_VAL_LAYOUT_TYPE_AM_PM                                      93
#define RSSPECAN_VAL_LAYOUT_TYPE_AM_EVM                                     94
#define RSSPECAN_VAL_LAYOUT_TYPE_PRIS                                       95
#define RSSPECAN_VAL_LAYOUT_TYPE_PFAL                                       96
#define RSSPECAN_VAL_LAYOUT_TYPE_STAB                                       97
#define RSSPECAN_VAL_LAYOUT_TYPE_RTIME_SPECTRUM                             98
#define RSSPECAN_VAL_LAYOUT_TYPE_RTIME_PERSISTENCE                          99
#define RSSPECAN_VAL_LAYOUT_TYPE_RTIME_PVT                                  100
#define RSSPECAN_VAL_LAYOUT_TYPE_RTIME_PVT_WATERFALL                        101
#define RSSPECAN_VAL_LAYOUT_TYPE_RTIME_SPECTROGRAM                          102
#define RSSPECAN_VAL_LAYOUT_TYPE_RRSP                                       103
#define RSSPECAN_VAL_LAYOUT_TYPE_SFLATNESS_DIFF                             104
#define RSSPECAN_VAL_LAYOUT_TYPE_SFLATNESS_SRS                              105
#define RSSPECAN_VAL_LAYOUT_TYPE_ADEM_RF_TIME_DOMAIN                        106
#define RSSPECAN_VAL_LAYOUT_TYPE_ADEM_AM_TIME_DOMAIN                        107
#define RSSPECAN_VAL_LAYOUT_TYPE_ADEM_AM_SPECTRUM                           108
#define RSSPECAN_VAL_LAYOUT_TYPE_ADEM_FM_TIME_DOMAIN                        109
#define RSSPECAN_VAL_LAYOUT_TYPE_ADEM_FM_SPECTRUM                           110
#define RSSPECAN_VAL_LAYOUT_TYPE_ADEM_PM_TIME_DOMAIN                        111
#define RSSPECAN_VAL_LAYOUT_TYPE_ADEM_PM_SPECTRUM                           112
#define RSSPECAN_VAL_LAYOUT_TYPE_ADEM_RF_SPECTRUM                           113
#define RSSPECAN_VAL_LAYOUT_TYPE_CFR                                        114
#define RSSPECAN_VAL_LAYOUT_TYPE_DBST                                       115
#define RSSPECAN_VAL_LAYOUT_TYPE_DDBS                                       116
#define RSSPECAN_VAL_LAYOUT_TYPE_HBST                                       117
#define RSSPECAN_VAL_LAYOUT_TYPE_HDBS                                       118
#define RSSPECAN_VAL_LAYOUT_TYPE_HEAD                                       119
#define RSSPECAN_VAL_LAYOUT_TYPE_PEVS                                       120
#define RSSPECAN_VAL_LAYOUT_TYPE_PTVS                                       121
#define RSSPECAN_VAL_LAYOUT_TYPE_DSUMMARY                                   123
#define RSSPECAN_VAL_LAYOUT_TYPE_SSUMMARY                                   124
#define RSSPECAN_VAL_LAYOUT_TYPE_MSPECTRUM                                  125
#define RSSPECAN_VAL_LAYOUT_TYPE_DPH                                        126
#define RSSPECAN_VAL_LAYOUT_TYPE_GDVT                                       127
#define RSSPECAN_VAL_LAYOUT_TYPE_PDVT                                       128
#define RSSPECAN_VAL_LAYOUT_TYPE_MERM                                       129
#define RSSPECAN_VAL_LAYOUT_TYPE_PCAR                                       130
#define RSSPECAN_VAL_LAYOUT_TYPE_PHAC                                       131
#define RSSPECAN_VAL_LAYOUT_TYPE_PLCM                                       132
#define RSSPECAN_VAL_LAYOUT_TYPE_PIAQ                                       133
#define RSSPECAN_VAL_LAYOUT_TYPE_HARM                                       134
#define RSSPECAN_VAL_LAYOUT_TYPE_INO                                        135
#define RSSPECAN_VAL_LAYOUT_TYPE_POW                                        136
#define RSSPECAN_VAL_LAYOUT_TYPE_SENS                                       137
#define RSSPECAN_VAL_LAYOUT_TYPE_SNT                                        138
#define RSSPECAN_VAL_LAYOUT_TYPE_SUPP                                       139
#define RSSPECAN_VAL_LAYOUT_TYPE_SFS                                        140
#define RSSPECAN_VAL_LAYOUT_TYPE_CJPH                                       141
#define RSSPECAN_VAL_LAYOUT_TYPE_CJT                                        142
#define RSSPECAN_VAL_LAYOUT_TYPE_XCOR                                       143
#define RSSPECAN_VAL_LAYOUT_TYPE_PINS                                       144
#define RSSPECAN_VAL_LAYOUT_TYPE_SJPH                                       145
#define RSSPECAN_VAL_LAYOUT_TYPE_SJT                                        146
#define RSSPECAN_VAL_LAYOUT_TYPE_PULSE                                      147
#define RSSPECAN_VAL_LAYOUT_TYPE_ALLOCATION_MATRIX                          148
#define RSSPECAN_VAL_LAYOUT_TYPE_CONSTELLATION_CARRIER                      149
#define RSSPECAN_VAL_LAYOUT_TYPE_CHANNEL_FLATNESS                           150
#define RSSPECAN_VAL_LAYOUT_TYPE_CONSTELLATION_SYMBOL                       151
#define RSSPECAN_VAL_LAYOUT_TYPE_TRIGGER_SYNC                               153
#define RSSPECAN_VAL_LAYOUT_TYPE_IMPULSE_RESPONSE                           154

// rsspecan_rngOFDMVSALayoutType
#define RSSPECAN_VAL_LAYOUT_TYPE_PSC                                        109
#define RSSPECAN_VAL_LAYOUT_TYPE_BITS                                       116
#define RSSPECAN_VAL_LAYOUT_TYPE_EVM_CARRIER                                152

// rsspecan_rngTraceExportAll
#define RSSPECAN_TRAC_EXPORT_MODE_SINGLE                                    0
#define RSSPECAN_TRAC_EXPORT_MODE_ALL                                       1

// rsspecan_rngMSRTxTechnology
#define RSSPECAN_VAL_MSR_TX_TECHNOLOGY_GSM                                  0
#define RSSPECAN_VAL_MSR_TX_TECHNOLOGY_WCDMA                                1
#define RSSPECAN_VAL_MSR_TX_TECHNOLOGY_LTE_1_40                             2
#define RSSPECAN_VAL_MSR_TX_TECHNOLOGY_LTE_3_00                             3
#define RSSPECAN_VAL_MSR_TX_TECHNOLOGY_LTE_5_00                             4
#define RSSPECAN_VAL_MSR_TX_TECHNOLOGY_LTE_10_00                            5
#define RSSPECAN_VAL_MSR_TX_TECHNOLOGY_LTE_15_00                            6
#define RSSPECAN_VAL_MSR_TX_TECHNOLOGY_LTE_20_00                            7
#define RSSPECAN_VAL_MSR_TX_TECHNOLOGY_USER                                 8
#define RSSPECAN_VAL_MSR_TX_TECHNOLOGY_NR5G_FR1_5MHZ                        9
#define RSSPECAN_VAL_MSR_TX_TECHNOLOGY_NR5G_FR1_10MHZ                       10
#define RSSPECAN_VAL_MSR_TX_TECHNOLOGY_NR5G_FR1_15MHZ                       11
#define RSSPECAN_VAL_MSR_TX_TECHNOLOGY_NR5G_FR1_20MHZ                       12
#define RSSPECAN_VAL_MSR_TX_TECHNOLOGY_NR5G_FR1_25MHZ                       13
#define RSSPECAN_VAL_MSR_TX_TECHNOLOGY_NR5G_FR1_30MHZ                       14
#define RSSPECAN_VAL_MSR_TX_TECHNOLOGY_NR5G_FR1_40MHZ                       15
#define RSSPECAN_VAL_MSR_TX_TECHNOLOGY_NR5G_FR1_50MHZ                       16
#define RSSPECAN_VAL_MSR_TX_TECHNOLOGY_NR5G_FR1_60MHZ                       17
#define RSSPECAN_VAL_MSR_TX_TECHNOLOGY_NR5G_FR1_70MHZ                       18
#define RSSPECAN_VAL_MSR_TX_TECHNOLOGY_NR5G_FR1_80MHZ                       19
#define RSSPECAN_VAL_MSR_TX_TECHNOLOGY_NR5G_FR1_90MHZ                       20
#define RSSPECAN_VAL_MSR_TX_TECHNOLOGY_NR5G_FR1_100MHZ                      21
#define RSSPECAN_VAL_MSR_TX_TECHNOLOGY_NR5G_FR2_50MHZ                       22
#define RSSPECAN_VAL_MSR_TX_TECHNOLOGY_NR5G_FR2_100MHZ                      23
#define RSSPECAN_VAL_MSR_TX_TECHNOLOGY_NR5G_FR2_200MHZ                      24
#define RSSPECAN_VAL_MSR_TX_TECHNOLOGY_NR5G_FR2_400MHZ                      25
#define RSSPECAN_VAL_MSR_TX_TECHNOLOGY_WCDM                                 26

// rsspecan_rngMCGDResultPhaseUnit
#define RSSPECAN_VAL_MCGD_RESULT_PHASE_UNIT_DEG                             0
#define RSSPECAN_VAL_MCGD_RESULT_PHASE_UNIT_RAD                             1

// rsspecan_rngMCGDCarrierOffsetMode
#define RSSPECAN_VAL_MCGD_CARRIER_OFFSET_ESTIMATED                          0
#define RSSPECAN_VAL_MCGD_CARRIER_OFFSET_FIXED                              1
#define RSSPECAN_VAL_MCGD_CARRIER_OFFSET_DCOM                               2

// rsspecan_rngServiceBBSignalType
#define RSSPECAN_VAL_BB_SIGNAL_TYPE_OFF                                     0
#define RSSPECAN_VAL_BB_SIGNAL_TYPE_AC                                      1
#define RSSPECAN_VAL_BB_SIGNAL_TYPE_DC                                      2

// rsspecan_rngMSRADataEvaluation
#define RSSPECAN_VAL_MSRA_EVAL_TIME_DOM                                     0
#define RSSPECAN_VAL_MSRA_EVAL_FREQUENCY_DOM                                1
#define RSSPECAN_VAL_MSRA_EVAL_IQ_DATA                                      2

// rsspecan_rngRealtimeMeasurement
#define RSSPECAN_VAL_REALTIME_HIGH_RESOLUTION                               0
#define RSSPECAN_VAL_REALTIME_MULTI_DOMAIN                                  1

// rsspecan_rngExternalGeneratorInterfaceType
#define RSSPECAN_VAL_INTERFACE_GPIB                                         0
#define RSSPECAN_VAL_INTERFACE_TCPIP                                        1

// rsspecan_rngSpemSearchMode
#define RSSPECAN_VAL_SPEM_SEARCH_MIN                                        0
#define RSSPECAN_VAL_SPEM_SEARCH_MAX                                        1

// rsspecan_rngSpemSearchDirection
#define RSSPECAN_VAL_SPEM_SEARCH_PEAK                                       0
#define RSSPECAN_VAL_SPEM_SEARCH_ABOVE                                      1
#define RSSPECAN_VAL_SPEM_SEARCH_BELOW                                      2
#define RSSPECAN_VAL_SPEM_SEARCH_NEXT                                       3

// rsspecan_rngAxis
#define RSSPECAN_VAL_AXIS_X                                                 0
#define RSSPECAN_VAL_AXIS_Y                                                 1

// rsspecan_rngMaskLowerUpper
#define RSSPECAN_VAL_MASK_LOWER                                             0
#define RSSPECAN_VAL_MASK_UPPER                                             1

// rsspecan_rngRealtimeTraceType
#define RSSPECAN_VAL_RTIM_TRACE_SPECTROGRAM                                 0
#define RSSPECAN_VAL_RTIM_TRACE_PERSISTENCE                                 1
#define RSSPECAN_VAL_RTIM_TRACE_MAXHOLD                                     2

// rsspecan_rngTransducerSetUnit
#define RSSPECAN_VAL_TSET_UNIT_DB                                           0
#define RSSPECAN_VAL_TSET_UNIT_DBM                                          1
#define RSSPECAN_VAL_TSET_UNIT_DBUV                                         2
#define RSSPECAN_VAL_TSET_UNIT_DBUVM                                        3
#define RSSPECAN_VAL_TSET_UNIT_DBUA                                         4
#define RSSPECAN_VAL_TSET_UNIT_DBUAM                                        5
#define RSSPECAN_VAL_TSET_UNIT_DBPW                                         6
#define RSSPECAN_VAL_TSET_UNIT_DBPT                                         7

// rsspecan_rngMathMode
#define RSSPECAN_VAL_POWER                                                  2

// rsspecan_rngCMAPItem
#define RSSPECAN_VAL_CMAP1                                                  1
#define RSSPECAN_VAL_CMAP2                                                  2
#define RSSPECAN_VAL_CMAP3                                                  3
#define RSSPECAN_VAL_CMAP4                                                  4
#define RSSPECAN_VAL_CMAP5                                                  5
#define RSSPECAN_VAL_CMAP6                                                  6
#define RSSPECAN_VAL_CMAP7                                                  7
#define RSSPECAN_VAL_CMAP8                                                  8
#define RSSPECAN_VAL_CMAP9                                                  9
#define RSSPECAN_VAL_CMAP10                                                 10
#define RSSPECAN_VAL_CMAP11                                                 11
#define RSSPECAN_VAL_CMAP12                                                 12
#define RSSPECAN_VAL_CMAP13                                                 13
#define RSSPECAN_VAL_CMAP14                                                 14
#define RSSPECAN_VAL_CMAP15                                                 15
#define RSSPECAN_VAL_CMAP16                                                 16
#define RSSPECAN_VAL_CMAP17                                                 17
#define RSSPECAN_VAL_CMAP18                                                 18
#define RSSPECAN_VAL_CMAP19                                                 19
#define RSSPECAN_VAL_CMAP20                                                 20
#define RSSPECAN_VAL_CMAP21                                                 21
#define RSSPECAN_VAL_CMAP22                                                 22
#define RSSPECAN_VAL_CMAP23                                                 23
#define RSSPECAN_VAL_CMAP24                                                 24
#define RSSPECAN_VAL_CMAP25                                                 25
#define RSSPECAN_VAL_CMAP26                                                 26
#define RSSPECAN_VAL_CMAP27                                                 27
#define RSSPECAN_VAL_CMAP28                                                 28
#define RSSPECAN_VAL_CMAP29                                                 29
#define RSSPECAN_VAL_CMAP30                                                 30
#define RSSPECAN_VAL_CMAP31                                                 31
#define RSSPECAN_VAL_CMAP32                                                 32
#define RSSPECAN_VAL_CMAP33                                                 33
#define RSSPECAN_VAL_CMAP34                                                 34
#define RSSPECAN_VAL_CMAP35                                                 35
#define RSSPECAN_VAL_CMAP36                                                 36
#define RSSPECAN_VAL_CMAP37                                                 37
#define RSSPECAN_VAL_CMAP38                                                 38
#define RSSPECAN_VAL_CMAP39                                                 39
#define RSSPECAN_VAL_CMAP40                                                 40
#define RSSPECAN_VAL_CMAP41                                                 41

// rsspecan_rngDisplayMarkerTable
#define RSSPECAN_VAL_MARKER_TABLE_OFF                                       0
#define RSSPECAN_VAL_MARKER_TABLE_ON                                        1
#define RSSPECAN_VAL_MARKER_TABLE_AUTO                                      2

// rsspecan_rngStatisticsResultType
#define RSSPECAN_VAL_STAT_RESULT_MEAN                                       0
#define RSSPECAN_VAL_STAT_RESULT_PEAK                                       1
#define RSSPECAN_VAL_STAT_RESULT_CFAC                                       2

// rsspecan_rngChannelType
#define RSSPECAN_VAL_CHANNEL_TYPE_SPECTRUM                                  0
#define RSSPECAN_VAL_CHANNEL_TYPE_IQ                                        1
#define RSSPECAN_VAL_CHANNEL_TYPE_PULSE                                     2
#define RSSPECAN_VAL_CHANNEL_TYPE_ADEM                                      3
#define RSSPECAN_VAL_CHANNEL_TYPE_MCGD                                      4
#define RSSPECAN_VAL_CHANNEL_TYPE_NOISE                                     5
#define RSSPECAN_VAL_CHANNEL_TYPE_PNOISE                                    6
#define RSSPECAN_VAL_CHANNEL_TYPE_VSA                                       7
#define RSSPECAN_VAL_CHANNEL_TYPE_3GPP_BTS                                  8
#define RSSPECAN_VAL_CHANNEL_TYPE_3GPP_UE                                   9
#define RSSPECAN_VAL_CHANNEL_TYPE_BC2K                                      10
#define RSSPECAN_VAL_CHANNEL_TYPE_MC2K                                      11
#define RSSPECAN_VAL_CHANNEL_TYPE_MDO                                       12
#define RSSPECAN_VAL_CHANNEL_TYPE_BDO                                       13
#define RSSPECAN_VAL_CHANNEL_TYPE_GSM                                       14
#define RSSPECAN_VAL_CHANNEL_TYPE_WLAN                                      15
#define RSSPECAN_VAL_CHANNEL_TYPE_LTE                                       16
#define RSSPECAN_VAL_CHANNEL_TYPE_RTIM                                      17
#define RSSPECAN_VAL_CHANNEL_TYPE_TA                                        18
#define RSSPECAN_VAL_CHANNEL_TYPE_BTDS                                      19
#define RSSPECAN_VAL_CHANNEL_TYPE_MTDS                                      20
#define RSSPECAN_VAL_CHANNEL_TYPE_WIGIG                                     21
#define RSSPECAN_VAL_CHANNEL_TYPE_EDMG                                      22
#define RSSPECAN_VAL_CHANNEL_TYPE_AMPL                                      23
#define RSSPECAN_VAL_CHANNEL_TYPE_AVIONICS                                  24
#define RSSPECAN_VAL_CHANNEL_TYPE_DOCSIS                                    25
#define RSSPECAN_VAL_CHANNEL_TYPE_NBIOT                                     26
#define RSSPECAN_VAL_CHANNEL_TYPE_NR5G                                      27
#define RSSPECAN_VAL_CHANNEL_TYPE_ONEWEB                                    28
#define RSSPECAN_VAL_CHANNEL_TYPE_SPURIOUS                                  29
#define RSSPECAN_VAL_CHANNEL_TYPE_V5GT                                      30
#define RSSPECAN_VAL_CHANNEL_TYPE_BLUETOOTH                                 31
#define RSSPECAN_VAL_CHANNEL_TYPE_HRP_UWB                                   32
#define RSSPECAN_VAL_CHANNEL_TYPE_OFDM_VSA                                  33

// rsspecan_rngLayoutDirection
#define RSSPECAN_VAL_LAYOUT_DIRECTION_LEFT                                  0
#define RSSPECAN_VAL_LAYOUT_DIRECTION_RIGHT                                 1
#define RSSPECAN_VAL_LAYOUT_DIRECTION_ABOVE                                 2
#define RSSPECAN_VAL_LAYOUT_DIRECTION_BELOW                                 3

// rsspecan_rngLimitCheckState
#define RSSPECAN_VAL_LIMIT_CHECK_STATE_PASS                                 0
#define RSSPECAN_VAL_LIMIT_CHECK_STATE_FAIL                                 1

// rsspecan_rngMeasStatProbability
#define RSSPECAN_VAL_SSTAT_PROB_0_01                                        0
#define RSSPECAN_VAL_SSTAT_PROB_0_1                                         1
#define RSSPECAN_VAL_SSTAT_PROB_1                                           2
#define RSSPECAN_VAL_SSTAT_PROB_10                                          3

// rsspecan_rngDetector
#define RSSPECAN_VAL_BURS_DETECTOR_MEAN                                     0
#define RSSPECAN_VAL_BURS_DETECTOR_PEAK                                     1

// rsspecan_rngAcquisitionStatus
#define RSSPECAN_VAL_ACQUISITION_STATUS_INPROGRESS                          0
#define RSSPECAN_VAL_ACQUISITION_STATUS_COMPLETE                            1

// rsspecan_rngTraceResultType
#define RSSPECAN_VAL_TRACE_RESULT_TRACE1                                    0
#define RSSPECAN_VAL_TRACE_RESULT_TRACE2                                    1
#define RSSPECAN_VAL_TRACE_RESULT_TRACE3                                    2
#define RSSPECAN_VAL_TRACE_RESULT_TRACE4                                    3
#define RSSPECAN_VAL_TRACE_RESULT_TRACE5                                    4
#define RSSPECAN_VAL_TRACE_RESULT_TRACE6                                    5
#define RSSPECAN_VAL_TRACE_RESULT_LIST                                      6
#define RSSPECAN_VAL_TRACE_RESULT_SPURIOUS                                  7
#define RSSPECAN_VAL_TRACE_RESULT_SPECTROGRAM                               8
#define RSSPECAN_VAL_TRACE_RESULT_XGAIN                                     9

// rsspecan_rngStatusRegisterOperation
#define RSSPECAN_VAL_STAT_REG_QUESTIONABLE                                  1
#define RSSPECAN_VAL_STAT_REG_OPERATION                                     2

// rsspecan_rngStatusRegister
#define RSSPECAN_VAL_STAT_REG_MAIN                                          0
#define RSSPECAN_VAL_STAT_REG_POWER                                         1
#define RSSPECAN_VAL_STAT_REG_LIMIT                                         2
#define RSSPECAN_VAL_STAT_REG_LMARGIN                                       3
#define RSSPECAN_VAL_STAT_REG_SYNC                                          4
#define RSSPECAN_VAL_STAT_REG_ACP_LIMIT                                     5
#define RSSPECAN_VAL_STAT_REG_FREQUENCY                                     6
#define RSSPECAN_VAL_STAT_REG_DIQ                                           7
#define RSSPECAN_VAL_STAT_REG_EXTENDED                                      8
#define RSSPECAN_VAL_STAT_REG_EXTENDED_INFO                                 9
#define RSSPECAN_VAL_STAT_REG_TEMP                                          10
#define RSSPECAN_VAL_STAT_REG_TIME                                          11
#define RSSPECAN_VAL_STAT_REG_POW_DCPN                                      12
#define RSSPECAN_VAL_STAT_REG_PNOISE                                        13

// rsspecan_rngStatusRegisterQuery
#define RSSPECAN_VAL_STAT_REG_QUERY_COND                                    0
#define RSSPECAN_VAL_STAT_REG_QUERY_EVENT                                   1
#define RSSPECAN_VAL_STAT_REG_QUERY_ENABLE                                  2
#define RSSPECAN_VAL_STAT_REG_QUERY_PTRANSITION                             3
#define RSSPECAN_VAL_STAT_REG_QUERY_NTRANSITION                             4

// rsspecan_rngPmetCoupling
#define RSSPECAN_VAL_PMET_COUP_OFF                                          0
#define RSSPECAN_VAL_PMET_COUP_CENT                                         1
#define RSSPECAN_VAL_PMET_COUP_MARK                                         2

// rsspecan_rngBtoGeog
#define RSSPECAN_BTO_GEOG_EUR                                               0
#define RSSPECAN_BTO_GEOG_USA                                               1
#define RSSPECAN_BTO_GEOG_FRAN                                              2

// rsspecan_rngPmetMeasTime
#define RSSPECAN_VAL_PMET_MEASTIME_NORM                                     0
#define RSSPECAN_VAL_PMET_MEASTIME_SHORT                                    1
#define RSSPECAN_VAL_PMET_MEASTIME_LONG                                     2

// rsspecan_rngPmetUnitAbs
#define RSSPECAN_VAL_ABSUNIT_DB                                             3
#define RSSPECAN_VAL_ABSUNIT_PCT                                            4

// rsspecan_rngBtoPtype
#define RSSPECAN_VAL_BTO_PTYPE_AUTO                                         0
#define RSSPECAN_VAL_BTO_PTYPE_DH1                                          1
#define RSSPECAN_VAL_BTO_PTYPE_DH3                                          2
#define RSSPECAN_VAL_BTO_PTYPE_DH5                                          3
#define RSSPECAN_BTO_PTYPE_DM1                                              4
#define RSSPECAN_BTO_PTYPE_DM3                                              5
#define RSSPECAN_BTO_PTYPE_DM5                                              6
#define RSSPECAN_BTO_PTYPE_FHS                                              7
#define RSSPECAN_BTO_PTYPE_HV1                                              8
#define RSSPECAN_BTO_PTYPE_HV2                                              9
#define RSSPECAN_BTO_PTYPE_HV3                                              10
#define RSSPECAN_BTO_PTYPE_DV                                               11
#define RSSPECAN_BTO_PTYPE_NULL                                             12
#define RSSPECAN_BTO_PTYPE_POLL                                             13
#define RSSPECAN_BTO_PTYPE_AUX1                                             14
#define RSSPECAN_BTO_PTYPE_UNDEF                                            15

// rsspecan_rngBtoMeasFilt
#define RSSPECAN_BTO_MEASFILT_OFF                                           0
#define RSSPECAN_BTO_MEASFILT_BTO                                           1

// rsspecan_rngBtoTraceDetector
#define RSSPECAN_BTO_TRAC_DET_APEAK                                         0
#define RSSPECAN_BTO_TRAC_DET_NEG                                           1
#define RSSPECAN_BTO_TRAC_DET_POS                                           2
#define RSSPECAN_BTO_TRAC_DET_SAMPLE                                        3
#define RSSPECAN_BTO_TRAC_DET_RMS                                           4
#define RSSPECAN_BTO_TRAC_DET_AVER                                          5

// rsspecan_rngBtoMeasMode
#define RSSPECAN_BTO_MEAS_MODE_OPOW                                         0
#define RSSPECAN_BTO_MEAS_MODE_ACLR                                         1
#define RSSPECAN_BTO_MEAS_MODE_MCH                                          2
#define RSSPECAN_BTO_MEAS_MODE_ICFT                                         3
#define RSSPECAN_BTO_MEAS_MODE_CFDR                                         4
#define RSSPECAN_BTO_MEAS_MODE_RTP                                          5
#define RSSPECAN_BTO_MEAS_MODE_IBS                                          6
#define RSSPECAN_BTO_MEAS_MODE_DPEN                                         7
#define RSSPECAN_BTO_MEAS_MODE_CFST                                         8

// rsspecan_rngBtoWritePtype
#define RSSPECAN_VAL_BTO_PTYPE_LE1M                                         4
#define RSSPECAN_VAL_BTO_PTYPE_LE2M                                         5
#define RSSPECAN_VAL_BTO_PTYPE_LECODED                                      6

// rsspecan_rngFmTriggerSource
#define RSSPECAN_VAL_FM_TRIGGER_SOURCE_IMM                                  0
#define RSSPECAN_VAL_FM_TRIGGER_SOURCE_EXT                                  1
#define RSSPECAN_VAL_FM_TRIGGER_SOURCE_EXT2                                 2
#define RSSPECAN_VAL_FM_TRIGGER_SOURCE_EXT3                                 3
#define RSSPECAN_VAL_FM_TRIGGER_SOURCE_RFP                                  4
#define RSSPECAN_VAL_FM_TRIGGER_SOURCE_IFP                                  5
#define RSSPECAN_VAL_FM_TRIGGER_SOURCE_IQP                                  6
#define RSSPECAN_VAL_FM_TRIGGER_SOURCE_TIME                                 7
#define RSSPECAN_VAL_FM_TRIGGER_SOURCE_BBP                                  8
#define RSSPECAN_VAL_FM_TRIGGER_SOURCE_PSEN                                 9
#define RSSPECAN_VAL_FM_TRIGGER_SOURCE_AF                                   10
#define RSSPECAN_VAL_FM_TRIGGER_SOURCE_FM                                   11
#define RSSPECAN_VAL_FM_TRIGGER_SOURCE_AM                                   12
#define RSSPECAN_VAL_FM_TRIGGER_SOURCE_AMR                                  13
#define RSSPECAN_VAL_FM_TRIGGER_SOURCE_PM                                   14
#define RSSPECAN_VAL_FM_TRIGGER_SOURCE_GP0                                  15
#define RSSPECAN_VAL_FM_TRIGGER_SOURCE_GP1                                  16
#define RSSPECAN_VAL_FM_TRIGGER_SOURCE_GP2                                  17
#define RSSPECAN_VAL_FM_TRIGGER_SOURCE_GP3                                  18
#define RSSPECAN_VAL_FM_TRIGGER_SOURCE_GP4                                  19
#define RSSPECAN_VAL_FM_TRIGGER_SOURCE_GP5                                  20

// rsspecan_rngAdemPhaseWrap
#define RSSPECAN_VAL_ADEM_UPHAS                                             0
#define RSSPECAN_VAL_ADEM_PHAS                                              1

// rsspecan_rngUnitAngle
#define RSSPECAN_VAL_UNIT_ANGLE_DEG                                         0
#define RSSPECAN_VAL_UNIT_ANGLE_RAD                                         1

// rsspecan_rngFMSTriggerSource
#define RSSPECAN_VAL_FMS_TRG_IMM                                            0
#define RSSPECAN_VAL_FMS_TRG_EXT                                            1
#define RSSPECAN_VAL_FMS_TRG_IFP                                            2
#define RSSPECAN_VAL_FMS_TRG_TIME                                           3
#define RSSPECAN_VAL_FMS_TRG_LEFT                                           4
#define RSSPECAN_VAL_FMS_TRG_RIGH                                           5
#define RSSPECAN_VAL_FMS_TRG_MPX                                            6
#define RSSPECAN_VAL_FMS_TRG_MONO                                           7
#define RSSPECAN_VAL_FMS_TRG_STER                                           8
#define RSSPECAN_VAL_FMS_TRG_RDS                                            9
#define RSSPECAN_VAL_FMS_TRG_PIL                                            10

// rsspecan_rngFMSDisplayConfig
#define RSSPECAN_VAL_FMS_DISP_SUMM                                          1
#define RSSPECAN_VAL_FMS_DISP_TD_RF                                         2
#define RSSPECAN_VAL_FMS_DISP_TD_LEFT                                       3
#define RSSPECAN_VAL_FMS_DISP_TD_RIGHT                                      4
#define RSSPECAN_VAL_FMS_DISP_TD_MPX                                        5
#define RSSPECAN_VAL_FMS_DISP_TD_MONO                                       6
#define RSSPECAN_VAL_FMS_DISP_TD_STER                                       7
#define RSSPECAN_VAL_FMS_DISP_TD_RDS                                        8
#define RSSPECAN_VAL_FMS_DISP_TD_PILOT                                      9
#define RSSPECAN_VAL_FMS_DISP_SPEC_RF                                       10
#define RSSPECAN_VAL_FMS_DISP_SPEC_LEFT                                     11
#define RSSPECAN_VAL_FMS_DISP_SPEC_RIGHT                                    12
#define RSSPECAN_VAL_FMS_DISP_SPEC_MPX                                      13
#define RSSPECAN_VAL_FMS_DISP_SPEC_MONO                                     14
#define RSSPECAN_VAL_FMS_DISP_SPEC_STER                                     15
#define RSSPECAN_VAL_FMS_DISP_SPEC_RDS                                      16
#define RSSPECAN_VAL_FMS_DISP_SPEC_PILOT                                    17

// rsspecan_rngFMSRSumDetector
#define RSSPECAN_VAL_FMS_RSUM_DETEC_RMS                                     0
#define RSSPECAN_VAL_FMS_RSUM_DETEC_SRMS                                    1
#define RSSPECAN_VAL_FMS_RSUM_DETEC_PPE                                     2
#define RSSPECAN_VAL_FMS_RSUM_DETEC_NPE                                     3
#define RSSPECAN_VAL_FMS_RSUM_DETEC_PAV                                     4
#define RSSPECAN_VAL_FMS_RSUM_DETEC_QPE                                     5
#define RSSPECAN_VAL_FMS_RSUM_DETEC_SQP                                     6

// rsspecan_rngFMSRSumMode
#define RSSPECAN_VAL_FMS_RSUM_MODE_WRIT                                     0
#define RSSPECAN_VAL_FMS_RSUM_MODE_AVER                                     1
#define RSSPECAN_VAL_FMS_RSUM_MODE_PHOL                                     2

// rsspecan_rngPulseSignalPeriod
#define RSSPECAN_VAL_PULSE_SIGNAL_PERIOD_HL                                 0
#define RSSPECAN_VAL_PULSE_SIGNAL_PERIOD_LH                                 1

// rsspecan_rngPulseSignalModulation
#define RSSPECAN_VAL_PULSE_SIGNAL_MODULATION_ARB                            0
#define RSSPECAN_VAL_PULSE_SIGNAL_MODULATION_CW                             1
#define RSSPECAN_VAL_PULSE_SIGNAL_MODULATION_LFM                            2
#define RSSPECAN_VAL_PULSE_SIGNAL_MODULATION_RIQ                            3

// rsspecan_rngPulseInputSignalSource
#define RSSPECAN_VAL_PULSE_INPUT_RF                                         0
#define RSSPECAN_VAL_PULSE_INPUT_FIQ                                        1

// rsspecan_rngPulseAcquisitionFilterType
#define RSSPECAN_VAL_PULSE_FILTER_TYPE_FLAT                                 0
#define RSSPECAN_VAL_PULSE_FILTER_TYPE_GAUSS                                1

// rsspecan_rngPulseDetectionReferenceSource
#define RSSPECAN_VAL_PULSE_DETECTION_REF_SOURCE_REFLEVEL                    0
#define RSSPECAN_VAL_PULSE_DETECTION_REF_SOURCE_PEAK                        1
#define RSSPECAN_VAL_PULSE_DETECTION_REF_SOURCE_NOISE                       2
#define RSSPECAN_VAL_PULSE_DETECTION_REF_SOURCE_ABSOLUTE                    3

// rsspecan_rngPulseReferencePointType
#define RSSPECAN_VAL_PULSE_REFERENCE_POINT_RISE                             0
#define RSSPECAN_VAL_PULSE_REFERENCE_POINT_CENTER                           1
#define RSSPECAN_VAL_PULSE_REFERENCE_POINT_FALL                             2
#define RSSPECAN_VAL_PULSE_REFERENCE_POINT_TRIGGER                          3

// rsspecan_rngPulseReferenceRangeType
#define RSSPECAN_VAL_PULSE_REFERENCE_RANGE_CENTER                           0
#define RSSPECAN_VAL_PULSE_REFERENCE_RANGE_EDGE                             1

// rsspecan_rngPulseTopLevelPosition
#define RSSPECAN_VAL_PULSE_TOP_LEVEL_POSITION_EDGE                          0
#define RSSPECAN_VAL_PULSE_TOP_LEVEL_POSITION_CENTER                        1

// rsspecan_rngPulseTopLevelAlgorithm
#define RSSPECAN_VAL_PULSE_TOP_LEVEL_ALGORITHM_MEAN                         0
#define RSSPECAN_VAL_PULSE_TOP_LEVEL_ALGORITHM_MEDIAN                       1
#define RSSPECAN_VAL_PULSE_TOP_LEVEL_ALGORITHM_PEAK_POWER                   4
#define RSSPECAN_VAL_PULSE_TOP_LEVEL_ALGORITHM_PEAK_FIXED                   5

// rsspecan_rngPulseReferenceLevelUnit
#define RSSPECAN_VAL_PULSE_REFERENCE_LEVEL_UNIT_V                           0
#define RSSPECAN_VAL_PULSE_REFERENCE_LEVEL_UNIT_W                           1
#define RSSPECAN_VAL_PULSE_REFERENCE_LEVEL_UNIT_DBM                         2

// rsspecan_rngPulseResultRangeReference
#define RSSPECAN_VAL_PULSE_RESULT_RANGE_REFERENCE_RISE                      0
#define RSSPECAN_VAL_PULSE_RESULT_RANGE_REFERENCE_CENTER                    1
#define RSSPECAN_VAL_PULSE_RESULT_RANGE_REFERENCE_FALL                      2
#define RSSPECAN_VAL_PULSE_RESULT_RANGE_REFERENCE_TRIG                      3

// rsspecan_rngPulseResultRangeAlignment
#define RSSPECAN_VAL_PULSE_RESULT_RANGE_ALIGNMENT_LEFT                      0
#define RSSPECAN_VAL_PULSE_RESULT_RANGE_ALIGNMENT_CENTER                    1
#define RSSPECAN_VAL_PULSE_RESULT_RANGE_ALIGNMENT_RIGHT                     2

// rsspecan_rngPulseResultPhaseUnit
#define RSSPECAN_VAL_PULSE_RESULT_PHASE_UNIT_DEG                            0
#define RSSPECAN_VAL_PULSE_RESULT_PHASE_UNIT_RAD                            1

// rsspecan_rngPulseParameterPowerTrend
#define RSSPECAN_VAL_PULSE_RESULT_POWER_TOP                                 1
#define RSSPECAN_VAL_PULSE_RESULT_POWER_BASE                                2
#define RSSPECAN_VAL_PULSE_RESULT_POWER_AMPL                                3
#define RSSPECAN_VAL_PULSE_RESULT_POWER_ON                                  4
#define RSSPECAN_VAL_PULSE_RESULT_POWER_AVG                                 5
#define RSSPECAN_VAL_PULSE_RESULT_POWER_MIN                                 6
#define RSSPECAN_VAL_PULSE_RESULT_POWER_MAX                                 7
#define RSSPECAN_VAL_PULSE_RESULT_POWER_PON                                 8
#define RSSPECAN_VAL_PULSE_RESULT_POWER_PAVG                                9
#define RSSPECAN_VAL_PULSE_RESULT_POWER_PMIN                                10
#define RSSPECAN_VAL_PULSE_RESULT_POWER_ADP                                 11
#define RSSPECAN_VAL_PULSE_RESULT_POWER_ADDB                                12
#define RSSPECAN_VAL_PULSE_RESULT_POWER_RPER                                13
#define RSSPECAN_VAL_PULSE_RESULT_POWER_RDB                                 14
#define RSSPECAN_VAL_PULSE_RESULT_POWER_OPER                                15
#define RSSPECAN_VAL_PULSE_RESULT_POWER_ODB                                 16
#define RSSPECAN_VAL_PULSE_RESULT_POWER_POIN                                17
#define RSSPECAN_VAL_PULSE_RESULT_POWER_PPR                                 18
#define RSSPECAN_VAL_PULSE_RESULT_POWER_AMPL_I                              19
#define RSSPECAN_VAL_PULSE_RESULT_POWER_AMPL_Q                              20
#define RSSPECAN_VAL_PULSE_RESULT_POWER_I                                   21
#define RSSPECAN_VAL_PULSE_RESULT_POWER_Q                                   22

// rsspecan_rngPulseParameterTiming
#define RSSPECAN_VAL_PULSE_RESULT_TIMING_TST                                1
#define RSSPECAN_VAL_PULSE_RESULT_TIMING_SETT                               2
#define RSSPECAN_VAL_PULSE_RESULT_TIMING_RISE                               3
#define RSSPECAN_VAL_PULSE_RESULT_TIMING_FALL                               4
#define RSSPECAN_VAL_PULSE_RESULT_TIMING_PWID                               5
#define RSSPECAN_VAL_PULSE_RESULT_TIMING_OFF                                6
#define RSSPECAN_VAL_PULSE_RESULT_TIMING_DRAT                               7
#define RSSPECAN_VAL_PULSE_RESULT_TIMING_DCYC                               8
#define RSSPECAN_VAL_PULSE_RESULT_TIMING_PRI                                9
#define RSSPECAN_VAL_PULSE_RESULT_TIMING_PRF                                10

// rsspecan_rngPulseParameterFrequency
#define RSSPECAN_VAL_PULSE_RESULT_FREQ_POIN                                 1
#define RSSPECAN_VAL_PULSE_RESULT_FREQ_PPFR                                 2
#define RSSPECAN_VAL_PULSE_RESULT_FREQ_RERR                                 3
#define RSSPECAN_VAL_PULSE_RESULT_FREQ_PERR                                 4
#define RSSPECAN_VAL_PULSE_RESULT_FREQ_DEV                                  5
#define RSSPECAN_VAL_PULSE_RESULT_FREQ_CRAT                                 6

// rsspecan_rngPulseParameterPhase
#define RSSPECAN_VAL_PULSE_RESULT_PHASE_POIN                                1
#define RSSPECAN_VAL_PULSE_RESULT_PHASE_PPPH                                2
#define RSSPECAN_VAL_PULSE_RESULT_PHASE_RERR                                3
#define RSSPECAN_VAL_PULSE_RESULT_PHASE_PERR                                4
#define RSSPECAN_VAL_PULSE_RESULT_PHASE_DEV                                 5

// rsspecan_rngPulseParameterYAxis
#define RSSPECAN_VAL_PULSE_RESULT_Y_AXIS_COUNT                              0
#define RSSPECAN_VAL_PULSE_RESULT_Y_AXIS_OCCURANCE                          1

// rsspecan_rngPulseColumns
#define RSSPECAN_VAL_PULSE_COLLUMNS_SELECTED                                0
#define RSSPECAN_VAL_PULSE_COLLUMNS_ALL                                     1

// rsspecan_rngPulseQueryRange
#define RSSPECAN_VAL_PULSE_QUERY_RANGE_CURRENT                              0
#define RSSPECAN_VAL_PULSE_QUERY_RANGE_ALL                                  1
#define RSSPECAN_VAL_PULSE_QUERY_RANGE_SELECTED                             2

// rsspecan_rngPulseResultType
#define RSSPECAN_VAL_PULSE_MEASTYPE_MIN                                     1
#define RSSPECAN_VAL_PULSE_MEASTYPE_MAX                                     2
#define RSSPECAN_VAL_PULSE_MEASTYPE_AVER                                    3
#define RSSPECAN_VAL_PULSE_MEASTYPE_SDEV                                    4
#define RSSPECAN_VAL_PULSE_MEASTYPE_CURR                                    0

// rsspecan_rngAdemOperatingMode
#define RSSPECAN_VAL_FM_FEED_AM_REL                                         0
#define RSSPECAN_VAL_FM_FEED_AM                                             1
#define RSSPECAN_VAL_FM_FEED_SPEC                                           2
#define RSSPECAN_VAL_FM_FEED_FM                                             3
#define RSSPECAN_VAL_FM_FEED_PM                                             4
#define RSSPECAN_VAL_FM_FEED_AMS_REL                                        5
#define RSSPECAN_VAL_FM_FEED_AMS                                            6
#define RSSPECAN_VAL_FM_FEED_FMS                                            7
#define RSSPECAN_VAL_FM_FEED_PMS                                            8
#define RSSPECAN_VAL_FM_FEED_FM_AFSP                                        9
#define RSSPECAN_VAL_FM_FEED_PM_AFSP                                        10
#define RSSPECAN_VAL_FM_FEED_AM_REL_AFSP                                    11
#define RSSPECAN_VAL_FM_FEED_RFP_AFSP                                       12
#define RSSPECAN_VAL_FSM_FEED_SUMM                                          13

// rsspecan_rngFMDemodType
#define RSSPECAN_VAL_FMDEM_AM                                               0
#define RSSPECAN_VAL_FMDEM_FM                                               1
#define RSSPECAN_VAL_FMDEM_PM                                               2
#define RSSPECAN_VAL_FMDEM_AFSAM                                            3
#define RSSPECAN_VAL_FMDEM_AFSFM                                            4
#define RSSPECAN_VAL_FMDEM_AFSPM                                            5
#define RSSPECAN_VAL_FMDEM_RF                                               6
#define RSSPECAN_VAL_FMDEM_AMREL                                            7
#define RSSPECAN_VAL_FMDEM_AFSAMREL                                         8
#define RSSPECAN_VAL_FMDEM_ACV                                              9
#define RSSPECAN_VAL_FMDEM_ACV_SPECTRUM                                     10

// rsspecan_rngADemOffsetResultType
#define RSSPECAN_VAL_FMDEM_IMM                                              0
#define RSSPECAN_VAL_FMDEM_AVG                                              1

// rsspecan_rngFMSChanType
#define RSSPECAN_VAL_FSM_CHAN_TYPE_LEFT                                     0
#define RSSPECAN_VAL_FSM_CHAN_TYPE_RIGHT                                    1
#define RSSPECAN_VAL_FSM_CHAN_TYPE_MPX                                      2
#define RSSPECAN_VAL_FSM_CHAN_TYPE_MONO                                     3
#define RSSPECAN_VAL_FSM_CHAN_TYPE_STER                                     4
#define RSSPECAN_VAL_FSM_CHAN_TYPE_RDS                                      5
#define RSSPECAN_VAL_FSM_CHAN_TYPE_PILOT                                    6

// rsspecan_rngFMSMeasType
#define RSSPECAN_VAL_FMS_AFSP                                               0
#define RSSPECAN_VAL_FMS_TDOM                                               1

// rsspecan_rngGSMDUT
#define RSSPECAN_VAL_GSM_K10_BTS_NORMAL                                     0
#define RSSPECAN_VAL_GSM_K10_BTS_MICRO                                      1
#define RSSPECAN_VAL_GSM_K10_BTS_PICO                                       2
#define RSSPECAN_VAL_GSM_K10_MS_NORMAL                                      3
#define RSSPECAN_VAL_GSM_K10_MS_SMALL                                       4
#define RSSPECAN_VAL_GSM_K10_MCB_LOCAL                                      5
#define RSSPECAN_VAL_GSM_K10_MCB_MEDIUM                                     6
#define RSSPECAN_VAL_GSM_K10_MCB_WIDE                                       7

// rsspecan_rngGSMBandClass
#define RSSPECAN_VAL_GSM_K10_BAND_CLASS_PGSM                                0
#define RSSPECAN_VAL_GSM_K10_BAND_CLASS_EGSM                                1
#define RSSPECAN_VAL_GSM_K10_BAND_CLASS_DCS                                 2
#define RSSPECAN_VAL_GSM_K10_BAND_CLASS_PCS                                 3
#define RSSPECAN_VAL_GSM_K10_BAND_CLASS_TGSM                                4
#define RSSPECAN_VAL_GSM_K10_BAND_CLASS_RGSM                                5
#define RSSPECAN_VAL_GSM_K10_BAND_CLASS_GSM                                 6

// rsspecan_rngGSMPowClass
#define RSSPECAN_VAL_GSM_K10_POW_CLASS_1                                    0
#define RSSPECAN_VAL_GSM_K10_POW_CLASS_2                                    1
#define RSSPECAN_VAL_GSM_K10_POW_CLASS_3                                    2
#define RSSPECAN_VAL_GSM_K10_POW_CLASS_4                                    3
#define RSSPECAN_VAL_GSM_K10_POW_CLASS_5                                    4
#define RSSPECAN_VAL_GSM_K10_POW_CLASS_6                                    5
#define RSSPECAN_VAL_GSM_K10_POW_CLASS_7                                    6
#define RSSPECAN_VAL_GSM_K10_POW_CLASS_8                                    7
#define RSSPECAN_VAL_GSM_K10_POW_CLASS_E1                                   8
#define RSSPECAN_VAL_GSM_K10_POW_CLASS_E2                                   9
#define RSSPECAN_VAL_GSM_K10_POW_CLASS_E3                                   10
#define RSSPECAN_VAL_GSM_K10_POW_CLASS_M1                                   11
#define RSSPECAN_VAL_GSM_K10_POW_CLASS_M2                                   12
#define RSSPECAN_VAL_GSM_K10_POW_CLASS_M3                                   13
#define RSSPECAN_VAL_GSM_K10_POW_CLASS_P1                                   14
#define RSSPECAN_VAL_GSM_K10_POW_CLASS_NONE                                 15

// rsspecan_rngGSMSync
#define RSSPECAN_VAL_GSM_K10_SYNC_ALL                                       0
#define RSSPECAN_VAL_GSM_K10_SYNC_TSC                                       1
#define RSSPECAN_VAL_GSM_K10_SYNC_BURS                                      2
#define RSSPECAN_VAL_GSM_K10_SYNC_NONE                                      3

// rsspecan_rngGSMResultDisplay
#define RSSPECAN_VAL_GSM_K10_DISPLAY_FREQ                                   0
#define RSSPECAN_VAL_GSM_K10_DISPLAY_LIST                                   1

// rsspecan_rngGSMBurstType
#define RSSPECAN_VAL_GSM_K10_BURST_NORMAL                                   0
#define RSSPECAN_VAL_GSM_K10_BURST_HIGHER_RATE                              1
#define RSSPECAN_VAL_GSM_K10_BURST_ACCESS                                   2

// rsspecan_rngGSMMod
#define RSSPECAN_VAL_GSM_K10_MOD_GMSK                                       0
#define RSSPECAN_VAL_GSM_K10_MOD_QPSK                                       1
#define RSSPECAN_VAL_GSM_K10_MOD_8PSK                                       2
#define RSSPECAN_VAL_GSM_K10_MOD_16QAM                                      3
#define RSSPECAN_VAL_GSM_K10_MOD_32QAM                                      4

// rsspecan_rngGSMFilter
#define RSSPECAN_VAL_GSM_K10_FILTER_GMSK                                    0
#define RSSPECAN_VAL_GSM_K10_FILTER_LIN_GMSK                                1
#define RSSPECAN_VAL_GSM_K10_FILTER_NARROW                                  2
#define RSSPECAN_VAL_GSM_K10_FILTER_WIDE                                    3

// rsspecan_rngGSMTSC
#define RSSPECAN_VAL_TSC_SLOT_0                                             0
#define RSSPECAN_VAL_TSC_SLOT_1                                             1
#define RSSPECAN_VAL_TSC_SLOT_2                                             2
#define RSSPECAN_VAL_TSC_SLOT_3                                             3
#define RSSPECAN_VAL_TSC_SLOT_4                                             4
#define RSSPECAN_VAL_TSC_SLOT_5                                             5
#define RSSPECAN_VAL_TSC_SLOT_6                                             6
#define RSSPECAN_VAL_TSC_SLOT_7                                             7
#define RSSPECAN_VAL_TSC_SLOT_USER                                          -1
#define RSSPECAN_VAL_TSC_SLOT_0_1                                           8
#define RSSPECAN_VAL_TSC_SLOT_1_1                                           9
#define RSSPECAN_VAL_TSC_SLOT_2_1                                           10
#define RSSPECAN_VAL_TSC_SLOT_3_1                                           11
#define RSSPECAN_VAL_TSC_SLOT_4_1                                           12
#define RSSPECAN_VAL_TSC_SLOT_5_1                                           13
#define RSSPECAN_VAL_TSC_SLOT_6_1                                           14
#define RSSPECAN_VAL_TSC_SLOT_7_1                                           15
#define RSSPECAN_VAL_TSC_SLOT_0_2                                           16
#define RSSPECAN_VAL_TSC_SLOT_1_2                                           17
#define RSSPECAN_VAL_TSC_SLOT_2_2                                           18
#define RSSPECAN_VAL_TSC_SLOT_3_2                                           19
#define RSSPECAN_VAL_TSC_SLOT_4_2                                           20
#define RSSPECAN_VAL_TSC_SLOT_5_2                                           21
#define RSSPECAN_VAL_TSC_SLOT_6_2                                           22
#define RSSPECAN_VAL_TSC_SLOT_7_2                                           23
#define RSSPECAN_VAL_TSC_SLOT_TS0                                           24
#define RSSPECAN_VAL_TSC_SLOT_TS1                                           25
#define RSSPECAN_VAL_TSC_SLOT_TS2                                           26

// rsspecan_rngGSMBurstSync
#define RSSPECAN_VAL_SYNC_USER                                              -1
#define RSSPECAN_VAL_SYNC_0                                                 0
#define RSSPECAN_VAL_SYNC_1                                                 1
#define RSSPECAN_VAL_SYNC_2                                                 2

// rsspecan_rngGSMTraceType
#define RSSPECAN_TRAC_MOD_PDF                                               1

// rsspecan_rngGSMBurstSel
#define RSSPECAN_VAL_GSM_K10_BURST_SEL_FULL                                 0
#define RSSPECAN_VAL_GSM_K10_BURST_SEL_TH_RES                               1
#define RSSPECAN_VAL_GSM_K10_BURST_SEL_RS                                   2

// rsspecan_rngGSMPvsTFilter
#define RSSPECAN_VAL_GSM_K10_FILTER_1MHZ                                    0
#define RSSPECAN_VAL_GSM_K10_FILTER_500KHZ                                  1
#define RSSPECAN_VAL_GSM_K10_FILTER_600KHZ                                  2
#define RSSPECAN_VAL_GSM_K10_FILTER_MC400                                   3
#define RSSPECAN_VAL_GSM_K10_FILTER_MC300                                   4

// rsspecan_rngGSMAlignment
#define RSSPECAN_VAL_GSM_K10_ALIGN_STM                                      0
#define RSSPECAN_VAL_GSM_K10_ALIGN_PSL                                      1

// rsspecan_rngGSMFilterType
#define RSSPECAN_VAL_GSM_K10_FILTER_TYPE_NORMAL                             0
#define RSSPECAN_VAL_GSM_K10_FILTER_TYPE_P5                                 1
#define RSSPECAN_VAL_GSM_K10_FILTER_TYPE_CFIL                               2
#define RSSPECAN_VAL_GSM_K10_FILTER_TYPE_RRC                                3
#define RSSPECAN_VAL_GSM_K10_FILTER_TYPE_CISP                               4
#define RSSPECAN_VAL_GSM_K10_FILTER_TYPE_MIL                                6

// rsspecan_rngGSMReferencePower
#define RSSPECAN_VAL_GSM_K10_REF_POWER_RMS                                  0
#define RSSPECAN_VAL_GSM_K10_REF_POWER_PEAK                                 1

// rsspecan_rngGSMFrequencyList
#define RSSPECAN_VAL_GSM_K10_FREQUENCY_LIST_NARROW                          0
#define RSSPECAN_VAL_GSM_K10_FREQUENCY_LIST_WIDE                            1

// rsspecan_rngGSMDemSymDec
#define RSSPECAN_VAL_GSM_K10_SYMBOL_DECISION_AUTO                           0
#define RSSPECAN_VAL_GSM_K10_SYMBOL_DECISION_LINEAR                         1
#define RSSPECAN_VAL_GSM_K10_SYMBOL_DECISION_SEQUENCE                       2

// rsspecan_rngGSMTailTSCBits
#define RSSPECAN_VAL_GSM_K10_STD_BITS_DETECTED                              0
#define RSSPECAN_VAL_GSM_K10_STD_BITS_STANDARD                              1

// rsspecan_rngGSMMultiCarrierFilter
#define RSSPECAN_VAL_GSM_K10_MCAR_FILT_400                                  0
#define RSSPECAN_VAL_GSM_K10_MCAR_FILT_300                                  1

// rsspecan_rngC2KLimMode
#define RSSPECAN_VAL_C2K_LIM_MODE_AUTO                                      0
#define RSSPECAN_VAL_C2K_LIM_MODE_MAN                                       1
#define RSSPECAN_VAL_C2K_LIM_MODE_USER                                      2

// rsspecan_rngC2KFeed
#define RSSPECAN_VAL_C2K_FEED_CDP                                           0
#define RSSPECAN_VAL_C2K_FEED_CDP_RAT                                       1
#define RSSPECAN_VAL_C2K_FEED_CDEP                                          2
#define RSSPECAN_VAL_C2K_FEED_ERR_CTAB                                      3
#define RSSPECAN_VAL_C2K_FEED_MACC                                          4
#define RSSPECAN_VAL_C2K_FEED_PVSL                                          5
#define RSSPECAN_VAL_C2K_FEED_PVSY                                          6
#define RSSPECAN_VAL_C2K_FEED_BSTR                                          7
#define RSSPECAN_VAL_C2K_FEED_ERR_SUMM                                      8
#define RSSPECAN_VAL_C2K_FEED_ERR_PCD                                       9
#define RSSPECAN_VAL_C2K_FEED_SYMB_CONS                                     10
#define RSSPECAN_VAL_C2K_FEED_SYMB_EVM                                      11
#define RSSPECAN_VAL_C2K_FEED_COMP_CONS                                     12
#define RSSPECAN_VAL_C2K_FEED_PVCH                                          13
#define RSSPECAN_VAL_C2K_FEED_CHAN_BSTR                                     15
#define RSSPECAN_VAL_C2K_FEED_CHAN_CONS                                     16
#define RSSPECAN_VAL_C2K_FEED_CHAN_SEVM                                     17

// rsspecan_rngC2KMeasSelect
#define RSSPECAN_VAL_C2K_MEAS_SEL_POWER                                     0
#define RSSPECAN_VAL_C2K_MEAS_SEL_ACLR                                      1
#define RSSPECAN_VAL_C2K_MEAS_SEL_MC_ACLR                                   2
#define RSSPECAN_VAL_C2K_MEAS_SEL_ESP                                       3
#define RSSPECAN_VAL_C2K_MEAS_SEL_OBW                                       4
#define RSSPECAN_VAL_C2K_MEAS_SEL_CDP                                       5
#define RSSPECAN_VAL_C2K_MEAS_SEL_CCDF                                      6
#define RSSPECAN_VAL_C2K_MEAS_SEL_PVTIME                                    7

// rsspecan_rngC2KSpreadingFactor
#define RSSPECAN_VAL_C2K_SFACTOR_64                                         0
#define RSSPECAN_VAL_C2K_SFACTOR_128                                        1

// rsspecan_rngC2KSideBand
#define RSSPECAN_VAL_C2K_SBAND_NORMAL                                       0
#define RSSPECAN_VAL_C2K_SBAND_INVERS                                       1

// rsspecan_rngC2KPowerReference
#define RSSPECAN_VAL_C2K_PREF_TOTAL                                         0
#define RSSPECAN_VAL_C2K_PREF_PICH                                          1

// rsspecan_rngC2KAntenna
#define RSSPECAN_VAL_C2K_ANTENNA_OFF                                        0
#define RSSPECAN_VAL_C2K_ANTENNA_1                                          1
#define RSSPECAN_VAL_C2K_ANTENNA_2                                          2

// rsspecan_rngC2KOrder
#define RSSPECAN_VAL_C2K_ORDER_HADAMARD                                     0
#define RSSPECAN_VAL_C2K_ORDER_BITREVERSE                                   1

// rsspecan_rngBDOLimPVTReference
#define RSSPECAN_VAL_C2K_LIM_PVT_REF_AUTO                                   0
#define RSSPECAN_VAL_C2K_LIM_PVT_REF_MAN                                    1
#define RSSPECAN_VAL_C2K_LIM_PVT_REF_ONCE                                   2

// rsspecan_rngBDOCDPowerRFSlot
#define RSSPECAN_VAL_C2K_CDP_RFSLOT_FULL                                    0
#define RSSPECAN_VAL_C2K_CDP_RFSLOT_IDLE                                    1

// rsspecan_rngBDOCDPowerMMode
#define RSSPECAN_VAL_C2K_CDP_MMODE_AUTO                                     0
#define RSSPECAN_VAL_C2K_CDP_MMODE_IOQ                                      1
#define RSSPECAN_VAL_C2K_CDP_MMODE_COMPLEX                                  2

// rsspecan_rngBDOCDPowerMapping
#define RSSPECAN_VAL_C2K_CDP_MAPPING_I                                      0
#define RSSPECAN_VAL_C2K_CDP_MAPPING_Q                                      1

// rsspecan_rngBDOCDPowerCType
#define RSSPECAN_VAL_C2K_CDP_CTYPE_PILOT                                    0
#define RSSPECAN_VAL_C2K_CDP_CTYPE_MAC                                      1
#define RSSPECAN_VAL_C2K_CDP_CTYPE_PREAMBLE                                 2
#define RSSPECAN_VAL_C2K_CDP_CTYPE_DATA                                     3

// rsspecan_rngMC2KCDPowerMapping
#define RSSPECAN_VAL_MC2K_CDP_MAPPING_I                                     0
#define RSSPECAN_VAL_MC2K_CDP_MAPPING_Q                                     1

// rsspecan_rngMDOCDPowerOperation
#define RSSPECAN_VAL_MDO_CDP_OPER_ACCESS                                    0
#define RSSPECAN_VAL_MDO_CDP_OPER_TRAFFIC                                   1

// rsspecan_rngC2KAdjustSettings
#define RSSPECAN_VAL_C2K_ACH_PRES_ACP                                       0
#define RSSPECAN_VAL_C2K_ACH_PRES_CPOW                                      1
#define RSSPECAN_VAL_C2K_ACH_PRES_MCAC                                      2
#define RSSPECAN_VAL_C2K_ACH_PRES_OBW                                       3
#define RSSPECAN_VAL_C2K_ACH_PRES_CN                                        4
#define RSSPECAN_VAL_C2K_ACH_PRES_CN0                                       5

// rsspecan_rngMC2KLCodeMode
#define RSSPECAN_VAL_MC2K_LCODE_STANDARD                                    0
#define RSSPECAN_VAL_MC2K_LCODE_ESG101                                      1

// rsspecan_rngC2KMCarrierFilterType
#define RSSPECAN_VAL_C2K_MCARRIER_FTYPE_LPASS                               0
#define RSSPECAN_VAL_C2K_MCARRIER_FTYPE_RRC                                 1

// rsspecan_rngBDORevision
#define RSSPECAN_VAL_BDO_REVISION_0                                         0
#define RSSPECAN_VAL_BDO_REVISION_A                                         1
#define RSSPECAN_VAL_BDO_REVISION_B                                         2

// rsspecan_rngBDOSubtype
#define RSSPECAN_VAL_BDO_SUBTYPE_01                                         0
#define RSSPECAN_VAL_BDO_SUBTYPE_2                                          2
#define RSSPECAN_VAL_BDO_SUBTYPE_3                                          3

// rsspecan_rngC2KResultType
#define RSSPECAN_VAL_C2K_RESULT_TYPE_ACTIVE                                 0
#define RSSPECAN_VAL_C2K_RESULT_TYPE_CDP_ABS                                1
#define RSSPECAN_VAL_C2K_RESULT_TYPE_CDP_REL                                2
#define RSSPECAN_VAL_C2K_RESULT_TYPE_CERR                                   3
#define RSSPECAN_VAL_C2K_RESULT_TYPE_CHAN                                   4
#define RSSPECAN_VAL_C2K_RESULT_TYPE_DMT                                    5
#define RSSPECAN_VAL_C2K_RESULT_TYPE_EVMR                                   6
#define RSSPECAN_VAL_C2K_RESULT_TYPE_EVMP                                   7
#define RSSPECAN_VAL_C2K_RESULT_TYPE_FERP                                   8
#define RSSPECAN_VAL_C2K_RESULT_TYPE_FERR                                   9
#define RSSPECAN_VAL_C2K_RESULT_TYPE_IQIM                                   10
#define RSSPECAN_VAL_C2K_RESULT_TYPE_IQOF                                   11
#define RSSPECAN_VAL_C2K_RESULT_TYPE_MACC                                   12
#define RSSPECAN_VAL_C2K_RESULT_TYPE_PCD                                    13
#define RSSPECAN_VAL_C2K_RESULT_TYPE_POFF                                   14
#define RSSPECAN_VAL_C2K_RESULT_TYPE_PPIC                                   15
#define RSSPECAN_VAL_C2K_RESULT_TYPE_PTOT                                   16
#define RSSPECAN_VAL_C2K_RESULT_TYPE_RHO                                    17
#define RSSPECAN_VAL_C2K_RESULT_TYPE_SFAC                                   18
#define RSSPECAN_VAL_C2K_RESULT_TYPE_SLOT                                   19
#define RSSPECAN_VAL_C2K_RESULT_TYPE_SRAT                                   20
#define RSSPECAN_VAL_C2K_RESULT_TYPE_TFR                                    21
#define RSSPECAN_VAL_C2K_RESULT_TYPE_TOFF                                   22
#define RSSPECAN_VAL_C2K_RESULT_TYPE_DACT                                   23
#define RSSPECAN_VAL_C2K_RESULT_TYPE_IPMM                                   24
#define RSSPECAN_VAL_C2K_RESULT_TYPE_MTYP                                   25
#define RSSPECAN_VAL_C2K_RESULT_TYPE_PDAT                                   26
#define RSSPECAN_VAL_C2K_RESULT_TYPE_PDMAX                                  27
#define RSSPECAN_VAL_C2K_RESULT_TYPE_PDMIN                                  28
#define RSSPECAN_VAL_C2K_RESULT_TYPE_PLENG                                  29
#define RSSPECAN_VAL_C2K_RESULT_TYPE_PMAC                                   30
#define RSSPECAN_VAL_C2K_RESULT_TYPE_PPIL                                   31
#define RSSPECAN_VAL_C2K_RESULT_TYPE_PPR                                    32
#define RSSPECAN_VAL_C2K_RESULT_TYPE_MACT                                   33
#define RSSPECAN_VAL_C2K_RESULT_TYPE_RHO1                                   34
#define RSSPECAN_VAL_C2K_RESULT_TYPE_RHO2                                   35
#define RSSPECAN_VAL_C2K_RESULT_TYPE_RHOD                                   36
#define RSSPECAN_VAL_C2K_RESULT_TYPE_RHOM                                   37
#define RSSPECAN_VAL_C2K_RESULT_TYPE_RHOP                                   38
#define RSSPECAN_VAL_C2K_RESULT_TYPE_RHOV                                   39
#define RSSPECAN_VAL_C2K_RESULT_TYPE_CDER                                   40
#define RSSPECAN_VAL_C2K_RESULT_TYPE_CDEP                                   41
#define RSSPECAN_VAL_C2K_RESULT_TYPE_CODM                                   42
#define RSSPECAN_VAL_C2K_RESULT_TYPE_CODP                                   43
#define RSSPECAN_VAL_C2K_RESULT_TYPE_DRP                                    44
#define RSSPECAN_VAL_C2K_RESULT_TYPE_PRRI                                   45

// rsspecan_rng3GPPFeed
#define RSSPECAN_VAL_3GPP_RES_CDP                                           0
#define RSSPECAN_VAL_3GPP_RES_CDPRAT                                        1
#define RSSPECAN_VAL_3GPP_RES_CDPOVER                                       2
#define RSSPECAN_VAL_3GPP_RES_CDEP                                          3
#define RSSPECAN_VAL_3GPP_RES_SUM                                           4
#define RSSPECAN_VAL_3GPP_RES_CTAB                                          5
#define RSSPECAN_VAL_3GPP_RES_PCD                                           6
#define RSSPECAN_VAL_3GPP_RES_MACC                                          7
#define RSSPECAN_VAL_3GPP_RES_PVSLA                                         8
#define RSSPECAN_VAL_3GPP_RES_PVSLR                                         9
#define RSSPECAN_VAL_3GPP_RES_BSTR                                          10
#define RSSPECAN_VAL_3GPP_RES_PVSY                                          11
#define RSSPECAN_VAL_3GPP_RES_SYMBCONST                                     12
#define RSSPECAN_VAL_3GPP_RES_SYMBEVM                                       13
#define RSSPECAN_VAL_3GPP_RES_FVSL                                          14
#define RSSPECAN_VAL_3GPP_RES_PSVSL                                         15
#define RSSPECAN_VAL_3GPP_RES_EVM_MAGN                                      16
#define RSSPECAN_VAL_3GPP_RES_EVM_PHAS                                      17
#define RSSPECAN_VAL_3GPP_RES_COMP_CONS                                     18
#define RSSPECAN_VAL_3GPP_RES_CHIP_EVM                                      19
#define RSSPECAN_VAL_3GPP_RES_CHIP_MAGN                                     20
#define RSSPECAN_VAL_3GPP_RES_CHIP_PHAS                                     21

// rsspecan_rng3GUEFeed
#define RSSPECAN_VAL_3GUE_RES_CDP                                           0
#define RSSPECAN_VAL_3GUE_RES_CDPRAT                                        1
#define RSSPECAN_VAL_3GUE_RES_CDPOVER                                       2
#define RSSPECAN_VAL_3GUE_RES_CDEP                                          3
#define RSSPECAN_VAL_3GUE_RES_SUM                                           4
#define RSSPECAN_VAL_3GUE_RES_CTAB                                          5
#define RSSPECAN_VAL_3GUE_RES_PCD                                           6
#define RSSPECAN_VAL_3GUE_RES_MACC                                          7
#define RSSPECAN_VAL_3GUE_RES_PVSLA                                         8
#define RSSPECAN_VAL_3GUE_RES_PVSLR                                         9
#define RSSPECAN_VAL_3GUE_RES_BSTR                                          10
#define RSSPECAN_VAL_3GUE_RES_PVSY                                          11
#define RSSPECAN_VAL_3GUE_RES_SYMBCONST                                     12
#define RSSPECAN_VAL_3GUE_RES_SYMBEVM                                       13
#define RSSPECAN_VAL_3GUE_RES_FVSL                                          14
#define RSSPECAN_VAL_3GUE_RES_PSVSL                                         15
#define RSSPECAN_VAL_3GUE_RES_EVM_MAGN                                      16
#define RSSPECAN_VAL_3GUE_RES_EVM_PHAS                                      17
#define RSSPECAN_VAL_3GUE_RES_COMP_CONS                                     18
#define RSSPECAN_VAL_3GUE_RES_CHIP_EVM                                      19
#define RSSPECAN_VAL_3GUE_RES_CHIP_MAGN                                     20
#define RSSPECAN_VAL_3GUE_RES_CHIP_PHAS                                     21

// rsspecan_rng3GUECDPBase
#define RSSPECAN_VAL_3GUE_CDP_BASE_SLOT                                     0
#define RSSPECAN_VAL_3GUE_CDP_BASE_FRAME                                    1

// rsspecan_rngIQ
#define RSSPECAN_VAL_I                                                      0
#define RSSPECAN_VAL_Q                                                      1

// rsspecan_rng3GUECDPScrType
#define RSSPECAN_VAL_3GUE_CDP_SCR_LONG                                      0
#define RSSPECAN_VAL_3GUE_CDP_SCR_SHORT                                     1

// rsspecan_rng3GPPACLRMode
#define RSSPECAN_VAL_3GPP_BS_ACLR_MODE_ABS                                  0
#define RSSPECAN_VAL_3GPP_BS_ACLR_MODE_REL                                  1

// rsspecan_rng3GPPACLRRef
#define RSSPECAN_VAL_3GPP_BS_ACLR_CPACP_CONF_ACP_REF_SETTINGS_MIN           0
#define RSSPECAN_VAL_3GPP_BS_ACLR_CPACP_CONF_ACP_REF_SETTINGS_MAX           1
#define RSSPECAN_VAL_3GPP_BS_ACLR_CPACP_CONF_ACP_REF_SETTINGS_LHIG          2

// rsspecan_rng3GBSCompareMode
#define RSSPECAN_VAL_3GPP_BS_SETTINGS_COMP_MODE_PILOT                       0
#define RSSPECAN_VAL_3GPP_BS_SETTINGS_COMP_MODE_SLOT                        1

// rsspecan_rng3GPPSBand
#define RSSPECAN_VAL_3GPP_SBAND_NORM                                        0
#define RSSPECAN_VAL_3GPP_SBAND_INV                                         1

// rsspecan_rng3GBSPowRef
#define RSSPECAN_VAL_3GPP_BS_SETTINGS_POW_REF_TOT                           0
#define RSSPECAN_VAL_3GPP_BS_SETTINGS_POW_REF_CPIC                          1

// rsspecan_rng3GBSSyncType
#define RSSPECAN_VAL_3GPP_BS_SETTINGS_SYNC_TYPE_CPIC                        0
#define RSSPECAN_VAL_3GPP_BS_SETTINGS_SYNC_TYPE_SCH                         1

// rsspecan_rng3GBSAntenna
#define RSSPECAN_VAL_3GPP_BS_SETTINGS_ANTENNA_OFF                           0
#define RSSPECAN_VAL_3GPP_BS_SETTINGS_ANTENNA_1                             1
#define RSSPECAN_VAL_3GPP_BS_SETTINGS_ANTENNA_2                             2

// rsspecan_rng3GUEMeas
#define RSSPECAN_VAL_3GPPUE_MEAS_POW                                        0
#define RSSPECAN_VAL_3GPPUE_MEAS_ACLR                                       1
#define RSSPECAN_VAL_3GPPUE_MEAS_ESP                                        2
#define RSSPECAN_VAL_3GPPUE_MEAS_OBW                                        3
#define RSSPECAN_VAL_3GPPUE_MEAS_WCDP                                       4
#define RSSPECAN_VAL_3GPPUE_MEAS_CCDF                                       5

// rsspecan_rng3GBSMeas
#define RSSPECAN_VAL_3GPPBS_MEAS_POW                                        0
#define RSSPECAN_VAL_3GPPBS_MEAS_ACLR                                       1
#define RSSPECAN_VAL_3GPPBS_MEAS_ESP                                        2
#define RSSPECAN_VAL_3GPPBS_MEAS_OBW                                        3
#define RSSPECAN_VAL_3GPPBS_MEAS_WCDP                                       4
#define RSSPECAN_VAL_3GPPBS_MEAS_CCDF                                       5
#define RSSPECAN_VAL_3GPPBS_MEAS_MCAC                                       6
#define RSSPECAN_VAL_3GPPBS_MEAS_RFC                                        7
#define RSSPECAN_VAL_3GPPBS_MEAS_TAER                                       10

// rsspecan_rng3GBSTOffset
#define RSSPECAN_VAL_3GPP_TOFFSET_PRED                                      0
#define RSSPECAN_VAL_3GPP_TOFFSET_MEAS                                      1

// rsspecan_rng3GBSFeed
#define RSSPECAN_VAL_3GBS_RES_CDP                                           0
#define RSSPECAN_VAL_3GBS_RES_CDPRAT                                        1
#define RSSPECAN_VAL_3GBS_RES_CDEP                                          2
#define RSSPECAN_VAL_3GBS_RES_SUM                                           3
#define RSSPECAN_VAL_3GBS_RES_CTAB                                          4
#define RSSPECAN_VAL_3GBS_RES_PCD                                           5
#define RSSPECAN_VAL_3GBS_RES_MACC                                          6
#define RSSPECAN_VAL_3GBS_RES_PVSLA                                         7
#define RSSPECAN_VAL_3GBS_RES_PVSLR                                         8
#define RSSPECAN_VAL_3GBS_RES_BSTR                                          9
#define RSSPECAN_VAL_3GBS_RES_SYMBCONST                                     10
#define RSSPECAN_VAL_3GBS_RES_SYMBEVM                                       11
#define RSSPECAN_VAL_3GBS_RES_FVSL                                          12
#define RSSPECAN_VAL_3GBS_RES_PVSY                                          13
#define RSSPECAN_VAL_3GBS_RES_EVM_MAGN                                      14
#define RSSPECAN_VAL_3GBS_RES_EVM_PHAS                                      15
#define RSSPECAN_VAL_3GBS_RES_COMP_CONS                                     16
#define RSSPECAN_VAL_3GBS_RES_CHIP_EVM                                      17
#define RSSPECAN_VAL_3GBS_RES_CHIP_MAGN                                     18
#define RSSPECAN_VAL_3GBS_RES_CHIP_PHAS                                     19
#define RSSPECAN_VAL_3GBS_RES_PSVSL                                         20
#define RSSPECAN_VAL_3GBS_RES_CDPOVER                                       21

// rsspecan_rng3GPPResultSumView
#define RSSPECAN_VAL_3GPP_RES_SUM_NORMAL                                    0
#define RSSPECAN_VAL_3GPP_RES_SUM_EXTENDED                                  1

// rsspecan_rng3GBSStandard
#define RSSPECAN_VAL_3GPP_BS_HOME                                           0
#define RSSPECAN_VAL_3GPP_BS_NORMAL                                         1

// rsspecan_rng3GPPDataTraceAdditional
#define RSSPECAN_VAL_3GPP_DATA_CTAB                                         0
#define RSSPECAN_VAL_3GPP_DATA_CWCD                                         1
#define RSSPECAN_VAL_3GPP_DATA_LIST                                         2
#define RSSPECAN_VAL_3GPP_DATA_PWCD                                         3
#define RSSPECAN_VAL_3GPP_DATA_TPVS                                         4
#define RSSPECAN_VAL_3GPP_DATA_FINAL                                        5
#define RSSPECAN_VAL_3GPP_DATA_CEVM                                         6
#define RSSPECAN_VAL_3GPP_DATA_ABIT                                         7

// rsspecan_rng3GPPDataTrace
#define RSSPECAN_VAL_3GPP_DATA_TRACE                                        0
#define RSSPECAN_VAL_3GPP_DATA_ATRACE                                       1

// rsspecan_rng3GBSResults
#define RSSPECAN_VAL_WCDP_RES_PTOT                                          0
#define RSSPECAN_VAL_WCDP_RES_FERR                                          1
#define RSSPECAN_VAL_WCDP_RES_TFR                                           2
#define RSSPECAN_VAL_WCDP_RES_MACC                                          3
#define RSSPECAN_VAL_WCDP_RES_PCD                                           4
#define RSSPECAN_VAL_WCDP_RES_EVMR                                          5
#define RSSPECAN_VAL_WCDP_RES_EVMP                                          6
#define RSSPECAN_VAL_WCDP_RES_CERR                                          7
#define RSSPECAN_VAL_WCDP_RES_CSL                                           8
#define RSSPECAN_VAL_WCDP_RES_SRAT                                          9
#define RSSPECAN_VAL_WCDP_RES_CHAN                                          10
#define RSSPECAN_VAL_WCDP_RES_CDPA                                          11
#define RSSPECAN_VAL_WCDP_RES_CDPR                                          12
#define RSSPECAN_VAL_WCDP_RES_IQOF                                          13
#define RSSPECAN_VAL_WCDP_RES_IQIM                                          14
#define RSSPECAN_VAL_WCDP_RES_MTYP                                          15
#define RSSPECAN_VAL_WCDP_RES_RHO                                           16
#define RSSPECAN_VAL_WCDP_RES_TOFF                                          17
#define RSSPECAN_VAL_WCDP_RES_PSYM                                          18
#define RSSPECAN_VAL_WCDP_RES_ACH                                           19
#define RSSPECAN_VAL_WCDP_RES_MPIC                                          20
#define RSSPECAN_VAL_WCDP_RES_RCDE                                          21
#define RSSPECAN_VAL_WCDP_RES_ARCD                                          22
#define RSSPECAN_VAL_WCDP_RES_IOFF                                          23
#define RSSPECAN_VAL_WCDP_RES_QOFF                                          24
#define RSSPECAN_VAL_WCDP_RES_CMAP                                          25

// rsspecan_rng3GPPTaerResult
#define RSSPECAN_VAL_TAER_RES_TAER                                          0

// rsspecan_rngTDSFeed
#define RSSPECAN_VAL_TDS_RES_CDP                                            0
#define RSSPECAN_VAL_TDS_RES_CDPRAT                                         1
#define RSSPECAN_VAL_TDS_RES_CDEP                                           2
#define RSSPECAN_VAL_TDS_RES_MACC                                           3
#define RSSPECAN_VAL_TDS_RES_PVSLA                                          4
#define RSSPECAN_VAL_TDS_RES_PVSLR                                          5
#define RSSPECAN_VAL_TDS_RES_PVSYM                                          6
#define RSSPECAN_VAL_TDS_RES_BSTR                                           7
#define RSSPECAN_VAL_TDS_RES_SUM                                            8
#define RSSPECAN_VAL_TDS_RES_CTAB                                           9
#define RSSPECAN_VAL_TDS_RES_PCD                                            10
#define RSSPECAN_VAL_TDS_RES_SYMBCONST                                      11
#define RSSPECAN_VAL_TDS_RES_SYMBEVM                                        12
#define RSSPECAN_VAL_TDS_RES_COMPCONST                                      13
#define RSSPECAN_VAL_TDS_RES_COMPEVM                                        14

// rsspecan_rngTDSCtabOrder
#define RSSPECAN_VAL_TDS_CTAB_ORDER_CODE                                    0
#define RSSPECAN_VAL_TDS_CTAB_ORDER_MID                                     1

// rsspecan_rngTDSSideBand
#define RSSPECAN_VAL_TDS_CDP_SBAND_NORM                                     0
#define RSSPECAN_VAL_TDS_CDP_SBAND_INV                                      1

// rsspecan_rngTDSUEMeas
#define RSSPECAN_VAL_TDS_MEAS_POW                                           0
#define RSSPECAN_VAL_TDS_MEAS_ACLR                                          1
#define RSSPECAN_VAL_TDS_MEAS_ESP                                           2
#define RSSPECAN_VAL_TDS_MEAS_OBAN                                          3
#define RSSPECAN_VAL_TDS_MEAS_PVT                                           4
#define RSSPECAN_VAL_TDS_MEAS_CDP                                           5
#define RSSPECAN_VAL_TDS_MEAS_CCDF                                          6

// rsspecan_rngTDBSMaxMode
#define RSSPECAN_VAL_TDS_MMAX_QPSK                                          0
#define RSSPECAN_VAL_TDS_MMAX_8PSK                                          1
#define RSSPECAN_VAL_TDS_MMAX_16QAM                                         2
#define RSSPECAN_VAL_TDS_MMAX_64QAM                                         3

// rsspecan_rngTDSSyncToSlotMode
#define RSSPECAN_VAL_TDS_STS_CODE                                           0
#define RSSPECAN_VAL_TDS_STS_MA                                             1

// rsspecan_rngTDSResultType
#define RSSPECAN_VAL_TDS_RESULT_SLOT                                        0
#define RSSPECAN_VAL_TDS_RESULT_PDAT                                        1
#define RSSPECAN_VAL_TDS_RESULT_PD1                                         2
#define RSSPECAN_VAL_TDS_RESULT_PD2                                         3
#define RSSPECAN_VAL_TDS_RESULT_PMID                                        4
#define RSSPECAN_VAL_TDS_RESULT_RHO                                         5
#define RSSPECAN_VAL_TDS_RESULT_MACC                                        6
#define RSSPECAN_VAL_TDS_RESULT_PCD                                         7
#define RSSPECAN_VAL_TDS_RESULT_FERR                                        8
#define RSSPECAN_VAL_TDS_RESULT_CERR                                        9
#define RSSPECAN_VAL_TDS_RESULT_TFR                                         10
#define RSSPECAN_VAL_TDS_RESULT_IQIM                                        11
#define RSSPECAN_VAL_TDS_RESULT_IQOF                                        12
#define RSSPECAN_VAL_TDS_RESULT_ACT                                         13
#define RSSPECAN_VAL_TDS_RESULT_SRAT                                        14
#define RSSPECAN_VAL_TDS_RESULT_CHAN                                        15
#define RSSPECAN_VAL_TDS_RESULT_SFAC                                        16
#define RSSPECAN_VAL_TDS_RESULT_CDPR                                        17
#define RSSPECAN_VAL_TDS_RESULT_CDP                                         18
#define RSSPECAN_VAL_TDS_RESULT_EVMR                                        19
#define RSSPECAN_VAL_TDS_RESULT_EVMP                                        20
#define RSSPECAN_VAL_TDS_RESULT_ARCDE                                       21
#define RSSPECAN_VAL_TDS_RESULT_DACT                                        22
#define RSSPECAN_VAL_TDS_RESULT_DPOW                                        23
#define RSSPECAN_VAL_TDS_RESULT_DRHO                                        24
#define RSSPECAN_VAL_TDS_RESULT_DERM                                        25
#define RSSPECAN_VAL_TDS_RESULT_DEPK                                        26
#define RSSPECAN_VAL_TDS_RESULT_SFR                                         27
#define RSSPECAN_VAL_TDS_RESULT_UACT                                        28
#define RSSPECAN_VAL_TDS_RESULT_UPOW                                        29
#define RSSPECAN_VAL_TDS_RESULT_URHO                                        30
#define RSSPECAN_VAL_TDS_RESULT_UERM                                        31
#define RSSPECAN_VAL_TDS_RESULT_UEPK                                        32

// rsspecan_rngLTEDuplexing
#define RSSPECAN_VAL_LTE_DUPLEXING_TDD                                      0
#define RSSPECAN_VAL_LTE_DUPLEXING_FDD                                      1

// rsspecan_rngLTELinkDirection
#define RSSPECAN_VAL_LTE_DOWNLINK                                           0
#define RSSPECAN_VAL_LTE_UPLINK                                             1

// rsspecan_rngLTEDownlinkMIMOConfiguration
#define RSSPECAN_VAL_LTE_DOWNLINK_MIMO_TX1                                  0
#define RSSPECAN_VAL_LTE_DOWNLINK_MIMO_TX2                                  1
#define RSSPECAN_VAL_LTE_DOWNLINK_MIMO_TX4                                  2

// rsspecan_rngLTEDownlinkBandwidth
#define RSSPECAN_VAL_LTE_DOWNLINK_BW1_40                                    0
#define RSSPECAN_VAL_LTE_DOWNLINK_BW3_00                                    2
#define RSSPECAN_VAL_LTE_DOWNLINK_BW5_00                                    3
#define RSSPECAN_VAL_LTE_DOWNLINK_BW10_00                                   4
#define RSSPECAN_VAL_LTE_DOWNLINK_BW15_00                                   5
#define RSSPECAN_VAL_LTE_DOWNLINK_BW20_00                                   6

// rsspecan_rngLTEDownlinkCyclicPrefix
#define RSSPECAN_VAL_LTE_DOWNLINK_CPREFIX_NORMAL                            0
#define RSSPECAN_VAL_LTE_DOWNLINK_CPREFIX_EXTENDED                          1
#define RSSPECAN_VAL_LTE_DOWNLINK_CPREFIX_AUTO                              2

// rsspecan_rngLTEDownlinkMIMOAntenna
#define RSSPECAN_VAL_LTE_DOWNLINK_MIMO_ANT_ALL                              0
#define RSSPECAN_VAL_LTE_DOWNLINK_MIMO_ANT1                                 1
#define RSSPECAN_VAL_LTE_DOWNLINK_MIMO_ANT2                                 2
#define RSSPECAN_VAL_LTE_DOWNLINK_MIMO_ANT3                                 3
#define RSSPECAN_VAL_LTE_DOWNLINK_MIMO_ANT4                                 4
#define RSSPECAN_VAL_LTE_DOWNLINK_MIMO_AUT1                                 5
#define RSSPECAN_VAL_LTE_DOWNLINK_MIMO_AUT2                                 6
#define RSSPECAN_VAL_LTE_DOWNLINK_MIMO_AUT4                                 7
#define RSSPECAN_VAL_LTE_DOWNLINK_MIMO_AUTO                                 8

// rsspecan_rngLTEDownlinkChannelEstimation
#define RSSPECAN_VAL_LTE_DOWNLINK_CEST_TGPP                                 0
#define RSSPECAN_VAL_LTE_DOWNLINK_CEST_PILOT                                1
#define RSSPECAN_VAL_LTE_DOWNLINK_CEST_PILPAY                               2
#define RSSPECAN_VAL_LTE_DOWNLINK_CEST_OFF                                  3

// rsspecan_rngLTEDownlinkEVMCalculation
#define RSSPECAN_ATTR_LTE_DOWNLINK_EVM_CALC_TGPP                            0
#define RSSPECAN_ATTR_LTE_DOWNLINK_EVM_CALC_OTP                             1

// rsspecan_rngLTEDownlinkPhaseTracking
#define RSSPECAN_VAL_LTE_DOWNLINK_PHASE_TRACKING_OFF                        0
#define RSSPECAN_VAL_LTE_DOWNLINK_PHASE_TRACKING_PILOT                      1
#define RSSPECAN_VAL_LTE_DOWNLINK_PHASE_TRACKING_PILPAY                     2

// rsspecan_rngLTEDownlinkModulation
#define RSSPECAN_VAL_LTE_DOWNLINK_MODULATION_QPSK                           0
#define RSSPECAN_VAL_LTE_DOWNLINK_MODULATION_16QAM                          1
#define RSSPECAN_VAL_LTE_DOWNLINK_MODULATION_64QAM                          2
#define RSSPECAN_VAL_LTE_DOWNLINK_MODULATION_256QAM                         3
#define RSSPECAN_VAL_LTE_DOWNLINK_MODULATION_1024QAM                        4

// rsspecan_rngLTEDownlinkPHICHDuration
#define RSSPECAN_VAL_LTE_DOWNLINK_PHICH_DURATION_NORMAL                     0
#define RSSPECAN_VAL_LTE_DOWNLINK_PHICH_DURATION_EXTENDED                   1

// rsspecan_rngLTEDownlinkBitStreamFormat
#define RSSPECAN_VAL_LTE_DOWNLINK_STREAM_SYMBOLS                            0
#define RSSPECAN_VAL_LTE_DOWNLINK_STREAM_BITS                               1

// rsspecan_rngLTEDownlinkSEMChannel
#define RSSPECAN_VAL_LTE_DOWNLINK_CATEGORY_A                                0
#define RSSPECAN_VAL_LTE_DOWNLINK_CATEGORY_B_1                              1
#define RSSPECAN_VAL_LTE_DOWNLINK_CATEGORY_B_2                              2
#define RSSPECAN_VAL_LTE_DOWNLINK_CATEGORY_HOME                             3
#define RSSPECAN_VAL_LTE_DOWNLINK_CATEGORY_LARE                             4
#define RSSPECAN_VAL_LTE_DOWNLINK_CATEGORY_MED                              5

// rsspecan_rngLTEDownlinkAssumedCarrier
#define RSSPECAN_VAL_LTE_DOWNLINK_ASSUMED_EUTRA                             0
#define RSSPECAN_VAL_LTE_DOWNLINK_ASSUMED_UTRA128                           1
#define RSSPECAN_VAL_LTE_DOWNLINK_ASSUMED_UTRA384                           2
#define RSSPECAN_VAL_LTE_DOWNLINK_ASSUMED_UTRA768                           3

// rsspecan_rngLTEDownlinkMeasurementType
#define RSSPECAN_VAL_LTE_DOWNLINK_CBUF                                      0
#define RSSPECAN_VAL_LTE_DOWNLINK_EVCA                                      1
#define RSSPECAN_VAL_LTE_DOWNLINK_EVSY                                      2
#define RSSPECAN_VAL_LTE_DOWNLINK_FEVS                                      3
#define RSSPECAN_VAL_LTE_DOWNLINK_EVSU                                      4
#define RSSPECAN_VAL_LTE_DOWNLINK_SEM                                       5
#define RSSPECAN_VAL_LTE_DOWNLINK_ACP                                       6
#define RSSPECAN_VAL_LTE_DOWNLINK_PSPE                                      7
#define RSSPECAN_VAL_LTE_DOWNLINK_PVRP                                      8
#define RSSPECAN_VAL_LTE_DOWNLINK_PVRR                                      9
#define RSSPECAN_VAL_LTE_DOWNLINK_FLAT                                      10
#define RSSPECAN_VAL_LTE_DOWNLINK_GDEL                                      11
#define RSSPECAN_VAL_LTE_DOWNLINK_FDIF                                      12
#define RSSPECAN_VAL_LTE_DOWNLINK_CONS                                      13
#define RSSPECAN_VAL_LTE_DOWNLINK_DFTC                                      14
#define RSSPECAN_VAL_LTE_DOWNLINK_CCDF                                      15
#define RSSPECAN_VAL_LTE_DOWNLINK_SFLO                                      16
#define RSSPECAN_VAL_LTE_DOWNLINK_ASUM                                      17
#define RSSPECAN_VAL_LTE_DOWNLINK_BSTR                                      18
#define RSSPECAN_VAL_LTE_DOWNLINK_EVRP                                      19
#define RSSPECAN_VAL_LTE_DOWNLINK_OOP                                       20
#define RSSPECAN_VAL_LTE_DOWNLINK_TAER                                      21

// rsspecan_rngLTEDownlinkConstellationLocation
#define RSSPECAN_VAL_LTE_DOWNLINK_LOC_AMD                                   0
#define RSSPECAN_VAL_LTE_DOWNLINK_LOC_BMD                                   1

// rsspecan_rngLTEUplinkBandwidth
#define RSSPECAN_VAL_LTE_UPLINK_BW1_40                                      0
#define RSSPECAN_VAL_LTE_UPLINK_BW3_00                                      2
#define RSSPECAN_VAL_LTE_UPLINK_BW5_00                                      3
#define RSSPECAN_VAL_LTE_UPLINK_BW10_00                                     4
#define RSSPECAN_VAL_LTE_UPLINK_BW15_00                                     5
#define RSSPECAN_VAL_LTE_UPLINK_BW20_00                                     6

// rsspecan_rngLTEUplinkCyclicPrefix
#define RSSPECAN_VAL_LTE_UPLINK_CPREFIX_NORMAL                              0
#define RSSPECAN_VAL_LTE_UPLINK_CPREFIX_EXTENDED                            1
#define RSSPECAN_VAL_LTE_UPLINK_CPREFIX_AUTO                                2

// rsspecan_rngLTEUplinkChannelEstimation
#define RSSPECAN_VAL_LTE_UPLINK_CEST_PILOT                                  0
#define RSSPECAN_VAL_LTE_UPLINK_CEST_PILPAY                                 1

// rsspecan_rngLTEUplinkPhaseTracking
#define RSSPECAN_VAL_LTE_UPLINK_PHASE_TRACKING_OFF                          0
#define RSSPECAN_VAL_LTE_UPLINK_PHASE_TRACKING_PILOT                        1
#define RSSPECAN_VAL_LTE_UPLINK_PHASE_TRACKING_PILPAY                       2

// rsspecan_rngLTEUplinkSubframeEnable
#define RSSPECAN_VAL_LTE_UPLINK_SUBFRAME_PUSCH                              0
#define RSSPECAN_VAL_LTE_UPLINK_SUBFRAME_PUCCH                              1
#define RSSPECAN_VAL_LTE_UPLINK_SUBFRAME_PSCC                               2
#define RSSPECAN_VAL_LTE_UPLINK_SUBFRAME_NONE                               3

// rsspecan_rngLTEUplinkModulation
#define RSSPECAN_VAL_LTE_UPLINK_MODULATION_QPSK                             0
#define RSSPECAN_VAL_LTE_UPLINK_MODULATION_16QAM                            1
#define RSSPECAN_VAL_LTE_UPLINK_MODULATION_64QAM                            2
#define RSSPECAN_VAL_LTE_UPLINK_MODULATION_256QAM                           3

// rsspecan_rngLTEUplinkBitStreamFormat
#define RSSPECAN_VAL_LTE_UPLINK_STREAM_SYMBOLS                              0
#define RSSPECAN_VAL_LTE_UPLINK_STREAM_BITS                                 1

// rsspecan_rngLTEUplinkSEMChannel
#define RSSPECAN_VAL_LTE_UPLINK_CATEGORY_A                                  0
#define RSSPECAN_VAL_LTE_UPLINK_CATEGORY_B                                  1

// rsspecan_rngLTEUplinkAssumedCarrier
#define RSSPECAN_VAL_LTE_UPLINK_ASSUMED_EUTRA                               0
#define RSSPECAN_VAL_LTE_UPLINK_ASSUMED_UTRA128                             1
#define RSSPECAN_VAL_LTE_UPLINK_ASSUMED_UTRA384                             2
#define RSSPECAN_VAL_LTE_UPLINK_ASSUMED_UTRA768                             3

// rsspecan_rngLTEUplinkMeasurementType
#define RSSPECAN_VAL_LTE_UPLINK_CBUF                                        0
#define RSSPECAN_VAL_LTE_UPLINK_EVCA                                        1
#define RSSPECAN_VAL_LTE_UPLINK_EVSY                                        2
#define RSSPECAN_VAL_LTE_UPLINK_FEVS                                        3
#define RSSPECAN_VAL_LTE_UPLINK_EVSU                                        4
#define RSSPECAN_VAL_LTE_UPLINK_SEM                                         5
#define RSSPECAN_VAL_LTE_UPLINK_ACP                                         6
#define RSSPECAN_VAL_LTE_UPLINK_PSPE                                        7
#define RSSPECAN_VAL_LTE_UPLINK_FLAT                                        8
#define RSSPECAN_VAL_LTE_UPLINK_GDEL                                        9
#define RSSPECAN_VAL_LTE_UPLINK_FDIF                                        10
#define RSSPECAN_VAL_LTE_UPLINK_IE                                          11
#define RSSPECAN_VAL_LTE_UPLINK_CONS                                        12
#define RSSPECAN_VAL_LTE_UPLINK_DFTC                                        13
#define RSSPECAN_VAL_LTE_UPLINK_CCDF                                        14
#define RSSPECAN_VAL_LTE_UPLINK_ASUM                                        15
#define RSSPECAN_VAL_LTE_UPLINK_BSTR                                        16
#define RSSPECAN_VAL_LTE_UPLINK_TAER                                        17

// rsspecan_rngLTEUplinkPUSCHFreqHoppingMode
#define RSSPECAN_VAL_LTE_UPLINK_PUSCH_FREQUENCY_HOPPING_OFF                 0
#define RSSPECAN_VAL_LTE_UPLINK_PUSCH_FREQUENCY_HOPPING_INTER               1
#define RSSPECAN_VAL_LTE_UPLINK_PUSCH_FREQUENCY_HOPPING_INTRA               2

// rsspecan_rngLTEUplinkPUCCHFormat
#define RSSPECAN_VAL_LTE_UPLINK_PUCCH_FORMAT_F2                             6
#define RSSPECAN_VAL_LTE_UPLINK_PUCCH_FORMAT_F2A                            7
#define RSSPECAN_VAL_LTE_UPLINK_PUCCH_FORMAT_F2B                            8
#define RSSPECAN_VAL_LTE_UPLINK_PUCCH_FORMAT_F3                             9
#define RSSPECAN_VAL_LTE_UPLINK_PUCCH_FORMAT_F1                             10
#define RSSPECAN_VAL_LTE_UPLINK_PUCCH_FORMAT_F1A                            11
#define RSSPECAN_VAL_LTE_UPLINK_PUCCH_FORMAT_F1B                            12
#define RSSPECAN_VAL_LTE_UPLINK_PUCCH_FORMAT_SUBF                           13

// rsspecan_rngLTESymbolOffset
#define RSSPECAN_VAL_LTE_SYMBOL_OFFSET_0                                    0
#define RSSPECAN_VAL_LTE_SYMBOL_OFFSET_1                                    1
#define RSSPECAN_VAL_LTE_SYMBOL_OFFSET_2                                    2
#define RSSPECAN_VAL_LTE_SYMBOL_OFFSET_3                                    3
#define RSSPECAN_VAL_LTE_SYMBOL_OFFSET_4                                    4
#define RSSPECAN_VAL_LTE_SYMBOL_OFFSET_AUTO                                 5

// rsspecan_rngDL_PDSCHSubframeDetect
#define RSSPECAN_VAL_LTE_DOWNLINK_PDSCH_SUBFRAME_DETECT_OFF                 0
#define RSSPECAN_VAL_LTE_DOWNLINK_PDSCH_SUBFRAME_DETECT_PDCCH               1
#define RSSPECAN_VAL_LTE_DOWNLINK_PDSCH_SUBFRAME_DETECT_PHYDET              2

// rsspecan_rngDL_PDSCHReferenceData
#define RSSPECAN_VAL_LTE_DOWNLINK_PDSCH_REFERENCE_DATA_AUTO                 0
#define RSSPECAN_VAL_LTE_DOWNLINK_PDSCH_REFERENCE_DATA_ALL0                 1

// rsspecan_rngLTEUplinkSlotSelection
#define RSSPECAN_VAL_LTE_UPLINK_SLOT_SELECTION_ALL                          0
#define RSSPECAN_VAL_LTE_UPLINK_SLOT_SELECTION_S0                           1
#define RSSPECAN_VAL_LTE_UPLINK_SLOT_SELECTION_S1                           2

// rsspecan_rngLTEDownlinkPHICHNgParameter
#define RSSPECAN_VAL_LTE_DOWNLINK_PHICH_NG_AUTO                             0
#define RSSPECAN_VAL_LTE_DOWNLINK_PHICH_NG_CUSTOM                           1
#define RSSPECAN_VAL_LTE_DOWNLINK_PHICH_NG_1_6                              2
#define RSSPECAN_VAL_LTE_DOWNLINK_PHICH_NG_1_2                              3
#define RSSPECAN_VAL_LTE_DOWNLINK_PHICH_NG_1                                4
#define RSSPECAN_VAL_LTE_DOWNLINK_PHICH_NG_2                                5

// rsspecan_rngLTEUplinkPRACHSequenceIndex
#define RSSPECAN_VAL_LTE_UPLINK_PRACH_SEQ_INDEX_AUTO                        1

// rsspecan_rngLTEDownlinkMeasurement
#define RSSPECAN_VAL_LTE_DOWNLINK_MEAS_ACLR                                 0
#define RSSPECAN_VAL_LTE_DOWNLINK_MEAS_ESP                                  1
#define RSSPECAN_VAL_LTE_DOWNLINK_MEAS_EVM                                  2
#define RSSPECAN_VAL_LTE_DOWNLINK_MEAS_TAER                                 3
#define RSSPECAN_VAL_LTE_DOWNLINK_MEAS_TPOO                                 4
#define RSSPECAN_VAL_LTE_DOWNLINK_MEAS_MCAC                                 5
#define RSSPECAN_VAL_LTE_DOWNLINK_MEAS_CACL                                 6
#define RSSPECAN_VAL_LTE_DOWNLINK_MEAS_MCES                                 7

// rsspecan_rngLTEDownlinkCSIRSAntenna
#define RSSPECAN_VAL_LTE_DOWNLINK_TX1                                       1
#define RSSPECAN_VAL_LTE_DOWNLINK_TX2                                       2
#define RSSPECAN_VAL_LTE_DOWNLINK_TX4                                       4
#define RSSPECAN_VAL_LTE_DOWNLINK_TX8                                       8

// rsspecan_rngLTEDownlinkPDSCHAllocationStartOffset
#define RSSPECAN_VAL_SUBFRAME_START_OFFSET_0                                0
#define RSSPECAN_VAL_SUBFRAME_START_OFFSET_1                                1
#define RSSPECAN_VAL_SUBFRAME_START_OFFSET_2                                2
#define RSSPECAN_VAL_SUBFRAME_START_OFFSET_3                                3
#define RSSPECAN_VAL_SUBFRAME_START_OFFSET_4                                4
#define RSSPECAN_VAL_SUBFRAME_START_OFFSET_COMMON                           5

// rsspecan_rngLTEDownlinkTestModel
#define RSSPECAN_VAL_LTE_TEST_MODEL_1_1                                     0
#define RSSPECAN_VAL_LTE_TEST_MODEL_1_2                                     1
#define RSSPECAN_VAL_LTE_TEST_MODEL_2                                       2
#define RSSPECAN_VAL_LTE_TEST_MODEL_3_1                                     3
#define RSSPECAN_VAL_LTE_TEST_MODEL_3_2                                     4
#define RSSPECAN_VAL_LTE_TEST_MODEL_3_3                                     5
#define RSSPECAN_VAL_LTE_TEST_MODEL_2A                                      6
#define RSSPECAN_VAL_LTE_TEST_MODEL_3_1A                                    7
#define RSSPECAN_VAL_LTE_TEST_MODEL_1_1_1_4MHZ                              8
#define RSSPECAN_VAL_LTE_TEST_MODEL_1_2_1_4MHZ                              9
#define RSSPECAN_VAL_LTE_TEST_MODEL_2_1_4MHZ                                10
#define RSSPECAN_VAL_LTE_TEST_MODEL_3_1_1_4MHZ                              11
#define RSSPECAN_VAL_LTE_TEST_MODEL_3_2_1_4MHZ                              12
#define RSSPECAN_VAL_LTE_TEST_MODEL_3_3_1_4MHZ                              13
#define RSSPECAN_VAL_LTE_TEST_MODEL_2A_1_4MHZ                               14
#define RSSPECAN_VAL_LTE_TEST_MODEL_3_1A_1_4MHZ                             15
#define RSSPECAN_VAL_LTE_TEST_MODEL_1_1_3MHZ                                16
#define RSSPECAN_VAL_LTE_TEST_MODEL_1_2_3MHZ                                17
#define RSSPECAN_VAL_LTE_TEST_MODEL_2_3MHZ                                  18
#define RSSPECAN_VAL_LTE_TEST_MODEL_3_1_3MHZ                                19
#define RSSPECAN_VAL_LTE_TEST_MODEL_3_2_3MHZ                                20
#define RSSPECAN_VAL_LTE_TEST_MODEL_3_3_3MHZ                                21
#define RSSPECAN_VAL_LTE_TEST_MODEL_2A_3MHZ                                 22
#define RSSPECAN_VAL_LTE_TEST_MODEL_3_1A_3MHZ                               23
#define RSSPECAN_VAL_LTE_TEST_MODEL_1_1_5MHZ                                24
#define RSSPECAN_VAL_LTE_TEST_MODEL_1_2_5MHZ                                25
#define RSSPECAN_VAL_LTE_TEST_MODEL_2_5MHZ                                  26
#define RSSPECAN_VAL_LTE_TEST_MODEL_3_1_5MHZ                                27
#define RSSPECAN_VAL_LTE_TEST_MODEL_3_2_5MHZ                                28
#define RSSPECAN_VAL_LTE_TEST_MODEL_3_3_5MHZ                                29
#define RSSPECAN_VAL_LTE_TEST_MODEL_2A_5MHZ                                 30
#define RSSPECAN_VAL_LTE_TEST_MODEL_3_1A_5MHZ                               31
#define RSSPECAN_VAL_LTE_TEST_MODEL_1_1_15MHZ                               32
#define RSSPECAN_VAL_LTE_TEST_MODEL_1_2_15MHZ                               33
#define RSSPECAN_VAL_LTE_TEST_MODEL_2_15MHZ                                 34
#define RSSPECAN_VAL_LTE_TEST_MODEL_3_1_15MHZ                               35
#define RSSPECAN_VAL_LTE_TEST_MODEL_3_2_15MHZ                               36
#define RSSPECAN_VAL_LTE_TEST_MODEL_3_3_15MHZ                               37
#define RSSPECAN_VAL_LTE_TEST_MODEL_2A_15MHZ                                38
#define RSSPECAN_VAL_LTE_TEST_MODEL_3_1A_15MHZ                              39
#define RSSPECAN_VAL_LTE_TEST_MODEL_1_1_20MHZ                               40
#define RSSPECAN_VAL_LTE_TEST_MODEL_1_2_20MHZ                               41
#define RSSPECAN_VAL_LTE_TEST_MODEL_2_20MHZ                                 42
#define RSSPECAN_VAL_LTE_TEST_MODEL_3_1_20MHZ                               43
#define RSSPECAN_VAL_LTE_TEST_MODEL_3_2_20MHZ                               44
#define RSSPECAN_VAL_LTE_TEST_MODEL_3_3_20MHZ                               45
#define RSSPECAN_VAL_LTE_TEST_MODEL_2A_20MHZ                                46
#define RSSPECAN_VAL_LTE_TEST_MODEL_3_1A_20MHZ                              47

// rsspecan_rngLTEDownlinkLimitCheckType
#define RSSPECAN_VAL_LIMIT_REL                                              1
#define RSSPECAN_VAL_LIMIT_ABS                                              2

// rsspecan_rngLTEDownlinkTransientLimitCheckType
#define RSSPECAN_VAL_LIMIT_ALL                                              0
#define RSSPECAN_VAL_LIMIT_FALLING                                          1
#define RSSPECAN_VAL_LIMIT_RISING                                           2

// rsspecan_rngLTELimitCheckResult
#define RSSPECAN_VAL_LTE_LIMIT_CHECK_FAILED                                 0
#define RSSPECAN_VAL_LTE_LIMIT_CHECK_PASSED                                 1
#define RSSPECAN_VAL_LTE_LIMIT_CHECK_NOT_EVALUATED                          2

// rsspecan_rngLTEUplinkAnalysisMode
#define RSSPECAN_VAL_LTE_UPLINK_ANALYSIS_PUSCH                              0
#define RSSPECAN_VAL_LTE_UPLINK_ANALYSIS_PRACH                              1

// rsspecan_rngLTEUplinkSEMRequirement
#define RSSPECAN_VAL_LTE_SEM_REQUIREMENT_GENERAL                            0
#define RSSPECAN_VAL_LTE_SEM_REQUIREMENT_NS3                                1
#define RSSPECAN_VAL_LTE_SEM_REQUIREMENT_NS4                                2
#define RSSPECAN_VAL_LTE_SEM_REQUIREMENT_NS67                               3
#define RSSPECAN_VAL_LTE_SEM_REQUIREMENT_NS27                               4
#define RSSPECAN_VAL_LTE_SEM_REQUIREMENT_NS35                               5

// rsspecan_rngNoiseDispDataTrac
#define RSSPECAN_VAL_NOISE_DISP_DATA_TEFF                                   1
#define RSSPECAN_VAL_NOISE_DISP_DATA_NFIGURE                                2

// rsspecan_rngNoiseCalcLimLine
#define RSSPECAN_VAL_NOISE_CALC_LIM_LINE_NFIGURE                            0
#define RSSPECAN_VAL_NOISE_CALC_LIM_LINE_TEFFECTIVE                         1
#define RSSPECAN_VAL_NOISE_CALC_LIM_LINE_GAIN                               2

// rsspecan_rngNoiseERNModeSel
#define RSSPECAN_VAL_NOISE_ERN_MODE_SEL_TABLE                               0
#define RSSPECAN_VAL_NOISE_ERN_MODE_SEL_SPOT                                1

// rsspecan_rngNoiseLossMode
#define RSSPECAN_VAL_NOISE_LOSS_MODE_TABLE                                  0
#define RSSPECAN_VAL_NOISE_LOSS_MODE_SPOT                                   1

// rsspecan_rngNoiseConfSystLosc
#define RSSPECAN_VAL_NOISE_CONF_SYST_LOSC_FIXED                             0
#define RSSPECAN_VAL_NOISE_CONF_SYST_LOSC_VARIABLE                          1

// rsspecan_rngNoiseConfDUT
#define RSSPECAN_VAL_NOISE_CONF_DUT_AMPLIFIER                               0
#define RSSPECAN_VAL_NOISE_CONF_DUT_DOWNCONV                                1
#define RSSPECAN_VAL_NOISE_CONF_DUT_UPCONV                                  2
#define RSSPECAN_VAL_NOISE_CONF_DUT_DDOW                                    3

// rsspecan_rngNoiseXAxisFrequency
#define RSSPECAN_VAL_NOISE_XAXIS_IF                                         0
#define RSSPECAN_VAL_NOISE_XAXIS_RF                                         1

// rsspecan_rngNoiseMarkerAssignFSV
#define RSSPECAN_VAL_TRACE_NOISE_CURRENT                                    1
#define RSSPECAN_VAL_TRACE_NOISE_MEM1                                       2
#define RSSPECAN_VAL_TRACE_NOISE_MEM2                                       3
#define RSSPECAN_VAL_TRACE_NOISE_MEM3                                       4
#define RSSPECAN_VAL_TRACE_GAIN_CURRENT                                     5
#define RSSPECAN_VAL_TRACE_GAIN_MEM1                                        6
#define RSSPECAN_VAL_TRACE_GAIN_MEM2                                        7
#define RSSPECAN_VAL_TRACE_GAIN_MEM3                                        8

// rsspecan_rngNoiseCalcLimitResultType
#define RSSPECAN_VAL_NOISE_CALC_LIM_LINE_TYPE_GAIN                          0
#define RSSPECAN_VAL_NOISE_CALC_LIM_LINE_TYPE_NOISE                         1
#define RSSPECAN_VAL_NOISE_CALC_LIM_LINE_TYPE_PCOLD                         2
#define RSSPECAN_VAL_NOISE_CALC_LIM_LINE_TYPE_PHOT                          3
#define RSSPECAN_VAL_NOISE_CALC_LIM_LINE_TYPE_TEMP                          4
#define RSSPECAN_VAL_NOISE_CALC_LIM_LINE_TYPE_YFAC                          5
#define RSSPECAN_VAL_NOISE_CALC_LIM_LINE_TYPE_ENR                           6

// rsspecan_rngSweMode
#define RSSPECAN_VAL_SWE_MODE_FAST                                          0
#define RSSPECAN_VAL_SWE_MODE_NORM                                          1
#define RSSPECAN_VAL_SWE_MODE_AVER                                          2
#define RSSPECAN_VAL_SWE_MODE_MAN                                           3

// rsspecan_rngBandType
#define RSSPECAN_VAL_BAND_TYPE_NORM                                         0
#define RSSPECAN_VAL_BAND_TYPE_FFT                                          1
#define RSSPECAN_VAL_BAND_TYPE_IQFFT                                        2

// rsspecan_rngPhaseSmoothingType
#define RSSPECAN_VAL_PHASE_SMO_TYPE_LIN                                     0
#define RSSPECAN_VAL_PHASE_SMO_TYPE_LOG                                     1
#define RSSPECAN_VAL_PHASE_SMO_TYPE_MED                                     2

// rsspecan_rngPhaseUserTrace
#define RSSPECAN_VAL_PHASE_NONE                                             0
#define RSSPECAN_VAL_PHASE_TRACE1                                           1
#define RSSPECAN_VAL_PHASE_TRACE2                                           2
#define RSSPECAN_VAL_PHASE_TRACE3                                           3
#define RSSPECAN_VAL_PHASE_TRACE4                                           4
#define RSSPECAN_VAL_PHASE_TRACE5                                           5
#define RSSPECAN_VAL_PHASE_TRACE6                                           6

// rsspecan_rngPhaseXAxisScope
#define RSSPECAN_VAL_NOISE_X_SCOPE_MANUAL                                   0
#define RSSPECAN_VAL_NOISE_X_SCOPE_HDECADE                                  1
#define RSSPECAN_VAL_NOISE_X_SCOPE_MRANGE                                   2

// rsspecan_rngPhaseYAxisManualScaling
#define RSSPECAN_VAL_PHASE_Y_SCALE_OFF                                      0
#define RSSPECAN_VAL_PHASE_Y_SCALE_TOP_BOTTOM                               1
#define RSSPECAN_VAL_PHASE_Y_SCALE_TOP_RANGE                                2
#define RSSPECAN_VAL_PHASE_Y_SCALE_BOTTOM_RANGE                             3

// rsspecan_rngPhaseNoiseLimitShapeType
#define RSSPECAN_VAL_PHASE_LIMIT_NONE                                       0
#define RSSPECAN_VAL_PHASE_LIMIT_FC1                                        1
#define RSSPECAN_VAL_PHASE_LIMIT_FC2                                        2
#define RSSPECAN_VAL_PHASE_LIMIT_FC3                                        3
#define RSSPECAN_VAL_PHASE_LIMIT_FC4                                        4
#define RSSPECAN_VAL_PHASE_LIMIT_FC5                                        5

// rsspecan_rngPhaseWindowFunction
#define RSSPECAN_VAL_PHASE_WINDOW_CHEB                                      0
#define RSSPECAN_VAL_PHASE_WINDOW_GAUS                                      1
#define RSSPECAN_VAL_PHASE_WINDOW_RECT                                      2
#define RSSPECAN_VAL_PHASE_WINDOW_BHAR                                      3

// rsspecan_rngPhaseSignalVerificationFailed
#define RSSPECAN_VAL_PHASE_SIGNAL_VERIFICATION_OFF                          0
#define RSSPECAN_VAL_PHASE_SIGNAL_VERIFICATION_ON                           1
#define RSSPECAN_VAL_PHASE_SIGNAL_VERIFICATION_AUTO                         2

// rsspecan_rngHorzVert
#define RSSPECAN_VAL_HORZ                                                   0
#define RSSPECAN_VAL_VERT                                                   1

// rsspecan_rngVSABurstMode
#define RSSPECAN_VAL_VSA_BURS_MODE_MEAS                                     0
#define RSSPECAN_VAL_VSA_BURS_MODE_BURS                                     1

// rsspecan_rngVSAResultFormat
#define RSSPECAN_VAL_VSA_RESULT_FORMAT_MAGN                                 0
#define RSSPECAN_VAL_VSA_RESULT_FORMAT_PHAS                                 1
#define RSSPECAN_VAL_VSA_RESULT_FORMAT_UPHAS                                2
#define RSSPECAN_VAL_VSA_RESULT_FORMAT_RIM                                  3
#define RSSPECAN_VAL_VSA_RESULT_FORMAT_FREQ                                 4
#define RSSPECAN_VAL_VSA_RESULT_FORMAT_COMP                                 5
#define RSSPECAN_VAL_VSA_RESULT_FORMAT_CONS                                 6
#define RSSPECAN_VAL_VSA_RESULT_FORMAT_GDEL                                 7
#define RSSPECAN_VAL_VSA_RESULT_FORMAT_IEYE                                 8
#define RSSPECAN_VAL_VSA_RESULT_FORMAT_QEYE                                 9
#define RSSPECAN_VAL_VSA_RESULT_FORMAT_FEYE                                 11
#define RSSPECAN_VAL_VSA_RESULT_FORMAT_CONF                                 12
#define RSSPECAN_VAL_VSA_RESULT_FORMAT_CONVF                                13
#define RSSPECAN_VAL_VSA_RESULT_FORMAT_NONE                                 15
#define RSSPECAN_VAL_VSA_RESULT_FORMAT_RCON                                 16
#define RSSPECAN_VAL_VSA_RESULT_FORMAT_BER                                  18
#define RSSPECAN_VAL_VSA_RESULT_FORMAT_RSUM                                 17
#define RSSPECAN_VAL_VSA_RESULT_FORMAT_MOV                                  19
#define RSSPECAN_VAL_VSA_RESULT_FORMAT_BIN                                  20
#define RSSPECAN_VAL_VSA_RESULT_FORMAT_OCT                                  21
#define RSSPECAN_VAL_VSA_RESULT_FORMAT_DEC                                  22
#define RSSPECAN_VAL_VSA_RESULT_FORMAT_HEX                                  23

// rsspecan_rngVSASyncSearchMode
#define RSSPECAN_VAL_VSA_SYNC_MODE_MEAS                                     0
#define RSSPECAN_VAL_VSA_SYNC_MODE_SYNC                                     1

// rsspecan_rngVSAResult
#define RSSPECAN_VAL_RESULT_XTIM_DDEM_MEAS                                  0
#define RSSPECAN_VAL_RESULT_XTIM_DDEM_REF                                   1
#define RSSPECAN_VAL_RESULT_XTIM_DDEM_ERR_MPH                               2
#define RSSPECAN_VAL_RESULT_XTIM_DDEM_ERR_VECT                              3
#define RSSPECAN_VAL_RESULT_XTIM_DDEM_IMP                                   4
#define RSSPECAN_VAL_RESULT_XFR_DDEM_RAT                                    5
#define RSSPECAN_VAL_RESULT_XFR_DDEM_IRAT                                   6
#define RSSPECAN_VAL_RESULT_XTIM_DDEM_SYMB                                  7
#define RSSPECAN_VAL_RESULT_TCAP                                            8
#define RSSPECAN_VAL_RESULT_MACC                                            9

// rsspecan_rngVSAModulationFormat
#define RSSPECAN_VAL_VSA_MOD_FORMAT_QPSK                                    0
#define RSSPECAN_VAL_VSA_MOD_FORMAT_PSK                                     1
#define RSSPECAN_VAL_VSA_MOD_FORMAT_MSK                                     2
#define RSSPECAN_VAL_VSA_MOD_FORMAT_QAM                                     3
#define RSSPECAN_VAL_VSA_MOD_FORMAT_FSK                                     4
#define RSSPECAN_VAL_VSA_MOD_FORMAT_VSB                                     5
#define RSSPECAN_VAL_VSA_MOD_FORMAT_UQAM                                    6
#define RSSPECAN_VAL_VSA_MOD_FORMAT_ASK                                     7
#define RSSPECAN_VAL_VSA_MOD_FORMAT_APSK                                    8

// rsspecan_rngVSAModulQPSKFormat
#define RSSPECAN_VAL_MOD_QPSK_NORM                                          0
#define RSSPECAN_VAL_MOD_QPSK_DIFF                                          1
#define RSSPECAN_VAL_MOD_QPSK_OFFS                                          2
#define RSSPECAN_VAL_MOD_QPSK_DPI4                                          3
#define RSSPECAN_VAL_MOD_QPSK_NPI4                                          4
#define RSSPECAN_VAL_MOD_QPSK_N3PI4                                         5
#define RSSPECAN_VAL_MOD_QPSK_SOFF                                          6

// rsspecan_rngVSAModulPSKFormat
#define RSSPECAN_VAL_MOD_PSK_NORM                                           0
#define RSSPECAN_VAL_MOD_PSK_DIFF                                           1
#define RSSPECAN_VAL_MOD_PSK_N3PI8                                          2
#define RSSPECAN_VAL_MOD_PSK_PI8D8PSK                                       3
#define RSSPECAN_VAL_MOD_PSK_NPI2                                           4
#define RSSPECAN_VAL_MOD_PSK_DPI2                                           5
#define RSSPECAN_VAL_MOD_PSK_MNPI2                                          6

// rsspecan_rngVSAModulMSKFormat
#define RSSPECAN_VAL_MOD_MSK_NORM                                           0
#define RSSPECAN_VAL_MOD_MSK_DIFF                                           1

// rsspecan_rngVSAModulQAMFormat
#define RSSPECAN_VAL_MOD_QAM_NORM                                           0
#define RSSPECAN_VAL_MOD_QAM_DIFF                                           1
#define RSSPECAN_VAL_MOD_QAM_NPI4                                           2
#define RSSPECAN_VAL_MOD_QAM_MNPI4                                          3

// rsspecan_rngVSAModulationEVMCalc
#define RSSPECAN_VAL_MOD_EVM_SYMBOL                                         0
#define RSSPECAN_VAL_MOD_EVM_SIGNAL                                         1
#define RSSPECAN_VAL_MOD_EVM_MECP                                           2
#define RSSPECAN_VAL_MOD_EVM_MACP                                           3

// rsspecan_rngVSADemodSband
#define RSSPECAN_VAL_VSA_SBAND_NORM                                         0
#define RSSPECAN_VAL_VSA_SBAND_INV                                          1

// rsspecan_rngVSAConfDisplayUnitType
#define RSSPECAN_VAL_VSA_UNIT_TIME_S                                        0
#define RSSPECAN_VAL_VSA_UNIT_TIME_SYM                                      1

// rsspecan_rngVSAReference
#define RSSPECAN_VAL_VSA_FIT_TRIGGER                                        0
#define RSSPECAN_VAL_VSA_FIT_BURST                                          1
#define RSSPECAN_VAL_VSA_FIT_PATTERN                                        2

// rsspecan_rngVSAResultAlignment
#define RSSPECAN_VAL_FIT_ALIGN_LEFT                                         0
#define RSSPECAN_VAL_FIT_ALIGN_CENTER                                       1
#define RSSPECAN_VAL_FIT_ALIGN_RIGHT                                        2

// rsspecan_rngVSAExportDataMode
#define RSSPECAN_VAL_EXP_MODE_RAW                                           0
#define RSSPECAN_VAL_EXP_MODE_TRACE                                         1

// rsspecan_rngVSAFactoryDefaults
#define RSSPECAN_VAL_VSA_FACTORY_STAN                                       1
#define RSSPECAN_VAL_VSA_FACTORY_PATT                                       3
#define RSSPECAN_VAL_VSA_FACTORY_ALL                                        5

// rsspecan_rngVSAExportDataRawMode
#define RSSPECAN_VAL_EXP_RAW_MODE_ASC                                       0
#define RSSPECAN_VAL_EXP_RAW_MODE_WAV                                       1
#define RSSPECAN_VAL_EXP_RAW_MODE_BIN                                       2

// rsspecan_rngVSAMarkerSearch
#define RSSPECAN_VAL_VSA_MARKER_SEARCH_REAL                                 0
#define RSSPECAN_VAL_VSA_MARKER_SEARCH_IMAG                                 1

// rsspecan_rngVSASignalType
#define RSSPECAN_VAL_VSA_SIGNAL_TYPE_CONT                                   0
#define RSSPECAN_VAL_VSA_SIGNAL_TYPE_BURST                                  1

// rsspecan_rngVSATXFilterType
#define RSSPECAN_VAL_VSA_TX_FILTER_TYPE_NONE                                0
#define RSSPECAN_VAL_VSA_TX_FILTER_TYPE_RC                                  1
#define RSSPECAN_VAL_VSA_TX_FILTER_TYPE_RRC                                 2
#define RSSPECAN_VAL_VSA_TX_FILTER_TYPE_GAUSS                               3
#define RSSPECAN_VAL_VSA_TX_FILTER_TYPE_GMSK                                4
#define RSSPECAN_VAL_VSA_TX_FILTER_TYPE_LIN_GMSK                            5
#define RSSPECAN_VAL_VSA_TX_FILTER_TYPE_APCO25                              6
#define RSSPECAN_VAL_VSA_TX_FILTER_TYPE_CDMA2K_FWD                          7
#define RSSPECAN_VAL_VSA_TX_FILTER_TYPE_CDMA2K_REV                          8
#define RSSPECAN_VAL_VSA_TX_FILTER_TYPE_RECT                                9
#define RSSPECAN_VAL_VSA_TX_FILTER_TYPE_HALF_SINE                           10
#define RSSPECAN_VAL_VSA_TX_FILTER_TYPE_APCO25_HCPM                         11
#define RSSPECAN_VAL_VSA_TX_FILTER_TYPE_APCO25_HDQPSK                       12
#define RSSPECAN_VAL_VSA_TX_FILTER_TYPE_APCO25_HD8PSK_NARROW                13
#define RSSPECAN_VAL_VSA_TX_FILTER_TYPE_APCO25_HD8PSK_WIDE                  14
#define RSSPECAN_VAL_VSA_TX_FILTER_TYPE_EDGE_NARROW                         15
#define RSSPECAN_VAL_VSA_TX_FILTER_TYPE_EDGE_WIDE                           16
#define RSSPECAN_VAL_VSA_TX_FILTER_USER                                     17

// rsspecan_rngVSAMFilterType
#define RSSPECAN_VAL_VSA_MEAS_FILTER_TYPE_NONE                              0
#define RSSPECAN_VAL_VSA_MEAS_FILTER_TYPE_RRC                               1
#define RSSPECAN_VAL_VSA_MEAS_FILTER_TYPE_EDGE                              2
#define RSSPECAN_VAL_VSA_MEAS_FILTER_TYPE_EDGE_HSR_NARROW                   3
#define RSSPECAN_VAL_VSA_MEAS_FILTER_TYPE_EDGE_HSR_WIDE                     4
#define RSSPECAN_VAL_VSA_MEAS_FILTER_TYPE_GAUSS                             5
#define RSSPECAN_VAL_VSA_MEAS_FILTER_TYPE_LOW_ISI                           6
#define RSSPECAN_VAL_VSA_MEAS_FILTER_TYPE_LOW_PASS_NARROW                   7
#define RSSPECAN_VAL_VSA_MEAS_FILTER_TYPE_LOW_PASS_WIDE                     8
#define RSSPECAN_VAL_VSA_MEAS_FILTER_TYPE_RECTANGULAR                       9
#define RSSPECAN_VAL_VSA_MEAS_FILTER_TYPE_USER                              10

// rsspecan_rngVSACaptureOversampling
#define RSSPECAN_VAL_STAT_MODE_SONLY                                        0
#define RSSPECAN_VAL_STAT_MODE_INFINITE                                     1

// rsspecan_rngVSATraceEvaluation
#define RSSPECAN_VAL_TRAC_EVAL_MEAS                                         0
#define RSSPECAN_VAL_TRAC_EVAL_REF                                          1
#define RSSPECAN_VAL_TRAC_EVAL_ERROR                                        2
#define RSSPECAN_VAL_TRAC_EVAL_TCAP                                         3

// rsspecan_rngVSALimitCheck
#define RSSPECAN_VAL_VSA_LIMIT_STATE_NONE                                   -1
#define RSSPECAN_VAL_VSA_LIMIT_STATE_PASS                                   0
#define RSSPECAN_VAL_VSA_LIMIT_STATE_FAIL                                   1
#define RSSPECAN_VAL_VSA_LIMIT_STATE_MARGIN                                 2

// rsspecan_rngVSAFineSynchronisationMode
#define RSSPECAN_VAL_VSA_FINE_SYNC_KDAT                                     0
#define RSSPECAN_VAL_VSA_FINE_SYNC_PATT                                     1
#define RSSPECAN_VAL_VSA_FINE_SYNC_DDAT                                     2

// rsspecan_rngVSAModulationOptimization
#define RSSPECAN_VAL_VSA_MOD_OPTIMIZATION_RMS                               0
#define RSSPECAN_VAL_VSA_MOD_OPTIMIZATION_EVM                               1

// rsspecan_rngVSAScope
#define RSSPECAN_VAL_VSA_SCOPE_CURRENT                                      0
#define RSSPECAN_VAL_VSA_SCOPE_ALL                                          1

// rsspecan_rngVSAEqualizerMode
#define RSSPECAN_VAL_VSA_EQUALIZER_MODE_NORMAL                              0
#define RSSPECAN_VAL_VSA_EQUALIZER_MODE_TRACKING                            1
#define RSSPECAN_VAL_VSA_EQUALIZER_MODE_FREEZE                              2
#define RSSPECAN_VAL_VSA_EQUALIZER_MODE_USER                                3
#define RSSPECAN_VAL_VSA_EQUALIZER_MODE_AVERAGING                           4

// rsspecan_rngVSAStatisticType
#define RSSPECAN_VAL_VSA_STATISTIC_CURRENT                                  0
#define RSSPECAN_VAL_VSA_STATISTIC_AVG                                      1
#define RSSPECAN_VAL_VSA_STATISTIC_RPE                                      2
#define RSSPECAN_VAL_VSA_STATISTIC_SDEV                                     3
#define RSSPECAN_VAL_VSA_STATISTIC_PCTL                                     4

// rsspecan_rngWlanPvtRPow
#define RSSPECAN_VAL_PVT_MEAN                                               0
#define RSSPECAN_VAL_PVT_MAX                                                1

// rsspecan_rngResults
#define RSSPECAN_VAL_RES_FAIL                                               0
#define RSSPECAN_VAL_RES_PASS                                               1

// rsspecan_rngTrigMode
#define RSSPECAN_VAL_TRIG_MODE_IMM                                          0
#define RSSPECAN_VAL_TRIG_MODE_EXT                                          1
#define RSSPECAN_VAL_TRIG_MODE_POW                                          2
#define RSSPECAN_VAL_TRIG_MODE_PSEN                                         3
#define RSSPECAN_VAL_TRIG_MODE_RFP                                          4
#define RSSPECAN_VAL_TRIG_MODE_EFAL                                         5
#define RSSPECAN_VAL_TRIG_MODE_ERIS                                         6
#define RSSPECAN_VAL_TRIG_MODE_TUN                                          7

// rsspecan_rngBurstSel
#define RSSPECAN_VAL_SELECTION_ALL                                          0
#define RSSPECAN_VAL_SELECTION_PIL                                          1

// rsspecan_rngWlanPvtSel
#define RSSPECAN_VAL_PVT_EDGE                                               0
#define RSSPECAN_VAL_PVT_FULL                                               1
#define RSSPECAN_VAL_PVT_RISE                                               2
#define RSSPECAN_VAL_PVT_FALL                                               3

// rsspecan_rngSpecMask
#define RSSPECAN_VAL_SPEC_IEEE                                              0
#define RSSPECAN_VAL_SPEC_ETSI                                              1
#define RSSPECAN_VAL_SPEC_ETSI5                                             2
#define RSSPECAN_VAL_SPEC_ETSI10                                            3

// rsspecan_rngWlanDemod
#define RSSPECAN_VAL_WLAN_BPSK                                              0
#define RSSPECAN_VAL_WLAN_QPSK                                              1
#define RSSPECAN_VAL_WLAN_QAM16                                             2
#define RSSPECAN_VAL_WLAN_QAM64                                             3
#define RSSPECAN_VAL_WLAN_BPSK6                                             4
#define RSSPECAN_VAL_WLAN_BPSK9                                             5
#define RSSPECAN_VAL_WLAN_QPSK12                                            6
#define RSSPECAN_VAL_WLAN_QPSK18                                            7
#define RSSPECAN_VAL_WLAN_QAM1624                                           8
#define RSSPECAN_VAL_WLAN_QAM1636                                           9
#define RSSPECAN_VAL_WLAN_QAM6448                                           10
#define RSSPECAN_VAL_WLAN_QAM6454                                           11
#define RSSPECAN_VAL_WLAN_DBPSK                                             12
#define RSSPECAN_VAL_WLAN_DQPSK                                             13
#define RSSPECAN_VAL_WLAN_CCK55                                             14
#define RSSPECAN_VAL_WLAN_CCK11                                             15
#define RSSPECAN_VAL_WLAN_PBCC55                                            16
#define RSSPECAN_VAL_WLAN_PBCC11                                            17
#define RSSPECAN_VAL_WLAN_PBCC22                                            18
#define RSSPECAN_VAL_WLAN_BPSK3                                             19
#define RSSPECAN_VAL_WLAN_BPSK45                                            20
#define RSSPECAN_VAL_WLAN_QPSK6                                             21
#define RSSPECAN_VAL_WLAN_QPSK9                                             22
#define RSSPECAN_VAL_WLAN_QAM1612                                           23
#define RSSPECAN_VAL_WLAN_QAM1618                                           24
#define RSSPECAN_VAL_WLAN_QAM6424                                           25
#define RSSPECAN_VAL_WLAN_QAM6427                                           26

// rsspecan_rngWlanBurstType
#define RSSPECAN_VAL_WLAN_BTYP_DIRECT                                       0
#define RSSPECAN_VAL_WLAN_BTYP_LOFDM                                        1
#define RSSPECAN_VAL_WLAN_BTYP_SOFDM                                        2
#define RSSPECAN_VAL_WLAN_BTYP_LONG                                         3
#define RSSPECAN_VAL_WLAN_BTYP_SHORT                                        4
#define RSSPECAN_VAL_WLAN_BTYP_MM20                                         5
#define RSSPECAN_VAL_WLAN_BTYP_GFM20                                        6
#define RSSPECAN_VAL_WLAN_BTYP_MM40                                         7
#define RSSPECAN_VAL_WLAN_BTYP_GFM40                                        8

// rsspecan_rngPreamble
#define RSSPECAN_VAL_PRE_FREQ                                               0
#define RSSPECAN_VAL_PRE_PHAS                                               1

// rsspecan_rngTableSEMType
#define RSSPECAN_VAL_SEM_IEEE                                               0
#define RSSPECAN_VAL_SEM_ETSI                                               1
#define RSSPECAN_VAL_SEM_USER                                               2

// rsspecan_rngWLANFFTStartOffset
#define RSSPECAN_VAL_WLAN_FFT_START_OFFSET_AUTO                             0
#define RSSPECAN_VAL_WLAN_FFT_START_OFFSET_GIC                              1
#define RSSPECAN_VAL_WLAN_FFT_START_OFFSET_PEAK                             2

// rsspecan_rngVectMeasRange
#define RSSPECAN_VAL_WLAN_MEAS_RANGE_ALL                                    0
#define RSSPECAN_VAL_WLAN_MEAS_RANGE_PSDU                                   1

// rsspecan_rngWlanPVRMode
#define RSSPECAN_VAL_WLAN_PVR_ALL                                           0
#define RSSPECAN_VAL_WLAN_PVR_PSDU                                          1

// rsspecan_rngAntennaWLAN
#define RSSPECAN_VAL_WLAN_ANTENNA1                                          1
#define RSSPECAN_VAL_WLAN_ANTENNA2                                          2
#define RSSPECAN_VAL_WLAN_ANTENNA3                                          3
#define RSSPECAN_VAL_WLAN_ANTENNA4                                          4
#define RSSPECAN_VAL_WLAN_ANTENNA5                                          5
#define RSSPECAN_VAL_WLAN_ANTENNA6                                          6
#define RSSPECAN_VAL_WLAN_ANTENNA7                                          7
#define RSSPECAN_VAL_WLAN_ANTENNA8                                          8

// rsspecan_rngAntennasNumberWLAN
#define RSSPECAN_VAL_WLAN_DUTC_TX1                                          1
#define RSSPECAN_VAL_WLAN_DUTC_TX2                                          2
#define RSSPECAN_VAL_WLAN_DUTC_TX3                                          3
#define RSSPECAN_VAL_WLAN_DUTC_TX4                                          4
#define RSSPECAN_VAL_WLAN_DUTC_TX5                                          5
#define RSSPECAN_VAL_WLAN_DUTC_TX6                                          6
#define RSSPECAN_VAL_WLAN_DUTC_TX7                                          7
#define RSSPECAN_VAL_WLAN_DUTC_TX8                                          8

// rsspecan_rngSTBCField
#define RSSPECAN_VAL_WLAN_EXT_STBC_FBUR                                     0
#define RSSPECAN_VAL_WLAN_EXT_STBC_ALL                                      1
#define RSSPECAN_VAL_WLAN_EXT_STBC_M0                                       2
#define RSSPECAN_VAL_WLAN_EXT_STBC_M1                                       3
#define RSSPECAN_VAL_WLAN_EXT_STBC_M2                                       4
#define RSSPECAN_VAL_WLAN_EXT_STBC_D0                                       5
#define RSSPECAN_VAL_WLAN_EXT_STBC_D1                                       6
#define RSSPECAN_VAL_WLAN_EXT_STBC_D2                                       7

// rsspecan_rngWLANExtensionSpatialStreams
#define RSSPECAN_VAL_WLAN_EXT_SPATIAL_STREAMS_FBUR                          0
#define RSSPECAN_VAL_WLAN_EXT_SPATIAL_STREAMS_ALL                           1
#define RSSPECAN_VAL_WLAN_EXT_SPATIAL_STREAMS_M0                            2
#define RSSPECAN_VAL_WLAN_EXT_SPATIAL_STREAMS_M1                            3
#define RSSPECAN_VAL_WLAN_EXT_SPATIAL_STREAMS_M2                            4
#define RSSPECAN_VAL_WLAN_EXT_SPATIAL_STREAMS_M3                            5
#define RSSPECAN_VAL_WLAN_EXT_SPATIAL_STREAMS_D0                            6
#define RSSPECAN_VAL_WLAN_EXT_SPATIAL_STREAMS_D1                            7
#define RSSPECAN_VAL_WLAN_EXT_SPATIAL_STREAMS_D2                            8
#define RSSPECAN_VAL_WLAN_EXT_SPATIAL_STREAMS_D3                            9

// rsspecan_rngSourceOfPayloadLength
#define RSSPECAN_VAL_WLAN_SOURCE_OF_PAYLOAD_LENGTH_EST                      0
#define RSSPECAN_VAL_WLAN_SOURCE_OF_PAYLOAD_LENGTH_HTS                      1
#define RSSPECAN_VAL_WLAN_SOURCE_OF_PAYLOAD_LENGTH_LS                       2

// rsspecan_rngMCSIndexToUse
#define RSSPECAN_VAL_WLAN_MCS_INDEX_TO_USE_FBUR                             0
#define RSSPECAN_VAL_WLAN_MCS_INDEX_TO_USE_ALL                              1
#define RSSPECAN_VAL_WLAN_MCS_INDEX_TO_USE_MEAS                             2
#define RSSPECAN_VAL_WLAN_MCS_INDEX_TO_USE_DEMOD                            3

// rsspecan_rngWLANMIMOSpatialMappingMode
#define RSSPECAN_VAL_WLAN_MIMO_SPATIAL_MAPPING_MODE_DIR                     0
#define RSSPECAN_VAL_WLAN_MIMO_SPATIAL_MAPPING_MODE_SEXP                    1
#define RSSPECAN_VAL_WLAN_MIMO_SPATIAL_MAPPING_MODE_USER                    2

// rsspecan_rngWLAN_SpectFlat
#define RSSPECAN_VAL_WLAN_SPEC_MODE_SEL_FLAT                                5
#define RSSPECAN_VAL_WLAN_SPEC_MODE_SEL_GRD                                 6

// rsspecan_rngWLANChannelSelect
#define RSSPECAN_VAL_WLAN_SPEC_MODE_CSEL_EFF                                0
#define RSSPECAN_VAL_WLAN_SPEC_MODE_CSEL_PHY                                1

// rsspecan_rngWLAN_MIMO_DUTConfiguration
#define RSSPECAN_VAL_WLAN_MIMO_ANT_CAPTURE_RX1                              1
#define RSSPECAN_VAL_WLAN_MIMO_ANT_CAPTURE_RX2                              2
#define RSSPECAN_VAL_WLAN_MIMO_ANT_CAPTURE_RX3                              3
#define RSSPECAN_VAL_WLAN_MIMO_ANT_CAPTURE_RX4                              4
#define RSSPECAN_VAL_WLAN_MIMO_ANT_CAPTURE_RX5                              5
#define RSSPECAN_VAL_WLAN_MIMO_ANT_CAPTURE_RX6                              6
#define RSSPECAN_VAL_WLAN_MIMO_ANT_CAPTURE_RX7                              7
#define RSSPECAN_VAL_WLAN_MIMO_ANT_CAPTURE_RX8                              8

// rsspecan_rngWLAN_MIMOAntennaSignalCapture
#define RSSPECAN_VAL_WLAN_MIMO_ANT_CAPTURE_TYPE_SIM                         0
#define RSSPECAN_VAL_WLAN_MIMO_ANT_CAPTURE_TYPE_OSP                         1
#define RSSPECAN_VAL_WLAN_MIMO_ANT_CAPTURE_TYPE_MAN                         2

// rsspecan_rngWLANTrackingPilots
#define RSSPECAN_VAL_WLAN_TRACKING_PILOTS_STANDARD                          0
#define RSSPECAN_VAL_WLAN_TRACKING_PILOTS_DETECTED                          1

// rsspecan_rngWLANMIMOPayloadLengthSource
#define RSSPECAN_VAL_WLAN_MIMO_PAYLOAD_LENGTH_ESTIMATE                      0
#define RSSPECAN_VAL_WLAN_MIMO_PAYLOAD_LENGTH_HTSIGNAL                      1
#define RSSPECAN_VAL_WLAN_MIMO_PAYLOAD_LENGTH_LSIG                          2

// rsspecan_rngWLANPPDUFormat
#define RSSPECAN_VAL_WLAN_PPDUS_FORMAT_FBUR                                 0
#define RSSPECAN_VAL_WLAN_PPDUS_FORMAT_ALL                                  1
#define RSSPECAN_VAL_WLAN_PPDUS_FORMAT_MNHT                                 2
#define RSSPECAN_VAL_WLAN_PPDUS_FORMAT_DNHT                                 3

// rsspecan_rngWLANChannelBandwidth
#define RSSPECAN_VAL_WLAN_BANDWIDTH_FBUR                                    0
#define RSSPECAN_VAL_WLAN_BANDWIDTH_ALL                                     1
#define RSSPECAN_VAL_WLAN_BANDWIDTH_MB5                                     2
#define RSSPECAN_VAL_WLAN_BANDWIDTH_MB10                                    3
#define RSSPECAN_VAL_WLAN_BANDWIDTH_MB20                                    4
#define RSSPECAN_VAL_WLAN_BANDWIDTH_DB5                                     5
#define RSSPECAN_VAL_WLAN_BANDWIDTH_DB10                                    6
#define RSSPECAN_VAL_WLAN_BANDWIDTH_DB20                                    7
#define RSSPECAN_VAL_WLAN_BANDWIDTH_DB25                                    8

// rsspecan_rngWLANPSDUModulation
#define RSSPECAN_VAL_WLAN_MODULATION_BPSK6                                  0
#define RSSPECAN_VAL_WLAN_MODULATION_BPSK9                                  1
#define RSSPECAN_VAL_WLAN_MODULATION_QPSK12                                 2
#define RSSPECAN_VAL_WLAN_MODULATION_QPSK18                                 3
#define RSSPECAN_VAL_WLAN_MODULATION_QAM1624                                4
#define RSSPECAN_VAL_WLAN_MODULATION_QAM1636                                5
#define RSSPECAN_VAL_WLAN_MODULATION_QAM6448                                6
#define RSSPECAN_VAL_WLAN_MODULATION_QAM6454                                7

// rsspecan_rngWLANPSDUModulation802_11bg
#define RSSPECAN_VAL_WLAN_MODULATION_CCK11                                  0
#define RSSPECAN_VAL_WLAN_MODULATION_CCK55                                  1
#define RSSPECAN_VAL_WLAN_MODULATION_DBPSK1                                 2
#define RSSPECAN_VAL_WLAN_MODULATION_DQPSK2                                 3
#define RSSPECAN_VAL_WLAN_MODULATION_PBCC11                                 4
#define RSSPECAN_VAL_WLAN_MODULATION_PBCC22                                 5
#define RSSPECAN_VAL_WLAN_MODULATION_PBCC55                                 6

// rsspecan_rngWLANPPDUFormat802_11n
#define RSSPECAN_VAL_WLAN_PPDU_FORMAT_FBUR                                  0
#define RSSPECAN_VAL_WLAN_PPDU_FORMAT_ALL                                   1
#define RSSPECAN_VAL_WLAN_PPDU_FORMAT_MMIX                                  2
#define RSSPECAN_VAL_WLAN_PPDU_FORMAT_MGRF                                  3
#define RSSPECAN_VAL_WLAN_PPDU_FORMAT_DMIX                                  4
#define RSSPECAN_VAL_WLAN_PPDU_FORMAT_DGRF                                  5

// rsspecan_rngWLANChannelBandwidth802_11n
#define RSSPECAN_VAL_WLAN_BANDWIDTH_802_11N_FBUR                            0
#define RSSPECAN_VAL_WLAN_BANDWIDTH_802_11N_ALL                             1
#define RSSPECAN_VAL_WLAN_BANDWIDTH_802_11N_MB20                            2
#define RSSPECAN_VAL_WLAN_BANDWIDTH_802_11N_MB40                            3
#define RSSPECAN_VAL_WLAN_BANDWIDTH_802_11N_DB20                            4
#define RSSPECAN_VAL_WLAN_BANDWIDTH_802_11N_DB40                            5
#define RSSPECAN_VAL_WLAN_BANDWIDTH_802_11N_DB80                            6
#define RSSPECAN_VAL_WLAN_BANDWIDTH_802_11N_DB160                           7

// rsspecan_rngWLANGuardIntervalLength
#define RSSPECAN_VAL_WLAN_GUARD_INTERVAL_FBUR                               0
#define RSSPECAN_VAL_WLAN_GUARD_INTERVAL_ALL                                1
#define RSSPECAN_VAL_WLAN_GUARD_INTERVAL_MS                                 2
#define RSSPECAN_VAL_WLAN_GUARD_INTERVAL_ML                                 3
#define RSSPECAN_VAL_WLAN_GUARD_INTERVAL_DS                                 4
#define RSSPECAN_VAL_WLAN_GUARD_INTERVAL_DL                                 5
#define RSSPECAN_VAL_WLAN_GUARD_INTERVAL_L1G1                               6
#define RSSPECAN_VAL_WLAN_GUARD_INTERVAL_L1G2                               7
#define RSSPECAN_VAL_WLAN_GUARD_INTERVAL_L2G2                               8
#define RSSPECAN_VAL_WLAN_GUARD_INTERVAL_L2G1                               9
#define RSSPECAN_VAL_WLAN_GUARD_INTERVAL_L4G4                               10
#define RSSPECAN_VAL_WLAN_GUARD_INTERVAL_L4G1                               11

// rsspecan_rngWLANPPDUFormat802_11ac
#define RSSPECAN_VAL_WLAN_PPDU_FORMAT_MVHT                                  2
#define RSSPECAN_VAL_WLAN_PPDU_FORMAT_DVHT                                  3

// rsspecan_rngWLANPPDUFormat802_11bg
#define RSSPECAN_VAL_WLAN_PPDU_FORMAT_FAMA                                  0
#define RSSPECAN_VAL_WLAN_PPDU_FORMAT_FMMA                                  1
#define RSSPECAN_VAL_WLAN_PPDU_FORMAT_FAMM                                  2
#define RSSPECAN_VAL_WLAN_PPDU_FORMAT_FMMM                                  3
#define RSSPECAN_VAL_WLAN_PPDU_FORMAT_FMMD                                  4

// rsspecan_rngWLANChannelBandwidth802_11ac
#define RSSPECAN_VAL_WLAN_BANDWIDTH_802_11AC_FBUR                           0
#define RSSPECAN_VAL_WLAN_BANDWIDTH_802_11AC_ALL                            1
#define RSSPECAN_VAL_WLAN_BANDWIDTH_802_11AC_MB20                           2
#define RSSPECAN_VAL_WLAN_BANDWIDTH_802_11AC_MB40                           3
#define RSSPECAN_VAL_WLAN_BANDWIDTH_802_11AC_MB80                           4
#define RSSPECAN_VAL_WLAN_BANDWIDTH_802_11AC_MB160                          5
#define RSSPECAN_VAL_WLAN_BANDWIDTH_802_11AC_DB20                           6
#define RSSPECAN_VAL_WLAN_BANDWIDTH_802_11AC_DB40                           7
#define RSSPECAN_VAL_WLAN_BANDWIDTH_802_11AC_DB80                           8
#define RSSPECAN_VAL_WLAN_BANDWIDTH_802_11AC_DB160                          9

// rsspecan_rngWLANNSTSIndexMode
#define RSSPECAN_VAL_WLAN_NSTS_INDEX_FBUR                                   0
#define RSSPECAN_VAL_WLAN_NSTS_INDEX_ALL                                    1
#define RSSPECAN_VAL_WLAN_NSTS_INDEX_MEAS                                   2
#define RSSPECAN_VAL_WLAN_NSTS_INDEX_DEMOD                                  3

// rsspecan_rngWLANBurstUnit
#define RSSPECAN_VAL_BURST_UNIT_SYMBOL                                      0
#define RSSPECAN_VAL_BURST_UNIT_SAMPLE                                      1

// rsspecan_rngWLANTableItem
#define RSSPECAN_VAL_WLAN_ITEM_TX_ALL                                       0
#define RSSPECAN_VAL_WLAN_ITEM_TX_IQ_OFFSET                                 1
#define RSSPECAN_VAL_WLAN_ITEM_TX_QUAD_OFFSET                               2
#define RSSPECAN_VAL_WLAN_ITEM_TX_CREST_FACTOR                              3
#define RSSPECAN_VAL_WLAN_ITEM_TX_GAIN_IMBALANCE                            4
#define RSSPECAN_VAL_WLAN_ITEM_TX_PPDU_POWER                                5
#define RSSPECAN_VAL_WLAN_ITEM_RX_ALL                                       6
#define RSSPECAN_VAL_WLAN_ITEM_RX_PPDU_POWER                                7
#define RSSPECAN_VAL_WLAN_ITEM_RX_CREST_FACTOR                              8
#define RSSPECAN_VAL_WLAN_ITEM_STREAM_ALL                                   9
#define RSSPECAN_VAL_WLAN_ITEM_STREAM_BER_PILOT                             10
#define RSSPECAN_VAL_WLAN_ITEM_STREAM_EVM_DATA                              11
#define RSSPECAN_VAL_WLAN_ITEM_STREAM_EVM_ALL                               12
#define RSSPECAN_VAL_WLAN_ITEM_STREAM_EVM_PILOT                             13
#define RSSPECAN_VAL_WLAN_ITEM_PILOT_BIT_ERROR                              14
#define RSSPECAN_VAL_WLAN_ITEM_EVM_ALL                                      15
#define RSSPECAN_VAL_WLAN_ITEM_EVM_DATA                                     16
#define RSSPECAN_VAL_WLAN_ITEM_EVM_PILOT                                    17
#define RSSPECAN_VAL_WLAN_ITEM_CENTER_FREQ_ERROR                            18
#define RSSPECAN_VAL_WLAN_ITEM_SYMBOL_CLOCK_ERROR                           19
#define RSSPECAN_VAL_WLAN_ITEM_PILOT_VECTOR_ERROR_DSSS                      20
#define RSSPECAN_VAL_WLAN_ITEM_PPDU_EVM_DSSS                                21
#define RSSPECAN_VAL_WLAN_ITEM_IQ_OFFSET_DSSS                               22
#define RSSPECAN_VAL_WLAN_ITEM_GAIN_IMBALANCE_DSSS                          23
#define RSSPECAN_VAL_WLAN_ITEM_QUADRATURE_ERROR_DSSS                        24
#define RSSPECAN_VAL_WLAN_ITEM_CENTER_FREQ_ERROR_DSSS                       25
#define RSSPECAN_VAL_WLAN_ITEM_CHIP_CLOCK_ERROR_DSSS                        26
#define RSSPECAN_VAL_WLAN_ITEM_RISE_TIME_DSSS                               27
#define RSSPECAN_VAL_WLAN_ITEM_FALL_TIME_DSSS                               28
#define RSSPECAN_VAL_WLAN_ITEM_MEAN_POWER_DSSS                              29
#define RSSPECAN_VAL_WLAN_ITEM_PEAK_POWER_DSSS                              30
#define RSSPECAN_VAL_WLAN_ITEM_CREST_FACTOR_DSSS                            31
#define RSSPECAN_VAL_WLAN_ITEM_IQ_SKEW                                      32
#define RSSPECAN_VAL_WLAN_ITEM_RX_MIMO_CROSS_POWER                          33
#define RSSPECAN_VAL_WLAN_ITEM_RX_CENTER_FREQ_ERROR                         34
#define RSSPECAN_VAL_WLAN_ITEM_RX_SYMBOL_CLOCK_ERROR                        35
#define RSSPECAN_VAL_WLAN_ITEM_RX_COMMON_PHASE_ERROR                        36
#define RSSPECAN_VAL_WLAN_ITEM_TX_MIMO_CHANNEL_POWER                        37

// rsspecan_rngACPMeasType
#define RSSPECAN_VAL_WLAN_CURRENT                                           0
#define RSSPECAN_VAL_WLAN_MAXHOLD                                           1

// rsspecan_rngWiMAXFlatSel
#define RSSPECAN_VAL_WIMAX_FLAT_FLAT                                        0
#define RSSPECAN_VAL_WIMAX_FLAT_GRD                                         1
#define RSSPECAN_VAL_WIMAX_FLAT_DIF                                         2

// rsspecan_rngIQFilt
#define RSSPECAN_VAL_IQ_FILT_NORM                                           0
#define RSSPECAN_VAL_IQ_FILT_WIDE                                           1

// rsspecan_rngWiMAXFreqBand
#define RSSPECAN_VAL_WIMAX_FBAN_UNSP                                        0
#define RSSPECAN_VAL_WIMAX_FBAN_ETSI                                        1
#define RSSPECAN_VAL_WIMAX_FBAN_MMDS                                        2
#define RSSPECAN_VAL_WIMAX_FBAN_WCS                                         3
#define RSSPECAN_VAL_WIMAX_FBAN_CEPT                                        4
#define RSSPECAN_VAL_WIMAX_FBAN_UNI                                         5
#define RSSPECAN_VAL_WIMAX_FBAN_ETSI2                                       6
#define RSSPECAN_VAL_WIMAX_FBAN_MMDS2                                       7
#define RSSPECAN_VAL_WIMAX_FBAN_WCS2                                        8
#define RSSPECAN_VAL_WIMAX_FBAN_CEPT2                                       9
#define RSSPECAN_VAL_WIMAX_FBAN_UNI2                                        10

// rsspecan_rngWiMAXLinkMode
#define RSSPECAN_VAL_WIMAX_LMOD_DL                                          0
#define RSSPECAN_VAL_WIMAX_LMOD_UL                                          1

// rsspecan_rngWiMAXDemod
#define RSSPECAN_VAL_WIMAX_BPSK1_2                                          0
#define RSSPECAN_VAL_WIMAX_QPSK1_2                                          1
#define RSSPECAN_VAL_WIMAX_QPSK3_4                                          2
#define RSSPECAN_VAL_WIMAX_16QAM1_2                                         3
#define RSSPECAN_VAL_WIMAX_16QAM3_4                                         4
#define RSSPECAN_VAL_WIMAX_64QAM2_3                                         5
#define RSSPECAN_VAL_WIMAX_64QAM3_4                                         6

// rsspecan_rngWiMAXBconAuto
#define RSSPECAN_VAL_WIMAX_BCON_NONE                                        0
#define RSSPECAN_VAL_WIMAX_BCON_FIRST                                       1
#define RSSPECAN_VAL_WIMAX_BCON_USER                                        2
#define RSSPECAN_VAL_WIMAX_BCON_ALL                                         3
#define RSSPECAN_VAL_WIMAX_BCON_SIGNAL                                      4
#define RSSPECAN_VAL_WIMAX_BCON_PREDEF                                      5
#define RSSPECAN_VAL_WIMAX_BCON_ULPHY                                       6
#define RSSPECAN_VAL_WIMAX_BCON_ULMAP                                       7
#define RSSPECAN_VAL_WIMAX_BCON_DLMAP                                       8
#define RSSPECAN_VAL_WIMAX_BCON_ALL64                                       9
#define RSSPECAN_VAL_WIMAX_BCON_ALL256                                      10

// rsspecan_rngWiMAXModulForm
#define RSSPECAN_VAL_WIMAX_MODUL_ALL                                        0
#define RSSPECAN_VAL_WIMAX_MODUL_QPSK                                       1
#define RSSPECAN_VAL_WIMAX_MODUL_QAM16                                      2
#define RSSPECAN_VAL_WIMAX_MODUL_QAM64                                      3

// rsspecan_rngWiMAXZoneType
#define RSSPECAN_VAL_WIMAX_ZONE_TYPE_DLFUSC                                 0
#define RSSPECAN_VAL_WIMAX_ZONE_TYPE_DLPUSC                                 1
#define RSSPECAN_VAL_WIMAX_ZONE_TYPE_ULPUSC                                 2

// rsspecan_rngWiMAXZoneBurstType
#define RSSPECAN_VAL_WIMAX_ZONE_BURS_TYPE_FCH                               0
#define RSSPECAN_VAL_WIMAX_ZONE_BURS_TYPE_DLMAP                             1
#define RSSPECAN_VAL_WIMAX_ZONE_BURS_TYPE_ULMAP                             2
#define RSSPECAN_VAL_WIMAX_ZONE_BURS_TYPE_DATA                              3
#define RSSPECAN_VAL_WIMAX_ZONE_BURS_TYPE_HARQ                              4
#define RSSPECAN_VAL_WIMAX_ZONE_BURS_TYPE_FASTF                             5

// rsspecan_rngWiMAXCestType
#define RSSPECAN_VAL_WIMAX_CEST_PREAMONLY                                   0
#define RSSPECAN_VAL_WIMAX_CEST_PREAMPAYL                                   1
#define RSSPECAN_VAL_WIMAX_CEST_PAYLONLY                                    2

// rsspecan_rngWiMAXTracPilots
#define RSSPECAN_VAL_WIMAX_TRAC_PIL_PRED                                    0
#define RSSPECAN_VAL_WIMAX_TRAC_PIL_DET                                     1

// rsspecan_rngWiMAXTracCest
#define RSSPECAN_VAL_WIMAX_TRAC_CEST_FULL                                   0
#define RSSPECAN_VAL_WIMAX_TRAC_CEST_USER                                   1

// rsspecan_rngWiMAXNumberOfAntennas
#define RSSPECAN_VAL_WIMAX_DUTC_TX1                                         0
#define RSSPECAN_VAL_WIMAX_DUTC_TX2                                         1

// rsspecan_rngWiMAXSpaceTimeCoding
#define RSSPECAN_VAL_WIMAX_ZONE_STCC_NOST                                   0
#define RSSPECAN_VAL_WIMAX_ZONE_STCC_TX2                                    1
#define RSSPECAN_VAL_WIMAX_ZONE_STCC_TX1                                    2

// rsspecan_rngWiMAXTTCStart
#define RSSPECAN_VAL_WIMAX_TTC_FFT                                          0
#define RSSPECAN_VAL_WIMAX_TTC_FRAM                                         1

// rsspecan_rngWiMAXMatrix
#define RSSPECAN_VAL_WIMAX_MATRIX_A                                         1
#define RSSPECAN_VAL_WIMAX_MATRIX_B                                         2

// rsspecan_rngWIMAXSubframeRPT
#define RSSPECAN_VAL_WIMAX_SUBFRAME_RPTY_RMS                                0
#define RSSPECAN_VAL_WIMAX_SUBFRAME_RPTY_PEAK                               1
#define RSSPECAN_VAL_WIMAX_SUBFRAME_RPTY_USER                               2

// rsspecan_rngWIMAXSubframeRPOT
#define RSSPECAN_VAL_WIMAX_SUBFRAME_RPOT_PSN                                0
#define RSSPECAN_VAL_WIMAX_SUBFRAME_RPOT_USER                               1

// rsspecan_rngWIMAXBurstBoost
#define RSSPECAN_VAL_WIMAX_BURST_BOOST_PRED                                 0
#define RSSPECAN_VAL_WIMAX_BURST_BOOST_EST                                  1

// rsspecan_rngWiMAXAntenna
#define RSSPECAN_VAL_WIMAX_ANTENNA0                                         0
#define RSSPECAN_VAL_WIMAX_ANTENNA1                                         1
#define RSSPECAN_VAL_WIMAX_ANTENNA2                                         2
#define RSSPECAN_VAL_WIMAX_ANTENNA3                                         3

// rsspecan_rngWIMAXSTCMode
#define RSSPECAN_VAL_WIMAX_STCMODE_STTD                                     0
#define RSSPECAN_VAL_WIMAX_STCMODE_1TX                                      1
#define RSSPECAN_VAL_WIMAX_STCMODE_2TX                                      2

// rsspecan_rngWIMAXUplinkZoneSTCM
#define RSSPECAN_VAL_WIMAX_STCMODE_MULTIPLEXING                             1
#define RSSPECAN_VAL_WIMAX_STCMODE_BURS                                     2

// rsspecan_rngMeasSEResultType
#define RSSPECAN_VAL_MEAS_SE_RESULT_FINAL1                                  0
#define RSSPECAN_VAL_MEAS_SE_RESULT_FINAL2                                  1
#define RSSPECAN_VAL_MEAS_SE_RESULT_FINAL3                                  2
#define RSSPECAN_VAL_MEAS_SE_RESULT_FINAL4                                  3
#define RSSPECAN_VAL_MEAS_SE_RESULT_FINAL5                                  4
#define RSSPECAN_VAL_MEAS_SE_RESULT_FINAL6                                  5

// rsspecan_rngGapLimitCheckState
#define RSSPECAN_VAL_GAP_LIMIT_CHECK_STATE_PASS                             0
#define RSSPECAN_VAL_GAP_LIMIT_CHECK_STATE_FAIL                             1
#define RSSPECAN_VAL_GAP_LIMIT_CHECK_STATE_NONE                             2

// RsSpecAn_rngBSCPICHPattern
#define RSSPECAN_VAL_3GPP_BS_PATTERN_OFF                                    0
#define RSSPECAN_VAL_3GPP_BS_PATTERN_1                                      1
#define RSSPECAN_VAL_3GPP_BS_PATTERN_2                                      2

// rsspecan_rngPulseResultTiming
#define RSSPECAN_VAL_PULSE_RESULT_TIMING_ALL                                0

// rsspecan_rngPulseResultFrequency
#define RSSPECAN_VAL_PULSE_RESULT_FREQ_ALL                                  0

// RsSpecAn_rngWIMAXFFTSize
#define RSSPECAN_VAL_WIMAX_FFT_SIZE_128                                     0
#define RSSPECAN_VAL_WIMAX_FFT_SIZE_512                                     1
#define RSSPECAN_VAL_WIMAX_FFT_SIZE_1024                                    2
#define RSSPECAN_VAL_WIMAX_FFT_SIZE_2048                                    3

// rsspecan_rngWIMAXPredefinedMap
#define RSSPECAN_VAL_WIMAX_PREDEFINED_MAP_CONFIG                            0
#define RSSPECAN_VAL_WIMAX_PREDEFINED_MAP_IQDL                              1
#define RSSPECAN_VAL_WIMAX_PREDEFINED_MAP_IQUL                              2
#define RSSPECAN_VAL_WIMAX_PREDEFINED_MAP_SMU                               3

// RsSpecAn_rngLTEDownlinkMIMOInstrumentState
#define RSSPECAN_VAL_MIMO_LED_RED                                           0
#define RSSPECAN_VAL_MIMO_LED_GREEN                                         1
#define RSSPECAN_VAL_MIMO_LED_GREY                                          2

// RsSpecAn_rngLTEDownlinkBeamformingAntennaPort
#define RSSPECAN_VAL_BEAMFORMING_ANTENNA_PORT_5_7                           0
#define RSSPECAN_VAL_BEAMFORMING_ANTENNA_PORT_8                             1
#define RSSPECAN_VAL_BEAMFORMING_ANTENNA_PORT_9                             2
#define RSSPECAN_VAL_BEAMFORMING_ANTENNA_PORT_10                            3
#define RSSPECAN_VAL_BEAMFORMING_ANTENNA_PORT_11                            4
#define RSSPECAN_VAL_BEAMFORMING_ANTENNA_PORT_12                            5
#define RSSPECAN_VAL_BEAMFORMING_ANTENNA_PORT_13                            6
#define RSSPECAN_VAL_BEAMFORMING_ANTENNA_PORT_14                            7
#define RSSPECAN_VAL_BEAMFORMING_ANTENNA_PORT_ALL                           8

// RsSpecAn_rngLTEDownlinkPDSCHAllocationPrecoding
#define RSSPECAN_VAL_LTE_DOWNLINK_PRECODING_NONE                            0
#define RSSPECAN_VAL_LTE_DOWNLINK_PRECODING_BF                              1
#define RSSPECAN_VAL_LTE_DOWNLINK_PRECODING_SPM                             2
#define RSSPECAN_VAL_LTE_DOWNLINK_PRECODING_TXD                             3

// RsSpecAn_rngLTEDownlinkPDSCHAllocationCodewordToLayerMapping
#define RSSPECAN_VAL_LTE_DOWNLINK_LAYER_MAPPING_11                          0
#define RSSPECAN_VAL_LTE_DOWNLINK_LAYER_MAPPING_21                          1
#define RSSPECAN_VAL_LTE_DOWNLINK_LAYER_MAPPING_31                          2
#define RSSPECAN_VAL_LTE_DOWNLINK_LAYER_MAPPING_41                          3
#define RSSPECAN_VAL_LTE_DOWNLINK_LAYER_MAPPING_22                          4
#define RSSPECAN_VAL_LTE_DOWNLINK_LAYER_MAPPING_32                          5
#define RSSPECAN_VAL_LTE_DOWNLINK_LAYER_MAPPING_42                          6
#define RSSPECAN_VAL_LTE_DOWNLINK_LAYER_MAPPING_52                          7
#define RSSPECAN_VAL_LTE_DOWNLINK_LAYER_MAPPING_62                          8
#define RSSPECAN_VAL_LTE_DOWNLINK_LAYER_MAPPING_72                          9
#define RSSPECAN_VAL_LTE_DOWNLINK_LAYER_MAPPING_82                          10

// RsSpecAn_rngLTEUplinkAutomaticDemodulation
#define RSSPECAN_VAL_LTE_AUTO_DEMOD_OFF                                     0
#define RSSPECAN_VAL_LTE_AUTO_DEMOD_SCON                                    1
#define RSSPECAN_VAL_LTE_AUTO_DEMOD_ALL                                     2

// RsSpecAn_rngLTEDownlinkTriggerPort
#define RSSPECAN_VAL_TRIGGER_PORT_1                                         1
#define RSSPECAN_VAL_TRIGGER_PORT_2                                         2
#define RSSPECAN_VAL_TRIGGER_PORT_3                                         3

// RsSpecAn_rngLTEDownlinkCarrierAxes
#define RSSPECAN_VAL_UNIT_CARR                                              0

// RsSpecAn_rngLTEDownlinkSyncAntenna
#define RSSPECAN_VAL_LTE_SYNC_ANTENNA_NONE                                  0
#define RSSPECAN_VAL_LTE_SYNC_ANTENNA_ANT1                                  1
#define RSSPECAN_VAL_LTE_SYNC_ANTENNA_ANT2                                  2
#define RSSPECAN_VAL_LTE_SYNC_ANTENNA_ANT3                                  3
#define RSSPECAN_VAL_LTE_SYNC_ANTENNA_ANT4                                  4
#define RSSPECAN_VAL_LTE_SYNC_ANTENNA_ALL                                   5

// rsspecan_rngRFInputEnabled
#define RSSPECAN_VAL_RF_INPUT_RF                                            0
#define RSSPECAN_VAL_RF_INPUT_DIQ                                           1
#define RSSPECAN_VAL_RF_INPUT_AIQ                                           2
#define RSSPECAN_VAL_RF_INPUT_FIQ                                           3
#define RSSPECAN_VAL_RF_INPUT_ABB                                           4
#define RSSPECAN_VAL_RF_INPUT_OBB                                           5
#define RSSPECAN_VAL_RF_INPUT_RF_AND_AIQ                                    6

// RsSpecAn_rngServiceCalibrationFrequencyRFSpectrum
#define RSSPECAN_VAL_CAL_FREQ_SPEC_NARR                                     0
#define RSSPECAN_VAL_CAL_FREQ_SPEC_BRO                                      1

// RsSpecAn_rngServiceCalibrationFrequencyMW
#define RSSPECAN_VAL_CAL_FREQ_MW_SMAL                                       0
#define RSSPECAN_VAL_CAL_FREQ_MW_WIDE                                       1

// RsSpecAn_rngIQBandwidthMode
#define RSSPECAN_VAL_IQ_BWID_MODE_MAN                                       0
#define RSSPECAN_VAL_IQ_BWID_MODE_AUTO                                      1
#define RSSPECAN_VAL_IQ_BWID_MODE_FFT                                       2

// RsSpecAn_rngIQFFTAlgorithm
#define RSSPECAN_VAL_IQ_FFT_ALG_SING                                        0
#define RSSPECAN_VAL_IQ_FFT_ALG_AVER                                        1

// RsSpecAn_rngIQFFTWindowType
#define RSSPECAN_VAL_IQ_WIN_TYPE_BLACK                                      0
#define RSSPECAN_VAL_IQ_WIN_TYPE_FLAT                                       1
#define RSSPECAN_VAL_IQ_WIN_TYPE_GAUS                                       2
#define RSSPECAN_VAL_IQ_WIN_TYPE_RECT                                       3
#define RSSPECAN_VAL_IQ_WIN_TYPE_P5                                         4

// rsspecan_rngBBSignalPath
#define RSSPECAN_VAL_BB_SIG_PATH_I                                          0
#define RSSPECAN_VAL_BB_SIG_PATH_Q                                          1
#define RSSPECAN_VAL_BB_SIG_PATH_IQ                                         2

// rsspecan_rngFMSTraceMode
#define RSSPECAN_VAL_FMS_TRAC_MOD_WRITE                                     0
#define RSSPECAN_VAL_FMS_TRAC_MOD_VIEW                                      1
#define RSSPECAN_VAL_FMS_TRAC_MOD_AVER                                      2
#define RSSPECAN_VAL_FMS_TRAC_MOD_MAXH                                      3
#define RSSPECAN_VAL_FMS_TRAC_MOD_MINH                                      4
#define RSSPECAN_VAL_FMS_TRAC_MOD_OFF                                       5

// RsSpecAn_rngVSAStatusRegister
#define RSSPECAN_VAL_VSA_STAT_REG_MAIN                                      0
#define RSSPECAN_VAL_VSA_STAT_REG_CFR                                       1
#define RSSPECAN_VAL_VSA_STAT_REG_EVM                                       2
#define RSSPECAN_VAL_VSA_STAT_REG_FSK                                       3
#define RSSPECAN_VAL_VSA_STAT_REG_IQRH                                      4
#define RSSPECAN_VAL_VSA_STAT_REG_MAGN                                      5
#define RSSPECAN_VAL_VSA_STAT_REG_PHAS                                      6

// RsSpecAn_rngWiMAXSEMAnalysis
#define RSSPECAN_VAL_WIMAX_SEM_ANALYSIS_DL                                  0
#define RSSPECAN_VAL_WIMAX_SEM_ANALYSIS_UL                                  1

// RsSpecAn_rngWLANToleranceLimit
#define RSSPECAN_VAL_WLAN_TOL_LIM_PRIOR11_2012                              0
#define RSSPECAN_VAL_WLAN_TOL_LIM_STD11_2012                                1
#define RSSPECAN_VAL_WLAN_TOL_LIM_P11ACD5_1                                 2

// RsSpecAn_rngPulseParameterSpectrumWindowType
#define RSSPECAN_VAL_PULS_FFT_WINDOW_BLACKMAN                               0
#define RSSPECAN_VAL_PULS_FFT_WINDOW_HANNING                                1
#define RSSPECAN_VAL_PULS_FFT_WINDOW_RECTANGLE                              2
#define RSSPECAN_VAL_PULS_FFT_WINDOW_HAMMING                                3
#define RSSPECAN_VAL_PULS_FFT_WINDOW_BARTLETT                               4

// RsSpecAn_rngPhaseSweepListResultType
#define RSSPECAN_VAL_PHASE_SWEEP_START                                      0
#define RSSPECAN_VAL_PHASE_SWEEP_STOP                                       1
#define RSSPECAN_VAL_PHASE_SWEEP_AVG                                        2
#define RSSPECAN_VAL_PHASE_SWEEP_FDR                                        3
#define RSSPECAN_VAL_PHASE_SWEEP_MDR                                        4
#define RSSPECAN_VAL_PHASE_SWEEP_LDR                                        5
#define RSSPECAN_VAL_PHASE_SWEEP_SRAT                                       6

// RsSpecAn_rngGSMMeasurement
#define RSSPECAN_VAL_GSM_MEAS_IQ                                            0
#define RSSPECAN_VAL_GSM_MEAS_MCWN                                          1

// RsSpecAn_rngGSMMCWNCarrierAllocation
#define RSSPECAN_VAL_GSM_FALL_CONT                                          0
#define RSSPECAN_VAL_GSM_FALL_NCON                                          1

// RsSpecAn_rngGSMMCWNCarrierModulation
#define RSSPECAN_VAL_GSM_CARRIER_MOD_AGMS                                   0
#define RSSPECAN_VAL_GSM_CARRIER_MOD_HN16QAM                                1
#define RSSPECAN_VAL_GSM_CARRIER_MOD_HN32QAM                                2
#define RSSPECAN_VAL_GSM_CARRIER_MOD_HNQPSK                                 3
#define RSSPECAN_VAL_GSM_CARRIER_MOD_HW16QAM                                4
#define RSSPECAN_VAL_GSM_CARRIER_MOD_HW32QAM                                5
#define RSSPECAN_VAL_GSM_CARRIER_MOD_HWQPSK                                 6
#define RSSPECAN_VAL_GSM_CARRIER_MOD_N16QAM                                 7
#define RSSPECAN_VAL_GSM_CARRIER_MOD_N32QAM                                 8
#define RSSPECAN_VAL_GSM_CARRIER_MOD_N8PSK                                  9
#define RSSPECAN_VAL_GSM_CARRIER_MOD_NAQPSK                                 10
#define RSSPECAN_VAL_GSM_CARRIER_MOD_NGMSK                                  11

// RsSpecAn_rngGSMMCWNIntermodulationOrder
#define RSSPECAN_VAL_GSM_INTERMODULATION_0                                  0
#define RSSPECAN_VAL_GSM_INTERMODULATION_3                                  1
#define RSSPECAN_VAL_GSM_INTERMODULATION_35                                 2

// RsSpecAn_rngGSMMCWNIntermodulationProducts
#define RSSPECAN_VAL_GSM_IM_INNER                                           0
#define RSSPECAN_VAL_GSM_IM_OUTER                                           1

// RsSpecAn_rngGSMLimitCheck
#define RSSPECAN_VALE_GSM_LIM_FAILED                                        0
#define RSSPECAN_VALE_GSM_LIM_PASSED                                        1
#define RSSPECAN_VALE_GSM_LIM_EXCEPTION                                     2

// RsSpecAn_rngGSMRefernecePowerType
#define RSSPECAN_VAL_GSM_RPOW_NONE                                          0
#define RSSPECAN_VAL_GSM_RPOW_REF                                           1
#define RSSPECAN_VAL_GSM_RPOW_MAX                                           2

// RsSpecAn_rngTDSTimeReference
#define RSSPECAN_VAL_TDS_TREF_DPTS                                          1
#define RSSPECAN_VAL_TDS_TREF_UPTS                                          2
#define RSSPECAN_VAL_TDS_TREF_SLOT                                          0

// RsSpecAn_rngIFOutputSideband
#define RSSPECAN_VAL_IFOUT_SBAND_NORMAL                                     0
#define RSSPECAN_VAL_IFOUT_SBAND_INVERTED                                   1

// rsspecan_rngTransientModel
#define RSSPECAN_VAL_TRANSIENT_MODEL_NONE                                   0
#define RSSPECAN_VAL_TRANSIENT_MODEL_HOP                                    1
#define RSSPECAN_VAL_TRANSIENT_MODEL_CHIRP                                  2

// rsspecan_rngTransientFFTWindow
#define RSSPECAN_VAL_TRANSIENT_FFT_WINDOW_BLACKHARRIS                       0
#define RSSPECAN_VAL_TRANSIENT_FFT_WINDOW_FLATTOP                           1
#define RSSPECAN_VAL_TRANSIENT_FFT_WINDOW_GAUSSIAN                          2
#define RSSPECAN_VAL_TRANSIENT_FFT_WINDOW_HANNING                           3
#define RSSPECAN_VAL_TRANSIENT_FFT_WINDOW_RECTANGULAR                       4
#define RSSPECAN_VAL_TRANSIENT_FFT_WINDOW_HAMMING                           5
#define RSSPECAN_VAL_TRANSIENT_FFT_WINDOW_CHEBYSHEV                         6

// rsspecan_rngTransientFMVideoBandwidth
#define RSSPECAN_VAL_TRANSIENT_FM_VIDEO_BANDWIDTH_NONE                      0
#define RSSPECAN_VAL_TRANSIENT_FM_VIDEO_BANDWIDTH_LP01                      1
#define RSSPECAN_VAL_TRANSIENT_FM_VIDEO_BANDWIDTH_LP1                       2
#define RSSPECAN_VAL_TRANSIENT_FM_VIDEO_BANDWIDTH_LP5                       3
#define RSSPECAN_VAL_TRANSIENT_FM_VIDEO_BANDWIDTH_LP10                      4
#define RSSPECAN_VAL_TRANSIENT_FM_VIDEO_BANDWIDTH_LP25                      5

// rsspecan_rngTransientReference
#define RSSPECAN_VAL_TRANSIENT_REFERENCE_EDGE                               0
#define RSSPECAN_VAL_TRANSIENT_REFERENCE_CENTER                             1
#define RSSPECAN_VAL_TRANSIENT_REFERENCE_FM_SETTLING                        2
#define RSSPECAN_VAL_TRANSIENT_REFERENCE_PM_SETTLING                        3

// rsspecan_rngResultRangeReference
#define RSSPECAN_VAL_RESULT_RANGE_REFERENCE_RISE                            0
#define RSSPECAN_VAL_RESULT_RANGE_REFERENCE_CENTER                          1
#define RSSPECAN_VAL_RESULT_RANGE_REFERENCE_FALL                            2

// rsspecan_rngResultRangeAlignment
#define RSSPECAN_VAL_RESULT_RANGE_ALIGNMENT_LEFT                            0
#define RSSPECAN_VAL_RESULT_RANGE_ALIGNMENT_CENTER                          1
#define RSSPECAN_VAL_RESULT_RANGE_ALIGNMENT_RIGHT                           2

// rsspecan_rngTransientHopHeader
#define RSSPECAN_VAL_TRANSIENT_HOP_HEADER_ALL                               0
#define RSSPECAN_VAL_TRANSIENT_HOP_HEADER_STATE                             1
#define RSSPECAN_VAL_TRANSIENT_HOP_HEADER_BEGIN                             2
#define RSSPECAN_VAL_TRANSIENT_HOP_HEADER_DWELL                             3
#define RSSPECAN_VAL_TRANSIENT_HOP_HEADER_SWITCHING                         4
#define RSSPECAN_VAL_TRANSIENT_HOP_HEADER_FREQUENCY                         5
#define RSSPECAN_VAL_TRANSIENT_HOP_HEADER_FMERROR                           6
#define RSSPECAN_VAL_TRANSIENT_HOP_HEADER_MAXFM                             7
#define RSSPECAN_VAL_TRANSIENT_HOP_HEADER_RMSFM                             8
#define RSSPECAN_VAL_TRANSIENT_HOP_HEADER_AVGFM                             9
#define RSSPECAN_VAL_TRANSIENT_HOP_HEADER_AVGPOWER                          10
#define RSSPECAN_VAL_TRANSIENT_HOP_HEADER_MIN_POWER                         11
#define RSSPECAN_VAL_TRANSIENT_HOP_HEADER_MAX_POWER                         12
#define RSSPECAN_VAL_TRANSIENT_HOP_HEADER_RIPPLE_POWER                      13
#define RSSPECAN_VAL_TRANSIENT_HOP_HEADER_STATE_FREQ                        14
#define RSSPECAN_VAL_TRANSIENT_HOP_HEADER_RELATIVE_FREQ                     15
#define RSSPECAN_VAL_TRANSIENT_HOP_HEADER_FM_SETTLING_POINT                 16
#define RSSPECAN_VAL_TRANSIENT_HOP_HEADER_FM_SETTLING_TIME                  17
#define RSSPECAN_VAL_TRANSIENT_HOP_HEADER_FM_SETTLING_LENGTH                18
#define RSSPECAN_VAL_TRANSIENT_HOP_HEADER_PM_SETTLING_POINT                 19
#define RSSPECAN_VAL_TRANSIENT_HOP_HEADER_PM_SETTLING_TIME                  20
#define RSSPECAN_VAL_TRANSIENT_HOP_HEADER_PM_SETTLING_LENGTH                21
#define RSSPECAN_VAL_TRANSIENT_HOP_HEADER_MAX_PHASE                         22
#define RSSPECAN_VAL_TRANSIENT_HOP_HEADER_RMS_PHASE                         23
#define RSSPECAN_VAL_TRANSIENT_HOP_HEADER_AVG_PHASE                         24

// rsspecan_rngTransientChirpHeader
#define RSSPECAN_VAL_TRANSIENT_CHIRP_HEADER_ALL                             0
#define RSSPECAN_VAL_TRANSIENT_CHIRP_HEADER_STATE                           1
#define RSSPECAN_VAL_TRANSIENT_CHIRP_HEADER_BEGIN                           2
#define RSSPECAN_VAL_TRANSIENT_CHIRP_HEADER_LENGTH                          3
#define RSSPECAN_VAL_TRANSIENT_CHIRP_HEADER_RATE                            4
#define RSSPECAN_VAL_TRANSIENT_CHIRP_HEADER_FREQUENCY                       5
#define RSSPECAN_VAL_TRANSIENT_CHIRP_HEADER_CHERROR                         6
#define RSSPECAN_VAL_TRANSIENT_CHIRP_HEADER_MAXFM                           7
#define RSSPECAN_VAL_TRANSIENT_CHIRP_HEADER_RMSFM                           8
#define RSSPECAN_VAL_TRANSIENT_CHIRP_HEADER_AVGFM                           9
#define RSSPECAN_VAL_TRANSIENT_CHIRP_HEADER_AVGPOWER                        10
#define RSSPECAN_VAL_TRANSIENT_CHIRP_HEADER_MIN_POWER                       11
#define RSSPECAN_VAL_TRANSIENT_CHIRP_HEADER_MAX_POWER                       12
#define RSSPECAN_VAL_TRANSIENT_CHIRP_HEADER_RIPPLE_POWER                    13
#define RSSPECAN_VAL_TRANSIENT_CHIRP_HEADER_AVG_PHASE_DEV                   14
#define RSSPECAN_VAL_TRANSIENT_CHIRP_HEADER_MAX_PHASE_DEV                   15
#define RSSPECAN_VAL_TRANSIENT_CHIRP_HEADER_RMS_PHASE_DEV                   16
#define RSSPECAN_VAL_TRANSIENT_CHIRP_HEADER_FM_SETTLING_POINT               17
#define RSSPECAN_VAL_TRANSIENT_CHIRP_HEADER_FM_SETTLING_TIME                18
#define RSSPECAN_VAL_TRANSIENT_CHIRP_HEADER_FM_SETTLED_LENGTH               19
#define RSSPECAN_VAL_TRANSIENT_CHIRP_HEADER_PM_SETTLING_POINT               20
#define RSSPECAN_VAL_TRANSIENT_CHIRP_HEADER_PM_SETTLING_TIME                21
#define RSSPECAN_VAL_TRANSIENT_CHIRP_HEADER_PM_SETTLED_LENGTH               22
#define RSSPECAN_VAL_TRANSIENT_CHIRP_HEADER_BANDWIDTH                       23
#define RSSPECAN_VAL_TRANSIENT_CHIRP_HEADER_AVG_FREQ_NON_LINEARITY          24
#define RSSPECAN_VAL_TRANSIENT_CHIRP_HEADER_RMS_FREQ_NON_LINEARITY          25
#define RSSPECAN_VAL_TRANSIENT_CHIRP_HEADER_MAX_FREQ_NON_LINEARITY          26

// rsspecan_rngTransientColumns
#define RSSPECAN_VAL_TRANSIENT_COLLUMNS_SELECTED                            0
#define RSSPECAN_VAL_TRANSIENT_COLLUMNS_ALL                                 1

// rsspecan_rngTransientEvaluationBasis
#define RSSPECAN_VAL_TRANSIENT_BASIS_FULL                                   0
#define RSSPECAN_VAL_TRANSIENT_BASIS_REGION                                 1
#define RSSPECAN_VAL_TRANSIENT_BASIS_HOP                                    2
#define RSSPECAN_VAL_TRANSIENT_BASIS_CHIRP                                  3

// rsspecan_rngTransientSpectogramDetector
#define RSSPECAN_VAL_SPECTOGRAM_DETECTOR_SUM                                0
#define RSSPECAN_VAL_SPECTOGRAM_DETECTOR_AVERAGE                            1
#define RSSPECAN_VAL_SPECTOGRAM_DETECTOR_RMS                                2
#define RSSPECAN_VAL_SPECTOGRAM_DETECTOR_MAXIMUM                            3
#define RSSPECAN_VAL_SPECTOGRAM_DETECTOR_MINIMUM                            4
#define RSSPECAN_VAL_SPECTOGRAM_DETECTOR_SAMPLE                             5

// rsspecan_rngQueryRange
#define RSSPECAN_VAL_QUERY_RANGE_SELECTED                                   0
#define RSSPECAN_VAL_QUERY_RANGE_CURRENT                                    1
#define RSSPECAN_VAL_QUERY_RANGE_ALL                                        2

// rsspecan_rngResultType
#define RSSPECAN_VAL_MEASTYPE_MIN                                           1
#define RSSPECAN_VAL_MEASTYPE_MAX                                           2
#define RSSPECAN_VAL_MEASTYPE_AVER                                          4
#define RSSPECAN_VAL_MEASTYPE_SDEV                                          3
#define RSSPECAN_VAL_MEASTYPE_CURR                                          0

// rsspecan_rngTransientParameterHopFrequency
#define RSSPECAN_VAL_HOP_RESULT_FREQUENCY_AVER                              0
#define RSSPECAN_VAL_HOP_RESULT_FREQUENCY_FMER                              1
#define RSSPECAN_VAL_HOP_RESULT_FREQUENCY_MAXD                              2
#define RSSPECAN_VAL_HOP_RESULT_FREQUENCY_RMSD                              3
#define RSSPECAN_VAL_HOP_RESULT_FREQUENCY_AVGD                              4
#define RSSPECAN_VAL_HOP_RESULT_FREQUENCY_RELF                              5
#define RSSPECAN_VAL_HOP_RESULT_FREQUENCY_FMSL                              6
#define RSSPECAN_VAL_HOP_RESULT_FREQUENCY_FMSP                              7
#define RSSPECAN_VAL_HOP_RESULT_FREQUENCY_FMST                              8
#define RSSPECAN_VAL_HOP_RESULT_FREQUENCY_PMSL                              9
#define RSSPECAN_VAL_HOP_RESULT_FREQUENCY_PMSP                              10
#define RSSPECAN_VAL_HOP_RESULT_FREQUENCY_PMST                              11

// rsspecan_rngTransientParameterHopTiming
#define RSSPECAN_VAL_HOP_RESULT_TIMING_BEGIN                                0
#define RSSPECAN_VAL_HOP_RESULT_TIMING_DWELL                                1
#define RSSPECAN_VAL_HOP_RESULT_TIMING_SWITCH                               2

// rsspecan_rngTransientParameterHopPower
#define RSSPECAN_VAL_HOP_RESULT_POWER_AVERAGE                               0
#define RSSPECAN_VAL_HOP_RESULT_POWER_MAX                                   1
#define RSSPECAN_VAL_HOP_RESULT_POWER_MIN                                   2
#define RSSPECAN_VAL_HOP_RESULT_POWER_RIPPLE                                3

// rsspecan_rngTransientParameterChirpFrequency
#define RSSPECAN_VAL_CHIRP_RESULT_FREQUENCY_AVER                            0
#define RSSPECAN_VAL_CHIRP_RESULT_FREQUENCY_CHER                            1
#define RSSPECAN_VAL_CHIRP_RESULT_FREQUENCY_MAXD                            2
#define RSSPECAN_VAL_CHIRP_RESULT_FREQUENCY_RMSD                            3
#define RSSPECAN_VAL_CHIRP_RESULT_FREQUENCY_AVGD                            4
#define RSSPECAN_VAL_CHIRP_RESULT_FREQUENCY_AVGN                            5
#define RSSPECAN_VAL_CHIRP_RESULT_FREQUENCY_MAXN                            6
#define RSSPECAN_VAL_CHIRP_RESULT_FREQUENCY_RMSN                            7
#define RSSPECAN_VAL_CHIRP_RESULT_FREQUENCY_BWID                            8
#define RSSPECAN_VAL_CHIRP_RESULT_FREQUENCY_FMSL                            9
#define RSSPECAN_VAL_CHIRP_RESULT_FREQUENCY_FMSP                            10
#define RSSPECAN_VAL_CHIRP_RESULT_FREQUENCY_FMST                            11
#define RSSPECAN_VAL_CHIRP_RESULT_FREQUENCY_PMSL                            12
#define RSSPECAN_VAL_CHIRP_RESULT_FREQUENCY_PMSP                            13
#define RSSPECAN_VAL_CHIRP_RESULT_FREQUENCY_PMST                            14
#define RSSPECAN_VAL_CHIRP_RESULT_FREQUENCY_OVER                            15
#define RSSPECAN_VAL_CHIRP_RESULT_FREQUENCY_UNDER                           16

// rsspecan_rngTransientParameterChirpTiming
#define RSSPECAN_VAL_CHIRP_RESULT_TIMING_BEGIN                              0
#define RSSPECAN_VAL_CHIRP_RESULT_TIMING_LENGTH                             1
#define RSSPECAN_VAL_CHIRP_RESULT_TIMING_RATE                               2
#define RSSPECAN_VAL_CHIRP_RESULT_TIMING_SWITCH                             3

// rsspecan_rngTransientParameterChirpPower
#define RSSPECAN_VAL_CHIRP_RESULT_POWER_AVERAGE                             0
#define RSSPECAN_VAL_CHIRP_RESULT_POWER_MAX                                 1
#define RSSPECAN_VAL_CHIRP_RESULT_POWER_MIN                                 2
#define RSSPECAN_VAL_CHIRP_RESULT_POWER_RIPPLE                              3

// RsSpecAn_rngLTEUplinkCarriersBandwidths
#define RSSPECAN_VAL_LTE_UPLINK_CABW_USER                                   0
#define RSSPECAN_VAL_LTE_UPLINK_CABW_B520                                   1
#define RSSPECAN_VAL_LTE_UPLINK_CABW_B1020                                  2
#define RSSPECAN_VAL_LTE_UPLINK_CABW_B1520                                  3
#define RSSPECAN_VAL_LTE_UPLINK_CABW_B2020                                  4
#define RSSPECAN_VAL_LTE_UPLINK_CABW_B1515                                  5

// rsspecan_rngMeasPowerSelectMSR
#define RSSPECAN_VAL_MEAS_POW_GACLR                                         8

// RsSpecAn_rngNoisePowerMeasurement
#define RSSPECAN_VAL_POWER_MEASURE_HOT                                      0
#define RSSPECAN_VAL_POWER_MEASURE_COLD                                     1

// RsSpecAn_rngNoiseENRNoiseSource
#define RSSPECAN_VAL_NOISE_SOURCE_DIODE                                     0
#define RSSPECAN_VAL_NOISE_SOURCE_RESISTOR                                  1
#define RSSPECAN_VAL_NOISE_SOURCE_SMART                                     2

// rsspecan_rngNoiseLayoutType
#define RSSPECAN_VAL_LAYOUT_TYPE_CPCOLD                                     106
#define RSSPECAN_VAL_LAYOUT_TYPE_CPHOT                                      107
#define RSSPECAN_VAL_LAYOUT_TYPE_CYFAC                                      108
#define RSSPECAN_VAL_LAYOUT_TYPE_ENR                                        109

// rsspecan_rngTransientParameterHopState
#define RSSPECAN_VAL_HOP_RESULT_STATE_STAF                                  0

// rsspecan_rngTransientFrequencyScaling
#define RSSPECAN_VAL_TRANSIENT_FREQ_SCALING_HZ                              0
#define RSSPECAN_VAL_TRANSIENT_FREQ_SCALING_KHZ                             1
#define RSSPECAN_VAL_TRANSIENT_FREQ_SCALING_MHZ                             2
#define RSSPECAN_VAL_TRANSIENT_FREQ_SCALING_GHZ                             3

// RsSpecAn_rngTransientResultFrequency
#define RSSPECAN_VAL_TRANSIENT_RESULT_FREQ_ALL                              0
#define RSSPECAN_VAL_TRANSIENT_RESULT_FREQ_AVGFM                            1
#define RSSPECAN_VAL_TRANSIENT_RESULT_FREQ_CHER                             2
#define RSSPECAN_VAL_TRANSIENT_RESULT_FREQ_FREQ                             3
#define RSSPECAN_VAL_TRANSIENT_RESULT_FREQ_MAXF                             4
#define RSSPECAN_VAL_TRANSIENT_RESULT_FREQ_RMSF                             5
#define RSSPECAN_VAL_TRANSIENT_RESULT_FREQ_BW                               6
#define RSSPECAN_VAL_TRANSIENT_RESULT_FREQ_OVER                             7
#define RSSPECAN_VAL_TRANSIENT_RESULT_FREQ_UNDER                            8

// RsSpecAn_rngTransientHopResultTiming
#define RSSPECAN_VAL_TRANSIENT_RESULT_TIMING_ALL                            0
#define RSSPECAN_VAL_TRANSIENT_RESULT_TIMING_BEGIN                          1
#define RSSPECAN_VAL_TRANSIENT_RESULT_TIMING_DWELL                          2
#define RSSPECAN_VAL_TRANSIENT_RESULT_TIMING_SWITCHING                      3

// rsspecan_rngTransientResultTimeScaling
#define RSSPECAN_VAL_TRANSIENT_TIME_SCALING_NS_HZ                           0
#define RSSPECAN_VAL_TRANSIENT_TIME_SCALING_US_KHZ                          1
#define RSSPECAN_VAL_TRANSIENT_TIME_SCALING_MS_MHZ                          2
#define RSSPECAN_VAL_TRANSIENT_TIME_SCALING_S_GHZ                           3

// RsSpecAn_rngTransientHopResultFrequency
#define RSSPECAN_VAL_TRANSIENT_RESULT_FREQ_FMER                             2
#define RSSPECAN_VAL_TRANSIENT_RESULT_FREQ_RELF                             6

// RsSpecAn_rngTransientChirpResultTiming
#define RSSPECAN_VAL_TRANSIENT_RESULT_TIMING_LENGTH                         2
#define RSSPECAN_VAL_TRANSIENT_RESULT_TIMING_RATE                           3
#define RSSPECAN_VAL_TRANSIENT_RESULT_TIMING_SWITCH                         4

// rsspecan_rngTransientParameterChirpState
#define RSSPECAN_VAL_CHIRP_RESULT_STATE_INDEX                               0

// rsspecan_rngTransientParameterHopStateResultDisplay
#define RSSPECAN_VAL_HOP_RESULT_STATE_INDEX                                 1

// RsSpecAn_rngTransientTrendXAxisResult
#define RSSPECAN_VAL_TRANSIENT_RESULT_X_AXIS_BEGIN                          0

// rsspecan_rngTransientParameterYAxis
#define RSSPECAN_VAL_TRANSIENT_RESULT_Y_AXIS_COUNT                          0
#define RSSPECAN_VAL_TRANSIENT_RESULT_Y_AXIS_OCCURANCE                      1

// rsspecan_rngTransientLayoutType
#define RSSPECAN_VAL_LAYOUT_TYPE_RFPT                                       106
#define RSSPECAN_VAL_LAYOUT_TYPE_FMT                                        107
#define RSSPECAN_VAL_LAYOUT_TYPE_FDEV                                       108
#define RSSPECAN_VAL_LAYOUT_TYPE_PMT                                        109
#define RSSPECAN_VAL_LAYOUT_TYPE_PMWR                                       110
#define RSSPECAN_VAL_LAYOUT_TYPE_RFSP                                       111
#define RSSPECAN_VAL_LAYOUT_TYPE_CRT                                        112
#define RSSPECAN_VAL_LAYOUT_TYPE_PDEV                                       113
#define RSSPECAN_VAL_LAYOUT_TYPE_IQTIME                                     114

// RsSpecAn_rngLEDState
#define RSSPECAN_VAL_LED_STATE_RED                                          0
#define RSSPECAN_VAL_LED_STATE_GREEN                                        1
#define RSSPECAN_VAL_LED_STATE_UNKNOWN                                      2

// RsSpecAn_rngAmplifierInputConnector
#define RSSPECAN_VAL_INPUT_CONNECTOR_RF                                     0
#define RSSPECAN_VAL_INPUT_CONNECTOR_AIQI                                   1

// RsSpecAn_rngAmplifierGeneratorTargetPath
#define RSSPECAN_VAL_GENERATOR_TARGET_PATH_A                                0
#define RSSPECAN_VAL_GENERATOR_TARGET_PATH_B                                1

// RsSpecAn_rngAmplifierSynchronizationDomain
#define RSSPECAN_VAL_SYNCHRONIZATION_DOMAIN_IQDIRECT                        0
#define RSSPECAN_VAL_SYNCHRONIZATION_DOMAIN_IQPDIFF                         1
#define RSSPECAN_VAL_SYNCHRONIZATION_DOMAIN_IMAGNITUDE                      2
#define RSSPECAN_VAL_SYNCHRONIZATION_DOMAIN_NONE                            3

// RsSpecAn_rngAmplifierSelectingDPDShapingMethod
#define RSSPECAN_VAL_SELECTING_DPD_SHAPING_METHOD_POLYNOMIAL                0
#define RSSPECAN_VAL_SELECTING_DPD_SHAPING_METHOD_TABLE                     1

// RsSpecAn_rngAmplifierDPDModelingOrder
#define RSSPECAN_VAL_DPD_MODELING_ORDER_AM_FIRST                            0
#define RSSPECAN_VAL_DPD_MODELING_ORDER_PM_FIRST                            1

// RsSpecAn_rngAmplifierSelectingTraceInformationMode
#define RSSPECAN_VAL_SELECTING_TRACE_INFORMATION_MODE_BBI                   0
#define RSSPECAN_VAL_SELECTING_TRACE_INFORMATION_MODE_BLANK                 1
#define RSSPECAN_VAL_SELECTING_TRACE_INFORMATION_MODE_BBPOWER               2
#define RSSPECAN_VAL_SELECTING_TRACE_INFORMATION_MODE_BBQ                   3
#define RSSPECAN_VAL_SELECTING_TRACE_INFORMATION_MODE_MODEL                 4
#define RSSPECAN_VAL_SELECTING_TRACE_INFORMATION_MODE_REFERENCE             5
#define RSSPECAN_VAL_SELECTING_TRACE_INFORMATION_MODE_RF                    6
#define RSSPECAN_VAL_SELECTING_TRACE_INFORMATION_MODE_WRIT                  7

// RsSpecAn_rngAmplifierResultSummaryTableItem
#define RSSPECAN_VAL_RESULT_SUMMARY_REVM                                    0
#define RSSPECAN_VAL_RESULT_SUMMARY_RMEV                                    1
#define RSSPECAN_VAL_RESULT_SUMMARY_FERR                                    2
#define RSSPECAN_VAL_RESULT_SUMMARY_SRER                                    3
#define RSSPECAN_VAL_RESULT_SUMMARY_MERR                                    4
#define RSSPECAN_VAL_RESULT_SUMMARY_PERR                                    5
#define RSSPECAN_VAL_RESULT_SUMMARY_QERR                                    6
#define RSSPECAN_VAL_RESULT_SUMMARY_GIMB                                    7
#define RSSPECAN_VAL_RESULT_SUMMARY_IQIM                                    8
#define RSSPECAN_VAL_RESULT_SUMMARY_IQOF                                    9
#define RSSPECAN_VAL_RESULT_SUMMARY_PINP                                    10
#define RSSPECAN_VAL_RESULT_SUMMARY_POUT                                    11
#define RSSPECAN_VAL_RESULT_SUMMARY_GAIN                                    12
#define RSSPECAN_VAL_RESULT_SUMMARY_CFIN                                    13
#define RSSPECAN_VAL_RESULT_SUMMARY_CFOU                                    14
#define RSSPECAN_VAL_RESULT_SUMMARY_AMW                                     15
#define RSSPECAN_VAL_RESULT_SUMMARY_PMW                                     16
#define RSSPECAN_VAL_RESULT_SUMMARY_BBIV                                    17
#define RSSPECAN_VAL_RESULT_SUMMARY_BBQV                                    18
#define RSSPECAN_VAL_RESULT_SUMMARY_VCC                                     19
#define RSSPECAN_VAL_RESULT_SUMMARY_ICC                                     20
#define RSSPECAN_VAL_RESULT_SUMMARY_BBP                                     21
#define RSSPECAN_VAL_RESULT_SUMMARY_APAE                                    22
#define RSSPECAN_VAL_RESULT_SUMMARY_P1DB                                    23
#define RSSPECAN_VAL_RESULT_SUMMARY_P2DB                                    24
#define RSSPECAN_VAL_RESULT_SUMMARY_P3DB                                    25
#define RSSPECAN_VAL_RESULT_SUMMARY_OUTPUT_P1DB                             26
#define RSSPECAN_VAL_RESULT_SUMMARY_OUTPUT_P2DB                             27
#define RSSPECAN_VAL_RESULT_SUMMARY_OUTPUT_P3DB                             28
#define RSSPECAN_VAL_RESULT_SUMMARY_AVG_POWER_CONSUMPTION                   29
#define RSSPECAN_VAL_RESULT_SUMMARY_PC_BASED_APAE                           30

// RsSpecAn_rngAmplifierGainXAxis
#define RSSPECAN_VAL_GAIN_X_AXIS_INPUT                                      0
#define RSSPECAN_VAL_GAIN_X_AXIS_OUTPUT                                     1

// RsSpecAn_rngAmplifierGeneratorExternalOscilatorReferenceFrequency
#define RSSPECAN_VAL_REFRENCE_FREQUENCY_EXT                                 0
#define RSSPECAN_VAL_REFRENCE_FREQUENCY_INT                                 1

// rsspecan_rngAmplifierLayoutQueryWindowType
#define RSSPECAN_VAL_LAYOUT_TYPE_GC                                         106
#define RSSPECAN_VAL_LAYOUT_TYPE_IMAG                                       107
#define RSSPECAN_VAL_LAYOUT_TYPE_ISP                                        108
#define RSSPECAN_VAL_LAYOUT_TYPE_PAEI                                       109
#define RSSPECAN_VAL_LAYOUT_TYPE_PAET                                       110
#define RSSPECAN_VAL_LAYOUT_TYPE_PVT                                        111
#define RSSPECAN_VAL_LAYOUT_TYPE_QMAG                                       112
#define RSSPECAN_VAL_LAYOUT_TYPE_QSP                                        113
#define RSSPECAN_VAL_LAYOUT_TYPE_REVM                                       114
#define RSSPECAN_VAL_LAYOUT_TYPE_RFM                                        115
#define RSSPECAN_VAL_LAYOUT_TYPE_ARFSP                                      116
#define RSSPECAN_VAL_LAYOUT_TYPE_ARTAB                                      117
#define RSSPECAN_VAL_LAYOUT_TYPE_SEVM                                       118
#define RSSPECAN_VAL_LAYOUT_TYPE_ACP                                        119
#define RSSPECAN_VAL_LAYOUT_TYPE_PAEO                                       120
#define RSSPECAN_VAL_LAYOUT_TYPE_PSW                                        121
#define RSSPECAN_VAL_LAYOUT_TYPE_PTAB                                       122
#define RSSPECAN_VAL_LAYOUT_TYPE_VICC                                       123

// rsspecan_rngGSMLayoutType
#define RSSPECAN_VAL_LAYOUT_TYPE_ETIM                                       106
#define RSSPECAN_VAL_LAYOUT_TYPE_MSFD                                       107
#define RSSPECAN_VAL_LAYOUT_TYPE_MST                                        108
#define RSSPECAN_VAL_LAYOUT_TYPE_PERR                                       109
#define RSSPECAN_VAL_LAYOUT_TYPE_PTF                                        110
#define RSSPECAN_VAL_LAYOUT_TYPE_TGSG                                       111
#define RSSPECAN_VAL_LAYOUT_TYPE_TGST                                       112
#define RSSPECAN_VAL_LAYOUT_TYPE_TSFD                                       113
#define RSSPECAN_VAL_LAYOUT_TYPE_TST                                        114
#define RSSPECAN_VAL_LAYOUT_TYPE_IIMP                                       115
#define RSSPECAN_VAL_LAYOUT_TYPE_INAR                                       116
#define RSSPECAN_VAL_LAYOUT_TYPE_IWID                                       117
#define RSSPECAN_VAL_LAYOUT_TYPE_OIMP                                       118
#define RSSPECAN_VAL_LAYOUT_TYPE_ONAR                                       119
#define RSSPECAN_VAL_LAYOUT_TYPE_OWID                                       120
#define RSSPECAN_VAL_LAYOUT_TYPE_WSFD                                       121
#define RSSPECAN_VAL_LAYOUT_TYPE_WSRP                                       122

// RsSpecAn_rngGSMMCWNInnerOutter
#define RSSPECAN_VAL_GSM_INNER                                              0
#define RSSPECAN_VAL_GSM_OUTER                                              1

// rsspecan_rngVSALayoutType
#define RSSPECAN_VAL_LAYOUT_TYPE_MCOM                                       106

// RsSpecAn_rngCarrierPattern
#define RSSPECAN_VAL_CARRIER_PATTERN_NONE                                   0
#define RSSPECAN_VAL_CARRIER_PATTERN_1                                      1
#define RSSPECAN_VAL_CARRIER_PATTERN_2                                      2

// RsSpecAn_rngWLANReferenceFrequencyCoupling
#define RSSPECAN_VAL_WLAN_COUPLING_OFF                                      0
#define RSSPECAN_VAL_WLAN_COUPLING_AUTO                                     1
#define RSSPECAN_VAL_WLAN_COUPLING_EXTERNAL                                 2

// RsSpecAn_rngDirectPath
#define RSSPECAN_VAL_DIRECT_PATH_OFF                                        0
#define RSSPECAN_VAL_DIRECT_PATH_AUTO                                       1

// RsSpecAn_rngTriggerIFPowerCoupling
#define RSSPECAN_VAL_IF_POWER_COUPLING_DC_50_OHM                            0
#define RSSPECAN_VAL_IF_POWER_COUPLING_DC_1_M_OHM                           1
#define RSSPECAN_VAL_IF_POWER_COUPLING_AC                                   2

// RsSpecAn_rngOscilloscopeLANConnectionState
#define RSSPECAN_VAL_LAN_CONNECTION_RED                                     0
#define RSSPECAN_VAL_LAN_CONNECTION_GREEN                                   1
#define RSSPECAN_VAL_LAN_CONNECTION_GRAY                                    2

// rsspecan_rngLTEDownlinkLayoutType
#define RSSPECAN_VAL_LAYOUT_TYPE_TAL                                        106
#define RSSPECAN_VAL_LAYOUT_TYPE_CRWM                                       107
#define RSSPECAN_VAL_LAYOUT_TYPE_CRWP                                       108
#define RSSPECAN_VAL_LAYOUT_TYPE_IRWM                                       109
#define RSSPECAN_VAL_LAYOUT_TYPE_IRWP                                       110
#define RSSPECAN_VAL_LAYOUT_TYPE_URWA                                       111
#define RSSPECAN_VAL_LAYOUT_TYPE_URWP                                       112
#define RSSPECAN_VAL_LAYOUT_TYPE_DIAG                                       113
#define RSSPECAN_VAL_LAYOUT_TYPE_FAL1                                       114
#define RSSPECAN_VAL_LAYOUT_TYPE_FAL2                                       115
#define RSSPECAN_VAL_LAYOUT_TYPE_OOPL                                       116
#define RSSPECAN_VAL_LAYOUT_TYPE_RIS1                                       117
#define RSSPECAN_VAL_LAYOUT_TYPE_RIS2                                       118
#define RSSPECAN_VAL_LAYOUT_TYPE_PEAK                                       119

// rsspecan_rngLTEUplinkLayoutType
#define RSSPECAN_VAL_LAYOUT_TYPE_FERR                                       107

// RsSpecAn_rngSpectrumEmissionMaskMSRBaseStationClass
#define RSSPECAN_VAL_SEM_MSR_BS_CLASS_WIDE                                  0
#define RSSPECAN_VAL_SEM_MSR_BS_CLASS_MEDIUM                                1
#define RSSPECAN_VAL_SEM_MSR_BS_CLASS_LOCAL                                 2

// RsSpecAn_rngLTEDownlinkOnOffUnit
#define RSSPECAN_VAL_UNIT_DBHZ                                              1

// RsSpecAn_rngSpectrumEmissionMaskMSRBands
#define RSSPECAN_VAL_SEM_MSR_BAND_LOW                                       0
#define RSSPECAN_VAL_SEM_MSR_BAND_HIGH                                      1

// RsSpecAn_rngSpectrumEmissionMaskRangeMultiLimitCalc
#define RSSPECAN_VAL_SEM_RANGE_MULTI_LIMIT_CALC_NONE                        0
#define RSSPECAN_VAL_SEM_RANGE_MULTI_LIMIT_CALC_SUM                         1
#define RSSPECAN_VAL_SEM_RANGE_MULTI_LIMIT_CALC_MAX                         2

// RsSpecAn_rngTransientResultPhase
#define RSSPECAN_VAL_TRANSIENT_RESULT_PHASE_ALL                             0
#define RSSPECAN_VAL_TRANSIENT_RESULT_PHASE_PEAK                            1
#define RSSPECAN_VAL_TRANSIENT_RESULT_PHASE_RMS                             2
#define RSSPECAN_VAL_TRANSIENT_RESULT_PHASE_AVERAGE                         3
#define RSSPECAN_VAL_TRANSIENT_RESULT_PHASE_OVER                            4
#define RSSPECAN_VAL_TRANSIENT_RESULT_PHASE_UNDER                           5

// rsspecan_rngTransientResultPhaseScaling
#define RSSPECAN_VAL_TRANSIENT_PHASE_SCALING_DEG                            0
#define RSSPECAN_VAL_TRANSIENT_PHASE_SCALING_RAD                            1

// rsspecan_rngTransientParameterChirpPhase
#define RSSPECAN_VAL_CHIRP_RESULT_PHASE_AVERAGE                             0
#define RSSPECAN_VAL_CHIRP_RESULT_PHASE_MAX                                 1
#define RSSPECAN_VAL_CHIRP_RESULT_PHASE_RMS                                 2
#define RSSPECAN_VAL_CHIRP_RESULT_PHASE_OVER                                3
#define RSSPECAN_VAL_CHIRP_RESULT_PHASE_UNDER                               4

// rsspecan_rngTransientParameterHopPhase
#define RSSPECAN_VAL_HOP_RESULT_PHASE_AVERAGE                               0
#define RSSPECAN_VAL_HOP_RESULT_PHASE_MAX                                   1
#define RSSPECAN_VAL_HOP_RESULT_PHASE_RMS                                   2

// RsSpecAn_rngPulseFMVideoBandwidth
#define RSSPECAN_VAL_PULSE_FM_VIDEO_BAND_NONE                               0
#define RSSPECAN_VAL_PULSE_FM_VIDEO_BAND_LP01                               1
#define RSSPECAN_VAL_PULSE_FM_VIDEO_BAND_LP1                                2
#define RSSPECAN_VAL_PULSE_FM_VIDEO_BAND_LP5                                3
#define RSSPECAN_VAL_PULSE_FM_VIDEO_BAND_LP10                               4
#define RSSPECAN_VAL_PULSE_FM_VIDEO_BAND_LP25                               5

// RsSpecAn_rngDOCSISNFFT
#define RSSPECAN_VAL_DOCSIS_NFFT_FFT4K                                      0
#define RSSPECAN_VAL_DOCSIS_NFFT_FFT8K                                      1
#define RSSPECAN_VAL_DOCSIS_NFFT_FFT2K                                      2

// RsSpecAn_rngDOCSISCyclicPrefixCP
#define RSSPECAN_VAL_DOCSIS_CYCLIC_PREFIX_CP_AUTO                           0
#define RSSPECAN_VAL_DOCSIS_CYCLIC_PREFIX_CP_192                            1
#define RSSPECAN_VAL_DOCSIS_CYCLIC_PREFIX_CP_256                            2
#define RSSPECAN_VAL_DOCSIS_CYCLIC_PREFIX_CP_512                            3
#define RSSPECAN_VAL_DOCSIS_CYCLIC_PREFIX_CP_768                            4
#define RSSPECAN_VAL_DOCSIS_CYCLIC_PREFIX_CP_1024                           5
#define RSSPECAN_VAL_DOCSIS_CYCLIC_PREFIX_CP_96                             6
#define RSSPECAN_VAL_DOCSIS_CYCLIC_PREFIX_CP_128                            7
#define RSSPECAN_VAL_DOCSIS_CYCLIC_PREFIX_CP_160                            8
#define RSSPECAN_VAL_DOCSIS_CYCLIC_PREFIX_CP_224                            9
#define RSSPECAN_VAL_DOCSIS_CYCLIC_PREFIX_CP_288                            10
#define RSSPECAN_VAL_DOCSIS_CYCLIC_PREFIX_CP_320                            11
#define RSSPECAN_VAL_DOCSIS_CYCLIC_PREFIX_CP_384                            12
#define RSSPECAN_VAL_DOCSIS_CYCLIC_PREFIX_CP_640                            13

// RsSpecAn_rngDOCSISRollOff
#define RSSPECAN_VAL_DOCSIS_ROLL_OFF_AMRO                                   0
#define RSSPECAN_VAL_DOCSIS_ROLL_OFF_S0                                     1
#define RSSPECAN_VAL_DOCSIS_ROLL_OFF_S64                                    2
#define RSSPECAN_VAL_DOCSIS_ROLL_OFF_S128                                   3
#define RSSPECAN_VAL_DOCSIS_ROLL_OFF_S192                                   4
#define RSSPECAN_VAL_DOCSIS_ROLL_OFF_S256                                   5
#define RSSPECAN_VAL_DOCSIS_ROLL_OFF_S32                                    6
#define RSSPECAN_VAL_DOCSIS_ROLL_OFF_S96                                    7
#define RSSPECAN_VAL_DOCSIS_ROLL_OFF_S160                                   8
#define RSSPECAN_VAL_DOCSIS_ROLL_OFF_S224                                   9

// RsSpecAn_rngDOCSISNCPModulation
#define RSSPECAN_VAL_DOCSIS_NCP_MODULATION_QPSK                             1
#define RSSPECAN_VAL_DOCSIS_NCP_MODULATION_16_QAM                           2
#define RSSPECAN_VAL_DOCSIS_NCP_MODULATION_64_QAM                           3

// RsSpecAn_rngDOCSISConfigureType
#define RSSPECAN_VAL_DOCSIS_CONFIGURE_TYPE_PLC                              0
#define RSSPECAN_VAL_DOCSIS_CONFIGURE_TYPE_CPIL                             1
#define RSSPECAN_VAL_DOCSIS_CONFIGURE_TYPE_ESUB                             2

// RsSpecAn_rngDOCSISProfileList
#define RSSPECAN_VAL_DOCSIS_PROFILE_LIST_SELECT_NONE                        0
#define RSSPECAN_VAL_DOCSIS_PROFILE_LIST_SELECT_A                           1
#define RSSPECAN_VAL_DOCSIS_PROFILE_LIST_SELECT_B                           2
#define RSSPECAN_VAL_DOCSIS_PROFILE_LIST_SELECT_C                           3
#define RSSPECAN_VAL_DOCSIS_PROFILE_LIST_SELECT_D                           4
#define RSSPECAN_VAL_DOCSIS_PROFILE_LIST_SELECT_E                           5
#define RSSPECAN_VAL_DOCSIS_PROFILE_LIST_SELECT_F                           6
#define RSSPECAN_VAL_DOCSIS_PROFILE_LIST_SELECT_G                           7
#define RSSPECAN_VAL_DOCSIS_PROFILE_LIST_SELECT_H                           8
#define RSSPECAN_VAL_DOCSIS_PROFILE_LIST_SELECT_I                           9
#define RSSPECAN_VAL_DOCSIS_PROFILE_LIST_SELECT_J                           10
#define RSSPECAN_VAL_DOCSIS_PROFILE_LIST_SELECT_K                           11
#define RSSPECAN_VAL_DOCSIS_PROFILE_LIST_SELECT_L                           12
#define RSSPECAN_VAL_DOCSIS_PROFILE_LIST_SELECT_M                           13
#define RSSPECAN_VAL_DOCSIS_PROFILE_LIST_SELECT_N                           14
#define RSSPECAN_VAL_DOCSIS_PROFILE_LIST_SELECT_O                           15
#define RSSPECAN_VAL_DOCSIS_PROFILE_LIST_SELECT_P                           16

// RsSpecAn_rngDOCSISPLCModulation
#define RSSPECAN_VAL_DOCSIS_PLC_MODULATION_16_QAM                           0

// RsSpecAn_rngDOCSISAutoDetectionContinuousPilots
#define RSSPECAN_VAL_DOCSIS_CONTINUOUS_PILOTS_SIGNAL                        0
#define RSSPECAN_VAL_DOCSIS_CONTINUOUS_PILOTS_USER                          1

// RsSpecAn_rngPulseTimeSidelobeRange
#define RSSPECAN_VAL_PULSE_TS_RANGE_RESULT                                  0
#define RSSPECAN_VAL_PULSE_TS_RANGE_MANUAL                                  1

// RsSpecAn_rngPulseTimeSidelobeAlignment
#define RSSPECAN_VAL_PULSE_ALIGNMENT_LEFT                                   0
#define RSSPECAN_VAL_PULSE_ALIGNMENT_CENTER                                 1
#define RSSPECAN_VAL_PULSE_ALIGNMENT_RIGHT                                  2

// RsSpecAn_rngDOCSISModulation
#define RSSPECAN_VAL_DOCSIS_MODULATION_QPSK                                 0
#define RSSPECAN_VAL_DOCSIS_MODULATION_QAM16                                1
#define RSSPECAN_VAL_DOCSIS_MODULATION_QAM64                                2
#define RSSPECAN_VAL_DOCSIS_MODULATION_QAM128                               3
#define RSSPECAN_VAL_DOCSIS_MODULATION_QAM256                               4
#define RSSPECAN_VAL_DOCSIS_MODULATION_QAM512                               5
#define RSSPECAN_VAL_DOCSIS_MODULATION_QAM1024                              6
#define RSSPECAN_VAL_DOCSIS_MODULATION_QAM2048                              7
#define RSSPECAN_VAL_DOCSIS_MODULATION_QAM4096                              8
#define RSSPECAN_VAL_DOCSIS_MODULATION_QAM8192                              9
#define RSSPECAN_VAL_DOCSIS_MODULATION_QAM16384                             10
#define RSSPECAN_VAL_DOCSIS_MODULATION_ZEROBIT                              11

// RsSpecAn_rngDOCSISAutoDetectionNCPAuto
#define RSSPECAN_VAL_DOCSIS_NCP_DETECTION_AUTO_SIGNAL                       0
#define RSSPECAN_VAL_DOCSIS_NCP_DETECTION_AUTO_USER                         1

// RsSpecAn_rngDOCSISInputDirectPath
#define RSSPECAN_VAL_DOCSIS_OFF                                             0
#define RSSPECAN_VAL_DOCSIS_AUTO                                            1

// RsSpecAn_rngDOCSISCarrierAxesUnit
#define RSSPECAN_VAL_DOCSIS_CARRIER_AXES_UNIT_CARR                          0
#define RSSPECAN_VAL_DOCSIS_CARRIER_AXES_UNIT_HZ                            1

// RsSpecAn_rngDOCSISQueryCalcLimitCheck
#define RSSPECAN_VAL_DOCSIS_QUERY_LIMIT_CHECK_FAILED                        0
#define RSSPECAN_VAL_DOCSIS_QUERY_LIMIT_CHECK_PASSED                        1
#define RSSPECAN_VAL_DOCSIS_QUERY_LIMIT_CHECK_NOT_EVALUATED                 2

// RsSpecAn_rngDOCSISAutoMode
#define RSSPECAN_VAL_DOCSIS_AUTO_MODE_HYSTERESIS                            0
#define RSSPECAN_VAL_DOCSIS_AUTO_MODE_MEMORY                                1

// RsSpecAn_rngDOCSISAutoFixRange
#define RSSPECAN_VAL_DOCSIS_NONE                                            0
#define RSSPECAN_VAL_DOCSIS_LOWER                                           1
#define RSSPECAN_VAL_DOCSIS_UPPER                                           2

// RsSpecAn_rngPulseReferencePulsePulseMeasurementsMode
#define RSSPECAN_VAL_PULSE_MEAS_MODE_FIXED                                  0
#define RSSPECAN_VAL_PULSE_MEAS_MODE_SELECTED                               1
#define RSSPECAN_VAL_PULSE_MEAS_MODE_BEFORE_PULSE                           2
#define RSSPECAN_VAL_PULSE_MEAS_MODE_AFTER_PULSE                            3

// rsspecan_rngPulseLayoutType
#define RSSPECAN_VAL_LAYOUT_TYPE_CPM                                        106
#define RSSPECAN_VAL_LAYOUT_TYPE_CMC                                        107
#define RSSPECAN_VAL_LAYOUT_TYPE_PFER                                       108
#define RSSPECAN_VAL_LAYOUT_TYPE_PPER                                       109
#define RSSPECAN_VAL_LAYOUT_TYPE_PPSP                                       134
#define RSSPECAN_VAL_LAYOUT_TYPE_SWAT                                       135

// RsSpecAn_rngPulseReferenceIQType
#define RSSPECAN_VAL_PULSE_REFERENCE_IQ_TYPE_FIQ                            0
#define RSSPECAN_VAL_PULSE_REFERENCE_IQ_TYPE_PFM                            1
#define RSSPECAN_VAL_PULSE_REFERENCE_IQ_TYPE_BARK                           2
#define RSSPECAN_VAL_PULSE_REFERENCE_IQ_TYPE_EBAR                           3

// RsSpecAn_rngPulseTraceNormalization
#define RSSPECAN_VAL_PULSE_TRACE_NORMALIZATION_OFF                          0
#define RSSPECAN_VAL_PULSE_TRACE_NORMALIZATION_MEASURED                     1
#define RSSPECAN_VAL_PULSE_TRACE_NORMALIZATION_REFERENCE                    2

// RsSpecAn_rngPulseParameterTimeSidelobe
#define RSSPECAN_VAL_PULSE_RESULT_TIME_SIDELOBE_PS_LEVEL                    1
#define RSSPECAN_VAL_PULSE_RESULT_TIME_SIDELOBE_IS_LEVEL                    2
#define RSSPECAN_VAL_PULSE_RESULT_TIME_SIDELOBE_MWIDTH                      3
#define RSSPECAN_VAL_PULSE_RESULT_TIME_SIDELOBE_SDELAY                      4
#define RSSPECAN_VAL_PULSE_RESULT_TIME_SIDELOBE_CRATIO                      5
#define RSSPECAN_VAL_PULSE_RESULT_TIME_SIDELOBE_IM_POWER                    6
#define RSSPECAN_VAL_PULSE_RESULT_TIME_SIDELOBE_AM_POWER                    7
#define RSSPECAN_VAL_PULSE_RESULT_TIME_SIDELOBE_PCOR                        8
#define RSSPECAN_VAL_PULSE_RESULT_TIME_SIDELOBE_MPHASE                      9
#define RSSPECAN_VAL_PULSE_RESULT_TIME_SIDELOBE_MFREQUENCY                  10

// RsSpecAn_rngAmplifierOrder
#define RSSPECAN_VAL_ORDER_AM_FIRST                                         0
#define RSSPECAN_VAL_ORDER_PM_FIRST                                         1

// RsSpecAn_rngAmplifierParameterSweepSetting
#define RSSPECAN_VAL_AMPLIFIER_PSWEEP_BIAS                                  0
#define RSSPECAN_VAL_AMPLIFIER_PSWEEP_DELAY                                 1
#define RSSPECAN_VAL_AMPLIFIER_PSWEEP_FREQUENCY                             2
#define RSSPECAN_VAL_AMPLIFIER_PSWEEP_POWER                                 3

// rsspecan_rngAmplifierParametersSweepResultType
#define RSSPECAN_VAL_RESULT_PSWEEP_ACP                                      0
#define RSSPECAN_VAL_RESULT_PSWEEP_ACL1                                     1
#define RSSPECAN_VAL_RESULT_PSWEEP_ACU1                                     2
#define RSSPECAN_VAL_RESULT_PSWEEP_AMW                                      3
#define RSSPECAN_VAL_RESULT_PSWEEP_CFAC                                     4
#define RSSPECAN_VAL_RESULT_PSWEEP_EVM                                      5
#define RSSPECAN_VAL_RESULT_PSWEEP_GAIN                                     6
#define RSSPECAN_VAL_RESULT_PSWEEP_ICC                                      7
#define RSSPECAN_VAL_RESULT_PSWEEP_VCC                                      8
#define RSSPECAN_VAL_RESULT_PSWEEP_POW                                      9
#define RSSPECAN_VAL_RESULT_PSWEEP_PAE                                      10
#define RSSPECAN_VAL_RESULT_PSWEEP_PMW                                      11
#define RSSPECAN_VAL_RESULT_PSWEEP_RMS                                      12
#define RSSPECAN_VAL_RESULT_PSWEEP_ACBM                                     13

// rsspecan_rngPulseLimitResults
#define RSSPECAN_VAL_PULSE_LIMIT_PASS                                       0
#define RSSPECAN_VAL_PULSE_LIMIT_FLOW                                       1
#define RSSPECAN_VAL_PULSE_LIMIT_FHIGH                                      2

// RsSpecAn_rngLTEUplinkSubframeAllocationPrecodingCodewordToLayerMapping
#define RSSPECAN_VAL_LTE_CODEWORD_TO_LAYER_MAPPING_1_1                      0
#define RSSPECAN_VAL_LTE_CODEWORD_TO_LAYER_MAPPING_2_1                      1
#define RSSPECAN_VAL_LTE_CODEWORD_TO_LAYER_MAPPING_2_2                      2

// rsspecan_rngDOCSISLayoutType
#define RSSPECAN_VAL_LAYOUT_TYPE_MERC                                       106
#define RSSPECAN_VAL_LAYOUT_TYPE_MERS                                       107
#define RSSPECAN_VAL_LAYOUT_TYPE_MERSC                                      108
#define RSSPECAN_VAL_LAYOUT_TYPE_SCD                                        110
#define RSSPECAN_VAL_LAYOUT_TYPE_SCS                                        117
#define RSSPECAN_VAL_LAYOUT_TYPE_SBP                                        140

// RsSpecAn_rngDOCSISBitstream
#define RSSPECAN_VAL_DOCSIS_BITSTREAM_IBDP                                  0
#define RSSPECAN_VAL_DOCSIS_BITSTREAM_RBIT                                  1
#define RSSPECAN_VAL_DOCSIS_BITSTREAM_RBD                                   2
#define RSSPECAN_VAL_DOCSIS_BITSTREAM_IBLD                                  3
#define RSSPECAN_VAL_DOCSIS_BITSTREAM_OBLD                                  4

// RsSpecAn_rngDOCSISChannelEstimation
#define RSSPECAN_VAL_DOCSIS_CHANNEL_ESTIMATION_OFF                          0
#define RSSPECAN_VAL_DOCSIS_CHANNEL_ESTIMATION_DOCS                         1
#define RSSPECAN_VAL_DOCSIS_CHANNEL_ESTIMATION_PILOTS                       2
#define RSSPECAN_VAL_DOCSIS_CHANNEL_ESTIMATION_PDATA                        3
#define RSSPECAN_VAL_DOCSIS_CHANNEL_ESTIMATION_EMER                         4
#define RSSPECAN_VAL_DOCSIS_CHANNEL_ESTIMATION_UMER                         5
#define RSSPECAN_VAL_DOCSIS_CHANNEL_ESTIMATION_PEQ                          6

// rsspecan_rngDOCSISConstellationModulation
#define RSSPECAN_VAL_DOCSIS_CONSTELLATION_MODULATION_ALL                    0
#define RSSPECAN_VAL_DOCSIS_CONSTELLATION_MODULATION_ZEROBIT                1
#define RSSPECAN_VAL_DOCSIS_CONSTELLATION_MODULATION_BPSK                   2
#define RSSPECAN_VAL_DOCSIS_CONSTELLATION_MODULATION_QPSK                   3
#define RSSPECAN_VAL_DOCSIS_CONSTELLATION_MODULATION_QAM16                  4
#define RSSPECAN_VAL_DOCSIS_CONSTELLATION_MODULATION_QAM64                  5
#define RSSPECAN_VAL_DOCSIS_CONSTELLATION_MODULATION_QAM128                 6
#define RSSPECAN_VAL_DOCSIS_CONSTELLATION_MODULATION_QAM256                 7
#define RSSPECAN_VAL_DOCSIS_CONSTELLATION_MODULATION_QAM512                 8
#define RSSPECAN_VAL_DOCSIS_CONSTELLATION_MODULATION_QAM1024                9
#define RSSPECAN_VAL_DOCSIS_CONSTELLATION_MODULATION_QAM2048                10
#define RSSPECAN_VAL_DOCSIS_CONSTELLATION_MODULATION_QAM4096                11
#define RSSPECAN_VAL_DOCSIS_CONSTELLATION_MODULATION_QAM8192                12
#define RSSPECAN_VAL_DOCSIS_CONSTELLATION_MODULATION_QAM16384               13

// rsspecan_rngDOCSISConstellationObject
#define RSSPECAN_VAL_DOCSIS_CONSTELLATION_OBJECT_ALL                        0
#define RSSPECAN_VAL_DOCSIS_CONSTELLATION_OBJECT_ZEROBIT                    1
#define RSSPECAN_VAL_DOCSIS_CONSTELLATION_OBJECT_BPSK                       2
#define RSSPECAN_VAL_DOCSIS_CONSTELLATION_OBJECT_QPSK                       3
#define RSSPECAN_VAL_DOCSIS_CONSTELLATION_OBJECT_QAM16                      4
#define RSSPECAN_VAL_DOCSIS_CONSTELLATION_OBJECT_QAM64                      5
#define RSSPECAN_VAL_DOCSIS_CONSTELLATION_OBJECT_QAM128                     6
#define RSSPECAN_VAL_DOCSIS_CONSTELLATION_OBJECT_QAM256                     7
#define RSSPECAN_VAL_DOCSIS_CONSTELLATION_OBJECT_QAM512                     8
#define RSSPECAN_VAL_DOCSIS_CONSTELLATION_OBJECT_QAM1024                    9
#define RSSPECAN_VAL_DOCSIS_CONSTELLATION_OBJECT_QAM2048                    10
#define RSSPECAN_VAL_DOCSIS_CONSTELLATION_OBJECT_QAM4096                    11
#define RSSPECAN_VAL_DOCSIS_CONSTELLATION_OBJECT_QAM8192                    12
#define RSSPECAN_VAL_DOCSIS_CONSTELLATION_OBJECT_QAM16384                   13
#define RSSPECAN_VAL_DOCSIS_CONSTELLATION_OBJECT_PLCD                       14
#define RSSPECAN_VAL_DOCSIS_CONSTELLATION_OBJECT_PLCP                       15
#define RSSPECAN_VAL_DOCSIS_CONSTELLATION_OBJECT_NCPA                       16
#define RSSPECAN_VAL_DOCSIS_CONSTELLATION_OBJECT_A                          17
#define RSSPECAN_VAL_DOCSIS_CONSTELLATION_OBJECT_B                          18
#define RSSPECAN_VAL_DOCSIS_CONSTELLATION_OBJECT_C                          19
#define RSSPECAN_VAL_DOCSIS_CONSTELLATION_OBJECT_D                          20
#define RSSPECAN_VAL_DOCSIS_CONSTELLATION_OBJECT_E                          21
#define RSSPECAN_VAL_DOCSIS_CONSTELLATION_OBJECT_F                          22
#define RSSPECAN_VAL_DOCSIS_CONSTELLATION_OBJECT_G                          23
#define RSSPECAN_VAL_DOCSIS_CONSTELLATION_OBJECT_H                          24
#define RSSPECAN_VAL_DOCSIS_CONSTELLATION_OBJECT_I                          25
#define RSSPECAN_VAL_DOCSIS_CONSTELLATION_OBJECT_J                          26
#define RSSPECAN_VAL_DOCSIS_CONSTELLATION_OBJECT_K                          27
#define RSSPECAN_VAL_DOCSIS_CONSTELLATION_OBJECT_L                          28
#define RSSPECAN_VAL_DOCSIS_CONSTELLATION_OBJECT_M                          29
#define RSSPECAN_VAL_DOCSIS_CONSTELLATION_OBJECT_N                          30
#define RSSPECAN_VAL_DOCSIS_CONSTELLATION_OBJECT_O                          31
#define RSSPECAN_VAL_DOCSIS_CONSTELLATION_OBJECT_P                          32
#define RSSPECAN_VAL_DOCSIS_CONSTELLATION_OBJECT_PILOTS                     33
#define RSSPECAN_VAL_DOCSIS_CONSTELLATION_OBJECT_CPILOTS                    34
#define RSSPECAN_VAL_DOCSIS_CONSTELLATION_OBJECT_SPILOTS                    35
#define RSSPECAN_VAL_DOCSIS_CONSTELLATION_OBJECT_CONPILOTS                  36
#define RSSPECAN_VAL_DOCSIS_CONSTELLATION_OBJECT_PROFILE                    37

// RsSpecAn_rngDOCSISBitsreamFormat
#define RSSPECAN_VAL_DOCSIS_BITSTREAM_FORMAT_BIT                            0
#define RSSPECAN_VAL_DOCSIS_BITSTREAM_FORMAT_BYTE                           1

// rsspecan_rngDOCSISTableItem
#define RSSPECAN_VAL_DOCSIS_ITEM_MER                                        0
#define RSSPECAN_VAL_DOCSIS_ITEM_MERD                                       1
#define RSSPECAN_VAL_DOCSIS_ITEM_MERP                                       2
#define RSSPECAN_VAL_DOCSIS_ITEM_FERR                                       3
#define RSSPECAN_VAL_DOCSIS_ITEM_CERR                                       4
#define RSSPECAN_VAL_DOCSIS_ITEM_TPLC                                       5
#define RSSPECAN_VAL_DOCSIS_ITEM_POW                                        6
#define RSSPECAN_VAL_DOCSIS_ITEM_ZBIT                                       7
#define RSSPECAN_VAL_DOCSIS_ITEM_TFR                                        8
#define RSSPECAN_VAL_DOCSIS_ITEM_POW_SPLC                                   9
#define RSSPECAN_VAL_DOCSIS_ITEM_POW_AMIN                                   10
#define RSSPECAN_VAL_DOCSIS_ITEM_POW_DATA                                   11
#define RSSPECAN_VAL_DOCSIS_ITEM_POW_PILOTS                                 12
#define RSSPECAN_VAL_DOCSIS_ITEM_POW_SPIL                                   13
#define RSSPECAN_VAL_DOCSIS_ITEM_POW_CONP                                   14

// rsspecan_rngDOCSISBitstreamLayout
#define RSSPECAN_VAL_DOCSIS_BITSTREAM_LAYOUT_COMP                           0
#define RSSPECAN_VAL_DOCSIS_BITSTREAM_LAYOUT_EXP                            1

// RsSpecAn_rngInstrumentMode
#define RSSPECAN_VAL_INSTRUMENT_SAN                                         0
#define RSSPECAN_VAL_INSTRUMENT_BDO                                         1
#define RSSPECAN_VAL_INSTRUMENT_MDO                                         2
#define RSSPECAN_VAL_INSTRUMENT_BWCD                                        3
#define RSSPECAN_VAL_INSTRUMENT_MWCD                                        4
#define RSSPECAN_VAL_INSTRUMENT_AMPL                                        5
#define RSSPECAN_VAL_INSTRUMENT_ADEM                                        6
#define RSSPECAN_VAL_INSTRUMENT_BC2K                                        7
#define RSSPECAN_VAL_INSTRUMENT_MC2K                                        8
#define RSSPECAN_VAL_INSTRUMENT_MCGD                                        9
#define RSSPECAN_VAL_INSTRUMENT_GSM                                         10
#define RSSPECAN_VAL_INSTRUMENT_IQ                                          11
#define RSSPECAN_VAL_INSTRUMENT_LTE                                         12
#define RSSPECAN_VAL_INSTRUMENT_NOISE                                       13
#define RSSPECAN_VAL_INSTRUMENT_PNOISE                                      14
#define RSSPECAN_VAL_INSTRUMENT_PULSE                                       15
#define RSSPECAN_VAL_INSTRUMENT_RTIM                                        16
#define RSSPECAN_VAL_INSTRUMENT_BTDS                                        17
#define RSSPECAN_VAL_INSTRUMENT_MTDS                                        18
#define RSSPECAN_VAL_INSTRUMENT_TA                                          19
#define RSSPECAN_VAL_INSTRUMENT_DDEM                                        20
#define RSSPECAN_VAL_INSTRUMENT_WLAN                                        21
#define RSSPECAN_VAL_INSTRUMENT_DOCS                                        22
#define RSSPECAN_VAL_INSTRUMENT_SFM                                         23
#define RSSPECAN_VAL_INSTRUMENT_BTO                                         24
#define RSSPECAN_VAL_INSTRUMENT_WIMAX                                       25
#define RSSPECAN_VAL_INSTRUMENT_SPUR                                        26
#define RSSPECAN_VAL_INSTRUMENT_VSA                                         27
#define RSSPECAN_VAL_INSTRUMENT_NR5G                                        28
#define RSSPECAN_VAL_INSTRUMENT_ONE_WEB                                     29
#define RSSPECAN_VAL_INSTRUMENT_V5GTF                                       30

// RsSpecAn_rngMinMax
#define RSSPECAN_VAL_MIN                                                    0
#define RSSPECAN_VAL_MAX                                                    1

// RsSpecAn_rngPhaseNoiseMeasurement
#define RSSPECAN_VAL_PNO_MEAS_PNO                                           0
#define RSSPECAN_VAL_PNO_MEAS_PPN                                           1
#define RSSPECAN_VAL_PNO_MEAS_RESIDUAL                                      2
#define RSSPECAN_VAL_PNO_MEAS_PRESIDUAL                                     3
#define RSSPECAN_VAL_PNO_MEAS_BASEBAND                                      4

// RsSpecAn_rngPhaseSpurSuppressionMode
#define RSSPECAN_VAL_PNOISE_SPUR_SUPP_NONE                                  0
#define RSSPECAN_VAL_PNOISE_SPUR_SUPP_UNSEL                                 1
#define RSSPECAN_VAL_PNOISE_SPUR_SUPP_SEL                                   2
#define RSSPECAN_VAL_PNOISE_SPUR_SUPP_ALL                                   3

// RsSpecAn_rngDCPowerSupplyPortOutputMode
#define RSSPECAN_VAL_OUTPUT_DC_SUPPLY_CURRENT                               0
#define RSSPECAN_VAL_OUTPUT_DC_SUPPLY_VOLTAGE                               1

// RsSpecAn_rngPresetFilter
#define RSSPECAN_VAL_PRESET_FILT_NORMAL                                     0
#define RSSPECAN_VAL_PRESET_FILT_PULSE                                      1
#define RSSPECAN_VAL_PRESET_FILT_NOISE                                      2

// RsSpecAn_rngPhaseIntegrationTrace
#define RSSPECAN_VAL_INT_TRACE_NONE                                         0
#define RSSPECAN_VAL_INT_TRACE_1                                            1
#define RSSPECAN_VAL_INT_TRACE_2                                            2
#define RSSPECAN_VAL_INT_TRACE_3                                            3
#define RSSPECAN_VAL_INT_TRACE_4                                            4
#define RSSPECAN_VAL_INT_TRACE_5                                            5
#define RSSPECAN_VAL_INT_TRACE_6                                            6

// RsSpecAn_rngTraceDisplayedResult
#define RSSPECAN_VAL_TRACE_DISP_PN                                          0
#define RSSPECAN_VAL_TRACE_DISP_AM                                          1
#define RSSPECAN_VAL_TRACE_DISP_PNAM                                        2

// rsspecan_rngPhaseNoiseLayoutType
#define RSSPECAN_VAL_LAYOUT_TYPE_ADEV                                       140
#define RSSPECAN_VAL_LAYOUT_TYPE_AVAR                                       141
#define RSSPECAN_VAL_LAYOUT_TYPE_LF                                         142
#define RSSPECAN_VAL_LAYOUT_TYPE_SNTP                                       143
#define RSSPECAN_VAL_LAYOUT_TYPE_SNU                                        144
#define RSSPECAN_VAL_LAYOUT_TYPE_SPHI                                       145
#define RSSPECAN_VAL_LAYOUT_TYPE_SY                                         146
#define RSSPECAN_VAL_LAYOUT_TYPE_PWR_FREQ                                   147

// RsSpecAn_rngPulseRIQWindowType
#define RSSPECAN_VAL_PULS_RIQ_WINDOW_BLACKMAN                               0
#define RSSPECAN_VAL_PULS_RIQ_WINDOW_HANNING                                1
#define RSSPECAN_VAL_PULS_RIQ_WINDOW_RECTANGLE                              2
#define RSSPECAN_VAL_PULS_RIQ_WINDOW_HAMMING                                3
#define RSSPECAN_VAL_PULS_RIQ_WINDOW_GAUSS                                  4
#define RSSPECAN_VAL_PULS_RIQ_WINDOW_CHEBYSHEV                              5
#define RSSPECAN_VAL_PULS_RIQ_WINDOW_FLATTOP                                6

// RsSpecAn_rngWiGigBitstreamFormat
#define RSSPECAN_VAL_WIGIG_BSTR_OCT                                         0
#define RSSPECAN_VAL_WIGIG_BSTR_HEX                                         1

// rsspecan_rngWIGIGLayoutWindowType
#define RSSPECAN_VAL_LAYOUT_TYPE_MARKER_PEAK_LIST                           91
#define RSSPECAN_VAL_LAYOUT_TYPE_PSP_WIGIG                                  122

// RsSpecAn_rngDOCSISMERExcludingSubcarriersMode
#define RSSPECAN_VAL_DOCSIS_EXCLUDING_SUBCARRIERS_MODE_OFF                  0
#define RSSPECAN_VAL_DOCSIS_EXCLUDING_SUBCARRIERS_MODE_AUTO                 1
#define RSSPECAN_VAL_DOCSIS_EXCLUDING_SUBCARRIERS_MODE_USER                 2

// RsSpecAn_rngDOCSISStreamDirection
#define RSSPECAN_DOCSIS_STREAM_DIRECTION_DS                                 0
#define RSSPECAN_DOCSIS_STREAM_DIRECTION_US                                 1

// RsSpecAn_rngDOCSISUpstreamModulation
#define RSSPECAN_VAL_DOCSIS_UPSTREAM_MODULATION_QAM8                        3
#define RSSPECAN_VAL_DOCSIS_UPSTREAM_MODULATION_QAM16                       4
#define RSSPECAN_VAL_DOCSIS_UPSTREAM_MODULATION_QAM64                       5
#define RSSPECAN_VAL_DOCSIS_UPSTREAM_MODULATION_QAM128                      6
#define RSSPECAN_VAL_DOCSIS_UPSTREAM_MODULATION_QAM256                      7
#define RSSPECAN_VAL_DOCSIS_UPSTREAM_MODULATION_QAM512                      8
#define RSSPECAN_VAL_DOCSIS_UPSTREAM_MODULATION_QAM1024                     9
#define RSSPECAN_VAL_DOCSIS_UPSTREAM_MODULATION_QAM2048                     10
#define RSSPECAN_VAL_DOCSIS_UPSTREAM_MODULATION_QAM4096                     11
#define RSSPECAN_VAL_DOCSIS_UPSTREAM_MODULATION_ZERO                        0
#define RSSPECAN_VAL_DOCSIS_UPSTREAM_MODULATION_BPSK                        1
#define RSSPECAN_VAL_DOCSIS_UPSTREAM_MODULATION_QPSK                        2
#define RSSPECAN_VAL_DOCSIS_UPSTREAM_MODULATION_QAM32                       12
#define RSSPECAN_VAL_DOCSIS_UPSTREAM_MODULATION_UNUSED                      13

// RsSpecAn_rngConfigureWIGIGResultTable
#define RSSPECAN_VAL_WIGIG_ITEM_EVM_ALL                                     0
#define RSSPECAN_VAL_WIGIG_ITEM_EVM_DATA                                    1
#define RSSPECAN_VAL_WIGIG_ITEM_EVM_PILOT                                   2
#define RSSPECAN_VAL_WIGIG_ITEM_IQ_OFFSET                                   3
#define RSSPECAN_VAL_WIGIG_ITEM_GAIN_IMBALANCE                              4
#define RSSPECAN_VAL_WIGIG_ITEM_QUADRATURE_ERROR                            5
#define RSSPECAN_VAL_WIGIG_ITEM_FREQUENCY_ERROR                             6
#define RSSPECAN_VAL_WIGIG_ITEM_SYMBOL_ERROR                                7
#define RSSPECAN_VAL_WIGIG_ITEM_RISE_TIME                                   8
#define RSSPECAN_VAL_WIGIG_ITEM_FALL_TIME                                   9
#define RSSPECAN_VAL_WIGIG_ITEM_TIME_SKEW                                   10
#define RSSPECAN_VAL_WIGIG_ITEM_TIME_POWER                                  11
#define RSSPECAN_VAL_WIGIG_ITEM_CREST_FACTOR                                12
#define RSSPECAN_VAL_WIGIG_ITEM_HEADER_BER                                  13
#define RSSPECAN_VAL_WIGIG_ITEM_PAYLOAD_BER                                 14

// RsSpecAn_rngAvionicsMeasurementType
#define RSSPECAN_VAL_MEASUREMENT_TYPE_VOR                                   0
#define RSSPECAN_VAL_MEASUREMENT_TYPE_ILS                                   1

// RsSpecAn_rngAvionicsQueryAMDepthResults
#define RSSPECAN_VAL_AVIONICS_FREQUENCY_30                                  0
#define RSSPECAN_VAL_AVIONICS_FREQUENCY_30FM                                1
#define RSSPECAN_VAL_AVIONICS_FREQUENCY_90                                  2
#define RSSPECAN_VAL_AVIONICS_FREQUENCY_150                                 3
#define RSSPECAN_VAL_AVIONICS_FREQUENCY_90_150                              4
#define RSSPECAN_VAL_AVIONICS_FREQUENCY_9960                                5
#define RSSPECAN_VAL_AVIONICS_FREQUENCY_ID                                  6

// RsSpecAn_rngAvionicsILSDDMUnit
#define RSSPECAN_VAL_DDM_UNIT_UNITLESS                                      0
#define RSSPECAN_VAL_DDM_UNIT_PCT                                           1

// RsSpecAn_rngAvionicsVORDirection
#define RSSPECAN_VAL_VOR_FROM                                               0
#define RSSPECAN_VAL_VOR_TO                                                 1

// RsSpecAn_rngLTEUplinkLocalOscillatorLocation
#define RSSPECAN_VAL_LTE_UPLINK_CACB                                        0
#define RSSPECAN_VAL_LTE_UPLINK_CCC                                         1
#define RSSPECAN_VAL_LTE_UPLINK_USER                                        2

// RsSpecAn_rngLTEUplinkDRSCellID
#define RSSPECAN_VAL_LTE_UPLINK_FROM_CELL_ID                                0

// RsSpecAn_rngLTEUplinkPUCCHCellID
#define RSSPECAN_VAL_LTE_UPLINK_PUCCH_FROM_CELL_ID                          0

// RsSpecAn_rngWLANSpectrumFlatnessUnit
#define RSSPECAN_VAL_WLAN_DB                                                0
#define RSSPECAN_VAL_WLAN_DBM                                               1

// RsSpecAn_rngTransientTraceEvaluation
#define RSSPECAN_VAL_TRANSIENT_ITIME                                        0
#define RSSPECAN_VAL_TRANSIENT_QTIME                                        1

// RsSpecAn_rngMCGDCarrierReferenceType
#define RSSPECAN_VAL_MCGD_AVERAGE                                           0
#define RSSPECAN_VAL_MCGD_CENTER                                            1
#define RSSPECAN_VAL_MCGD_MANUAL                                            2

// RsSpecAn_rngAmplifierSynchronizationIQData
#define RSSPECAN_VAL_AMPLIFIER_BB                                           0
#define RSSPECAN_VAL_AMPLIFIER_RF                                           1

// RsSpecAn_rngNoiseResultSummaryDisplay
#define RSSPECAN_VAL_NOISE_ITEM_CALIBRATION_POWER_COLD                      0
#define RSSPECAN_VAL_NOISE_ITEM_CALIBRATION_POWER_HOT                       1
#define RSSPECAN_VAL_NOISE_ITEM_CALIBRATION_Y_FACTOR                        2
#define RSSPECAN_VAL_NOISE_ITEM_GAIN                                        3
#define RSSPECAN_VAL_NOISE_ITEM_NOISE                                       4
#define RSSPECAN_VAL_NOISE_ITEM_NUNCERTAINTY                                5
#define RSSPECAN_VAL_NOISE_ITEM_POWER_COLD                                  6
#define RSSPECAN_VAL_NOISE_ITEM_POWER_HOT                                   7
#define RSSPECAN_VAL_NOISE_ITEM_TEMPERATURE                                 8
#define RSSPECAN_VAL_NOISE_ITEM_Y_FACTOR                                    9
#define RSSPECAN_VAL_NOISE_ITEM_ENR                                         10

// RsSpecAn_rngDOCSISNextCodewordPointerModulation
#define RSSPECAN_VAL_DOCSIS_NCP_MODULATION_ZEROBIT                          0
#define RSSPECAN_VAL_DOCSIS_NCP_MODULATION_QAM16                            2
#define RSSPECAN_VAL_DOCSIS_NCP_MODULATION_QAM64                            3

// RsSpecAn_rngDOCSISEvaluationRangeSymbolSize
#define RSSPECAN_VAL_DOCSIS_SYMBOL_SIZE_S1                                  1
#define RSSPECAN_VAL_DOCSIS_SYMBOL_SIZE_S2                                  2
#define RSSPECAN_VAL_DOCSIS_SYMBOL_SIZE_S3                                  3

// RsSpecAn_rngDOCSISResultSummaryUnit
#define RSSPECAN_VAL_DOCSIS_RESULT_ABSOLUTE                                 0
#define RSSPECAN_VAL_DOCSIS_RESULT_RELATIVE                                 1

// RsSpecAn_rngPhaseYAxisUnit
#define RSSPECAN_VAL_PHASE_UNIT_DBM                                         0
#define RSSPECAN_VAL_PHASE_UNIT_DBV                                         1
#define RSSPECAN_VAL_PHASE_UNIT_DBUV                                        2
#define RSSPECAN_VAL_PHASE_UNIT_HZ                                          3
#define RSSPECAN_VAL_PHASE_UNIT_DEG                                         4
#define RSSPECAN_VAL_PHASE_UNIT_RAD                                         5
#define RSSPECAN_VAL_PHASE_UNIT_V_SQRT_HZ                                   6

// RsSpecAn_rngPhasePulseExternalGateType
#define RSSPECAN_VAL_PHASE_EGAT_TRIG_LEV                                    0
#define RSSPECAN_VAL_PHASE_EGAT_TRIG_EDGE                                   1
#define RSSPECAN_VAL_PHASE_EGAT_TRIG_OFF                                    2

// RsSpecAn_rngPhaseSignalCaptureRange
#define RSSPECAN_VAL_PHASE_RANGE_NORMAL                                     0
#define RSSPECAN_VAL_PHASE_RANGE_WIDE                                       1
#define RSSPECAN_VAL_PHASE_RANGE_40_MHZ                                     2

// RsSpecAn_rngServiceInternalWidebandCalibrationFrequency
#define RSSPECAN_VAL_CALIBRATION_WB2_16MHZ                                  0
#define RSSPECAN_VAL_CALIBRATION_WB2_32MHZ                                  1

// RsSpecAn_rngSystemTreeWalking
#define RSSPECAN_VAL_TREE_NORMAL                                            0
#define RSSPECAN_VAL_TREE_ENHANCED                                          1

// RsSpecAn_rngHcopyPageMarginUnit
#define RSSPECAN_VAL_MARGIN_MM                                              0
#define RSSPECAN_VAL_MARGIN_IN                                              1

// RsSpecAn_rngHcopyContent
#define RSSPECAN_VAL_CONTENT_TYPE_WINDOWS                                   0
#define RSSPECAN_VAL_CONTENT_TYPE_HCOPY                                     1

// RsSpecAn_rngSourcePulseTriggerOutput
#define RSSPECAN_VAL_OUTPUT_TRIGGER_OFF                                     0
#define RSSPECAN_VAL_OUTPUT_TRIGGER_HIGH                                    1
#define RSSPECAN_VAL_OUTPUT_TRIGGER_LOW                                     2

// rsspecan_rngFileExportType
#define RSSPECAN_TRAC_FILE_TYPE_CSV                                         1
#define RSSPECAN_TRAC_FILE_TYPE_DAT                                         0

// rsspecan_rngTraceImportAll
#define RSSPECAN_TRAC_IMPORT_MODE_SINGLE                                    0
#define RSSPECAN_TRAC_IMPORT_MODE_ALL                                       1

// rsspecan_rngInputImpedancePad
#define RSSPECAN_VAL_IMP_PAD_SRES                                           0
#define RSSPECAN_VAL_IMP_PAD_MLP                                            1

// RsSpecAn_rngSpuriousDirectedDetector
#define RSSPECAN_VAL_SPUR_DIRECT_DETECTOR_POS                               0
#define RSSPECAN_VAL_SPUR_DIRECT_DETECTOR_RMS                               1
#define RSSPECAN_VAL_SPUR_DIRECT_DETECTOR_AVER                              2

// RsSpecAn_rngSETypeOfSearch
#define RSSPECAN_SE_STYPE_WIDE                                              0
#define RSSPECAN_SE_STYPE_DIR                                               1

// RsSpecAn_rngSEPerformedSteps
#define RSSPECAN_SE_STEPS_SOV                                               0
#define RSSPECAN_SE_STEPS_NEST                                              1
#define RSSPECAN_SE_STEPS_SDET                                              2
#define RSSPECAN_SE_STEPS_SPOT                                              3

// RsSpecAn_rngAmplifierAMPMDefinition
#define RSSPECAN_VAL_AMPLIFIER_AMPM_DEF_MREF                                0
#define RSSPECAN_VAL_AMPLIFIER_AMPM_DEF_REFM                                1

// RsSpecAn_rngAmplifierPowerReference
#define RSSPECAN_VAL_AMPLIFIER_POWER_REF_INPUT                              0
#define RSSPECAN_VAL_AMPLIFIER_POWER_REF_OUTPUT                             1

// RsSpecAn_rngAmplifierDPDMethod
#define RSSPECAN_VAL_AMPLIFIER_DPD_METHOD_GENERATOR                         0
#define RSSPECAN_VAL_AMPLIFIER_DPD_METHOD_WFILE                             1

// RsSpecAn_rngVSATraceSymbolError
#define RSSPECAN_VAL_TRACE_MSTR                                             0
#define RSSPECAN_VAL_TRACE_PSTR                                             1

// RsSpecAn_rngSweepConnectorSource
#define RSSPECAN_VAL_CONN_VAUX                                              0
#define RSSPECAN_VAL_CONN_VSUP                                              1
#define RSSPECAN_VAL_CONN_VTUN                                              2

// rsspecan_rngFormat
#define RSSPECAN_VAL_FORMAT_FLOAT                                           0

// RsSpecAn_rngPhaseNoiseBasebandInputConnector
#define RSSPECAN_VAL_PNOISE_INPUT_XCOR                                      0
#define RSSPECAN_VAL_PNOISE_INPUT_CH1                                       1
#define RSSPECAN_VAL_PNOISE_INPUT_CH2                                       2

// RsSpecAn_rngDOCSISUpstreamAutoDetection
#define RSSPECAN_VAL_DOCSIS_UPSTREAM_AUTO_DETECTION_FROM_SIGNAL             0
#define RSSPECAN_VAL_DOCSIS_UPSTREAM_AUTO_DETECTION_USER_DEFINED            1

// RsSpecAn_rngDOCSISAutomaticBandConfiguration
#define RSSPECAN_VAL_DOCSIS_AUTO_BAND_CONFIG_STANDARD                       0
#define RSSPECAN_VAL_DOCSIS_AUTO_BAND_CONFIG_USER                           1

// RsSpecAn_rngDOCSISAutoBandAppliedTo
#define RSSPECAN_VAL_DOCSIS_AUTO_BANDS_EDGE                                 0
#define RSSPECAN_VAL_DOCSIS_AUTO_BANDS_ALL                                  1

// RsSpecAn_rngDOCSISUserConfigLocationReference
#define RSSPECAN_VAL_DOCSIS_LOCATION_LEFT                                   0
#define RSSPECAN_VAL_DOCSIS_LOCATION_RIGHT                                  1

// RsSpecAn_rngAmplifierDPDLedState
#define RSSPECAN_VAL_DPD_LED_STATE_RED                                      0
#define RSSPECAN_VAL_DPD_LED_STATE_GREEN                                    1
#define RSSPECAN_VAL_DPD_LED_STATE_UNKNOWN                                  2

// RsSpecAn_rngLTEDownlinkBeamformingAntennaPortCellReferenceSignal
#define RSSPECAN_VAL_BEAMFORMING_ANTENNA_PORT_0                             0
#define RSSPECAN_VAL_BEAMFORMING_ANTENNA_PORT_1                             1
#define RSSPECAN_VAL_BEAMFORMING_ANTENNA_PORT_2                             2
#define RSSPECAN_VAL_BEAMFORMING_ANTENNA_PORT_3                             3

// RsSpecAn_rngLTEDownlinkBeamformingAntennaPortCSIReferenceSignal
#define RSSPECAN_VAL_BEAMFORMING_ANTENNA_PORT_15                            15
#define RSSPECAN_VAL_BEAMFORMING_ANTENNA_PORT_16                            16
#define RSSPECAN_VAL_BEAMFORMING_ANTENNA_PORT_17                            17
#define RSSPECAN_VAL_BEAMFORMING_ANTENNA_PORT_18                            18
#define RSSPECAN_VAL_BEAMFORMING_ANTENNA_PORT_19                            19
#define RSSPECAN_VAL_BEAMFORMING_ANTENNA_PORT_20                            20
#define RSSPECAN_VAL_BEAMFORMING_ANTENNA_PORT_21                            21
#define RSSPECAN_VAL_BEAMFORMING_ANTENNA_PORT_22                            22

// RsSpecAn_rngPulseParameterEnvelopeModel
#define RSSPECAN_VAL_PULSE_RESULT_ENVELOPE_MODEL_RISE_BASE_POINT_TIME       1
#define RSSPECAN_VAL_PULSE_RESULT_ENVELOPE_MODEL_RISE_LOW_POINT_TIME        2
#define RSSPECAN_VAL_PULSE_RESULT_ENVELOPE_MODEL_RISE_MID_POINT_TIME        3
#define RSSPECAN_VAL_PULSE_RESULT_ENVELOPE_MODEL_RISE_HIGH_POINT_TIME       4
#define RSSPECAN_VAL_PULSE_RESULT_ENVELOPE_MODEL_RISE_TOP_POINT_TIME        5
#define RSSPECAN_VAL_PULSE_RESULT_ENVELOPE_MODEL_RISE_LOW_POINT_LEVEL       6
#define RSSPECAN_VAL_PULSE_RESULT_ENVELOPE_MODEL_RISE_MID_POINT_LEVEL       7
#define RSSPECAN_VAL_PULSE_RESULT_ENVELOPE_MODEL_RISE_HIGH_POINT_LEVEL      8
#define RSSPECAN_VAL_PULSE_RESULT_ENVELOPE_MODEL_RISE_TOP_POINT_LEVEL       9
#define RSSPECAN_VAL_PULSE_RESULT_ENVELOPE_MODEL_FALL_BASE_POINT_TIME       10
#define RSSPECAN_VAL_PULSE_RESULT_ENVELOPE_MODEL_FALL_LOW_POINT_TIME        11
#define RSSPECAN_VAL_PULSE_RESULT_ENVELOPE_MODEL_FALL_MID_POINT_TIME        12
#define RSSPECAN_VAL_PULSE_RESULT_ENVELOPE_MODEL_FALL_HIGH_POINT_TIME       13
#define RSSPECAN_VAL_PULSE_RESULT_ENVELOPE_MODEL_FALL_TOP_POINT_TIME        14
#define RSSPECAN_VAL_PULSE_RESULT_ENVELOPE_MODEL_FALL_LOW_POINT_LEVEL       15
#define RSSPECAN_VAL_PULSE_RESULT_ENVELOPE_MODEL_FALL_MID_POINT_LEVEL       16
#define RSSPECAN_VAL_PULSE_RESULT_ENVELOPE_MODEL_FALL_HIGH_POINT_LEVEL      17
#define RSSPECAN_VAL_PULSE_RESULT_ENVELOPE_MODEL_FALL_TOP_POINT_LEVEL       18

// RsSpecAn_rngPulseTraceEvaluation
#define RSSPECAN_VAL_PULSE_I                                                0
#define RSSPECAN_VAL_PULSE_Q                                                1

// RsSpecAn_rngPulseResultParameterTrendDisplayStyle
#define RSSPECAN_VAL_TREND_DISPLAY_STYLE_AUTO                               0
#define RSSPECAN_VAL_TREND_DISPLAY_STYLE_DOTS                               1
#define RSSPECAN_VAL_TREND_DISPLAY_STYLE_LINES                              2
#define RSSPECAN_VAL_TREND_DISPLAY_STYLE_BOTH                               3

// rsspecan_rngWLANPPDUFormat802_11ax
#define RSSPECAN_VAL_WLAN_PPDU_FORMAT_AIHS                                  0
#define RSSPECAN_VAL_WLAN_PPDU_FORMAT_AIES                                  1
#define RSSPECAN_VAL_WLAN_PPDU_FORMAT_AIHT                                  2
#define RSSPECAN_VAL_WLAN_PPDU_FORMAT_DHEP                                  3
#define RSSPECAN_VAL_WLAN_PPDU_FORMAT_AIHM                                  4

// RsSpecAn_rngWLANHEPPDUFormat
#define RSSPECAN_VAL_HE_PPDU_FORMAT_SU                                      0
#define RSSPECAN_VAL_HE_PPDU_FORMAT_MU                                      1
#define RSSPECAN_VAL_HE_PPDU_FORMAT_TRIG                                    2
#define RSSPECAN_VAL_HE_PPDU_FORMAT_ESU                                     3

// RsSpecAn_rngWLANNHELTF
#define RSSPECAN_VAL_N_HE_LTF_STA                                           1
#define RSSPECAN_VAL_N_HE_LTF_S1                                            2
#define RSSPECAN_VAL_N_HE_LTF_S2                                            3
#define RSSPECAN_VAL_N_HE_LTF_S4                                            4
#define RSSPECAN_VAL_N_HE_LTF_S6                                            5
#define RSSPECAN_VAL_N_HE_LTF_S8                                            6

// rsspecan_rngWLANChannelBandwidth802_11ax
#define RSSPECAN_VAL_WLAN_BANDWIDTH_802_11AX_FBUR                           0
#define RSSPECAN_VAL_WLAN_BANDWIDTH_802_11AX_ALL                            1
#define RSSPECAN_VAL_WLAN_BANDWIDTH_802_11AX_MB20                           2
#define RSSPECAN_VAL_WLAN_BANDWIDTH_802_11AX_MB40                           3
#define RSSPECAN_VAL_WLAN_BANDWIDTH_802_11AX_MB80                           4
#define RSSPECAN_VAL_WLAN_BANDWIDTH_802_11AX_MB160                          5
#define RSSPECAN_VAL_WLAN_BANDWIDTH_802_11AX_DB20                           6
#define RSSPECAN_VAL_WLAN_BANDWIDTH_802_11AX_DB40                           7
#define RSSPECAN_VAL_WLAN_BANDWIDTH_802_11AX_DB80                           8
#define RSSPECAN_VAL_WLAN_BANDWIDTH_802_11AX_DB160                          9

// RsSpecAn_rngWLANHEPPDUConfigRUSize
#define RSSPECAN_VAL_HE_PPDU_CONFIG_RU_SIZE_SU                              0
#define RSSPECAN_VAL_HE_PPDU_CONFIG_RU_SIZE_S0                              1
#define RSSPECAN_VAL_HE_PPDU_CONFIG_RU_SIZE_S26                             2
#define RSSPECAN_VAL_HE_PPDU_CONFIG_RU_SIZE_S52                             3
#define RSSPECAN_VAL_HE_PPDU_CONFIG_RU_SIZE_S106                            4
#define RSSPECAN_VAL_HE_PPDU_CONFIG_RU_SIZE_S242                            5
#define RSSPECAN_VAL_HE_PPDU_CONFIG_RU_SIZE_S484                            6
#define RSSPECAN_VAL_HE_PPDU_CONFIG_RU_SIZE_S996                            7
#define RSSPECAN_VAL_HE_PPDU_CONFIG_RU_SIZE_S2X996                          8
#define RSSPECAN_VAL_HE_PPDU_CONFIG_RU_SIZE_S4X996                          9
#define RSSPECAN_VAL_HE_PPDU_CONFIG_RU_SIZE_S52S26                          10
#define RSSPECAN_VAL_HE_PPDU_CONFIG_RU_SIZE_S106S26                         11
#define RSSPECAN_VAL_HE_PPDU_CONFIG_RU_SIZE_S484S242                        12
#define RSSPECAN_VAL_HE_PPDU_CONFIG_RU_SIZE_S996S484                        13
#define RSSPECAN_VAL_HE_PPDU_CONFIG_RU_SIZE_S996S484S242                    14
#define RSSPECAN_VAL_HE_PPDU_CONFIG_RU_SIZE_S2X996S484                      15
#define RSSPECAN_VAL_HE_PPDU_CONFIG_RU_SIZE_S3X996                          16
#define RSSPECAN_VAL_HE_PPDU_CONFIG_RU_SIZE_S3X996S484                      17

// RsSpecAn_rngTransientAnalysisMode
#define RSSPECAN_VAL_TRANSIENT_MODE_NARROW                                  0
#define RSSPECAN_VAL_TRANSIENT_MODE_WIDE                                    1

// RsSpecAn_rngPhaseNoiseLocalOscillatorExternalLevel
#define RSSPECAN_VAL_LOCAL_OSCILLATOR_EXTERNAL_LEVEL_LOW                    0
#define RSSPECAN_VAL_LOCAL_OSCILLATOR_EXTERNAL_LEVEL_HIGH                   1

// RsSpecAn_rngDOCSISUpstreamProfileConfigurationFirstMinislot
#define RSSPECAN_VAL_FIRST_MINISLOT_EDGE                                    0
#define RSSPECAN_VAL_FIRST_MINISLOT_BODY                                    1

// RsSpecAn_rngWLANAmplitudeSettingsCoupling
#define RSSPECAN_VAL_WLAN_AMPLITUDE_COUPLING_OFF                            0
#define RSSPECAN_VAL_WLAN_AMPLITUDE_COUPLING_AUTO                           1
#define RSSPECAN_VAL_WLAN_AMPLITUDE_COUPLING_MASTER                         2

// RsSpecAn_rngWLANReferenceLevelOffset
#define RSSPECAN_VAL_WLAN_REFERENCE_LEVEL_OFFSET_PRIMARY                    0
#define RSSPECAN_VAL_WLAN_REFERENCE_LEVEL_OFFSET_MANUAL                     1

// RsSpecAn_rngWLANPeambleChannelEstimation
#define RSSPECAN_VAL_PREAMBLE_CHANNEL_ESTIMATION_PAYLOAD                    0
#define RSSPECAN_VAL_PREAMBLE_CHANNEL_ESTIMATION_VHT                        1

// rsspecan_rngNoiseResultType
#define RSSPECAN_VAL_NOISE_RESULT_NOISE                                     0
#define RSSPECAN_VAL_NOISE_RESULT_GAIN                                      1
#define RSSPECAN_VAL_NOISE_RESULT_TEMPERATURE                               2
#define RSSPECAN_VAL_NOISE_RESULT_YFACTOR                                   3
#define RSSPECAN_VAL_NOISE_RESULT_PHOT                                      4
#define RSSPECAN_VAL_NOISE_RESULT_PCOLD                                     5
#define RSSPECAN_VAL_NOISE_RESULT_CYF                                       6
#define RSSPECAN_VAL_NOISE_RESULT_CPH                                       7
#define RSSPECAN_VAL_NOISE_RESULT_CPC                                       8
#define RSSPECAN_VAL_NOISE_RESULT_NUNC                                      9
#define RSSPECAN_VAL_NOISE_RESULT_ENR                                       10

// RsSpecAn_rngUserCorrectionApplyTo
#define RSSPECAN_VAL_APPLY_TO_CHANNEL                                       0
#define RSSPECAN_VAL_APPLY_TO_ALL                                           1

// RsSpecAn_rngProbeAttenuationRatio
#define RSSPECAN_VAL_PROBE_ATTN_RATIO_10                                    0
#define RSSPECAN_VAL_PROBE_ATTN_RATIO_2                                     1

// RsSpecAn_rngProbeMode
#define RSSPECAN_VAL_PROBE_DM                                               0
#define RSSPECAN_VAL_PROBE_CM                                               1
#define RSSPECAN_VAL_PROBE_PM                                               2
#define RSSPECAN_VAL_PROBE_NM                                               3

// RsSpecAn_rngTuningRange
#define RSSPECAN_VAL_TUNING_SMALL                                           0
#define RSSPECAN_VAL_TUNING_WIDE                                            1

// RsSpecAn_rngNPRResultType
#define RSSPECAN_VAL_NPR_CPOWER                                             0
#define RSSPECAN_VAL_NPR_NPOWER                                             1
#define RSSPECAN_VAL_NPR_NPRATIO                                            2

// RsSpecAn_rngDisplayTimeFormat
#define RSSPECAN_VAL_TIME_FORMAT_DE                                         0
#define RSSPECAN_VAL_TIME_FORMAT_US                                         1
#define RSSPECAN_VAL_TIME_FORMAT_ISO                                        2

// RsSpecAn_rngInputConnectorType
#define RSSPECAN_VAL_INPUT_AIQI                                             1
#define RSSPECAN_VAL_INPUT_RFPROBE                                          2

// RsSpecAn_rngParameterSynchronizationDirection
#define RSSPECAN_VAL_SYNC_BIDIR                                             0
#define RSSPECAN_VAL_SYNC_LTOR                                              1
#define RSSPECAN_VAL_SYNC_RTOL                                              2

// RsSpecAn_rngCMTSReferenceSampleClockUnit
#define RSSPECAN_VAL_SAMPLE_CLOCK_UNIT_HZ                                   0
#define RSSPECAN_VAL_SAMPLE_CLOCK_UNIT_PPM                                  1

// RsSpecAn_rngRelay
#define RSSPECAN_VAL_RELAY_ACDC                                             0
#define RSSPECAN_VAL_RELAY_ATT_5                                            1
#define RSSPECAN_VAL_RELAY_ATT_10                                           2
#define RSSPECAN_VAL_RELAY_ATT_20                                           3
#define RSSPECAN_VAL_RELAY_ATT_40                                           4
#define RSSPECAN_VAL_RELAY_CAL                                              5
#define RSSPECAN_VAL_RELAY_EATT                                             6
#define RSSPECAN_VAL_RELAY_PREAMP                                           7
#define RSSPECAN_VAL_RELAY_PRESELECTOR                                      8
#define RSSPECAN_VAL_RELAY_RFAB                                             9
#define RSSPECAN_VAL_RELAY_PREAMP_30MHZ                                     10
#define RSSPECAN_VAL_RELAY_ATT_INPUT2                                       11
#define RSSPECAN_VAL_RELAY_INPUT2                                           12
#define RSSPECAN_VAL_RELAY_EXT_RELAIS                                       13

// rsspecan_rngTransientDistributionXAxis
#define RSSPECAN_VAL_TA_DISTRIBUTION_X_AXIS_AVER                            0
#define RSSPECAN_VAL_TA_DISTRIBUTION_X_AXIS_FMER                            1
#define RSSPECAN_VAL_TA_DISTRIBUTION_X_AXIS_MAXD                            2
#define RSSPECAN_VAL_TA_DISTRIBUTION_X_AXIS_RMSD                            3
#define RSSPECAN_VAL_TA_DISTRIBUTION_X_AXIS_AVGD                            4
#define RSSPECAN_VAL_TA_DISTRIBUTION_X_AXIS_RELF                            5
#define RSSPECAN_VAL_TA_DISTRIBUTION_X_AXIS_FMSL                            6
#define RSSPECAN_VAL_TA_DISTRIBUTION_X_AXIS_FMSP                            7
#define RSSPECAN_VAL_TA_DISTRIBUTION_X_AXIS_FMST                            8
#define RSSPECAN_VAL_TA_DISTRIBUTION_X_AXIS_PMSL                            9
#define RSSPECAN_VAL_TA_DISTRIBUTION_X_AXIS_PMSP                            10
#define RSSPECAN_VAL_TA_DISTRIBUTION_X_AXIS_PMST                            11
#define RSSPECAN_VAL_TA_DISTRIBUTION_X_AXIS_AVGP                            12
#define RSSPECAN_VAL_TA_DISTRIBUTION_X_AXIS_MAXP                            13
#define RSSPECAN_VAL_TA_DISTRIBUTION_X_AXIS_MINP                            14
#define RSSPECAN_VAL_TA_DISTRIBUTION_X_AXIS_PWRR                            15
#define RSSPECAN_VAL_TA_DISTRIBUTION_X_AXIS_AVPH                            16
#define RSSPECAN_VAL_TA_DISTRIBUTION_X_AXIS_MXPH                            17
#define RSSPECAN_VAL_TA_DISTRIBUTION_X_AXIS_RMSP                            18
#define RSSPECAN_VAL_TA_DISTRIBUTION_X_AXIS_BEG                             19
#define RSSPECAN_VAL_TA_DISTRIBUTION_X_AXIS_DWELL                           20
#define RSSPECAN_VAL_TA_DISTRIBUTION_X_AXIS_SWIT                            21
#define RSSPECAN_VAL_TA_DISTRIBUTION_X_AXIS_STAF                            22
#define RSSPECAN_VAL_TA_DISTRIBUTION_X_AXIS_INDEX                           23

// rsspecan_rngTransientParameterTrendAxis
#define RSSPECAN_VAL_TA_TREND_AXIS_AVER                                     0
#define RSSPECAN_VAL_TA_TREND_AXIS_CHER                                     1
#define RSSPECAN_VAL_TA_TREND_AXIS_MAXD                                     2
#define RSSPECAN_VAL_TA_TREND_AXIS_RMSD                                     3
#define RSSPECAN_VAL_TA_TREND_AXIS_AVGD                                     4
#define RSSPECAN_VAL_TA_TREND_AXIS_AVGN                                     5
#define RSSPECAN_VAL_TA_TREND_AXIS_MAXN                                     6
#define RSSPECAN_VAL_TA_TREND_AXIS_RMSN                                     7
#define RSSPECAN_VAL_TA_TREND_AXIS_BWID                                     8
#define RSSPECAN_VAL_TA_TREND_AXIS_FMSL                                     9
#define RSSPECAN_VAL_TA_TREND_AXIS_FMSP                                     10
#define RSSPECAN_VAL_TA_TREND_AXIS_FMST                                     11
#define RSSPECAN_VAL_TA_TREND_AXIS_PMSL                                     12
#define RSSPECAN_VAL_TA_TREND_AXIS_PMSP                                     13
#define RSSPECAN_VAL_TA_TREND_AXIS_PMST                                     14
#define RSSPECAN_VAL_TA_TREND_AXIS_FMER                                     15
#define RSSPECAN_VAL_TA_TREND_AXIS_RELF                                     16
#define RSSPECAN_VAL_TA_TREND_AXIS_AVGP                                     17
#define RSSPECAN_VAL_TA_TREND_AXIS_MAXP                                     18
#define RSSPECAN_VAL_TA_TREND_AXIS_MINP                                     19
#define RSSPECAN_VAL_TA_TREND_AXIS_PWRR                                     20
#define RSSPECAN_VAL_TA_TREND_AXIS_AVPH                                     21
#define RSSPECAN_VAL_TA_TREND_AXIS_MXPH                                     22
#define RSSPECAN_VAL_TA_TREND_AXIS_RMSP                                     23
#define RSSPECAN_VAL_TA_TREND_AXIS_INDEX                                    24
#define RSSPECAN_VAL_TA_TREND_AXIS_STAF                                     25
#define RSSPECAN_VAL_TA_TREND_AXIS_BEGIN                                    26
#define RSSPECAN_VAL_TA_TREND_AXIS_LENGTH                                   27
#define RSSPECAN_VAL_TA_TREND_AXIS_RATE                                     28
#define RSSPECAN_VAL_TA_TREND_AXIS_DWELL                                    29
#define RSSPECAN_VAL_TA_TREND_AXIS_SWITCH                                   30

// RsSpecAn_rngWLANSEMChannelBandwidth
#define RSSPECAN_VAL_WLAN_CBW_BW2_5                                         0
#define RSSPECAN_VAL_WLAN_CBW_BW5                                           1
#define RSSPECAN_VAL_WLAN_CBW_BW10                                          2
#define RSSPECAN_VAL_WLAN_CBW_BW20                                          3
#define RSSPECAN_VAL_WLAN_CBW_BW40                                          4
#define RSSPECAN_VAL_WLAN_CBW_BW80                                          5
#define RSSPECAN_VAL_WLAN_CBW_BW160                                         6

// RsSpecAn_rngEVMStandard
#define RSSPECAN_VAL_STD802_11B_1999                                        0
#define RSSPECAN_VAL_STD802_11B_2012                                        1
#define RSSPECAN_VAL_STD802_11B_2016                                        2

// RsSpecAn_rngWLANGuardTime
#define RSSPECAN_VAL_WLAN_GUARD_TIME_SHORT                                  0
#define RSSPECAN_VAL_WLAN_GUARD_TIME_NORMAL                                 1

// RsSpecAn_rngWLANCaptureTimeSync
#define RSSPECAN_VAL_WLAN_CAPTURE_SYNC_APPLY                                0
#define RSSPECAN_VAL_WLAN_CAPTURE_SYNC_IGNORE                               1

// RsSpecAn_rngWLANAutoFixRange
#define RSSPECAN_VAL_WLAN_AUTO_FIX_NONE                                     0
#define RSSPECAN_VAL_WLAN_AUTO_FIX_LOWER                                    1
#define RSSPECAN_VAL_WLAN_AUTO_FIX_UPPER                                    2

// RsSpecAn_rngWLANAutoRescaleMode
#define RSSPECAN_VAL_WLAN_RESCALE_HYSTERESIS                                0
#define RSSPECAN_VAL_WLAN_RESCALE_MEMORY                                    1

// RsSpecAn_rngSCDetailedObject
#define RSSPECAN_VAL_WLAN_SCD_HELTF                                         0
#define RSSPECAN_VAL_WLAN_SCD_DPILOT                                        1
#define RSSPECAN_VAL_WLAN_SCD_LLTF                                          2

// RsSpecan_rngIQFormat
#define RSSPECAN_VAL_IQ_FORMAT_FLOAT32                                      0
#define RSSPECAN_VAL_IQ_FORMAT_INT32                                        1

// RsSpecan_rngIQDataFormat
#define RSSPECAN_VAL_IQ_DATA_COMPLEX                                        0
#define RSSPECAN_VAL_IQ_DATA_REAL                                           1

// RsSpecAn_rngWLANMIMOCaptureSignalPath
#define RSSPECAN_VAL_WLAN_CAPTURE_SIGNAL_PATH_RX1                           1
#define RSSPECAN_VAL_WLAN_CAPTURE_SIGNAL_PATH_RX2                           2
#define RSSPECAN_VAL_WLAN_CAPTURE_SIGNAL_PATH_RX3                           3
#define RSSPECAN_VAL_WLAN_CAPTURE_SIGNAL_PATH_RX4                           4
#define RSSPECAN_VAL_WLAN_CAPTURE_SIGNAL_PATH_RX5                           5
#define RSSPECAN_VAL_WLAN_CAPTURE_SIGNAL_PATH_RX6                           6
#define RSSPECAN_VAL_WLAN_CAPTURE_SIGNAL_PATH_RX7                           7
#define RSSPECAN_VAL_WLAN_CAPTURE_SIGNAL_PATH_RX8                           8

// RsSpecAn_rngWLANMIMOAntennaSignalCaptureOSPModule
#define RSSPECAN_VAL_OSP_MODULE_A11                                         1
#define RSSPECAN_VAL_OSP_MODULE_A12                                         2
#define RSSPECAN_VAL_OSP_MODULE_A13                                         3

// RsSpecAn_rngWLANBurstPPDUStatus
#define RSSPECAN_VAL_WLAN_PPDU_STATUS_OK                                    0
#define RSSPECAN_VAL_WLAN_PPDU_STATUS_WARNING                               1
#define RSSPECAN_VAL_WLAN_PPDU_STATUS_SELECTED                              2

// RsSpecAn_rngSECarrierSearchRange
#define RSSPECAN_VAL_SEARCH_RANGE_GMAXIMUM                                  0
#define RSSPECAN_VAL_SEARCH_RANGE_RMAXIMUM                                  1

// RsSpecAn_rngSESpuriousDetectionTableContentHeader
#define RSSPECAN_VAL_SE_HEADER_ALL                                          0
#define RSSPECAN_VAL_SE_HEADER_START                                        1
#define RSSPECAN_VAL_SE_HEADER_STOP                                         2
#define RSSPECAN_VAL_SE_HEADER_FREQ                                         3
#define RSSPECAN_VAL_SE_HEADER_POWER                                        4
#define RSSPECAN_VAL_SE_HEADER_DELTA                                        5
#define RSSPECAN_VAL_SE_HEADER_RBW                                          6
#define RSSPECAN_VAL_SE_HEADER_IDENT                                        7

// RsSpecAn_rngVSAFrameModulation
#define RSSPECAN_VAL_VSA_FRAME_MODULATION_DATA                              0
#define RSSPECAN_VAL_VSA_FRAME_MODULATION_PATTERN                           1
#define RSSPECAN_VAL_VSA_FRAME_MODULATION_AUTO                              2

// RsSpecAn_rngVSAFrameMode
#define RSSPECAN_VAL_VSA_FRAME_MODE_DEFAULT                                 0
#define RSSPECAN_VAL_VSA_FRAME_MODE_USER                                    1

// RsSpecAn_rngVSAIndividualResults
#define RSSPECAN_VAL_VSA_INDIVIDUAL_RESULTS_ALL                             0
#define RSSPECAN_VAL_VSA_INDIVIDUAL_RESULTS_EVMR                            1
#define RSSPECAN_VAL_VSA_INDIVIDUAL_RESULTS_EVMP                            2
#define RSSPECAN_VAL_VSA_INDIVIDUAL_RESULTS_PERM                            3
#define RSSPECAN_VAL_VSA_INDIVIDUAL_RESULTS_PEP                             4
#define RSSPECAN_VAL_VSA_INDIVIDUAL_RESULTS_MERM                            5
#define RSSPECAN_VAL_VSA_INDIVIDUAL_RESULTS_MEP                             6
#define RSSPECAN_VAL_VSA_INDIVIDUAL_RESULTS_CFER                            7
#define RSSPECAN_VAL_VSA_INDIVIDUAL_RESULTS_RHO                             8
#define RSSPECAN_VAL_VSA_INDIVIDUAL_RESULTS_IQOF                            9
#define RSSPECAN_VAL_VSA_INDIVIDUAL_RESULTS_FERM                            10
#define RSSPECAN_VAL_VSA_INDIVIDUAL_RESULTS_FEP                             11
#define RSSPECAN_VAL_VSA_INDIVIDUAL_RESULTS_FDER                            12

// RsSpecAn_rngVSAMaxAnalysisLimit
#define RSSPECAN_VAL_VSA_MAX_ANALYSIS_LIMIT_80MHZ                           0
#define RSSPECAN_VAL_VSA_MAX_ANALYSIS_LIMIT_160MHZ                          1
#define RSSPECAN_VAL_VSA_MAX_ANALYSIS_LIMIT_1200MHZ                         2
#define RSSPECAN_VAL_VSA_MAX_ANALYSIS_LIMIT_500MHZ                          3
#define RSSPECAN_VAL_VSA_MAX_ANALYSIS_LIMIT_320MHZ                          4
#define RSSPECAN_VAL_VSA_MAX_ANALYSIS_LIMIT_MAX                             5

// RsSpecAn_rngVSAKnownDataPRBSSource
#define RSSPECAN_VAL_VSA_PRBS_SOURCE_FILE                                   0
#define RSSPECAN_VAL_VSA_PRBS_SOURCE_PRBS                                   1

// RsSpecAn_rngVSAFrameFormat
#define RSSPECAN_VAL_VSA_FRAME_FORMAT_MSK                                   0
#define RSSPECAN_VAL_VSA_FRAME_FORMAT_PSK                                   1
#define RSSPECAN_VAL_VSA_FRAME_FORMAT_QAM                                   2
#define RSSPECAN_VAL_VSA_FRAME_FORMAT_QPSK                                  3
#define RSSPECAN_VAL_VSA_FRAME_FORMAT_FSK                                   4
#define RSSPECAN_VAL_VSA_FRAME_FORMAT_ASK                                   5
#define RSSPECAN_VAL_VSA_FRAME_FORMAT_APSK                                  6
#define RSSPECAN_VAL_VSA_FRAME_FORMAT_UQAM                                  7

// RsSpecAn_rngVSAFramePSKFormat
#define RSSPECAN_VAL_VSA_PSK_FORMAT_NORMAL                                  0
#define RSSPECAN_VAL_VSA_PSK_FORMAT_DIFFERENTIAL                            1

// RsSpecAn_rng5GNRMeasurement
#define RSSPECAN_VAL_5G_NR_MEASUREMENT_ACLR                                 0
#define RSSPECAN_VAL_5G_NR_MEASUREMENT_ESPECTRUM                            1
#define RSSPECAN_VAL_5G_NR_MEASUREMENT_EVM                                  2
#define RSSPECAN_VAL_5G_NR_MEASUREMENT_CACL                                 3
#define RSSPECAN_VAL_5G_NR_MEASUREMENT_MCAC                                 4
#define RSSPECAN_VAL_5G_NR_MEASUREMENT_MCES                                 5
#define RSSPECAN_VAL_5G_NR_MEASUREMENT_TAER                                 6
#define RSSPECAN_VAL_5G_NR_MEASUREMENT_TPOO                                 7
#define RSSPECAN_VAL_5G_NR_MEASUREMENT_PRAC                                 8
#define RSSPECAN_VAL_5G_NR_MEASUREMENT_CME                                  9

// RsSpecAn_rng5GNRDirection
#define RSSPECAN_VAL_5G_NR_DL                                               0
#define RSSPECAN_VAL_5G_NR_UL                                               1

// RsSpecAn_rng5GNRComponentCarrierMode
#define RSSPECAN_VAL_5G_NR_COMPONENT_CARRIER_MODE_AUTO                      0
#define RSSPECAN_VAL_5G_NR_COMPONENT_CARRIER_MODE_SINGLE                    1

// RsSpecAn_rng5GNROperatingBand
#define RSSPECAN_VAL_5G_NR_OB_N1                                            0
#define RSSPECAN_VAL_5G_NR_OB_N2                                            1
#define RSSPECAN_VAL_5G_NR_OB_N3                                            2
#define RSSPECAN_VAL_5G_NR_OB_N5                                            3
#define RSSPECAN_VAL_5G_NR_OB_N7                                            4
#define RSSPECAN_VAL_5G_NR_OB_N8                                            5
#define RSSPECAN_VAL_5G_NR_OB_N20                                           6
#define RSSPECAN_VAL_5G_NR_OB_N28                                           7
#define RSSPECAN_VAL_5G_NR_OB_N38                                           8
#define RSSPECAN_VAL_5G_NR_OB_N41                                           9
#define RSSPECAN_VAL_5G_NR_OB_N50                                           10
#define RSSPECAN_VAL_5G_NR_OB_N51                                           11
#define RSSPECAN_VAL_5G_NR_OB_N66                                           12
#define RSSPECAN_VAL_5G_NR_OB_N70                                           13
#define RSSPECAN_VAL_5G_NR_OB_N71                                           14
#define RSSPECAN_VAL_5G_NR_OB_N74                                           15
#define RSSPECAN_VAL_5G_NR_OB_N75                                           16
#define RSSPECAN_VAL_5G_NR_OB_N76                                           17
#define RSSPECAN_VAL_5G_NR_OB_N77                                           18
#define RSSPECAN_VAL_5G_NR_OB_N78                                           19
#define RSSPECAN_VAL_5G_NR_OB_N79                                           20
#define RSSPECAN_VAL_5G_NR_OB_N80                                           21
#define RSSPECAN_VAL_5G_NR_OB_N81                                           22
#define RSSPECAN_VAL_5G_NR_OB_N82                                           23
#define RSSPECAN_VAL_5G_NR_OB_N83                                           24
#define RSSPECAN_VAL_5G_NR_OB_N84                                           25
#define RSSPECAN_VAL_5G_NR_OB_N257                                          26
#define RSSPECAN_VAL_5G_NR_OB_N258                                          27
#define RSSPECAN_VAL_5G_NR_OB_N260                                          28
#define RSSPECAN_VAL_5G_NR_OB_N12                                           29
#define RSSPECAN_VAL_5G_NR_OB_N25                                           30
#define RSSPECAN_VAL_5G_NR_OB_N34                                           31
#define RSSPECAN_VAL_5G_NR_OB_N39                                           32
#define RSSPECAN_VAL_5G_NR_OB_N40                                           33
#define RSSPECAN_VAL_5G_NR_OB_N65                                           34
#define RSSPECAN_VAL_5G_NR_OB_N86                                           35
#define RSSPECAN_VAL_5G_NR_OB_N261                                          36
#define RSSPECAN_VAL_5G_NR_OB_N14                                           37
#define RSSPECAN_VAL_5G_NR_OB_N18                                           38
#define RSSPECAN_VAL_5G_NR_OB_N26                                           39
#define RSSPECAN_VAL_5G_NR_OB_N29                                           40
#define RSSPECAN_VAL_5G_NR_OB_N30                                           41
#define RSSPECAN_VAL_5G_NR_OB_N48                                           42
#define RSSPECAN_VAL_5G_NR_OB_N53                                           43
#define RSSPECAN_VAL_5G_NR_OB_N89                                           44
#define RSSPECAN_VAL_5G_NR_OB_N90                                           45
#define RSSPECAN_VAL_5G_NR_OB_N91                                           46
#define RSSPECAN_VAL_5G_NR_OB_N92                                           47
#define RSSPECAN_VAL_5G_NR_OB_N93                                           48
#define RSSPECAN_VAL_5G_NR_OB_N94                                           49
#define RSSPECAN_VAL_5G_NR_OB_N95                                           50

// RsSpecAn_rng5GNRBandwidth
#define RSSPECAN_VAL_5G_NR_BW5                                              0
#define RSSPECAN_VAL_5G_NR_BW10                                             1
#define RSSPECAN_VAL_5G_NR_BW15                                             2
#define RSSPECAN_VAL_5G_NR_BW20                                             3
#define RSSPECAN_VAL_5G_NR_BW25                                             4
#define RSSPECAN_VAL_5G_NR_BW30                                             5
#define RSSPECAN_VAL_5G_NR_BW40                                             6
#define RSSPECAN_VAL_5G_NR_BW50                                             7
#define RSSPECAN_VAL_5G_NR_BW60                                             8
#define RSSPECAN_VAL_5G_NR_BW70                                             9
#define RSSPECAN_VAL_5G_NR_BW80                                             10
#define RSSPECAN_VAL_5G_NR_BW90                                             11
#define RSSPECAN_VAL_5G_NR_BW100                                            12
#define RSSPECAN_VAL_5G_NR_BW200                                            13
#define RSSPECAN_VAL_5G_NR_BW400                                            14

// RsSpecAn_rng5GNRDeployment
#define RSSPECAN_VAL_5G_NR_DEPLOYMENT_LOW                                   0
#define RSSPECAN_VAL_5G_NR_DEPLOYMENT_MIDDLE                                1
#define RSSPECAN_VAL_5G_NR_DEPLOYMENT_HIGH                                  2

// RsSpecAn_rng5GNRSubcarrierSpacing
#define RSSPECAN_VAL_5G_NR_SUBCARRIER_SPACING_SS15                          0
#define RSSPECAN_VAL_5G_NR_SUBCARRIER_SPACING_SS30                          1
#define RSSPECAN_VAL_5G_NR_SUBCARRIER_SPACING_SS60                          2
#define RSSPECAN_VAL_5G_NR_SUBCARRIER_SPACING_SS120                         3
#define RSSPECAN_VAL_5G_NR_SUBCARRIER_SPACING_SS240                         4
#define RSSPECAN_VAL_5G_NR_SUBCARRIER_SPACING_SSX60                         5

// RsSpecAn_rng5GNRSlotAllocation
#define RSSPECAN_VAL_5G_NR_SLOT_ALLOCATION_DATA                             0
#define RSSPECAN_VAL_5G_NR_SLOT_ALLOCATION_UNUSED                           1

// RsSpecAn_rng5GNRDLEnhancedPDSCHMapping
#define RSSPECAN_VAL_5G_NR_ENHANCED_PDSCH_MAPPING_LC11                      0
#define RSSPECAN_VAL_5G_NR_ENHANCED_PDSCH_MAPPING_LC21                      1
#define RSSPECAN_VAL_5G_NR_ENHANCED_PDSCH_MAPPING_LC31                      2
#define RSSPECAN_VAL_5G_NR_ENHANCED_PDSCH_MAPPING_LC41                      3

// RsSpecAn_rng5GNRDMRSMappingType
#define RSSPECAN_VAL_5G_NR_DMRS_MAPPING_A                                   0
#define RSSPECAN_VAL_5G_NR_DMRS_MAPPING_B                                   1

// RsSpecAn_rng5GNRDMRSMethod
#define RSSPECAN_VAL_5G_NR_DMRS_METHOD_DSID                                 0
#define RSSPECAN_VAL_5G_NR_DMRS_METHOD_NIDCELL                              1

// RsSpecAn_rng5GNRPTRSRelativeOffset
#define RSSPECAN_VAL_5G_NR_PTRS_RELATIVE_OFFSET_OS00                        0
#define RSSPECAN_VAL_5G_NR_PTRS_RELATIVE_OFFSET_OS01                        1
#define RSSPECAN_VAL_5G_NR_PTRS_RELATIVE_OFFSET_OS10                        2
#define RSSPECAN_VAL_5G_NR_PTRS_RELATIVE_OFFSET_OS11                        3
#define RSSPECAN_VAL_5G_NR_PTRS_RELATIVE_OFFSET_NONE                        4

// RsSpecAn_rng5GNRPDSCHModulation
#define RSSPECAN_VAL_5G_NR_PDSCH_MODULATION_QPSK                            0
#define RSSPECAN_VAL_5G_NR_PDSCH_MODULATION_QAM16                           1
#define RSSPECAN_VAL_5G_NR_PDSCH_MODULATION_QAM64                           2
#define RSSPECAN_VAL_5G_NR_PDSCH_MODULATION_QAM256                          3
#define RSSPECAN_VAL_5G_NR_PDSCH_MODULATION_DMRS                            4

// RsSpecAn_rng5GNRBurstPeriodicity
#define RSSPECAN_VAL_5G_NR_BURST_PERIODICITY_AUTO                           0
#define RSSPECAN_VAL_5G_NR_BURST_PERIODICITY_MS05                           1
#define RSSPECAN_VAL_5G_NR_BURST_PERIODICITY_MS10                           2
#define RSSPECAN_VAL_5G_NR_BURST_PERIODICITY_MS20                           3
#define RSSPECAN_VAL_5G_NR_BURST_PERIODICITY_MS40                           4

// RsSpecAn_rng5GNRSyncSignConfigMode
#define RSSPECAN_VAL_5G_NR_SYNC_SIGNAL_MODE_AUTO                            0
#define RSSPECAN_VAL_5G_NR_SYNC_SIGNAL_MODE_MANUAL                          1

// RsSpecAn_rng5GNRSyncSignalPattern
#define RSSPECAN_VAL_5G_NR_SYNC_SIGNAL_PATTERN_A                            0
#define RSSPECAN_VAL_5G_NR_SYNC_SIGNAL_PATTERN_B                            1
#define RSSPECAN_VAL_5G_NR_SYNC_SIGNAL_PATTERN_C                            2
#define RSSPECAN_VAL_5G_NR_SYNC_SIGNAL_PATTERN_D                            3
#define RSSPECAN_VAL_5G_NR_SYNC_SIGNAL_PATTERN_E                            4

// RsSpecAn_rng5GNRRFUpconversionMode
#define RSSPECAN_VAL_5G_NR_RF_UPCONVERSION_MODE_CF                          0
#define RSSPECAN_VAL_5G_NR_RF_UPCONVERSION_MODE_MANUAL                      1

// RsSpecAn_rng5GNRYScalingAuto
#define RSSPECAN_VAL_5G_NR_Y_SCALING_AUTO_DEFAULT                           0
#define RSSPECAN_VAL_5G_NR_Y_SCALING_AUTO_ONCE                              1
#define RSSPECAN_VAL_5G_NR_Y_SCALING_AUTO_ALL                               2

// RsSpecAn_rng5GNRShowResult
#define RSSPECAN_VAL_5G_NR_SHOW_RESULT_SUMM_AVERAGE                         0
#define RSSPECAN_VAL_5G_NR_SHOW_RESULT_SUMM_SINGLE                          1

// RsSpecAn_rng5GNRXScaleUnit
#define RSSPECAN_VAL_5G_NR_X_SCALE_UNIT_SYMBOL                              0
#define RSSPECAN_VAL_5G_NR_X_SCALE_UNIT_TIME                                1

// RsSpecAn_rng5GNRACLRBW
#define RSSPECAN_VAL_5G_NR_ACLR_BW_E500                                     0
#define RSSPECAN_VAL_5G_NR_ACLR_BW_NOSBW                                    1

// RsSpecAn_rng5GNRBSCategory
#define RSSPECAN_VAL_5G_NR_BS_CATEGORY_A                                    0
#define RSSPECAN_VAL_5G_NR_BS_CATEGORY_B                                    1
#define RSSPECAN_VAL_5G_NR_BS_CATEGORY_LARE                                 2
#define RSSPECAN_VAL_5G_NR_BS_CATEGORY_MED                                  3

// RsSpecAn_rngPulseIQDetector
#define RSSPECAN_VAL_PULSE_IQ_DETECTOR_ITIM                                 0
#define RSSPECAN_VAL_PULSE_IQ_DETECTOR_QTIM                                 1

// RsSpecAn_rngPulseTraceStatistic
#define RSSPECAN_VAL_PULSE_TRACE_STATISTIC_SEL                              0
#define RSSPECAN_VAL_PULSE_TRACE_STATISTIC_ALL                              1

// RsSpecAn_rngNoiseResultsType
#define RSSPECAN_VAL_NOISE_RESULTS_TYPE_ARRAY                               0
#define RSSPECAN_VAL_NOISE_RESULTS_TYPE_SCALAR                              1

// RsSpecAn_rngNBIoTDeployment
#define RSSPECAN_VAL_NBIOT_DL_DEPLOYMENT_INBAND                             0
#define RSSPECAN_VAL_NBIOT_DL_DEPLOYMENT_STANDALONE                         1
#define RSSPECAN_VAL_NBIOT_DL_DEPLOYMENT_GUARDBAND                          2

// RsSpecAn_rngNBIoTDLMIMOAntennaSelection
#define RSSPECAN_VAL_NBIOT_DL_MIMO_ANTENNA_ANT1                             0
#define RSSPECAN_VAL_NBIOT_DL_MIMO_ANTENNA_ANT2                             1

// RsSpecAn_rngNBIoTDLMIMOConfig
#define RSSPECAN_VAL_NBIOT_DOWNLINK_MIMO_CONFIG_TX1                         0
#define RSSPECAN_VAL_NBIOT_DOWNLINK_MIMO_CONFIG_TX2                         1

// RsSpecAn_rngNBIoTDLFilterType
#define RSSPECAN_VAL_NBIOT_DOWNLINK_FILTER_TYPE_NORMAL                      0
#define RSSPECAN_VAL_NBIOT_DOWNLINK_FILTER_TYPE_WIDE                        1

// RsSpecAn_rngNBIoTDLEVMCalculation
#define RSSPECAN_VAL_NBIOT_DOWNLINK_EVM_CALC_3GPP                           0
#define RSSPECAN_VAL_NBIOT_DOWNLINK_EVM_CALC_OTP                            1

// RsSpecAn_rngNBIoTDLReferenceData
#define RSSPECAN_VAL_NBIOT_DOWNLINK_REF_DATA_AUTO                           0
#define RSSPECAN_VAL_NBIOT_DOWNLINK_REF_DATA_ALL0                           1

// RsSpecAn_rngNBIoTDLChannelEstimationType
#define RSSPECAN_VAL_NBIOT_DOWNLINK_CEST_PIL                                0
#define RSSPECAN_VAL_NBIOT_DOWNLINK_CEST_PILP                               1
#define RSSPECAN_VAL_NBIOT_DOWNLINK_CEST_TGPP                               2

// RsSpecAn_rngNBIoTDLPhaseTrackingType
#define RSSPECAN_VAL_NBIOT_DOWNLINK_PHASE_TRACKING_OFF                      0
#define RSSPECAN_VAL_NBIOT_DOWNLINK_PHASE_TRACKING_PIL                      1
#define RSSPECAN_VAL_NBIOT_DOWNLINK_PHASE_TRACKING_PILP                     2

// RsSpecAn_rngNBIoTDLResult
#define RSSPECAN_VAL_NBIOT_RESULT_STAT                                      0
#define RSSPECAN_VAL_NBIOT_RESULT_CBUF                                      1
#define RSSPECAN_VAL_NBIOT_RESULT_CCDF                                      2
#define RSSPECAN_VAL_NBIOT_RESULT_CONS                                      3
#define RSSPECAN_VAL_NBIOT_RESULT_EVM_CARR                                  4
#define RSSPECAN_VAL_NBIOT_RESULT_EVM_SUBF                                  5
#define RSSPECAN_VAL_NBIOT_RESULT_EVM_SYMB                                  6
#define RSSPECAN_VAL_NBIOT_RESULT_DFLAT                                     7
#define RSSPECAN_VAL_NBIOT_RESULT_GDEL                                      8
#define RSSPECAN_VAL_NBIOT_RESULT_FEVS                                      9
#define RSSPECAN_VAL_NBIOT_RESULT_PSPE                                      10
#define RSSPECAN_VAL_NBIOT_RESULT_SFLAT                                     11

// RsSpecAn_rngNBIoTDLTriggerSource
#define RSSPECAN_VAL_NBIOT_TRIGGER_SOURCE_EXT                               0
#define RSSPECAN_VAL_NBIOT_TRIGGER_SOURCE_IFP                               1
#define RSSPECAN_VAL_NBIOT_TRIGGER_SOURCE_IMM                               2
#define RSSPECAN_VAL_NBIOT_TRIGGER_SOURCE_PSEN                              3
#define RSSPECAN_VAL_NBIOT_TRIGGER_SOURCE_RFP                               4

// RsSpecAn_rngFileDecimalTraces
#define RSSPECAN_VAL_DECIMAL_TRACES_ALL                                     0
#define RSSPECAN_VAL_DECIMAL_TRACES_SINGLE                                  1

// RsSpecAn_rngSignalLevelMode
#define RSSPECAN_VAL_LOW                                                    0
#define RSSPECAN_VAL_HIGH                                                   1

// RsSpecAn_rngAdvRefOscillatorBandwidthMode
#define RSSPECAN_VAL_MANUAL                                                 1

// RsSpecAn_rngRefFreqCouplingState
#define RSSPECAN_VAL_REF_FREQ_COUPLING_OFF                                  0
#define RSSPECAN_VAL_REF_FREQ_COUPLING_ON                                   1
#define RSSPECAN_VAL_REF_FREQ_COUPLING_AUTO                                 2

// RsSpecAn_rngPulseGateSource
#define RSSPECAN_VAL_EXTERNAL                                               0
#define RSSPECAN_VAL_IFPOWER                                                1

// RsSpecAn_rngMessageType
#define RSSPECAN_VAL_MESSAGE_TYPE_ALL                                       0
#define RSSPECAN_VAL_MESSAGE_TYPE_INFO                                      1
#define RSSPECAN_VAL_MESSAGE_TYPE_WARNING                                   2
#define RSSPECAN_VAL_MESSAGE_TYPE_ERROR                                     3
#define RSSPECAN_VAL_MESSAGE_TYPE_FATAL                                     4
#define RSSPECAN_VAL_MESSAGE_TYPE_MESSAGE                                   5

// RsSpecAn_rngTraceXgind
#define RSSPECAN_VAL_RESULT_XGINDICATOR                                     6

// RsSpecAn_rngGeneratorConnectionStatus
#define RSSPECAN_VAL_CONNECTION_STATUS_UNKNOWN                              0
#define RSSPECAN_VAL_CONNECTION_STATUS_CONNECTED                            1
#define RSSPECAN_VAL_CONNECTION_STATUS_DISCONNECTED                         2

// RsSpecAn_rngAFTMOverallStatus
#define RSSPECAN_VAL_AFTM_OVERALL_STATUS_OFF                                0
#define RSSPECAN_VAL_AFTM_OVERALL_STATUS_SUCCESSFUL                         1
#define RSSPECAN_VAL_AFTM_OVERALL_STATUS_ERROR                              2

// RsSpecAn_rngMCGDConnectionStatus
#define RSSPECAN_VAL_MCGD_CONNECTION_STATUS_OFF                             0
#define RSSPECAN_VAL_MCGD_CONNECTION_STATUS_SUCCESSFUL                      1
#define RSSPECAN_VAL_MCGD_CONNECTION_STATUS_ERROR                           2

// RsSpecAn_rngGeneratorSettingsOverallStatus
#define RSSPECAN_VAL_OVERALL_STATUS_OFF                                     0
#define RSSPECAN_VAL_OVERALL_STATUS_SUCCESSFUL                              1
#define RSSPECAN_VAL_OVERALL_STATUS_ERROR                                   2

// RsSpecAn_rngFPLANComponentType
#define RSSPECAN_VAL_FPLAN_COMPONENT_TYPE_MIXER                             0
#define RSSPECAN_VAL_FPLAN_COMPONENT_TYPE_AMPLIFIER                         1
#define RSSPECAN_VAL_FPLAN_COMPONENT_TYPE_MULTIPLIER                        2
#define RSSPECAN_VAL_FPLAN_COMPONENT_TYPE_DIVIDER                           3

// RsSpecAn_rngFPLANIdentifier
#define RSSPECAN_VAL_FPLAN_IDENTIFIER_LO                                    0
#define RSSPECAN_VAL_FPLAN_IDENTIFIER_CLOCK                                 1

// RsSpecAn_rngAmplifierMaximumPowerLEDState
#define RSSPECAN_GENERATOR_MAXIMUM_POWER_LED_GREEN                          0
#define RSSPECAN_GENERATOR_MAXIMUM_POWER_LED_GREY                           1
#define RSSPECAN_GENERATOR_MAXIMUM_POWER_LED_RED                            2

// RsSpecAn_rngPulsePulseWindowType
#define RSSPECAN_VAL_PULSEPULSE_WINDOW_TYPE_RECTANGLE                       0
#define RSSPECAN_VAL_PULSEPULSE_WINDOW_TYPE_BARTLETT                        1
#define RSSPECAN_VAL_PULSEPULSE_WINDOW_TYPE_HAMMING                         2
#define RSSPECAN_VAL_PULSEPULSE_WINDOW_TYPE_HANNING                         3
#define RSSPECAN_VAL_PULSEPULSE_WINDOW_TYPE_BLACKMAN                        4

// RsSpecAn_rngQueryPulseCountRange
#define RSSPECAN_VAL_PULSE_COUNT_RANGE_CURRENT                              0
#define RSSPECAN_VAL_PULSE_COUNT_RANGE_ALL                                  1

// RsSpecAn_rngPulseParameterStability
#define RSSPECAN_VAL_PULSE_STABILITY_AMPLITUDE                              0
#define RSSPECAN_VAL_PULSE_STABILITY_BURST                                  1
#define RSSPECAN_VAL_PULSE_STABILITY_PHASE                                  2
#define RSSPECAN_VAL_PULSE_STABILITY_PIBURST                                3
#define RSSPECAN_VAL_PULSE_STABILITY_TOTAL                                  4

// RsSpecAn_rngVSAEqualizerFormat
#define RSSPECAN_VAL_VSA_EQUALIZER_FILE_FORMAT_VAE                          0
#define RSSPECAN_VAL_VSA_EQUALIZER_FILE_FORMAT_FRES                         1

// RsSpecAn_rngWLANInterpolationType
#define RSSPECAN_VAL_WLAN_INTERPOLATION_WIENER                              0
#define RSSPECAN_VAL_WLAN_INTERPOLATION_LINEAR                              1

// RsSpecAn_rngPassFailStatus
#define RSSPECAN_VAL_FAIL                                                   0
#define RSSPECAN_VAL_PASS                                                   1

// RsSpecAn_rngEDMGUserChipRate
#define RSSPECAN_VAL_USER_DEFINED_CHIP_RATE_STANDARD                        0
#define RSSPECAN_VAL_USER_DEFINED_CHIP_RATE_CUSTOM                          1

// RsSpecAn_rngNBIoTDownlinkSEMCategory
#define RSSPECAN_VAL_NB_IOT_SEM_CATEGORY_HOME                               0
#define RSSPECAN_VAL_NB_IOT_SEM_CATEGORY_LARE                               1
#define RSSPECAN_VAL_NB_IOT_SEM_CATEGORY_MED                                2
#define RSSPECAN_VAL_NB_IOT_SEM_CATEGORY_WARE                               3

// RsSpecAn_rngNBIoTDownlinkSEMNBIoTCarrierPowerMode
#define RSSPECAN_VAL_NB_IOT_SEM_POWER_CARRIER_MODE_AUTO                     0
#define RSSPECAN_VAL_NB_IOT_SEM_POWER_CARRIER_MODE_MAN                      1

// rsspecan_rngLTEUplinkDetectedPrefix
#define RSSPECAN_VAL_LTE_UPLINK_DET_CPREFIX_NORMAL                          0
#define RSSPECAN_VAL_LTE_UPLINK_DET_CPREFIX_EXTENDED                        1

// RsSpecAn_rngOWULSelectedMeasurement
#define RSSPECAN_VAL_OWUL_MEAS_ACLR                                         0
#define RSSPECAN_VAL_OWUL_MEAS_EVM                                          1

// RsSpecAn_rngOWULLOLocation
#define RSSPECAN_VAL_OW_LO_CACB                                             0
#define RSSPECAN_VAL_OW_LO_CCC                                              1
#define RSSPECAN_VAL_OW_LO_USER                                             2

// RsSpecAn_rngOWULAutoDemodulation
#define RSSPECAN_VAL_OW_UL_AUTO_DEMOD_ALL                                   0
#define RSSPECAN_VAL_OW_UL_AUTO_DEMOD_OFF                                   1
#define RSSPECAN_VAL_OW_UL_AUTO_DEMOD_SCON                                  2

// RsSpecAn_rngOWBandwidth
#define RSSPECAN_VAL_OW_BW_20_00                                            0

// RsSpecAn_rngOWULDetectedCyclicPrefix
#define RSSPECAN_VAL_OW_UL_CYCLIC_NAN                                       0
#define RSSPECAN_VAL_OW_UL_CYCLIC_NORM                                      1
#define RSSPECAN_VAL_OW_UL_CYCLIC_EXT                                       2

// RsSpecAn_rngOWULAllocationContent
#define RSSPECAN_VAL_OW_UL_ALLOCATION_CONTENT_NONE                          0
#define RSSPECAN_VAL_OW_UL_ALLOCATION_CONTENT_PUSCH                         1
#define RSSPECAN_VAL_OW_UL_ALLOCATION_CONTENT_PSCC                          2
#define RSSPECAN_VAL_OW_UL_ALLOCATION_CONTENT_PUACH                         3

// RsSpecAn_rngOWULSubframeModulation
#define RSSPECAN_VAL_OW_UL_MODULATION_QPSK                                  0
#define RSSPECAN_VAL_OW_UL_MODULATION_PSK8                                  1
#define RSSPECAN_VAL_OW_UL_MODULATION_QAM16                                 2

// RsSpecAn_rngOWULPUCCHFormat
#define RSSPECAN_VAL_OW_UL_PUCCH_FORMAT_F1                                  0
#define RSSPECAN_VAL_OW_UL_PUCCH_FORMAT_F1A                                 1
#define RSSPECAN_VAL_OW_UL_PUCCH_FORMAT_F1B                                 2
#define RSSPECAN_VAL_OW_UL_PUCCH_FORMAT_F2                                  3
#define RSSPECAN_VAL_OW_UL_PUCCH_FORMAT_F2A                                 4
#define RSSPECAN_VAL_OW_UL_PUCCH_FORMAT_F2B                                 5
#define RSSPECAN_VAL_OW_UL_PUCCH_FORMAT_F3                                  6
#define RSSPECAN_VAL_OW_UL_PUCCH_FORMAT_SUBF                                7

// RsSpecAn_rngOWULDemodulationMode
#define RSSPECAN_VAL_OW_UL_DEMODULATION_MODE_PUSCH                          0
#define RSSPECAN_VAL_OW_UL_DEMODULATION_MODE_PRACH                          1

// RsSpecAn_rngOWULChannelEstimationRange
#define RSSPECAN_VAL_OW_UL_ESTIMATION_RANGE_PIL                             1
#define RSSPECAN_VAL_OW_UL_ESTIMATION_RANGE_PILP                            2

// RsSpecAn_rngOWULTrackingPhase
#define RSSPECAN_VAL_OW_UL_TRACKING_PHASE_OFF                               0
#define RSSPECAN_VAL_OW_UL_TRACKING_PHASE_PIL                               1
#define RSSPECAN_VAL_OW_UL_TRACKING_PHASE_PILP                              2

// RsSpecAn_rngOWULSlot
#define RSSPECAN_VAL_OW_UL_SLOT_S0                                          0
#define RSSPECAN_VAL_OW_UL_SLOT_S1                                          1
#define RSSPECAN_VAL_OW_UL_SLOT_ALL                                         2

// RsSpecAn_rngOWULLimitCheckResult
#define RSSPECAN_VAL_OW_UL_LIMIT_CHECK_RESULT_FAILED                        0
#define RSSPECAN_VAL_OW_UL_LIMIT_CHECK_RESULT_PASSED                        1
#define RSSPECAN_VAL_OW_UL_LIMIT_CHECK_RESULT_NOTEVALUATED                  2

// RsSpecAn_rngOWULUnits
#define RSSPECAN_VAL_OW_UL_UNITS_CARR                                       0
#define RSSPECAN_VAL_OW_UL_UNITS_HZ                                         1

// RsSpecAn_rngNPRGeneratorState
#define RSSPECAN_VAL_NPR_GEN_STATE_OFF                                      0
#define RSSPECAN_VAL_NPR_GEN_STATE_SUCC                                     1
#define RSSPECAN_VAL_NPR_GEN_STATE_ERR                                      2

// RsSpecAn_rngNPRGeneratorTargetPath
#define RSSPECAN_VAL_NPR_GEN_PATH_A                                         0
#define RSSPECAN_VAL_NPR_GEN_PATH_B                                         1

// RsSpecAn_rngMSRGapLowUpp
#define RSSPECAN_MSR_GAP_LOW                                                0
#define RSSPECAN_MSR_GAP_UPP                                                1

// RsSpecAn_rngMSRSBGaps
#define RSSPECAN_VAL_MSR_SBGAPS_AB                                          0
#define RSSPECAN_VAL_MSR_SBGAPS_BC                                          1
#define RSSPECAN_VAL_MSR_SBGAPS_CD                                          2
#define RSSPECAN_VAL_MSR_SBGAPS_DE                                          3
#define RSSPECAN_VAL_MSR_SBGAPS_EF                                          4
#define RSSPECAN_VAL_MSR_SBGAPS_FG                                          5
#define RSSPECAN_VAL_MSR_SBGAPS_GH                                          6

// RsSpecAn_rngMSRGapMode
#define RSSPECAN_MSR_GAP_MODE_AUTO                                          0
#define RSSPECAN_MSR_GAP_MODE_MANUAL                                        1

// RsSpecAn_rngInputType
#define RSSPECAN_VAL_INPUT_TYPE_INPUT1                                      0
#define RSSPECAN_VAL_INPUT_TYPE_INPUT2                                      1

// RsSpecAn_rngUserCorrBBFreqRespMove
#define RSSPECAN_VAL_RESPONSE_MOVE_UP                                       0
#define RSSPECAN_VAL_RESPONSE_MOVE_DOWN                                     1

// RsSpecAn_rngNPRGeneratorConnectionState
#define RSSPECAN_VAL_NPR_GEN_CONN_STATE_UNKN                                0
#define RSSPECAN_VAL_NPR_GEN_CONN_STATE_CONN                                1
#define RSSPECAN_VAL_NPR_GEN_CONN_STATE_NCON                                2

// RsSpecAn_rngIQOscCoupling
#define RSSPECAN_VAL_IQ_OSC_COUPLING_DC                                     0
#define RSSPECAN_VAL_IQ_OSC_COUPLING_DCL                                    1
#define RSSPECAN_VAL_IQ_OSC_COUPLING_AC                                     2

// RsSpecAn_rngTransientResultFMSettling
#define RSSPECAN_VAL_TRANSIENT_RESULT_SETTLING_ALL                          0
#define RSSPECAN_VAL_TRANSIENT_RESULT_SETTLING_LENGTH                       1
#define RSSPECAN_VAL_TRANSIENT_RESULT_SETTLING_POINT                        2
#define RSSPECAN_VAL_TRANSIENT_RESULT_SETTLING_TIME                         3

// RsSpecAn_rngTransientTrendChirpXAxis
#define RSSPECAN_VAL_TRANSIENT_TREND_CHIRP_X_AXIS_CHER                      0
#define RSSPECAN_VAL_TRANSIENT_TREND_CHIRP_X_AXIS_FREQ                      1
#define RSSPECAN_VAL_TRANSIENT_TREND_CHIRP_X_AXIS_MAXF                      2
#define RSSPECAN_VAL_TRANSIENT_TREND_CHIRP_X_AXIS_RMSF                      3
#define RSSPECAN_VAL_TRANSIENT_TREND_CHIRP_X_AXIS_AVGF                      4
#define RSSPECAN_VAL_TRANSIENT_TREND_CHIRP_X_AXIS_FMSP                      5
#define RSSPECAN_VAL_TRANSIENT_TREND_CHIRP_X_AXIS_FMST                      6
#define RSSPECAN_VAL_TRANSIENT_TREND_CHIRP_X_AXIS_FMSL                      7
#define RSSPECAN_VAL_TRANSIENT_TREND_CHIRP_X_AXIS_BWID                      8
#define RSSPECAN_VAL_TRANSIENT_TREND_CHIRP_X_AXIS_AVGN                      9
#define RSSPECAN_VAL_TRANSIENT_TREND_CHIRP_X_AXIS_RMSN                      10
#define RSSPECAN_VAL_TRANSIENT_TREND_CHIRP_X_AXIS_MAXN                      11
#define RSSPECAN_VAL_TRANSIENT_TREND_CHIRP_X_AXIS_PMSP                      12
#define RSSPECAN_VAL_TRANSIENT_TREND_CHIRP_X_AXIS_PMST                      13
#define RSSPECAN_VAL_TRANSIENT_TREND_CHIRP_X_AXIS_PMSL                      14
#define RSSPECAN_VAL_TRANSIENT_TREND_CHIRP_X_AXIS_BEG                       15
#define RSSPECAN_VAL_TRANSIENT_TREND_CHIRP_X_AXIS_LENG                      16
#define RSSPECAN_VAL_TRANSIENT_TREND_CHIRP_X_AXIS_RAT                       17
#define RSSPECAN_VAL_TRANSIENT_TREND_CHIRP_X_AXIS_AVGP                      18
#define RSSPECAN_VAL_TRANSIENT_TREND_CHIRP_X_AXIS_MINP                      19
#define RSSPECAN_VAL_TRANSIENT_TREND_CHIRP_X_AXIS_MAXP                      20
#define RSSPECAN_VAL_TRANSIENT_TREND_CHIRP_X_AXIS_PWRR                      21
#define RSSPECAN_VAL_TRANSIENT_TREND_CHIRP_X_AXIS_IND                       22

// RsSpecAn_rngTransientTrendHopXAxis
#define RSSPECAN_VAL_TRANSIENT_TREND_HOP_X_AXIS_FREQ                        0
#define RSSPECAN_VAL_TRANSIENT_TREND_HOP_X_AXIS_RELF                        1
#define RSSPECAN_VAL_TRANSIENT_TREND_HOP_X_AXIS_FMER                        2
#define RSSPECAN_VAL_TRANSIENT_TREND_HOP_X_AXIS_MAXF                        3
#define RSSPECAN_VAL_TRANSIENT_TREND_HOP_X_AXIS_RMSF                        4
#define RSSPECAN_VAL_TRANSIENT_TREND_HOP_X_AXIS_AVGF                        5
#define RSSPECAN_VAL_TRANSIENT_TREND_HOP_X_AXIS_FMSP                        6
#define RSSPECAN_VAL_TRANSIENT_TREND_HOP_X_AXIS_FMST                        7
#define RSSPECAN_VAL_TRANSIENT_TREND_HOP_X_AXIS_FMSL                        8
#define RSSPECAN_VAL_TRANSIENT_TREND_HOP_X_AXIS_AVPH                        9
#define RSSPECAN_VAL_TRANSIENT_TREND_HOP_X_AXIS_MXPH                        10
#define RSSPECAN_VAL_TRANSIENT_TREND_HOP_X_AXIS_RMSP                        11
#define RSSPECAN_VAL_TRANSIENT_TREND_HOP_X_AXIS_PMSP                        12
#define RSSPECAN_VAL_TRANSIENT_TREND_HOP_X_AXIS_PMST                        13
#define RSSPECAN_VAL_TRANSIENT_TREND_HOP_X_AXIS_PMSL                        14
#define RSSPECAN_VAL_TRANSIENT_TREND_HOP_X_AXIS_MINP                        15
#define RSSPECAN_VAL_TRANSIENT_TREND_HOP_X_AXIS_MAXP                        16
#define RSSPECAN_VAL_TRANSIENT_TREND_HOP_X_AXIS_AVGP                        17
#define RSSPECAN_VAL_TRANSIENT_TREND_HOP_X_AXIS_PWRR                        18
#define RSSPECAN_VAL_TRANSIENT_TREND_HOP_X_AXIS_IND                         19
#define RSSPECAN_VAL_TRANSIENT_TREND_HOP_X_AXIS_STAF                        20
#define RSSPECAN_VAL_TRANSIENT_TREND_HOP_X_AXIS_BEG                         21
#define RSSPECAN_VAL_TRANSIENT_TREND_HOP_X_AXIS_DWEL                        22
#define RSSPECAN_VAL_TRANSIENT_TREND_HOP_X_AXIS_SWIT                        23

// RsSpecAn_rng5GNRDecodedData
#define RSSPECAN_VAL_5G_NR_DECODED_DATA_ADESCRAMBLE                         0
#define RSSPECAN_VAL_5G_NR_DECODED_DATA_BDESCRAMBLE                         1
#define RSSPECAN_VAL_5G_NR_DECODED_DATA_DPDATA                              2

// RsSpecAn_rng5GNRULDMRSMethod
#define RSSPECAN_VAL_5G_BR_UL_DMRS_METHOD_NIDPUSCH                          0
#define RSSPECAN_VAL_5G_BR_UL_DMRS_METHOD_NIDCELL                           1

// RsSpecAn_rng5GNRMCSTableOrder
#define RSSPECAN_VAL_5G_NR_MCS_TABLE_ORDER_Q64                              0
#define RSSPECAN_VAL_5G_NR_MCS_TABLE_ORDER_Q64L                             1
#define RSSPECAN_VAL_5G_NR_MCS_TABLE_ORDER_Q256                             2

// RsSpecAn_rng5GNRPUCCHGroupHopp
#define RSSPECAN_VAL_5G_NR_HOPP_DISABLE                                     0
#define RSSPECAN_VAL_5G_NR_HOPP_ENABLE                                      1
#define RSSPECAN_VAL_5G_NR_HOPP_NEITHER                                     2

// RsSpecAn_rng5GNRHoppID
#define RSSPECAN_VAL_5G_NR_HOPPING_ID_NID                                   0
#define RSSPECAN_VAL_5G_NR_HOPPING_ID_NIDCELL                               1

// RsSpecAn_rng5GNRSeqGenMethod
#define RSSPECAN_VAL_5G_NR_SEQ_GEN_METHOD_DSID                              0
#define RSSPECAN_VAL_5G_NR_SEQ_GEN_METHOD_NIDCELL                           1

// RsSpecAn_rng5GNRHopping
#define RSSPECAN_VAL_5G_NR_HOPPING_GROUP                                    0
#define RSSPECAN_VAL_5G_NR_HOPPING_SEQUENCE                                 1
#define RSSPECAN_VAL_5G_NR_HOPPING_NONE                                     2

// RsSpecAn_rng5GNRSRSSlotPeriodicity
#define RSSPECAN_VAL_5G_NR_SRS_SLOT_PERIODICITY_SL1                         0
#define RSSPECAN_VAL_5G_NR_SRS_SLOT_PERIODICITY_SL2                         1
#define RSSPECAN_VAL_5G_NR_SRS_SLOT_PERIODICITY_SL4                         2
#define RSSPECAN_VAL_5G_NR_SRS_SLOT_PERIODICITY_SL5                         3
#define RSSPECAN_VAL_5G_NR_SRS_SLOT_PERIODICITY_SL8                         4
#define RSSPECAN_VAL_5G_NR_SRS_SLOT_PERIODICITY_SL10                        5

// RsSpecAn_rng5GNRPUSCHHopping
#define RSSPECAN_VAL_5G_NR_PUSCH_HOPPING_GROUP                              0
#define RSSPECAN_VAL_5G_NR_PUSCH_HOPPING_NONE                               1
#define RSSPECAN_VAL_5G_NR_PUSCH_HOPPING_SEQUENCE                           2

// RsSpecAn_rng5GNRAntennaPortSelect
#define RSSPECAN_VAL_5G_NR_ANT_PORT_OFF                                     0
#define RSSPECAN_VAL_5G_NR_ANT_PORT_ON                                      1
#define RSSPECAN_VAL_5G_NR_ANT_PORT_ALL                                     2
#define RSSPECAN_VAL_5G_NR_ANT_PORT_NONE                                    3

// RsSpecAn_rng5GNRULYScalingAuto
#define RSSPECAN_VAL_5G_NR_UL_Y_SCALING_AUTO_ALL                            0
#define RSSPECAN_VAL_5G_NR_UL_Y_SCALING_AUTO_DEFAULT                        1
#define RSSPECAN_VAL_5G_NR_UL_Y_SCALING_AUTO_ONCE                           2

// RsSpecAn_rng5GNRChannelEstMethod
#define RSSPECAN_VAL_5G_NR_CHAN_EST_METHOD_LINT                             0
#define RSSPECAN_VAL_5G_NR_CHAN_EST_METHOD_OFF                              1
#define RSSPECAN_VAL_5G_NR_CHAN_EST_METHOD_PILPAY                           2

// RsSpecAn_rng5GNRTrackingPhase
#define RSSPECAN_VAL_5G_NR_TRACKING_PHASE_OFF                               0
#define RSSPECAN_VAL_5G_NR_TRACKING_PHASE_PIL                               1
#define RSSPECAN_VAL_5G_NR_TRACKING_PHASE_PILPAY                            2

// RsSpecAn_rng5GNRPort
#define RSSPECAN_VAL_5G_NR_PORT1                                            0
#define RSSPECAN_VAL_5G_NR_PORT2                                            1
#define RSSPECAN_VAL_5G_NR_PORT3                                            2

// RsSpecAn_rng5GNRTriggerSource
#define RSSPECAN_VAL_5G_NR_TRIGGER_SOURCE_IMMEDIATE                         0
#define RSSPECAN_VAL_5G_NR_TRIGGER_SOURCE_EXTERNAL                          1
#define RSSPECAN_VAL_5G_NR_TRIGGER_SOURCE_EXT2                              2
#define RSSPECAN_VAL_5G_NR_TRIGGER_SOURCE_EXT3                              3
#define RSSPECAN_VAL_5G_NR_TRIGGER_SOURCE_RFPOWER                           4
#define RSSPECAN_VAL_5G_NR_TRIGGER_SOURCE_IFPOWER                           5
#define RSSPECAN_VAL_5G_NR_TRIGGER_SOURCE_IQPOWER                           6
#define RSSPECAN_VAL_5G_NR_TRIGGER_SOURCE_BBPOWER                           7
#define RSSPECAN_VAL_5G_NR_TRIGGER_SOURCE_PSEN                              8
#define RSSPECAN_VAL_5G_NR_TRIGGER_SOURCE_GP0                               9
#define RSSPECAN_VAL_5G_NR_TRIGGER_SOURCE_GP1                               10
#define RSSPECAN_VAL_5G_NR_TRIGGER_SOURCE_GP2                               11
#define RSSPECAN_VAL_5G_NR_TRIGGER_SOURCE_GP3                               12
#define RSSPECAN_VAL_5G_NR_TRIGGER_SOURCE_GP4                               13
#define RSSPECAN_VAL_5G_NR_TRIGGER_SOURCE_GP5                               14

// RsSpecAn_rng5GNRLevelUnit
#define RSSPECAN_VAL_5G_NR_LEVEL_UNIT_DBM                                   0
#define RSSPECAN_VAL_5G_NR_LEVEL_UNIT_DBPW                                  1
#define RSSPECAN_VAL_5G_NR_LEVEL_UNIT_WATT                                  2
#define RSSPECAN_VAL_5G_NR_LEVEL_UNIT_DBUV                                  3
#define RSSPECAN_VAL_5G_NR_LEVEL_UNIT_DBMV                                  4
#define RSSPECAN_VAL_5G_NR_LEVEL_UNIT_VOLT                                  5
#define RSSPECAN_VAL_5G_NR_LEVEL_UNIT_DBUA                                  6
#define RSSPECAN_VAL_5G_NR_LEVEL_UNIT_AMP                                   7

// RsSpecAn_rng5GNRIQInputType
#define RSSPECAN_VAL_5G_NR_INPUT_IQ_IQ                                      0
#define RSSPECAN_VAL_5G_NR_INPUT_IQ_I                                       1
#define RSSPECAN_VAL_5G_NR_INPUT_IQ_Q                                       2

// RsSpecAn_rng5GNRInputType
#define RSSPECAN_VAL_5G_NR_INPUT_1                                          1
#define RSSPECAN_VAL_5G_NR_INPUT_2                                          2

// RsSpecAn_rng5GNRSubcXAxis
#define RSSPECAN_VAL_5G_NR_SUBC_X_CARR                                      0
#define RSSPECAN_VAL_5G_NR_SUBC_X_HZ                                        1

// RsSpecAn_rng5GNRReferenceXAxis
#define RSSPECAN_VAL_5G_NR_REF_X_LRB                                        0
#define RSSPECAN_VAL_5G_NR_REF_X_RTCF                                       1

// RsSpecAn_rng5GNRMultipleCarrier
#define RSSPECAN_VAL_5G_NR_MULTIPLE_CARRIER_ALL                             0
#define RSSPECAN_VAL_5G_NR_MULTIPLE_CARRIER_VIEWED                          1

// RsSpecAn_rng5GNRPowerClass
#define RSSPECAN_VAL_5G_NR_POWER_CLASS_PC2                                  0
#define RSSPECAN_VAL_5G_NR_POWER_CLASS_PC3                                  1

// RsSpecAn_rng5GNREVMLimits
#define RSSPECAN_VAL_5G_UL_EVM_LIMITS_FAILED                                0
#define RSSPECAN_VAL_5G_UL_EVM_LIMITS_PASSED                                1
#define RSSPECAN_VAL_5G_UL_EVM_LIMITS_NOTEVALUATED                          2

// RsSpecAn_rngV5GTFLinkDirection
#define RSSPECAN_VAL_V5GTF_DIRECTION_DL                                     0
#define RSSPECAN_VAL_V5GTF_DIRECTION_UL                                     1

// RsSpecAn_rngV5GTFCaptureMode
#define RSSPECAN_VAL_V5GTF_CAPTURE_MODE_AUTO                                0
#define RSSPECAN_VAL_V5GTF_CAPTURE_MODE_SINGLE                              1

// RsSpecAn_rngV5GTFDLAllocations
#define RSSPECAN_VAL_V5GTF_DL_ALLOC_UNUSED                                  0
#define RSSPECAN_VAL_V5GTF_DL_ALLOC_DATA                                    1
#define RSSPECAN_VAL_V5GTF_DL_ALLOC_XRACH                                   2
#define RSSPECAN_VAL_V5GTF_DL_ALLOC_EPBCH                                   3
#define RSSPECAN_VAL_V5GTF_DL_ALLOC_SYNC                                    4

// RsSpecAn_rngV5GTFOptionalRefSignals
#define RSSPECAN_VAL_V5GTF_DL_REF_NONE                                      0
#define RSSPECAN_VAL_V5GTF_DL_REF_CSIRS                                     1

// RsSpecAn_rngV5GTFDLSubframeType
#define RSSPECAN_VAL_V5GTF_SUBFRAME_TYPE_A                                  0
#define RSSPECAN_VAL_V5GTF_SUBFRAME_TYPE_B                                  1

// RsSpecAn_rngV5GTFULAllocations
#define RSSPECAN_VAL_V5GTF_UL_ALLOC_UNUSED                                  0
#define RSSPECAN_VAL_V5GTF_UL_ALLOC_DATA                                    1
#define RSSPECAN_VAL_V5GTF_UL_ALLOC_XRACH                                   2

// RsSpecAn_rngV5GTFULSubframeType
#define RSSPECAN_VAL_V5GTF_SUBFRAME_TYPE_C                                  2
#define RSSPECAN_VAL_V5GTF_SUBFRAME_TYPE_D                                  3

// RsSpecAn_rngV5GTFSubframeModulation
#define RSSPECAN_VAL_V5GTF_MODULATION_QPSK                                  0
#define RSSPECAN_VAL_V5GTF_MODULATION_QAM16                                 1
#define RSSPECAN_VAL_V5GTF_MODULATION_QAM64                                 2
#define RSSPECAN_VAL_V5GTF_MODULATION_QAM256                                3

// RsSpecAn_rngV5GTFxPDSCHPrecoding
#define RSSPECAN_VAL_V5GTF_XPDSCH_PRECODING_NONE                            0
#define RSSPECAN_VAL_V5GTF_XPDSCH_PRECODING_SPM                             1
#define RSSPECAN_VAL_V5GTF_XPDSCH_PRECODING_TXD                             2

// RsSpecAn_rngV5GTFxPDSCHSubframeAntennaPort
#define RSSPECAN_VAL_V5GTF_XPDSCH_AP_8                                      0
#define RSSPECAN_VAL_V5GTF_XPDSCH_AP_9                                      1
#define RSSPECAN_VAL_V5GTF_XPDSCH_AP_10                                     2
#define RSSPECAN_VAL_V5GTF_XPDSCH_AP_11                                     3
#define RSSPECAN_VAL_V5GTF_XPDSCH_AP_8_9                                    4
#define RSSPECAN_VAL_V5GTF_XPDSCH_AP_10_11                                  5
#define RSSPECAN_VAL_V5GTF_XPDSCH_AP_8_12                                   6
#define RSSPECAN_VAL_V5GTF_XPDSCH_AP_9_13                                   7
#define RSSPECAN_VAL_V5GTF_XPDSCH_AP_10_14                                  8
#define RSSPECAN_VAL_V5GTF_XPDSCH_AP_11_15                                  9

// RsSpecAn_rngV5GTFSequenceGeneration
#define RSSPECAN_VAL_V5GTF_SEQ_NICELL                                       0
#define RSSPECAN_VAL_V5GTF_SEQ_NIDMRS                                       1

// RsSpecAn_rngV5GTFxPDSCHSubframePCRSAntennaPort
#define RSSPECAN_VAL_V5GTF_PCRS_AP_60                                       0
#define RSSPECAN_VAL_V5GTF_PCRS_AP_61                                       1
#define RSSPECAN_VAL_V5GTF_PCRS_AP_60_61                                    2

// RsSpecAn_rngV5GTFSubframePCRSTransmission
#define RSSPECAN_VAL_V5GTF_PCRS_TRANS_SINGLE                                0
#define RSSPECAN_VAL_V5GTF_PCRS_TRANS_DUAL                                  1

// RsSpecAn_rngV5GTFRefSignalPrecoding
#define RSSPECAN_VAL_V5GTF_REF_PRECODING_NONE                               0
#define RSSPECAN_VAL_V5GTF_REF_PRECODING_TXD                                1

// RsSpecAn_rngV5GTFExtendedEnabled
#define RSSPECAN_VAL_V5GTF_OFF                                              0
#define RSSPECAN_VAL_V5GTF_ON                                               1
#define RSSPECAN_VAL_V5GTF_NONE                                             2
#define RSSPECAN_VAL_V5GTF_ALL                                              3

// RsSpecAn_rngV5GTFControlChannelPrecoding
#define RSSPECAN_VAL_V5GTF_CCH_PRECODING_NONE                               0
#define RSSPECAN_VAL_V5GTF_CCH_PRECODING_TXD                                1

// RsSpecAn_rngV5GTFControlSequenceGeneration
#define RSSPECAN_VAL_V5GTF_SEQUENCE_NCID                                    0
#define RSSPECAN_VAL_V5GTF_SEQUENCE_NXPID                                   1

// RsSpecAn_rngV5GTFYAxisAutoScaling
#define RSSPECAN_VAL_V5GTF_Y_AXIS_AUTO_ALL                                  0
#define RSSPECAN_VAL_V5GTF_Y_AXIS_AUTO_DEFAULT                              1
#define RSSPECAN_VAL_V5GTF_Y_AXIS_AUTO_ONCE                                 2

// RsSpecAn_rngV5GTFXAxisUnits
#define RSSPECAN_VAL_V5GTF_UNITS_CARR                                       0
#define RSSPECAN_VAL_V5GTF_UNITS_HZ                                         1

// RsSpecAn_rngV5GTFChannelOrSignal
#define RSSPECAN_VAL_V5GTF_SYNC                                             0
#define RSSPECAN_VAL_V5GTF_XPDSCH                                           1
#define RSSPECAN_VAL_V5GTF_XPDCCH                                           2
#define RSSPECAN_VAL_V5GTF_XPBCH                                            3
#define RSSPECAN_VAL_V5GTF_EPBCH                                            4
#define RSSPECAN_VAL_V5GTF_CSI_RS                                           5
#define RSSPECAN_VAL_V5GTF_PCRS                                             6

// RsSpecAn_rngMinMaxAvg
#define RSSPECAN_VAL_AVG                                                    2

// RsSpecAn_rngOWULConstellationType
#define RSSPECAN_VAL_OW_UL_CONSTELLATION_ALL                                0
#define RSSPECAN_VAL_OW_UL_CONSTELLATION_REF_SYMBOLS                        1
#define RSSPECAN_VAL_OW_UL_CONSTELLATION_SRS                                2

// RsSpecAn_rngOWULInbandEmissionType
#define RSSPECAN_VAL_OW_UL_INBAND_X                                         0
#define RSSPECAN_VAL_OW_UL_INBAND_EMISSION                                  1
#define RSSPECAN_VAL_OW_UL_INBAND_UPPER_LIMIT                               2

// RsSpecAn_rng5GNRChannelEstimation
#define RSSPECAN_VAL_5G_NR_CHANNEL_ESTIMATION_OFF                           0
#define RSSPECAN_VAL_5G_NR_CHANNEL_ESTIMATION_LINT                          1
#define RSSPECAN_VAL_5G_NR_CHANNEL_ESTIMATION_PILPAY                        2
#define RSSPECAN_VAL_5G_NR_CHANNEL_ESTIMATION_NORMAL                        3

// RsSpecAn_rng5GNRChannelEstimationTimeAveraging
#define RSSPECAN_VAL_5G_NR_TIME_AVERAGING_PALL                              0
#define RSSPECAN_VAL_5G_NR_TIME_AVERAGING_TGPP                              1

// RsSpecAn_rng5GNRDemodulationData
#define RSSPECAN_VAL_5G_NR_DEMOD_DATA_AFTER_DESCRAMBLE                      0
#define RSSPECAN_VAL_5G_NR_DEMOD_DATA_BEFORE_DESCRAMBLE                     1
#define RSSPECAN_VAL_5G_NR_DEMOD_DATA_PAYLOAD_DATA                          2

// RsSpecAn_rng5GNRDLAnalysisCCResult
#define RSSPECAN_VAL_5G_NR_CC_RESULT_ALL                                    0
#define RSSPECAN_VAL_5G_NR_CC_RESULT_VIEWED                                 1

// RsSpecAn_rng5GNRDLTrackingPhase
#define RSSPECAN_VAL_5G_NR_TRACKING_PHASE_TGPP                              3

// RsSpecAn_rng5GNRDLBaseStationType
#define RSSPECAN_VAL_5G_NR_BS_TYPE_FR1C                                     0
#define RSSPECAN_VAL_5G_NR_BS_TYPE_FR1H                                     1
#define RSSPECAN_VAL_5G_NR_BS_TYPE_FR1O                                     2
#define RSSPECAN_VAL_5G_NR_BS_TYPE_FR2O                                     3

// RsSpecAn_rng5GNRCSIRSSlotMode
#define RSSPECAN_VAL_5G_NR_CSI_RS_SLOT_MODE_APERIODIC                       0
#define RSSPECAN_VAL_5G_NR_CSI_RS_SLOT_MODE_PERIODIC                        1

// RsSpecAn_rng5GNRBandwidthPartsDemodulation
#define RSSPECAN_VAL_5G_NR_BW_PARTS_DEMODULATION_AUTO                       0
#define RSSPECAN_VAL_5G_NR_BW_PARTS_DEMODULATION_MANUAL                     1

// RsSpecAn_rng5GNRChannelCodingMCSTable
#define RSSPECAN_VAL_5G_NR_CHANNEL_CODING_MCS_TABLE_Q64                     0
#define RSSPECAN_VAL_5G_NR_CHANNEL_CODING_MCS_TABLE_Q64L                    1
#define RSSPECAN_VAL_5G_NR_CHANNEL_CODING_MCS_TABLE_Q256                    2

// RsSpecAn_rng5GNRPDSCHReferencePoint
#define RSSPECAN_VAL_5G_NR_PDSCH_REFERENCE_POINT_BWPSTART                   0
#define RSSPECAN_VAL_5G_NR_PDSCH_REFERENCE_POINT_RPA                        1

// RsSpecAn_rng5GNRScrambling
#define RSSPECAN_VAL_5G_NR_SCRAMBLING_DS_ID                                 0
#define RSSPECAN_VAL_5G_NR_SCRAMBLING_N_ID_CELL                             1

// RsSpecAn_rng5GNRCORESETReferencePoint
#define RSSPECAN_VAL_5G_NR_CORESET_REFERENCE_POINT_CORESET_START            0
#define RSSPECAN_VAL_5G_NR_CORESET_REFERENCE_POINT_A                        1

// RsSpecAn_rng5GNRCORESETPrecoderGranularity
#define RSSPECAN_VAL_5G_NR_PRECODER_GRANULARITY_ALL_CONTINUOUS_RBS          0
#define RSSPECAN_VAL_5G_NR_PRECODER_GRANULARITY_REG_BUNDLE                  1

// RsSpecAn_rng5GNRSynchronizationSignalOffsetRelativeTo
#define RSSPECAN_VAL_5G_NR_DL_SYNCH_OFFSET_REL_TO_RPA                       0
#define RSSPECAN_VAL_5G_NR_DL_SYNCH_OFFSET_REL_TO_TXBW                      1

// RsSpecAn_rngCarrierAxesReference
#define RSSPECAN_VAL_CARRIER_AXES_REFERENCE_LOWEST_RB                       0
#define RSSPECAN_VAL_CARRIER_AXES_REFERENCE_RELATIVE_TO_CF                  1

// rsspecan_rngLimitCheckResult
#define RSSPECAN_VAL_LIMIT_CHECK_FAILED                                     0
#define RSSPECAN_VAL_LIMIT_CHECK_PASSED                                     1
#define RSSPECAN_VAL_LIMIT_CHECK_NOT_EVALUATED                              2

// RsSpecAn_rng5GNRCSIRSSlotPeriodicity
#define RSSPECAN_VAL_5G_NR_CSI_RS_SLOT_PERIODICITY_4                        4
#define RSSPECAN_VAL_5G_NR_CSI_RS_SLOT_PERIODICITY_5                        5
#define RSSPECAN_VAL_5G_NR_CSI_RS_SLOT_PERIODICITY_8                        8
#define RSSPECAN_VAL_5G_NR_CSI_RS_SLOT_PERIODICITY_10                       10
#define RSSPECAN_VAL_5G_NR_CSI_RS_SLOT_PERIODICITY_16                       16
#define RSSPECAN_VAL_5G_NR_CSI_RS_SLOT_PERIODICITY_20                       20
#define RSSPECAN_VAL_5G_NR_CSI_RS_SLOT_PERIODICITY_32                       32
#define RSSPECAN_VAL_5G_NR_CSI_RS_SLOT_PERIODICITY_40                       40
#define RSSPECAN_VAL_5G_NR_CSI_RS_SLOT_PERIODICITY_64                       64
#define RSSPECAN_VAL_5G_NR_CSI_RS_SLOT_PERIODICITY_80                       80
#define RSSPECAN_VAL_5G_NR_CSI_RS_SLOT_PERIODICITY_160                      160
#define RSSPECAN_VAL_5G_NR_CSI_RS_SLOT_PERIODICITY_320                      320
#define RSSPECAN_VAL_5G_NR_CSI_RS_SLOT_PERIODICITY_640                      640

// RsSpecAn_rng5GNRCSIRSCDMType
#define RSSPECAN_VAL_5G_NR_CSI_RS_CDM_TYPE_NONE                             0
#define RSSPECAN_VAL_5G_NR_CSI_RS_CDM_TYPE_FD_CDM_2                         1
#define RSSPECAN_VAL_5G_NR_CSI_RS_CDM_TYPE_CDM_4                            2
#define RSSPECAN_VAL_5G_NR_CSI_RS_CDM_TYPE_CDM_8                            3

// RsSpecAn_rng5GNRCSIRSDensity
#define RSSPECAN_VAL_5G_NR_CSI_RS_DENSITY_EVEN                              0
#define RSSPECAN_VAL_5G_NR_CSI_RS_DENSITY_ODD                               1
#define RSSPECAN_VAL_5G_NR_CSI_RS_DENSITY_1                                 2
#define RSSPECAN_VAL_5G_NR_CSI_RS_DENSITY_3                                 3

// RsSpecAn_rng5GNRCORESETInterleavingShiftIndex
#define RSSPECAN_VAL_5G_NR_INTERLEAVING_SHIFT_INDEX_N_ID_CELL               0
#define RSSPECAN_VAL_5G_NR_INTERLEAVING_SHIFT_INDEX_N_SHIFT                 1

// RsSpecAn_rng5GNRResultSummaryItems
#define RSSPECAN_VAL_5G_NR_ITEM_CREST                                       0
#define RSSPECAN_VAL_5G_NR_ITEM_DSQP                                        1
#define RSSPECAN_VAL_5G_NR_ITEM_DSSF                                        2
#define RSSPECAN_VAL_5G_NR_ITEM_DSST                                        3
#define RSSPECAN_VAL_5G_NR_ITEM_DSTS                                        4
#define RSSPECAN_VAL_5G_NR_ITEM_EVM                                         5
#define RSSPECAN_VAL_5G_NR_ITEM_P_CHANNEL                                   6
#define RSSPECAN_VAL_5G_NR_ITEM_P_SIGNAL                                    7
#define RSSPECAN_VAL_5G_NR_ITEM_FREQ_ERROR                                  8
#define RSSPECAN_VAL_5G_NR_ITEM_SAMPLING_ERROR                              9
#define RSSPECAN_VAL_5G_NR_ITEM_QUADRATURE                                  10
#define RSSPECAN_VAL_5G_NR_ITEM_GAIN_IMBALANCE                              11
#define RSSPECAN_VAL_5G_NR_ITEM_IQ_OFFSET                                   12
#define RSSPECAN_VAL_5G_NR_ITEM_POWER                                       13
#define RSSPECAN_VAL_5G_NR_ITEM_PPRE                                        14
#define RSSPECAN_VAL_5G_NR_ITEM_MODULATION                                  15
#define RSSPECAN_VAL_5G_NR_ITEM_NORB                                        16
#define RSSPECAN_VAL_5G_NR_ITEM_OSTP                                        17
#define RSSPECAN_VAL_5G_NR_ITEM_RSTP                                        18
#define RSSPECAN_VAL_5G_NR_ITEM_SS_POWER                                    19
#define RSSPECAN_VAL_5G_NR_ITEM_CSI_POWER                                   20
#define RSSPECAN_VAL_5G_NR_ITEM_SDPB                                        21
#define RSSPECAN_VAL_5G_NR_ITEM_SDQP                                        22
#define RSSPECAN_VAL_5G_NR_ITEM_SDSF                                        23
#define RSSPECAN_VAL_5G_NR_ITEM_SDST                                        24
#define RSSPECAN_VAL_5G_NR_ITEM_SDTS                                        25
#define RSSPECAN_VAL_5G_NR_ITEM_UCCD                                        26
#define RSSPECAN_VAL_5G_NR_ITEM_UCCH                                        27
#define RSSPECAN_VAL_5G_NR_ITEM_USPB                                        28
#define RSSPECAN_VAL_5G_NR_ITEM_USQP                                        29
#define RSSPECAN_VAL_5G_NR_ITEM_USSF                                        30
#define RSSPECAN_VAL_5G_NR_ITEM_USST                                        31
#define RSSPECAN_VAL_5G_NR_ITEM_USTS                                        32
#define RSSPECAN_VAL_5G_NR_ITEM_DS1K                                        33
#define RSSPECAN_VAL_5G_NR_ITEM_DS4K                                        34
#define RSSPECAN_VAL_5G_NR_ITEM_UPR                                         35
#define RSSPECAN_VAL_5G_NR_ITEM_FSOF                                        36
#define RSSPECAN_VAL_5G_NR_ITEM_EVMP                                        37
#define RSSPECAN_VAL_5G_NR_ITEM_RSSI                                        38
#define RSSPECAN_VAL_5G_NR_ITEM_SD1K                                        39
#define RSSPECAN_VAL_5G_NR_ITEM_SD4K                                        40
#define RSSPECAN_VAL_5G_NR_ITEM_US1K                                        41
#define RSSPECAN_VAL_5G_NR_ITEM_US4K                                        42
#define RSSPECAN_VAL_5G_NR_ITEM_TST                                         43
#define RSSPECAN_VAL_5G_NR_ITEM_TSD                                         44
#define RSSPECAN_VAL_5G_NR_ITEM_MID                                         45
#define RSSPECAN_VAL_5G_NR_ITEM_SST                                         46
#define RSSPECAN_VAL_5G_NR_ITEM_APF                                         47
#define RSSPECAN_VAL_5G_NR_ITEM_AAPF                                        48
#define RSSPECAN_VAL_5G_NR_ITEM_ARPF                                        49
#define RSSPECAN_VAL_5G_NR_ITEM_SPF                                         50

// RsSpecAn_rng5GNRTimeAlignmentErrorResultType
#define RSSPECAN_VAL_5G_NR_TAER_RESULT_TYPE_MIN                             0
#define RSSPECAN_VAL_5G_NR_TAER_RESULT_TYPE_MEAN                            1
#define RSSPECAN_VAL_5G_NR_TAER_RESULT_TYPE_MAX                             2

// RsSpecAn_rng5GNRDLLayoutType
#define RSSPECAN_VAL_LAYOUT_TYPE_BSUM                                       107
#define RSSPECAN_VAL_LAYOUT_TYPE_FVCA                                       108
#define RSSPECAN_VAL_LAYOUT_TYPE_RSPD                                       109
#define RSSPECAN_VAL_LAYOUT_TYPE_RSWP                                       110
#define RSSPECAN_VAL_LAYOUT_TYPE_FALL                                       114
#define RSSPECAN_VAL_LAYOUT_TYPE_RIS                                        117
#define RSSPECAN_VAL_LAYOUT_TYPE_FVSU                                       120
#define RSSPECAN_VAL_LAYOUT_TYPE_RSMA                                       121
#define RSSPECAN_VAL_LAYOUT_TYPE_ADI                                        122
#define RSSPECAN_VAL_LAYOUT_TYPE_ARS                                        123
#define RSSPECAN_VAL_LAYOUT_TYPE_CMS                                        124
#define RSSPECAN_VAL_LAYOUT_TYPE_SDI                                        125
#define RSSPECAN_VAL_LAYOUT_TYPE_SRS                                        126

// RsSpecAn_rng5GNRULLayoutType
#define RSSPECAN_VAL_5G_NR_UL_LAYOUT_AISC                                   0
#define RSSPECAN_VAL_5G_NR_UL_LAYOUT_ASUM                                   1
#define RSSPECAN_VAL_5G_NR_UL_LAYOUT_CBUF                                   2
#define RSSPECAN_VAL_5G_NR_UL_LAYOUT_CCDF                                   3
#define RSSPECAN_VAL_5G_NR_UL_LAYOUT_CONS                                   4
#define RSSPECAN_VAL_5G_NR_UL_LAYOUT_EVCA                                   5
#define RSSPECAN_VAL_5G_NR_UL_LAYOUT_EVRP                                   6
#define RSSPECAN_VAL_5G_NR_UL_LAYOUT_EVSC                                   7
#define RSSPECAN_VAL_5G_NR_UL_LAYOUT_EVSY                                   8
#define RSSPECAN_VAL_5G_NR_UL_LAYOUT_FEVS                                   9
#define RSSPECAN_VAL_5G_NR_UL_LAYOUT_FVCA                                   10
#define RSSPECAN_VAL_5G_NR_UL_LAYOUT_IE                                     11
#define RSSPECAN_VAL_5G_NR_UL_LAYOUT_IEA                                    12
#define RSSPECAN_VAL_5G_NR_UL_LAYOUT_MTAB                                   13
#define RSSPECAN_VAL_5G_NR_UL_LAYOUT_PSPE                                   14
#define RSSPECAN_VAL_5G_NR_UL_LAYOUT_PVSC                                   15
#define RSSPECAN_VAL_5G_NR_UL_LAYOUT_RSUM                                   16
#define RSSPECAN_VAL_5G_NR_UL_LAYOUT_DIAG                                   17
#define RSSPECAN_VAL_5G_NR_UL_LAYOUT_PEAK                                   18
#define RSSPECAN_VAL_5G_NR_UL_LAYOUT_BITS                                   19
#define RSSPECAN_VAL_5G_NR_UL_LAYOUT_SFL                                    20
#define RSSPECAN_VAL_5G_NR_UL_LAYOUT_FVSU                                   21
#define RSSPECAN_VAL_5G_NR_UL_LAYOUT_IES                                    22
#define RSSPECAN_VAL_5G_NR_UL_LAYOUT_ADI                                    23
#define RSSPECAN_VAL_5G_NR_UL_LAYOUT_ARS                                    24
#define RSSPECAN_VAL_5G_NR_UL_LAYOUT_CMS                                    25
#define RSSPECAN_VAL_5G_NR_UL_LAYOUT_SDI                                    26
#define RSSPECAN_VAL_5G_NR_UL_LAYOUT_SRS                                    27

// RsSpecAn_rngOWULFilterModulation
#define RSSPECAN_VAL_OW_UL_FILTER_MODULATION_ALL                            -1
#define RSSPECAN_VAL_OW_UL_FILTER_MODULATION_UNRECOGNIZED                   0
#define RSSPECAN_VAL_OW_UL_FILTER_MODULATION_RBPSK                          1
#define RSSPECAN_VAL_OW_UL_FILTER_MODULATION_QPSK                           2
#define RSSPECAN_VAL_OW_UL_FILTER_MODULATION_16QAM                          3
#define RSSPECAN_VAL_OW_UL_FILTER_MODULATION_64QAM                          4
#define RSSPECAN_VAL_OW_UL_FILTER_MODULATION_8PSK                           5
#define RSSPECAN_VAL_OW_UL_FILTER_MODULATION_CAZAC                          6
#define RSSPECAN_VAL_OW_UL_FILTER_MODULATION_MIXED                          7
#define RSSPECAN_VAL_OW_UL_FILTER_MODULATION_BPSK                           8
#define RSSPECAN_VAL_OW_UL_FILTER_MODULATION_256QAM                         14

// RsSpecAn_rngV5GTFSequenceGenerationPCRS
#define RSSPECAN_VAL_V5GTF_SEQ_NIPC                                         1

// rsspecan_rngNBIotDownlinkMeasurement
#define RSSPECAN_VAL_NB_IOT_DOWNLINK_MEAS_ACLR                              0
#define RSSPECAN_VAL_NB_IOT_DOWNLINK_MEAS_ESP                               1
#define RSSPECAN_VAL_NB_IOT_DOWNLINK_MEAS_EVM                               2
#define RSSPECAN_VAL_NB_IOT_DOWNLINK_MEAS_TAER                              3

// RsSpecAn_rng5GNRULPDSCHModulation
#define RSSPECAN_VAL_5G_NR_PDSCH_MODULATION_PIT_BPSK                        5

// RsSpecAn_rngSubwindowTracePreset
#define RSSPECAN_VAL_SUBWINDOW_TRACE_PRESET_ALL                             0
#define RSSPECAN_VAL_SUBWINDOW_TRACE_PRESET_MAM                             1
#define RSSPECAN_VAL_SUBWINDOW_TRACE_PRESET_MCM                             2

// RsSpecAn_rngUpDown
#define RSSPECAN_VAL_UP                                                     0
#define RSSPECAN_VAL_DOWN                                                   1

// RsSpecAn_rngTransientSettlingTimeTrace
#define RSSPECAN_VAL_SETTLING_TIME_TRACE_1                                  1
#define RSSPECAN_VAL_SETTLING_TIME_TRACE_2                                  2
#define RSSPECAN_VAL_SETTLING_TIME_TRACE_3                                  3
#define RSSPECAN_VAL_SETTLING_TIME_TRACE_4                                  4
#define RSSPECAN_VAL_SETTLING_TIME_TRACE_5                                  5
#define RSSPECAN_VAL_SETTLING_TIME_TRACE_6                                  6

// RsSpecAn_rngTransientFrequencyDeviationTrace
#define RSSPECAN_VAL_FREQUENCY_DEVIATION_TRACE_1                            1
#define RSSPECAN_VAL_FREQUENCY_DEVIATION_TRACE_2                            2
#define RSSPECAN_VAL_FREQUENCY_DEVIATION_TRACE_3                            3
#define RSSPECAN_VAL_FREQUENCY_DEVIATION_TRACE_4                            4
#define RSSPECAN_VAL_FREQUENCY_DEVIATION_TRACE_5                            5
#define RSSPECAN_VAL_FREQUENCY_DEVIATION_TRACE_6                            6

// RsSpecAn_rngPhaseSpuriousSortOrder
#define RSSPECAN_VAL_SPUR_SORT_ORDER_POWER                                  0
#define RSSPECAN_VAL_SPUR_SORT_ORDER_OFFSET                                 1

// RsSpecAn_rngPhaseSpuriousFilterMode
#define RSSPECAN_VAL_SPURIOUS_FILTER_MODE_OFF                               0
#define RSSPECAN_VAL_SPURIOUS_FILTER_MODE_SUPPRESS                          1
#define RSSPECAN_VAL_SPURIOUS_FILTER_MODE_SHOW                              2

// RsSpecAn_rngAutoAdjustSearchMode
#define RSSPECAN_VAL_SEARCH_MODE_FAST                                       0
#define RSSPECAN_VAL_SEARCH_MODE_POPT                                       1

// RsSpecAn_rngMCGDModeEstimation
#define RSSPECAN_VAL_MODE_ESTIMATION_CARR                                   0
#define RSSPECAN_VAL_MODE_ESTIMATION_OFFS                                   1
#define RSSPECAN_VAL_MODE_ESTIMATION_OFF                                    2

// RsSpecAn_rngNBIoTDownlinkAnalysisLocationSelect
#define RSSPECAN_VAL_NB_IOT_LOCATION_SELECT_AMD                             0
#define RSSPECAN_VAL_NB_IOT_LOCATION_SELECT_BMD                             1

// RsSpecAn_rngAmplifierCrestFactorEVMReference
#define RSSPECAN_VAL_CRF_REFERENCE_NEW_ARB                                  0
#define RSSPECAN_VAL_CRF_REFERENCE_ORIGINAL                                 1

// RsSpecAn_rng5GNRDLEnhancedPDSCHVRBtoPRBInterleaver
#define RSSPECAN_VAL_5G_NR_VRB_TO_PRB_INTERLEAVER_NONE                      0
#define RSSPECAN_VAL_5G_NR_VRB_TO_PRB_INTERLEAVER_2                         2
#define RSSPECAN_VAL_5G_NR_VRB_TO_PRB_INTERLEAVER_4                         4

// RsSpecAn_rng5GNRDLDMRSMethod
#define RSSPECAN_VAL_5G_NR_DMRS_METHOD_NIDDMRS                              0

// RsSpecAn_rng5GNRHandlingOfCarrierLeakage
#define RSSPECAN_VAL_5G_NR_CARRIER_LEAKAGE_NONE                             0
#define RSSPECAN_VAL_5G_NR_CARRIER_LEAKAGE_IGNORE_DC                        1
#define RSSPECAN_VAL_5G_NR_CARRIER_LEAKAGE_COMPENSATE                       2

// RsSpecAn_rng5GNRLTEResourceBlocks
#define RSSPECAN_VAL_5G_NR_LTE_RESOURCE_BLOCKS_6                            0
#define RSSPECAN_VAL_5G_NR_LTE_RESOURCE_BLOCKS_15                           1
#define RSSPECAN_VAL_5G_NR_LTE_RESOURCE_BLOCKS_25                           2
#define RSSPECAN_VAL_5G_NR_LTE_RESOURCE_BLOCKS_50                           3
#define RSSPECAN_VAL_5G_NR_LTE_RESOURCE_BLOCKS_75                           4
#define RSSPECAN_VAL_5G_NR_LTE_RESOURCE_BLOCKS_100                          5

// RsSpecAn_rng5GNRLTEAntennaPorts
#define RSSPECAN_VAL_5G_NR_LTE_ANTENNA_PORTS_1                              1
#define RSSPECAN_VAL_5G_NR_LTE_ANTENNA_PORTS_2                              2
#define RSSPECAN_VAL_5G_NR_LTE_ANTENNA_PORTS_4                              4

// RsSpecAn_rng5GNRDLEVMCalculationMethod
#define RSSPECAN_VAL_5G_NR_EVM_CALCULATION_METHOD_TGPP                      0
#define RSSPECAN_VAL_5G_NR_EVM_CALCULATION_METHOD_AOTP                      1
#define RSSPECAN_VAL_5G_NR_EVM_CALCULATION_METHOD_HPOS                      2
#define RSSPECAN_VAL_5G_NR_EVM_CALCULATION_METHOD_LPOS                      3
#define RSSPECAN_VAL_5G_NR_EVM_CALCULATION_METHOD_OTP                       4
#define RSSPECAN_VAL_5G_NR_EVM_CALCULATION_METHOD_CUST                      5

// RsSpecAn_rng5GNRBSCategoryBOption
#define RSSPECAN_VAL_5G_NR_BS_CATEGORY_B_OPTION_1                           1
#define RSSPECAN_VAL_5G_NR_BS_CATEGORY_B_OPTION_2                           2

// RsSpecAn_rng5GNREVMCalculationMethod
#define RSSPECAN_VAL_5G_NR_CMETHOD_TGPP                                     0
#define RSSPECAN_VAL_5G_NR_CMETHOD_OTP                                      1
#define RSSPECAN_VAL_5G_NR_CMETHOD_HPOS                                     2
#define RSSPECAN_VAL_5G_NR_CMETHOD_LPOS                                     3

// RsSpecAn_rng5GNRULPowerClass
#define RSSPECAN_VAL_5G_NR_POWER_CLASS_1                                    0
#define RSSPECAN_VAL_5G_NR_POWER_CLASS_2                                    1
#define RSSPECAN_VAL_5G_NR_POWER_CLASS_3                                    2
#define RSSPECAN_VAL_5G_NR_POWER_CLASS_4                                    3

// RsSpecAn_rng5GNRULPRACHFormat
#define RSSPECAN_VAL_5G_NR_PRACH_FORMAT_0                                   0
#define RSSPECAN_VAL_5G_NR_PRACH_FORMAT_1                                   1
#define RSSPECAN_VAL_5G_NR_PRACH_FORMAT_2                                   2
#define RSSPECAN_VAL_5G_NR_PRACH_FORMAT_3                                   3
#define RSSPECAN_VAL_5G_NR_PRACH_FORMAT_A1                                  4
#define RSSPECAN_VAL_5G_NR_PRACH_FORMAT_A2                                  5
#define RSSPECAN_VAL_5G_NR_PRACH_FORMAT_A3                                  6
#define RSSPECAN_VAL_5G_NR_PRACH_FORMAT_B1                                  7
#define RSSPECAN_VAL_5G_NR_PRACH_FORMAT_B2                                  8
#define RSSPECAN_VAL_5G_NR_PRACH_FORMAT_B3                                  9
#define RSSPECAN_VAL_5G_NR_PRACH_FORMAT_B4                                  10
#define RSSPECAN_VAL_5G_NR_PRACH_FORMAT_C0                                  11
#define RSSPECAN_VAL_5G_NR_PRACH_FORMAT_C2                                  12

// RsSpecAn_rng5GNRULPRACHRestrictedSet
#define RSSPECAN_VAL_5G_NR_PRACH_RSET_NONE                                  0
#define RSSPECAN_VAL_5G_NR_PRACH_RSET_A                                     1
#define RSSPECAN_VAL_5G_NR_PRACH_RSET_B                                     2

// RsSpecAn_rng5GNRULPRACHSubcarrierSpacing
#define RSSPECAN_VAL_5G_NR_PRACH_SCS_SS1_25                                 0
#define RSSPECAN_VAL_5G_NR_PRACH_SCS_SS5                                    1
#define RSSPECAN_VAL_5G_NR_PRACH_SCS_SS15                                   2
#define RSSPECAN_VAL_5G_NR_PRACH_SCS_SS30                                   3
#define RSSPECAN_VAL_5G_NR_PRACH_SCS_SS60                                   4
#define RSSPECAN_VAL_5G_NR_PRACH_SCS_SS120                                  5

// RsSpecAn_rng5GNRIFF
#define RSSPECAN_VAL_5G_NR_IFF_DUT15                                        0
#define RSSPECAN_VAL_5G_NR_IFF_DUT30                                        1

// RsSpecAn_rng5GNRLimitResult
#define RSSPECAN_VAL_5G_NR_LIMIT_FAILED                                     0
#define RSSPECAN_VAL_5G_NR_LIMIT_PASSED                                     1
#define RSSPECAN_VAL_5G_NR_LIMIT_NONE                                       2

// RsSpecAn_rng5GNRLimitCheckType
#define RSSPECAN_VAL_5G_NR_LIMIT_ABS                                        0
#define RSSPECAN_VAL_5G_NR_LIMIT_REL                                        1
#define RSSPECAN_VAL_5G_NR_LIMIT_DEFAULT                                    2

// RsSpecAn_rngHRPUWBMeasurementChannelNumber
#define RSSPECAN_VAL_HRPUWB_MEASUREMENT_CHANNEL_NUMBER_C0                   0
#define RSSPECAN_VAL_HRPUWB_MEASUREMENT_CHANNEL_NUMBER_C1                   1
#define RSSPECAN_VAL_HRPUWB_MEASUREMENT_CHANNEL_NUMBER_C2                   2
#define RSSPECAN_VAL_HRPUWB_MEASUREMENT_CHANNEL_NUMBER_C3                   3
#define RSSPECAN_VAL_HRPUWB_MEASUREMENT_CHANNEL_NUMBER_C4                   4
#define RSSPECAN_VAL_HRPUWB_MEASUREMENT_CHANNEL_NUMBER_C5                   5
#define RSSPECAN_VAL_HRPUWB_MEASUREMENT_CHANNEL_NUMBER_C6                   6
#define RSSPECAN_VAL_HRPUWB_MEASUREMENT_CHANNEL_NUMBER_C7                   7
#define RSSPECAN_VAL_HRPUWB_MEASUREMENT_CHANNEL_NUMBER_C8                   8
#define RSSPECAN_VAL_HRPUWB_MEASUREMENT_CHANNEL_NUMBER_C9                   9
#define RSSPECAN_VAL_HRPUWB_MEASUREMENT_CHANNEL_NUMBER_C10                  10
#define RSSPECAN_VAL_HRPUWB_MEASUREMENT_CHANNEL_NUMBER_C11                  11
#define RSSPECAN_VAL_HRPUWB_MEASUREMENT_CHANNEL_NUMBER_C12                  12
#define RSSPECAN_VAL_HRPUWB_MEASUREMENT_CHANNEL_NUMBER_C13                  13
#define RSSPECAN_VAL_HRPUWB_MEASUREMENT_CHANNEL_NUMBER_C14                  14
#define RSSPECAN_VAL_HRPUWB_MEASUREMENT_CHANNEL_NUMBER_C15                  15

// RsSpecAn_rngHRPUWBOversamplingFactor
#define RSSPECAN_VAL_HRPUWB_OVERSAMPLING_FACTOR_OV4                         4
#define RSSPECAN_VAL_HRPUWB_OVERSAMPLING_FACTOR_OV6                         6
#define RSSPECAN_VAL_HRPUWB_OVERSAMPLING_FACTOR_OV8                         8
#define RSSPECAN_VAL_HRPUWB_OVERSAMPLING_FACTOR_OV10                        10
#define RSSPECAN_VAL_HRPUWB_OVERSAMPLING_FACTOR_OV12                        12

// RsSpecAn_rngHRPUWBDemodulationMode
#define RSSPECAN_VAL_HRPUWB_DEMODULATION_MODE_HRP                           0
#define RSSPECAN_VAL_HRPUWB_DEMODULATION_MODE_BPRF                          1
#define RSSPECAN_VAL_HRPUWB_DEMODULATION_MODE_HPRF                          2

// RsSpecAn_rngHRPUWBDemodulationPHYDataRateMode
#define RSSPECAN_VAL_HRPUWB_DEMODULATION_PHY_DATA_RATE_MODE_DRBM_LP         0
#define RSSPECAN_VAL_HRPUWB_DEMODULATION_PHY_DATA_RATE_MODE_DRBM_HP         1
#define RSSPECAN_VAL_HRPUWB_DEMODULATION_PHY_DATA_RATE_MODE_DRHM_LR         2
#define RSSPECAN_VAL_HRPUWB_DEMODULATION_PHY_DATA_RATE_MODE_DRHM_HR         3

// RsSpecAn_rngHRPUWBDemodulationHPRFPayloadSize
#define RSSPECAN_VAL_HRPUWB_DEMODULATION_HPRF_PAYLOAD_SIZE_S0               0
#define RSSPECAN_VAL_HRPUWB_DEMODULATION_HPRF_PAYLOAD_SIZE_S1               1
#define RSSPECAN_VAL_HRPUWB_DEMODULATION_HPRF_PAYLOAD_SIZE_S2               2

// RsSpecAn_rngHRPUWBDemodulationPacketFormat
#define RSSPECAN_VAL_HRPUWB_DEMODULATION_PACKET_FORMAT_OFF                  0
#define RSSPECAN_VAL_HRPUWB_DEMODULATION_PACKET_FORMAT_1                    1
#define RSSPECAN_VAL_HRPUWB_DEMODULATION_PACKET_FORMAT_2                    2
#define RSSPECAN_VAL_HRPUWB_DEMODULATION_PACKET_FORMAT_3                    3

// RsSpecAn_rngHRPUWBDemodulationActiveSegmentLength
#define RSSPECAN_VAL_HRPUWB_DEMODULATION_ACTIVE_SEGMENT_LENGTH_L16          0
#define RSSPECAN_VAL_HRPUWB_DEMODULATION_ACTIVE_SEGMENT_LENGTH_L32          1
#define RSSPECAN_VAL_HRPUWB_DEMODULATION_ACTIVE_SEGMENT_LENGTH_L64          2
#define RSSPECAN_VAL_HRPUWB_DEMODULATION_ACTIVE_SEGMENT_LENGTH_L128         3
#define RSSPECAN_VAL_HRPUWB_DEMODULATION_ACTIVE_SEGMENT_LENGTH_L256         4

// RsSpecAn_rngHRPUWBDemodulationActiveSegments
#define RSSPECAN_VAL_HRPUWB_DEMODULATION_ACTIVE_SEGMENTS_S1                 1
#define RSSPECAN_VAL_HRPUWB_DEMODULATION_ACTIVE_SEGMENTS_S2                 2
#define RSSPECAN_VAL_HRPUWB_DEMODULATION_ACTIVE_SEGMENTS_S3                 3
#define RSSPECAN_VAL_HRPUWB_DEMODULATION_ACTIVE_SEGMENTS_S4                 4

// RsSpecAn_rngHRPUWBDemodulationRange
#define RSSPECAN_VAL_HRPUWB_DEMODULATION_RANGE_CAPTURE                      0
#define RSSPECAN_VAL_HRPUWB_DEMODULATION_RANGE_COUNT                        1

// RsSpecAn_rngHRPUWBAllPacketsResult
#define RSSPECAN_VAL_HRPUWB_ALL_PACKETS_RESULT_IDENTICAL                    0
#define RSSPECAN_VAL_HRPUWB_ALL_PACKETS_RESULT_MIXED                        1

// RsSpecAn_rngHRPUWBPacketDataConstraintLength
#define RSSPECAN_VAL_HRPUWB_PACKET_DATA_CONSTRAINT_LENGTH_CL3               0
#define RSSPECAN_VAL_HRPUWB_PACKET_DATA_CONSTRAINT_LENGTH_CL7               1

// RsSpecAn_rng5GNRULAutoDemodulation
#define RSSPECAN_VAL_NR_UL_DEMOD_AUTO                                       0
#define RSSPECAN_VAL_NR_UL_DEMOD_MAN                                        1

// RsSpecAn_rng5GNRCopyMode
#define RSSPECAN_VAL_COPY_MODE_CUSTOM                                       0
#define RSSPECAN_VAL_COPY_MODE_SLOT                                         1

// RsSpecAn_rng5GNRNIRS
#define RSSPECAN_VAL_NR_N_ID_RS_NIDCELL                                     0
#define RSSPECAN_VAL_NR_N_ID_RS_NIDPSCH                                     1
#define RSSPECAN_VAL_NR_N_ID_RS_NIDNSCID                                    2

// RsSpecAn_rngPTRSNID
#define RSSPECAN_VAL_PTRS_NID_NIDCELL                                       0
#define RSSPECAN_VAL_PTRS_NID_NIDPUSCH                                      1

// RsSpecAn_rngPasteMode
#define RSSPECAN_VAL_PASTE_MODE_CUSTOM                                      0
#define RSSPECAN_VAL_PASTE_MODE_DATA                                        1
#define RSSPECAN_VAL_PASTE_MODE_SLOTS                                       2
#define RSSPECAN_VAL_PASTE_MODE_UNUSED                                      3

// RsSpecAn_rngPUCCHCopyMode
#define RSSPECAN_VAL_PUCCH_COPY_MODE_CUSTOM                                 0
#define RSSPECAN_VAL_PUCCH_COPY_MODE_SLOT                                   1

// RsSpecAn_rngPUSCHFrequencyHopping
#define RSSPECAN_VAL_PUSCH_FREQ_HOPPING_DISABLE                             0
#define RSSPECAN_VAL_PUSCH_FREQ_HOPPING_INTRA                               1

// RsSpecAn_rngLimitPassMode
#define RSSPECAN_VAL_LIMIT_PASS_MODE_AND                                    0
#define RSSPECAN_VAL_LIMIT_PASS_MODE_OR                                     1

// RsSpecAn_rngCombinedMeasOptimization
#define RSSPECAN_VAL_COMBINED_MEAS_OPTIMIZATION_DYNAMIC                     0
#define RSSPECAN_VAL_COMBINED_MEAS_OPTIMIZATION_SPEED                       1

// RsSpecAn_rngCombinedMeasFrameTrigger
#define RSSPECAN_VAL_FRAME_TRIGGER_AVAILABLE                                0
#define RSSPECAN_VAL_FRAME_TRIGGER_NAVAILABLE                               1
#define RSSPECAN_VAL_FRAME_TRIGGER_RSLOTS                                   2

// RsSpecAn_rngCombinedMeasTriggerSequence
#define RSSPECAN_VAL_COMBINED_MEAS_TRIGGER_SEQUENCE_CLOOP                   0
#define RSSPECAN_VAL_COMBINED_MEAS_TRIGGER_SEQUENCE_MANUAL                  1
#define RSSPECAN_VAL_COMBINED_MEAS_TRIGGER_SEQUENCE_OLOOP                   2
#define RSSPECAN_VAL_COMBINED_MEAS_TRIGGER_SEQUENCE_PERIODIC                3

// RsSpecAn_rngChannelRaster
#define RSSPECAN_VAL_CHANNEL_RASTER_C15                                     0
#define RSSPECAN_VAL_CHANNEL_RASTER_C100                                    1

// RsSpecAn_rngOffsetRefPoint
#define RSSPECAN_VAL_OFFSET_REF_POINT_CC1                                   0
#define RSSPECAN_VAL_OFFSET_REF_POINT_GMCFREQ                               1

// RsSpecAn_rngFreqOffsetMode
#define RSSPECAN_VAL_FREQ_OFFSET_MODE_ARBITRARY                             0
#define RSSPECAN_VAL_FREQ_OFFSET_MODE_EQUIDISTANT                           1

// RsSpecAn_rngNR5GScalingFactor
#define RSSPECAN_VAL_NR5G_SCALING_FACTOR_ONE                                0
#define RSSPECAN_VAL_NR5G_SCALING_FACTOR_HALF                               1
#define RSSPECAN_VAL_NR5G_SCALING_FACTOR_QUAR                               2

// RsSpecAn_rngNR5GCopyMode
#define RSSPECAN_VAL_NR5G_COPY_MODE_CUSTOM                                  0
#define RSSPECAN_VAL_NR5G_COPY_MODE_SLOT                                    1

// RsSpecAn_rngNR5GDCIFormat
#define RSSPECAN_VAL_NR5G_DCI_FORMAT_F00                                    0
#define RSSPECAN_VAL_NR5G_DCI_FORMAT_F01                                    1
#define RSSPECAN_VAL_NR5G_DCI_FORMAT_F02                                    2
#define RSSPECAN_VAL_NR5G_DCI_FORMAT_F10                                    3
#define RSSPECAN_VAL_NR5G_DCI_FORMAT_F11                                    4
#define RSSPECAN_VAL_NR5G_DCI_FORMAT_F12                                    5
#define RSSPECAN_VAL_NR5G_DCI_FORMAT_F20                                    6
#define RSSPECAN_VAL_NR5G_DCI_FORMAT_F21                                    7
#define RSSPECAN_VAL_NR5G_DCI_FORMAT_F22                                    8
#define RSSPECAN_VAL_NR5G_DCI_FORMAT_F23                                    9
#define RSSPECAN_VAL_NR5G_DCI_FORMAT_F24                                    10
#define RSSPECAN_VAL_NR5G_DCI_FORMAT_F25                                    11
#define RSSPECAN_VAL_NR5G_DCI_FORMAT_F26                                    12

// RsSpecAn_rngResourceAssignment
#define RSSPECAN_VAL_RESOURCE_ASSIGNMENT_CUSTOM                             0
#define RSSPECAN_VAL_RESOURCE_ASSIGNMENT_ALL                                1

// RsSpecAn_rngDciScope
#define RSSPECAN_VAL_DCI_SCOPE_ICDF                                         0
#define RSSPECAN_VAL_DCI_SCOPE_SPUS                                         1

// RsSpecAn_rngNrPDSCHUsage
#define RSSPECAN_VAL_NR_PDSCH_USAGE_CRNT                                    0
#define RSSPECAN_VAL_NR_PDSCH_USAGE_CSRN                                    1
#define RSSPECAN_VAL_NR_PDSCH_USAGE_MCSC                                    2
#define RSSPECAN_VAL_NR_PDSCH_USAGE_TCRN                                    3
#define RSSPECAN_VAL_NR_PDSCH_USAGE_SPCR                                    4
#define RSSPECAN_VAL_NR_PDSCH_USAGE_PRNT                                    5
#define RSSPECAN_VAL_NR_PDSCH_USAGE_SIRN                                    6
#define RSSPECAN_VAL_NR_PDSCH_USAGE_RNRN                                    7
#define RSSPECAN_VAL_NR_PDSCH_USAGE_MSGB                                    8
#define RSSPECAN_VAL_NR_PDSCH_USAGE_SFIR                                    9
#define RSSPECAN_VAL_NR_PDSCH_USAGE_INTR                                    10
#define RSSPECAN_VAL_NR_PDSCH_USAGE_TPUS                                    11
#define RSSPECAN_VAL_NR_PDSCH_USAGE_TPUC                                    12
#define RSSPECAN_VAL_NR_PDSCH_USAGE_TSRN                                    13
#define RSSPECAN_VAL_NR_PDSCH_USAGE_CIRN                                    14
#define RSSPECAN_VAL_NR_PDSCH_USAGE_AIRN                                    15
#define RSSPECAN_VAL_NR_PDSCH_USAGE_PSRN                                    16

// RsSpecAn_rngHalfFrameOffset
#define RSSPECAN_VAL_HALF_FRAME_OFFSET_ZERO                                 0
#define RSSPECAN_VAL_HALF_FRAME_OFFSET_ONE                                  1

// RsSpecAn_rngNROverload
#define RSSPECAN_VAL_NR_OVERLOAD_RF_OVLD                                    0
#define RSSPECAN_VAL_NR_OVERLOAD_NAN                                        1
#define RSSPECAN_VAL_NR_OVERLOAD_OVLD                                       2
#define RSSPECAN_VAL_NR_OVERLOAD_INPUT_OVLD                                 3

// RsSpecAn_rngACLRLimitPassMode
#define RSSPECAN_VAL_ACLR_LIMIT_PASS_MODE_ABSOLUTE                          0
#define RSSPECAN_VAL_ACLR_LIMIT_PASS_MODE_RELATIVE                          1
#define RSSPECAN_VAL_ACLR_LIMIT_PASS_MODE_ABSOLUTE_OR_RELATIVE              2

// RsSpecAn_rngNrReferenceData
#define RSSPECAN_VAL_NR_REFERENCE_DATA_AUTO                                 0
#define RSSPECAN_VAL_NR_REFERENCE_DATA_ALL0                                 1
#define RSSPECAN_VAL_NR_REFERENCE_DATA_PN23                                 2

// RsSpecAn_rngDCIFields
#define RSSPECAN_VAL_DCI_FIELDS_IFDF                                        0
#define RSSPECAN_VAL_DCI_FIELDS_FDR                                         1
#define RSSPECAN_VAL_DCI_FIELDS_TDR                                         2
#define RSSPECAN_VAL_DCI_FIELDS_FHFL                                        3
#define RSSPECAN_VAL_DCI_FIELDS_MCS1                                        4
#define RSSPECAN_VAL_DCI_FIELDS_NDI1                                        5
#define RSSPECAN_VAL_DCI_FIELDS_RV1                                         6
#define RSSPECAN_VAL_DCI_FIELDS_HPN                                         7
#define RSSPECAN_VAL_DCI_FIELDS_TCSP                                        8
#define RSSPECAN_VAL_DCI_FIELDS_PBIT                                        9
#define RSSPECAN_VAL_DCI_FIELDS_USUL                                        10
#define RSSPECAN_VAL_DCI_FIELDS_CACP                                        11
#define RSSPECAN_VAL_DCI_FIELDS_CARR                                        12
#define RSSPECAN_VAL_DCI_FIELDS_DFIF                                        13
#define RSSPECAN_VAL_DCI_FIELDS_HAB                                         14
#define RSSPECAN_VAL_DCI_FIELDS_BPIN                                        15
#define RSSPECAN_VAL_DCI_FIELDS_DAI1                                        16
#define RSSPECAN_VAL_DCI_FIELDS_DAI2                                        17
#define RSSPECAN_VAL_DCI_FIELDS_SRIN                                        18
#define RSSPECAN_VAL_DCI_FIELDS_PINL                                        19
#define RSSPECAN_VAL_DCI_FIELDS_APOR                                        20
#define RSSPECAN_VAL_DCI_FIELDS_SRSR                                        21
#define RSSPECAN_VAL_DCI_FIELDS_CSIR                                        22
#define RSSPECAN_VAL_DCI_FIELDS_CBGT                                        23
#define RSSPECAN_VAL_DCI_FIELDS_PDAS                                        24
#define RSSPECAN_VAL_DCI_FIELDS_BOIN                                        25
#define RSSPECAN_VAL_DCI_FIELDS_DSIN                                        26
#define RSSPECAN_VAL_DCI_FIELDS_USCH                                        27
#define RSSPECAN_VAL_DCI_FIELDS_CAPC                                        28
#define RSSPECAN_VAL_DCI_FIELDS_OPCP                                        29
#define RSSPECAN_VAL_DCI_FIELDS_PIND                                        30
#define RSSPECAN_VAL_DCI_FIELDS_ISP                                         31
#define RSSPECAN_VAL_DCI_FIELDS_MAS                                         32
#define RSSPECAN_VAL_DCI_FIELDS_SDIN                                        33
#define RSSPECAN_VAL_DCI_FIELDS_RAP                                         34
#define RSSPECAN_VAL_DCI_FIELDS_SPIN                                        35
#define RSSPECAN_VAL_DCI_FIELDS_PMIN                                        36
#define RSSPECAN_VAL_DCI_FIELDS_RBIT                                        37
#define RSSPECAN_VAL_DCI_FIELDS_VTPM                                        38
#define RSSPECAN_VAL_DCI_FIELDS_PRIN                                        39
#define RSSPECAN_VAL_DCI_FIELDS_PHFT                                        40
#define RSSPECAN_VAL_DCI_FIELDS_SMIN                                        41
#define RSSPECAN_VAL_DCI_FIELDS_SMES                                        42
#define RSSPECAN_VAL_DCI_FIELDS_TBSC                                        43
#define RSSPECAN_VAL_DCI_FIELDS_SIIN                                        44
#define RSSPECAN_VAL_DCI_FIELDS_LOSF                                        45
#define RSSPECAN_VAL_DCI_FIELDS_PBS                                         46
#define RSSPECAN_VAL_DCI_FIELDS_RMIN                                        47
#define RSSPECAN_VAL_DCI_FIELDS_ZCRT                                        48
#define RSSPECAN_VAL_DCI_FIELDS_MSC2                                        49
#define RSSPECAN_VAL_DCI_FIELDS_NDI2                                        50
#define RSSPECAN_VAL_DCI_FIELDS_RV2                                         51
#define RSSPECAN_VAL_DCI_FIELDS_OHAR                                        52
#define RSSPECAN_VAL_DCI_FIELDS_PGIN                                        53
#define RSSPECAN_VAL_DCI_FIELDS_NFIN                                        54
#define RSSPECAN_VAL_DCI_FIELDS_NRPG                                        55
#define RSSPECAN_VAL_DCI_FIELDS_TCIN                                        56
#define RSSPECAN_VAL_DCI_FIELDS_CBGF                                        57
#define RSSPECAN_VAL_DCI_FIELDS_SFIN                                        58
#define RSSPECAN_VAL_DCI_FIELDS_ARBS                                        59
#define RSSPECAN_VAL_DCI_FIELDS_COTD                                        60
#define RSSPECAN_VAL_DCI_FIELDS_MGFL                                        61
#define RSSPECAN_VAL_DCI_FIELDS_PEIN                                        62
#define RSSPECAN_VAL_DCI_FIELDS_CLIN                                        63
#define RSSPECAN_VAL_DCI_FIELDS_TPCC                                        64
#define RSSPECAN_VAL_DCI_FIELDS_CIND                                        65
#define RSSPECAN_VAL_DCI_FIELDS_HIND                                        66
#define RSSPECAN_VAL_DCI_FIELDS_WUIN                                        67
#define RSSPECAN_VAL_DCI_FIELDS_N                                           68
#define RSSPECAN_VAL_DCI_FIELDS_N1                                          69
#define RSSPECAN_VAL_DCI_FIELDS_N2                                          70
#define RSSPECAN_VAL_DCI_FIELDS_M                                           71

// RsSpecAn_rngNrEventCondition
#define RSSPECAN_VAL_NR_EVENT_CONDITION_FAILED                              0
#define RSSPECAN_VAL_NR_EVENT_CONDITION_PASSED                              1
#define RSSPECAN_VAL_NR_EVENT_CONDITION_GTEQUAL                             2
#define RSSPECAN_VAL_NR_EVENT_CONDITION_LTEQUAL                             3

// RsSpecAn_rngSEListRangeInsertRange
#define RSSPECAN_VAL_INSERT_RANGE_LEFT                                      0
#define RSSPECAN_VAL_INSERT_RANGE_RIGHT                                     1

// RsSpecAn_rngSEListRangeSelectionForAll
#define RSSPECAN_VAL_SELECTED_PARAMETER_ARBW                                0
#define RSSPECAN_VAL_SELECTED_PARAMETER_LOFFSET                             1
#define RSSPECAN_VAL_SELECTED_PARAMETER_MFRBW                               2
#define RSSPECAN_VAL_SELECTED_PARAMETER_NFFT                                3
#define RSSPECAN_VAL_SELECTED_PARAMETER_PAValue                             4
#define RSSPECAN_VAL_SELECTED_PARAMETER_PEXCursion                          5
#define RSSPECAN_VAL_SELECTED_PARAMETER_RBW                                 6
#define RSSPECAN_VAL_SELECTED_PARAMETER_RFATtenuation                       7
#define RSSPECAN_VAL_SELECTED_PARAMETER_RLEVel                              8
#define RSSPECAN_VAL_SELECTED_PARAMETER_SNRatio                             9
#define RSSPECAN_VAL_SELECTED_PARAMETER_TSTR                                10
#define RSSPECAN_VAL_SELECTED_PARAMETER_TSTP                                11

// RsSpecAn_rngWLANEHTPPDUFormat
#define RSSPECAN_VAL_WLAN_EHT_PPDU_FORMAT_MU                                0

// RsSpecAn_rngAmplifierDefaultDetector
#define RSSPECAN_VAL_AMPLIFIER_DEFAULT_DETECTOR_OFF                         0
#define RSSPECAN_VAL_AMPLIFIER_DEFAULT_DETECTOR_AVERAGE                     1

// RsSpecAn_rngAmplifierSweepStatisticsMode
#define RSSPECAN_VAL_AMPLIFIER_SWEEP_STATISTICS_MODE_IQ_AVERAGING           0
#define RSSPECAN_VAL_AMPLIFIER_SWEEP_STATISTICS_MODE_TRACE                  1

// RsSpecAn_rngAmplifierCrestFactorReductionFilterMode
#define RSSPECAN_VAL_AMPLIFIER_CFR_FILTER_MODE_SIMPLE                       0
#define RSSPECAN_VAL_AMPLIFIER_CFR_FILTER_MODE_ENHANCED                     1

// RsSpecAn_rngAmplifierDDPDResultsType
#define RSSPECAN_VAL_AMPLIFIER_DDPD_RESULTS_TYPE_EVM                        0
#define RSSPECAN_VAL_AMPLIFIER_DDPD_RESULTS_TYPE_ACLR_LOWER_1               1
#define RSSPECAN_VAL_AMPLIFIER_DDPD_RESULTS_TYPE_ACLU_UPPER_1               2
#define RSSPECAN_VAL_AMPLIFIER_DDPD_RESULTS_TYPE_ACLR_BALANCED_1            3

// RsSpecAn_rngAmplifierEqualizerFilterFileFormat
#define RSSPECAN_VAL_AMPLIFIER_EQUALIZER_FILTER_FILE_FORMAT_CSV             0
#define RSSPECAN_VAL_AMPLIFIER_EQUALIZER_FILTER_FILE_FORMAT_FRES            1

// RsSpecAn_rngAmplifierMemoryPolynomialDPDWaveformType
#define RSSPECAN_VAL_AMPLIFIER_DPD_WAVEFORM_TYPE_DDPD                       0
#define RSSPECAN_VAL_AMPLIFIER_DPD_WAVEFORM_TYPE_MDPD                       1

// RsSpecAn_rngAmplifierTraceType
#define RSSPECAN_VAL_AMPLIFIER_TRACE_TYPE_BBI                               0
#define RSSPECAN_VAL_AMPLIFIER_TRACE_TYPE_BBPOWER                           1
#define RSSPECAN_VAL_AMPLIFIER_TRACE_TYPE_BBQ                               2
#define RSSPECAN_VAL_AMPLIFIER_TRACE_TYPE_RF                                3
#define RSSPECAN_VAL_AMPLIFIER_TRACE_TYPE_MEAS                              4
#define RSSPECAN_VAL_AMPLIFIER_TRACE_TYPE_MODEL                             5
#define RSSPECAN_VAL_AMPLIFIER_TRACE_TYPE_REFERENCE                         6

// RsSpecAn_rngWindowAxisAutoScaling
#define RSSPECAN_VAL_WINDOW_AXIS_AUTO_SCALING_OFF                           0
#define RSSPECAN_VAL_WINDOW_AXIS_AUTO_SCALING_ON                            2
#define RSSPECAN_VAL_WINDOW_AXIS_AUTO_SCALING_ONCE                          4

// RsSpecAn_rngHorizontalVertical
#define RSSPECAN_VAL_HORIZONTAL_VERTICAL_HORIZONTAL                         0
#define RSSPECAN_VAL_HORIZONTAL_VERTICAL_VERTICAL                           1

// rsspecan_rngAmplifierDetector
#define RSSPECAN_VAL_AMPLIFIER_DETECTOR_OFF                                 0
#define RSSPECAN_VAL_AMPLIFIER_DETECTOR_AVERAGE                             1
#define RSSPECAN_VAL_AMPLIFIER_DETECTOR_POSITIVE_PEAK                       2
#define RSSPECAN_VAL_AMPLIFIER_DETECTOR_NEGATIVE_PEAK                       3

// rsspecan_rngAmplifierTraceMode
#define RSSPECAN_AMPLIFIER_TRACE_MODE_WRITE                                 0
#define RSSPECAN_AMPLIFIER_TRACE_MODE_VIEW                                  1
#define RSSPECAN_AMPLIFIER_TRACE_MODE_AVER                                  2
#define RSSPECAN_AMPLIFIER_TRACE_MODE_MAXH                                  3
#define RSSPECAN_AMPLIFIER_TRACE_MODE_MINH                                  4
#define RSSPECAN_AMPLIFIER_TRACE_MODE_BLANK                                 5

// RsSpecAn_rngNPRMode
#define RSSPECAN_VAL_NPR_MODE_DENSITY                                       0
#define RSSPECAN_VAL_NPR_MODE_POWER                                         1

// RsSpecAn_rngHUMSDataFormat
#define RSSPECAN_VAL_NETWORK_HUMS_DATA_FORMAT_JSON                          0
#define RSSPECAN_VAL_NETWORK_HUMS_DATA_FORMAT_XML                           1

// RsSpecAn_rngFileColumnSeparator
#define RSSPECAN_VAL_FILE_COLUMN_SEPARATOR_COMMA                            0
#define RSSPECAN_VAL_FILE_COLUMN_SEPARATOR_SEMICOLON                        1
#define RSSPECAN_VAL_FILE_COLUMN_SEPARATOR_TAB                              2

// RsSpecAn_rngFileXValuesDistribution
#define RSSPECAN_VAL_FILE_X_VALUES_DISTRIBUTION_BINCENTERED                 0
#define RSSPECAN_VAL_FILE_X_VALUES_DISTRIBUTION_STARTSTOP                   1

// RsSpecAn_rngGPIBTerminator
#define RSSPECAN_VAL_GPIB_TERMINATOR_LFEOI                                  0
#define RSSPECAN_VAL_GPIB_TERMINATOR_EOI                                    1

// RsSpecAn_rngHUMSUserAccess
#define RSSPECAN_VAL_HUMS_SNMP_USER_ACCESS_RO                               0
#define RSSPECAN_VAL_HUMS_SNMP_USER_ACCESS_RW                               1

// RsSpecAn_rngHUMSSNMPAuthenticationLevel
#define RSSPECAN_VAL_HUMS_SNMP_USER_AUTHENTICATION_NOAUTH                   0
#define RSSPECAN_VAL_HUMS_SNMP_USER_AUTHENTICATION_AUTH                     1
#define RSSPECAN_VAL_HUMS_SNMP_USER_AUTHENTICATION_PRIVACY                  2

// RsSpecAn_rngIDQueryFormat
#define RSSPECAN_VAL_ID_QUERY_FORMAT_LEGACY                                 0
#define RSSPECAN_VAL_ID_QUERY_FORMAT_NEW                                    1
#define RSSPECAN_VAL_ID_QUERY_FORMAT_FSL                                    2

// RsSpecAn_rngApplicationGroup
#define RSSPECAN_VAL_APPLICATION_STARTER_GROUP_EXTERNAL                     0
#define RSSPECAN_VAL_APPLICATION_STARTER_GROUP_USER                         1

// RsSpecAn_rngHUMSSNMPVersion
#define RSSPECAN_VAL_HUMS_SNMP_VERSION_OFF                                  0
#define RSSPECAN_VAL_HUMS_SNMP_VERSION_V12                                  1
#define RSSPECAN_VAL_HUMS_SNMP_VERSION_V123                                 2
#define RSSPECAN_VAL_HUMS_SNMP_VERSION_V3                                   3
#define RSSPECAN_VAL_HUMS_SNMP_VERSION_DEFAULT                              4

// RsSpecAn_rngPowerUnit
#define RSSPECAN_VAL_POWER_UNIT_ABS                                         0
#define RSSPECAN_VAL_POWER_UNIT_PHZ                                         1
#define RSSPECAN_VAL_POWER_UNIT_MPHZ                                        2

// rsspecan_rngSweepAutoType
#define RSSPECAN_VAL_SWEEP_AUTO_TYPE_AUTO_SWEEP                             1
#define RSSPECAN_VAL_SWEEP_AUTO_TYPE_AUTO_FFT                               2

// RsSpecAn_rngMCGDConfigurationTransmissionType
#define RSSPECAN_VAL_MCGD_CONFIGURATION_TRANSMISSION_OWAY                   0
#define RSSPECAN_VAL_MCGD_CONFIGURATION_TRANSMISSION_TWAY                   1

// RsSpecAn_rngMCGDSubspanMode
#define RSSPECAN_VAL_MCGD_SUBSPAN_MODE_OFF                                  0
#define RSSPECAN_VAL_MCGD_SUBSPAN_MODE_ON                                   1
#define RSSPECAN_VAL_MCGD_SUBSPAN_MODE_AUTO                                 2

// RsSpecAn_rngAccess
#define RSSPECAN_VAL_ACCESS_READ_ONLY                                       0
#define RSSPECAN_VAL_ACCESS_READ_WRITE                                      1

// RsSpecAn_rng5GNREventFilterParamResult
#define RSSPECAN_VAL_5G_NR_RESULT_CREST                                     0
#define RSSPECAN_VAL_5G_NR_RESULT_DSQP                                      1
#define RSSPECAN_VAL_5G_NR_RESULT_DSSF                                      2
#define RSSPECAN_VAL_5G_NR_RESULT_DSST                                      3
#define RSSPECAN_VAL_5G_NR_RESULT_DSTS                                      4
#define RSSPECAN_VAL_5G_NR_RESULT_EVM                                       5
#define RSSPECAN_VAL_5G_NR_RESULT_P_CHANNEL                                 6
#define RSSPECAN_VAL_5G_NR_RESULT_P_SIGNAL                                  7
#define RSSPECAN_VAL_5G_NR_RESULT_FREQ_ERROR                                8
#define RSSPECAN_VAL_5G_NR_RESULT_SAMPLING_ERROR                            9
#define RSSPECAN_VAL_5G_NR_RESULT_QUADRATURE                                10
#define RSSPECAN_VAL_5G_NR_RESULT_GAIN_IMBALANCE                            11
#define RSSPECAN_VAL_5G_NR_RESULT_IQ_OFFSET                                 12
#define RSSPECAN_VAL_5G_NR_RESULT_POWER                                     13
#define RSSPECAN_VAL_5G_NR_RESULT_MODULATION                                14
#define RSSPECAN_VAL_5G_NR_RESULT_NORB                                      15
#define RSSPECAN_VAL_5G_NR_RESULT_OSTP                                      16
#define RSSPECAN_VAL_5G_NR_RESULT_RSTP                                      17
#define RSSPECAN_VAL_5G_NR_RESULT_SS_POWER                                  18
#define RSSPECAN_VAL_5G_NR_RESULT_CSI_POWER                                 19
#define RSSPECAN_VAL_5G_NR_RESULT_FSOF                                      20
#define RSSPECAN_VAL_5G_NR_RESULT_RSSI                                      21
#define RSSPECAN_VAL_5G_NR_RESULT_TST                                       22
#define RSSPECAN_VAL_5G_NR_RESULT_TSD                                       23
#define RSSPECAN_VAL_5G_NR_RESULT_MID                                       24
#define RSSPECAN_VAL_5G_NR_RESULT_OVLD                                      25
#define RSSPECAN_VAL_5G_NR_RESULT_SST                                       26
#define RSSPECAN_VAL_5G_NR_RESULT_APF                                       27
#define RSSPECAN_VAL_5G_NR_RESULT_AAPF                                      28
#define RSSPECAN_VAL_5G_NR_RESULT_ARPF                                      29
#define RSSPECAN_VAL_5G_NR_RESULT_SPF                                       30

// RsSpecAn_rng5GNRACLRIFF
#define RSSPECAN_VAL_5G_NR_IFF_NONE                                         0

// RsSpecAn_rngTestReportsPageFormat
#define RSSPECAN_VAL_PAGE_FORMAT_A4                                         0
#define RSSPECAN_VAL_PAGE_FORMAT_LETTER                                     1

// RsSpecAn_rngPrintMode
#define RSSPECAN_VAL_PRINT_MODE_SCREEN                                      0
#define RSSPECAN_VAL_PRINT_MODE_REPORT                                      1

// RsSpecAn_rngTestReportsVisibility
#define RSSPECAN_VAL_VISIBILITY_NEVER                                       0
#define RSSPECAN_VAL_VISIBILITY_GLOBAL                                      1
#define RSSPECAN_VAL_VISIBILITY_SUBREPORT                                   2

// RsSpecAn_rngTestReportsLogoVisibility
#define RSSPECAN_VAL_LOGO_VISIBILITY_NEVER                                  0
#define RSSPECAN_VAL_LOGO_VISIBILITY_GLOBAL                                 1
#define RSSPECAN_VAL_LOGO_VISIBILITY_ALWAYS                                 2
#define RSSPECAN_VAL_LOGO_VISIBILITY_ONCE                                   3

// RsSpecAn_rngMissingExternalReferenceBehavior
#define RSSPECAN_VAL_MISSING_EXT_REF_SHOW_ERROR_FLAG                        0
#define RSSPECAN_VAL_MISSING_EXT_REF_SWITCH_TO_INT_REF                      1

// RsSpecAn_rngPhaseSpuriousColor
#define RSSPECAN_VAL_PHASE_SPURIOUS_COLOR_TRACE                             0
#define RSSPECAN_VAL_PHASE_SPURIOUS_COLOR_SHIFTED                           1

// RsSpecAn_rngPulseTraceResults
#define RSSPECAN_VAL_PULSE_TRACE_RESULTS_SELECTED_PULSE                     0
#define RSSPECAN_VAL_PULSE_TRACE_RESULTS_CAPTURE_AVERAGE                    1
#define RSSPECAN_VAL_PULSE_TRACE_RESULTS_PULSE_PULSE_AVERAGE                2
#define RSSPECAN_VAL_PULSE_TRACE_RESULTS_POSITION_AVERAGE                   3
#define RSSPECAN_VAL_PULSE_TRACE_RESULTS_BURST_AVERAGE                      4
#define RSSPECAN_VAL_PULSE_TRACE_RESULTS_POSITION_POSITION_AVERAGE          5

// RsSpecAn_rngDigitizerLowNoiseMode
#define RSSPECAN_VAL_DIGITIZER_LOW_NOISE_MODE_ABSOLUTE                      0
#define RSSPECAN_VAL_DIGITIZER_LOW_NOISE_MODE_ADDITIVE                      1

// RsSpecAn_rngDigitizerSelection
#define RSSPECAN_VAL_DIGITIZER_SELECTION_WIDEBAND                           0
#define RSSPECAN_VAL_DIGITIZER_SELECTION_LOW_NOISE                          1

// RsSpecAn_rngPulseStabilityBurstMode
#define RSSPECAN_VAL_PULSE_STABILITY_BURST_MODE_SINGLE                      0
#define RSSPECAN_VAL_PULSE_STABILITY_BURST_MODE_MULTIPLE                    1

// RsSpecAn_rngPulseStabilityFreqCompensation
#define RSSPECAN_VAL_PULSE_STABILITY_FREQ_COMPENSATION_OFF                  0
#define RSSPECAN_VAL_PULSE_STABILITY_FREQ_COMPENSATION_CAPTURE              1

// RsSpecAn_rngPulseTraceStabilityEvaluation
#define RSSPECAN_VAL_PULSE_TRACE_STABILITY_EVALUATION_AMPLITUDE             0
#define RSSPECAN_VAL_PULSE_TRACE_STABILITY_EVALUATION_PHASE                 1
#define RSSPECAN_VAL_PULSE_TRACE_STABILITY_EVALUATION_TOTAL                 2

// RsSpecAn_rngPulseBurstTriggerPosition
#define RSSPECAN_VAL_PULSE_BURST_TRIGGER_POSITION_EACH_PULSE                0
#define RSSPECAN_VAL_PULSE_BURST_TRIGGER_POSITION_EACH_BURST                1
#define RSSPECAN_VAL_PULSE_BURST_TRIGGER_POSITION_SPECIFIC_BURST            2
#define RSSPECAN_VAL_PULSE_BURST_TRIGGER_POSITION_SEQUENCE                  3

// RsSpecAn_rngBTOSpuriousMeasurementGeography
#define RSSPECAN_VAL_BTO_SPURIOUS_MEASUREMENT_GEOGRAPHY_FRANCE              0
#define RSSPECAN_VAL_BTO_SPURIOUS_MEASUREMENT_GEOGRAPHY_OTHER               1

// RsSpecAn_rngBTOLowEnergyStandard
#define RSSPECAN_VAL_BTO_LOW_ENERGY_STANDARD_LE1M                           0
#define RSSPECAN_VAL_BTO_LOW_ENERGY_STANDARD_LE2M                           1
#define RSSPECAN_VAL_BTO_LOW_ENERGY_STANDARD_LECODED                        2

// RsSpecAn_rngBTOStandard
#define RSSPECAN_VAL_BTO_STANDARD_BR                                        0
#define RSSPECAN_VAL_BTO_STANDARD_EDR                                       1
#define RSSPECAN_VAL_BTO_STANDARD_LE                                        2

// RsSpecAn_rngAnalogDemodZeroPhaseRefPointMode
#define RSSPECAN_VAL_ANALOG_DEMOD_ZERO_PHASE_REF_POINT_MODE_MANUAL          0
#define RSSPECAN_VAL_ANALOG_DEMOD_ZERO_PHASE_REF_POINT_MODE_RIGHT           1

// RsSpecAn_rngAmplifierPowerServoingTargetParameter
#define RSSPECAN_VAL_AMPLIFIER_POWER_SERVOING_TARGET_PARAMETER_POUT         0
#define RSSPECAN_VAL_AMPLIFIER_POWER_SERVOING_TARGET_PARAMETER_EVM          1
#define RSSPECAN_VAL_AMPLIFIER_POWER_SERVOING_TARGET_PARAMETER_LADJ         2
#define RSSPECAN_VAL_AMPLIFIER_POWER_SERVOING_TARGET_PARAMETER_UADJ         3
#define RSSPECAN_VAL_AMPLIFIER_POWER_SERVOING_TARGET_PARAMETER_LALT         4
#define RSSPECAN_VAL_AMPLIFIER_POWER_SERVOING_TARGET_PARAMETER_UALT         5

// RsSpecAn_rngAmplifierSensePowerServoingGeneratorLevel
#define RSSPECAN_VAL_AMPLIFIER_SENSE_POWER_SERVOING_GENERATOR_LEVEL_RFL     0
#define RSSPECAN_VAL_AMPLIFIER_SENSE_POWER_SERVOING_GENERATOR_LEVEL_DATT    1

// RsSpecAn_rngGeneratorWaveform
#define RSSPECAN_VAL_GENERATOR_WAVEFORM_REFERENCE                           0
#define RSSPECAN_VAL_GENERATOR_WAVEFORM_DDPD                                1

// rsspecan_rngTransientResultScaling
#define RSSPECAN_VAL_TRANSIENT_RESULT_SCALING_S                             0
#define RSSPECAN_VAL_TRANSIENT_RESULT_SCALING_MS                            1
#define RSSPECAN_VAL_TRANSIENT_RESULT_SCALING_US                            2
#define RSSPECAN_VAL_TRANSIENT_RESULT_SCALING_NS                            3

// RsSpecAn_rngTransientTraceLongCaptureBuffer
#define RSSPECAN_VAL_TRANSIENT_TRACE_LONG_CAPTURE_BUFFER_AUTO               0
#define RSSPECAN_VAL_TRANSIENT_TRACE_LONG_CAPTURE_BUFFER_ON                 1
#define RSSPECAN_VAL_TRANSIENT_TRACE_LONG_CAPTURE_BUFFER_OFF                2

// RsSpecAn_rngFrequencyAnnotationMode
#define RSSPECAN_VAL_FREQUENCY_ANNOTATION_MODE_CENTER_SPAN                  0
#define RSSPECAN_VAL_FREQUENCY_ANNOTATION_MODE_START_STOP                   1

// RsSpecAn_rngDay
#define RSSPECAN_VAL_DAY_ALL                                                0
#define RSSPECAN_VAL_DAY_MONDAY                                             1
#define RSSPECAN_VAL_DAY_TUESDAY                                            2
#define RSSPECAN_VAL_DAY_WEDNESDAY                                          3
#define RSSPECAN_VAL_DAY_THURSDAY                                           4
#define RSSPECAN_VAL_DAY_FRIDAY                                             5
#define RSSPECAN_VAL_DAY_SATURDAY                                           6
#define RSSPECAN_VAL_DAY_SUNDAY                                             7

// RsSpecAn_rngDayOptional
#define RSSPECAN_VAL_DAY_NAN                                                0

// RsSpecAn_rngFloatingLicenseState
#define RSSPECAN_VAL_FLOATING_LICENSE_ON                                    0
#define RSSPECAN_VAL_FLOATING_LICENSE_OFF                                   1
#define RSSPECAN_VAL_FLOATING_LICENSE_OCCUPY                                2

// RsSpecAn_rngSCPIRecorderFormat
#define RSSPECAN_VAL_SCPI_RECORDER_FORMAT_CPLUSPLUS                         0
#define RSSPECAN_VAL_SCPI_RECORDER_FORMAT_MATLAB                            1
#define RSSPECAN_VAL_SCPI_RECORDER_FORMAT_CVI                               2
#define RSSPECAN_VAL_SCPI_RECORDER_FORMAT_SCPI                              3
#define RSSPECAN_VAL_SCPI_RECORDER_FORMAT_PYTHON                            4

// RsSpecAn_rngHRPUWBMagnitudeCaptureState
#define RSSPECAN_VAL_HRPUWB_MAGNITUDE_CAPTURE_STATE_SELECTED                0
#define RSSPECAN_VAL_HRPUWB_MAGNITUDE_CAPTURE_STATE_OK                      1

// RsSpecAn_rngHRPUWBResultDisplayConfig
#define RSSPECAN_VAL_HRPUWB_RESULT_DISPLAY_CONFIG_DEFAULT                   0
#define RSSPECAN_VAL_HRPUWB_RESULT_DISPLAY_CONFIG_USER                      1

// RsSpecAn_rngHRPUWBDemodulationMACFCS
#define RSSPECAN_VAL_HRP_UWB_DEMODULATION_MAC_FCS_OFF                       0
#define RSSPECAN_VAL_HRP_UWB_DEMODULATION_MAC_FCS_O2                        1
#define RSSPECAN_VAL_HRP_UWB_DEMODULATION_MAC_FCS_O4                        2

// RsSpecAn_rngEVMAveragingMethod
#define RSSPECAN_VAL_EVM_AVERAGING_METHOD_MSQ                               0
#define RSSPECAN_VAL_EVM_AVERAGING_METHOD_RMS                               1

// RsSpecAn_rngEVMNormalizationMethod
#define RSSPECAN_VAL_EVM_NORMALIZATION_METHOD_NONE                          0
#define RSSPECAN_VAL_EVM_NORMALIZATION_METHOD_PEAK_DATA                     1
#define RSSPECAN_VAL_EVM_NORMALIZATION_METHOD_PEAK_PILOTS_DATA              2
#define RSSPECAN_VAL_EVM_NORMALIZATION_METHOD_PEAK_PILOTS                   3
#define RSSPECAN_VAL_EVM_NORMALIZATION_METHOD_RMS_DATA                      4
#define RSSPECAN_VAL_EVM_NORMALIZATION_METHOD_RMS_PILOTS_DATA               5
#define RSSPECAN_VAL_EVM_NORMALIZATION_METHOD_RMS_PILOTS                    6

// RsSpecAn_rngOFDMVSASynchronizationParameterEstimation
#define RSSPECAN_VAL_OFDM_VSA_SYNCHRONIZATION_PARAMETER_ESTIMATION_DATA     0
#define RSSPECAN_VAL_OFDM_VSA_SYNCHRONIZATION_PARAMETER_ESTIMATION_PILOT    1
#define RSSPECAN_VAL_OFDM_VSA_SYNCHRONIZATION_PARAMETER_ESTIMATION_NONE     2

// RsSpecAn_rngOFDMVSASynchronizationModulationDetection
#define RSSPECAN_VAL_OFDM_VSA_SYNCHRONIZATION_MODULATION_DETECTION_CARRIER  0
#define RSSPECAN_VAL_OFDM_VSA_SYNCHRONIZATION_MODULATION_DETECTION_CONFIG_FILE 1
#define RSSPECAN_VAL_OFDM_VSA_SYNCHRONIZATION_MODULATION_DETECTION_SYMBOL   2

// RsSpecAn_rngOFDMVSASynchronizationTimeMode
#define RSSPECAN_VAL_OFDM_VSA_SYNCHRONIZATION_TIME_MODE_CYCLIC_PREFIX       0
#define RSSPECAN_VAL_OFDM_VSA_SYNCHRONIZATION_TIME_MODE_PREAMBLE            1

// RsSpecAn_rngBitstreamDisplayFormat
#define RSSPECAN_VAL_BITSTREAM_DISPLAY_FORMAT_BINARY                        0
#define RSSPECAN_VAL_BITSTREAM_DISPLAY_FORMAT_OCTAL                         1
#define RSSPECAN_VAL_BITSTREAM_DISPLAY_FORMAT_DECIMAL                       2
#define RSSPECAN_VAL_BITSTREAM_DISPLAY_FORMAT_HEXADECIMAL                   3

// RsSpecAn_rngConstellationDiagramModulationType
#define RSSPECAN_VAL_CONSTELLATION_DIAGRAM_MODULATION_TYPE_PILOT_AND_DATA   0
#define RSSPECAN_VAL_CONSTELLATION_DIAGRAM_MODULATION_TYPE_PILOTS           1
#define RSSPECAN_VAL_CONSTELLATION_DIAGRAM_MODULATION_TYPE_DATA             2

// RsSpecAn_rngOFDMVSAPhaseCompensationMode
#define RSSPECAN_VAL_OFDM_VSA_PHASE_COMPENSATION_MODE_CF                    0
#define RSSPECAN_VAL_OFDM_VSA_PHASE_COMPENSATION_MODE_MANUAL                1

// RsSpecAn_rngOFDMVSACyclicPrefixMode
#define RSSPECAN_VAL_OFDM_VSA_CYCLIC_PREFIX_MODE_CONVENTIONAL               0
#define RSSPECAN_VAL_OFDM_VSA_CYCLIC_PREFIX_MODE_GU2                        1

// RsSpecAn_rngOFDMVSAUnitImpulseResponse
#define RSSPECAN_VAL_OFDM_VSA_UNIT_IMPULSE_RESPONSE_DB                      0
#define RSSPECAN_VAL_OFDM_VSA_UNIT_IMPULSE_RESPONSE_LIN                     1

// RsSpecAn_rngOFDMVSAUnitsMagnitudeCapture
#define RSSPECAN_VAL_OFDM_VSA_UNITS_MAGNITUDE_CAPTURE_SECOND                0
#define RSSPECAN_VAL_OFDM_VSA_UNITS_MAGNITUDE_CAPTURE_SAMPLE                1
#define RSSPECAN_VAL_OFDM_VSA_UNITS_MAGNITUDE_CAPTURE_SYMBOL                2

// RsSpecAn_rngOFDMVSAUnitsCarrier
#define RSSPECAN_VAL_OFDM_VSA_UNITS_CARRIER                                 0
#define RSSPECAN_VAL_OFDM_VSA_UNITS_HZ                                      1

// RsSpecAn_rngAnalogDemodTraceMode
#define RSSPECAN_VAL_ANALOG_DEMOD_TRACE_MODE_WRITE                          1
#define RSSPECAN_VAL_ANALOG_DEMOD_TRACE_MODE_AVERAGE                        2
#define RSSPECAN_VAL_ANALOG_DEMOD_TRACE_MODE_MAXHOLD                        3
#define RSSPECAN_VAL_ANALOG_DEMOD_TRACE_MODE_MINHOLD                        4

// RsSpecAn_rngOFDMVSASignalFlowState
#define RSSPECAN_VAL_OFDMVSA_SIGNAL_FLOW_STATE_FAILED                       0
#define RSSPECAN_VAL_OFDMVSA_SIGNAL_FLOW_STATE_PASSED                       1
#define RSSPECAN_VAL_OFDMVSA_SIGNAL_FLOW_STATE_DISABLED                     2
#define RSSPECAN_VAL_OFDMVSA_SIGNAL_FLOW_STATE_WARNING                      3

// RsSpecAn_rngWLANAdjustReferenceLevelDurationMode
#define RSSPECAN_VAL_WLAN_ADJUST_REFERENCE_LEVEL_DURATION_MODE_AUTO         0
#define RSSPECAN_VAL_WLAN_ADJUST_REFERENCE_LEVEL_DURATION_MODE_MANUAL       1

// RsSpecAn_rngWLANDemodulationDataMode
#define RSSPECAN_VAL_WLAN_DEMODULATION_DATA_MODE_ACDS                       0
#define RSSPECAN_VAL_WLAN_DEMODULATION_DATA_MODE_ALBD                       1

// RsSpecAn_rngWLANTrackingWienerRelativeDelaySpreadEnabled
#define RSSPECAN_VAL_WLAN_TRACKING_WIENER_RELATIVE_DELAY_SPREAD_ENABLED_OFF 0
#define RSSPECAN_VAL_WLAN_TRACKING_WIENER_RELATIVE_DELAY_SPREAD_ENABLED_MANUAL 1

// RsSpecAn_rngWLANPowerOptimisation
#define RSSPECAN_VAL_WLAN_POWER_OPTIMISATION_OFF                            0
#define RSSPECAN_VAL_WLAN_POWER_OPTIMISATION_PAON                           1
#define RSSPECAN_VAL_WLAN_POWER_OPTIMISATION_FULL                           2

// RsSpecAn_rngLTEDownlinkCarrierType
#define RSSPECAN_VAL_LTE_DOWNLINK_CARRIER_TYPE_ANCHOR                       0
#define RSSPECAN_VAL_LTE_DOWNLINK_CARRIER_TYPE_NANCHOR                      1

// RsSpecAn_rng5GNRSweepMode
#define RSSPECAN_VAL_NR_5G_SWEEP_MODE_AUTO                                  0
#define RSSPECAN_VAL_NR_5G_SWEEP_MODE_TX                                    1

// RsSpecAn_rng5GNRULPUCCHDMRSOCCLength
#define RSSPECAN_VAL_NR_5G_ULPUCCHDMRSOCC_LENGTH_N2                         0
#define RSSPECAN_VAL_NR_5G_ULPUCCHDMRSOCC_LENGTH_N4                         1

// RsSpecAn_rng5GNRULPRACHLRA
#define RSSPECAN_VAL_NR5_GULPRACHLRA_L139                                   0
#define RSSPECAN_VAL_NR5_GULPRACHLRA_L571                                   1
#define RSSPECAN_VAL_NR5_GULPRACHLRA_L839                                   2
#define RSSPECAN_VAL_NR5_GULPRACHLRA_L1151                                  3

// RsSpecAn_rng5GNRULConstellationColors
#define RSSPECAN_VAL_NR5_GUL_CONSTELLATION_COLORS_ALL                       0
#define RSSPECAN_VAL_NR5_GUL_CONSTELLATION_COLORS_MOD                       1

// RsSpecAn_rng5GNRULConfigurationFilterType
#define RSSPECAN_VAL_NR_5G_UL_CONFIGURATION_FILTER_TYPE_MFIL                0
#define RSSPECAN_VAL_NR_5G_UL_CONFIGURATION_FILTER_TYPE_NONE                1
#define RSSPECAN_VAL_NR_5G_UL_CONFIGURATION_FILTER_TYPE_PBWP                2

// RsSpecAn_rng5GNRULPUSCHReferenceData
#define RSSPECAN_VAL_NR_5G_UL_PUSCH_REFERENCE_DATA_AUTO                     0
#define RSSPECAN_VAL_NR_5G_UL_PUSCH_REFERENCE_DATA_PASL                     1
#define RSSPECAN_VAL_NR_5G_UL_PUSCH_REFERENCE_DATA_PN23                     2

// RsSpecAn_rng5GNRULPowerAveragingMode
#define RSSPECAN_VAL_NR_5G_UL_POWER_AVERAGING_MODE_AASL                     0
#define RSSPECAN_VAL_NR_5G_UL_POWER_AVERAGING_MODE_AASY                     1

// RsSpecAn_rng5GNRTestID
#define RSSPECAN_VAL_NR_5G_TEST_ID_NONE                                     0
#define RSSPECAN_VAL_NR_5G_TEST_ID_ID1                                      1
#define RSSPECAN_VAL_NR_5G_TEST_ID_ID2                                      2
#define RSSPECAN_VAL_NR_5G_TEST_ID_ID3                                      3
#define RSSPECAN_VAL_NR_5G_TEST_ID_ID4                                      4
#define RSSPECAN_VAL_NR_5G_TEST_ID_ID5                                      5
#define RSSPECAN_VAL_NR_5G_TEST_ID_ID6                                      6
#define RSSPECAN_VAL_NR_5G_TEST_ID_ID7                                      7
#define RSSPECAN_VAL_NR_5G_TEST_ID_ID8                                      8
#define RSSPECAN_VAL_NR_5G_TEST_ID_ID9                                      9
#define RSSPECAN_VAL_NR_5G_TEST_ID_ID10                                     10
#define RSSPECAN_VAL_NR_5G_TEST_ID_ID11                                     11
#define RSSPECAN_VAL_NR_5G_TEST_ID_ID12                                     12
#define RSSPECAN_VAL_NR_5G_TEST_ID_ID13                                     13
#define RSSPECAN_VAL_NR_5G_TEST_ID_ID14                                     14
#define RSSPECAN_VAL_NR_5G_TEST_ID_ID15                                     15

// RsSpecAn_rngSEMatchingCondition
#define RSSPECAN_VAL_SE_MATCHING_CONDITION_DMIN                             0
#define RSSPECAN_VAL_SE_MATCHING_CONDITION_PMAX                             1

// RsSpecAn_rngOFDMVSAUnitsSymbols
#define RSSPECAN_VAL_OFDMVSA_UNITS_SYMBOLS_SYM                              0
#define RSSPECAN_VAL_OFDMVSA_UNITS_SYMBOLS_SEC                              1

// RsSpecAn_rngWLANPPDUFormatToMeasure80211be
#define RSSPECAN_VAL_WLANPPDU_FORMAT_TO_MEASURE_80211BE_AIET                0
#define RSSPECAN_VAL_WLANPPDU_FORMAT_TO_MEASURE_80211BE_DEHT                1
#define RSSPECAN_VAL_WLANPPDU_FORMAT_TO_MEASURE_80211BE_FBUR                2
#define RSSPECAN_VAL_WLANPPDU_FORMAT_TO_MEASURE_80211BE_ALL                 3
#define RSSPECAN_VAL_WLANPPDU_FORMAT_TO_MEASURE_80211BE_DEHP                4
#define RSSPECAN_VAL_WLANPPDU_FORMAT_TO_MEASURE_80211BE_AIEM                5

// rsspecan_rngWLANChannelBandwidth802_11be
#define RSSPECAN_VAL_WLAN_CHANNEL_BANDWIDTH_802_11BE_DB20                   0
#define RSSPECAN_VAL_WLAN_CHANNEL_BANDWIDTH_802_11BE_DB40                   1
#define RSSPECAN_VAL_WLAN_CHANNEL_BANDWIDTH_802_11BE_DB80                   2
#define RSSPECAN_VAL_WLAN_CHANNEL_BANDWIDTH_802_11BE_DB160                  3
#define RSSPECAN_VAL_WLAN_CHANNEL_BANDWIDTH_802_11BE_DB320                  4
#define RSSPECAN_VAL_WLAN_CHANNEL_BANDWIDTH_802_11BE_MB20                   5
#define RSSPECAN_VAL_WLAN_CHANNEL_BANDWIDTH_802_11BE_FBURS                  6

// RsSpecAn_rng5GNRULDemodulationFilter
#define RSSPECAN_VAL_NR_5G_UL_DEMODULATION_FILTER_MFIL                      0
#define RSSPECAN_VAL_NR_5G_UL_DEMODULATION_FILTER_NONE                      1
#define RSSPECAN_VAL_NR_5G_UL_DEMODULATION_FILTER_PBWP                      2

//repcap Marker
#define RSSPECAN_REPCAP_MARKER_M1                                           1
#define RSSPECAN_REPCAP_MARKER_M2                                           2
#define RSSPECAN_REPCAP_MARKER_M3                                           3
#define RSSPECAN_REPCAP_MARKER_M4                                           4
#define RSSPECAN_REPCAP_MARKER_M5                                           5
#define RSSPECAN_REPCAP_MARKER_M6                                           6
#define RSSPECAN_REPCAP_MARKER_M7                                           7
#define RSSPECAN_REPCAP_MARKER_M8                                           8
#define RSSPECAN_REPCAP_MARKER_M9                                           9
#define RSSPECAN_REPCAP_MARKER_M10                                          10
#define RSSPECAN_REPCAP_MARKER_M11                                          11
#define RSSPECAN_REPCAP_MARKER_M12                                          12
#define RSSPECAN_REPCAP_MARKER_M13                                          13
#define RSSPECAN_REPCAP_MARKER_M14                                          14
#define RSSPECAN_REPCAP_MARKER_M15                                          15
#define RSSPECAN_REPCAP_MARKER_M16                                          16

//repcap Window
#define RSSPECAN_REPCAP_WINDOW_WIN0                                         0
#define RSSPECAN_REPCAP_WINDOW_WIN1                                         1
#define RSSPECAN_REPCAP_WINDOW_WIN2                                         2
#define RSSPECAN_REPCAP_WINDOW_WIN3                                         3
#define RSSPECAN_REPCAP_WINDOW_WIN4                                         4
#define RSSPECAN_REPCAP_WINDOW_WIN5                                         5
#define RSSPECAN_REPCAP_WINDOW_WIN6                                         6
#define RSSPECAN_REPCAP_WINDOW_WIN7                                         7
#define RSSPECAN_REPCAP_WINDOW_WIN8                                         8
#define RSSPECAN_REPCAP_WINDOW_WIN9                                         9
#define RSSPECAN_REPCAP_WINDOW_WIN10                                        10
#define RSSPECAN_REPCAP_WINDOW_WIN11                                        11
#define RSSPECAN_REPCAP_WINDOW_WIN12                                        12
#define RSSPECAN_REPCAP_WINDOW_WIN13                                        13
#define RSSPECAN_REPCAP_WINDOW_WIN14                                        14
#define RSSPECAN_REPCAP_WINDOW_WIN15                                        15
#define RSSPECAN_REPCAP_WINDOW_WIN16                                        16

//repcap Destination
#define RSSPECAN_REPCAP_DESTINATION_DE1                                     1
#define RSSPECAN_REPCAP_DESTINATION_DE2                                     2

//repcap CMAP
#define RSSPECAN_REPCAP_CMAP_CM1                                            1
#define RSSPECAN_REPCAP_CMAP_CM2                                            2
#define RSSPECAN_REPCAP_CMAP_CM3                                            3
#define RSSPECAN_REPCAP_CMAP_CM4                                            4
#define RSSPECAN_REPCAP_CMAP_CM5                                            5
#define RSSPECAN_REPCAP_CMAP_CM6                                            6
#define RSSPECAN_REPCAP_CMAP_CM7                                            7
#define RSSPECAN_REPCAP_CMAP_CM8                                            8
#define RSSPECAN_REPCAP_CMAP_CM9                                            9
#define RSSPECAN_REPCAP_CMAP_CM10                                           10
#define RSSPECAN_REPCAP_CMAP_CM11                                           11
#define RSSPECAN_REPCAP_CMAP_CM12                                           12
#define RSSPECAN_REPCAP_CMAP_CM13                                           13
#define RSSPECAN_REPCAP_CMAP_CM14                                           14
#define RSSPECAN_REPCAP_CMAP_CM15                                           15
#define RSSPECAN_REPCAP_CMAP_CM16                                           16
#define RSSPECAN_REPCAP_CMAP_CM17                                           17
#define RSSPECAN_REPCAP_CMAP_CM18                                           18
#define RSSPECAN_REPCAP_CMAP_CM19                                           19
#define RSSPECAN_REPCAP_CMAP_CM20                                           20
#define RSSPECAN_REPCAP_CMAP_CM21                                           21
#define RSSPECAN_REPCAP_CMAP_CM22                                           22
#define RSSPECAN_REPCAP_CMAP_CM23                                           23
#define RSSPECAN_REPCAP_CMAP_CM24                                           24
#define RSSPECAN_REPCAP_CMAP_CM25                                           25
#define RSSPECAN_REPCAP_CMAP_CM26                                           26
#define RSSPECAN_REPCAP_CMAP_CM27                                           27
#define RSSPECAN_REPCAP_CMAP_CM28                                           28
#define RSSPECAN_REPCAP_CMAP_CM29                                           29
#define RSSPECAN_REPCAP_CMAP_CM30                                           30
#define RSSPECAN_REPCAP_CMAP_CM31                                           31
#define RSSPECAN_REPCAP_CMAP_CM32                                           32
#define RSSPECAN_REPCAP_CMAP_CM33                                           33
#define RSSPECAN_REPCAP_CMAP_CM34                                           34
#define RSSPECAN_REPCAP_CMAP_CM35                                           35
#define RSSPECAN_REPCAP_CMAP_CM36                                           36
#define RSSPECAN_REPCAP_CMAP_CM37                                           37
#define RSSPECAN_REPCAP_CMAP_CM38                                           38
#define RSSPECAN_REPCAP_CMAP_CM39                                           39
#define RSSPECAN_REPCAP_CMAP_CM40                                           40
#define RSSPECAN_REPCAP_CMAP_CM41                                           41

//repcap Limit
#define RSSPECAN_REPCAP_LIMIT_L1                                            1
#define RSSPECAN_REPCAP_LIMIT_L2                                            2
#define RSSPECAN_REPCAP_LIMIT_L3                                            3
#define RSSPECAN_REPCAP_LIMIT_L4                                            4
#define RSSPECAN_REPCAP_LIMIT_L5                                            5
#define RSSPECAN_REPCAP_LIMIT_L6                                            6
#define RSSPECAN_REPCAP_LIMIT_L7                                            7
#define RSSPECAN_REPCAP_LIMIT_L8                                            8

//repcap Range
#define RSSPECAN_REPCAP_RANGE_R1                                            1
#define RSSPECAN_REPCAP_RANGE_R2                                            2
#define RSSPECAN_REPCAP_RANGE_R3                                            3
#define RSSPECAN_REPCAP_RANGE_R4                                            4
#define RSSPECAN_REPCAP_RANGE_R5                                            5
#define RSSPECAN_REPCAP_RANGE_R6                                            6
#define RSSPECAN_REPCAP_RANGE_R7                                            7
#define RSSPECAN_REPCAP_RANGE_R8                                            8
#define RSSPECAN_REPCAP_RANGE_R9                                            9
#define RSSPECAN_REPCAP_RANGE_R10                                           10
#define RSSPECAN_REPCAP_RANGE_R11                                           11
#define RSSPECAN_REPCAP_RANGE_R12                                           12
#define RSSPECAN_REPCAP_RANGE_R13                                           13
#define RSSPECAN_REPCAP_RANGE_R14                                           14
#define RSSPECAN_REPCAP_RANGE_R15                                           15
#define RSSPECAN_REPCAP_RANGE_R16                                           16
#define RSSPECAN_REPCAP_RANGE_R17                                           17
#define RSSPECAN_REPCAP_RANGE_R18                                           18
#define RSSPECAN_REPCAP_RANGE_R19                                           19
#define RSSPECAN_REPCAP_RANGE_R20                                           20
#define RSSPECAN_REPCAP_RANGE_R21                                           21
#define RSSPECAN_REPCAP_RANGE_R22                                           22
#define RSSPECAN_REPCAP_RANGE_R23                                           23
#define RSSPECAN_REPCAP_RANGE_R24                                           24
#define RSSPECAN_REPCAP_RANGE_R25                                           25
#define RSSPECAN_REPCAP_RANGE_R26                                           26
#define RSSPECAN_REPCAP_RANGE_R27                                           27
#define RSSPECAN_REPCAP_RANGE_R28                                           28
#define RSSPECAN_REPCAP_RANGE_R29                                           29
#define RSSPECAN_REPCAP_RANGE_R30                                           30
#define RSSPECAN_REPCAP_RANGE_R31                                           31
#define RSSPECAN_REPCAP_RANGE_R32                                           32
#define RSSPECAN_REPCAP_RANGE_R33                                           33
#define RSSPECAN_REPCAP_RANGE_R34                                           34
#define RSSPECAN_REPCAP_RANGE_R35                                           35
#define RSSPECAN_REPCAP_RANGE_R36                                           36
#define RSSPECAN_REPCAP_RANGE_R37                                           37
#define RSSPECAN_REPCAP_RANGE_R38                                           38
#define RSSPECAN_REPCAP_RANGE_R39                                           39
#define RSSPECAN_REPCAP_RANGE_R40                                           40
#define RSSPECAN_REPCAP_RANGE_R41                                           41
#define RSSPECAN_REPCAP_RANGE_R42                                           42
#define RSSPECAN_REPCAP_RANGE_R43                                           43
#define RSSPECAN_REPCAP_RANGE_R44                                           44
#define RSSPECAN_REPCAP_RANGE_R45                                           45
#define RSSPECAN_REPCAP_RANGE_R46                                           46
#define RSSPECAN_REPCAP_RANGE_R47                                           47
#define RSSPECAN_REPCAP_RANGE_R48                                           48
#define RSSPECAN_REPCAP_RANGE_R49                                           49
#define RSSPECAN_REPCAP_RANGE_R50                                           50
#define RSSPECAN_REPCAP_RANGE_R51                                           51
#define RSSPECAN_REPCAP_RANGE_R52                                           52
#define RSSPECAN_REPCAP_RANGE_R53                                           53
#define RSSPECAN_REPCAP_RANGE_R54                                           54
#define RSSPECAN_REPCAP_RANGE_R55                                           55
#define RSSPECAN_REPCAP_RANGE_R56                                           56
#define RSSPECAN_REPCAP_RANGE_R57                                           57
#define RSSPECAN_REPCAP_RANGE_R58                                           58
#define RSSPECAN_REPCAP_RANGE_R59                                           59
#define RSSPECAN_REPCAP_RANGE_R60                                           60
#define RSSPECAN_REPCAP_RANGE_R61                                           61
#define RSSPECAN_REPCAP_RANGE_R62                                           62
#define RSSPECAN_REPCAP_RANGE_R63                                           63
#define RSSPECAN_REPCAP_RANGE_R64                                           64
#define RSSPECAN_REPCAP_RANGE_R65                                           65
#define RSSPECAN_REPCAP_RANGE_R66                                           66
#define RSSPECAN_REPCAP_RANGE_R67                                           67
#define RSSPECAN_REPCAP_RANGE_R68                                           68
#define RSSPECAN_REPCAP_RANGE_R69                                           69
#define RSSPECAN_REPCAP_RANGE_R70                                           70
#define RSSPECAN_REPCAP_RANGE_R71                                           71
#define RSSPECAN_REPCAP_RANGE_R72                                           72
#define RSSPECAN_REPCAP_RANGE_R73                                           73
#define RSSPECAN_REPCAP_RANGE_R74                                           74
#define RSSPECAN_REPCAP_RANGE_R75                                           75
#define RSSPECAN_REPCAP_RANGE_R76                                           76
#define RSSPECAN_REPCAP_RANGE_R77                                           77
#define RSSPECAN_REPCAP_RANGE_R78                                           78
#define RSSPECAN_REPCAP_RANGE_R79                                           79
#define RSSPECAN_REPCAP_RANGE_R80                                           80
#define RSSPECAN_REPCAP_RANGE_R81                                           81
#define RSSPECAN_REPCAP_RANGE_R82                                           82
#define RSSPECAN_REPCAP_RANGE_R83                                           83
#define RSSPECAN_REPCAP_RANGE_R84                                           84
#define RSSPECAN_REPCAP_RANGE_R85                                           85
#define RSSPECAN_REPCAP_RANGE_R86                                           86
#define RSSPECAN_REPCAP_RANGE_R87                                           87
#define RSSPECAN_REPCAP_RANGE_R88                                           88
#define RSSPECAN_REPCAP_RANGE_R89                                           89
#define RSSPECAN_REPCAP_RANGE_R90                                           90
#define RSSPECAN_REPCAP_RANGE_R91                                           91
#define RSSPECAN_REPCAP_RANGE_R92                                           92
#define RSSPECAN_REPCAP_RANGE_R93                                           93
#define RSSPECAN_REPCAP_RANGE_R94                                           94
#define RSSPECAN_REPCAP_RANGE_R95                                           95
#define RSSPECAN_REPCAP_RANGE_R96                                           96
#define RSSPECAN_REPCAP_RANGE_R97                                           97
#define RSSPECAN_REPCAP_RANGE_R98                                           98
#define RSSPECAN_REPCAP_RANGE_R99                                           99
#define RSSPECAN_REPCAP_RANGE_R100                                          100
#define RSSPECAN_REPCAP_RANGE_R101                                          101
#define RSSPECAN_REPCAP_RANGE_R102                                          102
#define RSSPECAN_REPCAP_RANGE_R103                                          103
#define RSSPECAN_REPCAP_RANGE_R104                                          104
#define RSSPECAN_REPCAP_RANGE_R105                                          105
#define RSSPECAN_REPCAP_RANGE_R106                                          106
#define RSSPECAN_REPCAP_RANGE_R107                                          107
#define RSSPECAN_REPCAP_RANGE_R108                                          108
#define RSSPECAN_REPCAP_RANGE_R109                                          109
#define RSSPECAN_REPCAP_RANGE_R110                                          110
#define RSSPECAN_REPCAP_RANGE_R111                                          111
#define RSSPECAN_REPCAP_RANGE_R112                                          112
#define RSSPECAN_REPCAP_RANGE_R113                                          113
#define RSSPECAN_REPCAP_RANGE_R114                                          114
#define RSSPECAN_REPCAP_RANGE_R115                                          115
#define RSSPECAN_REPCAP_RANGE_R116                                          116
#define RSSPECAN_REPCAP_RANGE_R117                                          117
#define RSSPECAN_REPCAP_RANGE_R118                                          118
#define RSSPECAN_REPCAP_RANGE_R119                                          119
#define RSSPECAN_REPCAP_RANGE_R120                                          120
#define RSSPECAN_REPCAP_RANGE_R121                                          121
#define RSSPECAN_REPCAP_RANGE_R122                                          122
#define RSSPECAN_REPCAP_RANGE_R123                                          123
#define RSSPECAN_REPCAP_RANGE_R124                                          124
#define RSSPECAN_REPCAP_RANGE_R125                                          125
#define RSSPECAN_REPCAP_RANGE_R126                                          126
#define RSSPECAN_REPCAP_RANGE_R127                                          127
#define RSSPECAN_REPCAP_RANGE_R128                                          128
#define RSSPECAN_REPCAP_RANGE_R129                                          129
#define RSSPECAN_REPCAP_RANGE_R130                                          130
#define RSSPECAN_REPCAP_RANGE_R131                                          131
#define RSSPECAN_REPCAP_RANGE_R132                                          132
#define RSSPECAN_REPCAP_RANGE_R133                                          133
#define RSSPECAN_REPCAP_RANGE_R134                                          134
#define RSSPECAN_REPCAP_RANGE_R135                                          135
#define RSSPECAN_REPCAP_RANGE_R136                                          136
#define RSSPECAN_REPCAP_RANGE_R137                                          137
#define RSSPECAN_REPCAP_RANGE_R138                                          138
#define RSSPECAN_REPCAP_RANGE_R139                                          139
#define RSSPECAN_REPCAP_RANGE_R140                                          140
#define RSSPECAN_REPCAP_RANGE_R141                                          141
#define RSSPECAN_REPCAP_RANGE_R142                                          142
#define RSSPECAN_REPCAP_RANGE_R143                                          143
#define RSSPECAN_REPCAP_RANGE_R144                                          144
#define RSSPECAN_REPCAP_RANGE_R145                                          145
#define RSSPECAN_REPCAP_RANGE_R146                                          146
#define RSSPECAN_REPCAP_RANGE_R147                                          147
#define RSSPECAN_REPCAP_RANGE_R148                                          148
#define RSSPECAN_REPCAP_RANGE_R149                                          149
#define RSSPECAN_REPCAP_RANGE_R150                                          150
#define RSSPECAN_REPCAP_RANGE_R151                                          151
#define RSSPECAN_REPCAP_RANGE_R152                                          152
#define RSSPECAN_REPCAP_RANGE_R153                                          153
#define RSSPECAN_REPCAP_RANGE_R154                                          154
#define RSSPECAN_REPCAP_RANGE_R155                                          155
#define RSSPECAN_REPCAP_RANGE_R156                                          156
#define RSSPECAN_REPCAP_RANGE_R157                                          157
#define RSSPECAN_REPCAP_RANGE_R158                                          158
#define RSSPECAN_REPCAP_RANGE_R159                                          159
#define RSSPECAN_REPCAP_RANGE_R160                                          160
#define RSSPECAN_REPCAP_RANGE_R161                                          161
#define RSSPECAN_REPCAP_RANGE_R162                                          162
#define RSSPECAN_REPCAP_RANGE_R163                                          163
#define RSSPECAN_REPCAP_RANGE_R164                                          164
#define RSSPECAN_REPCAP_RANGE_R165                                          165
#define RSSPECAN_REPCAP_RANGE_R166                                          166
#define RSSPECAN_REPCAP_RANGE_R167                                          167
#define RSSPECAN_REPCAP_RANGE_R168                                          168
#define RSSPECAN_REPCAP_RANGE_R169                                          169
#define RSSPECAN_REPCAP_RANGE_R170                                          170
#define RSSPECAN_REPCAP_RANGE_R171                                          171
#define RSSPECAN_REPCAP_RANGE_R172                                          172
#define RSSPECAN_REPCAP_RANGE_R173                                          173
#define RSSPECAN_REPCAP_RANGE_R174                                          174
#define RSSPECAN_REPCAP_RANGE_R175                                          175
#define RSSPECAN_REPCAP_RANGE_R176                                          176
#define RSSPECAN_REPCAP_RANGE_R177                                          177
#define RSSPECAN_REPCAP_RANGE_R178                                          178
#define RSSPECAN_REPCAP_RANGE_R179                                          179
#define RSSPECAN_REPCAP_RANGE_R180                                          180
#define RSSPECAN_REPCAP_RANGE_R181                                          181
#define RSSPECAN_REPCAP_RANGE_R182                                          182
#define RSSPECAN_REPCAP_RANGE_R183                                          183
#define RSSPECAN_REPCAP_RANGE_R184                                          184
#define RSSPECAN_REPCAP_RANGE_R185                                          185
#define RSSPECAN_REPCAP_RANGE_R186                                          186
#define RSSPECAN_REPCAP_RANGE_R187                                          187
#define RSSPECAN_REPCAP_RANGE_R188                                          188
#define RSSPECAN_REPCAP_RANGE_R189                                          189
#define RSSPECAN_REPCAP_RANGE_R190                                          190
#define RSSPECAN_REPCAP_RANGE_R191                                          191
#define RSSPECAN_REPCAP_RANGE_R192                                          192
#define RSSPECAN_REPCAP_RANGE_R193                                          193
#define RSSPECAN_REPCAP_RANGE_R194                                          194
#define RSSPECAN_REPCAP_RANGE_R195                                          195
#define RSSPECAN_REPCAP_RANGE_R196                                          196
#define RSSPECAN_REPCAP_RANGE_R197                                          197
#define RSSPECAN_REPCAP_RANGE_R198                                          198
#define RSSPECAN_REPCAP_RANGE_R199                                          199
#define RSSPECAN_REPCAP_RANGE_R200                                          200
#define RSSPECAN_REPCAP_RANGE_R201                                          201
#define RSSPECAN_REPCAP_RANGE_R202                                          202
#define RSSPECAN_REPCAP_RANGE_R203                                          203
#define RSSPECAN_REPCAP_RANGE_R204                                          204
#define RSSPECAN_REPCAP_RANGE_R205                                          205
#define RSSPECAN_REPCAP_RANGE_R206                                          206
#define RSSPECAN_REPCAP_RANGE_R207                                          207
#define RSSPECAN_REPCAP_RANGE_R208                                          208
#define RSSPECAN_REPCAP_RANGE_R209                                          209
#define RSSPECAN_REPCAP_RANGE_R210                                          210
#define RSSPECAN_REPCAP_RANGE_R211                                          211
#define RSSPECAN_REPCAP_RANGE_R212                                          212
#define RSSPECAN_REPCAP_RANGE_R213                                          213
#define RSSPECAN_REPCAP_RANGE_R214                                          214
#define RSSPECAN_REPCAP_RANGE_R215                                          215
#define RSSPECAN_REPCAP_RANGE_R216                                          216
#define RSSPECAN_REPCAP_RANGE_R217                                          217
#define RSSPECAN_REPCAP_RANGE_R218                                          218
#define RSSPECAN_REPCAP_RANGE_R219                                          219
#define RSSPECAN_REPCAP_RANGE_R220                                          220
#define RSSPECAN_REPCAP_RANGE_R221                                          221
#define RSSPECAN_REPCAP_RANGE_R222                                          222
#define RSSPECAN_REPCAP_RANGE_R223                                          223
#define RSSPECAN_REPCAP_RANGE_R224                                          224
#define RSSPECAN_REPCAP_RANGE_R225                                          225
#define RSSPECAN_REPCAP_RANGE_R226                                          226
#define RSSPECAN_REPCAP_RANGE_R227                                          227
#define RSSPECAN_REPCAP_RANGE_R228                                          228
#define RSSPECAN_REPCAP_RANGE_R229                                          229
#define RSSPECAN_REPCAP_RANGE_R230                                          230
#define RSSPECAN_REPCAP_RANGE_R231                                          231
#define RSSPECAN_REPCAP_RANGE_R232                                          232
#define RSSPECAN_REPCAP_RANGE_R233                                          233
#define RSSPECAN_REPCAP_RANGE_R234                                          234
#define RSSPECAN_REPCAP_RANGE_R235                                          235
#define RSSPECAN_REPCAP_RANGE_R236                                          236
#define RSSPECAN_REPCAP_RANGE_R237                                          237
#define RSSPECAN_REPCAP_RANGE_R238                                          238
#define RSSPECAN_REPCAP_RANGE_R239                                          239
#define RSSPECAN_REPCAP_RANGE_R240                                          240
#define RSSPECAN_REPCAP_RANGE_R241                                          241
#define RSSPECAN_REPCAP_RANGE_R242                                          242
#define RSSPECAN_REPCAP_RANGE_R243                                          243
#define RSSPECAN_REPCAP_RANGE_R244                                          244
#define RSSPECAN_REPCAP_RANGE_R245                                          245
#define RSSPECAN_REPCAP_RANGE_R246                                          246
#define RSSPECAN_REPCAP_RANGE_R247                                          247
#define RSSPECAN_REPCAP_RANGE_R248                                          248
#define RSSPECAN_REPCAP_RANGE_R249                                          249
#define RSSPECAN_REPCAP_RANGE_R250                                          250
#define RSSPECAN_REPCAP_RANGE_R251                                          251
#define RSSPECAN_REPCAP_RANGE_R252                                          252
#define RSSPECAN_REPCAP_RANGE_R253                                          253
#define RSSPECAN_REPCAP_RANGE_R254                                          254
#define RSSPECAN_REPCAP_RANGE_R255                                          255
#define RSSPECAN_REPCAP_RANGE_R256                                          256
#define RSSPECAN_REPCAP_RANGE_R257                                          257
#define RSSPECAN_REPCAP_RANGE_R258                                          258
#define RSSPECAN_REPCAP_RANGE_R259                                          259
#define RSSPECAN_REPCAP_RANGE_R260                                          260
#define RSSPECAN_REPCAP_RANGE_R261                                          261
#define RSSPECAN_REPCAP_RANGE_R262                                          262
#define RSSPECAN_REPCAP_RANGE_R263                                          263
#define RSSPECAN_REPCAP_RANGE_R264                                          264
#define RSSPECAN_REPCAP_RANGE_R265                                          265
#define RSSPECAN_REPCAP_RANGE_R266                                          266
#define RSSPECAN_REPCAP_RANGE_R267                                          267
#define RSSPECAN_REPCAP_RANGE_R268                                          268
#define RSSPECAN_REPCAP_RANGE_R269                                          269
#define RSSPECAN_REPCAP_RANGE_R270                                          270
#define RSSPECAN_REPCAP_RANGE_R271                                          271
#define RSSPECAN_REPCAP_RANGE_R272                                          272
#define RSSPECAN_REPCAP_RANGE_R273                                          273
#define RSSPECAN_REPCAP_RANGE_R274                                          274
#define RSSPECAN_REPCAP_RANGE_R275                                          275
#define RSSPECAN_REPCAP_RANGE_R276                                          276
#define RSSPECAN_REPCAP_RANGE_R277                                          277
#define RSSPECAN_REPCAP_RANGE_R278                                          278
#define RSSPECAN_REPCAP_RANGE_R279                                          279
#define RSSPECAN_REPCAP_RANGE_R280                                          280
#define RSSPECAN_REPCAP_RANGE_R281                                          281
#define RSSPECAN_REPCAP_RANGE_R282                                          282
#define RSSPECAN_REPCAP_RANGE_R283                                          283
#define RSSPECAN_REPCAP_RANGE_R284                                          284
#define RSSPECAN_REPCAP_RANGE_R285                                          285
#define RSSPECAN_REPCAP_RANGE_R286                                          286
#define RSSPECAN_REPCAP_RANGE_R287                                          287
#define RSSPECAN_REPCAP_RANGE_R288                                          288
#define RSSPECAN_REPCAP_RANGE_R289                                          289
#define RSSPECAN_REPCAP_RANGE_R290                                          290
#define RSSPECAN_REPCAP_RANGE_R291                                          291
#define RSSPECAN_REPCAP_RANGE_R292                                          292
#define RSSPECAN_REPCAP_RANGE_R293                                          293
#define RSSPECAN_REPCAP_RANGE_R294                                          294
#define RSSPECAN_REPCAP_RANGE_R295                                          295
#define RSSPECAN_REPCAP_RANGE_R296                                          296
#define RSSPECAN_REPCAP_RANGE_R297                                          297
#define RSSPECAN_REPCAP_RANGE_R298                                          298
#define RSSPECAN_REPCAP_RANGE_R299                                          299
#define RSSPECAN_REPCAP_RANGE_R300                                          300
#define RSSPECAN_REPCAP_RANGE_R301                                          301
#define RSSPECAN_REPCAP_RANGE_R302                                          302
#define RSSPECAN_REPCAP_RANGE_R303                                          303
#define RSSPECAN_REPCAP_RANGE_R304                                          304
#define RSSPECAN_REPCAP_RANGE_R305                                          305
#define RSSPECAN_REPCAP_RANGE_R306                                          306
#define RSSPECAN_REPCAP_RANGE_R307                                          307
#define RSSPECAN_REPCAP_RANGE_R308                                          308
#define RSSPECAN_REPCAP_RANGE_R309                                          309
#define RSSPECAN_REPCAP_RANGE_R310                                          310
#define RSSPECAN_REPCAP_RANGE_R311                                          311
#define RSSPECAN_REPCAP_RANGE_R312                                          312
#define RSSPECAN_REPCAP_RANGE_R313                                          313
#define RSSPECAN_REPCAP_RANGE_R314                                          314
#define RSSPECAN_REPCAP_RANGE_R315                                          315
#define RSSPECAN_REPCAP_RANGE_R316                                          316
#define RSSPECAN_REPCAP_RANGE_R317                                          317
#define RSSPECAN_REPCAP_RANGE_R318                                          318
#define RSSPECAN_REPCAP_RANGE_R319                                          319
#define RSSPECAN_REPCAP_RANGE_R320                                          320
#define RSSPECAN_REPCAP_RANGE_R321                                          321
#define RSSPECAN_REPCAP_RANGE_R322                                          322
#define RSSPECAN_REPCAP_RANGE_R323                                          323
#define RSSPECAN_REPCAP_RANGE_R324                                          324
#define RSSPECAN_REPCAP_RANGE_R325                                          325
#define RSSPECAN_REPCAP_RANGE_R326                                          326
#define RSSPECAN_REPCAP_RANGE_R327                                          327
#define RSSPECAN_REPCAP_RANGE_R328                                          328
#define RSSPECAN_REPCAP_RANGE_R329                                          329
#define RSSPECAN_REPCAP_RANGE_R330                                          330
#define RSSPECAN_REPCAP_RANGE_R331                                          331
#define RSSPECAN_REPCAP_RANGE_R332                                          332
#define RSSPECAN_REPCAP_RANGE_R333                                          333
#define RSSPECAN_REPCAP_RANGE_R334                                          334
#define RSSPECAN_REPCAP_RANGE_R335                                          335
#define RSSPECAN_REPCAP_RANGE_R336                                          336
#define RSSPECAN_REPCAP_RANGE_R337                                          337
#define RSSPECAN_REPCAP_RANGE_R338                                          338
#define RSSPECAN_REPCAP_RANGE_R339                                          339
#define RSSPECAN_REPCAP_RANGE_R340                                          340
#define RSSPECAN_REPCAP_RANGE_R341                                          341
#define RSSPECAN_REPCAP_RANGE_R342                                          342
#define RSSPECAN_REPCAP_RANGE_R343                                          343
#define RSSPECAN_REPCAP_RANGE_R344                                          344
#define RSSPECAN_REPCAP_RANGE_R345                                          345
#define RSSPECAN_REPCAP_RANGE_R346                                          346
#define RSSPECAN_REPCAP_RANGE_R347                                          347
#define RSSPECAN_REPCAP_RANGE_R348                                          348
#define RSSPECAN_REPCAP_RANGE_R349                                          349
#define RSSPECAN_REPCAP_RANGE_R350                                          350
#define RSSPECAN_REPCAP_RANGE_R351                                          351
#define RSSPECAN_REPCAP_RANGE_R352                                          352
#define RSSPECAN_REPCAP_RANGE_R353                                          353
#define RSSPECAN_REPCAP_RANGE_R354                                          354
#define RSSPECAN_REPCAP_RANGE_R355                                          355
#define RSSPECAN_REPCAP_RANGE_R356                                          356
#define RSSPECAN_REPCAP_RANGE_R357                                          357
#define RSSPECAN_REPCAP_RANGE_R358                                          358
#define RSSPECAN_REPCAP_RANGE_R359                                          359
#define RSSPECAN_REPCAP_RANGE_R360                                          360
#define RSSPECAN_REPCAP_RANGE_R361                                          361
#define RSSPECAN_REPCAP_RANGE_R362                                          362
#define RSSPECAN_REPCAP_RANGE_R363                                          363
#define RSSPECAN_REPCAP_RANGE_R364                                          364
#define RSSPECAN_REPCAP_RANGE_R365                                          365
#define RSSPECAN_REPCAP_RANGE_R366                                          366
#define RSSPECAN_REPCAP_RANGE_R367                                          367
#define RSSPECAN_REPCAP_RANGE_R368                                          368
#define RSSPECAN_REPCAP_RANGE_R369                                          369
#define RSSPECAN_REPCAP_RANGE_R370                                          370
#define RSSPECAN_REPCAP_RANGE_R371                                          371
#define RSSPECAN_REPCAP_RANGE_R372                                          372
#define RSSPECAN_REPCAP_RANGE_R373                                          373
#define RSSPECAN_REPCAP_RANGE_R374                                          374
#define RSSPECAN_REPCAP_RANGE_R375                                          375
#define RSSPECAN_REPCAP_RANGE_R376                                          376
#define RSSPECAN_REPCAP_RANGE_R377                                          377
#define RSSPECAN_REPCAP_RANGE_R378                                          378
#define RSSPECAN_REPCAP_RANGE_R379                                          379
#define RSSPECAN_REPCAP_RANGE_R380                                          380
#define RSSPECAN_REPCAP_RANGE_R381                                          381
#define RSSPECAN_REPCAP_RANGE_R382                                          382
#define RSSPECAN_REPCAP_RANGE_R383                                          383
#define RSSPECAN_REPCAP_RANGE_R384                                          384
#define RSSPECAN_REPCAP_RANGE_R385                                          385
#define RSSPECAN_REPCAP_RANGE_R386                                          386
#define RSSPECAN_REPCAP_RANGE_R387                                          387
#define RSSPECAN_REPCAP_RANGE_R388                                          388
#define RSSPECAN_REPCAP_RANGE_R389                                          389
#define RSSPECAN_REPCAP_RANGE_R390                                          390
#define RSSPECAN_REPCAP_RANGE_R391                                          391
#define RSSPECAN_REPCAP_RANGE_R392                                          392
#define RSSPECAN_REPCAP_RANGE_R393                                          393
#define RSSPECAN_REPCAP_RANGE_R394                                          394
#define RSSPECAN_REPCAP_RANGE_R395                                          395
#define RSSPECAN_REPCAP_RANGE_R396                                          396
#define RSSPECAN_REPCAP_RANGE_R397                                          397
#define RSSPECAN_REPCAP_RANGE_R398                                          398
#define RSSPECAN_REPCAP_RANGE_R399                                          399
#define RSSPECAN_REPCAP_RANGE_R400                                          400
#define RSSPECAN_REPCAP_RANGE_R401                                          401
#define RSSPECAN_REPCAP_RANGE_R402                                          402
#define RSSPECAN_REPCAP_RANGE_R403                                          403
#define RSSPECAN_REPCAP_RANGE_R404                                          404
#define RSSPECAN_REPCAP_RANGE_R405                                          405
#define RSSPECAN_REPCAP_RANGE_R406                                          406
#define RSSPECAN_REPCAP_RANGE_R407                                          407
#define RSSPECAN_REPCAP_RANGE_R408                                          408
#define RSSPECAN_REPCAP_RANGE_R409                                          409
#define RSSPECAN_REPCAP_RANGE_R410                                          410
#define RSSPECAN_REPCAP_RANGE_R411                                          411
#define RSSPECAN_REPCAP_RANGE_R412                                          412
#define RSSPECAN_REPCAP_RANGE_R413                                          413
#define RSSPECAN_REPCAP_RANGE_R414                                          414
#define RSSPECAN_REPCAP_RANGE_R415                                          415
#define RSSPECAN_REPCAP_RANGE_R416                                          416
#define RSSPECAN_REPCAP_RANGE_R417                                          417
#define RSSPECAN_REPCAP_RANGE_R418                                          418
#define RSSPECAN_REPCAP_RANGE_R419                                          419
#define RSSPECAN_REPCAP_RANGE_R420                                          420
#define RSSPECAN_REPCAP_RANGE_R421                                          421
#define RSSPECAN_REPCAP_RANGE_R422                                          422
#define RSSPECAN_REPCAP_RANGE_R423                                          423
#define RSSPECAN_REPCAP_RANGE_R424                                          424
#define RSSPECAN_REPCAP_RANGE_R425                                          425
#define RSSPECAN_REPCAP_RANGE_R426                                          426
#define RSSPECAN_REPCAP_RANGE_R427                                          427
#define RSSPECAN_REPCAP_RANGE_R428                                          428
#define RSSPECAN_REPCAP_RANGE_R429                                          429
#define RSSPECAN_REPCAP_RANGE_R430                                          430
#define RSSPECAN_REPCAP_RANGE_R431                                          431
#define RSSPECAN_REPCAP_RANGE_R432                                          432
#define RSSPECAN_REPCAP_RANGE_R433                                          433
#define RSSPECAN_REPCAP_RANGE_R434                                          434
#define RSSPECAN_REPCAP_RANGE_R435                                          435
#define RSSPECAN_REPCAP_RANGE_R436                                          436
#define RSSPECAN_REPCAP_RANGE_R437                                          437
#define RSSPECAN_REPCAP_RANGE_R438                                          438
#define RSSPECAN_REPCAP_RANGE_R439                                          439
#define RSSPECAN_REPCAP_RANGE_R440                                          440
#define RSSPECAN_REPCAP_RANGE_R441                                          441
#define RSSPECAN_REPCAP_RANGE_R442                                          442
#define RSSPECAN_REPCAP_RANGE_R443                                          443
#define RSSPECAN_REPCAP_RANGE_R444                                          444
#define RSSPECAN_REPCAP_RANGE_R445                                          445
#define RSSPECAN_REPCAP_RANGE_R446                                          446
#define RSSPECAN_REPCAP_RANGE_R447                                          447
#define RSSPECAN_REPCAP_RANGE_R448                                          448
#define RSSPECAN_REPCAP_RANGE_R449                                          449
#define RSSPECAN_REPCAP_RANGE_R450                                          450
#define RSSPECAN_REPCAP_RANGE_R451                                          451
#define RSSPECAN_REPCAP_RANGE_R452                                          452
#define RSSPECAN_REPCAP_RANGE_R453                                          453
#define RSSPECAN_REPCAP_RANGE_R454                                          454
#define RSSPECAN_REPCAP_RANGE_R455                                          455
#define RSSPECAN_REPCAP_RANGE_R456                                          456
#define RSSPECAN_REPCAP_RANGE_R457                                          457
#define RSSPECAN_REPCAP_RANGE_R458                                          458
#define RSSPECAN_REPCAP_RANGE_R459                                          459
#define RSSPECAN_REPCAP_RANGE_R460                                          460
#define RSSPECAN_REPCAP_RANGE_R461                                          461
#define RSSPECAN_REPCAP_RANGE_R462                                          462
#define RSSPECAN_REPCAP_RANGE_R463                                          463
#define RSSPECAN_REPCAP_RANGE_R464                                          464
#define RSSPECAN_REPCAP_RANGE_R465                                          465
#define RSSPECAN_REPCAP_RANGE_R466                                          466
#define RSSPECAN_REPCAP_RANGE_R467                                          467
#define RSSPECAN_REPCAP_RANGE_R468                                          468
#define RSSPECAN_REPCAP_RANGE_R469                                          469
#define RSSPECAN_REPCAP_RANGE_R470                                          470
#define RSSPECAN_REPCAP_RANGE_R471                                          471
#define RSSPECAN_REPCAP_RANGE_R472                                          472
#define RSSPECAN_REPCAP_RANGE_R473                                          473
#define RSSPECAN_REPCAP_RANGE_R474                                          474
#define RSSPECAN_REPCAP_RANGE_R475                                          475
#define RSSPECAN_REPCAP_RANGE_R476                                          476
#define RSSPECAN_REPCAP_RANGE_R477                                          477
#define RSSPECAN_REPCAP_RANGE_R478                                          478
#define RSSPECAN_REPCAP_RANGE_R479                                          479
#define RSSPECAN_REPCAP_RANGE_R480                                          480
#define RSSPECAN_REPCAP_RANGE_R481                                          481
#define RSSPECAN_REPCAP_RANGE_R482                                          482
#define RSSPECAN_REPCAP_RANGE_R483                                          483
#define RSSPECAN_REPCAP_RANGE_R484                                          484
#define RSSPECAN_REPCAP_RANGE_R485                                          485
#define RSSPECAN_REPCAP_RANGE_R486                                          486
#define RSSPECAN_REPCAP_RANGE_R487                                          487
#define RSSPECAN_REPCAP_RANGE_R488                                          488
#define RSSPECAN_REPCAP_RANGE_R489                                          489
#define RSSPECAN_REPCAP_RANGE_R490                                          490
#define RSSPECAN_REPCAP_RANGE_R491                                          491
#define RSSPECAN_REPCAP_RANGE_R492                                          492
#define RSSPECAN_REPCAP_RANGE_R493                                          493
#define RSSPECAN_REPCAP_RANGE_R494                                          494
#define RSSPECAN_REPCAP_RANGE_R495                                          495
#define RSSPECAN_REPCAP_RANGE_R496                                          496
#define RSSPECAN_REPCAP_RANGE_R497                                          497
#define RSSPECAN_REPCAP_RANGE_R498                                          498
#define RSSPECAN_REPCAP_RANGE_R499                                          499
#define RSSPECAN_REPCAP_RANGE_R500                                          500
#define RSSPECAN_REPCAP_RANGE_R501                                          501
#define RSSPECAN_REPCAP_RANGE_R502                                          502
#define RSSPECAN_REPCAP_RANGE_R503                                          503
#define RSSPECAN_REPCAP_RANGE_R504                                          504
#define RSSPECAN_REPCAP_RANGE_R505                                          505
#define RSSPECAN_REPCAP_RANGE_R506                                          506
#define RSSPECAN_REPCAP_RANGE_R507                                          507
#define RSSPECAN_REPCAP_RANGE_R508                                          508
#define RSSPECAN_REPCAP_RANGE_R509                                          509
#define RSSPECAN_REPCAP_RANGE_R510                                          510
#define RSSPECAN_REPCAP_RANGE_R511                                          511
#define RSSPECAN_REPCAP_RANGE_R512                                          512
#define RSSPECAN_REPCAP_RANGE_R513                                          513
#define RSSPECAN_REPCAP_RANGE_R514                                          514
#define RSSPECAN_REPCAP_RANGE_R515                                          515
#define RSSPECAN_REPCAP_RANGE_R516                                          516
#define RSSPECAN_REPCAP_RANGE_R517                                          517
#define RSSPECAN_REPCAP_RANGE_R518                                          518
#define RSSPECAN_REPCAP_RANGE_R519                                          519
#define RSSPECAN_REPCAP_RANGE_R520                                          520
#define RSSPECAN_REPCAP_RANGE_R521                                          521
#define RSSPECAN_REPCAP_RANGE_R522                                          522
#define RSSPECAN_REPCAP_RANGE_R523                                          523
#define RSSPECAN_REPCAP_RANGE_R524                                          524
#define RSSPECAN_REPCAP_RANGE_R525                                          525
#define RSSPECAN_REPCAP_RANGE_R526                                          526
#define RSSPECAN_REPCAP_RANGE_R527                                          527
#define RSSPECAN_REPCAP_RANGE_R528                                          528
#define RSSPECAN_REPCAP_RANGE_R529                                          529
#define RSSPECAN_REPCAP_RANGE_R530                                          530
#define RSSPECAN_REPCAP_RANGE_R531                                          531
#define RSSPECAN_REPCAP_RANGE_R532                                          532
#define RSSPECAN_REPCAP_RANGE_R533                                          533
#define RSSPECAN_REPCAP_RANGE_R534                                          534
#define RSSPECAN_REPCAP_RANGE_R535                                          535
#define RSSPECAN_REPCAP_RANGE_R536                                          536
#define RSSPECAN_REPCAP_RANGE_R537                                          537
#define RSSPECAN_REPCAP_RANGE_R538                                          538
#define RSSPECAN_REPCAP_RANGE_R539                                          539
#define RSSPECAN_REPCAP_RANGE_R540                                          540
#define RSSPECAN_REPCAP_RANGE_R541                                          541
#define RSSPECAN_REPCAP_RANGE_R542                                          542
#define RSSPECAN_REPCAP_RANGE_R543                                          543
#define RSSPECAN_REPCAP_RANGE_R544                                          544
#define RSSPECAN_REPCAP_RANGE_R545                                          545
#define RSSPECAN_REPCAP_RANGE_R546                                          546
#define RSSPECAN_REPCAP_RANGE_R547                                          547
#define RSSPECAN_REPCAP_RANGE_R548                                          548
#define RSSPECAN_REPCAP_RANGE_R549                                          549
#define RSSPECAN_REPCAP_RANGE_R550                                          550
#define RSSPECAN_REPCAP_RANGE_R551                                          551
#define RSSPECAN_REPCAP_RANGE_R552                                          552
#define RSSPECAN_REPCAP_RANGE_R553                                          553
#define RSSPECAN_REPCAP_RANGE_R554                                          554
#define RSSPECAN_REPCAP_RANGE_R555                                          555
#define RSSPECAN_REPCAP_RANGE_R556                                          556
#define RSSPECAN_REPCAP_RANGE_R557                                          557
#define RSSPECAN_REPCAP_RANGE_R558                                          558
#define RSSPECAN_REPCAP_RANGE_R559                                          559
#define RSSPECAN_REPCAP_RANGE_R560                                          560
#define RSSPECAN_REPCAP_RANGE_R561                                          561
#define RSSPECAN_REPCAP_RANGE_R562                                          562
#define RSSPECAN_REPCAP_RANGE_R563                                          563
#define RSSPECAN_REPCAP_RANGE_R564                                          564
#define RSSPECAN_REPCAP_RANGE_R565                                          565
#define RSSPECAN_REPCAP_RANGE_R566                                          566
#define RSSPECAN_REPCAP_RANGE_R567                                          567
#define RSSPECAN_REPCAP_RANGE_R568                                          568
#define RSSPECAN_REPCAP_RANGE_R569                                          569
#define RSSPECAN_REPCAP_RANGE_R570                                          570
#define RSSPECAN_REPCAP_RANGE_R571                                          571
#define RSSPECAN_REPCAP_RANGE_R572                                          572
#define RSSPECAN_REPCAP_RANGE_R573                                          573
#define RSSPECAN_REPCAP_RANGE_R574                                          574
#define RSSPECAN_REPCAP_RANGE_R575                                          575
#define RSSPECAN_REPCAP_RANGE_R576                                          576
#define RSSPECAN_REPCAP_RANGE_R577                                          577
#define RSSPECAN_REPCAP_RANGE_R578                                          578
#define RSSPECAN_REPCAP_RANGE_R579                                          579
#define RSSPECAN_REPCAP_RANGE_R580                                          580
#define RSSPECAN_REPCAP_RANGE_R581                                          581
#define RSSPECAN_REPCAP_RANGE_R582                                          582
#define RSSPECAN_REPCAP_RANGE_R583                                          583
#define RSSPECAN_REPCAP_RANGE_R584                                          584
#define RSSPECAN_REPCAP_RANGE_R585                                          585
#define RSSPECAN_REPCAP_RANGE_R586                                          586
#define RSSPECAN_REPCAP_RANGE_R587                                          587
#define RSSPECAN_REPCAP_RANGE_R588                                          588
#define RSSPECAN_REPCAP_RANGE_R589                                          589
#define RSSPECAN_REPCAP_RANGE_R590                                          590
#define RSSPECAN_REPCAP_RANGE_R591                                          591
#define RSSPECAN_REPCAP_RANGE_R592                                          592
#define RSSPECAN_REPCAP_RANGE_R593                                          593
#define RSSPECAN_REPCAP_RANGE_R594                                          594
#define RSSPECAN_REPCAP_RANGE_R595                                          595
#define RSSPECAN_REPCAP_RANGE_R596                                          596
#define RSSPECAN_REPCAP_RANGE_R597                                          597
#define RSSPECAN_REPCAP_RANGE_R598                                          598
#define RSSPECAN_REPCAP_RANGE_R599                                          599
#define RSSPECAN_REPCAP_RANGE_R600                                          600
#define RSSPECAN_REPCAP_RANGE_R601                                          601
#define RSSPECAN_REPCAP_RANGE_R602                                          602
#define RSSPECAN_REPCAP_RANGE_R603                                          603
#define RSSPECAN_REPCAP_RANGE_R604                                          604
#define RSSPECAN_REPCAP_RANGE_R605                                          605
#define RSSPECAN_REPCAP_RANGE_R606                                          606
#define RSSPECAN_REPCAP_RANGE_R607                                          607
#define RSSPECAN_REPCAP_RANGE_R608                                          608
#define RSSPECAN_REPCAP_RANGE_R609                                          609
#define RSSPECAN_REPCAP_RANGE_R610                                          610
#define RSSPECAN_REPCAP_RANGE_R611                                          611
#define RSSPECAN_REPCAP_RANGE_R612                                          612
#define RSSPECAN_REPCAP_RANGE_R613                                          613
#define RSSPECAN_REPCAP_RANGE_R614                                          614
#define RSSPECAN_REPCAP_RANGE_R615                                          615
#define RSSPECAN_REPCAP_RANGE_R616                                          616
#define RSSPECAN_REPCAP_RANGE_R617                                          617
#define RSSPECAN_REPCAP_RANGE_R618                                          618
#define RSSPECAN_REPCAP_RANGE_R619                                          619
#define RSSPECAN_REPCAP_RANGE_R620                                          620
#define RSSPECAN_REPCAP_RANGE_R621                                          621
#define RSSPECAN_REPCAP_RANGE_R622                                          622
#define RSSPECAN_REPCAP_RANGE_R623                                          623
#define RSSPECAN_REPCAP_RANGE_R624                                          624
#define RSSPECAN_REPCAP_RANGE_R625                                          625
#define RSSPECAN_REPCAP_RANGE_R626                                          626
#define RSSPECAN_REPCAP_RANGE_R627                                          627
#define RSSPECAN_REPCAP_RANGE_R628                                          628
#define RSSPECAN_REPCAP_RANGE_R629                                          629
#define RSSPECAN_REPCAP_RANGE_R630                                          630
#define RSSPECAN_REPCAP_RANGE_R631                                          631
#define RSSPECAN_REPCAP_RANGE_R632                                          632
#define RSSPECAN_REPCAP_RANGE_R633                                          633
#define RSSPECAN_REPCAP_RANGE_R634                                          634
#define RSSPECAN_REPCAP_RANGE_R635                                          635
#define RSSPECAN_REPCAP_RANGE_R636                                          636
#define RSSPECAN_REPCAP_RANGE_R637                                          637
#define RSSPECAN_REPCAP_RANGE_R638                                          638
#define RSSPECAN_REPCAP_RANGE_R639                                          639
#define RSSPECAN_REPCAP_RANGE_R640                                          640
#define RSSPECAN_REPCAP_RANGE_R641                                          641
#define RSSPECAN_REPCAP_RANGE_R642                                          642
#define RSSPECAN_REPCAP_RANGE_R643                                          643
#define RSSPECAN_REPCAP_RANGE_R644                                          644
#define RSSPECAN_REPCAP_RANGE_R645                                          645
#define RSSPECAN_REPCAP_RANGE_R646                                          646
#define RSSPECAN_REPCAP_RANGE_R647                                          647
#define RSSPECAN_REPCAP_RANGE_R648                                          648
#define RSSPECAN_REPCAP_RANGE_R649                                          649
#define RSSPECAN_REPCAP_RANGE_R650                                          650
#define RSSPECAN_REPCAP_RANGE_R651                                          651
#define RSSPECAN_REPCAP_RANGE_R652                                          652
#define RSSPECAN_REPCAP_RANGE_R653                                          653
#define RSSPECAN_REPCAP_RANGE_R654                                          654
#define RSSPECAN_REPCAP_RANGE_R655                                          655
#define RSSPECAN_REPCAP_RANGE_R656                                          656
#define RSSPECAN_REPCAP_RANGE_R657                                          657
#define RSSPECAN_REPCAP_RANGE_R658                                          658
#define RSSPECAN_REPCAP_RANGE_R659                                          659
#define RSSPECAN_REPCAP_RANGE_R660                                          660
#define RSSPECAN_REPCAP_RANGE_R661                                          661
#define RSSPECAN_REPCAP_RANGE_R662                                          662
#define RSSPECAN_REPCAP_RANGE_R663                                          663
#define RSSPECAN_REPCAP_RANGE_R664                                          664
#define RSSPECAN_REPCAP_RANGE_R665                                          665
#define RSSPECAN_REPCAP_RANGE_R666                                          666
#define RSSPECAN_REPCAP_RANGE_R667                                          667
#define RSSPECAN_REPCAP_RANGE_R668                                          668
#define RSSPECAN_REPCAP_RANGE_R669                                          669
#define RSSPECAN_REPCAP_RANGE_R670                                          670
#define RSSPECAN_REPCAP_RANGE_R671                                          671
#define RSSPECAN_REPCAP_RANGE_R672                                          672
#define RSSPECAN_REPCAP_RANGE_R673                                          673
#define RSSPECAN_REPCAP_RANGE_R674                                          674
#define RSSPECAN_REPCAP_RANGE_R675                                          675
#define RSSPECAN_REPCAP_RANGE_R676                                          676
#define RSSPECAN_REPCAP_RANGE_R677                                          677
#define RSSPECAN_REPCAP_RANGE_R678                                          678
#define RSSPECAN_REPCAP_RANGE_R679                                          679
#define RSSPECAN_REPCAP_RANGE_R680                                          680
#define RSSPECAN_REPCAP_RANGE_R681                                          681
#define RSSPECAN_REPCAP_RANGE_R682                                          682
#define RSSPECAN_REPCAP_RANGE_R683                                          683
#define RSSPECAN_REPCAP_RANGE_R684                                          684
#define RSSPECAN_REPCAP_RANGE_R685                                          685
#define RSSPECAN_REPCAP_RANGE_R686                                          686
#define RSSPECAN_REPCAP_RANGE_R687                                          687
#define RSSPECAN_REPCAP_RANGE_R688                                          688
#define RSSPECAN_REPCAP_RANGE_R689                                          689
#define RSSPECAN_REPCAP_RANGE_R690                                          690
#define RSSPECAN_REPCAP_RANGE_R691                                          691
#define RSSPECAN_REPCAP_RANGE_R692                                          692
#define RSSPECAN_REPCAP_RANGE_R693                                          693
#define RSSPECAN_REPCAP_RANGE_R694                                          694
#define RSSPECAN_REPCAP_RANGE_R695                                          695
#define RSSPECAN_REPCAP_RANGE_R696                                          696
#define RSSPECAN_REPCAP_RANGE_R697                                          697
#define RSSPECAN_REPCAP_RANGE_R698                                          698
#define RSSPECAN_REPCAP_RANGE_R699                                          699
#define RSSPECAN_REPCAP_RANGE_R700                                          700
#define RSSPECAN_REPCAP_RANGE_R701                                          701
#define RSSPECAN_REPCAP_RANGE_R702                                          702
#define RSSPECAN_REPCAP_RANGE_R703                                          703
#define RSSPECAN_REPCAP_RANGE_R704                                          704
#define RSSPECAN_REPCAP_RANGE_R705                                          705
#define RSSPECAN_REPCAP_RANGE_R706                                          706
#define RSSPECAN_REPCAP_RANGE_R707                                          707
#define RSSPECAN_REPCAP_RANGE_R708                                          708
#define RSSPECAN_REPCAP_RANGE_R709                                          709
#define RSSPECAN_REPCAP_RANGE_R710                                          710
#define RSSPECAN_REPCAP_RANGE_R711                                          711
#define RSSPECAN_REPCAP_RANGE_R712                                          712
#define RSSPECAN_REPCAP_RANGE_R713                                          713
#define RSSPECAN_REPCAP_RANGE_R714                                          714
#define RSSPECAN_REPCAP_RANGE_R715                                          715
#define RSSPECAN_REPCAP_RANGE_R716                                          716
#define RSSPECAN_REPCAP_RANGE_R717                                          717
#define RSSPECAN_REPCAP_RANGE_R718                                          718
#define RSSPECAN_REPCAP_RANGE_R719                                          719
#define RSSPECAN_REPCAP_RANGE_R720                                          720
#define RSSPECAN_REPCAP_RANGE_R721                                          721
#define RSSPECAN_REPCAP_RANGE_R722                                          722
#define RSSPECAN_REPCAP_RANGE_R723                                          723
#define RSSPECAN_REPCAP_RANGE_R724                                          724
#define RSSPECAN_REPCAP_RANGE_R725                                          725
#define RSSPECAN_REPCAP_RANGE_R726                                          726
#define RSSPECAN_REPCAP_RANGE_R727                                          727
#define RSSPECAN_REPCAP_RANGE_R728                                          728
#define RSSPECAN_REPCAP_RANGE_R729                                          729
#define RSSPECAN_REPCAP_RANGE_R730                                          730
#define RSSPECAN_REPCAP_RANGE_R731                                          731
#define RSSPECAN_REPCAP_RANGE_R732                                          732
#define RSSPECAN_REPCAP_RANGE_R733                                          733
#define RSSPECAN_REPCAP_RANGE_R734                                          734
#define RSSPECAN_REPCAP_RANGE_R735                                          735
#define RSSPECAN_REPCAP_RANGE_R736                                          736
#define RSSPECAN_REPCAP_RANGE_R737                                          737
#define RSSPECAN_REPCAP_RANGE_R738                                          738
#define RSSPECAN_REPCAP_RANGE_R739                                          739
#define RSSPECAN_REPCAP_RANGE_R740                                          740
#define RSSPECAN_REPCAP_RANGE_R741                                          741
#define RSSPECAN_REPCAP_RANGE_R742                                          742
#define RSSPECAN_REPCAP_RANGE_R743                                          743
#define RSSPECAN_REPCAP_RANGE_R744                                          744
#define RSSPECAN_REPCAP_RANGE_R745                                          745
#define RSSPECAN_REPCAP_RANGE_R746                                          746
#define RSSPECAN_REPCAP_RANGE_R747                                          747
#define RSSPECAN_REPCAP_RANGE_R748                                          748
#define RSSPECAN_REPCAP_RANGE_R749                                          749
#define RSSPECAN_REPCAP_RANGE_R750                                          750
#define RSSPECAN_REPCAP_RANGE_R751                                          751
#define RSSPECAN_REPCAP_RANGE_R752                                          752
#define RSSPECAN_REPCAP_RANGE_R753                                          753
#define RSSPECAN_REPCAP_RANGE_R754                                          754
#define RSSPECAN_REPCAP_RANGE_R755                                          755
#define RSSPECAN_REPCAP_RANGE_R756                                          756
#define RSSPECAN_REPCAP_RANGE_R757                                          757
#define RSSPECAN_REPCAP_RANGE_R758                                          758
#define RSSPECAN_REPCAP_RANGE_R759                                          759
#define RSSPECAN_REPCAP_RANGE_R760                                          760
#define RSSPECAN_REPCAP_RANGE_R761                                          761
#define RSSPECAN_REPCAP_RANGE_R762                                          762
#define RSSPECAN_REPCAP_RANGE_R763                                          763
#define RSSPECAN_REPCAP_RANGE_R764                                          764
#define RSSPECAN_REPCAP_RANGE_R765                                          765
#define RSSPECAN_REPCAP_RANGE_R766                                          766
#define RSSPECAN_REPCAP_RANGE_R767                                          767
#define RSSPECAN_REPCAP_RANGE_R768                                          768
#define RSSPECAN_REPCAP_RANGE_R769                                          769
#define RSSPECAN_REPCAP_RANGE_R770                                          770
#define RSSPECAN_REPCAP_RANGE_R771                                          771
#define RSSPECAN_REPCAP_RANGE_R772                                          772
#define RSSPECAN_REPCAP_RANGE_R773                                          773
#define RSSPECAN_REPCAP_RANGE_R774                                          774
#define RSSPECAN_REPCAP_RANGE_R775                                          775
#define RSSPECAN_REPCAP_RANGE_R776                                          776
#define RSSPECAN_REPCAP_RANGE_R777                                          777
#define RSSPECAN_REPCAP_RANGE_R778                                          778
#define RSSPECAN_REPCAP_RANGE_R779                                          779
#define RSSPECAN_REPCAP_RANGE_R780                                          780
#define RSSPECAN_REPCAP_RANGE_R781                                          781
#define RSSPECAN_REPCAP_RANGE_R782                                          782
#define RSSPECAN_REPCAP_RANGE_R783                                          783
#define RSSPECAN_REPCAP_RANGE_R784                                          784
#define RSSPECAN_REPCAP_RANGE_R785                                          785
#define RSSPECAN_REPCAP_RANGE_R786                                          786
#define RSSPECAN_REPCAP_RANGE_R787                                          787
#define RSSPECAN_REPCAP_RANGE_R788                                          788
#define RSSPECAN_REPCAP_RANGE_R789                                          789
#define RSSPECAN_REPCAP_RANGE_R790                                          790
#define RSSPECAN_REPCAP_RANGE_R791                                          791
#define RSSPECAN_REPCAP_RANGE_R792                                          792
#define RSSPECAN_REPCAP_RANGE_R793                                          793
#define RSSPECAN_REPCAP_RANGE_R794                                          794
#define RSSPECAN_REPCAP_RANGE_R795                                          795
#define RSSPECAN_REPCAP_RANGE_R796                                          796
#define RSSPECAN_REPCAP_RANGE_R797                                          797
#define RSSPECAN_REPCAP_RANGE_R798                                          798
#define RSSPECAN_REPCAP_RANGE_R799                                          799
#define RSSPECAN_REPCAP_RANGE_R800                                          800
#define RSSPECAN_REPCAP_RANGE_R801                                          801
#define RSSPECAN_REPCAP_RANGE_R802                                          802
#define RSSPECAN_REPCAP_RANGE_R803                                          803
#define RSSPECAN_REPCAP_RANGE_R804                                          804
#define RSSPECAN_REPCAP_RANGE_R805                                          805
#define RSSPECAN_REPCAP_RANGE_R806                                          806
#define RSSPECAN_REPCAP_RANGE_R807                                          807
#define RSSPECAN_REPCAP_RANGE_R808                                          808
#define RSSPECAN_REPCAP_RANGE_R809                                          809
#define RSSPECAN_REPCAP_RANGE_R810                                          810
#define RSSPECAN_REPCAP_RANGE_R811                                          811
#define RSSPECAN_REPCAP_RANGE_R812                                          812
#define RSSPECAN_REPCAP_RANGE_R813                                          813
#define RSSPECAN_REPCAP_RANGE_R814                                          814
#define RSSPECAN_REPCAP_RANGE_R815                                          815
#define RSSPECAN_REPCAP_RANGE_R816                                          816
#define RSSPECAN_REPCAP_RANGE_R817                                          817
#define RSSPECAN_REPCAP_RANGE_R818                                          818
#define RSSPECAN_REPCAP_RANGE_R819                                          819
#define RSSPECAN_REPCAP_RANGE_R820                                          820
#define RSSPECAN_REPCAP_RANGE_R821                                          821
#define RSSPECAN_REPCAP_RANGE_R822                                          822
#define RSSPECAN_REPCAP_RANGE_R823                                          823
#define RSSPECAN_REPCAP_RANGE_R824                                          824
#define RSSPECAN_REPCAP_RANGE_R825                                          825
#define RSSPECAN_REPCAP_RANGE_R826                                          826
#define RSSPECAN_REPCAP_RANGE_R827                                          827
#define RSSPECAN_REPCAP_RANGE_R828                                          828
#define RSSPECAN_REPCAP_RANGE_R829                                          829
#define RSSPECAN_REPCAP_RANGE_R830                                          830
#define RSSPECAN_REPCAP_RANGE_R831                                          831
#define RSSPECAN_REPCAP_RANGE_R832                                          832
#define RSSPECAN_REPCAP_RANGE_R833                                          833
#define RSSPECAN_REPCAP_RANGE_R834                                          834
#define RSSPECAN_REPCAP_RANGE_R835                                          835
#define RSSPECAN_REPCAP_RANGE_R836                                          836
#define RSSPECAN_REPCAP_RANGE_R837                                          837
#define RSSPECAN_REPCAP_RANGE_R838                                          838
#define RSSPECAN_REPCAP_RANGE_R839                                          839
#define RSSPECAN_REPCAP_RANGE_R840                                          840
#define RSSPECAN_REPCAP_RANGE_R841                                          841
#define RSSPECAN_REPCAP_RANGE_R842                                          842
#define RSSPECAN_REPCAP_RANGE_R843                                          843
#define RSSPECAN_REPCAP_RANGE_R844                                          844
#define RSSPECAN_REPCAP_RANGE_R845                                          845
#define RSSPECAN_REPCAP_RANGE_R846                                          846
#define RSSPECAN_REPCAP_RANGE_R847                                          847
#define RSSPECAN_REPCAP_RANGE_R848                                          848
#define RSSPECAN_REPCAP_RANGE_R849                                          849
#define RSSPECAN_REPCAP_RANGE_R850                                          850
#define RSSPECAN_REPCAP_RANGE_R851                                          851
#define RSSPECAN_REPCAP_RANGE_R852                                          852
#define RSSPECAN_REPCAP_RANGE_R853                                          853
#define RSSPECAN_REPCAP_RANGE_R854                                          854
#define RSSPECAN_REPCAP_RANGE_R855                                          855
#define RSSPECAN_REPCAP_RANGE_R856                                          856
#define RSSPECAN_REPCAP_RANGE_R857                                          857
#define RSSPECAN_REPCAP_RANGE_R858                                          858
#define RSSPECAN_REPCAP_RANGE_R859                                          859
#define RSSPECAN_REPCAP_RANGE_R860                                          860
#define RSSPECAN_REPCAP_RANGE_R861                                          861
#define RSSPECAN_REPCAP_RANGE_R862                                          862
#define RSSPECAN_REPCAP_RANGE_R863                                          863
#define RSSPECAN_REPCAP_RANGE_R864                                          864
#define RSSPECAN_REPCAP_RANGE_R865                                          865
#define RSSPECAN_REPCAP_RANGE_R866                                          866
#define RSSPECAN_REPCAP_RANGE_R867                                          867
#define RSSPECAN_REPCAP_RANGE_R868                                          868
#define RSSPECAN_REPCAP_RANGE_R869                                          869
#define RSSPECAN_REPCAP_RANGE_R870                                          870
#define RSSPECAN_REPCAP_RANGE_R871                                          871
#define RSSPECAN_REPCAP_RANGE_R872                                          872
#define RSSPECAN_REPCAP_RANGE_R873                                          873
#define RSSPECAN_REPCAP_RANGE_R874                                          874
#define RSSPECAN_REPCAP_RANGE_R875                                          875
#define RSSPECAN_REPCAP_RANGE_R876                                          876
#define RSSPECAN_REPCAP_RANGE_R877                                          877
#define RSSPECAN_REPCAP_RANGE_R878                                          878
#define RSSPECAN_REPCAP_RANGE_R879                                          879
#define RSSPECAN_REPCAP_RANGE_R880                                          880
#define RSSPECAN_REPCAP_RANGE_R881                                          881
#define RSSPECAN_REPCAP_RANGE_R882                                          882
#define RSSPECAN_REPCAP_RANGE_R883                                          883
#define RSSPECAN_REPCAP_RANGE_R884                                          884
#define RSSPECAN_REPCAP_RANGE_R885                                          885
#define RSSPECAN_REPCAP_RANGE_R886                                          886
#define RSSPECAN_REPCAP_RANGE_R887                                          887
#define RSSPECAN_REPCAP_RANGE_R888                                          888
#define RSSPECAN_REPCAP_RANGE_R889                                          889
#define RSSPECAN_REPCAP_RANGE_R890                                          890
#define RSSPECAN_REPCAP_RANGE_R891                                          891
#define RSSPECAN_REPCAP_RANGE_R892                                          892
#define RSSPECAN_REPCAP_RANGE_R893                                          893
#define RSSPECAN_REPCAP_RANGE_R894                                          894
#define RSSPECAN_REPCAP_RANGE_R895                                          895
#define RSSPECAN_REPCAP_RANGE_R896                                          896
#define RSSPECAN_REPCAP_RANGE_R897                                          897
#define RSSPECAN_REPCAP_RANGE_R898                                          898
#define RSSPECAN_REPCAP_RANGE_R899                                          899
#define RSSPECAN_REPCAP_RANGE_R900                                          900
#define RSSPECAN_REPCAP_RANGE_R901                                          901
#define RSSPECAN_REPCAP_RANGE_R902                                          902
#define RSSPECAN_REPCAP_RANGE_R903                                          903
#define RSSPECAN_REPCAP_RANGE_R904                                          904
#define RSSPECAN_REPCAP_RANGE_R905                                          905
#define RSSPECAN_REPCAP_RANGE_R906                                          906
#define RSSPECAN_REPCAP_RANGE_R907                                          907
#define RSSPECAN_REPCAP_RANGE_R908                                          908
#define RSSPECAN_REPCAP_RANGE_R909                                          909
#define RSSPECAN_REPCAP_RANGE_R910                                          910
#define RSSPECAN_REPCAP_RANGE_R911                                          911
#define RSSPECAN_REPCAP_RANGE_R912                                          912
#define RSSPECAN_REPCAP_RANGE_R913                                          913
#define RSSPECAN_REPCAP_RANGE_R914                                          914
#define RSSPECAN_REPCAP_RANGE_R915                                          915
#define RSSPECAN_REPCAP_RANGE_R916                                          916
#define RSSPECAN_REPCAP_RANGE_R917                                          917
#define RSSPECAN_REPCAP_RANGE_R918                                          918
#define RSSPECAN_REPCAP_RANGE_R919                                          919
#define RSSPECAN_REPCAP_RANGE_R920                                          920
#define RSSPECAN_REPCAP_RANGE_R921                                          921
#define RSSPECAN_REPCAP_RANGE_R922                                          922
#define RSSPECAN_REPCAP_RANGE_R923                                          923
#define RSSPECAN_REPCAP_RANGE_R924                                          924
#define RSSPECAN_REPCAP_RANGE_R925                                          925
#define RSSPECAN_REPCAP_RANGE_R926                                          926
#define RSSPECAN_REPCAP_RANGE_R927                                          927
#define RSSPECAN_REPCAP_RANGE_R928                                          928
#define RSSPECAN_REPCAP_RANGE_R929                                          929
#define RSSPECAN_REPCAP_RANGE_R930                                          930
#define RSSPECAN_REPCAP_RANGE_R931                                          931
#define RSSPECAN_REPCAP_RANGE_R932                                          932
#define RSSPECAN_REPCAP_RANGE_R933                                          933
#define RSSPECAN_REPCAP_RANGE_R934                                          934
#define RSSPECAN_REPCAP_RANGE_R935                                          935
#define RSSPECAN_REPCAP_RANGE_R936                                          936
#define RSSPECAN_REPCAP_RANGE_R937                                          937
#define RSSPECAN_REPCAP_RANGE_R938                                          938
#define RSSPECAN_REPCAP_RANGE_R939                                          939
#define RSSPECAN_REPCAP_RANGE_R940                                          940
#define RSSPECAN_REPCAP_RANGE_R941                                          941
#define RSSPECAN_REPCAP_RANGE_R942                                          942
#define RSSPECAN_REPCAP_RANGE_R943                                          943
#define RSSPECAN_REPCAP_RANGE_R944                                          944
#define RSSPECAN_REPCAP_RANGE_R945                                          945
#define RSSPECAN_REPCAP_RANGE_R946                                          946
#define RSSPECAN_REPCAP_RANGE_R947                                          947
#define RSSPECAN_REPCAP_RANGE_R948                                          948
#define RSSPECAN_REPCAP_RANGE_R949                                          949
#define RSSPECAN_REPCAP_RANGE_R950                                          950
#define RSSPECAN_REPCAP_RANGE_R951                                          951
#define RSSPECAN_REPCAP_RANGE_R952                                          952
#define RSSPECAN_REPCAP_RANGE_R953                                          953
#define RSSPECAN_REPCAP_RANGE_R954                                          954
#define RSSPECAN_REPCAP_RANGE_R955                                          955
#define RSSPECAN_REPCAP_RANGE_R956                                          956
#define RSSPECAN_REPCAP_RANGE_R957                                          957
#define RSSPECAN_REPCAP_RANGE_R958                                          958
#define RSSPECAN_REPCAP_RANGE_R959                                          959
#define RSSPECAN_REPCAP_RANGE_R960                                          960
#define RSSPECAN_REPCAP_RANGE_R961                                          961
#define RSSPECAN_REPCAP_RANGE_R962                                          962
#define RSSPECAN_REPCAP_RANGE_R963                                          963
#define RSSPECAN_REPCAP_RANGE_R964                                          964
#define RSSPECAN_REPCAP_RANGE_R965                                          965
#define RSSPECAN_REPCAP_RANGE_R966                                          966
#define RSSPECAN_REPCAP_RANGE_R967                                          967
#define RSSPECAN_REPCAP_RANGE_R968                                          968
#define RSSPECAN_REPCAP_RANGE_R969                                          969
#define RSSPECAN_REPCAP_RANGE_R970                                          970
#define RSSPECAN_REPCAP_RANGE_R971                                          971
#define RSSPECAN_REPCAP_RANGE_R972                                          972
#define RSSPECAN_REPCAP_RANGE_R973                                          973
#define RSSPECAN_REPCAP_RANGE_R974                                          974
#define RSSPECAN_REPCAP_RANGE_R975                                          975
#define RSSPECAN_REPCAP_RANGE_R976                                          976
#define RSSPECAN_REPCAP_RANGE_R977                                          977
#define RSSPECAN_REPCAP_RANGE_R978                                          978
#define RSSPECAN_REPCAP_RANGE_R979                                          979
#define RSSPECAN_REPCAP_RANGE_R980                                          980
#define RSSPECAN_REPCAP_RANGE_R981                                          981
#define RSSPECAN_REPCAP_RANGE_R982                                          982
#define RSSPECAN_REPCAP_RANGE_R983                                          983
#define RSSPECAN_REPCAP_RANGE_R984                                          984
#define RSSPECAN_REPCAP_RANGE_R985                                          985
#define RSSPECAN_REPCAP_RANGE_R986                                          986
#define RSSPECAN_REPCAP_RANGE_R987                                          987
#define RSSPECAN_REPCAP_RANGE_R988                                          988
#define RSSPECAN_REPCAP_RANGE_R989                                          989
#define RSSPECAN_REPCAP_RANGE_R990                                          990
#define RSSPECAN_REPCAP_RANGE_R991                                          991
#define RSSPECAN_REPCAP_RANGE_R992                                          992
#define RSSPECAN_REPCAP_RANGE_R993                                          993
#define RSSPECAN_REPCAP_RANGE_R994                                          994
#define RSSPECAN_REPCAP_RANGE_R995                                          995
#define RSSPECAN_REPCAP_RANGE_R996                                          996
#define RSSPECAN_REPCAP_RANGE_R997                                          997
#define RSSPECAN_REPCAP_RANGE_R998                                          998
#define RSSPECAN_REPCAP_RANGE_R999                                          999
#define RSSPECAN_REPCAP_RANGE_R1000                                         1000

//repcap DeltaMarker
#define RSSPECAN_REPCAP_DELTAMARKER_DM1                                     1
#define RSSPECAN_REPCAP_DELTAMARKER_DM2                                     2
#define RSSPECAN_REPCAP_DELTAMARKER_DM3                                     3
#define RSSPECAN_REPCAP_DELTAMARKER_DM4                                     4
#define RSSPECAN_REPCAP_DELTAMARKER_DM5                                     5
#define RSSPECAN_REPCAP_DELTAMARKER_DM6                                     6
#define RSSPECAN_REPCAP_DELTAMARKER_DM7                                     7
#define RSSPECAN_REPCAP_DELTAMARKER_DM8                                     8
#define RSSPECAN_REPCAP_DELTAMARKER_DM9                                     9
#define RSSPECAN_REPCAP_DELTAMARKER_DM10                                    10
#define RSSPECAN_REPCAP_DELTAMARKER_DM11                                    11
#define RSSPECAN_REPCAP_DELTAMARKER_DM12                                    12
#define RSSPECAN_REPCAP_DELTAMARKER_DM13                                    13
#define RSSPECAN_REPCAP_DELTAMARKER_DM14                                    14
#define RSSPECAN_REPCAP_DELTAMARKER_DM15                                    15
#define RSSPECAN_REPCAP_DELTAMARKER_DM16                                    16

//repcap Trace
#define RSSPECAN_REPCAP_TRACE_TR1                                           1
#define RSSPECAN_REPCAP_TRACE_TR2                                           2
#define RSSPECAN_REPCAP_TRACE_TR3                                           3
#define RSSPECAN_REPCAP_TRACE_TR4                                           4
#define RSSPECAN_REPCAP_TRACE_TR5                                           5
#define RSSPECAN_REPCAP_TRACE_TR6                                           6
#define RSSPECAN_REPCAP_TRACE_TR7                                           7
#define RSSPECAN_REPCAP_TRACE_TR8                                           8

//repcap Line
#define RSSPECAN_REPCAP_LINE_LINE1                                          1
#define RSSPECAN_REPCAP_LINE_LINE2                                          2

//repcap Generator
#define RSSPECAN_REPCAP_GENERATOR_G1                                        1
#define RSSPECAN_REPCAP_GENERATOR_G2                                        2

//repcap Channel
#define RSSPECAN_REPCAP_CHANNEL_CH1                                         1
#define RSSPECAN_REPCAP_CHANNEL_CH2                                         2
#define RSSPECAN_REPCAP_CHANNEL_CH3                                         3
#define RSSPECAN_REPCAP_CHANNEL_CH4                                         4
#define RSSPECAN_REPCAP_CHANNEL_CH5                                         5
#define RSSPECAN_REPCAP_CHANNEL_CH6                                         6
#define RSSPECAN_REPCAP_CHANNEL_CH7                                         7
#define RSSPECAN_REPCAP_CHANNEL_CH8                                         8
#define RSSPECAN_REPCAP_CHANNEL_CH9                                         9
#define RSSPECAN_REPCAP_CHANNEL_CH10                                        10
#define RSSPECAN_REPCAP_CHANNEL_CH11                                        11

//repcap PClass
#define RSSPECAN_REPCAP_PCLASS_PC1                                          1
#define RSSPECAN_REPCAP_PCLASS_PC2                                          2
#define RSSPECAN_REPCAP_PCLASS_PC3                                          3
#define RSSPECAN_REPCAP_PCLASS_PC4                                          4

//repcap EGateRange
#define RSSPECAN_REPCAP_EGATERANGE_ER1                                      1
#define RSSPECAN_REPCAP_EGATERANGE_ER2                                      2
#define RSSPECAN_REPCAP_EGATERANGE_ER3                                      3

//repcap PortNumber
#define RSSPECAN_REPCAP_PORTNUMBER_PORT1                                    1
#define RSSPECAN_REPCAP_PORTNUMBER_PORT2                                    2
#define RSSPECAN_REPCAP_PORTNUMBER_PORT3                                    3

//repcap TXChannel
#define RSSPECAN_REPCAP_TXCHANNEL_TX1                                       1
#define RSSPECAN_REPCAP_TXCHANNEL_TX2                                       2
#define RSSPECAN_REPCAP_TXCHANNEL_TX3                                       3
#define RSSPECAN_REPCAP_TXCHANNEL_TX4                                       4
#define RSSPECAN_REPCAP_TXCHANNEL_TX5                                       5
#define RSSPECAN_REPCAP_TXCHANNEL_TX6                                       6
#define RSSPECAN_REPCAP_TXCHANNEL_TX7                                       7
#define RSSPECAN_REPCAP_TXCHANNEL_TX8                                       8
#define RSSPECAN_REPCAP_TXCHANNEL_TX9                                       9
#define RSSPECAN_REPCAP_TXCHANNEL_TX10                                      10
#define RSSPECAN_REPCAP_TXCHANNEL_TX11                                      11
#define RSSPECAN_REPCAP_TXCHANNEL_TX12                                      12
#define RSSPECAN_REPCAP_TXCHANNEL_TX13                                      13
#define RSSPECAN_REPCAP_TXCHANNEL_TX14                                      14
#define RSSPECAN_REPCAP_TXCHANNEL_TX15                                      15
#define RSSPECAN_REPCAP_TXCHANNEL_TX16                                      16
#define RSSPECAN_REPCAP_TXCHANNEL_TX17                                      17
#define RSSPECAN_REPCAP_TXCHANNEL_TX18                                      18

//repcap Subwindow
#define RSSPECAN_REPCAP_SUBWINDOW_SW1                                       1
#define RSSPECAN_REPCAP_SUBWINDOW_SW2                                       2
#define RSSPECAN_REPCAP_SUBWINDOW_SW3                                       3
#define RSSPECAN_REPCAP_SUBWINDOW_SW4                                       4
#define RSSPECAN_REPCAP_SUBWINDOW_SW5                                       5
#define RSSPECAN_REPCAP_SUBWINDOW_SW6                                       6
#define RSSPECAN_REPCAP_SUBWINDOW_SW7                                       7
#define RSSPECAN_REPCAP_SUBWINDOW_SW8                                       8
#define RSSPECAN_REPCAP_SUBWINDOW_SW9                                       9
#define RSSPECAN_REPCAP_SUBWINDOW_SW10                                      10
#define RSSPECAN_REPCAP_SUBWINDOW_SW11                                      11
#define RSSPECAN_REPCAP_SUBWINDOW_SW12                                      12
#define RSSPECAN_REPCAP_SUBWINDOW_SW13                                      13
#define RSSPECAN_REPCAP_SUBWINDOW_SW14                                      14
#define RSSPECAN_REPCAP_SUBWINDOW_SW15                                      15
#define RSSPECAN_REPCAP_SUBWINDOW_SW16                                      16
#define RSSPECAN_REPCAP_SUBWINDOW_SW17                                      17
#define RSSPECAN_REPCAP_SUBWINDOW_SW18                                      18
#define RSSPECAN_REPCAP_SUBWINDOW_SW19                                      19
#define RSSPECAN_REPCAP_SUBWINDOW_SW20                                      20
#define RSSPECAN_REPCAP_SUBWINDOW_SW21                                      21
#define RSSPECAN_REPCAP_SUBWINDOW_SW22                                      22
#define RSSPECAN_REPCAP_SUBWINDOW_SW23                                      23
#define RSSPECAN_REPCAP_SUBWINDOW_SW24                                      24
#define RSSPECAN_REPCAP_SUBWINDOW_SW25                                      25
#define RSSPECAN_REPCAP_SUBWINDOW_SW26                                      26
#define RSSPECAN_REPCAP_SUBWINDOW_SW27                                      27
#define RSSPECAN_REPCAP_SUBWINDOW_SW28                                      28
#define RSSPECAN_REPCAP_SUBWINDOW_SW29                                      29
#define RSSPECAN_REPCAP_SUBWINDOW_SW30                                      30
#define RSSPECAN_REPCAP_SUBWINDOW_SW31                                      31
#define RSSPECAN_REPCAP_SUBWINDOW_SW32                                      32
#define RSSPECAN_REPCAP_SUBWINDOW_SW33                                      33
#define RSSPECAN_REPCAP_SUBWINDOW_SW34                                      34
#define RSSPECAN_REPCAP_SUBWINDOW_SW35                                      35
#define RSSPECAN_REPCAP_SUBWINDOW_SW36                                      36
#define RSSPECAN_REPCAP_SUBWINDOW_SW37                                      37
#define RSSPECAN_REPCAP_SUBWINDOW_SW38                                      38
#define RSSPECAN_REPCAP_SUBWINDOW_SW39                                      39
#define RSSPECAN_REPCAP_SUBWINDOW_SW40                                      40
#define RSSPECAN_REPCAP_SUBWINDOW_SW41                                      41
#define RSSPECAN_REPCAP_SUBWINDOW_SW42                                      42
#define RSSPECAN_REPCAP_SUBWINDOW_SW43                                      43
#define RSSPECAN_REPCAP_SUBWINDOW_SW44                                      44
#define RSSPECAN_REPCAP_SUBWINDOW_SW45                                      45
#define RSSPECAN_REPCAP_SUBWINDOW_SW46                                      46
#define RSSPECAN_REPCAP_SUBWINDOW_SW47                                      47
#define RSSPECAN_REPCAP_SUBWINDOW_SW48                                      48
#define RSSPECAN_REPCAP_SUBWINDOW_SW49                                      49
#define RSSPECAN_REPCAP_SUBWINDOW_SW50                                      50
#define RSSPECAN_REPCAP_SUBWINDOW_SW51                                      51
#define RSSPECAN_REPCAP_SUBWINDOW_SW52                                      52
#define RSSPECAN_REPCAP_SUBWINDOW_SW53                                      53
#define RSSPECAN_REPCAP_SUBWINDOW_SW54                                      54
#define RSSPECAN_REPCAP_SUBWINDOW_SW55                                      55
#define RSSPECAN_REPCAP_SUBWINDOW_SW56                                      56
#define RSSPECAN_REPCAP_SUBWINDOW_SW57                                      57
#define RSSPECAN_REPCAP_SUBWINDOW_SW58                                      58
#define RSSPECAN_REPCAP_SUBWINDOW_SW59                                      59
#define RSSPECAN_REPCAP_SUBWINDOW_SW60                                      60
#define RSSPECAN_REPCAP_SUBWINDOW_SW61                                      61
#define RSSPECAN_REPCAP_SUBWINDOW_SW62                                      62
#define RSSPECAN_REPCAP_SUBWINDOW_SW63                                      63
#define RSSPECAN_REPCAP_SUBWINDOW_SW64                                      64

//repcap ZoomWindow
#define RSSPECAN_REPCAP_ZOOMWINDOW_ZWIN0                                    0
#define RSSPECAN_REPCAP_ZOOMWINDOW_ZWIN1                                    1
#define RSSPECAN_REPCAP_ZOOMWINDOW_ZWIN2                                    2
#define RSSPECAN_REPCAP_ZOOMWINDOW_ZWIN3                                    3
#define RSSPECAN_REPCAP_ZOOMWINDOW_ZWIN4                                    4

//repcap SecondMarker
#define RSSPECAN_REPCAP_SECONDMARKER_SM1                                    1
#define RSSPECAN_REPCAP_SECONDMARKER_SM2                                    2
#define RSSPECAN_REPCAP_SECONDMARKER_SM3                                    3
#define RSSPECAN_REPCAP_SECONDMARKER_SM4                                    4
#define RSSPECAN_REPCAP_SECONDMARKER_SM5                                    5
#define RSSPECAN_REPCAP_SECONDMARKER_SM6                                    6
#define RSSPECAN_REPCAP_SECONDMARKER_SM7                                    7
#define RSSPECAN_REPCAP_SECONDMARKER_SM8                                    8
#define RSSPECAN_REPCAP_SECONDMARKER_SM9                                    9
#define RSSPECAN_REPCAP_SECONDMARKER_SM10                                   10
#define RSSPECAN_REPCAP_SECONDMARKER_SM11                                   11
#define RSSPECAN_REPCAP_SECONDMARKER_SM12                                   12
#define RSSPECAN_REPCAP_SECONDMARKER_SM13                                   13
#define RSSPECAN_REPCAP_SECONDMARKER_SM14                                   14
#define RSSPECAN_REPCAP_SECONDMARKER_SM15                                   15
#define RSSPECAN_REPCAP_SECONDMARKER_SM16                                   16

//repcap Subblock
#define RSSPECAN_REPCAP_SUBBLOCK_SB1                                        1
#define RSSPECAN_REPCAP_SUBBLOCK_SB2                                        2
#define RSSPECAN_REPCAP_SUBBLOCK_SB3                                        3
#define RSSPECAN_REPCAP_SUBBLOCK_SB4                                        4
#define RSSPECAN_REPCAP_SUBBLOCK_SB5                                        5
#define RSSPECAN_REPCAP_SUBBLOCK_SB6                                        6
#define RSSPECAN_REPCAP_SUBBLOCK_SB7                                        7
#define RSSPECAN_REPCAP_SUBBLOCK_SB8                                        8

//repcap Gap
#define RSSPECAN_REPCAP_GAP_GAP1                                            1
#define RSSPECAN_REPCAP_GAP_GAP2                                            2

//repcap Probe
#define RSSPECAN_REPCAP_PROBE_PROBE1                                        1
#define RSSPECAN_REPCAP_PROBE_PROBE2                                        2
#define RSSPECAN_REPCAP_PROBE_PROBE3                                        3

//repcap Frequency
#define RSSPECAN_REPCAP_FREQUENCY_F1                                        1
#define RSSPECAN_REPCAP_FREQUENCY_F2                                        2
#define RSSPECAN_REPCAP_FREQUENCY_F3                                        3
#define RSSPECAN_REPCAP_FREQUENCY_F4                                        4

//repcap Calculate
#define RSSPECAN_REPCAP_CALCULATE_C0                                        0
#define RSSPECAN_REPCAP_CALCULATE_C1                                        1
#define RSSPECAN_REPCAP_CALCULATE_C2                                        2
#define RSSPECAN_REPCAP_CALCULATE_C3                                        3
#define RSSPECAN_REPCAP_CALCULATE_C4                                        4
#define RSSPECAN_REPCAP_CALCULATE_C5                                        5
#define RSSPECAN_REPCAP_CALCULATE_C6                                        6
#define RSSPECAN_REPCAP_CALCULATE_C7                                        7
#define RSSPECAN_REPCAP_CALCULATE_C8                                        8
#define RSSPECAN_REPCAP_CALCULATE_C9                                        9
#define RSSPECAN_REPCAP_CALCULATE_C10                                       10
#define RSSPECAN_REPCAP_CALCULATE_C11                                       11
#define RSSPECAN_REPCAP_CALCULATE_C12                                       12
#define RSSPECAN_REPCAP_CALCULATE_C13                                       13
#define RSSPECAN_REPCAP_CALCULATE_C14                                       14
#define RSSPECAN_REPCAP_CALCULATE_C15                                       15
#define RSSPECAN_REPCAP_CALCULATE_C16                                       16

//repcap PowerMeter
#define RSSPECAN_REPCAP_POWERMETER_PM0                                      0
#define RSSPECAN_REPCAP_POWERMETER_PM1                                      1
#define RSSPECAN_REPCAP_POWERMETER_PM2                                      2
#define RSSPECAN_REPCAP_POWERMETER_PM3                                      3
#define RSSPECAN_REPCAP_POWERMETER_PM4                                      4

//repcap GSMSlot
#define RSSPECAN_REPCAP_GSMSLOT_GSMS0                                       0
#define RSSPECAN_REPCAP_GSMSLOT_GSMS1                                       1
#define RSSPECAN_REPCAP_GSMSLOT_GSMS2                                       2
#define RSSPECAN_REPCAP_GSMSLOT_GSMS3                                       3
#define RSSPECAN_REPCAP_GSMSLOT_GSMS4                                       4
#define RSSPECAN_REPCAP_GSMSLOT_GSMS5                                       5
#define RSSPECAN_REPCAP_GSMSLOT_GSMS6                                       6
#define RSSPECAN_REPCAP_GSMSLOT_GSMS7                                       7

//repcap GSMSubchannel
#define RSSPECAN_REPCAP_GSMSUBCHANNEL_SUB1                                  1
#define RSSPECAN_REPCAP_GSMSUBCHANNEL_SUB2                                  2

//repcap Alternate
#define RSSPECAN_REPCAP_ALTERNATE_A1                                        1
#define RSSPECAN_REPCAP_ALTERNATE_A2                                        2
#define RSSPECAN_REPCAP_ALTERNATE_A3                                        3
#define RSSPECAN_REPCAP_ALTERNATE_A4                                        4

//repcap LTESubframe
#define RSSPECAN_REPCAP_LTESUBFRAME_SFR0                                    0
#define RSSPECAN_REPCAP_LTESUBFRAME_SFR1                                    1
#define RSSPECAN_REPCAP_LTESUBFRAME_SFR2                                    2
#define RSSPECAN_REPCAP_LTESUBFRAME_SFR3                                    3
#define RSSPECAN_REPCAP_LTESUBFRAME_SFR4                                    4
#define RSSPECAN_REPCAP_LTESUBFRAME_SFR5                                    5
#define RSSPECAN_REPCAP_LTESUBFRAME_SFR6                                    6
#define RSSPECAN_REPCAP_LTESUBFRAME_SFR7                                    7
#define RSSPECAN_REPCAP_LTESUBFRAME_SFR8                                    8
#define RSSPECAN_REPCAP_LTESUBFRAME_SFR9                                    9

//repcap LTEAllocation
#define RSSPECAN_REPCAP_LTEALLOCATION_AL0                                   0
#define RSSPECAN_REPCAP_LTEALLOCATION_AL1                                   1
#define RSSPECAN_REPCAP_LTEALLOCATION_AL2                                   2
#define RSSPECAN_REPCAP_LTEALLOCATION_AL3                                   3
#define RSSPECAN_REPCAP_LTEALLOCATION_AL4                                   4
#define RSSPECAN_REPCAP_LTEALLOCATION_AL5                                   5
#define RSSPECAN_REPCAP_LTEALLOCATION_AL6                                   6
#define RSSPECAN_REPCAP_LTEALLOCATION_AL7                                   7
#define RSSPECAN_REPCAP_LTEALLOCATION_AL8                                   8
#define RSSPECAN_REPCAP_LTEALLOCATION_AL9                                   9
#define RSSPECAN_REPCAP_LTEALLOCATION_AL10                                  10
#define RSSPECAN_REPCAP_LTEALLOCATION_AL11                                  11
#define RSSPECAN_REPCAP_LTEALLOCATION_AL12                                  12
#define RSSPECAN_REPCAP_LTEALLOCATION_AL13                                  13
#define RSSPECAN_REPCAP_LTEALLOCATION_AL14                                  14
#define RSSPECAN_REPCAP_LTEALLOCATION_AL15                                  15
#define RSSPECAN_REPCAP_LTEALLOCATION_AL16                                  16
#define RSSPECAN_REPCAP_LTEALLOCATION_AL17                                  17
#define RSSPECAN_REPCAP_LTEALLOCATION_AL18                                  18
#define RSSPECAN_REPCAP_LTEALLOCATION_AL19                                  19
#define RSSPECAN_REPCAP_LTEALLOCATION_AL20                                  20
#define RSSPECAN_REPCAP_LTEALLOCATION_AL21                                  21
#define RSSPECAN_REPCAP_LTEALLOCATION_AL22                                  22
#define RSSPECAN_REPCAP_LTEALLOCATION_AL23                                  23
#define RSSPECAN_REPCAP_LTEALLOCATION_AL24                                  24
#define RSSPECAN_REPCAP_LTEALLOCATION_AL25                                  25
#define RSSPECAN_REPCAP_LTEALLOCATION_AL26                                  26
#define RSSPECAN_REPCAP_LTEALLOCATION_AL27                                  27
#define RSSPECAN_REPCAP_LTEALLOCATION_AL28                                  28
#define RSSPECAN_REPCAP_LTEALLOCATION_AL29                                  29
#define RSSPECAN_REPCAP_LTEALLOCATION_AL30                                  30
#define RSSPECAN_REPCAP_LTEALLOCATION_AL31                                  31
#define RSSPECAN_REPCAP_LTEALLOCATION_AL32                                  32
#define RSSPECAN_REPCAP_LTEALLOCATION_AL33                                  33
#define RSSPECAN_REPCAP_LTEALLOCATION_AL34                                  34
#define RSSPECAN_REPCAP_LTEALLOCATION_AL35                                  35
#define RSSPECAN_REPCAP_LTEALLOCATION_AL36                                  36
#define RSSPECAN_REPCAP_LTEALLOCATION_AL37                                  37
#define RSSPECAN_REPCAP_LTEALLOCATION_AL38                                  38
#define RSSPECAN_REPCAP_LTEALLOCATION_AL39                                  39
#define RSSPECAN_REPCAP_LTEALLOCATION_AL40                                  40
#define RSSPECAN_REPCAP_LTEALLOCATION_AL41                                  41
#define RSSPECAN_REPCAP_LTEALLOCATION_AL42                                  42
#define RSSPECAN_REPCAP_LTEALLOCATION_AL43                                  43
#define RSSPECAN_REPCAP_LTEALLOCATION_AL44                                  44
#define RSSPECAN_REPCAP_LTEALLOCATION_AL45                                  45
#define RSSPECAN_REPCAP_LTEALLOCATION_AL46                                  46
#define RSSPECAN_REPCAP_LTEALLOCATION_AL47                                  47
#define RSSPECAN_REPCAP_LTEALLOCATION_AL48                                  48
#define RSSPECAN_REPCAP_LTEALLOCATION_AL49                                  49
#define RSSPECAN_REPCAP_LTEALLOCATION_AL50                                  50
#define RSSPECAN_REPCAP_LTEALLOCATION_AL51                                  51
#define RSSPECAN_REPCAP_LTEALLOCATION_AL52                                  52
#define RSSPECAN_REPCAP_LTEALLOCATION_AL53                                  53
#define RSSPECAN_REPCAP_LTEALLOCATION_AL54                                  54
#define RSSPECAN_REPCAP_LTEALLOCATION_AL55                                  55
#define RSSPECAN_REPCAP_LTEALLOCATION_AL56                                  56
#define RSSPECAN_REPCAP_LTEALLOCATION_AL57                                  57
#define RSSPECAN_REPCAP_LTEALLOCATION_AL58                                  58
#define RSSPECAN_REPCAP_LTEALLOCATION_AL59                                  59
#define RSSPECAN_REPCAP_LTEALLOCATION_AL60                                  60
#define RSSPECAN_REPCAP_LTEALLOCATION_AL61                                  61
#define RSSPECAN_REPCAP_LTEALLOCATION_AL62                                  62
#define RSSPECAN_REPCAP_LTEALLOCATION_AL63                                  63
#define RSSPECAN_REPCAP_LTEALLOCATION_AL64                                  64
#define RSSPECAN_REPCAP_LTEALLOCATION_AL65                                  65
#define RSSPECAN_REPCAP_LTEALLOCATION_AL66                                  66
#define RSSPECAN_REPCAP_LTEALLOCATION_AL67                                  67
#define RSSPECAN_REPCAP_LTEALLOCATION_AL68                                  68
#define RSSPECAN_REPCAP_LTEALLOCATION_AL69                                  69
#define RSSPECAN_REPCAP_LTEALLOCATION_AL70                                  70
#define RSSPECAN_REPCAP_LTEALLOCATION_AL71                                  71
#define RSSPECAN_REPCAP_LTEALLOCATION_AL72                                  72
#define RSSPECAN_REPCAP_LTEALLOCATION_AL73                                  73
#define RSSPECAN_REPCAP_LTEALLOCATION_AL74                                  74
#define RSSPECAN_REPCAP_LTEALLOCATION_AL75                                  75
#define RSSPECAN_REPCAP_LTEALLOCATION_AL76                                  76
#define RSSPECAN_REPCAP_LTEALLOCATION_AL77                                  77
#define RSSPECAN_REPCAP_LTEALLOCATION_AL78                                  78
#define RSSPECAN_REPCAP_LTEALLOCATION_AL79                                  79
#define RSSPECAN_REPCAP_LTEALLOCATION_AL80                                  80
#define RSSPECAN_REPCAP_LTEALLOCATION_AL81                                  81
#define RSSPECAN_REPCAP_LTEALLOCATION_AL82                                  82
#define RSSPECAN_REPCAP_LTEALLOCATION_AL83                                  83
#define RSSPECAN_REPCAP_LTEALLOCATION_AL84                                  84
#define RSSPECAN_REPCAP_LTEALLOCATION_AL85                                  85
#define RSSPECAN_REPCAP_LTEALLOCATION_AL86                                  86
#define RSSPECAN_REPCAP_LTEALLOCATION_AL87                                  87
#define RSSPECAN_REPCAP_LTEALLOCATION_AL88                                  88
#define RSSPECAN_REPCAP_LTEALLOCATION_AL89                                  89
#define RSSPECAN_REPCAP_LTEALLOCATION_AL90                                  90
#define RSSPECAN_REPCAP_LTEALLOCATION_AL91                                  91
#define RSSPECAN_REPCAP_LTEALLOCATION_AL92                                  92
#define RSSPECAN_REPCAP_LTEALLOCATION_AL93                                  93
#define RSSPECAN_REPCAP_LTEALLOCATION_AL94                                  94
#define RSSPECAN_REPCAP_LTEALLOCATION_AL95                                  95
#define RSSPECAN_REPCAP_LTEALLOCATION_AL96                                  96
#define RSSPECAN_REPCAP_LTEALLOCATION_AL97                                  97
#define RSSPECAN_REPCAP_LTEALLOCATION_AL98                                  98
#define RSSPECAN_REPCAP_LTEALLOCATION_AL99                                  99

//repcap LTECodeword
#define RSSPECAN_REPCAP_LTECODEWORD_CW1                                     1
#define RSSPECAN_REPCAP_LTECODEWORD_CW2                                     2

//repcap LTETAERAntenna
#define RSSPECAN_REPCAP_LTETAERANTENNA_TAERANT2                             0
#define RSSPECAN_REPCAP_LTETAERANTENNA_TAERANT3                             1
#define RSSPECAN_REPCAP_LTETAERANTENNA_TAERANT4                             2

//repcap Offset
#define RSSPECAN_REPCAP_OFFSET_OFF1                                         1
#define RSSPECAN_REPCAP_OFFSET_OFF2                                         2

//repcap NoiseTraceMemory
#define RSSPECAN_REPCAP_NOISETRACEMEMORY_MEM1                               1
#define RSSPECAN_REPCAP_NOISETRACEMEMORY_MEM2                               2
#define RSSPECAN_REPCAP_NOISETRACEMEMORY_MEM3                               3

//repcap PnoRange
#define RSSPECAN_REPCAP_PNORANGE_PNR1                                       1
#define RSSPECAN_REPCAP_PNORANGE_PNR2                                       2
#define RSSPECAN_REPCAP_PNORANGE_PNR3                                       3
#define RSSPECAN_REPCAP_PNORANGE_PNR4                                       4
#define RSSPECAN_REPCAP_PNORANGE_PNR5                                       5
#define RSSPECAN_REPCAP_PNORANGE_PNR6                                       6
#define RSSPECAN_REPCAP_PNORANGE_PNR7                                       7
#define RSSPECAN_REPCAP_PNORANGE_PNR8                                       8
#define RSSPECAN_REPCAP_PNORANGE_PNR9                                       9
#define RSSPECAN_REPCAP_PNORANGE_PNR10                                      10
#define RSSPECAN_REPCAP_PNORANGE_PNR11                                      11
#define RSSPECAN_REPCAP_PNORANGE_PNR12                                      12
#define RSSPECAN_REPCAP_PNORANGE_PNR13                                      13
#define RSSPECAN_REPCAP_PNORANGE_PNR14                                      14
#define RSSPECAN_REPCAP_PNORANGE_PNR15                                      15
#define RSSPECAN_REPCAP_PNORANGE_PNR16                                      16
#define RSSPECAN_REPCAP_PNORANGE_PNR17                                      17
#define RSSPECAN_REPCAP_PNORANGE_PNR18                                      18
#define RSSPECAN_REPCAP_PNORANGE_PNR19                                      19
#define RSSPECAN_REPCAP_PNORANGE_PNR20                                      20

//repcap Snoise
#define RSSPECAN_REPCAP_SNOISE_SN1                                          1
#define RSSPECAN_REPCAP_SNOISE_SN2                                          2
#define RSSPECAN_REPCAP_SNOISE_SN3                                          3
#define RSSPECAN_REPCAP_SNOISE_SN4                                          4
#define RSSPECAN_REPCAP_SNOISE_SN5                                          5
#define RSSPECAN_REPCAP_SNOISE_SN6                                          6
#define RSSPECAN_REPCAP_SNOISE_SN7                                          7
#define RSSPECAN_REPCAP_SNOISE_SN8                                          8

//repcap pnlSegment
#define RSSPECAN_REPCAP_PNLSEGMENT_PNL1                                     1
#define RSSPECAN_REPCAP_PNLSEGMENT_PNL2                                     2
#define RSSPECAN_REPCAP_PNLSEGMENT_PNL3                                     3
#define RSSPECAN_REPCAP_PNLSEGMENT_PNL4                                     4
#define RSSPECAN_REPCAP_PNLSEGMENT_PNL5                                     5

//repcap User
#define RSSPECAN_REPCAP_USER_U1                                             1
#define RSSPECAN_REPCAP_USER_U2                                             2
#define RSSPECAN_REPCAP_USER_U3                                             3

//repcap LimitLine
#define RSSPECAN_REPCAP_LIMITLINE_LL1                                       1
#define RSSPECAN_REPCAP_LIMITLINE_LL2                                       2
#define RSSPECAN_REPCAP_LIMITLINE_LL3                                       3
#define RSSPECAN_REPCAP_LIMITLINE_LL4                                       4
#define RSSPECAN_REPCAP_LIMITLINE_LL5                                       5
#define RSSPECAN_REPCAP_LIMITLINE_LL6                                       6
#define RSSPECAN_REPCAP_LIMITLINE_LL7                                       7
#define RSSPECAN_REPCAP_LIMITLINE_LL8                                       8

//repcap AntMatrix
#define RSSPECAN_REPCAP_ANTMATRIX_ANTMATRIX1                                1
#define RSSPECAN_REPCAP_ANTMATRIX_ANTMATRIX2                                2
#define RSSPECAN_REPCAP_ANTMATRIX_ANTMATRIX3                                3
#define RSSPECAN_REPCAP_ANTMATRIX_ANTMATRIX4                                4
#define RSSPECAN_REPCAP_ANTMATRIX_ANTMATRIX5                                5
#define RSSPECAN_REPCAP_ANTMATRIX_ANTMATRIX6                                6
#define RSSPECAN_REPCAP_ANTMATRIX_ANTMATRIX7                                7
#define RSSPECAN_REPCAP_ANTMATRIX_ANTMATRIX8                                8

//repcap WLAN_MIMOAntenna
#define RSSPECAN_REPCAP_WLAN_MIMOANTENNA_WLANTX1                            1
#define RSSPECAN_REPCAP_WLAN_MIMOANTENNA_WLANTX2                            2
#define RSSPECAN_REPCAP_WLAN_MIMOANTENNA_WLANTX3                            3
#define RSSPECAN_REPCAP_WLAN_MIMOANTENNA_WLANTX4                            4

//repcap WLAN_MIMO_Stream
#define RSSPECAN_REPCAP_WLAN_MIMO_STREAM_STREAM1                            1
#define RSSPECAN_REPCAP_WLAN_MIMO_STREAM_STREAM2                            2
#define RSSPECAN_REPCAP_WLAN_MIMO_STREAM_STREAM3                            3
#define RSSPECAN_REPCAP_WLAN_MIMO_STREAM_STREAM4                            4

//repcap Bitmap
#define RSSPECAN_REPCAP_BITMAP_B1                                           1
#define RSSPECAN_REPCAP_BITMAP_B2                                           2
#define RSSPECAN_REPCAP_BITMAP_B3                                           3
#define RSSPECAN_REPCAP_BITMAP_B4                                           4
#define RSSPECAN_REPCAP_BITMAP_B5                                           5
#define RSSPECAN_REPCAP_BITMAP_B6                                           6
#define RSSPECAN_REPCAP_BITMAP_B7                                           7
#define RSSPECAN_REPCAP_BITMAP_B8                                           8
#define RSSPECAN_REPCAP_BITMAP_B9                                           9
#define RSSPECAN_REPCAP_BITMAP_B10                                          10
#define RSSPECAN_REPCAP_BITMAP_B11                                          11
#define RSSPECAN_REPCAP_BITMAP_B12                                          12
#define RSSPECAN_REPCAP_BITMAP_B13                                          13
#define RSSPECAN_REPCAP_BITMAP_B14                                          14
#define RSSPECAN_REPCAP_BITMAP_B15                                          15
#define RSSPECAN_REPCAP_BITMAP_B16                                          16
#define RSSPECAN_REPCAP_BITMAP_B17                                          17
#define RSSPECAN_REPCAP_BITMAP_B18                                          18

//repcap Burst
#define RSSPECAN_REPCAP_BURST_BU1                                           1
#define RSSPECAN_REPCAP_BURST_BU2                                           2
#define RSSPECAN_REPCAP_BURST_BU3                                           3
#define RSSPECAN_REPCAP_BURST_BU4                                           4
#define RSSPECAN_REPCAP_BURST_BU5                                           5
#define RSSPECAN_REPCAP_BURST_BU6                                           6
#define RSSPECAN_REPCAP_BURST_BU7                                           7
#define RSSPECAN_REPCAP_BURST_BU8                                           8
#define RSSPECAN_REPCAP_BURST_BU9                                           9
#define RSSPECAN_REPCAP_BURST_BU10                                          10
#define RSSPECAN_REPCAP_BURST_BU11                                          11
#define RSSPECAN_REPCAP_BURST_BU12                                          12
#define RSSPECAN_REPCAP_BURST_BU13                                          13
#define RSSPECAN_REPCAP_BURST_BU14                                          14
#define RSSPECAN_REPCAP_BURST_BU15                                          15
#define RSSPECAN_REPCAP_BURST_BU16                                          16
#define RSSPECAN_REPCAP_BURST_BU17                                          17
#define RSSPECAN_REPCAP_BURST_BU18                                          18
#define RSSPECAN_REPCAP_BURST_BU19                                          19
#define RSSPECAN_REPCAP_BURST_BU20                                          20
#define RSSPECAN_REPCAP_BURST_BU21                                          21
#define RSSPECAN_REPCAP_BURST_BU22                                          22
#define RSSPECAN_REPCAP_BURST_BU23                                          23
#define RSSPECAN_REPCAP_BURST_BU24                                          24
#define RSSPECAN_REPCAP_BURST_BU25                                          25
#define RSSPECAN_REPCAP_BURST_BU26                                          26
#define RSSPECAN_REPCAP_BURST_BU27                                          27
#define RSSPECAN_REPCAP_BURST_BU28                                          28
#define RSSPECAN_REPCAP_BURST_BU29                                          29
#define RSSPECAN_REPCAP_BURST_BU30                                          30
#define RSSPECAN_REPCAP_BURST_BU31                                          31
#define RSSPECAN_REPCAP_BURST_BU32                                          32

//repcap Segment
#define RSSPECAN_REPCAP_SEGMENT_SEG1                                        1
#define RSSPECAN_REPCAP_SEGMENT_SEG2                                        2
#define RSSPECAN_REPCAP_SEGMENT_SEG3                                        3

//repcap Zone
#define RSSPECAN_REPCAP_ZONE_Z1                                             1
#define RSSPECAN_REPCAP_ZONE_Z2                                             2
#define RSSPECAN_REPCAP_ZONE_Z3                                             3
#define RSSPECAN_REPCAP_ZONE_Z4                                             4
#define RSSPECAN_REPCAP_ZONE_Z5                                             5
#define RSSPECAN_REPCAP_ZONE_Z6                                             6
#define RSSPECAN_REPCAP_ZONE_Z7                                             7
#define RSSPECAN_REPCAP_ZONE_Z8                                             8
#define RSSPECAN_REPCAP_ZONE_Z9                                             9
#define RSSPECAN_REPCAP_ZONE_Z10                                            10
#define RSSPECAN_REPCAP_ZONE_Z11                                            11
#define RSSPECAN_REPCAP_ZONE_Z12                                            12
#define RSSPECAN_REPCAP_ZONE_Z13                                            13
#define RSSPECAN_REPCAP_ZONE_Z14                                            14
#define RSSPECAN_REPCAP_ZONE_Z15                                            15
#define RSSPECAN_REPCAP_ZONE_Z16                                            16
#define RSSPECAN_REPCAP_ZONE_Z17                                            17
#define RSSPECAN_REPCAP_ZONE_Z18                                            18
#define RSSPECAN_REPCAP_ZONE_Z19                                            19
#define RSSPECAN_REPCAP_ZONE_Z20                                            20
#define RSSPECAN_REPCAP_ZONE_Z21                                            21
#define RSSPECAN_REPCAP_ZONE_Z22                                            22
#define RSSPECAN_REPCAP_ZONE_Z23                                            23
#define RSSPECAN_REPCAP_ZONE_Z24                                            24
#define RSSPECAN_REPCAP_ZONE_Z25                                            25
#define RSSPECAN_REPCAP_ZONE_Z26                                            26

//repcap InputChannel
#define RSSPECAN_REPCAP_INPUTCHANNEL_ICH1                                   1
#define RSSPECAN_REPCAP_INPUTCHANNEL_ICH2                                   2
#define RSSPECAN_REPCAP_INPUTCHANNEL_ICH3                                   3
#define RSSPECAN_REPCAP_INPUTCHANNEL_ICH4                                   4

//repcap WCDMAAntenna
#define RSSPECAN_REPCAP_WCDMAANTENNA_ANT1                                   1
#define RSSPECAN_REPCAP_WCDMAANTENNA_ANT2                                   2

//repcap GSMCarrier
#define RSSPECAN_REPCAP_GSMCARRIER_GSMCARR1                                 1
#define RSSPECAN_REPCAP_GSMCARRIER_GSMCARR2                                 2
#define RSSPECAN_REPCAP_GSMCARRIER_GSMCARR3                                 3
#define RSSPECAN_REPCAP_GSMCARRIER_GSMCARR4                                 4
#define RSSPECAN_REPCAP_GSMCARRIER_GSMCARR5                                 5
#define RSSPECAN_REPCAP_GSMCARRIER_GSMCARR6                                 6
#define RSSPECAN_REPCAP_GSMCARRIER_GSMCARR7                                 7
#define RSSPECAN_REPCAP_GSMCARRIER_GSMCARR8                                 8
#define RSSPECAN_REPCAP_GSMCARRIER_GSMCARR9                                 9
#define RSSPECAN_REPCAP_GSMCARRIER_GSMCARR10                                10
#define RSSPECAN_REPCAP_GSMCARRIER_GSMCARR11                                11
#define RSSPECAN_REPCAP_GSMCARRIER_GSMCARR12                                12
#define RSSPECAN_REPCAP_GSMCARRIER_GSMCARR13                                13
#define RSSPECAN_REPCAP_GSMCARRIER_GSMCARR14                                14
#define RSSPECAN_REPCAP_GSMCARRIER_GSMCARR15                                15
#define RSSPECAN_REPCAP_GSMCARRIER_GSMCARR16                                16

//repcap GSMLimit
#define RSSPECAN_REPCAP_GSMLIMIT_GL1                                        1
#define RSSPECAN_REPCAP_GSMLIMIT_GL2                                        2
#define RSSPECAN_REPCAP_GSMLIMIT_GL3                                        3
#define RSSPECAN_REPCAP_GSMLIMIT_GL5                                        4
#define RSSPECAN_REPCAP_GSMLIMIT_GL6                                        5

//repcap Detector
#define RSSPECAN_REPCAP_DETECTOR_DETPPEAK                                   1
#define RSSPECAN_REPCAP_DETECTOR_DETNPEAK                                   2
#define RSSPECAN_REPCAP_DETECTOR_DETAPEAK                                   3
#define RSSPECAN_REPCAP_DETECTOR_DETRMS                                     4

//repcap WCDMACarrier
#define RSSPECAN_REPCAP_WCDMACARRIER_CARR1                                  1
#define RSSPECAN_REPCAP_WCDMACARRIER_CARR2                                  2
#define RSSPECAN_REPCAP_WCDMACARRIER_CARR3                                  3
#define RSSPECAN_REPCAP_WCDMACARRIER_CARR4                                  4

//repcap MultiSEMSubBlock
#define RSSPECAN_REPCAP_MULTISEMSUBBLOCK_MSEMSBA                            1
#define RSSPECAN_REPCAP_MULTISEMSUBBLOCK_MSEMSBB                            2
#define RSSPECAN_REPCAP_MULTISEMSUBBLOCK_MSEMSBC                            3

//repcap DOCSISSetIndex
#define RSSPECAN_REPCAP_DOCSISSETINDEX_I1                                   1
#define RSSPECAN_REPCAP_DOCSISSETINDEX_I2                                   2
#define RSSPECAN_REPCAP_DOCSISSETINDEX_I3                                   3
#define RSSPECAN_REPCAP_DOCSISSETINDEX_I4                                   4
#define RSSPECAN_REPCAP_DOCSISSETINDEX_I5                                   5
#define RSSPECAN_REPCAP_DOCSISSETINDEX_I6                                   6
#define RSSPECAN_REPCAP_DOCSISSETINDEX_I7                                   7
#define RSSPECAN_REPCAP_DOCSISSETINDEX_I8                                   8
#define RSSPECAN_REPCAP_DOCSISSETINDEX_I9                                   9
#define RSSPECAN_REPCAP_DOCSISSETINDEX_I10                                  10
#define RSSPECAN_REPCAP_DOCSISSETINDEX_I11                                  11
#define RSSPECAN_REPCAP_DOCSISSETINDEX_I12                                  12
#define RSSPECAN_REPCAP_DOCSISSETINDEX_I13                                  13
#define RSSPECAN_REPCAP_DOCSISSETINDEX_I14                                  14
#define RSSPECAN_REPCAP_DOCSISSETINDEX_I15                                  15
#define RSSPECAN_REPCAP_DOCSISSETINDEX_I16                                  16
#define RSSPECAN_REPCAP_DOCSISSETINDEX_I17                                  17
#define RSSPECAN_REPCAP_DOCSISSETINDEX_I18                                  18
#define RSSPECAN_REPCAP_DOCSISSETINDEX_I19                                  19
#define RSSPECAN_REPCAP_DOCSISSETINDEX_I20                                  20
#define RSSPECAN_REPCAP_DOCSISSETINDEX_I21                                  21
#define RSSPECAN_REPCAP_DOCSISSETINDEX_I22                                  22
#define RSSPECAN_REPCAP_DOCSISSETINDEX_I23                                  23
#define RSSPECAN_REPCAP_DOCSISSETINDEX_I24                                  24
#define RSSPECAN_REPCAP_DOCSISSETINDEX_I25                                  25
#define RSSPECAN_REPCAP_DOCSISSETINDEX_I26                                  26
#define RSSPECAN_REPCAP_DOCSISSETINDEX_I27                                  27
#define RSSPECAN_REPCAP_DOCSISSETINDEX_I28                                  28
#define RSSPECAN_REPCAP_DOCSISSETINDEX_I29                                  29
#define RSSPECAN_REPCAP_DOCSISSETINDEX_I30                                  30
#define RSSPECAN_REPCAP_DOCSISSETINDEX_I31                                  31
#define RSSPECAN_REPCAP_DOCSISSETINDEX_I32                                  32
#define RSSPECAN_REPCAP_DOCSISSETINDEX_I33                                  33
#define RSSPECAN_REPCAP_DOCSISSETINDEX_I34                                  34
#define RSSPECAN_REPCAP_DOCSISSETINDEX_I35                                  35
#define RSSPECAN_REPCAP_DOCSISSETINDEX_I36                                  36
#define RSSPECAN_REPCAP_DOCSISSETINDEX_I37                                  37
#define RSSPECAN_REPCAP_DOCSISSETINDEX_I38                                  38
#define RSSPECAN_REPCAP_DOCSISSETINDEX_I39                                  39
#define RSSPECAN_REPCAP_DOCSISSETINDEX_I40                                  40
#define RSSPECAN_REPCAP_DOCSISSETINDEX_I41                                  41
#define RSSPECAN_REPCAP_DOCSISSETINDEX_I42                                  42
#define RSSPECAN_REPCAP_DOCSISSETINDEX_I43                                  43
#define RSSPECAN_REPCAP_DOCSISSETINDEX_I44                                  44
#define RSSPECAN_REPCAP_DOCSISSETINDEX_I45                                  45
#define RSSPECAN_REPCAP_DOCSISSETINDEX_I46                                  46
#define RSSPECAN_REPCAP_DOCSISSETINDEX_I47                                  47
#define RSSPECAN_REPCAP_DOCSISSETINDEX_I48                                  48
#define RSSPECAN_REPCAP_DOCSISSETINDEX_I49                                  49
#define RSSPECAN_REPCAP_DOCSISSETINDEX_I50                                  50
#define RSSPECAN_REPCAP_DOCSISSETINDEX_I51                                  51
#define RSSPECAN_REPCAP_DOCSISSETINDEX_I52                                  52
#define RSSPECAN_REPCAP_DOCSISSETINDEX_I53                                  53
#define RSSPECAN_REPCAP_DOCSISSETINDEX_I54                                  54
#define RSSPECAN_REPCAP_DOCSISSETINDEX_I55                                  55
#define RSSPECAN_REPCAP_DOCSISSETINDEX_I56                                  56
#define RSSPECAN_REPCAP_DOCSISSETINDEX_I57                                  57
#define RSSPECAN_REPCAP_DOCSISSETINDEX_I58                                  58
#define RSSPECAN_REPCAP_DOCSISSETINDEX_I59                                  59
#define RSSPECAN_REPCAP_DOCSISSETINDEX_I60                                  60
#define RSSPECAN_REPCAP_DOCSISSETINDEX_I61                                  61
#define RSSPECAN_REPCAP_DOCSISSETINDEX_I62                                  62
#define RSSPECAN_REPCAP_DOCSISSETINDEX_I63                                  63
#define RSSPECAN_REPCAP_DOCSISSETINDEX_I64                                  64
#define RSSPECAN_REPCAP_DOCSISSETINDEX_I65                                  65
#define RSSPECAN_REPCAP_DOCSISSETINDEX_I66                                  66
#define RSSPECAN_REPCAP_DOCSISSETINDEX_I67                                  67
#define RSSPECAN_REPCAP_DOCSISSETINDEX_I68                                  68
#define RSSPECAN_REPCAP_DOCSISSETINDEX_I69                                  69
#define RSSPECAN_REPCAP_DOCSISSETINDEX_I70                                  70
#define RSSPECAN_REPCAP_DOCSISSETINDEX_I71                                  71
#define RSSPECAN_REPCAP_DOCSISSETINDEX_I72                                  72
#define RSSPECAN_REPCAP_DOCSISSETINDEX_I73                                  73
#define RSSPECAN_REPCAP_DOCSISSETINDEX_I74                                  74
#define RSSPECAN_REPCAP_DOCSISSETINDEX_I75                                  75
#define RSSPECAN_REPCAP_DOCSISSETINDEX_I76                                  76
#define RSSPECAN_REPCAP_DOCSISSETINDEX_I77                                  77
#define RSSPECAN_REPCAP_DOCSISSETINDEX_I78                                  78
#define RSSPECAN_REPCAP_DOCSISSETINDEX_I79                                  79
#define RSSPECAN_REPCAP_DOCSISSETINDEX_I80                                  80
#define RSSPECAN_REPCAP_DOCSISSETINDEX_I81                                  81
#define RSSPECAN_REPCAP_DOCSISSETINDEX_I82                                  82
#define RSSPECAN_REPCAP_DOCSISSETINDEX_I83                                  83
#define RSSPECAN_REPCAP_DOCSISSETINDEX_I84                                  84
#define RSSPECAN_REPCAP_DOCSISSETINDEX_I85                                  85
#define RSSPECAN_REPCAP_DOCSISSETINDEX_I86                                  86
#define RSSPECAN_REPCAP_DOCSISSETINDEX_I87                                  87
#define RSSPECAN_REPCAP_DOCSISSETINDEX_I88                                  88
#define RSSPECAN_REPCAP_DOCSISSETINDEX_I89                                  89
#define RSSPECAN_REPCAP_DOCSISSETINDEX_I90                                  90
#define RSSPECAN_REPCAP_DOCSISSETINDEX_I91                                  91
#define RSSPECAN_REPCAP_DOCSISSETINDEX_I92                                  92
#define RSSPECAN_REPCAP_DOCSISSETINDEX_I93                                  93
#define RSSPECAN_REPCAP_DOCSISSETINDEX_I94                                  94
#define RSSPECAN_REPCAP_DOCSISSETINDEX_I95                                  95
#define RSSPECAN_REPCAP_DOCSISSETINDEX_I96                                  96
#define RSSPECAN_REPCAP_DOCSISSETINDEX_I97                                  97
#define RSSPECAN_REPCAP_DOCSISSETINDEX_I98                                  98
#define RSSPECAN_REPCAP_DOCSISSETINDEX_I99                                  99
#define RSSPECAN_REPCAP_DOCSISSETINDEX_I100                                 100
#define RSSPECAN_REPCAP_DOCSISSETINDEX_I101                                 101
#define RSSPECAN_REPCAP_DOCSISSETINDEX_I102                                 102
#define RSSPECAN_REPCAP_DOCSISSETINDEX_I103                                 103
#define RSSPECAN_REPCAP_DOCSISSETINDEX_I104                                 104
#define RSSPECAN_REPCAP_DOCSISSETINDEX_I105                                 105
#define RSSPECAN_REPCAP_DOCSISSETINDEX_I106                                 106
#define RSSPECAN_REPCAP_DOCSISSETINDEX_I107                                 107
#define RSSPECAN_REPCAP_DOCSISSETINDEX_I108                                 108
#define RSSPECAN_REPCAP_DOCSISSETINDEX_I109                                 109
#define RSSPECAN_REPCAP_DOCSISSETINDEX_I110                                 110
#define RSSPECAN_REPCAP_DOCSISSETINDEX_I111                                 111
#define RSSPECAN_REPCAP_DOCSISSETINDEX_I112                                 112
#define RSSPECAN_REPCAP_DOCSISSETINDEX_I113                                 113
#define RSSPECAN_REPCAP_DOCSISSETINDEX_I114                                 114
#define RSSPECAN_REPCAP_DOCSISSETINDEX_I115                                 115
#define RSSPECAN_REPCAP_DOCSISSETINDEX_I116                                 116
#define RSSPECAN_REPCAP_DOCSISSETINDEX_I117                                 117
#define RSSPECAN_REPCAP_DOCSISSETINDEX_I118                                 118
#define RSSPECAN_REPCAP_DOCSISSETINDEX_I119                                 119
#define RSSPECAN_REPCAP_DOCSISSETINDEX_I120                                 120
#define RSSPECAN_REPCAP_DOCSISSETINDEX_I121                                 121
#define RSSPECAN_REPCAP_DOCSISSETINDEX_I122                                 122
#define RSSPECAN_REPCAP_DOCSISSETINDEX_I123                                 123
#define RSSPECAN_REPCAP_DOCSISSETINDEX_I124                                 124
#define RSSPECAN_REPCAP_DOCSISSETINDEX_I125                                 125
#define RSSPECAN_REPCAP_DOCSISSETINDEX_I126                                 126
#define RSSPECAN_REPCAP_DOCSISSETINDEX_I127                                 127
#define RSSPECAN_REPCAP_DOCSISSETINDEX_I128                                 128
#define RSSPECAN_REPCAP_DOCSISSETINDEX_I129                                 129
#define RSSPECAN_REPCAP_DOCSISSETINDEX_I130                                 130
#define RSSPECAN_REPCAP_DOCSISSETINDEX_I131                                 131
#define RSSPECAN_REPCAP_DOCSISSETINDEX_I132                                 132
#define RSSPECAN_REPCAP_DOCSISSETINDEX_I133                                 133
#define RSSPECAN_REPCAP_DOCSISSETINDEX_I134                                 134
#define RSSPECAN_REPCAP_DOCSISSETINDEX_I135                                 135
#define RSSPECAN_REPCAP_DOCSISSETINDEX_I136                                 136
#define RSSPECAN_REPCAP_DOCSISSETINDEX_I137                                 137
#define RSSPECAN_REPCAP_DOCSISSETINDEX_I138                                 138
#define RSSPECAN_REPCAP_DOCSISSETINDEX_I139                                 139
#define RSSPECAN_REPCAP_DOCSISSETINDEX_I140                                 140
#define RSSPECAN_REPCAP_DOCSISSETINDEX_I141                                 141
#define RSSPECAN_REPCAP_DOCSISSETINDEX_I142                                 142
#define RSSPECAN_REPCAP_DOCSISSETINDEX_I143                                 143
#define RSSPECAN_REPCAP_DOCSISSETINDEX_I144                                 144
#define RSSPECAN_REPCAP_DOCSISSETINDEX_I145                                 145
#define RSSPECAN_REPCAP_DOCSISSETINDEX_I146                                 146
#define RSSPECAN_REPCAP_DOCSISSETINDEX_I147                                 147
#define RSSPECAN_REPCAP_DOCSISSETINDEX_I148                                 148
#define RSSPECAN_REPCAP_DOCSISSETINDEX_I149                                 149
#define RSSPECAN_REPCAP_DOCSISSETINDEX_I150                                 150
#define RSSPECAN_REPCAP_DOCSISSETINDEX_I151                                 151
#define RSSPECAN_REPCAP_DOCSISSETINDEX_I152                                 152
#define RSSPECAN_REPCAP_DOCSISSETINDEX_I153                                 153
#define RSSPECAN_REPCAP_DOCSISSETINDEX_I154                                 154
#define RSSPECAN_REPCAP_DOCSISSETINDEX_I155                                 155
#define RSSPECAN_REPCAP_DOCSISSETINDEX_I156                                 156
#define RSSPECAN_REPCAP_DOCSISSETINDEX_I157                                 157
#define RSSPECAN_REPCAP_DOCSISSETINDEX_I158                                 158
#define RSSPECAN_REPCAP_DOCSISSETINDEX_I159                                 159
#define RSSPECAN_REPCAP_DOCSISSETINDEX_I160                                 160
#define RSSPECAN_REPCAP_DOCSISSETINDEX_I161                                 161
#define RSSPECAN_REPCAP_DOCSISSETINDEX_I162                                 162
#define RSSPECAN_REPCAP_DOCSISSETINDEX_I163                                 163
#define RSSPECAN_REPCAP_DOCSISSETINDEX_I164                                 164
#define RSSPECAN_REPCAP_DOCSISSETINDEX_I165                                 165
#define RSSPECAN_REPCAP_DOCSISSETINDEX_I166                                 166
#define RSSPECAN_REPCAP_DOCSISSETINDEX_I167                                 167
#define RSSPECAN_REPCAP_DOCSISSETINDEX_I168                                 168
#define RSSPECAN_REPCAP_DOCSISSETINDEX_I169                                 169
#define RSSPECAN_REPCAP_DOCSISSETINDEX_I170                                 170
#define RSSPECAN_REPCAP_DOCSISSETINDEX_I171                                 171
#define RSSPECAN_REPCAP_DOCSISSETINDEX_I172                                 172
#define RSSPECAN_REPCAP_DOCSISSETINDEX_I173                                 173
#define RSSPECAN_REPCAP_DOCSISSETINDEX_I174                                 174
#define RSSPECAN_REPCAP_DOCSISSETINDEX_I175                                 175
#define RSSPECAN_REPCAP_DOCSISSETINDEX_I176                                 176
#define RSSPECAN_REPCAP_DOCSISSETINDEX_I177                                 177
#define RSSPECAN_REPCAP_DOCSISSETINDEX_I178                                 178
#define RSSPECAN_REPCAP_DOCSISSETINDEX_I179                                 179
#define RSSPECAN_REPCAP_DOCSISSETINDEX_I180                                 180
#define RSSPECAN_REPCAP_DOCSISSETINDEX_I181                                 181
#define RSSPECAN_REPCAP_DOCSISSETINDEX_I182                                 182
#define RSSPECAN_REPCAP_DOCSISSETINDEX_I183                                 183
#define RSSPECAN_REPCAP_DOCSISSETINDEX_I184                                 184
#define RSSPECAN_REPCAP_DOCSISSETINDEX_I185                                 185
#define RSSPECAN_REPCAP_DOCSISSETINDEX_I186                                 186
#define RSSPECAN_REPCAP_DOCSISSETINDEX_I187                                 187
#define RSSPECAN_REPCAP_DOCSISSETINDEX_I188                                 188
#define RSSPECAN_REPCAP_DOCSISSETINDEX_I189                                 189
#define RSSPECAN_REPCAP_DOCSISSETINDEX_I190                                 190
#define RSSPECAN_REPCAP_DOCSISSETINDEX_I191                                 191
#define RSSPECAN_REPCAP_DOCSISSETINDEX_I192                                 192
#define RSSPECAN_REPCAP_DOCSISSETINDEX_I193                                 193
#define RSSPECAN_REPCAP_DOCSISSETINDEX_I194                                 194
#define RSSPECAN_REPCAP_DOCSISSETINDEX_I195                                 195
#define RSSPECAN_REPCAP_DOCSISSETINDEX_I196                                 196
#define RSSPECAN_REPCAP_DOCSISSETINDEX_I197                                 197
#define RSSPECAN_REPCAP_DOCSISSETINDEX_I198                                 198
#define RSSPECAN_REPCAP_DOCSISSETINDEX_I199                                 199
#define RSSPECAN_REPCAP_DOCSISSETINDEX_I200                                 200

//repcap DOCSISCodewordIndex
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1                             1
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI2                             2
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI3                             3
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI4                             4
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI5                             5
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI6                             6
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI7                             7
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI8                             8
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI9                             9
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI10                            10
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI11                            11
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI12                            12
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI13                            13
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI14                            14
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI15                            15
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI16                            16
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI17                            17
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI18                            18
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI19                            19
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI20                            20
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI21                            21
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI22                            22
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI23                            23
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI24                            24
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI25                            25
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI26                            26
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI27                            27
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI28                            28
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI29                            29
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI30                            30
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI31                            31
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI32                            32
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI33                            33
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI34                            34
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI35                            35
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI36                            36
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI37                            37
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI38                            38
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI39                            39
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI40                            40
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI41                            41
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI42                            42
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI43                            43
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI44                            44
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI45                            45
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI46                            46
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI47                            47
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI48                            48
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI49                            49
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI50                            50
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI51                            51
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI52                            52
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI53                            53
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI54                            54
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI55                            55
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI56                            56
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI57                            57
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI58                            58
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI59                            59
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI60                            60
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI61                            61
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI62                            62
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI63                            63
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI64                            64
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI65                            65
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI66                            66
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI67                            67
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI68                            68
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI69                            69
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI70                            70
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI71                            71
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI72                            72
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI73                            73
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI74                            74
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI75                            75
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI76                            76
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI77                            77
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI78                            78
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI79                            79
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI80                            80
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI81                            81
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI82                            82
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI83                            83
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI84                            84
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI85                            85
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI86                            86
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI87                            87
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI88                            88
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI89                            89
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI90                            90
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI91                            91
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI92                            92
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI93                            93
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI94                            94
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI95                            95
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI96                            96
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI97                            97
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI98                            98
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI99                            99
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI100                           100
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI101                           101
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI102                           102
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI103                           103
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI104                           104
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI105                           105
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI106                           106
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI107                           107
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI108                           108
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI109                           109
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI110                           110
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI111                           111
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI112                           112
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI113                           113
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI114                           114
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI115                           115
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI116                           116
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI117                           117
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI118                           118
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI119                           119
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI120                           120
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI121                           121
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI122                           122
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI123                           123
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI124                           124
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI125                           125
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI126                           126
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI127                           127
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI128                           128
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI129                           129
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI130                           130
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI131                           131
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI132                           132
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI133                           133
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI134                           134
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI135                           135
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI136                           136
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI137                           137
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI138                           138
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI139                           139
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI140                           140
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI141                           141
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI142                           142
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI143                           143
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI144                           144
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI145                           145
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI146                           146
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI147                           147
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI148                           148
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI149                           149
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI150                           150
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI151                           151
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI152                           152
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI153                           153
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI154                           154
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI155                           155
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI156                           156
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI157                           157
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI158                           158
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI159                           159
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI160                           160
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI161                           161
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI162                           162
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI163                           163
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI164                           164
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI165                           165
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI166                           166
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI167                           167
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI168                           168
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI169                           169
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI170                           170
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI171                           171
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI172                           172
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI173                           173
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI174                           174
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI175                           175
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI176                           176
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI177                           177
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI178                           178
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI179                           179
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI180                           180
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI181                           181
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI182                           182
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI183                           183
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI184                           184
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI185                           185
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI186                           186
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI187                           187
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI188                           188
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI189                           189
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI190                           190
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI191                           191
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI192                           192
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI193                           193
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI194                           194
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI195                           195
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI196                           196
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI197                           197
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI198                           198
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI199                           199
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI200                           200
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI201                           201
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI202                           202
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI203                           203
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI204                           204
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI205                           205
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI206                           206
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI207                           207
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI208                           208
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI209                           209
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI210                           210
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI211                           211
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI212                           212
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI213                           213
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI214                           214
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI215                           215
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI216                           216
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI217                           217
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI218                           218
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI219                           219
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI220                           220
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI221                           221
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI222                           222
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI223                           223
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI224                           224
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI225                           225
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI226                           226
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI227                           227
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI228                           228
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI229                           229
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI230                           230
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI231                           231
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI232                           232
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI233                           233
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI234                           234
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI235                           235
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI236                           236
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI237                           237
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI238                           238
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI239                           239
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI240                           240
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI241                           241
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI242                           242
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI243                           243
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI244                           244
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI245                           245
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI246                           246
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI247                           247
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI248                           248
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI249                           249
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI250                           250
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI251                           251
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI252                           252
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI253                           253
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI254                           254
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI255                           255
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI256                           256
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI257                           257
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI258                           258
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI259                           259
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI260                           260
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI261                           261
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI262                           262
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI263                           263
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI264                           264
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI265                           265
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI266                           266
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI267                           267
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI268                           268
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI269                           269
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI270                           270
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI271                           271
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI272                           272
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI273                           273
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI274                           274
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI275                           275
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI276                           276
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI277                           277
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI278                           278
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI279                           279
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI280                           280
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI281                           281
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI282                           282
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI283                           283
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI284                           284
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI285                           285
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI286                           286
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI287                           287
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI288                           288
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI289                           289
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI290                           290
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI291                           291
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI292                           292
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI293                           293
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI294                           294
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI295                           295
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI296                           296
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI297                           297
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI298                           298
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI299                           299
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI300                           300
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI301                           301
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI302                           302
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI303                           303
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI304                           304
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI305                           305
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI306                           306
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI307                           307
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI308                           308
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI309                           309
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI310                           310
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI311                           311
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI312                           312
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI313                           313
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI314                           314
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI315                           315
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI316                           316
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI317                           317
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI318                           318
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI319                           319
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI320                           320
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI321                           321
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI322                           322
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI323                           323
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI324                           324
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI325                           325
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI326                           326
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI327                           327
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI328                           328
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI329                           329
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI330                           330
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI331                           331
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI332                           332
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI333                           333
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI334                           334
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI335                           335
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI336                           336
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI337                           337
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI338                           338
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI339                           339
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI340                           340
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI341                           341
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI342                           342
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI343                           343
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI344                           344
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI345                           345
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI346                           346
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI347                           347
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI348                           348
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI349                           349
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI350                           350
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI351                           351
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI352                           352
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI353                           353
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI354                           354
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI355                           355
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI356                           356
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI357                           357
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI358                           358
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI359                           359
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI360                           360
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI361                           361
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI362                           362
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI363                           363
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI364                           364
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI365                           365
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI366                           366
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI367                           367
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI368                           368
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI369                           369
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI370                           370
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI371                           371
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI372                           372
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI373                           373
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI374                           374
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI375                           375
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI376                           376
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI377                           377
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI378                           378
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI379                           379
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI380                           380
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI381                           381
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI382                           382
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI383                           383
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI384                           384
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI385                           385
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI386                           386
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI387                           387
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI388                           388
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI389                           389
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI390                           390
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI391                           391
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI392                           392
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI393                           393
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI394                           394
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI395                           395
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI396                           396
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI397                           397
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI398                           398
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI399                           399
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI400                           400
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI401                           401
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI402                           402
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI403                           403
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI404                           404
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI405                           405
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI406                           406
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI407                           407
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI408                           408
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI409                           409
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI410                           410
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI411                           411
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI412                           412
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI413                           413
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI414                           414
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI415                           415
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI416                           416
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI417                           417
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI418                           418
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI419                           419
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI420                           420
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI421                           421
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI422                           422
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI423                           423
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI424                           424
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI425                           425
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI426                           426
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI427                           427
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI428                           428
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI429                           429
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI430                           430
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI431                           431
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI432                           432
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI433                           433
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI434                           434
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI435                           435
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI436                           436
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI437                           437
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI438                           438
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI439                           439
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI440                           440
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI441                           441
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI442                           442
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI443                           443
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI444                           444
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI445                           445
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI446                           446
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI447                           447
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI448                           448
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI449                           449
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI450                           450
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI451                           451
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI452                           452
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI453                           453
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI454                           454
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI455                           455
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI456                           456
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI457                           457
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI458                           458
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI459                           459
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI460                           460
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI461                           461
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI462                           462
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI463                           463
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI464                           464
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI465                           465
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI466                           466
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI467                           467
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI468                           468
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI469                           469
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI470                           470
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI471                           471
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI472                           472
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI473                           473
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI474                           474
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI475                           475
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI476                           476
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI477                           477
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI478                           478
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI479                           479
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI480                           480
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI481                           481
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI482                           482
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI483                           483
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI484                           484
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI485                           485
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI486                           486
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI487                           487
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI488                           488
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI489                           489
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI490                           490
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI491                           491
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI492                           492
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI493                           493
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI494                           494
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI495                           495
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI496                           496
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI497                           497
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI498                           498
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI499                           499
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI500                           500
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI501                           501
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI502                           502
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI503                           503
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI504                           504
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI505                           505
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI506                           506
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI507                           507
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI508                           508
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI509                           509
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI510                           510
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI511                           511
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI512                           512
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI513                           513
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI514                           514
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI515                           515
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI516                           516
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI517                           517
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI518                           518
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI519                           519
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI520                           520
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI521                           521
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI522                           522
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI523                           523
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI524                           524
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI525                           525
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI526                           526
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI527                           527
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI528                           528
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI529                           529
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI530                           530
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI531                           531
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI532                           532
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI533                           533
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI534                           534
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI535                           535
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI536                           536
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI537                           537
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI538                           538
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI539                           539
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI540                           540
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI541                           541
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI542                           542
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI543                           543
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI544                           544
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI545                           545
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI546                           546
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI547                           547
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI548                           548
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI549                           549
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI550                           550
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI551                           551
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI552                           552
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI553                           553
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI554                           554
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI555                           555
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI556                           556
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI557                           557
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI558                           558
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI559                           559
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI560                           560
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI561                           561
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI562                           562
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI563                           563
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI564                           564
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI565                           565
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI566                           566
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI567                           567
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI568                           568
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI569                           569
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI570                           570
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI571                           571
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI572                           572
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI573                           573
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI574                           574
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI575                           575
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI576                           576
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI577                           577
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI578                           578
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI579                           579
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI580                           580
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI581                           581
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI582                           582
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI583                           583
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI584                           584
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI585                           585
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI586                           586
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI587                           587
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI588                           588
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI589                           589
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI590                           590
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI591                           591
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI592                           592
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI593                           593
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI594                           594
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI595                           595
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI596                           596
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI597                           597
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI598                           598
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI599                           599
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI600                           600
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI601                           601
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI602                           602
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI603                           603
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI604                           604
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI605                           605
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI606                           606
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI607                           607
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI608                           608
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI609                           609
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI610                           610
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI611                           611
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI612                           612
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI613                           613
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI614                           614
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI615                           615
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI616                           616
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI617                           617
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI618                           618
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI619                           619
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI620                           620
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI621                           621
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI622                           622
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI623                           623
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI624                           624
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI625                           625
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI626                           626
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI627                           627
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI628                           628
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI629                           629
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI630                           630
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI631                           631
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI632                           632
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI633                           633
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI634                           634
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI635                           635
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI636                           636
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI637                           637
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI638                           638
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI639                           639
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI640                           640
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI641                           641
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI642                           642
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI643                           643
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI644                           644
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI645                           645
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI646                           646
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI647                           647
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI648                           648
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI649                           649
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI650                           650
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI651                           651
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI652                           652
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI653                           653
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI654                           654
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI655                           655
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI656                           656
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI657                           657
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI658                           658
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI659                           659
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI660                           660
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI661                           661
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI662                           662
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI663                           663
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI664                           664
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI665                           665
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI666                           666
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI667                           667
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI668                           668
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI669                           669
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI670                           670
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI671                           671
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI672                           672
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI673                           673
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI674                           674
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI675                           675
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI676                           676
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI677                           677
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI678                           678
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI679                           679
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI680                           680
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI681                           681
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI682                           682
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI683                           683
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI684                           684
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI685                           685
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI686                           686
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI687                           687
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI688                           688
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI689                           689
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI690                           690
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI691                           691
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI692                           692
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI693                           693
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI694                           694
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI695                           695
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI696                           696
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI697                           697
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI698                           698
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI699                           699
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI700                           700
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI701                           701
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI702                           702
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI703                           703
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI704                           704
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI705                           705
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI706                           706
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI707                           707
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI708                           708
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI709                           709
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI710                           710
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI711                           711
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI712                           712
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI713                           713
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI714                           714
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI715                           715
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI716                           716
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI717                           717
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI718                           718
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI719                           719
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI720                           720
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI721                           721
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI722                           722
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI723                           723
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI724                           724
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI725                           725
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI726                           726
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI727                           727
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI728                           728
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI729                           729
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI730                           730
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI731                           731
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI732                           732
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI733                           733
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI734                           734
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI735                           735
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI736                           736
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI737                           737
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI738                           738
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI739                           739
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI740                           740
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI741                           741
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI742                           742
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI743                           743
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI744                           744
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI745                           745
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI746                           746
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI747                           747
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI748                           748
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI749                           749
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI750                           750
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI751                           751
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI752                           752
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI753                           753
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI754                           754
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI755                           755
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI756                           756
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI757                           757
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI758                           758
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI759                           759
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI760                           760
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI761                           761
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI762                           762
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI763                           763
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI764                           764
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI765                           765
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI766                           766
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI767                           767
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI768                           768
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI769                           769
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI770                           770
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI771                           771
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI772                           772
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI773                           773
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI774                           774
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI775                           775
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI776                           776
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI777                           777
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI778                           778
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI779                           779
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI780                           780
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI781                           781
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI782                           782
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI783                           783
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI784                           784
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI785                           785
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI786                           786
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI787                           787
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI788                           788
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI789                           789
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI790                           790
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI791                           791
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI792                           792
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI793                           793
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI794                           794
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI795                           795
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI796                           796
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI797                           797
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI798                           798
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI799                           799
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI800                           800
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI801                           801
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI802                           802
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI803                           803
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI804                           804
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI805                           805
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI806                           806
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI807                           807
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI808                           808
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI809                           809
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI810                           810
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI811                           811
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI812                           812
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI813                           813
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI814                           814
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI815                           815
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI816                           816
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI817                           817
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI818                           818
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI819                           819
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI820                           820
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI821                           821
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI822                           822
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI823                           823
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI824                           824
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI825                           825
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI826                           826
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI827                           827
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI828                           828
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI829                           829
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI830                           830
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI831                           831
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI832                           832
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI833                           833
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI834                           834
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI835                           835
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI836                           836
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI837                           837
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI838                           838
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI839                           839
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI840                           840
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI841                           841
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI842                           842
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI843                           843
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI844                           844
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI845                           845
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI846                           846
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI847                           847
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI848                           848
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI849                           849
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI850                           850
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI851                           851
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI852                           852
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI853                           853
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI854                           854
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI855                           855
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI856                           856
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI857                           857
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI858                           858
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI859                           859
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI860                           860
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI861                           861
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI862                           862
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI863                           863
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI864                           864
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI865                           865
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI866                           866
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI867                           867
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI868                           868
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI869                           869
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI870                           870
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI871                           871
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI872                           872
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI873                           873
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI874                           874
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI875                           875
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI876                           876
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI877                           877
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI878                           878
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI879                           879
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI880                           880
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI881                           881
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI882                           882
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI883                           883
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI884                           884
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI885                           885
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI886                           886
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI887                           887
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI888                           888
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI889                           889
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI890                           890
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI891                           891
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI892                           892
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI893                           893
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI894                           894
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI895                           895
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI896                           896
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI897                           897
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI898                           898
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI899                           899
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI900                           900
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI901                           901
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI902                           902
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI903                           903
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI904                           904
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI905                           905
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI906                           906
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI907                           907
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI908                           908
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI909                           909
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI910                           910
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI911                           911
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI912                           912
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI913                           913
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI914                           914
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI915                           915
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI916                           916
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI917                           917
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI918                           918
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI919                           919
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI920                           920
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI921                           921
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI922                           922
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI923                           923
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI924                           924
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI925                           925
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI926                           926
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI927                           927
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI928                           928
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI929                           929
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI930                           930
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI931                           931
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI932                           932
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI933                           933
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI934                           934
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI935                           935
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI936                           936
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI937                           937
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI938                           938
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI939                           939
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI940                           940
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI941                           941
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI942                           942
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI943                           943
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI944                           944
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI945                           945
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI946                           946
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI947                           947
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI948                           948
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI949                           949
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI950                           950
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI951                           951
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI952                           952
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI953                           953
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI954                           954
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI955                           955
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI956                           956
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI957                           957
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI958                           958
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI959                           959
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI960                           960
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI961                           961
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI962                           962
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI963                           963
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI964                           964
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI965                           965
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI966                           966
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI967                           967
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI968                           968
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI969                           969
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI970                           970
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI971                           971
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI972                           972
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI973                           973
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI974                           974
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI975                           975
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI976                           976
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI977                           977
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI978                           978
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI979                           979
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI980                           980
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI981                           981
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI982                           982
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI983                           983
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI984                           984
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI985                           985
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI986                           986
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI987                           987
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI988                           988
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI989                           989
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI990                           990
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI991                           991
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI992                           992
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI993                           993
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI994                           994
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI995                           995
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI996                           996
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI997                           997
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI998                           998
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI999                           999
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1000                          1000
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1001                          1001
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1002                          1002
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1003                          1003
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1004                          1004
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1005                          1005
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1006                          1006
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1007                          1007
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1008                          1008
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1009                          1009
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1010                          1010
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1011                          1011
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1012                          1012
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1013                          1013
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1014                          1014
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1015                          1015
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1016                          1016
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1017                          1017
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1018                          1018
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1019                          1019
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1020                          1020
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1021                          1021
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1022                          1022
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1023                          1023
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1024                          1024
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1025                          1025
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1026                          1026
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1027                          1027
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1028                          1028
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1029                          1029
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1030                          1030
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1031                          1031
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1032                          1032
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1033                          1033
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1034                          1034
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1035                          1035
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1036                          1036
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1037                          1037
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1038                          1038
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1039                          1039
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1040                          1040
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1041                          1041
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1042                          1042
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1043                          1043
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1044                          1044
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1045                          1045
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1046                          1046
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1047                          1047
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1048                          1048
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1049                          1049
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1050                          1050
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1051                          1051
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1052                          1052
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1053                          1053
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1054                          1054
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1055                          1055
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1056                          1056
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1057                          1057
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1058                          1058
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1059                          1059
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1060                          1060
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1061                          1061
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1062                          1062
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1063                          1063
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1064                          1064
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1065                          1065
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1066                          1066
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1067                          1067
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1068                          1068
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1069                          1069
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1070                          1070
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1071                          1071
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1072                          1072
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1073                          1073
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1074                          1074
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1075                          1075
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1076                          1076
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1077                          1077
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1078                          1078
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1079                          1079
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1080                          1080
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1081                          1081
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1082                          1082
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1083                          1083
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1084                          1084
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1085                          1085
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1086                          1086
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1087                          1087
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1088                          1088
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1089                          1089
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1090                          1090
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1091                          1091
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1092                          1092
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1093                          1093
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1094                          1094
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1095                          1095
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1096                          1096
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1097                          1097
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1098                          1098
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1099                          1099
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1100                          1100
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1101                          1101
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1102                          1102
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1103                          1103
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1104                          1104
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1105                          1105
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1106                          1106
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1107                          1107
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1108                          1108
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1109                          1109
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1110                          1110
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1111                          1111
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1112                          1112
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1113                          1113
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1114                          1114
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1115                          1115
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1116                          1116
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1117                          1117
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1118                          1118
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1119                          1119
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1120                          1120
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1121                          1121
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1122                          1122
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1123                          1123
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1124                          1124
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1125                          1125
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1126                          1126
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1127                          1127
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1128                          1128
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1129                          1129
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1130                          1130
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1131                          1131
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1132                          1132
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1133                          1133
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1134                          1134
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1135                          1135
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1136                          1136
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1137                          1137
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1138                          1138
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1139                          1139
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1140                          1140
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1141                          1141
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1142                          1142
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1143                          1143
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1144                          1144
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1145                          1145
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1146                          1146
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1147                          1147
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1148                          1148
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1149                          1149
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1150                          1150
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1151                          1151
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1152                          1152
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1153                          1153
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1154                          1154
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1155                          1155
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1156                          1156
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1157                          1157
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1158                          1158
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1159                          1159
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1160                          1160
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1161                          1161
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1162                          1162
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1163                          1163
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1164                          1164
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1165                          1165
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1166                          1166
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1167                          1167
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1168                          1168
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1169                          1169
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1170                          1170
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1171                          1171
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1172                          1172
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1173                          1173
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1174                          1174
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1175                          1175
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1176                          1176
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1177                          1177
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1178                          1178
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1179                          1179
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1180                          1180
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1181                          1181
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1182                          1182
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1183                          1183
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1184                          1184
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1185                          1185
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1186                          1186
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1187                          1187
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1188                          1188
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1189                          1189
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1190                          1190
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1191                          1191
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1192                          1192
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1193                          1193
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1194                          1194
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1195                          1195
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1196                          1196
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1197                          1197
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1198                          1198
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1199                          1199
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1200                          1200
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1201                          1201
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1202                          1202
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1203                          1203
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1204                          1204
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1205                          1205
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1206                          1206
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1207                          1207
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1208                          1208
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1209                          1209
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1210                          1210
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1211                          1211
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1212                          1212
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1213                          1213
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1214                          1214
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1215                          1215
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1216                          1216
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1217                          1217
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1218                          1218
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1219                          1219
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1220                          1220
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1221                          1221
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1222                          1222
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1223                          1223
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1224                          1224
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1225                          1225
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1226                          1226
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1227                          1227
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1228                          1228
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1229                          1229
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1230                          1230
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1231                          1231
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1232                          1232
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1233                          1233
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1234                          1234
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1235                          1235
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1236                          1236
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1237                          1237
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1238                          1238
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1239                          1239
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1240                          1240
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1241                          1241
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1242                          1242
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1243                          1243
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1244                          1244
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1245                          1245
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1246                          1246
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1247                          1247
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1248                          1248
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1249                          1249
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1250                          1250
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1251                          1251
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1252                          1252
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1253                          1253
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1254                          1254
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1255                          1255
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1256                          1256
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1257                          1257
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1258                          1258
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1259                          1259
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1260                          1260
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1261                          1261
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1262                          1262
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1263                          1263
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1264                          1264
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1265                          1265
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1266                          1266
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1267                          1267
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1268                          1268
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1269                          1269
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1270                          1270
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1271                          1271
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1272                          1272
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1273                          1273
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1274                          1274
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1275                          1275
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1276                          1276
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1277                          1277
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1278                          1278
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1279                          1279
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1280                          1280
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1281                          1281
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1282                          1282
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1283                          1283
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1284                          1284
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1285                          1285
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1286                          1286
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1287                          1287
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1288                          1288
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1289                          1289
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1290                          1290
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1291                          1291
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1292                          1292
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1293                          1293
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1294                          1294
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1295                          1295
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1296                          1296
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1297                          1297
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1298                          1298
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1299                          1299
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1300                          1300
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1301                          1301
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1302                          1302
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1303                          1303
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1304                          1304
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1305                          1305
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1306                          1306
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1307                          1307
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1308                          1308
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1309                          1309
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1310                          1310
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1311                          1311
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1312                          1312
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1313                          1313
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1314                          1314
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1315                          1315
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1316                          1316
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1317                          1317
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1318                          1318
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1319                          1319
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1320                          1320
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1321                          1321
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1322                          1322
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1323                          1323
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1324                          1324
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1325                          1325
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1326                          1326
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1327                          1327
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1328                          1328
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1329                          1329
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1330                          1330
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1331                          1331
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1332                          1332
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1333                          1333
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1334                          1334
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1335                          1335
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1336                          1336
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1337                          1337
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1338                          1338
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1339                          1339
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1340                          1340
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1341                          1341
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1342                          1342
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1343                          1343
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1344                          1344
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1345                          1345
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1346                          1346
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1347                          1347
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1348                          1348
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1349                          1349
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1350                          1350
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1351                          1351
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1352                          1352
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1353                          1353
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1354                          1354
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1355                          1355
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1356                          1356
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1357                          1357
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1358                          1358
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1359                          1359
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1360                          1360
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1361                          1361
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1362                          1362
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1363                          1363
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1364                          1364
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1365                          1365
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1366                          1366
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1367                          1367
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1368                          1368
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1369                          1369
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1370                          1370
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1371                          1371
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1372                          1372
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1373                          1373
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1374                          1374
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1375                          1375
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1376                          1376
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1377                          1377
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1378                          1378
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1379                          1379
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1380                          1380
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1381                          1381
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1382                          1382
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1383                          1383
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1384                          1384
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1385                          1385
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1386                          1386
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1387                          1387
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1388                          1388
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1389                          1389
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1390                          1390
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1391                          1391
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1392                          1392
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1393                          1393
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1394                          1394
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1395                          1395
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1396                          1396
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1397                          1397
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1398                          1398
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1399                          1399
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1400                          1400
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1401                          1401
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1402                          1402
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1403                          1403
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1404                          1404
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1405                          1405
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1406                          1406
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1407                          1407
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1408                          1408
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1409                          1409
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1410                          1410
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1411                          1411
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1412                          1412
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1413                          1413
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1414                          1414
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1415                          1415
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1416                          1416
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1417                          1417
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1418                          1418
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1419                          1419
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1420                          1420
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1421                          1421
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1422                          1422
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1423                          1423
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1424                          1424
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1425                          1425
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1426                          1426
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1427                          1427
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1428                          1428
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1429                          1429
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1430                          1430
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1431                          1431
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1432                          1432
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1433                          1433
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1434                          1434
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1435                          1435
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1436                          1436
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1437                          1437
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1438                          1438
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1439                          1439
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1440                          1440
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1441                          1441
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1442                          1442
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1443                          1443
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1444                          1444
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1445                          1445
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1446                          1446
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1447                          1447
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1448                          1448
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1449                          1449
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1450                          1450
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1451                          1451
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1452                          1452
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1453                          1453
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1454                          1454
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1455                          1455
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1456                          1456
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1457                          1457
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1458                          1458
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1459                          1459
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1460                          1460
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1461                          1461
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1462                          1462
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1463                          1463
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1464                          1464
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1465                          1465
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1466                          1466
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1467                          1467
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1468                          1468
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1469                          1469
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1470                          1470
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1471                          1471
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1472                          1472
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1473                          1473
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1474                          1474
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1475                          1475
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1476                          1476
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1477                          1477
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1478                          1478
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1479                          1479
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1480                          1480
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1481                          1481
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1482                          1482
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1483                          1483
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1484                          1484
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1485                          1485
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1486                          1486
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1487                          1487
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1488                          1488
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1489                          1489
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1490                          1490
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1491                          1491
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1492                          1492
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1493                          1493
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1494                          1494
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1495                          1495
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1496                          1496
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1497                          1497
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1498                          1498
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1499                          1499
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1500                          1500
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1501                          1501
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1502                          1502
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1503                          1503
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1504                          1504
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1505                          1505
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1506                          1506
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1507                          1507
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1508                          1508
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1509                          1509
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1510                          1510
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1511                          1511
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1512                          1512
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1513                          1513
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1514                          1514
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1515                          1515
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1516                          1516
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1517                          1517
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1518                          1518
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1519                          1519
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1520                          1520
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1521                          1521
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1522                          1522
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1523                          1523
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1524                          1524
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1525                          1525
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1526                          1526
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1527                          1527
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1528                          1528
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1529                          1529
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1530                          1530
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1531                          1531
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1532                          1532
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1533                          1533
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1534                          1534
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1535                          1535
#define RSSPECAN_REPCAP_DOCSISCODEWORDINDEX_CI1536                          1536

//repcap LTECluster
#define RSSPECAN_REPCAP_LTECLUSTER_CLUSTER1                                 1
#define RSSPECAN_REPCAP_LTECLUSTER_CLUSTER2                                 2

//repcap IntRange
#define RSSPECAN_REPCAP_INTRANGE_IRN1                                       1
#define RSSPECAN_REPCAP_INTRANGE_IRN2                                       2
#define RSSPECAN_REPCAP_INTRANGE_IRN3                                       3
#define RSSPECAN_REPCAP_INTRANGE_IRN4                                       4

//repcap DOCSISBand
#define RSSPECAN_REPCAP_DOCSISBAND_DBAND1                                   1
#define RSSPECAN_REPCAP_DOCSISBAND_DBAND2                                   2
#define RSSPECAN_REPCAP_DOCSISBAND_DBAND3                                   3
#define RSSPECAN_REPCAP_DOCSISBAND_DBAND4                                   4
#define RSSPECAN_REPCAP_DOCSISBAND_DBAND5                                   5
#define RSSPECAN_REPCAP_DOCSISBAND_DBAND6                                   6
#define RSSPECAN_REPCAP_DOCSISBAND_DBAND7                                   7
#define RSSPECAN_REPCAP_DOCSISBAND_DBAND8                                   8
#define RSSPECAN_REPCAP_DOCSISBAND_DBAND9                                   9
#define RSSPECAN_REPCAP_DOCSISBAND_DBAND10                                  10
#define RSSPECAN_REPCAP_DOCSISBAND_DBAND11                                  11
#define RSSPECAN_REPCAP_DOCSISBAND_DBAND12                                  12
#define RSSPECAN_REPCAP_DOCSISBAND_DBAND13                                  13
#define RSSPECAN_REPCAP_DOCSISBAND_DBAND14                                  14
#define RSSPECAN_REPCAP_DOCSISBAND_DBAND15                                  15
#define RSSPECAN_REPCAP_DOCSISBAND_DBAND16                                  16
#define RSSPECAN_REPCAP_DOCSISBAND_DBAND17                                  17
#define RSSPECAN_REPCAP_DOCSISBAND_DBAND18                                  18
#define RSSPECAN_REPCAP_DOCSISBAND_DBAND19                                  19
#define RSSPECAN_REPCAP_DOCSISBAND_DBAND20                                  20

//repcap WLANaxSegment
#define RSSPECAN_REPCAP_WLANAXSEGMENT_WLANSEG1                              1
#define RSSPECAN_REPCAP_WLANAXSEGMENT_WLANSEG2                              2

//repcap WLANaxChannel
#define RSSPECAN_REPCAP_WLANAXCHANNEL_WLANCHANNEL1                          1
#define RSSPECAN_REPCAP_WLANAXCHANNEL_WLANCHANNEL2                          2
#define RSSPECAN_REPCAP_WLANAXCHANNEL_WLANCHANNEL3                          3
#define RSSPECAN_REPCAP_WLANAXCHANNEL_WLANCHANNEL4                          4
#define RSSPECAN_REPCAP_WLANAXCHANNEL_WLANCHANNEL5                          5
#define RSSPECAN_REPCAP_WLANAXCHANNEL_WLANCHANNEL6                          6
#define RSSPECAN_REPCAP_WLANAXCHANNEL_WLANCHANNEL7                          7
#define RSSPECAN_REPCAP_WLANAXCHANNEL_WLANCHANNEL8                          8

//repcap WLANaxRUIndex
#define RSSPECAN_REPCAP_WLANAXRUINDEX_WLANRUINDEX1                          1
#define RSSPECAN_REPCAP_WLANAXRUINDEX_WLANRUINDEX2                          2
#define RSSPECAN_REPCAP_WLANAXRUINDEX_WLANRUINDEX3                          3
#define RSSPECAN_REPCAP_WLANAXRUINDEX_WLANRUINDEX4                          4
#define RSSPECAN_REPCAP_WLANAXRUINDEX_WLANRUINDEX5                          5
#define RSSPECAN_REPCAP_WLANAXRUINDEX_WLANRUINDEX6                          6
#define RSSPECAN_REPCAP_WLANAXRUINDEX_WLANRUINDEX7                          7
#define RSSPECAN_REPCAP_WLANAXRUINDEX_WLANRUINDEX8                          8
#define RSSPECAN_REPCAP_WLANAXRUINDEX_WLANRUINDEX9                          9
#define RSSPECAN_REPCAP_WLANAXRUINDEX_WLANRUINDEX10                         10
#define RSSPECAN_REPCAP_WLANAXRUINDEX_WLANRUINDEX11                         11
#define RSSPECAN_REPCAP_WLANAXRUINDEX_WLANRUINDEX12                         12
#define RSSPECAN_REPCAP_WLANAXRUINDEX_WLANRUINDEX13                         13
#define RSSPECAN_REPCAP_WLANAXRUINDEX_WLANRUINDEX14                         14
#define RSSPECAN_REPCAP_WLANAXRUINDEX_WLANRUINDEX15                         15
#define RSSPECAN_REPCAP_WLANAXRUINDEX_WLANRUINDEX16                         16
#define RSSPECAN_REPCAP_WLANAXRUINDEX_WLANRUINDEX17                         17
#define RSSPECAN_REPCAP_WLANAXRUINDEX_WLANRUINDEX18                         18
#define RSSPECAN_REPCAP_WLANAXRUINDEX_WLANRUINDEX19                         19
#define RSSPECAN_REPCAP_WLANAXRUINDEX_WLANRUINDEX20                         20
#define RSSPECAN_REPCAP_WLANAXRUINDEX_WLANRUINDEX21                         21
#define RSSPECAN_REPCAP_WLANAXRUINDEX_WLANRUINDEX22                         22
#define RSSPECAN_REPCAP_WLANAXRUINDEX_WLANRUINDEX23                         23
#define RSSPECAN_REPCAP_WLANAXRUINDEX_WLANRUINDEX24                         24
#define RSSPECAN_REPCAP_WLANAXRUINDEX_WLANRUINDEX25                         25
#define RSSPECAN_REPCAP_WLANAXRUINDEX_WLANRUINDEX26                         26
#define RSSPECAN_REPCAP_WLANAXRUINDEX_WLANRUINDEX27                         27
#define RSSPECAN_REPCAP_WLANAXRUINDEX_WLANRUINDEX28                         28
#define RSSPECAN_REPCAP_WLANAXRUINDEX_WLANRUINDEX29                         29
#define RSSPECAN_REPCAP_WLANAXRUINDEX_WLANRUINDEX30                         30
#define RSSPECAN_REPCAP_WLANAXRUINDEX_WLANRUINDEX31                         31
#define RSSPECAN_REPCAP_WLANAXRUINDEX_WLANRUINDEX32                         32
#define RSSPECAN_REPCAP_WLANAXRUINDEX_WLANRUINDEX33                         33
#define RSSPECAN_REPCAP_WLANAXRUINDEX_WLANRUINDEX34                         34
#define RSSPECAN_REPCAP_WLANAXRUINDEX_WLANRUINDEX35                         35
#define RSSPECAN_REPCAP_WLANAXRUINDEX_WLANRUINDEX36                         36
#define RSSPECAN_REPCAP_WLANAXRUINDEX_WLANRUINDEX37                         37
#define RSSPECAN_REPCAP_WLANAXRUINDEX_WLANRUINDEX38                         38
#define RSSPECAN_REPCAP_WLANAXRUINDEX_WLANRUINDEX39                         39
#define RSSPECAN_REPCAP_WLANAXRUINDEX_WLANRUINDEX40                         40
#define RSSPECAN_REPCAP_WLANAXRUINDEX_WLANRUINDEX41                         41
#define RSSPECAN_REPCAP_WLANAXRUINDEX_WLANRUINDEX42                         42
#define RSSPECAN_REPCAP_WLANAXRUINDEX_WLANRUINDEX43                         43
#define RSSPECAN_REPCAP_WLANAXRUINDEX_WLANRUINDEX44                         44
#define RSSPECAN_REPCAP_WLANAXRUINDEX_WLANRUINDEX45                         45
#define RSSPECAN_REPCAP_WLANAXRUINDEX_WLANRUINDEX46                         46
#define RSSPECAN_REPCAP_WLANAXRUINDEX_WLANRUINDEX47                         47
#define RSSPECAN_REPCAP_WLANAXRUINDEX_WLANRUINDEX48                         48
#define RSSPECAN_REPCAP_WLANAXRUINDEX_WLANRUINDEX49                         49
#define RSSPECAN_REPCAP_WLANAXRUINDEX_WLANRUINDEX50                         50
#define RSSPECAN_REPCAP_WLANAXRUINDEX_WLANRUINDEX51                         51
#define RSSPECAN_REPCAP_WLANAXRUINDEX_WLANRUINDEX52                         52
#define RSSPECAN_REPCAP_WLANAXRUINDEX_WLANRUINDEX53                         53
#define RSSPECAN_REPCAP_WLANAXRUINDEX_WLANRUINDEX54                         54
#define RSSPECAN_REPCAP_WLANAXRUINDEX_WLANRUINDEX55                         55
#define RSSPECAN_REPCAP_WLANAXRUINDEX_WLANRUINDEX56                         56
#define RSSPECAN_REPCAP_WLANAXRUINDEX_WLANRUINDEX57                         57
#define RSSPECAN_REPCAP_WLANAXRUINDEX_WLANRUINDEX58                         58
#define RSSPECAN_REPCAP_WLANAXRUINDEX_WLANRUINDEX59                         59
#define RSSPECAN_REPCAP_WLANAXRUINDEX_WLANRUINDEX60                         60
#define RSSPECAN_REPCAP_WLANAXRUINDEX_WLANRUINDEX61                         61
#define RSSPECAN_REPCAP_WLANAXRUINDEX_WLANRUINDEX62                         62
#define RSSPECAN_REPCAP_WLANAXRUINDEX_WLANRUINDEX63                         63
#define RSSPECAN_REPCAP_WLANAXRUINDEX_WLANRUINDEX64                         64
#define RSSPECAN_REPCAP_WLANAXRUINDEX_WLANRUINDEX65                         65
#define RSSPECAN_REPCAP_WLANAXRUINDEX_WLANRUINDEX66                         66
#define RSSPECAN_REPCAP_WLANAXRUINDEX_WLANRUINDEX67                         67
#define RSSPECAN_REPCAP_WLANAXRUINDEX_WLANRUINDEX68                         68
#define RSSPECAN_REPCAP_WLANAXRUINDEX_WLANRUINDEX69                         69
#define RSSPECAN_REPCAP_WLANAXRUINDEX_WLANRUINDEX70                         70
#define RSSPECAN_REPCAP_WLANAXRUINDEX_WLANRUINDEX71                         71
#define RSSPECAN_REPCAP_WLANAXRUINDEX_WLANRUINDEX72                         72

//repcap WLANaxUserIndex
#define RSSPECAN_REPCAP_WLANAXUSERINDEX_WLANUSERINDEX1                      1
#define RSSPECAN_REPCAP_WLANAXUSERINDEX_WLANUSERINDEX2                      2
#define RSSPECAN_REPCAP_WLANAXUSERINDEX_WLANUSERINDEX3                      3
#define RSSPECAN_REPCAP_WLANAXUSERINDEX_WLANUSERINDEX4                      4
#define RSSPECAN_REPCAP_WLANAXUSERINDEX_WLANUSERINDEX5                      5
#define RSSPECAN_REPCAP_WLANAXUSERINDEX_WLANUSERINDEX6                      6
#define RSSPECAN_REPCAP_WLANAXUSERINDEX_WLANUSERINDEX7                      7
#define RSSPECAN_REPCAP_WLANAXUSERINDEX_WLANUSERINDEX8                      8

//repcap Flist
#define RSSPECAN_REPCAP_FLIST_FLIST1                                        1
#define RSSPECAN_REPCAP_FLIST_FLIST2                                        2
#define RSSPECAN_REPCAP_FLIST_FLIST3                                        3
#define RSSPECAN_REPCAP_FLIST_FLIST4                                        4
#define RSSPECAN_REPCAP_FLIST_FLIST5                                        5
#define RSSPECAN_REPCAP_FLIST_FLIST6                                        6
#define RSSPECAN_REPCAP_FLIST_FLIST7                                        7
#define RSSPECAN_REPCAP_FLIST_FLIST8                                        8
#define RSSPECAN_REPCAP_FLIST_FLIST9                                        9
#define RSSPECAN_REPCAP_FLIST_FLIST10                                       10
#define RSSPECAN_REPCAP_FLIST_FLIST11                                       11
#define RSSPECAN_REPCAP_FLIST_FLIST12                                       12
#define RSSPECAN_REPCAP_FLIST_FLIST13                                       13
#define RSSPECAN_REPCAP_FLIST_FLIST14                                       14
#define RSSPECAN_REPCAP_FLIST_FLIST15                                       15

//repcap Slist
#define RSSPECAN_REPCAP_SLIST_SLIST1                                        1
#define RSSPECAN_REPCAP_SLIST_SLIST2                                        2
#define RSSPECAN_REPCAP_SLIST_SLIST3                                        3
#define RSSPECAN_REPCAP_SLIST_SLIST4                                        4
#define RSSPECAN_REPCAP_SLIST_SLIST5                                        5
#define RSSPECAN_REPCAP_SLIST_SLIST6                                        6
#define RSSPECAN_REPCAP_SLIST_SLIST7                                        7
#define RSSPECAN_REPCAP_SLIST_SLIST8                                        8
#define RSSPECAN_REPCAP_SLIST_SLIST9                                        9
#define RSSPECAN_REPCAP_SLIST_SLIST10                                       10
#define RSSPECAN_REPCAP_SLIST_SLIST11                                       11
#define RSSPECAN_REPCAP_SLIST_SLIST12                                       12
#define RSSPECAN_REPCAP_SLIST_SLIST13                                       13
#define RSSPECAN_REPCAP_SLIST_SLIST14                                       14
#define RSSPECAN_REPCAP_SLIST_SLIST15                                       15

//repcap NPRNotch
#define RSSPECAN_REPCAP_NPRNOTCH_NPRNOTCH1                                  1
#define RSSPECAN_REPCAP_NPRNOTCH_NPRNOTCH2                                  2
#define RSSPECAN_REPCAP_NPRNOTCH_NPRNOTCH3                                  3
#define RSSPECAN_REPCAP_NPRNOTCH_NPRNOTCH4                                  4
#define RSSPECAN_REPCAP_NPRNOTCH_NPRNOTCH5                                  5
#define RSSPECAN_REPCAP_NPRNOTCH_NPRNOTCH6                                  6
#define RSSPECAN_REPCAP_NPRNOTCH_NPRNOTCH7                                  7
#define RSSPECAN_REPCAP_NPRNOTCH_NPRNOTCH8                                  8
#define RSSPECAN_REPCAP_NPRNOTCH_NPRNOTCH9                                  9
#define RSSPECAN_REPCAP_NPRNOTCH_NPRNOTCH10                                 10
#define RSSPECAN_REPCAP_NPRNOTCH_NPRNOTCH11                                 11
#define RSSPECAN_REPCAP_NPRNOTCH_NPRNOTCH12                                 12
#define RSSPECAN_REPCAP_NPRNOTCH_NPRNOTCH13                                 13
#define RSSPECAN_REPCAP_NPRNOTCH_NPRNOTCH14                                 14
#define RSSPECAN_REPCAP_NPRNOTCH_NPRNOTCH15                                 15
#define RSSPECAN_REPCAP_NPRNOTCH_NPRNOTCH16                                 16
#define RSSPECAN_REPCAP_NPRNOTCH_NPRNOTCH17                                 17
#define RSSPECAN_REPCAP_NPRNOTCH_NPRNOTCH18                                 18
#define RSSPECAN_REPCAP_NPRNOTCH_NPRNOTCH19                                 19
#define RSSPECAN_REPCAP_NPRNOTCH_NPRNOTCH20                                 20
#define RSSPECAN_REPCAP_NPRNOTCH_NPRNOTCH21                                 21
#define RSSPECAN_REPCAP_NPRNOTCH_NPRNOTCH22                                 22
#define RSSPECAN_REPCAP_NPRNOTCH_NPRNOTCH23                                 23
#define RSSPECAN_REPCAP_NPRNOTCH_NPRNOTCH24                                 24
#define RSSPECAN_REPCAP_NPRNOTCH_NPRNOTCH25                                 25

//repcap UserCoupling
#define RSSPECAN_REPCAP_USERCOUPLING_UC1                                    1
#define RSSPECAN_REPCAP_USERCOUPLING_UC2                                    2
#define RSSPECAN_REPCAP_USERCOUPLING_UC3                                    3
#define RSSPECAN_REPCAP_USERCOUPLING_UC4                                    4
#define RSSPECAN_REPCAP_USERCOUPLING_UC5                                    5
#define RSSPECAN_REPCAP_USERCOUPLING_UC6                                    6
#define RSSPECAN_REPCAP_USERCOUPLING_UC7                                    7
#define RSSPECAN_REPCAP_USERCOUPLING_UC8                                    8
#define RSSPECAN_REPCAP_USERCOUPLING_UC9                                    9
#define RSSPECAN_REPCAP_USERCOUPLING_UC10                                   10
#define RSSPECAN_REPCAP_USERCOUPLING_UC11                                   11
#define RSSPECAN_REPCAP_USERCOUPLING_UC12                                   12
#define RSSPECAN_REPCAP_USERCOUPLING_UC13                                   13
#define RSSPECAN_REPCAP_USERCOUPLING_UC14                                   14
#define RSSPECAN_REPCAP_USERCOUPLING_UC15                                   15
#define RSSPECAN_REPCAP_USERCOUPLING_UC16                                   16
#define RSSPECAN_REPCAP_USERCOUPLING_UC17                                   17
#define RSSPECAN_REPCAP_USERCOUPLING_UC18                                   18
#define RSSPECAN_REPCAP_USERCOUPLING_UC19                                   19
#define RSSPECAN_REPCAP_USERCOUPLING_UC20                                   20

//repcap cc
#define RSSPECAN_REPCAP_CC_CC1                                              1
#define RSSPECAN_REPCAP_CC_CC2                                              2
#define RSSPECAN_REPCAP_CC_CC3                                              3
#define RSSPECAN_REPCAP_CC_CC4                                              4
#define RSSPECAN_REPCAP_CC_CC5                                              5
#define RSSPECAN_REPCAP_CC_CC6                                              6
#define RSSPECAN_REPCAP_CC_CC7                                              7
#define RSSPECAN_REPCAP_CC_CC8                                              8
#define RSSPECAN_REPCAP_CC_CC9                                              9
#define RSSPECAN_REPCAP_CC_CC10                                             10
#define RSSPECAN_REPCAP_CC_CC11                                             11
#define RSSPECAN_REPCAP_CC_CC12                                             12
#define RSSPECAN_REPCAP_CC_CC13                                             13
#define RSSPECAN_REPCAP_CC_CC14                                             14
#define RSSPECAN_REPCAP_CC_CC15                                             15
#define RSSPECAN_REPCAP_CC_CC16                                             16

//repcap BWPart
#define RSSPECAN_REPCAP_BWPART_BWPART0                                      0
#define RSSPECAN_REPCAP_BWPART_BWPART1                                      1
#define RSSPECAN_REPCAP_BWPART_BWPART2                                      2
#define RSSPECAN_REPCAP_BWPART_BWPART3                                      3
#define RSSPECAN_REPCAP_BWPART_BWPART4                                      4
#define RSSPECAN_REPCAP_BWPART_BWPART5                                      5
#define RSSPECAN_REPCAP_BWPART_BWPART6                                      6
#define RSSPECAN_REPCAP_BWPART_BWPART7                                      7
#define RSSPECAN_REPCAP_BWPART_BWPART8                                      8
#define RSSPECAN_REPCAP_BWPART_BWPART9                                      9
#define RSSPECAN_REPCAP_BWPART_BWPART10                                     10
#define RSSPECAN_REPCAP_BWPART_BWPART11                                     11

//repcap NR5GFrame
#define RSSPECAN_REPCAP_NR5GFRAME_NR5GFRAME1                                1
#define RSSPECAN_REPCAP_NR5GFRAME_NR5GFRAME2                                2
#define RSSPECAN_REPCAP_NR5GFRAME_NR5GFRAME3                                3
#define RSSPECAN_REPCAP_NR5GFRAME_NR5GFRAME4                                4
#define RSSPECAN_REPCAP_NR5GFRAME_NR5GFRAME5                                5

//repcap NR5GSlot
#define RSSPECAN_REPCAP_NR5GSLOT_NR5GSLOT0                                  0
#define RSSPECAN_REPCAP_NR5GSLOT_NR5GSLOT1                                  1
#define RSSPECAN_REPCAP_NR5GSLOT_NR5GSLOT2                                  2
#define RSSPECAN_REPCAP_NR5GSLOT_NR5GSLOT3                                  3
#define RSSPECAN_REPCAP_NR5GSLOT_NR5GSLOT4                                  4
#define RSSPECAN_REPCAP_NR5GSLOT_NR5GSLOT5                                  5
#define RSSPECAN_REPCAP_NR5GSLOT_NR5GSLOT6                                  6
#define RSSPECAN_REPCAP_NR5GSLOT_NR5GSLOT7                                  7
#define RSSPECAN_REPCAP_NR5GSLOT_NR5GSLOT8                                  8
#define RSSPECAN_REPCAP_NR5GSLOT_NR5GSLOT9                                  9
#define RSSPECAN_REPCAP_NR5GSLOT_NR5GSLOT10                                 10
#define RSSPECAN_REPCAP_NR5GSLOT_NR5GSLOT11                                 11
#define RSSPECAN_REPCAP_NR5GSLOT_NR5GSLOT12                                 12
#define RSSPECAN_REPCAP_NR5GSLOT_NR5GSLOT13                                 13
#define RSSPECAN_REPCAP_NR5GSLOT_NR5GSLOT14                                 14
#define RSSPECAN_REPCAP_NR5GSLOT_NR5GSLOT15                                 15
#define RSSPECAN_REPCAP_NR5GSLOT_NR5GSLOT16                                 16
#define RSSPECAN_REPCAP_NR5GSLOT_NR5GSLOT17                                 17
#define RSSPECAN_REPCAP_NR5GSLOT_NR5GSLOT18                                 18
#define RSSPECAN_REPCAP_NR5GSLOT_NR5GSLOT19                                 19
#define RSSPECAN_REPCAP_NR5GSLOT_NR5GSLOT20                                 20
#define RSSPECAN_REPCAP_NR5GSLOT_NR5GSLOT21                                 21
#define RSSPECAN_REPCAP_NR5GSLOT_NR5GSLOT22                                 22
#define RSSPECAN_REPCAP_NR5GSLOT_NR5GSLOT23                                 23
#define RSSPECAN_REPCAP_NR5GSLOT_NR5GSLOT24                                 24
#define RSSPECAN_REPCAP_NR5GSLOT_NR5GSLOT25                                 25
#define RSSPECAN_REPCAP_NR5GSLOT_NR5GSLOT26                                 26
#define RSSPECAN_REPCAP_NR5GSLOT_NR5GSLOT27                                 27
#define RSSPECAN_REPCAP_NR5GSLOT_NR5GSLOT28                                 28
#define RSSPECAN_REPCAP_NR5GSLOT_NR5GSLOT29                                 29
#define RSSPECAN_REPCAP_NR5GSLOT_NR5GSLOT30                                 30
#define RSSPECAN_REPCAP_NR5GSLOT_NR5GSLOT31                                 31
#define RSSPECAN_REPCAP_NR5GSLOT_NR5GSLOT32                                 32
#define RSSPECAN_REPCAP_NR5GSLOT_NR5GSLOT33                                 33
#define RSSPECAN_REPCAP_NR5GSLOT_NR5GSLOT34                                 34
#define RSSPECAN_REPCAP_NR5GSLOT_NR5GSLOT35                                 35
#define RSSPECAN_REPCAP_NR5GSLOT_NR5GSLOT36                                 36
#define RSSPECAN_REPCAP_NR5GSLOT_NR5GSLOT37                                 37
#define RSSPECAN_REPCAP_NR5GSLOT_NR5GSLOT38                                 38
#define RSSPECAN_REPCAP_NR5GSLOT_NR5GSLOT39                                 39

//repcap NR5GAlloc
#define RSSPECAN_REPCAP_NR5GALLOC_ALLOC0                                    0
#define RSSPECAN_REPCAP_NR5GALLOC_ALLOC1                                    1
#define RSSPECAN_REPCAP_NR5GALLOC_ALLOC2                                    2
#define RSSPECAN_REPCAP_NR5GALLOC_ALLOC3                                    3
#define RSSPECAN_REPCAP_NR5GALLOC_ALLOC4                                    4
#define RSSPECAN_REPCAP_NR5GALLOC_ALLOC5                                    5
#define RSSPECAN_REPCAP_NR5GALLOC_ALLOC6                                    6
#define RSSPECAN_REPCAP_NR5GALLOC_ALLOC7                                    7
#define RSSPECAN_REPCAP_NR5GALLOC_ALLOC8                                    8
#define RSSPECAN_REPCAP_NR5GALLOC_ALLOC9                                    9
#define RSSPECAN_REPCAP_NR5GALLOC_ALLOC10                                   10
#define RSSPECAN_REPCAP_NR5GALLOC_ALLOC11                                   11
#define RSSPECAN_REPCAP_NR5GALLOC_ALLOC12                                   12
#define RSSPECAN_REPCAP_NR5GALLOC_ALLOC13                                   13
#define RSSPECAN_REPCAP_NR5GALLOC_ALLOC14                                   14
#define RSSPECAN_REPCAP_NR5GALLOC_ALLOC15                                   15
#define RSSPECAN_REPCAP_NR5GALLOC_ALLOC16                                   16
#define RSSPECAN_REPCAP_NR5GALLOC_ALLOC17                                   17
#define RSSPECAN_REPCAP_NR5GALLOC_ALLOC18                                   18
#define RSSPECAN_REPCAP_NR5GALLOC_ALLOC19                                   19
#define RSSPECAN_REPCAP_NR5GALLOC_ALLOC20                                   20
#define RSSPECAN_REPCAP_NR5GALLOC_ALLOC21                                   21
#define RSSPECAN_REPCAP_NR5GALLOC_ALLOC22                                   22
#define RSSPECAN_REPCAP_NR5GALLOC_ALLOC23                                   23
#define RSSPECAN_REPCAP_NR5GALLOC_ALLOC24                                   24
#define RSSPECAN_REPCAP_NR5GALLOC_ALLOC25                                   25
#define RSSPECAN_REPCAP_NR5GALLOC_ALLOC26                                   26
#define RSSPECAN_REPCAP_NR5GALLOC_ALLOC27                                   27
#define RSSPECAN_REPCAP_NR5GALLOC_ALLOC28                                   28
#define RSSPECAN_REPCAP_NR5GALLOC_ALLOC29                                   29
#define RSSPECAN_REPCAP_NR5GALLOC_ALLOC30                                   30
#define RSSPECAN_REPCAP_NR5GALLOC_ALLOC31                                   31
#define RSSPECAN_REPCAP_NR5GALLOC_ALLOC32                                   32
#define RSSPECAN_REPCAP_NR5GALLOC_ALLOC33                                   33
#define RSSPECAN_REPCAP_NR5GALLOC_ALLOC34                                   34
#define RSSPECAN_REPCAP_NR5GALLOC_ALLOC35                                   35
#define RSSPECAN_REPCAP_NR5GALLOC_ALLOC36                                   36
#define RSSPECAN_REPCAP_NR5GALLOC_ALLOC37                                   37
#define RSSPECAN_REPCAP_NR5GALLOC_ALLOC38                                   38
#define RSSPECAN_REPCAP_NR5GALLOC_ALLOC39                                   39
#define RSSPECAN_REPCAP_NR5GALLOC_ALLOC40                                   40
#define RSSPECAN_REPCAP_NR5GALLOC_ALLOC41                                   41
#define RSSPECAN_REPCAP_NR5GALLOC_ALLOC42                                   42
#define RSSPECAN_REPCAP_NR5GALLOC_ALLOC43                                   43
#define RSSPECAN_REPCAP_NR5GALLOC_ALLOC44                                   44
#define RSSPECAN_REPCAP_NR5GALLOC_ALLOC45                                   45
#define RSSPECAN_REPCAP_NR5GALLOC_ALLOC46                                   46
#define RSSPECAN_REPCAP_NR5GALLOC_ALLOC47                                   47
#define RSSPECAN_REPCAP_NR5GALLOC_ALLOC48                                   48
#define RSSPECAN_REPCAP_NR5GALLOC_ALLOC49                                   49
#define RSSPECAN_REPCAP_NR5GALLOC_ALLOC50                                   50
#define RSSPECAN_REPCAP_NR5GALLOC_ALLOC51                                   51
#define RSSPECAN_REPCAP_NR5GALLOC_ALLOC52                                   52
#define RSSPECAN_REPCAP_NR5GALLOC_ALLOC53                                   53
#define RSSPECAN_REPCAP_NR5GALLOC_ALLOC54                                   54
#define RSSPECAN_REPCAP_NR5GALLOC_ALLOC55                                   55
#define RSSPECAN_REPCAP_NR5GALLOC_ALLOC56                                   56
#define RSSPECAN_REPCAP_NR5GALLOC_ALLOC57                                   57
#define RSSPECAN_REPCAP_NR5GALLOC_ALLOC58                                   58
#define RSSPECAN_REPCAP_NR5GALLOC_ALLOC59                                   59
#define RSSPECAN_REPCAP_NR5GALLOC_ALLOC60                                   60
#define RSSPECAN_REPCAP_NR5GALLOC_ALLOC61                                   61
#define RSSPECAN_REPCAP_NR5GALLOC_ALLOC62                                   62
#define RSSPECAN_REPCAP_NR5GALLOC_ALLOC63                                   63
#define RSSPECAN_REPCAP_NR5GALLOC_ALLOC64                                   64
#define RSSPECAN_REPCAP_NR5GALLOC_ALLOC65                                   65
#define RSSPECAN_REPCAP_NR5GALLOC_ALLOC66                                   66
#define RSSPECAN_REPCAP_NR5GALLOC_ALLOC67                                   67
#define RSSPECAN_REPCAP_NR5GALLOC_ALLOC68                                   68
#define RSSPECAN_REPCAP_NR5GALLOC_ALLOC69                                   69
#define RSSPECAN_REPCAP_NR5GALLOC_ALLOC70                                   70
#define RSSPECAN_REPCAP_NR5GALLOC_ALLOC71                                   71
#define RSSPECAN_REPCAP_NR5GALLOC_ALLOC72                                   72
#define RSSPECAN_REPCAP_NR5GALLOC_ALLOC73                                   73
#define RSSPECAN_REPCAP_NR5GALLOC_ALLOC74                                   74
#define RSSPECAN_REPCAP_NR5GALLOC_ALLOC75                                   75
#define RSSPECAN_REPCAP_NR5GALLOC_ALLOC76                                   76
#define RSSPECAN_REPCAP_NR5GALLOC_ALLOC77                                   77
#define RSSPECAN_REPCAP_NR5GALLOC_ALLOC78                                   78
#define RSSPECAN_REPCAP_NR5GALLOC_ALLOC79                                   79
#define RSSPECAN_REPCAP_NR5GALLOC_ALLOC80                                   80
#define RSSPECAN_REPCAP_NR5GALLOC_ALLOC81                                   81
#define RSSPECAN_REPCAP_NR5GALLOC_ALLOC82                                   82
#define RSSPECAN_REPCAP_NR5GALLOC_ALLOC83                                   83
#define RSSPECAN_REPCAP_NR5GALLOC_ALLOC84                                   84
#define RSSPECAN_REPCAP_NR5GALLOC_ALLOC85                                   85
#define RSSPECAN_REPCAP_NR5GALLOC_ALLOC86                                   86
#define RSSPECAN_REPCAP_NR5GALLOC_ALLOC87                                   87
#define RSSPECAN_REPCAP_NR5GALLOC_ALLOC88                                   88
#define RSSPECAN_REPCAP_NR5GALLOC_ALLOC89                                   89
#define RSSPECAN_REPCAP_NR5GALLOC_ALLOC90                                   90
#define RSSPECAN_REPCAP_NR5GALLOC_ALLOC91                                   91
#define RSSPECAN_REPCAP_NR5GALLOC_ALLOC92                                   92
#define RSSPECAN_REPCAP_NR5GALLOC_ALLOC93                                   93
#define RSSPECAN_REPCAP_NR5GALLOC_ALLOC94                                   94
#define RSSPECAN_REPCAP_NR5GALLOC_ALLOC95                                   95
#define RSSPECAN_REPCAP_NR5GALLOC_ALLOC96                                   96
#define RSSPECAN_REPCAP_NR5GALLOC_ALLOC97                                   97
#define RSSPECAN_REPCAP_NR5GALLOC_ALLOC98                                   98
#define RSSPECAN_REPCAP_NR5GALLOC_ALLOC99                                   99

//repcap NR5GCORESET
#define RSSPECAN_REPCAP_NR5GCORESET_CORESET0                                0
#define RSSPECAN_REPCAP_NR5GCORESET_CORESET1                                1
#define RSSPECAN_REPCAP_NR5GCORESET_CORESET2                                2
#define RSSPECAN_REPCAP_NR5GCORESET_CORESET3                                3
#define RSSPECAN_REPCAP_NR5GCORESET_CORESET4                                4
#define RSSPECAN_REPCAP_NR5GCORESET_CORESET5                                5
#define RSSPECAN_REPCAP_NR5GCORESET_CORESET6                                6
#define RSSPECAN_REPCAP_NR5GCORESET_CORESET7                                7
#define RSSPECAN_REPCAP_NR5GCORESET_CORESET8                                8
#define RSSPECAN_REPCAP_NR5GCORESET_CORESET9                                9
#define RSSPECAN_REPCAP_NR5GCORESET_CORESET10                               10
#define RSSPECAN_REPCAP_NR5GCORESET_CORESET11                               11
#define RSSPECAN_REPCAP_NR5GCORESET_CORESET12                               12
#define RSSPECAN_REPCAP_NR5GCORESET_CORESET13                               13
#define RSSPECAN_REPCAP_NR5GCORESET_CORESET14                               14
#define RSSPECAN_REPCAP_NR5GCORESET_CORESET15                               15
#define RSSPECAN_REPCAP_NR5GCORESET_CORESET16                               16
#define RSSPECAN_REPCAP_NR5GCORESET_CORESET17                               17
#define RSSPECAN_REPCAP_NR5GCORESET_CORESET18                               18
#define RSSPECAN_REPCAP_NR5GCORESET_CORESET19                               19
#define RSSPECAN_REPCAP_NR5GCORESET_CORESET20                               20
#define RSSPECAN_REPCAP_NR5GCORESET_CORESET21                               21
#define RSSPECAN_REPCAP_NR5GCORESET_CORESET22                               22
#define RSSPECAN_REPCAP_NR5GCORESET_CORESET23                               23
#define RSSPECAN_REPCAP_NR5GCORESET_CORESET24                               24
#define RSSPECAN_REPCAP_NR5GCORESET_CORESET25                               25
#define RSSPECAN_REPCAP_NR5GCORESET_CORESET26                               26
#define RSSPECAN_REPCAP_NR5GCORESET_CORESET27                               27
#define RSSPECAN_REPCAP_NR5GCORESET_CORESET28                               28
#define RSSPECAN_REPCAP_NR5GCORESET_CORESET29                               29
#define RSSPECAN_REPCAP_NR5GCORESET_CORESET30                               30
#define RSSPECAN_REPCAP_NR5GCORESET_CORESET31                               31
#define RSSPECAN_REPCAP_NR5GCORESET_CORESET32                               32
#define RSSPECAN_REPCAP_NR5GCORESET_CORESET33                               33
#define RSSPECAN_REPCAP_NR5GCORESET_CORESET34                               34
#define RSSPECAN_REPCAP_NR5GCORESET_CORESET35                               35
#define RSSPECAN_REPCAP_NR5GCORESET_CORESET36                               36
#define RSSPECAN_REPCAP_NR5GCORESET_CORESET37                               37
#define RSSPECAN_REPCAP_NR5GCORESET_CORESET38                               38
#define RSSPECAN_REPCAP_NR5GCORESET_CORESET39                               39
#define RSSPECAN_REPCAP_NR5GCORESET_CORESET40                               40
#define RSSPECAN_REPCAP_NR5GCORESET_CORESET41                               41
#define RSSPECAN_REPCAP_NR5GCORESET_CORESET42                               42
#define RSSPECAN_REPCAP_NR5GCORESET_CORESET43                               43
#define RSSPECAN_REPCAP_NR5GCORESET_CORESET44                               44
#define RSSPECAN_REPCAP_NR5GCORESET_CORESET45                               45
#define RSSPECAN_REPCAP_NR5GCORESET_CORESET46                               46
#define RSSPECAN_REPCAP_NR5GCORESET_CORESET47                               47
#define RSSPECAN_REPCAP_NR5GCORESET_CORESET48                               48
#define RSSPECAN_REPCAP_NR5GCORESET_CORESET49                               49
#define RSSPECAN_REPCAP_NR5GCORESET_CORESET50                               50
#define RSSPECAN_REPCAP_NR5GCORESET_CORESET51                               51
#define RSSPECAN_REPCAP_NR5GCORESET_CORESET52                               52
#define RSSPECAN_REPCAP_NR5GCORESET_CORESET53                               53
#define RSSPECAN_REPCAP_NR5GCORESET_CORESET54                               54
#define RSSPECAN_REPCAP_NR5GCORESET_CORESET55                               55
#define RSSPECAN_REPCAP_NR5GCORESET_CORESET56                               56
#define RSSPECAN_REPCAP_NR5GCORESET_CORESET57                               57
#define RSSPECAN_REPCAP_NR5GCORESET_CORESET58                               58
#define RSSPECAN_REPCAP_NR5GCORESET_CORESET59                               59
#define RSSPECAN_REPCAP_NR5GCORESET_CORESET60                               60
#define RSSPECAN_REPCAP_NR5GCORESET_CORESET61                               61
#define RSSPECAN_REPCAP_NR5GCORESET_CORESET62                               62
#define RSSPECAN_REPCAP_NR5GCORESET_CORESET63                               63
#define RSSPECAN_REPCAP_NR5GCORESET_CORESET64                               64
#define RSSPECAN_REPCAP_NR5GCORESET_CORESET65                               65
#define RSSPECAN_REPCAP_NR5GCORESET_CORESET66                               66
#define RSSPECAN_REPCAP_NR5GCORESET_CORESET67                               67
#define RSSPECAN_REPCAP_NR5GCORESET_CORESET68                               68
#define RSSPECAN_REPCAP_NR5GCORESET_CORESET69                               69
#define RSSPECAN_REPCAP_NR5GCORESET_CORESET70                               70
#define RSSPECAN_REPCAP_NR5GCORESET_CORESET71                               71
#define RSSPECAN_REPCAP_NR5GCORESET_CORESET72                               72
#define RSSPECAN_REPCAP_NR5GCORESET_CORESET73                               73
#define RSSPECAN_REPCAP_NR5GCORESET_CORESET74                               74
#define RSSPECAN_REPCAP_NR5GCORESET_CORESET75                               75
#define RSSPECAN_REPCAP_NR5GCORESET_CORESET76                               76
#define RSSPECAN_REPCAP_NR5GCORESET_CORESET77                               77
#define RSSPECAN_REPCAP_NR5GCORESET_CORESET78                               78
#define RSSPECAN_REPCAP_NR5GCORESET_CORESET79                               79
#define RSSPECAN_REPCAP_NR5GCORESET_CORESET80                               80
#define RSSPECAN_REPCAP_NR5GCORESET_CORESET81                               81
#define RSSPECAN_REPCAP_NR5GCORESET_CORESET82                               82
#define RSSPECAN_REPCAP_NR5GCORESET_CORESET83                               83
#define RSSPECAN_REPCAP_NR5GCORESET_CORESET84                               84
#define RSSPECAN_REPCAP_NR5GCORESET_CORESET85                               85
#define RSSPECAN_REPCAP_NR5GCORESET_CORESET86                               86
#define RSSPECAN_REPCAP_NR5GCORESET_CORESET87                               87
#define RSSPECAN_REPCAP_NR5GCORESET_CORESET88                               88
#define RSSPECAN_REPCAP_NR5GCORESET_CORESET89                               89
#define RSSPECAN_REPCAP_NR5GCORESET_CORESET90                               90
#define RSSPECAN_REPCAP_NR5GCORESET_CORESET91                               91
#define RSSPECAN_REPCAP_NR5GCORESET_CORESET92                               92
#define RSSPECAN_REPCAP_NR5GCORESET_CORESET93                               93
#define RSSPECAN_REPCAP_NR5GCORESET_CORESET94                               94
#define RSSPECAN_REPCAP_NR5GCORESET_CORESET95                               95
#define RSSPECAN_REPCAP_NR5GCORESET_CORESET96                               96
#define RSSPECAN_REPCAP_NR5GCORESET_CORESET97                               97
#define RSSPECAN_REPCAP_NR5GCORESET_CORESET98                               98
#define RSSPECAN_REPCAP_NR5GCORESET_CORESET99                               99

//repcap ss
#define RSSPECAN_REPCAP_SS_SS0                                              0
#define RSSPECAN_REPCAP_SS_SS1                                              1
#define RSSPECAN_REPCAP_SS_SS2                                              2
#define RSSPECAN_REPCAP_SS_SS3                                              3
#define RSSPECAN_REPCAP_SS_SS4                                              4
#define RSSPECAN_REPCAP_SS_SS5                                              5
#define RSSPECAN_REPCAP_SS_SS6                                              6
#define RSSPECAN_REPCAP_SS_SS7                                              7
#define RSSPECAN_REPCAP_SS_SS8                                              8
#define RSSPECAN_REPCAP_SS_SS9                                              9
#define RSSPECAN_REPCAP_SS_SS10                                             10
#define RSSPECAN_REPCAP_SS_SS11                                             11
#define RSSPECAN_REPCAP_SS_SS12                                             12
#define RSSPECAN_REPCAP_SS_SS13                                             13
#define RSSPECAN_REPCAP_SS_SS14                                             14
#define RSSPECAN_REPCAP_SS_SS15                                             15
#define RSSPECAN_REPCAP_SS_SS16                                             16
#define RSSPECAN_REPCAP_SS_SS17                                             17
#define RSSPECAN_REPCAP_SS_SS18                                             18
#define RSSPECAN_REPCAP_SS_SS19                                             19
#define RSSPECAN_REPCAP_SS_SS20                                             20
#define RSSPECAN_REPCAP_SS_SS21                                             21
#define RSSPECAN_REPCAP_SS_SS22                                             22
#define RSSPECAN_REPCAP_SS_SS23                                             23
#define RSSPECAN_REPCAP_SS_SS24                                             24
#define RSSPECAN_REPCAP_SS_SS25                                             25
#define RSSPECAN_REPCAP_SS_SS26                                             26
#define RSSPECAN_REPCAP_SS_SS27                                             27
#define RSSPECAN_REPCAP_SS_SS28                                             28
#define RSSPECAN_REPCAP_SS_SS29                                             29
#define RSSPECAN_REPCAP_SS_SS30                                             30
#define RSSPECAN_REPCAP_SS_SS31                                             31
#define RSSPECAN_REPCAP_SS_SS32                                             32
#define RSSPECAN_REPCAP_SS_SS33                                             33
#define RSSPECAN_REPCAP_SS_SS34                                             34
#define RSSPECAN_REPCAP_SS_SS35                                             35
#define RSSPECAN_REPCAP_SS_SS36                                             36
#define RSSPECAN_REPCAP_SS_SS37                                             37
#define RSSPECAN_REPCAP_SS_SS38                                             38
#define RSSPECAN_REPCAP_SS_SS39                                             39
#define RSSPECAN_REPCAP_SS_SS40                                             40
#define RSSPECAN_REPCAP_SS_SS41                                             41
#define RSSPECAN_REPCAP_SS_SS42                                             42
#define RSSPECAN_REPCAP_SS_SS43                                             43
#define RSSPECAN_REPCAP_SS_SS44                                             44
#define RSSPECAN_REPCAP_SS_SS45                                             45
#define RSSPECAN_REPCAP_SS_SS46                                             46
#define RSSPECAN_REPCAP_SS_SS47                                             47
#define RSSPECAN_REPCAP_SS_SS48                                             48
#define RSSPECAN_REPCAP_SS_SS49                                             49
#define RSSPECAN_REPCAP_SS_SS50                                             50
#define RSSPECAN_REPCAP_SS_SS51                                             51
#define RSSPECAN_REPCAP_SS_SS52                                             52
#define RSSPECAN_REPCAP_SS_SS53                                             53
#define RSSPECAN_REPCAP_SS_SS54                                             54
#define RSSPECAN_REPCAP_SS_SS55                                             55
#define RSSPECAN_REPCAP_SS_SS56                                             56
#define RSSPECAN_REPCAP_SS_SS57                                             57
#define RSSPECAN_REPCAP_SS_SS58                                             58
#define RSSPECAN_REPCAP_SS_SS59                                             59
#define RSSPECAN_REPCAP_SS_SS60                                             60
#define RSSPECAN_REPCAP_SS_SS61                                             61
#define RSSPECAN_REPCAP_SS_SS62                                             62
#define RSSPECAN_REPCAP_SS_SS63                                             63
#define RSSPECAN_REPCAP_SS_SS64                                             64

//repcap View
#define RSSPECAN_REPCAP_VIEW_VIEW1                                          1
#define RSSPECAN_REPCAP_VIEW_VIEW2                                          2

//repcap Tab
#define RSSPECAN_REPCAP_TAB_TAB1                                            1
#define RSSPECAN_REPCAP_TAB_TAB2                                            2
#define RSSPECAN_REPCAP_TAB_TAB3                                            3

//repcap FPLANComponent
#define RSSPECAN_REPCAP_FPLANCOMPONENT_COM1                                 1
#define RSSPECAN_REPCAP_FPLANCOMPONENT_COM2                                 2
#define RSSPECAN_REPCAP_FPLANCOMPONENT_COM3                                 3
#define RSSPECAN_REPCAP_FPLANCOMPONENT_COM4                                 4
#define RSSPECAN_REPCAP_FPLANCOMPONENT_COM5                                 5
#define RSSPECAN_REPCAP_FPLANCOMPONENT_COM6                                 6

//repcap FPLANInput
#define RSSPECAN_REPCAP_FPLANINPUT_FPLANINP1                                1
#define RSSPECAN_REPCAP_FPLANINPUT_FPLANINP2                                2

//repcap OWULSubframe
#define RSSPECAN_REPCAP_OWULSUBFRAME_OWULSUBF0                              0
#define RSSPECAN_REPCAP_OWULSUBFRAME_OWULSUBF1                              1
#define RSSPECAN_REPCAP_OWULSUBFRAME_OWULSUBF2                              2
#define RSSPECAN_REPCAP_OWULSUBFRAME_OWULSUBF3                              3
#define RSSPECAN_REPCAP_OWULSUBFRAME_OWULSUBF4                              4
#define RSSPECAN_REPCAP_OWULSUBFRAME_OWULSUBF5                              5
#define RSSPECAN_REPCAP_OWULSUBFRAME_OWULSUBF6                              6
#define RSSPECAN_REPCAP_OWULSUBFRAME_OWULSUBF7                              7
#define RSSPECAN_REPCAP_OWULSUBFRAME_OWULSUBF8                              8
#define RSSPECAN_REPCAP_OWULSUBFRAME_OWULSUBF9                              9

//repcap OWULCluster
#define RSSPECAN_REPCAP_OWULCLUSTER_OWULCL1                                 1
#define RSSPECAN_REPCAP_OWULCLUSTER_OWULCL2                                 2

//repcap Instrument
#define RSSPECAN_REPCAP_INSTRUMENT_INSTR1                                   1
#define RSSPECAN_REPCAP_INSTRUMENT_INSTR2                                   2
#define RSSPECAN_REPCAP_INSTRUMENT_INSTR3                                   3
#define RSSPECAN_REPCAP_INSTRUMENT_INSTR4                                   4

//repcap V5GTFSubframe
#define RSSPECAN_REPCAP_V5GTFSUBFRAME_V5GSUBF0                              0
#define RSSPECAN_REPCAP_V5GTFSUBFRAME_V5GSUBF1                              1
#define RSSPECAN_REPCAP_V5GTFSUBFRAME_V5GSUBF2                              2
#define RSSPECAN_REPCAP_V5GTFSUBFRAME_V5GSUBF3                              3
#define RSSPECAN_REPCAP_V5GTFSUBFRAME_V5GSUBF4                              4
#define RSSPECAN_REPCAP_V5GTFSUBFRAME_V5GSUBF5                              5
#define RSSPECAN_REPCAP_V5GTFSUBFRAME_V5GSUBF6                              6
#define RSSPECAN_REPCAP_V5GTFSUBFRAME_V5GSUBF7                              7
#define RSSPECAN_REPCAP_V5GTFSUBFRAME_V5GSUBF8                              8
#define RSSPECAN_REPCAP_V5GTFSUBFRAME_V5GSUBF9                              9
#define RSSPECAN_REPCAP_V5GTFSUBFRAME_V5GSUBF10                             10
#define RSSPECAN_REPCAP_V5GTFSUBFRAME_V5GSUBF11                             11
#define RSSPECAN_REPCAP_V5GTFSUBFRAME_V5GSUBF12                             12
#define RSSPECAN_REPCAP_V5GTFSUBFRAME_V5GSUBF13                             13
#define RSSPECAN_REPCAP_V5GTFSUBFRAME_V5GSUBF14                             14
#define RSSPECAN_REPCAP_V5GTFSUBFRAME_V5GSUBF15                             15
#define RSSPECAN_REPCAP_V5GTFSUBFRAME_V5GSUBF16                             16
#define RSSPECAN_REPCAP_V5GTFSUBFRAME_V5GSUBF17                             17
#define RSSPECAN_REPCAP_V5GTFSUBFRAME_V5GSUBF18                             18
#define RSSPECAN_REPCAP_V5GTFSUBFRAME_V5GSUBF19                             19
#define RSSPECAN_REPCAP_V5GTFSUBFRAME_V5GSUBF20                             20
#define RSSPECAN_REPCAP_V5GTFSUBFRAME_V5GSUBF21                             21
#define RSSPECAN_REPCAP_V5GTFSUBFRAME_V5GSUBF22                             22
#define RSSPECAN_REPCAP_V5GTFSUBFRAME_V5GSUBF23                             23
#define RSSPECAN_REPCAP_V5GTFSUBFRAME_V5GSUBF24                             24
#define RSSPECAN_REPCAP_V5GTFSUBFRAME_V5GSUBF25                             25
#define RSSPECAN_REPCAP_V5GTFSUBFRAME_V5GSUBF26                             26
#define RSSPECAN_REPCAP_V5GTFSUBFRAME_V5GSUBF27                             27
#define RSSPECAN_REPCAP_V5GTFSUBFRAME_V5GSUBF28                             28
#define RSSPECAN_REPCAP_V5GTFSUBFRAME_V5GSUBF29                             29
#define RSSPECAN_REPCAP_V5GTFSUBFRAME_V5GSUBF30                             30
#define RSSPECAN_REPCAP_V5GTFSUBFRAME_V5GSUBF31                             31
#define RSSPECAN_REPCAP_V5GTFSUBFRAME_V5GSUBF32                             32
#define RSSPECAN_REPCAP_V5GTFSUBFRAME_V5GSUBF33                             33
#define RSSPECAN_REPCAP_V5GTFSUBFRAME_V5GSUBF34                             34
#define RSSPECAN_REPCAP_V5GTFSUBFRAME_V5GSUBF35                             35
#define RSSPECAN_REPCAP_V5GTFSUBFRAME_V5GSUBF36                             36
#define RSSPECAN_REPCAP_V5GTFSUBFRAME_V5GSUBF37                             37
#define RSSPECAN_REPCAP_V5GTFSUBFRAME_V5GSUBF38                             38
#define RSSPECAN_REPCAP_V5GTFSUBFRAME_V5GSUBF39                             39
#define RSSPECAN_REPCAP_V5GTFSUBFRAME_V5GSUBF40                             40
#define RSSPECAN_REPCAP_V5GTFSUBFRAME_V5GSUBF41                             41
#define RSSPECAN_REPCAP_V5GTFSUBFRAME_V5GSUBF42                             42
#define RSSPECAN_REPCAP_V5GTFSUBFRAME_V5GSUBF43                             43
#define RSSPECAN_REPCAP_V5GTFSUBFRAME_V5GSUBF44                             44
#define RSSPECAN_REPCAP_V5GTFSUBFRAME_V5GSUBF45                             45
#define RSSPECAN_REPCAP_V5GTFSUBFRAME_V5GSUBF46                             46
#define RSSPECAN_REPCAP_V5GTFSUBFRAME_V5GSUBF47                             47
#define RSSPECAN_REPCAP_V5GTFSUBFRAME_V5GSUBF48                             48
#define RSSPECAN_REPCAP_V5GTFSUBFRAME_V5GSUBF49                             49

//repcap V5GTFDLAllocation
#define RSSPECAN_REPCAP_V5GTFDLALLOCATION_V5GDLALLOC0                       0
#define RSSPECAN_REPCAP_V5GTFDLALLOCATION_V5GDLALLOC1                       1
#define RSSPECAN_REPCAP_V5GTFDLALLOCATION_V5GDLALLOC2                       2
#define RSSPECAN_REPCAP_V5GTFDLALLOCATION_V5GDLALLOC3                       3
#define RSSPECAN_REPCAP_V5GTFDLALLOCATION_V5GDLALLOC4                       4
#define RSSPECAN_REPCAP_V5GTFDLALLOCATION_V5GDLALLOC5                       5
#define RSSPECAN_REPCAP_V5GTFDLALLOCATION_V5GDLALLOC6                       6
#define RSSPECAN_REPCAP_V5GTFDLALLOCATION_V5GDLALLOC7                       7
#define RSSPECAN_REPCAP_V5GTFDLALLOCATION_V5GDLALLOC8                       8
#define RSSPECAN_REPCAP_V5GTFDLALLOCATION_V5GDLALLOC9                       9
#define RSSPECAN_REPCAP_V5GTFDLALLOCATION_V5GDLALLOC10                      10
#define RSSPECAN_REPCAP_V5GTFDLALLOCATION_V5GDLALLOC11                      11
#define RSSPECAN_REPCAP_V5GTFDLALLOCATION_V5GDLALLOC12                      12
#define RSSPECAN_REPCAP_V5GTFDLALLOCATION_V5GDLALLOC13                      13
#define RSSPECAN_REPCAP_V5GTFDLALLOCATION_V5GDLALLOC14                      14
#define RSSPECAN_REPCAP_V5GTFDLALLOCATION_V5GDLALLOC15                      15
#define RSSPECAN_REPCAP_V5GTFDLALLOCATION_V5GDLALLOC16                      16

//repcap V5GTFULAllocation
#define RSSPECAN_REPCAP_V5GTFULALLOCATION_V5GULALLOC0                       0
#define RSSPECAN_REPCAP_V5GTFULALLOCATION_V5GULALLOC1                       1

//repcap V5GTFOFDMSymbol
#define RSSPECAN_REPCAP_V5GTFOFDMSYMBOL_V5GOFDMS0                           0
#define RSSPECAN_REPCAP_V5GTFOFDMSYMBOL_V5GOFDMS1                           1
#define RSSPECAN_REPCAP_V5GTFOFDMSYMBOL_V5GOFDMS2                           2
#define RSSPECAN_REPCAP_V5GTFOFDMSYMBOL_V5GOFDMS3                           3
#define RSSPECAN_REPCAP_V5GTFOFDMSYMBOL_V5GOFDMS4                           4
#define RSSPECAN_REPCAP_V5GTFOFDMSYMBOL_V5GOFDMS5                           5
#define RSSPECAN_REPCAP_V5GTFOFDMSYMBOL_V5GOFDMS6                           6
#define RSSPECAN_REPCAP_V5GTFOFDMSYMBOL_V5GOFDMS7                           7
#define RSSPECAN_REPCAP_V5GTFOFDMSYMBOL_V5GOFDMS8                           8
#define RSSPECAN_REPCAP_V5GTFOFDMSYMBOL_V5GOFDMS9                           9
#define RSSPECAN_REPCAP_V5GTFOFDMSYMBOL_V5GOFDMS10                          10
#define RSSPECAN_REPCAP_V5GTFOFDMSYMBOL_V5GOFDMS11                          11
#define RSSPECAN_REPCAP_V5GTFOFDMSYMBOL_V5GOFDMS12                          12
#define RSSPECAN_REPCAP_V5GTFOFDMSYMBOL_V5GOFDMS13                          13

//repcap V5GTFxREG
#define RSSPECAN_REPCAP_V5GTFXREG_V5GXREG1                                  1
#define RSSPECAN_REPCAP_V5GTFXREG_V5GXREG2                                  2
#define RSSPECAN_REPCAP_V5GTFXREG_V5GXREG3                                  3
#define RSSPECAN_REPCAP_V5GTFXREG_V5GXREG4                                  4
#define RSSPECAN_REPCAP_V5GTFXREG_V5GXREG5                                  5
#define RSSPECAN_REPCAP_V5GTFXREG_V5GXREG6                                  6
#define RSSPECAN_REPCAP_V5GTFXREG_V5GXREG7                                  7
#define RSSPECAN_REPCAP_V5GTFXREG_V5GXREG8                                  8
#define RSSPECAN_REPCAP_V5GTFXREG_V5GXREG9                                  9
#define RSSPECAN_REPCAP_V5GTFXREG_V5GXREG10                                 10
#define RSSPECAN_REPCAP_V5GTFXREG_V5GXREG11                                 11
#define RSSPECAN_REPCAP_V5GTFXREG_V5GXREG12                                 12
#define RSSPECAN_REPCAP_V5GTFXREG_V5GXREG13                                 13
#define RSSPECAN_REPCAP_V5GTFXREG_V5GXREG14                                 14
#define RSSPECAN_REPCAP_V5GTFXREG_V5GXREG15                                 15
#define RSSPECAN_REPCAP_V5GTFXREG_V5GXREG16                                 16

//repcap V5GTFPhysicalAntenna
#define RSSPECAN_REPCAP_V5GTFPHYSICALANTENNA_V5GPHYSICALANTENNA1            1
#define RSSPECAN_REPCAP_V5GTFPHYSICALANTENNA_V5GPHYSICALANTENNA2            2

//repcap V5GTFxREGSymbol
#define RSSPECAN_REPCAP_V5GTFXREGSYMBOL_V5GXREGSYMBOL0                      0
#define RSSPECAN_REPCAP_V5GTFXREGSYMBOL_V5GXREGSYMBOL1                      1

//repcap NR5GPUCCH
#define RSSPECAN_REPCAP_NR5GPUCCH_PUCCH0                                    0
#define RSSPECAN_REPCAP_NR5GPUCCH_PUCCH1                                    1
#define RSSPECAN_REPCAP_NR5GPUCCH_PUCCH2                                    2
#define RSSPECAN_REPCAP_NR5GPUCCH_PUCCH3                                    3
#define RSSPECAN_REPCAP_NR5GPUCCH_PUCCH4                                    4
#define RSSPECAN_REPCAP_NR5GPUCCH_PUCCH5                                    5
#define RSSPECAN_REPCAP_NR5GPUCCH_PUCCH6                                    6
#define RSSPECAN_REPCAP_NR5GPUCCH_PUCCH7                                    7
#define RSSPECAN_REPCAP_NR5GPUCCH_PUCCH8                                    8
#define RSSPECAN_REPCAP_NR5GPUCCH_PUCCH9                                    9
#define RSSPECAN_REPCAP_NR5GPUCCH_PUCCH10                                   10
#define RSSPECAN_REPCAP_NR5GPUCCH_PUCCH11                                   11
#define RSSPECAN_REPCAP_NR5GPUCCH_PUCCH12                                   12
#define RSSPECAN_REPCAP_NR5GPUCCH_PUCCH13                                   13
#define RSSPECAN_REPCAP_NR5GPUCCH_PUCCH14                                   14
#define RSSPECAN_REPCAP_NR5GPUCCH_PUCCH15                                   15
#define RSSPECAN_REPCAP_NR5GPUCCH_PUCCH16                                   16
#define RSSPECAN_REPCAP_NR5GPUCCH_PUCCH17                                   17
#define RSSPECAN_REPCAP_NR5GPUCCH_PUCCH18                                   18
#define RSSPECAN_REPCAP_NR5GPUCCH_PUCCH19                                   19
#define RSSPECAN_REPCAP_NR5GPUCCH_PUCCH20                                   20
#define RSSPECAN_REPCAP_NR5GPUCCH_PUCCH21                                   21
#define RSSPECAN_REPCAP_NR5GPUCCH_PUCCH22                                   22
#define RSSPECAN_REPCAP_NR5GPUCCH_PUCCH23                                   23
#define RSSPECAN_REPCAP_NR5GPUCCH_PUCCH24                                   24
#define RSSPECAN_REPCAP_NR5GPUCCH_PUCCH25                                   25
#define RSSPECAN_REPCAP_NR5GPUCCH_PUCCH26                                   26
#define RSSPECAN_REPCAP_NR5GPUCCH_PUCCH27                                   27
#define RSSPECAN_REPCAP_NR5GPUCCH_PUCCH28                                   28
#define RSSPECAN_REPCAP_NR5GPUCCH_PUCCH29                                   29
#define RSSPECAN_REPCAP_NR5GPUCCH_PUCCH30                                   30
#define RSSPECAN_REPCAP_NR5GPUCCH_PUCCH31                                   31
#define RSSPECAN_REPCAP_NR5GPUCCH_PUCCH32                                   32
#define RSSPECAN_REPCAP_NR5GPUCCH_PUCCH33                                   33
#define RSSPECAN_REPCAP_NR5GPUCCH_PUCCH34                                   34
#define RSSPECAN_REPCAP_NR5GPUCCH_PUCCH35                                   35
#define RSSPECAN_REPCAP_NR5GPUCCH_PUCCH36                                   36
#define RSSPECAN_REPCAP_NR5GPUCCH_PUCCH37                                   37
#define RSSPECAN_REPCAP_NR5GPUCCH_PUCCH38                                   38
#define RSSPECAN_REPCAP_NR5GPUCCH_PUCCH39                                   39
#define RSSPECAN_REPCAP_NR5GPUCCH_PUCCH40                                   40
#define RSSPECAN_REPCAP_NR5GPUCCH_PUCCH41                                   41
#define RSSPECAN_REPCAP_NR5GPUCCH_PUCCH42                                   42
#define RSSPECAN_REPCAP_NR5GPUCCH_PUCCH43                                   43
#define RSSPECAN_REPCAP_NR5GPUCCH_PUCCH44                                   44
#define RSSPECAN_REPCAP_NR5GPUCCH_PUCCH45                                   45
#define RSSPECAN_REPCAP_NR5GPUCCH_PUCCH46                                   46
#define RSSPECAN_REPCAP_NR5GPUCCH_PUCCH47                                   47
#define RSSPECAN_REPCAP_NR5GPUCCH_PUCCH48                                   48
#define RSSPECAN_REPCAP_NR5GPUCCH_PUCCH49                                   49
#define RSSPECAN_REPCAP_NR5GPUCCH_PUCCH50                                   50
#define RSSPECAN_REPCAP_NR5GPUCCH_PUCCH51                                   51
#define RSSPECAN_REPCAP_NR5GPUCCH_PUCCH52                                   52
#define RSSPECAN_REPCAP_NR5GPUCCH_PUCCH53                                   53
#define RSSPECAN_REPCAP_NR5GPUCCH_PUCCH54                                   54
#define RSSPECAN_REPCAP_NR5GPUCCH_PUCCH55                                   55
#define RSSPECAN_REPCAP_NR5GPUCCH_PUCCH56                                   56
#define RSSPECAN_REPCAP_NR5GPUCCH_PUCCH57                                   57
#define RSSPECAN_REPCAP_NR5GPUCCH_PUCCH58                                   58
#define RSSPECAN_REPCAP_NR5GPUCCH_PUCCH59                                   59
#define RSSPECAN_REPCAP_NR5GPUCCH_PUCCH60                                   60
#define RSSPECAN_REPCAP_NR5GPUCCH_PUCCH61                                   61
#define RSSPECAN_REPCAP_NR5GPUCCH_PUCCH62                                   62
#define RSSPECAN_REPCAP_NR5GPUCCH_PUCCH63                                   63
#define RSSPECAN_REPCAP_NR5GPUCCH_PUCCH64                                   64
#define RSSPECAN_REPCAP_NR5GPUCCH_PUCCH65                                   65
#define RSSPECAN_REPCAP_NR5GPUCCH_PUCCH66                                   66
#define RSSPECAN_REPCAP_NR5GPUCCH_PUCCH67                                   67
#define RSSPECAN_REPCAP_NR5GPUCCH_PUCCH68                                   68
#define RSSPECAN_REPCAP_NR5GPUCCH_PUCCH69                                   69
#define RSSPECAN_REPCAP_NR5GPUCCH_PUCCH70                                   70
#define RSSPECAN_REPCAP_NR5GPUCCH_PUCCH71                                   71
#define RSSPECAN_REPCAP_NR5GPUCCH_PUCCH72                                   72
#define RSSPECAN_REPCAP_NR5GPUCCH_PUCCH73                                   73
#define RSSPECAN_REPCAP_NR5GPUCCH_PUCCH74                                   74
#define RSSPECAN_REPCAP_NR5GPUCCH_PUCCH75                                   75
#define RSSPECAN_REPCAP_NR5GPUCCH_PUCCH76                                   76
#define RSSPECAN_REPCAP_NR5GPUCCH_PUCCH77                                   77
#define RSSPECAN_REPCAP_NR5GPUCCH_PUCCH78                                   78
#define RSSPECAN_REPCAP_NR5GPUCCH_PUCCH79                                   79
#define RSSPECAN_REPCAP_NR5GPUCCH_PUCCH80                                   80
#define RSSPECAN_REPCAP_NR5GPUCCH_PUCCH81                                   81
#define RSSPECAN_REPCAP_NR5GPUCCH_PUCCH82                                   82
#define RSSPECAN_REPCAP_NR5GPUCCH_PUCCH83                                   83
#define RSSPECAN_REPCAP_NR5GPUCCH_PUCCH84                                   84
#define RSSPECAN_REPCAP_NR5GPUCCH_PUCCH85                                   85
#define RSSPECAN_REPCAP_NR5GPUCCH_PUCCH86                                   86
#define RSSPECAN_REPCAP_NR5GPUCCH_PUCCH87                                   87
#define RSSPECAN_REPCAP_NR5GPUCCH_PUCCH88                                   88
#define RSSPECAN_REPCAP_NR5GPUCCH_PUCCH89                                   89
#define RSSPECAN_REPCAP_NR5GPUCCH_PUCCH90                                   90
#define RSSPECAN_REPCAP_NR5GPUCCH_PUCCH91                                   91
#define RSSPECAN_REPCAP_NR5GPUCCH_PUCCH92                                   92
#define RSSPECAN_REPCAP_NR5GPUCCH_PUCCH93                                   93
#define RSSPECAN_REPCAP_NR5GPUCCH_PUCCH94                                   94
#define RSSPECAN_REPCAP_NR5GPUCCH_PUCCH95                                   95
#define RSSPECAN_REPCAP_NR5GPUCCH_PUCCH96                                   96
#define RSSPECAN_REPCAP_NR5GPUCCH_PUCCH97                                   97
#define RSSPECAN_REPCAP_NR5GPUCCH_PUCCH98                                   98
#define RSSPECAN_REPCAP_NR5GPUCCH_PUCCH99                                   99
#define RSSPECAN_REPCAP_NR5GPUCCH_PUCCH100                                  100

//repcap Ant
#define RSSPECAN_REPCAP_ANT_ANTPORT0                                        0
#define RSSPECAN_REPCAP_ANT_ANTPORT1                                        1
#define RSSPECAN_REPCAP_ANT_ANTPORT2                                        2
#define RSSPECAN_REPCAP_ANT_ANTPORT3                                        3
#define RSSPECAN_REPCAP_ANT_ANTPORT4                                        4
#define RSSPECAN_REPCAP_ANT_ANTPORT5                                        5
#define RSSPECAN_REPCAP_ANT_ANTPORT6                                        6
#define RSSPECAN_REPCAP_ANT_ANTPORT7                                        7
#define RSSPECAN_REPCAP_ANT_ANTPORT8                                        8
#define RSSPECAN_REPCAP_ANT_ANTPORT9                                        9
#define RSSPECAN_REPCAP_ANT_ANTPORT10                                       10
#define RSSPECAN_REPCAP_ANT_ANTPORT11                                       11

//repcap AntennaConfig
#define RSSPECAN_REPCAP_ANTENNACONFIG_AC1                                   1
#define RSSPECAN_REPCAP_ANTENNACONFIG_AC2                                   2

//repcap TrgPort
#define RSSPECAN_REPCAP_TRGPORT_TP1                                         1
#define RSSPECAN_REPCAP_TRGPORT_TP2                                         2

//repcap Inp
#define RSSPECAN_REPCAP_INP_INP1                                            1
#define RSSPECAN_REPCAP_INP_INP2                                            2

//repcap TRepLine
#define RSSPECAN_REPCAP_TREPLINE_TRL1                                       1
#define RSSPECAN_REPCAP_TREPLINE_TRL2                                       2
#define RSSPECAN_REPCAP_TREPLINE_TRL3                                       3
#define RSSPECAN_REPCAP_TREPLINE_TRL4                                       4
#define RSSPECAN_REPCAP_TREPLINE_TRL5                                       5

//repcap CSI
#define RSSPECAN_REPCAP_CSI_CSI0                                            0
#define RSSPECAN_REPCAP_CSI_CSI1                                            1
#define RSSPECAN_REPCAP_CSI_CSI2                                            2
#define RSSPECAN_REPCAP_CSI_CSI3                                            3
#define RSSPECAN_REPCAP_CSI_CSI4                                            4
#define RSSPECAN_REPCAP_CSI_CSI5                                            5
#define RSSPECAN_REPCAP_CSI_CSI6                                            6
#define RSSPECAN_REPCAP_CSI_CSI7                                            7
#define RSSPECAN_REPCAP_CSI_CSI8                                            8
#define RSSPECAN_REPCAP_CSI_CSI9                                            9
#define RSSPECAN_REPCAP_CSI_CSI10                                           10
#define RSSPECAN_REPCAP_CSI_CSI11                                           11
#define RSSPECAN_REPCAP_CSI_CSI12                                           12
#define RSSPECAN_REPCAP_CSI_CSI13                                           13
#define RSSPECAN_REPCAP_CSI_CSI14                                           14
#define RSSPECAN_REPCAP_CSI_CSI15                                           15
#define RSSPECAN_REPCAP_CSI_CSI16                                           16
#define RSSPECAN_REPCAP_CSI_CSI17                                           17
#define RSSPECAN_REPCAP_CSI_CSI18                                           18
#define RSSPECAN_REPCAP_CSI_CSI19                                           19
#define RSSPECAN_REPCAP_CSI_CSI20                                           20
#define RSSPECAN_REPCAP_CSI_CSI21                                           21
#define RSSPECAN_REPCAP_CSI_CSI22                                           22
#define RSSPECAN_REPCAP_CSI_CSI23                                           23
#define RSSPECAN_REPCAP_CSI_CSI24                                           24
#define RSSPECAN_REPCAP_CSI_CSI25                                           25
#define RSSPECAN_REPCAP_CSI_CSI26                                           26
#define RSSPECAN_REPCAP_CSI_CSI27                                           27
#define RSSPECAN_REPCAP_CSI_CSI28                                           28
#define RSSPECAN_REPCAP_CSI_CSI29                                           29
#define RSSPECAN_REPCAP_CSI_CSI30                                           30
#define RSSPECAN_REPCAP_CSI_CSI31                                           31
#define RSSPECAN_REPCAP_CSI_CSI32                                           32
#define RSSPECAN_REPCAP_CSI_CSI33                                           33
#define RSSPECAN_REPCAP_CSI_CSI34                                           34
#define RSSPECAN_REPCAP_CSI_CSI35                                           35
#define RSSPECAN_REPCAP_CSI_CSI36                                           36
#define RSSPECAN_REPCAP_CSI_CSI37                                           37
#define RSSPECAN_REPCAP_CSI_CSI38                                           38
#define RSSPECAN_REPCAP_CSI_CSI39                                           39
#define RSSPECAN_REPCAP_CSI_CSI40                                           40
#define RSSPECAN_REPCAP_CSI_CSI41                                           41
#define RSSPECAN_REPCAP_CSI_CSI42                                           42
#define RSSPECAN_REPCAP_CSI_CSI43                                           43
#define RSSPECAN_REPCAP_CSI_CSI44                                           44
#define RSSPECAN_REPCAP_CSI_CSI45                                           45
#define RSSPECAN_REPCAP_CSI_CSI46                                           46
#define RSSPECAN_REPCAP_CSI_CSI47                                           47
#define RSSPECAN_REPCAP_CSI_CSI48                                           48
#define RSSPECAN_REPCAP_CSI_CSI49                                           49
#define RSSPECAN_REPCAP_CSI_CSI50                                           50
#define RSSPECAN_REPCAP_CSI_CSI51                                           51
#define RSSPECAN_REPCAP_CSI_CSI52                                           52
#define RSSPECAN_REPCAP_CSI_CSI53                                           53
#define RSSPECAN_REPCAP_CSI_CSI54                                           54
#define RSSPECAN_REPCAP_CSI_CSI55                                           55
#define RSSPECAN_REPCAP_CSI_CSI56                                           56
#define RSSPECAN_REPCAP_CSI_CSI57                                           57
#define RSSPECAN_REPCAP_CSI_CSI58                                           58
#define RSSPECAN_REPCAP_CSI_CSI59                                           59
#define RSSPECAN_REPCAP_CSI_CSI60                                           60
#define RSSPECAN_REPCAP_CSI_CSI61                                           61
#define RSSPECAN_REPCAP_CSI_CSI62                                           62
#define RSSPECAN_REPCAP_CSI_CSI63                                           63
#define RSSPECAN_REPCAP_CSI_CSI64                                           64

//repcap Subframe
#define RSSPECAN_REPCAP_SUBFRAME_SUBFRAME1                                  1
#define RSSPECAN_REPCAP_SUBFRAME_SUBFRAME2                                  2
#define RSSPECAN_REPCAP_SUBFRAME_SUBFRAME3                                  3
#define RSSPECAN_REPCAP_SUBFRAME_SUBFRAME4                                  4
#define RSSPECAN_REPCAP_SUBFRAME_SUBFRAME5                                  5
#define RSSPECAN_REPCAP_SUBFRAME_SUBFRAME6                                  6
#define RSSPECAN_REPCAP_SUBFRAME_SUBFRAME7                                  7
#define RSSPECAN_REPCAP_SUBFRAME_SUBFRAME8                                  8
#define RSSPECAN_REPCAP_SUBFRAME_SUBFRAME9                                  9

//repcap TestReportLine
#define RSSPECAN_REPCAP_TESTREPORTLINE_TESTREPORTLINE1                      1
#define RSSPECAN_REPCAP_TESTREPORTLINE_TESTREPORTLINE2                      2
#define RSSPECAN_REPCAP_TESTREPORTLINE_TESTREPORTLINE3                      3
#define RSSPECAN_REPCAP_TESTREPORTLINE_TESTREPORTLINE4                      4
#define RSSPECAN_REPCAP_TESTREPORTLINE_TESTREPORTLINE5                      5
#define RSSPECAN_REPCAP_TESTREPORTLINE_TESTREPORTLINE6                      6

//repcap BTOPacketType
#define RSSPECAN_REPCAP_BTOPACKETTYPE_BTOPACKETTYPE1                        1
#define RSSPECAN_REPCAP_BTOPACKETTYPE_BTOPACKETTYPE3                        2
#define RSSPECAN_REPCAP_BTOPACKETTYPE_BTOPACKETTYPE5                        3

//repcap UWBFilter
#define RSSPECAN_REPCAP_UWBFILTER_FILT1                                     1
#define RSSPECAN_REPCAP_UWBFILTER_FILT2                                     2
#define RSSPECAN_REPCAP_UWBFILTER_FILT3                                     3
#define RSSPECAN_REPCAP_UWBFILTER_FILT4                                     4
#define RSSPECAN_REPCAP_UWBFILTER_FILT5                                     5
#define RSSPECAN_REPCAP_UWBFILTER_FILT6                                     6
#define RSSPECAN_REPCAP_UWBFILTER_FILT7                                     7
#define RSSPECAN_REPCAP_UWBFILTER_FILT8                                     8
#define RSSPECAN_REPCAP_UWBFILTER_FILT9                                     9
#define RSSPECAN_REPCAP_UWBFILTER_FILT10                                    10
#define RSSPECAN_REPCAP_UWBFILTER_FILT11                                    11
#define RSSPECAN_REPCAP_UWBFILTER_FILT12                                    12
#define RSSPECAN_REPCAP_UWBFILTER_FILT13                                    13
#define RSSPECAN_REPCAP_UWBFILTER_FILT14                                    14
#define RSSPECAN_REPCAP_UWBFILTER_FILT15                                    15
#define RSSPECAN_REPCAP_UWBFILTER_FILT16                                    16
#define RSSPECAN_REPCAP_UWBFILTER_FILT17                                    17
#define RSSPECAN_REPCAP_UWBFILTER_FILT18                                    18
#define RSSPECAN_REPCAP_UWBFILTER_FILT19                                    19
#define RSSPECAN_REPCAP_UWBFILTER_FILT20                                    20
#define RSSPECAN_REPCAP_UWBFILTER_FILT21                                    21
#define RSSPECAN_REPCAP_UWBFILTER_FILT22                                    22
#define RSSPECAN_REPCAP_UWBFILTER_FILT23                                    23
#define RSSPECAN_REPCAP_UWBFILTER_FILT24                                    24
#define RSSPECAN_REPCAP_UWBFILTER_FILT25                                    25
#define RSSPECAN_REPCAP_UWBFILTER_FILT26                                    26
#define RSSPECAN_REPCAP_UWBFILTER_FILT27                                    27
#define RSSPECAN_REPCAP_UWBFILTER_FILT28                                    28
#define RSSPECAN_REPCAP_UWBFILTER_FILT29                                    29
#define RSSPECAN_REPCAP_UWBFILTER_FILT30                                    30
#define RSSPECAN_REPCAP_UWBFILTER_FILT31                                    31
#define RSSPECAN_REPCAP_UWBFILTER_FILT32                                    32
#define RSSPECAN_REPCAP_UWBFILTER_FILT33                                    33
#define RSSPECAN_REPCAP_UWBFILTER_FILT34                                    34
#define RSSPECAN_REPCAP_UWBFILTER_FILT35                                    35
#define RSSPECAN_REPCAP_UWBFILTER_FILT36                                    36
#define RSSPECAN_REPCAP_UWBFILTER_FILT37                                    37
#define RSSPECAN_REPCAP_UWBFILTER_FILT38                                    38
#define RSSPECAN_REPCAP_UWBFILTER_FILT39                                    39
#define RSSPECAN_REPCAP_UWBFILTER_FILT40                                    40
#define RSSPECAN_REPCAP_UWBFILTER_FILT41                                    41
#define RSSPECAN_REPCAP_UWBFILTER_FILT42                                    42
#define RSSPECAN_REPCAP_UWBFILTER_FILT43                                    43
#define RSSPECAN_REPCAP_UWBFILTER_FILT44                                    44
#define RSSPECAN_REPCAP_UWBFILTER_FILT45                                    45
#define RSSPECAN_REPCAP_UWBFILTER_FILT46                                    46
#define RSSPECAN_REPCAP_UWBFILTER_FILT47                                    47
#define RSSPECAN_REPCAP_UWBFILTER_FILT48                                    48
#define RSSPECAN_REPCAP_UWBFILTER_FILT49                                    49
#define RSSPECAN_REPCAP_UWBFILTER_FILT50                                    50
#define RSSPECAN_REPCAP_UWBFILTER_FILT51                                    51
#define RSSPECAN_REPCAP_UWBFILTER_FILT52                                    52
#define RSSPECAN_REPCAP_UWBFILTER_FILT53                                    53
#define RSSPECAN_REPCAP_UWBFILTER_FILT54                                    54
#define RSSPECAN_REPCAP_UWBFILTER_FILT55                                    55
#define RSSPECAN_REPCAP_UWBFILTER_FILT56                                    56
#define RSSPECAN_REPCAP_UWBFILTER_FILT57                                    57
#define RSSPECAN_REPCAP_UWBFILTER_FILT58                                    58
#define RSSPECAN_REPCAP_UWBFILTER_FILT59                                    59
#define RSSPECAN_REPCAP_UWBFILTER_FILT60                                    60
#define RSSPECAN_REPCAP_UWBFILTER_FILT61                                    61
#define RSSPECAN_REPCAP_UWBFILTER_FILT62                                    62
#define RSSPECAN_REPCAP_UWBFILTER_FILT63                                    63
#define RSSPECAN_REPCAP_UWBFILTER_FILT64                                    64
#define RSSPECAN_REPCAP_UWBFILTER_FILT65                                    65
#define RSSPECAN_REPCAP_UWBFILTER_FILT66                                    66
#define RSSPECAN_REPCAP_UWBFILTER_FILT67                                    67
#define RSSPECAN_REPCAP_UWBFILTER_FILT68                                    68
#define RSSPECAN_REPCAP_UWBFILTER_FILT69                                    69
#define RSSPECAN_REPCAP_UWBFILTER_FILT70                                    70
#define RSSPECAN_REPCAP_UWBFILTER_FILT71                                    71
#define RSSPECAN_REPCAP_UWBFILTER_FILT72                                    72
#define RSSPECAN_REPCAP_UWBFILTER_FILT73                                    73
#define RSSPECAN_REPCAP_UWBFILTER_FILT74                                    74
#define RSSPECAN_REPCAP_UWBFILTER_FILT75                                    75
#define RSSPECAN_REPCAP_UWBFILTER_FILT76                                    76
#define RSSPECAN_REPCAP_UWBFILTER_FILT77                                    77
#define RSSPECAN_REPCAP_UWBFILTER_FILT78                                    78
#define RSSPECAN_REPCAP_UWBFILTER_FILT79                                    79
#define RSSPECAN_REPCAP_UWBFILTER_FILT80                                    80
#define RSSPECAN_REPCAP_UWBFILTER_FILT81                                    81
#define RSSPECAN_REPCAP_UWBFILTER_FILT82                                    82
#define RSSPECAN_REPCAP_UWBFILTER_FILT83                                    83
#define RSSPECAN_REPCAP_UWBFILTER_FILT84                                    84
#define RSSPECAN_REPCAP_UWBFILTER_FILT85                                    85
#define RSSPECAN_REPCAP_UWBFILTER_FILT86                                    86
#define RSSPECAN_REPCAP_UWBFILTER_FILT87                                    87
#define RSSPECAN_REPCAP_UWBFILTER_FILT88                                    88
#define RSSPECAN_REPCAP_UWBFILTER_FILT89                                    89
#define RSSPECAN_REPCAP_UWBFILTER_FILT90                                    90
#define RSSPECAN_REPCAP_UWBFILTER_FILT91                                    91
#define RSSPECAN_REPCAP_UWBFILTER_FILT92                                    92
#define RSSPECAN_REPCAP_UWBFILTER_FILT93                                    93
#define RSSPECAN_REPCAP_UWBFILTER_FILT94                                    94
#define RSSPECAN_REPCAP_UWBFILTER_FILT95                                    95
#define RSSPECAN_REPCAP_UWBFILTER_FILT96                                    96
#define RSSPECAN_REPCAP_UWBFILTER_FILT97                                    97
#define RSSPECAN_REPCAP_UWBFILTER_FILT98                                    98
#define RSSPECAN_REPCAP_UWBFILTER_FILT99                                    99
#define RSSPECAN_REPCAP_UWBFILTER_FILT100                                   100

//repcap CyclicPrefix
#define RSSPECAN_REPCAP_CYCLICPREFIX_CYCLICPREFIX1                          1
#define RSSPECAN_REPCAP_CYCLICPREFIX_CYCLICPREFIX2                          2
/* END GENERATE */

/*****************************************************************************
 *- Hidden Attribute Declarations -------------------------------------------*
 *****************************************************************************/

#define RSSPECAN_ATTR_OPC_TIMEOUT               (RS_ATTR_OPC_TIMEOUT)           /* ViInt32 */
#define RSSPECAN_ATTR_IO_SESSION                (RS_ATTR_IO_SESSION)            /* ViSession */
#define RSSPECAN_ATTR_OPC_CALLBACK              (RS_ATTR_OPC_CALLBACK)          /* ViAddr */
#define RSSPECAN_ATTR_CHECK_STATUS_CALLBACK     (RS_ATTR_CHECK_STATUS_CALLBACK) /* ViAddr */

 /****************************************************************************
  *------------------------ Attribute Value Defines -------------------------*
  ****************************************************************************/

  /*- Defined values for RSSPECAN_ATTR_TRACE_TYPE --------------------------*/

#define RSSPECAN_VAL_TRACE_TYPE_CLEAR_WRITE                         (1L)
#define RSSPECAN_VAL_TRACE_TYPE_MAX_HOLD                            (2L)
#define RSSPECAN_VAL_TRACE_TYPE_MIN_HOLD                            (3L)
#define RSSPECAN_VAL_TRACE_TYPE_VIDEO_AVERAGE                       (4L)
#define RSSPECAN_VAL_TRACE_TYPE_VIEW                                (5L)
#define RSSPECAN_VAL_TRACE_TYPE_STORE                               (6L)

#define RSSPECAN_VAL_TRACE_TYPE_CLASS_EXT_BASE                      (500L)
#define RSSPECAN_VAL_TRACE_TYPE_SPECIFIC_EXT_BASE                   (1000L)

    /*- Defined values for RSSPECAN_ATTR_VERTICAL_SCALE ----------------------*/

#define RSSPECAN_VAL_VERTICAL_SCALE_LINEAR                          (1L)
#define RSSPECAN_VAL_VERTICAL_SCALE_LOGARITHMIC                     (2L)

#define RSSPECAN_VAL_VERTICAL_SCALE_CLASS_EXT_BASE                  (500L)
#define RSSPECAN_VAL_VERTICAL_SCALE_SPECIFIC_EXT_BASE               (1000L)

    /*- Defined values for RSSPECAN_ATTR_MARKER_TYPE -------------------------*/

#define RSSPECAN_VAL_MARKER_TYPE_NORMAL                             (1L)
#define RSSPECAN_VAL_MARKER_TYPE_DELTA                              (2L)

#define RSSPECAN_VAL_MARKER_TYPE_CLASS_EXT_BASE                     (500L)
#define RSSPECAN_VAL_MARKER_TYPE_SPECIFIC_EXT_BASE                  (1000L)

    /*- Defined values for RSSPECAN_ATTR_TRIGGER_SOURCE ----------------------*/

#define RSSPECAN_VAL_TRIGGER_SOURCE_EXTERNAL                        (1L)
#define RSSPECAN_VAL_TRIGGER_SOURCE_IMMEDIATE                       (2L)
#define RSSPECAN_VAL_TRIGGER_SOURCE_SOFTWARE                        (3L)
#define RSSPECAN_VAL_TRIGGER_SOURCE_AC_LINE                         (4L)
#define RSSPECAN_VAL_TRIGGER_SOURCE_VIDEO                           (5L)

#define RSSPECAN_VAL_TRIGGER_SOURCE_CLASS_EXT_BASE                  (500L)
#define RSSPECAN_VAL_TRIGGER_SOURCE_SPECIFIC_EXT_BASE               (1000L)

    /*- Defined values for RSSPECAN_ATTR_EXTERNAL_TRIGGER_SLOPE --------------*/

#define RSSPECAN_VAL_EXTERNAL_TRIGGER_SLOPE_POSITIVE                (1L)
#define RSSPECAN_VAL_EXTERNAL_TRIGGER_SLOPE_NEGATIVE                (2L)

#define RSSPECAN_VAL_EXTERNAL_TRIGGER_SLOPE_CLASS_EXT_BASE          (500L)
#define RSSPECAN_VAL_EXTERNAL_TRIGGER_SLOPE_SPECIFIC_EXT_BASE       (1000L)

    /*- Defined values for RSSPECAN_ATTR_VIDEO_TRIGGER_SLOPE --------------*/

#define RSSPECAN_VAL_VIDEO_TRIGGER_SLOPE_POSITIVE                   (1L)
#define RSSPECAN_VAL_VIDEO_TRIGGER_SLOPE_NEGATIVE                   (2L)

#define RSSPECAN_VAL_VIDEO_TRIGGER_SLOPE_CLASS_EXT_BASE             (500L)
#define RSSPECAN_VAL_VIDEO_TRIGGER_SLOPE_SPECIFIC_EXT_BASE          (1000L)

    /*- Defined values for rsspecan_ReadXYTrace & rsspecan_ReadYTrace ----------*/

#define RSSPECAN_VAL_MAX_TIME_IMMEDIATE                             (0x0L)
#define RSSPECAN_VAL_MAX_TIME_INFINITE                              (0xFFFFFFFFUL)

    /*- Defined values for rsspecan_AcquisitionStatus ------------------------*/

//#define RSSPECAN_VAL_ACQUISITION_STATUS_COMPLETE                    (1L)
#define RSSPECAN_VAL_ACQUISITION_STATUS_IN_PROGRESS                 (0L)
#define RSSPECAN_VAL_ACQUISITION_STATUS_UNKNOWN                     (-1L)

#define RSSPECAN_VAL_ACQUISITION_STATUS_CLASS_EXT_BASE              (500L)
#define RSSPECAN_VAL_ACQUISITION_STATUS_SPECIFIC_EXT_BASE           (1000L)

    /*- Defined values for rsspecan_MarkerSearch -----------------------------*/

#define RSSPECAN_VAL_MARKER_SEARCH_HIGHEST                          (1L)
#define RSSPECAN_VAL_MARKER_SEARCH_MINIMUM                          (2L)
#define RSSPECAN_VAL_MARKER_SEARCH_NEXT_PEAK                        (3L)
#define RSSPECAN_VAL_MARKER_SEARCH_NEXT_PEAK_LEFT                   (4L)
#define RSSPECAN_VAL_MARKER_SEARCH_NEXT_PEAK_RIGHT                  (5L)
#define RSSPECAN_VAL_MARKER_SEARCH_NEXT_MINIMUM                     (6L)
#define RSSPECAN_VAL_MARKER_SEARCH_NEXT_MINIMUM_LEFT                (7L)
#define RSSPECAN_VAL_MARKER_SEARCH_NEXT_MINIMUM_RIGHT               (8L)

#define RSSPECAN_VAL_MARKER_SEARCH_CLASS_EXT_BASE                   (500L)
#define RSSPECAN_VAL_MARKER_SEARCH_SPECIFIC_EXT_BASE                (1000L)

    /*- Defined values for rsspecan_SetInstrumentFromMarker -----------------------*/

#define RSSPECAN_VAL_INSTRUMENT_SETTING_FREQUENCY_CENTER            (1L)
#define RSSPECAN_VAL_INSTRUMENT_SETTING_FREQUENCY_STEP              (2L)
#define RSSPECAN_VAL_INSTRUMENT_SETTING_REFERENCE_LEVEL             (3L)

#define RSSPECAN_VAL_INSTRUMENT_SETTING_CLASS_EXT_BASE              (500L)
#define RSSPECAN_VAL_INSTRUMENT_SETTING_SPECIFIC_EXT_BASE           (1000L)

    /*- Instrument specific attribute value definitions --------------------*/

/****************************************************************************
 *---------------- Constant definition  ------------------------------------*
 ****************************************************************************/
 /* Base */
#define RSSPECAN_VAL_OFF                   0
#define RSSPECAN_VAL_ON                    1
#define RSSPECAN_VAL_ALL                   2

#define RSSPECAN_VAL_EATT_OFF   0
#define RSSPECAN_VAL_EATT_AUTO  1
#define RSSPECAN_VAL_EATT_MAN   2

#define RSSPECAN_VAL_COUPLING_RBW        0
#define RSSPECAN_VAL_COUPLING_VBW        1
#define RSSPECAN_VAL_COUPLING_SWEEP_TIME 2

#define RSSPECAN_VAL_ADJ_ALL               0
#define RSSPECAN_VAL_ADJ_FREQ              1
#define RSSPECAN_VAL_ADJ_LEV               2

#define RSSPECAN_VAL_SAN_CREATE 0
#define RSSPECAN_VAL_SAN_SELECT 1
#define RSSPECAN_VAL_SAN_DELETE 2

#define RSSPECAN_VAL_LINE_THR           0
#define RSSPECAN_VAL_LINE_DISPLAY       1
#define RSSPECAN_VAL_LINE_FREQ          2
#define RSSPECAN_VAL_LINE_TIME          3

#define RSSPECAN_VAL_LIMIT_LOWER   0
#define RSSPECAN_VAL_LIMIT_UPPER   1
#define RSSPECAN_VAL_LIMIT_CONTROL 2

#define RSSPECAN_VAL_LOWER               0
#define RSSPECAN_VAL_UPPER               1

#define RSSPECAN_VAL_HCOPY_DEST_BMP  0
#define RSSPECAN_VAL_HCOPY_DEST_WMF  1
#define RSSPECAN_VAL_HCOPY_DEST_EMF  2
#define RSSPECAN_VAL_HCOPY_DEST_CLP  3
#define RSSPECAN_VAL_HCOPY_DEST_PRN  4
#define RSSPECAN_VAL_HCOPY_DEST_JPG  5
#define RSSPECAN_VAL_HCOPY_DEST_PNG  6
#define RSSPECAN_VAL_HCOPY_DEST_PDF  7
#define RSSPECAN_VAL_HCOPY_DEST_SVG  8
#define RSSPECAN_VAL_HCOPY_DEST_DOC  9
#define RSSPECAN_VAL_HCOPY_DEST_RTF  10

#define RSSPECAN_VAL_FORMAT_ASC  0
#define RSSPECAN_VAL_FORMAT_REAL 1

#define RSSPECAN_VAL_HCOP_ALL   0
#define RSSPECAN_VAL_HCOP_TABLE 1
#define RSSPECAN_VAL_HCOP_TRACE 2

#define RSSPECAN_VAL_FILE_NEW    0
#define RSSPECAN_VAL_FILE_COPY   1
#define RSSPECAN_VAL_FILE_RENAME 2
#define RSSPECAN_VAL_FILE_DELETE 3
#define RSSPECAN_VAL_FILE_RDIR   4
#define RSSPECAN_VAL_FILE_FORMAT 5
#define RSSPECAN_VAL_FILE_CDIR   6
#define RSSPECAN_VAL_FILE_CDISC  7
#define RSSPECAN_VAL_FILE_DELETE_IMM    8

#define RSSPECAN_VAL_MEM_SAVE    0
#define RSSPECAN_VAL_MEM_RECALL  1
#define RSSPECAN_VAL_MEM_DELETE  2
#define RSSPECAN_VAL_MEM_STARTUP 3

#define RSSPECAN_VAL_FILE_ITEMS_NONE            0
#define RSSPECAN_VAL_FILE_ITEMS_DEFAULT         1
#define RSSPECAN_VAL_FILE_ITEMS_ALL             2
#define RSSPECAN_VAL_FILE_ITEMS_HWSETTINGS      3
#define RSSPECAN_VAL_FILE_ITEMS_ALLTRACES       4
#define RSSPECAN_VAL_FILE_ITEMS_ALLLINES        5
#define RSSPECAN_VAL_FILE_ITEMS_ALLTRAN         7
#define RSSPECAN_VAL_FILE_ITEMS_SPECTROGRAM     8
#define RSSPECAN_VAL_FILE_ITEMS_IQ_DATA         11
#define RSSPECAN_VAL_FILE_ITEMS_WEIGHTING       12
#define RSSPECAN_VAL_FILE_ITEMS_CALIBRATION_SOURCE 13

#define RSSPECAN_VAL_USER_PORT_INP      0
#define RSSPECAN_VAL_USER_PORT_OUT      1

#define RSSPECAN_VAL_USTD_SET              0
#define RSSPECAN_VAL_USTD_SAVE             1
#define RSSPECAN_VAL_USTD_DEL              2

#define RSSPECAN_VAL_ACP_TX         0
#define RSSPECAN_VAL_ACP_ADJ        1
#define RSSPECAN_VAL_ACP_ALT        2

#define RSSPECAN_VAL_WFILTER_ALL           0
#define RSSPECAN_VAL_WFILTER_ADJ           1
#define RSSPECAN_VAL_WFILTER_ALT           2
#define RSSPECAN_VAL_WFILTER_TX            3

#define RSSPECAN_VAL_SSTAT_OFF      0
#define RSSPECAN_VAL_SSTAT_APD      1
#define RSSPECAN_VAL_SSTAT_CCDF     2

#define RSSPECAN_VAL_STAT_REG_PRESET 0

#define RSSPECAN_VAL_TDOM_RESULT              0
#define RSSPECAN_VAL_TDOM_AVG_RESULT          1
#define RSSPECAN_VAL_TDOM_MAX_HOLD_RESULT     2

#define RSSPECAN_VAL_TDOM_PEAK            0
#define RSSPECAN_VAL_TDOM_RMS             1
#define RSSPECAN_VAL_TDOM_MEAN            2
#define RSSPECAN_VAL_TDOM_SDEV            3

#define RSSPECAN_VAL_LIMIT_MOVE_OFFSET 0
#define RSSPECAN_VAL_LIMIT_MOVE_SHIFT  1

#define RSSPECAN_VAL_MEASTYPE_NONE  0
#define RSSPECAN_VAL_MEASTYPE_MIN   1
#define RSSPECAN_VAL_MEASTYPE_MAX   2
#define RSSPECAN_VAL_MEASTYPE_PEAK  3
#define RSSPECAN_VAL_MEASTYPE_AVER  4

#define RSSPECAN_VAL_WIGIG_MEAS_TDOM    0
#define RSSPECAN_VAL_WIGIG_MEAS_CREST   1

#define RSSPECAN_VAL_WIGIG_ERR_CFREQ 0
#define RSSPECAN_VAL_WIGIG_ERR_SYMB  1

#define RSSPECAN_VAL_WIGIG_IQ_OFFS   0
#define RSSPECAN_VAL_WIGIG_IQ_IMBA   1
#define RSSPECAN_VAL_WIGIG_IQ_QUAD   2

#define RSSPECAN_VAL_WIGIG_EVM_ALL   0
#define RSSPECAN_VAL_WIGIG_EVM_DATA  1
#define RSSPECAN_VAL_WIGIG_EVM_PILOT 2

#define RSSPECAN_VAL_WIGIG_TIME_FALL 0
#define RSSPECAN_VAL_WIGIG_TIME_RISE 1
#define RSSPECAN_VAL_WIGIG_TIME_SKEW 2

/* B10 - External Generator */
#define RSSPECAN_VAL_TGEN_CAL_TRAN       0
#define RSSPECAN_VAL_TGEN_CAL_REFLO      1
#define RSSPECAN_VAL_TGEN_CAL_REFLS      2

#define RSSPECAN_VAL_EXTGEN_SET         0
#define RSSPECAN_VAL_EXTGEN_MIN         1
#define RSSPECAN_VAL_EXTGEN_MAX         2

/* B9 - Tracking Generator */
#define RSSPECAN_VAL_TGEN_EXT_AM         0
#define RSSPECAN_VAL_TGEN_EXT_FM         1
#define RSSPECAN_VAL_TGEN_EXT_IQ         2

/* B17 - Digital Baseband*/
#define RSSPECAN_VAL_DIQ_IN                0
#define RSSPECAN_VAL_DIQ_OUT               1

/* B21 External Mixer */
#define RSSPECAN_VAL_MIXER_LOSS_AVER           0
#define RSSPECAN_VAL_MIXER_LOSS_TABL           1

/* Pulse Measurement */
#define RSSPECAN_VAL_PULSE_RESULT_X_AXIS_PNUMBER     0
#define RSSPECAN_VAL_PULSE_RESULT_X_AXIS_TSTAMP      1
#define RSSPECAN_VAL_PULSE_RESULT_POWER_ALL          0
#define RSSPECAN_VAL_PULSE_RESULT_PHASE_ALL          0
#define RSSPECAN_VAL_PULSE_RESULT_TIME_SIDELOBE_ALL  0
#define RSSPECAN_VAL_PULSE_RESULT_ENVELOPE_MODEL_ALL 0
#define RSSPECAN_VAL_PULSE_STABILITY_ALL             5

/* K7 - Analog Demodulation */
#define RSSPECAN_VAL_ADEM_PRESET_LOAD    0
#define RSSPECAN_VAL_ADEM_PRESET_STORE   1
#define RSSPECAN_VAL_ADEM_PRESET_RESTORE 2

#define RSSPECAN_VAL_FMDEM_FILTER_HP     0
#define RSSPECAN_VAL_FMDEM_FILTER_LP     1
#define RSSPECAN_VAL_FMDEM_FILTER_DEEMP  2
#define RSSPECAN_VAL_FMDEM_FILTER_LP_REL 3

#define RSSPECAN_VAL_FMDEM_RESULT_OFF  0
#define RSSPECAN_VAL_FMDEM_RESULT_CURR 1
#define RSSPECAN_VAL_FMDEM_RESULT_AVG  2
#define RSSPECAN_VAL_FMDEM_RESULT_MAX  3
#define RSSPECAN_VAL_FMDEM_RESULT_MIN  4
#define RSSPECAN_VAL_FMDEM_RESULT_VIEW 5

#define RSSPECAN_VAL_FMDEM_RLEN  0
#define RSSPECAN_VAL_FMDEM_SRATE 1

#define RSSPECAN_VAL_FMDEM_DET_PPE 0
#define RSSPECAN_VAL_FMDEM_DET_MPE 1
#define RSSPECAN_VAL_FMDEM_DET_MID 2
#define RSSPECAN_VAL_FMDEM_DET_RMS 3

#define RSSPECAN_VAL_FMDEM_MEAS_AFR 0
#define RSSPECAN_VAL_FMDEM_MEAS_FER 1
#define RSSPECAN_VAL_FMDEM_MEAS_SIN 2
#define RSSPECAN_VAL_FMDEM_MEAS_THD 3
#define RSSPECAN_VAL_FMDEM_MEAS_CAR 4

/* K7S */
#define RSSPECAN_VAL_FMS_HPAS_FILT_NONE        0
#define RSSPECAN_VAL_FMS_HPAS_FILT_20HZ        1
#define RSSPECAN_VAL_FMS_HPAS_FILT_50HZ        2
#define RSSPECAN_VAL_FMS_HPAS_FILT_300HZ       3

#define RSSPECAN_VAL_FMS_LOWPAS_FILT_NONE        0
#define RSSPECAN_VAL_FMS_LOWPAS_FILT_5PCT        1
#define RSSPECAN_VAL_FMS_LOWPAS_FILT_10PCT       2
#define RSSPECAN_VAL_FMS_LOWPAS_FILT_25PCT       3
#define RSSPECAN_VAL_FMS_LOWPAS_FILT_3KHZ        10
#define RSSPECAN_VAL_FMS_LOWPAS_FILT_15KHZ       11
#define RSSPECAN_VAL_FMS_LOWPAS_FILT_23KHZ       12
#define RSSPECAN_VAL_FMS_LOWPAS_FILT_150KHZ      13

#define RSSPECAN_VAL_FMS_WEIGHTING_NONE        0
#define RSSPECAN_VAL_FMS_WEIGHTING_CCIT        1
#define RSSPECAN_VAL_FMS_WEIGHTING_CCIRU       2
#define RSSPECAN_VAL_FMS_WEIGHTING_CCIRW       3
#define RSSPECAN_VAL_FMS_WEIGHTING_A           4

#define RSSPECAN_VAL_FMS_DEEMPH_NONE           0
#define RSSPECAN_VAL_FMS_DEEMPH_25             1
#define RSSPECAN_VAL_FMS_DEEMPH_50             2
#define RSSPECAN_VAL_FMS_DEEMPH_75             3
#define RSSPECAN_VAL_FMS_DEEMPH_750            4

#define RSSPECAN_VAL_FMS_AFSP                  0
#define RSSPECAN_VAL_FMS_TDOM                  1

#define RSSPECAN_VAL_FMS_MEAS_TYPE_ADEV        0
#define RSSPECAN_VAL_FMS_MEAS_TYPE_RDEV        1
#define RSSPECAN_VAL_FMS_MEAS_TYPE_SINAD       2
#define RSSPECAN_VAL_FMS_MEAS_TYPE_THD         3
#define RSSPECAN_VAL_FMS_MEAS_TYPE_AFR         4
#define RSSPECAN_VAL_FSM_MEAS_TYPE_XTALK       5

/* K8 Bluetooth constants*/

#define RSSPECAN_VAL_MEASTYPE_MAXMIN    0
#define RSSPECAN_VAL_MEASTYPE_MAXMAX    1
#define RSSPECAN_VAL_MEASTYPE_MAXAVER   2
#define RSSPECAN_VAL_MEASTYPE_AVERMIN   3
#define RSSPECAN_VAL_MEASTYPE_AVERMAX   4

#define RSSPECAN_VAL_BTO_CFS_FERR_INITIAL           0
#define RSSPECAN_VAL_BTO_CFS_FERR_BLOCK             1
#define RSSPECAN_VAL_BTO_CFS_FERR_TOTAL             2

#define RSSPECAN_VAL_BTO_CFS_DEVM_RMS               0
#define RSSPECAN_VAL_BTO_CFS_DEVM_99PCT             1
#define RSSPECAN_VAL_BTO_CFS_DEVM_PEAK              2

#define RSSPECAN_VAL_BTO_DPEN_TESTED                0
#define RSSPECAN_VAL_BTO_DPEN_BER                   1
#define RSSPECAN_VAL_BTO_DPEN_PASSED                2

#define RSSPECAN_VAL_BTO_SE_TX                  0
#define RSSPECAN_VAL_BTO_SE_HADJ_LOW            1
#define RSSPECAN_VAL_BTO_SE_HADJ_UPP            2

#define RSSPECAN_VAL_BTO_RTP_GFSK           0
#define RSSPECAN_VAL_BTO_RTP_DPSK           1
#define RSSPECAN_VAL_BTO_RTP_RATIO          2

/* K9 Power Meter - constants */
#define RSSPECAN_VAL_PWMET_MEASTIME_NORM    0
#define RSSPECAN_VAL_PWMET_MEASTIME_SHORT   1
#define RSSPECAN_VAL_PWMET_MEASTIME_LONG    2
#define RSSPECAN_VAL_PWMET_MEASTIME_MANUAL 3

/* K10 constants*/
#define RSSPECAN_VAL_GSM_K10_FREQ_BAND_TGSM_380 0
#define RSSPECAN_VAL_GSM_K10_FREQ_BAND_TGSM_410 1
#define RSSPECAN_VAL_GSM_K10_FREQ_BAND_GSM_450  2
#define RSSPECAN_VAL_GSM_K10_FREQ_BAND_GSM_480  3
#define RSSPECAN_VAL_GSM_K10_FREQ_BAND_GSM_710  4
#define RSSPECAN_VAL_GSM_K10_FREQ_BAND_GSM_750  5
#define RSSPECAN_VAL_GSM_K10_FREQ_BAND_TGSM_810 6
#define RSSPECAN_VAL_GSM_K10_FREQ_BAND_GSM_850  7
#define RSSPECAN_VAL_GSM_K10_FREQ_BAND_PGSM_900 8
#define RSSPECAN_VAL_GSM_K10_FREQ_BAND_EGSM_900 9
#define RSSPECAN_VAL_GSM_K10_FREQ_BAND_RGSM_900 10
#define RSSPECAN_VAL_GSM_K10_FREQ_BAND_TGSM_900 11
#define RSSPECAN_VAL_GSM_K10_FREQ_BAND_DCS_1800 12
#define RSSPECAN_VAL_GSM_K10_FREQ_BAND_PCS_1900 13

#define RSSPECAN_VAL_GSM_K10_MACC  0    // - Modulation Accuracy
#define RSSPECAN_VAL_GSM_K10_PFER  1    // - Phase Error vs Time
#define RSSPECAN_VAL_GSM_K10_ETIM  2    // - EVM vs Time
#define RSSPECAN_VAL_GSM_K10_MERR  3    // - Magnitude Error
#define RSSPECAN_VAL_GSM_K10_CONST 4    // - Constellation
#define RSSPECAN_VAL_GSM_K10_PTEM  5    // - Power vs. Time
#define RSSPECAN_VAL_GSM_K10_MOD   6    // - Modulation Spectrum
#define RSSPECAN_VAL_GSM_K10_SWIT  7    // - Transient Spectrum
#define RSSPECAN_VAL_GSM_K10_WIDE  8    // - Wide Spectrum
#define RSSPECAN_VAL_GSM_K10_TS    9    // - Trigger to Sync

#define RSSPECAN_VAL_GSM_K10_MEAS_EVM_RMS         0
#define RSSPECAN_VAL_GSM_K10_MEAS_EVM_PEAK        1
#define RSSPECAN_VAL_GSM_K10_MEAS_MERR_RMS        2
#define RSSPECAN_VAL_GSM_K10_MEAS_MERR_PEAK       3
#define RSSPECAN_VAL_GSM_K10_MEAS_PERR_RMS        4
#define RSSPECAN_VAL_GSM_K10_MEAS_PERR_PEAK       5
#define RSSPECAN_VAL_GSM_K10_MEAS_OSUPP           6
#define RSSPECAN_VAL_GSM_K10_MEAS_IQOF            7
#define RSSPECAN_VAL_GSM_K10_MEAS_IQIM            8
#define RSSPECAN_VAL_GSM_K10_MEAS_FREQ            9
#define RSSPECAN_VAL_GSM_K10_MEAS_BPOW            10
#define RSSPECAN_VAL_GSM_K10_MEAS_ADR             11
#define RSSPECAN_VAL_GSM_K10_MEAS_PERC_EVM        12
#define RSSPECAN_VAL_GSM_K10_MEAS_PERC_MERR       13
#define RSSPECAN_VAL_GSM_K10_MEAS_PERC_PERR       14
#define RSSPECAN_VAL_GSM_K10_MEAS_TRIGGER_TO_SYNC 15

#define RSSPECAN_VAL_GSM_K10_MOD_AVG            0
#define RSSPECAN_VAL_GSM_K10_MOD_CUR            1
#define RSSPECAN_VAL_GSM_K10_MOD_MAX            2
#define RSSPECAN_VAL_GSM_K10_MOD_DEV            3
#define RSSPECAN_VAL_GSM_K10_MOD_MIN            4

#define RSSPECAN_VAL_GSM_K10_BURST_ALL          0
#define RSSPECAN_VAL_GSM_K10_BURST_CURR         1

#define RSSPECAN_VAL_GSM_K10_BURST_MOD_AVG_PWR  0
#define RSSPECAN_VAL_GSM_K10_BURST_MOD_CREST    1
#define RSSPECAN_VAL_GSM_K10_BURST_MOD_MAX_PWR  2

#define RSSPECAN_VAL_GSM_K10_AUTO_OFF           0
#define RSSPECAN_VAL_GSM_K10_AUTO_ON            1
#define RSSPECAN_VAL_GSM_K10_AUTO_ONCE          2

#define RSSPECAN_VAL_GSM_K10_MOD_SPEC    0
#define RSSPECAN_VAL_GSM_K10_TRANS_SPEC  1
#define RSSPECAN_VAL_GSM_K10_WIDE_SPEC   2

#define RSSPECAN_VAL_GSM_K10_LIM_TYPE_CURR 0
#define RSSPECAN_VAL_GSM_K10_LIM_TYPE_MAX  1

#define RSSPECAN_VAL_GSM_IM_INNER 0
#define RSSPECAN_VAL_GSM_IM_OUTER 1

#define RSSPECAN_VAL_GSM_RPOW_NONE    0
#define RSSPECAN_VAL_GSM_RPOW_REF     1
#define RSSPECAN_VAL_GSM_RPOW_MAX     2

/* End of K10 constants*/

/* K14 */
#define RSSPECAN_VAL_SPECM_MARK_MAX_PEAK     0
#define RSSPECAN_VAL_SPECM_MARK_MAX_NEXT     1
#define RSSPECAN_VAL_SPECM_MARK_MAX_LEFT     2
#define RSSPECAN_VAL_SPECM_MARK_MAX_RIGHT    3
#define RSSPECAN_VAL_SPECM_MARK_MIN_PEAK     4
#define RSSPECAN_VAL_SPECM_MARK_MIN_NEXT     5
#define RSSPECAN_VAL_SPECM_MARK_MIN_LEFT     6
#define RSSPECAN_VAL_SPECM_MARK_MIN_RIGHT    7
#define RSSPECAN_VAL_SPECM_MARK_Y_MAX_PEAK   8
#define RSSPECAN_VAL_SPECM_MARK_Y_MAX_NEXT   9
#define RSSPECAN_VAL_SPECM_MARK_Y_MAX_ABOVE 10
#define RSSPECAN_VAL_SPECM_MARK_Y_MAX_BELOW 11
#define RSSPECAN_VAL_SPECM_MARK_Y_MIN_PEAK  12
#define RSSPECAN_VAL_SPECM_MARK_Y_MIN_NEXT  13
#define RSSPECAN_VAL_SPECM_MARK_Y_MIN_ABOVE 14
#define RSSPECAN_VAL_SPECM_MARK_Y_MIN_BELOW 15
#define RSSPECAN_VAL_SPECM_MARK_XY_MAX      16
#define RSSPECAN_VAL_SPECM_MARK_XY_MIN      17
#define RSSPECAN_VAL_SPECM_MARK_Y_TRIGGER   18

#define RSSPECAN_VAL_SPECM_TSTAMP_CURR   0
#define RSSPECAN_VAL_SPECM_TSTAMP_ALL   1

#define RSSPECAN_VAL_SPECM_FRAME_CURRENT 0
#define RSSPECAN_VAL_SPECM_FRAME_ALL     1

/* K30 - Noise Figure - constants  */
#define RSSPECAN_VAL_NOISE_DUT_MODE_DIRECT        0
#define RSSPECAN_VAL_NOISE_DUT_MODE_FIXED_IF_PLUS 1
#define RSSPECAN_VAL_NOISE_DUT_MODE_FIXED_IF_ABS  2
#define RSSPECAN_VAL_NOISE_DUT_MODE_FIXED_LO_PLUS 3
#define RSSPECAN_VAL_NOISE_DUT_MODE_FIXED_LO_ABS  4

#define RSSPECAN_VAL_NOISE_SECOND_STAGE_CORR_MEAS  0
#define RSSPECAN_VAL_NOISE_CONT_FREQ_LIST_MEAS     1
#define RSSPECAN_VAL_NOISE_SINGLE_FREQ_LIST_MEAS   2
#define RSSPECAN_VAL_NOISE_SINGLE_FREQ_MEAS        3
#define RSSPECAN_VAL_NOISE_CONT_FREQ_SINGLE_MEAS   4
#define RSSPECAN_VAL_NOISE_SINGLE_FREQ_SINGLE_MEAS 5

#define RSSPECAN_VAL_DISP_FORMAT_SING             0
#define RSSPECAN_VAL_DISP_FORMAT_SPL              1

#define RSSPECAN_VAL_NOISE_DISP_DATA_OFF          0

#define RSSPECAN_VAL_NOISE_NOISE                1
#define RSSPECAN_VAL_NOISE_GAIN                 2

#define RSSPECAN_VAL_NOISE_RESULT_TYPE_GAIN  0
#define RSSPECAN_VAL_NOISE_RESULT_TYPE_NOISE 1
#define RSSPECAN_VAL_NOISE_RESULT_TYPE_PCOLD 2
#define RSSPECAN_VAL_NOISE_RESULT_TYPE_PHOT  3
#define RSSPECAN_VAL_NOISE_RESULT_TYPE_TEMP  4
#define RSSPECAN_VAL_NOISE_RESULT_TYPE_YFAC  5
#define RSSPECAN_VAL_NOISE_RESULT_TYPE_CYF   6
#define RSSPECAN_VAL_NOISE_RESULT_TYPE_CPH   7
#define RSSPECAN_VAL_NOISE_RESULT_TYPE_CPC   8
#define RSSPECAN_VAL_NOISE_RESULT_TYPE_NUNC  9
#define RSSPECAN_VAL_NOISE_RESULT_TYPE_ENR   10

/* K40 - Phase Noise - constants  */
#define RSSPECAN_VAL_PHASE_RFM 0
#define RSSPECAN_VAL_PHASE_RPM 1
#define RSSPECAN_VAL_PHASE_RMS 2

#define RSSPECAN_VAL_DC_PORT_SUPPLY 0
#define RSSPECAN_VAL_DC_PORT_TUNE   1
#define RSSPECAN_VAL_DC_PORT_AUX    2

#define RSSPECAN_VAL_JITTER_DISCRETE 0
#define RSSPECAN_VAL_JITTER_RANDOM   1

#define RSSPECAN_VAL_PHASE_RESIDUAL_IPN 0
#define RSSPECAN_VAL_PHASE_RESIDUAL_RFM 1
#define RSSPECAN_VAL_PHASE_RESIDUAL_RMS 2
#define RSSPECAN_VAL_PHASE_RESIDUAL_RPM 3
#define RSSPECAN_VAL_PHASE_RESIDUAL_RAM 4

/* WCDMA FDD K72/73/74*/
#define RSSPECAN_VAL_CDP_TABLE_OFF          0
#define RSSPECAN_VAL_CDP_TABLE_SELECT       1
#define RSSPECAN_VAL_CDP_TABLE_NEW          2
#define RSSPECAN_VAL_CDP_TABLE_DELETE       3
#define RSSPECAN_VAL_CDP_TABLE_COPY         4

#define RSSPECAN_VAL_CDP_TABLE_CHANNEL_DPCH     0
#define RSSPECAN_VAL_CDP_TABLE_CHANNEL_PICH     1
#define RSSPECAN_VAL_CDP_TABLE_CHANNEL_SCCPCH   2
#define RSSPECAN_VAL_CDP_TABLE_CHANNEL_HSSCCH   3
#define RSSPECAN_VAL_CDP_TABLE_CHANNEL_HSPDSCH  4
#define RSSPECAN_VAL_CDP_TABLE_CHANNEL_CHAN     5
#define RSSPECAN_VAL_CDP_TABLE_CHANNEL_CPRSD    10
#define RSSPECAN_VAL_CDP_TABLE_CHANNEL_CPRTPC   11
#define RSSPECAN_VAL_CDP_TABLE_CHANNEL_CPRSF    12
#define RSSPECAN_VAL_CDP_TABLE_CHANNEL_CPRSFTPC 13
#define RSSPECAN_VAL_CDP_TABLE_CHANNEL_EHICH    14
#define RSSPECAN_VAL_CDP_TABLE_CHANNEL_EAGCH    15
#define RSSPECAN_VAL_CDP_TABLE_CHANNEL_SCPICH   16

#define RSSPECAN_VAL_3GPP_MARKER_CPICH  0
#define RSSPECAN_VAL_3GPP_MARKER_PCCPCH 1

#define RSSPECAN_VAL_3GBS_TRAC_CDP           0
#define RSSPECAN_VAL_3GBS_TRAC_CDPRAT        1
#define RSSPECAN_VAL_3GBS_TRAC_CDEP          2
#define RSSPECAN_VAL_3GBS_TRAC_CTAB          3
#define RSSPECAN_VAL_3GBS_TRAC_CWCD          4
#define RSSPECAN_VAL_3GBS_TRAC_PWCD          5
#define RSSPECAN_VAL_3GBS_TRAC_PCD           6
#define RSSPECAN_VAL_3GBS_TRAC_MACC          7
#define RSSPECAN_VAL_3GBS_TRAC_PVSLA         8
#define RSSPECAN_VAL_3GBS_TRAC_PVSLR         9
#define RSSPECAN_VAL_3GBS_TRAC_BSTR          10
#define RSSPECAN_VAL_3GBS_TRAC_SYMBCONST     11
#define RSSPECAN_VAL_3GBS_TRAC_SYMBEVM       12
#define RSSPECAN_VAL_3GBS_TRAC_FVSL          13
#define RSSPECAN_VAL_3GBS_TRAC_PVSY          14
#define RSSPECAN_VAL_3GBS_TRAC_EVM_MAGN      15
#define RSSPECAN_VAL_3GBS_TRAC_EVM_PHAS      16
#define RSSPECAN_VAL_3GBS_TRAC_COMP_CONS     17
#define RSSPECAN_VAL_3GBS_TRAC_CHIP_EVM      18
#define RSSPECAN_VAL_3GBS_TRAC_CHIP_MAGN     19
#define RSSPECAN_VAL_3GBS_TRAC_CHIP_PHAS     20
#define RSSPECAN_VAL_3GBS_TRAC_ATRACE2       21

#define RSSPECAN_VAL_3GUE_TRAC_CDP          0
#define RSSPECAN_VAL_3GUE_TRAC_CDPRAT       1
#define RSSPECAN_VAL_3GUE_TRAC_CDPOVER      2
#define RSSPECAN_VAL_3GUE_TRAC_CDEP         3
#define RSSPECAN_VAL_3GUE_TRAC_CTAB         4
#define RSSPECAN_VAL_3GUE_TRAC_PCD          5
#define RSSPECAN_VAL_3GUE_TRAC_MACC         6
#define RSSPECAN_VAL_3GUE_TRAC_PVSLA        7
#define RSSPECAN_VAL_3GUE_TRAC_PVSLR        8
#define RSSPECAN_VAL_3GUE_TRAC_BSTR         9
#define RSSPECAN_VAL_3GUE_TRAC_PVSY         10
#define RSSPECAN_VAL_3GUE_TRAC_SYMBCONST    11
#define RSSPECAN_VAL_3GUE_TRAC_SYMBEVM      12
#define RSSPECAN_VAL_3GUE_TRAC_FVSL         13
#define RSSPECAN_VAL_3GUE_TRAC_PSVSL        14
#define RSSPECAN_VAL_3GUE_TRAC_EVM_MAGN     15
#define RSSPECAN_VAL_3GUE_TRAC_EVM_PHAS     16
#define RSSPECAN_VAL_3GUE_TRAC_COMP_CONS    17
#define RSSPECAN_VAL_3GUE_TRAC_CHIP_EVM     18
#define RSSPECAN_VAL_3GUE_TRAC_CHIP_MAGN    19
#define RSSPECAN_VAL_3GUE_TRAC_CHIP_PHAS    20
#define RSSPECAN_VAL_3GUE_TRAC_EVM_RMS      21
/* End of K72/73/74 constants*/

/* K76/77*/
#define RSSPECAN_VAL_TDS_TABLE_OFF      0
#define RSSPECAN_VAL_TDS_TABLE_SELECT   1
#define RSSPECAN_VAL_TDS_TABLE_NEW      2
#define RSSPECAN_VAL_TDS_TABLE_DELETE   3
#define RSSPECAN_VAL_TDS_TABLE_COPY     4

#define RSSPECAN_VAL_TDS_TABLE_CHANNEL_INAC     0
#define RSSPECAN_VAL_TDS_TABLE_CHANNEL_MID      1
#define RSSPECAN_VAL_TDS_TABLE_CHANNEL_DPCH     2
#define RSSPECAN_VAL_TDS_TABLE_CHANNEL_PCCPCH   3
#define RSSPECAN_VAL_TDS_TABLE_CHANNEL_SCCPCH   4
#define RSSPECAN_VAL_TDS_TABLE_CHANNEL_FPACH    5
#define RSSPECAN_VAL_TDS_TABLE_CHANNEL_PDSCH    6
#define RSSPECAN_VAL_TDS_TABLE_CHANNEL_PICH     7

#define RSSPECAN_VAL_TDS_MOD_INVALID        0
#define RSSPECAN_VAL_TDS_MOD_QPSK           1
#define RSSPECAN_VAL_TDS_MOD_8PSK           2
#define RSSPECAN_VAL_TDS_MOD_16QAM          3
#define RSSPECAN_VAL_TDS_MOD_64QAM          4

/* CDMA2000 */
#define RSSPECAN_VAL_TABLE_OFF          0
#define RSSPECAN_VAL_TABLE_SELECT       1
#define RSSPECAN_VAL_TABLE_NEW          2
#define RSSPECAN_VAL_TABLE_DELETE       3
#define RSSPECAN_VAL_TABLE_COPY         4

/* 91 - WLAN constants  */
#define RSSPECAN_VAL_WLAN_GATE_TIME      0
#define RSSPECAN_VAL_WLAN_GATE_SAMPLE    1

#define RSSPECAN_VAL_WLAN_EVM_MODE_SYMBOL    0
#define RSSPECAN_VAL_WLAN_EVM_MODE_CARRIER   1
#define RSSPECAN_VAL_WLAN_EVM_MODE_PHAS      2
#define RSSPECAN_VAL_WLAN_EVM_MODE_FREQ      3
#define RSSPECAN_VAL_WLAN_EVM_MODE_CHIP      4

#define RSSPECAN_VAL_WLAN_SPEC_MODE_IEEE     0
#define RSSPECAN_VAL_WLAN_SPEC_MODE_ETSI     1
#define RSSPECAN_VAL_WLAN_SPEC_MODE_FFT      2
#define RSSPECAN_VAL_WLAN_SPEC_MODE_ACPR     3
#define RSSPECAN_VAL_WLAN_SPEC_MODE_FLAT     4

#define RSSPECAN_VAL_WLAN_CONS_SYM       0
#define RSSPECAN_VAL_WLAN_CONS_CARR      1

#define RSSPECAN_VAL_WLAN_STAT_CCDF     0
#define RSSPECAN_VAL_WLAN_STAT_BSTR     1
#define RSSPECAN_VAL_WLAN_STAT_SFI      2

#define RSSPECAN_VAL_WLAN_POS_X          0
#define RSSPECAN_VAL_WLAN_POS_Y          1

#define RSSPECAN_VAL_WLAN_DEM_DUR_MS     0
#define RSSPECAN_VAL_WLAN_DEM_DUR_BYTE   1

#define RSSPECAN_VAL_WLAN_LIM_FERR      0
#define RSSPECAN_VAL_WLAN_LIM_SYMB      1
#define RSSPECAN_VAL_WLAN_LIM_IQOFF     2
#define RSSPECAN_VAL_WLAN_LIM_EVM_ALL   3
#define RSSPECAN_VAL_WLAN_LIM_EVM_DATA  4
#define RSSPECAN_VAL_WLAN_LIM_EVM_PIL   5
#define RSSPECAN_VAL_WLAN_LIM_RTIME     6
#define RSSPECAN_VAL_WLAN_LIM_FTIME     7
#define RSSPECAN_VAL_WLAN_LIM_EVM       8

#define RSSPECAN_VAL_WLAN_ACP_ADJ       0
#define RSSPECAN_VAL_WLAN_ACP_ALT       1

#define RSSPECAN_VAL_ACP_CURR           0
#define RSSPECAN_VAL_ACP_MAX            1

#define RSSPECAN_VAL_WLAN_MEAS_PREA       0
#define RSSPECAN_VAL_WLAN_MEAS_PAYL       1
#define RSSPECAN_VAL_WLAN_MEAS_RMS        2
#define RSSPECAN_VAL_WLAN_MEAS_PEAK       3
#define RSSPECAN_VAL_WLAN_MEAS_CREST      4
#define RSSPECAN_VAL_WLAN_MEAS_MCPOW_MIN  9
#define RSSPECAN_VAL_WLAN_MEAS_MCPOW_MAX  10
#define RSSPECAN_VAL_WLAN_MEAS_MCPOW_AVER 11

#define RSSPECAN_VAL_WLAN_TIME_FALL     0
#define RSSPECAN_VAL_WLAN_TIME_RISE     1

#define RSSPECAN_VAL_WLAN_ERR_FREQ      0
#define RSSPECAN_VAL_WLAN_ERR_SYMB      1
#define RSSPECAN_VAL_WLAN_ERR_CPER      2

#define RSSPECAN_VAL_WLAN_IQ_OFFS       0
#define RSSPECAN_VAL_WLAN_IQ_IMBA       1
#define RSSPECAN_VAL_WLAN_IQ_QUAD       2
#define RSSPECAN_VAL_WLAN_IQ_SKEW       3

#define RSSPECAN_VAL_WLAN_EVM_ALL       0
#define RSSPECAN_VAL_WLAN_EVM_DATA      1
#define RSSPECAN_VAL_WLAN_EVM_PILOT     2
#define RSSPECAN_VAL_WLAN_EVM           3
#define RSSPECAN_VAL_WLAN_EVM_PEAK      4

#define RSSPECAN_VAL_WLAN_AM_AM         0
#define RSSPECAN_VAL_WLAN_AM_PM         1
#define RSSPECAN_VAL_WLAN_AM_EVM        2

#define RSSPECAN_VAL_WLAN_SCD_SUBCARRIERS     0
#define RSSPECAN_VAL_WLAN_SCD_RU              1
#define RSSPECAN_VAL_WLAN_SCD_SUBCARRIERS_RUS 2

/* K93 */
#define RSSPECAN_VAL_X  0
#define RSSPECAN_VAL_Y  1

#define RSSPECAN_VAL_WIMAX_GATE_TIME    0
#define RSSPECAN_VAL_WIMAX_GATE_SAMPLE  1

#define RSSPECAN_VAL_WIMAX_MEASUREMENT_RF   0
#define RSSPECAN_VAL_WIMAX_MEASUREMENT_IQ   1

#define RSSPECAN_VAL_WIMAX_CONS_SYM     0
#define RSSPECAN_VAL_WIMAX_CONS_CARR    1
#define RSSPECAN_VAL_WIMAX_CONS_BURST   2

#define RSSPECAN_VAL_POW                1

#define RSSPECAN_VAL_WIMAX_PREDEF_MAP_CONFIG   0
#define RSSPECAN_VAL_WIMAX_PREDEF_MAP_IQDL     1
#define RSSPECAN_VAL_WIMAX_PREDEF_MAP_IQUL     2
#define RSSPECAN_VAL_WIMAX_PREDEF_MAP_SMU      3

#define RSSPECAN_VAL_SELECTION_NUM    2

#define RSSPECAN_VAL_WIMAX_MEAS_RMS       0
#define RSSPECAN_VAL_WIMAX_MEAS_CREST     1
#define RSSPECAN_VAL_WIMAX_MEAS_RMS_ALL   2
#define RSSPECAN_VAL_WIMAX_MEAS_RMS_PILOT 3
#define RSSPECAN_VAL_WIMAX_MEAS_RMS_DATA  4
#define RSSPECAN_VAL_WIMAX_MEAS_RMS_DLPR  5

#define RSSPECAN_VAL_WIMAX_ERR_FREQ      0
#define RSSPECAN_VAL_WIMAX_ERR_SYMB      1

#define RSSPECAN_VAL_WIMAX_IQ_OFFS 0
#define RSSPECAN_VAL_WIMAX_IQ_IMBA 1
#define RSSPECAN_VAL_WIMAX_IQ_QUAD 2

#define RSSPECAN_VAL_WIMAX_EVM_ALL       0
#define RSSPECAN_VAL_WIMAX_EVM_DATA      1
#define RSSPECAN_VAL_WIMAX_EVM_PILOT     2
#define RSSPECAN_VAL_WIMAX_EVM_PREAM     4

#define RSSPECAN_VAL_WIMAX_BURS          0
#define RSSPECAN_VAL_WIMAX_SYMB          1
#define RSSPECAN_VAL_WIMAX_ZONE          2
#define RSSPECAN_VAL_WIMAX_STAT_ZONE     3

#define RSSPECAN_VAL_WIMAX_LIM_FERR      0
#define RSSPECAN_VAL_WIMAX_LIM_SYMB      1
#define RSSPECAN_VAL_WIMAX_LIM_SST       2
#define RSSPECAN_VAL_WIMAX_LIM_EVM_ALL   3
#define RSSPECAN_VAL_WIMAX_LIM_EVM_DATA  4
#define RSSPECAN_VAL_WIMAX_LIM_IQOFF     5
#define RSSPECAN_VAL_WIMAX_LIM_BERP      6
#define RSSPECAN_VAL_WIMAX_LIM_USERR     7

#define RSSPECAN_VAL_WIMAX_MODE_IEEE       0
#define RSSPECAN_VAL_WIMAX_MODE_ETSI       1
#define RSSPECAN_VAL_WIMAX_MODE_FFT        2
#define RSSPECAN_VAL_WIMAX_MODE_FLAT       4
#define RSSPECAN_VAL_WIMAX_MODE_FLAT_GRDEL 5
#define RSSPECAN_VAL_WIMAX_MODE_FLAT_DIFF  6
#define RSSPECAN_VAL_WIMAX_MODE_ACPR_ABS   7
#define RSSPECAN_VAL_WIMAX_MODE_ACPR_REL   8
#define RSSPECAN_VAL_WIMAX_MODE_MASK       9
#define RSSPECAN_VAL_WIMAX_MODE_PREAMBLE   10
#define RSSPECAN_VAL_WIMAX_MODE_ETSI5      11
#define RSSPECAN_VAL_WIMAX_MODE_ETSI10     12

#define RSSPECAN_VAL_WIMAX_EVM_MODE_SYMBOL    0
#define RSSPECAN_VAL_WIMAX_EVM_MODE_CARRIER   1
#define RSSPECAN_VAL_WIMAX_EVM_MODE_PHAS  2
#define RSSPECAN_VAL_WIMAX_EVM_MODE_FREQ  3

#define RSSPECAN_VAL_WIMAX_STAT_CCDF    0
#define RSSPECAN_VAL_WIMAX_STAT_BSTR    1
#define RSSPECAN_VAL_WIMAX_STAT_BSUM    2

#define RSSPECAN_VAL_WIMAX_PREA          0
#define RSSPECAN_VAL_WIMAX_SUBF          1

 //
/* K70 constants */
#define RSSPECAN_VAL_SYNC_PATT_ADD          0
#define RSSPECAN_VAL_SYNC_PATT_SELECT       1
#define RSSPECAN_VAL_SYNC_PATT_DELETE       2
#define RSSPECAN_VAL_SYNC_PATT_REMOVE       3
#define RSSPECAN_VAL_SYNC_PATT_REMOVE_ALL   4
#define RSSPECAN_VAL_SYNC_PATT_COPY         5

#define RSSPECAN_VAL_VSA_PATTERN_ALL        0
#define RSSPECAN_VAL_VSA_PATTERN_CURRENT    1

#define RSSPECAN_VAL_VSA_EQUALIZER_LOAD              0
#define RSSPECAN_VAL_VSA_EQUALIZER_SAVE              1
#define RSSPECAN_VAL_VSA_EQUALIZER_RESET             2

#define RSSPECAN_VAL_VSA_DEM_RLEN_AUTO_ON               0
#define RSSPECAN_VAL_VSA_DEM_RLEN_SYMBOL                1
#define RSSPECAN_VAL_VSA_DEM_RLEN_TIME                  2

#define RSSPECAN_VAL_VSA_MPOW            0
#define RSSPECAN_VAL_VSA_DTTS            1
#define RSSPECAN_VAL_VSA_ADR             2
#define RSSPECAN_VAL_VSA_CFER            3
#define RSSPECAN_VAL_VSA_EVM             4
#define RSSPECAN_VAL_VSA_FSK_CDR         5
#define RSSPECAN_VAL_VSA_FSK_DERR        6
#define RSSPECAN_VAL_VSA_FSK_MDEV        7
#define RSSPECAN_VAL_VSA_IQIM            8
#define RSSPECAN_VAL_VSA_GIMB            9
#define RSSPECAN_VAL_VSA_MERR            10
#define RSSPECAN_VAL_VSA_OOFF            11
#define RSSPECAN_VAL_VSA_PERR            12
#define RSSPECAN_VAL_VSA_PLER            13
#define RSSPECAN_VAL_VSA_QERR            14
#define RSSPECAN_VAL_VSA_RHO             15
#define RSSPECAN_VAL_VSA_SNR             16
#define RSSPECAN_VAL_VSA_FDER            17
#define RSSPECAN_VAL_VSA_FSK_RDEV        18
#define RSSPECAN_VAL_VSA_SR_ERROR        19
#define RSSPECAN_VAL_VSA_IQSK            20

#define RSSPECAN_VAL_VSA_MODIF_NONE     0
#define RSSPECAN_VAL_VSA_MODIF_RMS      1
#define RSSPECAN_VAL_VSA_MODIF_AVG      2
#define RSSPECAN_VAL_VSA_MODIF_SDEV     3
#define RSSPECAN_VAL_VSA_MODIF_TPE      4
#define RSSPECAN_VAL_VSA_MODIF_PEAK     5
#define RSSPECAN_VAL_VSA_MODIF_ASYM     6
#define RSSPECAN_VAL_VSA_MODIF_PCTL     7
#define RSSPECAN_VAL_VSA_MODIF_FSV_CURR 8
#define RSSPECAN_VAL_VSA_MODIF_FSV_AVG  9
#define RSSPECAN_VAL_VSA_MODIF_FSV_RPE  10
#define RSSPECAN_VAL_VSA_MODIF_FSV_SDEV 11
#define RSSPECAN_VAL_VSA_MODIF_FSV_PCTL 12
#define RSSPECAN_VAL_VSA_MODIF_FSV_PEAK 13
#define RSSPECAN_VAL_VSA_MODIF_FSV_PAVG 14
#define RSSPECAN_VAL_VSA_MODIF_FSV_TPE  15
#define RSSPECAN_VAL_VSA_MODIF_FSV_PSD  16
#define RSSPECAN_VAL_VSA_MODIF_FSV_PPCT 17

#define RSSPECAN_VAL_VSA_CPOINT_PSHIFT  0
#define RSSPECAN_VAL_VSA_CPOINT_POWER   1
#define RSSPECAN_VAL_VSA_CPOINT_DATA    2

#define RSSPECAN_VAL_VSA_MOD_ORDER_BPSK       0
#define RSSPECAN_VAL_VSA_MOD_ORDER_QPSK       1
#define RSSPECAN_VAL_VSA_MOD_ORDER_OQPSK      2
#define RSSPECAN_VAL_VSA_MOD_ORDER_DQPSK      3
#define RSSPECAN_VAL_VSA_MOD_ORDER_PI4QPSK    4
#define RSSPECAN_VAL_VSA_MOD_ORDER_DPI4QPSK   5
#define RSSPECAN_VAL_VSA_MOD_ORDER_3PI4QPSK   6
#define RSSPECAN_VAL_VSA_MOD_ORDER_8PSK       7
#define RSSPECAN_VAL_VSA_MOD_ORDER_D8PSK      8
#define RSSPECAN_VAL_VSA_MOD_ORDER_N3PI8PSK   9
#define RSSPECAN_VAL_VSA_MOD_ORDER_PI8D8PSK   10
#define RSSPECAN_VAL_VSA_MOD_ORDER_MSK        11
#define RSSPECAN_VAL_VSA_MOD_ORDER_DMSK       12
#define RSSPECAN_VAL_VSA_MOD_ORDER_16QAM      13
#define RSSPECAN_VAL_VSA_MOD_ORDER_PI4_16QAM  14
#define RSSPECAN_VAL_VSA_MOD_ORDER_32QAM      15
#define RSSPECAN_VAL_VSA_MOD_ORDER_NPI4_32QAM 16
#define RSSPECAN_VAL_VSA_MOD_ORDER_64QAM      17
#define RSSPECAN_VAL_VSA_MOD_ORDER_128QAM     18
#define RSSPECAN_VAL_VSA_MOD_ORDER_256QAM     19
#define RSSPECAN_VAL_VSA_MOD_ORDER_2FSK       20
#define RSSPECAN_VAL_VSA_MOD_ORDER_4FSK       21
#define RSSPECAN_VAL_VSA_MOD_ORDER_8FSK       22
#define RSSPECAN_VAL_VSA_MOD_ORDER_USER_2QAM  23
#define RSSPECAN_VAL_VSA_MOD_ORDER_USER_4QAM  24
#define RSSPECAN_VAL_VSA_MOD_ORDER_USER_16QAM 25
#define RSSPECAN_VAL_VSA_MOD_ORDER_USER_32QAM 26
#define RSSPECAN_VAL_VSA_MOD_ORDER_2ASK       27
#define RSSPECAN_VAL_VSA_MOD_ORDER_4ASK       28
#define RSSPECAN_VAL_VSA_MOD_ORDER_16APSK     29
#define RSSPECAN_VAL_VSA_MOD_ORDER_32APSK     30
#define RSSPECAN_VAL_VSA_MOD_ORDER_SOQPSK     31
#define RSSPECAN_VAL_VSA_MOD_ORDER_NPI2       32
#define RSSPECAN_VAL_VSA_MOD_ORDER_DPI2       33
#define RSSPECAN_VAL_VSA_MOD_ORDER_MNPI2      34

#define RSSPECAN_VAL_VSA_REF_DEV_MODE_ABS 0
#define RSSPECAN_VAL_VSA_REF_DEV_MODE_REL 1

#define RSSPECAN_VAL_VSA_PATT_SYNC_OFF  0
#define RSSPECAN_VAL_VSA_PATT_SYNC_ON   1
#define RSSPECAN_VAL_VSA_PATT_SYNC_AUTO 2

#define RSSPECAN_VAL_VSA_AUTOMATIC_ESTIMATION  0
#define RSSPECAN_VAL_VSA_ESTIMATION_SYMBOL_1   1
#define RSSPECAN_VAL_VSA_ESTIMATION_SYMBOL_2   2
#define RSSPECAN_VAL_VSA_ESTIMATION_CAPTURE_OV 4

#define RSSPECAN_VAL_VSA_RESULT_DISP_MAG_ABS      0
#define RSSPECAN_VAL_VSA_RESULT_DISP_MAG_REL      1
#define RSSPECAN_VAL_VSA_RESULT_DISP_PHA_WRAP     2
#define RSSPECAN_VAL_VSA_RESULT_DISP_PHA_UNWRAP   3
#define RSSPECAN_VAL_VSA_RESULT_DISP_FREQ_ABS     4
#define RSSPECAN_VAL_VSA_RESULT_DISP_FREQ_REL     5
#define RSSPECAN_VAL_VSA_RESULT_DISP_RI           6
#define RSSPECAN_VAL_VSA_RESULT_DISP_EYE_R        7
#define RSSPECAN_VAL_VSA_RESULT_DISP_EYE_I        8
#define RSSPECAN_VAL_VSA_RESULT_DISP_EYE_FREQ     9
#define RSSPECAN_VAL_VSA_RESULT_DISP_IQ_CONST     10
#define RSSPECAN_VAL_VSA_RESULT_DISP_IQ_VECT      11
#define RSSPECAN_VAL_VSA_RESULT_DISP_CONST_FREQ   12
#define RSSPECAN_VAL_VSA_RESULT_DISP_VECT_FREQ    13
#define RSSPECAN_VAL_VSA_RESULT_DISP_EVM          14
#define RSSPECAN_VAL_VSA_RESULT_DISP_MAG_ERR      15
#define RSSPECAN_VAL_VSA_RESULT_DISP_PHA_ERR      16
#define RSSPECAN_VAL_VSA_RESULT_DISP_FREQ_ERR_ABS 17
#define RSSPECAN_VAL_VSA_RESULT_DISP_FREQ_ERR_REL 18
#define RSSPECAN_VAL_VSA_RESULT_DISP_MOD_ACC      19
#define RSSPECAN_VAL_VSA_RESULT_DISP_IQ_CON_ROT   20
#define RSSPECAN_VAL_VSA_RESULT_DISP_MOV          21

#define RSSPECAN_VAL_VSA_TRANSFORMATION_NORMAL 0
#define RSSPECAN_VAL_VSA_TRANSFORMATION_SPEC   1
#define RSSPECAN_VAL_VSA_TRANSFORMATION_STAT   2

#define RSSPECAN_VAL_VSA_RESULT_DISP_BIN 0
#define RSSPECAN_VAL_VSA_RESULT_DISP_DEC 1
#define RSSPECAN_VAL_VSA_RESULT_DISP_OCT 2
#define RSSPECAN_VAL_VSA_RESULT_DISP_HEX 3

#define RSSPECAN_VAL_VSA_LIMIT_CURRENT      0
#define RSSPECAN_VAL_VSA_LIMIT_MEAN         1
#define RSSPECAN_VAL_VSA_LIMIT_PEAK         2
#define RSSPECAN_VAL_VSA_LIMIT_CURRENT_RMS  3
#define RSSPECAN_VAL_VSA_LIMIT_MEAN_RMS     4
#define RSSPECAN_VAL_VSA_LIMIT_PEAK_RMS     5
#define RSSPECAN_VAL_VSA_LIMIT_CURRENT_PEAK 6
#define RSSPECAN_VAL_VSA_LIMIT_MEAN_PEAK    7
#define RSSPECAN_VAL_VSA_LIMIT_PEAK_PEAK    8

#define RSSPECAN_VAL_VSA_RESULT_EVM              0
#define RSSPECAN_VAL_VSA_RESULT_PHASE_ERROR      1
#define RSSPECAN_VAL_VSA_RESULT_MAGNITUDE_ERROR  2
#define RSSPECAN_VAL_VSA_RESULT_CF_ERROR         3
#define RSSPECAN_VAL_VSA_RESULT_RHO              4
#define RSSPECAN_VAL_VSA_RESULT_IQ_OFFSET        5
#define RSSPECAN_VAL_VSA_RESULT_FREQ_ERROR       6
#define RSSPECAN_VAL_VSA_RESULT_FSK_DEV_ERROR    7
#define RSSPECAN_VAL_VSA_RESULT_FSK_MEAS_DEV     8
#define RSSPECAN_VAL_VSA_RESULT_FSK_REF_DEV      9
#define RSSPECAN_VAL_VSA_RESULT_POWER            10
#define RSSPECAN_VAL_VSA_RESULT_CFD              11
#define RSSPECAN_VAL_VSA_RESULT_MOD_ERROR        12
#define RSSPECAN_VAL_VSA_RESULT_IQ_IMBALANCE     13
#define RSSPECAN_VAL_VSA_RESULT_GAIN_IMBALANCE   14
#define RSSPECAN_VAL_VSA_RESULT_QUADRATURE_ERROR 15
#define RSSPECAN_VAL_VSA_RESULT_AMPLITUDE_DROOP  16
#define RSSPECAN_VAL_VSA_RESULT_SRATE_ERROR      17
#define RSSPECAN_VAL_VSA_RESULT_IQ_SKEW          18

#define RSSPECAN_VAL_VSA_STAT_AVERAGE_SWEEP  0
#define RSSPECAN_VAL_VSA_STAT_AVERAGE        1
#define RSSPECAN_VAL_VSA_STAT_PEAK           2
#define RSSPECAN_VAL_VSA_STAT_SDEV           3
#define RSSPECAN_VAL_VSA_STAT_95_RMS         4
#define RSSPECAN_VAL_VSA_STAT_MAX_PEAK_SWEEP 5
#define RSSPECAN_VAL_VSA_STAT_MAX_AVG        6
#define RSSPECAN_VAL_VSA_STAT_MAX_PEAK       7
#define RSSPECAN_VAL_VSA_STAT_MAX_SDEV       8
#define RSSPECAN_VAL_VSA_STAT_MAX_95_RMS     9

#define RSSPECAN_VAL_VSA_LIMIT_MODULATION_ACCURACY 0

#define RSSPECAN_VAL_TRACE_1      1
#define RSSPECAN_VAL_TRACE_2      2
#define RSSPECAN_VAL_TRACE_3      3
#define RSSPECAN_VAL_TRACE_4      4
#define RSSPECAN_VAL_TRACE_5      5
#define RSSPECAN_VAL_TRACE_6      6
#define RSSPECAN_VAL_REAL_TRACE_1 7
#define RSSPECAN_VAL_REAL_TRACE_2 8
#define RSSPECAN_VAL_REAL_TRACE_3 9
#define RSSPECAN_VAL_IMAG_TRACE_1 10
#define RSSPECAN_VAL_IMAG_TRACE_2 11
#define RSSPECAN_VAL_IMAG_TRACE_3 12
#define RSSPECAN_VAL_IQC_TRACE_Y  13
#define RSSPECAN_VAL_IQC_TRACE_X  14

#define RSSPECAN_VAL_VSA_BER_RESULT_CURRENT       0
#define RSSPECAN_VAL_VSA_BER_RESULT_MINIMUM       1
#define RSSPECAN_VAL_VSA_BER_RESULT_MAXIMUM       2
#define RSSPECAN_VAL_VSA_BER_RESULT_ACCUMULATIVE  3

#define RSSPECAN_VAL_VSA_COMPENSATE_IQ_OFFSET   0
#define RSSPECAN_VAL_VSA_COMPENSATE_IQ_IMBAL    1
#define RSSPECAN_VAL_VSA_COMPENSATE_ADROOP      2
#define RSSPECAN_VAL_VSA_COMPENSATE_FSK_DEV_ERR 3
#define RSSPECAN_VAL_VSA_COMPENSATE_FSK_CFD     4
#define RSSPECAN_VAL_VSA_COMPENSATE_CHANNEL     5
#define RSSPECAN_VAL_VSA_COMPENSATE_SR_ERROR    6
#define RSSPECAN_VAL_VSA_COMPENSATE_IQ_SKEW     7

/* End of K70 constants */

/* K8x constants*/
#define RSSPECAN_VAL_C2K_CTABLE         0

#define RSSPECAN_VAL_C2K_SLOT           0
#define RSSPECAN_VAL_C2K_PTOTAL         1
#define RSSPECAN_VAL_C2K_PPICH          2
#define RSSPECAN_VAL_C2K_RHO            3
#define RSSPECAN_VAL_C2K_MACCURACY      4
#define RSSPECAN_VAL_C2K_PCDERROR       5
#define RSSPECAN_VAL_C2K_IQIMBALANCE    6
#define RSSPECAN_VAL_C2K_IQOFFSET       7
#define RSSPECAN_VAL_C2K_FERROR         8
#define RSSPECAN_VAL_C2K_FERPPM         9
#define RSSPECAN_VAL_C2K_CERROR         10
#define RSSPECAN_VAL_C2K_TFRAME         11
#define RSSPECAN_VAL_C2K_ACTIVE         12
#define RSSPECAN_VAL_C2K_SRATE          13
#define RSSPECAN_VAL_C2K_CHANNEL        14
#define RSSPECAN_VAL_C2K_SFACTOR        15
#define RSSPECAN_VAL_C2K_TOFFSET        16
#define RSSPECAN_VAL_C2K_POFFSET        17
#define RSSPECAN_VAL_C2K_CDPRELATIVE    18
#define RSSPECAN_VAL_C2K_CDPABSOLUTE    19
#define RSSPECAN_VAL_C2K_EVMRMS         20
#define RSSPECAN_VAL_C2K_EVMPEAK        21
#define RSSPECAN_VAL_C2K_RHOPILOT       22
#define RSSPECAN_VAL_C2K_RHO1           23
#define RSSPECAN_VAL_C2K_RHO2           24
#define RSSPECAN_VAL_C2K_PPILOT         25
#define RSSPECAN_VAL_C2K_PMAC           26
#define RSSPECAN_VAL_C2K_PDATA          27
#define RSSPECAN_VAL_C2K_PPREAMBLE      28
#define RSSPECAN_VAL_C2K_DMTYPE         29
#define RSSPECAN_VAL_C2K_MACTIVE        30
#define RSSPECAN_VAL_C2K_DACTIVE        31
#define RSSPECAN_VAL_C2K_PLENGTH        32
#define RSSPECAN_VAL_C2K_PDMAX          33
#define RSSPECAN_VAL_C2K_PDMIN          34
#define RSSPECAN_VAL_C2K_IPMMAX         35
#define RSSPECAN_VAL_C2K_MTYPE          36
#define RSSPECAN_VAL_C2K_PRRI           37
#define RSSPECAN_VAL_C2K_DRPICH         38
#define RSSPECAN_VAL_C2K_RHOVERALL      39
#define RSSPECAN_VAL_C2K_RHODATA        40
#define RSSPECAN_VAL_C2K_RHOMAC         41

#define RSSPECAN_VAL_C2K_PNF            42
#define RSSPECAN_VAL_C2K_PPAV           43
#define RSSPECAN_VAL_C2K_TAU            44
#define RSSPECAN_VAL_C2K_TACT           45
#define RSSPECAN_VAL_C2K_EVMP           46

#define RSSPECAN_VAL_C2K_CDEPEAK        47
#define RSSPECAN_VAL_C2K_CDERMS         48
#define RSSPECAN_VAL_C2K_CODMULATION    49
#define RSSPECAN_VAL_C2K_CODPOWER       50

/* End of K8x constants*/

#define RSSPECAN_VAL_LTE_DOWNLINK_QPSK      0
#define RSSPECAN_VAL_LTE_DOWNLINK_16QAM     1
#define RSSPECAN_VAL_LTE_DOWNLINK_64QAM     2
#define RSSPECAN_VAL_LTE_DOWNLINK_256QAM    3
#define RSSPECAN_VAL_LTE_DOWNLINK_1024QAM   4

#define RSSPECAN_VAL_LTE_UPLINK_QPSK   0
#define RSSPECAN_VAL_LTE_UPLINK_16QAM  1
#define RSSPECAN_VAL_LTE_UPLINK_64QAM  2

#define RSSPECAN_VAL_MEASTYPE_VALUES  0
#define RSSPECAN_VAL_MEASTYPE_SUMMARY 1

#define RSSPECAN_VAL_MEASTYPE_PROBABILITY 0
#define RSSPECAN_VAL_MEASTYPE_POWER_STEPS 1

#define RSSPECAN_VAL_LTE_DOWNLINK_CHBW  0
#define RSSPECAN_VAL_LTE_DOWNLINK_NRB   1

#define RSSPECAN_VAL_AUTOSCALE_ALL             0
#define RSSPECAN_VAL_AUTOSCALE_A               1
#define RSSPECAN_VAL_AUTOSCALE_B               2
#define RSSPECAN_VAL_AUTOSCALE_C               3
#define RSSPECAN_VAL_AUTOSCALE_D               4

#define RSSPECAN_VAL_TETRA_LIM_EVM_ALL         0
#define RSSPECAN_VAL_TETRA_LIM_EVM_DATA        1
#define RSSPECAN_VAL_TETRA_LIM_EVM_DHEA        2
#define RSSPECAN_VAL_TETRA_LIM_EVM_DSP         3
#define RSSPECAN_VAL_TETRA_LIM_EVM_SPIL        4
#define RSSPECAN_VAL_TETRA_LIM_IQOF            5
#define RSSPECAN_VAL_TETRA_LIM_FERR            6
#define RSSPECAN_VAL_TETRA_LIM_PHAS            7
#define RSSPECAN_VAL_TETRA_LIM_MAGN            8

#define RSSPECAN_VAL_TETRA_ACP_CHANNEL_NUMBER_NEG_3 0
#define RSSPECAN_VAL_TETRA_ACP_CHANNEL_NUMBER_NEG_2 1
#define RSSPECAN_VAL_TETRA_ACP_CHANNEL_NUMBER_NEG_1 2
#define RSSPECAN_VAL_TETRA_ACP_CHANNEL_NUMBER_TX    3
#define RSSPECAN_VAL_TETRA_ACP_CHANNEL_NUMBER_POS_1 4
#define RSSPECAN_VAL_TETRA_ACP_CHANNEL_NUMBER_POS_2 5
#define RSSPECAN_VAL_TETRA_ACP_CHANNEL_NUMBER_POS_3 6

#define RSSPECAN_VAL_TETRA_ACP_MODULATION       0
#define RSSPECAN_VAL_TETRA_ACP_TRANSIENT        1

#define RSSPECAN_VAL_TETRA_ACP_BW               0
#define RSSPECAN_VAL_TETRA_ACP_RCC              1
#define RSSPECAN_VAL_TETRA_ACP_OFFS             2
#define RSSPECAN_VAL_TETRA_ACP_ABS_POW          3
#define RSSPECAN_VAL_TETRA_ACP_REL_POW          4
#define RSSPECAN_VAL_TETRA_ACP_DLIMIT_POW       5
#define RSSPECAN_VAL_TETRA_ACP_RBW              6

#define RSSPECAN_VAL_TETRA_MEAS_NONE            0
#define RSSPECAN_VAL_TETRA_MEAS_MIN             1
#define RSSPECAN_VAL_TETRA_MEAS_MAX             2
#define RSSPECAN_VAL_TETRA_MEAS_AVER            3
#define RSSPECAN_VAL_TETRA_MEAS_CURR            4

#define RSSPECAN_VAL_CARRIER_ALL                0
#define RSSPECAN_VAL_CARRIER_NUM                1

#define RSSPECAN_VAL_TETRA_UNIT_SECONDS     0
#define RSSPECAN_VAL_TETRA_UNIT_SLOTS       1

#define RSSPECAN_VAL_TETRA_PVT      0
#define RSSPECAN_VAL_TETRA_EVMS     1
#define RSSPECAN_VAL_TETRA_EVMC     2
#define RSSPECAN_VAL_TETRA_MAGN     3
#define RSSPECAN_VAL_TETRA_PHAS     4
#define RSSPECAN_VAL_TETRA_FFT      5
#define RSSPECAN_VAL_TETRA_ACPM     6
#define RSSPECAN_VAL_TETRA_ACPT_UPP 7
#define RSSPECAN_VAL_TETRA_ACPT_LOW 8
#define RSSPECAN_VAL_TETRA_CSYM     9
#define RSSPECAN_VAL_TETRA_CCAR     10
#define RSSPECAN_VAL_TETRA_BSTR     11
#define RSSPECAN_VAL_TETRA_SUMM     12

#define RSSPECAN_VAL_TETRA_TRACE_CURR 1
#define RSSPECAN_VAL_TETRA_TRACE_MIN  2
#define RSSPECAN_VAL_TETRA_TRACE_PEAK 3
#define RSSPECAN_VAL_TETRA_TRACE_AVER 4

#define RSSPECAN_VAL_TETRA_MEAS_EVM_ALL  0
#define RSSPECAN_VAL_TETRA_MEAS_EVM_DSP  1
#define RSSPECAN_VAL_TETRA_MEAS_EVM_DHEA 2
#define RSSPECAN_VAL_TETRA_MEAS_EVM_DATA 3
#define RSSPECAN_VAL_TETRA_MEAS_EVM_SPIL 4
#define RSSPECAN_VAL_TETRA_MEAS_EVM_IQOF 5
#define RSSPECAN_VAL_TETRA_MEAS_EVM_GIMB 6
#define RSSPECAN_VAL_TETRA_MEAS_EVM_QUAD 7
#define RSSPECAN_VAL_TETRA_MEAS_EVM_AMPD 8
#define RSSPECAN_VAL_TETRA_MEAS_EVM_FERR 9
#define RSSPECAN_VAL_TETRA_MEAS_EVM_PHAS 10
#define RSSPECAN_VAL_TETRA_MEAS_EVM_MAG  11
#define RSSPECAN_VAL_TETRA_MEAS_EVM_BPWR 12
#define RSSPECAN_VAL_TETRA_MEAS_EVM_CRES 13

#define RSSPECAN_VAL_TETRA_PVT_REF         0
#define RSSPECAN_VAL_TETRA_PVT_SLOT        1
#define RSSPECAN_VAL_TETRA_PVT_BEFB        2
#define RSSPECAN_VAL_TETRA_PVT_REL_BEFB    3
#define RSSPECAN_VAL_TETRA_PVT_AFTB        4
#define RSSPECAN_VAL_TETRA_PVT_REL_AFTB    5
#define RSSPECAN_VAL_TETRA_PVT_TRG_TO_SYNC 6

/*
#define RSSPECAN_VAL_SPEM_GREY RSSPECAN_VAL_SPEM_GRAY
*/

#define RSSPECAN_VAL_MEASTYPE_RESOURCE_BLOCK_INDEXES 0
#define RSSPECAN_VAL_MEASTYPE_INBAND_EMISSION        1
#define RSSPECAN_VAL_MEASTYPE_UPPER_LIMIT_LINE       2

#define RSSPECAN_VAL_VSA_EVAL_STATISTICS 0
#define RSSPECAN_VAL_VSA_EVAL_CAPTURE    1

/* Function:Query LTE Downlink Measurement EVM Channel & Query LTE Uplink Measurement EVM Channel*/
#define RSSPECAN_VAL_CHANNEL_PSYN    0
#define RSSPECAN_VAL_CHANNEL_SSYN    1
#define RSSPECAN_VAL_CHANNEL_PBCH    2
#define RSSPECAN_VAL_CHANNEL_PCFI    3

#define RSSPECAN_VAL_LAYOUT_DIRECTION_LEFT   0
#define RSSPECAN_VAL_LAYOUT_DIRECTION_RIGHT  1
#define RSSPECAN_VAL_LAYOUT_DIRECTION_ABOWE  2
#define RSSPECAN_VAL_LAYOUT_DIRECTION_BELOW  3

#define RSSPECAN_VAL_LTE_ANT1  1
#define RSSPECAN_VAL_LTE_ANT2  2
#define RSSPECAN_VAL_LTE_ANT3  3
#define RSSPECAN_VAL_LTE_ANT4  4

#define RSSPECAN_VAL_RESULT_EVM 0
#define RSSPECAN_VAL_RESULT_POW 1

/* Function: rsspecan_Set_3GPPTAERBTSOperation */
#define RSSPECAN_VAL_3GPP_BTS_NUMBER_INSERT 0
#define RSSPECAN_VAL_3GPP_BTS_NUMBER_DELETE 1
#define RSSPECAN_VAL_3GPP_BTS_NUMBER_DELETE_ALL 2

/* Function: rsspecan_Set_3GPPTAERTableOperation */
#define RSSPECAN_VAL_3GPP_TAER_TABLE_DELETE 0
#define RSSPECAN_VAL_3GPP_TAER_TABLE_SAVE   1

/* Function: rsspecan_QueryLTEUplinkMeasurementResultSummary */
#define RSSPECAN_VAL_LTE_UPLINK_PUSCH_QPSK_AVER       0
#define RSSPECAN_VAL_LTE_UPLINK_PUSCH_16QAM_AVER      1
#define RSSPECAN_VAL_LTE_UPLINK_PUSCH_64QAM_AVER      2
#define RSSPECAN_VAL_LTE_UPLINK_DMRS_PUSCH_QPSK_AVER  3
#define RSSPECAN_VAL_LTE_UPLINK_DMRS_PUSCH_16QAM_AVER 4
#define RSSPECAN_VAL_LTE_UPLINK_DMRS_PUSCH_AVER       5
#define RSSPECAN_VAL_LTE_UPLINK_PUCCH_AVER            6
#define RSSPECAN_VAL_LTE_UPLINK_PRACH_AVER            7
#define RSSPECAN_VAL_LTE_UPLINK_PUSCH_QPSK_MAX        8
#define RSSPECAN_VAL_LTE_UPLINK_PUSCH_16QAM_MAX       9
#define RSSPECAN_VAL_LTE_UPLINK_PUSCH_64QAM_MAX      10
#define RSSPECAN_VAL_LTE_UPLINK_DMRS_PUSCH_QPSK_MAX  11
#define RSSPECAN_VAL_LTE_UPLINK_DMRS_PUSCH_16QAM_MAX 12
#define RSSPECAN_VAL_LTE_UPLINK_DMRS_PUSCH_MAX       13
#define RSSPECAN_VAL_LTE_UPLINK_PUCCH_MAX            14
#define RSSPECAN_VAL_LTE_UPLINK_PRACH_MAX            15
#define RSSPECAN_VAL_LTE_UPLINK_PUSCH_QPSK_MIN       16
#define RSSPECAN_VAL_LTE_UPLINK_PUSCH_16QAM_MIN      17
#define RSSPECAN_VAL_LTE_UPLINK_PUSCH_64QAM_MIN      18
#define RSSPECAN_VAL_LTE_UPLINK_DMRS_PUSCH_QPSK_MIN  19
#define RSSPECAN_VAL_LTE_UPLINK_DMRS_PUSCH_16QAM_MIN 20
#define RSSPECAN_VAL_LTE_UPLINK_DMRS_PUSCH_MIN       21
#define RSSPECAN_VAL_LTE_UPLINK_PUCCH_MIN            22
#define RSSPECAN_VAL_LTE_UPLINK_PRACH_MIN            23

/* Function: rsspecan_ConfigurePhaseDisplayYAxisSettings */
#define RSSPECAN_VAL_PHASE_Y_SCALE_AUTO 4

/* Function: rsspecan_ConfigurePhaseGlobalIQWindowSettings */
#define RSSPECAN_VAL_PHASE_ALL_DECADE         0
#define RSSPECAN_VAL_PHASE_PARTICULAR_DECADE  1

/* Another*/
#define RSSPECAN_VAL_NOISE_TABLE_SELECT 0
#define RSSPECAN_VAL_NOISE_TABLE_DELETE 1

#define RSSPECAN_VAL_NOISE_VSWR        0
#define RSSPECAN_VAL_NOISE_RETURN_LOSS 1

#define RSSPECAN_VAL_NOISE_INPUT  0
#define RSSPECAN_VAL_NOISE_OUTPUT 1

#define RSSPECAN_VAL_WLAN_ITEM_TX_ALL                  0
#define RSSPECAN_VAL_WLAN_ITEM_TX_IQ_OFFSET            1
#define RSSPECAN_VAL_WLAN_ITEM_TX_QUAD_OFFSET          2
#define RSSPECAN_VAL_WLAN_ITEM_TX_CREST_FACTOR         3
#define RSSPECAN_VAL_WLAN_ITEM_TX_GAIN_IMBALANCE       4
#define RSSPECAN_VAL_WLAN_ITEM_TX_PPDU_POWER           5
#define RSSPECAN_VAL_WLAN_ITEM_RX_ALL                  6
#define RSSPECAN_VAL_WLAN_ITEM_RX_PPDU_POWER           7
#define RSSPECAN_VAL_WLAN_ITEM_RX_CREST_FACTOR         8
#define RSSPECAN_VAL_WLAN_ITEM_STREAM_ALL              9
#define RSSPECAN_VAL_WLAN_ITEM_STREAM_BER_PILOT        10
#define RSSPECAN_VAL_WLAN_ITEM_STREAM_EVM_DATA         11
#define RSSPECAN_VAL_WLAN_ITEM_STREAM_EVM_ALL          12
#define RSSPECAN_VAL_WLAN_ITEM_STREAM_EVM_PILOT        13
#define RSSPECAN_VAL_WLAN_ITEM_PILOT_BIT_ERROR         14
#define RSSPECAN_VAL_WLAN_ITEM_EVM_ALL                 15
#define RSSPECAN_VAL_WLAN_ITEM_EVM_DATA                16
#define RSSPECAN_VAL_WLAN_ITEM_EVM_PILOT               17
#define RSSPECAN_VAL_WLAN_ITEM_CENTER_FREQ_ERROR       18
#define RSSPECAN_VAL_WLAN_ITEM_SYMBOL_CLOCK_ERROR      19
#define RSSPECAN_VAL_WLAN_ITEM_PILOT_VECTOR_ERROR_DSSS 20
#define RSSPECAN_VAL_WLAN_ITEM_PPDU_EVM_DSSS           21
#define RSSPECAN_VAL_WLAN_ITEM_IQ_OFFSET_DSSS          22
#define RSSPECAN_VAL_WLAN_ITEM_GAIN_IMBALANCE_DSSS     23
#define RSSPECAN_VAL_WLAN_ITEM_QUADRATURE_ERROR_DSSS   24
#define RSSPECAN_VAL_WLAN_ITEM_CENTER_FREQ_ERROR_DSSS  25
#define RSSPECAN_VAL_WLAN_ITEM_CHIP_CLOCK_ERROR_DSSS   26
#define RSSPECAN_VAL_WLAN_ITEM_RISE_TIME_DSSS          27
#define RSSPECAN_VAL_WLAN_ITEM_FALL_TIME_DSSS          28
#define RSSPECAN_VAL_WLAN_ITEM_MEAN_POWER_DSSS         29
#define RSSPECAN_VAL_WLAN_ITEM_PEAK_POWER_DSSS         30
#define RSSPECAN_VAL_WLAN_ITEM_CREST_FACTOR_DSSS       31

#define RSSPECAN_VAL_INT_GAIN  0
#define RSSPECAN_VAL_INT_NOISE 1

#define RSSPECAN_VAL_SYNC_TO_PCCPCH   0
#define RSSPECAN_VAL_SYNC_TO_MIDAMBLE 1

#define RSSPECAN_VAL_WLAN_CURRENT 0
#define RSSPECAN_VAL_WLAN_ALL     1

#define RSSPECAN_VAL_WLAN_MEAS_ACPR 0
#define RSSPECAN_VAL_WLAN_MEAS_MASK 1
#define RSSPECAN_VAL_WLAN_MEAS_OBAN 2
#define RSSPECAN_VAL_WLAN_MEAS_CCDF 3

#define RSSPECAN_VAL_WLAN_IQ_MEAS_GCAR  0
#define RSSPECAN_VAL_WLAN_IQ_MEAS_PTR   1
#define RSSPECAN_VAL_WLAN_IQ_MEAS_QCAR  2

//#define RSSPECAN_VAL_MEAS_POW_PPOW 6

#define RSSPECAN_VAL_WLAN_PSDU_FBUR  0
#define RSSPECAN_VAL_WLAN_PSDU_ALL   1
#define RSSPECAN_VAL_WLAN_PSDU_MEAS  2
#define RSSPECAN_VAL_WLAN_PSDU_DEMOD 3

#define RSSPECAN_VAL_LTE_UPLINK_PRACH_SEQ_INDEX_MANUAL 0

#define RSSPECAN_VAL_PULSE_RESULT_SCALE_OFF  0
#define RSSPECAN_VAL_PULSE_RESULT_SCALE_ON   1
#define RSSPECAN_VAL_PULSE_RESULT_SCALE_ONCE 2

#define RSSPECAN_VAL_WLAN_PPDU_LONG  0
#define RSSPECAN_VAL_WLAN_PPDU_SHORT 1

#define RSSPECAN_VAL_PULSE_RESULT_Y_AXIS_COUNT     0
#define RSSPECAN_VAL_PULSE_RESULT_Y_AXIS_OCCURANCE 1

#define RSSPECAN_VAL_GSM_K10_SLOT_SCOPE   0
#define RSSPECAN_VAL_GSM_K10_SLOT_MEASURE 1

#define RSSPECAN_VAL_GROUP_X 0
#define RSSPECAN_VAL_GROUP_Y 1

#define RSSPECAN_VAL_NOISE_LIM_LINE_TYPE_GAIN  0
#define RSSPECAN_VAL_NOISE_LIM_LINE_TYPE_NOISE 1
#define RSSPECAN_VAL_NOISE_LIM_LINE_TYPE_PCOLD 2
#define RSSPECAN_VAL_NOISE_LIM_LINE_TYPE_PHOT  3
#define RSSPECAN_VAL_NOISE_LIM_LINE_TYPE_TEMP  4
#define RSSPECAN_VAL_NOISE_LIM_LINE_TYPE_YFAC  5
#define RSSPECAN_VAL_NOISE_LIM_LINE_TYPE_NUNC  6
#define RSSPECAN_VAL_NOISE_LIM_LINE_TYPE_CYF   7
#define RSSPECAN_VAL_NOISE_LIM_LINE_TYPE_CPC   8
#define RSSPECAN_VAL_NOISE_LIM_LINE_TYPE_CPH   9

#define RSSPECAN_VAL_NOISE_FETC_FIGURE      0
#define RSSPECAN_VAL_NOISE_FETC_TEMPERATURE 1
#define RSSPECAN_VAL_NOISE_FETC_GAIN        2

#define RSSPECAN_VAL_GSM_K10_RBW_VBW_30KHZ  0
#define RSSPECAN_VAL_GSM_K10_RBW_VBW_100KHZ 1

#define RSSPECAN_VAL_LTE_TAER 0
#define RSSPECAN_VAL_LTE_FRER 1

#define RSSPECAN_VAL_LTE_FRER_PPM 1
#define RSSPECAN_VAL_LTE_FRER_HZ  2

#define RSSPECAN_VAL_LIMIT_CHECK_TYPE_ALL 0
#define RSSPECAN_VAL_LIMIT_CHECK_TYPE_REL 1
#define RSSPECAN_VAL_LIMIT_CHECK_TYPE_ABS 2

#define RSSPECAN_VAL_LTE_UPLINK_PUSCH_QPSK       0
#define RSSPECAN_VAL_LTE_UPLINK_PUSCH_16QAM      1
#define RSSPECAN_VAL_LTE_UPLINK_PUSCH_64QAM      2
#define RSSPECAN_VAL_LTE_UPLINK_DMRS_PUSCH_QPSK  3
#define RSSPECAN_VAL_LTE_UPLINK_DMRS_PUSCH_16QAM 4
#define RSSPECAN_VAL_LTE_UPLINK_DMRS_PUSCH_64QAM 5
#define RSSPECAN_VAL_LTE_UPLINK_DMRS_PUCCH       6
#define RSSPECAN_VAL_LTE_UPLINK_PUCCH            7
#define RSSPECAN_VAL_LTE_UPLINK_PRACH            8
#define RSSPECAN_VAL_LTE_UPLINK_PUSCH_256QAM      9
#define RSSPECAN_VAL_LTE_UPLINK_DMRS_PUSCH_256QAM 10

#define RSSPECAN_VAL_TRANSIENT_TABLE_SAVE   0
#define RSSPECAN_VAL_TRANSIENT_TABLE_LOAD   1

#define RSSPECAN_VAL_TRANSIENT_RESULT_POWER_ALL     0
#define RSSPECAN_VAL_TRANSIENT_RESULT_POWER_AVER    1
#define RSSPECAN_VAL_TRANSIENT_RESULT_POWER_MAX     2
#define RSSPECAN_VAL_TRANSIENT_RESULT_POWER_MIN     3
#define RSSPECAN_VAL_TRANSIENT_RESULT_POWER_RIPPLE  4

#define RSSPECAN_VAL_TRANSIENT_RESULT_STATE_ALL   0
#define RSSPECAN_VAL_TRANSIENT_RESULT_STATE_INDEX 1
#define RSSPECAN_VAL_TRANSIENT_RESULT_STATE_FREQ  2

#define RSSPECAN_VAL_TRANSIENT_AXIS_X 0
#define RSSPECAN_VAL_TRANSIENT_AXIS_Y 1

#define RSSPECAN_VAL_AMPLIFIER_IP_ADDRESS          0
#define RSSPECAN_VAL_AMPLIFIER_RMS_LEVEL           1
#define RSSPECAN_VAL_AMPLIFIER_CENTER_FREQUENCY    2
#define RSSPECAN_VAL_AMPLIFIER_REFERENCE_FREQUENCY 3
#define RSSPECAN_VAL_AMPLIFIER_LEVEL_OFFSET        4
#define RSSPECAN_VAL_AMPLIFIER_SEGMENT             5
#define RSSPECAN_VAL_AMPLIFIER_DIGITAL_ATT         6
#define RSSPECAN_VAL_AMPLIFIER_RF_OUT              7

#define RSSPECAN_VAL_DOCSIS_ALL             0
#define RSSPECAN_VAL_DOCSIS_NUMERIC         1

/* K18 constants*/
#define RSSPECAN_VAL_RESULT_TRIGGER_TO_FRAME    0

#define RSSPECAN_VAL_RESULT_ALL_MACC  0
#define RSSPECAN_VAL_RESULT_ALL_POWER 1
#define RSSPECAN_VAL_RESULT_ALL_VCURR 2

#define RSSPECAN_VAL_AXIS_NONE 2

#define RSSPECAN_VAL_RESULT_PSWEEP_1DB 0
#define RSSPECAN_VAL_RESULT_PSWEEP_2DB 1
#define RSSPECAN_VAL_RESULT_PSWEEP_3DB 2

/*End of K18 constants*/

#define RSSPECAN_VAL_DOCSIS_MEAS_TYPE_CENTER_ERROR                0
#define RSSPECAN_VAL_DOCSIS_ANALYSIS_RESULT_FETCH_FREQUENCY_ERROR 1
#define RSSPECAN_VAL_DOCSIS_ANALYSIS_RESULT_FETCH_MER             2
#define RSSPECAN_VAL_DOCSIS_ANALYSIS_RESULT_FETCH_MER_DATA        3
#define RSSPECAN_VAL_DOCSIS_ANALYSIS_RESULT_FETCH_MER_PILOT       4
#define RSSPECAN_VAL_DOCSIS_ANALYSIS_RESULT_PHASE_ERROR           5
#define RSSPECAN_VAL_DOCSIS_ANALYSIS_RESULT_FETCH_POWER           6
#define RSSPECAN_VAL_DOCSIS_ANALYSIS_RESULT_FETCH_TPLC            7
#define RSSPECAN_VAL_DOCSIS_ANALYSIS_RESULT_FETCH_ZBIT            8

#define RSSPECAN_VAL_DOCSIS_MODULATION_ERROR_RATIO_DATA_PILOT 0
#define RSSPECAN_VAL_DOCSIS_MODULATION_ERROR_RATIO_DATA       1
#define RSSPECAN_VAL_DOCSIS_MODULATION_ERROR_RATIO_PILOT      2

#define RSSPECAN_VAL_SUB_BLOCK_A 1
#define RSSPECAN_VAL_SUB_BLOCK_B 2
#define RSSPECAN_VAL_SUB_BLOCK_C 3

#define RSSPECAN_VAL_TAER_CARRIER_TABLE_NEW    0
#define RSSPECAN_VAL_TAER_CARRIER_TABLE_SAVE   1
#define RSSPECAN_VAL_TAER_CARRIER_TABLE_DELETE 2

#define RSSPECAN_VAL_TAER_CARRIER_INSERT     0
#define RSSPECAN_VAL_TAER_CARRIER_DELETE     1
#define RSSPECAN_VAL_TAER_CARRIER_DELETE_ALL 2

#define RSSPECAN_VAL_DOCSIS_PER_DIVISION_1_0 0
#define RSSPECAN_VAL_DOCSIS_PER_DIVISION_2_0 1
#define RSSPECAN_VAL_DOCSIS_PER_DIVISION_2_5 2
#define RSSPECAN_VAL_DOCSIS_PER_DIVISION_5_0 3

#define RSSPECAN_VAL_SCALING_PER_DIVISION_1_0 0
#define RSSPECAN_VAL_SCALING_PER_DIVISION_2_0 1
#define RSSPECAN_VAL_SCALING_PER_DIVISION_2_5 2
#define RSSPECAN_VAL_SCALING_PER_DIVISION_5_0 3

#define RSSPECAN_VAL_VERTICAL   0
#define RSSPECAN_VAL_HORIZONTAL 1

/* K15 constants*/

#define RSSPECAN_VAL_AVIONICS_HARMONIC_K2 0
#define RSSPECAN_VAL_AVIONICS_HARMONIC_K3 1

/* Fast Mode */
#define RSSPECAN_VAL_FAST_SWEEP_NORMAL 0
#define RSSPECAN_VAL_FAST_SWEEP_FAST   1

#define RSSPECAN_VAL_AMPLIFIER_DIRECT_DPD_START  0
#define RSSPECAN_VAL_AMPLIFIER_DIRECT_DPD_ABORT  1
#define RSSPECAN_VAL_AMPLIFIER_DIRECT_DPD_FINISH 2

#define RSSPECAN_VAL_VALUE 0

#define RSSPECAN_VAL_UCORR_TRACE_FREQUENCIES 0
#define RSSPECAN_VAL_UCORR_TRACE_MAGNITUDES  1
#define RSSPECAN_VAL_UCORR_TRACE_PHASES      2

#define RSSPECAN_VAL_PARM_COUPLING_CENTER_FREQUENCY     0
#define RSSPECAN_VAL_PARM_COUPLING_START_STOP_FREQUENCY 1
#define RSSPECAN_VAL_PARM_COUPLING_ATTENUATION          2
#define RSSPECAN_VAL_PARM_COUPLING_PREAMPLIFIER         3
#define RSSPECAN_VAL_PARM_COUPLING_VBW                  4
#define RSSPECAN_VAL_PARM_COUPLING_RBW                  5
#define RSSPECAN_VAL_PARM_COUPLING_LIMIT_LINES          6
#define RSSPECAN_VAL_PARM_COUPLING_AC_DC                7
#define RSSPECAN_VAL_PARM_COUPLING_MARKER_FREQUENCY     8
#define RSSPECAN_VAL_PARM_COUPLING_PRESELECTOR          9
#define RSSPECAN_VAL_PARM_COUPLING_AUDIO_DEMODULATOR   10
#define RSSPECAN_VAL_PARM_COUPLING_BASEBAND_IMPEDANCE  11
#define RSSPECAN_VAL_PARM_COUPLING_AMPLITUDE_UNIT      12
#define RSSPECAN_VAL_PARM_COUPLING_IMPEDANCE           13
#define RSSPECAN_VAL_PARM_COUPLING_LIMITED_LINES       14
#define RSSPECAN_VAL_PARM_COUPLING_REFERENCE_LEVEL     15

#define RSSPECAN_VAL_DOCSIS_SELECTED  1

#define RSSPECAN_VAL_DOCSIS_SUBCARRIER 0
#define RSSPECAN_VAL_DOCSIS_SET        1

#define RSSPECAN_VAL_DOCSIS_MEAS_TYPE_CENTER_ERROR                 0
#define RSSPECAN_VAL_DOCSIS_ANALYSIS_RESULT_FETCH_FREQUENCY_ERROR  1
#define RSSPECAN_VAL_DOCSIS_ANALYSIS_RESULT_FETCH_MER              2
#define RSSPECAN_VAL_DOCSIS_ANALYSIS_RESULT_FETCH_MER_DATA         3
#define RSSPECAN_VAL_DOCSIS_ANALYSIS_RESULT_FETCH_MER_PILOT        4
#define RSSPECAN_VAL_DOCSIS_ANALYSIS_RESULT_PHASE_ERROR            5
#define RSSPECAN_VAL_DOCSIS_ANALYSIS_RESULT_FETCH_POWER            6
#define RSSPECAN_VAL_DOCSIS_ANALYSIS_RESULT_FETCH_TPLC             7
#define RSSPECAN_VAL_DOCSIS_ANALYSIS_RESULT_FETCH_ZBIT             8
#define RSSPECAN_VAL_DOCSIS_ANALYSIS_RESULT_FETCH_AMINISLOTS       9
#define RSSPECAN_VAL_DOCSIS_ANALYSIS_RESULT_FETCH_TRIGGER_TO_FRAME 10

#define RSSPECAN_VAL_TRANSIENT_FM 0
#define RSSPECAN_VAL_TRANSIENT_PM 1

#define RSSPECAN_VAL_VSA_FRAME_TYPE_NEXT 0
#define RSSPECAN_VAL_VSA_FRAME_TYPE_PREV 1

#define RSSPECAN_VAL_5G_NR_POWER_TYPE_PSS 0
#define RSSPECAN_VAL_5G_NR_POWER_TYPE_SSS 1

#define RSSPECAN_VAL_5G_NR_ANALYSIS_MODE_ALL 0
#define RSSPECAN_VAL_5G_NR_ANALYSIS_MODE_VAL 1
#define RSSPECAN_VAL_5G_NR_ANALYSIS_MODE_SS_PBCH_BLOCK 3

#define RSSPECAN_VAL_5GNR_MODULATION_QPSK    0
#define RSSPECAN_VAL_5GNR_MODULATION_16QAM   1
#define RSSPECAN_VAL_5GNR_MODULATION_64QAM   2
#define RSSPECAN_VAL_5GNR_MODULATION_256QAM  3
#define RSSPECAN_VAL_5GNR_MODULATION_1024QAM 4

#define RSSPECAN_VAL_NBIOT_SEARCH_MARKER_MIN 0
#define RSSPECAN_VAL_NBIOT_SEARCH_MARKER_MAX 1

#define RSSPECAN_VAL_NBIOT_SELECT_ALL   0
#define RSSPECAN_VAL_NBIOT_SELECT_VALUE 1

#define RSSPECAN_VAL_NOISE_RESULTS_TYPE_ARRAY  0
#define RSSPECAN_VAL_NOISE_RESULTS_TYPE_SCALAR 1

#define RSSPECAN_VAL_FILE_ITEMS_SOUR_CAL_DATA   6
#define RSSPECAN_VAL_FILE_ITEMS_SPECTROGRAM     8
#define RSSPECAN_VAL_FILE_ITEMS_CLIST           9
#define RSSPECAN_VAL_FILE_ITEMS_FINAL           10

#define RSSPECAN_VAL_INPUT_TG  2

#define RSSPECAN_VAL_5G_NR_CSI_RS_L0 0
#define RSSPECAN_VAL_5G_NR_CSI_RS_L1 1

#define RSSPECAN_VAL_5G_NR_ALL   0
#define RSSPECAN_VAL_5G_NR_VALUE 1

#define RSSPECAN_VAL_OW_UL_LIMIT_EVM_ALL   0
#define RSSPECAN_VAL_OW_UL_LIMIT_EVM_PCH   1
#define RSSPECAN_VAL_OW_UL_LIMIT_EVM_PSIG  2
#define RSSPECAN_VAL_OW_UL_LIMIT_FERR      3
#define RSSPECAN_VAL_OW_UL_LIMIT_GIMB      4
#define RSSPECAN_VAL_OW_UL_LIMIT_IQ_OFFSET 5
#define RSSPECAN_VAL_OW_UL_LIMIT_QUAD      6
#define RSSPECAN_VAL_OW_UL_LIMIT_SERR      7

#define RSSPECAN_VAL_OW_UL_RES_ELEMENT_DMRS_PUSCH 0
#define RSSPECAN_VAL_OW_UL_RES_ELEMENT_DMRS_PUACH 1
#define RSSPECAN_VAL_OW_UL_RES_ELEMENT_PUSCH      2
#define RSSPECAN_VAL_OW_UL_RES_ELEMENT_PUACH      3
#define RSSPECAN_VAL_OW_UL_RES_ELEMENT_DMRS_PUCCH 4
#define RSSPECAN_VAL_OW_UL_RES_ELEMENT_PUCCH      5
#define RSSPECAN_VAL_OW_UL_RES_ELEMENT_PRACH      6

#define RSSPECAN_VAL_OW_UL_MEAS_CREST_FACTOR 0
#define RSSPECAN_VAL_OW_UL_MEAS_EVM_ALL      1
#define RSSPECAN_VAL_OW_UL_MEAS_EVM_CHANNEL  2
#define RSSPECAN_VAL_OW_UL_MEAS_EVM_SIGNAL   3
#define RSSPECAN_VAL_OW_UL_MEAS_FERR         4
#define RSSPECAN_VAL_OW_UL_MEAS_GIMB         5
#define RSSPECAN_VAL_OW_UL_MEAS_IQOF         6
#define RSSPECAN_VAL_OW_UL_MEAS_POW          7
#define RSSPECAN_VAL_OW_UL_MEAS_QERR         8
#define RSSPECAN_VAL_OW_UL_MEAS_SERR         9
#define RSSPECAN_VAL_OW_UL_MEAS_STOF         10

#define RSSPECAN_VAL_TRANSIENT_RESULT_FREQ_INL_AVER   0
#define RSSPECAN_VAL_TRANSIENT_RESULT_FREQ_INL_PEAK   1
#define RSSPECAN_VAL_TRANSIENT_RESULT_FREQ_INL_RMS    2

#define RSSPECAN_VAL_SCALING_AUTO        0
#define RSSPECAN_VAL_SCALING_RESTORE     1
#define RSSPECAN_VAL_SCALING_ALL_WINDOWS 2
#define RSSPECAN_VAL_SCALING_MAXIMUM     3
#define RSSPECAN_VAL_SCALING_MINIMUM     4

#define RSSPECAN_VAL_5G_NR_K0_15_KHZ  0
#define RSSPECAN_VAL_5G_NR_K0_30_KHZ  1
#define RSSPECAN_VAL_5G_NR_K0_60_KHZ  2
#define RSSPECAN_VAL_5G_NR_K0_120_KHZ 3
#define RSSPECAN_VAL_5G_NR_K0_480_KHZ 4
#define RSSPECAN_VAL_5G_NR_K0_960_KHZ 5
#define RSSPECAN_VAL_5G_NR_K0_TX_BW   6

#define RSSPECAN_VAL_CFR_SELECT_ENABLED    0
#define RSSPECAN_VAL_CFR_SELECT_DELTA      1
#define RSSPECAN_VAL_CFR_SELECT_ITERATIONS 2
#define RSSPECAN_VAL_CFR_SELECT_BANDWIDTH  3
#define RSSPECAN_VAL_CFR_SELECT_SPACING    4
#define RSSPECAN_VAL_CFR_SELECT_APPLY      5

#define RSSPECAN_VAL_LTE_DL_CC_REPCAP_MAX 5

#define RSSPECAN_VAL_AXIS_BOTH  2

#define RSSPECAN_VAL_NR5G_BITSTREAM_FULL      1
#define RSSPECAN_VAL_NR5G_BITSTREAM_INFO      2
#define RSSPECAN_VAL_NR5G_BITSTREAM_FULL_INFO 3

#define RSSPECAN_HRP_UWB_PACKET_SYNC_CODE_INDEX   0
#define RSSPECAN_HRP_UWB_PACKET_SYNC_CODE_LENGTH  1
#define RSSPECAN_HRP_UWB_PACKET_SYNC_DELTA_LENGTH 2
#define RSSPECAN_HRP_UWB_PACKET_SYNC_SYNC_LENGTH  3
#define RSSPECAN_HRP_UWB_PACKET_SYNC_SFD          4

#define RSSPECAN_VAL_SUMMARY_ALL              0
#define RSSPECAN_VAL_SUMMARY_PEAK             1
#define RSSPECAN_VAL_SUMMARY_PHYSICAL_CHANNEL 2
#define RSSPECAN_VAL_SUMMARY_PHYSICAL_SIGNAL  3
#define RSSPECAN_VAL_SUMMARY_POWER            4
#define RSSPECAN_VAL_SUMMARY_PDSCH_QPSK       5
#define RSSPECAN_VAL_SUMMARY_PDSCH_64QAM      6
#define RSSPECAN_VAL_SUMMARY_PDSCH_16QAM      7
#define RSSPECAN_VAL_SUMMARY_PDSCH_256QAM     8
#define RSSPECAN_VAL_SUMMARY_PDSCH_1024QAM    9

#define RSSPECAN_HRP_UWB_PACKET_DATA_RATE               0
#define RSSPECAN_HRP_UWB_PACKET_DATA_LENGTH             1
#define RSSPECAN_HRP_UWB_PACKET_DATA_RANGING_BIT        2
#define RSSPECAN_HRP_UWB_PACKET_DATA_SECDEC             3
#define RSSPECAN_HRP_UWB_PACKET_DATA_CONSTRAINT_LENGTH  4
#define RSSPECAN_HRP_UWB_PACKET_DATA_A0A1               5
#define RSSPECAN_HRP_UWB_PACKET_DATA_CBUR               6
#define RSSPECAN_HRP_UWB_PACKET_DATA_HBUR               7
#define RSSPECAN_HRP_UWB_PACKET_DATA_MAC_FCS            8
#define RSSPECAN_HRP_UWB_PACKET_DATA_RESERVED_BIT       9

#define RSSPECAN_VAL_COMPRESSION_POINT_1DB  1
#define RSSPECAN_VAL_COMPRESSION_POINT_2DB  2
#define RSSPECAN_VAL_COMPRESSION_POINT_3DB  3

#define RSSPECAN_VAL_ACLR_OVERALL  0
#define RSSPECAN_VAL_ACLR_ABSOLUTE 1
#define RSSPECAN_VAL_ACLR_RELATIVE 2

#define RSSPECAN_VAL_SUMMARY_DMRS_PI_2_BPSK        5
#define RSSPECAN_VAL_SUMMARY_DMRS_PUSCH_QPSK       6
#define RSSPECAN_VAL_SUMMARY_DMRS_PUSCH_16QAM      7
#define RSSPECAN_VAL_SUMMARY_DMRS_PUSCH_64QAM      8
#define RSSPECAN_VAL_SUMMARY_DMRS_PUSCH_256QAM     9
#define RSSPECAN_VAL_SUMMARY_PUCCH_DMRS            10
#define RSSPECAN_VAL_SUMMARY_PUCCH                 11
#define RSSPECAN_VAL_SUMMARY_PUSCH_PI_2_BPSK       12
#define RSSPECAN_VAL_SUMMARY_PUSCH_QPSK            13
#define RSSPECAN_VAL_SUMMARY_PUSCH_16QAM           14
#define RSSPECAN_VAL_SUMMARY_PUSCH_64QAM           15
#define RSSPECAN_VAL_SUMMARY_PUSCH_256QAM          16
#define RSSPECAN_VAL_SUMMARY_DMRS_PUSCH_1024QAM    17
#define RSSPECAN_VAL_SUMMARY_DMRS_PUSCH_RE_1024QAM 18

/****************************************************************************
 *------------------------ Error And Completion Codes ----------------------*
 ****************************************************************************/

#define RSSPECAN_WARN_MEASURE_UNCALIBRATED    (RS_CLASS_WARN_BASE + 0x0001L)
#define RSSPECAN_WARN_OVER_RANGE              (RS_CLASS_WARN_BASE + 0x0002L)

#define RSSPECAN_ERROR_INCORRECT_RESPONSE     (RS_CLASS_ERROR_BASE + 0x0004L)
#define RSSPECAN_ERROR_MARKER_NOT_ENABLED     (RS_CLASS_ERROR_BASE + 0x0003L)
#define RSSPECAN_ERROR_NOT_DELTA_MARKER       (RS_CLASS_ERROR_BASE + 0x0002L)
#define RSSPECAN_ERROR_MAX_TIME_EXCEEDED      (RS_CLASS_ERROR_BASE + 0x0001L)

#define RSSPECAN_WARNMSG_MEASURE_UNCALIBRATED "Uncalibrated measurement"
#define RSSPECAN_WARNMSG_OVER_RANGE           "Measurement overrange"

#define RSSPECAN_ERRMSG_INCORRECT_RESPONSE    "Response incorrect; report problem to R&S support!"
#define RSSPECAN_ERRMSG_MARKER_NOT_ENABLED    "Marker Not Enabled"
#define RSSPECAN_ERRMSG_NOT_DELTA_MARKER      "Not Delta Marker"
#define RSSPECAN_ERRMSG_MAX_TIME_EXCEEDED     "Max Time Exceeded"

#define RSSPECAN_ERROR_CODES_AND_MSGS \
    {RSSPECAN_WARN_MEASURE_UNCALIBRATED,  RSSPECAN_WARNMSG_MEASURE_UNCALIBRATED},\
    {RSSPECAN_WARN_OVER_RANGE,            RSSPECAN_WARNMSG_OVER_RANGE},\
    {RSSPECAN_ERROR_INCORRECT_RESPONSE,   RSSPECAN_ERRMSG_INCORRECT_RESPONSE},\
    {RSSPECAN_ERROR_MARKER_NOT_ENABLED,   RSSPECAN_ERRMSG_MARKER_NOT_ENABLED},\
    {RSSPECAN_ERROR_NOT_DELTA_MARKER,     RSSPECAN_ERRMSG_NOT_DELTA_MARKER},\
    {RSSPECAN_ERROR_MAX_TIME_EXCEEDED,    RSSPECAN_ERRMSG_MAX_TIME_EXCEEDED}

 /****************************************************************************
  *---------------- Instrument Driver Function Declarations -----------------*
  ****************************************************************************/
ViStatus _VI_FUNC rsspecan_init(ViRsrc     resourceName,
                                ViBoolean  IDQuery,
                                ViBoolean  resetDevice,
                                ViSession* instrSession);

ViStatus _VI_FUNC rsspecan_InitWithOptions(ViRsrc     resourceName,
                                           ViBoolean  IDQuery,
                                           ViBoolean  resetDevice,
                                           ViString   optionString,
                                           ViSession* instrSession);

ViStatus _VI_FUNC rsspecan_SetSANMode(ViSession instrSession);

ViStatus _VI_FUNC rsspecan_ConfigureAcquisition(ViSession instrSession,
                                                ViInt32   reserved,
                                                ViBoolean sweepModeContinuous,
                                                ViInt32   numberOfSweeps);

ViStatus _VI_FUNC rsspecan_ConfigureSweepTime(ViSession instrSession,
                                              ViReal64  sweepTime);

ViStatus _VI_FUNC rsspecan_ConfigureSweepMode(ViSession instrSession,
                                              ViBoolean sweepModeContinuous);

ViStatus _VI_FUNC rsspecan_AddMeasurementChannel(ViSession instrSession,
                                                 ViInt32   channelType,
                                                 ViString  channelName);

ViStatus _VI_FUNC rsspecan_SelectMeasurementChannel(ViSession instrSession,
                                                    ViString  channelName);

ViStatus _VI_FUNC rsspecan_DuplicateMeasurementChannel(ViSession instrSession);

ViStatus _VI_FUNC rsspecan_ReplaceMeasurementChannel(ViSession instrSession,
                                                     ViString  oldChannelName,
                                                     ViInt32   newChannelType,
                                                     ViString  newChannelName);

ViStatus _VI_FUNC rsspecan_RenameMeasurementChannel(ViSession instrSession,
                                                    ViString  oldChannelName,
                                                    ViString  newChannelName);

ViStatus _VI_FUNC rsspecan_DeleteMeasurementChannel(ViSession instrSession,
                                                    ViString  channelName);

ViStatus _VI_FUNC rsspecan_PresetMeasurementChannel(ViSession instrSession);

ViStatus _VI_FUNC rsspecan_QueryAllMeasurementChannels(ViSession instrSession,
                                                       ViInt32   arraySize,
                                                       ViInt32   channelType[],
                                                       ViChar    channelName[],
                                                       ViInt32*  returnedValues);

ViStatus _VI_FUNC rsspecan_InputProtectionReset(ViSession instrSession,
                                                ViInt32   reserved);

ViStatus _VI_FUNC rsspecan_ConfigureRFInputState(ViSession instrSession,
                                                 ViInt32   source);

ViStatus _VI_FUNC rsspecan_ConfigureRFInput(ViSession instrSession,
                                            ViInt32   inputCoupling,
                                            ViReal64  impedance);

ViStatus _VI_FUNC rsspecan_ConfigureRFInputWithUserImpedance(ViSession instrSession,
                                                             ViInt32   inputCoupling,
                                                             ViReal64  impedance,
                                                             ViInt32   padType);

ViStatus _VI_FUNC rsspecan_ConfigureRFInputType(ViSession instrSession,
                                                ViInt32   inputType);

ViStatus _VI_FUNC rsspecan_ConfigureInputConnector(ViSession instrSession,
                                                   ViInt32   inputConnector);

ViStatus _VI_FUNC rsspecan_ConfigureDirectPath(ViSession instrSession,
                                               ViInt32   directPath);

ViStatus _VI_FUNC rsspecan_ConfigureHighpassFilter(ViSession instrSession,
                                                   ViBoolean highpassFilter);

ViStatus _VI_FUNC rsspecan_ConfigureYIGFilter(ViSession instrSession,
                                              ViInt32   reserved,
                                              ViBoolean YIGFilter);

ViStatus _VI_FUNC rsspecan_ConfigurePreselectorAdjustment(ViSession instrSession,
                                                          ViBoolean preselectorAdjustment);

ViStatus _VI_FUNC rsspecan_ProbesMicroButtonAction(ViSession instrSession,
                                                   ViInt32   input,
                                                   ViInt32   microButtonAction);

ViStatus _VI_FUNC rsspecan_GetProbesInformation(ViSession instrSession,
                                                ViInt32   probeConnector,
                                                ViInt32   bufferSize,
                                                ViInt32*  state,
                                                ViChar    name[],
                                                ViChar    serialNumber[],
                                                ViChar    partNumber[],
                                                ViChar    probeType[]);

ViStatus _VI_FUNC rsspecan_ConfigureProbeMode(ViSession instrSession,
                                              ViInt32   probeConnector,
                                              ViInt32   probeMode,
                                              ViReal64  offset);

ViStatus _VI_FUNC rsspecan_ConfigureIFOutput(ViSession instrSession,
                                             ViInt32   outputSource,
                                             ViReal64  outputFrequency);

ViStatus _VI_FUNC rsspecan_QueryIFOutputSideband(ViSession instrSession,
                                                 ViInt32*  sideband);

ViStatus _VI_FUNC rsspecan_ConfigureNoiseSource(ViSession instrSession,
                                                ViInt32   reserved,
                                                ViBoolean noiseSource);

ViStatus _VI_FUNC rsspecan_ConfigureTriggerInput(ViSession instrSession,
                                                 ViInt32   port);

ViStatus _VI_FUNC rsspecan_ConfigureTriggerOutput(ViSession instrSession,
                                                  ViInt32   port,
                                                  ViInt32   outputType,
                                                  ViInt32   level,
                                                  ViReal64  length);

ViStatus _VI_FUNC rsspecan_GenerateTriggerPulse(ViSession instrSession,
                                                ViInt32   port);

ViStatus _VI_FUNC rsspecan_ConfigureControlLinesState(ViSession instrSession,
                                                      ViInt32   reserved,
                                                      ViBoolean state);

ViStatus _VI_FUNC rsspecan_ConfigureOutputControlLines(ViSession instrSession,
                                                       ViInt32   reserved,
                                                       ViString  userPorts);

ViStatus _VI_FUNC rsspecan_ConfigureAUXPortControlLinesState(ViSession instrSession,
                                                             ViBoolean state);

ViStatus _VI_FUNC rsspecan_GetControlLines(ViSession instrSession,
                                           ViInt32   reserved,
                                           ViInt32   lines,
                                           ViChar    userPorts[]);

ViStatus _VI_FUNC rsspecan_ConfigureFrequencyStartStop(ViSession instrSession,
                                                       ViInt32   reserved,
                                                       ViReal64  startFrequency,
                                                       ViReal64  stopFrequency);

ViStatus _VI_FUNC rsspecan_ConfigureFrequencyAnnotationMode(ViSession instrSession,
                                                            ViInt32   mode);

ViStatus _VI_FUNC rsspecan_ConfigureFrequencyCenterSpan(ViSession instrSession,
                                                        ViInt32   reserved,
                                                        ViReal64  centerFrequency,
                                                        ViReal64  span);

ViStatus _VI_FUNC rsspecan_ConfigureFrequencyCenter(ViSession instrSession,
                                                    ViInt32   reserved,
                                                    ViReal64  centerFrequency);

ViStatus _VI_FUNC rsspecan_ConfigureFrequencyStepSize(ViSession instrSession,
                                                      ViInt32   reserved,
                                                      ViReal64  stepSize);

ViStatus _VI_FUNC rsspecan_ConfigureFrequencyStepSizeAuto(ViSession instrSession,
                                                          ViInt32   reserved,
                                                          ViBoolean stepSizeAuto);

ViStatus _VI_FUNC rsspecan_ConfigureHorizontalScale(ViSession instrSession,
                                                    ViInt32   reserved,
                                                    ViInt32   horizontalScale);

ViStatus _VI_FUNC rsspecan_ConfigureFrequencySpanFull(ViSession instrSession,
                                                      ViInt32   reserved);

ViStatus _VI_FUNC rsspecan_ConfigureFrequencyZeroSpan(ViSession instrSession,
                                                      ViInt32   reserved);

ViStatus _VI_FUNC rsspecan_ConfigureFrequencyOffset(ViSession instrSession,
                                                    ViInt32   reserved,
                                                    ViReal64  frequencyOffset);

ViStatus _VI_FUNC rsspecan_ConfigureFrequencyCouplingFactor(ViSession instrSession,
                                                            ViInt32   reserved,
                                                            ViInt32   coupling,
                                                            ViInt32   couplingFactor);

ViStatus _VI_FUNC rsspecan_ConfSignalTrack(ViSession instrSession,
                                           ViInt32   marker,
                                           ViBoolean signalTrackEnabled,
                                           ViReal64  signalTrackBandwidth,
                                           ViReal64  signalTrackThreshold,
                                           ViInt32   trace);

ViStatus _VI_FUNC rsspecan_ConfigureFrequencyMode(ViSession instrSession,
                                                  ViInt32   reserved,
                                                  ViInt32   frequencyMode);

ViStatus _VI_FUNC rsspecan_ConfigureFrequencyLevel(ViSession instrSession,
                                                   ViInt32   frequencyMode);

ViStatus _VI_FUNC rsspecan_QueryFrequencyLevelSettingUsed(ViSession instrSession,
                                                          ViInt32   *levelSetting);

ViStatus _VI_FUNC rsspecan_ConfigureSubwindowHorizontalScale(ViSession instrSession,
                                                             ViInt32   window,
                                                             ViInt32   subwindow,
                                                             ViInt32   scale);

ViStatus _VI_FUNC rsspecan_ConfigureVerticalScaling(ViSession instrSession,
                                                    ViInt32   verticalScale,
                                                    ViInt32   yAxisGridMode);

ViStatus _VI_FUNC rsspecan_ConfigureSubwindowVerticalScale(ViSession instrSession,
                                                           ViInt32   window,
                                                           ViInt32   subwindow,
                                                           ViInt32   verticalScale);

ViStatus _VI_FUNC rsspecan_ConfigureVerticalRange(ViSession instrSession,
                                                  ViReal64  range,
                                                  ViReal64  referencePosition);

ViStatus _VI_FUNC rsspecan_ConfigureDisplayAutoscaleOnce(ViSession instrSession);

ViStatus _VI_FUNC rsspecan_ConfigureAutoScalingEnabled(ViSession instrSession,
                                                       ViInt32   window,
                                                       ViBoolean autoScalingEnabled);

ViStatus _VI_FUNC rsspecan_ConfigureReferenceLevel(ViSession instrSession,
                                                   ViInt32   reserved,
                                                   ViReal64  referenceLevel);

ViStatus _VI_FUNC rsspecan_SetReferenceLevelToLimit(ViSession instrSession,
                                                    ViInt32   limit);

ViStatus _VI_FUNC rsspecan_GetReferenceLevelRange(ViSession instrSession,
                                                  ViReal64* minimum,
                                                  ViReal64* maximum);

ViStatus _VI_FUNC rsspecan_ConfigureReferenceLevelOffset(ViSession instrSession,
                                                         ViInt32   reserved,
                                                         ViReal64  referenceLevelOffset);

ViStatus _VI_FUNC rsspecan_ConfigureReferenceLevelUnits(ViSession instrSession,
                                                        ViInt32   amplitudeUnits);

ViStatus _VI_FUNC rsspecan_ConfigureAttenuation(ViSession instrSession,
                                                ViInt32   reserved,
                                                ViBoolean attenuationAuto,
                                                ViReal64  attenuation);

ViStatus _VI_FUNC rsspecan_ConfigureAnalyzerAttenuation(ViSession instrSession,
                                                        ViBoolean automatic,
                                                        ViReal64  attenuation);

ViStatus _VI_FUNC rsspecan_ConfigureElectronicAttenuator(ViSession instrSession,
                                                         ViInt32   reserved,
                                                         ViInt32   mode,
                                                         ViInt32   manualValue);

ViStatus _VI_FUNC rsspecan_ConfigureSignalPathEnabled(ViSession instrSession,
                                                      ViBoolean signalPath);

ViStatus _VI_FUNC rsspecan_ConfigurePreamplifier(ViSession instrSession,
                                                 ViBoolean state,
                                                 ViInt32   preamplifierLevel);

ViStatus _VI_FUNC rsspecan_ConfigureSweepCoupling(ViSession instrSession,
                                                  ViInt32   reserved,
                                                  ViInt32   sweepCoupling,
                                                  ViReal64  couplingValue);

ViStatus _VI_FUNC rsspecan_ConfigureSweepCouplingAuto(ViSession instrSession,
                                                      ViInt32   reserved,
                                                      ViInt32   sweepCoupling);

ViStatus _VI_FUNC rsspecan_ConfigureSweepCouplingAdvanced(ViSession instrSession,
                                                          ViInt32   reserved,
                                                          ViInt32   ratioSelection,
                                                          ViReal64  value);

ViStatus _VI_FUNC rsspecan_ConfigureFilterType(ViSession instrSession,
                                               ViInt32   reserved,
                                               ViInt32   filterType);

ViStatus _VI_FUNC rsspecan_ConfigureVBWFilterType(ViSession instrSession,
                                                  ViInt32   reserved,
                                                  ViInt32   videoFilterType);

ViStatus _VI_FUNC rsspecan_ConfigureSweep(ViSession instrSession,
                                          ViBoolean sweepTimeAuto,
                                          ViReal64  time,
                                          ViInt32   points,
                                          ViInt32   type,
                                          ViInt32   optimization);

ViStatus _VI_FUNC rsspecan_QuerySweepAutoType(ViSession instrSession,
                                              ViInt32*  autoType);

ViStatus _VI_FUNC rsspecan_ConfigureSweepZeroSpan(ViSession instrSession,
                                                  ViReal64  time,
                                                  ViInt32   points,
                                                  ViInt32   optimization);

ViStatus _VI_FUNC rsspecan_ConfigureSweepPoints(ViSession instrSession,
                                                ViInt32   window,
                                                ViInt32   sweepPoints);

ViStatus _VI_FUNC rsspecan_ConfigureWindowSweepPoints(ViSession instrSession,
                                                      ViInt32   window,
                                                      ViInt32   sweepPoints);

ViStatus _VI_FUNC rsspecan_QuerySweepPoints(ViSession instrSession,
                                            ViInt32   window,
                                            ViInt32*  sweepPoints);

ViStatus _VI_FUNC rsspecan_QuerySweepDuration(ViSession instrSession,
                                              ViReal64* sweepDuration);

ViStatus _VI_FUNC rsspecan_QuerySweepSubspans(ViSession instrSession,
                                              ViInt32*  subspans);

ViStatus _VI_FUNC rsspecan_ConfigureTriggerSource(ViSession instrSession,
                                                  ViInt32   reserved,
                                                  ViInt32   triggerSource);

ViStatus _VI_FUNC rsspecan_ConfigureTrigger(ViSession instrSession,
                                            ViInt32   reserved,
                                            ViReal64  offset,
                                            ViInt32   slope);

ViStatus _VI_FUNC rsspecan_ConfigureTriggerDropoutTime(ViSession instrSession,
                                                       ViReal64  dropoutTime);

ViStatus _VI_FUNC rsspecan_ConfigureExternalTrigger(ViSession instrSession,
                                                    ViInt32   reserved,
                                                    ViReal64  level);

ViStatus _VI_FUNC rsspecan_ConfigureExternalTriggerPort(ViSession instrSession,
                                                        ViInt32   port,
                                                        ViReal64  externalTriggerLevel);

ViStatus _VI_FUNC rsspecan_ConfigureVideoTrigger(ViSession instrSession,
                                                 ViInt32   reserved,
                                                 ViReal64  triggerLevel);

ViStatus _VI_FUNC rsspecan_ConfIFPowerTrigger(ViSession instrSession,
                                              ViReal64  triggerLevel,
                                              ViReal64  hysteresis,
                                              ViReal64  holdoff);

ViStatus _VI_FUNC rsspecan_ConfigureIFPowerTriggerCoupling(ViSession instrSession,
                                                           ViInt32   coupling);

ViStatus _VI_FUNC rsspecan_ConfigureIQPowerTrigger(ViSession instrSession,
                                                   ViInt32   reserved,
                                                   ViReal64  triggerLevel);

ViStatus _VI_FUNC rsspecan_ConfigureRFPowerTrigger(ViSession instrSession,
                                                   ViInt32   reserved,
                                                   ViReal64  triggerLevel);

ViStatus _VI_FUNC rsspecan_ConfigureTimeTrigger(ViSession instrSession,
                                                ViInt32   reserved,
                                                ViReal64  timeInterval);

ViStatus _VI_FUNC rsspecan_ConfigureExternalGate(ViSession instrSession,
                                                 ViInt32   reserved,
                                                 ViBoolean gatedTrigger,
                                                 ViInt32   gateSource,
                                                 ViInt32   mode,
                                                 ViInt32   polarity,
                                                 ViReal64  delay,
                                                 ViReal64  length);

ViStatus _VI_FUNC rsspecan_ConfigureContinuousGate(ViSession instrSession,
                                                   ViBoolean continuousEnabled,
                                                   ViReal64  periodLength,
                                                   ViInt32   periodCount);

ViStatus _VI_FUNC rsspecan_ConfigureGateSourceMode(ViSession instrSession,
                                                   ViBoolean sameTriggerEnabled);

ViStatus _VI_FUNC rsspecan_ConfigureExternalGateRFLevel(ViSession instrSession,
                                                        ViReal64  RFLevel);

ViStatus _VI_FUNC rsspecan_ConfigureExternalGatePort(ViSession instrSession,
                                                     ViInt32   port,
                                                     ViReal64  externalGateLevel);

ViStatus _VI_FUNC rsspecan_ConfigureFrequencyTrigger(ViSession instrSession,
                                                     ViReal64  frequency,
                                                     ViReal64  hysteresis);

ViStatus _VI_FUNC rsspecan_ConfigureFrequencyMaskTriggerSource(ViSession instrSession);

ViStatus _VI_FUNC rsspecan_ConfigureFrequencyMask(ViSession instrSession,
                                                  ViString  name,
                                                  ViString  comment);

ViStatus _VI_FUNC rsspecan_ConfigureFrequencyMaskDirectory(ViSession instrSession,
                                                           ViString  directory);

ViStatus _VI_FUNC rsspecan_DeleteFrequencyMask(ViSession instrSession,
                                               ViString  name);

ViStatus _VI_FUNC rsspecan_ConfigureFrequencyMaskCondition(ViSession instrSession,
                                                           ViInt32   condition);

ViStatus _VI_FUNC rsspecan_ConfigureFrequencyMaskShape(ViSession instrSession,
                                                       ViInt32   arraySize,
                                                       ViInt32   mode,
                                                       ViReal64  lower[],
                                                       ViReal64  upper[]);

ViStatus _VI_FUNC rsspecan_ConfigureFrequencyMaskAuto(ViSession instrSession);

ViStatus _VI_FUNC rsspecan_ConfigureFrequencyMaskShift(ViSession instrSession,
                                                       ViInt32   type,
                                                       ViReal64  frequency,
                                                       ViReal64  amplitude);

ViStatus _VI_FUNC rsspecan_ConfigureFrequencyMaskState(ViSession instrSession,
                                                       ViInt32   type,
                                                       ViBoolean frequencyMask);

ViStatus _VI_FUNC rsspecan_ConfigureFrequencyMaskSpan(ViSession instrSession,
                                                      ViReal64  span);

ViStatus _VI_FUNC rsspecan_ConfigureAutoAdjust(ViSession instrSession,
                                               ViInt32   adjustSettings,
                                               ViInt32   measurementTimeMode,
                                               ViReal64  measurementTime);

ViStatus _VI_FUNC rsspecan_ConfigureAutoAdjustHystersis(ViSession instrSession,
                                                        ViReal64  lowerHystersis,
                                                        ViReal64  upperHystersis);

ViStatus _VI_FUNC rsspecan_ConfigureAutoAdjustTriggerState(ViSession instrSession,
                                                           ViBoolean state);

ViStatus _VI_FUNC rsspecan_ConfigureAutoAdjustSearchMode(ViSession instrSession,
                                                         ViInt32   searchMode);

ViStatus _VI_FUNC rsspecan_ConfigureDisplayYAxisScaleRange(ViSession instrSession,
                                                           ViInt32   window,
                                                           ViReal64  min,
                                                           ViReal64  max);

ViStatus _VI_FUNC rsspecan_ConfigureDisplayYAxisRange(ViSession instrSession,
                                                      ViInt32   window,
                                                      ViReal64  min,
                                                      ViReal64  max);

ViStatus _VI_FUNC rsspecan_ConfigureDisplayWindowYAxisGridSpacing(
                            ViSession instrSession,
                            ViInt32   window,
                            ViReal64  spacing);

ViStatus _VI_FUNC rsspecan_ConfigureReferenceLevelPosition(ViSession instrSession,
                                                           ViInt32   window,
                                                           ViReal64  referenceLevelPosition);

ViStatus _VI_FUNC rsspecan_ConfigureDisplayWindowReferenceLevel(
                            ViSession instrSession,
                            ViInt32   window,
                            ViReal64  displayWindowReferenceLevel);

ViStatus _VI_FUNC rsspecan_ConfigureDisplayWindowReferenceLevelRange(
                            ViSession instrSession,
                            ViInt32   window,
                            ViReal64  min,
                            ViReal64  max);

ViStatus _VI_FUNC rsspecan_ConfigureDisplayWindowUnitPhase(ViSession instrSession,
                                                           ViInt32   window,
                                                           ViInt32   unitPhase);

ViStatus _VI_FUNC rsspecan_ConfigureDisplayWindowUnitFrequency(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   unitFrequency);

ViStatus _VI_FUNC rsspecan_ConfigureDisplayWindowReferenceValue(
                            ViSession instrSession,
                            ViInt32   window,
                            ViReal64  referenceValue);

ViStatus _VI_FUNC rsspecan_ConfigureDisplayWindowMarkerInfo(ViSession instrSession,
                                                            ViInt32   window,
                                                            ViBoolean markerInfo);

ViStatus _VI_FUNC rsspecan_ConfigureDisplayZoom(ViSession instrSession,
                                                ViInt32   reserved,
                                                ViBoolean zoomState,
                                                ViReal64  x1,
                                                ViReal64  y1,
                                                ViReal64  x2,
                                                ViReal64  y2);

ViStatus _VI_FUNC rsspecan_ConfigureDisplayMultipleZoom(ViSession instrSession,
                                                        ViInt32   reserved,
                                                        ViInt32   zoomWindow,
                                                        ViBoolean enableZoom,
                                                        ViReal64  x1,
                                                        ViReal64  y1,
                                                        ViReal64  x2,
                                                        ViReal64  y2);

ViStatus _VI_FUNC rsspecan_ConfigureDisplaySubwindowMultipleZoom(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   subwindow,
                            ViInt32   zoomWindow,
                            ViBoolean enableZoom,
                            ViReal64  x1,
                            ViReal64  y1,
                            ViReal64  x2,
                            ViReal64  y2);

ViStatus _VI_FUNC rsspecan_ConfigureAveraging(ViSession instrSession,
                                              ViInt32   reserved,
                                              ViInt32   trace,
                                              ViBoolean state);

ViStatus _VI_FUNC rsspecan_ConfigureAveragingType(ViSession instrSession,
                                                  ViInt32   reserved,
                                                  ViInt32   averagingType);

ViStatus _VI_FUNC rsspecan_ConfigureAveragingCount(ViSession instrSession,
                                                   ViInt32   reserved,
                                                   ViInt32   count);

ViStatus _VI_FUNC rsspecan_ConfigureTrace(ViSession instrSession,
                                          ViInt32   window,
                                          ViInt32   trace,
                                          ViInt32   traceType);

ViStatus _VI_FUNC rsspecan_ConfigureTraceDetector(ViSession instrSession,
                                                  ViInt32   reserved,
                                                  ViInt32   trace,
                                                  ViBoolean detectorTypeAuto,
                                                  ViInt32   detectorType);

ViStatus _VI_FUNC rsspecan_ConfigureTraceSmoothing(ViSession instrSession,
                                                   ViInt32   window,
                                                   ViInt32   trace,
                                                   ViBoolean smoothingEnabled,
                                                   ViReal64  smoothingAperture);

ViStatus _VI_FUNC rsspecan_ConfigureTraceSymbolEnabled(ViSession instrSession,
                                                       ViInt32   window,
                                                       ViBoolean traceSymbolEnabled);

ViStatus _VI_FUNC rsspecan_ConfigureTraceResetBehavior(ViSession instrSession,
                                                       ViInt32   window,
                                                       ViInt32   trace,
                                                       ViBoolean resetAtChange);

ViStatus _VI_FUNC rsspecan_ConfigureTraceSubwindowState(ViSession instrSession,
                                                        ViInt32   window,
                                                        ViInt32   subwindow,
                                                        ViInt32   trace,
                                                        ViBoolean resetAtChange);

ViStatus _VI_FUNC rsspecan_ConfigureTraceSubwindowSmoothing(ViSession instrSession,
                                                            ViInt32   window,
                                                            ViInt32   subwindow,
                                                            ViInt32   trace,
                                                            ViBoolean enabled,
                                                            ViReal64  aperture);

ViStatus _VI_FUNC rsspecan_ConfigureTraceSubwindowResetBehavior(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   subwindow,
                            ViInt32   trace,
                            ViBoolean resetAtChange);

ViStatus _VI_FUNC rsspecan_ConfigureTraceSubwindowPreset(ViSession instrSession,
                                                         ViInt32   window,
                                                         ViInt32   subwindow,
                                                         ViInt32   trace,
                                                         ViInt32   preset);

ViStatus _VI_FUNC rsspecan_CopyTrace(ViSession instrSession,
                                     ViInt32   reserved,
                                     ViInt32   destinationTrace,
                                     ViInt32   sourceTrace);

ViStatus _VI_FUNC rsspecan_ConfigureSubtractTraces(ViSession instrSession,
                                                   ViInt32   reserved,
                                                   ViReal64  position,
                                                   ViInt32   mode,
                                                   ViBoolean traceMathState);

ViStatus _VI_FUNC rsspecan_SubtractTraces(ViSession instrSession,
                                          ViInt32   reserved,
                                          ViInt32   trace2);

ViStatus _VI_FUNC rsspecan_ConfigureTraceFileType(ViSession instrSession,
                                                  ViInt32   fileType);

ViStatus _VI_FUNC rsspecan_ConfigureTraceExport(ViSession instrSession,
                                                ViInt32   exportMode,
                                                ViBoolean exportHeader,
                                                ViInt32   decimalSeparator);

ViStatus _VI_FUNC rsspecan_ConfigureTraceExportColumnSeperator(
                            ViSession instrSession,
                            ViInt32   seperator);

ViStatus _VI_FUNC rsspecan_ConfigureTraceExportXValueDistribution(
                            ViSession instrSession,
                            ViInt32   distribution);

ViStatus _VI_FUNC rsspecan_StoreTraceToFile(ViSession instrSession,
                                            ViInt32   window,
                                            ViInt32   trace,
                                            ViString  fileName);

ViStatus _VI_FUNC rsspecan_ConfigureTraceImport(ViSession instrSession,
                                                ViInt32   importMode);

ViStatus _VI_FUNC rsspecan_ImportSingleTraceFromFile(ViSession instrSession,
                                                     ViInt32   window,
                                                     ViInt32   trace,
                                                     ViString  fileName);

ViStatus _VI_FUNC rsspecan_ConfigureMarker(ViSession instrSession,
                                           ViInt32   window,
                                           ViInt32   marker,
                                           ViBoolean markerState,
                                           ViInt32   trace);

ViStatus _VI_FUNC rsspecan_DisableAllMarkers(ViSession instrSession,
                                             ViInt32   window);

ViStatus _VI_FUNC rsspecan_MarkerSearch(ViSession instrSession,
                                        ViInt32   window,
                                        ViInt32   marker,
                                        ViInt32   markerSearch);

ViStatus _VI_FUNC rsspecan_MoveMarker(ViSession instrSession,
                                      ViInt32   window,
                                      ViInt32   marker,
                                      ViReal64  markerPosition);

ViStatus _VI_FUNC rsspecan_SetInstrumentFromMarker(ViSession instrSession,
                                                   ViInt32   window,
                                                   ViInt32   marker,
                                                   ViInt32   instrumentSetting);

ViStatus _VI_FUNC rsspecan_LinkMarkerToAnotherMarker(ViSession instrSession,
                                                     ViInt32   window,
                                                     ViInt32   firstMarker,
                                                     ViInt32   secondMarker,
                                                     ViBoolean state);

ViStatus _VI_FUNC rsspecan_LinkingToDeltaMarker(ViSession instrSession,
                                                ViInt32   window,
                                                ViInt32   marker,
                                                ViInt32   deltaMarker,
                                                ViBoolean link);

ViStatus _VI_FUNC rsspecan_QueryMarker(ViSession instrSession,
                                       ViInt32   window,
                                       ViInt32   marker,
                                       ViReal64* markerPosition,
                                       ViReal64* markerAmplitude);

ViStatus _VI_FUNC rsspecan_ConfigureMarkerZoom(ViSession instrSession,
                                               ViInt32   window,
                                               ViReal64  markerZoom,
                                               ViUInt32  timeout);

ViStatus _VI_FUNC rsspecan_ConfigureDisplayMarkerTable(ViSession instrSession,
                                                       ViInt32   markerTable);

ViStatus _VI_FUNC rsspecan_ConfigureMarkerStepSize(ViSession instrSession,
                                                   ViInt32   reserved,
                                                   ViInt32   stepSize);

ViStatus _VI_FUNC rsspecan_ConfigureMarkerInfo(ViSession instrSession,
                                               ViBoolean markerInfo);

ViStatus _VI_FUNC rsspecan_ConfigureMarkerSearch(ViSession instrSession,
                                                 ViBoolean localOscillatorSupression,
                                                 ViReal64  peakExcursion);

ViStatus _VI_FUNC rsspecan_MarkerPeakSearchAuto(ViSession instrSession,
                                                ViInt32   reserved,
                                                ViInt32   marker,
                                                ViInt32   markerSearch,
                                                ViBoolean autoPeak);

ViStatus _VI_FUNC rsspecan_ConfigMarkerSearchLimits(ViSession instrSession,
                                                    ViBoolean searchLimits,
                                                    ViReal64  searchLimitLeft,
                                                    ViReal64  searchLimitRight,
                                                    ViBoolean thresholdState,
                                                    ViReal64  markerThreshold,
                                                    ViBoolean searchLimitsZoom);

ViStatus _VI_FUNC rsspecan_DeactivateAllSearchLimits(ViSession instrSession);

ViStatus _VI_FUNC rsspecan_ConfigureSpuriousTracking(ViSession instrSession,
                                                     ViBoolean trackingState,
                                                     ViReal64  frequencyRange);

ViStatus _VI_FUNC rsspecan_ConfigureMarkerFrequencyCounter(ViSession instrSession,
                                                           ViInt32   reserved,
                                                           ViInt32   marker,
                                                           ViBoolean markerFrequencyCounter,
                                                           ViReal64  frequencyCounterResolution);

ViStatus _VI_FUNC rsspecan_QueryMarkerFrequencyCounterResult(ViSession instrSession,
                                                             ViInt32   reserved,
                                                             ViInt32   marker,
                                                             ViReal64* counterFrequency);

ViStatus _VI_FUNC rsspecan_ConfigureMarkerNoiseMeasurement(ViSession instrSession,
                                                           ViInt32   reserved,
                                                           ViInt32   marker,
                                                           ViBoolean state);

ViStatus _VI_FUNC rsspecan_QueryMarkerNoiseMeasurementResult(ViSession instrSession,
                                                             ViInt32   reserved,
                                                             ViInt32   marker,
                                                             ViReal64* result);

ViStatus _VI_FUNC rsspecan_MarkerNoiseMeasurementAllOff(ViSession instrSession);

ViStatus _VI_FUNC rsspecan_ConfigureMarkerPhaseNoiseMeasurement(
                            ViSession instrSession,
                            ViInt32   reserved,
                            ViBoolean state);

ViStatus _VI_FUNC rsspecan_PhaseNoiseReferencePoint(ViSession instrSession,
                                                    ViReal64  level,
                                                    ViReal64  frequency);

ViStatus _VI_FUNC rsspecan_PhaseNoiseRefPointPeakSearch(ViSession instrSession);

ViStatus _VI_FUNC rsspecan_PhaseNoisePeakSearchAuto(ViSession instrSession,
                                                    ViInt32   reserved,
                                                    ViBoolean state);

ViStatus _VI_FUNC rsspecan_QueryPhaseNoiseResult(ViSession instrSession,
                                                 ViReal64* phaseNoiseResult);

ViStatus _VI_FUNC rsspecan_PhaseNoiseAllOff(ViSession instrSession);

ViStatus _VI_FUNC rsspecan_ConfigureNdBPoints(ViSession instrSession,
                                              ViInt32   reserved,
                                              ViBoolean nDBState,
                                              ViReal64  nDBLevel);

ViStatus _VI_FUNC rsspecan_QueryNdBResult(ViSession instrSession,
                                          ViInt32   reserved,
                                          ViReal64* nDBResult);

ViStatus _VI_FUNC rsspecan_QueryNdBFrequencies(ViSession instrSession,
                                               ViInt32   reserved,
                                               ViReal64* nDBFrequencyLower,
                                               ViReal64* nDBFrequencyHigher);

ViStatus _VI_FUNC rsspecan_QueryNdBTimes(ViSession instrSession,
                                         ViInt32   reserved,
                                         ViReal64* nDBTimeLower,
                                         ViReal64* nDBTimeHigher);

ViStatus _VI_FUNC rsspecan_QueryNdBQFactor(ViSession instrSession,
                                           ViInt32   reserved,
                                           ViReal64* nDBQFactor);

ViStatus _VI_FUNC rsspecan_ConfigureMarkerBandPower(ViSession instrSession,
                                                    ViBoolean bandPowerEnabled,
                                                    ViInt32   displayMode,
                                                    ViReal64  span);

ViStatus _VI_FUNC rsspecan_QueryMarkerBandPower(ViSession instrSession,
                                                ViReal64* result);

ViStatus _VI_FUNC rsspecan_MarkerBandPowerAllOff(ViSession instrSession);

ViStatus _VI_FUNC rsspecan_ConfMarkerDemodulation(ViSession instrSession,
                                                  ViInt32   marker,
                                                  ViBoolean state,
                                                  ViInt32   demodulationType,
                                                  ViReal64  markerStopTime,
                                                  ViBoolean continuousDemodulation);

ViStatus _VI_FUNC rsspecan_ConfigureMarkerSelectiveDemodulation(
                            ViSession instrSession,
                            ViBoolean selectiveDemodulationEnabled,
                            ViReal64  demodulationThreshold);

ViStatus _VI_FUNC rsspecan_ConfigureMarkerPeakListState(ViSession instrSession,
                                                        ViInt32   reserved,
                                                        ViInt32   marker,
                                                        ViBoolean state);

ViStatus _VI_FUNC rsspecan_ConfMarkerPeakList(ViSession instrSession,
                                              ViInt32   marker,
                                              ViInt32   peakListSort,
                                              ViInt32   peakListSize,
                                              ViReal64  peakExcursion,
                                              ViBoolean displayMarkerNumbers);

ViStatus _VI_FUNC rsspecan_MarkerPeakListSearch(ViSession instrSession,
                                                ViInt32   marker,
                                                ViInt32   peakListCount);

ViStatus _VI_FUNC rsspecan_StoreMarkerPeakList(ViSession instrSession,
                                               ViString  path);

ViStatus _VI_FUNC rsspecan_QueryMarkerPeakListFound(ViSession instrSession,
                                                    ViInt32   reserved,
                                                    ViInt32   marker,
                                                    ViInt32*  peakListFound);

ViStatus _VI_FUNC rsspecan_QueryMarkerPeakList(ViSession instrSession,
                                               ViInt32   reserved,
                                               ViInt32   marker,
                                               ViInt32   arraySize,
                                               ViInt32   peakListSelection,
                                               ViReal64  peakList[]);

ViStatus _VI_FUNC rsspecan_ConfigureDeltaMarker(ViSession instrSession,
                                                ViInt32   window,
                                                ViInt32   deltaMarker,
                                                ViBoolean state,
                                                ViInt32   trace);

ViStatus _VI_FUNC rsspecan_ConfigureReferenceMarker(ViSession instrSession,
                                                    ViInt32   reserved,
                                                    ViInt32   marker,
                                                    ViInt32   referenceMarker);

ViStatus _VI_FUNC rsspecan_DisableAllDeltaMarkers(ViSession instrSession,
                                                  ViInt32   reserved);

ViStatus _VI_FUNC rsspecan_DeltaMarkerSearch(ViSession instrSession,
                                             ViInt32   reserved,
                                             ViInt32   deltaMarker,
                                             ViInt32   markerSearch);

ViStatus _VI_FUNC rsspecan_ConfigureDeltaMarkerPosition(ViSession instrSession,
                                                        ViInt32   window,
                                                        ViInt32   deltaMarker,
                                                        ViInt32   mode,
                                                        ViReal64  position);

ViStatus _VI_FUNC rsspecan_LinkingDeltaMarkerToMarker(ViSession instrSession,
                                                      ViInt32   deltaMarker,
                                                      ViInt32   secondMarker,
                                                      ViBoolean state);

ViStatus _VI_FUNC rsspecan_LinkingDeltaMarkerToAnotherDeltaMarker(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   deltaMarker,
                            ViInt32   secondMarker,
                            ViBoolean link);

ViStatus _VI_FUNC rsspecan_QueryDeltaMarker(ViSession instrSession,
                                            ViInt32   window,
                                            ViInt32   deltaMarker,
                                            ViInt32   mode,
                                            ViReal64* position,
                                            ViReal64* amplitude);

ViStatus _VI_FUNC rsspecan_ConfigureReferenceFixed(ViSession instrSession,
                                                   ViInt32   window,
                                                   ViBoolean fixedReference);

ViStatus _VI_FUNC rsspecan_ConfigureReferenceFixedPoint(ViSession instrSession,
                                                        ViInt32   window,
                                                        ViReal64  refPointLevel,
                                                        ViReal64  refPointLevelOffset,
                                                        ViReal64  refPointFrequencyTime);

ViStatus _VI_FUNC rsspecan_ReferenceFixedPeakSearch(ViSession instrSession,
                                                    ViInt32   reserved);

ViStatus _VI_FUNC rsspecan_ConfigureDeltaMarkerBandPower(ViSession instrSession,
                                                         ViBoolean bandPowerEnabled,
                                                         ViInt32   displayMode,
                                                         ViReal64  span);

ViStatus _VI_FUNC rsspecan_QueryDeltaMarkerBandPower(ViSession instrSession,
                                                     ViReal64* result);

ViStatus _VI_FUNC rsspecan_ConfigureDisplayLine(ViSession instrSession,
                                                ViInt32   line,
                                                ViInt32   type,
                                                ViBoolean state,
                                                ViReal64  position);

ViStatus _VI_FUNC rsspecan_CreateLimitLine(ViSession instrSession,
                                           ViInt32   reserved,
                                           ViInt32   limit,
                                           ViString  name,
                                           ViInt32   domain,
                                           ViString  comment,
                                           ViInt32   assignToTrace,
                                           ViBoolean deleteExistingLimitLine);

ViStatus _VI_FUNC rsspecan_ConfigureLimitLine(ViSession instrSession,
                                              ViInt32   limit,
                                              ViInt32   type,
                                              ViInt32   units,
                                              ViInt32   xAxisInterpolation,
                                              ViInt32   yAxisInterpolation,
                                              ViInt32   xAxisScaling,
                                              ViReal64  margin,
                                              ViReal64  threshold);

ViStatus _VI_FUNC rsspecan_DefineLimitLine(ViSession instrSession,
                                           ViInt32   limit,
                                           ViInt32   type,
                                           ViInt32   count,
                                           ViReal64  xAxis[],
                                           ViReal64  amplitude[]);

ViStatus _VI_FUNC rsspecan_ConfigureLimitLineCheckTrace(ViSession instrSession,
                                                        ViInt32   limit,
                                                        ViInt32   trace,
                                                        ViBoolean checkTrace);

ViStatus _VI_FUNC rsspecan_SelectLimitLine(ViSession instrSession,
                                           ViInt32   reserved,
                                           ViInt32   limit,
                                           ViString  name,
                                           ViBoolean limitEnabled);

ViStatus _VI_FUNC rsspecan_EnableLimitCheck(ViSession instrSession,
                                            ViInt32   reserved,
                                            ViInt32   limit,
                                            ViInt32   type,
                                            ViBoolean limitEnabled,
                                            ViBoolean checkEnabled);

ViStatus _VI_FUNC rsspecan_MoveLimitLine(ViSession instrSession,
                                         ViInt32   limit,
                                         ViInt32   type,
                                         ViInt32   method,
                                         ViReal64  offsetValue);

ViStatus _VI_FUNC rsspecan_CopyLimitLine(ViSession instrSession,
                                         ViInt32   limit,
                                         ViInt32   copyTo);

ViStatus _VI_FUNC rsspecan_DeleteLimitLine(ViSession instrSession,
                                           ViInt32   limit);

ViStatus _VI_FUNC rsspecan_ClearLimitLineResults(ViSession instrSession,
                                                 ViInt32   reserved,
                                                 ViInt32   limit);

ViStatus _VI_FUNC rsspecan_GetActiveLimitLines(ViSession instrSession,
                                               ViInt32   stringSize,
                                               ViChar    activeLimitLines[],
                                               ViInt32*  returnedStringLength);

ViStatus _VI_FUNC rsspecan_ImportLimit(ViSession instrSession,
                                       ViString  fileName);

ViStatus _VI_FUNC rsspecan_ExportLimit(ViSession instrSession,
                                       ViString  fileName,
                                       ViString  limitName);

ViStatus _VI_FUNC rsspecan_SpeakerVolume(ViSession instrSession,
                                         ViReal64  speakerVolume);

ViStatus _VI_FUNC rsspecan_ApplicationSetupRecovery(ViSession instrSession,
                                                    ViBoolean applicationSetupRecovery);

ViStatus _VI_FUNC rsspecan_ConfigureReferenceOscillator(ViSession instrSession,
                                                        ViInt32   source,
                                                        ViReal64  externalFrequency);

ViStatus _VI_FUNC rsspecan_QuerySelectedReferenceOscillatorSource(
                            ViSession instrSession,
                            ViInt32*  source);

ViStatus _VI_FUNC rsspecan_ConfigureMissingExternalReferenceBehavior(
                            ViSession instrSession,
                            ViInt32   missingExtReferenceBehavior);

ViStatus _VI_FUNC rsspecan_ConfigureExternalReferenceFrequency(
                            ViSession instrSession,
                            ViInt32   tuningRange,
                            ViReal64  frequency,
                            ViReal64  loopBandwidth);

ViStatus _VI_FUNC rsspecan_ConfigureAdvancedReferenceFrequencyInput(
                            ViSession instrSession,
                            ViReal64  frequency);

ViStatus _VI_FUNC rsspecan_ConfigureAdvancedExternalReferenceCoupling(
                            ViSession instrSession,
                            ViInt32   coupling,
                            ViInt32   bandwidth,
                            ViReal64  bandwidthValue);

ViStatus _VI_FUNC rsspecan_ConfigureReferenceOscillatorOutputs(
                            ViSession instrSession,
                            ViBoolean output100MHzEnabled,
                            ViBoolean syncTriggerOutputEnabled,
                            ViBoolean output640MHzEnabled);

ViStatus _VI_FUNC rsspecan_SelectTransducerFactor(ViSession instrSession,
                                                  ViInt32   reserved,
                                                  ViBoolean transducerState,
                                                  ViString  transducerName);

ViStatus _VI_FUNC rsspecan_ConfTransducerFactor(ViSession instrSession,
                                                ViString  name,
                                                ViInt32   unit,
                                                ViInt32   interpolation,
                                                ViString  comment,
                                                ViBoolean displayState,
                                                ViBoolean refLevAdj);

ViStatus _VI_FUNC rsspecan_DefineTransducerFactor(ViSession instrSession,
                                                  ViString  name,
                                                  ViInt32   noOfTestPoints,
                                                  ViReal64  frequencyData[],
                                                  ViReal64  levelData[]);

ViStatus _VI_FUNC rsspecan_GetTransducerFactorCatalog(ViSession instrSession,
                                                      ViInt32*  numberOfTransducerFactors,
                                                      ViInt32   bufferSize,
                                                      ViChar    transducerFactorsList[]);

ViStatus _VI_FUNC rsspecan_DeleteTransducerFactor(ViSession instrSession,
                                                  ViInt32   reserved,
                                                  ViString  transducerName);

ViStatus _VI_FUNC rsspecan_SelectTransducerSet(ViSession instrSession,
                                               ViString  transducerSetName,
                                               ViBoolean transducerSetState);

ViStatus _VI_FUNC rsspecan_ConfigureTransducerSet(ViSession instrSession,
                                                  ViString  transducerSetName,
                                                  ViString  comment,
                                                  ViInt32   unit,
                                                  ViBoolean transducerBreak,
                                                  ViInt32   range,
                                                  ViReal64  startFrequency,
                                                  ViReal64  stopFrequency,
                                                  ViString  factors);

ViStatus _VI_FUNC rsspecan_QueryTransducerSetCatalog(ViSession instrSession,
                                                     ViInt32*  numberOfTransducerSets,
                                                     ViInt32   bufferSize,
                                                     ViChar    transducerSetsList[]);

ViStatus _VI_FUNC rsspecan_DeleteTransducerSet(ViSession instrSession,
                                               ViString  transducerSetName);

ViStatus _VI_FUNC rsspecan_QueryActiveTransducerSet(ViSession instrSession,
                                                    ViChar    activeTransducerSet[]);

ViStatus _VI_FUNC rsspecan_ImportTransducerFactor(ViSession instrSession,
                                                  ViString  fileName);

ViStatus _VI_FUNC rsspecan_ExportTransducerFactor(ViSession instrSession,
                                                  ViString  fileName,
                                                  ViString  factorName);

ViStatus _VI_FUNC rsspecan_CalibrationSync(ViSession instrSession,
                                           ViUInt32  timeoutms,
                                           ViInt32*  result);

ViStatus _VI_FUNC rsspecan_CalibrationAbort(ViSession instrSession);

ViStatus _VI_FUNC rsspecan_CalibrationResult(ViSession instrSession,
                                             ViInt32   arraySize,
                                             ViChar    result[]);

ViStatus _VI_FUNC rsspecan_CalibrationState(ViSession instrSession,
                                            ViBoolean state);

ViStatus _VI_FUNC rsspecan_ConfigureCalibrationScheduleSettings(
                            ViSession instrSession,
                            ViBoolean calibrationSchedule,
                            ViBoolean warmup,
                            ViBoolean shutdown);

ViStatus _VI_FUNC rsspecan_ConfigureCalibrationSchedule(ViSession instrSession,
                                                        ViInt32   day1,
                                                        ViInt32   day2,
                                                        ViInt32   day3,
                                                        ViInt32   day4,
                                                        ViInt32   day5,
                                                        ViInt32   day6,
                                                        ViInt32   day7);

ViStatus _VI_FUNC rsspecan_ConfigureCalibrationScheduleTime(ViSession instrSession,
                                                            ViString  time);

ViStatus _VI_FUNC rsspecan_CalibrationPreselection(ViSession instrSession);

ViStatus _VI_FUNC rsspecan_ConfigureUserCorrection(ViSession instrSession,
                                                   ViBoolean userCorrectionEnabled,
                                                   ViBoolean frequencyResponseActive,
                                                   ViBoolean previewEnabled,
                                                   ViInt32   applyTo);

ViStatus _VI_FUNC rsspecan_QueryUserCorrectionIsValid(ViSession  instrSession,
                                                      ViBoolean* isValid);

ViStatus _VI_FUNC rsspecan_QueryUserCorrectionIQModeTraceValues(
                            ViSession instrSession,
                            ViInt32   selectedValues,
                            ViUInt32  arraySize,
                            ViReal64  data[],
                            ViUInt32* returnedValues);

ViStatus _VI_FUNC rsspecan_QueryUserCorrectionSpectrumModeTraceValues(
                            ViSession instrSession,
                            ViInt32   selectedValues,
                            ViUInt32  arraySize,
                            ViReal64  data[],
                            ViUInt32* returnedValues);

ViStatus _VI_FUNC rsspecan_ConfigureUserCorrectionFileAdjustReferenceLevel(
                            ViSession instrSession,
                            ViBoolean adjustReferenceLevel);

ViStatus _VI_FUNC rsspecan_ConfigureUserCorrectionAdjustReferenceLevel(
                            ViSession instrSession,
                            ViBoolean adjustReferenceLevel);

ViStatus _VI_FUNC rsspecan_QueryUserCorrectionFrequencyResponseSpanCover(
                            ViSession  instrSession,
                            ViBoolean* spanCover);

ViStatus _VI_FUNC rsspecan_ConfigureUserCorrectionBBEnabled(ViSession instrSession,
                                                            ViBoolean basebandEnabled);

ViStatus _VI_FUNC rsspecan_PresetUserCorrectionBB(ViSession instrSession);

ViStatus _VI_FUNC rsspecan_LoadUserCorrectionBB(ViSession instrSession,
                                                ViString  fileName);

ViStatus _VI_FUNC rsspecan_StoreUserCorrectionBB(ViSession instrSession,
                                                 ViString  fileName);

ViStatus _VI_FUNC rsspecan_ClearUserCorrectionBBTestSetup(ViSession instrSession);

ViStatus _VI_FUNC rsspecan_QueryUserCorrectionBBTestSetupCount(
                            ViSession instrSession,
                            ViInt32*  count);

ViStatus _VI_FUNC rsspecan_QueryUserCorrectionBBTestSetupCatalog(
                            ViSession instrSession,
                            ViUInt32  bufferSize,
                            ViChar    catalog[],
                            ViUInt32* returnedValues);

ViStatus _VI_FUNC rsspecan_ConfigureUserCorrectionBBTestSetup(
                            ViSession instrSession,
                            ViUInt32  index,
                            ViBoolean testSetupEnabled,
                            ViString  selected,
                            ViInt32   toPort,
                            ViInt32   fromPort);

ViStatus _VI_FUNC rsspecan_RemoveUserCorrectionBBTestSetup(ViSession instrSession,
                                                           ViUInt32  index);

ViStatus _VI_FUNC rsspecan_ClearUserCorrectionBBFRES(ViSession instrSession);

ViStatus _VI_FUNC rsspecan_QueryUserCorrectionBBFRESCount(ViSession instrSession,
                                                          ViInt32*  count);

ViStatus _VI_FUNC rsspecan_QueryUserCorrectionBBFRESCatalog(ViSession instrSession,
                                                            ViUInt32  bufferSize,
                                                            ViChar    catalog[],
                                                            ViUInt32* returnedValues);

ViStatus _VI_FUNC rsspecan_ConfigureUserCorrectionBBFRES(ViSession instrSession,
                                                         ViUInt32  index,
                                                         ViString  selected,
                                                         ViBoolean magnitudeEnabled,
                                                         ViBoolean phaseEnabled);

ViStatus _VI_FUNC rsspecan_RemoveUserCorrectionBBFRES(ViSession instrSession,
                                                      ViUInt32  index);

ViStatus _VI_FUNC rsspecan_ConfigureUserCorrectionBBFRESAdjustReferenceLevel(
                            ViSession instrSession,
                            ViBoolean adjustReferenceLevel);

ViStatus _VI_FUNC rsspecan_RefreshUserCorrectionBBFRES(ViSession instrSession);

ViStatus _VI_FUNC rsspecan_InsertCorrectionBBFRES(ViSession instrSession,
                                                  ViUInt32  index,
                                                  ViString  selected);

ViStatus _VI_FUNC rsspecan_InsertCorrectionInputFRES(ViSession instrSession,
                                                     ViUInt32  index,
                                                     ViString  selected);

ViStatus _VI_FUNC rsspecan_InsertCorrectionUserFRES(ViSession instrSession,
                                                    ViUInt32  index,
                                                    ViString  selected);

ViStatus _VI_FUNC rsspecan_InsertTouchstoneBBFRES(ViSession instrSession,
                                                  ViUInt32  index,
                                                  ViString  selected);

ViStatus _VI_FUNC rsspecan_InsertTouchstoneInputFRES(ViSession instrSession,
                                                     ViUInt32  index,
                                                     ViString  selected);

ViStatus _VI_FUNC rsspecan_InsertTouchstoneUserFRES(ViSession instrSession,
                                                    ViUInt32  index,
                                                    ViString  selected);

ViStatus _VI_FUNC rsspecan_MoveTouchstoneBBFRES(ViSession instrSession,
                                                ViUInt32  index,
                                                ViInt32   direction);

ViStatus _VI_FUNC rsspecan_MoveTouchstoneInputFRES(ViSession instrSession,
                                                   ViUInt32  index,
                                                   ViInt32   direction);

ViStatus _VI_FUNC rsspecan_MoveTouchstoneUserFRES(ViSession instrSession,
                                                  ViUInt32  index,
                                                  ViInt32   direction);

ViStatus _VI_FUNC rsspecan_ConfigureUserCorrectionRFInputEnabled(
                            ViSession instrSession,
                            ViBoolean RFInputEnabled);

ViStatus _VI_FUNC rsspecan_PresetUserCorrectionRFInput(ViSession instrSession);

ViStatus _VI_FUNC rsspecan_LoadUserCorrectionRFInput(ViSession instrSession,
                                                     ViString  fileName);

ViStatus _VI_FUNC rsspecan_StoreUserCorrectionRFInput(ViSession instrSession,
                                                      ViString  fileName);

ViStatus _VI_FUNC rsspecan_ConfigureUserCorrectionRFInputAdjustReferenceLevel(
                            ViSession instrSession,
                            ViBoolean adjustReferenceLevel);

ViStatus _VI_FUNC rsspecan_RefreshUserCorrectionRFInputFrequencyResponse(ViSession instrSession);

ViStatus _VI_FUNC rsspecan_ClearUserCorrectionRFInputTestSetup(ViSession instrSession);

ViStatus _VI_FUNC rsspecan_QueryUserCorrectionRFInputTestSetupCount(
                            ViSession instrSession,
                            ViInt32*  count);

ViStatus _VI_FUNC rsspecan_QueryUserCorrectionRFInputTestSetupCatalog(
                            ViSession instrSession,
                            ViUInt32  bufferSize,
                            ViChar    catalog[],
                            ViUInt32* returnedValues);

ViStatus _VI_FUNC rsspecan_ConfigureUserCorrectionRFInputTestSetup(
                            ViSession instrSession,
                            ViUInt32  index,
                            ViBoolean testSetupEnabled,
                            ViString  selected,
                            ViInt32   toPort,
                            ViInt32   fromPort);

ViStatus _VI_FUNC rsspecan_RemoveUserCorrectionRFInputTestSetup(
                            ViSession instrSession,
                            ViUInt32  index);

ViStatus _VI_FUNC rsspecan_ClearUserCorrectionRFInputFRES(ViSession instrSession);

ViStatus _VI_FUNC rsspecan_QueryUserCorrectionRFInputFRESCount(
                            ViSession instrSession,
                            ViInt32*  count);

ViStatus _VI_FUNC rsspecan_QueryUserCorrectionRFInputFRESCatalog(
                            ViSession instrSession,
                            ViUInt32  bufferSize,
                            ViChar    catalog[],
                            ViUInt32* returnedValues);

ViStatus _VI_FUNC rsspecan_ConfigureUserCorrectionRFInputFRES(
                            ViSession instrSession,
                            ViUInt32  index,
                            ViString  selected,
                            ViBoolean magnitudeEnabled,
                            ViBoolean phaseEnabled);

ViStatus _VI_FUNC rsspecan_RemoveUserCorrectionRFInputFRES(ViSession instrSession,
                                                           ViUInt32  index);

ViStatus _VI_FUNC rsspecan_PresetUserCorrectionAllInputs(ViSession instrSession);

ViStatus _VI_FUNC rsspecan_LoadUserCorrectionAllInputs(ViSession instrSession,
                                                       ViString  fileName);

ViStatus _VI_FUNC rsspecan_StoreUserCorrectionAllInputs(ViSession instrSession,
                                                        ViString  fileName);

ViStatus _VI_FUNC rsspecan_ClearUserCorrectionAllInputsTestSetup(ViSession instrSession);

ViStatus _VI_FUNC rsspecan_QueryUserCorrectionAllInputsTestSetupCount(
                            ViSession instrSession,
                            ViInt32*  count);

ViStatus _VI_FUNC rsspecan_QueryUserCorrectionAllInputsTestSetupCatalog(
                            ViSession instrSession,
                            ViUInt32  bufferSize,
                            ViChar    catalog[],
                            ViUInt32* returnedValues);

ViStatus _VI_FUNC rsspecan_ConfigureUserCorrectionAllInputsTestSetup(
                            ViSession instrSession,
                            ViUInt32  index,
                            ViBoolean testSetupEnabled,
                            ViString  selected,
                            ViInt32   toPort,
                            ViInt32   fromPort);

ViStatus _VI_FUNC rsspecan_QueryUserCorrectionAllInputsTestSetupTraceValues(
                            ViSession instrSession,
                            ViUInt32  index,
                            ViInt32   sPortPairIndex,
                            ViInt32   selectedValues,
                            ViUInt32  arraySize,
                            ViReal64  data[],
                            ViUInt32* returnedValues);

ViStatus _VI_FUNC rsspecan_RemoveUserCorrectionAllInputsTestSetup(
                            ViSession instrSession,
                            ViUInt32  index);

ViStatus _VI_FUNC rsspecan_ClearUserCorrectionAllInputsFRES(ViSession instrSession);

ViStatus _VI_FUNC rsspecan_QueryUserCorrectionAllInputsFRESCount(
                            ViSession instrSession,
                            ViInt32*  count);

ViStatus _VI_FUNC rsspecan_QueryUserCorrectionAllInputsFRESCatalog(
                            ViSession instrSession,
                            ViUInt32  bufferSize,
                            ViChar    catalog[],
                            ViUInt32* returnedValues);

ViStatus _VI_FUNC rsspecan_ConfigureUserCorrectionAllInputsFRES(
                            ViSession instrSession,
                            ViUInt32  index,
                            ViString  selected,
                            ViBoolean magnitudeEnabled,
                            ViBoolean phaseEnabled);

ViStatus _VI_FUNC rsspecan_QueryUserCorrectionAllInputsFRESTraceValues(
                            ViSession instrSession,
                            ViUInt32  index,
                            ViInt32   selectedValues,
                            ViUInt32  arraySize,
                            ViReal64  data[],
                            ViUInt32* returnedValues);

ViStatus _VI_FUNC rsspecan_RemoveUserCorrectionAllInputsFRES(ViSession instrSession,
                                                             ViUInt32  index);

ViStatus _VI_FUNC rsspecan_ConfigureSyncParameterCouplingEnabled(
                            ViSession instrSession,
                            ViInt32   parameter,
                            ViBoolean couplingEnabled);

ViStatus _VI_FUNC rsspecan_CreateSyncUserDefinedCoupling(ViSession instrSession,
                                                         ViString  channelName1,
                                                         ViString  window1,
                                                         ViString  parameter1,
                                                         ViString  channelName2,
                                                         ViString  window2,
                                                         ViString  parameter2,
                                                         ViInt32   direction,
                                                         ViBoolean enabled,
                                                         ViUInt32* index);

ViStatus _VI_FUNC rsspecan_EditSyncUserDefinedCoupling(ViSession instrSession,
                                                       ViUInt32  index,
                                                       ViString  channelName1,
                                                       ViString  window1,
                                                       ViString  parameter1,
                                                       ViString  channelName2,
                                                       ViString  window2,
                                                       ViString  parameter2,
                                                       ViInt32   direction,
                                                       ViBoolean enabled);

ViStatus _VI_FUNC rsspecan_ConfigureSyncUserDefinedParameterCouplingEnabled(
                            ViSession instrSession,
                            ViUInt32  index,
                            ViBoolean parameterCouplingEnabled);

ViStatus _VI_FUNC rsspecan_ConfigureSyncUserDefinedParameterCouplingDirection(
                            ViSession instrSession,
                            ViUInt32  index,
                            ViInt32   direction);

ViStatus _VI_FUNC rsspecan_QuerySyncUserDefinedParameterCouplingChannelList(
                            ViSession instrSession,
                            ViUInt32  bufferSize,
                            ViChar    list[],
                            ViUInt32* returnedValues);

ViStatus _VI_FUNC rsspecan_QuerySyncUserDefinedParameterCouplingParametersList(
                            ViSession instrSession,
                            ViString  channelName,
                            ViString  parameter,
                            ViUInt32  bufferSize,
                            ViChar    list[],
                            ViUInt32* returnedValues);

ViStatus _VI_FUNC rsspecan_QuerySyncUserDefinedParameterCouplingNumbersList(
                            ViSession instrSession,
                            ViUInt32  bufferSize,
                            ViChar    list[],
                            ViUInt32* returnedValues);

ViStatus _VI_FUNC rsspecan_SyncUserDefinedParameterCouplingWindowsList(
                            ViSession instrSession,
                            ViString  channelName,
                            ViString  parameter,
                            ViUInt32  bufferSize,
                            ViChar    list[],
                            ViUInt32* returnedValues);

ViStatus _VI_FUNC rsspecan_QuerySyncUserDefinedParameterCouplingInfo(
                            ViSession instrSession,
                            ViUInt32  index,
                            ViUInt32  bufferSize,
                            ViChar    info[],
                            ViUInt32* returnedValues);

ViStatus _VI_FUNC rsspecan_RemoveSyncUserDefinedParameterCoupling(
                            ViSession instrSession,
                            ViUInt32  index);

ViStatus _VI_FUNC rsspecan_RemoveAllSyncUserDefinedParameterCoupling(ViSession instrSession);

ViStatus _VI_FUNC rsspecan_ConfigureGeneratorCouplingEnabled(ViSession instrSession,
                                                             ViBoolean generatorCoupling);

ViStatus _VI_FUNC rsspecan_ConfigureGeneratorCouplingCenterFrequency(
                            ViSession instrSession,
                            ViBoolean enabled,
                            ViReal64  offset);

ViStatus _VI_FUNC rsspecan_ConfigureGeneratorCouplingReferenceLevelEnabled(
                            ViSession instrSession,
                            ViBoolean enabled,
                            ViReal64  offset);

ViStatus _VI_FUNC rsspecan_ServiceHWInfo(ViSession instrSession,
                                         ViInt32   arraySize,
                                         ViChar    HWInfo[]);

ViStatus _VI_FUNC rsspecan_ServiceVersionInfo(ViSession instrSession,
                                              ViInt32   arraySize,
                                              ViChar    versionInfo[]);

ViStatus _VI_FUNC rsspecan_QueryServiceLastDate(ViSession instrSession,
                                                ViChar    lastServiceDate[]);

ViStatus _VI_FUNC rsspecan_ConfigureCalibrationSignal(ViSession instrSession,
                                                      ViInt32   input);

ViStatus _VI_FUNC rsspecan_ConfigureCalibrationFreqRF(ViSession instrSession,
                                                      ViReal64  frequency,
                                                      ViInt32   RFSpectrum);

ViStatus _VI_FUNC rsspecan_ConfigureWidebandCalibrationFrequency(
                            ViSession instrSession,
                            ViReal64  frequency);

ViStatus _VI_FUNC rsspecan_ConfigureInternalWidebandCalibrationFrequency(
                            ViSession instrSession,
                            ViInt32   frequencyWB2);

ViStatus _VI_FUNC rsspecan_ConfigureCalibrationFreqMW(ViSession instrSession,
                                                      ViReal64  frequency,
                                                      ViInt32   distance);

ViStatus _VI_FUNC rsspecan_ConfigureCalibrationFreqMicrowave(ViSession instrSession,
                                                             ViReal64  frequency,
                                                             ViInt32   distance);

ViStatus _VI_FUNC rsspecan_ConfigureCalibrationLastDate(ViSession instrSession,
                                                        ViString  lastCalibrationDate);

ViStatus _VI_FUNC rsspecan_ConfigureCalibrationNextDate(ViSession instrSession,
                                                        ViString  nextCalibrationDate);

ViStatus _VI_FUNC rsspecan_QueryCalibrationInterval(ViSession instrSession,
                                                    ViChar    calibrationInterval[]);

ViStatus _VI_FUNC rsspecan_SystemShutdown(ViSession instrSession);

ViStatus _VI_FUNC rsspecan_SystemReboot(ViSession instrSession);

ViStatus _VI_FUNC rsspecan_ConfigurePresetOperatingMode(ViSession instrSession,
                                                        ViInt32   presetOperatingMode);

ViStatus _VI_FUNC rsspecan_ConfigurePresetFilter(ViSession instrSession,
                                                 ViInt32   filter);

ViStatus _VI_FUNC rsspecan_ConfigureSecureUserMode(ViSession instrSession,
                                                   ViBoolean secureUserModeEnabled);

ViStatus _VI_FUNC rsspecan_SystemTreeWalking(ViSession instrSession,
                                             ViInt32   mode);

ViStatus _VI_FUNC rsspecan_QuerySystemInfoDeviceFootprint(ViSession instrSession,
                                                          ViUInt32  bufferSize,
                                                          ViChar    deviceFootprint[],
                                                          ViUInt32* returnedValues);

ViStatus _VI_FUNC rsspecan_QuerySystemInfoSwitchingCyclesCount(
                            ViSession instrSession,
                            ViInt32   relay,
                            ViInt32*  cyclesCount);

ViStatus _VI_FUNC rsspecan_QuerySystemInfoBIOSVersion(ViSession instrSession,
                                                      ViUInt32  bufferSize,
                                                      ViChar    BIOSVersion[],
                                                      ViUInt32* returnedValues);

ViStatus _VI_FUNC rsspecan_QuerySystemInfoFrequencyBands(ViSession instrSession,
                                                         ViUInt32  bufferSize,
                                                         ViChar    frequencyBands[],
                                                         ViUInt32* returnedValues);

ViStatus _VI_FUNC rsspecan_ConfigureSystemFloatingOptionEnabled(
                            ViSession instrSession,
                            ViString  option,
                            ViInt32   state,
                            ViInt32   days);

ViStatus _VI_FUNC rsspecan_QuerySystemFloatingLicense(ViSession instrSession,
                                                      ViInt32   bufferSize,
                                                      ViString  option,
                                                      ViChar    response[],
                                                      ViInt32*  bytesRead);

ViStatus _VI_FUNC rsspecan_ConfigureSystemDisplayLock(ViSession instrSession,
                                                      ViBoolean lockEnabled);

ViStatus _VI_FUNC rsspecan_ConfigureSystemChangingHardwareEnabled(
                            ViSession instrSession,
                            ViBoolean HWEnabled);

ViStatus _VI_FUNC rsspecan_ConfigureSystemChangingHardware(ViSession instrSession);

ViStatus _VI_FUNC rsspecan_ConfigureSystemTrialOptionsEnabled(
                            ViSession instrSession,
                            ViBoolean enabled);

ViStatus _VI_FUNC rsspecan_ConfigureSystemTrialOptions(ViSession instrSession,
                                                       ViInt32   bufferSize,
                                                       ViChar    trialOptions[]);

ViStatus _VI_FUNC rsspecan_ConfigureSystemSCPIRecorderAutoRecordingEnabled(
                            ViSession instrSession,
                            ViBoolean autoRecording);

ViStatus _VI_FUNC rsspecan_ConfigureSystemSCPIRecorderAddSyncCommands(
                            ViSession instrSession,
                            ViBoolean addSyncCommands);

ViStatus _VI_FUNC rsspecan_QuerySystemSCPIRecorderListRecording(
                            ViSession instrSession,
                            ViInt32   bufferSize,
                            ViChar    record[],
                            ViInt32*  bytesRead);

ViStatus _VI_FUNC rsspecan_ExportSystemSCPIRecorder(ViSession instrSession,
                                                    ViInt32   format,
                                                    ViString  filename);

ViStatus _VI_FUNC rsspecan_SystemSCPIRecorderClearAll(ViSession instrSession);

ViStatus _VI_FUNC rsspecan_ConfigureDisplay(ViSession instrSession,
                                            ViBoolean statusBarState,
                                            ViBoolean softkeyBarState,
                                            ViBoolean toolBarState,
                                            ViInt32   touchscreenState,
                                            ViBoolean channelInformation,
                                            ViBoolean frequency,
                                            ViBoolean time,
                                            ViBoolean softFrontpanel);

ViStatus _VI_FUNC rsspecan_ConfigureDisplayPowerSave(ViSession instrSession,
                                                     ViBoolean state,
                                                     ViInt32   holdoff);

ViStatus _VI_FUNC rsspecan_ConfigureDisplayThemeSelect(ViSession instrSession,
                                                       ViString  theme);

ViStatus _VI_FUNC rsspecan_GetDisplayThemeCatalog(ViSession instrSession,
                                                  ViInt32*  numberOfThemes,
                                                  ViInt32   bufferSize,
                                                  ViChar    themeList[]);

ViStatus _VI_FUNC rsspecan_ConfigureDisplayFocusedArea(ViSession instrSession,
                                                       ViInt32   window,
                                                       ViInt32   subwindow);

ViStatus _VI_FUNC rsspecan_ConfigureDisplayComment(ViSession instrSession,
                                                   ViInt32   reserved,
                                                   ViBoolean state,
                                                   ViString  comment);

ViStatus _VI_FUNC rsspecan_ConfigureDisplayFormat(ViSession instrSession,
                                                  ViInt32   format);

ViStatus _VI_FUNC rsspecan_ConfigureDisplayWindowSize(ViSession instrSession,
                                                      ViInt32   windowSize);

ViStatus _VI_FUNC rsspecan_ConfigureDisplaySingleSweep(ViSession instrSession,
                                                       ViBoolean state);

ViStatus _VI_FUNC rsspecan_ConfigureDisplayUpdate(ViSession instrSession,
                                                  ViBoolean displayInRemote);

ViStatus _VI_FUNC rsspecan_ConfigureDisplayMeasurementWindowState(
                            ViSession instrSession,
                            ViInt32   window,
                            ViBoolean state);

ViStatus _VI_FUNC rsspecan_DisplaySetActiveWindow(ViSession instrSession,
                                                  ViInt32   window);

ViStatus _VI_FUNC rsspecan_ConfigureDisplayMultiViewTabEnabled(
                            ViSession instrSession,
                            ViBoolean multiViewTabEnabled);

ViStatus _VI_FUNC rsspecan_ConfigureDisplayTimeFormat(ViSession instrSession,
                                                      ViInt32   displayTimeFormat);

ViStatus _VI_FUNC rsspecan_ConfigureDisplaySubwindowZoomState(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   subwindow,
                            ViBoolean state);

ViStatus _VI_FUNC rsspecan_ConfigureDisplaySubwindowZoom(ViSession instrSession,
                                                         ViInt32   window,
                                                         ViInt32   subwindow,
                                                         ViBoolean enableZoom,
                                                         ViReal64  x1,
                                                         ViReal64  y1,
                                                         ViReal64  x2,
                                                         ViReal64  y2);

ViStatus _VI_FUNC rsspecan_ConfigureDisplaySubwindowAmplitudeGridMode(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   subwindow,
                            ViInt32   gridMode);

ViStatus _VI_FUNC rsspecan_ConfigureDisplaySubwindowReferenceLevel(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   subwindow,
                            ViReal64  referenceLevel);

ViStatus _VI_FUNC rsspecan_AddWindow(ViSession instrSession,
                                     ViInt32   stringSize,
                                     ViString  referenceWindowName,
                                     ViInt32   direction,
                                     ViInt32   windowType,
                                     ViChar    returnedWindowName[]);

ViStatus _VI_FUNC rsspecan_RemoveWindow(ViSession instrSession,
                                        ViString  windowName);

ViStatus _VI_FUNC rsspecan_ReplaceWindow(ViSession instrSession,
                                         ViString  windowName,
                                         ViInt32   windowType);

ViStatus _VI_FUNC rsspecan_MoveWindow(ViSession instrSession,
                                      ViString  windowName1,
                                      ViString  windowName2,
                                      ViInt32   direction);

ViStatus _VI_FUNC rsspecan_QueryWindowsList(ViSession instrSession,
                                            ViInt32   bufferSize,
                                            ViChar    results[]);

ViStatus _VI_FUNC rsspecan_QueryWindowIndex(ViSession instrSession,
                                            ViString  windowName,
                                            ViInt32*  index);

ViStatus _VI_FUNC rsspecan_QueryWindowName(ViSession instrSession,
                                           ViInt32   stringSize,
                                           ViInt32   index,
                                           ViChar    windowName[]);

ViStatus _VI_FUNC rsspecan_ConfigureSplitterPosition(ViSession instrSession,
                                                     ViString  firstWindowName,
                                                     ViString  secondWindowName,
                                                     ViInt32   position);

ViStatus _VI_FUNC rsspecan_DisplayColorDefault(ViSession instrSession,
                                               ViInt32   defaultSetting);

ViStatus _VI_FUNC rsspecan_ConfigureDisplayColor(ViSession instrSession,
                                                 ViInt32   colorMap,
                                                 ViReal64  TINT,
                                                 ViReal64  SATURATION,
                                                 ViReal64  BRIGHTNESS);

ViStatus _VI_FUNC rsspecan_ConfigureDisplayColorByPredefined(ViSession instrSession,
                                                             ViInt32   colorMap,
                                                             ViInt32   predefinedColors);

ViStatus _VI_FUNC rsspecan_HardcopyColor(ViSession instrSession,
                                         ViBoolean color,
                                         ViInt32   defaultSetting);

ViStatus _VI_FUNC rsspecan_ConfigureHardcopyColor(ViSession instrSession,
                                                  ViInt32   colorMap,
                                                  ViReal64  TINT,
                                                  ViReal64  SATURATION,
                                                  ViReal64  BRIGHTNESS);

ViStatus _VI_FUNC rsspecan_ConfigureHardcopyColorByPredefined(
                            ViSession instrSession,
                            ViInt32   colorMap,
                            ViInt32   predefinedColors);

ViStatus _VI_FUNC rsspecan_AddApplication(ViSession instrSession,
                                          ViInt32   applicationGroup,
                                          ViString  name,
                                          ViString  filePath);

ViStatus _VI_FUNC rsspecan_DeleteApplication(ViSession instrSession,
                                             ViInt32   applicationGroup,
                                             ViString  name);

ViStatus _VI_FUNC rsspecan_SelectApplication(ViSession instrSession,
                                             ViInt32   applicationGroup,
                                             ViString  name);

ViStatus _VI_FUNC rsspecan_ExecuteApplication(ViSession instrSession,
                                              ViInt32   applicationGroup,
                                              ViString  name);

ViStatus _VI_FUNC rsspecan_ConfigureApplicationWorkingDirectory(
                            ViSession instrSession,
                            ViString  directoryPath);

ViStatus _VI_FUNC rsspecan_ConfigureApplicationIcon(ViSession instrSession,
                                                    ViString  iconPath,
                                                    ViString  index);

ViStatus _VI_FUNC rsspecan_ConfigureApplicationNewPath(ViSession instrSession,
                                                       ViString  path);

ViStatus _VI_FUNC rsspecan_ConfigureApplicationName(ViSession instrSession,
                                                    ViString  name);

ViStatus _VI_FUNC rsspecan_ConfigureApplicationExecutionParameters(
                            ViSession instrSession,
                            ViString  applicationStarterParameters);

ViStatus _VI_FUNC rsspecan_ConfigureRemoteModeEnabled(ViSession instrSession,
                                                      ViBoolean remoteMode);

ViStatus _VI_FUNC rsspecan_ConfigureRemoteLoggingState(ViSession instrSession,
                                                       ViBoolean state);

ViStatus _VI_FUNC rsspecan_ConfigureGPIBSettings(ViSession instrSession,
                                                 ViInt32   address,
                                                 ViInt32   terminator);

ViStatus _VI_FUNC rsspecan_ConfigureLANSettings(ViSession instrSession,
                                                ViString  instrumentDescription,
                                                ViString  password);

ViStatus _VI_FUNC rsspecan_ResetLANSettings(ViSession instrSession);

ViStatus _VI_FUNC rsspecan_ConfigureHUMSGeneralSettings(ViSession instrSession,
                                                        ViBoolean enabled,
                                                        ViInt32   dataFormat);

ViStatus _VI_FUNC rsspecan_DeleteHUMSAllData(ViSession instrSession);

ViStatus _VI_FUNC rsspecan_QueryHUMSTagCatalog(ViSession instrSession,
                                               ViInt32   bufferSize,
                                               ViChar    tagCatalog[],
                                               ViInt32*  bytesRead);

ViStatus _VI_FUNC rsspecan_AddHUMSTag(ViSession instrSession,
                                      ViInt32   ID,
                                      ViString  key,
                                      ViString  value);

ViStatus _VI_FUNC rsspecan_DeleteHUMSTag(ViSession instrSession,
                                         ViInt32   tagID);

ViStatus _VI_FUNC rsspecan_DeleteAllHUMSTags(ViSession instrSession);

ViStatus _VI_FUNC rsspecan_QueryHUMSTagByID(ViSession instrSession,
                                            ViInt32   ID,
                                            ViInt32   bufferSize,
                                            ViChar    keyValuePair[],
                                            ViInt32*  returnedBytes);

ViStatus _VI_FUNC rsspecan_ConfigureHUMSSNMPVersion(ViSession instrSession,
                                                    ViInt32   version);

ViStatus _VI_FUNC rsspecan_QueryHUMSRESTState(ViSession  instrSession,
                                              ViBoolean* state);

ViStatus _VI_FUNC rsspecan_ConfigureHUMSSNMPAdminInformation(ViSession instrSession,
                                                             ViString  contact,
                                                             ViString  location);

ViStatus _VI_FUNC rsspecan_ConfigureHUMSSNMPCommunity(ViSession instrSession,
                                                      ViInt32   access,
                                                      ViString  communityString);

ViStatus _VI_FUNC rsspecan_AddHUMSSNMPv3User(ViSession instrSession,
                                             ViString  user,
                                             ViString  authenticationPassword,
                                             ViString  privacyPassword);

ViStatus _VI_FUNC rsspecan_DeleteHUMSSNMPUser(ViSession instrSession,
                                              ViString  user);

ViStatus _VI_FUNC rsspecan_DeleteAllHUMSSNMPUsers(ViSession instrSession);

ViStatus _VI_FUNC rsspecan_QueryHUMSSNMPv3UserList(ViSession instrSession,
                                                   ViInt32   bufferSize,
                                                   ViChar    userList[]);

ViStatus _VI_FUNC rsspecan_ReadYTrace(ViSession instrSession,
                                      ViInt32   window,
                                      ViInt32   trace,
                                      ViUInt32  maximumTime,
                                      ViInt32   arrayLength,
                                      ViInt32*  actualPoints,
                                      ViReal64  amplitude[]);

ViStatus _VI_FUNC rsspecan_FetchYTrace(ViSession instrSession,
                                       ViInt32   window,
                                       ViInt32   trace,
                                       ViInt32   arrayLength,
                                       ViInt32*  actualPoints,
                                       ViReal64  amplitude[]);

ViStatus _VI_FUNC rsspecan_ReadYTracePrevious(ViSession instrSession,
                                              ViInt32   window,
                                              ViInt32   trace,
                                              ViInt32   offsetSweepPoint,
                                              ViInt32   numberOfSweeps,
                                              ViInt32   arrayLength,
                                              ViInt32*  actualPoints,
                                              ViReal64  amplitude[]);

ViStatus _VI_FUNC rsspecan_FetchXTrace(ViSession instrSession,
                                       ViInt32   window,
                                       ViInt32   trace,
                                       ViInt32   arrayLength,
                                       ViInt32*  actualPoints,
                                       ViReal64  x[]);

ViStatus _VI_FUNC rsspecan_GetLimitCheckResult(ViSession instrSession,
                                               ViInt32   reserved,
                                               ViInt32   limit,
                                               ViInt32*  state);

ViStatus _VI_FUNC rsspecan_Initiate(ViSession instrSession,
                                    ViInt32   reserved,
                                    ViUInt32  timeout);

ViStatus _VI_FUNC rsspecan_InitiateSpurious(ViSession instrSession,
                                            ViInt32   reserved,
                                            ViUInt32  timeout);

ViStatus _VI_FUNC rsspecan_InitiateSEMMeasurement(ViSession instrSession,
                                                  ViInt32   reserved,
                                                  ViUInt32  timeout);

ViStatus _VI_FUNC rsspecan_Abort(ViSession instrSession);

ViStatus _VI_FUNC rsspecan_Continue(ViSession instrSession,
                                    ViInt32   reserved,
                                    ViUInt32  timeout);

ViStatus _VI_FUNC rsspecan_SendSoftwareTrigger(ViSession instrSession,
                                               ViUInt32  timeout);

ViStatus _VI_FUNC rsspecan_GetSweepNumber(ViSession instrSession,
                                          ViInt32   reserved,
                                          ViInt32*  numberOfSweeps);

ViStatus _VI_FUNC rsspecan_InitiateSequencer(ViSession instrSession,
                                             ViUInt32  timeout);

ViStatus _VI_FUNC rsspecan_AbortSequencer(ViSession instrSession);

ViStatus _VI_FUNC rsspecan_RefreshSequencer(ViSession instrSession);

ViStatus _VI_FUNC rsspecan_ConfigureSequencerState(ViSession instrSession,
                                                   ViBoolean state);

ViStatus _VI_FUNC rsspecan_ConfigureSequencerMode(ViSession instrSession,
                                                  ViInt32   sequencerMode);

ViStatus _VI_FUNC rsspecan_CalculateSequencerTimeTrigger(ViSession instrSession);

ViStatus _VI_FUNC rsspecan_SelectPowerMeasurement(ViSession instrSession,
                                                  ViInt32   reserved,
                                                  ViBoolean state,
                                                  ViInt32   powerMeasurement);

ViStatus _VI_FUNC rsspecan_PowerPresetMeasurement(ViSession instrSession,
                                                  ViInt32   reserved,
                                                  ViInt32   channelPowerType,
                                                  ViUInt32  timeout);

ViStatus _VI_FUNC rsspecan_QueryPowerResults(ViSession instrSession,
                                             ViInt32   reserved,
                                             ViInt32   powerMeasurement,
                                             ViInt32   arraySize,
                                             ViReal64  results[],
                                             ViInt32*  returnedValues);

ViStatus _VI_FUNC rsspecan_ConfigureListPowerSequence(ViSession instrSession,
                                                      ViInt32   reserved,
                                                      ViInt32   noOfListItems,
                                                      ViReal64  analyzerFrequency[],
                                                      ViReal64  referenceLevel[],
                                                      ViReal64  RFInputAttenuation[],
                                                      ViReal64  RFInputElectronicAttn[],
                                                      ViInt32   filterType[],
                                                      ViReal64  resolutionBandwidth[],
                                                      ViReal64  videoBandwidth[],
                                                      ViReal64  measTime[],
                                                      ViReal64  triggerLevel[]);

ViStatus _VI_FUNC rsspecan_ConfigureListPowerSet(ViSession instrSession,
                                                 ViInt32   reserved,
                                                 ViBoolean peakMeas,
                                                 ViBoolean RMSMeas,
                                                 ViBoolean AVGMeas,
                                                 ViInt32   triggerMode,
                                                 ViInt32   triggerSlope,
                                                 ViReal64  triggerOffset,
                                                 ViReal64  gateLength);

ViStatus _VI_FUNC rsspecan_ListPowerMeasurementOff(ViSession instrSession,
                                                   ViInt32   reserved);

ViStatus _VI_FUNC rsspecan_QueryListPowerSequence(ViSession instrSession,
                                                  ViInt32   reserved,
                                                  ViInt32   noOfListItems,
                                                  ViReal64  analyzerFrequency[],
                                                  ViReal64  referenceLevel[],
                                                  ViReal64  RFInputAttenuation[],
                                                  ViReal64  RFInputElectronicAttn[],
                                                  ViInt32   filterType[],
                                                  ViReal64  resolutionBandwidth[],
                                                  ViReal64  videoBandwidth[],
                                                  ViReal64  measTime[],
                                                  ViReal64  triggerLevel[],
                                                  ViReal64  listPowerResults[]);

ViStatus _VI_FUNC rsspecan_QueryListPowerResult(ViSession instrSession,
                                                ViInt32   reserved,
                                                ViInt32   noOfResults,
                                                ViReal64  listPowerResults[],
                                                ViInt32*  returnedValues);

ViStatus _VI_FUNC rsspecan_ConfigureChannelPowerStandard(ViSession instrSession,
                                                         ViInt32   reserved,
                                                         ViInt32   ACPStandard);

ViStatus _VI_FUNC rsspecan_ConfigureChannelPowerUserStandard(ViSession instrSession,
                                                             ViInt32   reserved,
                                                             ViInt32   operation,
                                                             ViString  userStandardName);

ViStatus _VI_FUNC rsspecan_ConfigureChannelPowerMeasurement(ViSession instrSession,
                                                            ViInt32   noOfTXChannels,
                                                            ViInt32   noOfAdjChannels,
                                                            ViInt32   channelMode,
                                                            ViInt32   powerMode,
                                                            ViBoolean powerUnit,
                                                            ViInt32   noiseCorrection,
                                                            ViInt32   traceNumber);

ViStatus _VI_FUNC rsspecan_ConfigurePowerUnit(ViSession instrSession,
                                              ViInt32   powerUnit);

ViStatus _VI_FUNC rsspecan_ConfigureACPFastACPState(ViSession instrSession,
                                                    ViInt32   reserved,
                                                    ViBoolean fastACPState);

ViStatus _VI_FUNC rsspecan_ConfigureACPReferenceChannelAuto(ViSession instrSession,
                                                            ViInt32   reserved,
                                                            ViInt32   reference);

ViStatus _VI_FUNC rsspecan_ConfigureACPReferenceChannelMan(ViSession instrSession,
                                                           ViInt32   reserved,
                                                           ViInt32   channel);

ViStatus _VI_FUNC rsspecan_PowerAdjustReferenceLevel(ViSession instrSession,
                                                     ViInt32   reserved,
                                                     ViUInt32  timeout);

ViStatus _VI_FUNC rsspecan_ACPChannelPowerAutoAdjust(ViSession instrSession,
                                                     ViInt32   reserved,
                                                     ViUInt32  timeout);

ViStatus _VI_FUNC rsspecan_QueryChannelPowerStandardCatalog(ViSession instrSession,
                                                            ViInt32   bufferSize,
                                                            ViChar    powerStandardCatalog[]);

ViStatus _VI_FUNC rsspecan_ConfigurePowerChannelBandwidth(ViSession instrSession,
                                                          ViInt32   reserved,
                                                          ViInt32   channel,
                                                          ViInt32   channelNumber,
                                                          ViReal64  bandwidth);

ViStatus _VI_FUNC rsspecan_ConfigurePowerChannelSpacing(ViSession instrSession,
                                                        ViInt32   reserved,
                                                        ViInt32   channel,
                                                        ViInt32   channelNumber,
                                                        ViReal64  spacing);

ViStatus _VI_FUNC rsspecan_ConfPowerChannelWeightingFilters(ViSession instrSession,
                                                            ViInt32   filterType,
                                                            ViInt32   channelNumber,
                                                            ViBoolean filterState,
                                                            ViReal64  filterValue);

ViStatus _VI_FUNC rsspecan_ConfigurePowerChannelName(ViSession instrSession,
                                                     ViInt32   reserved,
                                                     ViInt32   channel,
                                                     ViInt32   channelNumber,
                                                     ViString  name);

ViStatus _VI_FUNC rsspecan_ConfigureACPLimitState(ViSession instrSession,
                                                  ViInt32   reserved,
                                                  ViBoolean state);

ViStatus _VI_FUNC rsspecan_ConfigureACPAdjacentChannelLimit(ViSession instrSession,
                                                            ViInt32   reserved,
                                                            ViBoolean relativeState,
                                                            ViReal64  relativeValue,
                                                            ViBoolean absoluteState,
                                                            ViReal64  absoluteValue);

ViStatus _VI_FUNC rsspecan_ConfigureACPAlternateChannelLimit(ViSession instrSession,
                                                             ViInt32   reserved,
                                                             ViInt32   alternateChannelNumber,
                                                             ViBoolean relativeState,
                                                             ViReal64  relativeValue,
                                                             ViBoolean absoluteState,
                                                             ViReal64  absoluteValue);

ViStatus _VI_FUNC rsspecan_QueryACPAdjacentChannelLimitCheckResults(
                            ViSession instrSession,
                            ViInt32   reserved,
                            ViInt32*  lowerResult,
                            ViInt32*  upperResult);

ViStatus _VI_FUNC rsspecan_QueryACPAlternateChannelLimitCheckResults(
                            ViSession instrSession,
                            ViInt32   reserved,
                            ViInt32   alternateChannelNumber,
                            ViInt32*  lowerResult,
                            ViInt32*  upperResult);

ViStatus _VI_FUNC rsspecan_ConfigureMSRPowerMeasurement(ViSession instrSession,
                                                        ViInt32   ACLRMode,
                                                        ViInt32   powerMode,
                                                        ViBoolean powerUnit,
                                                        ViInt32   noiseCorrection,
                                                        ViInt32   traceNumber);

ViStatus _VI_FUNC rsspecan_QueryMSRGapName(ViSession instrSession,
                                           ViInt32   gap,
                                           ViInt32   arraySize,
                                           ViChar    gapName[]);

ViStatus _VI_FUNC rsspecan_ConfigureMSRNumberOfSubblocks(ViSession instrSession,
                                                         ViInt32   numberOfSubblocks);

ViStatus _VI_FUNC rsspecan_ConfigureMSRSymmetricalAdjacentSetup(
                            ViSession instrSession,
                            ViBoolean symmetricalAdjacentSetup);

ViStatus _VI_FUNC rsspecan_ConfigureMSRSubblockDefinition(ViSession instrSession,
                                                          ViInt32   subblock,
                                                          ViReal64  centerFrequency,
                                                          ViReal64  RFBandwidth,
                                                          ViInt32   txChannelCount);

ViStatus _VI_FUNC rsspecan_ConfigureMSRTxChannelDefinition(ViSession instrSession,
                                                           ViInt32   subblock,
                                                           ViInt32   txChannel,
                                                           ViReal64  txCenterFrequency,
                                                           ViInt32   technology,
                                                           ViReal64  txBandwidth,
                                                           ViBoolean weightingFilterState,
                                                           ViReal64  rollOffFactor);

ViStatus _VI_FUNC rsspecan_QueryMSRTxName(ViSession instrSession,
                                          ViInt32   subblock,
                                          ViInt32   txChannel,
                                          ViInt32   bufferSize,
                                          ViChar    name[]);

ViStatus _VI_FUNC rsspecan_ConfigureMSRUpperChannelName(ViSession instrSession,
                                                        ViInt32   channel,
                                                        ViInt32   channelNumber,
                                                        ViString  name);

ViStatus _VI_FUNC rsspecan_ConfigureMSRActivateGaps(ViSession instrSession,
                                                    ViBoolean activateGaps);

ViStatus _VI_FUNC rsspecan_QueryMSRMeasurementPowerResult(ViSession instrSession,
                                                          ViInt32   subblock,
                                                          ViInt32   powerMeasurement,
                                                          ViInt32   arraySize,
                                                          ViReal64  results[],
                                                          ViInt32*  returnedValues);

ViStatus _VI_FUNC rsspecan_ConfigureMSRGapSize(ViSession instrSession,
                                               ViInt32   gap,
                                               ViReal64  gapSize);

ViStatus _VI_FUNC rsspecan_ConfigureMSRGapChannel(ViSession instrSession,
                                                  ViInt32   gap,
                                                  ViReal64  channelSpacing,
                                                  ViReal64  channelBandwidths,
                                                  ViBoolean weightingFilterState,
                                                  ViReal64  rollOffFactor);

ViStatus _VI_FUNC rsspecan_ConfigureMSRGapLimitChecking(ViSession instrSession,
                                                        ViInt32   gap,
                                                        ViBoolean relativeState,
                                                        ViReal64  relativeValue,
                                                        ViBoolean absoluteState,
                                                        ViReal64  absoluteValue);

ViStatus _VI_FUNC rsspecan_ConfigureMSRGapChannelSpacing(ViSession instrSession,
                                                         ViInt32   gap,
                                                         ViInt32   type,
                                                         ViInt32   gaps,
                                                         ViReal64  spacing);

ViStatus _VI_FUNC rsspecan_QueryMSRGapChannelSpacing(ViSession instrSession,
                                                     ViInt32   gap,
                                                     ViInt32   type,
                                                     ViInt32   gaps,
                                                     ViReal64* spacing);

ViStatus _VI_FUNC rsspecan_ConfigureMSRGapChannelBandwidths(ViSession instrSession,
                                                            ViInt32   gap,
                                                            ViInt32   type,
                                                            ViInt32   gaps,
                                                            ViReal64  bandwidth);

ViStatus _VI_FUNC rsspecan_QueryMSRGapChannelBandwidths(ViSession instrSession,
                                                        ViInt32   gap,
                                                        ViInt32   type,
                                                        ViInt32   gaps,
                                                        ViReal64* bandwidth);

ViStatus _VI_FUNC rsspecan_ConfigureMSRGapWeightingFilterState(
                            ViSession instrSession,
                            ViInt32   gap,
                            ViInt32   type,
                            ViInt32   gaps,
                            ViBoolean state);

ViStatus _VI_FUNC rsspecan_QueryMSRGapWeightingFilterState(ViSession  instrSession,
                                                           ViInt32    gap,
                                                           ViInt32    type,
                                                           ViInt32    gaps,
                                                           ViBoolean* state);

ViStatus _VI_FUNC rsspecan_ConfigureMSRGapChannelRollOffFactor(
                            ViSession instrSession,
                            ViInt32   gap,
                            ViInt32   type,
                            ViInt32   gaps,
                            ViReal64  rollOffFactor);

ViStatus _VI_FUNC rsspecan_QueryMSRGapChannelRollOffFactor(ViSession instrSession,
                                                           ViInt32   gap,
                                                           ViInt32   type,
                                                           ViInt32   gaps,
                                                           ViReal64* rollOffFactor);

ViStatus _VI_FUNC rsspecan_ConfigureMSRGapLimitRelativeState(ViSession instrSession,
                                                             ViInt32   gap,
                                                             ViInt32   type,
                                                             ViInt32   gaps,
                                                             ViBoolean state);

ViStatus _VI_FUNC rsspecan_QueryMSRGapLimitRelativeState(ViSession  instrSession,
                                                         ViInt32    gap,
                                                         ViInt32    type,
                                                         ViInt32    gaps,
                                                         ViBoolean* state);

ViStatus _VI_FUNC rsspecan_ConfigureMSRGapLimitRelative(ViSession instrSession,
                                                        ViInt32   gap,
                                                        ViInt32   type,
                                                        ViInt32   gaps,
                                                        ViReal64  limitRelative);

ViStatus _VI_FUNC rsspecan_QueryMSRGapLimitRelative(ViSession instrSession,
                                                    ViInt32   gap,
                                                    ViInt32   type,
                                                    ViInt32   gaps,
                                                    ViReal64* limitRelative);

ViStatus _VI_FUNC rsspecan_ConfigureMSRGapLimitAbsoluteState(ViSession instrSession,
                                                             ViInt32   gap,
                                                             ViInt32   type,
                                                             ViInt32   gaps,
                                                             ViBoolean state);

ViStatus _VI_FUNC rsspecan_QueryMSRGapLimitAbsoluteState(ViSession  instrSession,
                                                         ViInt32    gap,
                                                         ViInt32    type,
                                                         ViInt32    gaps,
                                                         ViBoolean* state);

ViStatus _VI_FUNC rsspecan_ConfigureMSRGapLimitAbsolute(ViSession instrSession,
                                                        ViInt32   gap,
                                                        ViInt32   type,
                                                        ViInt32   gaps,
                                                        ViReal64  limitAbsolute);

ViStatus _VI_FUNC rsspecan_QueryMSRGapLimitAbsolute(ViSession instrSession,
                                                    ViInt32   gap,
                                                    ViInt32   type,
                                                    ViInt32   gaps,
                                                    ViReal64* limitAbsolute);

ViStatus _VI_FUNC rsspecan_ConfigureMSRGapLimitChannelsEnabled(
                            ViSession instrSession,
                            ViInt32   gap,
                            ViInt32   type,
                            ViInt32   gaps,
                            ViBoolean enabled);

ViStatus _VI_FUNC rsspecan_QueryMSRGapLimitChannelsEnabled(ViSession  instrSession,
                                                           ViInt32    gap,
                                                           ViInt32    type,
                                                           ViInt32    gaps,
                                                           ViBoolean* enabled);

ViStatus _VI_FUNC rsspecan_ConfigureMSRGapMode(ViSession instrSession,
                                               ViInt32   gap,
                                               ViInt32   mode);

ViStatus _VI_FUNC rsspecan_ConfigureMSRGapLimitACLRRelative(ViSession instrSession,
                                                            ViInt32   gap,
                                                            ViReal64  ACLRRelative);

ViStatus _VI_FUNC rsspecan_ConfigureMSRGapLimitACLRRelatives(ViSession instrSession,
                                                             ViInt32   gap,
                                                             ViInt32   type,
                                                             ViInt32   gaps,
                                                             ViReal64  ACLRRelative);

ViStatus _VI_FUNC rsspecan_QueryMSRGapLimitACLRRelatives(ViSession instrSession,
                                                         ViInt32   gap,
                                                         ViInt32   type,
                                                         ViInt32   gaps,
                                                         ViReal64* ACLRRelative);

ViStatus _VI_FUNC rsspecan_ConfigureMSRGapLimitACLRRelativeState(
                            ViSession instrSession,
                            ViInt32   gap,
                            ViBoolean state);

ViStatus _VI_FUNC rsspecan_ConfigureMSRGapLimitACLRRelativeStates(
                            ViSession instrSession,
                            ViInt32   gap,
                            ViInt32   type,
                            ViInt32   gaps,
                            ViBoolean state);

ViStatus _VI_FUNC rsspecan_QueryMSRGapLimitACLRRelativeStates(
                            ViSession  instrSession,
                            ViInt32    gap,
                            ViInt32    type,
                            ViInt32    gaps,
                            ViBoolean* state);

ViStatus _VI_FUNC rsspecan_ConfigureMSRGapLimitCACLRRelative(ViSession instrSession,
                                                             ViInt32   gap,
                                                             ViReal64  limit,
                                                             ViReal64  upperLimit);

ViStatus _VI_FUNC rsspecan_ConfigureMSRGapLimitCACLRRelativeState(
                            ViSession instrSession,
                            ViInt32   gap,
                            ViBoolean state);

ViStatus _VI_FUNC rsspecan_QueryMSRGapLimitACLRResult(ViSession instrSession,
                                                      ViInt32   gap,
                                                      ViInt32   arraySize,
                                                      ViReal64  result[]);

ViStatus _VI_FUNC rsspecan_ConfigureMSRUpperAdjChannel(ViSession instrSession,
                                                       ViReal64  channelSpacing,
                                                       ViReal64  channelBandwidth,
                                                       ViBoolean weightingFilterState,
                                                       ViReal64  rollOffFactor);

ViStatus _VI_FUNC rsspecan_ConfigureMSRUpperAlternateChannel(ViSession instrSession,
                                                             ViInt32   channelNumber,
                                                             ViReal64  channelSpacing,
                                                             ViReal64  channelBandwidth,
                                                             ViBoolean weightingFilterState,
                                                             ViReal64  rollOffFactor);

ViStatus _VI_FUNC rsspecan_ConfigureCarrierNoiseMeasurement(ViSession instrSession,
                                                            ViBoolean state,
                                                            ViInt32   measurement,
                                                            ViReal64  bandwidth);

ViStatus _VI_FUNC rsspecan_ConfigureOccupiedBandwidth(ViSession instrSession,
                                                      ViInt32   reserved,
                                                      ViReal64  powerBandwidth,
                                                      ViReal64  channelBandwidth);

ViStatus _VI_FUNC rsspecan_ConfigureOccupiedBadwidthLimits(ViSession instrSession,
                                                           ViBoolean searchLimits,
                                                           ViReal64  searchLimitLeft,
                                                           ViReal64  searchLimitRight);

ViStatus _VI_FUNC rsspecan_ConfigureSEMState(ViSession instrSession,
                                             ViBoolean state);

ViStatus _VI_FUNC rsspecan_SEMSearchPeak(ViSession instrSession);

ViStatus _VI_FUNC rsspecan_ConfigureSEMReferenceRange(ViSession instrSession,
                                                      ViInt32   subBlock,
                                                      ViInt32   powerReferenceType,
                                                      ViReal64  txBandwidth,
                                                      ViBoolean RRCFilterState,
                                                      ViReal64  RRCFilterAlpha);

ViStatus _VI_FUNC rsspecan_QuerySEMReferenceRangePosition(ViSession instrSession,
                                                          ViInt32   subBlock,
                                                          ViInt32*  referenceRangePosition);

ViStatus _VI_FUNC rsspecan_ConfigureSEMSubBlockCount(ViSession instrSession,
                                                     ViInt32   subBlockCount);

ViStatus _VI_FUNC rsspecan_ConfigureSEMSubBlockCenterFrequency(
                            ViSession instrSession,
                            ViInt32   subBlock,
                            ViReal64  centerFrequency);

ViStatus _VI_FUNC rsspecan_ConfigureSEMSymmetricalSetup(ViSession instrSession,
                                                        ViInt32   subBlock,
                                                        ViBoolean symmetricalSetup);

ViStatus _VI_FUNC rsspecan_ConfigureSEMRangeStartStop(ViSession instrSession,
                                                      ViInt32   subBlock,
                                                      ViInt32   range,
                                                      ViReal64  rangeStart,
                                                      ViReal64  rangeStop);

ViStatus _VI_FUNC rsspecan_ConfigureSEMRangeFilterType(ViSession instrSession,
                                                       ViInt32   subBlock,
                                                       ViInt32   range,
                                                       ViInt32   filterType);

ViStatus _VI_FUNC rsspecan_ConfigureSEMRangeBandwidth(ViSession instrSession,
                                                      ViInt32   subBlock,
                                                      ViInt32   range,
                                                      ViReal64  resolutionBandwidth,
                                                      ViReal64  videoBandwidth);

ViStatus _VI_FUNC rsspecan_ConfigureSEMRangeSweepTime(ViSession instrSession,
                                                      ViInt32   subBlock,
                                                      ViInt32   range,
                                                      ViBoolean sweepTimeMode,
                                                      ViReal64  sweepTime);

ViStatus _VI_FUNC rsspecan_ConfigureSEMRangeReferenceLevel(ViSession instrSession,
                                                           ViInt32   subBlock,
                                                           ViInt32   range,
                                                           ViReal64  referenceLevel);

ViStatus _VI_FUNC rsspecan_ConfigureSEMRangeRFAttenuation(ViSession instrSession,
                                                          ViInt32   subBlock,
                                                          ViInt32   range,
                                                          ViBoolean RFAttenuationMode,
                                                          ViReal64  RFAttenuator,
                                                          ViBoolean preamplifier);

ViStatus _VI_FUNC rsspecan_ConfigureSEMRangeRFPreamplifierLevel(
                            ViSession instrSession,
                            ViInt32   subBlock,
                            ViInt32   range,
                            ViInt32   preamplificationLevel);

ViStatus _VI_FUNC rsspecan_ConfigureSEMRangeTransducerFactor(ViSession instrSession,
                                                             ViInt32   subBlock,
                                                             ViInt32   range,
                                                             ViString  transducerFactor);

ViStatus _VI_FUNC rsspecan_ConfigureSEMRangeMinimumSweepPoints(
                            ViSession instrSession,
                            ViInt32   subBlock,
                            ViInt32   range,
                            ViInt32   minimumSweepPoints);

ViStatus _VI_FUNC rsspecan_ConfigureSEMRangeMultiLimitCalc(ViSession instrSession,
                                                           ViInt32   subBlock,
                                                           ViInt32   range,
                                                           ViInt32   multiLimitCalc);

ViStatus _VI_FUNC rsspecan_ConfigureSEMRangeLimits(ViSession instrSession,
                                                   ViInt32   subBlock,
                                                   ViInt32   range,
                                                   ViInt32   powerClass,
                                                   ViInt32   limitCheckType,
                                                   ViReal64  absoluteLimitStart,
                                                   ViReal64  absoluteLimitStop,
                                                   ViReal64  relativeLimitStart,
                                                   ViReal64  relativeLimitStop);

ViStatus _VI_FUNC rsspecan_ConfigureSEMRangeRelativeLimitsFunctions(
                            ViSession instrSession,
                            ViInt32   subBlock,
                            ViInt32   range,
                            ViInt32   powerClass,
                            ViInt32   startFunction,
                            ViReal64  absoluteLimitStart,
                            ViInt32   stopFunction,
                            ViReal64  absoluteLimitStop);

ViStatus _VI_FUNC rsspecan_GetSEMRangeCount(ViSession instrSession,
                                            ViInt32   subBlock,
                                            ViInt32*  rangeCount);

ViStatus _VI_FUNC rsspecan_SEMInsertNewRange(ViSession instrSession,
                                             ViInt32   subBlock,
                                             ViInt32   range,
                                             ViInt32   insertRange);

ViStatus _VI_FUNC rsspecan_SEMDeleteRange(ViSession instrSession,
                                          ViInt32   subBlock,
                                          ViInt32   range);

ViStatus _VI_FUNC rsspecan_ConfigureFastSEM(ViSession instrSession,
                                            ViInt32   subBlock,
                                            ViBoolean fastSEM);

ViStatus _VI_FUNC rsspecan_ConfigureSEM(ViSession instrSession,
                                        ViInt32   subBlock,
                                        ViInt32   limitLineType,
                                        ViReal64  limitLineValue,
                                        ViReal64  reserved);

ViStatus _VI_FUNC rsspecan_ConfigureSEMUsedPowerClasses(ViSession instrSession,
                                                        ViInt32   subBlock,
                                                        ViInt32   powerClassNumber,
                                                        ViInt32   usedPowerClass);

ViStatus _VI_FUNC rsspecan_ConfigureSEMPowerClassCount(ViSession instrSession,
                                                       ViInt32   subBlock,
                                                       ViInt32   count);

ViStatus _VI_FUNC rsspecan_ConfigureSEMPowerClassLimits(ViSession instrSession,
                                                        ViInt32   subBlock,
                                                        ViInt32   powerClassNumber,
                                                        ViInt32   limitState,
                                                        ViReal64  lowerLevelLimit,
                                                        ViReal64  upperLevelLimit);

ViStatus _VI_FUNC rsspecan_ConfigureSEMPowerClassAllLimits(ViSession instrSession,
                                                           ViInt32   subBlock,
                                                           ViInt32   numberOfLimits,
                                                           ViReal64  limits[]);

ViStatus _VI_FUNC rsspecan_QuerySEMPowerClassAllLimits(ViSession instrSession,
                                                       ViInt32   subBlock,
                                                       ViReal64  limits[]);

ViStatus _VI_FUNC rsspecan_ConfigureSEMMSRSettings(ViSession instrSession,
                                                   ViInt32   subBlock,
                                                   ViInt32   bandCategory,
                                                   ViReal64  baseStationRFBandwidth,
                                                   ViBoolean GSMEDGEPresent,
                                                   ViBoolean LTEFDD14MHz3MHzPresent);

ViStatus _VI_FUNC rsspecan_ConfigureSEMMSRAdditionalSettings(ViSession instrSession,
                                                             ViInt32   subBlock,
                                                             ViInt32   baseStationClass,
                                                             ViReal64  BSMaxOutputPower,
                                                             ViInt32   bands);

ViStatus _VI_FUNC rsspecan_ApplySEMMSRSettings(ViSession instrSession,
                                               ViInt32   subBlock);

ViStatus _VI_FUNC rsspecan_ConfigureSEMPresetStandard(ViSession instrSession,
                                                      ViInt32   subBlock,
                                                      ViString  presetStandard);

ViStatus _VI_FUNC rsspecan_StoreSEMPresetSettings(ViSession instrSession,
                                                  ViInt32   subBlock,
                                                  ViString  settingsFileName);

ViStatus _VI_FUNC rsspecan_RestoreSEMStandardFiles(ViSession instrSession,
                                                   ViInt32   subBlock);

ViStatus _VI_FUNC rsspecan_RestoreSEMStandardLines(ViSession instrSession);

ViStatus _VI_FUNC rsspecan_SEMListEvaluation(ViSession instrSession,
                                             ViBoolean state);

ViStatus _VI_FUNC rsspecan_ConfigureSEMListEvaluation(ViSession instrSession,
                                                      ViBoolean showPeaks,
                                                      ViReal64  margin,
                                                      ViInt32   reserved);

ViStatus _VI_FUNC rsspecan_SEMSaveListToFile(ViSession instrSession,
                                             ViString  fileName);

ViStatus _VI_FUNC rsspecan_ReadSEMListEvaluationResults(ViSession instrSession,
                                                        ViInt32   noOfValues,
                                                        ViReal64  rangeNumber[],
                                                        ViReal64  startFrequency[],
                                                        ViReal64  stopFrequency[],
                                                        ViReal64  resolutionBandwidth[],
                                                        ViReal64  frequency[],
                                                        ViReal64  powerAbs[],
                                                        ViReal64  powerRel[],
                                                        ViReal64  delta[],
                                                        ViReal64  limitCheck[],
                                                        ViReal64  reserved1[],
                                                        ViReal64  reserved2[],
                                                        ViInt32*  returnedValues);

ViStatus _VI_FUNC rsspecan_ConfigureSEState(ViSession instrSession,
                                            ViBoolean state);

ViStatus _VI_FUNC rsspecan_SESearchPeak(ViSession instrSession);

ViStatus _VI_FUNC rsspecan_StoreSEToFile(ViSession instrSession,
                                         ViString  fileName);

ViStatus _VI_FUNC rsspecan_ConfigureSESweepListStartStop(ViSession instrSession,
                                                         ViInt32   range,
                                                         ViReal64  startFrequency,
                                                         ViReal64  stopFrequency);

ViStatus _VI_FUNC rsspecan_ConfigureSESweepListFilterType(ViSession instrSession,
                                                          ViInt32   range,
                                                          ViInt32   filter);

ViStatus _VI_FUNC rsspecan_ConfigureSESweepListBandwidth(ViSession instrSession,
                                                         ViInt32   range,
                                                         ViReal64  resolutionBandwidth,
                                                         ViReal64  videoBandwidth);

ViStatus _VI_FUNC rsspecan_ConfigureSESweepSweepListTime(ViSession instrSession,
                                                         ViInt32   range,
                                                         ViBoolean sweepTimeAuto,
                                                         ViReal64  sweepTime);

ViStatus _VI_FUNC rsspecan_ConfigureSESweepListDetector(ViSession instrSession,
                                                        ViInt32   range,
                                                        ViInt32   detector);

ViStatus _VI_FUNC rsspecan_ConfigureSESweepListReferenceLevel(
                            ViSession instrSession,
                            ViInt32   range,
                            ViReal64  refLevelValue);

ViStatus _VI_FUNC rsspecan_ConfigureSESweepListRFAttenuation(ViSession instrSession,
                                                             ViInt32   range,
                                                             ViBoolean attenuatorAuto,
                                                             ViReal64  manualAttenuatorValue);

ViStatus _VI_FUNC rsspecan_ConfigureSESweepListPreamplifier(ViSession instrSession,
                                                            ViInt32   range,
                                                            ViBoolean preamplifier);

ViStatus _VI_FUNC rsspecan_ConfigureSESweepListPreamplifierLevel(
                            ViSession instrSession,
                            ViInt32   range,
                            ViInt32   preamplificationLevel);

ViStatus _VI_FUNC rsspecan_ConfigureSESweepListSweep(ViSession instrSession,
                                                     ViInt32   range,
                                                     ViInt32   sweepPoints,
                                                     ViBoolean stopAfterSweep);

ViStatus _VI_FUNC rsspecan_ConfigureSESweepListTransducerFactor(
                            ViSession instrSession,
                            ViInt32   range,
                            ViString  transducerFactor);

ViStatus _VI_FUNC rsspecan_ConfigureSESweepListLimit(ViSession instrSession,
                                                     ViInt32   range,
                                                     ViBoolean limitChecking,
                                                     ViReal64  absoluteLimitStart,
                                                     ViReal64  absoluteLimitStop);

ViStatus _VI_FUNC rsspecan_AdjustSpuriousEmissionsXAxisToRangeDefinitions(ViSession instrSession);

ViStatus _VI_FUNC rsspecan_SEDeleteRange(ViSession instrSession,
                                         ViInt32   range);

ViStatus _VI_FUNC rsspecan_GetSENumberOfListRanges(ViSession instrSession,
                                                   ViInt32*  numberOfRanges);

ViStatus _VI_FUNC rsspecan_SEListEvaluation(ViSession instrSession,
                                            ViBoolean state);

ViStatus _VI_FUNC rsspecan_ConfigureSEListEvaluation(ViSession instrSession,
                                                     ViBoolean showPeaks,
                                                     ViReal64  margin,
                                                     ViInt32   peaksPerRange);

ViStatus _VI_FUNC rsspecan_ConfigureSEDetail(ViSession instrSession,
                                             ViBoolean listDetail);

ViStatus _VI_FUNC rsspecan_SESaveListToFile(ViSession instrSession,
                                            ViString  fileName);

ViStatus _VI_FUNC rsspecan_ReadSEPeakList(ViSession instrSession,
                                          ViInt32   noOfValues,
                                          ViReal64  frequency[],
                                          ViReal64  level[],
                                          ViReal64  deltaLimit[],
                                          ViInt32*  returnedValues);

ViStatus _VI_FUNC rsspecan_ReadSEListEvaluation(ViSession instrSession,
                                                ViInt32   noOfValues,
                                                ViReal64  rangeNumber[],
                                                ViReal64  startFrequency[],
                                                ViReal64  stopFrequency[],
                                                ViReal64  resolutionBandwidth[],
                                                ViReal64  frequency[],
                                                ViReal64  levelAbs[],
                                                ViReal64  levelRel[],
                                                ViReal64  delta[],
                                                ViReal64  limitCheck[],
                                                ViReal64  reserved1[],
                                                ViReal64  reserved2[],
                                                ViInt32*  returnedValues);

ViStatus _VI_FUNC rsspecan_SpuriousApplicationMode(ViSession instrSession);

ViStatus _VI_FUNC rsspecan_ConfigureSESearch(ViSession instrSession,
                                             ViInt32   type,
                                             ViBoolean removeResiduals,
                                             ViBoolean markResiduals);

ViStatus _VI_FUNC rsspecan_ConfigureSEMeasurementSteps(ViSession instrSession,
                                                       ViInt32   step);

ViStatus _VI_FUNC rsspecan_SpuriousAutoAdjustCarrier(ViSession instrSession);

ViStatus _VI_FUNC rsspecan_ConfigureSEMatchingCondition(ViSession instrSession,
                                                        ViInt32   condition);

ViStatus _VI_FUNC rsspecan_SELoadRanges(ViSession instrSession,
                                        ViString  fileName);

ViStatus _VI_FUNC rsspecan_SESaveRanges(ViSession instrSession,
                                        ViString  fileName);

ViStatus _VI_FUNC rsspecan_SEClearRanges(ViSession instrSession);

ViStatus _VI_FUNC rsspecan_ConfigureSpuriousCarrierDetection(ViSession instrSession,
                                                             ViReal64  carrierLevel,
                                                             ViInt32   powerReference);

ViStatus _VI_FUNC rsspecan_ConfigureSpuriousCarrierFrequencyReference(
                            ViSession instrSession,
                            ViInt32   reference);

ViStatus _VI_FUNC rsspecan_ConfigureSpuriousCarrierFrequency(ViSession instrSession,
                                                             ViReal64  frequency);

ViStatus _VI_FUNC rsspecan_ConfigureSpuriousCarrierHarmonics(ViSession instrSession,
                                                             ViBoolean identifyHarmonics,
                                                             ViReal64  toleranceForIdentification,
                                                             ViInt32   maxHarmonicsNumber);

ViStatus _VI_FUNC rsspecan_ConfigureSpuriousCarrierGuardInterval(
                            ViSession instrSession,
                            ViBoolean guardIntervalIncluded,
                            ViReal64  guardIntervalValue);

ViStatus _VI_FUNC rsspecan_SpuriousTransferToWideSearch(ViSession instrSession);

ViStatus _VI_FUNC rsspecan_SpuriousTransferSpursToDirected(ViSession instrSession,
                                                           ViInt32   arraySize,
                                                           ViInt32   spurs[]);

ViStatus _VI_FUNC rsspecan_ConfigureSpuriousDetectionTableContent(
                            ViSession instrSession,
                            ViBoolean enabled,
                            ViUInt32  arraySize,
                            ViInt32   headers[]);

ViStatus _VI_FUNC rsspecan_ConfigureSpuriousFrequencyPlanSettings(
                            ViSession instrSession,
                            ViBoolean frequencyPlan,
                            ViInt32   component,
                            ViReal64  centerFrequency,
                            ViReal64  span);

ViStatus _VI_FUNC rsspecan_ConfigureSpuriousFrequencyPlanTolerance(
                            ViSession instrSession,
                            ViReal64  toleranceForIdentification);

ViStatus _VI_FUNC rsspecan_ConfigureSpuriousFrequencyPlanIdentifier(
                            ViSession instrSession,
                            ViInt32   component,
                            ViInt32   identifier);

ViStatus _VI_FUNC rsspecan_ConfigureSpuriousFrequencyPlanComponentType(
                            ViSession instrSession,
                            ViInt32   component,
                            ViInt32   type);

ViStatus _VI_FUNC rsspecan_ConfigureSpuriousFrequencyPlanComponentDelete(
                            ViSession instrSession,
                            ViInt32   component);

ViStatus _VI_FUNC rsspecan_AddSpuriousFrequencyPlanComponent(ViSession instrSession,
                                                             ViInt32   component);

ViStatus _VI_FUNC rsspecan_SaveSpuriousFrequencyPlan(ViSession instrSession,
                                                     ViString  filename);

ViStatus _VI_FUNC rsspecan_LoadSpuriousFrequencyPlan(ViSession instrSession,
                                                     ViString  filename);

ViStatus _VI_FUNC rsspecan_ExportSpuriousFrequencyPlan(ViSession instrSession,
                                                       ViString  filename);

ViStatus _VI_FUNC rsspecan_TransferSpuriousFrequencySpursToTable(ViSession instrSession);

ViStatus _VI_FUNC rsspecan_ConfigureSpuriousFrequencyPlanInputFrequencySettings(
                            ViSession instrSession,
                            ViInt32   component,
                            ViInt32   inputPort,
                            ViReal64  inputFrequency,
                            ViInt32   maximumHarmonics);

ViStatus _VI_FUNC rsspecan_ConfigureSpuriousLISN(ViSession instrSession,
                                                 ViInt32   LISN,
                                                 ViBoolean highPassFilter,
                                                 ViInt32   phase);

ViStatus _VI_FUNC rsspecan_ConfigureCarrierSearchRangeType(ViSession instrSession,
                                                           ViInt32   type);

ViStatus _VI_FUNC rsspecan_ConfigureCarrierSearchRangeCenterSpan(
                            ViSession instrSession,
                            ViReal64  centerFrequency,
                            ViReal64  span);

ViStatus _VI_FUNC rsspecan_ConfigureCarrierSearchRangeStartStop(
                            ViSession instrSession,
                            ViReal64  startFrequency,
                            ViReal64  stopFrequency);

ViStatus _VI_FUNC rsspecan_ConfigureSpuriousDirectedDetector(ViSession instrSession,
                                                             ViInt32   detector);

ViStatus _VI_FUNC rsspecan_ConfigureSpuriousDirectedSearch(ViSession instrSession,
                                                           ViReal64  limitOffset,
                                                           ViReal64  peakExcursion,
                                                           ViReal64  maximumFinalRBW,
                                                           ViReal64  referenceLevel,
                                                           ViInt32   numberOfFFTAverages);

ViStatus _VI_FUNC rsspecan_ConfigureSpuriousDirectedSearchSettings(
                            ViSession instrSession,
                            ViInt32   arraySize,
                            ViReal64  frequency[],
                            ViReal64  searchSpan[],
                            ViReal64  detectionThreshold[],
                            ViReal64  desiredSpurSNR[]);

ViStatus _VI_FUNC rsspecan_ConfigureSpuriousDirectedRFAttenuation(
                            ViSession instrSession,
                            ViInt32   RFAttenuation);

ViStatus _VI_FUNC rsspecan_ConfigureSpuriousDirectedPreamplifier(
                            ViSession instrSession,
                            ViBoolean state,
                            ViInt32   gain);

ViStatus _VI_FUNC rsspecan_SpuriousDirectedLoadTable(ViSession instrSession,
                                                     ViString  fileName);

ViStatus _VI_FUNC rsspecan_SpuriousDirectedSaveTable(ViSession instrSession,
                                                     ViString  fileName);

ViStatus _VI_FUNC rsspecan_QuerySpuriousRangeNumberOfRanges(ViSession instrSession,
                                                            ViInt32*  numberOfRanges);

ViStatus _VI_FUNC rsspecan_InsertSpuriousRange(ViSession instrSession,
                                               ViInt32   range,
                                               ViInt32   insertDirection);

ViStatus _VI_FUNC rsspecan_ConfigureSpuriousRangeUsSelectionForAllRanges(
                            ViSession instrSession,
                            ViInt32   range,
                            ViInt32   selectedSetting);

ViStatus _VI_FUNC rsspecan_ConfigureSERangeRFAttenuation(ViSession instrSession,
                                                         ViInt32   range,
                                                         ViReal64  attenuation);

ViStatus _VI_FUNC rsspecan_ConfigureSERangeBandwidth(ViSession instrSession,
                                                     ViInt32   range,
                                                     ViBoolean RBWAuto,
                                                     ViReal64  resolutionBandwidth,
                                                     ViReal64  maximumFinalRBW);

ViStatus _VI_FUNC rsspecan_ConfigureSERangeSpurDetection(ViSession instrSession,
                                                         ViInt32   range,
                                                         ViReal64  thresholdStart,
                                                         ViReal64  thresholdStop,
                                                         ViReal64  limitOffset,
                                                         ViReal64  peakExcursion,
                                                         ViReal64  minimumSNR);

ViStatus _VI_FUNC rsspecan_ConfigureSERangeNumberFFTAverage(ViSession instrSession,
                                                            ViInt32   range,
                                                            ViInt32   numberOfFFTAverages);

ViStatus _VI_FUNC rsspecan_ConfigureSignalStatisticMeasurement(
                            ViSession instrSession,
                            ViInt32   measurement,
                            ViInt32   numberOfSamples,
                            ViReal64  resolutionBandwidth);

ViStatus _VI_FUNC rsspecan_ConfigureSignalStatisticNormalDistributionEnabled(
                            ViSession instrSession,
                            ViBoolean showNormalDistribution);

ViStatus _VI_FUNC rsspecan_ProbabilityMarkerPosition(ViSession instrSession,
                                                     ViInt32   reserved,
                                                     ViInt32   marker,
                                                     ViReal64  probability);

ViStatus _VI_FUNC rsspecan_ConfigureSignalStatisticGate(ViSession instrSession,
                                                        ViInt32   trace,
                                                        ViString  comment,
                                                        ViReal64  period);

ViStatus _VI_FUNC rsspecan_ConfigureSignalStatisticGateRange(ViSession instrSession,
                                                             ViInt32   trace,
                                                             ViInt32   range,
                                                             ViBoolean state,
                                                             ViReal64  start,
                                                             ViReal64  stop);

ViStatus _VI_FUNC rsspecan_ConfigureSignalStatisticScaleXAxis(
                            ViSession instrSession,
                            ViReal64  xAxisRefLevel,
                            ViReal64  xAxisRange,
                            ViReal64  offset);

ViStatus _VI_FUNC rsspecan_ConfigureSignalStatisticMeasurementYAxis(
                            ViSession instrSession,
                            ViInt32   yUnit,
                            ViReal64  yAxisMaxValue,
                            ViReal64  yAxisMinValue);

ViStatus _VI_FUNC rsspecan_SignalStatisticMeasurementPresetScaling(ViSession instrSession);

ViStatus _VI_FUNC rsspecan_SignalStatisticMeasurementAdjustSettings(
                            ViSession instrSession,
                            ViUInt32  timeout);

ViStatus _VI_FUNC rsspecan_QuerySignalStatisticCCDFLevel(ViSession instrSession,
                                                         ViInt32   trace,
                                                         ViInt32   probability,
                                                         ViReal64* result);

ViStatus _VI_FUNC rsspecan_QuerySignalStatisticResult(ViSession instrSession,
                                                      ViInt32   trace,
                                                      ViInt32   resultType,
                                                      ViReal64* result);

ViStatus _VI_FUNC rsspecan_QuerySignalStatisticAllResults(ViSession instrSession,
                                                          ViInt32   trace,
                                                          ViInt32   arraySize,
                                                          ViReal64  results[]);

ViStatus _VI_FUNC rsspecan_ConfigureTimeDomainPowerMeasurementState(
                            ViSession instrSession,
                            ViInt32   reserved,
                            ViBoolean state);

ViStatus _VI_FUNC rsspecan_ConfigureTimeDomainPowerMeasurement(
                            ViSession instrSession,
                            ViInt32   reserved,
                            ViBoolean peak,
                            ViBoolean RMS,
                            ViBoolean mean,
                            ViBoolean standardDeviation,
                            ViBoolean average,
                            ViBoolean maxHold,
                            ViInt32   power);

ViStatus _VI_FUNC rsspecan_ConfigureTimeDomainPowerLimits(ViSession instrSession,
                                                          ViBoolean searchLimits,
                                                          ViReal64  searchLimitLeft,
                                                          ViReal64  searchLimitRight);

ViStatus _VI_FUNC rsspecan_TimeDomainPowerMeasurementAllOff(ViSession instrSession,
                                                            ViInt32   reserved);

ViStatus _VI_FUNC rsspecan_TimeDomainPowerMeasurementSetReference(
                            ViSession instrSession,
                            ViInt32   reserved);

ViStatus _VI_FUNC rsspecan_QueryTDomPowerResult(ViSession instrSession,
                                                ViInt32   reserved,
                                                ViInt32   measurement,
                                                ViInt32   resultType,
                                                ViReal64* result);

ViStatus _VI_FUNC rsspecan_QueryTDomPowerPulseSequenceResult(ViSession instrSession,
                                                             ViInt32   reserved,
                                                             ViReal64  timeOffsetOfFirstPulse,
                                                             ViReal64  measurementTime,
                                                             ViReal64  period,
                                                             ViInt32   numberOfPulses,
                                                             ViReal64  result[]);

ViStatus _VI_FUNC rsspecan_ConfigureHDistState(ViSession instrSession,
                                               ViBoolean state);

ViStatus _VI_FUNC rsspecan_ConfigureHDist(ViSession instrSession,
                                          ViInt32   noOfHarmonics,
                                          ViReal64  harmonicSweepTime,
                                          ViBoolean harmonicRBWAuto);

ViStatus _VI_FUNC rsspecan_AdjustHDistSettings(ViSession instrSession,
                                               ViUInt32  timeout);

ViStatus _VI_FUNC rsspecan_QueryHDist(ViSession instrSession,
                                      ViReal64* harmonicDistortionInPercent,
                                      ViReal64* harmonicDistortionInDB);

ViStatus _VI_FUNC rsspecan_QueryHDistHarmList(ViSession instrSession,
                                              ViInt32   arraySize,
                                              ViReal64  harmonicsList[],
                                              ViInt32*  returnedValues);

ViStatus _VI_FUNC rsspecan_ConfigureTOIMeasurement(ViSession instrSession,
                                                   ViInt32   reserved,
                                                   ViBoolean state);

ViStatus _VI_FUNC rsspecan_TOISignalSearch(ViSession instrSession,
                                           ViInt32   reserved,
                                           ViInt32   reserved2);

ViStatus _VI_FUNC rsspecan_QueryTOI(ViSession instrSession,
                                    ViInt32   reserved,
                                    ViReal64* TOIMeasurement);

ViStatus _VI_FUNC rsspecan_QueryTOIResults(ViSession instrSession,
                                           ViReal64* maximum,
                                           ViReal64* minimum);

ViStatus _VI_FUNC rsspecan_ConfigureModDepthMeasurement(ViSession instrSession,
                                                        ViInt32   reserved,
                                                        ViBoolean state);

ViStatus _VI_FUNC rsspecan_ModulationDepthSignalSearch(ViSession instrSession,
                                                       ViInt32   reserved);

ViStatus _VI_FUNC rsspecan_QueryModulationDepth(ViSession instrSession,
                                                ViInt32   reserved,
                                                ViReal64* modulationDepth);

ViStatus _VI_FUNC rsspecan_BurstPowerSequence(ViSession instrSession,
                                              ViInt32   reserved,
                                              ViReal64  frequencyHz,
                                              ViReal64  resolutionBandwidthHz,
                                              ViReal64  measTimes,
                                              ViInt32   triggerSource,
                                              ViReal64  triggerLevel,
                                              ViReal64  triggerOffsets,
                                              ViInt32   detector,
                                              ViInt32   noOfMeasurements);

ViStatus _VI_FUNC rsspecan_BurstPowerFilterType(ViSession instrSession,
                                                ViInt32   reserved,
                                                ViInt32   filterType);

ViStatus _VI_FUNC rsspecan_GetBurstPowerSequence(ViSession instrSession,
                                                 ViInt32   reserved,
                                                 ViReal64  frequencyHz,
                                                 ViReal64  resolutionBandwidthHz,
                                                 ViReal64  measTimes,
                                                 ViInt32   triggerSource,
                                                 ViReal64  triggerLevel,
                                                 ViReal64  triggerOffsets,
                                                 ViInt32   detector,
                                                 ViInt32   noOfMeasurements,
                                                 ViUInt32  timeout,
                                                 ViReal64  pulsePowerResults[]);

ViStatus _VI_FUNC rsspecan_GetBurstPowerResults(ViSession instrSession,
                                                ViInt32   reserved,
                                                ViInt32   noOfResults,
                                                ViReal64  pulsePowerResults[]);

ViStatus _VI_FUNC rsspecan_GetBurstPwrResultMin(ViSession instrSession,
                                                ViInt32   reserved,
                                                ViReal64* powerResultMin);

ViStatus _VI_FUNC rsspecan_ConfigureNPR(ViSession instrSession,
                                        ViBoolean NPREnabled,
                                        ViReal64  channelBandwidth,
                                        ViInt32   notchCount);

ViStatus _VI_FUNC rsspecan_ConfigureNPRMode(ViSession instrSession,
                                            ViInt32   mode);

ViStatus _VI_FUNC rsspecan_ConfigureNPRIntegrationBandwidth(ViSession instrSession,
                                                            ViBoolean bandwidthAutoEnabled,
                                                            ViReal64  integrationBandwidth,
                                                            ViReal64  frequencyOffset);

ViStatus _VI_FUNC rsspecan_ConfigureNPRNotch(ViSession instrSession,
                                             ViUInt32  notch,
                                             ViReal64  frequencyOffset,
                                             ViReal64  bandwidth);

ViStatus _VI_FUNC rsspecan_ConfigureNPRNotchBandwidthRelative(
                            ViSession instrSession,
                            ViUInt32  notch,
                            ViReal64  bandwidthRelative);

ViStatus _VI_FUNC rsspecan_QueryNPRResult(ViSession instrSession,
                                          ViInt32   resultType,
                                          ViReal64* result);

ViStatus _VI_FUNC rsspecan_QueryNPRResultAll(ViSession instrSession,
                                             ViUInt32  arraySize,
                                             ViReal64  result[],
                                             ViInt32*  returnedValues);

ViStatus _VI_FUNC rsspecan_ConfigureNPRGeneratorControl(ViSession instrSession,
                                                        ViBoolean enabled);

ViStatus _VI_FUNC rsspecan_ConfigureNPRGenerator(ViSession instrSession,
                                                 ViInt32   pathRF,
                                                 ViBoolean RFOutput,
                                                 ViReal64  frequency,
                                                 ViReal64  levelRMS,
                                                 ViReal64  levelOffset,
                                                 ViInt32   referenceFrequency);

ViStatus _VI_FUNC rsspecan_ConfigureNPRGeneratorFrequencyCoupling(
                            ViSession instrSession,
                            ViBoolean coupling,
                            ViReal64  numerator,
                            ViReal64  denominator,
                            ViReal64  frequencyOffset);

ViStatus _VI_FUNC rsspecan_QueryNPRGeneratorSettings(ViSession instrSession,
                                                     ViInt32   arraySize,
                                                     ViInt32*  pathBB,
                                                     ViChar    standard[],
                                                     ViChar    ARBWaveformFile[]);

ViStatus _VI_FUNC rsspecan_QueryNPRGeneratorStates(ViSession instrSession,
                                                   ViInt32*  generatorState,
                                                   ViInt32*  RFOutput,
                                                   ViInt32*  frequency,
                                                   ViInt32*  levelRMS,
                                                   ViInt32*  levelOffset,
                                                   ViInt32*  referenceFrequency,
                                                   ViInt32*  notchFilter);

ViStatus _VI_FUNC rsspecan_ConfigureNPRGeneratorIP(ViSession instrSession,
                                                   ViString  IPAddress,
                                                   ViBoolean connect);

ViStatus _VI_FUNC rsspecan_QueryNPRGeneratorIPConnectionState(
                            ViSession instrSession,
                            ViInt32   bufferSize,
                            ViChar    connectionState[],
                            ViInt32*  returnedBytes);

ViStatus _VI_FUNC rsspecan_ApplyNPRGeneratorAllSettings(ViSession instrSession);

ViStatus _VI_FUNC rsspecan_ApplyNPRGeneratorSettings(ViSession instrSession);

ViStatus _VI_FUNC rsspecan_ApplyNPRGeneratorNotchSettings(ViSession instrSession);

ViStatus _VI_FUNC rsspecan_QueryNPRGeneratorNotchesNumber(ViSession instrSession,
                                                          ViInt32*  notches);

ViStatus _VI_FUNC rsspecan_ConfigureNPRGeneratorNotch(ViSession instrSession,
                                                      ViInt32   notch,
                                                      ViBoolean enabled);

ViStatus _VI_FUNC rsspecan_QueryNPRGeneratorNotchStates(ViSession instrSession,
                                                        ViInt32   notch,
                                                        ViInt32*  bandwidth,
                                                        ViInt32*  frequencyOffset,
                                                        ViInt32*  state);

ViStatus _VI_FUNC rsspecan_QueryNPRGeneratorNotchClockFrequency(
                            ViSession instrSession,
                            ViReal64* clockFrequency);

ViStatus _VI_FUNC rsspecan_ApplyNPRGeneratorAllNotchSettings(ViSession instrSession);

ViStatus _VI_FUNC rsspecan_SetIQMode(ViSession instrSession);

ViStatus _VI_FUNC rsspecan_LoadIQData(ViSession instrSession,
                                      ViString  fileName);

ViStatus _VI_FUNC rsspecan_StoreIQData(ViSession instrSession,
                                       ViString  fileName);

ViStatus _VI_FUNC rsspecan_ConfigureIQDataExportFileDescription(
                            ViSession instrSession,
                            ViString  description);

ViStatus _VI_FUNC rsspecan_SelectIQDataFile(ViSession instrSession,
                                            ViString  fileName);

ViStatus _VI_FUNC rsspecan_ConfigureTraceIQBWExtension(ViSession instrSession,
                                                       ViInt32   reserved,
                                                       ViBoolean state);

ViStatus _VI_FUNC rsspecan_ConfigureTraceIQBW(ViSession instrSession,
                                              ViInt32   type,
                                              ViReal64  maximumBandwidth);

ViStatus _VI_FUNC rsspecan_ForceTraceIQBWExtension(ViSession instrSession);

ViStatus _VI_FUNC rsspecan_ConfigureIQSet(ViSession instrSession,
                                          ViReal64  samplingRate,
                                          ViInt32   triggerMode,
                                          ViInt32   triggerSlope,
                                          ViInt32   pretriggerSamples,
                                          ViInt32   noOfSamples);

ViStatus _VI_FUNC rsspecan_ConfigureIQDataAcquisition(ViSession instrSession,
                                                      ViReal64  samplingRate,
                                                      ViReal64  analysisBandwidth,
                                                      ViBoolean maximumBandwidth,
                                                      ViInt32   recordLength,
                                                      ViBoolean swapIQ);

ViStatus _VI_FUNC rsspecan_ConfigureTraceIQDataAcquisition(ViSession instrSession,
                                                           ViInt32   reserved,
                                                           ViBoolean state);

ViStatus _VI_FUNC rsspecan_ConfigureTraceIQDataAnalyzerEnabled(
                            ViSession instrSession,
                            ViInt32   reserved,
                            ViBoolean state);

ViStatus _VI_FUNC rsspecan_ConfigureIQBandwidth(ViSession instrSession,
                                                ViInt32   mode,
                                                ViReal64  resolution);

ViStatus _VI_FUNC rsspecan_ConfigureIQFFTMode(ViSession instrSession,
                                              ViInt32   algorithm,
                                              ViInt32   length,
                                              ViInt32   windowType,
                                              ViReal64  windowOverlap,
                                              ViInt32   windowLength);

ViStatus _VI_FUNC rsspecan_ConfigureTraceIQGate(ViSession instrSession,
                                                ViBoolean gate,
                                                ViInt32   type,
                                                ViInt32   gateLength,
                                                ViInt32   gateGap,
                                                ViInt32   numberOfGatePeriods);

ViStatus _VI_FUNC rsspecan_ConfigureTraceIQDataDisplayType(ViSession instrSession,
                                                           ViInt32   displayType);

ViStatus _VI_FUNC rsspecan_ConfigureIQAveragePowerConsumption(
                            ViSession instrSession,
                            ViBoolean state,
                            ViReal64  factorA,
                            ViReal64  factorB);

ViStatus _VI_FUNC rsspecan_ConfigureIQOscilloscopeCoupling(ViSession instrSession,
                                                           ViInt32   coupling);

ViStatus _VI_FUNC rsspecan_QueryIQAveragePowerConsumptionResult(
                            ViSession instrSession,
                            ViReal64* result);

ViStatus _VI_FUNC rsspecan_QueryTraceIQFilterBandwidth(ViSession instrSession,
                                                       ViReal64* filterBandwidth);

ViStatus _VI_FUNC rsspecan_ConfigureIQDataFormat(ViSession instrSession,
                                                 ViInt32   format,
                                                 ViInt32   dataFormat);

ViStatus _VI_FUNC rsspecan_QueryIQDataFormat(ViSession instrSession,
                                             ViInt32*  format,
                                             ViInt32*  dataFormat);

ViStatus _VI_FUNC rsspecan_QueryTriggerPositionInSample(ViSession instrSession,
                                                        ViInt32*  triggerPositionInSample);

ViStatus _VI_FUNC rsspecan_ReadTraceIQData(ViSession instrSession,
                                           ViInt32   reserved,
                                           ViUInt32  timeout,
                                           ViInt32   bufferSize,
                                           ViInt32*  noOfPoints,
                                           ViReal64  realParts_I[],
                                           ViReal64  imaginaryParts_Q[]);

ViStatus _VI_FUNC rsspecan_FetchTraceIQData(ViSession instrSession,
                                            ViInt32   offsetSamples,
                                            ViInt32   noOfSamples,
                                            ViInt32   bufferSize,
                                            ViInt32*  noOfPoints,
                                            ViReal64  realParts_I[],
                                            ViReal64  imaginaryParts_Q[]);

ViStatus _VI_FUNC rsspecan_ConfigureB2000State(ViSession instrSession,
                                               ViBoolean state);

ViStatus _VI_FUNC rsspecan_ConfigureOscilloscopeDisplayUpdate(
                            ViSession instrSession,
                            ViBoolean fastExport);

ViStatus _VI_FUNC rsspecan_ConfigureOscilloscopeTCPIPAddress(ViSession instrSession,
                                                             ViString  TCPIPAddress);

ViStatus _VI_FUNC rsspecan_QueryOscilloscopeLANConnectionState(
                            ViSession instrSession,
                            ViInt32*  LANConnectionState);

ViStatus _VI_FUNC rsspecan_QueryOscilloscopeInfo(ViSession  instrSession,
                                                 ViInt32    bufferSize,
                                                 ViChar     ID[],
                                                 ViBoolean* supportedInstrument,
                                                 ViBoolean* supportedFirmware);

ViStatus _VI_FUNC rsspecan_OscilloscopeAlignment(ViSession  instrSession,
                                                 ViUInt32   timeout,
                                                 ViBoolean* alignmentState);

ViStatus _VI_FUNC rsspecan_ConfigureOscilloscopeSelfAlignmentEnabled(
                            ViSession instrSession,
                            ViBoolean selfAlignmentEnabled);

ViStatus _VI_FUNC rsspecan_QueryOscilloscopeAlignmentDate(ViSession instrSession,
                                                          ViInt32   bufferSize,
                                                          ViChar    alignmentDate[]);

ViStatus _VI_FUNC rsspecan_ConfigureOscilloscopeSampleRate(ViSession instrSession,
                                                           ViInt32   sampleRate);

ViStatus _VI_FUNC rsspecan_ConfigureOscilloscopePowerSplitterMode(
                            ViSession instrSession,
                            ViBoolean powerSplitterMode);

ViStatus _VI_FUNC rsspecan_MSRAMode(ViSession instrSession);

ViStatus _VI_FUNC rsspecan_RTMSMode(ViSession instrSession);

ViStatus _VI_FUNC rsspecan_QueryMSRAAnalysisInterval(ViSession instrSession,
                                                     ViInt32   window,
                                                     ViReal64* intervalStart,
                                                     ViReal64* intervalStop);

ViStatus _VI_FUNC rsspecan_QueryMSRAAllAnalysisInterval(ViSession instrSession,
                                                        ViInt32   window,
                                                        ViInt32   arraySize,
                                                        ViReal64  intervalStart[],
                                                        ViReal64  intervalStop[]);

ViStatus _VI_FUNC rsspecan_MSRARefresh(ViSession instrSession);

ViStatus _VI_FUNC rsspecan_ConfigureMSRACaptureOffset(ViSession instrSession,
                                                      ViInt32   offset);

ViStatus _VI_FUNC rsspecan_ConfigureMSRAAnalysisLine(ViSession instrSession,
                                                     ViBoolean showLine,
                                                     ViReal64  position);

ViStatus _VI_FUNC rsspecan_ConfigureMSRADataEvaluation(ViSession instrSession,
                                                       ViInt32   dataEvaluation);

ViStatus _VI_FUNC rsspecan_ConfigureRTMSCaptureOffset(ViSession instrSession,
                                                      ViInt32   offset);

ViStatus _VI_FUNC rsspecan_ConfigureRTMSAnalysisLine(ViSession instrSession,
                                                     ViBoolean showLine,
                                                     ViReal64  position);

ViStatus _VI_FUNC rsspecan_QueryRTMSAnalysisInterval(ViSession instrSession,
                                                     ViInt32   window,
                                                     ViReal64* intervalStart,
                                                     ViReal64* intervalStop);

ViStatus _VI_FUNC rsspecan_Preset(ViSession instrSession);

ViStatus _VI_FUNC rsspecan_ConfigureHardcopyDevice(ViSession instrSession,
                                                   ViInt32   device,
                                                   ViInt32   destination,
                                                   ViInt32   pageOrientation);

ViStatus _VI_FUNC rsspecan_ConfigureHardcopyPrintMarginsSettings(
                            ViSession instrSession,
                            ViInt32   device,
                            ViBoolean state,
                            ViReal64  top,
                            ViReal64  bottom,
                            ViReal64  left,
                            ViReal64  right,
                            ViInt32   marginUnit);

ViStatus _VI_FUNC rsspecan_ConfigureHardcopyWindowsSettings(ViSession instrSession,
                                                            ViInt32   device,
                                                            ViString  channelName,
                                                            ViBoolean state,
                                                            ViInt32   windowsPerPage,
                                                            ViBoolean scaling);

ViStatus _VI_FUNC rsspecan_HardcopyAbort(ViSession instrSession);

ViStatus _VI_FUNC rsspecan_HardcopyContent(ViSession instrSession,
                                           ViInt32   content);

ViStatus _VI_FUNC rsspecan_HardcopyPrint(ViSession instrSession,
                                         ViInt32   device,
                                         ViInt32   items);

ViStatus _VI_FUNC rsspecan_HardcopyPrintNext(ViSession instrSession,
                                             ViInt32   device);

ViStatus _VI_FUNC rsspecan_HardcopySetFileName(ViSession instrSession,
                                               ViString  name);

ViStatus _VI_FUNC rsspecan_HardcopyGetPrinterList(ViSession instrSession,
                                                  ViInt32   bufferSize,
                                                  ViChar    printerList[]);

ViStatus _VI_FUNC rsspecan_HardcopySetPrinter(ViSession instrSession,
                                              ViInt32   device,
                                              ViString  printerName);

ViStatus _VI_FUNC rsspecan_HardcopyComment(ViSession instrSession,
                                           ViString  comment);

ViStatus _VI_FUNC rsspecan_HardcopyTimeDate(ViSession instrSession,
                                            ViInt32   device,
                                            ViBoolean timeAndDateStamp);

ViStatus _VI_FUNC rsspecan_HardcopyPageIncludeAllWindows(ViSession instrSession,
                                                         ViInt32   device,
                                                         ViString  channelName,
                                                         ViBoolean includeWindows);

ViStatus _VI_FUNC rsspecan_HardcopyPageIncludeSelectedWindow(ViSession instrSession,
                                                             ViInt32   device,
                                                             ViString  channelName,
                                                             ViString  windowName,
                                                             ViBoolean includeWindows);

ViStatus _VI_FUNC rsspecan_ConfigurePrintMode(ViSession instrSession,
                                              ViInt32   printMode);

ViStatus _VI_FUNC rsspecan_ConfigureTestReportsGeneralSettings(
                            ViSession instrSession,
                            ViInt32   pageFormat,
                            ViBoolean useScreenColors,
                            ViBoolean date,
                            ViBoolean pageCount);

ViStatus _VI_FUNC rsspecan_ConfigureTestReportsTitlePage(ViSession instrSession,
                                                         ViBoolean titlePage,
                                                         ViString  title,
                                                         ViString  abstract);

ViStatus _VI_FUNC rsspecan_ConfigureTestReportsMeasurementInfoState(
                            ViSession instrSession,
                            ViBoolean measurementInfoState);

ViStatus _VI_FUNC rsspecan_ConfigureTestReportsMeasurementInfo(
                            ViSession instrSession,
                            ViInt32   line,
                            ViString  title,
                            ViString  value,
                            ViInt32   visibility);

ViStatus _VI_FUNC rsspecan_ConfigureTestReportsMeasurementInfoLogo(
                            ViSession instrSession,
                            ViString  logo,
                            ViInt32   logoVisibility);

ViStatus _VI_FUNC rsspecan_ConfigureTestReportsItemSelect(ViSession instrSession,
                                                          ViString  item);

ViStatus _VI_FUNC rsspecan_QueryTestReportsItemList(ViSession instrSession,
                                                    ViString  channelType,
                                                    ViInt32   bufferSize,
                                                    ViChar    itemList[],
                                                    ViInt32*  returnedValues);

ViStatus _VI_FUNC rsspecan_QueryTestReportsTemplatesCatalog(ViSession instrSession,
                                                            ViInt32   bufferSize,
                                                            ViChar    templatesCatalog[],
                                                            ViInt32*  returnedValues);

ViStatus _VI_FUNC rsspecan_SaveTestReportsTemplate(ViSession instrSession,
                                                   ViString  templateName);

ViStatus _VI_FUNC rsspecan_LoadTestReportsTemplate(ViSession instrSession,
                                                   ViString  templateName);

ViStatus _VI_FUNC rsspecan_RestoreTestReportsDefaultConfiguration(ViSession instrSession);

ViStatus _VI_FUNC rsspecan_NewTestReport(ViSession instrSession);

ViStatus _VI_FUNC rsspecan_AppendTestReport(ViSession instrSession);

ViStatus _VI_FUNC rsspecan_RemoveTestReportsDataset(ViSession instrSession,
                                                    ViInt32   dataset);

ViStatus _VI_FUNC rsspecan_RemoveAllTestReportsDatasets(ViSession instrSession);

ViStatus _VI_FUNC rsspecan_FileDecimalSeparator(ViSession instrSession,
                                                ViInt32   decimalSeparator);

ViStatus _VI_FUNC rsspecan_FileManagerOperations(ViSession instrSession,
                                                 ViInt32   operation,
                                                 ViString  source,
                                                 ViString  destination);

ViStatus _VI_FUNC rsspecan_FileDirectory(ViSession instrSession,
                                         ViString  directory,
                                         ViInt32   bufferSize,
                                         ViChar    output[]);

ViStatus _VI_FUNC rsspecan_FileDirectoryPath(ViSession instrSession,
                                             ViString  directory,
                                             ViInt32   bufferSize,
                                             ViChar    output[]);

ViStatus _VI_FUNC rsspecan_FileExportMicroservice(ViSession instrSession,
                                                  ViString  file);

ViStatus _VI_FUNC rsspecan_DataSetFileOperations(ViSession instrSession,
                                                 ViInt32   operation,
                                                 ViString  path);

ViStatus _VI_FUNC rsspecan_DataSetFileSaveNext(ViSession instrSession);

ViStatus _VI_FUNC rsspecan_DataSetFileComment(ViSession instrSession,
                                              ViString  comment);

ViStatus _VI_FUNC rsspecan_DataSetFileClearAll(ViSession instrSession,
                                               ViString  directory);

ViStatus _VI_FUNC rsspecan_DataSetFileSelectItems(ViSession instrSession,
                                                  ViInt32   itemSelector,
                                                  ViBoolean itemState);

ViStatus _VI_FUNC rsspecan_DataSetFileSelectItemsFromChannel(ViSession instrSession,
                                                             ViInt32   itemSelector,
                                                             ViBoolean itemState);

ViStatus _VI_FUNC rsspecan_DataSetLoadMode(ViSession instrSession,
                                           ViInt32   mode);

ViStatus _VI_FUNC rsspecan_DataSetStoreMode(ViSession instrSession,
                                            ViInt32   mode);

ViStatus _VI_FUNC rsspecan_NetworkMapDrive(ViSession instrSession,
                                           ViString  driveLetter,
                                           ViString  hostNameIPAddress,
                                           ViBoolean useOptionalParameters,
                                           ViString  userName,
                                           ViString  password,
                                           ViBoolean reconnectAtLogon);

ViStatus _VI_FUNC rsspecan_NetworkDisconnectDrive(ViSession instrSession,
                                                  ViString  driveLetter);

ViStatus _VI_FUNC rsspecan_NetworkUnusedDrivesList(ViSession instrSession,
                                                   ViInt32   arraySize,
                                                   ViChar    unusedDrives[]);

ViStatus _VI_FUNC rsspecan_NetworkUsedDrivesList(ViSession instrSession,
                                                 ViInt32   arraySize,
                                                 ViChar    usedDrives[]);

ViStatus _VI_FUNC rsspecan_RealtimeSpectrumAnalysisMode(ViSession instrSession);

ViStatus _VI_FUNC rsspecan_SelectRealtimeSpectrumMeasurement(ViSession instrSession,
                                                             ViInt32   realtimeMeasurement);

ViStatus _VI_FUNC rsspecan_SpectrogramMode(ViSession instrSession,
                                           ViBoolean state);

ViStatus _VI_FUNC rsspecan_ConfigureSpectrogramFFTWindow(ViSession instrSession,
                                                         ViInt32   FFTWindowType);

ViStatus _VI_FUNC rsspecan_SpectrogramWindow(ViSession instrSession,
                                             ViInt32   window);

ViStatus _VI_FUNC rsspecan_DwellTime(ViSession instrSession,
                                     ViBoolean autoEnabled,
                                     ViReal64  dwellTime);

ViStatus _VI_FUNC rsspecan_ConfigurePersistenceSpectrumState(ViSession instrSession,
                                                             ViBoolean state);

ViStatus _VI_FUNC rsspecan_ConfigurePersistenceSpectrum(ViSession instrSession,
                                                        ViInt32   style,
                                                        ViReal64  persistence,
                                                        ViReal64  granularity);

ViStatus _VI_FUNC rsspecan_ConfigurePersistenceSpectrumMaxholdIntensity(
                            ViSession instrSession,
                            ViInt32   reserved,
                            ViBoolean enableMaxhold,
                            ViInt32   intensity);

ViStatus _VI_FUNC rsspecan_ConfigurePersistenceSpectrumMaxholdReset(ViSession instrSession);

ViStatus _VI_FUNC rsspecan_ConfigurePersistenceSpectrumMarkerPosition(
                            ViSession instrSession,
                            ViInt32   marker,
                            ViInt32   markerPostion);

ViStatus _VI_FUNC rsspecan_QueryPersistenceMarkerZAxis(ViSession instrSession,
                                                       ViInt32   marker,
                                                       ViReal64* zAxis);

ViStatus _VI_FUNC rsspecan_ConfigurePersistenceSpectrumDeltaMarkerPosition(
                            ViSession instrSession,
                            ViInt32   marker,
                            ViInt32   deltaMarkerPostion);

ViStatus _VI_FUNC rsspecan_QueryPersistenceDeltaMarkerZAxis(ViSession instrSession,
                                                            ViInt32   deltaMarker,
                                                            ViReal64* zAxis);

ViStatus _VI_FUNC rsspecan_ConfigurePersistenceSpectrumDisplayParameters(
                            ViSession instrSession,
                            ViReal64  shape,
                            ViInt32   style,
                            ViReal64  colorLower,
                            ViReal64  colorUpper);

ViStatus _VI_FUNC rsspecan_ConfigurePersistenceSpectrumDefaultColors(ViSession instrSession);

ViStatus _VI_FUNC rsspecan_ConfigurePersistenceSpectrumTruncate(
                            ViSession instrSession,
                            ViBoolean truncate);

ViStatus _VI_FUNC rsspecan_ConfigureSpectrogramMeasurementParameters(
                            ViSession instrSession,
                            ViInt32   historyDepth,
                            ViBoolean timeStamp,
                            ViInt32   frameCount,
                            ViBoolean continueFrame);

ViStatus _VI_FUNC rsspecan_ConfigureSpectrogramSelectedFrame(ViSession instrSession,
                                                             ViInt32   frameNumber);

ViStatus _VI_FUNC rsspecan_GetSpectrogramTimeStamp(ViSession instrSession,
                                                   ViInt32   frame,
                                                   ViInt32   arraySize,
                                                   ViInt32   part1[],
                                                   ViInt32   part2[],
                                                   ViInt32   part3[],
                                                   ViInt32   part4[],
                                                   ViInt32*  returnedValues);

ViStatus _VI_FUNC rsspecan_SpectrogramClear(ViSession instrSession);

ViStatus _VI_FUNC rsspecan_ConfigureSpectrogramDisplayParameters(
                            ViSession instrSession,
                            ViInt32   color,
                            ViInt32   size);

ViStatus _VI_FUNC rsspecan_ConfigureSpectrogramColors(ViSession instrSession,
                                                      ViReal64  shape,
                                                      ViInt32   colorStyle,
                                                      ViReal64  colorLower,
                                                      ViReal64  colorUpper);

ViStatus _VI_FUNC rsspecan_ConfigureSpectrogramDefaultColors(ViSession instrSession);

ViStatus _VI_FUNC rsspecan_ConfigureSpectrogramMarkerFrameNumber(
                            ViSession instrSession,
                            ViInt32   markerNumber,
                            ViInt32   frameNumber);

ViStatus _VI_FUNC rsspecan_GetSpectrogramMarkerFrameNumber(ViSession instrSession,
                                                           ViInt32   markerNumber,
                                                           ViInt32*  frameNumber);

ViStatus _VI_FUNC rsspecan_ConfigureSpectrogramMarkerSearchArea(
                            ViSession instrSession,
                            ViInt32   markerNumber,
                            ViInt32   searchArea);

ViStatus _VI_FUNC rsspecan_SpectrogramMarkerSearch(ViSession instrSession,
                                                   ViInt32   markerNumber,
                                                   ViInt32   searchMode);

ViStatus _VI_FUNC rsspecan_ConfigureSpectrogramDeltaMarkerFrameNumber(
                            ViSession instrSession,
                            ViInt32   deltaMarkerNumber,
                            ViInt32   frameNumber);

ViStatus _VI_FUNC rsspecan_GetSpectrogramDeltaMarkerFrameNumber(
                            ViSession instrSession,
                            ViInt32   deltaMarker,
                            ViInt32*  frameNumber);

ViStatus _VI_FUNC rsspecan_ConfigureSpectrogramDeltaMarkerSearchArea(
                            ViSession instrSession,
                            ViInt32   deltaMarker,
                            ViInt32   searchArea);

ViStatus _VI_FUNC rsspecan_SpectrogramDeltaMarkerSearch(ViSession instrSession,
                                                        ViInt32   deltaMarker,
                                                        ViInt32   searchMode);

ViStatus _VI_FUNC rsspecan_ConfigureSpectrogramResultDisplayMode(
                            ViSession instrSession,
                            ViInt32   resultDisplay);

ViStatus _VI_FUNC rsspecan_ReadSpectrogramTraceData(ViSession instrSession,
                                                    ViInt32   arraySize,
                                                    ViReal64  results[],
                                                    ViInt32*  returnedValues);

ViStatus _VI_FUNC rsspecan_ReadPersistenceSpectrumTraceData(ViSession instrSession,
                                                            ViUInt32  timeout,
                                                            ViInt32   arraySize,
                                                            ViReal64  results[],
                                                            ViInt32*  returnedValues);

ViStatus _VI_FUNC rsspecan_ReadMaxholdTraceData(ViSession instrSession,
                                                ViUInt32  timeout,
                                                ViInt32   arraySize,
                                                ViReal64  results[],
                                                ViInt32*  returnedValues);

ViStatus _VI_FUNC rsspecan_GetSpectrogramXAxisScaling(ViSession instrSession,
                                                      ViInt32   frame,
                                                      ViInt32   arraySize,
                                                      ViReal64  xAxis[],
                                                      ViInt32*  returnedValues);

ViStatus _VI_FUNC rsspecan_StoreSpectrogramToFile(ViSession instrSession,
                                                  ViString  fileName);

ViStatus _VI_FUNC rsspecan_ConfigureSpectrogramContinuousTrigger(
                            ViSession instrSession,
                            ViInt32   trigger);

ViStatus _VI_FUNC rsspecan_ConfigureSpectrogramTimeDomainTrigger(
                            ViSession instrSession,
                            ViReal64  timeDomainTriggerLevel);

ViStatus _VI_FUNC rsspecan_ConfigureExternalMixerEnabled(ViSession instrSession,
                                                         ViBoolean externalMixingEnabled);

ViStatus _VI_FUNC rsspecan_ConfigureExternalMixer(ViSession instrSession,
                                                  ViInt32   band,
                                                  ViInt32   harmonic,
                                                  ViInt32   harmonicType);

ViStatus _VI_FUNC rsspecan_ConfigureExternalMixerXCORR(ViSession instrSession,
                                                       ViBoolean mixingXCORR);

ViStatus _VI_FUNC rsspecan_ConfigureExternalMixerNumberOfPorts(
                            ViSession instrSession,
                            ViInt32   numberOfPorts);

ViStatus _VI_FUNC rsspecan_ConfigureExternalMixerHandoverFrequency(
                            ViSession instrSession,
                            ViReal64  handoverFrequency);

ViStatus _VI_FUNC rsspecan_ConfigureExternalMixerRFOverrange(ViSession instrSession,
                                                             ViBoolean RFOverrange);

ViStatus _VI_FUNC rsspecan_ConfigureExternalMixerPresetBand(ViSession instrSession);

ViStatus _VI_FUNC rsspecan_ConfigureExternalMixerHighHarmonic(
                            ViSession instrSession,
                            ViBoolean highHarmonic,
                            ViInt32   harmonicOrder);

ViStatus _VI_FUNC rsspecan_ConfigureExternalMixerSignalIDMode(
                            ViSession instrSession,
                            ViInt32   mode,
                            ViReal64  autoIDThreshold);

ViStatus _VI_FUNC rsspecan_ConfigureExternalMixerBias(ViSession instrSession,
                                                      ViInt32   range,
                                                      ViReal64  bias);

ViStatus _VI_FUNC rsspecan_ConfigureExternalMixerLowPower(ViSession instrSession,
                                                          ViReal64  lowPower);

ViStatus _VI_FUNC rsspecan_ConfigureExternalMixerConversionLoss(
                            ViSession instrSession,
                            ViInt32   range,
                            ViInt32   conversionLossMode,
                            ViReal64  conversionLoss,
                            ViString  conversionLossTable);

ViStatus _VI_FUNC rsspecan_CreateExternalMixerConversionLossTable(
                            ViSession instrSession,
                            ViInt32   reserved,
                            ViString  tableName,
                            ViString  mixerType,
                            ViInt32   band,
                            ViString  serialNumber,
                            ViInt32   harmonic,
                            ViInt32   ports,
                            ViReal64  bias,
                            ViString  comment,
                            ViInt32   numberOfValues,
                            ViReal64  frequencyValues[],
                            ViReal64  conversionLossValues[]);

ViStatus _VI_FUNC rsspecan_ExternalMixerConversionLossTableSelect(
                            ViSession instrSession,
                            ViInt32   reserved,
                            ViString  tableName);

ViStatus _VI_FUNC rsspecan_ExternalMixerConversionLossTableDelete(
                            ViSession instrSession,
                            ViInt32   reserved,
                            ViString  tableName);

ViStatus _VI_FUNC rsspecan_ExternalMixerConversionLossTableCatalog(
                            ViSession instrSession,
                            ViInt32*  numberOfCVL,
                            ViInt32   bufferSize,
                            ViChar    CVLNamesList[]);

ViStatus _VI_FUNC rsspecan_QueryExternalMixerRFStartStop(ViSession instrSession,
                                                         ViReal64* RFStart,
                                                         ViReal64* RFStop);

ViStatus _VI_FUNC rsspecan_ConfigureExternalFrontendEnabled(ViSession instrSession,
                                                            ViBoolean externalFrontend);

ViStatus _VI_FUNC rsspecan_ConfigureExternalFrontendConnectionSettings(
                            ViSession instrSession,
                            ViString  type,
                            ViString  IPAddress,
                            ViString  deviceID,
                            ViString  symbolicName);

ViStatus _VI_FUNC rsspecan_ConfigureExternalFrontendConnectionFirmwareState(
                            ViSession instrSession,
                            ViBoolean firmwareConnection);

ViStatus _VI_FUNC rsspecan_QueryExternalFrontendConnectionPhysicalState(
                            ViSession  instrSession,
                            ViBoolean* physicalConnection);

ViStatus _VI_FUNC rsspecan_ConfigureExternalFrontendNetworkSettings(
                            ViSession instrSession,
                            ViString  IPAddress,
                            ViString  subnet,
                            ViBoolean DHCP);

ViStatus _VI_FUNC rsspecan_QueryExternalFrontendIdentification(
                            ViSession instrSession,
                            ViInt32   bufferSize,
                            ViChar    identification[],
                            ViInt32*  bytesRead);

ViStatus _VI_FUNC rsspecan_QueryExternalFrontendFrequencyBandRangeList(
                            ViSession instrSession,
                            ViInt32   bufferSize,
                            ViChar    rangeList[],
                            ViInt32*  bytesRead);

ViStatus _VI_FUNC rsspecan_ConfigureExternalFrontendFrequencyBandRange(
                            ViSession instrSession,
                            ViString  range);

ViStatus _VI_FUNC rsspecan_ConfigureExternalFrontendFrequencyBandAutoEnabled(
                            ViSession instrSession,
                            ViBoolean autoFrequencyBand);

ViStatus _VI_FUNC rsspecan_QueryExternalFrontendFrequencyBandCount(
                            ViSession instrSession,
                            ViInt32*  frequencyBandCount);

ViStatus _VI_FUNC rsspecan_QueryExternalFrontendFrequencyBandRange(
                            ViSession instrSession,
                            ViInt32   frequencyBand,
                            ViReal64* lower,
                            ViReal64* upper);

ViStatus _VI_FUNC rsspecan_QueryExternalFrontendIntermediateFrequencySideband(
                            ViSession instrSession,
                            ViInt32   bufferSize,
                            ViChar    frequencySideband[],
                            ViInt32*  bytesRead);

ViStatus _VI_FUNC rsspecan_QueryExternalFrontendIntermediateFrequency(
                            ViSession instrSession,
                            ViReal64* intermediateFrequency);

ViStatus _VI_FUNC rsspecan_QueryExternalFrontendReferenceFrequencyList(
                            ViSession instrSession,
                            ViInt32   bufferSize,
                            ViChar    referenceFrequencyList[],
                            ViInt32*  bytesRead);

ViStatus _VI_FUNC rsspecan_ConfigureExternalFrontendReferenceFrequency(
                            ViSession instrSession,
                            ViReal64  referenceFrequency);

ViStatus _VI_FUNC rsspecan_ExternalFrontendUpdateFirmware(ViSession instrSession);

ViStatus _VI_FUNC rsspecan_ConfigureExternalFrontendAlignmentCorrection(
                            ViSession instrSession,
                            ViBoolean correction,
                            ViString  correctionFile);

ViStatus _VI_FUNC rsspecan_PerformExternalFrontendDiagnosticsSelftest(
                            ViSession instrSession,
                            ViInt32*  selftestResult);

ViStatus _VI_FUNC rsspecan_QueryExternalFrontendDiagnosticsSelftestResults(
                            ViSession instrSession,
                            ViInt32   bufferSize,
                            ViChar    results[],
                            ViInt32*  bytesRead);

ViStatus _VI_FUNC rsspecan_QueryExternalFrontendDeviceName(ViSession instrSession,
                                                           ViInt32   bufferSize,
                                                           ViChar    deviceName[],
                                                           ViInt32*  bytesRead);

ViStatus _VI_FUNC rsspecan_QueryExternalFrontendDeviceNameList(
                            ViSession instrSession,
                            ViInt32   bufferSize,
                            ViChar    nameList[],
                            ViInt32*  bytesRead);

ViStatus _VI_FUNC rsspecan_QueryExternalFrontendDeviceType(ViSession instrSession,
                                                           ViInt32   bufferSize,
                                                           ViChar    type[],
                                                           ViInt32*  bytesRead);

ViStatus _VI_FUNC rsspecan_QueryExternalFrontendDeviceFrequencyRange(
                            ViSession instrSession,
                            ViReal64* lower,
                            ViReal64* upper);

ViStatus _VI_FUNC rsspecan_QueryExternalFrontendDeviceGain(ViSession instrSession,
                                                           ViReal64* gain);

ViStatus _VI_FUNC rsspecan_ExternalFrontendDeviceRefresh(ViSession instrSession);

ViStatus _VI_FUNC rsspecan_ConfigureExternalFrontendDeviceCorrection(
                            ViSession instrSession,
                            ViBoolean correction,
                            ViString  correctionFile);

ViStatus _VI_FUNC rsspecan_QueryExternalFrontendDeviceCorrectionFileValid(
                            ViSession  instrSession,
                            ViBoolean* fileValidity);

ViStatus _VI_FUNC rsspecan_ConfigureAnalogBasebandTrigger(ViSession instrSession,
                                                          ViInt32   triggerSource);

ViStatus _VI_FUNC rsspecan_ConfigureBasebandPowerTriggerLevel(
                            ViSession instrSession,
                            ViReal64  triggerLevel);

ViStatus _VI_FUNC rsspecan_ConfigureAnalogBasebandInput(ViSession instrSession,
                                                        ViInt32   basebandInputs);

ViStatus _VI_FUNC rsspecan_ConfigureAnalogBasebandInputParameters(
                            ViSession instrSession,
                            ViInt32   iQMode,
                            ViBoolean inputConfiguration,
                            ViBoolean swapIQ);

ViStatus _VI_FUNC rsspecan_ConfigureAnalogBasebandFullscaleLevel(
                            ViSession instrSession,
                            ViBoolean mode,
                            ViReal64  level);

ViStatus _VI_FUNC rsspecan_ConfigureAnalogBasebandCalibrationSignalType(
                            ViSession instrSession,
                            ViInt32   signalType);

ViStatus _VI_FUNC rsspecan_ConfigureAnalogBasebandSignalPath(ViSession instrSession,
                                                             ViInt32   path);

ViStatus _VI_FUNC rsspecan_ConfigureAnalogBasebandSwapIQ(ViSession instrSession,
                                                         ViBoolean swapIQ);

ViStatus _VI_FUNC rsspecan_ConfigureAnalogBasebandHighAccuracyTiming(
                            ViSession instrSession,
                            ViBoolean highAccuracyTiming);

ViStatus _VI_FUNC rsspecan_ConfigureAnalogBasebandProbeCommonModeOffset(
                            ViSession instrSession,
                            ViInt32   probeConnector,
                            ViReal64  CMOffset);

ViStatus _VI_FUNC rsspecan_ConfigureAnalogBasebandDCOffset(ViSession instrSession,
                                                           ViInt32   iQMode,
                                                           ViReal64  DCOffset);

ViStatus _VI_FUNC rsspecan_ConfigureAnalogBasebandInputImpedance(
                            ViSession instrSession,
                            ViReal64  impedance,
                            ViInt32   padType);

ViStatus _VI_FUNC rsspecan_ConfigureAnalogBasebandInputAttenuationIQInterface(
                            ViSession instrSession,
                            ViInt32   IQInterface);

ViStatus _VI_FUNC rsspecan_ConfigureAnalogBasebandYIGFilterEnabled(
                            ViSession instrSession,
                            ViBoolean enabled);

ViStatus _VI_FUNC rsspecan_ConfigureDigitalBasebandInput(ViSession instrSession,
                                                         ViInt32   basebandInputs);

ViStatus _VI_FUNC rsspecan_ConfigureDigitalBasebandInputCoupling(
                            ViSession instrSession,
                            ViBoolean basebandInputCoupling);

ViStatus _VI_FUNC rsspecan_ConfigureDigitalBasebandInputParametersAuto(
                            ViSession instrSession,
                            ViBoolean autoRange,
                            ViBoolean autoSampleRate);

ViStatus _VI_FUNC rsspecan_ConfigureDigitalBasebandInputParameters(
                            ViSession instrSession,
                            ViReal64  range,
                            ViReal64  sampleRate);

ViStatus _VI_FUNC rsspecan_ConfigureDigitalBasebandDecimationFilter(
                            ViSession instrSession,
                            ViBoolean filter);

ViStatus _VI_FUNC rsspecan_ConfigureDigitalBasebandOutput(ViSession instrSession,
                                                          ViBoolean output);

ViStatus _VI_FUNC rsspecan_QueryDigitalBasebandInputOutputConfiguration(
                            ViSession instrSession,
                            ViInt32   digitalBasebandSelect,
                            ViInt32   arraySize,
                            ViChar    configuration[]);

ViStatus _VI_FUNC rsspecan_ExternalGeneratorNormalization(ViSession instrSession,
                                                          ViInt32   reserved,
                                                          ViBoolean state);

ViStatus _VI_FUNC rsspecan_ExternalGeneratorCalibration(ViSession instrSession,
                                                        ViInt32   reserved,
                                                        ViInt32   method,
                                                        ViUInt32  timeout);

ViStatus _VI_FUNC rsspecan_ExternalGeneratorRecall(ViSession instrSession,
                                                   ViInt32   reserved);

ViStatus _VI_FUNC rsspecan_ExternalGeneratorSaveAsTrdFactor(ViSession instrSession,
                                                            ViInt32   reserved,
                                                            ViString  name);

ViStatus _VI_FUNC rsspecan_ExternalGeneratorReference(ViSession instrSession,
                                                      ViReal64  referencePosition,
                                                      ViReal64  referenceValue);

ViStatus _VI_FUNC rsspecan_ConfigureExternalGeneratorGPIB(ViSession instrSession,
                                                          ViInt32   generator,
                                                          ViInt32   generatorAddress,
                                                          ViInt32   TTLHandshake,
                                                          ViString  generatorType,
                                                          ViInt32   reference);

ViStatus _VI_FUNC rsspecan_ConfigureExternalGeneratorTCPIP(ViSession instrSession,
                                                           ViInt32   generator,
                                                           ViString  generatorAddress,
                                                           ViString  generatorType,
                                                           ViInt32   reference);

ViStatus _VI_FUNC rsspecan_ConfigureExternalGeneratorMeasurement(
                            ViSession instrSession,
                            ViInt32   generator,
                            ViBoolean state,
                            ViReal64  power,
                            ViReal64  offset);

ViStatus _VI_FUNC rsspecan_ConfigureExternalGeneratorManualFrequency(
                            ViSession instrSession,
                            ViInt32   generator,
                            ViReal64  frequency);

ViStatus _VI_FUNC rsspecan_ConfigureExternalGeneratorAutomaticFrequency(
                            ViSession instrSession,
                            ViInt32   generator,
                            ViBoolean frequencySweep,
                            ViReal64  frequencyOffset,
                            ViInt32   denominator,
                            ViInt32   numerator);

ViStatus _VI_FUNC rsspecan_ConfigureExternalGain(ViSession instrSession,
                                                 ViInt32   reserved,
                                                 ViReal64  gain);

ViStatus _VI_FUNC rsspecan_ConfigureExternalGainPACorrectionEnabled(
                            ViSession instrSession,
                            ViBoolean PACorrectionEnabled);

ViStatus _VI_FUNC rsspecan_SendCommandToExternalGenerator(ViSession instrSession,
                                                          ViInt32   deviceAddress,
                                                          ViString  commandString);

ViStatus _VI_FUNC rsspecan_SetTrackingGeneratorState(ViSession instrSession,
                                                     ViInt32   reserved,
                                                     ViBoolean state);

ViStatus _VI_FUNC rsspecan_ConfigureTrackingGenerator(ViSession instrSession,
                                                      ViInt32   reserved,
                                                      ViReal64  sourcePower,
                                                      ViReal64  powerOffset,
                                                      ViReal64  frequencyOffset);

ViStatus _VI_FUNC rsspecan_ConfigureTrackingGeneratorExternalModulation(
                            ViSession instrSession,
                            ViInt32   reserved,
                            ViInt32   modulation,
                            ViBoolean modState,
                            ViReal64  FMDeviation);

ViStatus _VI_FUNC rsspecan_ConfigureTrackingGeneratorPowerSweep(
                            ViSession instrSession,
                            ViInt32   reserved,
                            ViInt32   powerSweep,
                            ViReal64  powerStart,
                            ViReal64  powerStop);

ViStatus _VI_FUNC rsspecan_PulseMeasurementMode(ViSession instrSession);

ViStatus _VI_FUNC rsspecan_ConfigurePulseSignalModel(ViSession instrSession,
                                                     ViInt32   pulsePeriod,
                                                     ViBoolean pulseHasDroop,
                                                     ViInt32   pulseModulation);

ViStatus _VI_FUNC rsspecan_ConfigurePulseSignalTimingAuto(ViSession instrSession);

ViStatus _VI_FUNC rsspecan_ConfigurePulseSignalTimingManual(ViSession instrSession,
                                                            ViReal64  minPulseWidth,
                                                            ViReal64  maxPulseWidth,
                                                            ViReal64  minPulseOffTime);

ViStatus _VI_FUNC rsspecan_ConfigurePulseSignalFrequencyOffset(
                            ViSession instrSession,
                            ViBoolean autoMode,
                            ViReal64  frequencyOffsetValue);

ViStatus _VI_FUNC rsspecan_ConfigurePulseSignalChirpRate(ViSession instrSession,
                                                         ViBoolean autoMode,
                                                         ViReal64  chirpRate);

ViStatus _VI_FUNC rsspecan_ConfigurePulseReferenceType(ViSession instrSession,
                                                       ViInt32   type);

ViStatus _VI_FUNC rsspecan_ConfigurePulseUserDefinedReferenceIQFile(
                            ViSession instrSession,
                            ViInt32   type,
                            ViString  inputFile);

ViStatus _VI_FUNC rsspecan_ConfigurePulseUserDefinedReferenceIQFileRange(
                            ViSession instrSession,
                            ViBoolean rangeAuto,
                            ViReal64  offset,
                            ViReal64  length);

ViStatus _VI_FUNC rsspecan_ConfigurePulseUserDefinedReferenceIQFileWindow(
                            ViSession instrSession,
                            ViInt32   window);

ViStatus _VI_FUNC rsspecan_ConfigurePulsePolynominaPhaseReferenceWaveform(
                            ViSession instrSession,
                            ViInt32   window,
                            ViReal64  pulseWidth,
                            ViInt32   arraySize,
                            ViInt32   coefficients[]);

ViStatus _VI_FUNC rsspecan_ConfigurePulseBarkerReferenceWaveform(
                            ViSession instrSession,
                            ViReal64  pulseWidth,
                            ViInt32   primaryCode);

ViStatus _VI_FUNC rsspecan_ConfigurePulseEmbeddedBarkerReferenceWaveform(
                            ViSession instrSession,
                            ViReal64  pulseWidth,
                            ViInt32   primaryCode,
                            ViInt32   secondaryCode);

ViStatus _VI_FUNC rsspecan_ConfigurePulseInputSignalSource(ViSession instrSession,
                                                           ViInt32   source);

ViStatus _VI_FUNC rsspecan_PulseSelectIQDataFile(ViSession instrSession,
                                                 ViString  fileName);

ViStatus _VI_FUNC rsspecan_ConfigurePulseSignalOutput(ViSession instrSession,
                                                      ViBoolean signalOutput);

ViStatus _VI_FUNC rsspecan_ConfigurePulseBurstMode(ViSession instrSession,
                                                   ViBoolean burstMode);

ViStatus _VI_FUNC rsspecan_ConfigurePulseBurstData(ViSession instrSession,
                                                   ViInt32   arraySize,
                                                   ViReal64  pulseWidth[],
                                                   ViReal64  pulsePeriod[],
                                                   ViInt32   numberOfReps[],
                                                   ViReal64  burstLength[]);

ViStatus _VI_FUNC rsspecan_QueryPulseBurstCount(ViSession instrSession,
                                                ViInt32*  burstCount);

ViStatus _VI_FUNC rsspecan_DeletePulseBurstTable(ViSession instrSession);

ViStatus _VI_FUNC rsspecan_SavePulseBurstTable(ViSession instrSession,
                                               ViString  filename);

ViStatus _VI_FUNC rsspecan_LoadPulseBurstTable(ViSession instrSession,
                                               ViString  filename);

ViStatus _VI_FUNC rsspecan_ConfigurePulseBurstInternalTrigger(
                            ViSession instrSession,
                            ViInt32   triggerPosition,
                            ViInt32   burstNumber);

ViStatus _VI_FUNC rsspecan_ConfigurePulseSegmentDataCapturingEnabled(
                            ViSession instrSession,
                            ViBoolean state);

ViStatus _VI_FUNC rsspecan_ConfigurePulseSegmentDataCapturing(
                            ViSession instrSession,
                            ViInt32   events,
                            ViReal64  length,
                            ViReal64  offset);

ViStatus _VI_FUNC rsspecan_ConfigurePulseMeasurementAcquisition(
                            ViSession instrSession,
                            ViInt32   filterType,
                            ViReal64  measurementBandwidth,
                            ViReal64  measurementTime);

ViStatus _VI_FUNC rsspecan_ConfigurePulseMeasurementAcquisitionDigitizer(
                            ViSession instrSession,
                            ViInt32   digitizerSelection,
                            ViInt32   lowNoiseMode);

ViStatus _VI_FUNC rsspecan_ConfigurePulseMeasurementFMVideoBandwidth(
                            ViSession instrSession,
                            ViInt32   FMVideoBandwidth);

ViStatus _VI_FUNC rsspecan_ConfigurePulseMeasurementDetection(
                            ViSession instrSession,
                            ViInt32   referenceSource,
                            ViReal64  threshold,
                            ViReal64  hysteresis);

ViStatus _VI_FUNC rsspecan_ConfigurePulseMeasurementDetectionSettings(
                            ViSession instrSession,
                            ViBoolean detection,
                            ViReal64  detectionStart,
                            ViReal64  detectionLength);

ViStatus _VI_FUNC rsspecan_ConfigurePulseMeasurementDetectionLimit(
                            ViSession instrSession,
                            ViBoolean detectionLimit,
                            ViInt32   maximumPulseCount);

ViStatus _VI_FUNC rsspecan_ConfigurePulseMarkerLinkTrendM1ToSelectedPulse(
                            ViSession instrSession,
                            ViBoolean linkState);

ViStatus _VI_FUNC rsspecan_ConfigurePulseMarkerLink(ViSession instrSession,
                                                    ViBoolean linkState);

ViStatus _VI_FUNC rsspecan_ConfigurePulseDeltamarkerLinkToMarker(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   deltamarker,
                            ViInt32   marker,
                            ViBoolean linkState);

ViStatus _VI_FUNC rsspecan_ConfigurePulseMarkerLinkToMarker(ViSession instrSession,
                                                            ViInt32   window,
                                                            ViInt32   marker1,
                                                            ViInt32   marker2,
                                                            ViBoolean linkState);

ViStatus _VI_FUNC rsspecan_ConfigurePulseMarkerLabelsForPeaks(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   marker,
                            ViBoolean labelsForPeaks);

ViStatus _VI_FUNC rsspecan_ConfigurePulseMesurementTraceNormalization(
                            ViSession instrSession,
                            ViInt32   normalization);

ViStatus _VI_FUNC rsspecan_ConfigurePulseMeasurementTraceResults(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   trace,
                            ViInt32   results);

ViStatus _VI_FUNC rsspecan_ConfigurePulseMesurementTraceEvaluation(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   trace,
                            ViInt32   evaluation);

ViStatus _VI_FUNC rsspecan_ConfigurePulseMeasurementTraceStabilityEvaluation(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   trace,
                            ViInt32   stabilityEvaluation);

ViStatus _VI_FUNC rsspecan_ConfigurePulseMesurementTraceIQDetector(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   trace,
                            ViInt32   detector);

ViStatus _VI_FUNC rsspecan_ConfigurePulseMesurementTraceStatisticType(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   type);

ViStatus _VI_FUNC rsspecan_PulseMeasurementAddWindow(ViSession instrSession,
                                                     ViInt32   stringSize,
                                                     ViString  referenceWindowName,
                                                     ViInt32   direction,
                                                     ViInt32   windowType,
                                                     ViChar    returnedWindowName[]);

ViStatus _VI_FUNC rsspecan_PulseMeasurementReplaceWindow(ViSession instrSession,
                                                         ViString  windowName,
                                                         ViInt32   windowType);

ViStatus _VI_FUNC rsspecan_ConfigurePulseMeasPoint(ViSession instrSession,
                                                   ViInt32   reference,
                                                   ViReal64  offset,
                                                   ViReal64  averagingWindow);

ViStatus _VI_FUNC rsspecan_ConfigurePulseReferenceForPulsePulseMeasurement(
                            ViSession instrSession,
                            ViInt32   mode,
                            ViInt32   pulse);

ViStatus _VI_FUNC rsspecan_ConfigurePulseMeasurementRange(ViSession instrSession,
                                                          ViInt32   reference,
                                                          ViReal64  length,
                                                          ViReal64  riseOffset,
                                                          ViReal64  fallOffset);

ViStatus _VI_FUNC rsspecan_ConfigurePulseTimeSidelobeRange(ViSession instrSession,
                                                           ViInt32   range,
                                                           ViInt32   alignment,
                                                           ViReal64  length);

ViStatus _VI_FUNC rsspecan_ConfigurePulseTimeSidelobeKeepOutTime(
                            ViSession instrSession,
                            ViBoolean autoMode,
                            ViReal64  length);

ViStatus _VI_FUNC rsspecan_ConfigurePulseStabilityBurstStatistics(
                            ViSession instrSession,
                            ViInt32   mode,
                            ViInt32   lengthPulses,
                            ViInt32   selectedPulse);

ViStatus _VI_FUNC rsspecan_ConfigurePulseStabilityBurstAnalysis(
                            ViSession instrSession,
                            ViInt32   referenceStart,
                            ViInt32   analysisStart,
                            ViInt32   pulsePulseOffset,
                            ViInt32   frequencyOffsetCompensation);

ViStatus _VI_FUNC rsspecan_ConfigurePulseTopLevel(ViSession instrSession,
                                                  ViInt32   position,
                                                  ViInt32   measurementAlgorithm,
                                                  ViReal64  ripplePortion);

ViStatus _VI_FUNC rsspecan_ConfigurePulseTopLevelFixed(ViSession instrSession,
                                                       ViReal64  levelValue);

ViStatus _VI_FUNC rsspecan_ConfigurePulseReferenceLevel(ViSession instrSession,
                                                        ViInt32   unit,
                                                        ViReal64  highThreshold,
                                                        ViReal64  middleThreshold,
                                                        ViReal64  lowerThreshold,
                                                        ViReal64  boundary);

ViStatus _VI_FUNC rsspecan_ConfigurePulseSelection(ViSession instrSession,
                                                   ViInt32   pulseNumber);

ViStatus _VI_FUNC rsspecan_ConfigurePulseResultAutomaticRangeScaling(
                            ViSession instrSession,
                            ViInt32   automaticRangeScaling);

ViStatus _VI_FUNC rsspecan_ConfigurePulseResultReferencePoint(
                            ViSession instrSession,
                            ViInt32   reference,
                            ViReal64  offset);

ViStatus _VI_FUNC rsspecan_ConfigurePulseResultRange(ViSession instrSession,
                                                     ViInt32   alignment,
                                                     ViReal64  length);

ViStatus _VI_FUNC rsspecan_ConfigurePulseResultRangeSpectrum(ViSession instrSession,
                                                             ViInt32   window,
                                                             ViInt32   windowType,
                                                             ViBoolean RBWAuto,
                                                             ViReal64  RBW);

ViStatus _VI_FUNC rsspecan_ConfigurePulsePhaseUnit(ViSession instrSession,
                                                   ViInt32   unit);

ViStatus _VI_FUNC rsspecan_ConfigurePulsePhaseNormalization(ViSession instrSession,
                                                            ViInt32   window,
                                                            ViReal64  normalization);

ViStatus _VI_FUNC rsspecan_ConfigurePulseFrequencyUnit(ViSession instrSession,
                                                       ViInt32   unit);

ViStatus _VI_FUNC rsspecan_ConfigurePulsePulseSettings(ViSession instrSession,
                                                       ViInt32   window,
                                                       ViBoolean autoSettings,
                                                       ViReal64  gapThreshold,
                                                       ViReal64  minimumSectionThreshold,
                                                       ViReal64  maximumFrequency);

ViStatus _VI_FUNC rsspecan_ConfigurePulsePulseWindowType(ViSession instrSession,
                                                         ViInt32   window,
                                                         ViInt32   windowType);

ViStatus _VI_FUNC rsspecan_QueryPulsePulseResolutionBandwidth(
                            ViSession instrSession,
                            ViInt32   window,
                            ViReal64* resolutionBandwidth);

ViStatus _VI_FUNC rsspecan_ConfigurePulseResultParameterTrendPower(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   group,
                            ViInt32   axis);

ViStatus _VI_FUNC rsspecan_ConfigurePulseResultParameterTrendTiming(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   group,
                            ViInt32   axis);

ViStatus _VI_FUNC rsspecan_ConfigurePulseResultParameterTrendFrequency(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   group,
                            ViInt32   axis);

ViStatus _VI_FUNC rsspecan_ConfigurePulseResultParameterTrendPhase(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   group,
                            ViInt32   axis);

ViStatus _VI_FUNC rsspecan_ConfigurePulseResultParameterTrendLimitLines(
                            ViSession instrSession,
                            ViInt32   window,
                            ViBoolean state);

ViStatus _VI_FUNC rsspecan_ConfigurePulseResultParameterTrendDisplayStyle(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   displayStyle);

ViStatus _VI_FUNC rsspecan_ConfigurePulseResultParameterTrendTimeSidelobe(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   group,
                            ViInt32   axis);

ViStatus _VI_FUNC rsspecan_ConfigurePulseResultParameterTrendEnvelopeModel(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   group,
                            ViInt32   axis);

ViStatus _VI_FUNC rsspecan_ConfigurePulseResultParameterTrendStability(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   group,
                            ViInt32   axis);

ViStatus _VI_FUNC rsspecan_ConfigurePulseResultParameterDistributionBins(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   numberOfBins);

ViStatus _VI_FUNC rsspecan_ConfigurePulseResultParameterDistributionPower(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   xAxis,
                            ViInt32   yAxis);

ViStatus _VI_FUNC rsspecan_ConfigurePulseResultParameterDistributionTiming(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   xAxis,
                            ViInt32   yAxis);

ViStatus _VI_FUNC rsspecan_ConfigurePulseResultParameterDistributionFrequency(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   xAxis,
                            ViInt32   yAxis);

ViStatus _VI_FUNC rsspecan_ConfigurePulseResultParameterDistributionPhase(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   xAxis,
                            ViInt32   yAxis);

ViStatus _VI_FUNC rsspecan_ConfigurePulseResultParameterDistributionLimitLines(
                            ViSession instrSession,
                            ViInt32   window,
                            ViBoolean state);

ViStatus _VI_FUNC rsspecan_ConfigurePulseResultParameterDistributionTimeSidelobe(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   xAxis,
                            ViInt32   yAxis);

ViStatus _VI_FUNC rsspecan_ConfigurePulseResultParameterDistributionEnvelopeModel(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   xAxis,
                            ViInt32   yAxis);

ViStatus _VI_FUNC rsspecan_ConfigurePulseResultParameterDistributionStability(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   xAxis,
                            ViInt32   yAxis);

ViStatus _VI_FUNC rsspecan_ConfigurePulseResultParameterSpectrumFullAuto(
                            ViSession instrSession,
                            ViInt32   window);

ViStatus _VI_FUNC rsspecan_ConfigurePulseResultParameterSpectrumManual(
                            ViSession instrSession,
                            ViInt32   window,
                            ViReal64  maximumFrequency,
                            ViInt32   windowType,
                            ViInt32   blockSize,
                            ViReal64  gapThreshold,
                            ViReal64  segmentThreshold);

ViStatus _VI_FUNC rsspecan_ConfigurePulseResultParameterSpectrumFrequency(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   parameter);

ViStatus _VI_FUNC rsspecan_ConfigurePulseResultParameterSpectrumPhase(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   parameter);

ViStatus _VI_FUNC rsspecan_ConfigurePulseResultParameterSpectrumPower(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   parameter);

ViStatus _VI_FUNC rsspecan_ConfigurePulseResultParameterSpectrumTiming(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   parameter);

ViStatus _VI_FUNC rsspecan_ConfigurePulseResultParameterSpectrumTimeSidelobe(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   parameter);

ViStatus _VI_FUNC rsspecan_ConfigurePulseResultParameterSpectrumEnvelopeModel(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   parameter);

ViStatus _VI_FUNC rsspecan_ConfigurePulseParameterSpectrumStability(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   parameter);

ViStatus _VI_FUNC rsspecan_ConfigurePulseResultParameterStabilityWaterfallZAxis(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   stabilityWaterfallZAxis);

ViStatus _VI_FUNC rsspecan_ConfigurePulseResultParameterStabilityWaterfallDisplay(
                            ViSession instrSession,
                            ViInt32   window,
                            ViBoolean xAxisGridLines,
                            ViBoolean yAxisGridLines,
                            ViBoolean measurementInfo,
                            ViBoolean zAxisMeasurementPlane);

ViStatus _VI_FUNC rsspecan_ConfigurePulseResultParameterStabilityWaterfallLinkWindows(
                            ViSession instrSession,
                            ViInt32   window,
                            ViBoolean cameraAngle,
                            ViBoolean zAxisScale);

ViStatus _VI_FUNC rsspecan_ConfigurePulseResultTablePower(ViSession instrSession,
                                                          ViInt32   window,
                                                          ViInt32   parameter,
                                                          ViBoolean visibility);

ViStatus _VI_FUNC rsspecan_ConfigurePulseResultTableTiming(ViSession instrSession,
                                                           ViInt32   window,
                                                           ViInt32   parameter,
                                                           ViBoolean visibility,
                                                           ViInt32   reserved);

ViStatus _VI_FUNC rsspecan_ConfigurePulseResultTableFrequency(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   parameter,
                            ViBoolean visibility,
                            ViInt32   reserved);

ViStatus _VI_FUNC rsspecan_ConfigurePulseResultTablePhase(ViSession instrSession,
                                                          ViInt32   window,
                                                          ViInt32   parameter,
                                                          ViBoolean visibility);

ViStatus _VI_FUNC rsspecan_ConfigurePulseResultTableTimeSidelobe(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   parameter,
                            ViBoolean visibility,
                            ViInt32   reserved);

ViStatus _VI_FUNC rsspecan_ConfigurePulseResultTableEnvelopeModel(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   parameter,
                            ViBoolean visibility);

ViStatus _VI_FUNC rsspecan_ConfigurePulseResultTableStability(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   parameter,
                            ViBoolean visibility);

ViStatus _VI_FUNC rsspecan_ExportPulseResultTable(ViSession instrSession,
                                                  ViInt32   window,
                                                  ViInt32   exportColumns,
                                                  ViString  filename);

ViStatus _VI_FUNC rsspecan_ConfigurePulseResultTableLimitCheckAll(
                            ViSession instrSession,
                            ViInt32   window,
                            ViBoolean state);

ViStatus _VI_FUNC rsspecan_ConfigurePulseResultTablePowerLimitCheckAll(
                            ViSession instrSession,
                            ViInt32   window,
                            ViBoolean state);

ViStatus _VI_FUNC rsspecan_ConfigurePulseResultTableTimingLimitCheckAll(
                            ViSession instrSession,
                            ViInt32   window,
                            ViBoolean state);

ViStatus _VI_FUNC rsspecan_ConfigurePulseResultTableFrequencyLimitCheckAll(
                            ViSession instrSession,
                            ViInt32   window,
                            ViBoolean state);

ViStatus _VI_FUNC rsspecan_ConfigurePulseResultTablePhaseLimitCheckAll(
                            ViSession instrSession,
                            ViInt32   window,
                            ViBoolean state);

ViStatus _VI_FUNC rsspecan_ConfigurePulseResultTableTimeSidelobeLimitCheckAll(
                            ViSession instrSession,
                            ViInt32   window,
                            ViBoolean state);

ViStatus _VI_FUNC rsspecan_ConfigurePulseResultTableEnvelopeModelLimitCheckAll(
                            ViSession instrSession,
                            ViInt32   window,
                            ViBoolean state);

ViStatus _VI_FUNC rsspecan_ConfigurePulseResultTableStabilityLimitCheckAll(
                            ViSession instrSession,
                            ViInt32   window,
                            ViBoolean state);

ViStatus _VI_FUNC rsspecan_ConfigurePulseResultTablePowerLimit(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   parameter,
                            ViBoolean state,
                            ViReal64  lowerLimit,
                            ViReal64  upperLimit);

ViStatus _VI_FUNC rsspecan_ConfigurePulseResultTableTimingLimit(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   parameter,
                            ViBoolean state,
                            ViReal64  lowerLimit,
                            ViReal64  upperLimit);

ViStatus _VI_FUNC rsspecan_ConfigurePulseResultTableFrequencyLimit(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   parameter,
                            ViBoolean state,
                            ViReal64  lowerLimit,
                            ViReal64  upperLimit);

ViStatus _VI_FUNC rsspecan_ConfigurePulseResultTablePhaseLimit(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   parameter,
                            ViBoolean state,
                            ViReal64  lowerLimit,
                            ViReal64  upperLimit);

ViStatus _VI_FUNC rsspecan_ConfigurePulseResultTableTimeSidelobeLimit(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   parameter,
                            ViBoolean state,
                            ViReal64  lowerLimit,
                            ViReal64  upperLimit);

ViStatus _VI_FUNC rsspecan_ConfigurePulseResultTableEnvelopeModelLimit(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   parameter,
                            ViBoolean state,
                            ViReal64  lowerLimit,
                            ViReal64  upperLimit);

ViStatus _VI_FUNC rsspecan_ConfigurePulseResultTableStabilityLimit(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   parameter,
                            ViBoolean state,
                            ViReal64  lowerLimit,
                            ViReal64  upperLimit);

ViStatus _VI_FUNC rsspecan_ConfigurePulseAutomaticGridScaling(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   reserved,
                            ViBoolean autoScale);

ViStatus _VI_FUNC rsspecan_ConfigurePulseAbsoluteScaling(ViSession instrSession,
                                                         ViInt32   window,
                                                         ViReal64  minimum,
                                                         ViReal64  maximum);

ViStatus _VI_FUNC rsspecan_ConfigurePulseRelativeScaling(ViSession instrSession,
                                                         ViInt32   window,
                                                         ViInt32   reserved,
                                                         ViReal64  perDivison,
                                                         ViReal64  referencePosition,
                                                         ViReal64  referenceValue);

ViStatus _VI_FUNC rsspecan_ConfigurePulseReferenceValues(ViSession instrSession,
                                                         ViInt32   window,
                                                         ViInt32   reserved,
                                                         ViReal64  minimum,
                                                         ViReal64  maximum);

ViStatus _VI_FUNC rsspecan_QueryPulseIDs(ViSession instrSession,
                                         ViInt32   queryRange,
                                         ViInt32   arraySize,
                                         ViReal64  pulseIDs[],
                                         ViInt32*  returnedValues);

ViStatus _VI_FUNC rsspecan_QueryPulseNumbers(ViSession instrSession,
                                             ViInt32   queryRange,
                                             ViInt32   arraySize,
                                             ViReal64  pulseNumbers[],
                                             ViInt32*  returnedValues);

ViStatus _VI_FUNC rsspecan_QueryPulseCountAll(ViSession instrSession,
                                              ViInt32*  pulseCount);

ViStatus _VI_FUNC rsspecan_QueryPulseCountCurrentBuffer(ViSession instrSession,
                                                        ViInt32*  pulseCount);

ViStatus _VI_FUNC rsspecan_QueryPulseResultPower(ViSession instrSession,
                                                 ViInt32   parameter,
                                                 ViInt32   resultType,
                                                 ViInt32   queryRange,
                                                 ViInt32   arraySize,
                                                 ViReal64  results[],
                                                 ViInt32*  returnedValues);

ViStatus _VI_FUNC rsspecan_QueryPulseCountPower(ViSession instrSession,
                                                ViInt32   parameterGroup,
                                                ViInt32   queryRange,
                                                ViInt32*  pulseCount);

ViStatus _VI_FUNC rsspecan_QueryPulseResultTiming(ViSession instrSession,
                                                  ViInt32   parameter,
                                                  ViInt32   resultType,
                                                  ViInt32   queryRange,
                                                  ViInt32   arraySize,
                                                  ViReal64  results[],
                                                  ViInt32*  returnedValues);

ViStatus _VI_FUNC rsspecan_QueryPulseCountTiming(ViSession instrSession,
                                                 ViInt32   groupParameter,
                                                 ViInt32   queryRange,
                                                 ViInt32*  pulseCount);

ViStatus _VI_FUNC rsspecan_QueryPulseResultFrequency(ViSession instrSession,
                                                     ViInt32   parameter,
                                                     ViInt32   resultType,
                                                     ViInt32   queryRange,
                                                     ViInt32   arraySize,
                                                     ViReal64  results[],
                                                     ViInt32*  returnedValues);

ViStatus _VI_FUNC rsspecan_QueryPulseCountFrequency(ViSession instrSession,
                                                    ViInt32   groupParameter,
                                                    ViInt32   queryRange,
                                                    ViInt32*  pulseCount);

ViStatus _VI_FUNC rsspecan_QueryPulseResultPhase(ViSession instrSession,
                                                 ViInt32   parameter,
                                                 ViInt32   resultType,
                                                 ViInt32   queryRange,
                                                 ViInt32   arraySize,
                                                 ViReal64  results[],
                                                 ViInt32*  returnedValues);

ViStatus _VI_FUNC rsspecan_QueryPulseCountPhase(ViSession instrSession,
                                                ViInt32   queryRange,
                                                ViInt32   groupParameter,
                                                ViInt32*  pulseCount);

ViStatus _VI_FUNC rsspecan_QueryPulseResultTimeSidelobe(ViSession instrSession,
                                                        ViInt32   parameter,
                                                        ViInt32   resultType,
                                                        ViInt32   queryRange,
                                                        ViInt32   arraySize,
                                                        ViReal64  results[],
                                                        ViInt32*  returnedValues);

ViStatus _VI_FUNC rsspecan_QueryPulseCountTimeSidelobe(ViSession instrSession,
                                                       ViInt32   groupParameter,
                                                       ViInt32   queryRange,
                                                       ViInt32*  pulseCount);

ViStatus _VI_FUNC rsspecan_QueryPulseResultEnvelopeModel(ViSession instrSession,
                                                         ViInt32   parameter,
                                                         ViInt32   resultType,
                                                         ViInt32   queryRange,
                                                         ViInt32   arraySize,
                                                         ViReal64  results[],
                                                         ViInt32*  returnedValues);

ViStatus _VI_FUNC rsspecan_QueryPulseCountEnvelopeModel(ViSession instrSession,
                                                        ViInt32   parameter,
                                                        ViInt32   queryRange,
                                                        ViInt32*  pulseCount);

ViStatus _VI_FUNC rsspecan_QueryPulseResultStability(ViSession instrSession,
                                                     ViInt32   parameter,
                                                     ViInt32   resultType,
                                                     ViInt32   queryRange,
                                                     ViInt32   arraySize,
                                                     ViReal64  results[],
                                                     ViInt32*  returnedValues);

ViStatus _VI_FUNC rsspecan_QueryPulseCountStability(ViSession instrSession,
                                                    ViInt32   parameter,
                                                    ViInt32   queryRange,
                                                    ViInt32*  pulseCount);

ViStatus _VI_FUNC rsspecan_QueryPulseSpectrumResolutionBandwidth(
                            ViSession instrSession,
                            ViInt32   window,
                            ViReal64* RBW);

ViStatus _VI_FUNC rsspecan_QueryPulseResultRangeIQStoredInMemory(
                            ViSession instrSession,
                            ViInt32   arraySize,
                            ViChar    storedInMemory[]);

ViStatus _VI_FUNC rsspecan_QueryPulseSegmentedDataCapturingBoundary(
                            ViSession instrSession,
                            ViInt32   arraySize,
                            ViInt32   sampleIndices[],
                            ViInt32*  returnedValues);

ViStatus _VI_FUNC rsspecan_QueryPulseSegmentedDataCapturingTimestamps(
                            ViSession instrSession,
                            ViInt32   arraySize,
                            ViReal64  timestamps[],
                            ViInt32*  returnedValues);

ViStatus _VI_FUNC rsspecan_QueryPulseSegmentedDataCapturingTrigger(
                            ViSession instrSession,
                            ViInt32   arraySize,
                            ViReal64  trigger[],
                            ViInt32*  returnedValues);

ViStatus _VI_FUNC rsspecan_QueryPulseResultLimitPower(ViSession instrSession,
                                                      ViInt32   parameter,
                                                      ViInt32   queryRange,
                                                      ViInt32   arraySize,
                                                      ViInt32   limitResults[],
                                                      ViInt32*  returnedValues);

ViStatus _VI_FUNC rsspecan_QueryPulseResultLimitTiming(ViSession instrSession,
                                                       ViInt32   parameter,
                                                       ViInt32   queryRange,
                                                       ViInt32   arraySize,
                                                       ViInt32   limitResults[],
                                                       ViInt32*  returnedValues);

ViStatus _VI_FUNC rsspecan_QueryPulseResultLimitFrequency(ViSession instrSession,
                                                          ViInt32   parameter,
                                                          ViInt32   queryRange,
                                                          ViInt32   arraySize,
                                                          ViInt32   limitResults[],
                                                          ViInt32*  returnedValues);

ViStatus _VI_FUNC rsspecan_QueryPulseResultLimitPhase(ViSession instrSession,
                                                      ViInt32   parameter,
                                                      ViInt32   queryRange,
                                                      ViInt32   arraySize,
                                                      ViInt32   limitResults[],
                                                      ViInt32*  returnedValues);

ViStatus _VI_FUNC rsspecan_QueryPulseResultLimitTimeSidelobe(ViSession instrSession,
                                                             ViInt32   parameter,
                                                             ViInt32   queryRange,
                                                             ViInt32   arraySize,
                                                             ViInt32   limitResults[],
                                                             ViInt32*  returnedValues);

ViStatus _VI_FUNC rsspecan_QueryPulseResultLimitEnvelopeModel(
                            ViSession instrSession,
                            ViInt32   parameter,
                            ViInt32   queryRange,
                            ViInt32   arraySize,
                            ViInt32   limitResults[],
                            ViInt32*  returnedValues);

ViStatus _VI_FUNC rsspecan_PulseQueryResultLimitStability(ViSession instrSession,
                                                          ViInt32   parameter,
                                                          ViInt32   queryRange,
                                                          ViInt32   arraySize,
                                                          ViInt32   limitResults[],
                                                          ViInt32*  returnedValues);

ViStatus _VI_FUNC rsspecan_ADemodMode(ViSession instrSession);

ViStatus _VI_FUNC rsspecan_SetADemodState(ViSession instrSession,
                                          ViBoolean state);

ViStatus _VI_FUNC rsspecan_ConfigureAnalogDemodPredefinedStandard(
                            ViSession instrSession,
                            ViInt32   operation,
                            ViString  standard);

ViStatus _VI_FUNC rsspecan_ConfigureADemodTrigger(ViSession instrSession,
                                                  ViInt32   triggerSource,
                                                  ViReal64  triggerOffset,
                                                  ViInt32   triggerPolarity,
                                                  ViReal64  triggerValue);

ViStatus _VI_FUNC rsspecan_ConfigureADemodOnlineOutput(ViSession instrSession,
                                                       ViBoolean outputState,
                                                       ViString  outputSelection,
                                                       ViReal64  ACCutoffFrequency,
                                                       ViBoolean phonesOutput);

ViStatus _VI_FUNC rsspecan_ConfigureADemodSet(ViSession instrSession,
                                              ViReal64  sampleRate,
                                              ViInt32   recordLength,
                                              ViInt32   triggerSource,
                                              ViInt32   triggerSlope,
                                              ViInt32   pretriggerSamples,
                                              ViInt32   numberOfMeas);

ViStatus _VI_FUNC rsspecan_ConfigureAnalogDemodScaling(ViSession instrSession,
                                                       ViInt32   window,
                                                       ViReal64  perDivison);

ViStatus _VI_FUNC rsspecan_ConfigureADemodBandwidth(ViSession instrSession,
                                                    ViReal64  demodulationBandwidth);

ViStatus _VI_FUNC rsspecan_ConfigureADemodMeasuringTime(ViSession instrSession,
                                                        ViReal64  measuringTime);

ViStatus _VI_FUNC rsspecan_ConfigureAnalogDemod(ViSession instrSession,
                                                ViBoolean squelchState,
                                                ViReal64  squelchLevel,
                                                ViInt32   AFCoupling,
                                                ViInt32   trace);

ViStatus _VI_FUNC rsspecan_ConfigureAnalogDemodZeroPhaseRefPointMode(
                            ViSession instrSession,
                            ViInt32   mode);

ViStatus _VI_FUNC rsspecan_ConfigureAnalogDemodPMZeroPhaseReferencePoint(
                            ViSession instrSession,
                            ViReal64  referencePoint);

ViStatus _VI_FUNC rsspecan_ConfigureAnalogDemodPMPhaseWrap(ViSession instrSession,
                                                           ViInt32   phaseWrap);

ViStatus _VI_FUNC rsspecan_ConfigureADemodZoom(ViSession instrSession,
                                               ViBoolean zoomState,
                                               ViReal64  startTime);

ViStatus _VI_FUNC rsspecan_ConfigureADemodZoomLength(ViSession instrSession,
                                                     ViInt32   zoomLengthMode,
                                                     ViReal64  zoomLength);

ViStatus _VI_FUNC rsspecan_ConfigureADemodUnit(ViSession instrSession,
                                               ViInt32   phaseUnit,
                                               ViInt32   THDUnit);

ViStatus _VI_FUNC rsspecan_ConfigureADemodRelativeUnit(ViSession instrSession,
                                                       ViInt32   relativeUnit);

ViStatus _VI_FUNC rsspecan_SelectADemodResult(ViSession instrSession,
                                              ViInt32   operatingMode,
                                              ViInt32   trace);

ViStatus _VI_FUNC rsspecan_ConfigureADemodDetectorMode(ViSession instrSession,
                                                       ViInt32   demodulationType,
                                                       ViInt32   detectorType,
                                                       ViInt32   mode);

ViStatus _VI_FUNC rsspecan_ConfigureADemodDetectorState(ViSession instrSession,
                                                        ViInt32   demodulationType,
                                                        ViInt32   detectorType,
                                                        ViBoolean detectorEnable);

ViStatus _VI_FUNC rsspecan_ConfigureADemodDetectorReference(ViSession instrSession,
                                                            ViInt32   demodulationType,
                                                            ViInt32   detectorType,
                                                            ViReal64  reference);

ViStatus _VI_FUNC rsspecan_ConfigureADemodDetectorMeasToRef(ViSession instrSession,
                                                            ViInt32   demodulationType,
                                                            ViInt32   detectorType);

ViStatus _VI_FUNC rsspecan_ConfigureADemodAFFrequencyStartStop(
                            ViSession instrSession,
                            ViReal64  startFrequency,
                            ViReal64  stopFrequency);

ViStatus _VI_FUNC rsspecan_ConfigureADemodAFFrequencyCenterSpan(
                            ViSession instrSession,
                            ViReal64  centerFrequency,
                            ViReal64  span);

ViStatus _VI_FUNC rsspecan_ConfigureADemodAFFullSpan(ViSession instrSession);

ViStatus _VI_FUNC rsspecan_ConfigureADemodAFParam(ViSession instrSession,
                                                  ViReal64  resolutionBandwidth);

ViStatus _VI_FUNC rsspecan_ConfigureADemodRFParam(ViSession instrSession,
                                                  ViReal64  resolutionBandwidth,
                                                  ViReal64  fullSpan);

ViStatus _VI_FUNC rsspecan_ConfigureADemodRFZoom(ViSession instrSession,
                                                 ViReal64  zoom);

ViStatus _VI_FUNC rsspecan_ConfigureADemodFilter(ViSession instrSession,
                                                 ViInt32   filterType,
                                                 ViBoolean filterState,
                                                 ViReal64  filterFrequency,
                                                 ViReal64  AMFMDeemphasisValue);

ViStatus _VI_FUNC rsspecan_ConfigureADemodFilterManual(ViSession instrSession,
                                                       ViInt32   filterType,
                                                       ViReal64  filterFrequency);

ViStatus _VI_FUNC rsspecan_ConfigureADemodFilterWeighting(ViSession instrSession,
                                                          ViBoolean CCITT,
                                                          ViBoolean CCIR);

ViStatus _VI_FUNC rsspecan_ConfigureADemodFilterAWeighting(ViSession instrSession,
                                                           ViBoolean aWeightingFilter);

ViStatus _VI_FUNC rsspecan_ConfigureAnalogDemodSettlingTimeEnabled(
                            ViSession instrSession,
                            ViBoolean settlingTime);

ViStatus _VI_FUNC rsspecan_ConfigureAnalogDemodSettlingLimits(
                            ViSession instrSession,
                            ViReal64  lowerLimit,
                            ViReal64  upperLimit);

ViStatus _VI_FUNC rsspecan_ConfigureADemodResultType(ViSession instrSession,
                                                     ViInt32   demodulationType,
                                                     ViInt32   resultTypes[]);

ViStatus _VI_FUNC rsspecan_GetADemodCurrentlySetValue(ViSession instrSession,
                                                      ViInt32   value,
                                                      ViReal64* result);

ViStatus _VI_FUNC rsspecan_GetADemodResultValues(ViSession instrSession,
                                                 ViInt32   demodulationType,
                                                 ViInt32   resultType,
                                                 ViUInt32  timeout,
                                                 ViInt32   arraySize,
                                                 ViReal64  resultValues[],
                                                 ViInt32*  returnedValues);

ViStatus _VI_FUNC rsspecan_GetADemodOffset(ViSession instrSession,
                                           ViInt32   resultType,
                                           ViReal64* value);

ViStatus _VI_FUNC rsspecan_GetADemodMarkerModulationValue(ViSession instrSession,
                                                          ViInt32   markerNumber,
                                                          ViInt32   demodulationType,
                                                          ViInt32   detectorType,
                                                          ViInt32   trace,
                                                          ViReal64* value);

ViStatus _VI_FUNC rsspecan_GetADemodMarkerModulationRelativeValue(
                            ViSession instrSession,
                            ViInt32   markerNumber,
                            ViInt32   demodulationType,
                            ViInt32   detectorType,
                            ViInt32   trace,
                            ViReal64* value);

ViStatus _VI_FUNC rsspecan_GetADemodMarkerMeasValue(ViSession instrSession,
                                                    ViInt32   markerNumber,
                                                    ViInt32   measurement,
                                                    ViInt32   trace,
                                                    ViReal64* value);

ViStatus _VI_FUNC rsspecan_GetAnalogDemodDistortionResult(ViSession instrSession,
                                                          ViInt32   marker,
                                                          ViInt32   trace,
                                                          ViReal64* distortion);

ViStatus _VI_FUNC rsspecan_GetAnalogDemodSettlingTime(ViSession instrSession,
                                                      ViInt32   trace,
                                                      ViReal64* settlingTime);

ViStatus _VI_FUNC rsspecan_FMStereoMode(ViSession instrSession);

ViStatus _VI_FUNC rsspecan_SetFMStereoState(ViSession instrSession,
                                            ViBoolean state);

ViStatus _VI_FUNC rsspecan_ConfigureFMStereoTrigger(ViSession instrSession,
                                                    ViInt32   triggerSource,
                                                    ViReal64  triggerLevel);

ViStatus _VI_FUNC rsspecan_ConfigureFMStereoPhaseNoiseMarker(ViSession instrSession,
                                                             ViInt32   window,
                                                             ViInt32   marker,
                                                             ViBoolean state);

ViStatus _VI_FUNC rsspecan_ConfigureFMStereoDifferenceFrequencyDistortion(
                            ViSession instrSession,
                            ViInt32   window,
                            ViBoolean state,
                            ViInt32   unit);

ViStatus _VI_FUNC rsspecan_ConfigureFMStereoIntermodulationDistortion(
                            ViSession instrSession,
                            ViInt32   window,
                            ViBoolean state,
                            ViInt32   unit);

ViStatus _VI_FUNC rsspecan_ConfigureFMStereoAFAutoScale(ViSession instrSession,
                                                        ViBoolean state);

ViStatus _VI_FUNC rsspecan_ConfigureFMStereoDisplayConfig(ViSession instrSession,
                                                          ViInt32   window,
                                                          ViInt32   displayConfiguration);

ViStatus _VI_FUNC rsspecan_ConfigureFMStereoReferenceDeviation(
                            ViSession instrSession,
                            ViReal64  referenceDeviation);

ViStatus _VI_FUNC rsspecan_ConfigureFMStereoResultSummarySetup(
                            ViSession instrSession,
                            ViInt32   channelType,
                            ViInt32   detector,
                            ViInt32   mode,
                            ViBoolean coupled);

ViStatus _VI_FUNC rsspecan_ConfigureFMStereoMeasToRef(ViSession instrSession,
                                                      ViInt32   channelType);

ViStatus _VI_FUNC rsspecan_ConfigureFMStereoMeasurementTime(ViSession instrSession,
                                                            ViReal64  measuringTime);

ViStatus _VI_FUNC rsspecan_ConfigureFMStereoAFFilter(ViSession instrSession,
                                                     ViInt32   channelType,
                                                     ViInt32   highPass,
                                                     ViInt32   lowPass,
                                                     ViInt32   weighting,
                                                     ViInt32   deemphasis,
                                                     ViBoolean coupled);

ViStatus _VI_FUNC rsspecan_AllFMStereoAFFiltersOff(ViSession instrSession);

ViStatus _VI_FUNC rsspecan_ConfigureFMStereoAFRange(ViSession instrSession,
                                                    ViInt32   deviation,
                                                    ViReal64  devPerDivision);

ViStatus _VI_FUNC rsspecan_ConfigureFMStereoAFRangeTHDUnit(ViSession instrSession,
                                                           ViInt32   unit);

ViStatus _VI_FUNC rsspecan_ConfigureFMStereoAFRangeDevUnit(ViSession instrSession,
                                                           ViInt32   absoluteDevUnit,
                                                           ViInt32   relativeDevUnit);

ViStatus _VI_FUNC rsspecan_ConfigureFMStereoTimeDomainZoom(ViSession instrSession,
                                                           ViBoolean zoomState,
                                                           ViReal64  startTime);

ViStatus _VI_FUNC rsspecan_ConfigureFMStereoTimeDomainZoomLength(
                            ViSession instrSession,
                            ViInt32   zoomLengthMode,
                            ViReal64  zoomLength);

ViStatus _VI_FUNC rsspecan_ConfigureFMStereoSquelch(ViSession instrSession,
                                                    ViBoolean squelchState,
                                                    ViReal64  squelchLevel);

ViStatus _VI_FUNC rsspecan_QueryFMStereoCarrierFrequency(ViSession instrSession,
                                                         ViReal64* carrierFrequency);

ViStatus _VI_FUNC rsspecan_ConfigureFMStereoTraceModes(ViSession instrSession,
                                                       ViInt32   channelType,
                                                       ViInt32   measurementType,
                                                       ViInt32   arraySize,
                                                       ViInt32   traceModes[]);

ViStatus _VI_FUNC rsspecan_FMStereoDifferenceFrequencyDistortionSearchSignal(
                            ViSession instrSession,
                            ViInt32   window,
                            ViUInt32  timeout);

ViStatus _VI_FUNC rsspecan_FMStereoIntermodulationDistortionSearchSignal(
                            ViSession instrSession,
                            ViInt32   window,
                            ViUInt32  timeout);

ViStatus _VI_FUNC rsspecan_QueryFMStereoPhaseNoiseResult(ViSession instrSession,
                                                         ViInt32   window,
                                                         ViInt32   marker,
                                                         ViReal64* phaseNoiseResult);

ViStatus _VI_FUNC rsspecan_QueryFMStereoDifferenceFrequencyDistortionResult(
                            ViSession instrSession,
                            ViInt32   window,
                            ViReal64  differenceFrequencyDistortion[]);

ViStatus _VI_FUNC rsspecan_QueryFMStereoIntermodulationDistortionResult(
                            ViSession instrSession,
                            ViInt32   window,
                            ViReal64  intermodulationDistortion[]);

ViStatus _VI_FUNC rsspecan_QueryFMStereoMeasurementResults(ViSession instrSession,
                                                           ViReal64  absoluteDeviation[],
                                                           ViReal64  relativeDeviation[],
                                                           ViReal64  SINAD[],
                                                           ViReal64  THD[],
                                                           ViReal64  modulationFrequency[]);

ViStatus _VI_FUNC rsspecan_QueryFMStereoChannelTypeResults(ViSession instrSession,
                                                           ViInt32   window,
                                                           ViInt32   channelType,
                                                           ViInt32   measurementType,
                                                           ViReal64* result);

ViStatus _VI_FUNC rsspecan_QueryFMStereoChannelTypeAllResults(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   channelType,
                            ViReal64  result[]);

ViStatus _VI_FUNC rsspecan_QueryFMStereoResultValues(ViSession instrSession,
                                                     ViInt32   channelType,
                                                     ViInt32   measurementType,
                                                     ViInt32   traceMode,
                                                     ViUInt32  timeout,
                                                     ViInt32   arraySize,
                                                     ViReal64  resultValues[],
                                                     ViInt32*  returnedValues);

ViStatus _VI_FUNC rsspecan_BTOMode(ViSession instrSession);

ViStatus _VI_FUNC rsspecan_ConfigureBTOStandard(ViSession instrSession,
                                                ViInt32   standard);

ViStatus _VI_FUNC rsspecan_ConfigureBTOLowEnergyStandard(ViSession instrSession,
                                                         ViInt32   standard);

ViStatus _VI_FUNC rsspecan_ConfigureBTOLoadSettings(ViSession instrSession,
                                                    ViString  settingsFile);

ViStatus _VI_FUNC rsspecan_ConfigureBTOSaveSettings(ViSession instrSession,
                                                    ViString  settingsFile);

ViStatus _VI_FUNC rsspecan_ConfigureBTOSaveMeasurement(ViSession instrSession,
                                                       ViString  measurementFile);

ViStatus _VI_FUNC rsspecan_ConfigureBTOMeasurement(ViSession instrSession,
                                                   ViInt32   geographicalRegion,
                                                   ViInt32   channel,
                                                   ViInt32   outputPowerClass,
                                                   ViInt32   samplesPerSymbols,
                                                   ViInt32   packetType,
                                                   ViReal64  antennaGain);

ViStatus _VI_FUNC rsspecan_ConfigureBTOMeasurementFilterEnabled(
                            ViSession instrSession,
                            ViBoolean filter);

ViStatus _VI_FUNC rsspecan_QueryBTOMeasurementDemodulationBandwidth(
                            ViSession instrSession,
                            ViReal64* demodulationBandwidth);

ViStatus _VI_FUNC rsspecan_ConfigureBTOAveragingMethod(ViSession instrSession,
                                                       ViReal64  startPosition,
                                                       ViReal64  stopPosition);

ViStatus _VI_FUNC rsspecan_ConfigureBTOSynchronization(ViSession instrSession,
                                                       ViBoolean findSync,
                                                       ViInt32   syncOffset,
                                                       ViBoolean findBurst,
                                                       ViReal64  burstOffset,
                                                       ViBoolean searchLengthAuto,
                                                       ViReal64  searchLength);

ViStatus _VI_FUNC rsspecan_ConfigureBTOSynchronizationLAP(ViSession instrSession,
                                                          ViString  LAP);

ViStatus _VI_FUNC rsspecan_ConfigureBTOSynchronizationSearchEnabled(
                            ViSession instrSession,
                            ViBoolean syncSearch);

ViStatus _VI_FUNC rsspecan_ConfigureBTOPulseSearchEnabled(ViSession instrSession,
                                                          ViBoolean pulseSearch);

ViStatus _VI_FUNC rsspecan_ConfigureBTOLAP(ViSession instrSession,
                                           ViInt32   lowest24BitsOfDevAddress);

ViStatus _VI_FUNC rsspecan_ConfigureBTOMeasurementMode(ViSession instrSession,
                                                       ViInt32   measurementMode);

ViStatus _VI_FUNC rsspecan_ConfigureBTONumberOfAdjacentChannels(
                            ViSession instrSession,
                            ViInt32   numberOfAdjacentChannels);

ViStatus _VI_FUNC rsspecan_ConfigureBTOReceiveFilter(ViSession instrSession,
                                                     ViInt32   filter);

ViStatus _VI_FUNC rsspecan_ConfigureBTOSweepCouplingAuto(ViSession instrSession,
                                                         ViInt32   sweepCoupling);

ViStatus _VI_FUNC rsspecan_ConfigureBTOSweepCoupling(ViSession instrSession,
                                                     ViInt32   sweepCoupling,
                                                     ViReal64  couplingValue);

ViStatus _VI_FUNC rsspecan_ConfigureBTOSpuriousEmissionsGeography(
                            ViSession instrSession,
                            ViInt32   geography);

ViStatus _VI_FUNC rsspecan_ConfigureBTOSpuriousEmissionsNumberOfCapturedChannels(
                            ViSession instrSession,
                            ViInt32   capturedChannels);

ViStatus _VI_FUNC rsspecan_ConfigureBTOAcquisition(ViSession instrSession,
                                                   ViBoolean sweepModeContinuous,
                                                   ViInt32   numberOfSweeps);

ViStatus _VI_FUNC rsspecan_SelectBTOTrace(ViSession instrSession,
                                          ViInt32   trace);

ViStatus _VI_FUNC rsspecan_ConfigureBTOTrace(ViSession instrSession,
                                             ViInt32   trace,
                                             ViInt32   traceMode,
                                             ViInt32   traceDetector);

ViStatus _VI_FUNC rsspecan_ConfigureBTOTraceLimitCheckEnabled(
                            ViSession instrSession,
                            ViInt32   trace,
                            ViBoolean limitCheck);

ViStatus _VI_FUNC rsspecan_ConfigureBTOPacketBytesSCO(ViSession instrSession,
                                                      ViInt32   packetBytesSCO);

ViStatus _VI_FUNC rsspecan_ConfigureBTOAnalogDemodulationZoom(
                            ViSession instrSession,
                            ViBoolean zoomState,
                            ViReal64  startTime);

ViStatus _VI_FUNC rsspecan_ConfigureBTODisplayYAxisScaling(ViSession instrSession,
                                                           ViReal64  scaling);

ViStatus _VI_FUNC rsspecan_ConfigureBTOLimitChecksFrequencyDrift(
                            ViSession instrSession,
                            ViReal64  lower,
                            ViReal64  upper);

ViStatus _VI_FUNC rsspecan_ConfigureBTOLimitChecksFrequencyOffsetLower(
                            ViSession instrSession,
                            ViReal64  lower);

ViStatus _VI_FUNC rsspecan_ConfigureBTOLimitChecksFrequencyOffsetUpper(
                            ViSession instrSession,
                            ViReal64  upper);

ViStatus _VI_FUNC rsspecan_ConfigureBTOLimitChecksICFT(ViSession instrSession,
                                                       ViReal64  lower,
                                                       ViReal64  upper);

ViStatus _VI_FUNC rsspecan_ConfigureBTOLimitChecksInitialFrequencyDrift(
                            ViSession instrSession,
                            ViReal64  lower,
                            ViReal64  upper);

ViStatus _VI_FUNC rsspecan_ConfigureBTOLimitChecksMaxDriftRate(
                            ViSession instrSession,
                            ViReal64  lower,
                            ViReal64  upper);

ViStatus _VI_FUNC rsspecan_ConfigureBTOLimitChecksF1Avg(ViSession instrSession,
                                                        ViReal64  lower,
                                                        ViReal64  upper);

ViStatus _VI_FUNC rsspecan_ConfigureBTOLimitChecksF1MaxInRange(
                            ViSession instrSession,
                            ViReal64  limit);

ViStatus _VI_FUNC rsspecan_ConfigureBTOLimitChecksF1MaxLower(ViSession instrSession,
                                                             ViReal64  limit);

ViStatus _VI_FUNC rsspecan_ConfigureBTOLimitChecksF2AvgF1AvgRatioLower(
                            ViSession instrSession,
                            ViReal64  limit);

ViStatus _VI_FUNC rsspecan_ConfigureBTOLimitChecksF2MaxInRange(
                            ViSession instrSession,
                            ViReal64  limit);

ViStatus _VI_FUNC rsspecan_ConfigureBTOLimitChecksF2MaxLower(ViSession instrSession,
                                                             ViReal64  limit);

ViStatus _VI_FUNC rsspecan_ConfigureBTOLimitChecksPowerAverage(
                            ViSession instrSession,
                            ViReal64  lower,
                            ViReal64  upper);

ViStatus _VI_FUNC rsspecan_ConfigureBTOLimitChecksPowerRelativePeakUpper(
                            ViSession instrSession,
                            ViReal64  limit);

ViStatus _VI_FUNC rsspecan_ConfigureBTOLimitChecksPowerPeakUpper(
                            ViSession instrSession,
                            ViReal64  limit);

ViStatus _VI_FUNC rsspecan_ConfigureBTOLimitChecksRelativeTXPowerAvg(
                            ViSession instrSession,
                            ViReal64  lower,
                            ViReal64  upper);

ViStatus _VI_FUNC rsspecan_ConfigureBTOLimitChecksPacketMaximumDrift(
                            ViSession instrSession,
                            ViInt32   packetType,
                            ViReal64  lower,
                            ViReal64  upper);

ViStatus _VI_FUNC rsspecan_ConfigureBTOLimitChecksPacketFrequencyDrift(
                            ViSession instrSession,
                            ViInt32   packetType,
                            ViReal64  lower,
                            ViReal64  upper);

ViStatus _VI_FUNC rsspecan_ConfigureBTOLimitChecksBlockFrequency(
                            ViSession instrSession,
                            ViReal64  lower,
                            ViReal64  upper);

ViStatus _VI_FUNC rsspecan_ConfigureBTOLimitChecksInitialFrequency(
                            ViSession instrSession,
                            ViReal64  lower,
                            ViReal64  upper);

ViStatus _VI_FUNC rsspecan_ConfigureBTOLimitChecksTotalFrequencyOffset(
                            ViSession instrSession,
                            ViReal64  lower,
                            ViReal64  upper);

ViStatus _VI_FUNC rsspecan_ConfigureBTOLimitChecksPeakDQPSKUpper(
                            ViSession instrSession,
                            ViReal64  limit);

ViStatus _VI_FUNC rsspecan_ConfigureBTOLimitChecksRMSDQPSKUpper(
                            ViSession instrSession,
                            ViReal64  limit);

ViStatus _VI_FUNC rsspecan_ConfigureBTOLimitChecks99DQPSKUpper(
                            ViSession instrSession,
                            ViReal64  limit);

ViStatus _VI_FUNC rsspecan_ConfigureBTOLimitChecksPeak8PSKUpper(
                            ViSession instrSession,
                            ViReal64  limit);

ViStatus _VI_FUNC rsspecan_ConfigureBTOLimitChecksRMS8PSKUpper(
                            ViSession instrSession,
                            ViReal64  limit);

ViStatus _VI_FUNC rsspecan_ConfigureBTOLimitChecks998PSKUpper(
                            ViSession instrSession,
                            ViReal64  limit);

ViStatus _VI_FUNC rsspecan_ConfigureBTOLimitChecksFarLimit(ViSession instrSession,
                                                           ViReal64  limit);

ViStatus _VI_FUNC rsspecan_ConfigureBTOLimitChecksNearLimit(ViSession instrSession,
                                                            ViReal64  limit);

ViStatus _VI_FUNC rsspecan_ConfigureBTOLimitChecksL2FrequencyPowerDifference(
                            ViSession instrSession,
                            ViReal64  powerDifference);

ViStatus _VI_FUNC rsspecan_ConfigureBTOLimitChecksL4FrequencyPowerDifference(
                            ViSession instrSession,
                            ViReal64  powerDifference);

ViStatus _VI_FUNC rsspecan_ConfigureBTOLimitChecksL5FrequencyPowerDifference(
                            ViSession instrSession,
                            ViReal64  powerDifference);

ViStatus _VI_FUNC rsspecan_ConfigureBTOLimitChecksAllowedExceptions(
                            ViSession instrSession,
                            ViInt32   allowedExceptions);

ViStatus _VI_FUNC rsspecan_ConfigureBTOLimitChecksRelative(ViSession instrSession,
                                                           ViReal64  limit);

ViStatus _VI_FUNC rsspecan_GetBTOMesurementStatus(ViSession instrSession,
                                                  ViInt32*  measurementStatus);

ViStatus _VI_FUNC rsspecan_GetBTOPacketInfo(ViSession instrSession,
                                            ViInt32*  type,
                                            ViInt32*  length);

ViStatus _VI_FUNC rsspecan_GetBTOOutputPower(ViSession instrSession,
                                             ViInt32   type,
                                             ViReal64* power);

ViStatus _VI_FUNC rsspecan_GetBTONumberOfExceptions(ViSession instrSession,
                                                    ViInt32*  numberOfExceptions);

ViStatus _VI_FUNC rsspecan_GetBTOPowerOfChannels(ViSession instrSession,
                                                 ViInt32   arraySize,
                                                 ViReal64  power[],
                                                 ViInt32*  returnedValues);

ViStatus _VI_FUNC rsspecan_GetBTOFrequencyDeviation(ViSession instrSession,
                                                    ViInt32   frequencyDeviation,
                                                    ViInt32   type,
                                                    ViReal64* value);

ViStatus _VI_FUNC rsspecan_GetBTORatioAverageFrequencyDeviation(
                            ViSession instrSession,
                            ViInt32   type,
                            ViReal64* value);

ViStatus _VI_FUNC rsspecan_GetBTOPercentageDeviation(ViSession instrSession,
                                                     ViReal64* value);

ViStatus _VI_FUNC rsspecan_GetBTOPacketDataBits(ViSession instrSession,
                                                ViUInt32  maximumTime,
                                                ViInt32   arrayLength,
                                                ViInt32*  actualPoints,
                                                ViReal64  values[]);

ViStatus _VI_FUNC rsspecan_ReadBTOFMTrace(ViSession instrSession,
                                          ViUInt32  maximumTime,
                                          ViInt32   arrayLength,
                                          ViInt32*  actualPoints,
                                          ViReal64  amplitude[]);

ViStatus _VI_FUNC rsspecan_GetBTOInitCarrFreqTol(ViSession instrSession,
                                                 ViInt32   type,
                                                 ViReal64* value);

ViStatus _VI_FUNC rsspecan_GetBTOCarrFreqDrift(ViSession instrSession,
                                               ViInt32   type,
                                               ViReal64* value);

ViStatus _VI_FUNC rsspecan_ConfigureBTOSENumAdjacentChannels(ViSession instrSession,
                                                             ViInt32   number);

ViStatus _VI_FUNC rsspecan_AdjustBTOSEGateSetting(ViSession instrSession);

ViStatus _VI_FUNC rsspecan_GetBTOChannelPowerOfSEMeasurement(ViSession instrSession,
                                                             ViInt32   parameter,
                                                             ViReal64* value);

ViStatus _VI_FUNC rsspecan_GetBTOSEPowerOfAdjacentChannels(ViSession instrSession,
                                                           ViInt32   arraySize,
                                                           ViReal64  power[],
                                                           ViInt32*  returnedValues);

ViStatus _VI_FUNC rsspecan_GetBTOSENumberOfExceptions(ViSession instrSession,
                                                      ViInt32*  exceptions);

ViStatus _VI_FUNC rsspecan_ConfigureBTOCFSBlockCount(ViSession instrSession,
                                                     ViInt32   blockCount);

ViStatus _VI_FUNC rsspecan_GetBTOCFSDeviation(ViSession instrSession,
                                              ViInt32   parameter,
                                              ViInt32   type,
                                              ViReal64* value);

ViStatus _VI_FUNC rsspecan_GetBTOCFSModulAccuracy(ViSession instrSession,
                                                  ViInt32   parameter,
                                                  ViInt32   type,
                                                  ViReal64* value);

ViStatus _VI_FUNC rsspecan_GetBTODifferentialPhaseEncodingValues(
                            ViSession instrSession,
                            ViInt32   parameter,
                            ViInt32*  value);

ViStatus _VI_FUNC rsspecan_ConfigureBTORTPMeasurement(ViSession instrSession,
                                                      ViInt32   packetSection,
                                                      ViReal64  start,
                                                      ViReal64  stop);

ViStatus _VI_FUNC rsspecan_GetBTORelativeTransmissionPower(ViSession instrSession,
                                                           ViInt32   parameter,
                                                           ViInt32   modificator,
                                                           ViReal64* value);

ViStatus _VI_FUNC rsspecan_PWMMode(ViSession instrSession,
                                   ViInt32   sensor,
                                   ViBoolean state);

ViStatus _VI_FUNC rsspecan_ConfigurePWMMeasurement(ViSession instrSession,
                                                   ViInt32   sensor,
                                                   ViInt32   measurementTime,
                                                   ViInt32   coupling,
                                                   ViReal64  frequency,
                                                   ViInt32   averageCount);

ViStatus _VI_FUNC rsspecan_ConfigurePWMRelative(ViSession instrSession,
                                                ViInt32   sensor,
                                                ViReal64  referenceValue,
                                                ViInt32   unit);

ViStatus _VI_FUNC rsspecan_ConfigurePWMAbsolute(ViSession instrSession,
                                                ViInt32   sensor,
                                                ViInt32   unit);

ViStatus _VI_FUNC rsspecan_ConfigurePWMReferenceLevelOffsetState(
                            ViSession instrSession,
                            ViInt32   sensor,
                            ViBoolean state);

ViStatus _VI_FUNC rsspecan_ConfigurePWMExternalPowerTrigger(ViSession instrSession,
                                                            ViInt32   sensor,
                                                            ViBoolean state,
                                                            ViReal64  level);

ViStatus _VI_FUNC rsspecan_ConfigurePWMExternalPowerTriggerAdvanced(
                            ViSession instrSession,
                            ViInt32   sensor,
                            ViInt32   hysteresis,
                            ViReal64  dropoutTime,
                            ViReal64  holdoffTime,
                            ViInt32   slope);

ViStatus _VI_FUNC rsspecan_ConfigurePWMCorrectionLevelAuto(ViSession instrSession,
                                                           ViBoolean autoLevel);

ViStatus _VI_FUNC rsspecan_QueryPWMCorrectionLevel(ViSession instrSession,
                                                   ViReal64* level);

ViStatus _VI_FUNC rsspecan_ConfigurePWMDutyCycle(ViSession instrSession,
                                                 ViInt32   sensor,
                                                 ViBoolean dutyCycle,
                                                 ViReal64  dutyCycleValue);

ViStatus _VI_FUNC rsspecan_ConfigurePowerSensorAssignment(ViSession instrSession,
                                                          ViInt32   sensor,
                                                          ViBoolean autoAssignment,
                                                          ViString  placeholder,
                                                          ViString  type,
                                                          ViString  intfc,
                                                          ViString  serialNumber);

ViStatus _VI_FUNC rsspecan_QueryPowerSensorCount(ViSession instrSession,
                                                 ViInt32*  powerSensorCount);

ViStatus _VI_FUNC rsspecan_ConfigurePWMContinuousUpdate(ViSession instrSession,
                                                        ViInt32   sensor,
                                                        ViBoolean state);

ViStatus _VI_FUNC rsspecan_ConfigurePWMSensorOffset(ViSession instrSession,
                                                    ViInt32   sensor,
                                                    ViReal64  offset);

ViStatus _VI_FUNC rsspecan_ConfigurePWMMeasToRef(ViSession instrSession,
                                                 ViInt32   sensor);

ViStatus _VI_FUNC rsspecan_ConfigurePWMZero(ViSession instrSession,
                                            ViInt32   sensor);

ViStatus _VI_FUNC rsspecan_ReadPWMResult(ViSession instrSession,
                                         ViInt32   sensor,
                                         ViUInt32  timeout,
                                         ViReal64* result);

ViStatus _VI_FUNC rsspecan_FetchPWMResult(ViSession instrSession,
                                          ViInt32   sensor,
                                          ViReal64* result);

ViStatus _VI_FUNC rsspecan_GSMK10Mode(ViSession instrSession);

ViStatus _VI_FUNC rsspecan_ConfigureGSMK10Trace(ViSession instrSession,
                                                ViInt32   window,
                                                ViInt32   trace,
                                                ViInt32   traceType);

ViStatus _VI_FUNC rsspecan_ConfigureGSMK10DUT(ViSession instrSession,
                                              ViInt32   deviceUnderTest);

ViStatus _VI_FUNC rsspecan_ConfigureGSMK10SignalCharacteristics(
                            ViSession instrSession,
                            ViInt32   bandClass,
                            ViInt32   ARFCN);

ViStatus _VI_FUNC rsspecan_ConfigureGSMK10LevelSettings(ViSession instrSession,
                                                        ViInt32   automaticReferenceLevel,
                                                        ViReal64  referenceLevel,
                                                        ViReal64  externalAttenuation,
                                                        ViInt32   powerClass,
                                                        ViInt32   reserved);

ViStatus _VI_FUNC rsspecan_ConfigureGSMK10CaptureSettings(ViSession instrSession,
                                                          ViInt32   signalSource,
                                                          ViReal64  captureTime,
                                                          ViInt32   triggerMode,
                                                          ViReal64  triggerOffset,
                                                          ViReal64  triggerLevel,
                                                          ViInt32   statisticCount);

ViStatus _VI_FUNC rsspecan_QueryGSMK10StatisticCount(ViSession instrSession,
                                                     ViInt32*  statisticCount);

ViStatus _VI_FUNC rsspecan_QueryGSMK10TriggerToSyncNumberOfAcquisitions(
                            ViSession instrSession,
                            ViInt32*  numberOfAcquisitions);

ViStatus _VI_FUNC rsspecan_ConfigureGSMK10CorrelationThreshold(
                            ViSession instrSession,
                            ViReal64  correlationThreshold);

ViStatus _VI_FUNC rsspecan_ConfigureGSMK10Synchronization(ViSession instrSession,
                                                          ViInt32   synchronization,
                                                          ViBoolean measureOnlyOnSync);

ViStatus _VI_FUNC rsspecan_ConfigureGSMK10AutoTrackTime(ViSession instrSession,
                                                        ViReal64  autoTrackTime);

ViStatus _VI_FUNC rsspecan_ConfigureGSMK10SwapIQ(ViSession instrSession,
                                                 ViBoolean swapIQ);

ViStatus _VI_FUNC rsspecan_ConfigureGSMK10MarkerZoom(ViSession instrSession,
                                                     ViInt32   zoomFactor);

ViStatus _VI_FUNC rsspecan_ConfigureGSMK10ResultDisplay(ViSession instrSession,
                                                        ViInt32   resultDisplay);

ViStatus _VI_FUNC rsspecan_AddGSMK10Window(ViSession instrSession,
                                           ViInt32   stringSize,
                                           ViString  referenceWindowName,
                                           ViInt32   direction,
                                           ViInt32   windowType,
                                           ViChar    returnedWindowName[]);

ViStatus _VI_FUNC rsspecan_ReplaceGSMK10Window(ViSession instrSession,
                                               ViString  windowName,
                                               ViInt32   windowType);

ViStatus _VI_FUNC rsspecan_ConfigureGSMK10AutoSettings(ViSession instrSession,
                                                       ViInt32   level,
                                                       ViInt32   frameConfiguration,
                                                       ViInt32   trigger);

ViStatus _VI_FUNC rsspecan_ConfigureGSMK10DemodulationSingleSlotMeasurements(
                            ViSession instrSession,
                            ViInt32   slotToMeasure);

ViStatus _VI_FUNC rsspecan_ConfigureGSMK10DemodulationMultiSlotMeasurements(
                            ViSession instrSession,
                            ViInt32   numberOfSlotsToMeasure,
                            ViInt32   firstSlotToMeasure);

ViStatus _VI_FUNC rsspecan_ConfigureGSMK10DemodulationEqualTimeslotLength(
                            ViSession instrSession,
                            ViBoolean equalTimeslotLength);

ViStatus _VI_FUNC rsspecan_ConfigureGSMK10Burst(ViSession instrSession,
                                                ViInt32   slot,
                                                ViBoolean active,
                                                ViInt32   burstType,
                                                ViInt32   reserved,
                                                ViInt32   modulation,
                                                ViInt32   filter,
                                                ViInt32   TSC);

ViStatus _VI_FUNC rsspecan_ConfigureGSMK10UserTSC(ViSession instrSession,
                                                  ViInt32   slot,
                                                  ViString  userTSC);

ViStatus _VI_FUNC rsspecan_ConfigureGSMK10AccessBurst(ViSession instrSession,
                                                      ViInt32   slot,
                                                      ViBoolean active,
                                                      ViInt32   timingAdvance,
                                                      ViInt32   sync,
                                                      ViString  userTSC);

ViStatus _VI_FUNC rsspecan_ConfigureGSMK10AQPSKBurst(ViSession instrSession,
                                                     ViInt32   slot,
                                                     ViBoolean active,
                                                     ViReal64  SCPIR,
                                                     ViInt32   subchannel1TSC,
                                                     ViString  subchannel1UserTSC,
                                                     ViInt32   subchannel2TSC,
                                                     ViString  subchannel2UserTSC);

ViStatus _VI_FUNC rsspecan_ConfigureGSMK10TriggerToSync(ViSession instrSession,
                                                        ViInt32   numberOfBins,
                                                        ViInt32   adaptiveDataSize);

ViStatus _VI_FUNC rsspecan_ConfigureGSMK10WideSpectrumResultsUnit(
                            ViSession instrSession,
                            ViInt32   unit);

ViStatus _VI_FUNC rsspecan_ConfigureGSMK10MCWNMeasurement(ViSession instrSession,
                                                          ViInt32   measurement);

ViStatus _VI_FUNC rsspecan_ConfigureGSMK10ReferencePowerMeasurement(
                            ViSession instrSession,
                            ViBoolean measure,
                            ViInt32   averageCount,
                            ViBoolean carrierSelection,
                            ViInt32   carrier);

ViStatus _VI_FUNC rsspecan_ConfigureGSMK10ReferencePowerManually(
                            ViSession instrSession,
                            ViReal64  powerLevel,
                            ViReal64  RWBsLevels[]);

ViStatus _VI_FUNC rsspecan_ConfigureGSMK10NoiseMasurement(ViSession instrSession,
                                                          ViInt32   averageCount,
                                                          ViBoolean narrowbandNoise,
                                                          ViBoolean widebandNoise,
                                                          ViInt32   intermodulation,
                                                          ViBoolean applyExceptions);

ViStatus _VI_FUNC rsspecan_ConfigureGSMK10MaximumOutputPowerPerCarrier(
                            ViSession instrSession,
                            ViBoolean mode,
                            ViReal64  powerValue);

ViStatus _VI_FUNC rsspecan_ConfigureGSMK10CarriersAllocation(ViSession instrSession,
                                                             ViInt32   carrierAllocation,
                                                             ViInt32   gapStart);

ViStatus _VI_FUNC rsspecan_ConfigureGSMK10Carrier(ViSession instrSession,
                                                  ViInt32   carrier,
                                                  ViBoolean active,
                                                  ViReal64  frequency,
                                                  ViInt32   modulation);

ViStatus _VI_FUNC rsspecan_ConfigureGSMK10Measurement(ViSession instrSession,
                                                      ViInt32   measurementMode);

ViStatus _VI_FUNC rsspecan_ConfigureGsmK10PVTMeasurement(ViSession instrSession,
                                                         ViInt32   burstSection);

ViStatus _VI_FUNC rsspecan_ConfigureGSMK10PVTFilter(ViSession instrSession,
                                                    ViInt32   powerVsTimeFilter);

ViStatus _VI_FUNC rsspecan_ConfigureGSMK10PVTAlignment(ViSession instrSession,
                                                       ViInt32   alignment);

ViStatus _VI_FUNC rsspecan_ConfigureGSMK10ModulationTransientSpectrum(
                            ViSession instrSession,
                            ViBoolean enableLeftLimit,
                            ViBoolean enableRightLimit);

ViStatus _VI_FUNC rsspecan_ConfigureGSMK10ModulationTransientSpectrumAdditional(
                            ViSession instrSession,
                            ViInt32   filterType,
                            ViInt32   transientReferencePower,
                            ViInt32   offsetFrequency,
                            ViInt32   modulationRBWAt1800kHz,
                            ViBoolean highDynamic,
                            ViInt32   modulationFrequencyList);

ViStatus _VI_FUNC rsspecan_ConfigureGSMK10Demodulation(ViSession instrSession,
                                                       ViInt32   symbolDecision,
                                                       ViInt32   tailTSCBits);

ViStatus _VI_FUNC rsspecan_ConfigureGSMK10MultiMeasurement(ViSession instrSession,
                                                           ViBoolean multipleMeasurementMode,
                                                           ViBoolean powerVsTime,
                                                           ViBoolean demodulation,
                                                           ViBoolean constellation,
                                                           ViBoolean modulationSpectrum,
                                                           ViBoolean transientSpectrum);

ViStatus _VI_FUNC rsspecan_ConfigureGSMK10MultiCarrier(ViSession instrSession,
                                                       ViBoolean multiCarrierBTS,
                                                       ViInt32   numberOfActiveCarriers,
                                                       ViInt32   BTSClass,
                                                       ViInt32   pvTFilter);

ViStatus _VI_FUNC rsspecan_GSMK10RefreshCapturedData(ViSession instrSession);

ViStatus _VI_FUNC rsspecan_QueryGSMK10LimitExceptions(ViSession instrSession,
                                                      ViInt32   window,
                                                      ViInt32   limit,
                                                      ViInt32   type,
                                                      ViInt32*  result);

ViStatus _VI_FUNC rsspecan_GetGSMLimitCheckResult(ViSession instrSession,
                                                  ViInt32*  state);

ViStatus _VI_FUNC rsspecan_FetchGSMMCWNIntermodulationProductsResults(
                            ViSession instrSession,
                            ViInt32   intermodulation,
                            ViInt32   arraySize,
                            ViReal64  frequencyAbs[],
                            ViReal64  frequencyRel[],
                            ViReal64  RBW[],
                            ViReal64  power[],
                            ViReal64  limit[],
                            ViInt32   mode[],
                            ViInt32   limitCheck[],
                            ViInt32*  returnedValues);

ViStatus _VI_FUNC rsspecan_FetchGSMMCWNNarrowbandResults(ViSession instrSession,
                                                         ViInt32   intermodulation,
                                                         ViInt32   arraySize,
                                                         ViReal64  frequencyAbs[],
                                                         ViReal64  frequencyRel[],
                                                         ViReal64  RBW[],
                                                         ViReal64  power[],
                                                         ViReal64  limit[],
                                                         ViInt32   mode[],
                                                         ViInt32   limitCheck[],
                                                         ViInt32*  returnedValues);

ViStatus _VI_FUNC rsspecan_FetchGSMMCWNWidebandNoiseResults(ViSession instrSession,
                                                            ViInt32   intermodulation,
                                                            ViInt32   arraySize,
                                                            ViReal64  startFrequencyAbs[],
                                                            ViReal64  stopFrequencyAbs[],
                                                            ViReal64  worstFreqAbs[],
                                                            ViReal64  worstFreqRel[],
                                                            ViReal64  RBW[],
                                                            ViReal64  powerAtWorst[],
                                                            ViReal64  limitAtWorst[],
                                                            ViInt32   mode[],
                                                            ViInt32   limitCheck[],
                                                            ViInt32*  returnedValues);

ViStatus _VI_FUNC rsspecan_FetchGSMMCWNReferencePowerLevels(ViSession instrSession,
                                                            ViInt32   arraySize,
                                                            ViInt32   activeCarrier[],
                                                            ViInt32   referenceType[],
                                                            ViReal64  frequencyAbs[],
                                                            ViReal64  powerAbs[],
                                                            ViReal64  absRef300[],
                                                            ViReal64  absRef100[],
                                                            ViReal64  absRef30[],
                                                            ViInt32*  returnedValues);

ViStatus _VI_FUNC rsspecan_ReadGSMK10MeasurementResults(ViSession instrSession,
                                                        ViUInt32  timeout,
                                                        ViInt32   measurement,
                                                        ViInt32   modifier,
                                                        ViReal64* result);

ViStatus _VI_FUNC rsspecan_FetchGSMK10MeasurementResults(ViSession instrSession,
                                                         ViInt32   measurement,
                                                         ViInt32   modifier,
                                                         ViReal64* result);

ViStatus _VI_FUNC rsspecan_ReadGSMK10ModulationAccuracyResults(
                            ViSession instrSession,
                            ViUInt32  timeout,
                            ViInt32   arraySize,
                            ViReal64  result[],
                            ViInt32*  returnedValues);

ViStatus _VI_FUNC rsspecan_FetchGSMK10ModulationAccuracyResults(
                            ViSession instrSession,
                            ViInt32   arraySize,
                            ViReal64  result[],
                            ViInt32*  returnedValues);

ViStatus _VI_FUNC rsspecan_ReadGSMK10BurstSlotResults(ViSession instrSession,
                                                      ViUInt32  timeout,
                                                      ViInt32   slot,
                                                      ViInt32   burstSelection,
                                                      ViInt32   modifier,
                                                      ViReal64* result);

ViStatus _VI_FUNC rsspecan_FetchGSMK10BurstSlotResults(ViSession instrSession,
                                                       ViInt32   slot,
                                                       ViInt32   burstSelection,
                                                       ViInt32   modifier,
                                                       ViReal64* result);

ViStatus _VI_FUNC rsspecan_ReadGSMK10BurstSlotDeltaToSync(ViSession instrSession,
                                                          ViUInt32  timeout,
                                                          ViInt32   slot,
                                                          ViReal64* result);

ViStatus _VI_FUNC rsspecan_FetchGSMK10BurstSlotDeltaToSync(ViSession instrSession,
                                                           ViInt32   slot,
                                                           ViReal64* result);

ViStatus _VI_FUNC rsspecan_ReadGSMK10SpectrumReferencePower(ViSession instrSession,
                                                            ViUInt32  timeout,
                                                            ViInt32   spectrumType,
                                                            ViReal64* level1,
                                                            ViReal64* level2,
                                                            ViReal64* resolutionBW);

ViStatus _VI_FUNC rsspecan_FetchGSMK10SpectrumReferencePower(ViSession instrSession,
                                                             ViInt32   spectrumType,
                                                             ViReal64* level1,
                                                             ViReal64* level2,
                                                             ViReal64* resolutionBW);

ViStatus _VI_FUNC rsspecan_ReadGSMK10ModulationSpectrumResults(
                            ViSession instrSession,
                            ViInt32   arraySize,
                            ViUInt32  timeout,
                            ViInt32   index[],
                            ViReal64  startFrequencies[],
                            ViReal64  stopFrequencies[],
                            ViReal64  levels[],
                            ViReal64  limits[],
                            ViInt32   absRel[],
                            ViInt32   status[],
                            ViInt32*  numOfResults);

ViStatus _VI_FUNC rsspecan_FetchGSMK10ModulationSpectrumResults(
                            ViSession instrSession,
                            ViInt32   arraySize,
                            ViInt32   index[],
                            ViReal64  startFrequencies[],
                            ViReal64  stopFrequencies[],
                            ViReal64  levels[],
                            ViReal64  limits[],
                            ViInt32   absRel[],
                            ViInt32   status[],
                            ViInt32*  numOfResults);

ViStatus _VI_FUNC rsspecan_ReadGSMK10TransientSpectrumResults(
                            ViSession instrSession,
                            ViInt32   arraySize,
                            ViUInt32  timeout,
                            ViInt32   index[],
                            ViReal64  startFrequencies[],
                            ViReal64  stopFrequencies[],
                            ViReal64  levels[],
                            ViReal64  limits[],
                            ViInt32   absRel[],
                            ViInt32   status[],
                            ViInt32*  numOfResults);

ViStatus _VI_FUNC rsspecan_FetchGSMK10TransientSpectrumResults(
                            ViSession instrSession,
                            ViInt32   arraySize,
                            ViInt32   index[],
                            ViReal64  startFrequencies[],
                            ViReal64  stopFrequencies[],
                            ViReal64  levels[],
                            ViReal64  limits[],
                            ViInt32   absRel[],
                            ViInt32   status[],
                            ViInt32*  numOfResults);

ViStatus _VI_FUNC rsspecan_ReadGSMK10WideSpectrumResults(ViSession instrSession,
                                                         ViInt32   arraySize,
                                                         ViUInt32  timeout,
                                                         ViInt32   index[],
                                                         ViReal64  absoluteOffsetFrequencies1[],
                                                         ViReal64  absoluteOffsetFrequencies2[],
                                                         ViReal64  levels[],
                                                         ViReal64  limits[],
                                                         ViInt32   absRel[],
                                                         ViInt32   status[],
                                                         ViInt32*  numOfResults);

ViStatus _VI_FUNC rsspecan_FetchGSMK10WideSpectrumResults(ViSession instrSession,
                                                          ViInt32   arraySize,
                                                          ViInt32   index[],
                                                          ViReal64  absoluteOffsetFrequencies1[],
                                                          ViReal64  absoluteOffsetFrequencies2[],
                                                          ViReal64  levels[],
                                                          ViReal64  limits[],
                                                          ViInt32   absRel[],
                                                          ViInt32   status[],
                                                          ViInt32*  numOfResults);

ViStatus _VI_FUNC rsspecan_ReadGSMK10WideSpectrumGating(ViSession instrSession,
                                                        ViUInt32  timeout,
                                                        ViReal64* triggerOffset,
                                                        ViReal64* gateLength);

ViStatus _VI_FUNC rsspecan_FetchGSMK10MagnitudeCaptureResults(
                            ViSession instrSession,
                            ViInt32   arraySize,
                            ViInt32   resultType,
                            ViReal64  position[],
                            ViReal64  length[],
                            ViInt32*  numOfResults);

ViStatus _VI_FUNC rsspecan_MultiCarrierGroupDelayMode(ViSession instrSession);

ViStatus _VI_FUNC rsspecan_ConfigureMultiCarrierGroupDelayInput(
                            ViSession instrSession,
                            ViInt32   inputCoupling,
                            ViInt32   impedance,
                            ViBoolean highpassFilter,
                            ViBoolean YIGPreselector);

ViStatus _VI_FUNC rsspecan_ConfigureMultiCarrierGroupDelayReferenceLevel(
                            ViSession instrSession,
                            ViInt32   reserved,
                            ViReal64  referenceLevel,
                            ViReal64  referenceLevelOffset);

ViStatus _VI_FUNC rsspecan_ConfigureMultiCarrierGroupDelayMechanicalAttenuator(
                            ViSession instrSession,
                            ViBoolean autoMode,
                            ViReal64  attenuationValue);

ViStatus _VI_FUNC rsspecan_ConfigureMultiCarrierGroupDelayElectronicAttenuator(
                            ViSession instrSession,
                            ViBoolean state,
                            ViBoolean electronicAttenuatorAuto,
                            ViInt32   manualValuedB);

ViStatus _VI_FUNC rsspecan_ConfigureMultiCarrierGroupDelayPreamplifier(
                            ViSession instrSession,
                            ViBoolean state,
                            ViInt32   preamplifierLevel);

ViStatus _VI_FUNC rsspecan_ConfigureMultiCarrierGroupDelayYAxisScaling(
                            ViSession instrSession,
                            ViInt32   reserved,
                            ViReal64  range,
                            ViReal64  referencePosition,
                            ViInt32   verticalScale,
                            ViInt32   yAxisGridMode);

ViStatus _VI_FUNC rsspecan_ConfigureMultiCarrierGroupDelayYAxisScalingMaxMin(
                            ViSession instrSession,
                            ViInt32   reserved,
                            ViReal64  maximumValue,
                            ViReal64  miniumumValue);

ViStatus _VI_FUNC rsspecan_ConfigureMultiCarrierGroupDelayYAxisScalingReferencePerDivision(
                            ViSession instrSession,
                            ViInt32   reserved,
                            ViReal64  scaling,
                            ViReal64  referencePosition,
                            ViReal64  referenceValue);

ViStatus _VI_FUNC rsspecan_ConfigureMultiCarrierGroupDelayPhaseUnit(
                            ViSession instrSession,
                            ViInt32   unit);

ViStatus _VI_FUNC rsspecan_ConfigureMultiCarrierGroupDelaySignalDescription(
                            ViSession instrSession,
                            ViInt32   numberOfCarriers,
                            ViReal64  carrierSpacing);

ViStatus _VI_FUNC rsspecan_ConfigureMultiCarrierGroupDelayMeasurementTime(
                            ViSession instrSession,
                            ViBoolean mode,
                            ViReal64  measurementTime);

ViStatus _VI_FUNC rsspecan_ConfigureMultiCarrierGroupDelaySettings(
                            ViSession instrSession,
                            ViInt32   sweepCount,
                            ViInt32   groupDelay,
                            ViInt32   carrierOffset);

ViStatus _VI_FUNC rsspecan_ConfigureMultiCarrierGroupDelayReference(
                            ViSession instrSession,
                            ViInt32   referenceType);

ViStatus _VI_FUNC rsspecan_ConfigureMultiCarrierGroupDelayReferenceFrequency(
                            ViSession instrSession,
                            ViReal64  referenceCarrierFrequency);

ViStatus _VI_FUNC rsspecan_ConfigureMultiCarrierGroupDelaySmoothing(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   trace,
                            ViBoolean smoothing,
                            ViInt32   apertureValue);

ViStatus _VI_FUNC rsspecan_QueryMultiCarrierGroupDelayCarrierOffset(
                            ViSession instrSession,
                            ViReal64* carrierOffset);

ViStatus _VI_FUNC rsspecan_ConfigureMultiCarrierGroupDelayModeEstimation(
                            ViSession instrSession,
                            ViInt32   estimation);

ViStatus _VI_FUNC rsspecan_ConfigureMultiCarrierGroupDelayMaxClockOffset(
                            ViSession instrSession,
                            ViReal64  offset);

ViStatus _VI_FUNC rsspecan_ConfigureMultiCarrierGroupRawPhaseEnabled(
                            ViSession instrSession,
                            ViBoolean rawPhase);

ViStatus _VI_FUNC rsspecan_ConfigureMultiCarrierGroupDelayLargeOffsetCompensationEnabled(
                            ViSession instrSession,
                            ViBoolean enabled);

ViStatus _VI_FUNC rsspecan_ConfigureMultiCarrierGroupDelayCarrierDecimationFactor(
                            ViSession instrSession,
                            ViInt32   decimationFactor);

ViStatus _VI_FUNC rsspecan_QueryMultiCarrierGroupDelayClockOffset(
                            ViSession instrSession,
                            ViReal64* clockOffset);

ViStatus _VI_FUNC rsspecan_AddMultiCarrierGroupDelayWindow(ViSession instrSession,
                                                           ViInt32   stringSize,
                                                           ViString  referenceWindowName,
                                                           ViInt32   direction,
                                                           ViInt32   windowType,
                                                           ViChar    returnedWindowName[]);

ViStatus _VI_FUNC rsspecan_ReplaceMultiCarrierGroupDelayWindow(
                            ViSession instrSession,
                            ViString  windowName,
                            ViInt32   windowType);

ViStatus _VI_FUNC rsspecan_ConfigureMCGDGeneratorConnectionSettings(
                            ViSession instrSession,
                            ViBoolean connectToGenerator,
                            ViString  generatorIPAddress,
                            ViBoolean RFOutput);

ViStatus _VI_FUNC rsspecan_QueryMCGDGeneratorConnectionStatus(
                            ViSession instrSession,
                            ViInt32   bufferSize,
                            ViChar    connectionStatus[],
                            ViInt32*  returnedBytes);

ViStatus _VI_FUNC rsspecan_QueryMCGDConnectionStatus(ViSession instrSession,
                                                     ViInt32*  MCGDConnectionStatus);

ViStatus _VI_FUNC rsspecan_ConfigureMCGDSettingsAppliedToGenerator(
                            ViSession instrSession,
                            ViBoolean carrierCount,
                            ViBoolean centerFrequency,
                            ViBoolean carrierSpacing);

ViStatus _VI_FUNC rsspecan_UpdateMCGDGeneratorSettings(ViSession instrSession);

ViStatus _VI_FUNC rsspecan_TransferMCGDAllSettingsFromGenerator(ViSession instrSession);

ViStatus _VI_FUNC rsspecan_ConfigureMCGDGeneratorDUTSettings(ViSession instrSession,
                                                             ViReal64  DUTGain,
                                                             ViBoolean DUTLimitState,
                                                             ViReal64  DUTLimit);

ViStatus _VI_FUNC rsspecan_ConfigureMCGDGeneratorControlSettings(
                            ViSession instrSession,
                            ViBoolean controlState);

ViStatus _VI_FUNC rsspecan_QueryMCGDControlSettingsOverallStatus(
                            ViSession instrSession,
                            ViInt32*  overallSettingsStatus);

ViStatus _VI_FUNC rsspecan_ConfigureMCGDGeneratorLevelSettings(
                            ViSession instrSession,
                            ViBoolean referenceLevelState,
                            ViBoolean generatorAutoLevel,
                            ViReal64  generatorLevel);

ViStatus _VI_FUNC rsspecan_ConfigureMCGDGeneratorSettlingDelay(
                            ViSession instrSession,
                            ViReal64  settlingDelay);

ViStatus _VI_FUNC rsspecan_ConfigureMultiCarrierGroupDelayAFTMEnabled(
                            ViSession instrSession,
                            ViBoolean enabled);

ViStatus _VI_FUNC rsspecan_ConfigureMultiCarrierGroupDelayAFTMGeneratorFrequency(
                            ViSession instrSession,
                            ViReal64  analyzerFrequency,
                            ViReal64  generatorFrequency,
                            ViReal64  frequencyOffset,
                            ViReal64  numerator,
                            ViReal64  denominator);

ViStatus _VI_FUNC rsspecan_QueryMultiCarrierGroupDelayAFTMOverallStatus(
                            ViSession instrSession,
                            ViInt32*  AFTMOverallStatus);

ViStatus _VI_FUNC rsspecan_InitiateMultiCarrierGroupDelayCalibration(ViSession instrSession);

ViStatus _VI_FUNC rsspecan_QueryMultiCarrierGroupDelayCalibrationStatus(
                            ViSession instrSession,
                            ViInt32*  calibrationStatus);

ViStatus _VI_FUNC rsspecan_StoreMultiCarrierGroupDelayCalibrationData(
                            ViSession instrSession,
                            ViString  fileName);

ViStatus _VI_FUNC rsspecan_LoadMultiCarrierGroupDelayCalibrationData(
                            ViSession instrSession,
                            ViString  fileName);

ViStatus _VI_FUNC rsspecan_StoreMultiCarrierGroupDelayMeasurementResults(
                            ViSession instrSession,
                            ViString  fileName);

ViStatus _VI_FUNC rsspecan_StoreMultiCarrierGroupDelayTraceToFile(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   trace,
                            ViString  fileName);

ViStatus _VI_FUNC rsspecan_QueryMultiCarrierGroupDelayNumberOfTracePoints(
                            ViSession instrSession,
                            ViInt32   reserved,
                            ViInt32   trace,
                            ViInt32*  numberOfTracePoints);

ViStatus _VI_FUNC rsspecan_QueryMultiCarrierGroupDelayMeasurementBandwidth(
                            ViSession instrSession,
                            ViReal64* bandwidth);

ViStatus _VI_FUNC rsspecan_QueryMultiCarrierGroupDelayRecordLength(
                            ViSession instrSession,
                            ViInt32*  recordLength);

ViStatus _VI_FUNC rsspecan_QueryMultiCarrierGroupDelaySampleRate(
                            ViSession instrSession,
                            ViReal64* sampleRate);

ViStatus _VI_FUNC rsspecan_AmplifierMode(ViSession instrSession);

ViStatus _VI_FUNC rsspecan_GetAmplifierReferenceSignalInformation(
                            ViSession instrSession,
                            ViInt32*  sampleRate,
                            ViInt32*  sampleLength);

ViStatus _VI_FUNC rsspecan_GetAmplifierReferenceSignalCrestFactor(
                            ViSession instrSession,
                            ViReal64* crestFactor);

ViStatus _VI_FUNC rsspecan_GetAmplifierReferenceSignalOWB(ViSession instrSession,
                                                          ViReal64* OBW);

ViStatus _VI_FUNC rsspecan_GetAmplifierReferenceSignalWaveformFile(
                            ViSession instrSession,
                            ViInt32   bufferSize,
                            ViChar    waveformFile[]);

ViStatus _VI_FUNC rsspecan_ConfigureAmplifierForceARBMode(ViSession instrSession,
                                                          ViBoolean forceARBMode);

ViStatus _VI_FUNC rsspecan_AmplifierReadReferenceSignalFromSMW(ViSession instrSession);

ViStatus _VI_FUNC rsspecan_GetAmplifierReferenceSignalTransmissionState(
                            ViSession instrSession,
                            ViInt32*  transmissionState);

ViStatus _VI_FUNC rsspecan_ConfigureAmplifierReferenceSignalWaveformFileTransfer(
                            ViSession instrSession,
                            ViBoolean exportToGenerator,
                            ViReal64  DUTPeakInputPower);

ViStatus _VI_FUNC rsspecan_ConfigureAmplifierReferenceSignalWaveformFileSegment(
                            ViSession instrSession,
                            ViInt32   segment);

ViStatus _VI_FUNC rsspecan_ConfigureAmplifierReferenceSignalWaveformFilePath(
                            ViSession instrSession,
                            ViString  filePath);

ViStatus _VI_FUNC rsspecan_AmplifierWriteWaveformFileToSMW(ViSession instrSession);

ViStatus _VI_FUNC rsspecan_GetAmplifierWaveformFileTransmissionState(
                            ViSession instrSession,
                            ViInt32*  transmissionState);

ViStatus _VI_FUNC rsspecan_ConfigureAmplifierOwnReferenceSignal(
                            ViSession instrSession,
                            ViReal64  clockRate,
                            ViReal64  signalBandwidth,
                            ViInt32   sampleLength,
                            ViReal64  crestFactor,
                            ViReal64  notchWidth,
                            ViReal64  notchPosition,
                            ViInt32   dutyCycle,
                            ViInt32   rampLength);

ViStatus _VI_FUNC rsspecan_GetAmplifierOwnSignalTransmissionState(
                            ViSession instrSession,
                            ViInt32*  transmissionState);

ViStatus _VI_FUNC rsspecan_ConfigureAmplifierOwnSignalWaveformFileName(
                            ViSession instrSession,
                            ViString  filePath);

ViStatus _VI_FUNC rsspecan_GenerateAmplifierOwnSignalAndPlay(ViSession instrSession);

ViStatus _VI_FUNC rsspecan_GetAmplifierReferenceSignalDuration(
                            ViSession instrSession,
                            ViReal64* signalDuration);

ViStatus _VI_FUNC rsspecan_ConfigureAmplifierCrestFactorReduction(
                            ViSession instrSession,
                            ViBoolean CFREnabled,
                            ViReal64  crestFactorDelta,
                            ViInt32   maxIterations);

ViStatus _VI_FUNC rsspecan_ConfigureAmplifierCrestFactorReductionSimpleMode(
                            ViSession instrSession,
                            ViBoolean signalBandwidthAuto,
                            ViInt32   signalBandwidth,
                            ViBoolean channelSpacingAuto,
                            ViInt32   channelSpacing);

ViStatus _VI_FUNC rsspecan_ConfigureAmplifierCrestFactorReductionFilterMode(
                            ViSession instrSession,
                            ViInt32   filterMode);

ViStatus _VI_FUNC rsspecan_GetAmplifierCrestFactorReductionFilterModeState(
                            ViSession instrSession,
                            ViInt32*  state);

ViStatus _VI_FUNC rsspecan_ConfigureAmplifierCrestFactorReductionMaximumFilterOrder(
                            ViSession instrSession,
                            ViInt32   maxFilterOrder);

ViStatus _VI_FUNC rsspecan_QueryAmplifierCrestFactorReductionMaximumFilterOrderState(
                            ViSession instrSession,
                            ViInt32*  state);

ViStatus _VI_FUNC rsspecan_ConfigureAmplifierCrestFactorReductionAttenuationFrequencies(
                            ViSession instrSession,
                            ViReal64  passbandFrequency,
                            ViReal64  stopbandFrequency);

ViStatus _VI_FUNC rsspecan_GetAmplifierCrestFactorReductionAttenuationFrequenciesState(
                            ViSession instrSession,
                            ViInt32*  passbandState,
                            ViInt32*  stopbandState);

ViStatus _VI_FUNC rsspecan_ApplyAmplifierCrestFactorReduction(ViSession instrSession);

ViStatus _VI_FUNC rsspecan_ApplyAmplifierCrestFactorReductionFromGenerator(ViSession instrSession);

ViStatus _VI_FUNC rsspecan_GetAmplifierCrestFactorReductionFromGenerator(
                            ViSession instrSession,
                            ViInt32*  fromGeneratorAndLoadState);

ViStatus _VI_FUNC rsspecan_ConfigureAmplifierCrestFactorReductionEVMRefSignal(
                            ViSession instrSession,
                            ViInt32   EVMRefSignal);

ViStatus _VI_FUNC rsspecan_GetAmplifierCrestFactorReductionEnabledState(
                            ViSession instrSession,
                            ViInt32*  CFREnabledState);

ViStatus _VI_FUNC rsspecan_GetAmplifierCrestFactorReductionCurrentValue(
                            ViSession instrSession,
                            ViReal64* currentCrestFactor);

ViStatus _VI_FUNC rsspecan_GetAmplifierCrestFactorReductionDeltaState(
                            ViSession instrSession,
                            ViInt32*  crestFactorDeltaState);

ViStatus _VI_FUNC rsspecan_GetAmplifierCrestFactorReductionIterationsState(
                            ViSession instrSession,
                            ViInt32*  maxIterationsState);

ViStatus _VI_FUNC rsspecan_GetAmplifierCrestFactorReductionSignalBandwidthState(
                            ViSession instrSession,
                            ViInt32*  signalBandwidthState);

ViStatus _VI_FUNC rsspecan_GetAmplifierCrestFactorReductionChannelSpacingState(
                            ViSession instrSession,
                            ViInt32*  channelSpacingState);

ViStatus _VI_FUNC rsspecan_GetAmplifierCrestFactorReductionApplyState(
                            ViSession instrSession,
                            ViInt32*  applyState);

ViStatus _VI_FUNC rsspecan_ConfigureAmplifierInputSourceConnector(
                            ViSession instrSession,
                            ViInt32   inputConnector);

ViStatus _VI_FUNC rsspecan_ConfigureAmplifierInputSourceAnalogBasebandCapture(
                            ViSession instrSession,
                            ViInt32   input,
                            ViBoolean enableParallelBBCapture);

ViStatus _VI_FUNC rsspecan_ConfigureAmplifierGeneratorSettings(
                            ViSession instrSession,
                            ViString  generatorIPAddress,
                            ViReal64  RMSLevel,
                            ViBoolean attachToFSWFrequency,
                            ViReal64  centerFrequency,
                            ViInt32   externalOscilator,
                            ViInt32   pathRF);

ViStatus _VI_FUNC rsspecan_ConfigureAmplifierGeneratorLevelOffset(
                            ViSession instrSession,
                            ViReal64  levelOffset);

ViStatus _VI_FUNC rsspecan_ConfigureAmplifierGeneratorDigitalAttenuation(
                            ViSession instrSession,
                            ViReal64  attenuation);

ViStatus _VI_FUNC rsspecan_ConfigureAmplifierGeneratorRFOutput(
                            ViSession instrSession,
                            ViBoolean state);

ViStatus _VI_FUNC rsspecan_ConfigureAmplifierGeneratorSegment(
                            ViSession instrSession,
                            ViInt32   segment);

ViStatus _VI_FUNC rsspecan_ConfigureAmplifierGeneratorMaximumPower(
                            ViSession instrSession,
                            ViReal64  maximumPower);

ViStatus _VI_FUNC rsspecan_QueryAmplifierGeneratorMaximumPowerLEDState(
                            ViSession instrSession,
                            ViInt32*  maximumPowerLEDState);

ViStatus _VI_FUNC rsspecan_ConfigureAmplifierGeneratorControlEnabled(
                            ViSession instrSession,
                            ViBoolean generatorControl);

ViStatus _VI_FUNC rsspecan_GetAmplifierGeneratorSetupLedState(
                            ViSession instrSession,
                            ViInt32   type,
                            ViInt32*  ledState);

ViStatus _VI_FUNC rsspecan_WriteAmplifierGeneratorCommand(ViSession instrSession,
                                                          ViString  commandBuffer);

ViStatus _VI_FUNC rsspecan_QueryAmplifierGeneratorResponse(ViSession instrSession,
                                                           ViInt32   bufferSize,
                                                           ViChar    instrumentResponse[],
                                                           ViInt32*  returnedBytes);

ViStatus _VI_FUNC rsspecan_UpdateAmplifierGeneratorSetupSettings(ViSession instrSession);

ViStatus _VI_FUNC rsspecan_UpdateAmplifierSettingsFromGenerator(ViSession instrSession);

ViStatus _VI_FUNC rsspecan_GetAmplifierGeneratorTargetPathBaseband(
                            ViSession instrSession,
                            ViInt32*  pathBB);

ViStatus _VI_FUNC rsspecan_ConfigureAmplifierAveragingIQData(ViSession instrSession,
                                                             ViBoolean state,
                                                             ViInt32   count);

ViStatus _VI_FUNC rsspecan_QueryAmplifierAveragingIQDataCount(
                            ViSession instrSession,
                            ViInt32*  count);

ViStatus _VI_FUNC rsspecan_ConfigureAmplifierSweepConfigurationStatisticsSettings(
                            ViSession instrSession,
                            ViBoolean state,
                            ViInt32   mode,
                            ViBoolean continuous,
                            ViInt32   count);

ViStatus _VI_FUNC rsspecan_ConfigureAmplifierSweepConfigurationSelectResultRange(
                            ViSession instrSession,
                            ViInt32   range);

ViStatus _VI_FUNC rsspecan_ConfigureAmplifierSynchronizationState(
                            ViSession instrSession,
                            ViBoolean state);

ViStatus _VI_FUNC rsspecan_ConfigureAmplifierSynchronizationConfidenceLevel(
                            ViSession instrSession,
                            ViReal64  confidenceLevel);

ViStatus _VI_FUNC rsspecan_ConfigureAmplifierMeasurementStop(ViSession instrSession,
                                                             ViBoolean measurementStop);

ViStatus _VI_FUNC rsspecan_ConfigureAmplifierSynchronizationDomain(
                            ViSession instrSession,
                            ViInt32   synchronizationDomain);

ViStatus _VI_FUNC rsspecan_ConfigureAmplifierEstimationRange(ViSession instrSession,
                                                             ViBoolean useFullRefSignal,
                                                             ViReal64  syncStart,
                                                             ViReal64  syncStop);

ViStatus _VI_FUNC rsspecan_QueryAmplifierSynchronizationState(
                            ViSession  instrSession,
                            ViBoolean* state);

ViStatus _VI_FUNC rsspecan_QueryAmplifierIQSynchronizationData(
                            ViSession instrSession,
                            ViInt32   impedanceMode,
                            ViInt32   arrayLength,
                            ViInt32*  actualPoints,
                            ViReal64  data[]);

ViStatus _VI_FUNC rsspecan_ConfigureAmplifierEvaluationRange(ViSession instrSession,
                                                             ViBoolean useFullRefSignal,
                                                             ViReal64  evalStart,
                                                             ViReal64  evalStop);

ViStatus _VI_FUNC rsspecan_ConfigureAmplifierEstimationStates(
                            ViSession instrSession,
                            ViBoolean amplitudeDroop,
                            ViBoolean iQImbalance,
                            ViBoolean sampleErrorRate);

ViStatus _VI_FUNC rsspecan_ConfigureAmplifierEstimationFrequencyErrorState(
                            ViSession instrSession,
                            ViBoolean state);

ViStatus _VI_FUNC rsspecan_ConfigureAmplifierEstimationIQOffsetState(
                            ViSession instrSession,
                            ViBoolean state);

ViStatus _VI_FUNC rsspecan_ConfigureAmplifierCompensationStates(
                            ViSession instrSession,
                            ViBoolean amplitudeDroop,
                            ViBoolean iQImbalance,
                            ViBoolean sampleErrorRate);

ViStatus _VI_FUNC rsspecan_ConfigureAmplifierCompensationFrequencyErrorState(
                            ViSession instrSession,
                            ViBoolean state);

ViStatus _VI_FUNC rsspecan_ConfigureAmplifierCompensationIQOffsetState(
                            ViSession instrSession,
                            ViBoolean state);

ViStatus _VI_FUNC rsspecan_ConfigureAmplifierEqualizerFilterLengthForTraining(
                            ViSession instrSession,
                            ViInt32   filterLength);

ViStatus _VI_FUNC rsspecan_AmplifierTrainEqualizerFilter(ViSession instrSession);

ViStatus _VI_FUNC rsspecan_AmplifierSaveEqualizerFilter(ViSession instrSession,
                                                        ViString  fileName);

ViStatus _VI_FUNC rsspecan_AmplifierLoadEqualizerFilter(ViSession instrSession,
                                                        ViString  fileName);

ViStatus _VI_FUNC rsspecan_ConfigureAmplifierEqualizerState(ViSession instrSession,
                                                            ViBoolean equalizerState);

ViStatus _VI_FUNC rsspecan_ConfigureAmplifierEqualizerFilterFileFormat(
                            ViSession instrSession,
                            ViInt32   format);

ViStatus _VI_FUNC rsspecan_AmplifierEqualizerManualFilterDefinition(
                            ViSession instrSession,
                            ViInt32   arraysLength,
                            ViInt32   iCoefficients[],
                            ViInt32   qCoefficients[]);

ViStatus _VI_FUNC rsspecan_ConfigureAmplifierSystemModelSettings(
                            ViSession instrSession,
                            ViBoolean modelingState,
                            ViString  modelingOrderAMPM,
                            ViString  modelingOrderAMAM,
                            ViReal64  modelingRange,
                            ViInt32   modelingPoints);

ViStatus _VI_FUNC rsspecan_ConfigureAmplifierSystemModelSequence(
                            ViSession instrSession,
                            ViInt32   sequence);

ViStatus _VI_FUNC rsspecan_ConfigureAmplifierSystemModelScale(
                            ViSession instrSession,
                            ViInt32   scale);

ViStatus _VI_FUNC rsspecan_ConfigureAmplifierDPDMethod(ViSession instrSession,
                                                       ViInt32   method);

ViStatus _VI_FUNC rsspecan_GenerateDPDWaveformFile(ViSession instrSession);

ViStatus _VI_FUNC rsspecan_GenerateDPDWaveformFileAll(ViSession instrSession);

ViStatus _VI_FUNC rsspecan_AmplifierDPDUpdateShappingTable(ViSession instrSession);

ViStatus _VI_FUNC rsspecan_AmplifierDPDUpdateAllShapingTables(ViSession instrSession);

ViStatus _VI_FUNC rsspecan_GetAmplifierDPDUpdateShappingTableStatus(
                            ViSession instrSession,
                            ViInt32*  shapingTableState);

ViStatus _VI_FUNC rsspecan_ConfigureAmplifierDPDShaping(ViSession instrSession,
                                                        ViInt32   shapingMethod,
                                                        ViString  DPDFileName,
                                                        ViInt32   DPDSequence,
                                                        ViBoolean AMAMState,
                                                        ViBoolean AMPMState);

ViStatus _VI_FUNC rsspecan_ConfigureAmplifierDPDAMxMState(ViSession instrSession,
                                                          ViBoolean state);

ViStatus _VI_FUNC rsspecan_ConfigureDPDPowerLinearityTradeoff(
                            ViSession instrSession,
                            ViInt32   tradeoff);

ViStatus _VI_FUNC rsspecan_QueryAmplifierDPDWaveformPath(ViSession instrSession,
                                                         ViInt32   bufferSize,
                                                         ViChar    path[],
                                                         ViInt32*  bytesRead);

ViStatus _VI_FUNC rsspecan_ExportDPDWaveformToFile(ViSession instrSession,
                                                   ViString  filename);

ViStatus _VI_FUNC rsspecan_GetAmplifierDPDLEDState(ViSession instrSession,
                                                   ViInt32*  AMAMState,
                                                   ViInt32*  AMPMState);

ViStatus _VI_FUNC rsspecan_ConfigureAmplifierDPDState(ViSession instrSession,
                                                      ViBoolean directDPD);

ViStatus _VI_FUNC rsspecan_AmplifierDirectDPDAction(ViSession instrSession,
                                                    ViInt32   action);

ViStatus _VI_FUNC rsspecan_ConfigureAmplifierDirectDPDGainExpansion(
                            ViSession instrSession,
                            ViReal64  gainExpansion);

ViStatus _VI_FUNC rsspecan_ConfigureAmplifierDirectDPDIterations(
                            ViSession instrSession,
                            ViInt32   iterations);

ViStatus _VI_FUNC rsspecan_QueryAmplifierDirectDPDCurrentIteration(
                            ViSession instrSession,
                            ViInt32*  numberOfCurrentIteration);

ViStatus _VI_FUNC rsspecan_ConfigureAmplifierDirectDPDPowerLinearityTradeoff(
                            ViSession instrSession,
                            ViReal64  tradeoff);

ViStatus _VI_FUNC rsspecan_ConfigureAmplifierDPDNameOnGenerator(
                            ViSession instrSession,
                            ViString  nameOnGenerator);

ViStatus _VI_FUNC rsspecan_StoreAmplifierDirectDPDWaveformFile(
                            ViSession instrSession,
                            ViString  waveformFile);

ViStatus _VI_FUNC rsspecan_ApplyAmplifierDirectDPD(ViSession instrSession,
                                                   ViBoolean apply);

ViStatus _VI_FUNC rsspecan_ConfigureAmplifierDirectDPDWrapEnabled(
                            ViSession instrSession,
                            ViBoolean wrap);

ViStatus _VI_FUNC rsspecan_QueryAmplifierDirectDPDRMSCurrent(ViSession instrSession,
                                                             ViReal64* RMSCurrent);

ViStatus _VI_FUNC rsspecan_QueryAmplifierDirectDPDWaveformPath(
                            ViSession instrSession,
                            ViInt32   bufferSize,
                            ViChar    path[],
                            ViInt32*  bytesRead);

ViStatus _VI_FUNC rsspecan_QueryAmplifierDirectDPDOperationStatus(
                            ViSession  instrSession,
                            ViBoolean* operationStatus);

ViStatus _VI_FUNC rsspecan_ConfigureAmplifierMemoryPolynomialDPDSettings(
                            ViSession instrSession,
                            ViBoolean state,
                            ViInt32   iterationStep,
                            ViString  polynomialOrder,
                            ViString  memoryOrder);

ViStatus _VI_FUNC rsspecan_CalculateAmplifierMemoryPolynomialDPDModel(ViSession instrSession);

ViStatus _VI_FUNC rsspecan_ConfigureAmplifierMemoryPolynomialDPDModelApply(
                            ViSession instrSession,
                            ViString  waveform);

ViStatus _VI_FUNC rsspecan_ConfigureAmplifierMemoryPolynomialDPDExportWaveformFile(
                            ViSession instrSession,
                            ViString  waveformFilePath);

ViStatus _VI_FUNC rsspecan_ConfigureAmplifierMemoryPolynomialDPDExportCoefficientsFile(
                            ViSession instrSession,
                            ViString  coefficientsFilePath);

ViStatus _VI_FUNC rsspecan_QueryAmplifierMemoryPolynomialDPDWaveformPath(
                            ViSession instrSession,
                            ViInt32   bufferSize,
                            ViChar    path[],
                            ViInt32*  bytesRead);

ViStatus _VI_FUNC rsspecan_QueryAmplifierMemoryPolynomialDPDRMSCurrent(
                            ViSession instrSession,
                            ViReal64* RMS);

ViStatus _VI_FUNC rsspecan_ConfigureAmplifierMemoryPolynomialDPDWaveformType(
                            ViSession instrSession,
                            ViInt32   waveformType);

ViStatus _VI_FUNC rsspecan_SendAmplifierMemoryPolynomialDPDWaveformToGenerator(ViSession instrSession);

ViStatus _VI_FUNC rsspecan_QueryAmplifierDirectDPDPolynomial(ViSession instrSession,
                                                             ViInt32   arraySize,
                                                             ViReal64  polynomialData[],
                                                             ViInt32*  numberOfDataPoints);

ViStatus _VI_FUNC rsspecan_ConfigureAmplifierHammersteinEnabled(
                            ViSession instrSession,
                            ViBoolean hammerstein);

ViStatus _VI_FUNC rsspecan_ConfigureAmplifierHammersteinGeneratorWaveform(
                            ViSession instrSession,
                            ViInt32   generatorWaveform);

ViStatus _VI_FUNC rsspecan_ConfigureAmplifierHammersteinIterationStep(
                            ViSession instrSession,
                            ViInt32   iterationStep);

ViStatus _VI_FUNC rsspecan_ConfigureAmplifierHammersteinMemoryOrder(
                            ViSession instrSession,
                            ViString  memoryOrder);

ViStatus _VI_FUNC rsspecan_ConfigureAmplifierHammersteinPolynomialOrder(
                            ViSession instrSession,
                            ViString  polynomialOrder);

ViStatus _VI_FUNC rsspecan_ConfigureAmplifierHammersteinNonLinearityEnabled(
                            ViSession instrSession,
                            ViBoolean nonLinearity);

ViStatus _VI_FUNC rsspecan_ConfigureAmplifierHammersteinFilterEnabled(
                            ViSession instrSession,
                            ViBoolean filter);

ViStatus _VI_FUNC rsspecan_StartAmplifierHammersteinGenerator(ViSession instrSession);

ViStatus _VI_FUNC rsspecan_CalculateAmplifierMSE(ViSession instrSession,
                                                 ViReal64* MSE);

ViStatus _VI_FUNC rsspecan_ConfigureAmplifierEnvelopeTracking(
                            ViSession instrSession,
                            ViReal64  iChannelMultiplier,
                            ViReal64  iChannelOffset,
                            ViReal64  iChannelResistance,
                            ViReal64  qChannelMultiplier,
                            ViReal64  qChannelOffset,
                            ViReal64  powerConsumptionParamererA,
                            ViReal64  powerConsumptionParameterB);

ViStatus _VI_FUNC rsspecan_ConfigureAmplifierPowerEvaluateOnlyDUTPower(
                            ViSession instrSession,
                            ViBoolean state);

ViStatus _VI_FUNC rsspecan_ConfigureAmplifierPowerCompressionPointCalculation(
                            ViSession instrSession,
                            ViBoolean referenceInputPower,
                            ViReal64  referencePoint);

ViStatus _VI_FUNC rsspecan_ConfigureAmplifierParameterSweepState(
                            ViSession instrSession,
                            ViBoolean state);

ViStatus _VI_FUNC rsspecan_ConfigureAmplifierParameterSweepXAxis(
                            ViSession instrSession,
                            ViInt32   setting,
                            ViReal64  start,
                            ViReal64  stop,
                            ViReal64  step);

ViStatus _VI_FUNC rsspecan_ConfigureAmplifierParameterSweepYAxisState(
                            ViSession instrSession,
                            ViBoolean state);

ViStatus _VI_FUNC rsspecan_ConfigureAmplifierParameterSweepYAxis(
                            ViSession instrSession,
                            ViInt32   setting,
                            ViReal64  start,
                            ViReal64  stop,
                            ViReal64  step);

ViStatus _VI_FUNC rsspecan_ConfigureAmplifierParameterSweepAdjustLevel(
                            ViSession instrSession,
                            ViBoolean state,
                            ViReal64  gain);

ViStatus _VI_FUNC rsspecan_ConfigureAmplifierPowerServoing(ViSession instrSession,
                                                           ViReal64  maximumAmplifierInputPower,
                                                           ViReal64  expectedAmplifierGain);

ViStatus _VI_FUNC rsspecan_ConfigureAmplifierPowerServoingTargetSettings(
                            ViSession instrSession,
                            ViReal64  targetOutputPower,
                            ViReal64  maxOutputPowerDeviation);

ViStatus _VI_FUNC rsspecan_ConfigureAmplifierPowerServoingDataAcquisition(
                            ViSession instrSession,
                            ViBoolean state,
                            ViReal64  measurementTime,
                            ViInt32   captureLength);

ViStatus _VI_FUNC rsspecan_ConfigureAmplifierPowerServoingStart(ViSession instrSession);

ViStatus _VI_FUNC rsspecan_ConfigureAmplifierPowerServoingStop(ViSession instrSession);

ViStatus _VI_FUNC rsspecan_ConfigureAmplifierSensePowerServoingEnabled(
                            ViSession instrSession,
                            ViBoolean enabled);

ViStatus _VI_FUNC rsspecan_ConfigureAmplifierSensePowerServoingTargetParameter(
                            ViSession instrSession,
                            ViInt32   targetParameter);

ViStatus _VI_FUNC rsspecan_ConfigureAmplifierSensePowerServoingTargetSettings(
                            ViSession instrSession,
                            ViInt32   targetValue,
                            ViReal64  targetTolerance);

ViStatus _VI_FUNC rsspecan_ConfigureAmplifierSensePowerServoingMaximumIterarions(
                            ViSession instrSession,
                            ViInt32   maximumIterarions);

ViStatus _VI_FUNC rsspecan_ConfigureAmplifierSensePowerServoingGeneratorLevel(
                            ViSession instrSession,
                            ViInt32   levelControl);

ViStatus _VI_FUNC rsspecan_ConfigureAmplifierSensePowerServoingStepSize(
                            ViSession instrSession,
                            ViReal64  stepSize);

ViStatus _VI_FUNC rsspecan_AmplifierSensePowerServoing(ViSession instrSession);

ViStatus _VI_FUNC rsspecan_QueryAmplifierSensePowerServoingStatus(
                            ViSession  instrSession,
                            ViBoolean* status);

ViStatus _VI_FUNC rsspecan_AmplifierAddWindow(ViSession instrSession,
                                              ViInt32   stringSize,
                                              ViString  referenceWindowName,
                                              ViInt32   direction,
                                              ViInt32   windowType,
                                              ViChar    returnedWindowName[]);

ViStatus _VI_FUNC rsspecan_AmplifierReplaceWindow(ViSession instrSession,
                                                  ViString  windowName,
                                                  ViInt32   windowType);

ViStatus _VI_FUNC rsspecan_AmplifierTraceMode(ViSession instrSession,
                                              ViInt32   trace,
                                              ViInt32   traceMode);

ViStatus _VI_FUNC rsspecan_ConfigureAmplifierTraceDetectorSettings(
                            ViSession instrSession,
                            ViInt32   trace,
                            ViInt32   defaultDetector,
                            ViInt32   maxTracePoints);

ViStatus _VI_FUNC rsspecan_ConfigureAmplifierWindowTraceSettings(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   trace,
                            ViInt32   mode,
                            ViInt32   type,
                            ViInt32   detector);

ViStatus _VI_FUNC rsspecan_DisplayAmplifierWindowStatisticsTableItem(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   item,
                            ViBoolean state);

ViStatus _VI_FUNC rsspecan_DisplayAmplifierWindowStatisticsTableItemsAll(
                            ViSession instrSession,
                            ViInt32   window,
                            ViBoolean MACC,
                            ViBoolean power,
                            ViBoolean voltCurr);

ViStatus _VI_FUNC rsspecan_ConfigureAmplifierResultSummaryDisplay(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   item,
                            ViBoolean displayItem);

ViStatus _VI_FUNC rsspecan_ConfigureAmplifierResultSummaryDisplayAll(
                            ViSession instrSession,
                            ViInt32   item,
                            ViBoolean displayItem);

ViStatus _VI_FUNC rsspecan_ConfigureAmplifierWindowResultSummaryDisplayAll(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   item,
                            ViBoolean displayItem);

ViStatus _VI_FUNC rsspecan_ConfigureAmplifierTimeDomainDisplaySettings(
                            ViSession instrSession,
                            ViBoolean normaliseTo1,
                            ViBoolean position,
                            ViReal64  offset,
                            ViReal64  duration);

ViStatus _VI_FUNC rsspecan_ConfigureAmplifierPhaseDisplaySettings(
                            ViSession instrSession,
                            ViInt32   unit,
                            ViInt32   AMPMDefinition);

ViStatus _VI_FUNC rsspecan_ConfigureAmplifierAMPMCurveWidthReferencePoint(
                            ViSession instrSession,
                            ViBoolean automatic,
                            ViReal64  referencePoint);

ViStatus _VI_FUNC rsspecan_ConfigureAmplifierPowerReferenceDisplaySettings(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   powerReference);

ViStatus _VI_FUNC rsspecan_ConfigureAmplifierResultParameterSweepTable(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   item,
                            ViBoolean displayItem);

ViStatus _VI_FUNC rsspecan_ConfigureAmplifierResultParameterSweepTableAll(
                            ViSession instrSession,
                            ViBoolean displayItem);

ViStatus _VI_FUNC rsspecan_ConfigureAmplifierWindowResultParameterSweepTableAll(
                            ViSession instrSession,
                            ViInt32   window,
                            ViBoolean displayItem);

ViStatus _VI_FUNC rsspecan_ConfigureAmplifierResultParameterSweepDisplay(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   item);

ViStatus _VI_FUNC rsspecan_ConfigureAmplifierDisplayWindowDDPDResultsType(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   resultType);

ViStatus _VI_FUNC rsspecan_GetAmplifierAllResultSummary(ViSession instrSession,
                                                        ViInt32   arrayLength,
                                                        ViReal64  results[],
                                                        ViInt32*  actualPoints);

ViStatus _VI_FUNC rsspecan_GetAmplifierPowerAllResult(ViSession instrSession,
                                                      ViInt32   arrayLength,
                                                      ViReal64  results[],
                                                      ViInt32*  actualPoints);

ViStatus _VI_FUNC rsspecan_GetAmplifierModulationAccuracyResults(
                            ViSession instrSession,
                            ViInt32   resultType,
                            ViInt32   measurementType,
                            ViReal64* result);

ViStatus _VI_FUNC rsspecan_GetAmplifierPhaseErrorResults(ViSession instrSession,
                                                         ViReal64* result);

ViStatus _VI_FUNC rsspecan_GetAmplifierPowerResults(ViSession instrSession,
                                                    ViInt32   resultType,
                                                    ViInt32   measurementType,
                                                    ViReal64* result);

ViStatus _VI_FUNC rsspecan_GetAmplifierPowerResultsInputSensor(
                            ViSession instrSession,
                            ViInt32   measurementType,
                            ViReal64* result);

ViStatus _VI_FUNC rsspecan_GetAmplifierPowerResultsOutputSensor(
                            ViSession instrSession,
                            ViInt32   measurementType,
                            ViReal64* result);

ViStatus _VI_FUNC rsspecan_GetAmplifierAMPMCurveWidthPeakCurrent(
                            ViSession instrSession,
                            ViReal64* result);

ViStatus _VI_FUNC rsspecan_GetAmplifierAMAMCurveWidthPeakCurrent(
                            ViSession instrSession,
                            ViReal64* result);

ViStatus _VI_FUNC rsspecan_GetAmplifierCompressionPointsOutputPower(
                            ViSession instrSession,
                            ViInt32   compressionPoint,
                            ViInt32   measurementType,
                            ViReal64* power);

ViStatus _VI_FUNC rsspecan_GetAmplifierBasebandCharacteristicsResults(
                            ViSession instrSession,
                            ViInt32   resultType,
                            ViInt32   measurementType,
                            ViReal64* result);

ViStatus _VI_FUNC rsspecan_GetAmplifierBasebandPowerResults(ViSession instrSession,
                                                            ViInt32   measurementType,
                                                            ViReal64* result);

ViStatus _VI_FUNC rsspecan_GetAmplifierParameterSweepTableResults(
                            ViSession instrSession,
                            ViInt32   resultType,
                            ViInt32   measurementType,
                            ViReal64* result);

ViStatus _VI_FUNC rsspecan_GetAmplifierParameterSweepTableResultsPosition(
                            ViSession instrSession,
                            ViInt32   resultType,
                            ViInt32   measurementType,
                            ViInt32   axis,
                            ViReal64* result);

ViStatus _VI_FUNC rsspecan_GetAmplifierParameterTablePowerOutputResults(
                            ViSession instrSession,
                            ViInt32   measurementType,
                            ViInt32   axis,
                            ViReal64* result);

ViStatus _VI_FUNC rsspecan_GetAmplifierCompressionPointsPower(
                            ViSession instrSession,
                            ViInt32   compressionPoint,
                            ViInt32   measurementType,
                            ViReal64* power);

ViStatus _VI_FUNC rsspecan_GetAmplifierTriggerToSyncResult(ViSession instrSession,
                                                           ViReal64* syncResult);

ViStatus _VI_FUNC rsspecan_GetAmplifierMemoryDPDCoefficients(ViSession instrSession,
                                                             ViInt32   bufferSize,
                                                             ViChar    DPDCoefficients[]);

ViStatus _VI_FUNC rsspecan_GetAmplifierCompressionPointSweepTableResults(
                            ViSession instrSession,
                            ViInt32   compressionPoint,
                            ViInt32   measurementType,
                            ViInt32   axis,
                            ViReal64* result);

ViStatus _VI_FUNC rsspecan_GetAmplifierGeneralNumericResults(ViSession instrSession,
                                                             ViInt32   resultType,
                                                             ViInt32   measurementType,
                                                             ViReal64* result);

ViStatus _VI_FUNC rsspecan_GetAmplifierStatisticsTableAmplitudeDroopAll(
                            ViSession instrSession,
                            ViInt32   measurementType,
                            ViReal64* result);

ViStatus _VI_FUNC rsspecan_GetAmplifierStatisticsTableAmplitudeDroopCurrent(
                            ViSession instrSession,
                            ViInt32   measurementType,
                            ViReal64* result);

ViStatus _VI_FUNC rsspecan_GetAmplifierStatisticsTableAmplitudeDroopSelectedResult(
                            ViSession instrSession,
                            ViReal64* result);

ViStatus _VI_FUNC rsspecan_GetAmplifierStatisticsTableAMAMCurveWidthAll(
                            ViSession instrSession,
                            ViInt32   measurementType,
                            ViReal64* result);

ViStatus _VI_FUNC rsspecan_GetAmplifierStatisticsTableAMAMCurveWidthCurrent(
                            ViSession instrSession,
                            ViInt32   measurementType,
                            ViReal64* result);

ViStatus _VI_FUNC rsspecan_GetAmplifierStatisticsTableAMAMCurveWidthSelectedResult(
                            ViSession instrSession,
                            ViReal64* result);

ViStatus _VI_FUNC rsspecan_GetAmplifierStatisticsTableAMPMCurveWidthAll(
                            ViSession instrSession,
                            ViInt32   measurementType,
                            ViReal64* result);

ViStatus _VI_FUNC rsspecan_GetAmplifierStatisticsTableAMPMCurveWidthCurrent(
                            ViSession instrSession,
                            ViInt32   measurementType,
                            ViReal64* result);

ViStatus _VI_FUNC rsspecan_GetAmplifierStatisticsTableAMPMCurveWidthSelectedResult(
                            ViSession instrSession,
                            ViReal64* result);

ViStatus _VI_FUNC rsspecan_GetAmplifierStatisticsTableAveragePAEAverageAll(
                            ViSession instrSession,
                            ViInt32   measurementType,
                            ViReal64* result);

ViStatus _VI_FUNC rsspecan_GetAmplifierStatisticsTableAveragePAEAverageCurrent(
                            ViSession instrSession,
                            ViInt32   measurementType,
                            ViReal64* result);

ViStatus _VI_FUNC rsspecan_GetAmplifierStatisticsTableAveragePAEAverageSelectedResult(
                            ViSession instrSession,
                            ViReal64* result);

ViStatus _VI_FUNC rsspecan_GetAmplifierStatisticsTableAveragePAEMaximumAll(
                            ViSession instrSession,
                            ViInt32   measurementType,
                            ViReal64* result);

ViStatus _VI_FUNC rsspecan_GetAmplifierStatisticsTableAveragePAEMaximumCurrent(
                            ViSession instrSession,
                            ViInt32   measurementType,
                            ViReal64* result);

ViStatus _VI_FUNC rsspecan_GetAmplifierStatisticsTableAveragePAEMaximumSelectedResult(
                            ViSession instrSession,
                            ViReal64* result);

ViStatus _VI_FUNC rsspecan_GetAmplifierStatisticsTableAveragePAEMinimumAll(
                            ViSession instrSession,
                            ViInt32   measurementType,
                            ViReal64* result);

ViStatus _VI_FUNC rsspecan_GetAmplifierStatisticsTableAveragePAEMinimumCurrent(
                            ViSession instrSession,
                            ViInt32   measurementType,
                            ViReal64* result);

ViStatus _VI_FUNC rsspecan_GetAmplifierStatisticsTableAveragePAEMinimumSelectedResult(
                            ViSession instrSession,
                            ViReal64* result);

ViStatus _VI_FUNC rsspecan_GetAmplifierStatisticsTablePowerVccxIccAverageAll(
                            ViSession instrSession,
                            ViInt32   measurementType,
                            ViReal64* result);

ViStatus _VI_FUNC rsspecan_GetAmplifierStatisticsTablePowerVccxIccAverageCurrent(
                            ViSession instrSession,
                            ViInt32   measurementType,
                            ViReal64* result);

ViStatus _VI_FUNC rsspecan_GetAmplifierStatisticsTablePowerVccxIccAverageSelectedResult(
                            ViSession instrSession,
                            ViReal64* result);

ViStatus _VI_FUNC rsspecan_GetAmplifierStatisticsTablePowerVccxIccMaximumAll(
                            ViSession instrSession,
                            ViInt32   measurementType,
                            ViReal64* result);

ViStatus _VI_FUNC rsspecan_GetAmplifierStatisticsTablePowerVccxIccMaximumCurrent(
                            ViSession instrSession,
                            ViInt32   measurementType,
                            ViReal64* result);

ViStatus _VI_FUNC rsspecan_GetAmplifierStatisticsTablePowerVccxIccMaximumSelectedResult(
                            ViSession instrSession,
                            ViReal64* result);

ViStatus _VI_FUNC rsspecan_GetAmplifierStatisticsTablePowerVccxIccMinimumAll(
                            ViSession instrSession,
                            ViInt32   measurementType,
                            ViReal64* result);

ViStatus _VI_FUNC rsspecan_GetAmplifierStatisticsTablePowerVccxIccMinimumCurrent(
                            ViSession instrSession,
                            ViInt32   measurementType,
                            ViReal64* result);

ViStatus _VI_FUNC rsspecan_GetAmplifierStatisticsTablePowerVccxIccMinimumSelectedResult(
                            ViSession instrSession,
                            ViReal64* result);

ViStatus _VI_FUNC rsspecan_GetAmplifierStatisticsTableCrestFactorInputAll(
                            ViSession instrSession,
                            ViInt32   measurementType,
                            ViReal64* result);

ViStatus _VI_FUNC rsspecan_GetAmplifierStatisticsTableCrestFactorInputCurrent(
                            ViSession instrSession,
                            ViInt32   measurementType,
                            ViReal64* result);

ViStatus _VI_FUNC rsspecan_GetAmplifierStatisticsTableCrestFactorInputSelectedResult(
                            ViSession instrSession,
                            ViReal64* result);

ViStatus _VI_FUNC rsspecan_GetAmplifierStatisticsTableCrestFactorOutputAll(
                            ViSession instrSession,
                            ViInt32   measurementType,
                            ViReal64* result);

ViStatus _VI_FUNC rsspecan_GetAmplifierStatisticsTableCrestFactorOutputCurrent(
                            ViSession instrSession,
                            ViInt32   measurementType,
                            ViReal64* result);

ViStatus _VI_FUNC rsspecan_GetAmplifierStatisticsTableCrestFactorOutputSelectedResult(
                            ViSession instrSession,
                            ViReal64* result);

ViStatus _VI_FUNC rsspecan_GetAmplifierStatisticsTableFrequencyErrorAll(
                            ViSession instrSession,
                            ViInt32   measurementType,
                            ViReal64* result);

ViStatus _VI_FUNC rsspecan_GetAmplifierStatisticsTableFrequencyErrorCurrent(
                            ViSession instrSession,
                            ViInt32   measurementType,
                            ViReal64* result);

ViStatus _VI_FUNC rsspecan_GetAmplifierStatisticsTableFrequencyErrorSelectedResult(
                            ViSession instrSession,
                            ViReal64* result);

ViStatus _VI_FUNC rsspecan_GetAmplifierStatisticsTableGainAll(
                            ViSession instrSession,
                            ViInt32   measurementType,
                            ViReal64* result);

ViStatus _VI_FUNC rsspecan_GetAmplifierStatisticsTableGainCurrent(
                            ViSession instrSession,
                            ViInt32   measurementType,
                            ViReal64* result);

ViStatus _VI_FUNC rsspecan_GetAmplifierStatisticsTableGainSelectedResult(
                            ViSession instrSession,
                            ViReal64* result);

ViStatus _VI_FUNC rsspecan_GetAmplifierStatisticsTableGainImbalanceAll(
                            ViSession instrSession,
                            ViInt32   measurementType,
                            ViReal64* result);

ViStatus _VI_FUNC rsspecan_GetAmplifierStatisticsTableGainImbalanceCurrent(
                            ViSession instrSession,
                            ViInt32   measurementType,
                            ViReal64* result);

ViStatus _VI_FUNC rsspecan_GetAmplifierStatisticsTableGainImbalanceSelectedResult(
                            ViSession instrSession,
                            ViReal64* result);

ViStatus _VI_FUNC rsspecan_GetAmplifierStatisticsTableCurrentIccAverageAll(
                            ViSession instrSession,
                            ViInt32   measurementType,
                            ViReal64* result);

ViStatus _VI_FUNC rsspecan_GetAmplifierStatisticsTableCurrentIccAverageCurrent(
                            ViSession instrSession,
                            ViInt32   measurementType,
                            ViReal64* result);

ViStatus _VI_FUNC rsspecan_GetAmplifierStatisticsTableCurrentIccAverageSelectedResult(
                            ViSession instrSession,
                            ViReal64* result);

ViStatus _VI_FUNC rsspecan_GetAmplifierStatisticsTableCurrentIccMaximumAll(
                            ViSession instrSession,
                            ViInt32   measurementType,
                            ViReal64* result);

ViStatus _VI_FUNC rsspecan_GetAmplifierStatisticsTableCurrentIccMaximumCurrent(
                            ViSession instrSession,
                            ViInt32   measurementType,
                            ViReal64* result);

ViStatus _VI_FUNC rsspecan_GetAmplifierStatisticsTableCurrentIccMaximumSelectedResult(
                            ViSession instrSession,
                            ViReal64* result);

ViStatus _VI_FUNC rsspecan_GetAmplifierStatisticsTableCurrentIccMinimumAll(
                            ViSession instrSession,
                            ViInt32   measurementType,
                            ViReal64* result);

ViStatus _VI_FUNC rsspecan_GetAmplifierStatisticsTableCurrentIccMinimumCurrent(
                            ViSession instrSession,
                            ViInt32   measurementType,
                            ViReal64* result);

ViStatus _VI_FUNC rsspecan_GetAmplifierStatisticsTableCurrentIccMinimumSelectedResult(
                            ViSession instrSession,
                            ViReal64* result);

ViStatus _VI_FUNC rsspecan_GetAmplifierStatisticsTableIQImbalanceAll(
                            ViSession instrSession,
                            ViInt32   measurementType,
                            ViReal64* result);

ViStatus _VI_FUNC rsspecan_GetAmplifierStatisticsTableIQImbalanceCurrent(
                            ViSession instrSession,
                            ViInt32   measurementType,
                            ViReal64* result);

ViStatus _VI_FUNC rsspecan_GetAmplifierStatisticsTableIQImbalanceSelectedResult(
                            ViSession instrSession,
                            ViReal64* result);

ViStatus _VI_FUNC rsspecan_GetAmplifierStatisticsTableIQOffsetAll(
                            ViSession instrSession,
                            ViInt32   measurementType,
                            ViReal64* result);

ViStatus _VI_FUNC rsspecan_GetAmplifierStatisticsTableIQOffsetCurrent(
                            ViSession instrSession,
                            ViInt32   measurementType,
                            ViReal64* result);

ViStatus _VI_FUNC rsspecan_GetAmplifierStatisticsTableIQOffsetSelectedResult(
                            ViSession instrSession,
                            ViReal64* result);

ViStatus _VI_FUNC rsspecan_GetAmplifierStatisticsTableBasebandIInputVoltageAverageAll(
                            ViSession instrSession,
                            ViInt32   measurementType,
                            ViReal64* result);

ViStatus _VI_FUNC rsspecan_GetAmplifierStatisticsTableBasebandIInputVoltageAverageCurrent(
                            ViSession instrSession,
                            ViInt32   measurementType,
                            ViReal64* result);

ViStatus _VI_FUNC rsspecan_GetAmplifierStatisticsTableBasebandIInputVoltageAverageSelectedResult(
                            ViSession instrSession,
                            ViReal64* result);

ViStatus _VI_FUNC rsspecan_GetAmplifierStatisticsTableBasebandIInputVoltageMaximumAll(
                            ViSession instrSession,
                            ViInt32   measurementType,
                            ViReal64* result);

ViStatus _VI_FUNC rsspecan_GetAmplifierStatisticsTableBasebandIInputVoltageMaximumCurrent(
                            ViSession instrSession,
                            ViInt32   measurementType,
                            ViReal64* result);

ViStatus _VI_FUNC rsspecan_GetAmplifierStatisticsTableBasebandIInputVoltageMaximumSelectedResult(
                            ViSession instrSession,
                            ViReal64* result);

ViStatus _VI_FUNC rsspecan_GetAmplifierStatisticsTableBasebandIInputVoltageMinimumAll(
                            ViSession instrSession,
                            ViInt32   measurementType,
                            ViReal64* result);

ViStatus _VI_FUNC rsspecan_GetAmplifierStatisticsTableBasebandIInputVoltageMinimumCurrent(
                            ViSession instrSession,
                            ViInt32   measurementType,
                            ViReal64* result);

ViStatus _VI_FUNC rsspecan_GetAmplifierStatisticsTableBasebandIInputVoltageMinimumSelectedResult(
                            ViSession instrSession,
                            ViReal64* result);

ViStatus _VI_FUNC rsspecan_GetAmplifierStatisticsTableMagnitudeErrorAll(
                            ViSession instrSession,
                            ViInt32   measurementType,
                            ViReal64* result);

ViStatus _VI_FUNC rsspecan_GetAmplifierStatisticsTableMagnitudeErrorCurrent(
                            ViSession instrSession,
                            ViInt32   measurementType,
                            ViReal64* result);

ViStatus _VI_FUNC rsspecan_GetAmplifierStatisticsTableMagnitudeErrorSelectedResult(
                            ViSession instrSession,
                            ViReal64* result);

ViStatus _VI_FUNC rsspecan_GetAmplifierStatisticsTableCompressionPointsdBInputAll(
                            ViSession instrSession,
                            ViInt32   compressionPoint,
                            ViInt32   measurementType,
                            ViReal64* result);

ViStatus _VI_FUNC rsspecan_GetAmplifierStatisticsTableCompressionPointsdBInputCurrent(
                            ViSession instrSession,
                            ViInt32   compressionPoint,
                            ViInt32   measurementType,
                            ViReal64* result);

ViStatus _VI_FUNC rsspecan_GetAmplifierStatisticsTableCompressionPointsdBInputSelectedResult(
                            ViSession instrSession,
                            ViInt32   compressionPoint,
                            ViReal64* result);

ViStatus _VI_FUNC rsspecan_GetAmplifierStatisticsTableCompressionPointsdBOutputAll(
                            ViSession instrSession,
                            ViInt32   compressionPoint,
                            ViInt32   measurementType,
                            ViReal64* result);

ViStatus _VI_FUNC rsspecan_GetAmplifierStatisticsTableCompressionPointsdBOutputCurrent(
                            ViSession instrSession,
                            ViInt32   compressionPoint,
                            ViInt32   measurementType,
                            ViReal64* result);

ViStatus _VI_FUNC rsspecan_GetAmplifierStatisticsTableCompressionPointsdBOutputSelectedResult(
                            ViSession instrSession,
                            ViInt32   compressionPoint,
                            ViReal64* result);

ViStatus _VI_FUNC rsspecan_GetAmplifierStatisticsTableAveragePowerConsumptionAll(
                            ViSession instrSession,
                            ViInt32   measurementType,
                            ViReal64* result);

ViStatus _VI_FUNC rsspecan_GetAmplifierStatisticsTableAveragePowerConsumptionCurrent(
                            ViSession instrSession,
                            ViInt32   measurementType,
                            ViReal64* result);

ViStatus _VI_FUNC rsspecan_GetAmplifierStatisticsTableAveragePowerConsumptionSelectedResult(
                            ViSession instrSession,
                            ViReal64* result);

ViStatus _VI_FUNC rsspecan_GetAmplifierStatisticsTablePCBasedAveragePAEAll(
                            ViSession instrSession,
                            ViInt32   measurementType,
                            ViReal64* result);

ViStatus _VI_FUNC rsspecan_GetAmplifierStatisticsTablePCBasedAveragePAECurrent(
                            ViSession instrSession,
                            ViInt32   measurementType,
                            ViReal64* result);

ViStatus _VI_FUNC rsspecan_GetAmplifierStatisticsTablePCBasedAveragePAESelectedResult(
                            ViSession instrSession,
                            ViReal64* result);

ViStatus _VI_FUNC rsspecan_GetAmplifierStatisticsTablePhaseErrorAll(
                            ViSession instrSession,
                            ViInt32   measurementType,
                            ViReal64* result);

ViStatus _VI_FUNC rsspecan_GetAmplifierStatisticsTablePhaseErrorCurrent(
                            ViSession instrSession,
                            ViInt32   measurementType,
                            ViReal64* result);

ViStatus _VI_FUNC rsspecan_GetAmplifierStatisticsTablePhaseErrorSelectedResult(
                            ViSession instrSession,
                            ViReal64* result);

ViStatus _VI_FUNC rsspecan_GetAmplifierStatisticsTablePowerInputAverageAll(
                            ViSession instrSession,
                            ViInt32   measurementType,
                            ViReal64* result);

ViStatus _VI_FUNC rsspecan_GetAmplifierStatisticsTablePowerInputAverageCurrent(
                            ViSession instrSession,
                            ViInt32   measurementType,
                            ViReal64* result);

ViStatus _VI_FUNC rsspecan_GetAmplifierStatisticsTablePowerInputAverageSelectedResult(
                            ViSession instrSession,
                            ViReal64* result);

ViStatus _VI_FUNC rsspecan_GetAmplifierStatisticsTablePowerInputMaximumAll(
                            ViSession instrSession,
                            ViInt32   measurementType,
                            ViReal64* result);

ViStatus _VI_FUNC rsspecan_GetAmplifierStatisticsTablePowerInputMaximumCurrent(
                            ViSession instrSession,
                            ViInt32   measurementType,
                            ViReal64* result);

ViStatus _VI_FUNC rsspecan_GetAmplifierStatisticsTablePowerInputMaximumSelectedResult(
                            ViSession instrSession,
                            ViReal64* result);

ViStatus _VI_FUNC rsspecan_GetAmplifierStatisticsTablePowerInputMinimumAll(
                            ViSession instrSession,
                            ViInt32   measurementType,
                            ViReal64* result);

ViStatus _VI_FUNC rsspecan_GetAmplifierStatisticsTablePowerInputMinimumCurrent(
                            ViSession instrSession,
                            ViInt32   measurementType,
                            ViReal64* result);

ViStatus _VI_FUNC rsspecan_GetAmplifierStatisticsTablePowerInputMinimumSelectedResult(
                            ViSession instrSession,
                            ViReal64* result);

ViStatus _VI_FUNC rsspecan_GetAmplifierStatisticsTablePowerOutputAverageAll(
                            ViSession instrSession,
                            ViInt32   measurementType,
                            ViReal64* result);

ViStatus _VI_FUNC rsspecan_GetAmplifierStatisticsTablePowerOutputAverageCurrent(
                            ViSession instrSession,
                            ViInt32   measurementType,
                            ViReal64* result);

ViStatus _VI_FUNC rsspecan_GetAmplifierStatisticsTablePowerOutputAverageSelectedResult(
                            ViSession instrSession,
                            ViReal64* result);

ViStatus _VI_FUNC rsspecan_GetAmplifierStatisticsTablePowerOutputMaximumAll(
                            ViSession instrSession,
                            ViInt32   measurementType,
                            ViReal64* result);

ViStatus _VI_FUNC rsspecan_GetAmplifierStatisticsTablePowerOuputMaximumCurrent(
                            ViSession instrSession,
                            ViInt32   measurementType,
                            ViReal64* result);

ViStatus _VI_FUNC rsspecan_GetAmplifierStatisticsTablePowerOutputMaximumSelectedResult(
                            ViSession instrSession,
                            ViReal64* result);

ViStatus _VI_FUNC rsspecan_GetAmplifierStatisticsTablePowerOutputMinimumAll(
                            ViSession instrSession,
                            ViInt32   measurementType,
                            ViReal64* result);

ViStatus _VI_FUNC rsspecan_GetAmplifierStatisticsTablePowerOutputMinimumCurrent(
                            ViSession instrSession,
                            ViInt32   measurementType,
                            ViReal64* result);

ViStatus _VI_FUNC rsspecan_GetAmplifierStatisticsTablePowerOutputMinimumSelectedResult(
                            ViSession instrSession,
                            ViReal64* result);

ViStatus _VI_FUNC rsspecan_GetAmplifierStatisticsTableQuadratureErrorAll(
                            ViSession instrSession,
                            ViInt32   measurementType,
                            ViReal64* result);

ViStatus _VI_FUNC rsspecan_GetAmplifierStatisticsTableQuadratureErrorCurrent(
                            ViSession instrSession,
                            ViInt32   measurementType,
                            ViReal64* result);

ViStatus _VI_FUNC rsspecan_GetAmplifierStatisticsTableQuadratureErrorSelectedResult(
                            ViSession instrSession,
                            ViReal64* result);

ViStatus _VI_FUNC rsspecan_GetAmplifierStatisticsTableBasebandQInputVoltageAverageAll(
                            ViSession instrSession,
                            ViInt32   measurementType,
                            ViReal64* result);

ViStatus _VI_FUNC rsspecan_GetAmplifierStatisticsTableBasebandQInputVoltageAverageCurrent(
                            ViSession instrSession,
                            ViInt32   measurementType,
                            ViReal64* result);

ViStatus _VI_FUNC rsspecan_GetAmplifierStatisticsTableBasebandQInputVoltageAverageSelectedResult(
                            ViSession instrSession,
                            ViReal64* result);

ViStatus _VI_FUNC rsspecan_GetAmplifierStatisticsTableBasebandQInputVoltageMaximumAll(
                            ViSession instrSession,
                            ViInt32   measurementType,
                            ViReal64* result);

ViStatus _VI_FUNC rsspecan_GetAmplifierStatisticsTableBasebandQInputVoltageMaximumCurrent(
                            ViSession instrSession,
                            ViInt32   measurementType,
                            ViReal64* result);

ViStatus _VI_FUNC rsspecan_GetAmplifierStatisticsTableBasebandQInputVoltageMaximumSelectedResult(
                            ViSession instrSession,
                            ViReal64* result);

ViStatus _VI_FUNC rsspecan_GetAmplifierStatisticsTableBasebandQInputVoltageMinimumAll(
                            ViSession instrSession,
                            ViInt32   measurementType,
                            ViReal64* result);

ViStatus _VI_FUNC rsspecan_GetAmplifierStatisticsTableBasebandQInputVoltageMinimumCurrent(
                            ViSession instrSession,
                            ViInt32   measurementType,
                            ViReal64* result);

ViStatus _VI_FUNC rsspecan_GetAmplifierStatisticsTableBasebandQInputVoltageMinimumSelectedResult(
                            ViSession instrSession,
                            ViReal64* result);

ViStatus _VI_FUNC rsspecan_GetAmplifierStatisticsTableRawEVMAverageAll(
                            ViSession instrSession,
                            ViInt32   measurementType,
                            ViReal64* result);

ViStatus _VI_FUNC rsspecan_GetAmplifierStatisticsTableRawEVMAverageCurrent(
                            ViSession instrSession,
                            ViInt32   measurementType,
                            ViReal64* result);

ViStatus _VI_FUNC rsspecan_GetAmplifierStatisticsTableRawEVMAverageSelectedResult(
                            ViSession instrSession,
                            ViReal64* result);

ViStatus _VI_FUNC rsspecan_GetAmplifierStatisticsTableRawEVMMaximumAll(
                            ViSession instrSession,
                            ViInt32   measurementType,
                            ViReal64* result);

ViStatus _VI_FUNC rsspecan_GetAmplifierStatisticsTableRawEVMMaximumCurrent(
                            ViSession instrSession,
                            ViInt32   measurementType,
                            ViReal64* result);

ViStatus _VI_FUNC rsspecan_GetAmplifierStatisticsTableRawEVMMaximumSelectedResult(
                            ViSession instrSession,
                            ViReal64* result);

ViStatus _VI_FUNC rsspecan_GetAmplifierStatisticsTableRawEVMMinimumAll(
                            ViSession instrSession,
                            ViInt32   measurementType,
                            ViReal64* result);

ViStatus _VI_FUNC rsspecan_GetAmplifierStatisticsTableRawEVMMinimumCurrent(
                            ViSession instrSession,
                            ViInt32   measurementType,
                            ViReal64* result);

ViStatus _VI_FUNC rsspecan_GetAmplifierStatisticsTableRawEVMMinimumSelectedResult(
                            ViSession instrSession,
                            ViReal64* result);

ViStatus _VI_FUNC rsspecan_GetAmplifierStatisticsTableRawModelEVMAverageAll(
                            ViSession instrSession,
                            ViInt32   measurementType,
                            ViReal64* result);

ViStatus _VI_FUNC rsspecan_GetAmplifierStatisticsTableRawModelEVMAverageCurrent(
                            ViSession instrSession,
                            ViInt32   measurementType,
                            ViReal64* result);

ViStatus _VI_FUNC rsspecan_GetAmplifierStatisticsTableRawModelEVMAverageSelectedResult(
                            ViSession instrSession,
                            ViReal64* result);

ViStatus _VI_FUNC rsspecan_GetAmplifierStatisticsTableRawModelEVMMaximumAll(
                            ViSession instrSession,
                            ViInt32   measurementType,
                            ViReal64* result);

ViStatus _VI_FUNC rsspecan_GetAmplifierStatisticsTableRawModelEVMMaximumCurrent(
                            ViSession instrSession,
                            ViInt32   measurementType,
                            ViReal64* result);

ViStatus _VI_FUNC rsspecan_GetAmplifierStatisticsTableRawModelEVMMaximumSelectedResult(
                            ViSession instrSession,
                            ViReal64* result);

ViStatus _VI_FUNC rsspecan_GetAmplifierStatisticsTableRawModelEVMMinimumAll(
                            ViSession instrSession,
                            ViInt32   measurementType,
                            ViReal64* result);

ViStatus _VI_FUNC rsspecan_GetAmplifierStatisticsTableRawModelEVMMinimumCurrent(
                            ViSession instrSession,
                            ViInt32   measurementType,
                            ViReal64* result);

ViStatus _VI_FUNC rsspecan_GetAmplifierStatisticsTableRawModelEVMMinimumSelectedResult(
                            ViSession instrSession,
                            ViReal64* result);

ViStatus _VI_FUNC rsspecan_GetAmplifierStatisticsTableSampleRateErrorAll(
                            ViSession instrSession,
                            ViInt32   measurementType,
                            ViReal64* result);

ViStatus _VI_FUNC rsspecan_GetAmplifierStatisticsTableSampleRateErrorCurrent(
                            ViSession instrSession,
                            ViInt32   measurementType,
                            ViReal64* result);

ViStatus _VI_FUNC rsspecan_GetAmplifierStatisticsTableSampleRateErrorSelectedResult(
                            ViSession instrSession,
                            ViReal64* result);

ViStatus _VI_FUNC rsspecan_GetAmplifierStatisticsTableVoltageVccAverageAll(
                            ViSession instrSession,
                            ViInt32   measurementType,
                            ViReal64* result);

ViStatus _VI_FUNC rsspecan_GetAmplifierStatisticsTableVoltageVccAverageCurrent(
                            ViSession instrSession,
                            ViInt32   measurementType,
                            ViReal64* result);

ViStatus _VI_FUNC rsspecan_GetAmplifierStatisticsTableVoltageVccAverageSelectedResult(
                            ViSession instrSession,
                            ViReal64* result);

ViStatus _VI_FUNC rsspecan_GetAmplifierStatisticsTableVoltageVccMaximumAll(
                            ViSession instrSession,
                            ViInt32   measurementType,
                            ViReal64* result);

ViStatus _VI_FUNC rsspecan_GetAmplifierStatisticsTableVoltageVccMaximumCurrent(
                            ViSession instrSession,
                            ViInt32   measurementType,
                            ViReal64* result);

ViStatus _VI_FUNC rsspecan_GetAmplifierStatisticsTableVoltageVccMaximumSelectedResult(
                            ViSession instrSession,
                            ViReal64* result);

ViStatus _VI_FUNC rsspecan_GetAmplifierStatisticsTableVoltageVccMinimumAll(
                            ViSession instrSession,
                            ViInt32   measurementType,
                            ViReal64* result);

ViStatus _VI_FUNC rsspecan_GetAmplifierStatisticsTableVoltageVccMinimumCurrent(
                            ViSession instrSession,
                            ViInt32   measurementType,
                            ViReal64* result);

ViStatus _VI_FUNC rsspecan_GetAmplifierStatisticsTableVoltageVccMinimumSelectedResult(
                            ViSession instrSession,
                            ViReal64* result);

ViStatus _VI_FUNC rsspecan_NoiseMode(ViSession instrSession);

ViStatus _VI_FUNC rsspecan_ConfigureNoiseFrequencySettings(ViSession instrSession,
                                                           ViReal64  startFrequency,
                                                           ViReal64  stopFrequency,
                                                           ViReal64  stepFrequency,
                                                           ViInt32   mode,
                                                           ViReal64  fixedLO,
                                                           ViReal64  fixedIF,
                                                           ViReal64  imageRejection,
                                                           ViReal64  fixedFrequency);

ViStatus _VI_FUNC rsspecan_ConfigureNoiseFrequencyTable(ViSession instrSession,
                                                        ViInt32   numberOfValues,
                                                        ViReal64  receiverFrequencyValues[],
                                                        ViReal64  localOscilatorFrequencyVal[],
                                                        ViReal64  intermediateFrequencyValues[]);

ViStatus _VI_FUNC rsspecan_ConfigureNoiseFrequencyBandwidthSweepTimeTable(
                            ViSession instrSession,
                            ViInt32   numberOfEntries,
                            ViReal64  frequencyValues[],
                            ViReal64  bandwidthValues[],
                            ViReal64  sweepTimeValues[]);

ViStatus _VI_FUNC rsspecan_ConfigureNoiseLocalOscillatorSettings(
                            ViSession instrSession,
                            ViInt32   mode,
                            ViReal64  fixedFrequency);

ViStatus _VI_FUNC rsspecan_ConfigureNoiseFrequencySettingsDigitalMode(
                            ViSession instrSession,
                            ViReal64  centerFrequency,
                            ViReal64  span,
                            ViInt32   points);

ViStatus _VI_FUNC rsspecan_ConfigureNoiseMeasurementMode(ViSession instrSession,
                                                         ViInt32   measurementMode);

ViStatus _VI_FUNC rsspecan_ConfigureNoiseCalibration(ViSession instrSession,
                                                     ViBoolean _2ndStageCorrection);

ViStatus _VI_FUNC rsspecan_ConfigureNoiseAnalyzerSettings(ViSession instrSession,
                                                          ViReal64  resolutionBandwidth,
                                                          ViReal64  sweepTime,
                                                          ViReal64  settlingTime,
                                                          ViInt32   average,
                                                          ViReal64  RFAttenuation,
                                                          ViReal64  range,
                                                          ViBoolean automaticRefLevel,
                                                          ViReal64  refLevel,
                                                          ViBoolean preselector,
                                                          ViBoolean preamplifier);

ViStatus _VI_FUNC rsspecan_ConfigureNoiseGeneratorSettings(ViSession instrSession,
                                                           ViBoolean automaticControl,
                                                           ViReal64  level,
                                                           ViString  generatorSelect,
                                                           ViInt32   GPIBAddress,
                                                           ViBoolean initBeforeMeasurement);

ViStatus _VI_FUNC rsspecan_ConfigureNoiseGeneratorAutoSwitchOff(
                            ViSession instrSession,
                            ViBoolean automaticSwitchOff);

ViStatus _VI_FUNC rsspecan_ConfigureNoiseGeneratorFrequency(ViSession instrSession,
                                                            ViReal64  offsetNr1,
                                                            ViReal64  offsetNr2,
                                                            ViInt32   numerator,
                                                            ViInt32   denominator);

ViStatus _VI_FUNC rsspecan_ConfigureNoiseSingleFrequency(ViSession instrSession,
                                                         ViReal64  singleFrequency,
                                                         ViBoolean coupledToSweepList);

ViStatus _VI_FUNC rsspecan_ConfigureNoiseMeasurementPoints(ViSession instrSession,
                                                           ViInt32   points);

ViStatus _VI_FUNC rsspecan_ConfigureNoiseFrequencyEntriesTable(
                            ViSession instrSession,
                            ViInt32   numberOfEntries,
                            ViReal64  frequencyTable[]);

ViStatus _VI_FUNC rsspecan_SaveNoiseCalibrationResults(ViSession instrSession,
                                                       ViString  calibrationResults);

ViStatus _VI_FUNC rsspecan_RecallNoiseCalibrationResults(ViSession instrSession,
                                                         ViString  calibrationResults);

ViStatus _VI_FUNC rsspecan_QueryNoiseAvailableCalibrationResults(
                            ViSession instrSession,
                            ViInt32   bufferSize,
                            ViChar    calibrationResults[]);

ViStatus _VI_FUNC rsspecan_ConfigureNoiseENRNoiseSource(ViSession instrSession,
                                                        ViInt32   noiseSource);

ViStatus _VI_FUNC rsspecan_ConfigureNoiseENRSettings(ViSession instrSession,
                                                     ViInt32   selectionMode,
                                                     ViReal64  ENRConstant,
                                                     ViReal64  roomTemperature);

ViStatus _VI_FUNC rsspecan_ConfigureNoiseENRResistorTemperatures(
                            ViSession instrSession,
                            ViReal64  hot,
                            ViReal64  cold);

ViStatus _VI_FUNC rsspecan_ConfigureNoiseENRCalibrationSNSSerialNumber(
                            ViSession instrSession,
                            ViString  serialNumber);

ViStatus _VI_FUNC rsspecan_ConfigureNoiseENRMeasurementSNSSerialNumber(
                            ViSession instrSession,
                            ViString  serialNumber);

ViStatus _VI_FUNC rsspecan_ConfigureNoiseENRTable(ViSession instrSession,
                                                  ViInt32   numberOfValues,
                                                  ViReal64  receiverFrequencyValues[],
                                                  ViReal64  ENRValues[]);

ViStatus _VI_FUNC rsspecan_QueryNoiseENRTableList(ViSession instrSession,
                                                  ViInt32   bufferSize,
                                                  ViChar    tableList[]);

ViStatus _VI_FUNC rsspecan_NoiseENRTableOperations(ViSession instrSession,
                                                   ViInt32   operation,
                                                   ViString  tableName);

ViStatus _VI_FUNC rsspecan_ConfigureNoiseENRCommonMode(ViSession instrSession,
                                                       ViBoolean commonMode);

ViStatus _VI_FUNC rsspecan_ConfigureNoiseENRCalibrationNoiseSource(
                            ViSession instrSession,
                            ViInt32   noiseSource);

ViStatus _VI_FUNC rsspecan_ConfigureNoiseENRCalibrationSettings(
                            ViSession instrSession,
                            ViInt32   selectionMode,
                            ViReal64  ENRConstant);

ViStatus _VI_FUNC rsspecan_ConfigureNoiseENRCalibrationResistorTemperatures(
                            ViSession instrSession,
                            ViReal64  hot,
                            ViReal64  cold);

ViStatus _VI_FUNC rsspecan_SelectNoiseENRCalibrationTable(ViSession instrSession,
                                                          ViString  tableName);

ViStatus _VI_FUNC rsspecan_ConfigureNoiseENRTemperatureTable(ViSession instrSession,
                                                             ViInt32   numberOfValues,
                                                             ViReal64  receiverFrequencyValues[],
                                                             ViReal64  temperatureHot[],
                                                             ViReal64  temperatureCold[]);

ViStatus _VI_FUNC rsspecan_DeleteNoiseENRTemperatureTable(ViSession instrSession,
                                                          ViString  tableName);

ViStatus _VI_FUNC rsspecan_QueryNoiseENRTemperatureTableList(ViSession instrSession,
                                                             ViInt32   bufferSize,
                                                             ViChar    temperatureTableList[]);

ViStatus _VI_FUNC rsspecan_ConfigureNoiseLossInputSettings(ViSession instrSession,
                                                           ViInt32   selectionMode,
                                                           ViReal64  lossInputConstant);

ViStatus _VI_FUNC rsspecan_ConfigureNoiseLossInputTable(ViSession instrSession,
                                                        ViInt32   numberOfValues,
                                                        ViReal64  receiverFrequencyValues[],
                                                        ViReal64  lossInputValues[]);

ViStatus _VI_FUNC rsspecan_NoiseLossInputTableOperations(ViSession instrSession,
                                                         ViInt32   operation,
                                                         ViString  tableName);

ViStatus _VI_FUNC rsspecan_ConfigureNoiseLossOutputSettings(ViSession instrSession,
                                                            ViInt32   selectionMode,
                                                            ViReal64  lossOutputConstant);

ViStatus _VI_FUNC rsspecan_ConfigureNoiseLossOutputTable(ViSession instrSession,
                                                         ViInt32   numberOfValues,
                                                         ViReal64  receiverFrequencyValues[],
                                                         ViReal64  lossOutputValues[]);

ViStatus _VI_FUNC rsspecan_NoiseLossOutputTableOperations(ViSession instrSession,
                                                          ViInt32   operation,
                                                          ViString  tableName);

ViStatus _VI_FUNC rsspecan_ConfigureNoiseResultSettings(ViSession instrSession,
                                                        ViInt32   combinedTraceDisplay,
                                                        ViBoolean tableFormResults);

ViStatus _VI_FUNC rsspecan_ConfigureNoiseTraceSettings(ViSession instrSession,
                                                       ViInt32   yAxis,
                                                       ViBoolean symbols,
                                                       ViBoolean automaticScaling,
                                                       ViReal64  minYAxis,
                                                       ViReal64  maxYAxis);

ViStatus _VI_FUNC rsspecan_ConfigureNoiseGainTraceSettings(ViSession instrSession,
                                                           ViBoolean yAxis,
                                                           ViBoolean symbols,
                                                           ViBoolean automaticScaling,
                                                           ViReal64  minYAxis,
                                                           ViReal64  maxYAxis);

ViStatus _VI_FUNC rsspecan_ConfigureNoiseTraceMemoryDisplay(ViSession instrSession,
                                                            ViInt32   memory,
                                                            ViBoolean state);

ViStatus _VI_FUNC rsspecan_ConfigureNoiseCurrentResultsDisplay(
                            ViSession instrSession,
                            ViBoolean state);

ViStatus _VI_FUNC rsspecan_ConfigureXAxisFrequencyDisplay(ViSession instrSession,
                                                          ViInt32   measurementResults,
                                                          ViInt32   frequency);

ViStatus _VI_FUNC rsspecan_ConfigureNoiseResultSummaryDisplay(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   item,
                            ViBoolean displayItem);

ViStatus _VI_FUNC rsspecan_ConfigureNoiseDisplayUncertainty(ViSession instrSession,
                                                            ViInt32   window,
                                                            ViBoolean uncertainty);

ViStatus _VI_FUNC rsspecan_CreateNoiseLimitLine(ViSession instrSession,
                                                ViInt32   limit,
                                                ViString  name,
                                                ViString  comment);

ViStatus _VI_FUNC rsspecan_DefineNoiseLimitLine(ViSession instrSession,
                                                ViInt32   limit,
                                                ViInt32   resultType,
                                                ViInt32   type,
                                                ViInt32   count,
                                                ViReal64  xAxis[],
                                                ViReal64  amplitude[]);

ViStatus _VI_FUNC rsspecan_ConfigureNoiseLimitLineType(ViSession instrSession,
                                                       ViInt32   limit,
                                                       ViInt32   resultType);

ViStatus _VI_FUNC rsspecan_AssignNoiseLimitLineToTrace(ViSession instrSession,
                                                       ViInt32   assignToTrace);

ViStatus _VI_FUNC rsspecan_EnableNoiseLimitCheck(ViSession instrSession,
                                                 ViInt32   limit,
                                                 ViInt32   type,
                                                 ViBoolean limitEnabled,
                                                 ViBoolean checkEnabled);

ViStatus _VI_FUNC rsspecan_MoveNoiseLimitLine(ViSession instrSession,
                                              ViInt32   limit,
                                              ViInt32   type,
                                              ViReal64  moveValue);

ViStatus _VI_FUNC rsspecan_CopyNoiseLimitLine(ViSession instrSession,
                                              ViInt32   limit,
                                              ViInt32   copyTo);

ViStatus _VI_FUNC rsspecan_ClearNoiseLimitLineResults(ViSession instrSession,
                                                      ViInt32   limit);

ViStatus _VI_FUNC rsspecan_DeleteNoiseLimitLine(ViSession instrSession,
                                                ViInt32   limit);

ViStatus _VI_FUNC rsspecan_SaveNoiseTraceDataToMemory(ViSession instrSession,
                                                      ViInt32   memory);

ViStatus _VI_FUNC rsspecan_AssignMarkerToTrace(ViSession instrSession,
                                               ViInt32   reserved,
                                               ViInt32   marker,
                                               ViInt32   trace);

ViStatus _VI_FUNC rsspecan_QueryNoiseMarkerAmplitude(ViSession instrSession,
                                                     ViInt32   reserved,
                                                     ViInt32   marker,
                                                     ViInt32   resultType,
                                                     ViReal64* markerAmplitude);

ViStatus _VI_FUNC rsspecan_QueryNoiseDeltaMarkerAmplitude(ViSession instrSession,
                                                          ViInt32   reserved,
                                                          ViInt32   deltaMarker,
                                                          ViInt32   resultType,
                                                          ViReal64* amplitude);

ViStatus _VI_FUNC rsspecan_ConfigureNoiseUncertaintyCommonSource(
                            ViSession instrSession,
                            ViBoolean state);

ViStatus _VI_FUNC rsspecan_ConfigureNoiseUncertaintySourceCharacteristics(
                            ViSession instrSession,
                            ViInt32   type,
                            ViReal64  sourceOutput,
                            ViReal64  ENRUncertainty);

ViStatus _VI_FUNC rsspecan_ConfigureNoiseUncertaintySourceCalibrationCharacteristics(
                            ViSession instrSession,
                            ViInt32   type,
                            ViReal64  sourceOutput,
                            ViReal64  ENRUncertainty);

ViStatus _VI_FUNC rsspecan_ConfigureNoiseTemperatureUncertainty(
                            ViSession instrSession,
                            ViReal64  coldTemperature,
                            ViReal64  hotTemperature);

ViStatus _VI_FUNC rsspecan_ConfigureNoiseTemperatureUncertaintyCalibration(
                            ViSession instrSession,
                            ViReal64  coldTemperature,
                            ViReal64  hotTemperature);

ViStatus _VI_FUNC rsspecan_ConfigureNoiseUncertaintyDUTCharacteristics(
                            ViSession instrSession,
                            ViInt32   direction,
                            ViInt32   type,
                            ViReal64  DUTUncertainty);

ViStatus _VI_FUNC rsspecan_ConfigureNoiseUncertaintyMeasurementValues(
                            ViSession instrSession,
                            ViBoolean autoEnabled,
                            ViReal64  noiseFigure,
                            ViReal64  gain,
                            ViReal64  frequency);

ViStatus _VI_FUNC rsspecan_ConfigureNoiseUncertaintyAnalyzerCharacteristics(
                            ViSession instrSession,
                            ViBoolean preamplifierState,
                            ViReal64  noiseFigure,
                            ViReal64  gain,
                            ViInt32   type,
                            ViReal64  analyzerUncertainty);

ViStatus _VI_FUNC rsspecan_QueryNoiseUncertainty(ViSession instrSession,
                                                 ViReal64* uncertainty);

ViStatus _VI_FUNC rsspecan_QuerySpectrumAnalyzerUncertainty(ViSession instrSession,
                                                            ViInt32   spectrumAnalyzer,
                                                            ViReal64* uncertainty);

ViStatus _VI_FUNC rsspecan_NoiseAddWindow(ViSession instrSession,
                                          ViInt32   stringSize,
                                          ViString  referenceWindowName,
                                          ViInt32   direction,
                                          ViInt32   windowType,
                                          ViChar    returnedWindowName[]);

ViStatus _VI_FUNC rsspecan_NoiseReplaceWindow(ViSession instrSession,
                                              ViString  windowName,
                                              ViInt32   windowType);

ViStatus _VI_FUNC rsspecan_ConfigureNoisePowerMeasurement(ViSession instrSession,
                                                          ViInt32   measurementMode,
                                                          ViInt32   type);

ViStatus _VI_FUNC rsspecan_InitNoiseGenerator(ViSession instrSession);

ViStatus _VI_FUNC rsspecan_FetchNoiseArray(ViSession instrSession,
                                           ViInt32   modifier,
                                           ViInt32   arraySize,
                                           ViReal64  values[]);

ViStatus _VI_FUNC rsspecan_FetchNoiseScalar(ViSession instrSession,
                                            ViInt32   modifier,
                                            ViReal64* value);

ViStatus _VI_FUNC rsspecan_FetchNoiseMemoryArray(ViSession instrSession,
                                                 ViInt32   modifier,
                                                 ViInt32   memory,
                                                 ViInt32   arraySize,
                                                 ViReal64  measResult[]);

ViStatus _VI_FUNC rsspecan_QueryNoiseResults(ViSession instrSession,
                                             ViInt32   reserved,
                                             ViInt32   trace,
                                             ViInt32   resultType,
                                             ViInt32   arrayLength,
                                             ViInt32*  actualPoints,
                                             ViReal64  results[]);

ViStatus _VI_FUNC rsspecan_GetNoiseLimitCheckResult(ViSession instrSession,
                                                    ViInt32   limit,
                                                    ViInt32*  state);

ViStatus _VI_FUNC rsspecan_QueryNoiseMemoryResults(ViSession instrSession,
                                                   ViInt32   memory,
                                                   ViInt32   resultType,
                                                   ViInt32   arrayLength,
                                                   ViInt32*  actualPoints,
                                                   ViReal64  results[]);

ViStatus _VI_FUNC rsspecan_QueryNoiseArrayScalarResults(ViSession instrSession,
                                                        ViInt32   resultType,
                                                        ViInt32   type,
                                                        ViInt32   arrayLength,
                                                        ViInt32*  actualPoints,
                                                        ViReal64  results[]);

ViStatus _VI_FUNC rsspecan_PhaseMode(ViSession instrSession);

ViStatus _VI_FUNC rsspecan_ConfigurePhaseRFInput(ViSession instrSession,
                                                 ViInt32   source,
                                                 ViInt32   inputCoupling);

ViStatus _VI_FUNC rsspecan_ConfigurePhaseBasebandInput(ViSession instrSession,
                                                       ViInt32   inputConnector);

ViStatus _VI_FUNC rsspecan_ConfigurePhaseLocalOscillator(ViSession instrSession,
                                                         ViInt32   source,
                                                         ViInt32   externalLevel);

ViStatus _VI_FUNC rsspecan_ConfigurePhaseSignalSettings(ViSession instrSession,
                                                        ViReal64  frequency,
                                                        ViReal64  level,
                                                        ViBoolean sweepForward,
                                                        ViBoolean verifyFrequencyLevel,
                                                        ViReal64  frequencyToleranceRelative,
                                                        ViReal64  levelTolerance);

ViStatus _VI_FUNC rsspecan_ConfigurePhaseFrequencyTolerance(ViSession instrSession,
                                                            ViReal64  absoluteFrequencyTolerance);

ViStatus _VI_FUNC rsspecan_ConfigurePhaseAFCSettings(ViSession instrSession,
                                                     ViBoolean trackFrequency,
                                                     ViBoolean trackLevel);

ViStatus _VI_FUNC rsspecan_ConfigurePhaseAMRejection(ViSession instrSession,
                                                     ViBoolean AMRejection);

ViStatus _VI_FUNC rsspecan_ConfigurePhaseTrackingBW(ViSession instrSession,
                                                    ViReal64  trackingBandwidth);

ViStatus _VI_FUNC rsspecan_ConfigurePhaseDigitalPLL(ViSession instrSession,
                                                    ViBoolean digitalPLL);

ViStatus _VI_FUNC rsspecan_ConfigurePhaseDecimation(ViSession instrSession,
                                                    ViBoolean decimation);

ViStatus _VI_FUNC rsspecan_ConfigurePhaseOnlineIQ(ViSession instrSession,
                                                  ViBoolean onlineIQ);

ViStatus _VI_FUNC rsspecan_ConfigurePhaseDisplaySettings(ViSession instrSession,
                                                         ViReal64  xAxisStart,
                                                         ViReal64  xAxisStop,
                                                         ViBoolean autoscaleOnce,
                                                         ViReal64  yAxisTop,
                                                         ViReal64  yAxisRange);

ViStatus _VI_FUNC rsspecan_ConfigurePhaseDisplayYAxisSettings(
                            ViSession instrSession,
                            ViInt32   yAxisScaling,
                            ViReal64  yAxisTop,
                            ViReal64  yAxisRange,
                            ViReal64  yAxisBottom);

ViStatus _VI_FUNC rsspecan_ConfigurePhaseDisplayYAxisUnit(ViSession instrSession,
                                                          ViInt32   unit);

ViStatus _VI_FUNC rsspecan_ConfigurePhaseDisplayXAxisSettings(
                            ViSession instrSession,
                            ViInt32   xAxisScope,
                            ViReal64  xAxisStart,
                            ViReal64  xAxisStop,
                            ViReal64  halfDecade);

ViStatus _VI_FUNC rsspecan_ConfigurePhaseTraceSettings(ViSession instrSession,
                                                       ViReal64  traceOffset,
                                                       ViInt32   traceSmoothing,
                                                       ViInt32   smoothingType);

ViStatus _VI_FUNC rsspecan_ConfigurePhaseResidualCalculations(
                            ViSession instrSession,
                            ViBoolean useMeasurementSettings,
                            ViReal64  evaluateFrom,
                            ViReal64  evaluateTo);

ViStatus _VI_FUNC rsspecan_ConfigurePhaseTraceCustomRange(ViSession instrSession,
                                                          ViInt32   reserved,
                                                          ViInt32   userRange,
                                                          ViInt32   trace);

ViStatus _VI_FUNC rsspecan_ConfigurePhaseUserResidualCalculations(
                            ViSession instrSession,
                            ViInt32   userRange,
                            ViReal64  evaluateFrom,
                            ViReal64  evaluateTo);

ViStatus _VI_FUNC rsspecan_ConfigurePhaseAnalyzerSettings(ViSession instrSession,
                                                          ViBoolean preselector,
                                                          ViBoolean preamplifier_B2B25B23);

ViStatus _VI_FUNC rsspecan_ConfigurePhaseSpotNoiseCalculation(
                            ViSession instrSession,
                            ViBoolean spotNoiseCalculation);

ViStatus _VI_FUNC rsspecan_ConfigurePhaseSpotNoiseSettings(ViSession instrSession,
                                                           ViBoolean enable,
                                                           ViReal64  offsetFrequency1,
                                                           ViReal64  offsetFrequency2,
                                                           ViReal64  offsetFrequency3,
                                                           ViReal64  offsetFrequency4);

ViStatus _VI_FUNC rsspecan_ConfigurePhaseSpotNoiseTraceSelection(
                            ViSession instrSession,
                            ViInt32   trace);

ViStatus _VI_FUNC rsspecan_ConfigurePhaseSpotNoise(ViSession instrSession,
                                                   ViInt32   marker,
                                                   ViBoolean enable,
                                                   ViReal64  offsetFrequency);

ViStatus _VI_FUNC rsspecan_ConfigurePhaseSpotNoiseCustom(ViSession instrSession,
                                                         ViBoolean state);

ViStatus _VI_FUNC rsspecan_ConfigurePhaseSpotNoiseDisplayInfo(
                            ViSession instrSession,
                            ViBoolean state);

ViStatus _VI_FUNC rsspecan_SetPhaseAllSpotNoiseMarkersOff(ViSession instrSession);

ViStatus _VI_FUNC rsspecan_ConfigurePhaseSpurSettings(ViSession instrSession,
                                                      ViBoolean spurSuppression,
                                                      ViReal64  spurThreshold);

ViStatus _VI_FUNC rsspecan_ConfigurePhaseLevelShifting(ViSession instrSession,
                                                       ViReal64  levelOffset);

ViStatus _VI_FUNC rsspecan_ConfigurePhaseSmoothingSettings(ViSession instrSession,
                                                           ViBoolean smoothingState,
                                                           ViReal64  smoothingValue);

ViStatus _VI_FUNC rsspecan_ConfigurePhaseSignalLevel(ViSession instrSession,
                                                     ViReal64* signalLevel);

ViStatus _VI_FUNC rsspecan_ConfigurePhasePulseSettings(ViSession instrSession,
                                                       ViInt32   mode,
                                                       ViReal64  repetitionInterval);

ViStatus _VI_FUNC rsspecan_ConfigurePhasePulseDetectionOnce(ViSession instrSession);

ViStatus _VI_FUNC rsspecan_ConfigurePhasePulseExternalGateType(
                            ViSession instrSession,
                            ViInt32   gateType);

ViStatus _VI_FUNC rsspecan_ConfigurePhasePulseGateSource(ViSession instrSession,
                                                         ViInt32   gateSource);

ViStatus _VI_FUNC rsspecan_ConfigurePhasePulseExternalGateLevel(
                            ViSession instrSession,
                            ViReal64  gateLevel);

ViStatus _VI_FUNC rsspecan_ConfigurePhaseCrossCorrelation(ViSession instrSession,
                                                          ViBoolean optimization,
                                                          ViInt32   factor);

ViStatus _VI_FUNC rsspecan_ConfigurePhaseCrossCorrelationOptimizeThreshold(
                            ViSession instrSession,
                            ViReal64  threshold);

ViStatus _VI_FUNC rsspecan_ConfigurePhaseCrossCorrelationFinishSegment(ViSession instrSession);

ViStatus _VI_FUNC rsspecan_ConfigurePhaseSignalSearching(ViSession instrSession,
                                                         ViBoolean signalSearchAuto,
                                                         ViReal64  lowerLimit,
                                                         ViReal64  upperLimit,
                                                         ViReal64  threshold);

ViStatus _VI_FUNC rsspecan_ConfigurePhaseSignalSearchingCount(
                            ViSession instrSession,
                            ViBoolean signalCount);

ViStatus _VI_FUNC rsspecan_ConfigurePhaseFrequencyStability(ViSession instrSession,
                                                            ViReal64  tauStart,
                                                            ViReal64  tauStop);

ViStatus _VI_FUNC rsspecan_ConfigurePhaseDisplayYAxis(ViSession instrSession,
                                                      ViInt32   window,
                                                      ViInt32   yAxisScaling,
                                                      ViReal64  yAxisTop,
                                                      ViReal64  yAxisRange,
                                                      ViReal64  yAxisBottom);

ViStatus _VI_FUNC rsspecan_ConfigurePhaseDisplayYAxisOnce(ViSession instrSession,
                                                          ViInt32   window);

ViStatus _VI_FUNC rsspecan_ConfigurePhaseTrace(ViSession instrSession,
                                               ViInt32   window,
                                               ViInt32   trace,
                                               ViInt32   resultType,
                                               ViBoolean smoothingState,
                                               ViInt32   smoothing,
                                               ViBoolean spursState,
                                               ViReal64  spursThreshold,
                                               ViBoolean offsetState,
                                               ViReal64  offset);

ViStatus _VI_FUNC rsspecan_ConfigurePhaseTraceMath(ViSession instrSession,
                                                   ViInt32   window,
                                                   ViInt32   firstTrace,
                                                   ViInt32   secondTrace);

ViStatus _VI_FUNC rsspecan_ConfigurePhaseTraceLabel(ViSession instrSession,
                                                    ViInt32   window,
                                                    ViInt32   trace,
                                                    ViBoolean state,
                                                    ViString  label);

ViStatus _VI_FUNC rsspecan_AddPhaseWindow(ViSession instrSession,
                                          ViInt32   stringSize,
                                          ViString  referenceWindowName,
                                          ViInt32   direction,
                                          ViInt32   windowType,
                                          ViChar    returnedWindowName[]);

ViStatus _VI_FUNC rsspecan_ReplacePhaseWindow(ViSession instrSession,
                                              ViString  windowName,
                                              ViInt32   windowType);

ViStatus _VI_FUNC rsspecan_ConfigurePhaseSweepSettings(ViSession instrSession,
                                                       ViInt32   sweepMode);

ViStatus _VI_FUNC rsspecan_ConfigurePhaseSignalVerificationFailed(
                            ViSession instrSession,
                            ViInt32   stopOnVerifyFailed);

ViStatus _VI_FUNC rsspecan_ConfigurePhaseSignalCaptureRange(ViSession instrSession,
                                                            ViInt32   captureRange);

ViStatus _VI_FUNC rsspecan_ConfigurePhaseSpanSettings(ViSession instrSession,
                                                      ViReal64  startOffset,
                                                      ViReal64  stopOffset);

ViStatus _VI_FUNC rsspecan_ConfigurePhaseCarrierFrequencyOffsetTable(
                            ViSession instrSession,
                            ViInt32   subband,
                            ViReal64  RBW,
                            ViInt32   average,
                            ViInt32   FFT);

ViStatus _VI_FUNC rsspecan_ConfigurePhasePresetSettings(ViSession instrSession,
                                                        ViReal64  RBW,
                                                        ViInt32   average,
                                                        ViInt32   FFT);

ViStatus _VI_FUNC rsspecan_ConfigurePhaseResolutionBandwidth(ViSession instrSession,
                                                             ViReal64  RBW);

ViStatus _VI_FUNC rsspecan_ConfigurePhaseUltraSmallRBWs(ViSession instrSession,
                                                        ViBoolean ultraSmallRBWs);

ViStatus _VI_FUNC rsspecan_ConfigurePhaseGlobalIQWindowSettings(
                            ViSession instrSession,
                            ViInt32   decade,
                            ViInt32   subband,
                            ViInt32   globalIQWindow);

ViStatus _VI_FUNC rsspecan_ConfigurePhaseResidualCalculationTrace(
                            ViSession instrSession,
                            ViInt32   range,
                            ViInt32   trace);

ViStatus _VI_FUNC rsspecan_ConfigurePhaseIntegrationRange(ViSession instrSession,
                                                          ViInt32   range,
                                                          ViBoolean overCompleteRange,
                                                          ViReal64  start,
                                                          ViReal64  stop,
                                                          ViString  weightingFilter);

ViStatus _VI_FUNC rsspecan_PhaseWeightingFilterCreate(ViSession instrSession,
                                                      ViString  name,
                                                      ViString  comment,
                                                      ViInt32   arraySize,
                                                      ViReal64  frequency[],
                                                      ViReal64  level[]);

ViStatus _VI_FUNC rsspecan_PhaseWeightingFilterDelete(ViSession instrSession,
                                                      ViString  filter);

ViStatus _VI_FUNC rsspecan_ConfigurePhaseSpuriousList(ViSession instrSession,
                                                      ViInt32   sortOrder,
                                                      ViString  weightingFilter);

ViStatus _VI_FUNC rsspecan_ConfigurePhaseSpuriousColor(ViSession instrSession,
                                                       ViInt32   color);

ViStatus _VI_FUNC rsspecan_ConfigurePhaseSpuriousFilter(ViSession instrSession,
                                                        ViInt32   mode,
                                                        ViBoolean includeHarmonics,
                                                        ViReal64  range);

ViStatus _VI_FUNC rsspecan_ConfigurePhaseSpuriousFilterOffset(
                            ViSession instrSession,
                            ViInt32   arraySize,
                            ViReal64  offsetValues[]);

ViStatus _VI_FUNC rsspecan_QueryPhaseSpuriousFilterName(ViSession instrSession,
                                                        ViInt32   bufferSize,
                                                        ViChar    name[]);

ViStatus _VI_FUNC rsspecan_ClearPhaseSpuriousFilter(ViSession instrSession);

ViStatus _VI_FUNC rsspecan_SavePhaseSpuriousFilter(ViSession instrSession,
                                                   ViString  filename);

ViStatus _VI_FUNC rsspecan_LoadPhaseSpuriousFilter(ViSession instrSession,
                                                   ViString  filename);

ViStatus _VI_FUNC rsspecan_DCOutputPowerState(ViSession instrSession,
                                              ViBoolean state);

ViStatus _VI_FUNC rsspecan_DCOutputPowerCoupling(ViSession instrSession,
                                                 ViBoolean coupling);

ViStatus _VI_FUNC rsspecan_ConfigureDCSupplyPortState(ViSession instrSession,
                                                      ViBoolean state);

ViStatus _VI_FUNC rsspecan_ConfigureDCSupplyPort(ViSession instrSession,
                                                 ViInt32   mode,
                                                 ViReal64  maxVA,
                                                 ViReal64  level,
                                                 ViReal64  highLimit,
                                                 ViReal64  lowLimit);

ViStatus _VI_FUNC rsspecan_ConfigureDCTuningPortState(ViSession instrSession,
                                                      ViBoolean state);

ViStatus _VI_FUNC rsspecan_ConfigureDCTuningPort(ViSession instrSession,
                                                 ViReal64  level,
                                                 ViReal64  highLimit,
                                                 ViReal64  lowLimit);

ViStatus _VI_FUNC rsspecan_ConfigureDCAUXPortState(ViSession instrSession,
                                                   ViBoolean state);

ViStatus _VI_FUNC rsspecan_ConfigureDCAUXPort(ViSession instrSession,
                                              ViReal64  level,
                                              ViReal64  highLimit,
                                              ViReal64  lowLimit);

ViStatus _VI_FUNC rsspecan_QueryDCPortMaximum(ViSession instrSession,
                                              ViInt32   port,
                                              ViReal64* value);

ViStatus _VI_FUNC rsspecan_QueryDCOutputVoltageResults(ViSession instrSession,
                                                       ViInt32   arraySize,
                                                       ViReal64  results[],
                                                       ViInt32*  returnedValues);

ViStatus _VI_FUNC rsspecan_QueryDCOutputCurrentResults(ViSession instrSession,
                                                       ViInt32   arraySize,
                                                       ViReal64  results[],
                                                       ViInt32*  returnedValues);

ViStatus _VI_FUNC rsspecan_QueryDCOutputPowerResults(ViSession instrSession,
                                                     ViInt32   arraySize,
                                                     ViReal64  results[],
                                                     ViInt32*  returnedValues);

ViStatus _VI_FUNC rsspecan_SignalSourceState(ViSession instrSession,
                                             ViBoolean state);

ViStatus _VI_FUNC rsspecan_SignalSourceSignalCharacteristics(ViSession instrSession,
                                                             ViReal64  frequency,
                                                             ViReal64  level);

ViStatus _VI_FUNC rsspecan_SignalSourceSignalFrequencyStepsize(
                            ViSession instrSession,
                            ViReal64  frequencyStepsize);

ViStatus _VI_FUNC rsspecan_SignalSourceCoupling(ViSession instrSession,
                                                ViBoolean coupling);

ViStatus _VI_FUNC rsspecan_SignalSourceDUTBypass(ViSession instrSession,
                                                 ViBoolean DUTBypass);

ViStatus _VI_FUNC rsspecan_SignalSourcePulseModulation(ViSession instrSession,
                                                       ViBoolean pulseModulation);

ViStatus _VI_FUNC rsspecan_SignalSourcePulseSettings(ViSession instrSession,
                                                     ViReal64  period,
                                                     ViReal64  width,
                                                     ViInt32   triggerOutput);

ViStatus _VI_FUNC rsspecan_VCOState(ViSession instrSession,
                                    ViBoolean state);

ViStatus _VI_FUNC rsspecan_VCODCSource(ViSession instrSession,
                                       ViInt32   sweepSource,
                                       ViInt32   fixSource);

ViStatus _VI_FUNC rsspecan_VCOSweep(ViSession instrSession,
                                    ViReal64  start,
                                    ViReal64  stop,
                                    ViInt32   measPoints,
                                    ViReal64  initialSettlingTime,
                                    ViReal64  settlingTime,
                                    ViReal64  frequencyResolution);

ViStatus _VI_FUNC rsspecan_SpotNoiseVsTuneState(ViSession instrSession,
                                                ViBoolean state);

ViStatus _VI_FUNC rsspecan_SpotNoiseVsTuneSource(ViSession instrSession,
                                                 ViInt32   sweepSource);

ViStatus _VI_FUNC rsspecan_SpotNoiseVsTuneSweep(ViSession instrSession,
                                                ViReal64  start,
                                                ViReal64  stop,
                                                ViInt32   measPoints,
                                                ViReal64  initialSettlingTime,
                                                ViReal64  settlingTime);

ViStatus _VI_FUNC rsspecan_TransientAnalysisState(ViSession instrSession,
                                                  ViBoolean state);

ViStatus _VI_FUNC rsspecan_TransientAnalysisMeasurementMode(ViSession instrSession,
                                                            ViInt32   mode);

ViStatus _VI_FUNC rsspecan_TransientAnalysisReferenceLevel(ViSession instrSession,
                                                           ViReal64  referenceLevel);

ViStatus _VI_FUNC rsspecan_TransientAnalysisYAxisScaleAFCoupling(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   AFCoupling,
                            ViInt32   zeroPhasePositioningMode,
                            ViReal64  zeroPhasePosition);

ViStatus _VI_FUNC rsspecan_TransientAnalysisYAxisUnit(ViSession instrSession,
                                                      ViInt32   window,
                                                      ViInt32   unit);

ViStatus _VI_FUNC rsspecan_TransientAnalysisYAxisGridMode(ViSession instrSession,
                                                          ViInt32   window,
                                                          ViInt32   yAxisGridMode);

ViStatus _VI_FUNC rsspecan_TransientAnalysisTracePersistence(ViSession instrSession,
                                                             ViInt32   window,
                                                             ViInt32   trace,
                                                             ViBoolean state,
                                                             ViReal64  decay);

ViStatus _VI_FUNC rsspecan_ConfigureTransientAnalysisSettlingTime(
                            ViSession instrSession,
                            ViBoolean state,
                            ViReal64  upperFrequencyLimit,
                            ViReal64  lowerFrequencyLimit,
                            ViInt32   diagram,
                            ViInt32   trace);

ViStatus _VI_FUNC rsspecan_QueryTransientAnalysisSettlingTimeResult(
                            ViSession instrSession,
                            ViReal64* result);

ViStatus _VI_FUNC rsspecan_ConfigureTransientAnalysisFrequencyDeviation(
                            ViSession instrSession,
                            ViBoolean state,
                            ViReal64  start,
                            ViReal64  stop,
                            ViInt32   diagram,
                            ViInt32   trace);

ViStatus _VI_FUNC rsspecan_QueryTransientAnalysisFrequencyDeviationRegressionLineData(
                            ViSession instrSession,
                            ViInt32   arraySize,
                            ViReal64  xValues[],
                            ViReal64  yValues[],
                            ViInt32*  returnedValues);

ViStatus _VI_FUNC rsspecan_QueryTransientAnalysisFrequencyDeviationRegressionLinePoints(
                            ViSession instrSession,
                            ViInt32*  points);

ViStatus _VI_FUNC rsspecan_QueryTransientAnalysisFrequencyDeviationRegressionLineSlope(
                            ViSession instrSession,
                            ViReal64* slope);

ViStatus _VI_FUNC rsspecan_CreatePhaseLimitLine(ViSession instrSession,
                                                ViInt32   reserved,
                                                ViInt32   limit,
                                                ViString  name,
                                                ViString  comment,
                                                ViInt32   assignToTrace);

ViStatus _VI_FUNC rsspecan_DefinePhaseLimitLine(ViSession instrSession,
                                                ViInt32   limit,
                                                ViInt32   type,
                                                ViInt32   count,
                                                ViReal64  xAxis[],
                                                ViReal64  amplitude[]);

ViStatus _VI_FUNC rsspecan_EnablePhaseLimitCheck(ViSession instrSession,
                                                 ViInt32   reserved,
                                                 ViInt32   limit,
                                                 ViInt32   type,
                                                 ViBoolean limitEnabled,
                                                 ViBoolean checkEnabled);

ViStatus _VI_FUNC rsspecan_MovePhaseLimitLine(ViSession instrSession,
                                              ViInt32   limit,
                                              ViInt32   type,
                                              ViReal64  moveValue);

ViStatus _VI_FUNC rsspecan_CopyPhaseLimitLine(ViSession instrSession,
                                              ViInt32   limit,
                                              ViInt32   copyTo);

ViStatus _VI_FUNC rsspecan_ClearPhaseLimitLineResults(ViSession instrSession,
                                                      ViInt32   limit);

ViStatus _VI_FUNC rsspecan_DeletePhaseLimitLine(ViSession instrSession,
                                                ViInt32   limit);

ViStatus _VI_FUNC rsspecan_GetActivePhaseLimitLine(ViSession instrSession,
                                                   ViInt32   bufferSize,
                                                   ViChar    activeLimitLines[]);

ViStatus _VI_FUNC rsspecan_ConfigurePhaseNoiseLimitSettings(ViSession instrSession,
                                                            ViInt32   shapeType,
                                                            ViInt32   selectedTrace,
                                                            ViReal64  noiseFloor,
                                                            ViInt32   segment,
                                                            ViReal64  cornerFrequency,
                                                            ViReal64  slope);

ViStatus _VI_FUNC rsspecan_PhaseCopyToUserLimitLine(ViSession instrSession,
                                                    ViInt32   limitLine);

ViStatus _VI_FUNC rsspecan_GetPhaseNoiseLimitCheckState(ViSession instrSession,
                                                        ViInt32*  limitCheck);

ViStatus _VI_FUNC rsspecan_GetPhaseNoiseLimitLineCompatible(ViSession instrSession,
                                                            ViInt32*  limitCompatible);

ViStatus _VI_FUNC rsspecan_TracePhaseSettings(ViSession instrSession,
                                              ViInt32   traceNumber,
                                              ViBoolean traceState,
                                              ViInt32   traceMode,
                                              ViBoolean smoothing);

ViStatus _VI_FUNC rsspecan_TracePhaseSpur(ViSession instrSession,
                                          ViInt32   traceNumber,
                                          ViBoolean spurRemoval);

ViStatus _VI_FUNC rsspecan_TracePhaseCrossCorrelation(ViSession instrSession,
                                                      ViBoolean state);

ViStatus _VI_FUNC rsspecan_PhaseFinishHalfDecade(ViSession instrSession);

ViStatus _VI_FUNC rsspecan_ReferencePhaseMeasurement(ViSession instrSession,
                                                     ViUInt32  timeout);

ViStatus _VI_FUNC rsspecan_GetPhaseEstimatedMeasurementTime(ViSession instrSession,
                                                            ViReal64* estimatedMeasurementTime);

ViStatus _VI_FUNC rsspecan_GetHalfDecadeMeasCharacteristics(ViSession instrSession,
                                                            ViInt32   range,
                                                            ViReal64* RBW,
                                                            ViReal64* startFrequencyOffset,
                                                            ViReal64* stopFrequencyOffset,
                                                            ViInt32*  XCorr);

ViStatus _VI_FUNC rsspecan_GetAllanVarianceAndDeviation(ViSession instrSession,
                                                        ViReal64* bandwidth);

ViStatus _VI_FUNC rsspecan_ReadPhaseTraceData(ViSession instrSession,
                                              ViInt32   window,
                                              ViInt32   sourceTrace,
                                              ViBoolean performSweep,
                                              ViUInt32  timeout,
                                              ViInt32*  noOfPoints,
                                              ViReal64  traceDataX[],
                                              ViReal64  traceDataY[]);

ViStatus _VI_FUNC rsspecan_ReadPhaseCrossCorrelationGainData(ViSession instrSession,
                                                             ViInt32   window,
                                                             ViInt32*  noOfPoints,
                                                             ViReal64  traceDataX[],
                                                             ViReal64  traceDataY[]);

ViStatus _VI_FUNC rsspecan_ReadPhaseTraceDataLinearInterpolation(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   sourceTrace,
                            ViInt32   noOfPoints,
                            ViReal64  start,
                            ViReal64  stop,
                            ViReal64  offsetFrequency[],
                            ViReal64  level[]);

ViStatus _VI_FUNC rsspecan_ReadPhaseTraceNumberOfMeasurementPoints(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   sourceTrace,
                            ViInt32*  noOfPoints);

ViStatus _VI_FUNC rsspecan_FetchPhaseResidualResults(ViSession instrSession,
                                                     ViInt32   modifier,
                                                     ViReal64* resultValue);

ViStatus _VI_FUNC rsspecan_FetchPhaseResidualUserResults(ViSession instrSession,
                                                         ViInt32   userRange,
                                                         ViInt32   modifier,
                                                         ViReal64* resultValue);

ViStatus _VI_FUNC rsspecan_FetchPhaseSpurs(ViSession instrSession,
                                           ViInt32*  noOfPoints,
                                           ViReal64  frequencyValues[],
                                           ViReal64  levelValues[]);

ViStatus _VI_FUNC rsspecan_FetchPhaseSpuriousList(ViSession instrSession,
                                                  ViInt32   window,
                                                  ViInt32   trace,
                                                  ViInt32*  noOfPoints,
                                                  ViReal64  frequencyValues[],
                                                  ViReal64  levelValues[]);

ViStatus _VI_FUNC rsspecan_FetchPhaseSpuriousListCount(ViSession instrSession,
                                                       ViInt32   window,
                                                       ViInt32   trace,
                                                       ViInt32*  listCount);

ViStatus _VI_FUNC rsspecan_FetchPhaseSpuriousJitter(ViSession instrSession,
                                                    ViInt32   window,
                                                    ViInt32   trace,
                                                    ViInt32*  noOfPoints,
                                                    ViReal64  jitterValues[]);

ViStatus _VI_FUNC rsspecan_FetchPhaseFrequencyLevel(ViSession instrSession,
                                                    ViReal64* frequency,
                                                    ViReal64* level);

ViStatus _VI_FUNC rsspecan_FetchPhaseResidualDUTGain(ViSession instrSession,
                                                     ViReal64* DUTGain);

ViStatus _VI_FUNC rsspecan_FetchSweepResultList(ViSession instrSession,
                                                ViInt32   type,
                                                ViInt32   arraySize,
                                                ViReal64  results[],
                                                ViInt32*  returnedValues);

ViStatus _VI_FUNC rsspecan_GetPhaseSpotNoiseYPosition(ViSession instrSession,
                                                      ViInt32   spotNoiseMarkerNumber,
                                                      ViReal64* spotNoiseMarkerResult);

ViStatus _VI_FUNC rsspecan_GetPhaseDecadeSpotNoiseResult(ViSession instrSession,
                                                         ViInt32   arraySize,
                                                         ViReal64  verticalPosition[],
                                                         ViReal64  horizontalPosition[],
                                                         ViInt32*  returnedValues);

ViStatus _VI_FUNC rsspecan_GetPhaseLimitCheckResult(ViSession instrSession,
                                                    ViInt32   reserved,
                                                    ViInt32   limit,
                                                    ViInt32*  state);

ViStatus _VI_FUNC rsspecan_GetPhaseJitterResult(ViSession instrSession,
                                                ViInt32   window,
                                                ViInt32   trace,
                                                ViInt32   type,
                                                ViReal64* value);

ViStatus _VI_FUNC rsspecan_GetPhaseIntegratedMeasurementResult(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   trace,
                            ViInt32   range,
                            ViInt32   type,
                            ViReal64* value);

ViStatus _VI_FUNC rsspecan_ExportPhaseSpurs(ViSession instrSession,
                                            ViInt32   window,
                                            ViInt32   trace,
                                            ViString  filename);

ViStatus _VI_FUNC rsspecan_GetPhaseNumberOfTraceDataPoints(ViSession instrSession,
                                                           ViInt32   window,
                                                           ViInt32   trace,
                                                           ViInt32   type,
                                                           ViInt32*  points);

ViStatus _VI_FUNC rsspecan_ConfigureEMIFinalMeasurementMarkerState(
                            ViSession instrSession,
                            ViBoolean state);

ViStatus _VI_FUNC rsspecan_ConfigureEMIFinalMeasurementMarkerDetector(
                            ViSession instrSession,
                            ViInt32   marker,
                            ViInt32   detector);

ViStatus _VI_FUNC rsspecan_EMIFinalMeasurementMarkerPeakSearch(ViSession instrSession);

ViStatus _VI_FUNC rsspecan_ConfigureEMIFinalMeasurementMarkerDwellTime(
                            ViSession instrSession,
                            ViReal64  dwellTime);

ViStatus _VI_FUNC rsspecan_QueryEMIFinalMeasurementMarkerLimitLine(
                            ViSession instrSession,
                            ViInt32   marker,
                            ViInt32   limit,
                            ViInt32*  markerCondition,
                            ViReal64* markerVerticalDistance);

ViStatus _VI_FUNC rsspecan_QueryEMIFinalMeasurementMarkerResult(
                            ViSession instrSession,
                            ViInt32   marker,
                            ViReal64* result);

ViStatus _VI_FUNC rsspecan_ConfigureEMIFinalMeasurementMarkerDwellTimer(
                            ViSession instrSession,
                            ViInt32   marker,
                            ViReal64  dwellTime);

ViStatus _VI_FUNC rsspecan_ConfigureEMIFinalMeasurementDeltaMarkerDetector(
                            ViSession instrSession,
                            ViInt32   deltaMarker,
                            ViInt32   detector);

ViStatus _VI_FUNC rsspecan_EMIFinalMeasurementDeltaMarkerPeakSearch(ViSession instrSession);

ViStatus _VI_FUNC rsspecan_QueryEMIFinalMeasurementDeltaMarkerLimitLine(
                            ViSession instrSession,
                            ViInt32   deltaMarker,
                            ViInt32   limit,
                            ViInt32*  deltaMarkerCondition,
                            ViReal64* deltaMarkerVerticalDistance);

ViStatus _VI_FUNC rsspecan_QueryEMIFinalMeasurementDeltaMarkerResult(
                            ViSession instrSession,
                            ViInt32   deltaMarker,
                            ViReal64* result);

ViStatus _VI_FUNC rsspecan_ConfigureEMIFinalMeasurementDeltaMarkerDwellTime(
                            ViSession instrSession,
                            ViInt32   deltaMarker,
                            ViReal64  dwellTime);

ViStatus _VI_FUNC rsspecan_ConfigureEMILISNVnetworkType(ViSession instrSession,
                                                        ViInt32   vNetworkType);

ViStatus _VI_FUNC rsspecan_ConfigureEMILISNVnetworkHighPassFilter(
                            ViSession instrSession,
                            ViBoolean highPassFilter);

ViStatus _VI_FUNC rsspecan_ConfigureEMILISNVnetworkPhase(ViSession instrSession,
                                                         ViInt32   phase);

ViStatus _VI_FUNC rsspecan_ConfigureEMIBandwithFilterType(ViSession instrSession,
                                                          ViInt32   filterType);

ViStatus _VI_FUNC rsspecan_TransientAnalysisMode(ViSession instrSession);

ViStatus _VI_FUNC rsspecan_ConfigureTransientAnalysisSignalModel(
                            ViSession instrSession,
                            ViInt32   model);

ViStatus _VI_FUNC rsspecan_GetTransientAnalysisNumberOfStates(
                            ViSession instrSession,
                            ViInt32   model,
                            ViInt32*  numberOfStates);

ViStatus _VI_FUNC rsspecan_ConfigureTransientAnalysisSignalDetection(
                            ViSession instrSession,
                            ViInt32   model,
                            ViBoolean autoMode,
                            ViInt32   arraySize,
                            ViReal64  frequencyOffset[],
                            ViReal64  tolerance[]);

ViStatus _VI_FUNC rsspecan_ConfigureTransientAnalysisSignalDetectionTiming(
                            ViSession instrSession,
                            ViInt32   model,
                            ViBoolean autoMode,
                            ViReal64  minTime,
                            ViReal64  maxTime);

ViStatus _VI_FUNC rsspecan_TransientAnalysisSignalStateTableOperation(
                            ViSession instrSession,
                            ViInt32   model,
                            ViInt32   operation,
                            ViString  tableName);

ViStatus _VI_FUNC rsspecan_GetTransientAnalysisGeneratedHopStatesParameters(
                            ViSession instrSession,
                            ViReal64* startFrequency,
                            ViReal64* stepSize,
                            ViInt32*  numberOfStates);

ViStatus _VI_FUNC rsspecan_AddTransientAnalysisHopStates(ViSession instrSession,
                                                         ViReal64  startFrequency,
                                                         ViReal64  stepSize,
                                                         ViInt32   numberOfStates);

ViStatus _VI_FUNC rsspecan_ReplaceTransientAnalysisHopStates(ViSession instrSession,
                                                             ViReal64  startFrequency,
                                                             ViReal64  stepSize,
                                                             ViInt32   numberOfStates);

ViStatus _VI_FUNC rsspecan_ApplyTransientAnalysisHopStatesGlobalValues(
                            ViSession instrSession,
                            ViReal64  tollerance,
                            ViReal64  offset);

ViStatus _VI_FUNC rsspecan_ConfigureTransientAnalysisDataAcquisition(
                            ViSession instrSession,
                            ViReal64  measurementBandwidth,
                            ViReal64  sampleRate,
                            ViReal64  measurementTime,
                            ViInt32   recordLength);

ViStatus _VI_FUNC rsspecan_ConfigureTransientAnalysisRegion(ViSession instrSession,
                                                            ViReal64  analysisBandwidth,
                                                            ViReal64  deltaFrequency,
                                                            ViReal64  timeGateLength,
                                                            ViReal64  timeGateStart);

ViStatus _VI_FUNC rsspecan_ConfigureTransientAnalysisLinkToFull(
                            ViSession instrSession,
                            ViBoolean bandwidthState,
                            ViReal64  bandwidthPercent,
                            ViBoolean timeState,
                            ViReal64  timePercent);

ViStatus _VI_FUNC rsspecan_ConfigureTransientAnalysisFFTWindow(
                            ViSession instrSession,
                            ViInt32   FFTWindowType);

ViStatus _VI_FUNC rsspecan_ConfigureTransientAnalysisFMVideoBandwidth(
                            ViSession instrSession,
                            ViInt32   FMVideoBandwidth);

ViStatus _VI_FUNC rsspecan_ConfigureTransientAnalysisTimeResolution(
                            ViSession instrSession,
                            ViInt32   autoMode,
                            ViReal64  timeResolution);

ViStatus _VI_FUNC rsspecan_ConfigureTransientAnalysisChirpDetectionEnabled(
                            ViSession instrSession,
                            ViBoolean enabled);

ViStatus _VI_FUNC rsspecan_ConfigureTransientAnalysisHopChirpFrequency(
                            ViSession instrSession,
                            ViInt32   model,
                            ViInt32   reference,
                            ViReal64  length,
                            ViReal64  offsetBegin,
                            ViReal64  offsetEnd);

ViStatus _VI_FUNC rsspecan_ConfigureTransientAnalysisHopChirpPower(
                            ViSession instrSession,
                            ViInt32   model,
                            ViInt32   reference,
                            ViReal64  length,
                            ViReal64  offsetBegin,
                            ViReal64  offsetEnd);

ViStatus _VI_FUNC rsspecan_ConfigureTransientAnalysisHopChirpFrequencyDeviation(
                            ViSession instrSession,
                            ViInt32   model,
                            ViInt32   reference,
                            ViReal64  length,
                            ViReal64  offsetBegin,
                            ViReal64  offsetEnd);

ViStatus _VI_FUNC rsspecan_ConfigureTransientAnalysisHopChirpPhaseDeviation(
                            ViSession instrSession,
                            ViInt32   model,
                            ViInt32   reference,
                            ViReal64  length,
                            ViReal64  offsetBegin,
                            ViReal64  offsetEnd);

ViStatus _VI_FUNC rsspecan_ConfigureTransientAnalysisCompensateHopFrequencyDeviation(
                            ViSession instrSession,
                            ViBoolean compensateHopFrequencyDev);

ViStatus _VI_FUNC rsspecan_ConfigureTransientAnalysisCompensateChirpRateDeviation(
                            ViSession instrSession,
                            ViBoolean compensateChirpRateDeviation);

ViStatus _VI_FUNC rsspecan_ConfigureTransientAnalysisHopChirpSettlingTolerance(
                            ViSession instrSession,
                            ViInt32   model,
                            ViInt32   signalModelState,
                            ViReal64  settlingTolerance);

ViStatus _VI_FUNC rsspecan_ConfigureTransientAnalysisHopChirpPhaseNoiseFrequency(
                            ViSession instrSession,
                            ViInt32   model,
                            ViInt32   reference,
                            ViReal64  length,
                            ViReal64  offsetBegin,
                            ViReal64  offsetEnd,
                            ViReal64  start,
                            ViReal64  stop);

ViStatus _VI_FUNC rsspecan_TransientAnalysisContinueSingleSweep(ViSession instrSession);

ViStatus _VI_FUNC rsspecan_TransienAnalysisWaitForEndOfAutoRefresh(ViSession instrSession);

ViStatus _VI_FUNC rsspecan_QueryTransientAnalysisFFTWindowLength(
                            ViSession instrSession,
                            ViInt32*  length);

ViStatus _VI_FUNC rsspecan_TransientAnalysisAddWindow(ViSession instrSession,
                                                      ViInt32   stringSize,
                                                      ViString  referenceWindowName,
                                                      ViInt32   direction,
                                                      ViInt32   windowType,
                                                      ViChar    returnedWindowName[]);

ViStatus _VI_FUNC rsspecan_TransientAnalysisReplaceWindow(ViSession instrSession,
                                                          ViString  windowName,
                                                          ViInt32   windowType);

ViStatus _VI_FUNC rsspecan_ConfigureTransientAnalysisSettlingTimePhaseLimits(
                            ViSession instrSession,
                            ViInt32   lowerLimit,
                            ViInt32   upperLimit);

ViStatus _VI_FUNC rsspecan_ConfigureTransientDigitalIQ40GEnabled(
                            ViSession instrSession,
                            ViBoolean enabled);

ViStatus _VI_FUNC rsspecan_QueryTransientDigitalIQ40GSampleRate(
                            ViSession instrSession,
                            ViReal64* sampleRate);

ViStatus _VI_FUNC rsspecan_QueryTransientDigitalIQ40GConnectedDevice(
                            ViSession instrSession,
                            ViInt32   bufferSize,
                            ViChar    connectedDevice[]);

ViStatus _VI_FUNC rsspecan_ConfigureTransientAnalysisResultAutomaticRangeScaling(
                            ViSession instrSession,
                            ViBoolean autoMode);

ViStatus _VI_FUNC rsspecan_ConfigureTransientAnalysisResultReferencePoint(
                            ViSession instrSession,
                            ViInt32   reference,
                            ViReal64  offset);

ViStatus _VI_FUNC rsspecan_ConfigureTransientAnalysisResultRange(
                            ViSession instrSession,
                            ViInt32   alignment,
                            ViReal64  length);

ViStatus _VI_FUNC rsspecan_ConfigureTransientAnalysisHopResultTable(
                            ViSession instrSession,
                            ViInt32   header,
                            ViBoolean visibility);

ViStatus _VI_FUNC rsspecan_ConfigureTransientAnalysisHopResultTableFrequency(
                            ViSession instrSession,
                            ViInt32   parameter,
                            ViBoolean visibility,
                            ViInt32   scaling);

ViStatus _VI_FUNC rsspecan_ConfigureTransientAnalysisHopResultTablePower(
                            ViSession instrSession,
                            ViInt32   parameter,
                            ViBoolean visibility);

ViStatus _VI_FUNC rsspecan_ConfigureTransientAnalysisHopResultTableState(
                            ViSession instrSession,
                            ViInt32   parameter,
                            ViBoolean visibility,
                            ViInt32   scaling);

ViStatus _VI_FUNC rsspecan_ConfigureTransientAnalysisHopResultTableTiming(
                            ViSession instrSession,
                            ViInt32   parameter,
                            ViBoolean visibility,
                            ViInt32   scaling);

ViStatus _VI_FUNC rsspecan_ConfigureTransientAnalysisHopResultTablePhase(
                            ViSession instrSession,
                            ViInt32   parameter,
                            ViBoolean visibility,
                            ViInt32   scaling);

ViStatus _VI_FUNC rsspecan_ConfigureTransientAnalysisHopResultTableFMSettling(
                            ViSession instrSession,
                            ViInt32   parameter,
                            ViBoolean visibility,
                            ViInt32   scaling);

ViStatus _VI_FUNC rsspecan_ConfigureTransientAnalysisHopResultTablePMSettling(
                            ViSession instrSession,
                            ViInt32   parameter,
                            ViBoolean visibility,
                            ViInt32   scaling);

ViStatus _VI_FUNC rsspecan_ConfigureTransientAnalysisChirpResultTable(
                            ViSession instrSession,
                            ViInt32   header,
                            ViBoolean visibility);

ViStatus _VI_FUNC rsspecan_ConfigureTransientAnalysisChirpResultTableFrequency(
                            ViSession instrSession,
                            ViInt32   parameter,
                            ViBoolean visibility,
                            ViInt32   scaling);

ViStatus _VI_FUNC rsspecan_ConfigureTransientAnalysisChirpResultTablePower(
                            ViSession instrSession,
                            ViInt32   parameter,
                            ViBoolean visibility);

ViStatus _VI_FUNC rsspecan_ConfigureTransientAnalysisChirpResultTableState(
                            ViSession instrSession,
                            ViInt32   parameter,
                            ViBoolean visibility,
                            ViInt32   reserved);

ViStatus _VI_FUNC rsspecan_ConfigureTransientAnalysisChirpResultTableTiming(
                            ViSession instrSession,
                            ViInt32   parameter,
                            ViBoolean visibility,
                            ViInt32   scaling);

ViStatus _VI_FUNC rsspecan_ConfigureTransientAnalysisChirpResultTablePhase(
                            ViSession instrSession,
                            ViInt32   parameter,
                            ViBoolean visibility,
                            ViInt32   scaling);

ViStatus _VI_FUNC rsspecan_ConfigureTransientAnalysisChirpResultTableFMSettling(
                            ViSession instrSession,
                            ViInt32   parameter,
                            ViBoolean visibility,
                            ViInt32   scaling);

ViStatus _VI_FUNC rsspecan_ConfigureTransientAnalysisChirpResultTablePMSettling(
                            ViSession instrSession,
                            ViInt32   parameter,
                            ViBoolean visibility,
                            ViInt32   scaling);

ViStatus _VI_FUNC rsspecan_ConfigureTransientAnalysisChirpResultTableFrequencyINL(
                            ViSession instrSession,
                            ViInt32   parameter,
                            ViBoolean visibility);

ViStatus _VI_FUNC rsspecan_ExportTransientAnalysisResultTable(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   exportCollumns,
                            ViString  filename);

ViStatus _VI_FUNC rsspecan_ConfigureTransientAnalysisParameterDistributionHistogramBins(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   histogramBins);

ViStatus _VI_FUNC rsspecan_QueryTransientAnalysisParameterDistributionAxis(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32*  xAxis,
                            ViInt32*  yAxis);

ViStatus _VI_FUNC rsspecan_ConfigureTransientAnalysisParameterDistributionHopFrequency(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   xAxis,
                            ViInt32   yAxis);

ViStatus _VI_FUNC rsspecan_ConfigureTransientAnalysisParameterDistributionHopPower(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   xAxis,
                            ViInt32   yAxis);

ViStatus _VI_FUNC rsspecan_ConfigureTransientAnalysisParameterDistributionHopTiming(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   xAxis,
                            ViInt32   yAxis);

ViStatus _VI_FUNC rsspecan_ConfigureTransientAnalysisParameterDistributionHopPhase(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   xAxis,
                            ViInt32   yAxis);

ViStatus _VI_FUNC rsspecan_ConfigureTransientAnalysisParameterDistributionHopState(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   xAxis,
                            ViInt32   yAxis);

ViStatus _VI_FUNC rsspecan_ConfigureTransientAnalysisParameterDistributionChirpFrequency(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   xAxis,
                            ViInt32   yAxis);

ViStatus _VI_FUNC rsspecan_ConfigureTransientAnalysisParameterDistributionChirpPower(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   xAxis,
                            ViInt32   yAxis);

ViStatus _VI_FUNC rsspecan_ConfigureTransientAnalysisParameterDistributionChirpTiming(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   xAxis,
                            ViInt32   yAxis);

ViStatus _VI_FUNC rsspecan_ConfigureTransientAnalysisParameterDistributionChirpPhase(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   xAxis,
                            ViInt32   yAxis);

ViStatus _VI_FUNC rsspecan_ConfigureTransientAnalysisParameterDistributionChirpState(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   xAxis,
                            ViInt32   yAxis);

ViStatus _VI_FUNC rsspecan_ConfigureTransientAnalysisParameterTrendSwapXY(
                            ViSession instrSession,
                            ViInt32   window);

ViStatus _VI_FUNC rsspecan_QueryTransientAnalysisParameterTrendAxis(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32*  xAxis,
                            ViInt32*  yAxis);

ViStatus _VI_FUNC rsspecan_ConfigureTransientAnalysisParameterTrendHopFrequency(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   yAxis,
                            ViInt32   xAxis);

ViStatus _VI_FUNC rsspecan_ConfigureTransientAnalysisParameterTrendHopFrequencyAxis(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   axis,
                            ViInt32   displayParameter);

ViStatus _VI_FUNC rsspecan_ConfigureTransientAnalysisParameterTrendHopPower(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   yAxis,
                            ViInt32   xAxis);

ViStatus _VI_FUNC rsspecan_ConfigureTransientAnalysisParameterTrendHopPowerAxis(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   axis,
                            ViInt32   displayParameter);

ViStatus _VI_FUNC rsspecan_ConfigureTransientAnalysisParameterTrendHopState(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   yAxis,
                            ViInt32   xAxis);

ViStatus _VI_FUNC rsspecan_ConfigureTransientAnalysisParameterTrendHopStateAxis(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   axis,
                            ViInt32   displayParameter);

ViStatus _VI_FUNC rsspecan_ConfigureTransientAnalysisParameterTrendHopTiming(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   yAxis,
                            ViInt32   xAxis);

ViStatus _VI_FUNC rsspecan_ConfigureTransientAnalysisParameterTrendHopTimingAxis(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   axis,
                            ViInt32   displayParameter);

ViStatus _VI_FUNC rsspecan_ConfigureTransientAnalysisParameterTrendHopPhase(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   yAxis,
                            ViInt32   xAxis);

ViStatus _VI_FUNC rsspecan_ConfigureTransientAnalysisParameterTrendHopPhaseAxis(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   axis,
                            ViInt32   displayParameter);

ViStatus _VI_FUNC rsspecan_ConfigureTransientAnalysisParameterTrendChirpFrequency(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   yAxis,
                            ViInt32   xAxis);

ViStatus _VI_FUNC rsspecan_ConfigureTransientAnalysisParameterTrendChirpFrequencyAxis(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   axis,
                            ViInt32   displayParameter);

ViStatus _VI_FUNC rsspecan_ConfigureTransientAnalysisParameterTrendChirpPower(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   yAxis,
                            ViInt32   xAxis);

ViStatus _VI_FUNC rsspecan_ConfigureTransientAnalysisParameterTrendChirpPowerAxis(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   axis,
                            ViInt32   displayParameter);

ViStatus _VI_FUNC rsspecan_ConfigureTransientAnalysisParameterTrendChirpState(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   yAxis,
                            ViInt32   xAxis);

ViStatus _VI_FUNC rsspecan_ConfigureTransientAnalysisParameterTrendChirpStateAxis(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   axis,
                            ViInt32   displayParameter);

ViStatus _VI_FUNC rsspecan_ConfigureTransientAnalysisParameterTrendChirpTiming(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   yAxis,
                            ViInt32   xAxis);

ViStatus _VI_FUNC rsspecan_ConfigureTransientAnalysisParameterTrendChirpTimingAxis(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   axis,
                            ViInt32   displayParameter);

ViStatus _VI_FUNC rsspecan_ConfigureTransientAnalysisParameterTrendChirpPhase(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   yAxis,
                            ViInt32   xAxis);

ViStatus _VI_FUNC rsspecan_ConfigureTransientAnalysisParameterTrendChirpPhaseAxis(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   axis,
                            ViInt32   displayParameter);

ViStatus _VI_FUNC rsspecan_ConfigureTransientAnalysisAutomaticGridScaling(
                            ViSession instrSession,
                            ViInt32   window,
                            ViBoolean autoScale);

ViStatus _VI_FUNC rsspecan_ConfigureTransientAnalysisAbsoluteScaling(
                            ViSession instrSession,
                            ViInt32   window,
                            ViReal64  minimum,
                            ViReal64  maximum);

ViStatus _VI_FUNC rsspecan_ConfigureTransientAnalysisRelativeScaling(
                            ViSession instrSession,
                            ViInt32   window,
                            ViReal64  perDivison,
                            ViReal64  referencePosition,
                            ViReal64  referenceValue);

ViStatus _VI_FUNC rsspecan_ConfigureTransientAnalysisScalingDisplayRange(
                            ViSession instrSession,
                            ViInt32   window,
                            ViReal64  range);

ViStatus _VI_FUNC rsspecan_ConfigureTransientAnalysisUnit(ViSession instrSession,
                                                          ViInt32   unit);

ViStatus _VI_FUNC rsspecan_ConfigureTransientAnalysisEvaluationBasis(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   evaluationBasis);

ViStatus _VI_FUNC rsspecan_ConfigureTransientAnalysisSelection(
                            ViSession instrSession,
                            ViInt32   model,
                            ViInt32   number);

ViStatus _VI_FUNC rsspecan_QueryTransientAnalysisTraceLength(ViSession instrSession,
                                                             ViInt32   window,
                                                             ViInt32   trace,
                                                             ViInt32*  numberOfTracePoints);

ViStatus _VI_FUNC rsspecan_ConfigureTransientAnalysisStatisticType(
                            ViSession instrSession,
                            ViInt32   statisticType);

ViStatus _VI_FUNC rsspecan_ConfigureTransientAnalysisTraceMaximumPoints(
                            ViSession instrSession,
                            ViInt32   maximumPoints);

ViStatus _VI_FUNC rsspecan_ConfigureTransientAnalysisLongCaptureBuffer(
                            ViSession instrSession,
                            ViInt32   longCaptureBuffer);

ViStatus _VI_FUNC rsspecan_ConfigureTransientAnalysisEvaluation(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   trace,
                            ViInt32   evaluationType);

ViStatus _VI_FUNC rsspecan_ConfigureTransientAnalysisSpectrogramDetector(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   detector);

ViStatus _VI_FUNC rsspecan_ConfigureTransientAnalysisLinkedMarker(
                            ViSession instrSession,
                            ViBoolean state);

ViStatus _VI_FUNC rsspecan_ConfigureTransientAnalysisLinkedDeltamarker(
                            ViSession instrSession,
                            ViInt32   markerNumber,
                            ViBoolean state);

ViStatus _VI_FUNC rsspecan_StoreTransientAnalysisCurrentMeasurement(
                            ViSession instrSession,
                            ViString  fileName);

ViStatus _VI_FUNC rsspecan_QueryTransientAnalysisHopTotal(ViSession instrSession,
                                                          ViInt32*  totalHops);

ViStatus _VI_FUNC rsspecan_QueryTransientAnalysisHopTotalInCaptureBuffer(
                            ViSession instrSession,
                            ViInt32*  totalHops);

ViStatus _VI_FUNC rsspecan_QueryTransientAnalysisHopIDs(ViSession instrSession,
                                                        ViInt32   queryRange,
                                                        ViInt32   arraySize,
                                                        ViInt32   IDs[],
                                                        ViInt32*  returnedValues);

ViStatus _VI_FUNC rsspecan_QueryTransientAnalysisHopNumbers(ViSession instrSession,
                                                            ViInt32   queryRange,
                                                            ViInt32   arraySize,
                                                            ViInt32   numbers[],
                                                            ViInt32*  returnedValues);

ViStatus _VI_FUNC rsspecan_QueryTransientAnalysisHopResultFrequency(
                            ViSession instrSession,
                            ViInt32   parameter,
                            ViInt32   resultType,
                            ViInt32   queryRange,
                            ViInt32   arraySize,
                            ViReal64  results[],
                            ViInt32*  returnedValues);

ViStatus _VI_FUNC rsspecan_QueryTransientAnalysisHopResultTiming(
                            ViSession instrSession,
                            ViInt32   parameter,
                            ViInt32   resultType,
                            ViInt32   queryRange,
                            ViInt32   arraySize,
                            ViReal64  results[],
                            ViInt32*  returnedValues);

ViStatus _VI_FUNC rsspecan_QueryTransientAnalysisHopResultPower(
                            ViSession instrSession,
                            ViInt32   parameter,
                            ViInt32   resultType,
                            ViInt32   queryRange,
                            ViInt32   arraySize,
                            ViReal64  results[],
                            ViInt32*  returnedValues);

ViStatus _VI_FUNC rsspecan_QueryTransientAnalysisHopResultPhase(
                            ViSession instrSession,
                            ViInt32   parameter,
                            ViInt32   resultType,
                            ViInt32   queryRange,
                            ViInt32   arraySize,
                            ViReal64  results[],
                            ViInt32*  returnedValues);

ViStatus _VI_FUNC rsspecan_QueryTransientAnalysisHopResultState(
                            ViSession instrSession,
                            ViInt32   resultType,
                            ViInt32   queryRange,
                            ViInt32   arraySize,
                            ViReal64  results[],
                            ViInt32*  returnedValues);

ViStatus _VI_FUNC rsspecan_QueryTransientAnalysisHopResultAdditionalState(
                            ViSession instrSession,
                            ViInt32   parameter,
                            ViInt32   resultType,
                            ViInt32   queryRange,
                            ViInt32   arraySize,
                            ViReal64  results[],
                            ViInt32*  returnedValues);

ViStatus _VI_FUNC rsspecan_QueryTransientAnalysisHopResultTable(
                            ViSession instrSession,
                            ViInt32   start,
                            ViInt32   stop,
                            ViInt32   arraySize,
                            ViReal64  results[],
                            ViInt32*  returnedValues);

ViStatus _VI_FUNC rsspecan_QueryTransientAnalysisChirpTotal(ViSession instrSession,
                                                            ViInt32*  totalHops);

ViStatus _VI_FUNC rsspecan_QueryTransientAnalysisChirpTotalInCaptureBuffer(
                            ViSession instrSession,
                            ViInt32*  totalChirps);

ViStatus _VI_FUNC rsspecan_QueryTransientAnalysisChirpIDs(ViSession instrSession,
                                                          ViInt32   queryRange,
                                                          ViInt32   arraySize,
                                                          ViInt32   IDs[],
                                                          ViInt32*  returnedValues);

ViStatus _VI_FUNC rsspecan_QueryTransientAnalysisChirpNumbers(
                            ViSession instrSession,
                            ViInt32   queryRange,
                            ViInt32   arraySize,
                            ViInt32   numbers[],
                            ViInt32*  returnedValues);

ViStatus _VI_FUNC rsspecan_QueryTransientAnalysisChirpResultFrequency(
                            ViSession instrSession,
                            ViInt32   parameter,
                            ViInt32   resultType,
                            ViInt32   queryRange,
                            ViInt32   arraySize,
                            ViReal64  results[],
                            ViInt32*  returnedValues);

ViStatus _VI_FUNC rsspecan_QueryTransientAnalysisChirpResultTiming(
                            ViSession instrSession,
                            ViInt32   parameter,
                            ViInt32   resultType,
                            ViInt32   queryRange,
                            ViInt32   arraySize,
                            ViReal64  results[],
                            ViInt32*  returnedValues);

ViStatus _VI_FUNC rsspecan_QueryTransientAnalysisChirpResultPower(
                            ViSession instrSession,
                            ViInt32   parameter,
                            ViInt32   resultType,
                            ViInt32   queryRange,
                            ViInt32   arraySize,
                            ViReal64  results[],
                            ViInt32*  returnedValues);

ViStatus _VI_FUNC rsspecan_QueryTransientAnalysisChirpResultPhase(
                            ViSession instrSession,
                            ViInt32   parameter,
                            ViInt32   resultType,
                            ViInt32   queryRange,
                            ViInt32   arraySize,
                            ViReal64  results[],
                            ViInt32*  returnedValues);

ViStatus _VI_FUNC rsspecan_QueryTransientAnalysisChirpResultState(
                            ViSession instrSession,
                            ViInt32   resultType,
                            ViInt32   queryRange,
                            ViInt32   arraySize,
                            ViReal64  results[],
                            ViInt32*  returnedValues);

ViStatus _VI_FUNC rsspecan_QueryTransientAnalysisChirpResultTable(
                            ViSession instrSession,
                            ViInt32   start,
                            ViInt32   stop,
                            ViInt32   arraySize,
                            ViReal64  results[],
                            ViInt32*  returnedValues);

ViStatus _VI_FUNC rsspecan_SetVSAMode(ViSession instrSession);

ViStatus _VI_FUNC rsspecan_SetVSAFactoryDefaults(ViSession instrSession,
                                                 ViInt32   defaults);

ViStatus _VI_FUNC rsspecan_PresetVSAScreens(ViSession instrSession);

ViStatus _VI_FUNC rsspecan_VSARefLevelAdjust(ViSession instrSession,
                                             ViInt32   window,
                                             ViUInt32  timeout);

ViStatus _VI_FUNC rsspecan_VSADigitalStandardLoad(ViSession instrSession,
                                                  ViInt32   reserved,
                                                  ViString  digitalStandard);

ViStatus _VI_FUNC rsspecan_VSADigitalStandardSave(ViSession instrSession,
                                                  ViInt32   reserved,
                                                  ViString  userDefinedStandard,
                                                  ViString  comment);

ViStatus _VI_FUNC rsspecan_VSADigitalStandardDelete(ViSession instrSession,
                                                    ViInt32   reserved,
                                                    ViString  digitalStandard);

ViStatus _VI_FUNC rsspecan_VSADigitalStandardPreset(ViSession instrSession);

ViStatus _VI_FUNC rsspecan_ConfigureVSADigitalIQ40GEnabled(ViSession instrSession,
                                                           ViBoolean enabled);

ViStatus _VI_FUNC rsspecan_QueryVSADigitalIQ40GSampleRate(ViSession instrSession,
                                                          ViReal64* sampleRate);

ViStatus _VI_FUNC rsspecan_QueryVSADigitalIQ40GConnectedDevice(
                            ViSession instrSession,
                            ViChar    connectedDevice[]);

ViStatus _VI_FUNC rsspecan_ConfigureVSAModulationSettings(ViSession instrSession,
                                                          ViInt32   modulation,
                                                          ViString  mappingName,
                                                          ViReal64  symbolRate);

ViStatus _VI_FUNC rsspecan_ConfigureVSAFSKRefDeviation(ViSession instrSession,
                                                       ViReal64  referenceDeviation,
                                                       ViInt32   mode);

ViStatus _VI_FUNC rsspecan_ConfigureVSATXFilter(ViSession instrSession,
                                                ViInt32   type,
                                                ViReal64  alphaBT);

ViStatus _VI_FUNC rsspecan_ConfigureVSAUserDefinedTXFilter(ViSession instrSession,
                                                           ViString  filterFileName);

ViStatus _VI_FUNC rsspecan_GetVSAModulationMappingCatalog(ViSession instrSession,
                                                          ViInt32*  numberOfMappings,
                                                          ViInt32   bufferSize,
                                                          ViChar    mappingsList[]);

ViStatus _VI_FUNC rsspecan_GetVSAModulationUQAMLevel(ViSession instrSession,
                                                     ViInt32*  specificLevel);

ViStatus _VI_FUNC rsspecan_VSALoadUserModulation(ViSession instrSession,
                                                 ViString  userModulation);

ViStatus _VI_FUNC rsspecan_ConfigureVSASignalType(ViSession instrSession,
                                                  ViInt32   signalType);

ViStatus _VI_FUNC rsspecan_ConfigureVSABurstSignal(ViSession instrSession,
                                                   ViInt32   usefulLength,
                                                   ViInt32   maxLength,
                                                   ViInt32   runIn,
                                                   ViInt32   runOut);

ViStatus _VI_FUNC rsspecan_ConfigureVSAPatternSignalState(ViSession instrSession,
                                                          ViBoolean patternSignal);

ViStatus _VI_FUNC rsspecan_ConfigureVSAPatternOffset(ViSession instrSession,
                                                     ViBoolean patternOffsetState,
                                                     ViInt32   patternOffset);

ViStatus _VI_FUNC rsspecan_ConfigureVSAFramePattern(ViSession instrSession,
                                                    ViBoolean sameAsDataSymbols,
                                                    ViInt32   type);

ViStatus _VI_FUNC rsspecan_GetVSAFrameMappingCatalog(ViSession instrSession,
                                                     ViInt32   bufferSize,
                                                     ViChar    mappingsList[]);

ViStatus _VI_FUNC rsspecan_VSALoadFrameConfig(ViSession instrSession,
                                              ViString  config);

ViStatus _VI_FUNC rsspecan_ConfigureVSAFrameMappingSelect(ViSession instrSession,
                                                          ViString  fileName);

ViStatus _VI_FUNC rsspecan_VSALoadFrameStructure(ViSession instrSession,
                                                 ViString  frameStructure);

ViStatus _VI_FUNC rsspecan_VSASaveFrameStructure(ViSession instrSession,
                                                 ViString  frameStructure);

ViStatus _VI_FUNC rsspecan_VSAFrameStructureEdit(ViSession instrSession,
                                                 ViString  structure);

ViStatus _VI_FUNC rsspecan_ConfigureVSAFrameMode(ViSession instrSession,
                                                 ViInt32   mode);

ViStatus _VI_FUNC rsspecan_ConfigureVSAFrameUserFile(ViSession instrSession,
                                                     ViString  fileName);

ViStatus _VI_FUNC rsspecan_ConfigureVSAFramePSK(ViSession instrSession,
                                                ViInt32   format,
                                                ViInt32   nState);

ViStatus _VI_FUNC rsspecan_ConfigureVSAFrameQAM(ViSession instrSession,
                                                ViInt32   format,
                                                ViInt32   nState);

ViStatus _VI_FUNC rsspecan_ConfigureVSAFrameQPSK(ViSession instrSession,
                                                 ViInt32   format);

ViStatus _VI_FUNC rsspecan_ConfigureVSAFrameAPSKNState(ViSession instrSession,
                                                       ViInt32   nState);

ViStatus _VI_FUNC rsspecan_ConfigureVSAFrameASKNState(ViSession instrSession,
                                                      ViInt32   nState);

ViStatus _VI_FUNC rsspecan_ConfigureVSAFrameBoosting(ViSession instrSession,
                                                     ViInt32   type,
                                                     ViReal64  boosting);

ViStatus _VI_FUNC rsspecan_ConfigureVSAFrameModulation(ViSession instrSession,
                                                       ViInt32   type,
                                                       ViInt32   modulation);

ViStatus _VI_FUNC rsspecan_ConfigureVSAFrameDescription(ViSession instrSession,
                                                        ViString  description);

ViStatus _VI_FUNC rsspecan_QueryVSAFrameStartSample(ViSession instrSession,
                                                    ViInt32*  startSample);

ViStatus _VI_FUNC rsspecan_ConfigureVSADataCaptureSettings(ViSession instrSession,
                                                           ViInt32   mode,
                                                           ViReal64  captureLength,
                                                           ViInt32   captureOversampling,
                                                           ViBoolean swapIQ);

ViStatus _VI_FUNC rsspecan_ConfigureVSADemodulationSymbolRecordLength(
                            ViSession instrSession,
                            ViInt32   recordLength_inSymbols);

ViStatus _VI_FUNC rsspecan_ConfigureVSABurstSearchAuto(ViSession instrSession,
                                                       ViInt32   burstSearch);

ViStatus _VI_FUNC rsspecan_ConfigureVSABurstSearch(ViSession instrSession,
                                                   ViBoolean searchState,
                                                   ViInt32   burstSearchMode);

ViStatus _VI_FUNC rsspecan_ConfigureVSABurstSearchAutoConfiguration(
                            ViSession instrSession,
                            ViBoolean autoConfiguration,
                            ViInt32   tolerance,
                            ViInt32   minGaplen);

ViStatus _VI_FUNC rsspecan_GetVSABurstFound(ViSession  instrSession,
                                            ViBoolean* found);

ViStatus _VI_FUNC rsspecan_ConfigureVSAPatternSearchAuto(ViSession instrSession,
                                                         ViInt32   patternSearch);

ViStatus _VI_FUNC rsspecan_ConfigureVSAPatternSearch(ViSession instrSession,
                                                     ViBoolean searchState,
                                                     ViInt32   patternSearchMode,
                                                     ViString  pattern);

ViStatus _VI_FUNC rsspecan_ConfigureVSAPatternSearchAutoConfiguration(
                            ViSession instrSession,
                            ViBoolean autoConfiguration,
                            ViReal64  IQCorrelationThreshold);

ViStatus _VI_FUNC rsspecan_GetVSAPatternFound(ViSession instrSession,
                                              ViInt32   reserved,
                                              ViInt32   bufferSize,
                                              ViChar    found[]);

ViStatus _VI_FUNC rsspecan_GetVSAPatternCatalog(ViSession instrSession,
                                                ViInt32   selection,
                                                ViInt32*  numberOfPatterns,
                                                ViInt32   bufferSize,
                                                ViChar    patternsList[]);

ViStatus _VI_FUNC rsspecan_ConfigureVSAPatternOperations(ViSession instrSession,
                                                         ViInt32   operation,
                                                         ViString  patternName);

ViStatus _VI_FUNC rsspecan_ConfigureVSAPatternSettings(ViSession instrSession,
                                                       ViString  patternName,
                                                       ViString  description,
                                                       ViString  comment,
                                                       ViInt32   degreeOfModulation,
                                                       ViString  patternData);

ViStatus _VI_FUNC rsspecan_ConfigureVSAResultLength(ViSession instrSession,
                                                    ViInt32   resultLength);

ViStatus _VI_FUNC rsspecan_ConfigureVSAResultRangeReference(ViSession instrSession,
                                                            ViInt32   reference);

ViStatus _VI_FUNC rsspecan_ConfigureVSADisplayAlignment(ViSession instrSession,
                                                        ViInt32   alignment,
                                                        ViInt32   alignOffset);

ViStatus _VI_FUNC rsspecan_ConfigureVSASymbolNoAtReference(ViSession instrSession,
                                                           ViInt32   symbolNumber);

ViStatus _VI_FUNC rsspecan_ConfigureVSAEvaluationRange(ViSession instrSession,
                                                       ViBoolean entireResultRange,
                                                       ViReal64  start,
                                                       ViReal64  stop);

ViStatus _VI_FUNC rsspecan_ConfigureVSAStatisticCount(ViSession instrSession,
                                                      ViInt32   statisticCount);

ViStatus _VI_FUNC rsspecan_ConfigureVSAResultRangeNumber(ViSession instrSession,
                                                         ViInt32   resultRangeNumber);

ViStatus _VI_FUNC rsspecan_GetVSACurrentStatisticsCounter(ViSession instrSession,
                                                          ViInt32   evaluationType,
                                                          ViInt32*  currentCounter);

ViStatus _VI_FUNC rsspecan_ConfigureVSACompensation(ViSession instrSession,
                                                    ViInt32   compensateFor,
                                                    ViBoolean compensationState);

ViStatus _VI_FUNC rsspecan_ConfigureVSAEqualizer(ViSession instrSession,
                                                 ViBoolean state,
                                                 ViInt32   mode,
                                                 ViInt32   length);

ViStatus _VI_FUNC rsspecan_ConfigureVSAEqualizerFileFormat(ViSession instrSession,
                                                           ViInt32   fileFormat);

ViStatus _VI_FUNC rsspecan_ConfigureVSAEqualizerOperations(ViSession instrSession,
                                                           ViInt32   operation,
                                                           ViString  equalizerName);

ViStatus _VI_FUNC rsspecan_ConfigureVSAEVMModulation(ViSession instrSession,
                                                     ViInt32   calculation);

ViStatus _VI_FUNC rsspecan_ConfigureVSAOptimization(ViSession instrSession,
                                                    ViInt32   optimization);

ViStatus _VI_FUNC rsspecan_ConfigureVSAEVMOffsetState(ViSession instrSession,
                                                      ViBoolean EVMOffset);

ViStatus _VI_FUNC rsspecan_ConfigureVSAPatternForSync(ViSession instrSession,
                                                      ViInt32   patternForSync);

ViStatus _VI_FUNC rsspecan_ConfigureVSAKnownDataSource(ViSession instrSession,
                                                       ViInt32   source);

ViStatus _VI_FUNC rsspecan_ConfigureVSAKnownDataPRBSType(ViSession instrSession,
                                                         ViInt32   type);

ViStatus _VI_FUNC rsspecan_ConfigureVSAKnownData(ViSession instrSession,
                                                 ViBoolean knownDataState,
                                                 ViString  dataFileName);

ViStatus _VI_FUNC rsspecan_ConfigureVSAKnownDataNegateFeedback(
                            ViSession instrSession,
                            ViBoolean autoEnabled,
                            ViBoolean enabled);

ViStatus _VI_FUNC rsspecan_ConfigureVSAKnownDataPolynomial(ViSession instrSession,
                                                           ViBoolean autoEnabled,
                                                           ViInt32   arraySize,
                                                           ViInt32   order[]);

ViStatus _VI_FUNC rsspecan_ConfigureVSAKnownDataPRBSPattern(ViSession instrSession,
                                                            ViBoolean autoEnabled,
                                                            ViBoolean enabled);

ViStatus _VI_FUNC rsspecan_ConfigureVSAFineSynchronization(ViSession instrSession,
                                                           ViBoolean autoSet,
                                                           ViInt32   mode,
                                                           ViReal64  SER);

ViStatus _VI_FUNC rsspecan_QueryVSAFineSynchronizationResult(ViSession instrSession,
                                                             ViInt32*  result);

ViStatus _VI_FUNC rsspecan_ConfigureVSAEstimationPointsSymbol(
                            ViSession instrSession,
                            ViInt32   estimationPointsSym);

ViStatus _VI_FUNC rsspecan_ConfigureVSAMeasurementFilter(ViSession instrSession,
                                                         ViBoolean autoMeasurementFilter,
                                                         ViInt32   type,
                                                         ViReal64  alphaBT);

ViStatus _VI_FUNC rsspecan_ConfigureVSAUserDefinedMeasurementFilter(
                            ViSession instrSession,
                            ViString  filterFileName);

ViStatus _VI_FUNC rsspecan_ConfigureVSAFilterCharacteristic(ViSession instrSession,
                                                            ViReal64  alphaBT);

ViStatus _VI_FUNC rsspecan_ConfigureVSAMarkerState(ViSession instrSession,
                                                   ViInt32   markerNumber,
                                                   ViBoolean markerState);

ViStatus _VI_FUNC rsspecan_ConfigureVSAScreenCoupling(ViSession instrSession,
                                                      ViInt32   reserved,
                                                      ViInt32   reserved2,
                                                      ViBoolean state);

ViStatus _VI_FUNC rsspecan_ConfigureVSAMarkerSearch(ViSession instrSession,
                                                    ViInt32   search);

ViStatus _VI_FUNC rsspecan_ConfigureVSAMarkerPeakSearch(ViSession instrSession,
                                                        ViInt32   window);

ViStatus _VI_FUNC rsspecan_ConfigureVSADeltamarkerPeakSearch(ViSession instrSession,
                                                             ViInt32   window,
                                                             ViInt32   marker);

ViStatus _VI_FUNC rsspecan_ConfigureVSADisplayYAxis(ViSession instrSession,
                                                    ViInt32   window,
                                                    ViReal64  scaling,
                                                    ViReal64  referenceValue);

ViStatus _VI_FUNC rsspecan_ConfigureVSADisplayYAxisStatisticMeasurement(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   yUnit,
                            ViReal64  yAxisMaxValue,
                            ViReal64  yAxisMinValue);

ViStatus _VI_FUNC rsspecan_ConfigureVSADisplayYAxisRefPosition(
                            ViSession instrSession,
                            ViReal64  referencePosition);

ViStatus _VI_FUNC rsspecan_ConfigureVSADisplayXAxis(ViSession instrSession,
                                                    ViInt32   window,
                                                    ViInt32   reserved,
                                                    ViInt32   quantize,
                                                    ViInt32   scaling);

ViStatus _VI_FUNC rsspecan_ConfigureVSADisplayXAxisReference(ViSession instrSession,
                                                             ViInt32   window,
                                                             ViInt32   reserved,
                                                             ViInt32   referenceValue,
                                                             ViInt32   referencePosition);

ViStatus _VI_FUNC rsspecan_ConfigureVSAUnitTime(ViSession instrSession,
                                                ViInt32   window,
                                                ViInt32   unit);

ViStatus _VI_FUNC rsspecan_ConfigureVSAUnitAngle(ViSession instrSession,
                                                 ViInt32   window,
                                                 ViInt32   unitAngle);

ViStatus _VI_FUNC rsspecan_ConfigureVSAYAxisUnitTime(ViSession instrSession,
                                                     ViInt32   window,
                                                     ViInt32   unit);

ViStatus _VI_FUNC rsspecan_AddVSAWindow(ViSession instrSession,
                                        ViInt32   stringSize,
                                        ViString  referenceWindowName,
                                        ViInt32   direction,
                                        ViInt32   windowType,
                                        ViChar    returnedWindowName[]);

ViStatus _VI_FUNC rsspecan_ReplaceVSAWindow(ViSession instrSession,
                                            ViString  windowName,
                                            ViInt32   windowType);

ViStatus _VI_FUNC rsspecan_ConfigureVSAFileImport(ViSession instrSession,
                                                  ViString  fileName);

ViStatus _VI_FUNC rsspecan_ConfigureVSAFileExport(ViSession instrSession,
                                                  ViInt32   trace,
                                                  ViInt32   decimalSeparator,
                                                  ViBoolean header,
                                                  ViInt32   dataMode,
                                                  ViString  fileName);

ViStatus _VI_FUNC rsspecan_ConfigureVSALimitsDefault(ViSession instrSession,
                                                     ViInt32   type);

ViStatus _VI_FUNC rsspecan_ConfigureVSALimitsEVM(ViSession instrSession,
                                                 ViInt32   limitType,
                                                 ViReal64  EVMRMS,
                                                 ViBoolean checkRMS,
                                                 ViReal64  EVMPeak,
                                                 ViBoolean checkPeak);

ViStatus _VI_FUNC rsspecan_ConfigureVSALimitsPhaseError(ViSession instrSession,
                                                        ViInt32   limitType,
                                                        ViReal64  phaseErrorRMS,
                                                        ViBoolean checkRMS,
                                                        ViReal64  phaseErrorPeak,
                                                        ViBoolean checkPeak);

ViStatus _VI_FUNC rsspecan_ConfigureVSALimitsMagnitudeError(ViSession instrSession,
                                                            ViInt32   limitType,
                                                            ViReal64  magnitudeErrorRMS,
                                                            ViBoolean checkRMS,
                                                            ViReal64  magnitudeErrorPeak,
                                                            ViBoolean checkPeak);

ViStatus _VI_FUNC rsspecan_ConfigureVSALimitsCarrierFrequencyError(
                            ViSession instrSession,
                            ViInt32   limitType,
                            ViReal64  carrierFrequencyError,
                            ViBoolean check);

ViStatus _VI_FUNC rsspecan_ConfigureVSALimitsRHO(ViSession instrSession,
                                                 ViInt32   limitType,
                                                 ViReal64  RHO,
                                                 ViBoolean check);

ViStatus _VI_FUNC rsspecan_ConfigureVSALimitsIQOffset(ViSession instrSession,
                                                      ViInt32   limitType,
                                                      ViReal64  IQOffset,
                                                      ViBoolean check);

ViStatus _VI_FUNC rsspecan_ConfigureVSALimitsFrequencyError(ViSession instrSession,
                                                            ViInt32   limitType,
                                                            ViReal64  frequencyErrorRMS,
                                                            ViBoolean checkRMS,
                                                            ViReal64  frequencyErrorPeak,
                                                            ViBoolean checkPeak);

ViStatus _VI_FUNC rsspecan_ConfigureVSALimitsFrequencyDeviationError(
                            ViSession instrSession,
                            ViInt32   limitType,
                            ViReal64  freqDevError,
                            ViBoolean check);

ViStatus _VI_FUNC rsspecan_ConfigureVSAEyeDiagramAbsoluteLimitLine(
                            ViSession instrSession,
                            ViInt32   limitLine,
                            ViBoolean absoluteEnabled,
                            ViReal64  absoluteValue);

ViStatus _VI_FUNC rsspecan_ConfigureVSAEyeDiagramRelativeLimitLine(
                            ViSession instrSession,
                            ViInt32   limitLine,
                            ViBoolean relativeEnabled,
                            ViReal64  relativeValue);

ViStatus _VI_FUNC rsspecan_ConfigureVSAIQLoadStream(ViSession instrSession,
                                                    ViBoolean autoEnabled,
                                                    ViString  fileName);

ViStatus _VI_FUNC rsspecan_QueryVSAIQStreamList(ViSession instrSession,
                                                ViInt32   arraySize,
                                                ViChar    streamList[]);

ViStatus _VI_FUNC rsspecan_ConfigureVSAHighlightSymbols(ViSession instrSession,
                                                        ViInt32   window,
                                                        ViInt32   reserved,
                                                        ViBoolean highlightSymbols);

ViStatus _VI_FUNC rsspecan_ConfigureVSACaptureBuffer(ViSession instrSession,
                                                     ViInt32   window,
                                                     ViInt32   resultType);

ViStatus _VI_FUNC rsspecan_ConfigureVSAMeasurementSignal(ViSession instrSession,
                                                         ViInt32   window,
                                                         ViInt32   resultType);

ViStatus _VI_FUNC rsspecan_ConfigureVSATraceEval(ViSession instrSession,
                                                 ViInt32   trace,
                                                 ViInt32   traceEvaluation);

ViStatus _VI_FUNC rsspecan_ConfigureVSASymbols(ViSession instrSession,
                                               ViInt32   window,
                                               ViInt32   resultType);

ViStatus _VI_FUNC rsspecan_ConfigureVSAErrorVector(ViSession instrSession,
                                                   ViInt32   window,
                                                   ViInt32   resultType);

ViStatus _VI_FUNC rsspecan_ConfigureVSAModulationErrors(ViSession instrSession,
                                                        ViInt32   window,
                                                        ViInt32   resultType);

ViStatus _VI_FUNC rsspecan_ConfigureVSAModulationAccuracy(ViSession instrSession,
                                                          ViInt32   window);

ViStatus _VI_FUNC rsspecan_ConfigureVSAModulationAccuracyAdditional(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   resultType);

ViStatus _VI_FUNC rsspecan_ConfigureVSAResultTypeTransformation(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   transformation);

ViStatus _VI_FUNC rsspecan_ConfigureVSACaptureOversampling(ViSession instrSession,
                                                           ViInt32   window,
                                                           ViInt32   oversampling,
                                                           ViInt32   statisticalMode);

ViStatus _VI_FUNC rsspecan_ConfigureVSADisplayPointPerSymbol(ViSession instrSession,
                                                             ViInt32   window,
                                                             ViInt32   pointsPerSymbol,
                                                             ViInt32   numberOfPointsPerSymbol);

ViStatus _VI_FUNC rsspecan_VSARefreshCapturedData(ViSession instrSession);

ViStatus _VI_FUNC rsspecan_GetVSAXAxisFirstValue(ViSession instrSession,
                                                 ViReal64* result);

ViStatus _VI_FUNC rsspecan_GetVSAXAxisLastValue(ViSession instrSession,
                                                ViReal64* result);

ViStatus _VI_FUNC rsspecan_GetVSAResult(ViSession instrSession,
                                        ViInt32   window,
                                        ViInt32   type,
                                        ViInt32   modifier,
                                        ViReal64* result);

ViStatus _VI_FUNC rsspecan_GetVSAAllResults(ViSession instrSession,
                                            ViInt32   window,
                                            ViReal64  results[]);

ViStatus _VI_FUNC rsspecan_GetVSAResultStart(ViSession instrSession,
                                             ViReal64* result);

ViStatus _VI_FUNC rsspecan_GetVSABurstLength(ViSession instrSession,
                                             ViInt32   window,
                                             ViInt32*  burstLength);

ViStatus _VI_FUNC rsspecan_ReadVSATraceData(ViSession instrSession,
                                            ViInt32   reserved,
                                            ViInt32   sourceTrace,
                                            ViInt32   arraySize,
                                            ViReal64  traceData[],
                                            ViInt32*  noOfValues);

ViStatus _VI_FUNC rsspecan_ReadVSATraceSymbols(ViSession instrSession,
                                               ViInt32   window,
                                               ViInt32   arraySize,
                                               ViInt32   traceData[],
                                               ViInt32*  noOfValues);

ViStatus _VI_FUNC rsspecan_ReadVSATraceSymbolErrors(ViSession instrSession,
                                                    ViInt32   window,
                                                    ViInt32   type,
                                                    ViInt32   arraySize,
                                                    ViBoolean data[],
                                                    ViInt32*  noOfValues);

ViStatus _VI_FUNC rsspecan_QueryVSABERResults(ViSession instrSession,
                                              ViInt32   reserved,
                                              ViInt32   resultType,
                                              ViReal64* bitErrorRate,
                                              ViInt32*  numberOfErrors,
                                              ViInt32*  numberOfBits);

ViStatus _VI_FUNC rsspecan_QueryVSABERSymbolError(ViSession instrSession,
                                                  ViInt32   resultType,
                                                  ViInt32*  symbolError);

ViStatus _VI_FUNC rsspecan_QueryVSAModulationAccuracyLimitsCheckResults(
                            ViSession instrSession,
                            ViInt32   reserved,
                            ViInt32   measurementType,
                            ViInt32   limitType,
                            ViInt32*  checkResults);

ViStatus _VI_FUNC rsspecan_QueryVSAModulationAccuracyStatisticResults(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   reserved,
                            ViInt32   measurementType,
                            ViInt32   statistic,
                            ViReal64* statistcValue);

ViStatus _VI_FUNC rsspecan_QueryVSABurstInformation(ViSession  instrSession,
                                                    ViInt32*   burstCount,
                                                    ViBoolean* burstPresent,
                                                    ViInt32*   burstStart,
                                                    ViReal64*  burstLength);

ViStatus _VI_FUNC rsspecan_QueryVSAPatternInformation(ViSession  instrSession,
                                                      ViInt32*   patternCount,
                                                      ViBoolean* patternPresent,
                                                      ViInt32*   patternStart,
                                                      ViBoolean* patternCorrectness,
                                                      ViReal64*  patternConfidence);

ViStatus _VI_FUNC rsspecan_QueryVSADeltamarkerAbsoluteX(ViSession instrSession,
                                                        ViInt32   window,
                                                        ViInt32   marker,
                                                        ViReal64* absoluteX);

ViStatus _VI_FUNC rsspecan_setVSAModStatusRegister(ViSession instrSession,
                                                   ViInt32   channel,
                                                   ViInt32   registerOperation,
                                                   ViInt32   questionableRegister,
                                                   ViInt32   enable,
                                                   ViInt32   PTransition,
                                                   ViInt32   NTransition);

ViStatus _VI_FUNC rsspecan_getVSAModStatusRegister(ViSession instrSession,
                                                   ViInt32   channel,
                                                   ViInt32   questionableRegister,
                                                   ViInt32   registerPart,
                                                   ViInt32*  registerValue);

ViStatus _VI_FUNC rsspecan_SetOFDMVSAMode(ViSession instrSession);

ViStatus _VI_FUNC rsspecan_UseOFDMVSAConfigurationFile(ViSession instrSession,
                                                       ViBoolean configurationFile);

ViStatus _VI_FUNC rsspecan_LoadOFDMVSAConfigurationFile(ViSession instrSession,
                                                        ViString  configurationFile);

ViStatus _VI_FUNC rsspecan_ConfigureOFDMVSAPhaseCompensationSettings(
                            ViSession instrSession,
                            ViBoolean enabled,
                            ViInt32   mode,
                            ViReal64  frequency);

ViStatus _VI_FUNC rsspecan_ConfigureOFDMVSAPreambleBlockLength(
                            ViSession instrSession,
                            ViInt32   preambleBlockLength);

ViStatus _VI_FUNC rsspecan_ConfigureOFDMVSAFrameStartOffset(ViSession instrSession,
                                                            ViInt32   frameStartOffset);

ViStatus _VI_FUNC rsspecan_ConfigureOFDMVSATransformPrecodingSettings(
                            ViSession instrSession,
                            ViBoolean transformPrecoding,
                            ViBoolean ignore);

ViStatus _VI_FUNC rsspecan_ConfigureOFDMVSAFFTSize(ViSession instrSession,
                                                   ViInt32   FFTSize);

ViStatus _VI_FUNC rsspecan_ConfigureOFDMVSACyclicPrefixMode(ViSession instrSession,
                                                            ViInt32   mode);

ViStatus _VI_FUNC rsspecan_ConfigureOFDMVSACyclicPrefixPeriodicRange(
                            ViSession instrSession,
                            ViBoolean periodicRange);

ViStatus _VI_FUNC rsspecan_ConfigureOFDMVSACyclicPrefixSymbolCount(
                            ViSession instrSession,
                            ViInt32   cyclicPrefix,
                            ViInt32   symbolCount);

ViStatus _VI_FUNC rsspecan_ConfigureOFDMVSACyclicPrefixLength(
                            ViSession instrSession,
                            ViInt32   cyclicPrefix,
                            ViInt32   length);

ViStatus _VI_FUNC rsspecan_ConfigureOFDMVSALowpassFilterEnabled(
                            ViSession instrSession,
                            ViBoolean filter);

ViStatus _VI_FUNC rsspecan_ConfigureOFDMVSALowpassFilterSettings(
                            ViSession instrSession,
                            ViReal64  bandwidth6DB,
                            ViReal64  bandwidth50DB);

ViStatus _VI_FUNC rsspecan_ConfigureOFDMVSAHighpassFilterEnabled(
                            ViSession instrSession,
                            ViBoolean highpassFilter);

ViStatus _VI_FUNC rsspecan_ConfigureOFDMVSAHighpassFilterSettings(
                            ViSession instrSession,
                            ViReal64  bandwidth6DB);

ViStatus _VI_FUNC rsspecan_QueryOFDMVSAHighpassFilter50dBFrequency(
                            ViSession instrSession,
                            ViReal64* bandwidth50DB);

ViStatus _VI_FUNC rsspecan_ConfigureOFDMVSABurstSearchEnabled(
                            ViSession instrSession,
                            ViBoolean burstSearch);

ViStatus _VI_FUNC rsspecan_ConfigureOFDMVSAMaximumFrames(ViSession instrSession,
                                                         ViInt32   maximumFrames,
                                                         ViInt32   resultLength);

ViStatus _VI_FUNC rsspecan_ConfigureOFDMVSASynchronizationTimeMode(
                            ViSession instrSession,
                            ViInt32   timeSynchronization);

ViStatus _VI_FUNC rsspecan_ConfigureOFDMVSASynchronizationParameterEstimation(
                            ViSession instrSession,
                            ViInt32   parameterEstimation);

ViStatus _VI_FUNC rsspecan_ConfigureOFDMVSASynchronizationModulationDetection(
                            ViSession instrSession,
                            ViInt32   modulationDetection);

ViStatus _VI_FUNC rsspecan_ConfigureOFDMVSAThresholds(ViSession instrSession,
                                                      ViReal64  minimumFrameSync,
                                                      ViReal64  minimumTimeSync);

ViStatus _VI_FUNC rsspecan_ConfigureOFDMVSADemodulationFFTShift(
                            ViSession instrSession,
                            ViReal64  FFTShift);

ViStatus _VI_FUNC rsspecan_ConfigureOFDMVSADemodulationMaximumCarrierOffset(
                            ViSession instrSession,
                            ViInt32   maximumCarrierOffset);

ViStatus _VI_FUNC rsspecan_ConfigureOFDMVSADemodulationCyclicDelay(
                            ViSession instrSession,
                            ViInt32   cyclicDelay);

ViStatus _VI_FUNC rsspecan_ConfigureOFDMVSATrackingPhaseEnabled(
                            ViSession instrSession,
                            ViBoolean phaseTracking);

ViStatus _VI_FUNC rsspecan_ConfigureOFDMVSATrackingTimingEnabled(
                            ViSession instrSession,
                            ViBoolean timingTracking);

ViStatus _VI_FUNC rsspecan_ConfigureOFDMVSATrackingLevelEnabled(
                            ViSession instrSession,
                            ViBoolean levelTracking);

ViStatus _VI_FUNC rsspecan_ConfigureOFDMVSATrackingChannelCompensationEnabled(
                            ViSession instrSession,
                            ViBoolean channelCompensation);

ViStatus _VI_FUNC rsspecan_IncludeOFDMVSAConstellationDiagramCarrier(
                            ViSession instrSession,
                            ViInt32   carrier);

ViStatus _VI_FUNC rsspecan_IncludeOFDMVSAConstellationDiagramCarriersAll(ViSession instrSession);

ViStatus _VI_FUNC rsspecan_IncludeOFDMVSAConstellationDiagramSelectedModulation(
                            ViSession instrSession,
                            ViString  selectedModulation);

ViStatus _VI_FUNC rsspecan_IncludeOFDMVSAConstellationDiagramModulationAll(ViSession instrSession);

ViStatus _VI_FUNC rsspecan_ConfigureOFDMVSAConstellationDiagramModulationType(
                            ViSession instrSession,
                            ViInt32   modulationType);

ViStatus _VI_FUNC rsspecan_IncludeOFDMVSAConstellationDiagramSpecifiedSymbolNumber(
                            ViSession instrSession,
                            ViInt32   symbolNumber);

ViStatus _VI_FUNC rsspecan_IncludeOFDMVSAConstellationDiagramSymbolNumberAll(ViSession instrSession);

ViStatus _VI_FUNC rsspecan_ConfigureOFDMVSAEVMAveragingMethod(
                            ViSession instrSession,
                            ViInt32   averagingMethod);

ViStatus _VI_FUNC rsspecan_ConfigureOFDMVSAEVMNormalizationMethod(
                            ViSession instrSession,
                            ViInt32   normalizationMethod);

ViStatus _VI_FUNC rsspecan_ConfigureOFDMVSAUnits(ViSession instrSession,
                                                 ViInt32   EVM,
                                                 ViInt32   symbolAxes,
                                                 ViInt32   carrierAxes,
                                                 ViInt32   magnitudeCapture,
                                                 ViInt32   powerSpectrum,
                                                 ViInt32   impulseResponse);

ViStatus _VI_FUNC rsspecan_OFDMVSAAddWindow(ViSession instrSession,
                                            ViInt32   bufferSize,
                                            ViString  windowName,
                                            ViInt32   direction,
                                            ViInt32   windowType,
                                            ViChar    returnedWindowName[],
                                            ViInt32*  bytesRead);

ViStatus _VI_FUNC rsspecan_OFDMVSAReplaceWindow(ViSession instrSession,
                                                ViString  windowName,
                                                ViInt32   windowType);

ViStatus _VI_FUNC rsspecan_ConfigureOFDMVSABitstreamDisplayFormat(
                            ViSession instrSession,
                            ViInt32   format);

ViStatus _VI_FUNC rsspecan_FetchOFDMVSAAllStates(ViSession instrSession,
                                                 ViInt32   bufferSize,
                                                 ViChar    allStates[],
                                                 ViInt32*  bytesRead);

ViStatus _VI_FUNC rsspecan_FetchOFDMVSABurstDetectionState(ViSession instrSession,
                                                           ViInt32*  burstDetection);

ViStatus _VI_FUNC rsspecan_FetchOFDMVSACompensationState(ViSession instrSession,
                                                         ViInt32*  compensation);

ViStatus _VI_FUNC rsspecan_FetchOFDMVSADataAidedEstimationState(
                            ViSession instrSession,
                            ViInt32*  dataAidedEstimation);

ViStatus _VI_FUNC rsspecan_FetchOFDMVSAPilotAidedEstimationState(
                            ViSession instrSession,
                            ViInt32*  pilotAidedEstimation);

ViStatus _VI_FUNC rsspecan_FetchOFDMVSAEVMMeasurementState(ViSession instrSession,
                                                           ViInt32*  EVMMeasurement);

ViStatus _VI_FUNC rsspecan_FetchOFDMVSAModulationDetectionState(
                            ViSession instrSession,
                            ViInt32*  modulationDetection);

ViStatus _VI_FUNC rsspecan_FetchOFDMVSAFrameSynchronizationResults(
                            ViSession instrSession,
                            ViInt32*  state,
                            ViReal64* frameSynchronization);

ViStatus _VI_FUNC rsspecan_FetchOFDMVSATimeSynchronizationResults(
                            ViSession instrSession,
                            ViInt32*  state,
                            ViReal64* timeSynchronization);

ViStatus _VI_FUNC rsspecan_FetchOFDMVSATriggerToSync(ViSession instrSession,
                                                     ViReal64* triggerToSync);

ViStatus _VI_FUNC rsspecan_FetchOFDMVSABurstCount(ViSession instrSession,
                                                  ViInt32*  burstCount);

ViStatus _VI_FUNC rsspecan_FetchOFDMVSABurstLengths(ViSession instrSession,
                                                    ViInt32   burstNumber,
                                                    ViInt32   burstLengths[],
                                                    ViInt32*  valuesRead);

ViStatus _VI_FUNC rsspecan_FetchOFDMVSABurstOffset(ViSession instrSession,
                                                   ViInt32   burstNumber,
                                                   ViReal64  burstOffsets[],
                                                   ViInt32*  valuesRead);

ViStatus _VI_FUNC rsspecan_FetchOFDMVSAAllResults(ViSession instrSession,
                                                  ViInt32   bufferSize,
                                                  ViChar    results[],
                                                  ViInt32*  bytesRead);

ViStatus _VI_FUNC rsspecan_FetchOFDMVSACrestFactorResults(ViSession instrSession,
                                                          ViReal64* maximum,
                                                          ViReal64* minimum,
                                                          ViReal64* average);

ViStatus _VI_FUNC rsspecan_FetchOFDMVSAEVMDataResult(ViSession instrSession,
                                                     ViReal64* maximum,
                                                     ViReal64* minimum,
                                                     ViReal64* average);

ViStatus _VI_FUNC rsspecan_FetchOFDMVSAEVMPilotResults(ViSession instrSession,
                                                       ViReal64* maximum,
                                                       ViReal64* minimum,
                                                       ViReal64* average);

ViStatus _VI_FUNC rsspecan_FetchOFDMVSAEVMPilotPercentAverage(
                            ViSession instrSession,
                            ViReal64* percentAverage);

ViStatus _VI_FUNC rsspecan_FetchOFDMVSAEVMAllPercentResults(ViSession instrSession,
                                                            ViReal64* maximum,
                                                            ViReal64* minimum,
                                                            ViReal64* average);

ViStatus _VI_FUNC rsspecan_FetchOFDMVSAEVMPeakDataMaximum(ViSession instrSession,
                                                          ViReal64* maximum,
                                                          ViReal64* percentMaximum);

ViStatus _VI_FUNC rsspecan_FetchOFDMVSAEVMPeakPilotMaximum(ViSession instrSession,
                                                           ViReal64* maximum,
                                                           ViReal64* percentMaximum);

ViStatus _VI_FUNC rsspecan_FetchOFDMVSAEVMPeakAllMaximum(ViSession instrSession,
                                                         ViReal64* maximum,
                                                         ViReal64* percentMaximum);

ViStatus _VI_FUNC rsspecan_FetchOFDMVSAFrequencyErrorResults(ViSession instrSession,
                                                             ViReal64* maximum,
                                                             ViReal64* minimum,
                                                             ViReal64* average);

ViStatus _VI_FUNC rsspecan_FetchOFDMVSASamplingErrorResults(ViSession instrSession,
                                                            ViReal64* maximum,
                                                            ViReal64* minimum,
                                                            ViReal64* average);

ViStatus _VI_FUNC rsspecan_FetchOFDMVSAQuadratureErrorResults(
                            ViSession instrSession,
                            ViReal64* maximum,
                            ViReal64* minimum,
                            ViReal64* average);

ViStatus _VI_FUNC rsspecan_FetchOFDMVSAModulationErrorResults(
                            ViSession instrSession,
                            ViReal64* maximum,
                            ViReal64* minimum,
                            ViReal64* average);

ViStatus _VI_FUNC rsspecan_FetchOFDMVSAGainImbalanceResults(ViSession instrSession,
                                                            ViReal64* maximum,
                                                            ViReal64* minimum,
                                                            ViReal64* average);

ViStatus _VI_FUNC rsspecan_FetchOFDMVSAIQOffsetResults(ViSession instrSession,
                                                       ViReal64* maximum,
                                                       ViReal64* minimum,
                                                       ViReal64* average);

ViStatus _VI_FUNC rsspecan_FetchOFDMVSAPowerResults(ViSession instrSession,
                                                    ViReal64* maximum,
                                                    ViReal64* minimum,
                                                    ViReal64* average);

ViStatus _VI_FUNC rsspecan_Set3GPPBSMode(ViSession instrSession);

ViStatus _VI_FUNC rsspecan_Set3GPPUEMode(ViSession instrSession);

ViStatus _VI_FUNC rsspecan_Configure3GPPBSStandard(ViSession instrSession,
                                                   ViInt32   baseStationMode);

ViStatus _VI_FUNC rsspecan_Set3GPPBSMeasurement(ViSession instrSession,
                                                ViInt32   measurement);

ViStatus _VI_FUNC rsspecan_Set3GPPUEMeasurement(ViSession instrSession,
                                                ViInt32   measurement);

ViStatus _VI_FUNC rsspecan_Configure3GPPBSYScaleDivision(ViSession instrSession,
                                                         ViReal64  yScaleDivision);

ViStatus _VI_FUNC rsspecan_SelectWCDMAIQFile(ViSession instrSession,
                                             ViString  fileName,
                                             ViReal64  analysisBW);

ViStatus _VI_FUNC rsspecan_Configure3GPPBSSignalDescription(ViSession instrSession,
                                                            ViBoolean HSDPAUPA,
                                                            ViInt32   compressedMode,
                                                            ViBoolean MIMO);

ViStatus _VI_FUNC rsspecan_Configure3GPPUESignalDescription(ViSession instrSession,
                                                            ViInt32   scramblingCode,
                                                            ViInt32   scramblingType,
                                                            ViBoolean HSDPAUPA);

ViStatus _VI_FUNC rsspecan_Configure3GPPScramblingCode(ViSession instrSession,
                                                       ViInt32   scramblingCode);

ViStatus _VI_FUNC rsspecan_Search3GPPScramblingCode(ViSession instrSession);

ViStatus _VI_FUNC rsspecan_Configure3GPPSignalCapture(ViSession instrSession,
                                                      ViBoolean invertQ,
                                                      ViBoolean RRCFilter,
                                                      ViInt32   sideBand);

ViStatus _VI_FUNC rsspecan_Configure3GPPBSMultiframe(ViSession instrSession,
                                                     ViInt32   captureLength,
                                                     ViInt32   frameToAnalyze);

ViStatus _VI_FUNC rsspecan_Configure3GPPUEFrameMode(ViSession instrSession,
                                                    ViInt32   analysisMode);

ViStatus _VI_FUNC rsspecan_Configure3GPPSynchronization(ViSession instrSession,
                                                        ViInt32   syncType,
                                                        ViInt32   antenna);

ViStatus _VI_FUNC rsspecan_Configure3GPPAntennaSync(ViSession instrSession,
                                                    ViInt32   antenna,
                                                    ViBoolean CPICHMode,
                                                    ViInt32   sCPICHCode,
                                                    ViInt32   sCPICHAntennaPattern);

ViStatus _VI_FUNC rsspecan_Configure3GPPBSMulticarrier(ViSession instrSession,
                                                       ViBoolean multicarrierState,
                                                       ViBoolean autoscale);

ViStatus _VI_FUNC rsspecan_Configure3GPPBSInactiveChannelThreshold(
                            ViSession instrSession,
                            ViReal64  inactiveChannelTreshold);

ViStatus _VI_FUNC rsspecan_Set3GPPBSASequence(ViSession instrSession);

ViStatus _VI_FUNC rsspecan_Configure3GPPQPSKModulationOnly(ViSession instrSession,
                                                           ViBoolean state);

ViStatus _VI_FUNC rsspecan_Configure3GPPBSPredefinedChannelTable(
                            ViSession instrSession,
                            ViBoolean usingPredefinedChannelTable,
                            ViBoolean measSignalCompare,
                            ViInt32   timingOffset);

ViStatus _VI_FUNC rsspecan_CreateWCDMABSChannelTableAccordingToMeasurement(ViSession instrSession);

ViStatus _VI_FUNC rsspecan_Configure3GPPFDDBSChannelTableOperations(
                            ViSession instrSession,
                            ViInt32   operation,
                            ViString  channelTable);

ViStatus _VI_FUNC rsspecan_Configure3GPPFDDBSChannelTableComment(
                            ViSession instrSession,
                            ViString  channelTableComment);

ViStatus _VI_FUNC rsspecan_Configure3GPPFDDBSChannelTableData(
                            ViSession instrSession,
                            ViInt32   arraySize,
                            ViInt32   codeClass[],
                            ViInt32   codeNumber[],
                            ViBoolean TFCI[],
                            ViInt32   timingOffset[],
                            ViInt32   pilotLength[],
                            ViInt32   channelType[],
                            ViBoolean status[],
                            ViReal64  CDPRelative[]);

ViStatus _VI_FUNC rsspecan_Query3GPPFDDBSChannelTableCatalog(ViSession instrSession,
                                                             ViInt32*  numberOfChannelTables,
                                                             ViInt32   bufferSize,
                                                             ViChar    channelTablesList[]);

ViStatus _VI_FUNC rsspecan_Configure3GPPUEPredefinedChannelTable(
                            ViSession instrSession,
                            ViBoolean usingPredefinedChannelTable);

ViStatus _VI_FUNC rsspecan_CreateWCDMAUEChannelTableAccordingToMeasurement(ViSession instrSession);

ViStatus _VI_FUNC rsspecan_Configure3GPPFDDUEChannelTableOperations(
                            ViSession instrSession,
                            ViInt32   operation,
                            ViString  channelTable);

ViStatus _VI_FUNC rsspecan_Configure3GPPFDDUEChannelTableComment(
                            ViSession instrSession,
                            ViString  channelTableComment);

ViStatus _VI_FUNC rsspecan_Configure3GPPUEChannelTableData(ViSession instrSession,
                                                           ViInt32   arraySize,
                                                           ViInt32   codeClass[],
                                                           ViInt32   noActiveChannels[],
                                                           ViInt32   pilotLength[]);

ViStatus _VI_FUNC rsspecan_Configure3GPPUEChannelTableDataEDPC(
                            ViSession instrSession,
                            ViInt32   arraySize,
                            ViInt32   codeClass[],
                            ViInt32   noActiveChannels[]);

ViStatus _VI_FUNC rsspecan_Configure3GPPUEChannelTableEDPCEntry(
                            ViSession instrSession,
                            ViBoolean state);

ViStatus _VI_FUNC rsspecan_Configure3GPPFDDUEChannelTableHighSpeedChannel(
                            ViSession instrSession,
                            ViBoolean state);

ViStatus _VI_FUNC rsspecan_Query3GPPFDDUEChannelTableCatalog(ViSession instrSession,
                                                             ViInt32*  numberOfChannelTables,
                                                             ViInt32   bufferSize,
                                                             ViChar    channelTablesList[]);

ViStatus _VI_FUNC rsspecan_WCDMATAELoadDefaultCarrierTable(ViSession instrSession,
                                                           ViString  carrierTable);

ViStatus _VI_FUNC rsspecan_WCDMATAECarrierTableOperations(ViSession instrSession,
                                                          ViInt32   operation,
                                                          ViString  carrierTable);

ViStatus _VI_FUNC rsspecan_GetWCDMATAECarrierTableCatalog(ViSession instrSession,
                                                          ViInt32   bufferSize,
                                                          ViChar    carrierTablesList[]);

ViStatus _VI_FUNC rsspecan_GetWCDMATAENumberOfCarriers(ViSession instrSession,
                                                       ViInt32*  numberOfCarriers);

ViStatus _VI_FUNC rsspecan_WCDMATAECarrierOperations(ViSession instrSession,
                                                     ViInt32   carrier,
                                                     ViInt32   operation);

ViStatus _VI_FUNC rsspecan_ConfigureWCDMATAECarrierTable(ViSession instrSession,
                                                         ViInt32   carrier,
                                                         ViReal64  frequencyOffset,
                                                         ViInt32   scramblingCode,
                                                         ViInt32   antenna,
                                                         ViInt32   CPICHNumber,
                                                         ViInt32   CPICHPattern);

ViStatus _VI_FUNC rsspecan_Configure3GPPBSResults(ViSession instrSession,
                                                  ViInt32   channel,
                                                  ViInt32   CPICHSlot,
                                                  ViInt32   spreadingFactor);

ViStatus _VI_FUNC rsspecan_Configure3GPPUEBranch(ViSession instrSession,
                                                 ViInt32   selectIQ);

ViStatus _VI_FUNC rsspecan_Configure3GPPBSCodeDomain(ViSession instrSession,
                                                     ViBoolean compensateIQOffset,
                                                     ViInt32   codePowerDisplay,
                                                     ViInt32   powerReference,
                                                     ViBoolean powerDifference,
                                                     ViInt32   constellationParameterB);

ViStatus _VI_FUNC rsspecan_Configure3GPPUECodeDomain(ViSession instrSession,
                                                     ViBoolean measurementInterval,
                                                     ViBoolean compensateIQOffset,
                                                     ViBoolean eliminateTailChips,
                                                     ViInt32   codePowerDisplay);

ViStatus _VI_FUNC rsspecan_Configure3GPPBSResultType(ViSession instrSession,
                                                     ViInt32   resultType);

ViStatus _VI_FUNC rsspecan_Configure3GPPBSResultSummaryView(ViSession instrSession,
                                                            ViInt32   view);

ViStatus _VI_FUNC rsspecan_Set3GPPBSMarkerToChannel(ViSession instrSession,
                                                    ViInt32   window,
                                                    ViInt32   marker,
                                                    ViInt32   channel);

ViStatus _VI_FUNC rsspecan_Set3GPPBSDeltaMarkerToChannel(ViSession instrSession,
                                                         ViInt32   window,
                                                         ViInt32   marker,
                                                         ViInt32   channel);

ViStatus _VI_FUNC rsspecan_Get3GPPBSCDPScramblingCodeResult(ViSession instrSession,
                                                            ViInt32   arraySize,
                                                            ViInt32   scrambingCode[],
                                                            ViReal64  power[],
                                                            ViInt32*  returnedValues);

ViStatus _VI_FUNC rsspecan_Get3GPPTrace(ViSession instrSession,
                                        ViInt32   window,
                                        ViInt32   type,
                                        ViInt32   trace,
                                        ViInt32   arrayLength,
                                        ViInt32*  actualPoints,
                                        ViReal64  values[]);

ViStatus _VI_FUNC rsspecan_Get3GPPTraceAdditional(ViSession instrSession,
                                                  ViInt32   window,
                                                  ViInt32   type,
                                                  ViInt32   arrayLength,
                                                  ViInt32*  actualPoints,
                                                  ViReal64  values[]);

ViStatus _VI_FUNC rsspecan_Get3GPPBSResult(ViSession instrSession,
                                           ViInt32   type,
                                           ViReal64* result);

ViStatus _VI_FUNC rsspecan_Get3GPPBSPowerResult(ViSession instrSession,
                                                ViInt32   marker,
                                                ViInt32   type,
                                                ViInt32   arraySize,
                                                ViReal64  results[],
                                                ViInt32*  returnedValues);

ViStatus _VI_FUNC rsspecan_Get3GPPBSCDPListEvaluationResult(ViSession instrSession,
                                                            ViInt32   noOfValues,
                                                            ViReal64  rangeNumber[],
                                                            ViReal64  startFrequency[],
                                                            ViReal64  stopFrequency[],
                                                            ViReal64  resolutionBandwidth[],
                                                            ViReal64  frequency[],
                                                            ViReal64  powerAbs[],
                                                            ViReal64  powerRel[],
                                                            ViReal64  delta[],
                                                            ViInt32   limitCheck[],
                                                            ViReal64  reserved1[],
                                                            ViReal64  reserved2[],
                                                            ViInt32*  returnedValues);

ViStatus _VI_FUNC rsspecan_Get3GPPBSTimeAlignmentErrorResult(ViSession instrSession,
                                                             ViInt32   type,
                                                             ViReal64* result);

ViStatus _VI_FUNC rsspecan_Get3GPPUEResult(ViSession instrSession,
                                           ViInt32   type,
                                           ViReal64* result);

ViStatus _VI_FUNC rsspecan_ExportWCDMAMarkerPeakListToFile(ViSession instrSession,
                                                           ViString  fileName);

ViStatus _VI_FUNC rsspecan_SetTDSBSMode(ViSession instrSession);

ViStatus _VI_FUNC rsspecan_SetTDSUEMode(ViSession instrSession);

ViStatus _VI_FUNC rsspecan_SelectTDSMeasurement(ViSession instrSession,
                                                ViInt32   measurement);

ViStatus _VI_FUNC rsspecan_ConfigureTDSSignalCapture(ViSession instrSession,
                                                     ViBoolean invertQ,
                                                     ViInt32   setCount,
                                                     ViInt32   setValue,
                                                     ViInt32   captureLength);

ViStatus _VI_FUNC rsspecan_ConfigureTDSYAxisScalingMaxMin(ViSession instrSession,
                                                          ViReal64  maximumValue,
                                                          ViReal64  minimumValue);

ViStatus _VI_FUNC rsspecan_ConfigureTDSBSPowerVsTime(ViSession instrSession,
                                                     ViInt32   switchingPoint,
                                                     ViInt32   subframes);

ViStatus _VI_FUNC rsspecan_ConfigureTDSBSAdaptToSignal(ViSession instrSession,
                                                       ViInt32   startSlot,
                                                       ViInt32   stopSlot);

ViStatus _VI_FUNC rsspecan_TDSAdaptSignalAutoLevelTime(ViSession instrSession,
                                                       ViInt32   timeout);

ViStatus _VI_FUNC rsspecan_ConfigureTDSSynchronization(ViSession instrSession,
                                                       ViInt32   scramblingCode,
                                                       ViInt32   midambleShiftCells,
                                                       ViInt32   syncTo,
                                                       ViBoolean rotateCodeChannels);

ViStatus _VI_FUNC rsspecan_ConfigureTDSUESyncToSlotMode(ViSession instrSession,
                                                        ViInt32   syncToSlotMode);

ViStatus _VI_FUNC rsspecan_ConfigureTDSUESyncPTSCode(ViSession instrSession,
                                                     ViInt32   code);

ViStatus _VI_FUNC rsspecan_ConfigureTDSTimeReference(ViSession instrSession,
                                                     ViInt32   timeReference);

ViStatus _VI_FUNC rsspecan_ConfigureTDSInactiveChannelThreshold(
                            ViSession instrSession,
                            ViReal64  inactiveChannelTreshold);

ViStatus _VI_FUNC rsspecan_TDSBSChannelTableOperations(ViSession instrSession,
                                                       ViInt32   operation,
                                                       ViString  channelTable);

ViStatus _VI_FUNC rsspecan_ConfigureTDSBSChannelTableComment(ViSession instrSession,
                                                             ViString  channelTableComment);

ViStatus _VI_FUNC rsspecan_ConfigureTDSBSChannelTableData(ViSession instrSession,
                                                          ViInt32   arraySize,
                                                          ViInt32   channelType[],
                                                          ViInt32   codeClass[],
                                                          ViInt32   codeNumber[],
                                                          ViInt32   modulationType[],
                                                          ViInt32   midambleShift[],
                                                          ViBoolean activeFlag[],
                                                          ViInt32   reserved1[],
                                                          ViInt32   reserved2[]);

ViStatus _VI_FUNC rsspecan_ConfigureTDSBSChannelTableOrder(ViSession instrSession,
                                                           ViInt32   order);

ViStatus _VI_FUNC rsspecan_ConfigureTDSBSChannelTableMidambleShift(
                            ViSession instrSession,
                            ViInt32   maximumOfMidambleShifts);

ViStatus _VI_FUNC rsspecan_ConfigureTDSBSChannelTableMaxModulation(
                            ViSession instrSession,
                            ViInt32   highestModulation);

ViStatus _VI_FUNC rsspecan_GetTDSBSChannelTableCatalog(ViSession instrSession,
                                                       ViInt32*  numberOfChannelTables,
                                                       ViInt32   bufferSize,
                                                       ViChar    channelTablesList[]);

ViStatus _VI_FUNC rsspecan_ConfigureTDSEvalRange(ViSession instrSession,
                                                 ViInt32   channel,
                                                 ViInt32   slot,
                                                 ViInt32   setToAnalyze);

ViStatus _VI_FUNC rsspecan_ConfigureTDSCodeDomainSettings(ViSession instrSession,
                                                          ViBoolean normalize,
                                                          ViInt32   codePowerDisplay);

ViStatus _VI_FUNC rsspecan_ConfigureTDSBSResults(ViSession instrSession,
                                                 ViInt32   resultType);

ViStatus _VI_FUNC rsspecan_ConfigureTDSPTS(ViSession instrSession,
                                           ViBoolean showResults);

ViStatus _VI_FUNC rsspecan_GetTDSBSPowerVsTimeResult(ViSession instrSession,
                                                     ViInt32   arraySize,
                                                     ViInt32   timeout,
                                                     ViReal64  results[],
                                                     ViInt32*  returnedValues);

ViStatus _VI_FUNC rsspecan_GetTDSResult(ViSession instrSession,
                                        ViInt32   type,
                                        ViUInt32  timeout,
                                        ViReal64* result);

ViStatus _VI_FUNC rsspecan_FetchTDSTrace(ViSession instrSession,
                                         ViInt32   trace,
                                         ViInt32   arrayLength,
                                         ViInt32*  actualPoints,
                                         ViReal64  values[]);

ViStatus _VI_FUNC rsspecan_AdjustC2KSettings(ViSession instrSession,
                                             ViInt32   reserved,
                                             ViInt32   adjustSettings,
                                             ViUInt32  timeout);

ViStatus _VI_FUNC rsspecan_AdjustC2KReferenceLevel(ViSession instrSession,
                                                   ViInt32   reserved,
                                                   ViUInt32  timeout);

ViStatus _VI_FUNC rsspecan_RestoreC2KSEMStandardLines(ViSession instrSession);

ViStatus _VI_FUNC rsspecan_ConfigureC2KSignalStatisticMeasurement(
                            ViSession instrSession,
                            ViInt32   measurement,
                            ViInt32   numberOfSamples);

ViStatus _VI_FUNC rsspecan_ConfigureC2KSignalStatisticsPercentMarker(
                            ViSession instrSession,
                            ViInt32   reserved,
                            ViInt32   marker,
                            ViReal64  percentMarker);

ViStatus _VI_FUNC rsspecan_ConfigureC2KChannelTableOperations(
                            ViSession instrSession,
                            ViInt32   operation,
                            ViString  channelTable);

ViStatus _VI_FUNC rsspecan_RestoreC2KChannelTable(ViSession instrSession);

ViStatus _VI_FUNC rsspecan_ConfigureC2KBasicSettings(ViSession instrSession,
                                                     ViInt32   bandClass,
                                                     ViBoolean timePhase,
                                                     ViReal64  inactChannelThreshold,
                                                     ViBoolean invertQ,
                                                     ViInt32   sideBand,
                                                     ViBoolean normalize);

ViStatus _VI_FUNC rsspecan_ConfigureC2KPowerDisplay(ViSession instrSession,
                                                    ViInt32   mode);

ViStatus _VI_FUNC rsspecan_ConfigureC2KAmplitudeYScaleDivision(
                            ViSession instrSession,
                            ViReal64  yScaleDiv);

ViStatus _VI_FUNC rsspecan_AdjustC2KAmplitudeReferenceLevel(ViSession instrSession,
                                                            ViUInt32  timeout);

ViStatus _VI_FUNC rsspecan_SelectC2KMeasurement(ViSession instrSession,
                                                ViInt32   measurementSelect);

ViStatus _VI_FUNC rsspecan_ConfigureC2KPCGSettings(ViSession instrSession,
                                                   ViInt32   setCount,
                                                   ViInt32   setToAnalyze);

ViStatus _VI_FUNC rsspecan_ConfigureC2KChannelTableComment(ViSession instrSession,
                                                           ViString  channelTableComment);

ViStatus _VI_FUNC rsspecan_QueryC2KChannelTableCatalog(ViSession instrSession,
                                                       ViInt32*  numberOfChannelTables,
                                                       ViInt32   bufferSize,
                                                       ViChar    channelTablesList[]);

ViStatus _VI_FUNC rsspecan_ReadC2KTraceData(ViSession instrSession,
                                            ViInt32   reserved,
                                            ViInt32   sourceTrace,
                                            ViInt32   arrayLength,
                                            ViInt32*  actualPoints,
                                            ViReal64  values[]);

ViStatus _VI_FUNC rsspecan_ReadC2KCDPTraceData(ViSession instrSession,
                                               ViInt32   reserved,
                                               ViInt32   sourceTrace,
                                               ViInt32   arrayLength,
                                               ViInt32*  actualPoints,
                                               ViReal64  values[]);

ViStatus _VI_FUNC rsspecan_BC2KMode(ViSession instrSession);

ViStatus _VI_FUNC rsspecan_ConfigureBC2KChannelTableData(ViSession instrSession,
                                                         ViInt32   channelType[],
                                                         ViInt32   codeClass[],
                                                         ViInt32   codeNumber[],
                                                         ViInt32   radioConfiguration[],
                                                         ViInt32   reserved1[],
                                                         ViInt32   reserved2[],
                                                         ViBoolean status[],
                                                         ViReal64  CDPRelative[],
                                                         ViInt32   numOfElements);

ViStatus _VI_FUNC rsspecan_ConfigureBC2KAdvancedSettings(ViSession instrSession,
                                                         ViInt32   PNOffset,
                                                         ViInt32   baseSpreadingFactor,
                                                         ViInt32   powerReference,
                                                         ViInt32   order);

ViStatus _VI_FUNC rsspecan_ConfigureBC2KAntennaSettings(ViSession instrSession,
                                                        ViBoolean antennaDiversity,
                                                        ViInt32   antennaNumber);

ViStatus _VI_FUNC rsspecan_ConfigureBC2KMultiCarrier(ViSession instrSession,
                                                     ViBoolean multiCarrier,
                                                     ViBoolean enhancedAlgorithm,
                                                     ViBoolean MCFilter,
                                                     ViInt32   filterType,
                                                     ViReal64  rollOff,
                                                     ViReal64  cutOffFrequency);

ViStatus _VI_FUNC rsspecan_ConfigureBC2KPichMarkerSettings(ViSession instrSession);

ViStatus _VI_FUNC rsspecan_ConfigureBC2KTDPichMarkerSettings(ViSession instrSession);

ViStatus _VI_FUNC rsspecan_ConfigureBC2KCodeDomainAnalyzerResults(
                            ViSession instrSession,
                            ViInt32   resultType);

ViStatus _VI_FUNC rsspecan_ConfigureBC2KCodeDomainAnalyzerSettings(
                            ViSession instrSession,
                            ViInt32   captureLength,
                            ViInt32   selectChannel,
                            ViInt32   selectPCG);

ViStatus _VI_FUNC rsspecan_QueryBC2KCodeDomainAnalyzerResultSummary(
                            ViSession instrSession,
                            ViInt32   reserved,
                            ViInt32   resultType,
                            ViReal64* result);

ViStatus _VI_FUNC rsspecan_MC2KMode(ViSession instrSession);

ViStatus _VI_FUNC rsspecan_ConfigureMC2KChannelTableData(ViSession instrSession,
                                                         ViInt32   channelType[],
                                                         ViInt32   codeClass[],
                                                         ViInt32   codeNumber[],
                                                         ViInt32   mapping[],
                                                         ViInt32   reserved1[],
                                                         ViInt32   reserved2[],
                                                         ViBoolean status[],
                                                         ViReal64  CDPRelative[],
                                                         ViInt32   numOfElements);

ViStatus _VI_FUNC rsspecan_ConfMC2KAdvancedSettings(ViSession instrSession,
                                                    ViInt32   selectIQ,
                                                    ViInt32   longCodeMode,
                                                    ViString  longCodeMask,
                                                    ViString  longCodeOffset);

ViStatus _VI_FUNC rsspecan_ConfigureMC2KCDPOverview(ViSession instrSession,
                                                    ViBoolean codeDomainOverview);

ViStatus _VI_FUNC rsspecan_ConfigureMC2KCodeDomainAnalyzerResults(
                            ViSession instrSession,
                            ViInt32   resultType);

ViStatus _VI_FUNC rsspecan_ConfMC2KCodeDomainAnalyzerSettings(
                            ViSession instrSession,
                            ViInt32   selectPCG,
                            ViInt32   selectMapping);

ViStatus _VI_FUNC rsspecan_QueryMC2KCodeDomainAnalyzerResultSummary(
                            ViSession instrSession,
                            ViInt32   reserved,
                            ViInt32   resultType,
                            ViReal64* result);

ViStatus _VI_FUNC rsspecan_BDOMode(ViSession instrSession);

ViStatus _VI_FUNC rsspecan_ConfigureBDOPowerVsTime(ViSession instrSession,
                                                   ViInt32   numberOfHalfslots,
                                                   ViInt32   referenceSlot,
                                                   ViBoolean burstFit,
                                                   ViInt32   referenceMeanPower,
                                                   ViReal64  referenceValue,
                                                   ViBoolean restartOnFail);

ViStatus _VI_FUNC rsspecan_ConfigureBDOPowerVsTimeListEvaluation(
                            ViSession instrSession,
                            ViBoolean state);

ViStatus _VI_FUNC rsspecan_RestoreBDOPowerVsTimeStandardLines(ViSession instrSession);

ViStatus _VI_FUNC rsspecan_ConfigureBDOChannelTableData(ViSession instrSession,
                                                        ViInt32   channelType[],
                                                        ViInt32   codeClass[],
                                                        ViInt32   codeNumber[],
                                                        ViInt32   modulation[],
                                                        ViInt32   reserved1[],
                                                        ViInt32   reserved2[],
                                                        ViBoolean channelStatus[],
                                                        ViReal64  CDPRelative[],
                                                        ViInt32   numOfElements);

ViStatus _VI_FUNC rsspecan_ConfigureBDOAdvancedSettings(ViSession instrSession,
                                                        ViInt32   PNOffset,
                                                        ViBoolean CDPAverage,
                                                        ViInt32   revision);

ViStatus _VI_FUNC rsspecan_ConfigureBDOMultiCarrier(ViSession instrSession,
                                                    ViBoolean multiCarrier,
                                                    ViBoolean enhancedAlgorithm,
                                                    ViBoolean MCFilter,
                                                    ViInt32   filterType,
                                                    ViReal64  rollOff,
                                                    ViReal64  cutOffFrequency);

ViStatus _VI_FUNC rsspecan_ConfigureBDOCodeDomainAnalyzerResults(
                            ViSession instrSession,
                            ViInt32   resultType);

ViStatus _VI_FUNC rsspecan_ConfBDOCodeDomainAnalyzerSettings(ViSession instrSession,
                                                             ViInt32   captureLength,
                                                             ViInt32   selectSlot);

ViStatus _VI_FUNC rsspecan_ConfigureBDOChannelType(ViSession instrSession,
                                                   ViInt32   channelType,
                                                   ViInt32   mapping,
                                                   ViInt32   select);

ViStatus _VI_FUNC rsspecan_ConfigureEVDOBSTPowerReference(ViSession instrSession,
                                                          ViInt32   powerReference);

ViStatus _VI_FUNC rsspecan_QueryBDOCodeDomainAnalyzerGeneralResults(
                            ViSession instrSession,
                            ViInt32   reserved,
                            ViInt32   resultType,
                            ViReal64* result);

ViStatus _VI_FUNC rsspecan_QueryBDOCodeDomainAnalyzerChannelResults(
                            ViSession instrSession,
                            ViInt32   reserved,
                            ViInt32   resultType,
                            ViReal64* result);

ViStatus _VI_FUNC rsspecan_ReadBDOPowerVsTimeListEvaluation(ViSession instrSession,
                                                            ViInt32   noOfValues,
                                                            ViReal64  rangeNumber[],
                                                            ViReal64  startTime[],
                                                            ViReal64  stopTime[],
                                                            ViReal64  avgPowerAbs[],
                                                            ViReal64  avgPowerRel[],
                                                            ViReal64  maxPowerAbs[],
                                                            ViReal64  maxPowerRel[],
                                                            ViReal64  minPowerAbs[],
                                                            ViReal64  minPowerRel[],
                                                            ViReal64  limitCheck[],
                                                            ViReal64  reserved1[],
                                                            ViReal64  reserved2[],
                                                            ViInt32*  returnedValues);

ViStatus _VI_FUNC rsspecan_MDOMode(ViSession instrSession);

ViStatus _VI_FUNC rsspecan_ConfigureMDOChannelTableData(ViSession instrSession,
                                                        ViInt32   channelType[],
                                                        ViInt32   codeClass[],
                                                        ViInt32   codeNumber[],
                                                        ViInt32   mapping[],
                                                        ViInt32   activity[],
                                                        ViInt32   reserved1[],
                                                        ViBoolean channelStatus[],
                                                        ViInt32   reserved2[],
                                                        ViInt32   numOfElements);

ViStatus _VI_FUNC rsspecan_ConfMDOAdvancedSettings(ViSession instrSession,
                                                   ViBoolean CDPAverage,
                                                   ViInt32   selectIQ);

ViStatus _VI_FUNC rsspecan_ConfigureMDOCDPOverview(ViSession instrSession,
                                                   ViBoolean codeDomainOverview);

ViStatus _VI_FUNC rsspecan_ConfigureMDOCodeDomainAnalyzerResults(
                            ViSession instrSession,
                            ViInt32   resultType);

ViStatus _VI_FUNC rsspecan_ConfigureMDOCodeDomainAnalyzerSettings(
                            ViSession instrSession,
                            ViInt32   captureLength,
                            ViInt32   selectChannel,
                            ViInt32   selectHalfslot,
                            ViInt32   select);

ViStatus _VI_FUNC rsspecan_QueryMDOCodeDomainAnalyzerResultSummary(
                            ViSession instrSession,
                            ViInt32   reserved,
                            ViInt32   resultType,
                            ViReal64* result);

ViStatus _VI_FUNC rsspecan_WlanSetWlanMode(ViSession instrSession);

ViStatus _VI_FUNC rsspecan_ConfigureWLANSignal(ViSession instrSession,
                                               ViInt32   standard,
                                               ViReal64  centerFrequency);

ViStatus _VI_FUNC rsspecan_ConfigureWLANToleranceLimit(ViSession instrSession,
                                                       ViInt32   toleranceLimit);

ViStatus _VI_FUNC rsspecan_ConfigureWLANStandardVersionErrorVectorMagnitude(
                            ViSession instrSession,
                            ViInt32   standardVersion);

ViStatus _VI_FUNC rsspecan_ConfigureWLANInputSource(ViSession instrSession,
                                                    ViInt32   inputCoupling,
                                                    ViInt32   impedance,
                                                    ViBoolean highpassFilter,
                                                    ViBoolean YIGPreselector);

ViStatus _VI_FUNC rsspecan_ConfigureWlanUseSignalFieldContent(
                            ViSession instrSession,
                            ViBoolean useSignalFieldContent);

ViStatus _VI_FUNC rsspecan_ConfWlanTrigger(ViSession instrSession,
                                           ViInt32   mode,
                                           ViReal64  offset,
                                           ViBoolean autoTriggerLevel,
                                           ViReal64  powerTriggerLevel,
                                           ViReal64  extTriggerLevel);

ViStatus _VI_FUNC rsspecan_ConfWlanGate(ViSession instrSession,
                                        ViBoolean gating,
                                        ViInt32   mode,
                                        ViReal64  delay,
                                        ViReal64  length,
                                        ViBoolean linkGateAndMarker);

ViStatus _VI_FUNC rsspecan_ConfigureWlanChannel(ViSession instrSession,
                                                ViInt32   channelNo);

ViStatus _VI_FUNC rsspecan_ConfWlanAutoLevelOnce(ViSession instrSession);

ViStatus _VI_FUNC rsspecan_ConfWLANReferenceLevel(ViSession instrSession,
                                                  ViBoolean autoLevel,
                                                  ViReal64  referenceLevel,
                                                  ViReal64  referenceLevelOffset,
                                                  ViReal64  signalLevel_RMS);

ViStatus _VI_FUNC rsspecan_ConfigureWLANReferenceLevelDuration(
                            ViSession instrSession,
                            ViInt32   mode,
                            ViReal64  duration);

ViStatus _VI_FUNC rsspecan_ConfigureWLANMechanicalAttenuation(
                            ViSession instrSession,
                            ViBoolean autoMode,
                            ViReal64  value);

ViStatus _VI_FUNC rsspecan_ConfigureWlanElectronicAtt(ViSession instrSession,
                                                      ViBoolean state,
                                                      ViBoolean electronicAttenuatorAuto,
                                                      ViInt32   manualValuedB);

ViStatus _VI_FUNC rsspecan_ConfigureWLANPreamplifier(ViSession instrSession,
                                                     ViBoolean state,
                                                     ViInt32   preamplifierLevel);

ViStatus _VI_FUNC rsspecan_ConfigureWLANSignalCapture(ViSession instrSession,
                                                      ViReal64  sampleRate,
                                                      ViReal64  captureTime,
                                                      ViBoolean swapIQ,
                                                      ViBoolean filterOutAdjacentChannels);

ViStatus _VI_FUNC rsspecan_ConfigureWLANSynchronization(ViSession instrSession,
                                                        ViBoolean powerIntervalSearch,
                                                        ViInt32   FFTStartOffset);

ViStatus _VI_FUNC rsspecan_ConfigureWLANChannelEstimation(ViSession instrSession,
                                                          ViBoolean channelEstimationRange);

ViStatus _VI_FUNC rsspecan_ConfigureWLANChannelBandwidthAuto(ViSession instrSession,
                                                             ViBoolean channelBandwidthAuto);

ViStatus _VI_FUNC rsspecan_ConfigureWLANPreambleChannelEstimation(
                            ViSession instrSession,
                            ViInt32   preambleChannelEstimation);

ViStatus _VI_FUNC rsspecan_ConfigureWlanTracking(ViSession instrSession,
                                                 ViBoolean phase,
                                                 ViBoolean timing,
                                                 ViBoolean level);

ViStatus _VI_FUNC rsspecan_ConfigureWLANTrackingPilots(ViSession instrSession,
                                                       ViInt32   pilots);

ViStatus _VI_FUNC rsspecan_ConfigureWlanTrackingWienerRelativeDelaySpread(
                            ViSession instrSession,
                            ViInt32   enabled,
                            ViReal64  spread);

ViStatus _VI_FUNC rsspecan_ConfigureWlanIQCompensation(ViSession instrSession,
                                                       ViBoolean IQCompensation);

ViStatus _VI_FUNC rsspecan_ConfigureWLANInterpolationType(ViSession instrSession,
                                                          ViInt32   interpolationType);

ViStatus _VI_FUNC rsspecan_ConfigureWLANCompensateCrosstalk(ViSession instrSession,
                                                            ViBoolean compensateCrosstalk);

ViStatus _VI_FUNC rsspecan_ConfigureWlanBurst(ViSession instrSession,
                                              ViInt32   burst,
                                              ViBoolean state);

ViStatus _VI_FUNC rsspecan_ConfigureWlanBurstUnit(ViSession instrSession,
                                                  ViInt32   unit);

ViStatus _VI_FUNC rsspecan_ConfigureWlanPowerOptimisation(ViSession instrSession,
                                                          ViInt32   optimisation);

ViStatus _VI_FUNC rsspecan_ConfWlanMarkPosition(ViSession instrSession,
                                                ViInt32   marker,
                                                ViInt32   position,
                                                ViReal64  positionValue);

ViStatus _VI_FUNC rsspecan_ConfWlanMarkerPositionConstellation(
                            ViSession instrSession,
                            ViInt32   marker,
                            ViInt32   type,
                            ViInt32   position);

ViStatus _VI_FUNC rsspecan_ConfWlanMarkerBurstSymbol(ViSession instrSession,
                                                     ViInt32   marker,
                                                     ViInt32   burstNumber,
                                                     ViInt32   symbolNumber);

ViStatus _VI_FUNC rsspecan_ConfigureWLANDemodulation802_11a(ViSession instrSession,
                                                            ViBoolean PPDUAnalysisMode,
                                                            ViInt32   PPDUFormatToMeasure,
                                                            ViInt32   channelBandwidthToMeasure,
                                                            ViInt32   PSDUModulationToUse,
                                                            ViInt32   PSDUModulation);

ViStatus _VI_FUNC rsspecan_ConfigureWLANDemodulation802_11n(ViSession instrSession,
                                                            ViBoolean PPDUAnalysisMode,
                                                            ViInt32   PPDUFormatToMeasure,
                                                            ViInt32   channelBandwidthToMeasure);

ViStatus _VI_FUNC rsspecan_ConfigureWLANDemodulation802_11nMCSIndex(
                            ViSession instrSession,
                            ViInt32   MCSIndexMode,
                            ViInt32   MCSIndex);

ViStatus _VI_FUNC rsspecan_ConfigureWLANDemodulation802_11nSTBCField(
                            ViSession instrSession,
                            ViInt32   STBCField);

ViStatus _VI_FUNC rsspecan_ConfigureWLANDemodulation802_11nGuardIntervalLength(
                            ViSession instrSession,
                            ViInt32   guardIntervalLength);

ViStatus _VI_FUNC rsspecan_ConfigureWLANDemodulation802_11nGuardInterval(
                            ViSession instrSession,
                            ViBoolean autoEnabled,
                            ViInt32   manualValue);

ViStatus _VI_FUNC rsspecan_ConfigureWLANDemodulation802_11nExtensionSpatialStreams(
                            ViSession instrSession,
                            ViInt32   extensionSpatialStreams);

ViStatus _VI_FUNC rsspecan_ConfigureWLANDemodulation802_11nData(
                            ViSession instrSession,
                            ViInt32   mode,
                            ViInt32   bitRate);

ViStatus _VI_FUNC rsspecan_ConfigureWLANDemodulation802_11b_g_DSSS(
                            ViSession instrSession,
                            ViInt32   PPDUFormatPSDUModulation,
                            ViInt32   PPDUFormat,
                            ViInt32   PSDUModulation);

ViStatus _VI_FUNC rsspecan_ConfigureWlan802_11b_g_PayloadLength(
                            ViSession instrSession,
                            ViInt32   lengthType,
                            ViBoolean equalBurstLength,
                            ViInt32   minPayloadLength,
                            ViInt32   maxPayloadLength);

ViStatus _VI_FUNC rsspecan_ConfWlanSelectFilter(ViSession instrSession,
                                                ViString  TXFilterName,
                                                ViString  RXFilterName);

ViStatus _VI_FUNC rsspecan_ConfWLANEqualizerFilterLength(ViSession instrSession,
                                                         ViInt32   filterLength);

ViStatus _VI_FUNC rsspecan_QueryWlanFilterCatalog(ViSession instrSession,
                                                  ViInt32   bufferSize,
                                                  ViChar    filterList[],
                                                  ViInt32*  returnedLength);

ViStatus _VI_FUNC rsspecan_ConfigureWLANDemodulation802_11ac(ViSession instrSession,
                                                             ViBoolean PPDUAnalysisMode,
                                                             ViInt32   PPDUFormatToMeasure,
                                                             ViInt32   channelBandwidthToMeasure);

ViStatus _VI_FUNC rsspecan_ConfigureWLANDemodulation802_11acMCSIndex(
                            ViSession instrSession,
                            ViInt32   MCSIndexMode,
                            ViInt32   MCSIndex);

ViStatus _VI_FUNC rsspecan_ConfigureWLANDemodulation802_11acSTBCField(
                            ViSession instrSession,
                            ViInt32   STBCField);

ViStatus _VI_FUNC rsspecan_ConfigureWLANDemodulation802_11acGuardIntervalLength(
                            ViSession instrSession,
                            ViInt32   guardIntervalLength);

ViStatus _VI_FUNC rsspecan_ConfigureWLANDemodulation802_11acGuardInterval(
                            ViSession instrSession,
                            ViBoolean autoEnabled,
                            ViInt32   manualValue);

ViStatus _VI_FUNC rsspecan_ConfigureWLANDemodulation802_11acNsts(
                            ViSession instrSession,
                            ViInt32   nstsIndexMode,
                            ViInt32   nstsIndex);

ViStatus _VI_FUNC rsspecan_ConfigureWLANDemodulation802_11ax(ViSession instrSession,
                                                             ViBoolean PPDUAnalysisMode,
                                                             ViInt32   PPDUFormatToMeasure,
                                                             ViInt32   channelBandwidthToMeasure,
                                                             ViInt32   guardIntervalLength);

ViStatus _VI_FUNC rsspecan_ConfigureWLANDemodulation802_11axEHTPPDUFormat(
                            ViSession instrSession,
                            ViInt32   EHTPPDUFormat);

ViStatus _VI_FUNC rsspecan_ConfigureWLANDemodulation802_11axHEPPDUConfig(
                            ViSession instrSession,
                            ViInt32   HEPPDUFormat,
                            ViInt32   nHELTF);

ViStatus _VI_FUNC rsspecan_QueryWLANDemodulation802_11axHEPPDURUActiveCount(
                            ViSession instrSession,
                            ViString  activeCount);

ViStatus _VI_FUNC rsspecan_QueryWLANDemodulation802_11axHEPPDURURefresh(ViSession instrSession);

ViStatus _VI_FUNC rsspecan_QueryWLANDemodulation802_11axHEPPDUConfigHighestRUIndex(
                            ViSession instrSession,
                            ViInt32*  highestRUIndex);

ViStatus _VI_FUNC rsspecan_QueryWLANDemodulation802_11axHEPPDUConfigUserIndex(
                            ViSession instrSession,
                            ViInt32   segment,
                            ViInt32   channel,
                            ViInt32   RUIndex,
                            ViInt32*  userIndex);

ViStatus _VI_FUNC rsspecan_ConfigureWLANDemodulation802_11axHEPPDUConfigUserIndexMRU(
                            ViSession instrSession,
                            ViInt32   segment,
                            ViInt32   channel,
                            ViInt32   MRUIndex,
                            ViInt32   userIndexMRU);

ViStatus _VI_FUNC rsspecan_ConfigureWLANDemodulation802_11axHEPPDUConfigUserIndexRU26(
                            ViSession instrSession,
                            ViInt32   segment,
                            ViInt32   channel,
                            ViInt32   RU26Index,
                            ViInt32   userIndexRU26);

ViStatus _VI_FUNC rsspecan_ConfigureWLANDemodulation802_11axHEPPDUConfigUserIndexRU6(
                            ViSession instrSession,
                            ViInt32   segment,
                            ViInt32   channel,
                            ViInt32   RU6Index,
                            ViInt32   userIndexRU6);

ViStatus _VI_FUNC rsspecan_ConfigureWLANDemodulation802_11axHEPPDUConfigRUIndex(
                            ViSession instrSession,
                            ViInt32   segment,
                            ViInt32   channel,
                            ViInt32   RUIndex,
                            ViInt32   index);

ViStatus _VI_FUNC rsspecan_QueryWLANDemodulation802_11axHEPPDUConfigRUIndexCount(
                            ViSession instrSession,
                            ViInt32   segment,
                            ViInt32   channel,
                            ViInt32   RUIndex,
                            ViInt32*  count);

ViStatus _VI_FUNC rsspecan_QueryWLANDemodulation802_11axHEPPDUConfigRUIndexHighest(
                            ViSession instrSession,
                            ViInt32   segment,
                            ViInt32   channel,
                            ViInt32   RUIndex,
                            ViInt32*  highest);

ViStatus _VI_FUNC rsspecan_ConfigureWLANDemodulation802_11axHEPPDUConfigRUSize(
                            ViSession instrSession,
                            ViInt32   segment,
                            ViInt32   channel,
                            ViInt32   RUIndex,
                            ViInt32   RUSize);

ViStatus _VI_FUNC rsspecan_ConfigureWLANDemodulation802_11axHEPPDUConfigMCSIndex(
                            ViSession instrSession,
                            ViInt32   segment,
                            ViInt32   channel,
                            ViInt32   RUIndex,
                            ViInt32   userIndex,
                            ViInt32   MCSIndex);

ViStatus _VI_FUNC rsspecan_ConfigureWLANDemodulation802_11axHEPPDUConfigNstsPerUser(
                            ViSession instrSession,
                            ViInt32   segment,
                            ViInt32   channel,
                            ViInt32   RUIndex,
                            ViInt32   userIndex,
                            ViInt32   nstsPerUser);

ViStatus _VI_FUNC rsspecan_ConfigureWLANDemodulation802_11axHEPPDUConfigTXBeamforming(
                            ViSession instrSession,
                            ViInt32   segment,
                            ViInt32   channel,
                            ViInt32   RUIndex,
                            ViInt32   userIndex,
                            ViBoolean TXBeamforming);

ViStatus _VI_FUNC rsspecan_ConfigureWLANDemodulation802_11axHEPPDUConfigDCM(
                            ViSession instrSession,
                            ViInt32   segment,
                            ViInt32   channel,
                            ViInt32   RUIndex,
                            ViInt32   userIndex,
                            ViBoolean DCM);

ViStatus _VI_FUNC rsspecan_ConfigureWLANDemodulation802_11axHEPPDUConfigCoding(
                            ViSession instrSession,
                            ViInt32   segment,
                            ViInt32   channel,
                            ViInt32   RUIndex,
                            ViInt32   userIndex,
                            ViInt32   coding);

ViStatus _VI_FUNC rsspecan_ConfigureWLANDemodulation802_11axHEPPDUConfigInsert(
                            ViSession instrSession,
                            ViInt32   segment,
                            ViInt32   channel,
                            ViInt32   RUIndex,
                            ViInt32   userIndex);

ViStatus _VI_FUNC rsspecan_ConfigureWLANDemodulation802_11axHEPPDUConfigDelete(
                            ViSession instrSession,
                            ViInt32   segment,
                            ViInt32   channel,
                            ViInt32   RUIndex,
                            ViInt32   userIndex);

ViStatus _VI_FUNC rsspecan_ConfigureWLANDemodulation802_11be(ViSession instrSession,
                                                             ViBoolean PPDUAnalysisMode,
                                                             ViInt32   PPDUFormatToMeasure,
                                                             ViInt32   channelBandwidthToMeasure);

ViStatus _VI_FUNC rsspecan_ConfWLANNumberOfAntennas(ViSession instrSession,
                                                    ViInt32   numberOfAntennas);

ViStatus _VI_FUNC rsspecan_ConfWLANMIMODUTConfiguration(ViSession instrSession,
                                                        ViInt32   DUTConfiguration);

ViStatus _VI_FUNC rsspecan_ConfWLANMIMOAntennaSignalCapture(ViSession instrSession,
                                                            ViInt32   MIMOAntennaSignalCapture);

ViStatus _VI_FUNC rsspecan_ConfigureWLANMIMOAntennaSignalCaptureOSP(
                            ViSession instrSession,
                            ViInt32   OSPModule,
                            ViString  OSPIPAddress);

ViStatus _VI_FUNC rsspecan_ConfigureWLANMIMOAntennaSignalCapturePath(
                            ViSession instrSession,
                            ViInt32   path);

ViStatus _VI_FUNC rsspecan_ConfigureWLANMIMOAntennaSignalCaptureTimeSync(
                            ViSession instrSession,
                            ViInt32   timeSync);

ViStatus _VI_FUNC rsspecan_ConfWLANSTCMIMO(ViSession instrSession,
                                           ViInt32   rxAnalyzer,
                                           ViBoolean antennaState,
                                           ViString  analyzerIPAddress,
                                           ViInt32   configuration);

ViStatus _VI_FUNC rsspecan_QueryWLANMIMOLANStatus(ViSession  instrSession,
                                                  ViInt32    rxAnalyzer,
                                                  ViBoolean* LANStatus);

ViStatus _VI_FUNC rsspecan_ConfWLANMIMOReferenceLevelOffset(ViSession instrSession,
                                                            ViInt32   rxAnalyzer,
                                                            ViInt32   coupling,
                                                            ViReal64  offset);

ViStatus _VI_FUNC rsspecan_ConfWLANMIMOReferenceFrequencyCoupling(
                            ViSession instrSession,
                            ViInt32   coupling);

ViStatus _VI_FUNC rsspecan_ConfWLANMIMOAmplitudeSettingsCoupling(
                            ViSession instrSession,
                            ViInt32   coupling);

ViStatus _VI_FUNC rsspecan_ConfWLANSimultaneousCaptureState(ViSession instrSession,
                                                            ViBoolean state);

ViStatus _VI_FUNC rsspecan_ConfWLANMIMOPayloadLengthSource(ViSession instrSession,
                                                           ViInt32   source);

ViStatus _VI_FUNC rsspecan_ConfWLAN_MIMOSpatialMappingConfiguration(
                            ViSession instrSession,
                            ViInt32   spatialMappingMode,
                            ViBoolean powerNormalize);

ViStatus _VI_FUNC rsspecan_ConfWLAN_MIMO_UDFSpatialMappingAntenna(
                            ViSession instrSession,
                            ViInt32   TX,
                            ViReal64  realValues[],
                            ViReal64  imaginaryValues[],
                            ViReal64  timeShift);

ViStatus _VI_FUNC rsspecan_ConfWLAN_MIMO_UDFSpatialMappingStream(
                            ViSession instrSession,
                            ViInt32   TX,
                            ViInt32   stream,
                            ViReal64  realValue,
                            ViReal64  imaginaryValue,
                            ViReal64  timeShift);

ViStatus _VI_FUNC rsspecan_SelectWLANIQFile(ViSession instrSession,
                                            ViString  fileName,
                                            ViReal64  analysisBW);

ViStatus _VI_FUNC rsspecan_QueryWLANIQFileChannelList(ViSession instrSession,
                                                      ViUInt32  arraySize,
                                                      ViChar    channelList[],
                                                      ViUInt32* returnedValues);

ViStatus _VI_FUNC rsspecan_ConfigureWLANIQFileChannel(ViSession instrSession,
                                                      ViBoolean autoInputEnabled,
                                                      ViString  selectedDataStream);

ViStatus _VI_FUNC rsspecan_ConfigureWLANIQFileRepetitionCount(
                            ViSession instrSession,
                            ViInt32   repetitionCount);

ViStatus _VI_FUNC rsspecan_ConfigureWLANEvaluationRangeStatistics(
                            ViSession instrSession,
                            ViBoolean PPDUStatisticCount,
                            ViInt32   noOfPPDUsToAnalyze);

ViStatus _VI_FUNC rsspecan_ConfigureWLANEvaluationRangeTimeDomain(
                            ViSession instrSession,
                            ViInt32   sourceOfPayloadLength,
                            ViBoolean equalPPDULength,
                            ViInt32   minNumberOfDataSymbols,
                            ViInt32   maxNumberOfDataSymbols);

ViStatus _VI_FUNC rsspecan_ConfigureWLANEvaluationRangeTimeDomainAnalysisInterval(
                            ViSession instrSession,
                            ViInt32   offset,
                            ViBoolean lengthState,
                            ViInt32   length);

ViStatus _VI_FUNC rsspecan_ConfigureWLANEvaluationRangePVT(ViSession instrSession,
                                                           ViInt32   averageLength,
                                                           ViInt32   referencePower);

ViStatus _VI_FUNC rsspecan_ConfigureWlanEvaluationRangeMeasRange(
                            ViSession instrSession,
                            ViInt32   measRange);

ViStatus _VI_FUNC rsspecan_ConfigureWlanGainImbalanceUnit(ViSession instrSession,
                                                          ViInt32   unit);

ViStatus _VI_FUNC rsspecan_ConfigureWlanEVMUnit(ViSession instrSession,
                                                ViInt32   unit);

ViStatus _VI_FUNC rsspecan_ConfigureWlanDisplay(ViSession instrSession,
                                                ViBoolean displayTable);

ViStatus _VI_FUNC rsspecan_ConfigureWlanResultSummaryDisplay(ViSession instrSession,
                                                             ViInt32   reserved,
                                                             ViInt32   item,
                                                             ViBoolean displayItem);

ViStatus _VI_FUNC rsspecan_ConfigureWLANPolynomialDegree(ViSession instrSession,
                                                         ViInt32   polynomialDegree);

ViStatus _VI_FUNC rsspecan_ConfigureWLANAMAMAutoscale(ViSession instrSession,
                                                      ViInt32   window,
                                                      ViInt32   axis,
                                                      ViBoolean autoscaleEnabled,
                                                      ViInt32   autoMode,
                                                      ViInt32   autoFixRange);

ViStatus _VI_FUNC rsspecan_ConfigureWLANAMAMFixed(ViSession instrSession,
                                                  ViInt32   window,
                                                  ViInt32   axis,
                                                  ViReal64  min,
                                                  ViReal64  max);

ViStatus _VI_FUNC rsspecan_ConfigureWLANAMAMAutoHysteresis(ViSession instrSession,
                                                           ViInt32   window,
                                                           ViInt32   axis,
                                                           ViReal64  lowerLower,
                                                           ViReal64  lowerUpper,
                                                           ViReal64  upperLower,
                                                           ViReal64  upperUpper);

ViStatus _VI_FUNC rsspecan_ConfigureWLANAMAMMemoryDepth(ViSession instrSession,
                                                        ViInt32   window,
                                                        ViInt32   axis,
                                                        ViInt32   memoryDepth);

ViStatus _VI_FUNC rsspecan_ConfigureWLANAMAMNumberOfDivisions(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   axis,
                            ViInt32   numberOfDivisions);

ViStatus _VI_FUNC rsspecan_ConfigureWLANAMAMScalingPerDivision(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   axis,
                            ViInt32   arraySize,
                            ViInt32   scalingPerDivision[]);

ViStatus _VI_FUNC rsspecan_ConfigureWlanLimit(ViSession instrSession,
                                              ViInt32   reserved,
                                              ViInt32   reserved2,
                                              ViInt32   limitType,
                                              ViInt32   mode,
                                              ViReal64  limitValue);

ViStatus _VI_FUNC rsspecan_ConfigureWLANAllLimits(ViSession instrSession,
                                                  ViInt32   reserved,
                                                  ViInt32   reserved2,
                                                  ViInt32   arraySize,
                                                  ViReal64  values[]);

ViStatus _VI_FUNC rsspecan_GetWlanAllLimits(ViSession instrSession,
                                            ViInt32   reserved,
                                            ViInt32   reserved2,
                                            ViReal64  values[]);

ViStatus _VI_FUNC rsspecan_QueryWlanACPChannelLimit(ViSession instrSession,
                                                    ViInt32   channelType,
                                                    ViReal64* value);

ViStatus _VI_FUNC rsspecan_QueryWlanSpectrumMaxOverstepping(ViSession instrSession,
                                                            ViInt32   channelType,
                                                            ViReal64* value);

ViStatus _VI_FUNC rsspecan_SelectWlanRFMeasurement(ViSession instrSession,
                                                   ViInt32   RFMeasurement);

ViStatus _VI_FUNC rsspecan_SelectWlanIQMeasurement(ViSession instrSession,
                                                   ViInt32   measurement);

ViStatus _VI_FUNC rsspecan_ConfWlanPVTMeas(ViSession instrSession,
                                           ViInt32   risingFalling);

ViStatus _VI_FUNC rsspecan_ConfWlanEVMMeasurement(ViSession instrSession,
                                                  ViInt32   mode,
                                                  ViBoolean autoScaling,
                                                  ViReal64  perDivision,
                                                  ViInt32   unit);

ViStatus _VI_FUNC rsspecan_ConfWlanSpectrumMeasurement(ViSession instrSession,
                                                       ViInt32   mode);

ViStatus _VI_FUNC rsspecan_ConfWLANChannelSelect(ViSession instrSession,
                                                 ViInt32   mode);

ViStatus _VI_FUNC rsspecan_ConfWlanConstellationMeasurement(ViSession instrSession,
                                                            ViInt32   constellationMode,
                                                            ViInt32   carrierSelection,
                                                            ViInt32   carrierNumber);

ViStatus _VI_FUNC rsspecan_ConfWlanStatisticsMeasurement(ViSession instrSession,
                                                         ViInt32   mode);

ViStatus _VI_FUNC rsspecan_ConfWlanSEMMeasurement(ViSession instrSession,
                                                  ViInt32   type,
                                                  ViString  fileName,
                                                  ViInt32   powerClass);

ViStatus _VI_FUNC rsspecan_ConfigureWLANSEMChannelBandwidth(ViSession instrSession,
                                                            ViInt32   channelBandwidth);

ViStatus _VI_FUNC rsspecan_ConfigureWLANSpectrumFlatnessUnit(ViSession instrSession,
                                                             ViInt32   units);

ViStatus _VI_FUNC rsspecan_ConfWlanAMMeasurement(ViSession instrSession,
                                                 ViInt32   measurement);

ViStatus _VI_FUNC rsspecan_ConfWlanACPMode(ViSession instrSession,
                                           ViInt32   ACPMode);

ViStatus _VI_FUNC rsspecan_ConfigureWLANPPDUSelectedEnabled(ViSession instrSession,
                                                            ViBoolean PPDUSelectedEnabled);

ViStatus _VI_FUNC rsspecan_ConfigureWLANHysteresisThreshold(ViSession instrSession,
                                                            ViReal64  lower,
                                                            ViReal64  upper);

ViStatus _VI_FUNC rsspecan_QueryWLANBurstPPDUStatus(ViSession instrSession,
                                                    ViInt32   arraySize,
                                                    ViInt32   PPDUStatus[],
                                                    ViInt32*  noOfValues);

ViStatus _VI_FUNC rsspecan_QueryWlanACPMeasurement(ViSession instrSession,
                                                   ViInt32   mode,
                                                   ViReal64  values[]);

ViStatus _VI_FUNC rsspecan_WlanBurstImmediate(ViSession instrSession);

ViStatus _VI_FUNC rsspecan_ReadWlanTraceData(ViSession instrSession,
                                             ViInt32   trace,
                                             ViInt32   arraySize,
                                             ViReal64  traceData[],
                                             ViInt32*  noOfValues);

ViStatus _VI_FUNC rsspecan_ReadWlanMemoryIQData(ViSession instrSession,
                                                ViInt32   offsetSamples,
                                                ViInt32   noOfSamples,
                                                ViInt32   bufferSize,
                                                ViInt32*  noOfPoints,
                                                ViReal64  realParts_I[],
                                                ViReal64  imaginaryParts_Q[]);

ViStatus _VI_FUNC rsspecan_ReadWlanSEMResults(ViSession instrSession,
                                              ViInt32   noOfValues,
                                              ViReal64  index[],
                                              ViReal64  startFrequencyBand[],
                                              ViReal64  stopFrequencyBand[],
                                              ViReal64  resolutionBandwidth[],
                                              ViReal64  limitFailFrequency[],
                                              ViReal64  powerAbs[],
                                              ViReal64  powerRel[],
                                              ViReal64  limitDistance[],
                                              ViReal64  failureFlag[],
                                              ViInt32*  returnedValues);

ViStatus _VI_FUNC rsspecan_FetchWlanBurstAll(ViSession instrSession,
                                             ViReal64  results[],
                                             ViInt32*  returnedValues);

ViStatus _VI_FUNC rsspecan_FetchWlanBurstPower(ViSession instrSession,
                                               ViInt32   powerMeasurement,
                                               ViInt32   resultType,
                                               ViUInt32  arraySize,
                                               ViReal64  values[],
                                               ViUInt32* returnedValues);

ViStatus _VI_FUNC rsspecan_FetchWlanBurstError(ViSession instrSession,
                                               ViInt32   errorType,
                                               ViInt32   mode,
                                               ViUInt32  arraySize,
                                               ViReal64  values[],
                                               ViUInt32* returnedValues);

ViStatus _VI_FUNC rsspecan_FetchWLANEffectiveChannelGain(ViSession instrSession,
                                                         ViInt32   arraySize,
                                                         ViReal64  results[],
                                                         ViInt32*  returnedValues);

ViStatus _VI_FUNC rsspecan_FetchWLANPhysicalChannelGain(ViSession instrSession,
                                                        ViInt32   arraySize,
                                                        ViReal64  results[],
                                                        ViInt32*  returnedValues);

ViStatus _VI_FUNC rsspecan_FetchWlanIQImp(ViSession instrSession,
                                          ViInt32   IQImpairment,
                                          ViInt32   mode,
                                          ViUInt32  arraySize,
                                          ViReal64  values[],
                                          ViUInt32* returnedValues);

ViStatus _VI_FUNC rsspecan_FetchWlanEVM(ViSession instrSession,
                                        ViInt32   EVM,
                                        ViInt32   mode,
                                        ViUInt32  arraySize,
                                        ViReal64  values[],
                                        ViUInt32* returnedValues);

ViStatus _VI_FUNC rsspecan_FetchWlanPPDUEVM(ViSession instrSession,
                                            ViInt32   EVM,
                                            ViUInt32  arraySize,
                                            ViReal64  values[],
                                            ViUInt32* returnedValues);

ViStatus _VI_FUNC rsspecan_FetchWLANSignalContentDetailedEVM(ViSession instrSession,
                                                             ViInt32   EVM,
                                                             ViUInt32  arraySize,
                                                             ViReal64  values[],
                                                             ViUInt32* returnedValues);

ViStatus _VI_FUNC rsspecan_FetchWLANSignalContentDetailedPPDU(
                            ViSession instrSession,
                            ViInt32   resultType,
                            ViInt32   subcarriers,
                            ViUInt32  arraySize,
                            ViReal64  values[],
                            ViUInt32* returnedValues);

ViStatus _VI_FUNC rsspecan_FetchWlanBurstCount(ViSession instrSession,
                                               ViInt32   type,
                                               ViInt32*  count);

ViStatus _VI_FUNC rsspecan_FetchWlanSymbolCount(ViSession instrSession,
                                                ViInt32   arraySize,
                                                ViInt32   symbolCount[],
                                                ViInt32*  noOfValues);

ViStatus _VI_FUNC rsspecan_FetchWlanBurstLength(ViSession instrSession,
                                                ViInt32*  length);

ViStatus _VI_FUNC rsspecan_FetchWLANBurstType(ViSession instrSession,
                                              ViInt32   bufferSize,
                                              ViChar    burstType[]);

ViStatus _VI_FUNC rsspecan_FetchWLANBurstGuardInterval(ViSession instrSession,
                                                       ViInt32*  guardInterval);

ViStatus _VI_FUNC rsspecan_FetchWLANMCSIndex(ViSession instrSession,
                                             ViInt32*  MCSIndex);

ViStatus _VI_FUNC rsspecan_FetchWlanBurstPayload(ViSession instrSession,
                                                 ViInt32   mode,
                                                 ViReal64* value);

ViStatus _VI_FUNC rsspecan_FetchWlanBurstPeak(ViSession instrSession,
                                              ViInt32   mode,
                                              ViReal64* value);

ViStatus _VI_FUNC rsspecan_FetchWlanBurstPreamble(ViSession instrSession,
                                                  ViInt32   mode,
                                                  ViReal64* value);

ViStatus _VI_FUNC rsspecan_FetchWlanBurstStart(ViSession instrSession,
                                               ViReal64* value);

ViStatus _VI_FUNC rsspecan_QueryWlanBurstTime(ViSession instrSession,
                                              ViInt32   time,
                                              ViInt32   mode,
                                              ViUInt32  arraySize,
                                              ViReal64  values[],
                                              ViUInt32* returnedValues);

ViStatus _VI_FUNC rsspecan_QueryWlanBurstErrorRateForPilots(ViSession instrSession,
                                                            ViInt32   mode,
                                                            ViUInt32  arraySize,
                                                            ViReal64  values[],
                                                            ViUInt32* returnedValues);

ViStatus _VI_FUNC rsspecan_FetchWlanStreamBurstError(ViSession instrSession,
                                                     ViInt32   errorType,
                                                     ViInt32   mode,
                                                     ViInt32   arraySize,
                                                     ViReal64  results[],
                                                     ViInt32*  returnedValues);

ViStatus _VI_FUNC rsspecan_FetchWlanStreamBurstPower(ViSession instrSession,
                                                     ViInt32   powerMeasurement,
                                                     ViInt32   arraySize,
                                                     ViReal64  results[],
                                                     ViInt32*  returnedValues);

ViStatus _VI_FUNC rsspecan_FetchWlanStreamBurstChannelPower(ViSession instrSession,
                                                            ViInt32   powerMeasurement,
                                                            ViInt32   arraySize,
                                                            ViReal64  results[],
                                                            ViInt32*  returnedValues);

ViStatus _VI_FUNC rsspecan_FetchWLANAMAMPolynomialCoefficients(
                            ViSession instrSession,
                            ViUInt32  arraySize,
                            ViReal64  results[],
                            ViUInt32* returnedValues);

ViStatus _VI_FUNC rsspecan_FetchWLANDetailedSignalContentIEEE802_11ax(
                            ViSession instrSession,
                            ViUInt32  arraySize,
                            ViChar    results[],
                            ViUInt32* returnedValues);

ViStatus _VI_FUNC rsspecan_FetchWLANSignalField(ViSession instrSession,
                                                ViUInt32  arraySize,
                                                ViChar    results[],
                                                ViUInt32* returnedValues);

ViStatus _VI_FUNC rsspecan_FetchWLANUnusedToneErrorSummary(ViSession instrSession,
                                                           ViUInt32  arraySize,
                                                           ViChar    results[],
                                                           ViUInt32* returnedValues);

ViStatus _VI_FUNC rsspecan_FetchWLANSpectrumFlatnessSummary(ViSession instrSession,
                                                            ViUInt32  arraySize,
                                                            ViUInt32* returnedValues,
                                                            ViInt32   startNumber[],
                                                            ViInt32   endNumber[],
                                                            ViReal64  minDistance[],
                                                            ViInt32   subcarrierMinDistance[],
                                                            ViInt32   minDistanceStatus[],
                                                            ViReal64  minUpperDistance[],
                                                            ViInt32   subcarrierUppMinDistance[],
                                                            ViInt32   minUppDistanceStatus[]);

ViStatus _VI_FUNC rsspecan_GetWlanPowerResults(ViSession instrSession,
                                               ViInt32   reserved,
                                               ViInt32   powerMeasurement,
                                               ViInt32   arraySize,
                                               ViReal64  results[],
                                               ViInt32*  returnedValues);

ViStatus _VI_FUNC rsspecan_GetWlanLimitCheckResult(ViSession instrSession,
                                                   ViInt32   reserved,
                                                   ViInt32   reserved2,
                                                   ViInt32   limitType,
                                                   ViInt32   mode,
                                                   ViInt32*  result);

ViStatus _VI_FUNC rsspecan_GetWlanACPChannLimitCheckResults(ViSession instrSession,
                                                            ViInt32   reserved,
                                                            ViInt32   reserved2,
                                                            ViInt32   channelType,
                                                            ViInt32   results[]);

ViStatus _VI_FUNC rsspecan_GetWlanAllLimitCheckResults(ViSession instrSession,
                                                       ViInt32   reserved,
                                                       ViInt32   reserved2,
                                                       ViInt32   results[]);

ViStatus _VI_FUNC rsspecan_GetWlanLimitCheck(ViSession instrSession,
                                             ViInt32   reserved,
                                             ViInt32   limit,
                                             ViInt32*  result);

ViStatus _VI_FUNC rsspecan_GetWLANUpperLimitLine(ViSession instrSession,
                                                 ViInt32   window,
                                                 ViInt32   limit,
                                                 ViInt32   arraySize,
                                                 ViReal64  results[],
                                                 ViInt32*  returnedValues);

ViStatus _VI_FUNC rsspecan_GetWLANLowerLimitLine(ViSession instrSession,
                                                 ViInt32   window,
                                                 ViInt32   limit,
                                                 ViInt32   arraySize,
                                                 ViReal64  results[],
                                                 ViInt32*  returnedValues);

ViStatus _VI_FUNC rsspecan_SetWiGigMode(ViSession instrSession);

ViStatus _VI_FUNC rsspecan_ConfigureWiGigWlanAutoLevelOnce(ViSession instrSession);

ViStatus _VI_FUNC rsspecan_ConfWiGigReferenceLevel(ViSession instrSession,
                                                   ViReal64  referenceLevel,
                                                   ViReal64  referenceLevelOffset,
                                                   ViInt32   units);

ViStatus _VI_FUNC rsspecan_ConfigureWiGigTracking(ViSession instrSession,
                                                  ViBoolean phase,
                                                  ViBoolean timing,
                                                  ViBoolean level);

ViStatus _VI_FUNC rsspecan_ConfigureWiGigIQMismatchCompensation(
                            ViSession instrSession,
                            ViBoolean IQMismatchCompensation);

ViStatus _VI_FUNC rsspecan_ConfigureWiGigDataAcquisitionRates(
                            ViSession instrSession,
                            ViInt32   contiguousNumber,
                            ViInt32   usedChipRate,
                            ViReal64  chipRate);

ViStatus _VI_FUNC rsspecan_SelectWiGigIQFile(ViSession instrSession,
                                             ViString  fileName,
                                             ViReal64  analysisBW);

ViStatus _VI_FUNC rsspecan_LoadWiGigSEMFile(ViSession instrSession,
                                            ViString  fileName);

ViStatus _VI_FUNC rsspecan_AddWiGigWindow(ViSession instrSession,
                                          ViInt32   stringSize,
                                          ViString  referenceWindowName,
                                          ViInt32   direction,
                                          ViInt32   windowType,
                                          ViChar    returnedWindowName[]);

ViStatus _VI_FUNC rsspecan_ReplaceWiGigWindow(ViSession instrSession,
                                              ViString  windowName,
                                              ViInt32   windowType);

ViStatus _VI_FUNC rsspecan_WiGigRecalculateIQMeasurementResults(ViSession instrSession);

ViStatus _VI_FUNC rsspecan_ConfigureWiGigStatisticCount(ViSession instrSession,
                                                        ViBoolean state,
                                                        ViInt32   noOfPPDU);

ViStatus _VI_FUNC rsspecan_ConfigureWiGigPPDUToAnalyze(ViSession instrSession,
                                                       ViBoolean state,
                                                       ViInt32   PPDUIndex);

ViStatus _VI_FUNC rsspecan_ConfigureWiGigSymbolNumberToAnalzye(
                            ViSession instrSession,
                            ViBoolean equalBurstLength,
                            ViInt32   minNumberOfSymbols,
                            ViInt32   maxNumberOfSymbols);

ViStatus _VI_FUNC rsspecan_FetchWiGigBurstAll(ViSession instrSession,
                                              ViReal64  results[],
                                              ViInt32*  returnedValues);

ViStatus _VI_FUNC rsspecan_FetchWiGigBurstCount(ViSession instrSession,
                                                ViInt32*  count);

ViStatus _VI_FUNC rsspecan_FetchWiGigBurstLengths(ViSession instrSession,
                                                  ViInt32   arraySize,
                                                  ViInt32   lengths[]);

ViStatus _VI_FUNC rsspecan_FetchWiGigBurstStart(ViSession instrSession,
                                                ViInt32   arraySize,
                                                ViInt32   starts[]);

ViStatus _VI_FUNC rsspecan_FetchWiGigPower(ViSession instrSession,
                                           ViInt32   powerMeasurement,
                                           ViInt32   mode,
                                           ViReal64* value);

ViStatus _VI_FUNC rsspecan_FetchWiGigError(ViSession instrSession,
                                           ViInt32   errorType,
                                           ViInt32   mode,
                                           ViReal64* value);

ViStatus _VI_FUNC rsspecan_FetchWiGigIQImp(ViSession instrSession,
                                           ViInt32   IQImpairment,
                                           ViInt32   mode,
                                           ViReal64* value);

ViStatus _VI_FUNC rsspecan_FetchWiGigEVM(ViSession instrSession,
                                         ViInt32   EVM,
                                         ViInt32   mode,
                                         ViReal64* value);

ViStatus _VI_FUNC rsspecan_FetchWiGigTime(ViSession instrSession,
                                          ViInt32   time,
                                          ViInt32   mode,
                                          ViReal64* value);

ViStatus _VI_FUNC rsspecan_FetchWiGigBERPPDUHeader(ViSession instrSession,
                                                   ViInt32   mode,
                                                   ViReal64* value);

ViStatus _VI_FUNC rsspecan_FetchWiGigBERPPDUPayload(ViSession instrSession,
                                                    ViInt32   mode,
                                                    ViReal64* value);

ViStatus _VI_FUNC rsspecan_FetchWiGigSignalToRatioStatistics(ViSession instrSession,
                                                             ViReal64* minimumSNR,
                                                             ViReal64* averageSNR,
                                                             ViReal64* maximumSNR);

ViStatus _VI_FUNC rsspecan_ConfigureWiGigResultSummaryDisplay(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   item,
                            ViBoolean displayItem);

ViStatus _VI_FUNC rsspecan_ConfigureWiGigBitSTreamFormat(ViSession instrSession,
                                                         ViInt32   format);

ViStatus _VI_FUNC rsspecan_ConfigureWiGigExportingTraceResultsToASCIIFileEnabled(
                            ViSession instrSession,
                            ViBoolean resultsToASCIIFileEnabled);

ViStatus _VI_FUNC rsspecan_SetLTEDownlinkMode(ViSession instrSession);

ViStatus _VI_FUNC rsspecan_ConfigureLTEDownlinkSignalCharacteristics(
                            ViSession instrSession,
                            ViInt32   componentCarrier,
                            ViInt32   duplexing,
                            ViReal64  frequency,
                            ViInt32   bandwidthSettingType,
                            ViInt32   channelBandwidth,
                            ViInt32   numberOfResourceBlocks,
                            ViInt32   cyclicPrefix);

ViStatus _VI_FUNC rsspecan_ConfigureLTEDownlinkTDDFrames(ViSession instrSession,
                                                         ViInt32   componentCarrier,
                                                         ViInt32   TDDULDLAllocations,
                                                         ViInt32   specialSubframe);

ViStatus _VI_FUNC rsspecan_ConfigureLTEDownlinkPhysicalLayerCellIdentity(
                            ViSession instrSession,
                            ViInt32   componentCarrier,
                            ViBoolean autoDetection,
                            ViInt32   cellIdentityGroup,
                            ViInt32   cellIdentity);

ViStatus _VI_FUNC rsspecan_QueryLTEDownlinkPhysicalLayerCellIdentity(
                            ViSession instrSession,
                            ViInt32   componentCarrier,
                            ViInt32*  cellIdentityGroup,
                            ViInt32*  cellIdentity);

ViStatus _VI_FUNC rsspecan_ConfigureLTEDownlinkExcludeInbandNBIoT(
                            ViSession instrSession,
                            ViInt32   componentCarrier,
                            ViBoolean enabled,
                            ViInt32   NBIoTRBOffset);

ViStatus _VI_FUNC rsspecan_StoreLTEDownlinkDemodulationSettings(
                            ViSession instrSession,
                            ViInt32   componentCarrier,
                            ViString  filename);

ViStatus _VI_FUNC rsspecan_ConfigureLTEDownlinkMIMO(ViSession instrSession,
                                                    ViInt32   componentCarrier,
                                                    ViInt32   configuration,
                                                    ViInt32   antennaSelection);

ViStatus _VI_FUNC rsspecan_ConfigureLTEDownlinkMIMOMultipleAnalyzers(
                            ViSession instrSession,
                            ViInt32   rxAnalyzer,
                            ViBoolean state,
                            ViString  analyzerIPAddress);

ViStatus _VI_FUNC rsspecan_ConfigureLTEDownlinkPDSCHSettings(ViSession instrSession,
                                                             ViBoolean autoDemodulation,
                                                             ViInt32   PDSCHSubframeDetect);

ViStatus _VI_FUNC rsspecan_ConfigureLTEDownlinkPDSCHConfigurableSubframes(
                            ViSession instrSession,
                            ViInt32   componentCarrier,
                            ViInt32   configurableSubframes);

ViStatus _VI_FUNC rsspecan_ConfigureLTEDownlinkPDSCHUsedAllocations(
                            ViSession instrSession,
                            ViInt32   componentCarrier,
                            ViInt32   subframe,
                            ViInt32   usedAllocations);

ViStatus _VI_FUNC rsspecan_ConfigureLTEDownlinkPDSCHUsedAllocation(
                            ViSession instrSession,
                            ViInt32   componentCarrier,
                            ViInt32   subframe,
                            ViInt32   usedAllocation,
                            ViInt32   IDN_RNTI,
                            ViInt32   codeword,
                            ViInt32   modulation,
                            ViInt32   VRBGap,
                            ViInt32   resourceBlocks,
                            ViInt32   resourceBlocksOffset,
                            ViReal64  power);

ViStatus _VI_FUNC rsspecan_ConfigureLTEDownlinkPDSCHUsedAllocationEnhancedSettings(
                            ViSession instrSession,
                            ViInt32   componentCarrier,
                            ViInt32   subframe,
                            ViInt32   usedAllocation,
                            ViInt32   precoding,
                            ViInt32   codewordToLayerMapping,
                            ViInt32   scramblingIdentity,
                            ViInt32   singleLayerAntennaPort,
                            ViInt32   codebookIndex,
                            ViBoolean cyclicDelayDiversity,
                            ViInt32   startOffset);

ViStatus _VI_FUNC rsspecan_ConfigureLTEDownlinkSynchronizationSignal(
                            ViSession instrSession,
                            ViInt32   componentCarrier,
                            ViInt32   pSSyncTxAntenna,
                            ViReal64  pSYNCRelativePower,
                            ViReal64  sSYNCRelativePower);

ViStatus _VI_FUNC rsspecan_ConfigureLTEDownlinkReferenceSignalCarr(
                            ViSession instrSession,
                            ViInt32   componentCarrier,
                            ViReal64  referencePower);

ViStatus _VI_FUNC rsspecan_ConfigureLTEDownlinkReferenceSignal(
                            ViSession instrSession,
                            ViReal64  referencePower);

ViStatus _VI_FUNC rsspecan_ConfigureLTEDownlinkPositioningReferenceSignal(
                            ViSession instrSession,
                            ViInt32   componentCarrier,
                            ViBoolean present,
                            ViInt32   bandwidth,
                            ViInt32   configurationIndex,
                            ViReal64  relativePower,
                            ViInt32   numberOfSubframes,
                            ViInt32   frameNumberOffset);

ViStatus _VI_FUNC rsspecan_ConfigureLTEDownlinkCSIReferenceSignal(
                            ViSession instrSession,
                            ViInt32   componentCarrier,
                            ViBoolean present,
                            ViInt32   antenaPorts,
                            ViInt32   configurationIndex,
                            ViBoolean overwritePDSCH,
                            ViReal64  relativePower,
                            ViInt32   subframeConfiguration,
                            ViInt32   frameNumberOffset);

ViStatus _VI_FUNC rsspecan_ConfigureLTEDownlinkPRBSymbolOffset(
                            ViSession instrSession,
                            ViInt32   componentCarrier,
                            ViInt32   PRBSymbolOffset);

ViStatus _VI_FUNC rsspecan_ConfigureLTEDownlinkPBCH(ViSession instrSession,
                                                    ViInt32   componentCarrier,
                                                    ViBoolean PBCHPresent,
                                                    ViReal64  PBCHRelativePower);

ViStatus _VI_FUNC rsspecan_ConfigureLTEDownlinkPCFICH(ViSession instrSession,
                                                      ViInt32   componentCarrier,
                                                      ViBoolean PCFICHPresent,
                                                      ViReal64  PCFICHRelativePower);

ViStatus _VI_FUNC rsspecan_ConfigureLTEDownlinkPHICH(ViSession instrSession,
                                                     ViInt32   componentCarrier,
                                                     ViInt32   PHICHNumberOfGroups,
                                                     ViInt32   PHICHDuration,
                                                     ViReal64  PHICHRelativePower);

ViStatus _VI_FUNC rsspecan_ConfigureLTEDownlinkPHICHNgParameter(
                            ViSession instrSession,
                            ViInt32   componentCarrier,
                            ViInt32   PHICHNgParameter,
                            ViInt32   PHICHNumberOfGroups);

ViStatus _VI_FUNC rsspecan_ConfigureLTEDownlinkPHICHEnhancedTestModels(
                            ViSession instrSession,
                            ViInt32   componentCarrier,
                            ViBoolean PHICHTDDM_i1);

ViStatus _VI_FUNC rsspecan_ConfigureLTEDownlinkPDCCH(ViSession instrSession,
                                                     ViInt32   componentCarrier,
                                                     ViInt32   numberOfPDCCHs,
                                                     ViInt32   PDCCHFormat,
                                                     ViReal64  PDCCHRelativePower);

ViStatus _VI_FUNC rsspecan_ConfigureLTEDownlinkPDSCHPowerRatio(
                            ViSession instrSession,
                            ViInt32   componentCarrier,
                            ViInt32   PDSCHPowerRatio);

ViStatus _VI_FUNC rsspecan_ConfigureLTEDownlinkEPDCCH(ViSession instrSession,
                                                      ViInt32   componentCarrier,
                                                      ViInt32   PRBPairs,
                                                      ViReal64  relativePower,
                                                      ViBoolean localized,
                                                      ViInt32   setID,
                                                      ViInt32   RBAssign);

ViStatus _VI_FUNC rsspecan_ConfigureLTEDownlinkMBSFN(ViSession instrSession,
                                                     ViInt32   componentCarrier,
                                                     ViBoolean present,
                                                     ViReal64  relativePower,
                                                     ViInt32   areaID,
                                                     ViInt32   nonMBSFNRegionLength);

ViStatus _VI_FUNC rsspecan_ConfigureLTEDownlinkMBSFNSubframe(ViSession instrSession,
                                                             ViInt32   componentCarrier,
                                                             ViInt32   subframe,
                                                             ViBoolean active,
                                                             ViBoolean PMCHPresent,
                                                             ViInt32   modulation);

ViStatus _VI_FUNC rsspecan_ConfigureLTEDownlinkCustomSyncWeightEnabled(
                            ViSession instrSession,
                            ViInt32   componentCarrier,
                            ViBoolean enabled);

ViStatus _VI_FUNC rsspecan_ConfigureLTEDownlinkCustomSyncWeightFirstHalf(
                            ViSession instrSession,
                            ViInt32   componentCarrier,
                            ViReal64  imaginary,
                            ViReal64  real);

ViStatus _VI_FUNC rsspecan_ConfigureLTEDownlinkCustomSyncWeightSecondHalf(
                            ViSession instrSession,
                            ViInt32   componentCarrier,
                            ViReal64  imaginary,
                            ViReal64  real);

ViStatus _VI_FUNC rsspecan_ConfigureLTEDownlinkSignalCapture(ViSession instrSession,
                                                             ViReal64  captureTime,
                                                             ViBoolean overallFrameCount,
                                                             ViBoolean autoAccordingToStandard,
                                                             ViInt32   numberOfFramesToAnalyzer,
                                                             ViBoolean swapIQ);

ViStatus _VI_FUNC rsspecan_ConfigureLTEDownlinkNumberOfSubframesToAnalyze(
                            ViSession instrSession,
                            ViInt32   subframesPerFrame);

ViStatus _VI_FUNC rsspecan_ConfigureLTEDownlinkParameterEstimation(
                            ViSession instrSession,
                            ViBoolean boostingEstimation,
                            ViInt32   channelEstimation);

ViStatus _VI_FUNC rsspecan_ConfigureLTEDownlinkTracking(ViSession instrSession,
                                                        ViInt32   phaseTracking,
                                                        ViBoolean timing);

ViStatus _VI_FUNC rsspecan_ConfigureLTEDownlinkDemodulation(ViSession instrSession,
                                                            ViInt32   componentCarrier,
                                                            ViBoolean multicarrierFilter,
                                                            ViBoolean compensateCrosstalk,
                                                            ViBoolean scramblingOfCodedBits,
                                                            ViBoolean decodeAllChannels,
                                                            ViInt32   EVMCalculationMethod,
                                                            ViInt32   PDSCHReferenceData);

ViStatus _VI_FUNC rsspecan_ConfigureLTEDownlinkSuppressInterfererForSynchronization(
                            ViSession instrSession,
                            ViBoolean suppressInterferer);

ViStatus _VI_FUNC rsspecan_ConfigureLTEDownlinkCarrierAggregation(
                            ViSession instrSession,
                            ViBoolean carrierAggregationState,
                            ViInt32   numberOfCarriers);

ViStatus _VI_FUNC rsspecan_ConfigureLTEDownlinkNumberOfDevices(
                            ViSession instrSession,
                            ViInt32   numberOfDevices);

ViStatus _VI_FUNC rsspecan_ConfigureLTEDownlinkPowerOnOffMeasurement(
                            ViSession instrSession,
                            ViInt32   numberOfFrames,
                            ViBoolean noiseCorrection);

ViStatus _VI_FUNC rsspecan_ConfigureLTEDownlinkFrequencySweepMeasurements(
                            ViSession instrSession,
                            ViInt32   assumedAdjChannelCarrier,
                            ViInt32   category,
                            ViReal64  homeAreaBasestationPower);

ViStatus _VI_FUNC rsspecan_ConfigureLTEDownlinkFrequencySweepMeasurementsAuto(
                            ViSession instrSession,
                            ViBoolean state);

ViStatus _VI_FUNC rsspecan_ConfigureLTEDownlinkSEMOperatingBand(
                            ViSession instrSession,
                            ViBoolean operatingBandLimitEnabled,
                            ViInt32   operatingBand);

ViStatus _VI_FUNC rsspecan_LTEDownlinkAddWindow(ViSession instrSession,
                                                ViInt32   stringSize,
                                                ViString  referenceWindowName,
                                                ViInt32   direction,
                                                ViInt32   windowType,
                                                ViChar    returnedWindowName[]);

ViStatus _VI_FUNC rsspecan_LTEDownlinkReplaceWindow(ViSession instrSession,
                                                    ViString  windowName,
                                                    ViInt32   windowType);

ViStatus _VI_FUNC rsspecan_LTEDownlinkSelectTab(ViSession instrSession,
                                                ViInt32   window,
                                                ViInt32   tab);

ViStatus _VI_FUNC rsspecan_ConfigureLTEDownlinkMeasurement(ViSession instrSession,
                                                           ViInt32   measurement);

ViStatus _VI_FUNC rsspecan_ConfigureLTEDownlinkMeasurementSignalDetection(ViSession instrSession);

ViStatus _VI_FUNC rsspecan_QueryLTEDownlinkMeasurementSynchronizationState(
                            ViSession  instrSession,
                            ViInt32    componentCarrier,
                            ViBoolean* OFDMSymbolTiming,
                            ViBoolean* pSYNCSynchronization,
                            ViBoolean* sSYNCSynchronization);

ViStatus _VI_FUNC rsspecan_QueryLTEDownlinkMeasurementResultSummary(
                            ViSession instrSession,
                            ViInt32   componentCarrier,
                            ViInt32   frameResult,
                            ViInt32   measurementType,
                            ViReal64* result);

ViStatus _VI_FUNC rsspecan_QueryLTEDownlinkMeasurementLimitCheckResult(
                            ViSession instrSession,
                            ViInt32   componentCarrier,
                            ViInt32   frameResult,
                            ViInt32*  average,
                            ViInt32*  maximum);

ViStatus _VI_FUNC rsspecan_QueryLTEDownlinkMeasurementEVMAll(ViSession instrSession,
                                                             ViInt32   componentCarrier,
                                                             ViInt32   measurementType,
                                                             ViReal64* result);

ViStatus _VI_FUNC rsspecan_QueryLTEDownlinkMeasurementEVMPhysicalChannel(
                            ViSession instrSession,
                            ViInt32   componentCarrier,
                            ViInt32   measurementType,
                            ViReal64* result);

ViStatus _VI_FUNC rsspecan_QueryLTEDownlinkMeasurementEVMPhysicalChannelLimitCheck(
                            ViSession instrSession,
                            ViInt32   componentCarrier,
                            ViInt32*  average,
                            ViInt32*  maximum);

ViStatus _VI_FUNC rsspecan_QueryLTEDownlinkMeasurementEVMPhysicalSignal(
                            ViSession instrSession,
                            ViInt32   componentCarrier,
                            ViInt32   measurementType,
                            ViReal64* result);

ViStatus _VI_FUNC rsspecan_QueryLTEDownlinkMeasurementEVMPhysicalSignalLimitCheck(
                            ViSession instrSession,
                            ViInt32   componentCarrier,
                            ViInt32*  average,
                            ViInt32*  maximum);

ViStatus _VI_FUNC rsspecan_QueryLTEDownlinkMeasurementFrequencyError(
                            ViSession instrSession,
                            ViInt32   componentCarrier,
                            ViInt32   measurementType,
                            ViReal64* result);

ViStatus _VI_FUNC rsspecan_QueryLTEDownlinkMeasurementFrequencyErrorLimitCheck(
                            ViSession instrSession,
                            ViInt32   componentCarrier,
                            ViInt32*  average,
                            ViInt32*  maximum);

ViStatus _VI_FUNC rsspecan_QueryLTEDownlinkMeasurementSamplingError(
                            ViSession instrSession,
                            ViInt32   componentCarrier,
                            ViInt32   measurementType,
                            ViReal64* result);

ViStatus _VI_FUNC rsspecan_QueryLTEDownlinkMeasurementSamplingErrorLimitCheck(
                            ViSession instrSession,
                            ViInt32   componentCarrier,
                            ViInt32*  average,
                            ViInt32*  maximum);

ViStatus _VI_FUNC rsspecan_QueryLTEDownlinkMeasurementCyclicPrefix(
                            ViSession instrSession,
                            ViInt32   componentCarrier,
                            ViInt32*  cyclicPrefix);

ViStatus _VI_FUNC rsspecan_QueryLTEDownlinkMeasurementIQOffset(
                            ViSession instrSession,
                            ViInt32   componentCarrier,
                            ViInt32   measurementType,
                            ViReal64* result);

ViStatus _VI_FUNC rsspecan_QueryLTEDownlinkMeasurementIQOffsetLimitCheck(
                            ViSession instrSession,
                            ViInt32   componentCarrier,
                            ViInt32*  average,
                            ViInt32*  maximum);

ViStatus _VI_FUNC rsspecan_QueryLTEDownlinkMeasurementIQGainImbalance(
                            ViSession instrSession,
                            ViInt32   componentCarrier,
                            ViInt32   measurementType,
                            ViReal64* result);

ViStatus _VI_FUNC rsspecan_QueryLTEDownlinkMeasurementIQGainImbalanceLimitCheck(
                            ViSession instrSession,
                            ViInt32   componentCarrier,
                            ViInt32*  average,
                            ViInt32*  maximum);

ViStatus _VI_FUNC rsspecan_QueryLTEDownlinkMeasurementIQQuadratureError(
                            ViSession instrSession,
                            ViInt32   componentCarrier,
                            ViInt32   measurementType,
                            ViReal64* result);

ViStatus _VI_FUNC rsspecan_QueryLTEDownlinkMeasurementIQQuadratureErrorLimitCheck(
                            ViSession instrSession,
                            ViInt32   componentCarrier,
                            ViInt32*  average,
                            ViInt32*  maximum);

ViStatus _VI_FUNC rsspecan_QueryLTEDownlinkMeasurementRSTP(ViSession instrSession,
                                                           ViInt32   componentCarrier,
                                                           ViInt32   measurementType,
                                                           ViReal64* result);

ViStatus _VI_FUNC rsspecan_QueryLTEDownlinkMeasurementOSTP(ViSession instrSession,
                                                           ViInt32   componentCarrier,
                                                           ViInt32   measurementType,
                                                           ViReal64* result);

ViStatus _VI_FUNC rsspecan_QueryLTEDownlinkMeasurementFramePower(
                            ViSession instrSession,
                            ViInt32   componentCarrier,
                            ViInt32   measurementType,
                            ViReal64* result);

ViStatus _VI_FUNC rsspecan_QueryLTEDownlinkMeasurementCrestFactor(
                            ViSession instrSession,
                            ViInt32   componentCarrier,
                            ViInt32   measurementType,
                            ViReal64* result);

ViStatus _VI_FUNC rsspecan_QueryLTEDownlinkMeasurementRSSI(ViSession instrSession,
                                                           ViInt32   componentCarrier,
                                                           ViInt32   measurementType,
                                                           ViReal64* result);

ViStatus _VI_FUNC rsspecan_QueryLTEDownlinkMeasurementFrameStartOffset(
                            ViSession instrSession,
                            ViInt32   componentCarrier,
                            ViReal64* result);

ViStatus _VI_FUNC rsspecan_ReadLTEDownlinkChannelDecoderResultsPBCH(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32*  subframeNumber,
                            ViInt32*  numberOfAntennas,
                            ViReal64* systemBandwidth,
                            ViInt32*  frame,
                            ViInt32*  PHICHDuration,
                            ViInt32*  PHICHResource);

ViStatus _VI_FUNC rsspecan_ReadLTEDownlinkChannelDecoderResultsPCFIC(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   arraySize,
                            ViInt32   subframeNumber[],
                            ViInt32   numberOfSymbols[],
                            ViInt32*  noOfValues);

ViStatus _VI_FUNC rsspecan_ReadLTEDownlinkChannelDecoderResultsPHICH(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   arraySize,
                            ViInt32   subframeNumber[],
                            ViInt32   ACKNACK[],
                            ViReal64  relativePower[],
                            ViInt32*  noOfValues);

ViStatus _VI_FUNC rsspecan_ReadLTEDownlinkChannelDecoderResultsPDCCH(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   arraySize,
                            ViInt32   bitStreamSize,
                            ViInt32   subframeNumber[],
                            ViInt32   RNTI[],
                            ViInt32   DCIFormat[],
                            ViInt32   PDCCHFormat[],
                            ViInt32   CCEOffset[],
                            ViInt32   numberOfTransmittedBits[],
                            ViChar    bitStreams[],
                            ViInt32*  noOfValues);

ViStatus _VI_FUNC rsspecan_ReadLTEDownlinkChannelDecoderResultsPDSCH(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   arraySize,
                            ViInt32   bitStreamSize,
                            ViInt32   subframeNumber[],
                            ViInt32   allocationID[],
                            ViInt32   codeword[],
                            ViInt32   numberOfTransmittedBits[],
                            ViChar    bitStreams[],
                            ViInt32*  noOfValues);

ViStatus _VI_FUNC rsspecan_ConfigureLTEDownlinkMeasurementSubframe(
                            ViSession instrSession,
                            ViInt32   componentCarrier,
                            ViInt32   subframe);

ViStatus _VI_FUNC rsspecan_ConfigureLTEDownlinkMeasurementSubframeAll(
                            ViSession instrSession,
                            ViInt32   componentCarrier);

ViStatus _VI_FUNC rsspecan_ConfigureLTEDownlinkMeasurementConstelllationModulation(
                            ViSession instrSession,
                            ViInt32   componentCarrier,
                            ViInt32   modulation);

ViStatus _VI_FUNC rsspecan_ConfigureLTEDownlinkMeasurementConstelllationModulationAll(
                            ViSession instrSession,
                            ViInt32   componentCarrier);

ViStatus _VI_FUNC rsspecan_ConfigureLTEDownlinkMeasurementConstelllationAllocation(
                            ViSession instrSession,
                            ViInt32   componentCarrier,
                            ViInt32   allocation);

ViStatus _VI_FUNC rsspecan_ConfigureLTEDownlinkMeasurementConstelllationAllocationAll(
                            ViSession instrSession,
                            ViInt32   componentCarrier);

ViStatus _VI_FUNC rsspecan_ConfigureLTEDownlinkMeasurementConstelllationOFDMSymbol(
                            ViSession instrSession,
                            ViInt32   componentCarrier,
                            ViInt32   symbol_OFDM);

ViStatus _VI_FUNC rsspecan_ConfigureLTEDownlinkMeasurementConstelllationOFDMSymbolAll(
                            ViSession instrSession,
                            ViInt32   componentCarrier);

ViStatus _VI_FUNC rsspecan_ConfigureLTEDownlinkMeasurementConstelllationCarrier(
                            ViSession instrSession,
                            ViInt32   componentCarrier,
                            ViInt32   carrier);

ViStatus _VI_FUNC rsspecan_ConfigureLTEDownlinkMeasurementConstelllationCarrierAll(
                            ViSession instrSession,
                            ViInt32   componentCarrier);

ViStatus _VI_FUNC rsspecan_ConfigureLTEDownlinkMeasurementConstelllationLocation(
                            ViSession instrSession,
                            ViInt32   componentCarrier,
                            ViInt32   location);

ViStatus _VI_FUNC rsspecan_ConfigureLTEDownlinkMeasurementAntennaPort(
                            ViSession instrSession,
                            ViInt32   componentCarrier,
                            ViInt32   port);

ViStatus _VI_FUNC rsspecan_ConfigureLTEDownlinkMeasurementAntennaPortCellReferenceSignal(
                            ViSession instrSession,
                            ViInt32   componentCarrier,
                            ViInt32   port);

ViStatus _VI_FUNC rsspecan_ConfigureLTEDownlinkMeasurementAntennaPortCSIReferenceSignal(
                            ViSession instrSession,
                            ViInt32   componentCarrier,
                            ViInt32   port);

ViStatus _VI_FUNC rsspecan_QueryLTEDownlinkCCDFResults(ViSession instrSession,
                                                       ViInt32   trace,
                                                       ViInt32   probability,
                                                       ViReal64* result);

ViStatus _VI_FUNC rsspecan_QueryLTEDownlinkCCDFStatisticalResults(
                            ViSession instrSession,
                            ViInt32   trace,
                            ViInt32   resultType,
                            ViReal64* result);

ViStatus _VI_FUNC rsspecan_QueryLTEDownlinkCCDFStatisticalAllResults(
                            ViSession instrSession,
                            ViInt32   trace,
                            ViInt32   arraySize,
                            ViReal64  results[]);

ViStatus _VI_FUNC rsspecan_QueryLTEDownlinkMeasurementTimeAlignmentError(
                            ViSession instrSession,
                            ViInt32   componentCarrier,
                            ViInt32   antenna,
                            ViInt32   frameResult,
                            ViInt32   measurementType,
                            ViReal64* result);

ViStatus _VI_FUNC rsspecan_QueryLTEDownlinkOffPeriodsLimitCheckResult(
                            ViSession instrSession,
                            ViInt32   arraySize,
                            ViBoolean results[],
                            ViInt32*  returnedValues);

ViStatus _VI_FUNC rsspecan_QueryLTEDownlinkTransientLimitCheckResult(
                            ViSession instrSession,
                            ViInt32   type,
                            ViInt32   arraySize,
                            ViBoolean results[],
                            ViInt32*  returnedValues);

ViStatus _VI_FUNC rsspecan_QueryLTEDownlinkACPChannelLimitCheckResults(
                            ViSession instrSession,
                            ViInt32   channelType,
                            ViInt32   limitCheckType,
                            ViInt32   results[]);

ViStatus _VI_FUNC rsspecan_QueryLTEDownlinkMeasurementPowerResult(
                            ViSession instrSession,
                            ViInt32   powerMeasurement,
                            ViInt32   arraySize,
                            ViReal64  results[],
                            ViInt32*  returnedValues);

ViStatus _VI_FUNC rsspecan_QueryLTEDownlinkMarkerZAxis(ViSession instrSession,
                                                       ViReal64* zAxis);

ViStatus _VI_FUNC rsspecan_QueryLTEDownlinkMarkerZAxisAll(ViSession instrSession,
                                                          ViInt32   arraySize,
                                                          ViReal64  zAxis[]);

ViStatus _VI_FUNC rsspecan_SetLTEUplinkMode(ViSession instrSession);

ViStatus _VI_FUNC rsspecan_ConfigureLTEUplinkNumberOfCarriers(
                            ViSession instrSession,
                            ViInt32   numberOfCarriers);

ViStatus _VI_FUNC rsspecan_ConfigureLTEUplinkLocalOscillatorLocation(
                            ViSession instrSession,
                            ViInt32   location);

ViStatus _VI_FUNC rsspecan_ConfigureLTEUplinkLocalOscillatorFrequency(
                            ViSession instrSession,
                            ViReal64  frequency);

ViStatus _VI_FUNC rsspecan_ConfigureLTEUplinkSignalCharacteristics(
                            ViSession instrSession,
                            ViInt32   componentCarrier,
                            ViInt32   duplexing,
                            ViReal64  frequency,
                            ViInt32   bandwidthSettingType,
                            ViInt32   channelBandwidth,
                            ViInt32   numberOfResourceBlocks,
                            ViInt32   cyclicPrefix);

ViStatus _VI_FUNC rsspecan_ConfigureLTEUplinkTDDFrames(ViSession instrSession,
                                                       ViInt32   componentCarrier,
                                                       ViInt32   TDDULDLAllocations,
                                                       ViInt32   specialSubframe);

ViStatus _VI_FUNC rsspecan_ConfigureLTEUplinkPhysicalLayerCellIdentity(
                            ViSession instrSession,
                            ViInt32   componentCarrier,
                            ViInt32   cellIdentityGroup,
                            ViInt32   identity);

ViStatus _VI_FUNC rsspecan_QueryLTEUplinkPhysicalLayerCellIdentity(
                            ViSession instrSession,
                            ViInt32   componentCarrier,
                            ViInt32*  cellIdentityGroup,
                            ViInt32*  cellIdentity);

ViStatus _VI_FUNC rsspecan_ConfigureLTEUplinkSpectrumFlatness(
                            ViSession instrSession,
                            ViInt32   componentCarrier,
                            ViBoolean extremeConditions,
                            ViInt32   operatingBandIndex);

ViStatus _VI_FUNC rsspecan_ConfigureLTEUplinkCarrierAgregationBandwidth(
                            ViSession instrSession,
                            ViInt32   channelBandwidth);

ViStatus _VI_FUNC rsspecan_ConfigureLTEUplinkCarrierConfiguration(
                            ViSession instrSession,
                            ViInt32   componentCarrier,
                            ViReal64  frequency,
                            ViReal64  frequencyOffset,
                            ViInt32   channelBandwidth);

ViStatus _VI_FUNC rsspecan_StoreLTEUplinkDemodulationSettings(
                            ViSession instrSession,
                            ViInt32   componentCarrier,
                            ViString  filename);

ViStatus _VI_FUNC rsspecan_ConfigureLTEUplinkMIMO(ViSession instrSession,
                                                  ViInt32   componentCarrier,
                                                  ViInt32   configuration,
                                                  ViInt32   antennaSelection);

ViStatus _VI_FUNC rsspecan_ConfigureLTEUplinkSubframes(ViSession instrSession,
                                                       ViInt32   autoDemodulation,
                                                       ViBoolean subframeConfDetection);

ViStatus _VI_FUNC rsspecan_ConfigureLTEUplinkConfigurableSubframes(
                            ViSession instrSession,
                            ViInt32   componentCarrier,
                            ViInt32   configurableSubframes);

ViStatus _VI_FUNC rsspecan_ConfigureLTEUplinkSubframeTable(ViSession instrSession,
                                                           ViInt32   componentCarrier,
                                                           ViInt32   subframe,
                                                           ViInt32   enableAllocation,
                                                           ViInt32   modulation,
                                                           ViInt32   cluster,
                                                           ViInt32   resourceBlocks,
                                                           ViInt32   resourceBlocksOffset);

ViStatus _VI_FUNC rsspecan_ConfigureLTEUplinkGlobalSettings(ViSession instrSession,
                                                            ViInt32   componentCarrier,
                                                            ViInt32   frameNumberOffset,
                                                            ViInt32   UEIDn_RNTI);

ViStatus _VI_FUNC rsspecan_ConfigureLTEUplinkReferenceSignal(ViSession instrSession,
                                                             ViInt32   componentCarrier,
                                                             ViReal64  relativePowerPUSCH,
                                                             ViReal64  relativePowerPUCCH,
                                                             ViBoolean groupHopping,
                                                             ViBoolean sequenceHopping,
                                                             ViInt32   deltaSequenceShift,
                                                             ViInt32   n_DMRS,
                                                             ViBoolean activateDMRSWithOCC);

ViStatus _VI_FUNC rsspecan_ConfigureLTEUplinkDRSCellID(ViSession instrSession,
                                                       ViInt32   componentCarrier,
                                                       ViInt32   cellID);

ViStatus _VI_FUNC rsspecan_ConfigureLTEUplinkDRSCellIDValue(ViSession instrSession,
                                                            ViInt32   componentCarrier,
                                                            ViInt32   cellIDValue);

ViStatus _VI_FUNC rsspecan_ConfigureLTEUplinkSoundingReferenceSignal(
                            ViSession instrSession,
                            ViInt32   componentCarrier,
                            ViBoolean present,
                            ViInt32   subframeConfiguration,
                            ViInt32   bandwidthB_SRS,
                            ViInt32   hoppingBWB_hop,
                            ViInt32   cyclixShiftN_CS,
                            ViReal64  relativePower,
                            ViInt32   BWConfC_SRS,
                            ViInt32   confIndexI_SRS,
                            ViInt32   transmissionCombK_TC,
                            ViInt32   frequencyDomainPosition,
                            ViBoolean ANSimultaneousTX);

ViStatus _VI_FUNC rsspecan_ConfigureLTEUplinkSoundingReferenceSignalState(
                            ViSession instrSession,
                            ViInt32   componentCarrier,
                            ViBoolean maxUpPts);

ViStatus _VI_FUNC rsspecan_ConfigureLTEUplinkPUSCHStructure(ViSession instrSession,
                                                            ViInt32   componentCarrier,
                                                            ViInt32   frequencyHoppingMode,
                                                            ViInt32   numberOfSubbands,
                                                            ViInt32   hoppingOffset,
                                                            ViInt32   infoInHoppingBits);

ViStatus _VI_FUNC rsspecan_ConfigureLTEUplinkPUSCHCellID(ViSession instrSession,
                                                         ViInt32   componentCarrier,
                                                         ViInt32   cellID);

ViStatus _VI_FUNC rsspecan_ConfigureLTEUplinkPUSCHCellIDValue(
                            ViSession instrSession,
                            ViInt32   componentCarrier,
                            ViInt32   cellIDValue);

ViStatus _VI_FUNC rsspecan_ConfigureLTEUplinkPUCCHStructure(ViSession instrSession,
                                                            ViInt32   componentCarrier,
                                                            ViInt32   numberOfRBsForPUCCH,
                                                            ViInt32   deltaShift,
                                                            ViInt32   n1_cs,
                                                            ViInt32   n2_RB,
                                                            ViInt32   format,
                                                            ViInt32   n_PUCCH);

ViStatus _VI_FUNC rsspecan_ConfigureLTEUplinkPUCCHCellID(ViSession instrSession,
                                                         ViInt32   componentCarrier,
                                                         ViInt32   cellID);

ViStatus _VI_FUNC rsspecan_ConfigureLTEUplinkPUCCHCellIDValue(
                            ViSession instrSession,
                            ViInt32   componentCarrier,
                            ViInt32   cellIDValue);

ViStatus _VI_FUNC rsspecan_ConfigureLTEUplinkPRACHStructure(ViSession instrSession,
                                                            ViInt32   componentCarrier,
                                                            ViInt32   PRACHConfiguration,
                                                            ViBoolean restrictedSet,
                                                            ViInt32   frequencyOffset,
                                                            ViInt32   ncsConfiguration,
                                                            ViInt32   logicalRootSequenceIndex,
                                                            ViInt32   sequenceIndex,
                                                            ViInt32   sequenceIndexValue);

ViStatus _VI_FUNC rsspecan_ConfigureLTEUplinkPRACHPreambleMapping(
                            ViSession instrSession,
                            ViInt32   componentCarrier,
                            ViBoolean automaticPreambleMapping,
                            ViInt32   frequencyIndex,
                            ViInt32   halfFrameIndicator);

ViStatus _VI_FUNC rsspecan_ConfigureLTEUplinkPUCCHResourceBlocksAuto(
                            ViSession instrSession,
                            ViInt32   componentCarrier);

ViStatus _VI_FUNC rsspecan_ConfigureLTEUplinkSignalCapture(ViSession instrSession,
                                                           ViReal64  captureTime,
                                                           ViBoolean overallFrameCount,
                                                           ViBoolean autoAccordingToStandard,
                                                           ViInt32   numberOfFramesToAnalyzer,
                                                           ViBoolean swapIQ);

ViStatus _VI_FUNC rsspecan_ConfigureLTEUplinkTracking(ViSession instrSession,
                                                      ViInt32   phaseTracking,
                                                      ViBoolean timing);

ViStatus _VI_FUNC rsspecan_ConfigureLTEUplinkDemodulation(ViSession instrSession,
                                                          ViInt32   analysisMode,
                                                          ViInt32   channelEstimation,
                                                          ViBoolean EVMWithExclusionPeriod,
                                                          ViBoolean reserved,
                                                          ViBoolean scramblingOfCodedBits,
                                                          ViBoolean suppressedInterferenceSync,
                                                          ViBoolean multicarrierFilter);

ViStatus _VI_FUNC rsspecan_ConfigureLTEUplinkTransientSlots(ViSession instrSession,
                                                            ViBoolean transientSlots);

ViStatus _VI_FUNC rsspecan_ConfigureLTEUplinkCarrierAggregation(
                            ViSession instrSession,
                            ViBoolean carrierAggregationState,
                            ViInt32   numberOfCarriers);

ViStatus _VI_FUNC rsspecan_ConfigureLTEUplinkFrequencySweepMeasurements(
                            ViSession instrSession,
                            ViInt32   assumedAdjChannelCarrier,
                            ViInt32   SEMRequirement);

ViStatus _VI_FUNC rsspecan_LTEUplinkAddWindow(ViSession instrSession,
                                              ViInt32   stringSize,
                                              ViString  referenceWindowName,
                                              ViInt32   direction,
                                              ViInt32   windowType,
                                              ViChar    returnedWindowName[]);

ViStatus _VI_FUNC rsspecan_LTEUplinkReplaceWindow(ViSession instrSession,
                                                  ViString  windowName,
                                                  ViInt32   windowType);

ViStatus _VI_FUNC rsspecan_LTEUplinkSelectTab(ViSession instrSession,
                                              ViInt32   window,
                                              ViInt32   tab);

ViStatus _VI_FUNC rsspecan_ConfigureLTEUplinkMeasurement(ViSession instrSession,
                                                         ViInt32   measurement);

ViStatus _VI_FUNC rsspecan_ConfigureLTEUplinkMeasurementSignalDetection(ViSession instrSession);

ViStatus _VI_FUNC rsspecan_QueryLTEUplinkMeasurementSynchronizationState(
                            ViSession  instrSession,
                            ViInt32    componentCarrier,
                            ViBoolean* synchronization);

ViStatus _VI_FUNC rsspecan_QueryLTEUplinkMeasurementResultSummary(
                            ViSession instrSession,
                            ViInt32   componentCarrier,
                            ViInt32   frameResult,
                            ViInt32   measurementType,
                            ViReal64* result);

ViStatus _VI_FUNC rsspecan_QueryLTEUplinkMeasurementLimitCheckResult(
                            ViSession instrSession,
                            ViInt32   componentCarrier,
                            ViInt32   frameResult,
                            ViInt32*  average);

ViStatus _VI_FUNC rsspecan_QueryLTEUplinkMeasurementEVMAll(ViSession instrSession,
                                                           ViInt32   componentCarrier,
                                                           ViInt32   measurementType,
                                                           ViReal64* result);

ViStatus _VI_FUNC rsspecan_QueryLTEUplinkMeasurementLimitCheckEVMAll(
                            ViSession instrSession,
                            ViInt32   componentCarrier,
                            ViInt32*  average,
                            ViInt32*  maximum);

ViStatus _VI_FUNC rsspecan_QueryLTEUplinkMeasurementEVMPhysicalChannel(
                            ViSession instrSession,
                            ViInt32   componentCarrier,
                            ViInt32   measurementType,
                            ViReal64* result);

ViStatus _VI_FUNC rsspecan_QueryLTEUplinkMeasurementEVMPhysicalChannelLimitCheck(
                            ViSession instrSession,
                            ViInt32   componentCarrier,
                            ViInt32*  average,
                            ViInt32*  maximum);

ViStatus _VI_FUNC rsspecan_QueryLTEUplinkMeasurementEVMPhysicalSignal(
                            ViSession instrSession,
                            ViInt32   componentCarrier,
                            ViInt32   measurementType,
                            ViReal64* result);

ViStatus _VI_FUNC rsspecan_QueryLTEUplinkMeasurementEVMPhysicalSignalLimitCheck(
                            ViSession instrSession,
                            ViInt32   componentCarrier,
                            ViInt32*  average,
                            ViInt32*  maximum);

ViStatus _VI_FUNC rsspecan_QueryLTEUplinkMeasurementFrequencyError(
                            ViSession instrSession,
                            ViInt32   componentCarrier,
                            ViInt32   measurementType,
                            ViReal64* result);

ViStatus _VI_FUNC rsspecan_QueryLTEUplinkMeasurementFrequencyErrorLimitCheck(
                            ViSession instrSession,
                            ViInt32   componentCarrier,
                            ViInt32*  average,
                            ViInt32*  maximum);

ViStatus _VI_FUNC rsspecan_QueryLTEUplinkMeasurementSamplingError(
                            ViSession instrSession,
                            ViInt32   componentCarrier,
                            ViInt32   measurementType,
                            ViReal64* result);

ViStatus _VI_FUNC rsspecan_QueryLTEUplinkMeasurementCyclicPrefix(
                            ViSession instrSession,
                            ViInt32   componentCarrier,
                            ViInt32*  cyclicPrefix);

ViStatus _VI_FUNC rsspecan_QueryLTEUplinkMeasurementSamplingErrorLimitCheck(
                            ViSession instrSession,
                            ViInt32   componentCarrier,
                            ViInt32*  average,
                            ViInt32*  maximum);

ViStatus _VI_FUNC rsspecan_QueryLTEUplinkMeasurementIQOffset(ViSession instrSession,
                                                             ViInt32   componentCarrier,
                                                             ViInt32   measurementType,
                                                             ViReal64* result);

ViStatus _VI_FUNC rsspecan_QueryLTEUplinkMeasurementIQOffsetLimitCheck(
                            ViSession instrSession,
                            ViInt32   componentCarrier,
                            ViInt32*  average,
                            ViInt32*  maximum);

ViStatus _VI_FUNC rsspecan_QueryLTEUplinkMeasurementIQGainImbalance(
                            ViSession instrSession,
                            ViInt32   componentCarrier,
                            ViInt32   measurementType,
                            ViReal64* result);

ViStatus _VI_FUNC rsspecan_QueryLTEUplinkMeasurementIQGainImbalanceLimitCheck(
                            ViSession instrSession,
                            ViInt32   componentCarrier,
                            ViInt32*  average,
                            ViInt32*  maximum);

ViStatus _VI_FUNC rsspecan_QueryLTEUplinkMeasurementIQQuadratureError(
                            ViSession instrSession,
                            ViInt32   componentCarrier,
                            ViInt32   measurementType,
                            ViReal64* result);

ViStatus _VI_FUNC rsspecan_QueryLTEUplinkMeasurementIQQuadratureErrorLimitCheck(
                            ViSession instrSession,
                            ViInt32   componentCarrier,
                            ViInt32*  average,
                            ViInt32*  maximum);

ViStatus _VI_FUNC rsspecan_QueryLTEUplinkMeasurementFramePower(
                            ViSession instrSession,
                            ViInt32   componentCarrier,
                            ViInt32   measurementType,
                            ViReal64* result);

ViStatus _VI_FUNC rsspecan_QueryLTEUplinkMeasurementCrestFactor(
                            ViSession instrSession,
                            ViInt32   componentCarrier,
                            ViInt32   measurementType,
                            ViReal64* result);

ViStatus _VI_FUNC rsspecan_QueryLTEUplinkMeasurementFrameStartOffset(
                            ViSession instrSession,
                            ViInt32   componentCarrier,
                            ViReal64* result);

ViStatus _VI_FUNC rsspecan_ConfigureLTEUplinkMeasurementSubframe(
                            ViSession instrSession,
                            ViInt32   componentCarrier,
                            ViInt32   subframe);

ViStatus _VI_FUNC rsspecan_ConfigureLTEUplinkMeasurementSubframeAll(
                            ViSession instrSession,
                            ViInt32   componentCarrier);

ViStatus _VI_FUNC rsspecan_ConfigureLTEUplinkMeasurementSlot(ViSession instrSession,
                                                             ViInt32   componentCarrier,
                                                             ViInt32   slot);

ViStatus _VI_FUNC rsspecan_ConfigureLTEUplinkMeasurementConstelllationModulation(
                            ViSession instrSession,
                            ViInt32   componentCarrier,
                            ViInt32   modulation);

ViStatus _VI_FUNC rsspecan_ConfigureLTEUplinkMeasurementConstelllationModulationAll(
                            ViSession instrSession,
                            ViInt32   componentCarrier);

ViStatus _VI_FUNC rsspecan_ConfigureLTEUplinkMeasurementConstelllationAllocation(
                            ViSession instrSession,
                            ViInt32   componentCarrier,
                            ViInt32   allocation);

ViStatus _VI_FUNC rsspecan_ConfigureLTEUplinkMeasurementConstelllationAllocationAll(
                            ViSession instrSession,
                            ViInt32   componentCarrier);

ViStatus _VI_FUNC rsspecan_ConfigureLTEUplinkMeasurementConstelllationOFDMSymbol(
                            ViSession instrSession,
                            ViInt32   componentCarrier,
                            ViInt32   symbol_OFDM);

ViStatus _VI_FUNC rsspecan_ConfigureLTEUplinkMeasurementConstelllationOFDMSymbolAll(
                            ViSession instrSession,
                            ViInt32   componentCarrier);

ViStatus _VI_FUNC rsspecan_ConfigureLTEUplinkMeasurementConstelllationCarrier(
                            ViSession instrSession,
                            ViInt32   componentCarrier,
                            ViInt32   carrier);

ViStatus _VI_FUNC rsspecan_ConfigureLTEUplinkMeasurementConstelllationCarrierAll(
                            ViSession instrSession,
                            ViInt32   componentCarrier);

ViStatus _VI_FUNC rsspecan_QueryLTEUplinkCCDFResults(ViSession instrSession,
                                                     ViInt32   trace,
                                                     ViInt32   probability,
                                                     ViReal64* result);

ViStatus _VI_FUNC rsspecan_QueryLTEUplinkCCDFStatisticalResults(
                            ViSession instrSession,
                            ViInt32   trace,
                            ViInt32   resultType,
                            ViReal64* result);

ViStatus _VI_FUNC rsspecan_QueryLTEUplinkCCDFStatisticalAllResults(
                            ViSession instrSession,
                            ViInt32   trace,
                            ViInt32   arraySize,
                            ViReal64  results[]);

ViStatus _VI_FUNC rsspecan_QueryLTEUplinkMeasurementTimeAlignmentError(
                            ViSession instrSession,
                            ViInt32   componentCarrier,
                            ViInt32   antenna,
                            ViInt32   frameResult,
                            ViInt32   measurementType,
                            ViReal64* result);

ViStatus _VI_FUNC rsspecan_QueryLTEUplinkACPChannelLimitCheckResults(
                            ViSession instrSession,
                            ViInt32   channelType,
                            ViInt32   limitCheckType,
                            ViInt32   results[]);

ViStatus _VI_FUNC rsspecan_QueryLTEUplinkMeasurementPowerResult(
                            ViSession instrSession,
                            ViInt32   powerMeasurement,
                            ViInt32   arraySize,
                            ViReal64  results[],
                            ViInt32*  returnedValues);

ViStatus _VI_FUNC rsspecan_QueryLTEUplinkMarkerZAxis(ViSession instrSession,
                                                     ViReal64* zAxis);

ViStatus _VI_FUNC rsspecan_QueryLTEUplinkMarkerZAxisAll(ViSession instrSession,
                                                        ViInt32   arraySize,
                                                        ViReal64  zAxis[]);

ViStatus _VI_FUNC rsspecan_SetNBIoTDownlinkMode(ViSession instrSession);

ViStatus _VI_FUNC rsspecan_ConfigureNBIoTDownlinkMIMO(ViSession instrSession,
                                                      ViInt32   DUT,
                                                      ViInt32   antenna);

ViStatus _VI_FUNC rsspecan_NBIoTDownlinkPreset(ViSession instrSession);

ViStatus _VI_FUNC rsspecan_StoreNBIoTDemodulationSettings(ViSession instrSession,
                                                          ViString  filename);

ViStatus _VI_FUNC rsspecan_ConfigureNBIoTDownlinkTestModel(ViSession instrSession,
                                                           ViInt32   componentCarrier,
                                                           ViString  testModel);

ViStatus _VI_FUNC rsspecan_NBIoTDownlinkLoadDemodulationSettings(
                            ViSession instrSession,
                            ViString  fileName);

ViStatus _VI_FUNC rsspecan_NBIoTDownlinkLoadIQ(ViSession instrSession,
                                               ViString  fileName);

ViStatus _VI_FUNC rsspecan_ConfigureNBIoTDownlinkWindowSelect(
                            ViSession instrSession,
                            ViInt32   window);

ViStatus _VI_FUNC rsspecan_ConfigureNBIoTDownlinkDeployment(ViSession instrSession,
                                                            ViInt32   deployment);

ViStatus _VI_FUNC rsspecan_ConfigureNBIoTDownlinkCarrierType(ViSession instrSession,
                                                             ViInt32   type);

ViStatus _VI_FUNC rsspecan_ConfigureNBIoTDownlinkInBand(ViSession instrSession,
                                                        ViReal64  frequency,
                                                        ViInt32   bandwidth,
                                                        ViInt32   CRSSequence,
                                                        ViInt32   PRBIndex);

ViStatus _VI_FUNC rsspecan_ConfigureNBIoTDownlinkStandAlone(ViSession instrSession,
                                                            ViReal64  frequency);

ViStatus _VI_FUNC rsspecan_ConfigureNBIoTDownlinkGuardband(ViSession instrSession,
                                                           ViInt32   carrierLocation);

ViStatus _VI_FUNC rsspecan_ConfigureNBIoTDownlinkGuardbandUser(
                            ViSession instrSession,
                            ViReal64  customLocation);

ViStatus _VI_FUNC rsspecan_ConfigureNBIoTDownlinkLevelOnce(ViSession instrSession);

ViStatus _VI_FUNC rsspecan_ConfigureNBIoTDownlinkLevelReference(
                            ViSession instrSession,
                            ViBoolean autoRef,
                            ViReal64  level);

ViStatus _VI_FUNC rsspecan_ConfigureNBIoTDownlinkLevelReferenceBaseband(
                            ViSession instrSession,
                            ViReal64  level);

ViStatus _VI_FUNC rsspecan_ConfigureNBIoTDownlinkLevelExternalAttenuation(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   trace,
                            ViReal64  level);

ViStatus _VI_FUNC rsspecan_ConfigureNBIoTDownlinkSwapIQ(ViSession instrSession,
                                                        ViBoolean swapIQ);

ViStatus _VI_FUNC rsspecan_ConfigureNBIoTDownlinkAutoLevelTrackTime(
                            ViSession instrSession,
                            ViReal64  autoLevelTrackTime);

ViStatus _VI_FUNC rsspecan_ConfigureNBIoTDownlinkFrame(ViSession instrSession,
                                                       ViBoolean enabled,
                                                       ViBoolean autoCount,
                                                       ViInt32   count);

ViStatus _VI_FUNC rsspecan_ConfigureNBIoTDownlinkFilter(ViSession instrSession,
                                                        ViInt32   type);

ViStatus _VI_FUNC rsspecan_ConfigureNBIoTDownlinkYIGFilter(ViSession instrSession,
                                                           ViBoolean enabled,
                                                           ViBoolean autoFilter);

ViStatus _VI_FUNC rsspecan_ConfigureNBIoTDownlinkNPDSCH(ViSession instrSession,
                                                        ViBoolean enabled,
                                                        ViString  subframes,
                                                        ViInt32   UEID);

ViStatus _VI_FUNC rsspecan_ConfigureNBIoTDownlinkDataAnalysis(
                            ViSession instrSession,
                            ViInt32   channelEstimation,
                            ViInt32   EVMCalculationMethod,
                            ViInt32   NPDSCHReferenceData,
                            ViBoolean multicarrierFilter);

ViStatus _VI_FUNC rsspecan_ConfigureNBIoTDownlinkCompensateCrosstalk(
                            ViSession instrSession,
                            ViBoolean compensateCrosstalk);

ViStatus _VI_FUNC rsspecan_ConfigureNBIoTDownlinkTracking(ViSession instrSession,
                                                          ViInt32   phase,
                                                          ViBoolean timing);

ViStatus _VI_FUNC rsspecan_ConfigureNBIoTDownlinkPhysicalLayerCellIdentity(
                            ViSession instrSession,
                            ViBoolean autoIdentity,
                            ViInt32   cellID,
                            ViInt32   cellIDGroup,
                            ViInt32   identity);

ViStatus _VI_FUNC rsspecan_ConfigureNBIoTDownlinkScramblingofCodedBits(
                            ViSession instrSession,
                            ViBoolean scrambling);

ViStatus _VI_FUNC rsspecan_ConfigureNBIoTDownlinkSEM(ViSession instrSession,
                                                     ViInt32   category,
                                                     ViBoolean txPowerEnabled,
                                                     ViReal64  txPower,
                                                     ViInt32   carrierPowerMode,
                                                     ViReal64  carrierPower);

ViStatus _VI_FUNC rsspecan_NBIoTDownlinkAddWindow(ViSession instrSession,
                                                  ViInt32   stringSize,
                                                  ViString  referenceWindowName,
                                                  ViInt32   direction,
                                                  ViInt32   windowType,
                                                  ViChar    returnedWindowName[]);

ViStatus _VI_FUNC rsspecan_NBIoTDownlinkReplaceWindow(ViSession instrSession,
                                                      ViString  windowName,
                                                      ViInt32   windowType);

ViStatus _VI_FUNC rsspecan_NBIoTDownlinkSelectTab(ViSession instrSession,
                                                  ViInt32   window,
                                                  ViInt32   tab);

ViStatus _VI_FUNC rsspecan_ConfigureNBIoTDownlinkMeasurementMode(
                            ViSession instrSession,
                            ViInt32   measurement);

ViStatus _VI_FUNC rsspecan_ConfigureNBIoTDownlinkAnalysisUnit(
                            ViSession instrSession,
                            ViInt32   EVMUnit);

ViStatus _VI_FUNC rsspecan_ConfigureNBIoTDownlinkAnalysisResultTableEnabled(
                            ViSession instrSession,
                            ViBoolean resultTable);

ViStatus _VI_FUNC rsspecan_QueryNBIoTDownlinkAnalysisSynchronizationState(
                            ViSession instrSession,
                            ViChar    state[]);

ViStatus _VI_FUNC rsspecan_ConfigureNBIoTDownlinkAnalysisMarkerSearch(
                            ViSession instrSession,
                            ViInt32   marker,
                            ViInt32   searchType);

ViStatus _VI_FUNC rsspecan_ConfigureNBIoTDownlinkAnalysisYScale(
                            ViSession instrSession,
                            ViBoolean yAuto,
                            ViReal64  pointOfOrigin,
                            ViReal64  distanceBetweenGrid);

ViStatus _VI_FUNC rsspecan_ConfigureNBIoTDownlinkAnalysisYAxisScaling(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   subwindow,
                            ViInt32   type,
                            ViReal64  maxMinValue);

ViStatus _VI_FUNC rsspecan_ConfigureNBIoTDownlinkMeasurement(ViSession instrSession,
                                                             ViInt32   window,
                                                             ViInt32   measurement);

ViStatus _VI_FUNC rsspecan_ConfigureNBIoTDownlinkBitStreamFormat(
                            ViSession instrSession,
                            ViInt32   bitStreamFormat);

ViStatus _VI_FUNC rsspecan_ConfigureNBIoTDownlinkAnalysisSelectAllocation(
                            ViSession instrSession,
                            ViInt32   select,
                            ViInt32   allocation);

ViStatus _VI_FUNC rsspecan_ConfigureNBIoTDownlinkAnalysisSelectCarrier(
                            ViSession instrSession,
                            ViInt32   select,
                            ViInt32   carrier);

ViStatus _VI_FUNC rsspecan_ConfigureNBIoTDownlinkAnalysisSelectModulation(
                            ViSession instrSession,
                            ViInt32   select,
                            ViInt32   modulation);

ViStatus _VI_FUNC rsspecan_ConfigureNBIoTDownlinkAnalysisSelectSymbol(
                            ViSession instrSession,
                            ViInt32   select,
                            ViInt32   symbol);

ViStatus _VI_FUNC rsspecan_ConfigureNBIoTDownlinkAnalysisLocationSelect(
                            ViSession instrSession,
                            ViInt32   select);

ViStatus _VI_FUNC rsspecan_ConfigureNBIoTDownlinkAnalysisSubframeSelectAll(ViSession instrSession);

ViStatus _VI_FUNC rsspecan_ConfigureNBIoTDownlinkAnalysisSubframeSelect(
                            ViSession instrSession,
                            ViInt32   subframe);

ViStatus _VI_FUNC rsspecan_QueryNBIoTDownlinkMeasurementEVMAll(
                            ViSession instrSession,
                            ViInt32   measurementType,
                            ViReal64* result);

ViStatus _VI_FUNC rsspecan_QueryNBIoTDownlinkMeasurementEVMPhysicalChannel(
                            ViSession instrSession,
                            ViInt32   measurementType,
                            ViReal64* result);

ViStatus _VI_FUNC rsspecan_QueryNBIoTDownlinkMeasurementEVMPhysicalSignal(
                            ViSession instrSession,
                            ViInt32   measurementType,
                            ViReal64* result);

ViStatus _VI_FUNC rsspecan_QueryNBIoTDownlinkMeasurementFrequencyError(
                            ViSession instrSession,
                            ViInt32   measurementType,
                            ViReal64* result);

ViStatus _VI_FUNC rsspecan_QueryNBIoTDownlinkMeasurementRSTP(ViSession instrSession,
                                                             ViInt32   measurementType,
                                                             ViReal64* result);

ViStatus _VI_FUNC rsspecan_QueryNBIoTDownlinkMeasurementOSTP(ViSession instrSession,
                                                             ViInt32   measurementType,
                                                             ViReal64* result);

ViStatus _VI_FUNC rsspecan_QueryNBIoTDownlinkMeasurementFramePower(
                            ViSession instrSession,
                            ViInt32   measurementType,
                            ViReal64* result);

ViStatus _VI_FUNC rsspecan_QueryNBIoTDownlinkMeasurementSamplingError(
                            ViSession instrSession,
                            ViInt32   measurementType,
                            ViReal64* result);

ViStatus _VI_FUNC rsspecan_QueryNBIoTDownlinkMeasurementTriggerToFrame(
                            ViSession instrSession,
                            ViReal64* result);

ViStatus _VI_FUNC rsspecan_QueryNBIoTDownlinkMeasurementCrestFactor(
                            ViSession instrSession,
                            ViReal64* result);

ViStatus _VI_FUNC rsspecan_FetchNBIoTDownlinkACLR(ViSession instrSession,
                                                  ViInt32   window,
                                                  ViInt32   arraySize,
                                                  ViReal64  results[],
                                                  ViInt32*  returnedValues);

ViStatus _VI_FUNC rsspecan_FetchNBIoTDownlinkAllocationIDvsSymbolxCarrier(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   arraySize,
                            ViInt32   results[],
                            ViInt32*  returnedValues);

ViStatus _VI_FUNC rsspecan_FetchNBIoTDownlinkAllocationSummary(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   arraySize,
                            ViInt32   subframe[],
                            ViInt32   allocationID[],
                            ViInt32   numberOfRB[],
                            ViReal64  relativePower[],
                            ViInt32   modulation[],
                            ViReal64  absolutePower[],
                            ViReal64  EVM[],
                            ViReal64  layerEVM[],
                            ViInt32*  returnedValues);

ViStatus _VI_FUNC rsspecan_FetchNBIoTDownlinkCaptureBuffer(ViSession instrSession,
                                                           ViInt32   window,
                                                           ViInt32   arraySize,
                                                           ViReal64  results[],
                                                           ViInt32*  returnedValues);

ViStatus _VI_FUNC rsspecan_FetchNBIoTDownlinkCCDF(ViSession instrSession,
                                                  ViInt32   window,
                                                  ViInt32   axis,
                                                  ViInt32   arraySize,
                                                  ViReal64  results[],
                                                  ViInt32*  returnedValues);

ViStatus _VI_FUNC rsspecan_FetchNBIoTDownlinkChannelAndSpectrumFlatness(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   statistical,
                            ViInt32   arraySize,
                            ViReal64  results[],
                            ViInt32*  returnedValues);

ViStatus _VI_FUNC rsspecan_FetchNBIoTDownlinkChannelAndSpectrumFlatnessDifference(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   statistical,
                            ViInt32   arraySize,
                            ViReal64  results[],
                            ViInt32*  returnedValues);

ViStatus _VI_FUNC rsspecan_FetchNBIoTDownlinkGroupDelay(ViSession instrSession,
                                                        ViInt32   window,
                                                        ViInt32   arraySize,
                                                        ViReal64  results[],
                                                        ViInt32*  returnedValues);

ViStatus _VI_FUNC rsspecan_FetchNBIoTDownlinkConstellation(ViSession instrSession,
                                                           ViInt32   window,
                                                           ViInt32   arraySize,
                                                           ViReal64  i[],
                                                           ViReal64  q[],
                                                           ViInt32*  returnedValues);

ViStatus _VI_FUNC rsspecan_FetchNBIoTDownlinkEVMvsCarrier(ViSession instrSession,
                                                          ViInt32   window,
                                                          ViInt32   statistical,
                                                          ViInt32   arraySize,
                                                          ViReal64  results[],
                                                          ViInt32*  returnedValues);

ViStatus _VI_FUNC rsspecan_FetchNBIoTDownlinkEVMvsSubframe(ViSession instrSession,
                                                           ViInt32   window,
                                                           ViInt32   arraySize,
                                                           ViReal64  results[],
                                                           ViInt32*  returnedValues);

ViStatus _VI_FUNC rsspecan_FetchNBIoTDownlinkEVMvsSymbol(ViSession instrSession,
                                                         ViInt32   window,
                                                         ViInt32   arraySize,
                                                         ViReal64  results[],
                                                         ViInt32*  returnedValues);

ViStatus _VI_FUNC rsspecan_FetchNBIoTDownlinkEVMvsSymbolxCarrier(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   arraySize,
                            ViReal64  results[],
                            ViInt32*  returnedValues);

ViStatus _VI_FUNC rsspecan_FetchNBIoTDownlinkFrequencyErrorvsSymbol(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   arraySize,
                            ViReal64  results[],
                            ViInt32*  returnedValues);

ViStatus _VI_FUNC rsspecan_FetchNBIoTDownlinkPowerSpectrum(ViSession instrSession,
                                                           ViInt32   window,
                                                           ViInt32   arraySize,
                                                           ViReal64  results[],
                                                           ViInt32*  returnedValues);

ViStatus _VI_FUNC rsspecan_FetchNBIoTDownlinkPowervsSymbolxCarrier(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   arraySize,
                            ViReal64  results[],
                            ViInt32*  returnedValues);

ViStatus _VI_FUNC rsspecan_FetchNBIoTDownlinkSEM(ViSession instrSession,
                                                 ViInt32   window,
                                                 ViInt32   arraySize,
                                                 ViReal64  results[],
                                                 ViInt32*  returnedValues);

ViStatus _VI_FUNC rsspecan_FetchNBIoTDownlinkSEMTable(ViSession instrSession,
                                                      ViInt32   window,
                                                      ViInt32   arraySize,
                                                      ViInt32   index[],
                                                      ViReal64  startFrequency[],
                                                      ViReal64  stopFrequency[],
                                                      ViReal64  RBW[],
                                                      ViReal64  limitFailFrequency[],
                                                      ViReal64  absolutePower[],
                                                      ViReal64  relativePower[],
                                                      ViReal64  limitDistance[],
                                                      ViInt32   limitCheckResult[],
                                                      ViReal64  reserved1[],
                                                      ViReal64  reserved2[],
                                                      ViInt32*  returnedValues);

ViStatus _VI_FUNC rsspecan_SetDOCSISMode(ViSession instrSession);

ViStatus _VI_FUNC rsspecan_ConfigureDOCSISOFDMChannelDescription(
                            ViSession instrSession,
                            ViReal64  OFDMSpectrumLocation,
                            ViInt32   nFFT,
                            ViInt32   cyclicPrefixCP,
                            ViInt32   rollOff,
                            ViInt32   timeInterleavingDepth,
                            ViInt32   NCPModulation);

ViStatus _VI_FUNC rsspecan_ConfigureDOCSISOFDMSpectrumLocation(
                            ViSession instrSession,
                            ViReal64  OFDMSpectrumLocation);

ViStatus _VI_FUNC rsspecan_ConfigureDOCSISStreamDirection(ViSession instrSession,
                                                          ViInt32   streamDirection);

ViStatus _VI_FUNC rsspecan_ConfigureDOCSISPLCStartIndexL(ViSession instrSession,
                                                         ViBoolean startIndexAuto,
                                                         ViInt32   startIndex);

ViStatus _VI_FUNC rsspecan_QueryDOCSISCPAESANumberOfEntries(ViSession instrSession,
                                                            ViInt32*  numberOfEntries);

ViStatus _VI_FUNC rsspecan_DeleteDOCSISCPAESATableRows(ViSession instrSession,
                                                       ViInt32   setIndex,
                                                       ViInt32   deleteRow);

ViStatus _VI_FUNC rsspecan_ConfigureDOCSISCPAESAType(ViSession instrSession,
                                                     ViInt32   setIndex,
                                                     ViInt32   type);

ViStatus _VI_FUNC rsspecan_ConfigureDOCSISCPAESASubcarrierStartStop(
                            ViSession instrSession,
                            ViInt32   setIndex,
                            ViInt32   start,
                            ViInt32   stop,
                            ViInt32   increment);

ViStatus _VI_FUNC rsspecan_ConfigureDOCSISCPAESASubcarrierSet(
                            ViSession instrSession,
                            ViInt32   setIndex,
                            ViInt32   arraySize,
                            ViInt32   set[]);

ViStatus _VI_FUNC rsspecan_ConfigureDOCSISAutoDetectionContinuousPilots(
                            ViSession instrSession,
                            ViInt32   continuousPilots);

ViStatus _VI_FUNC rsspecan_SelectDOCSISProfileList(ViSession instrSession,
                                                   ViInt32   profileList);

ViStatus _VI_FUNC rsspecan_QueryDOCSISProfileConfigurationNumberOfEntries(
                            ViSession instrSession,
                            ViInt32*  numberOfEntries);

ViStatus _VI_FUNC rsspecan_DeleteDOCSISProfileConfiguration(ViSession instrSession);

ViStatus _VI_FUNC rsspecan_QueryDOCSISProfileState(ViSession  instrSession,
                                                   ViBoolean* profileState);

ViStatus _VI_FUNC rsspecan_ConfigureDOCSISModulation(ViSession instrSession,
                                                     ViInt32   setIndex,
                                                     ViInt32   profileList);

ViStatus _VI_FUNC rsspecan_ConfigureDOCSISModulationSubcarrierStartStop(
                            ViSession instrSession,
                            ViInt32   setIndex,
                            ViInt32   start,
                            ViInt32   stop,
                            ViInt32   increment);

ViStatus _VI_FUNC rsspecan_ConfigureDOCSISModulationSubcarrierSet(
                            ViSession instrSession,
                            ViInt32   setIndex,
                            ViInt32   arraySize,
                            ViInt32   set[]);

ViStatus _VI_FUNC rsspecan_DeleteDOCSISModulationSubcarrierTableSet(
                            ViSession instrSession,
                            ViInt32   setIndex,
                            ViInt32   deleteRow);

ViStatus _VI_FUNC rsspecan_QueryDOCSISNextCodewordPointerNumberOfEntries(
                            ViSession instrSession,
                            ViInt32   setIndex,
                            ViInt32*  numberOfEntries);

ViStatus _VI_FUNC rsspecan_DeleteDOCSISNextCodewordPointerProfileRow(
                            ViSession instrSession,
                            ViInt32   setIndex,
                            ViInt32   deleteRow);

ViStatus _VI_FUNC rsspecan_ConfigureDOCSISNextCodewordPointerModulation(
                            ViSession instrSession,
                            ViInt32   setIndex,
                            ViInt32   modulation);

ViStatus _VI_FUNC rsspecan_ConfigureDOCSISNextCodewordPointerStartStop(
                            ViSession instrSession,
                            ViInt32   setIndex,
                            ViInt32   start,
                            ViInt32   stop,
                            ViInt32   increment);

ViStatus _VI_FUNC rsspecan_ConfigureDOCSISNextCodewordPointerSet(
                            ViSession instrSession,
                            ViInt32   setIndex,
                            ViInt32   arraySize,
                            ViInt32   set[]);

ViStatus _VI_FUNC rsspecan_QueryDOCSISCodewordNumberOfRows(ViSession instrSession,
                                                           ViInt32*  numberOfRows);

ViStatus _VI_FUNC rsspecan_DeleteDOCSISCodewordFrameTableRow(ViSession instrSession,
                                                             ViInt32   codewordIndex,
                                                             ViInt32   deleteRow);

ViStatus _VI_FUNC rsspecan_SelectDOCSISCodewordAssignProfile(ViSession instrSession,
                                                             ViInt32   codewordIndex,
                                                             ViInt32   assignProfile);

ViStatus _VI_FUNC rsspecan_ConfigureDOCSISCodewordSubcarrierFrame(
                            ViSession instrSession,
                            ViInt32   codewordIndex,
                            ViInt32   firstSubcarrier,
                            ViInt32   numberOfSubcarriers);

ViStatus _VI_FUNC rsspecan_ConfigureDOCSISCodewordSymbolFrame(
                            ViSession instrSession,
                            ViInt32   codewordIndex,
                            ViInt32   firstSymbol,
                            ViInt32   numberOfSymbols);

ViStatus _VI_FUNC rsspecan_ConfigureDOCSISCodewordDetectionNCPAuto(
                            ViSession instrSession,
                            ViInt32   NPCAuto);

ViStatus _VI_FUNC rsspecan_ConfigureDOCSISUpstreamSymbolsPerFrame(
                            ViSession instrSession,
                            ViInt32   startIndex);

ViStatus _VI_FUNC rsspecan_QueryDOCSISUpstreamESANumberOfEntries(
                            ViSession instrSession,
                            ViInt32*  numberOfEntries);

ViStatus _VI_FUNC rsspecan_DeleteDOCSISUpstreamESATableRow(ViSession instrSession,
                                                           ViInt32   setIndex,
                                                           ViInt32   deleteRow);

ViStatus _VI_FUNC rsspecan_QueryDOCSISUpstreamESAType(ViSession instrSession,
                                                      ViInt32   setIndex,
                                                      ViInt32*  type);

ViStatus _VI_FUNC rsspecan_ConfigureDOCSISUpstreamESASubcarrierStartStop(
                            ViSession instrSession,
                            ViInt32   setIndex,
                            ViInt32   start,
                            ViInt32   stop,
                            ViInt32   increment);

ViStatus _VI_FUNC rsspecan_ConfigureDOCSISUpstreamESASubcarrierSet(
                            ViSession instrSession,
                            ViInt32   setIndex,
                            ViInt32   arraySize,
                            ViInt32   set[]);

ViStatus _VI_FUNC rsspecan_ConfigureDOCSISUpstreamProfile(ViSession instrSession,
                                                          ViInt32   setIndex,
                                                          ViInt32   modulation,
                                                          ViInt32   pilotPattern,
                                                          ViInt32   numberOfMinislots);

ViStatus _VI_FUNC rsspecan_DeleteDOCSISUpstreamProfileTableRow(
                            ViSession instrSession,
                            ViInt32   setIndex,
                            ViInt32   deleteRow);

ViStatus _VI_FUNC rsspecan_ConfigureDOCSISInputDirectPath(ViSession instrSession,
                                                          ViInt32   path);

ViStatus _VI_FUNC rsspecan_QueryDOCSISNumberOfSamples(ViSession instrSession,
                                                      ViInt64*  numberOfSamples);

ViStatus _VI_FUNC rsspecan_ConfigureDOCSISFilterOutAdjacentChannels(
                            ViSession instrSession,
                            ViBoolean filterOutAdjacentChannels);

ViStatus _VI_FUNC rsspecan_ConfigureDOCSISPowerIntervalSearchEnabled(
                            ViSession instrSession,
                            ViBoolean enabled);

ViStatus _VI_FUNC rsspecan_ConfigureDOCSISChannelEstimation(ViSession instrSession,
                                                            ViInt32   channelEstimation);

ViStatus _VI_FUNC rsspecan_ConfigureDOCSISTracking(ViSession instrSession,
                                                   ViBoolean phaseTracking,
                                                   ViBoolean timeTracking);

ViStatus _VI_FUNC rsspecan_ConfigureDOCSISDecodedCodewords(ViSession instrSession,
                                                           ViBoolean decoded);

ViStatus _VI_FUNC rsspecan_ConfigureDOCSISDecodedBitstream(ViSession instrSession,
                                                           ViInt32   bitstream);

ViStatus _VI_FUNC rsspecan_ConfigureDOCSISUpstreamAutoDetection(
                            ViSession instrSession,
                            ViInt32   autoDetection);

ViStatus _VI_FUNC rsspecan_ConfigureDOCSISAnalyzeSingleFrame(ViSession instrSession,
                                                             ViBoolean specifiedFrame,
                                                             ViInt32   frame);

ViStatus _VI_FUNC rsspecan_ConfigureDOCSISFrameStatistic(ViSession instrSession,
                                                         ViBoolean frameStatisticCount,
                                                         ViInt32   numberOfFrame);

ViStatus _VI_FUNC rsspecan_ConfigureDOCSISLimitCheckState(ViSession instrSession,
                                                          ViBoolean state);

ViStatus _VI_FUNC rsspecan_ConfigureDOCSISExcludingSubcarriers(
                            ViSession instrSession,
                            ViBoolean autoState,
                            ViInt32   numberOfSubcarriers);

ViStatus _VI_FUNC rsspecan_ConfigureDOCSISExcludingUserSubcarriers(
                            ViSession instrSession,
                            ViInt32   arraySize,
                            ViInt32   subcarriers[]);

ViStatus _VI_FUNC rsspecan_ConfigureDOCSISDisplaySettings(ViSession instrSession,
                                                          ViInt32   object,
                                                          ViInt32   modulation);

ViStatus _VI_FUNC rsspecan_ConfigureDOCSISEvaluationRangeSubcarrier(
                            ViSession instrSession,
                            ViInt32   type,
                            ViInt32   diagramSubcarrier);

ViStatus _VI_FUNC rsspecan_ConfigureDOCSISEvaluationRangeSymbol(
                            ViSession instrSession,
                            ViInt32   type,
                            ViInt32   diagramSymbol);

ViStatus _VI_FUNC rsspecan_ConfigureDOCSISEvaluationRangeFastMode(
                            ViSession instrSession,
                            ViBoolean fastMode);

ViStatus _VI_FUNC rsspecan_ConfigureDOCSISEvaluationRangeSymbolSize(
                            ViSession instrSession,
                            ViInt32   symbolPointsSize);

ViStatus _VI_FUNC rsspecan_ConfigureDOCSISCarrierAxesUnit(ViSession instrSession,
                                                          ViInt32   axesUnit);

ViStatus _VI_FUNC rsspecan_ConfigureDOCSISAutoDetect(ViSession instrSession);

ViStatus _VI_FUNC rsspecan_ConfigureDOCSISBitstreamDisplaySettings(
                            ViSession instrSession,
                            ViInt32   bitstreamFormat,
                            ViInt32   bitstreamLayout);

ViStatus _VI_FUNC rsspecan_ConfigureDOCSISResultSummaryDisplay(
                            ViSession instrSession,
                            ViInt32   item,
                            ViBoolean displayItem);

ViStatus _VI_FUNC rsspecan_ConfigureDOCSISResultSummaryUnit(ViSession instrSession,
                                                            ViInt32   powerUnit);

ViStatus _VI_FUNC rsspecan_ConfigureDOCSISResultSummaryDisplaySettings(
                            ViSession instrSession,
                            ViBoolean normalizeCMSampleClockError,
                            ViReal64  CMTSReferenceSampleClock,
                            ViInt32   CMTSRefSampleClockUnit);

ViStatus _VI_FUNC rsspecan_ConfigureDOCSISPowerUnit(ViSession instrSession,
                                                    ViInt32   powerUnit);

ViStatus _VI_FUNC rsspecan_ConfigureDOCSISAutoConfigurationPerContinuousMinislotBlock(
                            ViSession instrSession,
                            ViBoolean autoConfiguration);

ViStatus _VI_FUNC rsspecan_ConfigureDOCSISBandAutoConfig(ViSession instrSession,
                                                         ViInt32   bandAutoConfig);

ViStatus _VI_FUNC rsspecan_ConfigureDOCSISBandConfigurationTableSubcarriers(
                            ViSession instrSession,
                            ViInt32   adjBandsOffsetSC,
                            ViInt32   alt1BandsOffsetSC,
                            ViInt32   adjBandsBandwidthSC,
                            ViInt32   alt1BandsBandwidthSC);

ViStatus _VI_FUNC rsspecan_ConfigureDOCSISBandConfigurationTableFrequency(
                            ViSession instrSession,
                            ViReal64  adjBandsOffsetFrequency,
                            ViReal64  alt1BandsOffsetFrequency,
                            ViReal64  adjBandsBandwidthFrequency,
                            ViReal64  alt1BandsBandwidthFrequency);

ViStatus _VI_FUNC rsspecan_ConfigureDOCSISAutoBandsAppliedTo(ViSession instrSession,
                                                             ViInt32   autoBandsAppliedTo);

ViStatus _VI_FUNC rsspecan_ConfigureDOCSISUserConfig(ViSession instrSession,
                                                     ViInt32   band,
                                                     ViBoolean state,
                                                     ViInt32   locationReference,
                                                     ViInt32   locationSC,
                                                     ViInt32   bandwidthSC);

ViStatus _VI_FUNC rsspecan_ConfigureDOCSISYScaling(ViSession instrSession,
                                                   ViInt32   window,
                                                   ViBoolean automaticGridScaling,
                                                   ViInt32   autoMode,
                                                   ViInt32   autoFixRange);

ViStatus _VI_FUNC rsspecan_ConfigureDOCSISYScalingMinMax(ViSession instrSession,
                                                         ViInt32   window,
                                                         ViReal64  minimum,
                                                         ViReal64  maximum);

ViStatus _VI_FUNC rsspecan_ConfigureDOCSISYScalingHysteresis(ViSession instrSession,
                                                             ViInt32   window,
                                                             ViReal64  hysteresisIntervalLowerUpper,
                                                             ViReal64  hysteresisIntervalLowerLower,
                                                             ViReal64  hysteresisIntervalUpperLower,
                                                             ViReal64  hysteresisIntervalUpperUpper);

ViStatus _VI_FUNC rsspecan_ConfigureDOCSISYScalingMemoryDepth(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   memoryDepth);

ViStatus _VI_FUNC rsspecan_ConfigureDOCSISYScalingNumberofDivisions(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   numberOfDivisions,
                            ViInt32   arraySize,
                            ViInt32   scalingPerDivision[]);

ViStatus _VI_FUNC rsspecan_AddDOCSISWindow(ViSession instrSession,
                                           ViInt32   stringSize,
                                           ViString  referenceWindowName,
                                           ViInt32   direction,
                                           ViInt32   windowType,
                                           ViChar    returnedWindowName[]);

ViStatus _VI_FUNC rsspecan_ReplaceDOCSISWindow(ViSession instrSession,
                                               ViString  windowName,
                                               ViInt32   windowType);

ViStatus _VI_FUNC rsspecan_QueryDOCSISWindowType(ViSession instrSession,
                                                 ViInt32   window,
                                                 ViInt32*  windowType);

ViStatus _VI_FUNC rsspecan_DOCSISAutoSetFromPLCAndRun(ViSession instrSession);

ViStatus _VI_FUNC rsspecan_DOCSISAutoDetectionAndRun(ViSession instrSession);

ViStatus _VI_FUNC rsspecan_FetchDOCSISResults(ViSession instrSession,
                                              ViInt32   specifiedParameter,
                                              ViReal64* average,
                                              ViReal64* maximum,
                                              ViReal64* minimum);

ViStatus _VI_FUNC rsspecan_FetchDOCSISContinuousPilotsResults(
                            ViSession instrSession,
                            ViReal64* average,
                            ViReal64* maximum,
                            ViReal64* minimum);

ViStatus _VI_FUNC rsspecan_FetchDOCSISDataResults(ViSession instrSession,
                                                  ViReal64* average,
                                                  ViReal64* maximum,
                                                  ViReal64* minimum);

ViStatus _VI_FUNC rsspecan_FetchDOCSISPowerPilotsResults(ViSession instrSession,
                                                         ViReal64* average,
                                                         ViReal64* maximum,
                                                         ViReal64* minimum);

ViStatus _VI_FUNC rsspecan_FetchDOCSISScatteredPilotsResults(ViSession instrSession,
                                                             ViReal64* average,
                                                             ViReal64* maximum,
                                                             ViReal64* minimum);

ViStatus _VI_FUNC rsspecan_FetchDOCSISPhysicalLinkChannelResults(
                            ViSession instrSession,
                            ViReal64* average,
                            ViReal64* maximum,
                            ViReal64* minimum);

ViStatus _VI_FUNC rsspecan_FetchDOCSISAllResults(ViSession instrSession,
                                                 ViInt32   arraySize,
                                                 ViReal64  results[],
                                                 ViInt32*  returnedValues);

ViStatus _VI_FUNC rsspecan_FetchDOCSISSignalContentDetailed(ViSession instrSession,
                                                            ViInt32   arraySize,
                                                            ViInt32*  returnedValues,
                                                            ViInt32   CWIndex[],
                                                            ViInt32   symbolStart[],
                                                            ViInt32   objectInformationType[],
                                                            ViInt32   profileList[],
                                                            ViInt32   modulation[],
                                                            ViReal64  MER[],
                                                            ViReal64  power[],
                                                            ViInt32   numberOfSubcarrier[],
                                                            ViInt32   LDPCIIterations[],
                                                            ViInt32   LDPCIBitErrPre[],
                                                            ViReal64  LDPCIBERPre[],
                                                            ViInt32   LDPCIBitErrPost[],
                                                            ViReal64  LDPCIBERPost[],
                                                            ViInt32   LDPCICWErrPost[],
                                                            ViReal64  LDPCICBLErrPost[],
                                                            ViInt32   reserved1[],
                                                            ViReal64  reserved2[],
                                                            ViInt32   reserved3[],
                                                            ViReal64  reserved4[]);

ViStatus _VI_FUNC rsspecan_FetchDOCSISSignalContentDetailedFormatted(
                            ViSession instrSession,
                            ViInt32   arraySize,
                            ViInt32*  returnedValues,
                            ViInt32   CWIndex[],
                            ViInt32   symbolStart[],
                            ViInt32   objectInformationType[],
                            ViInt32   modulation[],
                            ViReal64  MER[],
                            ViReal64  power[],
                            ViInt32   numberOfSubcarrier[],
                            ViInt32   LDPCIterations[],
                            ViInt32   bitErrPre[],
                            ViReal64  bitErrPreN[],
                            ViInt32   bitErrPost[],
                            ViReal64  bitErrPostN[],
                            ViInt32   CWErrPost[],
                            ViReal64  CWErrPostN[],
                            ViInt32   reserved1[],
                            ViReal64  reserved2[],
                            ViInt32   reserved3[],
                            ViReal64  reserved4[]);

ViStatus _VI_FUNC rsspecan_FetchDOCSISBitstreamResults(ViSession instrSession,
                                                       ViInt32   arraySize,
                                                       ViInt32*  returnedValues,
                                                       ViInt32   CWIndex[],
                                                       ViInt32   objectInformationType[],
                                                       ViInt32   modulation[],
                                                       ViInt32   numberOfBits[],
                                                       ViInt32   numberOfEntries[],
                                                       ViInt32   bits[]);

ViStatus _VI_FUNC rsspecan_FetchDOCSISSignalContentSummary(ViSession instrSession,
                                                           ViInt32   arraySize,
                                                           ViInt32*  returnedValues,
                                                           ViInt32   objectInformationType[],
                                                           ViInt32   modulation[],
                                                           ViReal64  MER[],
                                                           ViInt32   numberOfObject[],
                                                           ViInt32   LDPCIterations[],
                                                           ViInt32   bitErrPre[],
                                                           ViReal64  bitErrPreN[],
                                                           ViInt32   bitErrPost[],
                                                           ViReal64  bitErrPostN[],
                                                           ViInt32   CWErrPost[],
                                                           ViReal64  CWErrPostN[]);

ViStatus _VI_FUNC rsspecan_FetchDOCSISFrameCount(ViSession instrSession,
                                                 ViInt32*  frameCount);

ViStatus _VI_FUNC rsspecan_FetchDOCSISFrameCountAll(ViSession instrSession,
                                                    ViInt32*  frameCountAll);

ViStatus _VI_FUNC rsspecan_FetchDOCSISCyclicPrefixCP(ViSession instrSession,
                                                     ViInt32*  cyclicPrefixCP);

ViStatus _VI_FUNC rsspecan_QueryDOCSISSynchronousPowerBandResult(
                            ViSession instrSession,
                            ViInt32   band,
                            ViReal64* result);

ViStatus _VI_FUNC rsspecan_QueryDOCSISMarkerZAxis(ViSession instrSession,
                                                  ViInt32   window,
                                                  ViInt32   marker,
                                                  ViReal64* zAxis);

ViStatus _VI_FUNC rsspecan_FetchDOCSISPLCTimestamp(ViSession instrSession,
                                                   ViInt32*  timestamp);

ViStatus _VI_FUNC rsspecan_FetchDOCSISPLCOFDMChannelInformation(
                            ViSession instrSession,
                            ViReal64* spectrumLocation,
                            ViInt32*  FFTLength,
                            ViInt32*  cyclicPrefixCP,
                            ViInt32*  rollOff,
                            ViInt32*  timeInterleavingDepth,
                            ViInt32*  startIndexL,
                            ViInt32*  channelID,
                            ViInt32*  changeCount);

ViStatus _VI_FUNC rsspecan_FetchDOCSISPLCOFDMPilotsSubcarriers(
                            ViSession instrSession,
                            ViInt32   arraySize,
                            ViInt32   subcarriers[],
                            ViInt32*  returnedValues);

ViStatus _VI_FUNC rsspecan_FetchDOCSISPLCOFDMExcludedSubcarriers(
                            ViSession instrSession,
                            ViInt32   arraySize,
                            ViInt32   subcarriers[],
                            ViInt32*  returnedValues);

ViStatus _VI_FUNC rsspecan_FetchDOCSISPLCNCPInformation(ViSession instrSession,
                                                        ViInt32*  channelID,
                                                        ViInt32*  changeCount,
                                                        ViInt32*  modulation);

ViStatus _VI_FUNC rsspecan_FetchDOCSISPLCNCPSubcarriers(ViSession instrSession,
                                                        ViInt32   arraySize,
                                                        ViInt32   subcarriers[],
                                                        ViInt32*  returnedValues);

ViStatus _VI_FUNC rsspecan_FetchDOCSISPLCProfileInformation(ViSession instrSession,
                                                            ViInt32   setIndex,
                                                            ViInt32*  channelID,
                                                            ViInt32*  changeCount);

ViStatus _VI_FUNC rsspecan_FetchDOCSISPLCProfileSubcarrierModulation(
                            ViSession instrSession,
                            ViInt32   setIndex,
                            ViInt32   arraySize,
                            ViInt32   modulation[],
                            ViInt32*  returnedValues);

ViStatus _VI_FUNC rsspecan_FetchDOCSISPLCProfileSubcarriers(ViSession instrSession,
                                                            ViInt32   setIndex,
                                                            ViInt32   arraySize,
                                                            ViInt32   modulation,
                                                            ViInt32   subcarriers[],
                                                            ViInt32*  returnedValues);

ViStatus _VI_FUNC rsspecan_QueryDOCSISClockErrorLimit(ViSession instrSession,
                                                      ViReal64* average,
                                                      ViReal64* maximum);

ViStatus _VI_FUNC rsspecan_QueryDOCSISClockErrorLimitCheckResult(
                            ViSession instrSession,
                            ViInt32*  average,
                            ViInt32*  maximum);

ViStatus _VI_FUNC rsspecan_QueryDOCSISFrequencyErrorLimit(ViSession instrSession,
                                                          ViReal64* average,
                                                          ViReal64* maximum);

ViStatus _VI_FUNC rsspecan_QueryDOCSISFrequencyErrorLimitCheckResult(
                            ViSession instrSession,
                            ViInt32*  average,
                            ViInt32*  maximum);

ViStatus _VI_FUNC rsspecan_QueryDOCSISModulationErrorRatioLimitsMinimum(
                            ViSession instrSession,
                            ViInt32   resultType,
                            ViReal64* minimum);

ViStatus _VI_FUNC rsspecan_QueryDOCSISModulationErrorRatioLimitsMinimumResult(
                            ViSession instrSession,
                            ViInt32   resultType,
                            ViInt32*  minimum);

ViStatus _VI_FUNC rsspecan_SetAvionicsMode(ViSession instrSession);

ViStatus _VI_FUNC rsspecan_ConfigureAvionicsMeasurementType(ViSession instrSession,
                                                            ViInt32   standard);

ViStatus _VI_FUNC rsspecan_ConfigureAvionicsTotalHarmonicDistortionFrequency(
                            ViSession instrSession,
                            ViInt32   fundamentalFrequencyIdentity,
                            ViReal64  frequency);

ViStatus _VI_FUNC rsspecan_ConfigureAvionicsHarmonicDistortion(
                            ViSession instrSession,
                            ViBoolean state,
                            ViReal64  harmonicFrequency);

ViStatus _VI_FUNC rsspecan_AddAvionicsWindow(ViSession instrSession,
                                             ViInt32   window,
                                             ViInt32   stringSize,
                                             ViInt32   direction,
                                             ViInt32   windowType,
                                             ViChar    returnedWindowName[]);

ViStatus _VI_FUNC rsspecan_ReplaceAvionicsWindow(ViSession instrSession,
                                                 ViString  windowName,
                                                 ViInt32   windowType);

ViStatus _VI_FUNC rsspecan_ConfigureAvionicsDemodulationBandwidth(
                            ViSession instrSession,
                            ViBoolean state,
                            ViReal64  demodulationBandwidth);

ViStatus _VI_FUNC rsspecan_ConfigureAvionicsBandwidthResolution(
                            ViSession instrSession,
                            ViBoolean state,
                            ViReal64  resolutionBW);

ViStatus _VI_FUNC rsspecan_ConfigureAvionicsAFFrequencyStartStop(
                            ViSession instrSession,
                            ViReal64  startFrequency,
                            ViReal64  stopFrequency);

ViStatus _VI_FUNC rsspecan_ConfigureAvionicsAFFrequencyCenterSpan(
                            ViSession instrSession,
                            ViReal64  centerFrequency,
                            ViReal64  span);

ViStatus _VI_FUNC rsspecan_ConfigureAvionicsAFFullSpan(ViSession instrSession);

ViStatus _VI_FUNC rsspecan_ConfigureILSDDMUnit(ViSession instrSession,
                                               ViInt32   unit);

ViStatus _VI_FUNC rsspecan_ConfigureVORMeasurementUnit(ViSession instrSession,
                                                       ViInt32   VORDirection);

ViStatus _VI_FUNC rsspecan_QueryAvionicsAMDepthResults(ViSession instrSession,
                                                       ViInt32   fundamentalFrequencyIdentity,
                                                       ViReal64* depth);

ViStatus _VI_FUNC rsspecan_QueryAvionicsMorseCode(ViSession instrSession,
                                                  ViInt32   bufferSize,
                                                  ViChar    morseCode[]);

ViStatus _VI_FUNC rsspecan_QueryAvionicsRFLevelResults(ViSession instrSession,
                                                       ViReal64* RFLevel);

ViStatus _VI_FUNC rsspecan_QueryAvionicsDifferenceInDepthofModulation(
                            ViSession instrSession,
                            ViReal64* DDM);

ViStatus _VI_FUNC rsspecan_QueryAvionicsCarrierOffsetResults(ViSession instrSession,
                                                             ViReal64* carrierOffset);

ViStatus _VI_FUNC rsspecan_QueryAvionics30HzFMDeviation(ViSession instrSession,
                                                        ViReal64* modDepthDev);

ViStatus _VI_FUNC rsspecan_QueryAvionics30HzFMFrequency(ViSession instrSession,
                                                        ViReal64* frequency);

ViStatus _VI_FUNC rsspecan_QueryAvionicsPhaseResults(ViSession instrSession,
                                                     ViReal64* phase);

ViStatus _VI_FUNC rsspecan_QueryAvionicsRFFrequencySignal(ViSession instrSession,
                                                          ViReal64* RFFrequency);

ViStatus _VI_FUNC rsspecan_QueryAvionicsILSSDM(ViSession instrSession,
                                               ViReal64* SDM);

ViStatus _VI_FUNC rsspecan_QueryAvionicsAFFrequency(ViSession instrSession,
                                                    ViInt32   fundamentalFrequencyIdentity,
                                                    ViReal64* AFFrequency);

ViStatus _VI_FUNC rsspecan_QueryAvionicsTotalHarmonicDistortion(
                            ViSession instrSession,
                            ViInt32   fundamentalFrequencyIdentity,
                            ViReal64* result);

ViStatus _VI_FUNC rsspecan_QueryAvionicsHarmonicDistortionResults(
                            ViSession instrSession,
                            ViReal64* result);

ViStatus _VI_FUNC rsspecan_QueryAvionicsRelativeAmplitudeHarmonics(
                            ViSession instrSession,
                            ViInt32   harmonics,
                            ViInt32   fundamentalFrequencyIdentity,
                            ViReal64* relativeAmplitude);

ViStatus _VI_FUNC rsspecan_Set5GNRDownlinkMode(ViSession instrSession);

ViStatus _VI_FUNC rsspecan_Configure5GNR(ViSession instrSession,
                                         ViInt32   direction,
                                         ViInt32   mode);

ViStatus _VI_FUNC rsspecan_Configure5GNRORANTestcase(ViSession instrSession,
                                                     ViString  testcase);

ViStatus _VI_FUNC rsspecan_Configure5GNRChannelSpacingReset(ViSession instrSession);

ViStatus _VI_FUNC rsspecan_Configure5GNRSweepMode(ViSession instrSession,
                                                  ViInt32   mode);

ViStatus _VI_FUNC rsspecan_Configure5GNRDLCompleteSignalDemodulation(
                            ViSession instrSession,
                            ViInt32   CC,
                            ViBoolean signalDemodulationEnabled);

ViStatus _VI_FUNC rsspecan_Configure5GNRDL(ViSession instrSession,
                                           ViBoolean multicarrierFilter,
                                           ViInt32   componentCarriers,
                                           ViInt32   carrierMode);

ViStatus _VI_FUNC rsspecan_Configure5GNRDLDataCapture(ViSession instrSession,
                                                      ViBoolean frameCountEnabled,
                                                      ViInt32   frameCount,
                                                      ViInt32   subframesToAnalyze);

ViStatus _VI_FUNC rsspecan_Configure5GNRDLLongCapture(ViSession instrSession,
                                                      ViBoolean longCaptureEnabled);

ViStatus _VI_FUNC rsspecan_Configure5GNRDLDataCaptureSignalRepeatsMaxNoofSlotstoAnalyze(
                            ViSession instrSession,
                            ViBoolean maxNoOfSlotsToAnalyze);

ViStatus _VI_FUNC rsspecan_Configure5GNRDLDataCaptureAccordingToStandard(
                            ViSession instrSession,
                            ViBoolean autoFrameCountEnabled);

ViStatus _VI_FUNC rsspecan_Configure5GNRDLDataCaptureMaximumSlotsPerFrameToAnalyze(
                            ViSession instrSession,
                            ViInt32   select,
                            ViInt32   maxSlotsPerFrameToAnalyze);

ViStatus _VI_FUNC rsspecan_Configure5GNRDLSegmentedCapture(ViSession instrSession,
                                                           ViBoolean enabled,
                                                           ViInt32   events,
                                                           ViReal64  offset,
                                                           ViReal64  length);

ViStatus _VI_FUNC rsspecan_Configure5GNRDLTabSelect(ViSession instrSession,
                                                    ViInt32   window,
                                                    ViInt32   tab);

ViStatus _VI_FUNC rsspecan_Query5GNRDLTraceType(ViSession instrSession,
                                                ViInt32   window,
                                                ViInt32   arraySize,
                                                ViChar    type[]);

ViStatus _VI_FUNC rsspecan_Query5GNRDLMarkerZAll(ViSession instrSession,
                                                 ViInt32   window,
                                                 ViInt32   marker,
                                                 ViInt32   arraySize,
                                                 ViReal64  allResults[],
                                                 ViInt32*  returnedValues);

ViStatus _VI_FUNC rsspecan_Query5GNRDLMarkerZ(ViSession instrSession,
                                              ViInt32   window,
                                              ViInt32   marker,
                                              ViReal64* z);

ViStatus _VI_FUNC rsspecan_NR5GDLAutoEVM(ViSession instrSession);

ViStatus _VI_FUNC rsspecan_NR5GDLAutoLevelAll(ViSession instrSession);

ViStatus _VI_FUNC rsspecan_Configure5GNRDLAutoEVMSlots(ViSession instrSession,
                                                       ViInt32   numberOfSlots);

ViStatus _VI_FUNC rsspecan_Configure5GNRDLTracking(ViSession instrSession,
                                                   ViInt32   channelEstimation,
                                                   ViInt32   phaseTracking,
                                                   ViBoolean gainImbalanceQuadratureError,
                                                   ViInt32   timeAveraging);

ViStatus _VI_FUNC rsspecan_Configure5GNRDLTrackingTimeLevel(ViSession instrSession,
                                                            ViBoolean trackingTime,
                                                            ViBoolean trackingLevel);

ViStatus _VI_FUNC rsspecan_Configure5GNRDLDemodulation(ViSession instrSession,
                                                       ViInt32   demodulationData,
                                                       ViInt32   CORESETAnalysisMode);

ViStatus _VI_FUNC rsspecan_Configure5GNRDLDemodulationEVMCalculationMethod(
                            ViSession instrSession,
                            ViInt32   calculationMethod);

ViStatus _VI_FUNC rsspecan_Configure5GNRDLDemodulationSymbolTimeAdjustment(
                            ViSession instrSession,
                            ViReal64  symbolTimeAdjustment);

ViStatus _VI_FUNC rsspecan_Configure5GNRDLDemodulationPDSCHReferenceData(
                            ViSession instrSession,
                            ViInt32   PDSCHReferenceData);

ViStatus _VI_FUNC rsspecan_Configure5GNRDLDemodulationCoresetReferenceData(
                            ViSession instrSession,
                            ViInt32   coresetReferenceData);

ViStatus _VI_FUNC rsspecan_Configure5GNRDLDemodulationExtendedFrequencyLockRange(
                            ViSession instrSession,
                            ViBoolean extendedFrequencyLockRange);

ViStatus _VI_FUNC rsspecan_Configure5GNRDLPhysicalSettings(ViSession instrSession,
                                                           ViInt32   operatingBand,
                                                           ViReal64  generalFrequencyOffset);

ViStatus _VI_FUNC rsspecan_Configure5GNRDLChannelRaster(ViSession instrSession,
                                                        ViInt32   channelRaster);

ViStatus _VI_FUNC rsspecan_Configure5GNRDLMCSetup(ViSession instrSession,
                                                  ViReal64  globalMCFrequency,
                                                  ViInt32   frequencyOffsetRelativeTo,
                                                  ViInt32   frequencyOffsetMode,
                                                  ViReal64  carrierSpacing,
                                                  ViBoolean fixedCCOffset);

ViStatus _VI_FUNC rsspecan_Configure5GNRDLGlobalMCFrequencyOffset(
                            ViSession instrSession,
                            ViInt32   CC,
                            ViReal64  globalMCFrequencyOffset);

ViStatus _VI_FUNC rsspecan_NR5GDLSyncMCFrequencyToCenter(ViSession instrSession);

ViStatus _VI_FUNC rsspecan_Configure5GNRDLPhysicalSettingsLoadSignalDescription(
                            ViSession instrSession,
                            ViString  fileName);

ViStatus _VI_FUNC rsspecan_Configure5GNRDLPhysicalSettingsSaveSignalDescription(
                            ViSession instrSession,
                            ViString  fileName);

ViStatus _VI_FUNC rsspecan_Configure5GNRDLCCPhysicalSettings(ViSession instrSession,
                                                             ViInt32   CC,
                                                             ViInt32   bandwidth,
                                                             ViInt32   deploymentFrequency,
                                                             ViReal64  centerFrequency,
                                                             ViReal64  generalFrequencyOffset);

ViStatus _VI_FUNC rsspecan_Configure5GNRDLCCPhysicalSettingsCellID(
                            ViSession instrSession,
                            ViInt32   CC,
                            ViBoolean cellIDAuto,
                            ViInt32   cellID);

ViStatus _VI_FUNC rsspecan_Configure5GNRDLCCPhysicalSettingsLoadSignalDescription(
                            ViSession instrSession,
                            ViInt32   CC,
                            ViString  fileName);

ViStatus _VI_FUNC rsspecan_Configure5GNRDLCCPhysicalSettingsSaveSignalDescription(
                            ViSession instrSession,
                            ViInt32   CC,
                            ViString  fileName);

ViStatus _VI_FUNC rsspecan_Configure5GNRDLCCPhysicalSettingsLoadTestModel(
                            ViSession instrSession,
                            ViInt32   CC,
                            ViString  fileName);

ViStatus _VI_FUNC rsspecan_Configure5GNRDLBandwidthPartDemodulation(
                            ViSession instrSession,
                            ViInt32   CC,
                            ViInt32   bandwidthPartDemodulation);

ViStatus _VI_FUNC rsspecan_Configure5GNRDLBandwidthPartAdd(ViSession instrSession,
                                                           ViInt32   CC);

ViStatus _VI_FUNC rsspecan_Configure5GNRDLBandwidthPartDelete(
                            ViSession instrSession,
                            ViInt32   CC,
                            ViInt32   bandwidthPart);

ViStatus _VI_FUNC rsspecan_Configure5GNRDLBandwidthPartClear(ViSession instrSession,
                                                             ViInt32   CC);

ViStatus _VI_FUNC rsspecan_Configure5GNRDLBandwidthPartDuplicate(
                            ViSession instrSession,
                            ViInt32   CC,
                            ViInt32   bandwidthPart);

ViStatus _VI_FUNC rsspecan_Query5GNRDLBandwidthPartAnalyzed(ViSession instrSession,
                                                            ViInt32   CC,
                                                            ViInt32*  analyzedParts);

ViStatus _VI_FUNC rsspecan_Configure5GNRDLBandwidthPartResourceBlocks(
                            ViSession instrSession,
                            ViInt32   CC,
                            ViInt32   bandwidthPart,
                            ViInt32   count,
                            ViInt32   offset);

ViStatus _VI_FUNC rsspecan_Configure5GNRDLBandwidthPartSubcarrierSpacing(
                            ViSession instrSession,
                            ViInt32   CC,
                            ViInt32   bandwidthPart,
                            ViInt32   spacing);

ViStatus _VI_FUNC rsspecan_Configure5GNRDLFramePasteAll(ViSession instrSession,
                                                        ViInt32   CC);

ViStatus _VI_FUNC rsspecan_Configure5GNRDLFrameNumber(ViSession instrSession,
                                                      ViInt32   CC,
                                                      ViInt32   frames);

ViStatus _VI_FUNC rsspecan_Configure5GNRDLFrameCopy(ViSession instrSession,
                                                    ViInt32   CC,
                                                    ViInt32   frame);

ViStatus _VI_FUNC rsspecan_Configure5GNRDLFramePaste(ViSession instrSession,
                                                     ViInt32   CC,
                                                     ViInt32   frame);

ViStatus _VI_FUNC rsspecan_Configure5GNRDLFrameNumberOfSlots(ViSession instrSession,
                                                             ViInt32   CC,
                                                             ViInt32   frame,
                                                             ViInt32   bandwidthPart,
                                                             ViInt32   slots);

ViStatus _VI_FUNC rsspecan_Query5GNRDLFrameNumberOfAnalyzedSlots(
                            ViSession instrSession,
                            ViInt32   CC,
                            ViInt32   frame,
                            ViInt32   bandwidthPart,
                            ViInt32*  slots);

ViStatus _VI_FUNC rsspecan_Configure5GNRDLSlotPasteAll(ViSession instrSession,
                                                       ViInt32   CC,
                                                       ViInt32   frame,
                                                       ViInt32   bandwidthPart,
                                                       ViInt32   slot);

ViStatus _VI_FUNC rsspecan_Configure5GNRDLSlotPaste(ViSession instrSession,
                                                    ViInt32   CC,
                                                    ViInt32   frame,
                                                    ViInt32   bandwidthPart,
                                                    ViInt32   slot);

ViStatus _VI_FUNC rsspecan_Configure5GNRDLSlotPasteSettings(ViSession instrSession,
                                                            ViInt32   CC,
                                                            ViInt32   frame,
                                                            ViInt32   bandwidthPart,
                                                            ViInt32   slot,
                                                            ViInt32   mode,
                                                            ViInt32   period,
                                                            ViInt32   duration,
                                                            ViString  slots);

ViStatus _VI_FUNC rsspecan_NR5GDLSlotPasteTo(ViSession instrSession,
                                             ViInt32   CC,
                                             ViInt32   frame,
                                             ViInt32   bandwidthPart,
                                             ViInt32   slot);

ViStatus _VI_FUNC rsspecan_Configure5GNRDLSlotPreset(ViSession instrSession,
                                                     ViInt32   CC,
                                                     ViInt32   frame,
                                                     ViInt32   bandwidthPart);

ViStatus _VI_FUNC rsspecan_Configure5GNRDLPDSCHAllocations(ViSession instrSession,
                                                           ViInt32   CC,
                                                           ViInt32   frame,
                                                           ViInt32   bandwidthPart,
                                                           ViInt32   slot,
                                                           ViInt32   allocations);

ViStatus _VI_FUNC rsspecan_Configure5GNRDLPDSCHAllocationType(
                            ViSession instrSession,
                            ViInt32   CC,
                            ViInt32   frame,
                            ViInt32   bandwidthPart,
                            ViInt32   slot,
                            ViInt32   slotType);

ViStatus _VI_FUNC rsspecan_Configure5GNRDLPDSCHCopySlotConfiguration(
                            ViSession instrSession,
                            ViInt32   CC,
                            ViInt32   frame,
                            ViInt32   bandwidthPart,
                            ViInt32   slot);

ViStatus _VI_FUNC rsspecan_Configure5GNRDLPDSCHModulation(ViSession instrSession,
                                                          ViInt32   CC,
                                                          ViInt32   frame,
                                                          ViInt32   bandwidthPart,
                                                          ViInt32   slot,
                                                          ViInt32   allocation,
                                                          ViInt32   modulation);

ViStatus _VI_FUNC rsspecan_Configure5GNRDLPDSCHCodewordModulation(
                            ViSession instrSession,
                            ViInt32   CC,
                            ViInt32   frame,
                            ViInt32   bandwidthPart,
                            ViInt32   slot,
                            ViInt32   allocation,
                            ViInt32   codeword,
                            ViInt32   modulation);

ViStatus _VI_FUNC rsspecan_Configure5GNRDLPDSCHResourceBlocks(
                            ViSession instrSession,
                            ViInt32   CC,
                            ViInt32   frame,
                            ViInt32   bandwidthPart,
                            ViInt32   slot,
                            ViInt32   allocation,
                            ViInt32   count,
                            ViInt32   offset);

ViStatus _VI_FUNC rsspecan_Configure5GNRDLPDSCHSymbol(ViSession instrSession,
                                                      ViInt32   CC,
                                                      ViInt32   frame,
                                                      ViInt32   bandwidthPart,
                                                      ViInt32   slot,
                                                      ViInt32   allocation,
                                                      ViInt32   count,
                                                      ViInt32   offset);

ViStatus _VI_FUNC rsspecan_Configure5GNRDLCombinePDSCHAllocationsSameID(
                            ViSession instrSession,
                            ViInt32   CC,
                            ViInt32   frame,
                            ViInt32   bandwidthPart,
                            ViInt32   slot,
                            ViBoolean combineAllocations);

ViStatus _VI_FUNC rsspecan_Configure5GNRDLPDSCHCopySettings(ViSession instrSession,
                                                            ViInt32   CC,
                                                            ViInt32   frame,
                                                            ViInt32   bandwidthPart,
                                                            ViInt32   slot,
                                                            ViInt32   allocation,
                                                            ViInt32   mode,
                                                            ViInt32   period,
                                                            ViInt32   duration,
                                                            ViString  slots);

ViStatus _VI_FUNC rsspecan_NR5GDLPDSCHCopy(ViSession instrSession,
                                           ViInt32   CC,
                                           ViInt32   frame,
                                           ViInt32   bandwidthPart,
                                           ViInt32   slot,
                                           ViInt32   allocation);

ViStatus _VI_FUNC rsspecan_Configure5GNRDLEnhancedPDSCH(ViSession instrSession,
                                                        ViInt32   CC,
                                                        ViInt32   frame,
                                                        ViInt32   bandwidthPart,
                                                        ViInt32   slot,
                                                        ViInt32   allocation,
                                                        ViInt32   mapping,
                                                        ViReal64  power,
                                                        ViInt32   userID);

ViStatus _VI_FUNC rsspecan_Configure5GNRDLEnhancedPDSCHVRBtoPRBInterleave(
                            ViSession instrSession,
                            ViInt32   CC,
                            ViInt32   frame,
                            ViInt32   bandwidthPart,
                            ViInt32   slot,
                            ViInt32   allocation,
                            ViInt32   VRBtoPRBInterleave);

ViStatus _VI_FUNC rsspecan_Configure5GNRDLEnhancedPDSCHSettings(
                            ViSession instrSession,
                            ViInt32   CC,
                            ViInt32   frame,
                            ViInt32   bandwidthPart,
                            ViInt32   slot,
                            ViInt32   allocation,
                            ViString  antennaPort,
                            ViInt32   CDMGroupsWithoutData,
                            ViInt32   referencePoint);

ViStatus _VI_FUNC rsspecan_Configure5GNRDLEnhancedPDSCHPTRS(ViSession instrSession,
                                                            ViInt32   CC,
                                                            ViInt32   frame,
                                                            ViInt32   bandwidthPart,
                                                            ViInt32   slot,
                                                            ViInt32   allocation,
                                                            ViBoolean enabled,
                                                            ViInt32   timeDomainOffset,
                                                            ViReal64  power,
                                                            ViInt32   frequencyDomainOffset,
                                                            ViInt32   relativeOffset);

ViStatus _VI_FUNC rsspecan_Configure5GNRDLEnhancedPDSCHAdditionalDMRS(
                            ViSession instrSession,
                            ViInt32   CC,
                            ViInt32   frame,
                            ViInt32   bandwidthPart,
                            ViInt32   slot,
                            ViInt32   allocation,
                            ViInt32   additional);

ViStatus _VI_FUNC rsspecan_Configure5GNRDLEnhancedPDSCHDMRS(ViSession instrSession,
                                                            ViInt32   CC,
                                                            ViInt32   frame,
                                                            ViInt32   bandwidthPart,
                                                            ViInt32   slot,
                                                            ViInt32   allocation,
                                                            ViInt32   type,
                                                            ViInt32   length,
                                                            ViInt32   mapping,
                                                            ViInt32   scramblingID,
                                                            ViReal64  power,
                                                            ViInt32   method,
                                                            ViInt32   SID,
                                                            ViInt32   firstSymbol);

ViStatus _VI_FUNC rsspecan_Configure5GNRDLEnhancedPDSCHDMRSDownlink(
                            ViSession instrSession,
                            ViInt32   CC,
                            ViInt32   frame,
                            ViInt32   bandwidthPart,
                            ViInt32   slot,
                            ViInt32   allocation,
                            ViBoolean enabled);

ViStatus _VI_FUNC rsspecan_Configure5GNRDLEnhancedPDSCHChannelCoding(
                            ViSession instrSession,
                            ViInt32   CC,
                            ViInt32   frame,
                            ViInt32   bandwidthPart,
                            ViInt32   slot,
                            ViInt32   allocation,
                            ViInt32   MCSTable,
                            ViInt32   MCSIndex,
                            ViInt32   redundancyVersionIndex);

ViStatus _VI_FUNC rsspecan_Configure5GNRDLEnhancedPDSCHChannelCodingScalingFactor(
                            ViSession instrSession,
                            ViInt32   CC,
                            ViInt32   frame,
                            ViInt32   bandwidthPart,
                            ViInt32   slot,
                            ViInt32   allocation,
                            ViInt32   scalingFactor);

ViStatus _VI_FUNC rsspecan_Configure5GNRDLEnhancedPDSCHScrambling(
                            ViSession instrSession,
                            ViInt32   CC,
                            ViInt32   frame,
                            ViInt32   bandwidthPart,
                            ViInt32   slot,
                            ViInt32   allocation,
                            ViInt32   scramblingMethod,
                            ViInt32   seedValue);

ViStatus _VI_FUNC rsspecan_Configure5GNRDLCORESETNumbers(ViSession instrSession,
                                                         ViInt32   CC,
                                                         ViInt32   frame,
                                                         ViInt32   bandwidthPart,
                                                         ViInt32   slot,
                                                         ViInt32   CORESETs);

ViStatus _VI_FUNC rsspecan_Configure5GNRDLCORESETPrecoderGranularity(
                            ViSession instrSession,
                            ViInt32   CC,
                            ViInt32   frame,
                            ViInt32   bandwidthPart,
                            ViInt32   slot,
                            ViInt32   CORESET,
                            ViInt32   precoderGranularity);

ViStatus _VI_FUNC rsspecan_Configure5GNRDLCORESETPower(ViSession instrSession,
                                                       ViInt32   bandwidthPart,
                                                       ViInt32   slot,
                                                       ViInt32   CORESET,
                                                       ViReal64  power);

ViStatus _VI_FUNC rsspecan_Configure5GNRDLCORESETDMRS(ViSession instrSession,
                                                      ViInt32   CC,
                                                      ViInt32   frame,
                                                      ViInt32   bandwidthPart,
                                                      ViInt32   slot,
                                                      ViInt32   CORESET,
                                                      ViReal64  power,
                                                      ViInt32   sequenceGeneration,
                                                      ViInt32   SID);

ViStatus _VI_FUNC rsspecan_Configure5GNRDLCORESETDMRSReferencePoint(
                            ViSession instrSession,
                            ViInt32   CC,
                            ViInt32   frame,
                            ViInt32   bandwidthPart,
                            ViInt32   slot,
                            ViInt32   CORESET,
                            ViInt32   referencePoint);

ViStatus _VI_FUNC rsspecan_Configure5GNRDLCORESETResourceBlocks(
                            ViSession instrSession,
                            ViInt32   CC,
                            ViInt32   frame,
                            ViInt32   bandwidthPart,
                            ViInt32   slot,
                            ViInt32   CORESET,
                            ViInt32   count,
                            ViInt32   offset);

ViStatus _VI_FUNC rsspecan_Configure5GNRDLCORESETSymbol(ViSession instrSession,
                                                        ViInt32   CC,
                                                        ViInt32   frame,
                                                        ViInt32   bandwidthPart,
                                                        ViInt32   slot,
                                                        ViInt32   CORESET,
                                                        ViInt32   count,
                                                        ViInt32   offset);

ViStatus _VI_FUNC rsspecan_Configure5GNRDLCORESETInterleaving(
                            ViSession instrSession,
                            ViInt32   CC,
                            ViInt32   frame,
                            ViInt32   bandwidthPart,
                            ViInt32   slot,
                            ViInt32   CORESET,
                            ViBoolean enabled,
                            ViInt32   shiftIndex,
                            ViInt32   shiftIndexValue,
                            ViInt32   bundleSize,
                            ViInt32   interleaverSize);

ViStatus _VI_FUNC rsspecan_Configure5GNRDLCORESETPDCCHConfigUsage(
                            ViSession instrSession,
                            ViInt32   CC,
                            ViInt32   frame,
                            ViInt32   bandwidthPart,
                            ViInt32   slot,
                            ViInt32   CORESET,
                            ViInt32   usage);

ViStatus _VI_FUNC rsspecan_Configure5GNRDLCORESETPDCCHConfig(ViSession instrSession,
                                                             ViInt32   CC,
                                                             ViInt32   frame,
                                                             ViInt32   bandwidthPart,
                                                             ViInt32   slot,
                                                             ViInt32   CORESET,
                                                             ViInt32   RNTI,
                                                             ViInt32   aggregationLevel,
                                                             ViInt32   CCEIndex,
                                                             ViInt32   patternLength);

ViStatus _VI_FUNC rsspecan_Configure5GNRDLCORESETPDCCHConfigDCIFormat(
                            ViSession instrSession,
                            ViInt32   CC,
                            ViInt32   frame,
                            ViInt32   bandwidthPart,
                            ViInt32   slot,
                            ViInt32   CORESET,
                            ViInt32   DCIFormat);

ViStatus _VI_FUNC rsspecan_Configure5GNRDLCORESETPDCCHConfigFrequencyResourceAssignment(
                            ViSession instrSession,
                            ViInt32   CC,
                            ViInt32   frame,
                            ViInt32   bandwidthPart,
                            ViInt32   slot,
                            ViInt32   CORESET,
                            ViInt32   frequencyResourceAssignment);

ViStatus _VI_FUNC rsspecan_Configure5GNRDLCORESETPDCCHConfigDCIFields(
                            ViSession instrSession,
                            ViInt32   CC,
                            ViInt32   frame,
                            ViInt32   bandwidthPart,
                            ViInt32   slot,
                            ViInt32   CORESET,
                            ViInt32   DCIField,
                            ViString  bitLength);

ViStatus _VI_FUNC rsspecan_Query5GNRDLCORESETPDCCHConfigDCIFields(
                            ViSession instrSession,
                            ViInt32   CC,
                            ViInt32   frame,
                            ViInt32   bandwidthPart,
                            ViInt32   slot,
                            ViInt32   CORESET,
                            ViInt32   bufferSize,
                            ViChar    DCIAvailableFields[]);

ViStatus _VI_FUNC rsspecan_Configure5GNRDLCORESETPDCCHConfigDCIBlocks(
                            ViSession instrSession,
                            ViInt32   CC,
                            ViInt32   frame,
                            ViInt32   bandwidthPart,
                            ViInt32   slot,
                            ViInt32   CORESET,
                            ViInt32   numberOfBlocks);

ViStatus _VI_FUNC rsspecan_Configure5GNRDLCORESETPDCCHConfigDCIScope(
                            ViSession instrSession,
                            ViInt32   CC,
                            ViInt32   frame,
                            ViInt32   bandwidthPart,
                            ViInt32   slot,
                            ViInt32   CORESET,
                            ViInt32   scope);

ViStatus _VI_FUNC rsspecan_Configure5GNRDLCORESETPDCCHConfigDCITCPCommand(
                            ViSession instrSession,
                            ViInt32   CC,
                            ViInt32   frame,
                            ViInt32   bandwidthPart,
                            ViInt32   slot,
                            ViInt32   CORESET,
                            ViInt32   TCPCommand);

ViStatus _VI_FUNC rsspecan_Configure5GNRDLCORESETCopySettings(
                            ViSession instrSession,
                            ViInt32   CC,
                            ViInt32   frame,
                            ViInt32   bandwidthPart,
                            ViInt32   slot,
                            ViInt32   CORESET,
                            ViInt32   mode,
                            ViInt32   period,
                            ViInt32   duration,
                            ViString  slots);

ViStatus _VI_FUNC rsspecan_NR5GDLCORESETCopy(ViSession instrSession,
                                             ViInt32   CC,
                                             ViInt32   frame,
                                             ViInt32   bandwidthPart,
                                             ViInt32   slot,
                                             ViInt32   CORESET);

ViStatus _VI_FUNC rsspecan_Configure5GNRDLCSIRSEnabled(ViSession instrSession,
                                                       ViInt32   CC,
                                                       ViInt32   frame,
                                                       ViInt32   bandwidthPart,
                                                       ViInt32   CSI,
                                                       ViBoolean enabled);

ViStatus _VI_FUNC rsspecan_Configure5GNRDLCSIRSResources(ViSession instrSession,
                                                         ViInt32   CC,
                                                         ViInt32   frame,
                                                         ViInt32   bandwidthPart,
                                                         ViInt32   CSI,
                                                         ViInt32   resources);

ViStatus _VI_FUNC rsspecan_Configure5GNRDLCSIRSSlot(ViSession instrSession,
                                                    ViInt32   CC,
                                                    ViInt32   frame,
                                                    ViInt32   bandwidthPart,
                                                    ViInt32   CSI,
                                                    ViInt32   method,
                                                    ViInt32   periodicity,
                                                    ViInt32   offset,
                                                    ViString  aperiodicSlots);

ViStatus _VI_FUNC rsspecan_Configure5GNRDLCSIRSPower(ViSession instrSession,
                                                     ViInt32   CC,
                                                     ViInt32   frame,
                                                     ViInt32   bandwidthPart,
                                                     ViInt32   CSI,
                                                     ViBoolean zeroPower,
                                                     ViInt32   scramblingID,
                                                     ViReal64  relativePower);

ViStatus _VI_FUNC rsspecan_Configure5GNRDLCSIRSRB(ViSession instrSession,
                                                  ViInt32   CC,
                                                  ViInt32   frame,
                                                  ViInt32   bandwidthPart,
                                                  ViInt32   CSI,
                                                  ViInt32   numberOfRBs,
                                                  ViInt32   startRB);

ViStatus _VI_FUNC rsspecan_Configure5GNRDLCSIRSRow(ViSession instrSession,
                                                   ViInt32   CC,
                                                   ViInt32   frame,
                                                   ViInt32   bandwidthPart,
                                                   ViInt32   CSI,
                                                   ViInt32   row);

ViStatus _VI_FUNC rsspecan_Configure5GNRDLCSIRSDensity(ViSession instrSession,
                                                       ViInt32   CC,
                                                       ViInt32   frame,
                                                       ViInt32   bandwidthPart,
                                                       ViInt32   CSI,
                                                       ViInt32   density);

ViStatus _VI_FUNC rsspecan_Configure5GNRDLCSIRSBitmap(ViSession instrSession,
                                                      ViInt32   CC,
                                                      ViInt32   frame,
                                                      ViInt32   bandwidthPart,
                                                      ViInt32   CSI,
                                                      ViInt32   bitmap);

ViStatus _VI_FUNC rsspecan_Configure5GNRDLCSIRSLocation(ViSession instrSession,
                                                        ViInt32   CC,
                                                        ViInt32   frame,
                                                        ViInt32   bandwidthPart,
                                                        ViInt32   CSI,
                                                        ViInt32   parameter,
                                                        ViInt32   location);

ViStatus _VI_FUNC rsspecan_Query5GNRDLCSIRSPorts(ViSession instrSession,
                                                 ViInt32   CC,
                                                 ViInt32   frame,
                                                 ViInt32   bandwidthPart,
                                                 ViInt32   CSI,
                                                 ViInt32*  ports);

ViStatus _VI_FUNC rsspecan_Query5GNRDLCSIRSCDMType(ViSession instrSession,
                                                   ViInt32   CC,
                                                   ViInt32   frame,
                                                   ViInt32   bandwidthPart,
                                                   ViInt32   CSI,
                                                   ViInt32*  CDMType);

ViStatus _VI_FUNC rsspecan_Configure5GNRDLPRSEnabled(ViSession instrSession,
                                                     ViInt32   CC,
                                                     ViInt32   frame,
                                                     ViInt32   bandwidthPart,
                                                     ViBoolean PRSEnabled);

ViStatus _VI_FUNC rsspecan_Configure5GNRDLPRSSlotConfig(ViSession instrSession,
                                                        ViInt32   CC,
                                                        ViInt32   frame,
                                                        ViInt32   bandwidthPart,
                                                        ViString  slotConfig);

ViStatus _VI_FUNC rsspecan_Configure5GNRDLPRSRB(ViSession instrSession,
                                                ViInt32   CC,
                                                ViInt32   frame,
                                                ViInt32   bandwidthPart,
                                                ViInt32   numberOfRBs,
                                                ViInt32   startRB);

ViStatus _VI_FUNC rsspecan_Configure5GNRDLPRSL_PRS(ViSession instrSession,
                                                   ViInt32   CC,
                                                   ViInt32   frame,
                                                   ViInt32   bandwidthPart,
                                                   ViInt32   lPRSStart,
                                                   ViInt32   lPRS);

ViStatus _VI_FUNC rsspecan_Configure5GNRDLPRSN_PRSIDSeq(ViSession instrSession,
                                                        ViInt32   CC,
                                                        ViInt32   frame,
                                                        ViInt32   bandwidthPart,
                                                        ViInt32   nPRSIDSeq);

ViStatus _VI_FUNC rsspecan_Configure5GNRDLPRSK_PRS(ViSession instrSession,
                                                   ViInt32   CC,
                                                   ViInt32   frame,
                                                   ViInt32   bandwidthPart,
                                                   ViInt32   kPRSComb,
                                                   ViReal64  kPRSOffset);

ViStatus _VI_FUNC rsspecan_Configure5GNRDLPRSRelativePower(ViSession instrSession,
                                                           ViInt32   CC,
                                                           ViInt32   frame,
                                                           ViInt32   bandwidthPart,
                                                           ViReal64  relativePower);

ViStatus _VI_FUNC rsspecan_Configure5GNRDLSynchronizationSignalEnabled(
                            ViSession instrSession,
                            ViInt32   CC,
                            ViInt32   SS,
                            ViBoolean enabled);

ViStatus _VI_FUNC rsspecan_Configure5GNRDLSynchronizationSignal(
                            ViSession instrSession,
                            ViInt32   CC,
                            ViInt32   detection,
                            ViInt32   subcarrierSpacing,
                            ViInt32   pattern,
                            ViInt32   frequencyOffset,
                            ViInt32   offset,
                            ViInt32   periodicity);

ViStatus _VI_FUNC rsspecan_Configure5GNRDLSynchronizationSignalPBCH(
                            ViSession instrSession,
                            ViInt32   CC,
                            ViReal64  power,
                            ViReal64  DMRSPower);

ViStatus _VI_FUNC rsspecan_Configure5GNRDLSynchronizationSignalPower(
                            ViSession instrSession,
                            ViInt32   CC,
                            ViInt32   type,
                            ViReal64  power);

ViStatus _VI_FUNC rsspecan_Configure5GNRDLSynchronizationSignalOffsetRelativeTo(
                            ViSession instrSession,
                            ViInt32   CC,
                            ViInt32   offsetRelativeTo);

ViStatus _VI_FUNC rsspecan_Configure5GNRDLSynchronizationSignalLSelection(
                            ViSession instrSession,
                            ViInt32   CC,
                            ViInt32   lSelection);

ViStatus _VI_FUNC rsspecan_Configure5GNRDLSynchronizationSignalHalfFrameOffset(
                            ViSession instrSession,
                            ViInt32   CC,
                            ViInt32   halfFrameOffset);

ViStatus _VI_FUNC rsspecan_Configure5GNRDLAntennaPortMappingState(
                            ViSession instrSession,
                            ViInt32   CC,
                            ViInt32   antennaConfig,
                            ViBoolean enabled);

ViStatus _VI_FUNC rsspecan_Configure5GNRDLAntennaPortMappingCSIRS(
                            ViSession instrSession,
                            ViInt32   CC,
                            ViInt32   antennaConfig,
                            ViInt32   antennaPort,
                            ViBoolean enabled);

ViStatus _VI_FUNC rsspecan_Configure5GNRDLAntennaPortMappingPDSCH(
                            ViSession instrSession,
                            ViInt32   CC,
                            ViInt32   antennaConfig,
                            ViInt32   antennaPort,
                            ViInt32   enabled);

ViStatus _VI_FUNC rsspecan_Configure5GNRDLAdvancedSettingsHandlingOfCarrierLeakage(
                            ViSession instrSession,
                            ViInt32   CC,
                            ViInt32   handlingOfCarrierLeakage);

ViStatus _VI_FUNC rsspecan_Configure5GNRDLAdvancedSettingsFrameNumber(
                            ViSession instrSession,
                            ViInt32   CC,
                            ViInt32   frameNumber);

ViStatus _VI_FUNC rsspecan_Configure5GNRDLAdvancedSettingsRFUpconversion(
                            ViSession instrSession,
                            ViInt32   CC,
                            ViBoolean phaseCompensation,
                            ViInt32   f_0Mode,
                            ViReal64  frequency);

ViStatus _VI_FUNC rsspecan_Configure5GNRDLAdvancedSettingsExcludeUserIDs(
                            ViSession instrSession,
                            ViInt32   CC,
                            ViString  excludeUserIDs);

ViStatus _VI_FUNC rsspecan_Configure5GNRDLAdvancedSettingsFrequencyErrorLimitCheck(
                            ViSession instrSession,
                            ViBoolean frequencyErrorLimitCheck);

ViStatus _VI_FUNC rsspecan_Configure5GNRDLAdvancedSettingsSharedSpectrumChannelAccess(
                            ViSession instrSession,
                            ViInt32   CC,
                            ViBoolean sharedSpectrumChannelAccess);

ViStatus _VI_FUNC rsspecan_Configure5GNRDLAdvancedSettingsRefPointA(
                            ViSession instrSession,
                            ViInt32   CC,
                            ViReal64  centerFrequencyOffset,
                            ViInt32   k0Select,
                            ViInt32   offset);

ViStatus _VI_FUNC rsspecan_Query5GNRDLAdvancedSettingsRefPointAFrequency(
                            ViSession instrSession,
                            ViInt32   CC,
                            ViReal64* frequency);

ViStatus _VI_FUNC rsspecan_Configure5GNRDLAdvancedSettingsLTECRSCoexistence(
                            ViSession instrSession,
                            ViInt32   CC,
                            ViBoolean state,
                            ViInt32   offsetToPointA,
                            ViInt32   LTEBandwidthRBs,
                            ViInt32   vShift,
                            ViInt32   LTEAntennaPorts);

ViStatus _VI_FUNC rsspecan_Configure5GNRDLAdvancedSettingsLTECRSCoexistenceMBSFNSubframes(
                            ViSession instrSession,
                            ViInt32   CC,
                            ViInt32   subframe,
                            ViBoolean MBSFNSubframes);

ViStatus _VI_FUNC rsspecan_Configure5GNRDLGeneratorIPAddress(ViSession instrSession,
                                                             ViString  IPAddress);

ViStatus _VI_FUNC rsspecan_Query5GNRDLGeneratorConnectionState(
                            ViSession instrSession,
                            ViInt32*  connectionState);

ViStatus _VI_FUNC rsspecan_Configure5GNRDLGeneratorControlState(
                            ViSession instrSession,
                            ViBoolean state);

ViStatus _VI_FUNC rsspecan_Configure5GNRDLGeneratorRFOutput(ViSession instrSession,
                                                            ViBoolean RFOutput);

ViStatus _VI_FUNC rsspecan_Configure5GNRDLGeneratorFrequencySynchronization(
                            ViSession instrSession,
                            ViBoolean frequencySynchronization);

ViStatus _VI_FUNC rsspecan_Configure5GNRDLGeneratorLevel(ViSession instrSession,
                                                         ViBoolean levelState,
                                                         ViReal64  level_RMS,
                                                         ViBoolean DUTLimitState,
                                                         ViReal64  DUTLimitPeakInputPower,
                                                         ViReal64  DUTGain);

ViStatus _VI_FUNC rsspecan_NR5GDLGeneratorUploadRFSettings(ViSession instrSession);

ViStatus _VI_FUNC rsspecan_NR5GDLGeneratorDownloadRFSettings(ViSession instrSession);

ViStatus _VI_FUNC rsspecan_NR5GDLGeneratorDownload5GNRSettings(ViSession instrSession);

ViStatus _VI_FUNC rsspecan_Configure5GNRDLGeneratorSynchronization(
                            ViSession instrSession,
                            ViBoolean syncEnabled,
                            ViReal64  pollingInterval);

ViStatus _VI_FUNC rsspecan_Configure5GNRDLPowerOnOffBaseStationOutputPower(
                            ViSession instrSession,
                            ViInt32   CC,
                            ViReal64  EIRP,
                            ViReal64  TRP);

ViStatus _VI_FUNC rsspecan_Configure5GNRDLPowerOnOffTransitionPeriod(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   period);

ViStatus _VI_FUNC rsspecan_NR5GDLPowerOnOffAdjustTiming(ViSession instrSession);

ViStatus _VI_FUNC rsspecan_Configure5GNRDLPowerOnOffNoiseCorrection(
                            ViSession instrSession,
                            ViBoolean noiseCorrectionEnabled);

ViStatus _VI_FUNC rsspecan_Configure5GNRDLPowerOnOffUnit(ViSession instrSession,
                                                         ViInt32   unit);

ViStatus _VI_FUNC rsspecan_Configure5GNRDLPowerOnOffNumberOfFramesToAnalyze(
                            ViSession instrSession,
                            ViInt32   numberOfFramesToAnalyze);

ViStatus _VI_FUNC rsspecan_Configure5GNRDLCombinedMeasurement(
                            ViSession instrSession,
                            ViBoolean ACLR,
                            ViBoolean EVM,
                            ViBoolean SEM);

ViStatus _VI_FUNC rsspecan_Load5GNRDLCombinedMeasurementACLRSettings(
                            ViSession instrSession,
                            ViString  settingsFile);

ViStatus _VI_FUNC rsspecan_Clear5GNRDLCombinedMeasurementACLRSettings(ViSession instrSession);

ViStatus _VI_FUNC rsspecan_Load5GNRDLCombinedMeasurementSEMSettings(
                            ViSession instrSession,
                            ViString  settingsFile);

ViStatus _VI_FUNC rsspecan_Clear5GNRDLCombinedMeasurementSEMSettings(ViSession instrSession);

ViStatus _VI_FUNC rsspecan_Configure5GNRDLCombinedMeasurementSignalCapture(
                            ViSession instrSession,
                            ViInt32   numberOfEvents,
                            ViInt32   optimization,
                            ViBoolean sequenceAutoLevel);

ViStatus _VI_FUNC rsspecan_Configure5GNRDLCombinedMeasurementTriggerSequence(
                            ViSession instrSession,
                            ViInt32   sequence);

ViStatus _VI_FUNC rsspecan_Configure5GNRDLCombinedMeasurementFrameTrigger(
                            ViSession instrSession,
                            ViInt32   frameTrigger);

ViStatus _VI_FUNC rsspecan_Configure5GNRDLCombinedMeasurementTriggerEventDelay(
                            ViSession instrSession,
                            ViReal64  eventDelay);

ViStatus _VI_FUNC rsspecan_Configure5GNRDLCombinedMeasurementTriggerRecalibrationInterval(
                            ViSession instrSession,
                            ViInt32   recalibrationInterval);

ViStatus _VI_FUNC rsspecan_Send5GNRDLCombinedMeasurementTrigger3(ViSession instrSession);

ViStatus _VI_FUNC rsspecan_Configure5GNRDLCombinedMeasurementTriggerAutoGating(
                            ViSession instrSession,
                            ViBoolean autoGating);

ViStatus _VI_FUNC rsspecan_NR5GDLAddWindow(ViSession instrSession,
                                           ViInt32   stringSize,
                                           ViString  referenceWindowName,
                                           ViInt32   direction,
                                           ViInt32   windowType,
                                           ViChar    returnedWindowName[]);

ViStatus _VI_FUNC rsspecan_NR5GDLReplaceWindow(ViSession instrSession,
                                               ViString  windowName,
                                               ViInt32   windowType);

ViStatus _VI_FUNC rsspecan_Configure5GNRDLAnalysisShowResultSummary(
                            ViSession instrSession,
                            ViInt32   summary);

ViStatus _VI_FUNC rsspecan_Configure5GNRDLAnalysisShowResultSummaryItems(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   item,
                            ViBoolean displayItem);

ViStatus _VI_FUNC rsspecan_Configure5GNRDLAnalysisCCResult(ViSession instrSession,
                                                           ViInt32   CCResult);

ViStatus _VI_FUNC rsspecan_Configure5GNRDLAnalysisXScale(ViSession instrSession,
                                                         ViInt32   xScale);

ViStatus _VI_FUNC rsspecan_Configure5GNRDLAnalysisCarrierAxes(
                            ViSession instrSession,
                            ViInt32   carrierAxes,
                            ViInt32   carrierAxesReference);

ViStatus _VI_FUNC rsspecan_Configure5GNRDLAnalysisEVMMaxHold(ViSession instrSession,
                                                             ViBoolean EVMMaxHold);

ViStatus _VI_FUNC rsspecan_Configure5GNRDLAnalysis3DView(ViSession instrSession,
                                                         ViBoolean _3DView);

ViStatus _VI_FUNC rsspecan_Configure5GNRDLAnalysisACLRLimitPassMode(
                            ViSession instrSession,
                            ViInt32   ACLRLimitPassMode);

ViStatus _VI_FUNC rsspecan_Query5GNRDLAnalysisSynchronizationState(
                            ViSession instrSession,
                            ViInt32   CC,
                            ViChar    state[]);

ViStatus _VI_FUNC rsspecan_Query5GNRDLAnalysisCellID(ViSession instrSession,
                                                     ViInt32   CC,
                                                     ViInt32*  cellID);

ViStatus _VI_FUNC rsspecan_Configure5GNRDLAnalysisEvaluationRangeAllocation(
                            ViSession instrSession,
                            ViInt32   mode,
                            ViInt32   value);

ViStatus _VI_FUNC rsspecan_Configure5GNRDLAnalysisEvaluationRangeBandwidthPart(
                            ViSession instrSession,
                            ViInt32   mode,
                            ViInt32   value);

ViStatus _VI_FUNC rsspecan_Configure5GNRDLAnalysisEvaluationRangeCarrier(
                            ViSession instrSession,
                            ViInt32   mode,
                            ViInt32   value);

ViStatus _VI_FUNC rsspecan_Configure5GNRDLAnalysisEvaluationRangeFrame(
                            ViSession instrSession,
                            ViInt32   value);

ViStatus _VI_FUNC rsspecan_Configure5GNRDLAnalysisEvaluationRangeSelectedMeasID(
                            ViSession instrSession,
                            ViInt32   selectedMeasID);

ViStatus _VI_FUNC rsspecan_Configure5GNRDLAnalysisEvaluationRangeModulation(
                            ViSession instrSession,
                            ViInt32   mode,
                            ViInt32   value);

ViStatus _VI_FUNC rsspecan_Configure5GNRDLAnalysisEvaluationRangeSlot(
                            ViSession instrSession,
                            ViInt32   mode,
                            ViInt32   value);

ViStatus _VI_FUNC rsspecan_Configure5GNRDLAnalysisEvaluationRangeSubframe(
                            ViSession instrSession,
                            ViInt32   mode,
                            ViInt32   value);

ViStatus _VI_FUNC rsspecan_Configure5GNRDLAnalysisEvaluationRangeSymbol(
                            ViSession instrSession,
                            ViInt32   mode,
                            ViInt32   value);

ViStatus _VI_FUNC rsspecan_Configure5GNRDLAnalysisEvaluationRangeSegment(
                            ViSession instrSession,
                            ViInt32   value);

ViStatus _VI_FUNC rsspecan_Configure5GNRDLAnalysisEvaluationRangeBeamformingReferenceAP(
                            ViSession instrSession,
                            ViInt32   value);

ViStatus _VI_FUNC rsspecan_Configure5GNRDLAnalysisEvaluationRangeBeamformingAntennaPort(
                            ViSession instrSession,
                            ViInt32   CC,
                            ViInt32   mode,
                            ViInt32   value);

ViStatus _VI_FUNC rsspecan_Query5GNRDLAnalysisAllCarriers(ViSession instrSession,
                                                          ViInt32   type,
                                                          ViReal64* result);

ViStatus _VI_FUNC rsspecan_Query5GNRDLAnalysisEVMAll(ViSession instrSession,
                                                     ViInt32   CC,
                                                     ViInt32   frame,
                                                     ViReal64* all);

ViStatus _VI_FUNC rsspecan_Query5GNRDLAnalysisEVMAllStatistics(
                            ViSession instrSession,
                            ViInt32   CC,
                            ViInt32   frame,
                            ViInt32   measurementType,
                            ViReal64* result);

ViStatus _VI_FUNC rsspecan_Query5GNRDLAnalysisEVMPDSCH(ViSession instrSession,
                                                       ViInt32   CC,
                                                       ViInt32   frame,
                                                       ViInt32   modulation,
                                                       ViReal64* result);

ViStatus _VI_FUNC rsspecan_Query5GNRDLAnalysisEVMPDSCHStatistics(
                            ViSession instrSession,
                            ViInt32   CC,
                            ViInt32   frame,
                            ViInt32   modulation,
                            ViInt32   measurementType,
                            ViReal64* result);

ViStatus _VI_FUNC rsspecan_Query5GNRDLAnalysisEVMPDSCHLimitCheckResult(
                            ViSession instrSession,
                            ViInt32   CC,
                            ViInt32   frame,
                            ViInt32   modulation,
                            ViInt32   measurementType,
                            ViInt32*  result);

ViStatus _VI_FUNC rsspecan_Query5GNRDLAnalysisEVMPeak(ViSession instrSession,
                                                      ViInt32   CC,
                                                      ViInt32   frame,
                                                      ViInt32   measurementType,
                                                      ViReal64* result);

ViStatus _VI_FUNC rsspecan_Query5GNRDLAnalysisEVMPhysical(ViSession instrSession,
                                                          ViInt32   CC,
                                                          ViInt32   frame,
                                                          ViReal64* channel,
                                                          ViReal64* signal);

ViStatus _VI_FUNC rsspecan_Query5GNRDLAnalysisEVMPhysicalStatistics(
                            ViSession instrSession,
                            ViInt32   CC,
                            ViInt32   frame,
                            ViInt32   measurementType,
                            ViReal64* channel,
                            ViReal64* signal);

ViStatus _VI_FUNC rsspecan_Query5GNRDLAnalysisEVMSignalPower(ViSession instrSession,
                                                             ViInt32   CC,
                                                             ViInt32   frame,
                                                             ViReal64* power);

ViStatus _VI_FUNC rsspecan_Query5GNRDLAnalysisEVMSignalPowerStatistics(
                            ViSession instrSession,
                            ViInt32   CC,
                            ViInt32   frame,
                            ViInt32   measurementType,
                            ViReal64* power);

ViStatus _VI_FUNC rsspecan_Query5GNRDLAnalysisEVMFrequencyError(
                            ViSession instrSession,
                            ViInt32   CC,
                            ViInt32   frame,
                            ViReal64* frequencyError);

ViStatus _VI_FUNC rsspecan_Query5GNRDLAnalysisEVMFrequencyErrorStatistics(
                            ViSession instrSession,
                            ViInt32   CC,
                            ViInt32   frame,
                            ViInt32   measurementType,
                            ViReal64* frequencyError);

ViStatus _VI_FUNC rsspecan_Query5GNRDLAnalysisEVMFrequencyErrorLimitCheckResult(
                            ViSession instrSession,
                            ViInt32   CC,
                            ViInt32   frame,
                            ViInt32   measurementType,
                            ViInt32*  result);

ViStatus _VI_FUNC rsspecan_Query5GNRDLAnalysisEVMSamplingError(
                            ViSession instrSession,
                            ViInt32   CC,
                            ViInt32   frame,
                            ViReal64* samplingError);

ViStatus _VI_FUNC rsspecan_Query5GNRDLAnalysisEVMSamplingErrorStatistics(
                            ViSession instrSession,
                            ViInt32   CC,
                            ViInt32   frame,
                            ViInt32   measurementType,
                            ViReal64* samplingError);

ViStatus _VI_FUNC rsspecan_Query5GNRDLAnalysisIQGainImbalance(
                            ViSession instrSession,
                            ViInt32   CC,
                            ViInt32   frame,
                            ViInt32   measurementType,
                            ViReal64* iQGainImbalance);

ViStatus _VI_FUNC rsspecan_Query5GNRDLAnalysisIQOffset(ViSession instrSession,
                                                       ViInt32   CC,
                                                       ViInt32   frame,
                                                       ViInt32   measurementType,
                                                       ViReal64* iQOffset);

ViStatus _VI_FUNC rsspecan_Query5GNRDLAnalysisIQQuadratureError(
                            ViSession instrSession,
                            ViInt32   CC,
                            ViInt32   frame,
                            ViInt32   measurementType,
                            ViReal64* iQQuadratureError);

ViStatus _VI_FUNC rsspecan_Query5GNRDLAnalysisOSTP(ViSession instrSession,
                                                   ViInt32   CC,
                                                   ViInt32   frame,
                                                   ViInt32   measurementType,
                                                   ViReal64* OSTP);

ViStatus _VI_FUNC rsspecan_Query5GNRDLAnalysisCSIRSRP(ViSession instrSession,
                                                      ViInt32   CC,
                                                      ViInt32   frame,
                                                      ViInt32   measurementType,
                                                      ViReal64* CSIRSRP);

ViStatus _VI_FUNC rsspecan_Query5GNRDLAnalysisSSRSRP(ViSession instrSession,
                                                     ViInt32   CC,
                                                     ViInt32   frame,
                                                     ViInt32   measurementType,
                                                     ViReal64* SSRSRP);

ViStatus _VI_FUNC rsspecan_Query5GNRDLAnalysisRSTP(ViSession instrSession,
                                                   ViInt32   CC,
                                                   ViInt32   frame,
                                                   ViInt32   measurementType,
                                                   ViReal64* RSTP);

ViStatus _VI_FUNC rsspecan_Query5GNRDLAnalysisBLER(ViSession instrSession,
                                                   ViInt32   CC,
                                                   ViInt32   frame,
                                                   ViInt32   measurementType,
                                                   ViReal64* BLER);

ViStatus _VI_FUNC rsspecan_Query5GNRDLAnalysisTPUT(ViSession instrSession,
                                                   ViInt32   CC,
                                                   ViInt32   frame,
                                                   ViInt32   measurementType,
                                                   ViReal64* TPUT);

ViStatus _VI_FUNC rsspecan_Query5GNRDLAnalysisEVMSubframeStartOffset(
                            ViSession instrSession,
                            ViInt32   CC,
                            ViReal64* subframeStartOffset);

ViStatus _VI_FUNC rsspecan_Query5GNRDLAnalysisCrestFactor(ViSession instrSession,
                                                          ViInt32   CC,
                                                          ViInt32   measurementType,
                                                          ViReal64* result);

ViStatus _VI_FUNC rsspecan_Query5GNRDLAnalysisACLRLimitCheck(ViSession instrSession,
                                                             ViInt32   CC,
                                                             ViInt32   frame,
                                                             ViInt32   type,
                                                             ViInt32*  result);

ViStatus _VI_FUNC rsspecan_Query5GNRDLAnalysisOverload(ViSession instrSession,
                                                       ViInt32   CC,
                                                       ViInt32   frame,
                                                       ViInt32*  overload);

ViStatus _VI_FUNC rsspecan_Query5GNRDLAnalysisSEMPassFail(ViSession instrSession,
                                                          ViInt32   CC,
                                                          ViInt32   frame,
                                                          ViInt32*  result);

ViStatus _VI_FUNC rsspecan_Query5GNRDLAnalysisSyncState(ViSession instrSession,
                                                        ViInt32   CC,
                                                        ViInt32   frame,
                                                        ViInt32*  result);

ViStatus _VI_FUNC rsspecan_Query5GNRDLAnalysisTimeStampDelta(ViSession instrSession,
                                                             ViInt32   CC,
                                                             ViInt32   frame,
                                                             ViReal64* result);

ViStatus _VI_FUNC rsspecan_Query5GNRDLAnalysisTimeStamp(ViSession instrSession,
                                                        ViInt32   CC,
                                                        ViInt32   frame,
                                                        ViReal64* result);

ViStatus _VI_FUNC rsspecan_Configure5GNRDLAnalysisSweepBaseStationType(
                            ViSession instrSession,
                            ViInt32   baseStationType);

ViStatus _VI_FUNC rsspecan_Configure5GNRDLAnalysisSweepBWForACLR(
                            ViSession instrSession,
                            ViInt32   BW);

ViStatus _VI_FUNC rsspecan_Configure5GNRDLAnalysisSweepBS(ViSession instrSession,
                                                          ViInt32   category,
                                                          ViBoolean mediumRangeAuto,
                                                          ViReal64  mediumRangePower);

ViStatus _VI_FUNC rsspecan_Configure5GNRDLAnalysisSweepBSCategoryBOption(
                            ViSession instrSession,
                            ViInt32   categoryBOption);

ViStatus _VI_FUNC rsspecan_Configure5GNRDLAnalysisSweepSEMPosition(
                            ViSession instrSession,
                            ViInt32   n_TABconnectors,
                            ViInt32   n_TXU);

ViStatus _VI_FUNC rsspecan_Query5GNRDLACPChannelLimitCheckResults(
                            ViSession instrSession,
                            ViInt32   channelType,
                            ViInt32   limitCheckType,
                            ViInt32   results[]);

ViStatus _VI_FUNC rsspecan_Configure5GNRDLAnalysisSweepTotalLimitPassMode(
                            ViSession instrSession,
                            ViInt32   totalLimitPassMode);

ViStatus _VI_FUNC rsspecan_Query5GNRDLACLRLimitCheckResults(ViSession instrSession,
                                                            ViInt32   channelType,
                                                            ViInt32   limitCheckType,
                                                            ViInt32   results[]);

ViStatus _VI_FUNC rsspecan_Query5GNRDLACLRPowerLimitCheckResults(
                            ViSession instrSession,
                            ViInt32   limitCheckType,
                            ViInt32   results[]);

ViStatus _VI_FUNC rsspecan_Query5GNRDLCACLRLimitCheckResults(ViSession instrSession,
                                                             ViInt32   limitCheckType,
                                                             ViInt32   results[]);

ViStatus _VI_FUNC rsspecan_Configure5GNRDLAnalysisViewCarrier(
                            ViSession instrSession,
                            ViInt32   view,
                            ViInt32   carrier);

ViStatus _VI_FUNC rsspecan_Configure5GNRDLAnalysisViewFrame(ViSession instrSession,
                                                            ViInt32   view,
                                                            ViInt32   frame);

ViStatus _VI_FUNC rsspecan_Configure5GNRDLAnalysisViewWindowCoupling(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   view,
                            ViBoolean coupling);

ViStatus _VI_FUNC rsspecan_Configure5GNRDLAnalysisViewSelectResultWindow(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   view);

ViStatus _VI_FUNC rsspecan_Query5GNRDLOffPeriodsLimitCheckResult(
                            ViSession instrSession,
                            ViInt32   arraySize,
                            ViBoolean results[],
                            ViInt32*  returnedValues);

ViStatus _VI_FUNC rsspecan_Query5GNRDLTransientLimitCheckResult(
                            ViSession instrSession,
                            ViInt32   type,
                            ViInt32   arraySize,
                            ViBoolean results[],
                            ViInt32*  returnedValues);

ViStatus _VI_FUNC rsspecan_Configure5GNRDLCombinedMeasurementEventFilter(
                            ViSession instrSession,
                            ViBoolean eventFilter);

ViStatus _VI_FUNC rsspecan_Reset5GNRDLCombinedMeasurementEventFilter(ViSession instrSession);

ViStatus _VI_FUNC rsspecan_Configure5GNRDLCombinedMeasurementEventFilterParameters(
                            ViSession instrSession,
                            ViInt32   result,
                            ViInt32   condition,
                            ViReal64  value);

ViStatus _VI_FUNC rsspecan_Configure5GNRDLCombinedMeasurementEventFilterParametersEnabled(
                            ViSession instrSession,
                            ViInt32   result,
                            ViBoolean state);

ViStatus _VI_FUNC rsspecan_Query5GNRDLMeasurementTimeAlignmentError(
                            ViSession instrSession,
                            ViInt32   componentCarrier,
                            ViInt32   antenna,
                            ViInt32   measurementType,
                            ViReal64* result);

ViStatus _VI_FUNC rsspecan_Fetch5GNRDLAnalysisCaptureBuffer(ViSession instrSession,
                                                            ViInt32   window,
                                                            ViInt32   arraySize,
                                                            ViReal64  result[],
                                                            ViInt32*  returnedValues);

ViStatus _VI_FUNC rsspecan_Fetch5GNRDLAnalysisEVMvsCarrier(ViSession instrSession,
                                                           ViInt32   window,
                                                           ViInt32   trace,
                                                           ViInt32   arraySize,
                                                           ViReal64  results[],
                                                           ViInt32*  returnedValues);

ViStatus _VI_FUNC rsspecan_Fetch5GNRDLAnalysisEVMvsSymbol(ViSession instrSession,
                                                          ViInt32   window,
                                                          ViInt32   trace,
                                                          ViInt32   arraySize,
                                                          ViReal64  results[],
                                                          ViInt32*  returnedValues);

ViStatus _VI_FUNC rsspecan_Fetch5GNRDLAnalysisEVMvsRB(ViSession instrSession,
                                                      ViInt32   window,
                                                      ViInt32   trace,
                                                      ViInt32   arraySize,
                                                      ViReal64  results[],
                                                      ViInt32*  returnedValues);

ViStatus _VI_FUNC rsspecan_Fetch5GNRDLAnalysisFrequencyErrorvsSymbol(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   trace,
                            ViInt32   arraySize,
                            ViReal64  results[],
                            ViInt32*  returnedValues);

ViStatus _VI_FUNC rsspecan_Fetch5GNRDLAnalysisPowerSpectrum(ViSession instrSession,
                                                            ViInt32   window,
                                                            ViInt32   arraySize,
                                                            ViReal64  results[],
                                                            ViInt32*  returnedValues);

ViStatus _VI_FUNC rsspecan_Fetch5GNRDLAnalysisFlatnessvsCarrier(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   trace,
                            ViInt32   arraySize,
                            ViReal64  results[],
                            ViInt32*  returnedValues);

ViStatus _VI_FUNC rsspecan_Fetch5GNRDLAnalysisCCDF(ViSession instrSession,
                                                   ViInt32   window,
                                                   ViInt32   axis,
                                                   ViInt32   arraySize,
                                                   ViReal64  results[],
                                                   ViInt32*  returnedValues);

ViStatus _VI_FUNC rsspecan_Fetch5GNRDLAnalysisConstellation(ViSession instrSession,
                                                            ViInt32   window,
                                                            ViInt32   arraySize,
                                                            ViReal64  constellationI[],
                                                            ViReal64  constellationQ[],
                                                            ViInt32*  returnedValues);

ViStatus _VI_FUNC rsspecan_Fetch5GNRDLAnalysisAllocationSummary(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   arraySize,
                            ViInt32   BWPart[],
                            ViInt32   subframe[],
                            ViInt32   slot[],
                            ViInt32   allocationID[],
                            ViInt32   numberOfRB[],
                            ViReal64  relativePower[],
                            ViInt32   modulation[],
                            ViReal64  absolutePower[],
                            ViReal64  EVM[],
                            ViInt32*  returnedValues);

ViStatus _VI_FUNC rsspecan_Fetch5GNRDLAnalysisBitstream(ViSession instrSession,
                                                        ViInt32   window,
                                                        ViInt32   format,
                                                        ViInt32   arraySize,
                                                        ViChar    results[],
                                                        ViInt32*  returnedValues);

ViStatus _VI_FUNC rsspecan_Fetch5GNRDLAnalysisChannelDecoderResults(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   arraySize,
                            ViChar    results[],
                            ViInt32*  returnedValues);

ViStatus _VI_FUNC rsspecan_Fetch5GNRDLAnalysisEVMvsSymbolxCarrier(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   trace,
                            ViInt32   arraySize,
                            ViReal64  results[],
                            ViInt32*  returnedValues);

ViStatus _VI_FUNC rsspecan_Fetch5GNRDLAnalysisPowervsSymbolxCarrier(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   trace,
                            ViInt32   arraySize,
                            ViReal64  results[],
                            ViInt32*  returnedValues);

ViStatus _VI_FUNC rsspecan_Fetch5GNRDLAnalysisAllocationIDvsSymbolxCarrier(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   trace,
                            ViInt32   arraySize,
                            ViInt32   results[],
                            ViInt32*  returnedValues);

ViStatus _VI_FUNC rsspecan_Fetch5GNRDLAnalysisRSPhase(ViSession instrSession,
                                                      ViInt32   window,
                                                      ViInt32   trace,
                                                      ViInt32   arraySize,
                                                      ViReal64  results[],
                                                      ViInt32*  returnedValues);

ViStatus _VI_FUNC rsspecan_Fetch5GNRDLAnalysisRSPhaseDifference(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   trace,
                            ViInt32   arraySize,
                            ViReal64  results[],
                            ViInt32*  returnedValues);

ViStatus _VI_FUNC rsspecan_Fetch5GNRDLAnalysisACLR(ViSession instrSession,
                                                   ViInt32   window,
                                                   ViInt32   arraySize,
                                                   ViReal64  results[],
                                                   ViInt32*  returnedValues);

ViStatus _VI_FUNC rsspecan_Fetch5GNRDLAnalysisSEM(ViSession instrSession,
                                                  ViInt32   window,
                                                  ViInt32   arraySize,
                                                  ViReal64  results[],
                                                  ViInt32*  returnedValues);

ViStatus _VI_FUNC rsspecan_Fetch5GNRDLAnalysisSEMTable(ViSession instrSession,
                                                       ViInt32   window,
                                                       ViInt32   arraySize,
                                                       ViInt32   index[],
                                                       ViReal64  startFrequency[],
                                                       ViReal64  stopFrequency[],
                                                       ViReal64  RBW[],
                                                       ViReal64  limitFailFreq[],
                                                       ViReal64  absolutePower[],
                                                       ViReal64  relativePower[],
                                                       ViReal64  limitDistance[],
                                                       ViInt32   limitCheckResult[],
                                                       ViReal64  reserved1[],
                                                       ViReal64  reserved2[],
                                                       ViInt32*  returnedValues);

ViStatus _VI_FUNC rsspecan_Fetch5GNRDLAnalysisOnOffPower(ViSession instrSession,
                                                         ViInt32   window,
                                                         ViInt32   trace,
                                                         ViInt32   arraySize,
                                                         ViReal64  results[],
                                                         ViInt32*  returnedValues);

ViStatus _VI_FUNC rsspecan_Fetch5GNRDLAnalysisOnOffPowerTable(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   arraySize,
                            ViReal64  offPeriodStartLimit[],
                            ViReal64  offPeriodStopLimit[],
                            ViReal64  timeAtDeltaToLimit[],
                            ViReal64  absoluteOffPower[],
                            ViReal64  distanceToLimit[],
                            ViReal64  fallingTransientPeriod[],
                            ViReal64  risingTransientPeriod[],
                            ViInt32*  returnedValues);

ViStatus _VI_FUNC rsspecan_Fetch5GNRDLAnalysisCORESET(ViSession instrSession,
                                                      ViInt32   window,
                                                      ViInt32   arraySize,
                                                      ViReal64  results[],
                                                      ViInt32*  returnedValues);

ViStatus _VI_FUNC rsspecan_Fetch5GNRDLAnalysisCombinedMeasurement(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   trace,
                            ViInt32   arraySize,
                            ViReal64  results[],
                            ViInt32*  returnedValues);

ViStatus _VI_FUNC rsspecan_Configure5GNRUL(ViSession instrSession,
                                           ViBoolean multicarrierFilter,
                                           ViInt32   componentCarriers,
                                           ViInt32   carrierMode);

ViStatus _VI_FUNC rsspecan_Configure5GNRULDataCapture(ViSession instrSession,
                                                      ViBoolean frameCountEnabled,
                                                      ViBoolean autoFrameCount,
                                                      ViInt32   frameCount);

ViStatus _VI_FUNC rsspecan_Configure5GNRULTabSelect(ViSession instrSession,
                                                    ViInt32   window,
                                                    ViInt32   tab);

ViStatus _VI_FUNC rsspecan_Configure5GNRULDemodulation(ViSession instrSession,
                                                       ViInt32   demodulationData);

ViStatus _VI_FUNC rsspecan_Configure5GNRULDemodulationEVMCalculationMethod(
                            ViSession instrSession,
                            ViInt32   EVMCalculationMethod);

ViStatus _VI_FUNC rsspecan_Configure5GNRULDemodulationExtendedFrequencyLockRange(
                            ViSession instrSession,
                            ViBoolean extendedFrequencyLockRange);

ViStatus _VI_FUNC rsspecan_Query5GNRULTraceType(ViSession instrSession,
                                                ViInt32   window,
                                                ViInt32   arraySize,
                                                ViChar    type[]);

ViStatus _VI_FUNC rsspecan_Query5GNRULMarkerZAll(ViSession instrSession,
                                                 ViInt32   window,
                                                 ViInt32   arraySize,
                                                 ViReal64  allResults[],
                                                 ViInt32   marker,
                                                 ViInt32*  returnedValues);

ViStatus _VI_FUNC rsspecan_Configure5GNRULLongCapture(ViSession instrSession,
                                                      ViBoolean longCapture);

ViStatus _VI_FUNC rsspecan_Configure5GNRULSegmentCapture(ViSession instrSession,
                                                         ViBoolean segmentCapture,
                                                         ViInt32   events,
                                                         ViReal64  offset,
                                                         ViReal64  length);

ViStatus _VI_FUNC rsspecan_Configure5GNRULYScalingAuto(ViSession instrSession,
                                                       ViInt32   window,
                                                       ViInt32   subwindow,
                                                       ViInt32   autoScaling);

ViStatus _VI_FUNC rsspecan_Configure5GNRULYScaling(ViSession instrSession,
                                                   ViInt32   window,
                                                   ViInt32   subwindow,
                                                   ViReal64  maximum,
                                                   ViReal64  minimum);

ViStatus _VI_FUNC rsspecan_Configure5GNRULTracking(ViSession instrSession,
                                                   ViInt32   method,
                                                   ViBoolean gainImbalanceCalculation,
                                                   ViInt32   trackingPhase);

ViStatus _VI_FUNC rsspecan_Configure5GNRULTrackingLevel(ViSession instrSession,
                                                        ViBoolean trackingLevel);

ViStatus _VI_FUNC rsspecan_Configure5GNRULTrackingTime(ViSession instrSession,
                                                       ViBoolean trackingTime);

ViStatus _VI_FUNC rsspecan_Configure5GNRULAutomaticallyDemodulatesOnce(ViSession instrSession);

ViStatus _VI_FUNC rsspecan_Configure5GNRULConstellationColor(ViSession instrSession,
                                                             ViInt32   colors);

ViStatus _VI_FUNC rsspecan_Configure5GNRULBoostingFactor(ViSession instrSession,
                                                         ViBoolean enabled);

ViStatus _VI_FUNC rsspecan_Configure5GNRULPUSCHReferenceData(ViSession instrSession,
                                                             ViInt32   referenceData);

ViStatus _VI_FUNC rsspecan_Configure5GNRULDemodulationFilter(ViSession instrSession,
                                                             ViInt32   demodulationFilter);

ViStatus _VI_FUNC rsspecan_Configure5GNRULBitStreamFormat(ViSession instrSession,
                                                          ViInt32   bitStreamFormat);

ViStatus _VI_FUNC rsspecan_Configure5GNRULPhysicalSettings(ViSession instrSession,
                                                           ViInt32   operatingBand);

ViStatus _VI_FUNC rsspecan_Configure5GNRULChannelRaster(ViSession instrSession,
                                                        ViInt32   channelRaster);

ViStatus _VI_FUNC rsspecan_Configure5GNRULMCSetup(ViSession instrSession,
                                                  ViReal64  globalMCFrequency,
                                                  ViInt32   frequencyOffsetRelativeTo,
                                                  ViInt32   frequencyOffsetMode,
                                                  ViReal64  carrierSpacing,
                                                  ViBoolean fixedCCOffset);

ViStatus _VI_FUNC rsspecan_Configure5GNRULGlobalMCFrequencyOffset(
                            ViSession instrSession,
                            ViInt32   CC,
                            ViReal64  globalMCFrequencyOffset);

ViStatus _VI_FUNC rsspecan_NR5GULSyncMCFrequencyToCenter(ViSession instrSession);

ViStatus _VI_FUNC rsspecan_Configure5GNRULPhysicalSettingsLoadSignalDescription(
                            ViSession instrSession,
                            ViString  loadSignalDescription);

ViStatus _VI_FUNC rsspecan_Configure5GNRULPhysicalSettingsSaveSignalDescription(
                            ViSession instrSession,
                            ViString  saveSignalDescription);

ViStatus _VI_FUNC rsspecan_Configure5GNRULCCPhysicalSettingsAutomaticDemodulation(
                            ViSession instrSession,
                            ViInt32   CC,
                            ViInt32   automaticSignalDemodulation);

ViStatus _VI_FUNC rsspecan_Configure5GNRULCCPhysicalSettings(ViSession instrSession,
                                                             ViInt32   CC,
                                                             ViInt32   bandwidth,
                                                             ViInt32   deploymentFrequency,
                                                             ViReal64  centerFrequency,
                                                             ViReal64  generalFrequencyOffset);

ViStatus _VI_FUNC rsspecan_Configure5GNRULCCPhysicalSettingsCellID(
                            ViSession instrSession,
                            ViInt32   CC,
                            ViInt32   cellID);

ViStatus _VI_FUNC rsspecan_Configure5GNRULCCPhysicalSettingsDetection(
                            ViSession instrSession,
                            ViInt32   CC,
                            ViInt32   autoDetectionCellID);

ViStatus _VI_FUNC rsspecan_Configure5GNRULCCPhysicalSettingsPowerClass(
                            ViSession instrSession,
                            ViInt32   CC,
                            ViInt32   powerClass);

ViStatus _VI_FUNC rsspecan_Configure5GNRULCCPhysicalSettingsLoadSignalDescription(
                            ViSession instrSession,
                            ViInt32   CC,
                            ViString  fileName);

ViStatus _VI_FUNC rsspecan_Configure5GNRULCCPhysicalSettingsSaveSignalDescription(
                            ViSession instrSession,
                            ViInt32   CC,
                            ViString  fileName);

ViStatus _VI_FUNC rsspecan_Configure5GNRULCCPhysicalPRACHSettings(
                            ViSession instrSession,
                            ViInt32   CC,
                            ViInt32   format,
                            ViReal64  RBOffset,
                            ViInt32   restrictedSet,
                            ViInt32   zeroCorrelationZone,
                            ViInt32   PRACHSCS,
                            ViReal64  relPower,
                            ViInt32   logicalRootSequenceIndex,
                            ViInt32   preambleIndex);

ViStatus _VI_FUNC rsspecan_Configure5GNRULCCPhysicalPRACHLRA(ViSession instrSession,
                                                             ViInt32   CC,
                                                             ViInt32   LRA);

ViStatus _VI_FUNC rsspecan_Configure5GNRULBandwidthPartAdd(ViSession instrSession,
                                                           ViInt32   CC);

ViStatus _VI_FUNC rsspecan_Configure5GNRULBandwidthPartDelete(
                            ViSession instrSession,
                            ViInt32   CC,
                            ViInt32   bandwidthPart);

ViStatus _VI_FUNC rsspecan_Configure5GNRULBandwidthPartClear(ViSession instrSession,
                                                             ViInt32   CC);

ViStatus _VI_FUNC rsspecan_Configure5GNRULBandwidthPartDuplicate(
                            ViSession instrSession,
                            ViInt32   CC,
                            ViInt32   bandwidthPart);

ViStatus _VI_FUNC rsspecan_Query5GNRULBandwidthPartAnalyzed(ViSession instrSession,
                                                            ViInt32   CC,
                                                            ViInt32*  analyzedParts);

ViStatus _VI_FUNC rsspecan_Configure5GNRULBandwidthPartResourceBlocks(
                            ViSession instrSession,
                            ViInt32   CC,
                            ViInt32   bandwidthPart,
                            ViInt32   count,
                            ViInt32   offset);

ViStatus _VI_FUNC rsspecan_Configure5GNRULBandwidthPartSubcarrierSpacing(
                            ViSession instrSession,
                            ViInt32   CC,
                            ViInt32   bandwidthPart,
                            ViInt32   spacing);

ViStatus _VI_FUNC rsspecan_Configure5GNRULFrameCopy(ViSession instrSession,
                                                    ViInt32   CC,
                                                    ViInt32   frame);

ViStatus _VI_FUNC rsspecan_Configure5GNRULFramePaste(ViSession instrSession,
                                                     ViInt32   CC,
                                                     ViInt32   frame);

ViStatus _VI_FUNC rsspecan_Configure5GNRULFramePasteAll(ViSession instrSession,
                                                        ViInt32   CC);

ViStatus _VI_FUNC rsspecan_Configure5GNRULFrameNumberOfSlots(ViSession instrSession,
                                                             ViInt32   CC,
                                                             ViInt32   frame,
                                                             ViInt32   bandwidthPart,
                                                             ViInt32   slots);

ViStatus _VI_FUNC rsspecan_Configure5GNRULFrameCountAuto(ViSession instrSession,
                                                         ViBoolean autoFrameCount);

ViStatus _VI_FUNC rsspecan_Configure5GNRULFrameSlotAll(ViSession instrSession);

ViStatus _VI_FUNC rsspecan_Configure5GNRULFrameSlot(ViSession instrSession,
                                                    ViInt32   slot);

ViStatus _VI_FUNC rsspecan_Configure5GNRULRepeatedSlotAnalysis(
                            ViSession instrSession,
                            ViBoolean repeatedSlotAnalysis);

ViStatus _VI_FUNC rsspecan_Query5GNRULFrameNumberOfAnalyzedSlots(
                            ViSession instrSession,
                            ViInt32   CC,
                            ViInt32   frame,
                            ViInt32   bandwidthPart,
                            ViInt32*  slots);

ViStatus _VI_FUNC rsspecan_Configure5GNRULSlotCopyConfiguration(
                            ViSession instrSession,
                            ViInt32   CC,
                            ViInt32   frame,
                            ViInt32   bandwidthPart,
                            ViInt32   slot);

ViStatus _VI_FUNC rsspecan_Configure5GNRULSlotPasteAll(ViSession instrSession,
                                                       ViInt32   CC,
                                                       ViInt32   frame,
                                                       ViInt32   bandwidthPart,
                                                       ViInt32   slot);

ViStatus _VI_FUNC rsspecan_Configure5GNRULSlotPaste(ViSession instrSession,
                                                    ViInt32   CC,
                                                    ViInt32   frame,
                                                    ViInt32   bandwidthPart,
                                                    ViInt32   slot);

ViStatus _VI_FUNC rsspecan_Configure5GNRULSlotPreset(ViSession instrSession,
                                                     ViInt32   CC,
                                                     ViInt32   frame,
                                                     ViInt32   bandwidthPart);

ViStatus _VI_FUNC rsspecan_Configure5GNRULSlotPasteSettings(ViSession instrSession,
                                                            ViInt32   CC,
                                                            ViInt32   frame,
                                                            ViInt32   bandwidthPart,
                                                            ViInt32   slot,
                                                            ViInt32   mode,
                                                            ViInt32   period,
                                                            ViInt32   duration,
                                                            ViString  slots);

ViStatus _VI_FUNC rsspecan_NR5GULSlotPasteTo(ViSession instrSession,
                                             ViInt32   CC,
                                             ViInt32   frame,
                                             ViInt32   bandwidthPart,
                                             ViInt32   slot);

ViStatus _VI_FUNC rsspecan_Configure5GNRULSRSEnabled(ViSession instrSession,
                                                     ViInt32   CC,
                                                     ViInt32   frame,
                                                     ViInt32   bandwidthPart,
                                                     ViBoolean enabled);

ViStatus _VI_FUNC rsspecan_Configure5GNRULSRSSlotConfig(ViSession instrSession,
                                                        ViInt32   CC,
                                                        ViInt32   frame,
                                                        ViInt32   bandwidthPart,
                                                        ViInt32   periodicity,
                                                        ViReal64  offset);

ViStatus _VI_FUNC rsspecan_Configure5GNRULSRSStartPosition(ViSession instrSession,
                                                           ViInt32   CC,
                                                           ViInt32   frame,
                                                           ViInt32   bandwidthPart,
                                                           ViReal64  startPosition);

ViStatus _VI_FUNC rsspecan_Configure5GNRULSRSSymbols(ViSession instrSession,
                                                     ViInt32   CC,
                                                     ViInt32   frame,
                                                     ViInt32   bandwidthPart,
                                                     ViInt32   symbols);

ViStatus _VI_FUNC rsspecan_Configure5GNRULSRSFrequency(ViSession instrSession,
                                                       ViInt32   CC,
                                                       ViInt32   frame,
                                                       ViInt32   bandwidthPart,
                                                       ViReal64  position,
                                                       ViReal64  shift);

ViStatus _VI_FUNC rsspecan_Configure5GNRULSRSFrequencyHopping(
                            ViSession instrSession,
                            ViInt32   CC,
                            ViInt32   frame,
                            ViInt32   bandwidthPart,
                            ViReal64  bHop,
                            ViReal64  bSRS,
                            ViReal64  cSRS);

ViStatus _VI_FUNC rsspecan_Configure5GNRULSRSRepetitionFactor(
                            ViSession instrSession,
                            ViInt32   CC,
                            ViInt32   frame,
                            ViInt32   bandwidthPart,
                            ViInt32   repetitionFactor);

ViStatus _VI_FUNC rsspecan_Configure5GNRULSRSTransmissionComb(
                            ViSession instrSession,
                            ViInt32   CC,
                            ViInt32   frame,
                            ViInt32   bandwidthPart,
                            ViInt32   transmissionComb,
                            ViReal64  offset,
                            ViReal64  cyclicShift,
                            ViInt32   sequenceID,
                            ViInt32   hopping);

ViStatus _VI_FUNC rsspecan_Configure5GNRULSRSPorts(ViSession instrSession,
                                                   ViInt32   CC,
                                                   ViInt32   frame,
                                                   ViInt32   bandwidthPart,
                                                   ViInt32   ports);

ViStatus _VI_FUNC rsspecan_Configure5GNRULSRSRelativePower(ViSession instrSession,
                                                           ViInt32   CC,
                                                           ViInt32   frame,
                                                           ViInt32   bandwidthPart,
                                                           ViReal64  relativePower);

ViStatus _VI_FUNC rsspecan_Configure5GNRULEnhancedPUSCHUserID(
                            ViSession instrSession,
                            ViInt32   CC,
                            ViInt32   frame,
                            ViInt32   bandwidthPart,
                            ViInt32   slot,
                            ViInt32   allocation,
                            ViInt32   userID);

ViStatus _VI_FUNC rsspecan_Configure5GNRULEnhancedPUSCHMapping(
                            ViSession instrSession,
                            ViInt32   CC,
                            ViInt32   frame,
                            ViInt32   bandwidthPart,
                            ViInt32   slot,
                            ViInt32   allocation,
                            ViInt32   mapping);

ViStatus _VI_FUNC rsspecan_Configure5GNRULEnhancedPUSCHPower(ViSession instrSession,
                                                             ViInt32   CC,
                                                             ViInt32   frame,
                                                             ViInt32   bandwidthPart,
                                                             ViInt32   slot,
                                                             ViInt32   allocation,
                                                             ViReal64  power);

ViStatus _VI_FUNC rsspecan_Configure5GNRULEnhancedPUSCHSettings(
                            ViSession instrSession,
                            ViInt32   CC,
                            ViInt32   frame,
                            ViInt32   bandwidthPart,
                            ViInt32   slot,
                            ViInt32   allocation,
                            ViString  antennaPorts,
                            ViInt32   CDMGroupsWithoutData);

ViStatus _VI_FUNC rsspecan_Configure5GNRULEnhancedPUSCHPTRS(ViSession instrSession,
                                                            ViInt32   CC,
                                                            ViInt32   frame,
                                                            ViInt32   bandwidthPart,
                                                            ViInt32   slot,
                                                            ViInt32   allocation,
                                                            ViBoolean enabled,
                                                            ViReal64  power,
                                                            ViInt32   timeDomainOffset,
                                                            ViInt32   frequencyDomainOffset,
                                                            ViInt32   relativeOffset,
                                                            ViInt32   group,
                                                            ViInt32   samples);

ViStatus _VI_FUNC rsspecan_Configure5GNRULEnhancedPUSCHPTRSNID(
                            ViSession instrSession,
                            ViInt32   CC,
                            ViInt32   frame,
                            ViInt32   bandwidthPart,
                            ViInt32   slot,
                            ViInt32   allocation,
                            ViInt32   n_ID,
                            ViInt32   seedValue);

ViStatus _VI_FUNC rsspecan_Configure5GNRULEnhancedPUSCHDMRS(ViSession instrSession,
                                                            ViInt32   CC,
                                                            ViInt32   frame,
                                                            ViInt32   bandwidthPart,
                                                            ViInt32   slot,
                                                            ViInt32   allocation,
                                                            ViInt32   type,
                                                            ViInt32   aditional,
                                                            ViInt32   length,
                                                            ViInt32   mappingType,
                                                            ViReal64  power,
                                                            ViInt32   sequenceGenerationMethod,
                                                            ViInt32   seedValue,
                                                            ViInt32   firstSymbol);

ViStatus _VI_FUNC rsspecan_Configure5GNRULEnhancedPUSCHDMRSNIDRS(
                            ViSession instrSession,
                            ViInt32   CC,
                            ViInt32   frame,
                            ViInt32   bandwidthPart,
                            ViInt32   slot,
                            ViInt32   allocation,
                            ViInt32   sequenceGenerationMethod,
                            ViInt32   seedValue);

ViStatus _VI_FUNC rsspecan_Configure5GNRULEnhancedPUSCHDMRSUplink(
                            ViSession instrSession,
                            ViInt32   CC,
                            ViInt32   frame,
                            ViInt32   bandwidthPart,
                            ViInt32   slot,
                            ViInt32   allocation,
                            ViBoolean DMRSUplink);

ViStatus _VI_FUNC rsspecan_Configure5GNRULPUSCHAllocations(ViSession instrSession,
                                                           ViInt32   CC,
                                                           ViInt32   frame,
                                                           ViInt32   bandwidthPart,
                                                           ViInt32   slot,
                                                           ViInt32   allocation);

ViStatus _VI_FUNC rsspecan_Configure5GNRDLPUSCHAllocationType(
                            ViSession instrSession,
                            ViInt32   CC,
                            ViInt32   frame,
                            ViInt32   bandwidthPart,
                            ViInt32   slot,
                            ViInt32   slotType);

ViStatus _VI_FUNC rsspecan_Configure5GNRULPUSCH(ViSession instrSession,
                                                ViInt32   CC,
                                                ViInt32   frame,
                                                ViInt32   bandwidthPart,
                                                ViInt32   slot,
                                                ViInt32   allocation,
                                                ViInt32   modulation);

ViStatus _VI_FUNC rsspecan_Configure5GNRULPUSCHResourceBlocks(
                            ViSession instrSession,
                            ViInt32   CC,
                            ViInt32   frame,
                            ViInt32   bandwidthPart,
                            ViInt32   slot,
                            ViInt32   allocation,
                            ViInt32   count,
                            ViInt32   offset);

ViStatus _VI_FUNC rsspecan_Configure5GNRULPUSCHSymbol(ViSession instrSession,
                                                      ViInt32   CC,
                                                      ViInt32   frame,
                                                      ViInt32   bandwidthPart,
                                                      ViInt32   slot,
                                                      ViInt32   allocation,
                                                      ViInt32   count,
                                                      ViInt32   offset);

ViStatus _VI_FUNC rsspecan_Configure5GNRULPUSCHChannelCoding(ViSession instrSession,
                                                             ViInt32   CC,
                                                             ViInt32   frame,
                                                             ViInt32   bandwidthPart,
                                                             ViInt32   slot,
                                                             ViInt32   allocation,
                                                             ViInt32   order,
                                                             ViInt32   MCSIndex,
                                                             ViInt32   redundancyVersion);

ViStatus _VI_FUNC rsspecan_Configure5GNRULPUSCHScrambling(ViSession instrSession,
                                                          ViInt32   CC,
                                                          ViInt32   frame,
                                                          ViInt32   bandwidthPart,
                                                          ViInt32   slot,
                                                          ViInt32   allocation,
                                                          ViInt32   method,
                                                          ViInt32   defineSeed);

ViStatus _VI_FUNC rsspecan_Configure5GNRULPUSCHCopySettings(ViSession instrSession,
                                                            ViInt32   CC,
                                                            ViInt32   frame,
                                                            ViInt32   bandwidthPart,
                                                            ViInt32   slot,
                                                            ViInt32   allocation,
                                                            ViInt32   mode,
                                                            ViInt32   period,
                                                            ViInt32   duration,
                                                            ViString  slots);

ViStatus _VI_FUNC rsspecan_NR5GULPUSCHCopy(ViSession instrSession,
                                           ViInt32   CC,
                                           ViInt32   frame,
                                           ViInt32   bandwidthPart,
                                           ViInt32   slot,
                                           ViInt32   allocation);

ViStatus _VI_FUNC rsspecan_Configure5GNRULPUCCHAllocations(ViSession instrSession,
                                                           ViInt32   CC,
                                                           ViInt32   frame,
                                                           ViInt32   bandwidthPart,
                                                           ViInt32   slot,
                                                           ViInt32   PUCCHAllocations);

ViStatus _VI_FUNC rsspecan_Configure5GNRULPUCCHResourceBlocks(
                            ViSession instrSession,
                            ViInt32   CC,
                            ViInt32   frame,
                            ViInt32   bandwidthPart,
                            ViInt32   slot,
                            ViInt32   PUCCH,
                            ViInt32   resourceBlocks,
                            ViInt32   offsetRB);

ViStatus _VI_FUNC rsspecan_Configure5GNRULPUCCHSymbol(ViSession instrSession,
                                                      ViInt32   CC,
                                                      ViInt32   frame,
                                                      ViInt32   bandwidthPart,
                                                      ViInt32   slot,
                                                      ViInt32   PUCCH,
                                                      ViInt32   symbols,
                                                      ViInt32   offset);

ViStatus _VI_FUNC rsspecan_Configure5GNRULPUCCHRelativePower(ViSession instrSession,
                                                             ViInt32   CC,
                                                             ViInt32   frame,
                                                             ViInt32   bandwidthPart,
                                                             ViInt32   slot,
                                                             ViInt32   PUCCH,
                                                             ViReal64  relativePower);

ViStatus _VI_FUNC rsspecan_Configure5GNRULPUCCHFormat(ViSession instrSession,
                                                      ViInt32   CC,
                                                      ViInt32   frame,
                                                      ViInt32   bandwidthPart,
                                                      ViInt32   slot,
                                                      ViInt32   PUCCH,
                                                      ViInt32   format);

ViStatus _VI_FUNC rsspecan_Configure5GNRULPUCCHDMRSRelativePower(
                            ViSession instrSession,
                            ViInt32   CC,
                            ViInt32   frame,
                            ViInt32   bandwidthPart,
                            ViInt32   slot,
                            ViInt32   PUCCH,
                            ViReal64  DMRSRelativePower);

ViStatus _VI_FUNC rsspecan_Configure5GNRULPUCCHGroupHopping(ViSession instrSession,
                                                            ViInt32   CC,
                                                            ViInt32   frame,
                                                            ViInt32   bandwidthPart,
                                                            ViInt32   slot,
                                                            ViInt32   PUCCH,
                                                            ViInt32   groupHopping,
                                                            ViInt32   hoppingID,
                                                            ViInt32   seedValue);

ViStatus _VI_FUNC rsspecan_Configure5GNRULPUCCHIntraSlotFrequencyHopping(
                            ViSession instrSession,
                            ViInt32   CC,
                            ViInt32   frame,
                            ViInt32   bandwidthPart,
                            ViInt32   slot,
                            ViInt32   PUCCH,
                            ViBoolean intraSlotFrequencyHopping,
                            ViInt32   secondHopPRB);

ViStatus _VI_FUNC rsspecan_Configure5GNRULPUCCHInitialCyclicShift(
                            ViSession instrSession,
                            ViInt32   CC,
                            ViInt32   frame,
                            ViInt32   bandwidthPart,
                            ViInt32   slot,
                            ViInt32   PUCCH,
                            ViInt32   initialCyclicShift);

ViStatus _VI_FUNC rsspecan_Configure5GNRULPUCCHTimeDomainOCCIndex(
                            ViSession instrSession,
                            ViInt32   CC,
                            ViInt32   frame,
                            ViInt32   bandwidthPart,
                            ViInt32   slot,
                            ViInt32   PUCCH,
                            ViInt32   timeDomainOCCIndex);

ViStatus _VI_FUNC rsspecan_Configure5GNRULPUCCHTimeDomainOCCLength(
                            ViSession instrSession,
                            ViInt32   CC,
                            ViInt32   frame,
                            ViInt32   bandwidthPart,
                            ViInt32   slot,
                            ViInt32   PUCCH,
                            ViInt32   OCCLength);

ViStatus _VI_FUNC rsspecan_Configure5GNRULPUCCHAdditionalEnabled(
                            ViSession instrSession,
                            ViInt32   CC,
                            ViInt32   frame,
                            ViInt32   bandwidthPart,
                            ViInt32   slot,
                            ViInt32   PUCCH,
                            ViBoolean enabled);

ViStatus _VI_FUNC rsspecan_Configure5GNRULPUCCHDMRSSequenceGeneration(
                            ViSession instrSession,
                            ViInt32   CC,
                            ViInt32   frame,
                            ViInt32   bandwidthPart,
                            ViInt32   slot,
                            ViInt32   PUCCH,
                            ViInt32   sequenceGeneration,
                            ViInt32   seedValue);

ViStatus _VI_FUNC rsspecan_Configure5GNRULPUCCHCopySettings(ViSession instrSession,
                                                            ViInt32   CC,
                                                            ViInt32   frame,
                                                            ViInt32   bandwidthPart,
                                                            ViInt32   slot,
                                                            ViInt32   PUCCH,
                                                            ViInt32   mode,
                                                            ViInt32   period,
                                                            ViInt32   duration,
                                                            ViString  slots);

ViStatus _VI_FUNC rsspecan_NR5GULPUCCHCopy(ViSession instrSession,
                                           ViInt32   CC,
                                           ViInt32   frame,
                                           ViInt32   bandwidthPart,
                                           ViInt32   slot,
                                           ViInt32   PUCCH);

ViStatus _VI_FUNC rsspecan_Configure5GNRULNumberOfConfigurableFrames(
                            ViSession instrSession,
                            ViInt32   CC,
                            ViInt32   numberOfConfigurableFrames);

ViStatus _VI_FUNC rsspecan_Configure5GNRULAdvancedSettingsRFUpconversion(
                            ViSession instrSession,
                            ViInt32   CC,
                            ViBoolean enabled,
                            ViInt32   mode,
                            ViReal64  frequency);

ViStatus _VI_FUNC rsspecan_Configure5GNRULAdvancedSettingsRefPointA(
                            ViSession instrSession,
                            ViInt32   CC,
                            ViReal64  centerFrequencyOffset,
                            ViInt32   k0Select,
                            ViInt32   offset);

ViStatus _VI_FUNC rsspecan_Query5GNRULAdvancedSettingsRefPointAFrequency(
                            ViSession instrSession,
                            ViInt32   CC,
                            ViReal64* frequency);

ViStatus _VI_FUNC rsspecan_Query5GNRULAdvancedSettingsRefPointATxBwOffset(
                            ViSession instrSession,
                            ViInt32   CC,
                            ViReal64* offset);

ViStatus _VI_FUNC rsspecan_Configure5GNRULAdvancedSettingsDCCarrierEnabled(
                            ViSession instrSession,
                            ViInt32   CC,
                            ViBoolean enabled);

ViStatus _VI_FUNC rsspecan_Configure5GNRULAdvancedSettings(ViSession instrSession,
                                                           ViInt32   CC,
                                                           ViBoolean precoding,
                                                           ViInt32   hopping);

ViStatus _VI_FUNC rsspecan_Configure5GNRULAdvancedSettingsPUSCHFrequencyHopping(
                            ViSession instrSession,
                            ViInt32   CC,
                            ViInt32   PUSCHFrequencyHopping);

ViStatus _VI_FUNC rsspecan_Configure5GNRULAdvancedSettingsFrameNumber(
                            ViSession instrSession,
                            ViInt32   frameNumber);

ViStatus _VI_FUNC rsspecan_Configure5GNRULAdvancedSettingsExtremeConditions(
                            ViSession instrSession,
                            ViInt32   CC,
                            ViBoolean extremeConditions);

ViStatus _VI_FUNC rsspecan_Configure5GNRULAntennaPortEnabled(ViSession instrSession,
                                                             ViInt32   CC,
                                                             ViInt32   antennaConfiguration,
                                                             ViBoolean enabled);

ViStatus _VI_FUNC rsspecan_Configure5GNRULAntennaPortSelect(ViSession instrSession,
                                                            ViInt32   CC,
                                                            ViInt32   antennaConfiguration,
                                                            ViInt32   antenna,
                                                            ViInt32   portSelect);

ViStatus _VI_FUNC rsspecan_Configure5GNRULGeneratorIPAddress(ViSession instrSession,
                                                             ViString  IPAddress);

ViStatus _VI_FUNC rsspecan_Query5GNRULGeneratorConnectionState(
                            ViSession instrSession,
                            ViInt32*  connectionState);

ViStatus _VI_FUNC rsspecan_Configure5GNRULGeneratorControlState(
                            ViSession instrSession,
                            ViBoolean state);

ViStatus _VI_FUNC rsspecan_Configure5GNRULGeneratorRFOutput(ViSession instrSession,
                                                            ViBoolean RFOutput);

ViStatus _VI_FUNC rsspecan_Configure5GNRULGeneratorFrequencySynchronization(
                            ViSession instrSession,
                            ViBoolean frequencySynchronization);

ViStatus _VI_FUNC rsspecan_Configure5GNRULGeneratorLevel(ViSession instrSession,
                                                         ViBoolean levelState,
                                                         ViReal64  level_RMS,
                                                         ViBoolean DUTLimitState,
                                                         ViReal64  DUTLimitPeakInputPower,
                                                         ViReal64  DUTGain);

ViStatus _VI_FUNC rsspecan_NR5GULGeneratorUploadRFSettings(ViSession instrSession);

ViStatus _VI_FUNC rsspecan_NR5GULGeneratorDownloadRFSettings(ViSession instrSession);

ViStatus _VI_FUNC rsspecan_NR5GULGeneratorDownload5GNRSettings(ViSession instrSession);

ViStatus _VI_FUNC rsspecan_Configure5GNRULGeneratorSynchronization(
                            ViSession instrSession,
                            ViBoolean syncEnabled,
                            ViReal64  pollingInterval);

ViStatus _VI_FUNC rsspecan_Configure5GNRULInputType(ViSession instrSession,
                                                    ViInt32   connector,
                                                    ViInt32   inputType);

ViStatus _VI_FUNC rsspecan_Configure5GNRULInputCoupling(ViSession instrSession,
                                                        ViInt32   connector,
                                                        ViBoolean coupling);

ViStatus _VI_FUNC rsspecan_Configure5GNRULInputSampleRate(ViSession instrSession,
                                                          ViInt32   connector,
                                                          ViBoolean enabled,
                                                          ViReal64  sampleRate);

ViStatus _VI_FUNC rsspecan_Configure5GNRULInputFullScaleLevel(
                            ViSession instrSession,
                            ViInt32   connector,
                            ViBoolean autoLevel,
                            ViInt32   unit,
                            ViReal64  level);

ViStatus _VI_FUNC rsspecan_Configure5GNRULInputIQ(ViSession instrSession,
                                                  ViInt32   connector,
                                                  ViBoolean balancedEnabled,
                                                  ViInt32   type);

ViStatus _VI_FUNC rsspecan_Configure5GNRULCombinedMeasurementACLR(
                            ViSession instrSession,
                            ViBoolean ACLR,
                            ViBoolean EVM,
                            ViBoolean SEM);

ViStatus _VI_FUNC rsspecan_Load5GNRULCombinedMeasurementACLRSettings(
                            ViSession instrSession,
                            ViString  settingsFile);

ViStatus _VI_FUNC rsspecan_Clear5GNRULCombinedMeasurementACLRSettings(ViSession instrSession);

ViStatus _VI_FUNC rsspecan_Load5GNRULCombinedMeasurementSEMSettings(
                            ViSession instrSession,
                            ViString  settingsFile);

ViStatus _VI_FUNC rsspecan_Clear5GNRULCombinedMeasurementSEMSettings(ViSession instrSession);

ViStatus _VI_FUNC rsspecan_Configure5GNRULCombinedMeasurementSignalCapture(
                            ViSession instrSession,
                            ViInt32   numberOfEvents,
                            ViInt32   optimization,
                            ViBoolean sequenceAutoLevel);

ViStatus _VI_FUNC rsspecan_Configure5GNRULCombinedMeasurementTriggerSequence(
                            ViSession instrSession,
                            ViInt32   sequence);

ViStatus _VI_FUNC rsspecan_Configure5GNRULCombinedMeasurementFrameTrigger(
                            ViSession instrSession,
                            ViInt32   frameTrigger);

ViStatus _VI_FUNC rsspecan_Configure5GNRULCombinedMeasurementTriggerEventDelay(
                            ViSession instrSession,
                            ViReal64  eventDelay);

ViStatus _VI_FUNC rsspecan_Configure5GNRULCombinedMeasurementTriggerRecalibrationInterval(
                            ViSession instrSession,
                            ViInt32   recalibrationInterval);

ViStatus _VI_FUNC rsspecan_Send5GNRULCombinedMeasurementTrigger3(ViSession instrSession);

ViStatus _VI_FUNC rsspecan_Configure5GNRULCombinedMeasurementTriggerAutoGating(
                            ViSession instrSession,
                            ViBoolean autoGating);

ViStatus _VI_FUNC rsspecan_Query5GNRULCombinedMeasurementFilteredEvents(
                            ViSession instrSession,
                            ViInt32*  filteredEvents);

ViStatus _VI_FUNC rsspecan_Configure5GNRULCombinedMeasurementRestoreSettings(ViSession instrSession);

ViStatus _VI_FUNC rsspecan_Add5GNRULWindow(ViSession instrSession,
                                           ViInt32   stringSize,
                                           ViString  referenceWindowName,
                                           ViInt32   direction,
                                           ViChar    returnedWindowName[],
                                           ViInt32   windowType);

ViStatus _VI_FUNC rsspecan_Replace5GNRULWindow(ViSession instrSession,
                                               ViString  windowName,
                                               ViInt32   windowType);

ViStatus _VI_FUNC rsspecan_Configure5GNRULAnalysisShowResultSummary(
                            ViSession instrSession,
                            ViInt32   summary);

ViStatus _VI_FUNC rsspecan_Configure5GNRULAnalysisShowResultSummaryItems(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   item,
                            ViBoolean displayItem);

ViStatus _VI_FUNC rsspecan_Configure5GNRULAnalysisXScale(ViSession instrSession,
                                                         ViInt32   xScale);

ViStatus _VI_FUNC rsspecan_Configure5GNRULAnalysisXAxis(ViSession instrSession,
                                                        ViInt32   xAxis,
                                                        ViInt32   referenceXAxis);

ViStatus _VI_FUNC rsspecan_Configure5GNRULAnalysisEVMMaxHold(ViSession instrSession,
                                                             ViBoolean EVMMaxHold);

ViStatus _VI_FUNC rsspecan_Configure5GNRULAnalysis3DView(ViSession instrSession,
                                                         ViBoolean _3DView);

ViStatus _VI_FUNC rsspecan_Configure5GNRULAnalysisACLRLimitPassMode(
                            ViSession instrSession,
                            ViInt32   ACLRLimitPassMode);

ViStatus _VI_FUNC rsspecan_Configure5GNRULAnalysisMultipleCarriers(
                            ViSession instrSession,
                            ViInt32   multipleCarriers);

ViStatus _VI_FUNC rsspecan_Configure5GNRULAutomaticConfigurationEVM(ViSession instrSession);

ViStatus _VI_FUNC rsspecan_Configure5GNRULAutomaticEVMSlots(ViSession instrSession,
                                                            ViInt32   numberOfSlots);

ViStatus _VI_FUNC rsspecan_Configure5GNRULAnalysisEvaluationRangeSegment(
                            ViSession instrSession,
                            ViInt32   value);

ViStatus _VI_FUNC rsspecan_Query5GNRULAnalysisSynchronizationState(
                            ViSession instrSession,
                            ViInt32   CC,
                            ViInt32   arraySize,
                            ViChar    synchronizationState[]);

ViStatus _VI_FUNC rsspecan_Query5GNRULAnalysisCellID(ViSession instrSession,
                                                     ViInt32   CC,
                                                     ViInt32*  cellID);

ViStatus _VI_FUNC rsspecan_Query5GNRULAnalysisCrestFactor(ViSession instrSession,
                                                          ViInt32   CC,
                                                          ViReal64* crestFactor);

ViStatus _VI_FUNC rsspecan_Query5GNRULFrameStartOffset(ViSession instrSession,
                                                       ViInt32   CC,
                                                       ViReal64* startOffset);

ViStatus _VI_FUNC rsspecan_Configure5GNRULAnalysisPowerAveragingMode(
                            ViSession instrSession,
                            ViInt32   mode);

ViStatus _VI_FUNC rsspecan_Configure5GNRUThroughputMeasurementEnabled(
                            ViSession instrSession,
                            ViBoolean enabled);

ViStatus _VI_FUNC rsspecan_Configure5GNRULAnalysisFrequencySweepTestID(
                            ViSession instrSession,
                            ViInt32   testID);

ViStatus _VI_FUNC rsspecan_Configure5GNRULAnalysisEvaluationRangeAllocation(
                            ViSession instrSession,
                            ViInt32   mode,
                            ViInt32   value);

ViStatus _VI_FUNC rsspecan_Configure5GNRULAnalysisEvaluationRangeBandwidthPart(
                            ViSession instrSession,
                            ViInt32   mode,
                            ViInt32   value);

ViStatus _VI_FUNC rsspecan_Configure5GNRULAnalysisEvaluationRangeCarrier(
                            ViSession instrSession,
                            ViInt32   mode,
                            ViInt32   value);

ViStatus _VI_FUNC rsspecan_Configure5GNRULAnalysisEvaluationRangeFrame(
                            ViSession instrSession,
                            ViInt32   value);

ViStatus _VI_FUNC rsspecan_Configure5GNRULAnalysisEvaluationRangeSelectedMeasID(
                            ViSession instrSession,
                            ViInt32   selectedMeasID);

ViStatus _VI_FUNC rsspecan_Configure5GNRULAnalysisEvaluationRangeModulation(
                            ViSession instrSession,
                            ViInt32   mode,
                            ViInt32   value);

ViStatus _VI_FUNC rsspecan_Configure5GNRULAnalysisEvaluationRangeSlot(
                            ViSession instrSession,
                            ViInt32   mode,
                            ViInt32   value);

ViStatus _VI_FUNC rsspecan_Configure5GNRULAnalysisEvaluationRangePreamble(
                            ViSession instrSession,
                            ViInt32   mode,
                            ViInt32   value);

ViStatus _VI_FUNC rsspecan_Configure5GNRULAnalysisEvaluationRangeSubframe(
                            ViSession instrSession,
                            ViInt32   mode,
                            ViInt32   value);

ViStatus _VI_FUNC rsspecan_Configure5GNRULAnalysisEvaluationRangeSymbol(
                            ViSession instrSession,
                            ViInt32   mode,
                            ViInt32   value);

ViStatus _VI_FUNC rsspecan_Query5GNRULAnalysisAllCarriers(ViSession instrSession,
                                                          ViInt32   type,
                                                          ViReal64* result);

ViStatus _VI_FUNC rsspecan_Query5GNRULAnalysisSummaryAll(ViSession instrSession,
                                                         ViInt32   CC,
                                                         ViInt32   frame,
                                                         ViInt32   bufferSize,
                                                         ViChar    value[]);

ViStatus _VI_FUNC rsspecan_Query5GNRULAnalysisSummaryEVMAll(ViSession instrSession,
                                                            ViInt32   CC,
                                                            ViInt32   frame,
                                                            ViInt32   type,
                                                            ViReal64* value);

ViStatus _VI_FUNC rsspecan_Query5GNRULAnalysisSummaryEVMPhysicalChannel(
                            ViSession instrSession,
                            ViInt32   CC,
                            ViInt32   frame,
                            ViInt32   type,
                            ViReal64* value);

ViStatus _VI_FUNC rsspecan_Query5GNRULAnalysisSummaryEVMPhysicalSignal(
                            ViSession instrSession,
                            ViInt32   CC,
                            ViInt32   frame,
                            ViInt32   type,
                            ViReal64* value);

ViStatus _VI_FUNC rsspecan_Query5GNRULAnalysisSummaryEVMPeak(ViSession instrSession,
                                                             ViInt32   CC,
                                                             ViInt32   frame,
                                                             ViInt32   type,
                                                             ViReal64* value);

ViStatus _VI_FUNC rsspecan_Query5GNRULAnalysisSummaryEVMPRACH(
                            ViSession instrSession,
                            ViInt32   CC,
                            ViInt32   frame,
                            ViInt32   type,
                            ViReal64* value);

ViStatus _VI_FUNC rsspecan_Query5GNRULAnalysisSummaryEVMPI2BPSK(
                            ViSession instrSession,
                            ViInt32   CC,
                            ViInt32   frame,
                            ViInt32   type,
                            ViReal64* value);

ViStatus _VI_FUNC rsspecan_Query5GNRULAnalysisSummaryEVMQPSK(ViSession instrSession,
                                                             ViInt32   CC,
                                                             ViInt32   frame,
                                                             ViInt32   type,
                                                             ViReal64* value);

ViStatus _VI_FUNC rsspecan_Query5GNRULAnalysisSummaryEVM16QAM(
                            ViSession instrSession,
                            ViInt32   CC,
                            ViInt32   frame,
                            ViInt32   type,
                            ViReal64* value);

ViStatus _VI_FUNC rsspecan_Query5GNRULAnalysisSummaryEVM64QAM(
                            ViSession instrSession,
                            ViInt32   CC,
                            ViInt32   frame,
                            ViInt32   type,
                            ViReal64* value);

ViStatus _VI_FUNC rsspecan_Query5GNRULAnalysisSummaryEVM256QAM(
                            ViSession instrSession,
                            ViInt32   CC,
                            ViInt32   frame,
                            ViInt32   type,
                            ViReal64* value);

ViStatus _VI_FUNC rsspecan_Query5GNRULAnalysisSummaryEVMDMRSResourceElements(
                            ViSession instrSession,
                            ViInt32   CC,
                            ViInt32   frame,
                            ViInt32   type,
                            ViReal64* value);

ViStatus _VI_FUNC rsspecan_Query5GNRULAnalysisSummaryEVMResourceElements(
                            ViSession instrSession,
                            ViInt32   CC,
                            ViInt32   frame,
                            ViInt32   type,
                            ViReal64* value);

ViStatus _VI_FUNC rsspecan_Query5GNRULAnalysisSummaryEVMPUSCHPI2BPSK(
                            ViSession instrSession,
                            ViInt32   CC,
                            ViInt32   frame,
                            ViInt32   type,
                            ViReal64* value);

ViStatus _VI_FUNC rsspecan_Query5GNRULAnalysisSummaryEVMPUSCHQPSK(
                            ViSession instrSession,
                            ViInt32   CC,
                            ViInt32   frame,
                            ViInt32   type,
                            ViReal64* value);

ViStatus _VI_FUNC rsspecan_Query5GNRULAnalysisSummaryEVMPUSCH16QAM(
                            ViSession instrSession,
                            ViInt32   CC,
                            ViInt32   frame,
                            ViInt32   type,
                            ViReal64* value);

ViStatus _VI_FUNC rsspecan_Query5GNRULAnalysisSummaryEVMPUSCH64QAM(
                            ViSession instrSession,
                            ViInt32   CC,
                            ViInt32   frame,
                            ViInt32   type,
                            ViReal64* value);

ViStatus _VI_FUNC rsspecan_Query5GNRULAnalysisSummaryEVMPUSCH256QAM(
                            ViSession instrSession,
                            ViInt32   CC,
                            ViInt32   frame,
                            ViInt32   type,
                            ViReal64* value);

ViStatus _VI_FUNC rsspecan_Query5GNRULAnalysisSummaryEVMPUSCH1024QAM(
                            ViSession instrSession,
                            ViInt32   CC,
                            ViInt32   frame,
                            ViInt32   type,
                            ViReal64* value);

ViStatus _VI_FUNC rsspecan_Query5GNRULAnalysisSummaryEVMResourceElements1024QAM(
                            ViSession instrSession,
                            ViInt32   CC,
                            ViInt32   frame,
                            ViInt32   type,
                            ViReal64* value);

ViStatus _VI_FUNC rsspecan_Query5GNRULAnalysisSummaryEVMBLER(ViSession instrSession,
                                                             ViInt32   CC,
                                                             ViInt32   frame,
                                                             ViInt32   type,
                                                             ViReal64* value);

ViStatus _VI_FUNC rsspecan_Query5GNRULAnalysisSummaryEVMTPUT(ViSession instrSession,
                                                             ViInt32   CC,
                                                             ViInt32   frame,
                                                             ViInt32   type,
                                                             ViReal64* value);

ViStatus _VI_FUNC rsspecan_Query5GNRULAnalysisSummaryFrequencyError(
                            ViSession instrSession,
                            ViInt32   CC,
                            ViInt32   frame,
                            ViInt32   type,
                            ViReal64* value);

ViStatus _VI_FUNC rsspecan_Query5GNRULAnalysisSummaryGainImbalance(
                            ViSession instrSession,
                            ViInt32   CC,
                            ViInt32   frame,
                            ViInt32   type,
                            ViReal64* value);

ViStatus _VI_FUNC rsspecan_Query5GNRULAnalysisSummaryIQOffset(
                            ViSession instrSession,
                            ViInt32   CC,
                            ViInt32   frame,
                            ViInt32   type,
                            ViReal64* value);

ViStatus _VI_FUNC rsspecan_Query5GNRULAnalysisSummaryOSTP(ViSession instrSession,
                                                          ViInt32   CC,
                                                          ViInt32   frame,
                                                          ViInt32   type,
                                                          ViReal64* value);

ViStatus _VI_FUNC rsspecan_Query5GNRULAnalysisSummaryPower(ViSession instrSession,
                                                           ViInt32   CC,
                                                           ViInt32   frame,
                                                           ViInt32   type,
                                                           ViReal64* value);

ViStatus _VI_FUNC rsspecan_Query5GNRULAnalysisSummaryQuadratureError(
                            ViSession instrSession,
                            ViInt32   CC,
                            ViInt32   frame,
                            ViInt32   type,
                            ViReal64* value);

ViStatus _VI_FUNC rsspecan_Query5GNRULAnalysisSummarySamplingError(
                            ViSession instrSession,
                            ViInt32   CC,
                            ViInt32   frame,
                            ViInt32   type,
                            ViReal64* value);

ViStatus _VI_FUNC rsspecan_Query5GNRULAnalysisACLRLimitCheck(ViSession instrSession,
                                                             ViInt32   CC,
                                                             ViInt32   frame,
                                                             ViInt32   type,
                                                             ViInt32*  result);

ViStatus _VI_FUNC rsspecan_Query5GNRULAnalysisOverload(ViSession instrSession,
                                                       ViInt32   CC,
                                                       ViInt32   frame,
                                                       ViInt32*  overload);

ViStatus _VI_FUNC rsspecan_Query5GNRULAnalysisSEMPassFail(ViSession instrSession,
                                                          ViInt32   CC,
                                                          ViInt32   frame,
                                                          ViInt32*  result);

ViStatus _VI_FUNC rsspecan_Query5GNRULAnalysisSyncState(ViSession instrSession,
                                                        ViInt32   CC,
                                                        ViInt32   frame,
                                                        ViInt32*  result);

ViStatus _VI_FUNC rsspecan_Query5GNRULAnalysisTimeStampDelta(ViSession instrSession,
                                                             ViInt32   CC,
                                                             ViInt32   frame,
                                                             ViReal64* result);

ViStatus _VI_FUNC rsspecan_Query5GNRULAnalysisTimeStamp(ViSession instrSession,
                                                        ViInt32   CC,
                                                        ViInt32   frame,
                                                        ViReal64* result);

ViStatus _VI_FUNC rsspecan_Query5GNRULEVMLimitsFrequencyError(
                            ViSession instrSession,
                            ViInt32   measurementType,
                            ViInt32*  result);

ViStatus _VI_FUNC rsspecan_Query5GNRULEVMLimitsPUSCHResource(ViSession instrSession,
                                                             ViInt32*  withQPSK,
                                                             ViInt32*  with16QAM,
                                                             ViInt32*  with64QAM,
                                                             ViInt32*  with256QAM);

ViStatus _VI_FUNC rsspecan_Query5GNRULEVMLimitsPUSCHResource1024QAM(
                            ViSession instrSession,
                            ViInt32*  with1024QAM);

ViStatus _VI_FUNC rsspecan_Query5GNRULEVMLimitsPUSCHDMRS(ViSession instrSession,
                                                         ViInt32*  pi2BPSK,
                                                         ViInt32*  QPSK,
                                                         ViInt32*  _16QAM,
                                                         ViInt32*  _64QAM,
                                                         ViInt32*  _256QAM);

ViStatus _VI_FUNC rsspecan_Query5GNRULEVMLimitsPUSCHDMRS1024QAM(
                            ViSession instrSession,
                            ViInt32*  _1024QAM);

ViStatus _VI_FUNC rsspecan_Query5GNRULEVMLimitsPUCCHDMRS(ViSession instrSession,
                                                         ViInt32*  DMRS);

ViStatus _VI_FUNC rsspecan_Query5GNRULEVMLimitsPUCCHResourceElements(
                            ViSession instrSession,
                            ViInt32*  resourceElements);

ViStatus _VI_FUNC rsspecan_Query5GNRULEVMLimitsPDSCHPi2BPSK(ViSession instrSession,
                                                            ViInt32*  pi2BPSK);

ViStatus _VI_FUNC rsspecan_Query5GNRULEVMLimits3GPP(ViSession instrSession,
                                                    ViInt32   bufferSize,
                                                    ViChar    _3GPP[]);

ViStatus _VI_FUNC rsspecan_Configure5GNRULAnalysisView(ViSession instrSession,
                                                       ViInt32   subwindow,
                                                       ViInt32   carrier,
                                                       ViInt32   frame);

ViStatus _VI_FUNC rsspecan_Configure5GNRULAnalysisViewCoupling(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   subwindow,
                            ViBoolean coupling);

ViStatus _VI_FUNC rsspecan_Configure5GNRULAnalysisViewSelectResultWindow(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   subwindow);

ViStatus _VI_FUNC rsspecan_Configure5GNRULAnalysisFrequencySweep(
                            ViSession instrSession,
                            ViInt32   BWForACLR,
                            ViInt32   BSCategory,
                            ViInt32   powerClass);

ViStatus _VI_FUNC rsspecan_Configure5GNRULAnalysisFrequencySweepSEMIFF(
                            ViSession instrSession,
                            ViInt32   IFF);

ViStatus _VI_FUNC rsspecan_Configure5GNRULAnalysisFrequencySweepACLRIFF(
                            ViSession instrSession,
                            ViInt32   IFF);

ViStatus _VI_FUNC rsspecan_Configure5GNRULAnalysisTotalLimitPassMode(
                            ViSession instrSession,
                            ViInt32   totalLimitPassMode);

ViStatus _VI_FUNC rsspecan_Query5GNRULACPChannelLimitCheckResults(
                            ViSession instrSession,
                            ViInt32   channelType,
                            ViInt32   limitCheckType,
                            ViInt32   results[]);

ViStatus _VI_FUNC rsspecan_Query5GNRULACLRLimitCheckResults(ViSession instrSession,
                                                            ViInt32   limitCheckType,
                                                            ViInt32   results[]);

ViStatus _VI_FUNC rsspecan_Query5GNRULACLRPowerLimitCheckResults(
                            ViSession instrSession,
                            ViInt32   limitCheckType,
                            ViInt32   results[]);

ViStatus _VI_FUNC rsspecan_Query5GNRULCACLRLimitCheckResults(ViSession instrSession,
                                                             ViInt32   limitCheckType,
                                                             ViInt32   results[]);

ViStatus _VI_FUNC rsspecan_Configure5GNRULCombinedMeasurementEventFilter(
                            ViSession instrSession,
                            ViBoolean eventFilter);

ViStatus _VI_FUNC rsspecan_Reset5GNRULCombinedMeasurementEventFilter(ViSession instrSession);

ViStatus _VI_FUNC rsspecan_Configure5GNRULCombinedMeasurementEventFilterParameters(
                            ViSession instrSession,
                            ViInt32   result,
                            ViInt32   condition,
                            ViReal64  value);

ViStatus _VI_FUNC rsspecan_Configure5GNRULCombinedMeasurementEventFilterParametersEnabled(
                            ViSession instrSession,
                            ViInt32   result,
                            ViBoolean state);

ViStatus _VI_FUNC rsspecan_Query5GNRULACLRMeasurement(ViSession instrSession,
                                                      ViInt32   window,
                                                      ViInt32   marker,
                                                      ViInt32   arraySize,
                                                      ViReal64  result[],
                                                      ViInt32*  returnedValues);

ViStatus _VI_FUNC rsspecan_Fetch5GNRULAnalysisCaptureBuffer(ViSession instrSession,
                                                            ViInt32   window,
                                                            ViInt32   arraySize,
                                                            ViReal64  results[],
                                                            ViInt32*  returnedValues);

ViStatus _VI_FUNC rsspecan_Fetch5GNRULAnalysisEVMvsCarrier(ViSession instrSession,
                                                           ViInt32   window,
                                                           ViInt32   trace,
                                                           ViInt32   arraySize,
                                                           ViReal64  results[],
                                                           ViInt32*  returnedValues);

ViStatus _VI_FUNC rsspecan_Fetch5GNRULAnalysisEVMvsSymbol(ViSession instrSession,
                                                          ViInt32   window,
                                                          ViInt32   arraySize,
                                                          ViReal64  results[],
                                                          ViInt32*  returnedValues);

ViStatus _VI_FUNC rsspecan_Fetch5GNRULAnalysisEVMvsRB(ViSession instrSession,
                                                      ViInt32   window,
                                                      ViInt32   trace,
                                                      ViInt32   arraySize,
                                                      ViReal64  results[],
                                                      ViInt32*  returnedValues);

ViStatus _VI_FUNC rsspecan_Fetch5GNRULAnalysisFrequencyErrorvsSymbol(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   arraySize,
                            ViReal64  results[],
                            ViInt32*  returnedValues);

ViStatus _VI_FUNC rsspecan_Fetch5GNRULAnalysisPowerSpectrum(ViSession instrSession,
                                                            ViInt32   window,
                                                            ViInt32   arraySize,
                                                            ViReal64  results[],
                                                            ViInt32*  returnedValues);

ViStatus _VI_FUNC rsspecan_Fetch5GNRULAnalysisInbandEmission(ViSession instrSession,
                                                             ViInt32   window,
                                                             ViInt32   arraySize,
                                                             ViReal64  results[],
                                                             ViInt32*  returnedValues);

ViStatus _VI_FUNC rsspecan_Fetch5GNRULAnalysisInbandEmissionUpperLimitLine(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   arraySize,
                            ViReal64  results[],
                            ViInt32*  returnedValues);

ViStatus _VI_FUNC rsspecan_Fetch5GNRULAnalysisInbandEmissionRBIndices(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   arraySize,
                            ViInt32   results[],
                            ViInt32*  returnedValues);

ViStatus _VI_FUNC rsspecan_Fetch5GNRULAnalysisFlatnessvsCarrier(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   trace,
                            ViInt32   arraySize,
                            ViReal64  results[],
                            ViInt32*  returnedValues);

ViStatus _VI_FUNC rsspecan_Fetch5GNRULAnalysisCCDF(ViSession instrSession,
                                                   ViInt32   window,
                                                   ViInt32   axis,
                                                   ViInt32   arraySize,
                                                   ViReal64  results[],
                                                   ViInt32*  returnedValues);

ViStatus _VI_FUNC rsspecan_Fetch5GNRULAnalysisConstellation(ViSession instrSession,
                                                            ViInt32   window,
                                                            ViInt32   arraySize,
                                                            ViReal64  constellationI[],
                                                            ViReal64  constellationQ[],
                                                            ViInt32*  returnedValues);

ViStatus _VI_FUNC rsspecan_Fetch5GNRULAnalysisAllocationSummary(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   arraySize,
                            ViInt32   BWPart[],
                            ViInt32   subframe[],
                            ViInt32   slot[],
                            ViInt32   allocationID[],
                            ViInt32   numberOfRB[],
                            ViReal64  relativePower[],
                            ViInt32   modulation[],
                            ViReal64  absolutePower[],
                            ViReal64  EVM[],
                            ViInt32*  returnedValues);

ViStatus _VI_FUNC rsspecan_Fetch5GNRULAnalysisBitstream(ViSession instrSession,
                                                        ViInt32   window,
                                                        ViInt32   format,
                                                        ViInt32   arraySize,
                                                        ViChar    results[],
                                                        ViInt32*  returnedValues);

ViStatus _VI_FUNC rsspecan_Fetch5GNRULAnalysisEVMvsSymbolxCarrier(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   arraySize,
                            ViReal64  results[],
                            ViInt32*  returnedValues);

ViStatus _VI_FUNC rsspecan_Fetch5GNRULAnalysisPowervsSymbolxCarrier(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   arraySize,
                            ViReal64  results[],
                            ViInt32*  returnedValues);

ViStatus _VI_FUNC rsspecan_Fetch5GNRULAnalysisAllocationIDvsSymbolxCarrier(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   arraySize,
                            ViInt32   results[],
                            ViInt32*  returnedValues);

ViStatus _VI_FUNC rsspecan_Fetch5GNRULAnalysisACLR(ViSession instrSession,
                                                   ViInt32   window,
                                                   ViInt32   arraySize,
                                                   ViReal64  results[],
                                                   ViInt32*  returnedValues);

ViStatus _VI_FUNC rsspecan_Fetch5GNRULAnalysisSEM(ViSession instrSession,
                                                  ViInt32   window,
                                                  ViInt32   arraySize,
                                                  ViReal64  results[],
                                                  ViInt32*  returnedValues);

ViStatus _VI_FUNC rsspecan_Fetch5GNRULAnalysisSEMTable(ViSession instrSession,
                                                       ViInt32   window,
                                                       ViInt32   arraySize,
                                                       ViInt32   index[],
                                                       ViReal64  startFrequency[],
                                                       ViReal64  stopFrequency[],
                                                       ViReal64  RBW[],
                                                       ViReal64  limitFailFreq[],
                                                       ViReal64  absolutePower[],
                                                       ViReal64  relativePower[],
                                                       ViReal64  limitDistance[],
                                                       ViInt32   limitCheckResult[],
                                                       ViReal64  reserved1[],
                                                       ViReal64  reserved2[],
                                                       ViInt32*  returnedValues);

ViStatus _VI_FUNC rsspecan_Fetch5GNRULAnalysisCombinedMeasurement(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   trace,
                            ViInt32   arraySize,
                            ViReal64  results[],
                            ViInt32*  returnedValues);

ViStatus _VI_FUNC rsspecan_SetV5GTFMode(ViSession instrSession);

ViStatus _VI_FUNC rsspecan_ConfigureV5GTFLinkDirection(ViSession instrSession,
                                                       ViInt32   linkDirection);

ViStatus _VI_FUNC rsspecan_V5GTFAutoEVM(ViSession instrSession);

ViStatus _VI_FUNC rsspecan_ConfigureV5GTFNumberOfComponentCarriers(
                            ViSession instrSession,
                            ViInt32   numberOfComponentCarriers);

ViStatus _VI_FUNC rsspecan_ConfigureV5GTFCCSignalCapture(ViSession instrSession,
                                                         ViInt32   CCSignalCapture);

ViStatus _VI_FUNC rsspecan_V5GTFSaveDemodSettings(ViSession instrSession,
                                                  ViInt32   componentCarrier,
                                                  ViString  fileName);

ViStatus _VI_FUNC rsspecan_V5GTFSaveDemodAllSettings(ViSession instrSession,
                                                     ViString  fileName);

ViStatus _VI_FUNC rsspecan_V5GTFRestoreSavedDemodSettings(ViSession instrSession,
                                                          ViInt32   componentCarrier,
                                                          ViString  fileName);

ViStatus _VI_FUNC rsspecan_V5GTFRestoreSavedDemodAllSettings(ViSession instrSession,
                                                             ViString  fileName);

ViStatus _VI_FUNC rsspecan_ConfigureV5GTFCarrierFrequency(ViSession instrSession,
                                                          ViInt32   componentCarrier,
                                                          ViReal64  carrierFrequency);

ViStatus _VI_FUNC rsspecan_ConfigureV5GTFFrequencyOffset(ViSession instrSession,
                                                         ViInt32   componentCarrier,
                                                         ViReal64  frequencyOffset);

ViStatus _VI_FUNC rsspecan_ConfigureV5GTFCellID(ViSession instrSession,
                                                ViInt32   linkDirection,
                                                ViInt32   componentCarrier,
                                                ViInt32   cellID);

ViStatus _VI_FUNC rsspecan_ConfigureV5GTFConfigurableSubframes(
                            ViSession instrSession,
                            ViInt32   linkDirection,
                            ViInt32   componentCarrier,
                            ViInt32   configurableSubframes);

ViStatus _VI_FUNC rsspecan_ConfigureV5GTFRadioFrameSubframeAllocation(
                            ViSession instrSession,
                            ViInt32   linkDirection,
                            ViInt32   componentCarrier,
                            ViInt32   subframe,
                            ViInt32   allocation);

ViStatus _VI_FUNC rsspecan_ConfigureV5GTFRadioFrameDLSubframeOptionalRefSignals(
                            ViSession instrSession,
                            ViInt32   componentCarrier,
                            ViInt32   subframe,
                            ViInt32   optionalRefSignals);

ViStatus _VI_FUNC rsspecan_ConfigureV5GTFRadioFrameSubframeSelected(
                            ViSession instrSession,
                            ViInt32   linkDirection,
                            ViInt32   componentCarrier,
                            ViInt32   subframe,
                            ViInt32   selectedSubframe);

ViStatus _VI_FUNC rsspecan_ConfigureV5GTFRadioFrameSubframeType(
                            ViSession instrSession,
                            ViInt32   linkDirection,
                            ViInt32   componentCarrier,
                            ViInt32   subframe,
                            ViInt32   subframeType);

ViStatus _VI_FUNC rsspecan_V5GTFRadioFrameSubframeCopy(ViSession instrSession,
                                                       ViInt32   linkDirection,
                                                       ViInt32   componentCarrier,
                                                       ViInt32   subframe);

ViStatus _VI_FUNC rsspecan_V5GTFRadioFrameSubframePaste(ViSession instrSession,
                                                        ViInt32   linkDirection,
                                                        ViInt32   componentCarrier,
                                                        ViInt32   subframe);

ViStatus _VI_FUNC rsspecan_V5GTFRadioFrameSubframePasteAll(ViSession instrSession,
                                                           ViInt32   linkDirection,
                                                           ViInt32   componentCarrier,
                                                           ViInt32   subframe);

ViStatus _VI_FUNC rsspecan_V5GTFRadioFrameSubframeReset(ViSession instrSession,
                                                        ViInt32   linkDirection,
                                                        ViInt32   componentCarrier,
                                                        ViInt32   subframe);

ViStatus _VI_FUNC rsspecan_ConfigureV5GTFxPDSCHSubframeAllocations(
                            ViSession instrSession,
                            ViInt32   componentCarrier,
                            ViInt32   subframe,
                            ViInt32   allocations);

ViStatus _VI_FUNC rsspecan_ConfigureV5GTFxPDSCHSubframeTable(ViSession instrSession,
                                                             ViInt32   componentCarrier,
                                                             ViInt32   subframe,
                                                             ViInt32   allocation,
                                                             ViInt32   IDOrN_RNTI,
                                                             ViInt32   modulation,
                                                             ViInt32   numberOfRB,
                                                             ViInt32   offsetRB,
                                                             ViReal64  power);

ViStatus _VI_FUNC rsspecan_ConfigureV5GTFxPDSCHPrecoding(ViSession instrSession,
                                                         ViInt32   componentCarrier,
                                                         ViInt32   subframe,
                                                         ViInt32   allocation,
                                                         ViInt32   precoding);

ViStatus _VI_FUNC rsspecan_ConfigureV5GTFxPDSCHGeneralSettings(
                            ViSession instrSession,
                            ViInt32   componentCarrier,
                            ViInt32   subframe,
                            ViInt32   allocation,
                            ViInt32   antennaPort,
                            ViInt32   noOfLayers);

ViStatus _VI_FUNC rsspecan_ConfigureV5GTFxPDSCHScramblingIdentityn_SCID(
                            ViSession instrSession,
                            ViInt32   componentCarrier,
                            ViInt32   subframe,
                            ViInt32   allocation,
                            ViInt32   scramblingIdentityN_SCID);

ViStatus _VI_FUNC rsspecan_ConfigureV5GTFxPDSCHDMRS(ViSession instrSession,
                                                    ViInt32   componentCarrier,
                                                    ViInt32   subframe,
                                                    ViInt32   allocation,
                                                    ViReal64  power,
                                                    ViInt32   sequenceGeneration,
                                                    ViInt32   n_ID_DMRS);

ViStatus _VI_FUNC rsspecan_ConfigureV5GTFxPDSCHPCRS(ViSession instrSession,
                                                    ViInt32   componentCarrier,
                                                    ViInt32   subframe,
                                                    ViInt32   allocation,
                                                    ViBoolean PCRSEnabled,
                                                    ViReal64  power,
                                                    ViInt32   antennaPort,
                                                    ViInt32   sequenceGeneration,
                                                    ViInt32   n_ID_DMRS);

ViStatus _VI_FUNC rsspecan_ConfigureV5GTFxPUSCHSubframeAllocations(
                            ViSession instrSession,
                            ViInt32   componentCarrier,
                            ViInt32   subframe,
                            ViInt32   allocations);

ViStatus _VI_FUNC rsspecan_ConfigureV5GTFxPUSCHSubframeTable(ViSession instrSession,
                                                             ViInt32   componentCarrier,
                                                             ViInt32   subframe,
                                                             ViInt32   allocation,
                                                             ViInt32   modulation,
                                                             ViInt32   numberOfRB,
                                                             ViInt32   offsetRB,
                                                             ViReal64  power);

ViStatus _VI_FUNC rsspecan_ConfigureV5GTFxPUSCHREMappingIndex(
                            ViSession instrSession,
                            ViInt32   componentCarrier,
                            ViInt32   subframe,
                            ViInt32   allocation,
                            ViInt32   REMappingIndex);

ViStatus _VI_FUNC rsspecan_ConfigureV5GTFxPUSCHScramblingIdentityn_SCID(
                            ViSession instrSession,
                            ViInt32   componentCarrier,
                            ViInt32   subframe,
                            ViInt32   allocation,
                            ViInt32   scramblingIdentityN_SCID);

ViStatus _VI_FUNC rsspecan_ConfigureV5GTFxPUSCHDMRS(ViSession instrSession,
                                                    ViInt32   componentCarrier,
                                                    ViInt32   subframe,
                                                    ViInt32   allocation,
                                                    ViReal64  power,
                                                    ViInt32   sequenceGeneration,
                                                    ViInt32   n_ID_DMRS);

ViStatus _VI_FUNC rsspecan_ConfigureV5GTFxPUSCHPCRS(ViSession instrSession,
                                                    ViInt32   componentCarrier,
                                                    ViInt32   subframe,
                                                    ViInt32   allocation,
                                                    ViBoolean PCRSEnabled,
                                                    ViReal64  power,
                                                    ViInt32   transmission,
                                                    ViInt32   sequenceGeneration,
                                                    ViInt32   n_ID_DMRS);

ViStatus _VI_FUNC rsspecan_ConfigureV5GTFSynchronizationSignal(
                            ViSession instrSession,
                            ViInt32   componentCarrier,
                            ViReal64  pSyncRelPower,
                            ViReal64  sSyncRelPower,
                            ViReal64  eSyncRelPower);

ViStatus _VI_FUNC rsspecan_ConfigureV5GTFReferenceSignalePBCH(
                            ViSession instrSession,
                            ViInt32   componentCarrier,
                            ViInt32   precoding,
                            ViReal64  relPower);

ViStatus _VI_FUNC rsspecan_ConfigureV5GTFReferenceSignalxPDSCHPrecoding(
                            ViSession instrSession,
                            ViInt32   componentCarrier,
                            ViInt32   precoding);

ViStatus _VI_FUNC rsspecan_ConfigureV5GTFReferenceSignalCSI(ViSession instrSession,
                                                            ViInt32   componentCarrier,
                                                            ViInt32   subframe,
                                                            ViInt32   n_ID_CSI,
                                                            ViReal64  power);

ViStatus _VI_FUNC rsspecan_ConfigureV5GTFReferenceSignalSubframeSymbolTransmissionEnabled(
                            ViSession instrSession,
                            ViInt32   componentCarrier,
                            ViInt32   subframe,
                            ViInt32   OFDMSymbol,
                            ViInt32   symbolTransmissionEnabled);

ViStatus _VI_FUNC rsspecan_ConfigureV5GTFControlChannelxPDCCHSequenceGeneration(
                            ViSession instrSession,
                            ViInt32   componentCarrier,
                            ViInt32   subframe,
                            ViInt32   sequenceGeneration,
                            ViInt32   sequenceID);

ViStatus _VI_FUNC rsspecan_ConfigureV5GTFControlChannelxPDCCHPrecoding(
                            ViSession instrSession,
                            ViInt32   componentCarrier,
                            ViInt32   subframe,
                            ViInt32   precoding);

ViStatus _VI_FUNC rsspecan_ConfigureV5GTFControlChannelxPDCCHPower(
                            ViSession instrSession,
                            ViInt32   componentCarrier,
                            ViInt32   subframe,
                            ViReal64  power);

ViStatus _VI_FUNC rsspecan_ConfigureV5GTFControlChannelxPDCCHNumOfxREGs(
                            ViSession instrSession,
                            ViInt32   componentCarrier,
                            ViInt32   numOfXREGs);

ViStatus _VI_FUNC rsspecan_ConfigureV5GTFControlChannelxPDCCHxREGEnabled(
                            ViSession instrSession,
                            ViInt32   componentCarrier,
                            ViInt32   subframe,
                            ViInt32   symbol,
                            ViInt32   xReg,
                            ViInt32   xREGEnabled);

ViStatus _VI_FUNC rsspecan_ConfigureV5GTFControlChannelULxPUCCHState(
                            ViSession instrSession,
                            ViInt32   componentCarrier,
                            ViBoolean xPUCCHEnabled);

ViStatus _VI_FUNC rsspecan_ConfigureV5GTFControlChannelULxPUCCHPrecoding(
                            ViSession instrSession,
                            ViInt32   componentCarrier,
                            ViInt32   precoding);

ViStatus _VI_FUNC rsspecan_ConfigureV5GTFControlChannelULxPUCCHUsed(
                            ViSession instrSession,
                            ViInt32   componentCarrier,
                            ViInt32   location,
                            ViInt32   n_RNTI);

ViStatus _VI_FUNC rsspecan_ConfigureV5GTFControlChannelULxPUCCHRelativePower(
                            ViSession instrSession,
                            ViInt32   componentCarrier,
                            ViReal64  xPUCCHPower,
                            ViReal64  powerDMRS);

ViStatus _VI_FUNC rsspecan_ConfigureV5GTFControlChannelULxPUCCHSequenceGeneration(
                            ViSession instrSession,
                            ViInt32   componentCarrier,
                            ViInt32   sequenceGeneration,
                            ViInt32   ID);

ViStatus _VI_FUNC rsspecan_ConfigureV5GTFAntennaPortConfigurationEnabled(
                            ViSession instrSession,
                            ViInt32   componentCarrier,
                            ViInt32   physicalAntenna,
                            ViBoolean portConfigurationEnabled);

ViStatus _VI_FUNC rsspecan_ConfigureV5GTFAntennaPortMapping(ViSession instrSession,
                                                            ViInt32   componentCarrier,
                                                            ViInt32   physicalAntenna,
                                                            ViInt32   channelOrSignal,
                                                            ViInt32   antennaPort,
                                                            ViInt32   state);

ViStatus _VI_FUNC rsspecan_ConfigureV5GTFMaxNumberOfSubframes(
                            ViSession instrSession,
                            ViInt32   maxNumberOfSubframes);

ViStatus _VI_FUNC rsspecan_ConfigureV5GTFDemodulationMulticarrierFilterEnabled(
                            ViSession instrSession,
                            ViBoolean multicarrierFilterEnabled);

ViStatus _VI_FUNC rsspecan_ConfigureV5GTFLevelTrackingEnabled(
                            ViSession instrSession,
                            ViBoolean levelTrackingEnabled);

ViStatus _VI_FUNC rsspecan_AddV5GTFWindow(ViSession instrSession,
                                          ViInt32   stringSize,
                                          ViString  referenceWindowName,
                                          ViInt32   direction,
                                          ViInt32   windowType,
                                          ViChar    returnedWindowName[]);

ViStatus _VI_FUNC rsspecan_ReplaceV5GTFWindow(ViSession instrSession,
                                              ViString  windowName,
                                              ViInt32   windowType);

ViStatus _VI_FUNC rsspecan_ConfigureV5GTFSubwindowYAxisScaling(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   subwindow,
                            ViReal64  max,
                            ViReal64  min);

ViStatus _VI_FUNC rsspecan_ConfigureV5GTFSubwindowYAxisAutoScaling(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   subwindow,
                            ViInt32   autoScalingEnabled);

ViStatus _VI_FUNC rsspecan_ConfigureV5GTFSubwindowCouplingEnabled(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   subwindow,
                            ViBoolean couplingEnabled);

ViStatus _VI_FUNC rsspecan_SelectV5GTFSubwindow(ViSession instrSession,
                                                ViInt32   window,
                                                ViInt32   subwindow);

ViStatus _VI_FUNC rsspecan_QueryV5GTFSynchronizationState(ViSession instrSession,
                                                          ViInt32   bufferSize,
                                                          ViChar    synchronizationState[]);

ViStatus _VI_FUNC rsspecan_ConfigureV5GTFMarkerCouplingState(ViSession instrSession,
                                                             ViBoolean markerCouplingEnabled);

ViStatus _VI_FUNC rsspecan_ConfigureV5GTFXAxisUnit(ViSession instrSession,
                                                   ViInt32   xAxisUnit);

ViStatus _VI_FUNC rsspecan_ConfigureV5GTFEVMUnit(ViSession instrSession,
                                                 ViInt32   EVMUnit);

ViStatus _VI_FUNC rsspecan_QueryV5GTFMarkerZ(ViSession instrSession,
                                             ViInt32   window,
                                             ViReal64* EVM,
                                             ViReal64* power);

ViStatus _VI_FUNC rsspecan_QueryV5GTFResultCrestFactor(ViSession instrSession,
                                                       ViInt32   componentCarrier,
                                                       ViReal64* crestFactor);

ViStatus _VI_FUNC rsspecan_QueryV5GTFResultEVM(ViSession instrSession,
                                               ViInt32   componentCarrier,
                                               ViReal64* EVM);

ViStatus _VI_FUNC rsspecan_QueryV5GTFResultEVMxPDSCHQPSK(ViSession instrSession,
                                                         ViInt32   componentCarrier,
                                                         ViReal64* EVMXPDSCHQPSK);

ViStatus _VI_FUNC rsspecan_QueryV5GTFResultEVMxPDSCH16QAM(ViSession instrSession,
                                                          ViInt32   componentCarrier,
                                                          ViReal64* EVMXPDSCH16QAM);

ViStatus _VI_FUNC rsspecan_QueryV5GTFResultEVMxPDSCH64QAM(ViSession instrSession,
                                                          ViInt32   componentCarrier,
                                                          ViReal64* EVMXPDSCH64QAM);

ViStatus _VI_FUNC rsspecan_QueryV5GTFResultEVMxPDSCH256QAM(ViSession instrSession,
                                                           ViInt32   componentCarrier,
                                                           ViReal64* EVMXPDSCH256QAM);

ViStatus _VI_FUNC rsspecan_QueryV5GTFResultEVMPhysicalChannel(
                            ViSession instrSession,
                            ViInt32   componentCarrier,
                            ViReal64* EVMPhysicalChannel);

ViStatus _VI_FUNC rsspecan_QueryV5GTFResultEVMPhysicalSignal(ViSession instrSession,
                                                             ViInt32   componentCarrier,
                                                             ViReal64* EVMPhysicalSignal);

ViStatus _VI_FUNC rsspecan_QueryV5GTFResultEVMxPUSCHQPSK(ViSession instrSession,
                                                         ViInt32   componentCarrier,
                                                         ViReal64* EVMXPUSCHQPSK);

ViStatus _VI_FUNC rsspecan_QueryV5GTFResultEVMxPUSCH16QAM(ViSession instrSession,
                                                          ViInt32   componentCarrier,
                                                          ViReal64* EVMXPUSCH16QAM);

ViStatus _VI_FUNC rsspecan_QueryV5GTFResultEVMxPUSCH64QAM(ViSession instrSession,
                                                          ViInt32   componentCarrier,
                                                          ViReal64* EVMXPUSCH64QAM);

ViStatus _VI_FUNC rsspecan_QueryV5GTFResultEVMxPUSCH256QAM(ViSession instrSession,
                                                           ViInt32   componentCarrier,
                                                           ViReal64* EVMXPUSCH256QAM);

ViStatus _VI_FUNC rsspecan_QueryV5GTFResultFrequencyError(ViSession instrSession,
                                                          ViInt32   componentCarrier,
                                                          ViReal64* frequencyError);

ViStatus _VI_FUNC rsspecan_QueryV5GTFResultIQGainImbalance(ViSession instrSession,
                                                           ViInt32   componentCarrier,
                                                           ViReal64* IQGainImbalance);

ViStatus _VI_FUNC rsspecan_QueryV5GTFResultIQOffset(ViSession instrSession,
                                                    ViInt32   componentCarrier,
                                                    ViReal64* IQOffset);

ViStatus _VI_FUNC rsspecan_QueryV5GTFResultQuadratureError(ViSession instrSession,
                                                           ViInt32   componentCarrier,
                                                           ViReal64* quadratureError);

ViStatus _VI_FUNC rsspecan_QueryV5GTFResultTotalPower(ViSession instrSession,
                                                      ViInt32   componentCarrier,
                                                      ViReal64* totalPower);

ViStatus _VI_FUNC rsspecan_QueryV5GTFResultSamplingError(ViSession instrSession,
                                                         ViInt32   componentCarrier,
                                                         ViReal64* samplingError);

ViStatus _VI_FUNC rsspecan_QueryV5GTFResultSubframeStartOffset(
                            ViSession instrSession,
                            ViInt32   componentCarrier,
                            ViReal64* subframeStartOffset);

ViStatus _VI_FUNC rsspecan_V5GTFFetchCaptureBuffer(ViSession instrSession,
                                                   ViInt32   window,
                                                   ViInt32   arraySize,
                                                   ViReal64  result[],
                                                   ViInt32*  returnedValues);

ViStatus _VI_FUNC rsspecan_V5GTFFetchEVMvsCarrier(ViSession instrSession,
                                                  ViInt32   window,
                                                  ViInt32   arraySize,
                                                  ViReal64  result[],
                                                  ViInt32*  returnedValues);

ViStatus _VI_FUNC rsspecan_V5GTFFetchEVMvsSymbol(ViSession instrSession,
                                                 ViInt32   window,
                                                 ViInt32   arraySize,
                                                 ViReal64  result[],
                                                 ViInt32*  returnedValues);

ViStatus _VI_FUNC rsspecan_V5GTFFetchPowerSpectrum(ViSession instrSession,
                                                   ViInt32   window,
                                                   ViInt32   arraySize,
                                                   ViReal64  result[],
                                                   ViInt32*  returnedValues);

ViStatus _VI_FUNC rsspecan_V5GTFFetchChannelAndSpectrumFlatness(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   arraySize,
                            ViReal64  result[],
                            ViInt32*  returnedValues);

ViStatus _VI_FUNC rsspecan_V5GTFFetchGroupDelay(ViSession instrSession,
                                                ViInt32   window,
                                                ViInt32   arraySize,
                                                ViReal64  result[],
                                                ViInt32*  returnedValues);

ViStatus _VI_FUNC rsspecan_V5GTFFetchConstellation(ViSession instrSession,
                                                   ViInt32   window,
                                                   ViInt32   arraySize,
                                                   ViReal64  resultI[],
                                                   ViReal64  resultQ[],
                                                   ViInt32*  returnedValues);

ViStatus _VI_FUNC rsspecan_V5GTFFetchAllocationSummary(ViSession instrSession,
                                                       ViInt32   window,
                                                       ViInt32   arraySize,
                                                       ViInt32   subframe[],
                                                       ViInt32   numberOfRB[],
                                                       ViInt32   allocationID[],
                                                       ViInt32   modulation[],
                                                       ViReal64  relativePower[],
                                                       ViReal64  absolutePower[],
                                                       ViReal64  EVM[],
                                                       ViInt32*  returnedValues);

ViStatus _VI_FUNC rsspecan_V5GTFFetchEVMvsSymbolxCarrier(ViSession instrSession,
                                                         ViInt32   window,
                                                         ViInt32   arraySize,
                                                         ViReal64  result[],
                                                         ViInt32*  returnedValues);

ViStatus _VI_FUNC rsspecan_V5GTFFetchPowervsSymbolxCarrier(ViSession instrSession,
                                                           ViInt32   window,
                                                           ViInt32   arraySize,
                                                           ViReal64  result[],
                                                           ViInt32*  returnedValues);

ViStatus _VI_FUNC rsspecan_V5GTFFetchAllocationIDvsSymbolxCarrier(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   arraySize,
                            ViInt32   result[],
                            ViInt32*  returnedValues);

ViStatus _VI_FUNC rsspecan_SetOneWebULMode(ViSession instrSession);

ViStatus _VI_FUNC rsspecan_ConfigureOneWebULMeasurement(ViSession instrSession,
                                                        ViInt32   selectedMeasurement);

ViStatus _VI_FUNC rsspecan_ConfigureOneWebULDataCapture(ViSession instrSession,
                                                        ViBoolean overallFrameCountEnabled,
                                                        ViInt32   numberOfFramesToAnalyze,
                                                        ViBoolean autoAccordingToStandard,
                                                        ViBoolean singleSubframeModeEnabled);

ViStatus _VI_FUNC rsspecan_ConfigureOneWebULTracking(ViSession instrSession,
                                                     ViInt32   phaseTracking,
                                                     ViBoolean timeTrackingEnabled);

ViStatus _VI_FUNC rsspecan_ConfigureOneWebULSignalDemodulation(
                            ViSession instrSession,
                            ViInt32   analysisMode,
                            ViInt32   channelEstimationRange,
                            ViBoolean exclusionPeriodEnabled,
                            ViBoolean compensateDCOffsetEnabled,
                            ViBoolean descrambleCodedBits,
                            ViBoolean multicarrierFilterEnabled);

ViStatus _VI_FUNC rsspecan_ConfigureOneWebULNumberOfComponentCarriers(
                            ViSession instrSession,
                            ViInt32   numberOfComponentCarriers);

ViStatus _VI_FUNC rsspecan_ConfigureOneWebULLO(ViSession instrSession,
                                               ViInt32   location,
                                               ViReal64  manualFrequency);

ViStatus _VI_FUNC rsspecan_ConfigureOneWebULGlobalSignalCharacteristics(
                            ViSession instrSession,
                            ViInt32   componentCarrier,
                            ViInt32   frameNumberOffset,
                            ViInt32   UEID);

ViStatus _VI_FUNC rsspecan_ConfigureOneWebULSCellIndex(ViSession instrSession,
                                                       ViInt32   componentCarrier,
                                                       ViInt32   SCellIndex);

ViStatus _VI_FUNC rsspecan_ConfigureOneWebULCarrierFrequency(ViSession instrSession,
                                                             ViInt32   componentCarrier,
                                                             ViReal64  carrierFrequency);

ViStatus _VI_FUNC rsspecan_ConfigureOneWebULFrequencyOffset(ViSession instrSession,
                                                            ViInt32   componentCarrier,
                                                            ViReal64  frequencyOffset);

ViStatus _VI_FUNC rsspecan_ConfigureOneWebULBandwidth(ViSession instrSession,
                                                      ViInt32   componentCarrier,
                                                      ViInt32   bandwidth);

ViStatus _VI_FUNC rsspecan_ConfigureOneWebULCellIdentityGroup(
                            ViSession instrSession,
                            ViInt32   componentCarrier,
                            ViInt32   cellIdentityGroup);

ViStatus _VI_FUNC rsspecan_ConfigureOneWebULCellID(ViSession instrSession,
                                                   ViInt32   componentCarrier,
                                                   ViInt32   cellID);

ViStatus _VI_FUNC rsspecan_ConfigureOneWebULIdentity(ViSession instrSession,
                                                     ViInt32   componentCarrier,
                                                     ViInt32   identity);

ViStatus _VI_FUNC rsspecan_QueryOneWebULDetectedCyclicPrefix(ViSession instrSession,
                                                             ViInt32   componentCarrier,
                                                             ViInt32*  detectedCyclicPrefix);

ViStatus _VI_FUNC rsspecan_QueryOneWebULDetectedCellIdentityGroup(
                            ViSession instrSession,
                            ViInt32   componentCarrier,
                            ViInt32*  detectedCellIdentityGroup);

ViStatus _VI_FUNC rsspecan_QueryOneWebULDetectedIdentity(ViSession instrSession,
                                                         ViInt32   componentCarrier,
                                                         ViInt32*  detectedIdentity);

ViStatus _VI_FUNC rsspecan_ConfigureOneWebULRestoreSavedDemodSettings(
                            ViSession instrSession,
                            ViInt32   componentCarrier,
                            ViString  fileName);

ViStatus _VI_FUNC rsspecan_ConfigureOneWebULAutoDemodulationEnabled(
                            ViSession instrSession,
                            ViInt32   autoDemodulationEnabled);

ViStatus _VI_FUNC rsspecan_ConfigureOneWebULSubframeConfigurationDetectionEnabled(
                            ViSession instrSession,
                            ViBoolean detectionEnabled);

ViStatus _VI_FUNC rsspecan_ConfigureOneWebULConfigurableSubframes(
                            ViSession instrSession,
                            ViInt32   componentCarrier,
                            ViInt32   configurableSubframes);

ViStatus _VI_FUNC rsspecan_ConfigureOneWebULSubframe(ViSession instrSession,
                                                     ViInt32   componentCarrier,
                                                     ViInt32   subframe,
                                                     ViInt32   allocationContent,
                                                     ViInt32   modulation);

ViStatus _VI_FUNC rsspecan_ConfigureOneWebULSubframeResourceAllocationType1Enabled(
                            ViSession instrSession,
                            ViInt32   componentCarrier,
                            ViInt32   subframe,
                            ViBoolean allocationType1Enabled);

ViStatus _VI_FUNC rsspecan_ConfigureOneWebULSubframeEnhancedPUCCHFormat(
                            ViSession instrSession,
                            ViInt32   componentCarrier,
                            ViInt32   subframe,
                            ViInt32   format);

ViStatus _VI_FUNC rsspecan_ConfigureOneWebULSubframeEnhancedPUCCHnPUCCH(
                            ViSession instrSession,
                            ViInt32   componentCarrier,
                            ViInt32   subframe,
                            ViInt32   n_PUCCH);

ViStatus _VI_FUNC rsspecan_ConfigureOneWebULSubframeEnhancedDMRS(
                            ViSession instrSession,
                            ViInt32   componentCarrier,
                            ViInt32   subframe,
                            ViInt32   n_2_DMRS,
                            ViInt32   delta_PUACH);

ViStatus _VI_FUNC rsspecan_ConfigureOneWebULSubframeCluster(ViSession instrSession,
                                                            ViInt32   componentCarrier,
                                                            ViInt32   subframe,
                                                            ViInt32   cluster,
                                                            ViInt32   numberOfRB,
                                                            ViInt32   offsetRB);

ViStatus _VI_FUNC rsspecan_ConfigureOneWebULDRSRelativePowers(
                            ViSession instrSession,
                            ViInt32   componentCarrier,
                            ViReal64  relativePowerPUSCH,
                            ViReal64  relativePowerPUCCH);

ViStatus _VI_FUNC rsspecan_ConfigureOneWebULDRS(ViSession instrSession,
                                                ViInt32   componentCarrier,
                                                ViBoolean groupHoppingEnabled,
                                                ViInt32   n_1_DMRS,
                                                ViInt32   deltaSequenceShift);

ViStatus _VI_FUNC rsspecan_ConfigureOneWebULDRSSequenceHopping(
                            ViSession instrSession,
                            ViInt32   componentCarrier,
                            ViBoolean sequenceHoppingEnabled);

ViStatus _VI_FUNC rsspecan_ConfigureOneWebULSRSEnabled(ViSession instrSession,
                                                       ViInt32   componentCarrier,
                                                       ViBoolean SRSEnabled);

ViStatus _VI_FUNC rsspecan_ConfigureOneWebULSRSSubframeConfiguration(
                            ViSession instrSession,
                            ViInt32   componentCarrier,
                            ViInt32   SRSSubframeConfiguration);

ViStatus _VI_FUNC rsspecan_ConfigureOneWebULSRSBandwidthB_SRS(
                            ViSession instrSession,
                            ViInt32   componentCarrier,
                            ViInt32   SRSBandwidthB_SRS);

ViStatus _VI_FUNC rsspecan_ConfigureOneWebULSRSHoppingBW(ViSession instrSession,
                                                         ViInt32   componentCarrier,
                                                         ViInt32   SRSHoppingBW);

ViStatus _VI_FUNC rsspecan_ConfigureOneWebULSRSCyclicShiftN_CS(
                            ViSession instrSession,
                            ViInt32   componentCarrier,
                            ViInt32   SRSCyclicShiftN_CS);

ViStatus _VI_FUNC rsspecan_ConfigureOneWebULSRSRelPower(ViSession instrSession,
                                                        ViInt32   componentCarrier,
                                                        ViReal64  SRSRelPower);

ViStatus _VI_FUNC rsspecan_ConfigureOneWebULSRSBWConfC_SRS(ViSession instrSession,
                                                           ViInt32   componentCarrier,
                                                           ViInt32   SRSBWConfC_SRS);

ViStatus _VI_FUNC rsspecan_ConfigureOneWebULSRSConfIndexI_SRS(
                            ViSession instrSession,
                            ViInt32   componentCarrier,
                            ViInt32   confIndexI_SRS);

ViStatus _VI_FUNC rsspecan_ConfigureOneWebULSRSTransmCombk_TC(
                            ViSession instrSession,
                            ViInt32   componentCarrier,
                            ViInt32   transmCombK_TC);

ViStatus _VI_FUNC rsspecan_ConfigureOneWebULSRSFreqDomainPosn_RRC(
                            ViSession instrSession,
                            ViInt32   componentCarrier,
                            ViInt32   freqDomainPosN_RRC);

ViStatus _VI_FUNC rsspecan_ConfigureOneWebULSRSSimultaneousTXEnabled(
                            ViSession instrSession,
                            ViInt32   componentCarrier,
                            ViBoolean simultaneousTXEnabled);

ViStatus _VI_FUNC rsspecan_ConfigureOneWebULPUSCHNumberOfSubbands(
                            ViSession instrSession,
                            ViInt32   componentCarrier,
                            ViInt32   numberOfSubbands);

ViStatus _VI_FUNC rsspecan_ConfigureOneWebULPUCCHNoOfRBs(ViSession instrSession,
                                                         ViInt32   componentCarrier,
                                                         ViInt32   noOfRBs);

ViStatus _VI_FUNC rsspecan_OneWebULPUCCHNoOfRBsAutoDetect(ViSession instrSession,
                                                          ViInt32   componentCarrier);

ViStatus _VI_FUNC rsspecan_ConfigureOneWebULPUCCHN1_cs(ViSession instrSession,
                                                       ViInt32   componentCarrier,
                                                       ViInt32   n_1_cs);

ViStatus _VI_FUNC rsspecan_ConfigureOneWebULPUCCHDeltaShift(ViSession instrSession,
                                                            ViInt32   componentCarrier,
                                                            ViInt32   deltaShift);

ViStatus _VI_FUNC rsspecan_ConfigureOneWebULPUCCHFormat(ViSession instrSession,
                                                        ViInt32   componentCarrier,
                                                        ViInt32   format);

ViStatus _VI_FUNC rsspecan_ConfigureOneWebULPUCCHN2_RB(ViSession instrSession,
                                                       ViInt32   componentCarrier,
                                                       ViInt32   n_2_RB);

ViStatus _VI_FUNC rsspecan_ConfigureOneWebULPUCCHN_PUCCH(ViSession instrSession,
                                                         ViInt32   componentCarrier,
                                                         ViInt32   n_PUCCH);

ViStatus _VI_FUNC rsspecan_OneWebULPUCCHN_PUCCHAutoDetect(ViSession instrSession,
                                                          ViInt32   componentCarrier);

ViStatus _VI_FUNC rsspecan_SetOneWebULPUCCHN_PUCCHSubframe(ViSession instrSession,
                                                           ViInt32   componentCarrier);

ViStatus _VI_FUNC rsspecan_ConfigureOneWebULPRACHConfiguration(
                            ViSession instrSession,
                            ViInt32   componentCarrier,
                            ViInt32   PRACHConfiguration);

ViStatus _VI_FUNC rsspecan_ConfigureOneWebULPRACHRestrictedSet(
                            ViSession instrSession,
                            ViInt32   componentCarrier,
                            ViBoolean restrictedSet);

ViStatus _VI_FUNC rsspecan_ConfigureOneWebULPRACHFrequencyOffset(
                            ViSession instrSession,
                            ViInt32   componentCarrier,
                            ViReal64  frequencyOffset);

ViStatus _VI_FUNC rsspecan_ConfigureOneWebULPRACHNcsConf(ViSession instrSession,
                                                         ViInt32   componentCarrier,
                                                         ViInt32   ncsConfiguration);

ViStatus _VI_FUNC rsspecan_ConfigureOneWebULPRACHLogicalRootSeqIdx(
                            ViSession instrSession,
                            ViInt32   componentCarrier,
                            ViInt32   logicalRootSeqIdx);

ViStatus _VI_FUNC rsspecan_ConfigureOneWebULPRACHSequenceIndex(
                            ViSession instrSession,
                            ViInt32   componentCarrier,
                            ViInt32   sequenceIndex);

ViStatus _VI_FUNC rsspecan_OneWebULPRACHSequenceIndexAuto(ViSession instrSession,
                                                          ViInt32   componentCarrier);

ViStatus _VI_FUNC rsspecan_AddOneWebULWindow(ViSession instrSession,
                                             ViInt32   stringSize,
                                             ViString  referenceWindowName,
                                             ViInt32   direction,
                                             ViInt32   windowType,
                                             ViChar    returnedWindowName[]);

ViStatus _VI_FUNC rsspecan_ReplaceOneWebULWindow(ViSession instrSession,
                                                 ViString  windowName,
                                                 ViInt32   windowType);

ViStatus _VI_FUNC rsspecan_QueryOneWebULSynchronizationStatus(
                            ViSession  instrSession,
                            ViBoolean* synchronizationStatus);

ViStatus _VI_FUNC rsspecan_ConfigureOneWebULXAxisUnit(ViSession instrSession,
                                                      ViInt32   xAxisUnit);

ViStatus _VI_FUNC rsspecan_ConfigureOneWebULEVMUnit(ViSession instrSession,
                                                    ViInt32   EVMUnit);

ViStatus _VI_FUNC rsspecan_OneWebULMarkerZ(ViSession instrSession,
                                           ViInt32   window,
                                           ViReal64* EVM,
                                           ViReal64* power);

ViStatus _VI_FUNC rsspecan_QueryOneWebULLimitCheckResult(ViSession instrSession,
                                                         ViInt32   componentCarrier,
                                                         ViInt32   limitType,
                                                         ViInt32   statisticalType,
                                                         ViInt32*  limitCheckResult);

ViStatus _VI_FUNC rsspecan_ConfigureOneWebULMarkerCouplingEnabled(
                            ViSession instrSession,
                            ViBoolean markerCouplingEnabled);

ViStatus _VI_FUNC rsspecan_ConfigureOneWebULEvaluationRangeGlobal(
                            ViSession instrSession,
                            ViInt32   componentCarrier,
                            ViInt32   selectedSubframe,
                            ViInt32   selectedSlot);

ViStatus _VI_FUNC rsspecan_ConfigureOneWebULEvaluationRangeConstellationDiag(
                            ViSession instrSession,
                            ViInt32   componentCarrier,
                            ViInt32   modulation,
                            ViInt32   allocation,
                            ViInt32   symbol,
                            ViInt32   carrier);

ViStatus _VI_FUNC rsspecan_ConfigureOneWebULEvaluationRangePreamble(
                            ViSession instrSession,
                            ViInt32   componentCarrier,
                            ViInt32   preamble);

ViStatus _VI_FUNC rsspecan_OneWebULFetchACLR(ViSession instrSession,
                                             ViInt32   window,
                                             ViInt32   arraySize,
                                             ViReal64  result[],
                                             ViInt32*  returnedValues);

ViStatus _VI_FUNC rsspecan_OneWebULFetchAllocationSummary(ViSession instrSession,
                                                          ViInt32   window,
                                                          ViInt32   arraySize,
                                                          ViInt32   subframe[],
                                                          ViInt32   allocationID[],
                                                          ViInt32   numberOfRB[],
                                                          ViInt32   offsetRB[],
                                                          ViInt32   modulation[],
                                                          ViReal64  absolutePower[],
                                                          ViReal64  EVM[],
                                                          ViInt32*  returnedValues);

ViStatus _VI_FUNC rsspecan_OneWebULFetchBitstream(ViSession instrSession,
                                                  ViInt32   window,
                                                  ViInt32   arraySize,
                                                  ViInt32   result[],
                                                  ViInt32*  returnedValues);

ViStatus _VI_FUNC rsspecan_OneWebULFetchCaptureBuffer(ViSession instrSession,
                                                      ViInt32   window,
                                                      ViInt32   arraySize,
                                                      ViReal64  result[],
                                                      ViInt32*  returnedValues);

ViStatus _VI_FUNC rsspecan_OneWebULFetchCCDF(ViSession instrSession,
                                             ViInt32   window,
                                             ViInt32   axis,
                                             ViInt32   arraySize,
                                             ViReal64  result[],
                                             ViInt32*  returnedValues);

ViStatus _VI_FUNC rsspecan_OneWebULFetchChannelAndSpectrumFlatness(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   statisticalType,
                            ViInt32   arraySize,
                            ViReal64  result[],
                            ViInt32*  returnedValues);

ViStatus _VI_FUNC rsspecan_OneWebULFetchChannelAndSpectrumFlatnessDifference(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   statisticalType,
                            ViInt32   arraySize,
                            ViReal64  result[],
                            ViInt32*  returnedValues);

ViStatus _VI_FUNC rsspecan_OneWebULFetchChannelFlatnessSRS(ViSession instrSession,
                                                           ViInt32   window,
                                                           ViInt32   statisticalType,
                                                           ViInt32   arraySize,
                                                           ViReal64  result[],
                                                           ViInt32*  returnedValues);

ViStatus _VI_FUNC rsspecan_OneWebULFetchGroupDelay(ViSession instrSession,
                                                   ViInt32   window,
                                                   ViInt32   statisticalType,
                                                   ViInt32   arraySize,
                                                   ViReal64  result[],
                                                   ViInt32*  returnedValues);

ViStatus _VI_FUNC rsspecan_OneWebULFetchConstellation(ViSession instrSession,
                                                      ViInt32   window,
                                                      ViInt32   constellationType,
                                                      ViInt32   arraySize,
                                                      ViReal64  i[],
                                                      ViReal64  q[],
                                                      ViInt32*  returnedValues);

ViStatus _VI_FUNC rsspecan_OneWebULFetchEVMvsCarrier(ViSession instrSession,
                                                     ViInt32   window,
                                                     ViInt32   statisticalType,
                                                     ViInt32   arraySize,
                                                     ViReal64  result[],
                                                     ViInt32*  returnedValues);

ViStatus _VI_FUNC rsspecan_OneWebULFetchEVMvsSubframe(ViSession instrSession,
                                                      ViInt32   window,
                                                      ViInt32   arraySize,
                                                      ViReal64  result[],
                                                      ViInt32*  returnedValues);

ViStatus _VI_FUNC rsspecan_OneWebULFetchEVMvsSymbol(ViSession instrSession,
                                                    ViInt32   window,
                                                    ViInt32   arraySize,
                                                    ViReal64  result[],
                                                    ViInt32*  returnedValues);

ViStatus _VI_FUNC rsspecan_OneWebULFetchEVMvsSymbolxCarrier(ViSession instrSession,
                                                            ViInt32   window,
                                                            ViInt32   arraySize,
                                                            ViReal64  result[],
                                                            ViInt32*  returnedValues);

ViStatus _VI_FUNC rsspecan_OneWebULFetchInbandEmission(ViSession instrSession,
                                                       ViInt32   window,
                                                       ViInt32   resultType,
                                                       ViInt32   arraySize,
                                                       ViReal64  result[],
                                                       ViInt32*  returnedValues);

ViStatus _VI_FUNC rsspecan_OneWebULFetchPowerSpectrum(ViSession instrSession,
                                                      ViInt32   window,
                                                      ViInt32   arraySize,
                                                      ViReal64  result[],
                                                      ViInt32*  returnedValues);

ViStatus _VI_FUNC rsspecan_OneWebULFetchPowervsSymbolxCarrier(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   arraySize,
                            ViReal64  result[],
                            ViInt32*  returnedValues);

ViStatus _VI_FUNC rsspecan_QueryOneWebULACLRMeasurement(ViSession instrSession,
                                                        ViInt32   window,
                                                        ViInt32   marker,
                                                        ViInt32   arraySize,
                                                        ViReal64  result[],
                                                        ViInt32*  returnedValues);

ViStatus _VI_FUNC rsspecan_QueryOneWebULResultFrameEVM(ViSession instrSession,
                                                       ViInt32   componentCarrier,
                                                       ViInt32   resourceElement,
                                                       ViInt32   modulation,
                                                       ViReal64* result);

ViStatus _VI_FUNC rsspecan_QueryOneWebULResultSelection(ViSession instrSession,
                                                        ViInt32   componentCarrier,
                                                        ViInt32   measurement,
                                                        ViInt32   statisticalType,
                                                        ViReal64* result);

ViStatus _VI_FUNC rsspecan_SetHRPUWBMode(ViSession instrSession);

ViStatus _VI_FUNC rsspecan_ConfigureHRPUWBBurstSync(ViSession instrSession,
                                                    ViInt32   reference,
                                                    ViReal64  threshold);

ViStatus _VI_FUNC rsspecan_ConfigureHRPUWBBurstLength(ViSession instrSession,
                                                      ViReal64  minimum,
                                                      ViReal64  maximum);

ViStatus _VI_FUNC rsspecan_ConfigureHRPUWBBurstDetectedOffset(
                            ViSession instrSession,
                            ViReal64  detectedOffset);

ViStatus _VI_FUNC rsspecan_ConfigureHRPUWBBurstMaxOffTimeWithinBurst(
                            ViSession instrSession,
                            ViReal64  maxOffTimeWithinBurst);

ViStatus _VI_FUNC rsspecan_ConfigureHRPUWBIQLoadStreamAutoEnabled(
                            ViSession instrSession,
                            ViBoolean autoEnabled);

ViStatus _VI_FUNC rsspecan_ConfigureHRPUWBIQLoadStream(ViSession instrSession,
                                                       ViString  loadStream);

ViStatus _VI_FUNC rsspecan_QueryHRPUWBIQStreamLists(ViSession instrSession,
                                                    ViInt32   bufferSize,
                                                    ViChar    IQStreamLists[]);

ViStatus _VI_FUNC rsspecan_ConfigureHRPUWBMeasurement(ViSession instrSession,
                                                      ViInt32   channelNumber,
                                                      ViBoolean defaultValues);

ViStatus _VI_FUNC rsspecan_ConfigureHRPUWBOversamplingFactor(ViSession instrSession,
                                                             ViInt32   oversamplingFactor);

ViStatus _VI_FUNC rsspecan_HRPUWBSetCenterFrequencyToChannelFrequency(ViSession instrSession);

ViStatus _VI_FUNC rsspecan_ConfigureHRPUWBDemodulationMode(ViSession instrSession,
                                                           ViInt32   mode);

ViStatus _VI_FUNC rsspecan_ConfigureHRPUWBDemodulationPHYDataRateMode(
                            ViSession instrSession,
                            ViInt32   PHYDataRateMode);

ViStatus _VI_FUNC rsspecan_ConfigureHRPUWBDemodulationHPRFPayloadSize(
                            ViSession instrSession,
                            ViInt32   HPRFPayloadSize);

ViStatus _VI_FUNC rsspecan_ConfigureHRPUWBDemodulationPacketFormat(
                            ViSession instrSession,
                            ViInt32   packetFormat);

ViStatus _VI_FUNC rsspecan_ConfigureHRPUWBDemodulationActiveSegment(
                            ViSession instrSession,
                            ViInt32   length,
                            ViInt32   activeSegments);

ViStatus _VI_FUNC rsspecan_ConfigureHRPUWBDemodulationMACFCS(ViSession instrSession,
                                                             ViInt32   MACFCS);

ViStatus _VI_FUNC rsspecan_ConfigureHRPUWBDemodulationSTSGap(ViSession instrSession,
                                                             ViInt32   gap);

ViStatus _VI_FUNC rsspecan_ConfigureHRPUWBAnalysisSelectedPacket(
                            ViSession instrSession,
                            ViInt32   selectedPacket);

ViStatus _VI_FUNC rsspecan_ConfigureHRPUWBAnalysis(ViSession instrSession,
                                                   ViInt32   range,
                                                   ViInt32   count);

ViStatus _VI_FUNC rsspecan_ConfigureHRPUWBAnalysisRangePacketGroup(
                            ViSession instrSession,
                            ViInt32   group);

ViStatus _VI_FUNC rsspecan_ConfigureHRPUWBAnalysisRangeStatisticsGroup(
                            ViSession instrSession,
                            ViInt32   group);

ViStatus _VI_FUNC rsspecan_QueryHRPUWBAnalysisEvaluationPacketsAnalyzed(
                            ViSession instrSession,
                            ViInt32*  analyzedPackets);

ViStatus _VI_FUNC rsspecan_ConfigureHRPUWBAnalysisEvaluationFilterSet(
                            ViSession instrSession,
                            ViInt32   filter,
                            ViString  set);

ViStatus _VI_FUNC rsspecan_ConfigureHRPUWBAnalysisEvaluationFilterName(
                            ViSession instrSession,
                            ViInt32   filter,
                            ViString  name);

ViStatus _VI_FUNC rsspecan_ConfigureHRPUWBAnalysisEvaluationFilterDelete(
                            ViSession instrSession,
                            ViInt32   filter,
                            ViString  name);

ViStatus _VI_FUNC rsspecan_AddHRPUWBWindow(ViSession instrSession,
                                           ViInt32   stringSize,
                                           ViString  referenceWindowName,
                                           ViInt32   direction,
                                           ViInt32   windowType,
                                           ViChar    returnedWindowName[]);

ViStatus _VI_FUNC rsspecan_ReplaceHRPUWBWindow(ViSession instrSession,
                                               ViString  windowName,
                                               ViInt32   windowType);

ViStatus _VI_FUNC rsspecan_QueryHRPUWBPacketAll(ViSession instrSession,
                                                ViInt32   window,
                                                ViInt32   bufferSize,
                                                ViChar    packetAll[]);

ViStatus _VI_FUNC rsspecan_QueryHRPUWBPacketSYNC(ViSession instrSession,
                                                 ViInt32   window,
                                                 ViInt32   selectSYNC,
                                                 ViInt32*  packetSYNC);

ViStatus _VI_FUNC rsspecan_QueryHRPUWBPacketSYNCAll(ViSession instrSession,
                                                    ViInt32   window,
                                                    ViInt32   selectSYNC,
                                                    ViInt32*  packetSYNCAll);

ViStatus _VI_FUNC rsspecan_QueryHRPUWBPacketData(ViSession instrSession,
                                                 ViInt32   window,
                                                 ViInt32   selectPacketData,
                                                 ViReal64* packetData);

ViStatus _VI_FUNC rsspecan_QueryHRPUWBPacketDataA0A1(ViSession instrSession,
                                                     ViInt32   window,
                                                     ViInt32   bufferSize,
                                                     ViChar    packetDataA0A1[]);

ViStatus _VI_FUNC rsspecan_QueryHRPUWBPacketDataPayload(ViSession instrSession,
                                                        ViInt32   window,
                                                        ViInt32   bufferSize,
                                                        ViChar    packetDataPayload[]);

ViStatus _VI_FUNC rsspecan_QueryHRPUWBPacketDataRatePHR(ViSession instrSession,
                                                        ViInt32   window,
                                                        ViReal64* ratePHR);

ViStatus _VI_FUNC rsspecan_QueryHRPUWBPacketDataCBurst(ViSession instrSession,
                                                       ViInt32   window,
                                                       ViInt32   bufferSize,
                                                       ViChar    cBurst[]);

ViStatus _VI_FUNC rsspecan_QueryHRPUWBPacketDataHBurst(ViSession instrSession,
                                                       ViInt32   window,
                                                       ViInt32   bufferSize,
                                                       ViChar    hBurst[]);

ViStatus _VI_FUNC rsspecan_QueryHRPUWBPacketDataMACFCS(ViSession instrSession,
                                                       ViInt32   window,
                                                       ViInt32   bufferSize,
                                                       ViChar    MACFCS[]);

ViStatus _VI_FUNC rsspecan_QueryHRPUWBPacketDataReservedBit(ViSession instrSession,
                                                            ViInt32   window,
                                                            ViInt32   bufferSize,
                                                            ViChar    reservedBit[]);

ViStatus _VI_FUNC rsspecan_QueryHRPUWBPacketDataSTSBST(ViSession instrSession,
                                                       ViInt32   window,
                                                       ViInt32   bufferSize,
                                                       ViChar    STSBST[]);

ViStatus _VI_FUNC rsspecan_QueryHRPUWBPacketDataAll(ViSession instrSession,
                                                    ViInt32   window,
                                                    ViInt32   selectPacketData,
                                                    ViInt32*  packetDataAll);

ViStatus _VI_FUNC rsspecan_QueryHRPUWBResultAll(ViSession instrSession,
                                                ViInt32   window,
                                                ViInt32   bufferSize,
                                                ViChar    resultAll[]);

ViStatus _VI_FUNC rsspecan_QueryHRPUWBResultCompatible(ViSession instrSession,
                                                       ViInt32   channel,
                                                       ViInt32   bufferSize,
                                                       ViChar    resultCompatible[]);

ViStatus _VI_FUNC rsspecan_QueryHRPUWBResultCaptureLength(ViSession instrSession,
                                                          ViInt32*  captureLength);

ViStatus _VI_FUNC rsspecan_QueryHRPUWBResultCenterFrequencyOffsetSelPacket(
                            ViSession instrSession,
                            ViInt32   window,
                            ViReal64* frequencyOffset);

ViStatus _VI_FUNC rsspecan_QueryHRPUWBResultCenterFrequencyOffset(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   measurementType,
                            ViReal64* frequencyOffset);

ViStatus _VI_FUNC rsspecan_QueryHRPUWBResultCenterFrequencyOffsetPPMSelPacket(
                            ViSession instrSession,
                            ViInt32   window,
                            ViReal64* offsetPPM);

ViStatus _VI_FUNC rsspecan_QueryHRPUWBResultCenterFrequencyOffsetPPM(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   measurementType,
                            ViReal64* frequencyOffsetPPM);

ViStatus _VI_FUNC rsspecan_QueryHRPUWBResultFrequencyChipClockErrorSelPacket(
                            ViSession instrSession,
                            ViInt32   window,
                            ViReal64* chipClockError);

ViStatus _VI_FUNC rsspecan_QueryHRPUWBResultFrequencyChipClockError(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   measurementType,
                            ViReal64* chipClockError);

ViStatus _VI_FUNC rsspecan_QueryHRPUWBResultSymbolTimeJitterRMSSelPacket(
                            ViSession instrSession,
                            ViInt32   window,
                            ViReal64* timeJitterRMS);

ViStatus _VI_FUNC rsspecan_QueryHRPUWBResultSymbolTimeJitterRMSAverage(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   measurementType,
                            ViReal64* timeJitterRMS);

ViStatus _VI_FUNC rsspecan_QueryHRPUWBResultChipTimeJitterRMSSelPacket(
                            ViSession instrSession,
                            ViInt32   window,
                            ViReal64* timeJitterRMS);

ViStatus _VI_FUNC rsspecan_QueryHRPUWBResultChipTimeJitterRMSAverage(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   measurementType,
                            ViReal64* timeJitterRMS);

ViStatus _VI_FUNC rsspecan_QueryHRPUWBResultCorrelatedPulseNMSESelPacket(
                            ViSession instrSession,
                            ViInt32   window,
                            ViReal64* pulseNMSE);

ViStatus _VI_FUNC rsspecan_QueryHRPUWBResultCorrelatedPulseNMSE(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   measurementType,
                            ViReal64* NMSE);

ViStatus _VI_FUNC rsspecan_QueryHRPUWBResultCorrelatedPulseMainLobePeakSelPacket(
                            ViSession instrSession,
                            ViInt32   window,
                            ViReal64* mainLobePeak);

ViStatus _VI_FUNC rsspecan_QueryHRPUWBResultCorrelatedPulseMainLobePeak(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   measurementType,
                            ViReal64* mainLobePeak);

ViStatus _VI_FUNC rsspecan_QueryHRPUWBResultCorrelatedPulseMainLobeWidthSelPacket(
                            ViSession instrSession,
                            ViInt32   window,
                            ViReal64* mainLobeWidth);

ViStatus _VI_FUNC rsspecan_QueryHRPUWBResultCorrelatedPulseMainLobeWidth(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   measurementType,
                            ViReal64* mainLobeWidth);

ViStatus _VI_FUNC rsspecan_QueryHRPUWBResultCorrelatedPulseMainLobeWidthPassedSelPacket(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32*  lobeWidthPassed);

ViStatus _VI_FUNC rsspecan_QueryHRPUWBResultCorrelatedPulseMainLobeWidthPassed(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   measurementType,
                            ViInt32*  lobeWidthPassed);

ViStatus _VI_FUNC rsspecan_QueryHRPUWBResultCorrelatedPulseMainLobeMinWidthSelPacket(
                            ViSession instrSession,
                            ViInt32   window,
                            ViReal64* mainLobeMinWidth);

ViStatus _VI_FUNC rsspecan_QueryHRPUWBResultCorrelatedPulseMainLobeMinWidth(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   measurementType,
                            ViReal64* lobeMinWidth);

ViStatus _VI_FUNC rsspecan_QueryHRPUWBResultCorrelatedPulseSideLobePeakLocationSelPacket(
                            ViSession instrSession,
                            ViInt32   window,
                            ViReal64* lobePeakLocation);

ViStatus _VI_FUNC rsspecan_QueryHRPUWBResultCorrelatedPulseSideLobePeakLocation(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   measurementType,
                            ViReal64* lobePeakLocation);

ViStatus _VI_FUNC rsspecan_QueryHRPUWBResultCorrelatedPulseSideLobePeakSelPacket(
                            ViSession instrSession,
                            ViInt32   window,
                            ViReal64* sideLobePeak);

ViStatus _VI_FUNC rsspecan_QueryHRPUWBResultCorrelatedPulseSideLobePeak(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   measurementType,
                            ViReal64* sideLobePeak);

ViStatus _VI_FUNC rsspecan_QueryHRPUWBResultCorrelatedPulseSideLobePeakPassedSelPacket(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32*  lobePeakPassed);

ViStatus _VI_FUNC rsspecan_QueryHRPUWBResultCorrelatedPulseSideLobePeakPassed(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   measurementType,
                            ViInt32*  sideLobePeakPassed);

ViStatus _VI_FUNC rsspecan_QueryHRPUWBResultPulseMaskSelPacket(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32*  pulseMask);

ViStatus _VI_FUNC rsspecan_QueryHRPUWBResultPulseMask(ViSession instrSession,
                                                      ViInt32   window,
                                                      ViInt32   measurementType,
                                                      ViInt32*  pulseMask);

ViStatus _VI_FUNC rsspecan_QueryHRPUWBResultPulseRiseMonotonicSelPacket(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32*  riseMonotonic);

ViStatus _VI_FUNC rsspecan_QueryHRPUWBResultPulseRiseMonotonic(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   measurementType,
                            ViInt32*  pulseRiseMonotonic);

ViStatus _VI_FUNC rsspecan_QueryHRPUWBResultPulseRiseTime(ViSession instrSession,
                                                          ViInt32   window,
                                                          ViInt32   measurementType,
                                                          ViInt32*  pulseRiseTime);

ViStatus _VI_FUNC rsspecan_QueryHRPUWBResultPulseRiseTimeSelPacket(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32*  riseTime);

ViStatus _VI_FUNC rsspecan_QueryHRPUWBResultPulseRiseTimePassed(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   measurementType,
                            ViInt32*  riseTimePassed);

ViStatus _VI_FUNC rsspecan_QueryHRPUWBResultPulseRiseTimePassedSelPacket(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32*  riseTimePassed);

ViStatus _VI_FUNC rsspecan_QueryHRPUWBResultPulseLocationSFD(ViSession instrSession,
                                                             ViInt32   window,
                                                             ViInt32   measurementType,
                                                             ViInt32*  pulseLocationSFD);

ViStatus _VI_FUNC rsspecan_QueryHRPUWBResultPulseLocationSFDSelPacket(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32*  locationSFD);

ViStatus _VI_FUNC rsspecan_QueryHRPUWBResultPulseLocationSTS(ViSession instrSession,
                                                             ViInt32   window,
                                                             ViInt32   measurementType,
                                                             ViInt32*  pulseLocationSTS);

ViStatus _VI_FUNC rsspecan_QueryHRPUWBResultPulseLocationSTSSelPacket(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32*  locationSTS);

ViStatus _VI_FUNC rsspecan_QueryHRPUWBResultPulseLocationSYNC(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   measurementType,
                            ViInt32*  pulseLocationSYNC);

ViStatus _VI_FUNC rsspecan_QueryHRPUWBResultPulseLocationSYNCSelPacket(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32*  locationSYNC);

ViStatus _VI_FUNC rsspecan_QueryHRPUWBResultEVMSHRNRMSESelPacket(
                            ViSession instrSession,
                            ViInt32   window,
                            ViReal64* SHRNRMSE);

ViStatus _VI_FUNC rsspecan_QueryHRPUWBResultEVMSHRNRMSE(ViSession instrSession,
                                                        ViInt32   window,
                                                        ViInt32   measurementType,
                                                        ViReal64* SHRNRMSE);

ViStatus _VI_FUNC rsspecan_QueryHRPUWBResultEVMPHRNRMSESelPacket(
                            ViSession instrSession,
                            ViInt32   window,
                            ViReal64* PHRNRMSE);

ViStatus _VI_FUNC rsspecan_QueryHRPUWBResultEVMPHRNRMSE(ViSession instrSession,
                                                        ViInt32   window,
                                                        ViInt32   measurementType,
                                                        ViReal64* PHRNRMSE);

ViStatus _VI_FUNC rsspecan_QueryHRPUWBResultEVMPSDUNRMSESelPacket(
                            ViSession instrSession,
                            ViInt32   window,
                            ViReal64* PSDUNRMSE);

ViStatus _VI_FUNC rsspecan_QueryHRPUWBResultEVMPSDUNRMSE(ViSession instrSession,
                                                         ViInt32   window,
                                                         ViInt32   measurementType,
                                                         ViReal64* PSDUNRMSE);

ViStatus _VI_FUNC rsspecan_QueryHRPUWBResultEVMSTSNRMSESelPacket(
                            ViSession instrSession,
                            ViInt32   window,
                            ViReal64* STSNRMSE);

ViStatus _VI_FUNC rsspecan_QueryHRPUWBResultEVMSTSNRMSE(ViSession instrSession,
                                                        ViInt32   window,
                                                        ViInt32   measurementType,
                                                        ViReal64* STSNRMSE);

ViStatus _VI_FUNC rsspecan_QueryHRPUWBResultEVMPHRPulseLevelSelPacket(
                            ViSession instrSession,
                            ViInt32   window,
                            ViReal64* PHRPulseLevel);

ViStatus _VI_FUNC rsspecan_QueryHRPUWBResultEVMPHRPulseLevel(ViSession instrSession,
                                                             ViInt32   window,
                                                             ViInt32   measurementType,
                                                             ViReal64* PHRPulseLevel);

ViStatus _VI_FUNC rsspecan_QueryHRPUWBResultEVMPSDUPulseLevelSelPacket(
                            ViSession instrSession,
                            ViInt32   window,
                            ViReal64* PSDUPulseLevel);

ViStatus _VI_FUNC rsspecan_QueryHRPUWBResultEVMPSDUPulseLevel(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   measurementType,
                            ViReal64* PSDUPulseLevel);

ViStatus _VI_FUNC rsspecan_QueryHRPUWBResultEVMSTSPulseLevelSelPacket(
                            ViSession instrSession,
                            ViInt32   window,
                            ViReal64* STSPulseLevel);

ViStatus _VI_FUNC rsspecan_QueryHRPUWBResultEVMSTSPulseLevel(ViSession instrSession,
                                                             ViInt32   window,
                                                             ViInt32   measurementType,
                                                             ViReal64* STSPulseLevel);

ViStatus _VI_FUNC rsspecan_QueryHRPUWBResultSHRMeanSelPacket(ViSession instrSession,
                                                             ViInt32   window,
                                                             ViReal64* SHRMean);

ViStatus _VI_FUNC rsspecan_QueryHRPUWBResultSHRMean(ViSession instrSession,
                                                    ViInt32   window,
                                                    ViInt32   measurementType,
                                                    ViReal64* SHRMean);

ViStatus _VI_FUNC rsspecan_QueryHRPUWBResultSHRPeakSelPacket(ViSession instrSession,
                                                             ViInt32   window,
                                                             ViReal64* SHRPeak);

ViStatus _VI_FUNC rsspecan_QueryHRPUWBResultSHRPeak(ViSession instrSession,
                                                    ViInt32   window,
                                                    ViInt32   measurementType,
                                                    ViReal64* SHRPeak);

ViStatus _VI_FUNC rsspecan_QueryHRPUWBResultPSDUMeanSelPacket(
                            ViSession instrSession,
                            ViInt32   window,
                            ViReal64* PSDUMean);

ViStatus _VI_FUNC rsspecan_QueryHRPUWBResultPSDUMean(ViSession instrSession,
                                                     ViInt32   window,
                                                     ViInt32   measurementType,
                                                     ViReal64* PSDUMean);

ViStatus _VI_FUNC rsspecan_QueryHRPUWBResultPSDUPeakSelPacket(
                            ViSession instrSession,
                            ViInt32   window,
                            ViReal64* PSDUPeak);

ViStatus _VI_FUNC rsspecan_QueryHRPUWBResultPSDUPeak(ViSession instrSession,
                                                     ViInt32   window,
                                                     ViInt32   measurementType,
                                                     ViReal64* PSDUPeak);

ViStatus _VI_FUNC rsspecan_QueryHRPUWBResultPacketMeanSelPacket(
                            ViSession instrSession,
                            ViInt32   window,
                            ViReal64* packetMean);

ViStatus _VI_FUNC rsspecan_QueryHRPUWBResultPacketMean(ViSession instrSession,
                                                       ViInt32   window,
                                                       ViInt32   measurementType,
                                                       ViReal64* packetMean);

ViStatus _VI_FUNC rsspecan_QueryHRPUWBResultPacketPeakSelPacket(
                            ViSession instrSession,
                            ViInt32   window,
                            ViReal64* packetPeak);

ViStatus _VI_FUNC rsspecan_QueryHRPUWBResultPacketPeak(ViSession instrSession,
                                                       ViInt32   window,
                                                       ViInt32   measurementType,
                                                       ViReal64* packetPeak);

ViStatus _VI_FUNC rsspecan_QueryHRPUWBResultIQOffsetSelPacket(
                            ViSession instrSession,
                            ViInt32   window,
                            ViReal64* IQOffset);

ViStatus _VI_FUNC rsspecan_QueryHRPUWBResultIQOffset(ViSession instrSession,
                                                     ViInt32   window,
                                                     ViInt32   measurementType,
                                                     ViReal64* IQOffset);

ViStatus _VI_FUNC rsspecan_QueryHRPUWBResultSTSMeanSelPacket(ViSession instrSession,
                                                             ViInt32   window,
                                                             ViReal64* STSMean);

ViStatus _VI_FUNC rsspecan_QueryHRPUWBResultSTSMean(ViSession instrSession,
                                                    ViInt32   window,
                                                    ViInt32   measurementType,
                                                    ViReal64* STSMean);

ViStatus _VI_FUNC rsspecan_QueryHRPUWBResultSTSPeakSelPacket(ViSession instrSession,
                                                             ViInt32   window,
                                                             ViReal64* STSPeak);

ViStatus _VI_FUNC rsspecan_QueryHRPUWBResultSTSPeak(ViSession instrSession,
                                                    ViInt32   window,
                                                    ViInt32   measurementType,
                                                    ViReal64* STSPeak);

ViStatus _VI_FUNC rsspecan_QueryHRPUWBResultMaxSpectralPowerSelPacket(
                            ViSession instrSession,
                            ViInt32   window,
                            ViReal64* maxSpectralPower);

ViStatus _VI_FUNC rsspecan_QueryHRPUWBResultMaxSpectralPower(ViSession instrSession,
                                                             ViInt32   window,
                                                             ViInt32   measurementType,
                                                             ViReal64* maxSpectralPower);

ViStatus _VI_FUNC rsspecan_QueryHRPUWBResultSpectrumMaskPassedSelPacket(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32*  maskPassed);

ViStatus _VI_FUNC rsspecan_QueryHRPUWBResultSpectrumMaskPassed(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   measurementType,
                            ViInt32*  spectrumMaskPassed);

ViStatus _VI_FUNC rsspecan_QueryHRPUWBResultRangingMarker(ViSession instrSession,
                                                          ViInt32   window,
                                                          ViReal64* rangingMarker);

ViStatus _VI_FUNC rsspecan_QueryHRPUWBResultRangingSelectedMarker(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   marker,
                            ViReal64* rangingMarker);

ViStatus _VI_FUNC rsspecan_QueryHRPUWBResultMagnitudeCaptureStartAll(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   bufferSize,
                            ViChar    startAll[]);

ViStatus _VI_FUNC rsspecan_QueryHRPUWBResultMagnitudeCaptureStart(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   burst,
                            ViReal64* start);

ViStatus _VI_FUNC rsspecan_QueryHRPUWBResultMagnitudeCaptureStopAll(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   bufferSize,
                            ViChar    stopAll[]);

ViStatus _VI_FUNC rsspecan_QueryHRPUWBResultMagnitudeCaptureStop(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   burst,
                            ViReal64* stop);

ViStatus _VI_FUNC rsspecan_QueryHRPUWBResultMagnitudeCaptureStateAll(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   bufferSize,
                            ViChar    stateAll[]);

ViStatus _VI_FUNC rsspecan_QueryHRPUWBResultMagnitudeCaptureState(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   burst,
                            ViInt32*  state);

ViStatus _VI_FUNC rsspecan_ConfigureHRPUWBResultDisplayBins(ViSession instrSession,
                                                            ViInt32   window,
                                                            ViInt32   bins);

ViStatus _VI_FUNC rsspecan_ConfigureHRPUWBResultDisplayXScale(
                            ViSession instrSession,
                            ViInt32   window,
                            ViReal64  xScale);

ViStatus _VI_FUNC rsspecan_ConfigureHRPUWBResultDisplayConfig(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   configure);

ViStatus _VI_FUNC rsspecan_ConfigureHRPUWBResultDisplayConfigure(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   configure);

ViStatus _VI_FUNC rsspecan_ConfigureHRPUWBResultDisplayGroup(ViSession instrSession,
                                                             ViInt32   window,
                                                             ViInt32   group);

ViStatus _VI_FUNC rsspecan_ConfigureHRPUWBResultDisplayLink(ViSession instrSession,
                                                            ViInt32   window,
                                                            ViString  link);

ViStatus _VI_FUNC rsspecan_ConfigureHRPUWBResultDisplayPacket(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   packet);

ViStatus _VI_FUNC rsspecan_setStatusRegister(ViSession instrSession,
                                             ViInt32   registerOperation,
                                             ViInt32   questionableRegister,
                                             ViInt32   enable,
                                             ViInt32   PTransition,
                                             ViInt32   NTransition);

ViStatus _VI_FUNC rsspecan_getStatusRegisterQuery(ViSession instrSession,
                                                  ViInt32   registerOperation,
                                                  ViInt32   questionableRegister,
                                                  ViInt32   registerPart,
                                                  ViInt32*  registerValue);

ViStatus _VI_FUNC rsspecan_reset(ViSession instrSession);

ViStatus _VI_FUNC rsspecan_ConfigureErrorChecking(ViSession instrSession,
                                                  ViBoolean optionChecking,
                                                  ViBoolean rangeChecking,
                                                  ViBoolean statusChecking);

ViStatus _VI_FUNC rsspecan_self_test(ViSession instrSession,
                                     ViInt16*  selfTestResult,
                                     ViChar    selfTestMessage[]);

ViStatus _VI_FUNC rsspecan_SelfTestResults(ViSession instrSession,
                                           ViInt32   bufferSize,
                                           ViChar    results[]);

ViStatus _VI_FUNC rsspecan_revision_query(ViSession instrSession,
                                          ViChar    instrumentDriverRevision[],
                                          ViChar    firmwareRevision[]);

ViStatus _VI_FUNC rsspecan_SystemVersion(ViSession instrSession,
                                         ViInt32   length,
                                         ViChar    systemVersion[]);

ViStatus _VI_FUNC rsspecan_ApplicationsVersions(ViSession instrSession,
                                                ViInt32   bufferSize,
                                                ViChar    results[]);

ViStatus _VI_FUNC rsspecan_error_query(ViSession instrSession,
                                       ViInt32*  errorCode,
                                       ViChar    errorMessage[]);

ViStatus _VI_FUNC rsspecan_ErrorList(ViSession instrSession,
                                     ViInt32   bufferSize,
                                     ViChar    errors[]);

ViStatus _VI_FUNC rsspecan_ErrorListSpecificType(ViSession instrSession,
                                                 ViInt32   messageType,
                                                 ViString  channelName,
                                                 ViInt32   bufferSize,
                                                 ViChar    messages[]);

ViStatus _VI_FUNC rsspecan_error_message(ViSession instrSession,
                                         ViStatus  errorCode,
                                         ViChar    errorMessage[]);

ViStatus _VI_FUNC rsspecan_ClearInstrumentErrors(ViSession instrSession);

ViStatus _VI_FUNC rsspecan_ClearRemoteErrors(ViSession instrSession);

ViStatus _VI_FUNC rsspecan_DisplayRemoteErrors(ViSession instrSession,
                                               ViBoolean displayRemoteErrors);

ViStatus _VI_FUNC rsspecan_QueryRemoteErrors(ViSession instrSession,
                                             ViInt32   bufferSize,
                                             ViChar    errorTable[]);

ViStatus _VI_FUNC rsspecan_IDStringFactory(ViSession instrSession);

ViStatus _VI_FUNC rsspecan_GetIDStringFactory(ViSession instrSession,
                                              ViInt32*  factoryDefault);

ViStatus _VI_FUNC rsspecan_ConfigureIDQueryFormat(ViSession instrSession,
                                                  ViInt32   IDQueryFormat);

ViStatus _VI_FUNC rsspecan_SetTimeout(ViSession instrSession,
                                      ViUInt32  OPCTimeout);

ViStatus _VI_FUNC rsspecan_GetTimeout(ViSession instrSession,
                                      ViUInt32* timeout);

ViStatus _VI_FUNC rsspecan_SetVISATimeout(ViSession instrSession,
                                          ViUInt32  VISATimeout);

ViStatus _VI_FUNC rsspecan_GetVISATimeout(ViSession instrSession,
                                          ViUInt32* VISATimeout);

ViStatus _VI_FUNC rsspecan_LockLocalKey(ViSession instrSession,
                                        ViBoolean lockLocalKey);

ViStatus _VI_FUNC rsspecan_setCheckStatus(ViSession instrSession,
                                          ViBoolean state);

ViStatus _VI_FUNC rsspecan_setCheckOption(ViSession instrSession,
                                          ViBoolean optionChecking);

ViStatus _VI_FUNC rsspecan_InstrumentOptions(ViSession instrSession,
                                             ViInt32   bufferSize,
                                             ViChar    options[],
                                             ViInt32*  bytesRead);

ViStatus _VI_FUNC rsspecan_ClearStatus(ViSession instrSession);

ViStatus _VI_FUNC rsspecan_IDQueryResponse(ViSession instrSession,
                                           ViUInt32  bufferSize,
                                           ViChar    IDQueryResponse[]);

ViStatus _VI_FUNC rsspecan_ProcessAllPreviousCommands(ViSession instrSession);

ViStatus _VI_FUNC rsspecan_QueryOPC(ViSession instrSession,
                                    ViInt32*  OPC);

ViStatus _VI_FUNC rsspecan_SetFastSweepMode(ViSession instrSession,
                                            ViInt32   fastSweepMode);

ViStatus _VI_FUNC rsspecan_SetInstrumentDateTime(ViSession instrSession,
                                                 ViInt32   year,
                                                 ViInt32   month,
                                                 ViInt32   day,
                                                 ViInt32   hours,
                                                 ViInt32   minutes,
                                                 ViInt32   seconds);

ViStatus _VI_FUNC rsspecan_GetInstrumentDateTime(ViSession instrSession,
                                                 ViInt32*  year,
                                                 ViInt32*  month,
                                                 ViInt32*  day,
                                                 ViInt32*  hours,
                                                 ViInt32*  minutes,
                                                 ViInt32*  seconds);

ViStatus _VI_FUNC rsspecan_ConfigureAutoSystemErrQuery(ViSession instrSession,
                                                       ViBoolean autoSystErrQuery);

ViStatus _VI_FUNC rsspecan_ConfigureMultiThreadLocking(ViSession instrSession,
                                                       ViBoolean multiThreadLocking);

ViStatus _VI_FUNC rsspecan_GetError(ViSession instrSession,
                                    ViStatus* code,
                                    ViInt32   bufferSize,
                                    ViChar    description[]);

ViStatus _VI_FUNC rsspecan_ClearError(ViSession instrSession);

ViStatus _VI_FUNC rsspecan_ReadToFileFromInstrument(ViSession instrSession,
                                                    ViString  source,
                                                    ViString  destination);

ViStatus _VI_FUNC rsspecan_WriteFromFileToInstrument(ViSession instrSession,
                                                     ViString  source,
                                                     ViString  destination);

ViStatus _VI_FUNC rsspecan_WriteInstrData(ViSession instrSession,
                                          ViString  writeBuffer);

ViStatus _VI_FUNC rsspecan_ReadInstrData(ViSession instrSession,
                                         ViInt32   numberOfBytesToRead,
                                         ViChar    readBuffer[],
                                         ViUInt32* numBytesRead);

ViStatus _VI_FUNC rsspecan_QueryViBoolean(ViSession  instrSession,
                                          ViString   command,
                                          ViBoolean* value);

ViStatus _VI_FUNC rsspecan_QueryViInt32(ViSession instrSession,
                                        ViString  command,
                                        ViInt32*  value);

ViStatus _VI_FUNC rsspecan_QueryViReal64(ViSession instrSession,
                                         ViString  command,
                                         ViReal64* value);

ViStatus _VI_FUNC rsspecan_QueryViString(ViSession instrSession,
                                         ViString  command,
                                         ViInt32   bufferSize,
                                         ViChar    value[]);

ViStatus _VI_FUNC rsspecan_WriteCommandWithOPCSync(ViSession instrSession,
                                                   ViString  writeBuffer,
                                                   ViUInt32  timeout);

ViStatus _VI_FUNC rsspecan_QueryWithOPCSync(ViSession instrSession,
                                            ViString  writeBuffer,
                                            ViUInt32  timeout,
                                            ViInt32   bufferSize,
                                            ViChar    readBuffer[],
                                            ViUInt32* numBytesRead);

ViStatus _VI_FUNC rsspecan_SetAttributeViInt32(ViSession instrSession,
                                               ViString  repeatedCapabilityName,
                                               ViAttr    attributeID,
                                               ViInt32   attributeValue);

ViStatus _VI_FUNC rsspecan_SetAttributeViInt64(ViSession instrSession,
                                               ViString  repeatedCapabilityName,
                                               ViAttr    attributeID,
                                               ViInt64   attributeValue);

ViStatus _VI_FUNC rsspecan_SetAttributeViReal64(ViSession instrSession,
                                                ViString  repeatedCapabilityName,
                                                ViAttr    attributeID,
                                                ViReal64  attributeValue);

ViStatus _VI_FUNC rsspecan_SetAttributeViString(ViSession instrSession,
                                                ViString  repeatedCapabilityName,
                                                ViAttr    attributeID,
                                                ViString  attributeValue);

ViStatus _VI_FUNC rsspecan_SetAttributeViBoolean(ViSession instrSession,
                                                 ViString  repeatedCapabilityName,
                                                 ViAttr    attributeID,
                                                 ViBoolean attributeValue);

ViStatus _VI_FUNC rsspecan_GetAttributeViInt32(ViSession instrSession,
                                               ViString  repeatedCapabilityName,
                                               ViAttr    attributeID,
                                               ViInt32*  attributeValue);

ViStatus _VI_FUNC rsspecan_GetAttributeViInt64(ViSession instrSession,
                                               ViString  repeatedCapabilityName,
                                               ViAttr    attributeID,
                                               ViInt64*  attributeValue);

ViStatus _VI_FUNC rsspecan_GetAttributeViReal64(ViSession instrSession,
                                                ViString  repeatedCapabilityName,
                                                ViAttr    attributeID,
                                                ViReal64* attributeValue);

ViStatus _VI_FUNC rsspecan_GetAttributeViString(ViSession instrSession,
                                                ViString  repeatedCapabilityName,
                                                ViAttr    attributeID,
                                                ViInt32   bufferSize,
                                                ViChar    attributeValue[]);

ViStatus _VI_FUNC rsspecan_GetAttributeViBoolean(ViSession  instrSession,
                                                 ViString   repeatedCapabilityName,
                                                 ViAttr     attributeID,
                                                 ViBoolean* attributeValue);

ViStatus _VI_FUNC rsspecan_GetAttributeRepCapName(ViSession instrSession,
                                                  ViAttr    attributeID,
                                                  ViString  repCapNameID,
                                                  ViInt32   index,
                                                  ViInt32   bufferSize,
                                                  ViChar    repCapName[]);

ViStatus _VI_FUNC rsspecan_GetAttributeRepeatedCapabilityIds(ViSession instrSession,
                                                             ViAttr    attributeID,
                                                             ViInt32   bufferSize,
                                                             ViChar    repeatedCapabilityId_s[]);

ViStatus _VI_FUNC rsspecan_GetAttributeRepeatedCapabilityIdNames(
                            ViSession instrSession,
                            ViAttr    attributeID,
                            ViString  repeatedCapabilityId,
                            ViInt32   bufferSize,
                            ViChar    repeatedCapabilityIdName_s[]);

ViStatus _VI_FUNC rsspecan_close(ViSession instrSession);

#if defined(__cplusplus) || defined(__cplusplus__)
}
#endif
#endif /* __RSSPECAN_HEADER */

