/*****************************************************************************
 *  Rohde&Schwarz Spectrum Analyzer instrument driver
 *  5G NR Downlink option K144
 *
 *  Original Release: September 2018
 *  By: Petr Kominek
 *
 *  Should you have any technical questions please contact the hotline of
 *  Rohde & Schwarz Vertriebs-GmbH Rohde & Schwarz Support Center
 *
 *  e-mail: CustomerSupport@rohde-schwarz.com
 *
 *****************************************************************************/

#include "rsspecan.h"
#include "rsspecan_internal.h"

static ViString offOnArr[] = {"OFF", "ON", NULL};
static ViString traceArr[] = {"", "TRACE1", "TRACE2", "TRACE3", "TRACE4", "TRACE5", "TRACE6", "TRACE7", "TRACE8", NULL};

/// HIFN  This function selects a new measurement channel with the defined
/// HIFN  channel type.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsspecan_Set5GNRDownlinkMode(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_5G_NR_MODE, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures 5G NR.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR direction/Selects the link direction you want to analyze.
/// HIPAR mode/Selects the measurement.
ViStatus _VI_FUNC rsspecan_Configure5GNR(ViSession instrSession,
                                         ViInt32   direction,
                                         ViInt32   mode)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_5G_NR_DIRECTION, direction),
            2, "Direction");

    if (direction == RSSPECAN_VAL_5G_NR_UL && (mode >= RSSPECAN_VAL_5G_NR_MEASUREMENT_CACL && mode < RSSPECAN_VAL_5G_NR_MEASUREMENT_CME))
    {
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, mode),
            3, "Mode");
    }
    else
    {
        viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_5G_NR_MEASUREMENT, mode),
                3, "Mode");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects an O-RAN test case.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR testcase/Selects an O-RAN test case.
ViStatus _VI_FUNC rsspecan_Configure5GNRORANTestcase(ViSession instrSession,
                                                     ViString  testcase)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViString(instrSession, NULL, RSSPECAN_ATTR_5G_NR_O_RAN_TESTCASE, testcase),
            2, "Testcase");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Resets the channel spacing of component carriers to its default value.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsspecan_Configure5GNRChannelSpacingReset(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsspecan_SetAttributeViString(instrSession, NULL, RSSPECAN_ATTR_5G_NR_CHANNEL_SPACING_RESET, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Selects the capture mode for combined measurements.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR mode/Selects the capture mode for combined measurements.
ViStatus _VI_FUNC rsspecan_Configure5GNRSweepMode(ViSession instrSession,
                                                  ViInt32   mode)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, NULL, RSSPECAN_ATTR_5G_NR_SWEEP_MODE, mode),
            2, "Mode");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Turns automatic signal demodulation on and off.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR cc/Component Carrier parameter.
/// HIPAR signalDemodulationEnabled/Turns automatic signal demodulation on and off.
ViStatus _VI_FUNC rsspecan_Configure5GNRDLCompleteSignalDemodulation(
                            ViSession instrSession,
                            ViInt32   cc,
                            ViBoolean signalDemodulationEnabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_COMPLETE_SIGNAL_DEMODULATION_ENABLED,
            "cc", cc - 1, RS_REPCAP_BUF_SIZE, repCap));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, repCap, RSSPECAN_ATTR_5G_NR_DL_COMPLETE_SIGNAL_DEMODULATION_ENABLED, signalDemodulationEnabled),
            3, "Signal Demodulation Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures 5G NR Downlink.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR multicarrierFilter/Turns the multicarrier filter during demodulation on and off.
/// HIPAR componentCarriers/Selects the number of component carriers analyzed in the measurement.
/// HIPAR carrierMode/Selects the capture mode for measurements on multiple component
/// HIPAR carrierMode/carriers.
ViStatus _VI_FUNC rsspecan_Configure5GNRDL(ViSession instrSession,
                                           ViBoolean multicarrierFilter,
                                           ViInt32   componentCarriers,
                                           ViInt32   carrierMode)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_5G_NR_DL_COMPONENT_CARRIER_NUMBER, componentCarriers),
            3, "Component Carriers");

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_5G_NR_DL_CONFIGURATION_MULTICARRIER_FILTER, multicarrierFilter),
            2, "Multicarrier Filter");

    if (componentCarriers > 1)
    {
        viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_5G_NR_DL_COMPONENT_CARRIER_DATA_CAPTURE, carrierMode),
                4, "Carrier Mode");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures 5G NR Data Capture.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR frameCountEnabled/Turns manual definition of number of frames to analyze on and off.
/// HIPAR frameCount/Defines the number of frames to analyze.
/// HIPAR subframestoAnalyze/Reserved
ViStatus _VI_FUNC rsspecan_Configure5GNRDLDataCapture(ViSession instrSession,
                                                      ViBoolean frameCountEnabled,
                                                      ViInt32   frameCount,
                                                      ViInt32   subframestoAnalyze)
{
    ViStatus error = VI_SUCCESS;
             subframestoAnalyze;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_5G_NR_DL_DATA_CAPTURE_FRAME_COUNT_ENABLED, frameCountEnabled),
            2, "Frame Count Enabled");

    if (frameCountEnabled == VI_TRUE)
    {
        checkErr(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_5G_NR_DL_DATA_CAPTURE_FRAME_COUNT_AUTO_ENABLED, VI_FALSE));

        viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_5G_NR_DL_DATA_CAPTURE_FRAME_COUNT, frameCount),
                3, "Frame Count");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Turns the long capture on and off.
/// HIFN
/// HIFN     Prerequisites for this function
/// HIFN     - Number of component carriers must be '1'
/// HIFN  (RSSPECAN_ATTR_5G_NR_DL_COMPONENT_CARRIER_NUMBER).
/// HIFN     - Turn off segmented capture
/// HIFN  (RSSPECAN_ATTR_5G_NR_DL_SEGMENTED_CAPTURE_ENABLED).
/// HIFN     - Multi frame configuration is not supported. Every frame must have
/// HIFN  the same configuration.
/// HIFN
/// HIFN     Effects of this function
/// HIFN     - Frame count functions become unavailable:
/// HIFN     RSSPECAN_ATTR_5G_NR_DL_DATA_CAPTURE_FRAME_COUNT
/// HIFN     RSSPECAN_ATTR_5G_NR_DL_DATA_CAPTURE_FRAME_COUNT_AUTO_ENABLED
/// HIFN     RSSPECAN_ATTR_5G_NR_DL_DATA_CAPTURE_FRAME_COUNT_ENABLED
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR longCaptureEnabled/Turns the long capture on and off.
ViStatus _VI_FUNC rsspecan_Configure5GNRDLLongCapture(ViSession instrSession,
                                                      ViBoolean longCaptureEnabled)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, NULL, RSSPECAN_ATTR_5G_NR_DL_DATA_CAPTURE_LONG_CAPTURE_ENABLED, longCaptureEnabled),
            2, "Long Capture Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Turns analysis of custom signals with repeating slots on and off.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR maxNoofSlotstoAnalyze/Turns analysis of custom signals with repeating slots on and off.
ViStatus _VI_FUNC rsspecan_Configure5GNRDLDataCaptureSignalRepeatsMaxNoofSlotstoAnalyze(
                            ViSession instrSession,
                            ViBoolean signalRepeatsMaxSlots)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, NULL, RSSPECAN_ATTR_5G_NR_DL_DATA_CAPTURE_SIGNAL_REPEATS_MAX_NO_OF_SLOTS_TO_ANALYZE, signalRepeatsMaxSlots),
            2, "Max No of Slots to Analyze");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Turns automatic selection of the number of frames to analyze on and
/// HIFN  off.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR autoFrameCountEnabled/Turns automatic selection of the number of frames to analyze on and
/// HIPAR autoFrameCountEnabled/off.
ViStatus _VI_FUNC rsspecan_Configure5GNRDLDataCaptureAccordingToStandard(
                            ViSession instrSession,
                            ViBoolean autoFrameCountEnabled)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, NULL, RSSPECAN_ATTR_5G_NR_DL_DATA_CAPTURE_FRAME_COUNT_AUTO_ENABLED, autoFrameCountEnabled),
            2, "Auto Frame Count Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the number of slots that are analyzed.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR select/Selects the values to set.
/// HIPAR maxSlotsPerFrameToAnalyze/Defines the number of slots that are analyzed.
ViStatus _VI_FUNC rsspecan_Configure5GNRDLDataCaptureMaximumSlotsPerFrameToAnalyze(
                            ViSession instrSession,
                            ViInt32   select,
                            ViInt32   maxSlotsPerFrameToAnalyze)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, select, RSSPECAN_VAL_5G_NR_ALL, RSSPECAN_VAL_5G_NR_VALUE),
            2, "Select");

    if (select == RSSPECAN_VAL_5G_NR_ALL)
    {
        checkErr(rsspecan_SetAttributeViString(instrSession, NULL, RSSPECAN_ATTR_5G_NR_DL_DATA_CAPTURE_ALL_SLOTS_PER_FRAME_TO_ANALYZE, NULL));
    }
    else
    {
        viCheckParm(rsspecan_SetAttributeViInt32(instrSession, NULL, RSSPECAN_ATTR_5G_NR_DL_DATA_CAPTURE_MAXIMUM_SLOTS_PER_FRAME_TO_ANALYZE, maxSlotsPerFrameToAnalyze),
                3, "Max Slots Per Frame To Analyze");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the segmented capture.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR enabled/Turns segmented capture on and off.
/// HIPAR events/This control sets the number of trigger events for which data segments
/// HIPAR events/are to be captured.
/// HIPAR offset/This control defines an offset to the trigger event at which data
/// HIPAR offset/capturing starts. For a negative offset, data capturing starts before
/// HIPAR offset/the actual trigger event.
/// HIPAR length/This control defines a time period (starting from the trigger offset)
/// HIPAR length/in which data is captured. If multiple events occur within one segment
/// HIPAR length/length, the segment is extended.
ViStatus _VI_FUNC rsspecan_Configure5GNRDLSegmentedCapture(ViSession instrSession,
                                                           ViBoolean enabled,
                                                           ViInt32   events,
                                                           ViReal64  offset,
                                                           ViReal64  length)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, NULL, RSSPECAN_ATTR_5G_NR_DL_SEGMENTED_CAPTURE_ENABLED, enabled),
            2, "Enabled");

    if (enabled == VI_TRUE)
    {
        viCheckParm(rsspecan_SetAttributeViInt32(instrSession, NULL, RSSPECAN_ATTR_5G_NR_DL_SEGMENTED_CAPTURE_EVENTS, events),
                3, "Events");

        viCheckParm(rsspecan_SetAttributeViReal64(instrSession, NULL, RSSPECAN_ATTR_5G_NR_DL_SEGMENTED_CAPTURE_OFFSET, offset),
                4, "Offset");

        viCheckParm(rsspecan_SetAttributeViReal64(instrSession, NULL, RSSPECAN_ATTR_5G_NR_DL_SEGMENTED_CAPTURE_LENGTH, length),
                5, "Length");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function selects a tab in diagrams with multiple subwindows (or
/// HIFN  views).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR tab/Selects the tab parameter.
ViStatus _VI_FUNC rsspecan_Configure5GNRDLTabSelect(ViSession instrSession,
                                                    ViInt32   window,
                                                    ViInt32   tab)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_tab[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_TAB_SELECT,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_TAB_SELECT,
            "Tab", tab - 1, RS_REPCAP_BUF_SIZE, repCap_tab));

    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_tab, NULL);

    checkErr(rsspecan_SetAttributeViString(instrSession, repCap, RSSPECAN_ATTR_5G_NR_DL_TAB_SELECT, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function queries the tpyes of traces in a diagram.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR arraySize/Defines the size of array.
/// HIPAR type/Queries the tpyes of traces in a diagram.
ViStatus _VI_FUNC rsspecan_Query5GNRDLTraceType(ViSession instrSession,
                                                ViInt32   window,
                                                ViInt32   arraySize,
                                                ViChar    type[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_TRACE_TYPE,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap));

    viCheckParm(rsspecan_GetAttributeViString(instrSession, repCap, RSSPECAN_ATTR_5G_NR_DL_TRACE_TYPE,
            arraySize, type),
            4, "Type");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function queries the marker position on the z-axis of
/// HIFN  three-dimensional result displays. Instead of returning a certain type
/// HIFN  of value (EVM, Power or Allocation ID), which is possible with
/// HIFN  CALCulate<n>:MARKer<m>:Z?, returns all types of values (EVM, Power and
/// HIFN  Allocation ID), regardless of the result display type.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR marker/This control selects marker.
/// HIPAR arraySize/Defines the size of array.
/// HIPAR allResults/Queries the marker position on the z-axis of three-dimensional result
/// HIPAR allResults/displays. Instead of returning a certain type of value (EVM, Power or
/// HIPAR allResults/Allocation ID), which is possible with CALCulate<n>:MARKer<m>:Z?,
/// HIPAR allResults/returns all types of values (EVM, Power and Allocation ID), regardless
/// HIPAR allResults/of the result display type.
/// HIPAR returnedValues/Returns the number of returned values.
ViStatus _VI_FUNC rsspecan_Query5GNRDLMarkerZAll(ViSession instrSession,
                                                 ViInt32   window,
                                                 ViInt32   marker,
                                                 ViInt32   arraySize,
                                                 ViReal64  results[],
                                                 ViInt32   *returnedValues)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K144"));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CALC%ld:MARK%ld:Z:ALL?", window, marker);
    checkErr(RsCore_QueryFloatArrayToUserBuffer(instrSession, cmd, arraySize, results, returnedValues));

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function queries the marker position on the z-axis of
/// HIFN  three-dimensional result displays.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR marker/This control selects marker.
/// HIPAR z/Queries the marker position on the z-axis of three-dimensional result
/// HIPAR z/displays.
ViStatus _VI_FUNC rsspecan_Query5GNRDLMarkerZ(ViSession instrSession,
                                              ViInt32   window,
                                              ViInt32   marker,
                                              ViReal64  *z)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_marker[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_MARKER_POSITION_Z,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_MARKER_POSITION_Z,
            "Marker", marker - 1, RS_REPCAP_BUF_SIZE, repCap_marker));

    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_marker, NULL);

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_5G_NR_MARKER_POSITION_Z, z),
            4, "Z");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Adjusts the amplitude settings, including attenuator and preamplifier,
/// HIFN  to achieve the optimal EVM using the maximum dynamic range. Note that
/// HIFN  this process can up to several minutes, depending on the number of
/// HIFN  component carriers you are measuring. Use RSSPECAN_ATTR_OPC_TIMEOUT to
/// HIFN  define OPC timeout.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsspecan_NR5GDLAutoEVM(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsspecan_SetAttributeViString(instrSession, NULL, RSSPECAN_ATTR_5G_NR_DL_AUTO_EVM, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Determines the ideal reference level based on the current measurement
/// HIFN  data and settings on all connected input sources. This ensures that the
/// HIFN  settings of the RF attenuation and the reference level are optimally
/// HIFN  adjusted to the signal level without overloading the R&S FSW or
/// HIFN  limiting the dynamic range by an S/N ratio that is too small. The
/// HIFN  attribute is available for MIMO measurements with more than one input
/// HIFN  source.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsspecan_NR5GDLAutoLevelAll(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsspecan_SetAttributeViString(instrSession, NULL, RSSPECAN_ATTR_5G_NR_DL_AUTO_LEVEL_ALL, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects the number of slots to be used during the auto EVM routine.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR numberOfSlots/Selects the number of slots to be used during the auto EVM routine.
ViStatus _VI_FUNC rsspecan_Configure5GNRDLAutoEVMSlots(ViSession instrSession,
                                                       ViInt32   numberOfSlots)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, NULL, RSSPECAN_ATTR_5G_NR_DL_AUTO_EVM_SLOTS, numberOfSlots),
            2, "Number of Slots");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the tracking settings.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channelEstimation/Selects the channel estimation method.
/// HIPAR phaseTracking/Turns phase tracking on and off.
/// HIPAR gainImbalanceQuadratureError/Turns the calculation of the gain imbalance and the quadrature error
/// HIPAR gainImbalanceQuadratureError/in the result summary on and off.
/// HIPAR timeAveraging/Select the averaging interval for channel estimation.
ViStatus _VI_FUNC rsspecan_Configure5GNRDLTracking(ViSession instrSession,
                                                   ViInt32   channelEstimation,
                                                   ViInt32   phaseTracking,
                                                   ViBoolean gainImbalanceQuadratureError,
                                                   ViInt32   timeAveraging)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, NULL, RSSPECAN_ATTR_5G_NR_DL_TRACKING_CHANNEL_ESTIMATION, channelEstimation),
            2, "Channel Estimation");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, NULL, RSSPECAN_ATTR_5G_NR_DL_TRACKING_PHASE, phaseTracking),
            3, "Phase Tracking");

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, NULL, RSSPECAN_ATTR_5G_NR_DL_TRACKING_GAIN_IMBALANCE_QUADRATURE_ERROR_ENABLED, gainImbalanceQuadratureError),
            4, "Gain Imbalance/Quadrature Error");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, NULL, RSSPECAN_ATTR_5G_NR_DL_TRACKING_CHANNEL_ESTIMATION_TIME_AVERAGING, timeAveraging),
            5, "Time Averaging");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures time and level tracking on and off.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR trackingTime/Turns time tracking on and off.
/// HIPAR trackingLevel/Turns level tracking on and off.
ViStatus _VI_FUNC rsspecan_Configure5GNRDLTrackingTimeLevel(ViSession instrSession,
                                                            ViBoolean trackingTime,
                                                            ViBoolean trackingLevel)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, NULL, RSSPECAN_ATTR_5G_NR_DL_TRACKING_TIME, trackingTime),
            2, "Tracking Time");

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, NULL, RSSPECAN_ATTR_5G_NR_DL_TRACKING_LEVEL, trackingLevel),
            3, "Tracking Level");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the demodulation.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR demodulationData/Selects the point at which the data is demodulated for the bitstream.
/// HIPAR CORESETAnalysisMode/Selects the CORESET analysis mode.
ViStatus _VI_FUNC rsspecan_Configure5GNRDLDemodulation(ViSession instrSession,
                                                       ViInt32   demodulationData,
                                                       ViInt32   CORESETAnalysisMode)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, NULL, RSSPECAN_ATTR_5G_NR_DL_DEMODULATION_DATA, demodulationData),
            2, "Demodulation Data");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, NULL, RSSPECAN_ATTR_5G_NR_DL_CORESET_ANALYSIS_MODE, CORESETAnalysisMode),
            3, "CORESET Analysis Mode");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects the EVM calculation method.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR calculationMethod/Selects the EVM calculation method.
ViStatus _VI_FUNC rsspecan_Configure5GNRDLDemodulationEVMCalculationMethod(
                            ViSession instrSession,
                            ViInt32   calculationMethod)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, NULL, RSSPECAN_ATTR_5G_NR_DL_EVM_CALCULATION_METHOD, calculationMethod),
            2, "Calculation Method");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the symbol time adjustment.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR symbolTimeAdjustment/Defines the symbol time adjustment.
ViStatus _VI_FUNC rsspecan_Configure5GNRDLDemodulationSymbolTimeAdjustment(
                            ViSession instrSession,
                            ViReal64  symbolTimeAdjustment)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, NULL, RSSPECAN_ATTR_5G_NR_DL_SYMBOL_TIME_ADJUSTMENT, symbolTimeAdjustment),
            2, "Symbol Time Adjustment");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Selects the PDSCH reference data.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR pdschReferenceData/Selects the PDSCH reference data.
ViStatus _VI_FUNC rsspecan_Configure5GNRDLDemodulationPDSCHReferenceData(
                            ViSession instrSession,
                            ViInt32   pdschReferenceData)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, NULL, RSSPECAN_ATTR_5G_NR_DL_PDSCH_REFERENCE_DATA, pdschReferenceData),
            2, "PDSCH Reference Data");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Selects the CORESET reference data.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR coresetReferenceData/Selects the CORESET reference data.
ViStatus _VI_FUNC rsspecan_Configure5GNRDLDemodulationCoresetReferenceData(
                            ViSession instrSession,
                            ViInt32   coresetReferenceData)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, NULL, RSSPECAN_ATTR_5G_NR_DL_CORESET_REFERENCE_DATA, coresetReferenceData),
            2, "Coreset Reference Data");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Turns the extended frequency lock range on and off.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR extendedFrequencyLockRange/Turns the extended frequency lock range on and off.
ViStatus _VI_FUNC rsspecan_Configure5GNRDLDemodulationExtendedFrequencyLockRange(
                            ViSession instrSession,
                            ViBoolean extendedFrequencyLockRange)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, NULL, RSSPECAN_ATTR_5G_NR_DL_EXTENDED_FREQUENCY_LOCK_RANGE, extendedFrequencyLockRange),
            2, "Extended Frequency Lock Range");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures 5G NR Downlink Physical Settings.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR operatingBand/Selects the operating band.
/// HIPAR generalFrequencyOffset/Defines the general frequency offset or the frequency offset for a
/// HIPAR generalFrequencyOffset/component carrier.
ViStatus _VI_FUNC rsspecan_Configure5GNRDLPhysicalSettings(ViSession instrSession,
                                                           ViInt32   operatingBand,
                                                           ViReal64  generalFrequencyOffset)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_5G_NR_DL_PHYSICAL_SETTINGS_OPERATING_BAND, operatingBand),
            2, "Operating Band");

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_5G_NR_DL_GENERAL_FREQUENCY_OFFSET, generalFrequencyOffset),
            3, "General Frequency Offset");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects the channel raster of a component carrier.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channelRaster/Selects the channel raster of a component carrier.
ViStatus _VI_FUNC rsspecan_Configure5GNRDLChannelRaster(ViSession instrSession,
                                                        ViInt32   channelRaster)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, NULL, RSSPECAN_ATTR_5G_NR_DL_GENERAL_CHANNEL_RASTER, channelRaster),
            2, "Channel Raster");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function configures the frequency characteristics of all
/// HIFN  component carriers at the same time
/// HIFN     according to a certain logic.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR globalMCFrequency/Defines the global multicarrier frequency for component carrier
/// HIPAR globalMCFrequency/setups.
/// HIPAR frequencyOffsetRelativeTo/Selects the reference point for frequency offsets of component
/// HIPAR frequencyOffsetRelativeTo/carriers in a multicarrier setup.
/// HIPAR frequencyOffsetMode/Selects the frequency offset mode for component carriers in a
/// HIPAR frequencyOffsetMode/multicarrier setup.
/// HIPAR carrierSpacing/Defines the carrier spacing for equidistant frequency offsets in a
/// HIPAR carrierSpacing/multicarrier setup. This frequency offset applies to all component
/// HIPAR carrierSpacing/carriers in the setup.
/// HIPAR fixedCCOffset/Turns a fixed frequency offset for component carriers on and off.
ViStatus _VI_FUNC rsspecan_Configure5GNRDLMCSetup(ViSession instrSession,
                                                  ViReal64  globalMCFrequency,
                                                  ViInt32   frequencyOffsetRelativeTo,
                                                  ViInt32   frequencyOffsetMode,
                                                  ViReal64  carrierSpacing,
                                                  ViBoolean fixedCCOffset)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, NULL, RSSPECAN_ATTR_5G_NR_DL_GENERAL_GLOBAL_MC_FREQUENCY, globalMCFrequency),
            2, "Global MC Frequency");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, NULL, RSSPECAN_ATTR_5G_NR_DL_GENERAL_FREQUENCY_OFFSET_REFERENCE_POINT, frequencyOffsetRelativeTo),
            3, "Frequency Offset Relative To");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, NULL, RSSPECAN_ATTR_5G_NR_DL_GENERAL_FREQUENCY_OFFSET_MODE, frequencyOffsetMode),
            4, "Frequency Offset Mode");
    if (frequencyOffsetMode == RSSPECAN_VAL_FREQ_OFFSET_MODE_EQUIDISTANT)
    {
        viCheckParm(rsspecan_SetAttributeViReal64(instrSession, NULL, RSSPECAN_ATTR_5G_NR_DL_GENERAL_FREQUENCY_OFFSET_CARRIER_SPACING, carrierSpacing),
                5, "Carrier Spacing");
    }
    else if (frequencyOffsetMode == RSSPECAN_VAL_FREQ_OFFSET_MODE_ARBITRARY)
    {
        viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, NULL, RSSPECAN_ATTR_5G_NR_DL_GENERAL_FREQUENCY_OFFSET_FIXED, fixedCCOffset),
                6, "Fixed CC Offset");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Defines the MC frequency offset of a component carrier. This funtion
/// HIFN  can be used when Globacl MC Frequency is set to GMC Frequency with
/// HIFN  RSSPECAN_ATTR_5G_NR_DL_GENERAL_FREQUENCY_OFFSET_REFERENCE_POINT.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR cc/Component Carrier parameter.
/// HIPAR globalMCFrequencyOffset/Defines the MC frequency offset of a component carrier.
ViStatus _VI_FUNC rsspecan_Configure5GNRDLGlobalMCFrequencyOffset(
                            ViSession instrSession,
                            ViInt32   cc,
                            ViReal64  globalMCFrequencyOffset)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_COMPONENT_CARRIER_GENERAL_MC_FREQUENCY_OFFSET,
            "cc", cc - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "CC");

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_5G_NR_DL_COMPONENT_CARRIER_GENERAL_MC_FREQUENCY_OFFSET, globalMCFrequencyOffset),
            3, "Global MC Frequency Offset");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Synchronizes the global multicarrier frequency to the current center
/// HIFN  frequency (= center of all carriers).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsspecan_NR5GDLSyncMCFrequencyToCenter(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsspecan_SetAttributeViString(instrSession, NULL, RSSPECAN_ATTR_5G_NR_DL_GENERAL_SYNC_GLOBAL_MC_FREQUENCY_TO_CENTER, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Restores the signal description of multiple carriers from a single
/// HIFN  file.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR fileName/File Name parameter.
ViStatus _VI_FUNC rsspecan_Configure5GNRDLPhysicalSettingsLoadSignalDescription(
                            ViSession instrSession,
                            ViString  fileName)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_5G_NR_DL_PHYSICAL_SETTINGS_LOAD_SIGNAL_DESCRIPTION, fileName),
            2, "File Name");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Saves the signal description of multiple carriers in a single file.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR fileName/File Name parameter.
ViStatus _VI_FUNC rsspecan_Configure5GNRDLPhysicalSettingsSaveSignalDescription(
                            ViSession instrSession,
                            ViString  fileName)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_5G_NR_DL_PHYSICAL_SETTINGS_SAVE_SIGNAL_DESCRIPTION, fileName),
            2, "File Name");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures 5G NR Downlink Physical Settings with
/// HIFN  Component Carrier.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR cc/Component Carrier parameter.
/// HIPAR bandwidth/Select the channel bandwidth of the 5G NR carrier.
/// HIPAR deploymentFrequency/Selects the deployment frequency range of the signal.
/// HIPAR centerFrequency/Sets the center frequency for RF measurements.
/// HIPAR generalFrequencyOffset/Defines the general frequency offset or the frequency offset for a
/// HIPAR generalFrequencyOffset/component carrier.
ViStatus _VI_FUNC rsspecan_Configure5GNRDLCCPhysicalSettings(ViSession instrSession,
                                                             ViInt32   cc,
                                                             ViInt32   bandwidth,
                                                             ViInt32   deploymentFrequency,
                                                             ViReal64  centerFrequency,
                                                             ViReal64  generalFrequencyOffset)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_PHYSICAL_SETTINGS_BANDWIDTH,
            "cc", cc - 1, RS_REPCAP_BUF_SIZE, repCap));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_5G_NR_DL_PHYSICAL_SETTINGS_BANDWIDTH, bandwidth),
            3, "Bandwidth");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_5G_NR_DL_DEPLOYMENT_FREQUENCY_RANGE, deploymentFrequency),
            4, "Deployment Frequency");

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_5G_NR_DL_CENTER_FREQUENCY, centerFrequency),
            5, "Center Frequency");

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_5G_NR_DL_COMPONENT_CARRIER_GENERAL_FREQUENCY_OFFSET, generalFrequencyOffset),
            6, "General Frequency Offset");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function defines the cell ID.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR cc/Component Carrier parameter.
/// HIPAR idAuto/Defines the cell ID Auto settings.
/// HIPAR cellID/Defines the cell ID.
ViStatus _VI_FUNC rsspecan_Configure5GNRDLCCPhysicalSettingsCellID(
                            ViSession instrSession,
                            ViInt32   cc,
                            ViBoolean idAuto,
                            ViInt32   cellID)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_CELL_ID_AUTO,
            "cc", cc - 1, RS_REPCAP_BUF_SIZE, repCap));

    checkErr(rsspecan_SetAttributeViString(instrSession, repCap, RSSPECAN_ATTR_5G_NR_DL_CELL_ID_AUTO, NULL));

    if (idAuto == VI_FALSE)
    {
        viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_5G_NR_DL_CELL_ID, cellID),
                4, "Cell ID");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Restores the signal description.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR cc/Component Carrier parameter.
/// HIPAR fileName/File Name parameter.
ViStatus _VI_FUNC rsspecan_Configure5GNRDLCCPhysicalSettingsLoadSignalDescription(
                            ViSession instrSession,
                            ViInt32   cc,
                            ViString  fileName)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_LOAD_SIGNAL_DESCRIPTION,
            "cc", cc - 1, RS_REPCAP_BUF_SIZE, repCap));

    viCheckParm(rsspecan_SetAttributeViString(instrSession, repCap, RSSPECAN_ATTR_5G_NR_DL_LOAD_SIGNAL_DESCRIPTION, fileName),
            3, "File Name");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Saves the signal description.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR cc/Component Carrier parameter.
/// HIPAR fileName/File Name parameter.
ViStatus _VI_FUNC rsspecan_Configure5GNRDLCCPhysicalSettingsSaveSignalDescription(
                            ViSession instrSession,
                            ViInt32   cc,
                            ViString  fileName)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_SAVE_SIGNAL_DESCRIPTION,
            "cc", cc - 1, RS_REPCAP_BUF_SIZE, repCap));

    viCheckParm(rsspecan_SetAttributeViString(instrSession, repCap, RSSPECAN_ATTR_5G_NR_DL_SAVE_SIGNAL_DESCRIPTION, fileName),
            3, "File Name");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Loads an test model (NR-FR-TM) as defined by 3GPP (38.141-1 / -2).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR cc/Component Carrier parameter.
/// HIPAR fileName/File Name parameter.
ViStatus _VI_FUNC rsspecan_Configure5GNRDLCCPhysicalSettingsLoadTestModel(
                            ViSession instrSession,
                            ViInt32   cc,
                            ViString  fileName)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_LOAD_TEST_MODEL,
            "cc", cc - 1, RS_REPCAP_BUF_SIZE, repCap));

    viCheckParm(rsspecan_SetAttributeViString(instrSession, repCap, RSSPECAN_ATTR_5G_NR_DL_LOAD_TEST_MODEL, fileName),
            3, "File Name");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Turns automatic detection of the bandwidth part configuration on and
/// HIFN  off.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR cc/Component Carrier parameter.
/// HIPAR bandwidthPartDemodulation/Turns automatic detection of the bandwidth part configuration on and
/// HIPAR bandwidthPartDemodulation/off.
ViStatus _VI_FUNC rsspecan_Configure5GNRDLBandwidthPartDemodulation(
                            ViSession instrSession,
                            ViInt32   cc,
                            ViInt32   bandwidthPartDemodulation)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_BANDWIDTH_PARTS_DEMODULATION,
            "cc", cc - 1, RS_REPCAP_BUF_SIZE, repCap));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_5G_NR_DL_BANDWIDTH_PARTS_DEMODULATION, bandwidthPartDemodulation),
            3, "Bandwidth Parts Demodulation");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function adds a bandwidth part to the signal configuration.
/// HIFN     The new bandwidth part has the highest index number. Index numbers
/// HIFN  start at 0. For example, if you already have three bandwidth parts and
/// HIFN  a add a fourth one, the new one has the index number 3.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR cc/Component Carrier parameter.
ViStatus _VI_FUNC rsspecan_Configure5GNRDLBandwidthPartAdd(ViSession instrSession,
                                                           ViInt32   cc)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_BANDWIDTH_PART_ADD,
            "cc", cc - 1, RS_REPCAP_BUF_SIZE, repCap));

    checkErr(rsspecan_SetAttributeViString(instrSession, repCap, RSSPECAN_ATTR_5G_NR_DL_BANDWIDTH_PART_ADD, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function deletes a bandwidth part.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR cc/Component Carrier parameter.
/// HIPAR bandwidthPart/Sets the Bandwidth Part parameter.
ViStatus _VI_FUNC rsspecan_Configure5GNRDLBandwidthPartDelete(
                            ViSession instrSession,
                            ViInt32   cc,
                            ViInt32   bandwidthPart)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_bwp[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_BANDWIDTH_PART_DELETE,
            "cc", cc - 1, RS_REPCAP_BUF_SIZE, repCap));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_BANDWIDTH_PART_DELETE,
            "BWPart", bandwidthPart , RS_REPCAP_BUF_SIZE, repCap_bwp));

    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_bwp, NULL);

    checkErr(rsspecan_SetAttributeViString(instrSession, repCap, RSSPECAN_ATTR_5G_NR_DL_BANDWIDTH_PART_DELETE, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function deletes all bandwidth parts.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR cc/Component Carrier parameter.
ViStatus _VI_FUNC rsspecan_Configure5GNRDLBandwidthPartClear(ViSession instrSession,
                                                             ViInt32   cc)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_BANDWIDTH_PART_CLEAR,
            "cc", cc - 1, RS_REPCAP_BUF_SIZE, repCap));

    checkErr(rsspecan_SetAttributeViString(instrSession, repCap, RSSPECAN_ATTR_5G_NR_DL_BANDWIDTH_PART_CLEAR, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function duplicates an existing bandwidth part.
/// HIFN     A duplication of a bandwidth part also duplicates its slot and PDSCH
/// HIFN  configuration.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR cc/Component Carrier parameter.
/// HIPAR bandwidthPart/Sets the Bandwidth Part parameter.
ViStatus _VI_FUNC rsspecan_Configure5GNRDLBandwidthPartDuplicate(
                            ViSession instrSession,
                            ViInt32   cc,
                            ViInt32   bandwidthPart)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_bwp[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_BANDWIDTH_PART_DUPLICATE,
            "cc", cc - 1, RS_REPCAP_BUF_SIZE, repCap));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_BANDWIDTH_PART_DUPLICATE,
            "BWPart", bandwidthPart , RS_REPCAP_BUF_SIZE, repCap_bwp));

    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_bwp, NULL);

    checkErr(rsspecan_SetAttributeViString(instrSession, repCap, RSSPECAN_ATTR_5G_NR_DL_BANDWIDTH_PART_DUPLICATE, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function queries the number of analyzed bandwidth parts.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR cc/Component Carrier parameter.
/// HIPAR analyzedParts/Queries the number of analyzed bandwidth parts.
ViStatus _VI_FUNC rsspecan_Query5GNRDLBandwidthPartAnalyzed(ViSession instrSession,
                                                            ViInt32   cc,
                                                            ViInt32   *analyzedParts)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_BANDWIDTH_PART_ANALYZED,
            "cc", cc - 1, RS_REPCAP_BUF_SIZE, repCap));

    viCheckParm(rsspecan_GetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_5G_NR_DL_BANDWIDTH_PART_ANALYZED, analyzedParts),
            3, "Analyzed Parts");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures a Bandwidth Part resource blocks.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR cc/Component Carrier parameter.
/// HIPAR bandwidthPart/Sets the Bandwidth Part parameter.
/// HIPAR count/Defines the number of physical resource blocks a bandwidth part
/// HIPAR count/occupies.
/// HIPAR offset/Defines a resource block offset for a bandwidth part.
ViStatus _VI_FUNC rsspecan_Configure5GNRDLBandwidthPartResourceBlocks(
                            ViSession instrSession,
                            ViInt32   cc,
                            ViInt32   bandwidthPart,
                            ViInt32   count,
                            ViInt32   offset)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_bwp[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_BANDWIDTH_PART_RB_COUNT,
            "cc", cc - 1, RS_REPCAP_BUF_SIZE, repCap));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_BANDWIDTH_PART_RB_COUNT,
            "BWPart", bandwidthPart , RS_REPCAP_BUF_SIZE, repCap_bwp));

    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_bwp, NULL);

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_5G_NR_DL_BANDWIDTH_PART_RB_COUNT, count),
            5, "Count");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_5G_NR_DL_BANDWIDTH_PART_RB_OFFSET, offset),
            6, "Offset");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function selects the subcarrier spacing of a bandwidth part.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR cc/Component Carrier parameter.
/// HIPAR bandwidthPart/Sets the Bandwidth Part parameter.
/// HIPAR spacing/Selects the subcarrier spacing of a bandwidth part.
ViStatus _VI_FUNC rsspecan_Configure5GNRDLBandwidthPartSubcarrierSpacing(
                            ViSession instrSession,
                            ViInt32   cc,
                            ViInt32   bandwidthPart,
                            ViInt32   spacing)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_bwp[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_BANDWIDTH_PART_SUBCARRIER_SPACING,
            "cc", cc - 1, RS_REPCAP_BUF_SIZE, repCap));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_BANDWIDTH_PART_SUBCARRIER_SPACING,
            "BWPart", bandwidthPart , RS_REPCAP_BUF_SIZE, repCap_bwp));

    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_bwp, NULL);

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_5G_NR_DL_BANDWIDTH_PART_SUBCARRIER_SPACING, spacing),
            4, "Spacing");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function applies an existing frame configuration to all other
/// HIFN  frames.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR cc/Component Carrier parameter.
ViStatus _VI_FUNC rsspecan_Configure5GNRDLFramePasteAll(ViSession instrSession,
                                                        ViInt32   cc)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_FRAME_PASTE_ALL,
            "cc", cc - 1, RS_REPCAP_BUF_SIZE, repCap));

    checkErr(rsspecan_SetAttributeViString(instrSession, repCap, RSSPECAN_ATTR_5G_NR_DL_FRAME_PASTE_ALL, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function defines the number of configurable frames.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR cc/Component Carrier parameter.
/// HIPAR frames/Defines the number of configurable frames.
ViStatus _VI_FUNC rsspecan_Configure5GNRDLFrameNumber(ViSession instrSession,
                                                      ViInt32   cc,
                                                      ViInt32   frames)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_NUMBER_OF_CONFIGURABLE_FRAMES,
            "cc", cc - 1, RS_REPCAP_BUF_SIZE, repCap));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_5G_NR_DL_NUMBER_OF_CONFIGURABLE_FRAMES, frames),
            3, "Frames");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function copies a frame configuration.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR cc/Component Carrier parameter.
/// HIPAR frame/Sets the Frame parameter.
ViStatus _VI_FUNC rsspecan_Configure5GNRDLFrameCopy(ViSession instrSession,
                                                    ViInt32   cc,
                                                    ViInt32   frame)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_frame[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_FRAME_COPY,
            "cc", cc - 1, RS_REPCAP_BUF_SIZE, repCap));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_FRAME_COPY,
            "NR5GFrame", frame - 1, RS_REPCAP_BUF_SIZE, repCap_frame));

    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_frame, NULL);

    checkErr(rsspecan_SetAttributeViString(instrSession, repCap, RSSPECAN_ATTR_5G_NR_DL_FRAME_COPY, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function applies an existing frame configuration to another one.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR cc/Component Carrier parameter.
/// HIPAR frame/Sets the Frame parameter.
ViStatus _VI_FUNC rsspecan_Configure5GNRDLFramePaste(ViSession instrSession,
                                                     ViInt32   cc,
                                                     ViInt32   frame)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_frame[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_FRAME_PASTE,
            "cc", cc - 1, RS_REPCAP_BUF_SIZE, repCap));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_FRAME_PASTE,
            "NR5GFrame", frame - 1, RS_REPCAP_BUF_SIZE, repCap_frame));

    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_frame, NULL);

    checkErr(rsspecan_SetAttributeViString(instrSession, repCap, RSSPECAN_ATTR_5G_NR_DL_FRAME_PASTE, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function defines the number of slots that you can configure.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR cc/Component Carrier parameter.
/// HIPAR frame/Sets the Frame parameter.
/// HIPAR bandwidthPart/Sets the Bandwidth Part parameter.
/// HIPAR slots/Defines the number of slots that you can configure.
ViStatus _VI_FUNC rsspecan_Configure5GNRDLFrameNumberOfSlots(ViSession instrSession,
                                                             ViInt32   cc,
                                                             ViInt32   frame,
                                                             ViInt32   bandwidthPart,
                                                             ViInt32   slots)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_frame[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_bwp[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_NUMBER_OF_SLOTS_TO_CONFIGURE,
            "cc", cc - 1, RS_REPCAP_BUF_SIZE, repCap));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_NUMBER_OF_SLOTS_TO_CONFIGURE,
            "NR5GFrame", frame - 1, RS_REPCAP_BUF_SIZE, repCap_frame));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_NUMBER_OF_SLOTS_TO_CONFIGURE,
            "BWPart", bandwidthPart , RS_REPCAP_BUF_SIZE, repCap_bwp));

    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_frame, NULL);
    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_bwp, NULL);

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_5G_NR_DL_NUMBER_OF_SLOTS_TO_CONFIGURE, slots),
            5, "Slots");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function queries the number of analyzed slots.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR cc/Component Carrier parameter.
/// HIPAR frame/Sets the Frame parameter.
/// HIPAR bandwidthPart/Sets the Bandwidth Part parameter.
/// HIPAR slots/Queries the number of analyzed slots.
ViStatus _VI_FUNC rsspecan_Query5GNRDLFrameNumberOfAnalyzedSlots(
                            ViSession instrSession,
                            ViInt32   cc,
                            ViInt32   frame,
                            ViInt32   bandwidthPart,
                            ViInt32   *slots)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_frame[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_bwp[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_SLOTS_COUNTS,
            "cc", cc - 1, RS_REPCAP_BUF_SIZE, repCap));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_SLOTS_COUNTS,
            "NR5GFrame", frame - 1, RS_REPCAP_BUF_SIZE, repCap_frame));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_SLOTS_COUNTS,
            "BWPart", bandwidthPart , RS_REPCAP_BUF_SIZE, repCap_bwp));

    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_frame, NULL);
    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_bwp, NULL);

    viCheckParm(rsspecan_GetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_5G_NR_DL_SLOTS_COUNTS, slots),
            5, "Slots");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function applies an existing slot configuration to all other
/// HIFN  slots.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR cc/Component Carrier parameter.
/// HIPAR frame/Sets the Frame parameter.
/// HIPAR bandwidthPart/Sets the Bandwidth Part parameter.
/// HIPAR slot/Defines the number of slot.
ViStatus _VI_FUNC rsspecan_Configure5GNRDLSlotPasteAll(ViSession instrSession,
                                                       ViInt32   cc,
                                                       ViInt32   frame,
                                                       ViInt32   bandwidthPart,
                                                       ViInt32   slot)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_frame[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_bwp[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_slot[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_SLOT_PASTE_ALL,
            "cc", cc - 1, RS_REPCAP_BUF_SIZE, repCap));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_SLOT_PASTE_ALL,
            "NR5GFrame", frame - 1, RS_REPCAP_BUF_SIZE, repCap_frame));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_SLOT_PASTE_ALL,
            "BWPart", bandwidthPart , RS_REPCAP_BUF_SIZE, repCap_bwp));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_SLOT_PASTE_ALL,
            "NR5GSlot", slot , RS_REPCAP_BUF_SIZE, repCap_slot));

    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_frame, NULL);
    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_bwp, NULL);
    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_slot, NULL);

    checkErr(rsspecan_SetAttributeViString(instrSession, repCap, RSSPECAN_ATTR_5G_NR_DL_SLOT_PASTE_ALL, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function applies an existing slot configuration to another one.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR cc/Component Carrier parameter.
/// HIPAR frame/Sets the Frame parameter.
/// HIPAR bandwidthPart/Sets the Bandwidth Part parameter.
/// HIPAR slot/Defines the number of slot.
ViStatus _VI_FUNC rsspecan_Configure5GNRDLSlotPaste(ViSession instrSession,
                                                    ViInt32   cc,
                                                    ViInt32   frame,
                                                    ViInt32   bandwidthPart,
                                                    ViInt32   slot)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_frame[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_bwp[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_slot[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_SLOT_PASTE,
            "cc", cc - 1, RS_REPCAP_BUF_SIZE, repCap));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_SLOT_PASTE,
            "NR5GFrame", frame - 1, RS_REPCAP_BUF_SIZE, repCap_frame));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_SLOT_PASTE,
            "BWPart", bandwidthPart , RS_REPCAP_BUF_SIZE, repCap_bwp));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_SLOT_PASTE,
            "NR5GSlot", slot , RS_REPCAP_BUF_SIZE, repCap_slot));

    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_frame, NULL);
    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_bwp, NULL);
    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_slot, NULL);

    checkErr(rsspecan_SetAttributeViString(instrSession, repCap, RSSPECAN_ATTR_5G_NR_DL_SLOT_PASTE, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects the paste logic when you copy a slot configuration to other
/// HIFN  slots.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR cc/Component Carrier parameter.
/// HIPAR frame/Sets the Frame parameter.
/// HIPAR bandwidthPart/Sets the Bandwidth Part parameter.
/// HIPAR slot/Defines the number of slot.
/// HIPAR mode/Selects the paste logic when you copy a slot configuration to other
/// HIPAR mode/slots.
/// HIPAR period/Defines to which slots a slot configuration is copied to.
/// HIPAR duration/Defines to which slots a slot configuration is copied to.
/// HIPAR slots/Copies a slot configuration to a specific set of slots.
ViStatus _VI_FUNC rsspecan_Configure5GNRDLSlotPasteSettings(ViSession instrSession,
                                                            ViInt32   cc,
                                                            ViInt32   frame,
                                                            ViInt32   bandwidthPart,
                                                            ViInt32   slot,
                                                            ViInt32   mode,
                                                            ViInt32   period,
                                                            ViInt32   duration,
                                                            ViString  slots)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_frame[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_bwp[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_slot[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_SLOT_PASTE_MODE,
            "cc", cc - 1, RS_REPCAP_BUF_SIZE, repCap));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_SLOT_PASTE_MODE,
            "NR5GFrame", frame - 1, RS_REPCAP_BUF_SIZE, repCap_frame));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_SLOT_PASTE_MODE,
            "BWPart", bandwidthPart , RS_REPCAP_BUF_SIZE, repCap_bwp));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_SLOT_PASTE_MODE,
            "NR5GSlot", slot , RS_REPCAP_BUF_SIZE, repCap_slot));

    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_frame, NULL);
    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_bwp, NULL);
    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_slot, NULL);

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_5G_NR_DL_SLOT_PASTE_MODE, mode),
            6, "Mode");

    switch (mode)
    {
        case RSSPECAN_VAL_PASTE_MODE_CUSTOM:
            viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_5G_NR_DL_SLOT_PASTE_PERIOD, period),
                    7, "Period");

            viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_5G_NR_DL_SLOT_PASTE_DURATION, duration),
                    8, "Duration");
            break;
        case RSSPECAN_VAL_PASTE_MODE_SLOTS:
            viCheckParm(rsspecan_SetAttributeViString(instrSession, repCap, RSSPECAN_ATTR_5G_NR_DL_SLOT_PASTE_SLOT, slots),
                    9, "Slots");

            break;
        case RSSPECAN_VAL_PASTE_MODE_DATA:
            break;
        case RSSPECAN_VAL_PASTE_MODE_UNUSED:
            break;
        default:
            checkErr(RsCore_InvalidViInt32Value(instrSession, mode));
            break;
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Copies a slot configuration to other slots.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR cc/Component Carrier parameter.
/// HIPAR frame/Sets the Frame parameter.
/// HIPAR bandwidthPart/Sets the Bandwidth Part parameter.
/// HIPAR slot/Defines the number of slot.
ViStatus _VI_FUNC rsspecan_NR5GDLSlotPasteTo(ViSession instrSession,
                                             ViInt32   cc,
                                             ViInt32   frame,
                                             ViInt32   bandwidthPart,
                                             ViInt32   slot)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_frame[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_bwp[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_slot[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_SLOT_PASTE_TO,
            "cc", cc - 1, RS_REPCAP_BUF_SIZE, repCap));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_SLOT_PASTE_TO,
            "NR5GFrame", frame - 1, RS_REPCAP_BUF_SIZE, repCap_frame));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_SLOT_PASTE_TO,
            "BWPart", bandwidthPart , RS_REPCAP_BUF_SIZE, repCap_bwp));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_SLOT_PASTE_TO,
            "NR5GSlot", slot , RS_REPCAP_BUF_SIZE, repCap_slot));

    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_frame, NULL);
    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_bwp, NULL);
    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_slot, NULL);

    checkErr(rsspecan_SetAttributeViString(instrSession, repCap, RSSPECAN_ATTR_5G_NR_DL_SLOT_PASTE_TO, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function resets the slot configuration of a bandwidth part to its
/// HIFN  default state.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR cc/Component Carrier parameter.
/// HIPAR frame/Sets the Frame parameter.
/// HIPAR bandwidthPart/Sets the Bandwidth Part parameter.
/// HIPAR slot/Defines the number of slot.
ViStatus _VI_FUNC rsspecan_Configure5GNRDLSlotPreset(ViSession instrSession,
                                                     ViInt32   cc,
                                                     ViInt32   frame,
                                                     ViInt32   bandwidthPart)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_frame[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_bwp[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_SLOT_PRESET,
            "cc", cc - 1, RS_REPCAP_BUF_SIZE, repCap));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_SLOT_PRESET,
            "NR5GFrame", frame - 1, RS_REPCAP_BUF_SIZE, repCap_frame));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_SLOT_PRESET,
            "BWPart", bandwidthPart , RS_REPCAP_BUF_SIZE, repCap_bwp));

    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_frame, NULL);
    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_bwp, NULL);

    checkErr(rsspecan_SetAttributeViString(instrSession, repCap, RSSPECAN_ATTR_5G_NR_DL_SLOT_PRESET, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function defines the number of PDSCH allocations in a slot.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR cc/Component Carrier parameter.
/// HIPAR frame/Sets the Frame parameter.
/// HIPAR bandwidthPart/Sets the Bandwidth Part parameter.
/// HIPAR slot/Defines the number of slot.
/// HIPAR allocations/Defines the number of PDSCH allocations in a slot.
ViStatus _VI_FUNC rsspecan_Configure5GNRDLPDSCHAllocations(ViSession instrSession,
                                                           ViInt32   cc,
                                                           ViInt32   frame,
                                                           ViInt32   bandwidthPart,
                                                           ViInt32   slot,
                                                           ViInt32   allocations)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_frame[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_bwp[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_slot[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_PDSCH_ALLOCATIONS_IN_SLOT,
            "cc", cc - 1, RS_REPCAP_BUF_SIZE, repCap));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_PDSCH_ALLOCATIONS_IN_SLOT,
            "NR5GFrame", frame - 1, RS_REPCAP_BUF_SIZE, repCap_frame));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_PDSCH_ALLOCATIONS_IN_SLOT,
            "BWPart", bandwidthPart , RS_REPCAP_BUF_SIZE, repCap_bwp));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_PDSCH_ALLOCATIONS_IN_SLOT,
            "NR5GSlot", slot , RS_REPCAP_BUF_SIZE, repCap_slot));

    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_frame, NULL);
    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_bwp, NULL);
    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_slot, NULL);

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_5G_NR_DL_PDSCH_ALLOCATIONS_IN_SLOT, allocations),
            6, "Allocations");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function selects the allocation type of a slot.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR cc/Component Carrier parameter.
/// HIPAR frame/Sets the Frame parameter.
/// HIPAR bandwidthPart/Sets the Bandwidth Part parameter.
/// HIPAR slot/Defines the number of slot.
/// HIPAR slotType/Selects the allocation type of a slot.
ViStatus _VI_FUNC rsspecan_Configure5GNRDLPDSCHAllocationType(
                            ViSession instrSession,
                            ViInt32   cc,
                            ViInt32   frame,
                            ViInt32   bandwidthPart,
                            ViInt32   slot,
                            ViInt32   slotType)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_frame[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_bwp[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_slot[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_PDSCH_SLOT_ALLOCATION,
            "cc", cc - 1, RS_REPCAP_BUF_SIZE, repCap));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_PDSCH_SLOT_ALLOCATION,
            "NR5GFrame", frame - 1, RS_REPCAP_BUF_SIZE, repCap_frame));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_PDSCH_SLOT_ALLOCATION,
            "BWPart", bandwidthPart , RS_REPCAP_BUF_SIZE, repCap_bwp));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_PDSCH_SLOT_ALLOCATION,
            "NR5GSlot", slot , RS_REPCAP_BUF_SIZE, repCap_slot));

    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_frame, NULL);
    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_bwp, NULL);
    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_slot, NULL);

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_5G_NR_DL_PDSCH_SLOT_ALLOCATION, slotType),
            6, "Slot Type");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function copies a slot configuration.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR cc/Component Carrier parameter.
/// HIPAR frame/Sets the Frame parameter.
/// HIPAR bandwidthPart/Sets the Bandwidth Part parameter.
/// HIPAR slot/Defines the number of slot.
ViStatus _VI_FUNC rsspecan_Configure5GNRDLPDSCHCopySlotConfiguration(
                            ViSession instrSession,
                            ViInt32   cc,
                            ViInt32   frame,
                            ViInt32   bandwidthPart,
                            ViInt32   slot)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_frame[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_bwp[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_slot[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_PDSCH_COPY_SLOT_CONFIGURATION,
            "cc", cc - 1, RS_REPCAP_BUF_SIZE, repCap));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_PDSCH_COPY_SLOT_CONFIGURATION,
            "NR5GFrame", frame - 1, RS_REPCAP_BUF_SIZE, repCap_frame));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_PDSCH_COPY_SLOT_CONFIGURATION,
            "BWPart", bandwidthPart , RS_REPCAP_BUF_SIZE, repCap_bwp));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_PDSCH_COPY_SLOT_CONFIGURATION,
            "NR5GSlot", slot , RS_REPCAP_BUF_SIZE, repCap_slot));

    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_frame, NULL);
    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_bwp, NULL);
    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_slot, NULL);

    checkErr(rsspecan_SetAttributeViString(instrSession, repCap, RSSPECAN_ATTR_5G_NR_DL_PDSCH_COPY_SLOT_CONFIGURATION, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function selects the modulation of a PDSCH allocation.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR cc/Component Carrier parameter.
/// HIPAR frame/Sets the Frame parameter.
/// HIPAR bandwidthPart/Sets the Bandwidth Part parameter.
/// HIPAR slot/Defines the number of slot.
/// HIPAR allocation/Defines the number of PDSCH allocations in a slot.
/// HIPAR modulation/Selects the modulation of a PDSCH allocation.
ViStatus _VI_FUNC rsspecan_Configure5GNRDLPDSCHModulation(ViSession instrSession,
                                                          ViInt32   cc,
                                                          ViInt32   frame,
                                                          ViInt32   bandwidthPart,
                                                          ViInt32   slot,
                                                          ViInt32   allocation,
                                                          ViInt32   modulation)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_frame[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_bwp[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_slot[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_allocation[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_PDSCH_MODULATION,
            "cc", cc - 1, RS_REPCAP_BUF_SIZE, repCap));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_PDSCH_MODULATION,
            "NR5GFrame", frame - 1, RS_REPCAP_BUF_SIZE, repCap_frame));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_PDSCH_MODULATION,
            "BWPart", bandwidthPart , RS_REPCAP_BUF_SIZE, repCap_bwp));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_PDSCH_MODULATION,
            "NR5GSlot", slot , RS_REPCAP_BUF_SIZE, repCap_slot));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_PDSCH_MODULATION,
            "NR5GAlloc", allocation , RS_REPCAP_BUF_SIZE, repCap_allocation));

    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_frame, NULL);
    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_bwp, NULL);
    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_slot, NULL);
    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_allocation, NULL);

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_5G_NR_DL_PDSCH_MODULATION, modulation),
            6, "Modulation");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function selects the modulation of a PDSCH allocation.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR cc/Component Carrier parameter.
/// HIPAR frame/Sets the Frame parameter.
/// HIPAR bandwidthPart/Sets the Bandwidth Part parameter.
/// HIPAR slot/Defines the number of slot.
/// HIPAR allocation/Defines the number of PDSCH allocations in a slot.
/// HIPAR codeword/Selects codeword.
/// HIPAR modulation/Selects the modulation of a PDSCH allocation.
ViStatus _VI_FUNC rsspecan_Configure5GNRDLPDSCHCodewordModulation(
                            ViSession instrSession,
                            ViInt32   cc,
                            ViInt32   frame,
                            ViInt32   bandwidthPart,
                            ViInt32   slot,
                            ViInt32   allocation,
                            ViInt32   codeword,
                            ViInt32   modulation)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";
    ViChar   *p2string;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, cc, 1, 16),
            2, "CC");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, bandwidthPart, 0, 11),
            4, "Bandwidth Part");
    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &RsSpecAn_rng5GNRPDSCHModulation, modulation, &p2string), 7, "Modulation");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, allocation, 0, 99),
            6, "Allocation");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CONF:NR5G:DL:CC%ld:FRAM%ld:BWP%ld:SLOT%ld:ALL%ld:CW%ld:MOD %s", cc, frame, bandwidthPart, slot, allocation, codeword, p2string);
    checkErr(RsCore_Write(instrSession, cmd));
    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures PDSCH Resource Blocks.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR cc/Component Carrier parameter.
/// HIPAR frame/Sets the Frame parameter.
/// HIPAR bandwidthPart/Sets the Bandwidth Part parameter.
/// HIPAR slot/Defines the number of slot.
/// HIPAR allocation/Defines the number of PDSCH allocations in a slot.
/// HIPAR count/Selects the number of resource blocks that a PDSCH allocation
/// HIPAR count/occupies.
/// HIPAR offset/Defines the resource block offset of a PDSCH allocation.
ViStatus _VI_FUNC rsspecan_Configure5GNRDLPDSCHResourceBlocks(
                            ViSession instrSession,
                            ViInt32   cc,
                            ViInt32   frame,
                            ViInt32   bandwidthPart,
                            ViInt32   slot,
                            ViInt32   allocation,
                            ViInt32   count,
                            ViInt32   offset)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_frame[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_bwp[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_slot[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_alloc[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_PDSCH_RB_COUNT,
            "cc", cc - 1, RS_REPCAP_BUF_SIZE, repCap));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_PDSCH_RB_COUNT,
            "NR5GFrame", frame - 1, RS_REPCAP_BUF_SIZE, repCap_frame));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_PDSCH_RB_COUNT,
            "BWPart", bandwidthPart , RS_REPCAP_BUF_SIZE, repCap_bwp));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_PDSCH_RB_COUNT,
            "NR5GSlot", slot , RS_REPCAP_BUF_SIZE, repCap_slot));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_PDSCH_RB_COUNT,
            "NR5GAlloc", allocation , RS_REPCAP_BUF_SIZE, repCap_alloc));

    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_frame, NULL);
    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_bwp, NULL);
    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_slot, NULL);
    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_alloc, NULL);

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_5G_NR_DL_PDSCH_RB_COUNT, count),
            7, "Count");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_5G_NR_DL_PDSCH_RB_OFFSET, offset),
            8, "Offset");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures PDSCH Symbol.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR cc/Component Carrier parameter.
/// HIPAR frame/Sets the Frame parameter.
/// HIPAR bandwidthPart/Sets the Bandwidth Part parameter.
/// HIPAR slot/Defines the number of slot.
/// HIPAR allocation/Defines the number of PDSCH allocations in a slot.
/// HIPAR count/Selects the number of symbols that a PDSCH allocation occupies.
/// HIPAR offset/Defines the symbol offset of a PDSCH allocation.
ViStatus _VI_FUNC rsspecan_Configure5GNRDLPDSCHSymbol(ViSession instrSession,
                                                      ViInt32   cc,
                                                      ViInt32   frame,
                                                      ViInt32   bandwidthPart,
                                                      ViInt32   slot,
                                                      ViInt32   allocation,
                                                      ViInt32   count,
                                                      ViInt32   offset)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_frame[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_bwp[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_slot[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_alloc[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_PDSCH_SYMBOL_COUNT,
            "cc", cc - 1, RS_REPCAP_BUF_SIZE, repCap));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_PDSCH_SYMBOL_COUNT,
            "NR5GFrame", frame - 1, RS_REPCAP_BUF_SIZE, repCap_frame));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_PDSCH_SYMBOL_COUNT,
            "BWPart", bandwidthPart , RS_REPCAP_BUF_SIZE, repCap_bwp));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_PDSCH_SYMBOL_COUNT,
            "NR5GSlot", slot , RS_REPCAP_BUF_SIZE, repCap_slot));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_PDSCH_SYMBOL_COUNT,
            "NR5GAlloc", allocation , RS_REPCAP_BUF_SIZE, repCap_alloc));

    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_frame, NULL);
    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_bwp, NULL);
    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_slot, NULL);
    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_alloc, NULL);

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_5G_NR_DL_PDSCH_SYMBOL_COUNT, count),
            7, "Count");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_5G_NR_DL_PDSCH_SYMBOL_OFFSET, offset),
            8, "Offset");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Turns PRB bundling for PDSCH allocations on and off.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR cc/Component Carrier parameter.
/// HIPAR frame/Sets the Frame parameter.
/// HIPAR bandwidthPart/Sets the Bandwidth Part parameter.
/// HIPAR slot/Defines the number of slot.
/// HIPAR combineAllocations/Turns PRB bundling for PDSCH allocations on and off.
ViStatus _VI_FUNC rsspecan_Configure5GNRDLCombinePDSCHAllocationsSameID(
                            ViSession instrSession,
                            ViInt32   cc,
                            ViInt32   frame,
                            ViInt32   bandwidthPart,
                            ViInt32   slot,
                            ViBoolean combineAllocations)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_frame[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_bwp[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_slot[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_SLOT_COMBINE_PDSCH_ALLOCATIONS_SAME_ID,
            "cc", cc - 1, RS_REPCAP_BUF_SIZE, repCap));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_SLOT_COMBINE_PDSCH_ALLOCATIONS_SAME_ID,
            "NR5GFrame", frame - 1, RS_REPCAP_BUF_SIZE, repCap_frame));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_SLOT_COMBINE_PDSCH_ALLOCATIONS_SAME_ID,
            "BWPart", bandwidthPart , RS_REPCAP_BUF_SIZE, repCap_bwp));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_SLOT_COMBINE_PDSCH_ALLOCATIONS_SAME_ID,
            "NR5GSlot", slot , RS_REPCAP_BUF_SIZE, repCap_slot));

    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_frame, NULL);
    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_bwp, NULL);
    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_slot, NULL);

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, repCap, RSSPECAN_ATTR_5G_NR_DL_SLOT_COMBINE_PDSCH_ALLOCATIONS_SAME_ID, combineAllocations),
            6, "Combine Allocations");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function configures the PDSCH copy settings.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR cc/Component Carrier parameter.
/// HIPAR frame/Sets the Frame parameter.
/// HIPAR bandwidthPart/Sets the Bandwidth Part parameter.
/// HIPAR slot/Defines the number of slot.
/// HIPAR allocation/Defines the number of PDSCH allocations in a slot.
/// HIPAR mode/Selects the paste logic when you copy a PDSCH configuration to other
/// HIPAR mode/slots
/// HIPAR period/Defines to which slots a PDSCH configuration is copied to.
/// HIPAR duration/Defines to which slots a PDSCH configuration is copied to.
/// HIPAR slots/Copies a PDSCH allocation configuration to a specific set of slots.
ViStatus _VI_FUNC rsspecan_Configure5GNRDLPDSCHCopySettings(ViSession instrSession,
                                                            ViInt32   cc,
                                                            ViInt32   frame,
                                                            ViInt32   bandwidthPart,
                                                            ViInt32   slot,
                                                            ViInt32   allocation,
                                                            ViInt32   mode,
                                                            ViInt32   period,
                                                            ViInt32   duration,
                                                            ViString  slots)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_frame[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_bwp[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_slot[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_alloc[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_PDSCH_COPY_MODE,
            "cc", cc - 1, RS_REPCAP_BUF_SIZE, repCap));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_PDSCH_COPY_MODE,
            "NR5GFrame", frame - 1, RS_REPCAP_BUF_SIZE, repCap_frame));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_PDSCH_COPY_MODE,
            "BWPart", bandwidthPart , RS_REPCAP_BUF_SIZE, repCap_bwp));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_PDSCH_COPY_MODE,
            "NR5GSlot", slot , RS_REPCAP_BUF_SIZE, repCap_slot));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_PDSCH_COPY_MODE,
            "NR5GAlloc", allocation , RS_REPCAP_BUF_SIZE, repCap_alloc));

    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_frame, NULL);
    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_bwp, NULL);
    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_slot, NULL);
    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_alloc, NULL);

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_5G_NR_DL_PDSCH_COPY_MODE, mode),
            7, "Mode");

    switch (mode)
    {
        case RSSPECAN_VAL_NR5G_COPY_MODE_CUSTOM:
            viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_5G_NR_DL_PDSCH_COPY_PERIOD, period),
                    8, "Period");

            viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_5G_NR_DL_PDSCH_COPY_DURATION, duration),
                    9, "Duration");
            break;
        case RSSPECAN_VAL_NR5G_COPY_MODE_SLOT:
            viCheckParm(rsspecan_SetAttributeViString(instrSession, repCap, RSSPECAN_ATTR_5G_NR_DL_PDSCH_COPY_SLOT, slots),
                    10, "Slots");
            break;
        default:
            checkErr(RsCore_InvalidViInt32Value(instrSession, mode));
            break;
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function copies PDSCH configuration to other slots.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR cc/Component Carrier parameter.
/// HIPAR frame/Sets the Frame parameter.
/// HIPAR bandwidthPart/Sets the Bandwidth Part parameter.
/// HIPAR slot/Defines the number of slot.
/// HIPAR allocation/Defines the number of PDSCH allocations in a slot.
ViStatus _VI_FUNC rsspecan_NR5GDLPDSCHCopy(ViSession instrSession,
                                           ViInt32   cc,
                                           ViInt32   frame,
                                           ViInt32   bandwidthPart,
                                           ViInt32   slot,
                                           ViInt32   allocation)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_frame[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_bwp[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_slot[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_alloc[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_PDSCH_COPY,
            "cc", cc - 1, RS_REPCAP_BUF_SIZE, repCap));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_PDSCH_COPY,
            "NR5GFrame", frame - 1, RS_REPCAP_BUF_SIZE, repCap_frame));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_PDSCH_COPY,
            "BWPart", bandwidthPart , RS_REPCAP_BUF_SIZE, repCap_bwp));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_PDSCH_COPY,
            "NR5GSlot", slot , RS_REPCAP_BUF_SIZE, repCap_slot));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_PDSCH_COPY,
            "NR5GAlloc", allocation , RS_REPCAP_BUF_SIZE, repCap_alloc));

    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_frame, NULL);
    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_bwp, NULL);
    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_slot, NULL);
    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_alloc, NULL);

    checkErr(rsspecan_SetAttributeViString(instrSession, repCap, RSSPECAN_ATTR_5G_NR_DL_PDSCH_COPY, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures enhanced PDSCH.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR cc/Component Carrier parameter.
/// HIPAR frame/Sets the Frame parameter.
/// HIPAR bandwidthPart/Sets the Bandwidth Part parameter.
/// HIPAR slot/Defines the number of slot.
/// HIPAR allocation/Defines the number of PDSCH allocations in a slot.
/// HIPAR mapping/Selects the codeword to layer mapping.
/// HIPAR power/Defines the relative power of a PDSCH allocation.
/// HIPAR userID/Defines the user ID of a PDSCH allocation.
ViStatus _VI_FUNC rsspecan_Configure5GNRDLEnhancedPDSCH(ViSession instrSession,
                                                        ViInt32   cc,
                                                        ViInt32   frame,
                                                        ViInt32   bandwidthPart,
                                                        ViInt32   slot,
                                                        ViInt32   allocation,
                                                        ViInt32   mapping,
                                                        ViReal64  power,
                                                        ViInt32   userID)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_frame[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_bwp[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_slot[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_alloc[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_ENHANCED_PDSCH_MAPPING,
            "cc", cc - 1, RS_REPCAP_BUF_SIZE, repCap));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_ENHANCED_PDSCH_MAPPING,
            "NR5GFrame", frame - 1, RS_REPCAP_BUF_SIZE, repCap_frame));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_ENHANCED_PDSCH_MAPPING,
            "BWPart", bandwidthPart , RS_REPCAP_BUF_SIZE, repCap_bwp));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_ENHANCED_PDSCH_MAPPING,
            "NR5GSlot", slot , RS_REPCAP_BUF_SIZE, repCap_slot));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_ENHANCED_PDSCH_MAPPING,
            "NR5GAlloc", allocation , RS_REPCAP_BUF_SIZE, repCap_alloc));

    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_frame, NULL);
    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_bwp, NULL);
    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_slot, NULL);
    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_alloc, NULL);

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_5G_NR_DL_ENHANCED_PDSCH_MAPPING, mapping),
            7, "Mapping");

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_5G_NR_DL_ENHANCED_PDSCH_POWER, power),
            8, "Power");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_5G_NR_DL_ENHANCED_PDSCH_USER_ID, userID),
            9, "User ID");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the interleaving size of VRB bundles.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR cc/Component Carrier parameter.
/// HIPAR frame/Sets the Frame parameter.
/// HIPAR bandwidthPart/Sets the Bandwidth Part parameter.
/// HIPAR slot/Defines the number of slot.
/// HIPAR allocation/Defines the number of PDSCH allocations in a slot.
/// HIPAR vrbtoPRBInterleave/Selects the interleaving size of VRB bundles.
ViStatus _VI_FUNC rsspecan_Configure5GNRDLEnhancedPDSCHVRBtoPRBInterleave(
                            ViSession instrSession,
                            ViInt32   cc,
                            ViInt32   frame,
                            ViInt32   bandwidthPart,
                            ViInt32   slot,
                            ViInt32   allocation,
                            ViInt32   vrbtoPRBInterleave)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_frame[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_bwp[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_slot[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_alloc[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_ENHANCED_PDSCH_VRB_TO_PRB_INTERLEAVER,
            "cc", cc - 1, RS_REPCAP_BUF_SIZE, repCap));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_ENHANCED_PDSCH_VRB_TO_PRB_INTERLEAVER,
            "NR5GFrame", frame - 1, RS_REPCAP_BUF_SIZE, repCap_frame));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_ENHANCED_PDSCH_VRB_TO_PRB_INTERLEAVER,
            "BWPart", bandwidthPart , RS_REPCAP_BUF_SIZE, repCap_bwp));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_ENHANCED_PDSCH_VRB_TO_PRB_INTERLEAVER,
            "NR5GSlot", slot , RS_REPCAP_BUF_SIZE, repCap_slot));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_ENHANCED_PDSCH_VRB_TO_PRB_INTERLEAVER,
            "NR5GAlloc", allocation , RS_REPCAP_BUF_SIZE, repCap_alloc));

    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_frame, NULL);
    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_bwp, NULL);
    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_slot, NULL);
    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_alloc, NULL);

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_5G_NR_DL_ENHANCED_PDSCH_VRB_TO_PRB_INTERLEAVER, vrbtoPRBInterleave),
            7, "VRB-to-PRB Interleaver");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures enhanced PDSCH.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR cc/Component Carrier parameter.
/// HIPAR frame/Sets the Frame parameter.
/// HIPAR bandwidthPart/Sets the Bandwidth Part parameter.
/// HIPAR slot/Defines the number of slot.
/// HIPAR allocation/Defines the number of PDSCH allocations in a slot.
/// HIPAR antennaPort/Selects the antenna ports for PDSCH transmission.
/// HIPAR cdmGroupsWithoutData/Selects the number of CDM groups that are reserved and contain no
/// HIPAR cdmGroupsWithoutData/data. They are therefore not used by the PDSCH for data transmission.
/// HIPAR cdmGroupsWithoutData/In the resource grid, the resource elements for CDM (between PDSCH DMRS
/// HIPAR cdmGroupsWithoutData/resource elements) remain empty.
/// HIPAR referencePoint/Defines the reference point of the PDSCH DMRS.
ViStatus _VI_FUNC rsspecan_Configure5GNRDLEnhancedPDSCHSettings(
                            ViSession instrSession,
                            ViInt32   cc,
                            ViInt32   frame,
                            ViInt32   bandwidthPart,
                            ViInt32   slot,
                            ViInt32   allocation,
                            ViString  antennaPort,
                            ViInt32   cdmGroupsWithoutData,
                            ViInt32   referencePoint)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_frame[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_bwp[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_slot[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_alloc[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_ENHANCED_PDSCH_ANTENNA_PORT,
            "cc", cc - 1, RS_REPCAP_BUF_SIZE, repCap));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_ENHANCED_PDSCH_ANTENNA_PORT,
            "NR5GFrame", frame - 1, RS_REPCAP_BUF_SIZE, repCap_frame));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_ENHANCED_PDSCH_ANTENNA_PORT,
            "BWPart", bandwidthPart , RS_REPCAP_BUF_SIZE, repCap_bwp));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_ENHANCED_PDSCH_ANTENNA_PORT,
            "NR5GSlot", slot , RS_REPCAP_BUF_SIZE, repCap_slot));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_ENHANCED_PDSCH_ANTENNA_PORT,
            "NR5GAlloc", allocation , RS_REPCAP_BUF_SIZE, repCap_alloc));

    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_frame, NULL);
    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_bwp, NULL);
    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_slot, NULL);
    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_alloc, NULL);

    viCheckParm(rsspecan_SetAttributeViString(instrSession, repCap, RSSPECAN_ATTR_5G_NR_DL_ENHANCED_PDSCH_ANTENNA_PORT, antennaPort),
            7, "Antenna Port");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_5G_NR_DL_ENHANCED_PDSCH_CDM_GROUPS_WITHOUT_DATA, cdmGroupsWithoutData),
            8, "CDM Groups Without Data");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_5G_NR_DL_ENHANCED_PDSCH_REFERENCE_POINT, referencePoint),
            9, "Reference Point");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures enhanced PDSCH PTRS.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR cc/Component Carrier parameter.
/// HIPAR frame/Sets the Frame parameter.
/// HIPAR bandwidthPart/Sets the Bandwidth Part parameter.
/// HIPAR slot/Defines the number of slot.
/// HIPAR allocation/Defines the number of PDSCH allocations in a slot.
/// HIPAR enabled/Turns the PTRS on and off.
/// HIPAR timeDomainOffset/Defines an offset for the PTRS in the time domain.
/// HIPAR power/Defines the relative power of the PTRS.
/// HIPAR frequencyDomainOffset/Defines an offset for the PTRS in the frequency domain.
/// HIPAR relativeOffset/Defines the location of the PTRS in the frequency domain relative to
/// HIPAR relativeOffset/the first subcarrier.
ViStatus _VI_FUNC rsspecan_Configure5GNRDLEnhancedPDSCHPTRS(ViSession instrSession,
                                                            ViInt32   cc,
                                                            ViInt32   frame,
                                                            ViInt32   bandwidthPart,
                                                            ViInt32   slot,
                                                            ViInt32   allocation,
                                                            ViBoolean enabled,
                                                            ViInt32   timeDomainOffset,
                                                            ViReal64  power,
                                                            ViInt32   frequencyDomainOffset,
                                                            ViInt32   relativeOffset)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_frame[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_bwp[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_slot[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_alloc[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_ENHANCED_PDSCH_PTRS_ENABLED,
            "cc", cc - 1, RS_REPCAP_BUF_SIZE, repCap));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_ENHANCED_PDSCH_PTRS_ENABLED,
            "NR5GFrame", frame - 1, RS_REPCAP_BUF_SIZE, repCap_frame));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_ENHANCED_PDSCH_PTRS_ENABLED,
            "BWPart", bandwidthPart , RS_REPCAP_BUF_SIZE, repCap_bwp));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_ENHANCED_PDSCH_PTRS_ENABLED,
            "NR5GSlot", slot , RS_REPCAP_BUF_SIZE, repCap_slot));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_ENHANCED_PDSCH_PTRS_ENABLED,
            "NR5GAlloc", allocation , RS_REPCAP_BUF_SIZE, repCap_alloc));

    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_frame, NULL);
    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_bwp, NULL);
    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_slot, NULL);
    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_alloc, NULL);

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, repCap, RSSPECAN_ATTR_5G_NR_DL_ENHANCED_PDSCH_PTRS_ENABLED, enabled),
            7, "Enabled");

    if (enabled == VI_TRUE)
    {
        viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_5G_NR_DL_ENHANCED_PDSCH_PTRS_TIME_DOMAIN_OFFSET, timeDomainOffset),
                8, "Time Domain Offset");

        viCheckParm(rsspecan_SetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_5G_NR_DL_ENHANCED_PDSCH_PTRS_POWER, power),
                9, "Power");

        viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_5G_NR_DL_ENHANCED_PDSCH_PTRS_FREQUENCY_DOMAIN_OFFSET, frequencyDomainOffset),
                10, "Frequency Domain Offset");

        viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_5G_NR_DL_ENHANCED_PDSCH_PTRS_RELATIVE_OFFSET, relativeOffset),
                11, "Relative Offset");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function defines the position of additional DM-RS.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR cc/Component Carrier parameter.
/// HIPAR frame/Sets the Frame parameter.
/// HIPAR bandwidthPart/Sets the Bandwidth Part parameter.
/// HIPAR slot/Defines the number of slot.
/// HIPAR allocation/Defines the number of PDSCH allocations in a slot.
/// HIPAR additional/Defines the position of additional DM-RS.
ViStatus _VI_FUNC rsspecan_Configure5GNRDLEnhancedPDSCHAdditionalDMRS(
                            ViSession instrSession,
                            ViInt32   cc,
                            ViInt32   frame,
                            ViInt32   bandwidthPart,
                            ViInt32   slot,
                            ViInt32   allocation,
                            ViInt32   additional)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_frame[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_bwp[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_slot[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_alloc[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_ENHANCED_PDSCH_ADDITIONAL_DM_RS,
            "cc", cc - 1, RS_REPCAP_BUF_SIZE, repCap));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_ENHANCED_PDSCH_ADDITIONAL_DM_RS,
            "NR5GFrame", frame - 1, RS_REPCAP_BUF_SIZE, repCap_frame));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_ENHANCED_PDSCH_ADDITIONAL_DM_RS,
            "BWPart", bandwidthPart , RS_REPCAP_BUF_SIZE, repCap_bwp));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_ENHANCED_PDSCH_ADDITIONAL_DM_RS,
            "NR5GSlot", slot , RS_REPCAP_BUF_SIZE, repCap_slot));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_ENHANCED_PDSCH_ADDITIONAL_DM_RS,
            "NR5GAlloc", allocation , RS_REPCAP_BUF_SIZE, repCap_alloc));

    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_frame, NULL);
    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_bwp, NULL);
    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_slot, NULL);
    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_alloc, NULL);

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_5G_NR_DL_ENHANCED_PDSCH_ADDITIONAL_DM_RS, additional),
            7, "Additional");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the DM-RS.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR cc/Component Carrier parameter.
/// HIPAR frame/Sets the Frame parameter.
/// HIPAR bandwidthPart/Sets the Bandwidth Part parameter.
/// HIPAR slot/Defines the number of slot.
/// HIPAR allocation/Defines the number of PDSCH allocations in a slot.
/// HIPAR type/Selects the PDSCH DM-RS configuration type.
/// HIPAR length/Defines the length of the DM-RS.
/// HIPAR mapping/Selects the mapping type of the PDSCH DM-RS.
/// HIPAR scramblingID/Defines the scrambling ID for the PDSCH DM-RS sequence generation.
/// HIPAR power/Defines the PDSCH DM-RS power relative to the PDSCH.
/// HIPAR method/Selects the PDSCH DM-RS sequence generation method.
/// HIPAR sid/Defines the seed value for the PDSCH DM-RS sequence generation.
/// HIPAR firstSymbol/Defines the first symbol that the DM-RS uses.
ViStatus _VI_FUNC rsspecan_Configure5GNRDLEnhancedPDSCHDMRS(ViSession instrSession,
                                                            ViInt32   cc,
                                                            ViInt32   frame,
                                                            ViInt32   bandwidthPart,
                                                            ViInt32   slot,
                                                            ViInt32   allocation,
                                                            ViInt32   type,
                                                            ViInt32   length,
                                                            ViInt32   mapping,
                                                            ViInt32   scramblingID,
                                                            ViReal64  power,
                                                            ViInt32   method,
                                                            ViInt32   sid,
                                                            ViInt32   firstSymbol)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_frame[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_bwp[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_slot[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_alloc[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_ENHANCED_PDSCH_ADDITIONAL_DM_RS,
            "cc", cc - 1, RS_REPCAP_BUF_SIZE, repCap));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_ENHANCED_PDSCH_ADDITIONAL_DM_RS,
            "NR5GFrame", frame - 1, RS_REPCAP_BUF_SIZE, repCap_frame));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_ENHANCED_PDSCH_ADDITIONAL_DM_RS,
            "BWPart", bandwidthPart , RS_REPCAP_BUF_SIZE, repCap_bwp));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_ENHANCED_PDSCH_ADDITIONAL_DM_RS,
            "NR5GSlot", slot , RS_REPCAP_BUF_SIZE, repCap_slot));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_ENHANCED_PDSCH_ADDITIONAL_DM_RS,
            "NR5GAlloc", allocation , RS_REPCAP_BUF_SIZE, repCap_alloc));

    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_frame, NULL);
    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_bwp, NULL);
    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_slot, NULL);
    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_alloc, NULL);

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_5G_NR_DL_ENHANCED_PDSCH_ADDITIONAL_DM_RS, type),
            7, "Type");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_5G_NR_DL_ENHANCED_PDSCH_DM_RS_LENGTH, length),
            8, "Length");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_5G_NR_DL_ENHANCED_PDSCH_DM_RS_MAPPING_TYPE, mapping),
            9, "Mapping");

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_5G_NR_DL_ENHANCED_PDSCH_DM_RS_POWER, power),
            11, "Power");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_5G_NR_DL_ENHANCED_PDSCH_DM_RS_SEQUENCE_GENERATION_METHOD, method),
            12, "Method");

    if (method == RSSPECAN_VAL_5G_NR_DMRS_METHOD_NIDDMRS)
    {
        viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_5G_NR_DL_ENHANCED_PDSCH_DM_RS_SCRAMBLING_ID, scramblingID),
                10, "Scrambling ID");

        viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_5G_NR_DL_ENHANCED_PDSCH_DM_RS_SID, sid),
                13, "SID");
    }

    if (mapping == RSSPECAN_VAL_5G_NR_DMRS_MAPPING_A)
    {
        viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_5G_NR_DL_ENHANCED_PDSCH_DM_RS_FIRST_SYMBOL, firstSymbol),
                14, "First Symbol");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Turns low peak to average power ratio for the PDSCH DMRS on and off.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR cc/Component Carrier parameter.
/// HIPAR frame/Sets the Frame parameter.
/// HIPAR bandwidthPart/Sets the Bandwidth Part parameter.
/// HIPAR slot/Defines the number of slot.
/// HIPAR allocation/Defines the number of PDSCH allocations in a slot.
/// HIPAR enabled/Turns low peak to average power ratio for the PDSCH DMRS on and off.
ViStatus _VI_FUNC rsspecan_Configure5GNRDLEnhancedPDSCHDMRSDownlink(
                            ViSession instrSession,
                            ViInt32   cc,
                            ViInt32   frame,
                            ViInt32   bandwidthPart,
                            ViInt32   slot,
                            ViInt32   allocation,
                            ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_frame[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_bwp[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_slot[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_alloc[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_ENHANCED_PDSCH_DMRS_DOWNLINK_R16,
            "cc", cc - 1, RS_REPCAP_BUF_SIZE, repCap));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_ENHANCED_PDSCH_DMRS_DOWNLINK_R16,
            "NR5GFrame", frame - 1, RS_REPCAP_BUF_SIZE, repCap_frame));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_ENHANCED_PDSCH_DMRS_DOWNLINK_R16,
            "BWPart", bandwidthPart , RS_REPCAP_BUF_SIZE, repCap_bwp));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_ENHANCED_PDSCH_DMRS_DOWNLINK_R16,
            "NR5GSlot", slot , RS_REPCAP_BUF_SIZE, repCap_slot));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_ENHANCED_PDSCH_DMRS_DOWNLINK_R16,
            "NR5GAlloc", allocation , RS_REPCAP_BUF_SIZE, repCap_alloc));

    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_frame, NULL);
    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_bwp, NULL);
    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_slot, NULL);
    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_alloc, NULL);

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, repCap, RSSPECAN_ATTR_5G_NR_DL_ENHANCED_PDSCH_DMRS_DOWNLINK_R16, enabled),
            7, "Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the channel coding.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR cc/Component Carrier parameter.
/// HIPAR frame/Sets the Frame parameter.
/// HIPAR bandwidthPart/Sets the Bandwidth Part parameter.
/// HIPAR slot/Defines the number of slot.
/// HIPAR allocation/Defines the number of PDSCH allocations in a slot.
/// HIPAR mcsTable/Selects the modulation order (MCS table) of the PDSCH.
/// HIPAR mcsIndex/Selects the MCS index (IMCS).
/// HIPAR redundancyVersionIndex/Selects the redundancy version index.
ViStatus _VI_FUNC rsspecan_Configure5GNRDLEnhancedPDSCHChannelCoding(
                            ViSession instrSession,
                            ViInt32   cc,
                            ViInt32   frame,
                            ViInt32   bandwidthPart,
                            ViInt32   slot,
                            ViInt32   allocation,
                            ViInt32   mcsTable,
                            ViInt32   mcsIndex,
                            ViInt32   redundancyVersionIndex)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_frame[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_bwp[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_slot[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_alloc[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_ENHANCED_PDSCH_CHANNEL_CODING_MCS_TABLE,
            "cc", cc - 1, RS_REPCAP_BUF_SIZE, repCap));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_ENHANCED_PDSCH_CHANNEL_CODING_MCS_TABLE,
            "NR5GFrame", frame - 1, RS_REPCAP_BUF_SIZE, repCap_frame));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_ENHANCED_PDSCH_CHANNEL_CODING_MCS_TABLE,
            "BWPart", bandwidthPart , RS_REPCAP_BUF_SIZE, repCap_bwp));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_ENHANCED_PDSCH_CHANNEL_CODING_MCS_TABLE,
            "NR5GSlot", slot , RS_REPCAP_BUF_SIZE, repCap_slot));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_ENHANCED_PDSCH_CHANNEL_CODING_MCS_TABLE,
            "NR5GAlloc", allocation , RS_REPCAP_BUF_SIZE, repCap_alloc));

    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_frame, NULL);
    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_bwp, NULL);
    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_slot, NULL);
    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_alloc, NULL);

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_5G_NR_DL_ENHANCED_PDSCH_CHANNEL_CODING_MCS_TABLE, mcsTable),
            7, "MCS Table");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_5G_NR_DL_ENHANCED_PDSCH_CHANNEL_CODING_MCS_INDEX, mcsIndex),
            8, "MCS Index");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_5G_NR_DL_ENHANCED_PDSCH_CHANNEL_CODING_REDUNDANCY_VERSION_INDEX, redundancyVersionIndex),
            9, "Redundancy Version Index");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects the transport block scaling factor S.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR cc/Component Carrier parameter.
/// HIPAR frame/Sets the Frame parameter.
/// HIPAR bandwidthPart/Sets the Bandwidth Part parameter.
/// HIPAR slot/Defines the number of slot.
/// HIPAR allocation/Defines the number of PDSCH allocations in a slot.
/// HIPAR scalingFactor/Selects the transport block scaling factor S.
ViStatus _VI_FUNC rsspecan_Configure5GNRDLEnhancedPDSCHChannelCodingScalingFactor(
                            ViSession instrSession,
                            ViInt32   cc,
                            ViInt32   frame,
                            ViInt32   bandwidthPart,
                            ViInt32   slot,
                            ViInt32   allocation,
                            ViInt32   scalingFactor)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_frame[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_bwp[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_slot[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_alloc[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_ENHANCED_PDSCH_CHANNEL_CODING_SCALING_FACTOR,
            "cc", cc - 1, RS_REPCAP_BUF_SIZE, repCap));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_ENHANCED_PDSCH_CHANNEL_CODING_SCALING_FACTOR,
            "NR5GFrame", frame - 1, RS_REPCAP_BUF_SIZE, repCap_frame));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_ENHANCED_PDSCH_CHANNEL_CODING_SCALING_FACTOR,
            "BWPart", bandwidthPart , RS_REPCAP_BUF_SIZE, repCap_bwp));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_ENHANCED_PDSCH_CHANNEL_CODING_SCALING_FACTOR,
            "NR5GSlot", slot , RS_REPCAP_BUF_SIZE, repCap_slot));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_ENHANCED_PDSCH_CHANNEL_CODING_SCALING_FACTOR,
            "NR5GAlloc", allocation , RS_REPCAP_BUF_SIZE, repCap_alloc));

    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_frame, NULL);
    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_bwp, NULL);
    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_slot, NULL);
    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_alloc, NULL);

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_5G_NR_DL_ENHANCED_PDSCH_CHANNEL_CODING_SCALING_FACTOR, scalingFactor),
            7, "Scaling Factor");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the scrambling.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR cc/Component Carrier parameter.
/// HIPAR frame/Sets the Frame parameter.
/// HIPAR bandwidthPart/Sets the Bandwidth Part parameter.
/// HIPAR slot/Defines the number of slot.
/// HIPAR allocation/Defines the number of PDSCH allocations in a slot.
/// HIPAR scramblingMethod/Selects the PDSCH scrambling method.
/// HIPAR seedValue/Defines the seed value for the PDSCH scrambling.
ViStatus _VI_FUNC rsspecan_Configure5GNRDLEnhancedPDSCHScrambling(
                            ViSession instrSession,
                            ViInt32   cc,
                            ViInt32   frame,
                            ViInt32   bandwidthPart,
                            ViInt32   slot,
                            ViInt32   allocation,
                            ViInt32   scramblingMethod,
                            ViInt32   seedValue)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_frame[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_bwp[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_slot[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_alloc[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_ENHANCED_PDSCH_SCRAMBLING,
            "cc", cc - 1, RS_REPCAP_BUF_SIZE, repCap));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_ENHANCED_PDSCH_SCRAMBLING,
            "NR5GFrame", frame - 1, RS_REPCAP_BUF_SIZE, repCap_frame));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_ENHANCED_PDSCH_SCRAMBLING,
            "BWPart", bandwidthPart , RS_REPCAP_BUF_SIZE, repCap_bwp));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_ENHANCED_PDSCH_SCRAMBLING,
            "NR5GSlot", slot , RS_REPCAP_BUF_SIZE, repCap_slot));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_ENHANCED_PDSCH_SCRAMBLING,
            "NR5GAlloc", allocation , RS_REPCAP_BUF_SIZE, repCap_alloc));

    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_frame, NULL);
    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_bwp, NULL);
    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_slot, NULL);
    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_alloc, NULL);

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_5G_NR_DL_ENHANCED_PDSCH_SCRAMBLING, scramblingMethod),
            7, "Scrambling Method");

    if (scramblingMethod == RSSPECAN_VAL_5G_NR_SCRAMBLING_DS_ID)
    {
        viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_5G_NR_DL_ENHANCED_PDSCH_SCRAMBLING_ID, seedValue),
                8, "Seed Value");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function defines the number of CORESETs in a slot.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR cc/Component Carrier parameter.
/// HIPAR frame/Sets the Frame parameter.
/// HIPAR bandwidthPart/Sets the Bandwidth Part parameter.
/// HIPAR slot/Defines the number of slot.
/// HIPAR coresets/Defines the number of CORESETs in a slot.
ViStatus _VI_FUNC rsspecan_Configure5GNRDLCORESETNumbers(ViSession instrSession,
                                                         ViInt32   cc,
                                                         ViInt32   frame,
                                                         ViInt32   bandwidthPart,
                                                         ViInt32   slot,
                                                         ViInt32   coresets)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_frame[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_bwp[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_slot[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_CORESET_NUMBERS,
            "cc", cc - 1, RS_REPCAP_BUF_SIZE, repCap));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_CORESET_NUMBERS,
            "NR5GFrame", frame - 1, RS_REPCAP_BUF_SIZE, repCap_frame));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_CORESET_NUMBERS,
            "BWPart", bandwidthPart , RS_REPCAP_BUF_SIZE, repCap_bwp));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_CORESET_NUMBERS,
            "NR5GSlot", slot , RS_REPCAP_BUF_SIZE, repCap_slot));

    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_frame, NULL);
    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_bwp, NULL);
    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_slot, NULL);

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_5G_NR_DL_CORESET_NUMBERS, coresets),
            7, "Coresets");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects the precoder granularity for the PDCCH DMRS.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR cc/Component Carrier parameter.
/// HIPAR frame/Sets the Frame parameter.
/// HIPAR bandwidthPart/Sets the Bandwidth Part parameter.
/// HIPAR slot/Defines the number of slot.
/// HIPAR coreset/Sets the CORESET number.
/// HIPAR precoderGranularity/Selects the precoder granularity for the PDCCH DMRS.
ViStatus _VI_FUNC rsspecan_Configure5GNRDLCORESETPrecoderGranularity(
                            ViSession instrSession,
                            ViInt32   cc,
                            ViInt32   frame,
                            ViInt32   bandwidthPart,
                            ViInt32   slot,
                            ViInt32   coreset,
                            ViInt32   precoderGranularity)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_frame[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_bwp[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_slot[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_core[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_CORESET_PRECODER_GRANULARITY,
            "cc", cc - 1, RS_REPCAP_BUF_SIZE, repCap));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_CORESET_PRECODER_GRANULARITY,
            "NR5GFrame", frame - 1, RS_REPCAP_BUF_SIZE, repCap_frame));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_CORESET_PRECODER_GRANULARITY,
            "BWPart", bandwidthPart , RS_REPCAP_BUF_SIZE, repCap_bwp));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_CORESET_PRECODER_GRANULARITY,
            "NR5GSlot", slot , RS_REPCAP_BUF_SIZE, repCap_slot));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_CORESET_PRECODER_GRANULARITY,
            "NR5GCORESET", coreset , RS_REPCAP_BUF_SIZE, repCap_core));

    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_frame, NULL);
    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_bwp, NULL);
    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_slot, NULL);
    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_core, NULL);

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_5G_NR_DL_CORESET_PRECODER_GRANULARITY, precoderGranularity),
            7, "Precoder Granularity");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function defines the relative power of a CORESET.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR cc/Component Carrier parameter.
/// HIPAR frame/Sets the Frame parameter.
/// HIPAR bandwidthPart/Sets the Bandwidth Part parameter.
/// HIPAR slot/Defines the number of slot.
/// HIPAR coreset/Sets the CORESET number.
/// HIPAR power/Defines the relative power of a CORESET.
ViStatus _VI_FUNC rsspecan_Configure5GNRDLCORESETPower(ViSession instrSession,
                                                       ViInt32   bandwidthPart,
                                                       ViInt32   slot,
                                                       ViInt32   coreset,
                                                       ViReal64  power)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_slot[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_core[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_CORESET_POWER,
            "BWPart", bandwidthPart , RS_REPCAP_BUF_SIZE, repCap));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_CORESET_POWER,
            "NR5GSlot", slot , RS_REPCAP_BUF_SIZE, repCap_slot));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_CORESET_POWER,
            "NR5GCORESET", coreset , RS_REPCAP_BUF_SIZE, repCap_core));

    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_slot, NULL);
    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_core, NULL);

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_5G_NR_DL_CORESET_POWER, power),
            7, "Power");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the CORESET DM-RS.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR cc/Component Carrier parameter.
/// HIPAR frame/Sets the Frame parameter.
/// HIPAR bandwidthPart/Sets the Bandwidth Part parameter.
/// HIPAR slot/Defines the number of slot.
/// HIPAR coreset/Sets the CORESET number.
/// HIPAR power/Defines the power of a CORESET DM-RS relative to the CORESET.
/// HIPAR sequenceGeneration/Selects the CORESET DM-RS sequence generation method.
/// HIPAR sid/Defines the seed value for the CORESET DM-RS sequence generation.
ViStatus _VI_FUNC rsspecan_Configure5GNRDLCORESETDMRS(ViSession instrSession,
                                                      ViInt32   cc,
                                                      ViInt32   frame,
                                                      ViInt32   bandwidthPart,
                                                      ViInt32   slot,
                                                      ViInt32   coreset,
                                                      ViReal64  power,
                                                      ViInt32   sequenceGeneration,
                                                      ViInt32   sid)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_frame[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_bwp[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_slot[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_core[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_CORESET_DM_RS_POWER,
            "cc", cc - 1, RS_REPCAP_BUF_SIZE, repCap));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_CORESET_DM_RS_POWER,
            "NR5GFrame", frame - 1, RS_REPCAP_BUF_SIZE, repCap_frame));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_CORESET_DM_RS_POWER,
            "BWPart", bandwidthPart , RS_REPCAP_BUF_SIZE, repCap_bwp));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_CORESET_DM_RS_POWER,
            "NR5GSlot", slot , RS_REPCAP_BUF_SIZE, repCap_slot));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_CORESET_DM_RS_POWER,
            "NR5GCORESET", coreset , RS_REPCAP_BUF_SIZE, repCap_core));

    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_frame, NULL);
    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_bwp, NULL);
    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_slot, NULL);
    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_core, NULL);

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_5G_NR_DL_CORESET_DM_RS_POWER, power),
            7, "Power");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_5G_NR_DL_CORESET_DM_RS_SEQUENCE_GENERATION, sequenceGeneration),
            8, "Sequence Generation");

    if (sequenceGeneration == RSSPECAN_VAL_5G_NR_DMRS_METHOD_DSID)
    {
        viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_5G_NR_DL_CORESET_DM_RS_SID, sid),
                9, "SID");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the reference point of the CORESET DMRS.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR cc/Component Carrier parameter.
/// HIPAR frame/Sets the Frame parameter.
/// HIPAR bandwidthPart/Sets the Bandwidth Part parameter.
/// HIPAR slot/Defines the number of slot.
/// HIPAR coreset/Sets the CORESET number.
/// HIPAR referencePoint/Defines the reference point of the CORESET DMRS.
ViStatus _VI_FUNC rsspecan_Configure5GNRDLCORESETDMRSReferencePoint(
                            ViSession instrSession,
                            ViInt32   cc,
                            ViInt32   frame,
                            ViInt32   bandwidthPart,
                            ViInt32   slot,
                            ViInt32   coreset,
                            ViInt32   referencePoint)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_frame[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_bwp[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_slot[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_core[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_CORESET_DM_RS_POWER,
            "cc", cc - 1, RS_REPCAP_BUF_SIZE, repCap));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_CORESET_DM_RS_POWER,
            "NR5GFrame", frame - 1, RS_REPCAP_BUF_SIZE, repCap_frame));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_CORESET_DM_RS_POWER,
            "BWPart", bandwidthPart , RS_REPCAP_BUF_SIZE, repCap_bwp));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_CORESET_DM_RS_POWER,
            "NR5GSlot", slot , RS_REPCAP_BUF_SIZE, repCap_slot));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_CORESET_DM_RS_POWER,
            "NR5GCORESET", coreset , RS_REPCAP_BUF_SIZE, repCap_core));

    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_frame, NULL);
    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_bwp, NULL);
    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_slot, NULL);
    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_core, NULL);

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_5G_NR_DL_CORESET_DM_RS_REFERENCE_POINT, referencePoint),
            7, "Reference Point");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the CORESET Resource Blocks.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR cc/Component Carrier parameter.
/// HIPAR frame/Sets the Frame parameter.
/// HIPAR bandwidthPart/Sets the Bandwidth Part parameter.
/// HIPAR slot/Defines the number of slot.
/// HIPAR coreset/Sets the CORESET number.
/// HIPAR count/Selects the number of resource blocks that a CORESET allocation
/// HIPAR count/occupies.
/// HIPAR offset/Defines the resource block offset of a CORESET allocation.
ViStatus _VI_FUNC rsspecan_Configure5GNRDLCORESETResourceBlocks(
                            ViSession instrSession,
                            ViInt32   cc,
                            ViInt32   frame,
                            ViInt32   bandwidthPart,
                            ViInt32   slot,
                            ViInt32   coreset,
                            ViInt32   count,
                            ViInt32   offset)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_frame[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_bwp[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_slot[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_core[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_CORESET_RESOURCE_BLOCKS,
            "cc", cc - 1, RS_REPCAP_BUF_SIZE, repCap));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_CORESET_RESOURCE_BLOCKS,
            "NR5GFrame", frame - 1, RS_REPCAP_BUF_SIZE, repCap_frame));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_CORESET_RESOURCE_BLOCKS,
            "BWPart", bandwidthPart , RS_REPCAP_BUF_SIZE, repCap_bwp));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_CORESET_RESOURCE_BLOCKS,
            "NR5GSlot", slot , RS_REPCAP_BUF_SIZE, repCap_slot));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_CORESET_RESOURCE_BLOCKS,
            "NR5GCORESET", coreset , RS_REPCAP_BUF_SIZE, repCap_core));

    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_frame, NULL);
    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_bwp, NULL);
    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_slot, NULL);
    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_core, NULL);

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_5G_NR_DL_CORESET_RESOURCE_BLOCKS, count),
            7, "Count");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_5G_NR_DL_CORESET_RESOURCE_BLOCKS_OFFSET, offset),
            8, "Offset");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function configures the CORESET Symbol.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR cc/Component Carrier parameter.
/// HIPAR frame/Sets the Frame parameter.
/// HIPAR bandwidthPart/Sets the Bandwidth Part parameter.
/// HIPAR slot/Defines the number of slot.
/// HIPAR coreset/Sets the CORESET number.
/// HIPAR count/Selects the number of symbols that a CORESET allocation occupies.
/// HIPAR offset/Defines the symbol offset of a CORESET allocation.
ViStatus _VI_FUNC rsspecan_Configure5GNRDLCORESETSymbol(ViSession instrSession,
                                                        ViInt32   cc,
                                                        ViInt32   frame,
                                                        ViInt32   bandwidthPart,
                                                        ViInt32   slot,
                                                        ViInt32   coreset,
                                                        ViInt32   count,
                                                        ViInt32   offset)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_frame[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_bwp[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_slot[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_core[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_CORESET_SYMBOL_COUNT,
            "cc", cc - 1, RS_REPCAP_BUF_SIZE, repCap));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_CORESET_SYMBOL_COUNT,
            "NR5GFrame", frame - 1, RS_REPCAP_BUF_SIZE, repCap_frame));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_CORESET_SYMBOL_COUNT,
            "BWPart", bandwidthPart , RS_REPCAP_BUF_SIZE, repCap_bwp));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_CORESET_SYMBOL_COUNT,
            "NR5GSlot", slot , RS_REPCAP_BUF_SIZE, repCap_slot));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_CORESET_SYMBOL_COUNT,
            "NR5GCORESET", coreset , RS_REPCAP_BUF_SIZE, repCap_core));

    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_frame, NULL);
    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_bwp, NULL);
    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_slot, NULL);
    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_core, NULL);

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_5G_NR_DL_CORESET_SYMBOL_COUNT, count),
            7, "Count");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_5G_NR_DL_CORESET_SYMBOL_OFFSET, offset),
            8, "Offset");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the CORESET Interleaving.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR cc/Component Carrier parameter.
/// HIPAR frame/Sets the Frame parameter.
/// HIPAR bandwidthPart/Sets the Bandwidth Part parameter.
/// HIPAR slot/Defines the number of slot.
/// HIPAR coreset/Sets the CORESET number.
/// HIPAR enabled/Turns interleaving of the PDCCH on and off.
/// HIPAR shiftIndex/Defines the shift index of the PDCCH.
/// HIPAR shiftIndexValue/Defines the shift index of the PDCCH.
/// HIPAR bundleSize/Selects the REG bundle size of the PDCCH.
/// HIPAR interleaverSize/Selects the interleaving size of the PDCCH.
ViStatus _VI_FUNC rsspecan_Configure5GNRDLCORESETInterleaving(
                            ViSession instrSession,
                            ViInt32   cc,
                            ViInt32   frame,
                            ViInt32   bandwidthPart,
                            ViInt32   slot,
                            ViInt32   coreset,
                            ViBoolean enabled,
                            ViInt32   shiftIndex,
                            ViInt32   shiftIndexValue,
                            ViInt32   bundleSize,
                            ViInt32   interleaverSize)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_frame[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_bwp[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_slot[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_core[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_CORESET_INTERLEAVING_ENABLED,
            "cc", cc - 1, RS_REPCAP_BUF_SIZE, repCap));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_CORESET_INTERLEAVING_ENABLED,
            "NR5GFrame", frame - 1, RS_REPCAP_BUF_SIZE, repCap_frame));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_CORESET_INTERLEAVING_ENABLED,
            "BWPart", bandwidthPart , RS_REPCAP_BUF_SIZE, repCap_bwp));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_CORESET_INTERLEAVING_ENABLED,
            "NR5GSlot", slot , RS_REPCAP_BUF_SIZE, repCap_slot));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_CORESET_INTERLEAVING_ENABLED,
            "NR5GCORESET", coreset , RS_REPCAP_BUF_SIZE, repCap_core));

    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_frame, NULL);
    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_bwp, NULL);
    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_slot, NULL);
    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_core, NULL);

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, repCap, RSSPECAN_ATTR_5G_NR_DL_CORESET_INTERLEAVING_ENABLED, enabled),
            7, "Enabled");

    if (enabled == VI_TRUE)
    {
        shiftIndex;
//        viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap,
//            RSSPECAN_ATTR_5G_NR_DL_CORESET_INTERLEAVING_SHIFT_INDEX, shiftIndex), 8, "Shift Index");

        viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_5G_NR_DL_CORESET_INTERLEAVING_SHIFT_INDEX_VALUE, shiftIndexValue),
                9, "Shift Index Value");

        viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_5G_NR_DL_CORESET_INTERLEAVING_BUNDLE_SIZE, bundleSize),
                10, "Bundle Size");

        viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_5G_NR_DL_CORESET_INTERLEAVING_SIZE, interleaverSize),
                11, "Interleaving Size");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function selects the RNTI type for a PDCCH.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR cc/Component Carrier parameter.
/// HIPAR frame/Sets the Frame parameter.
/// HIPAR bandwidthPart/Sets the Bandwidth Part parameter.
/// HIPAR slot/Defines the number of slot.
/// HIPAR coreset/Sets the CORESET number.
/// HIPAR usage/Selects the RNTI type for a PDCCH.
ViStatus _VI_FUNC rsspecan_Configure5GNRDLCORESETPDCCHConfigUsage(
                            ViSession instrSession,
                            ViInt32   cc,
                            ViInt32   frame,
                            ViInt32   bandwidthPart,
                            ViInt32   slot,
                            ViInt32   coreset,
                            ViInt32   usage)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_frame[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_bwp[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_slot[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_core[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_CORESET_PDCCH_CONFIG_USAGE,
            "cc", cc - 1, RS_REPCAP_BUF_SIZE, repCap));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_CORESET_PDCCH_CONFIG_USAGE,
            "NR5GFrame", frame - 1, RS_REPCAP_BUF_SIZE, repCap_frame));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_CORESET_PDCCH_CONFIG_USAGE,
            "BWPart", bandwidthPart , RS_REPCAP_BUF_SIZE, repCap_bwp));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_CORESET_PDCCH_CONFIG_USAGE,
            "NR5GSlot", slot , RS_REPCAP_BUF_SIZE, repCap_slot));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_CORESET_PDCCH_CONFIG_USAGE,
            "NR5GCORESET", coreset , RS_REPCAP_BUF_SIZE, repCap_core));

    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_frame, NULL);
    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_bwp, NULL);
    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_slot, NULL);
    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_core, NULL);

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_5G_NR_DL_CORESET_PDCCH_CONFIG_USAGE, usage),
            7, "Usage");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the CORESET PDCCH.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR cc/Component Carrier parameter.
/// HIPAR frame/Sets the Frame parameter.
/// HIPAR bandwidthPart/Sets the Bandwidth Part parameter.
/// HIPAR slot/Defines the number of slot.
/// HIPAR coreset/Sets the CORESET number.
/// HIPAR rnti/Selects the CORESET radio network identifier (RNTI).
/// HIPAR aggregationLevel/Selects the aggregation level of the PDCCH.
/// HIPAR cceIndex/Select the CCE index for the PDCCH.
/// HIPAR patternLength/Defines the bit length of the PDCCH.
ViStatus _VI_FUNC rsspecan_Configure5GNRDLCORESETPDCCHConfig(ViSession instrSession,
                                                             ViInt32   cc,
                                                             ViInt32   frame,
                                                             ViInt32   bandwidthPart,
                                                             ViInt32   slot,
                                                             ViInt32   coreset,
                                                             ViInt32   rnti,
                                                             ViInt32   aggregationLevel,
                                                             ViInt32   cceIndex,
                                                             ViInt32   patternLength)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_frame[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_bwp[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_slot[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_core[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_CORESET_PDCCH_CONFIG_AGGREGATION_LEVEL,
            "cc", cc - 1, RS_REPCAP_BUF_SIZE, repCap));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_CORESET_PDCCH_CONFIG_AGGREGATION_LEVEL,
            "NR5GFrame", frame - 1, RS_REPCAP_BUF_SIZE, repCap_frame));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_CORESET_PDCCH_CONFIG_AGGREGATION_LEVEL,
            "BWPart", bandwidthPart , RS_REPCAP_BUF_SIZE, repCap_bwp));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_CORESET_PDCCH_CONFIG_AGGREGATION_LEVEL,
            "NR5GSlot", slot , RS_REPCAP_BUF_SIZE, repCap_slot));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_CORESET_PDCCH_CONFIG_AGGREGATION_LEVEL,
            "NR5GCORESET", coreset , RS_REPCAP_BUF_SIZE, repCap_core));

    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_frame, NULL);
    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_bwp, NULL);
    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_slot, NULL);
    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_core, NULL);

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_5G_NR_DL_CORESET_PDCCH_CONFIG_RNTI, rnti),
            7, "RNTI");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_5G_NR_DL_CORESET_PDCCH_CONFIG_AGGREGATION_LEVEL, aggregationLevel),
            8, "Aggregation Level");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_5G_NR_DL_CORESET_PDCCH_CONFIG_CCE_INDEX, cceIndex),
            9, "CCE Index");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_5G_NR_DL_CORESET_PDCCH_CONFIG_PATTERN_LENGTH, patternLength),
            10, "Pattern Length");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function selects the DCI format for a PDCCH.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR cc/Component Carrier parameter.
/// HIPAR frame/Sets the Frame parameter.
/// HIPAR bandwidthPart/Sets the Bandwidth Part parameter.
/// HIPAR slot/Defines the number of slot.
/// HIPAR coreset/Sets the CORESET number.
/// HIPAR DCIFormat/Selects the DCI format for a PDCCH.
ViStatus _VI_FUNC rsspecan_Configure5GNRDLCORESETPDCCHConfigDCIFormat(
                            ViSession instrSession,
                            ViInt32   cc,
                            ViInt32   frame,
                            ViInt32   bandwidthPart,
                            ViInt32   slot,
                            ViInt32   coreset,
                            ViInt32   DCIFormat)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_frame[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_bwp[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_slot[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_core[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_CORESET_PDCCH_CONFIG_DCI_FORMAT,
            "cc", cc - 1, RS_REPCAP_BUF_SIZE, repCap));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_CORESET_PDCCH_CONFIG_DCI_FORMAT,
            "NR5GFrame", frame - 1, RS_REPCAP_BUF_SIZE, repCap_frame));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_CORESET_PDCCH_CONFIG_DCI_FORMAT,
            "BWPart", bandwidthPart , RS_REPCAP_BUF_SIZE, repCap_bwp));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_CORESET_PDCCH_CONFIG_DCI_FORMAT,
            "NR5GSlot", slot , RS_REPCAP_BUF_SIZE, repCap_slot));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_CORESET_PDCCH_CONFIG_DCI_FORMAT,
            "NR5GCORESET", coreset , RS_REPCAP_BUF_SIZE, repCap_core));

    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_frame, NULL);
    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_bwp, NULL);
    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_slot, NULL);
    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_core, NULL);

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_5G_NR_DL_CORESET_PDCCH_CONFIG_DCI_FORMAT, DCIFormat),
            7, "DCI Format");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function defines the frequency domain resource assignment of the
/// HIFN  DCI fields.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR cc/Component Carrier parameter.
/// HIPAR frame/Sets the Frame parameter.
/// HIPAR bandwidthPart/Sets the Bandwidth Part parameter.
/// HIPAR slot/Defines the number of slot.
/// HIPAR coreset/Sets the CORESET number.
/// HIPAR frequencyResourceAssignment/Defines the frequency domain resource assignment of the DCI fields.
ViStatus _VI_FUNC rsspecan_Configure5GNRDLCORESETPDCCHConfigFrequencyResourceAssignment(
                            ViSession instrSession,
                            ViInt32   cc,
                            ViInt32   frame,
                            ViInt32   bandwidthPart,
                            ViInt32   slot,
                            ViInt32   coreset,
                            ViInt32   frequencyResourceAssignment)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_frame[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_bwp[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_slot[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_core[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_CORESET_PDCCH_CONFIG_FREQUENCY_RESOURCE_ASSIGNMENT,
            "cc", cc - 1, RS_REPCAP_BUF_SIZE, repCap));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_CORESET_PDCCH_CONFIG_FREQUENCY_RESOURCE_ASSIGNMENT,
            "NR5GFrame", frame - 1, RS_REPCAP_BUF_SIZE, repCap_frame));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_CORESET_PDCCH_CONFIG_FREQUENCY_RESOURCE_ASSIGNMENT,
            "BWPart", bandwidthPart , RS_REPCAP_BUF_SIZE, repCap_bwp));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_CORESET_PDCCH_CONFIG_FREQUENCY_RESOURCE_ASSIGNMENT,
            "NR5GSlot", slot , RS_REPCAP_BUF_SIZE, repCap_slot));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_CORESET_PDCCH_CONFIG_FREQUENCY_RESOURCE_ASSIGNMENT,
            "NR5GCORESET", coreset , RS_REPCAP_BUF_SIZE, repCap_core));

    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_frame, NULL);
    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_bwp, NULL);
    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_slot, NULL);
    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_core, NULL);

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_5G_NR_DL_CORESET_PDCCH_CONFIG_FREQUENCY_RESOURCE_ASSIGNMENT, frequencyResourceAssignment),
            7, "Frequency Resource Assignment");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function defines the bit length for DCI fields.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR cc/Component Carrier parameter.
/// HIPAR frame/Sets the Frame parameter.
/// HIPAR bandwidthPart/Sets the Bandwidth Part parameter.
/// HIPAR slot/Defines the number of slot.
/// HIPAR coreset/Sets the CORESET number.
/// HIPAR dciField/DCI field.
/// HIPAR bitLength/Bit length.
ViStatus _VI_FUNC rsspecan_Configure5GNRDLCORESETPDCCHConfigDCIFields(
                            ViSession instrSession,
                            ViInt32   cc,
                            ViInt32   frame,
                            ViInt32   bandwidthPart,
                            ViInt32   slot,
                            ViInt32   coreset,
                            ViInt32   dciField,
                            ViString  bitLength)
{
    ViStatus error = VI_SUCCESS;
    ViChar*  p2rngString;
    ViChar   param[RS_PARAM_BUF_SIZE] = "";
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_frame[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_bwp[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_slot[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_core[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_HIDDEN_5G_NR_DL_CORESET_PDCCH_CONFIG_DCI_FIELDS,
            "cc", cc - 1, RS_REPCAP_BUF_SIZE, repCap));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_HIDDEN_5G_NR_DL_CORESET_PDCCH_CONFIG_DCI_FIELDS,
            "NR5GFrame", frame - 1, RS_REPCAP_BUF_SIZE, repCap_frame));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_HIDDEN_5G_NR_DL_CORESET_PDCCH_CONFIG_DCI_FIELDS,
            "BWPart", bandwidthPart , RS_REPCAP_BUF_SIZE, repCap_bwp));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_HIDDEN_5G_NR_DL_CORESET_PDCCH_CONFIG_DCI_FIELDS,
            "NR5GSlot", slot , RS_REPCAP_BUF_SIZE, repCap_slot));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_HIDDEN_5G_NR_DL_CORESET_PDCCH_CONFIG_DCI_FIELDS,
            "NR5GCORESET", coreset , RS_REPCAP_BUF_SIZE, repCap_core));

    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_frame, NULL);
    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_bwp, NULL);
    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_slot, NULL);
    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_core, NULL);

    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &RsSpecAn_rngDCIFields, dciField, &p2rngString),
        7, "DCI Field");

    snprintf (param, RS_PARAM_BUF_SIZE, "%s,%s", p2rngString, bitLength);

    checkErr(rsspecan_SetAttributeViString(instrSession, repCap, RSSPECAN_ATTR_HIDDEN_5G_NR_DL_CORESET_PDCCH_CONFIG_DCI_FIELDS, param));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function queries the DCI fields available for the currently
/// HIFN  selected DCI format and the number of bits each DCI field uses.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR cc/Component Carrier parameter.
/// HIPAR frame/Sets the Frame parameter.
/// HIPAR bandwidthPart/Sets the Bandwidth Part parameter.
/// HIPAR slot/Defines the number of slot.
/// HIPAR coreset/Sets the CORESET number.
/// HIPAR bufferSize/Defines size of array 'DCI Available Fields'
/// HIPAR dciAvailableFields/String that contains the names of the DCI fields and the number of
/// HIPAR dciAvailableFields/bits they use as a comma separated list.
ViStatus _VI_FUNC rsspecan_Query5GNRDLCORESETPDCCHConfigDCIFields(
                            ViSession instrSession,
                            ViInt32   cc,
                            ViInt32   frame,
                            ViInt32   bandwidthPart,
                            ViInt32   slot,
                            ViInt32   coreset,
                            ViInt32   bufferSize,
                            ViChar    dciAvailableFields[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_frame[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_bwp[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_slot[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_core[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_CORESET_PDCCH_CONFIG_DCI_AVAILABLE_FIELDS,
            "cc", cc - 1, RS_REPCAP_BUF_SIZE, repCap));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_CORESET_PDCCH_CONFIG_DCI_AVAILABLE_FIELDS,
            "NR5GFrame", frame - 1, RS_REPCAP_BUF_SIZE, repCap_frame));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_CORESET_PDCCH_CONFIG_DCI_AVAILABLE_FIELDS,
            "BWPart", bandwidthPart , RS_REPCAP_BUF_SIZE, repCap_bwp));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_CORESET_PDCCH_CONFIG_DCI_AVAILABLE_FIELDS,
            "NR5GSlot", slot , RS_REPCAP_BUF_SIZE, repCap_slot));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_CORESET_PDCCH_CONFIG_DCI_AVAILABLE_FIELDS,
            "NR5GCORESET", coreset , RS_REPCAP_BUF_SIZE, repCap_core));

    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_frame, NULL);
    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_bwp, NULL);
    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_slot, NULL);
    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_core, NULL);

    viCheckParm(rsspecan_GetAttributeViString(instrSession, repCap, RSSPECAN_ATTR_5G_NR_DL_CORESET_PDCCH_CONFIG_DCI_AVAILABLE_FIELDS,
            bufferSize, dciAvailableFields),
            8, "DCI Available Fields");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function defines the number of blocks that DCI is transmitted in.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR cc/Component Carrier parameter.
/// HIPAR frame/Sets the Frame parameter.
/// HIPAR bandwidthPart/Sets the Bandwidth Part parameter.
/// HIPAR slot/Defines the number of slot.
/// HIPAR coreset/Sets the CORESET number.
/// HIPAR numberOfBlocks/Defines the number of blocks that DCI is transmitted in.
ViStatus _VI_FUNC rsspecan_Configure5GNRDLCORESETPDCCHConfigDCIBlocks(
                            ViSession instrSession,
                            ViInt32   cc,
                            ViInt32   frame,
                            ViInt32   bandwidthPart,
                            ViInt32   slot,
                            ViInt32   coreset,
                            ViInt32   numberOfBlocks)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_frame[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_bwp[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_slot[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_core[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_CORESET_PDCCH_CONFIG_DCI_NUMBER_OF_BLOCKS,
            "cc", cc - 1, RS_REPCAP_BUF_SIZE, repCap));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_CORESET_PDCCH_CONFIG_DCI_NUMBER_OF_BLOCKS,
            "NR5GFrame", frame - 1, RS_REPCAP_BUF_SIZE, repCap_frame));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_CORESET_PDCCH_CONFIG_DCI_NUMBER_OF_BLOCKS,
            "BWPart", bandwidthPart , RS_REPCAP_BUF_SIZE, repCap_bwp));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_CORESET_PDCCH_CONFIG_DCI_NUMBER_OF_BLOCKS,
            "NR5GSlot", slot , RS_REPCAP_BUF_SIZE, repCap_slot));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_CORESET_PDCCH_CONFIG_DCI_NUMBER_OF_BLOCKS,
            "NR5GCORESET", coreset , RS_REPCAP_BUF_SIZE, repCap_core));

    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_frame, NULL);
    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_bwp, NULL);
    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_slot, NULL);
    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_core, NULL);

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_5G_NR_DL_CORESET_PDCCH_CONFIG_DCI_NUMBER_OF_BLOCKS, numberOfBlocks),
            7, "Number of Blocks");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function defines the scope of the DCI fields.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR cc/Component Carrier parameter.
/// HIPAR frame/Sets the Frame parameter.
/// HIPAR bandwidthPart/Sets the Bandwidth Part parameter.
/// HIPAR slot/Defines the number of slot.
/// HIPAR coreset/Sets the CORESET number.
/// HIPAR scope/Defines the scope of the DCI fields.
ViStatus _VI_FUNC rsspecan_Configure5GNRDLCORESETPDCCHConfigDCIScope(
                            ViSession instrSession,
                            ViInt32   cc,
                            ViInt32   frame,
                            ViInt32   bandwidthPart,
                            ViInt32   slot,
                            ViInt32   coreset,
                            ViInt32   scope)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_frame[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_bwp[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_slot[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_core[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_CORESET_PDCCH_CONFIG_DCI_SCOPE,
            "cc", cc - 1, RS_REPCAP_BUF_SIZE, repCap));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_CORESET_PDCCH_CONFIG_DCI_SCOPE,
            "NR5GFrame", frame - 1, RS_REPCAP_BUF_SIZE, repCap_frame));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_CORESET_PDCCH_CONFIG_DCI_SCOPE,
            "BWPart", bandwidthPart , RS_REPCAP_BUF_SIZE, repCap_bwp));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_CORESET_PDCCH_CONFIG_DCI_SCOPE,
            "NR5GSlot", slot , RS_REPCAP_BUF_SIZE, repCap_slot));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_CORESET_PDCCH_CONFIG_DCI_SCOPE,
            "NR5GCORESET", coreset , RS_REPCAP_BUF_SIZE, repCap_core));

    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_frame, NULL);
    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_bwp, NULL);
    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_slot, NULL);
    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_core, NULL);

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_5G_NR_DL_CORESET_PDCCH_CONFIG_DCI_SCOPE, scope),
            7, "Scope");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function defines the number of TCP commands that are transmitted
/// HIFN  in the DCI.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR cc/Component Carrier parameter.
/// HIPAR frame/Sets the Frame parameter.
/// HIPAR bandwidthPart/Sets the Bandwidth Part parameter.
/// HIPAR slot/Defines the number of slot.
/// HIPAR coreset/Sets the CORESET number.
/// HIPAR tcpCommand/Defines the number of TCP commands that are transmitted in the DCI.
ViStatus _VI_FUNC rsspecan_Configure5GNRDLCORESETPDCCHConfigDCITCPCommand(
                            ViSession instrSession,
                            ViInt32   cc,
                            ViInt32   frame,
                            ViInt32   bandwidthPart,
                            ViInt32   slot,
                            ViInt32   coreset,
                            ViInt32   tcpCommand)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_frame[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_bwp[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_slot[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_core[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_CORESET_PDCCH_CONFIG_DCI_TCP_COMMAND,
            "cc", cc - 1, RS_REPCAP_BUF_SIZE, repCap));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_CORESET_PDCCH_CONFIG_DCI_TCP_COMMAND,
            "NR5GFrame", frame - 1, RS_REPCAP_BUF_SIZE, repCap_frame));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_CORESET_PDCCH_CONFIG_DCI_TCP_COMMAND,
            "BWPart", bandwidthPart , RS_REPCAP_BUF_SIZE, repCap_bwp));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_CORESET_PDCCH_CONFIG_DCI_TCP_COMMAND,
            "NR5GSlot", slot , RS_REPCAP_BUF_SIZE, repCap_slot));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_CORESET_PDCCH_CONFIG_DCI_TCP_COMMAND,
            "NR5GCORESET", coreset , RS_REPCAP_BUF_SIZE, repCap_core));

    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_frame, NULL);
    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_bwp, NULL);
    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_slot, NULL);
    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_core, NULL);

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_5G_NR_DL_CORESET_PDCCH_CONFIG_DCI_TCP_COMMAND, tcpCommand),
            7, "TCP Command");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function configures the CORESET copy settings.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR cc/Component Carrier parameter.
/// HIPAR frame/Sets the Frame parameter.
/// HIPAR bandwidthPart/Sets the Bandwidth Part parameter.
/// HIPAR slot/Defines the number of slot.
/// HIPAR coreset/Sets the CORESET number.
/// HIPAR mode/Selects the paste logic when you copy a CORESET configuration to other
/// HIPAR mode/slots.
/// HIPAR period/Defines to which slots a CORESET configuration is copied to.
/// HIPAR duration/Defines to which slots a CORESET configuration is copied to.
/// HIPAR slots/Copies a CORESET allocation configuration to a specific set of slots.
ViStatus _VI_FUNC rsspecan_Configure5GNRDLCORESETCopySettings(
                            ViSession instrSession,
                            ViInt32   cc,
                            ViInt32   frame,
                            ViInt32   bandwidthPart,
                            ViInt32   slot,
                            ViInt32   coreset,
                            ViInt32   mode,
                            ViInt32   period,
                            ViInt32   duration,
                            ViString  slots)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_frame[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_bwp[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_slot[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_core[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_CORESET_COPY_MODE,
            "cc", cc - 1, RS_REPCAP_BUF_SIZE, repCap));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_CORESET_COPY_MODE,
            "NR5GFrame", frame - 1, RS_REPCAP_BUF_SIZE, repCap_frame));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_CORESET_COPY_MODE,
            "BWPart", bandwidthPart , RS_REPCAP_BUF_SIZE, repCap_bwp));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_CORESET_COPY_MODE,
            "NR5GSlot", slot , RS_REPCAP_BUF_SIZE, repCap_slot));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_CORESET_COPY_MODE,
            "NR5GCORESET", coreset , RS_REPCAP_BUF_SIZE, repCap_core));

    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_frame, NULL);
    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_bwp, NULL);
    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_slot, NULL);
    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_core, NULL);

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_5G_NR_DL_CORESET_COPY_MODE, mode),
            7, "Mode");

    switch (mode)
    {
        case RSSPECAN_VAL_NR5G_COPY_MODE_CUSTOM:
            viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_5G_NR_DL_CORESET_COPY_PERIOD, period),
                    8, "Period");

            viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_5G_NR_DL_CORESET_COPY_DURATION, duration),
                    9, "Duration");
            break;
        case RSSPECAN_VAL_NR5G_COPY_MODE_SLOT:
            viCheckParm(rsspecan_SetAttributeViString(instrSession, repCap, RSSPECAN_ATTR_5G_NR_DL_CORESET_COPY_SLOT, slots),
                    10, "Slots");
            break;
        default:
            checkErr(RsCore_InvalidViInt32Value(instrSession, mode));
            break;
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function copies a CORESET configuration to other slots.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR cc/Component Carrier parameter.
/// HIPAR frame/Sets the Frame parameter.
/// HIPAR bandwidthPart/Sets the Bandwidth Part parameter.
/// HIPAR slot/Defines the number of slot.
/// HIPAR coreset/Sets the CORESET number.
ViStatus _VI_FUNC rsspecan_NR5GDLCORESETCopy(ViSession instrSession,
                                             ViInt32   cc,
                                             ViInt32   frame,
                                             ViInt32   bandwidthPart,
                                             ViInt32   slot,
                                             ViInt32   coreset)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_frame[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_bwp[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_slot[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_core[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_CORESET_COPY,
            "cc", cc - 1, RS_REPCAP_BUF_SIZE, repCap));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_CORESET_COPY,
            "NR5GFrame", frame - 1, RS_REPCAP_BUF_SIZE, repCap_frame));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_CORESET_COPY,
            "BWPart", bandwidthPart , RS_REPCAP_BUF_SIZE, repCap_bwp));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_CORESET_COPY,
            "NR5GSlot", slot , RS_REPCAP_BUF_SIZE, repCap_slot));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_CORESET_COPY,
            "NR5GCORESET", coreset , RS_REPCAP_BUF_SIZE, repCap_core));

    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_frame, NULL);
    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_bwp, NULL);
    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_slot, NULL);
    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_core, NULL);

    checkErr(rsspecan_SetAttributeViString(instrSession, repCap, RSSPECAN_ATTR_5G_NR_DL_CORESET_COPY, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Turns the transmission of the CSI reference signal on and off.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR cc/Component Carrier parameter.
/// HIPAR frame/Sets the Frame parameter.
/// HIPAR bandwidthPart/Sets the Bandwidth Part parameter.
/// HIPAR CSI/Selects a CSI-RS.
/// HIPAR enabled/Turns the transmission of the CSI reference signal on and off.
ViStatus _VI_FUNC rsspecan_Configure5GNRDLCSIRSEnabled(ViSession instrSession,
                                                       ViInt32   cc,
                                                       ViInt32   frame,
                                                       ViInt32   bandwidthPart,
                                                       ViInt32   CSI,
                                                       ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_frame[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_bwp[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_csi[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_CSI_RS_ENABLED,
            "cc", cc - 1, RS_REPCAP_BUF_SIZE, repCap));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_CSI_RS_ENABLED,
            "NR5GFrame", frame - 1, RS_REPCAP_BUF_SIZE, repCap_frame));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_CSI_RS_ENABLED,
            "BWPart", bandwidthPart , RS_REPCAP_BUF_SIZE, repCap_bwp));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_CSI_RS_ENABLED,
            "CSI", CSI - 1, RS_REPCAP_BUF_SIZE, repCap_csi));

    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_frame, NULL);
    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_bwp, NULL);
    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_csi, NULL);

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, repCap, RSSPECAN_ATTR_5G_NR_DL_CSI_RS_ENABLED, enabled),
            5, "Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the number of different CSI-RS resources.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR cc/Component Carrier parameter.
/// HIPAR frame/Sets the Frame parameter.
/// HIPAR bandwidthPart/Sets the Bandwidth Part parameter.
/// HIPAR csi/Selects a CSI-RS.
/// HIPAR resources/Defines the number of different CSI-RS resources.
ViStatus _VI_FUNC rsspecan_Configure5GNRDLCSIRSResources(ViSession instrSession,
                                                         ViInt32   cc,
                                                         ViInt32   frame,
                                                         ViInt32   bandwidthPart,
                                                         ViInt32   csi,
                                                         ViInt32   resources)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_frame[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_bwp[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_csi[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_CSI_RS_RESOURCES,
            "cc", cc - 1, RS_REPCAP_BUF_SIZE, repCap));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_CSI_RS_RESOURCES,
            "NR5GFrame", frame - 1, RS_REPCAP_BUF_SIZE, repCap_frame));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_CSI_RS_RESOURCES,
            "BWPart", bandwidthPart , RS_REPCAP_BUF_SIZE, repCap_bwp));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_CSI_RS_RESOURCES,
            "CSI", csi - 1, RS_REPCAP_BUF_SIZE, repCap_csi));

    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_frame, NULL);
    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_bwp, NULL);
    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_csi, NULL);

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_5G_NR_DL_CSI_RS_RESOURCES, resources),
            6, "Resources");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures in which slots the CSI reference signal appears
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR cc/Component Carrier parameter.
/// HIPAR frame/Sets the Frame parameter.
/// HIPAR bandwidthPart/Sets the Bandwidth Part parameter.
/// HIPAR CSI/Selects a CSI-RS.
/// HIPAR method/Selects the transmission method for the CSI reference signal.
/// HIPAR periodicity/Selects the periodicity of CSI-RS transmission in terms of slots.
/// HIPAR offset/Defines an offset for the first CSI-RS relative to the first slot in a
/// HIPAR offset/bandwidth part.
/// HIPAR aperiodicSlots/Defines the slots in which a CSI reference signal is transmitted.
ViStatus _VI_FUNC rsspecan_Configure5GNRDLCSIRSSlot(ViSession instrSession,
                                                    ViInt32   cc,
                                                    ViInt32   frame,
                                                    ViInt32   bandwidthPart,
                                                    ViInt32   CSI,
                                                    ViInt32   method,
                                                    ViInt32   periodicity,
                                                    ViInt32   offset,
                                                    ViString  aperiodicSlots)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_frame[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_bwp[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_csi[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_CSI_RS_SLOT_MODE,
            "cc", cc - 1, RS_REPCAP_BUF_SIZE, repCap));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_CSI_RS_SLOT_MODE,
            "NR5GFrame", frame - 1, RS_REPCAP_BUF_SIZE, repCap_frame));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_CSI_RS_SLOT_MODE,
            "BWPart", bandwidthPart , RS_REPCAP_BUF_SIZE, repCap_bwp));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_CSI_RS_SLOT_MODE,
            "CSI", CSI - 1 , RS_REPCAP_BUF_SIZE, repCap_csi));

    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_frame, NULL);
    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_bwp, NULL);
    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_csi, NULL);

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_5G_NR_DL_CSI_RS_SLOT_MODE, method),
            5, "Method");

    if (method == RSSPECAN_VAL_5G_NR_CSI_RS_SLOT_MODE_PERIODIC)
    {
        viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_5G_NR_DL_CSI_RS_SLOT_PERIODICITY, periodicity),
                6, "Periodicity");

        viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_5G_NR_DL_CSI_RS_SLOT_OFFSET, offset),
                7, "Offset");
    }
    else
    {
        viCheckParm(rsspecan_SetAttributeViString(instrSession, repCap, RSSPECAN_ATTR_5G_NR_DL_CSI_RS_APERIODIC_SLOTS, aperiodicSlots),
                8, "Aperiodic Slots");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures in power settings of the CSI RS.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR cc/Component Carrier parameter.
/// HIPAR frame/Sets the Frame parameter.
/// HIPAR bandwidthPart/Sets the Bandwidth Part parameter.
/// HIPAR CSI/Selects a CSI-RS.
/// HIPAR zeroPower/Turns zero power transmission of the CSI reference signal on and off.
/// HIPAR scramblingID/Defines the seed value for the CSI-RS sequence generation. This
/// HIPAR scramblingID/control is only available when Zero Power is turned off.
/// HIPAR relativePower/Defines the relative power of the CSI-RS. This control is only
/// HIPAR relativePower/available when Zero Power is turned off.
ViStatus _VI_FUNC rsspecan_Configure5GNRDLCSIRSPower(ViSession instrSession,
                                                     ViInt32   cc,
                                                     ViInt32   frame,
                                                     ViInt32   bandwidthPart,
                                                     ViInt32   CSI,
                                                     ViBoolean zeroPower,
                                                     ViInt32   scramblingID,
                                                     ViReal64  relativePower)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_frame[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_bwp[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_csi[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_CSI_RS_ZERO_POWER_ENABLED,
            "cc", cc - 1, RS_REPCAP_BUF_SIZE, repCap));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_CSI_RS_ZERO_POWER_ENABLED,
            "NR5GFrame", frame - 1, RS_REPCAP_BUF_SIZE, repCap_frame));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_CSI_RS_ZERO_POWER_ENABLED,
            "BWPart", bandwidthPart , RS_REPCAP_BUF_SIZE, repCap_bwp));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_CSI_RS_ZERO_POWER_ENABLED,
            "BWPart", CSI - 1 , RS_REPCAP_BUF_SIZE, repCap_csi));

    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_frame, NULL);
    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_bwp, NULL);
    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_csi, NULL);

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, repCap, RSSPECAN_ATTR_5G_NR_DL_CSI_RS_ZERO_POWER_ENABLED, zeroPower),
            5, "Zero Power");

    if (zeroPower == VI_FALSE)
    {
        viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_5G_NR_DL_CSI_RS_SCRAMBLING_ID, scramblingID),
                6, "Scrambling ID");

        viCheckParm(rsspecan_SetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_5G_NR_DL_CSI_RS_RELATIVE_POWER, relativePower),
                7, "Relative Power");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures in resource block settings of the CSI RS.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR cc/Component Carrier parameter.
/// HIPAR frame/Sets the Frame parameter.
/// HIPAR bandwidthPart/Sets the Bandwidth Part parameter.
/// HIPAR CSI/Selects a CSI-RS.
/// HIPAR numberofRBs/Selects the number of resource blocks occupied by the CSI reference
/// HIPAR numberofRBs/signal.
/// HIPAR startRB/Select the first resource block occupied by the CSI reference signal.
ViStatus _VI_FUNC rsspecan_Configure5GNRDLCSIRSRB(ViSession instrSession,
                                                  ViInt32   cc,
                                                  ViInt32   frame,
                                                  ViInt32   bandwidthPart,
                                                  ViInt32   CSI,
                                                  ViInt32   numberofRBs,
                                                  ViInt32   startRB)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_frame[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_bwp[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_csi[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_CSI_RS_NUMBER_OF_RBS,
            "cc", cc - 1, RS_REPCAP_BUF_SIZE, repCap));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_CSI_RS_NUMBER_OF_RBS,
            "NR5GFrame", frame - 1, RS_REPCAP_BUF_SIZE, repCap_frame));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_CSI_RS_NUMBER_OF_RBS,
            "BWPart", bandwidthPart , RS_REPCAP_BUF_SIZE, repCap_bwp));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_CSI_RS_NUMBER_OF_RBS,
            "CSI", CSI - 1 , RS_REPCAP_BUF_SIZE, repCap_csi));

    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_frame, NULL);
    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_bwp, NULL);
    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_csi, NULL);

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_5G_NR_DL_CSI_RS_NUMBER_OF_RBS, numberofRBs),
            5, "Number Of RBs");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_5G_NR_DL_CSI_RS_START_RB, startRB),
            6, "Start RB");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects the location setting for the CSI-RS.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR cc/Component Carrier parameter.
/// HIPAR frame/Sets the Frame parameter.
/// HIPAR bandwidthPart/Sets the Bandwidth Part parameter.
/// HIPAR CSI/Selects a CSI-RS.
/// HIPAR row/Selects the location setting for the CSI-RS.
ViStatus _VI_FUNC rsspecan_Configure5GNRDLCSIRSRow(ViSession instrSession,
                                                   ViInt32   cc,
                                                   ViInt32   frame,
                                                   ViInt32   bandwidthPart,
                                                   ViInt32   CSI,
                                                   ViInt32   row)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_frame[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_bwp[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_csi[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_CSI_RS_ROW,
            "cc", cc - 1, RS_REPCAP_BUF_SIZE, repCap));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_CSI_RS_ROW,
            "NR5GFrame", frame - 1, RS_REPCAP_BUF_SIZE, repCap_frame));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_CSI_RS_ROW,
            "BWPart", bandwidthPart , RS_REPCAP_BUF_SIZE, repCap_bwp));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_CSI_RS_ROW,
            "CSI", CSI - 1 , RS_REPCAP_BUF_SIZE, repCap_csi));

    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_frame, NULL);
    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_bwp, NULL);
    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_csi, NULL);

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_5G_NR_DL_CSI_RS_ROW, row),
            5, "Row");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects the density configuration for the CSI-RS.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR cc/Component Carrier parameter.
/// HIPAR frame/Sets the Frame parameter.
/// HIPAR bandwidthPart/Sets the Bandwidth Part parameter.
/// HIPAR CSI/Selects a CSI-RS.
/// HIPAR density/Selects the density configuration for the CSI-RS.
ViStatus _VI_FUNC rsspecan_Configure5GNRDLCSIRSDensity(ViSession instrSession,
                                                       ViInt32   cc,
                                                       ViInt32   frame,
                                                       ViInt32   bandwidthPart,
                                                       ViInt32   CSI,
                                                       ViInt32   density)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_frame[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_bwp[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_csi[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_CSI_RS_DENSITY,
            "cc", cc - 1, RS_REPCAP_BUF_SIZE, repCap));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_CSI_RS_DENSITY,
            "NR5GFrame", frame - 1, RS_REPCAP_BUF_SIZE, repCap_frame));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_CSI_RS_DENSITY,
            "BWPart", bandwidthPart , RS_REPCAP_BUF_SIZE, repCap_bwp));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_CSI_RS_DENSITY,
            "CSI", CSI - 1 , RS_REPCAP_BUF_SIZE, repCap_csi));

    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_frame, NULL);
    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_bwp, NULL);
    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_csi, NULL);

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_5G_NR_DL_CSI_RS_DENSITY, density),
            5, "Density");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the bitmap setting for the CSI-RS.
/// HIFN     Note that you cannot change the bitmap value for selected location
/// HIFN  settings (RSSPECAN_ATTR_5G_NR_DL_CSI_RS_ROW).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR cc/Component Carrier parameter.
/// HIPAR frame/Sets the Frame parameter.
/// HIPAR bandwidthPart/Sets the Bandwidth Part parameter.
/// HIPAR CSI/Selects a CSI-RS.
/// HIPAR bitmap/Defines the bitmap setting for the CSI-RS.
ViStatus _VI_FUNC rsspecan_Configure5GNRDLCSIRSBitmap(ViSession instrSession,
                                                      ViInt32   cc,
                                                      ViInt32   frame,
                                                      ViInt32   bandwidthPart,
                                                      ViInt32   CSI,
                                                      ViInt32   bitmap)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_frame[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_bwp[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_csi[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_CSI_RS_BITMAP,
            "cc", cc - 1, RS_REPCAP_BUF_SIZE, repCap));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_CSI_RS_BITMAP,
            "NR5GFrame", frame - 1, RS_REPCAP_BUF_SIZE, repCap_frame));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_CSI_RS_BITMAP,
            "BWPart", bandwidthPart , RS_REPCAP_BUF_SIZE, repCap_bwp));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_CSI_RS_BITMAP,
            "CSI", CSI - 1 , RS_REPCAP_BUF_SIZE, repCap_csi));

    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_frame, NULL);
    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_bwp, NULL);
    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_csi, NULL);

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_5G_NR_DL_CSI_RS_BITMAP, bitmap),
            5, "Bitmap");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines parameter l0 or l1 that define the location of the CSI-RS in
/// HIFN  the time domain. Note that l0  must be smaller than l1 . l0  is
/// HIFN  automatically adjusted if this is not the case.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR cc/Component Carrier parameter.
/// HIPAR frame/Sets the Frame parameter.
/// HIPAR bandwidthPart/Sets the Bandwidth Part parameter.
/// HIPAR CSI/Selects a CSI-RS.
/// HIPAR parameter/Selects the location parameter.
/// HIPAR location/Parameters l0  and l1  define the location of the CSI-RS in the time
/// HIPAR location/domain.
ViStatus _VI_FUNC rsspecan_Configure5GNRDLCSIRSLocation(ViSession instrSession,
                                                        ViInt32   cc,
                                                        ViInt32   frame,
                                                        ViInt32   bandwidthPart,
                                                        ViInt32   csi,
                                                        ViInt32   parameter,
                                                        ViInt32   location)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_frame[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_bwp[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_csi[RS_REPCAP_BUF_SIZE] = "";
    ViAttr   attr[] = {RSSPECAN_ATTR_5G_NR_DL_CSI_RS_L0, RSSPECAN_ATTR_5G_NR_DL_CSI_RS_L1};

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_CSI_RS_L0,
            "cc", cc - 1, RS_REPCAP_BUF_SIZE, repCap));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_CSI_RS_L0,
            "NR5GFrame", frame - 1, RS_REPCAP_BUF_SIZE, repCap_frame));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_CSI_RS_L0,
            "BWPart", bandwidthPart , RS_REPCAP_BUF_SIZE, repCap_bwp));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_CSI_RS_L0,
            "CSI", csi - 1 , RS_REPCAP_BUF_SIZE, repCap_csi));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, parameter, RSSPECAN_VAL_5G_NR_CSI_RS_L0, RSSPECAN_VAL_5G_NR_CSI_RS_L1),
            5, "Parameter");

    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_frame, NULL);
    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_bwp, NULL);
    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_csi, NULL);

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, attr[parameter], location),
            6, "Location");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the port configuration of the CSI-RS.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR cc/Component Carrier parameter.
/// HIPAR frame/Sets the Frame parameter.
/// HIPAR bandwidthPart/Sets the Bandwidth Part parameter.
/// HIPAR CSI/Selects a CSI-RS.
/// HIPAR ports/Queries the port configuration of the CSI-RS.
ViStatus _VI_FUNC rsspecan_Query5GNRDLCSIRSPorts(ViSession instrSession,
                                                 ViInt32   cc,
                                                 ViInt32   frame,
                                                 ViInt32   bandwidthPart,
                                                 ViInt32   CSI,
                                                 ViInt32*  ports)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_frame[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_bwp[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_csi[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_CSI_RS_PORTS,
            "cc", cc - 1, RS_REPCAP_BUF_SIZE, repCap));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_CSI_RS_PORTS,
            "NR5GFrame", frame - 1, RS_REPCAP_BUF_SIZE, repCap_frame));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_CSI_RS_PORTS,
            "BWPart", bandwidthPart , RS_REPCAP_BUF_SIZE, repCap_bwp));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_CSI_RS_PORTS,
            "CSI", CSI - 1 , RS_REPCAP_BUF_SIZE, repCap_csi));

    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_frame, NULL);
    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_bwp, NULL);
    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_csi, NULL);

    viCheckParm(rsspecan_GetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_5G_NR_DL_CSI_RS_PORTS, ports),
            5, "Ports");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the CDM type of the CSI-RS.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR cc/Component Carrier parameter.
/// HIPAR frame/Sets the Frame parameter.
/// HIPAR bandwidthPart/Sets the Bandwidth Part parameter.
/// HIPAR CSI/Selects a CSI-RS.
/// HIPAR cdmType/Queries the CDM type of the CSI-RS.
ViStatus _VI_FUNC rsspecan_Query5GNRDLCSIRSCDMType(ViSession instrSession,
                                                   ViInt32   cc,
                                                   ViInt32   frame,
                                                   ViInt32   bandwidthPart,
                                                   ViInt32   CSI,
                                                   ViInt32*  cdmType)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_frame[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_bwp[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_csi[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_CSI_RS_CDM_TYPE,
            "cc", cc - 1, RS_REPCAP_BUF_SIZE, repCap));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_CSI_RS_CDM_TYPE,
            "NR5GFrame", frame - 1, RS_REPCAP_BUF_SIZE, repCap_frame));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_CSI_RS_CDM_TYPE,
            "BWPart", bandwidthPart , RS_REPCAP_BUF_SIZE, repCap_bwp));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_CSI_RS_CDM_TYPE,
            "CSI", CSI - 1 , RS_REPCAP_BUF_SIZE, repCap_csi));

    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_frame, NULL);
    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_bwp, NULL);
    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_csi, NULL);

    viCheckParm(rsspecan_GetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_5G_NR_DL_CSI_RS_CDM_TYPE, cdmType),
            5, "CDM Type");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Tturns the transmission of the positioning reference signal on and
/// HIFN  off.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR cc/Component Carrier parameter.
/// HIPAR frame/Sets the Frame parameter.
/// HIPAR bandwidthPart/Sets the Bandwidth Part parameter.
/// HIPAR prsEnabled/Tturns the transmission of the positioning reference signal on and
/// HIPAR prsEnabled/off.
ViStatus _VI_FUNC rsspecan_Configure5GNRDLPRSEnabled(ViSession instrSession,
                                                     ViInt32   cc,
                                                     ViInt32   frame,
                                                     ViInt32   bandwidthPart,
                                                     ViBoolean prsEnabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_frame[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_bwp[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_PRS_ENABLED,
            "cc", cc - 1, RS_REPCAP_BUF_SIZE, repCap));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_PRS_ENABLED,
            "NR5GFrame", frame - 1, RS_REPCAP_BUF_SIZE, repCap_frame));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_PRS_ENABLED,
            "BWPart", bandwidthPart , RS_REPCAP_BUF_SIZE, repCap_bwp));

    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_frame, NULL);
    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_bwp, NULL);

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, repCap, RSSPECAN_ATTR_5G_NR_DL_PRS_ENABLED, prsEnabled),
            2, "");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Defines the slots in which a positioning reference signal is
/// HIFN  transmitted.
/// HIFN     String containing the slot numbers a PRS is allocated to.
/// HIFN     Example: To transmit the CSI-RS in slots 1,2,3,4,5,7,11, enter the
/// HIFN  string '1-5,7,11'.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR cc/Component Carrier parameter.
/// HIPAR frame/Sets the Frame parameter.
/// HIPAR bandwidthPart/Sets the Bandwidth Part parameter.
/// HIPAR slotConfig/Defines the slots in which a positioning reference signal is
/// HIPAR slotConfig/transmitted.
ViStatus _VI_FUNC rsspecan_Configure5GNRDLPRSSlotConfig(ViSession instrSession,
                                                        ViInt32   cc,
                                                        ViInt32   frame,
                                                        ViInt32   bandwidthPart,
                                                        ViString  slotConfig)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_frame[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_bwp[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_PRS_ENABLED,
            "cc", cc - 1, RS_REPCAP_BUF_SIZE, repCap));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_PRS_ENABLED,
            "NR5GFrame", frame - 1, RS_REPCAP_BUF_SIZE, repCap_frame));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_PRS_ENABLED,
            "BWPart", bandwidthPart , RS_REPCAP_BUF_SIZE, repCap_bwp));

    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_frame, NULL);
    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_bwp, NULL);

    viCheckParm(rsspecan_SetAttributeViString(instrSession, repCap, RSSPECAN_ATTR_5G_NR_DL_PRS_SLOT_CONFIG, slotConfig),
            5, "Slot Config");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Configures in resource block settings of the positioning reference
/// HIFN  signal.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR cc/Component Carrier parameter.
/// HIPAR frame/Sets the Frame parameter.
/// HIPAR bandwidthPart/Sets the Bandwidth Part parameter.
/// HIPAR numberOfRBs/Selects the number of resource blocks occupied by the positioning
/// HIPAR numberOfRBs/reference signal.
/// HIPAR startRB/Select the first resource block occupied by the positioning reference
/// HIPAR startRB/signal.
ViStatus _VI_FUNC rsspecan_Configure5GNRDLPRSRB(ViSession instrSession,
                                                ViInt32   cc,
                                                ViInt32   frame,
                                                ViInt32   bandwidthPart,
                                                ViInt32   numberOfRBs,
                                                ViInt32   startRB)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_frame[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_bwp[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_PRS_NUMBER_OF_RBS,
            "cc", cc - 1, RS_REPCAP_BUF_SIZE, repCap));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_PRS_NUMBER_OF_RBS,
            "NR5GFrame", frame - 1, RS_REPCAP_BUF_SIZE, repCap_frame));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_PRS_NUMBER_OF_RBS,
            "BWPart", bandwidthPart , RS_REPCAP_BUF_SIZE, repCap_bwp));

    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_frame, NULL);
    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_bwp, NULL);

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_5G_NR_DL_PRS_NUMBER_OF_RBS, numberOfRBs),
            6, "Number of RBs");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_5G_NR_DL_PRS_START_RB, startRB),
            7, "Start RB");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Configures the symbol settings of the positioning reference signal.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR cc/Component Carrier parameter.
/// HIPAR frame/Sets the Frame parameter.
/// HIPAR bandwidthPart/Sets the Bandwidth Part parameter.
/// HIPAR l^prsStart/Defines the first symbol the positioning reference signal uses.
/// HIPAR l^prs/Defines the number of symbols the positioning reference signal uses.
ViStatus _VI_FUNC rsspecan_Configure5GNRDLPRSL_PRS(ViSession instrSession,
                                                   ViInt32   cc,
                                                   ViInt32   frame,
                                                   ViInt32   bandwidthPart,
                                                   ViInt32   lprsStart,
                                                   ViInt32   lprs)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_frame[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_bwp[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_PRS_L_PRS_START,
            "cc", cc - 1, RS_REPCAP_BUF_SIZE, repCap));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_PRS_L_PRS_START,
            "NR5GFrame", frame - 1, RS_REPCAP_BUF_SIZE, repCap_frame));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_PRS_L_PRS_START,
            "BWPart", bandwidthPart , RS_REPCAP_BUF_SIZE, repCap_bwp));

    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_frame, NULL);
    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_bwp, NULL);

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_5G_NR_DL_PRS_L_PRS_START, lprsStart),
            6, "L PRS Start");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_5G_NR_DL_PRS_L_PRS, lprs),
            7, "L PRS");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Defines the seed value for the positioning reference signal sequence
/// HIFN  generation.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR cc/Component Carrier parameter.
/// HIPAR frame/Sets the Frame parameter.
/// HIPAR bandwidthPart/Sets the Bandwidth Part parameter.
/// HIPAR n^prsIDSeq/Defines the seed value for the positioning reference signal sequence
/// HIPAR n^prsIDSeq/generation.
ViStatus _VI_FUNC rsspecan_Configure5GNRDLPRSN_PRSIDSeq(ViSession instrSession,
                                                        ViInt32   cc,
                                                        ViInt32   frame,
                                                        ViInt32   bandwidthPart,
                                                        ViInt32   nprsIDSeq)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_frame[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_bwp[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_PRS_N_PRS_ID_SEQ,
            "cc", cc - 1, RS_REPCAP_BUF_SIZE, repCap));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_PRS_N_PRS_ID_SEQ,
            "NR5GFrame", frame - 1, RS_REPCAP_BUF_SIZE, repCap_frame));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_PRS_N_PRS_ID_SEQ,
            "BWPart", bandwidthPart , RS_REPCAP_BUF_SIZE, repCap_bwp));

    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_frame, NULL);
    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_bwp, NULL);

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_5G_NR_DL_PRS_N_PRS_ID_SEQ, nprsIDSeq),
            5, "nPRS ID Seq");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Configures the subcarrier settings of the positioning reference
/// HIFN  signal.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR cc/Component Carrier parameter.
/// HIPAR frame/Sets the Frame parameter.
/// HIPAR bandwidthPart/Sets the Bandwidth Part parameter.
/// HIPAR k^prsComb/Defines the number of subcarriers the positioning reference signal
/// HIPAR k^prsComb/uses.
/// HIPAR k^prsOffset/Defines an offset in the frequency domain for the positioning
/// HIPAR k^prsOffset/reference signal.
ViStatus _VI_FUNC rsspecan_Configure5GNRDLPRSK_PRS(ViSession instrSession,
                                                   ViInt32   cc,
                                                   ViInt32   frame,
                                                   ViInt32   bandwidthPart,
                                                   ViInt32   kprsComb,
                                                   ViReal64  kprsOffset)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_frame[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_bwp[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_PRS_K_PRS_COMB,
            "cc", cc - 1, RS_REPCAP_BUF_SIZE, repCap));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_PRS_K_PRS_COMB,
            "NR5GFrame", frame - 1, RS_REPCAP_BUF_SIZE, repCap_frame));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_PRS_K_PRS_COMB,
            "BWPart", bandwidthPart , RS_REPCAP_BUF_SIZE, repCap_bwp));

    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_frame, NULL);
    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_bwp, NULL);

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_5G_NR_DL_PRS_K_PRS_COMB, kprsComb),
            5, "K PRS Comb");

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_5G_NR_DL_PRS_K_PRS_OFFSET, kprsOffset),
            6, "K PRS Offset");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Defines the relative power of the positioning reference signal.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR cc/Component Carrier parameter.
/// HIPAR frame/Sets the Frame parameter.
/// HIPAR bandwidthPart/Sets the Bandwidth Part parameter.
/// HIPAR relativePower/Defines the relative power of the positioning reference signal.
ViStatus _VI_FUNC rsspecan_Configure5GNRDLPRSRelativePower(ViSession instrSession,
                                                           ViInt32   cc,
                                                           ViInt32   frame,
                                                           ViInt32   bandwidthPart,
                                                           ViReal64  relativePower)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_frame[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_bwp[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_PRS_RELATIVE_POWER,
            "cc", cc - 1, RS_REPCAP_BUF_SIZE, repCap));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_PRS_RELATIVE_POWER,
            "NR5GFrame", frame - 1, RS_REPCAP_BUF_SIZE, repCap_frame));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_PRS_RELATIVE_POWER,
            "BWPart", bandwidthPart , RS_REPCAP_BUF_SIZE, repCap_bwp));

    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_frame, NULL);
    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_bwp, NULL);

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_5G_NR_DL_PRS_RELATIVE_POWER, relativePower),
            5, "Relative Power");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function turns an individual SS/PBCH block on and off.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR cc/Component Carrier parameter.
/// HIPAR ss/Sets the SS parameter.
/// HIPAR enabled/Turns an individual SS/PBCH block on and off.
ViStatus _VI_FUNC rsspecan_Configure5GNRDLSynchronizationSignalEnabled(
                            ViSession instrSession,
                            ViInt32   cc,
                            ViInt32   ss,
                            ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_ss[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_SYNCHRONIZATION_SIGNAL_ENABLED,
            "cc", cc - 1, RS_REPCAP_BUF_SIZE, repCap));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_SYNCHRONIZATION_SIGNAL_ENABLED,
            "ss", ss , RS_REPCAP_BUF_SIZE, repCap_ss));

    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_ss, NULL);

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, repCap, RSSPECAN_ATTR_5G_NR_DL_SYNCHRONIZATION_SIGNAL_ENABLED, enabled),
            4, "Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the Synchonization Signal.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR cc/Component Carrier parameter.
/// HIPAR offset/Defines a frequency offset for the synchronization signal block.
/// HIPAR periodicity/Selects the burst set periodicity of a synchronization signal block.
/// HIPAR configurationMode/Selects the synchronization signal configuration mode.
/// HIPAR frequencyOffset/Defines a frequency offset for the synchronization signal block.
/// HIPAR pattern/Selects the pattern of a synchronization signal block.
/// HIPAR subcarrierSpacing/Selects the subcarrier spacing of a synchronization signal block.
ViStatus _VI_FUNC rsspecan_Configure5GNRDLSynchronizationSignal(
                            ViSession instrSession,
                            ViInt32   cc,
                            ViInt32   detection,
                            ViInt32   subcarrierSpacing,
                            ViInt32   pattern,
                            ViInt32   frequencyOffset,
                            ViInt32   offset,
                            ViInt32   periodicity)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_SYNCHRONIZATION_SIGNAL_OFFSET,
            "cc", cc - 1, RS_REPCAP_BUF_SIZE, repCap));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_5G_NR_DL_SYNCHRONIZATION_SIGNAL_CONFIGURATION_MODE, detection),
            5, "Configuration Mode");

    if (detection == RSSPECAN_VAL_5G_NR_SYNC_SIGNAL_MODE_MANUAL)
    {
        viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_5G_NR_DL_SYNCHRONIZATION_SIGNAL_OFFSET, offset),
                3, "Offset");

        viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_5G_NR_DL_SYNCHRONIZATION_SIGNAL_FREQUENCY_OFFSET, frequencyOffset),
                6, "Frequency Offset");

        viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_5G_NR_DL_SYNCHRONIZATION_SIGNAL_PATTERN, pattern),
                7, "Pattern");

        viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_5G_NR_DL_SYNCHRONIZATION_SIGNAL_SUBCARRIER_SPACING, subcarrierSpacing),
                8, "Subcarrier Spacing");
    }
    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_5G_NR_DL_SYNCHRONIZATION_SIGNAL_BURST_SET_PERIODICITY, periodicity),
            4, "Periodicity");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the Synchonization Signal PBCH.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR cc/Component Carrier parameter.
/// HIPAR power/Defines the relative power of the PBCH.
/// HIPAR dmrsPower/Defines the relative power of the PBCH DMRS.
ViStatus _VI_FUNC rsspecan_Configure5GNRDLSynchronizationSignalPBCH(
                            ViSession instrSession,
                            ViInt32   cc,
                            ViReal64  power,
                            ViReal64  dmrsPower)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_SYNCHRONIZATION_SIGNAL_PBCH_POWER,
            "cc", cc - 1, RS_REPCAP_BUF_SIZE, repCap));

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_5G_NR_DL_SYNCHRONIZATION_SIGNAL_PBCH_POWER, power),
            3, "Power");

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_5G_NR_DL_SYNCHRONIZATION_SIGNAL_PBCH_DMRS_POWER, dmrsPower),
            4, "DMRS Power");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the Synchonization Signal PBCH.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR cc/Component Carrier parameter.
/// HIPAR type/Sets the power type to select.
/// HIPAR power/Defines the relative power of the selected Type.
ViStatus _VI_FUNC rsspecan_Configure5GNRDLSynchronizationSignalPower(
                            ViSession instrSession,
                            ViInt32   cc,
                            ViInt32   type,
                            ViReal64  power)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViAttr   attr[] = {
             RSSPECAN_ATTR_5G_NR_DL_SYNCHRONIZATION_SIGNAL_PSS_POWER,
             RSSPECAN_ATTR_5G_NR_DL_SYNCHRONIZATION_SIGNAL_SSS_POWER
             };

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, type, 0, 1),
            3, "Type");

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_SYNCHRONIZATION_SIGNAL_PSS_POWER,
            "cc", cc - 1, RS_REPCAP_BUF_SIZE, repCap));

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, repCap, attr[type], power),
            4, "Power");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects the reference point for a synchronization signal offset. For
/// HIFN  selection of TxBW: Use at least one bandwidth part that has the same
/// HIFN  subcarrier spacing as the synchronization signal. For subcarrier
/// HIFN  spacing = 240 kHz, TxBW is not supported.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR cc/Component Carrier parameter.
/// HIPAR offsetRelativeTo/Selects the reference point for a synchronization signal offset.
ViStatus _VI_FUNC rsspecan_Configure5GNRDLSynchronizationSignalOffsetRelativeTo(
                            ViSession instrSession,
                            ViInt32   cc,
                            ViInt32   offsetRelativeTo)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_SYNCHRONIZATION_SIGNAL_OFFSET_RELATIVE_TO,
            "cc", cc - 1, RS_REPCAP_BUF_SIZE, repCap));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_5G_NR_DL_SYNCHRONIZATION_SIGNAL_OFFSET_RELATIVE_TO, offsetRelativeTo),
            3, "Offset Relative To");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects the number of SS/PBCH blocks in the deployment range < 3 GHz.
/// HIFN     Prerequisites for this attribute
/// HIFN     - Select a deployment < 3 GHz
/// HIFN  (RSSPECAN_ATTR_5G_NR_DL_DEPLOYMENT_FREQUENCY_RANGE).
/// HIFN     - Select a 30 kHz subcarrier spacing
/// HIFN  (RSSPECAN_ATTR_5G_NR_DL_SYNCHRONIZATION_SIGNAL_SUBCARRIER_SPACING).
/// HIFN     - Select the case C block pattern
/// HIFN  (RSSPECAN_ATTR_5G_NR_DL_SYNCHRONIZATION_SIGNAL_PATTERN).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR cc/Component Carrier parameter.
/// HIPAR lSelection/Selects the number of SS/PBCH blocks in the deployment range < 3 GHz.
ViStatus _VI_FUNC rsspecan_Configure5GNRDLSynchronizationSignalLSelection(
                            ViSession instrSession,
                            ViInt32   cc,
                            ViInt32   lSelection)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_SYNCHRONIZATION_SIGNAL_L_SELECTION,
            "cc", cc - 1, RS_REPCAP_BUF_SIZE, repCap));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_5G_NR_DL_SYNCHRONIZATION_SIGNAL_L_SELECTION, lSelection),
            3, "L Selection");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects the half frame that contains the synchronization signal.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR cc/Component Carrier parameter.
/// HIPAR halfFrameOffset/Selects the half frame that contains the synchronization signal.
ViStatus _VI_FUNC rsspecan_Configure5GNRDLSynchronizationSignalHalfFrameOffset(
                            ViSession instrSession,
                            ViInt32   cc,
                            ViInt32   halfFrameOffset)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_SYNCHRONIZATION_SIGNAL_HALF_FRAME_OFFSET,
            "cc", cc - 1, RS_REPCAP_BUF_SIZE, repCap));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_5G_NR_DL_SYNCHRONIZATION_SIGNAL_HALF_FRAME_OFFSET, halfFrameOffset),
            3, "Half Frame Offset");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects one of the antenna port configurations.
/// HIFN     Effects of this function
/// HIFN     If you turn on a configuration, the other antenna port configuration
/// HIFN  is automatically turned off (and vice versa).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR cc/Component Carrier parameter.
/// HIPAR antennaConfig/Antenna config parameter.
/// HIPAR enabled/Turn on or off the antenna port configuration.
ViStatus _VI_FUNC rsspecan_Configure5GNRDLAntennaPortMappingState(
                            ViSession instrSession,
                            ViInt32   cc,
                            ViInt32   antennaConfig,
                            ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_ac[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_ANTENNA_PORT_MAPPING_ENABLED,
            "cc", cc - 1, RS_REPCAP_BUF_SIZE, repCap));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_ANTENNA_PORT_MAPPING_ENABLED,
            "AntennaConfig", antennaConfig - 1, RS_REPCAP_BUF_SIZE, repCap_ac));

    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_ac, NULL);

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, repCap, RSSPECAN_ATTR_5G_NR_DL_ANTENNA_PORT_MAPPING_ENABLED, enabled),
            3, "Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects the antenna port(s) on which the CSI reference signal is
/// HIFN  transmitted.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR cc/Component Carrier parameter.
/// HIPAR antennaConfig/Antenna config parameter.
/// HIPAR antennaPort/Antenna port.
/// HIPAR enabled/Turns the transmission of the CSI-RS on a certain antenna port on and
/// HIPAR enabled/off.
ViStatus _VI_FUNC rsspecan_Configure5GNRDLAntennaPortMappingCSIRS(
                            ViSession instrSession,
                            ViInt32   cc,
                            ViInt32   antennaConfig,
                            ViInt32   antennaPort,
                            ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, cc, 1, 16),
            2, "CC");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, antennaConfig, 1, 2),
            3, "Antenna Config");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, antennaPort, 3000, 3031),
            4, "Antenna Port");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CONF:NR5G:DL:CC%ld:PAM%ld:CSIR:AP%ld %s", cc, antennaConfig, antennaPort, enabled ? "ON" : "OFF");
    checkErr(RsCore_Write(instrSession, cmd));
    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects the antenna port(s) on which the PDSCH is transmitted.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR cc/Component Carrier parameter.
/// HIPAR antennaConfig/Component Carrier parameter.
/// HIPAR antennaPort/Antenna port.
/// HIPAR enabled/Enables the PDSCH on antenna ports.
ViStatus _VI_FUNC rsspecan_Configure5GNRDLAntennaPortMappingPDSCH(
                            ViSession instrSession,
                            ViInt32   cc,
                            ViInt32   antennaConfig,
                            ViInt32   antennaPort,
                            ViInt32   enabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";
    ViChar   *p2string;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, cc, 1, 16),
            2, "CC");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, antennaConfig, 1, 2),
            3, "Antenna Config");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, antennaPort, 1000, 1011),
            4, "Antenna Port");
    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &RsSpecAn_rng5GNRAntennaPortSelect, enabled, &p2string),
            5, "enabled");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CONF:NR5G:DL:CC%ld:PAM%ld:PDSC:AP%ld %s", cc, antennaConfig, antennaPort, p2string);
    checkErr(RsCore_Write(instrSession, cmd));
    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Controls the way the DC carrier is handled during signal analysis.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR cc/Component Carrier parameter.
/// HIPAR handlingOfCarrierLeakage/Controls the way the DC carrier is handled during signal analysis.
ViStatus _VI_FUNC rsspecan_Configure5GNRDLAdvancedSettingsHandlingOfCarrierLeakage(
                            ViSession instrSession,
                            ViInt32   cc,
                            ViInt32   handlingOfCarrierLeakage)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_ADVANCED_SETTINGS_HANDLING_OF_CARRIER_LEAKAGE,
            "cc", cc - 1, RS_REPCAP_BUF_SIZE, repCap));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_5G_NR_DL_ADVANCED_SETTINGS_HANDLING_OF_CARRIER_LEAKAGE, handlingOfCarrierLeakage),
            3, "Handling Of Carrier Leakage");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the 5G NR system frame number.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR cc/Component Carrier parameter.
/// HIPAR frameNumber/Defines the 5G NR system frame number.
ViStatus _VI_FUNC rsspecan_Configure5GNRDLAdvancedSettingsFrameNumber(
                            ViSession instrSession,
                            ViInt32   cc,
                            ViInt32   frameNumber)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_ADVANCED_SETTINGS_FRAME_NUMBER,
            "cc", cc - 1, RS_REPCAP_BUF_SIZE, repCap));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_5G_NR_DL_ADVANCED_SETTINGS_FRAME_NUMBER, frameNumber),
            3, "Frame Number");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the RF upconversion.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR cc/Component Carrier parameter.
/// HIPAR phaseCompensation/Turns RF upconversion and corresponding phase compensation on and off.
/// HIPAR f_0Mode/Selects the frequency selection mode for RF upconversion. This control
/// HIPAR f_0Mode/is available when Phase Compensation is turned on.
/// HIPAR frequency/Selects a frequency for RF upconversion. This control is available
/// HIPAR frequency/when Phase Compensation is turned on and f_0 Mode is set to Manual.
ViStatus _VI_FUNC rsspecan_Configure5GNRDLAdvancedSettingsRFUpconversion(
                            ViSession instrSession,
                            ViInt32   cc,
                            ViBoolean phaseCompensation,
                            ViInt32   f_0Mode,
                            ViReal64  frequency)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_ADVANCED_SETTINGS_RF_UPCONVERSION_ENABLED,
            "cc", cc - 1, RS_REPCAP_BUF_SIZE, repCap));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, repCap, RSSPECAN_ATTR_5G_NR_DL_ADVANCED_SETTINGS_RF_UPCONVERSION_ENABLED, phaseCompensation),
            3, "Enabled");
    if (phaseCompensation == VI_TRUE)
    {
        viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_5G_NR_DL_ADVANCED_SETTINGS_RF_UPCONVERSION_MODE, f_0Mode),
                4, "f_0 Mode");
        if (f_0Mode == RSSPECAN_VAL_5G_NR_RF_UPCONVERSION_MODE_MANUAL)
        {
            viCheckParm(rsspecan_SetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_5G_NR_DL_ADVANCED_SETTINGS_RF_UPCONVERSION_FREQUENCY, frequency),
                    5, "Frequency");
        }
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines user IDs that are excluded from the calculation of
/// HIFN  modulation-specific EVM results.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR cc/Component Carrier parameter.
/// HIPAR excludeUserIDs/Defines user IDs that are excluded from the calculation of
/// HIPAR excludeUserIDs/modulation-specific EVM results.
ViStatus _VI_FUNC rsspecan_Configure5GNRDLAdvancedSettingsExcludeUserIDs(
                            ViSession instrSession,
                            ViInt32   cc,
                            ViString  excludeUserIDs)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_ADVANCED_SETTINGS_EXCLUDE_USER_IDS,
            "cc", cc - 1, RS_REPCAP_BUF_SIZE, repCap));

    viCheckParm(rsspecan_SetAttributeViString(instrSession, repCap, RSSPECAN_ATTR_5G_NR_DL_ADVANCED_SETTINGS_EXCLUDE_USER_IDS, excludeUserIDs),
            3, "Exclude User IDs");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Turns the frequency error limit check on and off.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR frequencyErrorLimitCheck/Turns the frequency error limit check on and off.
ViStatus _VI_FUNC rsspecan_Configure5GNRDLAdvancedSettingsFrequencyErrorLimitCheck(
                            ViSession instrSession,
                            ViBoolean frequencyErrorLimitCheck)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, NULL, RSSPECAN_ATTR_5G_NR_DL_ADVANCED_SETTINGS_FREQUENCY_ERROR_LIMIT_CHECK, frequencyErrorLimitCheck),
            2, "Frequency Error Limit Check");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Turns shared spectrum channel access on and off.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR cc/Component Carrier parameter.
/// HIPAR sharedSpectrumChannelAccess/Turns shared spectrum channel access on and off.
ViStatus _VI_FUNC rsspecan_Configure5GNRDLAdvancedSettingsSharedSpectrumChannelAccess(
                            ViSession instrSession,
                            ViInt32   cc,
                            ViBoolean sharedSpectrumChannelAccess)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_ADVANCED_SETTINGS_SHARED_SPECTRUM_CHANNEL_ACCESS,
            "cc", cc - 1, RS_REPCAP_BUF_SIZE, repCap));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, repCap, RSSPECAN_ATTR_5G_NR_DL_ADVANCED_SETTINGS_SHARED_SPECTRUM_CHANNEL_ACCESS, sharedSpectrumChannelAccess),
            3, "Shared Spectrum Channel Access");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures an offset relative to reference point A.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR cc/Component Carrier parameter.
/// HIPAR centerFrequencyOffset/Defines the frequency of the reference point A relative to the
/// HIPAR centerFrequencyOffset/carrier's center frequency.
/// HIPAR k0Select/Selects the K0 offset.
/// HIPAR offset/Defines an offset relative to reference point A for bandwidth parts
/// HIPAR offset/with selected subcarrier spacing.
ViStatus _VI_FUNC rsspecan_Configure5GNRDLAdvancedSettingsRefPointA(
                            ViSession instrSession,
                            ViInt32   cc,
                            ViReal64  centerFrequencyOffset,
                            ViInt32   k0Select,
                            ViInt32   offset)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViAttr   attr[] = {RSSPECAN_ATTR_5G_NR_DL_ADVANCED_SETTINGS_REF_POINT_A_15_KHZ_OFFSET,
             RSSPECAN_ATTR_5G_NR_DL_ADVANCED_SETTINGS_REF_POINT_A_30_KHZ_OFFSET,
             RSSPECAN_ATTR_5G_NR_DL_ADVANCED_SETTINGS_REF_POINT_A_60_KHZ_OFFSET,
             RSSPECAN_ATTR_5G_NR_DL_ADVANCED_SETTINGS_REF_POINT_A_120_KHZ_OFFSET};

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, k0Select, RSSPECAN_VAL_5G_NR_K0_15_KHZ, RSSPECAN_VAL_5G_NR_K0_120_KHZ),
            4, "K0 Select");

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_ADVANCED_SETTINGS_REF_POINT_A_CENTER_FREQUENCY_OFFSET,
            "cc", cc -1, RS_REPCAP_BUF_SIZE, repCap));

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_5G_NR_DL_ADVANCED_SETTINGS_REF_POINT_A_CENTER_FREQUENCY_OFFSET, centerFrequencyOffset),
            3, "Center Frequency Offset");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, attr[k0Select], offset),
            5, "Offset");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the absolute frequency of the reference point A.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR cc/Component Carrier parameter.
/// HIPAR frequency/Queries the absolute frequency of the reference point A.
ViStatus _VI_FUNC rsspecan_Query5GNRDLAdvancedSettingsRefPointAFrequency(
                            ViSession instrSession,
                            ViInt32   cc,
                            ViReal64* frequency)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_ADVANCED_SETTINGS_REF_POINT_A_FREQUENCY,
            "cc", cc -1, RS_REPCAP_BUF_SIZE, repCap));

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_5G_NR_DL_ADVANCED_SETTINGS_REF_POINT_A_FREQUENCY, frequency),
            3, "Frequency");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures coexistence with LTE resources.
/// HIFN     Prerequisites:
/// HIFN     - Select subcarrier spacing of 15 kHz
/// HIFN  (RSSPECAN_ATTR_5G_NR_DL_BANDWIDTH_PART_SUBCARRIER_SPACING)
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR cc/Component Carrier parameter.
/// HIPAR state/Turns coexistence with LTE resources on and off.
/// HIPAR offsettoPointA/Defines an LTE carrier offset relative to reference point A (in terms
/// HIPAR offsettoPointA/of resource blocks).
/// HIPAR lteBandwidthRBs/Selects the channel bandwidth of an LTE carrier.
/// HIPAR vshift/Selects the vShift parameter for an LTE signal.
/// HIPAR lteAntennaPorts/Selects the number of antenna ports for an LTE signal.
ViStatus _VI_FUNC rsspecan_Configure5GNRDLAdvancedSettingsLTECRSCoexistence(
                            ViSession instrSession,
                            ViInt32   cc,
                            ViBoolean state,
                            ViInt32   offsettoPointA,
                            ViInt32   lteBandwidthRBs,
                            ViInt32   vshift,
                            ViInt32   lteAntennaPorts)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_ADVANCED_SETTINGS_LTE_CRS_COEXISTENCE_STATE,
            "cc", cc - 1, RS_REPCAP_BUF_SIZE, repCap));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, repCap, RSSPECAN_ATTR_5G_NR_DL_ADVANCED_SETTINGS_LTE_CRS_COEXISTENCE_STATE, state),
            3, "State");

    if (state == VI_TRUE)
    {
        viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_5G_NR_DL_ADVANCED_SETTINGS_LTE_CRS_COEXISTENCE_OFFSET_TO_POINT_A, offsettoPointA),
                4, "Offset to Point A");

        viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_5G_NR_DL_ADVANCED_SETTINGS_LTE_CRS_COEXISTENCE_LTE_BANDWIDTH, lteBandwidthRBs),
                5, "LTE Bandwidth/RBs");

        viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_5G_NR_DL_ADVANCED_SETTINGS_LTE_CRS_COEXISTENCE_VSHIFT, vshift),
                6, "vShift");

        viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_5G_NR_DL_ADVANCED_SETTINGS_LTE_CRS_COEXISTENCE_LTE_ANTENNA_PORTS, lteAntennaPorts),
                7, "5LTE-CRS Coexistence LTE Antenna Ports");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Turns MBSFN transmission in specific subframes on and off.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR cc/Component Carrier parameter.
/// HIPAR subframe/Subframe parameter.
/// HIPAR mbsfnSubframes/Turns MBSFN transmission in specific subframes on and off.
ViStatus _VI_FUNC rsspecan_Configure5GNRDLAdvancedSettingsLTECRSCoexistenceMBSFNSubframes(
                            ViSession instrSession,
                            ViInt32   cc,
                            ViInt32   subframe,
                            ViBoolean mbsfnSubframes)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_subf[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_ADVANCED_SETTINGS_LTE_CRS_COEXISTENCE_MBSFN_SUBFRAMES,
            "cc", cc - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "CC");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_ADVANCED_SETTINGS_LTE_CRS_COEXISTENCE_MBSFN_SUBFRAMES,
            "Subframe", subframe - 1, RS_REPCAP_BUF_SIZE, repCap_subf),
            2, "Subframe");

    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_subf, NULL);

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, repCap, RSSPECAN_ATTR_5G_NR_DL_ADVANCED_SETTINGS_LTE_CRS_COEXISTENCE_MBSFN_SUBFRAMES, mbsfnSubframes),
            4, "MBSFN Subframes");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Defines the IP address of the connected signal generator.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR ipAddress/Defines the IP address of the connected signal generator.
ViStatus _VI_FUNC rsspecan_Configure5GNRDLGeneratorIPAddress(ViSession instrSession,
                                                             ViString  ipAddress)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViString(instrSession, NULL, RSSPECAN_ATTR_5G_NR_DL_GENERATOR_IP_ADDRESS, ipAddress),
            2, "IP Address");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the state of connection to the signal generator.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR connectionState/Queries the state of connection to the signal generator.
ViStatus _VI_FUNC rsspecan_Query5GNRDLGeneratorConnectionState(
                            ViSession instrSession,
                            ViInt32*  connectionState)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_GetAttributeViInt32(instrSession, NULL, RSSPECAN_ATTR_5G_NR_DL_GENERATOR_LED_STATE, connectionState),
            2, "Connection State");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Turns the generator control on and off.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR state/Turns the generator control on and off.
ViStatus _VI_FUNC rsspecan_Configure5GNRDLGeneratorControlState(
                            ViSession instrSession,
                            ViBoolean state)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, NULL, RSSPECAN_ATTR_5G_NR_DL_GENERATOR_CONTROL_ENABLED, state),
            2, "State");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Enables or disables RF output on the connected generator.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR rfOutput/Enables or disables RF output on the connected generator.
ViStatus _VI_FUNC rsspecan_Configure5GNRDLGeneratorRFOutput(ViSession instrSession,
                                                            ViBoolean rfOutput)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, NULL, RSSPECAN_ATTR_5G_NR_DL_GENERATOR_RF_OUTPUT_ENABLED, rfOutput),
            2, "RF Output");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Turns synchronization of the analyzer and generator frequency on and
/// HIFN  off.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR frequencySynchronization/Turns synchronization of the analyzer and generator frequency on and
/// HIPAR frequencySynchronization/off.
ViStatus _VI_FUNC rsspecan_Configure5GNRDLGeneratorFrequencySynchronization(
                            ViSession instrSession,
                            ViBoolean frequencySynchronization)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, NULL, RSSPECAN_ATTR_5G_NR_DL_GENERATOR_FREQUENCY_SYNCHRONIZATION, frequencySynchronization),
            2, "Frequency Synchronization");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Configures the level synchronization between analyzer and generator.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR levelState/Turns level synchronization between analyzer and generator on and off.
/// HIPAR level_RMS/Defines the output power RMS level of the generator.
/// HIPAR dutLimitState/Turns a limitation of the DUT peak input power on and off.
/// HIPAR dutLimitPeakInputPower/Defines the output power RMS level of the generator.
/// HIPAR dutGain/Defines DUT level gain for generator level control.
ViStatus _VI_FUNC rsspecan_Configure5GNRDLGeneratorLevel(ViSession instrSession,
                                                         ViBoolean levelState,
                                                         ViReal64  level_RMS,
                                                         ViBoolean dutLimitState,
                                                         ViReal64  dutLimitPeakInputPower,
                                                         ViReal64  dutGain)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, NULL, RSSPECAN_ATTR_5G_NR_DL_GENERATOR_LEVEL_CONTROL_ENABLED, levelState),
            2, "Level State");

    if (levelState == VI_TRUE)
    {
        viCheckParm(rsspecan_SetAttributeViReal64(instrSession, NULL, RSSPECAN_ATTR_5G_NR_DL_GENERATOR_POWER_LEVEL, level_RMS),
                3, "Level RMS");

        viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, NULL, RSSPECAN_ATTR_5G_NR_DL_GENERATOR_LEVEL_DUT_LIMIT_ENABLED, dutLimitState),
                4, "DUT Limit State");

        if (dutLimitState == VI_TRUE)
        {
            viCheckParm(rsspecan_SetAttributeViReal64(instrSession, NULL, RSSPECAN_ATTR_5G_NR_DL_GENERATOR_LEVEL_DUT_LIMIT, dutLimitPeakInputPower),
                    5, "DUT Limit Peak Input Power");

            viCheckParm(rsspecan_SetAttributeViReal64(instrSession, NULL, RSSPECAN_ATTR_5G_NR_DL_GENERATOR_LEVEL_DUT_GAIN, dutGain),
                    6, "DUT Gain");
        }
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Uploads the RF settings (frequency, level) from analyzer to the
/// HIFN  generator.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsspecan_NR5GDLGeneratorUploadRFSettings(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsspecan_SetAttributeViString(instrSession, NULL, RSSPECAN_ATTR_5G_NR_DL_GENERATOR_UPLOAD_RF_SETTINGS, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Downloads the RF settings (frequency, level) from the generator to the
/// HIFN  analyzer.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsspecan_NR5GDLGeneratorDownloadRFSettings(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsspecan_SetAttributeViString(instrSession, NULL, RSSPECAN_ATTR_5G_NR_DL_GENERATOR_DOWNLOAD_RF_SETTINGS, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Downloads the NR signal description from the generator to the
/// HIFN  analyzer.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsspecan_NR5GDLGeneratorDownload5GNRSettings(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsspecan_SetAttributeViString(instrSession, NULL, RSSPECAN_ATTR_5G_NR_DL_GENERATOR_DOWNLOAD_NR5G_SETTINGS, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Configures the periodic synchronization of the signal description on
/// HIFN  the analyzer.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR syncEnabled/Turns periodic synchronization of the signal description on the
/// HIPAR syncEnabled/analyzer on and off.
/// HIPAR pollingInterval/Defines the polling interval for periodic synchronization between
/// HIPAR pollingInterval/analyzer and generator.
ViStatus _VI_FUNC rsspecan_Configure5GNRDLGeneratorSynchronization(
                            ViSession instrSession,
                            ViBoolean syncEnabled,
                            ViReal64  pollingInterval)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, NULL, RSSPECAN_ATTR_5G_NR_DL_GENERATOR_SETTINGS_PERIODIC_SYNCHRONIZATION_ENABLED, syncEnabled),
            2, "Sync Enabled");

    if (syncEnabled == VI_TRUE)
    {
        viCheckParm(rsspecan_SetAttributeViReal64(instrSession, NULL, RSSPECAN_ATTR_5G_NR_DL_GENERATOR_SETTINGS_PERIODIC_SYNCHRONIZATION_INTERVAL, pollingInterval),
                3, "Polling Interval");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the number of frames to analyze in on / off power
/// HIFN  measurements.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR numberofFramestoAnalyze/Defines the number of frames to analyze in on / off power
/// HIPAR numberofFramestoAnalyze/measurements.
ViStatus _VI_FUNC rsspecan_Configure5GNRDLPowerOnOffNumberOfFramesToAnalyze(
                            ViSession instrSession,
                            ViInt32   numberofFramestoAnalyze)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, NULL, RSSPECAN_ATTR_5G_NR_DL_ON_OFF_NUMBER_OF_FRAMES_TO_ANALYZE, numberofFramestoAnalyze),
            2, "Number of Frame to Analyze");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Includes or excludes ACLR, EVM anr SEM measurements from combined
/// HIFN  measurements.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR aclr/Includes or excludes ACLR measurements from combined measurements.
/// HIPAR evm/Includes or excludes EVM measurements from combined measurements.
/// HIPAR sem/Includes or excludes SEM measurements from combined measurements.
ViStatus _VI_FUNC rsspecan_Configure5GNRDLCombinedMeasurement(
                            ViSession instrSession,
                            ViBoolean aclr,
                            ViBoolean evm,
                            ViBoolean sem)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, NULL, RSSPECAN_ATTR_5G_NR_DL_COMBINED_MEASUREMENT_ACLR, aclr),
            2, "ACLR");

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, NULL, RSSPECAN_ATTR_5G_NR_DL_COMBINED_MEASUREMENT_EVM, evm),
            3, "EVM");

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, NULL, RSSPECAN_ATTR_5G_NR_DL_COMBINED_MEASUREMENT_SEM, sem),
            4, "SEM");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Loads an ACLR settings file for combined measurements.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR settingsFile/The path to the settings file (.xml format).
ViStatus _VI_FUNC rsspecan_Load5GNRDLCombinedMeasurementACLRSettings(
                            ViSession instrSession,
                            ViString  settingsFile)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViString(instrSession, NULL, RSSPECAN_ATTR_5G_NR_DL_COMBINED_MEASUREMENT_LOAD_ACLR_SETTINGS, settingsFile),
            2, "Settings File");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Removes the file based settings
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsspecan_Clear5GNRDLCombinedMeasurementACLRSettings(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsspecan_SetAttributeViString(instrSession, NULL, RSSPECAN_ATTR_5G_NR_DL_COMBINED_MEASUREMENT_CLEAR_ACLR_SETTINGS, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Loads an SEM settings file (.xml) for combined measurements.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR settingsFile/The path to the settings file (.xml format).
ViStatus _VI_FUNC rsspecan_Load5GNRDLCombinedMeasurementSEMSettings(
                            ViSession instrSession,
                            ViString  settingsFile)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViString(instrSession, NULL, RSSPECAN_ATTR_5G_NR_DL_COMBINED_MEASUREMENT_LOAD_SEM_SETTINGS, settingsFile),
            2, "Settings File");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Removes the file based settings.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsspecan_Clear5GNRDLCombinedMeasurementSEMSettings(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsspecan_SetAttributeViString(instrSession, NULL, RSSPECAN_ATTR_5G_NR_DL_COMBINED_MEASUREMENT_CLEAR_SEM_SETTINGS, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Configures the combined measurement signal capture settings.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR numberOfEvents/Defines the number of events in a combined measurement sequence.
/// HIPAR optimization/Selects the data capture method for combined measurements.
/// HIPAR sequenceAutoLevel/Turns auto leveling for each event in combined measurement sequence on
/// HIPAR sequenceAutoLevel/and off.
ViStatus _VI_FUNC rsspecan_Configure5GNRDLCombinedMeasurementSignalCapture(
                            ViSession instrSession,
                            ViInt32   numberOfEvents,
                            ViInt32   optimization,
                            ViBoolean sequenceAutoLevel)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, NULL, RSSPECAN_ATTR_5G_NR_DL_COMBINED_MEASUREMENT_NR_OF_EVENTS, numberOfEvents),
            2, "Number Of Events");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, NULL, RSSPECAN_ATTR_5G_NR_DL_COMBINED_MEASUREMENT_OPTIMIZATION, optimization),
            3, "Optimization");

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, NULL, RSSPECAN_ATTR_5G_NR_DL_COMBINED_MEASUREMENT_SEQUENCE_AUTO_LEVEL, sequenceAutoLevel),
            4, "Sequence Auto Level");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Select the type of measurement sequence for combined measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR sequence/Select the type of measurement sequence for combined measurement.
ViStatus _VI_FUNC rsspecan_Configure5GNRDLCombinedMeasurementTriggerSequence(
                            ViSession instrSession,
                            ViInt32   sequence)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, NULL, RSSPECAN_ATTR_5G_NR_DL_COMBINED_MEASUREMENT_TRIGGER_SEQUENCE, sequence),
            2, "Sequence");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Select the availability of a frame trigger in a combined measurement
/// HIFN  sequence.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR frameTrigger/Select the availability of a frame trigger in a combined measurement
/// HIPAR frameTrigger/sequence.
ViStatus _VI_FUNC rsspecan_Configure5GNRDLCombinedMeasurementFrameTrigger(
                            ViSession instrSession,
                            ViInt32   frameTrigger)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, NULL, RSSPECAN_ATTR_5G_NR_DL_COMBINED_MEASUREMENT_FRAME_TRIGGER, frameTrigger),
            2, "Frame Trigger");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Defines the event delay for combined measurements.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR eventDelay/Defines the event delay for combined measurements.
ViStatus _VI_FUNC rsspecan_Configure5GNRDLCombinedMeasurementTriggerEventDelay(
                            ViSession instrSession,
                            ViReal64  eventDelay)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, NULL, RSSPECAN_ATTR_5G_NR_DL_COMBINED_MEASUREMENT_TRIGGER_EVENT_DELAY, eventDelay),
            2, "Event Delay");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Defines the recalibration interval for combined measurements.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR recalibrationInterval/Defines the recalibration interval for combined measurements.
ViStatus _VI_FUNC rsspecan_Configure5GNRDLCombinedMeasurementTriggerRecalibrationInterval(
                            ViSession instrSession,
                            ViInt32   recalibrationInterval)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, NULL, RSSPECAN_ATTR_5G_NR_DL_COMBINED_MEASUREMENT_TRIGGER_RECALIBRATION_INTERVAL, recalibrationInterval),
            2, "Recalibration Interval");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Sends a trigger signal from the analyzer to the DUT.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsspecan_Send5GNRDLCombinedMeasurementTrigger3(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsspecan_SetAttributeViString(instrSession, NULL, RSSPECAN_ATTR_5G_NR_DL_COMBINED_MEASUREMENT_SEND_TRIGGER_3, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Turns auto gating on and off.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR autoGating/Turns auto gating on and off.
ViStatus _VI_FUNC rsspecan_Configure5GNRDLCombinedMeasurementTriggerAutoGating(
                            ViSession instrSession,
                            ViBoolean autoGating)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, NULL, RSSPECAN_ATTR_5G_NR_DL_COMBINED_MEASUREMENT_TRIGGER_AUTO_GATING, autoGating),
            2, "Auto Gatting");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configure the output power of the base station for transmit on / off
/// HIFN  power measurements on 2-O type base stations. Select Deployment
/// HIFN  Frequency to High with rsspecan_Configure5GNRDLCCPhysicalSettings.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR cc/Component Carrier parameter.
/// HIPAR eirp/Defines the base station output power for limit selection (P rated ,
/// HIPAR eirp/c, EIRP).
/// HIPAR trp/Defines the base station output power for limit selection (P rated ,
/// HIPAR trp/c, TRP).
ViStatus _VI_FUNC rsspecan_Configure5GNRDLPowerOnOffBaseStationOutputPower(
                            ViSession instrSession,
                            ViInt32   cc,
                            ViReal64  eirp,
                            ViReal64  trp)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_ON_OFF_BASE_STATION_OUTPUT_POWER_EIRP,
            "cc", cc - 1, RS_REPCAP_BUF_SIZE, repCap));

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_5G_NR_DL_ON_OFF_BASE_STATION_OUTPUT_POWER_EIRP, eirp),
            3, "EIRP");

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_5G_NR_DL_ON_OFF_BASE_STATION_OUTPUT_POWER_TRP, trp),
            4, "TRP");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects the period to display the rising or falling edge in the on /
/// HIFN  off power measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR period/Selects the period to display the rising or falling edge in the on /
/// HIPAR period/off power measurement.
ViStatus _VI_FUNC rsspecan_Configure5GNRDLPowerOnOffTransitionPeriod(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   period)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_ON_OFF_TRANSITION_PERIOD,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_5G_NR_DL_ON_OFF_TRANSITION_PERIOD, period),
            3, "Period");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Adjusts the timing for on / off power measurements.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsspecan_NR5GDLPowerOnOffAdjustTiming(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsspecan_SetAttributeViString(instrSession, NULL, RSSPECAN_ATTR_5G_NR_DL_ON_OFF_ADJUST_TIMING, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Turns noise correction for transmit on / off power measurements on and
/// HIFN  off.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR noiseCorrectionEnabled/Turns noise correction for transmit on / off power measurements on and
/// HIPAR noiseCorrectionEnabled/off.
ViStatus _VI_FUNC rsspecan_Configure5GNRDLPowerOnOffNoiseCorrection(
                            ViSession instrSession,
                            ViBoolean noiseCorrectionEnabled)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, NULL, RSSPECAN_ATTR_5G_NR_DL_ON_OFF_NOISE_CORRECTION_ENABLED, noiseCorrectionEnabled),
            2, "Noise Correction Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects the unit the OFF Power (On / Off Power measurements) is
/// HIFN  displayed in.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR unit/Selects the unit the OFF Power (On / Off Power measurements) is
/// HIPAR unit/displayed in.
ViStatus _VI_FUNC rsspecan_Configure5GNRDLPowerOnOffUnit(ViSession instrSession,
                                                         ViInt32   unit)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, NULL, RSSPECAN_ATTR_5G_NR_DL_ON_OFF_UNIT, unit),
            2, "Unit");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function adds a window to the display.
/// HIFN
/// HIFN     Note(s):
/// HIFN     (1) This function is only available for R&S FSW.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR stringSize/This control sets the allocated length of Returned Window Name string.
/// HIPAR referenceWindowName/String containing the number or name of the window the new window is
/// HIPAR referenceWindowName/inserted next to. In the default state, the name of the window is its
/// HIPAR referenceWindowName/index.
/// HIPAR direction/Direction the window is added relative to the window set with
/// HIPAR direction/"Reference Window Name".
/// HIPAR windowType/Type of result display you want to add.
/// HIPAR returnedWindowName/When adding a new window, the function returns its name (by default
/// HIPAR returnedWindowName/the same as its number) at the same time.
ViStatus _VI_FUNC rsspecan_NR5GDLAddWindow(ViSession instrSession,
                                           ViInt32   stringSize,
                                           ViString  referenceWindowName,
                                           ViInt32   direction,
                                           ViInt32   windowType,
                                           ViChar    returnedWindowName[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViChar   response[RS_MAX_MESSAGE_BUF_SIZE];
    ViChar   *p2direction;
    ViChar   *p2type;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &rsspecan_rngLayoutDirection, direction, &p2direction),
            4, "Direction");
    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &RsSpecAn_rng5GNRDLLayoutType, windowType, &p2type),
            5, "Window Type");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "LAY:ADD? '%s',%s,%s", referenceWindowName, p2direction, p2type);
    checkErr(RsCore_QueryViString(instrSession, cmd, response));

    if (returnedWindowName != NULL)
    {
        RsCore_TrimString(response, RS_VAL_TRIM_WHITESPACES_AND_ALL_QUOTES);
        RsCore_CopyToUserBufferAsciiData(returnedWindowName, stringSize, response);
    }

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function replaces a window with another one.
/// HIFN
/// HIFN     Note(s):
/// HIFN     (1) This function is only available for R&S FSW.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR windowName/String containing the number or name of the window you want to
/// HIPAR windowType/Type of result display you want to replace.
ViStatus _VI_FUNC rsspecan_NR5GDLReplaceWindow(ViSession instrSession,
                                               ViString  windowName,
                                               ViInt32   windowType)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViChar   *p2type;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &RsSpecAn_rng5GNRDLLayoutType, windowType, &p2type),
            3, "Window Type");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "LAY:REPL '%s',%s", windowName, p2type);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function selects the way the contents of the result summary are
/// HIFN  calculated.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR summary/Selects the way the contents of the result summary are calculated.
ViStatus _VI_FUNC rsspecan_Configure5GNRDLAnalysisShowResultSummary(
                            ViSession instrSession,
                            ViInt32   summary)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_SHOW_RESULT_SUMMARY, summary),
            2, "Summary");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Turns the display of individual results in the numerical result
/// HIFN  summary on and off.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR item/Selects the item to be included in Result Summary.
/// HIPAR displayItem/This parameter selects whether the results summary item is displayed.
ViStatus _VI_FUNC rsspecan_Configure5GNRDLAnalysisShowResultSummaryItems(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   item,
                            ViBoolean displayItem)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";
    ViChar   *p2string;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentModel(instrSession, "FSW"));

    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &RsSpecAn_rng5GNRResultSummaryItems, item, &p2string),
            3, "Item");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "DISP:WIND%ld:TABL:ITEM %s,%s", window, p2string, offOnArr[displayItem]);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects the way multiple carriers are analyzed.
/// HIFN     Prerequisites for this attribute:
/// HIFN     Select mutiple carriers
/// HIFN  (RSSPECAN_ATTR_5G_NR_DL_COMPONENT_CARRIER_NUMBER).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR ccResult/Selects the way multiple carriers are analyzed.
ViStatus _VI_FUNC rsspecan_Configure5GNRDLAnalysisCCResult(ViSession instrSession,
                                                           ViInt32   ccResult)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, NULL, RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_CC_RESULT, ccResult),
            2, "CC Result");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function selects the scale of the x-axis for result displays that
/// HIFN  show symbol results.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR xScale/Selects the scale of the x-axis for result displays that show symbol
/// HIPAR xScale/results.
ViStatus _VI_FUNC rsspecan_Configure5GNRDLAnalysisXScale(ViSession instrSession,
                                                         ViInt32   xScale)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_X_SCALE, xScale),
            2, "X Scale");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the carrier axes.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR carrierAxes/This control selects the scale of the x-axis for result displays that
/// HIPAR carrierAxes/show subcarrier results.
/// HIPAR carrierAxesReference/Selects the reference for result displays whose x-axis shows frequency
/// HIPAR carrierAxesReference/characteristics of the signal.
ViStatus _VI_FUNC rsspecan_Configure5GNRDLAnalysisCarrierAxes(
                            ViSession instrSession,
                            ViInt32   carrierAxes,
                            ViInt32   carrierAxesReference)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, NULL, RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_CARRIER_AXES, carrierAxes),
            2, "Carrier Axes");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, NULL, RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_CARRIER_AXES_REFERENCE, carrierAxesReference),
            3, "Carrier Axes Reference");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the EVM max hold function on and off.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR evmMaxHold/Turns the EVM max hold function on and off.
ViStatus _VI_FUNC rsspecan_Configure5GNRDLAnalysisEVMMaxHold(ViSession instrSession,
                                                             ViBoolean evmMaxHold)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, NULL, RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_EVM_MAX_HOLD, evmMaxHold),
            2, "EVM Max Hold");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Turns the 3D view for selected diagrams on and off.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR 3dView/Turns the 3D view for selected diagrams on and off.
ViStatus _VI_FUNC rsspecan_Configure5GNRDLAnalysis3DView(ViSession instrSession,
                                                         ViBoolean view)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, NULL, RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_3D_VIEW, view),
            2, "3D View");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Selects the limit evaluation mode for ACLR measurements in combined
/// HIFN  measurement mode.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR aclrLimitPassMode/Selects the limit evaluation mode for ACLR measurements in combined
/// HIPAR aclrLimitPassMode/measurement mode.
ViStatus _VI_FUNC rsspecan_Configure5GNRDLAnalysisACLRLimitPassMode(
                            ViSession instrSession,
                            ViInt32   aclrLimitPassMode)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, NULL, RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_ACLR_LIMIT_PASS_MODE, aclrLimitPassMode),
            2, "");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function queries the current synchronization state.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR cc/Component Carrier parameter.
/// HIPAR state/Queries the current synchronization state.
ViStatus _VI_FUNC rsspecan_Query5GNRDLAnalysisSynchronizationState(
                            ViSession instrSession,
                            ViInt32   cc,
                            ViChar    state[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_SYNCHRONIZATION_STATE,
            "cc", cc - 1, RS_REPCAP_BUF_SIZE, repCap));

    viCheckParm(rsspecan_GetAttributeViString(instrSession, repCap, RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_SYNCHRONIZATION_STATE,
            512, state),
            3, "State");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function queries the cell ID.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR cc/Component Carrier parameter.
/// HIPAR cellID/Queries the cell ID.
ViStatus _VI_FUNC rsspecan_Query5GNRDLAnalysisCellID(ViSession instrSession,
                                                     ViInt32   cc,
                                                     ViInt32   *cellID)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_CELL_ID,
            "cc", cc - 1, RS_REPCAP_BUF_SIZE, repCap));

    viCheckParm(rsspecan_GetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_CELL_ID, cellID),
            3, "Cell ID");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function filters the displayed results in the constellation
/// HIFN  diagram by a certain allocation.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR mode/Filters the displayed results in the constellation diagram by a
/// HIPAR mode/certain allocation.
/// HIPAR value/Filters the displayed results in the constellation diagram by a
/// HIPAR value/certain allocation.
ViStatus _VI_FUNC rsspecan_Configure5GNRDLAnalysisEvaluationRangeAllocation(
                            ViSession instrSession,
                            ViInt32   mode,
                            ViInt32   value)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, mode, 0, 1),
            2, "Mode");

    if (mode == RSSPECAN_VAL_5G_NR_ANALYSIS_MODE_ALL)
    {
        checkErr(rsspecan_SetAttributeViString(instrSession, NULL, RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_EVALUATION_RANGE_ALLOCATION_ALL, NULL));
    }
    else
    {
        viCheckParm(rsspecan_SetAttributeViInt32(instrSession, NULL, RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_EVALUATION_RANGE_ALLOCATION, value),
                3, "Value");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function filters the displayed results in the constellation
/// HIFN  diagram by a certain allocation.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR mode/Filters the displayed results by a certain bandwidth part.
/// HIPAR value/Filters the displayed results by a certain bandwidth part.
ViStatus _VI_FUNC rsspecan_Configure5GNRDLAnalysisEvaluationRangeBandwidthPart(
                            ViSession instrSession,
                            ViInt32   mode,
                            ViInt32   value)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, mode, RSSPECAN_VAL_5G_NR_ANALYSIS_MODE_ALL, RSSPECAN_VAL_5G_NR_ANALYSIS_MODE_SS_PBCH_BLOCK),
            2, "Mode");

    if (mode == RSSPECAN_VAL_5G_NR_ANALYSIS_MODE_ALL)
    {
        checkErr(rsspecan_SetAttributeViString(instrSession, NULL, RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_EVALUATION_RANGE_BANDWIDTH_PART_ALL, NULL));
    }
    else if (mode == RSSPECAN_VAL_5G_NR_ANALYSIS_MODE_VAL)
    {
        viCheckParm(rsspecan_SetAttributeViInt32(instrSession, NULL, RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_EVALUATION_RANGE_BANDWIDTH_PART, value),
                3, "Value");
    }
    else
    {
        checkErr(rsspecan_SetAttributeViString(instrSession, NULL, RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_EVALUATION_RANGE_BANDWIDTH_PART_SS_PBCH_BLOCK, NULL));
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function filters the displayed results in the constellation
/// HIFN  diagram by a certain subcarrier.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR mode/Filters the displayed results in the constellation diagram by a
/// HIPAR mode/certain subcarrier.
/// HIPAR value/Filters the displayed results in the constellation diagram by a
/// HIPAR value/certain subcarrier.
ViStatus _VI_FUNC rsspecan_Configure5GNRDLAnalysisEvaluationRangeCarrier(
                            ViSession instrSession,
                            ViInt32   mode,
                            ViInt32   value)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, mode, 0, 1),
            2, "Mode");

    if (mode == RSSPECAN_VAL_5G_NR_ANALYSIS_MODE_ALL)
    {
        checkErr(rsspecan_SetAttributeViString(instrSession, NULL, RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_EVALUATION_RANGE_CARRIER_ALL, NULL));
    }
    else
    {
        viCheckParm(rsspecan_SetAttributeViInt32(instrSession, NULL, RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_EVALUATION_RANGE_CARRIER, value),
                3, "Value");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function filters the displayed results by a specific frame.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR mode/Filters the displayed results by a specific frame.
/// HIPAR value/Filters the displayed results by a specific frame.
ViStatus _VI_FUNC rsspecan_Configure5GNRDLAnalysisEvaluationRangeFrame(
                            ViSession instrSession,
                            ViInt32   value)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, NULL, RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_EVALUATION_RANGE_FRAME, value),
            3, "Value");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects a specific measurement (meas ID) from the measurement sequence
/// HIFN  in combined measurements.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR selectedMeasID/Selects a specific measurement (meas ID) from the measurement sequence
/// HIPAR selectedMeasID/in combined measurements.
ViStatus _VI_FUNC rsspecan_Configure5GNRDLAnalysisEvaluationRangeSelectedMeasID(
                            ViSession instrSession,
                            ViInt32   selectedMeasID)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, NULL, RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_EVALUATION_RANGE_SELECTED_MEAS_ID, selectedMeasID),
            2, "Selected Meas ID");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function filters the displayed results in the constellation
/// HIFN  diagram by a certain modulation type.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR mode/Filters the displayed results in the constellation diagram by a
/// HIPAR mode/certain modulation type.
/// HIPAR value/Filters the displayed results in the constellation diagram by a
/// HIPAR value/certain modulation type.
ViStatus _VI_FUNC rsspecan_Configure5GNRDLAnalysisEvaluationRangeModulation(
                            ViSession instrSession,
                            ViInt32   mode,
                            ViInt32   value)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, mode, 0, 1),
            2, "Mode");

    if (mode == RSSPECAN_VAL_5G_NR_ANALYSIS_MODE_ALL)
    {
        checkErr(rsspecan_SetAttributeViString(instrSession, NULL, RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_EVALUATION_RANGE_MODULATION_ALL, NULL));
    }
    else
    {
        viCheckParm(rsspecan_SetAttributeViInt32(instrSession, NULL, RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_EVALUATION_RANGE_MODULATION, value),
                3, "Value");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function filters the displayed results by a certain slot.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR mode/Filters the displayed results by a certain slot.
/// HIPAR value/Filters the displayed results by a certain slot.
ViStatus _VI_FUNC rsspecan_Configure5GNRDLAnalysisEvaluationRangeSlot(
                            ViSession instrSession,
                            ViInt32   mode,
                            ViInt32   value)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, mode, 0, 1),
            2, "Mode");

    if (mode == RSSPECAN_VAL_5G_NR_ANALYSIS_MODE_ALL)
    {
        checkErr(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_EVALUATION_RANGE_SLOT_ALL, NULL));
    }
    else
    {
        viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_EVALUATION_RANGE_SLOT, value),
                3, "Value");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function filters the displayed results by a certain OFDM
/// HIFN  subframe.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR mode/Filters the displayed results by a certain OFDM subframe.
/// HIPAR value/Filters the displayed results by a certain OFDM subframe.
ViStatus _VI_FUNC rsspecan_Configure5GNRDLAnalysisEvaluationRangeSubframe(
                            ViSession instrSession,
                            ViInt32   mode,
                            ViInt32   value)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, mode, 0, 1),
            2, "Mode");

    if (mode == RSSPECAN_VAL_5G_NR_ANALYSIS_MODE_ALL)
    {
        checkErr(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_EVALUATION_RANGE_SUBFRAME_ALL, NULL));
    }
    else
    {
        viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_EVALUATION_RANGE_SUBFRAME, value),
                3, "Value");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function filters the displayed results in the constellation
/// HIFN  diagram by a certain OFDM symbol.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR mode/Filters the displayed results in the constellation diagram by a
/// HIPAR mode/certain OFDM symbol.
/// HIPAR value/Filters the displayed results in the constellation diagram by a
/// HIPAR value/certain OFDM symbol.
ViStatus _VI_FUNC rsspecan_Configure5GNRDLAnalysisEvaluationRangeSymbol(
                            ViSession instrSession,
                            ViInt32   mode,
                            ViInt32   value)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, mode, 0, 1),
            2, "Mode");

    if (mode == RSSPECAN_VAL_5G_NR_ANALYSIS_MODE_ALL)
    {
        checkErr(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_EVALUATION_RANGE_SYMBOL_ALL, NULL));
    }
    else
    {
        viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_EVALUATION_RANGE_SYMBOL, value),
                3, "Value");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects the analyzed segment for a segmented capture.
/// HIFN     Prerequisites for this attribute
/// HIFN     Turn on segmented capture
/// HIFN  (RSSPECAN_ATTR_5G_NR_DL_SEGMENTED_CAPTURE_ENABLED).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR value/Selects the analyzed segment for a segmented capture.
ViStatus _VI_FUNC rsspecan_Configure5GNRDLAnalysisEvaluationRangeSegment(
                            ViSession instrSession,
                            ViInt32   value)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, NULL, RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_EVALUATION_RANGE_SEGMENT, value),
            2, "Value");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects the reference antenna port for relative beamforming results.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR value/Selects the reference antenna port for relative beamforming results.
ViStatus _VI_FUNC rsspecan_Configure5GNRDLAnalysisEvaluationRangeBeamformingReferenceAP(
                            ViSession instrSession,
                            ViInt32   value)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, NULL, RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_EVALUATION_RANGE_BEAMFORMING_REFERENCE_AP, value),
            2, "Value");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects an antenna port to analyze.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR cc/Component Carrier parameter.
/// HIPAR mode/Selects the mode.
/// HIPAR value/Selects an antenna port to analyze.
ViStatus _VI_FUNC rsspecan_Configure5GNRDLAnalysisEvaluationRangeBeamformingAntennaPort(
                            ViSession instrSession,
                            ViInt32   cc,
                            ViInt32   mode,
                            ViInt32   value)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_EVALUATION_RANGE_BEAMFORMING_ANTENNA_PORT_ALL,
            "cc", cc - 1, RS_REPCAP_BUF_SIZE, repCap));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, mode, RSSPECAN_VAL_5G_NR_ANALYSIS_MODE_ALL, RSSPECAN_VAL_5G_NR_ANALYSIS_MODE_VAL),
            3, "Mode");

    if (mode == RSSPECAN_VAL_5G_NR_ANALYSIS_MODE_ALL)
    {
        checkErr(rsspecan_SetAttributeViString(instrSession, repCap, RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_EVALUATION_RANGE_BEAMFORMING_ANTENNA_PORT_ALL, NULL));
    }
    else
    {
        viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_EVALUATION_RANGE_BEAMFORMING_ANTENNA_PORT, value),
                4, "Value");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the EVM of selected resource elements. Queries the average
/// HIFN  result over all carriers
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR type/Selects the type of measurement.
/// HIPAR result/Queries the EVM of selected resource elements. Queries the average
/// HIPAR result/result over all carriers
ViStatus _VI_FUNC rsspecan_Query5GNRDLAnalysisAllCarriers(ViSession instrSession,
                                                          ViInt32   type,
                                                          ViReal64* result)
{
    ViStatus error = VI_SUCCESS;
    ViAttr   attr = 0;

    checkErr(RsCore_LockSession(instrSession));

    switch (type)
    {
        case RSSPECAN_VAL_SUMMARY_ALL:
            attr = RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_SUMMARY_ALL_CARRIERS_EVM;
            break;
        case RSSPECAN_VAL_SUMMARY_PEAK:
            attr = RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_SUMMARY_ALL_CARRIERS_EVM_PEAK;
            break;
        case RSSPECAN_VAL_SUMMARY_PHYSICAL_CHANNEL:
            attr = RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_SUMMARY_ALL_CARRIERS_EVM_PHYSICAL_CHANNEL;
            break;
        case RSSPECAN_VAL_SUMMARY_PHYSICAL_SIGNAL:
            attr = RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_SUMMARY_ALL_CARRIERS_EVM_PHYSICAL_SIGNAL;
            break;
        case RSSPECAN_VAL_SUMMARY_POWER:
            attr = RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_SUMMARY_ALL_CARRIERS_SIGNAL_POWER;
            break;
        case RSSPECAN_VAL_SUMMARY_PDSCH_QPSK:
            attr = RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_SUMMARY_ALL_CARRIERS_EVM_PDSCH_WITH_QPSK;
            break;
        case RSSPECAN_VAL_SUMMARY_PDSCH_64QAM:
            attr = RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_SUMMARY_ALL_CARRIERS_EVM_PDSCH_WITH_64QAM;
            break;
        case RSSPECAN_VAL_SUMMARY_PDSCH_16QAM:
            attr = RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_SUMMARY_ALL_CARRIERS_EVM_PDSCH_WITH_16QAM;
            break;
        case RSSPECAN_VAL_SUMMARY_PDSCH_256QAM:
            attr = RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_SUMMARY_ALL_CARRIERS_EVM_PDSCH_WITH_256QAM;
            break;
        case RSSPECAN_VAL_SUMMARY_PDSCH_1024QAM:
            attr = RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_SUMMARY_ALL_CARRIERS_EVM_PDSCH_WITH_1024QAM;
            break;
        default:
            checkErr(RsCore_InvalidViInt32Value(instrSession, type));
            break;
    }

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, NULL, attr, result),
            3, "Result");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function queries the EVM of all resource element.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR cc/Component Carrier parameter.
/// HIPAR frame/Sets the Frame parameter.
/// HIPAR all/Queries the EVM of all resource element.
ViStatus _VI_FUNC rsspecan_Query5GNRDLAnalysisEVMAll(ViSession instrSession,
                                                     ViInt32   cc,
                                                     ViInt32   frame,
                                                     ViReal64  *all)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_frame[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_SUMMARY_EVM_ALL,
            "cc", cc - 1, RS_REPCAP_BUF_SIZE, repCap));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_SUMMARY_EVM_ALL,
            "NR5GFrame", frame - 1, RS_REPCAP_BUF_SIZE, repCap_frame));

    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_frame, NULL);

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_SUMMARY_EVM_ALL, all),
            4, "All");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function queries the EVM of all resource element.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR cc/Component Carrier parameter.
/// HIPAR frame/Sets the Frame parameter.
/// HIPAR measurementType/Selects measurement type.
/// HIPAR result/Queries the EVM of all resource element.
ViStatus _VI_FUNC rsspecan_Query5GNRDLAnalysisEVMAllStatistics(
                            ViSession instrSession,
                            ViInt32   cc,
                            ViInt32   frame,
                            ViInt32   measurementType,
                            ViReal64* result)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_frame[RS_REPCAP_BUF_SIZE] = "";
    ViAttr   attr[] = {
             RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_SUMMARY_EVM_ALL_MINIMUM,
             RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_SUMMARY_EVM_ALL_MAXIMUM,
             0,
             RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_SUMMARY_EVM_ALL
             };

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, measurementType, RSSPECAN_VAL_MEASTYPE_MIN, RSSPECAN_VAL_MEASTYPE_AVER),
            4, "Measurement Type");

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_SUMMARY_EVM_ALL,
            "cc", cc - 1, RS_REPCAP_BUF_SIZE, repCap));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_SUMMARY_EVM_ALL,
            "NR5GFrame", frame - 1, RS_REPCAP_BUF_SIZE, repCap_frame));

    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_frame, NULL);

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, repCap, attr[measurementType - 1], result),
            5, "Result");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function queries the EVM of all PDSCH elements.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR cc/Component Carrier parameter.
/// HIPAR frame/Sets the Frame parameter.
/// HIPAR modulation/Selects the modulation.
/// HIPAR result/Queries the EVM of all PDSCH resource elements with a selected
/// HIPAR result/modulation.
ViStatus _VI_FUNC rsspecan_Query5GNRDLAnalysisEVMPDSCH(ViSession instrSession,
                                                       ViInt32   cc,
                                                       ViInt32   frame,
                                                       ViInt32   modulation,
                                                       ViReal64  *result)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_frame[RS_REPCAP_BUF_SIZE] = "";
    ViAttr   attr[] = {
             RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_SUMMARY_EVM_ALL_PDSCH_WITH_QPSK,
             RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_SUMMARY_EVM_ALL_PDSCH_WITH_16QAM,
             RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_SUMMARY_EVM_ALL_PDSCH_WITH_64QAM,
             RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_SUMMARY_EVM_ALL_PDSCH_WITH_256QAM
             };

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, modulation, RSSPECAN_VAL_5GNR_MODULATION_QPSK, RSSPECAN_VAL_5GNR_MODULATION_256QAM),
            4, "Modulation");

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_SUMMARY_EVM_ALL_PDSCH_WITH_QPSK,
            "cc", cc - 1, RS_REPCAP_BUF_SIZE, repCap));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_SUMMARY_EVM_ALL_PDSCH_WITH_QPSK,
            "NR5GFrame", frame - 1, RS_REPCAP_BUF_SIZE, repCap_frame));

    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_frame, NULL);

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, repCap, attr[modulation], result),
            5, "Result");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function queries the EVM of all PDSCH elements.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR cc/Component Carrier parameter.
/// HIPAR frame/Sets the Frame parameter.
/// HIPAR modulation/Selects the modulation.
/// HIPAR measurementType/Selects measurement type.
/// HIPAR result/Queries the EVM of all PDSCH resource elements with a selected
/// HIPAR result/modulation.
ViStatus _VI_FUNC rsspecan_Query5GNRDLAnalysisEVMPDSCHStatistics(
                            ViSession instrSession,
                            ViInt32   cc,
                            ViInt32   frame,
                            ViInt32   modulation,
                            ViInt32   measurementType,
                            ViReal64* result)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_frame[RS_REPCAP_BUF_SIZE] = "";
    ViAttr   attr[4][5] = {{
             RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_SUMMARY_EVM_ALL_PDSCH_WITH_QPSK_MINIMUM,
             RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_SUMMARY_EVM_ALL_PDSCH_WITH_16QAM_MINIMUM,
             RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_SUMMARY_EVM_ALL_PDSCH_WITH_64QAM_MINIMUM,
             RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_SUMMARY_EVM_ALL_PDSCH_WITH_256QAM_MINIMUM,
             RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_SUMMARY_EVM_ALL_PDSCH_WITH_1024QAM_MINIMUM
             },
             {
             RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_SUMMARY_EVM_ALL_PDSCH_WITH_QPSK_MAXIMUM,
             RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_SUMMARY_EVM_ALL_PDSCH_WITH_16QAM_MAXIMUM,
             RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_SUMMARY_EVM_ALL_PDSCH_WITH_64QAM_MAXIMUM,
             RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_SUMMARY_EVM_ALL_PDSCH_WITH_256QAM_MAXIMUM,
             RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_SUMMARY_EVM_ALL_PDSCH_WITH_1024QAM_MAXIMUM
             },{0,0,0,0},
             {
             RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_SUMMARY_EVM_ALL_PDSCH_WITH_QPSK,
             RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_SUMMARY_EVM_ALL_PDSCH_WITH_16QAM,
             RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_SUMMARY_EVM_ALL_PDSCH_WITH_64QAM,
             RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_SUMMARY_EVM_ALL_PDSCH_WITH_256QAM,
             RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_SUMMARY_EVM_ALL_PDSCH_WITH_1024QAM
             }};

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, modulation, RSSPECAN_VAL_5GNR_MODULATION_QPSK, RSSPECAN_VAL_5GNR_MODULATION_1024QAM),
            4, "Modulation");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, measurementType, RSSPECAN_VAL_MEASTYPE_MIN, RSSPECAN_VAL_MEASTYPE_AVER),
            5, "Measurement Type");

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_SUMMARY_EVM_ALL_PDSCH_WITH_QPSK,
            "cc", cc - 1, RS_REPCAP_BUF_SIZE, repCap));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_SUMMARY_EVM_ALL_PDSCH_WITH_QPSK,
            "NR5GFrame", frame - 1, RS_REPCAP_BUF_SIZE, repCap_frame));

    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_frame, NULL);

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, repCap, attr[measurementType-1][modulation], result),
            6, "Result");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the limit specified by 3GPP for the EVM of all PDSCH resource
/// HIFN  elements with selected modulation.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR cc/Component Carrier parameter.
/// HIPAR frame/Sets the Frame parameter.
/// HIPAR modulation/Selects the modulation.
/// HIPAR measurementType/Selects measurement type.
/// HIPAR result/Queries the limit specified by 3GPP for the EVM of all PDSCH resource
/// HIPAR result/elements with selected modulation.
ViStatus _VI_FUNC rsspecan_Query5GNRDLAnalysisEVMPDSCHLimitCheckResult(
                            ViSession instrSession,
                            ViInt32   cc,
                            ViInt32   frame,
                            ViInt32   modulation,
                            ViInt32   measurementType,
                            ViInt32*  result)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_frame[RS_REPCAP_BUF_SIZE] = "";
    ViAttr   attr[4][4] = {
             {0,0,0,0},
             {0,0,0,0},
             {0,0,0,0},
             {
             RSSPECAN_ATTR_5G_NR_DL_EVM_PDSCH_WITH_QPSK_LIMIT_CHECK_RESULT_AVERAGE,
             RSSPECAN_ATTR_5G_NR_DL_EVM_PDSCH_WITH_16QAM_LIMIT_CHECK_RESULT_AVERAGE,
             RSSPECAN_ATTR_5G_NR_DL_EVM_PDSCH_WITH_64QAM_LIMIT_CHECK_RESULT_AVERAGE,
             RSSPECAN_ATTR_5G_NR_DL_EVM_PDSCH_WITH_256QAM_LIMIT_CHECK_RESULT_AVERAGE
             }};

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, modulation, RSSPECAN_VAL_5GNR_MODULATION_QPSK, RSSPECAN_VAL_5GNR_MODULATION_256QAM),
            4, "Modulation");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, measurementType, RSSPECAN_VAL_MEASTYPE_AVER, RSSPECAN_VAL_MEASTYPE_AVER),
            5, "Measurement Type");

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_SUMMARY_EVM_ALL_PDSCH_WITH_QPSK,
            "cc", cc - 1, RS_REPCAP_BUF_SIZE, repCap));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_SUMMARY_EVM_ALL_PDSCH_WITH_QPSK,
            "NR5GFrame", frame - 1, RS_REPCAP_BUF_SIZE, repCap_frame));

    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_frame, NULL);

    viCheckParm(rsspecan_GetAttributeViInt32(instrSession, repCap, attr[measurementType-1][modulation], result),
            6, "Result");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the peak EVM.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR cc/Component Carrier parameter.
/// HIPAR frame/Sets the Frame parameter.
/// HIPAR measurementType/Selects measurement type.
/// HIPAR result/Queries the peak EVM.
ViStatus _VI_FUNC rsspecan_Query5GNRDLAnalysisEVMPeak(ViSession instrSession,
                                                      ViInt32   cc,
                                                      ViInt32   frame,
                                                      ViInt32   measurementType,
                                                      ViReal64* result)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_frame[RS_REPCAP_BUF_SIZE] = "";
    ViAttr   attr[] = {
             RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_SUMMARY_EVM_PEAK_MINIMUM,
             RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_SUMMARY_EVM_PEAK_MAXIMUM,
             0,
             RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_SUMMARY_EVM_PEAK
             };

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, measurementType, RSSPECAN_VAL_MEASTYPE_MIN, RSSPECAN_VAL_MEASTYPE_AVER),
            4, "Measurement Type");

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_SUMMARY_EVM_PEAK,
            "cc", cc - 1, RS_REPCAP_BUF_SIZE, repCap));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_SUMMARY_EVM_PEAK,
            "NR5GFrame", frame - 1, RS_REPCAP_BUF_SIZE, repCap_frame));

    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_frame, NULL);

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, repCap, attr[measurementType - 1], result),
            5, "Result");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function queries the EVM of Physical elements.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR cc/Component Carrier parameter.
/// HIPAR frame/Sets the Frame parameter.
/// HIPAR channel/Queries the EVM of the physical channel.
/// HIPAR signal/Queries the EVM of the physical signal.
ViStatus _VI_FUNC rsspecan_Query5GNRDLAnalysisEVMPhysical(ViSession instrSession,
                                                          ViInt32   cc,
                                                          ViInt32   frame,
                                                          ViReal64  *channel,
                                                          ViReal64  *signal)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_frame[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_SUMMARY_EVM_PHYSICAL_CHANNEL,
            "cc", cc - 1, RS_REPCAP_BUF_SIZE, repCap));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_SUMMARY_EVM_PHYSICAL_CHANNEL,
            "NR5GFrame", frame - 1, RS_REPCAP_BUF_SIZE, repCap_frame));

    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_frame, NULL);

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_SUMMARY_EVM_PHYSICAL_CHANNEL, channel),
            4, "Channel");

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_SUMMARY_EVM_PHYSICAL_SIGNAL, signal),
            5, "Signal");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function queries the EVM of Physical elements.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR cc/Component Carrier parameter.
/// HIPAR frame/Sets the Frame parameter.
/// HIPAR measurementType/Selects measurement type.
/// HIPAR channel/Queries the EVM of the physical channel.
/// HIPAR signal/Queries the EVM of the physical signal.
ViStatus _VI_FUNC rsspecan_Query5GNRDLAnalysisEVMPhysicalStatistics(
                            ViSession instrSession,
                            ViInt32   cc,
                            ViInt32   frame,
                            ViInt32   measurementType,
                            ViReal64* channel,
                            ViReal64* signal)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_frame[RS_REPCAP_BUF_SIZE] = "";
    ViAttr   attr_channel[] = {
             RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_SUMMARY_EVM_PHYSICAL_CHANNEL_MINIMUM,
             RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_SUMMARY_EVM_PHYSICAL_CHANNEL_MAXIMUM,
             0,
             RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_SUMMARY_EVM_PHYSICAL_CHANNEL
             };
    ViAttr   attr_signal[] = {
             RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_SUMMARY_EVM_PHYSICAL_SIGNAL_MINIMUM,
             RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_SUMMARY_EVM_PHYSICAL_SIGNAL_MAXIMUM,
             0,
             RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_SUMMARY_EVM_PHYSICAL_SIGNAL
             };

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, measurementType, RSSPECAN_VAL_MEASTYPE_MIN, RSSPECAN_VAL_MEASTYPE_AVER),
            4, "Measurement Type");

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_SUMMARY_EVM_PHYSICAL_CHANNEL,
            "cc", cc - 1, RS_REPCAP_BUF_SIZE, repCap));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_SUMMARY_EVM_PHYSICAL_CHANNEL,
            "NR5GFrame", frame - 1, RS_REPCAP_BUF_SIZE, repCap_frame));

    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_frame, NULL);

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, repCap, attr_channel[measurementType - 1], channel),
            5, "Channel");

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, repCap, attr_signal[measurementType - 1], signal),
            6, "Signal");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function queries the total signal power.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR cc/Component Carrier parameter.
/// HIPAR frame/Sets the Frame parameter.
/// HIPAR power/Queries the total signal power.
ViStatus _VI_FUNC rsspecan_Query5GNRDLAnalysisEVMSignalPower(ViSession instrSession,
                                                             ViInt32   cc,
                                                             ViInt32   frame,
                                                             ViReal64  *power)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_frame[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_SUMMARY_SIGNAL_POWER,
            "cc", cc - 1, RS_REPCAP_BUF_SIZE, repCap));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_SUMMARY_SIGNAL_POWER,
            "NR5GFrame", frame - 1, RS_REPCAP_BUF_SIZE, repCap_frame));

    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_frame, NULL);

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_SUMMARY_SIGNAL_POWER, power),
            4, "Power");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function queries the total signal power.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR cc/Component Carrier parameter.
/// HIPAR frame/Sets the Frame parameter.
/// HIPAR measurementType/Selects measurement type.
/// HIPAR power/Queries the total signal power.
ViStatus _VI_FUNC rsspecan_Query5GNRDLAnalysisEVMSignalPowerStatistics(
                            ViSession instrSession,
                            ViInt32   cc,
                            ViInt32   frame,
                            ViInt32   measurementType,
                            ViReal64* power)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_frame[RS_REPCAP_BUF_SIZE] = "";
    ViAttr   attr[] = {
             RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_SUMMARY_SIGNAL_POWER_MINIMUM,
             RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_SUMMARY_SIGNAL_POWER_MAXIMUM,
             0,
             RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_SUMMARY_SIGNAL_POWER
             };

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, measurementType, RSSPECAN_VAL_MEASTYPE_MIN, RSSPECAN_VAL_MEASTYPE_AVER),
            4, "Measurement Type");

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_SUMMARY_SIGNAL_POWER,
            "cc", cc - 1, RS_REPCAP_BUF_SIZE, repCap));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_SUMMARY_SIGNAL_POWER,
            "NR5GFrame", frame - 1, RS_REPCAP_BUF_SIZE, repCap_frame));

    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_frame, NULL);

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, repCap, attr[measurementType - 1], power),
            5, "Power");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function queries the frequency error.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR cc/Component Carrier parameter.
/// HIPAR frame/Sets the Frame parameter.
/// HIPAR frequencyError/Queries the frequency error.
ViStatus _VI_FUNC rsspecan_Query5GNRDLAnalysisEVMFrequencyError(
                            ViSession instrSession,
                            ViInt32   cc,
                            ViInt32   frame,
                            ViReal64  *frequencyError)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_frame[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_SUMMARY_FREQUENCY_ERROR,
            "cc", cc - 1, RS_REPCAP_BUF_SIZE, repCap));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_SUMMARY_FREQUENCY_ERROR,
            "NR5GFrame", frame - 1, RS_REPCAP_BUF_SIZE, repCap_frame));

    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_frame, NULL);

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_SUMMARY_FREQUENCY_ERROR, frequencyError),
            4, "Frequency Error");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function queries the frequency error.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR cc/Component Carrier parameter.
/// HIPAR frame/Sets the Frame parameter.
/// HIPAR measurementType/Selects measurement type.
/// HIPAR frequencyError/Queries the frequency error.
ViStatus _VI_FUNC rsspecan_Query5GNRDLAnalysisEVMFrequencyErrorStatistics(
                            ViSession instrSession,
                            ViInt32   cc,
                            ViInt32   frame,
                            ViInt32   measurementType,
                            ViReal64* frequencyError)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_frame[RS_REPCAP_BUF_SIZE] = "";
    ViAttr   attr[] = {
             RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_SUMMARY_FREQUENCY_ERROR_MINIMUM,
             RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_SUMMARY_FREQUENCY_ERROR_MAXIMUM,
             0,
             RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_SUMMARY_FREQUENCY_ERROR
             };

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, measurementType, RSSPECAN_VAL_MEASTYPE_MIN, RSSPECAN_VAL_MEASTYPE_AVER),
            4, "Measurement Type");

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_SUMMARY_FREQUENCY_ERROR,
            "cc", cc - 1, RS_REPCAP_BUF_SIZE, repCap));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_SUMMARY_FREQUENCY_ERROR,
            "NR5GFrame", frame - 1, RS_REPCAP_BUF_SIZE, repCap_frame));

    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_frame, NULL);

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, repCap, attr[measurementType - 1], frequencyError),
            5, "Frequency Error");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the limit specified by 3GPP for the frequency error.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR cc/Component Carrier parameter.
/// HIPAR frame/Sets the Frame parameter.
/// HIPAR measurementType/Selects measurement type.
/// HIPAR result/Queries the limit specified by 3GPP for the frequency error.
ViStatus _VI_FUNC rsspecan_Query5GNRDLAnalysisEVMFrequencyErrorLimitCheckResult(
                            ViSession instrSession,
                            ViInt32   cc,
                            ViInt32   frame,
                            ViInt32   measurementType,
                            ViInt32*  result)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_frame[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_SUMMARY_FREQUENCY_ERROR_LIMIT_CHECK_RESULT,
            "cc", cc - 1, RS_REPCAP_BUF_SIZE, repCap));
    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_SUMMARY_FREQUENCY_ERROR_LIMIT_CHECK_RESULT,
            "NR5GFrame", frame - 1, RS_REPCAP_BUF_SIZE, repCap_frame));

    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_frame, NULL);

    switch (measurementType)
    {
        case RSSPECAN_VAL_MEASTYPE_AVER:
            viCheckParm(rsspecan_GetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_SUMMARY_FREQUENCY_ERROR_LIMIT_CHECK_RESULT, result),
                    5, "Result");
            break;
        default:
            checkErr(RsCore_InvalidViInt32Value(instrSession, measurementType));
            break;
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

 /// HIFN  This function queries the sampling error.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR cc/Component Carrier parameter.
/// HIPAR frame/Sets the Frame parameter.
/// HIPAR samplingError/Queries the sampling error.
ViStatus _VI_FUNC rsspecan_Query5GNRDLAnalysisEVMSamplingError(
                            ViSession instrSession,
                            ViInt32   cc,
                            ViInt32   frame,
                            ViReal64  *samplingError)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_frame[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_SUMMARY_SAMPLING_ERROR,
            "cc", cc - 1, RS_REPCAP_BUF_SIZE, repCap));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_SUMMARY_SAMPLING_ERROR,
            "NR5GFrame", frame - 1, RS_REPCAP_BUF_SIZE, repCap_frame));

    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_frame, NULL);

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_SUMMARY_SAMPLING_ERROR, samplingError),
            4, "Sampling Error");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function queries the sampling error.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR cc/Component Carrier parameter.
/// HIPAR frame/Sets the Frame parameter.
/// HIPAR measurementType/Selects measurement type.
/// HIPAR samplingError/Queries the sampling error.
ViStatus _VI_FUNC rsspecan_Query5GNRDLAnalysisEVMSamplingErrorStatistics(
                            ViSession instrSession,
                            ViInt32   cc,
                            ViInt32   frame,
                            ViInt32   measurementType,
                            ViReal64* samplingError)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_frame[RS_REPCAP_BUF_SIZE] = "";
    ViAttr   attr[] = {
             RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_SUMMARY_SAMPLING_ERROR_MINIMUM,
             RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_SUMMARY_SAMPLING_ERROR_MAXIMUM,
             0,
             RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_SUMMARY_SAMPLING_ERROR
             };

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, measurementType, RSSPECAN_VAL_MEASTYPE_MIN, RSSPECAN_VAL_MEASTYPE_AVER),
            4, "Measurement Type");

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_SUMMARY_SAMPLING_ERROR,
            "cc", cc - 1, RS_REPCAP_BUF_SIZE, repCap));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_SUMMARY_SAMPLING_ERROR,
            "NR5GFrame", frame - 1, RS_REPCAP_BUF_SIZE, repCap_frame));

    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_frame, NULL);

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, repCap, attr[measurementType - 1], samplingError),
            5, "Sampling Error");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function queries the gain imbalance.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR cc/Component Carrier parameter.
/// HIPAR frame/Sets the Frame parameter.
/// HIPAR measurementType/Selects measurement type.
/// HIPAR iqGainImbalance/Queries the gain imbalance.
ViStatus _VI_FUNC rsspecan_Query5GNRDLAnalysisIQGainImbalance(
                            ViSession instrSession,
                            ViInt32   cc,
                            ViInt32   frame,
                            ViInt32   measurementType,
                            ViReal64* iqGainImbalance)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_frame[RS_REPCAP_BUF_SIZE] = "";
    ViAttr   attr[] = {
             RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_SUMMARY_IQ_GAIN_IMBALANCE_MINIMUM,
             RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_SUMMARY_IQ_GAIN_IMBALANCE_MAXIMUM,
             0,
             RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_SUMMARY_IQ_GAIN_IMBALANCE_AVERAGE
             };

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, measurementType, RSSPECAN_VAL_MEASTYPE_MIN, RSSPECAN_VAL_MEASTYPE_AVER),
            4, "Measurement Type");

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_SUMMARY_IQ_GAIN_IMBALANCE_AVERAGE,
            "cc", cc - 1, RS_REPCAP_BUF_SIZE, repCap));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_SUMMARY_IQ_GAIN_IMBALANCE_AVERAGE,
            "NR5GFrame", frame - 1, RS_REPCAP_BUF_SIZE, repCap_frame));

    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_frame, NULL);

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, repCap, attr[measurementType - 1], iqGainImbalance),
            5, "I/Q Gain Imbalance");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function queries the I/Q offset.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR cc/Component Carrier parameter.
/// HIPAR frame/Sets the Frame parameter.
/// HIPAR measurementType/Selects measurement type.
/// HIPAR iqOffset/Queries the I/Q offset.
ViStatus _VI_FUNC rsspecan_Query5GNRDLAnalysisIQOffset(ViSession instrSession,
                                                       ViInt32   cc,
                                                       ViInt32   frame,
                                                       ViInt32   measurementType,
                                                       ViReal64* iqOffset)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_frame[RS_REPCAP_BUF_SIZE] = "";
    ViAttr   attr[] = {
             RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_SUMMARY_IQ_OFFSET_MINIMUM,
             RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_SUMMARY_IQ_OFFSET_MAXIMUM,
             0,
             RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_SUMMARY_IQ_OFFSET_AVERAGE
             };

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, measurementType, RSSPECAN_VAL_MEASTYPE_MIN, RSSPECAN_VAL_MEASTYPE_AVER),
            4, "Measurement Type");

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_SUMMARY_IQ_OFFSET_AVERAGE,
            "cc", cc - 1, RS_REPCAP_BUF_SIZE, repCap));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_SUMMARY_IQ_OFFSET_AVERAGE,
            "NR5GFrame", frame - 1, RS_REPCAP_BUF_SIZE, repCap_frame));

    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_frame, NULL);

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, repCap, attr[measurementType - 1], iqOffset),
            5, "I/Q Offset");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function queries the I/Q Quadrature Error.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR cc/Component Carrier parameter.
/// HIPAR frame/Sets the Frame parameter.
/// HIPAR measurementType/Selects measurement type.
/// HIPAR iqQuadratureError/Queries the I/Q Quadrature Error.
ViStatus _VI_FUNC rsspecan_Query5GNRDLAnalysisIQQuadratureError(
                            ViSession instrSession,
                            ViInt32   cc,
                            ViInt32   frame,
                            ViInt32   measurementType,
                            ViReal64* iqQuadratureError)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_frame[RS_REPCAP_BUF_SIZE] = "";
    ViAttr   attr[] = {
             RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_SUMMARY_IQ_QUADRATURE_ERROR_MINIMUM,
             RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_SUMMARY_IQ_QUADRATURE_ERROR_MAXIMUM,
             0,
             RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_SUMMARY_IQ_QUADRATURE_ERROR_AVERAGE
             };

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, measurementType, RSSPECAN_VAL_MEASTYPE_MIN, RSSPECAN_VAL_MEASTYPE_AVER),
            4, "Measurement Type");

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_SUMMARY_IQ_QUADRATURE_ERROR_AVERAGE,
            "cc", cc - 1, RS_REPCAP_BUF_SIZE, repCap));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_SUMMARY_IQ_QUADRATURE_ERROR_AVERAGE,
            "NR5GFrame", frame - 1, RS_REPCAP_BUF_SIZE, repCap_frame));

    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_frame, NULL);

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, repCap, attr[measurementType - 1], iqQuadratureError),
            5, "I/Q Quadrature Error");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function queries the OSTP.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR cc/Component Carrier parameter.
/// HIPAR frame/Sets the Frame parameter.
/// HIPAR measurementType/Selects measurement type.
/// HIPAR ostp/Queries the OSTP.
ViStatus _VI_FUNC rsspecan_Query5GNRDLAnalysisOSTP(ViSession instrSession,
                                                   ViInt32   cc,
                                                   ViInt32   frame,
                                                   ViInt32   measurementType,
                                                   ViReal64* ostp)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_frame[RS_REPCAP_BUF_SIZE] = "";
    ViAttr   attr[] = {
             RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_SUMMARY_OSTP_MINIMUM,
             RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_SUMMARY_OSTP_MAXIMUM,
             0,
             RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_SUMMARY_OSTP_AVERAGE
             };

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, measurementType, RSSPECAN_VAL_MEASTYPE_MIN, RSSPECAN_VAL_MEASTYPE_AVER),
            4, "Measurement Type");

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_SUMMARY_OSTP_AVERAGE,
            "cc", cc - 1, RS_REPCAP_BUF_SIZE, repCap));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_SUMMARY_OSTP_AVERAGE,
            "NR5GFrame", frame - 1, RS_REPCAP_BUF_SIZE, repCap_frame));

    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_frame, NULL);

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, repCap, attr[measurementType - 1], ostp),
            5, "OSTP");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function queries the CSI-RSRP.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR cc/Component Carrier parameter.
/// HIPAR frame/Sets the Frame parameter.
/// HIPAR measurementType/Selects measurement type.
/// HIPAR csiRSRP/Queries the CSI-RSRP.
ViStatus _VI_FUNC rsspecan_Query5GNRDLAnalysisCSIRSRP(ViSession instrSession,
                                                      ViInt32   cc,
                                                      ViInt32   frame,
                                                      ViInt32   measurementType,
                                                      ViReal64* csiRSRP)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_frame[RS_REPCAP_BUF_SIZE] = "";
    ViAttr   attr[] = {
             RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_SUMMARY_CSI_RSRP_MINIMUM,
             RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_SUMMARY_CSI_RSRP_MAXIMUM,
             0,
             RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_SUMMARY_CSI_RSRP_AVERAGE
             };

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, measurementType, RSSPECAN_VAL_MEASTYPE_MIN, RSSPECAN_VAL_MEASTYPE_AVER),
            4, "Measurement Type");

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_SUMMARY_CSI_RSRP_AVERAGE,
            "cc", cc - 1, RS_REPCAP_BUF_SIZE, repCap));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_SUMMARY_CSI_RSRP_AVERAGE,
            "NR5GFrame", frame - 1, RS_REPCAP_BUF_SIZE, repCap_frame));

    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_frame, NULL);

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, repCap, attr[measurementType - 1], csiRSRP),
            5, "CSI-RSRP");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function queries the SS-RSRP.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR cc/Component Carrier parameter.
/// HIPAR frame/Sets the Frame parameter.
/// HIPAR measurementType/Selects measurement type.
/// HIPAR ssRSRP/Queries the SS-RSRP.
ViStatus _VI_FUNC rsspecan_Query5GNRDLAnalysisSSRSRP(ViSession instrSession,
                                                     ViInt32   cc,
                                                     ViInt32   frame,
                                                     ViInt32   measurementType,
                                                     ViReal64* ssRSRP)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_frame[RS_REPCAP_BUF_SIZE] = "";
    ViAttr   attr[] = {
             RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_SUMMARY_SS_RSRP_MINIMUM,
             RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_SUMMARY_SS_RSRP_MAXIMUM,
             0,
             RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_SUMMARY_SS_RSRP_AVERAGE
             };

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, measurementType, RSSPECAN_VAL_MEASTYPE_MIN, RSSPECAN_VAL_MEASTYPE_AVER),
            4, "Measurement Type");

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_SUMMARY_SS_RSRP_AVERAGE,
            "cc", cc - 1, RS_REPCAP_BUF_SIZE, repCap));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_SUMMARY_SS_RSRP_AVERAGE,
            "NR5GFrame", frame - 1, RS_REPCAP_BUF_SIZE, repCap_frame));

    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_frame, NULL);

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, repCap, attr[measurementType - 1], ssRSRP),
            5, "SS-RSRP");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function queries the RSTP.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR cc/Component Carrier parameter.
/// HIPAR frame/Sets the Frame parameter.
/// HIPAR measurementType/Selects measurement type.
/// HIPAR rstp/Queries the RSTP.
ViStatus _VI_FUNC rsspecan_Query5GNRDLAnalysisRSTP(ViSession instrSession,
                                                   ViInt32   cc,
                                                   ViInt32   frame,
                                                   ViInt32   measurementType,
                                                   ViReal64* rstp)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_frame[RS_REPCAP_BUF_SIZE] = "";
    ViAttr   attr[] = {
             RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_SUMMARY_RSTP_MINIMUM,
             RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_SUMMARY_RSTP_MAXIMUM,
             0,
             RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_SUMMARY_RSTP_AVERAGE
             };

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, measurementType, RSSPECAN_VAL_MEASTYPE_MIN, RSSPECAN_VAL_MEASTYPE_AVER),
            4, "Measurement Type");

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_SUMMARY_RSTP_AVERAGE,
            "cc", cc - 1, RS_REPCAP_BUF_SIZE, repCap));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_SUMMARY_RSTP_AVERAGE,
            "NR5GFrame", frame - 1, RS_REPCAP_BUF_SIZE, repCap_frame));

    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_frame, NULL);

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, repCap, attr[measurementType - 1], rstp),
            5, "RSTP");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function queries the BLER.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR cc/Component Carrier parameter.
/// HIPAR frame/Sets the Frame parameter.
/// HIPAR measurementType/Selects measurement type.
/// HIPAR bler/Queries the BLER.
ViStatus _VI_FUNC rsspecan_Query5GNRDLAnalysisBLER(ViSession instrSession,
                                                   ViInt32   cc,
                                                   ViInt32   frame,
                                                   ViInt32   measurementType,
                                                   ViReal64* bler)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_frame[RS_REPCAP_BUF_SIZE] = "";
    ViAttr   attr[] = {
             RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_SUMMARY_BLER_MINIMUM,
             RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_SUMMARY_BLER_MAXIMUM,
             0,
             RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_SUMMARY_BLER_AVERAGE
             };

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, measurementType, RSSPECAN_VAL_MEASTYPE_MIN, RSSPECAN_VAL_MEASTYPE_AVER),
            4, "Measurement Type");

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_SUMMARY_BLER_AVERAGE,
            "cc", cc - 1, RS_REPCAP_BUF_SIZE, repCap));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_SUMMARY_BLER_AVERAGE,
            "NR5GFrame", frame - 1, RS_REPCAP_BUF_SIZE, repCap_frame));

    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_frame, NULL);

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, repCap, attr[measurementType - 1], bler),
            5, "BLER");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function queries the TPUT.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR cc/Component Carrier parameter.
/// HIPAR frame/Sets the Frame parameter.
/// HIPAR measurementType/Selects measurement type.
/// HIPAR tput/Queries the TPUT.
ViStatus _VI_FUNC rsspecan_Query5GNRDLAnalysisTPUT(ViSession instrSession,
                                                   ViInt32   cc,
                                                   ViInt32   frame,
                                                   ViInt32   measurementType,
                                                   ViReal64* tput)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_frame[RS_REPCAP_BUF_SIZE] = "";
    ViAttr   attr[] = {
             RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_SUMMARY_TPUT_MINIMUM,
             RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_SUMMARY_TPUT_MAXIMUM,
             0,
             RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_SUMMARY_TPUT_AVERAGE
             };

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, measurementType, RSSPECAN_VAL_MEASTYPE_MIN, RSSPECAN_VAL_MEASTYPE_AVER),
            4, "Measurement Type");

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_SUMMARY_TPUT_AVERAGE,
            "cc", cc - 1, RS_REPCAP_BUF_SIZE, repCap));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_SUMMARY_TPUT_AVERAGE,
            "NR5GFrame", frame - 1, RS_REPCAP_BUF_SIZE, repCap_frame));

    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_frame, NULL);

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, repCap, attr[measurementType - 1], tput),
            5, "BLER");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function queries the (sub)frame start offset as shown in the
/// HIFN  capture buffer.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR cc/Component Carrier parameter.
/// HIPAR subframeStartOffset/Queries the (sub)frame start offset as shown in the capture buffer.
ViStatus _VI_FUNC rsspecan_Query5GNRDLAnalysisEVMSubframeStartOffset(
                            ViSession instrSession,
                            ViInt32   cc,
                            ViReal64  *subframeStartOffset)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_SUMMARY_SUBFRAME_START_OFFSET,
            "cc", cc - 1, RS_REPCAP_BUF_SIZE, repCap));

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_SUMMARY_SUBFRAME_START_OFFSET, subframeStartOffset),
            4, "Subframe Start Offset");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the average crest factor as shown in the result summary.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR cc/Component Carrier parameter.
/// HIPAR measurementType/Sets the type of returned value.
/// HIPAR result/Queries the average crest factor as shown in the result summary.
ViStatus _VI_FUNC rsspecan_Query5GNRDLAnalysisCrestFactor(ViSession instrSession,
                                                          ViInt32   cc,
                                                          ViInt32   measurementType,
                                                          ViReal64* result)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViAttr   attr[] = {
             0,
             0,
             0,
             RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_CREST_FACTOR_RESULT_AVERAGE
             };

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, measurementType, RSSPECAN_VAL_MEASTYPE_AVER, RSSPECAN_VAL_MEASTYPE_AVER),
            3, "Measurement Type");

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_CREST_FACTOR_RESULT_AVERAGE,
            "cc", cc - 1, RS_REPCAP_BUF_SIZE, repCap));

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, repCap, attr[measurementType - 1], result),
            4, "Result");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the ACLR limit check of an event.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR cc/Component Carrier parameter.
/// HIPAR frame/Sets the Frame parameter.
/// HIPAR type/This control select the type of check.
/// HIPAR result/Queries the ACLR limit check of an event.
ViStatus _VI_FUNC rsspecan_Query5GNRDLAnalysisACLRLimitCheck(ViSession instrSession,
                                                             ViInt32   cc,
                                                             ViInt32   frame,
                                                             ViInt32   type,
                                                             ViInt32*  result)
{
    ViStatus error = VI_SUCCESS;
    ViAttr   attr = 0;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_frame[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_SUMMARY_ACLR_LIMIT_CHECK,
            "cc", cc - 1, RS_REPCAP_BUF_SIZE, repCap));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_SUMMARY_ACLR_LIMIT_CHECK,
            "NR5GFrame", frame - 1, RS_REPCAP_BUF_SIZE, repCap_frame));

    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_frame, NULL);

    switch (type)
    {
        case RSSPECAN_VAL_ACLR_OVERALL:
            attr = RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_SUMMARY_ACLR_LIMIT_CHECK;
            break;
        case RSSPECAN_VAL_ACLR_ABSOLUTE:
            attr = RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_SUMMARY_ACLR_LIMIT_CHECK_ABSOLUTE;
            break;
        case RSSPECAN_VAL_ACLR_RELATIVE:
            attr = RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_SUMMARY_ACLR_LIMIT_CHECK_RELATIVE;
            break;
        default:
            checkErr(RsCore_InvalidViInt32Value(instrSession, type));
            break;
    }

    viCheckParm(rsspecan_GetAttributeViInt32(instrSession, repCap, attr, result),
            5, "Result");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the overload state of an event.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR cc/Component Carrier parameter.
/// HIPAR frame/Sets the Frame parameter.
/// HIPAR overload/Queries the overload state of an event.
ViStatus _VI_FUNC rsspecan_Query5GNRDLAnalysisOverload(ViSession instrSession,
                                                       ViInt32   cc,
                                                       ViInt32   frame,
                                                       ViInt32*  overload)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_frame[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_SUMMARY_OVERLOAD,
            "cc", cc - 1, RS_REPCAP_BUF_SIZE, repCap));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_SUMMARY_OVERLOAD,
            "NR5GFrame", frame - 1, RS_REPCAP_BUF_SIZE, repCap_frame));

    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_frame, NULL);

    viCheckParm(rsspecan_GetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_SUMMARY_OVERLOAD, overload),
            4, "Overload");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the SEM limit check of an event.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR cc/Component Carrier parameter.
/// HIPAR frame/Sets the Frame parameter.
/// HIPAR result/Queries the SEM limit check of an event.
ViStatus _VI_FUNC rsspecan_Query5GNRDLAnalysisSEMPassFail(ViSession instrSession,
                                                          ViInt32   cc,
                                                          ViInt32   frame,
                                                          ViInt32*  result)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_frame[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_SUMMARY_SEM_PASS_FAIL,
            "cc", cc - 1, RS_REPCAP_BUF_SIZE, repCap));
    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_SUMMARY_SEM_PASS_FAIL,
            "NR5GFrame", frame - 1, RS_REPCAP_BUF_SIZE, repCap_frame));

    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_frame, NULL);

    viCheckParm(rsspecan_GetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_SUMMARY_SEM_PASS_FAIL, result),
            4, "Result");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the synchronization state of an event.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR cc/Component Carrier parameter.
/// HIPAR frame/Sets the Frame parameter.
/// HIPAR result/Queries the synchronization state of an event.
ViStatus _VI_FUNC rsspecan_Query5GNRDLAnalysisSyncState(ViSession instrSession,
                                                        ViInt32   cc,
                                                        ViInt32   frame,
                                                        ViInt32*  result)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_frame[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_SUMMARY_SYNC_STATE,
            "cc", cc - 1, RS_REPCAP_BUF_SIZE, repCap));
    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_SUMMARY_SYNC_STATE,
            "NR5GFrame", frame - 1, RS_REPCAP_BUF_SIZE, repCap_frame));

    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_frame, NULL);

    viCheckParm(rsspecan_GetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_SUMMARY_SYNC_STATE, result),
            4, "Result");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the time stamp delta of an event.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR cc/Component Carrier parameter.
/// HIPAR frame/Sets the Frame parameter.
/// HIPAR result/Queries the time stamp delta of an event.
ViStatus _VI_FUNC rsspecan_Query5GNRDLAnalysisTimeStampDelta(ViSession instrSession,
                                                             ViInt32   cc,
                                                             ViInt32   frame,
                                                             ViReal64* result)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_frame[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_SUMMARY_TIME_STAMP_DELTA,
            "cc", cc - 1, RS_REPCAP_BUF_SIZE, repCap));
    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_SUMMARY_TIME_STAMP_DELTA,
            "NR5GFrame", frame - 1, RS_REPCAP_BUF_SIZE, repCap_frame));

    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_frame, NULL);

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_SUMMARY_TIME_STAMP_DELTA, result),
            4, "Result");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the time stamp of an event.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR cc/Component Carrier parameter.
/// HIPAR frame/Sets the Frame parameter.
/// HIPAR result/Queries the time stamp of an event.
ViStatus _VI_FUNC rsspecan_Query5GNRDLAnalysisTimeStamp(ViSession instrSession,
                                                        ViInt32   cc,
                                                        ViInt32   frame,
                                                        ViReal64* result)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_frame[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_SUMMARY_TIME_STAMP,
            "cc", cc - 1, RS_REPCAP_BUF_SIZE, repCap));
    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_SUMMARY_TIME_STAMP,
            "NR5GFrame", frame - 1, RS_REPCAP_BUF_SIZE, repCap_frame));

    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_frame, NULL);

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_SUMMARY_TIME_STAMP, result),
            4, "Result");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects the base station type.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR baseStationType/Selects the base station type.
ViStatus _VI_FUNC rsspecan_Configure5GNRDLAnalysisSweepBaseStationType(
                            ViSession instrSession,
                            ViInt32   baseStationType)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_FREQUENCY_SWEEP_BASE_STATION_TYPE, baseStationType),
            2, "Base Station Type");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function selects the bandwidth of the adjacent channel for ACLR
/// HIFN  measurements.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR bw/Selects the bandwidth of the adjacent channel for ACLR measurements.
ViStatus _VI_FUNC rsspecan_Configure5GNRDLAnalysisSweepBWForACLR(
                            ViSession instrSession,
                            ViInt32   bw)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_FREQUENCY_SWEEP_BW_FOR_ACLR, bw),
            2, "BW");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the Sweep Analysis.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR category/Selects the bandwidth of the adjacent channel for ACLR measurements.
/// HIPAR mediumRangeAuto/Selects how the R&S FSW determines the power of a medium range base
/// HIPAR mediumRangeAuto/station.
/// HIPAR mediumRangePower/Defines the power of a medium range base station.
ViStatus _VI_FUNC rsspecan_Configure5GNRDLAnalysisSweepBS(ViSession instrSession,
                                                          ViInt32   category,
                                                          ViBoolean mediumRangeAuto,
                                                          ViReal64  mediumRangePower)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_FREQUENCY_SWEEP_BS_CATEGORY, category),
            2, "Category");

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_FREQUENCY_SWEEP_BS_MEDIUM_RANGE_AUTO, mediumRangeAuto),
            3, "Medium Range Auto");

    if (mediumRangeAuto == VI_TRUE)
    {
        viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_FREQUENCY_SWEEP_BS_MEDIUM_RANGE_POWER, mediumRangePower),
                4, "Medium Range Power");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects the limit table for category B stations.
/// HIFN     Prerequisites
/// HIFN     - Select category B base station
/// HIFN  rsspecan_Configure5GNRDLAnalysisSweepBS
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR categoryBOption/Selects the limit table for category B stations.
ViStatus _VI_FUNC rsspecan_Configure5GNRDLAnalysisSweepBSCategoryBOption(
                            ViSession instrSession,
                            ViInt32   categoryBOption)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, NULL, RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_FREQUENCY_SWEEP_BS_CATEGORY_B_OPTION, categoryBOption),
            2, "Category B Option");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the shift of the limits of the spectrum
/// HIFN  emission mask according to 3GPP 38.141-1, chapter 6.6.4.5.8.
/// HIFN
/// HIFN     Prerequisites for this attribute
/// HIFN     Select a 1-H base station
/// HIFN  (RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_FREQUENCY_SWEEP_BASE_STATION_TYPE).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR n_tabconnectors/Defines the parameter NTABconnectors that defines the position of the
/// HIPAR n_tabconnectors/spectrum emission mask for 1-H base stations.
/// HIPAR n_txu/Defines the parameter NTXU that defines the position of the spectrum
/// HIPAR n_txu/emission mask for 1-H base stations.
ViStatus _VI_FUNC rsspecan_Configure5GNRDLAnalysisSweepSEMPosition(
                            ViSession instrSession,
                            ViInt32   n_tabconnectors,
                            ViInt32   n_txu)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_FREQUENCY_SWEEP_SEM_N_TABCONNECTORS, n_tabconnectors),
            2, "N_TABconnectors");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_FREQUENCY_SWEEP_SEM_N_TXU, n_txu),
            3, "N_TXU");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function queries the state of the limit check for the adjacent or
/// HIFN  alternate channels in an ACLR measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channelType/Selects the ACP channel type.
/// HIPAR limitCheckType/Selects the limit check type.
/// HIPAR results/Returns two results. The first is the result for the lower, the second
/// HIPAR results/for the upper adjacent or alternate channel.
ViStatus _VI_FUNC rsspecan_Query5GNRDLACPChannelLimitCheckResults(
                            ViSession instrSession,
                            ViInt32   channelType,
                            ViInt32   limitCheckType,
                            ViInt32   results[])
{
    ViStatus           error = VI_SUCCESS;
    ViString           cmd = NULL;
    ViChar*            p2rngString;
    RsCoreAttributePtr attr = NULL;
    ViChar             response[RS_MAX_MESSAGE_BUF_SIZE] = "";
    ViChar*            p2buf;
    ViInt32            res = 0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &RsSpecAn_rng5GNRLimitCheckType, limitCheckType, &p2rngString),
        3, "Limit Check Type");

    switch (channelType)
    {
    case RSSPECAN_VAL_ACP_ADJ:
        attr = &g_RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_FREQUENCY_SWEEP_GET_ADJACENT_ACLR_LIMIT_CHECK_RESULT;
        break;
    case RSSPECAN_VAL_ACP_ALT:
        attr = &g_RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_FREQUENCY_SWEEP_GET_ALTERNATE_ACLR_LIMIT_CHECK_RESULT;
        break;
    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, channelType), 2, "Channel Type");
    }

    checkErr(RsCore_BuildAttrCommandString(instrSession, attr, NULL, &cmd));
    if (limitCheckType != RSSPECAN_VAL_5G_NR_LIMIT_DEFAULT)
        snprintf(cmd, RS_MAX_SHORT_MESSAGE_BUF_SIZE, "%s %s", cmd, p2rngString);

    checkErr(RsCore_QueryAttributeCommand(instrSession, attr, cmd, response));

    p2buf = strtok(response, ",\n\r");
    RsCore_FindRangeTableDiscreteNumericValue(instrSession, &RsSpecAn_rng5GNRLimitResult, p2buf, &res);
    results[0] = res;
    p2buf = strtok(NULL, ",\n\r");
    RsCore_FindRangeTableDiscreteNumericValue(instrSession, &RsSpecAn_rng5GNRLimitResult, p2buf, &res);
    results[1] = res;

Error:
    if (cmd) free(cmd);
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects the limit evaluation mode for ACLR measurements.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR totalLimitPassMode/Selects the limit evaluation mode for ACLR measurements.
ViStatus _VI_FUNC rsspecan_Configure5GNRDLAnalysisSweepTotalLimitPassMode(
                            ViSession instrSession,
                            ViInt32   totalLimitPassMode)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, NULL, RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_FREQUENCY_SWEEP_TOTAL_LIMIT_PASS_MODE, totalLimitPassMode),
            2, "Total Limit Pass Mode");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function queries the state of the limit check for the adjacent or
/// HIFN  alternate channels in an ACLR measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channelType/Selects the ACP channel type.
/// HIPAR limitCheckType/Selects the limit type.
/// HIPAR results/Returns two results. The first is the result for the lower, the second
/// HIPAR results/for the upper adjacent or alternate channel.
ViStatus _VI_FUNC rsspecan_Query5GNRDLACLRLimitCheckResults(ViSession instrSession,
                                                            ViInt32   channelType,
                                                            ViInt32   limitCheckType,
                                                            ViInt32   results[])
{
    ViStatus error = VI_SUCCESS;
    ViAttr   attr = 0;
    ViChar   response[RS_MAX_MESSAGE_BUF_SIZE] = "";
    ViChar*  p2buf;
    ViInt32  res = 0;

    checkErr(RsCore_LockSession(instrSession));

    switch (channelType)
    {
        case RSSPECAN_VAL_ACP_ADJ:
            switch (limitCheckType)
            {
                case RSSPECAN_VAL_5G_NR_LIMIT_ABS:
                    attr = RSSPECAN_ATTR_HIDDEN_5G_NR_DL_ANALYSIS_FREQUENCY_SWEEP_GET_ADJACENT_ACLR_LIMIT_CHECK_RESULT_ABSOLUTE;
                    break;
                case RSSPECAN_VAL_5G_NR_LIMIT_REL:
                    attr = RSSPECAN_ATTR_HIDDEN_5G_NR_DL_ANALYSIS_FREQUENCY_SWEEP_GET_ADJACENT_ACLR_LIMIT_CHECK_RESULT_RELATIVE;
                    break;
                default:
                    checkErr(RsCore_InvalidViInt32Value(instrSession, limitCheckType));
                    break;
            }
            break;
        case RSSPECAN_VAL_ACP_ALT:
            switch (limitCheckType)
            {
                case RSSPECAN_VAL_5G_NR_LIMIT_ABS:
                    attr = RSSPECAN_ATTR_HIDDEN_5G_NR_DL_ANALYSIS_FREQUENCY_SWEEP_GET_ALTERNATE_ACLR_LIMIT_CHECK_RESULT_ABSOLUTE;
                    break;
                case RSSPECAN_VAL_5G_NR_LIMIT_REL:
                    attr = RSSPECAN_ATTR_HIDDEN_5G_NR_DL_ANALYSIS_FREQUENCY_SWEEP_GET_ALTERNATE_ACLR_LIMIT_CHECK_RESULT_RELATIVE;
                    break;
                default:
                    checkErr(RsCore_InvalidViInt32Value(instrSession, limitCheckType));
                    break;
            }
            break;
        default:
            checkErr(RsCore_InvalidViInt32Value(instrSession, channelType));
            break;
    }

    checkErr(rsspecan_GetAttributeViString(instrSession, NULL,
                                                 attr, RS_MAX_MESSAGE_BUF_SIZE,
                                                 response));

    p2buf = strtok(response, ",\n\r");
    RsCore_FindRangeTableDiscreteNumericValue(instrSession, &RsSpecAn_rng5GNRLimitResult, p2buf, &res);
    results[0] = res;
    p2buf = strtok(NULL, ",\n\r");
    RsCore_FindRangeTableDiscreteNumericValue(instrSession, &RsSpecAn_rng5GNRLimitResult, p2buf, &res);
    results[1] = res;

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function queries the state of the power limit check for the gap
/// HIFN  channels in an MC ACLR measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR limitCheckType/Selects the limit type.
/// HIPAR results/Returns two results. The first is the result for the lower, the second
/// HIPAR results/for the upper adjacent channel.
ViStatus _VI_FUNC rsspecan_Query5GNRDLACLRPowerLimitCheckResults(
                            ViSession instrSession,
                            ViInt32   limitCheckType,
                            ViInt32   results[])
{
    ViStatus error = VI_SUCCESS;
    ViAttr   attr = 0;
    ViChar   response[RS_MAX_MESSAGE_BUF_SIZE] = "";
    ViChar*  p2buf;
    ViInt32  res = 0;

    checkErr(RsCore_LockSession(instrSession));

    switch (limitCheckType)
    {
        case RSSPECAN_VAL_5G_NR_LIMIT_ABS:
            attr = RSSPECAN_ATTR_HIDDEN_5G_NR_DL_ANALYSIS_FREQUENCY_SWEEP_GET_ACLR_POWER_LIMIT_CHECK_RESULT_ABSOLUTE;
            break;
        case RSSPECAN_VAL_5G_NR_LIMIT_REL:
            attr = RSSPECAN_ATTR_HIDDEN_5G_NR_DL_ANALYSIS_FREQUENCY_SWEEP_GET_ACLR_POWER_LIMIT_CHECK_RESULT_RELATIVE;
            break;
        case RSSPECAN_VAL_5G_NR_LIMIT_DEFAULT:
            attr = RSSPECAN_ATTR_HIDDEN_5G_NR_DL_ANALYSIS_FREQUENCY_SWEEP_GET_ACLR_POWER_LIMIT_CHECK_RESULT;
            break;
        default:
            break;
    }

    checkErr(rsspecan_GetAttributeViString(instrSession, NULL,
                                                 attr, RS_MAX_MESSAGE_BUF_SIZE,
                                                 response));

    p2buf = strtok(response, ",\n\r");
    RsCore_FindRangeTableDiscreteNumericValue(instrSession, &RsSpecAn_rng5GNRLimitResult, p2buf, &res);
    results[0] = res;
    p2buf = strtok(NULL, ",\n\r");
    RsCore_FindRangeTableDiscreteNumericValue(instrSession, &RsSpecAn_rng5GNRLimitResult, p2buf, &res);
    results[1] = res;

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function queries the state of the  limit check for the gap
/// HIFN  channels in an MC ACLR measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR limitCheckType/Selects the limit type.
/// HIPAR results/Returns two results. The first is the result for the lower, the second
/// HIPAR results/for the upper adjacent channel.
ViStatus _VI_FUNC rsspecan_Query5GNRDLCACLRLimitCheckResults(ViSession instrSession,
                                                             ViInt32   limitCheckType,
                                                             ViInt32   results[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   response[RS_MAX_MESSAGE_BUF_SIZE] = "";
    ViChar*  p2buf;
    ViAttr   attr = 0;
    ViInt32  res = 0;

    checkErr(RsCore_LockSession(instrSession));

    switch (limitCheckType)
    {
        case RSSPECAN_VAL_5G_NR_LIMIT_ABS:
            attr = RSSPECAN_ATTR_HIDDEN_5G_NR_DL_ANALYSIS_FREQUENCY_SWEEP_GET_CACLR_LIMIT_CHECK_RESULT_ABSOLUTE;
            break;
        case RSSPECAN_VAL_5G_NR_LIMIT_REL:
            attr = RSSPECAN_ATTR_HIDDEN_5G_NR_DL_ANALYSIS_FREQUENCY_SWEEP_GET_CACLR_LIMIT_CHECK_RESULT_RELATIVE;
            break;
        case RSSPECAN_VAL_5G_NR_LIMIT_DEFAULT:
            attr = RSSPECAN_ATTR_HIDDEN_5G_NR_DL_ANALYSIS_FREQUENCY_SWEEP_GET_CACLR_LIMIT_CHECK_RESULT;
            break;
        default:
            break;
    }

    checkErr(rsspecan_GetAttributeViString(instrSession, NULL,
                                                 attr, RS_MAX_MESSAGE_BUF_SIZE,
                                                 response));

    p2buf = strtok(response, ",\n\r");
    RsCore_FindRangeTableDiscreteNumericValue(instrSession, &RsSpecAn_rng5GNRLimitResult, p2buf, &res);
    results[0] = res;
    p2buf = strtok(NULL, ",\n\r");
    RsCore_FindRangeTableDiscreteNumericValue(instrSession, &RsSpecAn_rng5GNRLimitResult, p2buf, &res);
    results[1] = res;

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function assigns a specific component carrier to a view.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR view/View parameter.
/// HIPAR carrier/Assigns a specific component carrier to a view.
ViStatus _VI_FUNC rsspecan_Configure5GNRDLAnalysisViewCarrier(
                            ViSession instrSession,
                            ViInt32   view,
                            ViInt32   carrier)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_VIEW_CARRIER,
            "View", view - 1, RS_REPCAP_BUF_SIZE, repCap));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_VIEW_CARRIER, carrier),
            3, "Carrier");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function assigns a specific frame to a view.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR view/View parameter.
/// HIPAR frame/Assigns a specific frame to a view.
ViStatus _VI_FUNC rsspecan_Configure5GNRDLAnalysisViewFrame(ViSession instrSession,
                                                            ViInt32   view,
                                                            ViInt32   frame)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_VIEW_FRAME,
            "View", view - 1, RS_REPCAP_BUF_SIZE, repCap));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_VIEW_FRAME, frame),
            3, "Frame");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function couples or decouples result display tabs (subwindows).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR view/View parameter.
/// HIPAR coupling/Couples or decouples result display tabs (subwindows).
ViStatus _VI_FUNC rsspecan_Configure5GNRDLAnalysisViewWindowCoupling(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   view,
                            ViBoolean coupling)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_view[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_VIEW_WINDOW_COUPLING,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_VIEW_WINDOW_COUPLING,
            "View", view - 1, RS_REPCAP_BUF_SIZE, repCap_view));

    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_view, NULL);

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, repCap, RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_VIEW_WINDOW_COUPLING, coupling),
            4, "Coupling");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function sets the focus on the selected result display window.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR view/View parameter.
ViStatus _VI_FUNC rsspecan_Configure5GNRDLAnalysisViewSelectResultWindow(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   view)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_view[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_VIEW_SELECT_RESULT_WINDOW,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_VIEW_SELECT_RESULT_WINDOW,
            "View", view - 1, RS_REPCAP_BUF_SIZE, repCap_view));

    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_view, NULL);

    checkErr(rsspecan_SetAttributeViString(instrSession, repCap, RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_VIEW_SELECT_RESULT_WINDOW, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the results of the limit check in the 'Off' periods of On/Off
/// HIFN  Power measurements.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR arraySize/Defines the memory size allocated for Results.
/// HIPAR results/Returns one value for every 'Off' period.
/// HIPAR returnedValues/Returns number of returned values in Results array.
ViStatus _VI_FUNC rsspecan_Query5GNRDLOffPeriodsLimitCheckResult(
                            ViSession instrSession,
                            ViInt32   arraySize,
                            ViBoolean results[],
                            ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;
    ViChar   *pbuffer = NULL;
    ViChar   *pstring_value;
    ViInt32  i = 0;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K144"));

    checkErr(RsCore_QueryViStringUnknownLength(instrSession, "CALC:LIM:OOP:OFFP?", &pbuffer));

    pstring_value = strtok(pbuffer, ",");

    while (pstring_value != NULL && i < arraySize)
    {
        if (strncmp(pstring_value, "PASSED", 6) == 0)
            results[i] = VI_TRUE;
        else
            results[i] = VI_FALSE;

        pstring_value = strtok(NULL, ",");
        i++;
    }

    if (*returnedValues) *returnedValues = i;

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    if (pbuffer) free(pbuffer);
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the results of the limit check during the transient periods of
/// HIFN  the On/Off power measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR type/This control selects the limit check type.
/// HIPAR arraySize/Defines the memory size allocated for Results.
/// HIPAR results/Returns one value for every transient period.
/// HIPAR returnedValues/Returns number of returned values in Results array.
ViStatus _VI_FUNC rsspecan_Query5GNRDLTransientLimitCheckResult(
                            ViSession instrSession,
                            ViInt32   type,
                            ViInt32   arraySize,
                            ViBoolean results[],
                            ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViChar   *pbuffer = NULL;
    ViChar   *pstring_value;
    ViInt32  i = 0;
    ViString typeArr[] = {"ALL", "FALL", "RIS"};

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, type, RSSPECAN_VAL_LIMIT_ALL, RSSPECAN_VAL_LIMIT_RISING),
            2, "Type");

    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K100|K102"));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CALC:LIM:OOP:TRAN? %s", typeArr[type]);
    checkErr(RsCore_QueryViStringUnknownLength(instrSession, cmd, &pbuffer));

    pstring_value = strtok(pbuffer, ",");

    while (pstring_value != NULL && i < arraySize)
    {
        if (strncmp(pstring_value, "PASSED", 6) == 0)
            results[i] = VI_TRUE;
        else
            results[i] = VI_FALSE;

        pstring_value = strtok(NULL, ",");
        i++;
    }

    if (*returnedValues)
        *returnedValues = i;

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    if (pbuffer) free(pbuffer);
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Turns an event filter for combined measurement results on and off.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR eventFilter/Turns an event filter for combined measurement results on and off.
ViStatus _VI_FUNC rsspecan_Configure5GNRDLCombinedMeasurementEventFilter(
                            ViSession instrSession,
                            ViBoolean eventFilter)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, NULL, RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_COMBINED_MEASUREMENT_EVENT_FILTER, eventFilter),
            2, "Event Filter");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Removes all event filters.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsspecan_Reset5GNRDLCombinedMeasurementEventFilter(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsspecan_SetAttributeViString(instrSession, NULL, RSSPECAN_ATTR_5G_NR_DL_ANALYSIS_COMBINED_MEASUREMENT_EVENT_FILTER_RESET, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Defines a filter condition for the event filter in combined
/// HIFN  measurements.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR result/Selects the result that you want to define a filter for.
/// HIPAR condition/Result condition.
/// HIPAR value/Value of numeric result. Used only for numeric conditions
/// HIPAR value/Greater/Lower than or equal.
ViStatus _VI_FUNC rsspecan_Configure5GNRDLCombinedMeasurementEventFilterParameters(
                            ViSession instrSession,
                            ViInt32   result,
                            ViInt32   condition,
                            ViReal64  value)
{
    ViStatus error = VI_SUCCESS;
    ViChar   param[RS_PARAM_BUF_SIZE] = "";
    ViChar*  resultCmd;
    ViChar*  conditionCmd;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &RsSpecAn_rng5GNREventFilterParamResult, result, &resultCmd), 2, "Result");
    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &RsSpecAn_rngNrEventCondition, condition, &conditionCmd), 3, "Condition");

    if (result <= RSSPECAN_VAL_5G_NR_RESULT_OVLD)
    {
        viCheckParm(RsCore_InvalidViInt32Range(instrSession, condition, RSSPECAN_VAL_NR_EVENT_CONDITION_GTEQUAL, RSSPECAN_VAL_NR_EVENT_CONDITION_LTEQUAL),
                3, "Condition");

        snprintf (param, RS_PARAM_BUF_SIZE, "%s,%s,%.12lf", resultCmd, conditionCmd, value);
    }
    else
    {
        viCheckParm(RsCore_InvalidViInt32Range(instrSession, condition, RSSPECAN_VAL_NR_EVENT_CONDITION_FAILED, RSSPECAN_VAL_NR_EVENT_CONDITION_PASSED),
                3, "Condition");

        snprintf (param, RS_PARAM_BUF_SIZE, "%s,%s", resultCmd, conditionCmd);
    }

    checkErr(rsspecan_SetAttributeViString(instrSession, NULL, RSSPECAN_ATTR_HIDDEN_5G_NR_DL_ANALYSIS_COMBINED_MEASUREMENT_EVENT_FILTER_PARAMETERS, param));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Turns an event filter on and off.
/// HIFN
/// HIFN     Turning on an event filter adds it to the filter list and removes the
/// HIFN  results from the result summary that do not fulfill the filter
/// HIFN  conditions.
/// HIFN
/// HIFN     Turning it off removes it from the list.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR result/Selects the result that you want to define a filter for.
/// HIPAR state/Enable disable result.
ViStatus _VI_FUNC rsspecan_Configure5GNRDLCombinedMeasurementEventFilterParametersEnabled(
                            ViSession instrSession,
                            ViInt32   result,
                            ViBoolean state)
{
    ViStatus error = VI_SUCCESS;
    ViChar   param[RS_PARAM_BUF_SIZE] = "";
    ViChar*  resultCmd;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &RsSpecAn_rng5GNREventFilterParamResult, result, &resultCmd), 2, "Result");

    snprintf (param, RS_PARAM_BUF_SIZE, "%s,%s", resultCmd, state ? "ON" : "OFF");

    checkErr(rsspecan_SetAttributeViString(instrSession, NULL, RSSPECAN_ATTR_HIDDEN_5G_NR_DL_ANALYSIS_COMBINED_MEASUREMENT_EVENT_FILTER_PARAMETERS_ENABLED, param));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries 5G NR Downlink measurement time alignment error.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR componentCarrier/This control selects the component carrier to be used.
/// HIPAR antenna/This control selects the antenna port.
/// HIPAR measurementType/Selects measurement type.
/// HIPAR result/Returns selected result.
ViStatus _VI_FUNC rsspecan_Query5GNRDLMeasurementTimeAlignmentError(
                            ViSession instrSession,
                            ViInt32   componentCarrier,
                            ViInt32   antenna,
                            ViInt32   measurementType,
                            ViReal64* result)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";
    ViChar   *p2string;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, componentCarrier, 1, 16),
            2, "Component Carrier");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, measurementType, RSSPECAN_VAL_5G_NR_TAER_RESULT_TYPE_MIN, RSSPECAN_VAL_5G_NR_TAER_RESULT_TYPE_MAX),
            4, "Measurement Type");
    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &RsSpecAn_rng5GNRTimeAlignmentErrorResultType, measurementType, &p2string),
            4, "Measurement Type");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "FETCh:TAER:CC%ld:AP%ld:%s?", componentCarrier, antenna, p2string);
    checkErr(RsCore_QueryViReal64(instrSession, cmd, result));

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  For the capture buffer result display, the method returns one value
/// HIFN  for each I/Q sample in the capture buffer. The unit is always dBm.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR arraySize/Number of array points requested.
/// HIPAR result/Returns trace data.
/// HIPAR returnedValues/Returns the number of trace data values.
ViStatus _VI_FUNC rsspecan_Fetch5GNRDLAnalysisCaptureBuffer(ViSession instrSession,
                                                            ViInt32   window,
                                                            ViInt32   arraySize,
                                                            ViReal64  result[],
                                                            ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, window, 0, 16),
            2, "Window");
    checkErr(rsspecan_dataReadTrace(instrSession, window, "TRACE1", arraySize, result, returnedValues));
    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  For the EVM vs carrier result display, the method  returns one value
/// HIFN  for each subcarrier that has been analyzed.
/// HIFN
/// HIFN     The unit depends on UNIT:EVM (EVMUnit/RSSPECAN_ATTR_5G_NR_EVM_UNIT).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR trace/Selects the trace.
/// HIPAR arraySize/Number of array points requested.
/// HIPAR results/Returns the selected results.
/// HIPAR returnedValues/Returns the number of returned values.
ViStatus _VI_FUNC rsspecan_Fetch5GNRDLAnalysisEVMvsCarrier(ViSession instrSession,
                                                           ViInt32   window,
                                                           ViInt32   trace,
                                                           ViInt32   arraySize,
                                                           ViReal64  results[],
                                                           ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, window, 0, 16),
            2, "Window");
    checkErr(rsspecan_dataReadTrace(instrSession, window, traceArr[trace], arraySize, results, returnedValues));
    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  For the EVM vs symbol result display, the method  returns one value
/// HIFN  for each OFDM symbol that has been analyzed.
/// HIFN     The return values depend on the evaluation range:
/// HIFN     - A specific subframe: EVM of that subframe over all slots.
/// HIFN     - A specific slot: EVM of that slot.
/// HIFN
/// HIFN     The unit depends on UNIT:EVM (EVMUnit/RSSPECAN_ATTR_5G_NR_EVM_UNIT).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR trace/Selects the trace.
/// HIPAR arraySize/Number of array points requested.
/// HIPAR results/Returns the selected results.
/// HIPAR returnedValues/Returns the number of returned values.
ViStatus _VI_FUNC rsspecan_Fetch5GNRDLAnalysisEVMvsSymbol(ViSession instrSession,
                                                          ViInt32   window,
                                                          ViInt32   trace,
                                                          ViInt32   arraySize,
                                                          ViReal64  results[],
                                                          ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, window, 0, 16),
            2, "Window");
    checkErr(rsspecan_dataReadTrace(instrSession, window, traceArr[trace], arraySize, results, returnedValues));
    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  For the EVM vs RB result display, the method returns one value for
/// HIFN  each resource block that has been analyzed.
/// HIFN
/// HIFN     The unit depends on UNIT:EVM (EVMUnit/RSSPECAN_ATTR_5G_NR_EVM_UNIT).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR trace/Selects the trace.
/// HIPAR arraySize/Number of array points requested.
/// HIPAR results/Returns the selected results.
/// HIPAR returnedValues/Returns the number of returned values.
ViStatus _VI_FUNC rsspecan_Fetch5GNRDLAnalysisEVMvsRB(ViSession instrSession,
                                                      ViInt32   window,
                                                      ViInt32   trace,
                                                      ViInt32   arraySize,
                                                      ViReal64  results[],
                                                      ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, window, 0, 16),
            2, "Window");
    checkErr(rsspecan_dataReadTrace(instrSession, window, traceArr[trace], arraySize, results, returnedValues));
    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  For the frequency error vs symbol result display, the method returns
/// HIFN  one value for each OFDM symbol that has been analyzed.
/// HIFN
/// HIFN     The return values depend on the evaluation range:
/// HIFN     - A specific subframe: Frequency error of that subframe over all
/// HIFN  slots.
/// HIFN     - A specific slot: Frequency error of that slot.
/// HIFN
/// HIFN     The unit is always Hz.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR trace/Selects the trace.
/// HIPAR arraySize/Number of array points requested.
/// HIPAR results/Returns the selected results.
/// HIPAR returnedValues/Returns the number of returned values.
ViStatus _VI_FUNC rsspecan_Fetch5GNRDLAnalysisFrequencyErrorvsSymbol(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   trace,
                            ViInt32   arraySize,
                            ViReal64  results[],
                            ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, window, 0, 16),
            2, "Window");
    checkErr(rsspecan_dataReadTrace(instrSession, window, traceArr[trace], arraySize, results, returnedValues));
    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  For the power spectrum result display, the method returns one value
/// HIFN  for each trace point.
/// HIFN
/// HIFN     The unit is always dBm/Hz.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR arraySize/Number of array points requested.
/// HIPAR results/Returns the selected results.
/// HIPAR returnedValues/Returns the number of returned values.
ViStatus _VI_FUNC rsspecan_Fetch5GNRDLAnalysisPowerSpectrum(ViSession instrSession,
                                                            ViInt32   window,
                                                            ViInt32   arraySize,
                                                            ViReal64  results[],
                                                            ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, window, 0, 16),
            2, "Window");
    checkErr(rsspecan_dataReadTrace(instrSession, window, traceArr[1], arraySize, results, returnedValues));
    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  For the flatness vs carrier result display, the method returns one
/// HIFN  value for each trace point.
/// HIFN
/// HIFN     The unit is always dB.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR trace/Selects the trace.
/// HIPAR arraySize/Number of array points requested.
/// HIPAR results/Returns the selected results.
/// HIPAR returnedValues/Returns the number of returned values.
ViStatus _VI_FUNC rsspecan_Fetch5GNRDLAnalysisFlatnessvsCarrier(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   trace,
                            ViInt32   arraySize,
                            ViReal64  results[],
                            ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, window, 0, 16),
            2, "Window");
    checkErr(rsspecan_dataReadTrace(instrSession, window, traceArr[trace], arraySize, results, returnedValues));
    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  For the CCDF result display, the type of return values depends on the
/// HIFN  parameter Axis.
/// HIFN
/// HIFN     Y
/// HIFN     Returns the probability values (y-axis).
/// HIFN     <# of values>, <probability>, ...
/// HIFN     The unit is always %.
/// HIFN
/// HIFN     X
/// HIFN     Returns the corresponding power levels (x-axis).
/// HIFN     <# of values>, <relative power>, ...
/// HIFN     The unit is always dB.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR axis/Sets the axis.
/// HIPAR arraySize/Number of array points requested.
/// HIPAR results/Returns the selected results.
/// HIPAR returnedValues/Returns the number of returned values.
ViStatus _VI_FUNC rsspecan_Fetch5GNRDLAnalysisCCDF(ViSession instrSession,
                                                   ViInt32   window,
                                                   ViInt32   axis,
                                                   ViInt32   arraySize,
                                                   ViReal64  results[],
                                                   ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, window, 0, 16),
            2, "Window");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, axis, RSSPECAN_VAL_AXIS_X, RSSPECAN_VAL_AXIS_Y),
            3, "Axis");

    if (window == 0)
    {
        snprintf (cmd, RS_MAX_MESSAGE_BUF_SIZE, ":FORM REAL,32;:TRAC? TRACE%d", axis == RSSPECAN_VAL_AXIS_Y ? 1 : 2);
    }
    else
    {
        snprintf (cmd, RS_MAX_MESSAGE_BUF_SIZE, ":FORM REAL,32;:TRAC%ld? TRACE%d", window, axis == RSSPECAN_VAL_AXIS_Y ? 1 : 2);
    }

    checkErr(RsCore_QueryFloatArrayToUserBuffer(instrSession, cmd, arraySize, results, returnedValues));

    checkErr(RsCore_Write(instrSession, ":FORM ASC"));

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  For the constellation diagram, the method returns one value pair for
/// HIFN  each constellation.
/// HIFN
/// HIFN     <I[SF0][Sym0][Carrier1]>, <Q[SF0][Sym0][Carrier1]>, ...,
/// HIFN  <I[SF0][Sym0][Carrier(n)]>, <Q[SF0][Sym0][Carrier(n)]>,
/// HIFN     <I[SF0][Sym1][Carrier1]>, <Q[SF0][Sym1][Carrier1]>, ...,
/// HIFN  <I[SF0][Sym1][Carrier(n)]>, <Q[SF0][Sym1][Carrier(n)]>,
/// HIFN     <I[SF0][Sym(n)][Carrier1]>, <Q[SF0][Sym(n)][Carrier1]>, ...,
/// HIFN  <I[SF0][Sym(n)][Carrier(n)]>, <Q[SF0][Sym(n)][Carrier(n)]>,
/// HIFN     <I[SF1][Sym0][Carrier1]>, <Q[SF1][Sym0][Carrier1]>, ...,
/// HIFN  <I[SF1][Sym0][Carrier(n)]>, <Q[SF1][Sym0][Carrier(n)]>,
/// HIFN     <I[SF1][Sym1][Carrier1]>, <Q[SF1][Sym1][Carrier1]>, ...,
/// HIFN  <I[SF1][Sym1][Carrier(n)]>, <Q[SF1][Sym1][Carrier(n)]>,
/// HIFN     <I[SF(n)][Sym(n)][Carrier1]>, <Q[SF(n)][Sym(n)][Carrier1]>, ...,
/// HIFN  <I[SF(n)][Sym(n)][Carrier(n)]>, <Q[SF(n)][Sym(n)][Carrier(n)]>
/// HIFN
/// HIFN     The I and Q values have no unit.
/// HIFN
/// HIFN     The number of return values depends on the constellation selection. By
/// HIFN  default, it returns all resource elements including the DC carrier.
/// HIFN     For each bandwidth part, the command returns the values for all
/// HIFN  resource elements. However, only the resource elements allocated to the
/// HIFN  corresponding bandwidth part have a value. All others are returnd as
/// HIFN  NAN. If you query the constellation for a signal with SS/PBCH and 2
/// HIFN  bandwidth parts, for example, the stream of values would look like
/// HIFN  this:
/// HIFN     1st block of values:
/// HIFN  NAN,NAN,(...),RE[SS/PBCH],RE[SS/PBCH],(...),NAN,NAN,(...)
/// HIFN     2nd block of values:
/// HIFN  NAN,NAN,(...),RE[BWP0],RE[BWP0],(...),NAN,NAN,(...)
/// HIFN     3rd block of values:
/// HIFN  NAN,NAN,(...),RE[BWP1],RE[BWP1],(...),NAN,NAN,(...)
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR arraySize/Number of array points requested.
/// HIPAR constellationI/Returns the Constellation I.
/// HIPAR constellationQ/Returns the Constellation Q.
/// HIPAR returnedValues/Returns the number of returned values.
ViStatus _VI_FUNC rsspecan_Fetch5GNRDLAnalysisConstellation(ViSession instrSession,
                                                            ViInt32   window,
                                                            ViInt32   arraySize,
                                                            ViReal64  constellationI[],
                                                            ViReal64  constellationQ[],
                                                            ViInt32*  returnedValues)
{
    ViStatus  error = VI_SUCCESS;
    ViPReal64 res = NULL;
    ViInt32   ret_cnt;
    ViInt32   size;
    ViChar    cmd[RS_MAX_MESSAGE_BUF_SIZE] = ":FORM REAL,32;:TRAC? TRACE1";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, window, 0, 16),
            2, "Window");

    if (window != 0)
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, ":FORM REAL,32;:TRAC%ld? TRACE1", window);
    checkErr(RsCore_QueryFloatArray(instrSession, cmd, &res, &ret_cnt));
    checkErr(rsspecan_CheckStatus(instrSession));
    checkErr(RsCore_Write(instrSession, ":FORM ASC"));

    size = (arraySize > (ret_cnt / 2)) ? ret_cnt / 2 : arraySize;

    checkErr(RsCore_DecimateViReal64ArrayToUserBuffers(res, ret_cnt, size, returnedValues, constellationI, constellationQ, NULL, NULL, NULL, NULL));

Error:
    if (res) free (res);
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  For the allocation summary, the method returns values for each line of
/// HIFN  the table.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR arraySize/Number of array points requested.
/// HIPAR bwPart/The <bwp> for bandwidth part containing the synchronization signals is
/// HIPAR bwPart/-1. For all other bandwidth parts, the corresponding bandwidth part
/// HIPAR bwPart/number, beginning with 0.
/// HIPAR subframe/Queries the subframes.
/// HIPAR slot/Queries the slot.
/// HIPAR allocationID/Queries the Allocation ID.
/// HIPAR numberOfRB/Queries the number of resouce blocks.
/// HIPAR relativePower/Queries the Relative Power.
/// HIPAR modulation/Queries the Modulation.
/// HIPAR absolutePower/Queries the Absolute Power.
/// HIPAR evm/Queries the EVM.
/// HIPAR returnedValues/Returns the number of returned values.
ViStatus _VI_FUNC rsspecan_Fetch5GNRDLAnalysisAllocationSummary(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   arraySize,
                            ViInt32   bwPart[],
                            ViInt32   subframe[],
                            ViInt32   slot[],
                            ViInt32   allocationID[],
                            ViInt32   numberOfRB[],
                            ViReal64  relativePower[],
                            ViInt32   modulation[],
                            ViReal64  absolutePower[],
                            ViReal64  evm[],
                            ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = ":TRAC? TRACE1";
    ViChar   *data = NULL;
    ViChar   *pbuf;
    ViChar   *pbufEnd;
    ViInt32  idx = 0;
    ViInt32  elements_count;

    checkErr(RsCore_LockSession(instrSession));

    if (window != 0)
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, ":TRAC%ld? TRACE1", window);
        checkErr(RsCore_QueryViStringUnknownLength(instrSession, cmd, &data));

    elements_count = RsCore_GetElementsCount(data, ',') / 9;

    if (arraySize < elements_count)
        viCheckParm(RsCore_InvalidViInt32RangeMin(instrSession, arraySize, elements_count), 3, "Array Size");

    pbuf = data;
    while (idx < (elements_count))
    {
        pbufEnd = strchr(pbuf, ',');
        if (pbufEnd == NULL)
        {
            checkErr(RS_ERROR_UNEXPECTED_RESPONSE);
        }
        *pbufEnd = 0;
        if (!isNullOrEmpty(pbuf))
            bwPart[idx] = RsCore_Convert_String_To_ViInt32(pbuf);
        else
            bwPart[idx] = RS_VAL_NAN_VI_INT32;
        pbuf = pbufEnd + 1;

        pbufEnd = strchr(pbuf, ',');
        if (pbufEnd == NULL)
        {
            checkErr(RS_ERROR_UNEXPECTED_RESPONSE);
        }
        *pbufEnd = 0;
        if (!isNullOrEmpty(pbuf))
            subframe[idx] = RsCore_Convert_String_To_ViInt32(pbuf);
        else
            subframe[idx] = RS_VAL_NAN_VI_INT32;
        pbuf = pbufEnd + 1;

        pbufEnd = strchr(pbuf, ',');
        if (pbufEnd == NULL)
        {
            checkErr(RS_ERROR_UNEXPECTED_RESPONSE);
        }
        *pbufEnd = 0;
        if (!isNullOrEmpty(pbuf))
            slot[idx] = RsCore_Convert_String_To_ViInt32(pbuf);
        else
            slot[idx] = RS_VAL_NAN_VI_INT32;
        pbuf = pbufEnd + 1;

        pbufEnd = strchr(pbuf, ',');
        if (pbufEnd == NULL)
        {
            checkErr(RS_ERROR_UNEXPECTED_RESPONSE);
        }
        *pbufEnd = 0;
        if (!isNullOrEmpty(pbuf))
            allocationID[idx] = RsCore_Convert_String_To_ViInt32(pbuf);
        else
            allocationID[idx] = RS_VAL_NAN_VI_INT32;
        pbuf = pbufEnd + 1;

        pbufEnd = strchr(pbuf, ',');
        if (pbufEnd == NULL)
        {
            checkErr(RS_ERROR_UNEXPECTED_RESPONSE);
        }
        *pbufEnd = 0;
        if (!isNullOrEmpty(pbuf))
            numberOfRB[idx] = RsCore_Convert_String_To_ViInt32(pbuf);
        else
            numberOfRB[idx] = RS_VAL_NAN_VI_INT32;
        pbuf = pbufEnd + 1;

        pbufEnd = strchr(pbuf, ',');
        if (pbufEnd == NULL)
        {
            checkErr(RS_ERROR_UNEXPECTED_RESPONSE);
        }
        *pbufEnd = 0;
        if (!isNullOrEmpty(pbuf))
            relativePower[idx] = RsCore_Convert_String_To_ViReal64(pbuf);
        else
            relativePower[idx] = RS_VAL_NAN_VI_REAL64;
        pbuf = pbufEnd + 1;

        pbufEnd = strchr(pbuf, ',');
        if (pbufEnd == NULL)
        {
            checkErr(RS_ERROR_UNEXPECTED_RESPONSE);
        }
        *pbufEnd = 0;
        if (!isNullOrEmpty(pbuf))
            modulation[idx] = RsCore_Convert_String_To_ViInt32(pbuf);
        else
            modulation[idx] = RS_VAL_NAN_VI_INT32;
        pbuf = pbufEnd + 1;

        pbufEnd = strchr(pbuf, ',');
        if (pbufEnd == NULL)
        {
            checkErr(RS_ERROR_UNEXPECTED_RESPONSE);
        }
        *pbufEnd = 0;
        if (!isNullOrEmpty(pbuf))
            absolutePower[idx] = RsCore_Convert_String_To_ViReal64(pbuf);
        else
            absolutePower[idx] = RS_VAL_NAN_VI_REAL64;
        pbuf = pbufEnd + 1;

        pbufEnd = strchr(pbuf, ',');
        if (pbufEnd == NULL)
        {
            evm[idx] = RsCore_Convert_String_To_ViReal64(pbuf);
        }
        else
        {
            *pbufEnd = 0;
            if (!isNullOrEmpty(pbuf))
                evm[idx] = RsCore_Convert_String_To_ViReal64(pbuf);
            else
                evm[idx] = RS_VAL_NAN_VI_REAL64;
            pbuf = pbufEnd + 1;
        }
        idx++;
    }

    *returnedValues = idx - 1;

Error:
    if (data) free(data);
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  For the bitstream result display, the method returns five values and
/// HIFN  the bitstream for each line of the table.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR format/Sets the bitstream format.
/// HIPAR arraySize/Sets the array size parameter.
/// HIPAR results/Returns the selected results.
/// HIPAR returnedValues/Returns the number of returned values.
ViStatus _VI_FUNC rsspecan_Fetch5GNRDLAnalysisBitstream(ViSession instrSession,
                                                        ViInt32   window,
                                                        ViInt32   format,
                                                        ViInt32   arraySize,
                                                        ViChar    results[],
                                                        ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K144"));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, window, 0, 16),
            2, "Window");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, format, RSSPECAN_VAL_NR5G_BITSTREAM_FULL, RSSPECAN_VAL_NR5G_BITSTREAM_FULL_INFO),
            3, "Format");

    if (window == 0)
    {
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, ":TRAC? TRACE%ld", format);
    }
    else
    {
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, ":TRAC%ld? TRACE%ld", window, format);
    }

    checkErr(RsCore_QueryViStringUnknownLengthToUserBuffer(instrSession, cmd, arraySize, results, returnedValues));

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  For the channel decoder results, the method returns data for each line
/// HIFN  of the table.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR arraySize/Number of array points requested.
/// HIPAR results/Returns the results.
/// HIPAR returnedValues/Returns the number of returned values.
ViStatus _VI_FUNC rsspecan_Fetch5GNRDLAnalysisChannelDecoderResults(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   arraySize,
                            ViChar    results[],
                            ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K144"));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, window, 0, 16),
            2, "Window");

    if (window == 0)
    {
        snprintf (cmd, RS_MAX_MESSAGE_BUF_SIZE, ":TRAC? PBCH");
    }
    else
    {
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, ":TRAC%ld? PBCH", window);
    }

    checkErr(RsCore_QueryViStringUnknownLengthToUserBuffer(instrSession, cmd, arraySize, results, returnedValues));

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  For the EVM vs symbol x carrier, the method returns one value for each
/// HIFN  resource element.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR trace/Selects the trace.
/// HIPAR arraySize/Number of array points requested.
/// HIPAR results/Returns the results.
/// HIPAR returnedValues/Returns the number of returned values.
ViStatus _VI_FUNC rsspecan_Fetch5GNRDLAnalysisEVMvsSymbolxCarrier(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   trace,
                            ViInt32   arraySize,
                            ViReal64  results[],
                            ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, window, 0, 16),
            2, "Window");
    checkErr(rsspecan_dataReadTrace(instrSession, window, traceArr[trace], arraySize, results, returnedValues));
    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  For the power vs symbol x carrier, the method returns one value for
/// HIFN  each resource element.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR trace/Selects the trace.
/// HIPAR arraySize/Number of array points requested.
/// HIPAR results/Returns the results.
/// HIPAR returnedValues/Returns the number of returned values.
ViStatus _VI_FUNC rsspecan_Fetch5GNRDLAnalysisPowervsSymbolxCarrier(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   trace,
                            ViInt32   arraySize,
                            ViReal64  results[],
                            ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, window, 0, 16),
            2, "Window");
    checkErr(rsspecan_dataReadTrace(instrSession, window, traceArr[trace], arraySize, results, returnedValues));
    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  For the allocation ID vs symbol x carrier, the method returns one
/// HIFN  value for each resource element.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR trace/Selects the trace.
/// HIPAR arraySize/Number of array points requested.
/// HIPAR results/For the allocation ID vs symbol x carrier, the method returns one
/// HIPAR results/value for each resource element.
/// HIPAR returnedValues/Returns the number of returned values.
ViStatus _VI_FUNC rsspecan_Fetch5GNRDLAnalysisAllocationIDvsSymbolxCarrier(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   trace,
                            ViInt32   arraySize,
                            ViInt32   results[],
                            ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;
    ViPInt32 res = NULL;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, window, 0, 16),
            2, "Window");

    if (window != 0)
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, ":TRAC%ld? %s", window, traceArr[trace]);
    else
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, ":TRAC? %s", traceArr[trace]);
        checkErr(RsCore_QueryAsciiViInt32Array(instrSession, cmd, &res, returnedValues));
    checkErr(RsCore_CopyToUserBufferViInt32Array(results, arraySize, res, (ViInt64) *returnedValues));
    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    if (res) free (res);
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  For the RS phase result display, the method returns one value for each
/// HIFN  subcarrier that has been analyzed.
/// HIFN
/// HIFN     The unit is always degrees.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR trace/Selects the trace.
/// HIPAR arraySize/Number of array points requested.
/// HIPAR results/Returns the results.
/// HIPAR returnedValues/Returns the number of returned values.
ViStatus _VI_FUNC rsspecan_Fetch5GNRDLAnalysisRSPhase(ViSession instrSession,
                                                      ViInt32   window,
                                                      ViInt32   trace,
                                                      ViInt32   arraySize,
                                                      ViReal64  results[],
                                                      ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, window, 0, 16),
            2, "Window");
    checkErr(rsspecan_dataReadTrace(instrSession, window, traceArr[trace], arraySize, results, returnedValues));
    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  For the RS phase difference result display, the method returns one
/// HIFN  value for each subcarrier that has been analyzed.
/// HIFN
/// HIFN     The unit is always degrees.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR trace/Selects the trace.
/// HIPAR arraySize/Number of array points requested.
/// HIPAR results/Returns the results.
/// HIPAR returnedValues/Returns the number of returned values.
ViStatus _VI_FUNC rsspecan_Fetch5GNRDLAnalysisRSPhaseDifference(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   trace,
                            ViInt32   arraySize,
                            ViReal64  results[],
                            ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, window, 0, 16),
            2, "Window");
    checkErr(rsspecan_dataReadTrace(instrSession, window, traceArr[trace], arraySize, results, returnedValues));
    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  For the ACLR result display.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR arraySize/Number of array points requested.
/// HIPAR results/Returns the results.
/// HIPAR returnedValues/Returns the number of returned values.
ViStatus _VI_FUNC rsspecan_Fetch5GNRDLAnalysisACLR(ViSession instrSession,
                                                   ViInt32   window,
                                                   ViInt32   arraySize,
                                                   ViReal64  results[],
                                                   ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

viCheckParm(RsCore_InvalidViInt32Range(instrSession, window, 0, 16),
        2, "Window");
    checkErr(rsspecan_dataReadTrace(instrSession, window, traceArr[1], arraySize, results, returnedValues));
    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  For the SEM  result display.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR arraySize/Number of array points requested.
/// HIPAR results/Returns the results.
/// HIPAR returnedValues/Returns the number of returned values.
ViStatus _VI_FUNC rsspecan_Fetch5GNRDLAnalysisSEM(ViSession instrSession,
                                                  ViInt32   window,
                                                  ViInt32   arraySize,
                                                  ViReal64  results[],
                                                  ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, window, 0, 16),
            2, "Window");
    checkErr(rsspecan_dataReadTrace(instrSession, window, traceArr[1], arraySize, results, returnedValues));
    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Returns the contents of the SEM table.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR arraySize/Number of array points requested.
/// HIPAR index/Returns the Index.
/// HIPAR startFrequency/Returns the Start Frequency.
/// HIPAR stopFrequency/Returns the Start Frequency.
/// HIPAR rbw/Returns the RBW.
/// HIPAR limitFailFreq/Returns the Limit Fail Frequency.
/// HIPAR absolutePower/Returns the Absolute Power.
/// HIPAR relativePower/Returns the Relative Power.
/// HIPAR limitDistance/Returns the Limit Distance.
/// HIPAR limitCheckResult/Returns the Limit Check Result.
/// HIPAR reserved1/Reserved 1.
/// HIPAR reserved2/Reserved 2.
/// HIPAR returnedValues/Returns the number of returned values.
ViStatus _VI_FUNC rsspecan_Fetch5GNRDLAnalysisSEMTable(ViSession instrSession,
                                                       ViInt32   window,
                                                       ViInt32   arraySize,
                                                       ViInt32   index[],
                                                       ViReal64  startFrequency[],
                                                       ViReal64  stopFrequency[],
                                                       ViReal64  rbw[],
                                                       ViReal64  limitFailFreq[],
                                                       ViReal64  absolutePower[],
                                                       ViReal64  relativePower[],
                                                       ViReal64  limitDistance[],
                                                       ViInt32   limitCheckResult[],
                                                       ViReal64  reserved1[],
                                                       ViReal64  reserved2[],
                                                       ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = ":TRAC? LIST";
    ViChar   *data = NULL;
    ViChar   *pbuf;
    ViInt32  i = 0;

    checkErr(RsCore_LockSession(instrSession));

    if (window != 0)
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, ":TRAC%ld? LIST", window);
        checkErr(RsCore_QueryViStringUnknownLength(instrSession, cmd, &data));
    pbuf = strtok (data, ",\n");
    if (isNullOrEmpty(pbuf))
    {
        checkErr(RS_ERROR_UNEXPECTED_RESPONSE);
    }

    while ((i < arraySize) && (!isNullOrEmpty(pbuf)))
    {
        index[i] = RsCore_Convert_String_To_ViInt32(pbuf);
        pbuf = strtok (NULL, ",\n");
        if (isNullOrEmpty(pbuf))
        {
            checkErr(RS_ERROR_UNEXPECTED_RESPONSE);
        }
        startFrequency[i] = RsCore_Convert_String_To_ViReal64(pbuf);
        pbuf = strtok (NULL, ",\n");
        if (isNullOrEmpty(pbuf))
        {
            checkErr(RS_ERROR_UNEXPECTED_RESPONSE);
        }
        stopFrequency[i] = RsCore_Convert_String_To_ViReal64(pbuf);
        pbuf = strtok (NULL, ",\n");
        if (isNullOrEmpty(pbuf))
        {
            checkErr(RS_ERROR_UNEXPECTED_RESPONSE);
        }
        rbw[i] = RsCore_Convert_String_To_ViReal64(pbuf);
        pbuf = strtok (NULL, ",\n");
        if (isNullOrEmpty(pbuf))
        {
            checkErr(RS_ERROR_UNEXPECTED_RESPONSE);
        }
        limitFailFreq[i] = RsCore_Convert_String_To_ViReal64(pbuf);
        pbuf = strtok (NULL, ",\n");
        if (isNullOrEmpty(pbuf))
        {
            checkErr(RS_ERROR_UNEXPECTED_RESPONSE);
        }
        absolutePower[i] = RsCore_Convert_String_To_ViReal64(pbuf);
        pbuf = strtok (NULL, ",\n");
        if (isNullOrEmpty(pbuf))
        {
            checkErr(RS_ERROR_UNEXPECTED_RESPONSE);
        }
        relativePower[i] = RsCore_Convert_String_To_ViReal64(pbuf);
        pbuf = strtok (NULL, ",\n");
        if (isNullOrEmpty(pbuf))
        {
            checkErr(RS_ERROR_UNEXPECTED_RESPONSE);
        }
        limitDistance[i] = RsCore_Convert_String_To_ViReal64(pbuf);
        pbuf = strtok (NULL, ",\n");
        if (isNullOrEmpty(pbuf))
        {
            checkErr(RS_ERROR_UNEXPECTED_RESPONSE);
        }
        limitCheckResult[i] = RsCore_Convert_String_To_ViInt32(pbuf);
        pbuf = strtok (NULL, ",\n");
        if (isNullOrEmpty(pbuf))
        {
            checkErr(RS_ERROR_UNEXPECTED_RESPONSE);
        }
        reserved1[i] = RsCore_Convert_String_To_ViReal64(pbuf);
        pbuf = strtok (NULL, ",\n");
        if (isNullOrEmpty(pbuf))
        {
            checkErr(RS_ERROR_UNEXPECTED_RESPONSE);
        }
        reserved2[i] = RsCore_Convert_String_To_ViReal64(pbuf);
        pbuf = strtok (NULL, ",\n");
        i++;
    }

    if (returnedValues) *returnedValues = i;

Error:
    if (data) free (data);
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  For the on/off power measurement, the number and type of return values
/// HIFN  depend on the parameter 'trace'.
/// HIFN     The unit is always dBm.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR trace/Selects the trace.
/// HIPAR arraySize/Number of array points requested.
/// HIPAR results/Returns the results.
/// HIPAR returnedValues/Returns the number of returned values.
ViStatus _VI_FUNC rsspecan_Fetch5GNRDLAnalysisOnOffPower(ViSession instrSession,
                                                         ViInt32   window,
                                                         ViInt32   trace,
                                                         ViInt32   arraySize,
                                                         ViReal64  results[],
                                                         ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, window, 0, 16),
            2, "Window");
    checkErr(rsspecan_dataReadTrace(instrSession, window, traceArr[trace], arraySize, results, returnedValues));
    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  For the on/off power measurement, the number and type of return values
/// HIFN  depend on the parameter 'trace'.
/// HIFN     The unit is always dBm.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR arraySize/Number of array points requested.
/// HIPAR offPeriodStartLimit/Returns the beginning of the 'off' period relative to the frame start.
/// HIPAR offPeriodStopLimit/Returns the end of the 'off' period relative to the frame start.
/// HIPAR timeAtDeltaToLimit/Returns the trace point at which the lowest distance between trace and
/// HIPAR timeAtDeltaToLimit/limit line has been detected.
/// HIPAR absoluteOffPower/Returns the absolute power of the signal at the trace point with the
/// HIPAR absoluteOffPower/lowest distance to the limit line.
/// HIPAR distanceToLimit/Returns the distance between the trace and the limit line of the trace
/// HIPAR distanceToLimit/point with the lowest distance to the limit line in dB.
/// HIPAR fallingTransientPeriod/Returns the length of the falling transient.
/// HIPAR risingTransientPeriod/Returns the length of the rising transient.
/// HIPAR returnedValues/Returns the number of returned values.
ViStatus _VI_FUNC rsspecan_Fetch5GNRDLAnalysisOnOffPowerTable(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   arraySize,
                            ViReal64  offPeriodStartLimit[],
                            ViReal64  offPeriodStopLimit[],
                            ViReal64  timeAtDeltaToLimit[],
                            ViReal64  absoluteOffPower[],
                            ViReal64  distanceToLimit[],
                            ViReal64  fallingTransientPeriod[],
                            ViReal64  risingTransientPeriod[],
                            ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = ":TRAC? LIST";
    ViChar   *data = NULL;
    ViChar   *pbuf;
    ViInt32  i = 0;

    checkErr(RsCore_LockSession(instrSession));

    if (window != 0)
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, ":TRAC%ld? LIST", window);
        checkErr(RsCore_QueryViStringUnknownLength(instrSession, cmd, &data));
    pbuf = strtok (data, ",\n");
    if (isNullOrEmpty(pbuf))
    {
        checkErr(RS_ERROR_UNEXPECTED_RESPONSE);
    }

    while ((i < arraySize) && (!isNullOrEmpty(pbuf)))
    {
        offPeriodStartLimit[i] = RsCore_Convert_String_To_ViReal64(pbuf);
        pbuf = strtok (NULL, ",\n");
        if (isNullOrEmpty(pbuf))
        {
            checkErr(RS_ERROR_UNEXPECTED_RESPONSE);
        }
        offPeriodStopLimit[i] = RsCore_Convert_String_To_ViReal64(pbuf);
        pbuf = strtok (NULL, ",\n");
        if (isNullOrEmpty(pbuf))
        {
            checkErr(RS_ERROR_UNEXPECTED_RESPONSE);
        }
        timeAtDeltaToLimit[i] = RsCore_Convert_String_To_ViReal64(pbuf);
        pbuf = strtok (NULL, ",\n");
        if (isNullOrEmpty(pbuf))
        {
            checkErr(RS_ERROR_UNEXPECTED_RESPONSE);
        }
        absoluteOffPower[i] = RsCore_Convert_String_To_ViReal64(pbuf);
        pbuf = strtok (NULL, ",\n");
        if (isNullOrEmpty(pbuf))
        {
            checkErr(RS_ERROR_UNEXPECTED_RESPONSE);
        }
        distanceToLimit[i] = RsCore_Convert_String_To_ViReal64(pbuf);
        pbuf = strtok (NULL, ",\n");
        if (isNullOrEmpty(pbuf))
        {
            checkErr(RS_ERROR_UNEXPECTED_RESPONSE);
        }
        fallingTransientPeriod[i] = RsCore_Convert_String_To_ViReal64(pbuf);
        pbuf = strtok (NULL, ",\n");
        if (isNullOrEmpty(pbuf))
        {
            checkErr(RS_ERROR_UNEXPECTED_RESPONSE);
        }
        risingTransientPeriod[i] = RsCore_Convert_String_To_ViReal64(pbuf);
        pbuf = strtok (NULL, ",\n");
        i++;
    }

    if (returnedValues) *returnedValues = i;

Error:
    if (data) free (data);
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the results for the PDCCH in the channel decoder.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR arraySize/Number of array points requested.
/// HIPAR results/Returns the results.
/// HIPAR returnedValues/Returns the number of returned values.
ViStatus _VI_FUNC rsspecan_Fetch5GNRDLAnalysisCORESET(ViSession instrSession,
                                                      ViInt32   window,
                                                      ViInt32   arraySize,
                                                      ViReal64  results[],
                                                      ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, window, 0, 16),
            2, "Window");
    checkErr(rsspecan_dataReadTrace(instrSession, window, "CORESET", arraySize, results, returnedValues));
    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  For the on/off power measurement, the number and type of return values
/// HIFN  depend on the parameter 'trace'.
/// HIFN     The unit is always dBm.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR trace/Selects the trace.
/// HIPAR arraySize/Number of array points requested.
/// HIPAR results/Returns the results.
/// HIPAR returnedValues/Returns the number of returned values.
ViStatus _VI_FUNC rsspecan_Fetch5GNRDLAnalysisCombinedMeasurement(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   trace,
                            ViInt32   arraySize,
                            ViReal64  results[],
                            ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, window, 0, 16),
            2, "Window");
    checkErr(rsspecan_dataReadTrace(instrSession, window, traceArr[trace], arraySize, results, returnedValues));
    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}
