/*****************************************************************************
 *  Rohde&Schwarz Spectrum Analyzer instrument driver
 *  5G NR Uplink option K145
 *
 *  Original Release: September 2019
 *  By: Petr Kominek
 *
 *  Should you have any technical questions please contact the hotline of
 *  Rohde & Schwarz Vertriebs-GmbH Rohde & Schwarz Support Center
 *
 *  e-mail: CustomerSupport@rohde-schwarz.com
 *
 *****************************************************************************/

#include "rsspecan.h"
#include "rsspecan_internal.h"

static ViString offOnArr[] = {"OFF", "ON", NULL};

/// HIFN  This function configures 5G NR Uplink.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR multicarrierFilter/Turns the multicarrier filter during demodulation on and off.
/// HIPAR componentCarriers/Selects the number of component carriers analyzed in the measurement.
/// HIPAR carrierMode/Selects the capture mode for measurements on multiple component
/// HIPAR carrierMode/carriers.
ViStatus _VI_FUNC rsspecan_Configure5GNRUL(ViSession instrSession,
                                           ViBoolean multicarrierFilter,
                                           ViInt32   componentCarriers,
                                           ViInt32   carrierMode)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_5G_NR_UL_COMPONENT_CARRIER_NUMBER, componentCarriers),
            3, "Component Carriers");

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_5G_NR_UL_CONFIGURATION_MULTICARRIER_FILTER, multicarrierFilter),
            2, "Multi Carrier Filter");

    if (componentCarriers > 1)
    {
        viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_5G_NR_UL_COMPONENT_CARRIER_DATA_CAPTURE, carrierMode),
                4, "Carrier Mode");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function configures 5G NR Data Capture.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR frameCountEnabled/Turns manual definition of number of frames to analyze on and off.
/// HIPAR autoFrameCount/Turns automatic selection of the number of frames to analyze on and
/// HIPAR autoFrameCount/off.
/// HIPAR frameCount/Defines the number of frames to analyze.
ViStatus _VI_FUNC rsspecan_Configure5GNRULDataCapture(ViSession instrSession,
                                                      ViBoolean frameCountEnabled,
                                                      ViBoolean autoFrameCount,
                                                      ViInt32   frameCount)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_5G_NR_UL_DATA_CAPTURE_FRAME_COUNT_ENABLED, frameCountEnabled),
            2, "Frame Count Enabled");
    if (frameCountEnabled == VI_TRUE)
    {

        viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_5G_NR_UL_FRAME_COUNT_AUTO, autoFrameCount),
                3, "Auto Frame Count");

        if (autoFrameCount == VI_FALSE)
        {
            viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_5G_NR_UL_DATA_CAPTURE_FRAME_COUNT, frameCount),
                    4, "Frame Count");
        }
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Selects a tab in diagrams with multiple subwindows (or views).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR tab/Selects the tab parameter.
ViStatus _VI_FUNC rsspecan_Configure5GNRULTabSelect(ViSession instrSession,
                                                    ViInt32   window,
                                                    ViInt32   tab)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf (repCap, RS_REPCAP_BUF_SIZE, "Win%ld,Tab%ld", window, tab);

    checkErr(rsspecan_SetAttributeViString(instrSession, repCap, RSSPECAN_ATTR_5G_NR_UL_TAB_SELECT, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Selects the point at which the data is demodulated for the bitstream.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR demodulationData/Selects the point at which the data is demodulated for the bitstream.
ViStatus _VI_FUNC rsspecan_Configure5GNRULDemodulation(ViSession instrSession,
                                                       ViInt32   demodulationData)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_5G_NR_UL_DEMODULATION_DECODED_DATA, demodulationData),
            2, "Demodulation Data");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects the EVM calculation method.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR EVMCalculationMethod/Selects the EVM calculation method.
ViStatus _VI_FUNC rsspecan_Configure5GNRULDemodulationEVMCalculationMethod(
                            ViSession instrSession,
                            ViInt32   EVMCalculationMethod)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_5G_NR_UL_DEMODULATION_EVM_CALCULATION_METHOD, EVMCalculationMethod),
            2, "EVM Calculation Method");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Turns the extended frequency lock range on and off.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR extendedFrequencyLockRange/Turns the extended frequency lock range on and off.
ViStatus _VI_FUNC rsspecan_Configure5GNRULDemodulationExtendedFrequencyLockRange(
                            ViSession instrSession,
                            ViBoolean extendedFrequencyLockRange)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, NULL, RSSPECAN_ATTR_5G_NR_UL_EXTENDED_FREQUENCY_LOCK_RANGE, extendedFrequencyLockRange),
            2, "Extended Frequency Lock Range");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the tpyes of traces in a diagram.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR arraySize/Defines the size of array.
/// HIPAR type/Queries the tpyes of traces in a diagram.
ViStatus _VI_FUNC rsspecan_Query5GNRULTraceType(ViSession instrSession,
                                                ViInt32   window,
                                                ViInt32   arraySize,
                                                ViChar    type[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_TRACE_TYPE,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap));

    viCheckParm(rsspecan_GetAttributeViString(instrSession, repCap, RSSPECAN_ATTR_5G_NR_UL_TRACE_TYPE,
            arraySize, type),
            4, "Type");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function queries the marker position on the z-axis of
/// HIFN  three-dimensional result displays. Instead of returning a certain type
/// HIFN  of value (EVM, Power or Allocation ID), which is possible with
/// HIFN  CALCulate<n>:MARKer<m>:Z?, returns all types of values (EVM, Power and
/// HIFN  Allocation ID), regardless of the result display type.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR arraySize/Defines the size of array.
/// HIPAR allResults/Queries the marker position on the z-axis of three-dimensional result
/// HIPAR allResults/displays. Instead of returning a certain type of value (EVM, Power or
/// HIPAR allResults/Allocation ID), which is possible with CALCulate<n>:MARKer<m>:Z?,
/// HIPAR allResults/returns all types of values (EVM, Power and Allocation ID), regardless
/// HIPAR allResults/of the result display type.
/// HIPAR marker/This control selects marker.
/// HIPAR returnedValues/Returns the number of returned values.
ViStatus _VI_FUNC rsspecan_Query5GNRULMarkerZAll(ViSession instrSession,
                                                 ViInt32   window,
                                                 ViInt32   arraySize,
                                                 ViReal64  allResults[],
                                                 ViInt32   marker,
                                                 ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K145"));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CALC%ld:MARK%ld:Z:ALL?", window, marker);
    checkErr(RsCore_QueryFloatArrayToUserBuffer(instrSession, cmd, arraySize, allResults, returnedValues));

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Turns the long capture on and off.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR longCapture/Turns the long capture on and off.
ViStatus _VI_FUNC rsspecan_Configure5GNRULLongCapture(ViSession instrSession,
                                                      ViBoolean longCapture)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_5G_NR_UL_LONG_CAPTURE, longCapture),
            2, "Long Capture");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Turns the long capture on and off.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR enabled/Turns segmented capture on and off.
/// HIPAR events/This control sets the number of trigger events for which data segments are to be captured.
/// HIPAR offset/This control defines an offset to the trigger event at which data capturing starts.
/// HIPAR length/This control defines a time period (starting from the trigger offset) in which data is captured.
ViStatus _VI_FUNC rsspecan_Configure5GNRULSegmentCapture(ViSession instrSession,
                                                         ViBoolean enabled,
                                                         ViInt32   events,
                                                         ViReal64  offset,
                                                         ViReal64  length)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, NULL, RSSPECAN_ATTR_5G_NR_UL_SEGMENT_CAPTURE, enabled),
            2, "Enabled");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, NULL, RSSPECAN_ATTR_5G_NR_UL_SEGMENT_CAPTURE_EVENTS, events),
            3, "Events");

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, NULL, RSSPECAN_ATTR_5G_NR_UL_SEGMENT_CAPTURE_OFFSET, offset),
            4, "Offset");

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, NULL, RSSPECAN_ATTR_5G_NR_UL_SEGMENT_CAPTURE_LENGTH, length),
            5, "Length");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Automatically scales the y-axis of a diagram based on the displayed
/// HIFN  results.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR subwindow/This control selects the measurement subwindow.
/// HIPAR autoScaling/Automatically scales the y-axis of a diagram based on the displayed
/// HIPAR autoScaling/results.
ViStatus _VI_FUNC rsspecan_Configure5GNRULYScalingAuto(ViSession instrSession,
                                                       ViInt32   window,
                                                       ViInt32   subwindow,
                                                       ViInt32   autoScaling)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_subw[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_Y_SCALING_AUTO,
            "Window", window - 1, RS_REPCAP_BUF_SIZE, repCap));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_Y_SCALING_AUTO,
            "Subwindow", subwindow - 1, RS_REPCAP_BUF_SIZE, repCap_subw));

    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_subw, NULL);

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_5G_NR_UL_Y_SCALING_AUTO, autoScaling),
            4, "Auto Scaling");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function configures the Y Scaling
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR subwindow/This control selects the measurement subwindow.
/// HIPAR maximum/Defines the maximum value displayed on the y-axis of a diagram.
/// HIPAR minimum/Defines the minimum value displayed on the vertical diagram axis.
ViStatus _VI_FUNC rsspecan_Configure5GNRULYScaling(ViSession instrSession,
                                                   ViInt32   window,
                                                   ViInt32   subwindow,
                                                   ViReal64  maximum,
                                                   ViReal64  minimum)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_subw[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_Y_SCALING_MAXIMUM,
            "Window", window - 1, RS_REPCAP_BUF_SIZE, repCap));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_Y_SCALING_MAXIMUM,
            "Subwindow", subwindow - 1, RS_REPCAP_BUF_SIZE, repCap_subw));

    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_subw, NULL);

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_5G_NR_UL_Y_SCALING_MAXIMUM, maximum),
            4, "Maximum");

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_5G_NR_UL_Y_SCALING_MINIMUM, minimum),
            5, "Minimum");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Selects the channel estimation method.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR method/Selects the channel estimation method.
/// HIPAR gainImbalanceCalculation/Turns the calculation of the gain imbalance and the quadrature error
/// HIPAR gainImbalanceCalculation/in the result summary on and off.
/// HIPAR trackingPhase/Turns phase tracking on and off.
ViStatus _VI_FUNC rsspecan_Configure5GNRULTracking(ViSession instrSession,
                                                   ViInt32   method,
                                                   ViBoolean gainImbalanceCalculation,
                                                   ViInt32   trackingPhase)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_5G_NR_UL_TRACKING_CHANNEL_ESTIMATION_METHOD, method),
            2, "Method");

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_5G_NR_UL_TRACKING_GAIN_IMBALANCE_CALCULATION, gainImbalanceCalculation),
            3, "Gain Imbalance Calculation");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_5G_NR_UL_TRACKING_PHASE, trackingPhase),
            4, "Tracking Phase");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Turns level tracking on and off.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR trackingLevel/Turns level tracking on and off.
ViStatus _VI_FUNC rsspecan_Configure5GNRULTrackingLevel(ViSession instrSession,
                                                        ViBoolean trackingLevel)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_5G_NR_UL_TRACKING_LEVEL, trackingLevel),
            2, "Tracking Level");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Turns time tracking on and off.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR trackingTime/Turns time tracking on and off.
ViStatus _VI_FUNC rsspecan_Configure5GNRULTrackingTime(ViSession instrSession,
                                                       ViBoolean trackingTime)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_5G_NR_UL_TRACKING_TIME, trackingTime),
            2, "Tracking Time");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Automatically demodulates the signal once.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsspecan_Configure5GNRULAutomaticallyDemodulatesOnce(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsspecan_SetAttributeViString(instrSession, NULL, RSSPECAN_ATTR_5G_NR_UL_AUTOMATICALLY_DEMODULATES_ONCE, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Selects the information that the colors of the constellation points in
/// HIFN  the constellation diagram represent.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR colors/Selects the information that the colors of the constellation points in
/// HIPAR colors/the constellation diagram represent.
ViStatus _VI_FUNC rsspecan_Configure5GNRULConstellationColor(ViSession instrSession,
                                                             ViInt32   colors)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, NULL, RSSPECAN_ATTR_5G_NR_UL_CONSTELLATION_COLORS, colors),
            2, "Colors");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Turns the consideration of a boosting factor to calculate the
/// HIFN  constellation points in the constellation diagram on and off.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR enabled/Turns the consideration of a boosting factor to calculate the
/// HIPAR enabled/constellation points in the constellation diagram on and off.
ViStatus _VI_FUNC rsspecan_Configure5GNRULBoostingFactor(ViSession instrSession,
                                                         ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, NULL, RSSPECAN_ATTR_5G_NR_UL_BOOSTING_FACTOR_ENABLED, enabled),
            2, "Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Selects the PUSCH reference data.
/// HIFN
/// HIFN     Note that when you select an ORAN test case, this setting is
/// HIFN  automatically adjusted to the ORAN test case.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR referenceData/Selects the PUSCH reference data.
ViStatus _VI_FUNC rsspecan_Configure5GNRULPUSCHReferenceData(ViSession instrSession,
                                                             ViInt32   referenceData)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, NULL, RSSPECAN_ATTR_5G_NR_UL_PUSCH_REFERENCE_DATA, referenceData),
            2, "Reference Data");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Selects the filter for suppression of neighboring channels.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR demodulationFilter/Selects the filter for suppression of neighboring channels.
ViStatus _VI_FUNC rsspecan_Configure5GNRULDemodulationFilter(ViSession instrSession,
                                                             ViInt32   demodulationFilter)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, NULL, RSSPECAN_ATTR_5G_NR_UL_DEMODULATION_FILTER, demodulationFilter),
            2, "Demodulation Filter");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Specifies if the bit stream gets displayed using bits or using
/// HIFN  symbols.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR bitStreamFormat/Specifies if the bit stream gets displayed using bits or using
/// HIPAR bitStreamFormat/symbols.
ViStatus _VI_FUNC rsspecan_Configure5GNRULBitStreamFormat(ViSession instrSession,
                                                          ViInt32   bitStreamFormat)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, NULL, RSSPECAN_ATTR_5G_NR_UL_BIT_STREAM_FORMAT, bitStreamFormat),
            2, "Bit Stream Format");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures 5G NR Uplink Physical Settings.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR operatingBand/Selects the operating band.
ViStatus _VI_FUNC rsspecan_Configure5GNRULPhysicalSettings(ViSession instrSession,
                                                           ViInt32   operatingBand)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_5G_NR_UL_PHYSICAL_SETTINGS_OPERATING_BAND, operatingBand),
            2, "Operating Band");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects the channel raster of a component carrier.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channelRaster/Selects the channel raster of a component carrier.
ViStatus _VI_FUNC rsspecan_Configure5GNRULChannelRaster(ViSession instrSession,
                                                        ViInt32   channelRaster)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, NULL, RSSPECAN_ATTR_5G_NR_UL_GENERAL_CHANNEL_RASTER, channelRaster),
            2, "Channel Raster");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function configures the frequency characteristics of all
/// HIFN  component carriers at the same time
/// HIFN     according to a certain logic.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR globalMCFrequency/Defines the global multicarrier frequency for component carrier
/// HIPAR globalMCFrequency/setups.
/// HIPAR frequencyOffsetRelativeTo/Selects the reference point for frequency offsets of component
/// HIPAR frequencyOffsetRelativeTo/carriers in a multicarrier setup.
/// HIPAR frequencyOffsetMode/Selects the frequency offset mode for component carriers in a
/// HIPAR frequencyOffsetMode/multicarrier setup.
/// HIPAR carrierSpacing/Defines the carrier spacing for equidistant frequency offsets in a
/// HIPAR carrierSpacing/multicarrier setup. This frequency offset applies to all component
/// HIPAR carrierSpacing/carriers in the setup.
/// HIPAR fixedCCOffset/Turns a fixed frequency offset for component carriers on and off.
ViStatus _VI_FUNC rsspecan_Configure5GNRULMCSetup(ViSession instrSession,
                                                  ViReal64  globalMCFrequency,
                                                  ViInt32   frequencyOffsetRelativeTo,
                                                  ViInt32   frequencyOffsetMode,
                                                  ViReal64  carrierSpacing,
                                                  ViBoolean fixedCCOffset)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, NULL, RSSPECAN_ATTR_5G_NR_UL_GENERAL_GLOBAL_MC_FREQUENCY, globalMCFrequency),
            2, "Global MC Frequency");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, NULL, RSSPECAN_ATTR_5G_NR_UL_GENERAL_FREQUENCY_OFFSET_REFERENCE_POINT, frequencyOffsetRelativeTo),
            3, "Frequency Offset Relative To");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, NULL, RSSPECAN_ATTR_5G_NR_UL_GENERAL_FREQUENCY_OFFSET_MODE, frequencyOffsetMode),
            4, "Frequency Offset Mode");
    if (frequencyOffsetMode == RSSPECAN_VAL_FREQ_OFFSET_MODE_EQUIDISTANT)
    {
        viCheckParm(rsspecan_SetAttributeViReal64(instrSession, NULL, RSSPECAN_ATTR_5G_NR_UL_GENERAL_FREQUENCY_OFFSET_CARRIER_SPACING, carrierSpacing),
                5, "Carrier Spacing");
    }
    else if (frequencyOffsetMode == RSSPECAN_VAL_FREQ_OFFSET_MODE_ARBITRARY)
    {
        viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, NULL, RSSPECAN_ATTR_5G_NR_UL_GENERAL_FREQUENCY_OFFSET_FIXED, fixedCCOffset),
                6, "Fixed CC Offset");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Defines the MC frequency offset of a component carrier. This funtion
/// HIFN  can be used when Globacl MC Frequency is set to GMC Frequency with
/// HIFN  RSSPECAN_ATTR_5G_NR_UL_GENERAL_FREQUENCY_OFFSET_REFERENCE_POINT.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR cc/Component Carrier parameter.
/// HIPAR globalMCFrequencyOffset/Defines the MC frequency offset of a component carrier.
ViStatus _VI_FUNC rsspecan_Configure5GNRULGlobalMCFrequencyOffset(
                            ViSession instrSession,
                            ViInt32   cc,
                            ViReal64  globalMCFrequencyOffset)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_COMPONENT_CARRIER_GENERAL_MC_FREQUENCY_OFFSET,
            "cc", cc - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "CC");

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_5G_NR_UL_COMPONENT_CARRIER_GENERAL_MC_FREQUENCY_OFFSET, globalMCFrequencyOffset),
            3, "Global MC Frequency Offset");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Synchronizes the global multicarrier frequency to the current center
/// HIFN  frequency (= center of all carriers).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsspecan_NR5GULSyncMCFrequencyToCenter(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsspecan_SetAttributeViString(instrSession, NULL, RSSPECAN_ATTR_5G_NR_UL_GENERAL_SYNC_GLOBAL_MC_FREQUENCY_TO_CENTER, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Restores the signal description of multiple carriers from a single
/// HIFN  file.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR loadSignalDescription/Restores the signal description of multiple carriers from a single
/// HIPAR loadSignalDescription/file.
ViStatus _VI_FUNC rsspecan_Configure5GNRULPhysicalSettingsLoadSignalDescription(
                            ViSession instrSession,
                            ViString  loadSignalDescription)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_5G_NR_UL_PHYSICAL_SETTINGS_LOAD_SIGNAL_DESCRIPTION, loadSignalDescription),
            2, "Load Signal Description");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Saves the signal description of multiple carriers in a single file.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR saveSignalDescription/Saves the signal description of multiple carriers in a single file.
ViStatus _VI_FUNC rsspecan_Configure5GNRULPhysicalSettingsSaveSignalDescription(
                            ViSession instrSession,
                            ViString  saveSignalDescription)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_5G_NR_UL_PHYSICAL_SETTINGS_SAVE_SIGNAL_DESCRIPTION, saveSignalDescription),
            2, "Save Signal Description");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Turns automatic signal demodulation on and off.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR cc/Component Carrier parameter.
/// HIPAR automaticSignalDemodulation/Turns automatic signal demodulation on and off.
ViStatus _VI_FUNC rsspecan_Configure5GNRULCCPhysicalSettingsAutomaticDemodulation(
                            ViSession instrSession,
                            ViInt32   cc,
                            ViInt32   automaticSignalDemodulation)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_BANDWIDTH_PART_AUTOMATIC_SIGNAL_DEMODULATION,
            "cc", cc - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "CC");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_5G_NR_UL_BANDWIDTH_PART_AUTOMATIC_SIGNAL_DEMODULATION, automaticSignalDemodulation),
            3, "Automatic Signal Demodulation");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Select the channel bandwidth of the 5G NR carrier.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR cc/Component Carrier parameter.
/// HIPAR bandwidth/Select the channel bandwidth of the 5G NR carrier.
/// HIPAR deploymentFrequency/Selects the deployment frequency range of the signal.
/// HIPAR centerFrequency/Sets the center frequency for RF measurements.
/// HIPAR generalFrequencyOffset/Defines the general frequency offset or the frequency offset for a
/// HIPAR generalFrequencyOffset/component carrier.
ViStatus _VI_FUNC rsspecan_Configure5GNRULCCPhysicalSettings(ViSession instrSession,
                                                             ViInt32   cc,
                                                             ViInt32   bandwidth,
                                                             ViInt32   deploymentFrequency,
                                                             ViReal64  centerFrequency,
                                                             ViReal64  generalFrequencyOffset)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_PHYSICAL_SETTINGS_BANDWIDTH,
            "cc", cc - 1, RS_REPCAP_BUF_SIZE, repCap));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_5G_NR_UL_PHYSICAL_SETTINGS_BANDWIDTH, bandwidth),
            3, "Bandwidth");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_5G_NR_UL_DEPLOYMENT_FREQUENCY_RANGE, deploymentFrequency),
            4, "Deployment Frequency");

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_5G_NR_UL_CENTER_FREQUENCY, centerFrequency),
            5, "Center Frequency");

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_5G_NR_UL_COMPONENT_CARRIER_GENERAL_FREQUENCY_OFFSET, generalFrequencyOffset),
            6, "General Frequency Offset");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Defines the cell ID.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR cc/Component Carrier parameter.
/// HIPAR cellID/Defines the cell ID.
ViStatus _VI_FUNC rsspecan_Configure5GNRULCCPhysicalSettingsCellID(
                            ViSession instrSession,
                            ViInt32   cc,
                            ViInt32   cellID)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_CELL_ID,
            "cc", cc - 1, RS_REPCAP_BUF_SIZE, repCap));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_5G_NR_UL_CELL_ID, cellID),
            3, "Cell ID");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Turns automatic detection of the cell ID on and off.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR cc/Component Carrier parameter.
/// HIPAR autoDetectionCellID/Turns automatic detection of the cell ID on and off.
ViStatus _VI_FUNC rsspecan_Configure5GNRULCCPhysicalSettingsDetection(
                            ViSession instrSession,
                            ViInt32   cc,
                            ViInt32   autoDetectionCellID)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_PHYSICAL_SETTINGS_DETECTION,
            "cc", cc - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "CC");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_5G_NR_UL_PHYSICAL_SETTINGS_DETECTION, autoDetectionCellID),
            3, "Auto Detection Cell ID");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects the power class to select the appropriate inband emission limits.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR cc/Component Carrier parameter.
/// HIPAR powerClass/Selects the power class.
ViStatus _VI_FUNC rsspecan_Configure5GNRULCCPhysicalSettingsPowerClass(
                            ViSession instrSession,
                            ViInt32   cc,
                            ViInt32   powerClass)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_POWER_CLASS,
            "cc", cc - 1, RS_REPCAP_BUF_SIZE, repCap));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_5G_NR_UL_POWER_CLASS, powerClass),
            3, "Power Class");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Restores the signal description.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR cc/Component Carrier parameter.
/// HIPAR fileName/Restores the signal description.
ViStatus _VI_FUNC rsspecan_Configure5GNRULCCPhysicalSettingsLoadSignalDescription(
                            ViSession instrSession,
                            ViInt32   cc,
                            ViString  fileName)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_LOAD_SIGNAL_DESCRIPTION,
            "cc", cc - 1, RS_REPCAP_BUF_SIZE, repCap));

    viCheckParm(rsspecan_SetAttributeViString(instrSession, repCap, RSSPECAN_ATTR_5G_NR_UL_LOAD_SIGNAL_DESCRIPTION, fileName),
            3, "File Name");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Saves the signal description.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR cc/Component Carrier parameter.
/// HIPAR fileName/Saves the signal description.
ViStatus _VI_FUNC rsspecan_Configure5GNRULCCPhysicalSettingsSaveSignalDescription(
                            ViSession instrSession,
                            ViInt32   cc,
                            ViString  fileName)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_SAVE_SIGNAL_DESCRIPTION,
            "cc", cc - 1, RS_REPCAP_BUF_SIZE, repCap));

    viCheckParm(rsspecan_SetAttributeViString(instrSession, repCap, RSSPECAN_ATTR_5G_NR_UL_SAVE_SIGNAL_DESCRIPTION, fileName),
            3, "File Name");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures 5G NR Uplink Physical PRACH Settings.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR cc/Component Carrier parameter.
/// HIPAR format/Selects the PRACH format.
/// HIPAR RBOffset/Defines the offset of the PRACH resource element in the frequency domain,
/// HIPAR RBOffset/relative to the first subcarrier in the resource grid and in terms of
/// HIPAR RBOffset/the subcarrier spacing of the PUSCH.
/// HIPAR restrictedSet/Selects the restricted set for the PRACH.
/// HIPAR zeroCorrelationZone/Selects a value that defines, in combination with the
/// HIPAR zeroCorrelationZone/restricted set, the Ncs parameter. Ncs in turn defines the
/// HIPAR zeroCorrelationZone/cylic shift of the PRACH.
/// HIPAR PRACHSCS/Select the subcarrier spacing of the PRACH.
/// HIPAR relPower/Defines the relative power of the PRACH.
/// HIPAR logicalRootSequenceIndex/Selects the logical root sequence index.
/// HIPAR preambleIndex/Defines the preamble index.
ViStatus _VI_FUNC rsspecan_Configure5GNRULCCPhysicalPRACHSettings(
                            ViSession instrSession,
                            ViInt32   cc,
                            ViInt32   format,
                            ViReal64  RBOffset,
                            ViInt32   restrictedSet,
                            ViInt32   zeroCorrelationZone,
                            ViInt32   PRACHSCS,
                            ViReal64  relPower,
                            ViInt32   logicalRootSequenceIndex,
                            ViInt32   preambleIndex)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_PRACH_PREAMBLE_FORMAT,
            "cc", cc - 1, RS_REPCAP_BUF_SIZE, repCap));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_5G_NR_UL_PRACH_PREAMBLE_FORMAT, format),
            3, "Format");

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_5G_NR_UL_PRACH_RB_OFFSET, RBOffset),
            4, "RB Offset");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_5G_NR_UL_PRACH_RESTRICTED_SET, restrictedSet),
            5, "Restricted Set");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_5G_NR_UL_PRACH_ZERO_CORRELATION_ZONE, zeroCorrelationZone),
            6, "Zero Correlation Zone");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_5G_NR_UL_PRACH_SUBCARRIER_SPACING, PRACHSCS),
            7, "PRACH SCS");

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_5G_NR_UL_PRACH_RELATIVE_POWER, relPower),
            8, "Rel Power");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_5G_NR_UL_PRACH_LOGICAL_ROOT_SEQUENCE_INDEX, logicalRootSequenceIndex),
            9, "Logical Root Sequence Index");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_5G_NR_UL_PRACH_PREAMBLE_INDEX, preambleIndex),
            10, "Preamble Index");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects the PRACH paramater LRA.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR cc/Component Carrier parameter.
/// HIPAR lra/Selects the PRACH paramater LRA.
ViStatus _VI_FUNC rsspecan_Configure5GNRULCCPhysicalPRACHLRA(ViSession instrSession,
                                                             ViInt32   cc,
                                                             ViInt32   lra)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_PRACH_LRA,
            "cc", cc - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "CC");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_5G_NR_UL_PRACH_LRA, lra),
            3, "LRA");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Adds a bandwidth part to the signal configuration.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR cc/Component Carrier parameter.
ViStatus _VI_FUNC rsspecan_Configure5GNRULBandwidthPartAdd(ViSession instrSession,
                                                           ViInt32   cc)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_BANDWIDTH_PART_ADD,
            "cc", cc - 1, RS_REPCAP_BUF_SIZE, repCap));

    checkErr(rsspecan_SetAttributeViString(instrSession, repCap, RSSPECAN_ATTR_5G_NR_UL_BANDWIDTH_PART_ADD, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Deletes a bandwidth part.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR cc/Component Carrier parameter.
/// HIPAR bandwidthPart/Sets the Bandwidth Part parameter.
ViStatus _VI_FUNC rsspecan_Configure5GNRULBandwidthPartDelete(
                            ViSession instrSession,
                            ViInt32   cc,
                            ViInt32   bandwidthPart)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_bwp[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_BANDWIDTH_PART_DELETE,
            "cc", cc - 1, RS_REPCAP_BUF_SIZE, repCap));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_BANDWIDTH_PART_DELETE,
            "BWPart", bandwidthPart , RS_REPCAP_BUF_SIZE, repCap_bwp));

    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_bwp, NULL);

    checkErr(rsspecan_SetAttributeViString(instrSession, repCap, RSSPECAN_ATTR_5G_NR_UL_BANDWIDTH_PART_DELETE, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Deletes all bandwidth parts.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR cc/Component Carrier parameter.
ViStatus _VI_FUNC rsspecan_Configure5GNRULBandwidthPartClear(ViSession instrSession,
                                                             ViInt32   cc)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_BANDWIDTH_PART_CLEAR,
            "cc", cc - 1, RS_REPCAP_BUF_SIZE, repCap));

    checkErr(rsspecan_SetAttributeViString(instrSession, repCap, RSSPECAN_ATTR_5G_NR_UL_BANDWIDTH_PART_CLEAR, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Duplicates an existing bandwidth part.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR cc/Component Carrier parameter.
/// HIPAR bandwidthPart/Sets the Bandwidth Part parameter.
ViStatus _VI_FUNC rsspecan_Configure5GNRULBandwidthPartDuplicate(
                            ViSession instrSession,
                            ViInt32   cc,
                            ViInt32   bandwidthPart)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_bwp[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_BANDWIDTH_PART_DUPLICATE,
            "cc", cc - 1, RS_REPCAP_BUF_SIZE, repCap));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_BANDWIDTH_PART_DUPLICATE,
            "BWPart", bandwidthPart , RS_REPCAP_BUF_SIZE, repCap_bwp));

    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_bwp, NULL);

    checkErr(rsspecan_SetAttributeViString(instrSession, repCap, RSSPECAN_ATTR_5G_NR_UL_BANDWIDTH_PART_DUPLICATE, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the number of analyzed bandwidth parts.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR cc/Component Carrier parameter.
/// HIPAR analyzedParts/Queries the number of analyzed bandwidth parts.
ViStatus _VI_FUNC rsspecan_Query5GNRULBandwidthPartAnalyzed(ViSession instrSession,
                                                            ViInt32   cc,
                                                            ViInt32*  analyzedParts)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_BANDWIDTH_PART_ANALYZED,
            "cc", cc - 1, RS_REPCAP_BUF_SIZE, repCap));

    viCheckParm(rsspecan_GetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_5G_NR_UL_BANDWIDTH_PART_ANALYZED, analyzedParts),
            3, "Analyzed Parts");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function configures a Bandwidth Part resource blocks.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR cc/Component Carrier parameter.
/// HIPAR bandwidthPart/Sets the Bandwidth Part parameter.
/// HIPAR count/Defines the number of physical resource blocks a bandwidth part
/// HIPAR count/occupies.
/// HIPAR offset/Defines a resource block offset for a bandwidth part.
ViStatus _VI_FUNC rsspecan_Configure5GNRULBandwidthPartResourceBlocks(
                            ViSession instrSession,
                            ViInt32   cc,
                            ViInt32   bandwidthPart,
                            ViInt32   count,
                            ViInt32   offset)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_bwp[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_BANDWIDTH_PART_RB_COUNT,
            "cc", cc - 1, RS_REPCAP_BUF_SIZE, repCap));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_BANDWIDTH_PART_RB_COUNT,
            "BWPart", bandwidthPart , RS_REPCAP_BUF_SIZE, repCap_bwp));

    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_bwp, NULL);

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_5G_NR_UL_BANDWIDTH_PART_RB_COUNT, count),
            4, "Count");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_5G_NR_UL_BANDWIDTH_PART_RB_OFFSET, offset),
            5, "Offset");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Selects the subcarrier spacing of a bandwidth part.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR cc/Component Carrier parameter.
/// HIPAR bandwidthPart/Sets the Bandwidth Part parameter.
/// HIPAR spacing/Selects the subcarrier spacing of a bandwidth part.
ViStatus _VI_FUNC rsspecan_Configure5GNRULBandwidthPartSubcarrierSpacing(
                            ViSession instrSession,
                            ViInt32   cc,
                            ViInt32   bandwidthPart,
                            ViInt32   spacing)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_bwp[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_SLOTS_COUNTS,
            "cc", cc - 1, RS_REPCAP_BUF_SIZE, repCap));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_SLOTS_COUNTS,
            "BWPart", bandwidthPart , RS_REPCAP_BUF_SIZE, repCap_bwp));

    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_bwp, NULL);

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_5G_NR_UL_BANDWIDTH_PART_SUBCARRIER_SPACING, spacing),
            4, "Spacing");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Copies a frame configuration.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR cc/Component Carrier parameter.
/// HIPAR frame/Sets the Frame parameter.
ViStatus _VI_FUNC rsspecan_Configure5GNRULFrameCopy(ViSession instrSession,
                                                    ViInt32   cc,
                                                    ViInt32   frame)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_frame[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_FRAME_COPY,
            "cc", cc - 1, RS_REPCAP_BUF_SIZE, repCap));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_FRAME_COPY,
            "NR5GFrame", frame - 1, RS_REPCAP_BUF_SIZE, repCap_frame));

    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_frame, NULL);

    checkErr(rsspecan_SetAttributeViString(instrSession, repCap, RSSPECAN_ATTR_5G_NR_UL_FRAME_COPY, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Applies an existing frame configuration to another one.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR cc/Component Carrier parameter.
/// HIPAR frame/Sets the Frame parameter.
ViStatus _VI_FUNC rsspecan_Configure5GNRULFramePaste(ViSession instrSession,
                                                     ViInt32   cc,
                                                     ViInt32   frame)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_frame[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_FRAME_PASTE,
            "cc", cc - 1 , RS_REPCAP_BUF_SIZE, repCap));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_FRAME_PASTE,
            "NR5GFrame", frame - 1, RS_REPCAP_BUF_SIZE, repCap_frame));

    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_frame, NULL);

    checkErr(rsspecan_SetAttributeViString(instrSession, repCap, RSSPECAN_ATTR_5G_NR_UL_FRAME_PASTE, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Applies an existing frame configuration to all other frames.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR cc/Component Carrier parameter.
ViStatus _VI_FUNC rsspecan_Configure5GNRULFramePasteAll(ViSession instrSession,
                                                        ViInt32   cc)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_FRAME_PASTE_ALL,
            "cc", cc - 1, RS_REPCAP_BUF_SIZE, repCap));

    checkErr(rsspecan_SetAttributeViString(instrSession, repCap, RSSPECAN_ATTR_5G_NR_UL_FRAME_PASTE_ALL, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Defines the number of slots that you can configure.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR cc/Component Carrier parameter.
/// HIPAR frame/Sets the Frame parameter.
/// HIPAR bandwidthPart/Sets the Bandwidth Part parameter.
/// HIPAR slots/Defines the number of slots that you can configure.
ViStatus _VI_FUNC rsspecan_Configure5GNRULFrameNumberOfSlots(ViSession instrSession,
                                                             ViInt32   cc,
                                                             ViInt32   frame,
                                                             ViInt32   bandwidthPart,
                                                             ViInt32   slots)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_frame[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_bwp[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_NUMBER_OF_SLOTS_TO_CONFIGURE,
            "cc", cc - 1, RS_REPCAP_BUF_SIZE, repCap));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_NUMBER_OF_SLOTS_TO_CONFIGURE,
            "NR5GFrame", frame - 1, RS_REPCAP_BUF_SIZE, repCap_frame));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_NUMBER_OF_SLOTS_TO_CONFIGURE,
            "BWPart", bandwidthPart , RS_REPCAP_BUF_SIZE, repCap_bwp));

    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_frame, NULL);
    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_bwp, NULL);

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_5G_NR_UL_NUMBER_OF_SLOTS_TO_CONFIGURE, slots),
            5, "Slots");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Turns automatic selection of the number of frames to analyze on and
/// HIFN  off.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR autoFrameCount/Turns automatic selection of the number of frames to analyze on and
/// HIPAR autoFrameCount/off.
ViStatus _VI_FUNC rsspecan_Configure5GNRULFrameCountAuto(ViSession instrSession,
                                                         ViBoolean autoFrameCount)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_5G_NR_UL_FRAME_COUNT_AUTO, autoFrameCount),
            2, "Auto Frame Count");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Defines the number of slots that are analyzed.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsspecan_Configure5GNRULFrameSlotAll(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_5G_NR_UL_FRAME_SLOT_ALL, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Defines the number of slots that are analyzed.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR slot/Defines the number of slots that are analyzed.
ViStatus _VI_FUNC rsspecan_Configure5GNRULFrameSlot(ViSession instrSession,
                                                    ViInt32   slot)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_5G_NR_UL_FRAME_SLOT, slot),
            2, "Slot");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Turns analysis of custom signals with repeating slots on and off.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR autoFrameCount/Turns analysis of custom signals with repeating slots on and off.
ViStatus _VI_FUNC rsspecan_Configure5GNRULRepeatedSlotAnalysis(
                            ViSession instrSession,
                            ViBoolean repeatedSlotAnalysis)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_5G_NR_UL_REPEATED_SLOT_ANALYSIS, repeatedSlotAnalysis),
            2, "Repeated Slot Analysis");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the number of analyzed slots.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR cc/Component Carrier parameter.
/// HIPAR frame/Sets the Frame parameter.
/// HIPAR bandwidthPart/Sets the Bandwidth Part parameter.
/// HIPAR slots/Queries the number of analyzed slots.
ViStatus _VI_FUNC rsspecan_Query5GNRULFrameNumberOfAnalyzedSlots(
                            ViSession instrSession,
                            ViInt32   cc,
                            ViInt32   frame,
                            ViInt32   bandwidthPart,
                            ViInt32*  slots)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_frame[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_bwp[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_SLOTS_COUNTS,
            "cc", cc - 1, RS_REPCAP_BUF_SIZE, repCap));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_SLOTS_COUNTS,
            "NR5GFrame", frame - 1, RS_REPCAP_BUF_SIZE, repCap_frame));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_SLOTS_COUNTS,
            "BWPart", bandwidthPart , RS_REPCAP_BUF_SIZE, repCap_bwp));

    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_frame, NULL);
    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_bwp, NULL);

    viCheckParm(rsspecan_GetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_5G_NR_UL_SLOTS_COUNTS, slots),
            5, "Slots");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function copies a slot configuration.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR cc/Component Carrier parameter.
/// HIPAR frame/Sets the Frame parameter.
/// HIPAR bandwidthPart/Sets the Bandwidth Part parameter.
/// HIPAR slot/Defines the number of slot.
ViStatus _VI_FUNC rsspecan_Configure5GNRULSlotCopyConfiguration(
                            ViSession instrSession,
                            ViInt32   cc,
                            ViInt32   frame,
                            ViInt32   bandwidthPart,
                            ViInt32   slot)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_frame[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_bwp[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_slot[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_PUSCH_COPY_SLOT_CONFIGURATION,
            "cc", cc - 1, RS_REPCAP_BUF_SIZE, repCap));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_PUSCH_COPY_SLOT_CONFIGURATION,
            "NR5GFrame", frame - 1, RS_REPCAP_BUF_SIZE, repCap_frame));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_PUSCH_COPY_SLOT_CONFIGURATION,
            "BWPart", bandwidthPart , RS_REPCAP_BUF_SIZE, repCap_bwp));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_PUSCH_COPY_SLOT_CONFIGURATION,
            "NR5GSlot", slot , RS_REPCAP_BUF_SIZE, repCap_slot));

    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_frame, NULL);
    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_bwp, NULL);
    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_slot, NULL);

    checkErr(rsspecan_SetAttributeViString(instrSession, repCap, RSSPECAN_ATTR_5G_NR_UL_PUSCH_COPY_SLOT_CONFIGURATION, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function applies an existing slot configuration to all other
/// HIFN  slots.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR cc/Component Carrier parameter.
/// HIPAR frame/Sets the Frame parameter.
/// HIPAR bandwidthPart/Sets the Bandwidth Part parameter.
/// HIPAR slot/Defines the number of slot.
ViStatus _VI_FUNC rsspecan_Configure5GNRULSlotPasteAll(ViSession instrSession,
                                                       ViInt32   cc,
                                                       ViInt32   frame,
                                                       ViInt32   bandwidthPart,
                                                       ViInt32   slot)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_frame[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_bwp[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_slot[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_SLOT_PASTE_ALL,
            "cc", cc - 1, RS_REPCAP_BUF_SIZE, repCap));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_SLOT_PASTE_ALL,
            "NR5GFrame", frame - 1, RS_REPCAP_BUF_SIZE, repCap_frame));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_SLOT_PASTE_ALL,
            "BWPart", bandwidthPart , RS_REPCAP_BUF_SIZE, repCap_bwp));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_SLOT_PASTE_ALL,
            "NR5GSlot", slot , RS_REPCAP_BUF_SIZE, repCap_slot));

    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_frame, NULL);
    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_bwp, NULL);
    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_slot, NULL);

    checkErr(rsspecan_SetAttributeViString(instrSession, repCap, RSSPECAN_ATTR_5G_NR_UL_SLOT_PASTE_ALL, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function applies an existing slot configuration to another one.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR cc/Component Carrier parameter.
/// HIPAR frame/Sets the Frame parameter.
/// HIPAR bandwidthPart/Sets the Bandwidth Part parameter.
/// HIPAR slot/Defines the number of slot.
ViStatus _VI_FUNC rsspecan_Configure5GNRULSlotPaste(ViSession instrSession,
                                                    ViInt32   cc,
                                                    ViInt32   frame,
                                                    ViInt32   bandwidthPart,
                                                    ViInt32   slot)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_frame[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_bwp[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_slot[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_SLOT_PASTE,
            "cc", cc - 1, RS_REPCAP_BUF_SIZE, repCap));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_SLOT_PASTE,
            "NR5GFrame", frame - 1, RS_REPCAP_BUF_SIZE, repCap_frame));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_SLOT_PASTE,
            "BWPart", bandwidthPart , RS_REPCAP_BUF_SIZE, repCap_bwp));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_SLOT_PASTE,
            "NR5GSlot", slot , RS_REPCAP_BUF_SIZE, repCap_slot));

    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_frame, NULL);
    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_bwp, NULL);
    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_slot, NULL);

    checkErr(rsspecan_SetAttributeViString(instrSession, repCap, RSSPECAN_ATTR_5G_NR_UL_SLOT_PASTE, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function resets the slot configuration of a bandwidth part to its
/// HIFN  default state.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR cc/Component Carrier parameter.
/// HIPAR frame/Sets the Frame parameter.
/// HIPAR bandwidthPart/Sets the Bandwidth Part parameter.
ViStatus _VI_FUNC rsspecan_Configure5GNRULSlotPreset(ViSession instrSession,
                                                     ViInt32   cc,
                                                     ViInt32   frame,
                                                     ViInt32   bandwidthPart)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_frame[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_bwp[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_SLOT_PRESET,
            "cc", cc - 1, RS_REPCAP_BUF_SIZE, repCap));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_SLOT_PRESET,
            "NR5GFrame", frame - 1, RS_REPCAP_BUF_SIZE, repCap_frame));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_SLOT_PRESET,
            "BWPart", bandwidthPart , RS_REPCAP_BUF_SIZE, repCap_bwp));

    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_frame, NULL);
    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_bwp, NULL);

    checkErr(rsspecan_SetAttributeViString(instrSession, repCap, RSSPECAN_ATTR_5G_NR_UL_SLOT_PRESET, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects the paste logic when you copy a slot configuration to other
/// HIFN  slots.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR cc/Component Carrier parameter.
/// HIPAR frame/Sets the Frame parameter.
/// HIPAR bandwidthPart/Sets the Bandwidth Part parameter.
/// HIPAR slot/Defines the number of slot.
/// HIPAR mode/Selects the paste logic when you copy a slot configuration to other
/// HIPAR mode/slots.
/// HIPAR period/Defines to which slots a slot configuration is copied to.
/// HIPAR duration/Defines to which slots a slot configuration is copied to.
/// HIPAR slots/Copies a slot configuration to a specific set of slots.
ViStatus _VI_FUNC rsspecan_Configure5GNRULSlotPasteSettings(ViSession instrSession,
                                                            ViInt32   cc,
                                                            ViInt32   frame,
                                                            ViInt32   bandwidthPart,
                                                            ViInt32   slot,
                                                            ViInt32   mode,
                                                            ViInt32   period,
                                                            ViInt32   duration,
                                                            ViString  slots)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_frame[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_bwp[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_slot[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_SLOT_PASTE_TO_MODE,
            "cc", cc - 1, RS_REPCAP_BUF_SIZE, repCap));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_SLOT_PASTE_TO_MODE,
            "NR5GFrame", frame - 1, RS_REPCAP_BUF_SIZE, repCap_frame));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_SLOT_PASTE_TO_MODE,
            "BWPart", bandwidthPart , RS_REPCAP_BUF_SIZE, repCap_bwp));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_SLOT_PASTE_TO_MODE,
            "NR5GSlot", slot , RS_REPCAP_BUF_SIZE, repCap_slot));

    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_frame, NULL);
    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_bwp, NULL);
    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_slot, NULL);

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_5G_NR_UL_SLOT_PASTE_TO_MODE, mode),
            6, "Mode");

    switch (mode)
    {
        case RSSPECAN_VAL_PASTE_MODE_CUSTOM:
            viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_5G_NR_UL_SLOT_PASTE_TO_PERIOD, period),
                    7, "Period");

            viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_5G_NR_UL_SLOT_PASTE_TO_DURATION, duration),
                    8, "Duration");
            break;
        case RSSPECAN_VAL_PASTE_MODE_SLOTS:
            viCheckParm(rsspecan_SetAttributeViString(instrSession, repCap, RSSPECAN_ATTR_5G_NR_UL_SLOT_PASTE_TO_SLOT, slots),
                    9, "Slots");

            break;
        case RSSPECAN_VAL_PASTE_MODE_DATA:
            break;
        case RSSPECAN_VAL_PASTE_MODE_UNUSED:
            break;
        default:
            checkErr(RsCore_InvalidViInt32Value(instrSession, mode));
            break;
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Copies a slot configuration to other slots.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR cc/Component Carrier parameter.
/// HIPAR frame/Sets the Frame parameter.
/// HIPAR bandwidthPart/Sets the Bandwidth Part parameter.
/// HIPAR slot/Defines the number of slot.
ViStatus _VI_FUNC rsspecan_NR5GULSlotPasteTo(ViSession instrSession,
                                             ViInt32   cc,
                                             ViInt32   frame,
                                             ViInt32   bandwidthPart,
                                             ViInt32   slot)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_frame[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_bwp[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_slot[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_SLOT_PASTE_TO,
            "cc", cc - 1, RS_REPCAP_BUF_SIZE, repCap));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_SLOT_PASTE_TO,
            "NR5GFrame", frame - 1, RS_REPCAP_BUF_SIZE, repCap_frame));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_SLOT_PASTE_TO,
            "BWPart", bandwidthPart , RS_REPCAP_BUF_SIZE, repCap_bwp));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_SLOT_PASTE_TO_MODE,
            "NR5GSlot", slot , RS_REPCAP_BUF_SIZE, repCap_slot));

    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_frame, NULL);
    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_bwp, NULL);
    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_slot, NULL);

    checkErr(rsspecan_SetAttributeViString(instrSession, repCap, RSSPECAN_ATTR_5G_NR_UL_SLOT_PASTE_TO, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Turns the sounding reference signal on and off.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR cc/Component Carrier parameter.
/// HIPAR frame/Sets the Frame parameter.
/// HIPAR bandwidthPart/Sets the Bandwidth Part parameter.
/// HIPAR enabled/Turns the sounding reference signal on and off.
ViStatus _VI_FUNC rsspecan_Configure5GNRULSRSEnabled(ViSession instrSession,
                                                     ViInt32   cc,
                                                     ViInt32   frame,
                                                     ViInt32   bandwidthPart,
                                                     ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_frame[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_bwp[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_SRS_ENABLED,
            "cc", cc - 1, RS_REPCAP_BUF_SIZE, repCap));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_SRS_ENABLED,
            "NR5GFrame", frame - 1, RS_REPCAP_BUF_SIZE, repCap_frame));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_SRS_ENABLED,
            "BWPart", bandwidthPart , RS_REPCAP_BUF_SIZE, repCap_bwp));

    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_frame, NULL);
    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_bwp, NULL);

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, repCap, RSSPECAN_ATTR_5G_NR_UL_SRS_ENABLED, enabled),
            5, "Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function configures the SRS Slot.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR cc/Component Carrier parameter.
/// HIPAR frame/Sets the Frame parameter.
/// HIPAR bandwidthPart/Sets the Bandwidth Part parameter.
/// HIPAR periodicity/Defines the periodicity of the sounding reference signal.
/// HIPAR offset/Selects the first slot the sounding reference signal appears in.
ViStatus _VI_FUNC rsspecan_Configure5GNRULSRSSlotConfig(ViSession instrSession,
                                                        ViInt32   cc,
                                                        ViInt32   frame,
                                                        ViInt32   bandwidthPart,
                                                        ViInt32   periodicity,
                                                        ViReal64  offset)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_frame[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_bwp[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_SRS_SLOT_PERIODICITY,
            "cc", cc - 1, RS_REPCAP_BUF_SIZE, repCap));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_SRS_SLOT_PERIODICITY,
            "NR5GFrame", frame - 1, RS_REPCAP_BUF_SIZE, repCap_frame));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_SRS_SLOT_PERIODICITY,
            "BWPart", bandwidthPart , RS_REPCAP_BUF_SIZE, repCap_bwp));

    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_frame, NULL);
    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_bwp, NULL);

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_5G_NR_UL_SRS_SLOT_PERIODICITY, periodicity),
            5, "Periodicity");

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_5G_NR_UL_SRS_SLOT_OFFSET, offset),
            6, "Offset");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Defines the start position of the sounding reference signal in the
/// HIFN  time domain.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR cc/Component Carrier parameter.
/// HIPAR frame/Sets the Frame parameter.
/// HIPAR bandwidthPart/Sets the Bandwidth Part parameter.
/// HIPAR startPosition/Defines the start position of the sounding reference signal in the
/// HIPAR startPosition/time domain.
ViStatus _VI_FUNC rsspecan_Configure5GNRULSRSStartPosition(ViSession instrSession,
                                                           ViInt32   cc,
                                                           ViInt32   frame,
                                                           ViInt32   bandwidthPart,
                                                           ViReal64  startPosition)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_frame[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_bwp[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_SRS_START_POSITION,
            "cc", cc - 1, RS_REPCAP_BUF_SIZE, repCap));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_SRS_START_POSITION,
            "NR5GFrame", frame - 1, RS_REPCAP_BUF_SIZE, repCap_frame));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_SRS_START_POSITION,
            "BWPart", bandwidthPart , RS_REPCAP_BUF_SIZE, repCap_bwp));

    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_frame, NULL);
    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_bwp, NULL);

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_5G_NR_UL_SRS_START_POSITION, startPosition),
            5, "Start Position");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Selects the number of OFDM symbols used by the sounding reference
/// HIFN  signal (1, 2 or 4).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR cc/Component Carrier parameter.
/// HIPAR frame/Sets the Frame parameter.
/// HIPAR bandwidthPart/Sets the Bandwidth Part parameter.
/// HIPAR symbols/Selects the number of OFDM symbols used by the sounding reference
/// HIPAR symbols/signal (1, 2 or 4).
ViStatus _VI_FUNC rsspecan_Configure5GNRULSRSSymbols(ViSession instrSession,
                                                     ViInt32   cc,
                                                     ViInt32   frame,
                                                     ViInt32   bandwidthPart,
                                                     ViInt32   symbols)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_frame[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_bwp[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_SRS_SYMBOLS,
            "cc", cc - 1, RS_REPCAP_BUF_SIZE, repCap));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_SRS_SYMBOLS,
            "NR5GFrame", frame - 1, RS_REPCAP_BUF_SIZE, repCap_frame));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_SRS_SYMBOLS,
            "BWPart", bandwidthPart , RS_REPCAP_BUF_SIZE, repCap_bwp));

    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_frame, NULL);
    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_bwp, NULL);

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_5G_NR_UL_SRS_SYMBOLS, symbols),
            5, "Symbols");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function configures SRS Frequency.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR cc/Component Carrier parameter.
/// HIPAR frame/Sets the Frame parameter.
/// HIPAR bandwidthPart/Sets the Bandwidth Part parameter.
/// HIPAR position/Defines the starting position of the sounding reference signal in the
/// HIPAR position/frequency domain.
/// HIPAR shift/Defines a shift of the sounding reference signal in the frequency
/// HIPAR shift/domain.
ViStatus _VI_FUNC rsspecan_Configure5GNRULSRSFrequency(ViSession instrSession,
                                                       ViInt32   cc,
                                                       ViInt32   frame,
                                                       ViInt32   bandwidthPart,
                                                       ViReal64  position,
                                                       ViReal64  shift)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_frame[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_bwp[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_SRS_FREQUENCY_POSITION,
            "cc", cc - 1, RS_REPCAP_BUF_SIZE, repCap));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_SRS_FREQUENCY_POSITION,
            "NR5GFrame", frame - 1, RS_REPCAP_BUF_SIZE, repCap_frame));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_SRS_FREQUENCY_POSITION,
            "BWPart", bandwidthPart , RS_REPCAP_BUF_SIZE, repCap_bwp));

    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_frame, NULL);
    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_bwp, NULL);

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_5G_NR_UL_SRS_FREQUENCY_POSITION, position),
            5, "Position");

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_5G_NR_UL_SRS_FREQUENCY_SHIFT, shift),
            6, "Shift");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function configures the frequency hopping.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR cc/Component Carrier parameter.
/// HIPAR frame/Sets the Frame parameter.
/// HIPAR bandwidthPart/Sets the Bandwidth Part parameter.
/// HIPAR bHop/Defines the frequency hopping bandwidth of the sounding reference
/// HIPAR bHop/signal (BHOP).
/// HIPAR bSRS/Defines the bandwidth of the sounding reference signal (BSRS).
/// HIPAR cSRS/Defines the bandwidth of the sounding reference signal (BSRS).
ViStatus _VI_FUNC rsspecan_Configure5GNRULSRSFrequencyHopping(
                            ViSession instrSession,
                            ViInt32   cc,
                            ViInt32   frame,
                            ViInt32   bandwidthPart,
                            ViReal64  bHop,
                            ViReal64  bSRS,
                            ViReal64  cSRS)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_frame[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_bwp[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_SRS_B_HOP,
            "cc", cc - 1, RS_REPCAP_BUF_SIZE, repCap));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_SRS_B_HOP,
            "NR5GFrame", frame - 1, RS_REPCAP_BUF_SIZE, repCap_frame));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_SRS_B_HOP,
            "BWPart", bandwidthPart , RS_REPCAP_BUF_SIZE, repCap_bwp));

    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_frame, NULL);
    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_bwp, NULL);

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_5G_NR_UL_SRS_B_HOP, bHop),
            5, "B Hop");

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_5G_NR_UL_SRS_B_SRS, bSRS),
            6, "B SRS");

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_5G_NR_UL_SRS_CSRS, cSRS),
            7, "C SRS");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Defines the repetition factor of the sounding reference signal (1, 2
/// HIFN  or 4).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR cc/Component Carrier parameter.
/// HIPAR frame/Sets the Frame parameter.
/// HIPAR bandwidthPart/Sets the Bandwidth Part parameter.
/// HIPAR repetitionFactor/Defines the repetition factor of the sounding reference signal (1, 2
/// HIPAR repetitionFactor/or 4).
ViStatus _VI_FUNC rsspecan_Configure5GNRULSRSRepetitionFactor(
                            ViSession instrSession,
                            ViInt32   cc,
                            ViInt32   frame,
                            ViInt32   bandwidthPart,
                            ViInt32   repetitionFactor)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_frame[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_bwp[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_SRS_REPETITION_FACTOR,
            "cc", cc - 1, RS_REPCAP_BUF_SIZE, repCap));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_SRS_REPETITION_FACTOR,
            "NR5GFrame", frame - 1, RS_REPCAP_BUF_SIZE, repCap_frame));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_SRS_REPETITION_FACTOR,
            "BWPart", bandwidthPart , RS_REPCAP_BUF_SIZE, repCap_bwp));

    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_frame, NULL);
    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_bwp, NULL);

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_5G_NR_UL_SRS_REPETITION_FACTOR, repetitionFactor),
            5, "Repetition Factor");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Configures the transmission comb.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR cc/Component Carrier parameter.
/// HIPAR frame/Sets the Frame parameter.
/// HIPAR bandwidthPart/Sets the Bandwidth Part parameter.
/// HIPAR transmissionComb/Selects the transmission comb for the sounding reference signal (2 or
/// HIPAR transmissionComb/4).
/// HIPAR offset/Defines an offset for the transmission comb of the sounding reference
/// HIPAR offset/signal.
/// HIPAR cyclicShift/Defines the cyclic shift of the sounding reference signal.
/// HIPAR sequenceID/Defines the pseudo-random seed value for the SRS sequence generation.
/// HIPAR hopping/Select the frequency hopping method for the sounding reference signal.
ViStatus _VI_FUNC rsspecan_Configure5GNRULSRSTransmissionComb(
                            ViSession instrSession,
                            ViInt32   cc,
                            ViInt32   frame,
                            ViInt32   bandwidthPart,
                            ViInt32   transmissionComb,
                            ViReal64  offset,
                            ViReal64  cyclicShift,
                            ViInt32   sequenceID,
                            ViInt32   hopping)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_frame[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_bwp[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_SRS_TRANSMISSION_COMB,
            "cc", cc - 1, RS_REPCAP_BUF_SIZE, repCap));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_SRS_TRANSMISSION_COMB,
            "NR5GFrame", frame - 1, RS_REPCAP_BUF_SIZE, repCap_frame));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_SRS_TRANSMISSION_COMB,
            "BWPart", bandwidthPart , RS_REPCAP_BUF_SIZE, repCap_bwp));

    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_frame, NULL);
    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_bwp, NULL);

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_5G_NR_UL_SRS_TRANSMISSION_COMB, transmissionComb),
            5, "Transmission Comb");

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_5G_NR_UL_SRS_TRANSMISSION_COMB_OFFSET, offset),
            6, "Offset");

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_5G_NR_UL_SRS_CYCLIC_SHIFT, cyclicShift),
            7, "Cyclic Shift");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_5G_NR_UL_SRS_SEQUENCE_ID, sequenceID),
            8, "Sequence ID");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_5G_NR_UL_SRS_HOPPING, hopping),
            9, "Hopping");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Defines the number of antenna ports used by the sounding reference
/// HIFN  signal (1, 2 or 4).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR cc/Component Carrier parameter.
/// HIPAR frame/Sets the Frame parameter.
/// HIPAR bandwidthPart/Sets the Bandwidth Part parameter.
/// HIPAR ports/Defines the number of antenna ports used by the sounding reference
/// HIPAR ports/signal (1, 2 or 4).
ViStatus _VI_FUNC rsspecan_Configure5GNRULSRSPorts(ViSession instrSession,
                                                   ViInt32   cc,
                                                   ViInt32   frame,
                                                   ViInt32   bandwidthPart,
                                                   ViInt32   ports)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_frame[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_bwp[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_SRS_ANTENNA_PORTS,
            "cc", cc - 1, RS_REPCAP_BUF_SIZE, repCap));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_SRS_ANTENNA_PORTS,
            "NR5GFrame", frame - 1, RS_REPCAP_BUF_SIZE, repCap_frame));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_SRS_ANTENNA_PORTS,
            "BWPart", bandwidthPart , RS_REPCAP_BUF_SIZE, repCap_bwp));

    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_frame, NULL);
    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_bwp, NULL);

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_5G_NR_UL_SRS_ANTENNA_PORTS, ports),
            5, "Ports");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Defines the relative power of the sounding reference signal.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR cc/Component Carrier parameter.
/// HIPAR frame/Sets the Frame parameter.
/// HIPAR bandwidthPart/Sets the Bandwidth Part parameter.
/// HIPAR relativePower/Defines the relative power of the sounding reference signal.
ViStatus _VI_FUNC rsspecan_Configure5GNRULSRSRelativePower(ViSession instrSession,
                                                           ViInt32   cc,
                                                           ViInt32   frame,
                                                           ViInt32   bandwidthPart,
                                                           ViReal64  relativePower)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_frame[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_bwp[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_SRS_RELATIVE_POWER,
            "cc", cc - 1, RS_REPCAP_BUF_SIZE, repCap));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_SRS_RELATIVE_POWER,
            "NR5GFrame", frame - 1, RS_REPCAP_BUF_SIZE, repCap_frame));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_SRS_RELATIVE_POWER,
            "BWPart", bandwidthPart , RS_REPCAP_BUF_SIZE, repCap_bwp));

    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_frame, NULL);
    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_bwp, NULL);

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_5G_NR_UL_SRS_RELATIVE_POWER, relativePower),
            5, "Relative Power");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Defines the user ID of a PUSCH allocation.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR cc/Component Carrier parameter.
/// HIPAR frame/Sets the Frame parameter.
/// HIPAR bandwidthPart/Sets the Bandwidth Part parameter.
/// HIPAR slot/Defines the number of slot.
/// HIPAR allocation/Defines the number of allocation.
/// HIPAR userID/Defines the user ID of a PUSCH allocation.
ViStatus _VI_FUNC rsspecan_Configure5GNRULEnhancedPUSCHUserID(
                            ViSession instrSession,
                            ViInt32   cc,
                            ViInt32   frame,
                            ViInt32   bandwidthPart,
                            ViInt32   slot,
                            ViInt32   allocation,
                            ViInt32   userID)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_frame[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_bwp[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_slot[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_core[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_ENHANCED_PUSCH_USER_ID,
            "cc", cc - 1, RS_REPCAP_BUF_SIZE, repCap));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_ENHANCED_PUSCH_USER_ID,
            "NR5GFrame", frame - 1, RS_REPCAP_BUF_SIZE, repCap_frame));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_ENHANCED_PUSCH_USER_ID,
            "BWPart", bandwidthPart , RS_REPCAP_BUF_SIZE, repCap_bwp));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_ENHANCED_PUSCH_USER_ID,
            "NR5GSlot", slot , RS_REPCAP_BUF_SIZE, repCap_slot));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_ENHANCED_PUSCH_USER_ID,
            "NR5GAlloc", allocation , RS_REPCAP_BUF_SIZE, repCap_core));

    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_frame, NULL);
    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_bwp, NULL);
    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_slot, NULL);
    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_core, NULL);

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_5G_NR_UL_ENHANCED_PUSCH_USER_ID, userID),
            7, "User ID");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Selects the codeword to layer mapping.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR cc/Component Carrier parameter.
/// HIPAR frame/Sets the Frame parameter.
/// HIPAR bandwidthPart/Sets the Bandwidth Part parameter.
/// HIPAR slot/Defines the number of slot.
/// HIPAR allocation/Defines the number of allocation.
/// HIPAR mapping/Selects the codeword to layer mapping.
ViStatus _VI_FUNC rsspecan_Configure5GNRULEnhancedPUSCHMapping(
                            ViSession instrSession,
                            ViInt32   cc,
                            ViInt32   frame,
                            ViInt32   bandwidthPart,
                            ViInt32   slot,
                            ViInt32   allocation,
                            ViInt32   mapping)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_frame[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_bwp[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_slot[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_core[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_ENHANCED_PUSCH_MAPPING,
            "cc", cc - 1, RS_REPCAP_BUF_SIZE, repCap));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_ENHANCED_PUSCH_MAPPING,
            "NR5GFrame", frame - 1, RS_REPCAP_BUF_SIZE, repCap_frame));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_ENHANCED_PUSCH_MAPPING,
            "BWPart", bandwidthPart , RS_REPCAP_BUF_SIZE, repCap_bwp));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_ENHANCED_PUSCH_MAPPING,
            "NR5GSlot", slot , RS_REPCAP_BUF_SIZE, repCap_slot));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_ENHANCED_PUSCH_MAPPING,
            "NR5GAlloc", allocation , RS_REPCAP_BUF_SIZE, repCap_core));

    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_frame, NULL);
    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_bwp, NULL);
    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_slot, NULL);
    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_core, NULL);

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_5G_NR_UL_ENHANCED_PUSCH_MAPPING, mapping),
            7, "Mapping");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Defines the relative power of a PUSCH allocation.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR cc/Component Carrier parameter.
/// HIPAR frame/Sets the Frame parameter.
/// HIPAR bandwidthPart/Sets the Bandwidth Part parameter.
/// HIPAR slot/Defines the number of slot.
/// HIPAR allocation/Defines the number of allocation.
/// HIPAR power/Defines the relative power of a PUSCH allocation.
ViStatus _VI_FUNC rsspecan_Configure5GNRULEnhancedPUSCHPower(ViSession instrSession,
                                                             ViInt32   cc,
                                                             ViInt32   frame,
                                                             ViInt32   bandwidthPart,
                                                             ViInt32   slot,
                                                             ViInt32   allocation,
                                                             ViReal64  power)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_frame[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_bwp[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_slot[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_core[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_ENHANCED_PUSCH_POWER,
            "cc", cc - 1, RS_REPCAP_BUF_SIZE, repCap));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_ENHANCED_PUSCH_POWER,
            "NR5GFrame", frame - 1, RS_REPCAP_BUF_SIZE, repCap_frame));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_ENHANCED_PUSCH_POWER,
            "BWPart", bandwidthPart , RS_REPCAP_BUF_SIZE, repCap_bwp));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_ENHANCED_PUSCH_POWER,
            "NR5GSlot", slot , RS_REPCAP_BUF_SIZE, repCap_slot));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_ENHANCED_PUSCH_POWER,
            "NR5GAlloc", allocation , RS_REPCAP_BUF_SIZE, repCap_core));

    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_frame, NULL);
    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_bwp, NULL);
    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_slot, NULL);
    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_core, NULL);

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_5G_NR_UL_ENHANCED_PUSCH_POWER, power),
            7, "Power");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures enhanced PUSCH.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR cc/Component Carrier parameter.
/// HIPAR frame/Sets the Frame parameter.
/// HIPAR bandwidthPart/Sets the Bandwidth Part parameter.
/// HIPAR slot/Defines the number of slot.
/// HIPAR allocation/Defines the number of allocation.
/// HIPAR antennaPorts/Selects the antenna ports for PUSCH transmission.
/// HIPAR cdmGroupsWithoutData/Select the CMD groups that contain no data.
ViStatus _VI_FUNC rsspecan_Configure5GNRULEnhancedPUSCHSettings(
                            ViSession instrSession,
                            ViInt32   cc,
                            ViInt32   frame,
                            ViInt32   bandwidthPart,
                            ViInt32   slot,
                            ViInt32   allocation,
                            ViString  antennaPorts,
                            ViInt32   cdmGroupsWithoutData)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_frame[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_bwp[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_slot[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_alloc[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_ENHANCED_PUSCH_ANTENNA_PORTS,
            "cc", cc - 1, RS_REPCAP_BUF_SIZE, repCap));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_ENHANCED_PUSCH_ANTENNA_PORTS,
            "NR5GFrame", frame - 1, RS_REPCAP_BUF_SIZE, repCap_frame));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_ENHANCED_PUSCH_ANTENNA_PORTS,
            "BWPart", bandwidthPart , RS_REPCAP_BUF_SIZE, repCap_bwp));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_ENHANCED_PUSCH_ANTENNA_PORTS,
            "NR5GSlot", slot , RS_REPCAP_BUF_SIZE, repCap_slot));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_ENHANCED_PUSCH_ANTENNA_PORTS,
            "NR5GAlloc", allocation , RS_REPCAP_BUF_SIZE, repCap_alloc));

    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_frame, NULL);
    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_bwp, NULL);
    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_slot, NULL);
    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_alloc, NULL);

    viCheckParm(rsspecan_SetAttributeViString(instrSession, repCap, RSSPECAN_ATTR_5G_NR_UL_ENHANCED_PUSCH_ANTENNA_PORTS, antennaPorts),
            7, "Antenna Ports");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_5G_NR_UL_ENHANCED_PUSCH_CDM_GROUPS_WITHOUT_DATA, cdmGroupsWithoutData),
            8, "CDM Groups Without Data");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the Enhanced PUSCH PTRS.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR cc/Component Carrier parameter.
/// HIPAR frame/Sets the Frame parameter.
/// HIPAR bandwidthPart/Sets the Bandwidth Part parameter.
/// HIPAR slot/Defines the number of slot.
/// HIPAR allocation/Defines the number of allocation.
/// HIPAR enabled/Turns the PTRS on and off.
/// HIPAR power/Defines the relative power of the PTRS.
/// HIPAR timeDomainOffset/Defines an offset for the PTRS in the time domain.
/// HIPAR frequencyDomainOffset/Defines an offset for the PTRS in the frequency domain.
/// HIPAR relativeOffset/Defines the location of the PTRS in the frequency domain relative to
/// HIPAR relativeOffset/the first subcarrier.
/// HIPAR group/Defines the number of PTRS groups.
/// HIPAR samples/Defines the number of samples allocated to a PTRS group.
ViStatus _VI_FUNC rsspecan_Configure5GNRULEnhancedPUSCHPTRS(ViSession instrSession,
                                                            ViInt32   cc,
                                                            ViInt32   frame,
                                                            ViInt32   bandwidthPart,
                                                            ViInt32   slot,
                                                            ViInt32   allocation,
                                                            ViBoolean enabled,
                                                            ViReal64  power,
                                                            ViInt32   timeDomainOffset,
                                                            ViInt32   frequencyDomainOffset,
                                                            ViInt32   relativeOffset,
                                                            ViInt32   group,
                                                            ViInt32   samples)
{
    ViStatus  error = VI_SUCCESS;
    ViChar    repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar    repCap_frame[RS_REPCAP_BUF_SIZE] = "";
    ViChar    repCap_bwp[RS_REPCAP_BUF_SIZE] = "";
    ViChar    repCap_slot[RS_REPCAP_BUF_SIZE] = "";
    ViChar    repCap_core[RS_REPCAP_BUF_SIZE] = "";
    ViBoolean value = VI_FALSE;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_ENHANCED_PUSCH_PTRS_ENABLED,
            "cc", cc - 1, RS_REPCAP_BUF_SIZE, repCap));

    checkErr(rsspecan_GetAttributeViBoolean(instrSession, repCap, RSSPECAN_ATTR_5G_NR_UL_ADVANCED_SETTINGS_TRANSFORM_PRECODING, &value));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_ENHANCED_PUSCH_PTRS_ENABLED,
            "NR5GFrame", frame - 1, RS_REPCAP_BUF_SIZE, repCap_frame));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_ENHANCED_PUSCH_PTRS_ENABLED,
            "BWPart", bandwidthPart , RS_REPCAP_BUF_SIZE, repCap_bwp));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_ENHANCED_PUSCH_PTRS_ENABLED,
            "NR5GSlot", slot , RS_REPCAP_BUF_SIZE, repCap_slot));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_ENHANCED_PUSCH_PTRS_ENABLED,
            "NR5GAlloc", allocation , RS_REPCAP_BUF_SIZE, repCap_core));

    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_frame, NULL);
    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_bwp, NULL);
    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_slot, NULL);
    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_core, NULL);

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, repCap, RSSPECAN_ATTR_5G_NR_UL_ENHANCED_PUSCH_PTRS_ENABLED, enabled),
            7, "Enabled");
    if (enabled == VI_TRUE)
    {
        viCheckParm(rsspecan_SetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_5G_NR_UL_ENHANCED_PUSCH_PTRS_POWER, power),
                8, "Power");

        viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_5G_NR_UL_ENHANCED_PUSCH_PTRS_TIME_DOMAIN_OFFSET, timeDomainOffset),
                9, "Time Domain Offset");

        if (value == VI_FALSE)
        {
            viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_5G_NR_UL_ENHANCED_PUSCH_PTRS_FREQUENCY_DOMAIN_OFFSET, frequencyDomainOffset),
                    10, "Frequency Domain Offset");

            viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_5G_NR_UL_ENHANCED_PUSCH_PTRS_RELATIVE_OFFSET, relativeOffset),
                    11, "Relative Offset");
        }
        else
        {
            viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_5G_NR_UL_ENHANCED_PUSCH_PTRS_GROUP, group),
                    12, "Group");

            viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_5G_NR_UL_ENHANCED_PUSCH_PTRS_SAMPLES, samples),
                    13, "Samples");
        }
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects the PUSCH PTRS sequence generation method.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR cc/Component Carrier parameter.
/// HIPAR frame/Sets the Frame parameter.
/// HIPAR bandwidthPart/Sets the Bandwidth Part parameter.
/// HIPAR slot/Defines the number of slot.
/// HIPAR allocation/Defines the number of allocation.
/// HIPAR n_id/Selects the PUSCH PTRS sequence generation method.
/// HIPAR seedValue/Defines the seed value for the PUSCH PTRS sequence generation.
ViStatus _VI_FUNC rsspecan_Configure5GNRULEnhancedPUSCHPTRSNID(
                            ViSession instrSession,
                            ViInt32   cc,
                            ViInt32   frame,
                            ViInt32   bandwidthPart,
                            ViInt32   slot,
                            ViInt32   allocation,
                            ViInt32   n_id,
                            ViInt32   seedValue)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_frame[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_bwp[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_slot[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_core[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_ENHANCED_PUSCH_PTRS_N_ID,
            "cc", cc - 1, RS_REPCAP_BUF_SIZE, repCap));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_ENHANCED_PUSCH_PTRS_N_ID,
            "NR5GFrame", frame - 1, RS_REPCAP_BUF_SIZE, repCap_frame));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_ENHANCED_PUSCH_PTRS_N_ID,
            "BWPart", bandwidthPart , RS_REPCAP_BUF_SIZE, repCap_bwp));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_ENHANCED_PUSCH_PTRS_N_ID,
            "NR5GSlot", slot , RS_REPCAP_BUF_SIZE, repCap_slot));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_ENHANCED_PUSCH_PTRS_N_ID,
            "NR5GAlloc", allocation , RS_REPCAP_BUF_SIZE, repCap_core));

    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_frame, NULL);
    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_bwp, NULL);
    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_slot, NULL);
    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_core, NULL);

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_5G_NR_UL_ENHANCED_PUSCH_PTRS_N_ID, n_id),
            7, "N_ID");

    if (n_id == RSSPECAN_VAL_PTRS_NID_NIDPUSCH)
    {
        viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_5G_NR_UL_ENHANCED_PUSCH_PTRS_N_ID_SEED_VALUE, seedValue),
                8, "Seed Value");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function configures the PDSCH DM-RS.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR cc/Component Carrier parameter.
/// HIPAR frame/Sets the Frame parameter.
/// HIPAR bandwidthPart/Sets the Bandwidth Part parameter.
/// HIPAR slot/Defines the number of slot.
/// HIPAR allocation/Defines the number of allocation.
/// HIPAR type/Selects the PDSCH DM-RS configuration type.
/// HIPAR aditional/Defines the position of additional DM-RS.
/// HIPAR length/Defines the length of the DM-RS.
/// HIPAR mappingType/Selects the mapping type of the PDSCH DM-RS.
/// HIPAR power/Defines the PDSCH DM-RS power relative to the PDSCH.
/// HIPAR sequenceGenerationMethod/Selects the PDSCH DM-RS sequence generation method.
/// HIPAR seedValue/Defines the seed value for the PUSCH DM-RS sequence generation.
/// HIPAR firstSymbol/Defines the first symbol that the DM-RS uses.
ViStatus _VI_FUNC rsspecan_Configure5GNRULEnhancedPUSCHDMRS(ViSession instrSession,
                                                            ViInt32   cc,
                                                            ViInt32   frame,
                                                            ViInt32   bandwidthPart,
                                                            ViInt32   slot,
                                                            ViInt32   allocation,
                                                            ViInt32   type,
                                                            ViInt32   aditional,
                                                            ViInt32   length,
                                                            ViInt32   mappingType,
                                                            ViReal64  power,
                                                            ViInt32   sequenceGenerationMethod,
                                                            ViInt32   seedValue,
                                                            ViInt32   firstSymbol)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_frame[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_bwp[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_slot[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_core[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_ENHANCED_PUSCH_DM_RS_TYPE,
            "cc", cc - 1, RS_REPCAP_BUF_SIZE, repCap));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_ENHANCED_PUSCH_DM_RS_TYPE,
            "NR5GFrame", frame - 1, RS_REPCAP_BUF_SIZE, repCap_frame));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_ENHANCED_PUSCH_DM_RS_TYPE,
            "BWPart", bandwidthPart , RS_REPCAP_BUF_SIZE, repCap_bwp));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_ENHANCED_PUSCH_DM_RS_TYPE,
            "NR5GSlot", slot , RS_REPCAP_BUF_SIZE, repCap_slot));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_ENHANCED_PUSCH_DM_RS_TYPE,
            "NR5GAlloc", allocation , RS_REPCAP_BUF_SIZE, repCap_core));

    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_frame, NULL);
    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_bwp, NULL);
    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_slot, NULL);
    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_core, NULL);

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_5G_NR_UL_ENHANCED_PUSCH_DM_RS_TYPE, type),
            7, "Type");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_5G_NR_UL_ENHANCED_PUSCH_ADDITIONAL_DM_RS, aditional),
            8, "Aditional");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_5G_NR_UL_ENHANCED_PUSCH_DM_RS_LENGTH, length),
            9, "Length");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_5G_NR_UL_ENHANCED_PUSCH_DM_RS_MAPPING_TYPE, mappingType),
            10, "Mapping Type");

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_5G_NR_UL_ENHANCED_PUSCH_DM_RS_POWER, power),
            11, "Power");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_5G_NR_UL_ENHANCED_PUSCH_DM_RS_SEQUENCE_GENERATION_METHOD, sequenceGenerationMethod),
            12, "Sequence Generation Method");

    if (sequenceGenerationMethod == RSSPECAN_VAL_5G_BR_UL_DMRS_METHOD_NIDPUSCH)
    {
        viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_5G_NR_UL_ENHANCED_PUSCH_DM_RS_SEED_VALUE, seedValue),
                13, "Seed Value");
    }

    if (mappingType == RSSPECAN_VAL_5G_NR_DMRS_MAPPING_A)
    {
        viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_5G_NR_UL_ENHANCED_PUSCH_DM_RS_FIRST_SYMBOL, firstSymbol),
                14, "First Symbol");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects the PDSCH DM-RS sequence generation method when you turn on
/// HIFN  transform precoding.
/// HIFN
/// HIFN     Prerequisites for this function:
/// HIFN     - Turn on transform precoding
/// HIFN  (RSSPECAN_ATTR_5G_NR_UL_ADVANCED_SETTINGS_TRANSFORM_PRECODING).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR cc/Component Carrier parameter.
/// HIPAR frame/Sets the Frame parameter.
/// HIPAR bandwidthPart/Sets the Bandwidth Part parameter.
/// HIPAR slot/Defines the number of slot.
/// HIPAR allocation/Defines the number of allocation.
/// HIPAR sequenceGenerationMethod/Selects the PDSCH DM-RS sequence generation method.
/// HIPAR seedValue/Defines the seed value for the PUSCH DM-RS sequence generation
/// HIPAR seedValue/(transform precoding = on).
ViStatus _VI_FUNC rsspecan_Configure5GNRULEnhancedPUSCHDMRSNIDRS(
                            ViSession instrSession,
                            ViInt32   cc,
                            ViInt32   frame,
                            ViInt32   bandwidthPart,
                            ViInt32   slot,
                            ViInt32   allocation,
                            ViInt32   sequenceGenerationMethod,
                            ViInt32   seedValue)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_frame[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_bwp[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_slot[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_core[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_ENHANCED_PUSCH_DM_RS_N_ID_RS,
            "cc", cc - 1, RS_REPCAP_BUF_SIZE, repCap));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_ENHANCED_PUSCH_DM_RS_N_ID_RS,
            "NR5GFrame", frame - 1, RS_REPCAP_BUF_SIZE, repCap_frame));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_ENHANCED_PUSCH_DM_RS_N_ID_RS,
            "BWPart", bandwidthPart , RS_REPCAP_BUF_SIZE, repCap_bwp));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_ENHANCED_PUSCH_DM_RS_N_ID_RS,
            "NR5GSlot", slot , RS_REPCAP_BUF_SIZE, repCap_slot));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_ENHANCED_PUSCH_DM_RS_N_ID_RS,
            "NR5GAlloc", allocation , RS_REPCAP_BUF_SIZE, repCap_core));

    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_frame, NULL);
    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_bwp, NULL);
    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_slot, NULL);
    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_core, NULL);

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_5G_NR_UL_ENHANCED_PUSCH_DM_RS_N_ID_RS, sequenceGenerationMethod),
            7, "Sequence Generation Method");

    if (sequenceGenerationMethod == RSSPECAN_VAL_NR_N_ID_RS_NIDPSCH)
    {
        viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_5G_NR_UL_ENHANCED_PUSCH_DM_RS_N_ID_RS_SEED_VALUE, seedValue),
                8, "Seed Value");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Turns low peak to average power ratio for the PUSCH DMRS on and off.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR cc/Component Carrier parameter.
/// HIPAR frame/Sets the Frame parameter.
/// HIPAR bandwidthPart/Sets the Bandwidth Part parameter.
/// HIPAR slot/Defines the number of slot.
/// HIPAR allocation/Defines the number of allocation.
/// HIPAR dmRSUplink/Turns low peak to average power ratio for the PUSCH DMRS on and off.
ViStatus _VI_FUNC rsspecan_Configure5GNRULEnhancedPUSCHDMRSUplink(
                            ViSession instrSession,
                            ViInt32   cc,
                            ViInt32   frame,
                            ViInt32   bandwidthPart,
                            ViInt32   slot,
                            ViInt32   allocation,
                            ViBoolean dmRSUplink)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_frame[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_bwp[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_slot[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_core[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_ENHANCED_PUSCH_DM_RS_UPLINK_R16,
            "cc", cc - 1, RS_REPCAP_BUF_SIZE, repCap));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_ENHANCED_PUSCH_DM_RS_UPLINK_R16,
            "NR5GFrame", frame - 1, RS_REPCAP_BUF_SIZE, repCap_frame));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_ENHANCED_PUSCH_DM_RS_UPLINK_R16,
            "BWPart", bandwidthPart , RS_REPCAP_BUF_SIZE, repCap_bwp));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_ENHANCED_PUSCH_DM_RS_UPLINK_R16,
            "NR5GSlot", slot , RS_REPCAP_BUF_SIZE, repCap_slot));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_ENHANCED_PUSCH_DM_RS_UPLINK_R16,
            "NR5GAlloc", allocation , RS_REPCAP_BUF_SIZE, repCap_core));

    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_frame, NULL);
    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_bwp, NULL);
    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_slot, NULL);
    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_core, NULL);

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, repCap, RSSPECAN_ATTR_5G_NR_UL_ENHANCED_PUSCH_DM_RS_UPLINK_R16, dmRSUplink),
            7, "DM-RS Uplink");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the number of PUSCH allocations in a slot.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR cc/Component Carrier parameter.
/// HIPAR frame/Sets the Frame parameter.
/// HIPAR bandwidthPart/Sets the Bandwidth Part parameter.
/// HIPAR slot/Defines the number of slot.
/// HIPAR allocation/Defines the number of PUSCH allocations in a slot.
ViStatus _VI_FUNC rsspecan_Configure5GNRULPUSCHAllocations(ViSession instrSession,
                                                           ViInt32   cc,
                                                           ViInt32   frame,
                                                           ViInt32   bandwidthPart,
                                                           ViInt32   slot,
                                                           ViInt32   allocation)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_frame[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_bwp[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_slot[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_PUSCH_ALLOCATIONS_IN_SLOT,
            "cc", cc - 1, RS_REPCAP_BUF_SIZE, repCap));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_PUSCH_ALLOCATIONS_IN_SLOT,
            "NR5GFrame", frame - 1, RS_REPCAP_BUF_SIZE, repCap_frame));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_PUSCH_ALLOCATIONS_IN_SLOT,
            "BWPart", bandwidthPart , RS_REPCAP_BUF_SIZE, repCap_bwp));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_PUSCH_ALLOCATIONS_IN_SLOT,
            "NR5GSlot", slot , RS_REPCAP_BUF_SIZE, repCap_slot));

    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_frame, NULL);
    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_bwp, NULL);
    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_slot, NULL);

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_5G_NR_UL_PUSCH_ALLOCATIONS_IN_SLOT, allocation),
            6, "Allocation");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function selects the allocation type of a slot.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR cc/Component Carrier parameter.
/// HIPAR frame/Sets the Frame parameter.
/// HIPAR bandwidthPart/Sets the Bandwidth Part parameter.
/// HIPAR slot/Defines the number of slot.
/// HIPAR slotType/Selects the allocation type of a slot.
ViStatus _VI_FUNC rsspecan_Configure5GNRDLPUSCHAllocationType(
                            ViSession instrSession,
                            ViInt32   cc,
                            ViInt32   frame,
                            ViInt32   bandwidthPart,
                            ViInt32   slot,
                            ViInt32   slotType)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_frame[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_bwp[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_slot[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_PUSCH_SLOT_ALLOCATION,
            "cc", cc - 1, RS_REPCAP_BUF_SIZE, repCap));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_PUSCH_SLOT_ALLOCATION,
            "NR5GFrame", frame - 1, RS_REPCAP_BUF_SIZE, repCap_frame));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_PUSCH_SLOT_ALLOCATION,
            "BWPart", bandwidthPart , RS_REPCAP_BUF_SIZE, repCap_bwp));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_PUSCH_SLOT_ALLOCATION,
            "NR5GSlot", slot , RS_REPCAP_BUF_SIZE, repCap_slot));

    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_frame, NULL);
    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_bwp, NULL);
    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_slot, NULL);

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_5G_NR_UL_PUSCH_SLOT_ALLOCATION, slotType),
            6, "Slot Type");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects the modulation of a PUSCH allocation.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR cc/Component Carrier parameter.
/// HIPAR frame/Sets the Frame parameter.
/// HIPAR bandwidthPart/Sets the Bandwidth Part parameter.
/// HIPAR slot/Defines the number of slot.
/// HIPAR allocation/Defines the number of allocation.
/// HIPAR modulation/Selects the modulation of a PUSCH allocation.
ViStatus _VI_FUNC rsspecan_Configure5GNRULPUSCH(ViSession instrSession,
                                                ViInt32   cc,
                                                ViInt32   frame,
                                                ViInt32   bandwidthPart,
                                                ViInt32   slot,
                                                ViInt32   allocation,
                                                ViInt32   modulation)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_frame[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_bwp[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_slot[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_core[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_PUSCH_MODULATION,
            "cc", cc - 1, RS_REPCAP_BUF_SIZE, repCap));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_PUSCH_MODULATION,
            "NR5GFrame", frame - 1, RS_REPCAP_BUF_SIZE, repCap_frame));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_PUSCH_MODULATION,
            "BWPart", bandwidthPart , RS_REPCAP_BUF_SIZE, repCap_bwp));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_PUSCH_MODULATION,
            "NR5GSlot", slot , RS_REPCAP_BUF_SIZE, repCap_slot));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_PUSCH_MODULATION,
            "NR5GAlloc", allocation , RS_REPCAP_BUF_SIZE, repCap_core));

    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_frame, NULL);
    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_bwp, NULL);
    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_slot, NULL);
    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_core, NULL);

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_5G_NR_UL_PUSCH_MODULATION, modulation),
            7, "Modulation");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function configures the PUSCH Resource Blocks.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR offset/Defines the resource block offset of a PUSCH allocation.
/// HIPAR cc/Component Carrier parameter.
/// HIPAR frame/Sets the Frame parameter.
/// HIPAR bandwidthPart/Sets the Bandwidth Part parameter.
/// HIPAR slot/Defines the number of slot.
/// HIPAR count/Selects the number of resource blocks that a PUSCH allocation
/// HIPAR count/occupies.
/// HIPAR allocation/Defines the number of allocation.
ViStatus _VI_FUNC rsspecan_Configure5GNRULPUSCHResourceBlocks(
                            ViSession instrSession,
                            ViInt32   cc,
                            ViInt32   frame,
                            ViInt32   bandwidthPart,
                            ViInt32   slot,
                            ViInt32   allocation,
                            ViInt32   count,
                            ViInt32   offset)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_frame[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_bwp[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_slot[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_core[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_PUSCH_RB_COUNT,
            "cc", cc - 1, RS_REPCAP_BUF_SIZE, repCap));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_PUSCH_RB_COUNT,
            "NR5GFrame", frame - 1, RS_REPCAP_BUF_SIZE, repCap_frame));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_PUSCH_RB_COUNT,
            "BWPart", bandwidthPart , RS_REPCAP_BUF_SIZE, repCap_bwp));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_PUSCH_RB_COUNT,
            "NR5GSlot", slot , RS_REPCAP_BUF_SIZE, repCap_slot));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_PUSCH_RB_COUNT,
            "NR5GAlloc", allocation , RS_REPCAP_BUF_SIZE, repCap_core));

    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_frame, NULL);
    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_bwp, NULL);
    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_slot, NULL);
    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_core, NULL);

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_5G_NR_UL_PUSCH_RB_COUNT, count),
            7, "Count");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_5G_NR_UL_PUSCH_RB_OFFSET, offset),
            8, "Offset");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function configures the PUSCH Szmbol.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR cc/Component Carrier parameter.
/// HIPAR frame/Sets the Frame parameter.
/// HIPAR bandwidthPart/Sets the Bandwidth Part parameter.
/// HIPAR slot/Defines the number of slot.
/// HIPAR allocation/Defines the number of allocation.
/// HIPAR count/Selects the number of symbols that a PUSCH allocation occupies.
/// HIPAR offset/Defines the symbol offset of a PUSCH allocation.
ViStatus _VI_FUNC rsspecan_Configure5GNRULPUSCHSymbol(ViSession instrSession,
                                                      ViInt32   cc,
                                                      ViInt32   frame,
                                                      ViInt32   bandwidthPart,
                                                      ViInt32   slot,
                                                      ViInt32   allocation,
                                                      ViInt32   count,
                                                      ViInt32   offset)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_frame[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_bwp[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_slot[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_core[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_PUSCH_SYMBOL_COUNT,
            "cc", cc - 1, RS_REPCAP_BUF_SIZE, repCap));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_PUSCH_SYMBOL_COUNT,
            "NR5GFrame", frame - 1, RS_REPCAP_BUF_SIZE, repCap_frame));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_PUSCH_SYMBOL_COUNT,
            "BWPart", bandwidthPart , RS_REPCAP_BUF_SIZE, repCap_bwp));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_PUSCH_SYMBOL_COUNT,
            "NR5GSlot", slot , RS_REPCAP_BUF_SIZE, repCap_slot));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_PUSCH_SYMBOL_COUNT,
            "NR5GAlloc", allocation , RS_REPCAP_BUF_SIZE, repCap_core));

    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_frame, NULL);
    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_bwp, NULL);
    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_slot, NULL);
    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_core, NULL);

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_5G_NR_UL_PUSCH_SYMBOL_COUNT, count),
            7, "Count");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_5G_NR_UL_PUSCH_SYMBOL_OFFSET, offset),
            8, "Offset");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function configures the PUSCH Channel Coding.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR cc/Component Carrier parameter.
/// HIPAR frame/Sets the Frame parameter.
/// HIPAR bandwidthPart/Sets the Bandwidth Part parameter.
/// HIPAR slot/Defines the number of slot.
/// HIPAR allocation/Defines the number of allocation.
/// HIPAR order/Selects the modulation order (MCS table) of the PDSCH.
/// HIPAR mcsIndex/Selects the MCS index (IMCS).
/// HIPAR redundancyVersion/Selects the redundancy version index.
ViStatus _VI_FUNC rsspecan_Configure5GNRULPUSCHChannelCoding(ViSession instrSession,
                                                             ViInt32   cc,
                                                             ViInt32   frame,
                                                             ViInt32   bandwidthPart,
                                                             ViInt32   slot,
                                                             ViInt32   allocation,
                                                             ViInt32   order,
                                                             ViInt32   mcsIndex,
                                                             ViInt32   redundancyVersion)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_frame[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_bwp[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_slot[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_core[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_PUSCH_SCRAMBLING_MCS_TABLE_ORDER,
            "cc", cc - 1, RS_REPCAP_BUF_SIZE, repCap));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_PUSCH_SCRAMBLING_MCS_TABLE_ORDER,
            "NR5GFrame", frame - 1, RS_REPCAP_BUF_SIZE, repCap_frame));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_PUSCH_SCRAMBLING_MCS_TABLE_ORDER,
            "BWPart", bandwidthPart , RS_REPCAP_BUF_SIZE, repCap_bwp));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_PUSCH_SCRAMBLING_MCS_TABLE_ORDER,
            "NR5GSlot", slot , RS_REPCAP_BUF_SIZE, repCap_slot));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_PUSCH_SCRAMBLING_MCS_TABLE_ORDER,
            "NR5GAlloc", allocation , RS_REPCAP_BUF_SIZE, repCap_core));

    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_frame, NULL);
    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_bwp, NULL);
    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_slot, NULL);
    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_core, NULL);

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_5G_NR_UL_PUSCH_SCRAMBLING_MCS_TABLE_ORDER, order),
            7, "Order");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_5G_NR_UL_PUSCH_SCRAMBLING_MCS_INDEX, mcsIndex),
            8, "MCS Index");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_5G_NR_UL_PUSCH_SCRAMBLING_REDUNDANCY_VERSION_INDEX, redundancyVersion),
            9, "Redundancy Version");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function configures the PUSCH Scrambling.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR cc/Component Carrier parameter.
/// HIPAR frame/Sets the Frame parameter.
/// HIPAR bandwidthPart/Sets the Bandwidth Part parameter.
/// HIPAR slot/Defines the number of slot.
/// HIPAR allocation/Defines the number of allocation.
/// HIPAR method/Selects the PUSCH scrambling method.
/// HIPAR defineSeed/Defines the seed value for the PDSCH scrambling.
ViStatus _VI_FUNC rsspecan_Configure5GNRULPUSCHScrambling(ViSession instrSession,
                                                          ViInt32   cc,
                                                          ViInt32   frame,
                                                          ViInt32   bandwidthPart,
                                                          ViInt32   slot,
                                                          ViInt32   allocation,
                                                          ViInt32   method,
                                                          ViInt32   defineSeed)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_frame[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_bwp[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_slot[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_core[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_PUSCH_SCRAMBLING_METHOD,
            "cc", cc - 1, RS_REPCAP_BUF_SIZE, repCap));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_PUSCH_SCRAMBLING_METHOD,
            "NR5GFrame", frame - 1, RS_REPCAP_BUF_SIZE, repCap_frame));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_PUSCH_SCRAMBLING_METHOD,
            "BWPart", bandwidthPart , RS_REPCAP_BUF_SIZE, repCap_bwp));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_PUSCH_SCRAMBLING_METHOD,
            "NR5GSlot", slot , RS_REPCAP_BUF_SIZE, repCap_slot));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_PUSCH_SCRAMBLING_METHOD,
            "NR5GAlloc", allocation , RS_REPCAP_BUF_SIZE, repCap_core));

    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_frame, NULL);
    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_bwp, NULL);
    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_slot, NULL);
    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_core, NULL);

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_5G_NR_UL_PUSCH_SCRAMBLING_METHOD, method),
            7, "Method");
    if (method == RSSPECAN_VAL_5G_NR_DMRS_METHOD_DSID)
    {
        viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_5G_NR_UL_PUSCH_SCRAMBLING_DEFINE_SEED, defineSeed),
                8, "Define Seed");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects the paste logic when you copy a PUSCH configuration to other
/// HIFN  slots.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR cc/Component Carrier parameter.
/// HIPAR frame/Sets the Frame parameter.
/// HIPAR bandwidthPart/Sets the Bandwidth Part parameter.
/// HIPAR slot/Defines the number of slot.
/// HIPAR allocation/Defines the number of allocation.
/// HIPAR mode/Selects the paste logic when you copy a PUSCH configuration to other
/// HIPAR mode/slots.
/// HIPAR period/Defines to which slots a PUSCH configuration is copied to.
/// HIPAR duration/Defines to which slots a PUSCH configuration is copied to.
/// HIPAR slots/Copies a PUSCH allocation configuration to a specific set of slots.
/// HIPAR slots/Slots are specified as comma-separated list (2,5,8), a range (3-5) or a
/// HIPAR slots/combination of both (2-5,8,10).
ViStatus _VI_FUNC rsspecan_Configure5GNRULPUSCHCopySettings(ViSession instrSession,
                                                            ViInt32   cc,
                                                            ViInt32   frame,
                                                            ViInt32   bandwidthPart,
                                                            ViInt32   slot,
                                                            ViInt32   allocation,
                                                            ViInt32   mode,
                                                            ViInt32   period,
                                                            ViInt32   duration,
                                                            ViString  slots)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_frame[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_bwp[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_slot[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_core[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_PUSCH_COPY_MODE,
            "cc", cc - 1, RS_REPCAP_BUF_SIZE, repCap));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_PUSCH_COPY_MODE,
            "NR5GFrame", frame - 1, RS_REPCAP_BUF_SIZE, repCap_frame));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_PUSCH_COPY_MODE,
            "BWPart", bandwidthPart , RS_REPCAP_BUF_SIZE, repCap_bwp));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_PUSCH_COPY_MODE,
            "NR5GSlot", slot , RS_REPCAP_BUF_SIZE, repCap_slot));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_PUSCH_COPY_MODE,
            "NR5GAlloc", allocation , RS_REPCAP_BUF_SIZE, repCap_core));

    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_frame, NULL);
    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_bwp, NULL);
    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_slot, NULL);
    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_core, NULL);

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_5G_NR_UL_PUSCH_COPY_MODE, mode),
            7, "Mode");

    switch (mode)
    {
        case RSSPECAN_VAL_COPY_MODE_CUSTOM:
            viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_5G_NR_UL_PUSCH_COPY_PERIOD, period),
                    8, "Period");

            viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_5G_NR_UL_PUSCH_COPY_DURATION, duration),
                    9, "Duration");

            break;
        case RSSPECAN_VAL_COPY_MODE_SLOT:
            viCheckParm(rsspecan_SetAttributeViString(instrSession, repCap, RSSPECAN_ATTR_5G_NR_UL_PUSCH_COPY_SLOT, slots),
                    10, "Slots");
            break;
        default:
            checkErr(RsCore_InvalidViInt32Value(instrSession, mode));
            break;
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Copies a PUSCH configuration to other slots.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR cc/Component Carrier parameter.
/// HIPAR frame/Sets the Frame parameter.
/// HIPAR bandwidthPart/Sets the Bandwidth Part parameter.
/// HIPAR slot/Defines the number of slot.
/// HIPAR allocation/Defines the number of allocation.
ViStatus _VI_FUNC rsspecan_NR5GULPUSCHCopy(ViSession instrSession,
                                           ViInt32   cc,
                                           ViInt32   frame,
                                           ViInt32   bandwidthPart,
                                           ViInt32   slot,
                                           ViInt32   allocation)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_frame[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_bwp[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_slot[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_core[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_PUSCH_COPY,
            "cc", cc - 1, RS_REPCAP_BUF_SIZE, repCap));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_PUSCH_COPY,
            "NR5GFrame", frame - 1, RS_REPCAP_BUF_SIZE, repCap_frame));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_PUSCH_COPY,
            "BWPart", bandwidthPart , RS_REPCAP_BUF_SIZE, repCap_bwp));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_PUSCH_COPY,
            "NR5GSlot", slot , RS_REPCAP_BUF_SIZE, repCap_slot));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_PUSCH_COPY,
            "NR5GAlloc", allocation , RS_REPCAP_BUF_SIZE, repCap_core));

    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_frame, NULL);
    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_bwp, NULL);
    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_slot, NULL);
    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_core, NULL);

    checkErr(rsspecan_SetAttributeViString(instrSession, repCap, RSSPECAN_ATTR_5G_NR_UL_PUSCH_COPY, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the number of PUCCHs in a slot.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR cc/Component Carrier parameter.
/// HIPAR frame/Sets the Frame parameter.
/// HIPAR bandwidthPart/Sets the Bandwidth Part parameter.
/// HIPAR slot/Defines the number of slot.
/// HIPAR pucchAllocations/Defines the number of PUCCHs in a slot.
ViStatus _VI_FUNC rsspecan_Configure5GNRULPUCCHAllocations(ViSession instrSession,
                                                           ViInt32   cc,
                                                           ViInt32   frame,
                                                           ViInt32   bandwidthPart,
                                                           ViInt32   slot,
                                                           ViInt32   pucchAllocations)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_frame[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_bwp[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_slot[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_NUMBER_OF_PUCCH,
            "cc", cc - 1, RS_REPCAP_BUF_SIZE, repCap));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_NUMBER_OF_PUCCH,
            "NR5GFrame", frame - 1, RS_REPCAP_BUF_SIZE, repCap_frame));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_NUMBER_OF_PUCCH,
            "BWPart", bandwidthPart , RS_REPCAP_BUF_SIZE, repCap_bwp));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_NUMBER_OF_PUCCH,
            "NR5GSlot", slot , RS_REPCAP_BUF_SIZE, repCap_slot));

    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_frame, NULL);
    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_bwp, NULL);
    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_slot, NULL);

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_5G_NR_UL_NUMBER_OF_PUCCH, pucchAllocations),
            6, "PUCCH Allocations");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects the number of resource blocks that a PUCCH allocation occupies
/// HIFN  and RB offset.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR cc/Component Carrier parameter.
/// HIPAR frame/Sets the Frame parameter.
/// HIPAR bandwidthPart/Sets the Bandwidth Part parameter.
/// HIPAR slot/Defines the number of slot.
/// HIPAR pucch/Defines the number of PUCCH allocations in a slot.
/// HIPAR resourceBlocks/Selects the number of resource blocks that a PUCCH allocation
/// HIPAR resourceBlocks/occupies.
/// HIPAR offsetRB/Defines the resource block offset of a PUCCH allocation.
ViStatus _VI_FUNC rsspecan_Configure5GNRULPUCCHResourceBlocks(
                            ViSession instrSession,
                            ViInt32   cc,
                            ViInt32   frame,
                            ViInt32   bandwidthPart,
                            ViInt32   slot,
                            ViInt32   pucch,
                            ViInt32   resourceBlocks,
                            ViInt32   offsetRB)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_frame[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_bwp[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_slot[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_pucch[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_PUCCH_RESOURCE_BLOCKS,
            "cc", cc - 1, RS_REPCAP_BUF_SIZE, repCap));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_PUCCH_RESOURCE_BLOCKS,
            "NR5GFrame", frame - 1, RS_REPCAP_BUF_SIZE, repCap_frame));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_PUCCH_RESOURCE_BLOCKS,
            "BWPart", bandwidthPart , RS_REPCAP_BUF_SIZE, repCap_bwp));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_PUCCH_RESOURCE_BLOCKS,
            "NR5GSlot", slot , RS_REPCAP_BUF_SIZE, repCap_slot));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_PUCCH_RESOURCE_BLOCKS,
            "NR5GPUCCH", pucch , RS_REPCAP_BUF_SIZE, repCap_pucch));

    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_frame, NULL);
    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_bwp, NULL);
    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_slot, NULL);
    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_pucch, NULL);

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_5G_NR_UL_PUCCH_RESOURCE_BLOCKS, resourceBlocks),
            7, "Resource Blocks");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_5G_NR_UL_PUCCH_RB_OFFSET, offsetRB),
            8, "Offset RB");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects the number of symbols that a PUCCH allocation occupies and
/// HIFN  Symbol offset.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR cc/Component Carrier parameter.
/// HIPAR frame/Sets the Frame parameter.
/// HIPAR bandwidthPart/Sets the Bandwidth Part parameter.
/// HIPAR slot/Defines the number of slot.
/// HIPAR pucch/Defines the number of PUCCH allocations in a slot.
/// HIPAR symbols/Selects the number of symbols that a PUCCH allocation occupies.
/// HIPAR offset/Defines the symbol offset of a PUCCH allocation.
ViStatus _VI_FUNC rsspecan_Configure5GNRULPUCCHSymbol(ViSession instrSession,
                                                      ViInt32   cc,
                                                      ViInt32   frame,
                                                      ViInt32   bandwidthPart,
                                                      ViInt32   slot,
                                                      ViInt32   pucch,
                                                      ViInt32   symbols,
                                                      ViInt32   offset)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_frame[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_bwp[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_slot[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_pucch[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_PUCCH_SYMBOLS,
            "cc", cc - 1, RS_REPCAP_BUF_SIZE, repCap));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_PUCCH_SYMBOLS,
            "NR5GFrame", frame - 1, RS_REPCAP_BUF_SIZE, repCap_frame));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_PUCCH_SYMBOLS,
            "BWPart", bandwidthPart , RS_REPCAP_BUF_SIZE, repCap_bwp));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_PUCCH_SYMBOLS,
            "NR5GSlot", slot , RS_REPCAP_BUF_SIZE, repCap_slot));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_PUCCH_SYMBOLS,
            "NR5GPUCCH", pucch , RS_REPCAP_BUF_SIZE, repCap_pucch));

    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_frame, NULL);
    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_bwp, NULL);
    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_slot, NULL);
    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_pucch, NULL);

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_5G_NR_UL_PUCCH_SYMBOLS, symbols),
            7, "Symbols");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_5G_NR_UL_PUCCH_SYMBOLS_OFFSET, offset),
            8, "Offset");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Defines the relative power of a PUCCH.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR cc/Component Carrier parameter.
/// HIPAR frame/Sets the Frame parameter.
/// HIPAR bandwidthPart/Sets the Bandwidth Part parameter.
/// HIPAR slot/Defines the number of slot.
/// HIPAR pucch/Defines the number of PUCCH allocations in a slot.
/// HIPAR relativePower/Defines the relative power of a PUCCH.
ViStatus _VI_FUNC rsspecan_Configure5GNRULPUCCHRelativePower(ViSession instrSession,
                                                             ViInt32   cc,
                                                             ViInt32   frame,
                                                             ViInt32   bandwidthPart,
                                                             ViInt32   slot,
                                                             ViInt32   pucch,
                                                             ViReal64  relativePower)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_frame[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_bwp[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_slot[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_pucch[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_PUCCH_POWER,
            "cc", cc - 1, RS_REPCAP_BUF_SIZE, repCap));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_PUCCH_POWER,
            "NR5GFrame", frame - 1, RS_REPCAP_BUF_SIZE, repCap_frame));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_PUCCH_POWER,
            "BWPart", bandwidthPart , RS_REPCAP_BUF_SIZE, repCap_bwp));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_PUCCH_POWER,
            "NR5GSlot", slot , RS_REPCAP_BUF_SIZE, repCap_slot));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_PUCCH_POWER,
            "NR5GPUCCH", pucch , RS_REPCAP_BUF_SIZE, repCap_pucch));

    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_frame, NULL);
    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_bwp, NULL);
    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_slot, NULL);
    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_pucch, NULL);

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_5G_NR_UL_PUCCH_POWER, relativePower),
            7, "Relative Power");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Selects the format of a PUCCH allocation.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR cc/Component Carrier parameter.
/// HIPAR frame/Sets the Frame parameter.
/// HIPAR bandwidthPart/Sets the Bandwidth Part parameter.
/// HIPAR slot/Defines the number of slot.
/// HIPAR pucch/Defines the number of PUCCH allocations in a slot.
/// HIPAR format/Selects the format of a PUCCH allocation.
ViStatus _VI_FUNC rsspecan_Configure5GNRULPUCCHFormat(ViSession instrSession,
                                                      ViInt32   cc,
                                                      ViInt32   frame,
                                                      ViInt32   bandwidthPart,
                                                      ViInt32   slot,
                                                      ViInt32   pucch,
                                                      ViInt32   format)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_frame[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_bwp[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_slot[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_pucch[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_PUCCH_FORMAT,
            "cc", cc - 1, RS_REPCAP_BUF_SIZE, repCap));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_PUCCH_FORMAT,
            "NR5GFrame", frame - 1, RS_REPCAP_BUF_SIZE, repCap_frame));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_PUCCH_FORMAT,
            "BWPart", bandwidthPart , RS_REPCAP_BUF_SIZE, repCap_bwp));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_PUCCH_FORMAT,
            "NR5GSlot", slot , RS_REPCAP_BUF_SIZE, repCap_slot));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_PUCCH_FORMAT,
            "NR5GPUCCH", pucch , RS_REPCAP_BUF_SIZE, repCap_pucch));

    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_frame, NULL);
    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_bwp, NULL);
    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_slot, NULL);
    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_pucch, NULL);

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_5G_NR_UL_PUCCH_FORMAT, format),
            7, "Format");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Defines the power of a PUCCH DM-RS relative to the PUCCH.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR cc/Component Carrier parameter.
/// HIPAR frame/Sets the Frame parameter.
/// HIPAR bandwidthPart/Sets the Bandwidth Part parameter.
/// HIPAR slot/Defines the number of slot.
/// HIPAR pucch/Defines the number of PUCCH allocations in a slot.
/// HIPAR dmrsRelativePower/Defines the power of a PUCCH DM-RS relative to the PUCCH.
ViStatus _VI_FUNC rsspecan_Configure5GNRULPUCCHDMRSRelativePower(
                            ViSession instrSession,
                            ViInt32   cc,
                            ViInt32   frame,
                            ViInt32   bandwidthPart,
                            ViInt32   slot,
                            ViInt32   pucch,
                            ViReal64  dmrsRelativePower)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_frame[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_bwp[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_slot[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_pucch[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_PUCCH_DMRS_POWER,
            "cc", cc - 1, RS_REPCAP_BUF_SIZE, repCap));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_PUCCH_DMRS_POWER,
            "NR5GFrame", frame - 1, RS_REPCAP_BUF_SIZE, repCap_frame));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_PUCCH_DMRS_POWER,
            "BWPart", bandwidthPart , RS_REPCAP_BUF_SIZE, repCap_bwp));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_PUCCH_DMRS_POWER,
            "NR5GSlot", slot , RS_REPCAP_BUF_SIZE, repCap_slot));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_PUCCH_DMRS_POWER,
            "NR5GPUCCH", pucch , RS_REPCAP_BUF_SIZE, repCap_pucch));

    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_frame, NULL);
    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_bwp, NULL);
    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_slot, NULL);
    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_pucch, NULL);

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_5G_NR_UL_PUCCH_DMRS_POWER, dmrsRelativePower),
            7, "DMRS Relative Power");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Turns group and sequence hopping for the PUCCH demodulation reference
/// HIFN  signal on and off
/// HIFN
/// HIFN     Note(s):
/// HIFN
/// HIFN     (1) Available for PUCCH formats 0 and 1.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR cc/Component Carrier parameter.
/// HIPAR frame/Sets the Frame parameter.
/// HIPAR bandwidthPart/Sets the Bandwidth Part parameter.
/// HIPAR slot/Defines the number of slot.
/// HIPAR pucch/Defines the number of PUCCH allocations in a slot.
/// HIPAR groupHopping/Selects the group hopping mode for the PUCCH.
/// HIPAR hoppingID/Selects the PUCCH DM-RS hopping ID generation method.
/// HIPAR seedValue/Defines the seed value for the PUCCH DM-RS hopping ID sequence
/// HIPAR seedValue/generation.
ViStatus _VI_FUNC rsspecan_Configure5GNRULPUCCHGroupHopping(ViSession instrSession,
                                                            ViInt32   cc,
                                                            ViInt32   frame,
                                                            ViInt32   bandwidthPart,
                                                            ViInt32   slot,
                                                            ViInt32   pucch,
                                                            ViInt32   groupHopping,
                                                            ViInt32   hoppingID,
                                                            ViInt32   seedValue)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_frame[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_bwp[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_slot[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_pucch[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_PUCCH_GROUP_HOPPING,
            "cc", cc - 1, RS_REPCAP_BUF_SIZE, repCap));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_PUCCH_GROUP_HOPPING,
            "NR5GFrame", frame - 1, RS_REPCAP_BUF_SIZE, repCap_frame));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_PUCCH_GROUP_HOPPING,
            "BWPart", bandwidthPart , RS_REPCAP_BUF_SIZE, repCap_bwp));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_PUCCH_GROUP_HOPPING,
            "NR5GSlot", slot , RS_REPCAP_BUF_SIZE, repCap_slot));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_PUCCH_GROUP_HOPPING,
            "NR5GPUCCH", pucch , RS_REPCAP_BUF_SIZE, repCap_pucch));

    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_frame, NULL);
    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_bwp, NULL);
    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_slot, NULL);
    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_pucch, NULL);

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_5G_NR_UL_PUCCH_GROUP_HOPPING, groupHopping),
            7, "Group Hopping");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_5G_NR_UL_PUCCH_HOPPING_ID, hoppingID),
            8, "Hopping ID");

    if (hoppingID == RSSPECAN_VAL_5G_NR_HOPPING_ID_NID)
    {
        viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_5G_NR_UL_PUCCH_SEED, seedValue),
                9, "Seed Value");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Turns hopping of the PUCCH demodulation reference signal within a slot
/// HIFN  on and off. The state of this parameter controls the value of the
/// HIFN  frequency hopping index n hop  used to calculate the group hopping
/// HIFN  sequence. When you turn on intra slot frequency hopping, you can define
/// HIFN  the physical resource
/// HIFN     block offset for the second hop ('Second Hop PRB').
/// HIFN
/// HIFN     Note(s):
/// HIFN
/// HIFN     (1) Available for PUCCH formats 0 and 1.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR cc/Component Carrier parameter.
/// HIPAR frame/Sets the Frame parameter.
/// HIPAR bandwidthPart/Sets the Bandwidth Part parameter.
/// HIPAR slot/Defines the number of slot.
/// HIPAR pucch/Defines the number of PUCCH allocations in a slot.
/// HIPAR intraSlotFrequencyHopping/Selects the group hopping mode for the PUCCH.
/// HIPAR secondHopPRB/Selects the second hop PRB.
ViStatus _VI_FUNC rsspecan_Configure5GNRULPUCCHIntraSlotFrequencyHopping(
                            ViSession instrSession,
                            ViInt32   cc,
                            ViInt32   frame,
                            ViInt32   bandwidthPart,
                            ViInt32   slot,
                            ViInt32   pucch,
                            ViBoolean intraSlotFrequencyHopping,
                            ViInt32   secondHopPRB)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_frame[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_bwp[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_slot[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_pucch[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_PUCCH_INTRA_SLOT_ENABLED,
            "cc", cc - 1, RS_REPCAP_BUF_SIZE, repCap));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_PUCCH_INTRA_SLOT_ENABLED,
            "NR5GFrame", frame - 1, RS_REPCAP_BUF_SIZE, repCap_frame));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_PUCCH_INTRA_SLOT_ENABLED,
            "BWPart", bandwidthPart , RS_REPCAP_BUF_SIZE, repCap_bwp));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_PUCCH_INTRA_SLOT_ENABLED,
            "NR5GSlot", slot , RS_REPCAP_BUF_SIZE, repCap_slot));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_PUCCH_INTRA_SLOT_ENABLED,
            "NR5GPUCCH", pucch , RS_REPCAP_BUF_SIZE, repCap_pucch));

    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_frame, NULL);
    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_bwp, NULL);
    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_slot, NULL);
    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_pucch, NULL);

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, repCap, RSSPECAN_ATTR_5G_NR_UL_PUCCH_INTRA_SLOT_ENABLED, intraSlotFrequencyHopping),
            7, "Intra Slot Frequency Hopping");

    if (intraSlotFrequencyHopping == VI_TRUE)
    {
        viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_5G_NR_UL_PUCCH_HOP_PRB, secondHopPRB),
                8, "Second Hop PRB");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  As defined in TS 38.211, the value is required to calculate the cyclic
/// HIFN  shift for the hopping sequence and thus defines the frequency positions
/// HIFN  of the different hops.
/// HIFN
/// HIFN     Note(s):
/// HIFN
/// HIFN     (1) Available for PUCCH formats 0 and 1.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR cc/Component Carrier parameter.
/// HIPAR frame/Sets the Frame parameter.
/// HIPAR bandwidthPart/Sets the Bandwidth Part parameter.
/// HIPAR slot/Defines the number of slot.
/// HIPAR pucch/Defines the number of PUCCH allocations in a slot.
/// HIPAR initialCyclicShift/Selects the initial cyclic shift.
ViStatus _VI_FUNC rsspecan_Configure5GNRULPUCCHInitialCyclicShift(
                            ViSession instrSession,
                            ViInt32   cc,
                            ViInt32   frame,
                            ViInt32   bandwidthPart,
                            ViInt32   slot,
                            ViInt32   pucch,
                            ViInt32   initialCyclicShift)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_frame[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_bwp[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_slot[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_pucch[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_PUCCH_INITIAL_CYCLIC_SHIFT,
            "cc", cc - 1, RS_REPCAP_BUF_SIZE, repCap));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_PUCCH_INITIAL_CYCLIC_SHIFT,
            "NR5GFrame", frame - 1, RS_REPCAP_BUF_SIZE, repCap_frame));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_PUCCH_INITIAL_CYCLIC_SHIFT,
            "BWPart", bandwidthPart , RS_REPCAP_BUF_SIZE, repCap_bwp));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_PUCCH_INITIAL_CYCLIC_SHIFT,
            "NR5GSlot", slot , RS_REPCAP_BUF_SIZE, repCap_slot));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_PUCCH_INITIAL_CYCLIC_SHIFT,
            "NR5GPUCCH", pucch , RS_REPCAP_BUF_SIZE, repCap_pucch));

    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_frame, NULL);
    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_bwp, NULL);
    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_slot, NULL);
    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_pucch, NULL);

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_5G_NR_UL_PUCCH_INITIAL_CYCLIC_SHIFT, initialCyclicShift),
            7, "Initial Cyclic Shift");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Defines the orthogonal sequence index I. As defined in TS 38.211, the
/// HIFN  value is required to select the orthogonal sequence, used for the
/// HIFN  blockwise spreading.
/// HIFN
/// HIFN     Note(s):
/// HIFN
/// HIFN     (1) Available for PUCCH formats 1.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR cc/Component Carrier parameter.
/// HIPAR frame/Sets the Frame parameter.
/// HIPAR bandwidthPart/Sets the Bandwidth Part parameter.
/// HIPAR slot/Defines the number of slot.
/// HIPAR pucch/Defines the number of PUCCH allocations in a slot.
/// HIPAR timeDomainOCCIndex/Defines the orthogonal sequence index I. As defined in TS 38.211, the
/// HIPAR timeDomainOCCIndex/value is required to select the orthogonal sequence, used for the
/// HIPAR timeDomainOCCIndex/blockwise spreading.
ViStatus _VI_FUNC rsspecan_Configure5GNRULPUCCHTimeDomainOCCIndex(
                            ViSession instrSession,
                            ViInt32   cc,
                            ViInt32   frame,
                            ViInt32   bandwidthPart,
                            ViInt32   slot,
                            ViInt32   pucch,
                            ViInt32   timeDomainOCCIndex)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_frame[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_bwp[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_slot[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_pucch[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_PUCCH_DMRS_TIME_DOMAIN_INDEX,
            "cc", cc - 1, RS_REPCAP_BUF_SIZE, repCap));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_PUCCH_DMRS_TIME_DOMAIN_INDEX,
            "NR5GFrame", frame - 1, RS_REPCAP_BUF_SIZE, repCap_frame));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_PUCCH_DMRS_TIME_DOMAIN_INDEX,
            "BWPart", bandwidthPart , RS_REPCAP_BUF_SIZE, repCap_bwp));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_PUCCH_DMRS_TIME_DOMAIN_INDEX,
            "NR5GSlot", slot , RS_REPCAP_BUF_SIZE, repCap_slot));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_PUCCH_DMRS_TIME_DOMAIN_INDEX,
            "NR5GPUCCH", pucch , RS_REPCAP_BUF_SIZE, repCap_pucch));

    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_frame, NULL);
    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_bwp, NULL);
    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_slot, NULL);
    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_pucch, NULL);

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_5G_NR_UL_PUCCH_DMRS_TIME_DOMAIN_INDEX, timeDomainOCCIndex),
            7, "Time Domain OCC Index");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects the OCC length.
/// HIFN
/// HIFN     Note(s):
/// HIFN
/// HIFN     (1) Available for PUCCH formats 1.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR cc/Component Carrier parameter.
/// HIPAR frame/Sets the Frame parameter.
/// HIPAR bandwidthPart/Sets the Bandwidth Part parameter.
/// HIPAR slot/Defines the number of slot.
/// HIPAR pucch/Defines the number of PUCCH allocations in a slot.
/// HIPAR occLength/Selects the OCC length.
ViStatus _VI_FUNC rsspecan_Configure5GNRULPUCCHTimeDomainOCCLength(
                            ViSession instrSession,
                            ViInt32   cc,
                            ViInt32   frame,
                            ViInt32   bandwidthPart,
                            ViInt32   slot,
                            ViInt32   pucch,
                            ViInt32   occLength)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_frame[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_bwp[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_slot[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_pucch[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_PUCCH_DMRS_OCC_LENGTH,
            "cc", cc - 1, RS_REPCAP_BUF_SIZE, repCap));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_PUCCH_DMRS_OCC_LENGTH,
            "NR5GFrame", frame - 1, RS_REPCAP_BUF_SIZE, repCap_frame));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_PUCCH_DMRS_OCC_LENGTH,
            "BWPart", bandwidthPart , RS_REPCAP_BUF_SIZE, repCap_bwp));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_PUCCH_DMRS_OCC_LENGTH,
            "NR5GSlot", slot , RS_REPCAP_BUF_SIZE, repCap_slot));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_PUCCH_DMRS_OCC_LENGTH,
            "NR5GPUCCH", pucch , RS_REPCAP_BUF_SIZE, repCap_pucch));

    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_frame, NULL);
    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_bwp, NULL);
    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_slot, NULL);
    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_pucch, NULL);

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_5G_NR_UL_PUCCH_DMRS_OCC_LENGTH, occLength),
            7, "OCC Length");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Turns an additional PUCCH DMRS on and off.
/// HIFN
/// HIFN     Note(s):
/// HIFN
/// HIFN     (1) Available for PUCCH formats 1.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR cc/Component Carrier parameter.
/// HIPAR frame/Sets the Frame parameter.
/// HIPAR bandwidthPart/Sets the Bandwidth Part parameter.
/// HIPAR slot/Defines the number of slot.
/// HIPAR pucch/Defines the number of PUCCH allocations in a slot.
/// HIPAR enabled/Turns an additional PUCCH DMRS on and off.
ViStatus _VI_FUNC rsspecan_Configure5GNRULPUCCHAdditionalEnabled(
                            ViSession instrSession,
                            ViInt32   cc,
                            ViInt32   frame,
                            ViInt32   bandwidthPart,
                            ViInt32   slot,
                            ViInt32   pucch,
                            ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_frame[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_bwp[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_slot[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_pucch[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_PUCCH_DMRS_ADDITIONAL_ENABLED,
            "cc", cc - 1, RS_REPCAP_BUF_SIZE, repCap));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_PUCCH_DMRS_ADDITIONAL_ENABLED,
            "NR5GFrame", frame - 1, RS_REPCAP_BUF_SIZE, repCap_frame));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_PUCCH_DMRS_ADDITIONAL_ENABLED,
            "BWPart", bandwidthPart , RS_REPCAP_BUF_SIZE, repCap_bwp));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_PUCCH_DMRS_ADDITIONAL_ENABLED,
            "NR5GSlot", slot , RS_REPCAP_BUF_SIZE, repCap_slot));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_PUCCH_DMRS_ADDITIONAL_ENABLED,
            "NR5GPUCCH", pucch , RS_REPCAP_BUF_SIZE, repCap_pucch));

    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_frame, NULL);
    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_bwp, NULL);
    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_slot, NULL);
    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_pucch, NULL);

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, repCap, RSSPECAN_ATTR_5G_NR_UL_PUCCH_DMRS_ADDITIONAL_ENABLED, enabled),
            7, "Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Defines a method by which the PUCCH DMRS sequence can be calculated.
/// HIFN
/// HIFN     Note(s):
/// HIFN
/// HIFN     (1) Available for PUCCH formats 2.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR cc/Component Carrier parameter.
/// HIPAR frame/Sets the Frame parameter.
/// HIPAR bandwidthPart/Sets the Bandwidth Part parameter.
/// HIPAR slot/Defines the number of slot.
/// HIPAR pucch/Defines the number of PUCCH allocations in a slot.
/// HIPAR sequenceGeneration/Selects the PUCCH DM-RS sequence generation method.
/// HIPAR seedValue/Defines the seed value for the PUCCH DM-RS sequence generation.
ViStatus _VI_FUNC rsspecan_Configure5GNRULPUCCHDMRSSequenceGeneration(
                            ViSession instrSession,
                            ViInt32   cc,
                            ViInt32   frame,
                            ViInt32   bandwidthPart,
                            ViInt32   slot,
                            ViInt32   pucch,
                            ViInt32   sequenceGeneration,
                            ViInt32   seedValue)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_frame[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_bwp[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_slot[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_pucch[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_PUCCH_DMRS_SEQUENCE_GENERATION_METHOD,
            "cc", cc - 1, RS_REPCAP_BUF_SIZE, repCap));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_PUCCH_DMRS_SEQUENCE_GENERATION_METHOD,
            "NR5GFrame", frame - 1, RS_REPCAP_BUF_SIZE, repCap_frame));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_PUCCH_DMRS_SEQUENCE_GENERATION_METHOD,
            "BWPart", bandwidthPart , RS_REPCAP_BUF_SIZE, repCap_bwp));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_PUCCH_DMRS_SEQUENCE_GENERATION_METHOD,
            "NR5GSlot", slot , RS_REPCAP_BUF_SIZE, repCap_slot));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_PUCCH_DMRS_SEQUENCE_GENERATION_METHOD,
            "NR5GPUCCH", pucch , RS_REPCAP_BUF_SIZE, repCap_pucch));

    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_frame, NULL);
    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_bwp, NULL);
    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_slot, NULL);
    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_pucch, NULL);

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_5G_NR_UL_PUCCH_DMRS_SEQUENCE_GENERATION_METHOD, sequenceGeneration),
            7, "Sequence Generation");

    if (sequenceGeneration == RSSPECAN_VAL_5G_NR_SEQ_GEN_METHOD_DSID)
    {
        viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_5G_NR_UL_PUCCH_DMRS_SCRAMBLING_ID, seedValue),
                8, "Seed Value");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects the paste logic when you copy a PUCCH configuration to other
/// HIFN  slots.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR cc/Component Carrier parameter.
/// HIPAR frame/Sets the Frame parameter.
/// HIPAR bandwidthPart/Sets the Bandwidth Part parameter.
/// HIPAR slot/Defines the number of slot.
/// HIPAR pucch/Defines the number of PUCCH allocations in a slot.
/// HIPAR mode/Selects the paste logic when you copy a PUCCH configuration to other
/// HIPAR mode/slots.
/// HIPAR period/Defines to which slots a PUCCH configuration is copied to.
/// HIPAR duration/Defines to which slots a PUCCH configuration is copied to.
/// HIPAR slots/Copies a PUCCH allocation configuration to a specific set of slots.
ViStatus _VI_FUNC rsspecan_Configure5GNRULPUCCHCopySettings(ViSession instrSession,
                                                            ViInt32   cc,
                                                            ViInt32   frame,
                                                            ViInt32   bandwidthPart,
                                                            ViInt32   slot,
                                                            ViInt32   pucch,
                                                            ViInt32   mode,
                                                            ViInt32   period,
                                                            ViInt32   duration,
                                                            ViString  slots)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_frame[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_bwp[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_slot[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_pucch[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_PUCCH_COPY_MODE,
            "cc", cc - 1, RS_REPCAP_BUF_SIZE, repCap));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_PUCCH_COPY_MODE,
            "NR5GFrame", frame - 1, RS_REPCAP_BUF_SIZE, repCap_frame));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_PUCCH_COPY_MODE,
            "BWPart", bandwidthPart , RS_REPCAP_BUF_SIZE, repCap_bwp));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_PUCCH_COPY_MODE,
            "NR5GSlot", slot , RS_REPCAP_BUF_SIZE, repCap_slot));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_PUCCH_COPY_MODE,
            "NR5GPUCCH", pucch , RS_REPCAP_BUF_SIZE, repCap_pucch));

    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_frame, NULL);
    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_bwp, NULL);
    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_slot, NULL);
    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_pucch, NULL);

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_5G_NR_UL_PUCCH_COPY_MODE, mode),
            7, "Mode");

    switch (mode)
    {
        case RSSPECAN_VAL_PUCCH_COPY_MODE_CUSTOM:
            viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_5G_NR_UL_PUCCH_COPY_PERIOD, period),
                    8, "Period");

            viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_5G_NR_UL_PUCCH_COPY_DURATION, duration),
                    9, "Duration");

            break;
        case RSSPECAN_VAL_PUCCH_COPY_MODE_SLOT:
            viCheckParm(rsspecan_SetAttributeViString(instrSession, repCap, RSSPECAN_ATTR_5G_NR_UL_PUCCH_COPY_SLOTS, slots),
                    10, "Slots");
            break;
        default:
            checkErr(RsCore_InvalidViInt32Value(instrSession, mode));
            break;
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Defines to which slots a PUCCH configuration is copied to.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR cc/Selects repcap cc.
/// HIPAR nr5gframe/Selects repcap NR5GFrame.
/// HIPAR bwpart/Selects repcap BWPart.
/// HIPAR nr5gslot/Selects repcap NR5GSlot.
/// HIPAR nr5gpucch/Selects repcap NR5GPUCCH.
/// HIPAR 5gNRULPUCCHCopyDuration/Defines to which slots a PUCCH configuration is copied to.
ViStatus _VI_FUNC rsspecan_Configure5GNRULPUCCHCopyDuration(ViSession instrSession,
                                                            ViInt32   cc,
                                                            ViInt32   nr5gframe,
                                                            ViInt32   bwpart,
                                                            ViInt32   nr5gslot,
                                                            ViInt32   nr5gpucch,
                                                            ViInt32   NR5gULPUCCHCopyDuration)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_frame[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_bwp[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_slot[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_pucch[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_PUCCH_COPY_DURATION,
            "cc", cc - 1, RS_REPCAP_BUF_SIZE, repCap));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_PUCCH_COPY_DURATION,
            "NR5GFrame", nr5gframe - 1, RS_REPCAP_BUF_SIZE, repCap_frame));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_PUCCH_COPY_DURATION,
            "BWPart", bwpart , RS_REPCAP_BUF_SIZE, repCap_bwp));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_PUCCH_COPY_DURATION,
            "NR5GSlot", nr5gslot , RS_REPCAP_BUF_SIZE, repCap_slot));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_PUCCH_COPY_DURATION,
            "NR5GPUCCH", nr5gpucch , RS_REPCAP_BUF_SIZE, repCap_pucch));

    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_frame, NULL);
    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_bwp, NULL);
    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_slot, NULL);
    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_pucch, NULL);

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_5G_NR_UL_PUCCH_COPY_DURATION, NR5gULPUCCHCopyDuration),
            7, "5G NR UL PUCCH Copy Duration");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Defines to which slots a PUCCH configuration is copied to.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSSPECAN_ATTR_5G_NR_UL_PUCCH_COPY_PERIOD
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN
/// HIFN  CONFigure[:NR5G]:UL[:CC<cc>]:FRAMe<fr>:BWPart<bwp>:SLOT<sl>:PUCCh<cr>:COPY:PERiod
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR cc/Selects repcap cc.
/// HIPAR nr5gframe/Selects repcap NR5GFrame.
/// HIPAR bwpart/Selects repcap BWPart.
/// HIPAR nr5gslot/Selects repcap NR5GSlot.
/// HIPAR nr5gpucch/Selects repcap NR5GPUCCH.
/// HIPAR 5gNRULPUCCHCopyPeriod/Defines to which slots a PUCCH configuration is copied to.
ViStatus _VI_FUNC rsspecan_Configure5GNRULPUCCHCopyPeriod(ViSession instrSession,
                                                          ViInt32   cc,
                                                          ViInt32   nr5gframe,
                                                          ViInt32   bwpart,
                                                          ViInt32   nr5gslot,
                                                          ViInt32   nr5gpucch,
                                                          ViInt32   NR5gULPUCCHCopyPeriod)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_frame[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_bwp[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_slot[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_pucch[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_PUCCH_COPY_PERIOD,
            "cc", cc - 1, RS_REPCAP_BUF_SIZE, repCap));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_PUCCH_COPY_PERIOD,
            "NR5GFrame", nr5gframe - 1, RS_REPCAP_BUF_SIZE, repCap_frame));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_PUCCH_COPY_PERIOD,
            "BWPart", bwpart , RS_REPCAP_BUF_SIZE, repCap_bwp));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_PUCCH_COPY_PERIOD,
            "NR5GSlot", nr5gslot , RS_REPCAP_BUF_SIZE, repCap_slot));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_PUCCH_COPY_PERIOD,
            "NR5GPUCCH", nr5gpucch , RS_REPCAP_BUF_SIZE, repCap_pucch));

    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_frame, NULL);
    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_bwp, NULL);
    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_slot, NULL);
    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_pucch, NULL);

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_5G_NR_UL_PUCCH_COPY_PERIOD, NR5gULPUCCHCopyPeriod),
            7, "5G NR UL PUCCH Copy Period");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Copies a PUCCH allocation configuration to a specific set of slots.
/// HIFN     List of slots either as comma-separated list (2,5,8), a range (3-5) or
/// HIFN  a combination of both (2-5,8,10).
/// HIFN
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSSPECAN_ATTR_5G_NR_UL_PUCCH_COPY_SLOTS
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN
/// HIFN  CONFigure[:NR5G]:UL[:CC<cc>]:FRAMe<fr>:BWPart<bwp>:SLOT<sl>:PUCCh<cr>:COPY:SLOT
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR cc/Selects repcap cc.
/// HIPAR nr5gframe/Selects repcap NR5GFrame.
/// HIPAR bwpart/Selects repcap BWPart.
/// HIPAR nr5gslot/Selects repcap NR5GSlot.
/// HIPAR nr5gpucch/Selects repcap NR5GPUCCH.
/// HIPAR 5gNRULPUCCHCopySlots/Copies a PUCCH allocation configuration to a specific set of slots.
ViStatus _VI_FUNC rsspecan_Configure5GNRULPUCCHCopySlots(ViSession instrSession,
                                                         ViInt32   cc,
                                                         ViInt32   nr5gframe,
                                                         ViInt32   bwpart,
                                                         ViInt32   nr5gslot,
                                                         ViInt32   nr5gpucch,
                                                         ViString  NR5gULPUCCHCopySlots)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_frame[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_bwp[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_slot[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_pucch[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_PUCCH_COPY_SLOTS,
            "cc", cc - 1, RS_REPCAP_BUF_SIZE, repCap));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_PUCCH_COPY_SLOTS,
            "NR5GFrame", nr5gframe - 1, RS_REPCAP_BUF_SIZE, repCap_frame));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_PUCCH_COPY_SLOTS,
            "BWPart", bwpart , RS_REPCAP_BUF_SIZE, repCap_bwp));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_PUCCH_COPY_SLOTS,
            "NR5GSlot", nr5gslot , RS_REPCAP_BUF_SIZE, repCap_slot));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_PUCCH_COPY_SLOTS,
            "NR5GPUCCH", nr5gpucch , RS_REPCAP_BUF_SIZE, repCap_pucch));

    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_frame, NULL);
    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_bwp, NULL);
    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_slot, NULL);
    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_pucch, NULL);

    viCheckParm(rsspecan_SetAttributeViString(instrSession, repCap, RSSPECAN_ATTR_5G_NR_UL_PUCCH_COPY_SLOTS, NR5gULPUCCHCopySlots),
            7, "5G NR UL PUCCH");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Copies a PUCCH configuration to other slots.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR cc/Component Carrier parameter.
/// HIPAR frame/Sets the Frame parameter.
/// HIPAR bandwidthPart/Sets the Bandwidth Part parameter.
/// HIPAR slot/Defines the number of slot.
/// HIPAR pucch/Defines the number of PUCCH allocations in a slot.
ViStatus _VI_FUNC rsspecan_NR5GULPUCCHCopy(ViSession instrSession,
                                           ViInt32   cc,
                                           ViInt32   frame,
                                           ViInt32   bandwidthPart,
                                           ViInt32   slot,
                                           ViInt32   pucch)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_frame[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_bwp[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_slot[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_pucch[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_PUCCH_COPY,
            "cc", cc - 1, RS_REPCAP_BUF_SIZE, repCap));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_PUCCH_COPY,
            "NR5GFrame", frame - 1, RS_REPCAP_BUF_SIZE, repCap_frame));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_PUCCH_COPY,
            "BWPart", bandwidthPart , RS_REPCAP_BUF_SIZE, repCap_bwp));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_PUCCH_COPY,
            "NR5GSlot", slot , RS_REPCAP_BUF_SIZE, repCap_slot));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_PUCCH_COPY,
            "NR5GPUCCH", pucch , RS_REPCAP_BUF_SIZE, repCap_pucch));

    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_frame, NULL);
    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_bwp, NULL);
    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_slot, NULL);
    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_pucch, NULL);

    checkErr(rsspecan_SetAttributeViString(instrSession, repCap, RSSPECAN_ATTR_5G_NR_UL_PUCCH_COPY, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the number of configurable frames.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR cc/Component Carrier parameter.
/// HIPAR numberOfConfigurableFrames/Defines the number of configurable frames.
ViStatus _VI_FUNC rsspecan_Configure5GNRULNumberOfConfigurableFrames(
                            ViSession instrSession,
                            ViInt32   cc,
                            ViInt32   numberOfConfigurableFrames)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_NUMBER_OF_CONFIGURABLE_FRAMES,
            "cc", cc -1, RS_REPCAP_BUF_SIZE, repCap));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_5G_NR_UL_NUMBER_OF_CONFIGURABLE_FRAMES, numberOfConfigurableFrames),
            3, "Number Of Configurable Frames");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function configures the Component Carrier advanced settings.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR cc/Component Carrier parameter.
/// HIPAR enabled/Turns RF upconversion and corresponding phase compensation on and off.
/// HIPAR mode/Selects the frequency selection mode for RF upconversion.
/// HIPAR frequency/Selects a frequency for RF upconversion.
ViStatus _VI_FUNC rsspecan_Configure5GNRULAdvancedSettingsRFUpconversion(
                            ViSession instrSession,
                            ViInt32   cc,
                            ViBoolean enabled,
                            ViInt32   mode,
                            ViReal64  frequency)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_ADVANCED_SETTINGS_RF_UPCONVERSION_ENABLED,
            "cc", cc -1, RS_REPCAP_BUF_SIZE, repCap));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, repCap, RSSPECAN_ATTR_5G_NR_UL_ADVANCED_SETTINGS_RF_UPCONVERSION_ENABLED, enabled),
            3, "Enabled");
    if (enabled == VI_TRUE)
    {
        viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_5G_NR_UL_ADVANCED_SETTINGS_RF_UPCONVERSION_MODE, mode),
                4, "Mode");
        if (mode == RSSPECAN_VAL_5G_NR_RF_UPCONVERSION_MODE_MANUAL)
        {
            viCheckParm(rsspecan_SetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_5G_NR_UL_ADVANCED_SETTINGS_RF_UPCONVERSION_FREQUENCY, frequency),
                    5, "Frequency");
        }
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function configures an offset relative to reference point A.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR cc/Component Carrier parameter.
/// HIPAR centerFrequencyOffset/Defines the frequency of the reference point A relative to the
/// HIPAR centerFrequencyOffset/carrier's center frequency.
/// HIPAR k0Select/Selects the K0 offset.
/// HIPAR offset/Defines an offset relative to reference point A for bandwidth parts with
/// HIPAR offset/selected subcarrier spacing.
ViStatus _VI_FUNC rsspecan_Configure5GNRULAdvancedSettingsRefPointA(
                            ViSession instrSession,
                            ViInt32   cc,
                            ViReal64  centerFrequencyOffset,
                            ViInt32   k0Select,
                            ViInt32   offset)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViAttr   attr[] = {RSSPECAN_ATTR_5G_NR_UL_ADVANCED_SETTINGS_REF_POINT_A_15_KHZ_OFFSET,
             RSSPECAN_ATTR_5G_NR_UL_ADVANCED_SETTINGS_REF_POINT_A_30_KHZ_OFFSET,
             RSSPECAN_ATTR_5G_NR_UL_ADVANCED_SETTINGS_REF_POINT_A_60_KHZ_OFFSET,
             RSSPECAN_ATTR_5G_NR_UL_ADVANCED_SETTINGS_REF_POINT_A_120_KHZ_OFFSET,
             RSSPECAN_ATTR_5G_NR_UL_ADVANCED_SETTINGS_REF_POINT_A_480_KHZ_OFFSET,
             RSSPECAN_ATTR_5G_NR_UL_ADVANCED_SETTINGS_REF_POINT_A_960_KHZ_OFFSET,
             RSSPECAN_ATTR_5G_NR_UL_ADVANCED_SETTINGS_REF_POINT_A_TX_BW_OFFSET};

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, k0Select, RSSPECAN_VAL_5G_NR_K0_15_KHZ, RSSPECAN_VAL_5G_NR_K0_TX_BW),
            4, "K0 Select");

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_ADVANCED_SETTINGS_REF_POINT_A_CENTER_FREQUENCY_OFFSET,
            "cc", cc -1, RS_REPCAP_BUF_SIZE, repCap));

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_5G_NR_UL_ADVANCED_SETTINGS_REF_POINT_A_CENTER_FREQUENCY_OFFSET, centerFrequencyOffset),
            3, "Center Frequency Offset");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, attr[k0Select], offset),
            5, "Offset");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the absolute frequency of the reference point A.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR cc/Component Carrier parameter.
/// HIPAR frequency/Queries the absolute frequency of the reference point A.
ViStatus _VI_FUNC rsspecan_Query5GNRULAdvancedSettingsRefPointAFrequency(
                            ViSession instrSession,
                            ViInt32   cc,
                            ViReal64* frequency)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_ADVANCED_SETTINGS_REF_POINT_A_FREQUENCY,
            "cc", cc -1, RS_REPCAP_BUF_SIZE, repCap));

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_5G_NR_UL_ADVANCED_SETTINGS_REF_POINT_A_FREQUENCY, frequency),
            3, "Frequency");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the TxBW offset of the reference point A.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR cc/Component Carrier parameter.
/// HIPAR offset/Queries the TxBW offset of the reference point A.
ViStatus _VI_FUNC rsspecan_Query5GNRULAdvancedSettingsRefPointATxBwOffset(
                            ViSession instrSession,
                            ViInt32   cc,
                            ViReal64* offset)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_ADVANCED_SETTINGS_REF_POINT_A_TX_BW_OFFSET,
            "cc", cc -1, RS_REPCAP_BUF_SIZE, repCap));

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_5G_NR_UL_ADVANCED_SETTINGS_REF_POINT_A_TX_BW_OFFSET, offset),
            3, "Offset");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Turns analysis of the DC carrier on and off.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR cc/Component Carrier parameter.
/// HIPAR enabled/Turns analysis of the DC carrier on and off.
ViStatus _VI_FUNC rsspecan_Configure5GNRULAdvancedSettingsDCCarrierEnabled(
                            ViSession instrSession,
                            ViInt32   cc,
                            ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_ADVANCED_SETTINGS_DC_CARRIER_ENABLED,
            "cc", cc -1, RS_REPCAP_BUF_SIZE, repCap));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, repCap, RSSPECAN_ATTR_5G_NR_UL_ADVANCED_SETTINGS_DC_CARRIER_ENABLED, enabled),
            3, "Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function configures the Advanced Settings.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR cc/Component Carrier parameter.
/// HIPAR precoding/Turns transform precoding on and off.
/// HIPAR hopping/Selects a hopping mode for the PUSCH DMRS sequence.
ViStatus _VI_FUNC rsspecan_Configure5GNRULAdvancedSettings(ViSession instrSession,
                                                           ViInt32   cc,
                                                           ViBoolean precoding,
                                                           ViInt32   hopping)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_ADVANCED_SETTINGS_TRANSFORM_PRECODING,
            "cc", cc -1, RS_REPCAP_BUF_SIZE, repCap));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, repCap, RSSPECAN_ATTR_5G_NR_UL_ADVANCED_SETTINGS_TRANSFORM_PRECODING, precoding),
            3, "Precoding");
    if (precoding == VI_TRUE)
    {
        viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_5G_NR_UL_ADVANCED_SETTINGS_PUSCH_HOPPING, hopping),
                4, "Hopping");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects the PUSCH frequency hopping mode.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR cc/Component Carrier parameter.
/// HIPAR puschFrequencyHopping/Selects the PUSCH frequency hopping mode.
ViStatus _VI_FUNC rsspecan_Configure5GNRULAdvancedSettingsPUSCHFrequencyHopping(
                            ViSession instrSession,
                            ViInt32   cc,
                            ViInt32   puschFrequencyHopping)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_ADVANCED_SETTINGS_PUSCH_FREQUENCY_HOPPING,
            "cc", cc -1, RS_REPCAP_BUF_SIZE, repCap));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_5G_NR_UL_ADVANCED_SETTINGS_PUSCH_FREQUENCY_HOPPING, puschFrequencyHopping),
            3, "PUSCH Frequency Hopping");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the 5G NR frame number.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR frameNumber/Defines the 5G NR frame number.
ViStatus _VI_FUNC rsspecan_Configure5GNRULAdvancedSettingsFrameNumber(
                            ViSession instrSession,
                            ViInt32   frameNumber)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_5G_NR_UL_ADVANCED_SETTINGS_FRAME_NUMBER, frameNumber),
            2, "Frame Number");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function turns extreme conditions for spectrum flatness measurements on and off.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR cc/Component Carrier parameter.
/// HIPAR extremeConditions/Turns extreme conditions for spectrum flatness measurements on and off.
ViStatus _VI_FUNC rsspecan_Configure5GNRULAdvancedSettingsExtremeConditions(
                            ViSession instrSession,
                            ViInt32   cc,
                            ViBoolean extremeConditions)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_ADVANCED_SETTINGS_EXTREME_CONDITIONS,
            "cc", cc -1, RS_REPCAP_BUF_SIZE, repCap));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, repCap, RSSPECAN_ATTR_5G_NR_UL_ADVANCED_SETTINGS_EXTREME_CONDITIONS, extremeConditions),
            3, "Extreme Conditions");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects one of the antenna port configurations.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR cc/Component Carrier parameter.
/// HIPAR antennaConfiguration/Sets the antenna configuration parameter.
/// HIPAR enabled/Selects one of the antenna port configurations.
ViStatus _VI_FUNC rsspecan_Configure5GNRULAntennaPortEnabled(ViSession instrSession,
                                                             ViInt32   cc,
                                                             ViInt32   antennaConfiguration,
                                                             ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_antConfig[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_ANTENNA_PORT_ENABLED,
            "cc", cc - 1, RS_REPCAP_BUF_SIZE, repCap));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_ANTENNA_PORT_ENABLED,
            "AntennaConfig", antennaConfiguration - 1, RS_REPCAP_BUF_SIZE, repCap_antConfig));

    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_antConfig, NULL);

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, repCap, RSSPECAN_ATTR_5G_NR_UL_ANTENNA_PORT_ENABLED, enabled),
            4, "Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Selects the antenna port(s) on which the PDSCH is transmitted.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR cc/Component Carrier parameter.
/// HIPAR antennaConfiguration/Sets the antenna configuration parameter.
/// HIPAR antenna/Sets the antenna configuration parameter.
/// HIPAR portSelect/Selects the antenna port(s) on which the PDSCH is transmitted.
ViStatus _VI_FUNC rsspecan_Configure5GNRULAntennaPortSelect(ViSession instrSession,
                                                            ViInt32   cc,
                                                            ViInt32   antennaConfiguration,
                                                            ViInt32   antenna,
                                                            ViInt32   portSelect)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_antConfig[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_ant[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_ANTENNA_PORT_SELECT,
            "cc", cc - 1, RS_REPCAP_BUF_SIZE, repCap));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_ANTENNA_PORT_SELECT,
            "AntennaConfig", antennaConfiguration - 1, RS_REPCAP_BUF_SIZE, repCap_antConfig));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_ANTENNA_PORT_SELECT,
            "Ant", antenna - 1, RS_REPCAP_BUF_SIZE, repCap_ant));

    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_antConfig, NULL);
    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_ant, NULL);

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_5G_NR_UL_ANTENNA_PORT_SELECT, portSelect),
            5, "Port Select");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the IP address of the connected signal generator.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR ipAddress/Defines the IP address of the connected signal generator.
ViStatus _VI_FUNC rsspecan_Configure5GNRULGeneratorIPAddress(ViSession instrSession,
                                                             ViString  ipAddress)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViString(instrSession, NULL, RSSPECAN_ATTR_5G_NR_UL_GENERATOR_IP_ADDRESS, ipAddress),
            2, "IP Address");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the state of connection to the signal generator.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR connectionState/Queries the state of connection to the signal generator.
ViStatus _VI_FUNC rsspecan_Query5GNRULGeneratorConnectionState(
                            ViSession instrSession,
                            ViInt32*  connectionState)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_GetAttributeViInt32(instrSession, NULL, RSSPECAN_ATTR_5G_NR_UL_GENERATOR_LED_STATE, connectionState),
            2, "Connection State");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Turns the generator control on and off.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR state/Turns the generator control on and off.
ViStatus _VI_FUNC rsspecan_Configure5GNRULGeneratorControlState(
                            ViSession instrSession,
                            ViBoolean state)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, NULL, RSSPECAN_ATTR_5G_NR_UL_GENERATOR_CONTROL_ENABLED, state),
            2, "State");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Enables or disables RF output on the connected generator.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR rfOutput/Enables or disables RF output on the connected generator.
ViStatus _VI_FUNC rsspecan_Configure5GNRULGeneratorRFOutput(ViSession instrSession,
                                                            ViBoolean rfOutput)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, NULL, RSSPECAN_ATTR_5G_NR_UL_GENERATOR_RF_OUTPUT_ENABLED, rfOutput),
            2, "RF Output");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Turns synchronization of the analyzer and generator frequency on and
/// HIFN  off.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR frequencySynchronization/Turns synchronization of the analyzer and generator frequency on and
/// HIPAR frequencySynchronization/off.
ViStatus _VI_FUNC rsspecan_Configure5GNRULGeneratorFrequencySynchronization(
                            ViSession instrSession,
                            ViBoolean frequencySynchronization)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, NULL, RSSPECAN_ATTR_5G_NR_UL_GENERATOR_FREQUENCY_SYNCHRONIZATION, frequencySynchronization),
            2, "Frequency Synchronisation");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Configures the level synchronization between analyzer and generator.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR levelState/Turns level synchronization between analyzer and generator on and off.
/// HIPAR level_RMS/Defines the output power RMS level of the generator.
/// HIPAR dutLimitState/Turns a limitation of the DUT peak input power on and off.
/// HIPAR dutLimitPeakInputPower/Defines the output power RMS level of the generator.
/// HIPAR dutGain/Defines DUT level gain for generator level control.
ViStatus _VI_FUNC rsspecan_Configure5GNRULGeneratorLevel(ViSession instrSession,
                                                         ViBoolean levelState,
                                                         ViReal64  level_RMS,
                                                         ViBoolean dutLimitState,
                                                         ViReal64  dutLimitPeakInputPower,
                                                         ViReal64  dutGain)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, NULL, RSSPECAN_ATTR_5G_NR_UL_GENERATOR_LEVEL_CONTROL_ENABLED, levelState),
            2, "Level State");

    if (levelState == VI_TRUE)
    {
        viCheckParm(rsspecan_SetAttributeViReal64(instrSession, NULL, RSSPECAN_ATTR_5G_NR_UL_GENERATOR_POWER_LEVEL, level_RMS),
                3, "Level RMS");

        viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, NULL, RSSPECAN_ATTR_5G_NR_UL_GENERATOR_LEVEL_DUT_LIMIT_ENABLED, dutLimitState),
                4, "DUT Limit State");

        if (dutLimitState == VI_TRUE)
        {
            viCheckParm(rsspecan_SetAttributeViReal64(instrSession, NULL, RSSPECAN_ATTR_5G_NR_UL_GENERATOR_LEVEL_DUT_LIMIT, dutLimitPeakInputPower),
                    5, "DUT Limit Peak Input Power");

            viCheckParm(rsspecan_SetAttributeViReal64(instrSession, NULL, RSSPECAN_ATTR_5G_NR_UL_GENERATOR_LEVEL_DUT_GAIN, dutGain),
                    6, "DUT Gain");
        }
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Uploads the RF settings (frequency, level) from analyzer to the
/// HIFN  generator.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsspecan_NR5GULGeneratorUploadRFSettings(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsspecan_SetAttributeViString(instrSession, NULL, RSSPECAN_ATTR_5G_NR_UL_GENERATOR_UPLOAD_RF_SETTINGS, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Downloads the RF settings (frequency, level) from the generator to the
/// HIFN  analyzer.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsspecan_NR5GULGeneratorDownloadRFSettings(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsspecan_SetAttributeViString(instrSession, NULL, RSSPECAN_ATTR_5G_NR_UL_GENERATOR_DOWNLOAD_RF_SETTINGS, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Downloads the NR signal description from the generator to the
/// HIFN  analyzer.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsspecan_NR5GULGeneratorDownload5GNRSettings(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsspecan_SetAttributeViString(instrSession, NULL, RSSPECAN_ATTR_5G_NR_UL_GENERATOR_DOWNLOAD_NR5G_SETTINGS, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Configures the periodic synchronization of the signal description on
/// HIFN  the analyzer.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR syncEnabled/Turns periodic synchronization of the signal description on the
/// HIPAR syncEnabled/analyzer on and off.
/// HIPAR pollingInterval/Defines the polling interval for periodic synchronization between
/// HIPAR pollingInterval/analyzer and generator.
ViStatus _VI_FUNC rsspecan_Configure5GNRULGeneratorSynchronization(
                            ViSession instrSession,
                            ViBoolean syncEnabled,
                            ViReal64  pollingInterval)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, NULL, RSSPECAN_ATTR_5G_NR_UL_GENERATOR_SETTINGS_PERIODIC_SYNCHRONIZATION_ENABLED, syncEnabled),
            2, "Sync Enabled");

    if (syncEnabled == VI_TRUE)
    {
        viCheckParm(rsspecan_SetAttributeViReal64(instrSession, NULL, RSSPECAN_ATTR_5G_NR_UL_GENERATOR_SETTINGS_PERIODIC_SYNCHRONIZATION_INTERVAL, pollingInterval),
                3, "Polling Interval");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Selects the input path.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR connector/Sets the connector parameter.
/// HIPAR inputType/Selects the input path.
ViStatus _VI_FUNC rsspecan_Configure5GNRULInputType(ViSession instrSession,
                                                    ViInt32   connector,
                                                    ViInt32   inputType)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_INPUT_TYPE,
            "Inp", connector -1, RS_REPCAP_BUF_SIZE, repCap));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_5G_NR_UL_INPUT_TYPE, inputType),
            3, "Input Type");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  If enabled, the reference level for digital input is adjusted to the
/// HIFN  full scale level automatically if the full scale level changes.
/// HIFN     This command is only available if the optional Digital Baseband
/// HIFN  Interface is installed.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR connector/Sets the connector parameter.
/// HIPAR coupling/If enabled, the reference level for digital input is adjusted to the
/// HIPAR coupling/full scale level automatically if the full scale level changes.
ViStatus _VI_FUNC rsspecan_Configure5GNRULInputCoupling(ViSession instrSession,
                                                        ViInt32   connector,
                                                        ViBoolean coupling)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_INPUT_COUPLING,
            "Inp", connector -1, RS_REPCAP_BUF_SIZE, repCap));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, repCap, RSSPECAN_ATTR_5G_NR_UL_INPUT_COUPLING, coupling),
            3, "Coupling");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function configures the NR 5G Sample Rate.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR connector/Sets the connector parameter.
/// HIPAR enabled/If enabled, the sample rate of the digital I/Q input signal is set
/// HIPAR enabled/automatically by the connected device.
/// HIPAR sampleRate/Specifies or queries the sample rate of the input signal from the
/// HIPAR sampleRate/optional Digital Baseband Interface.
ViStatus _VI_FUNC rsspecan_Configure5GNRULInputSampleRate(ViSession instrSession,
                                                          ViInt32   connector,
                                                          ViBoolean enabled,
                                                          ViReal64  sampleRate)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_INPUT_SAMPLE_RATE_ENABLED,
            "Inp", connector -1, RS_REPCAP_BUF_SIZE, repCap));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, repCap, RSSPECAN_ATTR_5G_NR_UL_INPUT_SAMPLE_RATE_ENABLED, enabled),
            3, "Enabled");

    if (enabled == VI_TRUE)
    {
        viCheckParm(rsspecan_SetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_5G_NR_UL_INPUT_SAMPLE_RATE, sampleRate),
                4, "Sample Rate");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function configures the Input Full Scale.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR connector/Sets the connector parameter.
/// HIPAR autoLevel/If enabled, the digital input full scale level is automatically set to
/// HIPAR autoLevel/the value provided by the connected device (if available).
/// HIPAR unit/Defines the unit of the full scale level. The availability of units
/// HIPAR unit/depends on the measurement application you are using.
/// HIPAR level/Defines or queries the Full Scale Level, i.e. the level that
/// HIPAR level/corresponds to an I/Q sample with the magnitude 1.
ViStatus _VI_FUNC rsspecan_Configure5GNRULInputFullScaleLevel(
                            ViSession instrSession,
                            ViInt32   connector,
                            ViBoolean autoLevel,
                            ViInt32   unit,
                            ViReal64  level)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_INPUT_FULL_SCALE_LEVEL_AUTO,
            "Inp", connector -1, RS_REPCAP_BUF_SIZE, repCap));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, repCap, RSSPECAN_ATTR_5G_NR_UL_INPUT_FULL_SCALE_LEVEL_AUTO, autoLevel),
            3, "Auto Level");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_5G_NR_UL_INPUT_FULL_SCALE_LEVEL_UNIT, unit),
            4, "Unit");

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_5G_NR_UL_INPUT_FULL_SCALE_LEVEL, level),
            5, "Level");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function configures the Input IQ.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR connector/Sets the connector parameter.
/// HIPAR balancedEnabled/Defines whether the input is provided as a differential signal via all
/// HIPAR balancedEnabled/4 Analog Baseband connectors or as a plain I/Q signal via 2
/// HIPAR balancedEnabled/single-ended lines.
/// HIPAR type/Defines the format of the input signal.
ViStatus _VI_FUNC rsspecan_Configure5GNRULInputIQ(ViSession instrSession,
                                                  ViInt32   connector,
                                                  ViBoolean balancedEnabled,
                                                  ViInt32   type)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_INPUT_IQ_BALANCED_ENABLED,
            "Inp", connector -1, RS_REPCAP_BUF_SIZE, repCap));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, repCap, RSSPECAN_ATTR_5G_NR_UL_INPUT_IQ_BALANCED_ENABLED, balancedEnabled),
            3, "Balanced Enabled");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_5G_NR_UL_INPUT_IQ_TYPE, type),
            4, "Type");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Includes or excludes ACLR, EVM anr SEM measurements from combined
/// HIFN  measurements.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR aclr/Includes or excludes ACLR measurements from combined measurements.
/// HIPAR evm/Includes or excludes EVM measurements from combined measurements.
/// HIPAR sem/Includes or excludes SEM measurements from combined measurements.
ViStatus _VI_FUNC rsspecan_Configure5GNRULCombinedMeasurementACLR(
                            ViSession instrSession,
                            ViBoolean aclr,
                            ViBoolean evm,
                            ViBoolean sem)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, NULL, RSSPECAN_ATTR_5G_NR_UL_COMBINED_MEASUREMENT_ACLR, aclr),
            2, "ACLR");

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, NULL, RSSPECAN_ATTR_5G_NR_UL_COMBINED_MEASUREMENT_EVM, evm),
            3, "EVM");

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, NULL, RSSPECAN_ATTR_5G_NR_UL_COMBINED_MEASUREMENT_SEM, sem),
            4, "SEM");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Loads an ACLR settings file for combined measurements.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR settingsFile/The path to the settings file (.xml format).
ViStatus _VI_FUNC rsspecan_Load5GNRULCombinedMeasurementACLRSettings(
                            ViSession instrSession,
                            ViString  settingsFile)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViString(instrSession, NULL, RSSPECAN_ATTR_5G_NR_UL_COMBINED_MEASUREMENT_LOAD_ACLR_SETTINGS, settingsFile),
            2, "Settings File");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Removes the file based settings
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsspecan_Clear5GNRULCombinedMeasurementACLRSettings(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsspecan_SetAttributeViString(instrSession, NULL, RSSPECAN_ATTR_5G_NR_UL_COMBINED_MEASUREMENT_CLEAR_ACLR_SETTINGS, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Loads an SEM settings file (.xml) for combined measurements.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR settingsFile/The path to the settings file (.xml format).
ViStatus _VI_FUNC rsspecan_Load5GNRULCombinedMeasurementSEMSettings(
                            ViSession instrSession,
                            ViString  settingsFile)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViString(instrSession, NULL, RSSPECAN_ATTR_5G_NR_UL_COMBINED_MEASUREMENT_LOAD_SEM_SETTINGS, settingsFile),
            2, "Settings File");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Removes the file based settings.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsspecan_Clear5GNRULCombinedMeasurementSEMSettings(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsspecan_SetAttributeViString(instrSession, NULL, RSSPECAN_ATTR_5G_NR_UL_COMBINED_MEASUREMENT_CLEAR_SEM_SETTINGS, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Configures the combined measurement signal capture settings.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR numberOfEvents/Defines the number of events in a combined measurement sequence.
/// HIPAR optimization/Selects the data capture method for combined measurements.
/// HIPAR sequenceAutoLevel/Turns auto leveling for each event in combined measurement sequence on
/// HIPAR sequenceAutoLevel/and off.
ViStatus _VI_FUNC rsspecan_Configure5GNRULCombinedMeasurementSignalCapture(
                            ViSession instrSession,
                            ViInt32   numberOfEvents,
                            ViInt32   optimization,
                            ViBoolean sequenceAutoLevel)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, NULL, RSSPECAN_ATTR_5G_NR_UL_COMBINED_MEASUREMENT_NR_OF_EVENTS, numberOfEvents),
            2, "Number of Events");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, NULL, RSSPECAN_ATTR_5G_NR_UL_COMBINED_MEASUREMENT_OPTIMIZATION, optimization),
            3, "Optimization");

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, NULL, RSSPECAN_ATTR_5G_NR_UL_COMBINED_MEASUREMENT_SEQUENCE_AUTO_LEVEL, sequenceAutoLevel),
            4, "Sequence Auto Level");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Select the type of measurement sequence for combined measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR sequence/Select the type of measurement sequence for combined measurement.
ViStatus _VI_FUNC rsspecan_Configure5GNRULCombinedMeasurementTriggerSequence(
                            ViSession instrSession,
                            ViInt32   sequence)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, NULL, RSSPECAN_ATTR_5G_NR_UL_COMBINED_MEASUREMENT_TRIGGER_SEQUENCE, sequence),
            2, "Sequence");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Select the availability of a frame trigger in a combined measurement
/// HIFN  sequence.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR frameTrigger/Select the availability of a frame trigger in a combined measurement
/// HIPAR frameTrigger/sequence.
ViStatus _VI_FUNC rsspecan_Configure5GNRULCombinedMeasurementFrameTrigger(
                            ViSession instrSession,
                            ViInt32   frameTrigger)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, NULL, RSSPECAN_ATTR_5G_NR_UL_COMBINED_MEASUREMENT_FRAME_TRIGGER, frameTrigger),
            2, "Frame Trigger");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Defines the event delay for combined measurements.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR eventDelay/Defines the event delay for combined measurements.
ViStatus _VI_FUNC rsspecan_Configure5GNRULCombinedMeasurementTriggerEventDelay(
                            ViSession instrSession,
                            ViReal64  eventDelay)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, NULL, RSSPECAN_ATTR_5G_NR_UL_COMBINED_MEASUREMENT_TRIGGER_EVENT_DELAY, eventDelay),
            2, "Event Delay");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Defines the recalibration interval for combined measurements.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR recalibrationInterval/Defines the recalibration interval for combined measurements.
ViStatus _VI_FUNC rsspecan_Configure5GNRULCombinedMeasurementTriggerRecalibrationInterval(
                            ViSession instrSession,
                            ViInt32   recalibrationInterval)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, NULL, RSSPECAN_ATTR_5G_NR_UL_COMBINED_MEASUREMENT_TRIGGER_RECALIBRATION_INTERVAL, recalibrationInterval),
            2, "Recalibration Interval");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Sends a trigger signal from the analyzer to the DUT.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsspecan_Send5GNRULCombinedMeasurementTrigger3(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsspecan_SetAttributeViString(instrSession, NULL, RSSPECAN_ATTR_5G_NR_UL_COMBINED_MEASUREMENT_SEND_TRIGGER_3, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Turns auto gating on and off.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR autoGating/Turns auto gating on and off.
ViStatus _VI_FUNC rsspecan_Configure5GNRULCombinedMeasurementTriggerAutoGating(
                            ViSession instrSession,
                            ViBoolean autoGating)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, NULL, RSSPECAN_ATTR_5G_NR_DL_COMBINED_MEASUREMENT_TRIGGER_AUTO_GATING, autoGating),
            2, "Auto Gating");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the number of filtered events for combined measurements.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR filteredEvents/Queries the number of filtered events for combined measurements.
ViStatus _VI_FUNC rsspecan_Query5GNRULCombinedMeasurementFilteredEvents(
                            ViSession instrSession,
                            ViInt32*  filteredEvents)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_GetAttributeViInt32(instrSession, NULL, RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_COMBINED_MEASUREMENT_FILTERED_EVENTS, filteredEvents),
            2, "Filtered Events");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Restores the settings of the last measurement sequence in combined
/// HIFN  mode.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsspecan_Configure5GNRULCombinedMeasurementRestoreSettings(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsspecan_SetAttributeViString(instrSession, NULL, RSSPECAN_ATTR_5G_NR_UL_COMBINED_MEASUREMENT_RESTORE_SETTINGS, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function adds a window to the display.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR stringSize/This control sets the allocated length of Returned Window Name string.
/// HIPAR referenceWindowName/String containing the number or name of the window the new window is
/// HIPAR referenceWindowName/inserted next to. In the default state, the name of the window is its
/// HIPAR referenceWindowName/index.
/// HIPAR direction/Direction the window is added relative to the window set with
/// HIPAR direction/"Reference Window Name".
/// HIPAR returnedWindowName/When adding a new window, the function returns its name (by
/// HIPAR windowType/Type of result display (evaluation method) you want to add.
ViStatus _VI_FUNC rsspecan_Add5GNRULWindow(ViSession instrSession,
                                           ViInt32   stringSize,
                                           ViString  referenceWindowName,
                                           ViInt32   direction,
                                           ViChar    returnedWindowName[],
                                           ViInt32   windowType)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViChar   response[RS_MAX_MESSAGE_BUF_SIZE];
    ViChar   *p2direction;
    ViChar   *p2type;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &rsspecan_rngLayoutDirection, direction, &p2direction),
            4, "Direction");
    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &RsSpecAn_rng5GNRULLayoutType, windowType, &p2type),
            5, "Window Type");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "LAY:ADD? '%s',%s,%s", referenceWindowName, p2direction, p2type);
    checkErr(RsCore_QueryViString(instrSession, cmd, response));

    if (returnedWindowName != NULL)
    {
        RsCore_TrimString(response, RS_VAL_TRIM_WHITESPACES_AND_ALL_QUOTES);
        RsCore_CopyToUserBufferAsciiData(returnedWindowName, stringSize, response);
    }

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function replaces a window with another one.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR windowName/String containing the number or name of the window you want to
/// HIPAR windowType/Type of result display (evaluation method) you want to add.
ViStatus _VI_FUNC rsspecan_Replace5GNRULWindow(ViSession instrSession,
                                               ViString  windowName,
                                               ViInt32   windowType)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViChar   *p2type;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &RsSpecAn_rng5GNRULLayoutType, windowType, &p2type),
            3, "Window Type");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "LAY:REPL '%s',%s", windowName, p2type);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Selects the way the contents of the result summary are calculated.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR summary/Selects the way the contents of the result summary are calculated.
ViStatus _VI_FUNC rsspecan_Configure5GNRULAnalysisShowResultSummary(
                            ViSession instrSession,
                            ViInt32   summary)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_SHOW_RESULT_SUMMARY, summary),
            2, "Summary");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Turns the display of individual results in the numerical result
/// HIFN  summary on and off.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR item/Selects the item to be included in Result Summary.
/// HIPAR displayItem/This parameter selects whether the results summary item is displayed.
ViStatus _VI_FUNC rsspecan_Configure5GNRULAnalysisShowResultSummaryItems(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   item,
                            ViBoolean displayItem)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";
    ViChar   *p2string;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentModel(instrSession, "FSW"));

    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &RsSpecAn_rng5GNRResultSummaryItems, item, &p2string),
            3, "Item");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "DISP:WIND%ld:TABL:ITEM %s,%s", window, p2string, offOnArr[displayItem]);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects the scale of the x-axis for result displays that show symbol
/// HIFN  results.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR xScale/Selects the scale of the x-axis for result displays that show symbol
/// HIPAR xScale/results.
ViStatus _VI_FUNC rsspecan_Configure5GNRULAnalysisXScale(ViSession instrSession,
                                                         ViInt32   xScale)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_X_SCALE, xScale),
            2, "X Scale");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function configures the X Axis.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR xAxis/Selects the scale of the x-axis for result displays that show
/// HIPAR xAxis/subcarrier results.
/// HIPAR referenceXAxis/Selects the reference for result displays whose x-axis shows frequency
/// HIPAR referenceXAxis/characteristics of the signal.
ViStatus _VI_FUNC rsspecan_Configure5GNRULAnalysisXAxis(ViSession instrSession,
                                                        ViInt32   xAxis,
                                                        ViInt32   referenceXAxis)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_SUBCARRIER_X_AXIS, xAxis),
            2, "X Axis");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_REFERENCE_X_AXIS, referenceXAxis),
            3, "Reference X Axis");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Turns the EVM max hold function on and off.
/// HIFN  summary on and off.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR EVMMaxHold/This parameter turns the EVM max hold function on and off.
ViStatus _VI_FUNC rsspecan_Configure5GNRULAnalysisEVMMaxHold(ViSession instrSession,
                                                             ViBoolean EVMMaxHold)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_EVM_MAX_HOLD, EVMMaxHold),
            2, "EVM Max Hold");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Turns the 3D view for selected diagrams on and off.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR 3dView/Turns the 3D view for selected diagrams on and off.
ViStatus _VI_FUNC rsspecan_Configure5GNRULAnalysis3DView(ViSession instrSession,
                                                         ViBoolean view)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, NULL, RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_3D_VIEW, view),
            2, "View");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Selects the limit evaluation mode for ACLR measurements in combined
/// HIFN  measurement mode.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR aclrLimitPassMode/Selects the limit evaluation mode for ACLR measurements in combined
/// HIPAR aclrLimitPassMode/measurement mode.
ViStatus _VI_FUNC rsspecan_Configure5GNRULAnalysisACLRLimitPassMode(
                            ViSession instrSession,
                            ViInt32   aclrLimitPassMode)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, NULL, RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_ACLR_LIMIT_PASS_MODE, aclrLimitPassMode),
            2, "ACLR Limit Pass Mode");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects the way multiple carriers are analyzed.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR multipleCarriers/Selects the way multiple carriers are analyzed.
ViStatus _VI_FUNC rsspecan_Configure5GNRULAnalysisMultipleCarriers(
                            ViSession instrSession,
                            ViInt32   multipleCarriers)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_MULTIPLE_CARRIERS, multipleCarriers),
            2, "Multiple Carriers");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Adjusts the amplitude settings, including attenuator and preamplifier,
/// HIFN  to achieve the optimal EVM using the maximum dynamic range.
/// HIFN     Note that this process can up to several minutes, depending on the
/// HIFN  number of component carriers you are measuring.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsspecan_Configure5GNRULAutomaticConfigurationEVM(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_5G_NR_UL_AUTOMATIC_CONFIGURATION_EVM, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects the number of slots to be used during the auto EVM routine.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR numberOfSlots/Selects the number of slots to be used during the auto EVM routine.
ViStatus _VI_FUNC rsspecan_Configure5GNRULAutomaticEVMSlots(ViSession instrSession,
                                                            ViInt32   numberOfSlots)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, NULL, RSSPECAN_ATTR_5G_NR_UL_AUTO_EVM_SLOTS, numberOfSlots),
            2, "Number of Slots");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Selects the analyzed segment for a segmented capture.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR value/Selects the analyzed segment for a segmented capture.
ViStatus _VI_FUNC rsspecan_Configure5GNRULAnalysisEvaluationRangeSegment(
                            ViSession instrSession,
                            ViInt32   value)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_EVALUATION_RANGE_SEGMENT, value),
            2, "Value");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the current synchronization state.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR cc/Component Carrier parameter.
/// HIPAR arraySize/Sets the array size parameter.
/// HIPAR synchronizationState/Queries the current synchronization state.
ViStatus _VI_FUNC rsspecan_Query5GNRULAnalysisSynchronizationState(
                            ViSession instrSession,
                            ViInt32   cc,
                            ViInt32   arraySize,
                            ViChar    synchronizationState[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_SYNCHRONIZATION_STATE,
            "cc", cc - 1, RS_REPCAP_BUF_SIZE, repCap));

    viCheckParm(rsspecan_GetAttributeViString(instrSession, repCap, RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_SYNCHRONIZATION_STATE,
            arraySize, synchronizationState),
            4, "Synchronization State");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the cell ID.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR cc/Component Carrier parameter.
/// HIPAR cellID/Queries the cell ID.
ViStatus _VI_FUNC rsspecan_Query5GNRULAnalysisCellID(ViSession instrSession,
                                                     ViInt32   cc,
                                                     ViInt32*  cellID)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_CELL_ID,
            "cc", cc - 1, RS_REPCAP_BUF_SIZE, repCap));

    viCheckParm(rsspecan_GetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_CELL_ID, cellID),
            2, "Cell ID");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the average crest factor as shown in the result summary.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR cc/Component Carrier parameter.
/// HIPAR crestFactor/Queries the average crest factor as shown in the result summary.
ViStatus _VI_FUNC rsspecan_Query5GNRULAnalysisCrestFactor(ViSession instrSession,
                                                          ViInt32   cc,
                                                          ViReal64* crestFactor)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_CREST_FACTOR,
            "cc", cc - 1, RS_REPCAP_BUF_SIZE, repCap));

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_CREST_FACTOR, crestFactor),
            3, "Crest Factor");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the (sub)frame start offset as shown in the capture buffer.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR cc/Component Carrier parameter.
/// HIPAR startOffset/Queries the (sub)frame start offset as shown in the capture buffer.
ViStatus _VI_FUNC rsspecan_Query5GNRULFrameStartOffset(ViSession instrSession,
                                                       ViInt32   cc,
                                                       ViReal64* startOffset)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_FRAME_START_OFFSET,
            "cc", cc - 1, RS_REPCAP_BUF_SIZE, repCap));

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_5G_NR_UL_FRAME_START_OFFSET, startOffset),
            3, "Start Offset");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects the power averaging mode.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR mode/Selects the power averaging mode.
ViStatus _VI_FUNC rsspecan_Configure5GNRULAnalysisPowerAveragingMode(
                            ViSession instrSession,
                            ViInt32   mode)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, NULL, RSSPECAN_ATTR_5G_NR_UL_POWER_AVERAGING_MODE, mode),
            2, "Mode");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Turns the throuput measurement on and off.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR enabled/Turns the throuput measurement on and off.
ViStatus _VI_FUNC rsspecan_Configure5GNRUThroughputMeasurementEnabled(
                            ViSession instrSession,
                            ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, NULL, RSSPECAN_ATTR_5G_NR_UL_THROUGHPUT_MEASUREMENT_ENABLED, enabled),
            2, "Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Selects the test ID for ACLR measurements.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR testID/Selects the test ID for ACLR measurements.
ViStatus _VI_FUNC rsspecan_Configure5GNRULAnalysisFrequencySweepTestID(
                            ViSession instrSession,
                            ViInt32   testID)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, NULL, RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_FREQUENCY_SWEEP_TEST_ID, testID),
            2, "Test ID");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function filters the displayed results in the constellation
/// HIFN  diagram by a certain allocation.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR mode/Filters the displayed results in the constellation diagram by a
/// HIPAR mode/certain allocation.
/// HIPAR value/Filters the displayed results in the constellation diagram by a
/// HIPAR value/certain allocation.
ViStatus _VI_FUNC rsspecan_Configure5GNRULAnalysisEvaluationRangeAllocation(
                            ViSession instrSession,
                            ViInt32   mode,
                            ViInt32   value)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, mode, RSSPECAN_VAL_5G_NR_ANALYSIS_MODE_ALL, RSSPECAN_VAL_5G_NR_ANALYSIS_MODE_VAL),
            2, "Mode");

    if (mode == RSSPECAN_VAL_5G_NR_ANALYSIS_MODE_ALL)
    {
        checkErr(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_EVALUATION_RANGE_ALLOCATION_ALL, NULL));
    }
    else
    {
        viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_EVALUATION_RANGE_ALLOCATION, value),
                3, "Value");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function filters the displayed results in the constellation
/// HIFN  diagram by a certain allocation.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR mode/Filters the displayed results by a certain bandwidth part.
/// HIPAR value/Filters the displayed results by a certain bandwidth part.
ViStatus _VI_FUNC rsspecan_Configure5GNRULAnalysisEvaluationRangeBandwidthPart(
                            ViSession instrSession,
                            ViInt32   mode,
                            ViInt32   value)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, mode, RSSPECAN_VAL_5G_NR_ANALYSIS_MODE_ALL, RSSPECAN_VAL_5G_NR_ANALYSIS_MODE_VAL),
            2, "Mode");

    if (mode == RSSPECAN_VAL_5G_NR_ANALYSIS_MODE_ALL)
    {
        checkErr(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_EVALUATION_RANGE_BANDWIDTH_PART_ALL, NULL));
    }
    else
    {
        viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_EVALUATION_RANGE_BANDWIDTH_PART, value),
                3, "Value");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function filters the displayed results in the constellation
/// HIFN  diagram by a certain subcarrier.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR mode/Filters the displayed results in the constellation diagram by a
/// HIPAR mode/certain subcarrier.
/// HIPAR value/Filters the displayed results in the constellation diagram by a
/// HIPAR value/certain subcarrier.
ViStatus _VI_FUNC rsspecan_Configure5GNRULAnalysisEvaluationRangeCarrier(
                            ViSession instrSession,
                            ViInt32   mode,
                            ViInt32   value)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, mode, RSSPECAN_VAL_5G_NR_ANALYSIS_MODE_ALL, RSSPECAN_VAL_5G_NR_ANALYSIS_MODE_VAL),
            2, "Mode");

    if (mode == RSSPECAN_VAL_5G_NR_ANALYSIS_MODE_ALL)
    {
        checkErr(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_EVALUATION_RANGE_CARRIER_ALL, NULL));
    }
    else
    {
        viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_EVALUATION_RANGE_CARRIER, value),
                3, "Value");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Filters the displayed results by a specific frame.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR value/Filters the displayed results by a specific frame.
ViStatus _VI_FUNC rsspecan_Configure5GNRULAnalysisEvaluationRangeFrame(
                            ViSession instrSession,
                            ViInt32   value)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_EVALUATION_RANGE_FRAME, value),
            2, "Value");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects a specific measurement (meas ID) from the measurement sequence
/// HIFN  in combined measurements.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR selectedMeasID/Selects a specific measurement (meas ID) from the measurement sequence
/// HIPAR selectedMeasID/in combined measurements.
ViStatus _VI_FUNC rsspecan_Configure5GNRULAnalysisEvaluationRangeSelectedMeasID(
                            ViSession instrSession,
                            ViInt32   selectedMeasID)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, NULL, RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_EVALUATION_RANGE_SELECTED_MEAS_ID, selectedMeasID),
            2, "Selected Meas ID");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function filters the displayed results in the constellation
/// HIFN  diagram by a certain modulation type.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR mode/Filters the displayed results in the constellation diagram by a
/// HIPAR mode/certain modulation type.
/// HIPAR value/Filters the displayed results in the constellation diagram by a
/// HIPAR value/certain modulation type.
ViStatus _VI_FUNC rsspecan_Configure5GNRULAnalysisEvaluationRangeModulation(
                            ViSession instrSession,
                            ViInt32   mode,
                            ViInt32   value)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, mode, RSSPECAN_VAL_5G_NR_ANALYSIS_MODE_ALL, RSSPECAN_VAL_5G_NR_ANALYSIS_MODE_VAL),
            2, "Mode");

    if (mode == RSSPECAN_VAL_5G_NR_ANALYSIS_MODE_ALL)
    {
        checkErr(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_EVALUATION_RANGE_MODULATION_ALL, NULL));
    }
    else
    {
        viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_EVALUATION_RANGE_MODULATION, value),
                3, "Value");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function filters the displayed results by a certain slot.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR mode/Filters the displayed results by a certain slot.
/// HIPAR value/Filters the displayed results by a certain slot.
ViStatus _VI_FUNC rsspecan_Configure5GNRULAnalysisEvaluationRangeSlot(
                            ViSession instrSession,
                            ViInt32   mode,
                            ViInt32   value)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, mode, RSSPECAN_VAL_5G_NR_ANALYSIS_MODE_ALL, RSSPECAN_VAL_5G_NR_ANALYSIS_MODE_VAL),
            2, "Mode");

    if (mode == RSSPECAN_VAL_5G_NR_ANALYSIS_MODE_ALL)
    {
        checkErr(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_EVALUATION_RANGE_SLOT_ALL, NULL));
    }
    else
    {
        viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_EVALUATION_RANGE_SLOT, value),
                3, "Value");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function filters the displayed results by a certain preamble.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR mode/Filters the displayed results by a certain preamble.
/// HIPAR value/Filters the displayed results by a certain preamble.
ViStatus _VI_FUNC rsspecan_Configure5GNRULAnalysisEvaluationRangePreamble(
                            ViSession instrSession,
                            ViInt32   mode,
                            ViInt32   value)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, mode, RSSPECAN_VAL_5G_NR_ANALYSIS_MODE_ALL, RSSPECAN_VAL_5G_NR_ANALYSIS_MODE_VAL),
            2, "Mode");

    if (mode == RSSPECAN_VAL_5G_NR_ANALYSIS_MODE_ALL)
    {
        checkErr(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_EVALUATION_RANGE_PREAMBLE_ALL, NULL));
    }
    else
    {
        viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_EVALUATION_RANGE_PREAMBLE, value),
                3, "Value");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function filters the displayed results by a certain OFDM
/// HIFN  subframe.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR mode/Filters the displayed results by a certain OFDM subframe.
/// HIPAR value/Filters the displayed results by a certain OFDM subframe.
ViStatus _VI_FUNC rsspecan_Configure5GNRULAnalysisEvaluationRangeSubframe(
                            ViSession instrSession,
                            ViInt32   mode,
                            ViInt32   value)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, mode, RSSPECAN_VAL_5G_NR_ANALYSIS_MODE_ALL, RSSPECAN_VAL_5G_NR_ANALYSIS_MODE_VAL),
            2, "Mode");

    if (mode == RSSPECAN_VAL_5G_NR_ANALYSIS_MODE_ALL)
    {
        checkErr(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_EVALUATION_RANGE_SUBFRAME_ALL, NULL));
    }
    else
    {
        viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_EVALUATION_RANGE_SUBFRAME, value),
                3, "Value");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function filters the displayed results in the constellation
/// HIFN  diagram by a certain OFDM symbol.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR mode/Filters the displayed results in the constellation diagram by a
/// HIPAR mode/certain OFDM symbol.
/// HIPAR value/Filters the displayed results in the constellation diagram by a
/// HIPAR value/certain OFDM symbol.
ViStatus _VI_FUNC rsspecan_Configure5GNRULAnalysisEvaluationRangeSymbol(
                            ViSession instrSession,
                            ViInt32   mode,
                            ViInt32   value)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, mode, RSSPECAN_VAL_5G_NR_ANALYSIS_MODE_ALL, RSSPECAN_VAL_5G_NR_ANALYSIS_MODE_VAL),
            2, "Mode");

    if (mode == RSSPECAN_VAL_5G_NR_ANALYSIS_MODE_ALL)
    {
        checkErr(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_EVALUATION_RANGE_SYMBOL_ALL, NULL));
    }
    else
    {
        viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_EVALUATION_RANGE_SYMBOL, value),
                3, "Value");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the EVM of selected resource elements. Queries the average
/// HIFN  result over all carriers
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR result/Queries the EVM of selected resource elements. Queries the average
/// HIPAR result/result over all carriers
/// HIPAR type/Selects the type of measurement.
ViStatus _VI_FUNC rsspecan_Query5GNRULAnalysisAllCarriers(ViSession instrSession,
                                                          ViInt32   type,
                                                          ViReal64* result)
{
    ViStatus error = VI_SUCCESS;
    ViAttr   attr = 0;

    checkErr(RsCore_LockSession(instrSession));

    switch (type)
    {
        case RSSPECAN_VAL_SUMMARY_ALL:
            attr = RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_SUMMARY_ALL_CARRIERS_EVM;
            break;
        case RSSPECAN_VAL_SUMMARY_PEAK:
            attr = RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_SUMMARY_ALL_CARRIERS_EVM_PEAK;
            break;
        case RSSPECAN_VAL_SUMMARY_PHYSICAL_CHANNEL:
            attr = RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_SUMMARY_ALL_CARRIERS_EVM_PHYSICAL_CHANNEL;
            break;
        case RSSPECAN_VAL_SUMMARY_PHYSICAL_SIGNAL:
            attr = RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_SUMMARY_ALL_CARRIERS_EVM_PHYSICAL_SIGNAL;
            break;
        case RSSPECAN_VAL_SUMMARY_POWER:
            attr = RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_SUMMARY_ALL_CARRIERS_SIGNAL_POWER;
            break;
        case RSSPECAN_VAL_SUMMARY_DMRS_PI_2_BPSK:
            attr = RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_SUMMARY_ALL_CARRIERS_EVM_DMRS_PI2_BPSK;
            break;
        case RSSPECAN_VAL_SUMMARY_DMRS_PUSCH_QPSK:
            attr = RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_SUMMARY_ALL_CARRIERS_EVM_DMRS_PUSCH_QPSK;
            break;
        case RSSPECAN_VAL_SUMMARY_DMRS_PUSCH_16QAM:
            attr = RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_SUMMARY_ALL_CARRIERS_EVM_DMRS_PUSCH_16QAM;
            break;
        case RSSPECAN_VAL_SUMMARY_DMRS_PUSCH_64QAM:
            attr = RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_SUMMARY_ALL_CARRIERS_EVM_DMRS_PUSCH_64QAM;
            break;
        case RSSPECAN_VAL_SUMMARY_DMRS_PUSCH_256QAM:
            attr = RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_SUMMARY_ALL_CARRIERS_EVM_DMRS_PUSCH_256QAM;
            break;
        case RSSPECAN_VAL_SUMMARY_PUCCH_DMRS:
            attr = RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_SUMMARY_ALL_CARRIERS_EVM_PUCCH_DMRS;
            break;
        case RSSPECAN_VAL_SUMMARY_PUCCH:
            attr = RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_SUMMARY_ALL_CARRIERS_EVM_PUCCH;
            break;
        case RSSPECAN_VAL_SUMMARY_PUSCH_PI_2_BPSK:
            attr = RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_SUMMARY_ALL_CARRIERS_EVM_PUSCH_PI2_BPSK;
            break;
        case RSSPECAN_VAL_SUMMARY_PUSCH_QPSK:
            attr = RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_SUMMARY_ALL_CARRIERS_EVM_PUSCH_QPSK;
            break;
        case RSSPECAN_VAL_SUMMARY_PUSCH_16QAM:
            attr = RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_SUMMARY_ALL_CARRIERS_EVM_PUSCH_16QAM;
            break;
        case RSSPECAN_VAL_SUMMARY_PUSCH_64QAM:
            attr = RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_SUMMARY_ALL_CARRIERS_EVM_PUSCH_64QAM;
            break;
        case RSSPECAN_VAL_SUMMARY_PUSCH_256QAM:
            attr = RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_SUMMARY_ALL_CARRIERS_EVM_PUSCH_256QAM;
            break;
        case RSSPECAN_VAL_SUMMARY_DMRS_PUSCH_1024QAM:
            attr = RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_SUMMARY_ALL_CARRIERS_EVM_DMRS_PUSCH_1024QAM;
            break;
        case RSSPECAN_VAL_SUMMARY_DMRS_PUSCH_RE_1024QAM:
            attr = RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_SUMMARY_ALL_CARRIERS_EVM_DMRS_PUSCH_RESOURCE_ELEMENTS_1024QAM;
            break;
        default:
            checkErr(RsCore_InvalidViInt32Value(instrSession, type));
            break;
    }

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, NULL, attr, result),
            3, "Result");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the EVM of all resource element.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR cc/Component Carrier parameter.
/// HIPAR frame/Sets the Frame parameter.
/// HIPAR bufferSize/Sets the buffer size.
/// HIPAR value/Queries the EVM of all resource element.
ViStatus _VI_FUNC rsspecan_Query5GNRULAnalysisSummaryAll(ViSession instrSession,
                                                         ViInt32   cc,
                                                         ViInt32   frame,
                                                         ViInt32   bufferSize,
                                                         ViChar    value[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_frame[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_SUMMARY_EVM_ALL,
            "cc", cc - 1, RS_REPCAP_BUF_SIZE, repCap));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_SUMMARY_EVM_ALL,
            "NR5GFrame", frame - 1, RS_REPCAP_BUF_SIZE, repCap_frame));

    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_frame, NULL);

    viCheckParm(rsspecan_GetAttributeViString(instrSession, repCap, RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_SUMMARY_EVM_ALL,
            bufferSize, value),
            5, "Value");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the EVM of all resource element.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR cc/Component Carrier parameter.
/// HIPAR frame/Sets the Frame parameter.
/// HIPAR type/Sets the ratio of average frequency deviation type.
/// HIPAR value/Queries the EVM of all resource element.
ViStatus _VI_FUNC rsspecan_Query5GNRULAnalysisSummaryEVMAll(ViSession instrSession,
                                                            ViInt32   cc,
                                                            ViInt32   frame,
                                                            ViInt32   type,
                                                            ViReal64* value)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_frame[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_SUMMARY_EVM_ALL_MINIMUM,
            "cc", cc - 1, RS_REPCAP_BUF_SIZE, repCap));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_SUMMARY_EVM_ALL_MINIMUM,
            "NR5GFrame", frame - 1, RS_REPCAP_BUF_SIZE, repCap_frame));

    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_frame, NULL);

    switch (type)
    {
        case RSSPECAN_VAL_MEASTYPE_MIN:
            viCheckParm(rsspecan_GetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_SUMMARY_EVM_ALL_MINIMUM, value),
                    5, "Value");
            break;
        case RSSPECAN_VAL_MEASTYPE_MAX:
            viCheckParm(rsspecan_GetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_SUMMARY_EVM_ALL_MAXIMUM, value),
                    5, "Value");
            break;
        case RSSPECAN_VAL_MEASTYPE_AVER:
            viCheckParm(rsspecan_GetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_SUMMARY_EVM_ALL_AVERAGE, value),
                    5, "Value");
            break;
        default:
            viCheckParm(RsCore_InvalidViInt32Value(instrSession, type), 4, "Type");
            break;
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the EVM of the physical channel.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR cc/Component Carrier parameter.
/// HIPAR frame/Sets the Frame parameter.
/// HIPAR type/Sets the ratio of average frequency deviation type.
/// HIPAR value/Queries the EVM of the physical channel.
ViStatus _VI_FUNC rsspecan_Query5GNRULAnalysisSummaryEVMPhysicalChannel(
                            ViSession instrSession,
                            ViInt32   cc,
                            ViInt32   frame,
                            ViInt32   type,
                            ViReal64* value)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_frame[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_SUMMARY_EVM_PHYSICAL_CHANNEL_MINIMUM,
            "cc", cc - 1, RS_REPCAP_BUF_SIZE, repCap));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_SUMMARY_EVM_PHYSICAL_CHANNEL_MINIMUM,
            "NR5GFrame", frame - 1, RS_REPCAP_BUF_SIZE, repCap_frame));

    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_frame, NULL);

    switch (type)
    {
        case RSSPECAN_VAL_MEASTYPE_MIN:
            viCheckParm(rsspecan_GetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_SUMMARY_EVM_PHYSICAL_CHANNEL_MINIMUM, value),
                    5, "Value");
            break;
        case RSSPECAN_VAL_MEASTYPE_MAX:
            viCheckParm(rsspecan_GetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_SUMMARY_EVM_PHYSICAL_CHANNEL_MAXIMUM, value),
                    5, "Value");
            break;
        case RSSPECAN_VAL_MEASTYPE_AVER:
            viCheckParm(rsspecan_GetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_SUMMARY_EVM_PHYSICAL_CHANNEL_AVERAGE, value),
                    5, "Value");
            break;
        default:
            viCheckParm(RsCore_InvalidViInt32Value(instrSession, type), 4, "Type");
            break;
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the EVM of the physical signal.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR cc/Component Carrier parameter.
/// HIPAR frame/Sets the Frame parameter.
/// HIPAR type/Sets the ratio of average frequency deviation type.
/// HIPAR value/Queries the EVM of the physical signal.
ViStatus _VI_FUNC rsspecan_Query5GNRULAnalysisSummaryEVMPhysicalSignal(
                            ViSession instrSession,
                            ViInt32   cc,
                            ViInt32   frame,
                            ViInt32   type,
                            ViReal64* value)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_frame[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_SUMMARY_EVM_PHYSICAL_SIGNAL_MINIMUM,
            "cc", cc - 1, RS_REPCAP_BUF_SIZE, repCap));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_SUMMARY_EVM_PHYSICAL_SIGNAL_MINIMUM,
            "NR5GFrame", frame - 1, RS_REPCAP_BUF_SIZE, repCap_frame));

    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_frame, NULL);

    switch (type)
    {
        case RSSPECAN_VAL_MEASTYPE_MIN:
            viCheckParm(rsspecan_GetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_SUMMARY_EVM_PHYSICAL_SIGNAL_MINIMUM, value),
                    5, "Value");
            break;
        case RSSPECAN_VAL_MEASTYPE_MAX:
            viCheckParm(rsspecan_GetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_SUMMARY_EVM_PHYSICAL_SIGNAL_MAXIMUM, value),
                    5, "Value");
            break;
        case RSSPECAN_VAL_MEASTYPE_AVER:
            viCheckParm(rsspecan_GetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_SUMMARY_EVM_PHYSICAL_SIGNAL_AVERAGE, value),
                    5, "Value");
            break;
        default:
            viCheckParm(RsCore_InvalidViInt32Value(instrSession, type), 4, "Type");
            break;
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the peak EVM.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR cc/Component Carrier parameter.
/// HIPAR frame/Sets the Frame parameter.
/// HIPAR type/Sets the ratio of average frequency deviation type.
/// HIPAR value/Queries the peak EVM.
ViStatus _VI_FUNC rsspecan_Query5GNRULAnalysisSummaryEVMPeak(ViSession instrSession,
                                                             ViInt32   cc,
                                                             ViInt32   frame,
                                                             ViInt32   type,
                                                             ViReal64  *value)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_frame[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_SUMMARY_EVM_PEAK_MINIMUM,
            "cc", cc - 1, RS_REPCAP_BUF_SIZE, repCap));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_SUMMARY_EVM_PEAK_MINIMUM,
            "NR5GFrame", frame - 1, RS_REPCAP_BUF_SIZE, repCap_frame));

    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_frame, NULL);

    switch (type)
    {
        case RSSPECAN_VAL_MEASTYPE_MIN:
            viCheckParm(rsspecan_GetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_SUMMARY_EVM_PEAK_MINIMUM, value),
                    5, "Value");
            break;
        case RSSPECAN_VAL_MEASTYPE_MAX:
            viCheckParm(rsspecan_GetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_SUMMARY_EVM_PEAK_MAXIMUM, value),
                    5, "Value");
            break;
        case RSSPECAN_VAL_MEASTYPE_AVER:
            viCheckParm(rsspecan_GetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_SUMMARY_EVM_PEAK_AVERAGE, value),
                    5, "Value");
            break;
        default:
            viCheckParm(RsCore_InvalidViInt32Value(instrSession, type), 4, "Type");
            break;
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the EVM of the PRACH.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR cc/Component Carrier parameter.
/// HIPAR frame/Sets the Frame parameter.
/// HIPAR type/Sets the ratio of average frequency deviation type.
/// HIPAR value/Queries the EVM of the PRACH.
ViStatus _VI_FUNC rsspecan_Query5GNRULAnalysisSummaryEVMPRACH(
                            ViSession instrSession,
                            ViInt32   cc,
                            ViInt32   frame,
                            ViInt32   type,
                            ViReal64  *value)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_frame[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_SUMMARY_EVM_PRACH_MINIMUM,
            "cc", cc - 1, RS_REPCAP_BUF_SIZE, repCap));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_SUMMARY_EVM_PRACH_MINIMUM,
            "NR5GFrame", frame - 1, RS_REPCAP_BUF_SIZE, repCap_frame));

    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_frame, NULL);

    switch (type)
    {
        case RSSPECAN_VAL_MEASTYPE_MIN:
            viCheckParm(rsspecan_GetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_SUMMARY_EVM_PRACH_MINIMUM, value),
                    5, "Value");
            break;
        case RSSPECAN_VAL_MEASTYPE_MAX:
            viCheckParm(rsspecan_GetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_SUMMARY_EVM_PRACH_MAXIMUM, value),
                    5, "Value");
            break;
        case RSSPECAN_VAL_MEASTYPE_AVER:
            viCheckParm(rsspecan_GetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_SUMMARY_EVM_PRACH_AVERAGE, value),
                    5, "Value");
            break;
        default:
            viCheckParm(RsCore_InvalidViInt32Value(instrSession, type), 4, "Type");
            break;
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the EVM of all PUSCH DMRS resource elements with a PI/2 BPSK
/// HIFN  modulation.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR cc/Component Carrier parameter.
/// HIPAR frame/Sets the Frame parameter.
/// HIPAR type/Sets the ratio of average frequency deviation type.
/// HIPAR value/Queries the EVM of all PUSCH DMRS resource elements with a PI/2 BPSK
/// HIPAR value/modulation.
ViStatus _VI_FUNC rsspecan_Query5GNRULAnalysisSummaryEVMPI2BPSK(
                            ViSession instrSession,
                            ViInt32   cc,
                            ViInt32   frame,
                            ViInt32   type,
                            ViReal64* value)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_frame[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_SUMMARY_EVM_PI2_BPSK_MINIMUM,
            "cc", cc - 1, RS_REPCAP_BUF_SIZE, repCap));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_SUMMARY_EVM_PI2_BPSK_MINIMUM,
            "NR5GFrame", frame - 1, RS_REPCAP_BUF_SIZE, repCap_frame));

    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_frame, NULL);

    switch (type)
    {
        case RSSPECAN_VAL_MEASTYPE_MIN:
            viCheckParm(rsspecan_GetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_SUMMARY_EVM_PI2_BPSK_MINIMUM, value),
                    5, "Value");
            break;
        case RSSPECAN_VAL_MEASTYPE_MAX:
            viCheckParm(rsspecan_GetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_SUMMARY_EVM_PI2_BPSK_MAXIMUM, value),
                    5, "Value");
            break;
        case RSSPECAN_VAL_MEASTYPE_AVER:
            viCheckParm(rsspecan_GetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_SUMMARY_EVM_PI2_BPSK_AVERAGE, value),
                    5, "Value");
            break;
        default:
            viCheckParm(RsCore_InvalidViInt32Value(instrSession, type), 4, "Type");
            break;
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the EVM of all PUSCH DMRS resource elements with a QPSK
/// HIFN  modulation.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR cc/Component Carrier parameter.
/// HIPAR frame/Sets the Frame parameter.
/// HIPAR type/Sets the ratio of average frequency deviation type.
/// HIPAR value/Queries the EVM of all PUSCH DMRS resource elements with a QPSK
/// HIPAR value/modulation.
ViStatus _VI_FUNC rsspecan_Query5GNRULAnalysisSummaryEVMQPSK(ViSession instrSession,
                                                             ViInt32   cc,
                                                             ViInt32   frame,
                                                             ViInt32   type,
                                                             ViReal64* value)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_frame[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_SUMMARY_EVM_QPSK_MINIMUM,
            "cc", cc - 1, RS_REPCAP_BUF_SIZE, repCap));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_SUMMARY_EVM_QPSK_MINIMUM,
            "NR5GFrame", frame - 1, RS_REPCAP_BUF_SIZE, repCap_frame));

    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_frame, NULL);

    switch (type)
    {
        case RSSPECAN_VAL_MEASTYPE_MIN:
            viCheckParm(rsspecan_GetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_SUMMARY_EVM_QPSK_MINIMUM, value),
                    5, "Value");
            break;
        case RSSPECAN_VAL_MEASTYPE_MAX:
            viCheckParm(rsspecan_GetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_SUMMARY_EVM_QPSK_MAXIMUM, value),
                    5, "Value");
            break;
        case RSSPECAN_VAL_MEASTYPE_AVER:
            viCheckParm(rsspecan_GetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_SUMMARY_EVM_QPSK_AVERAGE, value),
                    5, "Value");
            break;
        default:
            viCheckParm(RsCore_InvalidViInt32Value(instrSession, type), 4, "Type");
            break;
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the EVM of all PUSCH DMRS resource elements with a 16QAM
/// HIFN  modulation.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR cc/Component Carrier parameter.
/// HIPAR frame/Sets the Frame parameter.
/// HIPAR type/Sets the ratio of average frequency deviation type.
/// HIPAR value/Queries the EVM of all PUSCH DMRS resource elements with a 16QAM
/// HIPAR value/modulation.
ViStatus _VI_FUNC rsspecan_Query5GNRULAnalysisSummaryEVM16QAM(
                            ViSession instrSession,
                            ViInt32   cc,
                            ViInt32   frame,
                            ViInt32   type,
                            ViReal64* value)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_frame[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_SUMMARY_EVM_16QAM_MINIMUM,
            "cc", cc - 1, RS_REPCAP_BUF_SIZE, repCap));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_SUMMARY_EVM_16QAM_MINIMUM,
            "NR5GFrame", frame - 1, RS_REPCAP_BUF_SIZE, repCap_frame));

    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_frame, NULL);

    switch (type)
    {
        case RSSPECAN_VAL_MEASTYPE_MIN:
            viCheckParm(rsspecan_GetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_SUMMARY_EVM_16QAM_MINIMUM, value),
                    5, "Value");
            break;
        case RSSPECAN_VAL_MEASTYPE_MAX:
            viCheckParm(rsspecan_GetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_SUMMARY_EVM_16QAM_MAXIMUM, value),
                    5, "Value");
            break;
        case RSSPECAN_VAL_MEASTYPE_AVER:
            viCheckParm(rsspecan_GetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_SUMMARY_EVM_16QAM_AVERAGE, value),
                    5, "Value");
            break;
        default:
            viCheckParm(RsCore_InvalidViInt32Value(instrSession, type), 4, "Type");
            break;
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the EVM of all PUSCH DMRS resource elements with a 64QAM
/// HIFN  modulation.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR cc/Component Carrier parameter.
/// HIPAR frame/Sets the Frame parameter.
/// HIPAR type/Sets the ratio of average frequency deviation type.
/// HIPAR value/Queries the EVM of all PUSCH DMRS resource elements with a 64QAM
/// HIPAR value/modulation.
ViStatus _VI_FUNC rsspecan_Query5GNRULAnalysisSummaryEVM64QAM(
                            ViSession instrSession,
                            ViInt32   cc,
                            ViInt32   frame,
                            ViInt32   type,
                            ViReal64* value)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_frame[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_SUMMARY_EVM_64QAM_MINIMUM,
            "cc", cc - 1, RS_REPCAP_BUF_SIZE, repCap));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_SUMMARY_EVM_64QAM_MINIMUM,
            "NR5GFrame", frame - 1, RS_REPCAP_BUF_SIZE, repCap_frame));

    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_frame, NULL);

    switch (type)
    {
        case RSSPECAN_VAL_MEASTYPE_MIN:
            viCheckParm(rsspecan_GetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_SUMMARY_EVM_64QAM_MINIMUM, value),
                    5, "Value");
            break;
        case RSSPECAN_VAL_MEASTYPE_MAX:
            viCheckParm(rsspecan_GetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_SUMMARY_EVM_64QAM_MAXIMUM, value),
                    5, "Value");
            break;
        case RSSPECAN_VAL_MEASTYPE_AVER:
            viCheckParm(rsspecan_GetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_SUMMARY_EVM_64QAM_AVERAGE, value),
                    5, "Value");
            break;
        default:
            viCheckParm(RsCore_InvalidViInt32Value(instrSession, type), 4, "Type");
            break;
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the EVM of all PUSCH DMRS resource elements with a 256QAM
/// HIFN  modulation.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR cc/Component Carrier parameter.
/// HIPAR frame/Sets the Frame parameter.
/// HIPAR type/Sets the ratio of average frequency deviation type.
/// HIPAR value/Queries the EVM of all PUSCH DMRS resource elements with a 256QAM
/// HIPAR value/modulation.
ViStatus _VI_FUNC rsspecan_Query5GNRULAnalysisSummaryEVM256QAM(
                            ViSession instrSession,
                            ViInt32   cc,
                            ViInt32   frame,
                            ViInt32   type,
                            ViReal64* value)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_frame[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_SUMMARY_EVM_256QAM_MINIMUM,
            "cc", cc - 1, RS_REPCAP_BUF_SIZE, repCap));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_SUMMARY_EVM_256QAM_MINIMUM,
            "NR5GFrame", frame - 1, RS_REPCAP_BUF_SIZE, repCap_frame));

    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_frame, NULL);

    switch (type)
    {
        case RSSPECAN_VAL_MEASTYPE_MIN:
            viCheckParm(rsspecan_GetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_SUMMARY_EVM_256QAM_MINIMUM, value),
                    5, "Value");
            break;
        case RSSPECAN_VAL_MEASTYPE_MAX:
            viCheckParm(rsspecan_GetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_SUMMARY_EVM_256QAM_MAXIMUM, value),
                    5, "Value");
            break;
        case RSSPECAN_VAL_MEASTYPE_AVER:
            viCheckParm(rsspecan_GetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_SUMMARY_EVM_256QAM_AVERAGE, value),
                    5, "Value");
            break;
        default:
            viCheckParm(RsCore_InvalidViInt32Value(instrSession, type), 4, "Type");
            break;
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the EVM of all PUCCH DMRS resource elements.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR cc/Component Carrier parameter.
/// HIPAR frame/Sets the Frame parameter.
/// HIPAR type/Sets the ratio of average frequency deviation type.
/// HIPAR value/Queries the EVM of all PUCCH DMRS resource elements.
ViStatus _VI_FUNC rsspecan_Query5GNRULAnalysisSummaryEVMDMRSResourceElements(
                            ViSession instrSession,
                            ViInt32   cc,
                            ViInt32   frame,
                            ViInt32   type,
                            ViReal64* value)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_frame[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_SUMMARY_EVM_DMRS_RESOURCE_ELEMENTS_MINIMUM,
            "cc", cc - 1, RS_REPCAP_BUF_SIZE, repCap));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_SUMMARY_EVM_DMRS_RESOURCE_ELEMENTS_MINIMUM,
            "NR5GFrame", frame - 1, RS_REPCAP_BUF_SIZE, repCap_frame));

    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_frame, NULL);

    switch (type)
    {
        case RSSPECAN_VAL_MEASTYPE_MIN:
            viCheckParm(rsspecan_GetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_SUMMARY_EVM_DMRS_RESOURCE_ELEMENTS_MINIMUM, value),
                    5, "Value");
            break;
        case RSSPECAN_VAL_MEASTYPE_MAX:
            viCheckParm(rsspecan_GetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_SUMMARY_EVM_DMRS_RESOURCE_ELEMENTS_MAXIMUM, value),
                    5, "Value");
            break;
        case RSSPECAN_VAL_MEASTYPE_AVER:
            viCheckParm(rsspecan_GetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_SUMMARY_EVM_DMRS_RESOURCE_ELEMENTS_AVERAGE, value),
                    5, "Value");
            break;
        default:
            viCheckParm(RsCore_InvalidViInt32Value(instrSession, type), 4, "Type");
            break;
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the EVM of all PUCCH resource elements.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR cc/Component Carrier parameter.
/// HIPAR frame/Sets the Frame parameter.
/// HIPAR type/Sets the ratio of average frequency deviation type.
/// HIPAR value/Queries the EVM of all PUCCH resource elements.
ViStatus _VI_FUNC rsspecan_Query5GNRULAnalysisSummaryEVMResourceElements(
                            ViSession instrSession,
                            ViInt32   cc,
                            ViInt32   frame,
                            ViInt32   type,
                            ViReal64* value)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_frame[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_SUMMARY_EVM_RESOURCE_ELEMENTS_MINIMUM,
            "cc", cc - 1, RS_REPCAP_BUF_SIZE, repCap));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_SUMMARY_EVM_RESOURCE_ELEMENTS_MINIMUM,
            "NR5GFrame", frame - 1, RS_REPCAP_BUF_SIZE, repCap_frame));

    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_frame, NULL);

    switch (type)
    {
        case RSSPECAN_VAL_MEASTYPE_MIN:
            viCheckParm(rsspecan_GetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_SUMMARY_EVM_RESOURCE_ELEMENTS_MINIMUM, value),
                    5, "Value");
            break;
        case RSSPECAN_VAL_MEASTYPE_MAX:
            viCheckParm(rsspecan_GetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_SUMMARY_EVM_RESOURCE_ELEMENTS_MAXIMUM, value),
                    5, "Value");
            break;
        case RSSPECAN_VAL_MEASTYPE_AVER:
            viCheckParm(rsspecan_GetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_SUMMARY_EVM_RESOURCE_ELEMENTS_AVERAGE, value),
                    5, "Value");
            break;
        default:
            viCheckParm(RsCore_InvalidViInt32Value(instrSession, type), 4, "Type");
            break;
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the EVM of all PUSCH resource elements with a pi/2-BPSK
/// HIFN  modulation.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR cc/Component Carrier parameter.
/// HIPAR frame/Sets the Frame parameter.
/// HIPAR type/Sets the ratio of average frequency deviation type.
/// HIPAR value/Queries the EVM of all PUSCH resource elements with a pi/2-BPSK
/// HIPAR value/modulation.
ViStatus _VI_FUNC rsspecan_Query5GNRULAnalysisSummaryEVMPUSCHPI2BPSK(
                            ViSession instrSession,
                            ViInt32   cc,
                            ViInt32   frame,
                            ViInt32   type,
                            ViReal64* value)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_frame[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_SUMMARY_EVM_PUSCH_PI2_BPSK_MINIMUM,
            "cc", cc - 1, RS_REPCAP_BUF_SIZE, repCap));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_SUMMARY_EVM_PUSCH_PI2_BPSK_MINIMUM,
            "NR5GFrame", frame - 1, RS_REPCAP_BUF_SIZE, repCap_frame));

    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_frame, NULL);

    switch (type)
    {
        case RSSPECAN_VAL_MEASTYPE_MIN:
            viCheckParm(rsspecan_GetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_SUMMARY_EVM_PUSCH_PI2_BPSK_MINIMUM, value),
                    5, "Value");
            break;
        case RSSPECAN_VAL_MEASTYPE_MAX:
            viCheckParm(rsspecan_GetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_SUMMARY_EVM_PUSCH_PI2_BPSK_MAXIMUM, value),
                    5, "Value");
            break;
        case RSSPECAN_VAL_MEASTYPE_AVER:
            viCheckParm(rsspecan_GetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_SUMMARY_EVM_PUSCH_PI2_BPSK_AVERAGE, value),
                    5, "Value");
            break;
        default:
            viCheckParm(RsCore_InvalidViInt32Value(instrSession, type), 4, "Type");
            break;
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the EVM of all PUSCH resource elements with a QPSK modulation.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR cc/Component Carrier parameter.
/// HIPAR frame/Sets the Frame parameter.
/// HIPAR type/Sets the ratio of average frequency deviation type.
/// HIPAR value/Queries the EVM of all PUSCH resource elements with a QPSK modulation.
ViStatus _VI_FUNC rsspecan_Query5GNRULAnalysisSummaryEVMPUSCHQPSK(
                            ViSession instrSession,
                            ViInt32   cc,
                            ViInt32   frame,
                            ViInt32   type,
                            ViReal64* value)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_frame[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_SUMMARY_EVM_PUSCH_QPSK_MINIMUM,
            "cc", cc - 1, RS_REPCAP_BUF_SIZE, repCap));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_SUMMARY_EVM_PUSCH_QPSK_MINIMUM,
            "NR5GFrame", frame - 1, RS_REPCAP_BUF_SIZE, repCap_frame));

    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_frame, NULL);

    switch (type)
    {
        case RSSPECAN_VAL_MEASTYPE_MIN:
            viCheckParm(rsspecan_GetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_SUMMARY_EVM_PUSCH_QPSK_MINIMUM, value),
                    5, "Value");
            break;
        case RSSPECAN_VAL_MEASTYPE_MAX:
            viCheckParm(rsspecan_GetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_SUMMARY_EVM_PUSCH_QPSK_MAXIMUM, value),
                    5, "Value");
            break;
        case RSSPECAN_VAL_MEASTYPE_AVER:
            viCheckParm(rsspecan_GetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_SUMMARY_EVM_PUSCH_QPSK_AVERAGE, value),
                    5, "Value");
            break;
        default:
            viCheckParm(RsCore_InvalidViInt32Value(instrSession, type), 4, "Type");
            break;
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the EVM of all PUSCH resource elements with a 16QAM
/// HIFN  modulation.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR cc/Component Carrier parameter.
/// HIPAR frame/Sets the Frame parameter.
/// HIPAR type/Sets the ratio of average frequency deviation type.
/// HIPAR value/Queries the EVM of all PUSCH resource elements with a 16QAM
/// HIPAR value/modulation.
ViStatus _VI_FUNC rsspecan_Query5GNRULAnalysisSummaryEVMPUSCH16QAM(
                            ViSession instrSession,
                            ViInt32   cc,
                            ViInt32   frame,
                            ViInt32   type,
                            ViReal64* value)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_frame[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_SUMMARY_EVM_PUSCH_16QAM_MINIMUM,
            "cc", cc - 1, RS_REPCAP_BUF_SIZE, repCap));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_SUMMARY_EVM_PUSCH_16QAM_MINIMUM,
            "NR5GFrame", frame - 1, RS_REPCAP_BUF_SIZE, repCap_frame));

    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_frame, NULL);

    switch (type)
    {
        case RSSPECAN_VAL_MEASTYPE_MIN:
            viCheckParm(rsspecan_GetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_SUMMARY_EVM_PUSCH_16QAM_MINIMUM, value),
                    5, "Value");
            break;
        case RSSPECAN_VAL_MEASTYPE_MAX:
            viCheckParm(rsspecan_GetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_SUMMARY_EVM_PUSCH_16QAM_MAXIMUM, value),
                    5, "Value");
            break;
        case RSSPECAN_VAL_MEASTYPE_AVER:
            viCheckParm(rsspecan_GetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_SUMMARY_EVM_PUSCH_16QAM_AVERAGE, value),
                    5, "Value");
            break;
        default:
            viCheckParm(RsCore_InvalidViInt32Value(instrSession, type), 4, "Type");
            break;
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the EVM of all PUSCH resource elements with a 64QAM
/// HIFN  modulation.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR cc/Component Carrier parameter.
/// HIPAR frame/Sets the Frame parameter.
/// HIPAR type/Sets the ratio of average frequency deviation type.
/// HIPAR value/Queries the EVM of all PUSCH resource elements with a 64QAM
/// HIPAR value/modulation.
ViStatus _VI_FUNC rsspecan_Query5GNRULAnalysisSummaryEVMPUSCH64QAM(
                            ViSession instrSession,
                            ViInt32   cc,
                            ViInt32   frame,
                            ViInt32   type,
                            ViReal64* value)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_frame[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_SUMMARY_EVM_PUSCH_64QAM_MINIMUM,
            "cc", cc - 1, RS_REPCAP_BUF_SIZE, repCap));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_SUMMARY_EVM_PUSCH_64QAM_MINIMUM,
            "NR5GFrame", frame - 1, RS_REPCAP_BUF_SIZE, repCap_frame));

    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_frame, NULL);

    switch (type)
    {
        case RSSPECAN_VAL_MEASTYPE_MIN:
            viCheckParm(rsspecan_GetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_SUMMARY_EVM_PUSCH_64QAM_MINIMUM, value),
                    5, "Value");
            break;
        case RSSPECAN_VAL_MEASTYPE_MAX:
            viCheckParm(rsspecan_GetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_SUMMARY_EVM_PUSCH_64QAM_MAXIMUM, value),
                    5, "Value");
            break;
        case RSSPECAN_VAL_MEASTYPE_AVER:
            viCheckParm(rsspecan_GetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_SUMMARY_EVM_PUSCH_64QAM_AVERAGE, value),
                    5, "Value");
            break;
        default:
            viCheckParm(RsCore_InvalidViInt32Value(instrSession, type), 4, "Type");
            break;
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the EVM of all PUSCH resource elements with a 256QAM
/// HIFN  modulation.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR cc/Component Carrier parameter.
/// HIPAR frame/Sets the Frame parameter.
/// HIPAR type/Sets the ratio of average frequency deviation type.
/// HIPAR value/Queries the EVM of all PUSCH resource elements with a 256QAM
/// HIPAR value/modulation.
ViStatus _VI_FUNC rsspecan_Query5GNRULAnalysisSummaryEVMPUSCH256QAM(
                            ViSession instrSession,
                            ViInt32   cc,
                            ViInt32   frame,
                            ViInt32   type,
                            ViReal64* value)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_frame[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_SUMMARY_EVM_PUSCH_256QAM_MINIMUM,
            "cc", cc - 1, RS_REPCAP_BUF_SIZE, repCap));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_SUMMARY_EVM_PUSCH_256QAM_MINIMUM,
            "NR5GFrame", frame - 1, RS_REPCAP_BUF_SIZE, repCap_frame));

    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_frame, NULL);

    switch (type)
    {
        case RSSPECAN_VAL_MEASTYPE_MIN:
            viCheckParm(rsspecan_GetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_SUMMARY_EVM_PUSCH_256QAM_MINIMUM, value),
                    5, "Value");
            break;
        case RSSPECAN_VAL_MEASTYPE_MAX:
            viCheckParm(rsspecan_GetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_SUMMARY_EVM_PUSCH_256QAM_MAXIMUM, value),
                    5, "Value");
            break;
        case RSSPECAN_VAL_MEASTYPE_AVER:
            viCheckParm(rsspecan_GetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_SUMMARY_EVM_PUSCH_256QAM_AVERAGE, value),
                    5, "Value");
            break;
        default:
            viCheckParm(RsCore_InvalidViInt32Value(instrSession, type), 4, "Type");
            break;
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the EVM of all PUSCH resource elements with a 1024QAM
/// HIFN  modulation.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR cc/Component Carrier parameter.
/// HIPAR frame/Sets the Frame parameter.
/// HIPAR type/Sets the ratio of average frequency deviation type.
/// HIPAR value/Queries the EVM of all PUSCH resource elements with a 1024QAM
/// HIPAR value/modulation.
ViStatus _VI_FUNC rsspecan_Query5GNRULAnalysisSummaryEVMPUSCH1024QAM(
                            ViSession instrSession,
                            ViInt32   cc,
                            ViInt32   frame,
                            ViInt32   type,
                            ViReal64* value)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_frame[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_SUMMARY_EVM_PUSCH_1024QAM_MINIMUM,
            "cc", cc - 1, RS_REPCAP_BUF_SIZE, repCap));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_SUMMARY_EVM_PUSCH_1024QAM_MINIMUM,
            "NR5GFrame", frame - 1, RS_REPCAP_BUF_SIZE, repCap_frame));

    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_frame, NULL);

    switch (type)
    {
        case RSSPECAN_VAL_MEASTYPE_MIN:
            viCheckParm(rsspecan_GetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_SUMMARY_EVM_PUSCH_1024QAM_MINIMUM, value),
                    5, "Value");
            break;
        case RSSPECAN_VAL_MEASTYPE_MAX:
            viCheckParm(rsspecan_GetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_SUMMARY_EVM_PUSCH_1024QAM_MAXIMUM, value),
                    5, "Value");
            break;
        case RSSPECAN_VAL_MEASTYPE_AVER:
            viCheckParm(rsspecan_GetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_SUMMARY_EVM_PUSCH_1024QAM_AVERAGE, value),
                    5, "Value");
            break;
        default:
            viCheckParm(RsCore_InvalidViInt32Value(instrSession, type), 4, "Type");
            break;
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the EVM of all PUSCH resource elements with a 1024QAM
/// HIFN  modulation.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN
/// HIFN  RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_SUMMARY_EVM_RESOURCE_ELEMENTS_1024QAM_MINIMUM
/// HIFN
/// HIFN  RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_SUMMARY_EVM_RESOURCE_ELEMENTS_1024QAM_MAXIMUM
/// HIFN
/// HIFN  RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_SUMMARY_EVM_RESOURCE_ELEMENTS_1024QAM_AVERAGE
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     FETCh[:CC<cc>][:ISRC<ant>][:FRAMe<fr>]:SUMMary:EVM:SD1K:MINimum?
/// HIFN     FETCh[:CC<cc>][:ISRC<ant>][:FRAMe<fr>]:SUMMary:EVM:SD1K:MAXimum?
/// HIFN     FETCh[:CC<cc>][:ISRC<ant>][:FRAMe<fr>]:SUMMary:EVM:SD1K[:AVERage]?
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR cc/Component Carrier parameter.
/// HIPAR frame/Sets the Frame parameter.
/// HIPAR type/Sets the ratio of average frequency deviation type.
/// HIPAR value/Queries the EVM of all PUSCH resource elements with a 1024QAM
/// HIPAR value/modulation.
ViStatus _VI_FUNC rsspecan_Query5GNRULAnalysisSummaryEVMResourceElements1024QAM(
                            ViSession instrSession,
                            ViInt32   cc,
                            ViInt32   frame,
                            ViInt32   type,
                            ViReal64* value)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_frame[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_SUMMARY_RESOURCE_ELEMENTS_1024QAM_MINIMUM,
            "cc", cc - 1, RS_REPCAP_BUF_SIZE, repCap));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_SUMMARY_RESOURCE_ELEMENTS_1024QAM_MINIMUM,
            "NR5GFrame", frame - 1, RS_REPCAP_BUF_SIZE, repCap_frame));

    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_frame, NULL);

    switch (type)
    {
        case RSSPECAN_VAL_MEASTYPE_MIN:
            viCheckParm(rsspecan_GetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_SUMMARY_RESOURCE_ELEMENTS_1024QAM_MINIMUM, value),
                    5, "Value");
            break;
        case RSSPECAN_VAL_MEASTYPE_MAX:
            viCheckParm(rsspecan_GetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_SUMMARY_RESOURCE_ELEMENTS_1024QAM_MAXIMUM, value),
                    5, "Value");
            break;
        case RSSPECAN_VAL_MEASTYPE_AVER:
            viCheckParm(rsspecan_GetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_SUMMARY_RESOURCE_ELEMENTS_1024QAM_AVERAGE, value),
                    5, "Value");
            break;
        default:
            viCheckParm(RsCore_InvalidViInt32Value(instrSession, type), 4, "Type");
            break;
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the peak of the BLER.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR cc/Component Carrier parameter.
/// HIPAR frame/Sets the Frame parameter.
/// HIPAR type/Sets the ratio of average frequency deviation type.
/// HIPAR value/Queries the peak of the BLER.
ViStatus _VI_FUNC rsspecan_Query5GNRULAnalysisSummaryEVMBLER(ViSession instrSession,
                                                             ViInt32   cc,
                                                             ViInt32   frame,
                                                             ViInt32   type,
                                                             ViReal64* value)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_frame[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_SUMMARY_EVM_BLER,
            "cc", cc - 1, RS_REPCAP_BUF_SIZE, repCap));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_SUMMARY_EVM_BLER,
            "NR5GFrame", frame - 1, RS_REPCAP_BUF_SIZE, repCap_frame));

    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_frame, NULL);

    switch (type)
    {
        case RSSPECAN_VAL_MEASTYPE_MIN:
            viCheckParm(rsspecan_GetAttributeViReal64(instrSession, NULL, RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_SUMMARY_ALL_CARRIERS_EVM_BLER_MINIMUM, value),
                    5, "Value");
            break;
        case RSSPECAN_VAL_MEASTYPE_MAX:
            viCheckParm(rsspecan_GetAttributeViReal64(instrSession, NULL, RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_SUMMARY_ALL_CARRIERS_EVM_BLER_MAXIMUM, value),
                    5, "Value");
            break;
        case RSSPECAN_VAL_MEASTYPE_AVER:
            viCheckParm(rsspecan_GetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_SUMMARY_EVM_BLER, value),
                    5, "Value");
            break;
        default:
            viCheckParm(RsCore_InvalidViInt32Value(instrSession, type), 4, "Type");
            break;
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the peak of the BLER.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR cc/Component Carrier parameter.
/// HIPAR frame/Sets the Frame parameter.
/// HIPAR type/Sets the ratio of average frequency deviation type.
/// HIPAR value/Queries the peak of the BLER.
ViStatus _VI_FUNC rsspecan_Query5GNRULAnalysisSummaryEVMTPUT(ViSession instrSession,
                                                             ViInt32   cc,
                                                             ViInt32   frame,
                                                             ViInt32   type,
                                                             ViReal64* value)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_frame[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_SUMMARY_EVM_TPUT,
            "cc", cc - 1, RS_REPCAP_BUF_SIZE, repCap));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_SUMMARY_EVM_TPUT,
            "NR5GFrame", frame - 1, RS_REPCAP_BUF_SIZE, repCap_frame));

    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_frame, NULL);

    switch (type)
    {
        case RSSPECAN_VAL_MEASTYPE_MIN:
            viCheckParm(rsspecan_GetAttributeViReal64(instrSession, NULL, RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_SUMMARY_ALL_CARRIERS_EVM_TPUT_MINIMUM, value),
                    5, "Value");
            break;
        case RSSPECAN_VAL_MEASTYPE_MAX:
            viCheckParm(rsspecan_GetAttributeViReal64(instrSession, NULL, RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_SUMMARY_ALL_CARRIERS_EVM_TPUT_MAXIMUM, value),
                    5, "Value");
            break;
        case RSSPECAN_VAL_MEASTYPE_AVER:
            viCheckParm(rsspecan_GetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_SUMMARY_EVM_TPUT, value),
                    5, "Value");
            break;
        default:
            viCheckParm(RsCore_InvalidViInt32Value(instrSession, type), 4, "Type");
            break;
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the frequency error.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR cc/Component Carrier parameter.
/// HIPAR frame/Sets the Frame parameter.
/// HIPAR type/Sets the ratio of average frequency deviation type.
/// HIPAR value/Queries the frequency error.
ViStatus _VI_FUNC rsspecan_Query5GNRULAnalysisSummaryFrequencyError(
                            ViSession instrSession,
                            ViInt32   cc,
                            ViInt32   frame,
                            ViInt32   type,
                            ViReal64* value)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_frame[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_SUMMARY_FREQUENCY_ERROR_MINIMUM,
            "cc", cc - 1, RS_REPCAP_BUF_SIZE, repCap));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_SUMMARY_FREQUENCY_ERROR_MINIMUM,
            "NR5GFrame", frame - 1, RS_REPCAP_BUF_SIZE, repCap_frame));

    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_frame, NULL);

    switch (type)
    {
        case RSSPECAN_VAL_MEASTYPE_MIN:
            viCheckParm(rsspecan_GetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_SUMMARY_FREQUENCY_ERROR_MINIMUM, value),
                    5, "Value");
            break;
        case RSSPECAN_VAL_MEASTYPE_MAX:
            viCheckParm(rsspecan_GetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_SUMMARY_FREQUENCY_ERROR_MAXIMUM, value),
                    5, "Value");
            break;
        case RSSPECAN_VAL_MEASTYPE_AVER:
            viCheckParm(rsspecan_GetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_SUMMARY_FREQUENCY_ERROR_AVERAGE, value),
                    5, "Value");
            break;
        default:
            viCheckParm(RsCore_InvalidViInt32Value(instrSession, type), 4, "Type");
            break;
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the gain imbalance.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR cc/Component Carrier parameter.
/// HIPAR frame/Sets the Frame parameter.
/// HIPAR type/Sets the ratio of average frequency deviation type.
/// HIPAR value/Queries the gain imbalance.
ViStatus _VI_FUNC rsspecan_Query5GNRULAnalysisSummaryGainImbalance(
                            ViSession instrSession,
                            ViInt32   cc,
                            ViInt32   frame,
                            ViInt32   type,
                            ViReal64* value)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_frame[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_SUMMARY_GAIN_IMBALANCE_MINIMUM,
            "cc", cc - 1, RS_REPCAP_BUF_SIZE, repCap));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_SUMMARY_GAIN_IMBALANCE_MINIMUM,
            "NR5GFrame", frame - 1, RS_REPCAP_BUF_SIZE, repCap_frame));

    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_frame, NULL);

    switch (type)
    {
        case RSSPECAN_VAL_MEASTYPE_MIN:
            viCheckParm(rsspecan_GetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_SUMMARY_GAIN_IMBALANCE_MINIMUM, value),
                    5, "Value");
            break;
        case RSSPECAN_VAL_MEASTYPE_MAX:
            viCheckParm(rsspecan_GetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_SUMMARY_GAIN_IMBALANCE_MAXIMUM, value),
                    5, "Value");
            break;
        case RSSPECAN_VAL_MEASTYPE_AVER:
            viCheckParm(rsspecan_GetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_SUMMARY_GAIN_IMBALANCE_AVERAGE, value),
                    5, "Value");
            break;
        default:
            viCheckParm(RsCore_InvalidViInt32Value(instrSession, type), 4, "Type");
            break;
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the I/Q offset.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR cc/Component Carrier parameter.
/// HIPAR frame/Sets the Frame parameter.
/// HIPAR type/Sets the ratio of average frequency deviation type.
/// HIPAR value/Queries the I/Q offset.
ViStatus _VI_FUNC rsspecan_Query5GNRULAnalysisSummaryIQOffset(
                            ViSession instrSession,
                            ViInt32   cc,
                            ViInt32   frame,
                            ViInt32   type,
                            ViReal64* value)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_frame[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_SUMMARY_IQ_OFFSET_MINIMUM,
            "cc", cc - 1, RS_REPCAP_BUF_SIZE, repCap));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_SUMMARY_IQ_OFFSET_MINIMUM,
            "NR5GFrame", frame - 1, RS_REPCAP_BUF_SIZE, repCap_frame));

    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_frame, NULL);

    switch (type)
    {
        case RSSPECAN_VAL_MEASTYPE_MIN:
            viCheckParm(rsspecan_GetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_SUMMARY_IQ_OFFSET_MINIMUM, value),
                    5, "Value");
            break;
        case RSSPECAN_VAL_MEASTYPE_MAX:
            viCheckParm(rsspecan_GetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_SUMMARY_IQ_OFFSET_MAXIMUM, value),
                    5, "Value");
            break;
        case RSSPECAN_VAL_MEASTYPE_AVER:
            viCheckParm(rsspecan_GetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_SUMMARY_IQ_OFFSET_AVERAGE, value),
                    5, "Value");
            break;
        default:
            viCheckParm(RsCore_InvalidViInt32Value(instrSession, type), 4, "Type");
            break;
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the OSTP.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR cc/Component Carrier parameter.
/// HIPAR frame/Sets the Frame parameter.
/// HIPAR type/Sets the ratio of average frequency deviation type.
/// HIPAR value/Queries the OSTP.
ViStatus _VI_FUNC rsspecan_Query5GNRULAnalysisSummaryOSTP(ViSession instrSession,
                                                          ViInt32   cc,
                                                          ViInt32   frame,
                                                          ViInt32   type,
                                                          ViReal64* value)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_frame[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_SUMMARY_OSTP_MINIMUM,
            "cc", cc - 1, RS_REPCAP_BUF_SIZE, repCap));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_SUMMARY_OSTP_MINIMUM,
            "NR5GFrame", frame - 1, RS_REPCAP_BUF_SIZE, repCap_frame));

    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_frame, NULL);

    switch (type)
    {
        case RSSPECAN_VAL_MEASTYPE_MIN:
            viCheckParm(rsspecan_GetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_SUMMARY_OSTP_MINIMUM, value),
                    5, "Value");
            break;
        case RSSPECAN_VAL_MEASTYPE_MAX:
            viCheckParm(rsspecan_GetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_SUMMARY_OSTP_MAXIMUM, value),
                    5, "Value");
            break;
        case RSSPECAN_VAL_MEASTYPE_AVER:
            viCheckParm(rsspecan_GetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_SUMMARY_OSTP_AVERAGE, value),
                    5, "Value");
            break;
        default:
            viCheckParm(RsCore_InvalidViInt32Value(instrSession, type), 4, "Type");
            break;
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the total signal power.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR cc/Component Carrier parameter.
/// HIPAR frame/Sets the Frame parameter.
/// HIPAR type/Sets the ratio of average frequency deviation type.
/// HIPAR value/Queries the total signal power.
ViStatus _VI_FUNC rsspecan_Query5GNRULAnalysisSummaryPower(ViSession instrSession,
                                                           ViInt32   cc,
                                                           ViInt32   frame,
                                                           ViInt32   type,
                                                           ViReal64* value)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_frame[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_SUMMARY_POWER_MINIMUM,
            "cc", cc - 1, RS_REPCAP_BUF_SIZE, repCap));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_SUMMARY_POWER_MINIMUM,
            "NR5GFrame", frame - 1, RS_REPCAP_BUF_SIZE, repCap_frame));

    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_frame, NULL);

    switch (type)
    {
        case RSSPECAN_VAL_MEASTYPE_MIN:
            viCheckParm(rsspecan_GetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_SUMMARY_POWER_MINIMUM, value),
                    5, "Value");
            break;
        case RSSPECAN_VAL_MEASTYPE_MAX:
            viCheckParm(rsspecan_GetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_SUMMARY_POWER_MAXIMUM, value),
                    5, "Value");
            break;
        case RSSPECAN_VAL_MEASTYPE_AVER:
            viCheckParm(rsspecan_GetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_SUMMARY_POWER_AVERAGE, value),
                    5, "Value");
            break;
        default:
            viCheckParm(RsCore_InvalidViInt32Value(instrSession, type), 4, "Type");
            break;
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the quadrature error.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR cc/Component Carrier parameter.
/// HIPAR frame/Sets the Frame parameter.
/// HIPAR type/Sets the ratio of average frequency deviation type.
/// HIPAR value/Queries the quadrature error.
ViStatus _VI_FUNC rsspecan_Query5GNRULAnalysisSummaryQuadratureError(
                            ViSession instrSession,
                            ViInt32   cc,
                            ViInt32   frame,
                            ViInt32   type,
                            ViReal64* value)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_frame[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_SUMMARY_QUADRATURE_ERROR_MINIMUM,
            "cc", cc - 1, RS_REPCAP_BUF_SIZE, repCap));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_SUMMARY_QUADRATURE_ERROR_MINIMUM,
            "NR5GFrame", frame - 1, RS_REPCAP_BUF_SIZE, repCap_frame));

    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_frame, NULL);

    switch (type)
    {
        case RSSPECAN_VAL_MEASTYPE_MIN:
            viCheckParm(rsspecan_GetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_SUMMARY_QUADRATURE_ERROR_MINIMUM, value),
                    5, "Value");
            break;
        case RSSPECAN_VAL_MEASTYPE_MAX:
            viCheckParm(rsspecan_GetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_SUMMARY_QUADRATURE_ERROR_MAXIMUM, value),
                    5, "Value");
            break;
        case RSSPECAN_VAL_MEASTYPE_AVER:
            viCheckParm(rsspecan_GetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_SUMMARY_QUADRATURE_ERROR_AVERAGE, value),
                    5, "Value");
            break;
        default:
            viCheckParm(RsCore_InvalidViInt32Value(instrSession, type), 4, "Type");
            break;
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the sampling error.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR cc/Component Carrier parameter.
/// HIPAR frame/Sets the Frame parameter.
/// HIPAR type/Sets the ratio of average frequency deviation type.
/// HIPAR value/Queries the sampling error.
ViStatus _VI_FUNC rsspecan_Query5GNRULAnalysisSummarySamplingError(
                            ViSession instrSession,
                            ViInt32   cc,
                            ViInt32   frame,
                            ViInt32   type,
                            ViReal64* value)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_frame[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_SUMMARY_SAMPLING_ERROR_MINIMUM,
            "cc", cc - 1, RS_REPCAP_BUF_SIZE, repCap));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_SUMMARY_SAMPLING_ERROR_MINIMUM,
            "NR5GFrame", frame - 1, RS_REPCAP_BUF_SIZE, repCap_frame));

    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_frame, NULL);

    switch (type)
    {
        case RSSPECAN_VAL_MEASTYPE_MIN:
            viCheckParm(rsspecan_GetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_SUMMARY_SAMPLING_ERROR_MINIMUM, value),
                    5, "Value");
            break;
        case RSSPECAN_VAL_MEASTYPE_MAX:
            viCheckParm(rsspecan_GetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_SUMMARY_SAMPLING_ERROR_MAXIMUM, value),
                    5, "Value");
            break;
        case RSSPECAN_VAL_MEASTYPE_AVER:
            viCheckParm(rsspecan_GetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_SUMMARY_SAMPLING_ERROR_AVERAGE, value),
                    5, "Value");
            break;
        default:
            viCheckParm(RsCore_InvalidViInt32Value(instrSession, type), 4, "Type");
            break;
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the ACLR limit check of an event.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR cc/Component Carrier parameter.
/// HIPAR frame/Sets the Frame parameter.
/// HIPAR type/This control select the type of check.
/// HIPAR result/Queries the ACLR limit check of an event.
ViStatus _VI_FUNC rsspecan_Query5GNRULAnalysisACLRLimitCheck(ViSession instrSession,
                                                             ViInt32   cc,
                                                             ViInt32   frame,
                                                             ViInt32   type,
                                                             ViInt32*  result)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_frame[RS_REPCAP_BUF_SIZE] = "";
    ViAttr   attr = 0;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_SUMMARY_ACLR_LIMIT_CHECK,
            "cc", cc - 1, RS_REPCAP_BUF_SIZE, repCap));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_SUMMARY_ACLR_LIMIT_CHECK,
            "NR5GFrame", frame - 1, RS_REPCAP_BUF_SIZE, repCap_frame));

    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_frame, NULL);

    switch (type)
    {
        case RSSPECAN_VAL_ACLR_OVERALL:
            attr = RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_SUMMARY_ACLR_LIMIT_CHECK;
            break;
        case RSSPECAN_VAL_ACLR_ABSOLUTE:
            attr = RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_SUMMARY_ACLR_LIMIT_CHECK_ABSOLUTE;
            break;
        case RSSPECAN_VAL_ACLR_RELATIVE:
            attr = RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_SUMMARY_ACLR_LIMIT_CHECK_RELATIVE;
            break;
        default:
            checkErr(RsCore_InvalidViInt32Value(instrSession, type));
            break;
    }

    viCheckParm(rsspecan_GetAttributeViInt32(instrSession, repCap, attr, result),
            5, "Result");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the overload state of an event.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR cc/Component Carrier parameter.
/// HIPAR frame/Sets the Frame parameter.
/// HIPAR overload/Queries the overload state of an event.
ViStatus _VI_FUNC rsspecan_Query5GNRULAnalysisOverload(ViSession instrSession,
                                                       ViInt32   cc,
                                                       ViInt32   frame,
                                                       ViInt32*  overload)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_frame[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_SUMMARY_OVERLOAD,
            "cc", cc - 1, RS_REPCAP_BUF_SIZE, repCap));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_SUMMARY_OVERLOAD,
            "NR5GFrame", frame - 1, RS_REPCAP_BUF_SIZE, repCap_frame));

    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_frame, NULL);

    viCheckParm(rsspecan_GetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_SUMMARY_OVERLOAD, overload),
            4, "Overload");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the SEM limit check of an event.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR cc/Component Carrier parameter.
/// HIPAR frame/Sets the Frame parameter.
/// HIPAR result/Queries the SEM limit check of an event.
ViStatus _VI_FUNC rsspecan_Query5GNRULAnalysisSEMPassFail(ViSession instrSession,
                                                          ViInt32   cc,
                                                          ViInt32   frame,
                                                          ViInt32*  result)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_frame[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_SUMMARY_SEM_PASS_FAIL,
            "cc", cc - 1, RS_REPCAP_BUF_SIZE, repCap));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_SUMMARY_SEM_PASS_FAIL,
            "NR5GFrame", frame - 1, RS_REPCAP_BUF_SIZE, repCap_frame));

    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_frame, NULL);

    viCheckParm(rsspecan_GetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_SUMMARY_SEM_PASS_FAIL, result),
            4, "Result");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the synchronization state of an event.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR cc/Component Carrier parameter.
/// HIPAR frame/Sets the Frame parameter.
/// HIPAR result/Queries the synchronization state of an event.
ViStatus _VI_FUNC rsspecan_Query5GNRULAnalysisSyncState(ViSession instrSession,
                                                        ViInt32   cc,
                                                        ViInt32   frame,
                                                        ViInt32*  result)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_frame[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_SUMMARY_SYNC_STATE,
            "cc", cc - 1, RS_REPCAP_BUF_SIZE, repCap));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_SUMMARY_SYNC_STATE,
            "NR5GFrame", frame - 1, RS_REPCAP_BUF_SIZE, repCap_frame));

    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_frame, NULL);

    viCheckParm(rsspecan_GetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_SUMMARY_SYNC_STATE, result),
            4, "Result");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the time stamp delta of an event.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR cc/Component Carrier parameter.
/// HIPAR frame/Sets the Frame parameter.
/// HIPAR result/Queries the time stamp delta of an event.
ViStatus _VI_FUNC rsspecan_Query5GNRULAnalysisTimeStampDelta(ViSession instrSession,
                                                             ViInt32   cc,
                                                             ViInt32   frame,
                                                             ViReal64* result)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_frame[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_SUMMARY_OVERLOAD,
            "cc", cc - 1, RS_REPCAP_BUF_SIZE, repCap));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_SUMMARY_OVERLOAD,
            "NR5GFrame", frame - 1, RS_REPCAP_BUF_SIZE, repCap_frame));

    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_frame, NULL);

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_SUMMARY_TIME_STAMP_DELTA, result),
            4, "Result");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the time stamp of an event.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR cc/Component Carrier parameter.
/// HIPAR frame/Sets the Frame parameter.
/// HIPAR result/Queries the time stamp of an event.
ViStatus _VI_FUNC rsspecan_Query5GNRULAnalysisTimeStamp(ViSession instrSession,
                                                        ViInt32   cc,
                                                        ViInt32   frame,
                                                        ViReal64* result)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_frame[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_SUMMARY_TIME_STAMP,
            "cc", cc - 1, RS_REPCAP_BUF_SIZE, repCap));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_SUMMARY_TIME_STAMP,
            "NR5GFrame", frame - 1, RS_REPCAP_BUF_SIZE, repCap_frame));

    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_frame, NULL);

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_SUMMARY_TIME_STAMP, result),
            4, "Result");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the limit specified by 3GPP for the frequency error.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR measurementType/Selects measurement type.
/// HIPAR result/Queries the limit specified by 3GPP for the frequency error.
ViStatus _VI_FUNC rsspecan_Query5GNRULEVMLimitsFrequencyError(
                            ViSession instrSession,
                            ViInt32   measurementType,
                            ViInt32*  result)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    switch (measurementType)
    {
        case RSSPECAN_VAL_MEASTYPE_AVER:
            viCheckParm(rsspecan_GetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_5G_NR_UL_EVM_LIMIT_FREQUENCY_ERROR, result),
                    3, "Result");
            break;
        default:
            checkErr(RsCore_InvalidViInt32Value(instrSession, measurementType));
            break;
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the limit specified by 3GPP for the EVM of all PUSCH resource
/// HIFN  elements with a QPSK modulation.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR withQPSK/Queries the limit specified by 3GPP for the EVM of all PUSCH resource
/// HIPAR withQPSK/elements with a QPSK modulation.
/// HIPAR with16QAM/Queries the limit specified by 3GPP for the EVM of all PUSCH resource
/// HIPAR with16QAM/elements with a 16QAM modulation.
/// HIPAR with64QAM/Queries the limit specified by 3GPP for the EVM of all PUSCH resource
/// HIPAR with64QAM/elements with a 64QAM modulation.
/// HIPAR with256QAM/Queries the limit specified by 3GPP for the EVM of all PUSCH resource
/// HIPAR with256QAM/elements with a 256QAM modulation.
ViStatus _VI_FUNC rsspecan_Query5GNRULEVMLimitsPUSCHResource(ViSession instrSession,
                                                             ViInt32*  withQPSK,
                                                             ViInt32*  with16QAM,
                                                             ViInt32*  with64QAM,
                                                             ViInt32*  with256QAM)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_GetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_5G_NR_UL_EVM_LIMITS_PUSCH_RESOURCE_WITH_QPSK, withQPSK),
            2, "With QPSK");

    viCheckParm(rsspecan_GetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_5G_NR_UL_EVM_LIMITS_PUSCH_RESOURCE_WITH_16QAM, with16QAM),
            3, "With 16QAM");

    viCheckParm(rsspecan_GetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_5G_NR_UL_EVM_LIMITS_PUSCH_RESOURCE_WITH_64QAM, with64QAM),
            4, "With 64QAM");

    viCheckParm(rsspecan_GetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_5G_NR_UL_EVM_LIMITS_PUSCH_RESOURCE_WITH_256QAM, with256QAM),
            5, "With 256QAM");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the limit specified by 3GPP for the EVM of all PUSCH resource
/// HIFN  elements with a 1024QAM modulation.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR with1024QAM/Queries the limit specified by 3GPP for the EVM of all PUSCH resource
/// HIPAR with1024QAM/elements with a 1024QAM modulation.
ViStatus _VI_FUNC rsspecan_Query5GNRULEVMLimitsPUSCHResource1024QAM(
                            ViSession instrSession,
                            ViInt32*  with1024QAM)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_GetAttributeViInt32(instrSession, NULL, RSSPECAN_ATTR_5G_NR_UL_EVM_LIMITS_PUSCH_RESOURCE_WITH_1024QAM, with1024QAM),
            2, "With 1024QAM");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function queries the EVM Limits DMRS.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR pi2BPSK/Queries the limit specified by 3GPP for the EVM of all PUSCH DMRS
/// HIPAR pi2BPSK/resource elements with a PI/2 BPSK modulation.
/// HIPAR qpsk/Queries the limit specified by 3GPP for the EVM of all PUSCH DMRS
/// HIPAR qpsk/resource elements with a QPSK modulation.
/// HIPAR 16qam/Queries the limit specified by 3GPP for the EVM of all PUSCH DMRS
/// HIPAR 16qam/resource elements with a 16QAM modulation.
/// HIPAR 64qam/Queries the limit specified by 3GPP for the EVM of all PUSCH DMRS
/// HIPAR 64qam/resource elements with a 64QAM modulation.
/// HIPAR 256qam/Queries the limit specified by 3GPP for the EVM of all PUSCH DMRS
/// HIPAR 256qam/resource elements with a 256QAM modulation.
ViStatus _VI_FUNC rsspecan_Query5GNRULEVMLimitsPUSCHDMRS(ViSession instrSession,
                                                         ViInt32*  pi2BPSK,
                                                         ViInt32*  qpsk,
                                                         ViInt32*  qam16,
                                                         ViInt32*  qam64,
                                                         ViInt32*  qam256)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_GetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_5G_NR_UL_EVM_LIMITS_PUSCH_DMRS_PI2_BPSK, pi2BPSK),
            2, "pi 2 BPSK");

    viCheckParm(rsspecan_GetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_5G_NR_UL_EVM_LIMITS_PUSCH_DMRS_QPSK, qpsk),
            3, "QPSK");

    viCheckParm(rsspecan_GetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_5G_NR_UL_EVM_LIMITS_PUSCH_DMRS_16QAM, qam16),
            4, "16 QAM");

    viCheckParm(rsspecan_GetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_5G_NR_UL_EVM_LIMITS_PUSCH_DMRS_64QAM, qam64),
            5, "64 QAM");

    viCheckParm(rsspecan_GetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_5G_NR_UL_EVM_LIMITS_PUSCH_DMRS_256QAM, qam256),
            6, "256 QAM");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the limit specified by 3GPP for the EVM of all PUSCH DMRS
/// HIFN  resource elements with a 1024QAM modulation.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR 1024qam/Queries the limit specified by 3GPP for the EVM of all PUSCH DMRS
/// HIPAR 1024qam/resource elements with a 1024QAM modulation.
ViStatus _VI_FUNC rsspecan_Query5GNRULEVMLimitsPUSCHDMRS1024QAM(
                            ViSession instrSession,
                            ViInt32*  qam1024)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_GetAttributeViInt32(instrSession, NULL, RSSPECAN_ATTR_5G_NR_UL_EVM_LIMITS_PUSCH_DMRS_1024QAM, qam1024),
            2, "1024 QAM");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the limit specified by 3GPP for the EVM of all PUCCH DMRS
/// HIFN  resource elements.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR dmrs/Queries the limit specified by 3GPP for the EVM of all PUCCH DMRS
/// HIPAR dmrs/resource elements.
ViStatus _VI_FUNC rsspecan_Query5GNRULEVMLimitsPUCCHDMRS(ViSession instrSession,
                                                         ViInt32*  dmrs)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_GetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_5G_NR_UL_EVM_LIMITS_PUCCH_DMRS, dmrs),
            2, "DM-RS");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the limit specified by 3GPP for the EVM of all PUCCH resource
/// HIFN  elements.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR resourceElements/Queries the limit specified by 3GPP for the EVM of all PUCCH resource
/// HIPAR resourceElements/elements.
ViStatus _VI_FUNC rsspecan_Query5GNRULEVMLimitsPUCCHResourceElements(
                            ViSession instrSession,
                            ViInt32*  resourceElements)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_GetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_5G_NR_UL_EVM_LIMITS_PUCCH_RESOURCE_ELEMENTS, resourceElements),
            2, "Resource Elements");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the limit specified by 3GPP for the EVM of all PDSCH resource
/// HIFN  elements with a PI/2 BPSK modulation.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR pi2BPSK/Queries the limit specified by 3GPP for the EVM of all PDSCH resource
/// HIPAR pi2BPSK/elements with a PI/2 BPSK modulation.
ViStatus _VI_FUNC rsspecan_Query5GNRULEVMLimitsPDSCHPi2BPSK(ViSession instrSession,
                                                            ViInt32*  pi2BPSK)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_GetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_5G_NR_UL_EVM_LIMITS_PDSCH_PI2_BPSK, pi2BPSK),
            2, "pi 2 BPSK");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the limits and limit check results for all numerical results
/// HIFN  that evaluate 3GPP limits.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR 3gpp/Queries the limits and limit check results for all numerical results
/// HIPAR 3gpp/that evaluate 3GPP limits.
ViStatus _VI_FUNC rsspecan_Query5GNRULEVMLimits3GPP(ViSession instrSession,
                                                    ViInt32   bufferSize,
                                                    ViChar    gpp3[])
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_GetAttributeViString(instrSession, NULL, RSSPECAN_ATTR_5G_NR_UL_EVM_LIMITS_3GPP,
            bufferSize, gpp3),
            3, "3 GPP");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function configures the analysis view.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR subwindow/This control selects the measurement subwindow.
/// HIPAR carrier/Assigns a specific component carrier to a view.
/// HIPAR frame/Assigns a specific frame to a view.
ViStatus _VI_FUNC rsspecan_Configure5GNRULAnalysisView(ViSession instrSession,
                                                       ViInt32   subwindow,
                                                       ViInt32   carrier,
                                                       ViInt32   frame)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_VIEW_CARRIER,
            "View", subwindow - 1, RS_REPCAP_BUF_SIZE, repCap));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_VIEW_CARRIER, carrier),
            3, "Carrier");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_VIEW_FRAME, frame),
            4, "Frame");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function configures the View Window.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR subwindow/This control selects the measurement subwindow.
/// HIPAR coupling/Couples or decouples result display tabs (subwindows).
ViStatus _VI_FUNC rsspecan_Configure5GNRULAnalysisViewCoupling(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   subwindow,
                            ViBoolean coupling)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_subw[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_VIEW_WINDOW_COUPLING,
            "Window", window - 1, RS_REPCAP_BUF_SIZE, repCap));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_VIEW_WINDOW_COUPLING,
            "View", subwindow - 1, RS_REPCAP_BUF_SIZE, repCap_subw));

    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_subw, NULL);

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, repCap, RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_VIEW_WINDOW_COUPLING, coupling),
            4, "Coupling");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Sets the focus on the selected result display window.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR subwindow/This control selects the measurement subwindow.
ViStatus _VI_FUNC rsspecan_Configure5GNRULAnalysisViewSelectResultWindow(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   subwindow)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_subw[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_VIEW_SELECT_RESULT_WINDOW,
            "Window", window - 1, RS_REPCAP_BUF_SIZE, repCap));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_VIEW_SELECT_RESULT_WINDOW,
            "View", subwindow - 1, RS_REPCAP_BUF_SIZE, repCap_subw));

    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_subw, NULL);

    checkErr(rsspecan_SetAttributeViString(instrSession, repCap, RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_VIEW_SELECT_RESULT_WINDOW, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function configures the Analysis Frequency Sweep.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR bwForACLR/Selects the bandwidth of the adjacent channel for ACLR measurements.
/// HIPAR bsCategory/Selects the base station category.
/// HIPAR powerClass/Selects the power class of a UE for ACLR measurements.
ViStatus _VI_FUNC rsspecan_Configure5GNRULAnalysisFrequencySweep(
                            ViSession instrSession,
                            ViInt32   bwForACLR,
                            ViInt32   bsCategory,
                            ViInt32   powerClass)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_FREQUENCY_SWEEP_BW_FOR_ACLR, bwForACLR),
            2, "BW For ACLR");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_FREQUENCY_SWEEP_BS_CATEGORY, bsCategory),
            3, "BS Category");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_FREQUENCY_SWEEP_POWER_CLASS, powerClass),
            4, "Power Class");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function selects the size of the DUT for the SEM measurement.
/// HIFN
/// HIFN     Note:
/// HIFN
/// HIFN     - Set deployment range to high:
/// HIFN  RSSPECAN_ATTR_5G_NR_UL_DEPLOYMENT_FREQUENCY_RANGE
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR iff/Selects the size of the DUT for the SEM measurement.
ViStatus _VI_FUNC rsspecan_Configure5GNRULAnalysisFrequencySweepSEMIFF(
                            ViSession instrSession,
                            ViInt32   iff)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, NULL, RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_FREQUENCY_SWEEP_IFF, iff),
            2, "IFF");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function selects the size of the DUT for the ACLR measurement.
/// HIFN
/// HIFN     Note:
/// HIFN
/// HIFN     - Set deployment range to high:
/// HIFN  RSSPECAN_ATTR_5G_NR_UL_DEPLOYMENT_FREQUENCY_RANGE
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR iff/Selects the size of the DUT for the ACLR measurement.
ViStatus _VI_FUNC rsspecan_Configure5GNRULAnalysisFrequencySweepACLRIFF(
                            ViSession instrSession,
                            ViInt32   iff)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, NULL, RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_FREQUENCY_SWEEP_ACLR_IFF, iff),
            2, "IFF");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Selects the limit evaluation mode for ACLR measurements.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR totalLimitPassMode/Selects the limit evaluation mode for ACLR measurements.
ViStatus _VI_FUNC rsspecan_Configure5GNRULAnalysisTotalLimitPassMode(
                            ViSession instrSession,
                            ViInt32   totalLimitPassMode)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, NULL, RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_FREQUENCY_SWEEP_TOTAL_LIMIT_PASS_MODE, totalLimitPassMode),
            2, "Total Limit Pass Mode");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function queries the state of the limit check for the adjacent or
/// HIFN  alternate channels in an ACLR measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channelType/Selects the ACP channel type.
/// HIPAR limitCheckType/Selects the limit check type.
/// HIPAR results/Returns two results. The first is the result for the lower, the second
/// HIPAR results/for the upper adjacent or alternate channel.
ViStatus _VI_FUNC rsspecan_Query5GNRULACPChannelLimitCheckResults(
                            ViSession instrSession,
                            ViInt32   channelType,
                            ViInt32   limitCheckType,
                            ViInt32   results[])
{
    ViStatus           error = VI_SUCCESS;
    ViString           cmd = NULL;
    ViChar*            p2rngString;
    RsCoreAttributePtr attr = NULL;
    ViChar             response[RS_MAX_MESSAGE_BUF_SIZE] = "";
    ViChar*            p2buf;
    ViInt32            res = 0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &RsSpecAn_rng5GNRLimitCheckType, limitCheckType, &p2rngString),
        3, "Limit Check Type");

    switch (channelType)
    {
        case RSSPECAN_VAL_ACP_ADJ:
            attr = &g_RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_FREQUENCY_SWEEP_GET_ADJACENT_ACLR_LIMIT_CHECK_RESULT;
            break;
        case RSSPECAN_VAL_ACP_ALT:
            attr = &g_RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_FREQUENCY_SWEEP_GET_ALTERNATE_ACLR_LIMIT_CHECK_RESULT;
            break;
        default:
            viCheckParm(RsCore_InvalidViInt32Value(instrSession, channelType), 2, "Channel Type");
    }

    checkErr(RsCore_BuildAttrCommandString(instrSession, attr, NULL, &cmd));
    if (limitCheckType != RSSPECAN_VAL_5G_NR_LIMIT_DEFAULT)
        snprintf(cmd, RS_MAX_SHORT_MESSAGE_BUF_SIZE, "%s %s", cmd, p2rngString);

    checkErr(RsCore_QueryAttributeCommand(instrSession, attr, cmd, response));

    p2buf = strtok(response, ",\n\r");
    RsCore_FindRangeTableDiscreteNumericValue(instrSession, &RsSpecAn_rng5GNRLimitResult, p2buf, &res);
    results[0] = res;
    p2buf = strtok(NULL, ",\n\r");
    RsCore_FindRangeTableDiscreteNumericValue(instrSession, &RsSpecAn_rng5GNRLimitResult, p2buf, &res);
    results[1] = res;

Error:
    if (cmd) free(cmd);
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function queries the state of the limit check for the adjacent
/// HIFN  channels in an ACLR measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channelType/Selects the ACP channel type.
/// HIPAR limitCheckType/Selects the limit type.
/// HIPAR results/Returns two results. The first is the result for the lower, the second
/// HIPAR results/for the upper adjacent or alternate channel.
ViStatus _VI_FUNC rsspecan_Query5GNRULACLRLimitCheckResults(ViSession instrSession,
                                                            ViInt32   limitCheckType,
                                                            ViInt32   results[])
{
    ViStatus error = VI_SUCCESS;
    ViAttr   attr = 0;
    ViChar   response[RS_MAX_MESSAGE_BUF_SIZE] = "";
    ViChar*  p2buf;
    ViInt32  res = 0;

    checkErr(RsCore_LockSession(instrSession));

    switch (limitCheckType)
    {
            case RSSPECAN_VAL_5G_NR_LIMIT_ABS:
                attr = RSSPECAN_ATTR_HIDDEN_5G_NR_UL_ANALYSIS_FREQUENCY_SWEEP_GET_ADJACENT_ACLR_LIMIT_CHECK_RESULT_ABSOLUTE;
                break;
            case RSSPECAN_VAL_5G_NR_LIMIT_REL:
                attr = RSSPECAN_ATTR_HIDDEN_5G_NR_UL_ANALYSIS_FREQUENCY_SWEEP_GET_ADJACENT_ACLR_LIMIT_CHECK_RESULT_RELATIVE;
                break;
            default:
                checkErr(RsCore_InvalidViInt32Value(instrSession, limitCheckType));
                break;
    }

    checkErr(rsspecan_GetAttributeViString(instrSession, NULL,
                                                 attr, RS_MAX_MESSAGE_BUF_SIZE,
                                                 response));

    p2buf = strtok(response, ",\n\r");
    RsCore_FindRangeTableDiscreteNumericValue(instrSession, &RsSpecAn_rng5GNRLimitResult, p2buf, &res);
    results[0] = res;
    p2buf = strtok(NULL, ",\n\r");
    RsCore_FindRangeTableDiscreteNumericValue(instrSession, &RsSpecAn_rng5GNRLimitResult, p2buf, &res);
    results[1] = res;

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function queries the state of the power limit check for the gap
/// HIFN  channels in an MC ACLR measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR limitCheckType/Selects the limit type.
/// HIPAR results/Returns two results. The first is the result for the lower, the second
/// HIPAR results/for the upper adjacent channel.
ViStatus _VI_FUNC rsspecan_Query5GNRULACLRPowerLimitCheckResults(
                            ViSession instrSession,
                            ViInt32   limitCheckType,
                            ViInt32   results[])
{
    ViStatus error = VI_SUCCESS;
    ViAttr   attr = 0;
    ViChar   response[RS_MAX_MESSAGE_BUF_SIZE] = "";
    ViChar*  p2buf;
    ViInt32  res = 0;

    checkErr(RsCore_LockSession(instrSession));

    switch (limitCheckType)
    {
        case RSSPECAN_VAL_5G_NR_LIMIT_ABS:
            attr = RSSPECAN_ATTR_HIDDEN_5G_NR_UL_ANALYSIS_FREQUENCY_SWEEP_GET_ACLR_POWER_LIMIT_CHECK_RESULT_ABSOLUTE;
            break;
        case RSSPECAN_VAL_5G_NR_LIMIT_REL:
            attr = RSSPECAN_ATTR_HIDDEN_5G_NR_UL_ANALYSIS_FREQUENCY_SWEEP_GET_ACLR_POWER_LIMIT_CHECK_RESULT_RELATIVE;
            break;
        case RSSPECAN_VAL_5G_NR_LIMIT_DEFAULT:
            attr = RSSPECAN_ATTR_HIDDEN_5G_NR_UL_ANALYSIS_FREQUENCY_SWEEP_GET_ACLR_POWER_LIMIT_CHECK_RESULT;
            break;
        default:
            break;
    }

    checkErr(rsspecan_GetAttributeViString(instrSession, NULL,
                                                 attr, RS_MAX_MESSAGE_BUF_SIZE,
                                                 response));

    p2buf = strtok(response, ",\n\r");
    RsCore_FindRangeTableDiscreteNumericValue(instrSession, &RsSpecAn_rng5GNRLimitResult, p2buf, &res);
    results[0] = res;
    p2buf = strtok(NULL, ",\n\r");
    RsCore_FindRangeTableDiscreteNumericValue(instrSession, &RsSpecAn_rng5GNRLimitResult, p2buf, &res);
    results[1] = res;

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function queries the state of the  limit check for the gap
/// HIFN  channels in an MC ACLR measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR limitCheckType/Selects the limit type.
/// HIPAR results/Returns two results. The first is the result for the lower, the second
/// HIPAR results/for the upper adjacent channel.
ViStatus _VI_FUNC rsspecan_Query5GNRULCACLRLimitCheckResults(ViSession instrSession,
                                                             ViInt32   limitCheckType,
                                                             ViInt32   results[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   response[RS_MAX_MESSAGE_BUF_SIZE] = "";
    ViChar*  p2buf;
    ViAttr   attr = 0;
    ViInt32  res = 0;

    checkErr(RsCore_LockSession(instrSession));

    switch (limitCheckType)
    {
        case RSSPECAN_VAL_5G_NR_LIMIT_ABS:
            attr = RSSPECAN_ATTR_HIDDEN_5G_NR_UL_ANALYSIS_FREQUENCY_SWEEP_GET_CACLR_LIMIT_CHECK_RESULT_ABSOLUTE;
            break;
        case RSSPECAN_VAL_5G_NR_LIMIT_REL:
            attr = RSSPECAN_ATTR_HIDDEN_5G_NR_UL_ANALYSIS_FREQUENCY_SWEEP_GET_CACLR_LIMIT_CHECK_RESULT_RELATIVE;
            break;
        case RSSPECAN_VAL_5G_NR_LIMIT_DEFAULT:
            attr = RSSPECAN_ATTR_HIDDEN_5G_NR_UL_ANALYSIS_FREQUENCY_SWEEP_GET_CACLR_LIMIT_CHECK_RESULT;
            break;
        default:
            break;
    }

    checkErr(rsspecan_GetAttributeViString(instrSession, NULL,
                                                 attr, RS_MAX_MESSAGE_BUF_SIZE,
                                                 response));

    p2buf = strtok(response, ",\n\r");
    RsCore_FindRangeTableDiscreteNumericValue(instrSession, &RsSpecAn_rng5GNRLimitResult, p2buf, &res);
    results[0] = res;
    p2buf = strtok(NULL, ",\n\r");
    RsCore_FindRangeTableDiscreteNumericValue(instrSession, &RsSpecAn_rng5GNRLimitResult, p2buf, &res);
    results[1] = res;

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Turns an event filter for combined measurement results on and off.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR eventFilter/Turns an event filter for combined measurement results on and off.
ViStatus _VI_FUNC rsspecan_Configure5GNRULCombinedMeasurementEventFilter(
                            ViSession instrSession,
                            ViBoolean eventFilter)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, NULL, RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_COMBINED_MEASUREMENT_EVENT_FILTER, eventFilter),
            2, "Event Filter");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Removes all event filters.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsspecan_Reset5GNRULCombinedMeasurementEventFilter(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsspecan_SetAttributeViString(instrSession, NULL, RSSPECAN_ATTR_5G_NR_UL_ANALYSIS_COMBINED_MEASUREMENT_EVENT_FILTER_RESET, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Defines a filter condition for the event filter in combined
/// HIFN  measurements.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR result/Selects the result that you want to define a filter for.
/// HIPAR condition/Result condition.
/// HIPAR value/Value of numeric result. Used only for numeric conditions
/// HIPAR value/Greater/Lower than or equal.
ViStatus _VI_FUNC rsspecan_Configure5GNRULCombinedMeasurementEventFilterParameters(
                            ViSession instrSession,
                            ViInt32   result,
                            ViInt32   condition,
                            ViReal64  value)
{
    ViStatus error = VI_SUCCESS;
    ViChar   param[RS_PARAM_BUF_SIZE] = "";
    ViChar*  resultCmd;
    ViChar*  conditionCmd;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &RsSpecAn_rng5GNREventFilterParamResult, result, &resultCmd), 2, "Result");
    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &RsSpecAn_rngNrEventCondition, condition, &conditionCmd), 3, "Condition");

    if (result <= RSSPECAN_VAL_5G_NR_RESULT_OVLD)
    {
        viCheckParm(RsCore_InvalidViInt32Range(instrSession, condition, RSSPECAN_VAL_NR_EVENT_CONDITION_GTEQUAL, RSSPECAN_VAL_NR_EVENT_CONDITION_LTEQUAL),
                3, "Condition");

        snprintf (param, RS_PARAM_BUF_SIZE, "%s,%s,%.12lf", resultCmd, conditionCmd, value);
    }
    else
    {
        viCheckParm(RsCore_InvalidViInt32Range(instrSession, condition, RSSPECAN_VAL_NR_EVENT_CONDITION_FAILED, RSSPECAN_VAL_NR_EVENT_CONDITION_PASSED),
                3, "Condition");

        snprintf (param, RS_PARAM_BUF_SIZE, "%s,%s", resultCmd, conditionCmd);
    }

    checkErr(rsspecan_SetAttributeViString(instrSession, NULL, RSSPECAN_ATTR_HIDDEN_5G_NR_UL_ANALYSIS_COMBINED_MEASUREMENT_EVENT_FILTER_PARAMETERS, param));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Turns an event filter on and off.
/// HIFN
/// HIFN     Turning on an event filter adds it to the filter list and removes the
/// HIFN  results from the result summary that do not fulfill the filter
/// HIFN  conditions.
/// HIFN
/// HIFN     Turning it off removes it from the list.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR result/Selects the result that you want to define a filter for.
/// HIPAR state/Enable disable result.
ViStatus _VI_FUNC rsspecan_Configure5GNRULCombinedMeasurementEventFilterParametersEnabled(
                            ViSession instrSession,
                            ViInt32   result,
                            ViBoolean state)
{
    ViStatus error = VI_SUCCESS;
    ViChar   param[RS_PARAM_BUF_SIZE] = "";
    ViChar*  resultCmd;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &RsSpecAn_rng5GNREventFilterParamResult, result, &resultCmd), 2, "Result");

    snprintf (param, RS_PARAM_BUF_SIZE, "%s,%s", resultCmd, state ? "ON" : "OFF");

    checkErr(rsspecan_SetAttributeViString(instrSession, NULL, RSSPECAN_ATTR_HIDDEN_5G_NR_UL_ANALYSIS_COMBINED_MEASUREMENT_EVENT_FILTER_PARAMETERS_ENABLED, param));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the results of the ACLR measurement.
/// HIFN     To get a valid result, you have to perform a complete measurement with
/// HIFN  synchronization to the end of the measurement before reading out the
/// HIFN  result. This is only possible for single sweeps.
/// HIFN
/// HIFN     Results for the ACLR measurements:
/// HIFN     Relative power levels of the ACLR channels. The number of return
/// HIFN  values depends on the number of transmission and adjacent channels. The
/// HIFN  order of return values is:
/// HIFN     - <TXChannelPower> is the power of the transmission channel in dBm
/// HIFN     - <LowerAdjChannelPower> is the relative power of the lower adjacent
/// HIFN  channel in dB
/// HIFN     - <UpperAdjChannelPower> is the relative power of the upper adjacent
/// HIFN  channel in dB
/// HIFN     - <1stLowerAltChannelPower> is the relative power of the first lower
/// HIFN  alternate channel in dB
/// HIFN     - <1stUpperAltChannelPower> is the relative power of the first lower
/// HIFN  alternate channel in dB
/// HIFN     (...)
/// HIFN     - <nthLowerAltChannelPower> is the relative power of a subsequent
/// HIFN  lower alternate channel in dB
/// HIFN     - <nthUpperAltChannelPower> is the relative power of a subsequent
/// HIFN  lower alternate channel in dB
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR marker/This control selects marker.
/// HIPAR arraySize/Number of array points requested.
/// HIPAR result/Queries the results of the ACLR measurement.
/// HIPAR returnedValues/Returns the number of Result values.
ViStatus _VI_FUNC rsspecan_Query5GNRULACLRMeasurement(ViSession instrSession,
                                                      ViInt32   window,
                                                      ViInt32   marker,
                                                      ViInt32   arraySize,
                                                      ViReal64  result[],
                                                      ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, window, 0, 16),
            2, "Window");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, marker, 1, 16),
            3, "Marker");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CALC%ld:MARK%ld:FUNC:POW:RES:DET? ACP", window, marker);
    checkErr(RsCore_QueryFloatArrayToUserBuffer(instrSession, cmd, arraySize, result, returnedValues));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  For the capture buffer result display, the method returns one value
/// HIFN  for each I/Q sample in the capture buffer.
/// HIFN     The unit is always dBm.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR arraySize/Number of array points requested.
/// HIPAR results/Returns the selected results.
/// HIPAR returnedValues/Returns the number of returned values.
ViStatus _VI_FUNC rsspecan_Fetch5GNRULAnalysisCaptureBuffer(ViSession instrSession,
                                                            ViInt32   window,
                                                            ViInt32   arraySize,
                                                            ViReal64  results[],
                                                            ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K145"));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, window, 0, 16),
            2, "Window");
    checkErr(rsspecan_dataReadTrace(instrSession, window, "TRACE1", arraySize, results, returnedValues));
    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  For the EVM vs carrier result display, the method  returns one value
/// HIFN  for each subcarrier that has been analyzed.
/// HIFN
/// HIFN     The unit depends on UNIT:EVM (EVMUnit/RSSPECAN_ATTR_5G_NR_EVM_UNIT).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR trace/Selects the trace.
/// HIPAR arraySize/Number of array points requested.
/// HIPAR results/Returns the selected results.
/// HIPAR returnedValues/Returns the number of returned values.
ViStatus _VI_FUNC rsspecan_Fetch5GNRULAnalysisEVMvsCarrier(ViSession instrSession,
                                                           ViInt32   window,
                                                           ViInt32   trace,
                                                           ViInt32   arraySize,
                                                           ViReal64  results[],
                                                           ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K145"));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, window, 0, 16),
            2, "Window");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, trace, 1, 3),
            3, "Trace");

    if (window == 0)
    {
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, ":FORM REAL,32;:TRAC? TRACE%ld", trace);
    }
    else
    {
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, ":FORM REAL,32;:TRAC%ld? TRACE%ld", window, trace);
    }

    checkErr(RsCore_QueryFloatArrayToUserBuffer(instrSession, cmd, arraySize, results, returnedValues));

    checkErr(RsCore_Write(instrSession, ":FORM ASC"));

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  For the EVM vs symbol result display, the method  returns one value
/// HIFN  for each OFDM symbol that has been analyzed.
/// HIFN     The return values depend on the evaluation range:
/// HIFN     - A specific subframe: EVM of that subframe over all slots.
/// HIFN     - A specific slot: EVM of that slot.
/// HIFN
/// HIFN     The unit depends on UNIT:EVM (EVMUnit/RSSPECAN_ATTR_5G_NR_EVM_UNIT).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR arraySize/Number of array points requested.
/// HIPAR results/Returns the selected results.
/// HIPAR returnedValues/Returns the number of returned values.
ViStatus _VI_FUNC rsspecan_Fetch5GNRULAnalysisEVMvsSymbol(ViSession instrSession,
                                                          ViInt32   window,
                                                          ViInt32   arraySize,
                                                          ViReal64  results[],
                                                          ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K145"));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, window, 0, 16),
            2, "Window");

    if (window == 0)
    {
        snprintf (cmd, RS_MAX_MESSAGE_BUF_SIZE, ":FORM REAL,32;:TRAC? TRACE1");
    }
    else
    {
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, ":FORM REAL,32;:TRAC%ld? TRACE1", window);
    }

    checkErr(RsCore_QueryFloatArrayToUserBuffer(instrSession, cmd, arraySize, results, returnedValues));

    checkErr(RsCore_Write(instrSession, ":FORM ASC"));

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  For the EVM vs RB result display, the method returns one value for
/// HIFN  each resource block that has been analyzed.
/// HIFN
/// HIFN     The unit depends on UNIT:EVM (EVMUnit/RSSPECAN_ATTR_5G_NR_EVM_UNIT).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR trace/Selects the trace.
/// HIPAR arraySize/Number of array points requested.
/// HIPAR results/Returns the selected results.
/// HIPAR returnedValues/Returns the number of returned values.
ViStatus _VI_FUNC rsspecan_Fetch5GNRULAnalysisEVMvsRB(ViSession instrSession,
                                                      ViInt32   window,
                                                      ViInt32   trace,
                                                      ViInt32   arraySize,
                                                      ViReal64  results[],
                                                      ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K145"));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, window, 0, 16),
            2, "Window");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, trace, 1, 3),
            3, "Trace");

    if (window == 0)
    {
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, ":FORM REAL,32;:TRAC? TRACE%ld", trace);
    }
    else
    {
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, ":FORM REAL,32;:TRAC%ld? TRACE%ld", window, trace);
    }

    checkErr(RsCore_QueryFloatArrayToUserBuffer(instrSession, cmd, arraySize, results, returnedValues));

    checkErr(RsCore_Write(instrSession, ":FORM ASC"));

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  For the frequency error vs symbol result display, the method returns
/// HIFN  one value for each OFDM symbol that has been analyzed.
/// HIFN
/// HIFN     The return values depend on the evaluation range:
/// HIFN     - A specific subframe: Frequency error of that subframe over all
/// HIFN  slots.
/// HIFN     - A specific slot: Frequency error of that slot.
/// HIFN
/// HIFN     The unit is always Hz.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR arraySize/Number of array points requested.
/// HIPAR results/Returns the selected results.
/// HIPAR returnedValues/Returns the number of returned values.
ViStatus _VI_FUNC rsspecan_Fetch5GNRULAnalysisFrequencyErrorvsSymbol(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   arraySize,
                            ViReal64  results[],
                            ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K145"));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, window, 0, 16),
            2, "Window");

    if (window == 0)
    {
        snprintf (cmd, RS_MAX_MESSAGE_BUF_SIZE, ":FORM REAL,32;:TRAC? TRACE1");
    }
    else
    {
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, ":FORM REAL,32;:TRAC%ld? TRACE1", window);
    }

    checkErr(RsCore_QueryFloatArrayToUserBuffer(instrSession, cmd, arraySize, results, returnedValues));

    checkErr(RsCore_Write(instrSession, ":FORM ASC"));

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  For the power spectrum result display, the method returns one value
/// HIFN  for each trace point.
/// HIFN
/// HIFN     The unit is always dBm/Hz.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR arraySize/Number of array points requested.
/// HIPAR results/Returns the selected results.
/// HIPAR returnedValues/Returns the number of returned values.
ViStatus _VI_FUNC rsspecan_Fetch5GNRULAnalysisPowerSpectrum(ViSession instrSession,
                                                            ViInt32   window,
                                                            ViInt32   arraySize,
                                                            ViReal64  results[],
                                                            ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K145"));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, window, 0, 16),
            2, "Window");

    if (window == 0)
    {
        snprintf (cmd, RS_MAX_MESSAGE_BUF_SIZE, ":FORM REAL,32;:TRAC? TRACE1");
    }
    else
    {
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, ":FORM REAL,32;:TRAC%ld? TRACE1", window);
    }

    checkErr(RsCore_QueryFloatArrayToUserBuffer(instrSession, cmd, arraySize, results, returnedValues));

    checkErr(RsCore_Write(instrSession, ":FORM ASC"));

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Returns one value for each resource block index.
/// HIFN     The unit of the relative inband emission is dB.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR arraySize/Number of array points requested.
/// HIPAR results/Returns the selected results.
/// HIPAR returnedValues/Returns the number of returned values.
ViStatus _VI_FUNC rsspecan_Fetch5GNRULAnalysisInbandEmission(ViSession instrSession,
                                                             ViInt32   window,
                                                             ViInt32   arraySize,
                                                             ViReal64  results[],
                                                             ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K145"));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, window, 0, 16),
            2, "Window");

    if (window == 0)
    {
        snprintf (cmd, RS_MAX_MESSAGE_BUF_SIZE, ":FORM REAL,32;:TRAC? TRACE2");
    }
    else
    {
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, ":FORM REAL,32;:TRAC%ld? TRACE2", window);
    }

    checkErr(RsCore_QueryFloatArrayToUserBuffer(instrSession, cmd, arraySize, results, returnedValues));

    checkErr(RsCore_Write(instrSession, ":FORM ASC"));

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Returns the data points of the upper limit line.
/// HIFN     The unit of the relative inband emission is dB.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR arraySize/Number of array points requested.
/// HIPAR results/Returns the selected results.
/// HIPAR returnedValues/Returns the number of returned values.
ViStatus _VI_FUNC rsspecan_Fetch5GNRULAnalysisInbandEmissionUpperLimitLine(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   arraySize,
                            ViReal64  results[],
                            ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K145"));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, window, 0, 16),
            2, "Window");

    if (window == 0)
    {
        snprintf (cmd, RS_MAX_MESSAGE_BUF_SIZE, ":FORM REAL,32;:TRAC? TRACE3");
    }
    else
    {
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, ":FORM REAL,32;:TRAC%ld? TRACE3", window);
    }

    checkErr(RsCore_QueryFloatArrayToUserBuffer(instrSession, cmd, arraySize, results, returnedValues));

    checkErr(RsCore_Write(instrSession, ":FORM ASC"));

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Returns the relative resource block indices (x-axis values).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR arraySize/Number of array points requested.
/// HIPAR results/Returns the selected results.
/// HIPAR returnedValues/Returns the number of returned values.
ViStatus _VI_FUNC rsspecan_Fetch5GNRULAnalysisInbandEmissionRBIndices(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   arraySize,
                            ViInt32   results[],
                            ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K145"));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, window, 0, 16),
            2, "Window");

    if (window == 0)
    {
        snprintf (cmd, RS_MAX_MESSAGE_BUF_SIZE, ":TRAC? TRACE1");
    }
    else
    {
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, ":TRAC%ld? TRACE1", window);
    }

    checkErr(RsCore_QueryIntegerArrayToUserBuffer(instrSession, cmd, arraySize, results, returnedValues));

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  For the flatness vs carrier result display, the method returns one
/// HIFN  value for each trace point.
/// HIFN
/// HIFN     The unit is always dB.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR trace/Selects the trace.
/// HIPAR arraySize/Number of array points requested.
/// HIPAR results/Returns the selected results.
/// HIPAR returnedValues/Returns the number of returned values.
ViStatus _VI_FUNC rsspecan_Fetch5GNRULAnalysisFlatnessvsCarrier(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   trace,
                            ViInt32   arraySize,
                            ViReal64  results[],
                            ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K145"));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, window, 0, 16),
            2, "Window");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, trace, 1, 3),
            3, "Trace");

    if (window == 0)
    {
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, ":FORM REAL,32;:TRAC? TRACE%ld", trace);
    }
    else
    {
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, ":FORM REAL,32;:TRAC%ld? TRACE%ld", window, trace);
    }

    checkErr(RsCore_QueryFloatArrayToUserBuffer(instrSession, cmd, arraySize, results, returnedValues));

    checkErr(RsCore_Write(instrSession, ":FORM ASC"));

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  For the flatness vs carrier result display, the method returns one
/// HIFN  value for each trace point.
/// HIFN
/// HIFN     The unit is always dB.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR axis/Sets the axis.
/// HIPAR arraySize/Number of array points requested.
/// HIPAR results/Returns the selected results.
/// HIPAR returnedValues/Returns the number of returned values.
ViStatus _VI_FUNC rsspecan_Fetch5GNRULAnalysisCCDF(ViSession instrSession,
                                                   ViInt32   window,
                                                   ViInt32   axis,
                                                   ViInt32   arraySize,
                                                   ViReal64  results[],
                                                   ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K145"));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, window, 0, 16),
            2, "Window");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, axis, 0, 1),
            3, "Axis");

    if (window == 0)
    {
        snprintf (cmd, RS_MAX_MESSAGE_BUF_SIZE, ":FORM REAL,32;:TRAC? TRACE%ld", (axis == RSSPECAN_VAL_AXIS_Y) ? 1 : 2);
    }
    else
    {
        snprintf (cmd, RS_MAX_MESSAGE_BUF_SIZE, ":FORM REAL,32;:TRAC%ld? TRACE%ld", window, (axis == RSSPECAN_VAL_AXIS_Y) ? 1 : 2);
    }

    checkErr(RsCore_QueryFloatArrayToUserBuffer(instrSession, cmd, arraySize, results, returnedValues));

    checkErr(RsCore_Write(instrSession, ":FORM ASC"));

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  For the constellation diagram, the method returns one value pair for
/// HIFN  each constellation.
/// HIFN
/// HIFN     <I[SF0][Sym0][Carrier1]>, <Q[SF0][Sym0][Carrier1]>, ...,
/// HIFN  <I[SF0][Sym0][Carrier(n)]>, <Q[SF0][Sym0][Carrier(n)]>,
/// HIFN     <I[SF0][Sym1][Carrier1]>, <Q[SF0][Sym1][Carrier1]>, ...,
/// HIFN  <I[SF0][Sym1][Carrier(n)]>, <Q[SF0][Sym1][Carrier(n)]>,
/// HIFN     <I[SF0][Sym(n)][Carrier1]>, <Q[SF0][Sym(n)][Carrier1]>, ...,
/// HIFN  <I[SF0][Sym(n)][Carrier(n)]>, <Q[SF0][Sym(n)][Carrier(n)]>,
/// HIFN     <I[SF1][Sym0][Carrier1]>, <Q[SF1][Sym0][Carrier1]>, ...,
/// HIFN  <I[SF1][Sym0][Carrier(n)]>, <Q[SF1][Sym0][Carrier(n)]>,
/// HIFN     <I[SF1][Sym1][Carrier1]>, <Q[SF1][Sym1][Carrier1]>, ...,
/// HIFN  <I[SF1][Sym1][Carrier(n)]>, <Q[SF1][Sym1][Carrier(n)]>,
/// HIFN     <I[SF(n)][Sym(n)][Carrier1]>, <Q[SF(n)][Sym(n)][Carrier1]>, ...,
/// HIFN  <I[SF(n)][Sym(n)][Carrier(n)]>, <Q[SF(n)][Sym(n)][Carrier(n)]>
/// HIFN
/// HIFN     The I and Q values have no unit.
/// HIFN
/// HIFN     The number of return values depends on the constellation selection. By
/// HIFN  default, it returns all resource elements including the DC carrier.
/// HIFN     For each bandwidth part, the command returns the values for all
/// HIFN  resource elements. However, only the resource elements allocated to the
/// HIFN  corresponding bandwidth part have a value. All others are returnd as
/// HIFN  NAN. If you query the constellation for a signal with SS/PBCH and 2
/// HIFN  bandwidth parts, for example, the stream of values would look like
/// HIFN  this:
/// HIFN     1st block of values:
/// HIFN  NAN,NAN,(...),RE[SS/PBCH],RE[SS/PBCH],(...),NAN,NAN,(...)
/// HIFN     2nd block of values:
/// HIFN  NAN,NAN,(...),RE[BWP0],RE[BWP0],(...),NAN,NAN,(...)
/// HIFN     3rd block of values:
/// HIFN  NAN,NAN,(...),RE[BWP1],RE[BWP1],(...),NAN,NAN,(...)
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR arraySize/Number of array points requested.
/// HIPAR constellationI/Returns the Constellation I.
/// HIPAR constellationQ/Returns the Constellation Q.
/// HIPAR returnedValues/Returns the number of returned values.
ViStatus _VI_FUNC rsspecan_Fetch5GNRULAnalysisConstellation(ViSession instrSession,
                                                            ViInt32   window,
                                                            ViInt32   arraySize,
                                                            ViReal64  constellationI[],
                                                            ViReal64  constellationQ[],
                                                            ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K145"));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, window, 0, 16),
            2, "Window");

    if (window == 0)
    {
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, ":FORM REAL,32;:TRAC? TRACE1");
    }
    else
    {
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, ":FORM REAL,32;:TRAC%ld? TRACE1", window);
    }

    checkErr(RsCore_QueryFloatArraysInterleavedToUserBuffer(instrSession, cmd, arraySize, returnedValues, constellationI, constellationQ, NULL, NULL, NULL));

    checkErr(RsCore_Write(instrSession, ":FORM ASC"));

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  For the allocation summary, the method returns NR5GULAllocationSummary
/// HIFN  object  for each line of the table.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR arraySize/Number of array points requested.
/// HIPAR bwPart/Returns the selected results.
/// HIPAR subframe/Queries the subframes.
/// HIPAR slot/Queries the slot.
/// HIPAR allocationID/Queries the Allocation ID.
/// HIPAR numberOfRB/Queries the Modulation.
/// HIPAR relativePower/Queries the Relative Power.
/// HIPAR modulation/Queries the Modulation.
/// HIPAR absolutePower/Queries the Absolute Power.
/// HIPAR evm/Queries the EVM.
/// HIPAR returnedValues/Returns the number of returned values.
ViStatus _VI_FUNC rsspecan_Fetch5GNRULAnalysisAllocationSummary(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   arraySize,
                            ViInt32   bwPart[],
                            ViInt32   subframe[],
                            ViInt32   slot[],
                            ViInt32   allocationID[],
                            ViInt32   numberOfRB[],
                            ViReal64  relativePower[],
                            ViInt32   modulation[],
                            ViReal64  absolutePower[],
                            ViReal64  evm[],
                            ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = ":TRAC? TRACE1";
    ViChar   *response = NULL;
    ViChar   *pbuf;
    ViChar   *pbufEnd;
    ViInt32  idx = 0;
    ViInt32  elements_count;

    checkErr(RsCore_LockSession(instrSession));

    if (window != 0)
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, ":TRAC%ld? TRACE1", window);
        checkErr(RsCore_QueryViStringUnknownLength(instrSession, cmd, &response));

    elements_count = RsCore_GetElementsCount(response, ',') / 9;

    if (arraySize < elements_count)
        viCheckParm(RsCore_InvalidViInt32RangeMin(instrSession, arraySize, elements_count), 3, "Array Size");

    pbuf = response;
    while (idx < (elements_count))
    {
        pbufEnd = strchr(pbuf, ',');
        if (pbufEnd == NULL)
        {
            checkErr(RS_ERROR_UNEXPECTED_RESPONSE);
        }
        *pbufEnd = 0;
        if (!isNullOrEmpty(pbuf))
            bwPart[idx] = RsCore_Convert_String_To_ViInt32(pbuf);
        else
            bwPart[idx] = RS_VAL_NAN_VI_INT32;
        pbuf = pbufEnd + 1;

        pbufEnd = strchr(pbuf, ',');
        if (pbufEnd == NULL)
        {
            checkErr(RS_ERROR_UNEXPECTED_RESPONSE);
        }
        *pbufEnd = 0;
        if (!isNullOrEmpty(pbuf))
            subframe[idx] = RsCore_Convert_String_To_ViInt32(pbuf);
        else
            subframe[idx] = RS_VAL_NAN_VI_INT32;
        pbuf = pbufEnd + 1;

        pbufEnd = strchr(pbuf, ',');
        if (pbufEnd == NULL)
        {
            checkErr(RS_ERROR_UNEXPECTED_RESPONSE);
        }
        *pbufEnd = 0;
        if (!isNullOrEmpty(pbuf))
            slot[idx] = RsCore_Convert_String_To_ViInt32(pbuf);
        else
            slot[idx] = RS_VAL_NAN_VI_INT32;
        pbuf = pbufEnd + 1;

        pbufEnd = strchr(pbuf, ',');
        if (pbufEnd == NULL)
        {
            checkErr(RS_ERROR_UNEXPECTED_RESPONSE);
        }
        *pbufEnd = 0;
        if (!isNullOrEmpty(pbuf))
            allocationID[idx] = RsCore_Convert_String_To_ViInt32(pbuf);
        else
            allocationID[idx] = RS_VAL_NAN_VI_INT32;
        pbuf = pbufEnd + 1;

        pbufEnd = strchr(pbuf, ',');
        if (pbufEnd == NULL)
        {
            checkErr(RS_ERROR_UNEXPECTED_RESPONSE);
        }
        *pbufEnd = 0;
        if (!isNullOrEmpty(pbuf))
            numberOfRB[idx] = RsCore_Convert_String_To_ViInt32(pbuf);
        else
            numberOfRB[idx] = RS_VAL_NAN_VI_INT32;
        pbuf = pbufEnd + 1;

        pbufEnd = strchr(pbuf, ',');
        if (pbufEnd == NULL)
        {
            checkErr(RS_ERROR_UNEXPECTED_RESPONSE);
        }
        *pbufEnd = 0;
        if (!isNullOrEmpty(pbuf))
            relativePower[idx] = RsCore_Convert_String_To_ViReal64(pbuf);
        else
            relativePower[idx] = RS_VAL_NAN_VI_REAL64;
        pbuf = pbufEnd + 1;

        pbufEnd = strchr(pbuf, ',');
        if (pbufEnd == NULL)
        {
            checkErr(RS_ERROR_UNEXPECTED_RESPONSE);
        }
        *pbufEnd = 0;
        if (!isNullOrEmpty(pbuf))
            modulation[idx] = RsCore_Convert_String_To_ViInt32(pbuf);
        else
            modulation[idx] = RS_VAL_NAN_VI_INT32;
        pbuf = pbufEnd + 1;

        pbufEnd = strchr(pbuf, ',');
        if (pbufEnd == NULL)
        {
            checkErr(RS_ERROR_UNEXPECTED_RESPONSE);
        }
        *pbufEnd = 0;
        if (!isNullOrEmpty(pbuf))
            absolutePower[idx] = RsCore_Convert_String_To_ViReal64(pbuf);
        else
            absolutePower[idx] = RS_VAL_NAN_VI_REAL64;
        pbuf = pbufEnd + 1;

        pbufEnd = strchr(pbuf, ',');
        if (pbufEnd == NULL)
        {
            evm[idx] = RsCore_Convert_String_To_ViReal64(pbuf);
        }
        else
        {
            *pbufEnd = 0;
            if (!isNullOrEmpty(pbuf))
                evm[idx] = RsCore_Convert_String_To_ViReal64(pbuf);
            else
                evm[idx] = RS_VAL_NAN_VI_REAL64;
            pbuf = pbufEnd + 1;
        }
        idx++;
    }

    *returnedValues = idx - 1;

Error:
    if (response) free(response);
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  For the bitstream result display, the method returns five values and
/// HIFN  the bitstream for each line of the table.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR format/Sets the bitstream format.
/// HIPAR arraySize/Sets the array size parameter.
/// HIPAR results/Returns the selected results.
/// HIPAR returnedValues/Returns the number of returned values.
ViStatus _VI_FUNC rsspecan_Fetch5GNRULAnalysisBitstream(ViSession instrSession,
                                                        ViInt32   window,
                                                        ViInt32   format,
                                                        ViInt32   arraySize,
                                                        ViChar    results[],
                                                        ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K145"));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, window, 0, 16),
            2, "Window");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, format, RSSPECAN_VAL_NR5G_BITSTREAM_FULL, RSSPECAN_VAL_NR5G_BITSTREAM_FULL_INFO),
            3, "Format");

    if (window == 0)
    {
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, ":TRAC? TRACE%ld", format);
    }
    else
    {
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, ":TRAC%ld? TRACE%ld", window, format);
    }

    checkErr(RsCore_QueryViStringUnknownLengthToUserBuffer(instrSession, cmd, arraySize, results, returnedValues));

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  For the EVM vs symbol x carrier, the method returns one value for each
/// HIFN  resource element.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR arraySize/Number of array points requested.
/// HIPAR results/Returns the selected results.
/// HIPAR returnedValues/Returns the number of returned values.
ViStatus _VI_FUNC rsspecan_Fetch5GNRULAnalysisEVMvsSymbolxCarrier(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   arraySize,
                            ViReal64  results[],
                            ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K145"));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, window, 0, 16),
            2, "Window");

    if (window == 0)
    {
        snprintf (cmd, RS_MAX_MESSAGE_BUF_SIZE, ":FORM REAL,32;:TRAC? TRACE1");
    }
    else
    {
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, ":FORM REAL,32;:TRAC%ld? TRACE1", window);
    }

    checkErr(RsCore_QueryFloatArrayToUserBuffer(instrSession, cmd, arraySize, results, returnedValues));

    checkErr(RsCore_Write(instrSession, ":FORM ASC"));

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  For the power vs symbol x carrier, the method returns one value for
/// HIFN  each resource element.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR arraySize/Number of array points requested.
/// HIPAR results/Returns the selected results.
/// HIPAR returnedValues/Returns the number of returned values.
ViStatus _VI_FUNC rsspecan_Fetch5GNRULAnalysisPowervsSymbolxCarrier(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   arraySize,
                            ViReal64  results[],
                            ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K145"));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, window, 0, 16),
            2, "Window");

    if (window == 0)
    {
        snprintf (cmd, RS_MAX_MESSAGE_BUF_SIZE, ":FORM REAL,32;:TRAC? TRACE1");
    }
    else
    {
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, ":FORM REAL,32;:TRAC%ld? TRACE1", window);
    }

    checkErr(RsCore_QueryFloatArrayToUserBuffer(instrSession, cmd, arraySize, results, returnedValues));

    checkErr(RsCore_Write(instrSession, ":FORM ASC"));

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  For the allocation ID vs symbol x carrier, the method returns one
/// HIFN  value for each resource element.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR arraySize/Number of array points requested.
/// HIPAR results/For the allocation ID vs symbol x carrier, the method returns one
/// HIPAR results/value for each resource element.
/// HIPAR returnedValues/Returns the number of returned values.
ViStatus _VI_FUNC rsspecan_Fetch5GNRULAnalysisAllocationIDvsSymbolxCarrier(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   arraySize,
                            ViInt32   results[],
                            ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K145"));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, window, 0, 16),
            2, "Window");

    if (window == 0)
    {
        snprintf (cmd, RS_MAX_MESSAGE_BUF_SIZE, ":TRAC? TRACE2");
    }
    else
    {
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, ":TRAC%ld? TRACE2", window);
    }

    checkErr(RsCore_QueryIntegerArrayToUserBuffer(instrSession, cmd, arraySize, results, returnedValues));
    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  For the ACLR result display.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR arraySize/Number of array points requested.
/// HIPAR results/Returns the selected results.
/// HIPAR returnedValues/Returns the number of returned values.
ViStatus _VI_FUNC rsspecan_Fetch5GNRULAnalysisACLR(ViSession instrSession,
                                                   ViInt32   window,
                                                   ViInt32   arraySize,
                                                   ViReal64  results[],
                                                   ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K145"));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, window, 0, 16),
            2, "Window");

    if (window == 0)
    {
        snprintf (cmd, RS_MAX_MESSAGE_BUF_SIZE, ":FORM REAL,32;:TRAC? TRACE1");
    }
    else
    {
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, ":FORM REAL,32;:TRAC%ld? TRACE1", window);
    }

    checkErr(RsCore_QueryFloatArrayToUserBuffer(instrSession, cmd, arraySize, results, returnedValues));

    checkErr(RsCore_Write(instrSession, ":FORM ASC"));

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  For the SEM  result display.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR arraySize/Number of array points requested.
/// HIPAR results/Returns the selected results.
/// HIPAR returnedValues/Returns the number of returned values.
ViStatus _VI_FUNC rsspecan_Fetch5GNRULAnalysisSEM(ViSession instrSession,
                                                  ViInt32   window,
                                                  ViInt32   arraySize,
                                                  ViReal64  results[],
                                                  ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K145"));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, window, 0, 16),
            2, "Window");

    if (window == 0)
    {
        snprintf (cmd, RS_MAX_MESSAGE_BUF_SIZE, ":FORM REAL,32;:TRAC? TRACE1");
    }
    else
    {
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, ":FORM REAL,32;:TRAC%ld? TRACE1", window);
    }

    checkErr(RsCore_QueryFloatArrayToUserBuffer(instrSession, cmd, arraySize, results, returnedValues));

    checkErr(RsCore_Write(instrSession, ":FORM ASC"));

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Returns the contents of the SEM table.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR arraySize/Number of array points requested.
/// HIPAR index/Returns the Index.
/// HIPAR startFrequency/Returns the Start Frequency.
/// HIPAR stopFrequency/Returns the Start Frequency.
/// HIPAR rbw/Returns the RBW.
/// HIPAR limitFailFreq/Returns the Limit Fail Frequency.
/// HIPAR absolutePower/Returns the Absolute Power.
/// HIPAR relativePower/Returns the Relative Power.
/// HIPAR limitDistance/Returns the Limit Distance.
/// HIPAR limitCheckResult/Returns the Limit Check Result.
/// HIPAR reserved1/Reserved 1.
/// HIPAR reserved2/Reserved 2.
/// HIPAR returnedValues/Returns the number of returned values.
ViStatus _VI_FUNC rsspecan_Fetch5GNRULAnalysisSEMTable(ViSession instrSession,
                                                       ViInt32   window,
                                                       ViInt32   arraySize,
                                                       ViInt32   index[],
                                                       ViReal64  startFrequency[],
                                                       ViReal64  stopFrequency[],
                                                       ViReal64  rbw[],
                                                       ViReal64  limitFailFreq[],
                                                       ViReal64  absolutePower[],
                                                       ViReal64  relativePower[],
                                                       ViReal64  limitDistance[],
                                                       ViInt32   limitCheckResult[],
                                                       ViReal64  reserved1[],
                                                       ViReal64  reserved2[],
                                                       ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViChar   *pbuffer = NULL;
    ViChar   *pstring_value;
    ViInt32  i = 0;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K145"));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, window, 0, 16),
            2, "Window");

    if (window == 0)
    {
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "TRACE? LIST");
    }
    else
    {
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "TRACE%ld? LIST", window);
    }

    checkErr(RsCore_QueryViStringUnknownLength(instrSession, cmd, &pbuffer));

    pstring_value = strtok(pbuffer, ",");

    while (pstring_value != NULL && i < arraySize)
    {
        index[i] = RsCore_Convert_String_To_ViInt32(pstring_value);

        pstring_value = strtok(NULL, ",\n");
        startFrequency[i] = RsCore_Convert_String_To_ViReal64(pstring_value);

        pstring_value = strtok(NULL, ",\n");
        stopFrequency[i++] = RsCore_Convert_String_To_ViReal64(pstring_value);

        pstring_value = strtok(NULL, ",\n");
        rbw[i++] = RsCore_Convert_String_To_ViReal64(pstring_value);

        pstring_value = strtok(NULL, ",\n");
        limitFailFreq[i++] = RsCore_Convert_String_To_ViReal64(pstring_value);

        pstring_value = strtok(NULL, ",\n");
        absolutePower[i++] = RsCore_Convert_String_To_ViReal64(pstring_value);

        pstring_value = strtok(NULL, ",\n");
        relativePower[i++] = RsCore_Convert_String_To_ViReal64(pstring_value);

        pstring_value = strtok(NULL, ",\n");
        limitDistance[i++] = RsCore_Convert_String_To_ViReal64(pstring_value);

        pstring_value = strtok(NULL, ",\n");
        limitCheckResult[i++] = RsCore_Convert_String_To_ViInt32(pstring_value);

        pstring_value = strtok(NULL, ",\n");
        reserved1[i++] = RsCore_Convert_String_To_ViReal64(pstring_value);

        pstring_value = strtok(NULL, ",\n");
        reserved2[i++] = RsCore_Convert_String_To_ViReal64(pstring_value);

        pstring_value = strtok(NULL, ",\n");
    }

    if (*returnedValues) *returnedValues = i;

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    if (pbuffer) free(pbuffer);
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  For the on/off power measurement, the number and type of return values
/// HIFN  depend on the parameter 'trace'.
/// HIFN     The unit is always dBm.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR trace/Selects the trace.
/// HIPAR arraySize/Number of array points requested.
/// HIPAR results/Returns the results.
/// HIPAR returnedValues/Returns the number of returned values.
ViStatus _VI_FUNC rsspecan_Fetch5GNRULAnalysisCombinedMeasurement(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   trace,
                            ViInt32   arraySize,
                            ViReal64  results[],
                            ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;
    ViChar   trc[RS_PARAM_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf (trc, RS_PARAM_BUF_SIZE, "TRACE%ld", trace);

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, window, 0, 16),
            2, "Window");
    checkErr(rsspecan_dataReadTrace(instrSession, window, trc, arraySize, results, returnedValues));
    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}
