/*****************************************************************************
 *  Rohde&Schwarz Spectrum Analyzer instrument driver
 *  WLAN option K90/K91
 *
 *  Original Release: September 2005
 *  By: Martin Koutny (instrument driver core)
 *      Helena Sucha (instrument specific implementation)
 *      Jiri Kominek (DrBase repository & tools)
 *
 *  Should you have any technical questions please contact the hotline of
 *  Rohde & Schwarz Vertriebs-GmbH Rohde & Schwarz Support Center
 *
 *  e-mail: CustomerSupport@rohde-schwarz.com
 *
 *****************************************************************************/

#include "rsspecan.h"

/*  Array definitions   */
static ViString powerMeasurementArr[] = {"ACP", "CPOW", "MCAC", "OBAN", "CN", "CN0", "AOB", "PPOW", NULL};
static ViString ACPMeas[] = {"CURR", "MAXH"};

/*****************************************************************************
 * Function: Set WLAN Mode
 * Purpose:  This function selects the WLAN analyzer mode.
*****************************************************************************/
ViStatus _VI_FUNC rsspecan_WlanSetWlanMode(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_WLAN_MODE, ""));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the expected input signal.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR standard/Sets the standard.
/// HIPAR centerFrequency/Specifies the center frequency of the signal to be measured.
ViStatus _VI_FUNC rsspecan_ConfigureWLANSignal(ViSession instrSession,
                                               ViInt32   standard,
                                               ViReal64  centerFrequency)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_WLAN_STAN, standard),
            2, "Standard");

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_FREQUENCY_CENTER, centerFrequency),
            3, "Center Frequency");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function configures the expected input signal.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR toleranceLimit/Sets the tolerance limit
ViStatus _VI_FUNC rsspecan_ConfigureWLANToleranceLimit(ViSession instrSession,
                                                       ViInt32   toleranceLimit)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_WLAN_TOLERANCE_LIMIT, toleranceLimit),
            2, "Tolerance Limit");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the standard version to be used for the EVM measurement.
/// HIFN     This function is only available for IEEE 802.11b and g (DSSS)
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR standardVersion/This command defines or queries the standard version to be used for
/// HIPAR standardVersion/the EVM measurement.
ViStatus _VI_FUNC rsspecan_ConfigureWLANStandardVersionErrorVectorMagnitude(
                            ViSession instrSession,
                            ViInt32   standardVersion)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, NULL, RSSPECAN_ATTR_WLAN_STANDARD_VERSION_ERROR_VECTOR_MAGNITUDE, standardVersion),
            2, "Standard Version");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures radio frequency input.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR inputCoupling/The RF input of the R&S FSW can be coupled by alternating current (AC)
/// HIPAR inputCoupling/or direct current (DC).
/// HIPAR impedance/This control selects the nominal input impedance.
/// HIPAR highpassFilter/Activates an additional high pass filter for RF input signals from
/// HIPAR highpassFilter/1...3 GHz.
/// HIPAR YIGPreselector/This control turns the YIG-preselector on and off.
ViStatus _VI_FUNC rsspecan_ConfigureWLANInputSource(ViSession instrSession,
                                                    ViInt32   inputCoupling,
                                                    ViInt32   impedance,
                                                    ViBoolean highpassFilter,
                                                    ViBoolean YIGPreselector)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_AMPL_RF_COUPLING, inputCoupling),
            2, "Input Coupling");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_INPUT_IMPEDANCE, impedance),
            3, "Impedance");

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_AMPL_HIGHPASS_FILTER, highpassFilter),
            4, "Highpass Filter");

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_AMPL_YIG_FILTER, YIGPreselector),
            5, "YIG Preselector");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function sets the signal symbol field.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR useSignalFieldContent/Sets the signal symbol field.
ViStatus _VI_FUNC rsspecan_ConfigureWlanUseSignalFieldContent(
                            ViSession instrSession,
                            ViBoolean useSignalFieldContent)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_WLAN_SIGS, useSignalFieldContent),
            2, "Use Signal Field Content");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the trigger parameters.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR mode/Sets the trigger mode.
/// HIPAR offset/Defines the length of the trigger delay. A negative delay time can be set.
/// HIPAR autoLevel/Specifies whether or not an automatic power trigger level
/// HIPAR autoLevel/calculation is performed before each main measurement.
/// HIPAR powerLevel/Specifies the level of the input signal for which triggering will occur.
/// HIPAR extTrigLevel/Specifies the level of the external trigger input for which triggering will occur.
ViStatus _VI_FUNC rsspecan_ConfWlanTrigger(ViSession instrSession,
                                           ViInt32   mode,
                                           ViReal64  offset,
                                           ViBoolean autoLevel,
                                           ViReal64  powerLevel,
                                           ViReal64  extTrigLevel)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_WLAN_TRIG_MODE, mode),
            2, "Mode");
    switch (mode)
    {
    case RSSPECAN_VAL_TRIG_MODE_IMM:
    case RSSPECAN_VAL_TRIG_MODE_TUN:
        break;
    case RSSPECAN_VAL_TRIG_MODE_EXT:
        viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_TRIGGER_DELAY, offset),
                3, "Offset");

        viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_EXTERNAL_TRIGGER_LEVEL, extTrigLevel),
                6, "Ext. Trigger Level");
        break;
    case RSSPECAN_VAL_TRIG_MODE_POW:
        viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_TRIGGER_DELAY, offset),
                3, "Offset");

        viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_WLAN_TRIG_POW_LEV_AUTO, autoLevel),
                4, "Auto Level");
        if (!autoLevel)
        {
            if (rsspecan_IsFSV(instrSession))
            {
                viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_WLAN_TRIG_POW_LEV, powerLevel),
                        5, "Power Level");
            }
            else
            {
                viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_TRIGGER_RFP_LEVEL, powerLevel),
                        5, "Power Level");
            }
        }

        break;
    case RSSPECAN_VAL_TRIG_MODE_PSEN:
        viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_TRIGGER_DELAY, offset),
                3, "Offset");
        break;
    case RSSPECAN_VAL_TRIG_MODE_RFP:
        viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_TRIGGER_DELAY, offset),
                3, "Offset");

        viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_WLAN_TRIG_POW_LEV_AUTO, autoLevel),
                4, "Auto Level");
        if (!autoLevel)
        {
            viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_TRIGGER_RFP_LEVEL, powerLevel),
                    5, "Power Level");
        }

        break;
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the trigger parameters.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR gating/This parameter switches on/off the gating.
/// HIPAR mode/Sets the way for delay and length specification.
/// HIPAR delay/Defines the gate delay in the capture buffer in time  units or samples.
/// HIPAR length/Specifies defines the gate time in the capture buffer in time units or samples.
/// HIPAR linkGateAndMarker/Links together the movement of the gating lines and the capture buffer marker.
ViStatus _VI_FUNC rsspecan_ConfWlanGate(ViSession instrSession,
                                        ViBoolean gating,
                                        ViInt32   mode,
                                        ViReal64  delay,
                                        ViReal64  length,
                                        ViBoolean linkGateAndMarker)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_WLAN_EGAT, gating),
            2, "Gating");
    if (gating)
    {
        viCheckParm(RsCore_InvalidViInt32Range(instrSession, mode, 0, 1),
                3, "Mode");

        switch (mode)
        {
        case RSSPECAN_VAL_WLAN_GATE_TIME:
            viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_WLAN_EGAT_HOLD_TIME, delay),
                    4, "Delay");

            viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_WLAN_EGAT_LENG_TIME, length),
                    5, "Length");
            break;
        case RSSPECAN_VAL_WLAN_GATE_SAMPLE:
            viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_WLAN_EGAT_HOLD_SAMP, (ViInt32)delay), 4, "Delay");

            viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_WLAN_EGAT_LENG_SAMP, (ViInt32)length), 5, "Length");
            break;
        }
        viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_WLAN_EGAT_LINK, linkGateAndMarker),
                6, "Link Gate Marker");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the channel.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR channelNo/Sets the input channel.
ViStatus _VI_FUNC rsspecan_ConfigureWlanChannel(ViSession instrSession,
                                                ViInt32   channelNo)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_WLAN_CHAN, channelNo), 2, "Channel No")

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function performs the auto level routine immediately one time regardless of the
/// HIFN  current state of the Auto Level State.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
ViStatus _VI_FUNC rsspecan_ConfWlanAutoLevelOnce(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_WLAN_POW_AUTO_ONCE, ""));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the reference and signal level and offset.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR autoLevel/Switches on or off automatic power level detection
/// HIPAR referenceLevel/This control sets the reference level
/// HIPAR referenceLevelOffset/Specifies the offset value to the reference level
/// HIPAR signalLevel_RMS/This control specifies the mean power level
ViStatus _VI_FUNC rsspecan_ConfWLANReferenceLevel(ViSession instrSession,
                                                  ViBoolean autoLevel,
                                                  ViReal64  referenceLevel,
                                                  ViReal64  referenceLevelOffset,
                                                  ViReal64  signalLevel_RMS)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_WLAN_POW_AUTO, autoLevel),
            2, "Auto Level");

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_REFERENCE_LEVEL, referenceLevel),
            3, "Reference Level");

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_REFERENCE_LEVEL_OFFSET, referenceLevelOffset),
            4, "Reference Level Offset");

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_WLAN_POW_EXP_RF, signalLevel_RMS),
            5, "Signal Level (RMS)");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the Reference Level Duration.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR mode/To determine the ideal reference level, the R&S FSW performs a
/// HIPAR mode/measurement on the current input data. Selects the way the R&S FSW
/// HIPAR mode/determines the length of the measurement.
/// HIPAR duration/To determine the ideal reference level, the R&S FSW performs a
/// HIPAR duration/measurement on the current input data.
ViStatus _VI_FUNC rsspecan_ConfigureWLANReferenceLevelDuration(
                            ViSession instrSession,
                            ViInt32   mode,
                            ViReal64  duration)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, NULL, RSSPECAN_ATTR_WLAN_ADJUST_REFERENCE_LEVEL_DURATION_MODE, mode),
            2, "Mode");

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, NULL, RSSPECAN_ATTR_WLAN_ADJUST_REFERENCE_LEVEL_DURATION, duration),
            3, "Duration");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function defines the mechanical attenuation for RF input.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR autoMode/This control couples or decouples the attenuation to the reference
/// HIPAR autoMode/level.
/// HIPAR value/This control defines the attenuation.
ViStatus _VI_FUNC rsspecan_ConfigureWLANMechanicalAttenuation(
                            ViSession instrSession,
                            ViBoolean autoMode,
                            ViReal64  value)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "Win0", RSSPECAN_ATTR_ATTENUATION_AUTO, autoMode),
            2, "Auto Mode");

    if (!autoMode)
    {
        viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "Win0", RSSPECAN_ATTR_ATTENUATION, value),
                3, "Value");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/*****************************************************************************
 * Function: Configure WLAN Electronic Attenuator
 * Purpose:  This function configures the signal characteristics.
 *
 *           Note:
 *           (1) This function is available only with option B25.
*****************************************************************************/
ViStatus _VI_FUNC rsspecan_ConfigureWlanElectronicAtt(ViSession instrSession,
                                                      ViBoolean state,
                                                      ViBoolean attenuatorAuto,
                                                      ViInt32   manualValue_dB)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_AMPL_EATT_STATE, state),
            2, "State");
    if (state)
        viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_AMPL_EATT_AUTO, attenuatorAuto),
                3, "Attenuator Auto");
    if (!attenuatorAuto)
        viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_AMPL_EATT_VALUE, manualValue_dB),
                4, "Manual Value [dB]");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the preamplifier for the instrument.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR state/Switches on or off the preamplifier for the instrument.
/// HIPAR preamplifierLevel/Selects the preamplification level.
ViStatus _VI_FUNC rsspecan_ConfigureWLANPreamplifier(ViSession instrSession,
                                                     ViBoolean state,
                                                     ViInt32   preamplifierLevel)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "Win0", RSSPECAN_ATTR_AMPL_PREAMPLIFIER, state),
            2, "State");

    if (state)
    {
        viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_AMPL_PREAMPLIFIER_LEVEL, preamplifierLevel),
                3, "Preamplifier Level");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures how much and which data is to be captured
/// HIFN  during the default WLAN measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR sampleRate/Sets the sampling rate for the I/Q data acquisition.
/// HIPAR captureTime/Specifies the duration (and therefore the amount of data) to be
/// HIPAR captureTime/captured in the capture buffer. If the capture time is too short,
/// HIPAR captureTime/demodulation will fail.
/// HIPAR swapIQ/Defines whether or not the recorded IQ pairs should be swapped (I/Q)
/// HIPAR swapIQ/before being processed.
/// HIPAR filterOutAdjacentChannels/If activated (default), only the useful signal is analyzed, all signal
/// HIPAR filterOutAdjacentChannels/data in adjacent channels is removed by the filter.
ViStatus _VI_FUNC rsspecan_ConfigureWLANSignalCapture(ViSession instrSession,
                                                      ViReal64  sampleRate,
                                                      ViReal64  captureTime,
                                                      ViBoolean swapIQ,
                                                      ViBoolean filterOutAdjacentChannels)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_WLAN_SRAT, sampleRate),
            2, "Sample Rate");

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "Win0", RSSPECAN_ATTR_SWEEP_TIME, captureTime),
            3, "Capture Time");

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_WLAN_SWAP_IQ, swapIQ),
            4, "Swap IQ");

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_WLAN_RBW_FILTER, filterOutAdjacentChannels),
            5, "filter Out Adjacent Channels");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Synchronization settings have an effect on which parts of the input
/// HIFN  signal are processed during the WLAN measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR powerIntervalSearch/If enabled, the R&S FSW WLAN application initially performs a coarse
/// HIPAR powerIntervalSearch/burst search on the input signal in which increases in the power vs
/// HIPAR powerIntervalSearch/time trace are detected. Further time consuming processing is then only
/// HIPAR powerIntervalSearch/performed where bursts are assumed. This improves the measurement speed
/// HIPAR powerIntervalSearch/for signals with low duty cycle rates.
/// HIPAR FFTStartOffset/This control specifies the FFT start offset.
ViStatus _VI_FUNC rsspecan_ConfigureWLANSynchronization(ViSession instrSession,
                                                        ViBoolean powerIntervalSearch,
                                                        ViInt32   FFTStartOffset)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_WLAN_POWER_INTERVAL_SEARCH, powerIntervalSearch),
            2, "Power Interval Search");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_WLAN_FFT_START_OFFSET, FFTStartOffset),
            3, "FFT Start Offset");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function defines whether channel estimation will be done in
/// HIFN  preamble and payload or only in preamble. The effect of this is most
/// HIFN  noticeable for the EVM measurement results, where the results will be
/// HIFN  improved when this feature is enabled.
/// HIFN  However, this functionality is not supported by the IEEE 802.11
/// HIFN  standard and must be disabled if the results are to be measured
/// HIFN  strictly according to the standard.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channelEstimationRange/Specifies the signal range used to estimate the channels.
ViStatus _VI_FUNC rsspecan_ConfigureWLANChannelEstimation(ViSession instrSession,
                                                          ViBoolean channelEstimationRange)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_WLAN_CEST, channelEstimationRange),
            2, "Channel Estimation Range");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines whether the channel bandwidth to be analyzed is determined
/// HIFN  automatically.
/// HIFN     This function is only available for standards IEEE 802.11a, ac, n.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channelBandwidthAuto/Defines whether the channel bandwidth to be analyzed is determined
/// HIPAR channelBandwidthAuto/automatically.
ViStatus _VI_FUNC rsspecan_ConfigureWLANChannelBandwidthAuto(ViSession instrSession,
                                                             ViBoolean channelBandwidthAuto)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, NULL, RSSPECAN_ATTR_WLAN_CHANNEL_BANDWIDTH_AUTO, channelBandwidthAuto),
            2, "Channel Bandwidth Auto");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function defines which results are used for preamble tracking
/// HIFN  prior to the preamble channel estimation.
/// HIFN
/// HIFN     Note(s):
/// HIFN
/// HIFN     (1) This function is only available for standards IEEE 802.11ac and n.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR preambleChannelEstimation/Defines which results are used for preamble tracking prior to the
/// HIPAR preambleChannelEstimation/preamble channel estimation.
ViStatus _VI_FUNC rsspecan_ConfigureWLANPreambleChannelEstimation(
                            ViSession instrSession,
                            ViInt32   preambleChannelEstimation)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_WLAN_PEAMBLE_CHANNEL_ESTIMATION, preambleChannelEstimation),
            2, "Preamble Channel Estimation");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/*****************************************************************************
 * Function: Configure WLAN Tracking
 * Purpose:  This function configures the tracking parameters.
*****************************************************************************/
ViStatus _VI_FUNC rsspecan_ConfigureWlanTracking(ViSession instrSession,
                                                 ViBoolean phase,
                                                 ViBoolean timing,
                                                 ViBoolean level)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_WLAN_TRAC_PHAS, phase),
            2, "Phase");

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_WLAN_TRAC_TIME, timing),
            3, "Time");

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_WLAN_TRAC_LEV, level),
            4, "Level");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function configures how the pilots of the signal is determined.
/// HIRET Returns the status code of this operation.
/// HIRET You may use multiple lines for documentation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init
/// HIPAR instrSession/or rsspecan_InitWithOptions function.
/// HIPAR pilots/This control configures how the pilots of the signal is determined.
ViStatus _VI_FUNC rsspecan_ConfigureWLANTrackingPilots(ViSession instrSession,
                                                       ViInt32   pilots)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_WLAN_TRAC_PILOTS, pilots),
            2, "Pilots");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the Tracking Wiener Relative Delay Spread.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR enabled/Defines whether the Wiener relative delay spread is disabled or
/// HIPAR enabled/defined manually.
/// HIPAR spread/Defines the value relative to the DFT period that is used for the
/// HIPAR spread/Wiener filter design. Decrease this setting to finetune the EVM result
/// HIPAR spread/if there is negligible delay spread, for example for a wired
/// HIPAR spread/connection.
ViStatus _VI_FUNC rsspecan_ConfigureWlanTrackingWienerRelativeDelaySpread(
                            ViSession instrSession,
                            ViInt32   enabled,
                            ViReal64  spread)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, NULL, RSSPECAN_ATTR_WLAN_TRACKING_WIENER_RELATIVE_DELAY_SPREAD_ENABLED, enabled),
            2, "Enabled");

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, NULL, RSSPECAN_ATTR_WLAN_TRACKING_WIENER_RELATIVE_DELAY_SPREAD, spread),
            3, "Spread");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the Burst.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR burst/This control selects the PPDU for which the trace data is queried
/// HIPAR burst/(using TRACe<n>[:DATA]) for the "EVM vs Symbol" and "EVM vs Carrier"
/// HIPAR burst/result displays if [SENSe:]BURSt:SELect:STATe is ON.
/// HIPAR state/Determines whether a selected PPDU (using [SENSe:]BURSt:SELect) is
/// HIPAR state/considered or ignored.
ViStatus _VI_FUNC rsspecan_ConfigureWlanBurst(ViSession instrSession,
                                              ViInt32   burst,
                                              ViBoolean state)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_WLAN_BURST_SELECT, burst),
            2, "Burst");

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_WLAN_BURST_STATE, state),
            3, "State");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function specifies the units for PPDU length results.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR unit/Specifies the units for PPDU length results.
ViStatus _VI_FUNC rsspecan_ConfigureWlanBurstUnit(ViSession instrSession,
                                                  ViInt32   unit)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_WLAN_BURST_UNIT, unit),
            2, "Unit");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines whether an optional iterative search is performed to determine
/// HIFN  the required settings for minimum residual EVM. If enabled, the
/// HIFN  required reference level, preamplifier and, optionally, attenuation are
/// HIFN  configured.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR optimisation/Defines whether an optional iterative search is performed to determine
/// HIPAR optimisation/the required settings for minimum residual EVM. If enabled, the
/// HIPAR optimisation/required reference level, preamplifier and, optionally, attenuation are
/// HIPAR optimisation/configured.
ViStatus _VI_FUNC rsspecan_ConfigureWlanPowerOptimisation(ViSession instrSession,
                                                          ViInt32   optimisation)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, NULL, RSSPECAN_ATTR_WLAN_POWER_OPTIMISATION, optimisation),
            2, "Optimisation");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the marker position.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR marker/The driver uses this value to set the repeated capability Marker of the attributes
/// HIPAR position/Sets the position
/// HIPAR positionValue/Sets the marker position value.
ViStatus _VI_FUNC rsspecan_ConfWlanMarkPosition(ViSession instrSession,
                                                ViInt32   marker,
                                                ViInt32   position,
                                                ViReal64  value)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, marker, 1, 4),
            2, "Marker");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, position, RSSPECAN_VAL_WLAN_POS_X, RSSPECAN_VAL_WLAN_POS_Y),
            3, "Position");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "M%ld", marker);
    switch (position)
    {
    case RSSPECAN_VAL_WLAN_POS_X:
        viCheckParm(rsspecan_SetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_MARKER_POSITION, value),
                4, "Value");
        break;
    case RSSPECAN_VAL_WLAN_POS_Y:
        viCheckParm(rsspecan_SetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_WLAN_MARK_Y, value),
                4, "Value");
        break;
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function positions the selected marker to the indicated symbol/carrier.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR marker/The driver uses this value to set the repeated capability Marker of the attributes
/// HIPAR type/Sets the type
/// HIPAR position/Sets the marker position value
ViStatus _VI_FUNC rsspecan_ConfWlanMarkerPositionConstellation(
                            ViSession instrSession,
                            ViInt32   marker,
                            ViInt32   type,
                            ViInt32   position)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, marker, 1, 4),
            2, "Marker");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, type, RSSPECAN_VAL_WLAN_CONS_SYM, RSSPECAN_VAL_WLAN_CONS_CARR),
            3, "Type");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "M%ld", marker);
    switch (type)
    {
    case RSSPECAN_VAL_WLAN_CONS_SYM:
        viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_WLAN_MARK_SYMB, position),
                4, "Position");
        break;
    case RSSPECAN_VAL_WLAN_CONS_CARR:
        viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_WLAN_MARK_CARR, position),
                4, "Position");
        break;
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function positions the selected marker to the indicated
/// HIFN  symbol in the indicated burst (Constellation vs Symbol and EVM
/// HIFN  vs Symbol).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR marker/
/// HIPAR burstNumber/
/// HIPAR symbolNumber/
ViStatus _VI_FUNC rsspecan_ConfWlanMarkerBurstSymbol(ViSession instrSession,
                                                     ViInt32   marker,
                                                     ViInt32   burstNumber,
                                                     ViInt32   symbolNumber)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, marker, 1, 4),
            2, "Marker");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CALC:MARK%ld:BSYM %ld,%ld", marker, burstNumber, symbolNumber);
    checkErr(RsCore_Write(instrSession, cmd));
    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function activates or deactivates the compensation for I/Q
/// HIFN  mismatch (gain imbalance, quadrature offset, I/Q skew, see chapter
/// HIFN  3.1.1.5, "I/Q Mismatch", on page 19).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR iqCompensation/Activates or deactivates the compensation for I/Q mismatch.
ViStatus _VI_FUNC rsspecan_ConfigureWlanIQCompensation(ViSession instrSession,
                                                       ViBoolean iqCompensation)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_WLAN_IQ_COMPENSATION, iqCompensation),
            2, "IQ Compensation");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Sets the interpolation for tracking and channel estimation.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR interpolationType/Sets the interpolation for tracking and channel estimation.
ViStatus _VI_FUNC rsspecan_ConfigureWLANInterpolationType(ViSession instrSession,
                                                          ViInt32   interpolationType)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_WLAN_INTERPOLATION_TYPE, interpolationType),
            2, "Interpolation Type");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Activates or deactivates the compensation for crosstalk between MIMO
/// HIFN  carriers.
/// HIFN     This function is only available for standard IEEE 802.11ac or n
/// HIFN  (MIMO).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR compensateCrosstalk/Activates or deactivates the compensation for crosstalk between MIMO
/// HIPAR compensateCrosstalk/carriers.
ViStatus _VI_FUNC rsspecan_ConfigureWLANCompensateCrosstalk(ViSession instrSession,
                                                            ViBoolean compensateCrosstalk)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, NULL, RSSPECAN_ATTR_WLAN_COMPENSATE_CROSSTALK, compensateCrosstalk),
            2, "Compensate Crosstalk");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the demodulation parameters for IEEE 802.11a.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR PPDUAnalysisMode/If this parameter sets the burst to analyze auto demodulation state.
/// HIPAR PPDUFormatToMeasure/This control specifies whether how signals are analyzed.
/// HIPAR channelBandwidthToMeasure/This control is used to specify which bursts are analyzed according to
/// HIPAR channelBandwidthToMeasure/STBC streams.
/// HIPAR PSDUModulationToUse/This control specifies whether how signals are analyzed.
/// HIPAR PSDUModulation/This control specifies which PSDUs are to be analyzed depending on
/// HIPAR PSDUModulation/their modulation. Only PSDUs using the selected modulation are
/// HIPAR PSDUModulation/considered in result analysis.
ViStatus _VI_FUNC rsspecan_ConfigureWLANDemodulation802_11a(ViSession instrSession,
                                                            ViBoolean PPDUAnalysisMode,
                                                            ViInt32   PPDUFormatToMeasure,
                                                            ViInt32   channelBandwidthToMeasure,
                                                            ViInt32   PSDUModulationToUse,
                                                            ViInt32   PSDUModulation)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_WLAN_BCON_AUTO, PPDUAnalysisMode),
            2, "PPDU Analysis Mode");

    if (PPDUAnalysisMode == VI_FALSE)
    {
        viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_WLAN_PPDU_FORMAT_TO_MEASURE, PPDUFormatToMeasure),
                3, "PPDU Format To Measure");

        viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_WLAN_CHANNEL_BANDWIDTH_TO_MEASURE, channelBandwidthToMeasure),
                4, "Channel Bandwidth To Measure");

        viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_WLAN_MCS_INDEX_TO_USE_FBUR, PSDUModulationToUse),
                5, "PSDU Modulation To Use");

        if (PSDUModulationToUse >= 2)
        {
            viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_WLAN_PSDU_MODULATION, PSDUModulation),
                    6, "PSDU Modulation");
        }
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the demodulation parameters for IEEE 802.11n.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR PPDUAnalysisMode/If this parameter sets the burst to analyze auto demodulation state.
/// HIPAR PPDUFormatToMeasure/This control specifies whether how signals are analyzed.
/// HIPAR channelBandwidthToMeasure/This control is used to specify which bursts are analyzed according to
/// HIPAR channelBandwidthToMeasure/STBC streams.
ViStatus _VI_FUNC rsspecan_ConfigureWLANDemodulation802_11n(ViSession instrSession,
                                                            ViBoolean PPDUAnalysisMode,
                                                            ViInt32   PPDUFormatToMeasure,
                                                            ViInt32   channelBandwidthToMeasure)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_WLAN_BCON_AUTO, PPDUAnalysisMode),
            2, "PPDU Analysis Mode");

    if (PPDUAnalysisMode == VI_FALSE)
    {
        viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_WLAN_PPDU_FORMAT_TO_MEASURE_802_11N, PPDUFormatToMeasure),
                3, "PPDU Format To Measure");

        viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_WLAN_CHANNEL_BANDWIDTH_TO_MEASURE_802_11N, channelBandwidthToMeasure),
                4, "Channel Bandwidth To Measure");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the Modulation and Coding
/// HIFN     Scheme (MCS) index parameters for IEEE 802.11n demodulation.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR MCSIndexMode/This control is used to specify how bursts are analyzed according to
/// HIPAR MCSIndexMode/the MCS index.
/// HIPAR MCSIndex/This Control allows the user to define the Modulation and
ViStatus _VI_FUNC rsspecan_ConfigureWLANDemodulation802_11nMCSIndex(
                            ViSession instrSession,
                            ViInt32   MCSIndexMode,
                            ViInt32   MCSIndex)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_WLAN_MCS_INDEX_TO_USE_FBUR, MCSIndexMode),
            2, "MCS Index Mode");

    if (MCSIndexMode == RSSPECAN_VAL_WLAN_MCS_INDEX_TO_USE_MEAS || MCSIndexMode == RSSPECAN_VAL_WLAN_MCS_INDEX_TO_USE_DEMOD)
    {
        viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_WLAN_MCS_INDEX, MCSIndex),
                3, "MCS Index");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures which PPDUs are analyzed according to STBC
/// HIFN  streams.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR STBCField/This control is used to specify which PPDUs are analyzed according to
/// HIPAR STBCField/STBC streams.
ViStatus _VI_FUNC rsspecan_ConfigureWLANDemodulation802_11nSTBCField(
                            ViSession instrSession,
                            ViInt32   STBCField)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_WLAN_STBC, STBCField),
            2, "STBC Field");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the PPDUs taking part in the analysis
/// HIFN  depending on the guard interval length.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR guardIntervalLength/This control is used to specify the guard interval length of the
ViStatus _VI_FUNC rsspecan_ConfigureWLANDemodulation802_11nGuardIntervalLength(
                            ViSession instrSession,
                            ViInt32   guardIntervalLength)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_WLAN_GUARD_INTERVAL_LENGTH, guardIntervalLength),
            2, "Guard Interval Length");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Specifies whether the guard time of the input signal is automatically
/// HIFN  detected or specified manually (IEEE 802.11n or ac only).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR autoEnabled/Specifies whether the guard time of the input signal is automatically
/// HIPAR autoEnabled/detected or specified manually (IEEE 802.11n or ac only).
/// HIPAR manualValue/Specifies the guard time the PPDUs in the IEEE 802.11n or ac input
/// HIPAR manualValue/signal should have. This control is valid only if 'Auto Enabled' ==
/// HIPAR manualValue/VI_FALSE.
ViStatus _VI_FUNC rsspecan_ConfigureWLANDemodulation802_11nGuardInterval(
                            ViSession instrSession,
                            ViBoolean autoEnabled,
                            ViInt32   manualValue)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    if (autoEnabled)
    {
        viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, NULL, RSSPECAN_ATTR_WLAN_GUARD_INTERVAL_LENGTH_AUTO_ENABLED, autoEnabled),
                2, "Auto Enabled");
    }
    else
    {
        viCheckParm(rsspecan_SetAttributeViInt32(instrSession, NULL, RSSPECAN_ATTR_WLAN_GUARD_INTERVAL_LENGTH_GUARD_TIME, manualValue),
                3, "Manual Value");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures which PPDUs are analyzed according to Ness
/// HIFN  (Extension Spatial Streams) field content.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR extensionSpatialStreams/This control is used to specify the PPDUs taking part in the analysis
/// HIPAR extensionSpatialStreams/according to the Ness field content.
ViStatus _VI_FUNC rsspecan_ConfigureWLANDemodulation802_11nExtensionSpatialStreams(
                            ViSession instrSession,
                            ViInt32   extensionSpatialStreams)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_WLAN_EXT_SPATIAL_STREAMS, extensionSpatialStreams),
            2, "Extension Spatial Streams");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the Demodulation Data Mode.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR mode/Defines when in the demodulation process the bitstream is determined
/// HIPAR mode/and thus which results are available.
/// HIPAR bitRate/Sets the bit rate.
ViStatus _VI_FUNC rsspecan_ConfigureWLANDemodulation802_11nData(
                            ViSession instrSession,
                            ViInt32   mode,
                            ViInt32   bitRate)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, NULL, RSSPECAN_ATTR_WLAN_DEMODULATION_DATA_MODE, mode),
            2, "Mode");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, NULL, RSSPECAN_ATTR_WLAN_DEMODULATION_DATA_BIT_RATE, bitRate),
            3, "Bit Rate");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the demodulation parameters for IEEE
/// HIFN  802.11b, g (DSSS).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR PPDUFormatPSDUModulation/Defines which PPDU formats/modulations are to be included in the
/// HIPAR PPDUFormatPSDUModulation/analysis.
/// HIPAR PPDUFormat/This control specifies how signals are analyzed.
/// HIPAR PSDUModulation/This control specifies which PSDUs are to be analyzed depending
/// HIPAR PSDUModulation/on their modulation.
ViStatus _VI_FUNC rsspecan_ConfigureWLANDemodulation802_11b_g_DSSS(
                            ViSession instrSession,
                            ViInt32   PPDUFormatPSDUModulation,
                            ViInt32   PPDUFormat,
                            ViInt32   PSDUModulation)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    PPDUFormat;

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_WLAN_PPDU_FORMAT_TO_MEASURE_802_11B_G, PPDUFormatPSDUModulation),
            2, "PPDU Format/PSDU Modulation");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_WLAN_PSDU_MODULATION_802_11B_G, PSDUModulation),
            4, "PSDU Modulation");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the Payload Length (IEEE 802.11b, g).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR lengthType/Specifies the units for min/max payload length.
/// HIPAR equalBurstLength/Enables/disables equal burst length to analyze.
/// HIPAR minPayloadLength/Specifies the duration in microseconds or bytes required for
/// HIPAR minPayloadLength/bursts to qualify for measurement analysis.
/// HIPAR maxPayloadLength/Specifies the maximum duration in microseconds or bytes required
/// HIPAR maxPayloadLength/for bursts to qualify for measurement analysis.
ViStatus _VI_FUNC rsspecan_ConfigureWlan802_11b_g_PayloadLength(
                            ViSession instrSession,
                            ViInt32   lengthType,
                            ViBoolean equalBurstLength,
                            ViInt32   minPayloadLength,
                            ViInt32   maxPayloadLength)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, lengthType, RSSPECAN_VAL_WLAN_DEM_DUR_MS, RSSPECAN_VAL_WLAN_DEM_DUR_BYTE),
            2, "Length Type");

    switch (lengthType)
    {
    case RSSPECAN_VAL_WLAN_DEM_DUR_MS:
        viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_WLAN_DUR_EQU, equalBurstLength),
                3, "Equal Burst Length");

        viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_WLAN_DUR_MIN, minPayloadLength),
                4, "Min Payload Length");
        if (!equalBurstLength)
            viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_WLAN_DUR_MAX, maxPayloadLength),
                    5, "Max Payload Length");
        break;
    case RSSPECAN_VAL_WLAN_DEM_DUR_BYTE:
        viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_WLAN_DBYT_EQU, equalBurstLength),
                3, "Equal Burst Length");

        viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_WLAN_DBYT_MIN, minPayloadLength),
                4, "Min Payload Length");
        if (!equalBurstLength)
        {
            viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_WLAN_DBYT_MAX, maxPayloadLength),
                    5, "Max Payload Length");
        }
        break;
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function selects the TX and RX filters.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR txFilterName/This parameter selects the TX filters name.
/// HIPAR rxFilterName/This parameter selects the RX filters name.
ViStatus _VI_FUNC rsspecan_ConfWlanSelectFilter(ViSession instrSession,
                                                ViString  txFilterName,
                                                ViString  rxFilterName)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K90|K91"));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "DEM:FILT:MOD '%s','%s'", txFilterName, rxFilterName);
    checkErr(RsCore_Write(instrSession, cmd));
    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the equalizer filter length.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR filterLength/The equalizer filter length in chips.
ViStatus _VI_FUNC rsspecan_ConfWLANEqualizerFilterLength(ViSession instrSession,
                                                         ViInt32   filterLength)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_WLAN_EQUALIZER_FILTER_LENGTH, filterLength),
            2, "Filter Length");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function reads the names of all available filters.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR bufferSize/Pass the number of bytes in the string you specify for the catalog list parameter.
/// HIPAR filterList/Returns the names of all available filters.
/// HIPAR returnedLength/Returns the length of filter string readed from instrument.
ViStatus _VI_FUNC rsspecan_QueryWlanFilterCatalog(ViSession instrSession,
                                                  ViInt32   bufferSize,
                                                  ViChar    filterList[],
                                                  ViInt32*  returnedLength)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsspecan_AssertFSV(instrSession));

    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K90|K91"));
    checkErr(RsCore_QueryCatalog(instrSession, "DEM:FILT:CAT?", bufferSize, filterList, returnedLength));

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the demodulation parameters for IEEE
/// HIFN  802.11ac.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR PPDUAnalysisMode/If this parameter sets the burst to analyze auto demodulation state.
/// HIPAR PPDUFormatToMeasure/This control specifies whether how signals are analyzed.
/// HIPAR channelBandwidthToMeasure/This control is used to specify which bursts are analyzed according to
/// HIPAR channelBandwidthToMeasure/STBC streams.
ViStatus _VI_FUNC rsspecan_ConfigureWLANDemodulation802_11ac(ViSession instrSession,
                                                             ViBoolean PPDUAnalysisMode,
                                                             ViInt32   PPDUFormatToMeasure,
                                                             ViInt32   channelBandwidthToMeasure)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_WLAN_BCON_AUTO, PPDUAnalysisMode),
            2, "PPDU Analysis Mode");

    if (PPDUAnalysisMode == VI_FALSE)
    {
        viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_WLAN_PPDU_FORMAT_TO_MEASURE_802_11AC, PPDUFormatToMeasure),
                3, "PPDU Format To Measure");

        viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_WLAN_CHANNEL_BANDWIDTH_TO_MEASURE_802_11AC, channelBandwidthToMeasure),
                4, "Channel Bandwidth To Measure");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the Modulation and Coding
/// HIFN     Scheme (MCS) index parameters for IEEE 802.11ac demodulation.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR MCSIndexMode/This control is used to specify how bursts are analyzed according to
/// HIPAR MCSIndexMode/the MCS index.
/// HIPAR MCSIndex/This Control allows the user to define the Modulation and
ViStatus _VI_FUNC rsspecan_ConfigureWLANDemodulation802_11acMCSIndex(
                            ViSession instrSession,
                            ViInt32   MCSIndexMode,
                            ViInt32   MCSIndex)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_WLAN_MCS_INDEX_TO_USE_FBUR, MCSIndexMode),
            2, "MCS Index Mode");

    if (MCSIndexMode == RSSPECAN_VAL_WLAN_MCS_INDEX_TO_USE_MEAS || MCSIndexMode == RSSPECAN_VAL_WLAN_MCS_INDEX_TO_USE_DEMOD)
    {
        viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_WLAN_MCS_INDEX, MCSIndex),
                3, "MCS Index");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures which PPDUs are analyzed according to STBC
/// HIFN  streams.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR STBCField/This control is used to specify which PPDUs are analyzed according to
/// HIPAR STBCField/STBC streams.
ViStatus _VI_FUNC rsspecan_ConfigureWLANDemodulation802_11acSTBCField(
                            ViSession instrSession,
                            ViInt32   STBCField)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_WLAN_STBC, STBCField),
            2, "STBC Field");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the PPDUs taking part in the analysis
/// HIFN  depending on the guard interval length.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR guardIntervalLength/This control is used to specify the guard interval length of the
ViStatus _VI_FUNC rsspecan_ConfigureWLANDemodulation802_11acGuardIntervalLength(
                            ViSession instrSession,
                            ViInt32   guardIntervalLength)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_WLAN_GUARD_INTERVAL_LENGTH, guardIntervalLength),
            2, "Guard Interval Length");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Specifies whether the guard time of the input signal is automatically
/// HIFN  detected or specified manually (IEEE 802.11n or ac only).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR autoEnabled/Specifies whether the guard time of the input signal is automatically
/// HIPAR autoEnabled/detected or specified manually (IEEE 802.11n or ac only).
/// HIPAR manualValue/Specifies the guard time the PPDUs in the IEEE 802.11n or ac input
/// HIPAR manualValue/signal should have. This control is valid only if 'Auto Enabled' ==
/// HIPAR manualValue/VI_FALSE.
ViStatus _VI_FUNC rsspecan_ConfigureWLANDemodulation802_11acGuardInterval(
                            ViSession instrSession,
                            ViBoolean autoEnabled,
                            ViInt32   manualValue)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    if (autoEnabled)
    {
        viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, NULL, RSSPECAN_ATTR_WLAN_GUARD_INTERVAL_LENGTH_AUTO_ENABLED, autoEnabled),
                2, "Auto Enabled");
    }
    else
    {
        viCheckParm(rsspecan_SetAttributeViInt32(instrSession, NULL, RSSPECAN_ATTR_WLAN_GUARD_INTERVAL_LENGTH_GUARD_TIME, manualValue),
                3, "Manual Value");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the Nsts parameters for IEEE 802.11ac
/// HIFN  demodulation.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR nstsIndexMode/This control is used to specify how bursts are analyzed according to
/// HIPAR nstsIndexMode/the Nsts index.
/// HIPAR nstsIndex/This control allows the user to define the Nsts index, of the bursts
/// HIPAR nstsIndex/taking part in the analysis, manually.
ViStatus _VI_FUNC rsspecan_ConfigureWLANDemodulation802_11acNsts(
                            ViSession instrSession,
                            ViInt32   nstsIndexMode,
                            ViInt32   nstsIndex)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_WLAN_NSTS_INDEX_MODE, nstsIndexMode),
            2, "NSTS Index Mode");

    if (nstsIndexMode == RSSPECAN_VAL_WLAN_NSTS_INDEX_MEAS || nstsIndexMode == RSSPECAN_VAL_WLAN_NSTS_INDEX_DEMOD)
    {
        viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_WLAN_NSTS_INDEX, nstsIndex),
                3, "NSTS Index");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the demodulation parameters for IEEE
/// HIFN  802.11ax.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR ppduAnalysisMode/Determines whether the PPDUs to be analyzed are determined
/// HIPAR ppduAnalysisMode/automatically or by the user.
/// HIPAR ppduFormatToMeasure/This control specifies how signals are analyzed.
/// HIPAR channelBandwidthToMeasure/Defines the channel bandwidth of the PPDUs taking part in the
/// HIPAR channelBandwidthToMeasure/analysis. Depending on which standards the communicating devices are
/// HIPAR channelBandwidthToMeasure/using, different PPDU formats and channel bandwidths are supported.
/// HIPAR guardIntervalLength/This control is used to specify the guard interval length of the
/// HIPAR guardIntervalLength/bursts taking part in the analysis.
ViStatus _VI_FUNC rsspecan_ConfigureWLANDemodulation802_11ax(ViSession instrSession,
                                                             ViBoolean ppduAnalysisMode,
                                                             ViInt32   ppduFormatToMeasure,
                                                             ViInt32   channelBandwidthToMeasure,
                                                             ViInt32   guardIntervalLength)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    /*    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "",
                                                     RSSPECAN_ATTR_WLAN_BCON_AUTO,
                                                     ppduAnalysisMode), 2, "PPDU Analysis Mode");
                                                     */
    if (ppduAnalysisMode == VI_FALSE)
    {
        viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_WLAN_PPDU_FORMAT_TO_MEASURE_802_11AX, ppduFormatToMeasure),
                3, "PPDU Format To Measure");

        viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_WLAN_CHANNEL_BANDWIDTH_TO_MEASURE_802_11AX, channelBandwidthToMeasure),
                4, "Channel Bandwidth To Measure");

        viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_WLAN_GUARD_INTERVAL_LENGTH, guardIntervalLength),
                5, "Guard Interval Length");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the format of the EHT PPDU. This format determines which other
/// HIFN  PPDU settings are available.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR ehtPPDUFormat/Defines the format of the EHT PPDU. This format determines which other
/// HIPAR ehtPPDUFormat/PPDU settings are available.
ViStatus _VI_FUNC rsspecan_ConfigureWLANDemodulation802_11axEHTPPDUFormat(
                            ViSession instrSession,
                            ViInt32   ehtPPDUFormat)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, NULL, RSSPECAN_ATTR_WLAN_EHT_PPDU_FORMAT, ehtPPDUFormat),
            2, "Eht PPDU Format");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function defines the HE PPDU configuration for IEEE 802.11ax.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR hePPDUFormat/Defines the format of the HE PPDU. This format determines which other
/// HIPAR hePPDUFormat/PPDU settings are available.
/// HIPAR nHELTF/Defines the length of the high-efficiency long training field (for
/// HIPAR nHELTF/trigger-based uplink PPDUs only - use control
/// HIPAR nHELTF/RSSPECAN_ATTR_WLAN_HE_PPDU_FORMAT).
ViStatus _VI_FUNC rsspecan_ConfigureWLANDemodulation802_11axHEPPDUConfig(
                            ViSession instrSession,
                            ViInt32   hePPDUFormat,
                            ViInt32   nHELTF)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_WLAN_HE_PPDU_CONFIG_FORMAT, hePPDUFormat),
            2, "HE PPDU Format");

    if (hePPDUFormat == RSSPECAN_VAL_HE_PPDU_FORMAT_TRIG || hePPDUFormat == RSSPECAN_VAL_HE_PPDU_FORMAT_MU)
    {
        viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_WLAN_HE_PPDU_CONFIG_N_HE_LTF, nHELTF),
                3, "N HE-LTF");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Determines the RUs for which the results are evaluated.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR activeCount/Determines the RUs for which the results are evaluated.
ViStatus _VI_FUNC rsspecan_QueryWLANDemodulation802_11axHEPPDURUActiveCount(
                            ViSession instrSession,
                            ViString  activeCount)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViString(instrSession, NULL, RSSPECAN_ATTR_WLAN_CONFIG_RU_ACTIVE_COUNT, activeCount),
            2, "Active Count");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Updates the result displays for the currently selected RUs in the PPDU
/// HIFN  configuration table.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsspecan_QueryWLANDemodulation802_11axHEPPDURURefresh(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsspecan_SetAttributeViString(instrSession, NULL, RSSPECAN_ATTR_WLAN_CONFIG_RU_REFRESH, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function queries the highest configured RU index in the entire
/// HIFN  signal.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR highestRUIndex/Queries the highest configured RU index in the entire signal.
ViStatus _VI_FUNC rsspecan_QueryWLANDemodulation802_11axHEPPDUConfigHighestRUIndex(
                            ViSession instrSession,
                            ViInt32*  highestRUIndex)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_GetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_WLAN_HE_PPDU_CONFG_HIGHEST_RU_INDEX, highestRUIndex),
            2, "Highest RU Index");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function queries the number of users configured for the specified
/// HIFN  RU.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR segment/80-MHz-Segment to be configured.
/// HIPAR channel/20-MHz-subchannel that RU allocation applies to.
/// HIPAR ruIndex/Index of the entry in the RU allocation table.
/// HIPAR userIndex/Queries the number of users configured for the specified RU.
ViStatus _VI_FUNC rsspecan_QueryWLANDemodulation802_11axHEPPDUConfigUserIndex(
                            ViSession instrSession,
                            ViInt32   segment,
                            ViInt32   channel,
                            ViInt32   ruIndex,
                            ViInt32*  userIndex)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap2[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap3[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, segment, 1, 2),
            2, "Segment");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, channel, 1, 8),
            3, "Channel");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, ruIndex, 1, 72),
            4, "RU Index");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_WLAN_HE_PPDU_CONFIG_USER_INDEX,
            "WLANaxSegment", segment , RS_REPCAP_BUF_SIZE, repCap),
            2, "Segment");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_WLAN_HE_PPDU_CONFIG_USER_INDEX,
                    "WLANaxChannel", channel - 1, RS_REPCAP_BUF_SIZE, repCap2),
                3, "Channel");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_WLAN_HE_PPDU_CONFIG_USER_INDEX,
                    "WLANaxRUIndex", ruIndex - 1, RS_REPCAP_BUF_SIZE, repCap3),
                4, "RU Index");

    strcat(repCap, ",");
    strcat(repCap, repCap2);
    strcat(repCap, ",");
    strcat(repCap, repCap3);

    viCheckParm(rsspecan_GetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_WLAN_HE_PPDU_CONFIG_USER_INDEX, userIndex),
            5, "User Index");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the index of the subcarrier and resource allocation for multiple
/// HIFN  RUs (MRU) as defined by the 802.11be standard. This index determines
/// HIFN  the position of the ignored subcarriers and used RUs within the
/// HIFN  spectrum.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR segment/Selects repcap WLANaxSegment.
/// HIPAR channel/Selects repcap WLANaxChannel.
/// HIPAR mruIndex/Selects repcap WLANaxRUIndex.
/// HIPAR userIndexMRU/Sets or queries the index of the subcarrier and resource allocation
/// HIPAR userIndexMRU/for multiple RUs (MRU) as defined by the 802.11be standard. This index
/// HIPAR userIndexMRU/determines the position of the ignored subcarriers and used RUs within
/// HIPAR userIndexMRU/the spectrum.
ViStatus _VI_FUNC rsspecan_ConfigureWLANDemodulation802_11axHEPPDUConfigUserIndexMRU(
                            ViSession instrSession,
                            ViInt32   segment,
                            ViInt32   channel,
                            ViInt32   mruIndex,
                            ViInt32   userIndexMRU)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_ch[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_ind[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_WLAN_HE_PPDU_CONFIG_RU_INDEX_MRU,
            "WLANaxSegment", segment - 1, RS_REPCAP_BUF_SIZE, repCap));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_WLAN_HE_PPDU_CONFIG_RU_INDEX_MRU,
            "WLANaxChannel", channel - 1, RS_REPCAP_BUF_SIZE, repCap_ch));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_WLAN_HE_PPDU_CONFIG_RU_INDEX_MRU,
            "WLANaxRUIndex", mruIndex - 1, RS_REPCAP_BUF_SIZE, repCap_ind));

    strcat(repCap, ",");
    strcat(repCap, repCap_ch);
    strcat(repCap, ",");
    strcat(repCap, repCap_ind);

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_WLAN_HE_PPDU_CONFIG_RU_INDEX_MRU, userIndexMRU),
            5, "User Index MRU");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Sets or queries the index of the resource unit based on 26-subcarrier
/// HIFN  units.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR segment/Selects repcap WLANaxSegment.
/// HIPAR channel/Selects repcap WLANaxChannel.
/// HIPAR ru26Index/Selects repcap WLANaxRUIndex.
/// HIPAR userIndexRU26/Sets or queries the index of the resource unit based on 26-subcarrier
/// HIPAR userIndexRU26/units.
ViStatus _VI_FUNC rsspecan_ConfigureWLANDemodulation802_11axHEPPDUConfigUserIndexRU26(
                            ViSession instrSession,
                            ViInt32   segment,
                            ViInt32   channel,
                            ViInt32   ru26Index,
                            ViInt32   userIndexRU26)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_ch[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_ind[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_WLAN_HE_PPDU_CONFIG_RU_INDEX_RU26,
            "WLANaxSegment", segment - 1, RS_REPCAP_BUF_SIZE, repCap));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_WLAN_HE_PPDU_CONFIG_RU_INDEX_RU26,
            "WLANaxChannel", channel - 1, RS_REPCAP_BUF_SIZE, repCap_ch));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_WLAN_HE_PPDU_CONFIG_RU_INDEX_RU26,
            "WLANaxRUIndex", ru26Index - 1, RS_REPCAP_BUF_SIZE, repCap_ind));

    strcat(repCap, ",");
    strcat(repCap, repCap_ch);
    strcat(repCap, ",");
    strcat(repCap, repCap_ind);

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_WLAN_HE_PPDU_CONFIG_RU_INDEX_RU26, userIndexRU26),
            5, "User Index RU26");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets or queries the index of the resource unit based on 6-subcarrier
/// HIFN  units.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR segment/Selects repcap WLANaxSegment.
/// HIPAR channel/Selects repcap WLANaxChannel.
/// HIPAR ru6Index/Selects repcap WLANaxRUIndex.
/// HIPAR userIndexRU6/Sets or queries the index of the resource unit based on 6-subcarrier
/// HIPAR userIndexRU6/units.
ViStatus _VI_FUNC rsspecan_ConfigureWLANDemodulation802_11axHEPPDUConfigUserIndexRU6(
                            ViSession instrSession,
                            ViInt32   segment,
                            ViInt32   channel,
                            ViInt32   ru6Index,
                            ViInt32   userIndexRU6)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_ch[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_ind[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_WLAN_HE_PPDU_CONFIG_RU_INDEX_RU6,
            "WLANaxSegment", segment - 1, RS_REPCAP_BUF_SIZE, repCap));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_WLAN_HE_PPDU_CONFIG_RU_INDEX_RU6,
            "WLANaxChannel", channel - 1, RS_REPCAP_BUF_SIZE, repCap_ch));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_WLAN_HE_PPDU_CONFIG_RU_INDEX_RU6,
            "WLANaxRUIndex", ru6Index - 1, RS_REPCAP_BUF_SIZE, repCap_ind));

    strcat(repCap, ",");
    strcat(repCap, repCap_ch);
    strcat(repCap, ",");
    strcat(repCap, repCap_ind);

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_WLAN_HE_PPDU_CONFIG_RU_INDEX_RU6, userIndexRU6),
            5, "User Index RU26");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function configures the RU index for the specified entry in the
/// HIFN  RU allocation table. This is especially useful for signals with larger
/// HIFN  bandwidths, when the RU index continues for multiple RU allocation
/// HIFN  channels.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR segment/80-MHz-Segment to be configured.
/// HIPAR channel/20-MHz-subchannel that RU allocation applies to.
/// HIPAR ruIndex/Index of the entry in the RU allocation table.
/// HIPAR index/Configures the RU index for the specified entry in the RU allocation
/// HIPAR index/table. This control is especially useful for signals with larger
/// HIPAR index/bandwidths, when the RU index continues for multiple RU allocation
/// HIPAR index/channels.
ViStatus _VI_FUNC rsspecan_ConfigureWLANDemodulation802_11axHEPPDUConfigRUIndex(
                            ViSession instrSession,
                            ViInt32   segment,
                            ViInt32   channel,
                            ViInt32   ruIndex,
                            ViInt32   index)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap2[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap3[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, segment, 1, 2),
            2, "Segment");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, channel, 1, 8),
            3, "Channel");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, ruIndex, 1, 72),
            4, "RU Index");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_WLAN_HE_PPDU_CONFIG_RU_INDEX,
            "WLANaxSegment", segment - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Segment");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_WLAN_HE_PPDU_CONFIG_RU_INDEX,
                    "WLANaxChannel", channel - 1, RS_REPCAP_BUF_SIZE, repCap2),
                3, "Channel");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_WLAN_HE_PPDU_CONFIG_RU_INDEX,
                    "WLANaxRUIndex", ruIndex - 1, RS_REPCAP_BUF_SIZE, repCap3),
                4, "RU Index");

    strcat(repCap, ",");
    strcat(repCap, repCap2);
    strcat(repCap, ",");
    strcat(repCap, repCap3);

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_WLAN_HE_PPDU_CONFIG_RU_INDEX, index),
            5, "Index");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the RU index for the specified entry in the RU allocation
/// HIFN  table.
/// HIFN     This function is especially useful for signals with larger bandwidths,
/// HIFN  when the RU index continues for multiple RU allocation channels.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR segment/80-MHz-Segment to be configured.
/// HIPAR channel/20-MHz-subchannel that RU allocation applies to.
/// HIPAR ruIndex/Index of the entry in the RU allocation table.
/// HIPAR count/Queries the RU index for the specified entry in the RU allocation
/// HIPAR count/table.
ViStatus _VI_FUNC rsspecan_QueryWLANDemodulation802_11axHEPPDUConfigRUIndexCount(
                            ViSession instrSession,
                            ViInt32   segment,
                            ViInt32   channel,
                            ViInt32   ruIndex,
                            ViInt32*  count)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap2[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap3[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, segment, 1, 2),
            2, "Segment");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, channel, 1, 8),
            3, "Channel");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, ruIndex, 1, 72),
            4, "RU Index");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_WLAN_HE_PPDU_CONFIG_RU_INDEX_COUNT,
            "WLANaxSegment", segment - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Segment");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_WLAN_HE_PPDU_CONFIG_RU_INDEX_COUNT,
                    "WLANaxChannel", channel - 1, RS_REPCAP_BUF_SIZE, repCap2),
                3, "Channel");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_WLAN_HE_PPDU_CONFIG_RU_INDEX_COUNT,
                    "WLANaxRUIndex", ruIndex - 1, RS_REPCAP_BUF_SIZE, repCap3),
                4, "RU Index");

    strncat(repCap, ",", RS_REPCAP_BUF_SIZE - strlen(repCap) - 1);
    strncat(repCap, repCap2, RS_REPCAP_BUF_SIZE - strlen(repCap) - 1);
    strncat(repCap, ",", RS_REPCAP_BUF_SIZE - strlen(repCap) - 1);
    strncat(repCap, repCap3, RS_REPCAP_BUF_SIZE - strlen(repCap) - 1);

    viCheckParm(rsspecan_GetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_WLAN_HE_PPDU_CONFIG_RU_INDEX_COUNT, count),
            5, "Count");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the highest configured RU index in the specified RU allocation
/// HIFN  table.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR segment/80-MHz-Segment to be configured.
/// HIPAR channel/20-MHz-subchannel that RU allocation applies to.
/// HIPAR ruIndex/Index of the entry in the RU allocation table.
/// HIPAR highest/Queries the highest configured RU index in the specified RU allocation
/// HIPAR highest/table.
ViStatus _VI_FUNC rsspecan_QueryWLANDemodulation802_11axHEPPDUConfigRUIndexHighest(
                            ViSession instrSession,
                            ViInt32   segment,
                            ViInt32   channel,
                            ViInt32   ruIndex,
                            ViInt32*  highest)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap2[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap3[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, segment, 1, 2),
            2, "Segment");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, channel, 1, 8),
            3, "Channel");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, ruIndex, 1, 72),
            4, "RU Index");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_WLAN_HE_PPDU_CONFIG_RU_INDEX_HIGHEST,
            "WLANaxSegment", segment - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Segment");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_WLAN_HE_PPDU_CONFIG_RU_INDEX_HIGHEST,
                    "WLANaxChannel", channel - 1, RS_REPCAP_BUF_SIZE, repCap2),
                3, "Channel");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_WLAN_HE_PPDU_CONFIG_RU_INDEX_HIGHEST,
                    "WLANaxRUIndex", ruIndex - 1, RS_REPCAP_BUF_SIZE, repCap3),
                4, "RU Index");

    strncat(repCap, ",", RS_REPCAP_BUF_SIZE - strlen(repCap) - 1);
    strncat(repCap, repCap2, RS_REPCAP_BUF_SIZE - strlen(repCap) - 1);
    strncat(repCap, ",", RS_REPCAP_BUF_SIZE - strlen(repCap) - 1);
    strncat(repCap, repCap3, RS_REPCAP_BUF_SIZE - strlen(repCap) - 1);

    viCheckParm(rsspecan_GetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_WLAN_HE_PPDU_CONFIG_RU_INDEX_HIGHEST, highest),
            5, "Highest");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function defines the size of the individual resource unit (=
/// HIFN  number of subcarriers or tones) for a single transmission package.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR segment/80-MHz-Segment to be configured.
/// HIPAR channel/20-MHz-subchannel that RU allocation applies to.
/// HIPAR ruIndex/Index of the entry in the RU allocation table.
/// HIPAR ruSize/Defines the size of the individual resource unit (= number of
/// HIPAR ruSize/subcarriers or tones) for a single transmission package.
ViStatus _VI_FUNC rsspecan_ConfigureWLANDemodulation802_11axHEPPDUConfigRUSize(
                            ViSession instrSession,
                            ViInt32   segment,
                            ViInt32   channel,
                            ViInt32   ruIndex,
                            ViInt32   ruSize)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap2[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap3[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, segment, 1, 2),
            2, "Segment");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, channel, 1, 8),
            3, "Channel");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, ruIndex, 1, 72),
            4, "RU Index");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_WLAN_HE_PPDU_CONFIG_RU_SIZE,
            "WLANaxSegment", segment - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Segment");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_WLAN_HE_PPDU_CONFIG_RU_SIZE,
                    "WLANaxChannel", channel - 1, RS_REPCAP_BUF_SIZE, repCap2),
                3, "Channel");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_WLAN_HE_PPDU_CONFIG_RU_SIZE,
                    "WLANaxRUIndex", ruIndex - 1, RS_REPCAP_BUF_SIZE, repCap3),
                4, "RU Index");

    strcat(repCap, ",");
    strcat(repCap, repCap2);
    strcat(repCap, ",");
    strcat(repCap, repCap3);

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_WLAN_HE_PPDU_CONFIG_RU_SIZE, ruSize),
            5, "RU Size");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the modulation and Coding Scheme (MCS) index
/// HIFN  of the PPDU.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR segment/80-MHz-Segment to be configured.
/// HIPAR channel/20-MHz-subchannel that RU allocation applies to.
/// HIPAR ruIndex/Index of the entry in the RU allocation table.
/// HIPAR userIndex/User index for RU in MIMO mode.
/// HIPAR mcsIndex/Modulation and Coding Scheme (MCS) index of the PPDU
ViStatus _VI_FUNC rsspecan_ConfigureWLANDemodulation802_11axHEPPDUConfigMCSIndex(
                            ViSession instrSession,
                            ViInt32   segment,
                            ViInt32   channel,
                            ViInt32   ruIndex,
                            ViInt32   userIndex,
                            ViInt32   mcsIndex)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap2[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap3[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap4[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, segment, 1, 2),
            2, "Segment");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, channel, 1, 8),
            3, "Channel");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, ruIndex, 1, 72),
            4, "RU Index");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, userIndex, 1, 8),
            5, "User Index");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_WLAN_HE_PPDU_CONFIG_MCS_INDEX,
            "WLANaxSegment", segment - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Segment");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_WLAN_HE_PPDU_CONFIG_MCS_INDEX,
                    "WLANaxChannel", channel - 1, RS_REPCAP_BUF_SIZE, repCap2),
                3, "Channel");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_WLAN_HE_PPDU_CONFIG_MCS_INDEX,
                    "WLANaxRUIndex", ruIndex - 1, RS_REPCAP_BUF_SIZE, repCap3),
                4, "RU Index");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_WLAN_HE_PPDU_CONFIG_MCS_INDEX,
                    "WLANaxUserIndex", userIndex - 1, RS_REPCAP_BUF_SIZE, repCap4),
                5, "User Index");

    strcat(repCap, ",");
    strcat(repCap, repCap2);
    strcat(repCap, ",");
    strcat(repCap, repCap3);
    strcat(repCap, ",");
    strcat(repCap, repCap4);

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_WLAN_HE_PPDU_CONFIG_MCS_INDEX, mcsIndex),
            6, "MCS Index");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the number of space-time streams (NSTS) for
/// HIFN  each user, for MIMO measurements only.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR segment/80-MHz-Segment to be configured.
/// HIPAR channel/20-MHz-subchannel that RU allocation applies to.
/// HIPAR ruIndex/Index of the entry in the RU allocation table.
/// HIPAR userIndex/User index for RU in MIMO mode.
/// HIPAR nstsPerUser/For MIMO measurements only: Number of space-time streams (NSTS) for
/// HIPAR nstsPerUser/each user
ViStatus _VI_FUNC rsspecan_ConfigureWLANDemodulation802_11axHEPPDUConfigNstsPerUser(
                            ViSession instrSession,
                            ViInt32   segment,
                            ViInt32   channel,
                            ViInt32   ruIndex,
                            ViInt32   userIndex,
                            ViInt32   nstsPerUser)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap2[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap3[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap4[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, segment, 1, 2),
            2, "Segment");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, channel, 1, 8),
            3, "Channel");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, ruIndex, 1, 72),
            4, "RU Index");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, userIndex, 1, 8),
            5, "User Index");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_WLAN_HE_PPDU_CONFIG_NSTS_PER_USER,
            "WLANaxSegment", segment - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Segment");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_WLAN_HE_PPDU_CONFIG_NSTS_PER_USER,
                    "WLANaxChannel", channel - 1, RS_REPCAP_BUF_SIZE, repCap2),
                3, "Channel");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_WLAN_HE_PPDU_CONFIG_NSTS_PER_USER,
                    "WLANaxRUIndex", ruIndex - 1, RS_REPCAP_BUF_SIZE, repCap3),
                4, "RU Index");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_WLAN_HE_PPDU_CONFIG_NSTS_PER_USER,
                    "WLANaxUserIndex", userIndex - 1, RS_REPCAP_BUF_SIZE, repCap4),
                5, "User Index");

    strcat(repCap, ",");
    strcat(repCap, repCap2);
    strcat(repCap, ",");
    strcat(repCap, repCap3);
    strcat(repCap, ",");
    strcat(repCap, repCap4);

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_WLAN_HE_PPDU_CONFIG_NSTS_PER_USER, nstsPerUser),
            6, "Nsts Per User");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function defines whether transmit beamforming is used.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR segment/80-MHz-Segment to be configured.
/// HIPAR channel/20-MHz-subchannel that RU allocation applies to.
/// HIPAR ruIndex/Index of the entry in the RU allocation table.
/// HIPAR userIndex/User index for RU in MIMO mode.
/// HIPAR txBeamforming/Defines whether transmit beamforming is used.
ViStatus _VI_FUNC rsspecan_ConfigureWLANDemodulation802_11axHEPPDUConfigTXBeamforming(
                            ViSession instrSession,
                            ViInt32   segment,
                            ViInt32   channel,
                            ViInt32   ruIndex,
                            ViInt32   userIndex,
                            ViBoolean txBeamforming)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap2[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap3[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap4[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, segment, 1, 2),
            2, "Segment");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, channel, 1, 8),
            3, "Channel");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, ruIndex, 1, 72),
            4, "RU Index");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, userIndex, 1, 8),
            5, "User Index");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_WLAN_HE_PPDU_CONFIG_TX_BEAMFORMING,
            "WLANaxSegment", segment - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Segment");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_WLAN_HE_PPDU_CONFIG_TX_BEAMFORMING,
                    "WLANaxChannel", channel - 1, RS_REPCAP_BUF_SIZE, repCap2),
                3, "Channel");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_WLAN_HE_PPDU_CONFIG_TX_BEAMFORMING,
                    "WLANaxRUIndex", ruIndex - 1, RS_REPCAP_BUF_SIZE, repCap3),
                4, "RU Index");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_WLAN_HE_PPDU_CONFIG_TX_BEAMFORMING,
                    "WLANaxUserIndex", userIndex - 1, RS_REPCAP_BUF_SIZE, repCap4),
                5, "User Index");

    strcat(repCap, ",");
    strcat(repCap, repCap2);
    strcat(repCap, ",");
    strcat(repCap, repCap3);
    strcat(repCap, ",");
    strcat(repCap, repCap4);

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, repCap, RSSPECAN_ATTR_WLAN_HE_PPDU_CONFIG_TX_BEAMFORMING, txBeamforming),
            6, "TX Beamforming");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function defines the use of dual carrier modulation for the
/// HIFN  specified user.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR segment/80-MHz-Segment to be configured.
/// HIPAR channel/20-MHz-subchannel that RU allocation applies to.
/// HIPAR ruIndex/Index of the entry in the RU allocation table.
/// HIPAR userIndex/User index for RU in MIMO mode.
/// HIPAR dcm/Defines the use of dual carrier modulation for the specified user.
ViStatus _VI_FUNC rsspecan_ConfigureWLANDemodulation802_11axHEPPDUConfigDCM(
                            ViSession instrSession,
                            ViInt32   segment,
                            ViInt32   channel,
                            ViInt32   ruIndex,
                            ViInt32   userIndex,
                            ViBoolean dcm)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap2[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap3[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap4[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, segment, 1, 2),
            2, "Segment");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, channel, 1, 8),
            3, "Channel");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, ruIndex, 1, 72),
            4, "RU Index");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, userIndex, 1, 8),
            5, "User Index");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_WLAN_HE_PPDU_CONFIG_DCM,
            "WLANaxSegment", segment - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Segment");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_WLAN_HE_PPDU_CONFIG_DCM,
                    "WLANaxChannel", channel - 1, RS_REPCAP_BUF_SIZE, repCap2),
                3, "Channel");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_WLAN_HE_PPDU_CONFIG_DCM,
                    "WLANaxRUIndex", ruIndex - 1, RS_REPCAP_BUF_SIZE, repCap3),
                4, "RU Index");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_WLAN_HE_PPDU_CONFIG_DCM,
                    "WLANaxUserIndex", userIndex - 1, RS_REPCAP_BUF_SIZE, repCap4),
                5, "User Index");

    strcat(repCap, ",");
    strcat(repCap, repCap2);
    strcat(repCap, ",");
    strcat(repCap, repCap3);
    strcat(repCap, ",");
    strcat(repCap, repCap4);

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, repCap, RSSPECAN_ATTR_WLAN_HE_PPDU_CONFIG_DCM, dcm),
            6, "DCM");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function defines the type of coding used by the PPDU.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR segment/80-MHz-Segment to be configured.
/// HIPAR channel/20-MHz-subchannel that RU allocation applies to.
/// HIPAR ruIndex/Index of the entry in the RU allocation table.
/// HIPAR userIndex/User index for RU in MIMO mode.
/// HIPAR coding/The type of coding used by the PPDU.
ViStatus _VI_FUNC rsspecan_ConfigureWLANDemodulation802_11axHEPPDUConfigCoding(
                            ViSession instrSession,
                            ViInt32   segment,
                            ViInt32   channel,
                            ViInt32   ruIndex,
                            ViInt32   userIndex,
                            ViInt32   coding)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap2[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap3[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap4[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, segment, 1, 2),
            2, "Segment");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, channel, 1, 8),
            3, "Channel");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, ruIndex, 1, 72),
            4, "RU Index");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, userIndex, 1, 8),
            5, "User Index");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_WLAN_HE_PPDU_CONFIG_CODING,
            "WLANaxSegment", segment - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Segment");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_WLAN_HE_PPDU_CONFIG_CODING,
                    "WLANaxChannel", channel - 1, RS_REPCAP_BUF_SIZE, repCap2),
                3, "Channel");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_WLAN_HE_PPDU_CONFIG_CODING,
                    "WLANaxRUIndex", ruIndex - 1, RS_REPCAP_BUF_SIZE, repCap3),
                4, "RU Index");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_WLAN_HE_PPDU_CONFIG_CODING,
                    "WLANaxUserIndex", userIndex - 1, RS_REPCAP_BUF_SIZE, repCap4),
                5, "User Index");

    strcat(repCap, ",");
    strcat(repCap, repCap2);
    strcat(repCap, ",");
    strcat(repCap, repCap3);
    strcat(repCap, ",");
    strcat(repCap, repCap4);

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_WLAN_HE_PPDU_CONFIG_CODING, coding),
            6, "Coding");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  For HE Multi-User Downlink PPDUs that support MIMOonly. Adds another
/// HIFN  user (station) for the selected resource unit (RU) to the configuration
/// HIFN  table. A maximum of 8 users can be assigned to a single resource unit
/// HIFN  in MIMO mode. This function is only available for RU sizes of at least
/// HIFN  106 subcarriers.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR segment/80-MHz-Segment to be configured.
/// HIPAR channel/20-MHz-subchannel that RU allocation applies to.
/// HIPAR ruIndex/Index of the entry in the RU allocation table.
/// HIPAR userIndex/User index for RU in MIMO mode.
ViStatus _VI_FUNC rsspecan_ConfigureWLANDemodulation802_11axHEPPDUConfigInsert(
                            ViSession instrSession,
                            ViInt32   segment,
                            ViInt32   channel,
                            ViInt32   ruIndex,
                            ViInt32   userIndex)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap2[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap3[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap4[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, segment, 1, 2),
            2, "Segment");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, channel, 1, 8),
            3, "Channel");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, ruIndex, 1, 72),
            4, "RU Index");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, userIndex, 1, 8),
            5, "User Index");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_WLAN_HE_PPDU_CONFIG_CODING,
            "WLANaxSegment", segment - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Segment");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_WLAN_HE_PPDU_CONFIG_CODING,
                    "WLANaxChannel", channel - 1, RS_REPCAP_BUF_SIZE, repCap2),
                3, "Channel");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_WLAN_HE_PPDU_CONFIG_CODING,
                    "WLANaxRUIndex", ruIndex - 1, RS_REPCAP_BUF_SIZE, repCap3),
                4, "RU Index");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_WLAN_HE_PPDU_CONFIG_CODING,
                    "WLANaxUserIndex", userIndex - 1, RS_REPCAP_BUF_SIZE, repCap4),
                5, "User Index");

    strcat(repCap, ",");
    strcat(repCap, repCap2);
    strcat(repCap, ",");
    strcat(repCap, repCap3);
    strcat(repCap, ",");
    strcat(repCap, repCap4);

    checkErr(rsspecan_SetAttributeViString(instrSession, repCap, RSSPECAN_ATTR_WLAN_HE_PPDU_CONFIG_INSERT_USER, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Deletes the selected user (station) from the HE Multi-User Downlink
/// HIFN  PPDU configuration table (for MIMO configuration only).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR segment/80-MHz-Segment to be configured.
/// HIPAR channel/20-MHz-subchannel that RU allocation applies to.
/// HIPAR ruIndex/Index of the entry in the RU allocation table.
/// HIPAR userIndex/User index for RU in MIMO mode.
ViStatus _VI_FUNC rsspecan_ConfigureWLANDemodulation802_11axHEPPDUConfigDelete(
                            ViSession instrSession,
                            ViInt32   segment,
                            ViInt32   channel,
                            ViInt32   ruIndex,
                            ViInt32   userIndex)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap2[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap3[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap4[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, segment, 1, 2),
            2, "Segment");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, channel, 1, 8),
            3, "Channel");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, ruIndex, 1, 72),
            4, "RU Index");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, userIndex, 1, 8),
            5, "User Index");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_WLAN_HE_PPDU_CONFIG_CODING,
            "WLANaxSegment", segment - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Segment");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_WLAN_HE_PPDU_CONFIG_CODING,
                    "WLANaxChannel", channel - 1, RS_REPCAP_BUF_SIZE, repCap2),
                3, "Channel");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_WLAN_HE_PPDU_CONFIG_CODING,
                    "WLANaxRUIndex", ruIndex - 1, RS_REPCAP_BUF_SIZE, repCap3),
                4, "RU Index");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_WLAN_HE_PPDU_CONFIG_CODING,
                    "WLANaxUserIndex", userIndex - 1, RS_REPCAP_BUF_SIZE, repCap4),
                5, "User Index");

    strcat(repCap, ",");
    strcat(repCap, repCap2);
    strcat(repCap, ",");
    strcat(repCap, repCap3);
    strcat(repCap, ",");
    strcat(repCap, repCap4);

    checkErr(rsspecan_SetAttributeViString(instrSession, repCap, RSSPECAN_ATTR_WLAN_HE_PPDU_CONFIG_DELETE_USER, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the demodulation parameters for IEEE
/// HIFN  802.11ac.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR ppduAnalysisMode/Determines whether the PPDUs to be analyzed are determined
/// HIPAR ppduAnalysisMode/automatically or by the user.
/// HIPAR ppduFormatToMeasure/This control specifies whether how signals are analyzed.
/// HIPAR channelBandwidthToMeasure/This control is used to specify which bursts are analyzed according to
/// HIPAR channelBandwidthToMeasure/STBC streams.
ViStatus _VI_FUNC rsspecan_ConfigureWLANDemodulation802_11be(ViSession instrSession,
                                                             ViBoolean ppduAnalysisMode,
                                                             ViInt32   ppduFormatToMeasure,
                                                             ViInt32   channelBandwidthToMeasure)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, NULL, RSSPECAN_ATTR_WLAN_BCON_AUTO, ppduAnalysisMode),
            2, "PPDU Analysis Mode");

    if (ppduAnalysisMode == VI_FALSE)
    {
        viCheckParm(rsspecan_SetAttributeViInt32(instrSession, NULL, RSSPECAN_ATTR_WLAN_PPDU_FORMAT_TO_MEASURE_802_11BE, ppduFormatToMeasure),
                3, "PPDU Format To Measure");

        viCheckParm(rsspecan_SetAttributeViInt32(instrSession, NULL, RSSPECAN_ATTR_WLAN_CHANNEL_BANDWIDTH_TO_MEASURE_802_11BE, channelBandwidthToMeasure),
                4, "Channel Bandwidth To Measure");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function specifies the number of antennas.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init.
/// HIPAR numberOfAntennas/This control specifies the number of antennas.
ViStatus _VI_FUNC rsspecan_ConfWLANNumberOfAntennas(ViSession instrSession,
                                                    ViInt32   numberOfAntennas)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_WLAN_NUMBER_OF_ANTENNAS, numberOfAntennas),
            2, "Number Of Antennas");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function defines the setup how the Tx antenna signals of
/// HIFN the device under test (DUT) are captured by the
/// HIFN analyzer/analyzers.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init.
/// HIPAR signalCapture/This control specifies how the Tx antenna signals of the device under test (DUT) are captured by the analyzer/analyzers.
ViStatus _VI_FUNC rsspecan_ConfWLANMIMODUTConfiguration(ViSession instrSession,
                                                        ViInt32   signalCapture)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_WLAN_MIMO_DUT_CONFIGURATION, signalCapture),
            2, "Signal Capture");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function specifies method for analyzing MIMO signals.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init.
/// HIPAR MIMOAntennaSignalCapture/This control specifies method for analyzing MIMO signals.
ViStatus _VI_FUNC rsspecan_ConfWLANMIMOAntennaSignalCapture(ViSession instrSession,
                                                            ViInt32   MIMOAntennaSignalCapture)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_WLAN_MIMO_ANT_SIGNAL_CAPTURE_TYPE, MIMOAntennaSignalCapture),
            2, "MIMO Antenna Signal Capture");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the switch unit to be used for automated sequential MIMO
/// HIFN  measurements. The supported unit is Rohde & Schwarz OSP 1505.3009.03
/// HIFN  with module option 1505.5101.02
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR ospModule/Specifies the module of the switch unit to be used for automated
/// HIPAR ospModule/sequential MIMO measurements. The supported unit is Rohde & Schwarz OSP
/// HIPAR ospModule/1505.3009.03 with module option 1505.5101.02
/// HIPAR ospIPAddress/Specifies the TCP/IP address of the switch unit to be used for
/// HIPAR ospIPAddress/automated sequential MIMO measurements. The supported unit is Rohde &
/// HIPAR ospIPAddress/Schwarz OSP 1505.3009.03 with module option 1505.5101.02
ViStatus _VI_FUNC rsspecan_ConfigureWLANMIMOAntennaSignalCaptureOSP(
                            ViSession instrSession,
                            ViInt32   ospModule,
                            ViString  ospIPAddress)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, NULL, RSSPECAN_ATTR_WLAN_MIMO_ANTENNA_SIGNAL_CAPTURE_OSP_MODULE, ospModule),
            2, "OSP Module");

    viCheckParm(rsspecan_SetAttributeViString(instrSession, NULL, RSSPECAN_ATTR_WLAN_MIMO_ANTENNA_SIGNAL_CAPTURE_OSP_IP_ADDRESS, ospIPAddress),
            3, "OSP IP Address");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Specifies the signal path to be captured in MIMO sequential manual
/// HIFN  measurements and immediately starts capturing data.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR path/Specifies the signal path to be captured in MIMO sequential manual
/// HIPAR path/measurements and immediately starts capturing data.
ViStatus _VI_FUNC rsspecan_ConfigureWLANMIMOAntennaSignalCapturePath(
                            ViSession instrSession,
                            ViInt32   path)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, NULL, RSSPECAN_ATTR_WLAN_MIMO_ANTENNA_SIGNAL_CAPTURE_PATH, path),
            2, "Path");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Determines whether or not the cyclic shift delay (CSD) is used for
/// HIFN  timing synchronisation.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR timeSync/Determines whether or not the cyclic shift delay (CSD) is used for
/// HIPAR timeSync/timing synchronisation.
ViStatus _VI_FUNC rsspecan_ConfigureWLANMIMOAntennaSignalCaptureTimeSync(
                            ViSession instrSession,
                            ViInt32   timeSync)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, NULL, RSSPECAN_ATTR_WLAN_MIMO_ANTENNA_SIGNAL_CAPTURE_TIME_SYNC, timeSync),
            2, "Time Sync");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function configures STC MIMO antenna assignment of the receive path.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init.
/// HIPAR rxAnalyzer/Defines the analyzer number.
/// HIPAR configuration/This control specifies the antenna assignment of the receive path.
/// HIPAR antennaState/Specifies ON/OFF state of the receive path.
/// HIPAR analyzerIPAddress/Specifies the TCP/IP address in IPV4 format.
ViStatus _VI_FUNC rsspecan_ConfWLANSTCMIMO(ViSession instrSession,
                                           ViInt32   rxAnalyzer,
                                           ViBoolean antennaState,
                                           ViString  analyzerIPAddress,
                                           ViInt32   configuration)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, rxAnalyzer, 1, 8),
            2, "Rx Analyzer");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, configuration, RSSPECAN_VAL_WLAN_ANTENNA1, RSSPECAN_VAL_WLAN_ANTENNA8),
            5, "Configuration");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "AntMatrix%ld", rxAnalyzer);

    if (rxAnalyzer > 1)
    {
        viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, repCap, RSSPECAN_ATTR_WLAN_ANALYZER_STATE, antennaState),
                3, "Antenna State");

        viCheckParm(rsspecan_SetAttributeViString(instrSession, repCap, RSSPECAN_ATTR_WLAN_ANALYZER_ADDRESS, analyzerIPAddress),
                4, "Analyzer IP Address");
    }

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_WLAN_ANTENNA_CONFIGURATION, configuration),
            5, "Configuration");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the connection state of the external reference for each
/// HIFN  channel.
/// HIFN     If the IP address of the antenna is not available or valid, or the
/// HIFN  selected antenna is not active, an error message is returned.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR rxAnalyzer/Defines the analyzer number.
/// HIPAR lanStatus/Queries the connection state of the external reference for each
/// HIPAR lanStatus/channel.
ViStatus _VI_FUNC rsspecan_QueryWLANMIMOLANStatus(ViSession  instrSession,
                                                  ViInt32    rxAnalyzer,
                                                  ViBoolean* lanStatus)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_WLAN_MIMO_LAN_STATUS,
            "AntMatrix", rxAnalyzer - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "RX Analyzer");

    viCheckParm(rsspecan_GetAttributeViBoolean(instrSession, repCap, RSSPECAN_ATTR_WLAN_MIMO_LAN_STATUS, lanStatus),
            3, "WLAN MIMO LAN Status");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures MIMO reference level offset settings.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR rxAnalyzer/Defines the analyzer number.
/// HIPAR coupling/This control determines whether the reference level for the master and
/// HIPAR coupling/slave devices in a simultaneous MIMO setup are coupled or not.
/// HIPAR offset/This control determines whether the reference value offset for the
/// HIPAR offset/specified antenna if the master and slave devices in a simultaneous
/// HIPAR offset/MIMO setup are not coupled.
ViStatus _VI_FUNC rsspecan_ConfWLANMIMOReferenceLevelOffset(ViSession instrSession,
                                                            ViInt32   rxAnalyzer,
                                                            ViInt32   coupling,
                                                            ViReal64  offset)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_WLAN_REFERENCE_LEVEL_OFFSET_COUPLING, coupling),
            3, "Coupling");

    if (coupling == RSSPECAN_VAL_WLAN_REFERENCE_LEVEL_OFFSET_MANUAL)
    {
        viCheckParm(RsCore_InvalidViInt32Range(instrSession, rxAnalyzer, 1, 8),
                2, "Rx Analyzer");

        snprintf(repCap, RS_REPCAP_BUF_SIZE, "AntMatrix%ld", rxAnalyzer);

        viCheckParm(rsspecan_SetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_WLAN_REFERENCE_LEVEL_OFFSET, offset),
                4, "Offset");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function determines whether the reference frequency for the
/// HIFN master and slave devices in a simultaneous MIMO setup are
/// HIFN coupled or not
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init.
/// HIPAR coupling/Determines whether the reference frequency for the master and
/// HIPAR coupling/slave devices in a simultaneous MIMO setup are coupled or not.
ViStatus _VI_FUNC rsspecan_ConfWLANMIMOReferenceFrequencyCoupling(
                            ViSession instrSession,
                            ViInt32   coupling)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_WLAN_REFERENCE_FREQUENCY_COUPLING, coupling),
            2, "Coupling");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function determines whether the amplitude settings for the master
/// HIFN  and slave devices in a simultaneous MIMO setup are coupled or not.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR coupling/This control determines whether the amplitude settings for the master
/// HIPAR coupling/and slave devices in a simultaneous MIMO setup are coupled or not.
ViStatus _VI_FUNC rsspecan_ConfWLANMIMOAmplitudeSettingsCoupling(
                            ViSession instrSession,
                            ViInt32   coupling)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_WLAN_AMPLITUDE_SETTINGS_COUPLING, coupling),
            2, "Coupling");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function configures how burst synchronization and tracking is performed
/// HIFN for multiple antennas.
/// HIRET Returns the status code of this operation.
/// HIRET You may use multiple lines for documentation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init
/// HIPAR instrSession/or rsspecan_InitWithOptions function.
/// HIPAR state/This control configures if antennas are synchronised and tracked
/// HIPAR state/together (ON) or separately (OFF).
ViStatus _VI_FUNC rsspecan_ConfWLANSimultaneousCaptureState(ViSession instrSession,
                                                            ViBoolean state)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_WLAN_ANTENNA_SIMULTANEOUS_CAPTURE_STATE, state),
            2, "State");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function configures the FS-K90 11n MIMO to determine if the payload length
/// HIFN should be taken from the signal field decoding result or from the signal.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init.
/// HIPAR source/This control specifies if the payload length should be taken
/// HIPAR source/from the signal field decoding result or from the signal.
ViStatus _VI_FUNC rsspecan_ConfWLANMIMOPayloadLengthSource(ViSession instrSession,
                                                           ViInt32   source)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_WLAN_MIMO_PAYLOAD_LENGTH_SOURCE, source),
            2, "Source");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function configures the mapping.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init.
/// HIPAR spatialMappingMode/This control is used to specify the mapping between streams and antennas.
/// HIPAR powerNormalize/Specifies whether an amplification of the signal power due to the spatial mapping is performed according to the matrix entries.
ViStatus _VI_FUNC rsspecan_ConfWLAN_MIMOSpatialMappingConfiguration(
                            ViSession instrSession,
                            ViInt32   spatialMappingMode,
                            ViBoolean powerNormalize)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_WLAN_MIMO_SPATIAL_MAPPING_MODE, spatialMappingMode),
            2, "Spatial Mapping Mode");

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_WLAN_MIMO_POWER_NORMALISE, powerNormalize),
            3, "Power Normalize");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function specifies User Defined Spatial Mapping - allows
/// HIFN the user to define his own spatial mapping between streams and
/// HIFN antennas.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init.
/// HIPAR TX/This control selects the antenna.
/// HIPAR realValues/Specifies the real data to mapping for all streams.
/// HIPAR imaginaryValues/Specifies the imaginary data to mapping for all streams.
/// HIPAR timeShift/This control specifies the timeshift in [ns] for a specific antenna.
ViStatus _VI_FUNC rsspecan_ConfWLAN_MIMO_UDFSpatialMappingAntenna(
                            ViSession instrSession,
                            ViInt32   TX,
                            ViReal64  realValues[],
                            ViReal64  imaginaryValues[],
                            ViReal64  timeShift)
{
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, TX, 1, 4),
            2, "TX");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CONF:WLAN:SMAP:TX%ld %.12lG,%.12lG, %.12lG,%.12lG, %.12lG,%.12lG, %.12lG,%.12lG, %.12lG", TX, realValues[0], imaginaryValues[0], realValues[1], imaginaryValues[1],
             realValues[2], imaginaryValues[2], realValues[3], imaginaryValues[3], timeShift);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function specifies User Defined Spatial Mapping for a
/// HIFN specific stream and antenna (real & imaginary data pair) and
/// HIFN User Defined timeshift for a specific antenna.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init.
/// HIPAR TX/This control selects the antenna.
/// HIPAR stream/This control selects the stream.
/// HIPAR realValue/Specifies the real data to mapping for a specific stream and antenna.
/// HIPAR imaginaryValue/Specifies the imaginary data to mapping for a specific stream and antenna.
/// HIPAR timeShift/This control specifies the timeshift in [ns] for a specific antenna.
ViStatus _VI_FUNC rsspecan_ConfWLAN_MIMO_UDFSpatialMappingStream(
                            ViSession instrSession,
                            ViInt32   TX,
                            ViInt32   stream,
                            ViReal64  realValue,
                            ViReal64  imaginaryValue,
                            ViReal64  timeShift)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, TX, 1, 4),
            2, "TX");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, stream, 1, 4),
            3, "Stream");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CONF:WLAN:SMAP:TX%ld:STR%ld %.12lG,%.12lG", TX, stream, realValue, imaginaryValue);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsspecan_CheckStatus(instrSession));

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "WLANTX%ld", TX);
    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_WLAN_MIMO_UDFSPATIAL_MAPPING_TIME_SHIFT, timeShift),
            6, "Time Shift");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects the I/Q data file to be used as input for further
/// HIFN  measurements. The I/Q data must have a specific format described in
/// HIFN  operating manual.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR fileName/String containing the path and name of the source file. The file
/// HIPAR fileName/extension is *.iq.tar.
/// HIPAR analysisBW/Optional. The analysis bandwidth to be used by the measurement. The
/// HIPAR analysisBW/bandwidth must be smaller than or equal to the bandwidth of the data
/// HIPAR analysisBW/that was stored in the file. If value is set to 0.0, the control is
/// HIPAR analysisBW/ignored.
ViStatus _VI_FUNC rsspecan_SelectWLANIQFile(ViSession instrSession,
                                            ViString  fileName,
                                            ViReal64  analysisBW)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "INP:FILE:PATH '%s',%.12G", fileName, analysisBW);
    checkErr(RsCore_Write(instrSession, cmd));
    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Returns the available channels in the currently loaded input file.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR arraySize/This parameter determines the size of array 'Channel List'.
/// HIPAR channelList/Returns the available channels in the currently loaded input file.
/// HIPAR returnedValues/Returns the number of returned values.
ViStatus _VI_FUNC rsspecan_QueryWLANIQFileChannelList(ViSession instrSession,
                                                      ViUInt32  arraySize,
                                                      ViChar    channelList[],
                                                      ViUInt32* returnedValues)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_QueryViStringUnknownLengthToUserBuffer(instrSession, "MMEM:LOAD:IQ:STR:LIST?", arraySize, channelList, (ViPInt32)returnedValues));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Only available for files that contain more than one data stream from
/// HIFN  multiple channels: selects which data stream in the file is used as
/// HIFN  input for the channel.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR autoInputEnabled/Only available for files that contain more than one data stream from
/// HIPAR autoInputEnabled/multiple channels: automatically defines which data stream in the file
/// HIPAR autoInputEnabled/is used as input for the channel.
/// HIPAR selectedDataStream/Only available for files that contain more than one data stream from
/// HIPAR selectedDataStream/multiple channels: selects the data stream to be used as input for the
/// HIPAR selectedDataStream/currently selected channel. Valid only if 'Auto Input Enabled' ==
/// HIPAR selectedDataStream/VI_FALSE.
ViStatus _VI_FUNC rsspecan_ConfigureWLANIQFileChannel(ViSession instrSession,
                                                      ViBoolean autoInputEnabled,
                                                      ViString  selectedDataStream)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, NULL, RSSPECAN_ATTR_WLAN_IQ_FILE_AUTO_INPUT_ENABLED, autoInputEnabled),
            2, "Auto Input Enabled");
    if (!autoInputEnabled)
    {
        viCheckParm(rsspecan_SetAttributeViString(instrSession, NULL, RSSPECAN_ATTR_WLAN_IQ_FILE_SELECTED_DATA_STREAM, selectedDataStream),
                3, "Selected Data Stream");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Determines how often the data stream is repeatedly copied in the I/Q
/// HIFN  data memory. If the available memory is not sufficient for the
/// HIFN  specified number of repetitions, the largest possible number of
/// HIFN  complete data streams is used.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR repetitionCount/Determines how often the data stream is repeatedly copied in the I/Q
/// HIPAR repetitionCount/data memory. If the available memory is not sufficient for the
/// HIPAR repetitionCount/specified number of repetitions, the largest possible number of
/// HIPAR repetitionCount/complete data streams is used.
ViStatus _VI_FUNC rsspecan_ConfigureWLANIQFileRepetitionCount(
                            ViSession instrSession,
                            ViInt32   repetitionCount)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, NULL, RSSPECAN_ATTR_WLAN_IQ_FILE_REPETITION_COUNT, repetitionCount),
            2, "Repetition Count");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the evaluation range statistics.
/// HIFN
/// HIFN     If  the statistic count is enabled, the specified number of PPDUs is
/// HIFN  taken into consideration for the statistical evaluation (maximally the
/// HIFN  number of PPDUs detected in the current capture buffer).
/// HIFN
/// HIFN     If disabled, all valid PPDUs in the current capture buffer are
/// HIFN  considered. Note that in this case, the number of PPDUs contributing to
/// HIFN  the current results may vary extremely.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR PPDUStatisticCount/Specifies whether a specified number of bursts is to be captured and
/// HIPAR PPDUStatisticCount/analyzed.
/// HIPAR noOfPPDUsToAnalyze/Sets the number of PPDUs for the statistical evaluation.
ViStatus _VI_FUNC rsspecan_ConfigureWLANEvaluationRangeStatistics(
                            ViSession instrSession,
                            ViBoolean PPDUStatisticCount,
                            ViInt32   noOfPPDUsToAnalyze)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_WLAN_BURS_COUN_STAT, PPDUStatisticCount),
            2, "PPDU Statistic Count");
    if (PPDUStatisticCount)
    {
        viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_WLAN_BURS_COUN, noOfPPDUsToAnalyze),
                3, "No Of PPDUs To Analyze");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function allows the user to specify the PPDUs to be analyzed. It
/// HIFN  provides also settings to adapt the synchronisation onto the channel
/// HIFN  conditions.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR sourceOfPayloadLength/This control is used to specify which signal source is used to
/// HIPAR sourceOfPayloadLength/determine the payload length of a PPDU.
/// HIPAR equalPPDULength/Enables/disables equal burst length to analyze.
/// HIPAR minNumberOfDataSymbols/Specifies the number of data symbols required for PPDUs to qualify for
/// HIPAR minNumberOfDataSymbols/measurement analysis.
/// HIPAR maxNumberOfDataSymbols/Specifies the maximum number of data symbols required for PPDUs to
/// HIPAR maxNumberOfDataSymbols/qualify for measurement analysis.
ViStatus _VI_FUNC rsspecan_ConfigureWLANEvaluationRangeTimeDomain(
                            ViSession instrSession,
                            ViInt32   sourceOfPayloadLength,
                            ViBoolean equalPPDULength,
                            ViInt32   minNumberOfDataSymbols,
                            ViInt32   maxNumberOfDataSymbols)
{
    ViStatus error = VI_SUCCESS;
    ViInt32  standard = 0;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsspecan_GetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_WLAN_STAN, &standard));

    if (standard != 0)
    {
        viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_WLAN_SOURCE_OF_PAYLOAD_LENGTH, sourceOfPayloadLength),
                2, "Source Of Payload Length");
    }

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_WLAN_SYM_EQU, equalPPDULength),
            3, "equal PPDU Length");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_WLAN_SYM_MIN, minNumberOfDataSymbols),
            4, "Min Number Of Data Symbols");

    if (equalPPDULength == VI_FALSE)
    {
        viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_WLAN_SYM_MAX, maxNumberOfDataSymbols),
                5, "Max Number Of Data Symbols");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the Time Domain Analysis Interval settings.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR offset/This control specifies the number of data symbols from the start of
/// HIPAR offset/each PPDU that are to be skipped before symbols take part in analysis.
/// HIPAR offset/For IEEE 802.11a, ac, g (OFDM), j, n, p signals only.
/// HIPAR lengthState/If enabled, the number of PPDU data symbols after the 'Offset' which
/// HIPAR lengthState/are to be analyzed. If disabled, all PPDU data symbols after the
/// HIPAR lengthState/'Offset' are evaluated.
/// HIPAR length/If the Length State control is set to false, this control specifies
/// HIPAR length/the number of PPDU data symbols after the Offset which are to be
/// HIPAR length/analyzed. If the Length State control is set to true, then this control
/// HIPAR length/has no effect.
ViStatus _VI_FUNC rsspecan_ConfigureWLANEvaluationRangeTimeDomainAnalysisInterval(
                            ViSession instrSession,
                            ViInt32   offset,
                            ViBoolean lengthState,
                            ViInt32   length)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_WLAN_ANALYSIS_INTERVAL_OFFSET, offset),
            2, "Offset");

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_WLAN_ANALYSIS_INTERVAL_LENGTH_STATE, lengthState),
            3, "Length State");

    if (lengthState == VI_FALSE)
    {
        viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_WLAN_ANALYSIS_INTERVAL_LENGTH, length),
                4, "Length");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function defines the number of samples and sets the
/// HIFN  reference for the rise and fall time in PVT calculation.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR averageLength/Defines the number of samples.
/// HIPAR referencePower/Configures the use of either mean or maximum PPDU power as a reference power.
ViStatus _VI_FUNC rsspecan_ConfigureWLANEvaluationRangePVT(ViSession instrSession,
                                                           ViInt32   averageLength,
                                                           ViInt32   referencePower)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_WLAN_PVT_AVER, averageLength),
            2, "Average Length");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_WLAN_PVT_RPOW, referencePower),
            3, "Reference Power");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the measurement range for the peak vector error.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR measRange/Specifies whether the Peak Error Vector results are calculated
/// HIPAR measRange/over the complete burst or just over the PSDU.
ViStatus _VI_FUNC rsspecan_ConfigureWlanEvaluationRangeMeasRange(
                            ViSession instrSession,
                            ViInt32   measRange)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K91"));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_WLAN_PEAK_ERROR_VECTOR_MEAS_RANGE, measRange),
            2, "Meas Range");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/*****************************************************************************
 * Function: Configure WLAN Gain Imbalance Unit
 * Purpose:  This function specifies the units for gain imbalance results.
*****************************************************************************/
ViStatus _VI_FUNC rsspecan_ConfigureWlanGainImbalanceUnit(ViSession instrSession,
                                                          ViInt32   unit)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_WLAN_UNIT_GIMB, unit),
            2, "Unit");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function specifies the units for EVM limits and results.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR unit/This parameter specifies the units for EVM limits and results.
ViStatus _VI_FUNC rsspecan_ConfigureWlanEVMUnit(ViSession instrSession,
                                                ViInt32   unit)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_WLAN_UNIT_EVM, unit),
            2, "Unit");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/*****************************************************************************
 * Function: Configure WLAN Display
 * Purpose:  This function configures the display parameters.
*****************************************************************************/
ViStatus _VI_FUNC rsspecan_ConfigureWlanDisplay(ViSession instrSession,
                                                ViBoolean displayTable)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "Win1", RSSPECAN_ATTR_WLAN_DISP_TABL, displayTable),
            2, "Display Table");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function defines which items are displayedin the Result Summary.
/// HIFN  Note that the results are always calculated, regardless of their
/// HIFN  visibility in the Result Summary.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR item/Selects the item to be included in Result Summary.
/// HIPAR displayItem/This parameter selects whether the results summary item is displayed.
ViStatus _VI_FUNC rsspecan_ConfigureWlanResultSummaryDisplay(ViSession instrSession,
                                                             ViInt32   reserved,
                                                             ViInt32   item,
                                                             ViBoolean displayItem)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";
    ViChar*  p2TableString;
             reserved;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentModel(instrSession, "FSW"));

    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &rsspecan_rngWLANTableItem, item, &p2TableString),
                3, "Item");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "DISP:WIND:TABL:ITEM %s,%s", p2TableString, displayItem ? "ON" : "OFF");
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Specifies the degree of the polynomial regression model used to
/// HIFN  determine the AM/AM result display.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR polynomialDegree/Specifies the degree of the polynomial regression model used to
/// HIPAR polynomialDegree/determine the AM/AM result display.
ViStatus _VI_FUNC rsspecan_ConfigureWLANPolynomialDegree(ViSession instrSession,
                                                         ViInt32   polynomialDegree)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, NULL, RSSPECAN_ATTR_WLAN_POLYNOMIAL_DEGREE, polynomialDegree),
            2, "Polynomial Degree");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Activates or deactivates automatic scaling of the x-axis or y-axis for
/// HIFN  the specified trace display. If enabled, the R&S FSW WLAN application
/// HIFN  automatically scales the x-axis or y-axis to best fit the measurement
/// HIFN  results.
/// HIFN     If disabled, the x-axis or y-axis is scaled according to the specified
/// HIFN  minimum/maximum values and number of divisions.
/// HIFN     This function is currently only supported for Amplitude Modulation
/// HIFN  measurements.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR autoscaleEnabled/Activates or deactivates automatic scaling of the x-axis or y-axis for
/// HIPAR autoscaleEnabled/the specified trace display. If enabled, the R&S FSW WLAN application
/// HIPAR autoscaleEnabled/automatically scales the x-axis or y-axis to best fit the measurement
/// HIPAR autoscaleEnabled/results.
/// HIPAR axis/Selects the axis to configure.
/// HIPAR window/This control selects the measurement window.
/// HIPAR autoMode/Determines which algorithm is used to determine whether the x-axis or
/// HIPAR autoMode/y-axis requires automatic rescaling.
/// HIPAR autoFixRange/Defines the use of fixed value limits. This control is currently only
/// HIPAR autoFixRange/supported for Amplitude Modulation measurements.
ViStatus _VI_FUNC rsspecan_ConfigureWLANAMAMAutoscale(ViSession instrSession,
                                                      ViInt32   window,
                                                      ViInt32   axis,
                                                      ViBoolean autoscaleEnabled,
                                                      ViInt32   autoMode,
                                                      ViInt32   autoFixRange)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, axis, 0, 1),
            3, "Axis");

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_WLAN_AM_AM_X_AXIS_AUTOSCALE_ENABLED,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, repCap,
                    axis ? RSSPECAN_ATTR_WLAN_AM_AM_Y_AXIS_AUTOSCALE_ENABLED : RSSPECAN_ATTR_WLAN_AM_AM_X_AXIS_AUTOSCALE_ENABLED, autoscaleEnabled), 4, "Autoscale Enabled");
    if (autoscaleEnabled)
    {
        viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap,
                        axis ? RSSPECAN_ATTR_WLAN_AM_AM_Y_AXIS_AUTO_MODE : RSSPECAN_ATTR_WLAN_AM_AM_X_AXIS_AUTO_MODE, autoMode), 5, "Auto Mode");

        viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap,
                        axis ? RSSPECAN_ATTR_WLAN_AM_AM_Y_AXIS_AUTO_FIX_RANGE : RSSPECAN_ATTR_WLAN_AM_AM_X_AXIS_AUTO_FIX_RANGE, autoFixRange), 6, "Auto Fix Range");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the range to be displayed on the x-axis or y-axis of the
/// HIFN  specified evaluation diagram.
/// HIFN     This function is currently only supported for Amplitude Modulation
/// HIFN  measurements.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR axis/Selects the axis to configure.
/// HIPAR window/This control selects the measurement window.
/// HIPAR min/Defines the minimum value to be displayed on the axis of the specified
/// HIPAR min/evaluation diagram.
/// HIPAR max/Defines the maximum value to be displayed on the axis of the specified
/// HIPAR max/evaluation diagram.
ViStatus _VI_FUNC rsspecan_ConfigureWLANAMAMFixed(ViSession instrSession,
                                                  ViInt32   window,
                                                  ViInt32   axis,
                                                  ViReal64  min,
                                                  ViReal64  max)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, axis, 0, 1),
            3, "Axis");
    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_WLAN_AM_AM_X_AXIS_AUTO_FIX_RANGE_MIN,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap));

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, repCap,
                    axis ? RSSPECAN_ATTR_WLAN_AM_AM_Y_AXIS_AUTO_FIX_RANGE_MIN : RSSPECAN_ATTR_WLAN_AM_AM_X_AXIS_AUTO_FIX_RANGE_MIN, min), 3, "Min");

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, repCap,
                    axis ? RSSPECAN_ATTR_WLAN_AM_AM_Y_AXIS_AUTO_FIX_RANGE_MAX : RSSPECAN_ATTR_WLAN_AM_AM_X_AXIS_AUTO_FIX_RANGE_MAX, max), 4, "Max");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures hysteris.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR axis/Selects the axis to configure.
/// HIPAR lowerLower/For automatic scaling based on hysteresis, this control defines the
/// HIPAR lowerLower/lower limit of the lower hysteresis interval.
/// HIPAR lowerUpper/For automatic scaling based on hysteresis, this control defines the
/// HIPAR lowerUpper/upper limit of the lower hysteresis interval.
/// HIPAR upperLower/For automatic scaling based on hysteresis, this control defines the
/// HIPAR upperLower/lower limit of the upper hysteresis interval.
/// HIPAR upperUpper/For automatic scaling based on hysteresis, this control defines the
/// HIPAR upperUpper/upper limit of the upper hysteresis interval.
ViStatus _VI_FUNC rsspecan_ConfigureWLANAMAMAutoHysteresis(ViSession instrSession,
                                                           ViInt32   window,
                                                           ViInt32   axis,
                                                           ViReal64  lowerLower,
                                                           ViReal64  lowerUpper,
                                                           ViReal64  upperLower,
                                                           ViReal64  upperUpper)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, axis, 0, 1),
            3, "Axis");
    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_WLAN_AM_AM_Y_AXIS_AUTO_HYSTERESIS_LOWER_LOWER,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap));
    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, repCap,
                    axis ? RSSPECAN_ATTR_WLAN_AM_AM_Y_AXIS_AUTO_HYSTERESIS_LOWER_LOWER : RSSPECAN_ATTR_WLAN_AM_AM_X_AXIS_AUTO_HYSTERESIS_LOWER_LOWER, lowerLower), 4, "Lower Lower");

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, repCap,
                    axis ? RSSPECAN_ATTR_WLAN_AM_AM_Y_AXIS_AUTO_HYSTERESIS_LOWER_UPPER : RSSPECAN_ATTR_WLAN_AM_AM_X_AXIS_AUTO_HYSTERESIS_LOWER_UPPER, lowerUpper), 5, "Lower Upper");

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, repCap,
                    axis ? RSSPECAN_ATTR_WLAN_AM_AM_Y_AXIS_AUTO_HYSTERESIS_UPPER_LOWER : RSSPECAN_ATTR_WLAN_AM_AM_X_AXIS_AUTO_HYSTERESIS_UPPER_LOWER, upperLower), 6, "Upper Lower");

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, repCap,
                    axis ? RSSPECAN_ATTR_WLAN_AM_AM_Y_AXIS_AUTO_HYSTERESIS_UPPER_UPPER : RSSPECAN_ATTR_WLAN_AM_AM_X_AXIS_AUTO_HYSTERESIS_UPPER_UPPER, upperUpper), 7, "Upper Upper");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures memory depth.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR axis/Selects the axis to configure.
/// HIPAR memoryDepth/For automatic scaling based on memory, this value defines the number
/// HIPAR memoryDepth/<x> of previous results to be considered when determining if rescaling
/// HIPAR memoryDepth/is required.
ViStatus _VI_FUNC rsspecan_ConfigureWLANAMAMMemoryDepth(ViSession instrSession,
                                                        ViInt32   window,
                                                        ViInt32   axis,
                                                        ViInt32   memoryDepth)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, axis, 0, 1),
            3, "Axis");
    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_WLAN_AM_AM_Y_AXIS_AUTO_HYSTERESIS_MEMORY_DEPTH,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap));
    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap,
                    axis ? RSSPECAN_ATTR_WLAN_AM_AM_Y_AXIS_AUTO_HYSTERESIS_MEMORY_DEPTH : RSSPECAN_ATTR_WLAN_AM_AM_X_AXIS_AUTO_HYSTERESIS_MEMORY_DEPTH, memoryDepth), 4, "Memory Depth");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the number of divisions to be used for the axis in the
/// HIFN  specified window.
/// HIFN     Separate division settings can be configured for individual result
/// HIFN  displays.
/// HIFN     This attribute is currently only supported for Amplitude Modulation
/// HIFN  measurements.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR axis/Selects the axis to configure.
/// HIPAR numberOfDivisions/Defines the number of divisions to be used for the x-axis in the
/// HIPAR numberOfDivisions/specified window.
ViStatus _VI_FUNC rsspecan_ConfigureWLANAMAMNumberOfDivisions(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   axis,
                            ViInt32   numberOfDivisions)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, axis, 0, 1),
            3, "Axis");

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_WLAN_AM_AM_Y_AXIS_NUMBER_OF_DIVISIONS,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap));
    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap,
                    axis ? RSSPECAN_ATTR_WLAN_AM_AM_Y_AXIS_NUMBER_OF_DIVISIONS : RSSPECAN_ATTR_WLAN_AM_AM_X_AXIS_NUMBER_OF_DIVISIONS, numberOfDivisions), 4, "Number Of Divisions");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Determines the values shown for each division on the axis in the
/// HIFN  specified window.
/// HIFN     One or more multiples of 10n can be selected. The R&S FSW WLAN
/// HIFN  application then selects the optimal scaling from the selected values.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR axis/Selects the axis to configure.
/// HIPAR arraySize/Sets the number of elements of the Scaling Per Division array to be
/// HIPAR arraySize/used.
/// HIPAR scalingPerDivision/Determines the values shown for each division on the y-axis in the
/// HIPAR scalingPerDivision/specified window. One or more multiples of 10^n can be selected.
ViStatus _VI_FUNC rsspecan_ConfigureWLANAMAMScalingPerDivision(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   axis,
                            ViInt32   arraySize,
                            ViInt32   scalingPerDivision[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";
    ViChar*  p2buf;
    ViInt32  i = 0;
    ViString scalingArray[] = {"1.0", "2.0", "2.5", "5.0"};

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, window, 0, 16),
            2, "Window");

    p2buf = cmd + snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "DISP:WIND%ld:TRAC:%c:SCAL:PDIV ", window, axis ? 'Y' : 'X');

    for (i = 0; i < arraySize; i++)
        p2buf += sprintf(p2buf, "%s,", scalingArray[scalingPerDivision[i]]);

    *p2buf = '\0';

    checkErr(RsCore_Write(instrSession, cmd));
    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/*****************************************************************************
 * Function: Configure WLAN Limit
 * Purpose:  This function configures the limit.
 *
 *           Note:
 *           (1) This function is not valid for standard IEEE 802.11b
*****************************************************************************/
ViStatus _VI_FUNC rsspecan_ConfigureWlanLimit(ViSession instrSession,
                                              ViInt32   reserved,
                                              ViInt32   reserved2,
                                              ViInt32   limitType,
                                              ViInt32   mode,
                                              ViReal64  value)
{
    ViStatus error = VI_SUCCESS;
             reserved;
             reserved2;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, limitType, RSSPECAN_VAL_WLAN_LIM_FERR, RSSPECAN_VAL_WLAN_LIM_EVM),
            4, "Limit Type");

    if (mode == RSSPECAN_VAL_MEASTYPE_MAX)
    {
        switch (limitType)
        {
        case RSSPECAN_VAL_WLAN_LIM_FERR:
            viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_WLAN_FERR_MAX, value),
                    6, "Value");
            break;
        case RSSPECAN_VAL_WLAN_LIM_SYMB:
            viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_WLAN_SYMB_MAX, value),
                    6, "Value");
            break;
        case RSSPECAN_VAL_WLAN_LIM_IQOFF:
            viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_WLAN_IQOF_MAX, value),
                    6, "Value");
            break;
        case RSSPECAN_VAL_WLAN_LIM_EVM_ALL:
            viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_WLAN_EVM_ALL_MAX, value),
                    6, "Value");
            break;
        case RSSPECAN_VAL_WLAN_LIM_EVM_DATA:
            viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_WLAN_EVM_DATA_MAX, value),
                    6, "Value");
            break;
        case RSSPECAN_VAL_WLAN_LIM_EVM_PIL:
            viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_WLAN_EVM_PIL_MAX, value),
                    6, "Value");
            break;
        case RSSPECAN_VAL_WLAN_LIM_RTIME:
            viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_WLAN_TRIS_MAXIMUM, value),
                    6, "Value");
            break;
        case RSSPECAN_VAL_WLAN_LIM_FTIME:
            viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_WLAN_TFAL_MAXIMUM, value),
                    6, "Value");
            break;
        case RSSPECAN_VAL_WLAN_LIM_EVM:
            viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_WLAN_EVM_MAXIMUM, value),
                    6, "Value");
            break;
        }
    }
    else if (mode == RSSPECAN_VAL_MEASTYPE_AVER)
    {
        switch (limitType)
        {
        case RSSPECAN_VAL_WLAN_LIM_FERR:
            viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_WLAN_FERR_AVERAGE, value),
                    6, "Value");
            break;
        case RSSPECAN_VAL_WLAN_LIM_SYMB:
            viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_WLAN_SYMB_AVERAGE, value),
                    6, "Value");
            break;
        case RSSPECAN_VAL_WLAN_LIM_IQOFF:
            viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_WLAN_IQOF_AVERAGE, value),
                    6, "Value");
            break;
        case RSSPECAN_VAL_WLAN_LIM_EVM_ALL:
            viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_WLAN_EVM_ALL_AVERAGE, value),
                    6, "Value");
            break;
        case RSSPECAN_VAL_WLAN_LIM_EVM_DATA:
            viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_WLAN_EVM_DATA_AVERAGE, value),
                    6, "Value");
            break;
        case RSSPECAN_VAL_WLAN_LIM_EVM_PIL:
            viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_WLAN_EVM_PIL_AVERAGE, value),
                    6, "Value");
            break;
        case RSSPECAN_VAL_WLAN_LIM_RTIME:
            viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_WLAN_TRIS, value),
                    6, "Value");
            break;
        case RSSPECAN_VAL_WLAN_LIM_FTIME:
            viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_WLAN_TFAL_AVERAGE, value),
                    6, "Value");
            break;
        case RSSPECAN_VAL_WLAN_LIM_EVM:
            viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_WLAN_EVM_AVERAGE, value),
                    6, "Value");
            break;
        }
    }
    else
    {
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, mode),
                    5, "Mode");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/*****************************************************************************
 * Function: Configure WLAN All Limits
 * Purpose:  This function sets all limits.
*****************************************************************************/
ViStatus _VI_FUNC rsspecan_ConfigureWLANAllLimits(ViSession instrSession,
                                                  ViInt32   reserved,
                                                  ViInt32   reserved2,
                                                  ViInt32   arraySize,
                                                  ViReal64  values[])
{
    ViStatus error = VI_SUCCESS;
             reserved;
             reserved2;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K91"));

    viCheckParm(RsCore_InvalidNullPointer(instrSession, values),
                5, "Values");

    checkErr(RsCore_WriteAsciiViReal64Array(instrSession, "CALC:LIM:BURSt:ALL ", values, arraySize));

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/*****************************************************************************
 * Function: Get WLAN All Limits
 * Purpose:  This function returns all limits.
*****************************************************************************/
ViStatus _VI_FUNC rsspecan_GetWlanAllLimits(ViSession instrSession,
                                            ViInt32   reserved,
                                            ViInt32   reserved2,
                                            ViReal64  values[])
{
    ViStatus error = VI_SUCCESS;
             reserved;
             reserved2;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K90|K91"));

    viCheckParm(RsCore_InvalidNullPointer(instrSession, values),
                2, "Values");

    checkErr(RsCore_QueryFloatArrayToUserBuffer(instrSession, ":CALC:LIM:BURS:ALL?", 12, values, NULL));

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function returns the ACP channel limit.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channelType/Sets the ACP channel type.
/// HIPAR value/Returns the ACP limit value.
ViStatus _VI_FUNC rsspecan_QueryWlanACPChannelLimit(ViSession instrSession,
                                                    ViInt32   channelType,
                                                    ViReal64* value)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, channelType, RSSPECAN_VAL_WLAN_ACP_ADJ, RSSPECAN_VAL_WLAN_ACP_ALT),
            2, "Channel Type");

    switch (channelType)
    {
    case RSSPECAN_VAL_WLAN_ACP_ADJ:
        viCheckParm(rsspecan_GetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_WLAN_ACP_ADJ, value),
                3, "Value");
        break;
    case RSSPECAN_VAL_WLAN_ACP_ALT:
        viCheckParm(rsspecan_GetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_WLAN_ACP_ALT, value),
                3, "Value");
        break;
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function returns the value at the maximum overstepping of the spectrum mask limits.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channelType/Sets the value type.
/// HIPAR value/Returns the value.
ViStatus _VI_FUNC rsspecan_QueryWlanSpectrumMaxOverstepping(ViSession instrSession,
                                                            ViInt32   channelType,
                                                            ViReal64* value)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, channelType, RSSPECAN_VAL_FREQ, RSSPECAN_VAL_POW),
            2, "Channel Type");

    switch (channelType)
    {
    case RSSPECAN_VAL_FREQ:
        viCheckParm(rsspecan_GetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_WLAN_SPEC_MASK_X, value),
                3, "Value");
        break;
    case RSSPECAN_VAL_POW:
        viCheckParm(rsspecan_GetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_WLAN_SPEC_MASK_Y, value),
                3, "Value");
        break;
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function selects a common RF measurement for WLAN signals.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR RFMeasurement/Selects the RF measurement.
ViStatus _VI_FUNC rsspecan_SelectWlanRFMeasurement(ViSession instrSession,
                                                   ViInt32   RFMeasurement)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    switch (RFMeasurement)
    {
    case RSSPECAN_VAL_WLAN_MEAS_ACPR:
        checkErr(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_WLAN_SPEC_ACPR, NULL));
        break;
    case RSSPECAN_VAL_WLAN_MEAS_MASK:
        checkErr(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_WLAN_SPEC_MASK, NULL));
        break;
    case RSSPECAN_VAL_WLAN_MEAS_OBAN:
        checkErr(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_WLAN_SPEC_OCCUPIED_BANDWIDTH, NULL));
        break;
    case RSSPECAN_VAL_WLAN_MEAS_CCDF:
        checkErr(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_WLAN_STAT_CCDF, NULL));
        break;
    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, RFMeasurement), 2, "RF Measurement");
        break;
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function selects a IQ measurement for WLAN signals.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR measurement/Selects the IQ measurement.
ViStatus _VI_FUNC rsspecan_SelectWlanIQMeasurement(ViSession instrSession,
                                                   ViInt32   measurement)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    switch (measurement)
    {
    case RSSPECAN_VAL_WLAN_IQ_MEAS_GCAR:
        checkErr(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_WLAN_GAIN_IMBALANCE_VS_CARRIER, NULL));
        break;
    case RSSPECAN_VAL_WLAN_IQ_MEAS_PTR:
        checkErr(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_WLAN_PHASE_TRACKING_VS_SYMBOL, NULL));
        break;
    case RSSPECAN_VAL_WLAN_IQ_MEAS_QCAR:
        checkErr(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_WLAN_QUADRATURE_ERROR_VS_CARRIER, NULL));
        break;
    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, measurement), 2, "Measurement");
        break;
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function sets measurement type to be Power Versus Time and configures measurement parameters.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR risingFalling/Sets the power versus time measurement results interpreting.
ViStatus _VI_FUNC rsspecan_ConfWlanPVTMeas(ViSession instrSession,
                                           ViInt32   risingFalling)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, risingFalling, RSSPECAN_VAL_PVT_EDGE, RSSPECAN_VAL_PVT_FALL),
            2, "Channel Type");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_WLAN_PVT_SEL, risingFalling),
            2, "Rising/Falling");
    checkErr(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_WLAN_PVT, ""));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function sets measurement type to be EVM.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR mode/Sets the EVT measurement mode.
/// HIPAR autoScaling/This parameter switches on or off automatic scaling.
/// HIPAR perDivision/This parameter sets the size of each Y scale division.
/// HIPAR unit/This parameter specifies the units for EVM trace results.
ViStatus _VI_FUNC rsspecan_ConfWlanEVMMeasurement(ViSession instrSession,
                                                  ViInt32   mode,
                                                  ViBoolean autoScaling,
                                                  ViReal64  perDivision,
                                                  ViInt32   unit)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, mode, RSSPECAN_VAL_WLAN_EVM_MODE_SYMBOL, RSSPECAN_VAL_WLAN_EVM_MODE_CHIP),
            2, "Mode");

    switch (mode)
    {
    case RSSPECAN_VAL_WLAN_EVM_MODE_SYMBOL:
        checkErr(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_WLAN_EVM_ESYM, ""));
        viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_WLAN_UNIT_EVM, unit),
                5, "Unit");
        break;
    case RSSPECAN_VAL_WLAN_EVM_MODE_CARRIER:
        checkErr(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_WLAN_EVM_ECAR, ""));
        viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_WLAN_UNIT_EVM, unit),
                5, "Unit");
        break;
    case RSSPECAN_VAL_WLAN_EVM_MODE_PHAS:
        checkErr(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_WLAN_PRE, ""));
        checkErr(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_WLAN_PRE_SEL, RSSPECAN_VAL_PRE_PHAS));
        break;
    case RSSPECAN_VAL_WLAN_EVM_MODE_FREQ:
        checkErr(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_WLAN_PRE, ""));
        checkErr(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_WLAN_PRE_SEL, RSSPECAN_VAL_PRE_FREQ));
        break;
    case RSSPECAN_VAL_WLAN_EVM_MODE_CHIP:
        if (rsspecan_IsFSV(instrSession))
        {
            checkErr(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_WLAN_EVM_ECHIP, ""));
        }
        else
        {
            checkErr(rsspecan_ReplaceWindow(instrSession, "1", RSSPECAN_VAL_LAYOUT_TYPE_EVM_CHIP));
        }
        viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_WLAN_UNIT_EVM, unit),
                5, "Unit");
        break;
    }

    if (RsCore_IsInstrumentModel(instrSession, "FSW"))
    {
        viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_WLAN_DISP_AUTO_Y, autoScaling),
                3, "Auto Scaling");

        viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_WLAN_Y_PDIV, perDivision),
                4, "Per Division");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function sets measurement type to be Spectrum.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR mode/Sets the spectrum measurement mode.
ViStatus _VI_FUNC rsspecan_ConfWlanSpectrumMeasurement(ViSession instrSession,
                                                       ViInt32   mode)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, mode, RSSPECAN_VAL_WLAN_SPEC_MODE_IEEE, RSSPECAN_VAL_WLAN_SPEC_MODE_SEL_GRD),
            2, "Mode");

    switch (mode)
    {
    case RSSPECAN_VAL_WLAN_SPEC_MODE_IEEE:
        checkErr(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_WLAN_SPEC_MASK_SEL, RSSPECAN_VAL_SPEC_IEEE));
        checkErr(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_WLAN_SPEC_MASK, ""));
        break;
    case RSSPECAN_VAL_WLAN_SPEC_MODE_ETSI:
        checkErr(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_WLAN_SPEC_MASK_SEL, RSSPECAN_VAL_SPEC_ETSI));
        checkErr(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_WLAN_SPEC_MASK, ""));
        break;
    case RSSPECAN_VAL_WLAN_SPEC_MODE_FFT:
        checkErr(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_WLAN_SPEC_FFT, ""));
        break;
    case RSSPECAN_VAL_WLAN_SPEC_MODE_ACPR:
        checkErr(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_WLAN_SPEC_ACPR, ""));
        break;
    case RSSPECAN_VAL_WLAN_SPEC_MODE_FLAT:
        checkErr(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_WLAN_SPEC_FLAT, ""));
        break;
    case RSSPECAN_VAL_WLAN_SPEC_MODE_SEL_FLAT:
        checkErr(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_WLAN_SPEC_SEL, RSSPECAN_VAL_WLAN_SPEC_MODE_SEL_FLAT));
        break;
    case RSSPECAN_VAL_WLAN_SPEC_MODE_SEL_GRD:
        checkErr(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_WLAN_SPEC_SEL, RSSPECAN_VAL_WLAN_SPEC_MODE_SEL_GRD));
        break;
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function sets the Spectrum Flatness channel selection to
/// HIFN either Effective or Physical.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init.
/// HIPAR mode/Sets the sets the Spectrum Flatness channel selection to either Effective or Physical. The driver uses this value to set the RSSPECAN_ATTR_WLAN_SPEC_CSEL attribute.
ViStatus _VI_FUNC rsspecan_ConfWLANChannelSelect(ViSession instrSession,
                                                 ViInt32   mode)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_WLAN_SPEC_CSEL, mode),
            2, "Mode");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the Constellation measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR constelationMode/Sets the measurement type.
/// HIPAR carrierSelection/Defines the result calculation of the selected carrier.
/// HIPAR carrierNumber/Defines the carrier number.
ViStatus _VI_FUNC rsspecan_ConfWlanConstellationMeasurement(ViSession instrSession,
                                                            ViInt32   constelationMode,
                                                            ViInt32   carrierSelection,
                                                            ViInt32   carrierNumber)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, constelationMode, RSSPECAN_VAL_WLAN_CONS_SYM, RSSPECAN_VAL_WLAN_CONS_CARR),
            2, "Constelation Mode");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, carrierSelection, RSSPECAN_VAL_SELECTION_ALL, RSSPECAN_VAL_SELECTION_NUM),
            3, "Carrier Selection");

    switch (constelationMode)
    {
    case RSSPECAN_VAL_WLAN_CONS_SYM:
        checkErr(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_WLAN_CONS_CSYM, ""));
        switch (carrierSelection)
        {
        case RSSPECAN_VAL_SELECTION_ALL:
            viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_WLAN_CONS_CARR_SEL, RSSPECAN_VAL_SELECTION_ALL),
                    3, "Carrier Selection");
            break;
        case RSSPECAN_VAL_SELECTION_PIL:
            viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_WLAN_CONS_CARR_SEL, RSSPECAN_VAL_SELECTION_PIL),
                    3, "Carrier Selection");
            break;
        case RSSPECAN_VAL_SELECTION_NUM:
            viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_WLAN_CONS_CARR_SEL_NR, carrierNumber),
                    4, "Carrier Number");
            break;
        }
        break;
    case RSSPECAN_VAL_WLAN_CONS_CARR:
        checkErr(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_WLAN_CONS_CCAR, ""));
        break;
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the statistics measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR mode/Sets the measurement type.
ViStatus _VI_FUNC rsspecan_ConfWlanStatisticsMeasurement(ViSession instrSession,
                                                         ViInt32   mode)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, mode, RSSPECAN_VAL_WLAN_STAT_CCDF, RSSPECAN_VAL_WLAN_STAT_SFI),
            2, "Mode");

    switch (mode)
    {
    case RSSPECAN_VAL_WLAN_STAT_CCDF:
        checkErr(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_WLAN_STAT_CCDF, ""));
        break;
    case RSSPECAN_VAL_WLAN_STAT_BSTR:
        checkErr(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_WLAN_STAT_BSTR, ""));
        break;
    case RSSPECAN_VAL_WLAN_STAT_SFI:
        checkErr(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_WLAN_STAT_SFI, ""));
        break;
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the Spectrum Emission Mask measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR type/Sets the Spectrum Emission Mask (SEM) measurement type.
/// HIPAR fileName/This control is used to load a K91 spectrum emission mask setup from an xml file.
/// HIPAR powerClass/This control sets the Spectrum Emission Mask (SEM) power class index.
ViStatus _VI_FUNC rsspecan_ConfWlanSEMMeasurement(ViSession instrSession,
                                                  ViInt32   type,
                                                  ViString  fileName,
                                                  ViInt32   powerClass)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_WLAN_SEM_TYPE, type),
            2, "Type");
    if (type == RSSPECAN_VAL_SEM_USER)
    {
        viCheckParm(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_WLAN_SEM_FILE, fileName),
                3, "File Name");
    }
    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_WLAN_SEM_CLASS, powerClass),
            4, "Power Class");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the channel bandwidth to be applied for the Spectrum Emission
/// HIFN  Mask measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channelBandwidth/Sets the channel bandwidth to be applied for the Spectrum Emission
/// HIPAR channelBandwidth/Mask measurement.
ViStatus _VI_FUNC rsspecan_ConfigureWLANSEMChannelBandwidth(ViSession instrSession,
                                                            ViInt32   channelBandwidth)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, NULL, RSSPECAN_ATTR_WLAN_SPECTRUM_EMISSION_MASK_CHANNEL_BANDWIDTH, channelBandwidth),
            2, "Channel Bandwidth");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function switches between relative (dB) and absolute (dBm) results for Spectrum Flatness results.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR type/Sets the Spectrum Emission Mask (SEM) measurement type.
/// HIPAR units/Selects between relative (dB) and absolute (dBm) results for Spectrum Flatness results.
ViStatus _VI_FUNC rsspecan_ConfigureWLANSpectrumFlatnessUnit(ViSession instrSession,
                                                             ViInt32   units)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_WLAN_SPECTRUM_FLATNESS_UNIT, units),
            2, "Units");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the Spectrum Emission Mask measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR measurement/Sets the measured and referenced signal for result display.
ViStatus _VI_FUNC rsspecan_ConfWlanAMMeasurement(ViSession instrSession,
                                                 ViInt32   measurement)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    switch (measurement)
    {
    case RSSPECAN_VAL_WLAN_AM_AM:
        checkErr(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_WLAN_AM_AM, ""));
        break;
    case RSSPECAN_VAL_WLAN_AM_PM:
        checkErr(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_WLAN_AM_PM, ""));
        break;
    case RSSPECAN_VAL_WLAN_AM_EVM:
        checkErr(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_WLAN_AM_EVM, ""));
        break;
    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, measurement), 2, "Register Operation");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function sets the ACP measurement mode to either absolute or relative.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR acpMode/This control sets the ACP measurement mode
ViStatus _VI_FUNC rsspecan_ConfWlanACPMode(ViSession instrSession,
                                           ViInt32   acpMode)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_WLAN_ACH_MODE, acpMode),
            2, "ACP Mode");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Determines whether a selected PPDU is considered or ignored.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR ppduSelectedEnabled/Determines whether a selected PPDU is considered or ignored.
ViStatus _VI_FUNC rsspecan_ConfigureWLANPPDUSelectedEnabled(ViSession instrSession,
                                                            ViBoolean ppduSelectedEnabled)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, NULL, RSSPECAN_ATTR_WLAN_PPDU_SELECT_STATE, ppduSelectedEnabled),
            2, "PPDU Selected Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the Hysteresis Threshold.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR lower/When the reference level is adjusted automatically, the internal
/// HIPAR lower/attenuators and the preamplifier are also adjusted. To avoid frequent
/// HIPAR lower/adaptation due to small changes in the input signal, you can define a
/// HIPAR lower/hysteresis. This setting defines a lower threshold the signal must fall
/// HIPAR lower/below (compared to the last measurement) before the reference level is
/// HIPAR lower/adapted automatically.
/// HIPAR upper/When the reference level is adjusted automatically, the internal
/// HIPAR upper/attenuators and the preamplifier are also adjusted. To avoid frequent
/// HIPAR upper/adaptation due to small changes in the input signal, you can define a
/// HIPAR upper/hysteresis. This setting defines a upper threshold the signal must fall
/// HIPAR upper/below (compared to the last measurement) before the reference level is
/// HIPAR upper/adapted automatically.
ViStatus _VI_FUNC rsspecan_ConfigureWLANHysteresisThreshold(ViSession instrSession,
                                                            ViReal64  lower,
                                                            ViReal64  upper)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, NULL, RSSPECAN_ATTR_WLAN_HYSTERESIS_THRESHOLD_LOWER, lower),
            2, "Lower");

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, NULL, RSSPECAN_ATTR_WLAN_HYSTERESIS_THRESHOLD_UPPER, upper),
            3, "Upper");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the status of the analyzed PPDUs in the current capture
/// HIFN  buffer.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR ppduStatus/Queries the status of the analyzed PPDUs in the current capture
/// HIPAR ppduStatus/buffer.
/// HIPAR arraySize/Number of array points requested.
/// HIPAR noOfValues/Returns the number of trace data values.
ViStatus _VI_FUNC rsspecan_QueryWLANBurstPPDUStatus(ViSession instrSession,
                                                    ViInt32   arraySize,
                                                    ViInt32   PPDUStatus[],
                                                    ViInt32*  noOfValues)
{
    ViStatus error = VI_SUCCESS;
    ViString PPDUStatusArr[] = {"OK", "WARN", "SEL", NULL};

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_QueryStringIndexesArrayToUserBuffer(instrSession, "FETC:BURS:PPDU:STAT?", PPDUStatusArr, arraySize, PPDUStatus, noOfValues));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function returns the result values of the adjacent channel power measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR mode/Sets the ACP measurement results.
/// HIPAR values/Returns the measured values.
ViStatus _VI_FUNC rsspecan_QueryWlanACPMeasurement(ViSession instrSession,
                                                   ViInt32   mode,
                                                   ViReal64  values[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K90|K91"));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, mode, RSSPECAN_VAL_ACP_CURR, RSSPECAN_VAL_ACP_MAX),
            2, "Mode");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, values),
                3, "Values");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, ":CALC:MARK:FUNC:POW:RES:%s?", ACPMeas[mode]);
    checkErr(RsCore_QueryFloatArrayToUserBuffer(instrSession, cmd, 12, values, NULL));

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function forces the IQ measurement results to be recalculated according to the current settings.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsspecan_WlanBurstImmediate(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_WLAN_CALC_BURS, ""));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/*****************************************************************************
 * Function: Read WLAN Trace Data
 * Purpose:  This function returns all the measured data that relates to the
 *           currently selected measurement type. The returned data is
 *           particular to the currently selected measurement type and is
 *           specified in the Trace Data parameter description.
*****************************************************************************/
ViStatus _VI_FUNC rsspecan_ReadWlanTraceData(ViSession instrSession,
                                             ViInt32   sourceTrace,
                                             ViInt32   arraySize,
                                             ViReal64  traceData[],
                                             ViPInt32  noOfValues)
{
    ViStatus error = VI_SUCCESS;
    ViChar   trace[RS_MAX_SHORT_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K90|K91"));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, sourceTrace, 1, 6),
            2, "Source Trace");

    snprintf(trace, RS_MAX_SHORT_MESSAGE_BUF_SIZE, "TRACE%ld", sourceTrace);
    checkErr(rsspecan_dataReadTrace(instrSession, 1, trace, arraySize, traceData, noOfValues));

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);

    return error;
}

/*****************************************************************************
 * Function: Read WLAN Memory IQ Data
 * Purpose:  This function returns all the IQ data associated with the
 *           measurement acquisition time. The result values are scaled
 *           linear in unit Volt and correspond to the voltage at the RF
 *           input of the instrument. The command returns a comma-separated
 *           list of the measured voltage values in floating point format
 *           (Comma Separated Values = CSV). The number of values returned is
 *           2 * number of samples, the first half being the I values, the
 *           second half the Q values.
 *
 *           Notes:
 *           (1) If there are not I/Q data available in memory because the
 *           corresponding measurement had not been started, the function
 *           will cause a Query Error.
 *
*****************************************************************************/
ViStatus _VI_FUNC rsspecan_ReadWlanMemoryIQData(ViSession instrSession,
                                                ViInt32   offsetSamples,
                                                ViInt32   noofSamples,
                                                ViInt32   bufferSize,
                                                ViInt32*  noofPoints,
                                                ViReal64  realPartsI[],
                                                ViReal64  imaginaryPartsQ[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K90|K91"));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, ":FORM REAL,32;:TRAC:IQ:DATA:MEM? %ld,%ld", offsetSamples, noofSamples);
    checkErr(rsspecan_dataReadInterleaved(instrSession, cmd, 0, bufferSize, realPartsI, imaginaryPartsQ, noofPoints));

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);

    return error;
}

/*****************************************************************************
 * Function: Read WLAN SEM Results
 * Purpose:  This function returns the Spectrum Emission Mask summary results.
*****************************************************************************/
ViStatus _VI_FUNC rsspecan_ReadWlanSEMResults(ViSession instrSession,
                                              ViInt32   noOfValues,
                                              ViReal64  index[],
                                              ViReal64  startFrequencyBand[],
                                              ViReal64  stopFrequencyBand[],
                                              ViReal64  resolutionBandwidth[],
                                              ViReal64  limitFailFrequency[],
                                              ViReal64  powerAbs[],
                                              ViReal64  powerRel[],
                                              ViReal64  limitDistance[],
                                              ViReal64  failureFlag[],
                                              ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;
    ViChar*  buffer = NULL;
    ViChar*  pbuf;
    ViInt32  count = 0;
    ViInt32  i;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidNullPointer(instrSession, index),
                2, "Index");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, startFrequencyBand),
                3, "Start Frequency Band");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, stopFrequencyBand),
                4, "Stop Frequency Band");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, resolutionBandwidth),
                5, "Resolution Bandwidth");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, limitFailFrequency),
                6, "Limit Fail Frequency");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, powerAbs),
                7, "Power Abs");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, powerRel),
                8, "Power Rel");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, limitDistance),
                9, "Limit Distance");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, failureFlag),
                10, "Failure Flag");

    checkErr(RsCore_QueryViStringUnknownLength(instrSession, "FORM ASCII;:TRAC:DATA? LIST", &buffer));

    pbuf = strtok(buffer, ",");
    if (pbuf)
        do
        {
            if (count < noOfValues)
            {
                (void)sscanf(pbuf, "%le", &index[count]);
                pbuf = strtok(NULL, ",");
                (void)sscanf(pbuf, "%le", &startFrequencyBand[count]);
                pbuf = strtok(NULL, ",");
                (void)sscanf(pbuf, "%le", &stopFrequencyBand[count]);
                pbuf = strtok(NULL, ",");
                (void)sscanf(pbuf, "%le", &resolutionBandwidth[count]);
                pbuf = strtok(NULL, ",");
                (void)sscanf(pbuf, "%le", &limitFailFrequency[count]);
                pbuf = strtok(NULL, ",");
                (void)sscanf(pbuf, "%le", &powerAbs[count]);
                pbuf = strtok(NULL, ",");
                (void)sscanf(pbuf, "%le", &powerRel[count]);
                pbuf = strtok(NULL, ",");
                (void)sscanf(pbuf, "%le", &limitDistance[count]);
                pbuf = strtok(NULL, ",");
                (void)sscanf(pbuf, "%le", &failureFlag[count]);
                pbuf = strtok(NULL, ",");
            }
            else
            {
                for (i = 0; i < 11; i++)
                    pbuf = strtok(NULL, ",");
            }
            count++;
        }
        while (pbuf);

    if (returnedValues)
        *returnedValues = count;

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    if (buffer) free(buffer);
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/*****************************************************************************
 * Function: Fetch WLAN Burst All
 * Purpose:  This function returns all the results.
 *
 *           Note:
 *           (1) If no measurement has been performed yet, a query error
 *           will occur.
*****************************************************************************/
ViStatus _VI_FUNC rsspecan_FetchWlanBurstAll(ViSession instrSession,
                                             ViReal64  results[],
                                             ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K90|K91"));

    checkErr(RsCore_QueryFloatArrayToUserBuffer(instrSession, "FETC:BURS:ALL:FORM?", 65535, results, returnedValues));

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function returns the power measurement value, measured during the
/// HIFN  measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR powerMeasurement/Sets the power measurement.
/// HIPAR resultType/Selects the result type.
/// HIPAR arraySize/This parameter determines the size of array 'Values'.
/// HIPAR values/This control returns the selected power values.
/// HIPAR returnedValues/Returns the number of returned values.
ViStatus _VI_FUNC rsspecan_FetchWlanBurstPower(ViSession instrSession,
                                               ViInt32   powerMeasurement,
                                               ViInt32   resultType,
                                               ViUInt32  arraySize,
                                               ViReal64  values[],
                                               ViUInt32* returnedValues)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViString meas_arr[5] = {"PRE", "PAYL", "RMS", "PEAK", "CRES"};

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, powerMeasurement, RSSPECAN_VAL_WLAN_MEAS_PREA, RSSPECAN_VAL_WLAN_MEAS_CREST),
            2, "Power Measurement");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "FETC:BURS:%s", meas_arr[powerMeasurement]);
    switch (resultType)
    {
    case RSSPECAN_VAL_MEASTYPE_MIN:
        RsCore_StrcatMaxLen(cmd, RS_MAX_MESSAGE_BUF_SIZE, ":MIN?", NULL, NULL);
        break;
    case RSSPECAN_VAL_MEASTYPE_MAX:
        RsCore_StrcatMaxLen(cmd, RS_MAX_MESSAGE_BUF_SIZE, ":MAX?", NULL, NULL);
        break;
    case RSSPECAN_VAL_MEASTYPE_AVER:
        RsCore_StrcatMaxLen(cmd, RS_MAX_MESSAGE_BUF_SIZE, ":AVER?", NULL, NULL);
        break;
    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, resultType), 3, "Result Type");
    }

    checkErr(RsCore_QueryFloatArrayToUserBuffer(instrSession, cmd, arraySize, values, (ViPInt32)returnedValues));

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function returns the burst errors.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR errorType/Sets the power measurement.
/// HIPAR mode/Sets the measurement mode.
/// HIPAR arraySize/This parameter determines the size of array 'Values'.
/// HIPAR values/This control returns the selected error values.
/// HIPAR returnedValues/Returns the number of returned values.
ViStatus _VI_FUNC rsspecan_FetchWlanBurstError(ViSession instrSession,
                                               ViInt32   errorType,
                                               ViInt32   mode,
                                               ViUInt32  arraySize,
                                               ViReal64  values[],
                                               ViUInt32* returnedValues)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViString meas_arr[2] = {"FERR", "SYMB"};

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, errorType, RSSPECAN_VAL_WLAN_ERR_FREQ, RSSPECAN_VAL_WLAN_ERR_SYMB),
            2, "Error Type");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "FETC:BURS:%s", meas_arr[errorType]);
    switch (mode)
    {
    case RSSPECAN_VAL_MEASTYPE_MIN:
        RsCore_StrcatMaxLen(cmd, RS_MAX_MESSAGE_BUF_SIZE, ":MIN?", NULL, NULL);
        break;
    case RSSPECAN_VAL_MEASTYPE_MAX:
        RsCore_StrcatMaxLen(cmd, RS_MAX_MESSAGE_BUF_SIZE, ":MAX?", NULL, NULL);
        break;
    case RSSPECAN_VAL_MEASTYPE_AVER:
        RsCore_StrcatMaxLen(cmd, RS_MAX_MESSAGE_BUF_SIZE, ":AVER?", NULL, NULL);
        break;
    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, mode), 3, "Mode");
    }

    checkErr(RsCore_QueryFloatArrayToUserBuffer(instrSession, cmd, arraySize, values, (ViPInt32)returnedValues));

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/*****************************************************************************
 * Function: Fetch WLAN Effective Channel Gain
 * Purpose:  Returns the effective channel gain result which is used as the
 *           reference for the Spectrum Flatness limits when Spectrum Flatness results
 *           are based on effective channels.
*****************************************************************************/
ViStatus _VI_FUNC rsspecan_FetchWLANEffectiveChannelGain(ViSession instrSession,
                                                         ViInt32   arraySize,
                                                         ViReal64  results[],
                                                         ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_QueryFloatArrayToUserBuffer(instrSession, "FETC:BURS:ECMG?", arraySize, results, returnedValues));

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/*****************************************************************************
 * Function: Fetch WLAN Physical Channel Gain
 * Purpose:  Returns the physical channel gain result which is used as the
 *           reference for the Spectrum Flatness limits when Spectrum
 *           Flatness results are based on physical channels.
*****************************************************************************/
ViStatus _VI_FUNC rsspecan_FetchWLANPhysicalChannelGain(ViSession instrSession,
                                                        ViInt32   arraySize,
                                                        ViReal64  results[],
                                                        ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_QueryFloatArrayToUserBuffer(instrSession, "FETC:BURS:PCMG?", arraySize, results, returnedValues));

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function returns the burst errors.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR iqImpairment/Sets the IQ Impairment. The driver uses this value to select teh
/// HIPAR iqImpairment/attribute.
/// HIPAR mode/Sets the measurement mode.
/// HIPAR arraySize/This parameter determines the size of array 'Values'.
/// HIPAR values/This control returns the selected impairment value.
/// HIPAR returnedValues/Returns the number of returned values.
ViStatus _VI_FUNC rsspecan_FetchWlanIQImp(ViSession instrSession,
                                          ViInt32   iqImpairment,
                                          ViInt32   mode,
                                          ViUInt32  arraySize,
                                          ViReal64  values[],
                                          ViUInt32* returnedValues)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViString meas_arr[4] = {"IQOF", "GIMB", "QUAD", "IQSK"};

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, iqImpairment, RSSPECAN_VAL_WLAN_IQ_OFFS, RSSPECAN_VAL_WLAN_IQ_SKEW),
            2, "IQ Impairment");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "FETC:BURS:%s", meas_arr[iqImpairment]);
    switch (mode)
    {
    case RSSPECAN_VAL_MEASTYPE_MIN:
        RsCore_StrcatMaxLen(cmd, RS_MAX_MESSAGE_BUF_SIZE, ":MIN?", NULL, NULL);
        break;
    case RSSPECAN_VAL_MEASTYPE_MAX:
        RsCore_StrcatMaxLen(cmd, RS_MAX_MESSAGE_BUF_SIZE, ":MAX?", NULL, NULL);
        break;
    case RSSPECAN_VAL_MEASTYPE_AVER:
        RsCore_StrcatMaxLen(cmd, RS_MAX_MESSAGE_BUF_SIZE, ":AVER?", NULL, NULL);
        break;
    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, mode), 3, "Mode");
    }

    checkErr(RsCore_QueryFloatArrayToUserBuffer(instrSession, cmd, arraySize, values, (ViPInt32)returnedValues));

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function returns the EVM measurement value.
/// HIFN
/// HIFN     Note:
/// HIFN     (1) If no measurement has been performed yet, a query error will
/// HIFN  occur.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR evm/Sets the EVM measurement.
/// HIPAR mode/Sets the measurement mode.
/// HIPAR arraySize/This parameter determines the size of array 'Values'.
/// HIPAR values/This control returns the selected EVM value.
/// HIPAR returnedValues/Returns the number of returned values.
ViStatus _VI_FUNC rsspecan_FetchWlanEVM(ViSession instrSession,
                                        ViInt32   evm,
                                        ViInt32   mode,
                                        ViUInt32  arraySize,
                                        ViReal64  values[],
                                        ViUInt32* returnedValues)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViString meas_arr[5] = {"ALL", "DATA", "PIL", "DIR", "IEEE"};

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, evm, RSSPECAN_VAL_WLAN_EVM_ALL, RSSPECAN_VAL_WLAN_EVM_PEAK),
            2, "EVM");
    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "FETC:BURS:EVM:%s", meas_arr[evm]);
    switch (mode)
    {
    case RSSPECAN_VAL_MEASTYPE_MIN:
        RsCore_StrcatMaxLen(cmd, RS_MAX_MESSAGE_BUF_SIZE, ":MIN?", NULL, NULL);
        break;
    case RSSPECAN_VAL_MEASTYPE_MAX:
        RsCore_StrcatMaxLen(cmd, RS_MAX_MESSAGE_BUF_SIZE, ":MAX?", NULL, NULL);
        break;
    case RSSPECAN_VAL_MEASTYPE_AVER:
        RsCore_StrcatMaxLen(cmd, RS_MAX_MESSAGE_BUF_SIZE, ":AVER?", NULL, NULL);
        break;
    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, mode), 3, "Mode");
    }

    checkErr(RsCore_QueryFloatArrayToUserBuffer(instrSession, cmd, arraySize, values, (ViPInt32)returnedValues));

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Returns the error vector magnitude results for each PPDU, averaged
/// HIFN  over all selected subcarriers for all symbols and streams.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR evm/Sets the EVM measurement.
/// HIPAR arraySize/This parameter determines the size of array 'Values'.
/// HIPAR values/This control returns the selected EVM value.
/// HIPAR returnedValues/Returns the number of returned values.
ViStatus _VI_FUNC rsspecan_FetchWlanPPDUEVM(ViSession instrSession,
                                            ViInt32   evm,
                                            ViUInt32  arraySize,
                                            ViReal64  values[],
                                            ViUInt32* returnedValues)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViString meas_arr[3] = {"ALL", "DATA", "PIL"};

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, evm, RSSPECAN_VAL_WLAN_EVM_ALL, RSSPECAN_VAL_WLAN_EVM_PILOT),
            2, "EVM");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "FETC:BURS:PPDU:EVM:%s:AVER?", meas_arr[evm]);
    checkErr(RsCore_QueryFloatArrayToUserBuffer(instrSession, cmd, arraySize, values, (ViPInt32)returnedValues));

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Returns the EVM for all selected subcarriers. The result is a list of
/// HIFN  values, one for each PPDU and each RU.
/// HIFN     These results are only available if the Signal Content Detailed result
/// HIFN  display is currently active.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR evm/Sets the EVM measurement.
/// HIPAR arraySize/This parameter determines the size of array 'Values'.
/// HIPAR values/This control returns the selected EVM value.
/// HIPAR returnedValues/Returns the number of returned values.
ViStatus _VI_FUNC rsspecan_FetchWLANSignalContentDetailedEVM(ViSession instrSession,
                                                             ViInt32   evm,
                                                             ViUInt32  arraySize,
                                                             ViReal64  values[],
                                                             ViUInt32* returnedValues)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViString meas_arr[3] = {"ALL", "DATA", "PIL"};

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, evm, RSSPECAN_VAL_WLAN_EVM_ALL, RSSPECAN_VAL_WLAN_EVM_PILOT),
            2, "EVM");
    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "FETC:SCD:EVM:%s?", meas_arr[evm]);
    checkErr(RsCore_QueryFloatArrayToUserBuffer(instrSession, cmd, arraySize, values, (ViPInt32)returnedValues));

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Returns the power results for each PPDU for the selected subcarriers.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR resultType/Selects the result type.
/// HIPAR subcarriers/Selects the subcarriers.
/// HIPAR arraySize/This parameter determines the size of array 'Values'.
/// HIPAR values/This control returns the selected EVM value.
/// HIPAR returnedValues/Returns the number of returned values.
ViStatus _VI_FUNC rsspecan_FetchWLANSignalContentDetailedPPDU(
                            ViSession instrSession,
                            ViInt32   resultType,
                            ViInt32   subcarriers,
                            ViUInt32  arraySize,
                            ViReal64  values[],
                            ViUInt32* returnedValues)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViString meas_arr[3] = {"PPDU", "RU", ""};
    ViString subc_arr[3] = {"HELT", "DPIL", "LLTF"};

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, resultType, RSSPECAN_VAL_WLAN_SCD_SUBCARRIERS, RSSPECAN_VAL_WLAN_SCD_SUBCARRIERS_RUS),
            2, "Result Type");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, subcarriers, RSSPECAN_VAL_WLAN_SCD_HELTF, RSSPECAN_VAL_WLAN_SCD_LLTF),
            3, "Subcarriers");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "FETC:SCD:POW:%s? %s", meas_arr[resultType], subc_arr[subcarriers]);
    checkErr(RsCore_QueryFloatArrayToUserBuffer(instrSession, cmd, arraySize, values, (ViPInt32)returnedValues));

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function returns the number of analyzed PPDUs from the current
/// HIFN  capture buffer or in all measurements.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR type/Sets the count type. The driver uses this value to select the
/// HIPAR type/attribute.
/// HIPAR count/This control returns the number of analyzed PPDUs from the current
/// HIPAR count/capture buffer or in all measurements.
ViStatus _VI_FUNC rsspecan_FetchWlanBurstCount(ViSession instrSession,
                                               ViInt32   type,
                                               ViInt32*  count)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, type, RSSPECAN_VAL_WLAN_CURRENT, RSSPECAN_VAL_WLAN_ALL),
            2, "type");

    if (type == RSSPECAN_VAL_WLAN_CURRENT)
    {
        viCheckParm(rsspecan_GetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_WLAN_FETC_BURS_COUNT, count),
                3, "Count");
    }
    else
    {
        viCheckParm(rsspecan_GetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_WLAN_FETC_BURST_COUNT_ALL, count),
                3, "Count");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function returns the number of symbols in each analyzed PPDU.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR arraySize/Number of array points requested.
/// HIPAR symbolCount/This control returns the number of symbols in each analyzed PPDU.
/// HIPAR noOfValues/Returns the number of symbols.
ViStatus _VI_FUNC rsspecan_FetchWlanSymbolCount(ViSession instrSession,
                                                ViInt32   arraySize,
                                                ViInt32   symbolCount[],
                                                ViInt32*  noOfValues)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_QueryIntegerArrayToUserBuffer(instrSession, "FETC:SYMB:COUN?", arraySize, symbolCount, noOfValues));

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the type of the first analyzed PPDU in the current capture
/// HIFN  buffer.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR bufferSize/Size of output array.
/// HIPAR burstType/Queries the type of the first analyzed PPDU in the current capture
/// HIPAR burstType/buffer.
ViStatus _VI_FUNC rsspecan_FetchWLANBurstType(ViSession instrSession,
                                              ViInt32   bufferSize,
                                              ViChar    burstType[])
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_GetAttributeViString(instrSession, NULL, RSSPECAN_ATTR_WLAN_FETCH_BURST_TYPE,
            bufferSize, burstType),
            3, "Burst Type");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the guard interval of the first analyzed PPDU.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR guardInterval/Queries the guard interval of the first analyzed PPDU.
ViStatus _VI_FUNC rsspecan_FetchWLANBurstGuardInterval(ViSession instrSession,
                                                       ViInt32*  guardInterval)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_GetAttributeViInt32(instrSession, NULL, RSSPECAN_ATTR_WLAN_FETCH_BURST_GUARD_INTERVAL, guardInterval),
            2, "Guard Interval");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the Modulation and Coding Scheme (MCS) index of the first
/// HIFN  analyzed PPDU.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR mcsIndex/Queries the Modulation and Coding Scheme (MCS) index of the first
/// HIPAR mcsIndex/analyzed PPDU.
ViStatus _VI_FUNC rsspecan_FetchWLANMCSIndex(ViSession instrSession,
                                             ViInt32*  mcsIndex)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_GetAttributeViInt32(instrSession, NULL, RSSPECAN_ATTR_WLAN_FETCH_MCS_INDEX, mcsIndex),
            2, "MCS Index");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function returns the length of the analyzed PPDUs from the
/// HIFN  current measurement. If the number of PPDUs to analyze is greater than
/// HIFN  the number of PPDUs that can be captured in one buffer, this command
/// HIFN  only returns the lengths of the PPDUs in the current capture buffer.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR length/This control returns the length of the analyzed PPDUs from the current
/// HIPAR length/measurement.
ViStatus _VI_FUNC rsspecan_FetchWlanBurstLength(ViSession instrSession,
                                                ViInt32*  length)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_GetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_WLAN_FETC_BURST_LENGTH, length),
            3, "Length");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function returns the average, maximum or minimum of the "Peak
/// HIFN  Power per PPDU" (in dBm). All analyzed PPDUs, up to the statistic
/// HIFN  length, take part in the statistical evaluation.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR mode/Sets the measurement mode.
/// HIPAR value/This control returns the selected power value .
ViStatus _VI_FUNC rsspecan_FetchWlanBurstPeak(ViSession instrSession,
                                              ViInt32   mode,
                                              ViReal64* value)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    switch (mode)
    {
    case RSSPECAN_VAL_MEASTYPE_MIN:
        viCheckParm(rsspecan_GetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_WLAN_FETC_PEAK_MIN, value),
                3, "Value");
        break;
    case RSSPECAN_VAL_MEASTYPE_MAX:
        viCheckParm(rsspecan_GetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_WLAN_FETC_PEAK_MAX, value),
                3, "Value");
        break;
    case RSSPECAN_VAL_MEASTYPE_AVER:
        viCheckParm(rsspecan_GetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_WLAN_FETC_PEAK_AVERAGE, value),
                3, "Value");
        break;
    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, mode), 2, "Mode");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function returns the average, maximum or minimum of the "Payload
/// HIFN  Power per PPDU" (in dBm). All analyzed PPDUs, up to the statistic
/// HIFN  length, take part in the statistical evaluation.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR mode/Sets the measurement mode.
/// HIPAR value/This control returns the selected power value .
ViStatus _VI_FUNC rsspecan_FetchWlanBurstPayload(ViSession instrSession,
                                                 ViInt32   mode,
                                                 ViReal64* value)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    switch (mode)
    {
    case RSSPECAN_VAL_MEASTYPE_MIN:
        viCheckParm(rsspecan_GetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_WLAN_FETC_PAYL_MIN, value),
                3, "Value");
        break;
    case RSSPECAN_VAL_MEASTYPE_MAX:
        viCheckParm(rsspecan_GetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_WLAN_FETC_PAYL_MAX, value),
                3, "Value");
        break;
    case RSSPECAN_VAL_MEASTYPE_AVER:
        viCheckParm(rsspecan_GetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_WLAN_FETC_PAYL_AVERAGE, value),
                3, "Value");
        break;
    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, mode), 2, "Mode");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function returns the average, maximum or minimum of the "Peak
/// HIFN  Power per PPDU" (in dBm). All analyzed PPDUs, up to the statistic
/// HIFN  length, take part in the statistical evaluation.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR mode/Sets the measurement mode.
/// HIPAR value/This control returns the selected power value .
ViStatus _VI_FUNC rsspecan_FetchWlanBurstPreamble(ViSession instrSession,
                                                  ViInt32   mode,
                                                  ViReal64* value)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    switch (mode)
    {
    case RSSPECAN_VAL_MEASTYPE_MIN:
        viCheckParm(rsspecan_GetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_WLAN_FETC_PRE_MIN, value),
                3, "Value");
        break;
    case RSSPECAN_VAL_MEASTYPE_MAX:
        viCheckParm(rsspecan_GetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_WLAN_FETC_PRE_MAX, value),
                3, "Value");
        break;
    case RSSPECAN_VAL_MEASTYPE_AVER:
        viCheckParm(rsspecan_GetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_WLAN_FETC_PRE_AVERAGE, value),
                3, "Value");
        break;
    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, mode), 2, "Mode");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function returns the start position of each analyzed PPDU in the
/// HIFN  current capture buffer.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR value/This control returns the start position of each analyzed PPDU in the
/// HIPAR value/current capture buffer.
ViStatus _VI_FUNC rsspecan_FetchWlanBurstStart(ViSession instrSession,
                                               ViReal64* value)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_WLAN_FETCH_BURST_START, value),
            2, "Value");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function returns the burst rise/falling time.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR time/Sets the time measurement.
/// HIPAR mode/Sets the measurement mode.
/// HIPAR arraySize/This parameter determines the size of array 'Values'.
/// HIPAR values/This control returns the selected power value.
/// HIPAR returnedValues/Returns the number of returned values.
ViStatus _VI_FUNC rsspecan_QueryWlanBurstTime(ViSession instrSession,
                                              ViInt32   time,
                                              ViInt32   mode,
                                              ViUInt32  arraySize,
                                              ViReal64  values[],
                                              ViUInt32* returnedValues)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViString meas_arr[2] = {"TFAL", "TRIS"};

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, time, RSSPECAN_VAL_WLAN_TIME_FALL, RSSPECAN_VAL_WLAN_TIME_RISE),
            2, "Time");
    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "FETC:BURS:%s", meas_arr[time]);
    switch (mode)
    {
    case RSSPECAN_VAL_MEASTYPE_MIN:
        RsCore_StrcatMaxLen(cmd, RS_MAX_MESSAGE_BUF_SIZE, ":MIN?", NULL, NULL);
        break;
    case RSSPECAN_VAL_MEASTYPE_MAX:
        RsCore_StrcatMaxLen(cmd, RS_MAX_MESSAGE_BUF_SIZE, ":MAX?", NULL, NULL);
        break;
    case RSSPECAN_VAL_MEASTYPE_AVER:
        RsCore_StrcatMaxLen(cmd, RS_MAX_MESSAGE_BUF_SIZE, ":AVER?", NULL, NULL);
        break;
    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, mode), 3, "Mode");
    }
    checkErr(RsCore_QueryFloatArrayToUserBuffer(instrSession, cmd, arraySize, values, (ViPInt32)returnedValues));

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function returns the Burst Error Rate for Pilots (average,
/// HIFN  minimum or maximum value) in % for the 802.11n MIMO standard.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR mode/Sets the measurement mode.
/// HIPAR arraySize/This parameter determines the size of array 'Values'.
/// HIPAR values/This control returns the selected power value.
/// HIPAR returnedValues/Returns the number of returned values.
ViStatus _VI_FUNC rsspecan_QueryWlanBurstErrorRateForPilots(ViSession instrSession,
                                                            ViInt32   mode,
                                                            ViUInt32  arraySize,
                                                            ViReal64  values[],
                                                            ViUInt32* returnedValues)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "FETC:BURS:BERP";

    checkErr(RsCore_LockSession(instrSession));

    switch (mode)
    {
    case RSSPECAN_VAL_MEASTYPE_MIN:
        RsCore_StrcatMaxLen(cmd, RS_MAX_MESSAGE_BUF_SIZE, ":MIN?", NULL, NULL);
        break;
    case RSSPECAN_VAL_MEASTYPE_MAX:
        RsCore_StrcatMaxLen(cmd, RS_MAX_MESSAGE_BUF_SIZE, ":MAX?", NULL, NULL);
        break;
    case RSSPECAN_VAL_MEASTYPE_AVER:
        RsCore_StrcatMaxLen(cmd, RS_MAX_MESSAGE_BUF_SIZE, ":AVER?", NULL, NULL);
        break;
    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, mode), 3, "Mode");
    }

    checkErr(RsCore_QueryFloatArrayToUserBuffer(instrSession, cmd, arraySize, values, (ViPInt32)returnedValues));

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function returns the errors (average, maximum or minimum
/// HIFN value) for all streams of selected measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR errorType/Sets the power measurement.
/// HIPAR mode/Sets the measurement mode.
/// HIPAR arraySize/Defines the memory size allocated for Results array.
/// HIPAR results/This control returns the results for all stream.
/// HIPAR returnedValues/Returns the number of values in Results array.
ViStatus _VI_FUNC rsspecan_FetchWlanStreamBurstError(ViSession instrSession,
                                                     ViInt32   errorType,
                                                     ViInt32   mode,
                                                     ViInt32   arraySize,
                                                     ViReal64  results[],
                                                     ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K91"));

    switch (errorType)
    {
    case RSSPECAN_VAL_WLAN_ERR_CPER:
        ;
        break;
    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, errorType), 2, "Error Type");
    }

    switch (mode)
    {
    case RSSPECAN_VAL_MEASTYPE_MIN:
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "FETC:BURS:CPER:MIN?");
        break;
    case RSSPECAN_VAL_MEASTYPE_MAX:
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "FETC:BURS:CPER:MAX?");
        break;
    case RSSPECAN_VAL_MEASTYPE_AVER:
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "FETC:BURS:CPER:AVER?");
        break;
    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, mode), 3, "Mode");
    }

    checkErr(RsCore_QueryFloatArrayToUserBuffer(instrSession, cmd, arraySize, results, returnedValues));

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function returns teh power measurement values for all
/// HIFN streams, measured during the measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOption
/// HIPAR powerMeasurement/Sets the power measurement.
/// HIPAR arraySize/Defines the memory size allocated for Results array.
/// HIPAR results/This control returns the results for all stream.
/// HIPAR returnedValues/Returns the number of values in Results array.
ViStatus _VI_FUNC rsspecan_FetchWlanStreamBurstPower(ViSession instrSession,
                                                     ViInt32   powerMeasurement,
                                                     ViInt32   arraySize,
                                                     ViReal64  results[],
                                                     ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K91"));

    switch (powerMeasurement)
    {
    case RSSPECAN_VAL_WLAN_MEAS_MCPOW_MIN:
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "FETC:BURS:MCP:MIN?");
        break;
    case RSSPECAN_VAL_WLAN_MEAS_MCPOW_MAX:
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "FETC:BURS:MCP:MAX?");
        break;
    case RSSPECAN_VAL_WLAN_MEAS_MCPOW_AVER:
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "FETC:BURS:MCP:AVER?");
        break;
    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, powerMeasurement), 2, "Power Measurement");
    }

    checkErr(RsCore_QueryFloatArrayToUserBuffer(instrSession, cmd, arraySize, results, returnedValues));

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function returns the channel power measurement values for all
/// HIFN  streams, measured during the measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR powerMeasurement/Sets the power measurement.
/// HIPAR arraySize/Defines the memory size allocated for Results array.
/// HIPAR results/This control returns the results for active streams.
/// HIPAR returnedValues/Returns the number of values in Results array.
ViStatus _VI_FUNC rsspecan_FetchWlanStreamBurstChannelPower(ViSession instrSession,
                                                            ViInt32   powerMeasurement,
                                                            ViInt32   arraySize,
                                                            ViReal64  results[],
                                                            ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K91"));

    switch (powerMeasurement)
    {
    case RSSPECAN_VAL_WLAN_MEAS_MCPOW_MIN:
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "FETC:BURS:MCHP:MIN?");
        break;
    case RSSPECAN_VAL_WLAN_MEAS_MCPOW_MAX:
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "FETC:BURS:MCHP:MAX?");
        break;
    case RSSPECAN_VAL_WLAN_MEAS_MCPOW_AVER:
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "FETC:BURS:MCHP:AVER?");
        break;
    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, powerMeasurement), 2, "Power Measurement");
    }

    checkErr(RsCore_QueryFloatArrayToUserBuffer(instrSession, cmd, arraySize, results, returnedValues));

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Returns the coefficients of the polynomial regression model used to
/// HIFN  determine the AM/AM result display.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR arraySize/This parameter determines the size of array 'Results'.
/// HIPAR results/This control returns the selected EVM value.
/// HIPAR returnedValues/Returns the number of returned values.
ViStatus _VI_FUNC rsspecan_FetchWLANAMAMPolynomialCoefficients(
                            ViSession instrSession,
                            ViUInt32  arraySize,
                            ViReal64  results[],
                            ViUInt32* returnedValues)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_QueryFloatArrayToUserBuffer(instrSession, "FETC:BURS:AM:AM:COEF?", arraySize, results, (ViPInt32)returnedValues));

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Returns detailed signal information for each decoded RU and for each
/// HIFN  object. The result is a list of values with 4 rows per RU, in the same
/// HIFN  order as the Signal Content Detailed result display.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR arraySize/This parameter determines the size of array 'Results'.
/// HIPAR results/This control returns the selected EVM value.
/// HIPAR returnedValues/Returns the number of returned values.
ViStatus _VI_FUNC rsspecan_FetchWLANDetailedSignalContentIEEE802_11ax(
                            ViSession instrSession,
                            ViUInt32  arraySize,
                            ViChar    results[],
                            ViUInt32* returnedValues)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_QueryViStringUnknownLengthToUserBuffer(instrSession, "FETC:SCD:ALL?", arraySize, results, (ViPInt32)returnedValues));

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Returns the results of the Signal Fields table, including column
/// HIFN  headers. The result is a comma-separated list of values for the
/// HIFN  selected PPDU. This method is only available for the IEEE 802.11ax
/// HIFN  standard.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR arraySize/This parameter determines the size of array 'Results'.
/// HIPAR results/This control returns the selected EVM value.
/// HIPAR returnedValues/Returns the number of returned values.
ViStatus _VI_FUNC rsspecan_FetchWLANSignalField(ViSession instrSession,
                                                ViUInt32  arraySize,
                                                ViChar    results[],
                                                ViUInt32* returnedValues)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_QueryViStringUnknownLengthToUserBuffer(instrSession, "FETC:SFI:ALL?", arraySize, results, (ViPInt32)returnedValues));

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Returns the results of the Unused Tone Error Summary. The result is a
/// HIFN  comma-separated list of values for up to 37 measurement points in the
/// HIFN  channel. Which subcarriers are measured depends on the size and
/// HIFN  position of the RU being transmitted. This result is required by the
/// HIFN  IEEE 802.11ax standard for HE trigger-based PPDUs with a maximum
/// HIFN  channel bandwidth of 80 MHz.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR arraySize/This parameter determines the size of array 'Results'.
/// HIPAR results/This control returns the selected EVM value.
/// HIPAR returnedValues/Returns the number of returned values.
ViStatus _VI_FUNC rsspecan_FetchWLANUnusedToneErrorSummary(ViSession instrSession,
                                                           ViUInt32  arraySize,
                                                           ViChar    results[],
                                                           ViUInt32* returnedValues)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_QueryViStringUnknownLengthToUserBuffer(instrSession, "FETC:UTES:ALL?", arraySize, results, (ViPInt32)returnedValues));

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Returns the numeric results of the Spectrum Flatness trace.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR arraySize/This parameter determines the size of array 'Results'.
/// HIPAR returnedValues/Returns the number of returned values.
/// HIPAR startNumber/Returns the Start number.
/// HIPAR endNumber/Returns the End number.
/// HIPAR minDistance/Returns the Minimal Distance.
/// HIPAR subcarrierMinDistance/Returns the Subcarrier Minimal Distance.
/// HIPAR minDistanceStatus/Returns the Minimal Distance Status.
/// HIPAR minUpperDistance/Returns Minimal Upper Distance.
/// HIPAR subcarrierUppMinDistance/Returns the Subcarrier Upper Minimal Distance.
/// HIPAR minUppDistanceStatus/Returns the Minimal Upper Distance Status.
ViStatus _VI_FUNC rsspecan_FetchWLANSpectrumFlatnessSummary(ViSession instrSession,
                                                            ViUInt32  arraySize,
                                                            ViUInt32* returnedValues,
                                                            ViInt32   startNumber[],
                                                            ViInt32   endNumber[],
                                                            ViReal64  minDistance[],
                                                            ViInt32   subcarrierMinDistance[],
                                                            ViInt32   minDistanceStatus[],
                                                            ViReal64  minUpperDistance[],
                                                            ViInt32   subcarrierUppMinDistance[],
                                                            ViInt32   minUppDistanceStatus[])
{
    ViStatus error = VI_SUCCESS;
    ViChar*  response = NULL;
    ViChar*  p2buffer;
    ViUInt32 totalNumber;
    ViInt32  number = 0;
    ViUInt32 loopCount;
    ViUInt32 i = 0;
    ViChar   cmd[RS_MAX_SHORT_MESSAGE_BUF_SIZE] = "FETC:SFS:ALL?";
    const    char s[2] = ",";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_QueryViStringUnknownLength(instrSession, cmd, &response));

    totalNumber = RsCore_GetElementsCount(response, ',');
    *returnedValues = (ViUInt32)floor(totalNumber / 8.0);

    if (*returnedValues != (totalNumber / 8)) // We expect to receive 8 items per table row from the instrument
        checkErr(RS_ERROR_UNEXPECTED_RESPONSE); // If we have not received a valid number of responses, throw error

    loopCount = (arraySize < *returnedValues) ? arraySize : *returnedValues;

    for (i = 0; i < loopCount; i++)
    {
        if (i == 0)
        {
            p2buffer = strtok(response, s);
        }
        else
        {
            p2buffer = strtok(NULL, s);
        }
        startNumber[i] = (ViInt32)atoi(p2buffer);

        p2buffer = strtok(NULL, s);
        endNumber[i] = (ViInt32)atoi(p2buffer);

        p2buffer = strtok(NULL, s);
        minDistance[i] = (ViReal64)atof(p2buffer);

        p2buffer = strtok(NULL, s);
        subcarrierMinDistance[i] = (ViInt32)atoi(p2buffer);

        p2buffer = strtok(NULL, s);
        checkErr(RsCore_FindRangeTableDiscreteNumericValue(instrSession, &RsSpecAn_rngPassFailStatus, p2buffer, &number));
        minDistanceStatus[i] = number;

        p2buffer = strtok(NULL, s);
        minUpperDistance[i] = (ViReal64)atof(p2buffer);

        p2buffer = strtok(NULL, s);
        subcarrierUppMinDistance[i] = (ViInt32)atoi(p2buffer);

        p2buffer = strtok(NULL, s);
        (void)RsCore_TrimString((ViString)p2buffer, RS_VAL_TRIM_WHITESPACES_AND_ALL_QUOTES);
        checkErr(RsCore_FindRangeTableDiscreteNumericValue(instrSession, &RsSpecAn_rngPassFailStatus, p2buffer, &number));
        minUppDistanceStatus[i] = (ViInt32)number;
    }

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    if (response) free(response);
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function returns the power measurement results.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR powerMeasurement/Selects the ACP (adjacent channel power) standard.
/// HIPAR arraySize/Defines the memory size allocated for Results.
/// HIPAR results/Returns the measured values:
/// HIPAR returnedValues/Returns number of returned values in Results array.
ViStatus _VI_FUNC rsspecan_GetWlanPowerResults(ViSession instrSession,
                                               ViInt32   reserved,
                                               ViInt32   powerMeasurement,
                                               ViInt32   arraySize,
                                               ViReal64  results[],
                                               ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";
             reserved;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, powerMeasurement, RSSPECAN_VAL_MEAS_POW_ACP, RSSPECAN_VAL_MEAS_POW_PPOW),
            3, "Power Measurement");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CALC:MARK:FUNC:POW:RES? %s", powerMeasurementArr[powerMeasurement]);
    checkErr(RsCore_QueryFloatArrayToUserBuffer(instrSession, cmd, arraySize, results, returnedValues));

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/*****************************************************************************
 * Function: Get WLAN Limit Check Result
 * Purpose:  This function returns the limit results
*****************************************************************************/
ViStatus _VI_FUNC rsspecan_GetWlanLimitCheckResult(ViSession instrSession,
                                                   ViInt32   reserved,
                                                   ViInt32   reserved2,
                                                   ViInt32   limitType,
                                                   ViInt32   mode,
                                                   ViInt32*  result)
{
    ViStatus error = VI_SUCCESS;
             reserved;
             reserved2;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, limitType, RSSPECAN_VAL_WLAN_LIM_FERR, RSSPECAN_VAL_WLAN_LIM_EVM),
            42, "Limit Type");

    switch (limitType)
    {
    case RSSPECAN_VAL_WLAN_LIM_FERR:
        switch (mode)
        {
        case RSSPECAN_VAL_MEASTYPE_MAX:
            viCheckParm(rsspecan_GetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_WLAN_FERR_RES_MAX, result),
                    6, "Result");
            break;
        case RSSPECAN_VAL_MEASTYPE_AVER:
            viCheckParm(rsspecan_GetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_WLAN_FERR_RES_AVERAGE, result),
                    6, "Result");
            break;
        default:
            viCheckParm(RsCore_InvalidViInt32Value(instrSession, limitType), 5, "Mode");
        }
        break;
    case RSSPECAN_VAL_WLAN_LIM_SYMB:
        switch (mode)
        {
        case RSSPECAN_VAL_MEASTYPE_MAX:
            viCheckParm(rsspecan_GetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_WLAN_SYMB_RES_MAX, result),
                    6, "Result");
            break;
        case RSSPECAN_VAL_MEASTYPE_AVER:
            viCheckParm(rsspecan_GetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_WLAN_SYMB_RES_AVERAGE, result),
                    6, "Result");
            break;
        default:
            viCheckParm(RsCore_InvalidViInt32Value(instrSession, limitType), 5, "Mode");
        }
        break;
    case RSSPECAN_VAL_WLAN_LIM_IQOFF:
        switch (mode)
        {
        case RSSPECAN_VAL_MEASTYPE_MAX:
            viCheckParm(rsspecan_GetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_WLAN_IQOF_RES_MAX, result),
                    6, "Result");
            break;
        case RSSPECAN_VAL_MEASTYPE_AVER:
            viCheckParm(rsspecan_GetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_WLAN_IQOF_RES_AVERAGE, result),
                    6, "Result");
            break;
        default:
            viCheckParm(RsCore_InvalidViInt32Value(instrSession, limitType), 5, "Mode");
        }
        break;
    case RSSPECAN_VAL_WLAN_LIM_EVM_ALL:
        switch (mode)
        {
        case RSSPECAN_VAL_MEASTYPE_MAX:
            viCheckParm(rsspecan_GetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_WLAN_EVM_ALL_RES_MAX, result),
                    6, "Result");
            break;
        case RSSPECAN_VAL_MEASTYPE_AVER:
            viCheckParm(rsspecan_GetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_WLAN_EVM_ALL_RES_AVERAGE, result),
                    6, "Result");
            break;
        default:
            viCheckParm(RsCore_InvalidViInt32Value(instrSession, limitType), 5, "Mode");
        }
        break;
    case RSSPECAN_VAL_WLAN_LIM_EVM_DATA:
        switch (mode)
        {
        case RSSPECAN_VAL_MEASTYPE_MAX:
            viCheckParm(rsspecan_GetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_WLAN_EVM_DATA_RES_MAX, result),
                    6, "Result");
            break;
        case RSSPECAN_VAL_MEASTYPE_AVER:
            viCheckParm(rsspecan_GetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_WLAN_EVM_DATA_RES_AVERAGE, result),
                    6, "Result");
            break;
        default:
            viCheckParm(RsCore_InvalidViInt32Value(instrSession, limitType), 5, "Mode");
        }
        break;
    case RSSPECAN_VAL_WLAN_LIM_EVM_PIL:
        switch (mode)
        {
        case RSSPECAN_VAL_MEASTYPE_MAX:
            viCheckParm(rsspecan_GetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_WLAN_EVM_PIL_RES_MAX, result),
                    6, "Result");
            break;
        case RSSPECAN_VAL_MEASTYPE_AVER:
            viCheckParm(rsspecan_GetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_WLAN_EVM_PIL_RES_AVERAGE, result),
                    6, "Result");
            break;
        default:
            viCheckParm(RsCore_InvalidViInt32Value(instrSession, limitType), 5, "Mode");
        }
        break;
    case RSSPECAN_VAL_WLAN_LIM_FTIME:
        switch (mode)
        {
        case RSSPECAN_VAL_MEASTYPE_MAX:
            viCheckParm(rsspecan_GetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_WLAN_TFAL_RES_MAXIMUM, result),
                    6, "Result");
            break;
        case RSSPECAN_VAL_MEASTYPE_AVER:
            viCheckParm(rsspecan_GetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_WLAN_TFAL_RES_AVERAGE, result),
                    6, "Result");
            break;
        default:
            viCheckParm(RsCore_InvalidViInt32Value(instrSession, limitType), 5, "Mode");
        }
        break;
    case RSSPECAN_VAL_WLAN_LIM_RTIME:
        switch (mode)
        {
        case RSSPECAN_VAL_MEASTYPE_MAX:
            viCheckParm(rsspecan_GetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_WLAN_TRIS_RES_MAX, result),
                    6, "Result");
            break;
        case RSSPECAN_VAL_MEASTYPE_AVER:
            viCheckParm(rsspecan_GetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_WLAN_TRIS_RES_AVERAGE, result),
                    6, "Result");
            break;
        default:
            viCheckParm(RsCore_InvalidViInt32Value(instrSession, limitType), 5, "Mode");
        }
        break;
    case RSSPECAN_VAL_WLAN_LIM_EVM:
        switch (mode)
        {
        case RSSPECAN_VAL_MEASTYPE_MAX:
            viCheckParm(rsspecan_GetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_WLAN_EVM_RES_MAX, result),
                    6, "Result");
            break;
        case RSSPECAN_VAL_MEASTYPE_AVER:
            viCheckParm(rsspecan_GetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_WLAN_EVM_RES_AVERAGE, result),
                    6, "Result");
            break;
        default:
            viCheckParm(RsCore_InvalidViInt32Value(instrSession, limitType), 5, "Mode");
        }
        break;
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/*****************************************************************************
 * Function: Get WLAN ACP Channel Limit Check Results
 * Purpose:  This function queries the state of the limit check for the
 *           adjacent or alternate channels in an ACLR measurement.
*****************************************************************************/
ViStatus _VI_FUNC rsspecan_GetWlanACPChannLimitCheckResults(ViSession instrSession,
                                                            ViInt32   reserved,
                                                            ViInt32   reserved2,
                                                            ViInt32   channelType,
                                                            ViInt32   results[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   response[RS_MAX_MESSAGE_BUF_SIZE] = "";
    ViChar*  p2buf;
             reserved;
             reserved2;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, channelType, RSSPECAN_VAL_WLAN_ACP_ADJ, RSSPECAN_VAL_WLAN_ACP_ALT),
            4, "Channel Type");

    switch (channelType)
    {
    case RSSPECAN_VAL_WLAN_ACP_ADJ:
        checkErr(RsCore_QueryViString(instrSession, "CALC:LIM:ACP:ACH:RES?", response));
        break;
    case RSSPECAN_VAL_WLAN_ACP_ALT:
        checkErr(RsCore_QueryViString(instrSession, "CALC:LIM:ACP:ALT:RES?", response));
        break;
    }

    p2buf = strtok(response, ",\n\r");

    if (strcmp(p2buf, "PASSED") == 0)
        results[0] = 1;
    else
        results[0] = 0;

    p2buf = strtok(NULL, ",\r\n");

    if (strcmp(p2buf, "PASSED") == 0)
        results[1] = 1;
    else
        results[1] = 0;

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/*****************************************************************************
 * Function: Get WLAN All Limit Check Results
 * Purpose:  This function returns all limit results.
*****************************************************************************/
ViStatus _VI_FUNC rsspecan_GetWlanAllLimitCheckResults(ViSession instrSession,
                                                       ViInt32   reserved,
                                                       ViInt32   reserved2,
                                                       ViInt32   results[])
{
    ViStatus error = VI_SUCCESS;
             reserved;
             reserved2;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K90|K91"));

    checkErr(RsCore_QueryIntegerArrayToUserBuffer(instrSession, "CALC:LIM:BURS:ALL:RESULT?", 12, results, NULL));

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/*****************************************************************************
 * Function: Get WLAN Limit Check
 * Purpose:  This function returns the result of the limit check of the
 *           limit line indicated in the selected
 *           measurement window.
*****************************************************************************/
ViStatus _VI_FUNC rsspecan_GetWlanLimitCheck(ViSession instrSession,
                                             ViInt32   reserved,
                                             ViInt32   limit,
                                             ViInt32*  result)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE];
             reserved;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, limit, 1, 8),
            3, "Limit");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "L%ld", limit);
    viCheckParm(rsspecan_GetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_WLAN_LIM_FAIL_RES, result),
            4, "Result");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/*****************************************************************************
 * Function: Get WLAN Upper Limit Line
 *           This function queries the upper limit line y-values as defined
 *           by the standard for the specified window.
*****************************************************************************/
ViStatus _VI_FUNC rsspecan_GetWLANUpperLimitLine(ViSession instrSession,
                                                 ViInt32   window,
                                                 ViInt32   limit,
                                                 ViInt32   arraySize,
                                                 ViReal64  results[],
                                                 ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CALC%ld:LIM%ld:UPP:FULL?", window, limit);
    checkErr(RsCore_QueryFloatArrayToUserBuffer(instrSession, cmd, arraySize, results, returnedValues));

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/*****************************************************************************
 * Function: Get WLAN Upper Limit Line
 *           This function queries the lower limit line y-values as defined
 *           by the standard for the specified window.
*****************************************************************************/
ViStatus _VI_FUNC rsspecan_GetWLANLowerLimitLine(ViSession instrSession,
                                                 ViInt32   window,
                                                 ViInt32   limit,
                                                 ViInt32   arraySize,
                                                 ViReal64  results[],
                                                 ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CALC%ld:LIM%ld:LOW:FULL?", window, limit);
    checkErr(RsCore_QueryFloatArrayToUserBuffer(instrSession, cmd, arraySize, results, returnedValues));

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}
