/*****************************************************************************
 *  Rohde&Schwarz Spectrum Analyzer instrument driver
 *  HRP UWB Measurement option K149
 *
 *  Original Release: January 2022
 *  By: Petr Kominek
 *
 *  Should you have any technical questions please contact the hotline of
 *  Rohde & Schwarz Vertriebs-GmbH Rohde & Schwarz Support Center
 *
 *  e-mail: CustomerSupport@rohde-schwarz.com
 *
 *****************************************************************************/

#include "rsspecan.h"
#include "rsspecan_internal.h"

/// HIFN  Switches instrument to HRP UWB (Ultrawideband) mode.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsspecan_SetHRPUWBMode(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsspecan_SetAttributeViString(instrSession, NULL, RSSPECAN_ATTR_HRP_UWB_MODE, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the Burst Sync.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR reference/Selects the reference for packet detection.
/// HIPAR threshold/Sets the detection threshold in dB/dBm relative to the burst detection
/// HIPAR threshold/reference level.
ViStatus _VI_FUNC rsspecan_ConfigureHRPUWBBurstSync(ViSession instrSession,
                                                    ViInt32   reference,
                                                    ViReal64  threshold)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, NULL, RSSPECAN_ATTR_HRP_UWB_BURST_SEARCH_REFERENCE, reference),
            2, "Reference");

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, NULL, RSSPECAN_ATTR_HRP_UWB_BURST_SEARCH_THRESHOLD, threshold),
            3, "Threshold");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the Burst Length.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR minimum/Defines the minimum burst length.
/// HIPAR maximum/Defines the maximum burst length.
ViStatus _VI_FUNC rsspecan_ConfigureHRPUWBBurstLength(ViSession instrSession,
                                                      ViReal64  minimum,
                                                      ViReal64  maximum)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, NULL, RSSPECAN_ATTR_HRP_UWB_BURST_LENGTH_MINIMUM, minimum),
            2, "Minimum");

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, NULL, RSSPECAN_ATTR_HRP_UWB_BURST_LENGTH_MAXIMUM, maximum),
            3, "Maximum");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the offset of the beginning of the detected burst to where to
/// HIFN  start detection within SYNC section.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR detectedOffset/Sets the offset of the beginning of the detected burst to where to
/// HIPAR detectedOffset/start detection within SYNC section.
ViStatus _VI_FUNC rsspecan_ConfigureHRPUWBBurstDetectedOffset(
                            ViSession instrSession,
                            ViReal64  detectedOffset)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, NULL, RSSPECAN_ATTR_HRP_UWB_BURST_DETECTED_OFFSET, detectedOffset),
            2, "Detected Offset");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Defines the maximum allowed off time within a burst.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR maxOffTimeWithinBurst/Defines the maximum allowed off time within a burst.
ViStatus _VI_FUNC rsspecan_ConfigureHRPUWBBurstMaxOffTimeWithinBurst(
                            ViSession instrSession,
                            ViReal64  maxOffTimeWithinBurst)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, NULL, RSSPECAN_ATTR_HRP_UWB_BURST_MAX_OFF_TIME_WITHIN_BURST, maxOffTimeWithinBurst),
            2, "Max Off Time Within Burst");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Only available for files that contain more than one data stream from
/// HIFN  multiple channels: automatically defines which data stream in the file
/// HIFN  is used as input for the channel.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR autoEnabled/Only available for files that contain more than one data stream from
/// HIPAR autoEnabled/multiple channels: automatically defines which data stream in the file
/// HIPAR autoEnabled/is used as input for the channel.
ViStatus _VI_FUNC rsspecan_ConfigureHRPUWBIQLoadStreamAutoEnabled(
                            ViSession instrSession,
                            ViBoolean autoEnabled)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, NULL, RSSPECAN_ATTR_HRP_UWB_IQ_LOAD_STREAM_AUTO_ENABLED, autoEnabled),
            2, "Auto Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Only available for files that contain more than one data stream from
/// HIFN  multiple channels: selects the data stream to be used as input for the
/// HIFN  currently selected channel.
/// HIFN     Automatic mode (MMEMory:LOAD:IQ:STReam:AUTO) is set to OFF.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR loadStream/Only available for files that contain more than one data stream from
/// HIPAR loadStream/multiple channels: selects the data stream to be used as input for the
/// HIPAR loadStream/currently selected channel.
ViStatus _VI_FUNC rsspecan_ConfigureHRPUWBIQLoadStream(ViSession instrSession,
                                                       ViString  loadStream)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViString(instrSession, NULL, RSSPECAN_ATTR_HRP_UWB_IQ_LOAD_STREAM, loadStream),
            2, "Load Stream");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Returns the available channels in the currently loaded input file.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR bufferSize/Size of output array.
/// HIPAR iqStreamLists/Returns the available channels in the currently loaded input file.
ViStatus _VI_FUNC rsspecan_QueryHRPUWBIQStreamLists(ViSession instrSession,
                                                    ViInt32   bufferSize,
                                                    ViChar    iqStreamLists[])
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_GetAttributeViString(instrSession, NULL, RSSPECAN_ATTR_QUERY_HRP_UWB_IQ_STREAM_LISTS,
            bufferSize, iqStreamLists),
            3, "IQ Stream List");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Configures the Data Acquisition.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channelNumber/Selects the HRP UWB Channel.
/// HIPAR defaultValues/Toggles the default values between on/off.
ViStatus _VI_FUNC rsspecan_ConfigureHRPUWBMeasurement(ViSession instrSession,
                                                      ViInt32   channelNumber,
                                                      ViBoolean defaultValues)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, NULL, RSSPECAN_ATTR_HRP_UWB_MEASUREMENT_CHANNEL_NUMBER, channelNumber),
            2, "Channel Number");

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, NULL, RSSPECAN_ATTR_HRP_UWB_MEASUREMENT_DEFAULT_VALUES, defaultValues),
            3, "Default Value");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Sets the oversampling factor.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR oversamplingFactor/Sets the oversampling factor.
ViStatus _VI_FUNC rsspecan_ConfigureHRPUWBOversamplingFactor(ViSession instrSession,
                                                             ViInt32   oversamplingFactor)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, NULL, RSSPECAN_ATTR_HRP_UWB_OVERSAMPLING_FACTOR, oversamplingFactor),
            2, "Oversampling Factor");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Sets the center frequency to the channels frequency.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsspecan_HRPUWBSetCenterFrequencyToChannelFrequency(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsspecan_SetAttributeViString(instrSession, NULL, RSSPECAN_ATTR_HRP_UWB_SET_CENTER_FREQUENCY_TO_CHANNEL_FREQUENCY, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Defines the demodulation mode (the demodulation standard).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR mode/Defines the demodulation mode (the demodulation standard).
ViStatus _VI_FUNC rsspecan_ConfigureHRPUWBDemodulationMode(ViSession instrSession,
                                                           ViInt32   mode)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, NULL, RSSPECAN_ATTR_HRP_UWB_DEMODULATION_MODE, mode),
            2, "Mode");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Configures the Demodulation Payload.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR phyDataRateMode/Selects PHY Mode/Rate modes. In 'BPRF' mode, 'DRBM_LP' and 'DRBM_HP'
/// HIPAR phyDataRateMode/can be selected. In 'HPRF' mode, 'DRHM_LR' and 'DRHM_HR' can be
/// HIPAR phyDataRateMode/selected.
ViStatus _VI_FUNC rsspecan_ConfigureHRPUWBDemodulationPHYDataRateMode(
                            ViSession instrSession,
                            ViInt32   phyDataRateMode)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, NULL, RSSPECAN_ATTR_HRP_UWB_DEMODULATION_PHY_DATA_RATE_MODE, phyDataRateMode),
            2, "PHY Data Rate mode");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Defines the maximum payload size in 'HPRF' mode. It influences how
/// HIFN  A0A1-Bits in the header are handled.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR hprfPayloadSize/Defines the maximum payload size in 'HPRF' mode. It influences how
/// HIPAR hprfPayloadSize/A0A1-Bits in the header are handled.
ViStatus _VI_FUNC rsspecan_ConfigureHRPUWBDemodulationHPRFPayloadSize(
                            ViSession instrSession,
                            ViInt32   hprfPayloadSize)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, NULL, RSSPECAN_ATTR_HRP_UWB_DEMODULATION_HPRF_PAYLOAD_SIZE, hprfPayloadSize),
            2, "HPRF Payload Size");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Sets the scrambled timestamp sequence (STS) packet configuration.
/// HIFN     STS settings are only available in 'BPRF' or 'HPRF' mode.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR packetFormat/Sets the scrambled timestamp sequence (STS) packet configuration.
ViStatus _VI_FUNC rsspecan_ConfigureHRPUWBDemodulationPacketFormat(
                            ViSession instrSession,
                            ViInt32   packetFormat)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, NULL, RSSPECAN_ATTR_HRP_UWB_DEMODULATION_PACKET_FORMAT, packetFormat),
            2, "Packet Format");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Configures the STS Active Segment.
/// HIFN     STS settings are only available in 'BPRF' or 'HPRF' mode.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR length/If the packet format is not 'Off', the active STS segment length can
/// HIPAR length/be defined.
/// HIPAR activeSegments/If the packet format is not 'Off', the number of active STS segments
/// HIPAR activeSegments/can be defined.
ViStatus _VI_FUNC rsspecan_ConfigureHRPUWBDemodulationActiveSegment(
                            ViSession instrSession,
                            ViInt32   length,
                            ViInt32   activeSegments)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, NULL, RSSPECAN_ATTR_HRP_UWB_DEMODULATION_ACTIVE_SEGMENT_LENGTH, length),
            2, "Length");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, NULL, RSSPECAN_ATTR_HRP_UWB_DEMODULATION_ACTIVE_SEGMENTS, activeSegments),
            3, "Active Segments");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Enable FCS check of payload either with 2 octet or with 4 octet
/// HIFN  format.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR macFCS/Enable FCS check of payload either with 2 octet or with 4 octet
/// HIPAR macFCS/format.
ViStatus _VI_FUNC rsspecan_ConfigureHRPUWBDemodulationMACFCS(ViSession instrSession,
                                                             ViInt32   macFCS)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, NULL, RSSPECAN_ATTR_HRP_UWB_DEMODULATION_MAC_FCS, macFCS),
            2, "MAC FCS");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Gap between payload and STS section in packet configuration 2.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR gap/Gap between payload and STS section in packet configuration 2.
ViStatus _VI_FUNC rsspecan_ConfigureHRPUWBDemodulationSTSGap(ViSession instrSession,
                                                             ViInt32   gap)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, NULL, RSSPECAN_ATTR_HRP_UWB_DEMODULATION_STS_GAP, gap),
            2, "Gap");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Sets the default group to be analyzed for all displays.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR group/Sets the default group to be analyzed for all displays.
ViStatus _VI_FUNC rsspecan_ConfigureHRPUWBAnalysisRangePacketGroup(
                            ViSession instrSession,
                            ViInt32   group)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, NULL, RSSPECAN_ATTR_HRP_UWB_EVALUATION_RANGE_PACKET_GROUP, group),
            2, "Group");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Sets the statistics group to be analyzed.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR group/Sets the statistics group to be analyzed.
ViStatus _VI_FUNC rsspecan_ConfigureHRPUWBAnalysisRangeStatisticsGroup(
                            ViSession instrSession,
                            ViInt32   group)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, NULL, RSSPECAN_ATTR_HRP_UWB_EVALUATION_RANGE_STATISTICS_GROUP, group),
            2, "Group");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Returns the number of packets analyzed.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR analyzedPackets/Returns the number of packets analyzed.
ViStatus _VI_FUNC rsspecan_QueryHRPUWBAnalysisEvaluationPacketsAnalyzed(
                            ViSession instrSession,
                            ViInt32*  analyzedPackets)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_GetAttributeViInt32(instrSession, NULL, RSSPECAN_ATTR_HRP_UWB_EVALUATION_PACKETS_ANALYZED, analyzedPackets),
            2, "Analyzed Packets");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Sets the packets for the specified packet filter.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR filter/This control selects the UWB Filter.
/// HIPAR set/Sets the packets for the specified packet filter.
ViStatus _VI_FUNC rsspecan_ConfigureHRPUWBAnalysisEvaluationFilterSet(
                            ViSession instrSession,
                            ViInt32   filter,
                            ViString  set)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_HRP_UWB_EVALUATION_FILTER_SET,
            "UWBFilter", filter - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Filter");

    viCheckParm(rsspecan_SetAttributeViString(instrSession, repCap, RSSPECAN_ATTR_HRP_UWB_EVALUATION_FILTER_SET, set),
            3, "Set");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Creates a new packet filter or changes the name of an existing filter.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR filter/This control selects the UWB Filter.
/// HIPAR name/Creates a new packet filter or changes the name of an existing filter.
ViStatus _VI_FUNC rsspecan_ConfigureHRPUWBAnalysisEvaluationFilterName(
                            ViSession instrSession,
                            ViInt32   filter,
                            ViString  name)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_HRP_UWB_EVALUATION_FILTER_NAME,
            "UWBFilter", filter - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Filter");

    viCheckParm(rsspecan_SetAttributeViString(instrSession, repCap, RSSPECAN_ATTR_HRP_UWB_EVALUATION_FILTER_NAME, name),
            3, "Name");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Deletes the specified packet filter.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR filter/This control selects the UWB Filter.
/// HIPAR name/Deletes the specified packet filter.
ViStatus _VI_FUNC rsspecan_ConfigureHRPUWBAnalysisEvaluationFilterDelete(
                            ViSession instrSession,
                            ViInt32   filter,
                            ViString  name)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_HRP_UWB_EVALUATION_FILTER_DELETE,
            "UWBFilter", filter - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Filter");

    viCheckParm(rsspecan_SetAttributeViString(instrSession, repCap, RSSPECAN_ATTR_HRP_UWB_EVALUATION_FILTER_DELETE, name),
            3, "Name");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Configures the Analysis.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR selectedPacket/Sets the number of the packet within its class, referring to packets
/// HIPAR selectedPacket/in the current capture buffer.
ViStatus _VI_FUNC rsspecan_ConfigureHRPUWBAnalysisSelectedPacket(
                            ViSession instrSession,
                            ViInt32   selectedPacket)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, NULL, RSSPECAN_ATTR_HRP_UWB_DEMODULATION_SELECTED_PACKET, selectedPacket),
            2, "Selected Packet");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Configures the Analysis.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR range/Sets the type of range used for evaluating packets.
/// HIPAR count/Sets the number of packets to capture.
ViStatus _VI_FUNC rsspecan_ConfigureHRPUWBAnalysis(ViSession instrSession,
                                                   ViInt32   range,
                                                   ViInt32   count)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, NULL, RSSPECAN_ATTR_HRP_UWB_DEMODULATION_RANGE, range),
            2, "Range");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, NULL, RSSPECAN_ATTR_HRP_UWB_DEMODULATION_COUNT, count),
            3, "Count");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function adds a window to the display.
/// HIFN
/// HIFN     Note(s):
/// HIFN     (1) This function is only available for R&S FSW.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR stringSize/This control sets the allocated length of Returned Window Name string.
/// HIPAR referenceWindowName/String containing the number or name of the window the new window is
/// HIPAR referenceWindowName/inserted next to. In the default state, the name of the window is its
/// HIPAR referenceWindowName/index.
/// HIPAR direction/Direction the window is added relative to the window set with
/// HIPAR direction/"Reference Window Name".
/// HIPAR windowType/Type of result display you want to add.
/// HIPAR returnedWindowName/When adding a new window, the function returns its name (by
ViStatus _VI_FUNC rsspecan_AddHRPUWBWindow(ViSession instrSession,
                                           ViInt32   stringSize,
                                           ViString  referenceWindowName,
                                           ViInt32   direction,
                                           ViInt32   windowType,
                                           ViChar    returnedWindowName[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViChar   response[RS_MAX_MESSAGE_BUF_SIZE];
    ViChar*  p2direction;
    ViChar*  p2type;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &rsspecan_rngLayoutDirection, direction, &p2direction),
                4, "Direction");
    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &RsSpecAn_rngHRPUWBLayoutType, windowType, &p2type),
                5, "Window Type");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "LAY:ADD? '%s',%s,%s", referenceWindowName, p2direction, p2type);
    checkErr(RsCore_QueryViString(instrSession, cmd, response));

    if (returnedWindowName != NULL)
    {
        RsCore_TrimString(response, RS_VAL_TRIM_WHITESPACES_AND_ALL_QUOTES);
        RsCore_CopyToUserBufferAsciiData(returnedWindowName, stringSize, response);
    }

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function replaces a window with another one.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR windowName/String containing the number or name of the window you want to
/// HIPAR windowType/Type of result display you want to add.
ViStatus _VI_FUNC rsspecan_ReplaceHRPUWBWindow(ViSession instrSession,
                                               ViString  windowName,
                                               ViInt32   windowType)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViChar*  p2type;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &RsSpecAn_rngHRPUWBLayoutType, windowType, &p2type),
                3, "Window Type");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "LAY:REPL '%s',%s", windowName, p2type);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  All packet insights data.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR bufferSize/Size of output array.
/// HIPAR packetAll/All packet insights data.
ViStatus _VI_FUNC rsspecan_QueryHRPUWBPacketAll(ViSession instrSession,
                                                ViInt32   window,
                                                ViInt32   bufferSize,
                                                ViChar    packetAll[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_HRP_UWB_PACKET_ALL,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap),
            2, "Window");

    viCheckParm(rsspecan_GetAttributeViString(instrSession, repCap, RSSPECAN_ATTR_HRP_UWB_PACKET_ALL,
            bufferSize, packetAll),
            4, "Packet All");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Returns selected SYNC data.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR selectSYNC/Select SYNC.
/// HIPAR packetSYNC/Returns selected SYNC data.
ViStatus _VI_FUNC rsspecan_QueryHRPUWBPacketSYNC(ViSession instrSession,
                                                 ViInt32   window,
                                                 ViInt32   selectSYNC,
                                                 ViInt32*  packetSYNC)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViAttr   attr = 0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_HRP_UWB_PACKET_SYNC_CODE_INDEX,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap),
            2, "Window");

    switch (selectSYNC)
    {
    case RSSPECAN_HRP_UWB_PACKET_SYNC_CODE_INDEX:
        attr = RSSPECAN_ATTR_HRP_UWB_PACKET_SYNC_CODE_INDEX;
        break;
    case RSSPECAN_HRP_UWB_PACKET_SYNC_CODE_LENGTH:
        attr = RSSPECAN_ATTR_HRP_UWB_PACKET_SYNC_CODE_LENGTH;
        break;
    case RSSPECAN_HRP_UWB_PACKET_SYNC_DELTA_LENGTH:
        attr = RSSPECAN_ATTR_HRP_UWB_PACKET_SYNC_DELTA_LENGTH;
        break;
    case RSSPECAN_HRP_UWB_PACKET_SYNC_SYNC_LENGTH:
        attr = RSSPECAN_ATTR_HRP_UWB_PACKET_SYNC_SYNC_LENGTH;
        break;
    case RSSPECAN_HRP_UWB_PACKET_SYNC_SFD:
        attr = RSSPECAN_ATTR_HRP_UWB_PACKET_SYNC_SFD;
        break;
    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, selectSYNC), 3, "Select SYNC");
        break;
    }

    viCheckParm(rsspecan_GetAttributeViInt32(instrSession, repCap, attr, packetSYNC),
            4, "Packet SYNC");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Returns selected All SYNC data.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR selectSYNC/Select SYNC.
/// HIPAR packetSYNCAll/Code index of all packets.
ViStatus _VI_FUNC rsspecan_QueryHRPUWBPacketSYNCAll(ViSession instrSession,
                                                    ViInt32   window,
                                                    ViInt32   selectSYNC,
                                                    ViInt32*  packetSYNCAll)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViAttr   attr = 0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_HRP_UWB_PACKET_SYNC_CODE_INDEX,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap),
            2, "Window");

    switch (selectSYNC)
    {
    case RSSPECAN_HRP_UWB_PACKET_SYNC_CODE_INDEX:
        attr = RSSPECAN_ATTR_HRP_UWB_PACKET_SYNC_CODE_INDEX_ALL;
        break;
    case RSSPECAN_HRP_UWB_PACKET_SYNC_CODE_LENGTH:
        attr = RSSPECAN_ATTR_HRP_UWB_PACKET_SYNC_CODE_LENGTH_ALL;
        break;
    case RSSPECAN_HRP_UWB_PACKET_SYNC_DELTA_LENGTH:
        attr = RSSPECAN_ATTR_HRP_UWB_PACKET_SYNC_DELTA_LENGTH_ALL;
        break;
    case RSSPECAN_HRP_UWB_PACKET_SYNC_SYNC_LENGTH:
        attr = RSSPECAN_ATTR_HRP_UWB_PACKET_SYNC_SYNC_LENGTH_ALL;
        break;
    case RSSPECAN_HRP_UWB_PACKET_SYNC_SFD:
        attr = RSSPECAN_ATTR_HRP_UWB_PACKET_SYNC_SFD_ALL;
        break;
    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, selectSYNC),
                    3, "Select SYNC");
        break;
    }

    viCheckParm(rsspecan_GetAttributeViInt32(instrSession, repCap, attr, packetSYNCAll),
            4, "Packet SYNC All");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries selected packet data.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR selectPacketData/Select Packet Data.
/// HIPAR packetData/Queries selected packet data.
ViStatus _VI_FUNC rsspecan_QueryHRPUWBPacketData(ViSession instrSession,
                                                 ViInt32   window,
                                                 ViInt32   selectPacketData,
                                                 ViReal64* packetData)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_HRP_UWB_PACKET_DATA_RATE,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap),
            2, "Window");

    switch (selectPacketData)
    {
    case RSSPECAN_HRP_UWB_PACKET_DATA_RATE:
        viCheckParm(rsspecan_GetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_HRP_UWB_PACKET_DATA_RATE, packetData),
                4, "Packet Data");

        break;

    case RSSPECAN_HRP_UWB_PACKET_DATA_LENGTH:
        viCheckParm(rsspecan_GetAttributeViInt32(instrSession, repCap,
                        RSSPECAN_ATTR_HRP_UWB_PACKET_DATA_LENGTH,
                        (ViInt32*)packetData), 4, "Packet Data");
        break;

    case RSSPECAN_HRP_UWB_PACKET_DATA_RANGING_BIT:
        viCheckParm(rsspecan_GetAttributeViInt32(instrSession, repCap,
                        RSSPECAN_ATTR_HRP_UWB_PACKET_DATA_RANGING_BIT,
                        (ViInt32*)packetData), 4, "Packet Data");
        break;

    case RSSPECAN_HRP_UWB_PACKET_DATA_CONSTRAINT_LENGTH:
        viCheckParm(rsspecan_GetAttributeViInt32(instrSession, repCap,
                        RSSPECAN_ATTR_HRP_UWB_PACKET_DATA_CONSTRAINT_LENGTH,
                        (ViInt32*)packetData), 4, "Packet Data");
        break;

    case RSSPECAN_HRP_UWB_PACKET_DATA_SECDEC:
        viCheckParm(rsspecan_GetAttributeViInt32(instrSession, repCap,
                        RSSPECAN_ATTR_HRP_UWB_PACKET_DATA_SECDEC,
                        (ViInt32*)packetData), 4, "Packet Data");
        break;

    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, selectPacketData),
                    3, "Select Packet Data");
        break;
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries selected packet data.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR bufferSize/Size of output array.
/// HIPAR packetDataA0A1/Queries A0A1 packet data.
ViStatus _VI_FUNC rsspecan_QueryHRPUWBPacketDataA0A1(ViSession instrSession,
                                                     ViInt32   window,
                                                     ViInt32   bufferSize,
                                                     ViChar    packetDataA0A1[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_HRP_UWB_PACKET_DATA_A0A1,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap),
            2, "Window");

    viCheckParm(rsspecan_GetAttributeViString(instrSession, repCap, RSSPECAN_ATTR_HRP_UWB_PACKET_DATA_A0A1,
            bufferSize, packetDataA0A1),
            4, "Packet Data A0A1");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries packet data payload.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR bufferSize/Size of output array.
/// HIPAR packetData/Queries packet data payload.
ViStatus _VI_FUNC rsspecan_QueryHRPUWBPacketDataPayload(ViSession instrSession,
                                                        ViInt32   window,
                                                        ViInt32   bufferSize,
                                                        ViChar    packetDataPayload[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_HRP_UWB_PACKET_DATA_PAYLOAD,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap),
            2, "Window");

    viCheckParm(rsspecan_GetAttributeViString(instrSession, repCap, RSSPECAN_ATTR_HRP_UWB_PACKET_DATA_PAYLOAD,
            bufferSize, packetDataPayload),
            4, "Packet Data Payload");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Data rate PHR in Mb/s.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR ratePHR/Data rate PHR in Mb/s.
ViStatus _VI_FUNC rsspecan_QueryHRPUWBPacketDataRatePHR(ViSession instrSession,
                                                        ViInt32   window,
                                                        ViReal64* ratePHR)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_HRP_UWB_PACKET_DATA_RATE_PHR,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap),
            2, "Window");

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_HRP_UWB_PACKET_DATA_RATE_PHR, ratePHR),
            3, "Rate PHR");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries C Burst.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR bufferSize/Size of output array.
/// HIPAR cBurst/Queries C Burst.
ViStatus _VI_FUNC rsspecan_QueryHRPUWBPacketDataCBurst(ViSession instrSession,
                                                       ViInt32   window,
                                                       ViInt32   bufferSize,
                                                       ViChar    cBurst[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_HRP_UWB_PACKET_DATA_C_BURST,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap),
            2, "Window");

    viCheckParm(rsspecan_GetAttributeViString(instrSession, repCap, RSSPECAN_ATTR_HRP_UWB_PACKET_DATA_C_BURST,
            bufferSize, cBurst),
            4, "C Burst");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries H Burst.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR bufferSize/Size of output array.
/// HIPAR hBurst/Queries H Burst.
ViStatus _VI_FUNC rsspecan_QueryHRPUWBPacketDataHBurst(ViSession instrSession,
                                                       ViInt32   window,
                                                       ViInt32   bufferSize,
                                                       ViChar    hBurst[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_HRP_UWB_PACKET_DATA_H_BURST,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap),
            2, "Window");

    viCheckParm(rsspecan_GetAttributeViString(instrSession, repCap, RSSPECAN_ATTR_HRP_UWB_PACKET_DATA_H_BURST,
            bufferSize, hBurst),
            4, "H Burst");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries MAC FCS.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR bufferSize/Size of output array.
/// HIPAR macFCS/Queries MAC FCS.
ViStatus _VI_FUNC rsspecan_QueryHRPUWBPacketDataMACFCS(ViSession instrSession,
                                                       ViInt32   window,
                                                       ViInt32   bufferSize,
                                                       ViChar    macFCS[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_HRP_UWB_PACKET_DATA_MAC_FCS,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap),
            2, "Window");

    viCheckParm(rsspecan_GetAttributeViString(instrSession, repCap, RSSPECAN_ATTR_HRP_UWB_PACKET_DATA_MAC_FCS,
            bufferSize, macFCS),
            4, "MAC FCS");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries Reserved Bit.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR bufferSize/Size of output array.
/// HIPAR reservedBit/Queries Reserved Bit.
ViStatus _VI_FUNC rsspecan_QueryHRPUWBPacketDataReservedBit(ViSession instrSession,
                                                            ViInt32   window,
                                                            ViInt32   bufferSize,
                                                            ViChar    reservedBit[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_HRP_UWB_PACKET_DATA_RESERVED_BIT,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap),
            2, "Window");

    viCheckParm(rsspecan_GetAttributeViString(instrSession, repCap, RSSPECAN_ATTR_HRP_UWB_PACKET_DATA_RESERVED_BIT,
            bufferSize, reservedBit),
            4, "Reserved Bit");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries STS BST.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR bufferSize/Size of output array.
/// HIPAR stsBST/Queries STS BST.
ViStatus _VI_FUNC rsspecan_QueryHRPUWBPacketDataSTSBST(ViSession instrSession,
                                                       ViInt32   window,
                                                       ViInt32   bufferSize,
                                                       ViChar    stsBST[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_HRP_UWB_PACKET_DATA_STS_BST,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap),
            2, "Window");

    viCheckParm(rsspecan_GetAttributeViString(instrSession, repCap, RSSPECAN_ATTR_HRP_UWB_PACKET_DATA_STS_BST,
            bufferSize, stsBST),
            4, "STS BST");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries selected all packet data.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR packetDataAll/Data rate of all packets.
/// HIPAR window/This control selects the measurement window.
/// HIPAR selectPacketData/Select Packet Data.
ViStatus _VI_FUNC rsspecan_QueryHRPUWBPacketDataAll(ViSession instrSession,
                                                    ViInt32   window,
                                                    ViInt32   selectPacketData,
                                                    ViInt32*  packetDataAll)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViAttr   attr = 0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_HRP_UWB_PACKET_DATA_RATE,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap),
            2, "Window");

    switch (selectPacketData)
    {
    case RSSPECAN_HRP_UWB_PACKET_DATA_RATE:
        attr = RSSPECAN_ATTR_HRP_UWB_PACKET_DATA_RATE_ALL;
        break;
    case RSSPECAN_HRP_UWB_PACKET_DATA_LENGTH:
        attr = RSSPECAN_ATTR_HRP_UWB_PACKET_DATA_LENGTH_ALL;
        break;
    case RSSPECAN_HRP_UWB_PACKET_DATA_RANGING_BIT:
        attr = RSSPECAN_ATTR_HRP_UWB_PACKET_DATA_RANGING_BIT_ALL;
        break;
    case RSSPECAN_HRP_UWB_PACKET_DATA_SECDEC:
        attr = RSSPECAN_ATTR_HRP_UWB_PACKET_DATA_SECDEC_ALL;
        break;
    case RSSPECAN_HRP_UWB_PACKET_DATA_CONSTRAINT_LENGTH:
        attr = RSSPECAN_ATTR_HRP_UWB_PACKET_DATA_CONSTRAINT_LENGTH_ALL;
        break;
    case RSSPECAN_HRP_UWB_PACKET_DATA_A0A1:
        attr = RSSPECAN_ATTR_HRP_UWB_PACKET_DATA_A0A1_ALL;
        break;
    case RSSPECAN_HRP_UWB_PACKET_DATA_CBUR:
        attr = RSSPECAN_ATTR_HRP_UWB_PACKET_DATA_C_BURST_ALL;
        break;
    case RSSPECAN_HRP_UWB_PACKET_DATA_HBUR:
        attr = RSSPECAN_ATTR_HRP_UWB_PACKET_DATA_H_BURST_ALL;
        break;
    case RSSPECAN_HRP_UWB_PACKET_DATA_MAC_FCS:
        attr = RSSPECAN_ATTR_HRP_UWB_PACKET_DATA_MAC_FCS_ALL;
        break;
    case RSSPECAN_HRP_UWB_PACKET_DATA_RESERVED_BIT:
        attr = RSSPECAN_ATTR_HRP_UWB_PACKET_DATA_RESERVED_BIT_ALL;
        break;
    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, selectPacketData),
                    3, "Select Packet Data");
        break;
    }

    viCheckParm(rsspecan_GetAttributeViInt32(instrSession, repCap, attr, packetDataAll),
            4, "Packet Data All");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  All result summary data.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR bufferSize/Size of output array.
/// HIPAR resultAll/All result summary data.
ViStatus _VI_FUNC rsspecan_QueryHRPUWBResultAll(ViSession instrSession,
                                                ViInt32   window,
                                                ViInt32   bufferSize,
                                                ViChar    resultAll[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_HRP_UWB_RESULT_ALL,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap),
            2, "Window");

    viCheckParm(rsspecan_GetAttributeViString(instrSession, repCap, RSSPECAN_ATTR_HRP_UWB_RESULT_ALL,
            bufferSize, resultAll),
            3, "Result All");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Checks if the analysis bandwidth is high enough to demodulate the
/// HIFN  selected UWB channel in the signal capture dialog.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channel/This control selects the channel.
/// HIPAR bufferSize/Size of output array.
/// HIPAR resultCompatible/Checks if the analysis bandwidth is high enough to demodulate the
/// HIPAR resultCompatible/selected UWB channel in the signal capture dialog.
ViStatus _VI_FUNC rsspecan_QueryHRPUWBResultCompatible(ViSession instrSession,
                                                       ViInt32   channel,
                                                       ViInt32   bufferSize,
                                                       ViChar    resultCompatible[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   param[RS_PARAM_BUF_SIZE] = "";
    ViInt32* responseLength = 0;

    checkErr(RsCore_LockSession(instrSession));

    snprintf (param, RS_PARAM_BUF_SIZE, "%ld", channel);

    viCheckParm(RsCore_GetAttributeViRawStringWithQparam(instrSession, NULL, RSSPECAN_ATTR_HIDDEN_HRP_UWB_RESULT_COMPATIBLE,
                                                          param, bufferSize, resultCompatible, responseLength), 4, "Result Compatible");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Returns the capture length in chips.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR captureLength/Returns the capture length in chips.
ViStatus _VI_FUNC rsspecan_QueryHRPUWBResultCaptureLength(ViSession instrSession,
                                                          ViInt32*  captureLength)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_GetAttributeViInt32(instrSession, NULL, RSSPECAN_ATTR_HRP_UWB_RESULT_CAPTURE_LENGTH, captureLength),
            2, "Capture Length");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Center frequency offset in Hz of selected packet.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR frequencyOffset/Center frequency offset in Hz of selected packet.
ViStatus _VI_FUNC rsspecan_QueryHRPUWBResultCenterFrequencyOffsetSelPacket(
                            ViSession instrSession,
                            ViInt32   window,
                            ViReal64* frequencyOffset)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_HRP_UWB_RESULT_CENTER_FREQUENCY_OFFSET_SEL_PACKET,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap),
            2, "Window");

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_HRP_UWB_RESULT_CENTER_FREQUENCY_OFFSET_SEL_PACKET, frequencyOffset),
            3, "Frequency Offset");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Average value of center frequency offset in Hz.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR measurementType/Selects measurement type.
/// HIPAR frequencyOffset/Average value of center frequency offset in Hz.
ViStatus _VI_FUNC rsspecan_QueryHRPUWBResultCenterFrequencyOffset(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   measurementType,
                            ViReal64* frequencyOffset)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViAttr   attr = 0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_HRP_UWB_RESULT_CENTER_FREQUENCY_OFFSET_AVERAGE,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap),
            2, "Window");

    switch (measurementType)
    {
    case RSSPECAN_VAL_MEASTYPE_MIN:
        attr = RSSPECAN_ATTR_HRP_UWB_RESULT_CENTER_FREQUENCY_OFFSET_MINIMUM;
        break;
    case RSSPECAN_VAL_MEASTYPE_MAX:
        attr = RSSPECAN_ATTR_HRP_UWB_RESULT_CENTER_FREQUENCY_OFFSET_MAXIMUM;
        break;
    case RSSPECAN_VAL_MEASTYPE_AVER:
        attr = RSSPECAN_ATTR_HRP_UWB_RESULT_CENTER_FREQUENCY_OFFSET_AVERAGE;
        break;
    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, measurementType),
                    3, "Measurement Type");
        break;
    }

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, repCap, attr, frequencyOffset),
            4, "Frequency Offset");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Center frequency offset in ppm of selected packet.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR offsetPPM/Center frequency offset in ppm of selected packet.
ViStatus _VI_FUNC rsspecan_QueryHRPUWBResultCenterFrequencyOffsetPPMSelPacket(
                            ViSession instrSession,
                            ViInt32   window,
                            ViReal64* offsetPPM)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_HRP_UWB_RESULT_CENTER_FREQUENCY_OFFSET_PPM_SEL_PACKET,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap),
            2, "Window");

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_HRP_UWB_RESULT_CENTER_FREQUENCY_OFFSET_PPM_SEL_PACKET, offsetPPM),
            3, "Offset PPM");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Average value of center frequency offset in ppm.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR measurementType/Selects measurement type.
/// HIPAR frequencyOffsetPPM/Average value of center frequency offset in ppm.
ViStatus _VI_FUNC rsspecan_QueryHRPUWBResultCenterFrequencyOffsetPPM(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   measurementType,
                            ViReal64* frequencyOffsetPPM)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViAttr   attr = 0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_HRP_UWB_RESULT_CENTER_FREQUENCY_OFFSET_PPM_AVERAGE,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap),
            2, "Window");

    switch (measurementType)
    {
    case RSSPECAN_VAL_MEASTYPE_MIN:
        attr = RSSPECAN_ATTR_HRP_UWB_RESULT_CENTER_FREQUENCY_OFFSET_PPM_MINIMUM;
        break;
    case RSSPECAN_VAL_MEASTYPE_MAX:
        attr = RSSPECAN_ATTR_HRP_UWB_RESULT_CENTER_FREQUENCY_OFFSET_PPM_MAXIMUM;
        break;
    case RSSPECAN_VAL_MEASTYPE_AVER:
        attr = RSSPECAN_ATTR_HRP_UWB_RESULT_CENTER_FREQUENCY_OFFSET_PPM_AVERAGE;
        break;
    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, measurementType),
                    3, "Measurement Type");
        break;
    }

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, repCap, attr, frequencyOffsetPPM),
            4, "Frequency Offset PPM");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Chip Clock Error in ppm of selected packet.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR chipClockError/Chip Clock Error in ppm of selected packet.
ViStatus _VI_FUNC rsspecan_QueryHRPUWBResultFrequencyChipClockErrorSelPacket(
                            ViSession instrSession,
                            ViInt32   window,
                            ViReal64* chipClockError)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_HRP_UWB_RESULT_FREQUENCY_CHIP_CLOCK_ERROR_SEL_PACKET,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap),
            2, "Window");

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_HRP_UWB_RESULT_FREQUENCY_CHIP_CLOCK_ERROR_SEL_PACKET, chipClockError),
            3, "Chip Clock Error");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Average value of chip clock error in ppm.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR measurementType/Selects measurement type.
/// HIPAR chipClockError/Average value of chip clock error in ppm.
ViStatus _VI_FUNC rsspecan_QueryHRPUWBResultFrequencyChipClockError(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   measurementType,
                            ViReal64* chipClockError)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViAttr   attr = 0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_HRP_UWB_RESULT_FREQUENCY_CHIP_CLOCK_ERROR_AVERAGE,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap),
            2, "Window");

    switch (measurementType)
    {
    case RSSPECAN_VAL_MEASTYPE_MIN:
        attr = RSSPECAN_ATTR_HRP_UWB_RESULT_FREQUENCY_CHIP_CLOCK_ERROR_MINIMUM;
        break;
    case RSSPECAN_VAL_MEASTYPE_MAX:
        attr = RSSPECAN_ATTR_HRP_UWB_RESULT_FREQUENCY_CHIP_CLOCK_ERROR_MAXIMUM;
        break;
    case RSSPECAN_VAL_MEASTYPE_AVER:
        attr = RSSPECAN_ATTR_HRP_UWB_RESULT_FREQUENCY_CHIP_CLOCK_ERROR_AVERAGE;
        break;
    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, measurementType),
                    3, "Measurement Type");
        break;
    }

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, repCap, attr, chipClockError),
            4, "Chip Clock Error");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Symbol Time Jitter in ps of selected packet.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR timeJitterRMS/Symbol Time Jitter in ps of selected packet.
ViStatus _VI_FUNC rsspecan_QueryHRPUWBResultSymbolTimeJitterRMSSelPacket(
                            ViSession instrSession,
                            ViInt32   window,
                            ViReal64* timeJitterRMS)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_HRP_UWB_RESULT_SYMBOL_TIME_JITTER_RMS_SEL_PACKET,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap),
            2, "Window");

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_HRP_UWB_RESULT_SYMBOL_TIME_JITTER_RMS_SEL_PACKET, timeJitterRMS),
            3, "Time Jitter RMS");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Value of symbol time jitter RMS in ps.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR measurementType/Selects measurement type.
/// HIPAR timeJitterRMS/Value of symbol time jitter RMS in ps.
ViStatus _VI_FUNC rsspecan_QueryHRPUWBResultSymbolTimeJitterRMSAverage(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   measurementType,
                            ViReal64* timeJitterRMS)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViAttr   attr = 0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_HRP_UWB_RESULT_SYMBOL_TIME_JITTER_RMS_AVERAGE,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap),
            2, "Window");

    switch (measurementType)
    {
    case RSSPECAN_VAL_MEASTYPE_MIN:
        attr = RSSPECAN_ATTR_HRP_UWB_RESULT_SYMBOL_TIME_JITTER_RMS_MINIMUM;
        break;
    case RSSPECAN_VAL_MEASTYPE_MAX:
        attr = RSSPECAN_ATTR_HRP_UWB_RESULT_SYMBOL_TIME_JITTER_RMS_MAXIMUM;
        break;
    case RSSPECAN_VAL_MEASTYPE_AVER:
        attr = RSSPECAN_ATTR_HRP_UWB_RESULT_SYMBOL_TIME_JITTER_RMS_AVERAGE;
        break;
    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, measurementType),
                    3, "Measurement Type");
        break;
    }

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, repCap, attr, timeJitterRMS),
            4, "Time Jitter RMS");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Chip Time Jitter in ps of selected packet.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR timeJitterRMS/Chip Time Jitter in ps of selected packet.
/// HIPAR window/This control selects the measurement window.
ViStatus _VI_FUNC rsspecan_QueryHRPUWBResultChipTimeJitterRMSSelPacket(
                            ViSession instrSession,
                            ViInt32   window,
                            ViReal64* timeJitterRMS)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_HRP_UWB_RESULT_CHIP_TIME_JITTER_RMS_SEL_PACKET,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap),
            2, "Window");

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_HRP_UWB_RESULT_CHIP_TIME_JITTER_RMS_SEL_PACKET, timeJitterRMS),
            3, "Time Jitter RMS");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Value of chip time jitter RMS in ps.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR measurementType/Selects measurement type.
/// HIPAR timeJitterRMS/Value of chip time jitter RMS in ps.
ViStatus _VI_FUNC rsspecan_QueryHRPUWBResultChipTimeJitterRMSAverage(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   measurementType,
                            ViReal64* timeJitterRMS)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViAttr   attr = 0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_HRP_UWB_RESULT_CHIP_TIME_JITTER_RMS_AVERAGE,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap),
            2, "Window");

    switch (measurementType)
    {
    case RSSPECAN_VAL_MEASTYPE_MIN:
        attr = RSSPECAN_ATTR_HRP_UWB_RESULT_CHIP_TIME_JITTER_RMS_MINIMUM;
        break;
    case RSSPECAN_VAL_MEASTYPE_MAX:
        attr = RSSPECAN_ATTR_HRP_UWB_RESULT_CHIP_TIME_JITTER_RMS_MAXIMUM;
        break;
    case RSSPECAN_VAL_MEASTYPE_AVER:
        attr = RSSPECAN_ATTR_HRP_UWB_RESULT_CHIP_TIME_JITTER_RMS_AVERAGE;
        break;
    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, measurementType),
                    3, "Measurement Type");
        break;
    }

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, repCap, attr, timeJitterRMS),
            4, "Time Jitter RMS");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  NMSE of correlated pulse for selected packet.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR pulseNMSE/NMSE of correlated pulse for selected packet.
ViStatus _VI_FUNC rsspecan_QueryHRPUWBResultCorrelatedPulseNMSESelPacket(
                            ViSession instrSession,
                            ViInt32   window,
                            ViReal64* pulseNMSE)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_HRP_UWB_RESULT_CORRELATED_PULSE_NMSE_SEL_PACKET,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap),
            2, "Window");

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_HRP_UWB_RESULT_CORRELATED_PULSE_NMSE_SEL_PACKET, pulseNMSE),
            3, "Pulse NMSE");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Value of NMSE of correlated pulse.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR measurementType/Selects measurement type.
/// HIPAR nmse/Value of NMSE of correlated pulse.
ViStatus _VI_FUNC rsspecan_QueryHRPUWBResultCorrelatedPulseNMSE(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   measurementType,
                            ViReal64* nmse)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViAttr   attr = 0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_HRP_UWB_RESULT_CORRELATED_PULSE_NMSE_AVERAGE,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap),
            2, "Window");

    switch (measurementType)
    {
    case RSSPECAN_VAL_MEASTYPE_MIN:
        attr = RSSPECAN_ATTR_HRP_UWB_RESULT_CORRELATED_PULSE_NMSE_MINIMUM;
        break;
    case RSSPECAN_VAL_MEASTYPE_MAX:
        attr = RSSPECAN_ATTR_HRP_UWB_RESULT_CORRELATED_PULSE_NMSE_MAXIMUM;
        break;
    case RSSPECAN_VAL_MEASTYPE_AVER:
        attr = RSSPECAN_ATTR_HRP_UWB_RESULT_CORRELATED_PULSE_NMSE_AVERAGE;
        break;
    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, measurementType),
                    3, "Measurement Type");
        break;
    }

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, repCap, attr, nmse),
            4, "NMSE");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Amplitude of the main lobe peak for selected packet.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR mainLobePeak/Amplitude of the main lobe peak for selected packet.
ViStatus _VI_FUNC rsspecan_QueryHRPUWBResultCorrelatedPulseMainLobePeakSelPacket(
                            ViSession instrSession,
                            ViInt32   window,
                            ViReal64* mainLobePeak)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_HRP_UWB_RESULT_CORRELATED_PULSE_MAIN_LOBE_PEAK_SEL_PACKET,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap),
            2, "Window");

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_HRP_UWB_RESULT_CORRELATED_PULSE_MAIN_LOBE_PEAK_SEL_PACKET, mainLobePeak),
            3, "Main Lobe Peak");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Value of amplitude of the main lobe peak.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR measurementType/Selects measurement type.
/// HIPAR mainLobePeak/Value of amplitude of the main lobe peak.
ViStatus _VI_FUNC rsspecan_QueryHRPUWBResultCorrelatedPulseMainLobePeak(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   measurementType,
                            ViReal64* mainLobePeak)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViAttr   attr = 0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_HRP_UWB_RESULT_CORRELATED_PULSE_MAIN_LOBE_PEAK_AVERAGE,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap),
            2, "Window");

    switch (measurementType)
    {
    case RSSPECAN_VAL_MEASTYPE_MIN:
        attr = RSSPECAN_ATTR_HRP_UWB_RESULT_CORRELATED_PULSE_MAIN_LOBE_PEAK_MINIMUM;
        break;
    case RSSPECAN_VAL_MEASTYPE_MAX:
        attr = RSSPECAN_ATTR_HRP_UWB_RESULT_CORRELATED_PULSE_MAIN_LOBE_PEAK_MAXIMUM;
        break;
    case RSSPECAN_VAL_MEASTYPE_AVER:
        attr = RSSPECAN_ATTR_HRP_UWB_RESULT_CORRELATED_PULSE_MAIN_LOBE_PEAK_AVERAGE;
        break;
    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, measurementType),
                    3, "Measurement Type");
        break;
    }

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, repCap, attr, mainLobePeak),
            4, "Main Lobe Peak");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Value of the main lobe width of the selected packet.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR mainLobeWidth/Value of the main lobe width of the selected packet.
ViStatus _VI_FUNC rsspecan_QueryHRPUWBResultCorrelatedPulseMainLobeWidthSelPacket(
                            ViSession instrSession,
                            ViInt32   window,
                            ViReal64* mainLobeWidth)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_HRP_UWB_RESULT_CORRELATED_PULSE_MAIN_LOBE_WIDTH_SEL_PACKET,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap),
            2, "Window");

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_HRP_UWB_RESULT_CORRELATED_PULSE_MAIN_LOBE_WIDTH_SEL_PACKET, mainLobeWidth),
            3, "Main Lobe Width");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Value of the main lobe width.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR measurementType/Selects measurement type.
/// HIPAR mainLobeWidth/Value of the main lobe width.
ViStatus _VI_FUNC rsspecan_QueryHRPUWBResultCorrelatedPulseMainLobeWidth(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   measurementType,
                            ViReal64* mainLobeWidth)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViAttr   attr = 0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_HRP_UWB_RESULT_CORRELATED_PULSE_MAIN_LOBE_WIDTH_AVERAGE,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap),
            2, "Window");

    switch (measurementType)
    {
    case RSSPECAN_VAL_MEASTYPE_MIN:
        attr = RSSPECAN_ATTR_HRP_UWB_RESULT_CORRELATED_PULSE_MAIN_LOBE_WIDTH_MINIMUM;
        break;
    case RSSPECAN_VAL_MEASTYPE_MAX:
        attr = RSSPECAN_ATTR_HRP_UWB_RESULT_CORRELATED_PULSE_MAIN_LOBE_WIDTH_MAXIMUM;
        break;
    case RSSPECAN_VAL_MEASTYPE_AVER:
        attr = RSSPECAN_ATTR_HRP_UWB_RESULT_CORRELATED_PULSE_MAIN_LOBE_WIDTH_AVERAGE;
        break;
    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, measurementType),
                    3, "Measurement Type");
        break;
    }

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, repCap, attr, mainLobeWidth),
            4, "Main Lobe Width");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Passed if main lobe width is above minimum main lobe width for
/// HIFN  selected packet.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR lobeWidthPassed/Passed if main lobe width is above minimum main lobe width for
/// HIPAR lobeWidthPassed/selected packet.
ViStatus _VI_FUNC rsspecan_QueryHRPUWBResultCorrelatedPulseMainLobeWidthPassedSelPacket(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32*  lobeWidthPassed)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_HRP_UWB_RESULT_CORRELATED_PULSE_MAIN_LOBE_WIDTH_PASSED_SEL_PACKET,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap),
            2, "Window");

    viCheckParm(rsspecan_GetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_HRP_UWB_RESULT_CORRELATED_PULSE_MAIN_LOBE_WIDTH_PASSED_SEL_PACKET, lobeWidthPassed),
            3, "Lobe Width Passed");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Passed if main lobe width is above minimum main lobe width for
/// HIFN  average.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR measurementType/Selects measurement type.
/// HIPAR lobeWidthPassed/Passed if main lobe width is above minimum main lobe width for
/// HIPAR lobeWidthPassed/average.
ViStatus _VI_FUNC rsspecan_QueryHRPUWBResultCorrelatedPulseMainLobeWidthPassed(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   measurementType,
                            ViInt32*  lobeWidthPassed)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViAttr   attr = 0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_HRP_UWB_RESULT_CORRELATED_PULSE_MAIN_LOBE_WIDTH_PASSED_AVERAGE,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap),
            2, "Window");

    switch (measurementType)
    {
    case RSSPECAN_VAL_MEASTYPE_MIN:
        attr = RSSPECAN_ATTR_HRP_UWB_RESULT_CORRELATED_PULSE_MAIN_LOBE_WIDTH_PASSED_MINIMUM;
        break;
    case RSSPECAN_VAL_MEASTYPE_MAX:
        attr = RSSPECAN_ATTR_HRP_UWB_RESULT_CORRELATED_PULSE_MAIN_LOBE_WIDTH_PASSED_MAXIMUM;
        break;
    case RSSPECAN_VAL_MEASTYPE_AVER:
        attr = RSSPECAN_ATTR_HRP_UWB_RESULT_CORRELATED_PULSE_MAIN_LOBE_WIDTH_PASSED_AVERAGE;
        break;
    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, measurementType),
                    3, "Measurement Type");
        break;
    }

    viCheckParm(rsspecan_GetAttributeViInt32(instrSession, repCap, attr, lobeWidthPassed),
            4, "Lobe Width Passed");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Value of main lobe min width in ns for selected packet.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR mainLobeMinWidth/Value of main lobe min width in ns for selected packet.
ViStatus _VI_FUNC rsspecan_QueryHRPUWBResultCorrelatedPulseMainLobeMinWidthSelPacket(
                            ViSession instrSession,
                            ViInt32   window,
                            ViReal64* mainLobeMinWidth)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_HRP_UWB_RESULT_CORRELATED_PULSE_MAIN_LOBE_MIN_WIDTH_SEL_PACKET,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap),
            2, "Window");

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_HRP_UWB_RESULT_CORRELATED_PULSE_MAIN_LOBE_MIN_WIDTH_SEL_PACKET, mainLobeMinWidth),
            3, "Main Lobe Min Width");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Value of main lobe min width in ns.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR measurementType/Selects measurement type.
/// HIPAR lobeMinWidth/Average value of main lobe min width in ns.
ViStatus _VI_FUNC rsspecan_QueryHRPUWBResultCorrelatedPulseMainLobeMinWidth(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   measurementType,
                            ViReal64* lobeMinWidth)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViAttr   attr = 0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_HRP_UWB_RESULT_CORRELATED_PULSE_MAIN_LOBE_MIN_WIDTH_AVERAGE,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap),
            2, "Window");

    switch (measurementType)
    {
    case RSSPECAN_VAL_MEASTYPE_MIN:
        attr = RSSPECAN_ATTR_HRP_UWB_RESULT_CORRELATED_PULSE_MAIN_LOBE_MIN_WIDTH_MINIMUM;
        break;
    case RSSPECAN_VAL_MEASTYPE_MAX:
        attr = RSSPECAN_ATTR_HRP_UWB_RESULT_CORRELATED_PULSE_MAIN_LOBE_MIN_WIDTH_MAXIMUM;
        break;
    case RSSPECAN_VAL_MEASTYPE_AVER:
        attr = RSSPECAN_ATTR_HRP_UWB_RESULT_CORRELATED_PULSE_MAIN_LOBE_MIN_WIDTH_AVERAGE;
        break;
    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, measurementType),
                    3, "Measurement Type");
        break;
    }

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, repCap, attr, lobeMinWidth),
            4, "Lobe Min Width");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Location of side lobe peak relative to main lobe peak for selected
/// HIFN  packet.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR lobePeakLocation/Location of side lobe peak relative to main lobe peak for selected
/// HIPAR lobePeakLocation/packet.
ViStatus _VI_FUNC rsspecan_QueryHRPUWBResultCorrelatedPulseSideLobePeakLocationSelPacket(
                            ViSession instrSession,
                            ViInt32   window,
                            ViReal64* lobePeakLocation)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_HRP_UWB_RESULT_CORRELATED_PULSE_SIDE_LOBE_PEAK_LOCATION_SEL_PACKET,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap),
            2, "Window");

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_HRP_UWB_RESULT_CORRELATED_PULSE_SIDE_LOBE_PEAK_LOCATION_SEL_PACKET, lobePeakLocation),
            3, "Lobe Peak Location");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Value of location of side lobe peak relative to main lobe peak.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR measurementType/Selects measurement type.
/// HIPAR lobePeakLocation/Average value of location of side lobe peak relative to main lobe
/// HIPAR lobePeakLocation/peak.
ViStatus _VI_FUNC rsspecan_QueryHRPUWBResultCorrelatedPulseSideLobePeakLocation(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   measurementType,
                            ViReal64* lobePeakLocation)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViAttr   attr = 0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_HRP_UWB_RESULT_CORRELATED_PULSE_SIDE_LOBE_PEAK_LOCATION_AVERAGE,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap),
            2, "Window");

    switch (measurementType)
    {
    case RSSPECAN_VAL_MEASTYPE_MIN:
        attr = RSSPECAN_ATTR_HRP_UWB_RESULT_CORRELATED_PULSE_SIDE_LOBE_PEAK_LOCATION_MINIMUM;
        break;
    case RSSPECAN_VAL_MEASTYPE_MAX:
        attr = RSSPECAN_ATTR_HRP_UWB_RESULT_CORRELATED_PULSE_SIDE_LOBE_PEAK_LOCATION_MAXIMUM;
        break;
    case RSSPECAN_VAL_MEASTYPE_AVER:
        attr = RSSPECAN_ATTR_HRP_UWB_RESULT_CORRELATED_PULSE_SIDE_LOBE_PEAK_LOCATION_AVERAGE;
        break;
    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, measurementType),
                    3, "Measurement Type");
        break;
    }

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, repCap, attr, lobePeakLocation),
            4, "Lobe Peak Location");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Amplitude of side lobe peak for selected packet.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR sideLobePeak/Amplitude of side lobe peak for selected packet.
ViStatus _VI_FUNC rsspecan_QueryHRPUWBResultCorrelatedPulseSideLobePeakSelPacket(
                            ViSession instrSession,
                            ViInt32   window,
                            ViReal64* sideLobePeak)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_HRP_UWB_RESULT_CORRELATED_PULSE_SIDE_LOBE_PEAK_SEL_PACKET,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap),
            2, "Window");

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_HRP_UWB_RESULT_CORRELATED_PULSE_SIDE_LOBE_PEAK_SEL_PACKET, sideLobePeak),
            3, "Side Lobe Peak");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Amplitude of side lobe peak.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR measurementType/Selects measurement type.
/// HIPAR sideLobePeak/Amplitude of side lobe peak.
ViStatus _VI_FUNC rsspecan_QueryHRPUWBResultCorrelatedPulseSideLobePeak(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   measurementType,
                            ViReal64* sideLobePeak)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViAttr   attr = 0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_HRP_UWB_RESULT_CORRELATED_PULSE_SIDE_LOBE_PEAK_AVERAGE,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap),
            2, "Window");

    switch (measurementType)
    {
    case RSSPECAN_VAL_MEASTYPE_MIN:
        attr = RSSPECAN_ATTR_HRP_UWB_RESULT_CORRELATED_PULSE_SIDE_LOBE_PEAK_MINIMUM;
        break;
    case RSSPECAN_VAL_MEASTYPE_MAX:
        attr = RSSPECAN_ATTR_HRP_UWB_RESULT_CORRELATED_PULSE_SIDE_LOBE_PEAK_MAXIMUM;
        break;
    case RSSPECAN_VAL_MEASTYPE_AVER:
        attr = RSSPECAN_ATTR_HRP_UWB_RESULT_CORRELATED_PULSE_SIDE_LOBE_PEAK_AVERAGE;
        break;
    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, measurementType),
                    3, "Measurement Type");
        break;
    }

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, repCap, attr, sideLobePeak),
            4, "Side Lobe Peak");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Passed if side lobe peak is below 30% for selected packet.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR lobePeakPassed/Passed if side lobe peak is below 30% for selected packet.
ViStatus _VI_FUNC rsspecan_QueryHRPUWBResultCorrelatedPulseSideLobePeakPassedSelPacket(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32*  lobePeakPassed)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_HRP_UWB_RESULT_CORRELATED_PULSE_SIDE_LOBE_PEAK_PASSED_SEL_PACKET,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap),
            2, "Window");

    viCheckParm(rsspecan_GetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_HRP_UWB_RESULT_CORRELATED_PULSE_SIDE_LOBE_PEAK_PASSED_SEL_PACKET, lobePeakPassed),
            3, "Lobe Peak Passed");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Passed if side lobe peak is below 30% for average.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR measurementType/Selects measurement type.
/// HIPAR sideLobePeakPassed/Passed if side lobe peak is below 30% for average.
ViStatus _VI_FUNC rsspecan_QueryHRPUWBResultCorrelatedPulseSideLobePeakPassed(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   measurementType,
                            ViInt32*  sideLobePeakPassed)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViAttr   attr = 0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_HRP_UWB_RESULT_CORRELATED_PULSE_SIDE_LOBE_PEAK_PASSED_AVERAGE,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap),
            2, "Window");

    switch (measurementType)
    {
    case RSSPECAN_VAL_MEASTYPE_MIN:
        attr = RSSPECAN_ATTR_HRP_UWB_RESULT_CORRELATED_PULSE_SIDE_LOBE_PEAK_PASSED_MINIMUM;
        break;
    case RSSPECAN_VAL_MEASTYPE_MAX:
        attr = RSSPECAN_ATTR_HRP_UWB_RESULT_CORRELATED_PULSE_SIDE_LOBE_PEAK_PASSED_MAXIMUM;
        break;
    case RSSPECAN_VAL_MEASTYPE_AVER:
        attr = RSSPECAN_ATTR_HRP_UWB_RESULT_CORRELATED_PULSE_SIDE_LOBE_PEAK_PASSED_AVERAGE;
        break;
    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, measurementType),
                    3, "Measurement Type");
        break;
    }

    viCheckParm(rsspecan_GetAttributeViInt32(instrSession, repCap, attr, sideLobePeakPassed),
            4, "Side Lobe Peak Passed");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Passed if time domain pulse meets mask for selected packet.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR pulseMask/Passed if time domain pulse meets mask for selected packet.
ViStatus _VI_FUNC rsspecan_QueryHRPUWBResultPulseMaskSelPacket(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32*  pulseMask)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_HRP_UWB_RESULT_PULSE_MASK_SEL_PACKET,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap),
            2, "Window");

    viCheckParm(rsspecan_GetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_HRP_UWB_RESULT_PULSE_MASK_SEL_PACKET, pulseMask),
            3, "Pulse Mask");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Passed if time domain pulse meets mask for average.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR measurementType/Selects measurement type.
/// HIPAR pulseMask/Passed if time domain pulse meets mask for average.
ViStatus _VI_FUNC rsspecan_QueryHRPUWBResultPulseMask(ViSession instrSession,
                                                      ViInt32   window,
                                                      ViInt32   measurementType,
                                                      ViInt32*  pulseMask)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViAttr   attr = 0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_HRP_UWB_RESULT_PULSE_MASK_AVERAGE,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap),
            2, "Window");

    switch (measurementType)
    {
    case RSSPECAN_VAL_MEASTYPE_MIN:
        attr = RSSPECAN_ATTR_HRP_UWB_RESULT_PULSE_MASK_MINIMUM;
        break;
    case RSSPECAN_VAL_MEASTYPE_MAX:
        attr = RSSPECAN_ATTR_HRP_UWB_RESULT_PULSE_MASK_MAXIMUM;
        break;
    case RSSPECAN_VAL_MEASTYPE_AVER:
        attr = RSSPECAN_ATTR_HRP_UWB_RESULT_PULSE_MASK_AVERAGE;
        break;
    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, measurementType),
                    3, "Measurement Type");
        break;
    }

    viCheckParm(rsspecan_GetAttributeViInt32(instrSession, repCap, attr, pulseMask),
            4, "Pulse Mask");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Passed if time domain pulse rises monotonically for selected packet.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR riseMonotonic/Passed if time domain pulse rises monotonically for selected packet.
ViStatus _VI_FUNC rsspecan_QueryHRPUWBResultPulseRiseMonotonicSelPacket(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32*  riseMonotonic)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_HRP_UWB_RESULT_PULSE_RISE_MONOTONIC_SEL_PACKET,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap),
            2, "Window");

    viCheckParm(rsspecan_GetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_HRP_UWB_RESULT_PULSE_RISE_MONOTONIC_SEL_PACKET, riseMonotonic),
            3, "Rise Monotonic");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Passed if time domain pulse rises monotonically for average.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR measurementType/Selects measurement type.
/// HIPAR pulseRiseMonotonic/Passed if time domain pulse rises monotonically for average.
ViStatus _VI_FUNC rsspecan_QueryHRPUWBResultPulseRiseMonotonic(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   measurementType,
                            ViInt32*  pulseRiseMonotonic)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViAttr   attr = 0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_HRP_UWB_RESULT_PULSE_RISE_MONOTONIC_AVERAGE,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap),
            2, "Window");

    switch (measurementType)
    {
    case RSSPECAN_VAL_MEASTYPE_MIN:
        attr = RSSPECAN_ATTR_HRP_UWB_RESULT_PULSE_RISE_MONOTONIC_MINIMUM;
        break;
    case RSSPECAN_VAL_MEASTYPE_MAX:
        attr = RSSPECAN_ATTR_HRP_UWB_RESULT_PULSE_RISE_MONOTONIC_MAXIMUM;
        break;
    case RSSPECAN_VAL_MEASTYPE_AVER:
        attr = RSSPECAN_ATTR_HRP_UWB_RESULT_PULSE_RISE_MONOTONIC_AVERAGE;
        break;
    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, measurementType),
                    3, "Measurement Type");
        break;
    }

    viCheckParm(rsspecan_GetAttributeViInt32(instrSession, repCap, attr, pulseRiseMonotonic),
            4, "Pulse Rise Monotonic");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Passed if time domain pulse rises time for average.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSSPECAN_ATTR_HRP_UWB_RESULT_PULSE_RISE_TIME_AVERAGE
/// HIFN     RSSPECAN_ATTR_HRP_UWB_RESULT_PULSE_RISE_TIME_MINIMUM
/// HIFN     RSSPECAN_ATTR_HRP_UWB_RESULT_PULSE_RISE_TIME_MAXIMUM
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     FETCh<n>:SUMMary:PULSe:RISE:TIME:AVERage?
/// HIFN     FETCh<n>:SUMMary:PULSe:RISE:TIME:MINimum?
/// HIFN     FETCh<n>:SUMMary:PULSe:RISE:TIME:MAXimum?
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR measurementType/Selects measurement type.
/// HIPAR pulseRiseTime/Passed if time domain pulse rises time for average.
ViStatus _VI_FUNC rsspecan_QueryHRPUWBResultPulseRiseTime(ViSession instrSession,
                                                          ViInt32   window,
                                                          ViInt32   measurementType,
                                                          ViInt32*  pulseRiseTime)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViAttr   attr = 0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_HRP_UWB_RESULT_PULSE_RISE_TIME_AVERAGE,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap),
            2, "Window");

    switch (measurementType)
    {
    case RSSPECAN_VAL_MEASTYPE_MIN:
        attr = RSSPECAN_ATTR_HRP_UWB_RESULT_PULSE_RISE_TIME_MINIMUM;
        break;
    case RSSPECAN_VAL_MEASTYPE_MAX:
        attr = RSSPECAN_ATTR_HRP_UWB_RESULT_PULSE_RISE_TIME_MAXIMUM;
        break;
    case RSSPECAN_VAL_MEASTYPE_AVER:
        attr = RSSPECAN_ATTR_HRP_UWB_RESULT_PULSE_RISE_TIME_AVERAGE;
        break;
    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, measurementType),
                    3, "Measurement Type");
        break;
    }

    viCheckParm(rsspecan_GetAttributeViInt32(instrSession, repCap, attr, pulseRiseTime),
            4, "Pulse Rise Time");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Passed if time domain pulse rises time for selected packet.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR riseTime/Passed if time domain pulse rises time for selected packet.
ViStatus _VI_FUNC rsspecan_QueryHRPUWBResultPulseRiseTimeSelPacket(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32*  riseTime)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_HRP_UWB_RESULT_PULSE_RISE_TIME_SEL_PACKET,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap),
            2, "Window");

    viCheckParm(rsspecan_GetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_HRP_UWB_PACKET_DATA_RESERVED_BIT, riseTime),
            3, "Rise Time");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Passed if time domain pulse rises time passed for average.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR measurementType/Selects measurement type.
/// HIPAR riseTimePassed/Passed if time domain pulse rises time passed for average.
ViStatus _VI_FUNC rsspecan_QueryHRPUWBResultPulseRiseTimePassed(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   measurementType,
                            ViInt32*  riseTimePassed)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViAttr   attr = 0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_HRP_UWB_RESULT_PULSE_RISE_TIME_PASSED_MAXIMUM,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap),
            2, "Window");

    switch (measurementType)
    {
    case RSSPECAN_VAL_MEASTYPE_MIN:
        attr = RSSPECAN_ATTR_HRP_UWB_RESULT_PULSE_RISE_TIME_PASSED_MINIMUM;
        break;
    case RSSPECAN_VAL_MEASTYPE_MAX:
        attr = RSSPECAN_ATTR_HRP_UWB_RESULT_PULSE_RISE_TIME_PASSED_MAXIMUM;
        break;
    case RSSPECAN_VAL_MEASTYPE_AVER:
        attr = RSSPECAN_ATTR_HRP_UWB_RESULT_PULSE_RISE_TIME_PASSED_AVERAGE;
        break;
    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, measurementType),
                    3, "Measurement Type");
        break;
    }

    viCheckParm(rsspecan_GetAttributeViInt32(instrSession, repCap, attr, riseTimePassed),
            4, "Rise time Passed");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Passed if time domain pulse rises time passed for selected packet.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR riseTimePassed/Passed if time domain pulse rises time passed for selected packet.
ViStatus _VI_FUNC rsspecan_QueryHRPUWBResultPulseRiseTimePassedSelPacket(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32*  riseTimePassed)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_HRP_UWB_RESULT_PULSE_RISE_TIME_PASSED_SEL_PACKET,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap),
            2, "Window");

    viCheckParm(rsspecan_GetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_HRP_UWB_RESULT_PULSE_RISE_TIME_PASSED_SEL_PACKET, riseTimePassed),
            3, "Rise Time Passed");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Passed if time domain pulse location SFD for average.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR measurementType/Selects measurement type.
/// HIPAR pulseLocationSFD/Passed if time domain pulse location SFD for average.
ViStatus _VI_FUNC rsspecan_QueryHRPUWBResultPulseLocationSFD(ViSession instrSession,
                                                             ViInt32   window,
                                                             ViInt32   measurementType,
                                                             ViInt32*  pulseLocationSFD)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViAttr   attr = 0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_HRP_UWB_RESULT_PULSE_LOCATION_SFD_MAXIMUM,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap),
            2, "Window");

    switch (measurementType)
    {
    case RSSPECAN_VAL_MEASTYPE_MIN:
        attr = RSSPECAN_ATTR_HRP_UWB_RESULT_PULSE_LOCATION_SFD_MINIMUM;
        break;
    case RSSPECAN_VAL_MEASTYPE_MAX:
        attr = RSSPECAN_ATTR_HRP_UWB_RESULT_PULSE_LOCATION_SFD_MAXIMUM;
        break;
    case RSSPECAN_VAL_MEASTYPE_AVER:
        attr = RSSPECAN_ATTR_HRP_UWB_RESULT_PULSE_LOCATION_SFD_AVERAGE;
        break;
    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, measurementType),
                    3, "Measurement Type");
        break;
    }

    viCheckParm(rsspecan_GetAttributeViInt32(instrSession, repCap, attr, pulseLocationSFD),
            4, "Pulse Location SFD");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Passed if time domain pulse location SFD for selected packet.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR locationSFD/Passed if time domain pulse location SFD for selected packet.
ViStatus _VI_FUNC rsspecan_QueryHRPUWBResultPulseLocationSFDSelPacket(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32*  locationSFD)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_HRP_UWB_RESULT_PULSE_LOCATION_SFD_SEL_PACKET,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap),
            2, "Window");

    viCheckParm(rsspecan_GetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_HRP_UWB_RESULT_PULSE_LOCATION_SFD_SEL_PACKET, locationSFD),
            3, "Location SFD");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Passed if time domain pulse location STS for average.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR measurementType/Selects measurement type.
/// HIPAR pulseLocationSTS/Passed if time domain pulse location STS for average.
ViStatus _VI_FUNC rsspecan_QueryHRPUWBResultPulseLocationSTS(ViSession instrSession,
                                                             ViInt32   window,
                                                             ViInt32   measurementType,
                                                             ViInt32*  pulseLocationSTS)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViAttr   attr = 0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_HRP_UWB_RESULT_PULSE_LOCATION_STS_AVERAGE,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap),
            2, "Window");

    switch (measurementType)
    {
    case RSSPECAN_VAL_MEASTYPE_MIN:
        attr = RSSPECAN_ATTR_HRP_UWB_RESULT_PULSE_LOCATION_STS_MINIMUM;
        break;
    case RSSPECAN_VAL_MEASTYPE_MAX:
        attr = RSSPECAN_ATTR_HRP_UWB_RESULT_PULSE_LOCATION_STS_MAXIMUM;
        break;
    case RSSPECAN_VAL_MEASTYPE_AVER:
        attr = RSSPECAN_ATTR_HRP_UWB_RESULT_PULSE_LOCATION_STS_AVERAGE;
        break;
    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, measurementType),
                    3, "Measurement Type");
        break;
    }

    viCheckParm(rsspecan_GetAttributeViInt32(instrSession, repCap, attr, pulseLocationSTS),
            4, "Pulse Location STS");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Passed if time domain pulse location SFD for selected packet.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR locationSTS/Passed if time domain pulse location STS for selected packet.
ViStatus _VI_FUNC rsspecan_QueryHRPUWBResultPulseLocationSTSSelPacket(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32*  locationSTS)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_HRP_UWB_RESULT_PULSE_LOCATION_STS_SEL_PACKET,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap),
            2, "Window");

    viCheckParm(rsspecan_GetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_HRP_UWB_RESULT_PULSE_LOCATION_STS_SEL_PACKET, locationSTS),
            3, "Location STS");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Passed if time domain pulse location SYNC for average.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR measurementType/Selects measurement type.
/// HIPAR pulseLocationSYNC/Passed if time domain pulse location SYNC for average.
ViStatus _VI_FUNC rsspecan_QueryHRPUWBResultPulseLocationSYNC(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   measurementType,
                            ViInt32*  pulseLocationSYNC)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViAttr   attr = 0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_HRP_UWB_RESULT_PULSE_LOCATION_SYNC_AVERAGE,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap),
            2, "Window");

    switch (measurementType)
    {
    case RSSPECAN_VAL_MEASTYPE_MIN:
        attr = RSSPECAN_ATTR_HRP_UWB_RESULT_PULSE_LOCATION_SYNC_MINIMUM;
        break;
    case RSSPECAN_VAL_MEASTYPE_MAX:
        attr = RSSPECAN_ATTR_HRP_UWB_RESULT_PULSE_LOCATION_SYNC_MAXIMUM;
        break;
    case RSSPECAN_VAL_MEASTYPE_AVER:
        attr = RSSPECAN_ATTR_HRP_UWB_RESULT_PULSE_LOCATION_SYNC_AVERAGE;
        break;
    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, measurementType),
                    3, "Measurement Type");
        break;
    }

    viCheckParm(rsspecan_GetAttributeViInt32(instrSession, repCap, attr, pulseLocationSYNC),
            4, "Pulse Location SYNC");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Passed if time domain pulse location SFD for selected packet.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR locationSYNC/Passed if time domain pulse location SYNC for selected packet.
ViStatus _VI_FUNC rsspecan_QueryHRPUWBResultPulseLocationSYNCSelPacket(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32*  locationSYNC)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_HRP_UWB_RESULT_PULSE_LOCATION_SYNC_SEL_PACKET,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap),
            2, "Window");

    viCheckParm(rsspecan_GetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_HRP_UWB_RESULT_PULSE_LOCATION_SYNC_SEL_PACKET, locationSYNC),
            3, "Location SYNC");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  SHR NRMSE in % value of selected packet.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR shrNRMSE/SHR NRMSE in % value of selected packet.
ViStatus _VI_FUNC rsspecan_QueryHRPUWBResultEVMSHRNRMSESelPacket(
                            ViSession instrSession,
                            ViInt32   window,
                            ViReal64* shrNRMSE)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_HRP_UWB_RESULT_EVM_SHR_NRMSE_SEL_PACKET,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap),
            2, "Window");

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_HRP_UWB_RESULT_EVM_SHR_NRMSE_SEL_PACKET, shrNRMSE),
            3, "SHR NMSE");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Value of SHR NRMSE in %.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR measurementType/Selects measurement type.
/// HIPAR shrNRMSE/Value of SHR NRMSE in %.
ViStatus _VI_FUNC rsspecan_QueryHRPUWBResultEVMSHRNRMSE(ViSession instrSession,
                                                        ViInt32   window,
                                                        ViInt32   measurementType,
                                                        ViReal64* shrNRMSE)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViAttr   attr = 0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_HRP_UWB_RESULT_EVM_SHR_NRMSE_AVERAGE,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap),
            2, "Window");

    switch (measurementType)
    {
    case RSSPECAN_VAL_MEASTYPE_MIN:
        attr = RSSPECAN_ATTR_HRP_UWB_RESULT_EVM_SHR_NRMSE_MINIMUM;
        break;
    case RSSPECAN_VAL_MEASTYPE_MAX:
        attr = RSSPECAN_ATTR_HRP_UWB_RESULT_EVM_SHR_NRMSE_MAXIMUM;
        break;
    case RSSPECAN_VAL_MEASTYPE_AVER:
        attr = RSSPECAN_ATTR_HRP_UWB_RESULT_EVM_SHR_NRMSE_AVERAGE;
        break;
    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, measurementType),
                    3, "Measurement Type");
        break;
    }

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, repCap, attr, shrNRMSE),
            4, "SHR NRMSE");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  PHR NRMSE in % value of selected packet.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR phrNRMSE/PHR NRMSE in % value of selected packet.
ViStatus _VI_FUNC rsspecan_QueryHRPUWBResultEVMPHRNRMSESelPacket(
                            ViSession instrSession,
                            ViInt32   window,
                            ViReal64* phrNRMSE)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_HRP_UWB_RESULT_EVM_PHR_NRMSE_SEL_PACKET,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap),
            2, "Window");

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_HRP_UWB_RESULT_EVM_PHR_NRMSE_SEL_PACKET, phrNRMSE),
            3, "PHR NMSE");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Value of PHR NRMSE in %.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR measurementType/Selects measurement type.
/// HIPAR phrNRMSE/Value of PHR NRMSE in %.
ViStatus _VI_FUNC rsspecan_QueryHRPUWBResultEVMPHRNRMSE(ViSession instrSession,
                                                        ViInt32   window,
                                                        ViInt32   measurementType,
                                                        ViReal64* phrNRMSE)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViAttr   attr = 0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_HRP_UWB_RESULT_EVM_PHR_NRMSE_AVERAGE,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap),
            2, "Window");

    switch (measurementType)
    {
    case RSSPECAN_VAL_MEASTYPE_MIN:
        attr = RSSPECAN_ATTR_HRP_UWB_RESULT_EVM_PHR_NRMSE_MINIMUM;
        break;
    case RSSPECAN_VAL_MEASTYPE_MAX:
        attr = RSSPECAN_ATTR_HRP_UWB_RESULT_EVM_PHR_NRMSE_MAXIMUM;
        break;
    case RSSPECAN_VAL_MEASTYPE_AVER:
        attr = RSSPECAN_ATTR_HRP_UWB_RESULT_EVM_PHR_NRMSE_AVERAGE;
        break;
    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, measurementType),
                    3, "Measurement Type");
        break;
    }

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, repCap, attr, phrNRMSE),
            4, "PHR NRMSE");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  PSDU NRMSE in % value of selected packet.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR psduNRMSE/PSDU NRMSE in % value of selected packet.
ViStatus _VI_FUNC rsspecan_QueryHRPUWBResultEVMPSDUNRMSESelPacket(
                            ViSession instrSession,
                            ViInt32   window,
                            ViReal64* psduNRMSE)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_HRP_UWB_RESULT_EVM_PSDU_NRMSE_SEL_PACKET,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap),
            2, "Window");

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_HRP_UWB_RESULT_EVM_PSDU_NRMSE_SEL_PACKET, psduNRMSE),
            3, "PSDU NMSE");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Value of PSDU NRMSE in %.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR measurementType/Selects measurement type.
/// HIPAR psduNRMSE/Value of PSDU NRMSE in %.
ViStatus _VI_FUNC rsspecan_QueryHRPUWBResultEVMPSDUNRMSE(ViSession instrSession,
                                                         ViInt32   window,
                                                         ViInt32   measurementType,
                                                         ViReal64* psduNRMSE)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViAttr   attr = 0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_HRP_UWB_RESULT_EVM_PSDU_NRMSE_AVERAGE,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap),
            2, "Window");

    switch (measurementType)
    {
    case RSSPECAN_VAL_MEASTYPE_MIN:
        attr = RSSPECAN_ATTR_HRP_UWB_RESULT_EVM_PSDU_NRMSE_MINIMUM;
        break;
    case RSSPECAN_VAL_MEASTYPE_MAX:
        attr = RSSPECAN_ATTR_HRP_UWB_RESULT_EVM_PSDU_NRMSE_MAXIMUM;
        break;
    case RSSPECAN_VAL_MEASTYPE_AVER:
        attr = RSSPECAN_ATTR_HRP_UWB_RESULT_EVM_PSDU_NRMSE_AVERAGE;
        break;
    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, measurementType),
                    3, "Measurement Type");
        break;
    }

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, repCap, attr, psduNRMSE),
            4, "PSDU NRMSE");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  STS NRMSE in % value of selected packet
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR stsNRMSE/STS NRMSE in % value of selected packet
ViStatus _VI_FUNC rsspecan_QueryHRPUWBResultEVMSTSNRMSESelPacket(
                            ViSession instrSession,
                            ViInt32   window,
                            ViReal64* stsNRMSE)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_HRP_UWB_RESULT_EVM_STS_NRMSE_SEL_PACKET,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap),
            2, "Window");

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_HRP_UWB_RESULT_EVM_STS_NRMSE_SEL_PACKET, stsNRMSE),
            3, "STS NMSE");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Value of STS NRMSE in %.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR measurementType/Selects measurement type.
/// HIPAR stsNRMSE/Value of STS NRMSE in %.
ViStatus _VI_FUNC rsspecan_QueryHRPUWBResultEVMSTSNRMSE(ViSession instrSession,
                                                        ViInt32   window,
                                                        ViInt32   measurementType,
                                                        ViReal64* stsNRMSE)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViAttr   attr = 0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_HRP_UWB_RESULT_EVM_STS_NRMSE_AVERAGE,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap),
            2, "Window");

    switch (measurementType)
    {
    case RSSPECAN_VAL_MEASTYPE_MIN:
        attr = RSSPECAN_ATTR_HRP_UWB_RESULT_EVM_STS_NRMSE_MINIMUM;
        break;
    case RSSPECAN_VAL_MEASTYPE_MAX:
        attr = RSSPECAN_ATTR_HRP_UWB_RESULT_EVM_STS_NRMSE_MAXIMUM;
        break;
    case RSSPECAN_VAL_MEASTYPE_AVER:
        attr = RSSPECAN_ATTR_HRP_UWB_RESULT_EVM_STS_NRMSE_AVERAGE;
        break;
    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, measurementType),
                    3, "Measurement Type");
        break;
    }

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, repCap, attr, stsNRMSE),
            4, "STS NRMSE");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  PHR Pulse Level in dB of selected packet.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR phrPulseLevel/PHR Pulse Level in dB of selected packet.
ViStatus _VI_FUNC rsspecan_QueryHRPUWBResultEVMPHRPulseLevelSelPacket(
                            ViSession instrSession,
                            ViInt32   window,
                            ViReal64* phrPulseLevel)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_HRP_UWB_RESULT_EVM_PHR_PULSE_LEVEL_SEL_PACKET,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap),
            2, "Window");

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_HRP_UWB_RESULT_EVM_PHR_PULSE_LEVEL_SEL_PACKET, phrPulseLevel),
            3, "PHR Pulse Level");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Value of PHR Pulse Level in dB.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR measurementType/Selects measurement type.
/// HIPAR phrPulseLevel/Value of PHR Pulse Level in dB.
ViStatus _VI_FUNC rsspecan_QueryHRPUWBResultEVMPHRPulseLevel(ViSession instrSession,
                                                             ViInt32   window,
                                                             ViInt32   measurementType,
                                                             ViReal64* phrPulseLevel)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViAttr   attr = 0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_HRP_UWB_RESULT_EVM_PHR_PULSE_LEVEL_AVERAGE,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap),
            2, "Window");

    switch (measurementType)
    {
    case RSSPECAN_VAL_MEASTYPE_MIN:
        attr = RSSPECAN_ATTR_HRP_UWB_RESULT_EVM_PHR_PULSE_LEVEL_MINIMUM;
        break;
    case RSSPECAN_VAL_MEASTYPE_MAX:
        attr = RSSPECAN_ATTR_HRP_UWB_RESULT_EVM_PHR_PULSE_LEVEL_MAXIMUM;
        break;
    case RSSPECAN_VAL_MEASTYPE_AVER:
        attr = RSSPECAN_ATTR_HRP_UWB_RESULT_EVM_PHR_PULSE_LEVEL_AVERAGE;
        break;
    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, measurementType),
                    3, "Measurement Type");
        break;
    }

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, repCap, attr, phrPulseLevel),
            4, "PHR Pulse Level");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  PSDU pulse level in dB of selected packet.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR psduPulseLevel/PSDU pulse level in dB of selected packet.
ViStatus _VI_FUNC rsspecan_QueryHRPUWBResultEVMPSDUPulseLevelSelPacket(
                            ViSession instrSession,
                            ViInt32   window,
                            ViReal64* psduPulseLevel)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_HRP_UWB_RESULT_EVM_PSDU_PULSE_LEVEL_SEL_PACKET,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap),
            2, "Window");

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_HRP_UWB_RESULT_EVM_PSDU_PULSE_LEVEL_SEL_PACKET, psduPulseLevel),
            3, "PSDU Pulse Level");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Value of PSDU pulse level in dB.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR measurementType/Selects measurement type.
/// HIPAR psduPulseLevel/Value of PSDU pulse level in dB.
ViStatus _VI_FUNC rsspecan_QueryHRPUWBResultEVMPSDUPulseLevel(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   measurementType,
                            ViReal64* psduPulseLevel)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViAttr   attr = 0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_HRP_UWB_RESULT_EVM_PSDU_PULSE_LEVEL_AVERAGE,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap),
            2, "Window");

    switch (measurementType)
    {
    case RSSPECAN_VAL_MEASTYPE_MIN:
        attr = RSSPECAN_ATTR_HRP_UWB_RESULT_EVM_PSDU_PULSE_LEVEL_MINIMUM;
        break;
    case RSSPECAN_VAL_MEASTYPE_MAX:
        attr = RSSPECAN_ATTR_HRP_UWB_RESULT_EVM_PSDU_PULSE_LEVEL_MAXIMUM;
        break;
    case RSSPECAN_VAL_MEASTYPE_AVER:
        attr = RSSPECAN_ATTR_HRP_UWB_RESULT_EVM_PSDU_PULSE_LEVEL_AVERAGE;
        break;
    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, measurementType),
                    3, "Measurement Type");
        break;
    }

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, repCap, attr, psduPulseLevel),
            4, "PSDU Pulse Level");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  STS pulse level in dB of selected packet.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR stsPulseLevel/STS pulse level in dB of selected packet.
ViStatus _VI_FUNC rsspecan_QueryHRPUWBResultEVMSTSPulseLevelSelPacket(
                            ViSession instrSession,
                            ViInt32   window,
                            ViReal64* stsPulseLevel)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_HRP_UWB_RESULT_EVM_STS_PULSE_LEVEL_SEL_PACKET,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap),
            2, "Window");

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_HRP_UWB_RESULT_EVM_STS_PULSE_LEVEL_SEL_PACKET, stsPulseLevel),
            3, "STS Pulse Level");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Value of STS pulse level in dB.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR measurementType/Selects measurement type.
/// HIPAR stsPulseLevel/Value of STS pulse level in dB.
ViStatus _VI_FUNC rsspecan_QueryHRPUWBResultEVMSTSPulseLevel(ViSession instrSession,
                                                             ViInt32   window,
                                                             ViInt32   measurementType,
                                                             ViReal64* stsPulseLevel)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViAttr   attr = 0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_HRP_UWB_RESULT_EVM_STS_PULSE_LEVEL_AVERAGE,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap),
            2, "Window");

    switch (measurementType)
    {
    case RSSPECAN_VAL_MEASTYPE_MIN:
        attr = RSSPECAN_ATTR_HRP_UWB_RESULT_EVM_STS_PULSE_LEVEL_MINIMUM;
        break;
    case RSSPECAN_VAL_MEASTYPE_MAX:
        attr = RSSPECAN_ATTR_HRP_UWB_RESULT_EVM_STS_PULSE_LEVEL_MAXIMUM;
        break;
    case RSSPECAN_VAL_MEASTYPE_AVER:
        attr = RSSPECAN_ATTR_HRP_UWB_RESULT_EVM_STS_PULSE_LEVEL_AVERAGE;
        break;
    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, measurementType),
                    3, "Measurement Type");
        break;
    }

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, repCap, attr, stsPulseLevel),
            4, "STS Pulse Level");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  SHR mean value in dBm of selected packet.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR shrMean/SHR mean value in dBm of selected packet.
ViStatus _VI_FUNC rsspecan_QueryHRPUWBResultSHRMeanSelPacket(ViSession instrSession,
                                                             ViInt32   window,
                                                             ViReal64* shrMean)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_HRP_UWB_RESULT_SHR_MEAN_SEL_PACKET,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap),
            2, "Window");

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_HRP_UWB_RESULT_SHR_MEAN_SEL_PACKET, shrMean),
            3, "SHR Mean");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Value of SHR mean in dBm.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR measurementType/Selects measurement type.
/// HIPAR shrMean/Value of SHR mean in dBm.
ViStatus _VI_FUNC rsspecan_QueryHRPUWBResultSHRMean(ViSession instrSession,
                                                    ViInt32   window,
                                                    ViInt32   measurementType,
                                                    ViReal64* shrMean)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViAttr   attr = 0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_HRP_UWB_RESULT_SHR_MEAN_AVERAGE,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap),
            2, "Window");

    switch (measurementType)
    {
    case RSSPECAN_VAL_MEASTYPE_MIN:
        attr = RSSPECAN_ATTR_HRP_UWB_RESULT_SHR_MEAN_MINIMUM;
        break;
    case RSSPECAN_VAL_MEASTYPE_MAX:
        attr = RSSPECAN_ATTR_HRP_UWB_RESULT_SHR_MEAN_MAXIMUM;
        break;
    case RSSPECAN_VAL_MEASTYPE_AVER:
        attr = RSSPECAN_ATTR_HRP_UWB_RESULT_SHR_MEAN_AVERAGE;
        break;
    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, measurementType),
                    3, "Measurement Type");
        break;
    }

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, repCap, attr, shrMean),
            4, "SHR Mean");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  SHR peak value in dBm of selected packet.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR shrPeak/SHR peak value in dBm of selected packet.
ViStatus _VI_FUNC rsspecan_QueryHRPUWBResultSHRPeakSelPacket(ViSession instrSession,
                                                             ViInt32   window,
                                                             ViReal64* shrPeak)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_HRP_UWB_RESULT_SHR_PEAK_SEL_PACKET,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap),
            2, "Window");

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_HRP_UWB_RESULT_SHR_PEAK_SEL_PACKET, shrPeak),
            3, "SHR Peak");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Value of SHR peak in dBm.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR measurementType/Selects measurement type.
/// HIPAR shrPeak/Value of SHR peak in dBm.
ViStatus _VI_FUNC rsspecan_QueryHRPUWBResultSHRPeak(ViSession instrSession,
                                                    ViInt32   window,
                                                    ViInt32   measurementType,
                                                    ViReal64* shrPeak)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViAttr   attr = 0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_HRP_UWB_RESULT_SHR_PEAK_AVERAGE,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap),
            2, "Window");

    switch (measurementType)
    {
    case RSSPECAN_VAL_MEASTYPE_MIN:
        attr = RSSPECAN_ATTR_HRP_UWB_RESULT_SHR_PEAK_MINIMUM;
        break;
    case RSSPECAN_VAL_MEASTYPE_MAX:
        attr = RSSPECAN_ATTR_HRP_UWB_RESULT_SHR_PEAK_MAXIMUM;
        break;
    case RSSPECAN_VAL_MEASTYPE_AVER:
        attr = RSSPECAN_ATTR_HRP_UWB_RESULT_SHR_PEAK_AVERAGE;
        break;
    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, measurementType),
                    3, "Measurement Type");
        break;
    }

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, repCap, attr, shrPeak),
            4, "SHR Peak");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  PSDU mean value in dBm of selected packet.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR psduMean/PSDU mean value in dBm of selected packet.
ViStatus _VI_FUNC rsspecan_QueryHRPUWBResultPSDUMeanSelPacket(
                            ViSession instrSession,
                            ViInt32   window,
                            ViReal64* psduMean)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_HRP_UWB_RESULT_PSDU_MEAN_SEL_PACKET,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap),
            2, "Window");

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_HRP_UWB_RESULT_PSDU_MEAN_SEL_PACKET, psduMean),
            3, "PSDU Mean");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Value of PSDU mean in dBm.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR measurementType/Selects measurement type.
/// HIPAR psduMean/Value of PSDU mean in dBm.
ViStatus _VI_FUNC rsspecan_QueryHRPUWBResultPSDUMean(ViSession instrSession,
                                                     ViInt32   window,
                                                     ViInt32   measurementType,
                                                     ViReal64* psduMean)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViAttr   attr = 0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_HRP_UWB_RESULT_PSDU_MEAN_AVERAGE,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap),
            2, "Window");

    switch (measurementType)
    {
    case RSSPECAN_VAL_MEASTYPE_MIN:
        attr = RSSPECAN_ATTR_HRP_UWB_RESULT_PSDU_MEAN_MINIMUM;
        break;
    case RSSPECAN_VAL_MEASTYPE_MAX:
        attr = RSSPECAN_ATTR_HRP_UWB_RESULT_PSDU_MEAN_MAXIMUM;
        break;
    case RSSPECAN_VAL_MEASTYPE_AVER:
        attr = RSSPECAN_ATTR_HRP_UWB_RESULT_PSDU_MEAN_AVERAGE;
        break;
    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, measurementType),
                    3, "Measurement Type");
        break;
    }

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, repCap, attr, psduMean),
            4, "PSDU Mean");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  PSDU peak value in dBm of selected packet.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR psduPeak/PSDU peak value in dBm of selected packet.
ViStatus _VI_FUNC rsspecan_QueryHRPUWBResultPSDUPeakSelPacket(
                            ViSession instrSession,
                            ViInt32   window,
                            ViReal64* psduPeak)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_HRP_UWB_RESULT_PSDU_PEAK_SEL_PACKET,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap),
            2, "Window");

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_HRP_UWB_RESULT_PSDU_PEAK_SEL_PACKET, psduPeak),
            3, "PSDU Peak");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Value of PSDU peak in dBm.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR measurementType/Selects measurement type.
/// HIPAR psduPeak/Value of PSDU peak in dBm.
ViStatus _VI_FUNC rsspecan_QueryHRPUWBResultPSDUPeak(ViSession instrSession,
                                                     ViInt32   window,
                                                     ViInt32   measurementType,
                                                     ViReal64* psduPeak)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViAttr   attr = 0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_HRP_UWB_RESULT_PSDU_PEAK_AVERAGE,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap),
            2, "Window");

    switch (measurementType)
    {
    case RSSPECAN_VAL_MEASTYPE_MIN:
        attr = RSSPECAN_ATTR_HRP_UWB_RESULT_PSDU_PEAK_MINIMUM;
        break;
    case RSSPECAN_VAL_MEASTYPE_MAX:
        attr = RSSPECAN_ATTR_HRP_UWB_RESULT_PSDU_PEAK_MAXIMUM;
        break;
    case RSSPECAN_VAL_MEASTYPE_AVER:
        attr = RSSPECAN_ATTR_HRP_UWB_RESULT_PSDU_PEAK_AVERAGE;
        break;
    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, measurementType),
                    3, "Measurement Type");
        break;
    }

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, repCap, attr, psduPeak),
            4, "PSDU Peak");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Packet mean value in dBm of selected packet.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR packetMean/Packet mean value in dBm of selected packet.
ViStatus _VI_FUNC rsspecan_QueryHRPUWBResultPacketMeanSelPacket(
                            ViSession instrSession,
                            ViInt32   window,
                            ViReal64* packetMean)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_HRP_UWB_RESULT_PACKET_MEAN_SEL_PACKET,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap),
            2, "Window");

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_HRP_UWB_RESULT_PACKET_MEAN_SEL_PACKET, packetMean),
            3, "Packet Mean");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Value of Packet mean in dBm.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR measurementType/Selects measurement type.
/// HIPAR packetMean/Value of Packet Mean in dBm.
ViStatus _VI_FUNC rsspecan_QueryHRPUWBResultPacketMean(ViSession instrSession,
                                                       ViInt32   window,
                                                       ViInt32   measurementType,
                                                       ViReal64* packetMean)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViAttr   attr = 0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_HRP_UWB_RESULT_PACKET_MEAN_AVERAGE,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap),
            2, "Window");

    switch (measurementType)
    {
    case RSSPECAN_VAL_MEASTYPE_MIN:
        attr = RSSPECAN_ATTR_HRP_UWB_RESULT_PACKET_MEAN_MINIMUM;
        break;
    case RSSPECAN_VAL_MEASTYPE_MAX:
        attr = RSSPECAN_ATTR_HRP_UWB_RESULT_PACKET_MEAN_MAXIMUM;
        break;
    case RSSPECAN_VAL_MEASTYPE_AVER:
        attr = RSSPECAN_ATTR_HRP_UWB_RESULT_PACKET_MEAN_AVERAGE;
        break;
    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, measurementType),
                    3, "Measurement Type");
        break;
    }

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, repCap, attr, packetMean),
            4, "Packet Mean");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Packet peak value in dBm of selected packet.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSSPECAN_ATTR_HRP_UWB_RESULT_PACKET_PEAK_SEL_PACKET
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     FETCh<n>:SUMMary:POWer:PACKet:PEAK?
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR packetPeak/Packet peak value in dBm of selected packet.
ViStatus _VI_FUNC rsspecan_QueryHRPUWBResultPacketPeakSelPacket(
                            ViSession instrSession,
                            ViInt32   window,
                            ViReal64* packetPeak)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_HRP_UWB_RESULT_PACKET_PEAK_SEL_PACKET,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap),
            2, "Window");

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_HRP_UWB_RESULT_PACKET_PEAK_SEL_PACKET, packetPeak),
            3, "Packet Peak");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Value of Packet Peak in dBm.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR measurementType/Selects measurement type.
/// HIPAR packetPeak/Value of Packet Peak in dBm.
ViStatus _VI_FUNC rsspecan_QueryHRPUWBResultPacketPeak(ViSession instrSession,
                                                       ViInt32   window,
                                                       ViInt32   measurementType,
                                                       ViReal64* packetPeak)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViAttr   attr = 0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_HRP_UWB_RESULT_PACKET_PEAK_AVERAGE,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap),
            2, "Window");

    switch (measurementType)
    {
    case RSSPECAN_VAL_MEASTYPE_MIN:
        attr = RSSPECAN_ATTR_HRP_UWB_RESULT_PACKET_PEAK_MINIMUM;
        break;
    case RSSPECAN_VAL_MEASTYPE_MAX:
        attr = RSSPECAN_ATTR_HRP_UWB_RESULT_PACKET_PEAK_MAXIMUM;
        break;
    case RSSPECAN_VAL_MEASTYPE_AVER:
        attr = RSSPECAN_ATTR_HRP_UWB_RESULT_PACKET_PEAK_AVERAGE;
        break;
    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, measurementType),
                    3, "Measurement Type");
        break;
    }

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, repCap, attr, packetPeak),
            4, "Packet Peak");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  IQ Offset value in dBm of selected packet.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR iqOffset/IQ Offset value in dBm of selected packet.
ViStatus _VI_FUNC rsspecan_QueryHRPUWBResultIQOffsetSelPacket(
                            ViSession instrSession,
                            ViInt32   window,
                            ViReal64* iqOffset)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_HRP_UWB_RESULT_IQ_OFFSET_SEL_PACKET,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap),
            2, "Window");

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_HRP_UWB_RESULT_IQ_OFFSET_SEL_PACKET, iqOffset),
            3, "IQ Offset");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Value of IQ Offset in dBm.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR measurementType/Selects measurement type.
/// HIPAR iqOffset/Value of IQ Offset in dBm.
ViStatus _VI_FUNC rsspecan_QueryHRPUWBResultIQOffset(ViSession instrSession,
                                                     ViInt32   window,
                                                     ViInt32   measurementType,
                                                     ViReal64* iqOffset)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViAttr   attr = 0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_HRP_UWB_RESULT_IQ_OFFSET_AVERAGE,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap),
            2, "Window");

    switch (measurementType)
    {
    case RSSPECAN_VAL_MEASTYPE_MIN:
        attr = RSSPECAN_ATTR_HRP_UWB_RESULT_IQ_OFFSET_MINIMUM;
        break;
    case RSSPECAN_VAL_MEASTYPE_MAX:
        attr = RSSPECAN_ATTR_HRP_UWB_RESULT_IQ_OFFSET_MAXIMUM;
        break;
    case RSSPECAN_VAL_MEASTYPE_AVER:
        attr = RSSPECAN_ATTR_HRP_UWB_RESULT_IQ_OFFSET_AVERAGE;
        break;
    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, measurementType),
                    3, "Measurement Type");
        break;
    }

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, repCap, attr, iqOffset),
            4, "IQ Offset");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  STS mean value in dBm of selected packet.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR stsMean/STS Mean value in dBm of selected packet.
ViStatus _VI_FUNC rsspecan_QueryHRPUWBResultSTSMeanSelPacket(ViSession instrSession,
                                                             ViInt32   window,
                                                             ViReal64* stsMean)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_HRP_UWB_RESULT_STS_MEAN_SEL_PACKET,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap),
            2, "Window");

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_HRP_UWB_RESULT_STS_MEAN_SEL_PACKET, stsMean),
            3, "STS Mean");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Value of STS Mean in dBm.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR measurementType/Selects measurement type.
/// HIPAR stsMean/Value of STS Mean in dBm.
ViStatus _VI_FUNC rsspecan_QueryHRPUWBResultSTSMean(ViSession instrSession,
                                                    ViInt32   window,
                                                    ViInt32   measurementType,
                                                    ViReal64* stsMean)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViAttr   attr = 0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_HRP_UWB_RESULT_STS_MEAN_AVERAGE,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap),
            2, "Window");

    switch (measurementType)
    {
    case RSSPECAN_VAL_MEASTYPE_MIN:
        attr = RSSPECAN_ATTR_HRP_UWB_RESULT_STS_MEAN_MINIMUM;
        break;
    case RSSPECAN_VAL_MEASTYPE_MAX:
        attr = RSSPECAN_ATTR_HRP_UWB_RESULT_STS_MEAN_MAXIMUM;
        break;
    case RSSPECAN_VAL_MEASTYPE_AVER:
        attr = RSSPECAN_ATTR_HRP_UWB_RESULT_STS_MEAN_AVERAGE;
        break;
    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, measurementType),
                    3, "Measurement Type");
        break;
    }

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, repCap, attr, stsMean),
            4, "STS Mean");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  STS peak value in dBm of selected packet.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR stsPeak/STS Peak value in dBm of selected packet.
ViStatus _VI_FUNC rsspecan_QueryHRPUWBResultSTSPeakSelPacket(ViSession instrSession,
                                                             ViInt32   window,
                                                             ViReal64* stsPeak)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_HRP_UWB_RESULT_STS_PEAK_SEL_PACKET,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap),
            2, "Window");

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_HRP_UWB_RESULT_STS_PEAK_SEL_PACKET, stsPeak),
            3, "STS Peak");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Value of STS Peak in dBm.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR measurementType/Selects measurement type.
/// HIPAR stsPeak/Value of STS Peak in dBm.
ViStatus _VI_FUNC rsspecan_QueryHRPUWBResultSTSPeak(ViSession instrSession,
                                                    ViInt32   window,
                                                    ViInt32   measurementType,
                                                    ViReal64* stsPeak)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViAttr   attr = 0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_HRP_UWB_RESULT_STS_PEAK_AVERAGE,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap),
            2, "Window");

    switch (measurementType)
    {
    case RSSPECAN_VAL_MEASTYPE_MIN:
        attr = RSSPECAN_ATTR_HRP_UWB_RESULT_STS_PEAK_MINIMUM;
        break;
    case RSSPECAN_VAL_MEASTYPE_MAX:
        attr = RSSPECAN_ATTR_HRP_UWB_RESULT_STS_PEAK_MAXIMUM;
        break;
    case RSSPECAN_VAL_MEASTYPE_AVER:
        attr = RSSPECAN_ATTR_HRP_UWB_RESULT_STS_PEAK_AVERAGE;
        break;
    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, measurementType),
                    3, "Measurement Type");
        break;
    }

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, repCap, attr, stsPeak),
            4, "STS Peak");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Max spectral power value in dBm/MHz of selected packet.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR maxSpectralPower/Max spectral power value in dBm/MHz of selected packet.
ViStatus _VI_FUNC rsspecan_QueryHRPUWBResultMaxSpectralPowerSelPacket(
                            ViSession instrSession,
                            ViInt32   window,
                            ViReal64* maxSpectralPower)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_HRP_UWB_RESULT_MAX_SPECTRAL_POWER_SEL_PACKET,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap),
            2, "Window");

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_HRP_UWB_RESULT_MAX_SPECTRAL_POWER_SEL_PACKET, maxSpectralPower),
            3, "Max Spectral Power");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Value of max spectral power value in dBm/MHz.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR measurementType/Selects measurement type.
/// HIPAR maxSpectralPower/Value of max spectral power value in dBm/MHz.
ViStatus _VI_FUNC rsspecan_QueryHRPUWBResultMaxSpectralPower(ViSession instrSession,
                                                             ViInt32   window,
                                                             ViInt32   measurementType,
                                                             ViReal64* maxSpectralPower)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViAttr   attr = 0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_HRP_UWB_RESULT_MAX_SPECTRAL_POWER_AVERAGE,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap),
            2, "Window");

    switch (measurementType)
    {
    case RSSPECAN_VAL_MEASTYPE_MIN:
        attr = RSSPECAN_ATTR_HRP_UWB_RESULT_MAX_SPECTRAL_POWER_MINIMUM;
        break;
    case RSSPECAN_VAL_MEASTYPE_MAX:
        attr = RSSPECAN_ATTR_HRP_UWB_RESULT_MAX_SPECTRAL_POWER_MAXIMUM;
        break;
    case RSSPECAN_VAL_MEASTYPE_AVER:
        attr = RSSPECAN_ATTR_HRP_UWB_RESULT_MAX_SPECTRAL_POWER_AVERAGE;
        break;
    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, measurementType),
                    3, "Measurement Type");
        break;
    }

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, repCap, attr, maxSpectralPower),
            4, "Max Spectral Power");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Passed if transmit PSD mask limits are met for selected packet.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR maskPassed/Passed if transmit PSD mask limits are met for selected packet.
ViStatus _VI_FUNC rsspecan_QueryHRPUWBResultSpectrumMaskPassedSelPacket(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32*  maskPassed)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_HRP_UWB_RESULT_SPECTRUM_MASK_PASSED_SEL_PACKET,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap),
            2, "Window");

    viCheckParm(rsspecan_GetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_HRP_UWB_RESULT_SPECTRUM_MASK_PASSED_SEL_PACKET, maskPassed),
            3, "Mask Passed");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Passed if transmit PSD mask limits are met for average.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR measurementType/Selects measurement type.
/// HIPAR spectrumMaskPassed/Passed if transmit PSD mask limits are met for average.
ViStatus _VI_FUNC rsspecan_QueryHRPUWBResultSpectrumMaskPassed(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   measurementType,
                            ViInt32*  spectrumMaskPassed)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViAttr   attr = 0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_HRP_UWB_RESULT_SPECTRUM_MASK_PASSED_AVERAGE,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap),
            2, "Window");

    switch (measurementType)
    {
    case RSSPECAN_VAL_MEASTYPE_MIN:
        attr = RSSPECAN_ATTR_HRP_UWB_RESULT_SPECTRUM_MASK_PASSED_MINIMUM;
        break;
    case RSSPECAN_VAL_MEASTYPE_MAX:
        attr = RSSPECAN_ATTR_HRP_UWB_RESULT_SPECTRUM_MASK_PASSED_MAXIMUM;
        break;
    case RSSPECAN_VAL_MEASTYPE_AVER:
        attr = RSSPECAN_ATTR_HRP_UWB_RESULT_SPECTRUM_MASK_PASSED_AVERAGE;
        break;
    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, measurementType),
                    3, "Measurement Type");
        break;
    }

    viCheckParm(rsspecan_GetAttributeViInt32(instrSession, repCap, attr, spectrumMaskPassed),
            4, "Spectrum Mask Passed");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Ranging marker location in ps.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR rangingMarker/Ranging marker location in ps.
ViStatus _VI_FUNC rsspecan_QueryHRPUWBResultRangingMarker(ViSession instrSession,
                                                          ViInt32   window,
                                                          ViReal64* rangingMarker)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_HRP_UWB_RESULT_RANGING_MARKER,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap),
            2, "Window");

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_HRP_UWB_RESULT_RANGING_MARKER, rangingMarker),
            3, "Ranging Marker");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Ranging marker location in ps.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR marker/This control selects the marker.
/// HIPAR rangingMarker/Ranging marker location in ps.
ViStatus _VI_FUNC rsspecan_QueryHRPUWBResultRangingSelectedMarker(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   marker,
                            ViReal64* rangingMarker)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCapM[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_HRP_UWB_RESULT_RANGING_SELECTED_MARKER,
            "Window", window - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Window");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_HRP_UWB_RESULT_RANGING_SELECTED_MARKER,
            "Marker", marker - 1, RS_REPCAP_BUF_SIZE, repCapM),
            3, "Marker");
    strcat(repCap, ",");
    strcat(repCap, repCapM);

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_HRP_UWB_RESULT_RANGING_SELECTED_MARKER, rangingMarker),
            4, "Ranging Marker");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Returns the list of positions of the start of each burst in the
/// HIFN  magnitude capture trace in window n (position values in seconds).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR bufferSize/Size of output array.
/// HIPAR startAll/Returns the list of positions of the start of each burst in the
/// HIPAR startAll/magnitude capture trace in window n (position values in seconds).
ViStatus _VI_FUNC rsspecan_QueryHRPUWBResultMagnitudeCaptureStartAll(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   bufferSize,
                            ViChar    startAll[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_HRP_UWB_MAGNITUDE_CAPTURE_START_ALL,
            "Window", window - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Window");

    viCheckParm(rsspecan_GetAttributeViString(instrSession, repCap, RSSPECAN_ATTR_HRP_UWB_MAGNITUDE_CAPTURE_START_ALL,
            bufferSize, startAll),
            4, "Start All");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Returns the position of the start of burst l in the magnitude capture
/// HIFN  trace in window n (position value in seconds).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR burst/Sets the burst number.
/// HIPAR start/Returns the position of the start of burst l in the magnitude capture
/// HIPAR start/trace in window n (position value in seconds).
ViStatus _VI_FUNC rsspecan_QueryHRPUWBResultMagnitudeCaptureStart(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   burst,
                            ViReal64* start)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap2[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_HRP_UWB_MAGNITUDE_CAPTURE_START,
            "Window", window - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Window");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_HRP_UWB_MAGNITUDE_CAPTURE_START,
            "Burst", burst - 1, RS_REPCAP_BUF_SIZE, repCap2),
            3, "Burst");
    strcat(repCap, ",");
    strcat(repCap, repCap2);

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_HRP_UWB_MAGNITUDE_CAPTURE_START, start),
            4, "Start");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Returns the list of positions of the end of each burst in the
/// HIFN  magnitude capture trace in window n (position values in seconds).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR bufferSize/Size of output array.
/// HIPAR stopAll/Returns the list of positions of the end of each burst in the
/// HIPAR stopAll/magnitude capture trace in window n (position values in seconds).
ViStatus _VI_FUNC rsspecan_QueryHRPUWBResultMagnitudeCaptureStopAll(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   bufferSize,
                            ViChar    stopAll[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_HRP_UWB_MAGNITUDE_CAPTURE_STOP_ALL,
            "Window", window - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Window");

    viCheckParm(rsspecan_GetAttributeViString(instrSession, repCap, RSSPECAN_ATTR_HRP_UWB_MAGNITUDE_CAPTURE_STOP_ALL,
            bufferSize, stopAll),
            4, "Stop All");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Returns the position of the end of burst l in the magnitude capture
/// HIFN  trace in window n (position value in seconds).
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSSPECAN_ATTR_HRP_UWB_MAGNITUDE_CAPTURE_STOP
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     FETCh<n>:TRACe:HLIGht:STOP<l>?
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR burst/Sets the burst number.
/// HIPAR stop/Returns the position of the end of burst l in the magnitude capture
/// HIPAR stop/trace in window n (position value in seconds).
ViStatus _VI_FUNC rsspecan_QueryHRPUWBResultMagnitudeCaptureStop(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   burst,
                            ViReal64* stop)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap2[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_HRP_UWB_MAGNITUDE_CAPTURE_STOP,
            "Window", window - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Window");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_HRP_UWB_MAGNITUDE_CAPTURE_STOP,
            "Burst", burst - 1, RS_REPCAP_BUF_SIZE, repCap2),
            3, "Burst");
    strcat(repCap, ",");
    strcat(repCap, repCap2);

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_HRP_UWB_MAGNITUDE_CAPTURE_STOP, stop),
            4, "Stop");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Returns the state of burst l in the magnitude capture trace in window
/// HIFN  n. The state can be: SELECTED, OK.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR bufferSize/Size of output array.
/// HIPAR stateAll/Returns the state of burst l in the magnitude capture trace in window
/// HIPAR stateAll/n. The state can be: SELECTED, OK.
ViStatus _VI_FUNC rsspecan_QueryHRPUWBResultMagnitudeCaptureStateAll(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   bufferSize,
                            ViChar    stateAll[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_HRP_UWB_MAGNITUDE_CAPTURE_STATE_ALL,
            "Window", window - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Window");

    viCheckParm(rsspecan_GetAttributeViString(instrSession, repCap, RSSPECAN_ATTR_HRP_UWB_MAGNITUDE_CAPTURE_STATE_ALL,
            bufferSize, stateAll),
            4, "State All");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Returns the list of states of bursts l in the magnitude capture trace
/// HIFN  in window n.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR burst/Sets the burst number.
/// HIPAR state/Returns the list of states of bursts l in the magnitude capture trace
/// HIPAR state/in window n. The state can be: SELECTED, OK.
ViStatus _VI_FUNC rsspecan_QueryHRPUWBResultMagnitudeCaptureState(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   burst,
                            ViInt32*  state)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap2[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_HRP_UWB_MAGNITUDE_CAPTURE_STATE,
            "Window", window - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Window");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_HRP_UWB_MAGNITUDE_CAPTURE_STATE,
            "Burst", burst - 1, RS_REPCAP_BUF_SIZE, repCap2),
            3, "Burst");
    strcat(repCap, ",");
    strcat(repCap, repCap2);

    viCheckParm(rsspecan_GetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_HRP_UWB_MAGNITUDE_CAPTURE_STATE, state),
            4, "State");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Sets the number of bins for the histogram trace results.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR bins/Sets the number of bins for the histogram trace results.
ViStatus _VI_FUNC rsspecan_ConfigureHRPUWBResultDisplayBins(ViSession instrSession,
                                                            ViInt32   window,
                                                            ViInt32   bins)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_HRP_UWB_RESULT_DISPLAY_BINS,
            "Window", window - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Window");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_HRP_UWB_RESULT_DISPLAY_BINS, bins),
            3, "Bins");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Sets the X scale for the histogram trace results.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR xScale/Sets the X scale for the histogram trace results.
ViStatus _VI_FUNC rsspecan_ConfigureHRPUWBResultDisplayXScale(
                            ViSession instrSession,
                            ViInt32   window,
                            ViReal64  xScale)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_HRP_UWB_RESULT_DISPLAY_X_SCALE,
            "Window", window - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Window");

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_HRP_UWB_RESULT_DISPLAY_X_SCALE, xScale),
            3, "X Scale");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Sets the result config configuration to default or user.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR configure/Sets the result config configuration to default or user.
ViStatus _VI_FUNC rsspecan_ConfigureHRPUWBResultDisplayConfig(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   configure)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_HRP_UWB_RESULT_DISPLAY_CONFIG,
            "Window", window - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Window");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_HRP_UWB_RESULT_DISPLAY_CONFIG, configure),
            3, "Configure");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Sets the configuration setting for this window.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR configure/Sets the configuration setting for this window.
ViStatus _VI_FUNC rsspecan_ConfigureHRPUWBResultDisplayConfigure(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   configure)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_HRP_UWB_RESULT_DISPLAY_CONFIGURE,
            "Window", window - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Window");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_HRP_UWB_RESULT_DISPLAY_CONFIGURE, configure),
            3, "Configure");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Sets the configuration setting for this window.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR group/Sets the Group for this window and for any windows this window is
/// HIPAR group/linked to.
ViStatus _VI_FUNC rsspecan_ConfigureHRPUWBResultDisplayGroup(ViSession instrSession,
                                                             ViInt32   window,
                                                             ViInt32   group)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_HRP_UWB_RESULT_DISPLAY_GROUP,
            "Window", window - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Window");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_HRP_UWB_RESULT_DISPLAY_GROUP, group),
            3, "Group");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Links a list of windows to this window. If a window exists in another
/// HIFN  set of linked windows, they are removed from that set.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR link/Links a list of windows to this window. If a window exists in another
/// HIPAR link/set of linked windows, they are removed from that set.
ViStatus _VI_FUNC rsspecan_ConfigureHRPUWBResultDisplayLink(ViSession instrSession,
                                                            ViInt32   window,
                                                            ViString  link)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_HRP_UWB_RESULT_DISPLAY_LINK,
            "Window", window - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Window");

    viCheckParm(rsspecan_SetAttributeViString(instrSession, repCap, RSSPECAN_ATTR_HRP_UWB_RESULT_DISPLAY_LINK, link),
            3, "Link");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Sets the packet for this window and for any other windows this window
/// HIFN  is linked to.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR packet/Sets the packet for this window and for any other windows this window
/// HIPAR packet/is linked to.
ViStatus _VI_FUNC rsspecan_ConfigureHRPUWBResultDisplayPacket(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   packet)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_HRP_UWB_RESULT_DISPLAY_PACKET,
            "Window", window - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Window");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_HRP_UWB_RESULT_DISPLAY_PACKET, packet),
            3, "Packet");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}
