/*****************************************************************************
 *  Rohde&Schwarz Spectrum Analyzer instrument driver
 *  K18 Amplifier Measurement
 *
 *  Original Release: October 2014
 *  By: Marketa Pavlikova
 *
 *  Should you have any technical questions please contact the hotline of
 *  Rohde & Schwarz Vertriebs-GmbH Rohde & Schwarz Support Center
 *
 *  e-mail: CustomerSupport@rohde-schwarz.com
 *
 *****************************************************************************/

#include "rsspecan.h"
#include "rsspecan_internal.h"

static ViString impedanceModeArr[] = {"BB", "RF", NULL};

/*****************************************************************************
 *- RSSpecAn K18 - --------------------------------*
 *****************************************************************************/


/// HIFN  This function sets the instrument to the Amplifier measurement Mode.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/*****************************************************************************
 * Function: Amplifier Mode
 * Purpose: This function sets the instrument to the Amplifier measurement Mode.
 *****************************************************************************/

ViStatus _VI_FUNC rsspecan_AmplifierMode(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_AMPLIFIER_MODE, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function returns some basic information about the reference
/// HIFN  signal that is currently in use.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR sampleRate/This control queries the sample rate of the currently used reference
/// HIPAR sampleRate/signal.
/// HIPAR sampleLength/This control queries the sample length of the currently used reference
/// HIPAR sampleLength/signal.
ViStatus _VI_FUNC rsspecan_GetAmplifierReferenceSignalInformation(
                            ViSession instrSession,
                            ViInt32*  sampleRate,
                            ViInt32*  sampleLength)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_GetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_AMPLIFIER_SAMPLE_RATE, sampleRate),
            2, "Sample Rate");

    viCheckParm(rsspecan_GetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_AMPLIFIER_SAMPLE_LENGTH, sampleLength),
            3, "Sample Length");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Returns the crest factor of the reference signal.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR crestFactor/Returns the crest factor of the reference signal.
ViStatus _VI_FUNC rsspecan_GetAmplifierReferenceSignalCrestFactor(
                            ViSession instrSession,
                            ViReal64* crestFactor)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, NULL, RSSPECAN_ATTR_AMPLIFIER_CREST_FACTOR, crestFactor),
            2, "Crest Factor");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Returns the occupied bandwidth of the reference signal.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR obw/Returns the occupied bandwidth of the reference signal.
ViStatus _VI_FUNC rsspecan_GetAmplifierReferenceSignalOWB(ViSession instrSession,
                                                          ViReal64* obw)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, NULL, RSSPECAN_ATTR_AMPLIFIER_OBW, obw),
            2, "OBW");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function queries the file name and location of the currently used
/// HIFN  reference signal.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR waveformFile/This control queries the file name and location of the currently used
/// HIPAR waveformFile/reference signal.
/// HIPAR bufferSize/Pass the number of bytes in the string you specify for the Waveform
/// HIPAR bufferSize/File parameter.
ViStatus _VI_FUNC rsspecan_GetAmplifierReferenceSignalWaveformFile(
                            ViSession instrSession,
                            ViInt32   bufferSize,
                            ViChar    waveformFile[])
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_GetAttributeViString(instrSession, "", RSSPECAN_ATTR_AMPLIFIER_WAVEFORM_FILE,
            bufferSize, waveformFile),
            3, "Waveform File");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  The "Force ARB Mode" switch forces the signal generator to use its ARB
/// HIFN  mode (arbitrary waveform) rather than its real-time mode, whenever
/// HIFN  possible. As a result, switching between DPD on and off state is
/// HIFN  significantly faster.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR forceARBMode/The "Force ARB Mode" switch forces the signal generator to use its ARB
/// HIPAR forceARBMode/mode (arbitrary waveform) rather than its real-time mode, whenever
/// HIPAR forceARBMode/possible. As a result, switching between DPD on and off state is
/// HIPAR forceARBMode/significantly faster.
ViStatus _VI_FUNC rsspecan_ConfigureAmplifierForceARBMode(ViSession instrSession,
                                                          ViBoolean forceARBMode)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, NULL, RSSPECAN_ATTR_AMPLIFIER_FORCE_ARB_MODE, forceARBMode),
            2, "Force ARB Mode");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function transfers a reference signal designed on the R&S SMW
/// HIFN  into the R&S FSW-K18.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsspecan_AmplifierReadReferenceSignalFromSMW(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_AMPLIFIER_READ_REFERENCE_SIGNAL_FROM_SMW, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function queries the transmission state of the reference signal
/// HIFN  designed on an R&S SMW.Available when you configure the reference
/// HIFN  signal on the R&S SMW.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR transmissionState/This control queries the transmission state of the reference signal
/// HIPAR transmissionState/designed on an R&S SMW.Available when you configure the reference
/// HIPAR transmissionState/signal on the R&S SMW.
ViStatus _VI_FUNC rsspecan_GetAmplifierReferenceSignalTransmissionState(
                            ViSession instrSession,
                            ViInt32*  transmissionState)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_GetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_AMPLIFIER_REFERENCE_SIGNAL_TRANSMISSION_STATE, transmissionState),
            2, "Transmission State");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function configures the reference signal waveform file transfer to generator.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR exportToGenerator/Turns the transfer of the reference signal data to a generator
/// HIPAR exportToGenerator/on and off.
/// HIPAR DUTPeakInputPower/Defines the peak input power of the DUT.
/// HIPAR segment/Selects the segment in a multi-waveform file that should be used to generate the reference signal.
ViStatus _VI_FUNC rsspecan_ConfigureAmplifierReferenceSignalWaveformFileTransfer(
                            ViSession instrSession,
                            ViBoolean exportToGenerator,
                            ViReal64  DUTPeakInputPower)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_AMPLIFIER_REFERENCE_SIGNAL_WAVEFORM_FILE_EXPORT_TO_GENERATOR_ENABLED, exportToGenerator),
            2, "Export To Generator");
    if (exportToGenerator == VI_FALSE)
    {
        viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_AMPLIFIER_REFERENCE_SIGNAL_WAVEFORM_FILE_DUT_PEAK_INPUT_POWER, DUTPeakInputPower),
                3, "DUT Peak Input Power");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function configures the reference signal waveform file segment.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR segment/Selects the segment in a multi-waveform file that should be used to generate the reference signal.
ViStatus _VI_FUNC rsspecan_ConfigureAmplifierReferenceSignalWaveformFileSegment(
                            ViSession instrSession,
                            ViInt32   segment)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_AMPLIFIER_REFERENCE_SIGNAL_WAVEFORM_FILE_SEGMENT, segment),
            2, "Segment");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function selects a waveform file containing a reference signal.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR filePath/This control selects a waveform file containing a reference signal.
ViStatus _VI_FUNC rsspecan_ConfigureAmplifierReferenceSignalWaveformFilePath(
                            ViSession instrSession,
                            ViString  filePath)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_AMPLIFIER_REFERENCE_SIGNAL_WAVEFORM_FILE_PATH, filePath),
            2, "File Path");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function transfers a waveform file that contains a reference
/// HIFN  signal to the R&S SMW.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsspecan_AmplifierWriteWaveformFileToSMW(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_AMPLIFIER_WRITE_WAVEFORM_FILE_TO_SMW, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function queries the transmission state of the reference signal
/// HIFN  to the signal generator.Available when you generate the reference
/// HIFN  signal with a waveform file.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR transmissionState/This control queries the transmission state of the reference signal to
/// HIPAR transmissionState/the signal generator.Available when you generate the reference signal
/// HIPAR transmissionState/with a waveform file.
ViStatus _VI_FUNC rsspecan_GetAmplifierWaveformFileTransmissionState(
                            ViSession instrSession,
                            ViInt32*  transmissionState)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_GetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_AMPLIFIER_WAVEFORM_FILE_TRANSMISSION_STATE, transmissionState),
            2, "Transmission State");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function configures own reference signal.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR clockRate/This control defines the clock (or sample) rate of the reference
/// HIPAR clockRate/signal generated by the signal generator.
/// HIPAR signalBandwidth/This atribute defines the bandwidth of the reference signal.
/// HIPAR sampleLength/This control queries the sample length of the currently used reference
/// HIPAR sampleLength/signal.
/// HIPAR crestFactor/This control defines the crest factor of the reference signal.
/// HIPAR notchWidth/This attribut defines the notch width of a reference signal.
/// HIPAR notchPosition/This control defines the offset of a notch relative to the center
/// HIPAR notchPosition/frequency in the reference signal.
/// HIPAR dutyCycle/This control defines the duty cycle of a pulsed reference signal.
/// HIPAR rampLength/
ViStatus _VI_FUNC rsspecan_ConfigureAmplifierOwnReferenceSignal(
                            ViSession instrSession,
                            ViReal64  clockRate,
                            ViReal64  signalBandwidth,
                            ViInt32   sampleLength,
                            ViReal64  crestFactor,
                            ViReal64  notchWidth,
                            ViReal64  notchPosition,
                            ViInt32   dutyCycle,
                            ViInt32   rampLength)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_AMPLIFIER_GENERATE_OWN_SIGNAL_CLOCK_RATE, clockRate),
            2, "Clock Rate");

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_AMPLIFIER_GENERATE_OWN_SIGNAL_BANDWIDTH, signalBandwidth),
            3, "Signal Bandwidth");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_AMPLIFIER_GENERATE_OWN_SIGNAL_SAMPLE_LENGTH, sampleLength),
            4, "Sample Length");

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_AMPLIFIER_GENERATE_OWN_SIGNAL_CREST_FACTOR, crestFactor),
            5, "Crest Factor");

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_AMPLIFIER_GENERATE_OWN_SIGNAL_NOTCH_WIDTH, notchWidth),
            6, "Notch Width");

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_AMPLIFIER_GENERATE_OWN_SIGNAL_NOTCH_POSITION, notchPosition),
            7, "Notch Possition");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_AMPLIFIER_GENERATE_OWN_SIGNAL_PULSE_DUTY_CYCLE, dutyCycle),
            8, "Duty Cycle");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_AMPLIFIER_GENERATE_OWN_SIGNAL_RAMP_LENGTH, rampLength),
            9, "Ramp Length");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function queries the transmission state of the reference signal
/// HIFN  to the signal generator. Available when you configure the reference
/// HIFN  signal within the R&S FSW-K18.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR transmissionState/This control queries the transmission state of the reference signal to
/// HIPAR transmissionState/the signal generator. Available when you configure the reference signal
/// HIPAR transmissionState/within the R&S FSW-K18.
ViStatus _VI_FUNC rsspecan_GetAmplifierOwnSignalTransmissionState(
                            ViSession instrSession,
                            ViInt32*  transmissionState)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_GetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_AMPLIFIER_GENERATE_OWN_SIGNAL_TRANSMISSION_STATE, transmissionState),
            2, "Transmission State");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function defines a file name for the waveform of the reference
/// HIFN  signal. The file extension (.wv) is added automatically.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR filePath/This attribut defines a file name for the waveform of the reference
/// HIPAR filePath/signal. The file extension (.wv) is added automatically.
ViStatus _VI_FUNC rsspecan_ConfigureAmplifierOwnSignalWaveformFileName(
                            ViSession instrSession,
                            ViString  filePath)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_AMPLIFIER_GENERATE_OWN_SIGNAL_WAVEFORM_FILE_NAME, filePath),
            2, "File Path");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Internally generates the reference signal based on the signal
/// HIFN  characteristics that you have defined.
/// HIFN     The waveform file that has been created is loaded into the DSP of the
/// HIFN  R&S FSW-K18 and is additionally transferred into the ARB of the signal
/// HIFN  generator.
/// HIFN     It may take some time before the waveform file has been successfully
/// HIFN  applied on the generator.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsspecan_GenerateAmplifierOwnSignalAndPlay(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsspecan_SetAttributeViString(instrSession, NULL, RSSPECAN_ATTR_AMPLIFIER_GENERATE_OWN_SIGNAL_GENERATE_AND_PLAY, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function queries the length of the reference signal as shown in
/// HIFN  the "Acquisition"dialog box.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR signalDuration/This control queries the length of the reference signal as shown in
/// HIPAR signalDuration/the "Acquisition"dialog box.
ViStatus _VI_FUNC rsspecan_GetAmplifierReferenceSignalDuration(
                            ViSession instrSession,
                            ViReal64* signalDuration)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_AMPLIFIER_REFERENCE_SIGNAL_DURATION, signalDuration),
            2, "Signal Duration");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function configures Crest Factor Reduction (K548/K18).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR cfrEnabled/Enables the crest factor reduction calculation.
/// HIPAR evmRefSignal/Switches the EVM reference signal.
/// HIPAR crestFactorDelta/Sets the value difference by which you want to change the crest
/// HIPAR crestFactorDelta/factor.
/// HIPAR maxIterations/Sets the crest factor reduction maximum iterations.
ViStatus _VI_FUNC rsspecan_ConfigureAmplifierCrestFactorReduction(
                            ViSession instrSession,
                            ViBoolean cfrEnabled,
                            ViReal64  crestFactorDelta,
                            ViInt32   maxIterations)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, NULL, RSSPECAN_ATTR_AMPLIFIER_CREST_FACTOR_REDUCTION_ENABLED, cfrEnabled),
            2, "CFR Enabled");
    if (cfrEnabled)
    {
        viCheckParm(rsspecan_SetAttributeViReal64(instrSession, NULL, RSSPECAN_ATTR_AMPLIFIER_CREST_FACTOR_REDUCTION_DELTA, crestFactorDelta),
                4, "Crest Factor Delta");

        viCheckParm(rsspecan_SetAttributeViInt32(instrSession, NULL, RSSPECAN_ATTR_AMPLIFIER_CREST_FACTOR_REDUCTION_MAX_ITERATIONS, maxIterations),
                5, "Max Iterations");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function configures Crest Factor Reduction Simple Mode.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR signalBandwidthAuto/Sets the signal bandwidth mode.
/// HIPAR signalBandwidth/Sets the signal bandwidth in manual mode.
/// HIPAR channelSpacingAuto/Sets the crest factor reduction channel spacing mode.
/// HIPAR channelSpacing/Sets the crest factor reduction channel spacing in manual mode.
ViStatus _VI_FUNC rsspecan_ConfigureAmplifierCrestFactorReductionSimpleMode(
                            ViSession instrSession,
                            ViBoolean signalBandwidthAuto,
                            ViInt32   signalBandwidth,
                            ViBoolean channelSpacingAuto,
                            ViInt32   channelSpacing)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, NULL, RSSPECAN_ATTR_AMPLIFIER_CREST_FACTOR_REDUCTION_SIGNAL_BANDWIDTH_AUTO, signalBandwidthAuto),
            2, "Signal bandwidth Auto");
    if (!signalBandwidthAuto)
        viCheckParm(rsspecan_SetAttributeViInt32(instrSession, NULL, RSSPECAN_ATTR_AMPLIFIER_CREST_FACTOR_REDUCTION_SIGNAL_BANDWIDTH, signalBandwidth),
                3, "Signal Bandwidth");

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, NULL, RSSPECAN_ATTR_AMPLIFIER_CREST_FACTOR_REDUCTION_CHANNEL_SPACING_AUTO, channelSpacingAuto),
            4, "Channel Spacing Auto");
    if (!channelSpacingAuto)
        viCheckParm(rsspecan_SetAttributeViInt32(instrSession, NULL, RSSPECAN_ATTR_AMPLIFIER_CREST_FACTOR_REDUCTION_CHANNEL_SPACING, channelSpacing),
                5, "Channel Spacing");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects simple or enhanced filter mode for crest factor reduction.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR filterMode/Selects simple or enhanced filter mode for crest factor reduction.
ViStatus _VI_FUNC rsspecan_ConfigureAmplifierCrestFactorReductionFilterMode(
                            ViSession instrSession,
                            ViInt32   filterMode)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, NULL, RSSPECAN_ATTR_AMPLIFIER_CREST_FACTOR_REDUCTION_FILTER_MODE, filterMode),
            2, "Filter Mode");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Reads the LED status of crest factor reduction filter mode.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR state/Reads the LED status of crest factor reduction filter mode.
ViStatus _VI_FUNC rsspecan_GetAmplifierCrestFactorReductionFilterModeState(
                            ViSession instrSession,
                            ViInt32*  state)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_GetAttributeViInt32(instrSession, NULL, RSSPECAN_ATTR_AMPLIFIER_CREST_FACTOR_REDUCTION_FILTER_MODE_STATE, state),
            2, "State");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Sets and queries the maximum filter order for crest factor reduction.
/// HIFN  Only available for Enhanced filter mode.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR maxFilterOrder/Sets and queries the maximum filter order for crest factor reduction.
/// HIPAR maxFilterOrder/Only available for Enhanced filter mode.
ViStatus _VI_FUNC rsspecan_ConfigureAmplifierCrestFactorReductionMaximumFilterOrder(
                            ViSession instrSession,
                            ViInt32   maxFilterOrder)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, NULL, RSSPECAN_ATTR_AMPLIFIER_CREST_FACTOR_REDUCTION_MAXIMUM_FILTER_ORDER, maxFilterOrder),
            2, "Max Filter Order");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Reads the LED status of crest factor reduction maximum filter order.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR state/Reads the LED status of crest factor reduction maximum filter order.
ViStatus _VI_FUNC rsspecan_QueryAmplifierCrestFactorReductionMaximumFilterOrderState(
                            ViSession instrSession,
                            ViInt32*  state)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_GetAttributeViInt32(instrSession, NULL, RSSPECAN_ATTR_AMPLIFIER_CREST_FACTOR_REDUCTION_MAXIMUM_FILTER_ORDER_STATE, state),
            2, "State");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Sets and queries the passband and stopband frequencies for crest
/// HIFN  factor reduction in Hz. Only available for Enhanced filter mode.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR passbandFrequency/Sets and queries the passband frequency for crest factor reduction in
/// HIPAR passbandFrequency/Hz. Only available for Enhanced filter mode.
/// HIPAR stopbandFrequency/Sets and queries the stopband frequency for crest factor reduction in
/// HIPAR stopbandFrequency/Hz. Only available for Enhanced filter mode.
ViStatus _VI_FUNC rsspecan_ConfigureAmplifierCrestFactorReductionAttenuationFrequencies(
                            ViSession instrSession,
                            ViReal64  passbandFrequency,
                            ViReal64  stopbandFrequency)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, NULL, RSSPECAN_ATTR_AMPLIFIER_CREST_FACTOR_REDUCTION_PASSBAND_FREQUENCY, passbandFrequency),
            2, "Passband Frequency");

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, NULL, RSSPECAN_ATTR_AMPLIFIER_CREST_FACTOR_REDUCTION_STOPBAND_FREQUENCY, stopbandFrequency),
            3, "Stopband Frequency");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Reads the LED status of crest factor reduction passband frequency.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR passbandState/Reads the LED status of crest factor reduction passband frequency.
/// HIPAR stopbandState/Reads the LED status of crest factor reduction stopband frequency.
ViStatus _VI_FUNC rsspecan_GetAmplifierCrestFactorReductionAttenuationFrequenciesState(
                            ViSession instrSession,
                            ViInt32*  passbandState,
                            ViInt32*  stopbandState)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_GetAttributeViInt32(instrSession, NULL, RSSPECAN_ATTR_AMPLIFIER_CREST_FACTOR_REDUCTION_PASSBAND_FREQUENCY_STATE, passbandState),
            2, "Passband State");

    viCheckParm(rsspecan_GetAttributeViInt32(instrSession, NULL, RSSPECAN_ATTR_AMPLIFIER_CREST_FACTOR_REDUCTION_STOPBAND_FREQUENCY_STATE, stopbandState),
            3, "Stopband State");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Transfers a reference signal designed on a signal generator into the
/// HIFN  R&S FSW-K18.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsspecan_ApplyAmplifierCrestFactorReduction(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsspecan_SetAttributeViString(instrSession, NULL, RSSPECAN_ATTR_AMPLIFIER_CREST_FACTOR_REDUCTION_APPLY, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Applies crest factor reduction on the connected signal generator.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsspecan_ApplyAmplifierCrestFactorReductionFromGenerator(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsspecan_SetAttributeViString(instrSession, NULL, RSSPECAN_ATTR_AMPLIFIER_READ_CFR_FROM_GENERATOR_AND_LOAD, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Reads the LED status of crest factor reduction apply on the connected
/// HIFN  signal generator.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR fromGeneratorAndLoadState/Reads the LED status of crest factor reduction apply on the connected
/// HIPAR fromGeneratorAndLoadState/signal generator.
ViStatus _VI_FUNC rsspecan_GetAmplifierCrestFactorReductionFromGenerator(
                            ViSession instrSession,
                            ViInt32*  fromGeneratorAndLoadState)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_GetAttributeViInt32(instrSession, NULL, RSSPECAN_ATTR_AMPLIFIER_READ_CFR_FROM_GENERATOR_AND_LOAD_STATE, fromGeneratorAndLoadState),
            2, "From Generator And Load State");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Switches the EVM reference signal.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR evmRefSignal/Switches the EVM reference signal.
ViStatus _VI_FUNC rsspecan_ConfigureAmplifierCrestFactorReductionEVMRefSignal(
                            ViSession instrSession,
                            ViInt32   evmRefSignal)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, NULL, RSSPECAN_ATTR_AMPLIFIER_CREST_FACTOR_REDUCTION_EVM_REFERENCE_SIGNAL, evmRefSignal),
            2, "EVM Ref Signal");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Reads the LED status of the crest factor reduction calculation.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR cfrEnabledState/Reads the LED status of the crest factor reduction calculation.
ViStatus _VI_FUNC rsspecan_GetAmplifierCrestFactorReductionEnabledState(
                            ViSession instrSession,
                            ViInt32*  cfrEnabledState)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_GetAttributeViInt32(instrSession, NULL, RSSPECAN_ATTR_AMPLIFIER_CREST_FACTOR_REDUCTION_ENABLED_STATE, cfrEnabledState),
            2, "CFR Enabled State");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the crest factor of the waveform after the calculation of the
/// HIFN  resulting crest factor is completed.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR currentCrestFactor/Queries the crest factor of the waveform after the calculation of the
/// HIPAR currentCrestFactor/resulting crest factor is completed.
ViStatus _VI_FUNC rsspecan_GetAmplifierCrestFactorReductionCurrentValue(
                            ViSession instrSession,
                            ViReal64* currentCrestFactor)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, NULL, RSSPECAN_ATTR_AMPLIFIER_CREST_FACTOR_REDUCTION_CURRENT_VALUE, currentCrestFactor),
            2, "Current Crest Factor");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Reads the LED status of the crest factor delta.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR crestFactorDeltaState/Reads the LED status of the crest factor delta.
ViStatus _VI_FUNC rsspecan_GetAmplifierCrestFactorReductionDeltaState(
                            ViSession instrSession,
                            ViInt32*  crestFactorDeltaState)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_GetAttributeViInt32(instrSession, NULL, RSSPECAN_ATTR_AMPLIFIER_CREST_FACTOR_REDUCTION_DELTA_STATE, crestFactorDeltaState),
            2, "Crest Factor Delta State");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Reads the LED status of the crest factor reduction maximum iterations.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR maxIterationsState/Reads the LED status of the crest factor reduction maximum iterations.
ViStatus _VI_FUNC rsspecan_GetAmplifierCrestFactorReductionIterationsState(
                            ViSession instrSession,
                            ViInt32*  maxIterationsState)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_GetAttributeViInt32(instrSession, NULL, RSSPECAN_ATTR_AMPLIFIER_CREST_FACTOR_REDUCTION_ITERATIONS_STATE, maxIterationsState),
            2, "Max Iterations State");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Reads the LED status of the signal bandwith.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR signalBandwidthState/Reads the LED status of the signal bandwith.
ViStatus _VI_FUNC rsspecan_GetAmplifierCrestFactorReductionSignalBandwidthState(
                            ViSession instrSession,
                            ViInt32*  signalBandwidthState)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_GetAttributeViInt32(instrSession, NULL, RSSPECAN_ATTR_AMPLIFIER_CREST_FACTOR_REDUCTION_SIGNAL_BANDWIDTH_STATE, signalBandwidthState),
            2, "Signal Bandwidth State");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Reads the LED status of the crest factor reduction channel spacing.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channelSpacingState/Reads the LED status of the crest factor reduction channel spacing.
ViStatus _VI_FUNC rsspecan_GetAmplifierCrestFactorReductionChannelSpacingState(
                            ViSession instrSession,
                            ViInt32*  channelSpacingState)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_GetAttributeViInt32(instrSession, NULL, RSSPECAN_ATTR_AMPLIFIER_CREST_FACTOR_REDUCTION_CHANNEL_SPACING_STATE, channelSpacingState),
            2, "Channel Spacing State");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Reads the LED status of crest factor reduction apply on the connected
/// HIFN  signal generator.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR applyState/Reads the LED status of crest factor reduction apply on the connected
/// HIPAR applyState/signal generator.
ViStatus _VI_FUNC rsspecan_GetAmplifierCrestFactorReductionApplyState(
                            ViSession instrSession,
                            ViInt32*  applyState)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_GetAttributeViInt32(instrSession, NULL, RSSPECAN_ATTR_AMPLIFIER_CREST_FACTOR_REDUCTION_APPLY_STATE, applyState),
            2, "Apply State");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Determines whether the RF input data is taken from the RF input
/// HIFN  connector or the optional Analog Baseband I connector. This attribute
/// HIFN  is only available if the Analog Baseband interface (R&S FSW-B71) is
/// HIFN  installed and active for input. For more information on the Analog
/// HIFN  Baseband Interface (R&S FSW-B71) see the R&S FSW I/Q Analyzer and I/Q
/// HIFN  Input User Manual.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR inputConnector/Determines whether the RF input data is taken from the RF input
/// HIPAR inputConnector/connector or the optional Analog Baseband I connector. This control is
/// HIPAR inputConnector/only available if the Analog Baseband interface (R&S FSW-B71) is
/// HIPAR inputConnector/installed and active for input. For more information on the Analog
/// HIPAR inputConnector/Baseband Interface (R&S FSW-B71) see the R&S FSW I/Q Analyzer and I/Q
/// HIPAR inputConnector/Input User Manual.
ViStatus _VI_FUNC rsspecan_ConfigureAmplifierInputSourceConnector(
                            ViSession instrSession,
                            ViInt32   inputConnector)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_AMPLIFIER_INPUT_SOURCE_CONNECTOR, inputConnector),
            2, "Input Connector");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function turns simultaneous use of RF input and analog baseband
/// HIFN  input on and off.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR enableParallelBBCapture/This control turns simultaneous use of RF input and analog baseband
/// HIPAR enableParallelBBCapture/input on and off.
ViStatus _VI_FUNC rsspecan_ConfigureAmplifierInputSourceAnalogBasebandCapture(
                            ViSession instrSession,
                            ViInt32   input,
                            ViBoolean enableParallelBBCapture)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_AMPLIFIER_INPUT_SOURCE_ANALOG_BASEBAND_CAPTURE,
            "Inp", input - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Input");

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, repCap, RSSPECAN_ATTR_AMPLIFIER_INPUT_SOURCE_ANALOG_BASEBAND_CAPTURE, enableParallelBBCapture),
            2, "Enable Parallel BB Capture");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function updates the generator settings as defined within
/// HIFN  the R&S FSW-K18.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR generatorIPAddress/This control defines the IP address of the connected signal generator.
/// HIPAR rmsLevel/This control defines the signal generator RMS level.
/// HIPAR attachToFSWFrequency/This control turns synchronization of the analyzer and generator
/// HIPAR attachToFSWFrequency/frequency on and
/// HIPAR centerFrequency/This control defines the frequency of the generator.
/// HIPAR externalOscilator/This control selects the source of the generator reference frequency.
/// HIPAR pathRF/This controls selects the signal path of the R&S SMW used for RF
/// HIPAR pathRF/signal generation.
ViStatus _VI_FUNC rsspecan_ConfigureAmplifierGeneratorSettings(
                            ViSession instrSession,
                            ViString  generatorIPAddress,
                            ViReal64  rmsLevel,
                            ViBoolean attachToFSWFrequency,
                            ViReal64  centerFrequency,
                            ViInt32   externalOscilator,
                            ViInt32   pathRF)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_AMPLIFIER_GENERATOR_IP_ADDRESS, generatorIPAddress),
            2, "IP Address");

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_AMPLIFIER_GENERATOR_RMS_LEVEL, rmsLevel),
            3, "RMS Level");

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_AMPLIFIER_ATTACH_TO_FSW_FREQUENCY, attachToFSWFrequency),
            4, "Attach To FSW Frequency");
    if (attachToFSWFrequency == VI_FALSE)
    {
        viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_AMPLIFIER_GENERATOR_CENTER_FREQUENCY, centerFrequency),
                5, "Center Frequency");
    }
    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_AMPLIFIER_GENERATOR_EXTERNAL_OSCILATOR, externalOscilator),
            6, "External Oscilator");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_AMPLIFIER_GENERATOR_TARGET_PATH_RF, pathRF),
            7, "Path RF");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN Defines a mathematical level offset for the signal generator
/// HIFN (for example to take external attenuation into account).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR levelOffset/Defines a mathematical level offset for the signal generator
ViStatus _VI_FUNC rsspecan_ConfigureAmplifierGeneratorLevelOffset(
                            ViSession instrSession,
                            ViReal64  levelOffset)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_AMPLIFIER_GENERATOR_LEVEL_OFFSET, levelOffset),
            2, "Level Offset");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN Defines digital attenuation that is applied to digitally
/// HIFN modulated I/Q signals.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR attenuation/Defines digital attenuation that is applied to digitally
/// HIPAR attenuation/modulated I/Q signals.
ViStatus _VI_FUNC rsspecan_ConfigureAmplifierGeneratorDigitalAttenuation(
                            ViSession instrSession,
                            ViReal64  attenuation)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_AMPLIFIER_GENERATOR_DIGITAL_ATTENUATION, attenuation),
            2, "Attenuation");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN Turns the RF output on the connected signal generator on and off.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR state/Turns the RF output on the connected signal generator on and off.
ViStatus _VI_FUNC rsspecan_ConfigureAmplifierGeneratorRFOutput(
                            ViSession instrSession,
                            ViBoolean state)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_AMPLIFIER_GENERATOR_RF_OUTPUT, state),
            2, "State");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN Selects the segment in a multi-waveform file that should be uploaded to the signal generator..
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR segment/Selects the segment in a multi-waveform file that should be
/// HIPAR segment/uploaded to the signal generator.
ViStatus _VI_FUNC rsspecan_ConfigureAmplifierGeneratorSegment(
                            ViSession instrSession,
                            ViInt32   segment)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_AMPLIFIER_GENERATOR_SEGMENT, segment),
            2, "Segment");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function queries the led state .
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR type/This control select type.
/// HIPAR ledState/This control queries the led state.
ViStatus _VI_FUNC rsspecan_GetAmplifierGeneratorSetupLedState(
                            ViSession instrSession,
                            ViInt32   type,
                            ViInt32*  ledState)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    switch (type)
    {
    case RSSPECAN_VAL_AMPLIFIER_IP_ADDRESS:
        viCheckParm(rsspecan_GetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_AMPLIFIER_GENERATOR_IP_LED_STATE, ledState),
                3, "Led State");
        break;
    case RSSPECAN_VAL_AMPLIFIER_RMS_LEVEL:
        viCheckParm(rsspecan_GetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_AMPLIFIER_GENERATOR_RMS_LEVEL_STATE, ledState),
                3, "Led State");
        break;
    case RSSPECAN_VAL_AMPLIFIER_CENTER_FREQUENCY:
        viCheckParm(rsspecan_GetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_AMPLIFIER_GENERATOR_CENTER_FREQUENCY_LED_STATE, ledState),
                3, "Led State");
        break;
    case RSSPECAN_VAL_AMPLIFIER_REFERENCE_FREQUENCY:
        viCheckParm(rsspecan_GetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_AMPLIFIER_GENERATOR_EXTERNAL_OSCILATOR_LED_STATE, ledState),
                3, "Led State");
        break;
    case RSSPECAN_VAL_AMPLIFIER_LEVEL_OFFSET:
        viCheckParm(rsspecan_GetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_AMPLIFIER_GENERATOR_LEVEL_OFFSET_STATE, ledState),
                3, "Led State");
        break;
    case RSSPECAN_VAL_AMPLIFIER_SEGMENT:
        viCheckParm(rsspecan_GetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_AMPLIFIER_GENERATOR_SEGMENT_STATE, ledState),
                3, "Led State");
        break;
    case RSSPECAN_VAL_AMPLIFIER_DIGITAL_ATT:
        viCheckParm(rsspecan_GetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_AMPLIFIER_GENERATOR_DIGITAL_ATTENUATION_STATE, ledState),
                3, "Led State");
        break;
    case RSSPECAN_VAL_AMPLIFIER_RF_OUT:
        viCheckParm(rsspecan_GetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_AMPLIFIER_GENERATOR_RF_OUTPUT_STATE, ledState),
                3, "Led State");
        break;
    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, type), 2, "Type");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Defines the maximum generator output power.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR maximumPower/Defines the maximum generator output power.
ViStatus _VI_FUNC rsspecan_ConfigureAmplifierGeneratorMaximumPower(
                            ViSession instrSession,
                            ViReal64  maximumPower)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_AMPLIFIER_GENERATOR_MAXIMUM_POWER, maximumPower),
            2, "Maximum Power");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the maximum output level configuration state on the generator.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR maximumPowerLEDState/Queries the maximum output level configuration state on the generator.
ViStatus _VI_FUNC rsspecan_QueryAmplifierGeneratorMaximumPowerLEDState(
                            ViSession instrSession,
                            ViInt32*  maximumPowerLEDState)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_GetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_AMPLIFIER_GENERATOR_MAXIMUM_POWER_LED_STATE, maximumPowerLEDState),
            2, "Maximum Power LED State");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Provides functionality to configure the signal generator directly
/// HIFN  through the R&SFSW-K18 application. It resends the string parameter as
/// HIFN  a SCPI command to the connected signal generator.
/// HIFN
/// HIFN     If the command contains a '?', use
/// HIFN  RSSPECAN_ATTR_AMPLIFIER_GENERATOR_RELAY_READ to read the answer.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR commandBuffer/Provides functionality to configure the signal generator directly
/// HIPAR commandBuffer/through the R&SFSW-K18 application. It resends the string parameter as
/// HIPAR commandBuffer/a SCPI command to the connected signal generator.
ViStatus _VI_FUNC rsspecan_WriteAmplifierGeneratorCommand(ViSession instrSession,
                                                          ViString  commandBuffer)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViString(instrSession, NULL, RSSPECAN_ATTR_AMPLIFIER_GENERATOR_RELAY_WRITE, commandBuffer),
            2, "Command Buffer");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Provides functionality to read the answer if the command that was sent
/// HIFN  to the signal generator using
/// HIFN  RSSPECAN_ATTR_AMPLIFIER_GENERATOR_RELAY_WRITE contained a '?'.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR bufferSize/Size of output array.
/// HIPAR instrumentResponse/Provides functionality to read the answer if the command that was sent
/// HIPAR instrumentResponse/to the signal generator using
/// HIPAR instrumentResponse/RSSPECAN_ATTR_AMPLIFIER_GENERATOR_RELAY_WRITE contained a '?'.
/// HIPAR returnedBytes/The number of bytes returned from the instrument. If Returned Bytes >
/// HIPAR returnedBytes/Buffer Size, some data was lost. Consider resizing the buffer and
/// HIPAR returnedBytes/calling again.
ViStatus _VI_FUNC rsspecan_QueryAmplifierGeneratorResponse(ViSession instrSession,
                                                           ViInt32   bufferSize,
                                                           ViChar    instrumentResponse[],
                                                           ViInt32*  returnedBytes)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeViStringWithRealLen(instrSession, NULL, RSSPECAN_ATTR_AMPLIFIER_GENERATOR_RELAY_READ, RS_VAL_DIRECT_USER_CALL, bufferSize, instrumentResponse, returnedBytes));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This command turns the generator control on and off.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR generatorControl/This command turns the generator control on and off.
ViStatus _VI_FUNC rsspecan_ConfigureAmplifierGeneratorControlEnabled(
                            ViSession instrSession,
                            ViBoolean generatorControl)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_SIGNAL_GENERATOR_CONTROL_ENABLED, generatorControl),
            2, "Generator Control");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function updates the generator settings as defined within the R&S
/// HIFN  FSW-K18.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsspecan_UpdateAmplifierGeneratorSetupSettings(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_AMPLIFIER_GENERATOR_SETUP_SETTINGS_UPDATE, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function transfers the current generator configuration into
/// HIFN the Amplifier application.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsspecan_UpdateAmplifierSettingsFromGenerator(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_AMPLIFIER_SETUP_FROM_GENERATOR, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function queries the signal path of the R&S SMW used for baseband
/// HIFN  signal generation. Note that the baseband path is always the same as
/// HIFN  the RF path selected with
/// HIFN  RSSPECAN_ATTR_AMPLIFIER_GENERATOR_TARGET_PATH_RF.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR pathBB/This atribute queries the signal path of the R&S SMW used for baseband
/// HIPAR pathBB/signal generation. Note that the baseband path is always the same as
/// HIPAR pathBB/the RF path selected with
ViStatus _VI_FUNC rsspecan_GetAmplifierGeneratorTargetPathBaseband(
                            ViSession instrSession,
                            ViInt32*  pathBB)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_GetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_AMPLIFIER_GENERATOR_TARGET_PATH_BASEBAND, pathBB),
            2, "Path Baseband");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the I/Q averaging.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR state/This control turns I/Q averaging on and off.
/// HIPAR count/This control defines the average count for I/Q averaging.
ViStatus _VI_FUNC rsspecan_ConfigureAmplifierAveragingIQData(ViSession instrSession,
                                                             ViBoolean state,
                                                             ViInt32   count)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_AMPLIFIER_AVERAGING_I_Q_DATA, state),
            2, "State");
    if (state == VI_TRUE)
    {
        viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_AMPLIFIER_AVERAGING_I_Q_DATA_COUNT, count),
                3, "Count");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function queries the current measurement out of a sequence of
/// HIFN  measurements that averages I/Q data.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR count/This control queries the current measurement out of a sequence of
/// HIPAR count/measurements that averages I/Q data.
ViStatus _VI_FUNC rsspecan_QueryAmplifierAveragingIQDataCount(
                            ViSession instrSession,
                            ViInt32*  count)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_GetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_AMPLIFIER_AVERAGING_I_Q_DATA_CURRENT_COUNT, count),
            2, "Count");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the sweep statistics setting.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR state/Sets and queries the sweep statistics setting.
/// HIPAR mode/Sets and queries the statistics mode.
/// HIPAR continuous/Sets and queries the continuous statistics setting.
/// HIPAR count/Sets and queries the sweep statistics count.
ViStatus _VI_FUNC rsspecan_ConfigureAmplifierSweepConfigurationStatisticsSettings(
                            ViSession instrSession,
                            ViBoolean state,
                            ViInt32   mode,
                            ViBoolean continuous,
                            ViInt32   count)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, NULL, RSSPECAN_ATTR_AMPLIFIER_SWEEP_CONFIGURATION_STATISTICS_STATE, state),
            2, "State");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, NULL, RSSPECAN_ATTR_AMPLIFIER_SWEEP_CONFIGURATION_STATISTICS_MODE, mode),
            3, "Mode");

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, NULL, RSSPECAN_ATTR_AMPLIFIER_SWEEP_CONFIGURATION_CONTINUOUS_STATISTICS_STATE, continuous),
            4, "Continuous");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, NULL, RSSPECAN_ATTR_AMPLIFIER_SWEEP_CONFIGURATION_STATISTICS_COUNT, count),
            5, "Count");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Sets and querys the selected result range.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR range/Sets and querys the selected result range.
ViStatus _VI_FUNC rsspecan_ConfigureAmplifierSweepConfigurationSelectResultRange(
                            ViSession instrSession,
                            ViInt32   range)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, NULL, RSSPECAN_ATTR_AMPLIFIER_SWEEP_CONFIGURATION_SELECT_RESULT_RNG, range),
            2, "Range");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function defines the evaluation range.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR useFullRefSignal/This control turns result evaluation over the complete capture
/// HIPAR useFullRefSignal/buffer on and off.
/// HIPAR evalStart/This control defines the start value of the evaluation range.
/// HIPAR evalStop/This control defines the stop value of the evaluation range.
ViStatus _VI_FUNC rsspecan_ConfigureAmplifierEvaluationRange(ViSession instrSession,
                                                             ViBoolean useFullRefSignal,
                                                             ViReal64  evalStart,
                                                             ViReal64  evalStop)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_AMPLIFIER_EVALUATION_USE_FULL_REF_SIGNAL_ENABLED, useFullRefSignal),
            2, "Use Full Ref Signal");
    if (useFullRefSignal == VI_FALSE)
    {
        viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_AMPLIFIER_EVALUATION_RANGE_START, evalStart),
                3, "Eval Start");

        viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_AMPLIFIER_EVALUATION_RANGE_STOP, evalStop),
                4, "Eval Stop");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function turns synchronization between reference and
/// HIFN measured signal on and off.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR state/This control  turns synchronization between reference and measured signal on and off.
ViStatus _VI_FUNC rsspecan_ConfigureAmplifierSynchronizationState(
                            ViSession instrSession,
                            ViBoolean state)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_AMPLIFIER_SYNCHRONIZATION_ENABLED, state),
            2, "State");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function defines the synchronization confidence level.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR confidenceLevel/Document parameter here.
ViStatus _VI_FUNC rsspecan_ConfigureAmplifierSynchronizationConfidenceLevel(
                            ViSession instrSession,
                            ViReal64  confidenceLevel)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_AMPLIFIER_SYNCHRONIZATION_CONFIDENCE_LEVEL, confidenceLevel),
            2, "Confidence Level");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function turns a measurement stop when synchronization of
/// HIFN  measured and reference signal fails on and off.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR measurementStop/This control turns a measurement stop when synchronization of measured
/// HIPAR measurementStop/and reference signal fails on and off.
ViStatus _VI_FUNC rsspecan_ConfigureAmplifierMeasurementStop(ViSession instrSession,
                                                             ViBoolean measurementStop)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_AMPLIFIER_MESUREMENT_STOP, measurementStop),
            2, "Measurement Stop");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function selects the synchronization method.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR synchronizationDomain/This control selects the synchronization method.
ViStatus _VI_FUNC rsspecan_ConfigureAmplifierSynchronizationDomain(
                            ViSession instrSession,
                            ViInt32   synchronizationDomain)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_AMPLIFIER_SYNCHRONIZATION_DOMAIN, synchronizationDomain),
            2, "Synchronization Domain");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function defines the estimation range.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR useFullRefSignal/This control  turns estimation over the complete reference signal on and off.
/// HIPAR syncStart/This control defines the start value of the estimation range.
/// HIPAR syncStop/This control defines the stop value of the estimation range.
ViStatus _VI_FUNC rsspecan_ConfigureAmplifierEstimationRange(ViSession instrSession,
                                                             ViBoolean useFullRefSignal,
                                                             ViReal64  syncStart,
                                                             ViReal64  syncStop)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_AMPLIFIER_ESTIMATION_USE_FULL_REF_SIGNAL_ENABLED, useFullRefSignal),
            2, "Use Full Ref Signal");
    if (useFullRefSignal == VI_FALSE)
    {
        viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_AMPLIFIER_ESTIMATION_RANGE_START, syncStart),
                3, "Sync Start");

        viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_AMPLIFIER_ESTIMATION_RANGE_STOP, syncStop),
                4, "Sync Stop");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function returns synchronization state.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR state/Returns synchronization state.
ViStatus _VI_FUNC rsspecan_QueryAmplifierSynchronizationState(
                            ViSession  instrSession,
                            ViBoolean* state)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_GetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_AMPLIFIER_SYNCHRONIZATION_STATE, state),
            2, "State");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN Queries the (measured) synchronized I/Q data.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR impedanceMode/Selects the input.
/// HIPAR arrayLength/Number of data points requested.
/// HIPAR actualPoints/Number of points actually returned in the array.
/// HIPAR data/A user allocated buffer into which the I/Q data is stored.
ViStatus _VI_FUNC rsspecan_QueryAmplifierIQSynchronizationData(
                            ViSession instrSession,
                            ViInt32   impedanceMode,
                            ViInt32   arrayLength,
                            ViInt32*  actualPoints,
                            ViReal64  data[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, impedanceMode, RSSPECAN_VAL_AMPLIFIER_BB, RSSPECAN_VAL_AMPLIFIER_RF),
            2, "Impedance Mode");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "TRAC:IQ:SYNC? %s", impedanceModeArr[impedanceMode]);
    checkErr(RsCore_QueryFloatArrayToUserBufferWithOpc(instrSession, cmd, 0, arrayLength, data, actualPoints));

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function turns compensation of the I/Q Imbalance, Amplitude
/// HIFN  Droop, Frequency Offset on and off.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR amplitudeDroop/This control  turns compensation of the Amplitude Droop on and off.
/// HIPAR iqImbalance/This atribute turns compensation of the I/Q Imbalance on and off.
/// HIPAR sampleErrorRate/This atribute turns compensation of the I/Q Imbalance on and off.
ViStatus _VI_FUNC rsspecan_ConfigureAmplifierCompensationStates(
                            ViSession instrSession,
                            ViBoolean amplitudeDroop,
                            ViBoolean iqImbalance,
                            ViBoolean sampleErrorRate)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_AMPLIFIER_COMPENSATION_AMPLITUDE_DROOP_STATE, amplitudeDroop),
            2, "Amplitude Droop");

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_AMPLIFIER_COMPENSATION_IQ_IMBALANCE_STATE, iqImbalance),
            3, "IQ Imbalance");

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_AMPLIFIER_COMPENSATION_SAMPLE_ERROR_RATE_STATE, sampleErrorRate),
            4, "Sample Error Rate");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function defines the length of the filter that the equalizer
/// HIFN  training is based on.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR filterLength/This control defines the length of the filter that the equalizer
/// HIPAR filterLength/training is based on.
ViStatus _VI_FUNC rsspecan_ConfigureAmplifierEqualizerFilterLengthForTraining(
                            ViSession instrSession,
                            ViInt32   filterLength)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_AMPLIFIER_EQUALIZER_FILTER_LENGTH_FOR_TRAINING, filterLength),
            2, "Filter Length");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function initiates a training sequence for the equalizer filter.
/// HIFN  Note that you have to synchronize the measurement before you can
/// HIFN  initiate a training sequence.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsspecan_AmplifierTrainEqualizerFilter(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_AMPLIFIER_EQUALIZER_TRAIN_FILTER, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function stores the equalizer filter that has been calculated.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR fileName/This control stores the equalizer filter that has been calculated.
ViStatus _VI_FUNC rsspecan_AmplifierSaveEqualizerFilter(ViSession instrSession,
                                                        ViString  fileName)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_AMPLIFIER_EQUALIZER_STORE_FILTER, fileName),
            2, "File Name");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function restores an equalizer filter that you have previously
/// HIFN  saved.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR fileName/This control restores an equalizer filter that you have previously
/// HIPAR fileName/saved.
ViStatus _VI_FUNC rsspecan_AmplifierLoadEqualizerFilter(ViSession instrSession,
                                                        ViString  fileName)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_AMPLIFIER_EQUALIZER_RESTORE_FILTER, fileName),
            2, "File Name");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function turns the equalizer on and off.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR equalizerState/This control turns the equalizer on and off.
ViStatus _VI_FUNC rsspecan_ConfigureAmplifierEqualizerState(ViSession instrSession,
                                                            ViBoolean equalizerState)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_AMPLIFIER_EQUALIZER_STATE, equalizerState),
            2, "Equalizer State");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects the file format to which the equalizer filter is exported.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR format/Selects the file format to which the equalizer filter is exported.
ViStatus _VI_FUNC rsspecan_ConfigureAmplifierEqualizerFilterFileFormat(
                            ViSession instrSession,
                            ViInt32   format)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, NULL, RSSPECAN_ATTR_AMPLIFIER_EQUALIZER_FILTER_FILE_FORMAT, format),
            2, "Format");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Defines the filter coefficients manually instead of training a filter. Each
/// HIFN  coefficient consists of a real and an imaginary value.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR iCoefficients/The real components of the filter coefficients
/// HIPAR qCoefficients/The imaginary components of the filter coefficients
ViStatus _VI_FUNC rsspecan_AmplifierEqualizerManualFilterDefinition(
                            ViSession instrSession,
                            ViInt32   arraysLength,
                            ViInt32   iCoefficients[],
                            ViInt32   qCoefficients[])
{
    ViStatus error = VI_SUCCESS;
    ViChar*  p2param;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_Convert_ViInt32ArraysInterleaved_To_String(instrSession, arraysLength, iCoefficients, qCoefficients, NULL, NULL, NULL, NULL, NULL, &p2param));

    checkErr(rsspecan_SetAttributeViString(instrSession, NULL, RSSPECAN_ATTR_HIDDEN_AMPLIFIER_EQUALIZER_MANUAL_FILTER_DEFINITION, p2param));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This functions turns estimations of the Droop State, IQ Imbalance
/// HIFN  State and Sample Error Rate, on and off.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR amplitudeDroop/This control turns estimation of the Amplitude Droop on and off.
/// HIPAR iqImbalance/This control turns estimation of the I/Q Imbalance on and off.
/// HIPAR sampleError/This control turns estimation of the Sample Error Rate on and off.
ViStatus _VI_FUNC rsspecan_ConfigureAmplifierEstimationStates(
                            ViSession instrSession,
                            ViBoolean amplitudeDroop,
                            ViBoolean iqImbalance,
                            ViBoolean sampleErrorRate)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_AMPLIFIER_ESTIMATION_AMPLITUDE_DROOP_STATE, amplitudeDroop),
            2, "Amplitude Droop");

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_AMPLIFIER_ESTIMATION_IQ_IMBALANCE_STATE, iqImbalance),
            3, "IQ Imbalance");

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_AMPLIFIER_ESTIMATION_SAMPLE_ERROR_RATE_STATE, sampleErrorRate),
            4, "Sample Error Rate");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Turns estimation of the frequency error on and off.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR state/Turns estimation of the frequency error on and off.
ViStatus _VI_FUNC rsspecan_ConfigureAmplifierEstimationFrequencyErrorState(
                            ViSession instrSession,
                            ViBoolean state)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, NULL, RSSPECAN_ATTR_AMPLIFIER_ESTIMATION_FREQUENCY_ERROR_STATE, state),
            2, "State");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Turns estimation of the I/Q offset on and off.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR state/Turns estimation of the I/Q offset on and off.
ViStatus _VI_FUNC rsspecan_ConfigureAmplifierEstimationIQOffsetState(
                            ViSession instrSession,
                            ViBoolean state)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, NULL, RSSPECAN_ATTR_AMPLIFIER_ESTIMATION_IQ_OFFSET_STATE, state),
            2, "State");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function configures System Models
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR modelingState/This control turns system modeling on and off.
/// HIPAR modelingOrderAMPM/This control defines the order (or degree) of the AM/AM model polynom
/// HIPAR modelingOrderAMAM/This control defines the order (or degree) of the AM/AM model polynom.
/// HIPAR modelingRange/This control defines the modeling level range.
/// HIPAR modelingPoints/This control defines the number of modeling points
ViStatus _VI_FUNC rsspecan_ConfigureAmplifierSystemModelSettings(
                            ViSession instrSession,
                            ViBoolean modelingState,
                            ViString  modelingOrderAMPM,
                            ViString  modelingOrderAMAM,
                            ViReal64  modelingRange,
                            ViInt32   modelingPoints)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_AMPLIFIER_MODELING_STATE, modelingState),
            2, "Modeling State");

    viCheckParm(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_AMPLIFIER_MODELING_ORDER_AMPM, modelingOrderAMPM),
            3, "Modeling Order AM PM");

    viCheckParm(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_AMPLIFIER_MODELING_ORDER_AMAM, modelingOrderAMAM),
            4, "Modeling Order AM AM");

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_AMPLIFIER_MODELING_RANGE, modelingRange),
            5, "Modeling Range");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_AMPLIFIER_MODELING_POINTS, modelingPoints),
            6, "Modeling Points");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Turns compensation of the frequency error on and off.
/// HIFN     Prerequisites: Turn on estimation of sample rate
/// HIFN  (RSSPECAN_ATTR_AMPLIFIER_ESTIMATION_FREQUENCY_ERROR_STATE).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR state/Turns compensation of the frequency error on and off.
ViStatus _VI_FUNC rsspecan_ConfigureAmplifierCompensationFrequencyErrorState(
                            ViSession instrSession,
                            ViBoolean state)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, NULL, RSSPECAN_ATTR_AMPLIFIER_COMPENSATION_FREQUENCY_ERROR_STATE, state),
            2, "State");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Turns compensation of the I/Q offset on and off.
/// HIFN     Prerequisites: Turn on estimation of sample rate
/// HIFN  (RSSPECAN_ATTR_AMPLIFIER_ESTIMATION_IQ_OFFSET_STATE).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR state/Turns compensation of the I/Q offset on and off.
ViStatus _VI_FUNC rsspecan_ConfigureAmplifierCompensationIQOffsetState(
                            ViSession instrSession,
                            ViBoolean state)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, NULL, RSSPECAN_ATTR_AMPLIFIER_COMPENSATION_IQ_OFFSET_STATE, state),
            2, "State");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function configures System Modeling Sequence.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR sequence/This control selects the sequence in which the models are calculated.
ViStatus _VI_FUNC rsspecan_ConfigureAmplifierSystemModelSequence(
                            ViSession instrSession,
                            ViInt32   sequence)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_AMPLIFIER_MODELING_SEQUENCE, sequence),
            2, "Sequence");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function selects the method by which the input power range is
/// HIFN  split into smaller ranges for the calculation of the amplifier model.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR scale/Selects the method by which the input power range is split into
/// HIPAR scale/smaller ranges for the calculation of the amplifier model.
ViStatus _VI_FUNC rsspecan_ConfigureAmplifierSystemModelScale(
                            ViSession instrSession,
                            ViInt32   scale)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_AMPLIFIER_MODELING_SCALE, scale),
            2, "Scale");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function configures the method with which the application
/// HIFN determines the DPD.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR method/This control selects the method with which the application
/// HIPAR method/determines the DPD.
ViStatus _VI_FUNC rsspecan_ConfigureAmplifierDPDMethod(ViSession instrSession,
                                                       ViInt32   method)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_AMPLIFIER_DPD_METHOD, method),
            2, "Method");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function generates the waveform files containing
/// HIFN predistortion information within the Amplifier application.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsspecan_GenerateDPDWaveformFile(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_AMPLIFIER_DPD_GENERATE_WAVEFORM_FILE, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function generates the waveform files containing predistortion
/// HIFN  information within the Amplifier application. All in all, the function
/// HIFN  generates three waveform files: AM/AM only, AM/PM only and AM/AM plus
/// HIFN  AM/PM. It also transfers these waveform files to the connected signal
/// HIFN  generator and turns on the AM/AM and AM/PM DPDs.
/// HIFN
/// HIFN     Note:
/// HIFN     (1) DPD method "Generate Predistorted Waveform File" has to be
/// HIFN  selected
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsspecan_GenerateDPDWaveformFileAll(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_AMPLIFIER_DPD_GENERATE_WAVEFORM_FILE_ALL, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function updates the DPD shaping tables on the R&S SMW when
/// HIFN  modeling parameters have changed.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsspecan_AmplifierDPDUpdateShappingTable(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_AMPLIFIER_DPD_UPDATE_SHAPPING_TABLE, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Updates the DPD shaping tables on the R&S SMW when new measurement
/// HIFN  data is available. In addition, it also turns on the DPD (AM/AM and
/// HIFN  AM/PM). Using one command only to do those things has the advantage of
/// HIFN  a slightly shorter execution time.
/// HIFN     Alternatively, you can do that with
/// HIFN  RSSPECAN_ATTR_AMPLIFIER_DPD_UPDATE_SHAPPING_TABLE and
/// HIFN  RSSPECAN_ATTR_AMPLIFIER_DPD_AM_XM_STATE.
/// HIFN     Prerequisites: Turn on polynomial DPD
/// HIFN  (RSSPECAN_ATTR_AMPLIFIER_DIRECT_DPD_STATE).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsspecan_AmplifierDPDUpdateAllShapingTables(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsspecan_SetAttributeViString(instrSession, NULL, RSSPECAN_ATTR_AMPLIFIER_DPD_UPDATE_ALL_SHAPING_TABLES, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function queries the state of an update of the shaping table.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR shapingTableState/This control queries the state of an update of the shaping table.
ViStatus _VI_FUNC rsspecan_GetAmplifierDPDUpdateShappingTableStatus(
                            ViSession instrSession,
                            ViInt32*  shapingTableState)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_GetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_AMPLIFIER_DPD_UPDATE_SHAPPING_TABLE_STATUS, shapingTableState),
            2, "Shaping Table State");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function updates the DPD shaping tables on the R&S SMW when
/// HIFN  modeling parameters have changed.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR shapingMethod/This control selects the method use to shape the DPD function
/// HIPAR nameDPDCorectionTable/This control defines a name for the DPD correction table.
/// HIPAR dpdOrderModelCalculation/This control selects the order in which the AM/AM and AM/PM distortion
/// HIPAR dpdOrderModelCalculation/are calculated.Configuring Amplifier Measurements R&S-K18 Remote
/// HIPAR dpdOrderModelCalculation/Control Commands for Amplifier Measurements User Manual 1176.9893.02
/// HIPAR dpdOrderModelCalculation/01 121 Available when both CONFigure: DPD: AMAM[: STATe] and CONFigure:
/// HIPAR dpdOrderModelCalculation/DPD: AMPM[: STATe] have been turned on.
/// HIPAR AMAMState/This control turns calculation of AM/AM predistortion on and off.
/// HIPAR AMPMState/This control turns calculation of AM/PM predistortion on and off.
ViStatus _VI_FUNC rsspecan_ConfigureAmplifierDPDShaping(ViSession instrSession,
                                                        ViInt32   shapingMethod,
                                                        ViString  nameDPDCorectionTable,
                                                        ViInt32   dpdSequence,
                                                        ViBoolean AMAMState,
                                                        ViBoolean AMPMState)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_AMPLIFIER_DPD_SHAPING_METHOD, shapingMethod),
            2, "Shaping Method");

    viCheckParm(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_AMPLIFIER_DPD_CORECTION_TABLE_NAME, nameDPDCorectionTable),
            3, "DPD File Name");

    if (AMAMState == VI_TRUE && AMPMState == VI_TRUE)
    {
        viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_AMPLIFIER_DPD_SEQUENCE, dpdSequence),
                4, "DPD Sequence");
        //modified in 3.4.0
        //viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "",
        //                                             RSSPECAN_ATTR_AMPLIFIER_DPD_MODELING_ORDER,
        //                                             dpdOrderModelCalculation), 4, "Order Model Calculation");
    }

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_AMPLIFIER_DPD_AM_AM_STATE, AMAMState),
            4, "AM AM State");

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_AMPLIFIER_DPD_AM_PM_STATE, AMPMState),
            5, "AM PM State");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Turns AM/AM and AM/PM predistortion on and off (at the same time).
/// HIFN     Alternatively, you can do that with
/// HIFN  RSSPECAN_ATTR_AMPLIFIER_DPD_AM_AM_STATE and
/// HIFN  RSSPECAN_ATTR_AMPLIFIER_DPD_AM_PM_STATE. However, using
/// HIFN  RSSPECAN_ATTR_AMPLIFIER_DPD_AM_XM_STATE is the smoother way.
/// HIFN     Prerequisites: Turn on polynomial DPD
/// HIFN  (RSSPECAN_ATTR_AMPLIFIER_DIRECT_DPD_STATE).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR state/Turns AM/AM and AM/PM predistortion on and off (at the same time).
ViStatus _VI_FUNC rsspecan_ConfigureAmplifierDPDAMxMState(ViSession instrSession,
                                                          ViBoolean state)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, NULL, RSSPECAN_ATTR_AMPLIFIER_DPD_AM_XM_STATE, state),
            2, "State");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function defines the power / linearity tradoff for DPD calculation.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR tradeoff/Defines the power / linearity tradoff for DPD calculation.
ViStatus _VI_FUNC rsspecan_ConfigureDPDPowerLinearityTradeoff(
                            ViSession instrSession,
                            ViInt32   tradeoff)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_AMPLIFIER_DPD_TRADEOFF, tradeoff),
            2, "Tradeoff");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the path of the Polynomial DPD waveform.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR bufferSize/Size of output array.
/// HIPAR path/Queries the path of the Polynomial DPD waveform.
/// HIPAR bytesRead/Returns the number of bytes read from instrument.
ViStatus _VI_FUNC rsspecan_QueryAmplifierDPDWaveformPath(ViSession instrSession,
                                                         ViInt32   bufferSize,
                                                         ViChar    path[],
                                                         ViInt32*  bytesRead)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeViStringWithRealLen(instrSession, NULL, RSSPECAN_ATTR_AMPLIFIER_DPD_WAVEFORM_PATH, RS_VAL_DIRECT_USER_CALL, bufferSize, path, bytesRead));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function generates and stores a waveform containing the DPD
/// HIFN in a file you have specified.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR filename/This parameter defines the path and file name of the target
/// HIPAR filename/file.
ViStatus _VI_FUNC rsspecan_ExportDPDWaveformToFile(ViSession instrSession,
                                                   ViString  filename)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_AMPLIFIER_DPD_EXPORT_WAVEFORM_FILE, filename),
            2, "File Name");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function  queries the state of the calculation of the AM/AM and
/// HIFN  AM/PM distortion curves.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR amamState/This control queries the state of the calculation of the AM/AM
/// HIPAR amamState/distortion curve.
/// HIPAR ampmState/This control queries the state of the calculation of the AM/PM
/// HIPAR ampmState/distortion curve.
ViStatus _VI_FUNC rsspecan_GetAmplifierDPDLEDState(ViSession instrSession,
                                                   ViInt32*  amamState,
                                                   ViInt32*  ampmState)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_GetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_AMPLIFIER_DPD_AM_AM_LED_STATE, amamState),
            2, "AM/AM State");

    viCheckParm(rsspecan_GetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_AMPLIFIER_DPD_AM_PM_LED_STATE, ampmState),
            3, "AM/PM State");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function selects the type of DPD.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR directDPD/This control selects the type of DPD.
ViStatus _VI_FUNC rsspecan_ConfigureAmplifierDPDState(ViSession instrSession,
                                                      ViBoolean directDPD)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_AMPLIFIER_DIRECT_DPD_STATE, directDPD),
            2, "Direct DPD");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function selects the action to be performed on Direct DPD.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR action/This control selects the action to be performed on Direct DPD.
ViStatus _VI_FUNC rsspecan_AmplifierDirectDPDAction(ViSession instrSession,
                                                    ViInt32   action)
{
    ViStatus error = VI_SUCCESS;
    ViAttr   attr[] = {
             RSSPECAN_ATTR_AMPLIFIER_DIRECT_DPD_START,
             RSSPECAN_ATTR_AMPLIFIER_DIRECT_DPD_ABORT,
             RSSPECAN_ATTR_AMPLIFIER_DIRECT_DPD_FINISH,
             RSSPECAN_ATTR_AMPLIFIER_DIRECT_DPD_CONTINUE
             };

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, action, RSSPECAN_VAL_AMPLIFIER_DIRECT_DPD_START, RSSPECAN_ATTR_AMPLIFIER_DIRECT_DPD_CONTINUE),
            2, "Action");

    checkErr(rsspecan_SetAttributeViString(instrSession, "", attr[action], NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This command sets the gain expansion for Direct DPD.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR gainExpansion/Sets the gain expansion for Direct DPD.
ViStatus _VI_FUNC rsspecan_ConfigureAmplifierDirectDPDGainExpansion(
                            ViSession instrSession,
                            ViReal64  gainExpansion)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_AMPLIFIER_DIRECT_DPD_GAIN_EXPANSION, gainExpansion),
            2, "Gain Expansion");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function defines the number of iteration in a direct DPD
/// HIFN  sequence.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR iterations/This control defines the number of iteration in a direct DPD sequence.
ViStatus _VI_FUNC rsspecan_ConfigureAmplifierDirectDPDIterations(
                            ViSession instrSession,
                            ViInt32   iterations)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_AMPLIFIER_DIRECT_DPD_ITERATIONS, iterations),
            2, "Iterations");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function queries the process of the direct DPD sequence (number
/// HIFN  of current iteration).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR numberofCurrentIteration/This control queries the process of the direct DPD sequence (number of
/// HIPAR numberofCurrentIteration/current iteration).
ViStatus _VI_FUNC rsspecan_QueryAmplifierDirectDPDCurrentIteration(
                            ViSession instrSession,
                            ViInt32*  numberofCurrentIteration)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_GetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_AMPLIFIER_DIRECT_DPD_CURRENT_ITERATION, numberofCurrentIteration),
            2, "Number of Current Iteration");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function defines the power / linearity tradeoff for direct DPD
/// HIFN  calculation.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR tradeoff/This control defines the power / linearity tradeoff for direct DPD
/// HIPAR tradeoff/calculation.
ViStatus _VI_FUNC rsspecan_ConfigureAmplifierDirectDPDPowerLinearityTradeoff(
                            ViSession instrSession,
                            ViReal64  tradeoff)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_AMPLIFIER_DIRECT_DPD_POWER_LINEARITY_TRADEOFF, tradeoff),
            2, "Tradeoff");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function defines a file name for the I/Q file that contains the
/// HIFN  predistorted I/Q data that was generated by the direct DPD.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR nameOnGenerator/This control defines a file name for the I/Q file that contains the
/// HIPAR nameOnGenerator/predistorted I/Q data that was generated by the direct DPD.
ViStatus _VI_FUNC rsspecan_ConfigureAmplifierDPDNameOnGenerator(
                            ViSession instrSession,
                            ViString  nameOnGenerator)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_AMPLIFIER_DIRECT_DPD_NAME_ON_GENERATOR, nameOnGenerator),
            2, "Name On Generator");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function stores the direct DPD information in a file.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR waveformFile/This control stores the direct DPD information in a file.
ViStatus _VI_FUNC rsspecan_StoreAmplifierDirectDPDWaveformFile(
                            ViSession instrSession,
                            ViString  waveformFile)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_AMPLIFIER_DIRECT_DPD_STORE_PREDISTORTION_WAVEFORM_FILE, waveformFile),
            2, "Waveform File");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function transfers the waveform file with the correction values
/// HIFN  to the signal generator and applies them to the input signal.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR apply/This control transfers the waveform file with the correction values to
/// HIPAR apply/the signal generator and applies them to the input signal.
ViStatus _VI_FUNC rsspecan_ApplyAmplifierDirectDPD(ViSession instrSession,
                                                   ViBoolean apply)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_AMPLIFIER_DIRECT_DPD_APPLY, apply),
            2, "Apply");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Smoothes start- and tail-samples down to '0' in order to avoid phase
/// HIFN  discontinuities when the file is cyclically played from a signal
/// HIFN  source.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR wrap/Smoothes start- and tail-samples down to '0' in order to avoid phase
/// HIPAR wrap/discontinuities when the file is cyclically played from a signal
/// HIPAR wrap/source.
ViStatus _VI_FUNC rsspecan_ConfigureAmplifierDirectDPDWrapEnabled(
                            ViSession instrSession,
                            ViBoolean wrap)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, NULL, RSSPECAN_ATTR_AMPLIFIER_DIRECT_DPD_WRAP_ENABLED, wrap),
            2, "Wrap");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Returns the current RMS power level in manual direct DPD mode.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR rmsCurrent/Returns the current RMS power level in manual direct DPD mode.
ViStatus _VI_FUNC rsspecan_QueryAmplifierDirectDPDRMSCurrent(ViSession instrSession,
                                                             ViReal64* rmsCurrent)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, NULL, RSSPECAN_ATTR_AMPLIFIER_DIRECT_DPD_RMS_CURRENT, rmsCurrent),
            2, "RMS Current");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the path of the Direct DPD waveform.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR bufferSize/Size of output array.
/// HIPAR path/Queries the path of the Direct DPD waveform.
/// HIPAR bytesRead/Returns the number of bytes read from instrument.
ViStatus _VI_FUNC rsspecan_QueryAmplifierDirectDPDWaveformPath(
                            ViSession instrSession,
                            ViInt32   bufferSize,
                            ViChar    path[],
                            ViInt32*  bytesRead)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeViStringWithRealLen(instrSession, NULL, RSSPECAN_ATTR_AMPLIFIER_DIRECT_DPD_WAVEFORM_PATH, RS_VAL_DIRECT_USER_CALL, bufferSize, path, bytesRead));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the state of a direct DPD operation.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR operationStatus/Queries the state of a direct DPD operation.
ViStatus _VI_FUNC rsspecan_QueryAmplifierDirectDPDOperationStatus(
                            ViSession  instrSession,
                            ViBoolean* operationStatus)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_GetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_AMPLIFIER_DIRECT_DPD_OPERATION_STATUS, operationStatus),
            2, "Operation Status");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures polynomial DPD settings.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR state/Switches the memory polynomial state on and off.
/// HIPAR iterationStep/Configures the iteration step for memory polynomial DPD.
/// HIPAR polynomialOrder/Sets the polynomial order for memory polynomial DPD as a string.
/// HIPAR memoryOrder/Sets the memory order for memory polynomial DPD as a string.
ViStatus _VI_FUNC rsspecan_ConfigureAmplifierMemoryPolynomialDPDSettings(
                            ViSession instrSession,
                            ViBoolean state,
                            ViInt32   iterationStep,
                            ViString  polynomialOrder,
                            ViString  memoryOrder)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, NULL, RSSPECAN_ATTR_AMPLIFIER_MEMORY_POLYNOMIAL_DPD_STATE, state),
            2, "State");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, NULL, RSSPECAN_ATTR_AMPLIFIER_MEMORY_POLYNOMIAL_DPD_ITERATION_STEP, iterationStep),
            3, "Iteration Step");

    viCheckParm(rsspecan_SetAttributeViString(instrSession, NULL, RSSPECAN_ATTR_AMPLIFIER_MEMORY_POLYNOMIAL_DPD_POLYNOMIAL_ORDER, polynomialOrder),
            4, "Polynomial Order");

    viCheckParm(rsspecan_SetAttributeViString(instrSession, NULL, RSSPECAN_ATTR_AMPLIFIER_MEMORY_POLYNOMIAL_DPD_MEMORY_ORDER, memoryOrder),
            5, "Memory Order");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Calculates the memory polynomial model.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsspecan_CalculateAmplifierMemoryPolynomialDPDModel(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsspecan_SetAttributeViString(instrSession, NULL, RSSPECAN_ATTR_AMPLIFIER_MEMORY_POLYNOMIAL_DPD_MODEL, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects the waveform to which the model should be applied.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR waveform/Selects the waveform to which the model should be applied.
ViStatus _VI_FUNC rsspecan_ConfigureAmplifierMemoryPolynomialDPDModelApply(
                            ViSession instrSession,
                            ViString  waveform)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViString(instrSession, NULL, RSSPECAN_ATTR_AMPLIFIER_MEMORY_POLYNOMIAL_DPD_MODEL_APPLY, waveform),
            2, "Waveform");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Saves the memory polynomial waveform at a user selected path.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR waveformFilePath/Saves the memory polynomial waveform at a user selected path.
ViStatus _VI_FUNC rsspecan_ConfigureAmplifierMemoryPolynomialDPDExportWaveformFile(
                            ViSession instrSession,
                            ViString  waveformFilePath)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViString(instrSession, NULL, RSSPECAN_ATTR_AMPLIFIER_MEMORY_POLYNOMIAL_DPD_EXPORT_WAVEFORM_FILE, waveformFilePath),
            2, "Waveform File Path");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Exports the memory DPD coefficients in a file in .csv format.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR coefficientsFilePath/Exports the memory DPD coefficients in a file in .csv format.
ViStatus _VI_FUNC rsspecan_ConfigureAmplifierMemoryPolynomialDPDExportCoefficientsFile(
                            ViSession instrSession,
                            ViString  coefficientsFilePath)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViString(instrSession, NULL, RSSPECAN_ATTR_AMPLIFIER_MEMORY_POLYNOMIAL_DPD_EXPORT_COEFFICIENTS_FILE, coefficientsFilePath),
            2, "Coefficients File Path");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Selects the type of DPD waveform to be used.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR waveformType/Selects the type of DPD waveform to be used.
ViStatus _VI_FUNC rsspecan_ConfigureAmplifierMemoryPolynomialDPDWaveformType(
                            ViSession instrSession,
                            ViInt32   waveformType)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, NULL, RSSPECAN_ATTR_AMPLIFIER_MEMORY_POLYNOMIAL_DPD_WAVEFORM_TYPE, waveformType),
            2, "Waveform Type");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the path of the Memory Polynomial DPD waveform.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR bufferSize/Size of output array.
/// HIPAR path/Queries the path of the Memory Polynomial DPD waveform.
/// HIPAR bytesRead/Returns the number of bytes read from instrument.
ViStatus _VI_FUNC rsspecan_QueryAmplifierMemoryPolynomialDPDWaveformPath(
                            ViSession instrSession,
                            ViInt32   bufferSize,
                            ViChar    path[],
                            ViInt32*  bytesRead)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeViStringWithRealLen(instrSession, NULL, RSSPECAN_ATTR_AMPLIFIER_MEMORY_POLYNOMIAL_DPD_WAVEFORM_PATH, RS_VAL_DIRECT_USER_CALL, bufferSize, path, bytesRead));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Returns the current RMS power level of the memory polynomial waveform.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR rms/Returns the current RMS power level of the memory polynomial waveform.
ViStatus _VI_FUNC rsspecan_QueryAmplifierMemoryPolynomialDPDRMSCurrent(
                            ViSession instrSession,
                            ViReal64* rms)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, NULL, RSSPECAN_ATTR_AMPLIFIER_MEMORY_POLYNOMIAL_DPD_RMS_CURRENT, rms),
            2, "RMS");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Sends the memory polynomial waveform to the signal generator.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsspecan_SendAmplifierMemoryPolynomialDPDWaveformToGenerator(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsspecan_SetAttributeViString(instrSession, NULL, RSSPECAN_ATTR_AMPLIFIER_MEMORY_POLYNOMIAL_DPD_SEND_WAVEFORM_TO_GENERATOR, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the polynomial factors of the correctional polynomial. The
/// HIFN  real and imaginary parts of the DPD coefficients are returned
/// HIFN  interleaved in the following order: real(a0), imag(a0), real(a1),
/// HIFN  imag(a1)...
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR arraySize/Number of array points requested.
/// HIPAR polynomialData/Returns the polynomial factors of the correctional polynomial. The
/// HIPAR polynomialData/real and imaginary parts of the DPD coefficients are returned
/// HIPAR polynomialData/interleaved in the following order: real(a0), imag(a0), real(a1),
/// HIPAR polynomialData/imag(a1)...
/// HIPAR numberofFactors/Returns the number of data points.
ViStatus _VI_FUNC rsspecan_QueryAmplifierDirectDPDPolynomial(ViSession instrSession,
                                                             ViInt32   arraySize,
                                                             ViReal64  polynomialData[],
                                                             ViInt32*  numberofFactors)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(
        RsCore_QueryFloatArrayToUserBuffer(instrSession, "FETC:DPD:POLY?", arraySize, polynomialData, numberofFactors));

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Switches Hammerstein mode on and off.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR hammerstein/Switches Hammerstein mode on and off.
ViStatus _VI_FUNC rsspecan_ConfigureAmplifierHammersteinEnabled(
                            ViSession instrSession,
                            ViBoolean hammerstein)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, NULL, RSSPECAN_ATTR_AMPLIFIER_HAMMERSTEIN_ENABLED, hammerstein),
            2, "Hammerstein");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Switches the generator waveform between reference and direct DPD.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR generatorWaveform/Switches the generator waveform between reference and direct DPD.
ViStatus _VI_FUNC rsspecan_ConfigureAmplifierHammersteinGeneratorWaveform(
                            ViSession instrSession,
                            ViInt32   generatorWaveform)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, NULL, RSSPECAN_ATTR_AMPLIFIER_HAMMERSTEIN_GENERATOR_WAVEFORM, generatorWaveform),
            2, "Generator Waveform");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Configures the iteration step for the Hammerstein model.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR iterationStep/Configures the iteration step for the Hammerstein model.
ViStatus _VI_FUNC rsspecan_ConfigureAmplifierHammersteinIterationStep(
                            ViSession instrSession,
                            ViInt32   iterationStep)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, NULL, RSSPECAN_ATTR_AMPLIFIER_HAMMERSTEIN_ITERATION_STEP, iterationStep),
            2, "Iteration Step");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Sets the memory order for the Hammerstein model as a string.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR memoryOrder/Sets the memory order for the Hammerstein model as a string.
ViStatus _VI_FUNC rsspecan_ConfigureAmplifierHammersteinMemoryOrder(
                            ViSession instrSession,
                            ViString  memoryOrder)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViString(instrSession, NULL, RSSPECAN_ATTR_AMPLIFIER_HAMMERSTEIN_MEMORY_ORDER, memoryOrder),
            2, "Memory Order");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Sets the polynomial order for the Hammerstein model as a string.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR polynomialOrder/Sets the polynomial order for the Hammerstein model as a string.
ViStatus _VI_FUNC rsspecan_ConfigureAmplifierHammersteinPolynomialOrder(
                            ViSession instrSession,
                            ViString  polynomialOrder)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViString(instrSession, NULL, RSSPECAN_ATTR_AMPLIFIER_HAMMERSTEIN_POLYNOMIAL_ORDER, polynomialOrder),
            2, "Polynomial Order");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Switches the non-linearity (SMx-K541) on and off.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR nonLinearity/Switches the non-linearity (SMx-K541) on and off.
ViStatus _VI_FUNC rsspecan_ConfigureAmplifierHammersteinNonLinearityEnabled(
                            ViSession instrSession,
                            ViBoolean nonLinearity)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, NULL, RSSPECAN_ATTR_AMPLIFIER_HAMMERSTEIN_NON_LINEARITY_ENABLED, nonLinearity),
            2, "Non Linearity");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Switches the filter (SMx-K544) on and off.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR filter/Switches the filter (SMx-K544) on and off.
ViStatus _VI_FUNC rsspecan_ConfigureAmplifierHammersteinFilterEnabled(
                            ViSession instrSession,
                            ViBoolean filter)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, NULL, RSSPECAN_ATTR_AMPLIFIER_HAMMERSTEIN_FILTER_ENABLED, filter),
            2, "Filter");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Starts the DSP and updates the generator.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsspecan_StartAmplifierHammersteinGenerator(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsspecan_SetAttributeViString(instrSession, NULL, RSSPECAN_ATTR_AMPLIFIER_HAMMERSTEIN_GENERATOR_START, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Calculates the detailed MSE and returns the result.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR mse/Calculates the detailed MSE and returns the result.
ViStatus _VI_FUNC rsspecan_CalculateAmplifierMSE(ViSession instrSession,
                                                 ViReal64* mse)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, NULL, RSSPECAN_ATTR_AMPLIFIER_CALCULATE_MSE, mse),
            2, "MSE");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures Envelope Tracking.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR iChannelMultiplier/This control defines a multiplier to take into account various effects
/// HIPAR iChannelMultiplier/resulting from the measurement equipment connected to the I channel.
/// HIPAR iChannelOffset/This control defines an offset for the I channel.
/// HIPAR iChannelResistance/This controls defines the characteristics of the shunt resistor used
/// HIPAR iChannelResistance/in the test setup.
/// HIPAR qChannelMultiplier/This controls defines a multiplier to take into account various
/// HIPAR qChannelMultiplier/effects resulting from the measurement equipment connected to the Q
/// HIPAR qChannelMultiplier/channel.
/// HIPAR qChannelOffset/This control defines an offset for the Q channel.
/// HIPAR powerConsuptionParamererB/Sets Envelope Power Consumption Parameter A
/// HIPAR powerConsumptionParameterB/Sets Envelope Power Consumption Parameter B
ViStatus _VI_FUNC rsspecan_ConfigureAmplifierEnvelopeTracking(
                            ViSession instrSession,
                            ViReal64  iChannelMultiplier,
                            ViReal64  iChannelOffset,
                            ViReal64  iChannelResistance,
                            ViReal64  qChannelMultiplier,
                            ViReal64  qChannelOffset,
                            ViReal64  powerConsumptionParamererA,
                            ViReal64  powerConsumptionParameterB)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_AMPLIFIER_ENVELOPE_TRACKING_I_CHANNEL_MULTIPLIER, iChannelMultiplier),
            2, "I Channel Multiplier");

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_AMPLIFIER_ENVELOPE_TRACKING_I_CHANNEL_OFFSET, iChannelOffset),
            3, "I Channel Offset");

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_AMPLIFIER_ENVELOPE_TRACKING_I_CHANNEL_RESISTANCE, iChannelResistance),
            4, "I Channel Resistance");

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_AMPLIFIER_ENVELOPE_TRACKING_Q_CHANNEL_MULTIPLIER, qChannelMultiplier),
            5, "Q Channel Multiplier");

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_AMPLIFIER_ENVELOPE_TRACKING_Q_CHANNEL_OFFSET, qChannelOffset),
            6, "Q Channel Offset");

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_AMPLIFIER_ENVELOPE_POWER_CONSUMPTION_PARAMETER_A, powerConsumptionParamererA),
            7, "Power Consumption Parameter A");

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_AMPLIFIER_ENVELOPE_POWER_CONSUMPTION_PARAMETER_B, powerConsumptionParameterB),
            8, "Power Consumption Parameter B");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function allows you to turn off the calculation of all results except the output power.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR state/This control turns evaluation of all results (numerical and graphical) except the output power on and off.
ViStatus _VI_FUNC rsspecan_ConfigureAmplifierPowerEvaluateOnlyDUTPower(
                            ViSession instrSession,
                            ViBoolean state)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_AMPLIFIER_POWER_CALCULATE_OUTPUT_POWER_ONLY_ENABLED, state),
            2, "State");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN This function defines the reference gain for three calculated compression points.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR referenceInputPower/This control  turns automatic calculation of the reference point
/// HIPAR referenceInputPower/required to determine the Compression Points (1 dB, 2 dB and 3 dB) on and of
/// HIPAR referencePoint/This control defines the input power corresponding to the gain
/// HIPAR referencePoint/reference required to calculate the Compression Points.
ViStatus _VI_FUNC rsspecan_ConfigureAmplifierPowerCompressionPointCalculation(
                            ViSession instrSession,
                            ViBoolean referenceInputPower,
                            ViReal64  referencePoint)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_AMPLIFIER_POWER_COMPRESSION_POINT_CALCULATION_ENABLED, referenceInputPower),
            2, "Reference Input Power");
    if (referenceInputPower == VI_FALSE)
    {
        viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_AMPLIFIER_POWER_COMPRESSION_POINT_CALCULATION, referencePoint),
                3, "Reference Point");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function turns the Parameter Sweep on and off.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR state/This control turns evaluation of all results (numerical and
/// HIPAR state/graphical) except the output power on and off.
ViStatus _VI_FUNC rsspecan_ConfigureAmplifierParameterSweepState(
                            ViSession instrSession,
                            ViBoolean state)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_AMPLIFIER_PARAMETER_SWEEP_ENABLED, state),
            2, "State");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function configures the X Axis Parameter Sweep.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR setting/Selects the parameter type for the first parameter controlled by the Parameter Sweep.
/// HIPAR start/Defines the start value for the first parameter controlled by the Parameter Sweep.
/// HIPAR stop/Defines the stop value for the first parameter controlled by the Parameter Sweep.
/// HIPAR step/Defines the stepsize for the first parameter controlled by the Parameter Sweep.
ViStatus _VI_FUNC rsspecan_ConfigureAmplifierParameterSweepXAxis(
                            ViSession instrSession,
                            ViInt32   setting,
                            ViReal64  start,
                            ViReal64  stop,
                            ViReal64  step)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_AMPLIFIER_PARAMETER_SWEEP_X_AXIS_SETTING, setting),
            2, "Setting");

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_AMPLIFIER_PARAMETER_SWEEP_X_AXIS_START, start),
            3, "Start");

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_AMPLIFIER_PARAMETER_SWEEP_X_AXIS_STOP, stop),
            4, "Stop");

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_AMPLIFIER_PARAMETER_SWEEP_X_AXIS_STEP, step),
            5, "Step");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function turns the second parameter controlled by the
/// HIFN Parameter Sweep on and off.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR state/This control turns the second parameter controlled by the Parameter Sweep on and off.
ViStatus _VI_FUNC rsspecan_ConfigureAmplifierParameterSweepYAxisState(
                            ViSession instrSession,
                            ViBoolean state)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_AMPLIFIER_PARAMETER_SWEEP_Y_AXIS_ENABLED, state),
            2, "State");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function configurest the Y Axis Parameter Sweep.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR setting/Selects the parameter type for the second parameter controlled by the Parameter Sweep.
/// HIPAR start/Defines the start value for the second parameter controlled by the Parameter Sweep.
/// HIPAR stop/Defines the stop value for the second parameter controlled by the Parameter Sweep.
/// HIPAR step/Defines the stepsize for the second parameter controlled by the Parameter Sweep.
ViStatus _VI_FUNC rsspecan_ConfigureAmplifierParameterSweepYAxis(
                            ViSession instrSession,
                            ViInt32   setting,
                            ViReal64  start,
                            ViReal64  stop,
                            ViReal64  step)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_AMPLIFIER_PARAMETER_SWEEP_Y_AXIS_SETTING, setting),
            2, "Setting");

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_AMPLIFIER_PARAMETER_SWEEP_Y_AXIS_START, start),
            3, "Start");

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_AMPLIFIER_PARAMETER_SWEEP_Y_AXIS_STOP, stop),
            4, "Stop");

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_AMPLIFIER_PARAMETER_SWEEP_Y_AXIS_STEP, step),
            5, "Step");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function configures the synchronization of reference level
/// HIFN of the analyzer and the RMS level of the generator.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR state/This control turns synchronization of the generator output level
/// HIPAR state/and the analyzer reference level on and off.
/// HIPAR gain/Defines the expected gain of the DUT.
ViStatus _VI_FUNC rsspecan_ConfigureAmplifierParameterSweepAdjustLevel(
                            ViSession instrSession,
                            ViBoolean state,
                            ViReal64  gain)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_AMPLIFIER_PARAMETER_SWEEP_SYNCHRONIZING_LEVELS_ENABLED, state),
            2, "State");
    if (state == VI_TRUE)
    {
        viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_AMPLIFIER_PARAMETER_SWEEP_EXPECTED_GAIN, gain),
                3, "Gain");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN Defines the (expected) amplifier gain considered during the power servoing
/// HIFN measurement and the maximum input power that is fed into the DUT.
/// HIFN of the analyzer and the RMS level of the generator.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR maximumAmplifierInputPower/Sets the (expected) amplifier gain considered during the power servoing measurement.
/// HIPAR expectedAmplifierGain/Sets the maximum input power that is fed into the DUT during the power servoing measurement.
ViStatus _VI_FUNC rsspecan_ConfigureAmplifierPowerServoing(ViSession instrSession,
                                                           ViReal64  maximumAmplifierInputPower,
                                                           ViReal64  expectedAmplifierGain)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_AMPLIFIER_POWER_SERVOING_MAXIMUM_INPUT, maximumAmplifierInputPower),
            2, "Maximum Amplifier Input Power");

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_AMPLIFIER_POWER_SERVOING_EXPECTED_GAIN, expectedAmplifierGain),
            3, "Expected Amplifier Gain");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN Defines the target output power for the power servoing measurement,
/// HIFN and sets the allowed level deviation from the target output power.
/// HIFN of the analyzer and the RMS level of the generator.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR targetOutputPower/Sets the target output power for the power servoing measurement.
/// HIPAR maxOutputPowerDeviation/Sets the allowed level deviation from the target output power.
ViStatus _VI_FUNC rsspecan_ConfigureAmplifierPowerServoingTargetSettings(
                            ViSession instrSession,
                            ViReal64  targetOutputPower,
                            ViReal64  maxOutputPowerDeviation)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_AMPLIFIER_POWER_SERVOING_TARGET_OUTPUT_POWER, targetOutputPower),
            2, "Target Output Power");

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_AMPLIFIER_POWER_SERVOING_MAXIMUM_OUTPUT_DEVIATION, maxOutputPowerDeviation),
            3, "Max Output Power Deviation");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN Configures the data acquisition for Power Servoing measurements.
/// HIFN and sets the allowed level deviation from the target output power.
/// HIFN of the analyzer and the RMS level of the generator.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR state/turns automatic selection of an appropriate capture time on and off in the Power Servoing measurement.
/// HIPAR measurementTime/Sets the capture time in the Power Servoing measurement.
/// HIPAR captureLength/Sets the capture length in the Power Servoing measurement.
ViStatus _VI_FUNC rsspecan_ConfigureAmplifierPowerServoingDataAcquisition(
                            ViSession instrSession,
                            ViBoolean state,
                            ViReal64  measurementTime,
                            ViInt32   captureLength)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_AMPLIFIER_POWER_SERVOING_MEASUREMENT_AUTO, state),
            2, "State");
    if (state == VI_FALSE)
    {
        viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_AMPLIFIER_POWER_SERVOING_MEASUREMENT_TIME, measurementTime),
                3, "Measurement Time");
    }

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_AMPLIFIER_POWER_SERVOING_CAPTURE_LENGTH, captureLength),
            4, "Capture Length");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN Initiates a power servoing measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsspecan_ConfigureAmplifierPowerServoingStart(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_AMPLIFIER_POWER_SERVOING_START, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN Stops a power servoing measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsspecan_ConfigureAmplifierPowerServoingStop(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_AMPLIFIER_POWER_SERVOING_STOP, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets and queries the power servoing state.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR enabled/Sets and queries the power servoing state.
ViStatus _VI_FUNC rsspecan_ConfigureAmplifierSensePowerServoingEnabled(
                            ViSession instrSession,
                            ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, NULL, RSSPECAN_ATTR_AMPLIFIER_SENSE_POWER_SERVOING_ENABLED, enabled),
            2, "Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Sets the power servoing target parameter.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR targetParameter/Sets the power servoing target parameter.
ViStatus _VI_FUNC rsspecan_ConfigureAmplifierSensePowerServoingTargetParameter(
                            ViSession instrSession,
                            ViInt32   targetParameter)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, NULL, RSSPECAN_ATTR_AMPLIFIER_SENSE_POWER_SERVOING_TARGET_PARAMETER, targetParameter),
            2, "Target Parameter");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Sets and queries the power servoing target value and tolerance. The
/// HIFN  unit depends on the selected target parameter.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR targetValue/Sets and queries the power servoing target value. The unit depends on
/// HIPAR targetValue/the selected target parameter.
/// HIPAR targetTolerance/Sets and queries the power servoing target tolerance. The unit depends
/// HIPAR targetTolerance/on the selected target parameter.
ViStatus _VI_FUNC rsspecan_ConfigureAmplifierSensePowerServoingTargetSettings(
                            ViSession instrSession,
                            ViInt32   targetValue,
                            ViReal64  targetTolerance)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, NULL, RSSPECAN_ATTR_AMPLIFIER_SENSE_POWER_SERVOING_TARGET_VALUE, targetValue),
            2, "Target Value");

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, NULL, RSSPECAN_ATTR_AMPLIFIER_SENSE_POWER_SERVOING_TARGET_TOLERANCE, targetTolerance),
            3, "Target Tolerance");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Sets and queries the maximum number of iteratons during the power
/// HIFN  servoing sequence.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR maximumIterarions/Sets and queries the maximum number of iteratons during the power
/// HIPAR maximumIterarions/servoing sequence.
ViStatus _VI_FUNC rsspecan_ConfigureAmplifierSensePowerServoingMaximumIterarions(
                            ViSession instrSession,
                            ViInt32   maximumIterarions)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, NULL, RSSPECAN_ATTR_AMPLIFIER_SENSE_POWER_SERVOING_MAXIMUM_ITERARION, maximumIterarions),
            2, "Maximum Iterarions");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Selects if the generator level is modified using input power or
/// HIFN  digital attenuation.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR levelControl/Selects if the generator level is modified using input power or
/// HIPAR levelControl/digital attenuation.
ViStatus _VI_FUNC rsspecan_ConfigureAmplifierSensePowerServoingGeneratorLevel(
                            ViSession instrSession,
                            ViInt32   levelControl)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, NULL, RSSPECAN_ATTR_AMPLIFIER_SENSE_POWER_SERVOING_GENERATOR_LEVEL, levelControl),
            2, "Level Control");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Defines the input power step size.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR stepSize/Defines the input power step size.
ViStatus _VI_FUNC rsspecan_ConfigureAmplifierSensePowerServoingStepSize(
                            ViSession instrSession,
                            ViReal64  stepSize)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, NULL, RSSPECAN_ATTR_AMPLIFIER_SENSE_POWER_SERVOING_STEP_SIZE, stepSize),
            2, "Step Size");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Starts the power servoing sequence.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsspecan_AmplifierSensePowerServoing(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsspecan_SetAttributeViString(instrSession, NULL, RSSPECAN_ATTR_AMPLIFIER_SENSE_POWER_SERVOING_START, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the status of the power servoing operation.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR status/Queries the status of the power servoing operation.
ViStatus _VI_FUNC rsspecan_QueryAmplifierSensePowerServoingStatus(
                            ViSession  instrSession,
                            ViBoolean* status)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_GetAttributeViBoolean(instrSession, NULL, RSSPECAN_ATTR_AMPLIFIER_SENSE_POWER_SERVOING_STATUS, status),
            2, "Status");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function adds a window to the display.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR stringSize/This control sets the allocated length of Returned Window Name string.
/// HIPAR referenceWindowName/String containing the number or name of the window the new window is
/// HIPAR referenceWindowName/inserted next to. In the default state, the name of
/// HIPAR referenceWindowName/the window is its index.
/// HIPAR direction/Direction the window is added relative to the window set with
/// HIPAR direction/"Window Name".
/// HIPAR windowType/Type of result display you want to add.
/// HIPAR windowType/returnedWindowName/Direction the window is added relative to the window set with
/// HIPAR windowType/"Window Name".
ViStatus _VI_FUNC rsspecan_AmplifierAddWindow(ViSession instrSession,
                                              ViInt32   stringSize,
                                              ViString  referenceWindowName,
                                              ViInt32   direction,
                                              ViInt32   windowType,
                                              ViString  returnedWindowName)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViChar   response[RS_MAX_MESSAGE_BUF_SIZE];
    ViChar*  p2direction;
    ViChar*  p2type;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &rsspecan_rngLayoutDirection, direction, &p2direction),
                4, "Direction");
    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &rsspecan_rngAmplifierLayoutQueryWindowType, windowType, &p2type),
                5, "Window Type");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "LAY:ADD? '%s',%s,%s", referenceWindowName, p2direction, p2type);
    checkErr(RsCore_QueryViString(instrSession, cmd, response));

    if (returnedWindowName != NULL)
    {
        RsCore_TrimString(response, RS_VAL_TRIM_WHITESPACES_AND_ALL_QUOTES);
        RsCore_CopyToUserBufferAsciiData(returnedWindowName, stringSize, response);
    }

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function replaces a window with another one..
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR windowName/String containing the number or name of the window you want to replace
/// HIPAR windowType/Type of result display you want to add.
ViStatus _VI_FUNC rsspecan_AmplifierReplaceWindow(ViSession instrSession,
                                                  ViString  windowName,
                                                  ViInt32   windowType)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViChar*  p2type;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &rsspecan_rngAmplifierLayoutQueryWindowType, windowType, &p2type),
                3, "Window Type");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "LAY:REPL '%s',%s", windowName, p2type);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function configures trace Mode.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR traceMode/This atribute selects the traces to be displayed in the
/// HIPAR traceMode/graphical result displays.
ViStatus _VI_FUNC rsspecan_AmplifierTraceMode(ViSession instrSession,
                                              ViInt32   trace,
                                              ViInt32   traceMode)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, trace, 1, 6),
            2, "Trace");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "TR%ld", trace);
    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_AMPLIFIER_TRACE_MODE, traceMode),
            3, "Trace Mode");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects the default detector for R&S FSW-K18 result displays.
/// HIFN     Note that changing the default detector to Average changes all traces
/// HIFN  with Detector Off to Detector Average. Setting Off will change all
/// HIFN  traces with Detector Average to Detector Off.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR trace/Selects the trace.
/// HIPAR defaultDetector/Selects the default detector for R&S FSW-K18 result displays.
/// HIPAR maxTracePoints/Sets the maximum number of trace points to be used by detectors.
ViStatus _VI_FUNC rsspecan_ConfigureAmplifierTraceDetectorSettings(
                            ViSession instrSession,
                            ViInt32   trace,
                            ViInt32   defaultDetector,
                            ViInt32   maxTracePoints)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_AMPLIFIER_TRACE_DEFAULT_DETECTOR,
            "Trace", trace - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Trace");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_AMPLIFIER_TRACE_DEFAULT_DETECTOR, defaultDetector),
            3, "Default Detector");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_AMPLIFIER_DETECTOR_MAX_TRACE_POINTS, maxTracePoints),
            4, "Max Trace Points");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Selects the traces to be displayed in the graphical result displays.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR trace/Selects the trace.
/// HIPAR mode/Selects the traces to be displayed in the graphical result displays.
/// HIPAR type/Sets and queries the trace result type for the selected result
/// HIPAR type/display.
/// HIPAR detector/Sets and queries the trace detector to be used for the selected result
/// HIPAR detector/display.
ViStatus _VI_FUNC rsspecan_ConfigureAmplifierWindowTraceSettings(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   trace,
                            ViInt32   mode,
                            ViInt32   type,
                            ViInt32   detector)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCapWin[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCapTrac[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_AMPLIFIER_WINDOW_TRACE_MODE,
                    "Window", window, RS_REPCAP_BUF_SIZE, repCapWin),
                2, "Window");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_AMPLIFIER_WINDOW_TRACE_MODE,
                    "Trace", trace - 1, RS_REPCAP_BUF_SIZE, repCapTrac),
                3, "Trace");
    snprintf(repCap, RS_REPCAP_BUF_SIZE, "%s,%s", repCapWin, repCapTrac);

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_AMPLIFIER_WINDOW_TRACE_MODE, mode),
            4, "Mode");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_AMPLIFIER_WINDOW_TRACE_TYPE, type),
            5, "Type");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_AMPLIFIER_WINDOW_TRACE_DETECTOR, detector),
            6, "Detector");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Adds and removes results from the statistics table.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR item/Item to be included in statistics table.
/// HIPAR state/Enables/disables selected item.
ViStatus _VI_FUNC rsspecan_DisplayAmplifierWindowStatisticsTableItem(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   item,
                            ViBoolean state)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   param[RS_PARAM_BUF_SIZE] = "";
    ViChar*  p2type;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_HIDDEN_AMPLIFIER_DISPLAY_WINDOW_STATISTICS_TABLE_ITEM,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap),
            2, "Window");
    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &RsSpecAn_rngAmplifierResultSummaryTableItem, item, &p2type),
                3, "Item");

    snprintf(param, RS_PARAM_BUF_SIZE, "%s,%s", p2type, state ? "OFF" : "ON");

    checkErr(rsspecan_SetAttributeViString(instrSession, repCap, RSSPECAN_ATTR_HIDDEN_AMPLIFIER_DISPLAY_WINDOW_STATISTICS_TABLE_ITEM, param));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Configures results visible in the statistics table.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR macc/Adds and removes all modulation accuracy results from the statistics
/// HIPAR macc/table.
/// HIPAR power/Adds and removes all power results from the statistics table.
/// HIPAR volt/curr/Adds and removes all voltage/current results from the statistics
/// HIPAR volt/curr/table. Prerequisites for this command: Select a baseband measurement.
ViStatus _VI_FUNC rsspecan_DisplayAmplifierWindowStatisticsTableItemsAll(
                            ViSession instrSession,
                            ViInt32   window,
                            ViBoolean macc,
                            ViBoolean power,
                            ViBoolean voltCurr)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_AMPLIFIER_DISPLAY_WINDOW_STATISTICS_TABLE_ITEM_MACC_ALL,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap),
            2, "Window");

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, repCap, RSSPECAN_ATTR_AMPLIFIER_DISPLAY_WINDOW_STATISTICS_TABLE_ITEM_MACC_ALL, macc),
            3, "MACC");

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, repCap, RSSPECAN_ATTR_AMPLIFIER_DISPLAY_WINDOW_STATISTICS_TABLE_ITEM_POWER_ALL, power),
            4, "Power");

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, repCap, RSSPECAN_ATTR_AMPLIFIER_DISPLAY_WINDOW_STATISTICS_TABLE_ITEM_VOLT_CURR_ALL, voltCurr),
            5, "Volt Curr");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Adds and removes results from the statistics table.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/Selects repeated capability Window.
/// HIPAR item/Item to be included in statistics table.
/// HIPAR state/Enables/disables selected item.
ViStatus _VI_FUNC rsspecan_AmplifierDisplayWindowStatisticsTableItem(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   item,
                            ViBoolean state)
{
    ViStatus error = VI_SUCCESS;
    ViChar   param[RS_PARAM_BUF_SIZE];
    ViChar   repCap[RS_REPCAP_BUF_SIZE];
    ViInt32  repcapCount = 0;
    ViChar*  p2itemString;

    checkErr(RsCore_LockSession(instrSession));

    (void)RsCore_GetRepCapIdItemsCount(instrSession, "Window", &repcapCount);

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, window, 0, repcapCount),
            2, "Window");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, instrSession,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap),
            2, "Window");
    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &RsSpecAn_rngAmplifierResultSummaryTableItem, item, &p2itemString),
                3, "Item");

    (void)snprintf(param, RS_PARAM_BUF_SIZE, "%s,%s", p2itemString, state ? "ON" : "OFF");

    checkErr(rsspecan_SetAttributeViString(instrSession, repCap, RSSPECAN_ATTR_HIDDEN_AMPLIFIER_DISPLAY_WINDOW_STATISTICS_TABLE_ITEM, param));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function defines which items are displayedin the Result Summary.
/// HIFN  Note that the results are always calculated, regardless of their
/// HIFN  visibility in the Result Summary.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR item/Selects the item to be included in Result Summary.
/// HIPAR displayItem/This parameter selects whether the results summary item is displayed.
ViStatus _VI_FUNC rsspecan_ConfigureAmplifierResultSummaryDisplay(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   item,
                            ViBoolean displayItem)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";
    ViChar*  p2string;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentModel(instrSession, "FSW"));

    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &RsSpecAn_rngAmplifierResultSummaryTableItem, item, &p2string),
                3, "Item");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "DISP:WIND%ld:TABL:ITEM %s,%s", window, p2string,
             displayItem ? "ON" : "OFF");
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function adds and removes all selected results from the
/// HIFN Result Summary.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR item/Selects the item in Result Summary.
/// HIPAR displayItem/This parameter selects whether the results summary items are
/// HIPAR displayItem/displayed.
ViStatus _VI_FUNC rsspecan_ConfigureAmplifierResultSummaryDisplayAll(
                            ViSession instrSession,
                            ViInt32   item,
                            ViBoolean displayItem)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    switch (item)
    {
    case RSSPECAN_VAL_RESULT_ALL_MACC:
        viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_AMPLIFIER_DISPLAY_WINDOW_TABLE_ITEM_MACC_ALL, displayItem),
                2, "Display Item");
        break;
    case RSSPECAN_VAL_RESULT_ALL_POWER:
        viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_AMPLIFIER_DISPLAY_WINDOW_TABLE_ITEM_POWER_ALL, displayItem),
                2, "Display Item");
        break;
    case RSSPECAN_VAL_RESULT_ALL_VCURR:
        viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_AMPLIFIER_DISPLAY_WINDOW_TABLE_ITEM_VC_ALL, displayItem),
                2, "Display Item");
        break;
    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, item), 2, "Result Type");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function adds and removes all selected results from the Result
/// HIFN  Summary.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR item/Selects the item in Result Summary.
/// HIPAR displayItem/This parameter selects whether the results summary items are
/// HIPAR displayItem/displayed.
ViStatus _VI_FUNC rsspecan_ConfigureAmplifierWindowResultSummaryDisplayAll(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   item,
                            ViBoolean displayItem)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, window, 1, 16),
            2, "Window");

    (void)snprintf(repCap, RS_REPCAP_BUF_SIZE, "Win%ld", window);

    switch (item)
    {
    case RSSPECAN_VAL_RESULT_ALL_MACC:
        viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, repCap, RSSPECAN_ATTR_AMPLIFIER_DISPLAY_WINDOW_RESULT_SUMMARY_TABLE_ITEM_MACC_ALL, displayItem),
                3, "Display Item");
        break;
    case RSSPECAN_VAL_RESULT_ALL_POWER:
        viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, repCap, RSSPECAN_ATTR_AMPLIFIER_DISPLAY_WINDOW_RESULT_SUMMARY_TABLE_ITEM_POWER_ALL, displayItem),
                3, "Display Item");
        break;
    case RSSPECAN_VAL_RESULT_ALL_VCURR:
        viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, repCap, RSSPECAN_ATTR_AMPLIFIER_DISPLAY_WINDOW_RESULT_SUMMARY_TABLE_ITEM_VC_ALL, displayItem),
                3, "Display Item");
        break;

    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, item), 3, "Item");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures Time Domain Display Settings.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR normaliseTo1/This control turns normalization of the results in the Time Domain
/// HIPAR normaliseTo1/result display on and off.
/// HIPAR position/This control turns automatic scaling of the x-axis in the Time Domain
/// HIPAR position/result display on and off.
/// HIPAR offset/This control defines the origin of the x-axis in the Time Domain
/// HIPAR offset/result display.
/// HIPAR duration/This control defines the amount of data displayed on the x-axis of the
/// HIPAR duration/Time Domain result display.
ViStatus _VI_FUNC rsspecan_ConfigureAmplifierTimeDomainDisplaySettings(
                            ViSession instrSession,
                            ViBoolean normaliseTo1,
                            ViBoolean position,
                            ViReal64  offset,
                            ViReal64  duration)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_AMPLIFIER_TIME_DOMAIN_NORMALIZATION_STATE, normaliseTo1),
            2, "Normalise To 1");

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_AMPLIFIER_TIME_DOMAIN_POSITION, position),
            3, "Position");

    if (position == VI_FALSE)
    {
        viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_AMPLIFIER_TIME_DOMAIN_OFFSET, offset),
                4, "Offset");
    }

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_AMPLIFIER_TIME_DOMAIN_DURATION, duration),
            5, "Duration");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function configures Phase Settings.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR unit/Selects the unit that the phase is shown in the AM/PM result
/// HIPAR unit/display.
/// HIPAR AMPMDefinition/Selects the way the AM/PM results are calculated.
ViStatus _VI_FUNC rsspecan_ConfigureAmplifierPhaseDisplaySettings(
                            ViSession instrSession,
                            ViInt32   unit,
                            ViInt32   AMPMDefinition)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_AMPLIFIER_AM_PM_DISPLAY_PHASE, unit),
            2, "Unit");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_AMPLIFIER_AM_PM_DEFINITION, AMPMDefinition),
            3, "AM/PM Definition");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets and queries the curve width computation refrence point mode and
/// HIFN  value.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR automatic/Sets and queries the curve width computation refrence point mode.
/// HIPAR referencePoint/Sets and queries the curve width computation reference point
ViStatus _VI_FUNC rsspecan_ConfigureAmplifierAMPMCurveWidthReferencePoint(
                            ViSession instrSession,
                            ViBoolean automatic,
                            ViReal64  referencePoint)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, NULL, RSSPECAN_ATTR_AMPLIFIER_AM_PM_CURVE_WIDTH_REFERENCE_POINT_AUTO_ENABLED, automatic),
            2, "Automatic");

    if (automatic == VI_FALSE)
        viCheckParm(rsspecan_SetAttributeViReal64(instrSession, NULL, RSSPECAN_ATTR_AMPLIFIER_AM_PM_CURVE_WIDTH_REFERENCE_POINT, referencePoint),
                3, "Reference Point");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function configures the information references on the x-axis.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR powerReference/Selects the type of information displayed on x-axis
ViStatus _VI_FUNC rsspecan_ConfigureAmplifierPowerReferenceDisplaySettings(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   powerReference)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_AMPLIFIER_POWER_REFERENCE,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap),
            2, "Window");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_AMPLIFIER_POWER_REFERENCE, powerReference),
            3, "Power Reference");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function adds and removes results from the Parameter Sweep Table.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR item/Selects the result.
/// HIPAR displayItem/This parameter selects whether the result item is displayed.
ViStatus _VI_FUNC rsspecan_ConfigureAmplifierResultParameterSweepTable(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   item,
                            ViBoolean displayItem)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViChar*  p2string;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentModel(instrSession, "FSW"));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, window, 1, 16),
            2, "Window");
    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &rsspecan_rngAmplifierParametersSweepResultType, item, &p2string),
                3, "Item");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "DISP:WIND%ld:PTAB:ITEM %s,%s", window, p2string, displayItem ? "ON" : "OFF");
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function adds and removes results from the Parameter Sweep Table.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR displayItem/This parameter selects whether the results summary item is
/// HIPAR displayItem/displayed.
ViStatus _VI_FUNC rsspecan_ConfigureAmplifierResultParameterSweepTableAll(
                            ViSession instrSession,
                            ViBoolean displayItem)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsspecan_ConfigureAmplifierWindowResultParameterSweepTableAll(instrSession, 0, displayItem));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function adds and removes results from the Parameter Sweep Table.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR displayItem/This parameter selects whether the results summary item is displayed.
ViStatus _VI_FUNC rsspecan_ConfigureAmplifierWindowResultParameterSweepTableAll(
                            ViSession instrSession,
                            ViInt32   window,
                            ViBoolean displayItem)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, window, 1, 16),
            2, "Window");
    snprintf(repCap, RS_REPCAP_BUF_SIZE, "Win%ld", window);

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, repCap, RSSPECAN_ATTR_AMPLIFIER_PARAMETERS_SWEEP_TABLE_ITEM_ALL, displayItem),
            3, "Display Item");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function  selects the result type displayed on the y-axis of the parameter sweep diagram.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR item/Selects the result type.
ViStatus _VI_FUNC rsspecan_ConfigureAmplifierResultParameterSweepDisplay(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   item)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_AMPLIFIER_PARAMETERS_SWEEP_RESULT_TYPE,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap),
            2, "Window");
    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_AMPLIFIER_PARAMETERS_SWEEP_RESULT_TYPE, item),
            3, "Item");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the result type of the DDPD Results result display.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR resultType/Configures the result type of the DDPD Results result display.
ViStatus _VI_FUNC rsspecan_ConfigureAmplifierDisplayWindowDDPDResultsType(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   resultType)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_AMPLIFIER_DISPLAY_WINDOW_DDPD_RESULTS_TYPE,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap),
            2, "Window");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_AMPLIFIER_DISPLAY_WINDOW_DDPD_RESULTS_TYPE, resultType),
            3, "Result Type");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function queries all numerical results shown in the Result Summary.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR arraySize/Number of array points requested.
/// HIPAR results/A user allocated buffer into which the result summary values is stored.
/// HIPAR actualPoints/Number of points actually returned in the array.
ViStatus _VI_FUNC rsspecan_GetAmplifierAllResultSummary(ViSession instrSession,
                                                        ViInt32   arraySize,
                                                        ViReal64  results[],
                                                        ViInt32*  actualPoints)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32RangeMin(instrSession, arraySize, 1),
                3, "Array Size");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, results), 4, "Results");

    checkErr(RsCore_QueryFloatArrayToUserBuffer(instrSession, "FETC:MACC:RES:ALL?", arraySize, results, actualPoints));

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries all power related numerical results as shown in the result summary.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR arraySize/Number of array points requested.
/// HIPAR results/A user allocated buffer into which the result summary values is stored.
/// HIPAR actualPoints/Number of points actually returned in the array.
ViStatus _VI_FUNC rsspecan_GetAmplifierPowerAllResult(ViSession instrSession,
                                                      ViInt32   arraySize,
                                                      ViReal64  results[],
                                                      ViInt32*  actualPoints)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32RangeMin(instrSession, arraySize, 1),
                3, "Array Size");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, results), 4, "Results");

    checkErr(RsCore_QueryFloatArrayToUserBuffer(instrSession, "FETC:POW:ALL?", arraySize, results, actualPoints));

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function reutrns results to check modulation accuracy.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR resultType/This control selects the result type to be queried.
/// HIPAR measurementType/Selects measurement type.
/// HIPAR result/Returns selected result.
ViStatus _VI_FUNC rsspecan_GetAmplifierModulationAccuracyResults(
                            ViSession instrSession,
                            ViInt32   resultType,
                            ViInt32   measurementType,
                            ViReal64* result)
{
    ViStatus                                           error = VI_SUCCESS;
    ViAttr                                             attr[10][3] = {
                                                       {
    RSSPECAN_ATTR_AMPLIFIER_RAW_EVM_CURRENT,           RSSPECAN_ATTR_AMPLIFIER_RAW_EVM_MINIMUM,
                                                       RSSPECAN_ATTR_AMPLIFIER_RAW_EVM_MAXIMUM
                                                       },
                                                       {
    RSSPECAN_ATTR_AMPLIFIER_RAW_MODEL_EVM_CURRENT,     RSSPECAN_ATTR_AMPLIFIER_RAW_MODEL_EVM_MINIMUM,
                                                       RSSPECAN_ATTR_AMPLIFIER_RAW_MODEL_EVM_MAXIMUM
                                                       },
                                                       {
    RSSPECAN_ATTR_AMPLIFIER_FREQUENCY_ERROR_CURRENT,   RSSPECAN_ATTR_AMPLIFIER_FREQUENCY_ERROR_MINIMUM,
                                                       RSSPECAN_ATTR_AMPLIFIER_FREQUENCY_ERROR_MAXIMUM
                                                       },
                                                       {
    RSSPECAN_ATTR_AMPLIFIER_SAMPLE_RATE_ERROR_CURRENT, RSSPECAN_ATTR_AMPLIFIER_SAMPLE_RATE_ERROR_MINIMUM,
                                                       RSSPECAN_ATTR_AMPLIFIER_SAMPLE_RATE_ERROR_MAXIMUM
                                                       },
                                                       {
    RSSPECAN_ATTR_AMPLIFIER_MAGNITUDE_ERROR_CURRENT,   RSSPECAN_ATTR_AMPLIFIER_MAGNITUDE_ERROR_MINIMUM,
                                                       RSSPECAN_ATTR_AMPLIFIER_MAGNITUDE_ERROR_MAXIMUM
                                                       },
                                                       {
    RSSPECAN_ATTR_AMPLIFIER_PHASE_ERROR_CURRENT,       RSSPECAN_ATTR_AMPLIFIER_PHASE_ERROR_MINIMUM,
                                                       RSSPECAN_ATTR_AMPLIFIER_PHASE_ERROR_MAXIMUM
                                                       },
                                                       {
    RSSPECAN_ATTR_AMPLIFIER_QUADRATURE_ERROR_CURRENT,  RSSPECAN_ATTR_AMPLIFIER_QUADRATURE_ERROR_MINIMUM,
                                                       RSSPECAN_ATTR_AMPLIFIER_QUADRATURE_ERROR_MAXIMUM
                                                       },
                                                       {
    RSSPECAN_ATTR_AMPLIFIER_GAIN_IMBALANCE_CURRENT,    RSSPECAN_ATTR_AMPLIFIER_GAIN_IMBALANCE_MINIMUM,
                                                       RSSPECAN_ATTR_AMPLIFIER_GAIN_IMBALANCE_MAXIMUM
                                                       },
                                                       {
    RSSPECAN_ATTR_AMPLIFIER_IQ_IMBALANCE_CURRENT,      RSSPECAN_ATTR_AMPLIFIER_IQ_IMBALANCE_MINIMUM,
                                                       RSSPECAN_ATTR_AMPLIFIER_IQ_IMBALANCE_MAXIMUM
                                                       },
                                                       {
    RSSPECAN_ATTR_AMPLIFIER_IQ_OFFSET_CURRENT,         RSSPECAN_ATTR_AMPLIFIER_IQ_OFFSET_MINIMUM,
                                                       RSSPECAN_ATTR_AMPLIFIER_IQ_OFFSET_MAXIMUM
                                                       }
                                                       };

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, resultType, RSSPECAN_VAL_RESULT_SUMMARY_REVM, RSSPECAN_VAL_RESULT_SUMMARY_IQOF),
            2, "Result Type");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, measurementType, RSSPECAN_VAL_MEASTYPE_CURR, RSSPECAN_VAL_MEASTYPE_MAX),
            3, "Measurement Type");

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, "", attr[resultType][measurementType], result),
            2, "Result");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the absolute phase value between reference signal and measured
/// HIFN  signal. Note that the absolute phase is not relevant for R&S FSW-K18
/// HIFN  measurements. However, it can be used to track the absolute phase
/// HIFN  stability between generator and analyzer (including their local
/// HIFN  oscillators).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR result/Queries the absolute phase value between reference signal and measured
/// HIPAR result/signal. Note that the absolute phase is not relevant for R&S FSW-K18
/// HIPAR result/measurements. However, it can be used to track the absolute phase
/// HIPAR result/stability between generator and analyzer (including their local
/// HIPAR result/oscillators).
ViStatus _VI_FUNC rsspecan_GetAmplifierPhaseErrorResults(ViSession instrSession,
                                                         ViReal64* result)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_AMPLIFIER_PHASE_ERROR_RESULT, result),
            2, "Result");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function reutrns results to check power characteristcs.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR resultType/This control selects the result type to be queried.
/// HIPAR measurementType/Selects measurement type.
/// HIPAR result/Returns selected result.
ViStatus _VI_FUNC rsspecan_GetAmplifierPowerResults(ViSession instrSession,
                                                    ViInt32   resultType,
                                                    ViInt32   measurementType,
                                                    ViReal64* result)
{
    ViStatus                                               error = VI_SUCCESS;
    ViAttr                                                 attr[10][3] = {
                                                           {
    RSSPECAN_ATTR_AMPLIFIER_POWER_INPUT_CURRENT,           RSSPECAN_ATTR_AMPLIFIER_POWER_INPUT_MINIMUM,
                                                           RSSPECAN_ATTR_AMPLIFIER_POWER_INPUT_MAXIMUM
                                                           },
                                                           {
    RSSPECAN_ATTR_AMPLIFIER_POWER_OUTPUT_CURRENT,          RSSPECAN_ATTR_AMPLIFIER_POWER_OUTPUT_MINIMUM,
                                                           RSSPECAN_ATTR_AMPLIFIER_POWER_OUTPUT_MAXIMUM
                                                           },
                                                           {
    RSSPECAN_ATTR_AMPLIFIER_SIGNAL_GAIN_CURRENT,           RSSPECAN_ATTR_AMPLIFIER_SIGNAL_GAIN_MINIMUM,
                                                           RSSPECAN_ATTR_AMPLIFIER_SIGNAL_GAIN_MAXIMUM
                                                           },
                                                           {
    RSSPECAN_ATTR_AMPLIFIER_CREST_FACTOR_INPUT_CURRENT,    RSSPECAN_ATTR_AMPLIFIER_CREST_FACTOR_INPUT_MINIMUM,
                                                           RSSPECAN_ATTR_AMPLIFIER_CREST_FACTOR_INPUT_MAXIMUM
                                                           },
                                                           {
    RSSPECAN_ATTR_AMPLIFIER_CREST_FACTOR_OUTPUT_CURRENT,   RSSPECAN_ATTR_AMPLIFIER_CREST_FACTOR_OUTPUT_MINIMUM,
                                                           RSSPECAN_ATTR_AMPLIFIER_CREST_FACTOR_OUTPUT_MAXIMUM
                                                           },
                                                           {
    RSSPECAN_ATTR_AMPLIFIER_AMAM_CURVE_WIDTH_CURRENT,      RSSPECAN_ATTR_AMPLIFIER_AMAM_CURVE_WIDTH_MINIMUM,
                                                           RSSPECAN_ATTR_AMPLIFIER_AMAM_CURVE_WIDTH_MAXIMUM
                                                           },
                                                           {
    RSSPECAN_ATTR_AMPLIFIER_AMPM_CURVE_WIDTH_CURRENT,      RSSPECAN_ATTR_AMPLIFIER_AMPM_CURVE_WIDTH_MINIMUM,
                                                           RSSPECAN_ATTR_AMPLIFIER_AMPM_CURVE_WIDTH_MAXIMUM
                                                           },
                                                           {
                                                           RSSPECAN_ATTR_AMPLIFIER_COMPRESSION_POINT_1DB_CURRENT,
    RSSPECAN_ATTR_AMPLIFIER_COMPRESSION_POINT_1DB_CURRENT, RSSPECAN_ATTR_AMPLIFIER_COMPRESSION_POINT_1DB_CURRENT
                                                           },
                                                           {
                                                           RSSPECAN_ATTR_AMPLIFIER_COMPRESSION_POINT_2DB_CURRENT,
    RSSPECAN_ATTR_AMPLIFIER_COMPRESSION_POINT_2DB_CURRENT, RSSPECAN_ATTR_AMPLIFIER_COMPRESSION_POINT_2DB_CURRENT
                                                           },
                                                           {
                                                           RSSPECAN_ATTR_AMPLIFIER_COMPRESSION_POINT_3DB_CURRENT,
    RSSPECAN_ATTR_AMPLIFIER_COMPRESSION_POINT_3DB_CURRENT, RSSPECAN_ATTR_AMPLIFIER_COMPRESSION_POINT_3DB_CURRENT
                                                           }
                                                           };

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, measurementType, RSSPECAN_VAL_MEASTYPE_CURR, RSSPECAN_VAL_MEASTYPE_MAX),
            3, "Measurement Type");

    switch (resultType)
    {
    case RSSPECAN_VAL_RESULT_SUMMARY_PINP:
        viCheckParm(rsspecan_GetAttributeViReal64(instrSession, "", attr[0][measurementType], result),
                2, "Result");
        break;
    case RSSPECAN_VAL_RESULT_SUMMARY_POUT:
        viCheckParm(rsspecan_GetAttributeViReal64(instrSession, "", attr[1][measurementType], result),
                2, "Result");
        break;
    case RSSPECAN_VAL_RESULT_SUMMARY_GAIN:
        viCheckParm(rsspecan_GetAttributeViReal64(instrSession, "", attr[2][measurementType], result),
                2, "Result");
        break;
    case RSSPECAN_VAL_RESULT_SUMMARY_CFIN:
        viCheckParm(rsspecan_GetAttributeViReal64(instrSession, "", attr[3][measurementType], result),
                2, "Result");
        break;
    case RSSPECAN_VAL_RESULT_SUMMARY_CFOU:
        viCheckParm(rsspecan_GetAttributeViReal64(instrSession, "", attr[4][measurementType], result),
                2, "Result");
        break;
    case RSSPECAN_VAL_RESULT_SUMMARY_AMW:
        viCheckParm(rsspecan_GetAttributeViReal64(instrSession, "", attr[5][measurementType], result),
                2, "Result");
        break;
    case RSSPECAN_VAL_RESULT_SUMMARY_PMW:
        viCheckParm(rsspecan_GetAttributeViReal64(instrSession, "", attr[6][measurementType], result),
                2, "Result");
        break;
    case RSSPECAN_VAL_RESULT_SUMMARY_P1DB:
        viCheckParm(RsCore_InvalidViInt32Range(instrSession, measurementType, RSSPECAN_VAL_MEASTYPE_CURR, RSSPECAN_VAL_MEASTYPE_CURR),
                3, "Measurement Type");
        viCheckParm(rsspecan_GetAttributeViReal64(instrSession, "", attr[7][measurementType], result),
                2, "Result");
        break;
    case RSSPECAN_VAL_RESULT_SUMMARY_P2DB:
        viCheckParm(RsCore_InvalidViInt32Range(instrSession, measurementType, RSSPECAN_VAL_MEASTYPE_CURR, RSSPECAN_VAL_MEASTYPE_CURR),
                3, "Measurement Type");
        viCheckParm(rsspecan_GetAttributeViReal64(instrSession, "", attr[8][measurementType], result),
                2, "Result");
        break;
    case RSSPECAN_VAL_RESULT_SUMMARY_P3DB:
        viCheckParm(RsCore_InvalidViInt32Range(instrSession, measurementType, RSSPECAN_VAL_MEASTYPE_CURR, RSSPECAN_VAL_MEASTYPE_CURR),
                3, "Measurement Type");
        viCheckParm(rsspecan_GetAttributeViReal64(instrSession, "", attr[9][measurementType], result),
                2, "Result");
        break;
    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, resultType), 2, "Result Type");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Fetches the signal power results at the input power sensor.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR measurementType/Selects measurement type.
/// HIPAR result/Fetches the minimum signal power at the input power sensor.
ViStatus _VI_FUNC rsspecan_GetAmplifierPowerResultsInputSensor(
                            ViSession instrSession,
                            ViInt32   measurementType,
                            ViReal64* result)
{
    ViStatus error = VI_SUCCESS;
    ViAttr   measurementAttributes[3] = { RSSPECAN_ATTR_AMPLIFIER_INPUT_POWER_SENSOR_CURRENT, RSSPECAN_ATTR_AMPLIFIER_INPUT_POWER_SENSOR_MINIMUM, RSSPECAN_ATTR_AMPLIFIER_INPUT_POWER_SENSOR_MAXIMUM };

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, measurementType, RSSPECAN_VAL_MEASTYPE_CURR, RSSPECAN_VAL_MEASTYPE_MAX),
            2, "Measurement Type");

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, NULL, measurementAttributes[measurementType], result),
            3, "Result");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Fetches the signal power results at the output power sensor.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR measurementType/Selects measurement type.
/// HIPAR result/Fetches the maximum signal power at the output power sensor.
ViStatus _VI_FUNC rsspecan_GetAmplifierPowerResultsOutputSensor(
                            ViSession instrSession,
                            ViInt32   measurementType,
                            ViReal64* result)
{
    ViStatus error = VI_SUCCESS;
    ViAttr   attribute;

    checkErr(RsCore_LockSession(instrSession));

    switch (measurementType)
    {
        case RSSPECAN_VAL_MEASTYPE_MAX:
            attribute = RSSPECAN_ATTR_AMPLIFIER_OUTPUT_POWER_SENSOR_MAXIMUM;
            break;

        case RSSPECAN_VAL_MEASTYPE_CURR:
        default:
            attribute = RSSPECAN_ATTR_AMPLIFIER_OUTPUT_POWER_SENSOR_CURRENT;
    }

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, NULL, attribute, result),
            2, "Result");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the 'AM/PM' peak curve width as shown in the result summary.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR result/Queries the 'AM/PM' peak curve width as shown in the result summary.
ViStatus _VI_FUNC rsspecan_GetAmplifierAMPMCurveWidthPeakCurrent(
                            ViSession instrSession,
                            ViReal64* result)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, NULL, RSSPECAN_ATTR_AMPLIFIER_AMPM_CURVE_WIDTH_PEAK_CURRENT, result),
            2, "Result");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the 'AM/AM' peak curve width as shown in the result summary.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR result/Queries the 'AM/AM' peak curve width as shown in the result summary.
ViStatus _VI_FUNC rsspecan_GetAmplifierAMAMCurveWidthPeakCurrent(
                            ViSession instrSession,
                            ViReal64* result)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, NULL, RSSPECAN_ATTR_AMPLIFIER_AMAM_CURVE_WIDTH_PEAK_CURRENT, result),
            2, "Result");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function returns results to check the power supply characteristics of the
/// HIFN  amplifier. These results are available when you turn on the baseband input.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR resultType/This control selects the result type to be queried.
/// HIPAR measurementType/Selects measurement type.
/// HIPAR result/Returns selected result.
ViStatus _VI_FUNC rsspecan_GetAmplifierBasebandCharacteristicsResults(
                            ViSession instrSession,
                            ViInt32   resultType,
                            ViInt32   measurementType,
                            ViReal64* result)
{
    ViStatus                                     error = VI_SUCCESS;
    ViAttr                                       attr[6][3] = {
                                                 {
                                                 RSSPECAN_ATTR_AMPLIFIER_BASEBAND_INPUT_VOLTAGE_I_CURRENT,
                                                 RSSPECAN_ATTR_AMPLIFIER_BASEBAND_INPUT_VOLTAGE_I_MINIMUM,
                                                 RSSPECAN_ATTR_AMPLIFIER_BASEBAND_INPUT_VOLTAGE_I_MAXIMUM
                                                 },
                                                 {
                                                 RSSPECAN_ATTR_AMPLIFIER_BASEBAND_INPUT_VOLTAGE_Q_CURRENT,
                                                 RSSPECAN_ATTR_AMPLIFIER_BASEBAND_INPUT_VOLTAGE_Q_MINIMUM,
                                                 RSSPECAN_ATTR_AMPLIFIER_BASEBAND_INPUT_VOLTAGE_Q_MAXIMUM
                                                 },
                                                 {
    RSSPECAN_ATTR_AMPLIFIER_VOLTAGE_CURRENT,     RSSPECAN_ATTR_AMPLIFIER_VOLTAGE_MINIMUM,
                                                 RSSPECAN_ATTR_AMPLIFIER_VOLTAGE_MAXIMUM
                                                 },
                                                 {
    RSSPECAN_ATTR_AMPLIFIER_CURRENT_CURRENT,     RSSPECAN_ATTR_AMPLIFIER_CURRENT_MINIMUM,
                                                 RSSPECAN_ATTR_AMPLIFIER_CURRENT_MAXIMUM
                                                 },
    {RSSPECAN_ATTR_AMPLIFIER_POWER_CURRENT,      0, 0},
                                                 {
    RSSPECAN_ATTR_AMPLIFIER_AVERAGE_PAE_CURRENT, RSSPECAN_ATTR_AMPLIFIER_AVERAGE_PAE_MINIMUM,
                                                 RSSPECAN_ATTR_AMPLIFIER_AVERAGE_PAE_MAXIMUM
                                                 }
                                                 };

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, resultType, RSSPECAN_VAL_RESULT_SUMMARY_BBIV, RSSPECAN_VAL_RESULT_SUMMARY_APAE),
            2, "Result Type");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, measurementType, RSSPECAN_VAL_MEASTYPE_CURR, RSSPECAN_VAL_MEASTYPE_MAX),
            3, "Measurement Type");

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, "",
                    attr[resultType-17][measurementType],
                    result), 2, "Result");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Returns the results of baseband power measurements.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR measurementType/Selects measurement type.
/// HIPAR result/Returns selected result.
ViStatus _VI_FUNC rsspecan_GetAmplifierBasebandPowerResults(ViSession instrSession,
                                                            ViInt32   measurementType,
                                                            ViReal64* result)
{
    ViStatus error = VI_SUCCESS;
    ViAttr   attrArr[3] = {
             RSSPECAN_ATTR_AMPLIFIER_BASEBAND_POWER_CURRENT,
             RSSPECAN_ATTR_AMPLIFIER_BASEBAND_POWER_MINIMUM,
             RSSPECAN_ATTR_AMPLIFIER_BASEBAND_POWER_MAXIMUM
             };

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, measurementType, 0, 2),
            2, "Measurement Type");
    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, "", attrArr[measurementType], result),
            3, "Result");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function returns Parameters Swep Table results.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR resultType/This control selects the result type to be queried.
/// HIPAR measurementType/Selects measurement type.
/// HIPAR result/Returns selected result.
ViStatus _VI_FUNC rsspecan_GetAmplifierParameterSweepTableResults(
                            ViSession instrSession,
                            ViInt32   resultType,
                            ViInt32   measurementType,
                            ViReal64* result)
{
    ViStatus error = VI_SUCCESS;
    ViAttr   attr = RSSPECAN_ATTR_AMPLIFIER_PTABLE_ACP_MINIMUM;

    checkErr(RsCore_LockSession(instrSession));

    switch (measurementType)
    {
    case RSSPECAN_VAL_MEASTYPE_MIN:
        switch (resultType)
        {
        case RSSPECAN_VAL_RESULT_PSWEEP_ACP:
            attr = RSSPECAN_ATTR_AMPLIFIER_PTABLE_ACP_MINIMUM;
            break;
        case RSSPECAN_VAL_RESULT_PSWEEP_ACL1:
            attr = RSSPECAN_ATTR_AMPLIFIER_PTABLE_ACP_LOWER_MINIMUM;
            break;
        case RSSPECAN_VAL_RESULT_PSWEEP_ACU1:
            attr = RSSPECAN_ATTR_AMPLIFIER_PTABLE_ACP_UPPER_MINIMUM;
            break;
        case RSSPECAN_VAL_RESULT_PSWEEP_AMW:
            attr = RSSPECAN_ATTR_AMPLIFIER_PTABLE_AMAM_CURVE_WIDTH_MINIMUM;
            break;
        case RSSPECAN_VAL_RESULT_PSWEEP_CFAC:
            attr = RSSPECAN_ATTR_AMPLIFIER_PTABLE_CREST_FACTOR_MINIMUM;
            break;
        case RSSPECAN_VAL_RESULT_PSWEEP_EVM:
            attr = RSSPECAN_ATTR_AMPLIFIER_PTABLE_EVM_MINIMUM;
            break;
        case RSSPECAN_VAL_RESULT_PSWEEP_GAIN:
            attr = RSSPECAN_ATTR_AMPLIFIER_PTABLE_GAIN_MINIMUM;
            break;
        case RSSPECAN_VAL_RESULT_PSWEEP_ICC:
            attr = RSSPECAN_ATTR_AMPLIFIER_PTABLE_ICC_MINIMUM;
            break;
        case RSSPECAN_VAL_RESULT_PSWEEP_VCC:
            attr = RSSPECAN_ATTR_AMPLIFIER_PTABLE_VCC_MINIMUM;
            break;
        case RSSPECAN_VAL_RESULT_PSWEEP_POW:
            attr = RSSPECAN_ATTR_AMPLIFIER_PTABLE_BB_POWER_MINIMUM;
            break;
        case RSSPECAN_VAL_RESULT_PSWEEP_PAE:
            attr = RSSPECAN_ATTR_AMPLIFIER_PTABLE_PAE_MINIMUM;
            break;
        case RSSPECAN_VAL_RESULT_PSWEEP_PMW:
            attr = RSSPECAN_ATTR_AMPLIFIER_PTABLE_AMPM_CURVE_WIDTH_MINIMUM;
            break;
        case RSSPECAN_VAL_RESULT_PSWEEP_RMS:
            attr = RSSPECAN_ATTR_AMPLIFIER_PTABLE_RMS_POWER_MINIMUM;
            break;
        case RSSPECAN_VAL_RESULT_PSWEEP_ACBM:
            attr = RSSPECAN_ATTR_AMPLIFIER_PTABLE_ACP_BALANCED_MINIMUM;
            break;
        default:
            viCheckParm(RsCore_InvalidViInt32Value(instrSession, measurementType), 2, "Result Type");
        }
        break;
    case RSSPECAN_VAL_MEASTYPE_MAX:
        switch (resultType)
        {
        case RSSPECAN_VAL_RESULT_PSWEEP_ACP:
            attr = RSSPECAN_ATTR_AMPLIFIER_PTABLE_ACP_MAXIMUM;
            break;
        case RSSPECAN_VAL_RESULT_PSWEEP_ACL1:
            attr = RSSPECAN_ATTR_AMPLIFIER_PTABLE_ACP_LOWER_MAXIMUM;
            break;
        case RSSPECAN_VAL_RESULT_PSWEEP_ACU1:
            attr = RSSPECAN_ATTR_AMPLIFIER_PTABLE_ACP_UPPER_MAXIMUM;
            break;
        case RSSPECAN_VAL_RESULT_PSWEEP_AMW:
            attr = RSSPECAN_ATTR_AMPLIFIER_PTABLE_AMAM_CURVE_WIDTH_MAXIMUM;
            break;
        case RSSPECAN_VAL_RESULT_PSWEEP_CFAC:
            attr = RSSPECAN_ATTR_AMPLIFIER_PTABLE_CREST_FACTOR_MAXIMUM;
            break;
        case RSSPECAN_VAL_RESULT_PSWEEP_EVM:
            attr = RSSPECAN_ATTR_AMPLIFIER_PTABLE_EVM_MAXIMUM;
            break;
        case RSSPECAN_VAL_RESULT_PSWEEP_GAIN:
            attr = RSSPECAN_ATTR_AMPLIFIER_PTABLE_GAIN_MAXIMUM;
            break;
        case RSSPECAN_VAL_RESULT_PSWEEP_ICC:
            attr = RSSPECAN_ATTR_AMPLIFIER_PTABLE_ICC_MAXIMUM;
            break;
        case RSSPECAN_VAL_RESULT_PSWEEP_VCC:
            attr = RSSPECAN_ATTR_AMPLIFIER_PTABLE_VCC_MAXIMUM;
            break;
        case RSSPECAN_VAL_RESULT_PSWEEP_POW:
            attr = RSSPECAN_ATTR_AMPLIFIER_PTABLE_BB_POWER_MAXIMUM;
            break;
        case RSSPECAN_VAL_RESULT_PSWEEP_PAE:
            attr = RSSPECAN_ATTR_AMPLIFIER_PTABLE_PAE_MAXIMUM;
            break;
        case RSSPECAN_VAL_RESULT_PSWEEP_PMW:
            attr = RSSPECAN_ATTR_AMPLIFIER_PTABLE_AMPM_CURVE_WIDTH_MAXIMUM;
            break;
        case RSSPECAN_VAL_RESULT_PSWEEP_RMS:
            attr = RSSPECAN_ATTR_AMPLIFIER_PTABLE_RMS_POWER_MAXIMUM;
            break;
        case RSSPECAN_VAL_RESULT_PSWEEP_ACBM:
            attr = RSSPECAN_ATTR_AMPLIFIER_PTABLE_ACP_BALANCED_MAXIMUM;
            break;
        default:
            viCheckParm(RsCore_InvalidViInt32Value(instrSession, measurementType), 2, "Result Type");
        }
        break;
    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, measurementType), 3, "Measurement Type");
    }
    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, "", attr, result),
            2, "Result");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function returns X or Y axis position of Parameters Swep Table results.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR resultType/This control selects the result type to be queried.
/// HIPAR measurementType/Selects measurement type.
/// HIPAR axis/Selects the axis for result location.
/// HIPAR result/Returns selected result.
ViStatus _VI_FUNC rsspecan_GetAmplifierParameterSweepTableResultsPosition(
                            ViSession instrSession,
                            ViInt32   resultType,
                            ViInt32   measurementType,
                            ViInt32   axis,
                            ViReal64* result)
{
    ViStatus error = VI_SUCCESS;
    ViAttr   attr = RSSPECAN_ATTR_AMPLIFIER_PTABLE_ACP_X_MINIMUM;

    checkErr(RsCore_LockSession(instrSession));

    switch (axis)
    {
    case RSSPECAN_VAL_AXIS_X:
        switch (measurementType)
        {
        case RSSPECAN_VAL_MEASTYPE_MIN:
            switch (resultType)
            {
            case RSSPECAN_VAL_RESULT_PSWEEP_ACP:
                attr = RSSPECAN_ATTR_AMPLIFIER_PTABLE_ACP_X_MINIMUM;
                break;
            case RSSPECAN_VAL_RESULT_PSWEEP_ACL1:
                attr = RSSPECAN_ATTR_AMPLIFIER_PTABLE_ACP_LOWER_X_MINIMUM;
                break;
            case RSSPECAN_VAL_RESULT_PSWEEP_ACU1:
                attr = RSSPECAN_ATTR_AMPLIFIER_PTABLE_ACP_UPPER_X_MINIMUM;
                break;
            case RSSPECAN_VAL_RESULT_PSWEEP_AMW:
                attr = RSSPECAN_ATTR_AMPLIFIER_PTABLE_AMAM_CURVE_WIDTH_X_MINIMUM;
                break;
            case RSSPECAN_VAL_RESULT_PSWEEP_CFAC:
                attr = RSSPECAN_ATTR_AMPLIFIER_PTABLE_CREST_FACTOR_X_MINIMUM;
                break;
            case RSSPECAN_VAL_RESULT_PSWEEP_EVM:
                attr = RSSPECAN_ATTR_AMPLIFIER_PTABLE_EVM_X_MINIMUM;
                break;
            case RSSPECAN_VAL_RESULT_PSWEEP_GAIN:
                attr = RSSPECAN_ATTR_AMPLIFIER_PTABLE_GAIN_X_MINIMUM;
                break;
            case RSSPECAN_VAL_RESULT_PSWEEP_ICC:
                attr = RSSPECAN_ATTR_AMPLIFIER_PTABLE_ICC_X_MINIMUM;
                break;
            case RSSPECAN_VAL_RESULT_PSWEEP_VCC:
                attr = RSSPECAN_ATTR_AMPLIFIER_PTABLE_VCC_X_MINIMUM;
                break;
            case RSSPECAN_VAL_RESULT_PSWEEP_POW:
                attr = RSSPECAN_ATTR_AMPLIFIER_PTABLE_BB_POWER_X_MINIMUM;
                break;
            case RSSPECAN_VAL_RESULT_PSWEEP_PAE:
                attr = RSSPECAN_ATTR_AMPLIFIER_PTABLE_PAE_X_MINIMUM;
                break;
            case RSSPECAN_VAL_RESULT_PSWEEP_PMW:
                attr = RSSPECAN_ATTR_AMPLIFIER_PTABLE_AMPM_CURVE_WIDTH_X_MINIMUM;
                break;
            case RSSPECAN_VAL_RESULT_PSWEEP_RMS:
                attr = RSSPECAN_ATTR_AMPLIFIER_PTABLE_RMS_POWER_X_MINIMUM;
                break;
            case RSSPECAN_VAL_RESULT_PSWEEP_ACBM:
                attr = RSSPECAN_ATTR_AMPLIFIER_PTABLE_ACP_BALANCED_X_MINIMUM;
                break;
            default:
                viCheckParm(RsCore_InvalidViInt32Value(instrSession, axis), 2, "Result Type");
            }
            break;
        case RSSPECAN_VAL_MEASTYPE_MAX:
            switch (resultType)
            {
            case RSSPECAN_VAL_RESULT_PSWEEP_ACP:
                attr = RSSPECAN_ATTR_AMPLIFIER_PTABLE_ACP_X_MAXIMUM;
                break;
            case RSSPECAN_VAL_RESULT_PSWEEP_ACL1:
                attr = RSSPECAN_ATTR_AMPLIFIER_PTABLE_ACP_LOWER_X_MAXIMUM;
                break;
            case RSSPECAN_VAL_RESULT_PSWEEP_ACU1:
                attr = RSSPECAN_ATTR_AMPLIFIER_PTABLE_ACP_UPPER_X_MAXIMUM;
                break;
            case RSSPECAN_VAL_RESULT_PSWEEP_AMW:
                attr = RSSPECAN_ATTR_AMPLIFIER_PTABLE_AMAM_CURVE_WIDTH_X_MAXIMUM;
                break;
            case RSSPECAN_VAL_RESULT_PSWEEP_CFAC:
                attr = RSSPECAN_ATTR_AMPLIFIER_PTABLE_CREST_FACTOR_X_MAXIMUM;
                break;
            case RSSPECAN_VAL_RESULT_PSWEEP_EVM:
                attr = RSSPECAN_ATTR_AMPLIFIER_PTABLE_EVM_X_MAXIMUM;
                break;
            case RSSPECAN_VAL_RESULT_PSWEEP_GAIN:
                attr = RSSPECAN_ATTR_AMPLIFIER_PTABLE_GAIN_X_MAXIMUM;
                break;
            case RSSPECAN_VAL_RESULT_PSWEEP_ICC:
                attr = RSSPECAN_ATTR_AMPLIFIER_PTABLE_ICC_X_MAXIMUM;
                break;
            case RSSPECAN_VAL_RESULT_PSWEEP_VCC:
                attr = RSSPECAN_ATTR_AMPLIFIER_PTABLE_VCC_X_MAXIMUM;
                break;
            case RSSPECAN_VAL_RESULT_PSWEEP_POW:
                attr = RSSPECAN_ATTR_AMPLIFIER_PTABLE_BB_POWER_X_MAXIMUM;
                break;
            case RSSPECAN_VAL_RESULT_PSWEEP_PAE:
                attr = RSSPECAN_ATTR_AMPLIFIER_PTABLE_PAE_X_MAXIMUM;
                break;
            case RSSPECAN_VAL_RESULT_PSWEEP_PMW:
                attr = RSSPECAN_ATTR_AMPLIFIER_PTABLE_AMPM_CURVE_WIDTH_X_MAXIMUM;
                break;
            case RSSPECAN_VAL_RESULT_PSWEEP_RMS:
                attr = RSSPECAN_ATTR_AMPLIFIER_PTABLE_RMS_POWER_X_MAXIMUM;
                break;
            case RSSPECAN_VAL_RESULT_PSWEEP_ACBM:
                attr = RSSPECAN_ATTR_AMPLIFIER_PTABLE_ACP_BALANCED_X_MAXIMUM;
                break;
            default:
                viCheckParm(RsCore_InvalidViInt32Value(instrSession, axis), 2, "Result Type");
            }
            break;
        default:
            viCheckParm(RsCore_InvalidViInt32Value(instrSession, axis), 3, "Measurement Type");
        }
        break;
    case RSSPECAN_VAL_AXIS_Y:
        switch (measurementType)
        {
        case RSSPECAN_VAL_MEASTYPE_MIN:
            switch (resultType)
            {
            case RSSPECAN_VAL_RESULT_PSWEEP_ACP:
                attr = RSSPECAN_ATTR_AMPLIFIER_PTABLE_ACP_Y_MINIMUM;
                break;
            case RSSPECAN_VAL_RESULT_PSWEEP_ACL1:
                attr = RSSPECAN_ATTR_AMPLIFIER_PTABLE_ACP_LOWER_Y_MINIMUM;
                break;
            case RSSPECAN_VAL_RESULT_PSWEEP_ACU1:
                attr = RSSPECAN_ATTR_AMPLIFIER_PTABLE_ACP_UPPER_Y_MINIMUM;
                break;
            case RSSPECAN_VAL_RESULT_PSWEEP_AMW:
                attr = RSSPECAN_ATTR_AMPLIFIER_PTABLE_AMAM_CURVE_WIDTH_Y_MINIMUM;
                break;
            case RSSPECAN_VAL_RESULT_PSWEEP_CFAC:
                attr = RSSPECAN_ATTR_AMPLIFIER_PTABLE_CREST_FACTOR_Y_MINIMUM;
                break;
            case RSSPECAN_VAL_RESULT_PSWEEP_EVM:
                attr = RSSPECAN_ATTR_AMPLIFIER_PTABLE_EVM_Y_MINIMUM;
                break;
            case RSSPECAN_VAL_RESULT_PSWEEP_GAIN:
                attr = RSSPECAN_ATTR_AMPLIFIER_PTABLE_GAIN_Y_MINIMUM;
                break;
            case RSSPECAN_VAL_RESULT_PSWEEP_ICC:
                attr = RSSPECAN_ATTR_AMPLIFIER_PTABLE_ICC_Y_MINIMUM;
                break;
            case RSSPECAN_VAL_RESULT_PSWEEP_VCC:
                attr = RSSPECAN_ATTR_AMPLIFIER_PTABLE_VCC_Y_MINIMUM;
                break;
            case RSSPECAN_VAL_RESULT_PSWEEP_POW:
                attr = RSSPECAN_ATTR_AMPLIFIER_PTABLE_BB_POWER_Y_MINIMUM;
                break;
            case RSSPECAN_VAL_RESULT_PSWEEP_PAE:
                attr = RSSPECAN_ATTR_AMPLIFIER_PTABLE_PAE_Y_MINIMUM;
                break;
            case RSSPECAN_VAL_RESULT_PSWEEP_PMW:
                attr = RSSPECAN_ATTR_AMPLIFIER_PTABLE_AMPM_CURVE_WIDTH_Y_MINIMUM;
                break;
            case RSSPECAN_VAL_RESULT_PSWEEP_RMS:
                attr = RSSPECAN_ATTR_AMPLIFIER_PTABLE_RMS_POWER_Y_MINIMUM;
                break;
            case RSSPECAN_VAL_RESULT_PSWEEP_ACBM:
                attr = RSSPECAN_ATTR_AMPLIFIER_PTABLE_ACP_BALANCED_Y_MINIMUM;
                break;
            default:
                viCheckParm(RsCore_InvalidViInt32Value(instrSession, axis), 2, "Result Type");
            }
            break;
        case RSSPECAN_VAL_MEASTYPE_MAX:
            switch (resultType)
            {
            case RSSPECAN_VAL_RESULT_PSWEEP_ACP:
                attr = RSSPECAN_ATTR_AMPLIFIER_PTABLE_ACP_Y_MAXIMUM;
                break;
            case RSSPECAN_VAL_RESULT_PSWEEP_ACL1:
                attr = RSSPECAN_ATTR_AMPLIFIER_PTABLE_ACP_LOWER_Y_MAXIMUM;
                break;
            case RSSPECAN_VAL_RESULT_PSWEEP_ACU1:
                attr = RSSPECAN_ATTR_AMPLIFIER_PTABLE_ACP_UPPER_Y_MAXIMUM;
                break;
            case RSSPECAN_VAL_RESULT_PSWEEP_AMW:
                attr = RSSPECAN_ATTR_AMPLIFIER_PTABLE_AMAM_CURVE_WIDTH_Y_MAXIMUM;
                break;
            case RSSPECAN_VAL_RESULT_PSWEEP_CFAC:
                attr = RSSPECAN_ATTR_AMPLIFIER_PTABLE_CREST_FACTOR_Y_MAXIMUM;
                break;
            case RSSPECAN_VAL_RESULT_PSWEEP_EVM:
                attr = RSSPECAN_ATTR_AMPLIFIER_PTABLE_EVM_Y_MAXIMUM;
                break;
            case RSSPECAN_VAL_RESULT_PSWEEP_GAIN:
                attr = RSSPECAN_ATTR_AMPLIFIER_PTABLE_GAIN_Y_MAXIMUM;
                break;
            case RSSPECAN_VAL_RESULT_PSWEEP_ICC:
                attr = RSSPECAN_ATTR_AMPLIFIER_PTABLE_ICC_Y_MAXIMUM;
                break;
            case RSSPECAN_VAL_RESULT_PSWEEP_VCC:
                attr = RSSPECAN_ATTR_AMPLIFIER_PTABLE_VCC_Y_MAXIMUM;
                break;
            case RSSPECAN_VAL_RESULT_PSWEEP_POW:
                attr = RSSPECAN_ATTR_AMPLIFIER_PTABLE_BB_POWER_Y_MAXIMUM;
                break;
            case RSSPECAN_VAL_RESULT_PSWEEP_PAE:
                attr = RSSPECAN_ATTR_AMPLIFIER_PTABLE_PAE_Y_MAXIMUM;
                break;
            case RSSPECAN_VAL_RESULT_PSWEEP_PMW:
                attr = RSSPECAN_ATTR_AMPLIFIER_PTABLE_AMPM_CURVE_WIDTH_Y_MAXIMUM;
                break;
            case RSSPECAN_VAL_RESULT_PSWEEP_RMS:
                attr = RSSPECAN_ATTR_AMPLIFIER_PTABLE_RMS_POWER_Y_MAXIMUM;
                break;
            case RSSPECAN_VAL_RESULT_PSWEEP_ACBM:
                attr = RSSPECAN_ATTR_AMPLIFIER_PTABLE_ACP_BALANCED_Y_MAXIMUM;
                break;
            default:
                viCheckParm(RsCore_InvalidViInt32Value(instrSession, axis), 2, "Result Type");
            }
            break;
        default:
            viCheckParm(RsCore_InvalidViInt32Value(instrSession, axis), 3, "Measurement Type");
        }
        break;
    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, axis), 4, "Axis");
    }
    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, "", attr, result),
            2, "Result");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the sweep table results for compression points.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR measurementType/Selects measurement type.
/// HIPAR axis/Selects the axis for result location.
/// HIPAR result/Returns selected result.
ViStatus _VI_FUNC rsspecan_GetAmplifierParameterTablePowerOutputResults(
                            ViSession instrSession,
                            ViInt32   measurementType,
                            ViInt32   axis,
                            ViReal64* result)
{
    ViStatus error = VI_SUCCESS;
    ViAttr   maximumArr[3] = {
             RSSPECAN_ATTR_AMPLIFIER_POWER_OUTPUT_MAXIMUM_X,
             RSSPECAN_ATTR_AMPLIFIER_POWER_OUTPUT_MAXIMUM_Y,
             RSSPECAN_ATTR_AMPLIFIER_PTABLE_POWER_OUTPUT_MAXIMUM
             };
    ViAttr   minimumArr[3] = {
             RSSPECAN_ATTR_AMPLIFIER_POWER_OUTPUT_MINIMUM_X,
             RSSPECAN_ATTR_AMPLIFIER_POWER_OUTPUT_MINIMUM_Y,
             RSSPECAN_ATTR_AMPLIFIER_PTABLE_POWER_OUTPUT_MINIMUM
             };

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, measurementType, 1, 2),
            2, "Measurement Type");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, axis, 0, 2),
            3, "Axis");

    switch (measurementType)
    {
    case RSSPECAN_VAL_PULSE_MEASTYPE_MIN:
        viCheckParm(rsspecan_GetAttributeViReal64(instrSession, "", minimumArr[axis], result),
                4, "Result");
        break;
    case RSSPECAN_VAL_PULSE_MEASTYPE_MAX:
        viCheckParm(rsspecan_GetAttributeViReal64(instrSession, "", maximumArr[axis], result),
                4, "Result");
        break;
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the minimum or maximum for selected Compression Point as shown
/// HIFN  in the Result Summary.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR compressionPoint/Selects the compression point for query.
/// HIPAR power/Queries the Compression Point as shown in the Result Summary.
/// HIPAR measurementType/Selects measurement type.
ViStatus _VI_FUNC rsspecan_GetAmplifierCompressionPointsPower(
                            ViSession instrSession,
                            ViInt32   compressionPoint,
                            ViInt32   measurementType,
                            ViReal64* power)
{
    ViStatus error = VI_SUCCESS;
    ViAttr   attr = 0;

    checkErr(RsCore_LockSession(instrSession));

    switch (compressionPoint)
    {
    case RSSPECAN_VAL_COMPRESSION_POINT_1DB:
        switch (measurementType)
        {
        case RSSPECAN_VAL_MEASTYPE_MIN:
            attr = RSSPECAN_ATTR_AMPLIFIER_COMPRESSION_POINT_1_DB_MINIMUM_POWER;
            break;
        case RSSPECAN_VAL_MEASTYPE_MAX:
            attr = RSSPECAN_ATTR_AMPLIFIER_COMPRESSION_POINT_1_DB_MAXIMUM_POWER;
            break;
        default:
            checkErr(RsCore_InvalidViInt32Value(instrSession, measurementType));
            break;
        }
        break;
    case RSSPECAN_VAL_COMPRESSION_POINT_2DB:
        switch (measurementType)
        {
        case RSSPECAN_VAL_MEASTYPE_MIN:
            attr = RSSPECAN_ATTR_AMPLIFIER_COMPRESSION_POINT_2_DB_MINIMUM_POWER;
            break;
        case RSSPECAN_VAL_MEASTYPE_MAX:
            attr = RSSPECAN_ATTR_AMPLIFIER_COMPRESSION_POINT_2_DB_MAXIMUM_POWER;
            break;
        default:
            checkErr(RsCore_InvalidViInt32Value(instrSession, measurementType));
            break;
        }
        break;
    case RSSPECAN_VAL_COMPRESSION_POINT_3DB:
        switch (measurementType)
        {
        case RSSPECAN_VAL_MEASTYPE_MIN:
            attr = RSSPECAN_ATTR_AMPLIFIER_COMPRESSION_POINT_3_DB_MINIMUM_POWER;
            break;
        case RSSPECAN_VAL_MEASTYPE_MAX:
            attr = RSSPECAN_ATTR_AMPLIFIER_COMPRESSION_POINT_3_DB_MAXIMUM_POWER;
            break;
        default:
            checkErr(RsCore_InvalidViInt32Value(instrSession, measurementType));
            break;
        }
        break;
    default:
        checkErr(RsCore_InvalidViInt32Value(instrSession, compressionPoint));
        break;
    }

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, NULL, attr, power),
            4, "Power");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the minimum or maximum for selected Compression Point as shown
/// HIFN  in the Result Summary.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR compressionPoint/Selects the compression point for query.
/// HIPAR power/Queries the Compression Point as shown in the Result Summary.
/// HIPAR measurementType/Selects measurement type.
ViStatus _VI_FUNC rsspecan_GetAmplifierCompressionPointsOutputPower(
                            ViSession instrSession,
                            ViInt32   compressionPoint,
                            ViInt32   measurementType,
                            ViReal64* power)
{
    ViStatus error = VI_SUCCESS;
    ViAttr   attr = 0;

    checkErr(RsCore_LockSession(instrSession));

    switch (compressionPoint)
    {
    case RSSPECAN_VAL_COMPRESSION_POINT_1DB:
        switch (measurementType)
        {
        case RSSPECAN_VAL_MEASTYPE_CURR:
            attr = RSSPECAN_ATTR_AMPLIFIER_COMPRESSION_POINT_1_DB_OUTPUT_CURRENT;
            break;
        case RSSPECAN_VAL_MEASTYPE_MIN:
            attr = RSSPECAN_ATTR_AMPLIFIER_COMPRESSION_POINT_1_DB_OUTPUT_MINIMUM_POWER;
            break;
        case RSSPECAN_VAL_MEASTYPE_MAX:
            attr = RSSPECAN_ATTR_AMPLIFIER_COMPRESSION_POINT_1_DB_OUTPUT_MAXIMUM_POWER;
            break;
        default:
            checkErr(RsCore_InvalidViInt32Value(instrSession, measurementType));
            break;
        }
        break;
    case RSSPECAN_VAL_COMPRESSION_POINT_2DB:
        switch (measurementType)
        {
        case RSSPECAN_VAL_MEASTYPE_CURR:
            attr = RSSPECAN_ATTR_AMPLIFIER_COMPRESSION_POINT_2_DB_OUTPUT_CURRENT;
            break;
        case RSSPECAN_VAL_MEASTYPE_MIN:
            attr = RSSPECAN_ATTR_AMPLIFIER_COMPRESSION_POINT_2_DB_OUTPUT_MINIMUM_POWER;
            break;
        case RSSPECAN_VAL_MEASTYPE_MAX:
            attr = RSSPECAN_ATTR_AMPLIFIER_COMPRESSION_POINT_2_DB_OUTPUT_MAXIMUM_POWER;
            break;
        default:
            checkErr(RsCore_InvalidViInt32Value(instrSession, measurementType));
            break;
        }
        break;
    case RSSPECAN_VAL_COMPRESSION_POINT_3DB:
        switch (measurementType)
        {
        case RSSPECAN_VAL_MEASTYPE_CURR:
            attr = RSSPECAN_ATTR_AMPLIFIER_COMPRESSION_POINT_3_DB_OUTPUT_CURRENT;
            break;
        case RSSPECAN_VAL_MEASTYPE_MIN:
            attr = RSSPECAN_ATTR_AMPLIFIER_COMPRESSION_POINT_3_DB_OUTPUT_MINIMUM_POWER;
            break;
        case RSSPECAN_VAL_MEASTYPE_MAX:
            attr = RSSPECAN_ATTR_AMPLIFIER_COMPRESSION_POINT_3_DB_OUTPUT_MAXIMUM_POWER;
            break;
        default:
            checkErr(RsCore_InvalidViInt32Value(instrSession, measurementType));
            break;
        }
        break;
    default:
        checkErr(RsCore_InvalidViInt32Value(instrSession, compressionPoint));
        break;
    }

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, NULL, attr, power),
            4, "Power");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the trigger to sync result in seconds.
/// HIFN     This is the time from start of capture (i.e. including pre-trigger
/// HIFN  samples) to the start of the sync range, which is not necessarily the
/// HIFN  beginning of the reference waveform.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR syncResult/Queries the trigger to sync result in seconds.
ViStatus _VI_FUNC rsspecan_GetAmplifierTriggerToSyncResult(ViSession instrSession,
                                                           ViReal64* syncResult)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, NULL, RSSPECAN_ATTR_AMPLIFIER_TRIGGER_TO_SYNC, syncResult),
            2, "Sync Result");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Fetches the MDPD coefficient values.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR bufferSize/Size of output array.
/// HIPAR dpdCoefficients/Fetches the MDPD coefficient values.
ViStatus _VI_FUNC rsspecan_GetAmplifierMemoryDPDCoefficients(ViSession instrSession,
                                                             ViInt32   bufferSize,
                                                             ViChar    dpdCoefficients[])
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_GetAttributeViString(instrSession, NULL, RSSPECAN_ATTR_AMPLIFIER_MEMORY_DPD_COEFFICIENTS,
            bufferSize, dpdCoefficients),
            3, "DPD Coefficients");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the sweep table results for compression points.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR compressionPoint/This control selects the result type to be queried.
/// HIPAR measurementType/Selects measurement type.
/// HIPAR axis/Selects the axis for result location.
/// HIPAR result/Returns selected result.
ViStatus _VI_FUNC rsspecan_GetAmplifierCompressionPointSweepTableResults(
                            ViSession instrSession,
                            ViInt32   compressionPoint,
                            ViInt32   measurementType,
                            ViInt32   axis,
                            ViReal64* result)
{
    ViStatus error = VI_SUCCESS;
    ViAttr   maximumArr[3][3] = {
             {
             RSSPECAN_ATTR_AMPLIFIER_COMPRESSION_POINT_1_DB_MAXIMUM_X,
             RSSPECAN_ATTR_AMPLIFIER_COMPRESSION_POINT_1_DB_MAXIMUM_Y,
             RSSPECAN_ATTR_AMPLIFIER_COMPRESSION_POINT_1_DB_MAXIMUM
             },
             {
             RSSPECAN_ATTR_AMPLIFIER_COMPRESSION_POINT_2_DB_MAXIMUM_X,
             RSSPECAN_ATTR_AMPLIFIER_COMPRESSION_POINT_2_DB_MAXIMUM_Y,
             RSSPECAN_ATTR_AMPLIFIER_COMPRESSION_POINT_2_DB_MAXIMUM
             },
             {
             RSSPECAN_ATTR_AMPLIFIER_COMPRESSION_POINT_3_DB_MAXIMUM_X,
             RSSPECAN_ATTR_AMPLIFIER_COMPRESSION_POINT_3_DB_MAXIMUM_Y,
             RSSPECAN_ATTR_AMPLIFIER_COMPRESSION_POINT_3_DB_MAXIMUM
             }
             };
    ViAttr   minimumArr[3][3] = {
             {
             RSSPECAN_ATTR_AMPLIFIER_COMPRESSION_POINT_1_DB_MINIMUM_X,
             RSSPECAN_ATTR_AMPLIFIER_COMPRESSION_POINT_1_DB_MINIMUM_Y,
             RSSPECAN_ATTR_AMPLIFIER_COMPRESSION_POINT_1_DB_MINIMUM
             },
             {
             RSSPECAN_ATTR_AMPLIFIER_COMPRESSION_POINT_2_DB_MINIMUM_X,
             RSSPECAN_ATTR_AMPLIFIER_COMPRESSION_POINT_2_DB_MINIMUM_Y,
             RSSPECAN_ATTR_AMPLIFIER_COMPRESSION_POINT_2_DB_MINIMUM
             },
             {
             RSSPECAN_ATTR_AMPLIFIER_COMPRESSION_POINT_3_DB_MINIMUM_X,
             RSSPECAN_ATTR_AMPLIFIER_COMPRESSION_POINT_3_DB_MINIMUM_Y,
             RSSPECAN_ATTR_AMPLIFIER_COMPRESSION_POINT_3_DB_MINIMUM
             }
             };

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, compressionPoint, 0, 2),
            2, "Compression Point");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, measurementType, 1, 2),
            3, "Measurement Type");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, axis, 0, 2),
            4, "Axis");

    switch (measurementType)
    {
    case RSSPECAN_VAL_PULSE_MEASTYPE_MIN:
        viCheckParm(rsspecan_GetAttributeViReal64(instrSession, "", minimumArr[compressionPoint][axis], result),
                5, "Result");
        break;
    case RSSPECAN_VAL_PULSE_MEASTYPE_MAX:
        viCheckParm(rsspecan_GetAttributeViReal64(instrSession, "", maximumArr[compressionPoint][axis], result),
                5, "Result");
        break;
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN This function reutrns general numeric results.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR resultType/This control selects the result type to be queried.
/// HIPAR measurementType/Selects measurement type.
/// HIPAR result/Returns selected result.
ViStatus _VI_FUNC rsspecan_GetAmplifierGeneralNumericResults(ViSession instrSession,
                                                             ViInt32   resultType,
                                                             ViInt32   measurementType,
                                                             ViReal64* result)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    switch (measurementType)
    {
    case RSSPECAN_VAL_MEASTYPE_CURR:
        switch (resultType)
        {
        case RSSPECAN_VAL_RESULT_TRIGGER_TO_FRAME:
            viCheckParm(rsspecan_GetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_AMPLIFIER_TRIGGERR_TO_FRAME, result),
                    2, "Result");
            break;
        default:
            viCheckParm(RsCore_InvalidViInt32Value(instrSession, measurementType), 2, "Result Type");
        }
        break;
    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, measurementType), 3, "Measurement Type");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the statistical Amplitude Droop value of all result ranges
/// HIFN  found in the entire measurement (corresponding to black area of
/// HIFN  statistics table).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR measurementType/Selects measurement type.
/// HIPAR result/Queries the statistical Amplitude Droop value of all result ranges
/// HIPAR result/found in the entire measurement (corresponding to black area of
/// HIPAR result/statistics table).
ViStatus _VI_FUNC rsspecan_GetAmplifierStatisticsTableAmplitudeDroopAll(
                            ViSession instrSession,
                            ViInt32   measurementType,
                            ViReal64* result)
{
    ViStatus error = VI_SUCCESS;
    ViAttr   attr = 0;

    checkErr(RsCore_LockSession(instrSession));

    switch (measurementType)
    {
    case RSSPECAN_VAL_MEASTYPE_MIN:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_AMPLITUDE_DROOP_MINIMUM_ALL;
        break;
    case RSSPECAN_VAL_MEASTYPE_MAX:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_AMPLITUDE_DROOP_MAXIMUM_ALL;
        break;
    case RSSPECAN_VAL_MEASTYPE_SDEV:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_AMPLITUDE_DROOP_STANDARD_DEVIATION_ALL;
        break;
    case RSSPECAN_VAL_MEASTYPE_AVER:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_AMPLITUDE_DROOP_AVERAGE_ALL;
        break;
    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, measurementType), 2, "Measurement Type");
        break;
    }

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, NULL, attr, result),
            3, "Result");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the statistical Amplitude Droop value of all result ranges
/// HIFN  found in the current capture (corresponding to green area of statistics
/// HIFN  table).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR measurementType/Selects measurement type.
/// HIPAR result/Queries the statistical Amplitude Droop value of all result ranges
/// HIPAR result/found in the current capture (corresponding to green area of statistics
/// HIPAR result/table).
ViStatus _VI_FUNC rsspecan_GetAmplifierStatisticsTableAmplitudeDroopCurrent(
                            ViSession instrSession,
                            ViInt32   measurementType,
                            ViReal64* result)
{
    ViStatus error = VI_SUCCESS;
    ViAttr   attr = 0;

    checkErr(RsCore_LockSession(instrSession));

    switch (measurementType)
    {
    case RSSPECAN_VAL_MEASTYPE_MIN:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_AMPLITUDE_DROOP_MIMIMUM_CURRENT;
        break;
    case RSSPECAN_VAL_MEASTYPE_MAX:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_AMPLITUDE_DROOP_MAXIMUM_CURRENT;
        break;
    case RSSPECAN_VAL_MEASTYPE_SDEV:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_AMPLITUDE_DROOP_STANDARD_DEVIATION_ALL;
        break;
    case RSSPECAN_VAL_MEASTYPE_AVER:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_AMPLITUDE_DROOP_AVERAGE_CURRENT;
        break;
    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, measurementType), 2, "Measurement Type");
        break;
    }

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, NULL, attr, result),
            3, "Result");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the Amplitude Droop result of the currently selected result
/// HIFN  range (corresponds to blue area of statistics table).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR result/Queries the Amplitude Droop result of the currently selected result
/// HIPAR result/range (corresponds to blue area of statistics table).
ViStatus _VI_FUNC rsspecan_GetAmplifierStatisticsTableAmplitudeDroopSelectedResult(
                            ViSession instrSession,
                            ViReal64* result)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, NULL, RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_AMPLITUDE_DROOP_SELECTED_RESULT, result),
            2, "Result");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the statistical AM/AM Curve Width value of all result ranges
/// HIFN  found in the entire measurement (corresponding to black area of
/// HIFN  statistics table).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR measurementType/Selects measurement type.
/// HIPAR result/Queries the statistical AM/AM Curve Width value of all result ranges
/// HIPAR result/found in the entire measurement (corresponding to black area of
/// HIPAR result/statistics table).
ViStatus _VI_FUNC rsspecan_GetAmplifierStatisticsTableAMAMCurveWidthAll(
                            ViSession instrSession,
                            ViInt32   measurementType,
                            ViReal64* result)
{
    ViStatus error = VI_SUCCESS;
    ViAttr   attr = 0;

    checkErr(RsCore_LockSession(instrSession));

    switch (measurementType)
    {
    case RSSPECAN_VAL_MEASTYPE_MIN:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_AM_AM_CURVE_WIDTH_MINIMUM_ALL;
        break;
    case RSSPECAN_VAL_MEASTYPE_MAX:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_AM_AM_CURVE_WIDTH_MAXIMUM_ALL;
        break;
    case RSSPECAN_VAL_MEASTYPE_SDEV:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_AM_AM_CURVE_WIDTH_STANDARD_DEVIATION_ALL;
        break;
    case RSSPECAN_VAL_MEASTYPE_AVER:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_AM_AM_CURVE_WIDTH_AVERAGE_ALL;
        break;
    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, measurementType), 2, "Measurement Type");
        break;
    }

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, NULL, attr, result),
            3, "Result");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the statistical AM/AM Curve Width value of all result ranges
/// HIFN  found in the current capture (corresponding to green area of statistics
/// HIFN  table).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR measurementType/Selects measurement type.
/// HIPAR result/Queries the statistical AM/AM Curve Width value of all result ranges
/// HIPAR result/found in the current capture (corresponding to green area of statistics
/// HIPAR result/table).
ViStatus _VI_FUNC rsspecan_GetAmplifierStatisticsTableAMAMCurveWidthCurrent(
                            ViSession instrSession,
                            ViInt32   measurementType,
                            ViReal64* result)
{
    ViStatus error = VI_SUCCESS;
    ViAttr   attr = 0;

    checkErr(RsCore_LockSession(instrSession));

    switch (measurementType)
    {
    case RSSPECAN_VAL_MEASTYPE_MIN:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_AM_AM_CURVE_WIDTH_MINIMUM_CURRENT;
        break;
    case RSSPECAN_VAL_MEASTYPE_MAX:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_AM_AM_CURVE_WIDTH_MAXIMUM_CURRENT;
        break;
    case RSSPECAN_VAL_MEASTYPE_SDEV:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_AM_AM_CURVE_WIDTH_STANDARD_DEVIATION_CURRENT;
        break;
    case RSSPECAN_VAL_MEASTYPE_AVER:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_AM_AM_CURVE_WIDTH_AVERAGE_CURRENT;
        break;
    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, measurementType), 2, "Measurement Type");
        break;
    }

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, NULL, attr, result),
            3, "Result");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the AM/AM Curve Width result of the currently selected result
/// HIFN  range (corresponds to blue area of statistics table).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR result/Queries the AM/AM Curve Width result of the currently selected result
/// HIPAR result/range (corresponds to blue area of statistics table).
ViStatus _VI_FUNC rsspecan_GetAmplifierStatisticsTableAMAMCurveWidthSelectedResult(
                            ViSession instrSession,
                            ViReal64* result)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, NULL, RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_AM_AM_CURVE_WIDTH_SELECTED_RESULT, result),
            2, "Result");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the statistical AM/PM Curve Width value of all result ranges
/// HIFN  found in the entire measurement (corresponding to black area of
/// HIFN  statistics table).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR measurementType/Selects measurement type.
/// HIPAR result/Queries the statistical AM/PM Curve Width value of all result ranges
/// HIPAR result/found in the entire measurement (corresponding to black area of
/// HIPAR result/statistics table).
ViStatus _VI_FUNC rsspecan_GetAmplifierStatisticsTableAMPMCurveWidthAll(
                            ViSession instrSession,
                            ViInt32   measurementType,
                            ViReal64* result)
{
    ViStatus error = VI_SUCCESS;
    ViAttr   attr = 0;

    checkErr(RsCore_LockSession(instrSession));

    switch (measurementType)
    {
    case RSSPECAN_VAL_MEASTYPE_MIN:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_AM_PM_CURVE_WIDTH_MINIMUM_ALL;
        break;
    case RSSPECAN_VAL_MEASTYPE_MAX:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_AM_PM_CURVE_WIDTH_MAXIMUM_ALL;
        break;
    case RSSPECAN_VAL_MEASTYPE_SDEV:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_AM_PM_CURVE_WIDTH_STANDARD_DEVIATION_ALL;
        break;
    case RSSPECAN_VAL_MEASTYPE_AVER:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_AM_PM_CURVE_WIDTH_AVERAGE_ALL;
        break;
    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, measurementType), 2, "Measurement Type");
        break;
    }

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, NULL, attr, result),
            3, "Result");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the statistical AM/PM Curve Width value of all result ranges
/// HIFN  found in the current capture (corresponding to green area of statistics
/// HIFN  table).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR measurementType/Selects measurement type.
/// HIPAR result/Queries the statistical AM/PM Curve Width value of all result ranges
/// HIPAR result/found in the current capture (corresponding to green area of statistics
/// HIPAR result/table).
ViStatus _VI_FUNC rsspecan_GetAmplifierStatisticsTableAMPMCurveWidthCurrent(
                            ViSession instrSession,
                            ViInt32   measurementType,
                            ViReal64* result)
{
    ViStatus error = VI_SUCCESS;
    ViAttr   attr = 0;

    checkErr(RsCore_LockSession(instrSession));

    switch (measurementType)
    {
    case RSSPECAN_VAL_MEASTYPE_MIN:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_AM_PM_CURVE_WIDTH_MINIMUM_CURRENT;
        break;
    case RSSPECAN_VAL_MEASTYPE_MAX:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_AM_PM_CURVE_WIDTH_MAXIMUM_CURRENT;
        break;
    case RSSPECAN_VAL_MEASTYPE_SDEV:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_AM_PM_CURVE_WIDTH_STANDARD_DEVIATION_CURRENT;
        break;
    case RSSPECAN_VAL_MEASTYPE_AVER:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_AM_PM_CURVE_WIDTH_AVERAGE_CURRENT;
        break;
    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, measurementType), 2, "Measurement Type");
        break;
    }

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, NULL, attr, result),
            3, "Result");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the AM/PM Curve Width result of the currently selected result
/// HIFN  range (corresponds to blue area of statistics table).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR result/Queries the AM/PM Curve Width result of the currently selected result
/// HIPAR result/range (corresponds to blue area of statistics table).
ViStatus _VI_FUNC rsspecan_GetAmplifierStatisticsTableAMPMCurveWidthSelectedResult(
                            ViSession instrSession,
                            ViReal64* result)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, NULL, RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_AM_PM_CURVE_WIDTH_SELECTED_RESULT, result),
            2, "Result");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the statistical Average PAE value of all result ranges found
/// HIFN  in the entire measurement (corresponding to black area of statistics
/// HIFN  table).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR measurementType/Selects measurement type.
/// HIPAR result/Queries the statistical Average PAE value of all result ranges found
/// HIPAR result/in the entire measurement (corresponding to black area of statistics
/// HIPAR result/table).
ViStatus _VI_FUNC rsspecan_GetAmplifierStatisticsTableAveragePAEAverageAll(
                            ViSession instrSession,
                            ViInt32   measurementType,
                            ViReal64* result)
{
    ViStatus error = VI_SUCCESS;
    ViAttr   attr = 0;

    checkErr(RsCore_LockSession(instrSession));

    switch (measurementType)
    {
    case RSSPECAN_VAL_MEASTYPE_MIN:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_AVERAGE_PAE_AVERAGE_MINIMUM_ALL;
        break;
    case RSSPECAN_VAL_MEASTYPE_MAX:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_AVERAGE_PAE_AVERAGE_MAXIMUM_ALL;
        break;
    case RSSPECAN_VAL_MEASTYPE_SDEV:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_AVERAGE_PAE_AVERAGE_STANDARD_DEVIATION_ALL;
        break;
    case RSSPECAN_VAL_MEASTYPE_AVER:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_AVERAGE_PAE_AVERAGE_AVERAGE_ALL;
        break;
    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, measurementType), 2, "Measurement Type");
        break;
    }

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, NULL, attr, result),
            3, "Result");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the statistical Average PAE value of all result ranges found
/// HIFN  in the current capture (corresponding to green area of statistics
/// HIFN  table).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR measurementType/Selects measurement type.
/// HIPAR result/Queries the statistical Average PAE value of all result ranges found
/// HIPAR result/in the current capture (corresponding to green area of statistics
/// HIPAR result/table).
ViStatus _VI_FUNC rsspecan_GetAmplifierStatisticsTableAveragePAEAverageCurrent(
                            ViSession instrSession,
                            ViInt32   measurementType,
                            ViReal64* result)
{
    ViStatus error = VI_SUCCESS;
    ViAttr   attr = 0;

    checkErr(RsCore_LockSession(instrSession));

    switch (measurementType)
    {
    case RSSPECAN_VAL_MEASTYPE_MIN:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_AVERAGE_PAE_AVERAGE_MINIMUM_CURRENT;
        break;
    case RSSPECAN_VAL_MEASTYPE_MAX:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_AVERAGE_PAE_AVERAGE_MAXIMUM_CURRENT;
        break;
    case RSSPECAN_VAL_MEASTYPE_SDEV:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_AVERAGE_PAE_AVERAGE_STANDARD_DEVIATION_CURRENT;
        break;
    case RSSPECAN_VAL_MEASTYPE_AVER:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_AVERAGE_PAE_AVERAGE_AVERAGE_CURRENT;
        break;
    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, measurementType), 2, "Measurement Type");
        break;
    }

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, NULL, attr, result),
            3, "Result");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the Average PAE result of the currently selected result range
/// HIFN  (corresponds to blue area of statistics table).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR result/Queries the Average PAE result of the currently selected result range
/// HIPAR result/(corresponds to blue area of statistics table).
ViStatus _VI_FUNC rsspecan_GetAmplifierStatisticsTableAveragePAEAverageSelectedResult(
                            ViSession instrSession,
                            ViReal64* result)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, NULL, RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_AVERAGE_PAE_AVERAGE_SELECTED_RESULT, result),
            2, "Result");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the statistical Average PAE value of all result ranges found
/// HIFN  in the entire measurement (corresponding to black area of statistics
/// HIFN  table).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR measurementType/Selects measurement type.
/// HIPAR result/Queries the statistical Average PAE value of all result ranges found
/// HIPAR result/in the entire measurement (corresponding to black area of statistics
/// HIPAR result/table).
ViStatus _VI_FUNC rsspecan_GetAmplifierStatisticsTableAveragePAEMaximumAll(
                            ViSession instrSession,
                            ViInt32   measurementType,
                            ViReal64* result)
{
    ViStatus error = VI_SUCCESS;
    ViAttr   attr = 0;

    checkErr(RsCore_LockSession(instrSession));

    switch (measurementType)
    {
    case RSSPECAN_VAL_MEASTYPE_MIN:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_AVERAGE_PAE_MAXIMUM_MINIMUM_ALL;
        break;
    case RSSPECAN_VAL_MEASTYPE_MAX:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_AVERAGE_PAE_MAXIMUM_MAXIMUM_ALL;
        break;
    case RSSPECAN_VAL_MEASTYPE_SDEV:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_AVERAGE_PAE_MAXIMUM_STANDARD_DEVIATION_ALL;
        break;
    case RSSPECAN_VAL_MEASTYPE_AVER:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_AVERAGE_PAE_MAXIMUM_AVERAGE_ALL;
        break;
    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, measurementType), 2, "Measurement Type");
        break;
    }

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, NULL, attr, result),
            3, "Result");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the statistical Average PAE value of all result ranges found
/// HIFN  in the current capture (corresponding to green area of statistics
/// HIFN  table).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR measurementType/Selects measurement type.
/// HIPAR result/Queries the statistical Average PAE value of all result ranges found
/// HIPAR result/in the current capture (corresponding to green area of statistics
/// HIPAR result/table).
ViStatus _VI_FUNC rsspecan_GetAmplifierStatisticsTableAveragePAEMaximumCurrent(
                            ViSession instrSession,
                            ViInt32   measurementType,
                            ViReal64* result)
{
    ViStatus error = VI_SUCCESS;
    ViAttr   attr = 0;

    checkErr(RsCore_LockSession(instrSession));

    switch (measurementType)
    {
    case RSSPECAN_VAL_MEASTYPE_MIN:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_AVERAGE_PAE_MAXIMUM_MINIMUM_CURRENT;
        break;
    case RSSPECAN_VAL_MEASTYPE_MAX:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_AVERAGE_PAE_MAXIMUM_MAXIMUM_CURRENT;
        break;
    case RSSPECAN_VAL_MEASTYPE_SDEV:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_AVERAGE_PAE_MAXIMUM_STANDARD_DEVIATION_CURRENT;
        break;
    case RSSPECAN_VAL_MEASTYPE_AVER:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_AVERAGE_PAE_MAXIMUM_AVERAGE_CURRENT;
        break;
    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, measurementType), 2, "Measurement Type");
        break;
    }

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, NULL, attr, result),
            3, "Result");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the Average PAE result of the currently selected result range
/// HIFN  (corresponds to blue area of statistics table).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR result/Queries the Average PAE result of the currently selected result range
/// HIPAR result/(corresponds to blue area of statistics table).
ViStatus _VI_FUNC rsspecan_GetAmplifierStatisticsTableAveragePAEMaximumSelectedResult(
                            ViSession instrSession,
                            ViReal64* result)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, NULL, RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_AVERAGE_PAE_MAXIMUM_SELECTED_RESULT, result),
            2, "Result");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the statistical Average PAE value of all result ranges found
/// HIFN  in the entire measurement (corresponding to black area of statistics
/// HIFN  table).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR measurementType/Selects measurement type.
/// HIPAR result/Queries the statistical Average PAE value of all result ranges found
/// HIPAR result/in the entire measurement (corresponding to black area of statistics
/// HIPAR result/table).
ViStatus _VI_FUNC rsspecan_GetAmplifierStatisticsTableAveragePAEMinimumAll(
                            ViSession instrSession,
                            ViInt32   measurementType,
                            ViReal64* result)
{
    ViStatus error = VI_SUCCESS;
    ViAttr   attr = 0;

    checkErr(RsCore_LockSession(instrSession));

    switch (measurementType)
    {
    case RSSPECAN_VAL_MEASTYPE_MIN:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_AVERAGE_PAE_MINIMUM_MINIMUM_ALL;
        break;
    case RSSPECAN_VAL_MEASTYPE_MAX:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_AVERAGE_PAE_MINIMUM_MAXIMUM_ALL;
        break;
    case RSSPECAN_VAL_MEASTYPE_SDEV:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_AVERAGE_PAE_MINIMUM_STANDARD_DEVIATION_ALL;
        break;
    case RSSPECAN_VAL_MEASTYPE_AVER:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_AVERAGE_PAE_MINIMUM_AVERAGE_ALL;
        break;
    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, measurementType), 2, "Measurement Type");
        break;
    }

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, NULL, attr, result),
            3, "Result");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the statistical Average PAE value of all result ranges found
/// HIFN  in the current capture (corresponding to green area of statistics
/// HIFN  table).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR measurementType/Selects measurement type.
/// HIPAR result/Queries the statistical Average PAE value of all result ranges found
/// HIPAR result/in the current capture (corresponding to green area of statistics
/// HIPAR result/table).
ViStatus _VI_FUNC rsspecan_GetAmplifierStatisticsTableAveragePAEMinimumCurrent(
                            ViSession instrSession,
                            ViInt32   measurementType,
                            ViReal64* result)
{
    ViStatus error = VI_SUCCESS;
    ViAttr   attr = 0;

    checkErr(RsCore_LockSession(instrSession));

    switch (measurementType)
    {
    case RSSPECAN_VAL_MEASTYPE_MIN:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_AVERAGE_PAE_MINIMUM_MINIMUM_CURRENT;
        break;
    case RSSPECAN_VAL_MEASTYPE_MAX:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_AVERAGE_PAE_MINIMUM_MAXIMUM_CURRENT;
        break;
    case RSSPECAN_VAL_MEASTYPE_SDEV:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_AVERAGE_PAE_MINIMUM_STANDARD_DEVIATION_CURRENT;
        break;
    case RSSPECAN_VAL_MEASTYPE_AVER:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_AVERAGE_PAE_MINIMUM_AVERAGE_CURRENT;
        break;
    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, measurementType), 2, "Measurement Type");
        break;
    }

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, NULL, attr, result),
            3, "Result");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the Average PAE result of the currently selected result range
/// HIFN  (corresponds to blue area of statistics table).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR result/Queries the Average PAE result of the currently selected result range
/// HIPAR result/(corresponds to blue area of statistics table).
ViStatus _VI_FUNC rsspecan_GetAmplifierStatisticsTableAveragePAEMinimumSelectedResult(
                            ViSession instrSession,
                            ViReal64* result)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, NULL, RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_AVERAGE_PAE_MINIMUM_SELECTED_RESULT, result),
            2, "Result");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the statistical Power (Vcc*Icc) value of all result ranges
/// HIFN  found in the entire measurement (corresponding to black area of
/// HIFN  statistics table).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR measurementType/Selects measurement type.
/// HIPAR result/Queries the statistical Power (Vcc*Icc) value of all result ranges
/// HIPAR result/found in the entire measurement (corresponding to black area of
/// HIPAR result/statistics table).
ViStatus _VI_FUNC rsspecan_GetAmplifierStatisticsTablePowerVccxIccAverageAll(
                            ViSession instrSession,
                            ViInt32   measurementType,
                            ViReal64* result)
{
    ViStatus error = VI_SUCCESS;
    ViAttr   attr = 0;

    checkErr(RsCore_LockSession(instrSession));

    switch (measurementType)
    {
    case RSSPECAN_VAL_MEASTYPE_MIN:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_POWER_VCC_X_ICC_AVERAGE_MINIMUM_ALL;
        break;
    case RSSPECAN_VAL_MEASTYPE_MAX:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_POWER_VCC_X_ICC_AVERAGE_MAXIMUM_ALL;
        break;
    case RSSPECAN_VAL_MEASTYPE_SDEV:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_POWER_VCC_X_ICC_AVERAGE_STANDARD_DEVIATION_ALL;
        break;
    case RSSPECAN_VAL_MEASTYPE_AVER:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_POWER_VCC_X_ICC_AVERAGE_AVERAGE_ALL;
        break;
    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, measurementType), 2, "Measurement Type");
        break;
    }

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, NULL, attr, result),
            3, "Result");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the statistical Power (Vcc*Icc) value of all result ranges
/// HIFN  found in the entire measurement (corresponding to black area of
/// HIFN  statistics table).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR measurementType/Selects measurement type.
/// HIPAR result/Queries the statistical Power (Vcc*Icc) value of all result ranges
/// HIPAR result/found in the entire measurement (corresponding to black area of
/// HIPAR result/statistics table).
ViStatus _VI_FUNC rsspecan_GetAmplifierStatisticsTablePowerVccxIccAverageCurrent(
                            ViSession instrSession,
                            ViInt32   measurementType,
                            ViReal64* result)
{
    ViStatus error = VI_SUCCESS;
    ViAttr   attr = 0;

    checkErr(RsCore_LockSession(instrSession));

    switch (measurementType)
    {
    case RSSPECAN_VAL_MEASTYPE_MIN:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_POWER_VCC_X_ICC_AVERAGE_MINIMUM_CURRENT;
        break;
    case RSSPECAN_VAL_MEASTYPE_MAX:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_POWER_VCC_X_ICC_AVERAGE_MAXIMUM_CURRENT;
        break;
    case RSSPECAN_VAL_MEASTYPE_SDEV:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_POWER_VCC_X_ICC_AVERAGE_STANDARD_DEVIATION_CURRENT;
        break;
    case RSSPECAN_VAL_MEASTYPE_AVER:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_POWER_VCC_X_ICC_AVERAGE_AVERAGE_CURRENT;
        break;
    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, measurementType), 2, "Measurement Type");
        break;
    }

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, NULL, attr, result),
            3, "Result");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the Power (Vcc*Icc) result of the currently selected result
/// HIFN  range (corresponds to blue area of statistics table).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR result/Queries the Power (Vcc*Icc) result of the currently selected result
/// HIPAR result/range (corresponds to blue area of statistics table).
ViStatus _VI_FUNC rsspecan_GetAmplifierStatisticsTablePowerVccxIccAverageSelectedResult(
                            ViSession instrSession,
                            ViReal64* result)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, NULL, RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_POWER_VCC_X_ICC_AVERAGE_SELECTED_RESULT, result),
            2, "Result");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the statistical Power (Vcc*Icc) value of all result ranges
/// HIFN  found in the entire measurement (corresponding to black area of
/// HIFN  statistics table).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR measurementType/Selects measurement type.
/// HIPAR result/Queries the statistical Power (Vcc*Icc) value of all result ranges
/// HIPAR result/found in the entire measurement (corresponding to black area of
/// HIPAR result/statistics table).
ViStatus _VI_FUNC rsspecan_GetAmplifierStatisticsTablePowerVccxIccMaximumAll(
                            ViSession instrSession,
                            ViInt32   measurementType,
                            ViReal64* result)
{
    ViStatus error = VI_SUCCESS;
    ViAttr   attr = 0;

    checkErr(RsCore_LockSession(instrSession));

    switch (measurementType)
    {
    case RSSPECAN_VAL_MEASTYPE_MIN:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_POWER_VCC_X_ICC_MAXIMUM_MINIMUM_ALL;
        break;
    case RSSPECAN_VAL_MEASTYPE_MAX:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_POWER_VCC_X_ICC_MAXIMUM_MAXIMUM_ALL;
        break;
    case RSSPECAN_VAL_MEASTYPE_SDEV:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_POWER_VCC_X_ICC_MAXIMUM_STANDARD_DEVIATION_ALL;
        break;
    case RSSPECAN_VAL_MEASTYPE_AVER:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_POWER_VCC_X_ICC_MAXIMUM_AVERAGE_ALL;
        break;
    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, measurementType), 2, "Measurement Type");
        break;
    }

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, NULL, attr, result),
            3, "Result");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the statistical Power (Vcc*Icc) value of all result ranges
/// HIFN  found in the entire measurement (corresponding to black area of
/// HIFN  statistics table).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR measurementType/Selects measurement type.
/// HIPAR result/Queries the statistical Power (Vcc*Icc) value of all result ranges
/// HIPAR result/found in the entire measurement (corresponding to black area of
/// HIPAR result/statistics table).
ViStatus _VI_FUNC rsspecan_GetAmplifierStatisticsTablePowerVccxIccMaximumCurrent(
                            ViSession instrSession,
                            ViInt32   measurementType,
                            ViReal64* result)
{
    ViStatus error = VI_SUCCESS;
    ViAttr   attr = 0;

    checkErr(RsCore_LockSession(instrSession));

    switch (measurementType)
    {
    case RSSPECAN_VAL_MEASTYPE_MIN:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_POWER_VCC_X_ICC_MAXIMUM_MINIMUM_CURRENT;
        break;
    case RSSPECAN_VAL_MEASTYPE_MAX:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_POWER_VCC_X_ICC_MAXIMUM_MAXIMUM_CURRENT;
        break;
    case RSSPECAN_VAL_MEASTYPE_SDEV:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_POWER_VCC_X_ICC_MAXIMUM_STANDARD_DEVIATION_CURRENT;
        break;
    case RSSPECAN_VAL_MEASTYPE_AVER:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_POWER_VCC_X_ICC_MAXIMUM_AVERAGE_CURRENT;
        break;
    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, measurementType), 2, "Measurement Type");
        break;
    }

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, NULL, attr, result),
            3, "Result");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the Power (Vcc*Icc) result of the currently selected result
/// HIFN  range (corresponds to blue area of statistics table).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR result/Queries the Power (Vcc*Icc) result of the currently selected result
/// HIPAR result/range (corresponds to blue area of statistics table).
ViStatus _VI_FUNC rsspecan_GetAmplifierStatisticsTablePowerVccxIccMaximumSelectedResult(
                            ViSession instrSession,
                            ViReal64* result)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, NULL, RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_POWER_VCC_X_ICC_MAXIMUM_SELECTED_RESULT, result),
            2, "Result");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the statistical Power (Vcc*Icc) value of all result ranges
/// HIFN  found in the entire measurement (corresponding to black area of
/// HIFN  statistics table).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR measurementType/Selects measurement type.
/// HIPAR result/Queries the statistical Power (Vcc*Icc) value of all result ranges
/// HIPAR result/found in the entire measurement (corresponding to black area of
/// HIPAR result/statistics table).
ViStatus _VI_FUNC rsspecan_GetAmplifierStatisticsTablePowerVccxIccMinimumAll(
                            ViSession instrSession,
                            ViInt32   measurementType,
                            ViReal64* result)
{
    ViStatus error = VI_SUCCESS;
    ViAttr   attr = 0;

    checkErr(RsCore_LockSession(instrSession));

    switch (measurementType)
    {
    case RSSPECAN_VAL_MEASTYPE_MIN:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_POWER_VCC_X_ICC_MINIMUM_MINIMUM_ALL;
        break;
    case RSSPECAN_VAL_MEASTYPE_MAX:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_POWER_VCC_X_ICC_MINIMUM_MAXIMUM_ALL;
        break;
    case RSSPECAN_VAL_MEASTYPE_SDEV:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_POWER_VCC_X_ICC_MINIMUM_STANDARD_DEVIATION_ALL;
        break;
    case RSSPECAN_VAL_MEASTYPE_AVER:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_POWER_VCC_X_ICC_MINIMUM_AVERAGE_ALL;
        break;
    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, measurementType), 2, "Measurement Type");
        break;
    }

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, NULL, attr, result),
            3, "Result");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the statistical Power (Vcc*Icc) value of all result ranges
/// HIFN  found in the entire measurement (corresponding to black area of
/// HIFN  statistics table).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR measurementType/Selects measurement type.
/// HIPAR result/Queries the statistical Power (Vcc*Icc) value of all result ranges
/// HIPAR result/found in the entire measurement (corresponding to black area of
/// HIPAR result/statistics table).
ViStatus _VI_FUNC rsspecan_GetAmplifierStatisticsTablePowerVccxIccMinimumCurrent(
                            ViSession instrSession,
                            ViInt32   measurementType,
                            ViReal64* result)
{
    ViStatus error = VI_SUCCESS;
    ViAttr   attr = 0;

    checkErr(RsCore_LockSession(instrSession));

    switch (measurementType)
    {
    case RSSPECAN_VAL_MEASTYPE_MIN:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_POWER_VCC_X_ICC_MINIMUM_MINIMUM_CURRENT;
        break;
    case RSSPECAN_VAL_MEASTYPE_MAX:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_POWER_VCC_X_ICC_MINIMUM_MAXIMUM_CURRENT;
        break;
    case RSSPECAN_VAL_MEASTYPE_SDEV:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_POWER_VCC_X_ICC_MINIMUM_STANDARD_DEVIATION_CURRENT;
        break;
    case RSSPECAN_VAL_MEASTYPE_AVER:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_POWER_VCC_X_ICC_MINIMUM_AVERAGE_CURRENT;
        break;
    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, measurementType), 2, "Measurement Type");
        break;
    }

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, NULL, attr, result),
            3, "Result");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the Power (Vcc*Icc) result of the currently selected result
/// HIFN  range (corresponds to blue area of statistics table).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR result/Queries the Power (Vcc*Icc) result of the currently selected result
/// HIPAR result/range (corresponds to blue area of statistics table).
ViStatus _VI_FUNC rsspecan_GetAmplifierStatisticsTablePowerVccxIccMinimumSelectedResult(
                            ViSession instrSession,
                            ViReal64* result)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, NULL, RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_POWER_VCC_X_ICC_MINIMUM_SELECTED_RESULT, result),
            2, "Result");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the statistical Crest Factor Input value of all result ranges
/// HIFN  found in the entire measurement (corresponding to black area of
/// HIFN  statistics table).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR measurementType/Selects measurement type.
/// HIPAR result/Queries the statistical Crest Factor Input value of all result ranges
/// HIPAR result/found in the entire measurement (corresponding to black area of
/// HIPAR result/statistics table).
ViStatus _VI_FUNC rsspecan_GetAmplifierStatisticsTableCrestFactorInputAll(
                            ViSession instrSession,
                            ViInt32   measurementType,
                            ViReal64* result)
{
    ViStatus error = VI_SUCCESS;
    ViAttr   attr = 0;

    checkErr(RsCore_LockSession(instrSession));

    switch (measurementType)
    {
    case RSSPECAN_VAL_MEASTYPE_MIN:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_CREST_FACTOR_INPUT_MINIMUM_ALL;
        break;
    case RSSPECAN_VAL_MEASTYPE_MAX:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_CREST_FACTOR_INPUT_MAXIMUM_ALL;
        break;
    case RSSPECAN_VAL_MEASTYPE_SDEV:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_CREST_FACTOR_INPUT_STANDARD_DEVIATION_ALL;
        break;
    case RSSPECAN_VAL_MEASTYPE_AVER:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_CREST_FACTOR_INPUT_AVERAGE_ALL;
        break;
    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, measurementType), 2, "Measurement Type");
        break;
    }

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, NULL, attr, result),
            3, "Result");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the statistical Crest Factor Input value of all result ranges
/// HIFN  found in the entire measurement (corresponding to black area of
/// HIFN  statistics table).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR measurementType/Selects measurement type.
/// HIPAR result/Queries the statistical Crest Factor Input value of all result ranges
/// HIPAR result/found in the entire measurement (corresponding to black area of
/// HIPAR result/statistics table).
ViStatus _VI_FUNC rsspecan_GetAmplifierStatisticsTableCrestFactorInputCurrent(
                            ViSession instrSession,
                            ViInt32   measurementType,
                            ViReal64* result)
{
    ViStatus error = VI_SUCCESS;
    ViAttr   attr = 0;

    checkErr(RsCore_LockSession(instrSession));

    switch (measurementType)
    {
    case RSSPECAN_VAL_MEASTYPE_MIN:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_CREST_FACTOR_INPUT_MINIMUM_CURRENT;
        break;
    case RSSPECAN_VAL_MEASTYPE_MAX:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_CREST_FACTOR_INPUT_MAXIMUM_CURRENT;
        break;
    case RSSPECAN_VAL_MEASTYPE_SDEV:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_CREST_FACTOR_INPUT_STANDARD_DEVIATION_CURRENT;
        break;
    case RSSPECAN_VAL_MEASTYPE_AVER:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_CREST_FACTOR_INPUT_AVERAGE_CURRENT;
        break;
    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, measurementType), 2, "Measurement Type");
        break;
    }

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, NULL, attr, result),
            3, "Result");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the Crest Factor Input result of the currently selected result
/// HIFN  range (corresponds to blue area of statistics table).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR result/Queries the Crest Factor Input result of the currently selected result
/// HIPAR result/range (corresponds to blue area of statistics table).
ViStatus _VI_FUNC rsspecan_GetAmplifierStatisticsTableCrestFactorInputSelectedResult(
                            ViSession instrSession,
                            ViReal64* result)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, NULL, RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_CREST_FACTOR_INPUT_SELECTED_RESULT, result),
            2, "Result");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the statistical Crest Factor Input value of all result ranges
/// HIFN  found in the entire measurement (corresponding to black area of
/// HIFN  statistics table).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR measurementType/Selects measurement type.
/// HIPAR result/Queries the statistical Crest Factor Input value of all result ranges
/// HIPAR result/found in the entire measurement (corresponding to black area of
/// HIPAR result/statistics table).
ViStatus _VI_FUNC rsspecan_GetAmplifierStatisticsTableCrestFactorOutputAll(
                            ViSession instrSession,
                            ViInt32   measurementType,
                            ViReal64* result)
{
    ViStatus error = VI_SUCCESS;
    ViAttr   attr = 0;

    checkErr(RsCore_LockSession(instrSession));

    switch (measurementType)
    {
    case RSSPECAN_VAL_MEASTYPE_MIN:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_CREST_FACTOR_OUTPUT_MINIMUM_ALL;
        break;
    case RSSPECAN_VAL_MEASTYPE_MAX:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_CREST_FACTOR_OUTPUT_MAXIMUM_ALL;
        break;
    case RSSPECAN_VAL_MEASTYPE_SDEV:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_CREST_FACTOR_OUTPUT_STANDARD_DEVIATION_ALL;
        break;
    case RSSPECAN_VAL_MEASTYPE_AVER:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_CREST_FACTOR_OUTPUT_AVERAGE_ALL;
        break;
    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, measurementType), 2, "Measurement Type");
        break;
    }

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, NULL, attr, result),
            3, "Result");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the statistical Crest Factor Input value of all result ranges
/// HIFN  found in the entire measurement (corresponding to black area of
/// HIFN  statistics table).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR measurementType/Selects measurement type.
/// HIPAR result/Queries the statistical Crest Factor Input value of all result ranges
/// HIPAR result/found in the entire measurement (corresponding to black area of
/// HIPAR result/statistics table).
ViStatus _VI_FUNC rsspecan_GetAmplifierStatisticsTableCrestFactorOutputCurrent(
                            ViSession instrSession,
                            ViInt32   measurementType,
                            ViReal64* result)
{
    ViStatus error = VI_SUCCESS;
    ViAttr   attr = 0;

    checkErr(RsCore_LockSession(instrSession));

    switch (measurementType)
    {
    case RSSPECAN_VAL_MEASTYPE_MIN:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_CREST_FACTOR_OUTPUT_MINIMUM_CURRENT;
        break;
    case RSSPECAN_VAL_MEASTYPE_MAX:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_CREST_FACTOR_OUTPUT_MAXIMUM_CURRENT;
        break;
    case RSSPECAN_VAL_MEASTYPE_SDEV:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_CREST_FACTOR_OUTPUT_STANDARD_DEVIATION_CURRENT;
        break;
    case RSSPECAN_VAL_MEASTYPE_AVER:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_CREST_FACTOR_OUTPUT_AVERAGE_CURRENT;
        break;
    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, measurementType), 2, "Measurement Type");
        break;
    }

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, NULL, attr, result),
            3, "Result");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the Crest Factor Out result of the currently selected result
/// HIFN  range (corresponds to blue area of statistics table).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR result/Queries the Crest Factor Out result of the currently selected result
/// HIPAR result/range (corresponds to blue area of statistics table).
ViStatus _VI_FUNC rsspecan_GetAmplifierStatisticsTableCrestFactorOutputSelectedResult(
                            ViSession instrSession,
                            ViReal64* result)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, NULL, RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_CREST_FACTOR_OUTPUT_SELECTED_RESULT, result),
            2, "Result");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the statistical Frequency Error value of all result ranges
/// HIFN  found in the entire measurement (corresponding to black area of
/// HIFN  statistics table).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR measurementType/Selects measurement type.
/// HIPAR result/Queries the statistical Frequency Error value of all result ranges
/// HIPAR result/found in the entire measurement (corresponding to black area of
/// HIPAR result/statistics table).
ViStatus _VI_FUNC rsspecan_GetAmplifierStatisticsTableFrequencyErrorAll(
                            ViSession instrSession,
                            ViInt32   measurementType,
                            ViReal64* result)
{
    ViStatus error = VI_SUCCESS;
    ViAttr   attr = 0;

    checkErr(RsCore_LockSession(instrSession));

    switch (measurementType)
    {
    case RSSPECAN_VAL_MEASTYPE_MIN:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_FREQUENCY_ERROR_MINIMUM_ALL;
        break;
    case RSSPECAN_VAL_MEASTYPE_MAX:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_FREQUENCY_ERROR_MAXIMUM_ALL;
        break;
    case RSSPECAN_VAL_MEASTYPE_SDEV:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_FREQUENCY_ERROR_STANDARD_DEVIATION_ALL;
        break;
    case RSSPECAN_VAL_MEASTYPE_AVER:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_FREQUENCY_ERROR_AVERAGE_ALL;
        break;
    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, measurementType), 2, "Measurement Type");
        break;
    }

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, NULL, attr, result),
            3, "Result");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the statistical Frequency Error value of all result ranges
/// HIFN  found in the entire measurement (corresponding to black area of
/// HIFN  statistics table).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR measurementType/Selects measurement type.
/// HIPAR result/Queries the statistical Frequency Error value of all result ranges
/// HIPAR result/found in the entire measurement (corresponding to black area of
/// HIPAR result/statistics table).
ViStatus _VI_FUNC rsspecan_GetAmplifierStatisticsTableFrequencyErrorCurrent(
                            ViSession instrSession,
                            ViInt32   measurementType,
                            ViReal64* result)
{
    ViStatus error = VI_SUCCESS;
    ViAttr   attr = 0;

    checkErr(RsCore_LockSession(instrSession));

    switch (measurementType)
    {
    case RSSPECAN_VAL_MEASTYPE_MIN:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_FREQUENCY_ERROR_MINIMUM_CURRENT;
        break;
    case RSSPECAN_VAL_MEASTYPE_MAX:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_FREQUENCY_ERROR_MAXIMUM_CURRENT;
        break;
    case RSSPECAN_VAL_MEASTYPE_SDEV:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_FREQUENCY_ERROR_STANDARD_DEVIATION_CURRENT;
        break;
    case RSSPECAN_VAL_MEASTYPE_AVER:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_FREQUENCY_ERROR_AVERAGE_CURRENT;
        break;
    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, measurementType), 2, "Measurement Type");
        break;
    }

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, NULL, attr, result),
            3, "Result");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the Frequency Error result of the currently selected result
/// HIFN  range (corresponds to blue area of statistics table).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR result/Queries the Frequency Error result of the currently selected result
/// HIPAR result/range (corresponds to blue area of statistics table).
ViStatus _VI_FUNC rsspecan_GetAmplifierStatisticsTableFrequencyErrorSelectedResult(
                            ViSession instrSession,
                            ViReal64* result)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, NULL, RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_FREQUENCY_ERROR_SELECTED_RESULT, result),
            2, "Result");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the statistical Gain value of all result ranges found in the
/// HIFN  entire measurement (corresponding to black area of statistics table).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR measurementType/Selects measurement type.
/// HIPAR result/Queries the statistical Gain value of all result ranges found in the
/// HIPAR result/entire measurement (corresponding to black area of statistics table).
ViStatus _VI_FUNC rsspecan_GetAmplifierStatisticsTableGainAll(
                            ViSession instrSession,
                            ViInt32   measurementType,
                            ViReal64* result)
{
    ViStatus error = VI_SUCCESS;
    ViAttr   attr = 0;

    checkErr(RsCore_LockSession(instrSession));

    switch (measurementType)
    {
    case RSSPECAN_VAL_MEASTYPE_MIN:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_GAIN_MINIMUM_ALL;
        break;
    case RSSPECAN_VAL_MEASTYPE_MAX:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_GAIN_MAXIMUM_ALL;
        break;
    case RSSPECAN_VAL_MEASTYPE_SDEV:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_GAIN_STANDARD_DEVIATION_ALL;
        break;
    case RSSPECAN_VAL_MEASTYPE_AVER:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_GAIN_AVERAGE_ALL;
        break;
    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, measurementType), 2, "Measurement Type");
        break;
    }

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, NULL, attr, result),
            3, "Result");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the statistical Gain value of all result ranges found in the
/// HIFN  entire measurement (corresponding to black area of statistics table).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR measurementType/Selects measurement type.
/// HIPAR result/Queries the statistical Gain value of all result ranges found in the
/// HIPAR result/entire measurement (corresponding to black area of statistics table).
ViStatus _VI_FUNC rsspecan_GetAmplifierStatisticsTableGainCurrent(
                            ViSession instrSession,
                            ViInt32   measurementType,
                            ViReal64* result)
{
    ViStatus error = VI_SUCCESS;
    ViAttr   attr = 0;

    checkErr(RsCore_LockSession(instrSession));

    switch (measurementType)
    {
    case RSSPECAN_VAL_MEASTYPE_MIN:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_GAIN_MINIMUM_CURRENT;
        break;
    case RSSPECAN_VAL_MEASTYPE_MAX:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_GAIN_MAXIMUM_CURRENT;
        break;
    case RSSPECAN_VAL_MEASTYPE_SDEV:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_GAIN_STANDARD_DEVIATION_CURRENT;
        break;
    case RSSPECAN_VAL_MEASTYPE_AVER:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_GAIN_AVERAGE_CURRENT;
        break;
    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, measurementType), 2, "Measurement Type");
        break;
    }

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, NULL, attr, result),
            3, "Result");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the Gain result of the currently selected result range
/// HIFN  (corresponds to blue area of statistics table).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR result/Queries the Gain result of the currently selected result range
/// HIPAR result/(corresponds to blue area of statistics table).
ViStatus _VI_FUNC rsspecan_GetAmplifierStatisticsTableGainSelectedResult(
                            ViSession instrSession,
                            ViReal64* result)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, NULL, RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_GAIN_SELECTED_RESULT, result),
            2, "Result");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the statistical Gain Imbalance value of all result ranges
/// HIFN  found in the entire measurement (corresponding to black area of
/// HIFN  statistics table).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR measurementType/Selects measurement type.
/// HIPAR result/Queries the statistical Gain Imbalance value of all result ranges
/// HIPAR result/found in the entire measurement (corresponding to black area of
/// HIPAR result/statistics table).
ViStatus _VI_FUNC rsspecan_GetAmplifierStatisticsTableGainImbalanceAll(
                            ViSession instrSession,
                            ViInt32   measurementType,
                            ViReal64* result)
{
    ViStatus error = VI_SUCCESS;
    ViAttr   attr = 0;

    checkErr(RsCore_LockSession(instrSession));

    switch (measurementType)
    {
    case RSSPECAN_VAL_MEASTYPE_MIN:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_GAIN_IMBALANCE_MINIMUM_ALL;
        break;
    case RSSPECAN_VAL_MEASTYPE_MAX:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_GAIN_IMBALANCE_MAXIMUM_ALL;
        break;
    case RSSPECAN_VAL_MEASTYPE_SDEV:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_GAIN_IMBALANCE_STANDARD_DEVIATION_ALL;
        break;
    case RSSPECAN_VAL_MEASTYPE_AVER:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_GAIN_IMBALANCE_AVERAGE_ALL;
        break;
    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, measurementType), 2, "Measurement Type");
        break;
    }

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, NULL, attr, result),
            3, "Result");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the statistical Gain Imbalance value of all result ranges
/// HIFN  found in the entire measurement (corresponding to black area of
/// HIFN  statistics table).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR measurementType/Selects measurement type.
/// HIPAR result/Queries the statistical Gain Imbalance value of all result ranges
/// HIPAR result/found in the entire measurement (corresponding to black area of
/// HIPAR result/statistics table).
ViStatus _VI_FUNC rsspecan_GetAmplifierStatisticsTableGainImbalanceCurrent(
                            ViSession instrSession,
                            ViInt32   measurementType,
                            ViReal64* result)
{
    ViStatus error = VI_SUCCESS;
    ViAttr   attr = 0;

    checkErr(RsCore_LockSession(instrSession));

    switch (measurementType)
    {
    case RSSPECAN_VAL_MEASTYPE_MIN:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_GAIN_IMBALANCE_MINIMUM_CURRENT;
        break;
    case RSSPECAN_VAL_MEASTYPE_MAX:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_GAIN_IMBALANCE_MAXIMUM_CURRENT;
        break;
    case RSSPECAN_VAL_MEASTYPE_SDEV:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_GAIN_IMBALANCE_STANDARD_DEVIATION_CURRENT;
        break;
    case RSSPECAN_VAL_MEASTYPE_AVER:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_GAIN_IMBALANCE_AVERAGE_CURRENT;
        break;
    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, measurementType), 2, "Measurement Type");
        break;
    }

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, NULL, attr, result),
            3, "Result");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the Gain Imbalance result of the currently selected result
/// HIFN  range (corresponds to blue area of statistics table).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR result/Queries the Gain Imbalance result of the currently selected result
/// HIPAR result/range (corresponds to blue area of statistics table).
ViStatus _VI_FUNC rsspecan_GetAmplifierStatisticsTableGainImbalanceSelectedResult(
                            ViSession instrSession,
                            ViReal64* result)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, NULL, RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_GAIN_IMBALANCE_SELECTED_RESULT, result),
            2, "Result");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the statistical Current (Icc) value of all result ranges found
/// HIFN  in the entire measurement (corresponding to black area of statistics
/// HIFN  table).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR measurementType/Selects measurement type.
/// HIPAR result/Queries the statistical Current (Icc) value of all result ranges found
/// HIPAR result/in the entire measurement (corresponding to black area of statistics
/// HIPAR result/table).
ViStatus _VI_FUNC rsspecan_GetAmplifierStatisticsTableCurrentIccAverageAll(
                            ViSession instrSession,
                            ViInt32   measurementType,
                            ViReal64* result)
{
    ViStatus error = VI_SUCCESS;
    ViAttr   attr = 0;

    checkErr(RsCore_LockSession(instrSession));

    switch (measurementType)
    {
    case RSSPECAN_VAL_MEASTYPE_MIN:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_CURRENT_ICC_AVERAGE_MINIMUM_ALL;
        break;
    case RSSPECAN_VAL_MEASTYPE_MAX:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_CURRENT_ICC_AVERAGE_MAXIMUM_ALL;
        break;
    case RSSPECAN_VAL_MEASTYPE_SDEV:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_CURRENT_ICC_AVERAGE_STANDARD_DEVIATION_ALL;
        break;
    case RSSPECAN_VAL_MEASTYPE_AVER:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_CURRENT_ICC_AVERAGE_AVERAGE_ALL;
        break;
    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, measurementType), 2, "Measurement Type");
        break;
    }

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, NULL, attr, result),
            3, "Result");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the statistical Current (Icc) value of all result ranges found
/// HIFN  in the entire measurement (corresponding to black area of statistics
/// HIFN  table).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR measurementType/Selects measurement type.
/// HIPAR result/Queries the statistical Current (Icc) value of all result ranges found
/// HIPAR result/in the entire measurement (corresponding to black area of statistics
/// HIPAR result/table).
ViStatus _VI_FUNC rsspecan_GetAmplifierStatisticsTableCurrentIccAverageCurrent(
                            ViSession instrSession,
                            ViInt32   measurementType,
                            ViReal64* result)
{
    ViStatus error = VI_SUCCESS;
    ViAttr   attr = 0;

    checkErr(RsCore_LockSession(instrSession));

    switch (measurementType)
    {
    case RSSPECAN_VAL_MEASTYPE_MIN:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_CURRENT_ICC_AVERAGE_MINIMUM_CURRENT;
        break;
    case RSSPECAN_VAL_MEASTYPE_MAX:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_CURRENT_ICC_AVERAGE_MAXIMUM_CURRENT;
        break;
    case RSSPECAN_VAL_MEASTYPE_SDEV:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_CURRENT_ICC_AVERAGE_STANDARD_DEVIATION_CURRENT;
        break;
    case RSSPECAN_VAL_MEASTYPE_AVER:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_CURRENT_ICC_AVERAGE_AVERAGE_CURRENT;
        break;
    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, measurementType), 2, "Measurement Type");
        break;
    }

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, NULL, attr, result),
            3, "Result");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the Current (Icc) result of the currently selected result
/// HIFN  range (corresponds to blue area of statistics table).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR result/Queries the Current (Icc) result of the currently selected result
/// HIPAR result/range (corresponds to blue area of statistics table).
ViStatus _VI_FUNC rsspecan_GetAmplifierStatisticsTableCurrentIccAverageSelectedResult(
                            ViSession instrSession,
                            ViReal64* result)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, NULL, RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_CURRENT_ICC_AVERAGE_SELECTED_RESULT, result),
            2, "Result");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the statistical Current (Icc) value of all result ranges found
/// HIFN  in the entire measurement (corresponding to black area of statistics
/// HIFN  table).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR measurementType/Selects measurement type.
/// HIPAR result/Queries the statistical Current (Icc) value of all result ranges found
/// HIPAR result/in the entire measurement (corresponding to black area of statistics
/// HIPAR result/table).
ViStatus _VI_FUNC rsspecan_GetAmplifierStatisticsTableCurrentIccMaximumAll(
                            ViSession instrSession,
                            ViInt32   measurementType,
                            ViReal64* result)
{
    ViStatus error = VI_SUCCESS;
    ViAttr   attr = 0;

    checkErr(RsCore_LockSession(instrSession));

    switch (measurementType)
    {
    case RSSPECAN_VAL_MEASTYPE_MIN:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_CURRENT_ICC_MAXIMUM_MINIMUM_ALL;
        break;
    case RSSPECAN_VAL_MEASTYPE_MAX:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_CURRENT_ICC_MAXIMUM_MAXIMUM_ALL;
        break;
    case RSSPECAN_VAL_MEASTYPE_SDEV:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_CURRENT_ICC_MAXIMUM_STANDARD_DEVIATION_ALL;
        break;
    case RSSPECAN_VAL_MEASTYPE_AVER:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_CURRENT_ICC_MAXIMUM_AVERAGE_ALL;
        break;
    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, measurementType), 2, "Measurement Type");
        break;
    }

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, NULL, attr, result),
            3, "Result");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the statistical Current (Icc) value of all result ranges found
/// HIFN  in the entire measurement (corresponding to black area of statistics
/// HIFN  table).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR measurementType/Selects measurement type.
/// HIPAR result/Queries the statistical Current (Icc) value of all result ranges found
/// HIPAR result/in the entire measurement (corresponding to black area of statistics
/// HIPAR result/table).
ViStatus _VI_FUNC rsspecan_GetAmplifierStatisticsTableCurrentIccMaximumCurrent(
                            ViSession instrSession,
                            ViInt32   measurementType,
                            ViReal64* result)
{
    ViStatus error = VI_SUCCESS;
    ViAttr   attr = 0;

    checkErr(RsCore_LockSession(instrSession));

    switch (measurementType)
    {
    case RSSPECAN_VAL_MEASTYPE_MIN:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_CURRENT_ICC_MAXIMUM_MINIMUM_CURRENT;
        break;
    case RSSPECAN_VAL_MEASTYPE_MAX:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_CURRENT_ICC_MAXIMUM_MAXIMUM_CURRENT;
        break;
    case RSSPECAN_VAL_MEASTYPE_SDEV:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_CURRENT_ICC_MAXIMUM_STANDARD_DEVIATION_CURRENT;
        break;
    case RSSPECAN_VAL_MEASTYPE_AVER:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_CURRENT_ICC_MAXIMUM_AVERAGE_CURRENT;
        break;
    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, measurementType), 2, "Measurement Type");
        break;
    }

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, NULL, attr, result),
            3, "Result");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the Current (Icc) result of the currently selected result
/// HIFN  range (corresponds to blue area of statistics table).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR result/Queries the Current (Icc) result of the currently selected result
/// HIPAR result/range (corresponds to blue area of statistics table).
ViStatus _VI_FUNC rsspecan_GetAmplifierStatisticsTableCurrentIccMaximumSelectedResult(
                            ViSession instrSession,
                            ViReal64* result)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, NULL, RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_CURRENT_ICC_MAXIMUM_SELECTED_RESULT, result),
            2, "Result");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the statistical Current (Icc) value of all result ranges found
/// HIFN  in the entire measurement (corresponding to black area of statistics
/// HIFN  table).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR measurementType/Selects measurement type.
/// HIPAR result/Queries the statistical Current (Icc) value of all result ranges found
/// HIPAR result/in the entire measurement (corresponding to black area of statistics
/// HIPAR result/table).
ViStatus _VI_FUNC rsspecan_GetAmplifierStatisticsTableCurrentIccMinimumAll(
                            ViSession instrSession,
                            ViInt32   measurementType,
                            ViReal64* result)
{
    ViStatus error = VI_SUCCESS;
    ViAttr   attr = 0;

    checkErr(RsCore_LockSession(instrSession));

    switch (measurementType)
    {
    case RSSPECAN_VAL_MEASTYPE_MIN:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_CURRENT_ICC_MINIMUM_MINIMUM_ALL;
        break;
    case RSSPECAN_VAL_MEASTYPE_MAX:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_CURRENT_ICC_MINIMUM_MAXIMUM_ALL;
        break;
    case RSSPECAN_VAL_MEASTYPE_SDEV:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_CURRENT_ICC_MINIMUM_STANDARD_DEVIATION_ALL;
        break;
    case RSSPECAN_VAL_MEASTYPE_AVER:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_CURRENT_ICC_MINIMUM_AVERAGE_ALL;
        break;
    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, measurementType), 2, "Measurement Type");
        break;
    }

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, NULL, attr, result),
            3, "Result");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the statistical Current (Icc) value of all result ranges found
/// HIFN  in the entire measurement (corresponding to black area of statistics
/// HIFN  table).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR measurementType/Selects measurement type.
/// HIPAR result/Queries the statistical Current (Icc) value of all result ranges found
/// HIPAR result/in the entire measurement (corresponding to black area of statistics
/// HIPAR result/table).
ViStatus _VI_FUNC rsspecan_GetAmplifierStatisticsTableCurrentIccMinimumCurrent(
                            ViSession instrSession,
                            ViInt32   measurementType,
                            ViReal64* result)
{
    ViStatus error = VI_SUCCESS;
    ViAttr   attr = 0;

    checkErr(RsCore_LockSession(instrSession));

    switch (measurementType)
    {
    case RSSPECAN_VAL_MEASTYPE_MIN:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_CURRENT_ICC_MINIMUM_MINIMUM_CURRENT;
        break;
    case RSSPECAN_VAL_MEASTYPE_MAX:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_CURRENT_ICC_MINIMUM_MAXIMUM_CURRENT;
        break;
    case RSSPECAN_VAL_MEASTYPE_SDEV:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_CURRENT_ICC_MINIMUM_STANDARD_DEVIATION_CURRENT;
        break;
    case RSSPECAN_VAL_MEASTYPE_AVER:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_CURRENT_ICC_MINIMUM_AVERAGE_CURRENT;
        break;
    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, measurementType), 2, "Measurement Type");
        break;
    }

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, NULL, attr, result),
            3, "Result");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the Current (Icc) result of the currently selected result
/// HIFN  range (corresponds to blue area of statistics table).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR result/Queries the Current (Icc) result of the currently selected result
/// HIPAR result/range (corresponds to blue area of statistics table).
ViStatus _VI_FUNC rsspecan_GetAmplifierStatisticsTableCurrentIccMinimumSelectedResult(
                            ViSession instrSession,
                            ViReal64* result)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, NULL, RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_CURRENT_ICC_MINIMUM_SELECTED_RESULT, result),
            2, "Result");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the statistical I/Q Imbalance value of all result ranges found
/// HIFN  in the entire measurement (corresponding to black area of statistics
/// HIFN  table).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR measurementType/Selects measurement type.
/// HIPAR result/Queries the statistical I/Q Imbalance value of all result ranges found
/// HIPAR result/in the entire measurement (corresponding to black area of statistics
/// HIPAR result/table).
ViStatus _VI_FUNC rsspecan_GetAmplifierStatisticsTableIQImbalanceAll(
                            ViSession instrSession,
                            ViInt32   measurementType,
                            ViReal64* result)
{
    ViStatus error = VI_SUCCESS;
    ViAttr   attr = 0;

    checkErr(RsCore_LockSession(instrSession));

    switch (measurementType)
    {
    case RSSPECAN_VAL_MEASTYPE_MIN:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_I_Q_IMBALANCE_MINIMUM_ALL;
        break;
    case RSSPECAN_VAL_MEASTYPE_MAX:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_I_Q_IMBALANCE_MAXIMUM_ALL;
        break;
    case RSSPECAN_VAL_MEASTYPE_SDEV:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_I_Q_IMBALANCE_STANDARD_DEVIATION_ALL;
        break;
    case RSSPECAN_VAL_MEASTYPE_AVER:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_I_Q_IMBALANCE_AVERAGE_ALL;
        break;
    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, measurementType), 2, "Measurement Type");
        break;
    }

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, NULL, attr, result),
            3, "Result");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the statistical I/Q Imbalance value of all result ranges found
/// HIFN  in the entire measurement (corresponding to black area of statistics
/// HIFN  table).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR measurementType/Selects measurement type.
/// HIPAR result/Queries the statistical I/Q Imbalance value of all result ranges found
/// HIPAR result/in the entire measurement (corresponding to black area of statistics
/// HIPAR result/table).
ViStatus _VI_FUNC rsspecan_GetAmplifierStatisticsTableIQImbalanceCurrent(
                            ViSession instrSession,
                            ViInt32   measurementType,
                            ViReal64* result)
{
    ViStatus error = VI_SUCCESS;
    ViAttr   attr = 0;

    checkErr(RsCore_LockSession(instrSession));

    switch (measurementType)
    {
    case RSSPECAN_VAL_MEASTYPE_MIN:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_I_Q_IMBALANCE_MINIMUM_CURRENT;
        break;
    case RSSPECAN_VAL_MEASTYPE_MAX:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_I_Q_IMBALANCE_MAXIMUM_CURRENT;
        break;
    case RSSPECAN_VAL_MEASTYPE_SDEV:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_I_Q_IMBALANCE_STANDARD_DEVIATION_CURRENT;
        break;
    case RSSPECAN_VAL_MEASTYPE_AVER:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_I_Q_IMBALANCE_AVERAGE_CURRENT;
        break;
    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, measurementType), 2, "Measurement Type");
        break;
    }

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, NULL, attr, result),
            3, "Result");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the I/Q Imbalance result of the currently selected result
/// HIFN  range (corresponds to blue area of statistics table).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR result/Queries the I/Q Imbalance result of the currently selected result
/// HIPAR result/range (corresponds to blue area of statistics table).
ViStatus _VI_FUNC rsspecan_GetAmplifierStatisticsTableIQImbalanceSelectedResult(
                            ViSession instrSession,
                            ViReal64* result)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, NULL, RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_I_Q_IMBALANCE_SELECTED_RESULT, result),
            2, "Result");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the statistical I/Q Offset value of all result ranges found in
/// HIFN  the entire measurement (corresponding to black area of statistics
/// HIFN  table).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR measurementType/Selects measurement type.
/// HIPAR result/Queries the statistical I/Q Offset value of all result ranges found in
/// HIPAR result/the entire measurement (corresponding to black area of statistics
/// HIPAR result/table).
ViStatus _VI_FUNC rsspecan_GetAmplifierStatisticsTableIQOffsetAll(
                            ViSession instrSession,
                            ViInt32   measurementType,
                            ViReal64* result)
{
    ViStatus error = VI_SUCCESS;
    ViAttr   attr = 0;

    checkErr(RsCore_LockSession(instrSession));

    switch (measurementType)
    {
    case RSSPECAN_VAL_MEASTYPE_MIN:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_I_Q_OFFSET_MINIMUM_ALL;
        break;
    case RSSPECAN_VAL_MEASTYPE_MAX:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_I_Q_OFFSET_MAXIMUM_ALL;
        break;
    case RSSPECAN_VAL_MEASTYPE_SDEV:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_I_Q_OFFSET_STANDARD_DEVIATION_ALL;
        break;
    case RSSPECAN_VAL_MEASTYPE_AVER:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_I_Q_OFFSET_AVERAGE_ALL;
        break;
    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, measurementType), 2, "Measurement Type");
        break;
    }

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, NULL, attr, result),
            3, "Result");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the statistical I/Q Offset value of all result ranges found in
/// HIFN  the entire measurement (corresponding to black area of statistics
/// HIFN  table).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR measurementType/Selects measurement type.
/// HIPAR result/Queries the statistical I/Q Offset value of all result ranges found in
/// HIPAR result/the entire measurement (corresponding to black area of statistics
/// HIPAR result/table).
ViStatus _VI_FUNC rsspecan_GetAmplifierStatisticsTableIQOffsetCurrent(
                            ViSession instrSession,
                            ViInt32   measurementType,
                            ViReal64* result)
{
    ViStatus error = VI_SUCCESS;
    ViAttr   attr = 0;

    checkErr(RsCore_LockSession(instrSession));

    switch (measurementType)
    {
    case RSSPECAN_VAL_MEASTYPE_MIN:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_I_Q_OFFSET_MINIMUM_CURRENT;
        break;
    case RSSPECAN_VAL_MEASTYPE_MAX:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_I_Q_OFFSET_MAXIMUM_CURRENT;
        break;
    case RSSPECAN_VAL_MEASTYPE_SDEV:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_I_Q_OFFSET_STANDARD_DEVIATION_CURRENT;
        break;
    case RSSPECAN_VAL_MEASTYPE_AVER:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_I_Q_OFFSET_AVERAGE_CURRENT;
        break;
    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, measurementType), 2, "Measurement Type");
        break;
    }

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, NULL, attr, result),
            3, "Result");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the I/Q Offset result of the currently selected result range
/// HIFN  (corresponds to blue area of statistics table).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR result/Queries the I/Q Offset result of the currently selected result range
/// HIPAR result/(corresponds to blue area of statistics table).
ViStatus _VI_FUNC rsspecan_GetAmplifierStatisticsTableIQOffsetSelectedResult(
                            ViSession instrSession,
                            ViReal64* result)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, NULL, RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_I_Q_OFFSET_SELECTED_RESULT, result),
            2, "Result");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the statistical Baseband I Input Voltage value of all result
/// HIFN  ranges found in the entire measurement (corresponding to black area of
/// HIFN  statistics table).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR measurementType/Selects measurement type.
/// HIPAR result/Queries the statistical Baseband I Input Voltage value of all result
/// HIPAR result/ranges found in the entire measurement (corresponding to black area of
/// HIPAR result/statistics table).
ViStatus _VI_FUNC rsspecan_GetAmplifierStatisticsTableBasebandIInputVoltageAverageAll(
                            ViSession instrSession,
                            ViInt32   measurementType,
                            ViReal64* result)
{
    ViStatus error = VI_SUCCESS;
    ViAttr   attr = 0;

    checkErr(RsCore_LockSession(instrSession));

    switch (measurementType)
    {
    case RSSPECAN_VAL_MEASTYPE_MIN:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_BASEBAND_I_INPUT_VOLTAGE_AVERAGE_MINIMUM_ALL;
        break;
    case RSSPECAN_VAL_MEASTYPE_MAX:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_BASEBAND_I_INPUT_VOLTAGE_AVERAGE_MAXIMUM_ALL;
        break;
    case RSSPECAN_VAL_MEASTYPE_SDEV:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_BASEBAND_I_INPUT_VOLTAGE_AVERAGE_STANDARD_DEVIATION_ALL;
        break;
    case RSSPECAN_VAL_MEASTYPE_AVER:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_BASEBAND_I_INPUT_VOLTAGE_AVERAGE_AVERAGE_ALL;
        break;
    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, measurementType), 2, "Measurement Type");
        break;
    }

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, NULL, attr, result),
            3, "Result");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the statistical Baseband I Input Voltage value of all result
/// HIFN  ranges found in the entire measurement (corresponding to black area of
/// HIFN  statistics table).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR measurementType/Selects measurement type.
/// HIPAR result/Queries the statistical Baseband I Input Voltage value of all result
/// HIPAR result/ranges found in the entire measurement (corresponding to black area of
/// HIPAR result/statistics table).
ViStatus _VI_FUNC rsspecan_GetAmplifierStatisticsTableBasebandIInputVoltageAverageCurrent(
                            ViSession instrSession,
                            ViInt32   measurementType,
                            ViReal64* result)
{
    ViStatus error = VI_SUCCESS;
    ViAttr   attr = 0;

    checkErr(RsCore_LockSession(instrSession));

    switch (measurementType)
    {
    case RSSPECAN_VAL_MEASTYPE_MIN:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_BASEBAND_I_INPUT_VOLTAGE_AVERAGE_MINIMUM_CURRENT;
        break;
    case RSSPECAN_VAL_MEASTYPE_MAX:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_BASEBAND_I_INPUT_VOLTAGE_AVERAGE_MAXIMUM_CURRENT;
        break;
    case RSSPECAN_VAL_MEASTYPE_SDEV:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_BASEBAND_I_INPUT_VOLTAGE_AVERAGE_STANDARD_DEVIATION_CURRENT;
        break;
    case RSSPECAN_VAL_MEASTYPE_AVER:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_BASEBAND_I_INPUT_VOLTAGE_AVERAGE_AVERAGE_CURRENT;
        break;
    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, measurementType), 2, "Measurement Type");
        break;
    }

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, NULL, attr, result),
            3, "Result");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the Baseband I Input Voltage result of the currently selected
/// HIFN  result range (corresponds to blue area of statistics table).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR result/Queries the Baseband I Input Voltage result of the currently selected
/// HIPAR result/result range (corresponds to blue area of statistics table).
ViStatus _VI_FUNC rsspecan_GetAmplifierStatisticsTableBasebandIInputVoltageAverageSelectedResult(
                            ViSession instrSession,
                            ViReal64* result)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, NULL, RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_BASEBAND_I_INPUT_VOLTAGE_AVERAGE_SELECTED_RESULT, result),
            2, "Result");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the statistical Baseband I Input Voltage value of all result
/// HIFN  ranges found in the entire measurement (corresponding to black area of
/// HIFN  statistics table).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR measurementType/Selects measurement type.
/// HIPAR result/Queries the statistical Baseband I Input Voltage value of all result
/// HIPAR result/ranges found in the entire measurement (corresponding to black area of
/// HIPAR result/statistics table).
ViStatus _VI_FUNC rsspecan_GetAmplifierStatisticsTableBasebandIInputVoltageMaximumAll(
                            ViSession instrSession,
                            ViInt32   measurementType,
                            ViReal64* result)
{
    ViStatus error = VI_SUCCESS;
    ViAttr   attr = 0;

    checkErr(RsCore_LockSession(instrSession));

    switch (measurementType)
    {
    case RSSPECAN_VAL_MEASTYPE_MIN:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_BASEBAND_I_INPUT_VOLTAGE_MAXIMUM_MINIMUM_ALL;
        break;
    case RSSPECAN_VAL_MEASTYPE_MAX:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_BASEBAND_I_INPUT_VOLTAGE_MAXIMUM_MAXIMUM_ALL;
        break;
    case RSSPECAN_VAL_MEASTYPE_SDEV:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_BASEBAND_I_INPUT_VOLTAGE_MAXIMUM_STANDARD_DEVIATION_ALL;
        break;
    case RSSPECAN_VAL_MEASTYPE_AVER:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_BASEBAND_I_INPUT_VOLTAGE_MAXIMUM_AVERAGE_ALL;
        break;
    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, measurementType), 2, "Measurement Type");
        break;
    }

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, NULL, attr, result),
            3, "Result");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the statistical Baseband I Input Voltage value of all result
/// HIFN  ranges found in the entire measurement (corresponding to black area of
/// HIFN  statistics table).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR measurementType/Selects measurement type.
/// HIPAR result/Queries the statistical Baseband I Input Voltage value of all result
/// HIPAR result/ranges found in the entire measurement (corresponding to black area of
/// HIPAR result/statistics table).
ViStatus _VI_FUNC rsspecan_GetAmplifierStatisticsTableBasebandIInputVoltageMaximumCurrent(
                            ViSession instrSession,
                            ViInt32   measurementType,
                            ViReal64* result)
{
    ViStatus error = VI_SUCCESS;
    ViAttr   attr = 0;

    checkErr(RsCore_LockSession(instrSession));

    switch (measurementType)
    {
    case RSSPECAN_VAL_MEASTYPE_MIN:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_BASEBAND_I_INPUT_VOLTAGE_MAXIMUM_MINIMUM_CURRENT;
        break;
    case RSSPECAN_VAL_MEASTYPE_MAX:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_BASEBAND_I_INPUT_VOLTAGE_MAXIMUM_MAXIMUM_CURRENT;
        break;
    case RSSPECAN_VAL_MEASTYPE_SDEV:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_BASEBAND_I_INPUT_VOLTAGE_MAXIMUM_STANDARD_DEVIATION_CURRENT;
        break;
    case RSSPECAN_VAL_MEASTYPE_AVER:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_BASEBAND_I_INPUT_VOLTAGE_MAXIMUM_AVERAGE_CURRENT;
        break;
    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, measurementType), 2, "Measurement Type");
        break;
    }

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, NULL, attr, result),
            3, "Result");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the Baseband I Input Voltage result of the currently selected
/// HIFN  result range (corresponds to blue area of statistics table).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR result/Queries the Baseband I Input Voltage result of the currently selected
/// HIPAR result/result range (corresponds to blue area of statistics table).
ViStatus _VI_FUNC rsspecan_GetAmplifierStatisticsTableBasebandIInputVoltageMaximumSelectedResult(
                            ViSession instrSession,
                            ViReal64* result)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, NULL, RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_BASEBAND_I_INPUT_VOLTAGE_MAXIMUM_SELECTED_RESULT, result),
            2, "Result");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the statistical Baseband I Input Voltage value of all result
/// HIFN  ranges found in the entire measurement (corresponding to black area of
/// HIFN  statistics table).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR measurementType/Selects measurement type.
/// HIPAR result/Queries the statistical Baseband I Input Voltage value of all result
/// HIPAR result/ranges found in the entire measurement (corresponding to black area of
/// HIPAR result/statistics table).
ViStatus _VI_FUNC rsspecan_GetAmplifierStatisticsTableBasebandIInputVoltageMinimumAll(
                            ViSession instrSession,
                            ViInt32   measurementType,
                            ViReal64* result)
{
    ViStatus error = VI_SUCCESS;
    ViAttr   attr = 0;

    checkErr(RsCore_LockSession(instrSession));

    switch (measurementType)
    {
    case RSSPECAN_VAL_MEASTYPE_MIN:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_BASEBAND_I_INPUT_VOLTAGE_MINIMUM_MINIMUM_ALL;
        break;
    case RSSPECAN_VAL_MEASTYPE_MAX:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_BASEBAND_I_INPUT_VOLTAGE_MINIMUM_MAXIMUM_ALL;
        break;
    case RSSPECAN_VAL_MEASTYPE_SDEV:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_BASEBAND_I_INPUT_VOLTAGE_MINIMUM_STANDARD_DEVIATION_ALL;
        break;
    case RSSPECAN_VAL_MEASTYPE_AVER:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_BASEBAND_I_INPUT_VOLTAGE_MINIMUM_AVERAGE_ALL;
        break;
    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, measurementType), 2, "Measurement Type");
        break;
    }

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, NULL, attr, result),
            3, "Result");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the statistical Baseband I Input Voltage value of all result
/// HIFN  ranges found in the entire measurement (corresponding to black area of
/// HIFN  statistics table).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR measurementType/Selects measurement type.
/// HIPAR result/Queries the statistical Baseband I Input Voltage value of all result
/// HIPAR result/ranges found in the entire measurement (corresponding to black area of
/// HIPAR result/statistics table).
ViStatus _VI_FUNC rsspecan_GetAmplifierStatisticsTableBasebandIInputVoltageMinimumCurrent(
                            ViSession instrSession,
                            ViInt32   measurementType,
                            ViReal64* result)
{
    ViStatus error = VI_SUCCESS;
    ViAttr   attr = 0;

    checkErr(RsCore_LockSession(instrSession));

    switch (measurementType)
    {
    case RSSPECAN_VAL_MEASTYPE_MIN:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_BASEBAND_I_INPUT_VOLTAGE_MINIMUM_MINIMUM_CURRENT;
        break;
    case RSSPECAN_VAL_MEASTYPE_MAX:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_BASEBAND_I_INPUT_VOLTAGE_MINIMUM_MAXIMUM_CURRENT;
        break;
    case RSSPECAN_VAL_MEASTYPE_SDEV:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_BASEBAND_I_INPUT_VOLTAGE_MINIMUM_STANDARD_DEVIATION_CURRENT;
        break;
    case RSSPECAN_VAL_MEASTYPE_AVER:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_BASEBAND_I_INPUT_VOLTAGE_MINIMUM_AVERAGE_CURRENT;
        break;
    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, measurementType), 2, "Measurement Type");
        break;
    }

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, NULL, attr, result),
            3, "Result");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the Baseband I Input Voltage result of the currently selected
/// HIFN  result range (corresponds to blue area of statistics table).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR result/Queries the Baseband I Input Voltage result of the currently selected
/// HIPAR result/result range (corresponds to blue area of statistics table).
ViStatus _VI_FUNC rsspecan_GetAmplifierStatisticsTableBasebandIInputVoltageMinimumSelectedResult(
                            ViSession instrSession,
                            ViReal64* result)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, NULL, RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_BASEBAND_I_INPUT_VOLTAGE_MINIMUM_SELECTED_RESULT, result),
            2, "Result");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the statistical Magnitude Error value of all result ranges
/// HIFN  found in the entire measurement (corresponding to black area of
/// HIFN  statistics table).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR measurementType/Selects measurement type.
/// HIPAR result/Queries the statistical Magnitude Error value of all result ranges
/// HIPAR result/found in the entire measurement (corresponding to black area of
/// HIPAR result/statistics table).
ViStatus _VI_FUNC rsspecan_GetAmplifierStatisticsTableMagnitudeErrorAll(
                            ViSession instrSession,
                            ViInt32   measurementType,
                            ViReal64* result)
{
    ViStatus error = VI_SUCCESS;
    ViAttr   attr = 0;

    checkErr(RsCore_LockSession(instrSession));

    switch (measurementType)
    {
    case RSSPECAN_VAL_MEASTYPE_MIN:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_MAGNITUDE_ERROR_MINIMUM_ALL;
        break;
    case RSSPECAN_VAL_MEASTYPE_MAX:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_MAGNITUDE_ERROR_MAXIMUM_ALL;
        break;
    case RSSPECAN_VAL_MEASTYPE_SDEV:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_MAGNITUDE_ERROR_STANDARD_DEVIATION_ALL;
        break;
    case RSSPECAN_VAL_MEASTYPE_AVER:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_MAGNITUDE_ERROR_AVERAGE_ALL;
        break;
    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, measurementType), 2, "Measurement Type");
        break;
    }

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, NULL, attr, result),
            3, "Result");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the statistical Magnitude Error value of all result ranges
/// HIFN  found in the entire measurement (corresponding to black area of
/// HIFN  statistics table).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR measurementType/Selects measurement type.
/// HIPAR result/Queries the statistical Magnitude Error value of all result ranges
/// HIPAR result/found in the entire measurement (corresponding to black area of
/// HIPAR result/statistics table).
ViStatus _VI_FUNC rsspecan_GetAmplifierStatisticsTableMagnitudeErrorCurrent(
                            ViSession instrSession,
                            ViInt32   measurementType,
                            ViReal64* result)
{
    ViStatus error = VI_SUCCESS;
    ViAttr   attr = 0;

    checkErr(RsCore_LockSession(instrSession));

    switch (measurementType)
    {
    case RSSPECAN_VAL_MEASTYPE_MIN:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_MAGNITUDE_ERROR_MINIMUM_CURRENT;
        break;
    case RSSPECAN_VAL_MEASTYPE_MAX:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_MAGNITUDE_ERROR_MAXIMUM_CURRENT;
        break;
    case RSSPECAN_VAL_MEASTYPE_SDEV:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_MAGNITUDE_ERROR_STANDARD_DEVIATION_CURRENT;
        break;
    case RSSPECAN_VAL_MEASTYPE_AVER:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_MAGNITUDE_ERROR_AVERAGE_CURRENT;
        break;
    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, measurementType), 2, "Measurement Type");
        break;
    }

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, NULL, attr, result),
            3, "Result");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the Magnitude Error result of the currently selected result
/// HIFN  range (corresponds to blue area of statistics table).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR result/Queries the Magnitude Error result of the currently selected result
/// HIPAR result/range (corresponds to blue area of statistics table).
ViStatus _VI_FUNC rsspecan_GetAmplifierStatisticsTableMagnitudeErrorSelectedResult(
                            ViSession instrSession,
                            ViReal64* result)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, NULL, RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_MAGNITUDE_ERROR_SELECTED_RESULT, result),
            2, "Result");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the statistical compression point Input value of all result
/// HIFN  ranges found in the entire measurement (corresponding to black area of
/// HIFN  statistics table).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR compressionPoint/Selects the compression point for query.
/// HIPAR measurementType/Selects measurement type.
/// HIPAR result/Queries the statistical compression point Input value of all result
/// HIPAR result/ranges found in the entire measurement (corresponding to black area of
/// HIPAR result/statistics table).
ViStatus _VI_FUNC rsspecan_GetAmplifierStatisticsTableCompressionPointsdBInputAll(
                            ViSession instrSession,
                            ViInt32   compressionPoint,
                            ViInt32   measurementType,
                            ViReal64* result)
{
    ViStatus error = VI_SUCCESS;
    ViAttr   attr = 0;

    checkErr(RsCore_LockSession(instrSession));

    switch (compressionPoint)
    {
    case RSSPECAN_VAL_COMPRESSION_POINT_1DB:
        switch (measurementType)
        {
        case RSSPECAN_VAL_MEASTYPE_MIN:
            attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_P1DB_INPUT_MINIMUM_ALL;
            break;
        case RSSPECAN_VAL_MEASTYPE_MAX:
            attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_P1DB_INPUT_MAXIMUM_ALL;
            break;
        case RSSPECAN_VAL_MEASTYPE_SDEV:
            attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_P1DB_INPUT_STANDARD_DEVIATION_ALL;
            break;
        case RSSPECAN_VAL_MEASTYPE_AVER:
            attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_P1DB_INPUT_AVERAGE_ALL;
            break;
        default:
            checkErr(RsCore_InvalidViInt32Value(instrSession, measurementType));
            break;
        }
        break;
    case RSSPECAN_VAL_COMPRESSION_POINT_2DB:
        switch (measurementType)
        {
        case RSSPECAN_VAL_MEASTYPE_MIN:
            attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_P2DB_INPUT_MINIMUM_ALL;
            break;
        case RSSPECAN_VAL_MEASTYPE_MAX:
            attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_P2DB_INPUT_MAXIMUM_ALL;
            break;
        case RSSPECAN_VAL_MEASTYPE_SDEV:
            attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_P2DB_INPUT_STANDARD_DEVIATION_ALL;
            break;
        case RSSPECAN_VAL_MEASTYPE_AVER:
            attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_P2DB_INPUT_AVERAGE_ALL;
            break;
        default:
            checkErr(RsCore_InvalidViInt32Value(instrSession, measurementType));
            break;
        }
        break;
    case RSSPECAN_VAL_COMPRESSION_POINT_3DB:
        switch (measurementType)
        {
        case RSSPECAN_VAL_MEASTYPE_MIN:
            attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_P3DB_INPUT_MINIMUM_ALL;
            break;
        case RSSPECAN_VAL_MEASTYPE_MAX:
            attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_P3DB_INPUT_MAXIMUM_ALL;
            break;
        case RSSPECAN_VAL_MEASTYPE_SDEV:
            attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_P3DB_INPUT_STANDARD_DEVIATION_ALL;
            break;
        case RSSPECAN_VAL_MEASTYPE_AVER:
            attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_P3DB_INPUT_AVERAGE_ALL;
            break;
        default:
            checkErr(RsCore_InvalidViInt32Value(instrSession, measurementType));
            break;
        }
        break;
    default:
        checkErr(RsCore_InvalidViInt32Value(instrSession, compressionPoint));
        break;
    }

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, NULL, attr, result),
            4, "Result");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the statistical compression point Input value of all result
/// HIFN  ranges found in the current capture (corresponding to green area of
/// HIFN  statistics table).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR compressionPoint/Selects the compression point for query.
/// HIPAR measurementType/Selects measurement type.
/// HIPAR result/Queries the statistical compression point Input value of all result
/// HIPAR result/ranges found in the current capture (corresponding to green area of
/// HIPAR result/statistics table).
ViStatus _VI_FUNC rsspecan_GetAmplifierStatisticsTableCompressionPointsdBInputCurrent(
                            ViSession instrSession,
                            ViInt32   compressionPoint,
                            ViInt32   measurementType,
                            ViReal64* result)
{
    ViStatus error = VI_SUCCESS;
    ViAttr   attr = 0;

    checkErr(RsCore_LockSession(instrSession));

    switch (compressionPoint)
    {
    case RSSPECAN_VAL_COMPRESSION_POINT_1DB:
        switch (measurementType)
        {
        case RSSPECAN_VAL_MEASTYPE_MIN:
            attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_P1DB_INPUT_MINIMUM_CURRENT;
            break;
        case RSSPECAN_VAL_MEASTYPE_MAX:
            attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_P1DB_INPUT_MAXIMUM_CURRENT;
            break;
        case RSSPECAN_VAL_MEASTYPE_SDEV:
            attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_P1DB_INPUT_STANDARD_DEVIATION_CURRENT;
            break;
        case RSSPECAN_VAL_MEASTYPE_AVER:
            attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_P1DB_INPUT_AVERAGE_CURRENT;
            break;
        default:
            checkErr(RsCore_InvalidViInt32Value(instrSession, measurementType));
            break;
        }
        break;
    case RSSPECAN_VAL_COMPRESSION_POINT_2DB:
        switch (measurementType)
        {
        case RSSPECAN_VAL_MEASTYPE_MIN:
            attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_P2DB_INPUT_MINIMUM_CURRENT;
            break;
        case RSSPECAN_VAL_MEASTYPE_MAX:
            attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_P2DB_INPUT_MAXIMUM_CURRENT;
            break;
        case RSSPECAN_VAL_MEASTYPE_SDEV:
            attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_P2DB_INPUT_STANDARD_DEVIATION_CURRENT;
            break;
        case RSSPECAN_VAL_MEASTYPE_AVER:
            attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_P2DB_INPUT_AVERAGE_CURRENT;
            break;
        default:
            checkErr(RsCore_InvalidViInt32Value(instrSession, measurementType));
            break;
        }
        break;
    case RSSPECAN_VAL_COMPRESSION_POINT_3DB:
        switch (measurementType)
        {
        case RSSPECAN_VAL_MEASTYPE_MIN:
            attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_P3DB_INPUT_MINIMUM_CURRENT;
            break;
        case RSSPECAN_VAL_MEASTYPE_MAX:
            attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_P3DB_INPUT_MAXIMUM_CURRENT;
            break;
        case RSSPECAN_VAL_MEASTYPE_SDEV:
            attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_P3DB_INPUT_STANDARD_DEVIATION_CURRENT;
            break;
        case RSSPECAN_VAL_MEASTYPE_AVER:
            attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_P3DB_INPUT_AVERAGE_CURRENT;
            break;
        default:
            checkErr(RsCore_InvalidViInt32Value(instrSession, measurementType));
            break;
        }
        break;
    default:
        checkErr(RsCore_InvalidViInt32Value(instrSession, compressionPoint));
        break;
    }

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, NULL, attr, result),
            4, "Result");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the compression point Input result of the currently selected
/// HIFN  result range (corresponds to blue area of statistics table).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR compressionPoint/Selects the compression point for query.
/// HIPAR result/Queries the compression Input result of the currently selected result
/// HIPAR result/range (corresponds to blue area of statistics table).
ViStatus _VI_FUNC rsspecan_GetAmplifierStatisticsTableCompressionPointsdBInputSelectedResult(
                            ViSession instrSession,
                            ViInt32   compressionPoint,
                            ViReal64* result)
{
    ViStatus error = VI_SUCCESS;
    ViAttr   attr = 0;

    checkErr(RsCore_LockSession(instrSession));

    switch (compressionPoint)
    {
    case RSSPECAN_VAL_COMPRESSION_POINT_1DB:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_P1DB_INPUT_SELECTED_RESULT;
        break;
    case RSSPECAN_VAL_COMPRESSION_POINT_2DB:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_P2DB_INPUT_SELECTED_RESULT;
        break;
    case RSSPECAN_VAL_COMPRESSION_POINT_3DB:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_P3DB_INPUT_SELECTED_RESULT;
        break;
    default:
        checkErr(RsCore_InvalidViInt32Value(instrSession, compressionPoint));
        break;
    }

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, NULL, attr, result),
            4, "Result");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the statistical compression point output value of all result
/// HIFN  ranges found in the entire measurement (corresponding to black area of
/// HIFN  statistics table).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR compressionPoint/Selects the compression point for query.
/// HIPAR measurementType/Selects measurement type.
/// HIPAR result/Queries the statistical compression point output value of all result
/// HIPAR result/ranges found in the entire measurement (corresponding to black area of
/// HIPAR result/statistics table).
ViStatus _VI_FUNC rsspecan_GetAmplifierStatisticsTableCompressionPointsdBOutputAll(
                            ViSession instrSession,
                            ViInt32   compressionPoint,
                            ViInt32   measurementType,
                            ViReal64* result)
{
    ViStatus error = VI_SUCCESS;
    ViAttr   attr = 0;

    checkErr(RsCore_LockSession(instrSession));

    switch (compressionPoint)
    {
    case RSSPECAN_VAL_COMPRESSION_POINT_1DB:
        switch (measurementType)
        {
        case RSSPECAN_VAL_MEASTYPE_MIN:
            attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_P1DB_OUTPUT_MINIMUM_ALL;
            break;
        case RSSPECAN_VAL_MEASTYPE_MAX:
            attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_P1DB_OUTPUT_MAXIMUM_ALL;
            break;
        case RSSPECAN_VAL_MEASTYPE_SDEV:
            attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_P1DB_OUTPUT_STANDARD_DEVIATION_ALL;
            break;
        case RSSPECAN_VAL_MEASTYPE_AVER:
            attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_P1DB_OUTPUT_AVERAGE_ALL;
            break;
        default:
            checkErr(RsCore_InvalidViInt32Value(instrSession, measurementType));
            break;
        }
        break;
    case RSSPECAN_VAL_COMPRESSION_POINT_2DB:
        switch (measurementType)
        {
        case RSSPECAN_VAL_MEASTYPE_MIN:
            attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_P2DB_OUTPUT_MINIMUM_ALL;
            break;
        case RSSPECAN_VAL_MEASTYPE_MAX:
            attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_P2DB_OUTPUT_MAXIMUM_ALL;
            break;
        case RSSPECAN_VAL_MEASTYPE_SDEV:
            attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_P2DB_OUTPUT_STANDARD_DEVIATION_ALL;
            break;
        case RSSPECAN_VAL_MEASTYPE_AVER:
            attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_P2DB_OUTPUT_AVERAGE_ALL;
            break;
        default:
            checkErr(RsCore_InvalidViInt32Value(instrSession, measurementType));
            break;
        }
        break;
    case RSSPECAN_VAL_COMPRESSION_POINT_3DB:
        switch (measurementType)
        {
        case RSSPECAN_VAL_MEASTYPE_MIN:
            attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_P3DB_OUTPUT_MINIMUM_ALL;
            break;
        case RSSPECAN_VAL_MEASTYPE_MAX:
            attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_P3DB_OUTPUT_MAXIMUM_ALL;
            break;
        case RSSPECAN_VAL_MEASTYPE_SDEV:
            attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_P3DB_OUTPUT_STANDARD_DEVIATION_ALL;
            break;
        case RSSPECAN_VAL_MEASTYPE_AVER:
            attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_P3DB_OUTPUT_AVERAGE_ALL;
            break;
        default:
            checkErr(RsCore_InvalidViInt32Value(instrSession, measurementType));
            break;
        }
        break;
    default:
        checkErr(RsCore_InvalidViInt32Value(instrSession, compressionPoint));
        break;
    }

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, NULL, attr, result),
            4, "Result");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the statistical compression point output value of all result
/// HIFN  ranges found in the current capture (corresponding to green area of
/// HIFN  statistics table).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR compressionPoint/Selects the compression point for query.
/// HIPAR measurementType/Selects measurement type.
/// HIPAR result/Queries the statistical compression point output value of all result
/// HIPAR result/ranges found in the current capture (corresponding to green area of
/// HIPAR result/statistics table).
ViStatus _VI_FUNC rsspecan_GetAmplifierStatisticsTableCompressionPointsdBOutputCurrent(
                            ViSession instrSession,
                            ViInt32   compressionPoint,
                            ViInt32   measurementType,
                            ViReal64* result)
{
    ViStatus error = VI_SUCCESS;
    ViAttr   attr = 0;

    checkErr(RsCore_LockSession(instrSession));

    switch (compressionPoint)
    {
    case RSSPECAN_VAL_COMPRESSION_POINT_1DB:
        switch (measurementType)
        {
        case RSSPECAN_VAL_MEASTYPE_MIN:
            attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_P1DB_OUTPUT_MINIMUM_CURRENT;
            break;
        case RSSPECAN_VAL_MEASTYPE_MAX:
            attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_P1DB_OUTPUT_MAXIMUM_CURRENT;
            break;
        case RSSPECAN_VAL_MEASTYPE_SDEV:
            attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_P1DB_OUTPUT_STANDARD_DEVIATION_CURRENT;
            break;
        case RSSPECAN_VAL_MEASTYPE_AVER:
            attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_P1DB_OUTPUT_AVERAGE_CURRENT;
            break;
        default:
            checkErr(RsCore_InvalidViInt32Value(instrSession, measurementType));
            break;
        }
        break;
    case RSSPECAN_VAL_COMPRESSION_POINT_2DB:
        switch (measurementType)
        {
        case RSSPECAN_VAL_MEASTYPE_MIN:
            attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_P2DB_OUTPUT_MINIMUM_CURRENT;
            break;
        case RSSPECAN_VAL_MEASTYPE_MAX:
            attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_P2DB_OUTPUT_MAXIMUM_CURRENT;
            break;
        case RSSPECAN_VAL_MEASTYPE_SDEV:
            attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_P2DB_OUTPUT_STANDARD_DEVIATION_CURRENT;
            break;
        case RSSPECAN_VAL_MEASTYPE_AVER:
            attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_P2DB_OUTPUT_AVERAGE_CURRENT;
            break;
        default:
            checkErr(RsCore_InvalidViInt32Value(instrSession, measurementType));
            break;
        }
        break;
    case RSSPECAN_VAL_COMPRESSION_POINT_3DB:
        switch (measurementType)
        {
        case RSSPECAN_VAL_MEASTYPE_MIN:
            attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_P3DB_OUTPUT_MINIMUM_CURRENT;
            break;
        case RSSPECAN_VAL_MEASTYPE_MAX:
            attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_P3DB_OUTPUT_MAXIMUM_CURRENT;
            break;
        case RSSPECAN_VAL_MEASTYPE_SDEV:
            attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_P3DB_OUTPUT_STANDARD_DEVIATION_CURRENT;
            break;
        case RSSPECAN_VAL_MEASTYPE_AVER:
            attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_P3DB_OUTPUT_AVERAGE_CURRENT;
            break;
        default:
            checkErr(RsCore_InvalidViInt32Value(instrSession, measurementType));
            break;
        }
        break;
    default:
        checkErr(RsCore_InvalidViInt32Value(instrSession, compressionPoint));
        break;
    }

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, NULL, attr, result),
            4, "Result");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the compression point Output result of the currently selected
/// HIFN  result range (corresponds to blue area of statistics table).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR compressionPoint/Selects the compression point for query.
/// HIPAR result/Queries the compression point Output result of the currently selected
/// HIPAR result/result range (corresponds to blue area of statistics table).
ViStatus _VI_FUNC rsspecan_GetAmplifierStatisticsTableCompressionPointsdBOutputSelectedResult(
                            ViSession instrSession,
                            ViInt32   compressionPoint,
                            ViReal64* result)
{
    ViStatus error = VI_SUCCESS;
    ViAttr   attr = 0;

    checkErr(RsCore_LockSession(instrSession));

    switch (compressionPoint)
    {
    case RSSPECAN_VAL_COMPRESSION_POINT_1DB:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_P1DB_OUTPUT_SELECTED_RESULT;
        break;
    case RSSPECAN_VAL_COMPRESSION_POINT_2DB:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_P2DB_OUTPUT_SELECTED_RESULT;
        break;
    case RSSPECAN_VAL_COMPRESSION_POINT_3DB:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_P3DB_OUTPUT_SELECTED_RESULT;
        break;
    default:
        checkErr(RsCore_InvalidViInt32Value(instrSession, compressionPoint));
        break;
    }

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, NULL, attr, result),
            4, "Result");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the statistical Average Power Consumption value of all result
/// HIFN  ranges found in the entire measurement (corresponding to black area of
/// HIFN  statistics table).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR measurementType/Selects measurement type.
/// HIPAR result/Queries the statistical Average Power Consumption value of all result
/// HIPAR result/ranges found in the entire measurement (corresponding to black area of
/// HIPAR result/statistics table).
ViStatus _VI_FUNC rsspecan_GetAmplifierStatisticsTableAveragePowerConsumptionAll(
                            ViSession instrSession,
                            ViInt32   measurementType,
                            ViReal64* result)
{
    ViStatus error = VI_SUCCESS;
    ViAttr   attr = 0;

    checkErr(RsCore_LockSession(instrSession));

    switch (measurementType)
    {
    case RSSPECAN_VAL_MEASTYPE_MIN:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_AVERAGE_POWER_CONSUMPTION_MINIMUM_ALL;
        break;
    case RSSPECAN_VAL_MEASTYPE_MAX:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_AVERAGE_POWER_CONSUMPTION_MAXIMUM_ALL;
        break;
    case RSSPECAN_VAL_MEASTYPE_SDEV:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_AVERAGE_POWER_CONSUMPTION_STANDARD_DEVIATION_ALL;
        break;
    case RSSPECAN_VAL_MEASTYPE_AVER:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_AVERAGE_POWER_CONSUMPTION_AVERAGE_ALL;
        break;
    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, measurementType), 2, "Measurement Type");
        break;
    }

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, NULL, attr, result),
            3, "Result");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the statistical Average Power Consumption value of all result
/// HIFN  ranges found in the entire measurement (corresponding to black area of
/// HIFN  statistics table).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR measurementType/Selects measurement type.
/// HIPAR result/Queries the statistical Average Power Consumption value of all result
/// HIPAR result/ranges found in the entire measurement (corresponding to black area of
/// HIPAR result/statistics table).
ViStatus _VI_FUNC rsspecan_GetAmplifierStatisticsTableAveragePowerConsumptionCurrent(
                            ViSession instrSession,
                            ViInt32   measurementType,
                            ViReal64* result)
{
    ViStatus error = VI_SUCCESS;
    ViAttr   attr = 0;

    checkErr(RsCore_LockSession(instrSession));

    switch (measurementType)
    {
    case RSSPECAN_VAL_MEASTYPE_MIN:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_AVERAGE_POWER_CONSUMPTION_MINIMUM_CURRENT;
        break;
    case RSSPECAN_VAL_MEASTYPE_MAX:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_AVERAGE_POWER_CONSUMPTION_MAXIMUM_CURRENT;
        break;
    case RSSPECAN_VAL_MEASTYPE_SDEV:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_AVERAGE_POWER_CONSUMPTION_STANDARD_DEVIATION_CURRENT;
        break;
    case RSSPECAN_VAL_MEASTYPE_AVER:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_AVERAGE_POWER_CONSUMPTION_AVERAGE_CURRENT;
        break;
    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, measurementType), 2, "Measurement Type");
        break;
    }

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, NULL, attr, result),
            3, "Result");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the Average Power Consumption result of the currently selected
/// HIFN  result range (corresponds to blue area of statistics table).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR result/Queries the Average Power Consumption result of the currently selected
/// HIPAR result/result range (corresponds to blue area of statistics table).
ViStatus _VI_FUNC rsspecan_GetAmplifierStatisticsTableAveragePowerConsumptionSelectedResult(
                            ViSession instrSession,
                            ViReal64* result)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, NULL, RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_AVERAGE_POWER_CONSUMPTION_SELECTED_RESULT, result),
            2, "Result");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the statistical PC-Based Average PAE value of all result
/// HIFN  ranges found in the entire measurement (corresponding to black area of
/// HIFN  statistics table).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR measurementType/Selects measurement type.
/// HIPAR result/Queries the statistical PC-Based Average PAE value of all result
/// HIPAR result/ranges found in the entire measurement (corresponding to black area of
/// HIPAR result/statistics table).
ViStatus _VI_FUNC rsspecan_GetAmplifierStatisticsTablePCBasedAveragePAEAll(
                            ViSession instrSession,
                            ViInt32   measurementType,
                            ViReal64* result)
{
    ViStatus error = VI_SUCCESS;
    ViAttr   attr = 0;

    checkErr(RsCore_LockSession(instrSession));

    switch (measurementType)
    {
    case RSSPECAN_VAL_MEASTYPE_MIN:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_PC_BASED_AVERAGE_PAE_MINIMUM_ALL;
        break;
    case RSSPECAN_VAL_MEASTYPE_MAX:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_PC_BASED_AVERAGE_PAE_MAXIMUM_ALL;
        break;
    case RSSPECAN_VAL_MEASTYPE_SDEV:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_PC_BASED_AVERAGE_PAE_STANDARD_DEVIATION_ALL;
        break;
    case RSSPECAN_VAL_MEASTYPE_AVER:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_PC_BASED_AVERAGE_PAE_AVERAGE_ALL;
        break;
    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, measurementType), 2, "Measurement Type");
        break;
    }

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, NULL, attr, result),
            3, "Result");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the statistical PC-Based Average PAE value of all result
/// HIFN  ranges found in the entire measurement (corresponding to black area of
/// HIFN  statistics table).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR measurementType/Selects measurement type.
/// HIPAR result/Queries the statistical PC-Based Average PAE value of all result
/// HIPAR result/ranges found in the entire measurement (corresponding to black area of
/// HIPAR result/statistics table).
ViStatus _VI_FUNC rsspecan_GetAmplifierStatisticsTablePCBasedAveragePAECurrent(
                            ViSession instrSession,
                            ViInt32   measurementType,
                            ViReal64* result)
{
    ViStatus error = VI_SUCCESS;
    ViAttr   attr = 0;

    checkErr(RsCore_LockSession(instrSession));

    switch (measurementType)
    {
    case RSSPECAN_VAL_MEASTYPE_MIN:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_PC_BASED_AVERAGE_PAE_MINIMUM_CURRENT;
        break;
    case RSSPECAN_VAL_MEASTYPE_MAX:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_PC_BASED_AVERAGE_PAE_MAXIMUM_CURRENT;
        break;
    case RSSPECAN_VAL_MEASTYPE_SDEV:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_PC_BASED_AVERAGE_PAE_STANDARD_DEVIATION_CURRENT;
        break;
    case RSSPECAN_VAL_MEASTYPE_AVER:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_PC_BASED_AVERAGE_PAE_AVERAGE_CURRENT;
        break;
    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, measurementType), 2, "Measurement Type");
        break;
    }

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, NULL, attr, result),
            3, "Result");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the PC based Average PAE result of the currently selected
/// HIFN  result range (corresponds to blue area of statistics table).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR result/Queries the PC based Average PAE result of the currently selected
/// HIPAR result/result range (corresponds to blue area of statistics table).
ViStatus _VI_FUNC rsspecan_GetAmplifierStatisticsTablePCBasedAveragePAESelectedResult(
                            ViSession instrSession,
                            ViReal64* result)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, NULL, RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_PC_BASED_AVERAGE_PAE_SELECTED_RESULT, result),
            2, "Result");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the statistical Phase Error value of all result ranges found
/// HIFN  in the entire measurement (corresponding to black area of statistics
/// HIFN  table).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR measurementType/Selects measurement type.
/// HIPAR result/Queries the statistical Phase Error value of all result ranges found
/// HIPAR result/in the entire measurement (corresponding to black area of statistics
/// HIPAR result/table).
ViStatus _VI_FUNC rsspecan_GetAmplifierStatisticsTablePhaseErrorAll(
                            ViSession instrSession,
                            ViInt32   measurementType,
                            ViReal64* result)
{
    ViStatus error = VI_SUCCESS;
    ViAttr   attr = 0;

    checkErr(RsCore_LockSession(instrSession));

    switch (measurementType)
    {
    case RSSPECAN_VAL_MEASTYPE_MIN:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_PHASE_ERROR_MINIMUM_ALL;
        break;
    case RSSPECAN_VAL_MEASTYPE_MAX:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_PHASE_ERROR_MAXIMUM_ALL;
        break;
    case RSSPECAN_VAL_MEASTYPE_SDEV:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_PHASE_ERROR_STANDARD_DEVIATION_ALL;
        break;
    case RSSPECAN_VAL_MEASTYPE_AVER:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_PHASE_ERROR_AVERAGE_ALL;
        break;
    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, measurementType), 2, "Measurement Type");
        break;
    }

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, NULL, attr, result),
            3, "Result");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the statistical Phase Error value of all result ranges found
/// HIFN  in the entire measurement (corresponding to black area of statistics
/// HIFN  table).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR measurementType/Selects measurement type.
/// HIPAR result/Queries the statistical Phase Error value of all result ranges found
/// HIPAR result/in the entire measurement (corresponding to black area of statistics
/// HIPAR result/table).
ViStatus _VI_FUNC rsspecan_GetAmplifierStatisticsTablePhaseErrorCurrent(
                            ViSession instrSession,
                            ViInt32   measurementType,
                            ViReal64* result)
{
    ViStatus error = VI_SUCCESS;
    ViAttr   attr = 0;

    checkErr(RsCore_LockSession(instrSession));

    switch (measurementType)
    {
    case RSSPECAN_VAL_MEASTYPE_MIN:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_PHASE_ERROR_MINIMUM_CURRENT;
        break;
    case RSSPECAN_VAL_MEASTYPE_MAX:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_PHASE_ERROR_MAXIMUM_CURRENT;
        break;
    case RSSPECAN_VAL_MEASTYPE_SDEV:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_PHASE_ERROR_STANDARD_DEVIATION_CURRENT;
        break;
    case RSSPECAN_VAL_MEASTYPE_AVER:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_PHASE_ERROR_AVERAGE_CURRENT;
        break;
    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, measurementType), 2, "Measurement Type");
        break;
    }

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, NULL, attr, result),
            3, "Result");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the Phase Error result of the currently selected result range
/// HIFN  (corresponds to blue area of statistics table).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR result/Queries the Phase Error result of the currently selected result range
/// HIPAR result/(corresponds to blue area of statistics table).
ViStatus _VI_FUNC rsspecan_GetAmplifierStatisticsTablePhaseErrorSelectedResult(
                            ViSession instrSession,
                            ViReal64* result)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, NULL, RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_PHASE_ERROR_SELECTED_RESULT, result),
            2, "Result");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the statistical Power Input value of all result ranges found
/// HIFN  in the entire measurement (corresponding to black area of statistics
/// HIFN  table).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR measurementType/Selects measurement type.
/// HIPAR result/Queries the statistical Power Input value of all result ranges found
/// HIPAR result/in the entire measurement (corresponding to black area of statistics
/// HIPAR result/table).
ViStatus _VI_FUNC rsspecan_GetAmplifierStatisticsTablePowerInputAverageAll(
                            ViSession instrSession,
                            ViInt32   measurementType,
                            ViReal64* result)
{
    ViStatus error = VI_SUCCESS;
    ViAttr   attr = 0;

    checkErr(RsCore_LockSession(instrSession));

    switch (measurementType)
    {
    case RSSPECAN_VAL_MEASTYPE_MIN:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_POWER_INPUT_AVERAGE_MINIMUM_ALL;
        break;
    case RSSPECAN_VAL_MEASTYPE_MAX:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_POWER_INPUT_AVERAGE_MAXIMUM_ALL;
        break;
    case RSSPECAN_VAL_MEASTYPE_SDEV:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_POWER_INPUT_AVERAGE_STANDARD_DEVIATION_ALL;
        break;
    case RSSPECAN_VAL_MEASTYPE_AVER:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_POWER_INPUT_AVERAGE_AVERAGE_ALL;
        break;
    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, measurementType), 2, "Measurement Type");
        break;
    }

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, NULL, attr, result),
            3, "Result");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the statistical Power Input value of all result ranges found
/// HIFN  in the entire measurement (corresponding to black area of statistics
/// HIFN  table).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR measurementType/Selects measurement type.
/// HIPAR result/Queries the statistical Power Input value of all result ranges found
/// HIPAR result/in the entire measurement (corresponding to black area of statistics
/// HIPAR result/table).
ViStatus _VI_FUNC rsspecan_GetAmplifierStatisticsTablePowerInputAverageCurrent(
                            ViSession instrSession,
                            ViInt32   measurementType,
                            ViReal64* result)
{
    ViStatus error = VI_SUCCESS;
    ViAttr   attr = 0;

    checkErr(RsCore_LockSession(instrSession));

    switch (measurementType)
    {
    case RSSPECAN_VAL_MEASTYPE_MIN:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_POWER_INPUT_AVERAGE_MINIMUM_CURRENT;
        break;
    case RSSPECAN_VAL_MEASTYPE_MAX:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_POWER_INPUT_AVERAGE_MAXIMUM_CURRENT;
        break;
    case RSSPECAN_VAL_MEASTYPE_SDEV:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_POWER_INPUT_AVERAGE_STANDARD_DEVIATION_CURRENT;
        break;
    case RSSPECAN_VAL_MEASTYPE_AVER:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_POWER_INPUT_AVERAGE_AVERAGE_CURRENT;
        break;
    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, measurementType), 2, "Measurement Type");
        break;
    }

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, NULL, attr, result),
            3, "Result");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the Power Input result of the currently selected result range
/// HIFN  (corresponds to blue area of statistics table).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR result/Queries the Power Input result of the currently selected result range
/// HIPAR result/(corresponds to blue area of statistics table).
ViStatus _VI_FUNC rsspecan_GetAmplifierStatisticsTablePowerInputAverageSelectedResult(
                            ViSession instrSession,
                            ViReal64* result)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, NULL, RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_POWER_INPUT_AVERAGE_SELECTED_RESULT, result),
            2, "Result");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the statistical Power Input value of all result ranges found
/// HIFN  in the entire measurement (corresponding to black area of statistics
/// HIFN  table).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR measurementType/Selects measurement type.
/// HIPAR result/Queries the statistical Power Input value of all result ranges found
/// HIPAR result/in the entire measurement (corresponding to black area of statistics
/// HIPAR result/table).
ViStatus _VI_FUNC rsspecan_GetAmplifierStatisticsTablePowerInputMaximumAll(
                            ViSession instrSession,
                            ViInt32   measurementType,
                            ViReal64* result)
{
    ViStatus error = VI_SUCCESS;
    ViAttr   attr = 0;

    checkErr(RsCore_LockSession(instrSession));

    switch (measurementType)
    {
    case RSSPECAN_VAL_MEASTYPE_MIN:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_POWER_INPUT_MAXIMUM_MINIMUM_ALL;
        break;
    case RSSPECAN_VAL_MEASTYPE_MAX:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_POWER_INPUT_MAXIMUM_MAXIMUM_ALL;
        break;
    case RSSPECAN_VAL_MEASTYPE_SDEV:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_POWER_INPUT_MAXIMUM_STANDARD_DEVIATION_ALL;
        break;
    case RSSPECAN_VAL_MEASTYPE_AVER:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_POWER_INPUT_MAXIMUM_AVERAGE_ALL;
        break;
    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, measurementType), 2, "Measurement Type");
        break;
    }

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, NULL, attr, result),
            3, "Result");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the statistical Power Input value of all result ranges found
/// HIFN  in the entire measurement (corresponding to black area of statistics
/// HIFN  table).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR measurementType/Selects measurement type.
/// HIPAR result/Queries the statistical Power Input value of all result ranges found
/// HIPAR result/in the entire measurement (corresponding to black area of statistics
/// HIPAR result/table).
ViStatus _VI_FUNC rsspecan_GetAmplifierStatisticsTablePowerInputMaximumCurrent(
                            ViSession instrSession,
                            ViInt32   measurementType,
                            ViReal64* result)
{
    ViStatus error = VI_SUCCESS;
    ViAttr   attr = 0;

    checkErr(RsCore_LockSession(instrSession));

    switch (measurementType)
    {
    case RSSPECAN_VAL_MEASTYPE_MIN:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_POWER_INPUT_MAXIMUM_MINIMUM_CURRENT;
        break;
    case RSSPECAN_VAL_MEASTYPE_MAX:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_POWER_INPUT_MAXIMUM_MAXIMUM_CURRENT;
        break;
    case RSSPECAN_VAL_MEASTYPE_SDEV:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_POWER_INPUT_MAXIMUM_STANDARD_DEVIATION_CURRENT;
        break;
    case RSSPECAN_VAL_MEASTYPE_AVER:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_POWER_INPUT_MAXIMUM_AVERAGE_CURRENT;
        break;
    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, measurementType), 2, "Measurement Type");
        break;
    }

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, NULL, attr, result),
            3, "Result");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the Power Input result of the currently selected result range
/// HIFN  (corresponds to blue area of statistics table).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR result/Queries the Power Input result of the currently selected result range
/// HIPAR result/(corresponds to blue area of statistics table).
ViStatus _VI_FUNC rsspecan_GetAmplifierStatisticsTablePowerInputMaximumSelectedResult(
                            ViSession instrSession,
                            ViReal64* result)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, NULL, RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_POWER_INPUT_MAXIMUM_SELECTED_RESULT, result),
            2, "Result");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the statistical Power Input value of all result ranges found
/// HIFN  in the entire measurement (corresponding to black area of statistics
/// HIFN  table).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR measurementType/Selects measurement type.
/// HIPAR result/Queries the statistical Power Input value of all result ranges found
/// HIPAR result/in the entire measurement (corresponding to black area of statistics
/// HIPAR result/table).
ViStatus _VI_FUNC rsspecan_GetAmplifierStatisticsTablePowerInputMinimumAll(
                            ViSession instrSession,
                            ViInt32   measurementType,
                            ViReal64* result)
{
    ViStatus error = VI_SUCCESS;
    ViAttr   attr = 0;

    checkErr(RsCore_LockSession(instrSession));

    switch (measurementType)
    {
    case RSSPECAN_VAL_MEASTYPE_MIN:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_POWER_INPUT_MINIMUM_MINIMUM_ALL;
        break;
    case RSSPECAN_VAL_MEASTYPE_MAX:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_POWER_INPUT_MINIMUM_MAXIMUM_ALL;
        break;
    case RSSPECAN_VAL_MEASTYPE_SDEV:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_POWER_INPUT_MINIMUM_STANDARD_DEVIATION_ALL;
        break;
    case RSSPECAN_VAL_MEASTYPE_AVER:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_POWER_INPUT_MINIMUM_AVERAGE_ALL;
        break;
    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, measurementType), 2, "Measurement Type");
        break;
    }

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, NULL, attr, result),
            3, "Result");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the statistical Power Input value of all result ranges found
/// HIFN  in the entire measurement (corresponding to black area of statistics
/// HIFN  table).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR measurementType/Selects measurement type.
/// HIPAR result/Queries the statistical Power Input value of all result ranges found
/// HIPAR result/in the entire measurement (corresponding to black area of statistics
/// HIPAR result/table).
ViStatus _VI_FUNC rsspecan_GetAmplifierStatisticsTablePowerInputMinimumCurrent(
                            ViSession instrSession,
                            ViInt32   measurementType,
                            ViReal64* result)
{
    ViStatus error = VI_SUCCESS;
    ViAttr   attr = 0;

    checkErr(RsCore_LockSession(instrSession));

    switch (measurementType)
    {
    case RSSPECAN_VAL_MEASTYPE_MIN:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_POWER_INPUT_MINIMUM_MINIMUM_CURRENT;
        break;
    case RSSPECAN_VAL_MEASTYPE_MAX:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_POWER_INPUT_MINIMUM_MAXIMUM_CURRENT;
        break;
    case RSSPECAN_VAL_MEASTYPE_SDEV:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_POWER_INPUT_MINIMUM_STANDARD_DEVIATION_CURRENT;
        break;
    case RSSPECAN_VAL_MEASTYPE_AVER:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_POWER_INPUT_MINIMUM_AVERAGE_CURRENT;
        break;
    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, measurementType), 2, "Measurement Type");
        break;
    }

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, NULL, attr, result),
            3, "Result");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the Power Input result of the currently selected result range
/// HIFN  (corresponds to blue area of statistics table).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR result/Queries the Power Input result of the currently selected result range
/// HIPAR result/(corresponds to blue area of statistics table).
ViStatus _VI_FUNC rsspecan_GetAmplifierStatisticsTablePowerInputMinimumSelectedResult(
                            ViSession instrSession,
                            ViReal64* result)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, NULL, RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_POWER_INPUT_MINIMUM_SELECTED_RESULT, result),
            2, "Result");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the statistical Power output value of all result ranges found
/// HIFN  in the entire measurement (corresponding to black area of statistics
/// HIFN  table).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR measurementType/Selects measurement type.
/// HIPAR result/Queries the statistical Power output value of all result ranges found
/// HIPAR result/in the entire measurement (corresponding to black area of statistics
/// HIPAR result/table).
ViStatus _VI_FUNC rsspecan_GetAmplifierStatisticsTablePowerOutputAverageAll(
                            ViSession instrSession,
                            ViInt32   measurementType,
                            ViReal64* result)
{
    ViStatus error = VI_SUCCESS;
    ViAttr   attr = 0;

    checkErr(RsCore_LockSession(instrSession));

    switch (measurementType)
    {
    case RSSPECAN_VAL_MEASTYPE_MIN:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_POWER_OUTPUT_AVERAGE_MINIMUM_ALL;
        break;
    case RSSPECAN_VAL_MEASTYPE_MAX:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_POWER_OUTPUT_AVERAGE_MAXIMUM_ALL;
        break;
    case RSSPECAN_VAL_MEASTYPE_SDEV:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_POWER_OUTPUT_AVERAGE_STANDARD_DEVIATION_ALL;
        break;
    case RSSPECAN_VAL_MEASTYPE_AVER:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_POWER_OUTPUT_AVERAGE_AVERAGE_ALL;
        break;
    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, measurementType), 2, "Measurement Type");
        break;
    }

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, NULL, attr, result),
            3, "Result");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the statistical Power Output value of all result ranges found
/// HIFN  in the entire measurement (corresponding to black area of statistics
/// HIFN  table).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR measurementType/Selects measurement type.
/// HIPAR result/Queries the statistical Power output value of all result ranges found
/// HIPAR result/in the entire measurement (corresponding to black area of statistics
/// HIPAR result/table).
ViStatus _VI_FUNC rsspecan_GetAmplifierStatisticsTablePowerOutputAverageCurrent(
                            ViSession instrSession,
                            ViInt32   measurementType,
                            ViReal64* result)
{
    ViStatus error = VI_SUCCESS;
    ViAttr   attr = 0;

    checkErr(RsCore_LockSession(instrSession));

    switch (measurementType)
    {
    case RSSPECAN_VAL_MEASTYPE_MIN:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_POWER_OUTPUT_AVERAGE_MINIMUM_CURRENT;
        break;
    case RSSPECAN_VAL_MEASTYPE_MAX:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_POWER_OUTPUT_AVERAGE_MAXIMUM_CURRENT;
        break;
    case RSSPECAN_VAL_MEASTYPE_SDEV:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_POWER_OUTPUT_AVERAGE_STANDARD_DEVIATION_CURRENT;
        break;
    case RSSPECAN_VAL_MEASTYPE_AVER:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_POWER_OUTPUT_AVERAGE_AVERAGE_CURRENT;
        break;
    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, measurementType), 2, "Measurement Type");
        break;
    }

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, NULL, attr, result),
            3, "Result");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the Power Output result of the currently selected result range
/// HIFN  (corresponds to blue area of statistics table).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR result/Queries the Power Output result of the currently selected result range
/// HIPAR result/(corresponds to blue area of statistics table).
ViStatus _VI_FUNC rsspecan_GetAmplifierStatisticsTablePowerOutputAverageSelectedResult(
                            ViSession instrSession,
                            ViReal64* result)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, NULL, RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_POWER_OUTPUT_AVERAGE_SELECTED_RESULT, result),
            2, "Result");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the statistical Power output value of all result ranges found
/// HIFN  in the entire measurement (corresponding to black area of statistics
/// HIFN  table).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR measurementType/Selects measurement type.
/// HIPAR result/Queries the statistical Power output value of all result ranges found
/// HIPAR result/in the entire measurement (corresponding to black area of statistics
/// HIPAR result/table).
ViStatus _VI_FUNC rsspecan_GetAmplifierStatisticsTablePowerOutputMaximumAll(
                            ViSession instrSession,
                            ViInt32   measurementType,
                            ViReal64* result)
{
    ViStatus error = VI_SUCCESS;
    ViAttr   attr = 0;

    checkErr(RsCore_LockSession(instrSession));

    switch (measurementType)
    {
    case RSSPECAN_VAL_MEASTYPE_MIN:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_POWER_OUTPUT_MAXIMUM_MINIMUM_ALL;
        break;
    case RSSPECAN_VAL_MEASTYPE_MAX:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_POWER_OUTPUT_MAXIMUM_MAXIMUM_ALL;
        break;
    case RSSPECAN_VAL_MEASTYPE_SDEV:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_POWER_OUTPUT_MAXIMUM_STANDARD_DEVIATION_ALL;
        break;
    case RSSPECAN_VAL_MEASTYPE_AVER:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_POWER_OUTPUT_MAXIMUM_AVERAGE_ALL;
        break;
    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, measurementType), 2, "Measurement Type");
        break;
    }

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, NULL, attr, result),
            3, "Result");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the statistical Power output value of all result ranges found
/// HIFN  in the entire measurement (corresponding to black area of statistics
/// HIFN  table).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR measurementType/Selects measurement type.
/// HIPAR result/Queries the statistical Power output value of all result ranges found
/// HIPAR result/in the entire measurement (corresponding to black area of statistics
/// HIPAR result/table).
ViStatus _VI_FUNC rsspecan_GetAmplifierStatisticsTablePowerOuputMaximumCurrent(
                            ViSession instrSession,
                            ViInt32   measurementType,
                            ViReal64* result)
{
    ViStatus error = VI_SUCCESS;
    ViAttr   attr = 0;

    checkErr(RsCore_LockSession(instrSession));

    switch (measurementType)
    {
    case RSSPECAN_VAL_MEASTYPE_MIN:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_POWER_OUTPUT_MAXIMUM_MINIMUM_CURRENT;
        break;
    case RSSPECAN_VAL_MEASTYPE_MAX:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_POWER_OUTPUT_MAXIMUM_MAXIMUM_CURRENT;
        break;
    case RSSPECAN_VAL_MEASTYPE_SDEV:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_POWER_OUTPUT_MAXIMUM_STANDARD_DEVIATION_CURRENT;
        break;
    case RSSPECAN_VAL_MEASTYPE_AVER:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_POWER_OUTPUT_MAXIMUM_AVERAGE_CURRENT;
        break;
    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, measurementType), 2, "Measurement Type");
        break;
    }

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, NULL, attr, result),
            3, "Result");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the Power Output result of the currently selected result range
/// HIFN  (corresponds to blue area of statistics table).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR result/Queries the Power Output result of the currently selected result range
/// HIPAR result/(corresponds to blue area of statistics table).
ViStatus _VI_FUNC rsspecan_GetAmplifierStatisticsTablePowerOutputMaximumSelectedResult(
                            ViSession instrSession,
                            ViReal64* result)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, NULL, RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_POWER_OUTPUT_MAXIMUM_SELECTED_RESULT, result),
            2, "Result");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the statistical Power output value of all result ranges found
/// HIFN  in the entire measurement (corresponding to black area of statistics
/// HIFN  table).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR measurementType/Selects measurement type.
/// HIPAR result/Queries the statistical Power output value of all result ranges found
/// HIPAR result/in the entire measurement (corresponding to black area of statistics
/// HIPAR result/table).
ViStatus _VI_FUNC rsspecan_GetAmplifierStatisticsTablePowerOutputMinimumAll(
                            ViSession instrSession,
                            ViInt32   measurementType,
                            ViReal64* result)
{
    ViStatus error = VI_SUCCESS;
    ViAttr   attr = 0;

    checkErr(RsCore_LockSession(instrSession));

    switch (measurementType)
    {
    case RSSPECAN_VAL_MEASTYPE_MIN:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_POWER_OUTPUT_MINIMUM_MINIMUM_ALL;
        break;
    case RSSPECAN_VAL_MEASTYPE_MAX:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_POWER_OUTPUT_MINIMUM_MAXIMUM_ALL;
        break;
    case RSSPECAN_VAL_MEASTYPE_SDEV:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_POWER_OUTPUT_MINIMUM_STANDARD_DEVIATION_ALL;
        break;
    case RSSPECAN_VAL_MEASTYPE_AVER:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_POWER_OUTPUT_MINIMUM_AVERAGE_ALL;
        break;
    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, measurementType), 2, "Measurement Type");
        break;
    }

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, NULL, attr, result),
            3, "Result");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the statistical Power output value of all result ranges found
/// HIFN  in the entire measurement (corresponding to black area of statistics
/// HIFN  table).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR measurementType/Selects measurement type.
/// HIPAR result/Queries the statistical Power output value of all result ranges found
/// HIPAR result/in the entire measurement (corresponding to black area of statistics
/// HIPAR result/table).
ViStatus _VI_FUNC rsspecan_GetAmplifierStatisticsTablePowerOutputMinimumCurrent(
                            ViSession instrSession,
                            ViInt32   measurementType,
                            ViReal64* result)
{
    ViStatus error = VI_SUCCESS;
    ViAttr   attr = 0;

    checkErr(RsCore_LockSession(instrSession));

    switch (measurementType)
    {
    case RSSPECAN_VAL_MEASTYPE_MIN:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_POWER_OUTPUT_MINIMUM_MINIMUM_CURRENT;
        break;
    case RSSPECAN_VAL_MEASTYPE_MAX:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_POWER_OUTPUT_MINIMUM_MAXIMUM_CURRENT;
        break;
    case RSSPECAN_VAL_MEASTYPE_SDEV:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_POWER_OUTPUT_MINIMUM_STANDARD_DEVIATION_CURRENT;
        break;
    case RSSPECAN_VAL_MEASTYPE_AVER:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_POWER_OUTPUT_MINIMUM_AVERAGE_CURRENT;
        break;
    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, measurementType), 2, "Measurement Type");
        break;
    }

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, NULL, attr, result),
            3, "Result");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the Power Output result of the currently selected result range
/// HIFN  (corresponds to blue area of statistics table).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR result/Queries the Power Output result of the currently selected result range
/// HIPAR result/(corresponds to blue area of statistics table).
ViStatus _VI_FUNC rsspecan_GetAmplifierStatisticsTablePowerOutputMinimumSelectedResult(
                            ViSession instrSession,
                            ViReal64* result)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, NULL, RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_POWER_OUTPUT_MINIMUM_SELECTED_RESULT, result),
            2, "Result");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the statistical Quadrature Error value of all result ranges
/// HIFN  found in the entire measurement (corresponding to black area of
/// HIFN  statistics table).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR measurementType/Selects measurement type.
/// HIPAR result/Queries the statistical Quadrature Error value of all result ranges
/// HIPAR result/found in the entire measurement (corresponding to black area of
/// HIPAR result/statistics table).
ViStatus _VI_FUNC rsspecan_GetAmplifierStatisticsTableQuadratureErrorAll(
                            ViSession instrSession,
                            ViInt32   measurementType,
                            ViReal64* result)
{
    ViStatus error = VI_SUCCESS;
    ViAttr   attr = 0;

    checkErr(RsCore_LockSession(instrSession));

    switch (measurementType)
    {
    case RSSPECAN_VAL_MEASTYPE_MIN:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_QUADRATURE_ERROR_MINIMUM_ALL;
        break;
    case RSSPECAN_VAL_MEASTYPE_MAX:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_QUADRATURE_ERROR_MAXIMUM_ALL;
        break;
    case RSSPECAN_VAL_MEASTYPE_SDEV:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_QUADRATURE_ERROR_STANDARD_DEVIATION_ALL;
        break;
    case RSSPECAN_VAL_MEASTYPE_AVER:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_QUADRATURE_ERROR_AVERAGE_ALL;
        break;
    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, measurementType), 2, "Measurement Type");
        break;
    }

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, NULL, attr, result),
            3, "Result");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the statistical Quadrature Error value of all result ranges
/// HIFN  found in the entire measurement (corresponding to black area of
/// HIFN  statistics table).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR measurementType/Selects measurement type.
/// HIPAR result/Queries the statistical Quadrature Error value of all result ranges
/// HIPAR result/found in the entire measurement (corresponding to black area of
/// HIPAR result/statistics table).
ViStatus _VI_FUNC rsspecan_GetAmplifierStatisticsTableQuadratureErrorCurrent(
                            ViSession instrSession,
                            ViInt32   measurementType,
                            ViReal64* result)
{
    ViStatus error = VI_SUCCESS;
    ViAttr   attr = 0;

    checkErr(RsCore_LockSession(instrSession));

    switch (measurementType)
    {
    case RSSPECAN_VAL_MEASTYPE_MIN:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_QUADRATURE_ERROR_MINIMUM_CURRENT;
        break;
    case RSSPECAN_VAL_MEASTYPE_MAX:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_QUADRATURE_ERROR_MAXIMUM_CURRENT;
        break;
    case RSSPECAN_VAL_MEASTYPE_SDEV:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_QUADRATURE_ERROR_STANDARD_DEVIATION_CURRENT;
        break;
    case RSSPECAN_VAL_MEASTYPE_AVER:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_QUADRATURE_ERROR_AVERAGE_CURRENT;
        break;
    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, measurementType), 2, "Measurement Type");
        break;
    }

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, NULL, attr, result),
            3, "Result");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the Quadrature Error result of the currently selected result
/// HIFN  range (corresponds to blue area of statistics table).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR result/Queries the Quadrature Error result of the currently selected result
/// HIPAR result/range (corresponds to blue area of statistics table).
ViStatus _VI_FUNC rsspecan_GetAmplifierStatisticsTableQuadratureErrorSelectedResult(
                            ViSession instrSession,
                            ViReal64* result)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, NULL, RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_QUADRATURE_ERROR_SELECTED_RESULT, result),
            2, "Result");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the statistical Baseband Q Input Voltage value of all result
/// HIFN  ranges found in the entire measurement (corresponding to black area of
/// HIFN  statistics table).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR measurementType/Selects measurement type.
/// HIPAR result/Queries the statistical Baseband Q Input Voltage value of all result
/// HIPAR result/ranges found in the entire measurement (corresponding to black area of
/// HIPAR result/statistics table).
ViStatus _VI_FUNC rsspecan_GetAmplifierStatisticsTableBasebandQInputVoltageAverageAll(
                            ViSession instrSession,
                            ViInt32   measurementType,
                            ViReal64* result)
{
    ViStatus error = VI_SUCCESS;
    ViAttr   attr = 0;

    checkErr(RsCore_LockSession(instrSession));

    switch (measurementType)
    {
    case RSSPECAN_VAL_MEASTYPE_MIN:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_BASEBAND_Q_INPUT_VOLTAGE_AVERAGE_MINIMUM_ALL;
        break;
    case RSSPECAN_VAL_MEASTYPE_MAX:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_BASEBAND_Q_INPUT_VOLTAGE_AVERAGE_MAXIMUM_ALL;
        break;
    case RSSPECAN_VAL_MEASTYPE_SDEV:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_BASEBAND_Q_INPUT_VOLTAGE_AVERAGE_STANDARD_DEVIATION_ALL;
        break;
    case RSSPECAN_VAL_MEASTYPE_AVER:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_BASEBAND_Q_INPUT_VOLTAGE_AVERAGE_AVERAGE_ALL;
        break;
    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, measurementType), 2, "Measurement Type");
        break;
    }

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, NULL, attr, result),
            3, "Result");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the statistical Baseband Q Input Voltage value of all result
/// HIFN  ranges found in the entire measurement (corresponding to black area of
/// HIFN  statistics table).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR measurementType/Selects measurement type.
/// HIPAR result/Queries the statistical Baseband Q Input Voltage value of all result
/// HIPAR result/ranges found in the entire measurement (corresponding to black area of
/// HIPAR result/statistics table).
ViStatus _VI_FUNC rsspecan_GetAmplifierStatisticsTableBasebandQInputVoltageAverageCurrent(
                            ViSession instrSession,
                            ViInt32   measurementType,
                            ViReal64* result)
{
    ViStatus error = VI_SUCCESS;
    ViAttr   attr = 0;

    checkErr(RsCore_LockSession(instrSession));

    switch (measurementType)
    {
    case RSSPECAN_VAL_MEASTYPE_MIN:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_BASEBAND_Q_INPUT_VOLTAGE_AVERAGE_MINIMUM_CURRENT;
        break;
    case RSSPECAN_VAL_MEASTYPE_MAX:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_BASEBAND_Q_INPUT_VOLTAGE_AVERAGE_MAXIMUM_CURRENT;
        break;
    case RSSPECAN_VAL_MEASTYPE_SDEV:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_BASEBAND_Q_INPUT_VOLTAGE_AVERAGE_STANDARD_DEVIATION_CURRENT;
        break;
    case RSSPECAN_VAL_MEASTYPE_AVER:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_BASEBAND_Q_INPUT_VOLTAGE_AVERAGE_AVERAGE_CURRENT;
        break;
    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, measurementType), 2, "Measurement Type");
        break;
    }

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, NULL, attr, result),
            3, "Result");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the Baseband Q Input Voltage result of the currently selected
/// HIFN  result range (corresponds to blue area of statistics table).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR result/Queries the Baseband Q Input Voltage result of the currently selected
/// HIPAR result/result range (corresponds to blue area of statistics table).
ViStatus _VI_FUNC rsspecan_GetAmplifierStatisticsTableBasebandQInputVoltageAverageSelectedResult(
                            ViSession instrSession,
                            ViReal64* result)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, NULL, RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_BASEBAND_Q_INPUT_VOLTAGE_AVERAGE_SELECTED_RESULT, result),
            2, "Result");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the statistical Baseband Q Input Voltage value of all result
/// HIFN  ranges found in the entire measurement (corresponding to black area of
/// HIFN  statistics table).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR measurementType/Selects measurement type.
/// HIPAR result/Queries the statistical Baseband Q Input Voltage value of all result
/// HIPAR result/ranges found in the entire measurement (corresponding to black area of
/// HIPAR result/statistics table).
ViStatus _VI_FUNC rsspecan_GetAmplifierStatisticsTableBasebandQInputVoltageMaximumAll(
                            ViSession instrSession,
                            ViInt32   measurementType,
                            ViReal64* result)
{
    ViStatus error = VI_SUCCESS;
    ViAttr   attr = 0;

    checkErr(RsCore_LockSession(instrSession));

    switch (measurementType)
    {
    case RSSPECAN_VAL_MEASTYPE_MIN:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_BASEBAND_Q_INPUT_VOLTAGE_MAXIMUM_MINIMUM_ALL;
        break;
    case RSSPECAN_VAL_MEASTYPE_MAX:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_BASEBAND_Q_INPUT_VOLTAGE_MAXIMUM_MAXIMUM_ALL;
        break;
    case RSSPECAN_VAL_MEASTYPE_SDEV:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_BASEBAND_Q_INPUT_VOLTAGE_MAXIMUM_STANDARD_DEVIATION_ALL;
        break;
    case RSSPECAN_VAL_MEASTYPE_AVER:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_BASEBAND_Q_INPUT_VOLTAGE_MAXIMUM_AVERAGE_ALL;
        break;
    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, measurementType), 2, "Measurement Type");
        break;
    }

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, NULL, attr, result),
            3, "Result");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the statistical Baseband Q Input Voltage value of all result
/// HIFN  ranges found in the entire measurement (corresponding to black area of
/// HIFN  statistics table).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR measurementType/Selects measurement type.
/// HIPAR result/Queries the statistical Baseband Q Input Voltage value of all result
/// HIPAR result/ranges found in the entire measurement (corresponding to black area of
/// HIPAR result/statistics table).
ViStatus _VI_FUNC rsspecan_GetAmplifierStatisticsTableBasebandQInputVoltageMaximumCurrent(
                            ViSession instrSession,
                            ViInt32   measurementType,
                            ViReal64* result)
{
    ViStatus error = VI_SUCCESS;
    ViAttr   attr = 0;

    checkErr(RsCore_LockSession(instrSession));

    switch (measurementType)
    {
    case RSSPECAN_VAL_MEASTYPE_MIN:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_BASEBAND_Q_INPUT_VOLTAGE_MAXIMUM_MINIMUM_CURRENT;
        break;
    case RSSPECAN_VAL_MEASTYPE_MAX:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_BASEBAND_Q_INPUT_VOLTAGE_MAXIMUM_MAXIMUM_CURRENT;
        break;
    case RSSPECAN_VAL_MEASTYPE_SDEV:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_BASEBAND_Q_INPUT_VOLTAGE_MAXIMUM_STANDARD_DEVIATION_CURRENT;
        break;
    case RSSPECAN_VAL_MEASTYPE_AVER:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_BASEBAND_Q_INPUT_VOLTAGE_MAXIMUM_AVERAGE_CURRENT;
        break;
    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, measurementType), 2, "Measurement Type");
        break;
    }

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, NULL, attr, result),
            3, "Result");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the Baseband Q Input Voltage result of the currently selected
/// HIFN  result range (corresponds to blue area of statistics table).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR result/Queries the Baseband Q Input Voltage result of the currently selected
/// HIPAR result/result range (corresponds to blue area of statistics table).
ViStatus _VI_FUNC rsspecan_GetAmplifierStatisticsTableBasebandQInputVoltageMaximumSelectedResult(
                            ViSession instrSession,
                            ViReal64* result)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, NULL, RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_BASEBAND_Q_INPUT_VOLTAGE_MAXIMUM_SELECTED_RESULT, result),
            2, "Result");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the statistical Baseband Q Input Voltage value of all result
/// HIFN  ranges found in the entire measurement (corresponding to black area of
/// HIFN  statistics table).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR measurementType/Selects measurement type.
/// HIPAR result/Queries the statistical Baseband Q Input Voltage value of all result
/// HIPAR result/ranges found in the entire measurement (corresponding to black area of
/// HIPAR result/statistics table).
ViStatus _VI_FUNC rsspecan_GetAmplifierStatisticsTableBasebandQInputVoltageMinimumAll(
                            ViSession instrSession,
                            ViInt32   measurementType,
                            ViReal64* result)
{
    ViStatus error = VI_SUCCESS;
    ViAttr   attr = 0;

    checkErr(RsCore_LockSession(instrSession));

    switch (measurementType)
    {
    case RSSPECAN_VAL_MEASTYPE_MIN:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_BASEBAND_Q_INPUT_VOLTAGE_MINIMUM_MINIMUM_ALL;
        break;
    case RSSPECAN_VAL_MEASTYPE_MAX:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_BASEBAND_Q_INPUT_VOLTAGE_MINIMUM_MAXIMUM_ALL;
        break;
    case RSSPECAN_VAL_MEASTYPE_SDEV:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_BASEBAND_Q_INPUT_VOLTAGE_MINIMUM_STANDARD_DEVIATION_ALL;
        break;
    case RSSPECAN_VAL_MEASTYPE_AVER:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_BASEBAND_Q_INPUT_VOLTAGE_MINIMUM_AVERAGE_ALL;
        break;
    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, measurementType), 2, "Measurement Type");
        break;
    }

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, NULL, attr, result),
            3, "Result");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the statistical Baseband Q Input Voltage value of all result
/// HIFN  ranges found in the entire measurement (corresponding to black area of
/// HIFN  statistics table).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR measurementType/Selects measurement type.
/// HIPAR result/Queries the statistical Baseband Q Input Voltage value of all result
/// HIPAR result/ranges found in the entire measurement (corresponding to black area of
/// HIPAR result/statistics table).
ViStatus _VI_FUNC rsspecan_GetAmplifierStatisticsTableBasebandQInputVoltageMinimumCurrent(
                            ViSession instrSession,
                            ViInt32   measurementType,
                            ViReal64* result)
{
    ViStatus error = VI_SUCCESS;
    ViAttr   attr = 0;

    checkErr(RsCore_LockSession(instrSession));

    switch (measurementType)
    {
    case RSSPECAN_VAL_MEASTYPE_MIN:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_BASEBAND_Q_INPUT_VOLTAGE_MINIMUM_MINIMUM_CURRENT;
        break;
    case RSSPECAN_VAL_MEASTYPE_MAX:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_BASEBAND_Q_INPUT_VOLTAGE_MINIMUM_MAXIMUM_CURRENT;
        break;
    case RSSPECAN_VAL_MEASTYPE_SDEV:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_BASEBAND_Q_INPUT_VOLTAGE_MINIMUM_STANDARD_DEVIATION_CURRENT;
        break;
    case RSSPECAN_VAL_MEASTYPE_AVER:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_BASEBAND_Q_INPUT_VOLTAGE_MINIMUM_AVERAGE_CURRENT;
        break;
    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, measurementType), 2, "Measurement Type");
        break;
    }

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, NULL, attr, result),
            3, "Result");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the Baseband Q Input Voltage result of the currently selected
/// HIFN  result range (corresponds to blue area of statistics table).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR result/Queries the Baseband Q Input Voltage result of the currently selected
/// HIPAR result/result range (corresponds to blue area of statistics table).
ViStatus _VI_FUNC rsspecan_GetAmplifierStatisticsTableBasebandQInputVoltageMinimumSelectedResult(
                            ViSession instrSession,
                            ViReal64* result)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, NULL, RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_BASEBAND_Q_INPUT_VOLTAGE_MINIMUM_SELECTED_RESULT, result),
            2, "Result");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the statistical Raw EVM value of all result ranges found in
/// HIFN  the entire measurement (corresponding to black area of statistics
/// HIFN  table).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR measurementType/Selects measurement type.
/// HIPAR result/Queries the statistical Raw EVM value of all result ranges found in
/// HIPAR result/the entire measurement (corresponding to black area of statistics
/// HIPAR result/table).
ViStatus _VI_FUNC rsspecan_GetAmplifierStatisticsTableRawEVMAverageAll(
                            ViSession instrSession,
                            ViInt32   measurementType,
                            ViReal64* result)
{
    ViStatus error = VI_SUCCESS;
    ViAttr   attr = 0;

    checkErr(RsCore_LockSession(instrSession));

    switch (measurementType)
    {
    case RSSPECAN_VAL_MEASTYPE_MIN:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_RAW_EVM_AVERAGE_MINIMUM_ALL;
        break;
    case RSSPECAN_VAL_MEASTYPE_MAX:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_RAW_EVM_AVERAGE_MAXIMUM_ALL;
        break;
    case RSSPECAN_VAL_MEASTYPE_SDEV:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_RAW_EVM_AVERAGE_STANDARD_DEVIATION_ALL;
        break;
    case RSSPECAN_VAL_MEASTYPE_AVER:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_RAW_EVM_AVERAGE_AVERAGE_ALL;
        break;
    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, measurementType), 2, "Measurement Type");
        break;
    }

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, NULL, attr, result),
            3, "Result");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the statistical Raw EVM value of all result ranges found in
/// HIFN  the entire measurement (corresponding to black area of statistics
/// HIFN  table).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR measurementType/Selects measurement type.
/// HIPAR result/Queries the statistical Raw EVM value of all result ranges found in
/// HIPAR result/the entire measurement (corresponding to black area of statistics
/// HIPAR result/table).
ViStatus _VI_FUNC rsspecan_GetAmplifierStatisticsTableRawEVMAverageCurrent(
                            ViSession instrSession,
                            ViInt32   measurementType,
                            ViReal64* result)
{
    ViStatus error = VI_SUCCESS;
    ViAttr   attr = 0;

    checkErr(RsCore_LockSession(instrSession));

    switch (measurementType)
    {
    case RSSPECAN_VAL_MEASTYPE_MIN:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_RAW_EVM_AVERAGE_MINIMUM_CURRENT;
        break;
    case RSSPECAN_VAL_MEASTYPE_MAX:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_RAW_EVM_AVERAGE_MAXIMUM_CURRENT;
        break;
    case RSSPECAN_VAL_MEASTYPE_SDEV:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_RAW_EVM_AVERAGE_STANDARD_DEVIATION_CURRENT;
        break;
    case RSSPECAN_VAL_MEASTYPE_AVER:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_RAW_EVM_AVERAGE_AVERAGE_CURRENT;
        break;
    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, measurementType), 2, "Measurement Type");
        break;
    }

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, NULL, attr, result),
            3, "Result");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the Raw EVM result of the currently selected result range
/// HIFN  (corresponds to blue area of statistics table).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR result/Queries the Raw EVM result of the currently selected result range
/// HIPAR result/(corresponds to blue area of statistics table).
ViStatus _VI_FUNC rsspecan_GetAmplifierStatisticsTableRawEVMAverageSelectedResult(
                            ViSession instrSession,
                            ViReal64* result)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, NULL, RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_RAW_EVM_AVERAGE_SELECTED_RESULT, result),
            2, "Result");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the statistical Raw EVM value of all result ranges found in
/// HIFN  the entire measurement (corresponding to black area of statistics
/// HIFN  table).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR measurementType/Selects measurement type.
/// HIPAR result/Queries the statistical Raw EVM value of all result ranges found in
/// HIPAR result/the entire measurement (corresponding to black area of statistics
/// HIPAR result/table).
ViStatus _VI_FUNC rsspecan_GetAmplifierStatisticsTableRawEVMMaximumAll(
                            ViSession instrSession,
                            ViInt32   measurementType,
                            ViReal64* result)
{
    ViStatus error = VI_SUCCESS;
    ViAttr   attr = 0;

    checkErr(RsCore_LockSession(instrSession));

    switch (measurementType)
    {
    case RSSPECAN_VAL_MEASTYPE_MIN:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_RAW_EVM_MAXIMUM_MINIMUM_ALL;
        break;
    case RSSPECAN_VAL_MEASTYPE_MAX:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_RAW_EVM_MAXIMUM_MAXIMUM_ALL;
        break;
    case RSSPECAN_VAL_MEASTYPE_SDEV:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_RAW_EVM_MAXIMUM_STANDARD_DEVIATION_ALL;
        break;
    case RSSPECAN_VAL_MEASTYPE_AVER:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_RAW_EVM_MAXIMUM_AVERAGE_ALL;
        break;
    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, measurementType), 2, "Measurement Type");
        break;
    }

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, NULL, attr, result),
            3, "Result");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the statistical Raw EVM value of all result ranges found in
/// HIFN  the entire measurement (corresponding to black area of statistics
/// HIFN  table).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR measurementType/Selects measurement type.
/// HIPAR result/Queries the statistical Raw EVM value of all result ranges found in
/// HIPAR result/the entire measurement (corresponding to black area of statistics
/// HIPAR result/table).
ViStatus _VI_FUNC rsspecan_GetAmplifierStatisticsTableRawEVMMaximumCurrent(
                            ViSession instrSession,
                            ViInt32   measurementType,
                            ViReal64* result)
{
    ViStatus error = VI_SUCCESS;
    ViAttr   attr = 0;

    checkErr(RsCore_LockSession(instrSession));

    switch (measurementType)
    {
    case RSSPECAN_VAL_MEASTYPE_MIN:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_RAW_EVM_MAXIMUM_MINIMUM_CURRENT;
        break;
    case RSSPECAN_VAL_MEASTYPE_MAX:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_RAW_EVM_MAXIMUM_MAXIMUM_CURRENT;
        break;
    case RSSPECAN_VAL_MEASTYPE_SDEV:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_RAW_EVM_MAXIMUM_STANDARD_DEVIATION_CURRENT;
        break;
    case RSSPECAN_VAL_MEASTYPE_AVER:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_RAW_EVM_MAXIMUM_AVERAGE_CURRENT;
        break;
    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, measurementType), 2, "Measurement Type");
        break;
    }

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, NULL, attr, result),
            3, "Result");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the Raw EVM result of the currently selected result range
/// HIFN  (corresponds to blue area of statistics table).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR result/Queries the Raw EVM result of the currently selected result range
/// HIPAR result/(corresponds to blue area of statistics table).
ViStatus _VI_FUNC rsspecan_GetAmplifierStatisticsTableRawEVMMaximumSelectedResult(
                            ViSession instrSession,
                            ViReal64* result)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, NULL, RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_RAW_EVM_MAXIMUM_SELECTED_RESULT, result),
            2, "Result");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the statistical Raw EVM value of all result ranges found in
/// HIFN  the entire measurement (corresponding to black area of statistics
/// HIFN  table).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR measurementType/Selects measurement type.
/// HIPAR result/Queries the statistical Raw EVM value of all result ranges found in
/// HIPAR result/the entire measurement (corresponding to black area of statistics
/// HIPAR result/table).
ViStatus _VI_FUNC rsspecan_GetAmplifierStatisticsTableRawEVMMinimumAll(
                            ViSession instrSession,
                            ViInt32   measurementType,
                            ViReal64* result)
{
    ViStatus error = VI_SUCCESS;
    ViAttr   attr = 0;

    checkErr(RsCore_LockSession(instrSession));

    switch (measurementType)
    {
    case RSSPECAN_VAL_MEASTYPE_MIN:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_RAW_EVM_MINIMUM_MINIMUM_ALL;
        break;
    case RSSPECAN_VAL_MEASTYPE_MAX:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_RAW_EVM_MINIMUM_MAXIMUM_ALL;
        break;
    case RSSPECAN_VAL_MEASTYPE_SDEV:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_RAW_EVM_MINIMUM_STANDARD_DEVIATION_ALL;
        break;
    case RSSPECAN_VAL_MEASTYPE_AVER:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_RAW_EVM_MINIMUM_AVERAGE_ALL;
        break;
    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, measurementType), 2, "Measurement Type");
        break;
    }

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, NULL, attr, result),
            3, "Result");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the statistical Raw EVM value of all result ranges found in
/// HIFN  the entire measurement (corresponding to black area of statistics
/// HIFN  table).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR measurementType/Selects measurement type.
/// HIPAR result/Queries the statistical Raw EVM value of all result ranges found in
/// HIPAR result/the entire measurement (corresponding to black area of statistics
/// HIPAR result/table).
ViStatus _VI_FUNC rsspecan_GetAmplifierStatisticsTableRawEVMMinimumCurrent(
                            ViSession instrSession,
                            ViInt32   measurementType,
                            ViReal64* result)
{
    ViStatus error = VI_SUCCESS;
    ViAttr   attr = 0;

    checkErr(RsCore_LockSession(instrSession));

    switch (measurementType)
    {
    case RSSPECAN_VAL_MEASTYPE_MIN:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_RAW_EVM_MINIMUM_MINIMUM_CURRENT;
        break;
    case RSSPECAN_VAL_MEASTYPE_MAX:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_RAW_EVM_MINIMUM_MAXIMUM_CURRENT;
        break;
    case RSSPECAN_VAL_MEASTYPE_SDEV:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_RAW_EVM_MINIMUM_STANDARD_DEVIATION_CURRENT;
        break;
    case RSSPECAN_VAL_MEASTYPE_AVER:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_RAW_EVM_MINIMUM_AVERAGE_CURRENT;
        break;
    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, measurementType), 2, "Measurement Type");
        break;
    }

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, NULL, attr, result),
            3, "Result");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the Raw EVM result of the currently selected result range
/// HIFN  (corresponds to blue area of statistics table).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR result/Queries the Raw EVM result of the currently selected result range
/// HIPAR result/(corresponds to blue area of statistics table).
ViStatus _VI_FUNC rsspecan_GetAmplifierStatisticsTableRawEVMMinimumSelectedResult(
                            ViSession instrSession,
                            ViReal64* result)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, NULL, RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_RAW_EVM_MINIMUM_SELECTED_RESULT, result),
            2, "Result");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the statistical Raw Model EVM value of all result ranges found
/// HIFN  in the entire measurement (corresponding to black area of statistics
/// HIFN  table).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR measurementType/Selects measurement type.
/// HIPAR result/Queries the statistical Raw Model EVM value of all result ranges found
/// HIPAR result/in the entire measurement (corresponding to black area of statistics
/// HIPAR result/table).
ViStatus _VI_FUNC rsspecan_GetAmplifierStatisticsTableRawModelEVMAverageAll(
                            ViSession instrSession,
                            ViInt32   measurementType,
                            ViReal64* result)
{
    ViStatus error = VI_SUCCESS;
    ViAttr   attr = 0;

    checkErr(RsCore_LockSession(instrSession));

    switch (measurementType)
    {
    case RSSPECAN_VAL_MEASTYPE_MIN:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_RAW_MODEL_EVM_AVERAGE_MINIMUM_ALL;
        break;
    case RSSPECAN_VAL_MEASTYPE_MAX:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_RAW_MODEL_EVM_AVERAGE_MAXIMUM_ALL;
        break;
    case RSSPECAN_VAL_MEASTYPE_SDEV:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_RAW_MODEL_EVM_AVERAGE_STANDARD_DEVIATION_ALL;
        break;
    case RSSPECAN_VAL_MEASTYPE_AVER:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_RAW_MODEL_EVM_AVERAGE_AVERAGE_ALL;
        break;
    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, measurementType), 2, "Measurement Type");
        break;
    }

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, NULL, attr, result),
            3, "Result");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the statistical Raw Model EVM value of all result ranges found
/// HIFN  in the entire measurement (corresponding to black area of statistics
/// HIFN  table).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR measurementType/Selects measurement type.
/// HIPAR result/Queries the statistical Raw Model EVM value of all result ranges found
/// HIPAR result/in the entire measurement (corresponding to black area of statistics
/// HIPAR result/table).
ViStatus _VI_FUNC rsspecan_GetAmplifierStatisticsTableRawModelEVMAverageCurrent(
                            ViSession instrSession,
                            ViInt32   measurementType,
                            ViReal64* result)
{
    ViStatus error = VI_SUCCESS;
    ViAttr   attr = 0;

    checkErr(RsCore_LockSession(instrSession));

    switch (measurementType)
    {
    case RSSPECAN_VAL_MEASTYPE_MIN:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_RAW_MODEL_EVM_AVERAGE_MINIMUM_CURRENT;
        break;
    case RSSPECAN_VAL_MEASTYPE_MAX:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_RAW_MODEL_EVM_AVERAGE_MAXIMUM_CURRENT;
        break;
    case RSSPECAN_VAL_MEASTYPE_SDEV:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_RAW_MODEL_EVM_AVERAGE_STANDARD_DEVIATION_CURRENT;
        break;
    case RSSPECAN_VAL_MEASTYPE_AVER:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_RAW_MODEL_EVM_AVERAGE_AVERAGE_CURRENT;
        break;
    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, measurementType), 2, "Measurement Type");
        break;
    }

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, NULL, attr, result),
            3, "Result");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the Raw Model EVM result of the currently selected result
/// HIFN  range (corresponds to blue area of statistics table).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR result/Queries the Raw Model EVM result of the currently selected result
/// HIPAR result/range (corresponds to blue area of statistics table).
ViStatus _VI_FUNC rsspecan_GetAmplifierStatisticsTableRawModelEVMAverageSelectedResult(
                            ViSession instrSession,
                            ViReal64* result)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, NULL, RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_RAW_MODEL_EVM_AVERAGE_SELECTED_RESULT, result),
            2, "Result");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the statistical Raw Model EVM value of all result ranges found
/// HIFN  in the entire measurement (corresponding to black area of statistics
/// HIFN  table).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR measurementType/Selects measurement type.
/// HIPAR result/Queries the statistical Raw Model EVM value of all result ranges found
/// HIPAR result/in the entire measurement (corresponding to black area of statistics
/// HIPAR result/table).
ViStatus _VI_FUNC rsspecan_GetAmplifierStatisticsTableRawModelEVMMaximumAll(
                            ViSession instrSession,
                            ViInt32   measurementType,
                            ViReal64* result)
{
    ViStatus error = VI_SUCCESS;
    ViAttr   attr = 0;

    checkErr(RsCore_LockSession(instrSession));

    switch (measurementType)
    {
    case RSSPECAN_VAL_MEASTYPE_MIN:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_RAW_MODEL_EVM_MAXIMUM_MINIMUM_ALL;
        break;
    case RSSPECAN_VAL_MEASTYPE_MAX:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_RAW_MODEL_EVM_MAXIMUM_MAXIMUM_ALL;
        break;
    case RSSPECAN_VAL_MEASTYPE_SDEV:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_RAW_MODEL_EVM_MAXIMUM_STANDARD_DEVIATION_ALL;
        break;
    case RSSPECAN_VAL_MEASTYPE_AVER:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_RAW_MODEL_EVM_MAXIMUM_AVERAGE_ALL;
        break;
    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, measurementType), 2, "Measurement Type");
        break;
    }

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, NULL, attr, result),
            3, "Result");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the statistical Raw Model EVM value of all result ranges found
/// HIFN  in the entire measurement (corresponding to black area of statistics
/// HIFN  table).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR measurementType/Selects measurement type.
/// HIPAR result/Queries the statistical Raw Model EVM value of all result ranges found
/// HIPAR result/in the entire measurement (corresponding to black area of statistics
/// HIPAR result/table).
ViStatus _VI_FUNC rsspecan_GetAmplifierStatisticsTableRawModelEVMMaximumCurrent(
                            ViSession instrSession,
                            ViInt32   measurementType,
                            ViReal64* result)
{
    ViStatus error = VI_SUCCESS;
    ViAttr   attr = 0;

    checkErr(RsCore_LockSession(instrSession));

    switch (measurementType)
    {
    case RSSPECAN_VAL_MEASTYPE_MIN:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_RAW_MODEL_EVM_MAXIMUM_MINIMUM_CURRENT;
        break;
    case RSSPECAN_VAL_MEASTYPE_MAX:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_RAW_MODEL_EVM_MAXIMUM_MAXIMUM_CURRENT;
        break;
    case RSSPECAN_VAL_MEASTYPE_SDEV:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_RAW_MODEL_EVM_MAXIMUM_STANDARD_DEVIATION_CURRENT;
        break;
    case RSSPECAN_VAL_MEASTYPE_AVER:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_RAW_MODEL_EVM_MAXIMUM_AVERAGE_CURRENT;
        break;
    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, measurementType), 2, "Measurement Type");
        break;
    }

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, NULL, attr, result),
            3, "Result");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the Raw Model EVM result of the currently selected result
/// HIFN  range (corresponds to blue area of statistics table).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR result/Queries the Raw Model EVM result of the currently selected result
/// HIPAR result/range (corresponds to blue area of statistics table).
ViStatus _VI_FUNC rsspecan_GetAmplifierStatisticsTableRawModelEVMMaximumSelectedResult(
                            ViSession instrSession,
                            ViReal64* result)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, NULL, RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_RAW_MODEL_EVM_MAXIMUM_SELECTED_RESULT, result),
            2, "Result");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the statistical Raw Model EVM value of all result ranges found
/// HIFN  in the entire measurement (corresponding to black area of statistics
/// HIFN  table).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR measurementType/Selects measurement type.
/// HIPAR result/Queries the statistical Raw Model EVM value of all result ranges found
/// HIPAR result/in the entire measurement (corresponding to black area of statistics
/// HIPAR result/table).
ViStatus _VI_FUNC rsspecan_GetAmplifierStatisticsTableRawModelEVMMinimumAll(
                            ViSession instrSession,
                            ViInt32   measurementType,
                            ViReal64* result)
{
    ViStatus error = VI_SUCCESS;
    ViAttr   attr = 0;

    checkErr(RsCore_LockSession(instrSession));

    switch (measurementType)
    {
    case RSSPECAN_VAL_MEASTYPE_MIN:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_RAW_MODEL_EVM_MINIMUM_MINIMUM_ALL;
        break;
    case RSSPECAN_VAL_MEASTYPE_MAX:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_RAW_MODEL_EVM_MINIMUM_MAXIMUM_ALL;
        break;
    case RSSPECAN_VAL_MEASTYPE_SDEV:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_RAW_MODEL_EVM_MINIMUM_STANDARD_DEVIATION_ALL;
        break;
    case RSSPECAN_VAL_MEASTYPE_AVER:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_RAW_MODEL_EVM_MINIMUM_AVERAGE_ALL;
        break;
    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, measurementType), 2, "Measurement Type");
        break;
    }

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, NULL, attr, result),
            3, "Result");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the statistical Raw Model EVM value of all result ranges found
/// HIFN  in the entire measurement (corresponding to black area of statistics
/// HIFN  table).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR measurementType/Selects measurement type.
/// HIPAR result/Queries the statistical Raw Model EVM value of all result ranges found
/// HIPAR result/in the entire measurement (corresponding to black area of statistics
/// HIPAR result/table).
ViStatus _VI_FUNC rsspecan_GetAmplifierStatisticsTableRawModelEVMMinimumCurrent(
                            ViSession instrSession,
                            ViInt32   measurementType,
                            ViReal64* result)
{
    ViStatus error = VI_SUCCESS;
    ViAttr   attr = 0;

    checkErr(RsCore_LockSession(instrSession));

    switch (measurementType)
    {
    case RSSPECAN_VAL_MEASTYPE_MIN:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_RAW_MODEL_EVM_MINIMUM_MINIMUM_CURRENT;
        break;
    case RSSPECAN_VAL_MEASTYPE_MAX:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_RAW_MODEL_EVM_MINIMUM_MAXIMUM_CURRENT;
        break;
    case RSSPECAN_VAL_MEASTYPE_SDEV:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_RAW_MODEL_EVM_MINIMUM_STANDARD_DEVIATION_CURRENT;
        break;
    case RSSPECAN_VAL_MEASTYPE_AVER:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_RAW_MODEL_EVM_MINIMUM_AVERAGE_CURRENT;
        break;
    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, measurementType), 2, "Measurement Type");
        break;
    }

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, NULL, attr, result),
            3, "Result");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the Raw Model EVM result of the currently selected result
/// HIFN  range (corresponds to blue area of statistics table).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR result/Queries the Raw Model EVM result of the currently selected result
/// HIPAR result/range (corresponds to blue area of statistics table).
ViStatus _VI_FUNC rsspecan_GetAmplifierStatisticsTableRawModelEVMMinimumSelectedResult(
                            ViSession instrSession,
                            ViReal64* result)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, NULL, RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_RAW_MODEL_EVM_MINIMUM_SELECTED_RESULT, result),
            2, "Result");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the statistical Sample Rate Error value of all result ranges
/// HIFN  found in the entire measurement (corresponding to black area of
/// HIFN  statistics table).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR measurementType/Selects measurement type.
/// HIPAR result/Queries the statistical Sample Rate Error value of all result ranges
/// HIPAR result/found in the entire measurement (corresponding to black area of
/// HIPAR result/statistics table).
ViStatus _VI_FUNC rsspecan_GetAmplifierStatisticsTableSampleRateErrorAll(
                            ViSession instrSession,
                            ViInt32   measurementType,
                            ViReal64* result)
{
    ViStatus error = VI_SUCCESS;
    ViAttr   attr = 0;

    checkErr(RsCore_LockSession(instrSession));

    switch (measurementType)
    {
    case RSSPECAN_VAL_MEASTYPE_MIN:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_SAMPLE_RATE_ERROR_MINIMUM_ALL;
        break;
    case RSSPECAN_VAL_MEASTYPE_MAX:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_SAMPLE_RATE_ERROR_MAXIMUM_ALL;
        break;
    case RSSPECAN_VAL_MEASTYPE_SDEV:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_SAMPLE_RATE_ERROR_STANDARD_DEVIATION_ALL;
        break;
    case RSSPECAN_VAL_MEASTYPE_AVER:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_SAMPLE_RATE_ERROR_AVERAGE_ALL;
        break;
    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, measurementType), 2, "Measurement Type");
        break;
    }

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, NULL, attr, result),
            3, "Result");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the statistical Sample Rate Error value of all result ranges
/// HIFN  found in the entire measurement (corresponding to black area of
/// HIFN  statistics table).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR measurementType/Selects measurement type.
/// HIPAR result/Queries the statistical Sample Rate Error value of all result ranges
/// HIPAR result/found in the entire measurement (corresponding to black area of
/// HIPAR result/statistics table).
ViStatus _VI_FUNC rsspecan_GetAmplifierStatisticsTableSampleRateErrorCurrent(
                            ViSession instrSession,
                            ViInt32   measurementType,
                            ViReal64* result)
{
    ViStatus error = VI_SUCCESS;
    ViAttr   attr = 0;

    checkErr(RsCore_LockSession(instrSession));

    switch (measurementType)
    {
    case RSSPECAN_VAL_MEASTYPE_MIN:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_SAMPLE_RATE_ERROR_MINIMUM_CURRENT;
        break;
    case RSSPECAN_VAL_MEASTYPE_MAX:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_SAMPLE_RATE_ERROR_MAXIMUM_CURRENT;
        break;
    case RSSPECAN_VAL_MEASTYPE_SDEV:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_SAMPLE_RATE_ERROR_STANDARD_DEVIATION_CURRENT;
        break;
    case RSSPECAN_VAL_MEASTYPE_AVER:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_SAMPLE_RATE_ERROR_AVERAGE_CURRENT;
        break;
    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, measurementType), 2, "Measurement Type");
        break;
    }

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, NULL, attr, result),
            3, "Result");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the Sample Rate Error result of the currently selected result
/// HIFN  range (corresponds to blue area of statistics table).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR result/Queries the Sample Rate Error result of the currently selected result
/// HIPAR result/range (corresponds to blue area of statistics table).
ViStatus _VI_FUNC rsspecan_GetAmplifierStatisticsTableSampleRateErrorSelectedResult(
                            ViSession instrSession,
                            ViReal64* result)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, NULL, RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_SAMPLE_RATE_ERROR_SELECTED_RESULT, result),
            2, "Result");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the statistical Voltage (Vcc) value of all result ranges found
/// HIFN  in the entire measurement (corresponding to black area of statistics
/// HIFN  table).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR measurementType/Selects measurement type.
/// HIPAR result/Queries the statistical Voltage (Vcc) value of all result ranges found
/// HIPAR result/in the entire measurement (corresponding to black area of statistics
/// HIPAR result/table).
ViStatus _VI_FUNC rsspecan_GetAmplifierStatisticsTableVoltageVccAverageAll(
                            ViSession instrSession,
                            ViInt32   measurementType,
                            ViReal64* result)
{
    ViStatus error = VI_SUCCESS;
    ViAttr   attr = 0;

    checkErr(RsCore_LockSession(instrSession));

    switch (measurementType)
    {
    case RSSPECAN_VAL_MEASTYPE_MIN:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_VOLTAGE_VCC_AVERAGE_MINIMUM_ALL;
        break;
    case RSSPECAN_VAL_MEASTYPE_MAX:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_VOLTAGE_VCC_AVERAGE_MAXIMUM_ALL;
        break;
    case RSSPECAN_VAL_MEASTYPE_SDEV:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_VOLTAGE_VCC_AVERAGE_STANDARD_DEVIATION_ALL;
        break;
    case RSSPECAN_VAL_MEASTYPE_AVER:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_VOLTAGE_VCC_AVERAGE_AVERAGE_ALL;
        break;
    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, measurementType), 2, "Measurement Type");
        break;
    }

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, NULL, attr, result),
            3, "Result");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the statistical Voltage (Vcc) value of all result ranges found
/// HIFN  in the entire measurement (corresponding to black area of statistics
/// HIFN  table).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR measurementType/Selects measurement type.
/// HIPAR result/Queries the statistical Voltage (Vcc) value of all result ranges found
/// HIPAR result/in the entire measurement (corresponding to black area of statistics
/// HIPAR result/table).
ViStatus _VI_FUNC rsspecan_GetAmplifierStatisticsTableVoltageVccAverageCurrent(
                            ViSession instrSession,
                            ViInt32   measurementType,
                            ViReal64* result)
{
    ViStatus error = VI_SUCCESS;
    ViAttr   attr = 0;

    checkErr(RsCore_LockSession(instrSession));

    switch (measurementType)
    {
    case RSSPECAN_VAL_MEASTYPE_MIN:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_VOLTAGE_VCC_AVERAGE_MINIMUM_CURRENT;
        break;
    case RSSPECAN_VAL_MEASTYPE_MAX:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_VOLTAGE_VCC_AVERAGE_MAXIMUM_CURRENT;
        break;
    case RSSPECAN_VAL_MEASTYPE_SDEV:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_VOLTAGE_VCC_AVERAGE_STANDARD_DEVIATION_CURRENT;
        break;
    case RSSPECAN_VAL_MEASTYPE_AVER:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_VOLTAGE_VCC_AVERAGE_AVERAGE_CURRENT;
        break;
    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, measurementType), 2, "Measurement Type");
        break;
    }

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, NULL, attr, result),
            3, "Result");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the Voltage (Vcc) result of the currently selected result
/// HIFN  range (corresponds to blue area of statistics table).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR result/Queries the Voltage (Vcc) result of the currently selected result
/// HIPAR result/range (corresponds to blue area of statistics table).
ViStatus _VI_FUNC rsspecan_GetAmplifierStatisticsTableVoltageVccAverageSelectedResult(
                            ViSession instrSession,
                            ViReal64* result)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, NULL, RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_VOLTAGE_VCC_AVERAGE_SELECTED_RESULT, result),
            2, "Result");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the statistical Voltage (Vcc) value of all result ranges found
/// HIFN  in the entire measurement (corresponding to black area of statistics
/// HIFN  table).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR measurementType/Selects measurement type.
/// HIPAR result/Queries the statistical Voltage (Vcc) value of all result ranges found
/// HIPAR result/in the entire measurement (corresponding to black area of statistics
/// HIPAR result/table).
ViStatus _VI_FUNC rsspecan_GetAmplifierStatisticsTableVoltageVccMaximumAll(
                            ViSession instrSession,
                            ViInt32   measurementType,
                            ViReal64* result)
{
    ViStatus error = VI_SUCCESS;
    ViAttr   attr = 0;

    checkErr(RsCore_LockSession(instrSession));

    switch (measurementType)
    {
    case RSSPECAN_VAL_MEASTYPE_MIN:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_VOLTAGE_VCC_MAXIMUM_MINIMUM_ALL;
        break;
    case RSSPECAN_VAL_MEASTYPE_MAX:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_VOLTAGE_VCC_MAXIMUM_MAXIMUM_ALL;
        break;
    case RSSPECAN_VAL_MEASTYPE_SDEV:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_VOLTAGE_VCC_MAXIMUM_STANDARD_DEVIATION_ALL;
        break;
    case RSSPECAN_VAL_MEASTYPE_AVER:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_VOLTAGE_VCC_MAXIMUM_AVERAGE_ALL;
        break;
    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, measurementType), 2, "Measurement Type");
        break;
    }

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, NULL, attr, result),
            3, "Result");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the statistical Voltage (Vcc) value of all result ranges found
/// HIFN  in the entire measurement (corresponding to black area of statistics
/// HIFN  table).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR measurementType/Selects measurement type.
/// HIPAR result/Queries the statistical Voltage (Vcc) value of all result ranges found
/// HIPAR result/in the entire measurement (corresponding to black area of statistics
/// HIPAR result/table).
ViStatus _VI_FUNC rsspecan_GetAmplifierStatisticsTableVoltageVccMaximumCurrent(
                            ViSession instrSession,
                            ViInt32   measurementType,
                            ViReal64* result)
{
    ViStatus error = VI_SUCCESS;
    ViAttr   attr = 0;

    checkErr(RsCore_LockSession(instrSession));

    switch (measurementType)
    {
    case RSSPECAN_VAL_MEASTYPE_MIN:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_VOLTAGE_VCC_MAXIMUM_MINIMUM_CURRENT;
        break;
    case RSSPECAN_VAL_MEASTYPE_MAX:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_VOLTAGE_VCC_MAXIMUM_MAXIMUM_CURRENT;
        break;
    case RSSPECAN_VAL_MEASTYPE_SDEV:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_VOLTAGE_VCC_MAXIMUM_STANDARD_DEVIATION_CURRENT;
        break;
    case RSSPECAN_VAL_MEASTYPE_AVER:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_VOLTAGE_VCC_MAXIMUM_AVERAGE_CURRENT;
        break;
    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, measurementType), 2, "Measurement Type");
        break;
    }

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, NULL, attr, result),
            3, "Result");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the Voltage (Vcc) result of the currently selected result
/// HIFN  range (corresponds to blue area of statistics table).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR result/Queries the Voltage (Vcc) result of the currently selected result
/// HIPAR result/range (corresponds to blue area of statistics table).
ViStatus _VI_FUNC rsspecan_GetAmplifierStatisticsTableVoltageVccMaximumSelectedResult(
                            ViSession instrSession,
                            ViReal64* result)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, NULL, RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_VOLTAGE_VCC_MAXIMUM_SELECTED_RESULT, result),
            2, "Result");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the statistical Voltage (Vcc) value of all result ranges found
/// HIFN  in the entire measurement (corresponding to black area of statistics
/// HIFN  table).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR measurementType/Selects measurement type.
/// HIPAR result/Queries the statistical Voltage (Vcc) value of all result ranges found
/// HIPAR result/in the entire measurement (corresponding to black area of statistics
/// HIPAR result/table).
ViStatus _VI_FUNC rsspecan_GetAmplifierStatisticsTableVoltageVccMinimumAll(
                            ViSession instrSession,
                            ViInt32   measurementType,
                            ViReal64* result)
{
    ViStatus error = VI_SUCCESS;
    ViAttr   attr = 0;

    checkErr(RsCore_LockSession(instrSession));

    switch (measurementType)
    {
    case RSSPECAN_VAL_MEASTYPE_MIN:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_VOLTAGE_VCC_MINIMUM_MINIMUM_ALL;
        break;
    case RSSPECAN_VAL_MEASTYPE_MAX:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_VOLTAGE_VCC_MINIMUM_MAXIMUM_ALL;
        break;
    case RSSPECAN_VAL_MEASTYPE_SDEV:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_VOLTAGE_VCC_MINIMUM_STANDARD_DEVIATION_ALL;
        break;
    case RSSPECAN_VAL_MEASTYPE_AVER:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_VOLTAGE_VCC_MINIMUM_AVERAGE_ALL;
        break;
    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, measurementType), 2, "Measurement Type");
        break;
    }

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, NULL, attr, result),
            3, "Result");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the statistical Voltage (Vcc) value of all result ranges found
/// HIFN  in the entire measurement (corresponding to black area of statistics
/// HIFN  table).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR measurementType/Selects measurement type.
/// HIPAR result/Queries the statistical Voltage (Vcc) value of all result ranges found
/// HIPAR result/in the entire measurement (corresponding to black area of statistics
/// HIPAR result/table).
ViStatus _VI_FUNC rsspecan_GetAmplifierStatisticsTableVoltageVccMinimumCurrent(
                            ViSession instrSession,
                            ViInt32   measurementType,
                            ViReal64* result)
{
    ViStatus error = VI_SUCCESS;
    ViAttr   attr = 0;

    checkErr(RsCore_LockSession(instrSession));

    switch (measurementType)
    {
    case RSSPECAN_VAL_MEASTYPE_MIN:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_VOLTAGE_VCC_MINIMUM_MINIMUM_CURRENT;
        break;
    case RSSPECAN_VAL_MEASTYPE_MAX:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_VOLTAGE_VCC_MINIMUM_MAXIMUM_CURRENT;
        break;
    case RSSPECAN_VAL_MEASTYPE_SDEV:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_VOLTAGE_VCC_MINIMUM_STANDARD_DEVIATION_CURRENT;
        break;
    case RSSPECAN_VAL_MEASTYPE_AVER:
        attr = RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_VOLTAGE_VCC_MINIMUM_AVERAGE_CURRENT;
        break;
    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, measurementType), 2, "Measurement Type");
        break;
    }

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, NULL, attr, result),
            3, "Result");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the Voltage (Vcc) result of the currently selected result
/// HIFN  range (corresponds to blue area of statistics table).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR result/Queries the Voltage (Vcc) result of the currently selected result
/// HIPAR result/range (corresponds to blue area of statistics table).
ViStatus _VI_FUNC rsspecan_GetAmplifierStatisticsTableVoltageVccMinimumSelectedResult(
                            ViSession instrSession,
                            ViReal64* result)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, NULL, RSSPECAN_ATTR_AMPLIFIER_STATISTICS_TABLE_VOLTAGE_VCC_MINIMUM_SELECTED_RESULT, result),
            2, "Result");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/*****************************************************************************
 *- Callback Definition   ---------------------------------------------------*
 *****************************************************************************/

/*****************************************************************************
 *- End Of RSSpecAn K18 Amplifier Measurement  ------------------------------*
 *****************************************************************************/
