/*****************************************************************************
 *- RSSpecAn K40 - Phase Noise ----------------------------------------------*
 *****************************************************************************/

#include <math.h>
#include "rsspecan.h"
#include "rsspecan_utility.h"
#include "rsspecan_internal.h"

//#define DATA_BUFFER_SIZE    126000L                      /* Data buffer size */

static ViString limTypeArr[] = {"LOW", "UPP", NULL};

//void rsspecan_SWAP_32(unsigned char *data, long points);

/*****************************************************************************
 * Function: Phase Mode
 * Purpose:  This function selects the Phase Noise mode.
 *****************************************************************************/
ViStatus _VI_FUNC rsspecan_PhaseMode(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_PHASE_MODE, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function configures RF Input.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR source/Selects the signal source for measurements
/// HIPAR inputCoupling/Selects the coupling type of the RF input.
ViStatus _VI_FUNC rsspecan_ConfigurePhaseRFInput(ViSession instrSession,
                                                 ViInt32   source,
                                                 ViInt32   inputCoupling)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_RF_INPUT, source),
            2, "Source");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_AMPL_RF_COUPLING, inputCoupling),
            3, "Input Coupling");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN Sets the measurement channel for baseband noise measurements.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR inputConnector/Selects the measurement channel for baseband noise measurements.
ViStatus _VI_FUNC rsspecan_ConfigurePhaseBasebandInput(ViSession instrSession,
                                                       ViInt32   inputConnector)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_PNOISE_BASEBAND_INPUT_CONNECTOR, inputConnector),
            2, "Input Connector");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the local oscillator.
/// HIFN
/// HIFN     Note(s):
/// HIFN
/// HIFN     (1) This function is avialable for FSWP only.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR source/This control selects the type of local oscillator in the test setup.
/// HIPAR externalLevel/This control selects the level of an external LO signal that is fed
/// HIPAR externalLevel/into the R&S FSWP.
ViStatus _VI_FUNC rsspecan_ConfigurePhaseLocalOscillator(ViSession instrSession,
                                                         ViInt32   source,
                                                         ViInt32   externalLevel)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_PHASE_NOISE_LOCAL_OSCILLATOR_SOURCE, source),
            2, "Source");

    if (source == RSSPECAN_VAL_SOUR_EXT)
    {
        viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_PHASE_NOISE_LOCAL_OSCILLATOR_EXTERNAL_LEVEL, externalLevel),
                3, "External Level");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/*****************************************************************************
 * Function: Configure Phase Signal Settings
 * Purpose:  This function configures the Phase Signal Settings.
 *****************************************************************************/
ViStatus _VI_FUNC rsspecan_ConfigurePhaseSignalSettings(ViSession instrSession,
                                                        ViReal64  frequency,
                                                        ViReal64  level,
                                                        ViBoolean sweepForward,
                                                        ViBoolean verifyFrequencyLevel,
                                                        ViReal64  frequencyTolerance,
                                                        ViReal64  levelTolerance)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViReal64Range(instrSession, level, -100.0, 30.0), 3, "Level");

    if (sweepForward == VI_TRUE)
    {
        viCheckParm(RsCore_InvalidViReal64Range(instrSession, frequencyTolerance, 1.0, 1.0e+6), 6,
                    "Frequency Tolerance");
    }
    else
    {
        viCheckParm(RsCore_InvalidViReal64Range(instrSession, frequencyTolerance, 1.0, 100.0), 6,
                    "Frequency Tolerance");
    }

    viCheckParm(RsCore_InvalidViReal64Range(instrSession, levelTolerance, 3.0, 20.0), 7, "Level Tolerance");

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "Win1", RSSPECAN_ATTR_FREQUENCY_CENTER, frequency),
            2, "Frequency");

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_PHASE_CONF_POW_EXP_RF, level),
            3, "Level");

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_PHASE_SWE_FORW, sweepForward),
            4, "Sweep Forward");

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_PHASE_FREQ_VER, verifyFrequencyLevel),
            5, "Verify Frequency Level");

    if (verifyFrequencyLevel == VI_TRUE)
    {
        viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_PHASE_FREQ_VER_TOL, frequencyTolerance),
                6, "Frequency Tolerance");

        viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_PHASE_POW_RLEV_VER_TOL, levelTolerance),
                7, "Level Tolerance");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/*****************************************************************************
 * Function: Configure Phase Frequency Tolerance
 * Purpose:  This function sets the absolute frequency tolerance. This
 *           functions automatically sets the Verify Frequency to ON. This is
 *           required for the Frequency Tolerance to be enabled and set.
*****************************************************************************/
ViStatus _VI_FUNC rsspecan_ConfigurePhaseFrequencyTolerance(ViSession instrSession,
                                                            ViReal64  absoluteFrequencyTolerance)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_PHASE_FREQ_VER, VI_TRUE),
            2, "");

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_PHASE_FREQ_VER_TOL_ABS, absoluteFrequencyTolerance),
            2, "Absolute Frequency Tolerance");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/*****************************************************************************
 * Function: Configure Phase AFC Settings
 * Purpose:  This function configures the Phase AFC Settings.
 *****************************************************************************/
ViStatus _VI_FUNC rsspecan_ConfigurePhaseAFCSettings(ViSession instrSession,
                                                     ViBoolean trackFrequency,
                                                     ViBoolean trackLevel)
{
    ViStatus  error = VI_SUCCESS;
    ViBoolean state = VI_FALSE;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsspecan_GetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_PHASE_FREQ_VER, &state));

    if (state == VI_TRUE)
    {
        viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_PHASE_FREQ_TRACK, trackFrequency),
                2, "Track Frequency");

        viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_PHASE_POW_TRACK, trackLevel),
                3, "Track Level");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function turns the suppression of AM noise on and off.
/// HIFN
/// HIFN     Note(s):
/// HIFN
/// HIFN     (1) AM rejection is available for the I/Q sweep mode.
/// HIFN
/// HIFN     (2) This function is available for R&S FSW only.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR amRejection/Turns the suppression of AM noise on and off.
ViStatus _VI_FUNC rsspecan_ConfigurePhaseAMRejection(ViSession instrSession,
                                                     ViBoolean amRejection)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_PHASE_AM_REJECTION, amRejection),
            2, "AM Rejection");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function defines the maximum tracking bandwidth (sample rate) for
/// HIFN  all half decades.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR trackingBandwidth/Defines the maximum tracking bandwidth (sample rate) for all half
/// HIPAR trackingBandwidth/decades.
ViStatus _VI_FUNC rsspecan_ConfigurePhaseTrackingBW(ViSession instrSession,
                                                    ViReal64  trackingBandwidth)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_PHASE_MAX_FREQ_DRIFT, trackingBandwidth),
            2, "Tracking Bandwidth");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function turns the digital PLL on and off.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR digitalPLL/Turns the digital PLL on and off.
ViStatus _VI_FUNC rsspecan_ConfigurePhaseDigitalPLL(ViSession instrSession,
                                                    ViBoolean digitalPLL)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_PHASE_DIGITAL_PLL, digitalPLL),
            2, "Digital PLL");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function turns decimation of results on and off.
/// HIFN
/// HIFN     Note(s):
/// HIFN
/// HIFN     (1) This function is available for R&S FSW only.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR decimation/This control turns decimation of results on and off.
ViStatus _VI_FUNC rsspecan_ConfigurePhaseDecimation(ViSession instrSession,
                                                    ViBoolean decimation)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_PHASE_DECIMATION, decimation),
            2, "Decimation");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function turns the I/Q online measurement mode on and off.
/// HIFN
/// HIFN     Note(s):
/// HIFN
/// HIFN     (1) This function is available for R&S FSW only.
/// HIFN
/// HIFN     (2) This mode is available for offset frequencies smaller than 30 kHz.
/// HIFN  Note that you have to
/// HIFN     - turn on decimation with RSSPECAN_ATTR_PHASE_DECIMATION
/// HIFN     - select the I/Q FFT mode for the affected half decades with
/// HIFN  RSSPECAN_ATTR_PHASE_BAND_TYPE
/// HIFN     - turn off forward sweep with RSSPECAN_ATTR_PHASE_SWE_FORW
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR onlineIQ/This control turns the I/Q online measurement mode on and off.
ViStatus _VI_FUNC rsspecan_ConfigurePhaseOnlineIQ(ViSession instrSession,
                                                  ViBoolean onlineIQ)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_PHASE_ONLINE_IQ, onlineIQ),
            2, "Online IQ");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/*****************************************************************************
 * Function: Configure Phase Display Settings
 * Purpose:  This function configures the Phase Display Settings.
 *****************************************************************************/
ViStatus _VI_FUNC rsspecan_ConfigurePhaseDisplaySettings(ViSession instrSession,
                                                         ViReal64  xAxisStart,
                                                         ViReal64  xAxisStop,
                                                         ViBoolean autoscaleOnce,
                                                         ViReal64  yAxisTop,
                                                         ViReal64  yAxisRange)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViReal64Range(instrSession, xAxisStart, 0.0, 300.0e+6), 2, "X Axis Start");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, xAxisStop, 3.0, 1.0e+9), 3, "X Axis Stop");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, yAxisTop, -200.0, 0.0), 5, "Y Axis Top");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, yAxisRange, 1.0, 200.0), 6, "Y Axis Range");

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_FREQUENCY_START, xAxisStart),
            2, "X Axis Start");

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_FREQUENCY_STOP, xAxisStop),
            3, "X Axis Stop");

    if (autoscaleOnce == VI_TRUE)
    {
        viCheckParm(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_PHASE_DISP_TRACE_Y_AUTO_ONCE, ""),
                4, "Autoscale Once");
    }
    else
    {
        viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_PHASE_DISP_TRACE_Y_AUTO, autoscaleOnce),
                4, "Autoscale Once");

        viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_PHASE_DISP_TRACE_Y_RLEV, yAxisTop),
                5, "Y Axis Top");

        viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_PHASE_DISP_TRACE_Y, yAxisRange),
                6, "Y Axis Range");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function configures the Phase Display Settings.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR yAxisScaling/Selects the way the application scales the horizontal axis.
/// HIPAR yAxisTop/Specifies the max Phase Noise level in y-axis for the trace results.
/// HIPAR yAxisRange/Specifies the distance from the top to the origin in y-axis.
/// HIPAR yAxisBottom/Defines the reference value or upper border of the diagram area.
ViStatus _VI_FUNC rsspecan_ConfigurePhaseDisplayYAxisSettings(
                            ViSession instrSession,
                            ViInt32   yAxisScaling,
                            ViReal64  yAxisTop,
                            ViReal64  yAxisRange,
                            ViReal64  yAxisBottom)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    switch (yAxisScaling)
    {
    case RSSPECAN_VAL_PHASE_Y_SCALE_OFF:
        viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_PHASE_Y_AXIS_MANUAL_SCALING, yAxisScaling),
                2, "Y Axis Scaling");
        break;
    case RSSPECAN_VAL_PHASE_Y_SCALE_TOP_BOTTOM:
        viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_PHASE_Y_AXIS_MANUAL_SCALING, yAxisScaling),
                2, "Y Axis Scaling");

        viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_PHASE_DISP_TRACE_Y_RLEV, yAxisTop),
                3, "Y Axis Top");

        viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_PHASE_DISP_TRACE_Y_BOTTOM, yAxisBottom),
                4, "Y Axis Bottom");
        break;
    case RSSPECAN_VAL_PHASE_Y_SCALE_TOP_RANGE:
        viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_PHASE_Y_AXIS_MANUAL_SCALING, yAxisScaling),
                2, "Y Axis Scaling");

        viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_PHASE_DISP_TRACE_Y_RLEV, yAxisTop),
                3, "Y Axis Top");

        viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_PHASE_DISP_TRACE_Y, yAxisRange),
                4, "Y Axis Range");
        break;
    case RSSPECAN_VAL_PHASE_Y_SCALE_BOTTOM_RANGE:
        viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_PHASE_Y_AXIS_MANUAL_SCALING, yAxisScaling),
                2, "Y Axis Scaling");

        viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_PHASE_DISP_TRACE_Y_BOTTOM, yAxisBottom),
                4, "Y Axis Bottom");

        viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_PHASE_DISP_TRACE_Y, yAxisRange),
                4, "Y Axis Range");
        break;
    case RSSPECAN_VAL_PHASE_Y_SCALE_AUTO:
        checkErr(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_PHASE_DISP_TRACE_Y_AUTO, VI_TRUE));

        break;
    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, yAxisScaling), 2, "Y Axis Scaling");
        break;
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function selects the unit of the y-axis.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR unit/Selects the unit of the y-axis.
ViStatus _VI_FUNC rsspecan_ConfigurePhaseDisplayYAxisUnit(ViSession instrSession,
                                                          ViInt32   unit)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_PHASE_Y_AXIS_UNIT, unit),
            2, "Unit");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function configures the phase display X axis settings.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR xAxisStart/Selects the way the application scales the horizontal axis.
/// HIPAR xAxisStop/Defines the start frequency of the horizontal axis.
/// HIPAR xAxisScope/Defines the stop frequency of the horizontal axis.
/// HIPAR halfDecade/Selects the half decade that is displayed.
ViStatus _VI_FUNC rsspecan_ConfigurePhaseDisplayXAxisSettings(
                            ViSession instrSession,
                            ViInt32   xAxisScope,
                            ViReal64  xAxisStart,
                            ViReal64  xAxisStop,
                            ViReal64  halfDecade)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_PHASE_X_AXIS_SCOPE, xAxisScope),
            2, "X Axis Scope");

    switch (xAxisScope)
    {
    case RSSPECAN_VAL_NOISE_X_SCOPE_MANUAL:
        viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_PHASE_X_AXIS_START_FREQUENCY, xAxisStart),
                3, "X Axis Start");

        viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_PHASE_X_AXIS_STOP_FREQUENCY, xAxisStop),
                4, "X Axis Stop");
        break;
    case RSSPECAN_VAL_NOISE_X_SCOPE_HDECADE:
        viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_PHASE_X_AXIS_HALF_DECADE, halfDecade),
                5, "Half Decade");
        break;
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/*****************************************************************************
 * Function: Configure Phase Trace Settings
 * Purpose:  This function configures the Phase Trace Settings.
 *****************************************************************************/
ViStatus _VI_FUNC rsspecan_ConfigurePhaseTraceSettings(ViSession instrSession,
                                                       ViReal64  traceOffset,
                                                       ViInt32   traceSmoothing,
                                                       ViInt32   smoothingType)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViReal64Range(instrSession, traceOffset, -200.0, 200.0e+6), 2, "Trace Offset");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, traceSmoothing, 1, 20),
            3, "Trace Smoothing");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, smoothingType, RSSPECAN_VAL_PHASE_SMO_TYPE_LIN, RSSPECAN_VAL_PHASE_SMO_TYPE_MED),
            4, "Smoothing Type");

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_PHASE_DISP_TRACE_Y_RLEV_OFFS, traceOffset),
            2, "Trace Offset");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_PHASE_DISP_TRACE_SMO_APER, traceSmoothing),
            3, "Trace Smoothing");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_PHASE_DISP_TRACE_SMO_TYPE, smoothingType),
            4, "Smoothing Type");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/*****************************************************************************
 * Function: Configure Phase Residual Calculations
 * Purpose:  This function configures the Phase Residual Calculations.
 *****************************************************************************/
ViStatus _VI_FUNC rsspecan_ConfigurePhaseResidualCalculations(
                            ViSession instrSession,
                            ViBoolean useMeasurementSettings,
                            ViReal64  evaluateFrom,
                            ViReal64  evaluateTo)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViReal64Range(instrSession, evaluateFrom, 1.0e+3, 1.0e+6), 3, "Evaluate From");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, evaluateTo, 1.0e+3, 1.0e+6), 4, "Evaluate To");

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_PHASE_CALC_EVAL,
                    !useMeasurementSettings), 2, "Use Measurement Settings");

    if (useMeasurementSettings == VI_FALSE)
    {
        viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_PHASE_CALC_EVAL_STAR, evaluateFrom),
                3, "Evaluate From");

        viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_PHASE_CALC_EVAL_STOP, evaluateTo),
                4, "Evaluate To");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function configures the trace for a custom residual noise calculation range.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR userRange/Selects the user range.
/// HIPAR trace/Selects the trace for a custom residual noise calculation range.
ViStatus _VI_FUNC rsspecan_ConfigurePhaseTraceCustomRange(ViSession instrSession,
                                                          ViInt32   reserved,
                                                          ViInt32   userRange,
                                                          ViInt32   trace)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
             reserved;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, userRange, 1, 3),
            3, "User Range");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "U%ld", userRange);

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_PHASE_USER_TRACE, trace),
            4, "trace");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN This function configures the Phase Residual User Calculations.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR userRange/Selects the user range.
/// HIPAR evaluateFrom/This control defines the start point of a custom residual noise calculation range.
/// HIPAR evaluateTo/This control defines the end point of a custom residual noise calculation range.
ViStatus _VI_FUNC rsspecan_ConfigurePhaseUserResidualCalculations(
                            ViSession instrSession,
                            ViInt32   userRange,
                            ViReal64  evaluateFrom,
                            ViReal64  evaluateTo)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, userRange, 1, 3),
            2, "User Range");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "U%ld", userRange);

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_PHASE_USER_EVALUATE_FROM, evaluateFrom),
            3, "Evaluate From");

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_PHASE_USER_EVALUATE_TO, evaluateTo),
            4, "Evaluate To");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/*****************************************************************************
 * Function: Configure Phase Spot Noise Settings
 * Purpose:  This function configures the Phase Spot Noise Settings.
 *****************************************************************************/
ViStatus _VI_FUNC rsspecan_ConfigurePhaseSpotNoiseSettings(ViSession instrSession,
                                                           ViBoolean enable,
                                                           ViReal64  offsetFrequency1,
                                                           ViReal64  offsetFrequency2,
                                                           ViReal64  offsetFrequency3,
                                                           ViReal64  offsetFrequency4)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViReal64Range(instrSession, offsetFrequency1, 1.0, 1.0e+9), 3, "Offset Frequency 1");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, offsetFrequency2, 1.0, 1.0e+9), 4, "Offset Frequency 2");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, offsetFrequency3, 1.0, 1.0e+9), 5, "Offset Frequency 3");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, offsetFrequency4, 1.0, 1.0e+9), 6, "Offset Frequency 4");

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "SN1", RSSPECAN_ATTR_PHASE_CALC_SNO_STAT, enable),
            2, "Enable");

    if (RsCore_IsInstrumentModel(instrSession, "FSW"))
    {
        viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "SN2", RSSPECAN_ATTR_PHASE_CALC_SNO_STAT, enable),
                2, "Enable");

        viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "SN3", RSSPECAN_ATTR_PHASE_CALC_SNO_STAT, enable),
                2, "Enable");

        viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "SN4", RSSPECAN_ATTR_PHASE_CALC_SNO_STAT, enable),
                2, "Enable");
    }

    if (enable == VI_TRUE)
    {
        viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "SN1", RSSPECAN_ATTR_PHASE_CALC_SNO_X, offsetFrequency1),
                3, "Offset Frequency 1");

        viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "SN2", RSSPECAN_ATTR_PHASE_CALC_SNO_X, offsetFrequency2),
                4, "Offset Frequency 2");

        viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "SN3", RSSPECAN_ATTR_PHASE_CALC_SNO_X, offsetFrequency3),
                5, "Offset Frequency 3");

        viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "SN4", RSSPECAN_ATTR_PHASE_CALC_SNO_X, offsetFrequency4),
                6, "Offset Frequency 4");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function configures the Phase Spot Noise Settings.
/// HIRET Document return value here.
/// HIRET You may use multiple lines for documentation.
/// HIPAR instrSession/Document parameter here.
/// HIPAR marker/Document parameter here.
/// HIPAR enable/Document parameter here.
/// HIPAR offsetFrequency/Document parameter here.
ViStatus _VI_FUNC rsspecan_ConfigurePhaseSpotNoise(ViSession instrSession,
                                                   ViInt32   marker,
                                                   ViBoolean enable,
                                                   ViReal64  offsetFrequency)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_PHASE_CALC_SNO_STAT,
            "Snoise", marker - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Marker");
    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, repCap, RSSPECAN_ATTR_PHASE_CALC_SNO_STAT, enable),
            3, "Enable");
    if (enable == VI_TRUE)
    {
        viCheckParm(rsspecan_SetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_PHASE_CALC_SNO_X, offsetFrequency),
                4, "Offset Frequency");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN Document your function here.
/// HIFN You may use multiple lines for documentation.
/// HIRET Document return value here.
/// HIRET You may use multiple lines for documentation.
/// HIPAR instrSession/Document parameter here.
/// HIPAR state/Document parameter here.
ViStatus _VI_FUNC rsspecan_ConfigurePhaseSpotNoiseCustom(ViSession instrSession,
                                                         ViBoolean state)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_PHASE_CUSTOM_SNO_ENABLED, state),
            2, "State");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function selects the trace for which spot noise results are
/// HIFN  calculated. A trace can only be selected if it has been turned on
/// HIFN  ('Trace Mode' is not 'Blank').
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR trace/This control selects the trace for which spot noise results are
/// HIPAR trace/calculated. A trace can only be selected if it has been turned on
/// HIPAR trace/('Trace Mode' is not 'Blank').
ViStatus _VI_FUNC rsspecan_ConfigurePhaseSpotNoiseTraceSelection(
                            ViSession instrSession,
                            ViInt32   trace)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    if (RsCore_IsInstrumentModel(instrSession, "FSWP"))
    {
        viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_PHASE_SPOT_NOISE_SELECT_TRACE_FSWP, trace),
                2, "Trace");
    }
    else
    {
        viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_PHASE_SPOT_NOISE_SELECT_TRACE, trace),
                2, "Trace");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/*****************************************************************************
 * Function: Configure Phase Analyzer Settings
 * Purpose:  This function configures the Phase Analyzer Settings.
 *****************************************************************************/
ViStatus _VI_FUNC rsspecan_ConfigurePhaseAnalyzerSettings(ViSession instrSession,
                                                          ViBoolean preselector,
                                                          ViBoolean preamplifier)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    if (RsCore_HasInstrumentOptions(instrSession, "B2"))
    {
        viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_NOISE_INP_PRES_STAT, preselector),
                2, "Pre-selector");
    }

    if (RsCore_HasInstrumentOptions(instrSession, "B2|B25|B23"))
    {
        viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_NOISE_INP_GAIN_STAT, preamplifier),
                3, "Pre-amplifier");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN This function configures the spot noise calculation for each decade.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR spotNoiseCalculation/This control turns the spot noise calculation on every
/// HIPAR spotNoiseCalculation/10x offset frequency on and off.
ViStatus _VI_FUNC rsspecan_ConfigurePhaseSpotNoiseCalculation(
                            ViSession instrSession,
                            ViBoolean spotNoiseCalculation)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_PHASE_SPOT_NOISE_CALCULATION, spotNoiseCalculation),
            2, "Spot Noise Calculation");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function turns the display of spot noise information in the
/// HIFN diagram area on and off.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR state/This control enables/disables displaying the spot noise info.
ViStatus _VI_FUNC rsspecan_ConfigurePhaseSpotNoiseDisplayInfo(
                            ViSession instrSession,
                            ViBoolean state)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_PHASE_SPOT_NOISE_DISPLAY_INFO_ENABLED, state),
            2, "State");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/*****************************************************************************
 * Function: Set Phase Spot Noise All Off
 * Purpose:  This function sets all the phase spot noise markers off.
 *****************************************************************************/
ViStatus _VI_FUNC rsspecan_SetPhaseAllSpotNoiseMarkersOff(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_PHASE_CALC_SNO_AOFF, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function configures the spur.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR spurSuppression/This control turns spur suppression on and off.
/// HIPAR spurThreshold/This control defines the level threshold for spur removal.
ViStatus _VI_FUNC rsspecan_ConfigurePhaseSpurSettings(ViSession instrSession,
                                                      ViBoolean spurSuppression,
                                                      ViReal64  spurThreshold)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_PHASE_SPUR_SUPPRESSION, spurSuppression),
            2, "Spur Suppression");

    if (spurSuppression == VI_TRUE)
    {
        viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_PHASE_SPUR_THRESHOLD, spurThreshold),
                3, "Spur Threshold");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN Defines an arithmetic level offset. The offset is applied to the
/// HIFN signal level.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR levelOffset/Defines the level offset
ViStatus _VI_FUNC rsspecan_ConfigurePhaseLevelShifting(ViSession instrSession,
                                                       ViReal64  levelOffset)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_PHASE_ARITHMETIC_LEVEL_OFFSET, levelOffset),
            2, "Level Offset");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the Smoothing traces.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSSPECAN_ATTR_SMOOTHING_ALL_TRACES_STATE
/// HIFN     RSSPECAN_ATTR_SMOOTHING_ALL_TRACES_APERTURE
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     [SENSe:]SMOothing[:STATe]
/// HIFN     [SENSe:]SMOothing:APERture
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR smoothingState/This control turns trace smoothing for a specific trace on and off.
/// HIPAR smoothingValue/This control defines the magnitude (aperture) of trace smoothing.
ViStatus _VI_FUNC rsspecan_ConfigurePhaseSmoothingSettings(ViSession instrSession,
                                                           ViBoolean smoothingState,
                                                           ViReal64  smoothingValue)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_SMOOTHING_ALL_TRACES_STATE, smoothingState),
            2, "Smoothing State");

    if (smoothingState == VI_TRUE)
    {
        viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_SMOOTHING_ALL_TRACES_APERTURE, smoothingValue),
                3, "Smoothing Value");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function queries the currently measured signal level.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSSPECAN_ATTR_CURRENTLY_MEASURED_SIGNAL_LEVEL
/// HIFN
/// HIFN     Note(s):
/// HIFN     (1) This function is available for R&S FSWP.
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     [SENSe:]POWer:RLEVel?
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR signalLevel/This control returns the currently measured signal level.
ViStatus _VI_FUNC rsspecan_ConfigurePhaseSignalLevel(ViSession instrSession,
                                                     ViReal64* signalLevel)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_CURRENTLY_MEASURED_SIGNAL_LEVEL, signalLevel),
            2, "Signal Level");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN Defines the pulse detection.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR mode/Selects the pulse detection mode.
/// HIPAR repetitionInterval/Defines the repetition interval.
ViStatus _VI_FUNC rsspecan_ConfigurePhasePulseSettings(ViSession instrSession,
                                                       ViInt32   mode,
                                                       ViReal64  repetitionInterval)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_PHASE_PULSE_DETECTION_MODE, mode),
            2, "Mode");

    if (mode == RSSPECAN_VAL_MAN)
    {
        viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_PHASE_PULSE_REPETITION_INTERVAL, repetitionInterval),
                3, "Repetition Interval");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN Initiates an immediate, automatic detection of the pulse
/// HIFN characteristics with a subsequent configuration of the gate.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsspecan_ConfigurePhasePulseDetectionOnce(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_PHASE_PULSE_DETECTION_MODE_ONCE, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN Selects the gate type.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR gateType/Selects the gate type.
ViStatus _VI_FUNC rsspecan_ConfigurePhasePulseExternalGateType(
                            ViSession instrSession,
                            ViInt32   gateType)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_PHASE_PULSE_EXTERNAL_GATE_TYPE, gateType),
            2, "Gate Type");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function selects the source for the pulse gate.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSSPECAN_ATTR_PHASE_PULSE_GATE_SOURCE
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     [SENSe:]SWEep:PULSe:GATE
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR gateSource/This command selects the source for the pulse gate.
ViStatus _VI_FUNC rsspecan_ConfigurePhasePulseGateSource(ViSession instrSession,
                                                         ViInt32   gateSource)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_PHASE_PULSE_GATE_SOURCE, gateSource),
            2, "Gate Source");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Selects the gate level.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSSPECAN_ATTR_PHASE_PULSE_GATE_LEVEL
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     [SENSe:]SWEep:EGATe:LEVel
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR gateLevel/This control defines the gate level.
ViStatus _VI_FUNC rsspecan_ConfigurePhasePulseExternalGateLevel(
                            ViSession instrSession,
                            ViReal64  gateLevel)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_PHASE_PULSE_GATE_LEVEL, gateLevel),
            2, "Gate Level");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function configures the cross correlations parameters.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR optimization/This control turns optimization of the cross-correlation
/// HIPAR optimization/operations on and off.
/// HIPAR factor/Defines the cross-correlation factor (XCORR factor) that defines
/// HIPAR factor/the cross-correlation operations in each half decade.
ViStatus _VI_FUNC rsspecan_ConfigurePhaseCrossCorrelation(ViSession instrSession,
                                                          ViBoolean optimization,
                                                          ViInt32   factor)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_PHASE_CROSS_CORREL_OPTIMIZE_ENABLED, optimization),
            2, "Optimization");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_PHASE_CROSS_CORREL_FACTOR, factor),
            3, "Factor");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function defines the threshold for the cross-correlation
/// HIFN  optimization feature. When the threshold (distance between measured
/// HIFN  values and gain indicator) is reached in any given half decade, the
/// HIFN  measurement in that half decade stops and resumes to measure the next
/// HIFN  one.
/// HIFN
/// HIFN     Note(s):
/// HIFN
/// HIFN     (1) This attribute requires the XCORR optimization to be turned on
/// HIFN  (set with function rsspecan_ConfigurePhaseCrossCorrelation).
/// HIFN
/// HIFN     (2) This function is available with R&S FSWP.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR threshold/This control defines the threshold for the cross-correlation
/// HIPAR threshold/optimization feature. When the threshold (distance between measured
/// HIPAR threshold/values and gain indicator) is reached in any given half decade, the
/// HIPAR threshold/measurement in that half decade stops and resumes to measure the next
/// HIPAR threshold/one.
ViStatus _VI_FUNC rsspecan_ConfigurePhaseCrossCorrelationOptimizeThreshold(
                            ViSession instrSession,
                            ViReal64  threshold)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_PHASE_CROSS_CORRELATION_OPTIMIZE_THRESHOLD, threshold),
            2, "Threshold");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function stops a measurement in any given half decade and resumes
/// HIFN  the measurement of the next half decade.
/// HIFN
/// HIFN     Note(s):
/// HIFN
/// HIFN     (1) This function is available with R&S FSWP.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsspecan_ConfigurePhaseCrossCorrelationFinishSegment(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_PHASE_FINISH_SEGMENT, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function configures the automatically scan of the frequency
/// HIFN spectrum for any signals before a phase noise measurement begins.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR signalSearchAuto/Turns the automatic (carrier) frequency search on and off.
/// HIPAR lowerLimit/Defines the lower limit of the frequency search range.
/// HIPAR upperLimit/Defines the upper limit of the frequency search range.
/// HIPAR threshold/Defines the threshold of the signal search.
ViStatus _VI_FUNC rsspecan_ConfigurePhaseSignalSearching(ViSession instrSession,
                                                         ViBoolean signalSearchAuto,
                                                         ViReal64  lowerLimit,
                                                         ViReal64  upperLimit,
                                                         ViReal64  threshold)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_SIGNAL_SEARCH_AUTO_ENABLED, signalSearchAuto),
            2, "Signal Search Auto");

    if (signalSearchAuto == VI_TRUE)
    {
        viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_SIGNAL_SEARCH_AUTO_LIMIT_MIN, lowerLimit),
                3, "Lower Limit");

        viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_SIGNAL_SEARCH_AUTO_LIMIT_MAX, upperLimit),
                4, "Upper Limit");

        viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_SIGNAL_SEARCH_THRESHOLD, threshold),
                5, "Threshold");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function turns verification of the signal on and off.
/// HIFN
/// HIFN     Note(s):
/// HIFN
/// HIFN     (1) This function is available with R&S FSWP
/// HIFN
/// HIFN     (2) This function is available only when automatic signal search is
/// HIFN  turned off (set with fucntions rsspecan_ConfigurePhaseSignalSearching).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR signalCount/This control turns verification of the signal frequency on and off.
ViStatus _VI_FUNC rsspecan_ConfigurePhaseSignalSearchingCount(
                            ViSession instrSession,
                            ViBoolean signalCount)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_SIGNAL_SEARCH_AUTO_COUNT, signalCount),
            2, "Signal Count");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the start/stop time for the frequency
/// HIFN  stability measurements.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSSPECAN_ATTR_PHASE_FREQUENCY_STABILITY_TAU_START
/// HIFN     RSSPECAN_ATTR_PHASE_FREQUENCY_STABILITY_TAU_STOP
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     [SENSe:]TIME:STARt
/// HIFN     [SENSe:]TIME:STOP
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR tauStart/This control defines the start time for the frequency stability
/// HIPAR tauStart/measurements.
/// HIPAR tauStop/This control defines the stop time for the frequency stability
/// HIPAR tauStop/measurements.
ViStatus _VI_FUNC rsspecan_ConfigurePhaseFrequencyStability(ViSession instrSession,
                                                            ViReal64  tauStart,
                                                            ViReal64  tauStop)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_PHASE_FREQUENCY_STABILITY_TAU_START, tauStart),
            2, "TAU Start");

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_PHASE_FREQUENCY_STABILITY_TAU_STOP, tauStop),
            3, "TAU Stop");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function configures the Phase Display Settings.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/Sets the window number.
/// HIPAR yAxisScaling/Selects the way the application scales the horizontal axis.
/// HIPAR yAxisTop/Specifies the max Phase Noise level in y-axis for the trace results.
/// HIPAR yAxisRange/Specifies the distance from the top to the origin in y-axis.
/// HIPAR yAxisBottom/Defines the reference value or upper border of the diagram area.
ViStatus _VI_FUNC rsspecan_ConfigurePhaseDisplayYAxis(ViSession instrSession,
                                                      ViInt32   window,
                                                      ViInt32   yAxisScaling,
                                                      ViReal64  yAxisTop,
                                                      ViReal64  yAxisRange,
                                                      ViReal64  yAxisBottom)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_PHASE_DISPLAY_TRACE_Y_AUTO,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap),
            2, "Window");
    switch (yAxisScaling)
    {
    case RSSPECAN_VAL_PHASE_Y_SCALE_TOP_BOTTOM:
        checkErr(rsspecan_SetAttributeViBoolean(instrSession, repCap, RSSPECAN_ATTR_PHASE_DISPLAY_TRACE_Y_AUTO, VI_FALSE));
        viCheckParm(rsspecan_SetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_PHASE_DISPLAY_TRACE_Y_RLEV, yAxisTop),
                3, "Y Axis Top");

        viCheckParm(rsspecan_SetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_PHASE_DISPLAY_TRACE_Y_BOTTOM, yAxisBottom),
                4, "Y Axis Bottom");
        break;
    case RSSPECAN_VAL_PHASE_Y_SCALE_TOP_RANGE:
        checkErr(rsspecan_SetAttributeViBoolean(instrSession, repCap, RSSPECAN_ATTR_PHASE_DISPLAY_TRACE_Y_AUTO, VI_FALSE));
        viCheckParm(rsspecan_SetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_PHASE_DISPLAY_TRACE_Y_RLEV, yAxisTop),
                3, "Y Axis Top");

        viCheckParm(rsspecan_SetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_PHASE_DISPLAY_TRACE_Y, yAxisRange),
                4, "Y Axis Range");
        break;
    case RSSPECAN_VAL_PHASE_Y_SCALE_BOTTOM_RANGE:
        checkErr(rsspecan_SetAttributeViBoolean(instrSession, repCap, RSSPECAN_ATTR_PHASE_DISPLAY_TRACE_Y_AUTO, VI_FALSE));
        viCheckParm(rsspecan_SetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_PHASE_DISPLAY_TRACE_Y_BOTTOM, yAxisBottom),
                4, "Y Axis Bottom");

        viCheckParm(rsspecan_SetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_PHASE_DISPLAY_TRACE_Y, yAxisRange),
                4, "Y Axis Range");
        break;
    case RSSPECAN_VAL_PHASE_Y_SCALE_AUTO:
        checkErr(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_PHASE_DISPLAY_TRACE_Y_AUTO, VI_TRUE));

        break;
    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, yAxisScaling), 2, "Y Axis Scaling");
        break;
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function scales the y-axis once. The scale remains the same
/// HIFN for subsequent measurements.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/Document parameter here.
ViStatus _VI_FUNC rsspecan_ConfigurePhaseDisplayYAxisOnce(ViSession instrSession,
                                                          ViInt32   window)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_PHASE_DISPLAY_TRACE_Y_AUTO_ONCE,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap),
            2, "Window");

    checkErr(rsspecan_SetAttributeViString(instrSession, repCap, RSSPECAN_ATTR_PHASE_DISPLAY_TRACE_Y_AUTO_ONCE, ""));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function configures the Phase Trace.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/Sets the window number.
/// HIPAR trace/Selects the trace.
/// HIPAR resultType/Selects the type of noise represented by the trace.
/// HIPAR smoothingState/Specifies whether smoothing of a particular trace is carried out.
/// HIPAR smoothing/Defines the magnitude (aperture) of trace smoothing.
/// HIPAR spursState/Turns spur suppression on individual traces on and off.
/// HIPAR spursThreshold/Defines the level threshold for spur removal.
/// HIPAR offset/Turns the offset for a trace on and off.
/// HIPAR offsetState/Defines the trace offset.
ViStatus _VI_FUNC rsspecan_ConfigurePhaseTrace(ViSession instrSession,
                                               ViInt32   window,
                                               ViInt32   trace,
                                               ViInt32   resultType,
                                               ViBoolean smoothingState,
                                               ViInt32   smoothing,
                                               ViBoolean spursState,
                                               ViReal64  spursThreshold,
                                               ViBoolean offsetState,
                                               ViReal64  offset)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_w[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_t[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_PHASE_TRACE_DISPLAYED_RESULT,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap_w),
            2, "Window");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, trace, 1, 8),
            3, "Trace");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_PHASE_TRACE_DISPLAYED_RESULT,
            "Trace", trace - 1, RS_REPCAP_BUF_SIZE, repCap_t),
            3, "Trace");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "%s,%s", repCap_w, repCap_t);

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_PHASE_TRACE_DISPLAYED_RESULT, resultType),
            4, "Result Type");

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, repCap, RSSPECAN_ATTR_PHASE_DISPLAY_TRACE_SMO_ENABLED, smoothingState),
            5, "Smoothing State");
    if (smoothingState == VI_TRUE)
    {
        viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_PHASE_DISPLAY_TRACE_SMO_APERTURE, smoothing),
                6, "Smoothing");
    }
    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, repCap, RSSPECAN_ATTR_PHASE_DISPLAY_TRACE_SPUR_SUPP_ENABLED, spursState),
            7, "Spurs State");
    if (spursState == VI_TRUE)
    {
        viCheckParm(rsspecan_SetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_PHASE_DISPLAY_TRACE_SPUR_THRESHOLD, spursThreshold),
                8, "Spurs Threshold");
    }
    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, repCap, RSSPECAN_ATTR_PHASE_DISPLAY_TRACE_OFFSET_ENABLED, offsetState),
            9, "Offset State");
    if (offsetState == VI_TRUE)
    {
        viCheckParm(rsspecan_SetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_PHASE_DISPLAY_TRACE_OFFSET, offset),
                10, "Offset");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN Document your function here.
/// HIFN You may use multiple lines for documentation.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/Sets the window number.
/// HIPAR firstTrace/Selects the first trace for subtraction.
/// HIPAR secondTrace/Selects the second trace for subtraction.
ViStatus _VI_FUNC rsspecan_ConfigurePhaseTraceMath(ViSession instrSession,
                                                   ViInt32   window,
                                                   ViInt32   firstTrace,
                                                   ViInt32   secondTrace)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, window, 0, 16),
            2, "Window");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CALC%ld:MATH (TRACE%ld-TRACE%ld)", window, firstTrace, secondTrace);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN Document your function here.
/// HIFN You may use multiple lines for documentation.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/Sets the window number.
/// HIPAR trace/Selects the trace.
/// HIPAR state/Switches the display of the trace label for a specific trace on and off.
/// HIPAR label/Defines the trace label.
ViStatus _VI_FUNC rsspecan_ConfigurePhaseTraceLabel(ViSession instrSession,
                                                    ViInt32   window,
                                                    ViInt32   trace,
                                                    ViBoolean state,
                                                    ViString  label)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_w[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_t[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_PHASE_TRACE_LABEL_ENABLED,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap_w),
            2, "Window");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_PHASE_TRACE_LABEL_ENABLED,
            "Trace", trace - 1, RS_REPCAP_BUF_SIZE, repCap_t),
            3, "Trace");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "%s,%s", repCap_w, repCap_t);

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, repCap, RSSPECAN_ATTR_PHASE_TRACE_LABEL_ENABLED, state),
            4, "State");

    viCheckParm(rsspecan_SetAttributeViString(instrSession, repCap, RSSPECAN_ATTR_PHASE_TRACE_LABEL, label),
            5, "Label");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function adds a window to the display.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init.
/// HIPAR stringSize/This control sets the allocated length of Returned Window Name string.
/// HIPAR referenceWindowName/String containing the number or name of the window the new window is inserted next to.
/// HIPAR direction/Direction the window is added relative to the window set with "Reference Window Name".
/// HIPAR windowType/Type of result display you want to add.
/// HIPAR returnedWindowName/Returns the name of the new window.
ViStatus _VI_FUNC rsspecan_AddPhaseWindow(ViSession instrSession,
                                          ViInt32   stringSize,
                                          ViString  referenceWindowName,
                                          ViInt32   direction,
                                          ViInt32   windowType,
                                          ViChar    returnedWindowName[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViChar   response[RS_MAX_MESSAGE_BUF_SIZE];
    ViChar*  p2direction;
    ViChar*  p2type;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &rsspecan_rngLayoutDirection, direction, &p2direction),
                4, "Direction");
    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &rsspecan_rngPhaseNoiseLayoutType, windowType, &p2type),
                5, "Window Type");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "LAY:ADD? '%s',%s,%s", referenceWindowName, p2direction, p2type);
    checkErr(RsCore_QueryViString(instrSession, cmd, response));

    if (returnedWindowName != NULL)
    {
        RsCore_TrimString(response, RS_VAL_TRIM_WHITESPACES_AND_ALL_QUOTES);
        RsCore_CopyToUserBufferAsciiData(returnedWindowName, stringSize, response);
    }

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function replaces a window with another one.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init.
/// HIPAR windowName/String containing the number or name of the window the new window is inserted next to.
/// HIPAR windowType/Type of result display you want to add.
ViStatus _VI_FUNC rsspecan_ReplacePhaseWindow(ViSession instrSession,
                                              ViString  windowName,
                                              ViInt32   windowType)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViChar*  p2type;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &rsspecan_rngPhaseNoiseLayoutType, windowType, &p2type),
                3, "Window Type");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "LAY:REPL '%s',%s", windowName, p2type);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/*****************************************************************************
 * Function: Configure Phase Sweep Settings
 * Purpose:  This function configures the Phase Sweep Settings.
 *****************************************************************************/
ViStatus _VI_FUNC rsspecan_ConfigurePhaseSweepSettings(ViSession instrSession,
                                                       ViInt32   sweepMode)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, sweepMode, RSSPECAN_VAL_SWE_MODE_FAST, RSSPECAN_VAL_SWE_MODE_MAN),
            2, "Sweep Mode");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_PHASE_SWEEP_MODE, sweepMode),
            2, "Sweep Mode");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function configures repeated tries to start the measurement
/// HIFN if signal verification fails on and off.
/// HIFN You may use multiple lines for documentation.
/// HIRET Document return value here.
/// HIRET You may use multiple lines for documentation.
/// HIPAR instrSession/Document parameter here.
/// HIPAR stopOnVerifyFailed/Turns repeated tries to start the measurement
/// HIPAR stopOnVerifyFailed/if signal verification fails on and off.
ViStatus _VI_FUNC rsspecan_ConfigurePhaseSignalVerificationFailed(
                            ViSession instrSession,
                            ViInt32   stopOnVerifyFailed)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_PHASE_SIGNAL_VERIFICATION_FAILED, stopOnVerifyFailed),
            2, "Stop On Verify Failed");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function selects the signal capture range.
/// HIFN if signal verification fails on and off.
/// HIFN You may use multiple lines for documentation.
/// HIRET Document return value here.
/// HIRET You may use multiple lines for documentation.
/// HIPAR instrSession/Document parameter here.
/// HIPAR captureRange/Selects the signal capture range.
ViStatus _VI_FUNC rsspecan_ConfigurePhaseSignalCaptureRange(ViSession instrSession,
                                                            ViInt32   captureRange)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_PHASE_SIGNAL_CAPTURE_RANGE, captureRange),
            2, "Capture Range");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/*****************************************************************************
 * Function: Configure Phase Span Settings
 * Purpose:  This function configures the Phase Span Settings.
 *****************************************************************************/
ViStatus _VI_FUNC rsspecan_ConfigurePhaseSpanSettings(ViSession instrSession,
                                                      ViReal64  startOffset,
                                                      ViReal64  stopOffset)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViReal64Range(instrSession, startOffset, 1.0, 300.0e+6), 2, "Start Offset");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, stopOffset, 3.0, 1.0e+9), 3, "Stop Offset");

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_FREQUENCY_START, startOffset),
            2, "Start Offset");

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_FREQUENCY_STOP, stopOffset),
            3, "Stop Offset");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/*****************************************************************************
 * Function: Configure Phase Carrier Frequency Offset Table
 * Purpose:  This function configures the Carrier Frequency Offset Table.
 *
 *           Notes:
 *
 *           (1) The Carrier Frequency Offset Table can be used only when the
 *           Configure Sweep Settings is set to Manual value.
 *****************************************************************************/
ViStatus _VI_FUNC rsspecan_ConfigurePhaseCarrierFrequencyOffsetTable(
                            ViSession instrSession,
                            ViInt32   subband,
                            ViReal64  rbw,
                            ViInt32   average,
                            ViInt32   fft)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  sweepMode;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsspecan_GetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_PHASE_SWEEP_MODE, &sweepMode));

    if (sweepMode == RSSPECAN_VAL_SWE_MODE_MAN)
    {
        viCheckParm(RsCore_InvalidViInt32Range(instrSession, subband, 1, 18),
                2, "Sub-band");
        viCheckParm(RsCore_InvalidViInt32Range(instrSession, average, 1, 10000),
                4, "Average");
        viCheckParm(RsCore_InvalidViInt32Range(instrSession, fft, RSSPECAN_VAL_BAND_TYPE_NORM, RSSPECAN_VAL_BAND_TYPE_IQFFT),
                5, "FFT");

        snprintf(repCap, RS_REPCAP_BUF_SIZE, "PNR%ld", subband);

        viCheckParm(rsspecan_SetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_PHASE_LIST_RANG_BAND, rbw),
                3, "RBW");

        viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_PHASE_LIST_RANG_SWE_COUN, average),
                4, "Average");

        viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_PHASE_LIST_RANG_FILT_TYPE, fft),
                5, "FFT");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/*****************************************************************************
 * Function: Configure Phase Preset Settings
 * Purpose:  This function configures the Phase Preset Settings.
 *
 *           Notes:
 *
 *           (1) The Preset Settings can be used only when the Configure Sweep
 *           Settings is set to Manual value.
 *****************************************************************************/
ViStatus _VI_FUNC rsspecan_ConfigurePhasePresetSettings(ViSession instrSession,
                                                        ViReal64  rbw,
                                                        ViInt32   average,
                                                        ViInt32   fft)
{
    ViStatus error = VI_SUCCESS;
    ViInt32  sweepMode;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsspecan_GetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_PHASE_SWEEP_MODE, &sweepMode));

    if (sweepMode == RSSPECAN_VAL_SWE_MODE_MAN)
    {
        viCheckParm(RsCore_InvalidViReal64Range(instrSession, rbw, 1.0, 300.0e+6), 2, "RBW");
        viCheckParm(RsCore_InvalidViInt32Range(instrSession, average, 1, 10000),
                3, "Average");
        viCheckParm(RsCore_InvalidViInt32Range(instrSession, fft, RSSPECAN_VAL_BAND_TYPE_NORM, RSSPECAN_VAL_BAND_TYPE_IQFFT),
                4, "FFT");

        viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "Win1", RSSPECAN_ATTR_PHASE_BAND_RAT, rbw),
                2, "RBW");

        viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "Win1", RSSPECAN_ATTR_PHASE_SWEEP_COUNT, average),
                3, "Average");

        viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "Win1", RSSPECAN_ATTR_PHASE_BAND_TYPE, fft),
                4, "FFT");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function configures the Phase Resolution Bandwidth
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR RBW/Defines the factor that the resolution bandwidth applied to each
/// HIPAR RBW/half decade is based on.
ViStatus _VI_FUNC rsspecan_ConfigurePhaseResolutionBandwidth(ViSession instrSession,
                                                             ViReal64  rbw)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_PHASE_BAND_RAT, rbw),
            2, "RBW");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Provides functionality to enable ultra-small resolution bandwidths
/// HIFN  (RBW) for the current measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR ultraSmallRBWs/Provides functionality to enable ultra-small resolution bandwidths
/// HIPAR ultraSmallRBWs/(RBW) for the current measurement.
ViStatus _VI_FUNC rsspecan_ConfigurePhaseUltraSmallRBWs(ViSession instrSession,
                                                        ViBoolean ultraSmallRBWs)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, NULL, RSSPECAN_ATTR_PHASE_ULTRA_SMALL_RBWS, ultraSmallRBWs),
            2, "Ultra Small RBWs");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function configures the window function for all or particular half decade.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR decade/This control selects the decades.
/// HIPAR subband/Specifies the sub-band for the IQ Window type.
/// HIPAR globalIQWindow/This control selects the window function for all or particular half decade.
ViStatus _VI_FUNC rsspecan_ConfigurePhaseGlobalIQWindowSettings(
                            ViSession instrSession,
                            ViInt32   decade,
                            ViInt32   subband,
                            ViInt32   globalIQWindow)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, decade, RSSPECAN_VAL_PHASE_ALL_DECADE, RSSPECAN_VAL_PHASE_PARTICULAR_DECADE),
            2, "Decade");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, subband, 1, 20),
            3, "Subband");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "PNR%ld", subband);

    if (decade == RSSPECAN_VAL_PHASE_PARTICULAR_DECADE)
    {
        viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_PHASE_IQ_WINDOW_FUNCTION_PARTICULAR_DECADE, globalIQWindow),
                4, "Global IQ Window");
    }
    else
    {
        viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_PHASE_IQ_WINDOW_FUNCTION_ALL_DECADE, globalIQWindow),
                5, "Global IQ Window");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function sets the trace for calculation of residual effects.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR range/This control selects the integration range.
/// HIPAR trace/This control selects the trace.
ViStatus _VI_FUNC rsspecan_ConfigurePhaseResidualCalculationTrace(
                            ViSession instrSession,
                            ViInt32   range,
                            ViInt32   trace)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_PHASE_INTEGRATION_TRACE,
            "IntRange", range - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Range");
    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_PHASE_INTEGRATION_TRACE, trace),
            3, "Trace");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function defines integration range.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR range/This control selects the integration range.
/// HIPAR overCompleteRange/This control turns integration of the entire measurement range
/// HIPAR overCompleteRange/for residual calculations on and off.
/// HIPAR start/This control defines the start frequency of a custom integration
/// HIPAR start/range for residual effects calculation.
/// HIPAR stop/This control defines the stop frequency of a custom integration
/// HIPAR stop/range for residual effects calculation.
/// HIPAR weightingFilter/Sets selects a weighting filter whose values are applied to the
/// HIPAR weightingFilter/calculation of residual effects.
ViStatus _VI_FUNC rsspecan_ConfigurePhaseIntegrationRange(ViSession instrSession,
                                                          ViInt32   range,
                                                          ViBoolean overCompleteRange,
                                                          ViReal64  start,
                                                          ViReal64  stop,
                                                          ViString  weightingFilter)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_PHASE_INTEGRATION_RANGE_ENABLED,
            "IntRange", range - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Range");
    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, repCap, RSSPECAN_ATTR_PHASE_INTEGRATION_RANGE_ENABLED, overCompleteRange),
            3, "Over Complete Range");
    if (overCompleteRange == VI_FALSE)
    {
        viCheckParm(rsspecan_SetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_PHASE_INTEGRATION_RANGE_START, start),
                4, "Start");

        viCheckParm(rsspecan_SetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_PHASE_INTEGRATION_RANGE_STOP, stop),
                4, "Stop");
    }

    if (strlen(weightingFilter) == 0)
    {
        checkErr(rsspecan_SetAttributeViString(instrSession, repCap, RSSPECAN_ATTR_PHASE_INTEGRATION_WEIGHTING_FILTER_NONE, NULL));
    }
    else
    {
        viCheckParm(rsspecan_SetAttributeViString(instrSession, repCap, RSSPECAN_ATTR_PHASE_INTEGRATION_WEIGHTING_FILTER, weightingFilter),
                5, "Weighting Filter");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN This function defines the new weigthing filter.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR name/This control specifies the name of new filter.
/// HIPAR comment/This control specifies the weigting filter comment.
/// HIPAR arraySize/Defines the number of of values in arrays Frequency and Level.
/// HIPAR frequency/Array of frequency values of the shape of the weighting filter.
/// HIPAR level/Array of level values of the shape of the weighting filter.
ViStatus _VI_FUNC rsspecan_PhaseWeightingFilterCreate(ViSession instrSession,
                                                      ViString  name,
                                                      ViString  comment,
                                                      ViInt32   arraySize,
                                                      ViReal64  frequency[],
                                                      ViReal64  level[])
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_PHASE_INTEGRATION_WEIGHTING_FILTER_NAME, name),
            2, "Name");

    viCheckParm(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_PHASE_INTEGRATION_WEIGHTING_FILTER_COMMENT, comment),
            3, "Comment");

    checkErr(
        RsCore_WriteAsciiViReal64ArraysInterleaved(instrSession, "CORR:WEIG:DATA ", arraySize, frequency, level, NULL,
            NULL, NULL, NULL));

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN Document your function here.
/// HIFN You may use multiple lines for documentation.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR filter/This control specifies the name of filter.
ViStatus _VI_FUNC rsspecan_PhaseWeightingFilterDelete(ViSession instrSession,
                                                      ViString  filter)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_PHASE_INTEGRATION_WEIGHTING_FILTER_SELECT, filter),
            2, "Filter");
    checkErr(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_PHASE_INTEGRATION_WEIGHTING_FILTER_DELETE, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function defines spur list.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR sortOrder/Selects the sort order of the spurs in the spur list.
/// HIPAR weightingFilter/Selects a spurious weighting filter.
ViStatus _VI_FUNC rsspecan_ConfigurePhaseSpuriousList(ViSession instrSession,
                                                      ViInt32   sortOrder,
                                                      ViString  weightingFilter)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, NULL, RSSPECAN_ATTR_PHASE_SPURIOUS_SORT_ORDER, sortOrder),
            2, "Sort Order");

    if (strlen(weightingFilter) == 0)
    {
        checkErr(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_PHASE_SPURIOUS_WEIGHTING_FILTER_NONE, NULL));
    }
    else
    {
        viCheckParm(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_PHASE_SPURIOUS_WEIGHTING_FILTER, weightingFilter),
                3, "Weighting Filter");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Provides functionality to enable an automatic shift of the color of
/// HIFN  the displayed spurious lines.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR color/Provides functionality to enable an automatic shift of the color of
/// HIPAR color/the displayed spurious lines.
ViStatus _VI_FUNC rsspecan_ConfigurePhaseSpuriousColor(ViSession instrSession,
                                                       ViInt32   color)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, NULL, RSSPECAN_ATTR_PHASE_SPURIOUS_COLOR, color),
            2, "Color");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Filtering spurs is a method to show or hide items of the spurious
/// HIFN  list.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR mode/Defines the spurious filter mode
/// HIPAR includeHarmonics/Defines if harmonics are included in the spurious filter.
/// HIPAR Range/Defines the range of the spurious filter. The range is defined as a
/// HIPAR Range/multiple of the resolution bandwidth.
ViStatus _VI_FUNC rsspecan_ConfigurePhaseSpuriousFilter(ViSession instrSession,
                                                        ViInt32   mode,
                                                        ViBoolean includeHarmonics,
                                                        ViReal64  range)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, NULL, RSSPECAN_ATTR_PHASE_SPURIOUS_FILTER_MODE, mode),
            2, "Mode");

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, NULL, RSSPECAN_ATTR_PHASE_SPURIOUS_FILTER_INCLUDE_HARMONICS, includeHarmonics),
            3, "Include Harmonics");

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, NULL, RSSPECAN_ATTR_PHASE_SPURIOUS_FILTER_RANGE, range),
            4, "Range");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function creates a new offset table with defined frequency
/// HIFN  ranges.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR arraySize/Sets the size of Offset Values.
/// HIPAR offsetValues/Creates a new offset table with defined frequency ranges.
ViStatus _VI_FUNC rsspecan_ConfigurePhaseSpuriousFilterOffset(
                            ViSession instrSession,
                            ViInt32   arraySize,
                            ViReal64  offsetValues[])
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_WriteAsciiViReal64Array(instrSession, "SENS:SPUR:FILT:OFFS ", offsetValues, arraySize));

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Returns the name of the selected spurious filter.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR bufferSize/Sets the size of Name parameter.
/// HIPAR name/Returns the name of the selected spurious filter.
ViStatus _VI_FUNC rsspecan_QueryPhaseSpuriousFilterName(ViSession instrSession,
                                                        ViInt32   bufferSize,
                                                        ViChar    name[])
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_GetAttributeViString(instrSession, NULL, RSSPECAN_ATTR_PHASE_SPURIOUS_FILTER_NAME,
            bufferSize, name),
            3, "Name");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Clears the currently selected spurious filter.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsspecan_ClearPhaseSpuriousFilter(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsspecan_SetAttributeViString(instrSession, NULL, RSSPECAN_ATTR_PHASE_SPURIOUS_FILTER_CLEAR, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Saves the spurious filter configuration to a file. The spurious filter
/// HIFN  name can be defined. Location of filter files:
/// HIFN  C:\R_S\instr\SpuriousFilter\
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR filename/Saves the spurious filter configuration to a file. The spurious filter
/// HIPAR filename/name can be defined. Location of filter files:
/// HIPAR filename/C:\R_S\instr\SpuriousFilter
ViStatus _VI_FUNC rsspecan_SavePhaseSpuriousFilter(ViSession instrSession,
                                                   ViString  filename)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViString(instrSession, NULL, RSSPECAN_ATTR_PHASE_SPURIOUS_FILTER_SAVE, filename),
            2, "Filename");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Loads the spurious filter configuration from a file. Location of
/// HIFN  filter files: C:\R_S\instr\SpuriousFilter\
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR filename/Loads the spurious filter configuration from a file. Location of
/// HIPAR filename/filter files: C:\R_S\instr\SpuriousFilter
ViStatus _VI_FUNC rsspecan_LoadPhaseSpuriousFilter(ViSession instrSession,
                                                   ViString  filename)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViString(instrSession, NULL, RSSPECAN_ATTR_PHASE_SPURIOUS_FILTER_LOAD, filename),
            2, "Filename");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function turns DC power sources on and off in general.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR state/Turns DC power sources on and off in general.
ViStatus _VI_FUNC rsspecan_DCOutputPowerState(ViSession instrSession,
                                              ViBoolean state)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_OUTPUT_DC_POWER_ENABLED, state),
            2, "State");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function couples or decouples the DC power configuration
/// HIFN across measurement channels.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR coupling/Couples or decouples the DC power configuration across
/// HIPAR coupling/measurement channels.
ViStatus _VI_FUNC rsspecan_DCOutputPowerCoupling(ViSession instrSession,
                                                 ViBoolean coupling)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_OUTPUT_DC_POWER_COUPLING_ENABLED, coupling),
            2, "Coupling");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function enables/disapbles DC Supply Port.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR state/Enables/disables the  DC power supply port.
ViStatus _VI_FUNC rsspecan_ConfigureDCSupplyPortState(ViSession instrSession,
                                                      ViBoolean state)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_OUTPUT_DC_POWER_SUPPLY_PORT_ENABLED, state),
            2, "State");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function configures DC Supply Port.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR mode/Select whether you want to control the output in terms of
/// HIPAR mode/current or voltage.
/// HIPAR maxVA/Defines the maximum current or voltage that may be supplied by
/// HIPAR maxVA/the Vsupply source.
/// HIPAR level/Defines the output level for the Vsupply source.
/// HIPAR highLimit/Defines the maximum value that may be supplied by the V supply
/// HIPAR highLimit/source.
/// HIPAR lowLimit/Defines the minimum value that may be supplied by the V supply
/// HIPAR lowLimit/source.
ViStatus _VI_FUNC rsspecan_ConfigureDCSupplyPort(ViSession instrSession,
                                                 ViInt32   mode,
                                                 ViReal64  maxVA,
                                                 ViReal64  level,
                                                 ViReal64  highLimit,
                                                 ViReal64  lowLimit)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_OUTUPT_DC_POWER_SUPPLY_PORT_MODE, mode),
            2, "Mode");

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_OUTPUT_DC_POWER_SUPPLY_PORT_MAXIMUM, maxVA),
            3, "Max V/A");

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_OUTPUT_DC_POWER_SUPPLY_PORT_LEVEL, level),
            4, "Level");

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_OUTPUT_DC_POWER_SUPPLY_PORT_LIMIT_MAX, highLimit),
            5, "High Limit");

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_OUTPUT_DC_POWER_SUPPLY_PORT_LIMIT_MIN, lowLimit),
            6, "Low Limit");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function enables/disbles DC Power Tuning Port.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR state/Enables/disables the  DC power tuning port.
ViStatus _VI_FUNC rsspecan_ConfigureDCTuningPortState(ViSession instrSession,
                                                      ViBoolean state)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_OUTPUT_DC_POWER_TUNING_PORT_ENABLED, state),
            2, "State");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function configures DC Power Tuning Port.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR level/Defines the output voltage level for the V tune source.
/// HIPAR highLimit/Defines the maximum value that may be supplied by the V tune source.
/// HIPAR lowLimit/Defines the minimum value that may be supplied by the V tune source.
ViStatus _VI_FUNC rsspecan_ConfigureDCTuningPort(ViSession instrSession,
                                                 ViReal64  level,
                                                 ViReal64  highLimit,
                                                 ViReal64  lowLimit)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_OUTPUT_DC_POWER_TUNING_PORT_LEVEL, level),
            2, "Level");

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_OUTPUT_DC_POWER_TUNING_PORT_LIMIT_MAX, highLimit),
            3, "High Limit");

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_OUTPUT_DC_POWER_TUNING_PORT_LIMIT_MIN, lowLimit),
            4, "Low Limit");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function enables/disables DC Power AUX Port.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR state/Enables/disables the DC power AUX port.
ViStatus _VI_FUNC rsspecan_ConfigureDCAUXPortState(ViSession instrSession,
                                                   ViBoolean state)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_OUTPUT_DC_POWER_AUX_PORT_ENABLED, state),
            2, "State");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function configures DC Power AUX Port.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR level/Defines the output voltage level for the Vaux source.
/// HIPAR highLimit/Defines the maximum value that may be supplied by the V aux source.
/// HIPAR lowLimit/Defines the minimum value that may be supplied by the V aux source.
ViStatus _VI_FUNC rsspecan_ConfigureDCAUXPort(ViSession instrSession,
                                              ViReal64  level,
                                              ViReal64  highLimit,
                                              ViReal64  lowLimit)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_OUTPUT_DC_POWER_AUX_PORT_LEVEL, level),
            2, "Level");

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_OUTPUT_DC_POWER_AUX_PORT_LIMIT_MAX, highLimit),
            3, "High Limit");

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_OUTPUT_DC_POWER_AUX_PORT_LIMIT_MIN, lowLimit),
            4, "Low Limit");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function returns the maximum current of selected port.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR port/Selects the type of port.
/// HIPAR value/Returns the maximum current of selected port.
ViStatus _VI_FUNC rsspecan_QueryDCPortMaximum(ViSession instrSession,
                                              ViInt32   port,
                                              ViReal64* value)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    switch (port)
    {
    case RSSPECAN_VAL_DC_PORT_SUPPLY:
        viCheckParm(rsspecan_GetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_OUTPUT_DC_POWER_SUPPLY_PORT_CURRENT_MAXIMUM, value),
                3, "Value");
        break;
    case RSSPECAN_VAL_DC_PORT_TUNE:
        viCheckParm(rsspecan_GetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_OUTPUT_DC_POWER_TUNING_PORT_CURRENT_MAXIMUM, value),
                3, "Value");
        break;
    case RSSPECAN_VAL_DC_PORT_AUX:
        viCheckParm(rsspecan_GetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_OUTPUT_DC_POWER_AUX_CURRENT_MAXIMUM, value),
                3, "Value");
        break;
    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, port), 2, "Port");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function returns a list of measured voltages from ports.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR arraySize/Sets the number of values allocated in Results array.
/// HIPAR results/Returns measured voltages from ports in the following order:
/// HIPAR returnedValues/Returns number of values returned from instrument.
ViStatus _VI_FUNC rsspecan_QueryDCOutputVoltageResults(ViSession instrSession,
                                                       ViInt32   arraySize,
                                                       ViReal64  results[],
                                                       ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentModel(instrSession, "FSWP"));

    checkErr(
        RsCore_QueryFloatArrayToUserBuffer(instrSession, "SOUR:VOLT:SEQ:RES?", arraySize, results, returnedValues));

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function returns a list of measured currents from ports.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR arraySize/Sets the number of values allocated in Results array.
/// HIPAR results/Returns measured currents from ports in the following order:
/// HIPAR returnedValues/Returns number of values returned from instrument.
ViStatus _VI_FUNC rsspecan_QueryDCOutputCurrentResults(ViSession instrSession,
                                                       ViInt32   arraySize,
                                                       ViReal64  results[],
                                                       ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentModel(instrSession, "FSWP"));

    checkErr(
        RsCore_QueryFloatArrayToUserBuffer(instrSession, "SOUR:CURR:SEQ:RES?", arraySize, results, returnedValues));

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function returns a list of measured powers from ports.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR arraySize/Sets the number of values allocated in Results array.
/// HIPAR results/Returns measured powers from ports in the following order:
/// HIPAR returnedValues/Returns number of values returned from instrument.
ViStatus _VI_FUNC rsspecan_QueryDCOutputPowerResults(ViSession instrSession,
                                                     ViInt32   arraySize,
                                                     ViReal64  results[],
                                                     ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentModel(instrSession, "FSWP"));

    checkErr(RsCore_QueryFloatArrayToUserBuffer(instrSession, "SOUR:POW:SEQ:RES?", arraySize, results, returnedValues));

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function turns the optional signal source output on and off.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR state/Enables/disables the specified DC power supply port.
ViStatus _VI_FUNC rsspecan_SignalSourceState(ViSession instrSession,
                                             ViBoolean state)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_OUTPUT_SIGNAL_ENABLED, state),
            2, "State");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function defines the signal characteristics of simple sine
/// HIFN signal generated by signal source.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR frequency/Defines the frequency of the signal that is generated by the
/// HIPAR frequency/signal source.
/// HIPAR level/Defines the level of the signal that is generated by the signal
/// HIPAR level/source.
ViStatus _VI_FUNC rsspecan_SignalSourceSignalCharacteristics(ViSession instrSession,
                                                             ViReal64  frequency,
                                                             ViReal64  level)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_OUTPUT_SIGNAL_FREQUENCY, frequency),
            2, "Frequency");

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_OUTPUT_SIGNAL_LEVEL, level),
            3, "Level");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function defines the frequency stepsize of the signal generated
/// HIFN  by the optional signal source.
/// HIFN
/// HIFN     Note(s):
/// HIFN     (1) This function is available with R&S FSWP only.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR frequencyStepsize/This control defines the frequency stepsize of the signal generated by
/// HIPAR frequencyStepsize/the optional signal source.
ViStatus _VI_FUNC rsspecan_SignalSourceSignalFrequencyStepsize(
                            ViSession instrSession,
                            ViReal64  frequencyStepsize)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_SIGNAL_SOURCE_FREQUENCY_STEPSIZE, frequencyStepsize),
            2, "Frequency Stepsize");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function couples or decouples the signal source
/// HIFN configuration across measurement channels.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR coupling/Couples or decouples the DC power configuration across
/// HIPAR coupling/measurement channels.
ViStatus _VI_FUNC rsspecan_SignalSourceCoupling(ViSession instrSession,
                                                ViBoolean coupling)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_OUTPUT_SIGNAL_SOURCE_COUPLING_ENABLED, coupling),
            2, "Coupling");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function turns the DUT bypass on and off.
/// HIFN configuration across measurement channels.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR DUTBypass/Enables or disables the DUT bypass on and off.
ViStatus _VI_FUNC rsspecan_SignalSourceDUTBypass(ViSession instrSession,
                                                 ViBoolean DUTBypass)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_SOURCE_DUT_BYPASS, DUTBypass),
            2, "DUT Bypass");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN Turns internal pulse modulation for pulsed measurements on and off.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR pulseModulation/Enables or disables the pulse modulation on and off.
ViStatus _VI_FUNC rsspecan_SignalSourcePulseModulation(ViSession instrSession,
                                                       ViBoolean pulseModulation)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_SOURCE_PULSE_MODULATION, pulseModulation),
            2, "Pulse Modulation");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function configures the pulse parameters.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR period/Sets the pulse period (distance between two consecutive pulses) of the pulse that is generated.
/// HIPAR width/Sets the length of the pulse that is generated.
/// HIPAR triggerOutput/Selects the signal type provided at the Trigger Output connector.
ViStatus _VI_FUNC rsspecan_SignalSourcePulseSettings(ViSession instrSession,
                                                     ViReal64  period,
                                                     ViReal64  width,
                                                     ViInt32   triggerOutput)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_SOURCE_PULSE_PERIOD, period),
            2, "Period");

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_SOURCE_PULSE_WIDTH, width),
            3, "Width");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_SOURCE_PULSE_TRIGGER_OUTPUT, triggerOutput),
            4, "Trigger Output");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN Turns VCO measurement on and off in general.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR state/Turns VCO measurement on and off in general.
ViStatus _VI_FUNC rsspecan_VCOState(ViSession instrSession,
                                    ViBoolean state)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_VCO_MEASUREMENT_STATE, state),
            2, "State");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function configures VCO DC source.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR sweepSource/Selects the source (connector) which tunes the VCO.
/// HIPAR fixSource/Selects the DC source that is considered in the Current /
/// HIPAR fixSource/Voltage result display.
ViStatus _VI_FUNC rsspecan_VCODCSource(ViSession instrSession,
                                       ViInt32   sweepSource,
                                       ViInt32   fixSource)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_VCO_SWEEP_SOURCE, sweepSource),
            2, "Sweep Source");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_VCO_SWEEP_FIX_SOURCE, fixSource),
            3, "Fix Source");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN This function configures VCO Sweep.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR start/Defines the start value of the tuning range.
/// HIPAR stop/Defines the stop value of the tuning range.
/// HIPAR measPoints/Defines the number of measurement points for the VCO
/// HIPAR measPoints/measurement.
/// HIPAR initialSettlingTime/Defines an initial delay time that allows the DUT to settle
/// HIPAR initialSettlingTime/before the first measurement starts.
/// HIPAR settlingTime/Defines a delay time that allows the DUT to settle when a new
/// HIPAR settlingTime/tuning voltage is applied and before the measurement starts.
/// HIPAR frequencyResolution/Selects the frequency resolution for VCO characterization.
ViStatus _VI_FUNC rsspecan_VCOSweep(ViSession instrSession,
                                    ViReal64  start,
                                    ViReal64  stop,
                                    ViInt32   measPoints,
                                    ViReal64  initialSettlingTime,
                                    ViReal64  settlingTime,
                                    ViReal64  frequencyResolution)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_VCO_SWEEP_START, start),
            2, "Start");

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_VCO_SWEEP_STOP, stop),
            3, "Stop");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_VCO_SWEEP_POINTS, measPoints),
            4, "Meas Points");

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_VCO_SWEEP_INITIAL_SETTLING_TIME, initialSettlingTime),
            5, "Initial Settling Time");

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_VCO_SWEEP_SETTLING_TIME, settlingTime),
            6, "Settling Time");

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_VCO_SWEEP_FREQUENCY_RESOLUTION, frequencyResolution),
            7, "Frequency Resolution");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function selects the Spot Noise vs Tune measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR state/Turns Spot Noise vs Tune measurement measurement on and off in
/// HIPAR state/general.
ViStatus _VI_FUNC rsspecan_SpotNoiseVsTuneState(ViSession instrSession,
                                                ViBoolean state)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_SPOT_NOISE_VS_TUNE_MEASUREMENT_STATE, state),
            2, "State");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function configures Spot Noise vs Tune DC source.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR sweepSource/Selects the source (connector) which tunes the Spot Noise vs
/// HIPAR sweepSource/Tune.
ViStatus _VI_FUNC rsspecan_SpotNoiseVsTuneSource(ViSession instrSession,
                                                 ViInt32   sweepSource)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_SPOT_NOISE_VS_TUNE_SWEEP_SOURCE, sweepSource),
            2, "Sweep Source");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function configures Spot Noise vs Tune Sweep.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR start/Defines the start value of the tuning range.
/// HIPAR stop/Defines the stop value of the tuning range.
/// HIPAR measPoints/Defines the number of measurement points for the Spot Noise vs
/// HIPAR sweepSource/Tune.
/// HIPAR initialSettlingTime/Defines an initial delay time that allows the DUT to settle
/// HIPAR initialSettlingTime/before the first measurement starts.
/// HIPAR settlingTime/Defines a delay time that allows the DUT to settle when a new
/// HIPAR settlingTime/tuning voltage is applied and before the measurement starts.
ViStatus _VI_FUNC rsspecan_SpotNoiseVsTuneSweep(ViSession instrSession,
                                                ViReal64  start,
                                                ViReal64  stop,
                                                ViInt32   measPoints,
                                                ViReal64  initialSettlingTime,
                                                ViReal64  settlingTime)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_SPOT_NOISE_VS_TUNE_SWEEP_START, start),
            2, "Start");

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_SPOT_NOISE_VS_TUNE_SWEEP_STOP, stop),
            3, "Stop");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_SPOT_NOISE_VS_TUNE_SWEEP_POINTS, measPoints),
            4, "Meas Points");

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_SPOT_NOISE_VS_TUNE_SWEEP_INITIAL_SETTLING_TIME, initialSettlingTime),
            5, "Initial Settling Time");

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_SPOT_NOISE_VS_TUNE_SWEEP_SETTLING_TIME, settlingTime),
            6, "Settling Time");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function selects the Transient Analysis measurement.
/// HIFN
/// HIFN     Note(s):
/// HIFN
/// HIFN     (1) This function is available with R&S FSWP only.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR state/This control selects the Transient Analysis measurement.
ViStatus _VI_FUNC rsspecan_TransientAnalysisState(ViSession instrSession,
                                                  ViBoolean state)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_TRANSIENT_ANALYSIS_MEASUREMENT_STATE, state),
            2, "State");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function selects the transient measurement mode.
/// HIFN
/// HIFN     Note(s):
/// HIFN
/// HIFN     (1) This function is available with R&S FSWP only.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR mode/This control selects the transient measurement mode.
ViStatus _VI_FUNC rsspecan_TransientAnalysisMeasurementMode(ViSession instrSession,
                                                            ViInt32   mode)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_TRANSIENT_ANALYSIS_MODE, mode),
            2, "Mode");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function defines the reference level (expected signal level).
/// HIFN  Note that the reference level is coupled to the attenuation.
/// HIFN
/// HIFN     Note(s):
/// HIFN
/// HIFN     (1) This function is available with R&S FSWP only.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR referenceLevel/This control defines the reference level (expected signal level). Note
/// HIPAR referenceLevel/that the reference level is coupled to the attenuation.
ViStatus _VI_FUNC rsspecan_TransientAnalysisReferenceLevel(ViSession instrSession,
                                                           ViReal64  referenceLevel)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_TRANSIENT_ANALYSIS_REFERENCE_LEVEL, referenceLevel),
            2, "Reference Level");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function selects the AF coupling settings.
/// HIFN
/// HIFN     Note(s):
/// HIFN
/// HIFN     (1) This function is available with R&S FSWP only.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR afCoupling/This control selects the AF coupling mode.
/// HIPAR zeroPhasePositioningMode/This control selects the zero phase positioning mode.
/// HIPAR zeroPhasePosition/This control defines the position of the zero phase.
ViStatus _VI_FUNC rsspecan_TransientAnalysisYAxisScaleAFCoupling(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   afCoupling,
                            ViInt32   zeroPhasePositioningMode,
                            ViReal64  zeroPhasePosition)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, window, 0, 16),
            2, "Window");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_TRANSIENT_ANALYSIS_Y_AXIS_UNIT,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap),
            3, "Window");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_TRANSIENT_ANALYSIS_Y_AXIS_SCALE_AF_COUPLING, afCoupling),
            2, "AF Coupling");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_TRANSIENT_ANALYSIS_Y_AXIS_SCALE_ZERO_PHASE_POSITIONING_MODE, zeroPhasePositioningMode),
            3, "Zero Phase Positioning Mode");

    if (afCoupling == RSSPECAN_VAL_DC && zeroPhasePositioningMode == RSSPECAN_VAL_MAN)
    {
        viCheckParm(rsspecan_SetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_TRANSIENT_ANALYSIS_Y_AXIS_SCALE_ZERO_PHASE_POSITION, zeroPhasePosition),
                4, "Zero Phase Position");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function selects the unit in which the phase results are
/// HIFN  displayed.
/// HIFN
/// HIFN     Note(s):
/// HIFN
/// HIFN     (1) This function is available with R&S FSWP only.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR unit/Selects the unit in which the phase results are displayed.
ViStatus _VI_FUNC rsspecan_TransientAnalysisYAxisUnit(ViSession instrSession,
                                                      ViInt32   window,
                                                      ViInt32   unit)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, window, 0, 16),
            2, "Window");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_TRANSIENT_ANALYSIS_Y_AXIS_UNIT,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap),
            3, "Window");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_TRANSIENT_ANALYSIS_Y_AXIS_UNIT, unit),
            3, "Unit");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


ViStatus _VI_FUNC rsspecan_TransientAnalysisYAxisGridMode(ViSession instrSession,
                                                          ViInt32   window,
                                                          ViInt32   yAxisGridMode)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_TRANSIENT_ANALYSIS_Y_AXIS_GRID_MODE,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap),
            2, "Window");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, window, 0, 16),
            2, "Window");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_TRANSIENT_ANALYSIS_Y_AXIS_GRID_MODE, yAxisGridMode),
            3, "Y Axis Grid Mode");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the trace persistence settings.
/// HIFN
/// HIFN     Note(s):
/// HIFN
/// HIFN     (1) This function is available with R&S FSWP only.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN
/// HIFN
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     DISPlay[:WINDow<n>]:TRACe<t>:PERSistence[:STATe]
/// HIFN     DISPlay[:WINDow<n>]:TRACe<t>:PERSistence:DECay
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR trace/Selects the trace.
/// HIPAR state/This control turns persistence for a trace on and off.
/// HIPAR decay/This control defines the time period that an event remains visible
/// HIPAR decay/until it fades away (decay).
ViStatus _VI_FUNC rsspecan_TransientAnalysisTracePersistence(ViSession instrSession,
                                                             ViInt32   window,
                                                             ViInt32   trace,
                                                             ViBoolean state,
                                                             ViReal64  decay)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_t[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, window, 0, 16),
            2, "Window");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, trace, 1, 6),
            3, "Trace");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_TRANSIENT_ANALYSIS_PERSISTENCE_STATE,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap),
            2, "Window");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_TRANSIENT_ANALYSIS_PERSISTENCE_STATE,
            "Trace", trace - 1, RS_REPCAP_BUF_SIZE, repCap_t),
            3, "Trace");

    strcat(repCap, ",");
    strcat(repCap, repCap_t);

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, repCap, RSSPECAN_ATTR_TRANSIENT_ANALYSIS_PERSISTENCE_STATE, state),
            4, "State");

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_TRANSIENT_ANALYSIS_PERSISTENCE_DECAY, decay),
            5, "Decay");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the settling time measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR state/Defines the state of settling time measurement.
/// HIPAR upperFrequencyLimit/Defines the upper frequency of settling time measurement.
/// HIPAR lowerFrequencyLimit/Defines the lower frequency of settling time measurement.
/// HIPAR diagram/Selects the diagram for settling time calculation.
/// HIPAR trace/Selects the trace for settling time calculation.
ViStatus _VI_FUNC rsspecan_ConfigureTransientAnalysisSettlingTime(
                            ViSession instrSession,
                            ViBoolean state,
                            ViReal64  upperFrequencyLimit,
                            ViReal64  lowerFrequencyLimit,
                            ViInt32   diagram,
                            ViInt32   trace)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, NULL, RSSPECAN_ATTR_TRANSIENT_ANALYSIS_SETTLING_TIME_ENABLED, state),
            2, "State");

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, NULL, RSSPECAN_ATTR_TRANSIENT_ANALYSIS_SETTLING_TIME_UPPER_FREQUENCY_LIMIT, upperFrequencyLimit),
            3, "Upper Frequency Limit");

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, NULL, RSSPECAN_ATTR_TRANSIENT_ANALYSIS_SETTLING_TIME_LOWER_FREQUENCY_LIMIT, lowerFrequencyLimit),
            4, "Lower Frequency Limit");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, NULL, RSSPECAN_ATTR_TRANSIENT_ANALYSIS_SETTLING_TIME_DIAGRAM, diagram),
            5, "Diagram");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, NULL, RSSPECAN_ATTR_TRANSIENT_ANALYSIS_SETTLING_TIME_TRACE, trace),
            6, "Trace");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Returns the settling time.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR result/Returns the settling time.
ViStatus _VI_FUNC rsspecan_QueryTransientAnalysisSettlingTimeResult(
                            ViSession instrSession,
                            ViReal64* result)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, NULL, RSSPECAN_ATTR_TRANSIENT_ANALYSIS_SETTLING_TIME_RESULT, result),
            2, "Result");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the frequency deviation measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR state/Defines the state of frequency deviation measurement.
/// HIPAR start/Defines the start frequency of frequency deviation measurement.
/// HIPAR stop/Defines the stop frequency of frequency deviation measurement.
/// HIPAR diagram/Selects the diagram for frequency deviation calculation.
/// HIPAR trace/Selects the trace for frequency deviation calculation.
ViStatus _VI_FUNC rsspecan_ConfigureTransientAnalysisFrequencyDeviation(
                            ViSession instrSession,
                            ViBoolean state,
                            ViReal64  start,
                            ViReal64  stop,
                            ViInt32   diagram,
                            ViInt32   trace)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, NULL, RSSPECAN_ATTR_TRANSIENT_ANALYSIS_FREQUENCY_DEVIATION_ENABLED, state),
            2, "State");

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, NULL, RSSPECAN_ATTR_TRANSIENT_ANALYSIS_FREQUENCY_DEVIATION_START, start),
            3, "Start");

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, NULL, RSSPECAN_ATTR_TRANSIENT_ANALYSIS_FREQUENCY_DEVIATION_STOP, stop),
            4, "Stop");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, NULL, RSSPECAN_ATTR_TRANSIENT_ANALYSIS_FREQUENCY_DEVIATION_DIAGRAM, diagram),
            5, "Diagram");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, NULL, RSSPECAN_ATTR_TRANSIENT_ANALYSIS_FREQUENCY_DEVIATION_TRACE, trace),
            6, "Trace");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Returns the data of the regression line.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR arraySize/Sets the size of X and Y Values.
/// HIPAR xValues/Creates a new offset table with defined frequency ranges.
/// HIPAR yValues/Creates a new offset table with defined frequency ranges.
/// HIPAR returnedValues/Returns the number of values readed from instrument.
ViStatus _VI_FUNC rsspecan_QueryTransientAnalysisFrequencyDeviationRegressionLineData(
                            ViSession instrSession,
                            ViInt32   arraySize,
                            ViReal64  xValues[],
                            ViReal64  yValues[],
                            ViInt32*  returnedValues)
{
    ViStatus  error = VI_SUCCESS;
    ViPReal64 res = NULL;
    ViInt32   ret_cnt;
    ViInt32   size;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_QueryFloatArray(instrSession, "CALC:FREQ:DEV:REGR?", &res, &ret_cnt));
    checkErr(rsspecan_CheckStatus(instrSession));

    size = (arraySize > (ret_cnt / 2)) ? ret_cnt / 2 : arraySize;

    checkErr(RsCore_DecimateViReal64ArrayToUserBuffers(res, ret_cnt, size, returnedValues, xValues, yValues, NULL, NULL, NULL, NULL));

Error:
    if (res) free(res);
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the number of points of the regression line.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR points/Queries the number of points of the regression line.
ViStatus _VI_FUNC rsspecan_QueryTransientAnalysisFrequencyDeviationRegressionLinePoints(
                            ViSession instrSession,
                            ViInt32*  points)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_GetAttributeViInt32(instrSession, NULL, RSSPECAN_ATTR_TRANSIENT_ANALYSIS_FREQUENCY_DEVIATION_REGRESSION_LINE_POINTS, points),
            2, "Points");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Returns the slope of the regression line. A single floating point
/// HIFN  value with the unit Hz/s is returned.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR slope/Returns the slope of the regression line. A single floating point
/// HIPAR slope/value with the unit Hz/s is returned.
ViStatus _VI_FUNC rsspecan_QueryTransientAnalysisFrequencyDeviationRegressionLineSlope(
                            ViSession instrSession,
                            ViReal64* slope)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, NULL, RSSPECAN_ATTR_TRANSIENT_ANALYSIS_FREQUENCY_DEVIATION_REGRESSION_LINE_SLOPE, slope),
            2, "Slope");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/*****************************************************************************
 * Function: Create Phase Limit Line2
 * Purpose:  This function creates new limit or selects existing one and
 *           configures basic parameters.
 *****************************************************************************/
ViStatus _VI_FUNC rsspecan_CreatePhaseLimitLine(ViSession instrSession,
                                                ViInt32   reserved,
                                                ViInt32   limit,
                                                ViString  name,
                                                ViString  comment,
                                                ViInt32   assigntoTrace)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE];
             reserved;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, limit, 1, 8),
            3, "Limit");
    viCheckParm(RsCore_InvalidStringLength(instrSession, name, 0, 8),
                4, "Name (string length)");
    viCheckParm(RsCore_InvalidStringLength(instrSession, comment, 0, 40),
                5, "Comment (string length)");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "L%ld", limit);

    viCheckParm(rsspecan_SetAttributeViString(instrSession, repCap, RSSPECAN_LIMIT_NAME, name),
            4, "Name");

    viCheckParm(rsspecan_SetAttributeViString(instrSession, repCap, RSSPECAN_LIMIT_COMMENT, comment),
            5, "Comment");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "L%ld", limit);
    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_LIMIT_ASSIGN_TRACE, assigntoTrace),
            6, "Assign to Trace");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/*****************************************************************************
 * Function: Define Phase Limit Line
 * Purpose:  This function defines specified limit line.
 *****************************************************************************/
ViStatus _VI_FUNC rsspecan_DefinePhaseLimitLine(ViSession instrSession,
                                                ViInt32   limit,
                                                ViInt32   type,
                                                ViInt32   count,
                                                ViReal64  xAxis[],
                                                ViReal64  amplitude[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, limit, 1, 8),
            2, "Limit");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, type, 0, 1),
            3, "Type");
    viCheckParm(RsCore_InvalidViInt32RangeMin(instrSession, count, 1),
                4, "Count");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, xAxis),
                5, "X Axis");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, amplitude),
                6, "Amplitude");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CALC:LIM%ld:CONT ", limit);
    checkErr(RsCore_WriteAsciiViReal64Array(instrSession, cmd, xAxis, count));
    checkErr(rsspecan_CheckStatus(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CALC:LIM%ld:%s ", limit, limTypeArr[type]);
    checkErr(RsCore_WriteAsciiViReal64Array(instrSession, cmd, xAxis, count));
    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/*****************************************************************************
 * Function: Enable Phase Limit Check2
 * Purpose:  This function switches on or off the limit check and/or limit
 *           line for the selected limit line in the selected measurement
 *           window.
 *****************************************************************************/
ViStatus _VI_FUNC rsspecan_EnablePhaseLimitCheck(ViSession instrSession,
                                                 ViInt32   reserved,
                                                 ViInt32   limit,
                                                 ViInt32   type,
                                                 ViBoolean limitEnabled,
                                                 ViBoolean checkEnabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE];
    ViInt32  attribute = 0;
             reserved;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, limit, 1, 8),
            3, "Limit");

    switch (type)
    {
    case RSSPECAN_VAL_LIMIT_LOWER:
        attribute = RSSPECAN_LIMIT_LOWER_STATE;
        break;
    case RSSPECAN_VAL_LIMIT_UPPER:
        attribute = RSSPECAN_LIMIT_UPPER_STATE;
        break;

    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, type), 4, "Type");
    }

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "L%ld", limit);

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, repCap, RSSPECAN_LIMIT_STATE, limitEnabled),
            5, "Limit Enabled");

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, repCap, attribute, checkEnabled),
            6, "Check Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/*****************************************************************************
 * Function: Move Phase Limit Line
 * Purpose:  This function moves specified limit line.
 *****************************************************************************/
ViStatus _VI_FUNC rsspecan_MovePhaseLimitLine(ViSession instrSession,
                                              ViInt32   limit,
                                              ViInt32   type,
                                              ViReal64  value)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE];
    ViInt32  attributes[] = {
             RSSPECAN_LIMIT_LOWER_SHIFT,
             RSSPECAN_LIMIT_UPPER_SHIFT,
             RSSPECAN_LIMIT_CONTROL_SHIFT
             };

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, limit, 1, 8),
            2, "Limit");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, type, 0, 2),
            3, "Type");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "L%ld", limit);

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, repCap, attributes[type], value),
            4, "Value");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/*****************************************************************************
 * Function: Copy Phase Limit Line
 * Purpose:  This function copies specified limit line into another one.
 *****************************************************************************/
ViStatus _VI_FUNC rsspecan_CopyPhaseLimitLine(ViSession instrSession,
                                              ViInt32   limit,
                                              ViInt32   copyTo)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, limit, 1, 8),
            2, "Limit");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "L%ld", limit);

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_LIMIT_COPY, copyTo),
            3, "Copy To");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/*****************************************************************************
 * Function: Clear Phase Limit Line Results
 * Purpose:  This function deletes the result of the current limit check.
 *****************************************************************************/
ViStatus _VI_FUNC rsspecan_ClearPhaseLimitLineResults(ViSession instrSession,
                                                      ViInt32   limit)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, limit, 1, 8),
            2, "Limit");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "L%ld", limit);

    checkErr(rsspecan_SetAttributeViString(instrSession, repCap, RSSPECAN_LIMIT_CLEAR, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/*****************************************************************************
 * Function: Delete Phase Limit Line
 * Purpose:  This function removes specified limit line.
 *****************************************************************************/
ViStatus _VI_FUNC rsspecan_DeletePhaseLimitLine(ViSession instrSession,
                                                ViInt32   limit)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, limit, 1, 8),
            2, "Limit");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "L%ld", limit);

    checkErr(rsspecan_SetAttributeViString(instrSession, repCap, RSSPECAN_LIMIT_DELETE, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function queries the names of all active limit lines.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR bufferSize/Pass the number of bytes in the string you specify for the Active Limit Lines parameter.
/// HIPAR activeLimitLines/This control returns the names of all active limit lines in  alphabetical order.
ViStatus _VI_FUNC rsspecan_GetActivePhaseLimitLine(ViSession instrSession,
                                                   ViInt32   bufferSize,
                                                   ViChar    activeLimitLines[])
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_GetAttributeViString(instrSession, "", RSSPECAN_ATTR_LIMIT_LINE_ACTIVE,
            bufferSize, activeLimitLines),
            3, "Active Limit Lines");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function configures the phase noise limit parameters.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR shapeType/This control selects the shape of a phase noise limit line.
/// HIPAR selectedTrace/Selects the trace(s) to assign a phase noise limit line to.
/// HIPAR noiseFloor/This control  defines the noise floor level of the DUT.
/// HIPAR segment/Selects the limit line segment.
/// HIPAR cornerFrequency/Sets the start frequency of a segment of a phase noise limit line.
/// HIPAR slope/This control defines the slope for a phase noise limit line segment.
ViStatus _VI_FUNC rsspecan_ConfigurePhaseNoiseLimitSettings(ViSession instrSession,
                                                            ViInt32   shapeType,
                                                            ViInt32   selectedTrace,
                                                            ViReal64  noiseFloor,
                                                            ViInt32   segment,
                                                            ViReal64  cornerFrequency,
                                                            ViReal64  slope)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, segment, 1, 5),
            5, "Segment");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_PHASE_NOISE_LIMIT_SLOPE,
            "pnlSegment", segment - 1, RS_REPCAP_BUF_SIZE, repCap),
            5, "Segment");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_PHASE_NOISE_LIMIT_SHAPE_TYPE, shapeType),
            2, "Shape Type");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_PHASE_NOISE_LIMIT_TRACE, selectedTrace),
            3, "Selected Trace");

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_PHASE_NOISE_LIMIT_LEVEL, noiseFloor),
            4, "Noise Floor");

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_PHASE_NOISE_LIMIT_CORNER_FREQUENCY, cornerFrequency),
            6, "Corner Frequency");

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_PHASE_NOISE_LIMIT_SLOPE, slope),
            7, "Slope");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function creates a new user limit line from the data of a phase
/// HIFN  noise limit line.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR limitLine/Number of the limit line the phase noise limit line is copied to. An
/// HIPAR limitLine/existing limit line in that slot is overwritten.
ViStatus _VI_FUNC rsspecan_PhaseCopyToUserLimitLine(ViSession instrSession,
                                                    ViInt32   limitLine)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, limitLine, 1, 8),
            2, "Limit Line");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "LL%ld", limitLine);

    checkErr(rsspecan_SetAttributeViString(instrSession, repCap, RSSPECAN_ATTR_PHASE_COPY_TO_USER_LIMIT_LINE, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function queries the limit check results for phase noise limit lines.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR limitCheck/This control returns the limit check results for phase noise limit lines.
ViStatus _VI_FUNC rsspecan_GetPhaseNoiseLimitCheckState(ViSession instrSession,
                                                        ViInt32*  limitCheck)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_GetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_PHASE_NOISE_LIMIT_CHECK_STATE, limitCheck),
            2, "Limit Check");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function queries the currently selected limit line is compatible
/// HIFN  to the current measurement configuration.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSSPECAN_ATTR_LIMIT_LINE_COMPATIBLE
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     CALCulate<n>:LIMit<k>:COMPatible?
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR limitCompatible/This control returns the currently selected limit line is compatible
/// HIPAR limitCompatible/to the current measurement configuration.
ViStatus _VI_FUNC rsspecan_GetPhaseNoiseLimitLineCompatible(ViSession instrSession,
                                                            ViInt32*  limitCompatible)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_GetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_LIMIT_LINE_COMPATIBLE, limitCompatible),
            2, "Limit Compatible");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/*****************************************************************************
 * Function: Get Phase Limit Check Result2
 * Purpose:  This function queries the result of the limit check of the limit
 *           line indicated in the selected measurement window. It should be
 *           noted that a complete sweep must have been performed for obtaining
 *           a valid result. A synchronization with  *OPC, *OPC? or *WAI should
 *           therefore be provided.
 *****************************************************************************/
ViStatus _VI_FUNC rsspecan_GetPhaseLimitCheckResult(ViSession instrSession,
                                                    ViInt32   reserved,
                                                    ViInt32   limit,
                                                    ViInt32*  state)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE];
             reserved;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, limit, 1, 8),
            3, "Limit");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "L%ld", limit);

    viCheckParm(rsspecan_GetAttributeViInt32(instrSession, repCap, RSSPECAN_LIMIT_CHECK_RESULT, state),
            4, "State");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function queries the result of the spurious jitter.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control sets the window.
/// HIPAR trace/Selects the trace.
/// HIPAR type/Selects the type of returned value.
/// HIPAR value/The jitter result value.
ViStatus _VI_FUNC rsspecan_GetPhaseJitterResult(ViSession instrSession,
                                                ViInt32   window,
                                                ViInt32   trace,
                                                ViInt32   type,
                                                ViReal64* value)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_w[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_t[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_PHASE_DISCRETE_JITTER_RESULT,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap_w),
            2, "Window");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_PHASE_DISCRETE_JITTER_RESULT,
            "Trace", trace - 1, RS_REPCAP_BUF_SIZE, repCap_t),
            3, "Trace");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "%s,%s", repCap_w, repCap_t);

    switch (type)
    {
    case RSSPECAN_VAL_JITTER_DISCRETE:
        viCheckParm(rsspecan_GetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_PHASE_DISCRETE_JITTER_RESULT, value),
                5, "Value");
        break;
    case RSSPECAN_VAL_JITTER_RANDOM:
        viCheckParm(rsspecan_GetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_PHASE_RANDOM_JITTER_RESULT, value),
                5, "Value");
        break;
    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, type), 4, "Type");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN This function queries the result of the integrated measurements.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control sets the window.
/// HIPAR trace/Selects the trace.
/// HIPAR range/This control selects the integration range.
/// HIPAR type/Selects the type of returned value.
/// HIPAR value/The result value.
ViStatus _VI_FUNC rsspecan_GetPhaseIntegratedMeasurementResult(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   trace,
                            ViInt32   range,
                            ViInt32   type,
                            ViReal64* value)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_w[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_t[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_r[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_PHASE_RANGE_RESIDUAL_IPN,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap_w),
            2, "Window");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_PHASE_RANGE_RESIDUAL_IPN,
            "Trace", trace - 1, RS_REPCAP_BUF_SIZE, repCap_t),
            3, "Trace");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_PHASE_RANGE_RESIDUAL_IPN,
            "IntRange", range - 1, RS_REPCAP_BUF_SIZE, repCap_r),
            4, "Range");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "%s,%s,%s", repCap_w, repCap_t, repCap_r);

    switch (type)
    {
    case RSSPECAN_VAL_PHASE_RESIDUAL_IPN:
        viCheckParm(rsspecan_GetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_PHASE_RANGE_RESIDUAL_IPN, value),
                6, "Value");
        break;
    case RSSPECAN_VAL_PHASE_RESIDUAL_RFM:
        viCheckParm(rsspecan_GetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_PHASE_RANGE_RESIDUAL_FM, value),
                6, "Value");
        break;
    case RSSPECAN_VAL_PHASE_RESIDUAL_RMS:
        viCheckParm(rsspecan_GetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_PHASE_RANGE_RESIDUAL_RMS_JITTER, value),
                6, "Value");
        break;
    case RSSPECAN_VAL_PHASE_RESIDUAL_RPM:
        viCheckParm(rsspecan_GetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_PHASE_RANGE_RESIDUAL_PM, value),
                6, "Value");
        break;
    case RSSPECAN_VAL_PHASE_RESIDUAL_RAM:
        viCheckParm(rsspecan_GetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_PHASE_RANGE_RESIDUAL_AM, value),
                6, "Value");
        break;
    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, type), 5, "Type");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Exports the spur information to a file.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control sets the window.
/// HIPAR trace/Selects the trace.
/// HIPAR filename/String containing the path and name of the target file.
ViStatus _VI_FUNC rsspecan_ExportPhaseSpurs(ViSession instrSession,
                                            ViInt32   window,
                                            ViInt32   trace,
                                            ViString  filename)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   parameters[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_HIDDEN_PHASE_EXPORT_SPUR,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap),
            2, "Window");

    snprintf (parameters, RS_MAX_MESSAGE_BUF_SIZE, "%s,%ld", filename, trace);

    viCheckParm(rsspecan_SetAttributeViString(instrSession, repCap, RSSPECAN_ATTR_HIDDEN_PHASE_EXPORT_SPUR, parameters),
            4, "Filename");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries number of points of trace data merged with spurs.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control sets the window.
/// HIPAR trace/Selects the trace.
/// HIPAR type/Selects the axis.
/// HIPAR points/Queries number of points of trace data merged with spurs.
ViStatus _VI_FUNC rsspecan_GetPhaseNumberOfTraceDataPoints(ViSession instrSession,
                                                           ViInt32   window,
                                                           ViInt32   trace,
                                                           ViInt32   type,
                                                           ViInt32*  points)
{
    ViStatus error = VI_SUCCESS;
    ViAttr   attr[] = {RSSPECAN_ATTR_HIDDEN_PHASE_NUMBER_OF_TRACE_DATA_POINTS_X,
             RSSPECAN_ATTR_HIDDEN_PHASE_NUMBER_OF_TRACE_DATA_POINTS_Y};
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   parameter[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_HIDDEN_PHASE_NUMBER_OF_TRACE_DATA_POINTS_X,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap),
            2, "Window");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, type, RSSPECAN_VAL_AXIS_X, RSSPECAN_VAL_AXIS_Y),
            4, "Type");

    snprintf (parameter, RS_MAX_MESSAGE_BUF_SIZE, "TRACe%ld", trace);

    viCheckParm(RsCore_GetAttributeViInt32WithQparam(instrSession, repCap, attr[type], parameter, points),
            5, "Points");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/*****************************************************************************
 * Function: Trace Phase Settings
 * Purpose:  This function configures the Phase Trace Settings.
 *****************************************************************************/
ViStatus _VI_FUNC rsspecan_TracePhaseSettings(ViSession instrSession,
                                              ViInt32   traceNumber,
                                              ViBoolean traceState,
                                              ViInt32   traceMode,
                                              ViBoolean smoothing)
{
    ViStatus error = VI_SUCCESS;
    ViChar   traceNR_W[RS_MAX_MESSAGE_BUF_SIZE] = "";
    ViChar   traceNR[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, traceNumber, 1, 3),
            2, "Trace Number");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, traceMode, RSSPECAN_TRAC_MOD_WRITE, RSSPECAN_TRAC_MOD_MINH),
            4, "Trace Mode");

    snprintf(traceNR, RS_MAX_MESSAGE_BUF_SIZE, "TR%ld", traceNumber);
    snprintf(traceNR_W, RS_MAX_MESSAGE_BUF_SIZE, "Win0,TR%ld", traceNumber);

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, traceNR_W, RSSPECAN_ATTR_TRACE_STATE, traceState),
            3, "Trace State");

    if (traceState)
    {
        viCheckParm(rsspecan_SetAttributeViInt32(instrSession, traceNR_W, RSSPECAN_ATTR_TRACE_TYPE, traceMode),
                4, "Trace Mode");

        viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, traceNR, RSSPECAN_ATTR_PHASE_DISP_TRACE_SMO, smoothing),
                5, "Smoothing");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function turns spur suppression on individual traces on and
/// HIFN off.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR traceNumber/This control selects the trace number.
/// HIPAR spurRemoval/Switches on or off spur suppression on individual trace.
ViStatus _VI_FUNC rsspecan_TracePhaseSpur(ViSession instrSession,
                                          ViInt32   traceNumber,
                                          ViBoolean spurRemoval)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_PHASE_DISP_TRACE_SPUR_SUPP_ENABLED,
            "Trace", traceNumber - 1, RS_REPCAP_BUF_SIZE, repCap));
    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, repCap, RSSPECAN_ATTR_PHASE_DISP_TRACE_SPUR_SUPP_ENABLED, spurRemoval),
            3, "Spur Removal");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function sets the cross-correlation gain indicator.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR state/Switches on or off the cross-correlation gain indicator (grey
/// HIPAR state/area in the Noise Spectrum result display).
ViStatus _VI_FUNC rsspecan_TracePhaseCrossCorrelation(ViSession instrSession,
                                                      ViBoolean state)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_PHASE_DISP_TRACE_XGAIN_ENABLED, state),
            2, "State");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function stops the measurement in the current half decade
/// HIFN and continues measuring in the subsequent half decade.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
ViStatus _VI_FUNC rsspecan_PhaseFinishHalfDecade(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_PHASE_FINISH_HALF_DECADE, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/*****************************************************************************
 * Function: Reference Phase Measurement
 * Purpose:  This function configures and initiates a reference measurement.
 *****************************************************************************/
ViStatus _VI_FUNC rsspecan_ReferencePhaseMeasurement(ViSession instrSession,
                                                     ViUInt32  timeout)
{
    ViStatus error = VI_SUCCESS;
    ViUInt32 old_timeout = 0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViUInt32Range(instrSession, timeout, 0, 4294967295UL), 2, "Timeout");

    checkErr(rsspecan_GetTimeout(instrSession, &old_timeout));
    checkErr(rsspecan_SetTimeout(instrSession, timeout));
    checkErr(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_PHASE_CONF_REFM, NULL));

Error:
    if (old_timeout > 0)
        rsspecan_SetTimeout(instrSession, old_timeout);

    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/*****************************************************************************
 * Function: Get Phase Estimated Measurement Time
 * Purpose:  This function returns the Phase Sweep Estimated Measurement Time.
 *****************************************************************************/
ViStatus _VI_FUNC rsspecan_GetPhaseEstimatedMeasurementTime(ViSession instrSession,
                                                            ViReal64* estimatedMeasurementTime)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_SWEEP_TIME, estimatedMeasurementTime),
            2, "Estimated Measurement Time");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function returns the measurement characteristics for half
/// HIFN decade.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR range/Selects the range (half decade).
/// HIPAR RBW/Returns resolution bandwidth applied to the corresponding half
/// HIPAR RBW/decade during the measurement.
/// HIPAR startFrequencyOffset/Returns the start frequency offset of a half decade.
/// HIPAR stopFrequencyOffset/Returns the stop frequency offset of a half decade.
/// HIPAR XCorr/Returns the number of cross-correlation operations done in a
/// HIPAR XCorr/half decade.
ViStatus _VI_FUNC rsspecan_GetHalfDecadeMeasCharacteristics(ViSession instrSession,
                                                            ViInt32   range,
                                                            ViReal64* RBW,
                                                            ViReal64* startFrequencyOffset,
                                                            ViReal64* stopFrequencyOffset,
                                                            ViInt32*  XCorr)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_PHASE_LIST_RANG_BAND,
            "PnoRange", range - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Range");

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_PHASE_LIST_RANG_BAND, RBW),
            3, "RBW");

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_PHASE_RANGE_START_FREQUENCY_OFFSET, startFrequencyOffset),
            4, "Start Frequency Offset");

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_PHASE_RANGE_STOP_FREQUENCY_OFFSET, stopFrequencyOffset),
            5, "Stop Frequency Offset");

    viCheckParm(rsspecan_GetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_PHASE_RANGE_CROSS_CORRELATION_OPERATIONS, XCorr),
            6, "XCorr");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function queries the measurement bandwidth of the frequency
/// HIFN  stability measurements (Allan variance and deviation).
/// HIFN
/// HIFN     Attribute:
/// HIFN     RSSPECAN_ATTR_PHASE_MEASURED_ALLAN_VARIANCE_AND_DEVIATION
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     [SENSe:]BWIDth:MEASurement?
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR noofPoints/This control returns the measurement bandwidth of the frequency
/// HIPAR noofPoints/stability measurements (Allan variance and deviation).
ViStatus _VI_FUNC rsspecan_GetAllanVarianceAndDeviation(ViSession instrSession,
                                                        ViReal64* bandwidth)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_PHASE_MEASURED_ALLAN_VARIANCE_AND_DEVIATION, bandwidth),
            2, "Bandwidth");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/*****************************************************************************
 * Function: Read Phase Trace Data
 * Purpose:  This function reads out trace data from the instrument.
*****************************************************************************/
ViStatus _VI_FUNC rsspecan_ReadPhaseTraceData(ViSession instrSession,
                                              ViInt32   window,
                                              ViInt32   sourceTrace,
                                              ViBoolean performSweep,
                                              ViUInt32  timeout,
                                              ViPInt32  noofPoints,
                                              ViReal64  traceDataX[],
                                              ViReal64  traceDataY[])
{
    ViStatus  error = VI_SUCCESS;
    ViInt32   dataSize = 0;
    ViInt32   pointsToRead = 0;
    ViChar    trace[RS_MAX_SHORT_MESSAGE_BUF_SIZE];
    ViAReal64 data = NULL;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, sourceTrace, 0, 5),
            2, "Source Trace");

    snprintf(trace, RS_MAX_SHORT_MESSAGE_BUF_SIZE, "TRACE%ld", sourceTrace + 1);

    if (performSweep)
    {
        checkErr(rsspecan_Initiate(instrSession, 0, timeout));
    }

    if (*noofPoints == 0)
        *noofPoints = RSSPECAN_DEFAULT_DATA_BUFFER_SIZE;
    pointsToRead = *noofPoints * 2;
    viCheckAlloc(data = (ViAReal64)malloc(pointsToRead * sizeof(ViReal64)));
    checkErr(rsspecan_dataReadTrace(instrSession, window, trace, pointsToRead, data, &dataSize));
    checkErr(
        RsCore_DecimateViReal64ArrayToUserBuffers(data, dataSize, *noofPoints, noofPoints, traceDataX, traceDataY, NULL,
            NULL, NULL, NULL));

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    if (data) free(data);
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function queries the data of the cross-correlation gain
/// HIFN indicator (grey area).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control sets the window number.
/// HIPAR noOfPoints/Returns the number of trace data points.
/// HIPAR traceDataX/Trace data array.
/// HIPAR traceDataY/Trace data array.
ViStatus _VI_FUNC rsspecan_ReadPhaseCrossCorrelationGainData(ViSession instrSession,
                                                             ViInt32   window,
                                                             ViInt32*  noOfPoints,
                                                             ViReal64  traceDataX[],
                                                             ViReal64  traceDataY[])
{
    ViStatus  error = VI_SUCCESS;
    ViInt32   pointsToRead = 0;
    ViInt32   dataSize = 0;
    ViAReal64 data = NULL;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentModel(instrSession, "FSWP"));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, window, 1, 16),
            2, "Window");

    if (*noOfPoints == 0)
        *noOfPoints = RSSPECAN_DEFAULT_DATA_BUFFER_SIZE;

    pointsToRead = (*noOfPoints) * 2;
    viCheckAlloc(data = (ViAReal64)malloc(pointsToRead * sizeof(ViReal64)));
    checkErr(rsspecan_dataReadTrace(instrSession, window, "XGIN", pointsToRead, data, &dataSize));
    checkErr(
        RsCore_DecimateViReal64ArrayToUserBuffers(data, dataSize, *noOfPoints, noOfPoints, traceDataX, traceDataY, NULL,
            NULL, NULL, NULL));

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    if (data) free(data);
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function queries the trace data with linear interpolation.
/// HIFN
/// HIFN     Returned values: pairs of offset frequency in Hz and level in dBc/Hz
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     TRACe<n>[:DATA]:LINear?
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR window/This control sets the window number.
/// HIPAR sourceTrace/This control sets the source trace data to be read from instrument.
/// HIPAR noofPoints/Defines the number of linearily spaced measurement points that
/// HIPAR start/Defines the start offset of the frequency range whose trace data
/// HIPAR stop/Defines the stop offset of the frequency range whose trace data
/// HIPAR offsetFrequency/Trace data array.
/// HIPAR level/Trace data array.
ViStatus _VI_FUNC rsspecan_ReadPhaseTraceDataLinearInterpolation(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   sourceTrace,
                            ViInt32   noOfPoints,
                            ViReal64  start,
                            ViReal64  stop,
                            ViReal64  offsetFrequency[],
                            ViReal64  level[])
{
    ViStatus  error = VI_SUCCESS;
    ViString  sourceTraceArr[] = {"TRACE1", "TRACE2", "TRACE3", "TRACE4", "TRACE5", "TRACE6", "XGIN",NULL};
    ViChar    trace[RS_MAX_MESSAGE_BUF_SIZE];
    ViReal64* data = NULL;
    ViInt32   dataSize = 0;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentModel(instrSession, "FSWP"));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, window, 1, 16),
            2, "Window");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, sourceTrace, RSSPECAN_VAL_TRACE_RESULT_TRACE1, RSSPECAN_VAL_RESULT_XGINDICATOR),
            3, "Source Trace");

    snprintf(trace, RS_MAX_MESSAGE_BUF_SIZE, "%s,%ld,%.12lG,%.12lG", sourceTraceArr[sourceTrace], noOfPoints, start,
             stop);
    viCheckAlloc(data = (ViReal64*)malloc(noOfPoints * sizeof(ViReal64) * 2));
    checkErr(rsspecan_dataReadTraceLinear(instrSession, window, trace, noOfPoints * 2, data, &dataSize));
    checkErr(
        RsCore_DecimateViReal64ArrayToUserBuffers(data, dataSize, noOfPoints, NULL, offsetFrequency, level, NULL, NULL,
            NULL, NULL));

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    if (data) free(data);
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function queries the number of measurement points of a specific
/// HIFN  trace.
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     TRACe<n>:POINts?
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR window/This control sets the window number.
/// HIPAR sourceTrace/This control sets the source trace data to be read from instrument.
/// HIPAR noofPoints/Returns the number of trace data points.
ViStatus _VI_FUNC rsspecan_ReadPhaseTraceNumberOfMeasurementPoints(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   sourceTrace,
                            ViInt32*  noofPoints)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";
    ViString sourceTraceArr[] = {"TRACE1", "TRACE2", "TRACE3", "TRACE4", "TRACE5", "TRACE6", "XGIN",NULL};

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, sourceTrace, RSSPECAN_VAL_TRACE_RESULT_TRACE1, RSSPECAN_VAL_RESULT_XGINDICATOR),
            3, "Source Trace");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "TRAC%ld:POIN? %s", window, sourceTraceArr[sourceTrace]);
    checkErr(RsCore_QueryViInt32(instrSession, cmd, noofPoints));

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/*****************************************************************************
 * Function: Fetch Phase Results
 * Purpose:  This function returns the phase measurement results.
 *****************************************************************************/
ViStatus _VI_FUNC rsspecan_FetchPhaseResidualResults(ViSession instrSession,
                                                     ViInt32   modifier,
                                                     ViReal64* value)
{
    ViStatus error = VI_SUCCESS;
    ViInt32  attribute = 0;

    checkErr(RsCore_LockSession(instrSession));

    switch (modifier)
    {
    case RSSPECAN_VAL_PHASE_RFM:
        attribute = RSSPECAN_ATTR_PHASE_RESIDUAL_FM;
        break;
    case RSSPECAN_VAL_PHASE_RPM:
        attribute = RSSPECAN_ATTR_PHASE_RESIDUAL_PM;
        break;
    case RSSPECAN_VAL_PHASE_RMS:
        attribute = RSSPECAN_ATTR_PHASE_RESIDUAL_RMS_JITTER;
        break;

    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, modifier), 2, "Modifier");
    }

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, "", attribute, value),
            3, "Result Value");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function returns the phase measurement results.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR userRange/Selects the user range.
/// HIPAR modifier/Sets the modifier of returned value.
/// HIPAR value/Returns the value.
ViStatus _VI_FUNC rsspecan_FetchPhaseResidualUserResults(ViSession instrSession,
                                                         ViInt32   userRange,
                                                         ViInt32   modifier,
                                                         ViReal64* value)
{
    ViStatus error = VI_SUCCESS;
    ViInt32  attribute = 0;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, userRange, 1, 3),
            2, "User Range");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "U%ld", userRange);

    switch (modifier)
    {
    case RSSPECAN_VAL_PHASE_RFM:
        attribute = RSSPECAN_ATTR_PHASE_USER_RESIDUAL_FM;
        break;
    case RSSPECAN_VAL_PHASE_RPM:
        attribute = RSSPECAN_ATTR_PHASE_USER_RESIDUAL_PM;
        break;
    case RSSPECAN_VAL_PHASE_RMS:
        attribute = RSSPECAN_ATTR_PHASE_USER_RESIDUAL_RMS_JITTER;
        break;

    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, modifier), 2, "Modifier");
    }

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, repCap, attribute, value),
            4, "Result Value");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function queries the location and level of all spurs that have
/// HIFN  been detected.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR noofPoints/Returns the number of trace data points.
/// HIPAR frequencyValues/Returns frequency values for each each spur that has been detected.
/// HIPAR levelValues/Returns level values for each each spur that has been detected.
ViStatus _VI_FUNC rsspecan_FetchPhaseSpurs(ViSession instrSession,
                                           ViInt32*  noofPoints,
                                           ViReal64  frequencyValues[],
                                           ViReal64  levelValues[])
{
    ViStatus error = VI_SUCCESS;
    ViUInt32 retCnt = 0;
    ViChar*  pbuffer = NULL;
    ViChar*  pstring_value;
    ViInt32  cnt = 0;

    checkErr(RsCore_LockSession(instrSession));

    if (RsCore_IsInstrumentModel(instrSession, "FSW"))
        checkErr(RsCore_CheckInstrumentOptions(instrSession, "K40"));

    if (*noofPoints != 0)
    {
        checkErr(RsCore_QueryViStringUnknownLength(instrSession, "FETC:PNO:SPUR?", &pbuffer));

        pstring_value = strtok(pbuffer, ",");

        if (strncmp(pstring_value, "\"No Spurs Found\"", 16) != 0)
        {
            while (pstring_value != NULL && cnt < (*noofPoints))
            {
                frequencyValues[cnt] = RsCore_Convert_String_To_ViReal64(pstring_value);
                pstring_value = strtok(NULL, ",");
                levelValues[cnt++] = RsCore_Convert_String_To_ViReal64(pstring_value);
                pstring_value = strtok(NULL, ",");
            }
        }
    }
    else
    {
        checkErr(RsCore_QueryViStringUnknownLength(instrSession, "FETC:PNO:SPUR?", &pbuffer));

        pstring_value = strtok(pbuffer, ",");

        if (strncmp(pstring_value, "\"No Spurs Found\"", 16) != 0)
        {
            while (pstring_value != NULL && cnt < ((ViInt32)retCnt / 2))
            {
                frequencyValues[cnt] = RsCore_Convert_String_To_ViReal64(pstring_value);
                pstring_value = strtok(NULL, ",");
                levelValues[cnt++] = RsCore_Convert_String_To_ViReal64(pstring_value);
                pstring_value = strtok(NULL, ",");
            }
        }
    }

    *noofPoints = cnt;

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    if (pbuffer) free(pbuffer);
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function queries the location and level of all spurs that have
/// HIFN  been detected.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/Sets the window number.
/// HIPAR trace/Selects the trace.
/// HIPAR noOfPoints/Returns the number of trace data points.
/// HIPAR frequencyValues/Returns frequency values for each each spur that has been detected.
/// HIPAR levelValues/Returns level values for each each spur that has been detected.
ViStatus _VI_FUNC rsspecan_FetchPhaseSpuriousList(ViSession instrSession,
                                                  ViInt32   window,
                                                  ViInt32   trace,
                                                  ViInt32*  noOfPoints,
                                                  ViReal64  frequencyValues[],
                                                  ViReal64  levelValues[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViChar*  pbuffer = NULL;
    ViChar*  pstring_value;
    ViInt32  cnt = 0;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentModel(instrSession, "FSWP"));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, window, 1, 16),
            2, "Window");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, trace, 1, 6),
            3, "Trace");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "FETC%ld:PNO%ld:SPUR?", window, trace);
    checkErr(RsCore_QueryViStringUnknownLength(instrSession, cmd, &pbuffer));
    pstring_value = strtok(pbuffer, ",");

    if (strncmp(pstring_value, "\"No Spurs Found\"", 16) != 0)
    {
        while (pstring_value != NULL)
        {
            if ((*noOfPoints != 0) && (cnt < (*noOfPoints)))
            {
                frequencyValues[cnt] = RsCore_Convert_String_To_ViReal64(pstring_value);
                pstring_value = strtok(NULL, ",");
                levelValues[cnt++] = RsCore_Convert_String_To_ViReal64(pstring_value);
                pstring_value = strtok(NULL, ",");
            }
            else
            {
                (void)strtok(NULL, ",");
                pstring_value = strtok(NULL, ",");
                cnt++;
            }
        }
    }

    *noOfPoints = cnt;

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    if (pbuffer) free(pbuffer);
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the number of spurious included in the spurious list.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/Sets the window number.
/// HIPAR trace/Selects the trace.
/// HIPAR listCount/Returns the number of spurious included in the spurious list.
ViStatus _VI_FUNC rsspecan_FetchPhaseSpuriousListCount(ViSession instrSession,
                                                       ViInt32   window,
                                                       ViInt32   trace,
                                                       ViInt32*  listCount)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_t[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_PHASE_SPURIOUS_LIST_COUNT,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap),
            2, "Window");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_PHASE_SPURIOUS_LIST_COUNT,
            "Trace", trace - 1, RS_REPCAP_BUF_SIZE, repCap_t),
            3, "Trace");

    strcat(repCap, ",");
    strcat(repCap, repCap_t);

    viCheckParm(rsspecan_GetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_PHASE_SPURIOUS_LIST_COUNT, listCount),
            4, "List Count");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function queries  the jitter of the measured spurs.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/Sets the window number.
/// HIPAR trace/Selects the trace.
/// HIPAR noOfPoints/Returns the number of trace data points.
/// HIPAR jitterValues/Returns jitter values of the spur in s.
ViStatus _VI_FUNC rsspecan_FetchPhaseSpuriousJitter(ViSession instrSession,
                                                    ViInt32   window,
                                                    ViInt32   trace,
                                                    ViInt32*  noOfPoints,
                                                    ViReal64  jitterValues[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViChar*  pbuffer = NULL;
    ViChar*  pstring_value;
    ViInt32  cnt = 0;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentModel(instrSession, "FSWP"));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, window, 1, 16),
            2, "Window");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, trace, 1, 6),
            3, "Trace");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "FETC%ld:PNO%ld:SPUR:JITT?", window, trace);
    checkErr(RsCore_QueryViStringUnknownLength(instrSession, cmd, &pbuffer));

    pstring_value = strtok(pbuffer, ",");

    if (strncmp(pstring_value, "\"No Spurs Found\"", 16) != 0)
    {
        while (pstring_value != NULL)
        {
            if ((*noOfPoints != 0) && (cnt < (*noOfPoints)))
                jitterValues[cnt] = RsCore_Convert_String_To_ViReal64(pstring_value);

            pstring_value = strtok(NULL, ",");
            cnt++;
        }
    }

    *noOfPoints = cnt;

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    if (pbuffer) free(pbuffer);
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function queries the carrier frequency and the level of the DUT
/// HIFN  that has been actually measured.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR frequency/Returns the carrier frequency that has been actually measured.
/// HIPAR level/Returns the level of the DUT that has been actually measured.
ViStatus _VI_FUNC rsspecan_FetchPhaseFrequencyLevel(ViSession instrSession,
                                                    ViReal64* frequency,
                                                    ViReal64* level)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_PHASE_MEASURED_FREQUENCY, frequency),
            2, "Frequency");

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_PHASE_MEASURED_LEVEL, level),
            3, "Level");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function queries the DUT gain.
/// HIFN
/// HIFN     Note:
/// HIFN
/// HIFN     (1) The DUT gain is the ratio of the power measured at the DUT output
/// HIFN  and the power applied to the DUT input (signal power at the signal
/// HIFN  source output).
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSSPECAN_ATTR_PHASE_RESIDUAL_DUT_GAIN
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     FETCh<n>[:RANGe<j>]:PNOise<t>:DUT:GAIN?
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR dutGain/This control returns the DUT gain.
ViStatus _VI_FUNC rsspecan_FetchPhaseResidualDUTGain(ViSession instrSession,
                                                     ViReal64* dutGain)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_PHASE_RESIDUAL_DUT_GAIN, dutGain),
            2, "DUT Gain");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function queries the the results of the phase noise measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR type/This control selects result type.
/// HIPAR arraySize/Sets the size of Results array.
/// HIPAR results/Returns frequency values for each each spur that has been
/// HIPAR results/detected.
/// HIPAR returnedValues/Returns the number of values readed from instrument.
ViStatus _VI_FUNC rsspecan_FetchSweepResultList(ViSession instrSession,
                                                ViInt32   type,
                                                ViInt32   arraySize,
                                                ViReal64  results[],
                                                ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViChar*  p2string;

    checkErr(RsCore_LockSession(instrSession));

    if (RsCore_IsInstrumentModel(instrSession, "FSW"))
        checkErr(RsCore_CheckInstrumentOptions(instrSession, "K40"));

    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &RsSpecAn_rngPhaseSweepListResultType, type, &p2string),
                2, "Type");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "FETCh:PNOise:SWE:%s?", p2string);
    checkErr(RsCore_QueryFloatArrayToUserBuffer(instrSession, cmd, arraySize, results, returnedValues));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/*****************************************************************************
 * Function: Get Phase Spot Noise Y Position
 * Purpose:  This function returns the Phase Spot Noise Y Position.
 *****************************************************************************/
ViStatus _VI_FUNC rsspecan_GetPhaseSpotNoiseYPosition(ViSession instrSession,
                                                      ViInt32   spotNoiseMarkerNumber,
                                                      ViReal64* spotNoiseMarkerResult)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_PHASE_CALC_SNO_Y,
            "Snoise", spotNoiseMarkerNumber - 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Spot Noise Marker Number");

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_PHASE_CALC_SNO_Y, spotNoiseMarkerResult),
            3, "Spot Noise Marker Result");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function returns the horizontal and vertical poistion of the 10x offset frequency spot noise markers.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR arraySize/This parameter determines the size of array result values.
/// HIPAR verticalPosition/Returns the list of level values, one for each 10x spot noise marker.
/// HIPAR horizontalPosition/Returns the list of offset frequencies, one for each 10x spot noise marker.
/// HIPAR returnedValues/Returns the number of returned values.
ViStatus _VI_FUNC rsspecan_GetPhaseDecadeSpotNoiseResult(ViSession instrSession,
                                                         ViInt32   arraySize,
                                                         ViReal64  verticalPosition[],
                                                         ViReal64  horizontalPosition[],
                                                         ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    if (RsCore_IsInstrumentModel(instrSession, "FSW"))
        checkErr(RsCore_CheckInstrumentOptions(instrSession, "K40"));

    checkErr(
        RsCore_QueryFloatArrayToUserBuffer(instrSession, "CALC:SNO:DEC:X?", arraySize, verticalPosition, returnedValues
        ));
    checkErr(rsspecan_CheckStatus(instrSession));

    checkErr(
        RsCore_QueryFloatArrayToUserBuffer(instrSession, "CALC:SNO:DEC:Y?", arraySize, horizontalPosition,
            returnedValues));
    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}
