/*****************************************************************************
 *  Rohde&Schwarz Pulse measurement instrument driver - option K6
 *
 *  VXIpnp Style Instrument Driver
 *  Original Release: June 2006
 *  By: Martin Koutny (instrument driver core)
 *      Helena Sucha (instrument specific implementation)
 *      Jiri Kominek (DrBase repository & tools)
 *
 *  Should you have any technical questions please contact the hotline of
 *  Rohde & Schwarz Vertriebs-GmbH Rohde & Schwarz Support Center
 *
 *  e-mail: CustomerSupport@rohde-schwarz.com
 *
 *****************************************************************************/

/*****************************************************************************
 *  K6 - Pulse Measurement
 *****************************************************************************/
#include <math.h>
#include "rsspecan.h"
#include "rsspecan_internal.h"
#include <ctype.h>

/// HIFN  This function selects the Pulse measurement mode.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsspecan_PulseMeasurementMode(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_PULSE_MODE, ""));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function configures pulse period, amplitude droop and pulse modulation.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR pulsePeriod/This control defines how a pulse is detected.
/// HIPAR pulseHasDroop/If ON, a pulse can be modeled as having amplitude droop,
/// HIPAR pulseHasDroop/otherwise the pulse top is assumed to be flat.
/// HIPAR pulseModulation/This control sets the type of pulse modulation which is expected.
ViStatus _VI_FUNC rsspecan_ConfigurePulseSignalModel(ViSession instrSession,
                                                     ViInt32   pulsePeriod,
                                                     ViBoolean pulseHasDroop,
                                                     ViInt32   pulseModulation)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_PULSE_SIGNAL_PERIOD, pulsePeriod),
            2, "Pulse Period");

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_PULSE_SIGNAL_DROOP_ENABLED, pulseHasDroop),
            3, "Pulse Has Droop");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_PULSE_SIGNAL_MODULATION, pulseModulation),
            4, "Pulse Modulation");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function sets automatic pulse signal timing.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsspecan_ConfigurePulseSignalTimingAuto(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_PULSE_SIGNAL_TIMING_AUTO_MODE, VI_TRUE));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function configures manual pulse signal timing.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR minPulseWidth/Defines a minimum pulse width
/// HIPAR maxPulseWidth/Defines a maximum pulse width
/// HIPAR minPulseOffTime/The minimum time the pulse is "off", i.e. the time between successive pulses.
ViStatus _VI_FUNC rsspecan_ConfigurePulseSignalTimingManual(ViSession instrSession,
                                                            ViReal64  minPulseWidth,
                                                            ViReal64  maxPulseWidth,
                                                            ViReal64  minPulseOffTime)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_PULSE_SIGNAL_TIMING_AUTO_MODE, VI_FALSE));

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_PULSE_SIGNAL_TIMING_WIDTH_MINIMUM, minPulseWidth),
            2, "Min Pulse Width");

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_PULSE_SIGNAL_TIMING_WIDTH_MAXIMUM, maxPulseWidth),
            3, "Max Pulse Width");

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_PULSE_SIGNAL_TIMING_OFF_TIME, minPulseOffTime),
            4, "Min Pulse Off Time");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function configures pulse signal frequency offset.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR autoMode/If enabled, the frequency offset is estimated automatically for each individual pulse.
/// HIPAR frequencyOffsetValue/Defines a known frequency offset to be corrected in the pulse acquisition data.
ViStatus _VI_FUNC rsspecan_ConfigurePulseSignalFrequencyOffset(
                            ViSession instrSession,
                            ViBoolean autoMode,
                            ViReal64  value)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_PULSE_SIGNAL_OFFSET_AUTO_MODE, autoMode),
            2, "Auto Mode");
    if (!autoMode)
    {
        viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_PULSE_SIGNAL_FREQUENCY_OFFSET, value),
                3, "Value");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function configures pulse signal chirp rate.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR autoMode/If enabled, the chirp rate is estimated automatically for each individual pulse.
/// HIPAR chirpRate/Defines a known frequency chirp rate (in Hz/us) to be used to generate an ideal
/// HIPAR chirpRate/pulse waveform for computing frequency and phase error parameters.
ViStatus _VI_FUNC rsspecan_ConfigurePulseSignalChirpRate(ViSession instrSession,
                                                         ViBoolean autoMode,
                                                         ViReal64  chirpRate)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_PULSE_SIGNAL_CHIRP_RATE_AUTO_MODE, autoMode),
            2, "Auto Mode");

    if (!autoMode)
    {
        viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_PULSE_SIGNAL_CHIRP_RATE, chirpRate),
                3, "Chirp Rate");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function selects the I/Q data file which contains the reference waveform.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR type/Defines how the reference waveform is defined.
ViStatus _VI_FUNC rsspecan_ConfigurePulseReferenceType(ViSession instrSession,
                                                       ViInt32   type)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_PULSE_REFERENCE_IQ_TYPE, type),
            2, "Type");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function selects the I/Q data file which contains the reference waveform.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR inputFile/Path to I/Q data file.
ViStatus _VI_FUNC rsspecan_ConfigurePulseUserDefinedReferenceIQFile(
                            ViSession instrSession,
                            ViInt32   type,
                            ViString  inputFile)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_PULSE_REFERENCE_IQ_TYPE, type),
            2, "Type");

    viCheckParm(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_PULSE_REFERENCE_IQ_INPUT_FILE, inputFile),
            3, "Input File");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function configures the range for reference pulse imported from an
/// HIFN I/Q waveform file with measured data.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR rangeAuto/Sets, if the data from the entire file is used as the time sidelobe range (Auto)
/// HIPAR rangeAuto/or the length and offset is defined (Manual).
/// HIPAR offset/Defines the starting time of the reference pulse as an offset
/// HIPAR offset/from the beginning of the data file.
/// HIPAR length/Defines the length of the reference pulse in the data file in seconds.
ViStatus _VI_FUNC rsspecan_ConfigurePulseUserDefinedReferenceIQFileRange(
                            ViSession instrSession,
                            ViBoolean rangeAuto,
                            ViReal64  offset,
                            ViReal64  length)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_PULSE_REFERENCE_IQ_RANGE_AUTO, rangeAuto),
            2, "Range Auto");

    if (rangeAuto == VI_FALSE)
    {
        viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_PULSE_REFERENCE_IQ_RANGE_OFFSET, offset),
                3, "Offset");

        viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_PULSE_REFERENCE_IQ_RANGE_LENGTH, length),
                4, "Length");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function defines the FFT window function to be applied to the reference I/Q data.
/// HIFN I/Q waveform file with measured data.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/Defines the FFT window function to be applied to the reference I/Q data.
ViStatus _VI_FUNC rsspecan_ConfigurePulseUserDefinedReferenceIQFileWindow(
                            ViSession instrSession,
                            ViInt32   window)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_PULSE_REFERENCE_IQ_WINDOW, window),
            2, "Window");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function configures signal with a polynomial phase of a specified order.
/// HIFN I/Q waveform file with measured data.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/Defines the FFT window function to be applied to the reference I/Q data.
/// HIPAR pulseWidth/Defines the width of the reference pulse.
/// HIPAR arraySize/Defines the size of array.
/// HIPAR coefficients/Array of coefficients for a polynome of order n, n+1
ViStatus _VI_FUNC rsspecan_ConfigurePulsePolynominaPhaseReferenceWaveform(
                            ViSession instrSession,
                            ViInt32   window,
                            ViReal64  pulseWidth,
                            ViInt32   arraySize,
                            ViInt32   coefficients[])
{
    ViStatus error = VI_SUCCESS;
    ViInt32  i = 0;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_PULSE_REFERENCE_IQ_POLYNOMIAL_PHASE_WINDOW, window),
            2, "Window");

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_PULSE_REFERENCE_IQ_POLYNOMIAL_PHASE_WIDTH, pulseWidth),
            3, "Pulse Width");

    viCheckParm(RsCore_InvalidViInt32RangeMin(instrSession, arraySize, 1),
                4, "Array Size");

    for (i = 1; i < arraySize; i++)
    {
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "RIQ:PFM:COEF%ld %ld", i, coefficients[i - 1]);
        checkErr(RsCore_Write(instrSession, cmd));
    }

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function configures Barker waveform with a specified primary code.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR pulseWidth/Defines the width of the reference pulse.
/// HIPAR primaryCode/Selects the reference IQ barker code length for time sidelobe measurements.
ViStatus _VI_FUNC rsspecan_ConfigurePulseBarkerReferenceWaveform(
                            ViSession instrSession,
                            ViReal64  pulseWidth,
                            ViInt32   primaryCode)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_PULSE_REFERENCE_IQ_BARKER_WIDTH, pulseWidth),
            2, "Pulse Width");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_PULSE_REFERENCE_IQ_BARKER_PRIMARY_CODE, primaryCode),
            3, "Primary Code");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function configures Embedded Barker waveform.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR pulseWidth/Defines the width of the reference pulse.
/// HIPAR primaryCode/Selects the reference IQ barker code length for time sidelobe measurements.
/// HIPAR secondaryCode/Code length of secondary Barker code used in an embeddedbarker code.
ViStatus _VI_FUNC rsspecan_ConfigurePulseEmbeddedBarkerReferenceWaveform(
                            ViSession instrSession,
                            ViReal64  pulseWidth,
                            ViInt32   primaryCode,
                            ViInt32   secondaryCode)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_PULSE_REFERENCE_IQ_EMBEDED_BARKER_WIDTH, pulseWidth),
            2, "Pulse Width");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_PULSE_REFERENCE_IQ_EMBEDED_BARKER_PRIMARY_CODE, primaryCode),
            3, "Primary Code");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_PULSE_REFERENCE_IQ_EMBEDED_BARKER_SECONDARY_CODE, secondaryCode),
            4, "Secondary Code");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function selects the signal source for measurements.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR source/This control selects the signal source for measurements.
ViStatus _VI_FUNC rsspecan_ConfigurePulseInputSignalSource(ViSession instrSession,
                                                           ViInt32   source)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_PULSE_INPUT_SIGNAL_SOURCE, source),
            2, "Source");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function selects the I/Q data file to be used as input for
/// HIFN  further measurements.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR fileName/String containing the path and name of the source file. The file
ViStatus _VI_FUNC rsspecan_PulseSelectIQDataFile(ViSession instrSession,
                                                 ViString  fileName)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_PULSE_SELECT_IQ_DATA_FILE, fileName),
            2, "File Name");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Enables or disables pulsed signal output.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR signalOutput/Enables or disables pulsed signal output.
ViStatus _VI_FUNC rsspecan_ConfigurePulseSignalOutput(ViSession instrSession,
                                                      ViBoolean signalOutput)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, NULL, RSSPECAN_ATTR_PULSE_SIGNAL_OUTPUT, signalOutput),
            2, "Signal Output");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Enables a pulsed signal with multiple pulses per burst.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR burstMode/Enables a pulsed signal with multiple pulses per burst.
ViStatus _VI_FUNC rsspecan_ConfigurePulseBurstMode(ViSession instrSession,
                                                   ViBoolean burstMode)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, NULL, RSSPECAN_ATTR_PULSE_BURST_MODE, burstMode),
            2, "Burst Mode");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Defines the burst table as a comma-separated list of burst parameters.
/// HIFN  Each set of 4 parameters defines one burst.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR arraySize/Sets the number of items in Pulse Width, Pulse Period, Number Of Reps,
/// HIPAR arraySize/Burst Length arrays.
/// HIPAR pulseWidth/Duration of an individual pulse in seconds.
/// HIPAR pulsePeriod/Interval between two consecutive pulses.
/// HIPAR numberOfReps/Number of pulse repetitions in one burst.
/// HIPAR burstLength/Total duration of a single burst.
ViStatus _VI_FUNC rsspecan_ConfigurePulseBurstData(ViSession instrSession,
                                                   ViInt32   arraySize,
                                                   ViReal64  pulseWidth[],
                                                   ViReal64  pulsePeriod[],
                                                   ViInt32   numberOfReps[],
                                                   ViReal64  burstLength[])
{
    ViStatus error = VI_SUCCESS;
    ViChar*  cmd = NULL;
    ViChar*  p2char;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32RangeMin(instrSession, arraySize, 1),
        2, "Array Size");

    viCheckAlloc(cmd = (ViChar*)malloc((size_t) (4 * arraySize * sizeof(ViReal64)) + 10));

    p2char = cmd;

    for (int i = 0; i < arraySize; i++)
    {
        p2char += sprintf(p2char, "%.12lG,%.12lG,%ld,%.12lG,", pulseWidth[i], pulsePeriod[i], numberOfReps[i], burstLength[i]);
    }

    *--p2char = '\0';  // Remove remaining comma

    checkErr(rsspecan_SetAttributeViString(instrSession, NULL, RSSPECAN_ATTR_HIDDEN_PULSE_BURST_DATA, cmd));

Error:
    if (cmd) free(cmd);
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Returns the number of bursts in the table data.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR burstCount/Returns the number of bursts in the table data.
ViStatus _VI_FUNC rsspecan_QueryPulseBurstCount(ViSession instrSession,
                                                ViInt32*  burstCount)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_GetAttributeViInt32(instrSession, NULL, RSSPECAN_ATTR_PULSE_BURST_COUNT, burstCount),
            2, "Burst Mode");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Deletes the contents of the burst mode table.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsspecan_DeletePulseBurstTable(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsspecan_SetAttributeViString(instrSession, NULL, RSSPECAN_ATTR_PULSE_BURST_TABLE_DELETE, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Saves the currently defined burst table settings to the specified file
/// HIFN  in .CSV format.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR filename/Filename with configuration.
ViStatus _VI_FUNC rsspecan_SavePulseBurstTable(ViSession instrSession,
                                               ViString  filename)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViString(instrSession, NULL, RSSPECAN_ATTR_PULSE_BURST_TABLE_SAVE, filename),
            2, "File Name");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Loads a burst table configuration from the specified .CSV file.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR filename/Filename with configuration.
ViStatus _VI_FUNC rsspecan_LoadPulseBurstTable(ViSession instrSession,
                                               ViString  filename)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViString(instrSession, NULL, RSSPECAN_ATTR_PULSE_BURST_TABLE_LOAD, filename),
            2, "File Name");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Configures the internal trigger.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR triggerPosition/Determines when the internal signal source is triggered.
/// HIPAR burstNumber/Specifies the burst which triggers the signal.
ViStatus _VI_FUNC rsspecan_ConfigurePulseBurstInternalTrigger(
                            ViSession instrSession,
                            ViInt32   triggerPosition,
                            ViInt32   burstNumber)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, NULL, RSSPECAN_ATTR_PULSE_BURST_INTERNAL_TRIGGER_POSITION, triggerPosition),
            2, "Trigger Position");

    if (triggerPosition == RSSPECAN_VAL_PULSE_BURST_TRIGGER_POSITION_SPECIFIC_BURST)
    {
        viCheckParm(rsspecan_SetAttributeViInt32(instrSession, NULL, RSSPECAN_ATTR_PULSE_BURST_INTERNAL_TRIGGER_BURST_NUMBER, burstNumber),
                3, "Burst Number");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function configures pulse segment data capturing state.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR state/This control activates/deactivates segmented datat capturing.
ViStatus _VI_FUNC rsspecan_ConfigurePulseSegmentDataCapturingEnabled(
                            ViSession instrSession,
                            ViBoolean state)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_PULSE_SEGMENTED_DATA_CAPTURING_ENABLED, state),
            2, "State");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function configures pulse segment data capturing.
/// HIFN You may use multiple lines for documentation.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR events/This control sets the number of trigger events for which data
/// HIPAR events/segments are to be captured.
/// HIPAR length/This control defines a time period (starting from the trigger
/// HIPAR length/offset) in which data is captured. If multiple events occur
/// HIPAR length/within one segment length, the segment is extended.
/// HIPAR offset/This control defines an offset to the trigger event at which
/// HIPAR offset/data capturing starts. For a negative offset, data capturing
/// HIPAR offset/starts before the actual trigger event.
ViStatus _VI_FUNC rsspecan_ConfigurePulseSegmentDataCapturing(
                            ViSession instrSession,
                            ViInt32   events,
                            ViReal64  length,
                            ViReal64  offset)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_PULSE_SEGMENTED_DATA_CAPTURING_EVENTS, events),
            2, "Events");

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_PULSE_SEGMENTED_DATA_CAPTURING_LENGTH, length),
            3, "Length");

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_PULSE_SEGMENTED_DATA_CAPTURING_OFFSET, offset),
            4, "Offset");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function configures the pulse measurement acquisition.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR filterType/This control defines the type of demodulation filter to be used.
/// HIPAR measurementBandwidth/Defines the measurement bandwidth in Hz.
/// HIPAR measurementTime/Defines how long data is captured for analysis.
ViStatus _VI_FUNC rsspecan_ConfigurePulseMeasurementAcquisition(
                            ViSession instrSession,
                            ViInt32   filterType,
                            ViReal64  measurementBandwidth,
                            ViReal64  measurementTime)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_PULSE_ACQUISITION_FILTER_TYPE, filterType),
            2, "Filter Type");

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_PULSE_ACQUISITION_BANDWIDTH, measurementBandwidth),
            3, "Measurement Bandwidth");

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_SWEEP_TIME, measurementTime),
            4, "Measurement Time");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Configures the digitizer.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR digitizerSelection/Determines which digitizer is used for the measurement.
/// HIPAR lowNoiseMode/Determines how the results are calculated, depending on the
/// HIPAR lowNoiseMode/measurement setup.
ViStatus _VI_FUNC rsspecan_ConfigurePulseMeasurementAcquisitionDigitizer(
                            ViSession instrSession,
                            ViInt32   digitizerSelection,
                            ViInt32   lowNoiseMode)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, NULL, RSSPECAN_ATTR_PULSE_ACQUISITION_DIGITIZER_SELECTION, digitizerSelection),
            2, "Digitizer Selection");

    if (digitizerSelection == RSSPECAN_VAL_DIGITIZER_SELECTION_LOW_NOISE)
    {
        viCheckParm(rsspecan_SetAttributeViInt32(instrSession, NULL, RSSPECAN_ATTR_PULSE_ACQUISITION_DIGITIZER_LOW_NOISE_MODE, lowNoiseMode),
                3, "Low Noise Mode");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function activates/deactivates additional filters applied
/// HIFN after demodulation to filter out unwanted signals, or correct
/// HIFN pre-emphasized input signals.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR FMVideoBandwidth/Sets additional filters applied after demodulation to filter
/// HIPAR FMVideoBandwidth/out unwanted signals, or correct pre-emphasized input signals.
ViStatus _VI_FUNC rsspecan_ConfigurePulseMeasurementFMVideoBandwidth(
                            ViSession instrSession,
                            ViInt32   FMVideoBandwidth)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_PULSE_FM_VIDEO_BANDWIDTH, FMVideoBandwidth),
            2, "FM Video Bandwidth");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function configures the pulse detection.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR referenceSource/This control defines the level to be used as a reference for the pulse detection threshold.
/// HIPAR threshold/Defines a threshold for pulse detection.
/// HIPAR hysteresis/Defines a hysteresis for pulse detection in dB in relation to the defined threshold.
ViStatus _VI_FUNC rsspecan_ConfigurePulseMeasurementDetection(
                            ViSession instrSession,
                            ViInt32   referenceSource,
                            ViReal64  threshold,
                            ViReal64  hysteresis)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_PULSE_DETECTION_REFERENCE_SOURCE, referenceSource),
            2, "Reference Source");

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_PULSE_DETECTION_THRESHOLD, threshold),
            3, "Threshold");

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_PULSE_DETECTION_HYSTERESIS, hysteresis),
            4, "Hysteresis");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Contains pulse detection settings.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR detection/Enables or disables the use of a detection range instead of the entire
/// HIPAR detection/capture buffer for analysis.
/// HIPAR detectionStart/Defines the beginning of the detection range as the time in seconds
/// HIPAR detectionStart/from the capture buffer start. This command is only available for
/// HIPAR detectionStart/[SENSe:]DETect:RANGe ON.
/// HIPAR detectionLength/Defines the length of the detection range as a time in seconds. This
/// HIPAR detectionLength/command is only available for [SENSe:]DETect:RANGe ON.
ViStatus _VI_FUNC rsspecan_ConfigurePulseMeasurementDetectionSettings(
                            ViSession instrSession,
                            ViBoolean detection,
                            ViReal64  detectionStart,
                            ViReal64  detectionLength)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_PULSE_DETECTION_ENABLED, detection),
            2, "Detection");

    if (detection == VI_TRUE)
    {
        viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_PULSE_DETECTION_START, detectionStart),
                3, "Detection Start");

        viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_PULSE_DETECTION_LENGTH, detectionLength),
                4, "Detection Length");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN This function configures the pulse detection limits.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR detectionLimit/If enabled, the number of pulses to be detected is restricted.
/// HIPAR maximumPulseCount/Defines the maximum number of pulses to be detected.
ViStatus _VI_FUNC rsspecan_ConfigurePulseMeasurementDetectionLimit(
                            ViSession instrSession,
                            ViBoolean detectionLimit,
                            ViInt32   maximumPulseCount)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_PULSE_DETECTION_LIMIT_ENABLED, detectionLimit),
            2, "Detection Limit");

    if (detectionLimit)
    {
        viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_PULSE_DETECTION_PULSE_COUNT, maximumPulseCount),
                3, "Maximum Pulse Count");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function configurest the marker M1 in Parameter Trend
/// HIFN displays linking to the pulse selection.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR linkState/Configurest the marker M1 in Parameter Trend displays linking to the pulse selection state.
ViStatus _VI_FUNC rsspecan_ConfigurePulseMarkerLinkTrendM1ToSelectedPulse(
                            ViSession instrSession,
                            ViBoolean linkState)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_PULSE_LINK_TREND_M1_TO_SELECTED_PULSE, linkState),
            2, "Link State");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function defines whether all markers within the selected result
/// HIFN  display are linked. If enabled, and you move one marker along the
/// HIFN  x-axis, all other markers in the display are moved to the same x-axis
/// HIFN  position.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR linkState/Defines whether all markers within the selected result display are
/// HIPAR linkState/linked. If enabled, and you move one marker along the x-axis, all other
/// HIPAR linkState/markers in the display are moved to the same x-axis position.
ViStatus _VI_FUNC rsspecan_ConfigurePulseMarkerLink(ViSession instrSession,
                                                    ViBoolean linkState)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, NULL, RSSPECAN_ATTR_PULSE_MARKER_LINK, linkState),
            4, "Link State");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function links delta marker m1 to any active normal marker m2. If
/// HIFN  you change the horizontal position of marker m2, delta marker m1
/// HIFN  changes its horizontal position to the same value.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR deltamarker/Sets the deltamarker parameter.
/// HIPAR linkState/Links delta marker m1 to any active normal marker m2. If you change
/// HIPAR linkState/the horizontal position of marker m2, delta marker m1 changes its
/// HIPAR linkState/horizontal position to the same value.
/// HIPAR marker/Sets the marker parameter.
ViStatus _VI_FUNC rsspecan_ConfigurePulseDeltamarkerLinkToMarker(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   deltamarker,
                            ViInt32   marker,
                            ViBoolean linkState)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, window, 0, 16),
            2, "Window");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, deltamarker, 1, 16),
            3, "Deltamarker");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, marker, 1, 16),
            4, "Marker");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "Win%ld,DM%ld,M%ld", window, deltamarker, marker);

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, repCap, RSSPECAN_ATTR_PULSE_DELTAMARKER_LINK_TO_MARKER, linkState),
            5, "Link State");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function links normal marker m1 to any active normal marker m2.
/// HIFN     If you change the horizontal position of marker m2, marker m1 changes
/// HIFN  its horizontal position to the same value.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR marker1/Sets the marker parameter.
/// HIPAR linkState/Links normal marker m1 to any active normal marker m2.
/// HIPAR marker2/Sets the marker parameter.
ViStatus _VI_FUNC rsspecan_ConfigurePulseMarkerLinkToMarker(ViSession instrSession,
                                                            ViInt32   window,
                                                            ViInt32   marker1,
                                                            ViInt32   marker2,
                                                            ViBoolean linkState)
{
    return rsspecan_LinkMarkerToAnotherMarker(instrSession, window, marker1, marker2, linkState);
}

/// HIFN  Turns labels for peaks found during a peak search on and off.
/// HIFN     The labels correspond to the marker number in the marker peak list.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR marker/Sets the marker parameter.
/// HIPAR labelsForPeaks/Turns labels for peaks found during a peak search on and off.
ViStatus _VI_FUNC rsspecan_ConfigurePulseMarkerLabelsForPeaks(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   marker,
                            ViBoolean labelsForPeaks)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  old_win = 0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, window, 0, 16),
            2, "Window");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, marker, 1, 16),
            3, "Marker");

    checkErr(rsspecan_GetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_ACTIVE_WINDOW, &old_win));
    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_ACTIVE_WINDOW, window),
            2, "Window");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "M%ld", marker);

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_MARKER_LABELS_FOR_PEAKS, labelsForPeaks),
            4, "Labels For Peaks");

Error:
    rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_ACTIVE_WINDOW, old_win);
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function configures normalization of the trace.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR normalization/Sets the normalization of trace.
ViStatus _VI_FUNC rsspecan_ConfigurePulseMesurementTraceNormalization(
                            ViSession instrSession,
                            ViInt32   normalization)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_PULSE_TRACE_NORMALIZATION, normalization),
            2, "Normalization");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Determines how the stability results are determined for the trace.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSSPECAN_ATTR_PULSE_TRACE_RESULTS
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     [SENSe:][WINDow<n>:]RESults<res>[:FUNCtion]
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR trace/Selects the trace.
/// HIPAR results/Determines how the stability results are determined for the trace.
ViStatus _VI_FUNC rsspecan_ConfigurePulseMeasurementTraceResults(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   trace,
                            ViInt32   results)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   windowRepcap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   traceRepcap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  windowMax = 0;
    ViInt32  traceMax = 0;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetRepCapIdItemsCount(instrSession, "Window", &windowMax));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, window, 0, windowMax - 1),
        2, "Window");

    checkErr(RsCore_GetRepCapIdItemsCount(instrSession, "Trace", &traceMax));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, trace, 1, traceMax),
            2, "Trace");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_PULSE_TRACE_RESULTS, "Window", window, RS_REPCAP_BUF_SIZE, windowRepcap),
        2, "Window");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_PULSE_TRACE_RESULTS, "Trace", trace - 1, RS_REPCAP_BUF_SIZE, traceRepcap),
        3, "Trace");

    (void)snprintf(repCap, RS_REPCAP_BUF_SIZE, "%s,%s", windowRepcap, traceRepcap);

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_PULSE_TRACE_RESULTS, results),
            4, "Results");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines which signal component (I/Q) is evaluated in which trace for
/// HIFN  the Pulse I and Q result display. This setting is not available for any
/// HIFN  other result displays. By default, the I component is displayed by
/// HIFN  trace 1, while the Q component is displayed by trace 4.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR trace/Selects the trace.
/// HIPAR evaluation/Defines which signal component (I/Q) is evaluated in which trace for
/// HIPAR evaluation/the Pulse I and Q result display. This setting is not available for any
/// HIPAR evaluation/other result displays. By default, the I component is displayed by
/// HIPAR evaluation/trace 1, while the Q component is displayed by trace 4.
ViStatus _VI_FUNC rsspecan_ConfigurePulseMesurementTraceEvaluation(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   trace,
                            ViInt32   evaluation)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_t[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, window, 0, 16),
            2, "Window");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, trace, 1, 6),
            2, "Trace");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_PULSE_TRACE_EVALUATION,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap),
            2, "Window");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_PULSE_TRACE_EVALUATION,
            "Trace", trace - 1, RS_REPCAP_BUF_SIZE, repCap_t),
            3, "Trace");

    strcat(repCap, ",");
    strcat(repCap, repCap_t);

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_PULSE_TRACE_EVALUATION, evaluation),
            4, "Evaluation");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Determines which trace evaluates which stability result for 'Pulse
/// HIFN  Stability(+)' and 'Pulse Stability Waterfall(+)' result displays.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR trace/Selects the trace.
/// HIPAR stabilityEvaluation/Determines which trace evaluates which stability result for 'Pulse
/// HIPAR stabilityEvaluation/Stability(+)' and 'Pulse Stability Waterfall(+)' result displays.
ViStatus _VI_FUNC rsspecan_ConfigurePulseMeasurementTraceStabilityEvaluation(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   trace,
                            ViInt32   stabilityEvaluation)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   windowRepcap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   traceRepcap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  windowMax = 0;
    ViInt32  traceMax = 0;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetRepCapIdItemsCount(instrSession, "Window", &windowMax));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, window, 0, windowMax - 1),
        2, "Window");

    checkErr(RsCore_GetRepCapIdItemsCount(instrSession, "Trace", &traceMax));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, trace, 1, traceMax),
            2, "Trace");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_PULSE_TRACE_STABILITY_EVALUATION, "Window", window, RS_REPCAP_BUF_SIZE, windowRepcap),
        2, "Window");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_PULSE_TRACE_STABILITY_EVALUATION, "Trace", trace - 1, RS_REPCAP_BUF_SIZE, traceRepcap),
        3, "Trace");

    (void)snprintf(repCap, RS_REPCAP_BUF_SIZE, "%s,%s", windowRepcap, traceRepcap);

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_PULSE_TRACE_STABILITY_EVALUATION, stabilityEvaluation),
            4, "Stability Evaluation");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines which signal component (I/Q) is evaluated in which trace for
/// HIFN  the Pulse I and Q result display. By default, the I component is
/// HIFN  displayed by trace 1, while the Q component is displayed by trace 4.
/// HIFN     This setting is not available for any other results displays.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR trace/Selects the trace.
/// HIPAR detector/Defines which signal component (I/Q) is evaluated in which trace for
/// HIPAR detector/the Pulse I and Q result display. By default, the I component is
/// HIPAR detector/displayed by trace 1, while the Q component is displayed by trace 4.
ViStatus _VI_FUNC rsspecan_ConfigurePulseMesurementTraceIQDetector(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   trace,
                            ViInt32   detector)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_t[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, window, 0, 16),
            2, "Window");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, trace, 1, 6),
            2, "Trace");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_PULSE_TRACE_IQ_DETECTOR,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap),
            2, "Window");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_PULSE_TRACE_IQ_DETECTOR,
            "Trace", trace - 1, RS_REPCAP_BUF_SIZE, repCap_t),
            3, "Trace");

    strcat(repCap, ",");
    strcat(repCap, repCap_t);

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_PULSE_TRACE_IQ_DETECTOR, detector),
            4, "Detector");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Sets the trace statistic type (e.g., Selected Pulse) for the given
/// HIFN  window.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR type/Sets the trace statistic type (e.g., Selected Pulse) for the given
/// HIPAR type/window.
ViStatus _VI_FUNC rsspecan_ConfigurePulseMesurementTraceStatisticType(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   type)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, window, 0, 16),
            2, "Window");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_PULSE_TRACE_STATISTIC_TYPE,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap),
            2, "Window");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_PULSE_TRACE_STATISTIC_TYPE, type),
            3, "Type");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function adds a window to the display.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init.
/// HIPAR stringSize/This control sets the allocated length of Returned Window Name string.
/// HIPAR referenceWindowName/String containing the number or name of the window the new window is inserted next to.
/// HIPAR direction/Direction the window is added relative to the window set with "Reference Window Name".
/// HIPAR windowType/Type of result display you want to add.
/// HIPAR returnedWindowName/Returns the name of the new window.
ViStatus _VI_FUNC rsspecan_PulseMeasurementAddWindow(ViSession instrSession,
                                                     ViInt32   stringSize,
                                                     ViString  referenceWindowName,
                                                     ViInt32   direction,
                                                     ViInt32   windowType,
                                                     ViChar    returnedWindowName[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViChar   response[RS_MAX_MESSAGE_BUF_SIZE];
    ViChar*  p2direction;
    ViChar*  p2type;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &rsspecan_rngLayoutDirection, direction, &p2direction),
                4, "Direction");
    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &rsspecan_rngPulseLayoutType, windowType, &p2type),
                5, "Window Type");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "LAY:ADD? '%s',%s,%s", referenceWindowName, p2direction, p2type);
    checkErr(RsCore_QueryViString(instrSession, cmd, response));

    if (returnedWindowName != NULL)
    {
        RsCore_TrimString(response, RS_VAL_TRIM_WHITESPACES_AND_ALL_QUOTES);
        RsCore_CopyToUserBufferAsciiData(returnedWindowName, stringSize, response);
    }

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function replaces a window with another one.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init.
/// HIPAR windowName/String containing the number or name of the window the new window is inserted next to.
/// HIPAR windowType/Type of result display you want to add.
ViStatus _VI_FUNC rsspecan_PulseMeasurementReplaceWindow(ViSession instrSession,
                                                         ViString  windowName,
                                                         ViInt32   windowType)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViChar*  p2type;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &rsspecan_rngPulseLayoutType, windowType, &p2type),
                3, "Window Type");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "LAY:REPL '%s',%s", windowName, p2type);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function configures the pulse measurement reference point.
/// HIFN Some specific pulse parameters, e.g. the phase or the frequency,
/// HIFN are determined at a specific time instant (measurement point)
/// HIFN within the pulse. You can configure this point based on a
/// HIFN reference and offset value.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR reference/Defines the reference point used for specifying the pulse time
/// HIPAR reference/instant.
/// HIPAR offset/The time instant used for in-pulse measurements e.g. phase or
/// HIPAR offset/for the pulse timestamp.
/// HIPAR averagingWindow/Size of the window around the measurement point used for
/// HIPAR averagingWindow/averaging.
ViStatus _VI_FUNC rsspecan_ConfigurePulseMeasPoint(ViSession instrSession,
                                                   ViInt32   reference,
                                                   ViReal64  offset,
                                                   ViReal64  averagingWindow)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_PULSE_REFERENCE_POINT_TYPE, reference),
            2, "Reference");

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_PULSE_REFERENCE_POINT_OFFSET, offset),
            3, "offset");

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_PULSE_REFERENCE_POINT_AVERAGING_WINDOW, averagingWindow),
            4, "Averaging Window");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function configures the reference pulse on which relative pulse results are based.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR mode/Defines the reference pulse on which relative pulse results are based.
/// HIPAR pulse/Selects a particular pulse to be used as a reference forrelative pulse parameters.
ViStatus _VI_FUNC rsspecan_ConfigurePulseReferenceForPulsePulseMeasurement(
                            ViSession instrSession,
                            ViInt32   mode,
                            ViInt32   pulse)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_PULSE_REFERENCE_PULSE_PULSE_MEASUREMENTS_MODE, mode),
            2, "Mode");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_PULSE_REFERENCE_PULSE_PULSE_MEASUREMENTS_VALUE, pulse),
            3, "Pulse");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function configures the pulse measurement reference level.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR reference/Defines the reference for the measurement range definition.
/// HIPAR length/The estimation range length as a percentage of the pulse top length.
/// HIPAR riseOffset/The offset in seconds from the pulse rising edge at which the estimation range begins.
/// HIPAR fallOffset/The offset in seconds from the pulse falling edge at which the estimation range ends.
ViStatus _VI_FUNC rsspecan_ConfigurePulseMeasurementRange(ViSession instrSession,
                                                          ViInt32   reference,
                                                          ViReal64  length,
                                                          ViReal64  riseOffset,
                                                          ViReal64  fallOffset)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_PULSE_REFERENCE_RANGE_TYPE, reference),
            2, "Reference");

    if (reference == RSSPECAN_VAL_PULSE_REFERENCE_RANGE_CENTER)
    {
        viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_PULSE_REFERENCE_RANGE_LENGTH, length),
                3, "Length");
    }
    else
    {
        viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_PULSE_REFERENCE_RANGE_OFFSET_LEFT, riseOffset),
                4, "Rise Offset");

        viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_PULSE_REFERENCE_RANGE_OFFSET_RIGHT, fallOffset),
                5, "Fall Offset");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function configures which part of the detected pulse is
/// HIFN evaluated for sidelobe results is configurable, similarly to the
/// HIFN result range for common pulse results.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR range/Defines which part of the detected pulse is evaluated for sidelobe results.
/// HIPAR alignment/Specifies the alignment with respect to the 'Peak Correlation' used to define the time sidelobe range.
/// HIPAR length/The length of the pulse time sidelobe range (in seconds).
ViStatus _VI_FUNC rsspecan_ConfigurePulseTimeSidelobeRange(ViSession instrSession,
                                                           ViInt32   range,
                                                           ViInt32   alignment,
                                                           ViReal64  length)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_PULSE_TIME_SIDELOBE_RANGE, range),
            2, "Range");
    if (range == RSSPECAN_VAL_PULSE_TS_RANGE_MANUAL)
    {
        viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_PULSE_TIME_SIDELOBE_ALIGNMENT, alignment),
                3, "Alignment");

        viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_PULSE_TIME_SIDELOBE_LENGTH, length),
                4, "Length");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function configures an excluded area around the center of
/// HIFN the time sidelobe range, assuming this
/// HIFN is the mainlobe, in which no results are calculated.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR autoMode/Defines an excluded area around the center of the time sidelobe
/// HIPAR autoMode/range, assuming this is the mainlobe, in which no results are calculated..
/// HIPAR length/Defines the length of the keep-out time.
ViStatus _VI_FUNC rsspecan_ConfigurePulseTimeSidelobeKeepOutTime(
                            ViSession instrSession,
                            ViBoolean autoMode,
                            ViReal64  length)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_PULSE_TIME_SIDELOBE_KEEP_OUT_TIME, autoMode),
            2, "Auto Mode");
    if (autoMode == VI_FALSE)
    {
        viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_PULSE_TIME_SIDELOBE_KEEP_OUT_TIME_LENGTH, length),
                3, "Length");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Configures the pulse stability measurement burst statistics.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR mode/Determines whether a the capture buffer contains one or more bursts.
/// HIPAR lengthPulses/Defines the number of pulses that are assigned to a single burst in
/// HIPAR lengthPulses/multiple mode, and thus the interval over which stability results are
/// HIPAR lengthPulses/calculated. At least 10 pulses are required for a stability
/// HIPAR lengthPulses/calculation.
/// HIPAR selectedPulse/Determines the number of the pulse to be analyzed for pulse-based
/// HIPAR selectedPulse/analysis.
ViStatus _VI_FUNC rsspecan_ConfigurePulseStabilityBurstStatistics(
                            ViSession instrSession,
                            ViInt32   mode,
                            ViInt32   lengthPulses,
                            ViInt32   selectedPulse)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, NULL, RSSPECAN_ATTR_PULSE_STABILITY_BURST_MODE, mode),
            2, "Mode");

    if (mode == RSSPECAN_VAL_PULSE_STABILITY_BURST_MODE_MULTIPLE)
    {
        viCheckParm(rsspecan_SetAttributeViInt32(instrSession, NULL, RSSPECAN_ATTR_PULSE_STABILITY_BURST_LENGTH, lengthPulses),
                3, "Length Pulse");
    }

    if (mode == RSSPECAN_VAL_PULSE_STABILITY_BURST_MODE_SINGLE)
    {
        viCheckParm(rsspecan_SetAttributeViInt32(instrSession, NULL, RSSPECAN_ATTR_PULSE_STABILITY_SELECTED_PULSE, selectedPulse),
                4, "Selected Pulse");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Configures pulse stability measurement burst analysis.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR referenceStart/Defines the first pulse number used to determine the reference values
/// HIPAR referenceStart/for pulse stability. This allows you to remove initial settling effects
/// HIPAR referenceStart/from the reference values.
/// HIPAR analysisStart/Defines the first measured pulse number included in pulse stability
/// HIPAR analysisStart/analysis. This allows you to remove initial settling effects from
/// HIPAR analysisStart/statistical results.
/// HIPAR pulsePulseOffset/Determines the number of pulses to be analyzed for pulse-to-pulse
/// HIPAR pulsePulseOffset/results.
/// HIPAR frequencyOffsetCompensation/Determines the interval over which frequency compensation is
/// HIPAR frequencyOffsetCompensation/performed.
ViStatus _VI_FUNC rsspecan_ConfigurePulseStabilityBurstAnalysis(
                            ViSession instrSession,
                            ViInt32   referenceStart,
                            ViInt32   analysisStart,
                            ViInt32   pulsePulseOffset,
                            ViInt32   frequencyOffsetCompensation)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, NULL, RSSPECAN_ATTR_PULSE_STABILITY_REFERENCE_START, referenceStart),
            2, "Reference Start");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, NULL, RSSPECAN_ATTR_PULSE_STABILITY_ANALYSIS_START, analysisStart),
            3, "Analysis Start");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, NULL, RSSPECAN_ATTR_PULSE_STABILITY_PULSE_PULSE_OFFSET, pulsePulseOffset),
            4, "Pulse-pulse Offset");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, NULL, RSSPECAN_ATTR_PULSE_STABILITY_FREQUENCY_OFFSET_COMPENSATION, frequencyOffsetCompensation),
            4, "Frequency Offset Compensation");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function configures the pulse measurement top (100%) level
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR position/Determines whether the 100% value (from base to top) for the rise and fall
/// HIPAR position/time measurements is calculated from the Edges.
/// HIPAR measurementAlgorithm/This control defines the algorithm used to detect the pulse top level.
/// HIPAR ripplePortion/Defines a hysteresis for pulse detection in dB in relation to the defined threshold.
ViStatus _VI_FUNC rsspecan_ConfigurePulseTopLevel(ViSession instrSession,
                                                  ViInt32   position,
                                                  ViInt32   measurementAlgorithm,
                                                  ViReal64  ripplePortion)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_PULSE_TOP_LEVEL_POSITION, position),
            2, "Position");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_PULSE_TOP_LEVEL_ALGORITHM, measurementAlgorithm),
            3, "Measurement Algorithm");

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_PULSE_TOP_LEVEL_RIPPLE_PORTION, ripplePortion),
            4, "Ripple Portion");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the top power level value to be used by the pulse measurement
/// HIFN  algorithm. This command is only available for
/// HIFN  SENSe:TRACe:MEASurement:ALGorithm FIXed
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR levelValue/Defines the top power level value to be used by the pulse measurement
/// HIPAR levelValue/algorithm. This command is only available for
/// HIPAR levelValue/SENSe:TRACe:MEASurement:ALGorithm FIXed
ViStatus _VI_FUNC rsspecan_ConfigurePulseTopLevelFixed(ViSession instrSession,
                                                       ViReal64  levelValue)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_PULSE_TOP_LEVEL_FIXED, levelValue),
            2, "Level Value");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function configures the pulse measurement reference level.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR unit/Defines the unit of the pulse amplitude values.
/// HIPAR highThreshold/The upper threshold in percent of the pulse amplitude.
/// HIPAR middleThreshold/The middle threshold in percent of the pulse amplitude.
/// HIPAR middleThreshold/The lower threshold in percent of the pulse amplitude.
/// HIPAR boundary/The boundary in percent of the pulse amplitude.
ViStatus _VI_FUNC rsspecan_ConfigurePulseReferenceLevel(ViSession instrSession,
                                                        ViInt32   unit,
                                                        ViReal64  highThreshold,
                                                        ViReal64  middleThreshold,
                                                        ViReal64  lowerThreshold,
                                                        ViReal64  boundary)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_PULSE_REFERENCE_LEVEL_UNIT, unit),
            2, "Unit");

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_PULSE_REFERENCE_LEVEL_THRESHOLD_HREF, highThreshold),
            3, "High Threshold");

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_PULSE_REFERENCE_LEVEL_THRESHOLD_REF, middleThreshold),
            4, "Middle Threshold");

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_PULSE_REFERENCE_LEVEL_THRESHOLD_LREF, lowerThreshold),
            5, "Lower Threshold");

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_PULSE_REFERENCE_LEVEL_BOUNDARY, boundary),
            6, "Boundary");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function selects a particular pulse for which the traces, parameters and results are displayed.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR pulseNumber/Selects a particular pulse for which the traces, parameters and results are displayed.
ViStatus _VI_FUNC rsspecan_ConfigurePulseSelection(ViSession instrSession,
                                                   ViInt32   pulseNumber)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_PULSE_SELECTION, pulseNumber),
            2, "Pulse Number");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures whether the result range length is
/// HIFN  determined automatically according to the width of the selected
/// HIFN  pulse.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init
/// HIPAR instrSession/or rsspecan_InitWithOptions function.
/// HIPAR automaticRangeScaling/If enabled, the result range length is determined automatically
/// HIPAR automaticRangeScaling/according to the width of the selected pulse.
ViStatus _VI_FUNC rsspecan_ConfigurePulseResultAutomaticRangeScaling(
                            ViSession instrSession,
                            ViInt32   automaticRangeScaling)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    if (automaticRangeScaling == RSSPECAN_VAL_PULSE_RESULT_SCALE_ONCE)
    {
        viCheckParm(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_PULSE_RESULT_AUTOMATIC_RANGE_SCALING_ONCE, NULL),
                2, "Automatic Range Scaling");
    }
    else
    {
        viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "",
                        RSSPECAN_ATTR_PULSE_RESULT_AUTOMATIC_RANGE_SCALING,
                        automaticRangeScaling != 0), 2, "Automatic Range Scaling");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function configures the pulse refernece point and offset.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR reference/Specifies the reference point used to define the result range.
/// HIPAR offset/The offset (in seconds) from the reference point at which the pulse result range is aligned.
ViStatus _VI_FUNC rsspecan_ConfigurePulseResultReferencePoint(
                            ViSession instrSession,
                            ViInt32   reference,
                            ViReal64  offset)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_PULSE_RESULT_RANGE_REFERENCE, reference),
            2, "Reference");

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_PULSE_RESULT_RANGE_OFFSET, offset),
            3, "Offset");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function configures the pulse result range.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR alignment/Specifies the alignment with respect to the reference point used to define the result range.
/// HIPAR length/The length of the pulse result range (in seconds).
ViStatus _VI_FUNC rsspecan_ConfigurePulseResultRange(ViSession instrSession,
                                                     ViInt32   alignment,
                                                     ViReal64  length)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_PULSE_RESULT_RANGE_ALIGNMENT, alignment),
            2, "Alignment");

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_PULSE_RESULT_RANGE_LENGTH, length),
            3, "Length");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function configures the pulse result range spectrum.
/// HIRET Document return value here.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR windowType/Defines the used FFT window type.
/// HIPAR RBWAuto/This control activates/deactivates the automatic selection of
/// HIPAR RBWAuto/optimal RBW for the Result Range Spectrum.
/// HIPAR RBW/This control sets the resolution bandwidth for Result Range
/// HIPAR RBW/Spectrum Settings.
ViStatus _VI_FUNC rsspecan_ConfigurePulseResultRangeSpectrum(ViSession instrSession,
                                                             ViInt32   window,
                                                             ViInt32   windowType,
                                                             ViBoolean RBWAuto,
                                                             ViReal64  RBW)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_PULSE_RESULT_RANGE_SPECTRUM_WINDOW_TYPE,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap),
            2, "Window");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_PULSE_RESULT_RANGE_SPECTRUM_WINDOW_TYPE, windowType),
            3, "Window Type");

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, repCap, RSSPECAN_ATTR_PULSE_RESULT_RANGE_SPECTRUM_RBW_AUTO, RBWAuto),
            4, "RBW Auto");
    if (RBWAuto == VI_FALSE)
    {
        viCheckParm(rsspecan_SetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_PULSE_RESULT_RANGE_SPECTRUM_RBW, RBW),
                5, "RBW");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN This function selects the unit for angles (for PM display).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR unit/This control selects the unit for angles (for PM display).
ViStatus _VI_FUNC rsspecan_ConfigurePulsePhaseUnit(ViSession instrSession,
                                                   ViInt32   unit)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_PULSE_RESULT_PHASE_UNIT, unit),
            2, "Unit");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function normalizes pulse phase traces to a specific phase value.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR normalization/Sets the normalization value.
ViStatus _VI_FUNC rsspecan_ConfigurePulsePhaseNormalization(ViSession instrSession,
                                                            ViInt32   window,
                                                            ViReal64  normalization)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_PULSE_RESULT_PHASE_NORMALIZATION,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap),
            2, "Window");

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_PULSE_RESULT_PHASE_NORMALIZATION, normalization),
            2, "Normalization");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN This function selects the unit for frequency values.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR unit/This control selects the unit for frequency
ViStatus _VI_FUNC rsspecan_ConfigurePulseFrequencyUnit(ViSession instrSession,
                                                       ViInt32   unit)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_PULSE_RESULT_FREQUENCY_UNIT, unit),
            2, "Unit");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Contains settings for the pulse-to-pulse spectrum evaluation.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR autoSettings/Enables or disables automatic configuration for Pulse-to-Pulse
/// HIPAR autoSettings/Spectrum displays. If enabled, the commands for individual settings are
/// HIPAR autoSettings/not available.
/// HIPAR gapThreshold/Defines the minimum time that must pass before a gap is detected as
/// HIPAR gapThreshold/such.
/// HIPAR minimumSectionThreshold/Defines the minimum section size. Sections that are smaller than the
/// HIPAR minimumSectionThreshold/threshold are ignored and considered to be part of the detected gap.
/// HIPAR maximumFrequency/Defines the maximum frequency span for which the Spectrum is
/// HIPAR maximumFrequency/calculated. Internally, the span is limited by the number of possible
/// HIPAR maximumFrequency/interpolation samples (100 000).
ViStatus _VI_FUNC rsspecan_ConfigurePulsePulseSettings(ViSession instrSession,
                                                       ViInt32   window,
                                                       ViBoolean autoSettings,
                                                       ViReal64  gapThreshold,
                                                       ViReal64  minimumSectionThreshold,
                                                       ViReal64  maximumFrequency)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "Win%ld", window);

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, repCap, RSSPECAN_ATTR_PULSE_PULSE_AUTO_ENABLED, autoSettings),
            2, "Auto Settings");

    if (autoSettings == VI_FALSE)
    {
        viCheckParm(rsspecan_SetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_PULSE_PULSE_GAP_THRESHOLD, gapThreshold),
                3, "Gap Threshold");

        viCheckParm(rsspecan_SetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_PULSE_PULSE_MINIMUM_SECTION_THRESHOLD, minimumSectionThreshold),
                4, "Minimum Section Threshold");

        viCheckParm(rsspecan_SetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_PULSE_PULSE_MAXIMUM_FREQUENCY, maximumFrequency),
                5, "Maximum Frequency");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Defines the used FFT window type for pulse-to-pulse spectrum displays.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR windowType/Defines the used FFT window type for pulse-to-pulse spectrum displays.
ViStatus _VI_FUNC rsspecan_ConfigurePulsePulseWindowType(ViSession instrSession,
                                                         ViInt32   window,
                                                         ViInt32   windowType)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "Win%ld", window);

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_PULSE_PULSE_WINDOW_TYPE, windowType),
            2, "Window Type");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the resulting resolution bandwidth for the spectrum. Depends
/// HIFN  on the block size (see CALCulate<n>:PSPectrum:BLOCksize).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR resolutionBandwith/Queries the resulting resolution bandwidth for the spectrum. Depends
/// HIPAR resolutionBandwith/on the block size (see CALCulate<n>:PSPectrum:BLOCksize).
ViStatus _VI_FUNC rsspecan_QueryPulsePulseResolutionBandwidth(
                            ViSession instrSession,
                            ViInt32   window,
                            ViReal64* resolutionBandwidth)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "Win%ld", window);

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_PULSE_PULSE_RESOLUTION_BANDWITH, resolutionBandwidth),
            2, "Resolution Bandwidth");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function configures the pulse result Power Parameter Trend.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR axis/Selects parameter group of parameter trend.
/// HIPAR group/Defines the scaling of the x-axis.
ViStatus _VI_FUNC rsspecan_ConfigurePulseResultParameterTrendPower(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   group,
                            ViInt32   axis)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "Win%ld", window);

    if (group == RSSPECAN_VAL_GROUP_Y)
    {
        viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_PULSE_RESULT_PARAMETER_TREND_POWER_Y_AXIS, axis),
                4, "Axis");
    }
    else
    {
        viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_PULSE_RESULT_PARAMETER_TREND_POWER_X_AXIS, axis),
                4, "Axis");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function configures the pulse result Timing Parameter Trend.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR axis/Selects parameter group of parameter trend.
/// HIPAR group/Defines the scaling of the x-axis.
ViStatus _VI_FUNC rsspecan_ConfigurePulseResultParameterTrendTiming(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   group,
                            ViInt32   axis)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "Win%ld", window);

    if (group == RSSPECAN_VAL_GROUP_Y)
    {
        viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_PULSE_RESULT_PARAMETER_TREND_TIMING_Y_AXIS, axis),
                4, "Axis");
    }
    else
    {
        viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_PULSE_RESULT_PARAMETER_TREND_TIMING_X_AXIS, axis),
                4, "Axis");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function configures the pulse result Frequency Parameter Trend.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR axis/Selects parameter group of parameter trend.
/// HIPAR group/Defines the scaling of the x-axis.
ViStatus _VI_FUNC rsspecan_ConfigurePulseResultParameterTrendFrequency(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   group,
                            ViInt32   axis)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "Win%ld", window);

    if (group == RSSPECAN_VAL_GROUP_Y)
    {
        viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_PULSE_RESULT_PARAMETER_TREND_FREQUENCY_Y_AXIS, axis),
                4, "Axis");
    }
    else
    {
        viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_PULSE_RESULT_PARAMETER_TREND_FREQUENCY_X_AXIS, axis),
                4, "Axis");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function configures the pulse result Phase Parameter Trend.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR axis/Selects parameter group of parameter trend.
/// HIPAR group/Defines the scaling of the x-axis.
ViStatus _VI_FUNC rsspecan_ConfigurePulseResultParameterTrendPhase(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   group,
                            ViInt32   axis)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "Win%ld", window);

    if (group == RSSPECAN_VAL_GROUP_Y)
    {
        viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_PULSE_RESULT_PARAMETER_TREND_PHASE_Y_AXIS, axis),
                4, "Axis");
    }
    else
    {
        viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_PULSE_RESULT_PARAMETER_TREND_PHASE_X_AXIS, axis),
                4, "Axis");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function hides or shows the limit lines in the selected
/// HIFN  Parameter Trend  result display.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR state/This control sets the limit display state.
ViStatus _VI_FUNC rsspecan_ConfigurePulseResultParameterTrendLimitLines(
                            ViSession instrSession,
                            ViInt32   window,
                            ViBoolean state)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_PULSE_RESULT_PARAMETER_TREND_DISPLAY_LIMIT_LINES_ENABLED,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap),
            2, "Window");

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, repCap, RSSPECAN_ATTR_PULSE_RESULT_PARAMETER_TREND_DISPLAY_LIMIT_LINES_ENABLED, state),
            3, "State");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Determines the trace style for the parameter trend.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR displayStyle/Determines the trace style for the parameter trend.
ViStatus _VI_FUNC rsspecan_ConfigurePulseResultParameterTrendDisplayStyle(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   displayStyle)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, window, 0, 16),
            2, "Window");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_PULSE_RESULT_PARAMETER_TREND_DISPLAY_STYLE,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap),
            2, "Window");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_PULSE_RESULT_PARAMETER_TREND_DISPLAY_STYLE, displayStyle),
            3, "Display Style");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function configures the pulse result Time Sidelobe Parameter Trend.
/// HIFN This function configures the pulse result Phase Parameter Trend.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR group/Defines the scaling of the x-axis.
/// HIPAR axis/Selects parameter group of parameter trend.
ViStatus _VI_FUNC rsspecan_ConfigurePulseResultParameterTrendTimeSidelobe(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   group,
                            ViInt32   axis)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_PULSE_RESULT_PARAMETER_TREND_TIME_SIDELOBE_Y_AXIS,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap),
            2, "Window");

    switch (group)
    {
    case RSSPECAN_VAL_GROUP_X:
        viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_PULSE_RESULT_PARAMETER_TREND_TIME_SIDELOBE_X_AXIS, axis),
                4, "Axis");
        break;
    case RSSPECAN_VAL_GROUP_Y:
        viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_PULSE_RESULT_PARAMETER_TREND_TIME_SIDELOBE_Y_AXIS, axis),
                4, "Axis");
        break;
    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, group), 3, "Group");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the pulse result Envelope Model Parameter
/// HIFN  Trend.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR group/Selects parameter group of parameter trend.
/// HIPAR axis/Defines the parameter for which the trend is displayed on the y-axis.
ViStatus _VI_FUNC rsspecan_ConfigurePulseResultParameterTrendEnvelopeModel(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   group,
                            ViInt32   axis)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "Win%ld", window);

    if (group == RSSPECAN_VAL_GROUP_Y)
    {
        viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_PULSE_RESULT_PARAMETER_TREND_ENVELOPE_MODEL_Y_AXIS, axis),
                4, "Axis");
    }
    else
    {
        viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_PULSE_RESULT_PARAMETER_TREND_ENVELOPE_MODEL_X_AXIS, axis),
                4, "Axis");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function configures the pulse result stability Parameter Trend.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR group/Selects parameter group of parameter trend.
/// HIPAR axis/Defines the parameter for which the trend is displayed on the axis.
ViStatus _VI_FUNC rsspecan_ConfigurePulseResultParameterTrendStability(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   group,
                            ViInt32   axis)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repcap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  repcapCount = 0;

    checkErr(RsCore_LockSession(instrSession));

    (void)RsCore_GetRepCapIdItemsCount(instrSession, "Window", &repcapCount);

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, window, 0, repcapCount - 1),
        2, "Window");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_PULSE_RESULT_PARAMETER_TREND_STABILITY_Y_AXIS, "Window", window, RS_REPCAP_BUF_SIZE, repcap),
        2, "Window");

    if(group == RSSPECAN_VAL_GROUP_Y)
        viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repcap, RSSPECAN_ATTR_PULSE_RESULT_PARAMETER_TREND_STABILITY_Y_AXIS, axis),
            3, "Axis")
    else
        viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repcap, RSSPECAN_ATTR_PULSE_RESULT_PARAMETER_TREND_STABILITY_X_AXIS, axis),
                3, "Axis");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function sets the number of bins used to calculate the histogram.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR numberOfBins/This control sets the number of bins used to calculate the histogram.
ViStatus _VI_FUNC rsspecan_ConfigurePulseResultParameterDistributionBins(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   numberOfBins)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "Win%ld", window);

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_PULSE_PARAMETER_DISTRIBUTION_BINS, numberOfBins),
            3, "Number Of Bins");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function configures the Power Parameter Distribution result
/// HIFN  display.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR yAxis/This control defines the parameter for which the trend is displayed on
/// HIPAR yAxis/the y-axis.
/// HIPAR xAxis/Defines the scaling of the x-axis.
ViStatus _VI_FUNC rsspecan_ConfigurePulseResultParameterDistributionPower(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   xAxis,
                            ViInt32   yAxis)
{
    ViStatus error = VI_SUCCESS;
    ViChar*  p2Ystring;
    ViChar*  p2string;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K6"));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, window, 0, 16),
            2, "Window");
    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &rsspecan_rngPulseParameterPowerTrend, xAxis, &p2string),
                3, "X Axis");
    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &rsspecan_rngPulseParameterYAxis, yAxis, &p2Ystring),
                4, "Y Axis");

    if (window == 0)
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CALC:DIST:POW %s,%s", p2string, p2Ystring);
    else
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CALC%ld:DIST:POW %s,%s", window, p2string, p2Ystring);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function configures the Timing Parameter Distribution result
/// HIFN  display.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR yAxis/Defines the parameter for which the trend is displayed on the y-axis.
/// HIPAR xAxis/Defines the scaling of the x-axis.
ViStatus _VI_FUNC rsspecan_ConfigurePulseResultParameterDistributionTiming(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   xAxis,
                            ViInt32   yAxis)
{
    ViStatus error = VI_SUCCESS;
    ViChar*  p2Ystring;
    ViChar*  p2string;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K6"));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, window, 0, 16),
            2, "Window");
    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &rsspecan_rngPulseParameterTiming, xAxis, &p2string),
                3, "X Axis");
    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &rsspecan_rngPulseParameterYAxis, yAxis, &p2Ystring),
                4, "Y Axis");

    if (window == 0)
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CALC:DIST:TIM %s,%s", p2string, p2Ystring);
    else
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CALC%ld:DIST:TIM %s,%s", window, p2string, p2Ystring);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function configures the Frequency Parameter Distribution result
/// HIFN  display.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR yAxis/Defines the parameter for which the trend is displayed on the y-axis.
/// HIPAR xAxis/Defines the scaling of the x-axis.
ViStatus _VI_FUNC rsspecan_ConfigurePulseResultParameterDistributionFrequency(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   xAxis,
                            ViInt32   yAxis)
{
    ViStatus error = VI_SUCCESS;
    ViChar*  p2Ystring;
    ViChar*  p2string;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K6"));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, window, 0, 16),
            2, "Window");
    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &rsspecan_rngPulseParameterFrequency, xAxis, &p2string),
                3, "X Axis");
    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &rsspecan_rngPulseParameterYAxis, yAxis, &p2Ystring),
                4, "Y Axis");

    if (window == 0)
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CALC:DIST:FREQ %s,%s", p2string, p2Ystring);
    else
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CALC%ld:DIST:FREQ %s,%s", window, p2string, p2Ystring);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function configures the Phase Parameter Distribution result
/// HIFN  display.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR yAxis/Defines the parameter for which the trend is displayed on the y-axis.
/// HIPAR xAxis/Defines the scaling of the x-axis.
ViStatus _VI_FUNC rsspecan_ConfigurePulseResultParameterDistributionPhase(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   xAxis,
                            ViInt32   yAxis)
{
    ViStatus error = VI_SUCCESS;
    ViChar*  p2Ystring;
    ViChar*  p2string;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K6"));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, window, 0, 16),
            2, "Window");
    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &rsspecan_rngPulseParameterPhase, xAxis, &p2string),
                3, "X Axis");
    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &rsspecan_rngPulseParameterYAxis, yAxis, &p2Ystring),
                4, "Y Axis");

    if (window == 0)
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CALC:DIST:PHAS %s,%s", p2string, p2Ystring);
    else
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CALC%ld:DIST:PHAS %s,%s", window, p2string, p2Ystring);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function hides or shows the limit lines in the selected
/// HIFN  Parameter Distribution result display.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR state/This control sets the limit display state.
ViStatus _VI_FUNC rsspecan_ConfigurePulseResultParameterDistributionLimitLines(
                            ViSession instrSession,
                            ViInt32   window,
                            ViBoolean state)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_PULSE_RESULT_PARAMETER_DISTRIBUTION_DISPLAY_LIMIT_LINES_ENABLED,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap),
            2, "Window");

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, repCap, RSSPECAN_ATTR_PULSE_RESULT_PARAMETER_DISTRIBUTION_DISPLAY_LIMIT_LINES_ENABLED, state),
            3, "State");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the Time Sidelobe Parameter Distribution result
/// HIFN  display.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR xAxis/Time Sidelobe parameter to be displayed on the x-axis.
/// HIPAR yAxis/Parameter to be displayed on the y-axis.
ViStatus _VI_FUNC rsspecan_ConfigurePulseResultParameterDistributionTimeSidelobe(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   xAxis,
                            ViInt32   yAxis)
{
    ViStatus error = VI_SUCCESS;
    ViChar*  p2Ystring;
    ViChar*  p2string;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K6S"));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, window, 0, 16),
            2, "Window");
    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &RsSpecAn_rngPulseParameterTimeSidelobe, xAxis, &p2string),
                3, "X Axis");
    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &rsspecan_rngPulseParameterYAxis, yAxis, &p2Ystring),
                4, "Y Axis");

    if (window == 0)
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CALC:DIST:TSID %s,%s", p2string, p2Ystring);
    else
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CALC%ld:DIST:TSID %s,%s", window, p2string, p2Ystring);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the Envelope Model Parameter Distribution
/// HIFN  result display.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     -
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     CALCulate<n>:DISTribution:EMODel
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR xAxis/Envelope Model parameter displayed on X Axis.
/// HIPAR yAxis/Parameter to be displayed on the y-axis.
ViStatus _VI_FUNC rsspecan_ConfigurePulseResultParameterDistributionEnvelopeModel(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   xAxis,
                            ViInt32   yAxis)
{
    ViStatus error = VI_SUCCESS;
    ViChar*  p2Ystring;
    ViChar*  p2string;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K6"));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, window, 0, 16),
            2, "Window");
    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &RsSpecAn_rngPulseParameterEnvelopeModel, xAxis, &p2string),
        3, "X Axis");
    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &rsspecan_rngPulseParameterYAxis, yAxis, &p2Ystring),
                4, "Y Axis");

    if (window == 0)
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CALC:DIST:EMOD %s,%s", p2string, p2Ystring);
    else
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CALC%ld:DIST:EMOD %s,%s", window, p2string, p2Ystring);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Configures the Stability Parameter Distribution result display.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR xAxis/Stability parameter displayed on X Axis.
/// HIPAR yAxis/Parameter to be displayed on the y-axis.
ViStatus _VI_FUNC rsspecan_ConfigurePulseResultParameterDistributionStability(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   xAxis,
                            ViInt32   yAxis)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repcap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  repcapCount = 0;
    ViChar   param[RS_PARAM_BUF_SIZE] = "";
    ViChar*  xAxisCmd;
    ViChar*  yAxisCmd;

    checkErr(RsCore_LockSession(instrSession));

    (void)RsCore_GetRepCapIdItemsCount(instrSession, "Window", &repcapCount);

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, window, 0, repcapCount - 1),
        2, "Window");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_HIDDEN_PULSE_CONFIGURE_RESULT_PARAMETER_DISTRIBUTION_STABILITY, "Window", window, RS_REPCAP_BUF_SIZE, repcap),
        2, "Window");
    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &RsSpecAn_rngPulseParameterStability, xAxis, &xAxisCmd),
        3, "X Axis");
    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &rsspecan_rngPulseParameterYAxis, yAxis, &yAxisCmd),
        4, "Y Axis");

    (void)snprintf(param, RS_PARAM_BUF_SIZE, "%s,%s", xAxisCmd, yAxisCmd);

    checkErr(rsspecan_SetAttributeViString(instrSession, repcap, RSSPECAN_ATTR_HIDDEN_PULSE_CONFIGURE_RESULT_PARAMETER_DISTRIBUTION_STABILITY, param));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the Phase Parameter Distribution result
/// HIFN  display.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
ViStatus _VI_FUNC rsspecan_ConfigurePulseResultParameterSpectrumFullAuto(
                            ViSession instrSession,
                            ViInt32   window)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_PULSE_PARAMETER_SPECTRUM_FULL_AUTO,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap),
            2, "Window");

    checkErr(rsspecan_SetAttributeViBoolean(instrSession, repCap, RSSPECAN_ATTR_PULSE_PARAMETER_SPECTRUM_FULL_AUTO, VI_TRUE));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function enables automatic configuration for Parameter Spectrum displays
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR maximumFrequency/Defines the maximum frequency span for which the Spectrum is calculated.
/// HIPAR windowType/Defines the used FFT window type.
/// HIPAR blockSize/Defines the size of blocks used in spectrum calculation.
/// HIPAR gapThreshold/Defines the minimum time that must pass before a gap is detected as such.
/// HIPAR segmentThreshold/Defines the minimum segment size.
ViStatus _VI_FUNC rsspecan_ConfigurePulseResultParameterSpectrumManual(
                            ViSession instrSession,
                            ViInt32   window,
                            ViReal64  maximumFrequency,
                            ViInt32   windowType,
                            ViInt32   blockSize,
                            ViReal64  gapThreshold,
                            ViReal64  segmentThreshold)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_PULSE_PARAMETER_SPECTRUM_FULL_AUTO,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap),
            2, "Window");

    checkErr(rsspecan_SetAttributeViBoolean(instrSession, repCap, RSSPECAN_ATTR_PULSE_PARAMETER_SPECTRUM_FULL_AUTO, VI_FALSE));
    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_PULSE_PARAMETER_SPECTRUM_MAXIMUM_FREQUENCY, maximumFrequency),
            4, "Maximum Frequency");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_PULSE_PARAMETER_SPECTRUM_WINDOW_TYPE, windowType),
            5, "Window Type");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_PULSE_PARAMETER_SPECTRUM_BLOCK_SIZE, blockSize),
            6, "Block Size");

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_PULSE_PARAMETER_SPECTRUM_GAP_THRESHOLD, gapThreshold),
            7, "Gap Threshold");

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_PULSE_PARAMETER_SPECTRUM_SEGMENT_THRESHOLD, segmentThreshold),
            8, "Segment Threshold");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures parameter to be displayed on the x-axis.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR parameter/This control selects the frequency parameter.
ViStatus _VI_FUNC rsspecan_ConfigurePulseResultParameterSpectrumFrequency(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   parameter)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, window, 0, 16),
            2, "Window");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "Win%ld", window);

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_PULSE_PARAMETER_SPECTRUM_FREQUENCY, parameter),
            3, "Parameter");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function configures parameter to be displayed on the x-axis.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSSPECAN_ATTR_PULSE_PARAMETER_SPECTRUM_PHASE
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     CALCulate<n>:PSPectrum:PHASe
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR parameter/This control selects the phase parameter.
ViStatus _VI_FUNC rsspecan_ConfigurePulseResultParameterSpectrumPhase(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   parameter)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, window, 0, 16),
            2, "Window");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "Win%ld", window);

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_PULSE_PARAMETER_SPECTRUM_PHASE, parameter),
            3, "Parameter");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function configures parameter to be displayed on the x-axis.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR parameter/This control selects the power parameter.
ViStatus _VI_FUNC rsspecan_ConfigurePulseResultParameterSpectrumPower(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   parameter)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, window, 0, 16),
            2, "Window");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "Win%ld", window);

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_PULSE_PARAMETER_SPECTRUM_POWER, parameter),
            3, "Parameter");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function configures parameter to be displayed on the x-axis.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSSPECAN_ATTR_PULSE_PARAMETER_SPECTRUM_TIMING
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     CALCulate<n>:PSPectrum:TIMing
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR parameter/This control selects the timing parameter.
ViStatus _VI_FUNC rsspecan_ConfigurePulseResultParameterSpectrumTiming(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   parameter)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, window, 0, 16),
            2, "Window");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "Win%ld", window);

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_PULSE_PARAMETER_SPECTRUM_TIMING, parameter),
            3, "Parameter");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the Time Sidelobe Parameter Spectrum
/// HIFN  result display.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR parameter/This control selects the time sidelobe parameter.
ViStatus _VI_FUNC rsspecan_ConfigurePulseResultParameterSpectrumTimeSidelobe(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   parameter)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_PULSE_PARAMETER_SPECTRUM_TIME_SIDELOBE,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap),
            2, "Window");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_PULSE_PARAMETER_SPECTRUM_TIME_SIDELOBE, parameter),
            3, "Parameter");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function configures the Envelope Model Parameter Spectrum result
/// HIFN  display.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR parameter/This control selects the envelope model parameter.
ViStatus _VI_FUNC rsspecan_ConfigurePulseResultParameterSpectrumEnvelopeModel(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   parameter)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, window, 0, 16),
            2, "Window");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "Win%ld", window);

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_PULSE_PARAMETER_SPECTRUM_ENVELOPE_MODEL, parameter),
            3, "Parameter");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Configures the Stability Parameter Spectrum result display.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR parameter/Configures the Stability Parameter Spectrum result display.
ViStatus _VI_FUNC rsspecan_ConfigurePulseParameterSpectrumStability(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   parameter)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repcap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  repcapCount = 0;

    checkErr(RsCore_LockSession(instrSession));

    (void)RsCore_GetRepCapIdItemsCount(instrSession, "Window", &repcapCount);

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, window, 0, repcapCount - 1),
        2, "Window");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_PULSE_PARAMETER_SPECTRUM_STABILITY, "Window", window, RS_REPCAP_BUF_SIZE, repcap),
        2, "Window");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repcap, RSSPECAN_ATTR_PULSE_PARAMETER_SPECTRUM_STABILITY, parameter),
            3, "Parameter");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Defines the parameter used for the z-axis of the Stability Waterfall
/// HIFN  diagram.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR stabilityWaterfallZAxis/Defines the parameter used for the z-axis of the Stability Waterfall
/// HIPAR stabilityWaterfallZAxis/diagram.
ViStatus _VI_FUNC rsspecan_ConfigurePulseResultParameterStabilityWaterfallZAxis(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   stabilityWaterfallZAxis)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repcap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  repcapCount = 0;

    checkErr(RsCore_LockSession(instrSession));

    (void)RsCore_GetRepCapIdItemsCount(instrSession, "Window", &repcapCount);

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, window, 0, repcapCount - 1),
        2, "Window");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_PULSE_STABILITY_WATERFALL_Z_AXIS, "Window", window, RS_REPCAP_BUF_SIZE, repcap),
        2, "Window");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repcap, RSSPECAN_ATTR_PULSE_STABILITY_WATERFALL_Z_AXIS, stabilityWaterfallZAxis),
            3, "Stability Waterfall Z Axis");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Hides or shows trace parameters in Stability Waterfall diagrams.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR xAxisGridLines/Hides or shows trace lines for the 2-dimensional x-z trace in
/// HIPAR xAxisGridLines/Stability Waterfall diagrams.
/// HIPAR yAxisGridLines/Hides or shows trace lines for the 2-dimensional y-z trace in
/// HIPAR yAxisGridLines/Stability Waterfall diagrams.
/// HIPAR measurementInfo/Hides or shows measurement information within the Stability Waterfall
/// HIPAR measurementInfo/diagram.
/// HIPAR zAxisMeasurementPlane/Hides or shows the z-axis measurement plane in the waterfall.
ViStatus _VI_FUNC rsspecan_ConfigurePulseResultParameterStabilityWaterfallDisplay(
                            ViSession instrSession,
                            ViInt32   window,
                            ViBoolean xAxisGridLines,
                            ViBoolean yAxisGridLines,
                            ViBoolean measurementInfo,
                            ViBoolean zAxisMeasurementPlane)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repcap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  repcapCount = 0;

    checkErr(RsCore_LockSession(instrSession));

    (void)RsCore_GetRepCapIdItemsCount(instrSession, "Window", &repcapCount);

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, window, 0, repcapCount - 1),
        2, "Window");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_PULSE_STABILITY_WATERFALL_X_AXIS_GRID_LINES, "Window", window, RS_REPCAP_BUF_SIZE, repcap),
        2, "Window");

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, repcap, RSSPECAN_ATTR_PULSE_STABILITY_WATERFALL_X_AXIS_GRID_LINES, xAxisGridLines),
            3, "X Axis Grid Lines");

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, repcap, RSSPECAN_ATTR_PULSE_STABILITY_WATERFALL_Y_AXIS_GRID_LINES, yAxisGridLines),
            4, "Y Axis Grid Lines");

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, repcap, RSSPECAN_ATTR_PULSE_STABILITY_WATERFALL_MEASUREMENT_INFO, measurementInfo),
            5, "Measurement Info");

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, repcap, RSSPECAN_ATTR_PULSE_STABILITY_WATERFALL_Z_AXIS_MEASUREMENT_PLANE, zAxisMeasurementPlane),
            6, "Z Axis Measurement Plane");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Links the parameters in all Stability Waterfall diagrams, so that a
/// HIFN  change in one diagram is applied to all others identically.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR cameraAngle/Links the camera angle in all Stability Waterfall diagrams, so that a
/// HIPAR cameraAngle/change in one diagram is applied to all others identically.
/// HIPAR zAxisScale/Links the scaling for the z-axis in all Stability Waterfall diagrams,
/// HIPAR zAxisScale/so that a change in one diagram is applied to all others identically.
ViStatus _VI_FUNC rsspecan_ConfigurePulseResultParameterStabilityWaterfallLinkWindows(
                            ViSession instrSession,
                            ViInt32   window,
                            ViBoolean cameraAngle,
                            ViBoolean zAxisScale)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repcap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  repcapCount = 0;

    checkErr(RsCore_LockSession(instrSession));

    (void)RsCore_GetRepCapIdItemsCount(instrSession, "Window", &repcapCount);

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, window, 0, repcapCount - 1),
        2, "Window");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_PULSE_STABILITY_WATERFALL_LINK_WINDOWS_CAMERA_ANGLE, "Window", window, RS_REPCAP_BUF_SIZE, repcap),
        2, "Window");

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, repcap, RSSPECAN_ATTR_PULSE_STABILITY_WATERFALL_LINK_WINDOWS_CAMERA_ANGLE, cameraAngle),
            3, "Camera Angle");

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, repcap, RSSPECAN_ATTR_PULSE_STABILITY_WATERFALL_LINK_WINDOWS_Z_AXIS_SCALE, zAxisScale),
            4, "Z Axis Scale");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function shows the power parameter in result table.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR parameter/This control selects the power parameter.
/// HIPAR visibility/Shows / hides the parameter in the result table.
ViStatus _VI_FUNC rsspecan_ConfigurePulseResultTablePower(ViSession instrSession,
                                                          ViInt32   window,
                                                          ViInt32   parameter,
                                                          ViBoolean visibility)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "Win%ld", window);

    switch (parameter)
    {
    case RSSPECAN_VAL_PULSE_RESULT_POWER_ALL:
        viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, repCap, RSSPECAN_ATTR_PULSE_RESULT_TABLE_POWER_ALL, visibility),
                4, "visibility");
        break;
    case RSSPECAN_VAL_PULSE_RESULT_POWER_TOP:
        viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, repCap, RSSPECAN_ATTR_PULSE_RESULT_TABLE_POWER_TOP, visibility),
                4, "visibility");
        break;
    case RSSPECAN_VAL_PULSE_RESULT_POWER_BASE:
        viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, repCap, RSSPECAN_ATTR_PULSE_RESULT_TABLE_POWER_BASE, visibility),
                4, "visibility");
        break;
    case RSSPECAN_VAL_PULSE_RESULT_POWER_AMPL:
        viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, repCap, RSSPECAN_ATTR_PULSE_RESULT_TABLE_POWER_AMPLITUDE, visibility),
                4, "visibility");
        break;
    case RSSPECAN_VAL_PULSE_RESULT_POWER_ON:
        viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, repCap, RSSPECAN_ATTR_PULSE_RESULT_TABLE_POWER_ON, visibility),
                4, "visibility");
        break;
    case RSSPECAN_VAL_PULSE_RESULT_POWER_AVG:
        viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, repCap, RSSPECAN_ATTR_PULSE_RESULT_TABLE_POWER_AVERAGE, visibility),
                4, "visibility");
        break;
    case RSSPECAN_VAL_PULSE_RESULT_POWER_MIN:
        viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, repCap, RSSPECAN_ATTR_PULSE_RESULT_TABLE_POWER_MIN, visibility),
                4, "visibility");
        break;
    case RSSPECAN_VAL_PULSE_RESULT_POWER_MAX:
        viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, repCap, RSSPECAN_ATTR_PULSE_RESULT_TABLE_POWER_MAX, visibility),
                4, "visibility");
        break;
    case RSSPECAN_VAL_PULSE_RESULT_POWER_PON:
        viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, repCap, RSSPECAN_ATTR_PULSE_RESULT_TABLE_POWER_POWER_ON, visibility),
                4, "visibility");
        break;
    case RSSPECAN_VAL_PULSE_RESULT_POWER_PAVG:
        viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, repCap, RSSPECAN_ATTR_PULSE_RESULT_TABLE_POWER_POWER_AVERAGE, visibility),
                4, "visibility");
        break;
    case RSSPECAN_VAL_PULSE_RESULT_POWER_PMIN:
        viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, repCap, RSSPECAN_ATTR_PULSE_RESULT_TABLE_POWER_POWER_MIN, visibility),
                4, "visibility");
        break;
    case RSSPECAN_VAL_PULSE_RESULT_POWER_ADP:
        viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, repCap, RSSPECAN_ATTR_PULSE_RESULT_TABLE_POWER_ADD, visibility),
                4, "visibility");
        break;
    case RSSPECAN_VAL_PULSE_RESULT_POWER_ADDB:
        viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, repCap, RSSPECAN_ATTR_PULSE_RESULT_TABLE_POWER_ADDB, visibility),
                4, "visibility");
        break;
    case RSSPECAN_VAL_PULSE_RESULT_POWER_RPER:
        viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, repCap, RSSPECAN_ATTR_PULSE_RESULT_TABLE_POWER_RPER, visibility),
                4, "visibility");
        break;
    case RSSPECAN_VAL_PULSE_RESULT_POWER_RDB:
        viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, repCap, RSSPECAN_ATTR_PULSE_RESULT_TABLE_POWER_RDB, visibility),
                4, "visibility");
        break;
    case RSSPECAN_VAL_PULSE_RESULT_POWER_OPER:
        viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, repCap, RSSPECAN_ATTR_PULSE_RESULT_TABLE_POWER_OVER, visibility),
                4, "visibility");
        break;
    case RSSPECAN_VAL_PULSE_RESULT_POWER_ODB:
        viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, repCap, RSSPECAN_ATTR_PULSE_RESULT_TABLE_POWER_OVERDB, visibility),
                4, "visibility");
        break;
    case RSSPECAN_VAL_PULSE_RESULT_POWER_POIN:
        viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, repCap, RSSPECAN_ATTR_PULSE_RESULT_TABLE_POWER_POINT, visibility),
                4, "visibility");
        break;
    case RSSPECAN_VAL_PULSE_RESULT_POWER_PPR:
        viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, repCap, RSSPECAN_ATTR_PULSE_RESULT_TABLE_POWER_PPR, visibility),
                4, "visibility");
        break;
    case RSSPECAN_VAL_PULSE_RESULT_POWER_AMPL_I:
        viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, repCap, RSSPECAN_ATTR_PULSE_RESULT_TABLE_POWER_AMPLITUDE_I, visibility),
                4, "visibility");
        break;
    case RSSPECAN_VAL_PULSE_RESULT_POWER_AMPL_Q:
        viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, repCap, RSSPECAN_ATTR_PULSE_RESULT_TABLE_POWER_AMPLITUDE_Q, visibility),
                4, "visibility");
        break;
    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, parameter), 3, "Parameter");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function shows the timing parameter in result table.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR parameter/This control selects the power parameter.
/// HIPAR visibility/Shows / hides the parameter in the result table.
ViStatus _VI_FUNC rsspecan_ConfigurePulseResultTableTiming(ViSession instrSession,
                                                           ViInt32   window,
                                                           ViInt32   parameter,
                                                           ViBoolean visibility,
                                                           ViInt32   reserved)
{
    ViStatus error = VI_SUCCESS;
    ViChar*  p2string;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";
             reserved;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K6"));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, window, 0, 16),
            2, "Window");
    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &rsspecan_rngPulseResultTiming, parameter, &p2string),
                3, "Parameter");

    if (window == 0)
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CALC:TABL:TIM:%s %s", p2string, (visibility ? "ON" : "OFF"));
    else
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CALC%ld:TABL:TIM:%s %s", window, p2string, (visibility ? "ON" : "OFF"));
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function shows the frequency parameter in result table.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR parameter/This control selects the power parameter.
/// HIPAR visibility/Shows / hides the parameter in the result table.
ViStatus _VI_FUNC rsspecan_ConfigurePulseResultTableFrequency(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   parameter,
                            ViBoolean visibility,
                            ViInt32   reserved)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";
    ViChar*  p2Result;
             reserved;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K6"));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, window, 0, 16),
            2, "Window");
    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &rsspecan_rngPulseResultFrequency, parameter, &p2Result),
                3, "Parameter");

    if (window == 0)
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CALC:TABL:FREQ:%s %s", p2Result, (visibility ? "ON" : "OFF"));
    else
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CALC%ld:TABL:FREQ:%s %s", window, p2Result,
                 (visibility ? "ON" : "OFF"));
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function shows the phase parameter in result table.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR parameter/This control selects the power parameter.
/// HIPAR visibility/Shows / hides the parameter in the result table.
ViStatus _VI_FUNC rsspecan_ConfigurePulseResultTablePhase(ViSession instrSession,
                                                          ViInt32   window,
                                                          ViInt32   parameter,
                                                          ViBoolean visibility)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "Win%ld", window);
    switch (parameter)
    {
    case RSSPECAN_VAL_PULSE_RESULT_PHASE_ALL:
        viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, repCap, RSSPECAN_ATTR_PULSE_RESULT_TABLE_PHASE_ALL, visibility),
                4, "visibility");
        break;
    case RSSPECAN_VAL_PULSE_RESULT_PHASE_POIN:
        viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, repCap, RSSPECAN_ATTR_PULSE_RESULT_TABLE_PHASE_POINT, visibility),
                4, "visibility");
        break;
    case RSSPECAN_VAL_PULSE_RESULT_PHASE_PPPH:
        viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, repCap, RSSPECAN_ATTR_PULSE_RESULT_TABLE_PHASE_PULSE_PULSE, visibility),
                4, "visibility");
        break;
    case RSSPECAN_VAL_PULSE_RESULT_PHASE_RERR:
        viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, repCap, RSSPECAN_ATTR_PULSE_RESULT_TABLE_PHASE_RMS, visibility),
                4, "visibility");
        break;
    case RSSPECAN_VAL_PULSE_RESULT_PHASE_PERR:
        viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, repCap, RSSPECAN_ATTR_PULSE_RESULT_TABLE_PHASE_PEAK, visibility),
                4, "visibility");
        break;
    case RSSPECAN_VAL_PULSE_RESULT_PHASE_DEV:
        viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, repCap, RSSPECAN_ATTR_PULSE_RESULT_TABLE_PHASE_DEVIATION, visibility),
                4, "visibility");
        break;
    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, parameter), 3, "Parameter");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function shows the phase parameter in result table.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR parameter/This control selects the power parameter.
/// HIPAR visibility/Shows / hides the parameter in the result table.
ViStatus _VI_FUNC rsspecan_ConfigurePulseResultTableTimeSidelobe(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   parameter,
                            ViBoolean visibility,
                            ViInt32   reserved)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
             reserved;

    checkErr(RsCore_LockSession(instrSession));

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "Win%ld", window);
    switch (parameter)
    {
    case RSSPECAN_VAL_PULSE_RESULT_TIME_SIDELOBE_ALL:
        viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, repCap, RSSPECAN_ATTR_PULSE_RESULT_TABLE_TIME_SIDELOBE_ALL, visibility),
                4, "visibility");
        break;
    case RSSPECAN_VAL_PULSE_RESULT_TIME_SIDELOBE_PS_LEVEL:
        viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, repCap, RSSPECAN_ATTR_PULSE_RESULT_TABLE_TIME_SIDELOBE_PEAK_TO_SIDELOBE_LEVEL, visibility),
                4, "visibility");
        break;
    case RSSPECAN_VAL_PULSE_RESULT_TIME_SIDELOBE_IS_LEVEL:
        viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, repCap, RSSPECAN_ATTR_PULSE_RESULT_TABLE_TIME_SIDELOBE_INTEGRATED_SIDELOBE_LEVEL, visibility),
                4, "visibility");
        break;
    case RSSPECAN_VAL_PULSE_RESULT_TIME_SIDELOBE_MWIDTH:
        if (window == 0)
        {
            snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CALC:TABL:TSID:MWID %s", (visibility ? "ON" : "OFF"));
            checkErr(RsCore_Write(instrSession, cmd));
        }
        else
        {
            snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CALC%ld:TABL:TSID:MWID %s", window, (visibility ? "ON" : "OFF"));
            checkErr(RsCore_Write(instrSession, cmd));
        }
        checkErr(rsspecan_CheckStatus(instrSession));

        break;
    case RSSPECAN_VAL_PULSE_RESULT_TIME_SIDELOBE_SDELAY:
        if (window == 0)
        {
            snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CALC:TABL:TSID:SDEL %s", (visibility ? "ON" : "OFF"));
            checkErr(RsCore_Write(instrSession, cmd));
        }
        else
        {
            snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CALC%ld:TABL:TSID:SDEL %s", window, (visibility ? "ON" : "OFF"));
            checkErr(RsCore_Write(instrSession, cmd));
        }
        checkErr(rsspecan_CheckStatus(instrSession));
        break;
    case RSSPECAN_VAL_PULSE_RESULT_TIME_SIDELOBE_CRATIO:
        viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, repCap, RSSPECAN_ATTR_PULSE_RESULT_TABLE_TIME_SIDELOBE_COMPRESSION_RATIO, visibility),
                4, "visibility");
        break;
    case RSSPECAN_VAL_PULSE_RESULT_TIME_SIDELOBE_IM_POWER:
        viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, repCap, RSSPECAN_ATTR_PULSE_RESULT_TABLE_TIME_SIDELOBE_INTEGRATED_MAINLOBE_POWER, visibility),
                4, "visibility");
        break;
    case RSSPECAN_VAL_PULSE_RESULT_TIME_SIDELOBE_AM_POWER:
        viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, repCap, RSSPECAN_ATTR_PULSE_RESULT_TABLE_TIME_SIDELOBE_AVERAGE_MAINLOBE_POWER, visibility),
                4, "visibility");
        break;
    case RSSPECAN_VAL_PULSE_RESULT_TIME_SIDELOBE_PCOR:
        viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, repCap, RSSPECAN_ATTR_PULSE_RESULT_TABLE_TIME_SIDELOBE_PEAK_CORRELATION, visibility),
                4, "visibility");
        break;
    case RSSPECAN_VAL_PULSE_RESULT_TIME_SIDELOBE_MPHASE:
        viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, repCap, RSSPECAN_ATTR_PULSE_RESULT_TABLE_TIME_SIDELOBE_MAINLOBE_PHASE, visibility),
                4, "visibility");
        break;
    case RSSPECAN_VAL_PULSE_RESULT_TIME_SIDELOBE_MFREQUENCY:
        if (window == 0)
        {
            snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CALC:TABL:TSID:MFR %s", (visibility ? "ON" : "OFF"));
            checkErr(RsCore_Write(instrSession, cmd));
        }
        else
        {
            snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CALC%ld:TABL:TSID:MFR %s", window, (visibility ? "ON" : "OFF"));
            checkErr(RsCore_Write(instrSession, cmd));
        }
        checkErr(rsspecan_CheckStatus(instrSession));
        break;
    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, parameter), 3, "Parameter");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function shows the frequency parameter in result table and sets
/// HIFN  it's scaling.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR parameter/This control selects the envelope model parameter.
/// HIPAR visibility/Shows / hides the parameter in the result table.
ViStatus _VI_FUNC rsspecan_ConfigurePulseResultTableEnvelopeModel(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   parameter,
                            ViBoolean visibility)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViAttr   attr[] = {
             RSSPECAN_ATTR_PULSE_RESULT_TABLE_ENVELOPE_MODEL_ALL,
             RSSPECAN_ATTR_PULSE_RESULT_TABLE_ENVELOPE_MODEL_RISE_BASE_POINT_TIME,
             RSSPECAN_ATTR_PULSE_RESULT_TABLE_ENVELOPE_MODEL_RISE_LOW_POINT_TIME,
             RSSPECAN_ATTR_PULSE_RESULT_TABLE_ENVELOPE_MODEL_RISE_MID_POINT_TIME,
             RSSPECAN_ATTR_PULSE_RESULT_TABLE_ENVELOPE_MODEL_RISE_HIGH_POINT_TIME,
             RSSPECAN_ATTR_PULSE_RESULT_TABLE_ENVELOPE_MODEL_RISE_TOP_POINT_TIME,
             RSSPECAN_ATTR_PULSE_RESULT_TABLE_ENVELOPE_MODEL_RISE_LOW_POINT_LEVEL,
             RSSPECAN_ATTR_PULSE_RESULT_TABLE_ENVELOPE_MODEL_RISE_MID_POINT_LEVEL,
             RSSPECAN_ATTR_PULSE_RESULT_TABLE_ENVELOPE_MODEL_RISE_HIGH_POINT_LEVEL,
             RSSPECAN_ATTR_PULSE_RESULT_TABLE_ENVELOPE_MODEL_RISE_TOP_POINT_LEVEL,
             RSSPECAN_ATTR_PULSE_RESULT_TABLE_ENVELOPE_MODEL_FALL_BASE_POINT_TIME,
             RSSPECAN_ATTR_PULSE_RESULT_TABLE_ENVELOPE_MODEL_FALL_LOW_POINT_TIME,
             RSSPECAN_ATTR_PULSE_RESULT_TABLE_ENVELOPE_MODEL_FALL_MID_POINT_TIME,
             RSSPECAN_ATTR_PULSE_RESULT_TABLE_ENVELOPE_MODEL_FALL_HIGH_POINT_TIME,
             RSSPECAN_ATTR_PULSE_RESULT_TABLE_ENVELOPE_MODEL_FALL_TOP_POINT_TIME,
             RSSPECAN_ATTR_PULSE_RESULT_TABLE_ENVELOPE_MODEL_FALL_LOW_POINT_LEVEL,
             RSSPECAN_ATTR_PULSE_RESULT_TABLE_ENVELOPE_MODEL_FALL_MID_POINT_LEVEL,
             RSSPECAN_ATTR_PULSE_RESULT_TABLE_ENVELOPE_MODEL_FALL_HIGH_POINT_LEVEL,
             RSSPECAN_ATTR_PULSE_RESULT_TABLE_ENVELOPE_MODEL_FALL_TOP_POINT_LEVEL
             };

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, window, 0, 16),
            2, "Window");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, parameter, RSSPECAN_VAL_PULSE_RESULT_ENVELOPE_MODEL_ALL, RSSPECAN_VAL_PULSE_RESULT_ENVELOPE_MODEL_FALL_TOP_POINT_LEVEL),
            3, "Parameter");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "Win%ld", window);

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, repCap, attr[parameter], visibility),
            4, "Visibility");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function shows the stability parameter in result table and sets
/// HIFN  it's scaling.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR parameter/This control selects the stability parameter
/// HIPAR visibility/Shows / hides the parameter in the result table.
ViStatus _VI_FUNC rsspecan_ConfigurePulseResultTableStability(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   parameter,
                            ViBoolean visibility)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  windowMax = 0;
    ViAttr   attributes[] = {
             RSSPECAN_ATTR_PULSE_RESULT_TABLE_STABILITY_ALL,
             RSSPECAN_ATTR_PULSE_RESULT_TABLE_STABILITY_AMPLITUDE,
             RSSPECAN_ATTR_PULSE_RESULT_TABLE_STABILITY_BURST_NUMBER,
             RSSPECAN_ATTR_PULSE_RESULT_TABLE_STABILITY_PHASE,
             RSSPECAN_ATTR_PULSE_RESULT_TABLE_STABILITY_POSITION_NUMBER_IN_BURST,
             RSSPECAN_ATTR_PULSE_RESULT_TABLE_STABILITY_TOTAL
             };

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetRepCapIdItemsCount(instrSession, "Window", &windowMax));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, window, 0, windowMax - 1),
        2, "Window");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_PULSE_RESULT_TABLE_STABILITY_ALL,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap),
            2, "Window");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, parameter, RSSPECAN_VAL_PULSE_STABILITY_AMPLITUDE, RSSPECAN_VAL_PULSE_STABILITY_ALL),
            3, "Parameter");

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, repCap, attributes[parameter], visibility),
            4, "Visibility");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function exports result table data from the specified window to
/// HIFN  an ASCii file (.DAT).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR exportColumns/Selects the columns to be stored in file.
/// HIPAR filename/This parameter defines the path and file name of the target file.
ViStatus _VI_FUNC rsspecan_ExportPulseResultTable(ViSession instrSession,
                                                  ViInt32   window,
                                                  ViInt32   exportCollumns,
                                                  ViString  filename)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViChar*  p2string;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K6"));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, window, 0, 16),
            2, "Window");
    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &rsspecan_rngPulseColumns, exportCollumns, &p2string),
                3, "Export Collumns");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "MMEM:STOR%ld:TABL %s,'%s'", window, p2string, filename);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function activates or deactivates a limit check for all
/// HIFN  parameters in all parameter groups.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR state/This control sets the limit checking state.
ViStatus _VI_FUNC rsspecan_ConfigurePulseResultTableLimitCheckAll(
                            ViSession instrSession,
                            ViInt32   window,
                            ViBoolean state)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_PULSE_RESULT_TABLE_LIMIT_ALL_ENABLED,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap),
            2, "Window");

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, repCap, RSSPECAN_ATTR_PULSE_RESULT_TABLE_LIMIT_ALL_ENABLED, state),
            3, "State");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function activates or deactivates a limit check for all
/// HIFN  parameters in power parameter group.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR state/This control sets the limit checking state.
ViStatus _VI_FUNC rsspecan_ConfigurePulseResultTablePowerLimitCheckAll(
                            ViSession instrSession,
                            ViInt32   window,
                            ViBoolean state)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_PULSE_RESULT_TABLE_POWER_LIMIT_ALL_ENABLED,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap),
            2, "Window");

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, repCap, RSSPECAN_ATTR_PULSE_RESULT_TABLE_POWER_LIMIT_ALL_ENABLED, state),
            3, "State");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function activates or deactivates a limit check for all
/// HIFN  parameters in timing parameter group.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR state/This control sets the limit checking state.
ViStatus _VI_FUNC rsspecan_ConfigurePulseResultTableTimingLimitCheckAll(
                            ViSession instrSession,
                            ViInt32   window,
                            ViBoolean state)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_PULSE_RESULT_TABLE_TIMING_LIMIT_ALL_ENABLED,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap),
            2, "Window");

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, repCap, RSSPECAN_ATTR_PULSE_RESULT_TABLE_TIMING_LIMIT_ALL_ENABLED, state),
            3, "State");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function activates or deactivates a limit check for all
/// HIFN  parameters in frequency parameter group.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR state/This control sets the limit checking state.
ViStatus _VI_FUNC rsspecan_ConfigurePulseResultTableFrequencyLimitCheckAll(
                            ViSession instrSession,
                            ViInt32   window,
                            ViBoolean state)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_PULSE_RESULT_TABLE_FREQUENCY_LIMIT_ALL_ENABLED,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap),
            2, "Window");

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, repCap, RSSPECAN_ATTR_PULSE_RESULT_TABLE_FREQUENCY_LIMIT_ALL_ENABLED, state),
            3, "State");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function activates or deactivates a limit check for all
/// HIFN  parameters in phase parameter group.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR state/This control sets the limit checking state.
ViStatus _VI_FUNC rsspecan_ConfigurePulseResultTablePhaseLimitCheckAll(
                            ViSession instrSession,
                            ViInt32   window,
                            ViBoolean state)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_PULSE_RESULT_TABLE_PHASE_LIMIT_ALL_ENABLED,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap),
            2, "Window");

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, repCap, RSSPECAN_ATTR_PULSE_RESULT_TABLE_PHASE_LIMIT_ALL_ENABLED, state),
            3, "State");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function activates or deactivates a limit check for all
/// HIFN  parameters in time sidelobe parameter group.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR state/This control sets the limit checking state.
ViStatus _VI_FUNC rsspecan_ConfigurePulseResultTableTimeSidelobeLimitCheckAll(
                            ViSession instrSession,
                            ViInt32   window,
                            ViBoolean state)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_PULSE_RESULT_TABLE_TIME_SIDELOBE_LIMIT_ALL_ENABLED,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap),
            2, "Window");

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, repCap, RSSPECAN_ATTR_PULSE_RESULT_TABLE_TIME_SIDELOBE_LIMIT_ALL_ENABLED, state),
            3, "State");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function activates or deactivates a limit check for all
/// HIFN  parameters in envelope model parameter group.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR state/This control sets the limit checking state.
ViStatus _VI_FUNC rsspecan_ConfigurePulseResultTableEnvelopeModelLimitCheckAll(
                            ViSession instrSession,
                            ViInt32   window,
                            ViBoolean state)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_PULSE_RESULT_TABLE_ENVELOPE_MODEL_LIMIT_ALL_ENABLED,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap),
            2, "Window");

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, repCap, RSSPECAN_ATTR_PULSE_RESULT_TABLE_ENVELOPE_MODEL_LIMIT_ALL_ENABLED, state),
            3, "State");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Activates or deactivates a limit check for all parameters in the
/// HIFN  selected parameter group.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR state/Activates or deactivates a limit check for all parameters in the
/// HIPAR state/selected parameter group.
ViStatus _VI_FUNC rsspecan_ConfigurePulseResultTableStabilityLimitCheckAll(
                            ViSession instrSession,
                            ViInt32   window,
                            ViBoolean state)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repcap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  repcapCount = 0;

    checkErr(RsCore_LockSession(instrSession));

    (void)RsCore_GetRepCapIdItemsCount(instrSession, "Window", &repcapCount);

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, window, 0, repcapCount - 1),
        2, "Window");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_PULSE_RESULT_TABLE_STABILITY_ALL_LIMIT_ENABLED, "Window", window, RS_REPCAP_BUF_SIZE, repcap),
        2, "Window");

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, repcap, RSSPECAN_ATTR_PULSE_RESULT_TABLE_STABILITY_ALL_LIMIT_ENABLED, state),
            3, "State");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN This function configures the limits in power parameter group.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR state/This control sets the limit checking state.
/// HIPAR lowerLimit/This control sets the lower limit of the valid value range.
/// HIPAR upperLimit/This control sets the upper limit of the valid value range.
ViStatus _VI_FUNC rsspecan_ConfigurePulseResultTablePowerLimit(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   parameter,
                            ViBoolean state,
                            ViReal64  lowerLimit,
                            ViReal64  upperLimit)
{
    ViStatus   error = VI_SUCCESS;
    ViChar     cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViChar     repCap[RS_REPCAP_BUF_SIZE] = "";
    ViString   power[] = {
    "",        "TOP", "BASE", "AMPL", "ON", "AVG", "MIN", "MAX", "PON", "PAVG", "PMIN", "ADR:PERC", "ADR:DB", "RIPP:PERC",
    "RIPP:DB", "OVER:PERC", "OVER:DB", "POIN", "PPR", "AMPL:I", "AMPL:Q"
               };

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_PULSE_RESULT_TABLE_POWER_LIMIT_TOP_ENABLED,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap),
            2, "Window");
    switch (parameter)
    {
    case RSSPECAN_VAL_PULSE_RESULT_POWER_TOP:
        viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, repCap, RSSPECAN_ATTR_PULSE_RESULT_TABLE_POWER_LIMIT_TOP_ENABLED, state),
                4, "State");
        break;
    case RSSPECAN_VAL_PULSE_RESULT_POWER_BASE:
        viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, repCap, RSSPECAN_ATTR_PULSE_RESULT_TABLE_POWER_LIMIT_BASE_ENABLED, state),
                4, "State");
        break;
    case RSSPECAN_VAL_PULSE_RESULT_POWER_AMPL:
        viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, repCap, RSSPECAN_ATTR_PULSE_RESULT_TABLE_POWER_LIMIT_AMPLITUDE_ENABLED, state),
                4, "State");
        break;
    case RSSPECAN_VAL_PULSE_RESULT_POWER_ON:
        viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, repCap, RSSPECAN_ATTR_PULSE_RESULT_TABLE_POWER_LIMIT_ON_ENABLED, state),
                4, "State");
        break;
    case RSSPECAN_VAL_PULSE_RESULT_POWER_AVG:
        viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, repCap, RSSPECAN_ATTR_PULSE_RESULT_TABLE_POWER_LIMIT_AVERAGE_ENABLED, state),
                4, "State");
        break;
    case RSSPECAN_VAL_PULSE_RESULT_POWER_MIN:
        viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, repCap, RSSPECAN_ATTR_PULSE_RESULT_TABLE_POWER_LIMIT_MIN_ENABLED, state),
                4, "State");
        break;
    case RSSPECAN_VAL_PULSE_RESULT_POWER_MAX:
        viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, repCap, RSSPECAN_ATTR_PULSE_RESULT_TABLE_POWER_LIMIT_MAX_ENABLED, state),
                4, "State");
        break;
    case RSSPECAN_VAL_PULSE_RESULT_POWER_PON:
        viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, repCap, RSSPECAN_ATTR_PULSE_RESULT_TABLE_POWER_LIMIT_POWER_ON_ENABLED, state),
                4, "State");
        break;
    case RSSPECAN_VAL_PULSE_RESULT_POWER_PAVG:
        viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, repCap, RSSPECAN_ATTR_PULSE_RESULT_TABLE_POWER_LIMIT_POWER_AVERAGE_ENABLED, state),
                4, "State");
        break;
    case RSSPECAN_VAL_PULSE_RESULT_POWER_PMIN:
        viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, repCap, RSSPECAN_ATTR_PULSE_RESULT_TABLE_POWER_LIMIT_POWER_MIN_ENABLED, state),
                4, "State");
        break;
    case RSSPECAN_VAL_PULSE_RESULT_POWER_ADP:
        viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, repCap, RSSPECAN_ATTR_PULSE_RESULT_TABLE_POWER_LIMIT_ADR_ENABLED, state),
                4, "State");
        break;
    case RSSPECAN_VAL_PULSE_RESULT_POWER_ADDB:
        viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, repCap, RSSPECAN_ATTR_PULSE_RESULT_TABLE_POWER_LIMIT_ADDB_ENABLED, state),
                4, "State");
        break;
    case RSSPECAN_VAL_PULSE_RESULT_POWER_RPER:
        viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, repCap, RSSPECAN_ATTR_PULSE_RESULT_TABLE_POWER_LIMIT_RPER_ENABLED, state),
                4, "State");
        break;
    case RSSPECAN_VAL_PULSE_RESULT_POWER_RDB:
        viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, repCap, RSSPECAN_ATTR_PULSE_RESULT_TABLE_POWER_LIMIT_RPDB_ENABLED, state),
                4, "State");
        break;
    case RSSPECAN_VAL_PULSE_RESULT_POWER_OPER:
        viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, repCap, RSSPECAN_ATTR_PULSE_RESULT_TABLE_POWER_LIMIT_OVER_ENABLED, state),
                4, "State");
        break;
    case RSSPECAN_VAL_PULSE_RESULT_POWER_ODB:
        viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, repCap, RSSPECAN_ATTR_PULSE_RESULT_TABLE_POWER_LIMIT_OVERDB_ENABLED, state),
                4, "State");
        break;
    case RSSPECAN_VAL_PULSE_RESULT_POWER_POIN:
        viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, repCap, RSSPECAN_ATTR_PULSE_RESULT_TABLE_POWER_LIMIT_POINT_ENABLED, state),
                4, "State");
        break;
    case RSSPECAN_VAL_PULSE_RESULT_POWER_PPR:
        viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, repCap, RSSPECAN_ATTR_PULSE_RESULT_TABLE_POWER_LIMIT_PPR_ENABLED, state),
                4, "State");
        break;
    case RSSPECAN_VAL_PULSE_RESULT_POWER_AMPL_I:
        viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, repCap, RSSPECAN_ATTR_PULSE_RESULT_TABLE_POWER_LIMIT_AMPLITUDE_I_ENABLED, state),
                4, "State");
        break;
    case RSSPECAN_VAL_PULSE_RESULT_POWER_AMPL_Q:
        viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, repCap, RSSPECAN_ATTR_PULSE_RESULT_TABLE_POWER_LIMIT_AMPLITUDE_Q_ENABLED, state),
                4, "State");
        break;
    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, parameter),
                    3, "Parameter");
    }
    if (state == VI_TRUE)
    {
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CALC%ld:TABL:POW:%s:LIM %.12lG,%.12lG", window, power[parameter],
                 lowerLimit, upperLimit);
        checkErr(RsCore_Write(instrSession, cmd));
        checkErr(rsspecan_CheckStatus(instrSession));
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function configures the limits in timing parameter group.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR state/This control sets the limit checking state.
/// HIPAR lowerLimit/This control sets the lower limit of the valid value range.
/// HIPAR upperLimit/This control sets the upper limit of the valid value range.
ViStatus _VI_FUNC rsspecan_ConfigurePulseResultTableTimingLimit(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   parameter,
                            ViBoolean state,
                            ViReal64  lowerLimit,
                            ViReal64  upperLimit)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar*  p2string;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_PULSE_RESULT_TABLE_TIMING_LIMIT_TIME_STAMP_ENABLED,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap),
            2, "Window");
    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &rsspecan_rngPulseParameterTiming, parameter, &p2string),
                3, "Parameter");

    switch (parameter)
    {
    case RSSPECAN_VAL_PULSE_RESULT_TIMING_TST:
        viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, repCap, RSSPECAN_ATTR_PULSE_RESULT_TABLE_TIMING_LIMIT_TIME_STAMP_ENABLED, state),
                4, "State");
        break;
    case RSSPECAN_VAL_PULSE_RESULT_TIMING_SETT:
        viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, repCap, RSSPECAN_ATTR_PULSE_RESULT_TABLE_TIMING_LIMIT_SETTLING_TIME_ENABLED, state),
                4, "State");
        break;
    case RSSPECAN_VAL_PULSE_RESULT_TIMING_RISE:
        viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, repCap, RSSPECAN_ATTR_PULSE_RESULT_TABLE_TIMING_LIMIT_RISE_ENABLED, state),
                4, "State");
        break;
    case RSSPECAN_VAL_PULSE_RESULT_TIMING_FALL:
        viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, repCap, RSSPECAN_ATTR_PULSE_RESULT_TABLE_TIMING_LIMIT_FALL_ENABLED, state),
                4, "State");
        break;
    case RSSPECAN_VAL_PULSE_RESULT_TIMING_PWID:
        viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, repCap, RSSPECAN_ATTR_PULSE_RESULT_TABLE_TIMING_LIMIT_PULSE_WIDTH_ENABLED, state),
                4, "State");
        break;
    case RSSPECAN_VAL_PULSE_RESULT_TIMING_OFF:
        viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, repCap, RSSPECAN_ATTR_PULSE_RESULT_TABLE_TIMING_LIMIT_OFF_ENABLED, state),
                4, "State");
        break;
    case RSSPECAN_VAL_PULSE_RESULT_TIMING_DRAT:
        viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, repCap, RSSPECAN_ATTR_PULSE_RESULT_TABLE_TIMING_LIMIT_DUTY_RATIO_ENABLED, state),
                4, "State");
        break;
    case RSSPECAN_VAL_PULSE_RESULT_TIMING_DCYC:
        viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, repCap, RSSPECAN_ATTR_PULSE_RESULT_TABLE_TIMING_LIMIT_DUTY_CYCLE_ENABLED, state),
                4, "State");
        break;
    case RSSPECAN_VAL_PULSE_RESULT_TIMING_PRI:
        viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, repCap, RSSPECAN_ATTR_PULSE_RESULT_TABLE_TIMING_LIMIT_PRI_ENABLED, state),
                4, "State");
        break;
    case RSSPECAN_VAL_PULSE_RESULT_TIMING_PRF:
        viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, repCap, RSSPECAN_ATTR_PULSE_RESULT_TABLE_TIMING_LIMIT_PRF_ENABLED, state),
                4, "State");
        break;
    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, parameter), 3, "Parameter");
    }
    if (state == VI_TRUE)
    {
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CALC%ld:TABL:TIM:%s:LIM %.12lG,%.12lG", window, p2string, lowerLimit,
                 upperLimit);
        checkErr(RsCore_Write(instrSession, cmd));
        checkErr(rsspecan_CheckStatus(instrSession));
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function configures the limits in frequency parameter group.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR state/This control sets the limit checking state.
/// HIPAR lowerLimit/This control sets the lower limit of the valid value range.
/// HIPAR upperLimit/This control sets the upper limit of the valid value range.
ViStatus _VI_FUNC rsspecan_ConfigurePulseResultTableFrequencyLimit(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   parameter,
                            ViBoolean state,
                            ViReal64  lowerLimit,
                            ViReal64  upperLimit)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar*  p2TableString;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_PULSE_RESULT_TABLE_TIMING_LIMIT_TIME_STAMP_ENABLED,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap),
            2, "Window");
    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &rsspecan_rngPulseParameterFrequency, parameter, &p2TableString),
        3, "Parameter");

    switch (parameter)
    {
    case RSSPECAN_VAL_PULSE_RESULT_FREQ_POIN:
        viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, repCap, RSSPECAN_ATTR_PULSE_RESULT_TABLE_FREQUENCY_LIMIT_POINT_ENABLED, state),
                4, "State");
        break;
    case RSSPECAN_VAL_PULSE_RESULT_FREQ_PPFR:
        viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, repCap, RSSPECAN_ATTR_PULSE_RESULT_TABLE_FREQUENCY_LIMIT_PPFR_ENABLED, state),
                4, "State");
        break;
    case RSSPECAN_VAL_PULSE_RESULT_FREQ_RERR:
        viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, repCap, RSSPECAN_ATTR_PULSE_RESULT_TABLE_FREQUENCY_LIMIT_ERROR_RMS_ENABLED, state),
                4, "State");
        break;
    case RSSPECAN_VAL_PULSE_RESULT_FREQ_PERR:
        viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, repCap, RSSPECAN_ATTR_PULSE_RESULT_TABLE_FREQUENCY_LIMIT_ERROR_PEAK_ENABLED, state),
                4, "State");
        break;
    case RSSPECAN_VAL_PULSE_RESULT_FREQ_DEV:
        viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, repCap, RSSPECAN_ATTR_PULSE_RESULT_TABLE_FREQUENCY_LIMIT_DEVIATION_ENABLED, state),
                4, "State");
        break;
    case RSSPECAN_VAL_PULSE_RESULT_FREQ_CRAT:
        viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, repCap, RSSPECAN_ATTR_PULSE_RESULT_TABLE_FREQUENCY_LIMIT_CHIP_RATE_ENABLED, state),
                4, "State");
        break;
    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, parameter), 3, "Parameter");
    }
    if (state == VI_TRUE)
    {
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CALC%ld:TABL:FREQ:%s:LIM %.12lG,%.12lG", window, p2TableString,
                 lowerLimit, upperLimit);
        checkErr(RsCore_Write(instrSession, cmd));

        checkErr(rsspecan_CheckStatus(instrSession));
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN This function configures the limits in phase parameter group.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR state/This control sets the limit checking state.
/// HIPAR lowerLimit/This control sets the lower limit of the valid value range.
/// HIPAR upperLimit/This control sets the upper limit of the valid value range.
ViStatus _VI_FUNC rsspecan_ConfigurePulseResultTablePhaseLimit(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   parameter,
                            ViBoolean state,
                            ViReal64  lowerLimit,
                            ViReal64  upperLimit)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar*  p2string;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_PULSE_RESULT_TABLE_TIMING_LIMIT_TIME_STAMP_ENABLED,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap),
            2, "Window");
    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &rsspecan_rngPulseParameterPhase, parameter, &p2string),
                3, "Parameter");

    switch (parameter)
    {
    case RSSPECAN_VAL_PULSE_RESULT_PHASE_POIN:
        viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, repCap, RSSPECAN_ATTR_PULSE_RESULT_TABLE_PHASE_LIMIT_POINT_ENABLED, state),
                4, "State");
        break;
    case RSSPECAN_VAL_PULSE_RESULT_PHASE_PPPH:
        viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, repCap, RSSPECAN_ATTR_PULSE_RESULT_TABLE_PHASE_LIMIT_PPFR_ENABLED, state),
                4, "State");
        break;
    case RSSPECAN_VAL_PULSE_RESULT_PHASE_RERR:
        viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, repCap, RSSPECAN_ATTR_PULSE_RESULT_TABLE_PHASE_LIMIT_ERROR_RMS_ENABLED, state),
                4, "State");
        break;
    case RSSPECAN_VAL_PULSE_RESULT_PHASE_PERR:
        viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, repCap, RSSPECAN_ATTR_PULSE_RESULT_TABLE_PHASE_LIMIT_ERROR_PEAK_ENABLED, state),
                4, "State");
        break;
    case RSSPECAN_VAL_PULSE_RESULT_PHASE_DEV:
        viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, repCap, RSSPECAN_ATTR_PULSE_RESULT_TABLE_PHASE_LIMIT_DEVIATION_ENABLED, state),
                4, "State");
        break;
    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, parameter), 3, "Parameter");
    }
    if (state == VI_TRUE)
    {
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CALC%ld:TABL:PHAS:%s:LIM %.12lG,%.12lG", window, p2string, lowerLimit,
                 upperLimit);
        checkErr(RsCore_Write(instrSession, cmd));
        checkErr(rsspecan_CheckStatus(instrSession));
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN This function configures the limits in time sidelobe parameter group.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR state/This control sets the limit checking state.
/// HIPAR lowerLimit/This control sets the lower limit of the valid value range.
/// HIPAR upperLimit/This control sets the upper limit of the valid value range.
ViStatus _VI_FUNC rsspecan_ConfigurePulseResultTableTimeSidelobeLimit(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   parameter,
                            ViBoolean state,
                            ViReal64  lowerLimit,
                            ViReal64  upperLimit)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar*  p2string;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_PULSE_RESULT_TABLE_TIME_SIDELOBE_LIMIT_PEAK_TO_SIDELOBE_LEVEL_ENABLED,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap),
            2, "Window");
    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &RsSpecAn_rngPulseParameterTimeSidelobe, parameter, &p2string),
        3, "Parameter");

    switch (parameter)
    {
    case RSSPECAN_VAL_PULSE_RESULT_TIME_SIDELOBE_PS_LEVEL:
        viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, repCap, RSSPECAN_ATTR_PULSE_RESULT_TABLE_TIME_SIDELOBE_LIMIT_PEAK_TO_SIDELOBE_LEVEL_ENABLED, state),
                4, "State");
        break;
    case RSSPECAN_VAL_PULSE_RESULT_TIME_SIDELOBE_IS_LEVEL:
        viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, repCap, RSSPECAN_ATTR_PULSE_RESULT_TABLE_TIME_SIDELOBE_LIMIT_INTEGRATED_SIDELOBE_LEVEL_ENABLED, state),
                4, "State");
        break;
    case RSSPECAN_VAL_PULSE_RESULT_TIME_SIDELOBE_MWIDTH:
        viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, repCap, RSSPECAN_ATTR_PULSE_RESULT_TABLE_TIME_SIDELOBE_LIMIT_MAINLOBE_3DB_WIDTH_ENABLED, state),
                4, "State");
        break;
    case RSSPECAN_VAL_PULSE_RESULT_TIME_SIDELOBE_SDELAY:
        viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, repCap, RSSPECAN_ATTR_PULSE_RESULT_TABLE_TIME_SIDELOBE_LIMIT_SIDELOBE_DELAY_ENABLED, state),
                4, "State");
        break;
    case RSSPECAN_VAL_PULSE_RESULT_TIME_SIDELOBE_CRATIO:
        viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, repCap, RSSPECAN_ATTR_PULSE_RESULT_TABLE_TIME_SIDELOBE_LIMIT_COMPRESSION_RATIO_ENABLED, state),
                4, "State");
        break;
    case RSSPECAN_VAL_PULSE_RESULT_TIME_SIDELOBE_IM_POWER:
        viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, repCap, RSSPECAN_ATTR_PULSE_RESULT_TABLE_TIME_SIDELOBE_LIMIT_INTEGRATED_MAINLOBE_POWER_ENABLED, state),
                4, "State");
        break;
    case RSSPECAN_VAL_PULSE_RESULT_TIME_SIDELOBE_AM_POWER:
        viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, repCap, RSSPECAN_ATTR_PULSE_RESULT_TABLE_TIME_SIDELOBE_LIMIT_AVERAGE_MAINLOBE_POWER_ENABLED, state),
                4, "State");
        break;
    case RSSPECAN_VAL_PULSE_RESULT_TIME_SIDELOBE_PCOR:
        viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, repCap, RSSPECAN_ATTR_PULSE_RESULT_TABLE_TIME_SIDELOBE_LIMIT_PEAK_CORRELATION_ENABLED, state),
                4, "State");
        break;
    case RSSPECAN_VAL_PULSE_RESULT_TIME_SIDELOBE_MPHASE:
        viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, repCap, RSSPECAN_ATTR_PULSE_RESULT_TABLE_TIME_SIDELOBE_LIMIT_MAINLOBE_PHASE_ENABLED, state),
                4, "State");
        break;
    case RSSPECAN_VAL_PULSE_RESULT_TIME_SIDELOBE_MFREQUENCY:
        viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, repCap, RSSPECAN_ATTR_PULSE_RESULT_TABLE_TIME_SIDELOBE_LIMIT_MAINLOBE_FREQUENCY_ENABLED, state),
                4, "State");
        break;
    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, parameter), 3, "Parameter");
    }
    if (state == VI_TRUE)
    {
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CALC%ld:TABL:TSID:%s:LIM %.12lG,%.12lG", window, p2string, lowerLimit,
                 upperLimit);
        checkErr(RsCore_Write(instrSession, cmd));
        checkErr(rsspecan_CheckStatus(instrSession));
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the limits in envelope model parameter group.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR parameter/This control selects the envelope model parameter.
/// HIPAR state/This control sets the selected parameter limit state.
/// HIPAR lowerLimit/This control sets the lower limit of the valid value range.
/// HIPAR upperLimit/This control sets the upper limit of the valid value range.
ViStatus _VI_FUNC rsspecan_ConfigurePulseResultTableEnvelopeModelLimit(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   parameter,
                            ViBoolean state,
                            ViReal64  lowerLimit,
                            ViReal64  upperLimit)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar*  p2string;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_PULSE_RESULT_TABLE_ENVELOPE_MODEL_FALL_BASE_POINT_TIME_LIMIT_ENABLED,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap),
            2, "Window");
    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &RsSpecAn_rngPulseParameterEnvelopeModel, parameter, &p2string),
        3, "Parameter");

    switch (parameter)
    {
    case RSSPECAN_VAL_PULSE_RESULT_ENVELOPE_MODEL_RISE_BASE_POINT_TIME:
        viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, repCap, RSSPECAN_ATTR_PULSE_RESULT_TABLE_ENVELOPE_MODEL_RISE_BASE_POINT_TIME_LIMIT_ENABLED, state),
                4, "State");
        break;
    case RSSPECAN_VAL_PULSE_RESULT_ENVELOPE_MODEL_RISE_LOW_POINT_TIME:
        viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, repCap, RSSPECAN_ATTR_PULSE_RESULT_TABLE_ENVELOPE_MODEL_RISE_LOW_POINT_TIME_LIMIT_ENABLED, state),
                4, "State");
        break;
    case RSSPECAN_VAL_PULSE_RESULT_ENVELOPE_MODEL_RISE_MID_POINT_TIME:
        viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, repCap, RSSPECAN_ATTR_PULSE_RESULT_TABLE_ENVELOPE_MODEL_RISE_MID_POINT_TIME_LIMIT_ENABLED, state),
                4, "State");
        break;
    case RSSPECAN_VAL_PULSE_RESULT_ENVELOPE_MODEL_RISE_HIGH_POINT_TIME:
        viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, repCap, RSSPECAN_ATTR_PULSE_RESULT_TABLE_ENVELOPE_MODEL_RISE_HIGH_POINT_TIME_LIMIT_ENABLED, state),
                4, "State");
        break;
    case RSSPECAN_VAL_PULSE_RESULT_ENVELOPE_MODEL_RISE_TOP_POINT_TIME:
        viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, repCap, RSSPECAN_ATTR_PULSE_RESULT_TABLE_ENVELOPE_MODEL_RISE_TOP_POINT_TIME_LIMIT_ENABLED, state),
                4, "State");
        break;

    case RSSPECAN_VAL_PULSE_RESULT_ENVELOPE_MODEL_RISE_LOW_POINT_LEVEL:
        viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, repCap, RSSPECAN_ATTR_PULSE_RESULT_TABLE_ENVELOPE_MODEL_RISE_LOW_POINT_LEVEL_LIMIT_ENABLED, state),
                4, "State");
        break;
    case RSSPECAN_VAL_PULSE_RESULT_ENVELOPE_MODEL_RISE_MID_POINT_LEVEL:
        viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, repCap, RSSPECAN_ATTR_PULSE_RESULT_TABLE_ENVELOPE_MODEL_RISE_MID_POINT_LEVEL_LIMIT_ENABLED, state),
                4, "State");
        break;
    case RSSPECAN_VAL_PULSE_RESULT_ENVELOPE_MODEL_RISE_HIGH_POINT_LEVEL:
        viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, repCap, RSSPECAN_ATTR_PULSE_RESULT_TABLE_ENVELOPE_MODEL_RISE_HIGH_POINT_LEVEL_LIMIT_ENABLED, state),
                4, "State");
        break;
    case RSSPECAN_VAL_PULSE_RESULT_ENVELOPE_MODEL_RISE_TOP_POINT_LEVEL:
        viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, repCap, RSSPECAN_ATTR_PULSE_RESULT_TABLE_ENVELOPE_MODEL_RISE_TOP_POINT_LEVEL_LIMIT_ENABLED, state),
                4, "State");
        break;
    case RSSPECAN_VAL_PULSE_RESULT_ENVELOPE_MODEL_FALL_BASE_POINT_TIME:
        viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, repCap, RSSPECAN_ATTR_PULSE_RESULT_TABLE_ENVELOPE_MODEL_FALL_BASE_POINT_TIME_LIMIT_ENABLED, state),
                4, "State");
        break;
    case RSSPECAN_VAL_PULSE_RESULT_ENVELOPE_MODEL_FALL_LOW_POINT_TIME:
        viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, repCap, RSSPECAN_ATTR_PULSE_RESULT_TABLE_ENVELOPE_MODEL_FALL_LOW_POINT_TIME_LIMIT_ENABLED, state),
                4, "State");
        break;
    case RSSPECAN_VAL_PULSE_RESULT_ENVELOPE_MODEL_FALL_MID_POINT_TIME:
        viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, repCap, RSSPECAN_ATTR_PULSE_RESULT_TABLE_ENVELOPE_MODEL_FALL_MID_POINT_TIME_LIMIT_ENABLED, state),
                4, "State");
        break;
    case RSSPECAN_VAL_PULSE_RESULT_ENVELOPE_MODEL_FALL_HIGH_POINT_TIME:
        viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, repCap, RSSPECAN_ATTR_PULSE_RESULT_TABLE_ENVELOPE_MODEL_FALL_HIGH_POINT_TIME_LIMIT_ENABLED, state),
                4, "State");
        break;
    case RSSPECAN_VAL_PULSE_RESULT_ENVELOPE_MODEL_FALL_TOP_POINT_TIME:
        viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, repCap, RSSPECAN_ATTR_PULSE_RESULT_TABLE_ENVELOPE_MODEL_FALL_TOP_POINT_TIME_LIMIT_ENABLED, state),
                4, "State");
        break;
    case RSSPECAN_VAL_PULSE_RESULT_ENVELOPE_MODEL_FALL_LOW_POINT_LEVEL:
        viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, repCap, RSSPECAN_ATTR_PULSE_RESULT_TABLE_ENVELOPE_MODEL_FALL_LOW_POINT_LEVEL_LIMIT_ENABLED, state),
                4, "State");
        break;
    case RSSPECAN_VAL_PULSE_RESULT_ENVELOPE_MODEL_FALL_MID_POINT_LEVEL:
        viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, repCap, RSSPECAN_ATTR_PULSE_RESULT_TABLE_ENVELOPE_MODEL_FALL_MID_POINT_LEVEL_LIMIT_ENABLED, state),
                4, "State");
        break;
    case RSSPECAN_VAL_PULSE_RESULT_ENVELOPE_MODEL_FALL_HIGH_POINT_LEVEL:
        viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, repCap, RSSPECAN_ATTR_PULSE_RESULT_TABLE_ENVELOPE_MODEL_FALL_HIGH_POINT_LEVEL_LIMIT_ENABLED, state),
                4, "State");
        break;
    case RSSPECAN_VAL_PULSE_RESULT_ENVELOPE_MODEL_FALL_TOP_POINT_LEVEL:
        viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, repCap, RSSPECAN_ATTR_PULSE_RESULT_TABLE_ENVELOPE_MODEL_FALL_TOP_POINT_LEVEL_LIMIT_ENABLED, state),
                4, "State");
        break;

    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, parameter), 3, "Parameter");
    }
    if (state == VI_TRUE)
    {
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CALC%ld:TABL:EMOD:%s:LIM %.12lG,%.12lG", window, p2string, lowerLimit,
                 upperLimit);
        checkErr(RsCore_Write(instrSession, cmd));
        checkErr(rsspecan_CheckStatus(instrSession));
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the limits in stability parameter group.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR parameter/This control selects the stability parameter.
/// HIPAR state/This control sets the selected parameter limit state.
/// HIPAR lowerLimit/This control sets the lower limit of the valid value range.
/// HIPAR upperLimit/This control sets the upper limit of the valid value range.
ViStatus _VI_FUNC rsspecan_ConfigurePulseResultTableStabilityLimit(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   parameter,
                            ViBoolean state,
                            ViReal64  lowerLimit,
                            ViReal64  upperLimit)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  windowMax = 0;
    ViAttr   boolAttributes[] = {
             RSSPECAN_ATTR_PULSE_RESULT_TABLE_STABILITY_AMPLITUDE_LIMIT_ENABLED,
             RSSPECAN_ATTR_PULSE_RESULT_TABLE_STABILITY_BURST_LIMIT_ENABLED,
             RSSPECAN_ATTR_PULSE_RESULT_TABLE_STABILITY_PHASE_LIMIT_ENABLED,
             RSSPECAN_ATTR_PULSE_RESULT_TABLE_STABILITY_POSITION_NUMBER_IN_BURST_LIMIT_ENABLED,
             RSSPECAN_ATTR_PULSE_RESULT_TABLE_STABILITY_TOTAL_LIMIT_ENABLED
             };
    ViChar*  cmdToken;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetRepCapIdItemsCount(instrSession, "Window", &windowMax));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, window, 0, windowMax - 1),
        2, "Window");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_PULSE_RESULT_TABLE_STABILITY_AMPLITUDE_LIMIT_ENABLED, "Window", window, RS_REPCAP_BUF_SIZE, repCap),
        2, "Window")

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, parameter, RSSPECAN_VAL_PULSE_STABILITY_AMPLITUDE, RSSPECAN_VAL_PULSE_STABILITY_TOTAL),
            3, "Parameter");

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, repCap, boolAttributes[parameter], state),
            4, "State");

    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &RsSpecAn_rngPulseParameterStability, parameter, &cmdToken),
        3, "Parameter");

    (void)snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CALC%ld:TABL:STAB:%s:LIM %lf,%lf", window, cmdToken, lowerLimit, upperLimit);
checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN The y-axis is scaled automatically according to the current measurement settings and results.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR autoScale/The y-axis is scaled automatically/manualy.
ViStatus _VI_FUNC rsspecan_ConfigurePulseAutomaticGridScaling(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   reserved,
                            ViBoolean autoScale)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
             reserved;

    checkErr(RsCore_LockSession(instrSession));

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "Win%ld", window);

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, repCap, RSSPECAN_ATTR_PULSE_AUTO_GRID_SCALING, autoScale),
            4, "autoScale");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function defines the scaling using absolute minimum and maximum values.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR maximum/This control defines the maximum value of the y-axis for the selected result display.
/// HIPAR minimum/This control defines the minimum value of the y-axis for the selected result display.
ViStatus _VI_FUNC rsspecan_ConfigurePulseAbsoluteScaling(ViSession instrSession,
                                                         ViInt32   window,
                                                         ViReal64  minimum,
                                                         ViReal64  maximum)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "Win%ld", window);

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_PULSE_ABSOLUTE_SCALING_MIN, minimum),
            3, "Minimum");

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_PULSE_ABSOLUTE_SCALING_MAX, maximum),
            4, "Maximum");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function defines the scaling relative to a reference value,
/// HIFN with a specified value range per division.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR perDivison/This control defines the value range to be displayed per division
/// HIPAR perDivison/of the diagram (1/10 of total range).
/// HIPAR referencePosition/This control defines the vertical position of the reference level
/// HIPAR referencePosition/on the display grid.
/// HIPAR referenceValue/This control defines the reference value to be displayed at the
/// HIPAR referenceValue/specified reference position.
ViStatus _VI_FUNC rsspecan_ConfigurePulseRelativeScaling(ViSession instrSession,
                                                         ViInt32   window,
                                                         ViInt32   reserved,
                                                         ViReal64  perDivison,
                                                         ViReal64  referencePosition,
                                                         ViReal64  referenceValue)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
             reserved;

    checkErr(RsCore_LockSession(instrSession));

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "Win%ld", window);

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_DISP_Y_AXIS_SCALING, perDivison),
            4, "Per Divison");

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_DISP_REF_POSITION, referencePosition),
            5, "Reference Position");

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_REFERENCE_LEVEL, referenceValue),
            6, "Reference Value");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function defines the minimum and maximum reference values assigned
/// HIFN to the reference position in the specified window
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR minimum/This control defines the minimum reference value assigned
/// HIPAR minimum/to the reference position in the specified window.
/// HIPAR maximum/This control defines the maximum reference value assigned
/// HIPAR maximum/to thereference position in the specified window.
ViStatus _VI_FUNC rsspecan_ConfigurePulseReferenceValues(ViSession instrSession,
                                                         ViInt32   window,
                                                         ViInt32   reserved,
                                                         ViReal64  minimum,
                                                         ViReal64  maximum)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
             reserved;

    checkErr(RsCore_LockSession(instrSession));

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "Win%ld", window);

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_PULSE_REFERENCE_VALUE_MIN, minimum),
            4, "Minimum");

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_PULSE_REFERENCE_VALUE_MAX, maximum),
            5, "Maximum");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN This function queries the ids of the detected pulses
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR queryRange/Selects the query range.
/// HIPAR arraySize/This parameter determines the size of array for Pulse IDs values.
/// HIPAR pulseIDs/This control returns the ids of the detected pulses.
/// HIPAR returnedValues/Returns the number of returned values.
ViStatus _VI_FUNC rsspecan_QueryPulseIDs(ViSession instrSession,
                                         ViInt32   queryRange,
                                         ViInt32   arraySize,
                                         ViReal64  pulseIDs[],
                                         ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViChar*  p2string;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K6"));

    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &rsspecan_rngPulseQueryRange, queryRange, &p2string),
                2, "Query Range");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SENS:PULS:ID? %s", p2string);
    checkErr(RsCore_QueryFloatArrayToUserBuffer(instrSession, cmd, arraySize, pulseIDs, returnedValues));

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function returns the detected pulse numbers.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR queryRange/Selects the query range.
/// HIPAR arraySize/This parameter determines the size of array for Pulse Numbers values.
/// HIPAR pulseNumbers/This control returns the detected pulse numbers.
/// HIPAR returnedValues/Returns the number of returned values.
ViStatus _VI_FUNC rsspecan_QueryPulseNumbers(ViSession instrSession,
                                             ViInt32   queryRange,
                                             ViInt32   arraySize,
                                             ViReal64  pulseNumbers[],
                                             ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViChar*  p2string;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K6"));
    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &rsspecan_rngPulseQueryRange, queryRange, &p2string),
                2, "Query Range");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SENS:PULS:NUMB? %s", p2string);
    checkErr(RsCore_QueryFloatArrayToUserBuffer(instrSession, cmd, arraySize, pulseNumbers, returnedValues));

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries all detected pulses in the entire measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR pulseCount/Queries all detected pulses in the entire measurement.
ViStatus _VI_FUNC rsspecan_QueryPulseCountAll(ViSession instrSession,
                                              ViInt32*  pulseCount)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_GetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_ALL_PULSE_COUNT, pulseCount),
            2, "Pulse Count");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the number of detected pulses in the current capture buffer.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR pulseCount/Queries the number of detected pulses in the current capture buffer.
ViStatus _VI_FUNC rsspecan_QueryPulseCountCurrentBuffer(ViSession instrSession,
                                                        ViInt32*  pulseCount)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_GetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_CURRENT_BUFFER_PULSE_COUNT, pulseCount),
            2, "Pulse Count");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function returns the calculated pulse power parameters values.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR parameter/This control selects the power parameter.
/// HIPAR resultType/Selects measurement type.
/// HIPAR queryRange/Selects the query range.
/// HIPAR arraySize/This parameter determines the size of array result values.
/// HIPAR results/Returns selected power results.
/// HIPAR returnedValues/Returns the number of returned values.
ViStatus _VI_FUNC rsspecan_QueryPulseResultPower(ViSession instrSession,
                                                 ViInt32   parameter,
                                                 ViInt32   resultType,
                                                 ViInt32   queryRange,
                                                 ViInt32   arraySize,
                                                 ViReal64  results[],
                                                 ViInt32*  returnedValues)
{
    ViStatus   error = VI_SUCCESS;
    ViChar     cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViChar*    p2resultType;
    ViChar*    p2queryRange;
    ViString   power[] = {
    "",        "TOP", "BASE", "AMPL", "ON", "AVG", "MIN", "MAX", "PON", "PAVG", "PMIN", "ADR:PERC", "ADR:DB", "RIPP:PERC",
    "RIPP:DB", "OVER:PERC", "OVER:DB", "POIN", "PPR", "AMPL:I", "AMPL:Q"
               };

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K6"));

    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &rsspecan_rngPulseResultType, resultType, &p2resultType),
                3, "Result Type");
    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &rsspecan_rngPulseQueryRange, queryRange, &p2queryRange),
                4, "Query Range");

    if (p2resultType == NULL) p2resultType = "";

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SENS:PULS:POW:%s%s? %s", power[parameter],
             p2resultType, p2queryRange);
    checkErr(RsCore_QueryFloatArrayToUserBuffer(instrSession, cmd, arraySize, results, returnedValues));

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Returns the pulse count for the power parameter group.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR parameterGroup/Selects the power parameter.
/// HIPAR queryRange/Range in which pulse count is queried.
/// HIPAR pulseCount/Returns the number of detected power pulses according to specified
/// HIPAR pulseCount/parameters.
ViStatus _VI_FUNC rsspecan_QueryPulseCountPower(ViSession instrSession,
                                                ViInt32   parameterGroup,
                                                ViInt32   queryRange,
                                                ViInt32*  pulseCount)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_LEN] = "";
    ViChar*  p2parameter;
    ViChar*  p2queryRange;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(
        RsCore_FindRangeTableDiscreteCmdString(instrSession, &rsspecan_rngPulseParameterPower, parameterGroup, &p2parameter));

    checkErr(
        RsCore_FindRangeTableDiscreteCmdString(instrSession, &RsSpecAn_rngQueryPulseCountRange, queryRange, &p2queryRange));

    snprintf(cmd, RS_MAX_MESSAGE_LEN, "PULS:POW:%s:COUN? %s", p2parameter, p2queryRange);

    checkErr(rsspecan_QueryViInt32(instrSession, cmd, pulseCount));

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function returns the calculated pulse timing parameters values.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR parameter/This control selects the power parameter.
/// HIPAR resultType/Selects measurement type.
/// HIPAR queryRange/Selects the query range.
/// HIPAR arraySize/This parameter determines the size of array result values.
/// HIPAR results/Returns selected timing results.
/// HIPAR returnedValues/Returns the number of returned values.
ViStatus _VI_FUNC rsspecan_QueryPulseResultTiming(ViSession instrSession,
                                                  ViInt32   parameter,
                                                  ViInt32   resultType,
                                                  ViInt32   queryRange,
                                                  ViInt32   arraySize,
                                                  ViReal64  results[],
                                                  ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViChar*  p2parameter;
    ViChar*  p2queryRange;
    ViChar*  p2resultType;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K6"));

    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &rsspecan_rngPulseParameterTiming, parameter, &p2parameter),
                2, "Parameter");
    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &rsspecan_rngPulseResultType, resultType, &p2resultType),
                3, "Result Type");
    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &rsspecan_rngPulseQueryRange, queryRange, &p2queryRange),
                4, "Query Range");

    if (queryRange == RSSPECAN_VAL_PULSE_QUERY_RANGE_SELECTED && resultType != RSSPECAN_VAL_PULSE_MEASTYPE_CURR)
    {
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, queryRange),
                    4, "Query Range");
    }

    if (p2resultType == NULL) p2resultType = "";

    if (parameter == RSSPECAN_VAL_PULSE_RESULT_TIMING_TST)
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SENS:PULS:TIM:TST? %s", p2queryRange);
    else
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SENS:PULS:TIM:%s%s? %s", p2parameter, p2resultType, p2queryRange);
    checkErr(RsCore_QueryFloatArrayToUserBuffer(instrSession, cmd, arraySize, results, returnedValues));

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Returns the pulse counts for timing parameter group.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR groupParameter/Selects the Timing parameter.
/// HIPAR queryRange/Range in which pulse count is queried.
/// HIPAR pulseCount/Returns the number of detected power pulses according to specified
/// HIPAR pulseCount/parameters.
ViStatus _VI_FUNC rsspecan_QueryPulseCountTiming(ViSession instrSession,
                                                 ViInt32   groupParameter,
                                                 ViInt32   queryRange,
                                                 ViInt32*  pulseCount)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_LEN] = "";
    ViChar*  p2parameter;
    ViChar*  p2queryRange;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(
        RsCore_FindRangeTableDiscreteCmdString(instrSession, &rsspecan_rngPulseParameterTiming, groupParameter, &p2parameter));

    checkErr(
        RsCore_FindRangeTableDiscreteCmdString(instrSession, &RsSpecAn_rngQueryPulseCountRange, queryRange, &p2queryRange));

    snprintf(cmd, RS_MAX_MESSAGE_LEN, "PULS:TIM:%s:COUN? %s", p2parameter, p2queryRange);

    checkErr(rsspecan_QueryViInt32(instrSession, cmd, pulseCount));

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN This function returns the calculated pulse frequency parameters values.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR parameter/This control selects the power parameter.
/// HIPAR resultType/Selects measurement type.
/// HIPAR queryRange/Selects the query range.
/// HIPAR arraySize/This parameter determines the size of array result values.
/// HIPAR results/Returns selected frequency results.
/// HIPAR returnedValues/Returns the number of returned values.
ViStatus _VI_FUNC rsspecan_QueryPulseResultFrequency(ViSession instrSession,
                                                     ViInt32   parameter,
                                                     ViInt32   resultType,
                                                     ViInt32   queryRange,
                                                     ViInt32   arraySize,
                                                     ViReal64  results[],
                                                     ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViChar*  p2parameter;
    ViChar*  p2queryRange;
    ViChar*  p2resultType;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K6"));

    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &rsspecan_rngPulseParameterFrequency, parameter, &p2parameter),
        2, "Parameter");
    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &rsspecan_rngPulseResultType, resultType, &p2resultType),
                3, "Result Type");
    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &rsspecan_rngPulseQueryRange, queryRange, &p2queryRange),
                4, "Query Range");

    if (p2resultType == NULL) p2resultType = "";

    if (queryRange == RSSPECAN_VAL_PULSE_QUERY_RANGE_SELECTED && resultType != RSSPECAN_VAL_PULSE_MEASTYPE_CURR)
    {
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, queryRange),
                    4, "Query Range");
    }

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SENS:PULS:FREQ:%s%s? %s", p2parameter, p2resultType, p2queryRange);
    checkErr(RsCore_QueryFloatArrayToUserBuffer(instrSession, cmd, arraySize, results, returnedValues));

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Returns the pulse counts for the frequency parameter group.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR groupParameter/Selects the Frequency parameter.
/// HIPAR queryRange/Range in which pulse count is queried.
/// HIPAR pulseCount/Returns the number of detected power pulses according to specified
/// HIPAR pulseCount/parameters.
ViStatus _VI_FUNC rsspecan_QueryPulseCountFrequency(ViSession instrSession,
                                                    ViInt32   groupParameter,
                                                    ViInt32   queryRange,
                                                    ViInt32*  pulseCount)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_LEN] = "";
    ViChar*  p2parameter;
    ViChar*  p2queryRange;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(
        RsCore_FindRangeTableDiscreteCmdString(instrSession, &rsspecan_rngPulseParameterFrequency, groupParameter, &p2parameter
        ));

    checkErr(
        RsCore_FindRangeTableDiscreteCmdString(instrSession, &RsSpecAn_rngQueryPulseCountRange, queryRange, &p2queryRange));

    snprintf(cmd, RS_MAX_MESSAGE_LEN, "PULS:FREQ:%s:COUN? %s", p2parameter, p2queryRange);

    checkErr(rsspecan_QueryViInt32(instrSession, cmd, pulseCount));

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN This function returns the calculated pulse phase parameters values.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR parameter/This control selects the power parameter.
/// HIPAR resultType/Selects measurement type.
/// HIPAR queryRange/Selects the query range.
/// HIPAR arraySize/This parameter determines the size of array result values.
/// HIPAR results/Returns selected phase results.
/// HIPAR returnedValues/Returns the number of returned values.
ViStatus _VI_FUNC rsspecan_QueryPulseResultPhase(ViSession instrSession,
                                                 ViInt32   parameter,
                                                 ViInt32   resultType,
                                                 ViInt32   queryRange,
                                                 ViInt32   arraySize,
                                                 ViReal64  results[],
                                                 ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViChar*  p2parameter;
    ViChar*  p2queryRange;
    ViChar*  p2resultType;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K6"));

    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &rsspecan_rngPulseParameterPhase, parameter, &p2parameter),
                2, "Parameter");
    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &rsspecan_rngPulseResultType, resultType, &p2resultType),
                3, "Result Type");
    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &rsspecan_rngPulseQueryRange, queryRange, &p2queryRange),
                4, "Query Range");

    if (p2resultType == NULL) p2resultType = "";

    if (queryRange == RSSPECAN_VAL_PULSE_QUERY_RANGE_SELECTED && resultType != RSSPECAN_VAL_PULSE_MEASTYPE_CURR)
    {
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, queryRange),
                    4, "Query Range");
    }

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SENS:PULS:PHAS:%s%s? %s", p2parameter, p2resultType, p2queryRange);
    checkErr(RsCore_QueryFloatArrayToUserBuffer(instrSession, cmd, arraySize, results, returnedValues));

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Returns the pulse counts for the phase parameter group.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR queryRange/Range in which pulse count is queried.
/// HIPAR groupParameter/Selects the Phase parameter.
/// HIPAR pulseCount/Returns the number of detected power pulses according to specified
/// HIPAR pulseCount/parameters.
ViStatus _VI_FUNC rsspecan_QueryPulseCountPhase(ViSession instrSession,
                                                ViInt32   queryRange,
                                                ViInt32   groupParameter,
                                                ViInt32*  pulseCount)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_LEN] = "";
    ViChar*  p2parameter;
    ViChar*  p2queryRange;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(
        RsCore_FindRangeTableDiscreteCmdString(instrSession, &rsspecan_rngPulseParameterPhase, groupParameter, &p2parameter));

    checkErr(
        RsCore_FindRangeTableDiscreteCmdString(instrSession, &RsSpecAn_rngQueryPulseCountRange, queryRange, &p2queryRange));

    snprintf(cmd, RS_MAX_MESSAGE_LEN, "PULS:PHAS:%s:COUN? %s", p2parameter, p2queryRange);

    checkErr(rsspecan_QueryViInt32(instrSession, cmd, pulseCount));

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN This function returns the calculated pulse time sidelobe parameters values.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR parameter/This control selects the power parameter.
/// HIPAR resultType/Selects measurement type.
/// HIPAR queryRange/Selects the query range.
/// HIPAR arraySize/This parameter determines the size of array result values.
/// HIPAR results/Returns selected timing results.
/// HIPAR returnedValues/Returns the number of returned values.
ViStatus _VI_FUNC rsspecan_QueryPulseResultTimeSidelobe(ViSession instrSession,
                                                        ViInt32   parameter,
                                                        ViInt32   resultType,
                                                        ViInt32   queryRange,
                                                        ViInt32   arraySize,
                                                        ViReal64  results[],
                                                        ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViChar*  p2parameter;
    ViChar*  p2queryRange;
    ViChar*  p2resultType;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K6S"));

    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &RsSpecAn_rngPulseParameterTimeSidelobe, parameter, &p2parameter),
        2, "Parameter");
    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &rsspecan_rngPulseResultType, resultType, &p2resultType),
                3, "Result Type");
    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &rsspecan_rngPulseQueryRange, queryRange, &p2queryRange),
                4, "Query Range");

    if (p2resultType == NULL) p2resultType = "";

    if (queryRange == RSSPECAN_VAL_PULSE_QUERY_RANGE_SELECTED && resultType != RSSPECAN_VAL_PULSE_MEASTYPE_CURR)
    {
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, queryRange),
                    4, "Query Range");
    }

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SENS:PULS:TSID:%s%s? %s", p2parameter,
             p2resultType, p2queryRange);
    checkErr(RsCore_QueryFloatArrayToUserBuffer(instrSession, cmd, arraySize, results, returnedValues));
    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Returns the pulse counts for the time sidelobe parameter group.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR groupParameter/Selects the Time sidelobe parameter.
/// HIPAR queryRange/Range in which pulse count is queried.
/// HIPAR pulseCount/Returns the number of detected power pulses according to specified
/// HIPAR pulseCount/parameters.
ViStatus _VI_FUNC rsspecan_QueryPulseCountTimeSidelobe(ViSession instrSession,
                                                       ViInt32   groupParameter,
                                                       ViInt32   queryRange,
                                                       ViInt32*  pulseCount)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_LEN] = "";
    ViChar*  p2parameter;
    ViChar*  p2queryRange;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(
        RsCore_FindRangeTableDiscreteCmdString(instrSession, &RsSpecAn_rngPulseParameterTimeSidelobe, groupParameter, &
            p2parameter));

    checkErr(
        RsCore_FindRangeTableDiscreteCmdString(instrSession, &RsSpecAn_rngQueryPulseCountRange, queryRange, &p2queryRange));

    snprintf(cmd, RS_MAX_MESSAGE_LEN, "PULS:TSID:%s:COUN? %s", p2parameter, p2queryRange);

    checkErr(rsspecan_QueryViInt32(instrSession, cmd, pulseCount));

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function returns the calculated pulse envelope model parameters
/// HIFN  values.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR parameter/This control selects the envelope model parameter.
/// HIPAR resultType/Selects measurement type.
/// HIPAR queryRange/Selects the query range.
/// HIPAR arraySize/This parameter determines the size of array result values. The Result
/// HIPAR arraySize/Values array must contain at least this number of items.
/// HIPAR results/Returns selected phase results.
/// HIPAR returnedValues/Returns the number of returned values.
ViStatus _VI_FUNC rsspecan_QueryPulseResultEnvelopeModel(ViSession instrSession,
                                                         ViInt32   parameter,
                                                         ViInt32   resultType,
                                                         ViInt32   queryRange,
                                                         ViInt32   arraySize,
                                                         ViReal64  results[],
                                                         ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViChar*  p2parameter;
    ViChar*  p2queryRange;
    ViChar*  p2resultType;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K6"));

    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &RsSpecAn_rngPulseParameterEnvelopeModel, parameter, &p2parameter),
        2, "Parameter");
    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &rsspecan_rngPulseResultType, resultType, &p2resultType),
                3, "Result Type");
    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &rsspecan_rngPulseQueryRange, queryRange, &p2queryRange),
                4, "Query Range");

    if (p2resultType == NULL) p2resultType = "";

    if (queryRange == RSSPECAN_VAL_PULSE_QUERY_RANGE_SELECTED && resultType != RSSPECAN_VAL_PULSE_MEASTYPE_CURR)
    {
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, queryRange),
                    4, "Query Range");
    }

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SENS:PULS:EMOD:%s%s? %s", p2parameter,
             p2resultType, p2queryRange);
    checkErr(RsCore_QueryFloatArrayToUserBuffer(instrSession, cmd, arraySize, results, returnedValues));

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Returns the pulse counts for envelope model parameter group.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR parameter/This control selects the envelope model parameter.
/// HIPAR queryRange/Range in which pulse count is queried.
/// HIPAR pulseCount/Returns the number of detected power pulses according to specified
/// HIPAR pulseCount/parameters.
ViStatus _VI_FUNC rsspecan_QueryPulseCountEnvelopeModel(ViSession instrSession,
                                                        ViInt32   parameter,
                                                        ViInt32   queryRange,
                                                        ViInt32*  pulseCount)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_LEN] = "";
    ViChar*  p2parameter;
    ViChar*  p2queryRange;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(
        RsCore_FindRangeTableDiscreteCmdString(instrSession, &RsSpecAn_rngPulseParameterEnvelopeModel, parameter, &p2parameter
        ));

    checkErr(
        RsCore_FindRangeTableDiscreteCmdString(instrSession, &RsSpecAn_rngQueryPulseCountRange, queryRange, &p2queryRange));

    snprintf(cmd, RS_MAX_MESSAGE_LEN, "PULS:EMOD:%s:COUN? %s", p2parameter, p2queryRange);

    checkErr(rsspecan_QueryViInt32(instrSession, cmd, pulseCount));

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Returns the calculated pulse stability parameters values.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR parameter/Selects the stability parameter.
/// HIPAR resultType/Selects measurement type.
/// HIPAR queryRange/Selects the query range.
/// HIPAR arraySize/This parameter determines the size of array result values. The Result
/// HIPAR arraySize/Values array must contain at least this number of items.
/// HIPAR results/Returns selected stability results.
/// HIPAR returnedValues/Returns the number of returned values.
ViStatus _VI_FUNC rsspecan_QueryPulseResultStability(ViSession instrSession,
                                                     ViInt32   parameter,
                                                     ViInt32   resultType,
                                                     ViInt32   queryRange,
                                                     ViInt32   arraySize,
                                                     ViReal64  results[],
                                                     ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViChar*  parameterCmd;
    ViChar*  resultTypeCmd;
    ViChar*  queryRangeCmd;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &RsSpecAn_rngPulseParameterStability, parameter, &parameterCmd),
        2, "Parameter");
    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &rsspecan_rngPulseResultType, resultType, &resultTypeCmd),
        3, "Result Type");
    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &rsspecan_rngPulseQueryRange, queryRange, &queryRangeCmd),
        4, "Query Range");

    if (resultTypeCmd == NULL) resultTypeCmd = "";

    if (queryRange == RSSPECAN_VAL_PULSE_QUERY_RANGE_SELECTED && resultType != RSSPECAN_VAL_PULSE_MEASTYPE_CURR)
    {
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, queryRange),
            4, "Query Range");
    }

    (void)snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "PULS:STAB:%s%s? %s", parameterCmd, resultTypeCmd, queryRangeCmd);
checkErr(RsCore_QueryFloatArrayToUserBuffer(instrSession, cmd, arraySize, results, returnedValues));

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Returns the pulse counts for stability parameter group.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR parameter/This control selects the stability parameter.
/// HIPAR queryRange/Range in which pulse count is queried.
/// HIPAR pulseCount/Returns the number of detected power pulses according to specified
/// HIPAR pulseCount/parameters.
ViStatus _VI_FUNC rsspecan_QueryPulseCountStability(ViSession instrSession,
                                                    ViInt32   parameter,
                                                    ViInt32   queryRange,
                                                    ViInt32*  pulseCount)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_LEN] = "";
    ViChar*  p2parameter;
    ViChar*  p2queryRange;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(
        RsCore_FindRangeTableDiscreteCmdString(instrSession, &RsSpecAn_rngPulseParameterStability, parameter, &p2parameter));

    checkErr(
        RsCore_FindRangeTableDiscreteCmdString(instrSession, &RsSpecAn_rngQueryPulseCountRange, queryRange, &p2queryRange));

    snprintf(cmd, RS_MAX_MESSAGE_LEN, "PULS:STAB:%s:COUN? %s", p2parameter, p2queryRange);

    checkErr(rsspecan_QueryViInt32(instrSession, cmd, pulseCount));

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function returns the resulting resolution bandwidth for the
/// HIFN spectrum.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR RBW/Returns the resulting resolution bandwidth for the spectrum.
ViStatus _VI_FUNC rsspecan_QueryPulseSpectrumResolutionBandwidth(
                            ViSession instrSession,
                            ViInt32   window,
                            ViReal64* RBW)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_PULSE_QUERY_SPECTRUM_RESOLUTION_BANDWIDTH,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap),
            2, "Window");

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_PULSE_QUERY_SPECTRUM_RESOLUTION_BANDWIDTH, RBW),
            2, "RBW");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function queries the I/Q data currently stored in the memory of
/// HIFN  the R&S FSW for the defined result range.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR arraySize/Sets the Array Size parameter.
/// HIPAR storedInMemory/Queries the I/Q data currently stored in the memory of the R&S FSW for
/// HIPAR storedInMemory/the defined result range.
ViStatus _VI_FUNC rsspecan_QueryPulseResultRangeIQStoredInMemory(
                            ViSession instrSession,
                            ViInt32   arraySize,
                            ViChar    storedInMemory[])
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_GetAttributeViString(instrSession, "", RSSPECAN_ATTR_PULSE_RESULT_RANGE_IQ_STORED_IN_MEMORY,
            arraySize, storedInMemory),
            3, "Stored In Memory");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function returns an array of sample indices for the start
/// HIFN of each captured data segment.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR arraySize/This parameter determines the size of array for Pulse Numbers values.
/// HIPAR sampleIndices/Returns an array of sample indices for the start of each captured data segment.
/// HIPAR returnedValues/Returns the number of returned values.
ViStatus _VI_FUNC rsspecan_QueryPulseSegmentedDataCapturingBoundary(
                            ViSession instrSession,
                            ViInt32   arraySize,
                            ViInt32   sampleIndices[],
                            ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K6"));

    checkErr(
        RsCore_QueryIntegerArrayToUserBuffer(instrSession, "TRAC:IQ:SCAP:BOUN?", arraySize, sampleIndices, (ViPInt32)
            returnedValues));

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function returns an array of timestamps for each segment
/// HIFN start in the captured data.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR arraySize/This parameter determines the size of array for Pulse Numbers values.
/// HIPAR timestamps/Returns an array of timestamps for each segment start in the captured data.
/// HIPAR returnedValues/Returns the number of returned values.
ViStatus _VI_FUNC rsspecan_QueryPulseSegmentedDataCapturingTimestamps(
                            ViSession instrSession,
                            ViInt32   arraySize,
                            ViReal64  timestamps[],
                            ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K6"));

    checkErr(
        RsCore_QueryFloatArrayToUserBuffer(instrSession, "TRAC:IQ:SCAP:TST:SST?", arraySize, timestamps, returnedValues
        ));

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function returns an array of trigger event time stamps for the captured data segments.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR arraySize/This parameter determines the size of array for Pulse Numbers values.
/// HIPAR trigger/Returns an array of trigger event time stamps for the captured data segments.
/// HIPAR returnedValues/Returns the number of returned values.
ViStatus _VI_FUNC rsspecan_QueryPulseSegmentedDataCapturingTrigger(
                            ViSession instrSession,
                            ViInt32   arraySize,
                            ViReal64  trigger[],
                            ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K6"));

    checkErr(
        RsCore_QueryFloatArrayToUserBuffer(instrSession, "TRAC:IQ:SCAP:TST:TRIG?", arraySize, trigger, returnedValues));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function returns results for the limit check of power.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR parameter/This control selects the power parameter.
/// HIPAR arraySize/This parameter determines the size of array result values.
/// HIPAR limitResults/Returns limit results.
/// HIPAR returnedValues/Returns the number of returned values.
ViStatus _VI_FUNC rsspecan_QueryPulseResultLimitPower(ViSession instrSession,
                                                      ViInt32   parameter,
                                                      ViInt32   queryRange,
                                                      ViInt32   arraySize,
                                                      ViInt32   limitResults[],
                                                      ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViChar*  p2parameter;
    ViChar*  p2queryRange;
    ViString limitResultsTable[] = {"PASSED", "FAILED LOW", "FAILED HIGH", NULL};

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K6"));

    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &rsspecan_rngPulseParameterPower, parameter, &p2parameter),
                2, "Parameter");
    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &rsspecan_rngPulseQueryRange, queryRange, &p2queryRange),
                3, "Query Range");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SENS:PULS:POW:%s:LIMit? %s", p2parameter, p2queryRange);
    checkErr(
        RsCore_QueryStringIndexesArrayToUserBuffer(instrSession, cmd, limitResultsTable, arraySize, limitResults,
            returnedValues));

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function returns results for the limit check of timing.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR parameter/This control selects the power parameter.
/// HIPAR arraySize/This parameter determines the size of array result values.
/// HIPAR limitResults/Returns limit results.
/// HIPAR returnedValues/Returns the number of returned values.
ViStatus _VI_FUNC rsspecan_QueryPulseResultLimitTiming(ViSession instrSession,
                                                       ViInt32   parameter,
                                                       ViInt32   queryRange,
                                                       ViInt32   arraySize,
                                                       ViInt32   limitResults[],
                                                       ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViChar*  p2parameter;
    ViChar*  p2queryRange;
    ViString limitResultsTable[] = {"PASSED", "FAILED LOW", "FAILED HIGH", NULL};

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K6"));

    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &rsspecan_rngPulseParameterTiming, parameter, &p2parameter),
        2, "Parameter");
    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &rsspecan_rngPulseQueryRange, queryRange, &p2queryRange),
                3, "Query Range");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SENS:PULS:TIM:%s:LIMit? %s", p2parameter, p2queryRange);
    checkErr(
        RsCore_QueryStringIndexesArrayToUserBuffer(instrSession, cmd, limitResultsTable, arraySize, limitResults,
            returnedValues));

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function returns results for the limit check of frequency.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR parameter/This control selects the frequency parameter.
/// HIPAR arraySize/This parameter determines the size of array result values.
/// HIPAR limitResults/Returns limit results.
/// HIPAR returnedValues/Returns the number of returned values.
ViStatus _VI_FUNC rsspecan_QueryPulseResultLimitFrequency(ViSession instrSession,
                                                          ViInt32   parameter,
                                                          ViInt32   queryRange,
                                                          ViInt32   arraySize,
                                                          ViInt32   limitResults[],
                                                          ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViChar*  p2parameter;
    ViChar*  p2queryRange;
    ViString limitResultsTable[] = {"PASSED", "FAILED LOW", "FAILED HIGH", NULL};

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K6"));

    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &rsspecan_rngPulseParameterFrequency, parameter, &p2parameter),
        2, "Parameter");
    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &rsspecan_rngPulseQueryRange, queryRange, &p2queryRange),
                3, "Query Range");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SENS:PULS:FREQ:%s:LIMit? %s", p2parameter, p2queryRange);
    checkErr(
        RsCore_QueryStringIndexesArrayToUserBuffer(instrSession, cmd, limitResultsTable, arraySize, limitResults,
            returnedValues));

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function returns results for the limit check of phase.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR parameter/This control selects the phase parameter.
/// HIPAR arraySize/This parameter determines the size of array result values.
/// HIPAR limitResults/Returns limit results.
/// HIPAR returnedValues/Returns the number of returned values.
ViStatus _VI_FUNC rsspecan_QueryPulseResultLimitPhase(ViSession instrSession,
                                                      ViInt32   parameter,
                                                      ViInt32   queryRange,
                                                      ViInt32   arraySize,
                                                      ViInt32   limitResults[],
                                                      ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViChar*  p2parameter;
    ViChar*  p2queryRange;
    ViString limitResultsTable[] = {"PASSED", "FAILED LOW", "FAILED HIGH", NULL};

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K6"));

    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &rsspecan_rngPulseParameterPhase, parameter, &p2parameter),
                2, "Parameter");
    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &rsspecan_rngPulseQueryRange, queryRange, &p2queryRange),
                3, "Query Range");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SENS:PULS:PHAS:%s:LIMit? %s", p2parameter, p2queryRange);
    checkErr(
        RsCore_QueryStringIndexesArrayToUserBuffer(instrSession, cmd, limitResultsTable, arraySize, limitResults,
            returnedValues));

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function returns the time sidelobe limits.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR parameter/This control selects the power parameter.
/// HIPAR arraySize/This parameter determines the size of array result values.
/// HIPAR limitResults/Returns limit results.
/// HIPAR returnedValues/Returns the number of returned values.
ViStatus _VI_FUNC rsspecan_QueryPulseResultLimitTimeSidelobe(ViSession instrSession,
                                                             ViInt32   parameter,
                                                             ViInt32   queryRange,
                                                             ViInt32   arraySize,
                                                             ViInt32   limitResults[],
                                                             ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViChar*  p2parameter;
    ViChar*  p2queryRange;
    ViString limitResultsTable[] = {"PASSED", "FAILED LOW", "FAILED HIGH", NULL};

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K6S"));

    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &RsSpecAn_rngPulseParameterTimeSidelobe, parameter, &p2parameter),
        2, "Parameter");
    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &rsspecan_rngPulseQueryRange, queryRange, &p2queryRange),
                3, "Query Range");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SENS:PULS:TSID:%s:LIMit? %s", p2parameter, p2queryRange);
    checkErr(
        RsCore_QueryStringIndexesArrayToUserBuffer(instrSession, cmd, limitResultsTable, arraySize, limitResults,
            returnedValues));

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function returns the results for the limit check of phase.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR parameter/This control selects the envelope model parameter.
/// HIPAR queryRange/Selects the query range.
/// HIPAR arraySize/This parameter determines the size of array result values.
/// HIPAR limitResults/Returns limit results.
/// HIPAR returnedValues/Returns the number of returned values.
ViStatus _VI_FUNC rsspecan_QueryPulseResultLimitEnvelopeModel(
                            ViSession instrSession,
                            ViInt32   parameter,
                            ViInt32   queryRange,
                            ViInt32   arraySize,
                            ViInt32   limitResults[],
                            ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViChar*  p2parameter;
    ViChar*  p2queryRange;
    ViString limitResultsTable[] = {"PASSED", "FAILED LOW", "FAILED HIGH", NULL};

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K6"));

    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &RsSpecAn_rngPulseParameterEnvelopeModel, parameter, &p2parameter),
        2, "Parameter");
    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &rsspecan_rngPulseQueryRange, queryRange, &p2queryRange),
        3, "Query Range");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SENS:PULS:EMOD:%s:LIM? %s", p2parameter, p2queryRange);
    checkErr(RsCore_QueryStringIndexesArrayToUserBuffer(instrSession, cmd, limitResultsTable, arraySize, limitResults, returnedValues));

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Returns the results for the limit check of phase.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR parameter/Selects the stability parameter.
/// HIPAR queryRange/Selects the query range.
/// HIPAR arraySize/This parameter determines the size of array result values.
/// HIPAR limitResults/Returns limit results.
/// HIPAR returnedValues/Returns the number of returned values.
ViStatus _VI_FUNC rsspecan_PulseQueryResultLimitStability(ViSession instrSession,
                                                          ViInt32   parameter,
                                                          ViInt32   queryRange,
                                                          ViInt32   arraySize,
                                                          ViInt32   limitResults[],
                                                          ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViChar*  parameterCmd;
    ViChar*  queryRangeCmd;
    ViString limitResultsTable[] = { "PASSED", "FAILED LOW", "FAILED HIGH", NULL };

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &RsSpecAn_rngPulseParameterStability, parameter, &parameterCmd),
        2, "Parameter");
    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &rsspecan_rngPulseQueryRange, queryRange, &queryRangeCmd),
        3, "Query Range");

    (void)snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "PULS:STAB:%s:LIM? %s", parameterCmd, queryRangeCmd);
checkErr(RsCore_QueryStringIndexesArrayToUserBuffer(instrSession, cmd, limitResultsTable, arraySize, limitResults, returnedValues));

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}
