/*****************************************************************************
 *  Rohde&Schwarz Transient Analysis   instrument driver - option K60
 *
 *  VXIpnp Style Instrument Driver
 *  Original Release: June 2006
 *  By: Martin Koutny (instrument driver core)
 *      Helena Sucha (instrument specific implementation)
 *      Jiri Kominek (DrBase repository & tools)
 *
 *  Should you have any technical questions please contact the hotline of
 *  Rohde & Schwarz Vertriebs-GmbH Rohde & Schwarz Support Center
 *
 *  e-mail: CustomerSupport@rohde-schwarz.com
 *
 *****************************************************************************/

/*****************************************************************************
 *  K60 - Transient Analysis
 *****************************************************************************/

#include <math.h>
#include "rsspecan.h"

static ViString paramArr[] = {"ALL:STAT", "AVGF", "FMER", "FREQ", "MAXF", "RMSF", "RELF"};
static ViString paramChirpArr[] = {"ALL:STAT", "AVGF", "CHER", "FREQ", "MAXF", "RMSF", "BWID", "OVER", "UND"};
static ViString freqScalingArr[] = {"HZ", "KHZ", "MHZ", "GHZ"};
static ViString freqScalingCherArr[] = {"HZ_US", "KHZ_US", "MHZ_US", "GHZ_US"};
static ViString paramTimingArr[] = {"ALL:STAT", "BEG", "DWEL", "SWIT"};
static ViString paramTimingChirpArr[] = {"ALL:STAT", "BEG", "LENG", "RATE", "SWIT"};
static ViString timingScalingArr[] = {"NS", "US", "MS", "S"};
static ViString timingScalingRateArr[] = {"HZ_US", "KHZ_US", "MHZ_US", "GHZ_US"};
static ViString HopPowerAxisArr[] = {"AVGP", "MAXP", "MINP", "PWRR"};
static ViString HopTimingAxisArr[] = {"BEG", "DWEL", "SWIT"};
static ViString HopStateAxisArr[] = {"STAF", "IND"};
static ViString ChirpTimingAxisArr[] = {"BEG", "LENG", "RATE"};
static ViString ChirpStateAxisArr[] = {"IND"};
static ViString YAxisArr[] = {"COUN", "OCC"};
static ViString FreqCommandGroupArr[] = {"FREQ", "FMS", "PMS"};


/// HIFN  This function selects the transient analysis mode.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsspecan_TransientAnalysisMode(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_TRANSIENT_MODE, ""));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function defines which type of signal to expect (if known), thus
/// HIFN  determining the analysis method.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR model/This control defines which type of signal to expect (if known), thus
/// HIPAR model/determining the analysis method.
ViStatus _VI_FUNC rsspecan_ConfigureTransientAnalysisSignalModel(
                            ViSession instrSession,
                            ViInt32   model)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_TRANSIENT_MODEL, model),
            2, "Model");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function queries the number of states in the state table.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR model/This control defines which type of signal to expect (if known), thus
/// HIPAR model/determining the analysis method.
/// HIPAR numberofStates/This control returns the number of states in the state table.
ViStatus _VI_FUNC rsspecan_GetTransientAnalysisNumberOfStates(
                            ViSession instrSession,
                            ViInt32   model,
                            ViInt32*  numberofStates)
{
    ViStatus error = VI_SUCCESS;
    ViAttr   attr[2] = {
             RSSPECAN_ATTR_TRANSIENT_HOP_NUMBER_OF_STATES,
             RSSPECAN_ATTR_TRANSIENT_CHIRP_NUMBER_OF_STATES
             };

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, model, RSSPECAN_VAL_TRANSIENT_MODEL_HOP, RSSPECAN_VAL_TRANSIENT_MODEL_CHIRP),
            2, "Model");

    viCheckParm(rsspecan_GetAttributeViInt32(instrSession, "",
                    attr[model - 1],
                    numberofStates), 3, "Number of States");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures signal detection. The signal detection
/// HIFN  settings define the conditions under which a hop/chirp is detected
/// HIFN  within the input signal.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR model/This control defines which type of signal to expect (if known), thus
/// HIPAR model/determining the analysis method.
/// HIPAR frequencyOffset/Frequency offsets from the center frequency.
/// HIPAR arraySize/This parameter determines the size of Frequency Offset and Tolerance
/// HIPAR arraySize/arrays.
/// HIPAR tolerance/Tolerance above or below the nominal frequency.
/// HIPAR autoMode/This control activates and deactivates the auto chirp/hop state
/// HIPAR autoMode/detection.
ViStatus _VI_FUNC rsspecan_ConfigureTransientAnalysisSignalDetection(
                            ViSession instrSession,
                            ViInt32   model,
                            ViBoolean autoMode,
                            ViInt32   arraySize,
                            ViReal64  frequencyOffset[],
                            ViReal64  tolerance[])
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32RangeMin(instrSession, arraySize, 1),
                2, "Array Size");

    if (model == RSSPECAN_VAL_TRANSIENT_MODEL_HOP)
    {
        viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_TRANSIENT_HOP_AUTO, autoMode),
                2, "autoMode");

        if (!autoMode)
        {
            checkErr(RsCore_WriteAsciiViReal64ArraysInterleaved(instrSession, "CALC:HOPD:STAT ", arraySize, frequencyOffset, tolerance, NULL, NULL, NULL, NULL));
        }
    }
    else
    {
        viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_TRANSIENT_CHIRP_AUTO, autoMode),
                2, "autoMode");
        if (!autoMode)
        {
            checkErr(RsCore_WriteAsciiViReal64ArraysInterleaved(instrSession, "CALC:CHRD:STAT ", arraySize, frequencyOffset, tolerance, NULL, NULL, NULL, NULL));
        }
    }

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures timing. The dwell time is the time the signal
/// HIFN  remains in the tolerance area of a nominal hop frequency, that is, the
/// HIFN  duration of a hop from beginning to end. A hop is only detected if its
/// HIFN  dwell time lies within the defined minimum and maximum values.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR model/This control defines which type of signal to expect (if known), thus
/// HIPAR model/determining the analysis method.
/// HIPAR minTime/This control sets the minimum time.
/// HIPAR maxTime/This control sets the maximum time.
/// HIPAR autoMode/If enabled, the frequency offset is estimated automatically for each
/// HIPAR autoMode/individual pulse.
ViStatus _VI_FUNC rsspecan_ConfigureTransientAnalysisSignalDetectionTiming(
                            ViSession instrSession,
                            ViInt32   model,
                            ViBoolean autoMode,
                            ViReal64  minTime,
                            ViReal64  maxTime)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    if (model == RSSPECAN_VAL_TRANSIENT_MODEL_HOP)
    {
        viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_TRANSIENT_HOP_TIMING_AUTO_MODE, autoMode),
                3, "autoMode");
        if (!autoMode)
        {
            viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_TRANSIENT_HOP_TIMING_MINIMUM, minTime),
                    4, "Min Time");

            viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_TRANSIENT_HOP_TIMING_MAXIMUM, maxTime),
                    5, "Max Time");
        }
    }
    else
    {
        viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_TRANSIENT_CHIRP_TIMING_AUTO_MODE, autoMode),
                3, "autoMode");
        if (!autoMode)
        {
            viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_TRANSIENT_CHIRP_TIMING_MINIMUM, minTime),
                    4, "Min Time");

            viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_TRANSIENT_CHIRP_TIMING_MAXIMUM, maxTime),
                    5, "Max Time");
        }
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function loads or saves the current signal state table
/// HIFN  configuration to a file for later use.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR model/This control defines which type of signal to expect (if known), thus
/// HIPAR model/determining the analysis method.
/// HIPAR operation/Selects the table operation.
/// HIPAR tableName/String containing the name of the table.
ViStatus _VI_FUNC rsspecan_TransientAnalysisSignalStateTableOperation(
                            ViSession instrSession,
                            ViInt32   model,
                            ViInt32   operation,
                            ViString  tableName)
{
    ViStatus                                                error = VI_SUCCESS;
    ViAttr                                                  attr[2][2] = {
    {RSSPECAN_ATTR_TRANSIENT_HOP_SAVE_SIGNAL_STATE_TABLE,   RSSPECAN_ATTR_TRANSIENT_HOP_LOAD_SIGNAL_STATE_TABLE},
    {RSSPECAN_ATTR_TRANSIENT_CHIRP_SAVE_SIGNAL_STATE_TABLE, RSSPECAN_ATTR_TRANSIENT_CHIRP_LOAD_SIGNAL_STATE_TABLE}
                                                            };

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, model, RSSPECAN_VAL_TRANSIENT_MODEL_HOP, RSSPECAN_VAL_TRANSIENT_MODEL_CHIRP),
            2, "Model");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, operation, RSSPECAN_VAL_TRANSIENT_TABLE_SAVE, RSSPECAN_VAL_TRANSIENT_TABLE_LOAD),
            3, "Operation");

    viCheckParm(rsspecan_SetAttributeViString(instrSession, "",
                    attr[model-1][operation],
                    tableName), 4, "Table Name");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function queries the last stored parameters for generating
/// HIFN  multiple states.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR startFrequency/This control queries the last stored starting value for generating
/// HIPAR startFrequency/multiple states.
/// HIPAR stepSize/This control queries the last stored step size for generating multiple
/// HIPAR stepSize/states.
/// HIPAR numberofStates/This control queries the number of hop states to be generated by a
/// HIPAR numberofStates/subsequent
ViStatus _VI_FUNC rsspecan_GetTransientAnalysisGeneratedHopStatesParameters(
                            ViSession instrSession,
                            ViReal64* startFrequency,
                            ViReal64* stepSize,
                            ViInt32*  numberofStates)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_GET_TRANSIENT_HOP_STATE_TABLE_START_FREQUENCY, startFrequency),
            2, "Start Frequency");

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_GET_TRANSIENT_HOP_STATE_TABLE_STEP_SIZE, stepSize),
            3, "Step Size");

    viCheckParm(rsspecan_GetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_GET_TRANSIENT_HOP_STATE_TABLE_NUMBER_OF_STATES, numberofStates),
            4, "Number of States");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function adds multiple generated states to the state table. Note
/// HIFN  that a maximum of 1000 states can be defined in one table. To determine
/// HIFN  how many states are currently defined, use the
/// HIFN  RSSPECAN_ATTR_TRANSIENT_HOP_NUMBER_OF_STATES attribute.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR startFrequency/The frequency at which the first hop state will be generated.
/// HIPAR stepSize/The distance between two hop states.
/// HIPAR numberofStates/Number of hop states to be generated.
ViStatus _VI_FUNC rsspecan_AddTransientAnalysisHopStates(ViSession instrSession,
                                                         ViReal64  startFrequency,
                                                         ViReal64  stepSize,
                                                         ViInt32   numberofStates)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CALC:HOPD:STAT:TABL:ADD %.0lf,%.0lf,%ld", startFrequency, stepSize, numberofStates);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function replaces the state table's current states with a new set
/// HIFN  of generated states.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR startFrequency/The frequency at which the first hop state will be generated.
/// HIPAR stepSize/The distance between two hop states.
/// HIPAR numberofStates/Number of hop states to be generated.
ViStatus _VI_FUNC rsspecan_ReplaceTransientAnalysisHopStates(ViSession instrSession,
                                                             ViReal64  startFrequency,
                                                             ViReal64  stepSize,
                                                             ViInt32   numberofStates)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CALC:HOPD:STAT:TABL:REPL %.0lf,%.0lf,%ld", startFrequency, stepSize, numberofStates);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function applies tollerance and offset to all states in the state
/// HIFN  table.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR tollerance/This control applies a tolerance to all states in the state table.
/// HIPAR offset/This control applies an offset to all states in the state table.
ViStatus _VI_FUNC rsspecan_ApplyTransientAnalysisHopStatesGlobalValues(
                            ViSession instrSession,
                            ViReal64  tollerance,
                            ViReal64  offset)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_TRANSIENT_HOP_STATE_TABLE_TOLERANCE, tollerance),
            2, "Tollerance");

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_TRANSIENT_HOP_STATE_TABLE_OFFSET, offset),
            3, "Offset");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures data acquisition.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR measurementBandwidth/Defines the measurement bandwidth in Hz.
/// HIPAR sampleRate/This control defines the sample rate for the current measurement.
/// HIPAR measurementTime/This control defines the time data is captured. Note that the record
/// HIPAR measurementTime/length and the measurement time are interdependent.
/// HIPAR recordLength/This control defines the record length (in samples) for the current
/// HIPAR recordLength/measurement.
ViStatus _VI_FUNC rsspecan_ConfigureTransientAnalysisDataAcquisition(
                            ViSession instrSession,
                            ViReal64  measurementBandwidth,
                            ViReal64  sampleRate,
                            ViReal64  measurementTime,
                            ViInt32   recordLength)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_TRANSIENT_MEASUREMENT_BANDWIDTH, measurementBandwidth),
            2, "Measurement Bandwidth");

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_TRANSIENT_SAMPLE_RATE, sampleRate),
            3, "Sample Rate");

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_TRANSIENT_MEASUREMENT_TIME, measurementTime),
            4, "Measurement Time");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_TRANSIENT_RECORD_LENGTH, recordLength),
            5, "Record Length");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures analysis region.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR analysisBandwidth/This control defines the analysis region's bandwidth.
/// HIPAR deltaFrequency/Defines the center of the frequency span for the analysis region. It
/// HIPAR deltaFrequency/is defined as an offset from the center frequency.
/// HIPAR timeGateLength/Defines the length of the time gate, that is, the duration (or height)
/// HIPAR timeGateLength/of the analysis region.
/// HIPAR timeGateStart/Defines the starting point of the time span for the analysis region.
/// HIPAR timeGateStart/The starting point is defined as a time offset from the capture start
/// HIPAR timeGateStart/time.
ViStatus _VI_FUNC rsspecan_ConfigureTransientAnalysisRegion(ViSession instrSession,
                                                            ViReal64  analysisBandwidth,
                                                            ViReal64  deltaFrequency,
                                                            ViReal64  timeGateLength,
                                                            ViReal64  timeGateStart)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_TRANSIENT_ANALYSIS_BANDWIDTH, analysisBandwidth),
            2, "Analysis Bandwidth");

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_TRANSIENT_DELTA_FREQUENCY, deltaFrequency),
            3, "Delta Frequency");

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_TRANSIENT_TIME_GATE_LENGTH, timeGateLength),
            4, "Time Gate Length");

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_TRANSIENT_TIME_GATE_START, timeGateStart),
            5, "Time Gate Start");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures link to full settings.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR bandwidthPercent/This control sets the width of the frequency span for the analysis
/// HIPAR bandwidthPercent/region is defined as a percentage of the full capture buffer.
/// HIPAR bandwidthState/If activated, the width of the frequency span for the analysis region
/// HIPAR bandwidthState/is defined as a percentage of the full capture buffer ("Bandwidth
/// HIPAR bandwidthState/Percent").
/// HIPAR timePercent/This control sets the length of the time gate, that is, the duration
/// HIPAR timePercent/(or height) of the analysis region, is defined as a percentage of the
/// HIPAR timePercent/full measurement time.
/// HIPAR timeState/If activated, the length of the time gate, that is, the duration (or
/// HIPAR timeState/height) of the analysis region, is defined as a percentage of the full
/// HIPAR timeState/measurement time.
ViStatus _VI_FUNC rsspecan_ConfigureTransientAnalysisLinkToFull(
                            ViSession instrSession,
                            ViBoolean bandwidthState,
                            ViReal64  bandwidthPercent,
                            ViBoolean timeState,
                            ViReal64  timePercent)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_TRANSIENT_LINK_BANDWIDTH_ENABLED, bandwidthState),
            2, "Bandwidth State");
    if (bandwidthState)
    {
        viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_TRANSIENT_LINK_BANDWIDTH, bandwidthPercent),
                3, "Bandwidth Percent");
    }

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_TRANSIENT_LINK_TIME_ENABLED, timeState),
            4, "Time State");
    if (timeState)
    {
        viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_TRANSIENT_LINK_TIME, timePercent),
                5, "Time Percent");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function sets the FFT windowing function..
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR fftWindowType/Sets the FFT windowing function..
ViStatus _VI_FUNC rsspecan_ConfigureTransientAnalysisFFTWindow(
                            ViSession instrSession,
                            ViInt32   fftWindowType)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_TRANSIENT_FFT_WINDOW, fftWindowType),
            2, "fft Window Type");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Activates or deactivates additional filters applied after demodulation
/// HIFN  to filter out unwanted signals, or correct pre-emphasized input
/// HIFN  signals.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR fmVideoBandwidth/Activates or deactivates additional filters applied after demodulation
/// HIPAR fmVideoBandwidth/to filter out unwanted signals, or correct pre-emphasized input
/// HIPAR fmVideoBandwidth/signals.
ViStatus _VI_FUNC rsspecan_ConfigureTransientAnalysisFMVideoBandwidth(
                            ViSession instrSession,
                            ViInt32   fmVideoBandwidth)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_TRANSIENT_FM_VIDEO_BANDWIDTH, fmVideoBandwidth),
            2, "FM Video Bandwidth");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures time resolution.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR timeResolution/This control sets the spectrogram time resolution, Auto must be set to
/// HIPAR timeResolution/manual.
/// HIPAR autoMode/This control switches the spectrogram time resolution from auto to
/// HIPAR autoMode/manual.
ViStatus _VI_FUNC rsspecan_ConfigureTransientAnalysisTimeResolution(
                            ViSession instrSession,
                            ViInt32   autoMode,
                            ViReal64  timeResolution)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_TRANSIENT_TIME_RESOLUTION_AUTO, autoMode),
            2, "Auto Mode");

    if (autoMode == RSSPECAN_VAL_MAN)
    {
        viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_TRANSIENT_TIME_RESOLUTION, timeResolution),
                3, "Time Resolution");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the chirp detection mode.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR enabled/Sets the chirp detection mode.
ViStatus _VI_FUNC rsspecan_ConfigureTransientAnalysisChirpDetectionEnabled(
                            ViSession instrSession,
                            ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, NULL, RSSPECAN_ATTR_TRANSIENT_CHIRP_DETECTION_ENABLED, enabled),
            2, "Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures "Frequency" settings. The "Frequency"
/// HIFN  settings determine which part of the hop/chirp is used to calculate the
/// HIFN  average frequency in one hop/chirp.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR model/This control defines which type of signal to expect (if known), thus
/// HIPAR model/determining the analysis method.
/// HIPAR reference/Defines the reference point for positioning the frequency measurement
/// HIPAR reference/range.
/// HIPAR length/Defines the length or duration of the frequency measurement range.
/// HIPAR offsetBegin/The offset in seconds from the beginning of the Reference. Only
/// HIPAR offsetBegin/available for Reference set to Edge.
/// HIPAR offsetEnd/The offset in seconds from the end of the Reference. Only available
/// HIPAR offsetEnd/for Reference set to Edge.
ViStatus _VI_FUNC rsspecan_ConfigureTransientAnalysisHopChirpFrequency(
                            ViSession instrSession,
                            ViInt32   model,
                            ViInt32   reference,
                            ViReal64  length,
                            ViReal64  offsetBegin,
                            ViReal64  offsetEnd)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    if (model == RSSPECAN_VAL_TRANSIENT_MODEL_HOP)
    {
        viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_TRANSIENT_HOP_FREQUENCY_REFERENCE, reference),
                3, "reference");

        if (reference == RSSPECAN_VAL_TRANSIENT_REFERENCE_CENTER)
        {
            viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_TRANSIENT_HOP_FREQUENCY_LENGTH, length),
                    4, "Length");
        }
        else if (reference == RSSPECAN_VAL_TRANSIENT_REFERENCE_EDGE)
        {
            viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_TRANSIENT_HOP_FREQUENCY_OFFSET_BEGIN, offsetBegin),
                    5, "Offset Begin");

            viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_TRANSIENT_HOP_FREQUENCY_OFFSET_END, offsetEnd),
                    6, "Offset End");
        }
    }
    else
    {
        viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_TRANSIENT_CHIRP_FREQUENCY_REFERENCE, reference),
                3, "reference");

        if (reference == RSSPECAN_VAL_TRANSIENT_REFERENCE_CENTER)
        {
            viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_TRANSIENT_CHIRP_FREQUENCY_LENGTH, length),
                    4, "Length");
        }
        else if (reference == RSSPECAN_VAL_TRANSIENT_REFERENCE_EDGE)
        {
            viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_TRANSIENT_CHIRP_FREQUENCY_OFFSET_BEGIN, offsetBegin),
                    5, "Offset Begin");

            viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_TRANSIENT_CHIRP_FREQUENCY_OFFSET_END, offsetEnd),
                    6, "Offset End");
        }
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures "Power" settings. The "Power" settings
/// HIFN  determine which part of the hop/chirp is used to calculate the average
/// HIFN  frequency in one hop/chirp.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR model/This control defines which type of signal to expect (if known), thus
/// HIPAR model/determining the analysis method.
/// HIPAR reference/Defines the reference point for positioning the power measurement
/// HIPAR reference/range.
/// HIPAR length/Defines the length or duration of the power measurement range.
/// HIPAR offsetBegin/The offset in seconds from the beginning of the Reference. Only
/// HIPAR offsetBegin/available for Reference set to Edge.
/// HIPAR offsetEnd/The offset in seconds from the end of the Reference. Only available
/// HIPAR offsetEnd/for Reference set to Edge.
ViStatus _VI_FUNC rsspecan_ConfigureTransientAnalysisHopChirpPower(
                            ViSession instrSession,
                            ViInt32   model,
                            ViInt32   reference,
                            ViReal64  length,
                            ViReal64  offsetBegin,
                            ViReal64  offsetEnd)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    if (model == RSSPECAN_VAL_TRANSIENT_MODEL_HOP)
    {
        viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_TRANSIENT_HOP_POWER_REFERENCE, reference),
                3, "reference");

        if (reference == RSSPECAN_VAL_TRANSIENT_REFERENCE_CENTER)
        {
            viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_TRANSIENT_HOP_POWER_LENGTH, length),
                    4, "Length");
        }
        else if (reference == RSSPECAN_VAL_TRANSIENT_REFERENCE_EDGE)
        {
            viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_TRANSIENT_HOP_POWER_OFFSET_BEGIN, offsetBegin),
                    5, "Offset Begin");

            viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_TRANSIENT_HOP_POWER_OFFSET_END, offsetEnd),
                    6, "Offset End");
        }
    }
    else
    {
        viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_TRANSIENT_CHIRP_POWER_REFERENCE, reference),
                3, "reference");

        if (reference == RSSPECAN_VAL_TRANSIENT_REFERENCE_CENTER)
        {
            viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_TRANSIENT_CHIRP_POWER_LENGTH, length),
                    4, "Length");
        }
        else if (reference == RSSPECAN_VAL_TRANSIENT_REFERENCE_EDGE)
        {
            viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_TRANSIENT_CHIRP_POWER_OFFSET_BEGIN, offsetBegin),
                    5, "Offset Begin");

            viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_TRANSIENT_CHIRP_POWER_OFFSET_END, offsetEnd),
                    6, "Offset End");
        }
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures "Frequency Deviation" settings. The
/// HIFN  "Frequency Deviation" settings determine which part of the hop/chirp is
/// HIFN  used to calculate the average frequency in one hop/chirp.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR model/This control defines which type of signal to expect (if known), thus
/// HIPAR model/determining the analysis method.
/// HIPAR reference/Defines the reference point for positioning the frequency deviation
/// HIPAR reference/measurement range. The Offset Begin / Offset End is given with respect
/// HIPAR reference/to this value.
/// HIPAR length/Defines the length or duration of the frequency deviation measurement
/// HIPAR length/range. Only available for Reference set to Center.
/// HIPAR offsetBegin/The offset in seconds from the beginning of the Reference. Only
/// HIPAR offsetBegin/available for Reference set to Edge.
/// HIPAR offsetEnd/The offset in seconds from the end of the Reference. Only available
/// HIPAR offsetEnd/for Reference set to Edge.
ViStatus _VI_FUNC rsspecan_ConfigureTransientAnalysisHopChirpFrequencyDeviation(
                            ViSession instrSession,
                            ViInt32   model,
                            ViInt32   reference,
                            ViReal64  length,
                            ViReal64  offsetBegin,
                            ViReal64  offsetEnd)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    if (model == RSSPECAN_VAL_TRANSIENT_MODEL_HOP)
    {
        viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_TRANSIENT_HOP_FREQUENCY_DEVIATION_REFERENCE, reference),
                3, "reference");

        if (reference == RSSPECAN_VAL_TRANSIENT_REFERENCE_CENTER)
        {
            viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_TRANSIENT_HOP_FREQUENCY_DEVIATION_LENGTH, length),
                    4, "Length");
        }
        else if (reference == RSSPECAN_VAL_TRANSIENT_REFERENCE_EDGE)
        {
            viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_TRANSIENT_HOP_FREQUENCY_DEVIATION_OFFSET_BEGIN, offsetBegin),
                    5, "Offset Begin");

            viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_TRANSIENT_HOP_FREQUENCY_DEVIATION_OFFSET_END, offsetEnd),
                    6, "Offset End");
        }
    }
    else
    {
        viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_TRANSIENT_CHIRP_FREQUENCY_DEVIATION_REFERENCE, reference),
                3, "reference");

        if (reference == RSSPECAN_VAL_TRANSIENT_REFERENCE_CENTER)
        {
            viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_TRANSIENT_CHIRP_FREQUENCY_DEVIATION_LENGTH, length),
                    4, "Length");
        }
        else if (reference == RSSPECAN_VAL_TRANSIENT_REFERENCE_EDGE)
        {
            viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_TRANSIENT_CHIRP_FREQUENCY_DEVIATION_OFFSET_BEGIN, offsetBegin),
                    5, "Offset Begin");

            viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_TRANSIENT_CHIRP_FREQUENCY_DEVIATION_OFFSET_END, offsetEnd),
                    6, "Offset End");
        }
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures "Phase Deviation" settings. The "Phase
/// HIFN  Deviation" settings determine which part of the hop/chirp is used to
/// HIFN  calculate the average frequency in one hop/chirp.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR model/This control defines which type of signal to expect (if known), thus
/// HIPAR model/determining the analysis method.
/// HIPAR reference/Defines the reference point for positioning the phase deviation
/// HIPAR reference/measurement range. The Offset Begin / Offset End is given with respect
/// HIPAR reference/to this value.
/// HIPAR length/Defines the length or duration of the phase deviation measurement
/// HIPAR length/range. Only available for Reference set to Center.
/// HIPAR offsetBegin/The offset in seconds from the beginning of the Reference. Only
/// HIPAR offsetBegin/available for Reference set to Edge.
/// HIPAR offsetEnd/The offset in seconds from the end of the Reference. Only available
/// HIPAR offsetEnd/for Reference set to Edge.
ViStatus _VI_FUNC rsspecan_ConfigureTransientAnalysisHopChirpPhaseDeviation(
                            ViSession instrSession,
                            ViInt32   model,
                            ViInt32   reference,
                            ViReal64  length,
                            ViReal64  offsetBegin,
                            ViReal64  offsetEnd)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    if (model == RSSPECAN_VAL_TRANSIENT_MODEL_HOP)
    {
        viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_TRANSIENT_HOP_PHASE_DEVIATION_REFERENCE, reference),
                3, "reference");

        if (reference == RSSPECAN_VAL_TRANSIENT_REFERENCE_CENTER)
        {
            viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_TRANSIENT_HOP_PHASE_DEVIATION_LENGTH, length),
                    4, "Length");
        }
        else if (reference == RSSPECAN_VAL_TRANSIENT_REFERENCE_EDGE)
        {
            viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_TRANSIENT_HOP_PHASE_DEVIATION_OFFSET_BEGIN, offsetBegin),
                    5, "Offset Begin");

            viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_TRANSIENT_HOP_PHASE_DEVIATION_OFFSET_END, offsetEnd),
                    6, "Offset End");
        }
    }
    else
    {
        viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_TRANSIENT_CHIRP_PHASE_DEVIATION_REFERENCE, reference),
                3, "reference");

        if (reference == RSSPECAN_VAL_TRANSIENT_REFERENCE_CENTER)
        {
            viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_TRANSIENT_CHIRP_PHASE_DEVIATION_LENGTH, length),
                    4, "Length");
        }
        else if (reference == RSSPECAN_VAL_TRANSIENT_REFERENCE_EDGE)
        {
            viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_TRANSIENT_CHIRP_PHASE_DEVIATION_OFFSET_BEGIN, offsetBegin),
                    5, "Offset Begin");

            viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_TRANSIENT_CHIRP_PHASE_DEVIATION_OFFSET_END, offsetEnd),
                    6, "Offset End");
        }
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  If activated, the calculated hop frequency is compensated for when
/// HIFN  determining the measurement range for frequency parameters.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR compensateHopFrequencyDev/If activated, the calculated hop frequency is compensated for when
/// HIPAR compensateHopFrequencyDev/determining the measurement range for frequency parameters.
ViStatus _VI_FUNC rsspecan_ConfigureTransientAnalysisCompensateHopFrequencyDeviation(
                            ViSession instrSession,
                            ViBoolean compensateHopFrequencyDev)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_TRANSIENT_HOP_COMPENSATE_HOP_FREQUENCY_DEVIATION, compensateHopFrequencyDev),
            2, "Compensate Hop Frequency Dev");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  If activated, the calculated chirp rate error is compensated for when
/// HIFN  determining the measurement range for frequency parameters.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR compensateChirpRateDeviation/If activated, the calculated chirp rate error is compensated for when
/// HIPAR compensateChirpRateDeviation/determining the measurement range for frequency parameters.
ViStatus _VI_FUNC rsspecan_ConfigureTransientAnalysisCompensateChirpRateDeviation(
                            ViSession instrSession,
                            ViBoolean compensateChirpRateDeviation)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_TRANSIENT_CHIRP_COMPENSATE_CHIRP_RATE_DEVIATION, compensateChirpRateDeviation),
            2, "Compensate Chirp Rate Deviation");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the allowed deviation from the detected FM or PM signal model
/// HIFN  state where the hop/chirp is considered as 'settled'.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR model/This control defines which type of signal to expect (if known), thus
/// HIPAR model/determining the analysis method.
/// HIPAR signalModelState/Selects the FM or PM signal model state.
/// HIPAR settlingTolerance/Defines the allowed deviation from the detected FM or PM signal model
/// HIPAR settlingTolerance/state where the hop/chirp is considered as 'settled'.
ViStatus _VI_FUNC rsspecan_ConfigureTransientAnalysisHopChirpSettlingTolerance(
                            ViSession instrSession,
                            ViInt32   model,
                            ViInt32   signalModelState,
                            ViReal64  settlingTolerance)
{
    ViStatus error = VI_SUCCESS;
    ViAttr   attr[2][2] = {
             {
             RSSPECAN_ATTR_TRANSIENT_HOP_FM_SETTLING_TOLERANCE,
             RSSPECAN_ATTR_TRANSIENT_HOP_PM_SETTLING_TOLERANCE
             },
             {
             RSSPECAN_ATTR_TRANSIENT_CHIRP_FM_SETTLING_TOLERANCE,
             RSSPECAN_ATTR_TRANSIENT_CHIRP_PM_SETTLING_TOLERANCE
             }
             };

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, model, RSSPECAN_VAL_TRANSIENT_MODEL_HOP, RSSPECAN_VAL_TRANSIENT_MODEL_CHIRP),
            2, "Model");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, signalModelState, RSSPECAN_VAL_TRANSIENT_FM, RSSPECAN_VAL_TRANSIENT_PM),
            3, "Signal Model State");

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, NULL, attr[model - 1][signalModelState], settlingTolerance), 4, "Settling Tolerance");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures phase noise "Frequency" settings. The
/// HIFN  "Frequency" settings determine which part of the hop/chirp is used to
/// HIFN  calculate the average frequency in one hop/chirp.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR model/This control defines which type of signal to expect (if known), thus
/// HIPAR model/determining the analysis method.
/// HIPAR reference/Defines the reference point for positioning the phase noise
/// HIPAR reference/measurement range.
/// HIPAR length/Defines the length or duration of the frequency measurement range.
/// HIPAR length/Only available for Reference set to Center.
/// HIPAR offsetBegin/The offset in seconds from the beginning of the Reference. Only
/// HIPAR offsetBegin/available for Reference set to Edge.
/// HIPAR offsetEnd/The offset in seconds from the end of the Reference. Only available
/// HIPAR offsetEnd/for Reference set to Edge.
/// HIPAR start/Sets the start frequency for the phase noise measurement.
/// HIPAR stop/Sets the stop frequency for the phase noise measurement.
ViStatus _VI_FUNC rsspecan_ConfigureTransientAnalysisHopChirpPhaseNoiseFrequency(
                            ViSession instrSession,
                            ViInt32   model,
                            ViInt32   reference,
                            ViReal64  length,
                            ViReal64  offsetBegin,
                            ViReal64  offsetEnd,
                            ViReal64  start,
                            ViReal64  stop)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    if (model == RSSPECAN_VAL_TRANSIENT_MODEL_HOP)
    {
        viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_TRANSIENT_HOP_PHASE_NOISE_REFERENCE, reference),
                3, "reference");

        if (reference == RSSPECAN_VAL_TRANSIENT_REFERENCE_CENTER)
        {
            viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_TRANSIENT_HOP_PHASE_NOISE_LENGTH, length),
                    4, "Length");
        }
        else if (reference == RSSPECAN_VAL_TRANSIENT_REFERENCE_EDGE)
        {
            viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_TRANSIENT_HOP_PHASE_NOISE_OFFSET_BEGIN, offsetBegin),
                    5, "Offset Begin");

            viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_TRANSIENT_HOP_PHASE_NOISE_OFFSET_END, offsetEnd),
                    6, "Offset End");
        }
        viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_TRANSIENT_HOP_PHASE_NOISE_FREQUENCY_START, start),
                7, "Start");

        viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_TRANSIENT_HOP_PHASE_NOISE_FREQUENCY_STOP, stop),
                8, "Stop");
    }
    else
    {
        viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_TRANSIENT_CHIRP_PHASE_NOISE_REFERENCE, reference),
                3, "reference");

        if (reference == RSSPECAN_VAL_TRANSIENT_REFERENCE_CENTER)
        {
            viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_TRANSIENT_CHIRP_PHASE_NOISE_LENGTH, length),
                    4, "Length");
        }
        else if (reference == RSSPECAN_VAL_TRANSIENT_REFERENCE_EDGE)
        {
            viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_TRANSIENT_CHIRP_PHASE_NOISE_OFFSET_BEGIN, offsetBegin),
                    5, "Offset Begin");

            viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_TRANSIENT_CHIRP_PHASE_NOISE_OFFSET_END, offsetEnd),
                    6, "Offset End");
        }

        viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_TRANSIENT_CHIRP_PHASE_NOISE_FREQUENCY_START, start),
                7, "Start");

        viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_TRANSIENT_CHIRP_PHASE_NOISE_FREQUENCY_STOP, stop),
                8, "Stop");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function restarts a (single) measurement that has been stopped.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
ViStatus _VI_FUNC rsspecan_TransientAnalysisContinueSingleSweep(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_TRANSIENT_CONTINUE_SINGLE_SWEEP, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function requests the R&S FSW Transient Analysis application
/// HIFN  option to wait until any auto-refresh operation is finished. However,
/// HIFN  it does not initiate a refresh itself.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
ViStatus _VI_FUNC rsspecan_TransienAnalysisWaitForEndOfAutoRefresh(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_TRANSIENT_WAIT_FOR_END_OF_AUTO_REFRESH, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function queries the FFT window length.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR length/Queries the queries the FFT window length.
ViStatus _VI_FUNC rsspecan_QueryTransientAnalysisFFTWindowLength(
                            ViSession instrSession,
                            ViInt32*  length)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_GetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_TRANSIENT_FFT_WINDOW_LENGTH, length),
            2, "Length");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function adds a window to the display.
/// HIFN
/// HIFN     Note(s):
/// HIFN     (1) This function is only available for R&S FSW.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR stringSize/This control sets the allocated length of Returned Window Name string.
/// HIPAR referenceWindowName/String containing the number or name of the window the new window is
/// HIPAR referenceWindowName/inserted next to. In the default state, the name of the window is its
/// HIPAR referenceWindowName/index.
/// HIPAR direction/Direction the window is added relative to the window set with "Reference Window
/// HIPAR direction/Name".
/// HIPAR windowType/Type of result display you want to add.
/// HIPAR returnedWindowName/When adding a new window, the function returns its name (by
ViStatus _VI_FUNC rsspecan_TransientAnalysisAddWindow(ViSession instrSession,
                                                      ViInt32   stringSize,
                                                      ViString  referenceWindowName,
                                                      ViInt32   direction,
                                                      ViInt32   windowType,
                                                      ViChar    returnedWindowName[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViChar   response[RS_MAX_MESSAGE_BUF_SIZE];
    ViChar*  p2direction;
    ViChar*  p2type;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &rsspecan_rngLayoutDirection, direction, &p2direction),
                4, "Direction");
    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &rsspecan_rngTransientLayoutType, windowType, &p2type),
                5, "Window Type");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "LAY:ADD? '%s',%s,%s", referenceWindowName, p2direction, p2type);
    checkErr(RsCore_QueryViString(instrSession, cmd, response));

    if (returnedWindowName != NULL)
    {
        RsCore_TrimString(response, RS_VAL_TRIM_WHITESPACES_AND_ALL_QUOTES);
        RsCore_CopyToUserBufferAsciiData(returnedWindowName, stringSize, response);
    }

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function replaces a window with another one.
/// HIFN
/// HIFN     Note(s):
/// HIFN     (1) This function is only available for R&S FSW.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR windowName/String containing the number or name of the window you want to
/// HIPAR windowType/Type of result display you want to add.
ViStatus _VI_FUNC rsspecan_TransientAnalysisReplaceWindow(ViSession instrSession,
                                                          ViString  windowName,
                                                          ViInt32   windowType)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViChar*  p2type;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &rsspecan_rngTransientLayoutType, windowType, &p2type),
                3, "Window Type");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "LAY:REPL '%s',%s", windowName, p2type);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function defines whether the result range length is determined
/// HIFN  automatically according to the width of the selected hop/chirp.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR autoMode/This control defines whether the result range length is determined
/// HIPAR autoMode/automatically according to the width of the selected hop/chirp.
ViStatus _VI_FUNC rsspecan_ConfigureTransientAnalysisResultAutomaticRangeScaling(
                            ViSession instrSession,
                            ViBoolean autoMode)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_TRANSIENT_RESULT_AUTOMATIC_RANGE_SCALING, autoMode),
            2, "Auto Mode");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the transient refernece point and offset.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR reference/Specifies the reference point used to define the result range.
/// HIPAR offset/The offset (in seconds) from the reference point at which the
/// HIPAR offset/transient result range is aligned.
ViStatus _VI_FUNC rsspecan_ConfigureTransientAnalysisResultReferencePoint(
                            ViSession instrSession,
                            ViInt32   reference,
                            ViReal64  offset)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_TRANSIENT_RESULT_RANGE_REFERENCE, reference),
            2, "Reference");

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_TRANSIENT_RESULT_RANGE_OFFSET, offset),
            2, "Offset");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the transient result range.
/// HIFN     The result range determines which data is displayed on the screen.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR alignment/Specifies the alignment with respect to the reference point used to
/// HIPAR alignment/define the result range.
/// HIPAR length/The length of the transient result range (in seconds).
ViStatus _VI_FUNC rsspecan_ConfigureTransientAnalysisResultRange(
                            ViSession instrSession,
                            ViInt32   alignment,
                            ViReal64  length)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_TRANSIENT_RESULT_RANGE_ALIGNMENT, alignment),
            2, "Alignment");

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_TRANSIENT_RESULT_RANGE_LENGTH, length),
            2, "Length");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function enables or disables columns in all hop results and
/// HIFN  statistics tables.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR header/This control selects the header.
/// HIPAR visibility/This control enables or disables columns in all hop results and
/// HIPAR visibility/statistics tables.
ViStatus _VI_FUNC rsspecan_ConfigureTransientAnalysisHopResultTable(
                            ViSession instrSession,
                            ViInt32   header,
                            ViBoolean visibility)
{
    ViStatus error = VI_SUCCESS;
    ViChar*  p2string;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K60"));

    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &rsspecan_rngTransientHopHeader, header, &p2string),
                2, "Parameter");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CALC:HOPD:TABL:COL %s,%s", (visibility ? "ON" : "OFF"), p2string);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function shows the frequency parameter in result table and sets
/// HIFN  it's scaling.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR parameter/This control selects the frequency parameter.
/// HIPAR visibility/Shows / hides the parameter in the result table.
/// HIPAR scaling/Defines the unit in which the results are displayed.
ViStatus _VI_FUNC rsspecan_ConfigureTransientAnalysisHopResultTableFrequency(
                            ViSession instrSession,
                            ViInt32   parameter,
                            ViBoolean visibility,
                            ViInt32   scaling)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, parameter, RSSPECAN_VAL_TRANSIENT_RESULT_FREQ_ALL, RSSPECAN_VAL_TRANSIENT_RESULT_FREQ_RELF),
            2, "Parameter");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, scaling, RSSPECAN_VAL_TRANSIENT_FREQ_SCALING_HZ, RSSPECAN_VAL_TRANSIENT_FREQ_SCALING_GHZ),
            4, "Scaling");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CALC:HOPD:TABL:FREQ:%s %s,%s", paramArr[parameter], visibility ? "ON" : "OFF", freqScalingArr[scaling]);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function shows the power parameter in result table.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR parameter/This control selects the power parameter.
/// HIPAR visibility/Shows / hides the parameter in the result table.
ViStatus _VI_FUNC rsspecan_ConfigureTransientAnalysisHopResultTablePower(
                            ViSession instrSession,
                            ViInt32   parameter,
                            ViBoolean visibility)
{
    ViStatus error = VI_SUCCESS;
    ViAttr   attr[5] = {
             RSSPECAN_ATTR_TRANSIENT_HOP_RESULT_TABLE_POWER_ALL,
             RSSPECAN_ATTR_TRANSIENT_HOP_RESULT_TABLE_POWER_AVERAGE,
             RSSPECAN_ATTR_TRANSIENT_HOP_RESULT_TABLE_POWER_MAXIMUM,
             RSSPECAN_ATTR_TRANSIENT_HOP_RESULT_TABLE_POWER_MINIMUM,
             RSSPECAN_ATTR_TRANSIENT_HOP_RESULT_TABLE_POWER_RIPPLE
             };

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, parameter, RSSPECAN_VAL_TRANSIENT_RESULT_POWER_ALL, RSSPECAN_VAL_TRANSIENT_RESULT_POWER_RIPPLE),
            2, "Parameter");

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", attr[parameter], visibility),
            3, "Visibility");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function shows the state parameter in result table.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR parameter/This control selects the state parameter.
/// HIPAR visibility/Shows / hides the parameter in the result table.
/// HIPAR scaling/Defines the unit in which the results are displayed.
ViStatus _VI_FUNC rsspecan_ConfigureTransientAnalysisHopResultTableState(
                            ViSession instrSession,
                            ViInt32   parameter,
                            ViBoolean visibility,
                            ViInt32   scaling)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViAttr   attr[2] = {
             RSSPECAN_ATTR_TRANSIENT_HOP_RESULT_TABLE_STATE_ALL,
             RSSPECAN_ATTR_TRANSIENT_HOP_RESULT_TABLE_STATE_INDEX
             };

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, parameter, RSSPECAN_VAL_TRANSIENT_RESULT_STATE_ALL, RSSPECAN_VAL_TRANSIENT_RESULT_STATE_FREQ),
            2, "Parameter");

    if (parameter == RSSPECAN_VAL_TRANSIENT_RESULT_STATE_FREQ)
    {
        viCheckParm(RsCore_InvalidViInt32Range(instrSession, scaling, RSSPECAN_VAL_TRANSIENT_FREQ_SCALING_HZ, RSSPECAN_VAL_TRANSIENT_FREQ_SCALING_GHZ),
                4, "Scaling");

        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CALC:HOPD:TABL:STAT:STAF %s,%s", visibility ? "ON" : "OFF", freqScalingArr[scaling]);
        checkErr(RsCore_Write(instrSession, cmd));

        checkErr(rsspecan_CheckStatus(instrSession));
    }
    else
    {
        viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", attr[parameter], visibility),
                3, "Visibility");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function shows the timing parameter in result table and sets it's
/// HIFN  scaling.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR parameter/This control selects the timing parameter.
/// HIPAR visibility/Shows / hides the parameter in the result table.
/// HIPAR scaling/Defines the unit in which the results are displayed.
ViStatus _VI_FUNC rsspecan_ConfigureTransientAnalysisHopResultTableTiming(
                            ViSession instrSession,
                            ViInt32   parameter,
                            ViBoolean visibility,
                            ViInt32   scaling)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, parameter, RSSPECAN_VAL_TRANSIENT_RESULT_TIMING_ALL, RSSPECAN_VAL_TRANSIENT_RESULT_TIMING_SWITCHING),
            2, "Parameter");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, scaling, RSSPECAN_VAL_TRANSIENT_TIME_SCALING_NS_HZ, RSSPECAN_VAL_TRANSIENT_TIME_SCALING_S_GHZ),
            4, "Scaling");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CALC:HOPD:TABL:TIM:%s %s,%s", paramTimingArr[parameter], visibility ? "ON" : "OFF", timingScalingArr[scaling]);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function shows the phase deviation parameter in result table and sets it's
/// HIFN  scaling.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR parameter/This control selects the timing parameter.
/// HIPAR visibility/Shows / hides the parameter in the result table.
/// HIPAR scaling/Defines the unit in which the results are displayed.
ViStatus _VI_FUNC rsspecan_ConfigureTransientAnalysisHopResultTablePhase(
                            ViSession instrSession,
                            ViInt32   parameter,
                            ViBoolean visibility,
                            ViInt32   scaling)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViChar*  p2param;
    ViChar*  p2scaling;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K60"));

    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &RsSpecAn_rngTransientResultPhase, parameter, &p2param),
                2, "Parameter");
    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &rsspecan_rngTransientResultPhaseScaling, scaling, &p2scaling),
                4, "Scaling");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CALC:HOPD:TABL:PHAS:%s %s,%s", p2param, visibility ? "ON" : "OFF", p2scaling);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function shows the FM settling parameter in result table and sets it's scaling.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR parameter/This control selects the frequency parameter.
/// HIPAR visibility/Shows / hides the parameter in the result table.
/// HIPAR scaling/Defines the unit in which the results are displayed.
ViStatus _VI_FUNC rsspecan_ConfigureTransientAnalysisHopResultTableFMSettling(
                            ViSession instrSession,
                            ViInt32   parameter,
                            ViBoolean visibility,
                            ViInt32   scaling)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, parameter, RSSPECAN_VAL_TRANSIENT_RESULT_SETTLING_ALL, RSSPECAN_VAL_TRANSIENT_RESULT_SETTLING_TIME),
            2, "Parameter");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, scaling, RSSPECAN_VAL_TRANSIENT_TIME_SCALING_NS_HZ, RSSPECAN_VAL_TRANSIENT_TIME_SCALING_S_GHZ),
            4, "Scaling");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CALC:HOPD:TABL:FMS:%s %s,%s", RsSpecAn_rngTransientResultFMSettling.entries[parameter].cmdString, visibility ? "ON" : "OFF", rsspecan_rngTransientResultTimeScaling.entries[scaling].cmdString);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function shows the PM settling parameter in result table and sets it's scaling.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR parameter/This control selects the frequency parameter.
/// HIPAR visibility/Shows / hides the parameter in the result table.
/// HIPAR scaling/Defines the unit in which the results are displayed.
ViStatus _VI_FUNC rsspecan_ConfigureTransientAnalysisHopResultTablePMSettling(
                            ViSession instrSession,
                            ViInt32   parameter,
                            ViBoolean visibility,
                            ViInt32   scaling)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, parameter, RSSPECAN_VAL_TRANSIENT_RESULT_SETTLING_ALL, RSSPECAN_VAL_TRANSIENT_RESULT_SETTLING_TIME),
            2, "Parameter");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, scaling, RSSPECAN_VAL_TRANSIENT_TIME_SCALING_NS_HZ, RSSPECAN_VAL_TRANSIENT_TIME_SCALING_S_GHZ),
            4, "Scaling");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CALC:HOPD:TABL:PMS:%s %s,%s", RsSpecAn_rngTransientResultPMSettling.entries[parameter].cmdString, visibility ? "ON" : "OFF", rsspecan_rngTransientResultTimeScaling.entries[scaling].cmdString);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function enables or disables columns in all hop results and
/// HIFN  statistics tables.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR header/This control selects the header.
/// HIPAR visibility/This control enables or disables columns in all hop results and
/// HIPAR visibility/statistics tables.
ViStatus _VI_FUNC rsspecan_ConfigureTransientAnalysisChirpResultTable(
                            ViSession instrSession,
                            ViInt32   header,
                            ViBoolean visibility)
{
    ViStatus error = VI_SUCCESS;
    ViChar*  p2string;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K60"));

    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &rsspecan_rngTransientChirpHeader, header, &p2string),
                2, "Parameter");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CALC:CHRD:TABL:COL %s,%s", (visibility ? "ON" : "OFF"), p2string);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function shows the frequency parameter in result table and sets
/// HIFN  it's scaling.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR parameter/This control selects the frequency parameter.
/// HIPAR visibility/Shows / hides the parameter in the result table.
/// HIPAR scaling/Defines the unit in which the results are displayed.
ViStatus _VI_FUNC rsspecan_ConfigureTransientAnalysisChirpResultTableFrequency(
                            ViSession instrSession,
                            ViInt32   parameter,
                            ViBoolean visibility,
                            ViInt32   scaling)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, parameter, RSSPECAN_VAL_TRANSIENT_RESULT_FREQ_ALL, RSSPECAN_VAL_TRANSIENT_RESULT_FREQ_UNDER),
            2, "Parameter");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, scaling, RSSPECAN_VAL_TRANSIENT_FREQ_SCALING_HZ, RSSPECAN_VAL_TRANSIENT_FREQ_SCALING_GHZ),
            4, "Scaling");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CALC:CHRD:TABL:FREQ:%s %s,%s", paramChirpArr[parameter], visibility ? "ON" : "OFF",
             (parameter != RSSPECAN_VAL_TRANSIENT_RESULT_FREQ_CHER) ? freqScalingArr[scaling] : freqScalingCherArr[scaling]);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function shows the power parameter in result table.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR parameter/This control selects the power parameter.
/// HIPAR visibility/Shows / hides the parameter in the result table.
ViStatus _VI_FUNC rsspecan_ConfigureTransientAnalysisChirpResultTablePower(
                            ViSession instrSession,
                            ViInt32   parameter,
                            ViBoolean visibility)
{
    ViStatus error = VI_SUCCESS;
    ViAttr   attr[5] = {
             RSSPECAN_ATTR_TRANSIENT_CHIRP_RESULT_TABLE_POWER_ALL,
             RSSPECAN_ATTR_TRANSIENT_CHIRP_RESULT_TABLE_POWER_AVERAGE,
             RSSPECAN_ATTR_TRANSIENT_CHIRP_RESULT_TABLE_POWER_MAXIMUM,
             RSSPECAN_ATTR_TRANSIENT_CHIRP_RESULT_TABLE_POWER_MINIMUM,
             RSSPECAN_ATTR_TRANSIENT_CHIRP_RESULT_TABLE_POWER_RIPPLE
             };

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, parameter, RSSPECAN_VAL_TRANSIENT_RESULT_POWER_ALL, RSSPECAN_VAL_TRANSIENT_RESULT_POWER_RIPPLE),
            2, "Parameter");

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", attr[parameter], visibility),
            3, "Visibility");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function shows the state parameter in result table.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR parameter/This control selects the state parameter.
/// HIPAR visibility/Shows / hides the parameter in the result table.
/// HIPAR reserved/This control is reserved for future use.
ViStatus _VI_FUNC rsspecan_ConfigureTransientAnalysisChirpResultTableState(
                            ViSession instrSession,
                            ViInt32   parameter,
                            ViBoolean visibility,
                            ViInt32   reserved)
{
    ViStatus error = VI_SUCCESS;
    ViAttr   attr[2] = {RSSPECAN_ATTR_TRANSIENT_CHIRP_RESULT_TABLE_STATE_ALL, RSSPECAN_ATTR_TRANSIENT_CHIRP_RESULT_TABLE_STATE_INDEX};
             reserved;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, parameter, RSSPECAN_VAL_TRANSIENT_RESULT_STATE_ALL, RSSPECAN_VAL_TRANSIENT_RESULT_STATE_INDEX),
            2, "Parameter");

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", attr[parameter], visibility),
            3, "Visibility");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function shows the timing parameter in result table and sets it's
/// HIFN  scaling.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR parameter/This control selects the timing parameter.
/// HIPAR visibility/Shows / hides the parameter in the result table.
/// HIPAR scaling/Defines the unit in which the results are displayed.
ViStatus _VI_FUNC rsspecan_ConfigureTransientAnalysisChirpResultTableTiming(
                            ViSession instrSession,
                            ViInt32   parameter,
                            ViBoolean visibility,
                            ViInt32   scaling)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, parameter, RSSPECAN_VAL_TRANSIENT_RESULT_TIMING_ALL, RSSPECAN_VAL_TRANSIENT_RESULT_TIMING_SWITCH),
            2, "Parameter");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, scaling, RSSPECAN_VAL_TRANSIENT_TIME_SCALING_NS_HZ, RSSPECAN_VAL_TRANSIENT_TIME_SCALING_S_GHZ),
            4, "Scaling");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CALC:CHRD:TABL:TIM:%s %s,%s", paramTimingChirpArr[parameter], visibility ? "ON" : "OFF",
             (parameter != RSSPECAN_VAL_TRANSIENT_RESULT_TIMING_RATE) ? timingScalingArr[scaling] : timingScalingRateArr[scaling]);
    checkErr(RsCore_Write(instrSession, cmd));
    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function shows the phase parameter in result table and sets it's
/// HIFN  scaling.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR parameter/This control selects the timing parameter.
/// HIPAR visibility/Shows / hides the parameter in the result table.
/// HIPAR scaling/Defines the unit in which the results are displayed.
ViStatus _VI_FUNC rsspecan_ConfigureTransientAnalysisChirpResultTablePhase(
                            ViSession instrSession,
                            ViInt32   parameter,
                            ViBoolean visibility,
                            ViInt32   scaling)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViChar*  p2param;
    ViChar*  p2scaling;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K60"));

    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &RsSpecAn_rngTransientResultPhase, parameter, &p2param),
                2, "Parameter");
    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &rsspecan_rngTransientResultPhaseScaling, scaling, &p2scaling),
                4, "Scaling");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CALC:CHRD:TABL:PHAS:%s %s,%s", p2param, visibility ? "ON" : "OFF", p2scaling);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function shows the FM settling parameter in result table and sets it's scaling.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR parameter/This control selects the frequency parameter.
/// HIPAR visibility/Shows / hides the parameter in the result table.
/// HIPAR scaling/Defines the unit in which the results are displayed.
ViStatus _VI_FUNC rsspecan_ConfigureTransientAnalysisChirpResultTableFMSettling(
                            ViSession instrSession,
                            ViInt32   parameter,
                            ViBoolean visibility,
                            ViInt32   scaling)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, parameter, RSSPECAN_VAL_TRANSIENT_RESULT_SETTLING_ALL, RSSPECAN_VAL_TRANSIENT_RESULT_SETTLING_TIME),
            2, "Parameter");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, scaling, RSSPECAN_VAL_TRANSIENT_TIME_SCALING_NS_HZ, RSSPECAN_VAL_TRANSIENT_TIME_SCALING_S_GHZ),
            4, "Scaling");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CALC:CHRD:TABL:FMS:%s %s,%s", RsSpecAn_rngTransientResultFMSettling.entries[parameter].cmdString, visibility ? "ON" : "OFF", rsspecan_rngTransientResultTimeScaling.entries[scaling].cmdString);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function shows the FM settling parameter in result table and sets it's scaling.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR parameter/This control selects the frequency parameter.
/// HIPAR visibility/Shows / hides the parameter in the result table.
/// HIPAR scaling/Defines the unit in which the results are displayed.
ViStatus _VI_FUNC rsspecan_ConfigureTransientAnalysisChirpResultTablePMSettling(
                            ViSession instrSession,
                            ViInt32   parameter,
                            ViBoolean visibility,
                            ViInt32   scaling)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, parameter, RSSPECAN_VAL_TRANSIENT_RESULT_SETTLING_ALL, RSSPECAN_VAL_TRANSIENT_RESULT_SETTLING_TIME),
            2, "Parameter");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, scaling, RSSPECAN_VAL_TRANSIENT_TIME_SCALING_NS_HZ, RSSPECAN_VAL_TRANSIENT_TIME_SCALING_S_GHZ),
            4, "Scaling");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CALC:CHRD:TABL:PMS:%s %s,%s", RsSpecAn_rngTransientResultPMSettling.entries[parameter].cmdString, visibility ? "ON" : "OFF", rsspecan_rngTransientResultTimeScaling.entries[scaling].cmdString);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function shows the frequency INL parameter in result table.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR parameter/This control selects the frequency INL parameter.
/// HIPAR visibility/Shows / hides the parameter in the result table.
ViStatus _VI_FUNC rsspecan_ConfigureTransientAnalysisChirpResultTableFrequencyINL(
                            ViSession instrSession,
                            ViInt32   parameter,
                            ViBoolean visibility)
{
    ViStatus error = VI_SUCCESS;
    ViAttr   attr[3] = {
             RSSPECAN_ATTR_TRANSIENT_CHIRP_RESULT_TABLE_FREQUENCY_INL_AVERAGE,
             RSSPECAN_ATTR_TRANSIENT_CHIRP_RESULT_TABLE_FREQUENCY_INL_PEAK,
             RSSPECAN_ATTR_TRANSIENT_CHIRP_RESULT_TABLE_FREQUENCY_INL_RMS
             };

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, parameter, RSSPECAN_VAL_TRANSIENT_RESULT_FREQ_INL_AVER, RSSPECAN_VAL_TRANSIENT_RESULT_FREQ_INL_RMS),
            2, "Parameter");

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", attr[parameter], visibility),
            3, "Visibility");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function exports result table data from the specified window to
/// HIFN  an ASCii file (.DAT).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR exportCollumns/Selects the columns to be stored in file.
/// HIPAR filename/This parameter defines the path and file name of the target file.
ViStatus _VI_FUNC rsspecan_ExportTransientAnalysisResultTable(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   exportCollumns,
                            ViString  filename)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViChar*  p2string;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K60"));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, window, 0, 16),
            2, "Window");
    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &rsspecan_rngTransientColumns, exportCollumns, &p2string),
                3, "Export Collumns");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "MMEM:STOR%ld:TABL %s,'%s'", window, p2string, filename);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function sets the number of bins used to calculate the histogram.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR histogramBins/This control sets the number of bins used to calculate the histogram.
ViStatus _VI_FUNC rsspecan_ConfigureTransientAnalysisParameterDistributionHistogramBins(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   histogramBins)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_TRANSIENT_PARAMETER_DISTRIBUTION_HISTOGRAM_BINS,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap),
            2, "Window");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_TRANSIENT_PARAMETER_DISTRIBUTION_HISTOGRAM_BINS, histogramBins),
            3, "Histogram Bins");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the x-axis and y-axis values of the specified Parameter
/// HIFN  Distribution display.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR xAxis/Queries the x-axis values of the specified Parameter Distribution
/// HIPAR xAxis/display.
/// HIPAR yAxis/Queries the y-axis values of the specified Parameter Distribution
/// HIPAR yAxis/display.
ViStatus _VI_FUNC rsspecan_QueryTransientAnalysisParameterDistributionAxis(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32*  xAxis,
                            ViInt32*  yAxis)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_TRANSIENT_PARAMETER_DISTRIBUTION_X_AXIS,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap));

    viCheckParm(rsspecan_GetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_TRANSIENT_PARAMETER_DISTRIBUTION_X_AXIS, xAxis),
            3, "X-Axis");

    viCheckParm(rsspecan_GetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_TRANSIENT_PARAMETER_DISTRIBUTION_Y_AXIS, yAxis),
            4, "Y-Axis");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the Parameter Distribution result display for
/// HIFN  hop frequency parameters.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR xAxis/Parameter to be displayed on the x-axis.
/// HIPAR yAxis/Parameter to be displayed on the y-axis.
ViStatus _VI_FUNC rsspecan_ConfigureTransientAnalysisParameterDistributionHopFrequency(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   xAxis,
                            ViInt32   yAxis)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViInt32  cmdGrpIndex = 0;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K60"));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, window, 1, 16),
            2, "Window");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, xAxis, RSSPECAN_VAL_HOP_RESULT_FREQUENCY_AVER, RSSPECAN_VAL_HOP_RESULT_FREQUENCY_PMST),
            3, "X Axis");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, yAxis, RSSPECAN_VAL_TRANSIENT_RESULT_Y_AXIS_COUNT, RSSPECAN_VAL_TRANSIENT_RESULT_Y_AXIS_OCCURANCE),
            4, "Y Axis");

    switch (xAxis)
    {
    case RSSPECAN_VAL_HOP_RESULT_FREQUENCY_AVER:
    case RSSPECAN_VAL_HOP_RESULT_FREQUENCY_FMER:
    case RSSPECAN_VAL_HOP_RESULT_FREQUENCY_MAXD:
    case RSSPECAN_VAL_HOP_RESULT_FREQUENCY_RMSD:
    case RSSPECAN_VAL_HOP_RESULT_FREQUENCY_AVGD:
    case RSSPECAN_VAL_HOP_RESULT_FREQUENCY_RELF:
        cmdGrpIndex = 0; //FREQ
        break;
    case RSSPECAN_VAL_HOP_RESULT_FREQUENCY_FMSL:
    case RSSPECAN_VAL_HOP_RESULT_FREQUENCY_FMSP:
    case RSSPECAN_VAL_HOP_RESULT_FREQUENCY_FMST:
        cmdGrpIndex = 1; //FMS
        break;
    case RSSPECAN_VAL_HOP_RESULT_FREQUENCY_PMSL:
    case RSSPECAN_VAL_HOP_RESULT_FREQUENCY_PMSP:
    case RSSPECAN_VAL_HOP_RESULT_FREQUENCY_PMST:
        cmdGrpIndex = 2; //PMS
        break;
    default:
        cmdGrpIndex = 0; //FREQ
        break;
    }

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CALC%ld:DIST:HOP:%s %s,%s", window, FreqCommandGroupArr[cmdGrpIndex], rsspecan_rngTransientParameterHopFrequency.entries[xAxis].cmdString, YAxisArr[yAxis]);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the Parameter Distribution result display for
/// HIFN  hop power parameters.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR xAxis/Parameter to be displayed on the x-axis.
/// HIPAR yAxis/Parameter to be displayed on the y-axis.
ViStatus _VI_FUNC rsspecan_ConfigureTransientAnalysisParameterDistributionHopPower(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   xAxis,
                            ViInt32   yAxis)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K60"));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, window, 1, 16),
            2, "Window");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, xAxis, RSSPECAN_VAL_HOP_RESULT_POWER_AVERAGE, RSSPECAN_VAL_HOP_RESULT_POWER_RIPPLE),
            3, "X Axis");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, yAxis, RSSPECAN_VAL_TRANSIENT_RESULT_Y_AXIS_COUNT, RSSPECAN_VAL_TRANSIENT_RESULT_Y_AXIS_OCCURANCE),
            4, "Y Axis");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CALC%ld:DIST:HOP:POW %s,%s", window, HopPowerAxisArr[xAxis], YAxisArr[yAxis]);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the Parameter Distribution result display for
/// HIFN  hop timing parameters.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR xAxis/Parameter to be displayed on the x-axis.
/// HIPAR yAxis/Parameter to be displayed on the y-axis.
ViStatus _VI_FUNC rsspecan_ConfigureTransientAnalysisParameterDistributionHopTiming(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   xAxis,
                            ViInt32   yAxis)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K60"));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, window, 1, 16),
            2, "Window");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, xAxis, RSSPECAN_VAL_HOP_RESULT_TIMING_BEGIN, RSSPECAN_VAL_HOP_RESULT_TIMING_SWITCH),
            3, "X Axis");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, yAxis, RSSPECAN_VAL_TRANSIENT_RESULT_Y_AXIS_COUNT, RSSPECAN_VAL_TRANSIENT_RESULT_Y_AXIS_OCCURANCE),
            4, "Y Axis");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CALC%ld:DIST:HOP:TIM %s,%s", window, HopTimingAxisArr[xAxis], YAxisArr[yAxis]);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the Parameter Distribution result display for
/// HIFN  hop phase parameters.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR xAxis/Parameter to be displayed on the x-axis.
/// HIPAR yAxis/Parameter to be displayed on the y-axis.
ViStatus _VI_FUNC rsspecan_ConfigureTransientAnalysisParameterDistributionHopPhase(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   xAxis,
                            ViInt32   yAxis)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViChar*  p2xAxis;
    ViChar*  p2yAxis;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K60"));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, window, 1, 16),
            2, "Window");
    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &rsspecan_rngTransientParameterHopPhase, xAxis, &p2xAxis),
                3, "X Axis");
    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &rsspecan_rngTransientParameterYAxis, yAxis, &p2yAxis),
                4, "Y Axis");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CALC%ld:DIST:HOP:PHAS %s,%s", window, p2xAxis, p2yAxis);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the Parameter Distribution result display for
/// HIFN  hop state parameters.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR xAxis/Parameter to be displayed on the x-axis.
/// HIPAR yAxis/Parameter to be displayed on the y-axis.
ViStatus _VI_FUNC rsspecan_ConfigureTransientAnalysisParameterDistributionHopState(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   xAxis,
                            ViInt32   yAxis)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K60"));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, window, 1, 16),
            2, "Window");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, xAxis, RSSPECAN_VAL_HOP_RESULT_STATE_STAF, RSSPECAN_VAL_HOP_RESULT_STATE_INDEX),
            3, "X Axis");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, yAxis, RSSPECAN_VAL_TRANSIENT_RESULT_Y_AXIS_COUNT, RSSPECAN_VAL_TRANSIENT_RESULT_Y_AXIS_OCCURANCE),
            4, "Y Axis");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CALC%ld:DIST:HOP:STAT %s,%s", window, HopStateAxisArr[xAxis], YAxisArr[yAxis]);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the Parameter Distribution result display for
/// HIFN  chirp frequency parameters.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR xAxis/Parameter to be displayed on the x-axis.
/// HIPAR yAxis/Parameter to be displayed on the y-axis.
ViStatus _VI_FUNC rsspecan_ConfigureTransientAnalysisParameterDistributionChirpFrequency(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   xAxis,
                            ViInt32   yAxis)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViInt32  cmdGrpIndex = 0;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K60"));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, window, 1, 16),
            2, "Window");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, xAxis, RSSPECAN_VAL_CHIRP_RESULT_FREQUENCY_AVER, RSSPECAN_VAL_CHIRP_RESULT_FREQUENCY_PMST),
            3, "X Axis");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, yAxis, RSSPECAN_VAL_TRANSIENT_RESULT_Y_AXIS_COUNT, RSSPECAN_VAL_TRANSIENT_RESULT_Y_AXIS_OCCURANCE),
            4, "Y Axis");

    switch (xAxis)
    {
    case RSSPECAN_VAL_CHIRP_RESULT_FREQUENCY_AVER:
    case RSSPECAN_VAL_CHIRP_RESULT_FREQUENCY_CHER:
    case RSSPECAN_VAL_CHIRP_RESULT_FREQUENCY_MAXD:
    case RSSPECAN_VAL_CHIRP_RESULT_FREQUENCY_RMSD:
    case RSSPECAN_VAL_CHIRP_RESULT_FREQUENCY_AVGD:
    case RSSPECAN_VAL_CHIRP_RESULT_FREQUENCY_AVGN:
    case RSSPECAN_VAL_CHIRP_RESULT_FREQUENCY_MAXN:
    case RSSPECAN_VAL_CHIRP_RESULT_FREQUENCY_RMSN:
    case RSSPECAN_VAL_CHIRP_RESULT_FREQUENCY_BWID:
        cmdGrpIndex = 0; //FREQ
        break;
    case RSSPECAN_VAL_CHIRP_RESULT_FREQUENCY_FMSL:
    case RSSPECAN_VAL_CHIRP_RESULT_FREQUENCY_FMSP:
    case RSSPECAN_VAL_CHIRP_RESULT_FREQUENCY_FMST:
        cmdGrpIndex = 1; //FMS
        break;
    case RSSPECAN_VAL_CHIRP_RESULT_FREQUENCY_PMSL:
    case RSSPECAN_VAL_CHIRP_RESULT_FREQUENCY_PMSP:
    case RSSPECAN_VAL_CHIRP_RESULT_FREQUENCY_PMST:
        cmdGrpIndex = 2; //PMS
        break;
    default:
        cmdGrpIndex = 0; //FREQ
        break;
    }

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CALC%ld:DIST:CHIR:%s %s,%s", window, FreqCommandGroupArr[cmdGrpIndex], rsspecan_rngTransientParameterChirpFrequency.entries[xAxis].cmdString, YAxisArr[yAxis]);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the Parameter Distribution result display for
/// HIFN  chirp power parameters.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR xAxis/Parameter to be displayed on the x-axis.
/// HIPAR yAxis/Parameter to be displayed on the y-axis.
ViStatus _VI_FUNC rsspecan_ConfigureTransientAnalysisParameterDistributionChirpPower(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   xAxis,
                            ViInt32   yAxis)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K60"));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, window, 1, 16),
            2, "Window");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, xAxis, RSSPECAN_VAL_CHIRP_RESULT_POWER_AVERAGE, RSSPECAN_VAL_CHIRP_RESULT_POWER_RIPPLE),
            3, "X Axis");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, yAxis, RSSPECAN_VAL_TRANSIENT_RESULT_Y_AXIS_COUNT, RSSPECAN_VAL_TRANSIENT_RESULT_Y_AXIS_OCCURANCE),
            4, "Y Axis");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CALC%ld:DIST:CHIR:POW %s,%s", window, HopPowerAxisArr[xAxis], YAxisArr[yAxis]);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the Parameter Distribution result display for
/// HIFN  chirp timing parameters.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR xAxis/Parameter to be displayed on the x-axis.
/// HIPAR yAxis/Parameter to be displayed on the y-axis.
ViStatus _VI_FUNC rsspecan_ConfigureTransientAnalysisParameterDistributionChirpTiming(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   xAxis,
                            ViInt32   yAxis)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K60"));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, window, 1, 16),
            2, "Window");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, xAxis, RSSPECAN_VAL_CHIRP_RESULT_TIMING_BEGIN, RSSPECAN_VAL_CHIRP_RESULT_TIMING_RATE),
            3, "X Axis");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, yAxis, RSSPECAN_VAL_TRANSIENT_RESULT_Y_AXIS_COUNT, RSSPECAN_VAL_TRANSIENT_RESULT_Y_AXIS_OCCURANCE),
            4, "Y Axis");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CALC%ld:DIST:CHIR:TIM %s,%s", window, ChirpTimingAxisArr[xAxis], YAxisArr[yAxis]);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the Parameter Distribution result display for
/// HIFN  chirp phase parameters.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR xAxis/Parameter to be displayed on the x-axis.
/// HIPAR yAxis/Parameter to be displayed on the y-axis.
ViStatus _VI_FUNC rsspecan_ConfigureTransientAnalysisParameterDistributionChirpPhase(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   xAxis,
                            ViInt32   yAxis)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViChar*  p2xAxis;
    ViChar*  p2yAxis;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K60"));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, window, 1, 16),
            2, "Window");
    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &rsspecan_rngTransientParameterChirpPhase, xAxis, &p2xAxis),
                3, "X Axis");
    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &rsspecan_rngTransientParameterYAxis, yAxis, &p2yAxis),
                4, "Y Axis");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CALC%ld:DIST:CHIR:PHAS %s,%s", window, p2xAxis, p2yAxis);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the Parameter Distribution result display for
/// HIFN  chirp state parameters.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR xAxis/Parameter to be displayed on the x-axis.
/// HIPAR yAxis/Parameter to be displayed on the y-axis.
ViStatus _VI_FUNC rsspecan_ConfigureTransientAnalysisParameterDistributionChirpState(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   xAxis,
                            ViInt32   yAxis)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K60"));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, window, 1, 16),
            2, "Window");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, xAxis, RSSPECAN_VAL_CHIRP_RESULT_STATE_INDEX, RSSPECAN_VAL_CHIRP_RESULT_STATE_INDEX),
            3, "X Axis");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, yAxis, RSSPECAN_VAL_TRANSIENT_RESULT_Y_AXIS_COUNT, RSSPECAN_VAL_TRANSIENT_RESULT_Y_AXIS_OCCURANCE),
            4, "Y Axis");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CALC%ld:DIST:CHIR:STAT %s,%s", window, ChirpStateAxisArr[xAxis], YAxisArr[yAxis]);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function swaps the x and y axis parameters of the Parameter Trend
/// HIFN  result display.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
ViStatus _VI_FUNC rsspecan_ConfigureTransientAnalysisParameterTrendSwapXY(
                            ViSession instrSession,
                            ViInt32   window)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_TRANSIENT_PARAMETER_TREND_SWAP_XY,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap),
            2, "Window");

    checkErr(rsspecan_SetAttributeViString(instrSession, repCap, RSSPECAN_ATTR_TRANSIENT_PARAMETER_TREND_SWAP_XY, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the x-axis and y-axis of the Parameter Trend result display.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR xAxis/Queries the x-axis of the Parameter Trend result display.
/// HIPAR yAxis/Queries the y-axis of the Parameter Trend result display.
ViStatus _VI_FUNC rsspecan_QueryTransientAnalysisParameterTrendAxis(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32*  xAxis,
                            ViInt32*  yAxis)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_TRANSIENT_PARAMETER_TREND_X_AXIS,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap));

    viCheckParm(rsspecan_GetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_TRANSIENT_PARAMETER_TREND_X_AXIS, xAxis),
            3, "X-Axis");

    viCheckParm(rsspecan_GetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_TRANSIENT_PARAMETER_TREND_Y_AXIS, yAxis),
            4, "Y-Axis");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the x-axis and y-axis of the Parameter Trend
/// HIFN  result display for hop trends over time.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR yAxis/Parameter to be displayed on the y-axis.
/// HIPAR xAxis/Parameter to be displayed on the x-axis.
ViStatus _VI_FUNC rsspecan_ConfigureTransientAnalysisParameterTrendHopFrequency(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   yAxis,
                            ViInt32   xAxis)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViInt32  cmdGrpIndex = 0;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K60"));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, window, 1, 16),
            2, "Window");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, yAxis, RSSPECAN_VAL_HOP_RESULT_FREQUENCY_AVER, RSSPECAN_VAL_HOP_RESULT_FREQUENCY_PMST),
            3, "Y Axis");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, xAxis, RSSPECAN_VAL_TRANSIENT_TREND_HOP_X_AXIS_FREQ, RSSPECAN_VAL_TRANSIENT_TREND_HOP_X_AXIS_SWIT),
            4, "X Axis");

    switch (yAxis)
    {
    case RSSPECAN_VAL_HOP_RESULT_FREQUENCY_AVER:
    case RSSPECAN_VAL_HOP_RESULT_FREQUENCY_FMER:
    case RSSPECAN_VAL_HOP_RESULT_FREQUENCY_MAXD:
    case RSSPECAN_VAL_HOP_RESULT_FREQUENCY_RMSD:
    case RSSPECAN_VAL_HOP_RESULT_FREQUENCY_AVGD:
    case RSSPECAN_VAL_HOP_RESULT_FREQUENCY_RELF:
        cmdGrpIndex = 0; //FREQ
        break;
    case RSSPECAN_VAL_HOP_RESULT_FREQUENCY_FMSL:
    case RSSPECAN_VAL_HOP_RESULT_FREQUENCY_FMSP:
    case RSSPECAN_VAL_HOP_RESULT_FREQUENCY_FMST:
        cmdGrpIndex = 1; //FMS
        break;
    case RSSPECAN_VAL_HOP_RESULT_FREQUENCY_PMSL:
    case RSSPECAN_VAL_HOP_RESULT_FREQUENCY_PMSP:
    case RSSPECAN_VAL_HOP_RESULT_FREQUENCY_PMST:
        cmdGrpIndex = 2; //PMS
        break;
    default:
        cmdGrpIndex = 0; //FREQ
        break;
    }

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CALC%ld:TREN:HOP:%s %s,%s", window, FreqCommandGroupArr[cmdGrpIndex], rsspecan_rngTransientParameterHopFrequency.entries[yAxis].cmdString, RsSpecAn_rngTransientTrendHopXAxis.entries[xAxis].cmdString);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the x or y axis of the Parameter Trend result
/// HIFN  display for hop frequency parameters.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR axis/This control selects the axis.
/// HIPAR displayParameter/Parameter to be displayed on the axis.
ViStatus _VI_FUNC rsspecan_ConfigureTransientAnalysisParameterTrendHopFrequencyAxis(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   axis,
                            ViInt32   displayParameter)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViInt32  cmdGrpIndex = 0;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K60"));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, window, 1, 16),
            2, "Window");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, axis, RSSPECAN_VAL_TRANSIENT_AXIS_X, RSSPECAN_VAL_TRANSIENT_AXIS_Y),
            3, "Axis");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, displayParameter, RSSPECAN_VAL_HOP_RESULT_FREQUENCY_AVER, RSSPECAN_VAL_HOP_RESULT_FREQUENCY_PMST),
            4, "Display Parameter");

    switch (displayParameter)
    {
    case RSSPECAN_VAL_HOP_RESULT_FREQUENCY_AVER:
    case RSSPECAN_VAL_HOP_RESULT_FREQUENCY_FMER:
    case RSSPECAN_VAL_HOP_RESULT_FREQUENCY_MAXD:
    case RSSPECAN_VAL_HOP_RESULT_FREQUENCY_RMSD:
    case RSSPECAN_VAL_HOP_RESULT_FREQUENCY_AVGD:
    case RSSPECAN_VAL_HOP_RESULT_FREQUENCY_RELF:
        cmdGrpIndex = 0; //FREQ
        break;
    case RSSPECAN_VAL_HOP_RESULT_FREQUENCY_FMSL:
    case RSSPECAN_VAL_HOP_RESULT_FREQUENCY_FMSP:
    case RSSPECAN_VAL_HOP_RESULT_FREQUENCY_FMST:
        cmdGrpIndex = 1; //FMS
        break;
    case RSSPECAN_VAL_HOP_RESULT_FREQUENCY_PMSL:
    case RSSPECAN_VAL_HOP_RESULT_FREQUENCY_PMSP:
    case RSSPECAN_VAL_HOP_RESULT_FREQUENCY_PMST:
        cmdGrpIndex = 2; //PMS
        break;
    default:
        cmdGrpIndex = 0; //FREQ
        break;
    }

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CALC%ld:TREN:HOP:%s:%s %s", window, FreqCommandGroupArr[cmdGrpIndex], (axis == RSSPECAN_VAL_TRANSIENT_AXIS_X) ? "X" : "Y", rsspecan_rngTransientParameterHopFrequency.entries[displayParameter].cmdString);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the x-axis and y-axis of the Parameter Trend
/// HIFN  result display for hop trends over time.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR yAxis/Parameter to be displayed on the y-axis.
/// HIPAR xAxis/Parameter to be displayed on the x-axis.
ViStatus _VI_FUNC rsspecan_ConfigureTransientAnalysisParameterTrendHopPower(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   yAxis,
                            ViInt32   xAxis)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K60"));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, window, 1, 16),
            2, "Window");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, yAxis, RSSPECAN_VAL_HOP_RESULT_POWER_AVERAGE, RSSPECAN_VAL_HOP_RESULT_POWER_RIPPLE),
            3, "Y Axis");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, xAxis, RSSPECAN_VAL_TRANSIENT_TREND_HOP_X_AXIS_FREQ, RSSPECAN_VAL_TRANSIENT_TREND_HOP_X_AXIS_SWIT),
            4, "X Axis");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CALC%ld:TREN:HOP:POW %s,%s", window, HopPowerAxisArr[yAxis], RsSpecAn_rngTransientTrendHopXAxis.entries[xAxis].cmdString);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the x or y axis of the Parameter Trend result
/// HIFN  display for hop power parameters.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR axis/This control selects the axis.
/// HIPAR displayParameter/Parameter to be displayed on the axis.
ViStatus _VI_FUNC rsspecan_ConfigureTransientAnalysisParameterTrendHopPowerAxis(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   axis,
                            ViInt32   displayParameter)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K60"));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, window, 1, 16),
            2, "Window");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, axis, RSSPECAN_VAL_TRANSIENT_AXIS_X, RSSPECAN_VAL_TRANSIENT_AXIS_Y),
            3, "Axis");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, displayParameter, RSSPECAN_VAL_HOP_RESULT_POWER_AVERAGE, RSSPECAN_VAL_HOP_RESULT_POWER_RIPPLE),
            4, "Display Parameter");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CALC%ld:TREN:HOP:POW:%s %s", window, (axis == RSSPECAN_VAL_TRANSIENT_AXIS_X) ? "X" : "Y", HopPowerAxisArr[displayParameter]);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the x-axis and y-axis of the Parameter Trend
/// HIFN  result display for hop trends over time.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR yAxis/Parameter to be displayed on the y-axis.
/// HIPAR xAxis/Parameter to be displayed on the x-axis.
ViStatus _VI_FUNC rsspecan_ConfigureTransientAnalysisParameterTrendHopState(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   yAxis,
                            ViInt32   xAxis)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K60"));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, window, 1, 16),
            2, "Window");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, yAxis, RSSPECAN_VAL_HOP_RESULT_STATE_STAF, RSSPECAN_VAL_HOP_RESULT_STATE_INDEX),
            3, "Y Axis");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, xAxis, RSSPECAN_VAL_TRANSIENT_TREND_HOP_X_AXIS_FREQ, RSSPECAN_VAL_TRANSIENT_TREND_HOP_X_AXIS_SWIT),
            4, "X Axis");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CALC%ld:TREN:HOP:STAT %s,%s", window, HopStateAxisArr[yAxis], RsSpecAn_rngTransientTrendHopXAxis.entries[xAxis].cmdString);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the x or y axis of the Parameter Trend result
/// HIFN  display for hop state parameters.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR axis/This control selects the axis.
/// HIPAR displayParameter/Parameter to be displayed on the axis.
ViStatus _VI_FUNC rsspecan_ConfigureTransientAnalysisParameterTrendHopStateAxis(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   axis,
                            ViInt32   displayParameter)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K60"));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, window, 1, 16),
            2, "Window");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, axis, RSSPECAN_VAL_TRANSIENT_AXIS_X, RSSPECAN_VAL_TRANSIENT_AXIS_Y),
            3, "Axis");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, displayParameter, RSSPECAN_VAL_HOP_RESULT_STATE_STAF, RSSPECAN_VAL_HOP_RESULT_STATE_INDEX),
            4, "Display Parameter");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CALC%ld:TREN:HOP:STAT:%s %s", window, (axis == RSSPECAN_VAL_TRANSIENT_AXIS_X) ? "X" : "Y", HopStateAxisArr[displayParameter]);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the x-axis and y-axis of the Parameter Trend
/// HIFN  result display for hop trends over time.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR yAxis/Parameter to be displayed on the y-axis.
/// HIPAR xAxis/Parameter to be displayed on the x-axis.
ViStatus _VI_FUNC rsspecan_ConfigureTransientAnalysisParameterTrendHopTiming(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   yAxis,
                            ViInt32   xAxis)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K60"));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, window, 1, 16),
            2, "Window");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, yAxis, RSSPECAN_VAL_HOP_RESULT_TIMING_BEGIN, RSSPECAN_VAL_HOP_RESULT_TIMING_SWITCH),
            3, "Y Axis");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, xAxis, RSSPECAN_VAL_TRANSIENT_TREND_HOP_X_AXIS_FREQ, RSSPECAN_VAL_TRANSIENT_TREND_HOP_X_AXIS_SWIT),
            4, "X Axis");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CALC%ld:TREN:HOP:TIM %s,%s", window, HopTimingAxisArr[yAxis], RsSpecAn_rngTransientTrendHopXAxis.entries[xAxis].cmdString);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the x or y axis of the Parameter Trend result
/// HIFN  display for hop timing parameters.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR axis/This control selects the axis.
/// HIPAR displayParameter/Parameter to be displayed on the axis.
ViStatus _VI_FUNC rsspecan_ConfigureTransientAnalysisParameterTrendHopTimingAxis(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   axis,
                            ViInt32   displayParameter)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K60"));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, window, 1, 16),
            2, "Window");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, axis, RSSPECAN_VAL_TRANSIENT_AXIS_X, RSSPECAN_VAL_TRANSIENT_AXIS_Y),
            3, "Axis");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, displayParameter, RSSPECAN_VAL_HOP_RESULT_TIMING_BEGIN, RSSPECAN_VAL_HOP_RESULT_TIMING_SWITCH),
            4, "Display Parameter");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CALC%ld:TREN:HOP:TIM:%s %s", window, (axis == RSSPECAN_VAL_TRANSIENT_AXIS_X) ? "X" : "Y", HopTimingAxisArr[displayParameter]);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the x-axis and y-axis of the Parameter Trend
/// HIFN  result display for hop trends over phase.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR yAxis/Parameter to be displayed on the y-axis.
/// HIPAR xAxis/Parameter to be displayed on the x-axis.
ViStatus _VI_FUNC rsspecan_ConfigureTransientAnalysisParameterTrendHopPhase(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   yAxis,
                            ViInt32   xAxis)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViChar*  p2xAxis;
    ViChar*  p2yAxis;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K60"));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, window, 1, 16),
            2, "Window");
    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &rsspecan_rngTransientParameterHopPhase, yAxis, &p2yAxis),
                3, "Y Axis");
    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &RsSpecAn_rngTransientTrendHopXAxis, xAxis, &p2xAxis),
                4, "X Axis");
    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CALC%ld:TREN:HOP:PHAS %s,%s", window, p2yAxis, p2xAxis);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the x or y axis of the Parameter Trend result
/// HIFN  display for hop phase parameters.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR axis/This control selects the axis.
/// HIPAR displayParameter/Parameter to be displayed on the axis.
ViStatus _VI_FUNC rsspecan_ConfigureTransientAnalysisParameterTrendHopPhaseAxis(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   axis,
                            ViInt32   displayParameter)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViChar*  p2displayParameter;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K60"));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, window, 1, 16),
            2, "Window");
    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &rsspecan_rngTransientParameterHopPhase, displayParameter, &p2displayParameter),
                3, "Y Axis");

    switch (axis)
    {
    case RSSPECAN_VAL_TRANSIENT_AXIS_X:
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CALC%ld:TREN:HOP:PHAS:X %s", window, p2displayParameter);
        checkErr(RsCore_Write(instrSession, cmd));
        break;
    case RSSPECAN_VAL_TRANSIENT_AXIS_Y:
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CALC%ld:TREN:HOP:PHAS:Y %s", window, p2displayParameter);
        checkErr(RsCore_Write(instrSession, cmd));
        break;
    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, axis), 3, "Axis");
    }
    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function configures the x-axis and y-axis of the Parameter Trend
/// HIFN  result display for chirp trends over time.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR yAxis/Parameter to be displayed on the y-axis.
/// HIPAR xAxis/Parameter to be displayed on the x-axis.
ViStatus _VI_FUNC rsspecan_ConfigureTransientAnalysisParameterTrendChirpFrequency(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   yAxis,
                            ViInt32   xAxis)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViInt32  cmdGrpIndex;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K60"));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, window, 1, 16),
            2, "Window");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, yAxis, RSSPECAN_VAL_CHIRP_RESULT_FREQUENCY_AVER, RSSPECAN_VAL_CHIRP_RESULT_FREQUENCY_PMST),
            3, "Y Axis");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, xAxis, RSSPECAN_VAL_TRANSIENT_TREND_CHIRP_X_AXIS_CHER, RSSPECAN_VAL_TRANSIENT_TREND_CHIRP_X_AXIS_IND),
            4, "X Axis");

    switch (yAxis)
    {
    case RSSPECAN_VAL_CHIRP_RESULT_FREQUENCY_AVER:
    case RSSPECAN_VAL_CHIRP_RESULT_FREQUENCY_CHER:
    case RSSPECAN_VAL_CHIRP_RESULT_FREQUENCY_MAXD:
    case RSSPECAN_VAL_CHIRP_RESULT_FREQUENCY_RMSD:
    case RSSPECAN_VAL_CHIRP_RESULT_FREQUENCY_AVGD:
    case RSSPECAN_VAL_CHIRP_RESULT_FREQUENCY_AVGN:
    case RSSPECAN_VAL_CHIRP_RESULT_FREQUENCY_MAXN:
    case RSSPECAN_VAL_CHIRP_RESULT_FREQUENCY_RMSN:
    case RSSPECAN_VAL_CHIRP_RESULT_FREQUENCY_BWID:
        cmdGrpIndex = 0; //FREQ
        break;
    case RSSPECAN_VAL_CHIRP_RESULT_FREQUENCY_FMSL:
    case RSSPECAN_VAL_CHIRP_RESULT_FREQUENCY_FMSP:
    case RSSPECAN_VAL_CHIRP_RESULT_FREQUENCY_FMST:
        cmdGrpIndex = 1; //FMS
        break;
    case RSSPECAN_VAL_CHIRP_RESULT_FREQUENCY_PMSL:
    case RSSPECAN_VAL_CHIRP_RESULT_FREQUENCY_PMSP:
    case RSSPECAN_VAL_CHIRP_RESULT_FREQUENCY_PMST:
        cmdGrpIndex = 2; //PMS
        break;
    default:
        cmdGrpIndex = 0; //FREQ
        break;
    }

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CALC%ld:TREN:CHIR:%s %s,%s", window, FreqCommandGroupArr[cmdGrpIndex], rsspecan_rngTransientParameterChirpFrequency.entries[yAxis].cmdString, RsSpecAn_rngTransientTrendChirpXAxis.entries[xAxis].cmdString);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the x or y axis of the Parameter Trend result
/// HIFN  display for chirp frequency parameters.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR axis/This control selects the axis.
/// HIPAR displayParameter/Parameter to be displayed on the axis.
ViStatus _VI_FUNC rsspecan_ConfigureTransientAnalysisParameterTrendChirpFrequencyAxis(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   axis,
                            ViInt32   displayParameter)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViInt32  cmdGrpIndex;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K60"));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, window, 1, 16),
            2, "Window");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, axis, RSSPECAN_VAL_TRANSIENT_AXIS_X, RSSPECAN_VAL_TRANSIENT_AXIS_Y),
            3, "Axis");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, displayParameter, RSSPECAN_VAL_CHIRP_RESULT_FREQUENCY_AVER, RSSPECAN_VAL_CHIRP_RESULT_FREQUENCY_PMST),
            4, "Display Parameter");

    switch (displayParameter)
    {
    case RSSPECAN_VAL_CHIRP_RESULT_FREQUENCY_AVER:
    case RSSPECAN_VAL_CHIRP_RESULT_FREQUENCY_CHER:
    case RSSPECAN_VAL_CHIRP_RESULT_FREQUENCY_MAXD:
    case RSSPECAN_VAL_CHIRP_RESULT_FREQUENCY_RMSD:
    case RSSPECAN_VAL_CHIRP_RESULT_FREQUENCY_AVGD:
    case RSSPECAN_VAL_CHIRP_RESULT_FREQUENCY_AVGN:
    case RSSPECAN_VAL_CHIRP_RESULT_FREQUENCY_MAXN:
    case RSSPECAN_VAL_CHIRP_RESULT_FREQUENCY_RMSN:
    case RSSPECAN_VAL_CHIRP_RESULT_FREQUENCY_BWID:
        cmdGrpIndex = 0; //FREQ
        break;
    case RSSPECAN_VAL_CHIRP_RESULT_FREQUENCY_FMSL:
    case RSSPECAN_VAL_CHIRP_RESULT_FREQUENCY_FMSP:
    case RSSPECAN_VAL_CHIRP_RESULT_FREQUENCY_FMST:
        cmdGrpIndex = 1; //FMS
        break;
    case RSSPECAN_VAL_CHIRP_RESULT_FREQUENCY_PMSL:
    case RSSPECAN_VAL_CHIRP_RESULT_FREQUENCY_PMSP:
    case RSSPECAN_VAL_CHIRP_RESULT_FREQUENCY_PMST:
        cmdGrpIndex = 2; //PMS
        break;
    default:
        cmdGrpIndex = 0; //FREQ
        break;
    }

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CALC%ld:TREN:CHIR:%s:%s %s", window, FreqCommandGroupArr[cmdGrpIndex], (axis == RSSPECAN_VAL_TRANSIENT_AXIS_X) ? "X" : "Y", rsspecan_rngTransientParameterChirpFrequency.entries[displayParameter].cmdString);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the x-axis and y-axis of the Parameter Trend
/// HIFN  result display for chirp trends over time.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR yAxis/Parameter to be displayed on the y-axis.
/// HIPAR xAxis/Parameter to be displayed on the x-axis.
ViStatus _VI_FUNC rsspecan_ConfigureTransientAnalysisParameterTrendChirpPower(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   yAxis,
                            ViInt32   xAxis)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K60"));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, window, 1, 16),
            2, "Window");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, yAxis, RSSPECAN_VAL_CHIRP_RESULT_POWER_AVERAGE, RSSPECAN_VAL_CHIRP_RESULT_POWER_RIPPLE),
            3, "Y Axis");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, xAxis, RSSPECAN_VAL_TRANSIENT_TREND_CHIRP_X_AXIS_CHER, RSSPECAN_VAL_TRANSIENT_TREND_CHIRP_X_AXIS_IND),
            4, "X Axis");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CALC%ld:TREN:CHIR:POW %s,%s", window, HopPowerAxisArr[yAxis], RsSpecAn_rngTransientTrendChirpXAxis.entries[xAxis].cmdString);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the x or y axis of the Parameter Trend result
/// HIFN  display for chirp power parameters.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR axis/This control selects the axis.
/// HIPAR displayParameter/Parameter to be displayed on the axis.
ViStatus _VI_FUNC rsspecan_ConfigureTransientAnalysisParameterTrendChirpPowerAxis(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   axis,
                            ViInt32   displayParameter)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K60"));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, window, 1, 16),
            2, "Window");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, axis, RSSPECAN_VAL_TRANSIENT_AXIS_X, RSSPECAN_VAL_TRANSIENT_AXIS_Y),
            3, "Axis");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, displayParameter, RSSPECAN_VAL_CHIRP_RESULT_POWER_AVERAGE, RSSPECAN_VAL_CHIRP_RESULT_POWER_RIPPLE),
            4, "Display Parameter");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CALC%ld:TREN:CHIR:POW:%s %s", window, (axis == RSSPECAN_VAL_TRANSIENT_AXIS_X) ? "X" : "Y", HopPowerAxisArr[displayParameter]);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the x-axis and y-axis of the Parameter Trend
/// HIFN  result display for chirp trends over time.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR yAxis/Parameter to be displayed on the y-axis.
/// HIPAR xAxis/Parameter to be displayed on the x-axis.
ViStatus _VI_FUNC rsspecan_ConfigureTransientAnalysisParameterTrendChirpState(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   yAxis,
                            ViInt32   xAxis)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K60"));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, window, 1, 16),
            2, "Window");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, yAxis, RSSPECAN_VAL_CHIRP_RESULT_STATE_INDEX, RSSPECAN_VAL_CHIRP_RESULT_STATE_INDEX),
            3, "Y Axis");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, xAxis, RSSPECAN_VAL_TRANSIENT_TREND_CHIRP_X_AXIS_CHER, RSSPECAN_VAL_TRANSIENT_TREND_CHIRP_X_AXIS_IND),
            4, "X Axis");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CALC%ld:TREN:CHIR:STAT %s,%s", window, ChirpStateAxisArr[yAxis], RsSpecAn_rngTransientTrendChirpXAxis.entries[xAxis].cmdString);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the x or y axis of the Parameter Trend result
/// HIFN  display for chirp state parameters.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR axis/This control selects the axis.
/// HIPAR displayParameter/Parameter to be displayed on the y-axis.
ViStatus _VI_FUNC rsspecan_ConfigureTransientAnalysisParameterTrendChirpStateAxis(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   axis,
                            ViInt32   displayParameter)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K60"));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, window, 1, 16),
            2, "Window");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, axis, RSSPECAN_VAL_TRANSIENT_AXIS_X, RSSPECAN_VAL_TRANSIENT_AXIS_Y),
            3, "Axis");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, displayParameter, RSSPECAN_VAL_CHIRP_RESULT_STATE_INDEX, RSSPECAN_VAL_CHIRP_RESULT_STATE_INDEX),
            4, "Display Parameter");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CALC%ld:TREN:CHIR:STAT:%s %s", window, (axis == RSSPECAN_VAL_TRANSIENT_AXIS_X) ? "X" : "Y", ChirpStateAxisArr[displayParameter]);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the x-axis and y-axis of the Parameter Trend
/// HIFN  result display for chirp trends over time.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR yAxis/Parameter to be displayed on the y-axis.
/// HIPAR xAxis/Parameter to be displayed on the x-axis.
ViStatus _VI_FUNC rsspecan_ConfigureTransientAnalysisParameterTrendChirpTiming(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   yAxis,
                            ViInt32   xAxis)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K60"));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, window, 1, 16),
            2, "Window");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, yAxis, RSSPECAN_VAL_CHIRP_RESULT_TIMING_BEGIN, RSSPECAN_VAL_CHIRP_RESULT_TIMING_RATE),
            3, "Y Axis");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, xAxis, RSSPECAN_VAL_TRANSIENT_TREND_CHIRP_X_AXIS_CHER, RSSPECAN_VAL_TRANSIENT_TREND_CHIRP_X_AXIS_IND),
            4, "X Axis");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CALC%ld:TREN:CHIR:TIM %s,%s", window, ChirpTimingAxisArr[yAxis], RsSpecAn_rngTransientTrendChirpXAxis.entries[xAxis].cmdString);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the x or y axis of the Parameter Trend result
/// HIFN  display for chirp timing parameters.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR axis/This control selects the axis.
/// HIPAR displayParameter/Parameter to be displayed on the axis.
ViStatus _VI_FUNC rsspecan_ConfigureTransientAnalysisParameterTrendChirpTimingAxis(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   axis,
                            ViInt32   displayParameter)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K60"));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, window, 1, 16),
            2, "Window");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, axis, RSSPECAN_VAL_TRANSIENT_AXIS_X, RSSPECAN_VAL_TRANSIENT_AXIS_Y),
            3, "Axis");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, displayParameter, RSSPECAN_VAL_CHIRP_RESULT_TIMING_BEGIN, RSSPECAN_VAL_CHIRP_RESULT_TIMING_RATE),
            4, "Display Parameter");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CALC%ld:TREN:CHIR:TIM:%s %s", window, (axis == RSSPECAN_VAL_TRANSIENT_AXIS_X) ? "X" : "Y", ChirpTimingAxisArr[displayParameter]);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the x-axis and y-axis of the Parameter Trend
/// HIFN  result display for chirp trends over phase.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR yAxis/Parameter to be displayed on the y-axis.
/// HIPAR xAxis/Parameter to be displayed on the x-axis.
ViStatus _VI_FUNC rsspecan_ConfigureTransientAnalysisParameterTrendChirpPhase(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   yAxis,
                            ViInt32   xAxis)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViChar*  p2xAxis;
    ViChar*  p2yAxis;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K60"));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, window, 1, 16),
            2, "Window");
    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &rsspecan_rngTransientParameterChirpPhase, yAxis, &p2yAxis),
                3, "Y Axis");
    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &RsSpecAn_rngTransientTrendChirpXAxis, xAxis, &p2xAxis),
                4, "X Axis");
    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CALC%ld:TREN:CHIR:PHAS %s,%s", window, p2yAxis, p2xAxis);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the x or y axis of the Parameter Trend result
/// HIFN  display for chirp phase parameters.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR axis/This control selects the axis.
/// HIPAR displayParameter/Parameter to be displayed on the axis.
ViStatus _VI_FUNC rsspecan_ConfigureTransientAnalysisParameterTrendChirpPhaseAxis(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   axis,
                            ViInt32   displayParameter)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViChar*  p2displayParameter;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K60"));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, window, 1, 16),
            2, "Window");
    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &rsspecan_rngTransientParameterChirpPhase, displayParameter, &p2displayParameter),
                3, "Y Axis");

    switch (axis)
    {
    case RSSPECAN_VAL_TRANSIENT_AXIS_X:
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CALC%ld:TREN:CHIR:PHAS:X %s", window, p2displayParameter);
        checkErr(RsCore_Write(instrSession, cmd));
        break;
    case RSSPECAN_VAL_TRANSIENT_AXIS_Y:
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CALC%ld:TREN:CHIR:PHAS:Y %s", window, p2displayParameter);
        checkErr(RsCore_Write(instrSession, cmd));
        break;
    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, axis), 3, "Axis");
    }
    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  The y-axis is scaled automatically according to the current
/// HIFN  measurement settings and results.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR autoScale/If On, the y-axis is scaled automatically according to the current
/// HIPAR autoScale/measurement settings and results.
ViStatus _VI_FUNC rsspecan_ConfigureTransientAnalysisAutomaticGridScaling(
                            ViSession instrSession,
                            ViInt32   window,
                            ViBoolean autoScale)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_TRANSIENT_AUTO_GRID_SCALING,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap),
            2, "Window");

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, repCap, RSSPECAN_ATTR_TRANSIENT_AUTO_GRID_SCALING, autoScale),
            3, "Auto Scale");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function defines the scaling using absolute minimum and maximum
/// HIFN  values.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR minimum/This control defines the minimum value of the y-axis for the selected
/// HIPAR minimum/result display.
/// HIPAR maximum/This control defines the maximum value of the y-axis for the selected
/// HIPAR maximum/result display.
ViStatus _VI_FUNC rsspecan_ConfigureTransientAnalysisAbsoluteScaling(
                            ViSession instrSession,
                            ViInt32   window,
                            ViReal64  minimum,
                            ViReal64  maximum)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_TRANSIENT_ABSOLUTE_SCALING_MIN,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap),
            2, "Window");

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_TRANSIENT_ABSOLUTE_SCALING_MIN, minimum),
            3, "Minimum");

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_TRANSIENT_ABSOLUTE_SCALING_MAX, maximum),
            3, "Maximum");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function defines the scaling relative to a reference value, with
/// HIFN  a specified value range per division.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR perDivison/This control defines the value range to be displayed per division of
/// HIPAR perDivison/the diagram (1/10 of total range).
/// HIPAR referencePosition/This control defines the vertical position of the reference level on
/// HIPAR referencePosition/the display grid.
/// HIPAR referenceValue/This control defines the reference value to be displayed at the
/// HIPAR referenceValue/specified reference position.
ViStatus _VI_FUNC rsspecan_ConfigureTransientAnalysisRelativeScaling(
                            ViSession instrSession,
                            ViInt32   window,
                            ViReal64  perDivison,
                            ViReal64  referencePosition,
                            ViReal64  referenceValue)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_DISPLAY_WINDOW_Y_AXIS_GRID_SPACING,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap));

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_DISPLAY_WINDOW_Y_AXIS_GRID_SPACING, perDivison),
            3, "Per Divison");

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_DISPLAY_WINDOW_REFERENCE_LEVEL_POSITION, referencePosition),
            4, "Reference Position");

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_DISPLAY_WINDOW_REFERENCE_LEVEL, referenceValue),
            5, "Reference Value");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function defines the display range of the y-axis.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR range/This control defines the display range of the y-axis.
ViStatus _VI_FUNC rsspecan_ConfigureTransientAnalysisScalingDisplayRange(
                            ViSession instrSession,
                            ViInt32   window,
                            ViReal64  range)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_TRANSIENT_SCALING_DISPLAY_RANGE,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap),
            2, "Window");

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_TRANSIENT_SCALING_DISPLAY_RANGE, range),
            3, "Range");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function selects the global unit for phase results.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR phaseUnit/Sets the default unit for angles.
ViStatus _VI_FUNC rsspecan_ConfigureTransientAnalysisUnit(ViSession instrSession,
                                                          ViInt32   phaseUnit)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_TRANSIENT_UNIT_ANGLE, phaseUnit),
            2, "Phase Unit");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function determines the evaluation basis for the specified result
/// HIFN  display.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR evaluationBasis/Determines the evaluation basis for the specified result display.
ViStatus _VI_FUNC rsspecan_ConfigureTransientAnalysisEvaluationBasis(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   evaluationBasis)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_TRANSIENT_EVALUATION_BASIS,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap),
            2, "Window");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_TRANSIENT_EVALUATION_BASIS, evaluationBasis),
            3, "Evaluation Basis");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function defines the individual hop or chirp for which results
/// HIFN  are calculated and displayed.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR model/This control defines which type of signal to expect (if known), thus
/// HIPAR model/determining the analysis method.
/// HIPAR number/This control defines the individual hop or chirp for which results are
/// HIPAR number/calculated and displayed.
ViStatus _VI_FUNC rsspecan_ConfigureTransientAnalysisSelection(
                            ViSession instrSession,
                            ViInt32   model,
                            ViInt32   number)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    if (model == RSSPECAN_VAL_TRANSIENT_MODEL_HOP)
    {
        viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_TRANSIENT_HOP_SELECTION, number),
                3, "Number");
    }
    else
    {
        viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_TRANSIENT_CHIRP_SELECTION, number),
                3, "Number");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function queries the trace length.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR trace/Selects the trace.
/// HIPAR numberOfTracePoints/Queries the trace length.
ViStatus _VI_FUNC rsspecan_QueryTransientAnalysisTraceLength(ViSession instrSession,
                                                             ViInt32   window,
                                                             ViInt32   trace,
                                                             ViInt32*  numberOfTracePoints)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_trace[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_TRANSIENT_TRACE_LENGTH,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap),
            2, "Window");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_TRANSIENT_TRACE_LENGTH,
            "Trace", trace - 1, RS_REPCAP_BUF_SIZE, repCap_trace),
            3, "Trace");
    strcat(repCap, ",");
    strcat(repCap, repCap_trace);

    viCheckParm(rsspecan_GetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_TRANSIENT_TRACE_LENGTH, numberOfTracePoints),
            4, "numberOfTracePoints");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function defines which hops/chirps are included in the
/// HIFN  statistical evaluation.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR statisticType/Defines which hops/chirps are included in the statistical evaluation.
ViStatus _VI_FUNC rsspecan_ConfigureTransientAnalysisStatisticType(
                            ViSession instrSession,
                            ViInt32   statisticType)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_TRANSIENT_STATISTIC_TYPE, statisticType),
            2, "Statistic Type");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function defines the maximum number of trace points within a
/// HIFN  trace.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR maximumPoints/This control defines the maximum number of trace points within a
/// HIPAR maximumPoints/trace.
ViStatus _VI_FUNC rsspecan_ConfigureTransientAnalysisTraceMaximumPoints(
                            ViSession instrSession,
                            ViInt32   maximumPoints)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_TRANSIENT_TRACE_MAXIMUM_POINTS, maximumPoints),
            2, "Maximum Points");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  The long capture buffer provides functionality to use the full I/Q
/// HIFN  memory depth of the R&S FSW for data acquisition.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR longCaptureBuffer/The long capture buffer provides functionality to use the full I/Q
/// HIPAR longCaptureBuffer/memory depth of the R&S FSW for data acquisition.
ViStatus _VI_FUNC rsspecan_ConfigureTransientAnalysisLongCaptureBuffer(
                            ViSession instrSession,
                            ViInt32   longCaptureBuffer)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, NULL, RSSPECAN_ATTR_TRANSIENT_TRACE_LONG_CAPTURE_BUFFER, longCaptureBuffer),
            2, "Long Capture Buffer");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines which signal component (I/Q) is evaluated in which trace
/// HIFN  for the I/Q Time Domain result display.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR trace/Selects the trace.
/// HIPAR evaluationType/Selects the evaluation type.
ViStatus _VI_FUNC rsspecan_ConfigureTransientAnalysisEvaluation(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   trace,
                            ViInt32   evaluationType)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_trace[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_TRANSIENT_TRACE_EVALUATION,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap),
            2, "Window");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_TRANSIENT_TRACE_EVALUATION,
            "Trace", trace - 1, RS_REPCAP_BUF_SIZE, repCap_trace),
            3, "Trace");
    strcat(repCap, ",");
    strcat(repCap, repCap_trace);

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_TRANSIENT_TRACE_EVALUATION, evaluationType),
            4, "Evaluation Type");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function sets the spectrogram detector type for the specified
/// HIFN  window.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR detector/Sets the spectrogram detector type for the specified window.
ViStatus _VI_FUNC rsspecan_ConfigureTransientAnalysisSpectrogramDetector(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   detector)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_TRANSIENT_SPECTROGRAM_DETECTOR,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap),
            2, "Window");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_TRANSIENT_SPECTROGRAM_DETECTOR, detector),
            3, "Detector");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function defines whether all markers within the selected result
/// HIFN  display are linked. If enabled, and you move one marker along the
/// HIFN  x-axis, all other markers in the display are moved to the same x-axis
/// HIFN  position.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR state/If enabled, the markers in all diagrams are linked.
ViStatus _VI_FUNC rsspecan_ConfigureTransientAnalysisLinkedMarker(
                            ViSession instrSession,
                            ViBoolean state)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_TRANSIENT_LINKED_MARKER, state),
            2, "State");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function links delta marker <marker> to marker 1.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR markerNumber/Selects the marker to configure.
/// HIPAR state/If enabled, the delta marker is linked to marker 1.
ViStatus _VI_FUNC rsspecan_ConfigureTransientAnalysisLinkedDeltamarker(
                            ViSession instrSession,
                            ViInt32   markerNumber,
                            ViBoolean state)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_TRANSIENT_LINKED_DELTAMARKER,
            "Marker", markerNumber -1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Marker");

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, repCap, RSSPECAN_ATTR_TRANSIENT_LINKED_DELTAMARKER, state),
            3, "state");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function stores the current measurement results (all enabled
/// HIFN  traces of all windows) into the specified .csv file.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR fileName/Specifies full path of the file, where measurement data will be
/// HIPAR fileName/stored.
ViStatus _VI_FUNC rsspecan_StoreTransientAnalysisCurrentMeasurement(
                            ViSession instrSession,
                            ViString  fileName)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_TRANSIENT_STORE_CURRENT_MEASUREMENT, fileName),
            2, "File Name");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function returns the total number of hops found.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR totalHops/This control returns the total number of hops found.
ViStatus _VI_FUNC rsspecan_QueryTransientAnalysisHopTotal(ViSession instrSession,
                                                          ViInt32*  totalHops)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_GetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_TRANSIENT_TOTAL_HOPS, totalHops),
            2, "Total Hops");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the number of hops in the current capture buffer.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR totalHops/Queries the number of hops in the current capture buffer.
ViStatus _VI_FUNC rsspecan_QueryTransientAnalysisHopTotalInCaptureBuffer(
                            ViSession instrSession,
                            ViInt32*  totalHops)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_GetAttributeViInt32(instrSession, NULL, RSSPECAN_ATTR_TRANSIENT_TOTAL_HOPS_IN_CAPTURE_BUFFER, totalHops),
            2, "Total Hops");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function queries the hop IDs from the Results table for the
/// HIFN  specified hop(s).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR queryRange/Selects the query range.
/// HIPAR arraySize/This parameter determines the size of array for Pulse IDs values. The
/// HIPAR arraySize/Result Values array must contain at least this number of items.
/// HIPAR ids/This control returns the hop IDs from the Results table for the
/// HIPAR ids/specified hop(s).
/// HIPAR returnedValues/Returns the number of returned values.
ViStatus _VI_FUNC rsspecan_QueryTransientAnalysisHopIDs(ViSession instrSession,
                                                        ViInt32   queryRange,
                                                        ViInt32   arraySize,
                                                        ViInt32   ids[],
                                                        ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViChar*  p2string;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K60"));

    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &rsspecan_rngQueryRange, queryRange, &p2string),
                2, "Query Range");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SENS:HOP:ID? %s", p2string);
    checkErr(RsCore_QueryIntegerArrayToUserBuffer(instrSession, cmd, arraySize, ids, returnedValues));

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function returns the hop numbers from the Results table for the
/// HIFN  specified hop(s).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR queryRange/Selects the query range.
/// HIPAR arraySize/This parameter determines the size of array for Pulse Numbers values.
/// HIPAR arraySize/The Result Values array must contain at least this number of items.
/// HIPAR numbers/This control returns the hop numbers from the Results table for the
/// HIPAR numbers/specified hop(s).
/// HIPAR returnedValues/Returns the number of returned values.
ViStatus _VI_FUNC rsspecan_QueryTransientAnalysisHopNumbers(ViSession instrSession,
                                                            ViInt32   queryRange,
                                                            ViInt32   arraySize,
                                                            ViInt32   numbers[],
                                                            ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViChar*  p2string;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K60"));
    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &rsspecan_rngQueryRange, queryRange, &p2string),
                2, "Query Range");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SENS:HOP:NUMB? %s", p2string);
    checkErr(RsCore_QueryIntegerArrayToUserBuffer(instrSession, cmd, arraySize, numbers, returnedValues));

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function returns the calculated hop frequency parameters values.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR parameter/This control selects the frequency parameter.
/// HIPAR resultType/Selects measurement type.
/// HIPAR queryRange/Selects the query range.
/// HIPAR arraySize/This parameter determines the size of array result values. The Result
/// HIPAR arraySize/Values array must contain at least this number of items.
/// HIPAR results/Returns selected frequency results.
/// HIPAR returnedValues/Returns the number of returned values.
ViStatus _VI_FUNC rsspecan_QueryTransientAnalysisHopResultFrequency(
                            ViSession instrSession,
                            ViInt32   parameter,
                            ViInt32   resultType,
                            ViInt32   queryRange,
                            ViInt32   arraySize,
                            ViReal64  results[],
                            ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViChar*  p2parameter;
    ViChar*  p2resultType;
    ViChar*  p2queryRange;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K60"));

    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &rsspecan_rngTransientParameterHopFrequency, parameter, &p2parameter),
                2, "Parameter");
    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &rsspecan_rngResultType, resultType, &p2resultType),
                3, "Result Type");
    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &rsspecan_rngQueryRange, queryRange, &p2queryRange),
                4, "Query Range");

    if (p2resultType == NULL) p2resultType = "";

    if (queryRange == RSSPECAN_VAL_QUERY_RANGE_SELECTED && resultType != RSSPECAN_VAL_MEASTYPE_CURR)
    {
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, queryRange),
                    4, "Query Range");
    }

    if (parameter < RSSPECAN_VAL_HOP_RESULT_FREQUENCY_FMSL)
    {
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SENS:HOP:FREQ:%s%s? %s", p2parameter, p2resultType, p2queryRange);
    }
    else if (parameter < RSSPECAN_VAL_HOP_RESULT_FREQUENCY_PMSL)
    {
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SENS:HOP:FMS:%s%s? %s", p2parameter, p2resultType, p2queryRange);
    }
    else
    {
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SENS:HOP:PMS:%s%s? %s", p2parameter, p2resultType, p2queryRange);
    }

    checkErr(RsCore_QueryFloatArrayToUserBuffer(instrSession, cmd, arraySize, results, returnedValues));

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function returns the calculated hop timing parameters values.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR parameter/This control selects the timing parameter.
/// HIPAR resultType/Selects measurement type.
/// HIPAR queryRange/Selects the query range.
/// HIPAR arraySize/This parameter determines the size of array result values. The Result
/// HIPAR arraySize/Values array must contain at least this number of items.
/// HIPAR results/Returns selected timing results.
/// HIPAR returnedValues/Returns the number of returned values.
ViStatus _VI_FUNC rsspecan_QueryTransientAnalysisHopResultTiming(
                            ViSession instrSession,
                            ViInt32   parameter,
                            ViInt32   resultType,
                            ViInt32   queryRange,
                            ViInt32   arraySize,
                            ViReal64  results[],
                            ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViChar*  p2parameter;
    ViChar*  p2resultType;
    ViChar*  p2queryRange;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K60"));

    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &rsspecan_rngTransientParameterHopTiming, parameter, &p2parameter),
                2, "Parameter");
    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &rsspecan_rngResultType, resultType, &p2resultType),
                3, "Result Type");
    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &rsspecan_rngQueryRange, queryRange, &p2queryRange),
                4, "Query Range");

    if (p2resultType == NULL) p2resultType = "";

    if (queryRange == RSSPECAN_VAL_QUERY_RANGE_SELECTED && resultType != RSSPECAN_VAL_MEASTYPE_CURR)
    {
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, queryRange),
                    4, "Query Range");
    }

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SENS:HOP:TIM:%s%s? %s", p2parameter,
             p2resultType, p2queryRange);
    checkErr(RsCore_QueryFloatArrayToUserBuffer(instrSession, cmd, arraySize, results, returnedValues));

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function returns the calculated hop power parameters values.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR parameter/This control selects the power parameter.
/// HIPAR resultType/Selects measurement type.
/// HIPAR queryRange/Selects the query range.
/// HIPAR arraySize/This parameter determines the size of array result values. The Result
/// HIPAR arraySize/Values array must contain at least this number of items.
/// HIPAR results/Returns selected power results.
/// HIPAR returnedValues/Returns the number of returned values.
ViStatus _VI_FUNC rsspecan_QueryTransientAnalysisHopResultPower(
                            ViSession instrSession,
                            ViInt32   parameter,
                            ViInt32   resultType,
                            ViInt32   queryRange,
                            ViInt32   arraySize,
                            ViReal64  results[],
                            ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViChar*  p2parameter;
    ViChar*  p2resultType;
    ViChar*  p2queryRange;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K60"));

    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &rsspecan_rngTransientParameterHopPower, parameter, &p2parameter),
                2, "Parameter");
    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &rsspecan_rngResultType, resultType, &p2resultType),
                3, "Result Type");
    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &rsspecan_rngQueryRange, queryRange, &p2queryRange),
                4, "Query Range");

    if (p2resultType == NULL) p2resultType = "";

    if (queryRange == RSSPECAN_VAL_QUERY_RANGE_SELECTED && resultType != RSSPECAN_VAL_MEASTYPE_CURR)
    {
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, queryRange),
                    4, "Query Range");
    }

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SENS:HOP:POW:%s%s? %s", p2parameter, p2resultType, p2queryRange);
    checkErr(RsCore_QueryFloatArrayToUserBuffer(instrSession, cmd, arraySize, results, returnedValues));

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function returns the calculated hop phase parameters values.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR parameter/This control selects the power parameter.
/// HIPAR resultType/Selects measurement type.
/// HIPAR queryRange/Selects the query range.
/// HIPAR arraySize/This parameter determines the size of array result values. The Result
/// HIPAR arraySize/Values array must contain at least this number of items.
/// HIPAR results/Returns selected power results.
/// HIPAR returnedValues/Returns the number of returned values.
ViStatus _VI_FUNC rsspecan_QueryTransientAnalysisHopResultPhase(
                            ViSession instrSession,
                            ViInt32   parameter,
                            ViInt32   resultType,
                            ViInt32   queryRange,
                            ViInt32   arraySize,
                            ViReal64  results[],
                            ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViChar*  p2resultType;
    ViChar*  p2queryRange;
    ViString params[] = {"AVGP", "MAXP", "RMSP"};

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K60"));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, parameter, RSSPECAN_VAL_CHIRP_RESULT_PHASE_AVERAGE, RSSPECAN_VAL_CHIRP_RESULT_PHASE_RMS),
            2, "Parameter");
    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &rsspecan_rngResultType, resultType, &p2resultType),
                3, "Result Type");
    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &rsspecan_rngQueryRange, queryRange, &p2queryRange),
                4, "Query Range");

    if (p2resultType == NULL) p2resultType = "";

    if (queryRange == RSSPECAN_VAL_QUERY_RANGE_SELECTED && resultType != RSSPECAN_VAL_MEASTYPE_CURR)
    {
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, queryRange),
                    4, "Query Range");
    }

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SENS:HOP:PHAS:%s%s? %s", params[parameter], p2resultType, p2queryRange);
    checkErr(RsCore_QueryFloatArrayToUserBuffer(instrSession, cmd, arraySize, results, returnedValues));

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function returns the calculated hop state parameters values.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR resultType/Selects measurement type.
/// HIPAR queryRange/Selects the query range.
/// HIPAR arraySize/This parameter determines the size of array result values. The Result
/// HIPAR arraySize/Values array must contain at least this number of items.
/// HIPAR results/Returns selected state results.
/// HIPAR returnedValues/Returns the number of returned values.
ViStatus _VI_FUNC rsspecan_QueryTransientAnalysisHopResultState(
                            ViSession instrSession,
                            ViInt32   resultType,
                            ViInt32   queryRange,
                            ViInt32   arraySize,
                            ViReal64  results[],
                            ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViChar*  p2resultType;
    ViChar*  p2queryRange;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K60"));

    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &rsspecan_rngResultType, resultType, &p2resultType),
                2, "Result Type");
    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &rsspecan_rngQueryRange, queryRange, &p2queryRange),
                3, "Query Range");

    if (p2resultType == NULL) p2resultType = "";

    if (queryRange == RSSPECAN_VAL_QUERY_RANGE_SELECTED && resultType != RSSPECAN_VAL_MEASTYPE_CURR)
    {
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, queryRange),
                    4, "Query Range");
    }

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SENS:HOP:STAT%s? %s", p2resultType, p2queryRange);
    checkErr(RsCore_QueryFloatArrayToUserBuffer(instrSession, cmd, arraySize, results, returnedValues));

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function returns the calculated hop state parameters values.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR parameter/This control selects the state parameter.
/// HIPAR resultType/Selects measurement type.
/// HIPAR queryRange/Selects the query range.
/// HIPAR arraySize/This parameter determines the size of array result values. The Result
/// HIPAR arraySize/Values array must contain at least this number of items.
/// HIPAR results/Returns selected state results.
/// HIPAR returnedValues/Returns the number of returned values.
ViStatus _VI_FUNC rsspecan_QueryTransientAnalysisHopResultAdditionalState(
                            ViSession instrSession,
                            ViInt32   parameter,
                            ViInt32   resultType,
                            ViInt32   queryRange,
                            ViInt32   arraySize,
                            ViReal64  results[],
                            ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViChar*  p2parameter;
    ViChar*  p2resultType;
    ViChar*  p2queryRange;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K60"));

    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &rsspecan_rngTransientParameterHopState, parameter, &p2parameter),
                2, "Parameter");
    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &rsspecan_rngResultType, resultType, &p2resultType),
                3, "Result Type");
    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &rsspecan_rngQueryRange, queryRange, &p2queryRange),
                4, "Query Range");

    if (p2resultType == NULL) p2resultType = "";

    if (queryRange == RSSPECAN_VAL_QUERY_RANGE_SELECTED && resultType != RSSPECAN_VAL_MEASTYPE_CURR)
    {
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, queryRange),
                    4, "Query Range");
    }

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SENS:HOP:STAT:%s%s? %s", p2parameter, p2resultType, p2queryRange);
    checkErr(RsCore_QueryFloatArrayToUserBuffer(instrSession, cmd, arraySize, results, returnedValues));

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function queries the hop results table. Which values are returned
/// HIFN  depends on the enabled parameters for the results tables (see
/// HIFN  rsspecan_ConfigureTransientResultTable).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR start/This parameter determines the size of array result values. The Result
/// HIPAR start/Values array must contain at least this number of items.
/// HIPAR stop/This parameter determines the size of array result values. The Result
/// HIPAR stop/Values array must contain at least this number of items.
/// HIPAR arraySize/This parameter determines the size of array result values. The Result
/// HIPAR arraySize/Values array must contain at least this number of items.
/// HIPAR results/Returns selected results. Which values are returned depends on the
/// HIPAR results/enabled parameters for the results tables.
/// HIPAR returnedValues/Returns the number of returned values.
ViStatus _VI_FUNC rsspecan_QueryTransientAnalysisHopResultTable(
                            ViSession instrSession,
                            ViInt32   start,
                            ViInt32   stop,
                            ViInt32   arraySize,
                            ViReal64  results[],
                            ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K60"));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CALC:HOPD:TABL:RES? %ld,%ld", start, stop);
    checkErr(RsCore_QueryFloatArrayToUserBuffer(instrSession, cmd, arraySize, results, returnedValues));

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function returns the total number of chirps found.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR totalHops/This control returns the total number of chirps found.
ViStatus _VI_FUNC rsspecan_QueryTransientAnalysisChirpTotal(ViSession instrSession,
                                                            ViInt32*  totalHops)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_GetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_TRANSIENT_TOTAL_CHIRPS, totalHops),
            2, "Total Hops");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the number of chirps in the current capture buffer.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR totalChirps/Queries the number of chirps in the current capture buffer.
ViStatus _VI_FUNC rsspecan_QueryTransientAnalysisChirpTotalInCaptureBuffer(
                            ViSession instrSession,
                            ViInt32*  totalChirps)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_GetAttributeViInt32(instrSession, NULL, RSSPECAN_ATTR_TRANSIENT_TOTAL_CHIRPS_IN_CAPTURE_BUFFER, totalChirps),
            2, "Total Chirps");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function queries the chirp IDs from the Results table for the
/// HIFN  specified chirp(s).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR queryRange/Selects the query range.
/// HIPAR arraySize/This parameter determines the size of array for Pulse IDs values. The
/// HIPAR arraySize/Result Values array must contain at least this number of items.
/// HIPAR ids/This control returns the chirp IDs from the Results table for the
/// HIPAR ids/specified chirp(s).
/// HIPAR returnedValues/Returns the number of returned values.
ViStatus _VI_FUNC rsspecan_QueryTransientAnalysisChirpIDs(ViSession instrSession,
                                                          ViInt32   queryRange,
                                                          ViInt32   arraySize,
                                                          ViInt32   ids[],
                                                          ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViChar*  p2string;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K60"));

    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &rsspecan_rngQueryRange, queryRange, &p2string),
                2, "Query Range");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SENS:CHIR:ID? %s", p2string);
    checkErr(RsCore_QueryIntegerArrayToUserBuffer(instrSession, cmd, arraySize, ids, returnedValues));

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function returns the chirp numbers from the Results table for the
/// HIFN  specified chirp(s).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR queryRange/Selects the query range.
/// HIPAR arraySize/This parameter determines the size of array for Pulse Numbers values.
/// HIPAR arraySize/The Result Values array must contain at least this number of items.
/// HIPAR numbers/This control returns the chirp numbers from the Results table for the
/// HIPAR numbers/specified chirp(s).
/// HIPAR returnedValues/Returns the number of returned values.
ViStatus _VI_FUNC rsspecan_QueryTransientAnalysisChirpNumbers(
                            ViSession instrSession,
                            ViInt32   queryRange,
                            ViInt32   arraySize,
                            ViInt32   numbers[],
                            ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViChar*  p2string;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K60"));
    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &rsspecan_rngQueryRange, queryRange, &p2string),
                2, "Query Range");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SENS:CHIR:NUMB? %s", p2string);
    checkErr(RsCore_QueryIntegerArrayToUserBuffer(instrSession, cmd, arraySize, numbers, returnedValues));

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function returns the calculated chirp frequency parameters
/// HIFN  values.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR parameter/This control selects the frequency parameter.
/// HIPAR resultType/Selects measurement type.
/// HIPAR queryRange/Selects the query range.
/// HIPAR arraySize/This parameter determines the size of array result values. The Result
/// HIPAR arraySize/Values array must contain at least this number of items.
/// HIPAR results/Returns selected frequency results.
/// HIPAR returnedValues/Returns the number of returned values.
ViStatus _VI_FUNC rsspecan_QueryTransientAnalysisChirpResultFrequency(
                            ViSession instrSession,
                            ViInt32   parameter,
                            ViInt32   resultType,
                            ViInt32   queryRange,
                            ViInt32   arraySize,
                            ViReal64  results[],
                            ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViChar*  p2parameter;
    ViChar*  p2resultType;
    ViChar*  p2queryRange;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K60"));

    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &rsspecan_rngTransientParameterChirpFrequency, parameter, &p2parameter),
                2, "Parameter");
    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &rsspecan_rngResultType, resultType, &p2resultType),
                3, "Result Type");
    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &rsspecan_rngQueryRange, queryRange, &p2queryRange),
                4, "Query Range");

    if (p2resultType == NULL) p2resultType = "";

    if (queryRange == RSSPECAN_VAL_QUERY_RANGE_SELECTED && resultType != RSSPECAN_VAL_MEASTYPE_CURR)
    {
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, queryRange),
                    4, "Query Range");
    }

    if ((parameter <= RSSPECAN_VAL_CHIRP_RESULT_FREQUENCY_BWID) && (parameter >= RSSPECAN_VAL_CHIRP_RESULT_FREQUENCY_OVER))
    {
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SENS:CHIR:FREQ:%s%s? %s", p2parameter, p2resultType, p2queryRange);
    }
    else if (parameter <= RSSPECAN_VAL_CHIRP_RESULT_FREQUENCY_FMST)
    {
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SENS:CHIR:FMS:%s%s? %s", p2parameter, p2resultType, p2queryRange);
    }
    else
    {
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SENS:CHIR:PMS:%s%s? %s", p2parameter, p2resultType, p2queryRange);
    }

    checkErr(RsCore_QueryFloatArrayToUserBuffer(instrSession, cmd, arraySize, results, returnedValues));

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function returns the calculated chirp timing parameters values.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR parameter/This control selects the timing parameter.
/// HIPAR resultType/Selects measurement type.
/// HIPAR queryRange/Selects the query range.
/// HIPAR arraySize/This parameter determines the size of array result values. The Result
/// HIPAR arraySize/Values array must contain at least this number of items.
/// HIPAR results/Returns selected timing results.
/// HIPAR returnedValues/Returns the number of returned values.
ViStatus _VI_FUNC rsspecan_QueryTransientAnalysisChirpResultTiming(
                            ViSession instrSession,
                            ViInt32   parameter,
                            ViInt32   resultType,
                            ViInt32   queryRange,
                            ViInt32   arraySize,
                            ViReal64  results[],
                            ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViChar*  p2parameter;
    ViChar*  p2resultType;
    ViChar*  p2queryRange;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K60"));

    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &rsspecan_rngTransientParameterChirpTiming, parameter, &p2parameter),
                2, "Parameter");
    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &rsspecan_rngResultType, resultType, &p2resultType),
                3, "Result Type");
    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &rsspecan_rngQueryRange, queryRange, &p2queryRange),
                4, "Query Range");

    if (p2resultType == NULL) p2resultType = "";

    if (queryRange == RSSPECAN_VAL_QUERY_RANGE_SELECTED && resultType != RSSPECAN_VAL_MEASTYPE_CURR)
    {
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, queryRange),
                    4, "Query Range");
    }

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SENS:CHIR:TIM:%s%s? %s", p2parameter, p2resultType, p2queryRange);
    checkErr(RsCore_QueryFloatArrayToUserBuffer(instrSession, cmd, arraySize, results, returnedValues));

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function returns the calculated chirp power parameters values.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR parameter/This control selects the power parameter.
/// HIPAR resultType/Selects measurement type.
/// HIPAR queryRange/Selects the query range.
/// HIPAR arraySize/This parameter determines the size of array result values. The Result
/// HIPAR arraySize/Values array must contain at least this number of items.
/// HIPAR results/Returns selected power results.
/// HIPAR returnedValues/Returns the number of returned values.
ViStatus _VI_FUNC rsspecan_QueryTransientAnalysisChirpResultPower(
                            ViSession instrSession,
                            ViInt32   parameter,
                            ViInt32   resultType,
                            ViInt32   queryRange,
                            ViInt32   arraySize,
                            ViReal64  results[],
                            ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViChar*  p2parameter;
    ViChar*  p2resultType;
    ViChar*  p2queryRange;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K60"));

    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &rsspecan_rngTransientParameterChirpPower, parameter, &p2parameter),
                2, "Parameter");
    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &rsspecan_rngResultType, resultType, &p2resultType),
                3, "Result Type");
    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &rsspecan_rngQueryRange, queryRange, &p2queryRange),
                4, "Query Range");

    if (p2resultType == NULL) p2resultType = "";

    if (queryRange == RSSPECAN_VAL_QUERY_RANGE_SELECTED && resultType != RSSPECAN_VAL_MEASTYPE_CURR)
    {
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, queryRange),
                    4, "Query Range");
    }

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SENS:CHIR:POW:%s%s? %s", p2parameter, p2resultType, p2queryRange);
    checkErr(RsCore_QueryFloatArrayToUserBuffer(instrSession, cmd, arraySize, results, returnedValues));

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function returns the calculated chirp phase parameters values.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR parameter/This control selects the power parameter.
/// HIPAR resultType/Selects measurement type.
/// HIPAR queryRange/Selects the query range.
/// HIPAR arraySize/This parameter determines the size of array result values. The Result
/// HIPAR arraySize/Values array must contain at least this number of items.
/// HIPAR results/Returns selected power results.
/// HIPAR returnedValues/Returns the number of returned values.
ViStatus _VI_FUNC rsspecan_QueryTransientAnalysisChirpResultPhase(
                            ViSession instrSession,
                            ViInt32   parameter,
                            ViInt32   resultType,
                            ViInt32   queryRange,
                            ViInt32   arraySize,
                            ViReal64  results[],
                            ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViChar*  p2resultType;
    ViChar*  p2queryRange;
    ViString params[] = {"AVGP", "MAXP", "RMSP", "OVER", "UND"};

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K60"));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, parameter, RSSPECAN_VAL_CHIRP_RESULT_PHASE_AVERAGE, RSSPECAN_VAL_CHIRP_RESULT_PHASE_UNDER),
            2, "Parameter");
    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &rsspecan_rngResultType, resultType, &p2resultType),
                3, "Result Type");
    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &rsspecan_rngQueryRange, queryRange, &p2queryRange),
                4, "Query Range");

    if (p2resultType == NULL) p2resultType = "";

    if (queryRange == RSSPECAN_VAL_QUERY_RANGE_SELECTED && resultType != RSSPECAN_VAL_MEASTYPE_CURR)
    {
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, queryRange),
                    4, "Query Range");
    }

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SENS:CHIR:PHAS:%s%s? %s", params[parameter], p2resultType, p2queryRange);
    checkErr(RsCore_QueryFloatArrayToUserBuffer(instrSession, cmd, arraySize, results, returnedValues));

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function returns the calculated chirp state parameters values.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR resultType/Selects measurement type.
/// HIPAR queryRange/Selects the query range.
/// HIPAR arraySize/This parameter determines the size of array result values. The Result
/// HIPAR arraySize/Values array must contain at least this number of items.
/// HIPAR results/Returns selected state results.
/// HIPAR returnedValues/Returns the number of returned values.
ViStatus _VI_FUNC rsspecan_QueryTransientAnalysisChirpResultState(
                            ViSession instrSession,
                            ViInt32   resultType,
                            ViInt32   queryRange,
                            ViInt32   arraySize,
                            ViReal64  results[],
                            ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViChar*  p2resultType;
    ViChar*  p2queryRange;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K60"));

    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &rsspecan_rngResultType, resultType, &p2resultType),
                2, "Result Type");
    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &rsspecan_rngQueryRange, queryRange, &p2queryRange),
                3, "Query Range");

    if (queryRange == RSSPECAN_VAL_QUERY_RANGE_SELECTED && resultType != RSSPECAN_VAL_MEASTYPE_CURR)
    {
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, queryRange),
                    4, "Query Range");
    }

    if (p2resultType != NULL)
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SENS:CHIR:STAT%s? %s", p2resultType, p2queryRange);
    else
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SENS:CHIR:STAT? %s", p2queryRange);
    checkErr(RsCore_QueryFloatArrayToUserBuffer(instrSession, cmd, arraySize, results, returnedValues));

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function queries the chirp results table. Which values are
/// HIFN  returned depends on the enabled parameters for the results tables (see
/// HIFN  rsspecan_ConfigureTransientResultTable).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR start/This parameter determines the size of array result values. The Result
/// HIPAR start/Values array must contain at least this number of items.
/// HIPAR stop/This parameter determines the size of array result values. The Result
/// HIPAR stop/Values array must contain at least this number of items.
/// HIPAR arraySize/This parameter determines the size of array result values. The Result
/// HIPAR arraySize/Values array must contain at least this number of items.
/// HIPAR results/Returns selected results. Which values are returned depends on the
/// HIPAR results/enabled parameters for the results tables.
/// HIPAR returnedValues/Returns the number of returned values.
ViStatus _VI_FUNC rsspecan_QueryTransientAnalysisChirpResultTable(
                            ViSession instrSession,
                            ViInt32   start,
                            ViInt32   stop,
                            ViInt32   arraySize,
                            ViReal64  results[],
                            ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K60"));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CALC:CHRD:TABL:RES? %ld,%ld", start, stop);
    checkErr(RsCore_QueryFloatArrayToUserBuffer(instrSession, cmd, arraySize, results, returnedValues));

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}
