/*****************************************************************************
 *  Rohde&Schwarz Spectrum Analyzer instrument driver -
 *                              VSA - Vector Signal Analysis
 *  VXIpnp Style Instrument Driver
 *  Original Release: June 2006
 *  By: Martin Koutny (instrument driver core)
 *      Helena Sucha (instrument specific implementation)
 *      Jiri Kominek (DrBase repository & tools)
 *
 *  Should you have any technical questions please contact the hotline of
 *  Rohde & Schwarz Vertriebs-GmbH Rohde & Schwarz Support Center
 *
 *  e-mail: CustomerSupport@rohde-schwarz.com
 *
 *****************************************************************************/

/*****************************************************************************
 *  VSA - Vector Signal Analysis
 *****************************************************************************/

#include <math.h>
#include "rsspecan.h"
#include "rscore.h"

static ViString VSAResultArr[] = {
    "DDEM:STAT:MPOW", "", "DDEM:STAT:ADR", "DDEM:STAT:CFER", "DDEM:STAT:EVM",
    "DDEM:STAT:FSK:CFDR", "DDEM:STAT:FSK:DERR", "DDEM:STAT:FSK:MDEV", "DDEM:STAT:IQIM",
    "DDEM:STAT:GIMB", "DDEM:STAT:MERR", "DDEM:STAT:OOFF", "DDEM:STAT:PERR", "",
    "DDEM:STAT:QERR", "DDEM:STAT:RHO", "DDEM:STAT:SNR", "DDEM:STAT:FDER", "DDEM:STAT:FSK:RDEV",
    "DDEM:STAT:SRER", "DDEM:STAT:IQSK",NULL
};
static ViString VSAModifierArr[] = {
    "", "RMS", "AVG", "SDEV", "TPE", "PEAK", "ASYM", "PCTL", "CURR", "AVG", "RPE", "SDEV", "PCTL", "PEAK", "PAVG",
    "TPE", "PSD", "PPCT",NULL
};
static ViString VSACatPatternArr[] = {"ALL", "CURR",NULL};
static ViString VSATraceArr[] = {
    "", "TRACE1", "TRACE2", "TRACE3", "TRACE4", "TRACE5", "TRACE6", "TRACE1R", "TRACE2R", "TRACE3R",
    "TRACE1I", "TRACE2I", "TRACE3I", "TRACEIQCX", "TRACEIQCY", NULL
};

/// HIFN  This function switches the instrument into Vector Signal Analysis
/// HIFN  mode.
/// HIFN
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsspecan_SetVSAMode(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_VSA_MODE, ""));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function restores the factory settings of the following
/// HIFN  parameters for the K70 option.
/// HIFN
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR defaults/Selects the item to be set to factory default.
ViStatus _VI_FUNC rsspecan_SetVSAFactoryDefaults(ViSession instrSession,
                                                 ViInt32   defaults)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    if ((defaults != RSSPECAN_VAL_VSA_FACTORY_STAN) &&
        (defaults != RSSPECAN_VAL_VSA_FACTORY_PATT) &&
        (defaults != RSSPECAN_VAL_VSA_FACTORY_ALL))
    {
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, defaults),
                    2, "Defaults");
    }

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_VSA_FACTORY_DEFAULTS, defaults),
            2, "Defaults");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function presets the screens.
/// HIFN
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsspecan_PresetVSAScreens(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_VSA_PRESET_SCREENS, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function initiates automatic setting of the RF attenuation and
/// HIFN  IF gain to the level of the applied signal.
/// HIFN
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR timeout/This control sets the timeout value.
ViStatus _VI_FUNC rsspecan_VSARefLevelAdjust(ViSession instrSession,
                                             ViInt32   window,
                                             ViUInt32  timeout)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE];
    ViUInt32 old_timeout = 0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViUInt32Range(instrSession, timeout, 0, 4294967295), 3, "Timeout");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "Win%ld", window);

    checkErr(rsspecan_GetTimeout(instrSession, &old_timeout));
    checkErr(rsspecan_SetTimeout(instrSession, timeout));
    checkErr(rsspecan_SetAttributeViString(instrSession, repCap, RSSPECAN_ATTR_VSA_REF_LEVEL_ADJUST, ""));

Error:
    if (old_timeout > 0)
        rsspecan_SetTimeout(instrSession, old_timeout);

    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function selects an automatic setting of all modulation
/// HIFN  parameters according to a standardized transmission method or a
/// HIFN  user-defined transmission method. The standardized transmission methods
/// HIFN  are available in the instrument as predefined standards.
/// HIFN
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR digitalStandard/Specifies the file name that contains the transmission method
ViStatus _VI_FUNC rsspecan_VSADigitalStandardLoad(ViSession instrSession,
                                                  ViInt32   reserved,
                                                  ViString  digitalStandard)
{
    ViStatus error = VI_SUCCESS;
             reserved;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_VSA_DIGITAL_STANDARD, digitalStandard),
            3, "Digital Standard");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function stores the current settings of the vector signal
/// HIFN  analysis as a new user-defined digital standard. If the name of the
/// HIFN  digital standard is already in use, an error message is output and a
/// HIFN  new name has to be selected. It is recommended to define a group, a
/// HIFN  comment and a prefix before storing the standard.
/// HIFN
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR userDefinedStandard/Define name of the user-defined standard. If the name of the digital
/// HIPAR userDefinedStandard/standard is already in use, an error message is output and a new name
/// HIPAR userDefinedStandard/has to be selected.
/// HIPAR comment/Enters the comment for a new standard. The comment is stored with the
/// HIPAR comment/standard and is only displayed in the selection menu (manual
/// HIPAR comment/operation). When remote control is used, the string is deleted after
/// HIPAR comment/the standard has been stored, allowing a new comment to be entered for
/// HIPAR comment/the next standard.
ViStatus _VI_FUNC rsspecan_VSADigitalStandardSave(ViSession instrSession,
                                                  ViInt32   reserved,
                                                  ViString  comment,
                                                  ViString  userDefinedStandard)
{
    ViStatus error = VI_SUCCESS;
             reserved;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_VSA_DIGITAL_STANDARD_COMMENT, comment),
            3, "Comment");

    viCheckParm(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_VSA_DIGITAL_STANDARD_SAVE, userDefinedStandard),
            4, "User Defined Standard");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function deletes the selected digital standard if it is a
/// HIFN  user-defined standard. If the standard is predefined, it cannot be
/// HIFN  deleted (error message).
/// HIFN
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR digitalStandard/Select digital standard if it is a user-defined standard. If the
/// HIPAR digitalStandard/standard is predefined, it cannot be deleted (error message).
ViStatus _VI_FUNC rsspecan_VSADigitalStandardDelete(ViSession instrSession,
                                                    ViInt32   reserved,
                                                    ViString  digitalStandard)
{
    ViStatus error = VI_SUCCESS;
             reserved;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_VSA_DIGITAL_STANDARD_DELETE, digitalStandard),
            3, "Digital Standard");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function restores the default settings of the currently selected
/// HIFN  standard.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
ViStatus _VI_FUNC rsspecan_VSADigitalStandardPreset(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_VSA_DIGITAL_STANDARD_PRESET, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function enables or disables a digital output stream to the
/// HIFN  optional QSFP+ connector, if available.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR enabled/Enables or disables a digital output stream to the optional QSFP+
/// HIPAR enabled/connector, if available.
ViStatus _VI_FUNC rsspecan_ConfigureVSADigitalIQ40GEnabled(ViSession instrSession,
                                                           ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_VSA_DIGITAL_IQ_40G_ENABLED, enabled),
            2, "Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function returns the currently used sample rate to transfer data
/// HIFN  via the Digital I/Q 40G Streaming Output interface.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR sampleRate/Returns the currently used sample rate to transfer data via the
/// HIPAR sampleRate/Digital I/Q 40G Streaming Output interface.
ViStatus _VI_FUNC rsspecan_QueryVSADigitalIQ40GSampleRate(ViSession instrSession,
                                                          ViReal64* sampleRate)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_VSA_DIGITAL_IQ_40G_SAMPLE_RATE, sampleRate),
            2, "Sample Rate");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function returns a comma-separated list of information on the
/// HIFN  instrument connected to the QSFP+ connector, if available.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR connectedDevice/Returns a comma-separated list of information on the instrument
/// HIPAR connectedDevice/connected to the QSFP+ connector, if available.
ViStatus _VI_FUNC rsspecan_QueryVSADigitalIQ40GConnectedDevice(
                            ViSession instrSession,
                            ViChar    connectedDevice[])
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_GetAttributeViString(instrSession, "", RSSPECAN_ATTR_VSA_DIGITAL_IQ_40G_CONNECTED_DEVICE,
            1024, connectedDevice),
            2, "Connected Device");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function configures the modulation.
/// HIFN
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR symbolRate/Sets the symbol rate.
/// HIPAR mappingName/Selects the mapping designated by Mapping Name for the digital
/// HIPAR mappingName/demodulation.
/// HIPAR modulation/Selects modulation type/order.
ViStatus _VI_FUNC rsspecan_ConfigureVSAModulationSettings(ViSession instrSession,
                                                          ViInt32   modulation,
                                                          ViString  mappingName,
                                                          ViReal64  symbolRate)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_VSA_MODULATION_SRATE, symbolRate),
            4, "Symbol Rate");

    switch (modulation)
    {
        //QPSK
    case RSSPECAN_VAL_VSA_MOD_ORDER_BPSK:
        checkErr(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_VSA_MODULATION_FORMAT, RSSPECAN_VAL_VSA_MOD_FORMAT_PSK)); // 1
        checkErr(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_VSA_MODULATION_PSK_NSTAT, 2));
        break;

    case RSSPECAN_VAL_VSA_MOD_ORDER_QPSK: //0
        checkErr(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_VSA_MODULATION_FORMAT, RSSPECAN_VAL_VSA_MOD_FORMAT_QPSK));
        checkErr(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_VSA_MODULATION_QPSK_FORMAT, RSSPECAN_VAL_MOD_QPSK_NORM));
        break;

    case RSSPECAN_VAL_VSA_MOD_ORDER_OQPSK:
        checkErr(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_VSA_MODULATION_FORMAT, RSSPECAN_VAL_VSA_MOD_FORMAT_QPSK));
        checkErr(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_VSA_MODULATION_QPSK_FORMAT, RSSPECAN_VAL_MOD_QPSK_OFFS));
        break;

    case RSSPECAN_VAL_VSA_MOD_ORDER_DQPSK:
        checkErr(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_VSA_MODULATION_FORMAT, RSSPECAN_VAL_VSA_MOD_FORMAT_QPSK));
        checkErr(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_VSA_MODULATION_QPSK_FORMAT, RSSPECAN_VAL_MOD_QPSK_DIFF));
        break;

    case RSSPECAN_VAL_VSA_MOD_ORDER_PI4QPSK:
        checkErr(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_VSA_MODULATION_FORMAT, RSSPECAN_VAL_VSA_MOD_FORMAT_QPSK));
        checkErr(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_VSA_MODULATION_QPSK_FORMAT, RSSPECAN_VAL_MOD_QPSK_NPI4));
        break;

    case RSSPECAN_VAL_VSA_MOD_ORDER_DPI4QPSK:
        checkErr(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_VSA_MODULATION_FORMAT, RSSPECAN_VAL_VSA_MOD_FORMAT_QPSK));
        checkErr(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_VSA_MODULATION_QPSK_FORMAT, RSSPECAN_VAL_MOD_QPSK_DPI4));
        break;

    case RSSPECAN_VAL_VSA_MOD_ORDER_3PI4QPSK:
        checkErr(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_VSA_MODULATION_FORMAT, RSSPECAN_VAL_VSA_MOD_FORMAT_QPSK));
        checkErr(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_VSA_MODULATION_QPSK_FORMAT, RSSPECAN_VAL_MOD_QPSK_N3PI4));
        break;

    case RSSPECAN_VAL_VSA_MOD_ORDER_8PSK:
        checkErr(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_VSA_MODULATION_FORMAT, RSSPECAN_VAL_VSA_MOD_FORMAT_PSK)); //1
        checkErr(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_VSA_MODULATION_PSK_NSTAT, 8));
        checkErr(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_VSA_MODULATION_PSK_FORMAT, RSSPECAN_VAL_MOD_PSK_NORM));
        break;

    case RSSPECAN_VAL_VSA_MOD_ORDER_D8PSK:
        checkErr(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_VSA_MODULATION_FORMAT, RSSPECAN_VAL_VSA_MOD_FORMAT_PSK)); //1
        checkErr(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_VSA_MODULATION_PSK_NSTAT, 8));
        checkErr(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_VSA_MODULATION_PSK_FORMAT, RSSPECAN_VAL_MOD_PSK_DIFF));
        break;

    case RSSPECAN_VAL_VSA_MOD_ORDER_N3PI8PSK:
        checkErr(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_VSA_MODULATION_FORMAT, RSSPECAN_VAL_VSA_MOD_FORMAT_PSK)); //1
        checkErr(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_VSA_MODULATION_PSK_NSTAT, 8));
        checkErr(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_VSA_MODULATION_PSK_FORMAT, RSSPECAN_VAL_MOD_PSK_N3PI8));
        break;

    case RSSPECAN_VAL_VSA_MOD_ORDER_PI8D8PSK:
        checkErr(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_VSA_MODULATION_FORMAT, RSSPECAN_VAL_VSA_MOD_FORMAT_PSK)); //1
        checkErr(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_VSA_MODULATION_PSK_NSTAT, 8));
        checkErr(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_VSA_MODULATION_PSK_FORMAT, RSSPECAN_VAL_MOD_PSK_PI8D8PSK));
        break;

    case RSSPECAN_VAL_VSA_MOD_ORDER_NPI2:
        checkErr(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_VSA_MODULATION_FORMAT, RSSPECAN_VAL_VSA_MOD_FORMAT_PSK)); //1
        checkErr(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_VSA_MODULATION_PSK_NSTAT, 2));
        checkErr(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_VSA_MODULATION_PSK_FORMAT, RSSPECAN_VAL_MOD_PSK_NPI2));
        break;

    case RSSPECAN_VAL_VSA_MOD_ORDER_DPI2:
        checkErr(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_VSA_MODULATION_FORMAT, RSSPECAN_VAL_VSA_MOD_FORMAT_PSK)); //1
        checkErr(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_VSA_MODULATION_PSK_NSTAT, 2));
        checkErr(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_VSA_MODULATION_PSK_FORMAT, RSSPECAN_VAL_MOD_PSK_DPI2));
        break;

    case RSSPECAN_VAL_VSA_MOD_ORDER_SOQPSK:
        checkErr(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_VSA_MODULATION_FORMAT, RSSPECAN_VAL_VSA_MOD_FORMAT_QPSK));
        checkErr(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_VSA_MODULATION_QPSK_FORMAT, RSSPECAN_VAL_MOD_QPSK_SOFF));
        break;
    case RSSPECAN_VAL_VSA_MOD_ORDER_MNPI2:
        checkErr(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_VSA_MODULATION_FORMAT, RSSPECAN_VAL_VSA_MOD_FORMAT_PSK));
        checkErr(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_VSA_MODULATION_PSK_NSTAT, 2));
        checkErr(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_VSA_MODULATION_PSK_FORMAT, RSSPECAN_VAL_MOD_PSK_MNPI2));
        break;
        //MSK
    case RSSPECAN_VAL_VSA_MOD_ORDER_MSK:
        checkErr(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_VSA_MODULATION_FORMAT, RSSPECAN_VAL_VSA_MOD_FORMAT_MSK)); //2
        checkErr(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_VSA_MODULATION_MSK_FORMAT, RSSPECAN_VAL_MOD_MSK_NORM));
        break;

    case RSSPECAN_VAL_VSA_MOD_ORDER_DMSK:
        checkErr(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_VSA_MODULATION_FORMAT, RSSPECAN_VAL_VSA_MOD_FORMAT_MSK)); //2
        checkErr(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_VSA_MODULATION_MSK_FORMAT, RSSPECAN_VAL_MOD_MSK_DIFF));
        break;

        //QAM
    case RSSPECAN_VAL_VSA_MOD_ORDER_16QAM:
        checkErr(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_VSA_MODULATION_FORMAT, RSSPECAN_VAL_VSA_MOD_FORMAT_QAM)); //3
        checkErr(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_VSA_MODULATION_QAM_FORMAT, RSSPECAN_VAL_MOD_QAM_NORM));
        checkErr(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_VSA_MODULATION_QAM_NSTAT, 16));
        break;

    case RSSPECAN_VAL_VSA_MOD_ORDER_PI4_16QAM:
        checkErr(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_VSA_MODULATION_FORMAT, RSSPECAN_VAL_VSA_MOD_FORMAT_QAM)); //3
        checkErr(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_VSA_MODULATION_QAM_FORMAT, RSSPECAN_VAL_MOD_QAM_NPI4));
        checkErr(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_VSA_MODULATION_QAM_NSTAT, 16));
        break;

    case RSSPECAN_VAL_VSA_MOD_ORDER_32QAM:
        checkErr(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_VSA_MODULATION_FORMAT, RSSPECAN_VAL_VSA_MOD_FORMAT_QAM)); //3
        checkErr(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_VSA_MODULATION_QAM_FORMAT, RSSPECAN_VAL_MOD_QAM_NORM));
        checkErr(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_VSA_MODULATION_QAM_NSTAT, 32));
        break;

    case RSSPECAN_VAL_VSA_MOD_ORDER_NPI4_32QAM:
        checkErr(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_VSA_MODULATION_FORMAT, RSSPECAN_VAL_VSA_MOD_FORMAT_QAM)); //3
        checkErr(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_VSA_MODULATION_QAM_FORMAT, RSSPECAN_VAL_MOD_QAM_MNPI4));
        checkErr(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_VSA_MODULATION_QAM_NSTAT, 32));
        break;

    case RSSPECAN_VAL_VSA_MOD_ORDER_64QAM:
        checkErr(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_VSA_MODULATION_FORMAT, RSSPECAN_VAL_VSA_MOD_FORMAT_QAM)); //3
        checkErr(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_VSA_MODULATION_QAM_FORMAT, RSSPECAN_VAL_MOD_QAM_NORM));
        checkErr(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_VSA_MODULATION_QAM_NSTAT, 64));
        break;

    case RSSPECAN_VAL_VSA_MOD_ORDER_128QAM:
        checkErr(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_VSA_MODULATION_FORMAT, RSSPECAN_VAL_VSA_MOD_FORMAT_QAM)); //3
        checkErr(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_VSA_MODULATION_QAM_FORMAT, RSSPECAN_VAL_MOD_QAM_NORM));
        checkErr(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_VSA_MODULATION_QAM_NSTAT, 128));
        break;

    case RSSPECAN_VAL_VSA_MOD_ORDER_256QAM:
        checkErr(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_VSA_MODULATION_FORMAT, RSSPECAN_VAL_VSA_MOD_FORMAT_QAM)); //3
        checkErr(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_VSA_MODULATION_QAM_FORMAT, RSSPECAN_VAL_MOD_QAM_NORM));
        checkErr(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_VSA_MODULATION_QAM_NSTAT, 256));
        break;

        //FSK
    case RSSPECAN_VAL_VSA_MOD_ORDER_2FSK:
        checkErr(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_VSA_MODULATION_FORMAT, RSSPECAN_VAL_VSA_MOD_FORMAT_FSK)); //4
        checkErr(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_VSA_MODULATION_FSK_NSTAT, 2));
        break;
    case RSSPECAN_VAL_VSA_MOD_ORDER_4FSK:
        checkErr(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_VSA_MODULATION_FORMAT, RSSPECAN_VAL_VSA_MOD_FORMAT_FSK)); //4
        checkErr(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_VSA_MODULATION_FSK_NSTAT, 4));
        break;
    case RSSPECAN_VAL_VSA_MOD_ORDER_8FSK:
        checkErr(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_VSA_MODULATION_FORMAT, RSSPECAN_VAL_VSA_MOD_FORMAT_FSK)); //4
        checkErr(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_VSA_MODULATION_FSK_NSTAT, 8));
        break;

        //User QAM
    case RSSPECAN_VAL_VSA_MOD_ORDER_USER_2QAM:
        checkErr(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_VSA_MODULATION_FORMAT, RSSPECAN_VAL_VSA_MOD_FORMAT_UQAM)); //6
        if (rsspecan_IsFSV(instrSession))
        {
            checkErr(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_VSA_MODULATION_UQAM_MAPPING, "2ary"));
        }
        else
        {
            checkErr(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_VSA_MODULATION_MAPPING, "2ary"));
        }
        break;

    case RSSPECAN_VAL_VSA_MOD_ORDER_USER_4QAM:
        checkErr(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_VSA_MODULATION_FORMAT, RSSPECAN_VAL_VSA_MOD_FORMAT_UQAM)); //6
        if (rsspecan_IsFSV(instrSession))
        {
            checkErr(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_VSA_MODULATION_UQAM_MAPPING, "4ary"));
        }
        else
        {
            checkErr(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_VSA_MODULATION_MAPPING, "4ary"));
        }
        break;

    case RSSPECAN_VAL_VSA_MOD_ORDER_USER_16QAM:
        checkErr(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_VSA_MODULATION_FORMAT, RSSPECAN_VAL_VSA_MOD_FORMAT_UQAM)); //6
        if (rsspecan_IsFSV(instrSession))
        {
            checkErr(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_VSA_MODULATION_UQAM_MAPPING, "16ary"));
        }
        else
        {
            checkErr(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_VSA_MODULATION_MAPPING, "16ary"));
        }
        break;

    case RSSPECAN_VAL_VSA_MOD_ORDER_USER_32QAM:
        checkErr(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_VSA_MODULATION_FORMAT, RSSPECAN_VAL_VSA_MOD_FORMAT_UQAM)); //6
        if (rsspecan_IsFSV(instrSession))
        {
            checkErr(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_VSA_MODULATION_UQAM_MAPPING, "32ary"));
        }
        else
        {
            checkErr(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_VSA_MODULATION_MAPPING, "32ary"));
        }
        break;

        //ASK
    case RSSPECAN_VAL_VSA_MOD_ORDER_2ASK:
        checkErr(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_VSA_MODULATION_FORMAT, RSSPECAN_VAL_VSA_MOD_FORMAT_ASK)); //7
        checkErr(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_VSA_MODULATION_ASK_MODE, 2));
        break;

    case RSSPECAN_VAL_VSA_MOD_ORDER_4ASK:
        checkErr(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_VSA_MODULATION_FORMAT, RSSPECAN_VAL_VSA_MOD_FORMAT_ASK)); //7
        checkErr(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_VSA_MODULATION_ASK_MODE, 4));
        break;

        //APSK
    case RSSPECAN_VAL_VSA_MOD_ORDER_16APSK:
        checkErr(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_VSA_MODULATION_FORMAT, RSSPECAN_VAL_VSA_MOD_FORMAT_APSK)); //8
        checkErr(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_VSA_MODULATION_APSK_MODE, 16));
        break;

    case RSSPECAN_VAL_VSA_MOD_ORDER_32APSK:
        checkErr(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_VSA_MODULATION_FORMAT, RSSPECAN_VAL_VSA_MOD_FORMAT_APSK)); //8
        checkErr(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_VSA_MODULATION_APSK_MODE, 32));
        break;

    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, modulation), 2, "Modulation");
        break;
    }

    switch (modulation)
    {
    case RSSPECAN_VAL_VSA_MOD_ORDER_USER_2QAM:
    case RSSPECAN_VAL_VSA_MOD_ORDER_USER_4QAM:
    case RSSPECAN_VAL_VSA_MOD_ORDER_USER_16QAM:
    case RSSPECAN_VAL_VSA_MOD_ORDER_USER_32QAM:
        break;
    default:
        if (mappingName && (strlen(mappingName) > 0))
        {
            viCheckParm(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_VSA_MODULATION_MAPPING, mappingName),
                    3, "Mapping Name");
        }
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the reference value of the frequency
/// HIFN  deviation for FSK modulation.
/// HIFN
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR referenceDeviation/Sets the reference value of the frequency deviation.
/// HIPAR mode/This control defines if Reference Deviation value is absolute or
/// HIPAR mode/relative to the symbol rate.
ViStatus _VI_FUNC rsspecan_ConfigureVSAFSKRefDeviation(ViSession instrSession,
                                                       ViReal64  referenceDeviation,
                                                       ViInt32   mode)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    switch (mode)
    {
    case RSSPECAN_VAL_VSA_REF_DEV_MODE_ABS:
        viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_VSA_MODULATION_FSK_DEV, referenceDeviation),
                2, "Reference Deviation");
        break;

    case RSSPECAN_VAL_VSA_REF_DEV_MODE_REL:
        viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_VSA_MODULATION_FSK_DEV_REL, referenceDeviation),
                2, "Reference Deviation");
        break;

    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, mode), 3, "Mode");
        break;
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function configures the TX filter.
/// HIFN
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR type/Defines the type of transmit filter.
/// HIPAR alphaBT/Determines the filter characteristic (ALPHA/BT).
ViStatus _VI_FUNC rsspecan_ConfigureVSATXFilter(ViSession instrSession,
                                                ViInt32   type,
                                                ViReal64  alphaBT)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    if (type == RSSPECAN_VAL_VSA_TX_FILTER_TYPE_NONE)
    {
        viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_VSA_TX_FILTER_STATE, VI_FALSE),
                4, "");
    }
    else
    {
        viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_VSA_TX_FILTER_TYPE, type),
                2, "Type");

        if (type <= RSSPECAN_VAL_VSA_TX_FILTER_TYPE_GMSK)
        {
            viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_VSA_TX_FILTER_ALPHA, alphaBT),
                    3, "Alpha BT");
        }
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function selects a user-defined transmit filter file.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR filterFileName/The name of the Transmit filter file name.
ViStatus _VI_FUNC rsspecan_ConfigureVSAUserDefinedTXFilter(ViSession instrSession,
                                                           ViString  filterFileName)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    if (filterFileName == NULL)
    {
        checkErr(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_VSA_TX_FILTER_STATE, VI_FALSE));
    }
    else
    {
        checkErr(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_VSA_TX_FILTER_STATE, VI_TRUE));
        checkErr(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_VSA_TX_FILTER_TYPE, RSSPECAN_VAL_VSA_TX_FILTER_USER));

        viCheckParm(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_VSA_TX_FILTER_USER_FILE, filterFileName),
                2, "Filter File Name");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function reads the names of all mappings stored on the hard disk.
/// HIFN  A mapping describes the assignment of constellation points to symbols.
/// HIFN
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR numberofMappings/This control returns number of the names of all digital standards
/// HIPAR numberofMappings/available.
/// HIPAR bufferSize/Pass the number of bytes in the string you specify for the catalog
/// HIPAR bufferSize/list parameter.
/// HIPAR mappingsList/This control returns array of the names of all digital standards
/// HIPAR mappingsList/available in CSV form.
ViStatus _VI_FUNC rsspecan_GetVSAModulationMappingCatalog(ViSession instrSession,
                                                          ViInt32*  numberofMappings,
                                                          ViInt32   bufferSize,
                                                          ViChar    mappingsList[])
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K70"));

    viCheckParm(RsCore_InvalidViInt32RangeMin(instrSession, bufferSize, 0),
                3, "Buffer Size");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, mappingsList),
                4, "Mapping List");

    checkErr(RsCore_QueryCatalog(instrSession, ":SENS:DDEM:MAPP:CAT?", bufferSize, mappingsList, numberofMappings));

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function returns the specific modulation level for FQAM.
/// HIFN
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR specificLevel/This control returns the specific modulation level for FQAM.
ViStatus _VI_FUNC rsspecan_GetVSAModulationUQAMLevel(ViSession instrSession,
                                                     ViInt32*  specificLevel)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_GetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_VSA_MODULATION_UQAM_NSTAT, specificLevel),
            2, "Specific Level");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function selects the file that contains the user-defined
/// HIFN  modulation to be loaded.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR userModulation/Selects the file that contains the user-defined modulation to be loaded.
ViStatus _VI_FUNC rsspecan_VSALoadUserModulation(ViSession instrSession,
                                                 ViString  userModulation)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_VSA_MODULATION_FORMAT, RSSPECAN_VAL_VSA_MOD_FORMAT_UQAM));

    viCheckParm(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_VSA_LOAD_USER_MODULATION, userModulation),
            2, "User Modulation");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function sets the instrument to look for a bursted or a
/// HIFN  continuous signal.
/// HIFN
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR signalType/This control sets the instrument to look for a bursted or a continuous
/// HIPAR signalType/signal.
ViStatus _VI_FUNC rsspecan_ConfigureVSASignalType(ViSession instrSession,
                                                  ViInt32   signalType)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_VSA_SIGNAL_TYPE, signalType),
            2, "Signal Type");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function defines the settings that are used to accurately control
/// HIFN  the behaviour of the burst search even under difficult measurement
/// HIFN  conditions.
/// HIFN
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR usefulLength/Sets the minimum length of a burst.
/// HIPAR maxLength/Sets the maximum length of a burst. Only those bursts will be
/// HIPAR maxLength/recognized that fall below this length.
/// HIPAR runIn/Sets the length of the rising burst edge which was not considered when
/// HIPAR runIn/evaluating the result.
/// HIPAR runOut/Sets the length of the falling burst edge which is not considered when
/// HIPAR runOut/evaluating the result.
ViStatus _VI_FUNC rsspecan_ConfigureVSABurstSignal(ViSession instrSession,
                                                   ViInt32   usefulLength,
                                                   ViInt32   maxLength,
                                                   ViInt32   runIn,
                                                   ViInt32   runOut)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_VSA_BURST_SEARCH_SKIP_RISE, runIn),
            4, "Run In");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_VSA_BURST_SEARCH_SKIP_FALL, runOut),
            5, "Run Out");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_VSA_BURST_SEARCH_LENGTH_MIN, usefulLength),
            2, "Useful Length");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_VSA_BURST_SEARCH_LENGTH_MAX, maxLength),
            3, "Max Length");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function sets the instrument to look for a pattern.
/// HIFN
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR patternSignal/Sets the instrument to look for a pattern.
ViStatus _VI_FUNC rsspecan_ConfigureVSAPatternSignalState(ViSession instrSession,
                                                          ViBoolean patternSignal)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_VSA_PATTERN_SIGNAL_STATE, patternSignal),
            2, "Pattern Signal");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function configures the pattern offset. If the position of
/// HIFN     the pattern within the burst is known, it is recommended that you
/// HIFN  define the offset. That will accelerate the pattern search and enhance
/// HIFN  the accuracy of the burst search.
/// HIFN
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR patternOffsetState/This control (de)activates the pattern offset.
/// HIPAR patternOffset/Defines a number of symbols which are ignored before the comparison
/// HIPAR patternOffset/with the pattern starts.
ViStatus _VI_FUNC rsspecan_ConfigureVSAPatternOffset(ViSession instrSession,
                                                     ViBoolean patternOffsetState,
                                                     ViInt32   patternOffset)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_VSA_PATTERN_OFFSET_STATE, patternOffsetState),
            2, "Pattern Offset State");

    if (patternOffsetState == VI_TRUE)
    {
        viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_VSA_FIT_SYNC_OFFSET, patternOffset),
                3, "Pattern Offset");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the Modulation Settings for Pattern.
/// HIRET Returns the status code of this operation.
/// HIPAR
/// HIPAR    Note(s) :
/// HIPAR
/// HIPAR    (1) If RSSPECAN_ATTR_VSA_FRAME_SAME_AS_DATA_SYMBOLS is set as on, only QAMand PSK
/// HIPAR    modulations are available.
/// HIPAR
/// HIPAR    (2) If RSSPECAN_ATTR_VSA_FRAME_SAME_AS_DATA_SYMBOLS is set as off, pattern modulation
/// HIPAR    can't be changed directly, only queried. To change modulation use RSSPECAN_ATTR_VSA_MODULATION_FORMAT.
/// HIPAR
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR sameAsDataSymbols/Determines whether the pattern uses a different modulation type than
/// HIPAR sameAsDataSymbols/the data symbols.
/// HIPAR type/Sets the VSA Frame Format.
ViStatus _VI_FUNC rsspecan_ConfigureVSAFramePattern(ViSession instrSession,
                                                    ViBoolean sameAsDataSymbols,
                                                    ViInt32   type)
{
    ViStatus                          error = VI_SUCCESS;
    ViInt32                           modulationArr[8] = {
    RSSPECAN_VAL_VSA_MOD_FORMAT_MSK,  RSSPECAN_VAL_VSA_MOD_FORMAT_PSK, RSSPECAN_VAL_VSA_MOD_FORMAT_QAM,
    RSSPECAN_VAL_VSA_MOD_FORMAT_QPSK, RSSPECAN_VAL_VSA_MOD_FORMAT_FSK, RSSPECAN_VAL_VSA_MOD_FORMAT_ASK,
    RSSPECAN_VAL_VSA_MOD_FORMAT_APSK, RSSPECAN_VAL_VSA_MOD_FORMAT_UQAM
                                      };

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_VSA_FRAME_SAME_AS_DATA_SYMBOLS, sameAsDataSymbols),
            2, "Same As Data Symbols");

    if (sameAsDataSymbols == VI_TRUE)
    {
        viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_VSA_FRAME_MODULATION_TYPE, type),
                3, "Type");
    }
    else
    {
        viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_VSA_MODULATION_FORMAT, modulationArr[type]),
                3, "Type");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function reads the names of all mappings stored on the hard disk.
/// HIFN  A mapping describes the assignment of constellation points to symbols.
/// HIFN
/// HIFN     Notes:
/// HIFN
/// HIFN     (1) Driver returns an instrument specific error, when catalog list is
/// HIFN  empty.
/// HIFN
/// HIFN     (2) This function is available only in conjunction with option FS-K70.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR bufferSize/Pass the number of bytes in the string you specify for the catalog
/// HIPAR bufferSize/list parameter.
/// HIPAR mappingsList/This control returns array of the names of all digital standards
/// HIPAR mappingsList/available in CSV form.
ViStatus _VI_FUNC rsspecan_GetVSAFrameMappingCatalog(ViSession instrSession,
                                                     ViInt32   bufferSize,
                                                     ViChar    mappingsList[])
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_GetAttributeViString(instrSession, "", RSSPECAN_ATTR_VSA_FRAME_MAPPINGS_CATALOG,
            bufferSize, mappingsList),
            3, "Mapping List");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function specifies an xml file for a user-defined frame structure
/// HIFN  configuration. The default storage location for such files is
/// HIFN  C:\R_S\INSTR\USER\vsa\FrameRangeStructure.
/// HIFN     This attribute is only available if the additional Multi-Modulation
/// HIFN  Analysis option (R&S FSW-K70M) is installed.
/// HIFN     If the specified file already exists, it is loaded for subsequent
/// HIFN  editing. Note that this command is a prerequisite to editing the frame
/// HIFN  structure of an existing file (using
/// HIFN  [SENSe:]DDEMod:PATTern:FRAMe:EDIT:STRucture or any other command
/// HIFN  starting with [SENS:]DDEM:PATT:FRAM:EDIT). It does not load the file
/// HIFN  for use in the current measurement (see
/// HIFN  [SENSe:]DDEMod:PATTern:FRAMe:LOAD). Therefore, you can edit a frame
/// HIFN  structure while simultaneously performing a measurement with another
/// HIFN  frame structure configuration.
/// HIFN     If the file does not yet exist, a new frame structure is created and
/// HIFN  will be stored to the specified file when the
/// HIFN  [SENSe:]DDEMod:PATTern:FRAMe:EDIT:SAVE command is executed.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR config/Loads a user-defined frame config file.
ViStatus _VI_FUNC rsspecan_VSALoadFrameConfig(ViSession instrSession,
                                              ViString  config)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_VSA_FRAME_CONFIG, config),
            2, "Config");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function selects the mapping for pattern demodulation.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR fileName/Selects the mapping for pattern demodulation.
ViStatus _VI_FUNC rsspecan_ConfigureVSAFrameMappingSelect(ViSession instrSession,
                                                          ViString  fileName)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_VSA_FRAME_MAPPING_SELECT, fileName),
            2, "File Name");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function loads a user-defined frame structure configuration to be
/// HIFN  used by the measurement from an xml file.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR frameStructure/Loads a user-defined frame structure configuration to be used by the
/// HIPAR frameStructure/measurement from an xml file.
ViStatus _VI_FUNC rsspecan_VSALoadFrameStructure(ViSession instrSession,
                                                 ViString  frameStructure)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_VSA_FRAME_LOAD, frameStructure),
            2, "Frame Structure");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function saves a user-defined frame structure configuration to be
/// HIFN  used by the measurement from an xml file.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR frameStructure/Saves a user-defined frame structure configuration to be used by the
/// HIPAR frameStructure/measurement from an xml file.
ViStatus _VI_FUNC rsspecan_VSASaveFrameStructure(ViSession instrSession,
                                                 ViString  frameStructure)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_VSA_FRAME_SAVE, frameStructure),
            2, "Frame Structure");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function defines the frame structure for a previously loaded
/// HIFN  file. For each subframe, all parameters must be defined.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR structure/Defines the frame structure for a previously loaded file. For each
/// HIPAR structure/subframe, all parameters must be defined.
ViStatus _VI_FUNC rsspecan_VSAFrameStructureEdit(ViSession instrSession,
                                                 ViString  structure)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SENS:DDEM:PATT:FRAM:EDIT:STR %s", structure);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function determines whether the frame structure of the signal is
/// HIFN  configured in reference to the result range or user-defined.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR mode/Determines whether the frame structure of the signal is configured in
/// HIPAR mode/reference to the result range or user-defined.
ViStatus _VI_FUNC rsspecan_ConfigureVSAFrameMode(ViSession instrSession,
                                                 ViInt32   mode)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_VSA_FRAME_MODE, mode),
            2, "Mode");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function selects the file that contains a user-defined
/// HIFN  modulation.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR fileName/Selects the file that contains a user-defined modulation.
ViStatus _VI_FUNC rsspecan_ConfigureVSAFrameUserFile(ViSession instrSession,
                                                     ViString  fileName)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_VSA_FRAME_USER_FILE_NAME, fileName),
            2, "File Name");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function configures the Frame PSK Modulation.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR format/Defines the demodulation order for PSK for the pattern.
/// HIPAR nState/Defines the demodulation order for PSK for the pattern.
ViStatus _VI_FUNC rsspecan_ConfigureVSAFramePSK(ViSession instrSession,
                                                ViInt32   format,
                                                ViInt32   nState)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_VSA_FRAME_PSK_FORMAT, format),
            2, "Format");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_VSA_FRAME_PSK_N_STATE, nState),
            3, "N State");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function configures the Frame QAM Modulation.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR format/Defines the specific demodulation order for QAM for the pattern.
/// HIPAR nState/Defines the demodulation order for QAM for the pattern.
ViStatus _VI_FUNC rsspecan_ConfigureVSAFrameQAM(ViSession instrSession,
                                                ViInt32   format,
                                                ViInt32   nState)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_VSA_FRAME_QAM_FORMAT, format),
            2, "Format");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_VSA_FRAME_QAM_N_STATE, nState),
            3, "N State");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function configures the Frame QPSK Modulation.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR format/Defines the demodulation order for QPSK for the pattern.
ViStatus _VI_FUNC rsspecan_ConfigureVSAFrameQPSK(ViSession instrSession,
                                                 ViInt32   format)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_VSA_FRAME_QPSK_FORMAT, format),
            2, "Format");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the demodulation order for APSK for the pattern (see also
/// HIFN  [SENSe:]DDEMod:PATTern:FORMat)
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR nState/Defines the demodulation order for APSK for the pattern (see also
/// HIPAR nState/[SENSe:]DDEMod:PATTern:FORMat).
ViStatus _VI_FUNC rsspecan_ConfigureVSAFrameAPSKNState(ViSession instrSession,
                                                       ViInt32   nState)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_VSA_FRAME_APSK_N_STATE, nState),
            2, "N State");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Defines the demodulation order for ASK for the pattern (see also
/// HIFN  [SENSe:]DDEMod:PATTern:FORMat).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR nState/Defines the demodulation order for ASK for the pattern (see also
/// HIPAR nState/[SENSe:]DDEMod:PATTern:FORMat).
ViStatus _VI_FUNC rsspecan_ConfigureVSAFrameASKNState(ViSession instrSession,
                                                      ViInt32   nState)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_VSA_FRAME_ASK_N_STATE, nState),
            2, "N State");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function determines which boosting is used to demodulate the
/// HIFN  frame next or previous to the last configured subframe.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR type/Defines the type of the operation.
/// HIPAR boosting/Determines which boosting is used to demodulate the frame next or
/// HIPAR boosting/previous to the last configured subframe.
ViStatus _VI_FUNC rsspecan_ConfigureVSAFrameBoosting(ViSession instrSession,
                                                     ViInt32   type,
                                                     ViReal64  boosting)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, type, RSSPECAN_VAL_VSA_FRAME_TYPE_NEXT, RSSPECAN_VAL_VSA_FRAME_TYPE_PREV),
            2, "Type");

    if (type == RSSPECAN_VAL_VSA_FRAME_TYPE_NEXT)
    {
        viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_VSA_FRAME_NEXT_BOOSTING, boosting),
                3, "Boosting");
    }
    else
    {
        viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_VSA_FRAME_PREVIOUS_BOOSTING, boosting),
                3, "Boosting");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function determines which modulation is used to demodulate the
/// HIFN  frame next or previous to the last configured subframe.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR type/Defines the type of the operation.
/// HIPAR modulation/Determines which modulation is used to demodulate the frame next or
/// HIPAR modulation/previous to the last configured subframe.
ViStatus _VI_FUNC rsspecan_ConfigureVSAFrameModulation(ViSession instrSession,
                                                       ViInt32   type,
                                                       ViInt32   modulation)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, type, RSSPECAN_VAL_VSA_FRAME_TYPE_NEXT, RSSPECAN_VAL_VSA_FRAME_TYPE_PREV),
            2, "Type");

    if (type == RSSPECAN_VAL_VSA_FRAME_TYPE_NEXT)
    {
        viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_VSA_FRAME_NEXT_MODULATION, modulation),
                3, "Modulation");
    }
    else
    {
        viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_VSA_FRAME_PREVIOUS_MODULATION, modulation),
                3, "Modulation");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function defines the description for the frame structure in a
/// HIFN  previously loaded file.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR description/Defines the description for the frame structure in a previously loaded
/// HIPAR description/file.
ViStatus _VI_FUNC rsspecan_ConfigureVSAFrameDescription(ViSession instrSession,
                                                        ViString  description)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_VSA_FRAME_DESCRIPTION, description),
            2, "Description");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function queries the start sample of the current result range
/// HIFN  within the capture buffer.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR startSample/Queries the start sample of the current result range within the
/// HIPAR startSample/capture buffer.
ViStatus _VI_FUNC rsspecan_QueryVSAFrameStartSample(ViSession instrSession,
                                                    ViInt32*  startSample)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_GetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_VSA_FRAME_START_SAMPLE, startSample),
            2, "Start Sample");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function configures settings of the measured I/Q data.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR mode/Selects whether or not is automatic adaptation of recording length
/// HIPAR mode/used. To manually set recording length, select if Symbols or Time units
/// HIPAR mode/are applied (automatic adaptation of recording length will be switched
/// HIPAR mode/to off).
/// HIPAR captureLength/Sets the capture length for further processing, e.g. for burst search.
/// HIPAR captureOversampling/Sets the number of points per symbol.
/// HIPAR swapIQ/Swaps the I and Q values of the signal.
ViStatus _VI_FUNC rsspecan_ConfigureVSADataCaptureSettings(ViSession instrSession,
                                                           ViInt32   mode,
                                                           ViReal64  captureLength,
                                                           ViInt32   captureOversampling,
                                                           ViBoolean swapIQ)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, mode, RSSPECAN_VAL_VSA_DEM_RLEN_AUTO_ON, RSSPECAN_VAL_VSA_DEM_RLEN_TIME),
            2, "Mode");

    switch (mode)
    {
    case RSSPECAN_VAL_VSA_DEM_RLEN_AUTO_ON:
        checkErr(rsspecan_SetAttributeViBoolean(instrSession, repCap, RSSPECAN_ATTR_VSA_DEMODULATION_REC_LENGTH_AUTO, VI_TRUE));
        break;

    case RSSPECAN_VAL_VSA_DEM_RLEN_SYMBOL:
        checkErr(rsspecan_SetAttributeViBoolean(instrSession, repCap, RSSPECAN_ATTR_VSA_DEMODULATION_REC_LENGTH_AUTO, VI_FALSE));
        viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_VSA_DEMODULATION_REC_LENGTH_SYM,
                        (ViInt32)captureLength), 3, "Capture Length");
        break;
    case RSSPECAN_VAL_VSA_DEM_RLEN_TIME:
        checkErr(rsspecan_SetAttributeViBoolean(instrSession, repCap, RSSPECAN_ATTR_VSA_DEMODULATION_REC_LENGTH_AUTO, VI_FALSE));
        viCheckParm(rsspecan_SetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_VSA_DEMODULATION_REC_LENGTH_S, captureLength),
                3, "Capture Length");
        break;
    }

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_VSA_MODULATION_PRATE, captureOversampling),
            4, "Capture Oversampling");

    if (swapIQ)
    {
        viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_VSA_DEMODULATION_SIDE_BAND, RSSPECAN_VAL_VSA_SBAND_INV),
                5, "Swap IQ");
    }
    else
    {
        viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_VSA_DEMODULATION_SIDE_BAND, RSSPECAN_VAL_VSA_SBAND_NORM),
                5, "Swap IQ");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This command defines the capture length for further processing, e.g.
/// HIFN  for burst search, in symbols.
/// HIFN     Note that the maximum record length depends on the sample rate for
/// HIFN  signal capture (see [SENSe:]DDEMod:PRATe).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR recordLength_inSymbols/This command defines the capture length for further processing, e.g.
/// HIPAR recordLength_inSymbols/for burst search, in symbols.
ViStatus _VI_FUNC rsspecan_ConfigureVSADemodulationSymbolRecordLength(
                            ViSession instrSession,
                            ViInt32   recordLength_inSymbols)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_VSA_DEMODULATION_RECORD_LENGTH, recordLength_inSymbols),
            2, "Record Length (in Symbols)");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function links the burst search to the type of signal. When a
/// HIFN  signal is marked as bursted, burst search is switched on automatically.
/// HIFN
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR burstSearch/Enables or disables burst searches. If "Auto" is selected, burst
/// HIPAR burstSearch/search is enabled only if "Bursted Signal" is selected.
ViStatus _VI_FUNC rsspecan_ConfigureVSABurstSearchAuto(ViSession instrSession,
                                                       ViInt32   burstSearch)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_VSA_BURST_SEARCH_AUTO, burstSearch),
            2, "Burst Search");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function configures the search for a signal burst.
/// HIFN
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR searchState/Switches the search for a signal burst on or off.
/// HIPAR burstSearchMode/Switches the vector analyzer so that a measurement is performed only
/// HIPAR burstSearchMode/if a burst is found (BURSt). The driver uses this value to set the
/// HIPAR burstSearchMode/RSSPECAN_ATTR_VSA_BURST_SEARCH_MODE attribute.
ViStatus _VI_FUNC rsspecan_ConfigureVSABurstSearch(ViSession instrSession,
                                                   ViBoolean searchState,
                                                   ViInt32   burstSearchMode)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_VSA_BURST_SEARCH_STATE, searchState),
            2, "Search State");

    if (searchState)
    {
        viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_VSA_BURST_SEARCH_MODE, burstSearchMode),
                3, "Burst Search Mode");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configure the search tolerance and the min gap length
/// HIFN  and their default values.
/// HIFN
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR autoConfiguration/Enables or disables burst searches. If switched ON, burst search is
/// HIPAR autoConfiguration/enabled only if "Bursted Signal" is selected.
/// HIPAR tolerance/Sets burst search tolerance.
/// HIPAR minGaplen/Sets the minimum time between two bursts. A minimum time with
/// HIPAR minGaplen/decreased level must occur between two bursts. A new burst is
/// HIPAR minGaplen/recognized only after this time.
ViStatus _VI_FUNC rsspecan_ConfigureVSABurstSearchAutoConfiguration(
                            ViSession instrSession,
                            ViBoolean autoConfiguration,
                            ViInt32   tolerance,
                            ViInt32   minGaplen)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_VSA_SEARCH_BURST_AUTO_CONFIGURATION, autoConfiguration),
            2, "autoConfiguration");
    if (autoConfiguration == VI_FALSE)
    {
        viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_VSA_SEARCH_BURST_TOLERANCE, tolerance),
                3, "Tolerance");

        viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_VSA_BURST_SEARCH_GLEN, minGaplen),
                4, "Min Gaplen");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function queries whether burst is available in the signal.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR found/Returns the burst found result.
ViStatus _VI_FUNC rsspecan_GetVSABurstFound(ViSession  instrSession,
                                            ViBoolean* found)
{
    ViStatus error = VI_SUCCESS;
    ViInt32  statusRegisterValue;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidNullPointer(instrSession, found),
                2, "Found");
    checkErr(
        rsspecan_getStatusRegisterQuery(instrSession, RSSPECAN_VAL_STAT_REG_QUESTIONABLE, RSSPECAN_VAL_STAT_REG_SYNC,
            RSSPECAN_VAL_STAT_REG_QUERY_COND, &statusRegisterValue));
    *found = (ViBoolean)(statusRegisterValue & 1);

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function links the pattern search to the type of signal. When a
/// HIFN  signal is marked as patterned, pattern search is switched on
/// HIFN  automatically.
/// HIFN
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR patternSearch/Enables or disables pattern searches. If switched ON, pattern search
/// HIPAR patternSearch/is enabled automatically if "Pattern" is selected.
ViStatus _VI_FUNC rsspecan_ConfigureVSAPatternSearchAuto(ViSession instrSession,
                                                         ViInt32   patternSearch)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_VSA_SYNC_SEARCH_AUTO, patternSearch),
            2, "Pattern Search");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function configures the search for a signal burst.
/// HIFN
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR searchState/Switches the search for a signal burst on or off.
/// HIPAR patternSearchMode/Switches the vector analyzer so that a measurement is performed only
/// HIPAR patternSearchMode/if a burst is found (BURSt).
/// HIPAR pattern/Sets a predefined sync pattern file. Up to 16 sync pattern files may
/// HIPAR pattern/be given.
ViStatus _VI_FUNC rsspecan_ConfigureVSAPatternSearch(ViSession instrSession,
                                                     ViBoolean searchState,
                                                     ViInt32   patternSearchMode,
                                                     ViString  pattern)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_VSA_SYNC_SEARCH_STATE, searchState),
            2, "Search State");

    if (searchState)
    {
        viCheckParm(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_VSA_SYNC_SEARCH_SELECTED_PATTERN, pattern),
                4, "Pattern");

        viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_VSA_SYNC_SEARCH_MODE, patternSearchMode),
                3, "Pattern Search Mode");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the pattern search and the IQ Correlation
/// HIFN  Threshold setting.
/// HIFN
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR autoConfiguration/Sets the IQ correlation threshold to its default value.
/// HIPAR IQCorrelationThreshold/Sets the IQ correlation threshold for pattern matching in percent. A
/// HIPAR IQCorrelationThreshold/high level means stricter matching.
ViStatus _VI_FUNC rsspecan_ConfigureVSAPatternSearchAutoConfiguration(
                            ViSession instrSession,
                            ViBoolean autoConfiguration,
                            ViReal64  IQCorrelationThreshold)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_VSA_SYNC_SEARCH_PATTERN_AUTO_CONFIGURATION, autoConfiguration),
            2, "Auto Configuration");

    if (autoConfiguration == VI_FALSE)
    {
        viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_VSA_SYNC_SEARCH_IQ_CORRELATION_THRESHOLD, IQCorrelationThreshold),
                3, "IQ Correlation Threshold");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function queries whether one of the selected sync patterns
/// HIFN  (DDEM:SEAR:SYNC:SEL) is available in the signal. Up to 16 sync pattern
/// HIFN  files can be selected.
/// HIFN
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR bufferSize/Pass the number of bytes in the string you specify for the found
/// HIPAR bufferSize/parameter.
/// HIPAR found/Returns the sync pattern found result.
ViStatus _VI_FUNC rsspecan_GetVSAPatternFound(ViSession instrSession,
                                              ViInt32   reserved,
                                              ViInt32   bufferSize,
                                              ViChar    found[])
{
    ViStatus error = VI_SUCCESS;
             reserved;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K70"));

    checkErr(
        RsCore_QueryViStringUnknownLengthToUserBuffer(instrSession, ":SENS:DDEM:SEAR:SYNC:FOUN?", bufferSize, found,
            NULL));

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function reads the names of all patterns stored on the hard disk.
/// HIFN
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR selection/Sets the pattern selection.
/// HIPAR numberofPatterns/This control returns number of the names of all digital standards
/// HIPAR numberofPatterns/available.
/// HIPAR bufferSize/Pass the number of bytes in the string you specify for the catalog
/// HIPAR bufferSize/list parameter.
/// HIPAR patternsList/This control returns array of the names of all digital standards
/// HIPAR patternsList/available in CSV form.
ViStatus _VI_FUNC rsspecan_GetVSAPatternCatalog(ViSession instrSession,
                                                ViInt32   selection,
                                                ViInt32*  numberofPatterns,
                                                ViInt32   bufferSize,
                                                ViChar    patternsList[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K70"));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, selection, RSSPECAN_VAL_VSA_PATTERN_ALL, RSSPECAN_VAL_VSA_PATTERN_CURRENT),
            2, "Selection");
    viCheckParm(RsCore_InvalidViInt32RangeMin(instrSession, bufferSize, 0),
                4, "Buffer Size");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, patternsList),
                5, "Patterns List");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SENS:DDEM:SEAR:SYNC:CAT? %s", VSACatPatternArr[selection]);
    checkErr(RsCore_QueryCatalog(instrSession, cmd, bufferSize, patternsList, numberofPatterns));

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function performs the selected operation with channel table.
/// HIFN
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR operation/This control selects the type of memory operation to be performed.
/// HIPAR patternName/This control selects a predefined channel table file.
ViStatus _VI_FUNC rsspecan_ConfigureVSAPatternOperations(ViSession instrSession,
                                                         ViInt32   operation,
                                                         ViString  patternName)
{
    ViStatus error = VI_SUCCESS;
    ViChar   copyName[RS_MAX_MESSAGE_LEN] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, operation, RSSPECAN_VAL_SYNC_PATT_ADD, RSSPECAN_VAL_SYNC_PATT_COPY),
            2, "Operation");

    switch (operation)
    {
    case RSSPECAN_VAL_SYNC_PATT_ADD:
        viCheckParm(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_VSA_SYNC_PATT_ADD, patternName),
                3, "Pattern Name");
        break;

    case RSSPECAN_VAL_SYNC_PATT_SELECT:
        viCheckParm(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_VSA_SYNC_NAME, patternName),
                3, "Pattern Name");
        break;

    case RSSPECAN_VAL_SYNC_PATT_DELETE:
        viCheckParm(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_VSA_SYNC_NAME, patternName),
                3, "Pattern Name");
        checkErr(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_VSA_SYNC_DELETE, ""));
        break;

    case RSSPECAN_VAL_SYNC_PATT_REMOVE:
        viCheckParm(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_VSA_SYNC_PATT_REMOVE, patternName),
                3, "Pattern Name");
        break;

    case RSSPECAN_VAL_SYNC_PATT_REMOVE_ALL:
        checkErr(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_VSA_SYNC_PATT_REMOVE_ALL, ""));
        break;

    case RSSPECAN_VAL_SYNC_PATT_COPY:
        viCheckParm(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_VSA_SYNC_NAME, patternName),
                3, "Pattern Name");

        (void)sprintf(copyName, "%s_copy", patternName);

        viCheckParm(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_VSA_SYNC_COPY, copyName),
                3, "Pattern Name");
        break;
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function configures the selected pattern.
/// HIFN
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR patternName/This control selects a predefined channel table file.
/// HIPAR description/Sets a text to explain the pattern. The text is displayed only in the
/// HIPAR description/selection menu (manual control). This text is supposed to be short and
/// HIPAR description/concise. Detailed information about the pattern is given in the
/// HIPAR description/comment.
/// HIPAR comment/Defines a comment to a sync pattern.
/// HIPAR degreeofModulation/Sets the degree of modulation (number of permitted states).
/// HIPAR patternData/Sets a sync sequence for the sync pattern. Four values represent a
/// HIPAR patternData/symbol. The value range of a symbol depends on the degree of
/// HIPAR patternData/modulation. FFFF entries are interpreted as "Don't Care Bits".
ViStatus _VI_FUNC rsspecan_ConfigureVSAPatternSettings(ViSession instrSession,
                                                       ViString  patternName,
                                                       ViString  description,
                                                       ViString  comment,
                                                       ViInt32   degreeofModulation,
                                                       ViString  patternData)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_VSA_SYNC_NAME, patternName),
            2, "Pattern Name");

    viCheckParm(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_VSA_PATTERN_DATA, patternData),
            6, "Pattern Data");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_VSA_SYNC_NSTAT, degreeofModulation),
            5, "Degree Of Modulation");

    viCheckParm(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_VSA_SYNC_TEXT, description),
            3, "Description");

    viCheckParm(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_VSA_SYNC_COMMENT, comment),
            4, "Comment");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the tesult length od modulation
/// HIFN
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR resultLength/Sets the number of displayed symbols (result length).
ViStatus _VI_FUNC rsspecan_ConfigureVSAResultLength(ViSession instrSession,
                                                    ViInt32   resultLength)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_VSA_DEMODULATION_RESULT_LENGTH, resultLength),
            2, "Result Length");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function sets the reference for the result range alignment.
/// HIFN
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR reference/Selects the reference point.
ViStatus _VI_FUNC rsspecan_ConfigureVSAResultRangeReference(ViSession instrSession,
                                                            ViInt32   reference)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_VSA_FIT_REF_POINT, reference),
            2, "Reference");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function sets the measurements results alignment on the display.
/// HIFN
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR alignment/Defines where the relevant event (reference point) is to appear in the
/// HIPAR alignment/diagram.
/// HIPAR alignOffset/Defines shifts the display range (relative to the reference time) by
/// HIPAR alignOffset/the number of given symbols. The resolution is 1 symbol. A value >0
/// HIPAR alignOffset/results in a shift towards the right, and a value <0 results in a shift
/// HIPAR alignOffset/towards the left.
ViStatus _VI_FUNC rsspecan_ConfigureVSADisplayAlignment(ViSession instrSession,
                                                        ViInt32   alignment,
                                                        ViInt32   alignOffset)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_VSA_FIT_ALIGN, alignment),
            2, "Alignment");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_VSA_FIT_ALIGN_OFFSET, alignOffset),
            3, "Alignment Offset");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function sets the measurements results alignment on the display.
/// HIFN
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR value/Sets the number of the symbol which marks the beginning of the
/// HIPAR value/alignment reference source. In effect, this setting defines an offset
/// HIPAR value/of the x-axis.
ViStatus _VI_FUNC rsspecan_ConfigureVSASymbolNoAtReference(ViSession instrSession,
                                                           ViInt32   value)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_VSA_FIT_SYMBOL_NUMBER, value),
            2, "Value");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function configures range of the result is to be evaluated,
/// HIFN     either the entire result range or only a specified part of it.
/// HIFN
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR entireResultRange/True switches the evaluation lines OFF. Start and Stop are ignored.
/// HIPAR entireResultRange/False switches the lines on and you can define them with Start and Stop
/// HIPAR start/Sets the start value for the evaluation range.
/// HIPAR stop/Sets the stop value for the evaluation range.
ViStatus _VI_FUNC rsspecan_ConfigureVSAEvaluationRange(ViSession instrSession,
                                                       ViBoolean entireResultRange,
                                                       ViReal64  start,
                                                       ViReal64  stop)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_VSA_EVAL_LINE_STATE, !entireResultRange),
                2, "Entire Result Range");

    if (!entireResultRange)
    {
        viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_VSA_EVAL_LINE_START, start),
                3, "Start");

        viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_VSA_EVAL_LINE_STOP, stop),
                4, "Stop");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function configures the statistics count. Entering 0 as a
/// HIFN  parameter activates "Auto" mode. Entering a number greater than
/// HIFN  0 activates "Manual" mode and sets the statistics count to the
/// HIFN  corresponding number.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR statisticCount/Sets the statistics count.
ViStatus _VI_FUNC rsspecan_ConfigureVSAStatisticCount(ViSession instrSession,
                                                      ViInt32   statisticCount)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_VSA_STATISTIC_COUNT, statisticCount),
            2, "Statistic Count");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the result range you want to analyze.
/// HIFN  By default, the R&S FSV shows the results over all result ranges
/// HIFN  that have been captured in the data capturing process and are in
/// HIFN  the R&S FSV's memory. By selecting a range number, you can
/// HIFN  analyze a specific result range, e.g. a particular burst. The
/// HIFN  range depends on the number of result ranges you have captured
/// HIFN  previously. A selection of the result range is possible in
/// HIFN  single sweep mode only.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR resultRangeNumber/Sets the result range to be displayed after a single sweep.
ViStatus _VI_FUNC rsspecan_ConfigureVSAResultRangeNumber(ViSession instrSession,
                                                         ViInt32   resultRangeNumber)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_VSA_RESULT_RANGE_NUMBER, resultRangeNumber),
            2, "Result Range Number");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function queries the current statistics counter value which
/// HIFN  indicates how many result ranges have been evaluated. For
/// HIFN  results that use the capture buffer as a source, the number of
/// HIFN  used capture buffers can be queried.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR evaluationType/Selects the evaluation type to be returned.
/// HIPAR currentCounter/Returns the current statistics counter value which indicates how
/// HIPAR currentCounter/many result ranges have been evaluated. For results that use the
/// HIPAR currentCounter/capture buffer as a source, the number of used capture buffers
/// HIPAR currentCounter/can be queried.
ViStatus _VI_FUNC rsspecan_GetVSACurrentStatisticsCounter(ViSession instrSession,
                                                          ViInt32   evaluationType,
                                                          ViInt32*  currentCounter)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    switch (evaluationType)
    {
    case RSSPECAN_VAL_VSA_EVAL_STATISTICS:
        viCheckParm(rsspecan_GetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_VSA_CURRENT_STATISTICS_COUNTER_EVALUATED, currentCounter),
                3, "Current Counter");
        break;

    case RSSPECAN_VAL_VSA_EVAL_CAPTURE:
        viCheckParm(rsspecan_GetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_VSA_CURRENT_STATISTICS_COUNTER_CAPTURE, currentCounter),
                3, "Current Counter");
        break;

    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, evaluationType), 2, "Evaluation Type");
        break;
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the compensation during demodulation.
/// HIFN
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR compensateFor/This control selects which distortion will be compensated during
/// HIPAR compensateFor/demodulation.
/// HIPAR compensationState/Sets the state of the compensation.
ViStatus _VI_FUNC rsspecan_ConfigureVSACompensation(ViSession instrSession,
                                                    ViInt32   compensateFor,
                                                    ViBoolean compensationState)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    switch (compensateFor)
    {
    case RSSPECAN_VAL_VSA_COMPENSATE_IQ_OFFSET:
        viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_VSA_COMPENSATE_IQ_OFFSET, compensationState),
                3, "Compensation State");
        break;

    case RSSPECAN_VAL_VSA_COMPENSATE_IQ_IMBAL:
        viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_VSA_COMPENSATE_IQ_IMBALANCE, compensationState),
                3, "Compensation State");
        break;

    case RSSPECAN_VAL_VSA_COMPENSATE_ADROOP:
        viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_VSA_COMPENSATE_AMPLITUDE_DROOP, compensationState),
                3, "Compensation State");
        break;

    case RSSPECAN_VAL_VSA_COMPENSATE_FSK_DEV_ERR:
        viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_VSA_COMPENSATE_FSK_DEVIATION, compensationState),
                3, "Compensation State");
        break;

    case RSSPECAN_VAL_VSA_COMPENSATE_FSK_CFD:
        viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_VSA_COMPENSATE_CARRIER_FREQUENCY_DRIFT, compensationState),
                3, "Compensation State");
        break;

    case RSSPECAN_VAL_VSA_COMPENSATE_CHANNEL:
        viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_VSA_COMPENSATE_CHANNEL, compensationState),
                3, "Compensation State");
        break;

    case RSSPECAN_VAL_VSA_COMPENSATE_SR_ERROR:
        viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_VSA_COMPENSATE_SYMBOL_RATE_ERROR, compensationState),
                3, "Compensation State");
        break;
    case RSSPECAN_VAL_VSA_COMPENSATE_IQ_SKEW:
        viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_VSA_COMPENSATE_IQ_SKEW, compensationState),
                3, "Compensation State");
        break;

    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, compensateFor), 2, "Compensate For");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/*****************************************************************************
 * Function: Configure VSA Equalizer Mode
 * Purpose:  This function configures the mode of an adaptive
 *           equalizer filter.
*****************************************************************************/
ViStatus _VI_FUNC rsspecan_ConfigureVSAEqualizer(ViSession instrSession,
                                                 ViBoolean state,
                                                 ViInt32   mode,
                                                 ViInt32   length)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_VSA_EQUALIZER_STATE, state),
            2, "State");
    if (state)
    {
        viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_VSA_EQUALIZER_MODE, mode),
                3, "Mode");
        if ((mode != RSSPECAN_VAL_VSA_EQUALIZER_MODE_FREEZE) && (mode != RSSPECAN_VAL_VSA_EQUALIZER_MODE_USER))
        {
            viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_VSA_EQUALIZER_LENGTH, length),
                    4, "Length");
        }
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Determines the file format for stored equalizer results.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR fileFormat/Determines the file format for stored equalizer results.
ViStatus _VI_FUNC rsspecan_ConfigureVSAEqualizerFileFormat(ViSession instrSession,
                                                           ViInt32   fileFormat)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_VSA_EQUALIZER_FILE_FORMAT, fileFormat),
            2, "File Format");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/*****************************************************************************
 * Function: VSA Equalizer Operations
 * Purpose:  This function performs the selected operation with equalizer.
*****************************************************************************/
ViStatus _VI_FUNC rsspecan_ConfigureVSAEqualizerOperations(ViSession instrSession,
                                                           ViInt32   operation,
                                                           ViString  equalizerName)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    switch (operation)
    {
    case RSSPECAN_VAL_VSA_EQUALIZER_LOAD:
        viCheckParm(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_VSA_EQUALIZER_LOAD, equalizerName),
                3, "Equalizer Name");
        break;
    case RSSPECAN_VAL_VSA_EQUALIZER_SAVE:
        viCheckParm(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_VSA_EQUALIZER_SAVE, equalizerName),
                3, "Equalizer Name");
        break;
    case RSSPECAN_VAL_VSA_EQUALIZER_RESET:
        checkErr(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_VSA_EQUALIZER_RESET, ""));
        break;

    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, operation), 2, "Operation");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function configures the EVM modulation parameters
/// HIFN
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR calculation/Selects the calculation formula for EVM modulation.
ViStatus _VI_FUNC rsspecan_ConfigureVSAEVMModulation(ViSession instrSession,
                                                     ViInt32   calculation)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_VSA_MODULATION_EVM_CALCULATION, calculation),
            2, "Calculation");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function determines the optimization criteria for the
/// HIFN  demodulation. RMS Min Optimization goal is the minimization of RMS of
/// HIFN  the error vector. EVM Min Optimization goal is the minimization of the
/// HIFN  EVM.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR optimization/This control determines the optimization criteria for the
/// HIPAR optimization/demodulation. RMS Min Optimization goal is the minimization of RMS of
/// HIPAR optimization/the error vector. EVM Min Optimization goal is the minimization of the
/// HIPAR optimization/EVM.
ViStatus _VI_FUNC rsspecan_ConfigureVSAOptimization(ViSession instrSession,
                                                    ViInt32   optimization)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_VSA_MODULATION_OPTIMIZATION, optimization),
            3, "Optimization");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function configures the calculation of the error vector
/// HIFN  magnitude trace for Offset-QPSK only. It has no effect for all
/// HIFN  other modulations. It has no effect on results based on the MEAS
/// HIFN  or the REF signal.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR EVMOffset/Selects the calculation of the error vector magnitude trace for
/// HIPAR EVMOffset/Offset-QPSK only.
ViStatus _VI_FUNC rsspecan_ConfigureVSAEVMOffsetState(ViSession instrSession,
                                                      ViBoolean EVMOffset)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_VSA_MODULATION_EVM_OFFSET, EVMOffset),
            2, "EVM Offset");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function configures the pattern for synchronization, in order to
/// HIFN  obtain the correct reference signal. Depending on the signal, making
/// HIFN  use of the pattern for synchronization speeds up your measurement
/// HIFN  considerably and makes it more robust against high carrier frequency
/// HIFN  offsets.
/// HIFN
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR patternforSync/Sets the state of the pattern synchronization.
ViStatus _VI_FUNC rsspecan_ConfigureVSAPatternForSync(ViSession instrSession,
                                                      ViInt32   patternforSync)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    switch (patternforSync)
    {
    case RSSPECAN_VAL_VSA_PATT_SYNC_OFF:
        viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_VSA_USE_PATTERN_FOR_SYNC_MODE, RSSPECAN_VAL_MAN),
                2, "Pattern for Sync");

        viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_VSA_USE_PATTERN_FOR_SYNC_STATE, VI_FALSE),
                2, "Pattern for Sync");
        break;

    case RSSPECAN_VAL_VSA_PATT_SYNC_ON:
        viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_VSA_USE_PATTERN_FOR_SYNC_MODE, RSSPECAN_VAL_MAN),
                2, "Pattern for Sync");

        viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_VSA_USE_PATTERN_FOR_SYNC_STATE, VI_TRUE),
                2, "Pattern for Sync");
        break;

    case RSSPECAN_VAL_VSA_PATT_SYNC_AUTO:
        viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_VSA_USE_PATTERN_FOR_SYNC_MODE, RSSPECAN_VAL_AUTO),
                2, "Pattern for Sync");
        break;

    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, patternforSync), 2, "Pattern for Sync");
        break;
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function selects the Known Data source.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR source/Selects the Known Data source.
ViStatus _VI_FUNC rsspecan_ConfigureVSAKnownDataSource(ViSession instrSession,
                                                       ViInt32   source)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_VSA_KNOWN_DATA_PRBS_SOURCE, source),
            2, "Source");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function defines the type of the used PRBS model. The type of the
/// HIFN  model defines the degree, complexity and number of terms in the
/// HIFN  polynomial model. If the PRBS data is generated in accordance with the
/// HIFN  ITU-T standard, no further settings are required.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR type/Defines the type of the used PRBS model. The type of the model defines
/// HIPAR type/the degree, complexity and number of terms in the polynomial model. If
/// HIPAR type/the PRBS data is generated in accordance with the ITU-T standard, no
/// HIPAR type/further settings are required.
ViStatus _VI_FUNC rsspecan_ConfigureVSAKnownDataPRBSType(ViSession instrSession,
                                                         ViInt32   type)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_VSA_KNOWN_DATA_PRBS_TYPE, type),
            2, "Type");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function configures the fine synchronization.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR autoSet/Sets wheater the fine synchronization is defined automatically.
/// HIPAR mode/Defines the fine synchronization mode used to calculate results, e.g. the bit error rate.
/// HIPAR SER/Defines a maximum symbol error rate for the known data in reference to the analyzed data.
ViStatus _VI_FUNC rsspecan_ConfigureVSAFineSynchronization(ViSession instrSession,
                                                           ViBoolean autoSet,
                                                           ViInt32   mode,
                                                           ViReal64  SER)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_VSA_FINE_SYNC_AUTO, autoSet),
            2, "Auto Set");

    if (!autoSet)
    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_VSA_FINE_SYNC_MODE, mode),
            3, "Mode");

    if (!autoSet && mode == RSSPECAN_VAL_VSA_FINE_SYNC_KDAT)
    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_VSA_FINE_SYNC_SER, SER),
            4, "SER");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function queries whether a loaded Known Data file was used for fine synchronization or not.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR result/Queries whether a loaded Known Data file was used for fine synchronization or not.
ViStatus _VI_FUNC rsspecan_QueryVSAFineSynchronizationResult(ViSession instrSession,
                                                             ViInt32*  result)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_GetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_VSA_FINE_SYNC_RESULT, result),
            2, "Result");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function configures the Known Data File.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR knownDataState/Activates or deactivates the use of the loaded data file.
/// HIPAR dataFileName/Selects the Known Data file.
ViStatus _VI_FUNC rsspecan_ConfigureVSAKnownData(ViSession instrSession,
                                                 ViBoolean knownDataState,
                                                 ViString  dataFileName)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_VSA_KNOWN_DATA_STATE, knownDataState),
            2, "Known Data State");

    if (knownDataState)
    viCheckParm(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_VSA_KNOWN_DATA_FILE_NAME, dataFileName),
            3, "Data File Name");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the Known Data Negate Feedback.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR autoFeedback/Determines how the feedback value is calculated.
/// HIPAR enabled/Determines how the feedback bit is calculated.
ViStatus _VI_FUNC rsspecan_ConfigureVSAKnownDataNegateFeedback(
                            ViSession instrSession,
                            ViBoolean autoFeedback,
                            ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_VSA_KNOWN_DATA_NEGATE_FEEDBACK_PATH_AUTO, autoFeedback),
            2, "Auto");

    if (autoFeedback == VI_FALSE)
    {
        viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_VSA_KNOWN_DATA_NEGATE_FEEDBACK_PATH_ENABLED, enabled),
                3, "Enabled");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function determines the coefficients of the polynomial and thus
/// HIFN  the feedback positions for the PRBS algorithm.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR autoPoly/Determines the coefficients of the polynomial and thus the feedback
/// HIPAR autoPoly/positions for the PRBS algorithm.
/// HIPAR arraySize/Sets the order Array Size.
/// HIPAR order/Determines the coefficients of the polynomial and thus the feedback
/// HIPAR order/positions for the PRBS algorithm.
ViStatus _VI_FUNC rsspecan_ConfigureVSAKnownDataPolynomial(ViSession instrSession,
                                                           ViBoolean autoPoly,
                                                           ViInt32   arraySize,
                                                           ViInt32   order[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViChar   orderArr[RS_MAX_MESSAGE_BUF_SIZE] = "";
    ViInt32  i = 0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_VSA_KNOWN_DATA_POLYNOMIAL_AUTO, autoPoly),
            2, "Auto");

    if (autoPoly == VI_FALSE)
    {
        for (i = 0; i < arraySize; i++)
        {
            snprintf(orderArr, RS_MAX_MESSAGE_BUF_SIZE, "%ld;", order[i]);
        }

        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SENS:DDEM:KDAT:PRBS:POLY '%s'", orderArr);
        checkErr(RsCore_Write(instrSession, cmd));
    }

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function configures the Known Data PRBS Pattern.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR autoPattern/If enabled, the R&S FSW VSA application assumes the pattern is part of
/// HIPAR autoPattern/the PRBS sequence.
/// HIPAR enabled/Determines whether the pattern symbols are part of the PRBS sequence
/// HIPAR enabled/and are thus treated as data symbols.
ViStatus _VI_FUNC rsspecan_ConfigureVSAKnownDataPRBSPattern(ViSession instrSession,
                                                            ViBoolean autoPattern,
                                                            ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_VSA_KNOWN_DATA_PRBS_PATTERN_AUTO, autoPattern),
            2, "Auto");

    if (autoPattern == VI_FALSE)
    {
        viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_VSA_KNOWN_DATA_PRBS_PATTERN_ENABLED, enabled),
                3, "Enabled");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function configures the estimation points per symbol. You
/// HIFN  can set the estimation points manually or let the analyzer decide
/// HIFN  which estimation points to use.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR selectsEstimationPointsSym/Selects Estimation Points/Sym.
ViStatus _VI_FUNC rsspecan_ConfigureVSAEstimationPointsSymbol(
                            ViSession instrSession,
                            ViInt32   selectsEstimationPointsSym)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    switch (selectsEstimationPointsSym)
    {
    case RSSPECAN_VAL_VSA_AUTOMATIC_ESTIMATION:
        viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_VSA_AUTOMATIC_ESTIMATION_STATE, VI_TRUE),
                2, "Estimation Points/Sym State");
        break;

    case RSSPECAN_VAL_VSA_ESTIMATION_SYMBOL_1:
        viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_VSA_AUTOMATIC_ESTIMATION_STATE, VI_FALSE),
                2, "Estimation Points/Sym State");

        viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_VSA_ESTIMATION_VALUE, 1),
                2, "Estimation Points/Sym");
        break;

    case RSSPECAN_VAL_VSA_ESTIMATION_SYMBOL_2:
        viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_VSA_AUTOMATIC_ESTIMATION_STATE, VI_FALSE),
                2, "Estimation Points/Sym State");

        viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_VSA_ESTIMATION_VALUE, 2),
                2, " Estimation Points/Sym Value");
        break;

    case RSSPECAN_VAL_VSA_ESTIMATION_CAPTURE_OV:
        viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_VSA_AUTOMATIC_ESTIMATION_STATE, VI_FALSE),
                2, "Estimation Points/Sym State");

        viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_VSA_ESTIMATION_VALUE, 4),
                2, " Estimation Points/Sym Value");
        break;

    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, selectsEstimationPointsSym), 2, "Estimation Points/Sym");
        break;
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function configures the measurement filter.
/// HIFN
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR autoMeasurementFilter/Sets wheater the measurement filter is defined automatically depending
/// HIPAR autoMeasurementFilter/on the TX filter (when set to On).
/// HIPAR type/Defines the measurement filter type, if the Auto setting is not
/// HIPAR type/enabled.
/// HIPAR alphaBT/Defines the roll-off factor (Alpha) or the filter bandwidth (BT).
ViStatus _VI_FUNC rsspecan_ConfigureVSAMeasurementFilter(ViSession instrSession,
                                                         ViBoolean autoMeasurementFilter,
                                                         ViInt32   type,
                                                         ViReal64  alphaBT)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_VSA_AUTO_MEASUREMENT_FILTER, autoMeasurementFilter),
            2, "Auto Measurement Filter");

    if (autoMeasurementFilter == VI_FALSE)
    {
        if (type == RSSPECAN_VAL_VSA_MEAS_FILTER_TYPE_NONE)
        {
            viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_VSA_MEASUREMENT_FILTER_STATE, VI_FALSE),
                    5, "");
        }
        else
        {
            viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_VSA_MEASUREMENT_FILTER_TYPE, type),
                    3, "Type");

            if (type <= RSSPECAN_VAL_VSA_MEAS_FILTER_TYPE_RRC)
            {
                viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_VSA_MEASUREMENT_FILTER_ALPHA, alphaBT),
                        4, "Alpha BT");
            }
        }
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function selects the user-defined measurement filter.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR filterFileName/The name of the user-defined filter
ViStatus _VI_FUNC rsspecan_ConfigureVSAUserDefinedMeasurementFilter(
                            ViSession instrSession,
                            ViString  filterFileName)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    if (filterFileName == NULL)
    {
        checkErr(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_VSA_MEASUREMENT_FILTER_STATE, VI_FALSE));
    }
    else
    {
        checkErr(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_VSA_MEASUREMENT_FILTER_STATE, VI_TRUE));
        checkErr(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_VSA_MEASUREMENT_FILTER_TYPE, RSSPECAN_VAL_VSA_MEAS_FILTER_TYPE_USER));

        viCheckParm(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_VSA_MEASUREMENT_FILTER_USER_FILE, filterFileName),
                2, "Filter File Name");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function configures the filter characteristic Alpha/BT.
/// HIFN
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR alphaBt/Sets the filter characteristic (ALPHA/BT). The resolution is 0.05.
ViStatus _VI_FUNC rsspecan_ConfigureVSAFilterCharacteristic(ViSession instrSession,
                                                            ViReal64  alphaBt)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_VSA_MODULATION_ALPHABT, alphaBt),
            2, "Alpha/BT");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function enables or disables selected marker.
/// HIFN
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR markerNumber/Selects the marker to configure.
/// HIPAR markerState/Enables or disables the selected marker.
ViStatus _VI_FUNC rsspecan_ConfigureVSAMarkerState(ViSession instrSession,
                                                   ViInt32   markerNumber,
                                                   ViBoolean markerState)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, markerNumber, 1, 4),
            2, "Marker Number");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "M%ld", markerNumber);

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, repCap, RSSPECAN_ATTR_MARKER_ENABLED, markerState),
            3, "Marker State");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function couples markers between several screens, i.e. uses the
/// HIFN  same stimulus. All screens can be linked with an X-axis scaled in
/// HIFN  symbols or time, except those showing the capture buffer.
/// HIFN
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR state/Specifies the state of the screen coupling.
ViStatus _VI_FUNC rsspecan_ConfigureVSAScreenCoupling(ViSession instrSession,
                                                      ViInt32   reserved,
                                                      ViInt32   reserved2,
                                                      ViBoolean state)
{
    ViStatus error = VI_SUCCESS;
             reserved;
             reserved2;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_VSA_SCREEN_COUPLING, state),
            2, "State");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures whether the marker search works on the real
/// HIFN  or the imag trace.
/// HIFN
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR search/Specifies whether the marker search works on the real or the imag
/// HIPAR search/trace.
ViStatus _VI_FUNC rsspecan_ConfigureVSAMarkerSearch(ViSession instrSession,
                                                    ViInt32   search)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_VSA_MARKER_SEARCH, search),
            2, "Search");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function searches for the max absolute peak of a trace
/// HIFN  (independent of its sign). This is especially usefull for measurements
/// HIFN  where a zero value is best, and either large positive or large negative
/// HIFN  values indicate a bad signal.
/// HIFN
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsspecan_ConfigureVSAMarkerPeakSearch(ViSession instrSession,
                                                        ViInt32   window)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_VSA_MARKER_PEAK_SEARCH,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap));
    checkErr(rsspecan_SetAttributeViString(instrSession, repCap, RSSPECAN_ATTR_VSA_MARKER_PEAK_SEARCH, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function positions the active marker or delta marker on the
/// HIFN  largest absolute peak value (maximum or minimum) of the selected trace.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR marker/Selects marker.
ViStatus _VI_FUNC rsspecan_ConfigureVSADeltamarkerPeakSearch(ViSession instrSession,
                                                             ViInt32   window,
                                                             ViInt32   marker)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_marker[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, window, 1, 2),
            2, "Window");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, marker, 1, 16),
            3, "Marker");

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_VSA_DELTAMARKER_PEAK_SEARCH,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap));
    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_VSA_DELTAMARKER_PEAK_SEARCH,
            "Marker", marker - 1, RS_REPCAP_BUF_SIZE, repCap_marker));

    RsCore_StrcatMaxLen(repCap, RS_REPCAP_BUF_SIZE, ",", repCap_marker, NULL);

    checkErr(rsspecan_SetAttributeViString(instrSession, repCap, RSSPECAN_ATTR_VSA_DELTAMARKER_PEAK_SEARCH, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function configures whether the trace displays the evaluation of
/// HIFN  the measured signal or the reference signal (if "Meas & Ref Signal" is
/// HIFN  used as the signal source).
/// HIFN
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR traceEvaluation/This control defines whether the trace displays the evaluation of the
/// HIPAR traceEvaluation/measured signal or the reference signal.
ViStatus _VI_FUNC rsspecan_ConfigureVSATraceEval(ViSession instrSession,
                                                 ViInt32   trace,
                                                 ViInt32   traceEvaluation)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE];
    ViInt32  win = 0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, trace, 1, 6),
            3, "Trace");

    checkErr(rsspecan_GetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_ACTIVE_WINDOW, &win));

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "Win,%ld,TR%ld", win, trace);

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_VSA_TRACE_EVALUATION, traceEvaluation),
            2, "Trace Evaluation");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function control the zero point and scaling of the y
/// HIFN     axis for I/Q, statistical and time displays:.
/// HIFN
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR scaling/Defines the scaling of the Y axis.
/// HIPAR referenceValue/Defines the reference value for the Y axis of the measurement diagram.
ViStatus _VI_FUNC rsspecan_ConfigureVSADisplayYAxis(ViSession instrSession,
                                                    ViInt32   window,
                                                    ViReal64  scaling,
                                                    ViReal64  referenceValue)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_VSA_SCALE_Y_PDIV,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap));

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_VSA_SCALE_Y_PDIV, scaling),
            2, "Scaling");

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_DISP_REF_VALUE, referenceValue),
            3, "Reference Value");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the y-axis ranges for statistical
/// HIFN  measurements. Use function
/// HIFN  rsspecan_ConfigureVSAResultTypeTransformation to enable
/// HIFN  statistical measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR yUnit/Toggles the scaling of Y axis between percentage and absolute.
/// HIPAR yAxisMaxValue/Defines the upper vertical limit of the diagram.
/// HIPAR yAxisMinValue/Defines the lower vertical limit of the diagram.
/// HIPAR window/This control selects the measurement window.
ViStatus _VI_FUNC rsspecan_ConfigureVSADisplayYAxisStatisticMeasurement(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   yUnit,
                            ViReal64  yAxisMaxValue,
                            ViReal64  yAxisMinValue)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_MEAS_STAT_Y_UNIT, yUnit),
            3, "Y Unit");
    snprintf(repCap, RS_REPCAP_BUF_SIZE, "Win%ld", window);
    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_VSA_DISPLAY_YAXIS_MAX, yAxisMaxValue),
            4, "Y Axis Max Value");

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_VSA_DISPLAY_YAXIS_MIN, yAxisMinValue),
            5, "Y Axis Min Value");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function controls the reference position of Y axis.
/// HIFN
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR referencePosition/Defines the position of the reference value for the X axis.
ViStatus _VI_FUNC rsspecan_ConfigureVSADisplayYAxisRefPosition(
                            ViSession instrSession,
                            ViReal64  referencePosition)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_DISP_REF_POSITION, referencePosition),
            2, "Reference Position");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function defines the scaling of the x axis for statistical
/// HIFN  displays.
/// HIFN
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR quantize/Defines defines the number of bars for the statistical functions.
/// HIPAR scaling/Defines the scaling of the X axis.
ViStatus _VI_FUNC rsspecan_ConfigureVSADisplayXAxis(ViSession instrSession,
                                                    ViInt32   window,
                                                    ViInt32   reserved,
                                                    ViInt32   quantize,
                                                    ViInt32   scaling)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE];
             reserved;

    checkErr(RsCore_LockSession(instrSession));

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "Win%ld", window);

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_VSA_SCALE_X_BAR_COUNT, quantize),
            4, "Quantize");
    snprintf(repCap, RS_REPCAP_BUF_SIZE, "Win%ld", window);
    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_VSA_SCALE_X_PDIV, scaling),
            5, "Scaling");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function defines the scaling referenece value of the x axis for
/// HIFN  I/Q and statistical displays.
/// HIFN
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR referenceValue/Defines the reference value for the X axis of the measurement diagram.
/// HIPAR referencePosition/Defines the position of the reference value for the X axis.
ViStatus _VI_FUNC rsspecan_ConfigureVSADisplayXAxisReference(ViSession instrSession,
                                                             ViInt32   window,
                                                             ViInt32   reserved,
                                                             ViInt32   referenceValue,
                                                             ViInt32   referencePosition)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE];
             reserved;

    checkErr(RsCore_LockSession(instrSession));

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "Win%ld", window);
    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_VSA_SCALE_X_RVAL, referenceValue),
            4, "Reference Value");
    snprintf(repCap, RS_REPCAP_BUF_SIZE, "Win%ld", window);
    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_VSA_SCALE_X_RPOS, referencePosition),
            5, "Reference Position");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function configures the default unit (symbols or seconds) for the
/// HIFN  x axis.
/// HIFN
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR unit/Sets the time unit.
ViStatus _VI_FUNC rsspecan_ConfigureVSAUnitTime(ViSession instrSession,
                                                ViInt32   window,
                                                ViInt32   unit)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "Win%ld", window);
    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_VSA_UNIT_TIME, unit),
            3, "Unit");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function the default unit for angles.
/// HIFN
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR unitAngle/Sets the default unit for angles.
ViStatus _VI_FUNC rsspecan_ConfigureVSAUnitAngle(ViSession instrSession,
                                                 ViInt32   window,
                                                 ViInt32   unitAngle)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "Win%ld", window);
    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_VSA_UNIT_ANGLE, unitAngle),
            3, "Unit Angle");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function configures the default unit (symbols or seconds) for the
/// HIFN  y axis of equalizer group delay measurements.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR unit/Sets the time unit.
ViStatus _VI_FUNC rsspecan_ConfigureVSAYAxisUnitTime(ViSession instrSession,
                                                     ViInt32   window,
                                                     ViInt32   unit)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_VSA_Y_AXIS_UNIT_TIME,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap),
            2, "Window");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_VSA_Y_AXIS_UNIT_TIME, unit),
            3, "Unit");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function adds a window to the display.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init.
/// HIPAR stringSize/This control sets the allocated length of Returned Window Name string.
/// HIPAR referenceWindowName/String containing the number or name of the window the new window is inserted next to.
/// HIPAR direction/Direction the window is added relative to the window set with "Window Name".
/// HIPAR windowType/Type of result display you want to add.
/// HIPAR returnedWindowName/Returns the name of the new window.
ViStatus _VI_FUNC rsspecan_AddVSAWindow(ViSession instrSession,
                                        ViInt32   stringSize,
                                        ViString  referenceWindowName,
                                        ViInt32   direction,
                                        ViInt32   windowType,
                                        ViChar    returnedWindowName[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViChar   response[RS_MAX_MESSAGE_BUF_SIZE];
    ViChar*  p2direction;
    ViChar*  p2type;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &rsspecan_rngLayoutDirection, direction, &p2direction),
                4, "Direction");
    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &rsspecan_rngVSALayoutType, windowType, &p2type),
                5, "Window Type");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "LAY:ADD? '%s',%s,%s", referenceWindowName, p2direction, p2type);
    checkErr(RsCore_QueryViString(instrSession, cmd, response));

    if (returnedWindowName != NULL)
    {
        RsCore_TrimString(response, RS_VAL_TRIM_WHITESPACES_AND_ALL_QUOTES);
        RsCore_CopyToUserBufferAsciiData(returnedWindowName, stringSize, response);
    }

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function replaces a window with another one.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init.
/// HIPAR windowName/String containing the number or name of the window the new window is inserted next to.
/// HIPAR windowType/Type of result display you want to add.
ViStatus _VI_FUNC rsspecan_ReplaceVSAWindow(ViSession instrSession,
                                            ViString  windowName,
                                            ViInt32   windowType)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViChar*  p2type;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &rsspecan_rngVSALayoutType, windowType, &p2type),
                3, "Window Type");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "LAY:REPL '%s',%s", windowName, p2type);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function loads I/Q RAW data files. The file name includes
/// HIFN  indication of the path and the drive name. The path name complies with
/// HIFN  DOS conventions. This function is only available if RAW data with
/// HIFN  binary format is selected (":FORM:DEXP:MODE RAW",  ":FORM:DEXP:RAW:FORM
/// HIFN  BIN" ).
/// HIFN
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR fileName/Specifies full path of the file, where trace data are stored.
ViStatus _VI_FUNC rsspecan_ConfigureVSAFileImport(ViSession instrSession,
                                                  ViString  fileName)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidNullPointer(instrSession, fileName),
                2, "File Name");

    checkErr(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_VSA_FILE_DATA_MODE, RSSPECAN_VAL_EXP_MODE_RAW));
    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "MMEM:LOAD:TRAC 1,'%s'", fileName);
    checkErr(RsCore_Write(instrSession, cmd));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function exports the data to file in defined format.
/// HIFN
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR trace/Selects the trace for Detector definition.
/// HIPAR decimalSeparator/Defines which decimal separator (decimal point or comma) is to be used
/// HIPAR decimalSeparator/for outputting measurement data to the file in ASCII format.
/// HIPAR header/Defines if a file header (including start frequency, sweep time,
/// HIPAR header/detector, etc.) is created or not. A small header with the instrument
/// HIPAR header/model, the version and the date is always transferred.
/// HIPAR fileName/Specifies full path of the file, where trace data will be stored.
ViStatus _VI_FUNC rsspecan_ConfigureVSAFileExport(ViSession instrSession,
                                                  ViInt32   trace,
                                                  ViInt32   decimalSeparator,
                                                  ViBoolean header,
                                                  ViInt32   dataMode,
                                                  ViString  fileName)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, trace, 1, 6),
            2, "Trace");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, fileName),
                6, "File Name");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_FILE_DEC_SEPARATOR, decimalSeparator),
            3, "Decimal Separator");

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_VSA_FILE_EXP_HEADER, header),
            4, "Header");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_VSA_FILE_DATA_MODE, dataMode),
            5, "Data Mode");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "FORM ASC;MMEM:STOR:TRAC %ld,'%s'", trace, fileName);
    checkErr(RsCore_Write(instrSession, cmd));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function restores the default limits and deactivates all checks.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR type/Selects the limit type.
ViStatus _VI_FUNC rsspecan_ConfigureVSALimitsDefault(ViSession instrSession,
                                                     ViInt32   type)
{
    ViStatus error = VI_SUCCESS;
             type;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsspecan_SetAttributeViString(instrSession, NULL, RSSPECAN_ATTR_VSA_LIMIT_MODULATION_ACCURACY_DEFAULTS, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function configures the value for EVM limits.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR limitType/Selects the limit with which the currently measured, mean or peak
/// HIPAR limitType/value is to be compared.
/// HIPAR evmRMS/EVM RMS limit value.
/// HIPAR checkRMS/Switches the limit check On or Off.
/// HIPAR evmPeak/EVM Peak limit value.
/// HIPAR checkPeak/Switches the limit check On or Off.
ViStatus _VI_FUNC rsspecan_ConfigureVSALimitsEVM(ViSession instrSession,
                                                 ViInt32   limitType,
                                                 ViReal64  evmRMS,
                                                 ViBoolean checkRMS,
                                                 ViReal64  evmPeak,
                                                 ViBoolean checkPeak)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    switch (limitType)
    {
    case RSSPECAN_VAL_VSA_LIMIT_CURRENT:
        viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_VSA_LIMIT_MOD_ACC_EVM_RMS_CURR, checkRMS),
                4, "Check RMS");

        viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_VSA_LIMIT_MOD_ACC_EVM_RMS_CURR_VAL, evmRMS),
                3, "EVM RMS");

        viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_VSA_LIMIT_MOD_ACC_EVM_PEAK_CURR, checkPeak),
                6, "Check Peak");

        viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_VSA_LIMIT_MOD_ACC_EVM_PEAK_CURR_VAL, evmPeak),
                5, "EVM Peak");
        break;

    case RSSPECAN_VAL_VSA_LIMIT_MEAN:
        viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_VSA_LIMIT_MOD_ACC_EVM_RMS_MEAN, checkRMS),
                4, "Check RMS");

        viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_VSA_LIMIT_MOD_ACC_EVM_RMS_MEAN_VAL, evmRMS),
                3, "EVM RMS");

        viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_VSA_LIMIT_MOD_ACC_EVM_PEAK_MEAN, checkPeak),
                6, "Check Peak");

        viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_VSA_LIMIT_MOD_ACC_EVM_PEAK_MEAN_VAL, evmPeak),
                5, "EVM Peak");
        break;

    case RSSPECAN_VAL_VSA_LIMIT_PEAK:
        viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_VSA_LIMIT_MOD_ACC_EVM_RMS_PEAK, checkRMS),
                4, "Check RMS");

        viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_VSA_LIMIT_MOD_ACC_EVM_RMS_PEAK_VAL, evmRMS),
                3, "EVM RMS");

        viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_VSA_LIMIT_MOD_ACC_EVM_PEAK_PEAK, checkPeak),
                6, "Check Peak");

        viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_VSA_LIMIT_MOD_ACC_EVM_PEAK_PEAK_VAL, evmPeak),
                5, "EVM Peak");
        break;

    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, limitType), 2, "Limit Type");
        break;
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function configures the value for phase error limits.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR limitType/Selects the limit with which the currently measured, mean or peak
/// HIPAR limitType/value is to be compared.
/// HIPAR phaseErrorRMS/Phase error RMS limit value.
/// HIPAR checkRMS/Switches the limit check On or Off.
/// HIPAR phaseErrorPeak/Phase error peak limit value.
/// HIPAR checkPeak/Switches the limit check On or Off.
ViStatus _VI_FUNC rsspecan_ConfigureVSALimitsPhaseError(ViSession instrSession,
                                                        ViInt32   limitType,
                                                        ViReal64  phaseErrorRMS,
                                                        ViBoolean checkRMS,
                                                        ViReal64  phaseErrorPeak,
                                                        ViBoolean checkPeak)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    switch (limitType)
    {
    case RSSPECAN_VAL_VSA_LIMIT_CURRENT:
        viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_VSA_LIMIT_MOD_ACC_PERR_RMS_CURR, checkRMS),
                4, "Check RMS");

        viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_VSA_LIMIT_MOD_ACC_PHASE_ERROR_RMS_CURR_VAL, phaseErrorRMS),
                3, "Phase Error RMS");

        viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_VSA_LIMIT_MOD_ACC_PERR_PEAK_CURR, checkPeak),
                6, "Check Peak");

        viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_VSA_LIMIT_MOD_ACC_PHASE_ERROR_PEAK_CURR_VAL, phaseErrorPeak),
                5, "Phase Error Peak");
        break;

    case RSSPECAN_VAL_VSA_LIMIT_MEAN:
        viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_VSA_LIMIT_MOD_ACC_PERR_RMS_MEAN, checkRMS),
                4, "Check RMS");

        viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_VSA_LIMIT_MOD_ACC_PHASE_ERROR_RMS_MEAN_VAL, phaseErrorRMS),
                3, "Phase Error RMS");

        viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_VSA_LIMIT_MOD_ACC_PERR_PEAK_MEAN, checkPeak),
                6, "Check Peak");

        viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_VSA_LIMIT_MOD_ACC_PHASE_ERROR_PEAK_MEAN_VAL, phaseErrorPeak),
                5, "Phase Error Peak");
        break;

    case RSSPECAN_VAL_VSA_LIMIT_PEAK:
        viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_VSA_LIMIT_MOD_ACC_PERR_RMS_PEAK, checkRMS),
                4, "Check RMS");

        viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_VSA_LIMIT_MOD_ACC_PHASE_ERROR_RMS_PEAK_VAL, phaseErrorRMS),
                3, "Phase Error RMS");

        viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_VSA_LIMIT_MOD_ACC_PERR_PEAK_PEAK, checkPeak),
                6, "Check Peak");

        viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_VSA_LIMIT_MOD_ACC_PHASE_ERROR_PEAK_PEAK_VAL, phaseErrorPeak),
                5, "Phase Error Peak");
        break;

    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, limitType), 2, "Limit Type");
        break;
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function configures the value for magnitude error limits.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR limitType/Selects the limit with which the currently measured, mean or peak
/// HIPAR limitType/value is to be compared.
/// HIPAR magnitudeErrorRMS/Magnitude error RMS limit value.
/// HIPAR checkRMS/Switches the limit check On or Off.
/// HIPAR magnitudeErrorPeak/Magnitude error peak limit value.
/// HIPAR checkPeak/Switches the limit check On or Off.
ViStatus _VI_FUNC rsspecan_ConfigureVSALimitsMagnitudeError(ViSession instrSession,
                                                            ViInt32   limitType,
                                                            ViReal64  magnitudeErrorRMS,
                                                            ViBoolean checkRMS,
                                                            ViReal64  magnitudeErrorPeak,
                                                            ViBoolean checkPeak)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    switch (limitType)
    {
    case RSSPECAN_VAL_VSA_LIMIT_CURRENT:
        viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_VSA_LIMIT_MOD_ACC_MERR_RMS_CURR, checkRMS),
                4, "Check RMS");

        viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_VSA_LIMIT_MOD_ACC_MAGNITUDE_ERROR_RMS_CURR_VAL, magnitudeErrorRMS),
                3, "Magnitude Error RMS");

        viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_VSA_LIMIT_MOD_ACC_MERR_PEAK_CURR, checkPeak),
                6, "Check Peak");

        viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_VSA_LIMIT_MOD_ACC_MAGNITUDE_ERROR_PEAK_CURR_VAL, magnitudeErrorPeak),
                5, "Magnitude Error Peak");
        break;

    case RSSPECAN_VAL_VSA_LIMIT_MEAN:
        viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_VSA_LIMIT_MOD_ACC_MERR_RMS_MEAN, checkRMS),
                4, "Check RMS");

        viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_VSA_LIMIT_MOD_ACC_MAGNITUDE_ERROR_RMS_MEAN_VAL, magnitudeErrorRMS),
                3, "Magnitude Error RMS");

        viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_VSA_LIMIT_MOD_ACC_MERR_PEAK_MEAN, checkPeak),
                6, "Check Peak");

        viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_VSA_LIMIT_MOD_ACC_MAGNITUDE_ERROR_PEAK_MEAN_VAL, magnitudeErrorPeak),
                5, "Magnitude Error Peak");
        break;

    case RSSPECAN_VAL_VSA_LIMIT_PEAK:
        viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_VSA_LIMIT_MOD_ACC_MERR_RMS_PEAK, checkRMS),
                4, "Check RMS");

        viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_VSA_LIMIT_MOD_ACC_MAGNITUDE_ERROR_RMS_PEAK_VAL, magnitudeErrorRMS),
                3, "Magnitude Error RMS");

        viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_VSA_LIMIT_MOD_ACC_MERR_PEAK_PEAK, checkPeak),
                6, "Check Peak");

        viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_VSA_LIMIT_MOD_ACC_MAGNITUDE_ERROR_PEAK_PEAK_VAL, magnitudeErrorPeak),
                5, "Magnitude Error Peak");
        break;

    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, limitType), 2, "Limit Type");
        break;
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function configures the value for carrier frequency error limits.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR limitType/Selects the limit with which the currently measured, mean or peak
/// HIPAR limitType/value is to be compared.
/// HIPAR carrierFrequencyError/Carrier frequency error limit value.
/// HIPAR check/Switches the limit check On or Off.
ViStatus _VI_FUNC rsspecan_ConfigureVSALimitsCarrierFrequencyError(
                            ViSession instrSession,
                            ViInt32   limitType,
                            ViReal64  carrierFrequencyError,
                            ViBoolean check)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    switch (limitType)
    {
    case RSSPECAN_VAL_VSA_LIMIT_CURRENT:
        viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_VSA_LIMIT_MOD_ACC_CFER_CURR, check),
                4, "Check");

        viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_VSA_LIMIT_MOD_ACC_CARRIER_FREQUENCY_ERROR_CURR_VAL, carrierFrequencyError),
                3, "Carrier Frequency Error");
        break;

    case RSSPECAN_VAL_VSA_LIMIT_MEAN:
        viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_VSA_LIMIT_MOD_ACC_CFER_MEAN, check),
                4, "Check");

        viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_VSA_LIMIT_MOD_ACC_CARRIER_FREQUENCY_ERROR_MEAN_VAL, carrierFrequencyError),
                3, "Carrier Frequency Error");
        break;

    case RSSPECAN_VAL_VSA_LIMIT_PEAK:
        viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_VSA_LIMIT_MOD_ACC_CFER_PEAK, check),
                4, "Check");

        viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_VSA_LIMIT_MOD_ACC_CARRIER_FREQUENCY_ERROR_PEAK_VAL, carrierFrequencyError),
                3, "Carrier Frequency Error");
        break;

    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, limitType), 2, "Limit Type");
        break;
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function configures the value for RHO limits.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR limitType/Selects the limit with which the currently measured, mean or peak
/// HIPAR limitType/value is to be compared.
/// HIPAR rho/RHO limit value.
/// HIPAR check/Switches the limit check On or Off.
ViStatus _VI_FUNC rsspecan_ConfigureVSALimitsRHO(ViSession instrSession,
                                                 ViInt32   limitType,
                                                 ViReal64  rho,
                                                 ViBoolean check)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    switch (limitType)
    {
    case RSSPECAN_VAL_VSA_LIMIT_CURRENT:
        viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_VSA_LIMIT_MOD_ACC_RHO_CURR, check),
                4, "Check");

        viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_VSA_LIMIT_MOD_ACC_RHO_CURR_VAL, rho),
                3, "RHO");
        break;

    case RSSPECAN_VAL_VSA_LIMIT_MEAN:
        viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_VSA_LIMIT_MOD_ACC_RHO_MEAN, check),
                4, "Check");

        viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_VSA_LIMIT_MOD_ACC_RHO_MEAN_VAL, rho),
                3, "RHO");
        break;

    case RSSPECAN_VAL_VSA_LIMIT_PEAK:
        viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_VSA_LIMIT_MOD_ACC_RHO_PEAK, check),
                4, "Check");

        viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_VSA_LIMIT_MOD_ACC_RHO_PEAK_VAL, rho),
                3, "RHO");
        break;

    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, limitType), 2, "Limit Type");
        break;
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function configures the value for IQ Offset limits.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR limitType/Selects the limit with which the currently measured, mean or peak
/// HIPAR limitType/value is to be compared.
/// HIPAR iqOffset/IQ offset limit value.
/// HIPAR check/Switches the limit check On or Off.
ViStatus _VI_FUNC rsspecan_ConfigureVSALimitsIQOffset(ViSession instrSession,
                                                      ViInt32   limitType,
                                                      ViReal64  iqOffset,
                                                      ViBoolean check)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    switch (limitType)
    {
    case RSSPECAN_VAL_VSA_LIMIT_CURRENT:
        viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_VSA_LIMIT_MOD_ACC_IQOF_CURR, check),
                4, "Check");

        viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_VSA_LIMIT_MOD_ACC_IQ_OFFSET_CURR_VAL, iqOffset),
                3, "IQ Offset");
        break;

    case RSSPECAN_VAL_VSA_LIMIT_MEAN:
        viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_VSA_LIMIT_MOD_ACC_IQOF_MEAN, check),
                4, "Check");

        viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_VSA_LIMIT_MOD_ACC_IQ_OFFSET_MEAN_VAL, iqOffset),
                3, "IQ Offset");
        break;

    case RSSPECAN_VAL_VSA_LIMIT_PEAK:
        viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_VSA_LIMIT_MOD_ACC_IQOF_PEAK, check),
                4, "Check");

        viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_VSA_LIMIT_MOD_ACC_IQ_OFFSET_PEAK_VAL, iqOffset),
                3, "IQ Offset");
        break;

    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, limitType), 2, "Limit Type");
        break;
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function configures the value for frequency error limits.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR limitType/Selects the limit with which the currently measured, mean or peak
/// HIPAR limitType/value is to be compared.
/// HIPAR frequencyErrorRMS/Frequency error RMS limit value
/// HIPAR checkRMS/Switches the limit check On or Off.
/// HIPAR frequencyErrorPeak/Frequency error Peak limit value.
/// HIPAR checkPeak/Switches the limit check On or Off.
ViStatus _VI_FUNC rsspecan_ConfigureVSALimitsFrequencyError(ViSession instrSession,
                                                            ViInt32   limitType,
                                                            ViReal64  frequencyErrorRMS,
                                                            ViBoolean checkRMS,
                                                            ViReal64  frequencyErrorPeak,
                                                            ViBoolean checkPeak)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    switch (limitType)
    {
    case RSSPECAN_VAL_VSA_LIMIT_CURRENT:
        viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_VSA_LIMIT_MOD_ACC_FERR_RMS_CURR, checkRMS),
                4, "Check RMS");

        viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_VSA_LIMIT_MOD_ACC_FERR_RMS_CURR_VAL, frequencyErrorRMS),
                3, "Frequency Error RMS");

        viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_VSA_LIMIT_MOD_ACC_FERR_PEAK_CURR, checkPeak),
                6, "Check Peak");

        viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_VSA_LIMIT_MOD_ACC_FERR_PEAK_CURR_VAL, frequencyErrorPeak),
                5, "Frequency Error Peak");
        break;

    case RSSPECAN_VAL_VSA_LIMIT_MEAN:
        viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_VSA_LIMIT_MOD_ACC_FERR_RMS_MEAN, checkRMS),
                4, "Check RMS");

        viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_VSA_LIMIT_MOD_ACC_FERR_RMS_MEAN_VAL, frequencyErrorRMS),
                3, "Frequency Error RMS");

        viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_VSA_LIMIT_MOD_ACC_FERR_PEAK_MEAN, checkPeak),
                6, "Check Peak");

        viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_VSA_LIMIT_MOD_ACC_FERR_PEAK_MEAN_VAL, frequencyErrorPeak),
                5, "Frequency Error Peak");
        break;

    case RSSPECAN_VAL_VSA_LIMIT_PEAK:
        viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_VSA_LIMIT_MOD_ACC_FERR_RMS_PEAK, checkRMS),
                4, "Check RMS");

        viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_VSA_LIMIT_MOD_ACC_FERR_RMS_PEAK_VAL, frequencyErrorRMS),
                3, "Frequency Error RMS");

        viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_VSA_LIMIT_MOD_ACC_FERR_PEAK_PEAK, checkPeak),
                6, "Check Peak");

        viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_VSA_LIMIT_MOD_ACC_FERR_PEAK_PEAK_VAL, frequencyErrorPeak),
                5, "Frequency Error Peak");
        break;

    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, limitType), 2, "Limit Type");
        break;
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function configures the value for center frequency deviation
/// HIFN  error limits.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR limitType/Selects the limit with which the currently measured, mean or peak
/// HIPAR limitType/value is to be compared.
/// HIPAR freqDevError/Center frequency deviation error limit value.
/// HIPAR check/Switches the limit check On or Off.
ViStatus _VI_FUNC rsspecan_ConfigureVSALimitsFrequencyDeviationError(
                            ViSession instrSession,
                            ViInt32   limitType,
                            ViReal64  freqDevError,
                            ViBoolean check)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    switch (limitType)
    {
    case RSSPECAN_VAL_VSA_LIMIT_CURRENT:
        viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_VSA_LIMIT_MOD_ACC_FDER_CURR, check),
                4, "Check");

        viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_VSA_LIMIT_MOD_ACC_FREQUENCY_DEVIATION_CURR_VAL, freqDevError),
                3, "Freq Dev Error");
        break;

    case RSSPECAN_VAL_VSA_LIMIT_MEAN:
        viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_VSA_LIMIT_MOD_ACC_FDER_MEAN, check),
                4, "Check");

        viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_VSA_LIMIT_MOD_ACC_FREQUENCY_DEVIATION_MEAN_VAL, freqDevError),
                3, "Freq Dev Error");
        break;

    case RSSPECAN_VAL_VSA_LIMIT_PEAK:
        viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_VSA_LIMIT_MOD_ACC_FDER_PEAK, check),
                4, "Check");

        viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_VSA_LIMIT_MOD_ACC_FREQUENCY_DEVIATION_PEAK_VAL, freqDevError),
                3, "Freq Dev Error");
        break;

    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, limitType), 2, "Limit Type");
        break;
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the absolute eye-diagram limit line.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR limitLine/Selects the limit line type.
/// HIPAR absoluteEnabled/Displays an absolute line in the specified window.
/// HIPAR absoluteValue/Defines or queries the x-value of the absolute vertical line in the
/// HIPAR absoluteValue/specified window. Only available for eye diagrams, and only if an
/// HIPAR absoluteValue/absolute vertical line is already available in the diagram
ViStatus _VI_FUNC rsspecan_ConfigureVSAEyeDiagramAbsoluteLimitLine(
                            ViSession instrSession,
                            ViInt32   limitLine,
                            ViBoolean absoluteEnabled,
                            ViReal64  absoluteValue)
{
    ViStatus error = VI_SUCCESS;
    ViInt32  win = 0;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsspecan_GetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_ACTIVE_WINDOW, &win));
    snprintf(repCap, RS_REPCAP_BUF_SIZE, "Win%ld", win);

    switch (limitLine)
    {
    case RSSPECAN_VAL_VERTICAL:
        viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, repCap, RSSPECAN_ATTR_VSA_EYE_DIAGRAM_VERTICAL_LIMIT_LINE_ABSOLUTE_ENABLED, absoluteEnabled),
                3, "Absolute Enabled");

        if (absoluteEnabled == VI_TRUE)
        {
            viCheckParm(rsspecan_SetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_VSA_EYE_DIAGRAM_VERTICAL_LIMIT_LINE_ABSOLUTE, absoluteValue),
                    4, "absoluteValue");
        }
        break;

    case RSSPECAN_VAL_HORIZONTAL:
        viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, repCap, RSSPECAN_ATTR_VSA_EYE_DIAGRAM_HORIZONTAL_LIMIT_LINE_ABSOLUTE_ENABLED, absoluteEnabled),
                3, "Absolute Enabled");

        if (absoluteEnabled == VI_TRUE)
        {
            viCheckParm(rsspecan_SetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_VSA_EYE_DIAGRAM_HORIZONTAL_LIMIT_LINE_ABSOLUTE, absoluteValue),
                    4, "absoluteValue");
        }
        break;

    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, limitLine), 2, "Limit Line");
        break;
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function configures the absolute eye-diagram limit line.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR limitLine/Selects the limit line type.
/// HIPAR relativeEnabled/Displays an relative line in the specified window.
/// HIPAR relativeValue/Defines or queries the x-value of the relative vertical line in the
/// HIPAR relativeValue/specified window. Only available for eye diagrams, and only if an
/// HIPAR relativeValue/absolute vertical line is already available in the diagram
ViStatus _VI_FUNC rsspecan_ConfigureVSAEyeDiagramRelativeLimitLine(
                            ViSession instrSession,
                            ViInt32   limitLine,
                            ViBoolean relativeEnabled,
                            ViReal64  relativeValue)
{
    ViStatus error = VI_SUCCESS;
    ViInt32  win = 0;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsspecan_GetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_ACTIVE_WINDOW, &win));
    snprintf(repCap, RS_REPCAP_BUF_SIZE, "Win%ld", win);

    switch (limitLine)
    {
    case RSSPECAN_VAL_VERTICAL:
        viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, repCap, RSSPECAN_ATTR_VSA_EYE_DIAGRAM_VERTICAL_LIMIT_LINE_RELATIVE_ENABLED, relativeEnabled),
                3, "Relative Enabled");

        if (relativeEnabled == VI_TRUE)
        {
            viCheckParm(rsspecan_SetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_VSA_EYE_DIAGRAM_VERTICAL_LIMIT_LINE_RELATIVE, relativeValue),
                    4, "Relative Value");
        }
        break;

    case RSSPECAN_VAL_HORIZONTAL:
        viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, repCap, RSSPECAN_ATTR_VSA_EYE_DIAGRAM_HORIZONTAL_LIMIT_LINE_RELATIVE_ENABLED, relativeEnabled),
                3, "Relative Enabled");

        if (relativeEnabled == VI_TRUE)
        {
            viCheckParm(rsspecan_SetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_VSA_EYE_DIAGRAM_HORIZONTAL_LIMIT_LINE_RELATIVE, relativeValue),
                    4, "Relative Value");
        }
        break;

    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, limitLine), 2, "Limit Line");
        break;
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the VSA IQ Stream.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR autoStream/Only available for files that contain more than one data stream from
/// HIPAR autoStream/multiple channels: automatically defines which data stream in the file
/// HIPAR autoStream/is used as input for the channel.
/// HIPAR fileName/Only available for files that contain more than one data stream from
/// HIPAR fileName/multiple channels: selects the data stream to be used as input for the
/// HIPAR fileName/currently selected channel.
ViStatus _VI_FUNC rsspecan_ConfigureVSAIQLoadStream(ViSession instrSession,
                                                    ViBoolean autoStream,
                                                    ViString  fileName)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_VSA_IQ_LOAD_STREAM_AUTO_ENABLED, autoStream),
            2, "Auto Stream");

    if (autoStream == VI_FALSE)
    {
        viCheckParm(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_VSA_IQ_LOAD_STREAM, fileName),
                3, "File Name");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function returns the available channels in the currently loaded
/// HIFN  input file.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR arraySize/Sets the Array Size parameter.
/// HIPAR streamList/Returns the available channels in the currently loaded input file.
ViStatus _VI_FUNC rsspecan_QueryVSAIQStreamList(ViSession instrSession,
                                                ViInt32   arraySize,
                                                ViChar    streamList[])
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_GetAttributeViString(instrSession, "", RSSPECAN_ATTR_QUERY_VSA_IQ_STREAM_LISTS,
            arraySize, streamList),
            3, "Stream List");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the display of the decision instants (time
/// HIFN  when the signals occurred) on the trace.
/// HIFN
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR highlightSymbols/If enabled, the symbol instants are highlighted as squares in the
/// HIPAR highlightSymbols/screen for measured and reference signals in time (normal) display, as
/// HIPAR highlightSymbols/well as error displays.
ViStatus _VI_FUNC rsspecan_ConfigureVSAHighlightSymbols(ViSession instrSession,
                                                        ViInt32   window,
                                                        ViInt32   reserved,
                                                        ViBoolean highlightSymbols)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE];
             reserved;

    checkErr(RsCore_LockSession(instrSession));

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "Win%ld", window);

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, repCap, RSSPECAN_ATTR_VSA_CONFIGURE_DISPLAY_TRACE_SYMBOL_STATE, highlightSymbols),
            4, "Highlight Symbols");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function configures the VSA measurement or reference signal
/// HIFN  result displays.
/// HIFN
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR resultType/Sets the displayed measurement.
ViStatus _VI_FUNC rsspecan_ConfigureVSACaptureBuffer(ViSession instrSession,
                                                     ViInt32   window,
                                                     ViInt32   resultType)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "Win%ld", window);

    if (rsspecan_IsFSV(instrSession))
    {
        checkErr(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_VSA_RESULT, RSSPECAN_VAL_RESULT_TCAP));
    }

    switch (resultType)
    {
    case RSSPECAN_VAL_VSA_RESULT_DISP_MAG_ABS:
        checkErr(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_VSA_RESULT_FORMAT, RSSPECAN_VAL_VSA_RESULT_FORMAT_MAGN));
        break;

    case RSSPECAN_VAL_VSA_RESULT_DISP_RI:
        checkErr(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_VSA_RESULT_FORMAT, RSSPECAN_VAL_VSA_RESULT_FORMAT_RIM));
        break;

    case RSSPECAN_VAL_VSA_RESULT_DISP_FREQ_ABS:
        checkErr(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_VSA_RESULT_FORMAT, RSSPECAN_VAL_VSA_RESULT_FORMAT_FREQ));
        checkErr(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_DISP_AMPLITUDE_GRID_MODE, RSSPECAN_VAL_ABS));
        break;

    case RSSPECAN_VAL_VSA_RESULT_DISP_IQ_VECT:
        checkErr(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_VSA_RESULT_FORMAT, RSSPECAN_VAL_VSA_RESULT_FORMAT_COMP));
        break;
    case RSSPECAN_VAL_VSA_RESULT_DISP_MOV:
        checkErr(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_VSA_RESULT_FORMAT, RSSPECAN_VAL_VSA_RESULT_FORMAT_MOV));
        break;

    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, resultType), 3, "Result Type");
        break;
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function configures the VSA measurement result displays.
/// HIFN
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR resultType/Sets the displayed measurement.
ViStatus _VI_FUNC rsspecan_ConfigureVSAMeasurementSignal(ViSession instrSession,
                                                         ViInt32   window,
                                                         ViInt32   resultType)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "Win%ld", window);

    if (rsspecan_IsFSV(instrSession))
    {
        checkErr(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_VSA_RESULT, RSSPECAN_VAL_RESULT_XTIM_DDEM_MEAS));
    }

    switch (resultType)
    {
    case RSSPECAN_VAL_VSA_RESULT_DISP_MAG_ABS:
        checkErr(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_VSA_RESULT_FORMAT, RSSPECAN_VAL_VSA_RESULT_FORMAT_MAGN));
        checkErr(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_DISP_AMPLITUDE_GRID_MODE, RSSPECAN_VAL_ABS));
        break;

    case RSSPECAN_VAL_VSA_RESULT_DISP_MAG_REL:
        checkErr(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_VSA_RESULT_FORMAT, RSSPECAN_VAL_VSA_RESULT_FORMAT_MAGN));
        checkErr(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_DISP_AMPLITUDE_GRID_MODE, RSSPECAN_VAL_REL));
        break;

    case RSSPECAN_VAL_VSA_RESULT_DISP_PHA_WRAP:
        checkErr(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_VSA_RESULT_FORMAT, RSSPECAN_VAL_VSA_RESULT_FORMAT_PHAS));
        break;

    case RSSPECAN_VAL_VSA_RESULT_DISP_PHA_UNWRAP:
        checkErr(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_VSA_RESULT_FORMAT, RSSPECAN_VAL_VSA_RESULT_FORMAT_UPHAS));
        break;

    case RSSPECAN_VAL_VSA_RESULT_DISP_FREQ_ABS:
        checkErr(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_VSA_RESULT_FORMAT, RSSPECAN_VAL_VSA_RESULT_FORMAT_FREQ));
        checkErr(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_DISP_AMPLITUDE_GRID_MODE, RSSPECAN_VAL_ABS));
        break;

    case RSSPECAN_VAL_VSA_RESULT_DISP_FREQ_REL:
        checkErr(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_VSA_RESULT_FORMAT, RSSPECAN_VAL_VSA_RESULT_FORMAT_FREQ));
        checkErr(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_DISP_AMPLITUDE_GRID_MODE, RSSPECAN_VAL_REL));
        break;

    case RSSPECAN_VAL_VSA_RESULT_DISP_RI:
        checkErr(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_VSA_RESULT_FORMAT, RSSPECAN_VAL_VSA_RESULT_FORMAT_RIM));
        break;

    case RSSPECAN_VAL_VSA_RESULT_DISP_EYE_R:
        checkErr(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_VSA_RESULT_FORMAT, RSSPECAN_VAL_VSA_RESULT_FORMAT_IEYE));
        break;

    case RSSPECAN_VAL_VSA_RESULT_DISP_EYE_I:
        checkErr(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_VSA_RESULT_FORMAT, RSSPECAN_VAL_VSA_RESULT_FORMAT_QEYE));
        break;

    case RSSPECAN_VAL_VSA_RESULT_DISP_EYE_FREQ:
        checkErr(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_VSA_RESULT_FORMAT, RSSPECAN_VAL_VSA_RESULT_FORMAT_FEYE));
        break;

    case RSSPECAN_VAL_VSA_RESULT_DISP_IQ_CONST:
        checkErr(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_VSA_RESULT_FORMAT, RSSPECAN_VAL_VSA_RESULT_FORMAT_CONS));
        break;

    case RSSPECAN_VAL_VSA_RESULT_DISP_IQ_VECT:
        checkErr(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_VSA_RESULT_FORMAT, RSSPECAN_VAL_VSA_RESULT_FORMAT_COMP));
        break;

    case RSSPECAN_VAL_VSA_RESULT_DISP_CONST_FREQ:
        checkErr(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_VSA_RESULT_FORMAT, RSSPECAN_VAL_VSA_RESULT_FORMAT_CONF));
        break;

    case RSSPECAN_VAL_VSA_RESULT_DISP_VECT_FREQ:
        checkErr(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_VSA_RESULT_FORMAT, RSSPECAN_VAL_VSA_RESULT_FORMAT_CONVF));
        break;
    case RSSPECAN_VAL_VSA_RESULT_DISP_IQ_CON_ROT:
        checkErr(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_VSA_RESULT_FORMAT, RSSPECAN_VAL_VSA_RESULT_FORMAT_RCON));
        break;

    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, resultType), 3, "Result Type");
        break;
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function configures the VSA measurement or reference signal
/// HIFN  result displays.
/// HIFN
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR resultType/Sets the displayed measurement.
ViStatus _VI_FUNC rsspecan_ConfigureVSASymbols(ViSession instrSession,
                                               ViInt32   window,
                                               ViInt32   resultType)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE];
             resultType;

    checkErr(RsCore_LockSession(instrSession));

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "Win%ld", window);

    checkErr(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_VSA_RESULT, RSSPECAN_VAL_RESULT_XTIM_DDEM_SYMB));

    switch (resultType)
    {
    case RSSPECAN_VAL_VSA_RESULT_DISP_BIN:
        checkErr(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_VSA_RESULT_FORMAT, RSSPECAN_VAL_VSA_RESULT_FORMAT_BIN));
        break;

    case RSSPECAN_VAL_VSA_RESULT_DISP_DEC:
        checkErr(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_VSA_RESULT_FORMAT, RSSPECAN_VAL_VSA_RESULT_FORMAT_DEC));
        break;

    case RSSPECAN_VAL_VSA_RESULT_DISP_OCT:
        checkErr(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_VSA_RESULT_FORMAT, RSSPECAN_VAL_VSA_RESULT_FORMAT_OCT));
        break;

    case RSSPECAN_VAL_VSA_RESULT_DISP_HEX:
        checkErr(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_VSA_RESULT_FORMAT, RSSPECAN_VAL_VSA_RESULT_FORMAT_HEX));
        break;

    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, resultType), 3, "Result Type");
        break;
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function configures the VSA measurement or reference signal
/// HIFN  result displays.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR resultType/Sets the displayed measurement.
ViStatus _VI_FUNC rsspecan_ConfigureVSAErrorVector(ViSession instrSession,
                                                   ViInt32   window,
                                                   ViInt32   resultType)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "Win%ld", window);

    checkErr(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_VSA_RESULT, RSSPECAN_VAL_RESULT_XTIM_DDEM_ERR_VECT));

    switch (resultType)
    {
    case RSSPECAN_VAL_VSA_RESULT_DISP_EVM:
        checkErr(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_VSA_RESULT_FORMAT, RSSPECAN_VAL_VSA_RESULT_FORMAT_MAGN));
        break;

    case RSSPECAN_VAL_VSA_RESULT_DISP_RI:
        checkErr(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_VSA_RESULT_FORMAT, RSSPECAN_VAL_VSA_RESULT_FORMAT_RIM));
        break;

    case RSSPECAN_VAL_VSA_RESULT_DISP_IQ_VECT:
        checkErr(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_VSA_RESULT_FORMAT, RSSPECAN_VAL_VSA_RESULT_FORMAT_COMP));
        break;

    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, resultType), 3, "Result Type");
        break;
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function configures the VSA measurement or reference signal
/// HIFN  result displays.
/// HIFN
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR resultType/Sets the displayed measurement.
ViStatus _VI_FUNC rsspecan_ConfigureVSAModulationErrors(ViSession instrSession,
                                                        ViInt32   window,
                                                        ViInt32   resultType)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "Win%ld", window);

    checkErr(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_VSA_RESULT, RSSPECAN_VAL_RESULT_XTIM_DDEM_ERR_MPH));

    switch (resultType)
    {
    case RSSPECAN_VAL_VSA_RESULT_DISP_MAG_ERR:
        checkErr(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_VSA_RESULT_FORMAT, RSSPECAN_VAL_VSA_RESULT_FORMAT_MAGN));
        break;

    case RSSPECAN_VAL_VSA_RESULT_DISP_PHA_ERR:
        checkErr(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_VSA_RESULT_FORMAT, RSSPECAN_VAL_VSA_RESULT_FORMAT_PHAS));
        break;

    case RSSPECAN_VAL_VSA_RESULT_DISP_FREQ_ERR_ABS:
        checkErr(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_VSA_RESULT_FORMAT, RSSPECAN_VAL_VSA_RESULT_FORMAT_FREQ));
        checkErr(rsspecan_SetAttributeViInt32(instrSession, "",
            RSSPECAN_ATTR_DISP_AMPLITUDE_GRID_MODE,
            RSSPECAN_VAL_ABS))
        break;

    case RSSPECAN_VAL_VSA_RESULT_DISP_FREQ_ERR_REL:
        checkErr(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_VSA_RESULT_FORMAT, RSSPECAN_VAL_VSA_RESULT_FORMAT_FREQ));
        checkErr(rsspecan_SetAttributeViInt32(instrSession, "",
            RSSPECAN_ATTR_DISP_AMPLITUDE_GRID_MODE,
            RSSPECAN_VAL_REL))
        break;

    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, resultType), 3, "Result Type");
        break;
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function configures the VSA signal source to modulation accuracy.
/// HIFN
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR window/This control selects the measurement window.
ViStatus _VI_FUNC rsspecan_ConfigureVSAModulationAccuracy(ViSession instrSession,
                                                          ViInt32   window)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "Win%ld", window);

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_VSA_RESULT, RSSPECAN_VAL_RESULT_MACC),
            3, "");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the VSA signal source to modulation accuracy.
/// HIFN
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR resultType/Sets the displayed measurement.
ViStatus _VI_FUNC rsspecan_ConfigureVSAModulationAccuracyAdditional(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   resultType)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "Win%ld", window);

    if (rsspecan_IsFSV(instrSession))
    {
        checkErr(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_VSA_RESULT, RSSPECAN_VAL_RESULT_MACC));
    }

    switch (resultType)
    {
    case RSSPECAN_VAL_VSA_RESULT_FORMAT_RSUM:
        checkErr(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_VSA_RESULT_FORMAT, RSSPECAN_VAL_VSA_RESULT_FORMAT_RSUM));
        break;

    case RSSPECAN_VAL_VSA_RESULT_FORMAT_BER:
        checkErr(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_VSA_RESULT_FORMAT, RSSPECAN_VAL_VSA_RESULT_FORMAT_BER));
        break;

    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, resultType), 3, "Result Type");
        break;
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function configures the evaluation method of the
/// HIFN  measurement results.
/// HIFN
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR window/This control selects the measurement window
/// HIPAR transformation/Sets the result type transformation

ViStatus _VI_FUNC rsspecan_ConfigureVSAResultTypeTransformation(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   transformation)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "Win%ld", window);

    switch (transformation)
    {
    case RSSPECAN_VAL_VSA_TRANSFORMATION_NORMAL:
        checkErr(rsspecan_SetAttributeViBoolean(instrSession, repCap, RSSPECAN_ATTR_VSA_SPECTRUM_STATE, VI_FALSE));
        checkErr(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_MEAS_STAT_CCDF_STATE, VI_FALSE));
        break;

    case RSSPECAN_VAL_VSA_TRANSFORMATION_SPEC:
        checkErr(rsspecan_SetAttributeViBoolean(instrSession, repCap, RSSPECAN_ATTR_VSA_SPECTRUM_STATE, VI_TRUE));
        checkErr(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_MEAS_STAT_CCDF_STATE, VI_FALSE));
        break;

    case RSSPECAN_VAL_VSA_TRANSFORMATION_STAT:
        checkErr(rsspecan_SetAttributeViBoolean(instrSession, repCap, RSSPECAN_ATTR_VSA_SPECTRUM_STATE, VI_FALSE));
        checkErr(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_MEAS_STAT_CCDF_STATE, VI_TRUE));
        break;

    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, transformation), 3, "Transformation");
        break;
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures whether only the symbol points or all points
/// HIFN  are considered for the statistical calculations.
/// HIFN
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR oversampling/This control sets the oversampling state.
/// HIPAR statisticalMode/This control defines whether only the symbol points or all points are
/// HIPAR statisticalMode/considered for the statistical calculations.
ViStatus _VI_FUNC rsspecan_ConfigureVSACaptureOversampling(ViSession instrSession,
                                                           ViInt32   window,
                                                           ViInt32   oversampling,
                                                           ViInt32   statisticalMode)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "Win%ld", window);

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_VSA_CONFIGURE_DISPLAY_POINTS_PER_SYMBOL_AUTO, oversampling),
            3, "Oversampling");

    if (oversampling == RSSPECAN_VAL_MAN)
    {
        viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_VSA_CAPTURE_OVERSAMPLING, statisticalMode),
                4, "Statistical Mode");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function configures the points per symbol settings.
/// HIFN
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR pointsPerSymbol/This control turns auto mode of points per symbol on or off.
/// HIPAR numberOfPointsPerSymbol/This control determines the number of points to be displayed per
/// HIPAR numberOfPointsPerSymbol/symbol if manual mode is selected.
ViStatus _VI_FUNC rsspecan_ConfigureVSADisplayPointPerSymbol(ViSession instrSession,
                                                             ViInt32   window,
                                                             ViInt32   pointsPerSymbol,
                                                             ViInt32   numberOfPointsPerSymbol)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "Win%ld", window);

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_VSA_CONFIGURE_DISPLAY_POINTS_PER_SYMBOL_AUTO, pointsPerSymbol),
            3, "Points Per Symbol");

    if (pointsPerSymbol == RSSPECAN_VAL_MAN)
    {
        viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_VSA_CONFIGURE_DISPLAY_POINTS_PER_SYMBOL, numberOfPointsPerSymbol),
                4, "number Of Points Per Symbol");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function repeats the evaluation of the data currently in the
/// HIFN  capture buffer without capturing new data. This is useful after
/// HIFN  changing settings, for example filters, patterns or evaluation ranges.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsspecan_VSARefreshCapturedData(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_VSA_REFRESH_CAPTURED_DATA, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function returns the first value of the X axis in symbols or
/// HIFN  time, depending on the unit setting for the X axis.
/// HIFN
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR result/Returns the first X-Axis value.
ViStatus _VI_FUNC rsspecan_GetVSAXAxisFirstValue(ViSession instrSession,
                                                 ViReal64* result)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_VSA_X_FIRST_VALUE_QUERY, result),
            2, "Result");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function returns the first value of the X axis in symbols or
/// HIFN  time, depending on the unit setting for the X axis.
/// HIFN
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR result/Returns the filast X-Axis value.
ViStatus _VI_FUNC rsspecan_GetVSAXAxisLastValue(ViSession instrSession,
                                                ViReal64* result)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_VSA_X_LAST_VALUE_QUERY, result),
            2, "Result");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function returns the results of the error measurement carried out
/// HIFN  for digital demodulation.
/// HIFN
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR type/This control sets the result.
/// HIPAR modifier/This control sets the result. Available values depend on instrument
/// HIPAR modifier/used, refer to the notes below.
/// HIPAR result/Returns the measured result.
ViStatus _VI_FUNC rsspecan_GetVSAResult(ViSession instrSession,
                                        ViInt32   window,
                                        ViInt32   type,
                                        ViInt32   modifier,
                                        ViReal64* result)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, type, RSSPECAN_VAL_VSA_MPOW, RSSPECAN_VAL_VSA_IQSK),
            2, "Type");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, modifier, RSSPECAN_VAL_VSA_MODIF_NONE, RSSPECAN_VAL_VSA_MODIF_FSV_PPCT),
            3, "Modifier");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, result),
                4, "Result");

    if ((modifier == RSSPECAN_VAL_VSA_MODIF_NONE) || (modifier == RSSPECAN_VAL_VSA_MODIF_FSV_CURR))
    {
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CALC%ld:MARK:FUNC:%s?", window, VSAResultArr[type]);
        checkErr(RsCore_QueryViReal64(instrSession, cmd, result));
    }
    else
    {
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CALC%ld:MARK:FUNC:%s? %s", window, VSAResultArr[type],
                 VSAModifierArr[modifier]);
        checkErr(RsCore_QueryViReal64(instrSession, cmd, result));
    }

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function queries all results of the result summary.
/// HIFN
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR results/Returns all result summary results.
ViStatus _VI_FUNC rsspecan_GetVSAAllResults(ViSession instrSession,
                                            ViInt32   window,
                                            ViReal64  results[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViInt32  arraySize;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsspecan_AssertFSV(instrSession));

    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K70"));

    viCheckParm(RsCore_InvalidNullPointer(instrSession, results),
                3, "Results");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CALC%ld:MARK:FUNC:DDEM:STAT:ALL?", window);
    checkErr(RsCore_QueryAsciiViReal64Array(instrSession, cmd, &results, &arraySize));

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function queries the start of the current result range
/// HIFN within the capture buffer..
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR result/Returns the start result range value.
ViStatus _VI_FUNC rsspecan_GetVSAResultStart(ViSession instrSession,
                                             ViReal64* result)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_VSA_CURRENT_RESULT_RANGE_START_VALUE, result),
            2, "Result");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function queries the length of a detected burst.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR burstLength/Returns the burst length.
ViStatus _VI_FUNC rsspecan_GetVSABurstLength(ViSession instrSession,
                                             ViInt32   window,
                                             ViInt32*  burstLength)
{
    ViStatus error = VI_SUCCESS;
             window;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_GetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_VSA_BURST_LENGTH, burstLength),
            2, "Burst Length");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function returns all the measured data that relates to the
/// HIFN  currently selected measurement type. The returned data is particular to
/// HIFN  the currently selected measurement type and is specified in the Trace
/// HIFN  Data parameter description.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR sourceTrace/Select trace to read.
/// HIPAR arraySize/Number of array points requested.
/// HIPAR traceData/Returns an array of trace data. Number and format of the measurement
/// HIPAR traceData/values for operating mode VSA:
/// HIPAR noofValues/Returns the number of trace data values.
ViStatus _VI_FUNC rsspecan_ReadVSATraceData(ViSession instrSession,
                                            ViInt32   reserved,
                                            ViInt32   sourceTrace,
                                            ViInt32   arraySize,
                                            ViReal64  traceData[],
                                            ViInt32*  noofValues)
{
    ViStatus error = VI_SUCCESS;
             reserved;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, sourceTrace, RSSPECAN_VAL_TRACE_1, RSSPECAN_VAL_IQC_TRACE_X),
            2, "Source Trace");

    checkErr(rsspecan_dataReadTrace(instrSession, 0, VSATraceArr[sourceTrace], arraySize, traceData, noofValues));
    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  The trace symbols are returned in the currently defined symbol format
/// HIFN  (RSSPECAN_ATTR_VSA_RESULT_FORMAT).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR reserved/This control is reserved.
/// HIPAR arraySize/Number of array points requested.
/// HIPAR traceData/Returns an array of trace data. Number and format of the measurement
/// HIPAR traceData/values for operating mode VSA:
/// HIPAR noofValues/Returns the number of trace data values.
ViStatus _VI_FUNC rsspecan_ReadVSATraceSymbols(ViSession instrSession,
                                               ViInt32   window,
                                               ViInt32   arraySize,
                                               ViInt32   traceData[],
                                               ViInt32*  noofValues)
{
    ViStatus error = VI_SUCCESS;
    ViChar*  response = NULL;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViChar*  pbuff;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  i;
    ViInt32  resultFormat = RSSPECAN_VAL_VSA_RESULT_FORMAT_DEC;
    ViInt32  number;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_VSA_RESULT_FORMAT,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap),
            2, "Window");

    checkErr(rsspecan_GetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_VSA_RESULT_FORMAT, &resultFormat));

    if (window == 0)
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "FORM ASC;TRAC? STR");
    else
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "FORM ASC;TRAC%ld? STR", window);
    checkErr(RsCore_QueryViStringUnknownLength(instrSession, cmd, &response));

    pbuff = strtok(response, ",");
    i = 0;
    do
    {
        switch (resultFormat)
        {
        case RSSPECAN_VAL_VSA_RESULT_FORMAT_BIN:
            traceData[i] = (ViInt32)RsCore_Convert_BinaryString_To_ViInt64(pbuff);
            break;
        case RSSPECAN_VAL_VSA_RESULT_FORMAT_OCT:
            (void)sscanf(pbuff, "%lo", &number);
            traceData[i] = number;
            break;
        case RSSPECAN_VAL_VSA_RESULT_FORMAT_DEC:
            traceData[i] = RsCore_Convert_String_To_ViInt32(pbuff);
            break;
        case RSSPECAN_VAL_VSA_RESULT_FORMAT_HEX:
            (void)scanf("%lx", &number);
            traceData[i] = number;
            break;
        default:

            break;
        }
        pbuff = strtok(NULL, ",");
        i++;
    }
    while ((pbuff != NULL) && (i < arraySize));

    if (noofValues) *noofValues = i;
    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    if (response)
        free(response);

    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN Document your function here.
/// HIFN You may use multiple lines for documentation.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR type/Sets the match type.
/// HIPAR arraySize/Number of array points requested.
/// HIPAR data/Returns an array of data.
/// HIPAR noOfValues/Returns the number of trace data values.
ViStatus _VI_FUNC rsspecan_ReadVSATraceSymbolErrors(ViSession instrSession,
                                                    ViInt32   window,
                                                    ViInt32   type,
                                                    ViInt32   arraySize,
                                                    ViBoolean data[],
                                                    ViInt32*  noOfValues)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViChar*  buffer = NULL;
    ViChar*  pbuff;
    ViChar*  p2string;
    ViInt32  i;
    ViUInt8  val;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &RsSpecAn_rngVSATraceSymbolError, type, &p2string),
                3, "Type");
    if (window == 0)
    {
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "FORM ASC;TRAC? %s", p2string);
    }
    else
    {
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "FORM ASC;TRAC%ld? %s", window, p2string);
    }

    checkErr(RsCore_QueryViStringUnknownLength(instrSession, cmd, &buffer));
    pbuff = strtok(buffer, ",");
    i = 0;
    do
    {
        if (i < arraySize)
        {
            (void)sscanf(pbuff, "%1hhu", &val);
            data[i] = (val != 0);
        }
        pbuff = strtok(NULL, ",");
        i++;
    }
    while (pbuff != NULL);

    if (noOfValues)
        *noOfValues = i;

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    if (buffer) free(buffer);
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function queries the BER results.
/// HIRET Returns the status code of this operation.
/// HIRET You may use multiple lines for documentation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init
/// HIPAR instrSession/or rsspecan_InitWithOptions function.
/// HIPAR resultType/Selects the result type.
/// HIPAR bitErrorRate/Returns the Bit Error Rate (error bits / number of analyzed bits)
/// HIPAR numberOfErrors/Returns the number of analyzed errors.
/// HIPAR numberOfBits/Returns the number of analyzed bits.
ViStatus _VI_FUNC rsspecan_QueryVSABERResults(ViSession instrSession,
                                              ViInt32   reserved,
                                              ViInt32   resultType,
                                              ViReal64* bitErrorRate,
                                              ViInt32*  numberOfErrors,
                                              ViInt32*  numberOfBits)
{
    ViStatus error = VI_SUCCESS;
             reserved;

    checkErr(RsCore_LockSession(instrSession));

    switch (resultType)
    {
    case RSSPECAN_VAL_VSA_BER_RESULT_CURRENT:
        viCheckParm(rsspecan_GetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_VSA_BER_BIT_ERROR_RATE_CURRENT, bitErrorRate),
                4, "Bit Error Rate");

        viCheckParm(rsspecan_GetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_VSA_BER_NUMBER_OF_ERRORS_CURRENT, numberOfErrors),
                5, "Number Of Errors");

        viCheckParm(rsspecan_GetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_VSA_BER_NUMBER_OF_BITS_CURRENT, numberOfBits),
                6, "Number Of Bits");
        break;

    case RSSPECAN_VAL_VSA_BER_RESULT_MINIMUM:
        viCheckParm(rsspecan_GetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_VSA_BER_BIT_ERROR_RATE_MIN, bitErrorRate),
                4, "Bit Error Rate");

        viCheckParm(rsspecan_GetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_VSA_BER_NUMBER_OF_ERRORS_MIN, numberOfErrors),
                5, "Number Of Errors");

        viCheckParm(rsspecan_GetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_VSA_BER_NUMBER_OF_BITS_MIN, numberOfBits),
                6, "Number Of Bits");
        break;
    case RSSPECAN_VAL_VSA_BER_RESULT_MAXIMUM:
        viCheckParm(rsspecan_GetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_VSA_BER_BIT_ERROR_RATE_MAX, bitErrorRate),
                4, "Reference Deviation");

        viCheckParm(rsspecan_GetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_VSA_BER_NUMBER_OF_ERRORS_MAX, numberOfErrors),
                5, "Number Of Errors");

        viCheckParm(rsspecan_GetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_VSA_BER_NUMBER_OF_BITS_MAX, numberOfBits),
                6, "Number Of Bits");
        break;

    case RSSPECAN_VAL_VSA_BER_RESULT_ACCUMULATIVE:
        viCheckParm(rsspecan_GetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_VSA_BER_BIT_ERROR_RATE_ACC, bitErrorRate),
                4, "Bit Error Rate");

        viCheckParm(rsspecan_GetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_VSA_BER_NUMBER_OF_ERRORS_ACC, numberOfErrors),
                5, "Number Of Errors");

        viCheckParm(rsspecan_GetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_VSA_BER_NUMBER_OF_BITS_ACC, numberOfBits),
                6, "Number Of Bits");
        break;

    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, resultType), 2, "Result Type");
        break;
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Returns the number of the symbol errors.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR resultType/Selects the result type.
/// HIPAR symbolError/Returns the number of the symbol errors.
ViStatus _VI_FUNC rsspecan_QueryVSABERSymbolError(ViSession instrSession,
                                                  ViInt32   resultType,
                                                  ViInt32*  symbolError)
{
    ViStatus error = VI_SUCCESS;
    ViAttr   arr[4] = {
             RSSPECAN_ATTR_VSA_BER_NUMBER_OF_SYMBOL_ERRORS_CURRENT,
             RSSPECAN_ATTR_VSA_BER_NUMBER_OF_SYMBOL_ERRORS_MINIMUM,
             RSSPECAN_ATTR_VSA_BER_NUMBER_OF_SYMBOL_ERRORS_MAXIMUM,
             RSSPECAN_ATTR_VSA_BER_NUMBER_OF_SYMBOL_ERRORS_ACCUMULATED
             };

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_GetAttributeViInt32(instrSession, "", arr[resultType], symbolError),
            3, "Symbol Error");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function queries whether the limit for the specified result type
/// HIFN  and limit type was violated.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR measurementType/Selects the measurement limit result.
/// HIPAR limitType/Selects the limit type.
/// HIPAR checkResults/Returns limit state code.
ViStatus _VI_FUNC rsspecan_QueryVSAModulationAccuracyLimitsCheckResults(
                            ViSession instrSession,
                            ViInt32   reserved,
                            ViInt32   measurementType,
                            ViInt32   limitType,
                            ViInt32*  checkResults)
{
    ViStatus error = VI_SUCCESS;
    ViInt32  maxStat = RSSPECAN_VAL_VSA_LIMIT_PEAK_PEAK;
    ViInt32  minStat = RSSPECAN_VAL_VSA_LIMIT_CURRENT;
    ViAttr   attr[8][9] = {
             {

            //       "EVM"
            0,
            0,
            0,
            RSSPECAN_ATTR_VSA_QUERY_MOD_ACC_EVM_RMS_CURR_LIMIT_CHECK_RESULT,
            RSSPECAN_ATTR_VSA_QUERY_MOD_ACC_EVM_RMS_MEAN_LIMIT_CHECK_RESULT,
            RSSPECAN_ATTR_VSA_QUERY_MOD_ACC_EVM_RMS_PEAK_LIMIT_CHECK_RESULT,
            RSSPECAN_ATTR_VSA_QUERY_MOD_ACC_EVM_PEAK_CURR_LIMIT_CHECK_RESULT,
            RSSPECAN_ATTR_VSA_QUERY_MOD_ACC_EVM_PEAK_MEAN_LIMIT_CHECK_RESULT,
            RSSPECAN_ATTR_VSA_QUERY_MOD_ACC_EVM_PEAK_PEAK_LIMIT_CHECK_RESULT
        },
        {
            //       "PERR"
            0,
            0,
            0,
            RSSPECAN_ATTR_VSA_QUERY_MOD_ACC_PERR_RMS_CURR_LIMIT_CHECK_RESULT,
            RSSPECAN_ATTR_VSA_QUERY_MOD_ACC_PERR_RMS_MEAN_LIMIT_CHECK_RESULT,
            RSSPECAN_ATTR_VSA_QUERY_MOD_ACC_PERR_RMS_PEAK_LIMIT_CHECK_RESULT,
            RSSPECAN_ATTR_VSA_QUERY_MOD_ACC_PERR_PEAK_CURR_LIMIT_CHECK_RESULT,
            RSSPECAN_ATTR_VSA_QUERY_MOD_ACC_PERR_PEAK_MEAN_LIMIT_CHECK_RESULT,
            RSSPECAN_ATTR_VSA_QUERY_MOD_ACC_PERR_PEAK_PEAK_LIMIT_CHECK_RESULT
        },
        {
            //       "MERR"
            0,
            0,
            0,
            RSSPECAN_ATTR_VSA_QUERY_MOD_ACC_MERR_RMS_CURR_LIMIT_CHECK_RESULT,
            RSSPECAN_ATTR_VSA_QUERY_MOD_ACC_MERR_RMS_MEAN_LIMIT_CHECK_RESULT,
            RSSPECAN_ATTR_VSA_QUERY_MOD_ACC_MERR_RMS_PEAK_LIMIT_CHECK_RESULT,
            RSSPECAN_ATTR_VSA_QUERY_MOD_ACC_MERR_PEAK_CURR_LIMIT_CHECK_RESULT,
            RSSPECAN_ATTR_VSA_QUERY_MOD_ACC_MERR_PEAK_MEAN_LIMIT_CHECK_RESULT,
            RSSPECAN_ATTR_VSA_QUERY_MOD_ACC_MERR_PEAK_PEAK_LIMIT_CHECK_RESULT
        },
        {
            //       "CFER"
            RSSPECAN_ATTR_VSA_QUERY_MOD_ACC_CFER_CURR_LIMIT_CHECK_RESULT,
            RSSPECAN_ATTR_VSA_QUERY_MOD_ACC_CFER_MEAN_LIMIT_CHECK_RESULT,
            RSSPECAN_ATTR_VSA_QUERY_MOD_ACC_CFER_PEAK_LIMIT_CHECK_RESULT,
            0,
            0,
            0,
            0,
            0,
            0
        },
        {
            //       "RHO"
            RSSPECAN_ATTR_VSA_QUERY_MOD_ACC_RHO_CURR_LIMIT_CHECK_RESULT,
            RSSPECAN_ATTR_VSA_QUERY_MOD_ACC_RHO_MEAN_LIMIT_CHECK_RESULT,
            RSSPECAN_ATTR_VSA_QUERY_MOD_ACC_RHO_PEAK_LIMIT_CHECK_RESULT,
            0,
            0,
            0,
            0,
            0,
            0
        },
        {
            //       "IQOffset"
            RSSPECAN_ATTR_VSA_QUERY_MOD_ACC_IQ_OFFSET_CURR_LIMIT_CHECK_RESULT,
            RSSPECAN_ATTR_VSA_QUERY_MOD_ACC_IQ_OFFSET_MEAN_LIMIT_CHECK_RESULT,
            RSSPECAN_ATTR_VSA_QUERY_MOD_ACC_IQ_OFFSET_PEAK_LIMIT_CHECK_RESULT,
            0,
            0,
            0,
            0,
            0,
            0
        },
        {
            //       "FreqErr"
            0,
            0,
            0,
            RSSPECAN_ATTR_VSA_QUERY_MOD_ACC_FERR_RMS_CURR_LIMIT_CHECK_RESULT,
            RSSPECAN_ATTR_VSA_QUERY_MOD_ACC_FERR_RMS_MEAN_LIMIT_CHECK_RESULT,
            RSSPECAN_ATTR_VSA_QUERY_MOD_ACC_FERR_RMS_PEAK_LIMIT_CHECK_RESULT,
            RSSPECAN_ATTR_VSA_QUERY_MOD_ACC_FERR_PEAK_CURR_LIMIT_CHECK_RESULT,
            RSSPECAN_ATTR_VSA_QUERY_MOD_ACC_FERR_PEAK_MEAN_LIMIT_CHECK_RESULT,
            RSSPECAN_ATTR_VSA_QUERY_MOD_ACC_FERR_PEAK_PEAK_LIMIT_CHECK_RESULT
        },
        {
            //       "FreqDevErr"
            RSSPECAN_ATTR_VSA_QUERY_MOD_ACC_FDER_CURR_LIMIT_CHECK_RESULT,
            RSSPECAN_ATTR_VSA_QUERY_MOD_ACC_FDER_MEAN_LIMIT_CHECK_RESULT,
            RSSPECAN_ATTR_VSA_QUERY_MOD_ACC_FDER_PEAK_LIMIT_CHECK_RESULT,
            0,
            0,
            0,
            0,
            0,
            0
        },
    };
    reserved;


    checkErr(RsCore_LockSession(instrSession));

    if ((measurementType == RSSPECAN_VAL_VSA_RESULT_RHO) ||
        (measurementType == RSSPECAN_VAL_VSA_RESULT_CF_ERROR) ||
        (measurementType == RSSPECAN_VAL_VSA_RESULT_IQ_OFFSET) ||
        (measurementType == RSSPECAN_VAL_VSA_RESULT_FSK_DEV_ERROR))
    {
        maxStat = RSSPECAN_VAL_VSA_LIMIT_PEAK;
    }
    else
    {
        minStat = RSSPECAN_VAL_VSA_LIMIT_CURRENT_RMS;
    }

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, measurementType, RSSPECAN_VAL_VSA_RESULT_EVM, RSSPECAN_VAL_VSA_RESULT_FSK_DEV_ERROR),
            3, "Measurement Type");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, limitType, minStat, maxStat),
            4, "Limit Type");

    viCheckParm(rsspecan_GetAttributeViInt32(instrSession, "", attr[measurementType][limitType], checkResults),
            5, "Check Results");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function queries the modulation accuracy statisitc result.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR measurementType/Selects the measurement result.
/// HIPAR statistic/Selects the queried statisitc.
/// HIPAR statisticValue/Returns statistc value.
ViStatus _VI_FUNC rsspecan_QueryVSAModulationAccuracyStatisticResults(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   reserved,
                            ViInt32   measurementType,
                            ViInt32   statistic,
                            ViReal64* statisticValue)
{
    ViStatus error = VI_SUCCESS;
    ViInt32  maxStat = RSSPECAN_VAL_VSA_STAT_95_RMS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE];
    ViAttr   attr[19][10] = {
             {

            //       "EVM"
            RSSPECAN_ATTR_VSA_RESULT_EVM_RMS_CURRENT,
            RSSPECAN_ATTR_VSA_RESULT_EVM_RMS_AVERAGE,
            RSSPECAN_ATTR_VSA_RESULT_EVM_RMS_PEAK,
            RSSPECAN_ATTR_VSA_RESULT_EVM_RMS_SDEV,
            RSSPECAN_ATTR_VSA_RESULT_EVM_RMS_CURRENT_PCTL,
            RSSPECAN_ATTR_VSA_RESULT_EVM_PEAK_CURRENT,
            RSSPECAN_ATTR_VSA_RESULT_EVM_PEAK_AVERAGE,
            RSSPECAN_ATTR_VSA_RESULT_EVM_PEAK_PEAK,
            RSSPECAN_ATTR_VSA_RESULT_EVM_PEAK_SDEV,
            RSSPECAN_ATTR_VSA_RESULT_EVM_PEAK_PCTL
        },
        {
            //       "PERR"
            RSSPECAN_ATTR_VSA_RESULT_PHASE_ERROR_RMS_CURRENT,
            RSSPECAN_ATTR_VSA_RESULT_PHASE_ERROR_RMS_AVERAGE,
            RSSPECAN_ATTR_VSA_RESULT_PHASE_ERROR_RMS_PEAK,
            RSSPECAN_ATTR_VSA_RESULT_PHASE_ERROR_RMS_SDEV,
            RSSPECAN_ATTR_VSA_RESULT_PHASE_ERROR_RMS_PCTL,
            RSSPECAN_ATTR_VSA_RESULT_PHASE_ERROR_PEAK_CURRENT,
            RSSPECAN_ATTR_VSA_RESULT_PHASE_ERROR_PEAK_AVERAGE,
            RSSPECAN_ATTR_VSA_RESULT_PHASE_ERROR_PEAK_PEAK,
            RSSPECAN_ATTR_VSA_RESULT_PHASE_ERROR_PEAK_SDEV,
            RSSPECAN_ATTR_VSA_RESULT_PHASE_ERROR_PEAK_PCTL
        },
        {
            //       "MERR"
            RSSPECAN_ATTR_VSA_RESULT_MERR_RMS_CURRENT,
            RSSPECAN_ATTR_VSA_RESULT_MERR_RMS_AVERAGE,
            RSSPECAN_ATTR_VSA_RESULT_MERR_RMS_PEAK,
            RSSPECAN_ATTR_VSA_RESULT_MERR_RMS_SDEV,
            RSSPECAN_ATTR_VSA_RESULT_MERR_RMS_PCTL,
            RSSPECAN_ATTR_VSA_RESULT_MERR_PEAK_CURRENT,
            RSSPECAN_ATTR_VSA_RESULT_MERR_PEAK_AVERAGE,
            RSSPECAN_ATTR_VSA_RESULT_MERR_PEAK_PEAK,
            RSSPECAN_ATTR_VSA_RESULT_MERR_PEAK_SDEV,
            RSSPECAN_ATTR_VSA_RESULT_MERR_PEAK_PCTL
        },
        {
            //       "CFER"
            RSSPECAN_ATTR_VSA_RESULT_CFER_CURRENT,
            RSSPECAN_ATTR_VSA_RESULT_CFER_AVERAGE,
            RSSPECAN_ATTR_VSA_RESULT_CFER_PEAK,
            RSSPECAN_ATTR_VSA_RESULT_CFER_SDEV,
            RSSPECAN_ATTR_VSA_RESULT_CFER_PCTL,
            0,
            0,
            0,
            0,
            0
        },
        {
            //       "RHO"
            RSSPECAN_ATTR_VSA_RESULT_RHO_CURRENT,
            RSSPECAN_ATTR_VSA_RESULT_RHO_AVERAGE,
            RSSPECAN_ATTR_VSA_RESULT_RHO_PEAK,
            RSSPECAN_ATTR_VSA_RESULT_RHO_SDEV,
            RSSPECAN_ATTR_VSA_RESULT_RHO_PCTL,
            0,
            0,
            0,
            0,
            0
        },
        {
            //       "IQOFF"
            RSSPECAN_ATTR_VSA_RESULT_IQOF_CURRENT,
            RSSPECAN_ATTR_VSA_RESULT_IQOF_AVERAGE,
            RSSPECAN_ATTR_VSA_RESULT_IQOF_PEAK,
            RSSPECAN_ATTR_VSA_RESULT_IQOF_SDEV,
            RSSPECAN_ATTR_VSA_RESULT_IQOF_PCTL,
            0,
            0,
            0,
            0,
            0
        },
        {
            //       " FSK Frequency Error
            RSSPECAN_ATTR_VSA_RESULT_FSK_DERR_RMS_CURRENT,
            RSSPECAN_ATTR_VSA_RESULT_FSK_DERR_RMS_AVERAGE,
            RSSPECAN_ATTR_VSA_RESULT_FSK_DERR_RMS_PEAK,
            RSSPECAN_ATTR_VSA_RESULT_FSK_DERR_RMS_SDEV,
            RSSPECAN_ATTR_VSA_RESULT_FSK_DERR_RMS_PCTL,
            RSSPECAN_ATTR_VSA_RESULT_FSK_DERR_PEAK_CURRENT,
            RSSPECAN_ATTR_VSA_RESULT_FSK_DERR_PEAK_AVERAGE,
            RSSPECAN_ATTR_VSA_RESULT_FSK_DERR_PEAK_PEAK,
            RSSPECAN_ATTR_VSA_RESULT_FSK_DERR_PEAK_SDEV,
            RSSPECAN_ATTR_VSA_RESULT_FSK_DERR_PEAK_PCTL
        },
        {
            //       " FSK Deviation Error
            RSSPECAN_ATTR_VSA_RESULT_FDER_CURRENT,
            RSSPECAN_ATTR_VSA_RESULT_FDER_AVERAGE,
            RSSPECAN_ATTR_VSA_RESULT_FDER_PEAK,
            RSSPECAN_ATTR_VSA_RESULT_FDER_SDEV,
            RSSPECAN_ATTR_VSA_RESULT_FDER_PCTL,
            0,
            0,
            0,
            0,
            0
        },
        {
            //       "FSK:MDEV"
            RSSPECAN_ATTR_VSA_RESULT_MDEV_CURRENT,
            RSSPECAN_ATTR_VSA_RESULT_MDEV_AVERAGE,
            RSSPECAN_ATTR_VSA_RESULT_MDEV_PEAK,
            RSSPECAN_ATTR_VSA_RESULT_MDEV_SDEV,
            RSSPECAN_ATTR_VSA_RESULT_MDEV_PCTL,
            0,
            0,
            0,
            0,
            0
        },
        {
            //       "FSK:RDEV"
            RSSPECAN_ATTR_VSA_RESULT_RDEV_CURRENT,
            RSSPECAN_ATTR_VSA_RESULT_RDEV_AVERAGE,
            RSSPECAN_ATTR_VSA_RESULT_RDEV_PEAK,
            RSSPECAN_ATTR_VSA_RESULT_RDEV_SDEV,
            RSSPECAN_ATTR_VSA_RESULT_RDEV_PCTL,
            0,
            0,
            0,
            0,
            0
        },
        {
            //       "MPOW"
            RSSPECAN_ATTR_VSA_RESULT_POWER_CURRENT,
            RSSPECAN_ATTR_VSA_RESULT_POWER_AVERAGE,
            RSSPECAN_ATTR_VSA_RESULT_POWER_PEAK,
            RSSPECAN_ATTR_VSA_RESULT_POWER_SDEV,
            RSSPECAN_ATTR_VSA_RESULT_POWER_PCTL,
            0,
            0,
            0,
            0,
            0
        },
        {
            //       "FSK:CFDR"
            RSSPECAN_ATTR_VSA_RESULT_FSK_CFDR_CURRENT,
            RSSPECAN_ATTR_VSA_RESULT_FSK_CFDR_AVERAGE,
            RSSPECAN_ATTR_VSA_RESULT_FSK_CFDR_PEAK,
            RSSPECAN_ATTR_VSA_RESULT_FSK_CFDR_SDEV,
            RSSPECAN_ATTR_VSA_RESULT_FSK_CFDR_CURRENT_PCTL,
            0,
            0,
            0,
            0,
            0
        },
        {
            //       "SNR Modulation Error Ratio"
            RSSPECAN_ATTR_VSA_RESULT_MER_RMS_CURRENT,
            RSSPECAN_ATTR_VSA_RESULT_MER_RMS_AVERAGE,
            RSSPECAN_ATTR_VSA_RESULT_MER_RMS_PEAK,
            RSSPECAN_ATTR_VSA_RESULT_MER_RMS_SDEV,
            RSSPECAN_ATTR_VSA_RESULT_MER_RMS_PCTL,
            RSSPECAN_ATTR_VSA_RESULT_MER_PEAK_CURRENT,
            RSSPECAN_ATTR_VSA_RESULT_MER_PEAK_AVERAGE,
            RSSPECAN_ATTR_VSA_RESULT_MER_PEAK_PEAK,
            RSSPECAN_ATTR_VSA_RESULT_MER_PEAK_SDEV,
            RSSPECAN_ATTR_VSA_RESULT_MER_PEAK_PCTL
        },
        {
            //       "IQIM"
            RSSPECAN_ATTR_VSA_RESULT_IQIM_CURRENT,
            RSSPECAN_ATTR_VSA_RESULT_IQIM_AVERAGE,
            RSSPECAN_ATTR_VSA_RESULT_IQIM_PEAK,
            RSSPECAN_ATTR_VSA_RESULT_IQIM_SDEV,
            RSSPECAN_ATTR_VSA_RESULT_IQIM_PCTL,
            0,
            0,
            0,
            0,
            0
        },
        {
            //       "GIMB"
            RSSPECAN_ATTR_VSA_RESULT_GIMB_CURRENT,
            RSSPECAN_ATTR_VSA_RESULT_GIMB_AVERAGE,
            RSSPECAN_ATTR_VSA_RESULT_GIMB_PEAK,
            RSSPECAN_ATTR_VSA_RESULT_GIMB_CURRENT_SDEV,
            RSSPECAN_ATTR_VSA_RESULT_GIMB_PCTL,
            0,
            0,
            0,
            0,
            0
        },
        {
            //       "QERR"
            RSSPECAN_ATTR_VSA_RESULT_QERR_CURRENT,
            RSSPECAN_ATTR_VSA_RESULT_QERR_AVERAGE,
            RSSPECAN_ATTR_VSA_RESULT_QERR_PEAK,
            RSSPECAN_ATTR_VSA_RESULT_QERR_SDEV,
            RSSPECAN_ATTR_VSA_RESULT_QERR_PCTL,
            0,
            0,
            0,
            0,
            0
        },
        {
            //       "ADR"
            RSSPECAN_ATTR_VSA_RESULT_AMPLITUDE_DROOP_CURRENT,
            RSSPECAN_ATTR_VSA_RESULT_AMPLITUDE_DROOP_AVERAGE,
            RSSPECAN_ATTR_VSA_RESULT_AMPLITUDE_DROOP_PEAK,
            RSSPECAN_ATTR_VSA_RESULT_AMPLITUDE_DROOP_SDEV,
            RSSPECAN_ATTR_VSA_RESULT_AMPLITUDE_DROOP_PCTL,
            0,
            0,
            0,
            0,
            0
        },
        {
            //       "Symbol Rate Error "
            RSSPECAN_ATTR_VSA_RESULT_SRER_CURRENT,
            RSSPECAN_ATTR_VSA_RESULT_SRER_AVERAGE,
            RSSPECAN_ATTR_VSA_RESULT_SRER_PEAK,
            RSSPECAN_ATTR_VSA_RESULT_SRER_SDEV,
            RSSPECAN_ATTR_VSA_RESULT_SRER_PCTL,
            0,
            0,
            0,
            0,
            0
        },
        {
            //       "IQ Skew"
            RSSPECAN_ATTR_VSA_RESULT_IQSK_CURRENT,
            RSSPECAN_ATTR_VSA_RESULT_IQSK_AVERAGE,
            RSSPECAN_ATTR_VSA_RESULT_IQSK_PEAK,
            RSSPECAN_ATTR_VSA_RESULT_IQSK_SDEV,
            RSSPECAN_ATTR_VSA_RESULT_IQSK_PCTL,
            0,
            0,
            0,
            0,
            0
        }
    };
    reserved;


    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, measurementType, RSSPECAN_VAL_VSA_RESULT_EVM, RSSPECAN_VAL_VSA_RESULT_SRATE_ERROR),
            5, "Measurement Type");

    if ((measurementType == RSSPECAN_VAL_VSA_RESULT_EVM) ||
        (measurementType == RSSPECAN_VAL_VSA_RESULT_PHASE_ERROR) ||
        (measurementType == RSSPECAN_VAL_VSA_RESULT_MAGNITUDE_ERROR) ||
        (measurementType == RSSPECAN_VAL_VSA_RESULT_FREQ_ERROR) ||
        (measurementType == RSSPECAN_VAL_VSA_RESULT_MOD_ERROR))
    {
        maxStat = RSSPECAN_VAL_VSA_STAT_MAX_95_RMS;
    }

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, statistic, RSSPECAN_VAL_VSA_STAT_AVERAGE_SWEEP, maxStat),
            6, "Statistic");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "Win%ld", window);

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, repCap, attr[measurementType][statistic], statisticValue),
            7, "Result");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries burst information found across the capture buffer.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR burstCount/Queries the number of bursts found across the capture buffer.
/// HIPAR burstPresent/Queries whether a burst is present or not in the current result range.
/// HIPAR burstStart/Queries the burst start in the current result range as an offset to
/// HIPAR burstStart/the capture buffer start.
/// HIPAR burstLength/Queries the length of the burst in the current result range.
ViStatus _VI_FUNC rsspecan_QueryVSABurstInformation(ViSession  instrSession,
                                                    ViInt32*   burstCount,
                                                    ViBoolean* burstPresent,
                                                    ViInt32*   burstStart,
                                                    ViReal64*  burstLength)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_GetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_DSP_BURST_COUNT, burstCount),
            2, "Burst Count");

    viCheckParm(rsspecan_GetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_DSP_BURST_PRESENT, burstPresent),
            3, "Burst Present");

    if (*burstPresent)
    {
        viCheckParm(rsspecan_GetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_DSP_BURST_START, burstStart),
                4, "Burst Start");

        viCheckParm(rsspecan_GetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_DSP_BURST_LENGTH, burstLength),
                5, "Burst Length");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the pattern information.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR patternCount/Queries the number of patterns found across the capture buffer.
/// HIPAR patternPresent/Queries whether a pattern is present or not in the current result
/// HIPAR patternPresent/range.
/// HIPAR patternStart/Queries the pattern start in the current result range as an offset to
/// HIPAR patternStart/the capture buffer start.
/// HIPAR patternCorrectness/Queries whether the pattern is correct or not in the current result
/// HIPAR patternCorrectness/range.
/// HIPAR patternConfidence/Queries the confidence with which the pattern was detected in the
/// HIPAR patternConfidence/current result range.
ViStatus _VI_FUNC rsspecan_QueryVSAPatternInformation(ViSession  instrSession,
                                                      ViInt32*   patternCount,
                                                      ViBoolean* patternPresent,
                                                      ViInt32*   patternStart,
                                                      ViBoolean* patternCorrectness,
                                                      ViReal64*  patternConfidence)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_GetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_DSP_PATTERN_COUNT, patternCount),
            2, "Pattern Count");

    viCheckParm(rsspecan_GetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_DSP_PATTERN_PRESENT, patternPresent),
            3, "Pattern Present");

    if (*patternPresent)
    {
        viCheckParm(rsspecan_GetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_DSP_PATTERN_START, patternStart),
                4, "Pattern Start");

        viCheckParm(rsspecan_GetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_DSP_PATTERN_CORRECTNESS, patternCorrectness),
                5, "Pattern Correctness");

        viCheckParm(rsspecan_GetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_DSP_PATTERN_CONFIDENCE, patternConfidence),
                6, "Pattern Confidence");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function queries the absolute x-value of the selected delta
/// HIFN  marker in the specified window. Activates the corresponding delta
/// HIFN  marker, if necessary.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR marker/Selects marker.
/// HIPAR absoluteX/Queries the absolute x-value of the selected delta marker in the
/// HIPAR absoluteX/specified window. Activates the corresponding delta marker, if
/// HIPAR absoluteX/necessary.
ViStatus _VI_FUNC rsspecan_QueryVSADeltamarkerAbsoluteX(ViSession instrSession,
                                                        ViInt32   window,
                                                        ViInt32   marker,
                                                        ViReal64* absoluteX)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, window, 0, 16),
            2, "Window");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, marker, 1, 16),
            3, "Marker");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "Win%ld,M%ld", window, marker);

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_VSA_DELTAMARKER_ABSOLUTE_X, absoluteX),
            4, "Absolute X");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function sets the Enable, NTransition, and PTransition bits    of
/// HIFN  the Main, and Carrier Frequency, FSK, IQ Offset &  RHO, Magnitude
/// HIFN  Error, Phase Error.
/// HIFN     This register comprises information about any limit violations that
/// HIFN  may occur after demodulation in any of the four windows.
/// HIFN
/// HIFN     Note(s):
/// HIFN
/// HIFN     (1) For detailed description of Status Reporting System see also
/// HIFN  operating manual.
/// HIFN
/// HIFN     (2) This function is available only with FSV instrument.
/// HIFN
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR channel/This control selects the number of active channel.
/// HIPAR registerOperation/Specifies the operation to perform.
/// HIPAR questionableRegister/This control selects the registers that will be configured.
/// HIPAR enable/This control sets the enable bits of the status register.
/// HIPAR PTransition/This control sets the PTransition bits (edge detectors) of the status
/// HIPAR PTransition/register.
/// HIPAR NTransition/This control sets the NTransition bits (edge detectors) of the status
/// HIPAR NTransition/register.
ViStatus _VI_FUNC rsspecan_setVSAModStatusRegister(ViSession instrSession,
                                                   ViInt32   channel,
                                                   ViInt32   registerOperation,
                                                   ViInt32   questionableRegister,
                                                   ViInt32   enable,
                                                   ViInt32   PTransition,
                                                   ViInt32   NTransition)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViChar*  p2string;

    checkErr(RsCore_LockSession(instrSession));

    switch (registerOperation)
    {
    case RSSPECAN_VAL_STAT_REG_QUESTIONABLE:
        viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &RsSpecAn_rngVSAStatusRegister, questionableRegister, &p2string),
            3, "Questionable Register");

        if (questionableRegister == 0)
            snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, ":STAT:QUES:MOD:ENAB %ld;PTR %ld;NTR %ld", enable, PTransition,
                     NTransition);
        else
            snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, ":STAT:QUES:MOD%ld:%s:ENAB %ld;PTR %ld;NTR %ld", channel, p2string,
                     enable, PTransition, NTransition);
        checkErr(RsCore_Write(instrSession, cmd));

        break;
    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, registerOperation), 2, "Register Operation");
    }

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function contains the commands for the querying status reporting
/// HIFN  system.
/// HIFN
/// HIFN     Note(s):
/// HIFN
/// HIFN     (1)For detailed description of Status Reporting System see also
/// HIFN  operating manual.
/// HIFN
/// HIFN     (2) This function is available only with FSV instrument.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR channel/This control selects the number of active channel.
/// HIPAR questionableRegister/This control selects the registers that will be configured.
/// HIPAR registerPart/This control selects the registers to be queried.
/// HIPAR registerValue/Returns content of selected register.
ViStatus _VI_FUNC rsspecan_getVSAModStatusRegister(ViSession instrSession,
                                                   ViInt32   channel,
                                                   ViInt32   questionableRegister,
                                                   ViInt32   registerPart,
                                                   ViInt32*  registerValue)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViChar*  p2string;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, registerPart, 0, 1),
            4, "Register Part");
    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &RsSpecAn_rngVSAStatusRegister, questionableRegister, &p2string),
        3, "Questionable Register");
    if (questionableRegister == 0)
    {
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, ":STAT:QUES:MOD:%s?",
                 rsspecan_rngStatusRegisterQuery.entries[registerPart].cmdString);
    }
    else
    {
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, ":STAT:QUES:MOD%ld:%s:%s?",
                 channel, p2string, rsspecan_rngStatusRegisterQuery.entries[registerPart].cmdString);
    }

    checkErr(RsCore_QueryViInt32(instrSession, cmd, registerValue));

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}
